#include <stdio.h>
#include <stdlib.h>
#include <nrutil.h>
#include <math.h>
#include <mconf.h>
#include "myio.h"
#include "mydefs.h"

#define SMALL	1.0e-15	        // used in root finding
#define INTACC	1.0e-11	        // target integration accuracy
#define ROOTACC	1.0e-12          // target root accuracy for golden
#define TINY	1.0e-300	// tiny number used by integrator
#define BIG	1.0e+300	// big number 
#define LENAW	32000		// number of quadrature points
#define IRCUT	1.0e-05	        // IR cut on int3 integration
#define UVCUT   2.0e3		// UV cut on inf integrations

FILE    *out,*out2,*out3,*out4;

double v,k,kz,t,w;
double q,qs,T;
double nx,nz,asi;

int     lenaw=LENAW;
double  aw[LENAW];
int verbose=0; 	// 0 - no messages
		// 1 - messages from top level integrations
	 	// 2 - messages from bottom level integrations


main(argc,argv)
int     argc;
char    **argv;
{
	int    i,j,nsteps;
	double qsmax,qsmin,th,step,temp;
	double int3(),conversion;
	double int2();
	double pow();
	char* text;


	printf("\nEnergy loss of the hard part, in units of mD^2 e^2/8/Pi\n");
	printf("Synopsis: Elosshard [paramters]\n");
	printf("where [parameters] have to be of the form: v th asi pqsmin pqsmax nsteps [outputfile]\n\n");
	printf("For Mike: if the first parameter is 't' (for test) then
you can specify as second and third parameters your favorite value of q* and
v and the program will output a table of d^2W/dx/dz (all the forefactors
included but without the z-integration done)\n\n");

	if ((argc==4)&&(*argv[1]=='t')){
	  printf("Test version. For internal debugging only\n");
	  qs=atof(argv[2]);
	  v=atof(argv[3]);
	  testmain();
	  exit(0);
	}
	if (argc!=8) {
	  printf("All parameters have to be specified! Exiting now...\n");
	  exit(1);
	}
	
	qs=0.01;
	k=0.5;
	/*v = 0.5;
	qs = 0.01;

	nz=0.9;	
	asi=1E-5;*/

	//printf("args #: %i",argc);

	v=atof(argv[1]);
	th=atof(argv[2]);
	asi=atof(argv[3]);
	qsmin=atof(argv[4]);
	qsmax=atof(argv[5]);
	nsteps=atoi(argv[6]);
	text=argv[7];

	nz=cos(th);
	nx=sin(th);
	
	printf("Calculating hard energy loss part for\n");
	printf(" v=%e\n v_z=%e\n asi=%e\n pqmax=%e\n pqmin=%e\n nstep=%i\n",v,v*nz,asi,qsmax,qsmin,nsteps);

	printf("Data output file:%s\n\n",text);
	conversion=3/PI/PI/PI/PI/v/2;
	//temp=int2();
	//temp=pow(10.0,-2.0);
	//printf("temp:%e, res:%e",temp,temp*conversion);
	
	IO(out,text,"w");

	step = (qsmax-qsmin)/nsteps;
	
	for (i=0;i<=nsteps;i++) {
	  qs = pow(10.0,qsmin + i*step); 
	  //printf("i:%i\n",i);
	  //qs=pow(10.0,qsmin);
	  temp = int3()*conversion;
	  printf("%e\t%e\n",qs,temp);
	  fprintf(out,"%e\t%e\n",qs,temp);
	  if (temp<1.0E-4) {printf("stopping at %e",qs); i=nsteps+1;}
	}
	fclose(out);
	
	//printf("%e",temp);
	/*
	qs=0.1;
	temp=int3()*3/PI/PI/PI/PI/v/2;
	printf("temp:%e\n",temp);

	qs=1.0;
	temp=int3()*3/PI/PI/PI/PI/v/2;
	printf("temp:%e\n",temp);	

	qs=10.0;
	temp=int3()*3/PI/PI/PI/PI/v/2;
	printf("temp:%e\n",temp);	

	qs=100.0;
	temp=int3()*3/PI/PI/PI/PI/v/2;
	printf("temp:%e\n",temp);   */
}

testmain(argc,argv)
int     argc;
char    **argv;
{
	int    i,j,nsteps;
	double qsmax,qsmin,th,step,temp;
	double int3(),conversion;
	double int2(),int1a();
	double pow();
	char* text;

	//	qs=10.0;
	k=3;
	//v = 0.5;
	//kz = 0.75;
	asi=1E-05;

	nz=cos(1.0);
	nx=sin(1.0);
	printf("Printing results to test.dat\n");
	IO(out,"test.dat","w");
	conversion=3/PI/PI/PI/PI/v/2;
	printf("qs:%e, v:%e",qs,v);
	//temp = int1a(0.75/k);
	temp=int2(0.26);
	//findchilimit(1E-8,&temp);
	//temp=int3();
	printf("temp:%e\n",temp);
	for (i=0;i<=50;i++) {
	  k=(1-v)/2+i*0.2;
	  temp = int2(k)*4*conversion;
	  printf("z:%e\t %e\n",k,temp);
	  fprintf(out,"%e\t%e\n",k,temp);
	}	
	fclose(out);
}

double int3()
{
  double res,err,int2(),temp;
  void intdeiini(int lenaw, double tiny, double eps, double *aw);
  void intdei(double (*f)(double), double a, double *aw, double *i, double *err);
  void intdeini(int lenaw, double tiny, double eps, double *aw);
  void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);

  intdeini(lenaw, TINY, INTACC, aw);
 
  intde(int2, (1-v)/2+IRCUT, (1+v)/2, aw, &temp, &err);
  if (err<0&&verbose==2) { 
    printf("unable to achieve desired accuracy in int3 integration...\n"); }
  //printf("res1:%e",temp);
  res=temp;
  intdeini(lenaw, TINY, INTACC, aw);
  intde(int2,(1+v)/2,UVCUT+8/qs, aw, &temp, &err);
  if (err<0&&verbose==2) { 
    printf("unable to achieve desired accuracy in int3 integration...\n"); }
  //printf("res2:%e",temp);
  res+=temp;
  return res*4;
}


double int2(zk)
double zk;
{
	double res,err,int1();
	void intdeini(int lenaw, double tiny, double eps, double *aw);
	void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);
	double chilimit;
	int j;
	double polylog();

	k=zk;
	//printf("k:%e\n",k-0.25);
	fflush(0);
	j=findchilimit(1E-8,&chilimit);
	//printf("limits:%i, at %e",j,chilimit);
	if (j==-1) { return 0;}
	if (j==0){
	  intdeini(lenaw, TINY, INTACC, aw);
	  intde(int1, -1.0, 1.0, aw, &res, &err);
	  if (err<0&&verbose==2) { 
	    printf("unable to achieve desired accuracy in int2 integration: 
k=%e, res=%e...\n",k,res); }
	  return res/k;
	}
	//if (chilimit==0) {printf("0 chilim: failure at k=%e\n",zk);}
	/*	if (int1(-1+1E-8)<1E-12) { 
	    intdeini(lenaw, TINY, INTACC, aw);
	  intde(int1, chilimit, 1.0, aw, &res, &err);
	  if (err<0&&verbose==2) { 
	    printf("unable to achieve desired accuracy in int2 integration...\n");
	  }
	  return res/k;
	  printf("Unable to handle!\n");
	  return 0;
	}*/
	if (j==1) { 
	  intdeini(lenaw, TINY, INTACC, aw); 
	  intde(int1, -1.0,chilimit, aw, &res, &err);
	  //printf("Integrating from -1 to chilim");
	  if (err<0&&verbose==2) { 
	    printf("unable to achieve desired accuracy in int2 integration...k=%e, res=%e...\n",k,res);
	  }
	  res=res/k;
	  return res;
	}
		       
	printf("Failure in int2-routine");
	return 0;

	//checked with Mathematica for k=0.26 & k=0.8
}



double int1b(chi)
double chi;
{
  double specaw[48000];
  double temp,res,err,nastyfunc();
  void intdeini(int lenaw, double tiny, double eps, double *aw);
  void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);
  
  kz=k*chi;
  intdeini(48000, TINY, INTACC, specaw);
  intde(nastyfunc,0,PI, specaw, &temp, &err);
  if (err<0&&verbose==2) { if (k<100)
    {
    printf("unable to achieve desired accuracy in int1b integration...
at k=%e,chi=%e,\n",k,chi); }
  }
  res=temp*2;
  //printf("2 Pi:%e",res/nastyfunc(0));
  return res;
}

double nastyfunc(double phi)
{
  double res,arg,temp,polylog(),log();
  //printf("k,kz,phi:%e,%e,%e    ",k,kz,phi);
  temp=nx*sqrt(1-kz/k*kz/k)*cos(phi)+nz*kz/k;
  arg=sqrt(1+asi*temp*temp);
  //printf("arg:%e\n",arg);
  res=-polylog(2,-exp(-qs*k*arg))
       +qs*k*arg*log(1+exp(-qs*k*arg));
  res/=arg*arg;
  //res=polylog(2,-exp(-qs*k*arg));
  //res=-k*qs*arg*log(1+exp(-qs*k*arg));
  //printf("res:%e\n",res);
  return res;
}


double int1a(chi)
double chi;
{ 
  double *mylimits;	
  int n,limits();
  double temp,res,err,F();
  double myArg(),lowlim,highlim;
  void intdeini(int lenaw, double tiny, double eps, double *aw);
  void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);
  

  res=0; 
  //printf("k:%e\n",k);
  kz=k*chi;
  mylimits = dvector(0,4);

  //printf("k:%e,kz:%e,",k,kz);
  //fflush(0);

  n=limits(mylimits);
  
  //printf("n:%i,%e,%e\n",n,mylimits[0],mylimits[1]);
  /*if (n>2) {
    printf("%e,%e\n",mylimits[2],mylimits[3]);
    }*/
  //lowlim=mylimits[0];
  //highlim= mylimits[1];
  //lowlim=0.0;
  //highlim= 0.5;
  if(mylimits[0]+INTACC<-k/v) {lowlim=-k/v;}
  if(mylimits[0]+INTACC>-k/v) {lowlim=mylimits[0]+INTACC;}
  //printf("myarg: %e\n",myArg(lowlim+1E-10));
  //printf("myarg: %e\n",myArg(highlim-1E-10));
  //printf("%e,%e\n",lowlim,highlim);
  if (n==0) return 0.0;
  if (n==2) {
    highlim=mylimits[1]-INTACC;
    if (lowlim<highlim) {
      intdeini(lenaw, TINY, INTACC, aw);
      intde(myArg,lowlim,highlim, aw, &temp, &err);
      if (err<0&&verbose==2) { 
	printf("unable to achieve desired accuracy in int1a integration...
at k=%e,chi=%e,\n",k,chi); 
      }
      res=temp;
    }
  }
 if (n==4) {
    highlim=mylimits[1]-INTACC;  
    if (lowlim<highlim) {
      intdeini(lenaw, TINY, INTACC, aw);
      intde(myArg,lowlim,highlim, aw, &temp, &err);
      if (err<0&&verbose==2) { 
	printf("unable to achieve desired accuracy in int1 integration...\n"); 
      }
      res=temp;
    }
    if (lowlim<mylimits[2]+INTACC) {lowlim=mylimits[2]+INTACC;}
    else
      { if (lowlim<mylimits[3]-INTACC)
	{
	  intdeini(lenaw, TINY, INTACC, aw);
	  intde(myArg,mylimits[2]+INTACC,mylimits[3]-INTACC, aw, &temp, &err);
	  if (err<0&&verbose==2) { 
	    printf("unable to achieve desired accuracy in int1a integration...at k=%e,chi=%e,\n",k,chi); 
	  }
	  res=res+temp;
	}
      else res=0.0;
      }
 }
 free_dvector(mylimits,0,4);
  //res/=(1 - v*kz/k)*(1 - v*kz/k);
  //printf("res:%e at chi: %e\n",res,chi);
  return res;
  //checked for k=1.0(& 0.8), kz=0.7, v=0.5 with Mathematica
}

double int1(double chi)
{
  //this only multiplies 1a and 1b integrations
  double res,temp;
  double int1a(),int1b();
  temp=int1a(chi);
  //printf("chi:%e, int1a:%e,\n",chi, temp);
  res=temp*int1b(chi);
  //printf("int1b:%e\n",res/temp);
  return res;
  //checked for k=0.8, kz=0.7, v=0.5, asi=1.0 with Mathematica
}

double F(x)
double x;
{
	double a,b2;
	a = 4*(k*k-kz*kz)*(1-x*x);
	b2 = ((1-v*v*x*x)+2*x*(kz - k*v));
	  return a-b2*b2;
}

double mF(x)
double x;
{
	double a,b2;
	a = 4*(k*k-kz*kz)*(1-x*x);
	b2 = ((1-v*v*x*x)+2*x*(kz - k*v));
	  return b2*b2-a;
}

int limits(lim)
double *lim;
{
	double F(),mF(),F0(),foR,foL,fo,zbrent(),golden(),mnbrak();
	double lower,x1,x2,x3,fa,fb,fc,yl,yr,ym;

	yl=0.0;
	yr=0.0;
	//looking for minima: from the left
	x1 = -1.0; x2 = -1.0+1e04*ROOTACC;
	mnbrak(&x1,&x2,&x3,&fa,&fb,&fc,mF);
	//printf("--> %e,%e,%e,%e,%e,%e,\n",x1,x2,x3,fa,fb,fc);	
	foL = golden(x1,x2,x3,mF,ROOTACC,&yl);
	//printf("left --> %e,%e,%e\n",yl,x2,x3);
		//printf("fo,y%e,%e\n",fo,y);
		//printf("--> %e,%e\n",y1,fo);
	//from the right:
	x1 = 1.0-1e04*ROOTACC; x2 = 1.0;
	mnbrak(&x1,&x2,&x3,&fa,&fb,&fc,mF);
	//printf("--> %e,%e,%e,%e,%e,%e,\n",x1,x2,x3,fa,fb,fc);	
	foR = golden(x1,x2,x3,mF,ROOTACC,&yr);
	//printf("right --> %e,%e,%e\n",yr,x2,x3);

	if ((foL>0)&&(foR>0)) {
	  //printf("here: %e",yr); 
	  return 0;}
	if ((foL<0)&&(foR<0)) {
	  if (yr-yl<1e-04) {
	    //printf("one minimum\n");
	    //printf("low: %e,x2: %e\n",mF(lower),mF(x2));
	    lim[0] = zbrent(mF,-1.0,yr,ROOTACC);
	    lim[1] = zbrent(mF,yr,1.0,ROOTACC);
	    return 2;
	  }
	  if (yr-yl>1e0-4){ 
	    /*printf("more than one mimimum:%e>%e\n",yr-yl,1e-04);
	      printf("k,kz:%e,%e\n",k,kz);
	      printf("yl:%e,yr:%e\n",yl,yr);*/
	    //testing if the function has zeros between the minima:
	    x1=yl; x2=(yl+yr)/2; x3=yr;
	    fo = golden(x1,x2,x3,F,ROOTACC,&ym);
	    //printf("medium --> %e,%e\n",ym,fo);
	    fflush(0);
	    if (fo<0)
	      {
		lim[0] = zbrent(mF,-1.0,yl,ROOTACC);
		lim[1] = zbrent(mF,yl,ym,ROOTACC);	 
		lim[2] = zbrent(mF,ym,yr,ROOTACC);
		lim[3] = zbrent(mF,yr,1.0,ROOTACC);
		return 4;
	      }
	    else
	      {
		//printf("no true zero\n");
		lim[0]=zbrent(mF,-1.0,yl,ROOTACC);
		lim[1]=zbrent(mF,yl,1.0,ROOTACC);
		return 2;
	      }
	  }
	}
	if ((foL<0)&&(foR>0)){
	  //only left minimum is real
	  lim[0] = zbrent(mF,-1.0,yl,ROOTACC);
	  lim[1] = zbrent(mF,yl,1.0,ROOTACC);
	  return 2;
	}
	if ((foL>0)&&(foR<0)){
	  //only right minimum is real
	  lim[0] = zbrent(mF,-1.0,yr,ROOTACC);
	  lim[1] = zbrent(mF,yr,1.0,ROOTACC);
	  return 2;
	}

}

double myArg(lw)
double lw;
{
  double f,F(),temp,temp2,temp3;
  f = F(lw);
  temp2=(1-lw*lw*v*v);
  temp3=(k-v*kz);
  temp=1/sqrt(f)*lw*v/temp2/temp2*(2*temp3*temp3-(1-v*v)/2*temp2);
  if (f<0) {
    //printf("fell here: lw:%e\n",lw);
    return 0;}
  //temp=lw*v/temp2/temp2*(2*temp3*temp3-(1-v*v)/2*temp2);
  //printf("o:%e\n",temp);
  return temp;
  //checked with mathematica for k=0.8, kz=0.7, v=0.5, xi=-0.5
}


int findchilimit(double eps,double* res)
{
  double *mylimits;
  double estimate,start,end,acc,chiint;

  start=-1.0;
  end=1.0;
  acc=0.01;
  estimate=1.0;

  mylimits = dvector(0,4);

  kz=-k+eps;

  //printf("k=%e,kz=%e,v=%e\n",k,kz,v);

  /*  kz=k*0.81;
  printf("lim=%i\n",limits(mylimits));

  kz=k*0.82;
  printf("lim=%i\n",limits(mylimits));

  kz=k*0.83;
  printf("lim=%i\n",limits(mylimits));

  kz=k*0.9;
  printf("lim=%i\n",limits(mylimits));

  kz=k*0.91;
  printf("lim=%i\n",limits(mylimits));
  kz=-k+eps;*/

  if (limits(mylimits)==0)
    {
      kz = k-eps;
      if (limits(mylimits)==0) {
	kz=0.0;
	if (limits(mylimits==0)) {*res=-1.0; return -1;}
      }
    }
  do { 
    for (chiint=start; chiint<=end+eps; chiint +=acc ) {
	kz=k*(chiint-acc/2);
	if((limits(mylimits)>0)||(chiint-acc/2<-1.0)) {
	  //printf("at chi:%e\n",chiint-acc/2);
	  kz=k*(chiint+acc/2);
	  //printf("at chi:%e\n",chiint); 
	  if (limits(mylimits)==0) {
	    //printf("found break at chi=%e\n",chiint);
	    estimate=chiint;}
	}
    }
    if (estimate==1.0)
      {
	//printf("finchi failure\n");
	acc=eps;
	*res=estimate;
	return 0;
      }
    start=estimate-acc;
    end=estimate+acc;
    if (end>1.0) end=1.0;
    acc=acc/10.0;
  } while (acc>eps);

  //kz=k*(1+0.05);
  //printf("th:%i",limits(mylimits));
  
 *res=estimate;
  return 1;
}
