#include <stdio.h>
#include <stdlib.h>
#include <nrutil.h>
#include <math.h>
#include <mconf.h>
#include "myio.h"
#include "mydefs.h"

#define SMALL	1.0e-15	        // used in root finding
#define INTACC	1.0e-11	        // target integration accuracy
#define ROOTACC	1.0e-12          // target root accuracy for golden
#define TINY	1.0e-300	// tiny number used by integrator
#define BIG	1.0e+300	// big number 
#define LENAW	32000		// number of quadrature points
#define IRCUT	1.0e-07		// IR cut on some integrations
#define UVCUT	1.0e4		// UV cut on inf integrations

FILE    *out,*out2,*out3,*out4;

double v,k,kz,t,w;
double q,qs,T;
double nx,nz,asi;

int     lenaw=LENAW;
double  aw[LENAW];
int verbose=1; 	// 0 - no messages
		// 1 - messages from top level integrations
	 	// 2 - messages from bottom level integrations

main(argc,argv)
int     argc;
char    **argv;
{
	int    i,j,nsteps;
	double qsmax,qsmin,th,step,temp;
	double int3(),conversion;
	double int2();
	double pow();
	char* text;

	qs=0.01;
	k=0.9;
	v = 0.9;
	kz = 0.75;

	temp = int1a(0.75/0.9);

	printf("%e\n",temp);
}


realmain(argc,argv)
int     argc;
char    **argv;
{
	int    i,j,nsteps;
	double qsmax,qsmin,th,step,temp;
	double int3(),conversion;
	double int2();
	double pow();
	char* text;


	printf("\nEnergy loss of the hard part, in units of mD^2 e^2/8/Pi\n");
	printf("Synopsis: Elosshard [paramters]\n");
	printf("where [parameters] have to be of the form: v th asi pqsmin pqsmax nsteps [outputfile]\n\n");

	if (argc!=8) {
	  printf("All parameters have to be specified! Exiting now...\n");
	  exit(1);
	}
	
	qs=0.01;
	k=0.5;
	/*v = 0.5;
	qs = 0.01;

	nz=0.9;	
	asi=1E-5;*/

	//printf("args #: %i",argc);

	v=atof(argv[1]);
	th=atof(argv[2]);
	asi=atof(argv[3]);
	qsmin=atof(argv[4]);
	qsmax=atof(argv[5]);
	nsteps=atoi(argv[6]);
	text=argv[7];

	nz=cos(th);
	nx=sin(th);
	
	printf("Calculating hard energy loss part for\n");
	printf(" v=%e\n v_z=%e\n asi=%e\n qmax=%e\n qmin=%e\n nstep=%i\n",v,nz,asi,qsmax,qsmin,nsteps);

	printf("Data output file:%s\n\n",text);
	conversion=3/PI/PI/PI/PI/v/2;
	//temp=int2();
	//temp=pow(10.0,-2.0);
	//printf("temp:%e, res:%e",temp,temp*conversion);
	
	IO(out,text,"w");

	step = (qsmax-qsmin)/nsteps;
	
	for (i=0;i<=nsteps;i++) {
	  qs = pow(10.0,qsmin + i*step); 
	  //printf("i:%i\n",i);
	  //qs=pow(10.0,qsmin);
	  temp = int3()*conversion;
	  printf("%e\t%e\n",qs,temp);
	  fprintf(out,"%e\t%e\n",qs,temp);
	}
	fclose(out);
	
	//printf("%e",temp);
	/*
	qs=0.1;
	temp=int3()*3/PI/PI/PI/PI/v/2;
	printf("temp:%e\n",temp);

	qs=1.0;
	temp=int3()*3/PI/PI/PI/PI/v/2;
	printf("temp:%e\n",temp);	

	qs=10.0;
	temp=int3()*3/PI/PI/PI/PI/v/2;
	printf("temp:%e\n",temp);	

	qs=100.0;
	temp=int3()*3/PI/PI/PI/PI/v/2;
	printf("temp:%e\n",temp);   */
}



double int3()
{
  double res,err,int2(),temp;
  void intdeiini(int lenaw, double tiny, double eps, double *aw);
  void intdei(double (*f)(double), double a, double *aw, double *i, double *err);
  void intdeini(int lenaw, double tiny, double eps, double *aw);
  void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);

  intdeini(lenaw, TINY, INTACC, aw);
  intde(int2, (1-v)/2, (1+v)/2, aw, &temp, &err);
  if (err<0&&verbose==2) { 
    printf("unable to achieve desired accuracy in int3 integration...\n"); }
  //printf("res1:%e",temp);
  res=temp;
  intdeini(lenaw, TINY, INTACC, aw);
  intde(int2,(1+v)/2,UVCUT, aw, &temp, &err);
  if (err<0&&verbose==2) { 
    printf("unable to achieve desired accuracy in int3 integration...\n"); }
  //printf("res2:%e",temp);
  res+=temp;
  return res*4;
}


double int2(zk)
double zk;
{
	double res,err,int1();
	void intdeini(int lenaw, double tiny, double eps, double *aw);
	void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);
	double chilimit;
	int j;
	double polylog();

	k=zk;

	j=findchilimit(1E-8,&chilimit);
	//printf("limits:%i, at %e",j,chilimit);
	if (j==0){
	  intdeini(lenaw, TINY, INTACC, aw);
	  intde(int1, -1.0, 1.0, aw, &res, &err);
	  if (err<0&&verbose==2) { 
	    printf("unable to achieve desired accuracy in int2 integration...\n"); }
	  return res/k;
	}
	if (chilimit==0) {printf("0 chilim: failure at k=%e\n",zk);}
	if (int1(-1+1E-8)<1E-12) { 
	  intdeini(lenaw, TINY, INTACC, aw);
	  intde(int1, chilimit, 1.0, aw, &res, &err);
	  if (err<0&&verbose==2) { 
	    printf("unable to achieve desired accuracy in int2 integration...\n");
	  }
	  return res/k;
	}
	if (int1(1-1E-8)<1E-12) { 
	  intdeini(lenaw, TINY, INTACC, aw); 
	  intde(int1, -1.0,chilimit, aw, &res, &err);
	  //printf("Integrating from -1 to chilim");
	  if (err<0&&verbose==2) { 
	    printf("unable to achieve desired accuracy in int2 integration...\n");
	  }
	  res=res/k;
	  return res;
	}
		       
	printf("Failure in int2-routine");
	return 0;

	//checked with Mathematica for k=0.26 & k=0.8
}



double int1b(chi)
double chi;
{
  double temp,res,err,nastyfunc();
  void intdeini(int lenaw, double tiny, double eps, double *aw);
  void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);
  
  kz=k*chi;
  intdeini(lenaw, TINY, INTACC, aw);
  intde(nastyfunc,0,PI, aw, &temp, &err);
  if (err<0&&verbose==2) { 
    printf("unable to achieve desired accuracy in int1 integration...\n"); 
  }
  res=temp*2;
  //printf("2 Pi:%e",res/nastyfunc(0));
  return res;
}

double nastyfunc(double phi)
{
  double res,arg,temp,polylog(),log();
  //printf("k,kz,phi:%e,%e,%e    ",k,kz,phi);
  temp=nx*sqrt(1-kz/k*kz/k)*cos(phi)+nz*kz/k;
  arg=sqrt(1+asi*temp*temp);
  //printf("arg:%e\n",arg);
  res=-polylog(2,-exp(-qs*k*arg))
       +qs*k*arg*log(1+exp(-qs*k*arg));
  res/=arg*arg;
  //res=polylog(2,-exp(-qs*k*arg));
  //res=-k*qs*arg*log(1+exp(-qs*k*arg));
  //printf("res:%e\n",res);
  return res;
}


double int1a(chi)
double chi;
{ 
  double *mylimits;	
  int n,limits();
  double temp,res,err,F();
  double myArg(),lowlim,highlim;
  void intdeini(int lenaw, double tiny, double eps, double *aw);
  void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);
  

  res=0; 
  //printf("k:%e\n",k);
  kz=k*chi;
  mylimits = dvector(0,4);
  
  // printf("k:%e,kz:%e,",k,kz);
  //fflush(0);
  n=limits(mylimits);

  printf("n:%i,%e,%e\n",n,mylimits[0],mylimits[1]);
  
  //lowlim=mylimits[0];
  //highlim= mylimits[1];
  //lowlim=0.0;
  //highlim= 0.5;
  lowlim=mylimits[0]+1E-11;
  highlim=mylimits[1]-1E-11;
  //printf("myarg: %e\n",myArg(lowlim+1E-10));
  //printf("myarg: %e\n",myArg(highlim-1E-10));
  //printf("%e,%e\n",lowlim,highlim);
  if (n==0) return 0.0;
  /*  if (n==2) {
    intdeini(lenaw, TINY, INTACC, aw);
    intde(myArg,lowlim,(highlim+lowlim)/2, aw, &temp, &err);
    if (err<0&&verbose==2) { 
      printf("unable to achieve desired accuracy in int1 integration...\n"); 
    }
    res=temp;
    //printf("temp:%e\n",temp);
    intdeini(lenaw, TINY, INTACC, aw);
    intde(myArg,(highlim+lowlim)/2, highlim,aw, &temp, &err);
    if (err<0&&verbose==2) { 
      printf("unable to achieve desired accuracy in int1 integration...\n"); 
      }*/
  if (n==2) {
    intdeini(lenaw, TINY, INTACC, aw);
    intde(myArg,lowlim,highlim, aw, &temp, &err);
    if (err<0&&verbose==2) { 
      printf("unable to achieve desired accuracy in int1 integration...\n"); 
    }
    res=temp;
  }
  free_dvector(mylimits,0,4);
  //res/=(1 - v*kz/k)*(1 - v*kz/k);
  //printf("res:%e at chi: %e\n",res,chi);
  return res;
  //checked for k=1.0(& 0.8), kz=0.7, v=0.5 with Mathematica
}

double int1(double chi)
{
  //this only multiplies 1a and 1b integrations
  double res,temp;
  double int1a(),int1b();
  temp=int1a(chi);
  //printf("int1a:%e,\n",temp);
  res=temp*int1b(chi);
  //printf("int1b:%e\n",res/temp);
  return res;
  //checked for k=0.8, kz=0.7, v=0.5, asi=1.0 with Mathematica
}

double F(x)
double x;
{
	double a,b2;
	a = 4*(k*k-kz*kz)*(1-x*x);
	b2 = ((1-v*v*x*x)+2*x*(kz - k*v));
	  return a-b2*b2;
}

double mF(x)
double x;
{
	double a,b2;
	a = 4*(k*k-kz*kz)*(1-x*x);
	b2 = ((1-v*v*x*x)+2*x*(kz - k*v));
	  return b2*b2-a;
}

int limits(lim)
double *lim;
{
	double F(),mF(),F0(),fo,zbrent(),golden(),mnbrak();
	double lower,x1,x2,x3,fa,fb,fc,y;

	y=0.0;
	//fo=F0();
	lower=-1.0;
	if (-k/v>-1.0) {lower=-k/v;}

	x1 = -1.0e-04; x2 = 1.0e-04;
	mnbrak(&x1,&x2,&x3,&fa,&fb,&fc,mF);
	//printf("--> %e,%e,%e,%e,%e,%e,\n",x1,x2,x3,fa,fb,fc);	
	fo = golden(x1,x2,x3,mF,ROOTACC,&y);
	//printf("--> %e,%e,%e\n",y,x2,x3);
		//printf("fo,y%e,%e\n",fo,y);
		//printf("--> %e,%e\n",y1,fo);
	if (fo<0) {
	  //printf("low: %e,x2: %e\n",mF(lower),mF(x2));
	  lim[0] = zbrent(mF,lower,y,ROOTACC);
	  lim[1] = zbrent(mF,y,1.0,ROOTACC);
	  return 2;
	}
	if (fo>0) {
	  return 0;
	}
	return 0;
}

double myArg(lw)
double lw;
{
  double f,F(),temp,temp2,temp3;
  f = F(lw);
  temp2=(1-lw*lw*v*v);
  temp3=(k-v*kz);
  temp=1/sqrt(f)*lw*v/temp2/temp2*(2*temp3*temp3-(1-v*v)/2*temp2);
  if (f<0) {
    //printf("fell here: lw:%e\n",lw);
    return 0;}
  //temp=lw*v/temp2/temp2*(2*temp3*temp3-(1-v*v)/2*temp2);
  //printf("o:%e\n",temp);
  return temp;
  //checked with mathematica for k=0.8, kz=0.7, v=0.5, xi=-0.5
}


int findchilimit(double eps,double* res)
{
  double *mylimits;
  double estimate,start,end,acc,chiint;

  start=-1.0;
  end=1.0;
  acc=0.1;
  estimate=0.0;

  mylimits = dvector(0,4);

  kz=-k+eps;

  //printf("limits:%i",limits(mylimits));

  chiint=-1.0;
  kz = k*chiint+eps;
  if (limits(mylimits)==2)
    {
      kz = k-eps;
      if (limits(mylimits)==2) return 0;
    }
  kz=-k+eps;
  if (limits(mylimits)==0)
    {
      kz = k-eps;
      if (limits(mylimits)==0) {*res=-1.0; return -1;}
    }
  do { 
    for (chiint=start; chiint<=end+eps; chiint +=acc ) {
	kz=k*(chiint-acc/2);
	if((limits(mylimits)==2)||(chiint-acc/2<-1.0)) {
	  //printf("at chi:%e\n",chiint);
	  kz=k*(chiint+acc/2); 
	  if ((limits(mylimits)!=2)||(chiint+acc/2>1.0)) {
	    estimate=chiint;}
	}
    }
    if (estimate==0)
      {
	printf("finchi failure\n");
	acc=eps;
      }   
    start=estimate-acc;
    end=estimate+acc;
    if (end>1.0) end=1.0;
    acc=acc/10.0;
  } while (acc>eps);

  //kz=k*(1+0.05);
  //printf("th:%i",limits(mylimits));
  
 *res=estimate;
  return 1;
}
