#Mike's code
#include <stdio.h>
#include <stdlib.h>
#include <nrutil.h>
#include <math.h>
#include "myio.h"
#include "mydefs.h"

#define SMALL	1.0e-05		// used in root finding
#define INTACC	1.0e-05		// target integration accuracy
#define ROOTACC	1.0e-12		// target root accuracy for zbrent
#define TINY	1.0e-300	// tiny number used by integrator
#define BIG	1.0e+300	// big number 
#define LENAW	8000		// number of quadrature points
#define IRCUT	1.0e-07		// IR cut on some integrations
#define UVCUT	1.0e3		// UV cut on inf integrations

FILE    *out,*out2,*out3,*out4;

double v,tv,k,t,w;
double qs,T;

int     lenaw=LENAW;
double  aw[LENAW];
int verbose=2; 	// 0 - no messages
		// 1 - messages from top level integrations
	 	// 2 - messages from bottom level integrations

main(argc,argv)
int     argc;
char    **argv;
{
	int    i,j,nsteps;
	double t0,tf,step,temp;
	double int1(),int2(),result();
	double F();

	v = 0.75;
	tv = 0.9;
	qs = 0.1;
	T = 1.0;
	//printf("%e",F(0.5));
	k = 1.0;
	t = Pi/3;

	//	printf("%e\t%e\n",qs,result());
	//exit(0);
	
	printf("%e\n",int1(-0.1));

	/*IO(out,"data.txt","w");
	t0 = -5;
	tf = -2;
	nsteps=10;
	step = (tf-t0)/nsteps;	
	for (i=0;i<=nsteps;i++) {
		qs = exp(t0 + i*step);
		temp = result();
		printf("%e\t%e\n",qs,temp);
		fprintf(out,"%e\t%e\n",qs,temp);
	}
	fclose(out);*/
	exit(0);

}

double result()
{
	double res,err,int2();
	void intdeini(int lenaw, double tiny, double eps, double *aw);
	void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);

        intdeini(lenaw, TINY, INTACC, aw);
        intde(int2, IRCUT, UVCUT*T/qs, aw, &res, &err);
        if (err<0&&verbose!=0) { printf("unable to achieve desired accuracy in the final integration...\n"); }
        return res;
	
}

double int2(lk)
double lk;
{
	double res,err,int1();
	void intdeini(int lenaw, double tiny, double eps, double *aw);
	void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);

	k=lk;
        intdeini(lenaw, TINY, INTACC, aw);
        intde(int1, 0.0, Pi, aw, &res, &err);
        if (err<0&&verbose==2) { printf("unable to achieve desired accuracy in int2 integration...\n"); }
        return res*k*log(1+exp(-qs*k/T));
	
}

double int1(lt)
double lt;
{
	int n,limits();
	double temp,res,err,myArg();
	void intdeini(int lenaw, double tiny, double eps, double *aw);
	void intde(double (*f)(double), double a, double b, double *aw, double *i, double *err);
	double *mylimits;

	mylimits = dvector(0,4);

	t=lt;
	n = limits(mylimits);
	printf("n:,%i",n);
	if (n==0) return 0.0;
	if (n==2) {
	        intdeini(lenaw, TINY, INTACC, aw);
	        intde(myArg, mylimits[0], mylimits[1],aw, &res, &err);
	        if (err<0&&verbose==2) { printf("unable to achieve desired accuracy in int1 integration...\n"); }
	        temp = res;
	}
	if (n==4) {
	        intdeini(lenaw, TINY, INTACC, aw);
	        intde(myArg, mylimits[0], mylimits[1], aw, &res, &err);
	        if (err<0&&verbose==2) { printf("unable to achieve desired accuracy in int1 integration...\n"); }
		temp = res;
	        intdeini(lenaw, TINY, INTACC, aw);
	        intde(myArg, mylimits[2], mylimits[3], aw, &res, &err);
	        if (err<0&&verbose==2) { printf("unable to achieve desired accuracy in int1 integration...\n"); }
		temp += res;
	}
	if (n<0||n==1||n==3||n>4){ 
		printf("unhandled case in int1 routine (n=%d) - exiting now\n",n);
		exit(0);
	}
	free_dvector(mylimits,0,4);
	return sin(t)*temp;
}


double F(x)
double x;
{
	double a,b2;
	a = 2*x*k/v - (x*(x+2*k)-1.0)*cos(tv-t); 
        b2 = (1-x*x)*((x+2*k)*(x+2*k)-1.0)*sin(tv-t)*sin(tv-t); 
	return b2-a*a;
}

double mF(x)
double x;
{ return -F(x); }

double F0()
{ return -1.0+2*k*k*(1.0-cos(2*(t-tv))); }

int limits(lim)
double *lim;
{
	double F(),mF(),F0(),fo,zbrent(),golden(),mnbrak();
	double wo,x1,x2,x3,fa,fb,fc,y;

	y=0.0;
	fo=F0();
	wo= fabs(1.0-k)-k;

	x1 = -1.0e-04; x2 = 1.0e-04;
	mnbrak(&x1,&x2,&x3,&fa,&fb,&fc,F);
	//printf("--> %e,%e,%e,%e\n",x1,x2,x3,wo);
	if ( (x1>=wo && x1<=1.0) && (x2>=wo && x2<=1.0) && (x3>=wo && x3<=1.0) ) {
		fo = golden(x1,x2,x3,F,ROOTACC,&y);
		//printf("--> %e,%e,%e\n",x1,x2,x3);
		//printf("--> %e,%e\n",y1,fo);
	}

	if (fo==0) {
		printf("unhandled case in limits routine - fo=0 - exiting now\n");
		exit(0);
	}
	if (fo>0) {
		golden(wo,y,1.0,mF,ROOTACC,&x1);
		//printf("%e\n",x1);
		lim[0] = zbrent(F,wo,x1,ROOTACC);
        	lim[1] = zbrent(F,x1,1.0,ROOTACC);
		return 2;
	}
	if (fo<0) {
		golden(wo,wo/2,y,mF,ROOTACC,&x1);
		golden(y,0.5,1.0,mF,ROOTACC,&x2);
		if (F(x1)>0 && F(x2)>0) {
			lim[0] = zbrent(F,wo,x1,ROOTACC);
        		lim[1] = zbrent(F,x1,y,ROOTACC);
			lim[2] = zbrent(F,y,x2,ROOTACC);
        		lim[3] = zbrent(F,x2,1.0,ROOTACC);
			return 4;
		}
		if (F(x1)>0 && F(x2)<=0) {
			lim[0] = zbrent(F,wo,x1,ROOTACC);
        		lim[1] = zbrent(F,x1,y,ROOTACC);
			return 2;
		}
		if (F(x1)<=0 && F(x2)>0) {
			lim[0] = zbrent(F,y,x2,ROOTACC);
        		lim[1] = zbrent(F,x2,1.0,ROOTACC);
			return 2;
		}
		if (F(x1)<=0 && F(x2)<=0)  {
			return 0;
		}
	}
	return 0;
}

double myArg(lw)
double lw;
{
	double f,F(),temp,ct,c2t,c2tv;
	f = F(lw);
	ct = cos(t);
	c2t = cos(2*t);
	c2tv = cos(2*tv);
	if (f<=0) { return 0; }
	temp = lw/((lw*lw-1.0)*(lw*lw-1.0)*sqrt(f));
	temp *= 1.0-2*v*ct*c2tv+v*v*(3.0+c2tv+c2t*(1.0+3*c2tv))/8.0 + (1-v*v)*(lw*lw-1)/(4*k*k);
	return temp;
}


