(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   5088150,     143817]*)
(*NotebookOutlinePosition[   5091674,     143909]*)
(*  CellTagsIndexPosition[   5091111,     143893]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Structure functions", "Title"],

Cell[CellGroupData[{

Cell["alpha", "Section"],

Cell[CellGroupData[{

Cell["Exact results -numerical ", "Subsection"],

Cell[CellGroupData[{

Cell["Preliminaries", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(s = Sqrt[\((1 - om^2/k^2)\)*\((1 - kz^2/k^2)\)]\)], "Input"],

Cell[BoxData[
    \(\@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi1[k_, kz_, om_] = om*kz/k^2 + s\)], "Input"],

Cell[BoxData[
    \(\(kz\ om\)\/k\^2 + \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi2[k_, kz_, om_] = om*kz/k^2 - s\)], "Input"],

Cell[BoxData[
    \(\(kz\ om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Imaginary part:", "Subsubsection"],

Cell[BoxData[
    \(Imalpha[md2_, k_, kz_, om_, a_] := 
      If[om > k, 
        0, \(-md2\)/\((k^2 - kz^2)\)*k/2*
          NIntegrate[
            1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
              Sqrt[\(-\((k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - om^2)\)*\((1 - 
                            kz^2/k^2)\))\)\)], {xi, 
              xi2[k, kz, om]*
                If[xi2[k, kz, om] > 
                    0, \((1 + 10^\((\(-50\))\))\), \((1 - 
                      10^\((\(-50\))\))\)], xi1[k, kz, om]}, 
            WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30]]\)], "Input",\

  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imalpha[1, 1, 1 - 10^\((\(-15\))\), 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.00015707962796710028883939564150404771`21.0756\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Real part:", "Subsubsection"],

Cell[BoxData[
    \(Realpha[md2_, k_, kz_, om_, a_] := 
      Module[{res, mxi1, mxi2}, 
        If[logging \[Equal] 0, Print["\<alpha\>"]]; \[IndentingNewLine]mxi1 = 
          If[k > om, xi1[k, kz, om]]; 
        mxi2 = If[k > om, xi2[k, kz, om]]; \[IndentingNewLine]If[om < k, 
          If[kz/k < 1 - 10^\((\(-10\))\), 
            res = \(md2/\((k^2 - kz^2)\)\)/
                  2*\((NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om^2 - 
                            a\ kz^2  xi^2)\), {xi, \(-1\), 1}, 
                      WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30] - 
                    NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
                        Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                  om^2)\)*\((1 - kz^2/k^2)\)], {xi, \(-1\), 
                        mxi2*If[
                            mxi2 < 
                              0, \((1 + 10^\((\(-50\))\))\), \((1 - 
                                10^\((\(-50\))\))\)]}, 
                      WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30] - 
                    Sign[om - kz]*
                      NIntegrate[
                        1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
                          Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                    om^2)\)*\((1 - kz^2/k^2)\)], {xi, 
                          mxi1*\((1 + 10^\((\(-50\))\))\), 1}, 
                        WorkingPrecision \[Rule] 30, 
                        MaxRecursion \[Rule] 30])\), 
            res = \(md2/\((k^2 - kz^2)\)\)/
                  2*\((NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om^2 - 
                            a\ kz^2\ xi^2)\), {xi, \(-1\), 1}, 
                      WorkingPrecision \[Rule] 50, MaxRecursion \[Rule] 30] - 
                    NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
                        Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                  om^2)\)*\((1 - kz^2/k^2)\)], {xi, \(-1\), 
                        mxi2*If[
                            mxi2 < 
                              0, \((1 + 10^\((\(-50\))\))\), \((1 - 
                                10^\((\(-50\))\))\)]}, 
                      WorkingPrecision \[Rule] 50, MaxRecursion \[Rule] 30] - 
                    Sign[om - kz]*
                      NIntegrate[
                        1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
                          Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                    om^2)\)*\((1 - kz^2/k^2)\)], {xi, 
                          mxi1*\((1 + 10^\((\(-50\))\))\), 1}, 
                        WorkingPrecision \[Rule] 50, 
                        MaxRecursion \[Rule] 30])\)]]; 
        If[om > k, 
          If[kz/k < 1 - 10^\((\(-10\))\), 
            res = \(md2/\((k^2 - kz^2)\)\)/
                  2*\((NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om^2 - 
                            a\ kz^2\ xi^2)\), {xi, \(-1\), 1}, 
                      WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30] - 
                    NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
                        Sign[om - kz\ xi]*
                        Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                  om^2)\)*\((1 - kz^2/k^2)\)], {xi, \(-1\), 
                        1}, WorkingPrecision \[Rule] 30, 
                      MaxRecursion \[Rule] 30])\), 
            res = \(md2/\((k^2 - kz^2)\)\)/
                  2*\((NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om^2 - 
                            a\ kz^2\ xi^2)\), {xi, \(-1\), 1}, 
                      WorkingPrecision \[Rule] 50, MaxRecursion \[Rule] 30] - 
                    NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
                        Sign[om - kz\ xi]*
                        Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                  om^2)\)*\((1 - kz^2/k^2)\)], {xi, \(-1\), 
                        1}, WorkingPrecision \[Rule] 50, 
                      MaxRecursion \[Rule] 30])\)]]; res]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      Integrate[\((1 - xi^2)\)/\((1 + a\ xi^2)\)^2, {xi, \(-1\), 
          1}]]\)], "Input"],

Cell[BoxData[
    \(\(\@a + \((\(-1\) + a)\)\ ArcTan[\@a]\)\/a\^\(3/2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% /. a \[Rule] 1\)], "Input"],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(fullalpha[md2_, k_, kz_, om_, a_] := 
      Realpha[md2, k, kz, om, a] + I*Imalpha[md2, k, kz, om, a]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fullalpha[1, 1/100, 1/200, 1, 1/1000]\)], "Input"],

Cell[BoxData[
    \(0.3332067514131851558369935621603759058`25.0797\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\((1 - xi^2)\)/\((1 + a\ xi^2)\)^2, {xi, \(-1\), 
        1}]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\(-\@a\) - a\^\(3/2\) + ArcTan[\@a] - 
              a\^2\ ArcTan[\@a]\)\/\(2\ a\^\(3/2\)\ \((1 + 
                  a)\)\)\)\) + \(\@a + a\^\(3/2\) - ArcTan[\@a] + a\^2\ \
ArcTan[\@a]\)\/\(2\ a\^\(3/2\)\ \((1 + a)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input"],

Cell[BoxData[
    \(\(\@a + \((\(-1\) + a)\)\ ArcTan[\@a]\)\/a\^\(3/2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Limit[%, a \[Rule] 0]\)], "Input"],

Cell[BoxData[
    \(4\/3\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\((%89)\)*3/4, {a, \(-1\), 10}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.11039 0.0865801 0.0147151 0.12125 [
[.28355 .00222 -3 -9 ]
[.28355 .00222 3 0 ]
[.45671 .00222 -3 -9 ]
[.45671 .00222 3 0 ]
[.62987 .00222 -3 -9 ]
[.62987 .00222 3 0 ]
[.80303 .00222 -3 -9 ]
[.80303 .00222 3 0 ]
[.97619 .00222 -6 -9 ]
[.97619 .00222 6 0 ]
[.09789 .13596 -6 -4.5 ]
[.09789 .13596 0 4.5 ]
[.09789 .25721 -6 -4.5 ]
[.09789 .25721 0 4.5 ]
[.09789 .37846 -6 -4.5 ]
[.09789 .37846 0 4.5 ]
[.09789 .49971 -6 -4.5 ]
[.09789 .49971 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.28355 .01472 m
.28355 .02097 L
s
[(2)] .28355 .00222 0 1 Mshowa
.45671 .01472 m
.45671 .02097 L
s
[(4)] .45671 .00222 0 1 Mshowa
.62987 .01472 m
.62987 .02097 L
s
[(6)] .62987 .00222 0 1 Mshowa
.80303 .01472 m
.80303 .02097 L
s
[(8)] .80303 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(10)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.15368 .01472 m
.15368 .01847 L
s
.19697 .01472 m
.19697 .01847 L
s
.24026 .01472 m
.24026 .01847 L
s
.32684 .01472 m
.32684 .01847 L
s
.37013 .01472 m
.37013 .01847 L
s
.41342 .01472 m
.41342 .01847 L
s
.5 .01472 m
.5 .01847 L
s
.54329 .01472 m
.54329 .01847 L
s
.58658 .01472 m
.58658 .01847 L
s
.67316 .01472 m
.67316 .01847 L
s
.71645 .01472 m
.71645 .01847 L
s
.75974 .01472 m
.75974 .01847 L
s
.84632 .01472 m
.84632 .01847 L
s
.88961 .01472 m
.88961 .01847 L
s
.9329 .01472 m
.9329 .01847 L
s
.0671 .01472 m
.0671 .01847 L
s
.02381 .01472 m
.02381 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.11039 .13596 m
.11664 .13596 L
s
[(1)] .09789 .13596 1 0 Mshowa
.11039 .25721 m
.11664 .25721 L
s
[(2)] .09789 .25721 1 0 Mshowa
.11039 .37846 m
.11664 .37846 L
s
[(3)] .09789 .37846 1 0 Mshowa
.11039 .49971 m
.11664 .49971 L
s
[(4)] .09789 .49971 1 0 Mshowa
.125 Mabswid
.11039 .03897 m
.11414 .03897 L
s
.11039 .06322 m
.11414 .06322 L
s
.11039 .08746 m
.11414 .08746 L
s
.11039 .11171 m
.11414 .11171 L
s
.11039 .16021 m
.11414 .16021 L
s
.11039 .18446 m
.11414 .18446 L
s
.11039 .20871 m
.11414 .20871 L
s
.11039 .23296 m
.11414 .23296 L
s
.11039 .28146 m
.11414 .28146 L
s
.11039 .30571 m
.11414 .30571 L
s
.11039 .32996 m
.11414 .32996 L
s
.11039 .35421 m
.11414 .35421 L
s
.11039 .40271 m
.11414 .40271 L
s
.11039 .42696 m
.11414 .42696 L
s
.11039 .45121 m
.11414 .45121 L
s
.11039 .47546 m
.11414 .47546 L
s
.11039 .52396 m
.11414 .52396 L
s
.11039 .54821 m
.11414 .54821 L
s
.11039 .57246 m
.11414 .57246 L
s
.11039 .59671 m
.11414 .59671 L
s
.25 Mabswid
.11039 0 m
.11039 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02477 .61803 m
.02499 .44605 L
.02605 .39057 L
.02729 .35399 L
.02846 .33042 L
.02954 .31374 L
.03053 .30113 L
.03279 .27893 L
.03506 .2621 L
.03756 .24753 L
.04262 .22561 L
.04755 .21003 L
.05285 .19706 L
.06244 .17957 L
.07373 .16477 L
.08445 .15411 L
.10458 .13937 L
.12439 .12885 L
.14277 .12125 L
.1819 .10926 L
.22348 .10023 L
.26355 .09365 L
.3021 .0886 L
.3431 .08418 L
.38259 .08062 L
.42056 .07767 L
.46099 .07493 L
.4999 .07262 L
.54126 .07043 L
.5811 .06854 L
.61943 .0669 L
.66022 .06531 L
.69948 .06391 L
.73723 .06267 L
.77744 .06144 L
.81612 .06035 L
.85726 .05927 L
.89689 .0583 L
.935 .05743 L
.97556 .05655 L
.97619 .05654 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00DP3oool4000002d0oooo00<000000?ooo`3oool0:`3oool2000002h0
oooo0P00000Y0?ooo`@000000`3oool2000000/0oooo001B0?ooo`030000003oool0oooo02h0oooo
00<000000?ooo`3oool0:P3oool010000000oooo0?ooo`00000/0?ooo`040000003oool0oooo0000
02X0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00P0oooo001C0?oo
o`030000003oool0oooo02X0oooo1@00000[0?ooo`040000003oool0oooo000002`0oooo00@00000
0?ooo`3oool00000:P3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0
203oool005@0oooo00<000000?ooo`3oool0:@3oool010000000oooo0?ooo`00000/0?ooo`<00000
;P3oool2000002/0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00P0
oooo001E0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`000000;03oool00`000000oooo
0?ooo`0]0?ooo`040000003oool0oooo000002X0oooo00D000000?ooo`3oool0oooo000000020?oo
o`030000003oool0oooo00P0oooo001B0?ooo`040000003oool0oooo000002`0oooo0P00000/0?oo
o`030000003oool0oooo02d0oooo00@000000?ooo`3oool00000:03oool3000000<0oooo00@00000
0?ooo`3oool000002P3oool002D0oooo00<000000?ooo`3oool0:`3oool2000002h0oooo00<00000
0?ooo`3oool0:`3oool3000002d0oooo0P00000[0?ooo`030000003oool0oooo0080oooo0P00000;
0?ooo`009@3oool00`000000oooo0?ooo`3h0?ooo`009@3oool00`000000oooo0?ooo`3h0?ooo`00
9@3oool00`000000oooo0?ooo`3h0?ooo`001`3ooooo000001800000203oool000d0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`009@3oool00`000000oooo0?ooo`0/0?oo
o`030000003oool0oooo02`0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0]0?oo
o`030000003oool0oooo02`0oooo00<000000?ooo`3oool0303oool002D0oooo00<000000?ooo`3o
ool0n03oool002D0oooo00<000000?ooo`3oool0n03oool002D0oooo00<000000?ooo`3oool0n03o
ool002D0oooo00<000000?ooo`3oool0n03oool002D0oooo0P00003i0?ooo`009@3oool00`000000
oooo0?ooo`3h0?ooo`009@3oool00`000000oooo0?ooo`3h0?ooo`009@3oool00`000000oooo0?oo
o`3h0?ooo`009@3oool00`000000oooo0?ooo`3T0?ooo`H000003P3oool002D0oooo00<000000?oo
o`3oool0^03oool/000001@0oooo000U0?ooo`800000V@3ooolP00000400oooo000U0?ooo`030000
003oool0oooo08<0oooo5@00001P0?ooo`009@3oool00`000000oooo0?ooo`1]0?oooaH00000M@3o
ool002D0oooo00<000000?ooo`3oool0E`3ooolF000008/0oooo000U0?ooo`030000003oool0oooo
04`0oooo2`00002Q0?ooo`009@3oool00`000000oooo0?ooo`120?ooo`X00000[03oool002D0oooo
00<000000?ooo`3oool0=`3oool;00000;H0oooo000U0?ooo`800000;@3oool;00000<40oooo000U
0?ooo`030000003oool0oooo02D0oooo1`00003<0?ooo`009@3oool00`000000oooo0?ooo`0O0?oo
o`H00000d`3oool002D0oooo00<000000?ooo`3oool06P3oool500000=T0oooo000U0?ooo`030000
003oool0oooo01H0oooo1000003N0?ooo`009@3oool00`000000oooo0?ooo`0B0?ooo`@00000hP3o
ool002D0oooo0P00000@0?ooo`<00000iP3oool002D0oooo00<000000?ooo`3oool02`3oool40000
0>T0oooo000U0?ooo`030000003oool0oooo00P0oooo0`00003]0?ooo`009@3oool00`000000oooo
0?ooo`050?ooo`<00000l03oool002D0oooo00<000000?ooo`3oool00`3oool200000?<0oooo000L
0?ooo`@000001@3oool00`000000oooo0?ooo`0300000?D0oooo000N0?ooo`030000003oool0oooo
00@0oooo0`00003h0?ooo`007P3oool00`000000oooo0?ooo`030?ooo`<00000n@3oool001h0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0000003j0?ooo`007P3oool01@000000oooo0?oo
o`3oool000000080oooo00<000000?ooo`3oool0n03oool001`0oooo0`0000000`3oool000000000
00030?ooo`030000003oool0oooo0?P0oooo000N0?ooo`8000001@3oool00`000000oooo0?ooo`3h
0?ooo`007P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?P0oooo000M0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0n03oool001`0oooo00<000000?ooo`3oool0
1P3oool200000?T0oooo000K0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0n03o
ool001X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3h0?ooo`006P3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo0?P0oooo000I0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0n03oool001P0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`3h0?ooo`005`3oool00`000000oooo0?ooo`0;0?ooo`800000n@3oool001L0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`3h0?ooo`005P3oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo0?P0oooo000F0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool0n03oool001D0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3h0?ooo`005@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?P0oooo000D0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0n03oool001@0oooo00<000000?ooo`3oool03P3oool20000
0?T0oooo000D0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0n03oool001<0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3h0?ooo`004`3oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo0?P0oooo000B0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0n03oool00180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3h0?oo
o`004P3oool00`000000oooo0?ooo`0@0?ooo`800000n@3oool00180oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`3h0?ooo`004@3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo0?P0oooo000A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0n03oool0
0140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3h0?ooo`004@3oool00`000000
oooo0?ooo`080?ooo`@000001@3oool00`000000oooo0?ooo`3h0?ooo`004@3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0n03oool00100oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`050?ooo`800000n@3oool00100oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?P0oooo000@0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3h0?oo
o`00403oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo000000D0oooo00<000000?oo
o`3oool0n03oool00100oooo00<000000?ooo`3oool02P3oool2000000H0oooo00<000000?ooo`3o
ool0n03oool00100oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3h0?ooo`00403o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0?P0oooo000@0?ooo`030000003oool0
oooo0180oooo0P00003i0?ooo`00403oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
0?P0oooo000@0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0n03oool000l0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3h0?ooo`003`3oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo0?P0oooo000?0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0n03oool000l0oooo00<000000?ooo`3oool04`3oool200000?T0oooo000?0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool0n03oool000l0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`3h0?ooo`003`3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo0?P0oooo000?0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0n03oool0
00l0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3h0?ooo`003P3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0?P0oooo000>0?ooo`030000003oool0oooo01@0oooo
0P00003i0?ooo`003P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0?P0oooo000>
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0n03oool000h0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`3h0?ooo`003P3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo0?P0oooo000>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
n03oool000h0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`3h0?ooo`003P3oool0
0`000000oooo0?ooo`0D0?ooo`800000n@3oool000h0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`3h0?ooo`003P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0?P0
oooo000>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0n03oool000h0oooo00<0
00000?ooo`3oool0303oool2000000H0oooo00<000000?ooo`3oool0n03oool000h0oooo00<00000
0?ooo`3oool02`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo0?P0oooo000>
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool00`3oool200000?T0oooo000>0?oo
o`030000003oool0oooo00`0oooo0P0000060?ooo`030000003oool0oooo0?P0oooo000>0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3h0?ooo`00
3P3oool00`000000oooo0?ooo`0;0?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3o
ool0n03oool000h0oooo00<000000?ooo`3oool0303oool2000000H0oooo00<000000?ooo`3oool0
n03oool000h0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`3h0?ooo`003P3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0?P0oooo000>0?ooo`030000003oool0oooo
01@0oooo0P00003i0?ooo`003P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0?P0
oooo000>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0n03oool000h0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`3h0?ooo`003P3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo0?P0oooo000=0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool0n03oool000d0oooo00<000000?ooo`3oool05@3oool200000?T0oooo000=0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`3h0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0
oooo0?P0oooo000=0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3h0?ooo`003@3oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo0?P0oooo000=0?ooo`030000003oool0oooo01D0oooo0P00
003i0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?P0oooo000=0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`3h0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo0?P0oooo000=0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0n03o
ool000d0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3h0?ooo`003@3oool00`00
0000oooo0?ooo`0E0?ooo`800000n@3oool000d0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`3h0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?P0oooo
000=0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?P0oooo000=0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3h0?oo
o`003@3oool00`000000oooo0?ooo`0;0?ooo`D000001@3oool200000?T0oooo000=0?ooo`030000
003oool0oooo00/0oooo00@000000?ooo`3oool000001P3oool00`000000oooo0?ooo`3h0?ooo`00
3@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0000000H0oooo00<000000?ooo`3oool0
n03oool000d0oooo00<000000?ooo`3oool03@3oool2000000H0oooo00<000000?ooo`3oool0n03o
ool000d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0?P0oooo000=0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3h0?ooo`003@3oool00`000000oooo
0?ooo`0E0?ooo`800000n@3oool000d0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`3h0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?P0oooo000=0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`3h0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo0?P0oooo000=0?ooo`030000003oool0oooo01D0oooo0P00003i0?ooo`003@3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?P0oooo000=0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0n03oool000d0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`3h0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?P0oooo
000=0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`3h0?ooo`003@3oool00`000000oooo0?ooo`0E0?oo
o`800000n@3oool000d0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3h0?ooo`00
3@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?P0oooo000=0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`3h0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
0?P0oooo000=0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0oooo
00<000000?ooo`3oool05@3oool200000?T0oooo000=0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool0n03oool000d0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3h
0?ooo`003@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?P0oooo000=0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool0n03oool000d0oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`3h0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.57768, -0.375263, \
0.0423531, 0.0302428}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["for imaginary om", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(R[a_, b_] = 
      1/2*\((1/\((a + b)\)*Sqrt[\((a + b)\)/\((a - b)\)] + 
            1/\((a - b)\)*Sqrt[\((a - b)\)/\((a + b)\)])\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(1\/2\ \((\@\(\(a - b\)\/\(a + b\)\)\/\(a - b\) + \@\(\(a + b\)\/\(a - b\
\)\)\/\(a + b\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(R[a_, b_] = 
      1/2*\((1/Sqrt[a^2 - b^2]*\((Sign[Re[a] + b] + 
                Sign[Re[a] - b])\))\)\)], "Input"],

Cell[BoxData[
    \(\(Sign[\(-b\) + Re[a]] + Sign[b + Re[a]]\)\/\(2\ \@\(a\^2 - \
b\^2\)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(alphaspecial[md2_, k_, kz_, om_, a_] := \(md2/\((k^2 - kz^2)\)\)/2*
        NIntegrate[
          1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*\((om - kz\ xi + 
                k^2*\((\((1 - om^2/k^2)\)*\((1 - kz^2/k^2)\) - \((xi - 
                            om\ kz/k^2)\)^2)\)*
                  R[om - kz\ xi, 
                    Sqrt[k^2 - kz^2]\ Sqrt[1 - xi^2]])\), {xi, \(-1\), 1}, 
          WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphaspecial[1, 1, 0.9, I, 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\[ImaginaryI] + \
\(\(0.9`\\ xi\)\))\)\)\\ \(\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\
\((1 + \(\(1\\ xi\^2\)\))\)\^2\)) is less than WorkingPrecision \
(\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(\(\(0.19954131591152685`\)\(\[InvisibleSpace]\)\) + 
      1.0418580329704263`*^-39\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphaspecial[1, 1, 0.9, I, 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\[ImaginaryI] + \
\(\(0.9`\\ xi\)\))\)\)\\ \(\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\
\((1 + \(\(1\\ xi\^2\)\))\)\^2\)) is less than WorkingPrecision \
(\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.4358898946084082`\)."\
\)], "Message"],

Cell[BoxData[
    \(\(-2.1043880815030263`\) + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      alphaspecial[1, 1, Cos[Pi/4], I*om, 1], {om, 0.01, 0.3}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.010000012083333334`\\ \[ImaginaryI]\)\))\)\) + \
\(\(\(\(\[LeftSkeleton] 19 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.02176442755614558`\\ \[ImaginaryI]\)\))\)\) + \
\(\(\(\(\[LeftSkeleton] 19 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.034594551959218366`\\ \[ImaginaryI]\)\))\)\) + \
\(\(\(\(\[LeftSkeleton] 19 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 3.1746 0.306132 3.55533 [
[.18254 .29363 -12 -9 ]
[.18254 .29363 12 0 ]
[.34127 .29363 -9 -9 ]
[.34127 .29363 9 0 ]
[.5 .29363 -12 -9 ]
[.5 .29363 12 0 ]
[.65873 .29363 -9 -9 ]
[.65873 .29363 9 0 ]
[.81746 .29363 -12 -9 ]
[.81746 .29363 12 0 ]
[.97619 .29363 -9 -9 ]
[.97619 .29363 9 0 ]
[.01131 .03948 -36 -4.5 ]
[.01131 .03948 0 4.5 ]
[.01131 .12837 -30 -4.5 ]
[.01131 .12837 0 4.5 ]
[.01131 .21725 -36 -4.5 ]
[.01131 .21725 0 4.5 ]
[.01131 .39502 -30 -4.5 ]
[.01131 .39502 0 4.5 ]
[.01131 .4839 -24 -4.5 ]
[.01131 .4839 0 4.5 ]
[.01131 .57278 -30 -4.5 ]
[.01131 .57278 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .30613 m
.18254 .31238 L
s
[(0.05)] .18254 .29363 0 1 Mshowa
.34127 .30613 m
.34127 .31238 L
s
[(0.1)] .34127 .29363 0 1 Mshowa
.5 .30613 m
.5 .31238 L
s
[(0.15)] .5 .29363 0 1 Mshowa
.65873 .30613 m
.65873 .31238 L
s
[(0.2)] .65873 .29363 0 1 Mshowa
.81746 .30613 m
.81746 .31238 L
s
[(0.25)] .81746 .29363 0 1 Mshowa
.97619 .30613 m
.97619 .31238 L
s
[(0.3)] .97619 .29363 0 1 Mshowa
.125 Mabswid
.05556 .30613 m
.05556 .30988 L
s
.0873 .30613 m
.0873 .30988 L
s
.11905 .30613 m
.11905 .30988 L
s
.15079 .30613 m
.15079 .30988 L
s
.21429 .30613 m
.21429 .30988 L
s
.24603 .30613 m
.24603 .30988 L
s
.27778 .30613 m
.27778 .30988 L
s
.30952 .30613 m
.30952 .30988 L
s
.37302 .30613 m
.37302 .30988 L
s
.40476 .30613 m
.40476 .30988 L
s
.43651 .30613 m
.43651 .30988 L
s
.46825 .30613 m
.46825 .30988 L
s
.53175 .30613 m
.53175 .30988 L
s
.56349 .30613 m
.56349 .30988 L
s
.59524 .30613 m
.59524 .30988 L
s
.62698 .30613 m
.62698 .30988 L
s
.69048 .30613 m
.69048 .30988 L
s
.72222 .30613 m
.72222 .30988 L
s
.75397 .30613 m
.75397 .30988 L
s
.78571 .30613 m
.78571 .30988 L
s
.84921 .30613 m
.84921 .30988 L
s
.88095 .30613 m
.88095 .30988 L
s
.9127 .30613 m
.9127 .30988 L
s
.94444 .30613 m
.94444 .30988 L
s
.25 Mabswid
0 .30613 m
1 .30613 L
s
.02381 .03948 m
.03006 .03948 L
s
[(-0.075)] .01131 .03948 1 0 Mshowa
.02381 .12837 m
.03006 .12837 L
s
[(-0.05)] .01131 .12837 1 0 Mshowa
.02381 .21725 m
.03006 .21725 L
s
[(-0.025)] .01131 .21725 1 0 Mshowa
.02381 .39502 m
.03006 .39502 L
s
[(0.025)] .01131 .39502 1 0 Mshowa
.02381 .4839 m
.03006 .4839 L
s
[(0.05)] .01131 .4839 1 0 Mshowa
.02381 .57278 m
.03006 .57278 L
s
[(0.075)] .01131 .57278 1 0 Mshowa
.125 Mabswid
.02381 .05726 m
.02756 .05726 L
s
.02381 .07504 m
.02756 .07504 L
s
.02381 .09281 m
.02756 .09281 L
s
.02381 .11059 m
.02756 .11059 L
s
.02381 .14614 m
.02756 .14614 L
s
.02381 .16392 m
.02756 .16392 L
s
.02381 .1817 m
.02756 .1817 L
s
.02381 .19947 m
.02756 .19947 L
s
.02381 .23503 m
.02756 .23503 L
s
.02381 .2528 m
.02756 .2528 L
s
.02381 .27058 m
.02756 .27058 L
s
.02381 .28836 m
.02756 .28836 L
s
.02381 .32391 m
.02756 .32391 L
s
.02381 .34169 m
.02756 .34169 L
s
.02381 .35946 m
.02756 .35946 L
s
.02381 .37724 m
.02756 .37724 L
s
.02381 .41279 m
.02756 .41279 L
s
.02381 .43057 m
.02756 .43057 L
s
.02381 .44835 m
.02756 .44835 L
s
.02381 .46612 m
.02756 .46612 L
s
.02381 .50168 m
.02756 .50168 L
s
.02381 .51945 m
.02756 .51945 L
s
.02381 .53723 m
.02756 .53723 L
s
.02381 .55501 m
.02756 .55501 L
s
.02381 .02171 m
.02756 .02171 L
s
.02381 .00393 m
.02756 .00393 L
s
.02381 .59056 m
.02756 .59056 L
s
.02381 .60834 m
.02756 .60834 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.0929 .04894 L
.13363 .08501 L
.17189 .11773 L
.20867 .14819 L
.24783 .17954 L
.28553 .20872 L
.32559 .23869 L
.36419 .26659 L
.40132 .29255 L
.44083 .31925 L
.47887 .34411 L
.51544 .36723 L
.55438 .39106 L
.59186 .41323 L
.63171 .43602 L
.6701 .45724 L
.70702 .47699 L
.74631 .49731 L
.78413 .51624 L
.82433 .53568 L
.86306 .55378 L
.90033 .57064 L
.93996 .58798 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`009`3oool00`000000oooo0?ooo`3f0?ooo`009`3oool200000?L0oooo000W0?ooo`030000
003oool0oooo0?H0oooo000W0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0kP3o
ool002L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3]0?ooo`009`3oool20000
00P0oooo00<000000?ooo`3oool0k03oool002L0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`3[0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo
00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`3Z0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000<0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0j@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool20000
00`0oooo00<000000?ooo`3oool0j03oool000030?ooo`0000000000008000000P3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0oooo0`00
00050?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0i`3oool000L0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000050?ooo`040000003oool0oooo000000L0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3V0?ooo`001`3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0i@3oool0
00P0oooo0P00000:0?ooo`8000000`3oool400000080oooo100000040?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0i03oool002L0oooo0P00000A0?ooo`030000003oool0oooo0><0
oooo000W0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0hP3oool002L0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`3Q0?ooo`009`3oool00`000000oooo0?ooo`0C
0?ooo`800000h@3oool002L0oooo0P00000F0?ooo`030000003oool0oooo0=h0oooo000W0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool0g@3oool002L0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`3L0?ooo`009`3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo0=/0oooo000W0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0fP3oool0
02L0oooo0P00000K0?ooo`800000fP3oool002L0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`3G0?ooo`009`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0=H0oooo
000W0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0e@3oool002L0oooo0P00000P
0?ooo`800000e@3oool002L0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`3B0?oo
o`009`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0=40oooo000>0?ooo`800000
103oool00`000000oooo0?ooo`030?ooo`8000000`3oool3000000D0oooo00<000000?ooo`3oool0
8`3oool00`000000oooo0?ooo`3@0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`040000
003oool0oooo000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0T0?ooo`03
0000003oool0oooo0<l0oooo000=0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000001@3oool00`000000oooo0?ooo`020?ooo`8000009P3oool00`000000oooo0?ooo`3>0?oo
o`001`3oool400000080oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000002
0?ooo`<000001@3oool00`000000oooo0?ooo`0V0?ooo`800000cP3oool000d0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0:03oool00`000000oooo0?ooo`3;0?ooo`003@3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo0<X0oooo000>0?ooo`8000002P3oool2000000<0oooo1000
00040?ooo`800000:`3oool00`000000oooo0?ooo`390?ooo`009`3oool00`000000oooo0?ooo`0[
0?ooo`030000003oool0oooo0<P0oooo000W0?ooo`030000003oool0oooo02`0oooo00<000000?oo
o`3oool0a`3oool002L0oooo00<000000?ooo`3oool0;@3oool200000<L0oooo000W0?ooo`030000
003oool0oooo02l0oooo00<000000?ooo`3oool0a03oool002L0oooo0P00000a0?ooo`030000003o
ool0oooo0<<0oooo000W0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0`P3oool0
02L0oooo00<000000?ooo`3oool0<P3oool200000<80oooo000W0?ooo`030000003oool0oooo03@0
oooo00<000000?ooo`3oool0_`3oool002L0oooo0P00000f0?ooo`030000003oool0oooo0;h0oooo
000W0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0_@3oool002L0oooo00<00000
0?ooo`3oool0=`3oool200000;d0oooo000W0?ooo`030000003oool0oooo03T0oooo00<000000?oo
o`3oool0^P3oool002L0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`2i0?ooo`00
9`3oool2000003`0oooo0P00002i0?ooo`009`3oool00`000000oooo0?ooo`0m0?ooo`030000003o
ool0oooo0;H0oooo00080?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool4
00000080oooo0`0000050?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0]@3oool0
00L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0o0?ooo`800000]@3o
ool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool200000480oooo00<000000?ooo`3oool0
/P3oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`0000080?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool0oooo0`0000050?ooo`030000003oool0oooo
0480oooo00<000000?ooo`3oool0/@3oool000L0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000050?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0@`3o
ool200000;40oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0A@3oool00`000000oooo0?ooo`2^0?ooo`00203oool2000000X0oooo0P0000040?ooo`800000
0`3oool4000000@0oooo0P0000170?ooo`800000[P3oool002L0oooo00<000000?ooo`3oool0B03o
ool00`000000oooo0?ooo`2[0?ooo`009`3oool00`000000oooo0?ooo`190?ooo`800000Z`3oool0
02L0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`2X0?ooo`009`3oool00`000000
oooo0?ooo`1<0?ooo`030000003oool0oooo0:L0oooo000W0?ooo`800000CP3oool200000:L0oooo
000W0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0Y03oool002L0oooo00<00000
0?ooo`3oool0D03oool200000:@0oooo000W0?ooo`030000003oool0oooo01X0oooo0P0000040?oo
o`030000003oool0oooo00<0oooo0P0000030?ooo`<000005`3oool2000000@0oooo00<000000?oo
o`3oool00P3oool4000001H0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?oo
o`<000005`3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000001H0oooo0P0000040?oo
o`030000003oool0oooo0080oooo100000020?ooo`<000005`3oool2000000@0oooo00<000000?oo
o`3oool00`3oool2000000<0oooo000W0?ooo`8000006P3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo000000D0oooo00<000000?ooo`3oool04`3oool010000000oooo0?ooo`00
000:0?ooo`8000005@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool04`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo01H0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
01<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`009`3oool0
0`000000oooo0?ooo`0I0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
1@3oool00`000000oooo0?ooo`0C0?ooo`040000003oool0oooo000000X0oooo00@000000?ooo`00
000000004`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool04`3oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo01D0oooo00@0
00000?ooo`3oool000002@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01<0oooo
00@000000?ooo`3oool000002`3oool100000040oooo0@3oool002L0oooo00<000000?ooo`3oool0
6@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`00000F0?oo
o`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo0000000B0?ooo`040000003o
ool0oooo000000X0oooo00@000000?ooo`3oool0oooo0`00000F0?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`00000:0?ooo`040000003oool0
oooo0?ooo`<000005P3oool010000000oooo0?ooo`0000090?ooo`8000000`3oool002L0oooo00<0
00000?ooo`3oool06@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool03`3oool010000000oooo0?ooo`00000:0?ooo`050000003oool0
oooo0?ooo`000000603oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo01<0oooo
00@000000?ooo`3oool000002`3oool010000000oooo0?ooo`00000H0?ooo`040000003oool0oooo
000000/0oooo0@0000010?ooo`40oooo000W0?ooo`030000003oool0oooo01T0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo01H0oooo00@0
00000?ooo`3oool00000203oool3000000D0oooo0P00000?0?ooo`040000003oool0oooo000000P0
oooo0`0000030?ooo`030000003oool0oooo01H0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`00000E0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo0?ooo`0F0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000000P3oool002L0oooo0P00000K0?ooo`8000002P3oool2000000<0oooo1000000F0?ooo`80
00002`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00d0oooo0P00000;0?ooo`04
0000003oool0oooo0?ooo`@000005P3oool2000000X0oooo0P00000G0?ooo`8000002P3oool20000
00<0oooo1000000F0?ooo`8000002P3oool2000000<0oooo000W0?ooo`030000003oool0oooo05/0
oooo00<000000?ooo`3oool0V03oool002L0oooo00<000000?ooo`3oool0G03oool2000009P0oooo
000W0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0U@3oool00240oooon`000004
0?ooo`009`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01@3oool010000000oooo0?ooo`3oool200000080oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00L0oooo000W0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo
02D0oooo00<000000?ooo`3oool01`3oool002L0oooo00<000000?ooo`3oool0H`3oool200000940
oooo000W0?ooo`030000003oool0oooo06D0oooo00<000000?ooo`3oool0SP3oool002L0oooo0P00
001W0?ooo`800000SP3oool002L0oooo00<000000?ooo`3oool0J03oool2000008`0oooo000W0?oo
o`030000003oool0oooo06X0oooo00<000000?ooo`3oool0R@3oool002L0oooo00<000000?ooo`3o
ool0J`3oool2000008T0oooo000W0?ooo`800000KP3oool2000008L0oooo000W0?ooo`030000003o
ool0oooo06l0oooo00<000000?ooo`3oool0Q03oool002L0oooo00<000000?ooo`3oool0L03oool2
000008@0oooo000W0?ooo`030000003oool0oooo0780oooo00<000000?ooo`3oool0P@3oool002L0
oooo00<000000?ooo`3oool0L`3oool200000840oooo000W0?ooo`800000MP3oool00`000000oooo
0?ooo`1n0?ooo`009`3oool00`000000oooo0?ooo`1f0?ooo`800000OP3oool002L0oooo00<00000
0?ooo`3oool0N03oool00`000000oooo0?ooo`1k0?ooo`009`3oool00`000000oooo0?ooo`1i0?oo
o`800000N`3oool002L0oooo0P00001l0?ooo`800000N@3oool002L0oooo00<000000?ooo`3oool0
O@3oool00`000000oooo0?ooo`1f0?ooo`009`3oool00`000000oooo0?ooo`1n0?ooo`800000MP3o
ool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`@000000P3oool3
000000D0oooo00<000000?ooo`3oool0P03oool2000007@0oooo00070?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0PP3oool00`000000oooo0?ooo`1a0?ooo`001`3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000<0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`020?ooo`800000Q03oool200000740oooo00070?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`3oool30000
00D0oooo00<000000?ooo`3oool0Q@3oool2000006l0oooo00070?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000001@3oool010000000oooo0?ooo`0000070?ooo`030000003o
ool0oooo08L0oooo0P00001]0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo08T0oooo0P00001[0?ooo`00203oool2000000X0oooo0P0000040?ooo`800000
0`3oool4000000@0oooo0P00002<0?ooo`030000003oool0oooo06P0oooo000W0?ooo`030000003o
ool0oooo08`0oooo0P00001X0?ooo`009`3oool00`000000oooo0?ooo`2>0?ooo`800000IP3oool0
02L0oooo00<000000?ooo`3oool0T03oool00`000000oooo0?ooo`1S0?ooo`009`3oool00`000000
oooo0?ooo`2A0?ooo`800000H`3oool002L0oooo0P00002D0?ooo`800000H@3oool002L0oooo00<0
00000?ooo`3oool0U@3oool00`000000oooo0?ooo`1N0?ooo`009`3oool00`000000oooo0?ooo`2F
0?ooo`800000GP3oool002L0oooo00<000000?ooo`3oool0V03oool2000005`0oooo000W0?ooo`80
0000V`3oool2000005X0oooo000W0?ooo`030000003oool0oooo09`0oooo0P00001H0?ooo`009`3o
ool00`000000oooo0?ooo`2N0?ooo`800000EP3oool002L0oooo00<000000?ooo`3oool0X03oool2
000005@0oooo000W0?ooo`800000X`3oool200000580oooo000W0?ooo`030000003oool0oooo0:@0
oooo0P00001@0?ooo`009`3oool00`000000oooo0?ooo`2V0?ooo`800000CP3oool000h0oooo0P00
00040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`<000001@3oool00`000000oooo0?oo
o`2X0?ooo`030000003oool0oooo04/0oooo000=0?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0:T0oooo
0P00001;0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool00P3oool200000:`0oooo0P0000190?ooo`003@3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`0000050?ooo`030000003oool0oooo
0:d0oooo0P0000170?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2_0?ooo`800000A@3oool0
00d0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0/@3oool2000004<0oooo000>0?ooo`8000002P3oool2
000000<0oooo100000040?ooo`800000]03oool200000440oooo000W0?ooo`030000003oool0oooo
0;D0oooo0P00000o0?ooo`009`3oool00`000000oooo0?ooo`2g0?ooo`800000?@3oool002L0oooo
00<000000?ooo`3oool0^@3oool2000003/0oooo000W0?ooo`030000003oool0oooo0;/0oooo00<0
00000?ooo`3oool0>03oool002L0oooo0P00002m0?ooo`800000>03oool002L0oooo00<000000?oo
o`3oool0_P3oool2000003H0oooo000W0?ooo`030000003oool0oooo0<00oooo0P00000d0?ooo`00
9`3oool00`000000oooo0?ooo`320?ooo`800000<P3oool002L0oooo0P0000350?ooo`<00000;`3o
ool002L0oooo00<000000?ooo`3oool0a`3oool2000002d0oooo000W0?ooo`030000003oool0oooo
0<T0oooo0`00000Z0?ooo`009`3oool00`000000oooo0?ooo`3<0?ooo`800000:03oool002L0oooo
00<000000?ooo`3oool0cP3oool2000002H0oooo000W0?ooo`800000d@3oool2000002@0oooo000W
0?ooo`030000003oool0oooo0=80oooo0P00000R0?ooo`00203oool2000000@0oooo00<000000?oo
o`3oool00`3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?oo
o`3oool0e03oool200000200oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool000000`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0eP3oool2000001h0oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo0P00
003I0?ooo`800000703oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`3J0?ooo`80
00006P3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?oo
o`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0g03oool3000001L0oooo00070?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?oo
o`0000020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0g`3oool2000001D0oooo
00080?ooo`8000002P3oool2000000<0oooo100000020?ooo`@00000103oool00`000000oooo0?oo
o`3Q0?ooo`<000004P3oool002L0oooo0P00003U0?ooo`800000403oool002L0oooo00<000000?oo
o`3oool0iP3oool3000000d0oooo000W0?ooo`030000003oool0oooo0>T0oooo0P00000;0?ooo`00
9`3oool00`000000oooo0?ooo`3[0?ooo`8000002@3oool002L0oooo0P00003g0?ooo`009`3oool0
0`000000oooo0?ooo`3f0?ooo`009`3oool00`000000oooo0?ooo`3f0?ooo`009`3oool00`000000
oooo0?ooo`3f0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?oo
o`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?oo
o`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0491532, -0.0984029, \
0.00125595, 0.00112146}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["small a", "Subsection"],

Cell[BoxData[
    \(\(Intexp = 
        Series[1/\((1 + a\ \((vn^2)\))\)^2\ \((\(-om\) - \ 
                  k\ nz\ vn\ a)\)/\((om - 
                  k\ xi)\)\ \((\(-k^2\) \((1 - nz^2)\) + k^2\ xi^2 - 
                2\ k\ nz\ k\ xi\ vn + k^2\ \((vn)\)^2)\), {a, 0, 
            toorder}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(a0 = 
      Simplify[md2/k^2*\(1/2\)/\((1 - nz^2)\)*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexp[\([3]\)]\)[\([1]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(md2\ om\ \((2\ k\ om + \((\(-k\^2\) + om\^2)\)\ Log[\(-k\) + om] + \
\((k\^2 - om\^2)\)\ Log[k + om])\)\)\/\(4\ k\^3\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      1/k^2*\(1/2\)/\((1 - nz^2)\)*\(1/2\)/Pi*
        Integrate[\(Intexp[\([3]\)]\)[\([1]\)], {phi, 0, 2*Pi}]]\)], "Input"],

Cell[BoxData[
    \(\(om\ \((\(-1\) + xi\^2)\)\)\/\(\(-4\)\ om + 4\ k\ xi\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apart[%]\)], "Input"],

Cell[BoxData[
    \(om\^2\/\(4\ k\^2\) + \(om\ xi\)\/\(4\ k\) + \(\(-k\^2\)\ om + om\^3\)\/\
\(4\ k\^2\ \((\(-om\) + k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      1/k^2*\(1/2\)/\((1 - nz^2)\)*\(1/2\)/Pi*
        Integrate[\(Intexp[\([3]\)]\)[\([2]\)], {phi, 0, 2*Pi}]]\)], "Input"],

Cell[BoxData[
    \(\(\((\(-1\) + xi\^2)\)\ \((\(-2\)\ k\ nz\^2\ xi + om\ \((1 - xi\^2 + nz\
\^2\ \((\(-1\) + 5\ xi\^2)\))\))\)\)\/\(8\ \((om - k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apart[%]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(2\ k\^4\ nz\^2 + 2\ k\^2\ om\^2 - 8\ k\^2\ nz\^2\ om\^2 - 
              om\^4 + 5\ nz\^2\ om\^4\)\/\(8\ k\^4\)\)\) + \(om\ \((\(-2\)\ k\
\^2 + 8\ k\^2\ nz\^2 + om\^2 - 5\ nz\^2\ om\^2)\)\ xi\)\/\(8\ k\^3\) + \(\((2\
\ k\^2\ nz\^2 + om\^2 - 5\ nz\^2\ om\^2)\)\ xi\^2\)\/\(8\ k\^2\) - \
\(\((\(-1\) + 5\ nz\^2)\)\ om\ xi\^3\)\/\(8\ k\) + \(k\^4\ om - 3\ k\^4\ \
nz\^2\ om - 2\ k\^2\ om\^3 + 8\ k\^2\ nz\^2\ om\^3 + om\^5 - 5\ nz\^2\ \
om\^5\)\/\(8\ k\^4\ \((\(-om\) + k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[\((k\^4\ om - 3\ k\^4\ nz\^2\ om - 2\ k\^2\ om\^3 + 
            8\ k\^2\ nz\^2\ om\^3 + om\^5 - 5\ nz\^2\ om\^5)\)/om]\)], "Input"],

Cell[BoxData[
    \(\(-\((k\^2 - 
            om\^2)\)\)\ \((k\^2\ \((\(-1\) + 3\ nz\^2)\) + \((1 - 
                5\ nz\^2)\)\ om\^2)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      2\ k\^2\ om\^2 - 8\ k\^2\ nz\^2\ om\^2 - om\^4 + 
        5\ nz\^2\ om\^4]\)], "Input"],

Cell[BoxData[
    \(om\^2\ \((k\^2\ \((2 - 8\ nz\^2)\) + \((\(-1\) + 
                5\ nz\^2)\)\ om\^2)\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Fast alpha", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imarg = 
      Compile[{md2, k, kz, om, a, xi}, 
        1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
          Sqrt[\(-\((k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - om^2)\)*\((1 - 
                        kz^2/k^2)\))\)\)]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(\((om + a\ kz\ xi)\)\ \@\(-\((k\^2\ \((xi - \(om\ \
kz\)\/k\^2)\)\^2 - \((k\^2 - om\^2)\)\ \((1 - kz\^2\/k\^2)\))\)\)\)\/\((1 + a\
\ xi\^2)\)\^2, "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(FastImalpha[md2_, k_, kz_, om_, a_] := 
      If[om > k, 
        0, \(-md2\)/\((k^2 - kz^2)\)*k/2*
          NIntegrate[
            Imarg[md2, k, kz, om, a, xi], {xi, 
              xi2[k, kz, om]*
                If[xi2[k, kz, om] > 
                    0, \((1 + 10^\((\(-15\))\))\), \((1 - 
                      10^\((\(-15\))\))\)], xi1[k, kz, om]}]]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Plot[FastImalpha[1, 1.0, 0.8, om, 1], {om, 0, 1}]]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.603319 1.82894 [
[.21429 .59082 -9 -9 ]
[.21429 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.59524 .59082 -9 -9 ]
[.59524 .59082 9 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.01131 .05464 -24 -4.5 ]
[.01131 .05464 0 4.5 ]
[.01131 .14608 -30 -4.5 ]
[.01131 .14608 0 4.5 ]
[.01131 .23753 -24 -4.5 ]
[.01131 .23753 0 4.5 ]
[.01131 .32898 -30 -4.5 ]
[.01131 .32898 0 4.5 ]
[.01131 .42042 -24 -4.5 ]
[.01131 .42042 0 4.5 ]
[.01131 .51187 -30 -4.5 ]
[.01131 .51187 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60332 m
.21429 .60957 L
s
[(0.2)] .21429 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(0.4)] .40476 .59082 0 1 Mshowa
.59524 .60332 m
.59524 .60957 L
s
[(0.6)] .59524 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(0.8)] .78571 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(1)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.2619 .60332 m
.2619 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.7381 .60332 m
.7381 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .05464 m
.03006 .05464 L
s
[(-0.3)] .01131 .05464 1 0 Mshowa
.02381 .14608 m
.03006 .14608 L
s
[(-0.25)] .01131 .14608 1 0 Mshowa
.02381 .23753 m
.03006 .23753 L
s
[(-0.2)] .01131 .23753 1 0 Mshowa
.02381 .32898 m
.03006 .32898 L
s
[(-0.15)] .01131 .32898 1 0 Mshowa
.02381 .42042 m
.03006 .42042 L
s
[(-0.1)] .01131 .42042 1 0 Mshowa
.02381 .51187 m
.03006 .51187 L
s
[(-0.05)] .01131 .51187 1 0 Mshowa
.125 Mabswid
.02381 .07293 m
.02756 .07293 L
s
.02381 .09122 m
.02756 .09122 L
s
.02381 .1095 m
.02756 .1095 L
s
.02381 .12779 m
.02756 .12779 L
s
.02381 .16437 m
.02756 .16437 L
s
.02381 .18266 m
.02756 .18266 L
s
.02381 .20095 m
.02756 .20095 L
s
.02381 .21924 m
.02756 .21924 L
s
.02381 .25582 m
.02756 .25582 L
s
.02381 .27411 m
.02756 .27411 L
s
.02381 .2924 m
.02756 .2924 L
s
.02381 .31069 m
.02756 .31069 L
s
.02381 .34727 m
.02756 .34727 L
s
.02381 .36556 m
.02756 .36556 L
s
.02381 .38385 m
.02756 .38385 L
s
.02381 .40214 m
.02756 .40214 L
s
.02381 .43871 m
.02756 .43871 L
s
.02381 .457 m
.02756 .457 L
s
.02381 .47529 m
.02756 .47529 L
s
.02381 .49358 m
.02756 .49358 L
s
.02381 .53016 m
.02756 .53016 L
s
.02381 .54845 m
.02756 .54845 L
s
.02381 .56674 m
.02756 .56674 L
s
.02381 .58503 m
.02756 .58503 L
s
.02381 .03635 m
.02756 .03635 L
s
.02381 .01806 m
.02756 .01806 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .53002 L
.10458 .45135 L
.14415 .37989 L
.18221 .31446 L
.22272 .24948 L
.26171 .19254 L
.30316 .13914 L
.34309 .09561 L
.3815 .0619 L
.40095 .04811 L
.42237 .03558 L
.44268 .02634 L
.46172 .0201 L
.4721 .01769 L
.47721 .01676 L
.48196 .01605 L
.48625 .01554 L
.49082 .01513 L
.49343 .01496 L
.49471 .01489 L
.49589 .01484 L
.49694 .0148 L
.4981 .01477 L
.49926 .01474 L
.50049 .01472 L
.50179 .01472 L
.503 .01472 L
.50363 .01472 L
.50432 .01473 L
.50505 .01475 L
.50573 .01476 L
.50698 .0148 L
.50812 .01484 L
.51069 .01497 L
.51286 .01511 L
.51523 .0153 L
.52 .01579 L
.52524 .01652 L
.53083 .01749 L
.54091 .01977 L
.55116 .02278 L
.56226 .02682 L
.58241 .03624 L
.62333 .06342 L
.66273 .09933 L
.70062 .14217 L
.74096 .1958 L
.77978 .25409 L
.82106 .32187 L
Mistroke
.86082 .39139 L
.89907 .4608 L
.93976 .53599 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3o
ool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`00
0000oooo0?ooo`3l0?ooo`008@3oool200000740oooo4P00001j0?ooo`008@3oool00`000000oooo
0?ooo`1^0?ooo`8000004P3oool3000007L0oooo000Q0?ooo`030000003oool0oooo06`0oooo0P00
000G0?ooo`<00000M03oool00240oooo00<000000?ooo`3oool0J@3oool3000001`0oooo0`00001a
0?ooo`008@3oool00`000000oooo0?ooo`1V0?ooo`<000008P3oool2000006l0oooo000Q0?ooo`80
0000I@3oool2000002L0oooo0P00001]0?ooo`008@3oool00`000000oooo0?ooo`1R0?ooo`800000
:`3oool2000006/0oooo000Q0?ooo`030000003oool0oooo0600oooo0P00000_0?ooo`030000003o
ool0oooo06P0oooo000>0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000001@3oool0
0`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0300oooo0P00001X0?ooo`003@3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0GP3o
ool00`000000oooo0?ooo`0c0?ooo`800000IP3oool000d0oooo00@000000?ooo`3oool000002`3o
ool00`000000oooo0?ooo`020?ooo`800000G@3oool2000003P0oooo00<000000?ooo`3oool0H`3o
ool000L0oooo100000020?ooo`040000003oool0oooo000000T0oooo0P0000050?ooo`030000003o
ool0oooo05/0oooo00<000000?ooo`3oool0>@3oool2000006<0oooo000=0?ooo`040000003oool0
oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1J0?ooo`030000003o
ool0oooo03`0oooo00<000000?ooo`3oool0H03oool000d0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo05T0oooo00<000000?ooo`3oool0
?P3oool00`000000oooo0?ooo`1O0?ooo`003P3oool2000000X0oooo0P0000050?ooo`030000003o
ool0oooo05P0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`1N0?ooo`008@3oool2
000005P0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1M0?ooo`008@3oool00`00
0000oooo0?ooo`1E0?ooo`800000AP3oool00`000000oooo0?ooo`1L0?ooo`008@3oool00`000000
oooo0?ooo`1D0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0F`3oool00240oooo
00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo05X0oooo
000Q0?ooo`800000D`3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo05T0oooo000Q
0?ooo`030000003oool0oooo0540oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`1H
0?ooo`008@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo04l0oooo00<000000?oo
o`3oool0E`3oool00240oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`1A0?ooo`03
0000003oool0oooo05H0oooo000Q0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0
D`3oool00`000000oooo0?ooo`1E0?ooo`008@3oool2000004h0oooo00<000000?ooo`3oool0E@3o
ool00`000000oooo0?ooo`1D0?ooo`008@3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0
oooo05L0oooo00<000000?ooo`3oool0D`3oool00240oooo00<000000?ooo`3oool0B`3oool00`00
0000oooo0?ooo`1I0?ooo`030000003oool0oooo0580oooo000Q0?ooo`030000003oool0oooo04X0
oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`1A0?ooo`008@3oool00`000000oooo
0?ooo`190?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool0D@3oool00240oooo0P00
00190?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0D03oool00240oooo00<00000
0?ooo`3oool0A`3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0oooo04l0oooo000Q0?oo
o`030000003oool0oooo04H0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`1>0?oo
o`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`0000050?ooo`03
0000003oool0oooo04D0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`1=0?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3o
ool0C03oool000L0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo0080oooo0P0000150?ooo`030000003oool0oooo06L0oooo00<000000?ooo`3o
ool0B`3oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`00000:0?ooo`04
0000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo
06P0oooo00<000000?ooo`3oool0B`3oool000L0oooo00@000000?ooo`3oool000002`3oool01000
0000oooo0?ooo`0000070?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0JP3oool0
0`000000oooo0?ooo`1:0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`110?ooo`030000003o
ool0oooo06`0oooo00<000000?ooo`3oool0B@3oool000P0oooo0P00000:0?ooo`8000000`3oool4
000000@0oooo0P0000120?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0B03oool0
0240oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`1^0?ooo`030000003oool0oooo
04P0oooo000Q0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0L03oool00`000000
oooo0?ooo`170?ooo`008@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0740oooo
00<000000?ooo`3oool0AP3oool00240oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?oo
o`1b0?ooo`030000003oool0oooo04H0oooo000Q0?ooo`800000?P3oool00`000000oooo0?ooo`1d
0?ooo`030000003oool0oooo04D0oooo000Q0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool0MP3oool00`000000oooo0?ooo`140?ooo`008@3oool00`000000oooo0?ooo`0l0?ooo`03
0000003oool0oooo07L0oooo00<000000?ooo`3oool0@`3oool00240oooo00<000000?ooo`3oool0
>`3oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo04<0oooo000Q0?ooo`030000003o
ool0oooo03X0oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`120?ooo`008@3oool2
000003/0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`110?ooo`008@3oool00`00
0000oooo0?ooo`0i0?ooo`030000003oool0oooo07`0oooo00<000000?ooo`3oool0@@3oool00240
oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`1n0?ooo`030000003oool0oooo0400
oooo000Q0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0P03oool00`000000oooo
0?ooo`0o0?ooo`008@3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0800oooo00<0
00000?ooo`3oool0?`3oool00240oooo0P00000g0?ooo`030000003oool0oooo0880oooo00<00000
0?ooo`3oool0?P3oool00240oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`240?oo
o`030000003oool0oooo03d0oooo000>0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@0
0000103oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo08D0oooo00<000000?ooo`3o
ool0?@3oool000d0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool0QP3oool00`000000oooo0?ooo`0l0?ooo`00
3@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo00@0oooo0P00000d0?ooo`03
0000003oool0oooo08P0oooo00<000000?ooo`3oool0>`3oool000L0oooo100000020?ooo`040000
003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0b0?ooo`03
0000003oool0oooo08T0oooo00<000000?ooo`3oool0>`3oool000d0oooo00@000000?ooo`3oool0
00002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0340oooo00<000000?ooo`3o
ool0R`3oool00`000000oooo0?ooo`0j0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`2<0?oo
o`030000003oool0oooo03T0oooo000>0?ooo`8000002P3oool2000000D0oooo00<000000?ooo`3o
ool0<03oool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo03T0oooo000Q0?ooo`800000
<03oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo03P0oooo000Q0?ooo`030000003o
ool0oooo02l0oooo00<000000?ooo`3oool0T03oool00`000000oooo0?ooo`0g0?ooo`008@3oool0
0`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0=`3oool0
0240oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`2B0?ooo`030000003oool0oooo
03H0oooo000Q0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0U03oool00`000000
oooo0?ooo`0e0?ooo`008@3oool2000002d0oooo00<000000?ooo`3oool0U@3oool00`000000oooo
0?ooo`0e0?ooo`008@3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo09H0oooo00<0
00000?ooo`3oool0=03oool00240oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`2H
0?ooo`030000003oool0oooo03<0oooo000Q0?ooo`030000003oool0oooo02X0oooo00<000000?oo
o`3oool0V@3oool00`000000oooo0?ooo`0c0?ooo`008@3oool00`000000oooo0?ooo`0Z0?ooo`03
0000003oool0oooo09X0oooo00<000000?ooo`3oool0<P3oool00240oooo0P00000Z0?ooo`030000
003oool0oooo09/0oooo00<000000?ooo`3oool0<P3oool00240oooo00<000000?ooo`3oool0:@3o
ool00`000000oooo0?ooo`2L0?ooo`030000003oool0oooo0340oooo000Q0?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool0WP3oool00`000000oooo0?ooo`0`0?ooo`008@3oool00`00
0000oooo0?ooo`0W0?ooo`030000003oool0oooo09l0oooo00<000000?ooo`3oool0<03oool00240
oooo0P00000X0?ooo`030000003oool0oooo0:00oooo00<000000?ooo`3oool0;`3oool00240oooo
00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`2R0?ooo`030000003oool0oooo02h0oooo
000Q0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0X`3oool00`000000oooo0?oo
o`0^0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`000005
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0Y03oool00`000000oooo0?ooo`0]
0?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0:H0oooo00<00000
0?ooo`3oool0;03oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0080oooo0P00000U0?ooo`030000003oool0oooo0:H0oooo00<00000
0?ooo`3oool0;03oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`00000:
0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`0S0?ooo`030000003o
ool0oooo0:P0oooo00<000000?ooo`3oool0:`3oool000L0oooo00@000000?ooo`3oool000002P3o
ool01@000000oooo0?ooo`3oool0000000L0oooo00<000000?ooo`3oool08P3oool00`000000oooo
0?ooo`2Y0?ooo`030000003oool0oooo02/0oooo00070?ooo`040000003oool0oooo000000P0oooo
0`0000030?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool08P3oool00`000000oooo
0?ooo`2Z0?ooo`030000003oool0oooo02X0oooo00080?ooo`8000002`3oool010000000oooo0?oo
o`3oool4000000@0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`2[0?ooo`030000
003oool0oooo02X0oooo000Q0?ooo`8000008P3oool00`000000oooo0?ooo`2/0?ooo`030000003o
ool0oooo02T0oooo000Q0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0[@3oool0
0`000000oooo0?ooo`0Y0?ooo`008@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo
0:h0oooo00<000000?ooo`3oool0:03oool00240oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`2`0?ooo`030000003oool0oooo02L0oooo000Q0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0W0?ooo`008@3oool2000001l0oooo00<0
00000?ooo`3oool0/P3oool00`000000oooo0?ooo`0V0?ooo`008@3oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool09P3oool00240oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`2d0?ooo`030000003oool0oooo02D0oooo000Q0?ooo`03
0000003oool0oooo01`0oooo00<000000?ooo`3oool0]@3oool00`000000oooo0?ooo`0U0?ooo`00
8@3oool2000001`0oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?ooo`0T0?ooo`008@3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0;L0oooo00<000000?ooo`3oool0903o
ool00240oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`2i0?ooo`030000003oool0
oooo02<0oooo000Q0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0^P3oool00`00
0000oooo0?ooo`0R0?ooo`008@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0;/0
oooo00<000000?ooo`3oool08P3oool00240oooo0P00000J0?ooo`030000003oool0oooo0;`0oooo
00<000000?ooo`3oool08@3oool00240oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`2m0?ooo`030000003oool0oooo0240oooo000Q0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0_P3oool00`000000oooo0?ooo`0P0?ooo`003P3oool2000000@0oooo00<000000?oo
o`3oool00P3oool4000000@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`2o0?oo
o`030000003oool0oooo0200oooo000=0?ooo`040000003oool0oooo000000X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0<00oooo00<000000?oo
o`3oool07`3oool000d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?oo
o`8000005`3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo01l0oooo00070?ooo`@0
00000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`320?ooo`030000003oool0oooo01h0oooo000=0?ooo`04
0000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0E0?oo
o`030000003oool0oooo0<@0oooo00<000000?ooo`3oool07@3oool000d0oooo00@000000?ooo`3o
ool00000203oool3000000D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`340?oo
o`030000003oool0oooo01d0oooo000>0?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool0aP3oool00`000000oooo0?ooo`0L0?ooo`00
8@3oool2000001D0oooo00<000000?ooo`3oool0aP3oool00`000000oooo0?ooo`0L0?ooo`008@3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0<P0oooo00<000000?ooo`3oool06`3o
ool00240oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`380?ooo`030000003oool0
oooo01/0oooo000Q0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0bP3oool00`00
0000oooo0?ooo`0J0?ooo`008@3oool200000180oooo00<000000?ooo`3oool0b`3oool00`000000
oooo0?ooo`0J0?ooo`008@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0<`0oooo
00<000000?ooo`3oool06@3oool00240oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`3>0?ooo`030000003oool0oooo01P0oooo000Q0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0cP3oool00`000000oooo0?ooo`0H0?ooo`008@3oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo0=00oooo00<000000?ooo`3oool05`3oool00240oooo0P00000@0?ooo`03
0000003oool0oooo0=00oooo00<000000?ooo`3oool05`3oool00240oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`3B0?ooo`030000003oool0oooo01H0oooo000Q0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`0F0?ooo`008@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0=@0oooo00<000000?ooo`3oool05@3oool0
0240oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo
01D0oooo000Q0?ooo`8000003@3oool00`000000oooo0?ooo`3F0?ooo`030000003oool0oooo01@0
oooo000Q0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0e`3oool00`000000oooo
0?ooo`0D0?ooo`008@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0=P0oooo00<0
00000?ooo`3oool04`3oool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000003
0?ooo`<000001@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0=X0oooo00<00000
0?ooo`3oool04P3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`3J0?ooo`030000003oool0oooo0180oooo00070?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`8000002P3oool00`000000oooo
0?ooo`3L0?ooo`030000003oool0oooo0140oooo00000`3oool000000000000200000080oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`<000001@3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0=`0oooo00<000000?ooo`3oool04@3oool000L0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3N0?ooo`030000003oool0oooo
0100oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0gP3oool0
0`000000oooo0?ooo`0@0?ooo`00203oool2000000X0oooo0P0000030?ooo`@00000103oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03`3oool00240
oooo0P0000080?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03`3oool00240oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3R0?ooo`030000003oool0oooo00h0oooo
000Q0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0hP3oool00`000000oooo0?oo
o`0>0?ooo`008@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0>@0oooo00<00000
0?ooo`3oool03@3oool00240oooo0P0000060?ooo`030000003oool0oooo0>@0oooo00<000000?oo
o`3oool03@3oool00240oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3V0?ooo`03
0000003oool0oooo00`0oooo000Q0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
iP3oool00`000000oooo0?ooo`0<0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0>P0oooo00<000000?ooo`3oool02`3oool00240oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0Q0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000008`3oool2
000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0Q0?ooo`800000103oool0
0`000000oooo0?ooo`030?ooo`800000903oool2000000@0oooo00<000000?ooo`3oool00`3oool2
000002@0oooo00@000000?ooo`3oool0oooo100000060?ooo`008@3oool2000000<0oooo00<00000
0?ooo`3oool08@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo02<0oooo00@0
00000?ooo`3oool000002P3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo000002@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`050?ooo`008@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool000002@3o
ool00`000000oooo0?ooo`0R0?ooo`040000003oool0oooo000000L0oooo1@00000Q0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo000002@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`050?ooo`008@3oool01@000000oooo0?ooo`3oool0000002@0oooo00@000000?ooo`3oool00000
2P3oool00`000000oooo0?ooo`0Q0?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3o
ool000008P3oool010000000oooo0?ooo`0000080?ooo`<000008`3oool010000000oooo0?ooo`00
00090?ooo`8000009P3oool01@000000oooo0?ooo`3oool0000000L0oooo000Q0?ooo`050000003o
ool0oooo0?ooo`000000903oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0200
oooo00@000000?ooo`3oool00000203oool00`000000oooo0000000R0?ooo`040000003oool0oooo
000000P0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000002D0oooo00D000000?ooo`3oool0oooo000000070?ooo`008@3oool010000000oooo
0?ooo`00000U0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000008P3oool0
10000000oooo0?ooo`0000090?ooo`8000008P3oool010000000oooo0?ooo`0000080?ooo`030000
003oool0oooo02<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000V0?oo
o`@000001`3oool00240oooo0P0000000`3oool000000?ooo`0U0?ooo`8000002P3oool2000002@0
oooo0P00000;0?ooo`030000003oool0oooo0240oooo0P00000:0?ooo`<000008`3oool2000000X0
oooo0P00000W0?ooo`040000003oool0oooo000000L0oooo000Q0?ooo`030000003oool000000?80
oooo00<000000?ooo`3oool01`3oool00240oooo00<000000?ooo`000000lP3oool00`000000oooo
0?ooo`070?ooo`008@3oool200000?@0oooo00<000000?ooo`3oool01P3oool00240oooo0P00003d
0?ooo`030000003oool0oooo00H0oooo000K0?ooool000001P0000000240oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00D0oooo000Q0?ooo`030000
003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.134313, -0.347005, \
0.00403944, 0.00210345}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{\(0.71`\ Second\), ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Plot[Imalpha[1, 1.0, 0.8, om, 1], {om, 0, 1}]]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((4.166666666666666`*^-8 + \(\(0.8`\\ xi\)\))\)\)\\ \@\(-\(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\)\/\((1 + \(\(1\\ xi\^2\)\))\)\^2\
\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((\(\(0.04056699157291579` \[InvisibleSpace]\)\) + \(\(0.8`\\ \
xi\)\))\)\)\\ \@\(-\(\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\)\/\((1 \
+ \(\(1\\ xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], \
"Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((\(\(0.08480879985937367` \[InvisibleSpace]\)\) + \(\(0.8`\\ \
xi\)\))\)\)\\ \@\(-\(\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\)\/\((1 \
+ \(\(1\\ xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], \
"Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.603319 1.82894 [
[.21429 .59082 -9 -9 ]
[.21429 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.59524 .59082 -9 -9 ]
[.59524 .59082 9 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.01131 .05464 -24 -4.5 ]
[.01131 .05464 0 4.5 ]
[.01131 .14608 -30 -4.5 ]
[.01131 .14608 0 4.5 ]
[.01131 .23753 -24 -4.5 ]
[.01131 .23753 0 4.5 ]
[.01131 .32898 -30 -4.5 ]
[.01131 .32898 0 4.5 ]
[.01131 .42042 -24 -4.5 ]
[.01131 .42042 0 4.5 ]
[.01131 .51187 -30 -4.5 ]
[.01131 .51187 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60332 m
.21429 .60957 L
s
[(0.2)] .21429 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(0.4)] .40476 .59082 0 1 Mshowa
.59524 .60332 m
.59524 .60957 L
s
[(0.6)] .59524 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(0.8)] .78571 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(1)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.2619 .60332 m
.2619 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.7381 .60332 m
.7381 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .05464 m
.03006 .05464 L
s
[(-0.3)] .01131 .05464 1 0 Mshowa
.02381 .14608 m
.03006 .14608 L
s
[(-0.25)] .01131 .14608 1 0 Mshowa
.02381 .23753 m
.03006 .23753 L
s
[(-0.2)] .01131 .23753 1 0 Mshowa
.02381 .32898 m
.03006 .32898 L
s
[(-0.15)] .01131 .32898 1 0 Mshowa
.02381 .42042 m
.03006 .42042 L
s
[(-0.1)] .01131 .42042 1 0 Mshowa
.02381 .51187 m
.03006 .51187 L
s
[(-0.05)] .01131 .51187 1 0 Mshowa
.125 Mabswid
.02381 .07293 m
.02756 .07293 L
s
.02381 .09122 m
.02756 .09122 L
s
.02381 .1095 m
.02756 .1095 L
s
.02381 .12779 m
.02756 .12779 L
s
.02381 .16437 m
.02756 .16437 L
s
.02381 .18266 m
.02756 .18266 L
s
.02381 .20095 m
.02756 .20095 L
s
.02381 .21924 m
.02756 .21924 L
s
.02381 .25582 m
.02756 .25582 L
s
.02381 .27411 m
.02756 .27411 L
s
.02381 .2924 m
.02756 .2924 L
s
.02381 .31069 m
.02756 .31069 L
s
.02381 .34727 m
.02756 .34727 L
s
.02381 .36556 m
.02756 .36556 L
s
.02381 .38385 m
.02756 .38385 L
s
.02381 .40214 m
.02756 .40214 L
s
.02381 .43871 m
.02756 .43871 L
s
.02381 .457 m
.02756 .457 L
s
.02381 .47529 m
.02756 .47529 L
s
.02381 .49358 m
.02756 .49358 L
s
.02381 .53016 m
.02756 .53016 L
s
.02381 .54845 m
.02756 .54845 L
s
.02381 .56674 m
.02756 .56674 L
s
.02381 .58503 m
.02756 .58503 L
s
.02381 .03635 m
.02756 .03635 L
s
.02381 .01806 m
.02756 .01806 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .53002 L
.10458 .45135 L
.14415 .37989 L
.18221 .31446 L
.22272 .24948 L
.26171 .19254 L
.30316 .13914 L
.34309 .09561 L
.3815 .0619 L
.40095 .04811 L
.42237 .03558 L
.44268 .02634 L
.46172 .0201 L
.4721 .01769 L
.47721 .01676 L
.48196 .01605 L
.48625 .01554 L
.49082 .01513 L
.49343 .01496 L
.49471 .01489 L
.49589 .01484 L
.49694 .0148 L
.4981 .01477 L
.49926 .01474 L
.50049 .01472 L
.50179 .01472 L
.503 .01472 L
.50363 .01472 L
.50432 .01473 L
.50505 .01475 L
.50573 .01476 L
.50698 .0148 L
.50812 .01484 L
.51069 .01497 L
.51286 .01511 L
.51523 .0153 L
.52 .01579 L
.52524 .01652 L
.53083 .01749 L
.54091 .01977 L
.55116 .02278 L
.56226 .02682 L
.58241 .03624 L
.62333 .06342 L
.66273 .09933 L
.70062 .14217 L
.74096 .1958 L
.77978 .25409 L
.82106 .32187 L
Mistroke
.86082 .39139 L
.89907 .4608 L
.93976 .53599 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3o
ool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`00
0000oooo0?ooo`3l0?ooo`008@3oool200000740oooo4P00001j0?ooo`008@3oool00`000000oooo
0?ooo`1^0?ooo`8000004P3oool3000007L0oooo000Q0?ooo`030000003oool0oooo06`0oooo0P00
000G0?ooo`<00000M03oool00240oooo00<000000?ooo`3oool0J@3oool3000001`0oooo0`00001a
0?ooo`008@3oool00`000000oooo0?ooo`1V0?ooo`<000008P3oool2000006l0oooo000Q0?ooo`80
0000I@3oool2000002L0oooo0P00001]0?ooo`008@3oool00`000000oooo0?ooo`1R0?ooo`800000
:`3oool2000006/0oooo000Q0?ooo`030000003oool0oooo0600oooo0P00000_0?ooo`030000003o
ool0oooo06P0oooo000>0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000001@3oool0
0`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0300oooo0P00001X0?ooo`003@3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0GP3o
ool00`000000oooo0?ooo`0c0?ooo`800000IP3oool000d0oooo00@000000?ooo`3oool000002`3o
ool00`000000oooo0?ooo`020?ooo`800000G@3oool2000003P0oooo00<000000?ooo`3oool0H`3o
ool000L0oooo100000020?ooo`040000003oool0oooo000000T0oooo0P0000050?ooo`030000003o
ool0oooo05/0oooo00<000000?ooo`3oool0>@3oool2000006<0oooo000=0?ooo`040000003oool0
oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1J0?ooo`030000003o
ool0oooo03`0oooo00<000000?ooo`3oool0H03oool000d0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo05T0oooo00<000000?ooo`3oool0
?P3oool00`000000oooo0?ooo`1O0?ooo`003P3oool2000000X0oooo0P0000050?ooo`030000003o
ool0oooo05P0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`1N0?ooo`008@3oool2
000005P0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1M0?ooo`008@3oool00`00
0000oooo0?ooo`1E0?ooo`800000AP3oool00`000000oooo0?ooo`1L0?ooo`008@3oool00`000000
oooo0?ooo`1D0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0F`3oool00240oooo
00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo05X0oooo
000Q0?ooo`800000D`3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo05T0oooo000Q
0?ooo`030000003oool0oooo0540oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`1H
0?ooo`008@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo04l0oooo00<000000?oo
o`3oool0E`3oool00240oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`1A0?ooo`03
0000003oool0oooo05H0oooo000Q0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0
D`3oool00`000000oooo0?ooo`1E0?ooo`008@3oool2000004h0oooo00<000000?ooo`3oool0E@3o
ool00`000000oooo0?ooo`1D0?ooo`008@3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0
oooo05L0oooo00<000000?ooo`3oool0D`3oool00240oooo00<000000?ooo`3oool0B`3oool00`00
0000oooo0?ooo`1I0?ooo`030000003oool0oooo0580oooo000Q0?ooo`030000003oool0oooo04X0
oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`1A0?ooo`008@3oool00`000000oooo
0?ooo`190?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool0D@3oool00240oooo0P00
00190?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0D03oool00240oooo00<00000
0?ooo`3oool0A`3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0oooo04l0oooo000Q0?oo
o`030000003oool0oooo04H0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`1>0?oo
o`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`0000050?ooo`03
0000003oool0oooo04D0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`1=0?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3o
ool0C03oool000L0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo0080oooo0P0000150?ooo`030000003oool0oooo06L0oooo00<000000?ooo`3o
ool0B`3oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`00000:0?ooo`04
0000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo
06P0oooo00<000000?ooo`3oool0B`3oool000L0oooo00@000000?ooo`3oool000002`3oool01000
0000oooo0?ooo`0000070?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0JP3oool0
0`000000oooo0?ooo`1:0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`110?ooo`030000003o
ool0oooo06`0oooo00<000000?ooo`3oool0B@3oool000P0oooo0P00000:0?ooo`8000000`3oool4
000000@0oooo0P0000120?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0B03oool0
0240oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`1^0?ooo`030000003oool0oooo
04P0oooo000Q0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0L03oool00`000000
oooo0?ooo`170?ooo`008@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0740oooo
00<000000?ooo`3oool0AP3oool00240oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?oo
o`1b0?ooo`030000003oool0oooo04H0oooo000Q0?ooo`800000?P3oool00`000000oooo0?ooo`1d
0?ooo`030000003oool0oooo04D0oooo000Q0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool0MP3oool00`000000oooo0?ooo`140?ooo`008@3oool00`000000oooo0?ooo`0l0?ooo`03
0000003oool0oooo07L0oooo00<000000?ooo`3oool0@`3oool00240oooo00<000000?ooo`3oool0
>`3oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo04<0oooo000Q0?ooo`030000003o
ool0oooo03X0oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`120?ooo`008@3oool2
000003/0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`110?ooo`008@3oool00`00
0000oooo0?ooo`0i0?ooo`030000003oool0oooo07`0oooo00<000000?ooo`3oool0@@3oool00240
oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`1n0?ooo`030000003oool0oooo0400
oooo000Q0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0P03oool00`000000oooo
0?ooo`0o0?ooo`008@3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0800oooo00<0
00000?ooo`3oool0?`3oool00240oooo0P00000g0?ooo`030000003oool0oooo0880oooo00<00000
0?ooo`3oool0?P3oool00240oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`240?oo
o`030000003oool0oooo03d0oooo000>0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@0
0000103oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo08D0oooo00<000000?ooo`3o
ool0?@3oool000d0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool0QP3oool00`000000oooo0?ooo`0l0?ooo`00
3@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo00@0oooo0P00000d0?ooo`03
0000003oool0oooo08P0oooo00<000000?ooo`3oool0>`3oool000L0oooo100000020?ooo`040000
003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0b0?ooo`03
0000003oool0oooo08T0oooo00<000000?ooo`3oool0>`3oool000d0oooo00@000000?ooo`3oool0
00002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0340oooo00<000000?ooo`3o
ool0R`3oool00`000000oooo0?ooo`0j0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`2<0?oo
o`030000003oool0oooo03T0oooo000>0?ooo`8000002P3oool2000000D0oooo00<000000?ooo`3o
ool0<03oool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo03T0oooo000Q0?ooo`800000
<03oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo03P0oooo000Q0?ooo`030000003o
ool0oooo02l0oooo00<000000?ooo`3oool0T03oool00`000000oooo0?ooo`0g0?ooo`008@3oool0
0`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0=`3oool0
0240oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`2B0?ooo`030000003oool0oooo
03H0oooo000Q0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0U03oool00`000000
oooo0?ooo`0e0?ooo`008@3oool2000002d0oooo00<000000?ooo`3oool0U@3oool00`000000oooo
0?ooo`0e0?ooo`008@3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo09H0oooo00<0
00000?ooo`3oool0=03oool00240oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`2H
0?ooo`030000003oool0oooo03<0oooo000Q0?ooo`030000003oool0oooo02X0oooo00<000000?oo
o`3oool0V@3oool00`000000oooo0?ooo`0c0?ooo`008@3oool00`000000oooo0?ooo`0Z0?ooo`03
0000003oool0oooo09X0oooo00<000000?ooo`3oool0<P3oool00240oooo0P00000Z0?ooo`030000
003oool0oooo09/0oooo00<000000?ooo`3oool0<P3oool00240oooo00<000000?ooo`3oool0:@3o
ool00`000000oooo0?ooo`2L0?ooo`030000003oool0oooo0340oooo000Q0?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool0WP3oool00`000000oooo0?ooo`0`0?ooo`008@3oool00`00
0000oooo0?ooo`0W0?ooo`030000003oool0oooo09l0oooo00<000000?ooo`3oool0<03oool00240
oooo0P00000X0?ooo`030000003oool0oooo0:00oooo00<000000?ooo`3oool0;`3oool00240oooo
00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`2R0?ooo`030000003oool0oooo02h0oooo
000Q0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0X`3oool00`000000oooo0?oo
o`0^0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`000005
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0Y03oool00`000000oooo0?ooo`0]
0?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0:H0oooo00<00000
0?ooo`3oool0;03oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0080oooo0P00000U0?ooo`030000003oool0oooo0:H0oooo00<00000
0?ooo`3oool0;03oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`00000:
0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`0S0?ooo`030000003o
ool0oooo0:P0oooo00<000000?ooo`3oool0:`3oool000L0oooo00@000000?ooo`3oool000002P3o
ool01@000000oooo0?ooo`3oool0000000L0oooo00<000000?ooo`3oool08P3oool00`000000oooo
0?ooo`2Y0?ooo`030000003oool0oooo02/0oooo00070?ooo`040000003oool0oooo000000P0oooo
0`0000030?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool08P3oool00`000000oooo
0?ooo`2Z0?ooo`030000003oool0oooo02X0oooo00080?ooo`8000002`3oool010000000oooo0?oo
o`3oool4000000@0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`2[0?ooo`030000
003oool0oooo02X0oooo000Q0?ooo`8000008P3oool00`000000oooo0?ooo`2/0?ooo`030000003o
ool0oooo02T0oooo000Q0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0[@3oool0
0`000000oooo0?ooo`0Y0?ooo`008@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo
0:h0oooo00<000000?ooo`3oool0:03oool00240oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`2`0?ooo`030000003oool0oooo02L0oooo000Q0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0W0?ooo`008@3oool2000001l0oooo00<0
00000?ooo`3oool0/P3oool00`000000oooo0?ooo`0V0?ooo`008@3oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool09P3oool00240oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`2d0?ooo`030000003oool0oooo02D0oooo000Q0?ooo`03
0000003oool0oooo01`0oooo00<000000?ooo`3oool0]@3oool00`000000oooo0?ooo`0U0?ooo`00
8@3oool2000001`0oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?ooo`0T0?ooo`008@3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0;L0oooo00<000000?ooo`3oool0903o
ool00240oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`2i0?ooo`030000003oool0
oooo02<0oooo000Q0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0^P3oool00`00
0000oooo0?ooo`0R0?ooo`008@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0;/0
oooo00<000000?ooo`3oool08P3oool00240oooo0P00000J0?ooo`030000003oool0oooo0;`0oooo
00<000000?ooo`3oool08@3oool00240oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`2m0?ooo`030000003oool0oooo0240oooo000Q0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0_P3oool00`000000oooo0?ooo`0P0?ooo`003P3oool2000000@0oooo00<000000?oo
o`3oool00P3oool4000000@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`2o0?oo
o`030000003oool0oooo0200oooo000=0?ooo`040000003oool0oooo000000X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0<00oooo00<000000?oo
o`3oool07`3oool000d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?oo
o`8000005`3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo01l0oooo00070?ooo`@0
00000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`320?ooo`030000003oool0oooo01h0oooo000=0?ooo`04
0000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0E0?oo
o`030000003oool0oooo0<@0oooo00<000000?ooo`3oool07@3oool000d0oooo00@000000?ooo`3o
ool00000203oool3000000D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`340?oo
o`030000003oool0oooo01d0oooo000>0?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool0aP3oool00`000000oooo0?ooo`0L0?ooo`00
8@3oool2000001D0oooo00<000000?ooo`3oool0aP3oool00`000000oooo0?ooo`0L0?ooo`008@3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0<P0oooo00<000000?ooo`3oool06`3o
ool00240oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`380?ooo`030000003oool0
oooo01/0oooo000Q0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0bP3oool00`00
0000oooo0?ooo`0J0?ooo`008@3oool200000180oooo00<000000?ooo`3oool0b`3oool00`000000
oooo0?ooo`0J0?ooo`008@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0<`0oooo
00<000000?ooo`3oool06@3oool00240oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`3>0?ooo`030000003oool0oooo01P0oooo000Q0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0cP3oool00`000000oooo0?ooo`0H0?ooo`008@3oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo0=00oooo00<000000?ooo`3oool05`3oool00240oooo0P00000@0?ooo`03
0000003oool0oooo0=00oooo00<000000?ooo`3oool05`3oool00240oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`3B0?ooo`030000003oool0oooo01H0oooo000Q0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`0F0?ooo`008@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0=@0oooo00<000000?ooo`3oool05@3oool0
0240oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo
01D0oooo000Q0?ooo`8000003@3oool00`000000oooo0?ooo`3F0?ooo`030000003oool0oooo01@0
oooo000Q0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0e`3oool00`000000oooo
0?ooo`0D0?ooo`008@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0=P0oooo00<0
00000?ooo`3oool04`3oool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000003
0?ooo`<000001@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0=X0oooo00<00000
0?ooo`3oool04P3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`3J0?ooo`030000003oool0oooo0180oooo00070?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`8000002P3oool00`000000oooo
0?ooo`3L0?ooo`030000003oool0oooo0140oooo00000`3oool000000000000200000080oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`<000001@3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0=`0oooo00<000000?ooo`3oool04@3oool000L0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3N0?ooo`030000003oool0oooo
0100oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0gP3oool0
0`000000oooo0?ooo`0@0?ooo`00203oool2000000X0oooo0P0000030?ooo`@00000103oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03`3oool00240
oooo0P0000080?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03`3oool00240oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3R0?ooo`030000003oool0oooo00h0oooo
000Q0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0hP3oool00`000000oooo0?oo
o`0>0?ooo`008@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0>@0oooo00<00000
0?ooo`3oool03@3oool00240oooo0P0000060?ooo`030000003oool0oooo0>@0oooo00<000000?oo
o`3oool03@3oool00240oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3V0?ooo`03
0000003oool0oooo00`0oooo000Q0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
iP3oool00`000000oooo0?ooo`0<0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0>P0oooo00<000000?ooo`3oool02`3oool00240oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0Q0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000008`3oool2
000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0Q0?ooo`800000103oool0
0`000000oooo0?ooo`030?ooo`800000903oool2000000@0oooo00<000000?ooo`3oool00`3oool2
000002@0oooo00@000000?ooo`3oool0oooo100000060?ooo`008@3oool2000000<0oooo00<00000
0?ooo`3oool08@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo02<0oooo00@0
00000?ooo`3oool000002P3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo000002@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`050?ooo`008@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool000002@3o
ool00`000000oooo0?ooo`0R0?ooo`040000003oool0oooo000000L0oooo1@00000Q0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo000002@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`050?ooo`008@3oool01@000000oooo0?ooo`3oool0000002@0oooo00@000000?ooo`3oool00000
2P3oool00`000000oooo0?ooo`0Q0?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3o
ool000008P3oool010000000oooo0?ooo`0000080?ooo`<000008`3oool010000000oooo0?ooo`00
00090?ooo`8000009P3oool01@000000oooo0?ooo`3oool0000000L0oooo000Q0?ooo`050000003o
ool0oooo0?ooo`000000903oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0200
oooo00@000000?ooo`3oool00000203oool00`000000oooo0000000R0?ooo`040000003oool0oooo
000000P0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000002D0oooo00D000000?ooo`3oool0oooo000000070?ooo`008@3oool010000000oooo
0?ooo`00000U0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000008P3oool0
10000000oooo0?ooo`0000090?ooo`8000008P3oool010000000oooo0?ooo`0000080?ooo`030000
003oool0oooo02<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000V0?oo
o`@000001`3oool00240oooo0P0000000`3oool000000?ooo`0U0?ooo`8000002P3oool2000002@0
oooo0P00000;0?ooo`030000003oool0oooo0240oooo0P00000:0?ooo`<000008`3oool2000000X0
oooo0P00000W0?ooo`040000003oool0oooo000000L0oooo000Q0?ooo`030000003oool000000?80
oooo00<000000?ooo`3oool01`3oool00240oooo00<000000?ooo`000000lP3oool00`000000oooo
0?ooo`070?ooo`008@3oool200000?@0oooo00<000000?ooo`3oool01P3oool00240oooo0P00003d
0?ooo`030000003oool0oooo00H0oooo000K0?ooool000001P0000000240oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00D0oooo000Q0?ooo`030000
003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.134313, -0.347005, \
0.00403944, 0.00210345}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{\(35.010000000000005`\ Second\), ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imalpha[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.32182012515571833610076579675953635681`21.2802\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImalpha[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(-0.3218201251567538`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rearg1 = 
      Compile[{md2, k, kz, om, a, xi}, 
        1/\((1 + a\ xi^2)\)^2*\((om^2 - a\ kz^2  xi^2)\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(om\^2 - a\ kz\^2\ xi\^2\)\/\((1 + a\ xi\^2)\)\^2, 
        "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rearg2 = 
      Compile[{md2, k, kz, om, a, xi}, 
        Sign[om - kz\ xi]/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
          Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - om^2)\)*\((1 - 
                    kz^2/k^2)\)]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(Sign[om - kz\ xi]\ \((om + a\ kz\ xi)\)\ \@\(k\^2\ \((xi - \
\(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - om\^2)\)\ \((1 - kz\^2\/k\^2)\)\)\)\/\((1 \
+ a\ xi\^2)\)\^2, "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(FastRealpha[md2_, k_, kz_, om_, a_] := 
      Module[{res, mxi1, mxi2}, \[IndentingNewLine]mxi1 = 
          If[k > om, xi1[k, kz, om], 0]; 
        mxi2 = If[k > om, xi2[k, kz, om], 0]; \[IndentingNewLine]If[om < k, 
          res = \(md2/\((k^2 - kz^2)\)\)/
                2*\((NIntegrate[
                    Rearg1[md2, k, kz, om, a, xi], {xi, \(-1\), 1}] - 
                  NIntegrate[
                    Rearg2[md2, k, kz, om, a, xi], {xi, \(-1\), 
                      mxi2*If[
                          mxi2 < 
                            0, \((1 + 10^\((\(-15\))\))\), \((1 - 
                              10^\((\(-15\))\))\)]}] - 
                  NIntegrate[
                    Rearg2[md2, k, kz, om, a, xi], {xi, 
                      mxi1*\((1 + 10^\((\(-15\))\))\), 1}])\), 
          res = \(md2/\((k^2 - kz^2)\)\)/
                2*\((NIntegrate[
                    Rearg1[md2, k, kz, om, a, xi], {xi, \(-1\), 1}] - 
                  NIntegrate[
                    Rearg2[md2, k, kz, om, a, xi], {xi, \(-1\), 1}])\)]; 
        res]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRealpha[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(0.18138268946349065`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Realpha[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(0.18138268946347862309168323957536`21.2867\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Falpha[md2_, k_, kz_, om_, a_] := 
      FastRealpha[md2, k, kz, om, a] + 
        I*FastImalpha[md2, k, kz, om, a]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Falpha[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(\(0.1813826894634907`\)\(\[InvisibleSpace]\)\) - 
      0.3218201251567538`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRealpha[1, 1, Cos[Pi/4], 0.9, 0]\)], "Input"],

Cell[BoxData[
    \(0.5308747663583672`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRealpha[1, 1, Cos[Pi/4], 0.9, 1]*Sqrt[2]\)], "Input"],

Cell[BoxData[
    \(0.610108790238555`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRealpha[1, 1, Cos[Pi/4], 0.9, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(0.7953057868604838`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{FastRealpha[1, 1, Cos[Pi/4], om, 0], 
        FastRealpha[1, 1, Cos[Pi/4], om, 1]*Sqrt[2], 
        FastRealpha[1, 1, Cos[Pi/4], om, 10]*Sqrt[11], 
        FastImalpha[1, 1, Cos[Pi/4], om, 0], 
        FastImalpha[1, 1, Cos[Pi/4], om, 1]*Sqrt[2], 
        FastImalpha[1, 1, Cos[Pi/4], om, 10]*Sqrt[11]}, {om, 0, 2}, 
      PlotStyle \[Rule] {{}, {Dashing[{0.002, 0.01}]}, {Dashing[{0.02, 
                0.02}]}, {}, {Dashing[{0.002, 0.01}]}, {Dashing[{0.02, 
                0.02}]}}, AxesOrigin \[Rule] {0, \(-0.8\)}, 
      Frame \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.302621 0.352298 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.2619 -0.0125 -9 -9 ]
[.2619 -0.0125 9 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -9 -9 ]
[.7381 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .0384 -30 -4.5 ]
[-0.0125 .0384 0 4.5 ]
[-0.0125 .12647 -24 -4.5 ]
[-0.0125 .12647 0 4.5 ]
[-0.0125 .21455 -30 -4.5 ]
[-0.0125 .21455 0 4.5 ]
[-0.0125 .30262 -6 -4.5 ]
[-0.0125 .30262 0 4.5 ]
[-0.0125 .3907 -24 -4.5 ]
[-0.0125 .3907 0 4.5 ]
[-0.0125 .47877 -18 -4.5 ]
[-0.0125 .47877 0 4.5 ]
[-0.0125 .56684 -24 -4.5 ]
[-0.0125 .56684 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.2619 0 m
.2619 .00625 L
s
[(0.5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(1)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(1.5)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(2)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0384 m
.00625 .0384 L
s
[(-0.75)] -0.0125 .0384 1 0 Mshowa
0 .12647 m
.00625 .12647 L
s
[(-0.5)] -0.0125 .12647 1 0 Mshowa
0 .21455 m
.00625 .21455 L
s
[(-0.25)] -0.0125 .21455 1 0 Mshowa
0 .30262 m
.00625 .30262 L
s
[(0)] -0.0125 .30262 1 0 Mshowa
0 .3907 m
.00625 .3907 L
s
[(0.25)] -0.0125 .3907 1 0 Mshowa
0 .47877 m
.00625 .47877 L
s
[(0.5)] -0.0125 .47877 1 0 Mshowa
0 .56684 m
.00625 .56684 L
s
[(0.75)] -0.0125 .56684 1 0 Mshowa
.125 Mabswid
0 .05601 m
.00375 .05601 L
s
0 .07363 m
.00375 .07363 L
s
0 .09124 m
.00375 .09124 L
s
0 .10886 m
.00375 .10886 L
s
0 .14409 m
.00375 .14409 L
s
0 .1617 m
.00375 .1617 L
s
0 .17932 m
.00375 .17932 L
s
0 .19693 m
.00375 .19693 L
s
0 .23216 m
.00375 .23216 L
s
0 .24978 m
.00375 .24978 L
s
0 .26739 m
.00375 .26739 L
s
0 .28501 m
.00375 .28501 L
s
0 .32024 m
.00375 .32024 L
s
0 .33785 m
.00375 .33785 L
s
0 .35547 m
.00375 .35547 L
s
0 .37308 m
.00375 .37308 L
s
0 .40831 m
.00375 .40831 L
s
0 .42593 m
.00375 .42593 L
s
0 .44354 m
.00375 .44354 L
s
0 .46116 m
.00375 .46116 L
s
0 .49639 m
.00375 .49639 L
s
0 .514 m
.00375 .514 L
s
0 .53162 m
.00375 .53162 L
s
0 .54923 m
.00375 .54923 L
s
0 .02078 m
.00375 .02078 L
s
0 .00317 m
.00375 .00317 L
s
0 .58446 m
.00375 .58446 L
s
0 .60207 m
.00375 .60207 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.2619 .61178 m
.2619 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.7381 .61178 m
.7381 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0384 m
1 .0384 L
s
.99375 .12647 m
1 .12647 L
s
.99375 .21455 m
1 .21455 L
s
.99375 .30262 m
1 .30262 L
s
.99375 .3907 m
1 .3907 L
s
.99375 .47877 m
1 .47877 L
s
.99375 .56684 m
1 .56684 L
s
.125 Mabswid
.99625 .05601 m
1 .05601 L
s
.99625 .07363 m
1 .07363 L
s
.99625 .09124 m
1 .09124 L
s
.99625 .10886 m
1 .10886 L
s
.99625 .14409 m
1 .14409 L
s
.99625 .1617 m
1 .1617 L
s
.99625 .17932 m
1 .17932 L
s
.99625 .19693 m
1 .19693 L
s
.99625 .23216 m
1 .23216 L
s
.99625 .24978 m
1 .24978 L
s
.99625 .26739 m
1 .26739 L
s
.99625 .28501 m
1 .28501 L
s
.99625 .32024 m
1 .32024 L
s
.99625 .33785 m
1 .33785 L
s
.99625 .35547 m
1 .35547 L
s
.99625 .37308 m
1 .37308 L
s
.99625 .40831 m
1 .40831 L
s
.99625 .42593 m
1 .42593 L
s
.99625 .44354 m
1 .44354 L
s
.99625 .46116 m
1 .46116 L
s
.99625 .49639 m
1 .49639 L
s
.99625 .514 m
1 .514 L
s
.99625 .53162 m
1 .53162 L
s
.99625 .54923 m
1 .54923 L
s
.99625 .02078 m
1 .02078 L
s
.99625 .00317 m
1 .00317 L
s
.99625 .58446 m
1 .58446 L
s
.99625 .60207 m
1 .60207 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30262 m
.02499 .30262 L
.02605 .30263 L
.02729 .30264 L
.02846 .30265 L
.03053 .30269 L
.03279 .30275 L
.03527 .30283 L
.0379 .30293 L
.04262 .30317 L
.04749 .30349 L
.05205 .30386 L
.06244 .30494 L
.07305 .30637 L
.08274 .30799 L
.10458 .31266 L
.14429 .32469 L
.18248 .34026 L
.22313 .3606 L
.26226 .38316 L
.30384 .40926 L
.34391 .4351 L
.38246 .45882 L
.402 .46966 L
.42346 .47996 L
.43332 .48391 L
.44388 .48745 L
.449 .48885 L
.45385 .48996 L
.45821 .49076 L
.46068 .49112 L
.46295 .49138 L
.46528 .49159 L
.4665 .49167 L
.46779 .49173 L
.46892 .49176 L
.46997 .49177 L
.47112 .49177 L
.47235 .49174 L
.47304 .49171 L
.47369 .49168 L
.47514 .49158 L
.47642 .49146 L
.47778 .49129 L
.48013 .49092 L
.48272 .49036 L
.48546 .48957 L
.48795 .48866 L
.49084 .48731 L
.49357 .48567 L
Mistroke
.4961 .48373 L
.49843 .48131 L
.49978 .4793 L
.50051 .47773 L
.50119 .47669 L
.5024 .47519 L
.50373 .4738 L
.50882 .46962 L
.51432 .46619 L
.52429 .46142 L
.54291 .4551 L
.5633 .45019 L
.58187 .44679 L
.60261 .44377 L
.62177 .44149 L
.66015 .43793 L
.69947 .43522 L
.73727 .43321 L
.77753 .43151 L
.81627 .43019 L
.85746 .42904 L
.89714 .42811 L
.9353 .42735 L
.97591 .42666 L
.97619 .42666 L
Mfstroke
[ .002 .01 ] 0 setdash
.02381 .25758 m
.02499 .25759 L
.02605 .2576 L
.02729 .25761 L
.02846 .25764 L
.03053 .2577 L
.03279 .25779 L
.03527 .25792 L
.0379 .2581 L
.04262 .2585 L
.04749 .25904 L
.05205 .25966 L
.06244 .26146 L
.07305 .26386 L
.08274 .26656 L
.10458 .27434 L
.14429 .29426 L
.18248 .31973 L
.22313 .35234 L
.26226 .38737 L
.30384 .42593 L
.34391 .46127 L
.38246 .49019 L
.40239 .502 L
.41256 .50695 L
.42346 .51134 L
.42873 .5131 L
.43373 .51452 L
.4386 .51567 L
.4431 .51652 L
.44735 .51712 L
.44951 .51734 L
.45074 .51745 L
.45188 .51753 L
.45303 .5176 L
.45425 .51765 L
.45547 .51768 L
.45616 .51769 L
.45681 .5177 L
.45798 .51769 L
.45904 .51767 L
.46027 .51763 L
.46143 .51757 L
.46258 .51749 L
.46382 .51739 L
.46606 .51714 L
.46872 .51675 L
.47115 .51629 L
.47643 .51495 L
.48144 .51317 L
Mistroke
.48603 .51102 L
.49027 .50847 L
.49493 .50479 L
.49628 .50347 L
.49754 .50207 L
.4999 .49855 L
.50234 .49459 L
.50371 .49303 L
.50496 .49175 L
.50969 .48772 L
.51461 .48432 L
.52001 .48117 L
.5385 .47303 L
.55762 .46703 L
.5784 .46209 L
.59782 .45842 L
.61923 .45513 L
.65855 .45046 L
.69881 .44691 L
.73755 .44428 L
.77874 .44206 L
.81841 .44032 L
.85657 .43895 L
.89719 .43771 L
.93629 .43671 L
.97619 .43583 L
Mfstroke
[ .02 .02 ] 0 setdash
.02381 .14792 m
.02499 .14793 L
.02605 .14794 L
.02729 .14797 L
.02846 .148 L
.03053 .14808 L
.03279 .1482 L
.03527 .14838 L
.0379 .14861 L
.04262 .14915 L
.04749 .14987 L
.05205 .1507 L
.06244 .15312 L
.07305 .15638 L
.08274 .16006 L
.10458 .17082 L
.12357 .18303 L
.14429 .19948 L
.18493 .24181 L
.22406 .29623 L
.26565 .37004 L
.30571 .4551 L
.34426 .53777 L
.3642 .57179 L
.37424 .58469 L
.3799 .5905 L
.38527 .59499 L
.39015 .59822 L
.39273 .59959 L
.39547 .60081 L
.39777 .60164 L
.4003 .60235 L
.40141 .6026 L
.40261 .60283 L
.40366 .60299 L
.40478 .60313 L
.40609 .60324 L
.40731 .6033 L
.40806 .60332 L
.40877 .60332 L
.41009 .60328 L
.41077 .60324 L
.41149 .60319 L
.41278 .60306 L
.41416 .60287 L
.41569 .6026 L
.41818 .60205 L
.42086 .60129 L
.42569 .59951 L
.43582 .59436 L
Mistroke
.44687 .587 L
.46697 .57053 L
.48833 .55016 L
.4984 .53871 L
.50282 .5323 L
.50766 .52728 L
.51239 .5232 L
.51737 .51943 L
.52785 .5127 L
.54685 .50314 L
.56639 .49555 L
.58732 .48907 L
.62874 .47945 L
.66863 .47274 L
.70702 .46781 L
.74785 .46368 L
.78717 .46048 L
.82497 .45795 L
.86523 .4557 L
.90397 .45387 L
.94516 .45222 L
.97619 .45113 L
Mfstroke
[ ] 0 setdash
.02381 .30262 m
.06244 .28032 L
.10458 .25704 L
.14415 .23716 L
.18221 .22077 L
.20342 .21311 L
.22272 .20721 L
.24276 .20229 L
.26171 .19889 L
.27141 .19765 L
.2763 .19716 L
.28168 .19672 L
.28627 .19645 L
.28885 .19633 L
.2912 .19624 L
.29339 .19618 L
.29463 .19616 L
.29578 .19614 L
.29688 .19613 L
.29788 .19612 L
.29904 .19612 L
.30012 .19613 L
.30129 .19614 L
.30253 .19615 L
.30371 .19617 L
.30478 .1962 L
.3074 .19628 L
.30989 .19639 L
.31483 .19668 L
.3202 .19712 L
.3298 .19824 L
.33867 .19966 L
.35734 .2039 L
.36797 .2071 L
.37794 .21065 L
.41663 .22969 L
.43667 .24305 L
.45777 .25988 L
.47773 .27853 L
.48644 .28753 L
.49588 .29789 L
.49843 .3008 L
.49971 .30229 L
.50044 .30262 L
.50113 .30262 L
.50596 .30262 L
.51549 .30262 L
.53644 .30262 L
.57526 .30262 L
.61654 .30262 L
Mistroke
.6563 .30262 L
.69454 .30262 L
.73524 .30262 L
.77442 .30262 L
.81606 .30262 L
.85618 .30262 L
.89478 .30262 L
.93584 .30262 L
.97537 .30262 L
.97619 .30262 L
Mfstroke
[ .002 .01 ] 0 setdash
.02381 .30262 m
.06244 .26834 L
.10458 .2329 L
.14415 .20336 L
.18221 .18004 L
.20342 .16978 L
.22272 .16242 L
.23233 .15951 L
.24276 .15695 L
.24754 .15599 L
.25261 .15513 L
.25728 .15448 L
.26171 .15398 L
.26406 .15377 L
.26662 .15358 L
.26896 .15344 L
.27112 .15334 L
.27244 .15329 L
.27369 .15326 L
.27481 .15324 L
.27602 .15323 L
.27665 .15323 L
.27734 .15323 L
.27807 .15323 L
.27874 .15324 L
.27995 .15326 L
.28124 .15329 L
.28247 .15333 L
.2836 .15337 L
.28616 .15351 L
.28842 .15366 L
.29082 .15386 L
.29497 .1543 L
.29954 .15492 L
.30964 .15678 L
.31889 .1591 L
.33975 .16646 L
.35867 .17567 L
.37937 .1884 L
.41993 .22054 L
.45898 .25854 L
.47889 .27964 L
.48938 .29101 L
.49511 .29727 L
.49791 .30034 L
.49914 .30168 L
.49983 .30244 L
.50048 .30262 L
.50166 .30262 L
.50293 .30262 L
Mistroke
.50525 .30262 L
.51038 .30262 L
.52128 .30262 L
.54046 .30262 L
.58209 .30262 L
.62221 .30262 L
.66081 .30262 L
.70187 .30262 L
.74141 .30262 L
.77943 .30262 L
.81991 .30262 L
.85887 .30262 L
.90028 .30262 L
.94018 .30262 L
.97619 .30262 L
Mfstroke
[ .02 .02 ] 0 setdash
.02381 .30262 m
.06244 .24708 L
.10458 .18789 L
.14415 .13514 L
.18221 .08887 L
.20342 .06595 L
.22272 .04757 L
.24299 .03162 L
.2519 .02597 L
.26171 .0209 L
.26619 .01903 L
.271 .01737 L
.27366 .01661 L
.27609 .01602 L
.2784 .01556 L
.28086 .01518 L
.28224 .01501 L
.2835 .01489 L
.28476 .0148 L
.28595 .01474 L
.28663 .01473 L
.28734 .01472 L
.28862 .01472 L
.28933 .01474 L
.29 .01477 L
.29073 .01481 L
.29151 .01487 L
.29281 .01499 L
.29423 .01517 L
.29681 .0156 L
.29932 .01617 L
.30164 .01683 L
.30627 .01853 L
.31127 .02097 L
.3203 .02707 L
.33105 .0373 L
.34091 .04957 L
.35983 .08047 L
.38053 .12253 L
.42109 .20757 L
.43983 .23966 L
.46014 .26721 L
.47124 .2793 L
.48148 .28881 L
.49189 .29707 L
.4945 .29893 L
.49698 .30064 L
.49806 .30136 L
.49921 .30211 L
.50047 .30262 L
Mistroke
.50163 .30262 L
.51068 .30262 L
.5204 .30262 L
.54104 .30262 L
.58267 .30262 L
.62279 .30262 L
.66139 .30262 L
.70245 .30262 L
.74199 .30262 L
.78001 .30262 L
.82049 .30262 L
.85945 .30262 L
.90086 .30262 L
.94076 .30262 L
.97619 .30262 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`009_oo0P00=?oo0P001?oo00<00?ooool0
0_oo0`00=?oo1000<_oo10000ooo00<00?ooool00_oo0`00<ooo10001_oo000Uool01000ooooo`00
<_oo00@00?ooool000_oo`03003ooooo03?oo`03003ooooo03?oo`03003ooooo00[oo`03003ooooo
033oo`03003ooooo00Ooo`009Ooo00@00?ooool003;oo`04003ooooo000;ool00`00ooooo`0cool0
0`00ooooo`0cool00`00ooooo`0:ool00`00ooooo`0aool00`00ooooo`06ool002Goo`04003ooooo
000bool01000ooooo`002?oo0`00=_oo00<00?ooool0<ooo00<00?ooool01ooo0`00=Ooo00<00?oo
ool01Ooo000Uool01000ooooo`00<_oo00@00?ooool000Soo`03003ooooo03Koo`03003ooooo03?o
o`03003ooooo00Ooo`03003ooooo03Koo`03003ooooo00Coo`009Ooo00@00?ooool003;oo`04003o
oooo0008ool00`00ooooo`0dool3000cool30009ool00`00ooooo`0cool01000ooooo`001_oo000V
ool2000dool20009ool4000eool00`00ooooo`0cool00`00ooooo`07ool4000cool20007ool00?oo
ob7oo`00oooo8Ooo003ooolQool0027oooh0007oo`008Ooo0P001?oo00<00?ooool02Ooo00<00?oo
ool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?oo
ool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?oo
ool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?oo
ool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool00_oo0P000Ooo
000Qool00`00ooooo`3jool10001ool0027oo`03003ooooo04Goo`<00;;oo`40007oo`008Ooo00<0
0?ooool0A?oo00<00?ooool00`00/?oo0@000Ooo000Qool2000oool2002iool20001ool0027oo`03
003ooooo03coo`80013oo`03003ooooo0:Woo`40007oo`002?oo0P001?oo00<00?ooool00ooo00<0
0?ooool00_oo0`001Ooo00<00?ooool0>ooo00<00?ooool04?oo00<00?ooool0ZOoo0@000Ooo0007
ool01000ooooo`002Ooo00<00?ooool01Ooo00<00?ooool00_oo00<00?ooool0Cooo0P00ZOoo0@00
0Ooo0007ool01000ooooo`002_oo00<00?ooool01?oo00<00?ooool00_oo0P00D_oo00<00?ooool0
YOoo0P000Ooo00000ooo000000020002ool01000ooooo`002_oo00@00?ooooooo`<000Goo`03003o
oooo03Ooo`03003ooooo01Ooo`03003ooooo0:Koo`40007oo`001ooo00@00?ooool000_oo`04003o
oooo0007ool00`00ooooo`0fool00`00ooooo`31ool10001ool000Ooo`04003ooooo0008ool01000
ooooo`000_oo00<00?ooool01Ooo00<00?ooool0=Ooo00<00?ooool0`_oo0@000Ooo0008ool20009
ool40002ool40004ool00`00ooooo`0dool00`00ooooo`33ool10001ool0027oo`8003Coo`03003o
oooo01koo`03003ooooo0:;oo`80007oo`008Ooo00<00?ooool0EOoo00<00?ooool0X_oo0@000Ooo
000Qool00`00ooooo`1Eool00`00ooooo`2Rool10001ool0027oo`03003ooooo033oo`03003ooooo
02?oo`03003ooooo0:7oo`40007oo`008Ooo0P00<?oo00<00?ooool09Ooo00<00?ooool0Wooo0P00
0Ooo000Qool00`00ooooo`0^ool00`00ooooo`39ool10001ool0027oo`03003ooooo02goo`03003o
oooo0<[oo`40007oo`008Ooo00<00?ooool0;?oo00<00?ooool0booo0@000Ooo000Qool00`00oooo
o`3jool10001ool0027oo`8005[oo`03003ooooo09goo`80007oo`008Ooo00<00?ooool0F_oo00<0
0?ooool0WOoo0@000Ooo000Qool00`00ooooo`0Yool00`00ooooo`0^ool00`00ooooo`2Mool10001
ool0027oo`03003ooooo02Soo`03003ooooo033oo`03003ooooo09coo`40007oo`008Ooo0P00:Ooo
00<00?ooool0<Ooo00<00?ooool0V_oo0P000Ooo000Qool00`00ooooo`0Wool00`00ooooo`3@ool1
0001ool0027oo`03003ooooo02Koo`03003ooooo0=7oo`40007oo`003_oo0P001?oo00<00?ooool0
0_oo0`001Ooo00<00?ooool0n_oo0@000Ooo000=ool01000ooooo`002ooo00<00?ooool00_oo00<0
0?ooool0n_oo0@000Ooo000=ool01000ooooo`002ooo00<00?ooool00_oo0P00Gooo00<00?ooool0
V?oo0P000Ooo0007ool40002ool01000ooooo`002?oo0`001Ooo00<00?ooool08_oo00<00?ooool0
>Ooo00<00?ooool0VOoo0@000Ooo000=ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0
8Ooo00<00?ooool0>ooo00<00?ooool0V?oo0@000Ooo000=ool01000ooooo`002?oo00<00?ooool0
1Ooo00<00?ooool08?oo00<00?ooool0??oo00<00?ooool0V?oo0@000Ooo000>ool20009ool40004
ool2000Qool00`00ooooo`0mool00`00ooooo`2Fool20001ool0027oo`03003ooooo00?oo`8001[o
o`03003ooooo03koo`03003ooooo09Ooo`40007oo`008Ooo00<00?ooool02Ooo0`00k_oo0@000Ooo
000Qool00`00ooooo`0<ool3000]ool01000ooooo`000_oo00<00?ooool00P0000Goo`00003oo`00
0:koo`40007oo`008Ooo00<00?ooool0>Ooo00<00?ooool04?oo00<00?oo0000Zooo0@000Ooo000Q
ool2000Dool00`00ooooo`06ool00`00ooooo`0Gool00`00ooooo`0Hool00`00ooooo`0>ool00`00
ooooo`2Dool20001ool0027oo`03003ooooo01Coo`8000Goo`03003ooooo01Goo`8001ooo`03003o
oooo00_oo`03003ooooo09Goo`40007oo`008Ooo00<00?ooool05_oo0P000ooo00<00?ooool04_oo
0P008_oo00<00?ooool03?oo00<00?ooool0U?oo0@000Ooo000Qool00`00ooooo`0Jool00`00oooo
o`0Aool00`00ooooo`0Uool2000;ool00`00ooooo`2Dool10001ool0027oo`8001[oo`03003ooooo
00ooo`03003ooooo02_oo`03003ooooo00Soo`03003ooooo09;oo`80007oo`008Ooo00<00?ooool0
7?oo00<00?ooool0AOoo00<00?ooool0Tooo0@000Ooo000Qool00`00ooooo`0Mool00`00ooooo`08
ool2000bool00`00ooooo`2Kool10001ool0027oo`03003ooooo01koo`8000Koo`03003ooooo0=7o
o`40007oo`008Ooo00<00?ooool05_oo00<00?ooool01ooo00<00?ooool00ooo00<00?ooool07_oo
10004ooo0P00V_oo0@000Ooo000Qool2000Fool00`00ooooo`0;ool00`00ooooo`0Gool;0003ool:
000<ool00`00ooooo`03ool00`00ooooo`2?ool20001ool0027oo`03003ooooo01Goo`03003ooooo
023oo`D001Soo`@000Soo`03003ooooo00?oo`03003ooooo093oo`40007oo`002?oo0P001?oo00<0
0?ooool00_oo10000_oo0`001Ooo00<00?ooool05?oo00<00?ooool02Ooo0P0000?oo`00ool03ooo
10008Ooo0`001ooo00<00?ooool00_oo00<00?ooool0Sooo0@000Ooo0007ool01000ooooo`002?oo
00<00?ooool01_oo00<00?ooool00_oo00<00?ooool04ooo00<00?ooool02?oo00<00?ooool00ooo
00<00?ooool02ooo0P00:?oo0P001Ooo00<00?ooool00_oo00<00?ooool0Sooo0@000Ooo0007ool0
1000ooooo`002Ooo00<00?ooool01Ooo00<00?ooool00_oo0P007ooo00<00?ooool01?oo00<00?oo
ool01ooo0`00;?oo0P001Ooo00D00?ooooooo`0008ooo`80007oo`0000?oo`0000000P000_oo00@0
0?ooool000[oo`04003oooooool30005ool00`00ooooo`0Lool00`00ooooo`07ool00`00ooooo`04
ool2000aool20004ool01000ooooo`00T?oo0@000Ooo0007ool01000ooooo`002ooo00@00?ooool0
00Ooo`03003ooooo01coo`03003ooooo00Soo`03003ooooo00<003Goo`8000Coo`03003ooooo08oo
o`40007oo`001ooo00@00?ooool000Soo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`0@
ool00`00ooooo`06ool2000=ool2000jool2002Dool10001ool000Soo`8000[oo`8000?oo`@000Co
o`03003ooooo00ooo`03003ooooo01?oo`<003koo`05003oooooool0002?ool10001ool0027oo`80
013oo`03003ooooo00Goo`03003ooooo00Woo`8004;oo`800003ool0000008goo`80007oo`008Ooo
00<00?ooool03_oo00<00?ooool03ooo0`001ooo00<00?ooool0??oo1000SOoo0@000Ooo000Qool0
0`00ooooo`0=ool00`00ooooo`04ool20008ool2000;ool2000nool2002=ool10001ool0027oo`03
003ooooo01coo`8000koo`03003ooooo03goo`8008coo`40007oo`008Ooo0P004ooo00<00?ooool0
1Ooo0P004Ooo00<00?ooool0?Ooo0`00ROoo0P000Ooo000Qool00`00ooooo`0Hool2000Cool00`00
ooooo`0nool2002:ool10001ool0027oo`03003ooooo00Goo`800004ool000000005ool00`00oooo
o`04ool00`00ooooo`1Fool20028ool10001ool0027oo`03003ooooo00Woo`06003oo`00003oo`00
1_oo0P00F_oo00<00?ooool0Q_oo0@000Ooo000Qool00`00ooooo`09ool01000oooooooo0`0000?o
o`0000000P00G?oo0`00Q_oo0@000Ooo000Qool20009ool00`00ooooo`07ool01000ool000006_oo
00<00?ooool0?_oo00@00?oo000008Coo`80007oo`008Ooo00<00?ooool02?oo00<00?ooool00P00
0ooo0P001Ooo00<00?ooool05ooo00<00?ooool0?_oo00@00?ooool008Coo`40007oo`008Ooo00<0
0?ooool01ooo00<00?ooool01?oo0P008Ooo00<00?ooool0?ooo00@00?ooool008?oo`40007oo`00
8Ooo00<00?ooool02Ooo00<00?ooool00P003?oo00<00?ooool05Ooo00<00?ooool0?ooo0P0000?o
o`00ool0POoo0@000Ooo000Qool2000;ool2000@ool2000Eool00`00ooooo`11ool00`00ooooo`1o
ool20001ool0027oo`03003ooooo00Ooo`<001Goo`03003ooooo05Koo`80083oo`40007oo`006_oo
0P001Ooo00<00?ooool01Ooo0`006?oo00<00?ooool0E_oo00<00?ooool0O_oo0@000Ooo000Iool0
1000ooooo`001?oo00<00?ooool01?oo0P007?oo00<00?ooool0E?oo0P00Oooo0@000Ooo000Iool0
1000ooooo`001?oo0P001?oo0P007ooo00<00?ooool03ooo00<00?ooool0@OooN`001?oo0P000Ooo
000Iool01000ooooo`001?oo00<00?ooool01?oo3@005?oo00<00?ooool03ooo00<00?ooool0`?oo
0@000Ooo000Iool01000ooooo`001?oo00<00?ooool04Ooo10004_oo00<00?ooool03Ooo00<00?oo
ool0`?oo0@000Ooo000Iool01000ooooo`001?oo00<00?ooool05Ooo10007ooo00<00?ooool0_ooo
0@000Ooo000Jool20005ool00`00ooooo`0Iool3000=ool2000=ool00`00ooooo`2oool10001ool0
027oo`8001goo`@001Woo`03003ooooo0;goo`80007oo`008Ooo00<00?ooool08?oo0`002Ooo00<0
0?ooool0booo0@000Ooo000Qool00`00ooooo`0Sool2003Eool10001ool0027oo`03003ooooo02Go
o`<000Koo`800<[oo`40007oo`008Ooo0P00:Ooo0P001ooo00<00?ooool01ooo00<00?ooool0^ooo
0P000Ooo000Qool00`00ooooo`0Zool20005ool00`00ooooo`08ool00`00ooooo`2kool10001ool0
027oo`03003ooooo02coo`8000Goo`03003ooooo00Koo`03003ooooo0;_oo`40007oo`008Ooo00<0
0?ooool0;_oo0P000ooo00<00?ooool01ooo00<00?ooool0^_oo0@000Ooo000Qool00`00ooooo`0`
ool20003ool00`00ooooo`06ool00`00ooooo`2iool10001ool0027oo`8003?oo`8000;oo`03003o
oooo0<3oo`80007oo`008Ooo00<00?ooool0=?oo0P000_oo00<00?ooool0_ooo0@000Ooo000Qool0
0`00ooooo`0fool30031ool10001ool0027oo`03003ooooo03Soo`<00;ooo`40007oo`008Ooo0P00
>ooo00<00?ooool00ooo00<00?ooool0]_oo0P000Ooo000Qool00`00ooooo`0kool20003ool00`00
ooooo`2gool10001ool0027oo`03003ooooo03goo`8000;oo`03003ooooo0;Koo`40007oo`002?oo
0P001?oo00<00?ooool00_oo10000_oo0`001Ooo00<00?ooool0?_oo0P0000?oo`00ool0]ooo0@00
0Ooo0007ool01000ooooo`002?oo00<00?ooool01_oo00<00?ooool00_oo00<00?ooool0?ooo1000
]ooo0@000Ooo0007ool01000ooooo`002Ooo00<00?ooool01Ooo00<00?ooool00_oo0P00@_oo0P00
]_oo0P000Ooo0007ool01000ooooo`002_oo00@00?ooooooo`<000Goo`03003ooooo047oo`04003o
o`00002eool10001ool000Ooo`04003ooooo000;ool01000ooooo`001ooo00<00?ooool0@ooo00@0
0?oo00000;?oo`40007oo`001ooo00@00?ooool000Soo`04003ooooo0002ool00`00ooooo`05ool0
0`00ooooo`13ool01@00oooooooo0000/_oo0@000Ooo0008ool2000:ool20003ool40004ool20015
ool01000oooooooo0P00[ooo0P000Ooo000Qool00`00ooooo`15ool20003ool00`00ooooo`2]ool1
0001ool0027oo`03003ooooo04Goo`03003ooooo00?oo`800:goo`40007oo`008Ooo00<00?ooool0
A_oo0`001?oo0P00Zooo0@000Ooo000Qool00`00ooooo`16ool00`00ooooo`06ool00`00ooooo`2X
ool10001ool0027oo`8004Soo`04003ooooo0005ool20029oolJ0004ool20001ool0027oo`03003o
oooo05;oo`8006Wooah001ooo`40007oo`008Ooo00<00?ooool0C?oo00<00?ooool01Ooo00<00?oo
ool0D_oo5000<_oo00@00?ooool000Ooo`40007oo`008Ooo00<00?ooool0EOoo0P00B?oo2P008_oo
00D00?oo00000?oo00800003ool0000000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?oo
ool000;oo`04003ooooo000=ool10001ool0027oo`8004[oo`03003ooooo00;oo`8000Ooo`8003[o
o``001Woo`800004ool00000ool200001?oo00000?oo0P0000Goo`00003oo`0002koo`80007oo`00
8Ooo00<00?ooool0BOoo00<00?ooool03Ooo00<00?ooool0<_oo1@006Ooo00@00?ooool000;oo`80
0003ool0000004?oo`40007oo`008Ooo00<00?ooool0B_oo00<00?ooool00ooo0P002?oo0P00;Ooo
1@004_oo00@00?ooool000;oo`04003ooooo0012ool60008ool10001ool0027oo`03003ooooo04[o
o`03003ooooo00ooo`8002Ooo`@0017oo`04003ooooo000iool60005ool60004ool00`00ooooo`0;
ool10001ool0027oo`03003ooooo04[oo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo02;o
o`8000ooo`04003ooooo000^ool30004ool6000Wool10001ool0027oo`8004coo`03003ooooo017o
o`80023oo`8000_oo`04003ooooo000Xool50004ool3000cool20001ool0027oo`03003ooooo05Go
o`03003ooooo00Woo`8001coo`8000[oo`03003ooooo02;oo`D000Coo`03003ooooo03koo`40007o
o`003_oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0F?oo00<00?ooool02?oo0P006?oo
0P002Ooo00<00?ooool07Ooo0`001?oo00<00?ooool0B?oo0@000Ooo000=ool01000ooooo`002ooo
00<00?ooool00_oo00<00?ooool0F?oo00<00?ooool02_oo0P005Ooo00<00?ooool01_oo00<00?oo
ool07Ooo0`00D_oo0@000Ooo000=ool01000ooooo`002ooo00<00?ooool00_oo0P00C_oo00<00?oo
ool02_oo00<00?ooool02_oo0`004Ooo0P001Ooo0P006?oo1@00F?oo0P000Ooo000=ool01000oooo
o`002?oo0`001Ooo00<00?ooool0COoo00<00?ooool02_oo00<00?ooool03Ooo0P003ooo00<00?oo
ool00_oo00<00?ooool06?oo00<00?ooool0G?oo0@000Ooo000=ool01000ooooo`002?oo00<00?oo
ool01Ooo00<00?ooool0C_oo00<00?ooool02ooo0P003_oo0P002ooo0P006Ooo0`00Hooo0@000Ooo
000=ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0C_oo00<00?ooool07Ooo2`001?oo
00<00?ooool04Ooo0`00I_oo0@000Ooo000>ool20009ool40004ool00`00ooooo`1?ool00`00oooo
o`0=ool00`00ooooo`2Hool10001ool0027oo`80053oo`03003ooooo013oo`03003ooooo01Koo`80
00goo`<006coo`80007oo`008Ooo00<00?ooool0H_oo00<00?ooool05_oo00<00?ooool02Ooo0`00
L?oo0@000Ooo000Qool00`00ooooo`1Tool2000Cool00`00ooooo`1nool10001ool0027oo`03003o
oooo06Ooo`03003ooooo00ooo`03003ooooo07koo`40007oo`008Ooo0P00Jooo00<00?ooool02Ooo
00<00?ooool02Ooo0P00MOoo0P000Ooo000Qool00`00ooooo`1Aool00`00ooooo`0Iool01000oooo
o`000_oo00<00?ooool02_oo0P00N?oo0@000Ooo000Qool00`00ooooo`1Bool00`00ooooo`0Zool0
0`00ooooo`1hool10001ool0027oo`03003ooooo05;oo`03003ooooo0:Goo`40007oo`008Ooo00<0
0?ooool0Dooo00<00?ooool0Y?oo0@000Ooo000Qool2001Eool00`00ooooo`2Rool20001ool0027o
o`03003ooooo07_oo`03003ooooo07coo`40007oo`008Ooo00<00?ooool0N_oo00<00?ooool0OOoo
0@000Ooo000Qool00`00ooooo`1jool00`00ooooo`1mool10001ool0027oo`8007[oo`03003ooooo
07goo`80007oo`008Ooo00<00?ooool0E_oo00<00?ooool07ooo00<00?ooool0Oooo0@000Ooo000Q
ool00`00ooooo`1Gool00`00ooooo`2Pool10001ool000Soo`8000Coo`03003ooooo00?oo`03003o
oooo00;oo`<000Goo`03003ooooo05Ooo`03003ooooo0:3oo`40007oo`001ooo00@00?ooool000Wo
o`03003ooooo00Goo`03003ooooo00;oo`03003ooooo05Soo`03003ooooo01Woo`03003ooooo08?o
o`40007oo`001ooo00@00?ooool000[oo`03003ooooo00Coo`03003ooooo00;oo`8005[oo`03003o
oooo01Ooo`03003ooooo08?oo`80007oo`001ooo00@00?ooool000[oo`04003oooooool30005ool0
0`00ooooo`1bool00`00ooooo`25ool10001ool000Ooo`04003ooooo000;ool01000ooooo`001ooo
00<00?ooool0LOoo00<00?ooool0Q_oo0@000Ooo0007ool01000ooooo`002?oo00@00?ooool000;o
o`03003ooooo00Goo`03003ooooo073oo`03003ooooo08Ooo`40007oo`002?oo0P002Ooo10000_oo
10001?oo0P00GOoo00<00?ooool0V_oo0P000Ooo000Qool00`00ooooo`1Lool00`00ooooo`2Kool1
0001ool0027oo`03003ooooo05goo`03003ooooo00coo`03003ooooo08_oo`40007oo`008Ooo00<0
0?ooool0G_oo0P002ooo00<00?ooool0S?oo0@000Ooo000Qool2001Xool3002?ool20001ool0027o
o`03003ooooo06Coo`<009?oo`40007oo`008Ooo00<00?ooool0n_oo0@000Ooo000Qool00`00oooo
o`03ool00`00ooooo`0iool00`00ooooo`0jool00`00ooooo`0iool00`00ooooo`0iool00`00oooo
o`03ool10001ool0027oo`03003ooooo00?oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo
00Woo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo
00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo
00Woo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo
00Woo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00?oo`40007oo`008OoooP000Ooo003o
oolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.324585, -1.03976, \
0.00827746, 0.0111884}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<alpha1.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("alpha1.eps"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(alphaspecial[md2_, k_, kz_, om_, a_]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{alphaspecial[1, 1, Cos[Pi/4], I\ om, 0], 
        alphaspecial[1, 1, Cos[Pi/4], I\ om, 1]*Sqrt[2], 
        alphaspecial[1, 1, Cos[Pi/4], I\ om, 10]*Sqrt[11]}, {om, 0, 2}, 
      PlotStyle \[Rule] {{}, {Dashing[{0.002, 0.01}]}, {Dashing[{0.02, 
                0.02}]}}, AxesOrigin \[Rule] {0, \(-0.2\)}, 
      Frame \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/\((1 + \(\(0\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.08113398314583158`\\ \[ImaginaryI]\)\))\)\) + \
\(\(0\\ \(\(\[LeftSkeleton] 19 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(0\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.16961759971874735`\\ \[ImaginaryI]\)\))\)\) + \
\(\(0\\ \(\(\[LeftSkeleton] 19 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(0\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.221336 1.09793 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.2619 -0.0125 -9 -9 ]
[.2619 -0.0125 9 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -9 -9 ]
[.7381 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .00175 -24 -4.5 ]
[-0.0125 .00175 0 4.5 ]
[-0.0125 .11154 -24 -4.5 ]
[-0.0125 .11154 0 4.5 ]
[-0.0125 .22134 -6 -4.5 ]
[-0.0125 .22134 0 4.5 ]
[-0.0125 .33113 -18 -4.5 ]
[-0.0125 .33113 0 4.5 ]
[-0.0125 .44092 -18 -4.5 ]
[-0.0125 .44092 0 4.5 ]
[-0.0125 .55072 -18 -4.5 ]
[-0.0125 .55072 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.2619 0 m
.2619 .00625 L
s
[(0.5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(1)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(1.5)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(2)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .00175 m
.00625 .00175 L
s
[(-0.2)] -0.0125 .00175 1 0 Mshowa
0 .11154 m
.00625 .11154 L
s
[(-0.1)] -0.0125 .11154 1 0 Mshowa
0 .22134 m
.00625 .22134 L
s
[(0)] -0.0125 .22134 1 0 Mshowa
0 .33113 m
.00625 .33113 L
s
[(0.1)] -0.0125 .33113 1 0 Mshowa
0 .44092 m
.00625 .44092 L
s
[(0.2)] -0.0125 .44092 1 0 Mshowa
0 .55072 m
.00625 .55072 L
s
[(0.3)] -0.0125 .55072 1 0 Mshowa
.125 Mabswid
0 .02371 m
.00375 .02371 L
s
0 .04567 m
.00375 .04567 L
s
0 .06763 m
.00375 .06763 L
s
0 .08958 m
.00375 .08958 L
s
0 .1335 m
.00375 .1335 L
s
0 .15546 m
.00375 .15546 L
s
0 .17742 m
.00375 .17742 L
s
0 .19938 m
.00375 .19938 L
s
0 .24329 m
.00375 .24329 L
s
0 .26525 m
.00375 .26525 L
s
0 .28721 m
.00375 .28721 L
s
0 .30917 m
.00375 .30917 L
s
0 .35309 m
.00375 .35309 L
s
0 .37505 m
.00375 .37505 L
s
0 .39701 m
.00375 .39701 L
s
0 .41896 m
.00375 .41896 L
s
0 .46288 m
.00375 .46288 L
s
0 .48484 m
.00375 .48484 L
s
0 .5068 m
.00375 .5068 L
s
0 .52876 m
.00375 .52876 L
s
0 .57267 m
.00375 .57267 L
s
0 .59463 m
.00375 .59463 L
s
0 .61659 m
.00375 .61659 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.2619 .61178 m
.2619 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.7381 .61178 m
.7381 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .00175 m
1 .00175 L
s
.99375 .11154 m
1 .11154 L
s
.99375 .22134 m
1 .22134 L
s
.99375 .33113 m
1 .33113 L
s
.99375 .44092 m
1 .44092 L
s
.99375 .55072 m
1 .55072 L
s
.125 Mabswid
.99625 .02371 m
1 .02371 L
s
.99625 .04567 m
1 .04567 L
s
.99625 .06763 m
1 .06763 L
s
.99625 .08958 m
1 .08958 L
s
.99625 .1335 m
1 .1335 L
s
.99625 .15546 m
1 .15546 L
s
.99625 .17742 m
1 .17742 L
s
.99625 .19938 m
1 .19938 L
s
.99625 .24329 m
1 .24329 L
s
.99625 .26525 m
1 .26525 L
s
.99625 .28721 m
1 .28721 L
s
.99625 .30917 m
1 .30917 L
s
.99625 .35309 m
1 .35309 L
s
.99625 .37505 m
1 .37505 L
s
.99625 .39701 m
1 .39701 L
s
.99625 .41896 m
1 .41896 L
s
.99625 .46288 m
1 .46288 L
s
.99625 .48484 m
1 .48484 L
s
.99625 .5068 m
1 .5068 L
s
.99625 .52876 m
1 .52876 L
s
.99625 .57267 m
1 .57267 L
s
.99625 .59463 m
1 .59463 L
s
.99625 .61659 m
1 .61659 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .22134 m
.06244 .28452 L
.10458 .33992 L
.14415 .38158 L
.18221 .41404 L
.22272 .44203 L
.26171 .46386 L
.30316 .48271 L
.34309 .4975 L
.3815 .50924 L
.42237 .5196 L
.46172 .52789 L
.49955 .53461 L
.53984 .54067 L
.57861 .54563 L
.61984 .55013 L
.65954 .55384 L
.69774 .55693 L
.73838 .5598 L
.77751 .56221 L
.81909 .56445 L
.85916 .56634 L
.89771 .56796 L
.93871 .56949 L
.97619 .57074 L
s
[ .002 .01 ] 0 setdash
.02381 .08097 m
.06244 .17741 L
.10458 .26049 L
.14415 .32195 L
.18221 .36918 L
.22272 .4094 L
.26171 .44043 L
.30316 .46697 L
.34309 .48763 L
.3815 .50392 L
.42237 .5182 L
.46172 .52958 L
.49955 .53876 L
.53984 .54701 L
.57861 .55374 L
.61984 .55983 L
.65954 .56484 L
.69774 .56901 L
.73838 .57287 L
.77751 .5761 L
.81909 .5791 L
.85916 .58164 L
.89771 .58381 L
.93871 .58585 L
.97619 .58752 L
s
[ .02 .02 ] 0 setdash
.09331 0 m
.10458 .03749 L
.14415 .14313 L
.18221 .22514 L
.22272 .29535 L
.26171 .34961 L
.30316 .39597 L
.34309 .43196 L
.3815 .46027 L
.42237 .48499 L
.46172 .50464 L
.49955 .52044 L
.53984 .53459 L
.57861 .54609 L
.61984 .55648 L
.65954 .56501 L
.69774 .57209 L
.73838 .57862 L
.77751 .58408 L
.81909 .58916 L
.85916 .59344 L
.89771 .59708 L
.93871 .60052 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Pool2000fool20004ool00`00ooooo`02ool3000eool4000cool4
0003ool00`00ooooo`02ool3000eool40006ool001ooo`04003ooooo000dool01000ooooo`002ooo
00<00?ooool0=?oo00<00?ooool0=?oo00<00?ooool02_oo00<00?ooool0<_oo00<00?ooool01ooo
000Oool01000ooooo`00=?oo00@00?ooool000_oo`03003ooooo03Coo`03003ooooo03Coo`03003o
oooo00[oo`03003ooooo03?oo`03003ooooo00Koo`007ooo00@00?ooool003Coo`04003ooooo0008
ool3000gool00`00ooooo`0dool00`00ooooo`07ool3000gool00`00ooooo`05ool001ooo`04003o
oooo000dool01000ooooo`002?oo00<00?ooool0=ooo00<00?ooool0=?oo00<00?ooool01ooo00<0
0?ooool0>?oo00<00?ooool01?oo000Oool01000ooooo`00=?oo00@00?ooool000Soo`03003ooooo
03Goo`<003Coo`<000Woo`03003ooooo03Goo`04003ooooo0006ool0023oo`8003Koo`8000Woo`@0
03Koo`03003ooooo03Coo`03003ooooo00Ooo`@003Goo`8000Ooo`00oooo8Ooo0008ool20004ool0
0`00ooooo`02ool4003oool:ool000Ooo`04003ooooo0008ool00`00ooooo`3oool;ool000Ooo`04
003ooooo0009ool00`00ooooo`04oooo00050001ool00003ool00000008000;oo`04003ooooo000:
ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`04
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`03ool10001ool000Oo
o`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`0?ool00`00ooooo`0Y
ool00`00ooooo`0kool00`00ooooo`0jool00`00ooooo`0kool00`00ooooo`03ool10001ool000Oo
o`04003ooooo0008ool01000ooooo`001?oo00<00?ooool05_oo00<00?ooool0iooo0@000Ooo0008
ool2000:ool20005ool00`00ooooo`0Fool00`00ooooo`3Wool10001ool001_oo`03003ooooo01Ko
o`03003ooooo0>Ooo`40007oo`006ooo0P00oooo0Ooo0P000Ooo000Kool00`00ooooo`3oool1ool1
0001ool001_oo`03003ooooo0?ooo`7oo`40007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo000K
ool00`00ooooo`0Hool00`00ooooo`3Uool10001ool001_oo`03003ooooo01Soo`03003ooooo0>Go
o`40007oo`006ooo0P006_oo00<00?ooool0hooo0P000Ooo000Kool00`00ooooo`0Iool00`00oooo
o`3Tool10001ool001_oo`03003ooooo01[oo`03003ooooo0>?oo`40007oo`006ooo00<00?ooool0
6_oo00<00?ooool0hooo0@000Ooo000Kool00`00ooooo`3oool1ool10001ool001_oo`800?ooo`7o
o`80007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo000Kool00`00ooooo`3oool1ool10001ool0
01_oo`03003ooooo01coo`03003ooooo0>7oo`40007oo`006ooo00<00?ooool00ooo00<00?ooool0
5_oo00<00?ooool0hOoo0@000Ooo000Kool00`00ooooo`0Lool00`00ooooo`3Qool10001ool001_o
o`8001koo`03003ooooo0=ooo`80007oo`006ooo00<00?ooool01?oo00<00?ooool05_oo00<00?oo
ool0h?oo0@000Ooo000Kool00`00ooooo`3oool1ool10001ool001_oo`03003ooooo0?ooo`7oo`40
007oo`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool01Ooo00<00?ooool0n?oo0@00
0Ooo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool0oooo0Ooo0@000Ooo0007ool0
1000ooooo`002_oo00<00?ooool00ooo0P008?oo00<00?ooool0gOoo0P000Ooo00000ooo00000002
0002ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool01_oo0P005ooo00<00?ooool0g_oo
0@000Ooo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool08?oo00<00?ooool0gOoo
0@000Ooo0007ool01000ooooo`002?oo0`001Ooo00<00?ooool08?oo00<00?ooool0gOoo0@000Ooo
0008ool2000;ool00`00ooooo`03ool00`00ooooo`08ool00`00ooooo`0Fool00`00ooooo`3Lool1
0001ool001_oo`03003ooooo027oo`03003ooooo0=coo`40007oo`006ooo0P002_oo00<00?ooool0
looo0P000Ooo000Kool00`00ooooo`09ool00`00ooooo`3dool10001ool001_oo`03003ooooo0?oo
o`7oo`40007oo`006ooo00<00?ooool02_oo00<00?ooool0looo0@000Ooo000Kool00`00ooooo`0:
ool00`00ooooo`0Fool00`00ooooo`3Jool10001ool001_oo`8002Coo`03003ooooo0=Woo`80007o
o`006ooo00<00?ooool02ooo00<00?ooool05_oo00<00?ooool0fOoo0@000Ooo000Kool00`00oooo
o`0;ool00`00ooooo`0Fool00`00ooooo`3Iool10001ool001_oo`03003ooooo02Goo`03003ooooo
0=Soo`40007oo`006ooo00<00?ooool03?oo00<00?ooool05_oo00<00?ooool0f?oo0@000Ooo000K
ool00`00ooooo`0<ool00`00ooooo`3aool10001ool001_oo`800?ooo`7oo`80007oo`006ooo00<0
0?ooool03Ooo0P00lOoo0@000Ooo000Kool00`00ooooo`0Wool00`00ooooo`3Fool10001ool001_o
o`03003ooooo02Soo`03003ooooo0=Goo`40007oo`006ooo00<00?ooool03ooo00<00?ooool05_oo
00<00?ooool0eOoo0@000Ooo000Kool00`00ooooo`0Yool00`00ooooo`3Dool10001ool001_oo`80
02[oo`03003ooooo0=?oo`80007oo`006ooo00<00?ooool04?oo00<00?ooool05ooo00<00?ooool0
dooo0@000Ooo000Kool00`00ooooo`3oool1ool10001ool001Coo`8000Goo`03003ooooo01;oo`03
003ooooo0>_oo`40007oo`004ooo00@00?ooool000Coo`03003ooooo01;oo`03003ooooo0>_oo`40
007oo`004ooo00@00?ooool000Coo`8000Coo`03003ooooo0?Woo`80007oo`004ooo00@00?ooool0
00Coo`03003ooooo00Coo`03003ooooo00coo`03003ooooo01Koo`03003ooooo0=7oo`40007oo`00
4ooo00@00?ooool000Coo`03003ooooo00Coo`03003ooooo00coo`03003ooooo01Koo`03003ooooo
0=7oo`40007oo`004ooo00@00?ooool000Coo`03003ooooo00Goo`03003ooooo02Goo`03003ooooo
0=3oo`40007oo`005?oo0P001Ooo00<00?ooool01Ooo00<00?ooool03?oo0P005ooo00<00?ooool0
d?oo0@000Ooo000Kool00`00ooooo`06ool00`00ooooo`0Uool00`00ooooo`3?ool10001ool001_o
o`8000Soo`03003ooooo0?Goo`80007oo`006ooo00<00?ooool01ooo00<00?ooool03?oo00<00?oo
ool0iooo0@000Ooo000Kool00`00ooooo`08ool00`00ooooo`3eool10001ool001_oo`03003ooooo
00Soo`03003ooooo00coo`03003ooooo0>Koo`40007oo`006ooo00<00?ooool02Ooo00<00?ooool0
3?oo00<00?ooool05_oo00<00?ooool0c?oo0@000Ooo000Kool00`00ooooo`09ool00`00ooooo`0U
ool00`00ooooo`3<ool10001ool001_oo`8000_oo`03003ooooo00coo`03003ooooo01Koo`03003o
oooo0<[oo`80007oo`006ooo00<00?ooool02ooo00<00?ooool02ooo00<00?ooool05_oo00<00?oo
ool0booo0@000Ooo000Kool00`00ooooo`0;ool00`00ooooo`0Uool00`00ooooo`3:ool10001ool0
01_oo`03003ooooo00coo`03003ooooo00coo`03003ooooo0>;oo`40007oo`006ooo00<00?ooool0
3?oo00<00?ooool0lOoo0@000Ooo000Kool2000>ool00`00ooooo`0<ool00`00ooooo`3Pool20001
ool001_oo`03003ooooo00koo`03003ooooo00coo`03003ooooo0>3oo`40007oo`006ooo00<00?oo
ool03ooo00<00?ooool09?oo00<00?ooool0aooo0@000Ooo000Kool00`00ooooo`0?ool00`00oooo
o`0<ool00`00ooooo`0Fool00`00ooooo`36ool10001ool001_oo`03003ooooo013oo`03003ooooo
02Goo`03003ooooo0<Goo`40007oo`006ooo00<00?ooool04Ooo00<00?ooool09?oo00<00?ooool0
aOoo0@000Ooo000Kool2000Cool00`00ooooo`0;ool00`00ooooo`0Fool00`00ooooo`33ool20001
ool001_oo`03003ooooo01;oo`03003ooooo0>_oo`40007oo`006ooo00<00?ooool04ooo00<00?oo
ool02ooo00<00?ooool0g?oo0@000Ooo000Kool00`00ooooo`0Dool00`00ooooo`0;ool00`00oooo
o`3Kool10001ool000Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo01Goo`03003ooooo
02Coo`03003ooooo0<7oo`40007oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo
01Koo`03003ooooo00[oo`03003ooooo01Ooo`03003ooooo0<3oo`40007oo`001ooo00@00?ooool0
00[oo`03003ooooo00?oo`8001Ooo`03003ooooo02Coo`03003ooooo0;ooo`80007oo`001ooo00@0
0?ooool000[oo`03003ooooo00?oo`03003ooooo01Ooo`03003ooooo00_oo`03003ooooo01Koo`03
003ooooo0;ooo`40007oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo01Soo`03
003ooooo00[oo`03003ooooo01Koo`03003ooooo0;ooo`40007oo`001ooo00@00?ooool000Soo`<0
00Goo`03003ooooo01Woo`03003ooooo02?oo`03003ooooo0;koo`40007oo`002?oo0P002ooo00<0
0?ooool00ooo00<00?ooool06_oo00<00?ooool02_oo00<00?ooool0e_oo0@000Ooo000Kool00`00
ooooo`0Kool00`00ooooo`3Rool10001ool001_oo`8001goo`03003ooooo00[oo`03003ooooo0=?o
o`80007oo`006ooo00<00?ooool07Ooo00<00?ooool08_oo00<00?ooool0^ooo0@000Ooo000Kool0
0`00ooooo`0Mool00`00ooooo`0;ool00`00ooooo`0Eool00`00ooooo`2jool10001ool001_oo`03
003ooooo01koo`03003ooooo00[oo`03003ooooo01Koo`800;[oo`40007oo`006ooo00<00?ooool0
7ooo00<00?ooool09?oo00<00?ooool0]ooo0@000Ooo000Kool2000Qool00`00ooooo`0:ool00`00
ooooo`3?ool20001ool001_oo`03003ooooo027oo`03003ooooo0=coo`40007oo`006ooo00<00?oo
ool08_oo00<00?ooool02_oo0P00cooo0@000Ooo000Kool00`00ooooo`0Sool00`00ooooo`0Sool0
0`00ooooo`2dool10001ool001_oo`03003ooooo02Coo`8000_oo`03003ooooo01Koo`03003ooooo
0;?oo`40007oo`006ooo00<00?ooool09_oo00<00?ooool02Ooo00<00?ooool05Ooo00<00?ooool0
/ooo0@000Ooo000Kool2000Xool00`00ooooo`0:ool00`00ooooo`0Dool00`00ooooo`2aool20001
ool001_oo`03003ooooo02Soo`03003ooooo00Woo`03003ooooo01Goo`800;;oo`40007oo`006ooo
00<00?ooool0:Ooo0P00eOoo0@000Ooo000Kool00`00ooooo`0[ool00`00ooooo`08ool00`00oooo
o`37ool10001ool001_oo`03003ooooo02coo`03003ooooo0=7oo`40007oo`006ooo00<00?ooool0
;Ooo0P002Ooo0P005ooo00<00?ooool0[?oo0@000Ooo000Kool2000`ool00`00ooooo`09ool00`00
ooooo`0Dool00`00ooooo`2Zool20001ool001_oo`03003ooooo033oo`03003ooooo00Soo`03003o
oooo01Goo`800:_oo`40007oo`006ooo00<00?ooool0<Ooo0P002_oo00<00?ooool05Ooo00<00?oo
ool0Z?oo0@000Ooo0008ool20004ool00`00ooooo`02ool40004ool00`00ooooo`0cool00`00oooo
o`07ool00`00ooooo`30ool10001ool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00oooo
o`0dool00`00ooooo`09ool00`00ooooo`2mool10001ool000Ooo`04003ooooo0009ool00`00oooo
o`04ool2000fool20009ool00`00ooooo`0Fool00`00ooooo`2Sool20001ool000Ooo`04003ooooo
000:ool00`00ooooo`03ool00`00ooooo`0gool00`00ooooo`08ool00`00ooooo`0Eool00`00oooo
o`2Sool10001ool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0hool2000Qool2
002Sool10001ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0>_oo0P002Ooo
00<00?ooool05Ooo00<00?ooool0X?oo0@000Ooo0008ool2000:ool20005ool00`00ooooo`0lool2
0032ool10001ool001_oo`03003ooooo03koo`8000Ooo`800;Ooo`40007oo`006ooo0P00@Ooo0P00
2?oo00<00?ooool05?oo00<00?ooool0Vooo0P000Ooo000Kool00`00ooooo`12ool2000Nool2002L
ool10001ool001_oo`03003ooooo04Coo`8000Ooo`03003ooooo01Coo`8009[oo`40007oo`006ooo
00<00?ooool0A_oo0P001ooo0P005Ooo00<00?ooool0Uooo0@000Ooo000Kool00`00ooooo`18ool2
002fool10001ool001_oo`03003ooooo04[oo`<000Goo`800:coo`40007oo`006ooo0P00C_oo0`00
1Ooo00<00?ooool04_oo0P00Tooo0P000Ooo000Kool00`00ooooo`1@ool20006ool00`00ooooo`0A
ool2002Bool10001ool001_oo`03003ooooo05;oo`<000?oo`03003ooooo01?oo`80093oo`40007o
o`006ooo00<00?ooool0EOoo0`000ooo00<00?ooool0X_oo0@000Ooo000Kool00`00ooooo`1Hool3
0002ool2000Dool00`00ooooo`2:ool10001ool001_oo`03003ooooo05_oo`@00003ool0000001;o
o`<008Woo`40007oo`006ooo0P00H?oo0`0000?oo`0000004_oo0P00Q_oo0P000Ooo000Kool00`00
ooooo`1Rool300000ooo0000002Hool10001ool001_oo`03003ooooo06Goo`H001;oo`03003ooooo
083oo`40007oo`006ooo00<00?ooool0JOoo1@004?oo0P00P?oo0@000Ooo000Kool00`00ooooo`1]
ool6000=ool3001mool10001ool001_oo`8007?oo`H008Ooo`80007oo`006ooo00<00?ooool0MOoo
0P0000?oo`0000000P002ooo00<00?ooool0M_oo0@000Ooo000Kool00`00ooooo`1kool90004ool3
001eool10001ool001_oo`03003ooooo07koo`800004ool00?ooool9001cool10001ool000Soo`80
00Coo`03003ooooo00?oo`8000Goo`03003ooooo08;oo`04003oo`000006ool6001^ool10001ool0
00Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0Qooo0P0000?oo`0000001Ooo2000
Iooo0@000Ooo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00Sooo00@00?ooool000Ooo`X0
05coo`80007oo`001ooo00@00?ooool000Woo`8000Goo`03003ooooo09Coo`04003ooooo0003ool4
0004ool:001Cool10001ool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`2Jool0
1@00ooooo`00ool00P003?oo5@00?_oo0@000Ooo0007ool01000ooooo`002?oo00@00?ooool000Co
o`03003ooooo0:3oo`800003ool0000000<001[ooaD002Woo`40007oo`002?oo0P002_oo0P001Ooo
00<00?ooool0Yooo100000Coo`00003oo`8002Kooa@001Goo`40007oo`006ooo00<00?ooool0[ooo
1@000_oo00@00?ooool0037ooa0000Goo`40007oo`006ooo0P00]?oo0P001ooo00@00?ooool000;o
o`800003ool0000003Soo`80007oo`006ooo00<00?ooool0^Ooo1P002Ooo0P0000Coo`00003oo`80
00;oo`03003ooooo02_oo`40007oo`006ooo00<00?ooool0`ooo0P004?oo00@00?ooool000;oo`03
003ooooo00800004ool00000ool2000Jool10001ool001_oo`03003ooooo0<Goo`D000Coo`8001Oo
o`800003ool0000000;oo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool00ooo0@000Ooo
000Kool00`00ooooo`3@ool4000/ool10001ool001_oo`800=Woo`H0027oo`80007oo`006ooo00<0
0?ooool0h_oo1P006?oo0@000Ooo000Kool00`00ooooo`3]ool60004ool20007ool10001ool001_o
o`03003ooooo0?Woo`8000Goo`40007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo000Kool00`00
ooooo`03ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00
ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00
ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00
ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00
ooooo`09ool00`00ooooo`03ool10001ool001_oool000D0007oo`00oooo8Ooo003ooolQool00001
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.270385, -0.25198, \
0.00808853, 0.00350813}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<alpha2.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("alpha2.eps"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["alpha+gamma kx=0", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\((1 - xi^2)\)/\((1 + a\ xi^2)\)^2\ \((eta + 
              a\ xi)\)/\((eta - xi + I\ eps)\), {xi, \(-1\), 1}]\)], "Input"],

Cell[BoxData[
    \(\((\(-2\)\ \@a + 2\ a\^\(3/2\)\ eps\^2 + 
          2\ \[ImaginaryI]\ \@a\ eps\ eta - 
          4\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta - 
          2\ \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ eta + 2\ \@a\ eta\^2 - 
          2\ a\^\(3/2\)\ eta\^2 - 6\ a\^\(3/2\)\ eps\^2\ eta\^2 + 
          6\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^3 + 
          2\ a\^\(3/2\)\ eta\^4 - \[ImaginaryI]\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
          3\ \[ImaginaryI]\ a\ eps\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\
\) + \@a)\)] + \[ImaginaryI]\ a\^2\ eps\^2\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
          eps\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] + 
          9\ a\ eps\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] \
+ 2\ a\^2\ eps\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] + 
          a\ eps\^3\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] \
- a\^2\ eps\^3\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - \
\[ImaginaryI]\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - 
          6\ \[ImaginaryI]\ a\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\
\) + \@a)\)] - \[ImaginaryI]\ a\^2\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - 
          3\ \[ImaginaryI]\ a\ eps\^2\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
          3\ \[ImaginaryI]\ a\^2\ eps\^2\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - 
          3\ a\ eps\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] + 
          3\ a\^2\ eps\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] + \[ImaginaryI]\ a\ eta\^4\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) \
+ \@a)\)] - \[ImaginaryI]\ a\^2\ eta\^4\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - \[ImaginaryI]\ a\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
          3\ \[ImaginaryI]\ a\ eps\^2\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - \[ImaginaryI]\ a\^2\ eps\^2\ Log[\(-\[ImaginaryI]\)\
\ \((\[ImaginaryI] + \@a)\)] - 
          eps\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - 
          9\ a\ eps\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] \
- 2\ a\^2\ eps\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - 
          a\ eps\^3\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] \
+ a\^2\ eps\^3\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + \
\[ImaginaryI]\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + 
          6\ \[ImaginaryI]\ a\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] + \[ImaginaryI]\ a\^2\ eta\^2\ Log[\(-\[ImaginaryI]\)\
\ \((\[ImaginaryI] + \@a)\)] + 
          3\ \[ImaginaryI]\ a\ eps\^2\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
          3\ \[ImaginaryI]\ a\^2\ eps\^2\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] + 
          3\ a\ eps\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] - 
          3\ a\^2\ eps\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] - \[ImaginaryI]\ a\ eta\^4\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] \
+ \@a)\)] + \[ImaginaryI]\ a\^2\ eta\^4\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] + \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + \[ImaginaryI]\ a\^\(3/2\)\ \
eps\^3\ Log[\(\(-1\) - \[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + \@a\ \
eta\ Log[\(\(-1\) - \[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          a\^\(3/2\)\ eta\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + \@a\ eps\^2\ eta\ Log[\(\(-1\) - \[ImaginaryI]\ \
eps + eta\)\/\(\(-1\) + eta\)] + 
          3\ a\^\(3/2\)\ eps\^2\ eta\ Log[\(\(-1\) - \[ImaginaryI]\ eps + eta\
\)\/\(\(-1\) + eta\)] - 
          2\ \[ImaginaryI]\ \@a\ eps\ eta\^2\ Log[\(\(-1\) - \[ImaginaryI]\ \
eps + eta\)\/\(\(-1\) + eta\)] - 
          3\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^2\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - \@a\ eta\^3\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          a\^\(3/2\)\ eta\^3\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[\(1 - \
\[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] - \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ \
Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] - \@a\ eta\ Log[\(1 - \
\[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] - 
          a\^\(3/2\)\ eta\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] \
- \@a\ eps\^2\ eta\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] - 
          3\ a\^\(3/2\)\ eps\^2\ eta\ Log[\(1 - \[ImaginaryI]\ eps + \
eta\)\/\(1 + eta\)] + 
          2\ \[ImaginaryI]\ \@a\ eps\ eta\^2\ Log[\(1 - \[ImaginaryI]\ eps + \
eta\)\/\(1 + eta\)] + 
          3\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^2\ Log[\(1 - \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] + \@a\ eta\^3\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\
\/\(1 + eta\)] + 
          a\^\(3/2\)\ eta\^3\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + eta\
\)] - \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[\(\(-1\) + \[ImaginaryI]\ eps + eta\
\)\/\(\(-1\) + eta\)] - \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - \@a\ eta\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          a\^\(3/2\)\ eta\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - \@a\ eps\^2\ eta\ Log[\(\(-1\) + \[ImaginaryI]\ \
eps + eta\)\/\(\(-1\) + eta\)] - 
          3\ a\^\(3/2\)\ eps\^2\ eta\ Log[\(\(-1\) + \[ImaginaryI]\ eps + eta\
\)\/\(\(-1\) + eta\)] + 
          2\ \[ImaginaryI]\ \@a\ eps\ eta\^2\ Log[\(\(-1\) + \[ImaginaryI]\ \
eps + eta\)\/\(\(-1\) + eta\)] + 
          3\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^2\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + \@a\ eta\^3\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          a\^\(3/2\)\ eta\^3\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[\(1 + \
\[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] + \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ \
Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] + \@a\ eta\ Log[\(1 + \
\[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] + 
          a\^\(3/2\)\ eta\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] \
+ \@a\ eps\^2\ eta\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] + 
          3\ a\^\(3/2\)\ eps\^2\ eta\ Log[\(1 + \[ImaginaryI]\ eps + \
eta\)\/\(1 + eta\)] - 
          2\ \[ImaginaryI]\ \@a\ eps\ eta\^2\ Log[\(1 + \[ImaginaryI]\ eps + \
eta\)\/\(1 + eta\)] - 
          3\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^2\ Log[\(1 + \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] - \@a\ eta\^3\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\
\/\(1 + eta\)] - 
          a\^\(3/2\)\ eta\^3\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + eta\
\)] - \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[
              1 + eps\^2 - 2\ eta + 
                eta\^2] - \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - \@a\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - 
          a\^\(3/2\)\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - \@a\ eps\^2\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - 
          3\ a\^\(3/2\)\ eps\^2\ eta\ Log[1 + eps\^2 - 2\ eta + eta\^2] + 
          2\ \[ImaginaryI]\ \@a\ eps\ eta\^2\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] + 
          3\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^2\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] + \@a\ eta\^3\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] + 
          a\^\(3/2\)\ eta\^3\ Log[
              1 + eps\^2 - 2\ eta + 
                eta\^2] + \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[
              1 + eps\^2 + 2\ eta + 
                eta\^2] + \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + \@a\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + 
          a\^\(3/2\)\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + \@a\ eps\^2\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + 
          3\ a\^\(3/2\)\ eps\^2\ eta\ Log[1 + eps\^2 + 2\ eta + eta\^2] - 
          2\ \[ImaginaryI]\ \@a\ eps\ eta\^2\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] - 
          3\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^2\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] - \@a\ eta\^3\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] - 
          a\^\(3/2\)\ eta\^3\ Log[
              1 + eps\^2 + 2\ eta + 
                eta\^2])\)/\((2\ \@a\ \((\(-1\) + a\ eps\^2 - 2\ \
\[ImaginaryI]\ a\ eps\ eta - a\ eta\^2)\)\^2)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Limit[%, eps \[Rule] 0]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(2\ \@a\ \((\(-1\) - a\ eta\^2)\)\^2\)\)\((\(-2\)\ \@a + 
        2\ \@a\ eta\^2 - 2\ a\^\(3/2\)\ eta\^2 + 
        2\ a\^\(3/2\)\ eta\^4 - \[ImaginaryI]\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - \[ImaginaryI]\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - 
        6\ \[ImaginaryI]\ a\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) \
+ \@a)\)] - \[ImaginaryI]\ a\^2\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\ eta\^4\ Log[\[ImaginaryI]\ \
\((\(-\[ImaginaryI]\) + \@a)\)] - \[ImaginaryI]\ a\^2\ eta\^4\ Log[\
\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ Log[\(-\
\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - \[ImaginaryI]\ a\ Log[\(-\
\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + \[ImaginaryI]\ eta\^2\ Log[\(-\
\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + 
        6\ \[ImaginaryI]\ a\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] \
+ \@a)\)] + \[ImaginaryI]\ a\^2\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - \[ImaginaryI]\ a\ eta\^4\ Log[\(-\[ImaginaryI]\)\ \
\((\[ImaginaryI] + \@a)\)] + \[ImaginaryI]\ a\^2\ eta\^4\ \
Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - \@a\ eta\ Log[
            1 - 2\ eta + eta\^2] - 
        a\^\(3/2\)\ eta\ Log[1 - 2\ eta + eta\^2] + \@a\ eta\^3\ Log[
            1 - 2\ eta + eta\^2] + 
        a\^\(3/2\)\ eta\^3\ Log[1 - 2\ eta + eta\^2] + \@a\ eta\ Log[
            1 + 2\ eta + eta\^2] + 
        a\^\(3/2\)\ eta\ Log[1 + 2\ eta + eta\^2] - \@a\ eta\^3\ Log[
            1 + 2\ eta + eta\^2] - 
        a\^\(3/2\)\ eta\^3\ Log[1 + 2\ eta + eta\^2])\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0[eta_, a_] = 
      1/4\ \((1 + a)\)^\((1/2)\) FullSimplify[%]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\) \((\[ImaginaryI]\ \@\(1 \
+ a\)\ \((\(-\((1 + eta\^2 + 
                        a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                    a)\)\ eta\^4)\))\)\)\ Log[
                    1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                      a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                  a)\)\ eta\^4)\))\)\ Log[
                    1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) + 
                      eta\^2)\)\ \((2 + 
                      2\ a\ eta\^2 + \((1 + 
                            a)\)\ eta\ \((Log[\((\(-1\) + eta)\)\^2] - 
                            Log[\((1 + eta)\)\^2])\))\))\))\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0[eta_, 
        a_] = \(-\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\) \((\[ImaginaryI]\
\ \@\(1 + a\)\ \((\(-\((1 + eta\^2 + 
                        a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                    a)\)\ eta\^4)\))\)\)\ Log[
                    1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                      a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                  a)\)\ eta\^4)\))\)\ Log[
                    1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) + 
                      eta\^2)\)\ \((2 + 
                      2\ a\ eta\^2 + \((1 + 
                            a)\)\ eta\ \((Log[\((\(-1\) + eta)\)\^2] - 
                            Log[\((1 + eta)\)\^2])\))\))\))\)\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\[ImaginaryI]\ \@\(1 + \
a\)\ \((\((\(-1\) - eta\^2 - 
                    a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                a)\)\ eta\^4)\))\)\ Log[
                  1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                    a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                a)\)\ eta\^4)\))\)\ Log[
                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) + 
                    eta\^2)\)\ \((2 + 
                    2\ a\ eta\^2 + \((1 + 
                          a)\)\ eta\ \((Log[\((\(-1\) + eta)\)\^2] - 
                          Log[\((1 + eta)\)\^2])\))\))\))\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0[0.3, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.6010357168904471`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0korr[eta_, 
        a_] = \(-\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\[ImaginaryI]\ \
\@\(1 + a\)\ \((\(-\((1 + eta\^2 + 
                          a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                      a)\)\ eta\^4)\))\)\)\ Log[
                      1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                        a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                    a)\)\ eta\^4)\))\)\ Log[
                      1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) \
+ eta\^2)\)\ \((2 + 2\ a\ eta\^2 + \((1 + 
                              a)\)\ eta\ \((Log[\((\(-1\) + eta)\)\^2] - 
                              Log[\((1 + eta)\)\^2])\))\))\))\)\)\) - \(\
\[ImaginaryI]\ \((1 + a)\)\^\(3/2\)\ eta\ \((1 - eta\^2)\)\ \[Pi]\ UnitStep[1 \
- Re[eta]]\)\/\(4\ \((1 + a\ eta\^2)\)\^2\)\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\[ImaginaryI]\ \@\(1 + \
a\)\ \((\((\(-1\) - eta\^2 - 
                      a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                  a)\)\ eta\^4)\))\)\ Log[
                    1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                      a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                  a)\)\ eta\^4)\))\)\ Log[
                    1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) + 
                      eta\^2)\)\ \((2 + 
                      2\ a\ eta\^2 + \((1 + 
                            a)\)\ eta\ \((Log[\((\(-1\) + eta)\)\^2] - 
                            Log[\((1 + eta)\)\^2])\))\))\))\)\)\) - \(\
\[ImaginaryI]\ \((1 + a)\)\^\(3/2\)\ eta\ \((1 - eta\^2)\)\ \[Pi]\ UnitStep[1 \
- Re[eta]]\)\/\(4\ \((1 + a\ eta\^2)\)\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0korr[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.0512681376293807`\)\(\[InvisibleSpace]\)\) - 
      1.4242016695726538`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0korr[0.4 + 0.001\ I, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.051285973191288`\)\(\[InvisibleSpace]\)\) + 
      0.0020826519455613046`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(modifying\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0korr[eta_, 
        a_] = \(-\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\) \((\[ImaginaryI]\
\ \@\(1 + a\)\ \((\(-\((1 + eta\^2 + 
                        a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                    a)\)\ eta\^4)\))\)\)\ Log[
                    1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                      a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                  a)\)\ eta\^4)\))\)\ Log[
                    1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) + 
                      eta\^2)\)\ \((2 + 
                      2\ a\ eta\^2 + \((1 + 
                            a)\)\ eta\ 2\ \((Log[\((\(-1\) + eta)\)] - 
                            Log[\((1 + eta)\)])\))\))\))\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(-\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\[ImaginaryI]\ \@\(1 + \
a\)\ \((\((\(-1\) - eta\^2 - 
                    a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                a)\)\ eta\^4)\))\)\ Log[
                  1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                    a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                a)\)\ eta\^4)\))\)\ Log[
                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) + 
                    eta\^2)\)\ \((2 + 2\ a\ eta\^2 + 
                    2\ \((1 + a)\)\ eta\ \((Log[\(-1\) + eta] - 
                          Log[1 + eta])\))\))\))\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0korr[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.0512681376293807`\)\(\[InvisibleSpace]\)\) - 
      1.4242016695726543`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0korr[0.4 + 0.001\ I, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.044725768971399`\)\(\[InvisibleSpace]\)\) - 
      1.4221081724857842`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSalphakx0korr[eta_, 
        a_] = \(-\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\) \((\[ImaginaryI]\
\ \@\(1 + a\)\ \((\(-\((1 + eta\^2 + 
                        a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                    a)\)\ eta\^4)\))\)\)\ Log[
                    1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                      a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                  a)\)\ eta\^4)\))\)\ Log[
                    1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) + 
                      eta\^2)\)\ \((2 + 
                      2\ a\ eta\^2 + \((1 + 
                            a)\)\ eta\ 2\ \((Log[\((\(-1\) + eta)\)] - 
                            Log[\((1 + eta)\)] + 
                            2\ Pi\ I\ If[\((Im[eta] < 0)\) && \((Re[eta] < 
                                      1)\), 1, 0])\))\))\))\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(-\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\[ImaginaryI]\ \@\(1 + \
a\)\ \((\((\(-1\) - eta\^2 - 
                    a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                a)\)\ eta\^4)\))\)\ Log[
                  1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                    a\ \((\(-1\) + \((6 + a)\)\ eta\^2 + \((\(-1\) + 
                                a)\)\ eta\^4)\))\)\ Log[
                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((\(-1\) + 
                    eta\^2)\)\ \((2 + 2\ a\ eta\^2 + 
                    2\ \((1 + 
                          a)\)\ eta\ \((2\ \[ImaginaryI]\ \[Pi]\ If[
                              Im[eta] < 0 && Re[eta] < 1, 1, 0] + 
                          Log[\(-1\) + eta] - 
                          Log[1 + eta])\))\))\))\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSalphakx0korr[0.4 - 0.001\ I, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.0578461774111771`\)\(\[InvisibleSpace]\)\) - 
      1.426273476376907`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NIntegrate[\(\(Sqrt[1 + a]/
                4 \((1 - xi^2)\)/\((1 + a\ xi^2)\)^2\ \((eta + 
                    a\ xi)\)/\((eta - xi + I\ eps)\) /. 
            eps \[Rule] 10^\((\(-10\))\)\) /. a \[Rule] 10\) /. 
        eta \[Rule] 0.4, {xi, \(-1\), \(-0.5\), 0.1, 0.399, 0.4, 0.401, 
        1}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.40000000011312065`\)."\
\)], "Message"],

Cell[BoxData[
    \(\(\(1.051268136492134`\)\(\[InvisibleSpace]\)\) - 
      1.4242016709780079`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRealpha[1, 1, 1 - 10^\((\(-4\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(1.0506064907138946`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRegamma[1, 1, 1 - 10^\((\(-6\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(-0.000032104998685727394`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.0512681376293807`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImalpha[1, 1, 1 - 10^\((\(-4\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(32\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(32\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(32\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(-1.4246952005650197`\) - 
      2.3457600923356023`*^-43\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImgamma[1, 1, 1 - 10^\((\(-5\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3958972299406299`\)."\)], "Message"],

Cell[BoxData[
    \(\(-0.00007351515545636133`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NIntegrate[\(\(eta^2\ \((1 + a)\)^\((3/2)\)/
                2 \( xi/\((1 + a\ xi^2)\)^2\)\ /\((eta - xi + I\ eps)\) /. 
            eps \[Rule] 10^\((\(-10\))\)\) /. a \[Rule] 10\) /. 
        eta \[Rule] 0.4, {xi, \(-1\), 0, 0.4, 1}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3999999992222327`\)."\)], "Message"],

Cell[BoxData[
    \(\(\(0.14786908419641076`\)\(\[InvisibleSpace]\)\) - 
      0.5425509355174426`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.14786621025190982`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(aha!\)\)], "Input"],

Cell[BoxData[
    \(also\ numerische\ Integration\ sollte\ schon\ \(\(stimmen\)\(.\)\)\)], \
"Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[alphakx0korr[eta, 10.0]], {eta, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.402135 0.178288 [
[.15079 .38963 -9 -9 ]
[.15079 .38963 9 0 ]
[.27778 .38963 -9 -9 ]
[.27778 .38963 9 0 ]
[.40476 .38963 -9 -9 ]
[.40476 .38963 9 0 ]
[.53175 .38963 -9 -9 ]
[.53175 .38963 9 0 ]
[.65873 .38963 -3 -9 ]
[.65873 .38963 3 0 ]
[.78571 .38963 -9 -9 ]
[.78571 .38963 9 0 ]
[.9127 .38963 -9 -9 ]
[.9127 .38963 9 0 ]
[.01131 .04556 -12 -4.5 ]
[.01131 .04556 0 4.5 ]
[.01131 .1347 -24 -4.5 ]
[.01131 .1347 0 4.5 ]
[.01131 .22385 -12 -4.5 ]
[.01131 .22385 0 4.5 ]
[.01131 .31299 -24 -4.5 ]
[.01131 .31299 0 4.5 ]
[.01131 .49128 -18 -4.5 ]
[.01131 .49128 0 4.5 ]
[.01131 .58042 -6 -4.5 ]
[.01131 .58042 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .40213 m
.15079 .40838 L
s
[(0.2)] .15079 .38963 0 1 Mshowa
.27778 .40213 m
.27778 .40838 L
s
[(0.4)] .27778 .38963 0 1 Mshowa
.40476 .40213 m
.40476 .40838 L
s
[(0.6)] .40476 .38963 0 1 Mshowa
.53175 .40213 m
.53175 .40838 L
s
[(0.8)] .53175 .38963 0 1 Mshowa
.65873 .40213 m
.65873 .40838 L
s
[(1)] .65873 .38963 0 1 Mshowa
.78571 .40213 m
.78571 .40838 L
s
[(1.2)] .78571 .38963 0 1 Mshowa
.9127 .40213 m
.9127 .40838 L
s
[(1.4)] .9127 .38963 0 1 Mshowa
.125 Mabswid
.05556 .40213 m
.05556 .40588 L
s
.0873 .40213 m
.0873 .40588 L
s
.11905 .40213 m
.11905 .40588 L
s
.18254 .40213 m
.18254 .40588 L
s
.21429 .40213 m
.21429 .40588 L
s
.24603 .40213 m
.24603 .40588 L
s
.30952 .40213 m
.30952 .40588 L
s
.34127 .40213 m
.34127 .40588 L
s
.37302 .40213 m
.37302 .40588 L
s
.43651 .40213 m
.43651 .40588 L
s
.46825 .40213 m
.46825 .40588 L
s
.5 .40213 m
.5 .40588 L
s
.56349 .40213 m
.56349 .40588 L
s
.59524 .40213 m
.59524 .40588 L
s
.62698 .40213 m
.62698 .40588 L
s
.69048 .40213 m
.69048 .40588 L
s
.72222 .40213 m
.72222 .40588 L
s
.75397 .40213 m
.75397 .40588 L
s
.81746 .40213 m
.81746 .40588 L
s
.84921 .40213 m
.84921 .40588 L
s
.88095 .40213 m
.88095 .40588 L
s
.94444 .40213 m
.94444 .40588 L
s
.97619 .40213 m
.97619 .40588 L
s
.25 Mabswid
0 .40213 m
1 .40213 L
s
.02381 .04556 m
.03006 .04556 L
s
[(-2)] .01131 .04556 1 0 Mshowa
.02381 .1347 m
.03006 .1347 L
s
[(-1.5)] .01131 .1347 1 0 Mshowa
.02381 .22385 m
.03006 .22385 L
s
[(-1)] .01131 .22385 1 0 Mshowa
.02381 .31299 m
.03006 .31299 L
s
[(-0.5)] .01131 .31299 1 0 Mshowa
.02381 .49128 m
.03006 .49128 L
s
[(0.5)] .01131 .49128 1 0 Mshowa
.02381 .58042 m
.03006 .58042 L
s
[(1)] .01131 .58042 1 0 Mshowa
.125 Mabswid
.02381 .06339 m
.02756 .06339 L
s
.02381 .08122 m
.02756 .08122 L
s
.02381 .09904 m
.02756 .09904 L
s
.02381 .11687 m
.02756 .11687 L
s
.02381 .15253 m
.02756 .15253 L
s
.02381 .17036 m
.02756 .17036 L
s
.02381 .18819 m
.02756 .18819 L
s
.02381 .20602 m
.02756 .20602 L
s
.02381 .24168 m
.02756 .24168 L
s
.02381 .2595 m
.02756 .2595 L
s
.02381 .27733 m
.02756 .27733 L
s
.02381 .29516 m
.02756 .29516 L
s
.02381 .33082 m
.02756 .33082 L
s
.02381 .34865 m
.02756 .34865 L
s
.02381 .36648 m
.02756 .36648 L
s
.02381 .38431 m
.02756 .38431 L
s
.02381 .41996 m
.02756 .41996 L
s
.02381 .43779 m
.02756 .43779 L
s
.02381 .45562 m
.02756 .45562 L
s
.02381 .47345 m
.02756 .47345 L
s
.02381 .50911 m
.02756 .50911 L
s
.02381 .52694 m
.02756 .52694 L
s
.02381 .54477 m
.02756 .54477 L
s
.02381 .56259 m
.02756 .56259 L
s
.02381 .02773 m
.02756 .02773 L
s
.02381 .0099 m
.02756 .0099 L
s
.02381 .59825 m
.02756 .59825 L
s
.02381 .61608 m
.02756 .61608 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.09769 0 m
.11039 .07114 L
.14855 .28165 L
.16787 .3678 L
.18914 .44367 L
.20932 .49823 L
.22823 .53598 L
.23854 .55193 L
.24834 .56447 L
.26674 .5821 L
.27627 .58866 L
.28642 .59405 L
.296 .59782 L
.30468 .60028 L
.30952 .6013 L
.31464 .60213 L
.31901 .60265 L
.32149 .60288 L
.3238 .60305 L
.32509 .60312 L
.32648 .60319 L
.3278 .60324 L
.32902 .60327 L
.33012 .6033 L
.33133 .60331 L
.33259 .60332 L
.33378 .60331 L
.33509 .6033 L
.33575 .60329 L
.33647 .60327 L
.33774 .60324 L
.33893 .60319 L
.34162 .60307 L
.34448 .60289 L
.34978 .60245 L
.35466 .60192 L
.36569 .60035 L
.3857 .59645 L
.42484 .58639 L
.46248 .57539 L
.50255 .56348 L
.54111 .5524 L
.58212 .54122 L
.62161 .53088 L
.6596 .52 L
.68082 .51446 L
.70003 .51076 L
.73895 .50495 L
.7788 .50042 L
.81715 .49695 L
Mistroke
.85793 .49393 L
.8972 .49151 L
.93497 .48954 L
.97518 .48774 L
.97619 .48769 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool04?oo00<00?ooool0
kooo000Kool00`00ooooo`0@ool00`00ooooo`3_ool001_oo`80017oo`03003ooooo0>ooo`006ooo
00<00?ooool04Ooo00<00?ooool0k_oo000Kool00`00ooooo`0Aool00`00ooooo`3^ool001_oo`03
003ooooo017oo`03003ooooo0>koo`006ooo00<00?ooool04Ooo00<00?ooool0k_oo000Kool2000B
ool00`00ooooo`3^ool001_oo`03003ooooo01;oo`03003ooooo0>goo`006ooo00<00?ooool04_oo
00<00?ooool0kOoo000Cool40004ool00`00ooooo`0Bool00`00ooooo`3]ool001?oo`03003ooooo
00Goo`03003ooooo01;oo`03003ooooo0>goo`005?oo00<00?ooool01?oo0P005?oo00<00?ooool0
k?oo000=ool40004ool00`00ooooo`03ool00`00ooooo`0Cool00`00ooooo`3/ool001Koo`03003o
oooo00;oo`03003ooooo01?oo`03003ooooo0>coo`004ooo00@00?ooool000Coo`03003ooooo01?o
o`03003ooooo0>coo`005?oo0P001Ooo00<00?ooool04ooo00<00?ooool0k?oo000Kool2000Eool0
0`00ooooo`3[ool001_oo`03003ooooo01Coo`03003ooooo0>_oo`006ooo00<00?ooool05?oo00<0
0?ooool0jooo000Kool00`00ooooo`0Dool00`00ooooo`3[ool001_oo`8001Goo`03003ooooo0>_o
o`006ooo00<00?ooool05Ooo00<00?ooool0j_oo000Kool00`00ooooo`0Eool00`00ooooo`3Zool0
01_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo00<00?ooool05Ooo00<00?ooool0j_oo000K
ool2000Fool00`00ooooo`3Zool001_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo00<00?oo
ool05_oo00<00?ooool0jOoo000Kool00`00ooooo`0Fool00`00ooooo`3Yool001_oo`03003ooooo
01Koo`03003ooooo0>Woo`006ooo0P005ooo00<00?ooool0jOoo000Kool00`00ooooo`0Fool00`00
ooooo`3Yool001_oo`03003ooooo01Koo`03003ooooo0>Woo`001ooo10000ooo00<00?ooool00_oo
0`001Ooo00<00?ooool05ooo00<00?ooool0j?oo0009ool00`00ooooo`0:ool00`00ooooo`02ool0
0`00ooooo`0Gool00`00ooooo`3Xool000Woo`03003ooooo00[oo`03003ooooo00;oo`8001Soo`03
003ooooo0>Soo`0000?oo`0000000P001?oo00<00?ooool01ooo0`001Ooo00<00?ooool05ooo00<0
0?ooool0j?oo0009ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`0Gool00`00ooooo`3X
ool000Ooo`<000Woo`03003ooooo00Goo`03003ooooo01Soo`03003ooooo0>Ooo`002Ooo00<00?oo
ool01ooo10001?oo0P006Ooo00<00?ooool0iooo000Kool00`00ooooo`0Hool00`00ooooo`3Wool0
01_oo`03003ooooo01Soo`03003ooooo0>Ooo`006ooo00<00?ooool06?oo00<00?ooool0iooo000K
ool00`00ooooo`0Hool00`00ooooo`3Wool001_oo`8001[oo`03003ooooo0>Koo`006ooo00<00?oo
ool06Ooo00<00?ooool0i_oo000Kool00`00ooooo`0Iool00`00ooooo`3Vool001_oo`03003ooooo
01Woo`03003ooooo0>Koo`006ooo00<00?ooool06Ooo00<00?ooool0i_oo000Kool2000Kool00`00
ooooo`3Uool001_oo`03003ooooo01[oo`03003ooooo0>Goo`006ooo00<00?ooool06_oo00<00?oo
ool0iOoo000Kool00`00ooooo`0Jool00`00ooooo`3Uool001_oo`03003ooooo01[oo`03003ooooo
0>Goo`006ooo0P006ooo00<00?ooool0iOoo000Kool00`00ooooo`0Kool00`00ooooo`3Tool001?o
o`@000Coo`03003ooooo01_oo`03003ooooo0>Coo`005Ooo00<00?ooool00ooo00<00?ooool06ooo
00<00?ooool0i?oo000Eool00`00ooooo`03ool2000Lool00`00ooooo`3Tool000goo`@000Coo`03
003ooooo00?oo`03003ooooo01_oo`03003ooooo0>Coo`005Ooo00<00?ooool00ooo00<00?ooool0
6ooo00<00?ooool0i?oo000Cool30005ool00`00ooooo`0Lool00`00ooooo`3Sool001Goo`03003o
oooo00?oo`03003ooooo01coo`03003ooooo0>?oo`006ooo0P007Ooo00<00?ooool0hooo000Kool0
0`00ooooo`0Lool00`00ooooo`3Sool001_oo`03003ooooo01coo`03003ooooo0>?oo`006ooo00<0
0?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Mool00`00ooooo`3Rool001_oo`8001ko
o`03003ooooo0>;oo`006ooo00<00?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Mool0
0`00ooooo`3Rool001_oo`03003ooooo01goo`03003ooooo0>;oo`006ooo00<00?ooool07_oo00<0
0?ooool0hOoo000Kool2000Oool00`00ooooo`3Qool001_oo`03003ooooo01koo`03003ooooo0>7o
o`006ooo00<00?ooool07_oo00<00?ooool0hOoo000Kool00`00ooooo`0Nool00`00ooooo`3Qool0
01_oo`80023oo`03003ooooo0>3oo`006ooo00<00?ooool07ooo00<00?ooool0h?oo000Kool00`00
ooooo`0Oool00`00ooooo`3Pool000Soo`8000Coo`03003ooooo00;oo`<000Goo`03003ooooo01oo
o`03003ooooo0>3oo`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo023oo`03003o
oooo0=ooo`001ooo00@00?ooool000_oo`03003ooooo00;oo`80027oo`03003ooooo0=ooo`0000?o
o`0000000P000_oo00@00?ooool000Soo`<000Goo`03003ooooo023oo`03003ooooo0=ooo`001ooo
00@00?ooool000Soo`03003ooooo00Goo`03003ooooo023oo`03003ooooo0=ooo`001ooo00@00?oo
ool000Soo`03003ooooo00Goo`03003ooooo023oo`03003ooooo0=ooo`002?oo0P002Ooo10001?oo
00<00?ooool08Ooo00<00?ooool0g_oo000Kool2000Rool00`00ooooo`3Nool001_oo`03003ooooo
027oo`03003ooooo0=koo`006ooo00<00?ooool08Ooo00<00?ooool0g_oo000Kool00`00ooooo`0Q
ool00`00ooooo`3Nool001_oo`03003ooooo02;oo`03003ooooo0=goo`006ooo0P008ooo00<00?oo
ool0gOoo000Kool00`00ooooo`0Rool00`00ooooo`3Mool001_oo`03003ooooo02;oo`03003ooooo
0=goo`006ooo00<00?ooool06?oo0P001?oo00<00?ooool00_oo10004_oo0P001?oo00<00?ooool0
1?oo00<00?ooool04_oo0P001?oo00<00?ooool00ooo0P005?oo0P001?oo00<00?ooool00ooo0P00
6Ooo10005ooo10000ooo00<00?ooool00_oo10004_oo10000ooo00<00?ooool01?oo00<00?ooool0
4?oo000Kool2000Hool01000ooooo`002?oo00<00?ooool04_oo00@00?ooool000[oo`03003ooooo
017oo`04003ooooo0008ool01000ooooo`004_oo00@00?ooool000Soo`04003ooooo000Jool00`00
ooooo`0Hool00`00ooooo`07ool00`00ooooo`0Eool00`00ooooo`09ool00`00ooooo`0@ool001_o
o`03003ooooo01Ooo`04003ooooo0008ool2000Cool01000ooooo`001ooo1@004_oo00@00?ooool0
00Soo`04003ooooo000Bool01000ooooo`002?oo00@00?ooool001[oo`03003ooooo01Soo`03003o
oooo00Soo`03003ooooo01Coo`03003ooooo00Koo`D0017oo`006ooo00<00?ooool05ooo00@00?oo
ool000Soo`03003oo`0001;oo`04003ooooo0007ool01000ooooo`004ooo00@00?ooool000Soo`<0
01?oo`04003ooooo0009ool2000Kool00`00ooooo`0Hool00`00ooooo`09ool00`00ooooo`0Cool0
0`00ooooo`06ool01000ooooo`004_oo000Kool00`00ooooo`0Gool01000ooooo`002Ooo00<00?oo
00004Ooo00@00?ooool000Soo`03003oo`0001?oo`04003ooooo0008ool00`00ooooo`0Cool01000
ooooo`002?oo00@00?ooool001[oo`03003ooooo01Soo`03003ooooo00[oo`03003ooooo01;oo`03
003ooooo00Ooo`03003oo`0001;oo`006ooo00<00?ooool05ooo00@00?ooool000Soo`800003ool0
0?oo013oo`04003ooooo0009ool2000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool0
00Soo`04003ooooo000Hool3000Hool30009ool01000ooooo`004_oo0`002_oo0P004_oo000Kool2
000Iool2000:ool2000Cool2000;ool00`00ooooo`0Bool2000:ool3000Cool2000:ool2000Kool0
0`00ooooo`0Hool00`00ooooo`08ool2000Eool00`00ooooo`09ool00`00ooooo`0@ool001_oo`03
003ooooo02Coo`03003ooooo0=_oo`006ooo00<00?ooool09Ooo00<00?ooool0f_oo000Kool00`00
ooooo`0Uool00`00ooooo`3Jool001_oo`03003ooooo02Goo`03003ooooo0=[oo`005?ooo`003@00
000Kool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`04ool20007ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`05ool001_oo`03003ooooo02Koo`03003ooooo0=Woo`006ooo00<00?ooool09ooo00<0
0?ooool0f?oo000Kool00`00ooooo`0Wool00`00ooooo`3Hool001_oo`8002Soo`03003ooooo0=So
o`006ooo00<00?ooool09ooo00<00?ooool0f?oo000Kool00`00ooooo`0Xool00`00ooooo`3Gool0
01_oo`03003ooooo02Soo`03003ooooo0=Ooo`006ooo0P00:Ooo00<00?ooool0eooo000Kool00`00
ooooo`0Yool00`00ooooo`3Fool001_oo`03003ooooo02Woo`03003ooooo0=Koo`006ooo00<00?oo
ool0:Ooo00<00?ooool0e_oo000Kool00`00ooooo`0Zool00`00ooooo`3Eool001_oo`8002_oo`03
003ooooo0=Goo`006ooo00<00?ooool0:_oo00<00?ooool0eOoo000Kool00`00ooooo`0[ool00`00
ooooo`3Dool001_oo`03003ooooo02_oo`03003ooooo0=Coo`006ooo00<00?ooool0:ooo00<00?oo
ool0e?oo000Kool2000]ool00`00ooooo`3Cool001_oo`03003ooooo02coo`03003ooooo0=?oo`00
6ooo00<00?ooool0;Ooo00<00?ooool0d_oo0008ool20004ool00`00ooooo`02ool30005ool00`00
ooooo`0]ool00`00ooooo`3Bool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0]
ool00`00ooooo`2joolA0007ool000Ooo`04003ooooo000;ool00`00ooooo`02ool2000_ool00`00
ooooo`2UoolD000Hool000Ooo`04003ooooo0008ool30005ool00`00ooooo`0^ool00`00ooooo`2J
ool;000/ool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0^ool00`00ooooo`2@
ool:000gool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0_ool00`00ooooo`27
ool80011ool000Soo`8000Woo`@000Coo`80033oo`03003ooooo08;oo`D004Woo`006ooo00<00?oo
ool0<?oo00<00?ooool0O?oo1@00C_oo000Kool00`00ooooo`0`ool00`00ooooo`1fool6001Cool0
01_oo`03003ooooo037oo`03003ooooo077oo`@005Woo`006ooo00<00?ooool0<Ooo00<00?ooool0
KOoo1000GOoo000Kool2000cool00`00ooooo`1Yool3001Qool001_oo`03003ooooo03;oo`03003o
oooo06Koo`<006Coo`006ooo00<00?ooool0<ooo00<00?ooool0HOoo1000Iooo000Kool00`00oooo
o`0dool00`00ooooo`1Lool4001[ool001_oo`03003ooooo03Coo`03003ooooo05Woo`<006ooo`00
6ooo0P00=_oo00<00?ooool0E?oo1000L_oo000Kool00`00ooooo`0eool00`00ooooo`1@ool4001f
ool001_oo`03003ooooo03Koo`03003ooooo04coo`<007[oo`006ooo00<00?ooool0=ooo00<00?oo
ool0Aooo1000OOoo000Kool2000iool00`00ooooo`13ool30021ool001_oo`03003ooooo03Woo`03
003ooooo03ooo`<008Coo`006ooo00<00?ooool0>_oo00<00?ooool0>_oo1000Qooo000Cool40004
ool00`00ooooo`0kool00`00ooooo`0fool3002;ool001Goo`03003ooooo00?oo`03003ooooo03_o
o`03003ooooo03?oo`<008koo`005Ooo00<00?ooool00ooo0P00?Ooo00<00?ooool0;_oo1000TOoo
000Eool00`00ooooo`03ool00`00ooooo`0mool2000[ool3002Eool001Goo`03003ooooo00?oo`03
003ooooo03ooo`8002Goo`@009Soo`004ooo0`001Ooo00<00?ooool0@Ooo0P007Ooo1P00W?oo000E
ool00`00ooooo`03ool00`00ooooo`13ool00`00ooooo`0Fool4002Rool001_oo`8004Goo`<001?o
o`800:Koo`006ooo00<00?ooool0Aooo4`00Z?oo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, -2.38772, \
0.00592092, 0.0210856}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Im[alphakx0korr[eta, 10.0]], {eta, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.603319 0.206619 [
[.15079 .59082 -9 -9 ]
[.15079 .59082 9 0 ]
[.27778 .59082 -9 -9 ]
[.27778 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.53175 .59082 -9 -9 ]
[.53175 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.9127 .59082 -9 -9 ]
[.9127 .59082 9 0 ]
[.01131 .08677 -24 -4.5 ]
[.01131 .08677 0 4.5 ]
[.01131 .19008 -12 -4.5 ]
[.01131 .19008 0 4.5 ]
[.01131 .29339 -24 -4.5 ]
[.01131 .29339 0 4.5 ]
[.01131 .3967 -12 -4.5 ]
[.01131 .3967 0 4.5 ]
[.01131 .50001 -24 -4.5 ]
[.01131 .50001 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .60332 m
.15079 .60957 L
s
[(0.2)] .15079 .59082 0 1 Mshowa
.27778 .60332 m
.27778 .60957 L
s
[(0.4)] .27778 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(0.6)] .40476 .59082 0 1 Mshowa
.53175 .60332 m
.53175 .60957 L
s
[(0.8)] .53175 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(1)] .65873 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(1.2)] .78571 .59082 0 1 Mshowa
.9127 .60332 m
.9127 .60957 L
s
[(1.4)] .9127 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.18254 .60332 m
.18254 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.34127 .60332 m
.34127 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.81746 .60332 m
.81746 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.97619 .60332 m
.97619 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .08677 m
.03006 .08677 L
s
[(-2.5)] .01131 .08677 1 0 Mshowa
.02381 .19008 m
.03006 .19008 L
s
[(-2)] .01131 .19008 1 0 Mshowa
.02381 .29339 m
.03006 .29339 L
s
[(-1.5)] .01131 .29339 1 0 Mshowa
.02381 .3967 m
.03006 .3967 L
s
[(-1)] .01131 .3967 1 0 Mshowa
.02381 .50001 m
.03006 .50001 L
s
[(-0.5)] .01131 .50001 1 0 Mshowa
.125 Mabswid
.02381 .10743 m
.02756 .10743 L
s
.02381 .1281 m
.02756 .1281 L
s
.02381 .14876 m
.02756 .14876 L
s
.02381 .16942 m
.02756 .16942 L
s
.02381 .21074 m
.02756 .21074 L
s
.02381 .23141 m
.02756 .23141 L
s
.02381 .25207 m
.02756 .25207 L
s
.02381 .27273 m
.02756 .27273 L
s
.02381 .31405 m
.02756 .31405 L
s
.02381 .33471 m
.02756 .33471 L
s
.02381 .35538 m
.02756 .35538 L
s
.02381 .37604 m
.02756 .37604 L
s
.02381 .41736 m
.02756 .41736 L
s
.02381 .43802 m
.02756 .43802 L
s
.02381 .45869 m
.02756 .45869 L
s
.02381 .47935 m
.02756 .47935 L
s
.02381 .52067 m
.02756 .52067 L
s
.02381 .54133 m
.02756 .54133 L
s
.02381 .562 m
.02756 .562 L
s
.02381 .58266 m
.02756 .58266 L
s
.02381 .06611 m
.02756 .06611 L
s
.02381 .04545 m
.02756 .04545 L
s
.02381 .02479 m
.02756 .02479 L
s
.02381 .00412 m
.02756 .00412 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03016 .54424 m
.04884 .37735 L
.06854 .22664 L
.07819 .16643 L
.08851 .11332 L
.09444 .08823 L
.09984 .06882 L
.10496 .05335 L
.11039 .03999 L
.1151 .03085 L
.11759 .02688 L
.12028 .02327 L
.12274 .02055 L
.12499 .01854 L
.12726 .01696 L
.12941 .01585 L
.13071 .01537 L
.13189 .01504 L
.13254 .01491 L
.13325 .01481 L
.13453 .01472 L
.13573 .01474 L
.137 .01488 L
.13809 .0151 L
.13928 .01543 L
.14069 .01594 L
.14197 .01653 L
.14485 .01824 L
.15005 .02262 L
.15483 .02797 L
.1599 .0349 L
.16901 .05014 L
.18963 .09412 L
.22713 .18859 L
.26707 .28556 L
.3055 .36389 L
.34636 .42929 L
.38572 .47714 L
.42358 .51203 L
.46387 .54002 L
.50265 .56027 L
.54387 .57647 L
.58359 .58822 L
.6218 .59685 L
.64154 .60051 L
.65162 .6022 L
.65452 .60266 L
.65595 .60289 L
.65728 .60309 L
.65847 .60328 L
Mistroke
.65975 .60332 L
.66048 .60332 L
.66116 .60332 L
.66245 .60332 L
.68189 .60332 L
.70008 .60332 L
.73843 .60332 L
.77921 .60332 L
.81848 .60332 L
.85625 .60332 L
.89646 .60332 L
.93516 .60332 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool2
003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`0Hool5003Uool001_oo`03003ooooo01Ooo`8000Coo`03003ooooo0>;oo`006ooo0P005ooo00<0
0?ooool01Ooo00<00?ooool0hOoo000Kool00`00ooooo`0Fool00`00ooooo`06ool2003Qool001_o
o`03003ooooo01Goo`03003ooooo00Soo`03003ooooo0=ooo`006ooo00<00?ooool05Ooo00<00?oo
ool02Ooo00<00?ooool0g_oo000Kool00`00ooooo`0Dool00`00ooooo`0:ool00`00ooooo`3Nool0
01_oo`03003ooooo01Coo`03003ooooo00_oo`03003ooooo0=goo`006ooo0P005?oo00<00?ooool0
3?oo00<00?ooool0gOoo000Kool00`00ooooo`0Cool00`00ooooo`0=ool00`00ooooo`3Lool001_o
o`03003ooooo01;oo`03003ooooo00koo`03003ooooo0=coo`006ooo00<00?ooool04_oo00<00?oo
ool03ooo00<00?ooool0fooo000Kool00`00ooooo`0Bool00`00ooooo`0?ool00`00ooooo`3Kool0
01_oo`8001;oo`03003ooooo017oo`03003ooooo0=[oo`006ooo00<00?ooool04Ooo00<00?ooool0
4Ooo00<00?ooool0f_oo000Kool00`00ooooo`0Aool00`00ooooo`0Bool00`00ooooo`3Iool001_o
o`03003ooooo017oo`03003ooooo01;oo`03003ooooo0=Woo`001ooo10000ooo00<00?ooool00_oo
0`001Ooo00<00?ooool04?oo00<00?ooool05?oo00<00?ooool0f?oo0007ool00`00ooooo`0<ool0
0`00ooooo`02ool00`00ooooo`0@ool00`00ooooo`0Dool00`00ooooo`3Hool000Soo`03003ooooo
00_oo`03003ooooo00;oo`80017oo`03003ooooo01Goo`03003ooooo0=Ooo`0000?oo`0000000P00
1?oo00<00?ooool01ooo0`001Ooo00<00?ooool04?oo00<00?ooool05Ooo00<00?ooool0eooo000:
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`0?ool00`00ooooo`0Gool00`00ooooo`3F
ool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0?ool00`00ooooo`0Gool00`00
ooooo`3Fool000Soo`8000Woo`@000Coo`03003ooooo00ooo`03003ooooo01Soo`03003ooooo0=Go
o`006ooo0P004?oo00<00?ooool06?oo00<00?ooool0eOoo000Kool00`00ooooo`0>ool00`00oooo
o`0Jool00`00ooooo`3Dool001_oo`03003ooooo00koo`03003ooooo01[oo`03003ooooo0=Coo`00
6ooo00<00?ooool03_oo00<00?ooool06_oo00<00?ooool0e?oo000Kool00`00ooooo`0>ool00`00
ooooo`0Kool00`00ooooo`3Cool001_oo`03003ooooo00goo`03003ooooo01coo`03003ooooo0=?o
o`006ooo0P003_oo00<00?ooool07Ooo00<00?ooool0d_oo000Kool00`00ooooo`0=ool00`00oooo
o`0Mool00`00ooooo`3Bool001_oo`03003ooooo00goo`03003ooooo01goo`03003ooooo0=;oo`00
6ooo00<00?ooool03Ooo00<00?ooool07_oo00<00?ooool0dOoo000Kool00`00ooooo`0<ool00`00
ooooo`0Oool00`00ooooo`3Aool001_oo`8000goo`03003ooooo023oo`03003ooooo0=3oo`006ooo
00<00?ooool03?oo00<00?ooool08?oo00<00?ooool0d?oo000Kool00`00ooooo`0<ool00`00oooo
o`0Pool00`00ooooo`3@ool001_oo`03003ooooo00_oo`03003ooooo02;oo`03003ooooo0<ooo`00
6ooo00<00?ooool02ooo00<00?ooool08_oo00<00?ooool0cooo000Kool00`00ooooo`0;ool00`00
ooooo`0Sool00`00ooooo`3>ool001_oo`8000coo`03003ooooo02?oo`03003ooooo0<koo`006ooo
00<00?ooool02ooo00<00?ooool08ooo00<00?ooool0c_oo000Kool00`00ooooo`0;ool00`00oooo
o`0Tool00`00ooooo`3=ool001?oo`@000Coo`03003ooooo00_oo`03003ooooo02Coo`03003ooooo
0<goo`004ooo00<00?ooool01Ooo00<00?ooool02_oo00<00?ooool09_oo00<00?ooool0c?oo000D
ool00`00ooooo`04ool2000;ool00`00ooooo`0Vool00`00ooooo`3<ool000goo`@000Coo`03003o
oooo00?oo`03003ooooo00[oo`03003ooooo02Koo`03003ooooo0<coo`005_oo00<00?ooool00_oo
00<00?ooool02_oo00<00?ooool09ooo00<00?ooool0booo000Cool01000ooooo`001?oo00<00?oo
ool02_oo00<00?ooool09ooo00<00?ooool0booo000Dool20005ool00`00ooooo`0:ool00`00oooo
o`0Xool00`00ooooo`3:ool001_oo`03003ooooo00[oo`03003ooooo02Soo`03003ooooo0<[oo`00
6ooo0P002ooo00<00?ooool0:Ooo00<00?ooool0bOoo000Kool00`00ooooo`09ool00`00ooooo`0Z
ool00`00ooooo`39ool001_oo`03003ooooo00Woo`03003ooooo02[oo`03003ooooo0<Woo`006ooo
00<00?ooool02Ooo00<00?ooool0:ooo00<00?ooool0b?oo000Kool00`00ooooo`09ool00`00oooo
o`0[ool00`00ooooo`38ool001_oo`8000[oo`03003ooooo02coo`03003ooooo0<Ooo`006ooo00<0
0?ooool02Ooo00<00?ooool0;?oo00<00?ooool0aooo000Kool00`00ooooo`09ool00`00ooooo`0/
ool00`00ooooo`37ool001_oo`03003ooooo00Soo`03003ooooo02koo`03003ooooo0<Koo`006ooo
00<00?ooool02?oo00<00?ooool0;_oo00<00?ooool0a_oo000Kool00`00ooooo`08ool00`00oooo
o`0_ool00`00ooooo`35ool001_oo`8000Woo`03003ooooo02ooo`03003ooooo0<Goo`006ooo00<0
0?ooool02?oo00<00?ooool0<?oo00<00?ooool0a?oo000Kool00`00ooooo`08ool00`00ooooo`0`
ool00`00ooooo`34ool001_oo`03003ooooo00Soo`03003ooooo033oo`03003ooooo0<Coo`006ooo
00<00?ooool01ooo00<00?ooool0<_oo00<00?ooool0`ooo000Kool20008ool00`00ooooo`0bool0
0`00ooooo`33ool001_oo`03003ooooo00Ooo`03003ooooo03?oo`03003ooooo0<;oo`006ooo00<0
0?ooool01ooo00<00?ooool0<ooo00<00?ooool0`_oo000Kool00`00ooooo`07ool00`00ooooo`0d
ool00`00ooooo`31ool000Ooo`@000?oo`03003ooooo00;oo`<000Goo`03003ooooo00Ooo`03003o
oooo03Coo`03003ooooo0<7oo`002Ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool01_oo
00<00?ooool0=Ooo00<00?ooool0`Ooo0009ool00`00ooooo`0:ool00`00ooooo`02ool20007ool0
0`00ooooo`0fool00`00ooooo`30ool00003ool00000008000Coo`03003ooooo00Ooo`<000Goo`03
003ooooo00Koo`03003ooooo03Koo`03003ooooo0<3oo`002Ooo00<00?ooool01ooo00<00?ooool0
1Ooo00<00?ooool01_oo00<00?ooool0=ooo00<00?ooool0_ooo0007ool30009ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`0gool00`00ooooo`2oool000Woo`03003ooooo00Ooo`@000Co
o`03003ooooo00Koo`03003ooooo03Soo`03003ooooo0;koo`006ooo0P001ooo00<00?ooool0>?oo
00<00?ooool0__oo000Kool00`00ooooo`05ool00`00ooooo`0jool00`00ooooo`2mool001_oo`03
003ooooo00Goo`03003ooooo03[oo`03003ooooo0;goo`006ooo00<00?ooool01Ooo00<00?ooool0
>ooo00<00?ooool0_?oo000Kool00`00ooooo`05ool00`00ooooo`0kool00`00ooooo`2lool001_o
o`03003ooooo00Goo`03003ooooo03coo`03003ooooo0;_oo`006ooo0P001_oo00<00?ooool0??oo
00<00?ooool0^ooo000Kool00`00ooooo`05ool00`00ooooo`0mool00`00ooooo`2jool001_oo`03
003ooooo00Coo`03003ooooo03koo`03003ooooo0;[oo`006ooo00<00?ooool01?oo00<00?ooool0
?ooo00<00?ooool0^Ooo000Kool00`00ooooo`04ool00`00ooooo`0oool00`00ooooo`2iool001_o
o`8000Goo`03003ooooo043oo`03003ooooo0;Soo`006ooo00<00?ooool01?oo00<00?ooool0@?oo
00<00?ooool0^?oo000Kool00`00ooooo`04ool00`00ooooo`11ool00`00ooooo`2gool001_oo`03
003ooooo00?oo`03003ooooo04;oo`03003ooooo0;Ooo`006ooo00<00?ooool00ooo00<00?ooool0
@ooo00<00?ooool0]_oo000Kool00`00ooooo`03ool00`00ooooo`13ool00`00ooooo`2fool001_o
o`8000Coo`03003ooooo04Coo`03003ooooo0;Goo`006ooo00<00?ooool00ooo00<00?ooool0AOoo
00<00?ooool0]?oo000Kool00`00ooooo`03ool00`00ooooo`15ool00`00ooooo`2dool001?oo`@0
00Coo`03003ooooo00?oo`03003ooooo04Koo`03003ooooo0;?oo`005Ooo00<00?ooool00ooo00<0
0?ooool00ooo00<00?ooool0Aooo00<00?ooool0/_oo000Eool00`00ooooo`03ool20004ool00`00
ooooo`17ool00`00ooooo`2bool000goo`@000Coo`03003ooooo00?oo`03003ooooo00;oo`03003o
oooo04Woo`03003ooooo0;7oo`005Ooo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0BOoo
00<00?ooool0/Ooo000Cool30005ool00`00ooooo`02ool00`00ooooo`1:ool00`00ooooo`2`ool0
01Goo`03003ooooo00?oo`03003ooooo00;oo`03003ooooo04_oo`03003ooooo0:ooo`006ooo00<0
0?ooool00_oo00<00?ooool0Booo00<00?ooool0[ooo000Kool20003ool00`00ooooo`1<ool00`00
ooooo`2^ool001_oo`03003ooooo00;oo`03003ooooo04goo`03003ooooo0:goo`006ooo00<00?oo
ool00_oo00<00?ooool0COoo00<00?ooool0[Ooo000Kool00`00ooooo`02ool00`00ooooo`1>ool0
0`00ooooo`2/ool001_oo`03003ooooo00;oo`03003ooooo04ooo`03003ooooo0:_oo`006ooo0P00
0ooo00<00?ooool0D?oo00<00?ooool0Z_oo000Kool01@00oooooooo0000Dooo00<00?ooool0Z_oo
000Kool01@00oooooooo0000E?oo00<00?ooool0ZOoo000Kool01@00oooooooo0000EOoo00<00?oo
ool0Z?oo000Kool01@00oooooooo0000E_oo00<00?ooool0Yooo000Kool01@00oooooooo0000E_oo
00<00?ooool0Yooo000Kool20002ool00`00ooooo`1Eool00`00ooooo`2Vool001_oo`05003ooooo
ool0001Hool00`00ooooo`2Uool001_oo`05003oooooool0001Iool00`00ooooo`2Tool001_oo`05
003oooooool0001Iool00`00ooooo`2Tool001_oo`05003oooooool0001Jool00`00ooooo`2Sool0
01_oo`8000;oo`03003ooooo05Woo`03003ooooo0:;oo`006ooo00D00?ooooooo`0005coo`03003o
oooo0:7oo`006ooo00@00?ooool005koo`03003ooooo0:3oo`006ooo00@00?ooool005ooo`03003o
oooo09ooo`002?oo0P001?oo00<00?ooool00_oo0`001Ooo00@00?ooool0063oo`03003ooooo09ko
o`001ooo00@00?ooool000_oo`03003ooooo00;oo`04003ooooo001Qool2002Nool000Ooo`04003o
oooo000;ool00`00ooooo`02ool200000ooo003oo`1Rool00`00ooooo`2Kool00003ool000000080
00;oo`04003ooooo0008ool30005ool01000ooooo`00I?oo00<00?ooool0V_oo0007ool01000oooo
o`002?oo00<00?ooool01Ooo00@00?ooool006Goo`03003ooooo09Woo`001ooo00@00?ooool000So
o`03003ooooo00Goo`04003ooooo001Vool00`00ooooo`2Hool000Soo`8000Woo`@000Coo`04003o
oooo001Wool2002Hool001_oo`800003ool00?oo06Soo`03003ooooo09Goo`006ooo00@00?ooool0
06[oo`8009Goo`006ooo00<00?oo0000KOoo0P00Tooo000Kool00`00ool0001_ool00`00ooooo`2@
ool001_oo`03003oo`00073oo`80093oo`006ooo00<00?oo0000L_oo00<00?ooool0SOoo000Kool3
001cool2002=ool001_oo`03003oo`0007Goo`8008_oo`006ooo00<00?ooool0Mooo00<00?ooool0
R?oo000Kool00`00ooooo`1hool20028ool001_oo`03003ooooo07[oo`8008Koo`006ooo0P00OOoo
0P00Q?oo000Kool00`00ooooo`0Hool20004ool00`00ooooo`02ool4000Bool20004ool00`00oooo
o`04ool00`00ooooo`0Bool20004ool00`00ooooo`03ool2000Dool40002ool00`00ooooo`03ool2
000Iool4000Gool40003ool00`00ooooo`02ool4000Bool40003ool00`00ooooo`04ool00`00oooo
o`0@ool001_oo`03003ooooo01Ooo`04003ooooo0008ool00`00ooooo`0Bool01000ooooo`002_oo
00<00?ooool04Ooo00@00?ooool000Soo`04003ooooo000Bool01@00ooooo`00ool00P001Ooo00@0
0?ooool001[oo`03003ooooo01Soo`03003ooooo00Ooo`03003ooooo01Goo`03003ooooo00Woo`03
003ooooo013oo`006ooo00<00?ooool05ooo00@00?ooool000Woo`03003ooooo017oo`04003ooooo
0007ool5000Bool01000ooooo`002?oo00@00?ooool001;oo`04003ooooo0003ool30002ool01000
ooooo`006_oo00<00?ooool06?oo00<00?ooool02?oo00<00?ooool05?oo00<00?ooool01_oo1@00
4Ooo000Kool00`00ooooo`0Gool01000ooooo`002_oo00<00?ooool04?oo00@00?ooool000Ooo`04
003ooooo000Cool01000ooooo`002?oo0`004ooo00@00?ooool000Koo`D001_oo`03003ooooo01So
o`03003ooooo00Woo`03003ooooo01?oo`03003ooooo00Koo`04003ooooo000Bool001_oo`03003o
oooo01Ooo`04003ooooo000;ool00`00ooooo`0?ool01000ooooo`002?oo00<00?oo00004ooo00@0
0?ooool000Soo`03003ooooo01?oo`04003ooooo0008ool5000Iool00`00ooooo`0Hool00`00oooo
o`0:ool00`00ooooo`0Bool00`00ooooo`07ool00`00ool0000Bool001_oo`8001Soo`04003ooooo
0008ool01000ooooo`004Ooo00@00?ooool000Woo`8001?oo`04003ooooo0008ool00`00ooooo`0C
ool01000ooooo`002?oo00D00?ooool00?oo00@001?oo`<001Soo`<000Woo`04003ooooo000Bool3
000:ool2000Bool001_oo`03003ooooo01Soo`8000[oo`8001?oo`8000_oo`03003ooooo01;oo`80
00[oo`<001?oo`8000[oo`8000Koo`<001;oo`03003ooooo01Soo`03003ooooo00Soo`8001Goo`03
003ooooo00Woo`03003ooooo013oo`006ooo00<00?ooool0U?oo0`00Jooo000Kool00`00ooooo`2G
ool4001Wool001_oo`03003ooooo09_oo`@006?oo`005?ooo`003@00000Kool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool0
01_oo`03003ooooo01ooo`03003ooooo01koo`03003ooooo01ooo`03003ooooo01ooo`03003ooooo
01ooo`03003ooooo01koo`03003ooooo01ooo`03003ooooo01Koo`006ooo00<00?ooool0oooo0ooo
000Kool00`00ooooo`3oool3ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo
003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, -3.03402, \
0.00592092, 0.0181944}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(diffs\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[eta], 
        Abs[Re[alphakx0korr[eta, 10.0]] - 
            Sqrt[11] FastRealpha[1, 1, 1 - 10^\((\(-6\))\), eta, 10]]}, {eta, 
        0.01, 1.2, 0.01}]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.01`, 0.002182302930692437`}, {0.02`, 
        0.020309602000534888`}, {0.03`, 0.00006752236381712962`}, {0.04`, 
        0.0005983966194200718`}, {0.05`, 0.012427656086924088`}, {0.06`, 
        0.000047040413460042174`}, {0.07`, 0.00008674163956934322`}, {0.08`, 
        0.0001297168835914242`}, {0.09`, 0.0001750206983825997`}, {0.1`, 
        0.00022172594322755756`}, {0.11`, 0.00026895325392350955`}, {0.12`, 
        0.00031589507891505164`}, {0.13`, 0.6074009980310788`}, {0.14`, 
        0.0004061447638050719`}, {0.15`, 0.0004483174340061957`}, {0.16`, 
        0.0005545842710867888`}, {0.17`, 0.0005433763982689577`}, {0.18`, 
        0.0005299414241854983`}, {0.19`, 0.0005146968845608502`}, {0.2`, 
        0.0004980217773822293`}, {0.21`, 0.0004802565829584715`}, {0.22`, 
        0.0004617027388378325`}, {0.23`, 0.00044262489442675745`}, {0.24`, 
        0.00042325114542280523`}, {0.25`, 0.00040377653697031`}, {0.26`, 
        0.0003843640876117216`}, {0.27`, 0.2053113596511924`}, {0.28`, 
        0.20133728311230484`}, {0.29`, 0.19708896374376428`}, {0.3`, 
        0.19261957748567515`}, {0.31`, 0.00029237836469631073`}, {0.32`, 
        0.00027552099714434064`}, {0.33`, 0.0007173210181935508`}, {0.34`, 
        0.0007114137254045083`}, {0.35000000000000003`, 
        0.0002299174994142561`}, {0.36`, 0.00021585079766106574`}, {0.37`, 
        0.00020245804841145798`}, {0.38`, 0.00018973149840206815`}, {0.39`, 
        0.00017765880910380893`}, {0.4`, 
        0.0001662245286073727`}, {0.41000000000000003`, 
        0.00015846343026248988`}, {0.42`, 0.00014519584627215032`}, {0.43`, 
        0.000135559262253393`}, {0.44`, 0.00012647789739905768`}, {0.45`, 
        0.00011792812601507663`}, {0.46`, 
        0.00010988639717446347`}, {0.47000000000000003`, 
        0.00010232878605531681`}, {0.48`, 0.00009523181340820308`}, {0.49`, 
        0.00008857233397940867`}, {0.5`, 0.00008232749223191504`}, {0.51`, 
        0.00007647535756327706`}, {0.52`, 0.0000709947140733469`}, {0.53`, 
        0.0000658647701325954`}, {0.54`, 0.00006106591039856113`}, {0.55`, 
        0.08467922044382781`}, {0.56`, 
        0.08157111311179066`}, {0.5700000000000001`, 
        0.07856104994262925`}, {0.58`, 0.07564627403679558`}, {0.59`, 
        0.07282394176894269`}, {0.6`, 0.07009114968778352`}, {0.61`, 
        0.06744495842143916`}, {0.62`, 0.06488241179628507`}, {0.63`, 
        0.062400553363119005`}, {0.64`, 0.05999643963587542`}, {0.65`, 
        0.000025225303392550913`}, {0.66`, 0.000023130315236263854`}, {0.67`, 
        0.000021195295686671223`}, {0.68`, 
        0.00001939725069211029`}, {0.6900000000000001`, 
        0.0000177344258769363`}, {0.7000000000000001`, 
        0.000016185064189655485`}, {0.71`, 0.000014748569758249452`}, {0.72`, 
        0.00001341768437745916`}, {0.73`, 0.000012185887582671917`}, {0.74`, 
        0.000011054142167177261`}, {0.75`, 0.000010002501601658054`}, {0.76`, 
        9.032165191436015`*^-6}, {0.77`, 8.136550438520018`*^-6}, {0.78`, 
        7.311809200993835`*^-6}, {0.79`, 6.37441568884789`*^-6}, {0.8`, 
        5.857407411657967`*^-6}, {0.81`, 
        5.220025116270399`*^-6}, {0.8200000000000001`, 
        4.637080605940369`*^-6}, {0.8300000000000001`, 
        4.105307360169341`*^-6}, {0.84`, 3.6213203760127044`*^-6}, {0.85`, 
        3.1823144688303273`*^-6}, {0.86`, 2.777185238556079`*^-6}, {0.87`, 
        0.004743889788025202`}, {0.88`, 0.009919970382346044`}, {0.89`, 
        0.009307377659422578`}, {0.9`, 0.00869132812173723`}, {0.91`, 
        0.014615776958428306`}, {0.92`, 0.00743556907324594`}, {0.93`, 
        0.011953407760167845`}, {0.9400000000000001`, 
        0.010594993636641958`}, {0.9500000000000001`, 
        0.009202886305519908`}, {0.96`, 0.007759253978918079`}, {0.97`, 
        0.006236344204728184`}, {0.98`, 0.004585129256444831`}, {0.99`, 
        0.0026965161613335953`}, {1.`, Indeterminate}, {1.01`, 
        3.3856242809271464`*^-7}, {1.02`, 7.690140647609311`*^-7}, {1.03`, 
        3.3645516006597376`*^-7}, {1.04`, 1.628890380356296`*^-7}, {1.05`, 
        1.0432978136964977`*^-7}, {1.06`, 8.571520759836915`*^-8}, {1.07`, 
        8.108339843992951`*^-8}, {1.08`, 8.120643979481201`*^-8}, {1.09`, 
        8.30780267913056`*^-8}, {1.1`, 8.533794737530798`*^-8}, {1.11`, 
        8.766491055300918`*^-8}, {1.12`, 
        8.994062450362605`*^-8}, {1.1300000000000001`, 
        9.178154181110898`*^-8}, {1.1400000000000001`, 
        9.376720000453531`*^-8}, {1.1500000000000001`, 
        9.534269795885564`*^-8}, {1.16`, 9.672814060834867`*^-8}, {1.17`, 
        9.80863067434612`*^-8}, {1.18`, 9.914994536508459`*^-8}, {1.19`, 
        1.0018349072549171`*^-7}, {1.2`, 7.804566193314599`*^-8}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotRange \[Rule] All]\)], "Input"],

Cell[BoxData[
    \(Graphics::"gptn" \(\(:\)\(\ \)\) 
      "Coordinate \!\(Indeterminate\) in \!\({1.`, Indeterminate}\) is not a \
floating-point number."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0158063 0.80032 0.014715 0.969053 [
[.17587 .00222 -9 -9 ]
[.17587 .00222 9 0 ]
[.33593 .00222 -9 -9 ]
[.33593 .00222 9 0 ]
[.496 .00222 -9 -9 ]
[.496 .00222 9 0 ]
[.65606 .00222 -9 -9 ]
[.65606 .00222 9 0 ]
[.81613 .00222 -3 -9 ]
[.81613 .00222 3 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.00331 .11162 -18 -4.5 ]
[.00331 .11162 0 4.5 ]
[.00331 .20853 -18 -4.5 ]
[.00331 .20853 0 4.5 ]
[.00331 .30543 -18 -4.5 ]
[.00331 .30543 0 4.5 ]
[.00331 .40234 -18 -4.5 ]
[.00331 .40234 0 4.5 ]
[.00331 .49924 -18 -4.5 ]
[.00331 .49924 0 4.5 ]
[.00331 .59615 -18 -4.5 ]
[.00331 .59615 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17587 .01472 m
.17587 .02097 L
s
[(0.2)] .17587 .00222 0 1 Mshowa
.33593 .01472 m
.33593 .02097 L
s
[(0.4)] .33593 .00222 0 1 Mshowa
.496 .01472 m
.496 .02097 L
s
[(0.6)] .496 .00222 0 1 Mshowa
.65606 .01472 m
.65606 .02097 L
s
[(0.8)] .65606 .00222 0 1 Mshowa
.81613 .01472 m
.81613 .02097 L
s
[(1)] .81613 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(1.2)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05582 .01472 m
.05582 .01847 L
s
.09584 .01472 m
.09584 .01847 L
s
.13585 .01472 m
.13585 .01847 L
s
.21589 .01472 m
.21589 .01847 L
s
.2559 .01472 m
.2559 .01847 L
s
.29592 .01472 m
.29592 .01847 L
s
.37595 .01472 m
.37595 .01847 L
s
.41597 .01472 m
.41597 .01847 L
s
.45598 .01472 m
.45598 .01847 L
s
.53601 .01472 m
.53601 .01847 L
s
.57603 .01472 m
.57603 .01847 L
s
.61605 .01472 m
.61605 .01847 L
s
.69608 .01472 m
.69608 .01847 L
s
.73609 .01472 m
.73609 .01847 L
s
.77611 .01472 m
.77611 .01847 L
s
.85614 .01472 m
.85614 .01847 L
s
.89616 .01472 m
.89616 .01847 L
s
.93617 .01472 m
.93617 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.01581 .11162 m
.02206 .11162 L
s
[(0.1)] .00331 .11162 1 0 Mshowa
.01581 .20853 m
.02206 .20853 L
s
[(0.2)] .00331 .20853 1 0 Mshowa
.01581 .30543 m
.02206 .30543 L
s
[(0.3)] .00331 .30543 1 0 Mshowa
.01581 .40234 m
.02206 .40234 L
s
[(0.4)] .00331 .40234 1 0 Mshowa
.01581 .49924 m
.02206 .49924 L
s
[(0.5)] .00331 .49924 1 0 Mshowa
.01581 .59615 m
.02206 .59615 L
s
[(0.6)] .00331 .59615 1 0 Mshowa
.125 Mabswid
.01581 .0341 m
.01956 .0341 L
s
.01581 .05348 m
.01956 .05348 L
s
.01581 .07286 m
.01956 .07286 L
s
.01581 .09224 m
.01956 .09224 L
s
.01581 .131 m
.01956 .131 L
s
.01581 .15038 m
.01956 .15038 L
s
.01581 .16976 m
.01956 .16976 L
s
.01581 .18914 m
.01956 .18914 L
s
.01581 .22791 m
.01956 .22791 L
s
.01581 .24729 m
.01956 .24729 L
s
.01581 .26667 m
.01956 .26667 L
s
.01581 .28605 m
.01956 .28605 L
s
.01581 .32481 m
.01956 .32481 L
s
.01581 .34419 m
.01956 .34419 L
s
.01581 .36357 m
.01956 .36357 L
s
.01581 .38296 m
.01956 .38296 L
s
.01581 .42172 m
.01956 .42172 L
s
.01581 .4411 m
.01956 .4411 L
s
.01581 .46048 m
.01956 .46048 L
s
.01581 .47986 m
.01956 .47986 L
s
.01581 .51862 m
.01956 .51862 L
s
.01581 .538 m
.01956 .538 L
s
.01581 .55738 m
.01956 .55738 L
s
.01581 .57677 m
.01956 .57677 L
s
.01581 .61553 m
.01956 .61553 L
s
.25 Mabswid
.01581 0 m
.01581 .61803 L
s
.008 w
.02381 .01683 Mdot
.03181 .0344 Mdot
.03982 .01478 Mdot
.04782 .01529 Mdot
.05582 .02676 Mdot
.06383 .01476 Mdot
.07183 .0148 Mdot
.07983 .01484 Mdot
.08784 .01488 Mdot
.09584 .01493 Mdot
.10384 .01498 Mdot
.11184 .01502 Mdot
.11985 .60332 Mdot
.12785 .01511 Mdot
.13585 .01515 Mdot
.14386 .01525 Mdot
.15186 .01524 Mdot
.15986 .01523 Mdot
.16787 .01521 Mdot
.17587 .0152 Mdot
.18387 .01518 Mdot
.19188 .01516 Mdot
.19988 .01514 Mdot
.20788 .01513 Mdot
.21589 .01511 Mdot
.22389 .01509 Mdot
.23189 .21367 Mdot
.2399 .20982 Mdot
.2479 .2057 Mdot
.2559 .20137 Mdot
.26391 .015 Mdot
.27191 .01498 Mdot
.27991 .01541 Mdot
.28792 .0154 Mdot
.29592 .01494 Mdot
.30392 .01492 Mdot
.31192 .01491 Mdot
.31993 .0149 Mdot
.32793 .01489 Mdot
.33593 .01488 Mdot
.34394 .01487 Mdot
.35194 .01486 Mdot
.35994 .01485 Mdot
.36795 .01484 Mdot
.37595 .01483 Mdot
.38395 .01482 Mdot
.39196 .01481 Mdot
.39996 .01481 Mdot
.40796 .0148 Mdot
.41597 .01479 Mdot
.42397 .01479 Mdot
.43197 .01478 Mdot
.43998 .01478 Mdot
.44798 .01477 Mdot
.45598 .09677 Mdot
.46399 .09376 Mdot
.47199 .09084 Mdot
.47999 .08802 Mdot
.488 .08529 Mdot
.496 .08264 Mdot
.504 .08007 Mdot
.512 .07759 Mdot
.52001 .07518 Mdot
.52801 .07285 Mdot
.53601 .01474 Mdot
.54402 .01474 Mdot
.55202 .01474 Mdot
.56002 .01473 Mdot
.56803 .01473 Mdot
.57603 .01473 Mdot
.58403 .01473 Mdot
.59204 .01473 Mdot
.60004 .01473 Mdot
.60804 .01473 Mdot
.61605 .01472 Mdot
.62405 .01472 Mdot
.63205 .01472 Mdot
.64006 .01472 Mdot
.64806 .01472 Mdot
.65606 .01472 Mdot
.66407 .01472 Mdot
.67207 .01472 Mdot
.68007 .01472 Mdot
.68808 .01472 Mdot
.69608 .01472 Mdot
.70408 .01472 Mdot
.71208 .01931 Mdot
.72009 .02433 Mdot
.72809 .02373 Mdot
.73609 .02314 Mdot
.7441 .02888 Mdot
.7521 .02192 Mdot
.7601 .0263 Mdot
.76811 .02498 Mdot
.77611 .02363 Mdot
.78411 .02223 Mdot
.79212 .02076 Mdot
.80012 .01916 Mdot
.80812 .01733 Mdot
.82413 .01472 Mdot
.83213 .01472 Mdot
.84014 .01472 Mdot
.84814 .01472 Mdot
.85614 .01472 Mdot
.86415 .01472 Mdot
.87215 .01472 Mdot
.88015 .01472 Mdot
.88816 .01472 Mdot
.89616 .01472 Mdot
.90416 .01472 Mdot
.91216 .01472 Mdot
.92017 .01472 Mdot
.92817 .01472 Mdot
.93617 .01472 Mdot
.94418 .01472 Mdot
.95218 .01472 Mdot
.96018 .01472 Mdot
.96819 .01472 Mdot
.97619 .01472 Mdot
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00>Ooo0P001?oo00<00?ooool00_oo10006ooo0P001?oo00<00?ooool01?oo00<0
0?ooool06ooo0P001?oo00<00?ooool00ooo0P007Ooo0P001?oo00<00?ooool00ooo0P008_oo1000
8Ooo10000ooo00<00?ooool00_oo10000_oo000hool01000ooooo`002?oo00<00?ooool06ooo00@0
0?ooool000[oo`03003ooooo01[oo`04003ooooo0008ool01000ooooo`006ooo00@00?ooool000So
o`04003ooooo000Sool00`00ooooo`0Rool00`00ooooo`07ool00`00ooooo`03ool003Soo`04003o
oooo0009ool00`00ooooo`0Jool01000ooooo`001ooo1@006ooo00@00?ooool000Soo`04003ooooo
000Kool01000ooooo`002?oo00@00?ooool002?oo`03003ooooo02;oo`03003ooooo00Soo`03003o
oooo00;oo`00>?oo00@00?ooool000[oo`03003ooooo01Woo`04003ooooo0007ool01000ooooo`00
7?oo00@00?ooool000Soo`<001coo`04003ooooo0009ool2000Tool00`00ooooo`0Rool00`00oooo
o`09ool00`00ooooo`01ool003Soo`04003ooooo000;ool00`00ooooo`0Hool01000ooooo`002?oo
00<00?oo00007?oo00@00?ooool000Soo`03003ooooo01coo`04003ooooo0008ool01000ooooo`00
8ooo00<00?ooool08_oo00<00?ooool02_oo0@000Ooo0Ooo000hool01000ooooo`002?oo00@00?oo
ool001[oo`04003ooooo0009ool2000Lool01000ooooo`002?oo00<00?ooool07?oo00@00?ooool0
00Soo`04003ooooo000Qool3000Rool30009ool01000ooooo`000_oo000iool2000:ool2000Lool2
000;ool00`00ooooo`0Kool2000:ool3000Lool2000:ool2000Tool00`00ooooo`0Rool00`00oooo
o`08ool20003ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`3oool9ool001Goo`03003ooooo00?oo`@000;oo`H00003ool0000000H000;oo`@00003ool0
000000h00003ool0000000@000Woo`h00003ool0000000`00003ool0000000h00003ool000000080
01Goo`@00003ool0000000`00003ool0000000`00003ool0000000X001koo`@00003ool0000000`0
0003ool0000000`00003ool0000000H000Woo`004Oooo`003@000ooo000Eool00`00ool00008ool0
0`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool0
0`00ooooo`08ool01@00oooooooo00001Ooo0P000_oo0P001Ooo00<00?oo00000`0000Koo`00ool0
0?oo0009ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool001Go
o`03003ooooo00Ooo`800:ooo`800005ool00000ool00003ool70002ool00`00ool0000jool001Go
o`03003ooooo00Soo`03003ooooo0:koo`04003ooooo0002ool20003ool01@00ool00?oo0000?ooo
000Eool01000oooooooo0P00_Ooo00<00?ooool0AOoo000Eool20003ool00`00ooooo`3oool4ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03
003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`80
08Coo`@0087oo`005Ooo00<00?ooool0POoo0P0000Coo`00ool0087oo`005Ooo00<00?ooool0OOoo
100000?oo`00ool0Q?oo000Eool00`00ooooo`1jool20002ool00`00ool00027ool001Goo`03003o
oooo07Koo`@00003ool00?oo08_oo`005Ooo0P00MOoo0P0000Coo`00ool008koo`005Ooo00<00?oo
ool0L_oo0P0000?oo`00ool0TOoo000Eool00`00ooooo`1cool00`00ooooo`2Bool000;oo`8000Co
o`03003ooooo00;oo`@000Coo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000[oo`03003ooooo
00?oo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`<00?ooo`Woo`00
00Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000[o
o`03003ooooo00?oo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000Soo`<000Goo`03003ooooo
0?ooo`Woo`000_oo0P002ooo00<00?ooool00ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3o
ool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03
003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03
003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[o
o`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool000;oo`8000Coo`03003ooooo
00;oo`@000Coo`03003ooooo03coo`800<[oo`0000Goo`00ooooo`0000Soo`03003ooooo00Goo`03
003ooooo03[oo`800003ool00?oo0<Woo`0000Goo`00ooooo`0000Woo`03003ooooo00Coo`<003So
o`800003ool00?oo0<_oo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo03Koo`80
0003ool00?oo0<goo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003ooooo03Ooo`03003o
oooo0<koo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3oool9ool000;oo`80
00[oo`8000Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03
003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`80
0?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Wo
o`005Ooo00<00?ooool0oooo2Ooo0002ool20004ool00`00ooooo`03ool20005ool00`00ooooo`3o
ool9ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0oooo2Ooo00001Ooo003o
oooo00002ooo00<00?ooool00_oo0`00oooo2Ooo00001Ooo003ooooo00002Ooo0P001Ooo00<00?oo
ool0oooo2Ooo00001Ooo003ooooo00002ooo00<00?ooool00_oo00<00?ooool0oooo2Ooo00001Ooo
003ooooo00002?oo00@00?ooool000Coo`03003ooooo0?ooo`Woo`000_oo0P002_oo0P001Ooo00<0
0?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo
000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool0
0`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool0
0`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00oooo
o`3oool9ool000;oo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo0?ooo`Woo`00
00Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000Oo
o`D000Coo`<00?ooo`Woo`0000Goo`00ooooo`0000Ooo`04003ooooo0005ool00`00ooooo`3oool9
ool00005ool00?ooool00008ool00`00ool00005ool00`00ooooo`3oool9ool00005ool00?ooool0
0009ool20005ool00`00ooooo`3oool9ool000;oo`8000_oo`03003ooooo00?oo`03003ooooo0?oo
o`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo
0002ool20004ool00`00ooooo`02ool30005ool00`00ooooo`3oool9ool00005ool00?ooool0000;
ool00`00ooooo`02ool00`00ooooo`3oool9ool00005ool00?ooool0000;ool00`00ooooo`02ool3
003oool9ool00005ool00?ooool00008ool30005ool00`00ooooo`3oool9ool00005ool00?ooool0
0008ool00`00ooooo`05ool00`00ooooo`3oool9ool00005ool00?ooool00008ool00`00ooooo`05
ool00`00ooooo`3oool9ool000;oo`8000Woo`@000Coo`03003ooooo0?ooo`Woo`005Ooo0P00oooo
2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo
2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000E
ool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000E
ool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003o
ool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo0002ool20004ool00`00
ooooo`03ool20005ool00`00ooooo`3oool9ool00005ool00?ooool00008ool01000ooooo`001?oo
00<00?ooool0oooo2Ooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`<00?ooo`Woo`0000Go
o`00ooooo`0000Soo`<000Goo`03003ooooo01Soo`800>koo`0000Goo`00ooooo`0000Soo`03003o
oooo00Goo`03003ooooo01Woo`03003ooooo0>coo`0000Goo`00ooooo`0000Soo`03003ooooo00Go
o`03003ooooo0?ooo`Woo`000_oo0P002_oo0`001?oo00<00?ooool0oooo2Ooo000Eool2003oool:
ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0997789, -0.0541438, \
0.00467548, 0.00386138}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRedelta[1, 1, 1 - 10^\((\(-6\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(39\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(39\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(39\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.40129574781642013`\)."\
\)], "Message"],

Cell[BoxData[
    \(0.5396538677162888`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImdelta[1, 1, 1 - 10^\((\(-6\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.40129574781564076`\)."\
\)], "Message"],

Cell[BoxData[
    \(2.209939429344497`\)], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["beta", "Section"],

Cell[CellGroupData[{

Cell["small a", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(toorder = 5\)], "Input"],

Cell[BoxData[
    \(5\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(vn = Sqrt[1 - nz^2]*Cos[phi]*Sqrt[1 - xi^2] + nz*xi\)], "Input"],

Cell[BoxData[
    \(nz\ xi + \@\(1 - nz\^2\)\ \@\(1 - xi\^2\)\ Cos[phi]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(Intexp = 
        Series[1/\((1 + a\ \((vn^2)\))\)^2\ om\ k\ xi \((k\ xi + \ 
                  a\ k\ nz\ vn)\)/\((om - k\ xi)\), {a, 0, 
            toorder}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(a0 = 
      Simplify[md2/k^2*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexp[\([3]\)]\)[\([1]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(md2\ om\^2\ \((2\ k + om\ Log[\(-k\) + om] - 
                om\ Log[k + om])\)\)\/\(2\ k\^3\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
        md2/k^2*1/2*\(1/2\)/Pi*
          Integrate[\(Intexp[\([3]\)]\)[\([1]\)], {phi, 0, 2*Pi}]]*
      k^2/om^2\)], "Input"],

Cell[BoxData[
    \(\(k\^2\ md2\ xi\^2\)\/\(om\ \((2\ om - 2\ k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apart[%55]\)], "Input"],

Cell[BoxData[
    \(\(-\(md2\/2\)\) - \(k\ md2\ xi\)\/\(2\ om\) - \(md2\ om\)\/\(2\ \
\((\(-om\) + k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(=\)\(PiL\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(a1 = 
      Simplify[md2/k^2*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexp[\([3]\)]\)[\([2]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(6\ k\^5\)\)\((md2\ om\^2\ \((4\ k\^3 - 6\ k\^3\ nz\^2 - 
            6\ k\ om\^2 + 18\ k\ nz\^2\ om\^2 - 
            3\ om\ \((k\^2\ \((\(-1\) + 2\ nz\^2)\) + \((1 - 
                        3\ nz\^2)\)\ om\^2)\)\ Log[\(-k\) + om] + 
            3\ om\ \((k\^2\ \((\(-1\) + 2\ nz\^2)\) + \((1 - 
                        3\ nz\^2)\)\ om\^2)\)\ Log[
                k + om])\))\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
        md2/k^2*1/2*\(1/2\)/Pi*
          Integrate[\(Intexp[\([3]\)]\)[\([2]\)], {phi, 0, 2*Pi}]]*
      k^2/om^2\)], "Input"],

Cell[BoxData[
    \(\(-\(\(k\^2\ md2\ xi\ \((xi - 
                2\ nz\^2\ xi + \((\(-1\) + 
                      3\ nz\^2)\)\ xi\^3)\)\)\/\(2\ om\ \((om - 
                k\ xi)\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apart[%57]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(md2\ \((\(-k\^2\) + 2\ k\^2\ nz\^2 + om\^2 - 
                  3\ nz\^2\ om\^2)\)\)\/\(2\ k\^2\)\)\) - \(md2\ \((\(-k\^2\) \
+ 2\ k\^2\ nz\^2 + om\^2 - 3\ nz\^2\ om\^2)\)\ xi\)\/\(2\ k\ om\) + 
      1\/2\ md2\ \((\(-1\) + 
            3\ nz\^2)\)\ xi\^2 + \(k\ md2\ \((\(-1\) + 3\ nz\^2)\)\ xi\^3\)\/\
\(2\ om\) + \(k\^2\ md2\ om - 2\ k\^2\ md2\ nz\^2\ om - md2\ om\^3 + 3\ md2\ \
nz\^2\ om\^3\)\/\(2\ k\^2\ \((\(-om\) + k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a2 = 
      Simplify[md2/k^2*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexp[\([3]\)]\)[\([3]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(240\ k\^7\)\)\((md2\ om\^2\ \((2\ k\ \((8\ k\^4\ \((9 - 
                          30\ nz\^2 + 20\ nz\^4)\) - 
                    15\ k\^2\ \((15 - 102\ nz\^2 + 95\ nz\^4)\)\ om\^2 + 
                    45\ \((3 - 30\ nz\^2 + 35\ nz\^4)\)\ om\^4)\) + 
              15\ om\ \((k\^4\ \((9 - 42\ nz\^2 + 33\ nz\^4)\) - 
                    2\ k\^2\ \((9 - 66\ nz\^2 + 65\ nz\^4)\)\ om\^2 + 
                    3\ \((3 - 30\ nz\^2 + 
                          35\ nz\^4)\)\ om\^4)\)\ Log[\(-k\) + om] - 
              15\ om\ \((k\^4\ \((9 - 42\ nz\^2 + 33\ nz\^4)\) - 
                    2\ k\^2\ \((9 - 66\ nz\^2 + 65\ nz\^4)\)\ om\^2 + 
                    3\ \((3 - 30\ nz\^2 + 35\ nz\^4)\)\ om\^4)\)\ Log[
                  k + om])\))\)\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a3 = 
      Simplify[md2/k^2*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexp[\([3]\)]\)[\([4]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(1680\ k\^9\)\)\((md2\ om\^2\ \((2\ k\ \((\(-24\)\ k\^6\ \
\((\(-20\) + 105\ nz\^2 - 140\ nz\^4 + 56\ nz\^6)\) + 
                  21\ k\^4\ \((\(-110\) + 1185\ nz\^2 - 2280\ nz\^4 + 
                        1197\ nz\^6)\)\ om\^2 - 
                  175\ k\^2\ \((\(-16\) + 255\ nz\^2 - 630\ nz\^4 + 
                        399\ nz\^6)\)\ om\^4 + 
                  210\ \((\(-5\) + 105\ nz\^2 - 315\ nz\^4 + 
                        231\ nz\^6)\)\ om\^6)\) - 
            105\ om\ \((5\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((\(-2\) + 
                        11\ nz\^2)\) - 
                  30\ k\^4\ \((\(-1\) + 12\ nz\^2 - 25\ nz\^4 + 
                        14\ nz\^6)\)\ om\^2 + 
                  3\ k\^2\ \((\(-10\) + 165\ nz\^2 - 420\ nz\^4 + 
                        273\ nz\^6)\)\ om\^4 + 
                  2\ \((5 - 105\ nz\^2 + 315\ nz\^4 - 
                        231\ nz\^6)\)\ om\^6)\)\ Log[\(-k\) + om] + 
            105\ om\ \((5\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((\(-2\) + 
                        11\ nz\^2)\) - 
                  30\ k\^4\ \((\(-1\) + 12\ nz\^2 - 25\ nz\^4 + 
                        14\ nz\^6)\)\ om\^2 + 
                  3\ k\^2\ \((\(-10\) + 165\ nz\^2 - 420\ nz\^4 + 
                        273\ nz\^6)\)\ om\^4 + 
                  2\ \((5 - 105\ nz\^2 + 315\ nz\^4 - 
                        231\ nz\^6)\)\ om\^6)\)\ Log[
                k + om])\))\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a4 = 
      Simplify[md2/k^2*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexp[\([3]\)]\)[\([5]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(80640\ k\^11\)\)\((md2\ om\^2\ \((2\ k\ \((128\ k\^8\ \
\((175 - 1260\ nz\^2 + 2520\ nz\^4 - 2016\ nz\^6 + 576\ nz\^8)\) - 
                    9\ k\^6\ \((16275 - 240940\ nz\^2 + 705810\ nz\^4 - 
                          746732\ nz\^6 + 265971\ nz\^8)\)\ om\^2 + 
                    105\ k\^4\ \((2555 - 56140\ nz\^2 + 213570\ nz\^4 - 
                          276108\ nz\^6 + 115995\ nz\^8)\)\ om\^4 - 
                    735\ k\^2\ \((275 - 7980\ nz\^2 + 37170\ nz\^4 - 
                          56364\ nz\^6 + 27027\ nz\^8)\)\ om\^6 + 
                    1575\ \((35 - 1260\ nz\^2 + 6930\ nz\^4 - 12012\ nz\^6 + 
                          6435\ nz\^8)\)\ om\^8)\) + 
              315\ om\ \((35\ k\^8\ \((\(-1\) + nz\^2)\)\^3\ \((\(-5\) + 
                          37\ nz\^2)\) - 
                    140\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((5 - 74\ nz\^2 + 
                          117\ nz\^4)\)\ om\^2 + 
                    42\ k\^4\ \((25 - 580\ nz\^2 + 2310\ nz\^4 - 
                          3108\ nz\^6 + 1353\ nz\^8)\)\ om\^4 - 
                    4\ k\^2\ \((175 - 5180\ nz\^2 + 24570\ nz\^4 - 
                          37884\ nz\^6 + 18447\ nz\^8)\)\ om\^6 + 
                    5\ \((35 - 1260\ nz\^2 + 6930\ nz\^4 - 12012\ nz\^6 + 
                          6435\ nz\^8)\)\ om\^8)\)\ Log[\(-k\) + om] - 
              315\ om\ \((35\ k\^8\ \((\(-1\) + nz\^2)\)\^3\ \((\(-5\) + 
                          37\ nz\^2)\) - 
                    140\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((5 - 74\ nz\^2 + 
                          117\ nz\^4)\)\ om\^2 + 
                    42\ k\^4\ \((25 - 580\ nz\^2 + 2310\ nz\^4 - 
                          3108\ nz\^6 + 1353\ nz\^8)\)\ om\^4 - 
                    4\ k\^2\ \((175 - 5180\ nz\^2 + 24570\ nz\^4 - 
                          37884\ nz\^6 + 18447\ nz\^8)\)\ om\^6 + 
                    5\ \((35 - 1260\ nz\^2 + 6930\ nz\^4 - 12012\ nz\^6 + 
                          6435\ nz\^8)\)\ om\^8)\)\ Log[
                  k + om])\))\)\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a5 = 
      Simplify[md2/k^2*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexp[\([3]\)]\)[\([6]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(887040\ k\^13\)\)\((md2\ om\^2\ \((\(-2\)\ k\ \((640\ k\^10\ \
\((\(-378\) + 3465\ nz\^2 - 9240\ nz\^4 + 11088\ nz\^6 - 6336\ nz\^8 + 
                        1408\ nz\^10)\) - 
                  165\ k\^8\ \((\(-12159\) + 229320\ nz\^2 - 903210\ nz\^4 + 
                        1439172\ nz\^6 - 1027503\ nz\^8 + 
                        274252\ nz\^10)\)\ om\^2 + 
                  495\ k\^6\ \((\(-9954\) + 279195\ nz\^2 - 1437240\ nz\^4 + 
                        2816730\ nz\^6 - 2384910\ nz\^8 + 
                        736307\ nz\^10)\)\ om\^4 - 
                  1617\ k\^4\ \((\(-3456\) + 128205\ nz\^2 - 811860\ nz\^4 + 
                        1878030\ nz\^6 - 1827540\ nz\^8 + 
                        636493\ nz\^10)\)\ om\^6 + 
                  10395\ k\^2\ \((\(-294\) + 13545\ nz\^2 - 101640\ nz\^4 + 
                        270270\ nz\^6 - 296010\ nz\^8 + 
                        114257\ nz\^10)\)\ om\^8 - 
                  10395\ \((\(-63\) + 3465\ nz\^2 - 30030\ nz\^4 + 
                        90090\ nz\^6 - 109395\ nz\^8 + 
                        46189\ nz\^10)\)\ om\^10)\) - 
            3465\ om\ \((63\ k\^10\ \((\(-1\) + nz\^2)\)\^4\ \((\(-3\) + 
                        28\ nz\^2)\) - 
                  105\ k\^8\ \((\(-1\) + nz\^2)\)\^3\ \((9 - 168\ nz\^2 + 
                        319\ nz\^4)\)\ om\^2 + 
                  630\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((\(-3\) + 84\ nz\^2 - 
                        319\ nz\^4 + 286\ nz\^6)\)\ om\^4 - 
                  90\ k\^4\ \((\(-21\) + 805\ nz\^2 - 5250\ nz\^4 + 
                        12474\ nz\^6 - 12441\ nz\^8 + 
                        4433\ nz\^10)\)\ om\^6 + 
                  5\ k\^2\ \((\(-189\) + 8820\ nz\^2 - 66990\ nz\^4 + 
                        180180\ nz\^6 - 199485\ nz\^8 + 
                        77792\ nz\^10)\)\ om\^8 - 
                  3\ \((\(-63\) + 3465\ nz\^2 - 30030\ nz\^4 + 90090\ nz\^6 - 
                        109395\ nz\^8 + 
                        46189\ nz\^10)\)\ om\^10)\)\ Log[\(-k\) + om] + 
            3465\ om\ \((63\ k\^10\ \((\(-1\) + nz\^2)\)\^4\ \((\(-3\) + 
                        28\ nz\^2)\) - 
                  105\ k\^8\ \((\(-1\) + nz\^2)\)\^3\ \((9 - 168\ nz\^2 + 
                        319\ nz\^4)\)\ om\^2 + 
                  630\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((\(-3\) + 84\ nz\^2 - 
                        319\ nz\^4 + 286\ nz\^6)\)\ om\^4 - 
                  90\ k\^4\ \((\(-21\) + 805\ nz\^2 - 5250\ nz\^4 + 
                        12474\ nz\^6 - 12441\ nz\^8 + 
                        4433\ nz\^10)\)\ om\^6 + 
                  5\ k\^2\ \((\(-189\) + 8820\ nz\^2 - 66990\ nz\^4 + 
                        180180\ nz\^6 - 199485\ nz\^8 + 
                        77792\ nz\^10)\)\ om\^8 - 
                  3\ \((\(-63\) + 3465\ nz\^2 - 30030\ nz\^4 + 90090\ nz\^6 - 
                        109395\ nz\^8 + 46189\ nz\^10)\)\ om\^10)\)\ Log[
                k + om])\))\)\)\)], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exact results - numerical", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(s = Sqrt[\((1 - om^2/k^2)\)*\((1 - kz^2/k^2)\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi1[k_, kz_, om_] = om*kz/k^2 + s\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(kz\ om\)\/k\^2 + \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi2[k_, kz_, om_] = om*kz/k^2 - s\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(kz\ om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Imaginary part:", "Subsubsection"],

Cell[BoxData[
    \(Imbeta[md2_, k_, kz_, om_, a_] := 
      If[om > k, 
        0, \(-md2\)/k^2*om/2*
          NIntegrate[
            1/\((1 + a\ xi^2)\)^2*\((om^2 + a*om*kz*xi)\)/
                Sqrt[\(-\((k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                              om^2)\)*\((1 - kz^2/k^2)\))\)\)], {xi, 
              xi2[k, kz, om], xi1[k, kz, om]}, WorkingPrecision \[Rule] 20, 
            MaxRecursion \[Rule] 20]]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imbeta[1, 1, 1/10, 1/2, 0]\)], "Input"],

Cell[BoxData[
    \(\(-0.196349540849362033939949`10.0784\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imbeta[1, 1, 1/2, 1/2, \(-9999\)/10000]\)], "Input"],

Cell[BoxData[
    \(\(-8.674039902721113543784988`10.2714\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi2[1, 1 - 10^\((\(-8\))\), 0.5]\)], "Input"],

Cell[BoxData[
    \(0.49987752051316703`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi1[1, 1 - 10^\((\(-8\))\), 0.5]\)], "Input"],

Cell[BoxData[
    \(0.5001224694868329`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImTable1 = {{\(-1.`\), \(-1.073786551519948016960269`10.1629\)}, \
{\(-0.9`\), \(-0.717131909230609767560375`10.0485\)}, {\(-0.8`\), \
\(-0.54303842484550195570444`10.0986\)}, {\(-0.7`\), \
\(-0.439143492661957801539551`10.0204\)}, {\(-0.6`\), \
\(-0.369893477013013984402209`10.0734\)}, {\(-0.5`\), \
\(-0.32037770710394398391424`10.1211\)}, {\(-0.4`\), \
\(-0.283201091205448075068668`10.046\)}, {\(-0.3`\), \
\(-0.254263145363604633304114`10.0892\)}, {\(-0.2`\), \
\(-0.231101600707046166682975`10.0059\)}, {\(-0.1`\), \
\(-0.212146645536784221228252`10.0432\)}, {0.`, \
\(-0.196349540849362033939949`10.0784\)}, {0.1`, \
\(-0.182982816014867721028435`10.0169\)}, {0.2`, \
\(-0.171525993249409711220583`10.0486\)}, {0.3`, \
\(-0.161596899039714659783067`10.0786\)}, {0.4`, \
\(-0.152908639423964553754563`10.1073\)}, {0.5`, \
\(-0.145241696560846885532921`10.1347\)}, {0.6`, \
\(-0.138425285211563476661952`10.1616\)}, {0.7`, \
\(-0.132324570643061034102251`10.0073\)}, {0.8`, \
\(-0.126831704959775532141805`10.0361\)}, {0.9`, \
\(-0.121859414481937335344693`10.0681\)}, {1.`, \
\(-0.117336329905540342527136`10.1066\)}}\)], "Input"],

Cell[BoxData[
    \({{\(-1.`\), \(-1.0737865515199480169602677426`10.1629\)}, {\(-0.9`\), \
\(-0.717131909230609767560373887`10.0485\)}, {\(-0.8`\), \
\(-0.5430384248455019557044394851`10.0986\)}, {\(-0.7`\), \
\(-0.4391434926619578015395504794`10.0204\)}, {\(-0.6`\), \
\(-0.3698934770130139844022077408`10.0734\)}, {\(-0.5`\), \
\(-0.3203777071039439839142394529`10.1211\)}, {\(-0.4`\), \
\(-0.2832010912054480750686671056`10.046\)}, {\(-0.3`\), \
\(-0.2542631453636046333041131189`10.0892\)}, {\(-0.2`\), \
\(-0.2311016007070461666829746655`10.0059\)}, {\(-0.1`\), \
\(-0.2121466455367842212282510715`10.0432\)}, {0.`, \
\(-0.1963495408493620339399481982`10.0784\)}, {0.1`, \
\(-0.1829828160148677210284343822`10.0169\)}, {0.2`, \
\(-0.1715259932494097112205823394`10.0486\)}, {0.3`, \
\(-0.1615968990397146597830663919`10.0786\)}, {0.4`, \
\(-0.1529086394239645537545623987`10.1073\)}, {0.5`, \
\(-0.1452416965608468855329205292`10.1347\)}, {0.6`, \
\(-0.1384252852115634766619514133`10.1616\)}, {0.7`, \
\(-0.1323245706430610341022502516`10.0073\)}, {0.8`, \
\(-0.1268317049597755321418044944`10.0361\)}, {0.9`, \
\(-0.1218594144819373353446925323`10.0681\)}, {1.`, \
\(-0.1173363299055403425271350397`10.1066\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImTable1 = 
      Table[{N[a], Imbeta[1, 1, 1/10, 1/2, a]}, {a, \(-1\), 1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \({{\(-1.`\), \(-1.073786551519948016960269`10.1629\)}, {\(-0.9`\), \
\(-0.717131909230609767560375`10.0485\)}, {\(-0.8`\), \
\(-0.54303842484550195570444`10.0986\)}, {\(-0.7`\), \
\(-0.439143492661957801539551`10.0204\)}, {\(-0.6`\), \
\(-0.369893477013013984402209`10.0734\)}, {\(-0.5`\), \
\(-0.32037770710394398391424`10.1211\)}, {\(-0.4`\), \
\(-0.283201091205448075068668`10.046\)}, {\(-0.3`\), \
\(-0.254263145363604633304114`10.0892\)}, {\(-0.2`\), \
\(-0.231101600707046166682975`10.0059\)}, {\(-0.1`\), \
\(-0.212146645536784221228252`10.0432\)}, {0.`, \
\(-0.196349540849362033939949`10.0784\)}, {0.1`, \
\(-0.182982816014867721028435`10.0169\)}, {0.2`, \
\(-0.171525993249409711220583`10.0486\)}, {0.3`, \
\(-0.161596899039714659783067`10.0786\)}, {0.4`, \
\(-0.152908639423964553754563`10.1073\)}, {0.5`, \
\(-0.145241696560846885532921`10.1347\)}, {0.6`, \
\(-0.138425285211563476661952`10.1616\)}, {0.7`, \
\(-0.132324570643061034102251`10.0073\)}, {0.8`, \
\(-0.126831704959775532141805`10.0361\)}, {0.9`, \
\(-0.121859414481937335344693`10.0681\)}, {1.`, \
\(-0.117336329905540342527136`10.1066\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImPlot1 = 
      ListPlot[ImTable1, PlotJoined \[Rule] True, 
        PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.47619 0.603319 0.548157 [
[.02381 .59082 -6 -9 ]
[.02381 .59082 6 0 ]
[.2619 .59082 -12 -9 ]
[.2619 .59082 12 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.4875 .05516 -12 -4.5 ]
[.4875 .05516 0 4.5 ]
[.4875 .16479 -24 -4.5 ]
[.4875 .16479 0 4.5 ]
[.4875 .27442 -24 -4.5 ]
[.4875 .27442 0 4.5 ]
[.4875 .38406 -24 -4.5 ]
[.4875 .38406 0 4.5 ]
[.4875 .49369 -24 -4.5 ]
[.4875 .49369 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60332 m
.02381 .60957 L
s
[(-1)] .02381 .59082 0 1 Mshowa
.2619 .60332 m
.2619 .60957 L
s
[(-0.5)] .2619 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.5)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(1)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.5 .05516 m
.50625 .05516 L
s
[(-1)] .4875 .05516 1 0 Mshowa
.5 .16479 m
.50625 .16479 L
s
[(-0.8)] .4875 .16479 1 0 Mshowa
.5 .27442 m
.50625 .27442 L
s
[(-0.6)] .4875 .27442 1 0 Mshowa
.5 .38406 m
.50625 .38406 L
s
[(-0.4)] .4875 .38406 1 0 Mshowa
.5 .49369 m
.50625 .49369 L
s
[(-0.2)] .4875 .49369 1 0 Mshowa
.125 Mabswid
.5 .08257 m
.50375 .08257 L
s
.5 .10998 m
.50375 .10998 L
s
.5 .13739 m
.50375 .13739 L
s
.5 .1922 m
.50375 .1922 L
s
.5 .21961 m
.50375 .21961 L
s
.5 .24702 m
.50375 .24702 L
s
.5 .30183 m
.50375 .30183 L
s
.5 .32924 m
.50375 .32924 L
s
.5 .35665 m
.50375 .35665 L
s
.5 .41146 m
.50375 .41146 L
s
.5 .43887 m
.50375 .43887 L
s
.5 .46628 m
.50375 .46628 L
s
.5 .5211 m
.50375 .5211 L
s
.5 .5485 m
.50375 .5485 L
s
.5 .57591 m
.50375 .57591 L
s
.5 .02775 m
.50375 .02775 L
s
.5 .00035 m
.50375 .00035 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.02381 .01472 m
.07143 .21022 L
.11905 .30565 L
.16667 .3626 L
.21429 .40056 L
.2619 .4277 L
.30952 .44808 L
.35714 .46394 L
.40476 .47664 L
.45238 .48703 L
.5 .49569 L
.54762 .50302 L
.59524 .5093 L
.64286 .51474 L
.69048 .5195 L
.7381 .5237 L
.78571 .52744 L
.83333 .53078 L
.88095 .5338 L
.92857 .53652 L
.97619 .539 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool2
000008l0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0
oooo002?0?ooo`030000003oool0oooo08h0oooo00070?ooo`030000o`3oool0oooo08D0oooo00<0
00000?ooo`3oool0SP3oool000L0oooo00<0003o0?ooo`3oool0Q@3oool00`000000oooo0?ooo`2>
0?ooo`001`3oool00`000?l0oooo0?ooo`250?ooo`030000003oool0oooo08h0oooo00080?ooo`03
0000o`3oool0oooo08@0oooo00<000000?ooo`3oool0SP3oool000P0oooo00<0003o0?ooo`3oool0
Q03oool2000008l0oooo00080?ooo`030000o`3oool0oooo08@0oooo00<000000?ooo`3oool0SP3o
ool000P0oooo00<0003o0?ooo`3oool0Q03oool00`000000oooo0?ooo`2>0?ooo`002@3oool00`00
0?l0oooo0?ooo`230?ooo`030000003oool0oooo08h0oooo00090?ooo`030000o`3oool0oooo08<0
oooo00<000000?ooo`3oool0SP3oool000T0oooo00<0003o0?ooo`3oool0NP3oool4000000D0oooo
00<000000?ooo`3oool0SP3oool000T0oooo00<0003o0?ooo`3oool0O03oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000o`3oool0oooo07/0oooo00<00000
0?ooo`3oool0103oool3000008h0oooo000:0?ooo`030000o`3oool0oooo07<0oooo100000040?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool000X0oooo00<0003o0?ooo`3o
ool0N`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000
o`3oool0oooo07T0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000o`3o
ool0oooo07/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`002`3oool0
0`000?l0oooo0?ooo`210?ooo`030000003oool0oooo08h0oooo000;0?ooo`030000o`3oool0oooo
0840oooo00<000000?ooo`3oool0SP3oool000/0oooo00<0003o0?ooo`3oool0P@3oool00`000000
oooo0?ooo`2>0?ooo`002`3oool00`000?l0oooo0?ooo`210?ooo`800000S`3oool000`0oooo00<0
003o0?ooo`3oool0P03oool00`000000oooo0?ooo`2>0?ooo`00303oool00`000?l0oooo0?ooo`20
0?ooo`030000003oool0oooo08h0oooo000<0?ooo`030000o`3oool0oooo0800oooo00<000000?oo
o`3oool0SP3oool000`0oooo00<0003o0?ooo`3oool0P03oool00`000000oooo0?ooo`2>0?ooo`00
3@3oool00`000?l0oooo0?ooo`1o0?ooo`030000003oool0oooo08h0oooo000=0?ooo`030000o`3o
ool0oooo07l0oooo00<000000?ooo`3oool0SP3oool000d0oooo00<0003o0?ooo`3oool0O`3oool0
0`000000oooo0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1o0?ooo`800000S`3oool000d0
oooo00<0003o0?ooo`3oool0O`3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo
0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<0
00000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`000000oooo0?ooo`2>
0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000?0?ooo`03
0000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0
O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`800000S`3o
ool000l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?ooo`00403oool00`00
0?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo07`0
oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool00`000000oooo
0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000A
0?ooo`030000o`3oool0oooo06L0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000006
0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo06H0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo000A
0?ooo`030000o`3oool0oooo06H0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000050?ooo`<00000SP3oool00140oooo00<0003o0?ooo`3oool0H03oool400000080oooo00@0
00000?ooo`3oool000002@3oool2000000H0oooo00<000000?ooo`3oool0SP3oool00140oooo00<0
003o0?ooo`3oool0IP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool0SP3oool00180oooo00<0003o0?ooo`3oool0I@3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0SP3oool00180
oooo00<0003o0?ooo`3oool0IP3oool2000000X0oooo0P0000060?ooo`030000003oool0oooo08h0
oooo000B0?ooo`030000o`3oool0oooo07X0oooo00<000000?ooo`3oool0SP3oool00180oooo00<0
003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo0P00002?0?oo
o`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000
o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0N03o
ool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`1h0?ooo`030000003oool0
oooo08h0oooo000D0?ooo`030000o`3oool0oooo07P0oooo00<000000?ooo`3oool0SP3oool001D0
oooo00<0003o0?ooo`3oool0M`3oool00`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo
0?ooo`1g0?ooo`800000S`3oool001H0oooo00<0003o0?ooo`3oool0MP3oool00`000000oooo0?oo
o`2>0?ooo`005P3oool00`000?l0oooo0?ooo`1f0?ooo`030000003oool0oooo08h0oooo000G0?oo
o`030000o`3oool0oooo07D0oooo00<000000?ooo`3oool0SP3oool001L0oooo00<0003o0?ooo`3o
ool0M@3oool00`000000oooo0?ooo`2>0?ooo`00603oool00`000?l0oooo0?ooo`1d0?ooo`030000
003oool0oooo08h0oooo000H0?ooo`030000o`3oool0oooo07@0oooo00<000000?ooo`3oool0SP3o
ool001T0oooo00<0003o0?ooo`3oool0L`3oool00`000000oooo0?ooo`2>0?ooo`006@3oool00`00
0?l0oooo0?ooo`1c0?ooo`800000S`3oool001X0oooo00<0003o0?ooo`3oool0LP3oool00`000000
oooo0?ooo`2>0?ooo`006P3oool00`000?l0oooo0?ooo`1b0?ooo`030000003oool0oooo08h0oooo
000K0?ooo`030000o`3oool0oooo0740oooo00<000000?ooo`3oool0SP3oool001/0oooo00<0003o
0?ooo`3oool0L@3oool00`000000oooo0?ooo`2>0?ooo`00703oool00`000?l0oooo0?ooo`1`0?oo
o`030000003oool0oooo08h0oooo000L0?ooo`030000o`3oool0oooo05`0oooo0P0000040?ooo`03
0000003oool0oooo00<0oooo0P0000060?ooo`030000003oool0oooo08h0oooo000M0?ooo`030000
o`3oool0oooo05X0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?oo
o`030000003oool0oooo08h0oooo000M0?ooo`030000o`3oool0oooo05X0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000050?ooo`<00000SP3oool001h0oooo00<0003o0?oo
o`3oool0D`3oool400000080oooo00@000000?ooo`3oool00000203oool3000000H0oooo00<00000
0?ooo`3oool0SP3oool001h0oooo00<0003o0?ooo`3oool0F@3oool010000000oooo0?ooo`000008
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool001l0oooo00<0003o0?oo
o`3oool0F03oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0SP3oool001l0oooo00<0003o0?ooo`3oool0F@3oool2000000X0oooo0`0000050?oo
o`030000003oool0oooo08h0oooo000P0?ooo`030000o`3oool0oooo06`0oooo00<000000?ooo`3o
ool0SP3oool00200oooo00<0003o0?ooo`3oool0K03oool00`000000oooo0?ooo`2>0?ooo`008@3o
ool00`000?l0oooo0?ooo`1[0?ooo`030000003oool0oooo08h0oooo000Q0?ooo`030000o`3oool0
oooo06/0oooo0P00002?0?ooo`008P3oool00`000?l0oooo0?ooo`1Z0?ooo`030000003oool0oooo
08h0oooo000S0?ooo`030000o`3oool0oooo06T0oooo00<000000?ooo`3oool0SP3oool002@0oooo
00<0003o0?ooo`3oool0J03oool00`000000oooo0?ooo`2>0?ooo`009@3oool00`000?l0oooo0?oo
o`1W0?ooo`030000003oool0oooo08h0oooo000U0?ooo`030000o`3oool0oooo06L0oooo00<00000
0?ooo`3oool0SP3oool002H0oooo00<0003o0?ooo`3oool0IP3oool00`000000oooo0?ooo`2>0?oo
o`009`3oool00`000?l0oooo0?ooo`1U0?ooo`030000003oool0oooo08h0oooo000X0?ooo`030000
o`3oool0oooo06@0oooo0P00002?0?ooo`00:@3oool00`000?l0oooo0?ooo`1S0?ooo`030000003o
ool0oooo08h0oooo000Z0?ooo`030000o`3oool0oooo0680oooo00<000000?ooo`3oool0SP3oool0
02/0oooo00<0003o0?ooo`3oool0H@3oool00`000000oooo0?ooo`2>0?ooo`00;03oool00`000?l0
oooo0?ooo`1P0?ooo`030000003oool0oooo08h0oooo000/0?ooo`030000o`3oool0oooo0600oooo
00<000000?ooo`3oool0SP3oool002d0oooo00<0003o0?ooo`3oool0G`3oool00`000000oooo0?oo
o`2>0?ooo`00;P3oool00`000?l0oooo0?ooo`1N0?ooo`030000003oool0oooo08h0oooo000_0?oo
o`030000o`3oool0oooo05d0oooo0P00002?0?ooo`00<03oool00`000?l0oooo0?ooo`1L0?ooo`03
0000003oool0oooo08h0oooo000a0?ooo`030000o`3oool0oooo05/0oooo00<000000?ooo`3oool0
SP3oool00380oooo00<0003o0?ooo`3oool0FP3oool00`000000oooo0?ooo`2>0?ooo`00<`3oool2
0000oeX0oooo00<000000?ooo`3oool0SP3oool003D0oooo00<0003o0?ooo`3oool0E`3oool00`00
0000oooo0?ooo`2>0?ooo`00=P3oool00`000?l0oooo0?ooo`120?ooo`800000103oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool003L0oooo
00<0003o0?ooo`3oool0@03oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0SP3oool003P0oooo00<0003o0?ooo`3oool0?`3oool010000000oooo
0?ooo`0000070?ooo`D000001@3oool3000008h0oooo000i0?ooo`80003o>@3oool400000080oooo
00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`0000060?ooo`030000003oool0oooo
08h0oooo000k0?ooo`030000o`3oool0oooo03`0oooo00@000000?ooo`3oool00000203oool00`00
0000oooo000000060?ooo`030000003oool0oooo08h0oooo000l0?ooo`030000o`3oool0oooo03/0
oooo00@000000?ooo`3oool000002@3oool2000000H0oooo00<000000?ooo`3oool0SP3oool003d0
oooo00<0003o0?ooo`3oool0>`3oool2000000/0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`2>0?ooo`00?P3oool20000odl0oooo00<000000?ooo`3oool0SP3oool00400oooo0P00
0?m=0?ooo`030000003oool0oooo08h0oooo00120?ooo`80003oB`3oool2000008l0oooo00140?oo
o`030000o`3oool0oooo04P0oooo00<000000?ooo`3oool0SP3oool004D0oooo0P000?m80?ooo`03
0000003oool0oooo08h0oooo00170?ooo`80003oAP3oool00`000000oooo0?ooo`2>0?ooo`00B@3o
ool20000od@0oooo00<000000?ooo`3oool0SP3oool004/0oooo0P000?m20?ooo`030000003oool0
oooo08h0oooo001=0?ooo`80003o@03oool00`000000oooo0?ooo`2>0?ooo`00C`3oool20000och0
oooo00<000000?ooo`3oool0SP3oool00540oooo0P000?ll0?ooo`800000S`3oool005<0oooo0P00
0?lj0?ooo`030000003oool0oooo08h0oooo001E0?ooo`80003o>03oool00`000000oooo0?ooo`2>
0?ooo`00E`3oool30000ocD0oooo00<000000?ooo`3oool0SP3oool005X0oooo10000?la0?ooo`03
0000003oool0oooo08h0oooo001N0?ooo`<0003o;P3oool00`000000oooo0?ooo`2>0?ooo`00H@3o
ool40000obX0oooo00<000000?ooo`3oool0SP3oool006D0oooo0`000?lW0?ooo`030000003oool0
oooo08h0oooo001X0?ooo`@0003o8`3oool2000008l0oooo001/0?ooo`<0003o803oool00`000000
oooo0?ooo`2>0?ooo`00K`3oool40000oa`0oooo00<000000?ooo`3oool0SP3oool007<0oooo1000
0?lH0?ooo`030000003oool0oooo08h0oooo001g0?ooo`@0003o503oool00`000000oooo0?ooo`2>
0?ooo`00N`3oool40000oa00oooo00<000000?ooo`3oool0SP3oool007/0oooo0P0000020?ooo`H0
003o00<0oooo0000000000000P0000050?ooo`030000003oool0oooo08h0oooo001j0?ooo`040000
003oool0oooo000000L0oooo1`000?l30?ooo`030000003oool0oooo08h0oooo001j0?ooo`040000
003oool0oooo000000T0oooo00<000000?ooo`3oool00P3oool60000ohh0oooo001d0?ooo`@00000
0P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool010000?n:0?ooo`00NP3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0103oool50000ohD0oooo001j0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`090?ooo`T0003oO03oool007/0
oooo0P00000:0?ooo`8000001P3oool00`000000oooo0?ooo`0B0?ooo`X0003oLP3oool008l0oooo
00<000000?ooo`3oool0703oool70000of/0oooo002?0?ooo`030000003oool0oooo02<0oooo2P00
0?mQ0?ooo`00S`3oool00`000000oooo0?ooo`0]0?ooo`h0003oD`3oool008l0oooo0P00000l0?oo
o`X0003oB@3oool008l0oooo00<000000?ooo`3oool0A@3oool70000od80oooo002?0?ooo`030000
003oool0oooo04`0oooo2P000?lh0?ooo`00S`3oool00`000000oooo0?ooo`1F0?oooa/0003o7@3o
ool008l0oooo00<000000?ooo`3oool0L@3oool>0000o`l0oooo002?0?ooo`030000003oool0oooo
07l0oooo1`000?l80?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo
0?ooo`2>0?ooo`00S`3oool2000008l0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?oo
o`030000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?ooo`030000
003oool0oooo08h0oooo00080?ooo`@00000>`3oool2000000@0oooo00<000000?ooo`3oool00P3o
ool3000003X0oooo00<000000?ooo`3oool0>P3oool2000000@0oooo00<000000?ooo`3oool00P3o
ool3000003/0oooo100000070?ooo`002P3oool00`000000oooo0?ooo`0i0?ooo`040000003oool0
oooo000000/0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0i0?ooo`040000003o
ool0oooo000000/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`060?ooo`002P3o
ool00`000000oooo0?ooo`0i0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0
=`3oool2000003X0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0j0?ooo`03
0000003oool0oooo00H0oooo00020?ooo`@00000103oool00`000000oooo0?ooo`0c0?ooo`@00000
0P3oool010000000oooo0?ooo`0000080?ooo`<00000>P3oool00`000000oooo0?ooo`0i0?ooo`04
0000003oool0oooo000000P0oooo0`00000m0?ooo`030000003oool0oooo00H0oooo000:0?ooo`03
0000003oool0oooo03T0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0j0?oo
o`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0m
0?ooo`030000003oool0oooo00H0oooo00080?ooo`<00000>`3oool010000000oooo0?ooo`000008
0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00
00080?ooo`030000003oool0oooo03/0oooo0`0000080?ooo`002P3oool00`000000oooo0?ooo`0j
0?ooo`8000002@3oool4000003T0oooo00<000000?ooo`3oool0>P3oool2000000T0oooo1000000l
0?ooo`030000003oool0oooo00H0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?ooo`03
0000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0oooo003o0000020000000@3o
ool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo00H0oooo00070?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0@@3oool00`00
0000oooo0?ooo`110?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool01P3oool008l0
oooo00<000000?ooo`3oool0SP3oool008l0oooo00<000000?ooo`3oool0SP3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05261, -1.10064, \
0.00733523, 0.00637219}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImTable2 = 
      Table[{N[a], Imbeta[1, 1, 1/2, 1/2, a]}, {a, \(-999\)/1000, 1.1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \({{\(-0.999`\), \(-2.860524368642120639054122`10.0106\)}, {\(-0.899`\), \
\(-0.432034080257451477420205`10.0393\)}, {\(-0.799`\), \
\(-0.346552904266502765110933`10.0641\)}, {\(-0.699`\), \
\(-0.304738762460353951463336`10.1055\)}, {\(-0.599`\), \
\(-0.277358766784470119797293`10.0064\)}, {\(-0.499`\), \
\(-0.256985606571515109975389`10.0506\)}, {\(-0.399`\), \
\(-0.240722261627795548489162`10.0869\)}, {\(-0.299`\), \
\(-0.227162932950256256676918`10.1187\)}, {\(-0.199`\), \
\(-0.215525278300790036451373`10.026\)}, {\(-0.099`\), \
\(-0.205330388601745866374536`10.0526\)}, {0.001`, \
\(-0.196263683243215616679117`10.0787\)}, {0.101`, \
\(-0.188106982819241884488587`10.0094\)}, {0.201`, \
\(-0.180702304255035534311754`10.0346\)}, {0.301`, \
\(-0.173931191995153162499663`10.0593\)}, {0.401`, \
\(-0.167702244279362470835532`10.0838\)}, {0.501`, \
\(-0.161943230515874529754926`10.1082\)}, {0.601`, \
\(-0.156595912554989985679441`10.1325\)}, {0.701`, \
\(-0.151612527190030250152938`10.1568\)}, {0.801`, \
\(-0.146953327029255176492311`10.0169\)}, {0.901`, \
\(-0.142584817290302821245438`10.0464\)}, {1.001`, \
\(-0.138478463068034593502415`10.0757\)}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImTable3 = 
      Table[{N[a], Imbeta[1, 1, 9/10, 1/2, a]}, {a, \(-1\), 1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \({{\(-1.`\), 0.107065597848344666571704`10.1225}, {\(-0.9`\), 
        0.005035399592787649682243`10.1728}, {\(-0.8`\), \
\(-0.057534099252069252201751`10.1289\)}, {\(-0.7`\), \
\(-0.098681900949153051439845`10.0599\)}, {\(-0.6`\), \
\(-0.127162490723953879734668`10.0257\)}, {\(-0.5`\), \
\(-0.1476626582748081584129`10.0969\)}, {\(-0.4`\), \
\(-0.162883243073362010837974`10.0742\)}, {\(-0.3`\), \
\(-0.174472583237544388831483`10.0676\)}, {\(-0.2`\), \
\(-0.183484063969736635315817`10.0682\)}, {\(-0.1`\), \
\(-0.190616773135724450444743`10.0724\)}, {0.`, \
\(-0.196349540849362033939949`10.0784\)}, {0.1`, \
\(-0.201019230952628456140194`10.0853\)}, {0.2`, \
\(-0.204868342185840167558684`10.0926\)}, {0.3`, \
\(-0.208074960392804073852137`10.003\)}, {0.4`, \
\(-0.210772173959062577675358`10.0091\)}, {0.5`, \
\(-0.213060990547276101597965`10.0149\)}, {0.6`, \
\(-0.215019129639048594997974`10.0203\)}, {0.7`, \
\(-0.216707130896138652668383`10.0254\)}, {0.8`, \
\(-0.218172675832081406050896`10.0301\)}, {0.9`, \
\(-0.219453695976303039916092`10.0345\)}, {1.`, \
\(-0.220580641708328771826902`10.0386\)}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImPlot3 = 
      ListPlot[ImTable3, PlotJoined \[Rule] True, 
        PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.47619 0.41098 1.79646 [
[.02381 .39848 -6 -9 ]
[.02381 .39848 6 0 ]
[.2619 .39848 -12 -9 ]
[.2619 .39848 12 0 ]
[.7381 .39848 -9 -9 ]
[.7381 .39848 9 0 ]
[.97619 .39848 -3 -9 ]
[.97619 .39848 3 0 ]
[.4875 .05169 -24 -4.5 ]
[.4875 .05169 0 4.5 ]
[.4875 .14151 -30 -4.5 ]
[.4875 .14151 0 4.5 ]
[.4875 .23133 -24 -4.5 ]
[.4875 .23133 0 4.5 ]
[.4875 .32116 -30 -4.5 ]
[.4875 .32116 0 4.5 ]
[.4875 .5008 -24 -4.5 ]
[.4875 .5008 0 4.5 ]
[.4875 .59063 -18 -4.5 ]
[.4875 .59063 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .41098 m
.02381 .41723 L
s
[(-1)] .02381 .39848 0 1 Mshowa
.2619 .41098 m
.2619 .41723 L
s
[(-0.5)] .2619 .39848 0 1 Mshowa
.7381 .41098 m
.7381 .41723 L
s
[(0.5)] .7381 .39848 0 1 Mshowa
.97619 .41098 m
.97619 .41723 L
s
[(1)] .97619 .39848 0 1 Mshowa
.125 Mabswid
.07143 .41098 m
.07143 .41473 L
s
.11905 .41098 m
.11905 .41473 L
s
.16667 .41098 m
.16667 .41473 L
s
.21429 .41098 m
.21429 .41473 L
s
.30952 .41098 m
.30952 .41473 L
s
.35714 .41098 m
.35714 .41473 L
s
.40476 .41098 m
.40476 .41473 L
s
.45238 .41098 m
.45238 .41473 L
s
.54762 .41098 m
.54762 .41473 L
s
.59524 .41098 m
.59524 .41473 L
s
.64286 .41098 m
.64286 .41473 L
s
.69048 .41098 m
.69048 .41473 L
s
.78571 .41098 m
.78571 .41473 L
s
.83333 .41098 m
.83333 .41473 L
s
.88095 .41098 m
.88095 .41473 L
s
.92857 .41098 m
.92857 .41473 L
s
.25 Mabswid
0 .41098 m
1 .41098 L
s
.5 .05169 m
.50625 .05169 L
s
[(-0.2)] .4875 .05169 1 0 Mshowa
.5 .14151 m
.50625 .14151 L
s
[(-0.15)] .4875 .14151 1 0 Mshowa
.5 .23133 m
.50625 .23133 L
s
[(-0.1)] .4875 .23133 1 0 Mshowa
.5 .32116 m
.50625 .32116 L
s
[(-0.05)] .4875 .32116 1 0 Mshowa
.5 .5008 m
.50625 .5008 L
s
[(0.05)] .4875 .5008 1 0 Mshowa
.5 .59063 m
.50625 .59063 L
s
[(0.1)] .4875 .59063 1 0 Mshowa
.125 Mabswid
.5 .06965 m
.50375 .06965 L
s
.5 .08762 m
.50375 .08762 L
s
.5 .10558 m
.50375 .10558 L
s
.5 .12355 m
.50375 .12355 L
s
.5 .15948 m
.50375 .15948 L
s
.5 .17744 m
.50375 .17744 L
s
.5 .1954 m
.50375 .1954 L
s
.5 .21337 m
.50375 .21337 L
s
.5 .2493 m
.50375 .2493 L
s
.5 .26726 m
.50375 .26726 L
s
.5 .28523 m
.50375 .28523 L
s
.5 .30319 m
.50375 .30319 L
s
.5 .33912 m
.50375 .33912 L
s
.5 .35709 m
.50375 .35709 L
s
.5 .37505 m
.50375 .37505 L
s
.5 .39302 m
.50375 .39302 L
s
.5 .42894 m
.50375 .42894 L
s
.5 .44691 m
.50375 .44691 L
s
.5 .46487 m
.50375 .46487 L
s
.5 .48284 m
.50375 .48284 L
s
.5 .51877 m
.50375 .51877 L
s
.5 .53673 m
.50375 .53673 L
s
.5 .5547 m
.50375 .5547 L
s
.5 .57266 m
.50375 .57266 L
s
.5 .03372 m
.50375 .03372 L
s
.5 .01576 m
.50375 .01576 L
s
.5 .60859 m
.50375 .60859 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.02381 .60332 m
.07143 .42003 L
.11905 .30762 L
.16667 .2337 L
.21429 .18254 L
.2619 .14571 L
.30952 .11837 L
.35714 .09755 L
.40476 .08136 L
.45238 .06854 L
.5 .05825 L
.54762 .04986 L
.59524 .04294 L
.64286 .03718 L
.69048 .03234 L
.7381 .02822 L
.78571 .02471 L
.83333 .02167 L
.88095 .01904 L
.92857 .01674 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool200000780oooo5@00
0?l80?ooo`00S`3oool00`000000oooo0?ooo`1S0?ooo`h0003o7@3oool008l0oooo00<000000?oo
o`3oool0EP3oool=0000ob/0oooo002?0?ooo`030000003oool0oooo04P0oooo3P000?lh0?ooo`00
S`3oool00`000000oooo0?ooo`0k0?ooo`d0003oAP3oool008l0oooo0P00000^0?ooo`h0003oD`3o
ool008l0oooo00<000000?ooo`3oool08`3oool:0000of40oooo002?0?ooo`030000003oool0oooo
01`0oooo1`000?m[0?ooo`00N`3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000D0
oooo00<000000?ooo`3oool05@3oool70000og80oooo001j0?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`H0003oN@3oool007X0oooo
00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`050?ooo`<00000203oool70000ogl0
oooo001d0?ooo`@000000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0
oooo00@000000?ooo`3oool0oooo1`000?n60?ooo`00NP3oool010000000oooo0?ooo`00000;0?oo
o`040000003oool0oooo0?ooo`H0003oS@3oool007X0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo0000o`040000o`80oooo00<000000?ooo`3oool0SP3oool007/0oooo0P0000070?oo
o`@0003o00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`00P03oool40000o`/0
oooo00<000000?ooo`3oool0SP3oool007d0oooo0`000?l?0?ooo`800000S`3oool007T0oooo1000
0?lB0?ooo`030000003oool0oooo08h0oooo001f0?ooo`<0003o5P3oool00`000000oooo0?ooo`2>
0?ooo`00L`3oool30000oaT0oooo00<000000?ooo`3oool0SP3oool006l0oooo10000?lL0?ooo`03
0000003oool0oooo08h0oooo001/0?ooo`<0003o803oool2000008l0oooo001X0?ooo`@0003o8`3o
ool00`000000oooo0?ooo`2>0?ooo`00I@3oool30000obL0oooo00<000000?ooo`3oool0SP3oool0
06<0oooo0P000?lZ0?ooo`030000003oool0oooo08h0oooo001Q0?ooo`80003o;03oool00`000000
oooo0?ooo`2>0?ooo`00GP3oool30000obh0oooo0P00002?0?ooo`00G03oool20000oc40oooo00<0
00000?ooo`3oool0SP3oool005X0oooo0P000?lc0?ooo`030000003oool0oooo08h0oooo001H0?oo
o`80003o=@3oool00`000000oooo0?ooo`2>0?ooo`00EP3oool20000ocL0oooo00<000000?ooo`3o
ool0SP3oool005@0oooo0P000?li0?ooo`800000S`3oool005<0oooo00<0003o0?ooo`3oool0>@3o
ool00`000000oooo0?ooo`2>0?ooo`00D@3oool20000oc`0oooo00<000000?ooo`3oool0SP3oool0
0500oooo00<0003o0?ooo`3oool08P3oool2000000@0oooo00<000000?ooo`3oool00P3oool40000
0080oooo0`0000060?ooo`030000003oool0oooo08h0oooo001>0?ooo`80003o903oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`2>0?ooo`00C03oool20000obH0oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo0`00002>0?ooo`00B`3oool00`000?l0
oooo0?ooo`0P0?ooo`@000000P3oool010000000oooo0?ooo`00000:0?ooo`040000003oool0oooo
0?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`00BP3oool00`000?l0oooo0?ooo`0W0?oo
o`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo000000080?ooo`030000003o
ool0oooo08h0oooo00180?ooo`80003o:P3oool010000000oooo0?ooo`0000080?ooo`<000000`3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo00170?ooo`030000o`3oool0
oooo02/0oooo0P00000;0?ooo`040000003oool0oooo0?ooo`@000001@3oool00`000000oooo0?oo
o`2>0?ooo`00AP3oool00`000?l0oooo0?ooo`160?ooo`800000S`3oool004@0oooo0P000?m90?oo
o`030000003oool0oooo08h0oooo00130?ooo`030000o`3oool0oooo04T0oooo00<000000?ooo`3o
ool0SP3oool00480oooo00<0003o0?ooo`3oool0BP3oool00`000000oooo0?ooo`2>0?ooo`00@@3o
ool00`000?l0oooo0?ooo`1;0?ooo`030000003oool0oooo08h0oooo000o0?ooo`80003oCP3oool2
000008l0oooo000n0?ooo`030000o`3oool0oooo04h0oooo00<000000?ooo`3oool0SP3oool003d0
oooo00<0003o0?ooo`3oool0C`3oool00`000000oooo0?ooo`2>0?ooo`00?03oool00`000?l0oooo
0?ooo`1@0?ooo`030000003oool0oooo08h0oooo000k0?ooo`030000o`3oool0oooo0540oooo00<0
00000?ooo`3oool0SP3oool003X0oooo00<0003o0?ooo`3oool0DP3oool00`000000oooo0?ooo`2>
0?ooo`00>@3oool00`000?l0oooo0?ooo`1C0?ooo`800000S`3oool003P0oooo00<0003o0?ooo`3o
ool0E03oool00`000000oooo0?ooo`2>0?ooo`00=`3oool00`000?l0oooo0?ooo`1E0?ooo`030000
003oool0oooo08h0oooo000g0?ooo`030000o`3oool0oooo05D0oooo00<000000?ooo`3oool0SP3o
ool003H0oooo00<0003o0?ooo`3oool0EP3oool00`000000oooo0?ooo`2>0?ooo`00=@3oool00`00
0?l0oooo0?ooo`1G0?ooo`800000S`3oool003@0oooo00<0003o0?ooo`3oool0F03oool00`000000
oooo0?ooo`2>0?ooo`00<`3oool00`000?l0oooo0?ooo`1I0?ooo`030000003oool0oooo08h0oooo
000b0?ooo`030000o`3oool0oooo04H0oooo0P0000040?ooo`030000003oool0oooo0080oooo1000
00050?ooo`030000003oool0oooo08h0oooo000a0?ooo`030000o`3oool0oooo04H0oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000`
0?ooo`030000o`3oool0oooo04L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?oo
o`040?ooo`<00000SP3oool002l0oooo00<0003o0?ooo`3oool0@P3oool400000080oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000_
0?ooo`030000o`3oool0oooo04P0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo08h0oooo000^0?ooo`030000o`3oool0oooo04T0oooo00@00000
0?ooo`3oool00000203oool3000000H0oooo00<000000?ooo`3oool0SP3oool002d0oooo00<0003o
0?ooo`3oool0B`3oool2000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>
0?ooo`00;@3oool00`000?l0oooo0?ooo`1O0?ooo`800000S`3oool002`0oooo00<0003o0?ooo`3o
ool0H03oool00`000000oooo0?ooo`2>0?ooo`00;03oool00`000?l0oooo0?ooo`1P0?ooo`030000
003oool0oooo08h0oooo000[0?ooo`030000o`3oool0oooo0640oooo00<000000?ooo`3oool0SP3o
ool002X0oooo00<0003o0?ooo`3oool0HP3oool00`000000oooo0?ooo`2>0?ooo`00:P3oool00`00
0?l0oooo0?ooo`1R0?ooo`800000S`3oool002T0oooo00<0003o0?ooo`3oool0H`3oool00`000000
oooo0?ooo`2>0?ooo`00:03oool00`000?l0oooo0?ooo`1T0?ooo`030000003oool0oooo08h0oooo
000X0?ooo`030000o`3oool0oooo06@0oooo00<000000?ooo`3oool0SP3oool002L0oooo00<0003o
0?ooo`3oool0I@3oool00`000000oooo0?ooo`2>0?ooo`009P3oool00`000?l0oooo0?ooo`1V0?oo
o`800000S`3oool002H0oooo00<0003o0?ooo`3oool0IP3oool00`000000oooo0?ooo`2>0?ooo`00
9@3oool00`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo08h0oooo000U0?ooo`030000o`3o
ool0oooo06L0oooo00<000000?ooo`3oool0SP3oool002@0oooo00<0003o0?ooo`3oool0J03oool0
0`000000oooo0?ooo`2>0?ooo`008`3oool00`000?l0oooo0?ooo`1Y0?ooo`800000S`3oool002<0
oooo00<0003o0?ooo`3oool0J@3oool00`000000oooo0?ooo`2>0?ooo`008P3oool00`000?l0oooo
0?ooo`1Z0?ooo`030000003oool0oooo08h0oooo000R0?ooo`030000o`3oool0oooo06X0oooo00<0
00000?ooo`3oool0SP3oool00240oooo00<0003o0?ooo`3oool0D@3oool2000000@0oooo00<00000
0?ooo`3oool00`3oool2000000<0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000Q0?oo
o`030000o`3oool0oooo0500oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0SP3oool00200oooo00<0003o
0?ooo`3oool0D@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo
00<000000?ooo`3oool00`3oool3000008h0oooo000P0?ooo`030000o`3oool0oooo04/0oooo1000
00020?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool3000000H0
oooo00<000000?ooo`3oool0SP3oool001l0oooo00<0003o0?ooo`3oool0DP3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`2>0?ooo`007`3oool00`000?l0oooo0?ooo`1B0?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo08h0oooo000O0?ooo`030000o`3oool0oooo05<0oooo0P00000:0?ooo`8000000`3oool40000
00D0oooo00<000000?ooo`3oool0SP3oool001h0oooo00<0003o0?ooo`3oool0KP3oool2000008l0
oooo000N0?ooo`030000o`3oool0oooo06h0oooo00<000000?ooo`3oool0SP3oool001d0oooo00<0
003o0?ooo`3oool0K`3oool00`000000oooo0?ooo`2>0?ooo`007@3oool00`000?l0oooo0?ooo`1_
0?ooo`030000003oool0oooo08h0oooo000L0?ooo`030000o`3oool0oooo0700oooo00<000000?oo
o`3oool0SP3oool001`0oooo00<0003o0?ooo`3oool0L03oool2000008l0oooo000K0?ooo`030000
o`3oool0oooo0740oooo00<000000?ooo`3oool0SP3oool001/0oooo00<0003o0?ooo`3oool0L@3o
ool00`000000oooo0?ooo`2>0?ooo`006`3oool00`000?l0oooo0?ooo`1a0?ooo`030000003oool0
oooo08h0oooo000J0?ooo`030000o`3oool0oooo0780oooo00<000000?ooo`3oool0SP3oool000P0
oooo1000000>0?ooo`030000o`3oool0oooo02X0oooo0P0000040?ooo`030000003oool0oooo0080
oooo0`00000j0?ooo`800000>`3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000003/0
oooo100000070?ooo`002P3oool00`000000oooo0?ooo`0<0?ooo`030000o`3oool0oooo02X0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo03T0
oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo
00H0oooo000:0?ooo`030000003oool0oooo00`0oooo00<0003o0?ooo`3oool0:P3oool010000000
oooo0?ooo`00000;0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0>@3oool01000
0000oooo0?ooo`00000;0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool01P3oool0
0080oooo100000040?ooo`030000003oool0oooo00/0oooo00<0003o0?ooo`3oool09@3oool40000
0080oooo00@000000?ooo`3oool00000203oool3000003X0oooo00<000000?ooo`3oool0>@3oool0
10000000oooo0?ooo`0000080?ooo`<00000?@3oool00`000000oooo0?ooo`060?ooo`002P3oool0
0`000000oooo0?ooo`0;0?ooo`030000o`3oool0oooo02/0oooo00@000000?ooo`3oool00000203o
ool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000
203oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00H0oooo00080?ooo`<000003@3o
ool00`000?l0oooo0?ooo`0[0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0
>P3oool2000003X0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0k0?ooo`<0
0000203oool000X0oooo00<000000?ooo`3oool02P3oool00`000?l0oooo0?ooo`0]0?ooo`800000
2@3oool4000003T0oooo00<000000?ooo`3oool0>P3oool2000000T0oooo1000000l0?ooo`030000
003oool0oooo00H0oooo000G0?ooo`030000o`3oool0oooo07D0oooo00<000000?ooo`3oool0SP3o
ool001H0oooo00<0003o0?ooo`3oool0MP3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`00
0?l0oooo0?ooo`1f0?ooo`030000003oool0oooo08h0oooo000E000000030000o`00000000000?l0
0000200000010?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`03000000000?l0oooo00/0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool01P3oool000L0oooo00<000000?ooo`3oool02P3oool00`000?l0oooo0?oo
o`0d0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?oo
o`110?ooo`030000003oool0oooo00H0oooo000D0?ooo`030000o`3oool0oooo07P0oooo00<00000
0?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`2>0?oo
o`00503oool00`000?l0oooo0?ooo`1h0?ooo`800000S`3oool001<0oooo00<0003o0?ooo`3oool0
N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003o
ool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool0
01<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004P3oool00`000?l0
oooo0?ooo`1j0?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000o`3oool0oooo07X0oooo
0P00002?0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`030000003oool0oooo08h0oooo000B
0?ooo`030000o`3oool0oooo07X0oooo00<000000?ooo`3oool0SP3oool00140oooo00<0003o0?oo
o`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`03
0000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo0P00002?0?ooo`004@3o
ool00`000?l0oooo0?ooo`1k0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0
oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool00`00
0000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0
oooo000@0?ooo`030000o`3oool0oooo07`0oooo0P00002?0?ooo`003`3oool00`000?l0oooo0?oo
o`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<00000
0?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0H`3oool2000000@0oooo00<000000?oo
o`3oool00`3oool2000000<0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000?0?ooo`03
0000o`3oool0oooo0680oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o0?oo
o`3oool0H`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<0
00000?ooo`3oool00`3oool3000008h0oooo000>0?ooo`030000o`3oool0oooo06<0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`<000001P3oool00`000000oooo
0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1S0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo08h0
oooo000>0?ooo`030000o`3oool0oooo06<0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool000d0
oooo00<0003o0?ooo`3oool0I@3oool2000000X0oooo0P0000030?ooo`@000001@3oool00`000000
oooo0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1o0?ooo`800000S`3oool000d0oooo00<0
003o0?ooo`3oool0O`3oool00`000000oooo0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1o
0?ooo`030000003oool0oooo08h0oooo000<0?ooo`030000o`3oool0oooo0800oooo00<000000?oo
o`3oool0SP3oool000`0oooo00<0003o0?ooo`3oool0P03oool00`000000oooo0?ooo`2>0?ooo`00
303oool00`000?l0oooo0?ooo`200?ooo`800000S`3oool000`0oooo00<0003o0?ooo`3oool0P03o
ool00`000000oooo0?ooo`2>0?ooo`002`3oool00`000?l0oooo0?ooo`210?ooo`030000003oool0
oooo08h0oooo000;0?ooo`030000o`3oool0oooo0840oooo00<000000?ooo`3oool0SP3oool000/0
oooo00<0003o0?ooo`3oool0P@3oool00`000000oooo0?ooo`2>0?ooo`002`3oool00`000?l0oooo
0?ooo`210?ooo`800000S`3oool000X0oooo00<0003o0?ooo`3oool0PP3oool00`000000oooo0?oo
o`2>0?ooo`002P3oool00`000?l0oooo0?ooo`220?ooo`030000003oool0oooo08h0oooo000:0?oo
o`030000o`3oool0oooo0880oooo00<000000?ooo`3oool0SP3oool000X0oooo00<0003o0?ooo`3o
ool0PP3oool00`000000oooo0?ooo`2>0?ooo`002@3oool00`000?l0oooo0?ooo`230?ooo`030000
003oool0oooo08h0oooo00090?ooo`030000o`3oool0oooo08<0oooo0P00002?0?ooo`002@3oool0
0`000?l0oooo0?ooo`230?ooo`030000003oool0oooo08h0oooo00090?ooo`030000o`3oool0oooo
08<0oooo00<000000?ooo`3oool0SP3oool000P0oooo00<0003o0?ooo`3oool0L03oool2000000@0
oooo00<000000?ooo`3oool00P3oool4000000D0oooo00<000000?ooo`3oool0SP3oool000P0oooo
00<0003o0?ooo`3oool0K`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0SP3oool000P0oooo00<0003o0?ooo`3oool0K`3oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo0`00002>0?ooo`00203oool00`000?l0oooo
0?ooo`1_0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`2>0?ooo`001`3oool00`000?l0oooo0?ooo`1`0?ooo`040000003oool0oooo000000X0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`001`3oool00`000?l0oooo
0?ooo`1`0?ooo`040000003oool0oooo000000P0oooo0`0000060?ooo`030000003oool0oooo08h0
oooo001k0?ooo`8000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo
002?0?ooo`800000S`3oool008l0oooo00<000000?ooo`3oool0SP3oool008l0oooo00<000000?oo
o`3oool0SP3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05261, -0.228774, \
0.00733523, 0.00194436}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Expand[\(-\((\((xi - om*kz)\)^2 - \((1 - om^2)\)*\((1 - kz^2)\))\)\) /. 
        kz \[Rule] 1 - kx]\)], "Input"],

Cell[BoxData[
    \(2\ kx - kx\^2 - om\^2 + 2\ om\ xi - 2\ kx\ om\ xi - xi\^2\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Real part:", "Subsubsection"],

Cell[BoxData[
    \(Rebeta[md2_, k_, kz_, om_, a_] := 
      Module[{res, mxi1, mxi2}, If[logging \[Equal] 0, Print["\<beta\>"]]; 
        If[om < k, mxi1 = If[k > om, xi1[k, kz, om], 0]; 
          mxi2 = If[k > om, xi2[k, kz, om], 0]; \[IndentingNewLine]res = 
            md2/k^2*om/
                2*\((\(-NIntegrate[1/\((1 + a\ xi^2)\)^2*om, {xi, \(-1\), 1}, 
                      WorkingPrecision \[Rule] 30, 
                      MaxRecursion \[Rule] 20]\) + 
                  NIntegrate[
                    1/\((1 + a\ xi^2)\)^2*\((om^2 + a*om*kz*xi)\)/
                        Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                  om^2)\)*\((1 - kz^2/k^2)\)], {xi, \(-1\), 
                      mxi2*If[
                          mxi2 < 
                            0, \((1 + 10^\((\(-30\))\))\), \((1 - 
                              10^\((\(-30\))\))\)]}, 
                    WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 20] + 
                  Sign[om - kz]*
                    NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om^2 + a*om*kz*xi)\)/
                          Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                    om^2)\)*\((1 - kz^2/k^2)\)], {xi, 
                        mxi1*\((1 + 10^\((\(-30\))\))\), 1}, 
                      WorkingPrecision \[Rule] 20, 
                      MaxRecursion \[Rule] 30])\)]; \ 
        If[om > k, \(res = 
              md2/k^2*om/
                  2*\((\(-NIntegrate[
                        1/\((1 + a\ xi^2)\)^2*om, {xi, \(-1\), 1}, 
                        WorkingPrecision \[Rule] 30, 
                        MaxRecursion \[Rule] 20]\) + 
                    NIntegrate[
                      Sign[om - kz\ xi]/\((1 + a\ xi^2)\)^2*\((om^2 + 
                              a*om*kz*xi)\)/
                          Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                    om^2)\)*\((1 - kz^2/k^2)\)], {xi, \(-1\), 
                        If[om\ kz/k^2 < 1, om\ kz/k^2, 0], \ 1}, 
                      WorkingPrecision \[Rule] 30, 
                      MaxRecursion \[Rule] 20])\);\)]; \ res]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rebeta[1, 1, 1/10, 1/2, 0]\)], "Input"],

Cell[BoxData[
    \(\(-0.18133673195824344393622`10.5019\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rebeta[1, 1, 1/10, 1/2, \(-99999\)/100000]\)], "Input"],

Cell[BoxData[
    \(2.6161404805133408047`6.1512\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ReTable1 = 
      Table[{N[a], Rebeta[1, 1, 1/10, 1/2, a]}, {a, \(-99999\)/100000, 1.1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \({{\(-0.99999`\), 
        2.6161404805133408047`6.1512}, {\(-0.89999`\), \
\(-0.038630286268108058968566`8.17\)}, {\(-0.79999`\), \
\(-0.157597268847689091198317`9.0369\)}, {\(-0.69999`\), \
\(-0.192925220519335708676419`9.3901\)}, {\(-0.59999`\), \
\(-0.203532771357275270601756`9.7485\)}, {\(-0.49999`\), \
\(-0.20471855370733435537764`9.7964\)}, {\(-0.39999`\), \
\(-0.20190920600300767019015`10.2127\)}, {\(-0.29999`\), \
\(-0.197367368409631965657292`9.9298\)}, {\(-0.19999`\), \
\(-0.19213112856584503238313`10.3775\)}, {\(-0.09999`\), \
\(-0.186704011141628141063883`10.3873\)}, {0.00001`, \
\(-0.181336202934116538432231`10.5019\)}, {0.10001`, \
\(-0.176150767649524371266427`10.5454\)}, {0.20001`, \
\(-0.171204473739308096959134`10.2167\)}, {0.30001`, \
\(-0.166518729167003065558545`10.6387\)}, {0.40001`, \
\(-0.162095966347226755260819`10.6728\)}, {0.50001`, \
\(-0.157928584699160230549394`10.6883\)}, {0.60001`, \
\(-0.154003932218153799578901`10.6667\)}, {0.70001`, \
\(-0.150307111174601572118526`10.5892\)}, {0.80001`, \
\(-0.146822558566406820250405`10.4385\)}, {0.90001`, \
\(-0.143534923716843896040076`10.2421\)}, {1.00001`, \
\(-0.140429537642555254767375`10.0404\)}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(RePlot1 = 
      ListPlot[ReTable1, PlotJoined \[Rule] True, 
        PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.499995 0.47619 0.863698 4.14707 [
[.0238 .02178 -6 -9 ]
[.0238 .02178 6 0 ]
[.2619 .02178 -12 -9 ]
[.2619 .02178 12 0 ]
[.73809 .02178 -9 -9 ]
[.73809 .02178 9 0 ]
[.97619 .02178 -3 -9 ]
[.97619 .02178 3 0 ]
[.4875 .11722 -30 -4.5 ]
[.4875 .11722 0 4.5 ]
[.4875 .20017 -30 -4.5 ]
[.4875 .20017 0 4.5 ]
[.4875 .28311 -30 -4.5 ]
[.4875 .28311 0 4.5 ]
[.4875 .36605 -30 -4.5 ]
[.4875 .36605 0 4.5 ]
[.4875 .44899 -24 -4.5 ]
[.4875 .44899 0 4.5 ]
[.4875 .53193 -30 -4.5 ]
[.4875 .53193 0 4.5 ]
[.4875 .61487 -30 -4.5 ]
[.4875 .61487 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.0238 .03428 m
.0238 .04053 L
s
[(-1)] .0238 .02178 0 1 Mshowa
.2619 .03428 m
.2619 .04053 L
s
[(-0.5)] .2619 .02178 0 1 Mshowa
.73809 .03428 m
.73809 .04053 L
s
[(0.5)] .73809 .02178 0 1 Mshowa
.97619 .03428 m
.97619 .04053 L
s
[(1)] .97619 .02178 0 1 Mshowa
.125 Mabswid
.07142 .03428 m
.07142 .03803 L
s
.11904 .03428 m
.11904 .03803 L
s
.16666 .03428 m
.16666 .03803 L
s
.21428 .03428 m
.21428 .03803 L
s
.30952 .03428 m
.30952 .03803 L
s
.35714 .03428 m
.35714 .03803 L
s
.40476 .03428 m
.40476 .03803 L
s
.45238 .03428 m
.45238 .03803 L
s
.54761 .03428 m
.54761 .03803 L
s
.59523 .03428 m
.59523 .03803 L
s
.64285 .03428 m
.64285 .03803 L
s
.69047 .03428 m
.69047 .03803 L
s
.78571 .03428 m
.78571 .03803 L
s
.83333 .03428 m
.83333 .03803 L
s
.88095 .03428 m
.88095 .03803 L
s
.92857 .03428 m
.92857 .03803 L
s
.25 Mabswid
0 .03428 m
1 .03428 L
s
.5 .11722 m
.50625 .11722 L
s
[(-0.18)] .4875 .11722 1 0 Mshowa
.5 .20017 m
.50625 .20017 L
s
[(-0.16)] .4875 .20017 1 0 Mshowa
.5 .28311 m
.50625 .28311 L
s
[(-0.14)] .4875 .28311 1 0 Mshowa
.5 .36605 m
.50625 .36605 L
s
[(-0.12)] .4875 .36605 1 0 Mshowa
.5 .44899 m
.50625 .44899 L
s
[(-0.1)] .4875 .44899 1 0 Mshowa
.5 .53193 m
.50625 .53193 L
s
[(-0.08)] .4875 .53193 1 0 Mshowa
.5 .61487 m
.50625 .61487 L
s
[(-0.06)] .4875 .61487 1 0 Mshowa
.125 Mabswid
.5 .05502 m
.50375 .05502 L
s
.5 .07575 m
.50375 .07575 L
s
.5 .09649 m
.50375 .09649 L
s
.5 .13796 m
.50375 .13796 L
s
.5 .1587 m
.50375 .1587 L
s
.5 .17943 m
.50375 .17943 L
s
.5 .2209 m
.50375 .2209 L
s
.5 .24164 m
.50375 .24164 L
s
.5 .26237 m
.50375 .26237 L
s
.5 .30384 m
.50375 .30384 L
s
.5 .32458 m
.50375 .32458 L
s
.5 .34531 m
.50375 .34531 L
s
.5 .38678 m
.50375 .38678 L
s
.5 .40752 m
.50375 .40752 L
s
.5 .42826 m
.50375 .42826 L
s
.5 .46973 m
.50375 .46973 L
s
.5 .49046 m
.50375 .49046 L
s
.5 .5112 m
.50375 .5112 L
s
.5 .55267 m
.50375 .55267 L
s
.5 .5734 m
.50375 .5734 L
s
.5 .59414 m
.50375 .59414 L
s
.5 .01355 m
.50375 .01355 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.07968 .61803 m
.11905 .21013 L
.16667 .06362 L
.21429 .01963 L
.2619 .01472 L
.30952 .02637 L
.35714 .0452 L
.40476 .06692 L
.45238 .08942 L
.5 .11168 L
.54762 .13319 L
.59524 .1537 L
.64286 .17313 L
.69048 .19147 L
.7381 .20876 L
.78571 .22503 L
.83333 .24036 L
.88095 .25481 L
.92857 .26845 L
.97619 .28133 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003@3oool4000003P0oooo0P0000040?ooo`030000003oool0oooo0080
oooo0`00001c0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<00000>03oool4000000`0
oooo000?0?ooo`030000003oool0oooo03H0oooo00@000000?ooo`3oool000002`3oool00`000000
oooo0?ooo`0e0?ooo`030000003oool0oooo03L0oooo00@000000?ooo`3oool000002`3oool00`00
0000oooo0?ooo`0g0?ooo`030000003oool0oooo00/0oooo000?0?ooo`030000003oool0oooo03H0
oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo
03L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0
oooo00/0oooo00070?ooo`@00000103oool00`000000oooo0?ooo`0`0?ooo`@000000P3oool01000
0000oooo0?ooo`0000080?ooo`<00000>03oool00`000000oooo0?ooo`0g0?ooo`040000003oool0
oooo000000P0oooo0`00000j0?ooo`030000003oool0oooo00/0oooo000?0?ooo`030000003oool0
oooo03H0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0h0?ooo`800000>03o
ool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0
2`3oool000d0oooo0`00000g0?ooo`T0003o103oool00`000000oooo0?ooo`0h0?ooo`030000003o
ool0oooo03L0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0h0?ooo`<00000
3@3oool000l0oooo00<000000?ooo`3oool0;P3oool70000o`80oooo0P0000050?ooo`@0003o1000
000g0?ooo`030000003oool0oooo03P0oooo0P0000090?ooo`@00000>@3oool00`000000oooo0?oo
o`0;0?ooo`00?`3oool00`000?l0oooo0?ooo`0B0?ooo`D0003o=P3oool00`000000oooo0?ooo`2>
0?ooo`00?P3oool00`000?l0oooo0?ooo`0H0?ooo`@0003o<P3oool00`000000oooo0?ooo`2>0?oo
o`00?@3oool00`000?l0oooo0?ooo`0M0?ooo`80003o<03oool00`000000oooo0?ooo`2>0?ooo`00
1@3ooolg000000030000o`0000000000020000000`000?nh000000H0oooo000<0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00D0oooo00<0003o0?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0103oool30000o`<0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0;0?ooo`00303oool00`000000oooo0?ooo`0Z0?ooo`80003o4P3oool0
0`000000oooo0?ooo`0E0?ooo`80003o:03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0
oooo03h0oooo00<000000?ooo`3oool02`3oool003P0oooo00<0003o0?ooo`3oool0;03oool30000
obD0oooo00<000000?ooo`3oool0SP3oool003L0oooo00<0003o0?ooo`3oool0<03oool20000ob<0
oooo00<000000?ooo`3oool0SP3oool003H0oooo00<0003o0?ooo`3oool0<`3oool20000ob40oooo
00<000000?ooo`3oool0SP3oool003D0oooo00<0003o0?ooo`3oool0=P3oool20000oal0oooo0P00
002?0?ooo`00=03oool00`000?l0oooo0?ooo`0i0?ooo`80003o7@3oool00`000000oooo0?ooo`2>
0?ooo`00<`3oool00`000?l0oooo0?ooo`0l0?ooo`80003o6`3oool00`000000oooo0?ooo`2>0?oo
o`00<`3oool00`000?l0oooo0?ooo`0n0?ooo`<0003o603oool00`000000oooo0?ooo`2>0?ooo`00
<P3oool00`000?l0oooo0?ooo`120?ooo`80003o5P3oool00`000000oooo0?ooo`2>0?ooo`00<P3o
ool00`000?l0oooo0?ooo`140?ooo`80003o503oool2000008l0oooo000b0?ooo`030000o`3oool0
oooo04H0oooo0P000?lB0?ooo`030000003oool0oooo08h0oooo000a0?ooo`030000o`3oool0oooo
04T0oooo0P000?l@0?ooo`030000003oool0oooo08h0oooo000a0?ooo`030000o`3oool0oooo04/0
oooo0P000?l>0?ooo`030000003oool0oooo08h0oooo000a0?ooo`030000o`3oool0oooo04d0oooo
0`000?l;0?ooo`030000003oool0oooo08h0oooo000`0?ooo`030000o`3oool0oooo0540oooo0P00
0?l90?ooo`030000003oool0oooo08h0oooo000`0?ooo`030000o`3oool0oooo05<0oooo0P000?l7
0?ooo`800000S`3oool00300oooo00<0003o0?ooo`3oool0E@3oool20000o`D0oooo00<000000?oo
o`3oool0SP3oool002l0oooo00<0003o0?ooo`3oool0F03oool20000o`<0oooo00<000000?ooo`3o
ool0SP3oool002l0oooo00<0003o0?ooo`3oool0FP3oool20000o`030?ooo`000000oooo08l0oooo
000_0?ooo`030000o`3oool0oooo04@0oooo0P0000040?ooo`030000003oool0oooo0080oooo1000
00030?ooo`800000103oool30000ohl0oooo000^0?ooo`030000o`3oool0oooo04@0oooo00@00000
0?ooo`3oool000002P3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0
0P3oool010000000oooo0000o`000?n=0?ooo`00;P3oool00`000?l0oooo0?ooo`140?ooo`040000
003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo
0080oooo0`0000000`3oool0003o0000o`2;0?ooo`00;P3oool00`000?l0oooo0?ooo`0n0?ooo`@0
00000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0080oooo0P0000050?oo
o`030000003oool0oooo00<0oooo0P000?n90?ooo`00;@3oool00`000?l0oooo0?ooo`150?ooo`04
0000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool01@3oool20000ohL0oooo000]0?ooo`030000o`3oool0oooo
04D0oooo00@000000?ooo`3oool00000203oool3000000<0oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`070?ooo`80003oQ@3oool002d0oooo00<0003o0?ooo`3oool0AP3oool2
000000/0oooo00<000000?ooo`3oool00P3oool2000000D0oooo00<000000?ooo`3oool02@3oool3
0000oh80oooo000]0?ooo`030000o`3oool0oooo05l0oooo00<000000?ooo`3oool0303oool20000
oh00oooo000/0?ooo`030000o`3oool0oooo0600oooo0P00000?0?ooo`80003oOP3oool002`0oooo
00<0003o0?ooo`3oool0H03oool00`000000oooo0?ooo`0@0?ooo`80003oO03oool002`0oooo00<0
003o0?ooo`3oool0H03oool00`000000oooo0?ooo`0B0?ooo`80003oNP3oool002/0oooo00<0003o
0?ooo`3oool0H@3oool00`000000oooo0?ooo`0D0?ooo`80003oN03oool002/0oooo00<0003o0?oo
o`3oool0H@3oool00`000000oooo0?ooo`0F0?ooo`<0003oM@3oool002/0oooo00<0003o0?ooo`3o
ool0H@3oool2000001X0oooo0P000?mc0?ooo`00:P3oool00`000?l0oooo0?ooo`1R0?ooo`030000
003oool0oooo01/0oooo0`000?m`0?ooo`00:P3oool00`000?l0oooo0?ooo`1R0?ooo`030000003o
ool0oooo01h0oooo0`000?m]0?ooo`00:P3oool00`000?l0oooo0?ooo`1R0?ooo`030000003oool0
oooo0240oooo0P000?m[0?ooo`00:@3oool00`000?l0oooo0?ooo`1S0?ooo`030000003oool0oooo
02<0oooo0`000?mX0?ooo`00:@3oool00`000?l0oooo0?ooo`1S0?ooo`030000003oool0oooo02H0
oooo0P000?mV0?ooo`00:@3oool00`000?l0oooo0?ooo`1S0?ooo`800000:@3oool30000of<0oooo
000X0?ooo`030000o`3oool0oooo06@0oooo00<000000?ooo`3oool0:`3oool30000of00oooo000X
0?ooo`030000o`3oool0oooo06@0oooo00<000000?ooo`3oool0;P3oool20000oeh0oooo000X0?oo
o`030000o`3oool0oooo06@0oooo00<000000?ooo`3oool0<03oool30000oe/0oooo000W0?ooo`03
0000o`3oool0oooo04`0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000030?ooo`80
00001@3oool00`000000oooo0?ooo`0c0?ooo`<0003oF03oool002L0oooo00<0003o0?ooo`3oool0
B`3oool010000000oooo0?ooo`00000:0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo03H0oooo0P000?mF0?ooo`009`3oool00`000?l0
oooo0?ooo`1;0?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo00000002
0?ooo`030000003oool0oooo0080oooo0`00000h0?ooo`<0003oD`3oool002H0oooo00<0003o0?oo
o`3oool0AP3oool400000080oooo00@000000?ooo`3oool000002P3oool010000000oooo0?ooo`3o
ool3000000D0oooo00<000000?ooo`3oool0>`3oool30000oe00oooo000V0?ooo`030000o`3oool0
oooo04`0oooo00@000000?ooo`3oool000002P3oool01@000000oooo0?ooo`3oool0000000L0oooo
00<000000?ooo`3oool0?P3oool30000odd0oooo000V0?ooo`030000o`3oool0oooo04`0oooo00@0
00000?ooo`3oool00000203oool3000000<0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`110?ooo`80003oB`3oool002H0oooo00<0003o0?ooo`3oool0C@3oool2000000/0oooo00<0
00000?ooo`3oool00P3oool3000000@0oooo00<000000?ooo`3oool0@`3oool30000odP0oooo000V
0?ooo`030000o`3oool0oooo06H0oooo0P0000170?ooo`<0003oA@3oool002H0oooo00<0003o0?oo
o`3oool0IP3oool00`000000oooo0?ooo`190?ooo`80003o@`3oool002H0oooo00<0003o0?ooo`3o
ool0IP3oool00`000000oooo0?ooo`1;0?ooo`<0003o@03oool002D0oooo00<0003o0?ooo`3oool0
I`3oool00`000000oooo0?ooo`1>0?ooo`<0003o?@3oool002D0oooo00<0003o0?ooo`3oool0I`3o
ool00`000000oooo0?ooo`1A0?ooo`@0003o>@3oool002D0oooo00<0003o0?ooo`3oool0I`3oool0
0`000000oooo0?ooo`1E0?ooo`<0003o=P3oool002D0oooo00<0003o0?ooo`3oool0I`3oool20000
05T0oooo0`000?lc0?ooo`009@3oool00`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo05/0
oooo0`000?l`0?ooo`009@3oool00`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo05h0oooo
10000?l/0?ooo`009@3oool00`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo0680oooo0`00
0?lY0?ooo`009@3oool00`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo06D0oooo0`000?lV
0?ooo`009@3oool00`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo06P0oooo0`000?lS0?oo
o`009@3oool00`000?l0oooo0?ooo`1W0?ooo`800000K03oool40000oal0oooo000T0?ooo`030000
o`3oool0oooo06P0oooo00<000000?ooo`3oool0K`3oool30000oa`0oooo000T0?ooo`030000o`3o
ool0oooo06P0oooo00<000000?ooo`3oool0LP3oool40000oaP0oooo000T0?ooo`030000o`3oool0
oooo06P0oooo00<000000?ooo`3oool0MP3oool40000oa@0oooo000T0?ooo`030000o`3oool0oooo
04l0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0NP3oool40000oa00oooo000T0?ooo`030000o`3oool0oooo04h0
oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0OP3oool30000o`d0oooo000T0?ooo`030000o`3oool0oooo04h0
oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`05000000@0oooo0`00002>0?oo
o`00903oool00`000?l0oooo0?ooo`180?ooo`@000000P3oool010000000oooo0?ooo`00000:0?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>
0?ooo`00903oool00`000?l0oooo0?ooo`1>0?ooo`040000003oool0oooo000000X0oooo00L00000
0?ooo`3oool0oooo0000003oool0000000D0oooo00<000000?ooo`3oool0SP3oool002@0oooo00<0
003o0?ooo`3oool0CP3oool010000000oooo0?ooo`0000080?ooo`<00000103oool2000000D0oooo
00<000000?ooo`3oool0SP3oool002@0oooo00<0003o0?ooo`3oool0C`3oool2000000/0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000S
0?ooo`030000o`3oool0oooo06T0oooo0P00002?0?ooo`008`3oool00`000?l0oooo0?ooo`1Y0?oo
o`030000003oool0oooo08h0oooo000S0?ooo`030000o`3oool0oooo06T0oooo00<000000?ooo`3o
ool0SP3oool002<0oooo00<0003o0?ooo`3oool0J@3oool00`000000oooo0?ooo`2>0?ooo`008`3o
ool00`000?l0oooo0?ooo`1Y0?ooo`030000003oool0oooo08h0oooo000S0?ooo`030000o`3oool0
oooo06T0oooo00<000000?ooo`3oool0SP3oool002<0oooo00<0003o0?ooo`3oool0J@3oool20000
08l0oooo000S0?ooo`030000o`3oool0oooo06T0oooo00<000000?ooo`3oool0SP3oool002<0oooo
00<0003o0?ooo`3oool0J@3oool00`000000oooo0?ooo`2>0?ooo`008`3oool00`000?l0oooo0?oo
o`1Y0?ooo`030000003oool0oooo08h0oooo000R0?ooo`030000o`3oool0oooo06X0oooo00<00000
0?ooo`3oool0SP3oool00280oooo00<0003o0?ooo`3oool0JP3oool00`000000oooo0?ooo`2>0?oo
o`008P3oool00`000?l0oooo0?ooo`1Z0?ooo`800000S`3oool00280oooo00<0003o0?ooo`3oool0
JP3oool00`000000oooo0?ooo`2>0?ooo`008P3oool00`000?l0oooo0?ooo`1Z0?ooo`030000003o
ool0oooo08h0oooo000R0?ooo`030000o`3oool0oooo0540oooo0P0000040?ooo`030000003oool0
oooo0080oooo100000020?ooo`@00000103oool00`000000oooo0?ooo`2>0?ooo`008P3oool00`00
0?l0oooo0?ooo`1@0?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo0000
00070?ooo`030000003oool0oooo08h0oooo000R0?ooo`030000o`3oool0oooo0500oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo0`00
002>0?ooo`008P3oool00`000?l0oooo0?ooo`1:0?ooo`@000000P3oool010000000oooo0?ooo`00
000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`2>0?ooo`008P3oool00`000?l0oooo0?ooo`1@0?ooo`040000003oool0oooo000000X0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo08h0oooo000R
0?ooo`030000o`3oool0oooo0500oooo00@000000?ooo`3oool00000203oool3000000<0oooo00@0
00000?ooo`3oool00000103oool00`000000oooo0?ooo`2>0?ooo`008@3oool00`000?l0oooo0?oo
o`1B0?ooo`8000002`3oool00`000000oooo0?ooo`020?ooo`8000001@3oool00`000000oooo0?oo
o`2>0?ooo`008@3oool00`000?l0oooo0?ooo`1[0?ooo`030000003oool0oooo08h0oooo000Q0?oo
o`030000o`3oool0oooo06/0oooo0P00002?0?ooo`008@3oool00`000?l0oooo0?ooo`1[0?ooo`03
0000003oool0oooo08h0oooo000Q0?ooo`030000o`3oool0oooo06/0oooo00<000000?ooo`3oool0
SP3oool00240oooo00<0003o0?ooo`3oool0J`3oool00`000000oooo0?ooo`2>0?ooo`008@3oool0
0`000?l0oooo0?ooo`1[0?ooo`030000003oool0oooo08h0oooo000Q0?ooo`030000o`3oool0oooo
06/0oooo00<000000?ooo`3oool0SP3oool00240oooo00<0003o0?ooo`3oool0J`3oool2000008l0
oooo000Q0?ooo`030000o`3oool0oooo06/0oooo00<000000?ooo`3oool0SP3oool00200oooo00<0
003o0?ooo`3oool0K03oool00`000000oooo0?ooo`2>0?ooo`00803oool00`000?l0oooo0?ooo`1/
0?ooo`030000003oool0oooo08h0oooo000P0?ooo`030000o`3oool0oooo06`0oooo00<000000?oo
o`3oool0SP3oool00200oooo00<0003o0?ooo`3oool0K03oool00`000000oooo0?ooo`2>0?ooo`00
803oool00`000?l0oooo0?ooo`1/0?ooo`800000S`3oool00200oooo00<0003o0?ooo`3oool0K03o
ool00`000000oooo0?ooo`2>0?ooo`00803oool00`000?l0oooo0?ooo`1/0?ooo`030000003oool0
oooo08h0oooo000P0?ooo`030000o`3oool0oooo05T0oooo0P0000040?ooo`030000003oool0oooo
0080oooo100000040?ooo`030000003oool0oooo08h0oooo000P0?ooo`030000o`3oool0oooo05P0
oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
08h0oooo000P0?ooo`030000o`3oool0oooo05P0oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`030?ooo`<00000SP3oool001l0oooo00<0003o0?ooo`3oool0D`3oool400000080
oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
08h0oooo000O0?ooo`030000o`3oool0oooo05T0oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000O0?ooo`030000o`3oool0oooo05T0
oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<000000?ooo`3oool0SP3oool001l0
oooo00<0003o0?ooo`3oool0FP3oool2000000/0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`2>0?ooo`007`3oool00`000?l0oooo0?ooo`1]0?ooo`030000003oool0oooo08h0oooo
000O0?ooo`030000o`3oool0oooo06d0oooo0P00002?0?ooo`007`3oool00`000?l0oooo0?ooo`1]
0?ooo`030000003oool0oooo08h0oooo000O0?ooo`030000o`3oool0oooo06d0oooo00<000000?oo
o`3oool0SP3oool001l0oooo00<0003o0?ooo`3oool0K@3oool00`000000oooo0?ooo`2>0?ooo`00
7`3oool00`000?l0oooo0?ooo`1]0?ooo`030000003oool0oooo08h0oooo000O0?ooo`030000o`3o
ool0oooo06d0oooo00<000000?ooo`3oool0SP3oool001h0oooo00<0003o0?ooo`3oool0KP3oool2
000008l0oooo000N0?ooo`030000o`3oool0oooo06h0oooo00<000000?ooo`3oool0SP3oool001h0
oooo00<0003o0?ooo`3oool0KP3oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000?l0oooo
0?ooo`1^0?ooo`030000003oool0oooo08h0oooo000N0?ooo`030000o`3oool0oooo06h0oooo00<0
00000?ooo`3oool0SP3oool001h0oooo00<0003o0?ooo`3oool0KP3oool2000008l0oooo000N0?oo
o`030000o`3oool0oooo06h0oooo00<000000?ooo`3oool0SP3oool001h0oooo00<0003o0?ooo`3o
ool0KP3oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000?l0oooo0?ooo`1^0?ooo`030000
003oool0oooo08h0oooo000N0?ooo`030000o`3oool0oooo05D0oooo0P0000040?ooo`030000003o
ool0oooo00<0oooo0P0000040?ooo`8000001@3oool00`000000oooo0?ooo`2>0?ooo`007@3oool0
0`000?l0oooo0?ooo`1E0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo08h0oooo000M0?ooo`030000
o`3oool0oooo05D0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo000000@0oooo0`00002>0?ooo`007@3oool00`000?l0oooo0?ooo`1?0?oo
o`@000000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000<0oooo0P00
00050?ooo`030000003oool0oooo08h0oooo000M0?ooo`030000o`3oool0oooo05D0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool0SP3oool001d0oooo00<0003o0?ooo`3oool0E@3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool0
0`000000oooo0?ooo`2>0?ooo`007@3oool00`000?l0oooo0?ooo`1F0?ooo`8000002P3oool20000
00@0oooo0P0000050?ooo`030000003oool0oooo08h0oooo000M0?ooo`030000o`3oool0oooo06l0
oooo00<000000?ooo`3oool0SP3oool001d0oooo00<0003o0?ooo`3oool0K`3oool2000008l0oooo
000M0?ooo`030000o`3oool0oooo06l0oooo00<000000?ooo`3oool0SP3oool001d0oooo00<0003o
0?ooo`3oool0K`3oool00`000000oooo0?ooo`2>0?ooo`00703oool00`000?l0oooo0?ooo`1`0?oo
o`030000003oool0oooo08h0oooo000L0?ooo`030000o`3oool0oooo0700oooo00<000000?ooo`3o
ool0SP3oool001`0oooo00<0003o0?ooo`3oool0L03oool00`000000oooo0?ooo`2>0?ooo`00703o
ool00`000?l0oooo0?ooo`1`0?ooo`800000S`3oool001`0oooo00<0003o0?ooo`3oool0L03oool0
0`000000oooo0?ooo`2>0?ooo`00703oool00`000?l0oooo0?ooo`1`0?ooo`030000003oool0oooo
08h0oooo000L0?ooo`030000o`3oool0oooo0700oooo00<000000?ooo`3oool0SP3oool001`0oooo
00<0003o0?ooo`3oool0L03oool00`000000oooo0?ooo`2>0?ooo`00703oool00`000?l0oooo0?oo
o`1`0?ooo`800000S`3oool001`0oooo00<0003o0?ooo`3oool0L03oool00`000000oooo0?ooo`2>
0?ooo`006`3oool00`000?l0oooo0?ooo`1a0?ooo`030000003oool0oooo08h0oooo000K0?ooo`03
0000o`3oool0oooo0740oooo00<000000?ooo`3oool0SP3oool001/0oooo00<0003o0?ooo`3oool0
F03oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P0000050?ooo`030000
003oool0oooo08h0oooo000K0?ooo`030000o`3oool0oooo05L0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo00<000000?oo
o`3oool0SP3oool001/0oooo00<0003o0?ooo`3oool0E`3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool3000008h0oooo000K
0?ooo`030000o`3oool0oooo0540oooo100000020?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool000000P3oool3000000D0oooo00<000000?ooo`3oool0SP3oool007D0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo09H0
oooo001e0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`2F0?ooo`00MP3oool2000000X0oooo0P0000040?ooo`<00000U@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.09356, -0.210891, \
0.00762067, 0.000875048}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ReTable3 = 
      Table[{N[a], Rebeta[1, 1, 9/10, 1/2, a]}, {a, \(-99999\)/100000, 1.1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \({{\(-0.99999`\), 
        0.1839508599579149246`5.0157}, {\(-0.89999`\), \
\(-0.230855875516954792237381`8.9444\)}, {\(-0.79999`\), \
\(-0.248050489935969429155819`9.3006\)}, {\(-0.69999`\), \
\(-0.245059079236821638060974`9.5113\)}, {\(-0.59999`\), \
\(-0.23630312866329774989628`9.8067\)}, {\(-0.49999`\), \
\(-0.226033994765728018530931`9.8377\)}, {\(-0.39999`\), \
\(-0.215762423249811994320277`10.1152\)}, {\(-0.29999`\), \
\(-0.206043833661890848246219`9.8383\)}, {\(-0.19999`\), \
\(-0.197059288022364829104879`10.0695\)}, {\(-0.09999`\), \
\(-0.188835516132006475110774`10.0223\)}, {0.00001`, \
\(-0.181336016303820965628516`10.023\)}, {0.10001`, \
\(-0.174500779687002454550652`10.144\)}, {0.20001`, \
\(-0.168263981596297220736507`9.9563\)}, {0.30001`, \
\(-0.162561746965455672790761`10.1105\)}, {0.40001`, \
\(-0.157335309783782622044404`10.0936\)}, {0.50001`, \
\(-0.152532013976967287714218`10.0742\)}, {0.60001`, \
\(-0.14810531845280292934708`10.0481\)}, {0.70001`, \
\(-0.144014369341582064012709`10.0097\)}, {0.80001`, \
\(-0.14022341334563590205524`10.0672\)}, {0.90001`, \
\(-0.136701183501446876175504`9.9585\)}, {1.00001`, \
\(-0.133420317372031690067569`9.8297\)}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(RePlot3 = 
      ListPlot[ReTable3, PlotJoined \[Rule] True, 
        PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.499995 0.47619 0.603319 2.37292 [
[.0238 .59082 -6 -9 ]
[.0238 .59082 6 0 ]
[.2619 .59082 -12 -9 ]
[.2619 .59082 12 0 ]
[.73809 .59082 -9 -9 ]
[.73809 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.4875 .01009 -30 -4.5 ]
[.4875 .01009 0 4.5 ]
[.4875 .12874 -24 -4.5 ]
[.4875 .12874 0 4.5 ]
[.4875 .24738 -30 -4.5 ]
[.4875 .24738 0 4.5 ]
[.4875 .36603 -24 -4.5 ]
[.4875 .36603 0 4.5 ]
[.4875 .48467 -30 -4.5 ]
[.4875 .48467 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.0238 .60332 m
.0238 .60957 L
s
[(-1)] .0238 .59082 0 1 Mshowa
.2619 .60332 m
.2619 .60957 L
s
[(-0.5)] .2619 .59082 0 1 Mshowa
.73809 .60332 m
.73809 .60957 L
s
[(0.5)] .73809 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(1)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07142 .60332 m
.07142 .60707 L
s
.11904 .60332 m
.11904 .60707 L
s
.16666 .60332 m
.16666 .60707 L
s
.21428 .60332 m
.21428 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54761 .60332 m
.54761 .60707 L
s
.59523 .60332 m
.59523 .60707 L
s
.64285 .60332 m
.64285 .60707 L
s
.69047 .60332 m
.69047 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.5 .01009 m
.50625 .01009 L
s
[(-0.25)] .4875 .01009 1 0 Mshowa
.5 .12874 m
.50625 .12874 L
s
[(-0.2)] .4875 .12874 1 0 Mshowa
.5 .24738 m
.50625 .24738 L
s
[(-0.15)] .4875 .24738 1 0 Mshowa
.5 .36603 m
.50625 .36603 L
s
[(-0.1)] .4875 .36603 1 0 Mshowa
.5 .48467 m
.50625 .48467 L
s
[(-0.05)] .4875 .48467 1 0 Mshowa
.125 Mabswid
.5 .03382 m
.50375 .03382 L
s
.5 .05755 m
.50375 .05755 L
s
.5 .08128 m
.50375 .08128 L
s
.5 .10501 m
.50375 .10501 L
s
.5 .15246 m
.50375 .15246 L
s
.5 .17619 m
.50375 .17619 L
s
.5 .19992 m
.50375 .19992 L
s
.5 .22365 m
.50375 .22365 L
s
.5 .27111 m
.50375 .27111 L
s
.5 .29484 m
.50375 .29484 L
s
.5 .31857 m
.50375 .31857 L
s
.5 .3423 m
.50375 .3423 L
s
.5 .38976 m
.50375 .38976 L
s
.5 .41349 m
.50375 .41349 L
s
.5 .43721 m
.50375 .43721 L
s
.5 .46094 m
.50375 .46094 L
s
.5 .5084 m
.50375 .5084 L
s
.5 .53213 m
.50375 .53213 L
s
.5 .55586 m
.50375 .55586 L
s
.5 .57959 m
.50375 .57959 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.04421 .61803 m
.07143 .05552 L
.11905 .01472 L
.16667 .02181 L
.21429 .04259 L
.2619 .06696 L
.30952 .09133 L
.35714 .11439 L
.40476 .13571 L
.45238 .15523 L
.5 .17302 L
.54762 .18924 L
.59524 .20404 L
.64286 .21757 L
.69048 .22997 L
.7381 .24137 L
.78571 .25188 L
.83333 .26158 L
.88095 .27058 L
.92857 .27894 L
.97619 .28672 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00M@3oool2000000@0oooo00<000000?ooo`3o
ool00P3oool400000080oooo0`0000060?ooo`030000003oool0oooo08h0oooo001d0?ooo`040000
003oool0oooo000000P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo08h0oooo001d0?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`030?ooo`<00000SP3oool002<0oooo10000?m70?ooo`@00000
0P3oool010000000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000001P3oool00`00
0000oooo0?ooo`2>0?ooo`008P3oool00`000?l0oooo0?ooo`020?ooo`H0003oA`3oool010000000
oooo0?ooo`00000;0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0SP3oool0
0240oooo00<0003o0?ooo`3oool02@3oool50000od80oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3o
ool001l0oooo0P000?lA0?ooo`80003o@@3oool2000000X0oooo0P0000030?ooo`@000001@3oool0
0`000000oooo0?ooo`2>0?ooo`007P3oool00`000?l0oooo0?ooo`0C0?ooo`80003oF@3oool00`00
0000oooo0?ooo`2>0?ooo`007@3oool00`000?l0oooo0?ooo`0F0?ooo`<0003oEP3oool00`000000
oooo0?ooo`2>0?ooo`00703oool00`000?l0oooo0?ooo`0J0?ooo`80003oE03oool2000008l0oooo
000K0?ooo`030000o`3oool0oooo01d0oooo0P000?mB0?ooo`030000003oool0oooo08h0oooo000J
0?ooo`030000o`3oool0oooo0200oooo0P000?m@0?ooo`030000003oool0oooo08h0oooo000H0?oo
o`80003o9@3oool20000odh0oooo00<000000?ooo`3oool0SP3oool001L0oooo00<0003o0?ooo`3o
ool09`3oool20000od`0oooo00<000000?ooo`3oool0SP3oool001H0oooo00<0003o0?ooo`3oool0
:P3oool20000odX0oooo00<000000?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0;@3o
ool20000odP0oooo00<000000?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0;`3oool2
0000odH0oooo0P00002?0?ooo`005@3oool00`000?l0oooo0?ooo`0a0?ooo`80003oA03oool00`00
0000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo0?ooo`0c0?ooo`030000o`3oool0oooo0440
oooo00<000000?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0=03oool20000od40oooo
00<000000?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0=P3oool20000ocl0oooo00<0
00000?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0>03oool20000ocd0oooo00<00000
0?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0>P3oool20000oc/0oooo0P00002?0?oo
o`005@3oool00`000?l0oooo0?ooo`0l0?ooo`80003o>@3oool00`000000oooo0?ooo`2>0?ooo`00
5@3oool00`000?l0oooo0?ooo`0n0?ooo`80003o=`3oool00`000000oooo0?ooo`2>0?ooo`005@3o
ool00`000?l0oooo0?ooo`100?ooo`80003o=@3oool00`000000oooo0?ooo`2>0?ooo`005@3oool0
0`000?l0oooo0?ooo`120?ooo`80003o<`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`00
0?l0oooo0?ooo`150?ooo`80003o<@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0
oooo0?ooo`170?ooo`80003o;`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo
0?ooo`190?ooo`80003o;@3oool2000008l0oooo000D0?ooo`030000o`3oool0oooo04/0oooo0P00
0?l[0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo04d0oooo0P000?lY
0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo04l0oooo0P000?lW0?oo
o`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo0540oooo0P000?lU0?ooo`03
0000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo05<0oooo0P000?l?0?ooo`800000
103oool00`000000oooo0?ooo`020?ooo`@000001@3oool00`000000oooo0?ooo`2>0?ooo`00503o
ool00`000?l0oooo0?ooo`1E0?ooo`<0003o2`3oool010000000oooo0?ooo`0000080?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0F03o
ool20000o`T0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`050?ooo`<00000
SP3oool001@0oooo00<0003o0?ooo`3oool0FP3oool20000o`030?ooo`0000000000008000000P3o
ool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
SP3oool001@0oooo00<0003o0?ooo`3oool0G03oool30000o`@0oooo00@000000?ooo`3oool00000
2`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3o
ool0oooo05l0oooo0P000?l20?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
00001@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`1Q0?ooo`<0003o
0P00000:0?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?oo
o`1T0?ooo`<0003o4@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`1W
0?ooo`80003o3`3oool2000008l0oooo000D0?ooo`030000o`3oool0oooo06T0oooo0`000?l<0?oo
o`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo06`0oooo0`000?l90?ooo`03
0000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo06l0oooo0P000?l70?ooo`030000
003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo0740oooo0`000?l40?ooo`030000003o
ool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo07@0oooo0`000?l00`3oool000000?ooo`2?
0?ooo`00503oool00`000?l0oooo0?ooo`1g0?ooo`<0003oS`3oool001@0oooo00<0003o0?ooo`3o
ool0N03oool2000000<0003oS03oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo
0?ooo`020?ooo`80003oRP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?oo
o`040?ooo`<0003oQ`3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`07
0?ooo`<0003oQ03oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`0:0?oo
o`<0003oP@3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`0=0?ooo`<0
003oOP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`0@0?ooo`@0003o
NP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool2000001D0oooo0`000?mg0?ooo`004`3oool0
0`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo01L0oooo0`000?md0?ooo`004`3oool00`00
0?l0oooo0?ooo`1i0?ooo`030000003oool0oooo01X0oooo10000?m`0?ooo`004`3oool00`000?l0
oooo0?ooo`1i0?ooo`030000003oool0oooo01h0oooo0`000?m]0?ooo`004`3oool00`000?l0oooo
0?ooo`1i0?ooo`030000003oool0oooo0240oooo10000?mY0?ooo`004`3oool00`000?l0oooo0?oo
o`1i0?ooo`030000003oool0oooo02D0oooo10000?mU0?ooo`004`3oool00`000?l0oooo0?ooo`1i
0?ooo`030000003oool0oooo02T0oooo10000?mQ0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?oo
o`800000;P3oool40000oed0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3o
ool0<@3oool40000oeT0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0
=@3oool40000oeD0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0>@3o
ool30000oe80oooo000C0?ooo`030000o`3oool0oooo05l0oooo0P0000040?ooo`030000003oool0
oooo0080oooo100000020?ooo`<000001P3oool00`000000oooo0?ooo`0l0?ooo`@0003oCP3oool0
01<0oooo00<0003o0?ooo`3oool0GP3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`100?ooo`@0003oBP3oool0
01<0oooo00<0003o0?ooo`3oool0GP3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool00`3oool3000004@0oooo10000?m60?ooo`004`3oool00`00
0?l0oooo0?ooo`1H0?ooo`@000000P3oool010000000oooo0?ooo`00000:0?ooo`040000003oool0
oooo0?ooo`<000001P3oool00`000000oooo0?ooo`180?ooo`@0003o@P3oool001<0oooo00<0003o
0?ooo`3oool0GP3oool010000000oooo0?ooo`00000:0?ooo`050000003oool0oooo0?ooo`000000
203oool00`000000oooo0?ooo`1<0?ooo`H0003o?03oool00180oooo00<0003o0?ooo`3oool0G`3o
ool010000000oooo0?ooo`0000080?ooo`<000000`3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo0580oooo1`000?le0?ooo`004P3oool00`000?l0oooo0?ooo`1P0?ooo`8000002`3o
ool010000000oooo0?ooo`3oool4000000D0oooo00<000000?ooo`3oool0F@3oool60000obl0oooo
000B0?ooo`030000o`3oool0oooo07X0oooo00<000000?ooo`3oool0G`3oool40000ob/0oooo000B
0?ooo`030000o`3oool0oooo07X0oooo00<000000?ooo`3oool0H`3oool40000obL0oooo000B0?oo
o`030000o`3oool0oooo07X0oooo0P00001X0?ooo`H0003o8@3oool00180oooo00<0003o0?ooo`3o
ool0NP3oool00`000000oooo0?ooo`1]0?ooo`L0003o6P3oool00180oooo00<0003o0?ooo`3oool0
NP3oool00`000000oooo0?ooo`1d0?ooo`H0003o503oool00180oooo00<0003o0?ooo`3oool0NP3o
ool00`000000oooo0?ooo`1j0?ooo`@0003o403oool00180oooo00<0003o0?ooo`3oool0NP3oool0
0`000000oooo0?ooo`1n0?ooo`@0003o303oool00180oooo00<0003o0?ooo`3oool0NP3oool00`00
0000oooo0?ooo`220?ooo`<0003o2@3oool00180oooo00<0003o0?ooo`3oool0NP3oool00`000000
oooo0?ooo`2>0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`800000S`3oool00180oooo00<0
003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`2>0?ooo`004P3oool00`000?l0oooo0?ooo`1j
0?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000o`3oool0oooo07X0oooo00<000000?oo
o`3oool0SP3oool00180oooo00<0003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`2>0?ooo`00
4P3oool00`000?l0oooo0?ooo`1j0?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000o`3o
ool0oooo07X0oooo00<000000?ooo`3oool0SP3oool00180oooo00<0003o0?ooo`3oool0NP3oool2
000008l0oooo000B0?ooo`030000o`3oool0oooo07X0oooo00<000000?ooo`3oool0SP3oool00180
oooo00<0003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`2>0?ooo`004P3oool00`000?l0oooo
0?ooo`1j0?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000o`3oool0oooo07X0oooo00<0
00000?ooo`3oool0SP3oool00140oooo00<0003o0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>
0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`800000S`3oool00140oooo00<0003o0?ooo`3o
ool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`030000
003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3o
ool00140oooo00<0003o0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`00
0?l0oooo0?ooo`1W0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000001@3oool00`00
0000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1V0?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0
oooo0?ooo`1V0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool30000
08h0oooo000A0?ooo`030000o`3oool0oooo0600oooo100000020?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0
oooo0?ooo`1V0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1V0?ooo`040000003oool0oooo0000
00P0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo06L0
oooo0P00000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool00140oooo
00<0003o0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?oo
o`1k0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo0P00002?
0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`030000003oool0oooo08h0oooo000A0?ooo`03
0000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3oool00140oooo00<0003o0?ooo`3oool0
N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`030000003o
ool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3oool0
0140oooo00<0003o0?ooo`3oool0N`3oool2000008l0oooo000A0?ooo`030000o`3oool0oooo07/0
oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool00`000000oooo
0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000@
0?ooo`030000o`3oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?oo
o`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`03
0000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo07`0oooo0P00002?0?ooo`00403o
ool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0
oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool00`00
0000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0
oooo000@0?ooo`030000o`3oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0
003o0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l
0?ooo`800000S`3oool00100oooo00<0003o0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?oo
o`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000
o`3oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03o
ool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1R0?ooo`800000103oool0
0`000000oooo0?ooo`030?ooo`8000000`3oool3000000H0oooo00<000000?ooo`3oool0SP3oool0
0100oooo00<0003o0?ooo`3oool0H@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo000000D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00403oool0
0`000?l0oooo0?ooo`1Q0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
1@3oool00`000000oooo0?ooo`030?ooo`<00000SP3oool00100oooo00<0003o0?ooo`3oool0F`3o
ool400000080oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`<0
00001P3oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1Q0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo0680oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0H`3oool2000000X0oooo0P0000030?oo
o`@000001@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`03
0000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo0P00002?0?ooo`003`3o
ool00`000?l0oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0
oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0O@3oool00`00
0000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003oool0oooo08h0
oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0oooo00<0
003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m
0?ooo`800000S`3oool000l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?oo
o`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000
o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0O@3o
ool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003oool0
oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0
oooo00<0003o0?ooo`3oool0O@3oool2000008l0oooo000?0?ooo`030000o`3oool0oooo07d0oooo
00<000000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?oo
o`2>0?ooo`002@3oool400000080oooo00<0003o0?ooo`3oool0=@3oool2000000@0oooo00<00000
0?ooo`3oool00P3oool3000003X0oooo00<000000?ooo`3oool0>P3oool2000000@0oooo00<00000
0?ooo`3oool00P3oool3000003X0oooo100000080?ooo`002`3oool01@000000oooo0?ooo`3oool0
003o03H0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0g0?ooo`030000003o
ool0oooo03T0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0i0?ooo`030000
003oool0oooo00L0oooo000;0?ooo`050000003oool0oooo0?ooo`000?l0=P3oool010000000oooo
0?ooo`00000;0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0>@3oool010000000
oooo0?ooo`00000;0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool01`3oool000<0
oooo100000040?ooo`040000003oool0oooo0000oc40oooo100000020?ooo`040000003oool0oooo
000000P0oooo0`00000j0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000203o
ool3000003`0oooo00<000000?ooo`3oool01`3oool000/0oooo00@000000?ooo`3oool0003o=`3o
ool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo03X0oooo0P00000j0?ooo`040000
003oool0oooo000000P0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`070?ooo`00
2@3oool300000080oooo00<0003o0?ooo`3oool0=@3oool010000000oooo0?ooo`0000080?ooo`03
0000003oool0oooo03X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`0000080?oo
o`030000003oool0oooo03X0oooo0`0000090?ooo`002`3oool010000000oooo0?ooo`000?lh0?oo
o`8000002@3oool4000003T0oooo00<000000?ooo`3oool0>P3oool2000000T0oooo1000000k0?oo
o`030000003oool0oooo00L0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<000000?ooo`3o
ool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`000000oooo0?ooo`2>0?ooo`003P3o
ool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo00000`3oool000000000000;
000000030000o`00000000000?l000003P0000020?ooo`00203oool00`000000oooo0?ooo`030?oo
o`030000o`3oool0oooo00@0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`070?oo
o`00203oool00`000000oooo0?ooo`030?ooo`030000o`3oool0oooo03X0oooo00<000000?ooo`3o
ool0@@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0400oooo00<000000?ooo`3o
ool01`3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`000000oooo0?ooo`2>0?ooo`003P3o
ool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.06243, -0.256687, \
0.00740377, 0.00148577}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Comparison to small a:", "Subsection"],

Cell[BoxData[
    \(choose\ order\ to\ compare\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(toorder = 5\)], "Input"],

Cell[BoxData[
    \(5\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(beta[md2_, k_, kz_, om_, a_] = 
        Normal[Series[
              a0 + a1*a + a2*a^2 + a3*a^3 + a4*a^4 + a5*a^5, {a, 0, 
                toorder}]] /. nz \[Rule] kz/k;\)\)], "Input"],

Cell[BoxData[
    \(Imaginary\ part\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Im[beta[1, 1, 1/10, 1/2, a]], {a, \(-1\), 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 0.603319 0.823674 [
[.02381 .59082 -6 -9 ]
[.02381 .59082 6 0 ]
[.2619 .59082 -12 -9 ]
[.2619 .59082 12 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.4875 .02675 -24 -4.5 ]
[.4875 .02675 0 4.5 ]
[.4875 .10911 -24 -4.5 ]
[.4875 .10911 0 4.5 ]
[.4875 .19148 -24 -4.5 ]
[.4875 .19148 0 4.5 ]
[.4875 .27385 -24 -4.5 ]
[.4875 .27385 0 4.5 ]
[.4875 .35622 -24 -4.5 ]
[.4875 .35622 0 4.5 ]
[.4875 .43858 -24 -4.5 ]
[.4875 .43858 0 4.5 ]
[.4875 .52095 -24 -4.5 ]
[.4875 .52095 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60332 m
.02381 .60957 L
s
[(-1)] .02381 .59082 0 1 Mshowa
.2619 .60332 m
.2619 .60957 L
s
[(-0.5)] .2619 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.5)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(1)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.5 .02675 m
.50625 .02675 L
s
[(-0.7)] .4875 .02675 1 0 Mshowa
.5 .10911 m
.50625 .10911 L
s
[(-0.6)] .4875 .10911 1 0 Mshowa
.5 .19148 m
.50625 .19148 L
s
[(-0.5)] .4875 .19148 1 0 Mshowa
.5 .27385 m
.50625 .27385 L
s
[(-0.4)] .4875 .27385 1 0 Mshowa
.5 .35622 m
.50625 .35622 L
s
[(-0.3)] .4875 .35622 1 0 Mshowa
.5 .43858 m
.50625 .43858 L
s
[(-0.2)] .4875 .43858 1 0 Mshowa
.5 .52095 m
.50625 .52095 L
s
[(-0.1)] .4875 .52095 1 0 Mshowa
.125 Mabswid
.5 .04322 m
.50375 .04322 L
s
.5 .05969 m
.50375 .05969 L
s
.5 .07617 m
.50375 .07617 L
s
.5 .09264 m
.50375 .09264 L
s
.5 .12559 m
.50375 .12559 L
s
.5 .14206 m
.50375 .14206 L
s
.5 .15853 m
.50375 .15853 L
s
.5 .17501 m
.50375 .17501 L
s
.5 .20796 m
.50375 .20796 L
s
.5 .22443 m
.50375 .22443 L
s
.5 .2409 m
.50375 .2409 L
s
.5 .25738 m
.50375 .25738 L
s
.5 .29032 m
.50375 .29032 L
s
.5 .3068 m
.50375 .3068 L
s
.5 .32327 m
.50375 .32327 L
s
.5 .33974 m
.50375 .33974 L
s
.5 .37269 m
.50375 .37269 L
s
.5 .38916 m
.50375 .38916 L
s
.5 .40564 m
.50375 .40564 L
s
.5 .42211 m
.50375 .42211 L
s
.5 .45506 m
.50375 .45506 L
s
.5 .47153 m
.50375 .47153 L
s
.5 .488 m
.50375 .488 L
s
.5 .50448 m
.50375 .50448 L
s
.5 .53742 m
.50375 .53742 L
s
.5 .5539 m
.50375 .5539 L
s
.5 .57037 m
.50375 .57037 L
s
.5 .58685 m
.50375 .58685 L
s
.5 .01027 m
.50375 .01027 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .09773 L
.10458 .17157 L
.14415 .22778 L
.18221 .27199 L
.22272 .31041 L
.26171 .34056 L
.30316 .36677 L
.34309 .3875 L
.3815 .40416 L
.42237 .41909 L
.46172 .43131 L
.49955 .44148 L
.53984 .45092 L
.57861 .4589 L
.61984 .46641 L
.65954 .47288 L
.69774 .47857 L
.73838 .48425 L
.77751 .48962 L
.81909 .49562 L
.85916 .50218 L
.89771 .50979 L
.93871 .52011 L
.97619 .53245 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00S`3oool2000008l0oooo00070?ooo`030000003oool0oooo08D0oooo00<0
00000?ooo`3oool0SP3oool000L0oooo00<000000?ooo`3oool0L@3oool2000000@0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo08h0oooo00080?oo
o`030000003oool0oooo06l0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo08h0oooo00080?ooo`030000003oool0oooo06l0oooo00@000000?oo
o`3oool000002P3oool00`000000oooo0?ooo`040?ooo`<00000SP3oool000T0oooo00<000000?oo
o`3oool0J03oool400000080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo08h0oooo00090?ooo`030000003oool0oooo06h0oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000:0?oo
o`030000003oool0oooo06d0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00050?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000003oool0oooo06h0oooo0P000009
0?ooo`@000001@3oool00`000000oooo0?ooo`2>0?ooo`002`3oool00`000000oooo0?ooo`210?oo
o`800000S`3oool000/0oooo00<000000?ooo`3oool0P@3oool00`000000oooo0?ooo`2>0?ooo`00
303oool00`000000oooo0?ooo`200?ooo`030000003oool0oooo08h0oooo000<0?ooo`030000003o
ool0oooo0800oooo00<000000?ooo`3oool0SP3oool000`0oooo00<000000?ooo`3oool0P03oool0
0`000000oooo0?ooo`2>0?ooo`003@3oool00`000000oooo0?ooo`1o0?ooo`800000S`3oool000d0
oooo00<000000?ooo`3oool0O`3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000000oooo
0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000003oool0oooo07h0oooo00<0
00000?ooo`3oool0SP3oool000l0oooo00<000000?ooo`3oool0O@3oool2000008l0oooo000?0?oo
o`030000003oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool00100oooo00<000000?ooo`3o
ool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000000oooo0?ooo`1l0?ooo`030000
003oool0oooo08h0oooo000A0?ooo`030000003oool0oooo07/0oooo00<000000?ooo`3oool0SP3o
ool00140oooo00<000000?ooo`3oool0N`3oool2000008l0oooo000B0?ooo`030000003oool0oooo
07X0oooo00<000000?ooo`3oool0SP3oool00180oooo00<000000?ooo`3oool0NP3oool00`000000
oooo0?ooo`2>0?ooo`004`3oool00`000000oooo0?ooo`1U0?ooo`800000103oool00`000000oooo
0?ooo`030?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000000oooo0?oo
o`1T0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`000000
oooo0?ooo`2>0?ooo`00503oool00`000000oooo0?ooo`1S0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000001@3oool3000008h0oooo000D0?ooo`030000003oool0oooo05d0
oooo100000020?ooo`040000003oool0oooo000000P0oooo0`0000060?ooo`030000003oool0oooo
08h0oooo000E0?ooo`030000003oool0oooo0680oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000003oool0oooo0680
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
08h0oooo000F0?ooo`030000003oool0oooo0680oooo0P00000:0?ooo`<000001@3oool00`000000
oooo0?ooo`2>0?ooo`005`3oool00`000000oooo0?ooo`1e0?ooo`800000S`3oool001L0oooo00<0
00000?ooo`3oool0M@3oool00`000000oooo0?ooo`2>0?ooo`00603oool00`000000oooo0?ooo`1d
0?ooo`030000003oool0oooo08h0oooo000H0?ooo`030000003oool0oooo07@0oooo00<000000?oo
o`3oool0SP3oool001T0oooo00<000000?ooo`3oool0L`3oool2000008l0oooo000I0?ooo`030000
003oool0oooo07<0oooo00<000000?ooo`3oool0SP3oool001X0oooo00<000000?ooo`3oool0LP3o
ool00`000000oooo0?ooo`2>0?ooo`006`3oool00`000000oooo0?ooo`1a0?ooo`030000003oool0
oooo08h0oooo000K0?ooo`030000003oool0oooo0740oooo00<000000?ooo`3oool0SP3oool001`0
oooo00<000000?ooo`3oool0L03oool2000008l0oooo000L0?ooo`030000003oool0oooo0700oooo
00<000000?ooo`3oool0SP3oool001d0oooo00<000000?ooo`3oool0K`3oool00`000000oooo0?oo
o`2>0?ooo`007@3oool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo08h0oooo000N0?oo
o`030000003oool0oooo06h0oooo00<000000?ooo`3oool0SP3oool001l0oooo00<000000?ooo`3o
ool0K@3oool2000008l0oooo000O0?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0
SP3oool00200oooo00<000000?ooo`3oool0F03oool2000000@0oooo00<000000?ooo`3oool00P3o
ool3000000H0oooo00<000000?ooo`3oool0SP3oool00240oooo00<000000?ooo`3oool0EP3oool0
10000000oooo0?ooo`00000;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0SP3o
ool00240oooo00<000000?ooo`3oool0EP3oool010000000oooo0?ooo`00000;0?ooo`030000003o
ool0oooo00<0oooo0`00002>0?ooo`008P3oool00`000000oooo0?ooo`1?0?ooo`@000000P3oool0
10000000oooo0?ooo`0000080?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`008`3oool0
0`000000oooo0?ooo`1D0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`2>0?ooo`008`3oool00`000000oooo0?ooo`1D0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2>0?ooo`00903oool0
0`000000oooo0?ooo`1D0?ooo`8000002@3oool4000000D0oooo00<000000?ooo`3oool0SP3oool0
02D0oooo00<000000?ooo`3oool0I`3oool2000008l0oooo000V0?ooo`030000003oool0oooo06H0
oooo00<000000?ooo`3oool0SP3oool002H0oooo00<000000?ooo`3oool0IP3oool00`000000oooo
0?ooo`2>0?ooo`009`3oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo08h0oooo000X
0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool0SP3oool002P0oooo00<000000?oo
o`3oool0I03oool2000008l0oooo000Y0?ooo`030000003oool0oooo06<0oooo00<000000?ooo`3o
ool0SP3oool002X0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`2>0?ooo`00:`3o
ool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo08h0oooo000/0?ooo`030000003oool0
oooo0600oooo00<000000?ooo`3oool0SP3oool002d0oooo00<000000?ooo`3oool0G`3oool20000
08l0oooo000^0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0SP3oool002h0oooo
00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`2>0?ooo`00;`3oool00`000000oooo0?oo
o`1M0?ooo`030000003oool0oooo08h0oooo000`0?ooo`030000003oool0oooo05`0oooo0P00002?
0?ooo`00<@3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo08h0oooo000b0?ooo`03
0000003oool0oooo05X0oooo00<000000?ooo`3oool0SP3oool003<0oooo00<000000?ooo`3oool0
A@3oool2000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo08h0oooo000d0?ooo`030000003oool0oooo04<0oooo00@000000?ooo`3oool00000
2P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000e0?ooo`030000003o
ool0oooo0480oooo00@000000?ooo`3oool000001`3oool5000000D0oooo0`00002>0?ooo`00=P3o
ool00`000000oooo0?ooo`0k0?ooo`@000000P3oool010000000oooo0?ooo`0000070?ooo`040000
003oool0oooo000000H0oooo00<000000?ooo`3oool0SP3oool003L0oooo00<000000?ooo`3oool0
@03oool010000000oooo0?ooo`0000080?ooo`030000003oool0000000H0oooo00<000000?ooo`3o
ool0SP3oool003P0oooo00<000000?ooo`3oool0?`3oool010000000oooo0?ooo`0000090?ooo`80
00001P3oool00`000000oooo0?ooo`2>0?ooo`00>@3oool00`000000oooo0?ooo`0o0?ooo`800000
2`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000j0?ooo`800000D`3o
ool2000008l0oooo000l0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool0SP3oool0
03d0oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`2>0?ooo`00?P3oool00`000000
oooo0?ooo`1>0?ooo`030000003oool0oooo08h0oooo000o0?ooo`030000003oool0oooo04d0oooo
0P00002?0?ooo`00@03oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo08h0oooo0011
0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0SP3oool00480oooo0P00001;0?oo
o`030000003oool0oooo08h0oooo00140?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3o
ool0SP3oool004D0oooo00<000000?ooo`3oool0A`3oool2000008l0oooo00160?ooo`030000003o
ool0oooo04H0oooo00<000000?ooo`3oool0SP3oool004L0oooo00<000000?ooo`3oool0A@3oool0
0`000000oooo0?ooo`2>0?ooo`00B03oool2000004D0oooo00<000000?ooo`3oool0SP3oool004X0
oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`2>0?ooo`00B`3oool00`000000oooo
0?ooo`110?ooo`800000S`3oool004`0oooo0P0000110?ooo`030000003oool0oooo08h0oooo001>
0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0SP3oool004l0oooo0P00000Z0?oo
o`800000103oool00`000000oooo0?ooo`030?ooo`8000001P3oool00`000000oooo0?ooo`2>0?oo
o`00D@3oool00`000000oooo0?ooo`0V0?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool000001@3oool00`000000oooo0?ooo`2>0?ooo`00DP3oool2000002H0oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`030?ooo`<00000SP3oool005@0oooo00<000000?oo
o`3oool07@3oool400000080oooo00@000000?ooo`3oool000002@3oool2000000H0oooo00<00000
0?ooo`3oool0SP3oool005D0oooo0P00000S0?ooo`040000003oool0oooo000000/0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00E`3oool200000240oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo001I
0?ooo`800000803oool2000000X0oooo0P0000060?ooo`800000S`3oool005/0oooo0P00000b0?oo
o`030000003oool0oooo08h0oooo001M0?ooo`800000<03oool00`000000oooo0?ooo`2>0?ooo`00
G`3oool2000002h0oooo00<000000?ooo`3oool0SP3oool00640oooo0`00000[0?ooo`030000003o
ool0oooo08h0oooo001T0?ooo`800000:@3oool2000008l0oooo001V0?ooo`8000009`3oool00`00
0000oooo0?ooo`2>0?ooo`00J03oool2000002D0oooo00<000000?ooo`3oool0SP3oool006X0oooo
0P00000S0?ooo`030000003oool0oooo08h0oooo001/0?ooo`<00000803oool00`000000oooo0?oo
o`2>0?ooo`00K`3oool2000001h0oooo0P00002?0?ooo`00L@3oool2000001`0oooo00<000000?oo
o`3oool0SP3oool007<0oooo0`00000I0?ooo`030000003oool0oooo08h0oooo001f0?ooo`800000
5`3oool00`000000oooo0?ooo`2>0?ooo`00N03oool3000001@0oooo0P00002?0?ooo`00N`3oool4
00000100oooo00<000000?ooo`3oool0SP3oool007l0oooo1000000<0?ooo`030000003oool0oooo
08h0oooo001k0?ooo`800000103oool00`000000oooo00000006000000D0oooo00<000000?ooo`3o
ool0SP3oool007X0oooo00@000000?ooo`3oool00000203oool4000000D0oooo00<000000?ooo`3o
ool0SP3oool007X0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`0400000004
0?ooo`0000000000000008h0oooo001d0?ooo`@000000P3oool010000000oooo0?ooo`00000:0?oo
o`030000003oool0oooo00<0oooo0`00002?0?ooo`00NP3oool010000000oooo0?ooo`00000;0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`0000000`00002;0?ooo`00NP3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00`3oool4
000008L0oooo001k0?ooo`8000002P3oool2000000H0oooo00<000000?ooo`3oool01`3oool40000
08<0oooo002?0?ooo`800000303oool6000007d0oooo002?0?ooo`030000003oool0oooo0140oooo
1P00001g0?ooo`00S`3oool00`000000oooo0?ooo`0G0?ooo`H00000L@3oool008l0oooo00<00000
0?ooo`3oool07@3oool5000006`0oooo002?0?ooo`030000003oool0oooo0280oooo1P00001V0?oo
o`00S`3oool2000002T0oooo1P00001P0?ooo`00S`3oool00`000000oooo0?ooo`0^0?ooo`H00000
FP3oool008l0oooo00<000000?ooo`3oool0=03oool800000580oooo002?0?ooo`030000003oool0
oooo03`0oooo2000001:0?ooo`00S`3oool2000004D0oooo1P0000140?ooo`00S`3oool00`000000
oooo0?ooo`1:0?ooo`P00000?03oool008l0oooo00<000000?ooo`3oool0DP3oool9000003<0oooo
002?0?ooo`030000003oool0oooo05/0oooo1P00000]0?ooo`00S`3oool00`000000oooo0?ooo`1Q
0?ooo`D00000:03oool008l0oooo0P00001W0?ooo`@00000903oool008l0oooo00<000000?ooo`3o
ool0JP3oool400000200oooo002?0?ooo`030000003oool0oooo06h0oooo1000000L0?ooo`00N`3o
ool2000000@0oooo00<000000?ooo`3oool00P3oool4000000D0oooo00<000000?ooo`3oool0LP3o
ool6000001H0oooo001j0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`1h0?ooo`D000004@3oool007X0oooo00@000000?ooo`3oool000002P3o
ool00`000000oooo0?ooo`040?ooo`<00000O@3oool2000000l0oooo001d0?ooo`@000000P3oool0
10000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0O`3o
ool3000000`0oooo001j0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`220?ooo`8000002P3oool007X0oooo00@000000?ooo`3oool00000203o
ool3000000H0oooo00<000000?ooo`3oool0Q03oool2000000P0oooo001k0?ooo`8000002`3oool0
0`000000oooo0?ooo`040?ooo`800000S`3oool008l0oooo00<000000?ooo`3oool0SP3oool008l0
oooo00<000000?ooo`3oool0SP3oool008l0oooo00<000000?ooo`3oool0SP3oool008l0oooo00<0
00000?ooo`3oool0SP3oool008l0oooo0P00002?0?ooo`00S`3oool00`000000oooo0?ooo`2>0?oo
o`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00203o
ool4000003/0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00000j0?ooo`030000003o
ool0oooo03X0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00000k0?ooo`@000001`3o
ool000X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00000;0?ooo`030000003o
ool0oooo03L0oooo0P00000j0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0
>P3oool00`000000oooo0?ooo`060?ooo`002P3oool00`000000oooo0?ooo`0i0?ooo`040000003o
ool0oooo000000/0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0i0?ooo`040000
003oool0oooo000000/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`060?ooo`00
0P3oool4000000@0oooo00<000000?ooo`3oool0<`3oool400000080oooo00@000000?ooo`3oool0
0000203oool3000003X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`0000080?oo
o`<00000?@3oool00`000000oooo0?ooo`060?ooo`002P3oool00`000000oooo0?ooo`0i0?ooo`04
0000003oool0oooo000000P0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0i0?oo
o`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`06
0?ooo`00203oool3000003/0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0j
0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?oo
o`0k0?ooo`<00000203oool000X0oooo00<000000?ooo`3oool0>P3oool2000000T0oooo1000000i
0?ooo`800000>`3oool2000000T0oooo1000000l0?ooo`030000003oool0oooo00H0oooo002?0?oo
o`030000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?ooo`030000
003oool0oooo08h0oooo003o0000020000000@3oool000L0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00H0oooo00070?ooo`030000003oool0oooo
0440oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo
0440oooo00<000000?ooo`3oool01P3oool008l0oooo00<000000?ooo`3oool0SP3oool008l0oooo
00<000000?ooo`3oool0SP3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.0526, -0.732477, \
0.00733521, 0.00424071}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, ImPlot1]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.47619 0.603319 0.548157 [
[.02381 .59082 -6 -9 ]
[.02381 .59082 6 0 ]
[.2619 .59082 -12 -9 ]
[.2619 .59082 12 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.4875 .05516 -12 -4.5 ]
[.4875 .05516 0 4.5 ]
[.4875 .16479 -24 -4.5 ]
[.4875 .16479 0 4.5 ]
[.4875 .27442 -24 -4.5 ]
[.4875 .27442 0 4.5 ]
[.4875 .38406 -24 -4.5 ]
[.4875 .38406 0 4.5 ]
[.4875 .49369 -24 -4.5 ]
[.4875 .49369 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60332 m
.02381 .60957 L
s
[(-1)] .02381 .59082 0 1 Mshowa
.2619 .60332 m
.2619 .60957 L
s
[(-0.5)] .2619 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.5)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(1)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.5 .05516 m
.50625 .05516 L
s
[(-1)] .4875 .05516 1 0 Mshowa
.5 .16479 m
.50625 .16479 L
s
[(-0.8)] .4875 .16479 1 0 Mshowa
.5 .27442 m
.50625 .27442 L
s
[(-0.6)] .4875 .27442 1 0 Mshowa
.5 .38406 m
.50625 .38406 L
s
[(-0.4)] .4875 .38406 1 0 Mshowa
.5 .49369 m
.50625 .49369 L
s
[(-0.2)] .4875 .49369 1 0 Mshowa
.125 Mabswid
.5 .08257 m
.50375 .08257 L
s
.5 .10998 m
.50375 .10998 L
s
.5 .13739 m
.50375 .13739 L
s
.5 .1922 m
.50375 .1922 L
s
.5 .21961 m
.50375 .21961 L
s
.5 .24702 m
.50375 .24702 L
s
.5 .30183 m
.50375 .30183 L
s
.5 .32924 m
.50375 .32924 L
s
.5 .35665 m
.50375 .35665 L
s
.5 .41146 m
.50375 .41146 L
s
.5 .43887 m
.50375 .43887 L
s
.5 .46628 m
.50375 .46628 L
s
.5 .5211 m
.50375 .5211 L
s
.5 .5485 m
.50375 .5485 L
s
.5 .57591 m
.50375 .57591 L
s
.5 .02775 m
.50375 .02775 L
s
.5 .00035 m
.50375 .00035 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .2116 m
.06244 .26685 L
.10458 .31599 L
.14415 .3534 L
.18221 .38282 L
.22272 .40838 L
.26171 .42845 L
.30316 .44589 L
.34309 .45969 L
.3815 .47078 L
.42237 .48071 L
.46172 .48885 L
.49955 .49561 L
.53984 .5019 L
.57861 .50721 L
.61984 .51221 L
.65954 .51651 L
.69774 .5203 L
.73838 .52408 L
.77751 .52766 L
.81909 .53165 L
.85916 .53601 L
.89771 .54108 L
.93871 .54795 L
.97619 .55615 L
s
0 0 1 r
.02381 .01472 m
.07143 .21022 L
.11905 .30565 L
.16667 .3626 L
.21429 .40056 L
.2619 .4277 L
.30952 .44808 L
.35714 .46394 L
.40476 .47664 L
.45238 .48703 L
.5 .49569 L
.54762 .50302 L
.59524 .5093 L
.64286 .51474 L
.69048 .5195 L
.7381 .5237 L
.78571 .52744 L
.83333 .53078 L
.88095 .5338 L
.92857 .53652 L
.97619 .539 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool2
000008l0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0
oooo002?0?ooo`030000003oool0oooo08h0oooo00070?ooo`030000o`3oool0oooo08D0oooo00<0
00000?ooo`3oool0SP3oool000L0oooo00<0003o0?ooo`3oool0Q@3oool00`000000oooo0?ooo`2>
0?ooo`001`3oool00`000?l0oooo0?ooo`250?ooo`030000003oool0oooo08h0oooo00080?ooo`03
0000o`3oool0oooo08@0oooo00<000000?ooo`3oool0SP3oool000P0oooo00<0003o0?ooo`3oool0
Q03oool2000008l0oooo00080?ooo`030000o`3oool0oooo08@0oooo00<000000?ooo`3oool0SP3o
ool000P0oooo00<0003o0?ooo`3oool0Q03oool00`000000oooo0?ooo`2>0?ooo`002@3oool00`00
0?l0oooo0?ooo`230?ooo`030000003oool0oooo08h0oooo00090?ooo`030000o`3oool0oooo08<0
oooo00<000000?ooo`3oool0SP3oool000T0oooo00<0003o0?ooo`3oool0NP3oool4000000D0oooo
00<000000?ooo`3oool0SP3oool000T0oooo00<0003o0?ooo`3oool0O03oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000o`3oool0oooo07/0oooo00<00000
0?ooo`3oool0103oool3000008h0oooo000:0?ooo`030000o`3oool0oooo07<0oooo100000040?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool000X0oooo00<0003o0?ooo`3o
ool0N`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000
o`3oool0oooo07T0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000o`3o
ool0oooo07/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`002`3oool0
0`000?l0oooo0?ooo`210?ooo`030000003oool0oooo08h0oooo000;0?ooo`030000o`3oool0oooo
0840oooo00<000000?ooo`3oool0SP3oool000/0oooo00<0003o0?ooo`3oool0P@3oool00`000000
oooo0?ooo`2>0?ooo`002`3oool00`000?l0oooo0?ooo`210?ooo`800000S`3oool000`0oooo00<0
003o0?ooo`3oool0P03oool00`000000oooo0?ooo`2>0?ooo`00303oool00`000?l0oooo0?ooo`20
0?ooo`030000003oool0oooo08h0oooo000<0?ooo`030000o`3oool0oooo0800oooo00<000000?oo
o`3oool0SP3oool000`0oooo00<0003o0?ooo`3oool0P03oool00`000000oooo0?ooo`2>0?ooo`00
3@3oool00`000?l0oooo0?ooo`1o0?ooo`030000003oool0oooo08h0oooo000=0?ooo`030000o`3o
ool0oooo07l0oooo00<000000?ooo`3oool0SP3oool000d0oooo00<0003o0?ooo`3oool0O`3oool0
0`000000oooo0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1o0?ooo`800000S`3oool000d0
oooo00<0003o0?ooo`3oool0O`3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo
0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<0
00000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`000000oooo0?ooo`2>
0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000?0?ooo`03
0000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0
O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`800000S`3o
ool000l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?ooo`00403oool00`00
0?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo07`0
oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool00`000000oooo
0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000A
0?ooo`030000o`3oool0oooo06L0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000006
0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo06H0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo000A
0?ooo`030000o`3oool0oooo06H0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000050?ooo`<00000SP3oool00140oooo00<0003o0?ooo`3oool0H03oool400000080oooo00@0
00000?ooo`3oool000002@3oool2000000H0oooo00<000000?ooo`3oool0SP3oool00140oooo00<0
003o0?ooo`3oool0IP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool0SP3oool00180oooo00<0003o0?ooo`3oool0I@3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0SP3oool00180
oooo00<0003o0?ooo`3oool0IP3oool2000000X0oooo0P0000060?ooo`030000003oool0oooo08h0
oooo000B0?ooo`030000o`3oool0oooo07X0oooo00<000000?ooo`3oool0SP3oool00180oooo00<0
003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo0P00002?0?oo
o`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000
o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0N03o
ool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`1h0?ooo`030000003oool0
oooo08h0oooo00070?ooo`030000003oool0oooo00X0oooo00<0003o0?ooo`3oool0N03oool00`00
0000oooo0?ooo`2>0?ooo`00203oool00`000000oooo0?ooo`0:0?ooo`030000o`3oool0oooo07L0
oooo00<000000?ooo`3oool0SP3oool000P0oooo00<000000?ooo`3oool02P3oool00`000?l0oooo
0?ooo`1g0?ooo`800000S`3oool000T0oooo00<000000?ooo`3oool02P3oool00`000?l0oooo0?oo
o`1f0?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000003oool0oooo00T0oooo00<0003o
0?ooo`3oool0MP3oool00`000000oooo0?ooo`2>0?ooo`002P3oool00`000000oooo0?ooo`0:0?oo
o`030000o`3oool0oooo07D0oooo00<000000?ooo`3oool0SP3oool000/0oooo00<000000?ooo`3o
ool02@3oool00`000?l0oooo0?ooo`1e0?ooo`030000003oool0oooo08h0oooo000<0?ooo`030000
003oool0oooo00T0oooo00<0003o0?ooo`3oool0M03oool00`000000oooo0?ooo`2>0?ooo`00303o
ool00`000000oooo0?ooo`090?ooo`030000o`3oool0oooo07@0oooo00<000000?ooo`3oool0SP3o
ool000d0oooo00<000000?ooo`3oool02@3oool00`000?l0oooo0?ooo`1c0?ooo`030000003oool0
oooo08h0oooo000>0?ooo`030000003oool0oooo00P0oooo00<0003o0?ooo`3oool0L`3oool20000
08l0oooo000?0?ooo`030000003oool0oooo00P0oooo00<0003o0?ooo`3oool0LP3oool00`000000
oooo0?ooo`2>0?ooo`003`3oool00`000000oooo0?ooo`080?ooo`030000o`3oool0oooo0780oooo
00<000000?ooo`3oool0SP3oool00100oooo00<000000?ooo`3oool0203oool00`000?l0oooo0?oo
o`1a0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000003oool0oooo00L0oooo00<0003o
0?ooo`3oool0L@3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000000oooo0?ooo`080?oo
o`030000o`3oool0oooo0700oooo00<000000?ooo`3oool0SP3oool00180oooo00<000000?ooo`3o
ool01`3oool00`000?l0oooo0?ooo`1L0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`80
00001P3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000000oooo0?ooo`070?ooo`030000
o`3oool0oooo05X0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?oo
o`030000003oool0oooo08h0oooo000D0?ooo`030000003oool0oooo00H0oooo00<0003o0?ooo`3o
ool0FP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo0`00002>
0?ooo`005@3oool00`000000oooo0?ooo`060?ooo`030000o`3oool0oooo05<0oooo100000020?oo
o`040000003oool0oooo000000P0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000E0?oo
o`030000003oool0oooo00H0oooo00<0003o0?ooo`3oool0F@3oool010000000oooo0?ooo`000008
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool001H0oooo00<000000?oo
o`3oool01P3oool00`000?l0oooo0?ooo`1H0?ooo`040000003oool0oooo000000P0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`2>0?ooo`005`3oool00`000000oooo0?ooo`050?oo
o`030000o`3oool0oooo05T0oooo0P00000:0?ooo`<000001@3oool00`000000oooo0?ooo`2>0?oo
o`00603oool00`000000oooo0?ooo`050?ooo`030000o`3oool0oooo06`0oooo00<000000?ooo`3o
ool0SP3oool001T0oooo00<000000?ooo`3oool0103oool00`000?l0oooo0?ooo`1/0?ooo`030000
003oool0oooo08h0oooo000J0?ooo`030000003oool0oooo00@0oooo00<0003o0?ooo`3oool0J`3o
ool00`000000oooo0?ooo`2>0?ooo`006`3oool00`000000oooo0?ooo`030?ooo`030000o`3oool0
oooo06/0oooo0P00002?0?ooo`006`3oool00`000000oooo0?ooo`040?ooo`030000o`3oool0oooo
06X0oooo00<000000?ooo`3oool0SP3oool001`0oooo00<000000?ooo`3oool0103oool00`000?l0
oooo0?ooo`1Y0?ooo`030000003oool0oooo08h0oooo000M0?ooo`030000003oool0oooo00@0oooo
00<0003o0?ooo`3oool0J03oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000000oooo0?oo
o`040?ooo`030000o`3oool0oooo06L0oooo00<000000?ooo`3oool0SP3oool001l0oooo00<00000
0?ooo`3oool00`3oool00`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo08h0oooo000P0?oo
o`030000003oool0oooo00<0oooo00<0003o0?ooo`3oool0IP3oool00`000000oooo0?ooo`2>0?oo
o`008@3oool00`000000oooo0?ooo`030?ooo`030000o`3oool0oooo06D0oooo00<000000?ooo`3o
ool0SP3oool00280oooo00<000000?ooo`3oool00`3oool00`000?l0oooo0?ooo`1T0?ooo`800000
S`3oool002<0oooo00<000000?ooo`3oool00`3oool00`000?l0oooo0?ooo`1S0?ooo`030000003o
ool0oooo08h0oooo000T0?ooo`030000003oool0oooo00<0oooo00<0003o0?ooo`3oool0HP3oool0
0`000000oooo0?ooo`2>0?ooo`009@3oool00`000000oooo0?ooo`030?ooo`030000o`3oool0oooo
0640oooo00<000000?ooo`3oool0SP3oool002H0oooo00<000000?ooo`3oool00`3oool00`000?l0
oooo0?ooo`1P0?ooo`030000003oool0oooo08h0oooo000W0?ooo`030000003oool0oooo0080oooo
00<0003o0?ooo`3oool0H03oool00`000000oooo0?ooo`2>0?ooo`00:03oool00`000000oooo0?oo
o`020?ooo`030000o`3oool0oooo05l0oooo00<000000?ooo`3oool0SP3oool002T0oooo00<00000
0?ooo`3oool00P3oool00`000?l0oooo0?ooo`1N0?ooo`030000003oool0oooo08h0oooo000Z0?oo
o`8000000`3oool00`000?l0oooo0?ooo`1M0?ooo`800000S`3oool002`0oooo00D000000?ooo`3o
ool0oooo0000o`1N0?ooo`030000003oool0oooo08h0oooo000]0?ooo`050000003oool0oooo0?oo
o`000?l0G@3oool00`000000oooo0?ooo`2>0?ooo`00;P3oool200000080oooo00<0003o0?ooo`3o
ool0FP3oool00`000000oooo0?ooo`2>0?ooo`00<03oool00`000000oooo0?ooo`020000oeX0oooo
00<000000?ooo`3oool0SP3oool00340oooo00D000000?ooo`3oool0oooo0000o`1I0?ooo`030000
003oool0oooo08h0oooo000b0?ooo`8000000P3oool00`000?l0oooo0?ooo`120?ooo`800000103o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3o
ool003@0oooo00@000000?ooo`3oool0003o@P3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool003D0oooo0P0000000`3oool0003o0?oo
o`100?ooo`040000003oool0oooo000000L0oooo1@0000050?ooo`<00000SP3oool003L0oooo0P00
00020000ocT0oooo100000020?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool0
00001P3oool00`000000oooo0?ooo`2>0?ooo`00>@3oool00`000000oooo0000o`0n0?ooo`040000
003oool0oooo000000P0oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`2>0?ooo`00
>P3oool2000000030000o`3oool0oooo03/0oooo00@000000?ooo`3oool000002@3oool2000000H0
oooo00<000000?ooo`3oool0SP3oool003`0oooo00<000000000o`3oool0?03oool2000000/0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`00?P3oool20000odl0oooo00<0
00000?ooo`3oool0SP3oool00400oooo0P000?m=0?ooo`030000003oool0oooo08h0oooo00110?oo
o`03000000000?l0003o04/0oooo0P00002?0?ooo`00@`3oool00`000000003o0?ooo`190?ooo`03
0000003oool0oooo08h0oooo00150?ooo`80003oB03oool00`000000oooo0?ooo`2>0?ooo`00A`3o
ool20000odH0oooo00<000000?ooo`3oool0SP3oool004T0oooo0P000?m40?ooo`030000003oool0
oooo08h0oooo001;0?ooo`80003o@P3oool00`000000oooo0?ooo`2>0?ooo`00C@3oool20000od00
oooo00<000000?ooo`3oool0SP3oool004l0oooo0P000?ln0?ooo`030000003oool0oooo08h0oooo
001A0?ooo`80003o00<000000?ooo`3oool0>@3oool2000008l0oooo001C0?ooo`80003o00<00000
0?ooo`3oool0=`3oool00`000000oooo0?ooo`2>0?ooo`00E@3oool20000o`800000=P3oool00`00
0000oooo0?ooo`2>0?ooo`00E`3oool30000o`800000<`3oool00`000000oooo0?ooo`2>0?ooo`00
FP3oool40000oc40oooo00<000000?ooo`3oool0SP3oool005h0oooo0`000?l^0?ooo`030000003o
ool0oooo08h0oooo001Q0?ooo`@0003o:P3oool00`000000oooo0?ooo`2>0?ooo`00I03oool01000
0000003o0000o`000?lW0?ooo`030000003oool0oooo08h0oooo001X0?ooo`@0003o8`3oool20000
08l0oooo001/0?ooo`<0003o803oool00`000000oooo0?ooo`2>0?ooo`00K`3oool40000oa`0oooo
00<000000?ooo`3oool0SP3oool007<0oooo10000?lH0?ooo`030000003oool0oooo08h0oooo001g
0?ooo`@0003o503oool00`000000oooo0?ooo`2>0?ooo`00N`3oool40000oa00oooo00<000000?oo
o`3oool0SP3oool007/0oooo0P0000020?ooo`H0003o00<0oooo0000000000000P0000050?ooo`03
0000003oool0oooo08h0oooo001j0?ooo`040000003oool0oooo000000D0oooo0P0000070000o`<0
oooo00<000000?ooo`3oool0SP3oool007X0oooo00@000000?ooo`3oool000002@3oool00`000000
oooo0?ooo`02000000H0003oSP3oool007@0oooo100000020?ooo`040000003oool0oooo000000X0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040000o`800000R03oool007X0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00@0
oooo1@000?l2000008<0oooo001j0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000001@3oool00`000000oooo0?ooo`090?ooo`T0003oO03oool007/0oooo0P00000:0?ooo`80
00001P3oool00`000000oooo0?ooo`0A0?ooo`03000000000?l0003o00P0003oLP3oool008l0oooo
00<000000?ooo`3oool06P3oool2000000L0003o0P00001Y0?ooo`00S`3oool00`000000oooo0?oo
o`0S0?ooo`X0003o00<000000?ooo`3oool0GP3oool008l0oooo00<000000?ooo`3oool0;@3oool>
0000oe<0oooo002?0?ooo`800000=@3oool7000000X0003oB@3oool008l0oooo00<000000?ooo`3o
ool0?03oool9000000L0003o@P3oool008l0oooo00<000000?ooo`3oool0A`3oool5000000X0003o
>03oool008l0oooo00<000000?ooo`3oool0DP3oool4000001/0003o7@3oool008l0oooo00<00000
0?ooo`3oool0GP3oool<000000L0oooo3P000?l?0?ooo`00S`3oool00`000000oooo0?ooo`1Z0?oo
o`P000003@3oool70000o`P0oooo002?0?ooo`030000003oool0oooo0780oooo1P00000F0?ooo`00
S`3oool00`000000oooo0?ooo`1h0?ooo`D000004@3oool008l0oooo0P00001n0?ooo`<000003P3o
ool008l0oooo00<000000?ooo`3oool0P03oool4000000X0oooo002?0?ooo`030000003oool0oooo
08@0oooo0P0000080?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo
0?ooo`2>0?ooo`00203oool4000003/0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00
000j0?ooo`030000003oool0oooo03X0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00
000k0?ooo`@000001`3oool000X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00
000;0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool01P3oool000X0oooo00<0
00000?ooo`3oool0>@3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo03L0oooo
0P00000j0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0>P3oool00`000000
oooo0?ooo`060?ooo`000P3oool4000000@0oooo00<000000?ooo`3oool0<`3oool400000080oooo
00@000000?ooo`3oool00000203oool3000003X0oooo00<000000?ooo`3oool0>@3oool010000000
oooo0?ooo`0000080?ooo`<00000?@3oool00`000000oooo0?ooo`060?ooo`002P3oool00`000000
oooo0?ooo`0i0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0>P3oool00`00
0000oooo0?ooo`0i0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0?@3oool0
0`000000oooo0?ooo`060?ooo`00203oool3000003/0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo0?ooo`0j0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000203o
ool00`000000oooo0?ooo`0k0?ooo`<00000203oool000X0oooo00<000000?ooo`3oool0>P3oool2
000000T0oooo1000000i0?ooo`030000003oool0oooo03X0oooo0P0000090?ooo`@00000?03oool0
0`000000oooo0?ooo`060?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00o`00000P00000040oooo0007
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`06
0?ooo`001`3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0440oooo00<000000?oo
o`3oool0@@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo00H0oooo002?0?ooo`03
0000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05261, -1.10064, \
0.00733523, 0.00637219}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Real\ \(\(part\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[beta[1, 1, 1/10, 1/2, a]], {a, \(-1\), 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 2.20302 10.531 [
[.02381 .08432 -6 -9 ]
[.02381 .08432 6 0 ]
[.2619 .08432 -12 -9 ]
[.2619 .08432 12 0 ]
[.7381 .08432 -9 -9 ]
[.7381 .08432 9 0 ]
[.97619 .08432 -3 -9 ]
[.97619 .08432 3 0 ]
[.4875 .20213 -30 -4.5 ]
[.4875 .20213 0 4.5 ]
[.4875 .30744 -30 -4.5 ]
[.4875 .30744 0 4.5 ]
[.4875 .41275 -30 -4.5 ]
[.4875 .41275 0 4.5 ]
[.4875 .51806 -30 -4.5 ]
[.4875 .51806 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .09682 m
.02381 .10307 L
s
[(-1)] .02381 .08432 0 1 Mshowa
.2619 .09682 m
.2619 .10307 L
s
[(-0.5)] .2619 .08432 0 1 Mshowa
.7381 .09682 m
.7381 .10307 L
s
[(0.5)] .7381 .08432 0 1 Mshowa
.97619 .09682 m
.97619 .10307 L
s
[(1)] .97619 .08432 0 1 Mshowa
.125 Mabswid
.07143 .09682 m
.07143 .10057 L
s
.11905 .09682 m
.11905 .10057 L
s
.16667 .09682 m
.16667 .10057 L
s
.21429 .09682 m
.21429 .10057 L
s
.30952 .09682 m
.30952 .10057 L
s
.35714 .09682 m
.35714 .10057 L
s
.40476 .09682 m
.40476 .10057 L
s
.45238 .09682 m
.45238 .10057 L
s
.54762 .09682 m
.54762 .10057 L
s
.59524 .09682 m
.59524 .10057 L
s
.64286 .09682 m
.64286 .10057 L
s
.69048 .09682 m
.69048 .10057 L
s
.78571 .09682 m
.78571 .10057 L
s
.83333 .09682 m
.83333 .10057 L
s
.88095 .09682 m
.88095 .10057 L
s
.92857 .09682 m
.92857 .10057 L
s
.25 Mabswid
0 .09682 m
1 .09682 L
s
.5 .20213 m
.50625 .20213 L
s
[(-0.19)] .4875 .20213 1 0 Mshowa
.5 .30744 m
.50625 .30744 L
s
[(-0.18)] .4875 .30744 1 0 Mshowa
.5 .41275 m
.50625 .41275 L
s
[(-0.17)] .4875 .41275 1 0 Mshowa
.5 .51806 m
.50625 .51806 L
s
[(-0.16)] .4875 .51806 1 0 Mshowa
.125 Mabswid
.5 .11788 m
.50375 .11788 L
s
.5 .13894 m
.50375 .13894 L
s
.5 .16001 m
.50375 .16001 L
s
.5 .18107 m
.50375 .18107 L
s
.5 .22319 m
.50375 .22319 L
s
.5 .24425 m
.50375 .24425 L
s
.5 .26532 m
.50375 .26532 L
s
.5 .28638 m
.50375 .28638 L
s
.5 .3285 m
.50375 .3285 L
s
.5 .34956 m
.50375 .34956 L
s
.5 .37063 m
.50375 .37063 L
s
.5 .39169 m
.50375 .39169 L
s
.5 .43381 m
.50375 .43381 L
s
.5 .45487 m
.50375 .45487 L
s
.5 .47594 m
.50375 .47594 L
s
.5 .497 m
.50375 .497 L
s
.5 .07576 m
.50375 .07576 L
s
.5 .0547 m
.50375 .0547 L
s
.5 .03363 m
.50375 .03363 L
s
.5 .01257 m
.50375 .01257 L
s
.5 .53912 m
.50375 .53912 L
s
.5 .56018 m
.50375 .56018 L
s
.5 .58125 m
.50375 .58125 L
s
.5 .60231 m
.50375 .60231 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .39051 m
.04262 .30146 L
.06244 .2239 L
.08426 .15579 L
.10458 .10668 L
.12297 .07276 L
.14264 .04618 L
.15285 .03593 L
.16394 .0273 L
.16902 .02415 L
.17441 .02135 L
.17946 .0192 L
.18408 .01761 L
.18891 .01634 L
.19349 .01548 L
.196 .01515 L
.19833 .01492 L
.19967 .01483 L
.20089 .01477 L
.20156 .01475 L
.20229 .01473 L
.20362 .01472 L
.20476 .01472 L
.20583 .01475 L
.207 .01479 L
.20825 .01486 L
.20957 .01496 L
.21081 .01507 L
.21315 .01533 L
.21735 .01599 L
.22192 .01696 L
.23132 .01974 L
.24169 .02396 L
.2627 .03575 L
.28205 .04986 L
.3029 .06788 L
.34404 .10963 L
.38367 .15467 L
.42177 .20004 L
.46234 .24876 L
.50138 .29498 L
.54288 .34264 L
.58286 .38677 L
.62133 .42736 L
.66225 .46827 L
.70165 .50501 L
.73954 .53716 L
.77988 .56667 L
.79845 .57808 L
.8187 .58855 L
Mistroke
.82969 .59324 L
.83987 .59687 L
.84961 .59966 L
.85501 .60089 L
.85998 .6018 L
.86499 .60252 L
.86752 .6028 L
.87029 .60304 L
.87175 .60313 L
.87307 .6032 L
.87381 .60324 L
.8746 .60326 L
.87536 .60329 L
.87605 .6033 L
.87729 .60332 L
.878 .60332 L
.87866 .60332 L
.87936 .60331 L
.87999 .6033 L
.88069 .60328 L
.88143 .60326 L
.8827 .60321 L
.88385 .60314 L
.88643 .60296 L
.88882 .60272 L
.89107 .60244 L
.89613 .60161 L
.90161 .60038 L
.9111 .5974 L
.92153 .59279 L
.93241 .58638 L
.94258 .57878 L
.96015 .56166 L
.97619 .54112 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00S`3oool2000008l0oooo000f0?ooo`/00000CP3oool00`000000oooo0?oo
o`2>0?ooo`00<`3oool3000000/0oooo0`00001;0?ooo`030000003oool0oooo08h0oooo000`0?oo
o`<000004@3oool2000004T0oooo00<000000?ooo`3oool0SP3oool002l0oooo00<000000?ooo`3o
ool0503oool2000004L0oooo00<000000?ooo`3oool0SP3oool002h0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo08h0oooo000]0?ooo`030000003o
ool0oooo01T0oooo0P0000140?ooo`800000S`3oool002`0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`110?ooo`030000003oool0oooo08h0oooo000[0?ooo`030000003oool0oooo
01h0oooo0P0000110?ooo`030000003oool0oooo08h0oooo000Z0?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`2>0?ooo`00:@3oool00`000000oooo
0?ooo`0S0?ooo`800000?P3oool00`000000oooo0?ooo`2>0?ooo`00:03oool00`000000oooo0?oo
o`0V0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0SP3oool002L0oooo00<00000
0?ooo`3oool0:03oool00`000000oooo0?ooo`0j0?ooo`800000S`3oool002H0oooo00<000000?oo
o`3oool0:P3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo08h0oooo00080?ooo`@0
00006P3oool00`000000oooo0?ooo`0N0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@0
0000>@3oool00`000000oooo0?ooo`0j0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<0
0000>`3oool4000000L0oooo000:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
7P3oool010000000oooo0?ooo`00000;0?ooo`800000>03oool00`000000oooo0?ooo`0i0?ooo`04
0000003oool0oooo000000/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`060?oo
o`002P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01l0oooo00@000000?ooo`3o
ool000002`3oool00`000000oooo0000000g0?ooo`030000003oool0oooo03T0oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo00H0oooo00020?oo
o`@00000103oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo100000020?oo
o`040000003oool0oooo000000P0oooo0`0000030?ooo`030000003oool0oooo03@0oooo00<00000
0?ooo`3oool0>@3oool010000000oooo0?ooo`0000080?ooo`<00000?@3oool00`000000oooo0?oo
o`060?ooo`002P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0200oooo00@00000
0?ooo`3oool00000203oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03<0oooo0P00
000j0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0?@3oool00`000000oooo
0?ooo`060?ooo`00203oool3000001L0oooo00<000000?ooo`3oool08@3oool010000000oooo0?oo
o`0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0<P3oool00`000000oooo
0?ooo`0i0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0>`3oool3000000P0
oooo000:0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool08P3oool2000000T0oooo
100000050?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0>P3oool2000000T0oooo
1000000l0?ooo`030000003oool0oooo00H0oooo000Q0?ooo`030000003oool0oooo03P0oooo00<0
00000?ooo`3oool0<03oool00`000000oooo0?ooo`2>0?ooo`008@3oool00`000000oooo0?ooo`0h
0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0SP3oool00200oooo00<000000?oo
o`3oool0>P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo08h0oooo003o00000200
00000@3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`080?ooo`040000
003oool0oooo000000d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool01P3oool000L0oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
;03oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0
1P3oool001h0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0[0?ooo`030000003o
ool0oooo08h0oooo000N0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`2>0?ooo`007@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool0SP3oool001d0oooo00<000000?ooo`3oool0A03oool00`000000
oooo0?ooo`0X0?ooo`800000S`3oool001`0oooo00<000000?ooo`3oool0AP3oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo08h0oooo000L0?ooo`030000003oool0oooo04L0oooo00<0
00000?ooo`3oool09P3oool00`000000oooo0?ooo`2>0?ooo`006`3oool00`000000oooo0?ooo`19
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0SP3oool001/0oooo00<000000?oo
o`3oool0BP3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo08h0oooo000K0?ooo`03
0000003oool0oooo04X0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`2>0?ooo`00
6P3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo02<0oooo0P00002?0?ooo`006P3o
ool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0SP3o
ool001T0oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo08h0oooo000I0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`2>0?ooo`00603oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool0SP3oool001P0oooo00<000000?ooo`3oool0D`3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo08h0oooo000H0?ooo`030000003oool0oooo05@0oooo00<0
00000?ooo`3oool07@3oool2000008l0oooo000G0?ooo`030000003oool0oooo05H0oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`2>0?ooo`005`3oool00`000000oooo0?ooo`1G0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool0SP3oool001L0oooo00<000000?ooo`3o
ool0E`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo08h0oooo000G0?ooo`030000
003oool0oooo05P0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`2>0?ooo`005P3o
ool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0SP3o
ool001H0oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`0H0?ooo`800000S`3oool0
01H0oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
08h0oooo000E0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0?ooo`2>0?ooo`005@3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool0SP3oool001D0oooo00<000000?ooo`3oool0G@3oool3000000<0oooo00<0
00000?ooo`3oool00P3oool400000080oooo0`0000060?ooo`030000003oool0oooo08h0oooo000D
0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool00P0000090?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`00
0000oooo0?ooo`1M0?ooo`060000003oool0oooo0000003oool00000203oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00<0oooo0`00002>0?ooo`00503oool00`000000oooo0?ooo`1G
0?ooo`@000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<00000
0?ooo`3oool0G@3oool010000000oooo0?ooo`0000030?ooo`030000003oool0oooo00@0oooo00D0
00000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
SP3oool001<0oooo00<000000?ooo`3oool0GP3oool010000000oooo0?ooo`0000030?ooo`030000
003oool0oooo0080oooo0`0000030?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3o
ool0SP3oool001<0oooo00<000000?ooo`3oool0G`3oool2000000D0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`004`3o
ool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0SP3o
ool00180oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`0>0?ooo`800000S`3oool0
0180oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
08h0oooo000B0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`2>0?ooo`004@3oool00`000000oooo0?ooo`1/0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0SP3oool00140oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000003oool0oooo06h0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000000oooo0?ooo`1^0?oo
o`030000003oool0oooo00X0oooo0P00002?0?ooo`00403oool00`000000oooo0?ooo`1`0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0SP3oool00100oooo00<000000?ooo`3oool0
L@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000003o
ool0oooo0780oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2>0?ooo`00403oool0
0`000000oooo0?ooo`1c0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool0
00l0oooo00<000000?ooo`3oool0M@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
08h0oooo000?0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`2>0?ooo`003`3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo00@0oooo
0P00002?0?ooo`003P3oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0SP3oool000h0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000003oool0oooo07X0oooo00D000000?oo
o`3oool0oooo0000002@0?ooo`003P3oool00`000000oooo0?ooo`1k0?ooo`040000003oool0oooo
00000900oooo000=0?ooo`030000003oool0oooo07`0oooo00@000000?ooo`3oool00000T03oool0
00d0oooo00<000000?ooo`3oool0O@3oool010000000oooo00000000002?0?ooo`003@3oool00`00
0000oooo0?ooo`1n0?ooo`800000T03oool000d0oooo00<000000?ooo`3oool0O`3oool00`000000
oooo0?ooo`2>0?ooo`00303oool00`000000oooo0?ooo`200?ooo`800000S`3oool000`0oooo00<0
00000?ooo`3oool0IP3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000<0oooo0P00
00060?ooo`030000003oool0000008h0oooo000<0?ooo`030000003oool0oooo06D0oooo00@00000
0?ooo`3oool000002P3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0
0`3oool010000000oooo0?ooo`00002=0?ooo`00303oool00`000000oooo0?ooo`1U0?ooo`040000
003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo
00<0oooo1000002=0?ooo`002`3oool00`000000oooo0?ooo`1P0?ooo`@000000P3oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo0080oooo0P0000060?ooo`050000003oool0oooo
0?ooo`000000S03oool000/0oooo00<000000?ooo`3oool0IP3oool010000000oooo0?ooo`00000:
0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0R@3oool000/0oooo00<000000?ooo`3oool0IP3oool0
10000000oooo0?ooo`0000080?ooo`<000000`3oool010000000oooo0?ooo`0000050?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0R03oool000/0oooo00<000000?ooo`3oool0I`3o
ool2000000/0oooo00<000000?ooo`3oool00P3oool2000000H0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`270?ooo`002`3oool00`000000oooo0?ooo`210?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0QP3oool000X0oooo00<000000?ooo`3oool0PP3oool20000
00L0oooo00<000000?ooo`3oool0Q@3oool000X0oooo00<000000?ooo`3oool0PP3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo08D0oooo000:0?ooo`030000003oool0oooo0880oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`240?ooo`002P3oool00`000000oooo0?oo
o`220?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0P`3oool000X0oooo00<00000
0?ooo`3oool0PP3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0880oooo00090?oo
o`030000003oool0oooo08<0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`210?oo
o`002@3oool00`000000oooo0?ooo`230?ooo`800000303oool00`000000oooo0?ooo`200?ooo`00
2@3oool00`000000oooo0?ooo`230?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
O`3oool000T0oooo00<000000?ooo`3oool0P`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo07h0oooo00090?ooo`030000003oool0oooo08<0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`1m0?ooo`00203oool00`000000oooo0?ooo`240?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0O03oool000P0oooo00<000000?ooo`3oool0Q03oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo07/0oooo00080?ooo`030000003oool0oooo08@0oooo
0P00000B0?ooo`030000003oool0oooo07X0oooo00080?ooo`030000003oool0oooo08@0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`1i0?ooo`00203oool00`000000oooo0?ooo`24
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0N03oool000L0oooo00<000000?oo
o`3oool0Q@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo07L0oooo00070?ooo`03
0000003oool0oooo08D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`1f0?ooo`00
1`3oool00`000000oooo0?ooo`250?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
M@3oool008l0oooo0P00000H0?ooo`030000003oool0oooo07@0oooo002?0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0L`3oool008l0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`1b0?ooo`00S`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0740
oooo001e0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000000`3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0L@3oool007@0oooo00@0
00000?ooo`3oool000002P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`1`0?ooo`00M03oool010000000oooo0?oo
o`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool3000001`0oooo
00<000000?ooo`3oool0K`3oool006h0oooo100000020?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool0KP3oool007@0oooo00@000000?ooo`3oool000002P3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`1]0?ooo`00M03oool010000000oooo0?ooo`0000080?ooo`<000000`3oool010000000oooo
0?ooo`0000050?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0K03oool007D0oooo
0P00000;0?ooo`040000003oool0oooo0?ooo`@000001@3oool00`000000oooo0?ooo`0P0?ooo`03
0000003oool0oooo06/0oooo002?0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0
JP3oool008l0oooo0P00000S0?ooo`030000003oool0oooo06T0oooo002?0?ooo`030000003oool0
oooo02<0oooo00<000000?ooo`3oool0J03oool008l0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`1W0?ooo`00S`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo06H0
oooo002?0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0IP3oool008l0oooo00<0
00000?ooo`3oool09P3oool00`000000oooo0?ooo`1U0?ooo`00S`3oool2000002P0oooo00<00000
0?ooo`3oool0I03oool008l0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1S0?oo
o`00S`3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0680oooo002?0?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool0H@3oool008l0oooo00<000000?ooo`3oool0:`3o
ool00`000000oooo0?ooo`1P0?ooo`00S`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0
oooo05l0oooo002?0?ooo`800000;P3oool00`000000oooo0?ooo`1N0?ooo`00S`3oool00`000000
oooo0?ooo`0^0?ooo`800000GP3oool008l0oooo00<000000?ooo`3oool0<03oool00`000000oooo
0?ooo`1K0?ooo`00S`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo05X0oooo002?
0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0F@3oool008l0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`1H0?ooo`00S`3oool2000003D0oooo0P00001H0?ooo`00
S`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo05D0oooo002?0?ooo`030000003o
ool0oooo03L0oooo00<000000?ooo`3oool0E03oool008l0oooo00<000000?ooo`3oool0>03oool0
0`000000oooo0?ooo`1C0?ooo`00M@3oool2000000@0oooo00<000000?ooo`3oool00P3oool40000
00<0oooo0P0000060?ooo`030000003oool0oooo03T0oooo0P00001C0?ooo`00M03oool010000000
oooo0?ooo`00000:0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0D03oool007@0oooo00@000000?oo
o`3oool000002P3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00`3o
ool3000003`0oooo00<000000?ooo`3oool0C`3oool006h0oooo100000020?ooo`040000003oool0
oooo000000X0oooo00@000000?ooo`3oool0oooo0`0000060?ooo`030000003oool0oooo03d0oooo
00<000000?ooo`3oool0CP3oool007@0oooo00@000000?ooo`3oool000002P3oool01@000000oooo
0?ooo`3oool0000000P0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`1=0?ooo`00
M03oool010000000oooo0?ooo`0000080?ooo`<000000`3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo03l0oooo0P00001=0?ooo`00M@3oool2000000/0oooo00<000000?ooo`3oool0
0P3oool3000000D0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`1:0?ooo`00S`3o
ool00`000000oooo0?ooo`120?ooo`030000003oool0oooo04T0oooo002?0?ooo`800000A03oool0
0`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00H0oooo002?0?ooo`030000003oool0oooo
04@0oooo0P00000n0?ooo`030000003oool0oooo00L0oooo002?0?ooo`030000003oool0oooo04H0
oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`070?ooo`00S`3oool00`000000oooo
0?ooo`170?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0203oool008l0oooo00<0
00000?ooo`3oool0B03oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo00T0oooo002?
0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`09
0?ooo`00S`3oool2000004/0oooo0P00000e0?ooo`030000003oool0oooo00X0oooo002?0?ooo`03
0000003oool0oooo04`0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0;0?ooo`00
S`3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
303oool008l0oooo00<000000?ooo`3oool0CP3oool2000002h0oooo00<000000?ooo`3oool03@3o
ool008l0oooo00<000000?ooo`3oool0D03oool2000002/0oooo00<000000?ooo`3oool03P3oool0
08l0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo
00l0oooo002?0?ooo`800000E03oool2000002D0oooo0P00000B0?ooo`00S`3oool00`000000oooo
0?ooo`1E0?ooo`8000008P3oool00`000000oooo0?ooo`0B0?ooo`00S`3oool00`000000oooo0?oo
o`1G0?ooo`<000007@3oool2000001D0oooo002?0?ooo`030000003oool0oooo05X0oooo0`00000I
0?ooo`030000003oool0oooo01D0oooo002?0?ooo`030000003oool0oooo05d0oooo0`00000D0?oo
o`800000603oool008l0oooo00<000000?ooo`3oool0H03oool2000000d0oooo1@00000J0?ooo`00
S`3oool2000006<0oooo3@00000O0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.0526, -0.209194, \
0.00733521, 0.000331684}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, RePlot1]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.499998 0.476188 0.892805 4.22572 [
[.02381 .03516 -6 -9 ]
[.02381 .03516 6 0 ]
[.2619 .03516 -12 -9 ]
[.2619 .03516 12 0 ]
[.73809 .03516 -9 -9 ]
[.73809 .03516 9 0 ]
[.97619 .03516 -3 -9 ]
[.97619 .03516 3 0 ]
[.4875 .13218 -30 -4.5 ]
[.4875 .13218 0 4.5 ]
[.4875 .21669 -30 -4.5 ]
[.4875 .21669 0 4.5 ]
[.4875 .3012 -30 -4.5 ]
[.4875 .3012 0 4.5 ]
[.4875 .38572 -30 -4.5 ]
[.4875 .38572 0 4.5 ]
[.4875 .47023 -24 -4.5 ]
[.4875 .47023 0 4.5 ]
[.4875 .55475 -30 -4.5 ]
[.4875 .55475 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .04766 m
.02381 .05391 L
s
[(-1)] .02381 .03516 0 1 Mshowa
.2619 .04766 m
.2619 .05391 L
s
[(-0.5)] .2619 .03516 0 1 Mshowa
.73809 .04766 m
.73809 .05391 L
s
[(0.5)] .73809 .03516 0 1 Mshowa
.97619 .04766 m
.97619 .05391 L
s
[(1)] .97619 .03516 0 1 Mshowa
.125 Mabswid
.07143 .04766 m
.07143 .05141 L
s
.11905 .04766 m
.11905 .05141 L
s
.16667 .04766 m
.16667 .05141 L
s
.21428 .04766 m
.21428 .05141 L
s
.30952 .04766 m
.30952 .05141 L
s
.35714 .04766 m
.35714 .05141 L
s
.40476 .04766 m
.40476 .05141 L
s
.45238 .04766 m
.45238 .05141 L
s
.54762 .04766 m
.54762 .05141 L
s
.59524 .04766 m
.59524 .05141 L
s
.64285 .04766 m
.64285 .05141 L
s
.69047 .04766 m
.69047 .05141 L
s
.78571 .04766 m
.78571 .05141 L
s
.83333 .04766 m
.83333 .05141 L
s
.88095 .04766 m
.88095 .05141 L
s
.92857 .04766 m
.92857 .05141 L
s
.25 Mabswid
0 .04766 m
1 .04766 L
s
.5 .13218 m
.50625 .13218 L
s
[(-0.18)] .4875 .13218 1 0 Mshowa
.5 .21669 m
.50625 .21669 L
s
[(-0.16)] .4875 .21669 1 0 Mshowa
.5 .3012 m
.50625 .3012 L
s
[(-0.14)] .4875 .3012 1 0 Mshowa
.5 .38572 m
.50625 .38572 L
s
[(-0.12)] .4875 .38572 1 0 Mshowa
.5 .47023 m
.50625 .47023 L
s
[(-0.1)] .4875 .47023 1 0 Mshowa
.5 .55475 m
.50625 .55475 L
s
[(-0.08)] .4875 .55475 1 0 Mshowa
.125 Mabswid
.5 .06879 m
.50375 .06879 L
s
.5 .08992 m
.50375 .08992 L
s
.5 .11105 m
.50375 .11105 L
s
.5 .1533 m
.50375 .1533 L
s
.5 .17443 m
.50375 .17443 L
s
.5 .19556 m
.50375 .19556 L
s
.5 .23782 m
.50375 .23782 L
s
.5 .25895 m
.50375 .25895 L
s
.5 .28008 m
.50375 .28008 L
s
.5 .32233 m
.50375 .32233 L
s
.5 .34346 m
.50375 .34346 L
s
.5 .36459 m
.50375 .36459 L
s
.5 .40685 m
.50375 .40685 L
s
.5 .42798 m
.50375 .42798 L
s
.5 .4491 m
.50375 .4491 L
s
.5 .49136 m
.50375 .49136 L
s
.5 .51249 m
.50375 .51249 L
s
.5 .53362 m
.50375 .53362 L
s
.5 .02653 m
.50375 .02653 L
s
.5 .0054 m
.50375 .0054 L
s
.5 .57588 m
.50375 .57588 L
s
.5 .597 m
.50375 .597 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .16551 m
.04262 .12978 L
.06244 .09865 L
.08426 .07132 L
.10458 .05162 L
.12297 .038 L
.14264 .02734 L
.15285 .02323 L
.16394 .01976 L
.16902 .0185 L
.17441 .01738 L
.17946 .01651 L
.18408 .01588 L
.18891 .01537 L
.19349 .01502 L
.196 .01489 L
.19833 .0148 L
.19967 .01476 L
.20089 .01474 L
.20156 .01473 L
.20229 .01472 L
.20362 .01472 L
.20476 .01472 L
.20582 .01473 L
.207 .01475 L
.20825 .01477 L
.20957 .01481 L
.21081 .01486 L
.21315 .01496 L
.21735 .01523 L
.22192 .01562 L
.23131 .01673 L
.24169 .01843 L
.2627 .02316 L
.28204 .02882 L
.3029 .03605 L
.34404 .0528 L
.38366 .07087 L
.42177 .08908 L
.46233 .10863 L
.50138 .12717 L
.54288 .1463 L
.58286 .16401 L
.62132 .18029 L
.66224 .19671 L
.70164 .21145 L
.73953 .22435 L
.77987 .23619 L
.79845 .24077 L
.8187 .24498 L
Mistroke
.82968 .24686 L
.83987 .24831 L
.84961 .24943 L
.85501 .24992 L
.85997 .25029 L
.86499 .25058 L
.86752 .25069 L
.87028 .25079 L
.87174 .25083 L
.87307 .25085 L
.8738 .25087 L
.8746 .25088 L
.87535 .25089 L
.87605 .25089 L
.87729 .2509 L
.878 .2509 L
.87866 .2509 L
.87935 .2509 L
.87999 .25089 L
.88069 .25089 L
.88142 .25088 L
.88269 .25086 L
.88384 .25083 L
.88642 .25076 L
.88881 .25066 L
.89107 .25055 L
.89613 .25021 L
.90161 .24972 L
.9111 .24852 L
.92152 .24668 L
.93241 .2441 L
.94258 .24106 L
.96015 .23418 L
.97619 .22594 L
Mfstroke
0 0 1 r
.082 .61803 m
.11905 .22684 L
.16667 .07756 L
.21429 .03273 L
.26191 .02772 L
.30953 .03959 L
.35715 .05879 L
.40476 .08091 L
.45238 .10385 L
.5 .12653 L
.54762 .14844 L
.59524 .16934 L
.64286 .18914 L
.69048 .20783 L
.7381 .22544 L
.78572 .24203 L
.83333 .25765 L
.88095 .27237 L
.92857 .28627 L
.97619 .29939 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool2000008l0oooo002?0?ooo`030000003oool0oooo08h0
oooo00080?ooo`@00000>`3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000003X0oooo
00<000000?ooo`3oool0>P3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000003/0oooo
100000070?ooo`002P3oool00`000000oooo0?ooo`0V0?oooa4000000P3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0>@3oool010000000oooo
0?ooo`00000;0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool01P3oool000X0oooo
00<000000?ooo`3oool08@3oool500000140oooo100000000`3oool000000?ooo`0:0?ooo`030000
003oool0oooo03L0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00000;0?ooo`03
0000003oool0oooo03X0oooo00<000000?ooo`3oool01P3oool00080oooo100000040?ooo`030000
003oool0oooo01h0oooo0`00000B0?ooo`@000000P3oool00`000000oooo00000004000000D0oooo
0`00000j0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000203oool3000003d0
oooo00<000000?ooo`3oool01P3oool000X0oooo00<000000?ooo`3oool06`3oool3000001/0oooo
00<000000?ooo`000?l010000?l300000080oooo00<000000?ooo`3oool0>P3oool2000003X0oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00H0
oooo00080?ooo`<000006`3oool2000001T0oooo1`000?l00`3oool000000?ooo`020?ooo`<0003o
0`00000l0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`0k0?ooo`<00000203oool000X0oooo00<000000?ooo`3oool05`3oool2000001L0oooo
10000?l60?ooo`8000001`3oool40000o`800000>@3oool00`000000oooo0?ooo`0j0?ooo`800000
2@3oool4000003`0oooo00<000000?ooo`3oool01P3oool002<0oooo00<000000?ooo`3oool05P3o
ool00`000?l0oooo0?ooo`0E0?ooo`<0003o0P00000f0?ooo`030000003oool0oooo08h0oooo000R
0?ooo`030000003oool0oooo01H0oooo00<0003o0?ooo`3oool06@3oool30000o`030000003oool0
oooo0380oooo00<000000?ooo`3oool0SP3oool00200oooo0P00000H0?ooo`030000o`3oool0oooo
01d0oooo0`000?lb0?ooo`030000003oool0oooo08h0oooo000i000000030000o`00000000000240
00000P000?o000000040oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
1`3oool01@000000oooo0?ooo`3oool0000000d0oooo00<000000?ooo`3oool01@3oool00`000?l0
oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`040?ooo`<0003o103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00H0
oooo00070?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool00`000?l0oooo
0?ooo`0A0?ooo`030000003oool0oooo01<0oooo00@000000000o`000?l0003o:P3oool00`000000
oooo0?ooo`110?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool01P3oool001`0oooo
00<000000?ooo`3oool05`3oool00`000?l0oooo0?ooo`0[0?ooo`03000000000?l0003o02P0oooo
00<000000?ooo`3oool0SP3oool001/0oooo00<000000?ooo`3oool05`3oool00`000?l0oooo0?oo
o`0^0?ooo`03000000000?l0003o02H0oooo00<000000?ooo`3oool0SP3oool001X0oooo00<00000
0?ooo`3oool05`3oool00`000?l0oooo0?ooo`0a0?ooo`03000000000?l0003o02@0oooo00<00000
0?ooo`3oool0SP3oool001T0oooo00<000000?ooo`3oool05`3oool00`000?l0oooo0?ooo`0e0?oo
o`80003o8P3oool2000008l0oooo000H0?ooo`030000003oool0oooo01L0oooo00<0003o0?ooo`3o
ool0>03oool20000o`030000003oool0oooo01d0oooo00<000000?ooo`3oool0SP3oool001L0oooo
00<000000?ooo`3oool05`3oool00`000?l0oooo0?ooo`0k0?ooo`80003o00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`2>0?ooo`005`3oool00`000000oooo0?ooo`0F0?ooo`030000o`3o
ool0oooo03h0oooo0P000?l00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo08h0oooo000F
0?ooo`030000003oool0oooo01L0oooo00<0003o0?ooo`3oool0@03oool30000oaT0oooo00<00000
0?ooo`3oool0SP3oool001D0oooo00<000000?ooo`3oool05`3oool00`000?l0oooo0?ooo`140?oo
o`80003o5`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000000oooo0?ooo`0H0?ooo`03
0000o`3oool0oooo04H0oooo0P000?lE0?ooo`800000S`3oool001@0oooo00<000000?ooo`3oool0
603oool00`000?l0oooo0?ooo`180?ooo`80003o4`3oool00`000000oooo0?ooo`2>0?ooo`004`3o
ool00`000000oooo0?ooo`0H0?ooo`030000o`3oool0oooo04/0oooo0P000?lA0?ooo`030000003o
ool0oooo08h0oooo000B0?ooo`030000003oool0oooo01T0oooo00<0003o0?ooo`3oool0C@3oool2
0000o`l0oooo00<000000?ooo`3oool0SP3oool00140oooo00<000000?ooo`3oool06P3oool00`00
0?l0oooo0?ooo`1?0?ooo`80003o3@3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000000
oooo0?ooo`0I0?ooo`030000o`3oool0oooo0580oooo0P000?l;0?ooo`030000003oool0oooo08h0
oooo000@0?ooo`030000003oool0oooo01X0oooo00<0003o0?ooo`3oool0E03oool20000o`T0oooo
0P00002?0?ooo`003`3oool00`000000oooo0?ooo`0K0?ooo`030000o`3oool0oooo05H0oooo0P00
0?l70?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000003oool0oooo01X0oooo00<0003o
0?ooo`3oool0F@3oool20000o`D0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<000000?oo
o`3oool06`3oool00`000?l0oooo0?ooo`1K0?ooo`80003o0`3oool00`000000oooo0?ooo`2>0?oo
o`003@3oool00`000000oooo0?ooo`0L0?ooo`030000o`3oool0oooo04H0oooo0P0000040?ooo`03
0000003oool0oooo0080oooo100000030?ooo`8000000`3oool20000o`030?ooo`000000oooo08l0
oooo000=0?ooo`030000003oool0oooo01/0oooo00<0003o0?ooo`3oool0AP3oool010000000oooo
0?ooo`00000:0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`020?oo
o`<0003oS`3oool000`0oooo00<000000?ooo`3oool0703oool00`000?l0oooo0?ooo`160?ooo`04
0000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0
oooo00<0oooo0P0000020000ohd0oooo000<0?ooo`030000003oool0oooo01`0oooo00<0003o0?oo
o`3oool0@03oool400000080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`02
0?ooo`8000001P3oool010000000oooo0?ooo`3oool20000oh/0oooo000;0?ooo`030000003oool0
oooo01`0oooo00<0003o0?ooo`3oool0A`3oool010000000oooo0?ooo`00000:0?ooo`050000003o
ool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo
0`000?n80?ooo`002`3oool00`000000oooo0?ooo`0L0?ooo`030000o`3oool0oooo04L0oooo00@0
00000?ooo`3oool00000203oool3000000<0oooo00@000000?ooo`3oool000001@3oool00`000000
oooo0?ooo`060?ooo`80003oQP3oool000X0oooo00<000000?ooo`3oool07@3oool00`000?l0oooo
0?ooo`180?ooo`8000002`3oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`000000oooo
0?ooo`080?ooo`80003oQ03oool000X0oooo00<000000?ooo`3oool0703oool00`000?l0oooo0?oo
o`1S0?ooo`030000003oool0oooo00X0oooo0`000?n10?ooo`002@3oool00`000000oooo0?ooo`0M
0?ooo`030000o`3oool0oooo06<0oooo0P00000=0?ooo`03000000000?l0003o07l0oooo00090?oo
o`030000003oool0oooo01d0oooo00<0003o0?ooo`3oool0H`3oool00`000000oooo0?ooo`0>0?oo
o`03000000000?l0003o07d0oooo00080?ooo`030000003oool0oooo01h0oooo00<0003o0?ooo`3o
ool0H`3oool00`000000oooo0?ooo`0@0?ooo`03000000000?l0003o07/0oooo00080?ooo`030000
003oool0oooo01d0oooo00<0003o0?ooo`3oool0I03oool00`000000oooo0?ooo`0B0?ooo`030000
00000?l0003o07T0oooo00070?ooo`030000003oool0oooo01h0oooo00<0003o0?ooo`3oool0I03o
ool00`000000oooo0?ooo`0D0?ooo`03000000000?l0003o07L0oooo000X0?ooo`030000o`3oool0
oooo06@0oooo00<000000?ooo`3oool05`3oool30000og@0oooo000W0?ooo`030000o`3oool0oooo
06D0oooo0P00000K0?ooo`80003oLP3oool002L0oooo00<0003o0?ooo`3oool0I@3oool00`000000
oooo0?ooo`0L0?ooo`80003o00<000000?ooo`3oool0K@3oool002L0oooo00<0003o0?ooo`3oool0
I@3oool00`000000oooo0?ooo`0N0?ooo`<0003o00<000000?ooo`3oool0JP3oool002H0oooo00<0
003o0?ooo`3oool0IP3oool00`000000oooo0?ooo`0Q0?ooo`80003o00<000000?ooo`3oool0J03o
ool002H0oooo00<0003o0?ooo`3oool0IP3oool00`000000oooo0?ooo`0S0?ooo`80003o00<00000
0?ooo`3oool0IP3oool002H0oooo00<0003o0?ooo`3oool0IP3oool00`000000oooo0?ooo`0U0?oo
o`<0003oIP3oool002D0oooo00<0003o0?ooo`3oool0I`3oool00`000000oooo0?ooo`0X0?ooo`<0
003oH`3oool002D0oooo00<0003o0?ooo`3oool0I`3oool2000002/0oooo00<000000000o`000?l0
H@3oool002D0oooo00<0003o0?ooo`3oool0I`3oool00`000000oooo0?ooo`0]0?ooo`<0003oGP3o
ool002@0oooo00<0003o0?ooo`3oool0J03oool00`000000oooo0?ooo`0`0?ooo`<0003oF`3oool0
02@0oooo00<0003o0?ooo`3oool0J03oool00`000000oooo0?ooo`0c0?ooo`<0003oF03oool002@0
oooo00<0003o0?ooo`3oool0CP3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000<0
oooo0P0000060?ooo`030000003oool0oooo03H0oooo0P000?l00`000000oooo0?ooo`1C0?ooo`00
8`3oool00`000?l0oooo0?ooo`1>0?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3o
ool0oooo000000020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0>03oool30000
o`030000003oool0oooo0500oooo000S0?ooo`030000o`3oool0oooo04h0oooo00@000000?ooo`3o
ool000002P3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00`3oool3
000003/0oooo0`000?m@0?ooo`008`3oool00`000?l0oooo0?ooo`180?ooo`@000000P3oool01000
0000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000001P3oool00`000000oooo0?oo
o`0n0?ooo`80003o00<000000?ooo`3oool0B`3oool00280oooo00<0003o0?ooo`3oool0C`3oool0
10000000oooo0?ooo`00000:0?ooo`050000003oool0oooo0?ooo`000000203oool00`000000oooo
0?ooo`100?ooo`<0003o00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`060?ooo`008P3o
ool00`000?l0oooo0?ooo`1?0?ooo`040000003oool0oooo000000P0oooo0`0000030?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0@`3oool30000o`800000>`3oool2000000T0oooo
000R0?ooo`030000o`3oool0oooo0500oooo0P00000;0?ooo`030000003oool0oooo0080oooo0`00
00050?ooo`030000003oool0oooo04H0oooo0P000?l4000003H0oooo00<000000?ooo`3oool02@3o
ool00280oooo00<0003o0?ooo`3oool0JP3oool00`000000oooo0?ooo`180?ooo`<0003o00<0oooo
0000000000000P00000`0?ooo`800000303oool00280oooo00<0003o0?ooo`3oool0JP3oool20000
04`0oooo0`000?l20?ooo`@00000:P3oool2000000h0oooo000R0?ooo`030000o`3oool0oooo06X0
oooo00<000000?ooo`3oool0CP3oool30000o`<0oooo0`00000Q0?ooo`H00000403oool00280oooo
00<0003o0?ooo`3oool0JP3oool00`000000oooo0?ooo`1A0?ooo`<0003o0`3oool6000001P0oooo
0`00000F0?ooo`008@3oool00`000?l0oooo0?ooo`1[0?ooo`030000003oool0oooo05@0oooo1000
0?l50?oooaP000006@3oool00240oooo00<0003o0?ooo`3oool0J`3oool00`000000oooo0?ooo`1H
0?ooo`<0003o<`3oool00240oooo00<0003o0?ooo`3oool0J`3oool00`000000oooo0?ooo`1K0?oo
o`<0003o<03oool00240oooo00<0003o0?ooo`3oool0J`3oool2000005l0oooo0`000?l]0?ooo`00
8@3oool00`000?l0oooo0?ooo`1[0?ooo`030000003oool0oooo0640oooo0P000?l[0?ooo`008@3o
ool00`000?l0oooo0?ooo`1[0?ooo`030000003oool0oooo06<0oooo0`000?lX0?ooo`008@3oool0
0`000?l0oooo0?ooo`1[0?ooo`030000003oool0oooo06H0oooo0`000?lU0?ooo`008@3oool00`00
0?l0oooo0?ooo`1[0?ooo`030000003oool0oooo06T0oooo0`000?lR0?ooo`008@3oool00`000?l0
oooo0?ooo`1[0?ooo`030000003oool0oooo06`0oooo10000?lN0?ooo`008@3oool00`000?l0oooo
0?ooo`1[0?ooo`800000L@3oool30000oa/0oooo000P0?ooo`030000o`3oool0oooo06`0oooo00<0
00000?ooo`3oool0L`3oool40000oaL0oooo000P0?ooo`030000o`3oool0oooo06`0oooo00<00000
0?ooo`3oool0M`3oool40000oa<0oooo000P0?ooo`030000o`3oool0oooo06`0oooo00<000000?oo
o`3oool0N`3oool40000o`l0oooo000P0?ooo`030000o`3oool0oooo0580oooo0P0000040?ooo`03
0000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0O`3oool40000o`/0oooo000P0?ooo`030000o`3oool0oooo0540oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0P`3oool30000o`P0oooo000P0?ooo`030000o`3oool0oooo0540oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`05000000D0oooo0`00002>0?ooo`00803oool00`000?l0oooo
0?ooo`1;0?ooo`@000000P3oool010000000oooo0?ooo`00000:0?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`00803oool00`000?l0
oooo0?ooo`1A0?ooo`040000003oool0oooo000000X0oooo00L000000?ooo`3oool0oooo0000003o
ool0000000H0oooo00<000000?ooo`3oool0SP3oool00200oooo00<0003o0?ooo`3oool0D@3oool0
10000000oooo0?ooo`0000080?ooo`<00000103oool2000000H0oooo00<000000?ooo`3oool0SP3o
ool00200oooo00<0003o0?ooo`3oool0DP3oool2000000/0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000O0?ooo`030000o`3oool0oooo
06d0oooo00<000000?ooo`3oool0SP3oool001l0oooo00<0003o0?ooo`3oool0K@3oool2000008l0
oooo000O0?ooo`030000o`3oool0oooo06d0oooo00<000000?ooo`3oool0SP3oool001l0oooo00<0
003o0?ooo`3oool0K@3oool00`000000oooo0?ooo`2>0?ooo`007`3oool00`000?l0oooo0?ooo`1]
0?ooo`030000003oool0oooo08h0oooo000O0?ooo`030000o`3oool0oooo06d0oooo00<000000?oo
o`3oool0SP3oool001l0oooo00<0003o0?ooo`3oool0K@3oool00`000000oooo0?ooo`2>0?ooo`00
7`3oool00`000?l0oooo0?ooo`1]0?ooo`800000S`3oool001l0oooo00<0003o0?ooo`3oool0K@3o
ool00`000000oooo0?ooo`2>0?ooo`007`3oool00`000?l0oooo0?ooo`1]0?ooo`030000003oool0
oooo08h0oooo000N0?ooo`030000o`3oool0oooo06h0oooo00<000000?ooo`3oool0SP3oool001h0
oooo00<0003o0?ooo`3oool0KP3oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000?l0oooo
0?ooo`1^0?ooo`030000003oool0oooo08h0oooo000N0?ooo`030000o`3oool0oooo06h0oooo0P00
002?0?ooo`007P3oool00`000?l0oooo0?ooo`1^0?ooo`030000003oool0oooo08h0oooo000N0?oo
o`030000o`3oool0oooo06h0oooo00<000000?ooo`3oool0SP3oool001h0oooo00<0003o0?ooo`3o
ool0KP3oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000?l0oooo0?ooo`1D0?ooo`800000
103oool00`000000oooo0?ooo`020?ooo`@000000P3oool4000000D0oooo00<000000?ooo`3oool0
SP3oool001h0oooo00<0003o0?ooo`3oool0D`3oool010000000oooo0?ooo`00000:0?ooo`050000
003oool0oooo0?ooo`000000203oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000?l0oooo
0?ooo`1C0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`050?ooo`<00000SP3oool001d0oooo00<0003o0?ooo`3oool0CP3oool400000080oooo
00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0SP3oool001d0oooo00<0003o0?ooo`3oool0E03oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`2>0?ooo`007@3oool00`000?l0oooo0?ooo`1D0?ooo`040000003oool0oooo000000P0
oooo0`0000030?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0SP3oool001d0
oooo00<0003o0?ooo`3oool0E@3oool2000000/0oooo00<000000?ooo`3oool00P3oool2000000H0
oooo00<000000?ooo`3oool0SP3oool001d0oooo00<0003o0?ooo`3oool0K`3oool00`000000oooo
0?ooo`2>0?ooo`007@3oool00`000?l0oooo0?ooo`1_0?ooo`800000S`3oool001d0oooo00<0003o
0?ooo`3oool0K`3oool00`000000oooo0?ooo`2>0?ooo`007@3oool00`000?l0oooo0?ooo`1_0?oo
o`030000003oool0oooo08h0oooo000M0?ooo`030000o`3oool0oooo06l0oooo00<000000?ooo`3o
ool0SP3oool001d0oooo00<0003o0?ooo`3oool0K`3oool00`000000oooo0?ooo`2>0?ooo`00703o
ool00`000?l0oooo0?ooo`1`0?ooo`030000003oool0oooo08h0oooo000L0?ooo`030000o`3oool0
oooo0700oooo0P00002?0?ooo`00703oool00`000?l0oooo0?ooo`1`0?ooo`030000003oool0oooo
08h0oooo000L0?ooo`030000o`3oool0oooo0700oooo00<000000?ooo`3oool0SP3oool001`0oooo
00<0003o0?ooo`3oool0L03oool00`000000oooo0?ooo`2>0?ooo`00703oool00`000?l0oooo0?oo
o`1`0?ooo`030000003oool0oooo08h0oooo000L0?ooo`030000o`3oool0oooo0700oooo00<00000
0?ooo`3oool0SP3oool001`0oooo00<0003o0?ooo`3oool0L03oool2000008l0oooo000L0?ooo`03
0000o`3oool0oooo0700oooo00<000000?ooo`3oool0SP3oool001`0oooo00<0003o0?ooo`3oool0
L03oool00`000000oooo0?ooo`2>0?ooo`006`3oool00`000?l0oooo0?ooo`1a0?ooo`030000003o
ool0oooo08h0oooo000K0?ooo`030000o`3oool0oooo05d0oooo0P0000040?ooo`030000003oool0
oooo0080oooo100000050?ooo`030000003oool0oooo08h0oooo000K0?ooo`030000o`3oool0oooo
05`0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo08h0oooo000K0?ooo`030000o`3oool0oooo05`0oooo00@000000?ooo`3oool000002P3oool0
0`000000oooo0?ooo`040?ooo`<00000SP3oool001/0oooo00<0003o0?ooo`3oool0EP3oool40000
0080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo08h0oooo000K0?ooo`030000o`3oool0oooo05`0oooo00@000000?ooo`3oool000002P3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000K0?ooo`030000o`3oool0oooo
05`0oooo00@000000?ooo`3oool00000203oool3000000H0oooo00<000000?ooo`3oool0SP3oool0
01/0oooo00<0003o0?ooo`3oool0G@3oool2000000/0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`2>0?ooo`006`3oool00`000?l0oooo0?ooo`1a0?ooo`030000003oool0oooo08h0
oooo000K0?ooo`030000o`3oool0oooo0740oooo0P00002?0?ooo`006P3oool00`000?l0oooo0?oo
o`1b0?ooo`030000003oool0oooo08h0oooo000J0?ooo`030000o`3oool0oooo0780oooo00<00000
0?ooo`3oool0SP3oool001X0oooo00<0003o0?ooo`3oool0LP3oool00`000000oooo0?ooo`2>0?oo
o`006P3oool00`000?l0oooo0?ooo`1b0?ooo`030000003oool0oooo08h0oooo000J0?ooo`030000
o`3oool0oooo0780oooo00<000000?ooo`3oool0SP3oool001X0oooo00<0003o0?ooo`3oool0LP3o
ool2000008l0oooo000J0?ooo`030000o`3oool0oooo0780oooo00<000000?ooo`3oool0SP3oool0
01X0oooo00<0003o0?ooo`3oool0LP3oool00`000000oooo0?ooo`2>0?ooo`006P3oool00`000?l0
oooo0?ooo`1b0?ooo`030000003oool0oooo08h0oooo000J0?ooo`030000o`3oool0oooo0780oooo
00<000000?ooo`3oool0SP3oool001T0oooo00<0003o0?ooo`3oool0L`3oool00`000000oooo0?oo
o`2>0?ooo`006@3oool00`000?l0oooo0?ooo`1c0?ooo`800000S`3oool001T0oooo00<0003o0?oo
o`3oool0L`3oool00`000000oooo0?ooo`2>0?ooo`006@3oool00`000?l0oooo0?ooo`1c0?ooo`03
0000003oool0oooo08h0oooo000I0?ooo`030000o`3oool0oooo07<0oooo00<000000?ooo`3oool0
SP3oool001T0oooo00<0003o0?ooo`3oool0F@3oool2000000@0oooo00<000000?ooo`3oool00`3o
ool2000000@0oooo0P0000060?ooo`030000003oool0oooo08h0oooo000I0?ooo`030000o`3oool0
oooo05P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000
003oool0oooo000000D0oooo00<000000?ooo`3oool0SP3oool001T0oooo00<0003o0?ooo`3oool0
F03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool000001@3oool3000008h0oooo000I0?ooo`030000o`3oool0oooo0580oooo100000020?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000`3oool2000000H0oooo00<0
00000?ooo`3oool0SP3oool001T0oooo00<0003o0?ooo`3oool0F03oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`000000
oooo0?ooo`2>0?ooo`00603oool00`000?l0oooo0?ooo`1I0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000003oool0
oooo08h0oooo000H0?ooo`030000o`3oool0oooo05X0oooo0P00000:0?ooo`800000103oool20000
00H0oooo00<000000?ooo`3oool0SP3oool001P0oooo00<0003o0?ooo`3oool0M03oool00`000000
oooo0?ooo`2>0?ooo`00603oool00`000?l0oooo0?ooo`1d0?ooo`800000S`3oool001P0oooo00<0
003o0?ooo`3oool0M03oool00`000000oooo0?ooo`2>0?ooo`00603oool00`000?l0oooo0?ooo`1d
0?ooo`030000003oool0oooo08h0oooo000H0?ooo`030000o`3oool0oooo07@0oooo00<000000?oo
o`3oool0SP3oool001P0oooo00<0003o0?ooo`3oool0M03oool00`000000oooo0?ooo`2>0?ooo`00
603oool00`000?l0oooo0?ooo`1d0?ooo`030000003oool0oooo08h0oooo000H0?ooo`030000o`3o
ool0oooo07@0oooo0P00002?0?ooo`005`3oool00`000?l0oooo0?ooo`1e0?ooo`030000003oool0
oooo08h0oooo000G0?ooo`030000o`3oool0oooo07D0oooo00<000000?ooo`3oool0SP3oool001L0
oooo00<0003o0?ooo`3oool0M@3oool00`000000oooo0?ooo`2>0?ooo`005`3oool00`000?l0oooo
0?ooo`1e0?ooo`030000003oool0oooo08h0oooo000G0?ooo`030000o`3oool0oooo07D0oooo00<0
00000?ooo`3oool0SP3oool001L0oooo00<0003o0?ooo`3oool0M@3oool00`000000oooo0?ooo`2>
0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05261, -0.21128, \
0.00733526, 0.000826596}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[beta[1, 1, 9/10, 1/2, a]], {a, \(-1\), 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 1.15629 3.98223 [
[.02381 .34735 -6 -9 ]
[.02381 .34735 6 0 ]
[.2619 .34735 -12 -9 ]
[.2619 .34735 12 0 ]
[.7381 .34735 -9 -9 ]
[.7381 .34735 9 0 ]
[.97619 .34735 -3 -9 ]
[.97619 .34735 3 0 ]
[.4875 .04127 -30 -4.5 ]
[.4875 .04127 0 4.5 ]
[.4875 .12091 -30 -4.5 ]
[.4875 .12091 0 4.5 ]
[.4875 .20056 -30 -4.5 ]
[.4875 .20056 0 4.5 ]
[.4875 .2802 -30 -4.5 ]
[.4875 .2802 0 4.5 ]
[.4875 .43949 -30 -4.5 ]
[.4875 .43949 0 4.5 ]
[.4875 .51914 -30 -4.5 ]
[.4875 .51914 0 4.5 ]
[.4875 .59878 -30 -4.5 ]
[.4875 .59878 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .35985 m
.02381 .3661 L
s
[(-1)] .02381 .34735 0 1 Mshowa
.2619 .35985 m
.2619 .3661 L
s
[(-0.5)] .2619 .34735 0 1 Mshowa
.7381 .35985 m
.7381 .3661 L
s
[(0.5)] .7381 .34735 0 1 Mshowa
.97619 .35985 m
.97619 .3661 L
s
[(1)] .97619 .34735 0 1 Mshowa
.125 Mabswid
.07143 .35985 m
.07143 .3636 L
s
.11905 .35985 m
.11905 .3636 L
s
.16667 .35985 m
.16667 .3636 L
s
.21429 .35985 m
.21429 .3636 L
s
.30952 .35985 m
.30952 .3636 L
s
.35714 .35985 m
.35714 .3636 L
s
.40476 .35985 m
.40476 .3636 L
s
.45238 .35985 m
.45238 .3636 L
s
.54762 .35985 m
.54762 .3636 L
s
.59524 .35985 m
.59524 .3636 L
s
.64286 .35985 m
.64286 .3636 L
s
.69048 .35985 m
.69048 .3636 L
s
.78571 .35985 m
.78571 .3636 L
s
.83333 .35985 m
.83333 .3636 L
s
.88095 .35985 m
.88095 .3636 L
s
.92857 .35985 m
.92857 .3636 L
s
.25 Mabswid
0 .35985 m
1 .35985 L
s
.5 .04127 m
.50625 .04127 L
s
[(-0.28)] .4875 .04127 1 0 Mshowa
.5 .12091 m
.50625 .12091 L
s
[(-0.26)] .4875 .12091 1 0 Mshowa
.5 .20056 m
.50625 .20056 L
s
[(-0.24)] .4875 .20056 1 0 Mshowa
.5 .2802 m
.50625 .2802 L
s
[(-0.22)] .4875 .2802 1 0 Mshowa
.5 .43949 m
.50625 .43949 L
s
[(-0.18)] .4875 .43949 1 0 Mshowa
.5 .51914 m
.50625 .51914 L
s
[(-0.16)] .4875 .51914 1 0 Mshowa
.5 .59878 m
.50625 .59878 L
s
[(-0.14)] .4875 .59878 1 0 Mshowa
.125 Mabswid
.5 .06118 m
.50375 .06118 L
s
.5 .08109 m
.50375 .08109 L
s
.5 .101 m
.50375 .101 L
s
.5 .14082 m
.50375 .14082 L
s
.5 .16074 m
.50375 .16074 L
s
.5 .18065 m
.50375 .18065 L
s
.5 .22047 m
.50375 .22047 L
s
.5 .24038 m
.50375 .24038 L
s
.5 .26029 m
.50375 .26029 L
s
.5 .30011 m
.50375 .30011 L
s
.5 .32003 m
.50375 .32003 L
s
.5 .33994 m
.50375 .33994 L
s
.5 .37976 m
.50375 .37976 L
s
.5 .39967 m
.50375 .39967 L
s
.5 .41958 m
.50375 .41958 L
s
.5 .4594 m
.50375 .4594 L
s
.5 .47931 m
.50375 .47931 L
s
.5 .49923 m
.50375 .49923 L
s
.5 .53905 m
.50375 .53905 L
s
.5 .55896 m
.50375 .55896 L
s
.5 .57887 m
.50375 .57887 L
s
.5 .02136 m
.50375 .02136 L
s
.5 .00145 m
.50375 .00145 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .05432 L
.10458 .09826 L
.14415 .13934 L
.18221 .17804 L
.22272 .21786 L
.26171 .25448 L
.30316 .29129 L
.34309 .32455 L
.3815 .35444 L
.42237 .38401 L
.46172 .41038 L
.49955 .4339 L
.53984 .45711 L
.57861 .47781 L
.61984 .4982 L
.65954 .51638 L
.69774 .5326 L
.73838 .54851 L
.77751 .56246 L
.81909 .57565 L
.85916 .5865 L
.89771 .59484 L
.91765 .59819 L
.92782 .59962 L
.93871 .60091 L
.94885 .60189 L
.9581 .60258 L
.96255 .60284 L
.96739 .60306 L
.96961 .60315 L
.97196 .60322 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool2
000008l0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0
oooo002?0?ooo`030000003oool0oooo08h0oooo00070?ooo`030000003oool0oooo08D0oooo00<0
00000?ooo`3oool0SP3oool000P0oooo00<000000?ooo`3oool0Q03oool00`000000oooo0?ooo`2>
0?ooo`002@3oool00`000000oooo0?ooo`230?ooo`800000S`3oool000X0oooo00<000000?ooo`3o
ool0PP3oool00`000000oooo0?ooo`2>0?ooo`002`3oool00`000000oooo0?ooo`210?ooo`030000
003oool0oooo08h0oooo000<0?ooo`030000003oool0oooo06H0oooo0P0000040?ooo`030000003o
ool0oooo0080oooo100000030?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`003@3oool0
0`000000oooo0?ooo`1T0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool00`3o
ool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000003o
ool0oooo06<0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo000000D0oooo0`00002>0?ooo`003`3oool00`000000oooo0?ooo`1L0?ooo`@00000
0P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0080oooo0P0000060?ooo`03
0000003oool0oooo08h0oooo000@0?ooo`030000003oool0oooo0640oooo00@000000?ooo`3oool0
00002`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0SP3oool00140oooo00<000000?ooo`3oool0H03oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?oo
o`2>0?ooo`004P3oool00`000000oooo0?ooo`1P0?ooo`8000002P3oool2000000@0oooo0P000006
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000003oool0oooo07T0oooo00<000000?oo
o`3oool0SP3oool001@0oooo00<000000?ooo`3oool0N03oool2000008l0oooo000E0?ooo`030000
003oool0oooo07L0oooo00<000000?ooo`3oool0SP3oool001H0oooo00<000000?ooo`3oool0MP3o
ool00`000000oooo0?ooo`2>0?ooo`005`3oool00`000000oooo0?ooo`1e0?ooo`030000003oool0
oooo08h0oooo000H0?ooo`030000003oool0oooo07@0oooo00<000000?ooo`3oool0SP3oool001P0
oooo00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`2>0?ooo`006@3oool00`000000oooo
0?ooo`1c0?ooo`800000S`3oool001X0oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?oo
o`2>0?ooo`006`3oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo08h0oooo000L0?oo
o`030000003oool0oooo0700oooo00<000000?ooo`3oool0SP3oool001d0oooo00<000000?ooo`3o
ool0K`3oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000000oooo0?ooo`1^0?ooo`800000
S`3oool001l0oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`2>0?ooo`00803oool0
0`000000oooo0?ooo`1/0?ooo`030000003oool0oooo08h0oooo000Q0?ooo`030000003oool0oooo
06/0oooo00<000000?ooo`3oool0SP3oool00280oooo00<000000?ooo`3oool0D03oool2000000@0
oooo00<000000?ooo`3oool00P3oool4000000<0oooo0P0000060?ooo`030000003oool0oooo08h0
oooo000S0?ooo`030000003oool0oooo04h0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`030?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0SP3oool002@0
oooo00<000000?ooo`3oool0C@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo
0080oooo00@000000?ooo`3oool000001@3oool3000008h0oooo000U0?ooo`030000003oool0oooo
04H0oooo100000020?ooo`040000003oool0oooo000000X0oooo00@000000?ooo`3oool0oooo0`00
00060?ooo`030000003oool0oooo08h0oooo000V0?ooo`030000003oool0oooo04/0oooo00@00000
0?ooo`3oool000002`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo08h0oooo
000W0?ooo`030000003oool0oooo04X0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool002P0oooo
00<000000?ooo`3oool0BP3oool2000000X0oooo0P0000040?ooo`<000001@3oool00`000000oooo
0?ooo`2>0?ooo`00:@3oool00`000000oooo0?ooo`1S0?ooo`030000003oool0oooo08h0oooo000Z
0?ooo`030000003oool0oooo0680oooo0P00002?0?ooo`00:`3oool00`000000oooo0?ooo`1Q0?oo
o`030000003oool0oooo08h0oooo000/0?ooo`030000003oool0oooo0600oooo00<000000?ooo`3o
ool0SP3oool002d0oooo00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`2>0?ooo`00;P3o
ool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo08h0oooo000^0?ooo`030000003oool0
oooo05h0oooo00<000000?ooo`3oool0SP3oool002l0oooo00<000000?ooo`3oool0G@3oool20000
08l0oooo000`0?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool0SP3oool00340oooo
00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`2>0?ooo`00<P3oool00`000000oooo0?oo
o`1J0?ooo`030000003oool0oooo08h0oooo000c0?ooo`030000003oool0oooo05T0oooo00<00000
0?ooo`3oool0SP3oool003@0oooo00<000000?ooo`3oool0F03oool2000008l0oooo000e0?ooo`03
0000003oool0oooo05L0oooo00<000000?ooo`3oool0SP3oool003H0oooo00<000000?ooo`3oool0
EP3oool00`000000oooo0?ooo`2>0?ooo`00=`3oool00`000000oooo0?ooo`1E0?ooo`030000003o
ool0oooo08h0oooo000h0?ooo`030000003oool0oooo03X0oooo0P0000040?ooo`030000003oool0
oooo0080oooo100000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool0
03T0oooo00<000000?ooo`3oool0>03oool010000000oooo0?ooo`0000080?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`00>P3oool20000
03P0oooo00@000000?ooo`3oool000002@3oool010000000oooo0?ooo`3oool5000000D0oooo0`00
002>0?ooo`00?03oool00`000000oooo0?ooo`0_0?ooo`@000000P3oool010000000oooo0?ooo`00
000:0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`2>0?ooo`00?@3oool00`000000oooo0?ooo`0d0?ooo`040000003oool0oooo000000/0oooo
00H000000?ooo`3oool000000?ooo`0000060?ooo`030000003oool0oooo08h0oooo000n0?ooo`03
0000003oool0oooo03<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000003
0?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`00?`3oool00`000000oooo0?ooo`0c0?oo
o`8000002P3oool2000000D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?oo
o`00@03oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo08h0oooo00110?ooo`030000
003oool0oooo04/0oooo0P00002?0?ooo`00@P3oool00`000000oooo0?ooo`1:0?ooo`030000003o
ool0oooo08h0oooo00130?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0SP3oool0
04@0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`2>0?ooo`00A@3oool2000004P0
oooo00<000000?ooo`3oool0SP3oool004L0oooo00<000000?ooo`3oool0A@3oool2000008l0oooo
00180?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0SP3oool004T0oooo00<00000
0?ooo`3oool0@`3oool00`000000oooo0?ooo`2>0?ooo`00BP3oool00`000000oooo0?ooo`120?oo
o`030000003oool0oooo08h0oooo001;0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3o
ool0SP3oool004`0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`2>0?ooo`00C@3o
ool00`000000oooo0?ooo`0o0?ooo`800000S`3oool004h0oooo00<000000?ooo`3oool0?P3oool0
0`000000oooo0?ooo`2>0?ooo`00C`3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo
08h0oooo001@0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0SP3oool00540oooo
0P00000R0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000000P3oool4000000D0oooo
00<000000?ooo`3oool0SP3oool005<0oooo00<000000?ooo`3oool07P3oool010000000oooo0?oo
o`0000080?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`2>0?ooo`00E03oool00`000000oooo0?ooo`0M0?ooo`040000003oool0oooo000000T0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`<00000SP3oool005D0oooo00<0
00000?ooo`3oool05P3oool400000080oooo00@000000?ooo`3oool000002P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool005H0oooo00<0
00000?ooo`3oool06`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00E`3oool00`000000oooo0?oo
o`0J0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000
oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo001H0?ooo`030000003oool0oooo01X0
oooo0P00000:0?ooo`800000103oool2000000H0oooo00<000000?ooo`3oool0SP3oool005T0oooo
0P00000d0?ooo`800000S`3oool005/0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?oo
o`2>0?ooo`00G03oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo08h0oooo001M0?oo
o`030000003oool0oooo02l0oooo00<000000?ooo`3oool0SP3oool005h0oooo00<000000?ooo`3o
ool0;P3oool00`000000oooo0?ooo`2>0?ooo`00G`3oool2000002h0oooo00<000000?ooo`3oool0
SP3oool00640oooo00<000000?ooo`3oool0:`3oool2000008l0oooo00080?ooo`@00000>`3oool2
000000@0oooo00<000000?ooo`3oool00P3oool3000000d0oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`0j0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<00000>`3oool4
000000L0oooo000:0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool000002`3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0>@3oool0
10000000oooo0?ooo`00000;0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool01P3o
ool000X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00000;0?ooo`030000003o
ool0oooo00`0oooo0P00000Y0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000
2`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo00H0oooo00020?ooo`@00000103o
ool00`000000oooo0?ooo`0c0?ooo`@000000P3oool010000000oooo0?ooo`0000080?ooo`<00000
4@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool0
0000203oool3000003d0oooo00<000000?ooo`3oool01P3oool000X0oooo00<000000?ooo`3oool0
>@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool09@3oool00`000000oooo0?ooo`0i0?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`3oool0?@3oool00`000000oooo0?ooo`060?ooo`00203oool3000003/0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo02@0oooo0P00000j0?oo
o`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0>`3oool3000000P0oooo000:0?oo
o`030000003oool0oooo03X0oooo0P0000090?ooo`@000004`3oool00`000000oooo0?ooo`0S0?oo
o`030000003oool0oooo03X0oooo0P0000090?ooo`@00000?03oool00`000000oooo0?ooo`060?oo
o`00JP3oool2000002<0oooo00<000000?ooo`3oool0SP3oool006`0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`2>0?ooo`00K@3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo08h0oooo001^0?ooo`8000007`3oool00`000000oooo0?ooo`2>0?ooo`00o`00000P0000
0040oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0203oool200000003
0?ooo`000000oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00H0oooo001c0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
SP3oool007@0oooo0P00000I0?ooo`030000003oool0oooo08h0oooo001f0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0SP3oool007L0oooo0P00000F0?ooo`800000S`3oool007T0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`2>0?ooo`00NP3oool2000001<0oooo
00<000000?ooo`3oool0SP3oool007`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`2>0?ooo`00O@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo08h0oooo001n0?oo
o`8000003`3oool00`000000oooo0?ooo`2>0?ooo`00P03oool00`000000oooo0?ooo`0<0?ooo`80
0000S`3oool00840oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`2>0?ooo`00PP3o
ool2000000/0oooo00<000000?ooo`3oool0SP3oool008@0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`2>0?ooo`00Q@3oool2000000P0oooo00<000000?ooo`3oool0SP3oool008L0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2>0?ooo`00R03oool2000000D0oooo
0P00002?0?ooo`00RP3oool2000000<0oooo00<000000?ooo`3oool0SP3oool008`0oooo00@00000
0?ooo`3oool00000T03oool007D0oooo0P0000040?ooo`030000003oool0oooo0080oooo10000003
0?ooo`800000103oool300000900oooo001d0?ooo`040000003oool0oooo000000X0oooo00D00000
0?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0SP3o
ool007@0oooo00@000000?ooo`3oool000002P3oool01@000000oooo0?ooo`3oool000000080oooo
00<000000?ooo`3oool00`3oool3000008h0oooo001^0?ooo`@000000P3oool010000000oooo0?oo
o`00000:0?ooo`030000003oool0oooo0080oooo0P0000060?ooo`030000003oool0oooo00800000
S03oool007@0oooo00@000000?ooo`3oool000002P3oool01@000000oooo0?ooo`3oool000000080
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`800000RP3oool007@0oooo
00@000000?ooo`3oool00000203oool3000000<0oooo00@000000?ooo`3oool000001@3oool00`00
0000oooo0?ooo`040?ooo`800000R03oool007D0oooo0P00000;0?ooo`030000003oool0oooo0080
oooo0P0000060?ooo`030000003oool0oooo00H0oooo0P0000260?ooo`00S`3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo08<0oooo002?0?ooo`8000002P3oool2000008<0oooo002?
0?ooo`030000003oool0oooo00/0oooo0P0000210?ooo`00S`3oool00`000000oooo0?ooo`0=0?oo
o`800000O`3oool008l0oooo00<000000?ooo`3oool03`3oool2000007d0oooo002?0?ooo`030000
003oool0oooo0140oooo0P00001k0?ooo`00S`3oool00`000000oooo0?ooo`0C0?ooo`800000N@3o
ool008l0oooo0P00000F0?ooo`800000M`3oool008l0oooo00<000000?ooo`3oool05`3oool20000
07D0oooo002?0?ooo`030000003oool0oooo01T0oooo0P00001c0?ooo`00S`3oool00`000000oooo
0?ooo`0K0?ooo`800000L@3oool008l0oooo00<000000?ooo`3oool07@3oool2000006l0oooo002?
0?ooo`800000803oool2000006d0oooo002?0?ooo`030000003oool0oooo0240oooo0P00001[0?oo
o`00S`3oool00`000000oooo0?ooo`0S0?ooo`800000J@3oool008l0oooo00<000000?ooo`3oool0
9@3oool3000006H0oooo001e0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000000`3o
ool2000000H0oooo00<000000?ooo`3oool0:03oool2000006@0oooo001d0?ooo`040000003oool0
oooo000000X0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0:P3oool300000640oooo001d0?ooo`040000003oool0oooo000000X0oooo
00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo0`00000]0?ooo`80
0000G`3oool006h0oooo100000020?ooo`040000003oool0oooo000000X0oooo00@000000?ooo`3o
ool0oooo0`0000060?ooo`030000003oool0oooo02l0oooo0P00001M0?ooo`00M03oool010000000
oooo0?ooo`00000:0?ooo`050000003oool0oooo0?ooo`000000203oool00`000000oooo0?ooo`0a
0?ooo`800000F`3oool007@0oooo00@000000?ooo`3oool00000203oool3000000<0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0c0?ooo`800000F@3oool007D0oooo0P00000;0?oo
o`030000003oool0oooo0080oooo0`0000050?ooo`030000003oool0oooo03D0oooo0`00001F0?oo
o`00S`3oool00`000000oooo0?ooo`0h0?ooo`800000E03oool008l0oooo0P00000k0?ooo`800000
DP3oool008l0oooo00<000000?ooo`3oool0?03oool200000500oooo002?0?ooo`030000003oool0
oooo03h0oooo0P00001>0?ooo`00S`3oool00`000000oooo0?ooo`100?ooo`<00000B`3oool008l0
oooo00<000000?ooo`3oool0@`3oool3000004P0oooo002?0?ooo`030000003oool0oooo04H0oooo
0P0000160?ooo`00S`3oool2000004T0oooo0`0000130?ooo`00S`3oool00`000000oooo0?ooo`1;
0?ooo`<00000@03oool008l0oooo00<000000?ooo`3oool0CP3oool4000003`0oooo002?0?ooo`03
0000003oool0oooo0580oooo1000000h0?ooo`00S`3oool00`000000oooo0?ooo`1F0?ooo`@00000
=03oool008l0oooo0P00001K0?ooo`@00000<03oool008l0oooo00<000000?ooo`3oool0GP3oool4
000002`0oooo002?0?ooo`030000003oool0oooo0680oooo1000000X0?ooo`00S`3oool00`000000
oooo0?ooo`1V0?ooo`@00000903oool007D0oooo0P0000040?ooo`030000003oool0oooo0080oooo
100000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0JP3oool400000200oooo
001d0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo06h0oooo1000000L0?ooo`00M03oool010000000oooo0?oo
o`00000:0?ooo`030000003oool0oooo00D000001@3oool300000780oooo1`00000E0?ooo`00KP3o
ool400000080oooo00@000000?ooo`3oool000002P3oool010000000oooo0?ooo`0000020?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0N@3oool=000000P0oooo001d0?ooo`040000
003oool0oooo000000X0oooo00L000000?ooo`3oool0oooo0000003oool0000000H0oooo00<00000
0?ooo`3oool0SP3oool007@0oooo00@000000?ooo`3oool00000203oool3000000@0oooo0P000006
0?ooo`030000003oool0oooo08h0oooo001e0?ooo`8000002`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool008l0oooo00<000000?ooo`3o
ool0SP3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.0526, -0.290363, \
0.00733521, 0.000877137}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, RePlot3]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.499998 0.476188 0.603319 2.05326 [
[.02381 .59082 -6 -9 ]
[.02381 .59082 6 0 ]
[.2619 .59082 -12 -9 ]
[.2619 .59082 12 0 ]
[.73809 .59082 -9 -9 ]
[.73809 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.4875 .09 -30 -4.5 ]
[.4875 .09 0 4.5 ]
[.4875 .19267 -24 -4.5 ]
[.4875 .19267 0 4.5 ]
[.4875 .29533 -30 -4.5 ]
[.4875 .29533 0 4.5 ]
[.4875 .39799 -24 -4.5 ]
[.4875 .39799 0 4.5 ]
[.4875 .50066 -30 -4.5 ]
[.4875 .50066 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60332 m
.02381 .60957 L
s
[(-1)] .02381 .59082 0 1 Mshowa
.2619 .60332 m
.2619 .60957 L
s
[(-0.5)] .2619 .59082 0 1 Mshowa
.73809 .60332 m
.73809 .60957 L
s
[(0.5)] .73809 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(1)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.21428 .60332 m
.21428 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.64285 .60332 m
.64285 .60707 L
s
.69047 .60332 m
.69047 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.5 .09 m
.50625 .09 L
s
[(-0.25)] .4875 .09 1 0 Mshowa
.5 .19267 m
.50625 .19267 L
s
[(-0.2)] .4875 .19267 1 0 Mshowa
.5 .29533 m
.50625 .29533 L
s
[(-0.15)] .4875 .29533 1 0 Mshowa
.5 .39799 m
.50625 .39799 L
s
[(-0.1)] .4875 .39799 1 0 Mshowa
.5 .50066 m
.50625 .50066 L
s
[(-0.05)] .4875 .50066 1 0 Mshowa
.125 Mabswid
.5 .11054 m
.50375 .11054 L
s
.5 .13107 m
.50375 .13107 L
s
.5 .1516 m
.50375 .1516 L
s
.5 .17213 m
.50375 .17213 L
s
.5 .2132 m
.50375 .2132 L
s
.5 .23373 m
.50375 .23373 L
s
.5 .25426 m
.50375 .25426 L
s
.5 .2748 m
.50375 .2748 L
s
.5 .31586 m
.50375 .31586 L
s
.5 .3364 m
.50375 .3364 L
s
.5 .35693 m
.50375 .35693 L
s
.5 .37746 m
.50375 .37746 L
s
.5 .41853 m
.50375 .41853 L
s
.5 .43906 m
.50375 .43906 L
s
.5 .45959 m
.50375 .45959 L
s
.5 .48012 m
.50375 .48012 L
s
.5 .52119 m
.50375 .52119 L
s
.5 .54172 m
.50375 .54172 L
s
.5 .56225 m
.50375 .56225 L
s
.5 .58279 m
.50375 .58279 L
s
.5 .06947 m
.50375 .06947 L
s
.5 .04894 m
.50375 .04894 L
s
.5 .02841 m
.50375 .02841 L
s
.5 .00787 m
.50375 .00787 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .03514 L
.10458 .05779 L
.14415 .07897 L
.18221 .09893 L
.22272 .11946 L
.26171 .13834 L
.30315 .15732 L
.34308 .17447 L
.3815 .18988 L
.42236 .20512 L
.46171 .21872 L
.49955 .23085 L
.53984 .24282 L
.57861 .25349 L
.61983 .264 L
.65954 .27338 L
.69773 .28174 L
.73838 .28994 L
.77751 .29714 L
.81909 .30394 L
.85916 .30953 L
.89771 .31383 L
.91765 .31556 L
.92781 .31629 L
.93871 .31696 L
.94885 .31747 L
.95809 .31782 L
.96254 .31795 L
.96738 .31807 L
.9696 .31811 L
.97196 .31815 L
.97619 .3182 L
s
0 0 1 r
.04411 .61803 m
.07143 .12931 L
.11905 .09401 L
.16667 .10015 L
.21429 .11813 L
.26191 .13921 L
.30953 .1603 L
.35715 .18026 L
.40476 .19871 L
.45238 .21559 L
.5 .23099 L
.54762 .24502 L
.59524 .25783 L
.64286 .26954 L
.69048 .28027 L
.7381 .29013 L
.78572 .29922 L
.83333 .30762 L
.88095 .3154 L
.92857 .32264 L
.97619 .32937 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool2000008l0
oooo002?0?ooo`030000003oool0oooo08h0oooo00070?ooo`030000003oool0oooo08D0oooo00<0
00000?ooo`3oool0SP3oool000P0oooo0P0000250?ooo`030000003oool0oooo08h0oooo000:0?oo
o`800000P`3oool00`000000oooo0?ooo`2>0?ooo`00303oool200000840oooo00<000000?ooo`3o
ool0SP3oool000h0oooo0P00001o0?ooo`800000S`3oool00100oooo0P00001m0?ooo`030000003o
ool0oooo08h0oooo000B0?ooo`030000003oool0oooo07X0oooo00<000000?ooo`3oool0SP3oool0
01<0oooo0P00001j0?ooo`030000003oool0oooo08h0oooo000E0?ooo`800000N03oool00`000000
oooo0?ooo`2>0?ooo`005`3oool2000007H0oooo00<000000?ooo`3oool0SP3oool001T0oooo0P00
001d0?ooo`800000S`3oool001/0oooo0P00001b0?ooo`030000003oool0oooo08h0oooo000M0?oo
o`800000L03oool00`000000oooo0?ooo`2>0?ooo`007`3oool2000006h0oooo00<000000?ooo`3o
ool0SP3oool00240oooo0P00001/0?ooo`030000003oool0oooo08h0oooo000S0?ooo`800000JP3o
ool2000008l0oooo000U0?ooo`800000J03oool00`000000oooo0?ooo`2>0?ooo`009`3oool20000
06H0oooo00<000000?ooo`3oool0SP3oool002T0oooo00<000000?ooo`3oool0H`3oool00`000000
oooo0?ooo`2>0?ooo`00:P3oool2000004T0oooo0P0000040?ooo`030000003oool0oooo0080oooo
100000020?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`00;03oool2000004H0oooo00@0
00000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0SP3oool002h0oooo0P0000140?ooo`040000003oool0oooo000000T0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`030?ooo`<00000SP3oool00280oooo1000
0?l:0?ooo`800000?03oool400000080oooo00@000000?ooo`3oool000002P3oool010000000oooo
0?ooo`3oool3000000H0oooo00<000000?ooo`3oool0SP3oool00240oooo00<0003o0?ooo`3oool0
0P3oool70000o`D0oooo0P0000100?ooo`040000003oool0oooo000000/0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0?ooo`2>0?ooo`007`3oool20000o``0oooo1@000?l20?ooo`03
0000003oool0oooo03d0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool001h0oooo00<0003o0?oo
o`3oool04@3oool20000o`030?ooo`000000000003h0oooo0P00000:0?ooo`8000000`3oool40000
00D0oooo00<000000?ooo`3oool0SP3oool001d0oooo00<0003o0?ooo`3oool0503oool30000o`80
0000EP3oool00`000000oooo0?ooo`2>0?ooo`006`3oool20000oaX0oooo0`000?l00`000000oooo
0?ooo`1B0?ooo`800000S`3oool001X0oooo00<0003o0?ooo`3oool07@3oool20000o`030000003o
ool0oooo0500oooo00<000000?ooo`3oool0SP3oool001T0oooo00<0003o0?ooo`3oool0803oool3
0000oe00oooo00<000000?ooo`3oool0SP3oool001P0oooo00<0003o0?ooo`3oool0903oool20000
o`030000003oool0oooo04/0oooo00<000000?ooo`3oool0SP3oool001H0oooo0P000?lY0?ooo`80
003o00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo
0?ooo`0[0?ooo`<0003oB@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?oo
o`0_0?ooo`80003oA`3oool2000008l0oooo000D0?ooo`030000o`3oool0oooo0340oooo0P000?m5
0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo03<0oooo0`000?m20?oo
o`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo03D0oooo00<000000000o`00
0?l0@03oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0g0?ooo`030000
00000?l0003o03h0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0>@3o
ool00`000000003o0000o`0l0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0
oooo03/0oooo00<000000000o`000?l0>P3oool2000008l0oooo000D0?ooo`030000o`3oool0oooo
03d0oooo00<000000000o`000?l0>03oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0
oooo0?ooo`0o0?ooo`04000000000?l0003o0000ocD0oooo00<000000?ooo`3oool0SP3oool001<0
oooo00<0003o0?ooo`3oool0@`3oool00`000000003o0000o`0c0?ooo`030000003oool0oooo08h0
oooo000C0?ooo`030000o`3oool0oooo04D0oooo00<000000000o`000?l0<@3oool00`000000oooo
0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`170?ooo`04000000000?l0003o0000obh0oooo
00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0B@3oool200000080003o;03o
ool2000008l0oooo000C0?ooo`030000o`3oool0oooo04/0oooo0P0000020000obX0oooo00<00000
0?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0CP3oool010000000003o0000o`000?lW
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo0540oooo00@000000000
o`000?l0003o903oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1C0?oo
o`8000000P000?l>0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000001@3oool00`00
0000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1F0?ooo`04000000000?l0003o0000
o`X0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo08h0oooo000C0?ooo`030000o`3oool0oooo05T0oooo00@000000000o`000?l0003o1`3oool0
10000000oooo0?ooo`0000090?ooo`030000003oool0oooo00D0oooo0`00002>0?ooo`004`3oool0
0`000?l0oooo0?ooo`1L0?ooo`04000000000?l0003o0000o`8000000P3oool010000000oooo0?oo
o`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0
003o0?ooo`3oool0G`3oool00`000000003o0000o`020?ooo`040000003oool0oooo000000/0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?oo
o`1R0?ooo`<0003o0P3oool00`000000oooo0?ooo`060?ooo`040000003oool0oooo000000D0oooo
00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0I@3oool30000o`T0oooo0P00
00060?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo06P0oooo0P000?l?
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo06X0oooo0`000?l<0?oo
o`800000S`3oool001<0oooo00<0003o0?ooo`3oool0K@3oool30000o`T0oooo00<000000?ooo`3o
ool0SP3oool001<0oooo00<0003o0?ooo`3oool0L03oool20000o`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0SP3oool00180oooo00<0003o0?ooo`3oool0L`3oool30000o`@0oooo00<0
00000?ooo`3oool0SP3oool00180oooo00<0003o0?ooo`3oool0MP3oool30000o`030?ooo`000000
oooo08l0oooo000B0?ooo`030000o`3oool0oooo07T0oooo0`000?n?0?ooo`004P3oool00`000?l0
oooo0?ooo`1j0?ooo`030000003oool0003o00<0003oR`3oool00180oooo00<0003o0?ooo`3oool0
NP3oool00`000000oooo0?ooo`030?ooo`<0003o00<000000?ooo`3oool0Q@3oool00180oooo00<0
003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`060?ooo`@0003oQ03oool00180oooo00<0003o
0?ooo`3oool0NP3oool00`000000oooo0?ooo`0:0?ooo`@0003oP03oool00180oooo00<0003o0?oo
o`3oool0NP3oool00`000000oooo0?ooo`0>0?ooo`@0003oO03oool00180oooo00<0003o0?ooo`3o
ool0NP3oool2000001<0oooo10000?mh0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`030000
003oool0oooo01H0oooo10000?md0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`030000003o
ool0oooo01X0oooo10000?m`0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`030000003oool0
oooo01h0oooo0`000?m]0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`030000003oool0oooo
0240oooo10000?mY0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`030000003oool0oooo02D0
oooo10000?mU0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`800000:P3oool40000o`030000
003oool0oooo05h0oooo000B0?ooo`030000o`3oool0oooo07X0oooo00<000000?ooo`3oool0;@3o
ool50000o`800000FP3oool00180oooo00<0003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`0b
0?ooo`D0003o00<000000?ooo`3oool0E03oool00140oooo00<0003o0?ooo`3oool0N`3oool00`00
0000oooo0?ooo`0g0?ooo`@0003o00<000000?ooo`3oool0D03oool00140oooo00<0003o0?ooo`3o
ool0H@3oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`0000060?ooo`03
0000003oool0oooo03/0oooo10000?l00`000000oooo0?ooo`1<0?ooo`004@3oool00`000?l0oooo
0?ooo`1P0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo03l0oooo1P000?m90?ooo`004@3oool00`000?l0oooo
0?ooo`1P0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`<00000A03oool00`000000003o0000o`050000od80oooo000A0?ooo`030000
o`3oool0oooo05X0oooo100000020?ooo`040000003oool0oooo000000X0oooo00@000000?ooo`3o
ool0oooo0`0000060?ooo`030000003oool0oooo04X0oooo0P0000060000oc`0oooo000A0?ooo`03
0000o`3oool0oooo0600oooo00@000000?ooo`3oool000002P3oool01@000000oooo0?ooo`3oool0
000000P0oooo00<000000?ooo`3oool0C`3oool3000000@0003o>03oool00140oooo00<0003o0?oo
o`3oool0H03oool010000000oooo0?ooo`0000080?ooo`<000000`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo05D0oooo00<000000000o`000?l00P000?l400000300oooo000A0?oo
o`030000o`3oool0oooo0640oooo0P00000;0?ooo`040000003oool0oooo0?ooo`@000001@3oool0
0`000000oooo0?ooo`1J0?ooo`H0003o2P00000T0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?oo
o`030000003oool0oooo0600oooo1`000?l30?ooo`P00000703oool00140oooo00<0003o0?ooo`3o
ool0N`3oool2000006P0oooo1`000?l40?ooo`d000003`3oool00140oooo00<0003o0?ooo`3oool0
N`3oool00`000000oooo0?ooo`1^0?ooo`L0003o2P3oool7000000P0oooo000A0?ooo`030000o`3o
ool0oooo07/0oooo00<000000?ooo`3oool0M@3oool70000oa80oooo000A0?ooo`030000o`3oool0
oooo07/0oooo00<000000?ooo`3oool0O03oool60000o``0oooo000A0?ooo`030000o`3oool0oooo
07/0oooo00<000000?ooo`3oool0PP3oool40000o`P0oooo000A0?ooo`030000o`3oool0oooo07/0
oooo00<000000?ooo`3oool0SP3oool00140oooo00<0003o0?ooo`3oool0N`3oool2000008l0oooo
000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o
0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?oo
o`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo07`0oooo00<000000?ooo`3o
ool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00403o
ool00`000?l0oooo0?ooo`1l0?ooo`800000S`3oool00100oooo00<0003o0?ooo`3oool0O03oool0
0`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo
08h0oooo000@0?ooo`030000o`3oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo
00<0003o0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?oo
o`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo07`0oooo0P00002?
0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`03
0000o`3oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0
O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1X0?ooo`800000103o
ool00`000000oooo0?ooo`020?ooo`@000001@3oool00`000000oooo0?ooo`2>0?ooo`00403oool0
0`000?l0oooo0?ooo`1W0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1W0?ooo`040000003oool0
oooo000000X0oooo00<000000?ooo`3oool0103oool3000008h0oooo000@0?ooo`030000o`3oool0
oooo0640oooo100000020?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1X0?ooo`040000003oool0
oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`003`3oool0
0`000?l0oooo0?ooo`1X0?ooo`040000003oool0oooo000000P0oooo0`0000060?ooo`030000003o
ool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo06T0oooo0P00000;0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0O@3oool20000
08l0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0oooo
00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?oo
o`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<00000
0?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?oo
o`003`3oool00`000?l0oooo0?ooo`1m0?ooo`800000S`3oool000l0oooo00<0003o0?ooo`3oool0
O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003o
ool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool0
00l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0
oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo
0P00002?0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000>
0?ooo`030000o`3oool0oooo07h0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o0?oo
o`3oool0OP3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`03
0000003oool0oooo08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<000000?ooo`3oool0
SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool2000008l0oooo000>0?ooo`030000o`3oool0
oooo07h0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`00
0000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0
oooo000>0?ooo`030000o`3oool0oooo06@0oooo0P0000040?ooo`030000003oool0oooo00<0oooo
0P0000030?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?oo
o`1S0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000o`3oool0oooo06<0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo
00<0oooo0`00002>0?ooo`003P3oool00`000?l0oooo0?ooo`1M0?ooo`@000000P3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`0000060?ooo`030000003oool0
oooo08h0oooo000>0?ooo`030000o`3oool0oooo06<0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3o
ool000h0oooo00<0003o0?ooo`3oool0H`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2>0?ooo`003P3o
ool00`000?l0oooo0?ooo`1T0?ooo`8000002P3oool2000000<0oooo100000050?ooo`030000003o
ool0oooo08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<000000?ooo`3oool0SP3oool0
00h0oooo00<0003o0?ooo`3oool0OP3oool2000008l0oooo000=0?ooo`030000o`3oool0oooo07l0
oooo00<000000?ooo`3oool0SP3oool000d0oooo00<0003o0?ooo`3oool0O`3oool00`000000oooo
0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1o0?ooo`030000003oool0oooo08h0oooo000=
0?ooo`030000o`3oool0oooo07l0oooo00<000000?ooo`3oool0SP3oool000d0oooo00<0003o0?oo
o`3oool0O`3oool00`000000oooo0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1o0?ooo`80
0000S`3oool000d0oooo00<0003o0?ooo`3oool0O`3oool00`000000oooo0?ooo`2>0?ooo`003@3o
ool00`000?l0oooo0?ooo`1o0?ooo`030000003oool0oooo08h0oooo000=0?ooo`030000o`3oool0
oooo07l0oooo00<000000?ooo`3oool0SP3oool000d0oooo00<0003o0?ooo`3oool0O`3oool00`00
0000oooo0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1o0?ooo`030000003oool0oooo08h0
oooo000=0?ooo`030000o`3oool0oooo07l0oooo0P00002?0?ooo`00203oool4000000030?ooo`00
0?l0oooo03P0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00000j0?ooo`030000003o
ool0oooo03X0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00000k0?ooo`@000001`3o
ool000X0oooo00@000000?ooo`3oool0003o>03oool010000000oooo0?ooo`00000;0?ooo`030000
003oool0oooo03L0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00000;0?ooo`03
0000003oool0oooo03X0oooo00<000000?ooo`3oool01P3oool000X0oooo00@000000?ooo`3oool0
003o>03oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo03L0oooo00<000000?oo
o`3oool0>@3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo03X0oooo00<00000
0?ooo`3oool01P3oool00080oooo100000040?ooo`040000003oool0oooo0000oc80oooo10000002
0?ooo`040000003oool0oooo000000P0oooo0`00000j0?ooo`030000003oool0oooo03T0oooo00@0
00000?ooo`3oool00000203oool3000003d0oooo00<000000?ooo`3oool01P3oool000X0oooo00@0
00000?ooo`3oool0003o>03oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo03X0
oooo0P00000j0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0?@3oool00`00
0000oooo0?ooo`060?ooo`00203oool300000080oooo00<0003o0?ooo`3oool0=P3oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0>@3oool01000
0000oooo0?ooo`0000080?ooo`030000003oool0oooo03/0oooo0`0000080?ooo`002P3oool00`00
0000oooo0000o`0j0?ooo`8000002@3oool4000003T0oooo00<000000?ooo`3oool0>P3oool20000
00T0oooo1000000l0?ooo`030000003oool0oooo00H0oooo000<0?ooo`030000o`3oool0oooo0800
oooo00<000000?ooo`3oool0SP3oool000`0oooo00<0003o0?ooo`3oool0P03oool00`000000oooo
0?ooo`2>0?ooo`00303oool00`000?l0oooo0?ooo`200?ooo`030000003oool0oooo08h0oooo000<
000000030000o`00000000000?l000004@0000010?ooo`001`3oool00`000000oooo0?ooo`020?oo
o`030000o`3oool0oooo00D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?oo
o`001`3oool00`000000oooo0?ooo`020?ooo`030000o`3oool0oooo03`0oooo00<000000?ooo`3o
ool0@@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0440oooo00<000000?ooo`3o
ool01P3oool000`0oooo00<0003o0?ooo`3oool0P03oool00`000000oooo0?ooo`2>0?ooo`00303o
ool00`000?l0oooo0?ooo`200?ooo`030000003oool0oooo08h0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05261, -0.293836, \
0.00733526, 0.00170118}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exact result - analytical", "Subsection"],

Cell[CellGroupData[{

Cell["Integrals", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(I1[cp_, dp_] = \(\(Pi/2\)/dp\)/Sqrt[Sqrt[cp^2 + dp^2]]*
        Cos[ArcTan[dp/cp]/2 - Pi/2*If[cp > 0, 1, 0]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\[Pi]\ Cos[1\/2\ ArcTan[dp\/cp] - 1\/2\ \[Pi]\ If[cp > 0, 1, \
0]]\)\/\(2\ dp\ \((cp\^2 + dp\^2)\)\^\(1/4\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(I2[cp_, dp_] = \(Pi/2\)/dp*Sqrt[Sqrt[cp^2 + dp^2]]*
        Cos[ArcTan[dp/cp]/2 - Pi/2*If[cp > 0, 1, 0]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\((cp\^2 + dp\^2)\)\^\(1/4\)\ \[Pi]\ Cos[1\/2\ ArcTan[dp\/cp] - 1\/2\ \
\[Pi]\ If[cp > 0, 1, 0]]\)\/\(2\ dp\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(I3[cp_, 
        dp_] = \(\(Pi/4\)/dp^2\)/
          Sqrt[Sqrt[
              cp^2 + dp^2]]*\((Cos[ArcTan[dp/cp]/2 - Pi/2*If[cp > 0, 1, 0]]/
              dp - \(1/2\)/\((cp^2 + dp^2)\)*\((\(-2\)\ cp + 
                  Sqrt[cp^2 + dp^2])\)*
              Sin[ArcTan[dp/cp]/2 - Pi/2*If[cp > 0, 1, 0]])\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\[Pi]\ \((Cos[1\/2\ ArcTan[dp\/cp] - 1\/2\ \[Pi]\ If[cp > 0, 1, \
0]]\/dp - \(\((\(-2\)\ cp + \@\(cp\^2 + dp\^2\))\)\ Sin[1\/2\ ArcTan[dp\/cp] \
- 1\/2\ \[Pi]\ If[cp > 0, 1, 0]]\)\/\(2\ \((cp\^2 + dp\^2)\)\))\)\)\/\(4\ \
dp\^2\ \((cp\^2 + dp\^2)\)\^\(1/4\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(I4[cp_, dp_] = \(Pi/4\)/dp^2*
        Sqrt[Sqrt[
            cp^2 + dp^2]]*\((Cos[ArcTan[dp/cp]/2 - Pi/2*If[cp > 0, 1, 0]]/
              dp + \(1/2\)/Sqrt[cp^2 + dp^2]*
              Sin[ArcTan[dp/cp]/2 - Pi/2*If[cp > 0, 1, 0]])\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\((cp\^2 + dp\^2)\)\^\(1/4\)\ \[Pi]\ \((Cos[1\/2\ ArcTan[dp\/cp] - \
1\/2\ \[Pi]\ If[cp > 0, 1, 0]]\/dp + Sin[1\/2\ ArcTan[dp\/cp] - 1\/2\ \[Pi]\ \
If[cp > 0, 1, 0]]\/\(2\ \@\(cp\^2 + dp\^2\)\))\)\)\/\(4\ dp\^2\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["old way to do it", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(aa = om + a\ kz\ xi2[k, kz, om]\)], "Input"],

Cell[BoxData[
    \(om + 
      a\ kz\ \((\(kz\ om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\
\^2)\)\))\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(b = 2\ a\ s\ kz\)], "Input"],

Cell[BoxData[
    \(2\ a\ kz\ \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(c1 = a\ xi2[k, kz, om]^2 + 1\)], "Input"],

Cell[BoxData[
    \(1 + 
      a\ \((\(kz\ om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\
\)\))\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(c2 = 4\ xi2[k, kz, om]\ a\ s\)], "Input"],

Cell[BoxData[
    \(4\ a\ \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\)\ \((\(kz\ om\)\
\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(d = 4\ a\ s^2\)], "Input"],

Cell[BoxData[
    \(4\ a\ \((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(Imbetaanalytical[md2_, k_, kz_, om_, 
          a_] = \(-md2\)\ om^2/
            k^3\ \((aa/c1^2\ I2[1 + c2/\((2  c1)\), 
                  Sqrt[d]/c1] + \((aa + b - aa\ c2/c1)\)/c1^2\ I1[
                  1 + c2/\((2  c1)\), 
                  Sqrt[d]/c1] - \((aa\ d + c2\ b - aa\ c2^2/c1)\)/c1^3\ I4[
                  1 + c2/\((2  c1)\), 
                  Sqrt[d]/
                    c1] - \((d\ b + c2\ b + aa\ d - aa\ c2^2/c1 - 
                      aa\ c2\ d/c1)\)/c1^3\ I3[1 + c2/\((2\ c1)\), 
                  Sqrt[d]/c1])\);\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Imbetaanalytical[1, 1, 1/10, 1/2, a], {a, \(-1\), 5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.18254 0.15873 0.603319 0.548158 [
[.02381 .59082 -6 -9 ]
[.02381 .59082 6 0 ]
[.34127 .59082 -3 -9 ]
[.34127 .59082 3 0 ]
[.5 .59082 -3 -9 ]
[.5 .59082 3 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.81746 .59082 -3 -9 ]
[.81746 .59082 3 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.17004 .05516 -12 -4.5 ]
[.17004 .05516 0 4.5 ]
[.17004 .16479 -24 -4.5 ]
[.17004 .16479 0 4.5 ]
[.17004 .27442 -24 -4.5 ]
[.17004 .27442 0 4.5 ]
[.17004 .38406 -24 -4.5 ]
[.17004 .38406 0 4.5 ]
[.17004 .49369 -24 -4.5 ]
[.17004 .49369 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60332 m
.02381 .60957 L
s
[(-1)] .02381 .59082 0 1 Mshowa
.34127 .60332 m
.34127 .60957 L
s
[(1)] .34127 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(2)] .5 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(3)] .65873 .59082 0 1 Mshowa
.81746 .60332 m
.81746 .60957 L
s
[(4)] .81746 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(5)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.15079 .60332 m
.15079 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.27778 .60332 m
.27778 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.53175 .60332 m
.53175 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.9127 .60332 m
.9127 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.18254 .05516 m
.18879 .05516 L
s
[(-1)] .17004 .05516 1 0 Mshowa
.18254 .16479 m
.18879 .16479 L
s
[(-0.8)] .17004 .16479 1 0 Mshowa
.18254 .27442 m
.18879 .27442 L
s
[(-0.6)] .17004 .27442 1 0 Mshowa
.18254 .38406 m
.18879 .38406 L
s
[(-0.4)] .17004 .38406 1 0 Mshowa
.18254 .49369 m
.18879 .49369 L
s
[(-0.2)] .17004 .49369 1 0 Mshowa
.125 Mabswid
.18254 .08257 m
.18629 .08257 L
s
.18254 .10998 m
.18629 .10998 L
s
.18254 .13738 m
.18629 .13738 L
s
.18254 .1922 m
.18629 .1922 L
s
.18254 .21961 m
.18629 .21961 L
s
.18254 .24702 m
.18629 .24702 L
s
.18254 .30183 m
.18629 .30183 L
s
.18254 .32924 m
.18629 .32924 L
s
.18254 .35665 m
.18629 .35665 L
s
.18254 .41146 m
.18629 .41146 L
s
.18254 .43887 m
.18629 .43887 L
s
.18254 .46628 m
.18629 .46628 L
s
.18254 .5211 m
.18629 .5211 L
s
.18254 .5485 m
.18629 .5485 L
s
.18254 .57591 m
.18629 .57591 L
s
.18254 .02775 m
.18629 .02775 L
s
.18254 .00035 m
.18629 .00035 L
s
.25 Mabswid
.18254 0 m
.18254 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02846 .09054 L
.03279 .14484 L
.03793 .19556 L
.04262 .23247 L
.0522 .28972 L
.06244 .33353 L
.07283 .36655 L
.0841 .39396 L
.09385 .41277 L
.10458 .42974 L
.12442 .45387 L
.14545 .47265 L
.16486 .48594 L
.18573 .49726 L
.22696 .5137 L
.26667 .52487 L
.30486 .53294 L
.34551 .53962 L
.38463 .54476 L
.42622 .54921 L
.46628 .55278 L
.50483 .55569 L
.54584 .55836 L
.58533 .56059 L
.6233 .56248 L
.66372 .56426 L
.70263 .5658 L
.74399 .56726 L
.78384 .56853 L
.82217 .56965 L
.86295 .57074 L
.90221 .5717 L
.93996 .57255 L
.97619 .57331 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003o=03oool2
00000>X0oooo000d0?ooo`030000003oool0oooo0>T0oooo000d0?ooo`030000003oool0oooo0>T0
oooo000d0?ooo`030000003oool0oooo0>T0oooo00070?ooo`030000003oool0oooo02X0oooo00<0
00000?ooo`3oool0j@3oool000L0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3Y
0?ooo`001`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0>T0oooo00070?ooo`03
0000003oool0oooo02X0oooo00<000000?ooo`3oool0j@3oool000L0oooo00<000000?ooo`3oool0
:P3oool200000>X0oooo00070?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0j@3o
ool000L0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3Y0?ooo`001`3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo0>T0oooo00070?ooo`030000003oool0oooo02X0
oooo00<000000?ooo`3oool0j@3oool000L0oooo00<000000?ooo`3oool08P3oool4000000@0oooo
00<000000?ooo`3oool0j@3oool000L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003oool0oooo02<0oooo00<00000
0?ooo`3oool00`3oool300000>T0oooo00080?ooo`030000003oool0oooo01/0oooo100000040?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0j@3oool000P0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000
003oool0oooo0240oooo0`0000050?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3Y0?ooo`00203oool0
0`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool0j@3oool000P0oooo00<000000?ooo`3oool0:@3oool00`000000
oooo0?ooo`3Y0?ooo`00203oool00`000000oooo0?ooo`0Y0?ooo`800000jP3oool000P0oooo00<0
00000?ooo`3oool0:@3oool00`000000oooo0?ooo`3Y0?ooo`00203oool00`000000oooo0?ooo`0Y
0?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003oool0oooo02T0oooo00<000000?oo
o`3oool0j@3oool000P0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`3Y0?ooo`00
203oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0j@3oool000P0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`3Y0?ooo`00203oool00`000000oooo0?ooo`0Y0?ooo`800000jP3oool000P0
oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`3Y0?ooo`00203oool00`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo0>T0oooo00090?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool0j@3oool000T0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`3Y
0?ooo`002@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0>T0oooo00090?ooo`03
0000003oool0oooo02P0oooo00<000000?ooo`3oool0j@3oool000T0oooo00<000000?ooo`3oool0
:03oool00`000000oooo0?ooo`3Y0?ooo`002@3oool00`000000oooo0?ooo`0X0?ooo`800000jP3o
ool000T0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`3Y0?ooo`002@3oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo0>T0oooo00090?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0j@3oool000T0oooo00<000000?ooo`3oool0:03oool00`000000oooo
0?ooo`3Y0?ooo`002@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0>T0oooo000:
0?ooo`030000003oool0oooo01@0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000005
0?ooo`030000003oool0oooo0>T0oooo000:0?ooo`030000003oool0oooo01<0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0>T0oooo000:
0?ooo`030000003oool0oooo01<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`<00000j@3oool000X0oooo00<000000?ooo`3oool03@3oool400000080oooo00@0
00000?ooo`3oool000002@3oool2000000D0oooo00<000000?ooo`3oool0j@3oool000X0oooo00<0
00000?ooo`3oool04`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool0j@3oool000X0oooo00<000000?ooo`3oool04`3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0j@3oool000X0
oooo00<000000?ooo`3oool0503oool2000000X0oooo0P0000050?ooo`030000003oool0oooo0>T0
oooo000:0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0j@3oool000/0oooo00<0
00000?ooo`3oool09P3oool00`000000oooo0?ooo`3Y0?ooo`002`3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo0>T0oooo000;0?ooo`030000003oool0oooo02H0oooo0P00003Z0?oo
o`002`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0>T0oooo000;0?ooo`030000
003oool0oooo02H0oooo00<000000?ooo`3oool0j@3oool000/0oooo00<000000?ooo`3oool09P3o
ool00`000000oooo0?ooo`3Y0?ooo`002`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo0>T0oooo000;0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0j@3oool000/0
oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`3Y0?ooo`00303oool00`000000oooo
0?ooo`0U0?ooo`800000jP3oool000`0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?oo
o`3Y0?ooo`00303oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0>T0oooo000<0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool0j@3oool000`0oooo00<000000?ooo`3o
ool09@3oool00`000000oooo0?ooo`3Y0?ooo`00303oool00`000000oooo0?ooo`0U0?ooo`030000
003oool0oooo0>T0oooo000<0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0j@3o
ool000d0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`3Y0?ooo`003@3oool00`00
0000oooo0?ooo`0T0?ooo`800000jP3oool000d0oooo00<000000?ooo`3oool0903oool00`000000
oooo0?ooo`3Y0?ooo`003@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0>T0oooo
000=0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0j@3oool000d0oooo00<00000
0?ooo`3oool0903oool00`000000oooo0?ooo`3Y0?ooo`003P3oool00`000000oooo0?ooo`0S0?oo
o`030000003oool0oooo0>T0oooo000>0?ooo`030000003oool0oooo0100oooo0P0000040?ooo`03
0000003oool0oooo00<0oooo0P0000050?ooo`030000003oool0oooo0>T0oooo000>0?ooo`030000
003oool0oooo00l0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo0>T0oooo000>0?ooo`030000003oool0oooo00l0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000040?ooo`<00000j@3oool000h0oooo00<000000?oo
o`3oool02@3oool400000080oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<00000
0?ooo`3oool0j@3oool000h0oooo00<000000?ooo`3oool03`3oool010000000oooo0?ooo`000008
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0j@3oool000l0oooo00<000000?oo
o`3oool03P3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0j@3oool000l0oooo00<000000?ooo`3oool03`3oool2000000X0oooo0`0000040?oo
o`030000003oool0oooo0>T0oooo000?0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3o
ool0j@3oool000l0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`3Y0?ooo`003`3o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0>T0oooo000@0?ooo`030000003oool0
oooo0240oooo0P00003Z0?ooo`00403oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0>T0oooo000@0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0j@3oool00100oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`3Y0?ooo`004@3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo0>T0oooo000A0?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool0j@3oool00140oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3Y0?oo
o`004@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0>T0oooo000B0?ooo`030000
003oool0oooo01l0oooo0P00003Z0?ooo`004P3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo0>T0oooo000B0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0j@3oool0
01<0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`3Y0?ooo`004`3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo0>T0oooo000C0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool0j@3oool001<0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?oo
o`3Y0?ooo`00503oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0>T0oooo000D0?oo
o`030000003oool0oooo01d0oooo0P00003Z0?ooo`00503oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo0>T0oooo000E0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0
j@3oool001D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3Y0?ooo`005@3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0>T0oooo000F0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool0j@3oool001H0oooo00<000000?ooo`3oool0203oool2000000@0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0>T0
oooo000G0?ooo`030000003oool0oooo00H0oooo00@000000?ooo`3oool000002P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0>T0oooo000G0?ooo`030000003oool0oooo00H0oooo
00@000000?ooo`3oool000001`3oool5000000@0oooo0`00003Y0?ooo`00603oool00`000000oooo
0000000300000080oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`0000050?oo
o`030000003oool0oooo0>T0oooo000H0?ooo`030000003oool0oooo00D0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo000000050?ooo`030000003oool0oooo0>T0oooo000H0?ooo`03
0000003oool0oooo00D0oooo00@000000?ooo`3oool000002@3oool2000000D0oooo00<000000?oo
o`3oool0j@3oool001T0oooo00<000000?ooo`3oool01@3oool2000000/0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`3Y0?ooo`006@3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo0>T0oooo000J0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0j@3o
ool001X0oooo00<000000?ooo`3oool05`3oool200000>X0oooo000K0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0j@3oool001`0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`3Y0?ooo`00703oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0>T0oooo
000M0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0j@3oool001d0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`3Y0?ooo`007P3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0>T0oooo000O0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0j@3oool00200oooo00<000000?ooo`3oool04@3oool200000>X0oooo000P0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0j@3oool00240oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`3Y0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
0>T0oooo000S0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0j@3oool002@0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3Y0?ooo`009@3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo0>T0oooo000V0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0j@3oool002L0oooo00<000000?ooo`3oool02P3oool200000>X0oooo000X0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0j@3oool002T0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`3Y0?ooo`00:P3oool2000000P0oooo00<000000?ooo`3oool0j@3o
ool002`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3Y0?ooo`00;@3oool20000
00D0oooo00<000000?ooo`3oool0j@3oool00240oooo0P0000040?ooo`030000003oool0oooo0080
oooo100000040?ooo`030000003oool0oooo0>T0oooo000P0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool0oooo0P0000020?ooo`030000003oool0oooo0>T0oooo000P0?ooo`04
0000003oool0oooo000000T0oooo00<000000?ooo`3oool00P3oool500000>T0oooo000J0?ooo`@0
00000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo0`00003Y0?oo
o`00803oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool00P00003W0?ooo`00803oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool00P3oool200000>D0oooo000Q0?ooo`8000002P3oool20000
00D0oooo00<000000?ooo`3oool0103oool300000>80oooo000d0?ooo`030000003oool0oooo00L0
oooo0P00003P0?ooo`00=03oool00`000000oooo0?ooo`090?ooo`<00000g@3oool003@0oooo00<0
00000?ooo`3oool0303oool400000=T0oooo000d0?ooo`8000004@3oool400000=D0oooo000d0?oo
o`030000003oool0oooo01@0oooo1000003A0?ooo`00=03oool00`000000oooo0?ooo`0H0?ooo`H0
0000b`3oool003@0oooo00<000000?ooo`3oool07P3oool500000<H0oooo000d0?ooo`030000003o
ool0oooo02<0oooo1P0000300?ooo`00=03oool00`000000oooo0?ooo`0Y0?ooo`H00000^P3oool0
03@0oooo00<000000?ooo`3oool0;`3oool600000;@0oooo000d0?ooo`030000003oool0oooo03D0
oooo2000002/0?ooo`00=03oool2000003h0oooo3000002P0?ooo`00=03oool00`000000oooo0?oo
o`190?ooo`/00000U@3oool003@0oooo00<000000?ooo`3oool0E03ooolG000007h0oooo000d0?oo
o`030000003oool0oooo06/0oooo2`00001c0?ooo`00=03oool00`000000oooo0?ooo`1f0?oooaL0
0000G03oool000P0oooo1000000X0?ooo`030000003oool0oooo02T0oooo1000000Y0?ooo`@00000
:P3oool2000000L0oooo8P0000040?ooo`030000003oool0oooo02P0oooo0`0000080?ooo`002P3o
ool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0:03o
ool00`000000oooo0?ooo`0Z0?ooo`040000003oool0oooo000002P0oooo;00000060?ooo`030000
003oool0oooo00D0oooo000:0?ooo`030000003oool0oooo02L0oooo0P00000/0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0W0?ooo`D000009P3o
ool7000000L0oooo00020?ooo`@00000103oool00`000000oooo0?ooo`0W0?ooo`030000003oool0
oooo02/0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0Y0?ooo`800000:P3oool0
10000000oooo0?ooo`00000Z0?ooo`<00000203oool000X0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`0[0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0000002X0oooo00<000000?ooo`3oool0203oool0
00P0oooo0`00000Y0?ooo`030000003oool0oooo02T0oooo0`00000Z0?ooo`040000003oool0oooo
000002T0oooo00@000000?ooo`3oool00000:`3oool2000002X0oooo00<000000?ooo`3oool0203o
ool000X0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo02T0oooo0P00000[0?ooo`800000;@3oool00`000000oooo0?ooo`0X0?ooo`@000001`3oool0
03@0oooo00<000000?ooo`3oool0j@3oool003@0oooo00<000000?ooo`3oool0j@3oool003@0oooo
00<000000?ooo`3oool0j@3oool00?l00000800000010?ooo`001`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`001`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0oooo00<000000?oo
o`3oool0:P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0oooo00<000000?oo
o`3oool0:P3oool00`000000oooo0?ooo`060?ooo`00=03oool00`000000oooo0?ooo`3Y0?ooo`00
=03oool00`000000oooo0?ooo`3Y0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.15782, -1.10064, \
0.0220057, 0.00637218}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["better way?", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(cc1 = \((om + a\ kz\ xi1[k, kz, om])\)\ \((1 + a\ xi2[k, kz, om]^2)\) - 
        4\ a\ s\ xi2[k, kz, om]\ \((om + a\ kz\ xi2[k, kz, om])\)\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(-4\)\ a\ \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\)\ \((\(kz\ \
om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\))\)\ \((om + 
            a\ kz\ \((\(kz\ om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\
\^2\/k\^2)\)\))\))\) + \((1 + 
            a\ \((\(kz\ om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\))\)\^2)\)\ \((om + 
            a\ kz\ \((\(kz\ om\)\/k\^2 + \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\
\^2\/k\^2)\)\))\))\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(cc3 = 
        8\ a^2\ s^2\ om\ kz/
              k^2\ \((1 + 
                a\ xi2[k, kz, om]^2)\) \((kz\ \((1 + a\ xi2[k, kz, om]^2)\) - 
                2\ xi2[k, kz, om] \((om + a\ kz\ xi2[k, kz, om])\))\) + 
          4\ a\ s^2 \((om + 
                a\ kz\ xi2[k, kz, om])\) \((1 + 
                  a\ xi2[k, kz, om]^2)\)^2;\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(cc4 = 
        4\ a\ s^2\ \((\((om + a\ kz\ xi2[k, kz, om])\) \((1 - 
                    3\ a\ xi2[k, kz, om]^2)\) + 
              2\ a\ kz\ xi2[k, kz, 
                  om]\ \((1 + a\ xi2[k, kz, om]^2)\))\);\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ImbetaA[md2_, k_, kz_, om_, a_] = 
        If[om > k, 0, 
          Evaluate[\(-md2\)\ \(om^2/
                  k^3\)\ /\((1 + a\ xi2[k, kz, om]^2)\)^\((3/2)\)\ \((cc1\ I1[
                      1 + a\ xi1[k, kz, om]\ xi2[k, kz, om], 
                      2\ s\ Sqrt[a]] + \((om + a\ kz\ xi2[k, kz, om])\)\ I2[
                      1 + a\ xi1[k, kz, om]\ xi2[k, kz, om], 2\ s\ Sqrt[a]] - 
                  cc3\ I3[1 + a\ xi1[k, kz, om]\ xi2[k, kz, om], 
                      2\ s\ Sqrt[a]] - 
                  cc4\ I4[1 + a\ xi1[k, kz, om]\ xi2[k, kz, om], 
                      2\ s\ Sqrt[a]])\)]];\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[ImbetaA[1, 1, 1/10, 1/2, a], {a, \(-1\), 5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.18254 0.15873 0.603319 0.548158 [
[.02381 .59082 -6 -9 ]
[.02381 .59082 6 0 ]
[.34127 .59082 -3 -9 ]
[.34127 .59082 3 0 ]
[.5 .59082 -3 -9 ]
[.5 .59082 3 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.81746 .59082 -3 -9 ]
[.81746 .59082 3 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.17004 .05516 -12 -4.5 ]
[.17004 .05516 0 4.5 ]
[.17004 .16479 -24 -4.5 ]
[.17004 .16479 0 4.5 ]
[.17004 .27442 -24 -4.5 ]
[.17004 .27442 0 4.5 ]
[.17004 .38406 -24 -4.5 ]
[.17004 .38406 0 4.5 ]
[.17004 .49369 -24 -4.5 ]
[.17004 .49369 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60332 m
.02381 .60957 L
s
[(-1)] .02381 .59082 0 1 Mshowa
.34127 .60332 m
.34127 .60957 L
s
[(1)] .34127 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(2)] .5 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(3)] .65873 .59082 0 1 Mshowa
.81746 .60332 m
.81746 .60957 L
s
[(4)] .81746 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(5)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.15079 .60332 m
.15079 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.27778 .60332 m
.27778 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.53175 .60332 m
.53175 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.9127 .60332 m
.9127 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.18254 .05516 m
.18879 .05516 L
s
[(-1)] .17004 .05516 1 0 Mshowa
.18254 .16479 m
.18879 .16479 L
s
[(-0.8)] .17004 .16479 1 0 Mshowa
.18254 .27442 m
.18879 .27442 L
s
[(-0.6)] .17004 .27442 1 0 Mshowa
.18254 .38406 m
.18879 .38406 L
s
[(-0.4)] .17004 .38406 1 0 Mshowa
.18254 .49369 m
.18879 .49369 L
s
[(-0.2)] .17004 .49369 1 0 Mshowa
.125 Mabswid
.18254 .08257 m
.18629 .08257 L
s
.18254 .10998 m
.18629 .10998 L
s
.18254 .13738 m
.18629 .13738 L
s
.18254 .1922 m
.18629 .1922 L
s
.18254 .21961 m
.18629 .21961 L
s
.18254 .24702 m
.18629 .24702 L
s
.18254 .30183 m
.18629 .30183 L
s
.18254 .32924 m
.18629 .32924 L
s
.18254 .35665 m
.18629 .35665 L
s
.18254 .41146 m
.18629 .41146 L
s
.18254 .43887 m
.18629 .43887 L
s
.18254 .46628 m
.18629 .46628 L
s
.18254 .5211 m
.18629 .5211 L
s
.18254 .5485 m
.18629 .5485 L
s
.18254 .57591 m
.18629 .57591 L
s
.18254 .02775 m
.18629 .02775 L
s
.18254 .00035 m
.18629 .00035 L
s
.25 Mabswid
.18254 0 m
.18254 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02846 .09054 L
.03279 .14484 L
.03793 .19556 L
.04262 .23247 L
.0522 .28972 L
.06244 .33353 L
.07283 .36655 L
.0841 .39396 L
.09385 .41277 L
.10458 .42974 L
.12442 .45387 L
.14545 .47265 L
.16486 .48594 L
.18573 .49726 L
.22696 .5137 L
.26667 .52487 L
.30486 .53294 L
.34551 .53962 L
.38463 .54476 L
.42622 .54921 L
.46628 .55278 L
.50483 .55569 L
.54584 .55836 L
.58533 .56059 L
.6233 .56248 L
.66372 .56426 L
.70263 .5658 L
.74399 .56726 L
.78384 .56853 L
.82217 .56965 L
.86295 .57074 L
.90221 .5717 L
.93996 .57255 L
.97619 .57331 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003o=03oool2
00000>X0oooo000d0?ooo`030000003oool0oooo0>T0oooo000d0?ooo`030000003oool0oooo0>T0
oooo000d0?ooo`030000003oool0oooo0>T0oooo00070?ooo`030000003oool0oooo02X0oooo00<0
00000?ooo`3oool0j@3oool000L0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3Y
0?ooo`001`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0>T0oooo00070?ooo`03
0000003oool0oooo02X0oooo00<000000?ooo`3oool0j@3oool000L0oooo00<000000?ooo`3oool0
:P3oool200000>X0oooo00070?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0j@3o
ool000L0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3Y0?ooo`001`3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo0>T0oooo00070?ooo`030000003oool0oooo02X0
oooo00<000000?ooo`3oool0j@3oool000L0oooo00<000000?ooo`3oool08P3oool4000000@0oooo
00<000000?ooo`3oool0j@3oool000L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003oool0oooo02<0oooo00<00000
0?ooo`3oool00`3oool300000>T0oooo00080?ooo`030000003oool0oooo01/0oooo100000040?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0j@3oool000P0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000
003oool0oooo0240oooo0`0000050?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3Y0?ooo`00203oool0
0`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool0j@3oool000P0oooo00<000000?ooo`3oool0:@3oool00`000000
oooo0?ooo`3Y0?ooo`00203oool00`000000oooo0?ooo`0Y0?ooo`800000jP3oool000P0oooo00<0
00000?ooo`3oool0:@3oool00`000000oooo0?ooo`3Y0?ooo`00203oool00`000000oooo0?ooo`0Y
0?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003oool0oooo02T0oooo00<000000?oo
o`3oool0j@3oool000P0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`3Y0?ooo`00
203oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0>T0oooo00080?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0j@3oool000P0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`3Y0?ooo`00203oool00`000000oooo0?ooo`0Y0?ooo`800000jP3oool000P0
oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`3Y0?ooo`00203oool00`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo0>T0oooo00090?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool0j@3oool000T0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`3Y
0?ooo`002@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0>T0oooo00090?ooo`03
0000003oool0oooo02P0oooo00<000000?ooo`3oool0j@3oool000T0oooo00<000000?ooo`3oool0
:03oool00`000000oooo0?ooo`3Y0?ooo`002@3oool00`000000oooo0?ooo`0X0?ooo`800000jP3o
ool000T0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`3Y0?ooo`002@3oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo0>T0oooo00090?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0j@3oool000T0oooo00<000000?ooo`3oool0:03oool00`000000oooo
0?ooo`3Y0?ooo`002@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0>T0oooo000:
0?ooo`030000003oool0oooo01@0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000005
0?ooo`030000003oool0oooo0>T0oooo000:0?ooo`030000003oool0oooo01<0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0>T0oooo000:
0?ooo`030000003oool0oooo01<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`<00000j@3oool000X0oooo00<000000?ooo`3oool03@3oool400000080oooo00@0
00000?ooo`3oool000002@3oool2000000D0oooo00<000000?ooo`3oool0j@3oool000X0oooo00<0
00000?ooo`3oool04`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool0j@3oool000X0oooo00<000000?ooo`3oool04`3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0j@3oool000X0
oooo00<000000?ooo`3oool0503oool2000000X0oooo0P0000050?ooo`030000003oool0oooo0>T0
oooo000:0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0j@3oool000/0oooo00<0
00000?ooo`3oool09P3oool00`000000oooo0?ooo`3Y0?ooo`002`3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo0>T0oooo000;0?ooo`030000003oool0oooo02H0oooo0P00003Z0?oo
o`002`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0>T0oooo000;0?ooo`030000
003oool0oooo02H0oooo00<000000?ooo`3oool0j@3oool000/0oooo00<000000?ooo`3oool09P3o
ool00`000000oooo0?ooo`3Y0?ooo`002`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo0>T0oooo000;0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0j@3oool000/0
oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`3Y0?ooo`00303oool00`000000oooo
0?ooo`0U0?ooo`800000jP3oool000`0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?oo
o`3Y0?ooo`00303oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0>T0oooo000<0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool0j@3oool000`0oooo00<000000?ooo`3o
ool09@3oool00`000000oooo0?ooo`3Y0?ooo`00303oool00`000000oooo0?ooo`0U0?ooo`030000
003oool0oooo0>T0oooo000<0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0j@3o
ool000d0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`3Y0?ooo`003@3oool00`00
0000oooo0?ooo`0T0?ooo`800000jP3oool000d0oooo00<000000?ooo`3oool0903oool00`000000
oooo0?ooo`3Y0?ooo`003@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0>T0oooo
000=0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0j@3oool000d0oooo00<00000
0?ooo`3oool0903oool00`000000oooo0?ooo`3Y0?ooo`003P3oool00`000000oooo0?ooo`0S0?oo
o`030000003oool0oooo0>T0oooo000>0?ooo`030000003oool0oooo0100oooo0P0000040?ooo`03
0000003oool0oooo00<0oooo0P0000050?ooo`030000003oool0oooo0>T0oooo000>0?ooo`030000
003oool0oooo00l0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo0>T0oooo000>0?ooo`030000003oool0oooo00l0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000040?ooo`<00000j@3oool000h0oooo00<000000?oo
o`3oool02@3oool400000080oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<00000
0?ooo`3oool0j@3oool000h0oooo00<000000?ooo`3oool03`3oool010000000oooo0?ooo`000008
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0j@3oool000l0oooo00<000000?oo
o`3oool03P3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0j@3oool000l0oooo00<000000?ooo`3oool03`3oool2000000X0oooo0`0000040?oo
o`030000003oool0oooo0>T0oooo000?0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3o
ool0j@3oool000l0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`3Y0?ooo`003`3o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0>T0oooo000@0?ooo`030000003oool0
oooo0240oooo0P00003Z0?ooo`00403oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0>T0oooo000@0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0j@3oool00100oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`3Y0?ooo`004@3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo0>T0oooo000A0?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool0j@3oool00140oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3Y0?oo
o`004@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0>T0oooo000B0?ooo`030000
003oool0oooo01l0oooo0P00003Z0?ooo`004P3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo0>T0oooo000B0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0j@3oool0
01<0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`3Y0?ooo`004`3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo0>T0oooo000C0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool0j@3oool001<0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?oo
o`3Y0?ooo`00503oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0>T0oooo000D0?oo
o`030000003oool0oooo01d0oooo0P00003Z0?ooo`00503oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo0>T0oooo000E0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0
j@3oool001D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3Y0?ooo`005@3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0>T0oooo000F0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool0j@3oool001H0oooo00<000000?ooo`3oool0203oool2000000@0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0>T0
oooo000G0?ooo`030000003oool0oooo00H0oooo00@000000?ooo`3oool000002P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0>T0oooo000G0?ooo`030000003oool0oooo00H0oooo
00@000000?ooo`3oool000001`3oool5000000@0oooo0`00003Y0?ooo`00603oool00`000000oooo
0000000300000080oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`0000050?oo
o`030000003oool0oooo0>T0oooo000H0?ooo`030000003oool0oooo00D0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo000000050?ooo`030000003oool0oooo0>T0oooo000H0?ooo`03
0000003oool0oooo00D0oooo00@000000?ooo`3oool000002@3oool2000000D0oooo00<000000?oo
o`3oool0j@3oool001T0oooo00<000000?ooo`3oool01@3oool2000000/0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`3Y0?ooo`006@3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo0>T0oooo000J0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0j@3o
ool001X0oooo00<000000?ooo`3oool05`3oool200000>X0oooo000K0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0j@3oool001`0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`3Y0?ooo`00703oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0>T0oooo
000M0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0j@3oool001d0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`3Y0?ooo`007P3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0>T0oooo000O0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0j@3oool00200oooo00<000000?ooo`3oool04@3oool200000>X0oooo000P0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0j@3oool00240oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`3Y0?ooo`008P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
0>T0oooo000S0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0j@3oool002@0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3Y0?ooo`009@3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo0>T0oooo000V0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0j@3oool002L0oooo00<000000?ooo`3oool02P3oool200000>X0oooo000X0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0j@3oool002T0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`3Y0?ooo`00:P3oool2000000P0oooo00<000000?ooo`3oool0j@3o
ool002`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3Y0?ooo`00;@3oool20000
00D0oooo00<000000?ooo`3oool0j@3oool00240oooo0P0000040?ooo`030000003oool0oooo0080
oooo100000040?ooo`030000003oool0oooo0>T0oooo000P0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool0oooo0P0000020?ooo`030000003oool0oooo0>T0oooo000P0?ooo`04
0000003oool0oooo000000T0oooo00<000000?ooo`3oool00P3oool500000>T0oooo000J0?ooo`@0
00000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo0`00003Y0?oo
o`00803oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool00P00003W0?ooo`00803oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool00P3oool200000>D0oooo000Q0?ooo`8000002P3oool20000
00D0oooo00<000000?ooo`3oool0103oool300000>80oooo000d0?ooo`030000003oool0oooo00L0
oooo0P00003P0?ooo`00=03oool00`000000oooo0?ooo`090?ooo`<00000g@3oool003@0oooo00<0
00000?ooo`3oool0303oool400000=T0oooo000d0?ooo`8000004@3oool400000=D0oooo000d0?oo
o`030000003oool0oooo01@0oooo1000003A0?ooo`00=03oool00`000000oooo0?ooo`0H0?ooo`H0
0000b`3oool003@0oooo00<000000?ooo`3oool07P3oool500000<H0oooo000d0?ooo`030000003o
ool0oooo02<0oooo1P0000300?ooo`00=03oool00`000000oooo0?ooo`0Y0?ooo`H00000^P3oool0
03@0oooo00<000000?ooo`3oool0;`3oool600000;@0oooo000d0?ooo`030000003oool0oooo03D0
oooo2000002/0?ooo`00=03oool2000003h0oooo3000002P0?ooo`00=03oool00`000000oooo0?oo
o`190?ooo`/00000U@3oool003@0oooo00<000000?ooo`3oool0E03ooolG000007h0oooo000d0?oo
o`030000003oool0oooo06/0oooo2`00001c0?ooo`00=03oool00`000000oooo0?ooo`1f0?oooaL0
0000G03oool000P0oooo1000000X0?ooo`030000003oool0oooo02T0oooo1000000Y0?ooo`@00000
:P3oool2000000L0oooo8P0000040?ooo`030000003oool0oooo02P0oooo0`0000080?ooo`002P3o
ool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0:03o
ool00`000000oooo0?ooo`0Z0?ooo`040000003oool0oooo000002P0oooo;00000060?ooo`030000
003oool0oooo00D0oooo000:0?ooo`030000003oool0oooo02L0oooo0P00000/0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0W0?ooo`D000009P3o
ool7000000L0oooo00020?ooo`@00000103oool00`000000oooo0?ooo`0W0?ooo`030000003oool0
oooo02/0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0Y0?ooo`800000:P3oool0
10000000oooo0?ooo`00000Z0?ooo`<00000203oool000X0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`0[0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0000002X0oooo00<000000?ooo`3oool0203oool0
00P0oooo0`00000Y0?ooo`030000003oool0oooo02T0oooo0`00000Z0?ooo`040000003oool0oooo
000002T0oooo00@000000?ooo`3oool00000:`3oool2000002X0oooo00<000000?ooo`3oool0203o
ool000X0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo02T0oooo0P00000[0?ooo`800000;@3oool00`000000oooo0?ooo`0X0?ooo`@000001`3oool0
03@0oooo00<000000?ooo`3oool0j@3oool003@0oooo00<000000?ooo`3oool0j@3oool003@0oooo
00<000000?ooo`3oool0j@3oool00?l00000800000010?ooo`001`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`001`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0oooo00<000000?oo
o`3oool0:P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0oooo00<000000?oo
o`3oool0:P3oool00`000000oooo0?ooo`060?ooo`00=03oool00`000000oooo0?ooo`3Y0?ooo`00
=03oool00`000000oooo0?ooo`3Y0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.15782, -1.10064, \
0.0220057, 0.00637218}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["for imaginary om", "Subsection"],

Cell[BoxData[
    \(betaspecial[md2_, k_, kz_, om_, a_] := 
      md2/k^2*om^2/
          2*\((\(-NIntegrate[1/\((1 + a\ xi^2)\)^2, {xi, \(-1\), 1}, 
                WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 20]\) + 
            NIntegrate[\((om + a*kz*xi)\)/\((1 + a\ xi^2)\)^2*
                R[om - kz\ xi, 
                  Sqrt[k^2 - kz^2]\ Sqrt[1 - xi^2]], {xi, \(-1\), \ 1}, 
              WorkingPrecision \[Rule] 30, 
              MaxRecursion \[Rule] 20])\)\)], "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["static limit", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[Integrate[1/\((1 + a\ xi^2)\)^2, {xi, 0, 1}]]\)], "Input"],

Cell[BoxData[
    \(\(\@a + \((1 + a)\)\ ArcTan[\@a]\)\/\(2\ \@a\ \((1 + a)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      Integrate[
        1/\((1 + a\ xi^2)\)^2*xi/Sqrt[xi^2 - kt^2], {xi, kt, 1}]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\@a\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - \((1 + 
                  a)\)\ ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ \
kt\^2\)]\)\/\(2\ \@a\ \((1 + 
                a)\)\ \((\(-1\) - a\ kt\^2)\)\^\(3/2\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(betamoz[md2_, kz_, k_, 
        a_] := \(-md2\)/
          k^2*\(\(\(\((Sqrt[a] + \((1 + a)\) ArcTan[Sqrt[a]] + a^\((3/2)\) + 
                    a\ \((1 + a)\) \((\((k^2 - kz^2)\)/k^2*ArcTan[Sqrt[a]] + 
                          kz/Sqrt[k^2 + a*\((k^2 - kz^2)\)]*
                            ArcTan[
                              Sqrt[a]*\(kz/k\)/
                                  Sqrt[1 + a*\((1 - kz^2/k^2)\)]])\))\)/2\)/
              Sqrt[a]\)/\((1 + a)\)\)/\((1 + 
              a*\((1 - kz^2/k^2)\))\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(betaspecial[1, 1/2, 1/2\ \ Cos[Pi/16], 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(21\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(-0.19489431381225586`\)."\)], "Message"],

Cell[BoxData[
    \(\(-4.9186281673029626`4.1891*^-8\) + 
      0``11.4972\ \[ImaginaryI]\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[betamoz[1, 1/2*Cos[Pi/16], 1/2, 1]]\)], "Input"],

Cell[BoxData[
    \(\(-4.918829408519437`\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[betamoz[1, Cos[th], 1, 10], {th, 0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.603319 0.218367 [
[.2619 .59082 -9 -9 ]
[.2619 .59082 9 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .0574 -24 -4.5 ]
[.01131 .0574 0 4.5 ]
[.01131 .16658 -12 -4.5 ]
[.01131 .16658 0 4.5 ]
[.01131 .27577 -24 -4.5 ]
[.01131 .27577 0 4.5 ]
[.01131 .38495 -12 -4.5 ]
[.01131 .38495 0 4.5 ]
[.01131 .49414 -24 -4.5 ]
[.01131 .49414 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(0.2)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(0.4)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.6)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(0.8)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.08333 .60332 m
.08333 .60707 L
s
.14286 .60332 m
.14286 .60707 L
s
.20238 .60332 m
.20238 .60707 L
s
.32143 .60332 m
.32143 .60707 L
s
.38095 .60332 m
.38095 .60707 L
s
.44048 .60332 m
.44048 .60707 L
s
.55952 .60332 m
.55952 .60707 L
s
.61905 .60332 m
.61905 .60707 L
s
.67857 .60332 m
.67857 .60707 L
s
.79762 .60332 m
.79762 .60707 L
s
.85714 .60332 m
.85714 .60707 L
s
.91667 .60332 m
.91667 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .0574 m
.03006 .0574 L
s
[(-2.5)] .01131 .0574 1 0 Mshowa
.02381 .16658 m
.03006 .16658 L
s
[(-2)] .01131 .16658 1 0 Mshowa
.02381 .27577 m
.03006 .27577 L
s
[(-1.5)] .01131 .27577 1 0 Mshowa
.02381 .38495 m
.03006 .38495 L
s
[(-1)] .01131 .38495 1 0 Mshowa
.02381 .49414 m
.03006 .49414 L
s
[(-0.5)] .01131 .49414 1 0 Mshowa
.125 Mabswid
.02381 .07924 m
.02756 .07924 L
s
.02381 .10107 m
.02756 .10107 L
s
.02381 .12291 m
.02756 .12291 L
s
.02381 .14475 m
.02756 .14475 L
s
.02381 .18842 m
.02756 .18842 L
s
.02381 .21026 m
.02756 .21026 L
s
.02381 .23209 m
.02756 .23209 L
s
.02381 .25393 m
.02756 .25393 L
s
.02381 .2976 m
.02756 .2976 L
s
.02381 .31944 m
.02756 .31944 L
s
.02381 .34128 m
.02756 .34128 L
s
.02381 .36311 m
.02756 .36311 L
s
.02381 .40679 m
.02756 .40679 L
s
.02381 .42862 m
.02756 .42862 L
s
.02381 .45046 m
.02756 .45046 L
s
.02381 .4723 m
.02756 .4723 L
s
.02381 .51597 m
.02756 .51597 L
s
.02381 .53781 m
.02756 .53781 L
s
.02381 .55965 m
.02756 .55965 L
s
.02381 .58148 m
.02756 .58148 L
s
.02381 .03556 m
.02756 .03556 L
s
.02381 .01373 m
.02756 .01373 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .01472 m
.0403 .01549 L
.04458 .01642 L
.05429 .01934 L
.06375 .0232 L
.07387 .0284 L
.09565 .04311 L
.11547 .06015 L
.15491 .10189 L
.19284 .1479 L
.2332 .19848 L
.27205 .24541 L
.30942 .28696 L
.3492 .32636 L
.38749 .35931 L
.4282 .38924 L
.46741 .41358 L
.50512 .43336 L
.54526 .45107 L
.5839 .46537 L
.62496 .47812 L
.66453 .48841 L
.7026 .49679 L
.74309 .50434 L
.78208 .51051 L
.81958 .5156 L
.8595 .52025 L
.89792 .5241 L
.93877 .52763 L
.97619 .53042 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool2
00000?l0oooo103oool001/0oooo00<000000?ooo`3oool01000003n0?ooo`006`3oool00`000000
oooo0?ooo`040?ooo`800000o03oool001/0oooo00<000000?ooo`3oool01P3oool300000?T0oooo
000K0?ooo`030000003oool0oooo00T0oooo0P00003g0?ooo`006`3oool00`000000oooo0?ooo`0;
0?ooo`800000m@3oool001/0oooo0P00000>0?ooo`030000003oool0oooo0?80oooo000K0?ooo`03
0000003oool0oooo00h0oooo0P00003b0?ooo`006`3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0>l0oooo000K0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0kP3o
ool000L0oooo100000030?ooo`030000003oool0oooo0080oooo0`0000050?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0k@3oool000L0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0k03oool000P0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`020?ooo`8000005@3oool00`000000
oooo0?ooo`3[0?ooo`0000<0oooo0000000000000P0000040?ooo`030000003oool0oooo00L0oooo
0`0000050?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0jP3oool000X0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01H0oooo00<0
00000?ooo`3oool0j@3oool000L0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0j03oool000P0oooo0P000009
0?ooo`@00000103oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0>L0oooo000K0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0iP3oool001/0oooo0P00000K0?ooo`03
0000003oool0oooo0>D0oooo000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
i@3oool001/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3T0?ooo`006`3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0><0oooo000K0?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool0hP3oool001/0oooo0P00000O0?ooo`030000003oool0oooo0>40
oooo000K0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0h03oool001/0oooo00<0
00000?ooo`3oool0803oool00`000000oooo0?ooo`3O0?ooo`006`3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0=h0oooo000K0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool0gP3oool001/0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`3M0?ooo`00
6`3oool2000002@0oooo00<000000?ooo`3oool0g03oool001/0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`3K0?ooo`006`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0
oooo0=X0oooo000K0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0f@3oool001/0
oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`3I0?ooo`006`3oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo0=P0oooo000K0?ooo`800000:@3oool00`000000oooo0?oo
o`3G0?ooo`006`3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0=H0oooo000K0?oo
o`030000003oool0oooo02X0oooo00<000000?ooo`3oool0e@3oool001/0oooo00<000000?ooo`3o
ool0:`3oool00`000000oooo0?ooo`3D0?ooo`004`3oool4000000@0oooo00<000000?ooo`3oool0
;03oool00`000000oooo0?ooo`3C0?ooo`004`3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo02`0oooo00<000000?ooo`3oool0d`3oool001@0oooo00<000000?ooo`3oool0103oool2
000002h0oooo00<000000?ooo`3oool0dP3oool000d0oooo100000040?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`3A0?ooo`005P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0d03oool001<0oooo
00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0<l0
oooo000D0?ooo`8000001@3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0<h0oooo
000K0?ooo`800000<P3oool00`000000oooo0?ooo`3>0?ooo`006`3oool00`000000oooo0?ooo`0b
0?ooo`030000003oool0oooo0<d0oooo000K0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool0c03oool001/0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`3;0?ooo`00
6`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo0<X0oooo000K0?ooo`030000003o
ool0oooo03H0oooo00<000000?ooo`3oool0b@3oool001/0oooo0P00000g0?ooo`030000003oool0
oooo0<T0oooo000K0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0b03oool001/0
oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`370?ooo`006`3oool00`000000oooo
0?ooo`0i0?ooo`030000003oool0oooo0<H0oooo000K0?ooo`030000003oool0oooo03X0oooo00<0
00000?ooo`3oool0a@3oool001/0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`34
0?ooo`006`3oool2000003`0oooo00<000000?ooo`3oool0a03oool001/0oooo00<000000?ooo`3o
ool0?03oool00`000000oooo0?ooo`330?ooo`006`3oool00`000000oooo0?ooo`0m0?ooo`030000
003oool0oooo0<80oooo000K0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0`@3o
ool001/0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`300?ooo`006`3oool00`00
0000oooo0?ooo`100?ooo`030000003oool0oooo0;l0oooo000K0?ooo`800000@P3oool00`000000
oooo0?ooo`2n0?ooo`006`3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0;d0oooo
000K0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0_03oool000L0oooo10000003
0?ooo`030000003oool0oooo0080oooo0`0000050?ooo`030000003oool0oooo04<0oooo00<00000
0?ooo`3oool0_03oool000T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo04@0oooo00<000000?ooo`3oool0^`3oool000T0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`020?ooo`800000AP3oool00`000000oooo0?ooo`2j0?ooo`00
00<0oooo0000000000000P0000040?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`030000
003oool0oooo04H0oooo00<000000?ooo`3oool0^@3oool000T0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0^03o
ool000L0oooo0`0000090?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0B03oool0
0`000000oooo0?ooo`2g0?ooo`002@3oool00`000000oooo0?ooo`070?ooo`@00000103oool00`00
0000oooo0?ooo`190?ooo`030000003oool0oooo0;H0oooo000K0?ooo`030000003oool0oooo04X0
oooo00<000000?ooo`3oool0]@3oool001/0oooo0P00001<0?ooo`030000003oool0oooo0;@0oooo
000K0?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0/`3oool001/0oooo00<00000
0?ooo`3oool0C@3oool00`000000oooo0?ooo`2b0?ooo`006`3oool00`000000oooo0?ooo`1=0?oo
o`030000003oool0oooo0;80oooo000K0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3o
ool0/@3oool001/0oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`2`0?ooo`006`3o
ool200000540oooo00<000000?ooo`3oool0[`3oool001/0oooo00<000000?ooo`3oool0D@3oool0
0`000000oooo0?ooo`2^0?ooo`006`3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo
0:d0oooo000K0?ooo`030000003oool0oooo05<0oooo0P00002]0?ooo`006`3oool00`000000oooo
0?ooo`1E0?ooo`030000003oool0oooo0:X0oooo000K0?ooo`030000003oool0oooo05H0oooo00<0
00000?ooo`3oool0Z@3oool001/0oooo0P00001H0?ooo`800000Z@3oool001/0oooo00<000000?oo
o`3oool0F@3oool00`000000oooo0?ooo`2V0?ooo`006`3oool00`000000oooo0?ooo`1J0?ooo`03
0000003oool0oooo0:D0oooo000K0?ooo`030000003oool0oooo05/0oooo0P00002U0?ooo`006`3o
ool00`000000oooo0?ooo`1M0?ooo`030000003oool0oooo0:80oooo000K0?ooo`800000G`3oool0
0`000000oooo0?ooo`2Q0?ooo`006`3oool00`000000oooo0?ooo`1O0?ooo`800000X@3oool001/0
oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?ooo`2N0?ooo`006`3oool00`000000oooo
0?ooo`1R0?ooo`030000003oool0oooo09d0oooo000C0?ooo`@00000103oool00`000000oooo0?oo
o`1S0?ooo`030000003oool0oooo09`0oooo000E0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0I03oool2000009`0oooo000E0?ooo`030000003oool0oooo00<0oooo0P00001W0?oo
o`030000003oool0oooo09T0oooo000=0?ooo`@00000103oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo06L0oooo00<000000?ooo`3oool0V03oool001D0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`1X0?ooo`800000V03oool001<0oooo0`0000050?ooo`030000003o
ool0oooo06X0oooo00<000000?ooo`3oool0U@3oool001D0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`1[0?ooo`800000U@3oool001/0oooo00<000000?ooo`3oool0K@3oool20000
09<0oooo000K0?ooo`800000L03oool00`000000oooo0?ooo`2@0?ooo`006`3oool00`000000oooo
0?ooo`1`0?ooo`800000T03oool001/0oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?oo
o`2=0?ooo`006`3oool00`000000oooo0?ooo`1c0?ooo`800000S@3oool001/0oooo00<000000?oo
o`3oool0M@3oool2000008/0oooo000K0?ooo`030000003oool0oooo07L0oooo0P0000290?ooo`00
6`3oool2000007X0oooo0P0000270?ooo`006`3oool00`000000oooo0?ooo`1k0?ooo`800000Q@3o
ool001/0oooo00<000000?ooo`3oool0O@3oool300000880oooo000K0?ooo`030000003oool0oooo
0800oooo0P0000200?ooo`006`3oool00`000000oooo0?ooo`220?ooo`<00000O@3oool001/0oooo
0P0000260?ooo`<00000NP3oool001/0oooo00<000000?ooo`3oool0R03oool2000007P0oooo000K
0?ooo`030000003oool0oooo08X0oooo0`00001e0?ooo`006`3oool00`000000oooo0?ooo`2=0?oo
o`800000L`3oool001/0oooo00<000000?ooo`3oool0S`3oool300000700oooo000K0?ooo`030000
003oool0oooo0980oooo0`00001]0?ooo`006`3oool2000009H0oooo0P00001[0?ooo`006`3oool0
0`000000oooo0?ooo`2G0?ooo`<00000J03oool001/0oooo00<000000?ooo`3oool0VP3oool40000
06@0oooo000K0?ooo`030000003oool0oooo09h0oooo1@00001O0?ooo`00203oool2000000@0oooo
00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0X`3oool5000005X0oooo
00070?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`2X0?ooo`D00000E@3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo
0?ooo`020?ooo`800000[P3oool500000500oooo00000`3oool000000000000200000080oooo00@0
00000?ooo`3oool00000203oool3000000D0oooo00<000000?ooo`3oool0/P3oool6000004X0oooo
00070?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`2h0?ooo`D00000A@3oool000L0oooo00@000000?ooo`3oool00000203oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo0;d0oooo1@0000100?ooo`00203oool2000000T0oooo1000
00040?ooo`030000003oool0oooo0<80oooo1`00000i0?ooo`006`3oool00`000000oooo0?ooo`39
0?ooo`P00000<@3oool001/0oooo0P00003B0?ooo`H00000:`3oool001/0oooo00<000000?ooo`3o
ool0e`3oool7000002@0oooo000K0?ooo`030000003oool0oooo0=h0oooo2`00000I0?ooo`006`3o
ool00`000000oooo0?ooo`3Y0?oooa0000002@3oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool200000?l0oooo103oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
6`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3o
ool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo000K0?ooo`800000o`3oool40?ooo`006`3oool00`000000oooo0?ooo`0d0?ooo`800000
103oool00`000000oooo0?ooo`020?ooo`@00000<03oool2000000@0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0_0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000
<@3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo000K0?ooo`030000003o
ool0oooo03<0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0`0?ooo`040000
003oool0oooo000000X0oooo00<000000?ooo`3oool0;P3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo000002l0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000020?ooo`006`3oool00`000000oooo0?ooo`0c0?ooo`040000003oool0oooo000000T0oooo
00<000000?ooo`3oool0;`3oool010000000oooo0?ooo`0000070?ooo`D00000;`3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000002l0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000020?ooo`006`3oool00`000000oooo0?ooo`0c0?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0;P3oool010000000oooo0?ooo`0000070?ooo`04
0000003oool0oooo00000300oooo00@000000?ooo`3oool00000203oool300000300oooo00@00000
0?ooo`3oool000002@3oool2000000<0oooo000K0?ooo`030000003oool0oooo03<0oooo00@00000
0?ooo`3oool000002`3oool00`000000oooo0?ooo`0]0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`000000<03oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo0300
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`006`3oool20000
03@0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000_0?ooo`040000003o
ool0oooo000000T0oooo0P00000`0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3o
ool0<03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo000K0?oo
o`030000003oool0oooo03@0oooo0P00000:0?ooo`800000<@3oool2000000/0oooo00<000000?oo
o`3oool0;`3oool2000000X0oooo0`00000`0?ooo`8000002P3oool2000000<0oooo000K0?ooo`03
0000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3o
ool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool0
01D0ooooo`000009000000<0oooo000K0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3o
ool0?03oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3o
ool01`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo
003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0870867, -2.88678, \
0.00319096, 0.0173963}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Fast beta", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rearg4 = 
      Compile[{md2, k, kz, om, a, xi}, 1/\((1 + a\ xi^2)\)^2*om]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, xi}, 
        om\/\((1 + a\ xi\^2)\)\^2, "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rearg5 = 
      Compile[{md2, k, kz, om, a, xi}, 
        Sign[om - kz\ xi]/\((1 + a\ xi^2)\)^2*\((om^2 + a*om*kz*xi)\)/
            Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - om^2)\)*\((1 - 
                      kz^2/k^2)\)]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(Sign[om - kz\ xi]\ \((om\^2 + a\ om\ kz\ xi)\)\)\/\(\((1 + a\
\ xi\^2)\)\^2\ \@\(k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - om\^2)\)\ \
\((1 - kz\^2\/k\^2)\)\)\), "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(FastRebeta[md2_, k_, kz_, om_, a_] := 
      Module[{res, mxi1, mxi2}, 
        If[om < k, mxi1 = If[k > om, xi1[k, kz, om], 0]; 
          mxi2 = If[k > om, xi2[k, kz, om], 0]; 
          If[logging \[Equal] 0, 
            Print["\<beta: k,kz,om,a:\>", k, kz, om, 
              a]]; \[IndentingNewLine]res = 
            md2/k^2*om/
                2*\((\(-NIntegrate[
                      Rearg4[md2, k, kz, om, a, xi], {xi, \(-1\), 1}]\) + 
                  If[mxi2 \[Equal] \(-1\), 0, 
                    NIntegrate[
                      Rearg5[md2, k, kz, om, a, xi], {xi, \(-1\), 
                        mxi2*If[
                            mxi2 < 
                              0, \((1 + 10^\((\(-11\))\))\), \((1 - 
                                10^\((\(-11\))\))\)]}]] + 
                  NIntegrate[
                    Rearg5[md2, k, kz, om, a, xi], {xi, 
                      mxi1*\((1 + 10^\((\(-11\))\))\), 1}])\), \(res = 
              md2/k^2*om/
                  2*\((\(-NIntegrate[
                        Rearg4[md2, k, kz, om, a, xi], {xi, \(-1\), 1}]\) + 
                    NIntegrate[
                      Rearg5[md2, k, kz, om, a, xi], {xi, \(-1\), 
                        If[om\ kz/k^2 < 1, om\ kz/k^2, 0], \ 1}])\);\)]; \ 
        res]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRebeta[1, 1, 8/10, 4/10, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.11444772322680133`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rebeta[1, 1, 8/10, 4/10, 1]\)], "Input"],

Cell[BoxData[
    \("beta"\)], "Print"],

Cell[BoxData[
    \(\(-0.114447835002736487540647`10.8531\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Fbeta[md2_, k_, kz_, om_, a_] := 
      FastRebeta[md2, k, kz, om, a] + 
        I*N[ImbetaA[md2, k, kz, om, a]]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fbeta[1, 1, 8/10, 4/10, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.11444783333953779`\) - 
      0.09958279445349062`\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["beta kx=0", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\(xi/\((1 + a\ xi^2)\)^2\)\ /\((eta - xi + 
            I\ eps)\), {xi, \(-1\), 1}]\)], "Input"],

Cell[BoxData[
    \(\((\(-2\)\ \@a + 2\ a\^\(3/2\)\ eps\^2 - 
          4\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta - 
          2\ a\^\(3/2\)\ eta\^2 + \[ImaginaryI]\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\ eps\^2\ Log[\[ImaginaryI]\ \
\((\(-\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\^2\ eps\^2\ Log[\
\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] + 
          2\ a\ eps\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] \
+ 2\ a\^2\ eps\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - \
\[ImaginaryI]\ a\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] \
- \[ImaginaryI]\ a\^2\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] - \[ImaginaryI]\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - \
\[ImaginaryI]\ a\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - \
\[ImaginaryI]\ a\ eps\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] \
- \[ImaginaryI]\ a\^2\ eps\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] - 
          2\ a\ eps\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] \
- 2\ a\^2\ eps\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + \
\[ImaginaryI]\ a\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] \
+ \[ImaginaryI]\ a\^2\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] + \[ImaginaryI]\ \@a\ eps\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + \@a\ eta\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          a\^\(3/2\)\ eta\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - \[ImaginaryI]\ \@a\ eps\ Log[\(1 - \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] - \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[\(1 - \
\[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] - \@a\ eta\ Log[\(1 - \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] - 
          a\^\(3/2\)\ eta\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] \
- \[ImaginaryI]\ \@a\ eps\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - \@a\ eta\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          a\^\(3/2\)\ eta\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + \[ImaginaryI]\ \@a\ eps\ Log[\(1 + \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] + \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[\(1 + \
\[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] + \@a\ eta\ Log[\(1 + \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] + 
          a\^\(3/2\)\ eta\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] \
- \[ImaginaryI]\ \@a\ eps\ Log[
              1 + eps\^2 - 2\ eta + 
                eta\^2] - \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - \@a\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - 
          a\^\(3/2\)\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] + \[ImaginaryI]\ \@a\ eps\ Log[
              1 + eps\^2 + 2\ eta + 
                eta\^2] + \[ImaginaryI]\ a\^\(3/2\)\ eps\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + \@a\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + 
          a\^\(3/2\)\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2])\)/\((2\ \@a\ \((1 + 
              a)\)\ \((\(-1\) + a\ eps\^2 - 2\ \[ImaginaryI]\ a\ eps\ eta - a\
\ eta\^2)\)\^2)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Limit[%, eps \[Rule] 0]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(2\ \@a\ \((1 + 
              a)\)\ \((1 + a\ eta\^2)\)\^2\)\)\((\(-2\)\ \@a - 
        2\ a\^\(3/2\)\ eta\^2 + \[ImaginaryI]\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - \[ImaginaryI]\ a\ eta\^2\ Log[\[ImaginaryI]\ \
\((\(-\[ImaginaryI]\) + \@a)\)] - \[ImaginaryI]\ a\^2\ eta\^2\ Log[\
\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - \[ImaginaryI]\ Log[\(-\
\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - \[ImaginaryI]\ a\ Log[\(-\
\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + \[ImaginaryI]\ a\ eta\^2\ \
Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + \[ImaginaryI]\ a\^2\ eta\
\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - \@a\ eta\ Log[
            1 - 2\ eta + eta\^2] - 
        a\^\(3/2\)\ eta\ Log[1 - 2\ eta + eta\^2] + \@a\ eta\ Log[
            1 + 2\ eta + eta\^2] + 
        a\^\(3/2\)\ eta\ Log[1 + 2\ eta + eta\^2])\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0[eta_, a_] = 
      eta^2/2\ \((1 + a)\)^\((3/2)\) FullSimplify[%]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(4\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\@\(1 + a\)\ eta\^2\ \((\
\[ImaginaryI]\ \((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                  a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + 
                        a\ eta\^2)\) + \((1 + 
                        a)\)\ eta\ \((\(-Log[\((\(-1\) + eta)\)\^2]\) + 
                        Log[\((1 + eta)\)\^2])\))\))\))\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[eta_, a_] = 
      betakx0[eta, a] - 
        I\ eta^3\ Pi/2\ \((1 + a)\)^\((3/2)\)/\((1 + a\ eta^2)\)^2\ UnitStep[
            1 - eta]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(4\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\@\(1 + a\)\ eta\^2\ \((\
\[ImaginaryI]\ \((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                  1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                    a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                  1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + 
                          a\ eta\^2)\) + \((1 + 
                          a)\)\ eta\ \((\(-Log[\((\(-1\) + eta)\)\^2]\) + 
                          Log[\((1 + eta)\)\^2])\))\))\))\)\) - \(\
\[ImaginaryI]\ \((1 + a)\)\^\(3/2\)\ eta\^3\ \[Pi]\ UnitStep[1 - eta]\)\/\(2\ \
\((1 + a\ eta\^2)\)\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[eta_, 
        a_] = \(1\/\(4\ \@a\ \((1 + a\ eta\^2)\)\^2\)\) \((\@\(1 + a\)\ \
eta\^2\ \((\[ImaginaryI]\ \((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                      1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                        a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                      1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + 
                              a\ eta\^2)\) + \((1 + 
                              a)\)\ eta\ \((\(-Log[\((\(-1\) + eta)\)\^2]\) + 
                              Log[\((1 + eta)\)\^2])\))\))\))\) - \(\
\[ImaginaryI]\ \((1 + a)\)\^\(3/2\)\ eta\^3\ \[Pi]\ UnitStep[1 - Re[eta]]\)\/\
\(2\ \((1 + a\ eta\^2)\)\^2\)\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(4\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\@\(1 + a\)\ eta\^2\ \((\
\[ImaginaryI]\ \((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                  1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                    a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                  1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + 
                          a\ eta\^2)\) + \((1 + 
                          a)\)\ eta\ \((\(-Log[\((\(-1\) + eta)\)\^2]\) + 
                          Log[\((1 + eta)\)\^2])\))\))\))\)\) - \(\
\[ImaginaryI]\ \((1 + a)\)\^\(3/2\)\ eta\^3\ \[Pi]\ UnitStep[1 - Re[eta]]\)\/\
\(2\ \((1 + a\ eta\^2)\)\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[0.4, 10^4]\)], "Input"],

Cell[BoxData[
    \(\(\(0.7846359060566892`\)\(\[InvisibleSpace]\)\) - 
      0.039226750042854794`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.14786621025190982`\)\(\[InvisibleSpace]\)\) - 
      0.5425530169800588`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(modifying\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[eta_, 
        a_] = \(1\/\(4\ \@a\ \((1 + a\ eta\^2)\)\^2\)\) \((\@\(1 + a\)\ \
eta\^2\ \((\[ImaginaryI]\ \((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                    1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                      a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                    1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + 
                            a\ eta\^2)\) + \((1 + 
                            a)\)\ eta\ 2\ \((\(-Log[\((\(-1\) + eta)\)]\) + 
                            Log[\((1 + eta)\)])\))\))\))\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(1\/\(4\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\@\(1 + a\)\ eta\^2\ \((\
\[ImaginaryI]\ \((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                  a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + 
                        a\ eta\^2)\) + 
                  2\ \((1 + a)\)\ eta\ \((\(-Log[\(-1\) + eta]\) + 
                        Log[1 + eta])\))\))\))\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0hand[eta_, xi_] = 
      eta^2 \(\( Sqrt[1 + xi]/4\)/
            Sqrt[xi]\)/\((1 + 
                xi\ eta^2)\)^2\ \((\(-\((1 + xi)\)\) \((1 - 
                  xi\ eta^2)\)\ 2\ ArcTan[Sqrt[xi]] + 
            2\ Sqrt[xi]\ \((\(-\((1 + xi\ eta^2)\)\) + \((1 + 
                        xi)\)\ eta\ \ Log[\((1 + eta)\)/\((\(-1\) + 
                            eta)\)])\))\)\)], "Input"],

Cell[BoxData[
    \(\(eta\^2\ \@\(1 + xi\)\ \((2\ \((\(-1\) - xi)\)\ \((1 - eta\^2\ xi)\)\ \
ArcTan[\@xi] + 2\ \@xi\ \((\(-1\) - eta\^2\ xi + eta\ \((1 + xi)\)\ Log[\(1 + \
eta\)\/\(\(-1\) + eta\)])\))\)\)\/\(4\ \@xi\ \((1 + eta\^2\ xi)\)\^2\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0hand[0.4 + I\ 10^\((\(-10\))\), 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.1478662103249458`\)\(\[InvisibleSpace]\)\) - 
      0.542553016812702`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.14786621025190982`\)\(\[InvisibleSpace]\)\) - 
      0.5425530169800589`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[0.4 - I\ \ 0.01, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.14058161073254719`\)\(\[InvisibleSpace]\)\) - 
      0.5596582079437287`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[1.1 - I\ \ 0.01, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.8476809789697757`\)\(\[InvisibleSpace]\)\) + 
      0.013835814424939618`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[1.1 - I\ 0.01, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.8476809789697757`\)\(\[InvisibleSpace]\)\) + 
      0.013835814424939618`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[eta_, 
        a_] = \(1\/\(4\ \@a\ \((1 + a\ eta\^2)\)\^2\)\) \((\@\(1 + a\)\ \
eta\^2\ \((\[ImaginaryI]\ \((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                    1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                      a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                    1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + 
                            a\ eta\^2)\) + \((1 + 
                            a)\)\ eta\ 2\ \((\(-Log[\((\(-1\) + eta)\)]\) + 
                            Log[\((1 + eta)\)] - \ 
                            2  Pi\ I\ If[\((Im[eta] < 0)\) && \((Re[eta] < 
                                      1)\), 1, 0])\))\))\))\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(1\/\(4\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\@\(1 + a\)\ eta\^2\ \((\
\[ImaginaryI]\ \((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                  a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + 
                        a\ eta\^2)\) + 
                  2\ \((1 + 
                        a)\)\ eta\ \((\(-2\)\ \[ImaginaryI]\ \[Pi]\ If[
                            Im[eta] < 0 && Re[eta] < 1, 1, 0] - 
                        Log[\(-1\) + eta] + 
                        Log[1 + eta])\))\))\))\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[0.4, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.1514798136715739`\) - 
      0.2113142895675798`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[0.4 + I\ 0.001, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.1501883246779093`\) - 
      0.2115200155819445`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[0.4 - I\ 0.001, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.15277510736585995`\) - 
      0.21110513691491803`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[0.9, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.5579727737854088`\)\(\[InvisibleSpace]\)\) - 
      0.9886333329011475`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[1.1, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.8439876429148118`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[0.4 + I\ 0.001, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.14859669805587325`\)\(\[InvisibleSpace]\)\) - 
      0.5408830743385498`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[0.4 - I\ 0.001, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.1471359892993161`\)\(\[InvisibleSpace]\)\) - 
      0.5442302230971223`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NIntegrate[\(\(eta^2\ \((1 + a)\)^\((3/2)\)/
                2 \( xi/\((1 + a\ xi^2)\)^2\)\ /\((eta - xi + I\ eps)\) /. 
            eps \[Rule] 10^\((\(-10\))\)\) /. a \[Rule] 10\) /. 
        eta \[Rule] 0.4, {xi, \(-1\), 0, 0.4, 1}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3999999992222327`\)."\)], "Message"],

Cell[BoxData[
    \(\(\(0.14786908419641076`\)\(\[InvisibleSpace]\)\) - 
      0.5425509355174426`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.14786621025190982`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(aha!\)\)], "Input"],

Cell[BoxData[
    \(also\ numerische\ Integration\ sollte\ schon\ \(\(stimmen\)\(.\)\)\)], \
"Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[eta], betakx0[eta, 10.0]}, {eta, 0.01, 5, 0.1}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, \(-0.0008925480791828562`\) + 
          0.`\ \[ImaginaryI]}, {0.11`, \(-0.07536912808618891`\) + 
          0.`\ \[ImaginaryI]}, {0.21000000000000002`, \(-0.10266287878236124`\
\) + 0.`\ \[ImaginaryI]}, {0.31`, \(\(0.0022201360252972775`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.41000000000000003`, \
\(\(0.164582444820044`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.51`, \(\(0.3244813229509469`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.61`, \(\(0.4640541408895969`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.7100000000000001`, \
\(\(0.5862252400114036`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.81`, \(\(0.7038588779647693`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.91`, \(\(0.8501640426474485`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.01`, \(\(1.189097973942366`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.11`, \(\(0.8352937277498539`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.21`, \(\(0.7511620344420324`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.31`, \(\(0.7063784693855385`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.4100000000000001`, \
\(\(0.6778137114170881`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.51`, \(\(0.6578809853632925`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.61`, \(\(0.6431780235733066`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.7100000000000002`, \
\(\(0.6319083332430742`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.81`, \(\(0.6230207339121872`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.9100000000000001`, \
\(\(0.6158547157972637`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.0100000000000002`, \
\(\(0.6099727857485227`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.1100000000000003`, \(\(0.605073111114841`\)\
\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.21`, \(\(0.6009405545621606`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.31`, \(\(0.5974176169914962`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.41`, \(\(0.594386384464894`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.5100000000000002`, \
\(\(0.5917568796289782`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.6100000000000003`, \
\(\(0.5894593031479328`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.71`, \(\(0.5874387242964629`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.81`, \(\(0.5856513616173571`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {2.91`, \(\(0.5840619236813213`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.0100000000000002`, \
\(\(0.5826416732323301`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.1100000000000003`, \
\(\(0.5813669951791443`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.21`, \(\(0.5802183219845349`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.31`, \(\(0.5791793167512993`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.41`, \(\(0.5782362448784841`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.5100000000000002`, \
\(\(0.5773774855630331`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.6100000000000003`, \
\(\(0.5765931482849225`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.7100000000000004`, \
\(\(0.5758747689898578`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.81`, \(\(0.5752150673985894`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {3.91`, \(\(0.574607751645905`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.01`, \(\(0.5740473598897455`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.11`, \(\(0.573529131035007`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.21`, \(\(0.5730488985607433`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.3100000000000005`, \(\(0.572603002811345`\)\
\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.41`, \(\(0.5721882181424632`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.510000000000001`, \(\(0.5718016920928581`\)\
\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.61`, \(\(0.5714408943494638`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.71`, \(\(0.5711035737318007`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.8100000000000005`, \
\(\(0.5707877217776433`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {4.91`, \(\(0.5704915417895955`\)\(\
\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[eta], 
        Sqrt[11] FastRebeta[1, 1, 1 - 10^\((\(-4\))\), eta, 10]}, {eta, 0.01, 
        5, 0.1}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, \(-0.0008900279566694939`\)}, {0.11`, \
\(-0.07532340527121177`\)}, {0.21000000000000002`, \
\(-0.10306751278967358`\)}, {0.31`, 
        0.0016043949817655873`}, {0.41000000000000003`, 
        0.16414673394378693`}, {0.51`, 0.32428877053274807`}, {0.61`, 
        0.46403183597947834`}, {0.7100000000000001`, 
        0.5862964651462478`}, {0.81`, 0.7039730498392454`}, {0.91`, 
        0.8502957054162483`}, {1.01`, 1.1892529899985504`}, {1.11`, 
        0.8353574362382619`}, {1.21`, 0.7511976144365515`}, {1.31`, 
        0.7063979043016689`}, {1.4100000000000001`, 
        0.6778224500469867`}, {1.51`, 0.6578820912110112`}, {1.61`, 
        0.6431734175673847`}, {1.7100000000000002`, 
        0.6318993058229485`}, {1.81`, 
        0.6230081950213616`}, {1.9100000000000001`, 
        0.6158393308656724`}, {2.0100000000000002`, 
        0.6099550555109899`}, {2.1100000000000003`, 
        0.6050534212137826`}, {2.21`, 0.6009192077808796`}, {2.31`, 
        0.59739485497018`}, {2.41`, 
        0.5943624027748672`}, {2.5100000000000002`, 
        0.5917318368327493`}, {2.6100000000000003`, 
        0.5894333335564328`}, {2.71`, 0.5874119406662196`}, {2.81`, 
        0.5856238557898409`}, {2.91`, 
        0.5840337751844873`}, {3.0100000000000002`, 
        0.5826129501300088`}, {3.1100000000000003`, 
        0.5813377560966415`}, {3.21`, 0.5801886177135459`}, {3.31`, 
        0.5791491915338971`}, {3.41`, 
        0.5782057374435123`}, {3.5100000000000002`, 
        0.5773466299694296`}, {3.6100000000000003`, 
        0.5765619746131492`}, {3.7100000000000004`, 
        0.5758433039127264`}, {3.81`, 0.5751833346555502`}, {3.91`, 
        0.5745757724395331`}, {4.01`, 0.5740151532189905`}, {4.11`, 
        0.5734967139768357`}, {4.21`, 
        0.5730162865092735`}, {4.3100000000000005`, 
        0.5725702096818687`}, {4.41`, 
        0.5721552565462777`}, {4.510000000000001`, 
        0.5717685734876512`}, {4.61`, 0.5714076291692466`}, {4.71`, 
        0.5710701714995261`}, {4.8100000000000005`, 
        0.5707541912031747`}, {4.91`, 0.5704578908552302`}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(diffs\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[eta], 
        Abs[Re[betakx0[eta, 10.0]] - 
            Sqrt[11] FastRebeta[1, 1, 1 - 10^\((\(-6\))\), eta, 10]]}, {eta, 
        0.01, 1.2, 0.05}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, 2.5326686570008305`*^-8}, {0.060000000000000005`, 
        6.17663064961943`*^-7}, {0.11`, 4.740962053634279`*^-7}, {0.16`, 
        1.4566106467106543`*^-6}, {0.21000000000000002`, 
        4.044419415802358`*^-6}, {0.26`, 5.794325266497702`*^-6}, {0.31`, 
        6.20931798815021`*^-6}, {0.36000000000000004`, 
        5.589977731840223`*^-6}, {0.41000000000000003`, 
        4.448199187212509`*^-6}, {0.46`, 3.1932032112380604`*^-6}, {0.51`, 
        2.0365770227459556`*^-6}, {0.56`, 1.0632389473852477`*^-6}, {0.61`, 
        3.2552953749265257`*^-7}, {0.66`, 
        2.265776577470291`*^-7}, {0.7100000000000001`, 
        6.200325030070175`*^-7}, {0.76`, 8.893767491358062`*^-7}, {0.81`, 
        1.0632113717212732`*^-6}, {0.8600000000000001`, 
        1.175702850519933`*^-6}, {0.91`, 
        1.266750427109109`*^-6}, {0.9600000000000001`, 
        1.395762329159389`*^-6}, {1.01`, 1.5501087649028022`*^-6}, {1.06`, 
        8.948236330796888`*^-7}, {1.11`, 
        6.370968036462088`*^-7}, {1.1600000000000001`, 
        4.734459710942218`*^-7}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.821018 0.0147151 94793.6 [
[.18801 .00222 -9 -9 ]
[.18801 .00222 9 0 ]
[.35222 .00222 -9 -9 ]
[.35222 .00222 9 0 ]
[.51642 .00222 -9 -9 ]
[.51642 .00222 9 0 ]
[.68062 .00222 -9 -9 ]
[.68062 .00222 9 0 ]
[.84483 .00222 -3 -9 ]
[.84483 .00222 3 0 ]
[.01131 .10951 -33 -6.25 ]
[.01131 .10951 0 6.25 ]
[.01131 .2043 -33 -6.25 ]
[.01131 .2043 0 6.25 ]
[.01131 .2991 -33 -6.25 ]
[.01131 .2991 0 6.25 ]
[.01131 .39389 -33 -6.25 ]
[.01131 .39389 0 6.25 ]
[.01131 .48868 -33 -6.25 ]
[.01131 .48868 0 6.25 ]
[.01131 .58348 -33 -6.25 ]
[.01131 .58348 0 6.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18801 .01472 m
.18801 .02097 L
s
[(0.2)] .18801 .00222 0 1 Mshowa
.35222 .01472 m
.35222 .02097 L
s
[(0.4)] .35222 .00222 0 1 Mshowa
.51642 .01472 m
.51642 .02097 L
s
[(0.6)] .51642 .00222 0 1 Mshowa
.68062 .01472 m
.68062 .02097 L
s
[(0.8)] .68062 .00222 0 1 Mshowa
.84483 .01472 m
.84483 .02097 L
s
[(1)] .84483 .00222 0 1 Mshowa
.125 Mabswid
.06486 .01472 m
.06486 .01847 L
s
.10591 .01472 m
.10591 .01847 L
s
.14696 .01472 m
.14696 .01847 L
s
.22906 .01472 m
.22906 .01847 L
s
.27011 .01472 m
.27011 .01847 L
s
.31117 .01472 m
.31117 .01847 L
s
.39327 .01472 m
.39327 .01847 L
s
.43432 .01472 m
.43432 .01847 L
s
.47537 .01472 m
.47537 .01847 L
s
.55747 .01472 m
.55747 .01847 L
s
.59852 .01472 m
.59852 .01847 L
s
.63957 .01472 m
.63957 .01847 L
s
.72167 .01472 m
.72167 .01847 L
s
.76273 .01472 m
.76273 .01847 L
s
.80378 .01472 m
.80378 .01847 L
s
.88588 .01472 m
.88588 .01847 L
s
.92693 .01472 m
.92693 .01847 L
s
.96798 .01472 m
.96798 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .10951 m
.03006 .10951 L
s
gsave
.01131 .10951 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.02381 .2043 m
.03006 .2043 L
s
gsave
.01131 .2043 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.02381 .2991 m
.03006 .2991 L
s
gsave
.01131 .2991 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(3) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.02381 .39389 m
.03006 .39389 L
s
gsave
.01131 .39389 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.02381 .48868 m
.03006 .48868 L
s
gsave
.01131 .48868 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.02381 .58348 m
.03006 .58348 L
s
gsave
.01131 .58348 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.125 Mabswid
.02381 .03367 m
.02756 .03367 L
s
.02381 .05263 m
.02756 .05263 L
s
.02381 .07159 m
.02756 .07159 L
s
.02381 .09055 m
.02756 .09055 L
s
.02381 .12847 m
.02756 .12847 L
s
.02381 .14743 m
.02756 .14743 L
s
.02381 .16638 m
.02756 .16638 L
s
.02381 .18534 m
.02756 .18534 L
s
.02381 .22326 m
.02756 .22326 L
s
.02381 .24222 m
.02756 .24222 L
s
.02381 .26118 m
.02756 .26118 L
s
.02381 .28014 m
.02756 .28014 L
s
.02381 .31805 m
.02756 .31805 L
s
.02381 .33701 m
.02756 .33701 L
s
.02381 .35597 m
.02756 .35597 L
s
.02381 .37493 m
.02756 .37493 L
s
.02381 .41285 m
.02756 .41285 L
s
.02381 .43181 m
.02756 .43181 L
s
.02381 .45077 m
.02756 .45077 L
s
.02381 .46972 m
.02756 .46972 L
s
.02381 .50764 m
.02756 .50764 L
s
.02381 .5266 m
.02756 .5266 L
s
.02381 .54556 m
.02756 .54556 L
s
.02381 .56452 m
.02756 .56452 L
s
.02381 .60244 m
.02756 .60244 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.03202 .01712 Mdot
.07307 .07327 Mdot
.11412 .05966 Mdot
.15517 .15279 Mdot
.19622 .3981 Mdot
.23727 .56398 Mdot
.27833 .60332 Mdot
.31938 .54461 Mdot
.36043 .43638 Mdot
.40148 .31741 Mdot
.44253 .20777 Mdot
.48358 .1155 Mdot
.52463 .04557 Mdot
.56568 .03619 Mdot
.60673 .07349 Mdot
.64778 .09902 Mdot
.68883 .1155 Mdot
.72989 .12616 Mdot
.77094 .13479 Mdot
.81199 .14702 Mdot
.85304 .16166 Mdot
.89409 .09954 Mdot
.93514 .07511 Mdot
.97619 .05959 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00Aooo0P001?oo00<00?ooool00_oo1000
6ooo0P001?oo00<00?ooool01?oo00<00?ooool06_oo0P001?oo00<00?ooool00ooo0P007?oo0P00
1?oo00<00?ooool00ooo0P008_oo10009ooo0016ool01000ooooo`002?oo00<00?ooool06ooo00@0
0?ooool000[oo`03003ooooo01Woo`04003ooooo0008ool01000ooooo`006_oo00@00?ooool000So
o`04003ooooo000Sool00`00ooooo`0Vool004Koo`04003ooooo0009ool00`00ooooo`0Jool01000
ooooo`001ooo1@006_oo00@00?ooool000Soo`04003ooooo000Jool01000ooooo`002?oo00@00?oo
ool002?oo`03003ooooo02Koo`00A_oo00@00?ooool000[oo`03003ooooo01Woo`04003ooooo0007
ool01000ooooo`006ooo00@00?ooool000Soo`<001_oo`04003ooooo0009ool2000Tool00`00oooo
o`0Vool004Koo`04003ooooo000;ool00`00ooooo`0Hool01000ooooo`002?oo00<00?oo00006ooo
00@00?ooool000Soo`03003ooooo01_oo`04003ooooo0008ool01000ooooo`008ooo00<00?ooool0
9_oo0016ool01000ooooo`002?oo00@00?ooool001[oo`04003ooooo0009ool2000Kool01000oooo
o`002?oo00<00?ooool06ooo00@00?ooool000Soo`04003ooooo000Qool3000Xool004Ooo`8000[o
o`8001coo`8000_oo`03003ooooo01[oo`8000[oo`<001_oo`8000[oo`8002Coo`03003ooooo02Ko
o`009?oo00<00?ooool0nOoo000Tool00`00ooooo`3iool002Coo`03003ooooo0?Woo`009?oo00<0
0?ooool0nOoo000Noooo0003000002Coo`03003oo`0000Ooo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00So
o`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00So
o`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Ooo`009?oo00<00?ooool0nOoo000Tool00`00ooooo`3iool002Coo`03003ooooo
0?Woo`009?oo0P00R?oo0P00L?oo000Tool00`00ooooo`28ool00`00ooooo`1^ool002Coo`03003o
oooo07goo`8007[oo`009?oo00<00?ooool0O_oo00<00?ooool0N?oo000Tool2003jool002Coo`03
003ooooo0?Woo`009?oo00<00?ooool04ooo0P00g?oo0P001_oo000Tool00`00ooooo`0Dool00`00
ooooo`3Kool00`00ooooo`04ool002Coo`03003ooooo0?Woo`009?oo0P002_oo0P00Qooo0P00IOoo
000Tool00`00ooooo`0:ool00`00ooooo`26ool00`00ooooo`1@ool2000Aool002Coo`03003ooooo
0>Ooo`03003ooooo00ooo`009?oo00<00?ooool0nOoo000Tool00`00ooooo`3iool002Coo`800?[o
o`009?oo00<00?ooool0nOoo00000ooo000000020002ool01@00oooooooo00000_oo10000ooo0P00
3Ooo00<00?ooool0WOoo0P00?Ooo0P006ooo0003ool00`00ooooo`02ool00`00ool00005ool01@00
oooooooo00000_oo00<00?ooool02_oo00<00?ooool0W_oo00<00?ooool0??oo00<00?ooool06Ooo
0003ool00`00ooooo`03ool00`00ooooo`04ool01@00oooooooo00000_oo00<00?ooool02_oo00<0
0?ooool0nOoo0003ool00`00ooooo`02ool00`00ool00005ool01@00oooooooo00000_oo00<00?oo
ool02_oo0P00n_oo0003ool01@00oooooooo00000ooo00<00?ooool00_oo00D00?ooooooo`0000;o
o`03003ooooo00[oo`03003ooooo07;oo`8003?oo`80053oo`0000Coo`00000000[oo`<000?oo`04
003ooooo0007ool20003ool00`00ooooo`1cool00`00ooooo`0bool00`00ooooo`1>ool000?oo`03
003ooooo00[oo`03003ooooo00;oo`8000Soo`03003oo`0000;oo`03003ooooo0?Woo`006Ooo1000
0_oo0`000_oo00<00?ooool0/_oo0P00AOoo000Oool00`00ooooo`02ool2002dool00`00ooooo`13
ool001ooo`03003ooooo00;oo`03003ooooo0;coo`8003_oo`008?oo0P000_oo00<00?ooool0_Ooo
00<00?ooool0>Ooo000Tool00`00ooooo`3iool002Coo`03003ooooo0<Ooo`80033oo`009?oo0P00
bOoo00<00?ooool0;_oo000Tool00`00ooooo`0Nool2003Iool002Coo`03003ooooo01ooo`03003o
oooo0=Ooo`009?oo00<00?ooool0dOoo0P009_oo000Tool00`00ooooo`3Bool00`00ooooo`0Tool0
02Coo`800?[oo`009?oo00<00?ooool0nOoo000Tool00`00ooooo`3iool002Coo`03003ooooo0?Wo
o`009?oo0P00n_oo000Tool00`00ooooo`3iool00003ool00000008000;oo`05003oooooool00002
ool40003ool2000=ool00`00ooooo`3iool00003ool00?oo00Goo`03003oo`0000Goo`05003ooooo
ool00002ool00`00ooooo`0:ool00`00ooooo`3iool000;oo`03003ooooo00Coo`03003ooooo00Co
o`05003oooooool00002ool00`00ooooo`0:ool00`00ooooo`3iool000?oo`03003ooooo00;oo`03
003oo`0000Goo`05003oooooool00002ool00`00ooooo`0:ool2003jool000Coo`04003ooooo0003
ool00`00ooooo`02ool01@00oooooooo00000_oo00<00?ooool02_oo00<00?ooool0J?oo0P00Sooo
00001Ooo003ooooo00002Ooo0`000ooo00@00?ooool000Ooo`8000?oo`03003ooooo06Woo`03003o
oooo08goo`000_oo0P003?oo00<00?ooool00_oo0P002?oo00<00?oo00000_oo00<00?ooool0nOoo
000Iool40002ool30002ool00`00ooooo`3iool001ooo`03003ooooo00;oo`800?[oo`007ooo00<0
0?ooool00_oo00<00?ooool0nOoo000Pool20002ool00`00ooooo`3iool002Coo`03003ooooo0?Wo
o`009?oo00<00?ooool0nOoo000Tool2003jool002Coo`03003ooooo0?Woo`009?oo00<00?ooool0
nOoo000Tool00`00ooooo`3iool002Coo`03003ooooo0?Woo`009?oo0P00n_oo000Tool00`00oooo
o`3iool002Coo`03003ooooo0?Woo`009?oo00<00?ooool0nOoo000Tool00`00ooooo`3iool002Co
o`800?[oo`000_oo0P000ooo00D00?ooooooo`0000;oo`@000?oo`8000goo`03003ooooo0?Woo`00
00Goo`00ooooo`0000?oo`03003oo`0000Goo`05003oooooool00002ool00`00ooooo`0:ool00`00
ooooo`3iool000Coo`03003ooooo00;oo`03003ooooo00Coo`05003oooooool00002ool00`00oooo
o`0:ool00`00ooooo`3iool000;oo`8000Coo`03003oo`0000Goo`05003oooooool00002ool00`00
ooooo`0:ool00`00ooooo`3iool000Coo`04003ooooo0003ool00`00ooooo`02ool01@00oooooooo
00000_oo00<00?ooool02_oo0P00n_oo00001Ooo003ooooo00002Ooo0`000ooo00@00?ooool000co
o`03003ooooo0?Woo`000_oo0P003?oo00<00?ooool00_oo0P002?oo0P000ooo00<00?ooool0nOoo
000Oool00`00ool00002ool00`00ooooo`3iool001Woo`@000;oo`<000;oo`03003ooooo05goo`80
09[oo`007ooo00<00?ooool00_oo0P00Gooo00<00?ooool0V?oo000Oool00`00ooooo`02ool00`00
ooooo`3iool0023oo`8000;oo`03003ooooo0?Woo`009?oo00<00?ooool0nOoo000Tool2003jool0
02Coo`03003ooooo0?Woo`009?oo00<00?ooool0nOoo000Tool00`00ooooo`3iool002Coo`03003o
oooo0?Woo`009?oo0P00n_oo000Tool00`00ooooo`3iool002Coo`03003ooooo0?Woo`009?oo00<0
0?ooool0nOoo000Tool00`00ooooo`3iool002Coo`800?[oo`009?oo00<00?ooool0nOoo0003ool0
1@00oooooooo00000ooo00<00?ooool010000ooo0P003Ooo00<00?ooool0nOoo0003ool00`00oooo
o`02ool00`00ool00005ool01@00oooooooo00000_oo00<00?ooool02_oo00<00?ooool0nOoo0005
0004ool00`00ooooo`04ool01@00oooooooo00000_oo00<00?ooool02_oo00<00?ooool0nOoo0000
1000ooooo`001?oo00<00?oo00001Ooo00D00?ooooooo`0000;oo`03003ooooo00[oo`800?[oo`00
00Coo`00ool000?oo`05003oooooool00004ool01@00oooooooo00000_oo00<00?ooool02_oo00<0
0?ooool0:Ooo0P00c_oo0002ool2000:ool30003ool01000ooooo`001ooo0P000ooo00<00?ooool0
:_oo00<00?ooool0c?oo0003ool00`00ooooo`0:ool00`00ooooo`02ool20008ool00`00ool00002
ool00`00ooooo`3iool001Woo`@000;oo`<000;oo`03003ooooo0?Woo`007ooo00<00?ooool00_oo
0P00n_oo000Oool00`00ooooo`02ool00`00ooooo`3iool0023oo`8000;oo`03003ooooo0?Woo`00
9?oo00<00?ooool0nOoo000Tool00`00ooooo`3iool002Coo`8005Coo`800:Coo`009?oo00<00?oo
ool0E?oo00<00?ooool0X_oo000Tool00`00ooooo`3iool002Coo`03003ooooo0?Woo`009?oo00<0
0?ooool0nOoo000Tool2003jool002Coo`03003ooooo0?Woo`009?oo00<00?ooool0nOoo000Tool0
0`00ooooo`3iool002Coo`03003ooooo0?Woo`009?oo0P00n_oo00001?oo000000000ooo00D00?oo
ooooo`0000;oo`@000?oo`8000goo`03003ooooo0?Woo`001?oo00L00?ooooooo`00ool00005ool0
1@00oooooooo00000_oo00<00?ooool02_oo00<00?ooool0nOoo0004ool00`00ooooo`02ool00`00
ooooo`04ool01@00oooooooo00000_oo00<00?ooool02_oo00<00?ooool0nOoo00001?oo00000000
1?oo00<00?oo00001Ooo00D00?ooooooo`0000;oo`03003ooooo00[oo`800?[oo`0000?oo`00ool0
1?oo00D00?ooooooo`0000Coo`05003oooooool00002ool00`00ooooo`0:ool00`00ooooo`3iool0
0003ool00?oo00_oo`<000?oo`04003ooooo0007ool20003ool00`00ooooo`3iool00003ool00000
008000_oo`03003ooooo00;oo`8000Soo`03003oo`0000;oo`03003ooooo0?Woo`006Ooo10000_oo
0`000_oo00<00?ooool0nOoo000Oool00`00ooooo`02ool2003jool001ooo`03003ooooo00;oo`03
003ooooo0?Woo`008?oo0P000_oo00<00?ooool0nOoo000Tool00`00ooooo`3iool002Coo`03003o
oooo0?Woo`009?oo0P00n_oo000Tool00`00ooooo`3iool002Coo`03003ooooo0?Woo`009?oo00<0
0?ooool0nOoo000Tool00`00ooooo`18ool2002_ool002Coo`8004[oo`03003ooooo0:goo`009?oo
00<00?ooool0nOoo000Tool00`00ooooo`3iool002Coo`03003ooooo0?Woo`009?oo00<00?ooool0
<ooo0P00a?oo000Tool2000eool00`00ooooo`32ool002Coo`03003ooooo0?Woo`000_oo0P000ooo
00D00?ooooooo`0000;oo`@000?oo`8000goo`03003ooooo0?Woo`0000Goo`00ooooo`0000?oo`03
003oo`0000Goo`05003oooooool00002ool00`00ooooo`0:ool00`00ooooo`3iool00005ool00?oo
ool00004ool00`00ooooo`04ool01@00oooooooo00000_oo00<00?ooool02_oo00<00?ooool0nOoo
00001?oo000000001?oo00<00?oo00001Ooo00D00?ooooooo`0000;oo`03003ooooo00[oo`800?[o
o`0000?oo`00ool01?oo00D00?ooooooo`0000Coo`05003oooooool00002ool00`00ooooo`0:ool0
0`00ooooo`3iool00003ool00?oo00_oo`<000?oo`04003ooooo0007ool20003ool00`00ooooo`3i
ool000;oo`<000_oo`03003ooooo00;oo`8000Soo`03003oo`0000;oo`03003ooooo0?Woo`006Ooo
10000_oo0`000_oo00<00?ooool0?_oo0P00^Ooo000Oool00`00ooooo`02ool20010ool00`00oooo
o`2gool001ooo`03003ooooo00;oo`03003ooooo0?Woo`008?oo0P000_oo00<00?ooool0nOoo000T
ool00`00ooooo`3iool002Coo`03003ooooo0?Woo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo
8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.171686, -7.01237*^-7, \
0.00474109, 4.10631*^-8}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(looks\ o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Sqrt[11] ImbetaA[1, 1, 1 - 10^\((\(-3\))\), eta, 10], {eta, 0.01, 
        5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 0.603319 0.993084 [
[.21429 .59082 -3 -9 ]
[.21429 .59082 3 0 ]
[.40476 .59082 -3 -9 ]
[.40476 .59082 3 0 ]
[.59524 .59082 -3 -9 ]
[.59524 .59082 3 0 ]
[.78571 .59082 -3 -9 ]
[.78571 .59082 3 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.01131 .00747 -24 -4.5 ]
[.01131 .00747 0 4.5 ]
[.01131 .10678 -24 -4.5 ]
[.01131 .10678 0 4.5 ]
[.01131 .20609 -24 -4.5 ]
[.01131 .20609 0 4.5 ]
[.01131 .30539 -24 -4.5 ]
[.01131 .30539 0 4.5 ]
[.01131 .4047 -24 -4.5 ]
[.01131 .4047 0 4.5 ]
[.01131 .50401 -24 -4.5 ]
[.01131 .50401 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60332 m
.21429 .60957 L
s
[(1)] .21429 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(2)] .40476 .59082 0 1 Mshowa
.59524 .60332 m
.59524 .60957 L
s
[(3)] .59524 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(4)] .78571 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(5)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.0619 .60332 m
.0619 .60707 L
s
.1 .60332 m
.1 .60707 L
s
.1381 .60332 m
.1381 .60707 L
s
.17619 .60332 m
.17619 .60707 L
s
.25238 .60332 m
.25238 .60707 L
s
.29048 .60332 m
.29048 .60707 L
s
.32857 .60332 m
.32857 .60707 L
s
.36667 .60332 m
.36667 .60707 L
s
.44286 .60332 m
.44286 .60707 L
s
.48095 .60332 m
.48095 .60707 L
s
.51905 .60332 m
.51905 .60707 L
s
.55714 .60332 m
.55714 .60707 L
s
.63333 .60332 m
.63333 .60707 L
s
.67143 .60332 m
.67143 .60707 L
s
.70952 .60332 m
.70952 .60707 L
s
.74762 .60332 m
.74762 .60707 L
s
.82381 .60332 m
.82381 .60707 L
s
.8619 .60332 m
.8619 .60707 L
s
.9 .60332 m
.9 .60707 L
s
.9381 .60332 m
.9381 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .00747 m
.03006 .00747 L
s
[(-0.6)] .01131 .00747 1 0 Mshowa
.02381 .10678 m
.03006 .10678 L
s
[(-0.5)] .01131 .10678 1 0 Mshowa
.02381 .20609 m
.03006 .20609 L
s
[(-0.4)] .01131 .20609 1 0 Mshowa
.02381 .30539 m
.03006 .30539 L
s
[(-0.3)] .01131 .30539 1 0 Mshowa
.02381 .4047 m
.03006 .4047 L
s
[(-0.2)] .01131 .4047 1 0 Mshowa
.02381 .50401 m
.03006 .50401 L
s
[(-0.1)] .01131 .50401 1 0 Mshowa
.125 Mabswid
.02381 .02733 m
.02756 .02733 L
s
.02381 .04719 m
.02756 .04719 L
s
.02381 .06705 m
.02756 .06705 L
s
.02381 .08692 m
.02756 .08692 L
s
.02381 .12664 m
.02756 .12664 L
s
.02381 .1465 m
.02756 .1465 L
s
.02381 .16636 m
.02756 .16636 L
s
.02381 .18622 m
.02756 .18622 L
s
.02381 .22595 m
.02756 .22595 L
s
.02381 .24581 m
.02756 .24581 L
s
.02381 .26567 m
.02756 .26567 L
s
.02381 .28553 m
.02756 .28553 L
s
.02381 .32526 m
.02756 .32526 L
s
.02381 .34512 m
.02756 .34512 L
s
.02381 .36498 m
.02756 .36498 L
s
.02381 .38484 m
.02756 .38484 L
s
.02381 .42456 m
.02756 .42456 L
s
.02381 .44443 m
.02756 .44443 L
s
.02381 .46429 m
.02756 .46429 L
s
.02381 .48415 m
.02756 .48415 L
s
.02381 .52387 m
.02756 .52387 L
s
.02381 .54373 m
.02756 .54373 L
s
.02381 .5636 m
.02756 .5636 L
s
.02381 .58346 m
.02756 .58346 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02571 .60327 m
.06427 .34773 L
.07397 .24251 L
.08434 .15038 L
.0903 .1095 L
.09572 .07985 L
.10117 .05672 L
.10632 .04033 L
.1111 .02929 L
.11351 .02506 L
.11617 .02135 L
.11843 .0189 L
.11968 .01781 L
.12086 .01694 L
.12196 .01627 L
.12316 .01569 L
.12425 .01528 L
.12526 .015 L
.12655 .01479 L
.12728 .01473 L
.12795 .01472 L
.12918 .01479 L
.13048 .015 L
.13164 .01529 L
.13273 .01565 L
.13521 .01678 L
.13789 .01842 L
.14033 .02027 L
.14582 .02547 L
.16486 .05098 L
.18558 .08473 L
.19576 .10179 L
.20532 .11761 L
.21011 .12542 L
.21143 .12754 L
.21281 .12976 L
.21399 .13165 L
.21528 .60332 L
.21602 .60332 L
.21673 .60332 L
.21804 .60332 L
.22098 .60332 L
.22628 .60332 L
.26467 .60332 L
.30551 .60332 L
.34483 .60332 L
.38264 .60332 L
.4229 .60332 L
.46165 .60332 L
.50284 .60332 L
Mistroke
.54252 .60332 L
.58069 .60332 L
.62131 .60332 L
.66041 .60332 L
.70196 .60332 L
.742 .60332 L
.78053 .60332 L
.8215 .60332 L
.86096 .60332 L
.89891 .60332 L
.93931 .60332 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0008ool20004ool00`00oooo
o`03ool20005ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<0
0?ooool0oooo0ooo0007ool01000ooooo`002?oo00@00?ooool000Coo`800?ooo`Coo`0000?oo`00
00000P000_oo00@00?ooool000Soo`<000Goo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000So
o`03003ooooo00Goo`03003ooooo01Ooo`@00>Ooo`001ooo00@00?ooool000Soo`03003ooooo00Go
o`03003ooooo01Koo`8000;oo`<00>Goo`002?oo0P002_oo0`001?oo00<00?ooool05Ooo00<00?oo
ool01?oo00<00?ooool0hooo000Kool2000Fool00`00ooooo`05ool00`00ooooo`3Rool001_oo`03
003ooooo01Coo`03003ooooo00Ooo`03003ooooo0>7oo`006ooo00<00?ooool05?oo00<00?ooool0
1ooo00<00?ooool0hOoo000Kool00`00ooooo`0Cool00`00ooooo`09ool00`00ooooo`3Pool001_o
o`03003ooooo01?oo`03003ooooo00[oo`03003ooooo0=ooo`006ooo00<00?ooool04ooo00<00?oo
ool02ooo00<00?ooool0g_oo000Kool2000Cool00`00ooooo`0<ool00`00ooooo`3Nool001_oo`03
003ooooo01;oo`03003ooooo00goo`03003ooooo0=goo`006ooo00<00?ooool04Ooo00<00?ooool0
3ooo00<00?ooool0g?oo000Kool00`00ooooo`0Aool00`00ooooo`0?ool00`00ooooo`3Lool001_o
o`03003ooooo017oo`03003ooooo013oo`03003ooooo0=_oo`006ooo0P004_oo00<00?ooool04Ooo
00<00?ooool0f_oo000Kool00`00ooooo`0@ool00`00ooooo`0Bool00`00ooooo`3Jool001_oo`03
003ooooo013oo`03003ooooo01?oo`03003ooooo0=Woo`006ooo00<00?ooool04?oo00<00?ooool0
5?oo00<00?ooool0f?oo000Kool00`00ooooo`0@ool00`00ooooo`0Dool00`00ooooo`3Hool001_o
o`80017oo`03003ooooo01Goo`03003ooooo0=Ooo`006ooo00<00?ooool04?oo00<00?ooool05_oo
00<00?ooool0e_oo000Kool00`00ooooo`0@ool00`00ooooo`0Fool00`00ooooo`3Fool000Soo`80
00Coo`03003ooooo00;oo`<000Goo`03003ooooo00ooo`03003ooooo01Soo`03003ooooo0=Goo`00
1ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo00ooo`03003ooooo01Woo`03003ooooo
0=Coo`001ooo00@00?ooool000_oo`03003ooooo00;oo`80013oo`03003ooooo01Woo`03003ooooo
0=Coo`0000?oo`0000000P000_oo00@00?ooool000Soo`<000Goo`03003ooooo00ooo`03003ooooo
01[oo`03003ooooo0=?oo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo00ooo`03
003ooooo01[oo`03003ooooo0=?oo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo
00ooo`03003ooooo01_oo`03003ooooo0=;oo`002?oo0P002Ooo10001?oo00<00?ooool03_oo00<0
0?ooool07?oo00<00?ooool0d_oo000Kool00`00ooooo`0>ool00`00ooooo`0Mool00`00ooooo`3A
ool001_oo`8000ooo`03003ooooo01koo`03003ooooo0=3oo`006ooo00<00?ooool03_oo00<00?oo
ool07_oo00<00?ooool0d?oo000Kool00`00ooooo`0>ool00`00ooooo`0Nool00`00ooooo`3@ool0
01_oo`03003ooooo00koo`03003ooooo01koo`03003ooooo0=3oo`006ooo00<00?ooool03Ooo00<0
0?ooool07ooo00<00?ooool0d?oo000Kool2000>ool00`00ooooo`0Oool00`00ooooo`3@ool001_o
o`03003ooooo00goo`03003ooooo01ooo`03003ooooo0=3oo`006ooo00<00?ooool03Ooo00<00?oo
ool07ooo00<00?ooool0d?oo000Kool00`00ooooo`0=ool00`00ooooo`0Oool00`00ooooo`3@ool0
01_oo`03003ooooo00goo`03003ooooo01ooo`03003ooooo0=3oo`006ooo0P003_oo00<00?ooool0
7ooo00<00?ooool0d?oo000Kool00`00ooooo`0<ool00`00ooooo`0Pool00`00ooooo`3@ool001_o
o`03003ooooo00coo`03003ooooo023oo`03003ooooo0=3oo`006ooo00<00?ooool03?oo00<00?oo
ool08?oo00<00?ooool0d?oo000Kool00`00ooooo`0<ool00`00ooooo`0Pool00`00ooooo`3@ool0
01_oo`03003ooooo00coo`03003ooooo023oo`03003ooooo0=3oo`006ooo0P003Ooo00<00?ooool0
8?oo00<00?ooool0d?oo000Kool00`00ooooo`0<ool00`00ooooo`0Pool00`00ooooo`3@ool001_o
o`03003ooooo00coo`03003ooooo023oo`03003ooooo0=3oo`002?oo0P001?oo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool02ooo00<00?ooool08Ooo00<00?ooool0d?oo0007ool01000oooo
o`002_oo00<00?ooool00ooo00<00?ooool02ooo00<00?ooool08Ooo00<00?ooool0d?oo0007ool0
1000ooooo`001ooo1@001?oo0P003?oo00<00?ooool08Ooo00<00?ooool0d?oo00000ooo00000002
0002ool01000ooooo`001ooo00@00?ooool000Goo`03003ooooo00_oo`03003ooooo027oo`03003o
oooo0=3oo`001ooo00@00?ooool000Soo`03003oo`0000Goo`03003ooooo00_oo`03003ooooo027o
o`03003ooooo0=3oo`001ooo00@00?ooool000Woo`8000Goo`03003ooooo00_oo`03003ooooo027o
o`03003ooooo0=3oo`002?oo0P002ooo00<00?ooool00ooo00<00?ooool02ooo00<00?ooool08Ooo
00<00?ooool0d?oo000Kool2000<ool00`00ooooo`0Qool00`00ooooo`3@ool001_oo`03003ooooo
00[oo`03003ooooo02;oo`03003ooooo0=3oo`006ooo00<00?ooool02_oo00<00?ooool08_oo00<0
0?ooool0d?oo000Kool00`00ooooo`0:ool00`00ooooo`0Rool00`00ooooo`3@ool001_oo`03003o
oooo00[oo`03003ooooo02;oo`03003ooooo0=3oo`006ooo0P002ooo00<00?ooool08_oo00<00?oo
ool0d?oo000Kool00`00ooooo`0:ool00`00ooooo`0Rool00`00ooooo`3@ool001_oo`03003ooooo
00[oo`03003ooooo02;oo`03003ooooo0=3oo`006ooo00<00?ooool02_oo00<00?ooool08_oo00<0
0?ooool0d?oo000Kool00`00ooooo`0:ool00`00ooooo`0Rool00`00ooooo`3@ool001_oo`03003o
oooo00[oo`03003ooooo02;oo`03003ooooo0=3oo`006ooo0P002ooo00<00?ooool08_oo00<00?oo
ool0d?oo000Kool00`00ooooo`0:ool00`00ooooo`0Rool00`00ooooo`3@ool001_oo`03003ooooo
00Woo`03003ooooo02?oo`03003ooooo0=3oo`006ooo00<00?ooool02Ooo00<00?ooool08ooo00<0
0?ooool0d?oo000Kool00`00ooooo`09ool00`00ooooo`0Sool00`00ooooo`3@ool001_oo`8000[o
o`03003ooooo02?oo`03003ooooo0=3oo`006ooo00<00?ooool02Ooo00<00?ooool08ooo00<00?oo
ool0d?oo000Kool00`00ooooo`09ool00`00ooooo`0Sool00`00ooooo`3@ool000Soo`8000Coo`03
003ooooo00?oo`8000Goo`03003ooooo00Woo`03003ooooo02?oo`03003ooooo0=3oo`001ooo00@0
0?ooool000Soo`04003ooooo0004ool00`00ooooo`09ool00`00ooooo`0Sool00`00ooooo`3@ool0
00Ooo`04003ooooo000;ool00`00ooooo`02ool2000:ool00`00ooooo`0Sool00`00ooooo`3@ool0
0003ool00000008000;oo`04003ooooo0009ool20005ool00`00ooooo`09ool00`00ooooo`0Sool0
0`00ooooo`3@ool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`09ool00`00oooo
o`0Sool00`00ooooo`3@ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool02Ooo
00<00?ooool08ooo00<00?ooool0d?oo0008ool2000:ool20005ool00`00ooooo`09ool00`00oooo
o`0Sool00`00ooooo`3@ool001_oo`03003ooooo00Woo`03003ooooo02?oo`03003ooooo0=3oo`00
6ooo0P002Ooo00<00?ooool09?oo00<00?ooool0d?oo000Kool00`00ooooo`08ool00`00ooooo`0T
ool00`00ooooo`3@ool001_oo`03003ooooo00Soo`03003ooooo02Coo`03003ooooo0=3oo`006ooo
00<00?ooool02?oo00<00?ooool09?oo00<00?ooool0d?oo000Kool00`00ooooo`08ool00`00oooo
o`0Tool00`00ooooo`3@ool001_oo`8000Woo`03003ooooo02Coo`03003ooooo0=3oo`006ooo00<0
0?ooool02?oo00<00?ooool09?oo00<00?ooool0d?oo000Kool00`00ooooo`08ool00`00ooooo`0T
ool00`00ooooo`3@ool001_oo`03003ooooo00Soo`03003ooooo02Coo`03003ooooo0=3oo`006ooo
00<00?ooool02?oo00<00?ooool09?oo00<00?ooool0d?oo000Kool20008ool00`00ooooo`0Uool0
0`00ooooo`3@ool001_oo`03003ooooo00Ooo`03003ooooo02Goo`03003ooooo0=3oo`006ooo00<0
0?ooool01ooo00<00?ooool09_oo00<00?ooool0cooo000Kool00`00ooooo`07ool00`00ooooo`0V
ool00`00ooooo`3?ool001_oo`03003ooooo00Ooo`03003ooooo02Koo`03003ooooo0<ooo`006ooo
0P002?oo00<00?ooool09_oo00<00?ooool0cooo000Kool00`00ooooo`06ool00`00ooooo`0Wool0
0`00ooooo`3?ool001_oo`03003ooooo00Koo`03003ooooo02Ooo`03003ooooo0<ooo`006ooo00<0
0?ooool01_oo00<00?ooool09ooo00<00?ooool0cooo0008ool20004ool00`00ooooo`02ool40004
ool00`00ooooo`06ool00`00ooooo`0Wool00`00ooooo`3?ool000Ooo`04003ooooo0008ool00`00
ooooo`05ool00`00ooooo`06ool00`00ooooo`0Wool00`00ooooo`3?ool000Ooo`04003ooooo0009
ool00`00ooooo`04ool20007ool00`00ooooo`0Wool00`00ooooo`3?ool00003ool00000008000;o
o`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`0Xool00`00ooooo`3?
ool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`05ool00`00ooooo`0Xool00`00
ooooo`3?ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool01Ooo00<00?ooool0
:?oo00<00?ooool0cooo0008ool2000:ool20005ool00`00ooooo`05ool00`00ooooo`0Xool00`00
ooooo`3?ool001_oo`8000Koo`03003ooooo02Soo`03003ooooo0<ooo`006ooo00<00?ooool01Ooo
00<00?ooool0:?oo00<00?ooool0cooo000Kool00`00ooooo`04ool00`00ooooo`0Yool00`00oooo
o`3?ool001_oo`03003ooooo00Coo`03003ooooo02Woo`03003ooooo0<ooo`006ooo00<00?ooool0
1?oo00<00?ooool0:Ooo00<00?ooool0cooo000Kool20005ool00`00ooooo`0Yool00`00ooooo`3?
ool001_oo`03003ooooo00Coo`03003ooooo02Woo`03003ooooo0<ooo`006ooo00<00?ooool01?oo
00<00?ooool0:Ooo00<00?ooool0cooo000Kool00`00ooooo`03ool00`00ooooo`0Zool00`00oooo
o`3?ool001_oo`03003ooooo00?oo`03003ooooo02[oo`03003ooooo0<ooo`006ooo00<00?ooool0
0ooo00<00?ooool0:_oo00<00?ooool0cooo000Kool20004ool00`00ooooo`0Zool00`00ooooo`3?
ool001_oo`03003ooooo00?oo`03003ooooo02[oo`03003ooooo0<ooo`006ooo00<00?ooool00ooo
00<00?ooool0:_oo00<00?ooool0cooo000Kool00`00ooooo`03ool00`00ooooo`0Zool00`00oooo
o`3?ool001_oo`03003ooooo00;oo`03003ooooo02_oo`03003ooooo0<ooo`006ooo0P000ooo00<0
0?ooool0:ooo00<00?ooool0cooo000Kool00`00ooooo`02ool00`00ooooo`0[ool00`00ooooo`3?
ool001_oo`03003ooooo00;oo`03003ooooo02_oo`03003ooooo0<ooo`002?oo0P001?oo00<00?oo
ool00_oo10001?oo00<00?ooool00_oo00<00?ooool0:ooo00<00?ooool0cooo0007ool01000oooo
o`002_oo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0:ooo00<00?ooool0cooo0007ool0
1000ooooo`002_oo00<00?ooool00ooo0P000_oo00<00?ooool0;?oo00<00?ooool0cooo00000ooo
000000020002ool01000ooooo`002_oo00<00?ooool00ooo00D00?ooooooo`0002koo`03003ooooo
0<ooo`001ooo00@00?ooool000[oo`03003ooooo00?oo`05003oooooool0000^ool00`00ooooo`3?
ool000Ooo`04003ooooo0008ool30005ool01@00oooooooo0000;_oo00<00?ooool0cooo0008ool2
000;ool00`00ooooo`03ool01@00oooooooo0000;_oo00<00?ooool0cooo000Kool20002ool00`00
ooooo`0/ool00`00ooooo`3?ool001_oo`04003ooooo000_ool00`00ooooo`3?ool001_oo`04003o
oooo000_ool00`00ooooo`3?ool001_oo`04003ooooo000_ool00`00ooooo`3?ool001_oo`04003o
oooo000_ool00`00ooooo`3?ool001_oo`04003ooooo000_ool00`00ooooo`3?ool001_oo`800003
ool00?oo02koo`03003ooooo0<ooo`006ooo00<00?oo0000<?oo00<00?ooool0cooo000Kool00`00
ool0000`ool00`00ooooo`3?ool001_oo`03003oo`00033oo`03003ooooo0<ooo`006ooo00<00?oo
0000<?oo00<00?ooool0cooo000Kool3000]ool4000_ool4000`ool2000aool00`00ooooo`0^ool3
0007ool001_oo`03003oo`0002ooo`8002ooo`03003ooooo033oo`04003ooooo000`ool00`00oooo
o`0aool00`00ooooo`04ool001_oo`80033oo`80033oo`03003ooooo03;oo`03003ooooo02_oo`D0
03;oo`03003ooooo00Coo`006ooo0P00<?oo0P00<Ooo00<00?ooool0;ooo0P00;_oo00@00?ooool0
033oo`<000Ooo`006ooo0P00<?oo0P00<_oo00<00?ooool0<?oo00<00?ooool0;?oo00<00?oo0000
<?oo00<00?ooool01ooo000Kool2000^ool4000_ool01000ooooo`00;ooo00@00?ooool002ooo`80
033oo`03003ooooo00Ooo`006ooo0P00<?oo0P00<?oo0P00<Ooo0P00<Ooo00<00?ooool0;_oo1000
1_oo000Kool2000aool00`00ooooo`3?ool001_oo`03003ooooo033oo`03003ooooo0<ooo`006ooo
00<00?ooool0<?oo00<00?ooool0cooo000Kool00`00ooooo`0`ool00`00ooooo`3?ool001Coool0
00d000006ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<0
0?ooool01ooo00<00?ooool02?oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<0
0?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool01ooo00<0
0?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<0
0?ooool01ooo00<00?ooool02?oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<0
0?ooool01ooo00<00?ooool01Ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.539323, -0.636008, \
0.0197364, 0.0037855}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      eta^3\ \((1 + a)\)^\((3/2)\)/\((1 + a\ eta^2)\)^2\ \((\(-Pi\)/2)\) /. 
        a \[Rule] 10, {eta, 0.01, 5}, 
      PlotStyle \[Rule] {RGBColor[1, 0, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 0.603319 1.00012 [
[.21429 .59082 -3 -9 ]
[.21429 .59082 3 0 ]
[.40476 .59082 -3 -9 ]
[.40476 .59082 3 0 ]
[.59524 .59082 -3 -9 ]
[.59524 .59082 3 0 ]
[.78571 .59082 -3 -9 ]
[.78571 .59082 3 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.01131 .00325 -24 -4.5 ]
[.01131 .00325 0 4.5 ]
[.01131 .10326 -24 -4.5 ]
[.01131 .10326 0 4.5 ]
[.01131 .20327 -24 -4.5 ]
[.01131 .20327 0 4.5 ]
[.01131 .30328 -24 -4.5 ]
[.01131 .30328 0 4.5 ]
[.01131 .40329 -24 -4.5 ]
[.01131 .40329 0 4.5 ]
[.01131 .50331 -24 -4.5 ]
[.01131 .50331 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60332 m
.21429 .60957 L
s
[(1)] .21429 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(2)] .40476 .59082 0 1 Mshowa
.59524 .60332 m
.59524 .60957 L
s
[(3)] .59524 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(4)] .78571 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(5)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.0619 .60332 m
.0619 .60707 L
s
.1 .60332 m
.1 .60707 L
s
.1381 .60332 m
.1381 .60707 L
s
.17619 .60332 m
.17619 .60707 L
s
.25238 .60332 m
.25238 .60707 L
s
.29048 .60332 m
.29048 .60707 L
s
.32857 .60332 m
.32857 .60707 L
s
.36667 .60332 m
.36667 .60707 L
s
.44286 .60332 m
.44286 .60707 L
s
.48095 .60332 m
.48095 .60707 L
s
.51905 .60332 m
.51905 .60707 L
s
.55714 .60332 m
.55714 .60707 L
s
.63333 .60332 m
.63333 .60707 L
s
.67143 .60332 m
.67143 .60707 L
s
.70952 .60332 m
.70952 .60707 L
s
.74762 .60332 m
.74762 .60707 L
s
.82381 .60332 m
.82381 .60707 L
s
.8619 .60332 m
.8619 .60707 L
s
.9 .60332 m
.9 .60707 L
s
.9381 .60332 m
.9381 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .00325 m
.03006 .00325 L
s
[(-0.6)] .01131 .00325 1 0 Mshowa
.02381 .10326 m
.03006 .10326 L
s
[(-0.5)] .01131 .10326 1 0 Mshowa
.02381 .20327 m
.03006 .20327 L
s
[(-0.4)] .01131 .20327 1 0 Mshowa
.02381 .30328 m
.03006 .30328 L
s
[(-0.3)] .01131 .30328 1 0 Mshowa
.02381 .40329 m
.03006 .40329 L
s
[(-0.2)] .01131 .40329 1 0 Mshowa
.02381 .50331 m
.03006 .50331 L
s
[(-0.1)] .01131 .50331 1 0 Mshowa
.125 Mabswid
.02381 .02325 m
.02756 .02325 L
s
.02381 .04325 m
.02756 .04325 L
s
.02381 .06325 m
.02756 .06325 L
s
.02381 .08326 m
.02756 .08326 L
s
.02381 .12326 m
.02756 .12326 L
s
.02381 .14326 m
.02756 .14326 L
s
.02381 .16326 m
.02756 .16326 L
s
.02381 .18327 m
.02756 .18327 L
s
.02381 .22327 m
.02756 .22327 L
s
.02381 .24327 m
.02756 .24327 L
s
.02381 .26328 m
.02756 .26328 L
s
.02381 .28328 m
.02756 .28328 L
s
.02381 .32328 m
.02756 .32328 L
s
.02381 .34329 m
.02756 .34329 L
s
.02381 .36329 m
.02756 .36329 L
s
.02381 .38329 m
.02756 .38329 L
s
.02381 .4233 m
.02756 .4233 L
s
.02381 .4433 m
.02756 .4433 L
s
.02381 .4633 m
.02756 .4633 L
s
.02381 .4833 m
.02756 .4833 L
s
.02381 .52331 m
.02756 .52331 L
s
.02381 .54331 m
.02756 .54331 L
s
.02381 .56331 m
.02756 .56331 L
s
.02381 .58332 m
.02756 .58332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.02571 .60326 m
.06427 .34246 L
.07397 .23836 L
.08434 .14798 L
.0903 .108 L
.09572 .079 L
.10117 .05634 L
.10632 .04025 L
.1111 .02937 L
.11351 .02518 L
.11617 .02149 L
.11843 .01904 L
.11968 .01795 L
.12086 .01707 L
.12196 .01639 L
.12316 .01579 L
.12425 .01536 L
.12526 .01506 L
.12655 .01482 L
.12728 .01474 L
.12795 .01472 L
.12918 .01476 L
.13048 .01493 L
.13164 .01519 L
.13273 .01552 L
.13521 .01657 L
.13789 .01812 L
.14033 .01988 L
.14582 .02487 L
.15571 .03664 L
.16486 .04968 L
.18558 .08281 L
.22477 .14595 L
.26244 .199 L
.30257 .24585 L
.34118 .28284 L
.38223 .31524 L
.42178 .34115 L
.45981 .36222 L
.50029 .38135 L
.53926 .39719 L
.58067 .41178 L
.62058 .42405 L
.65896 .43449 L
.6998 .44436 L
.73913 .45284 L
.77694 .4602 L
.8172 .46729 L
.85594 .47349 L
.89714 .4795 L
Mistroke
.93682 .48478 L
.97499 .48946 L
.97619 .4896 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0008ool20004ool00`00oooo
o`03ool2003oool;ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0oooo0ooo
0007ool01000ooooo`002?oo00@00?ooool000Coo`800?ooo`Coo`0000?oo`0000000P000_oo00@0
0?ooool000Soo`<000Goo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Soo`03003ooooo00Go
o`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo01Koo`Kh
0>Koo`002?oo0P002_oo0`001?oo00<00?ooool05Ooo0_P01Ooo00?h0?ooool0hooo000Kool2000F
ool00oP0ooooo`05ool00oP0ooooo`3Rool001_oo`03003ooooo01Coo`03n03ooooo00Ooo`03n03o
oooo0>7oo`006ooo00<00?ooool05?oo00?h0?ooool02?oo00?h0?ooool0h?oo000Kool00`00oooo
o`0Cool00oP0ooooo`0:ool00oP0ooooo`3Oool001_oo`03003ooooo01?oo`03n03ooooo00[oo`03
n03ooooo0=ooo`006ooo0P005?oo00?h0?ooool02ooo00?h0?ooool0g_oo000Kool00`00ooooo`0B
ool00oP0ooooo`0<ool00oP0ooooo`3Nool001_oo`03003ooooo01;oo`03n03ooooo00goo`03n03o
oooo0=goo`006ooo00<00?ooool04Ooo00?h0?ooool03ooo00?h0?ooool0g?oo000Kool00`00oooo
o`0Aool00oP0ooooo`0?ool00oP0ooooo`3Lool001_oo`8001;oo`03n03ooooo013oo`03n03ooooo
0=_oo`006ooo00<00?ooool04Ooo00?h0?ooool04Ooo00?h0?ooool0f_oo000Kool00`00ooooo`0A
ool00oP0ooooo`0Aool00oP0ooooo`3Jool001_oo`03003ooooo013oo`03n03ooooo01?oo`03n03o
oooo0=Woo`006ooo00<00?ooool04?oo00?h0?ooool05?oo00?h0?ooool0f?oo000Kool00`00oooo
o`0@ool00oP0ooooo`0Dool00oP0ooooo`3Hool001_oo`80017oo`03n03ooooo01Goo`03n03ooooo
0=Ooo`006ooo00<00?ooool04?oo00?h0?ooool05_oo00?h0?ooool0e_oo000Kool00`00ooooo`0@
ool00oP0ooooo`0Fool00oP0ooooo`3Fool000Soo`8000Coo`03003ooooo00;oo`<000Goo`03003o
oooo00ooo`03n03ooooo01Soo`03n03ooooo0=Goo`001ooo00@00?ooool000_oo`03003ooooo00;o
o`03003ooooo00ooo`03n03ooooo01Soo`03n03ooooo0=Goo`001ooo00@00?ooool000_oo`03003o
oooo00;oo`80013oo`03n03ooooo01Woo`03n03ooooo0=Coo`0000?oo`0000000P000_oo00@00?oo
ool000Soo`<000Goo`03003ooooo00ooo`03n03ooooo01[oo`03n03ooooo0=?oo`001ooo00@00?oo
ool000Soo`03003ooooo00Goo`03003ooooo00ooo`03n03ooooo01[oo`03n03ooooo0=?oo`001ooo
00@00?ooool000Soo`03003ooooo00Goo`03003ooooo00ooo`03n03ooooo01_oo`03n03ooooo0=;o
o`002?oo0P002Ooo10001?oo00<00?ooool03_oo00?h0?ooool07Ooo00?h0?ooool0dOoo000Kool2
000?ool00oP0ooooo`0Mool00oP0ooooo`3Aool001_oo`03003ooooo00koo`03n03ooooo01koo`03
n03ooooo0=3oo`006ooo00<00?ooool03_oo00?h0?ooool07_oo00?h0?ooool0d?oo000Kool00`00
ooooo`0>ool00oP0ooooo`0Oool00oP0ooooo`3?ool001_oo`03003ooooo00koo`03n03ooooo023o
o`03n03ooooo0<koo`006ooo00<00?ooool03Ooo00?h0?ooool08Ooo00?h0?ooool0c_oo000Kool2
000>ool00oP0ooooo`0Rool00oP0ooooo`3=ool001_oo`03003ooooo00goo`03n03ooooo02?oo`03
n03ooooo0<coo`006ooo00<00?ooool03Ooo00?h0?ooool08ooo00?h0?ooool0c?oo000Kool00`00
ooooo`0=ool00oP0ooooo`0Tool00oP0ooooo`3;ool001_oo`03003ooooo00goo`03n03ooooo02Go
o`03n03ooooo0<[oo`006ooo0P003_oo00?h0?ooool09Ooo00?h0?ooool0b_oo000Kool00`00oooo
o`0<ool00oP0ooooo`0Wool00oP0ooooo`39ool001_oo`03003ooooo00coo`03n03ooooo02Soo`03
n03ooooo0<Soo`006ooo00<00?ooool03?oo00?h0?ooool0:?oo00?h0?ooool0b?oo000Kool00`00
ooooo`0<ool00oP0ooooo`0Yool00oP0ooooo`37ool001_oo`8000goo`03n03ooooo02[oo`03n03o
oooo0<Koo`006ooo00<00?ooool03?oo00?h0?ooool0:_oo00?h0?ooool0a_oo000Kool00`00oooo
o`0<ool00oP0ooooo`0[ool00oP0ooooo`35ool001_oo`03003ooooo00coo`03n03ooooo02coo`03
n03ooooo0<Coo`002?oo0P001?oo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool02ooo00?h
0?ooool0;Ooo00?h0?ooool0a?oo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool0
2ooo00?h0?ooool0;_oo00?h0?ooool0`ooo0007ool01000ooooo`001ooo1@001?oo0P003?oo00?h
0?ooool0;ooo00?h0?ooool0`_oo00000ooo000000020002ool01000ooooo`001ooo00@00?ooool0
00Goo`03003ooooo00_oo`03n03ooooo033oo`03n03ooooo0<7oo`001ooo00@00?ooool000Soo`03
003oo`0000Goo`03003ooooo00_oo`03n03ooooo037oo`03n03ooooo0<3oo`001ooo00@00?ooool0
00Woo`8000Goo`03003ooooo00_oo`03n03ooooo03;oo`03n03ooooo0;ooo`002?oo0P002ooo00<0
0?ooool00ooo00<00?ooool02ooo00?h0?ooool0<ooo00?h0?ooool0__oo000Kool2000<ool00oP0
ooooo`0cool00oP0ooooo`2nool001_oo`03003ooooo00[oo`03n03ooooo03Goo`03n03ooooo0;go
o`006ooo00<00?ooool02_oo00?h0?ooool0=_oo00?h0?ooool0_?oo000Kool00`00ooooo`0:ool0
0oP0ooooo`0gool00oP0ooooo`2kool001_oo`03003ooooo00[oo`03n03ooooo03Soo`03n03ooooo
0;[oo`006ooo0P002ooo00?h0?ooool0>Ooo00?h0?ooool0^Ooo000Kool00`00ooooo`0:ool00oP0
ooooo`0jool00oP0ooooo`2hool001_oo`03003ooooo00[oo`03n03ooooo03_oo`03n03ooooo0;Oo
o`006ooo00<00?ooool02_oo00?h0?ooool0??oo00?h0?ooool0]_oo000Kool00`00ooooo`0:ool0
0oP0ooooo`0mool00oP0ooooo`2eool001_oo`03003ooooo00[oo`03n03ooooo03koo`03n03ooooo
0;Coo`006ooo0P002ooo00?h0?ooool0?ooo00?h0?ooool0/ooo000Kool00`00ooooo`09ool00oP0
ooooo`11ool00oP0ooooo`2bool001_oo`03003ooooo00Woo`03n03ooooo04;oo`03n03ooooo0;7o
o`006ooo00<00?ooool02Ooo00?h0?ooool0@ooo00?h0?ooool0/?oo000Kool00`00ooooo`09ool0
0oP0ooooo`14ool00oP0ooooo`2_ool001_oo`8000[oo`03n03ooooo04Goo`03n03ooooo0:koo`00
6ooo00<00?ooool02Ooo00?h0?ooool0A_oo00?h0?ooool0[Ooo000Kool00`00ooooo`09ool00oP0
ooooo`17ool2n02]ool000Soo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo00Woo`03n03o
oooo04Woo`03n03ooooo0:[oo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`09
ool00oP0ooooo`1:ool00oP0ooooo`2Yool000Ooo`04003ooooo000;ool00`00ooooo`02ool2000:
ool00oP0ooooo`1;ool00oP0ooooo`2Xool00003ool00000008000;oo`04003ooooo0009ool20005
ool00`00ooooo`09ool00oP0ooooo`1<ool00oP0ooooo`2Wool000Ooo`04003ooooo000;ool00`00
ooooo`02ool00`00ooooo`09ool00oP0ooooo`1=ool2n02Wool000Ooo`04003ooooo0008ool01000
ooooo`001?oo00<00?ooool02Ooo00?h0?ooool0Cooo00?h0?ooool0Y?oo0008ool2000:ool20005
ool00`00ooooo`09ool00oP0ooooo`1@ool00oP0ooooo`2Sool001_oo`03003ooooo00Soo`03n03o
oooo05;oo`;h0:?oo`006ooo0P002Ooo00?h0?ooool0E?oo0_P0XOoo000Kool00`00ooooo`08ool0
0oP0ooooo`1Fool2n02Oool001_oo`03003ooooo00Soo`03n03ooooo05Soo`;h09goo`006ooo00<0
0?ooool02?oo00?h0?ooool0F_oo0_P0Vooo000Kool00`00ooooo`08ool00oP0ooooo`1Lool00oP0
ooooo`2Hool001_oo`8000Woo`03n03ooooo05goo`;h09Soo`006ooo00<00?ooool02?oo00?h0?oo
ool0Gooo0_P0U_oo000Kool00`00ooooo`08ool00oP0ooooo`1Qool00oP0ooooo`2Cool001_oo`03
003ooooo00Soo`03n03ooooo06;oo`;h09?oo`006ooo00<00?ooool01ooo00?h0?ooool0IOoo0_P0
TOoo000Kool20008ool00oP0ooooo`1Wool2n02?ool001_oo`03003ooooo00Ooo`03n03ooooo06Wo
o`;h08goo`006ooo00<00?ooool01ooo00?h0?ooool0Jooo0_P0Rooo000Kool00`00ooooo`07ool0
0oP0ooooo`1]ool2n029ool001_oo`03003ooooo00Ooo`03n03ooooo06ooo`;h08Ooo`006ooo00<0
0?ooool01_oo00?h0?ooool0L_oo0oP0Q?oo000Kool20007ool00oP0ooooo`1eool2n022ool001_o
o`03003ooooo00Koo`03n03ooooo07Ooo`?h07ooo`006ooo00<00?ooool01_oo00?h0?ooool0N_oo
0_P0OOoo0008ool20004ool00`00ooooo`02ool40004ool00`00ooooo`06ool00oP0ooooo`1lool3
n01jool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`06ool00oP0ooooo`1oool3
n01gool000Ooo`04003ooooo0009ool00`00ooooo`04ool20006ool00oP0ooooo`23ool2n01eool0
0003ool00000008000;oo`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`05ool00oP0oooo
o`25ool3n01bool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`05ool00oP0oooo
o`28ool3n01_ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool01Ooo00?h0?oo
ool0Rooo1?P0Jooo0008ool2000:ool20005ool00`00ooooo`05ool00oP0ooooo`2?ool4n01Wool0
01_oo`8000Koo`03n03ooooo09?oo`?h06Coo`006ooo00<00?ooool01?oo00?h0?ooool0Uooo0oP0
HOoo000Kool00`00ooooo`04ool00oP0ooooo`2Jool4n01Mool001_oo`03003ooooo00Coo`03n03o
oooo09koo`?h05[oo`006ooo00<00?ooool01?oo00?h0?ooool0XOoo1?P0E_oo000Kool00`00oooo
o`04ool00oP0ooooo`2Uool4n01Bool001_oo`8000Goo`03n03ooooo0:Woo`Ch04koo`006ooo00<0
0?ooool01?oo00?h0?ooool0[Ooo1OP0BOoo000Kool00`00ooooo`03ool00oP0ooooo`2cool5n014
ool001_oo`03003ooooo00?oo`03n03ooooo0;Soo`Gh03ooo`006ooo00<00?ooool00ooo00?h0?oo
ool0_Ooo1OP0>_oo000Kool20004ool00oP0ooooo`32ool6n00dool001_oo`03003ooooo00?oo`03
n03ooooo0<Soo`Gh02ooo`006ooo00<00?ooool00ooo00?h0?ooool0cOoo1OP0:_oo000Kool00`00
ooooo`02ool00oP0ooooo`3Cool8n00Rool001_oo`03003ooooo00;oo`03n03ooooo0=_oo`Sh01[o
o`006ooo0P000ooo00?h0?ooool0hooo1_P05?oo000Kool00`00ooooo`02ool00oP0ooooo`3Yool7
n00=ool001_oo`03003ooooo00;oo`03n03ooooo0?3oo`Kh00Ooo`006ooo00<00?ooool00_oo00?h
0?ooool0oOoo0008ool20004ool00`00ooooo`02ool40004ool01@00oooooooon000oooo0Ooo0007
ool01000ooooo`002_oo00<00?ooool00ooo00D00?ooooooooP00?ooo`7oo`001ooo00@00?ooool0
00[oo`03003ooooo00?oo`8000;oo`03n03ooooo0?koo`0000?oo`0000000P000_oo00@00?ooool0
00[oo`03003ooooo00?oo`05003ooooooooh003oool1ool000Ooo`04003ooooo000:ool00`00oooo
o`03ool01@00oooooooon000oooo0Ooo0007ool01000ooooo`002?oo0`001Ooo00D00?ooooooooP0
0?ooo`7oo`002?oo0P002ooo00<00?ooool00ooo00D00?ooooooooP00?ooo`7oo`006ooo0P0000?o
ooP0ool0oooo0Ooo000Kool01000ooooooP0oooo0_oo000Kool01000ooooooP0oooo0_oo000Kool0
1000ooooooP0oooo0_oo000Kool01000ooooooP0oooo0_oo000Kool200000ooon03oo`3oool1ool0
01_oo`03003oooP00?ooo`?oo`006ooo00<00?oon000oooo0ooo000Kool00`00oooh003oool3ool0
01_oo`03003oooP00?ooo`?oo`006ooo0P0000?h0?ooool0oooo0Ooo000Kool00`00oooh000]ool4
000_ool4000`ool2000aool00`00ooooo`0^ool30007ool001_oo`03003h0?oo02ooo`03003ooooo
02koo`03003ooooo033oo`04003ooooo000`ool00`00ooooo`0aool00`00ooooo`04ool001_oo`03
003h0?oo02ooo`03003ooooo02ooo`03003ooooo03;oo`03003ooooo02_oo`D003;oo`03003ooooo
00Coo`006ooo00<00?P0ool0;ooo00<00?ooool0<?oo00<00?ooool0;ooo0P00;_oo00@00?ooool0
033oo`<000Ooo`006ooo00<00?P0ool0;ooo00<00?ooool0<Ooo00<00?ooool0<?oo00<00?ooool0
;?oo00<00?oo0000<?oo00<00?ooool01ooo000Kool00`00n03oo`0]ool3000`ool01000ooooo`00
;ooo00@00?ooool002ooo`80033oo`03003ooooo00Ooo`006ooo00<00?P0ool0;ooo00<00?ooool0
;ooo0P00<Ooo0P00<Ooo00<00?ooool0;_oo10001_oo000Kool00oP0ooooo`3oool3ool001_oo`03
n03ooooo0?ooo`?oo`006ooo00?h0?ooool0oooo0ooo000Dool700000oP00000003o0003000001_o
o`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Oo
o`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Oo
o`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Oo
o`03003ooooo00Goo`006ooo00<00?ooool0;ooo00<00?ooool0<?oo00<00?ooool0<?oo00<00?oo
ool0;ooo00<00?ooool0<?oo00<00?ooool01Ooo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0?ooo`?oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.539323, -0.633643, \
0.0197364, 0.00375886}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %2878]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 0.603319 0.993084 [
[.21429 .59082 -3 -9 ]
[.21429 .59082 3 0 ]
[.40476 .59082 -3 -9 ]
[.40476 .59082 3 0 ]
[.59524 .59082 -3 -9 ]
[.59524 .59082 3 0 ]
[.78571 .59082 -3 -9 ]
[.78571 .59082 3 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.01131 .00747 -24 -4.5 ]
[.01131 .00747 0 4.5 ]
[.01131 .10678 -24 -4.5 ]
[.01131 .10678 0 4.5 ]
[.01131 .20609 -24 -4.5 ]
[.01131 .20609 0 4.5 ]
[.01131 .30539 -24 -4.5 ]
[.01131 .30539 0 4.5 ]
[.01131 .4047 -24 -4.5 ]
[.01131 .4047 0 4.5 ]
[.01131 .50401 -24 -4.5 ]
[.01131 .50401 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60332 m
.21429 .60957 L
s
[(1)] .21429 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(2)] .40476 .59082 0 1 Mshowa
.59524 .60332 m
.59524 .60957 L
s
[(3)] .59524 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(4)] .78571 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(5)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.0619 .60332 m
.0619 .60707 L
s
.1 .60332 m
.1 .60707 L
s
.1381 .60332 m
.1381 .60707 L
s
.17619 .60332 m
.17619 .60707 L
s
.25238 .60332 m
.25238 .60707 L
s
.29048 .60332 m
.29048 .60707 L
s
.32857 .60332 m
.32857 .60707 L
s
.36667 .60332 m
.36667 .60707 L
s
.44286 .60332 m
.44286 .60707 L
s
.48095 .60332 m
.48095 .60707 L
s
.51905 .60332 m
.51905 .60707 L
s
.55714 .60332 m
.55714 .60707 L
s
.63333 .60332 m
.63333 .60707 L
s
.67143 .60332 m
.67143 .60707 L
s
.70952 .60332 m
.70952 .60707 L
s
.74762 .60332 m
.74762 .60707 L
s
.82381 .60332 m
.82381 .60707 L
s
.8619 .60332 m
.8619 .60707 L
s
.9 .60332 m
.9 .60707 L
s
.9381 .60332 m
.9381 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .00747 m
.03006 .00747 L
s
[(-0.6)] .01131 .00747 1 0 Mshowa
.02381 .10678 m
.03006 .10678 L
s
[(-0.5)] .01131 .10678 1 0 Mshowa
.02381 .20609 m
.03006 .20609 L
s
[(-0.4)] .01131 .20609 1 0 Mshowa
.02381 .30539 m
.03006 .30539 L
s
[(-0.3)] .01131 .30539 1 0 Mshowa
.02381 .4047 m
.03006 .4047 L
s
[(-0.2)] .01131 .4047 1 0 Mshowa
.02381 .50401 m
.03006 .50401 L
s
[(-0.1)] .01131 .50401 1 0 Mshowa
.125 Mabswid
.02381 .02733 m
.02756 .02733 L
s
.02381 .04719 m
.02756 .04719 L
s
.02381 .06705 m
.02756 .06705 L
s
.02381 .08692 m
.02756 .08692 L
s
.02381 .12664 m
.02756 .12664 L
s
.02381 .1465 m
.02756 .1465 L
s
.02381 .16636 m
.02756 .16636 L
s
.02381 .18622 m
.02756 .18622 L
s
.02381 .22595 m
.02756 .22595 L
s
.02381 .24581 m
.02756 .24581 L
s
.02381 .26567 m
.02756 .26567 L
s
.02381 .28553 m
.02756 .28553 L
s
.02381 .32526 m
.02756 .32526 L
s
.02381 .34512 m
.02756 .34512 L
s
.02381 .36498 m
.02756 .36498 L
s
.02381 .38484 m
.02756 .38484 L
s
.02381 .42456 m
.02756 .42456 L
s
.02381 .44443 m
.02756 .44443 L
s
.02381 .46429 m
.02756 .46429 L
s
.02381 .48415 m
.02756 .48415 L
s
.02381 .52387 m
.02756 .52387 L
s
.02381 .54373 m
.02756 .54373 L
s
.02381 .5636 m
.02756 .5636 L
s
.02381 .58346 m
.02756 .58346 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.02571 .60326 m
.06427 .34429 L
.07397 .24093 L
.08434 .15119 L
.0903 .11149 L
.09572 .08269 L
.10117 .06019 L
.10632 .04421 L
.1111 .03341 L
.11351 .02925 L
.11617 .02558 L
.11843 .02316 L
.11968 .02207 L
.12086 .0212 L
.12196 .02052 L
.12316 .01992 L
.12425 .0195 L
.12526 .0192 L
.12655 .01896 L
.12728 .01889 L
.12795 .01886 L
.12918 .0189 L
.13048 .01907 L
.13164 .01933 L
.13273 .01966 L
.13521 .0207 L
.13789 .02224 L
.14033 .02399 L
.14582 .02894 L
.15571 .04063 L
.16486 .05358 L
.18558 .08647 L
.22477 .14917 L
.26244 .20185 L
.30257 .24836 L
.34118 .28509 L
.38223 .31727 L
.42178 .343 L
.45981 .36392 L
.50029 .38292 L
.53926 .39864 L
.58067 .41313 L
.62058 .42532 L
.65896 .43568 L
.6998 .44548 L
.73913 .4539 L
.77694 .46121 L
.8172 .46825 L
.85594 .4744 L
.89714 .48037 L
Mistroke
.93682 .48562 L
.97499 .49026 L
.97619 .4904 L
Mfstroke
0 g
.02571 .60327 m
.06427 .34773 L
.07397 .24251 L
.08434 .15038 L
.0903 .1095 L
.09572 .07985 L
.10117 .05672 L
.10632 .04033 L
.1111 .02929 L
.11351 .02506 L
.11617 .02135 L
.11843 .0189 L
.11968 .01781 L
.12086 .01694 L
.12196 .01627 L
.12316 .01569 L
.12425 .01528 L
.12526 .015 L
.12655 .01479 L
.12728 .01473 L
.12795 .01472 L
.12918 .01479 L
.13048 .015 L
.13164 .01529 L
.13273 .01565 L
.13521 .01678 L
.13789 .01842 L
.14033 .02027 L
.14582 .02547 L
.16486 .05098 L
.18558 .08473 L
.19576 .10179 L
.20532 .11761 L
.21011 .12542 L
.21143 .12754 L
.21281 .12976 L
.21399 .13165 L
.21528 .60332 L
.21602 .60332 L
.21673 .60332 L
.21804 .60332 L
.22098 .60332 L
.22628 .60332 L
.26467 .60332 L
.30551 .60332 L
.34483 .60332 L
.38264 .60332 L
.4229 .60332 L
.46165 .60332 L
.50284 .60332 L
Mistroke
.54252 .60332 L
.58069 .60332 L
.62131 .60332 L
.66041 .60332 L
.70196 .60332 L
.742 .60332 L
.78053 .60332 L
.8215 .60332 L
.86096 .60332 L
.89891 .60332 L
.93931 .60332 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0008ool20004ool00`00oooo
o`03ool20005ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<0
0?ooool0oooo0ooo0007ool01000ooooo`002?oo00@00?ooool000Coo`800?ooo`Coo`0000?oo`00
00000P000_oo00@00?ooool000Soo`<000Goo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000So
o`03003ooooo00Goo`03003ooooo01Ooo`@00>Ooo`001ooo00@00?ooool000Soo`03003ooooo00Go
o`03003ooooo01Koo`8000;h00<00>Goo`002?oo0P002_oo0`001?oo00<00?ooool05Ooo00<00?P0
n0000_oo0_P000<00?ooool0hooo000Kool2000Fool00`00ooooo`04ool00oP0003oo`3Sool001_o
o`03003ooooo01Coo`03003h0?oo00Koo`03n0000?oo0>;oo`006ooo00<00?ooool05?oo00<00?oo
ool01ooo00<00?ooool0hOoo000Kool00`00ooooo`0Cool00`00n03oo`09ool00`00ooooo`3Pool0
01_oo`03003ooooo01?oo`03003ooooo00[oo`03003ooooo0=ooo`006ooo00<00?ooool04ooo00<0
0?ooool02ooo00<00?ooool0g_oo000Kool2000Cool00`00n03oo`0<ool00`00ooooo`3Nool001_o
o`03003ooooo01;oo`03003ooooo00goo`03003ooooo0=goo`006ooo00<00?ooool04Ooo00<00?P0
ool03ooo00<00?ooool0g?oo000Kool00`00ooooo`0Aool00`00ooooo`0?ool00`00ooooo`3Lool0
01_oo`03003ooooo017oo`03003ooooo013oo`03003ooooo0=_oo`006ooo0P004_oo00<00?ooool0
4Ooo00<00?ooool0f_oo000Kool00`00ooooo`0@ool00`00n03oo`0Bool00`00ooooo`3Jool001_o
o`03003ooooo013oo`03003ooooo01?oo`03003ooooo0=Woo`006ooo00<00?ooool04?oo00<00?oo
ool05?oo00<00?ooool0f?oo000Kool00`00ooooo`0@ool00`00ooooo`0Dool00`00ooooo`3Hool0
01_oo`80017oo`03003ooooo01Goo`03003ooooo0=Ooo`006ooo00<00?ooool04?oo00<00?ooool0
5_oo00<00?ooool0e_oo000Kool00`00ooooo`0@ool00`00ooooo`0Fool00`00ooooo`3Fool000So
o`8000Coo`03003ooooo00;oo`<000Goo`03003ooooo00ooo`03003h0?oo01Soo`03003ooooo0=Go
o`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo00ooo`03003ooooo01Soo`03n000
0?oo0=Goo`001ooo00@00?ooool000_oo`03003ooooo00;oo`80013oo`03003ooooo01Woo`03003o
oooo0=Coo`0000?oo`0000000P000_oo00@00?ooool000Soo`<000Goo`03003ooooo00ooo`03003o
oooo01[oo`03003ooooo0=?oo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo00oo
o`03003ooooo01[oo`03003ooooo0=?oo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03003o
oooo00ooo`03003ooooo01_oo`03003ooooo0=;oo`002?oo0P002Ooo10001?oo00<00?ooool03_oo
00<00?P0ool07?oo00<00?ooool0d_oo000Kool00`00ooooo`0>ool00`00ooooo`0Mool00`00oooo
o`3Aool001_oo`8000ooo`03003ooooo01goo`03n0000?oo0=7oo`006ooo00<00?ooool03_oo00<0
0?ooool07_oo00<00?ooool0d?oo000Kool00`00ooooo`0>ool00`00ooooo`0Nool00`00n03oo`3@
ool001_oo`03003ooooo00koo`03003ooooo01koo`03003h0?oo0=3oo`006ooo00<00?ooool03Ooo
00<00?ooool07ooo00<00?oon000d?oo000Kool2000>ool00`00ooooo`0Oool00`00oooh003@ool0
01_oo`03003ooooo00goo`03003ooooo01ooo`04003ooooon03?ool001_oo`03003ooooo00goo`03
003ooooo01ooo`05003ooooooooh003>ool001_oo`03003ooooo00goo`03003ooooo01ooo`05003o
oooooooh003>ool001_oo`03003ooooo00goo`03003ooooo01ooo`03003ooooo00;oo`03n03ooooo
0<_oo`006ooo0P003_oo00<00?ooool07ooo00<00?ooool00ooo00?h0?ooool0b_oo000Kool00`00
ooooo`0<ool00`00ooooo`0Pool00`00ooooo`04ool00oP0ooooo`39ool001_oo`03003ooooo00co
o`03003ooooo023oo`03003ooooo00Coo`03n03ooooo0<Woo`006ooo00<00?ooool03?oo00<00?oo
ool08?oo00<00?ooool01Ooo00?h0?ooool0b?oo000Kool00`00ooooo`0<ool00`00ooooo`0Pool0
0`00ooooo`06ool00oP0ooooo`37ool001_oo`03003ooooo00coo`03003ooooo023oo`03003ooooo
00Koo`03n03ooooo0<Ooo`006ooo0P003Ooo00<00?ooool08?oo00<00?ooool01ooo00?h0?ooool0
a_oo000Kool00`00ooooo`0<ool00`00ooooo`0Pool00`00ooooo`08ool00oP0ooooo`35ool001_o
o`03003ooooo00coo`03003ooooo023oo`03003ooooo00Woo`03n03ooooo0<Coo`002?oo0P001?oo
00<00?ooool01?oo00<00?ooool00ooo00<00?ooool02ooo00<00?ooool08Ooo00<00?ooool02Ooo
00?h0?ooool0a?oo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool02ooo00<00?oo
ool08Ooo00<00?ooool02_oo00?h0?ooool0`ooo0007ool01000ooooo`001ooo1@001?oo0P003?oo
00<00?ooool08Ooo00<00?ooool02ooo00?h0?ooool0`_oo00000ooo000000020002ool01000oooo
o`001ooo00@00?ooool000Goo`03003ooooo00_oo`03003ooooo027oo`03003ooooo00coo`03n03o
oooo0<7oo`001ooo00@00?ooool000Soo`03003oo`0000Goo`03003ooooo00_oo`03003ooooo027o
o`03003ooooo00goo`03n03ooooo0<3oo`001ooo00@00?ooool000Woo`8000Goo`03003ooooo00_o
o`03003ooooo027oo`03003ooooo00koo`03n03ooooo0;ooo`002?oo0P002ooo00<00?ooool00ooo
00<00?ooool02ooo00<00?ooool08Ooo00<00?ooool03ooo00?h0?ooool0__oo000Kool2000<ool0
0`00ooooo`0Qool00`00ooooo`0?ool00oP0ooooo`2nool001_oo`03003ooooo00[oo`03003ooooo
02;oo`03003ooooo013oo`03n03ooooo0;goo`006ooo00<00?ooool02_oo00<00?ooool08_oo00<0
0?ooool04Ooo00?h0?ooool0_?oo000Kool00`00ooooo`0:ool00`00ooooo`0Rool00`00ooooo`0B
ool00oP0ooooo`2kool001_oo`03003ooooo00[oo`03003ooooo02;oo`03003ooooo01?oo`03n03o
oooo0;[oo`006ooo0P002ooo00<00?ooool08_oo00<00?ooool05?oo00?h0?ooool0^Ooo000Kool0
0`00ooooo`0:ool00`00ooooo`0Rool00`00ooooo`0Eool00oP0ooooo`2hool001_oo`03003ooooo
00[oo`03003ooooo02;oo`03003ooooo01Koo`03n03ooooo0;Ooo`006ooo00<00?ooool02_oo00<0
0?ooool08_oo00<00?ooool05ooo00?h0?ooool0]_oo000Kool00`00ooooo`0:ool00`00ooooo`0R
ool00`00ooooo`0Hool00oP0ooooo`2eool001_oo`03003ooooo00[oo`03003ooooo02;oo`03003o
oooo01Woo`03n03ooooo0;Coo`006ooo0P002ooo00<00?ooool08_oo00<00?ooool06_oo00?h0?oo
ool0/ooo000Kool00`00ooooo`09ool00oP0003oo`0Sool00`00ooooo`0Kool00oP0ooooo`2bool0
01_oo`03003ooooo00Woo`03003ooooo02?oo`03003ooooo01coo`03n03ooooo0;7oo`006ooo00<0
0?ooool02Ooo00<00?ooool08ooo00<00?ooool07Ooo00?h0?ooool0/?oo000Kool00`00ooooo`09
ool00`00ooooo`0Sool00`00ooooo`0Nool00oP0ooooo`2_ool001_oo`8000[oo`03003ooooo02?o
o`03003ooooo01ooo`03n03ooooo0:koo`006ooo00<00?ooool02Ooo00<00?ooool08ooo00<00?oo
ool08?oo0_P0[_oo000Kool00`00ooooo`09ool00`00ooooo`0Sool00`00ooooo`0Rool00oP0oooo
o`2[ool000Soo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo00Woo`03003ooooo02?oo`03
003ooooo02?oo`03n03ooooo0:[oo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00oooo
o`09ool00`00ooooo`0Sool00`00ooooo`0Tool2n02Zool000Ooo`04003ooooo000;ool00`00oooo
o`02ool2000:ool00`00ooooo`0Sool00`00ooooo`0Vool00oP0ooooo`2Wool00003ool000000080
00;oo`04003ooooo0009ool20005ool00`00ooooo`09ool00`00ooooo`0Sool00`00ooooo`0Wool0
0oP0ooooo`2Vool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`09ool00`00oooo
o`0Sool00`00ooooo`0Xool2n02Vool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?oo
ool02Ooo00<00?ooool08ooo00<00?ooool0:_oo00?h0?ooool0Xooo0008ool2000:ool20005ool0
0`00ooooo`09ool00`00ooooo`0Sool00`00ooooo`0[ool2n02Sool001_oo`03003ooooo00Soo`03
n0000?oo02Coo`03003ooooo02goo`03n03ooooo0:3oo`006ooo0P002Ooo00<00?ooool09?oo00<0
0?ooool0;_oo0_P0X?oo000Kool00`00ooooo`08ool00`00ooooo`0Tool00`00ooooo`0`ool2n02N
ool001_oo`03003ooooo00Soo`03003ooooo02Coo`03003ooooo03;oo`03n03ooooo09_oo`006ooo
00<00?ooool02?oo00<00?ooool09?oo00<00?ooool0<ooo0_P0Vooo000Kool00`00ooooo`08ool0
0`00ooooo`0Tool00`00ooooo`0eool00oP0ooooo`2Hool001_oo`8000Woo`03003ooooo02Coo`03
003ooooo03Koo`;h09Soo`006ooo00<00?ooool02?oo00<00?ooool09?oo00<00?ooool0>?oo0_P0
U_oo000Kool00`00ooooo`08ool00`00ooooo`0Tool00`00ooooo`0jool00oP0ooooo`2Cool001_o
o`03003ooooo00Soo`03003ooooo02Coo`03003ooooo03_oo`;h09?oo`006ooo00<00?ooool01ooo
00?h0000ool09Ooo00<00?ooool0?Ooo0_P0TOoo000Kool20008ool00`00ooooo`0Uool00`00oooo
o`0oool2n02?ool001_oo`03003ooooo00Ooo`03003ooooo02Goo`03003ooooo047oo`;h08goo`00
6ooo00<00?ooool01ooo00<00?ooool09_oo00<00?ooool0@_oo0_P0Rooo000Kool00`00ooooo`07
ool00`00ooooo`0Vool00`00ooooo`14ool2n029ool001_oo`03003ooooo00Ooo`03003ooooo02Ko
o`03003ooooo04Koo`;h08Ooo`006ooo0P001ooo00?h0000ool09ooo00<00?ooool0B?oo0oP0Q?oo
000Kool00`00ooooo`06ool00`00ooooo`0Wool00`00ooooo`1;ool2n022ool001_oo`03003ooooo
00Koo`03003ooooo02Ooo`03003ooooo04goo`?h07ooo`006ooo00<00?ooool01_oo00<00?ooool0
9ooo00<00?ooool0D?oo0_P0OOoo0008ool20004ool00`00ooooo`02ool40004ool00`00ooooo`06
ool00`00ooooo`0Wool00`00ooooo`1Bool3n01jool000Ooo`04003ooooo0008ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`0Wool00`00ooooo`1Eool3n01gool000Ooo`04003ooooo0009
ool00`00ooooo`04ool20006ool00oP0003oo`0Xool00`00ooooo`1Hool2n01eool00003ool00000
008000;oo`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`0Xool00`00
ooooo`1Jool3n01bool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`05ool00`00
ooooo`0Xool00`00ooooo`1Mool3n01_ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<0
0?ooool01Ooo00<00?ooool0:?oo00<00?ooool0H?oo1?P0Jooo0008ool2000:ool20005ool00`00
ooooo`05ool00`00ooooo`0Xool00`00ooooo`1Tool4n01Wool001_oo`8000Koo`03003ooooo02So
o`03003ooooo06Soo`?h06Coo`006ooo00<00?ooool01?oo00?h0000ool0:Ooo00<00?ooool0Jooo
0oP0HOoo000Kool00`00ooooo`04ool00`00ooooo`0Yool00`00ooooo`1^ool4n01Mool001_oo`03
003ooooo00Coo`03003ooooo02Woo`03003ooooo07;oo`?h05[oo`006ooo00<00?ooool01?oo00<0
0?ooool0:Ooo00<00?ooool0MOoo1?P0E_oo000Kool20005ool00`00ooooo`0Yool00`00ooooo`1i
ool4n01Bool001_oo`03003ooooo00Coo`03003ooooo02Woo`03003ooooo07goo`Ch04koo`006ooo
00<00?ooool01?oo00<00?ooool0:Ooo00<00?ooool0POoo1OP0BOoo000Kool00`00ooooo`03ool0
0`00ooooo`0Zool00`00ooooo`26ool5n014ool001_oo`03003ooooo00?oo`03003ooooo02[oo`03
003ooooo08_oo`Gh03ooo`006ooo00<00?ooool00ooo00<00?ooool0:_oo00<00?ooool0T?oo1OP0
>_oo000Kool20004ool00`00ooooo`0Zool00`00ooooo`2Eool6n00dool001_oo`03003ooooo00?o
o`03003ooooo02[oo`03003ooooo09_oo`Oh02goo`006ooo00<00?ooool00ooo00<00?ooool0:_oo
00<00?ooool0X_oo2?P09Ooo000Kool00`00ooooo`02ool00oP0003oo`0[ool00`00ooooo`2Zool6
n00Oool001_oo`03003ooooo00;oo`03003ooooo02_oo`03003ooooo0;3oo`Sh01Ooo`006ooo0P00
0ooo00<00?ooool0:ooo00<00?ooool0^?oo2_P03Ooo000Kool00`00ooooo`02ool00`00ooooo`0[
ool00`00ooooo`32ool6n007ool001_oo`03003ooooo00;oo`03003ooooo02_oo`03003ooooo0<Oo
o`03n03ooooo00Goo`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool00_oo00<00?oo
ool0:ooo00<00?ooool0cooo0007ool01000ooooo`002_oo00<00?ooool00ooo00H00?ooooooooP0
000]ool00`00ooooo`3?ool000Ooo`04003ooooo000:ool00`00ooooo`03ool20002ool00`00oooo
o`0/ool00`00ooooo`3?ool00003ool00000008000;oo`04003ooooo000:ool00`00ooooo`03ool0
1@00oooooooo0000;_oo00<00?ooool0cooo0007ool01000ooooo`002_oo00<00?ooool00ooo00D0
0?ooooooo`0002koo`03003ooooo0<ooo`001ooo00@00?ooool000Soo`<000Goo`05003oooooool0
000^ool00`00ooooo`3?ool000Soo`8000_oo`03003ooooo00?oo`05003oooooool0000^ool00`00
ooooo`3?ool001_oo`8000;oo`03003ooooo02coo`03003ooooo0<ooo`006ooo00@00?ooool002oo
o`03003ooooo0<ooo`006ooo00@00?ooool002ooo`03003ooooo0<ooo`006ooo00@00?ooool002oo
o`03003ooooo0<ooo`006ooo00@00?ooool002ooo`03003ooooo0<ooo`006ooo00@00?ooool002oo
o`03003ooooo0<ooo`006ooo0P0000?oo`00ool0;_oo00<00?ooool0cooo000Kool00`00ool0000`
ool00`00ooooo`3?ool001_oo`03003oo`00033oo`03003ooooo0<ooo`006ooo00<00?oo0000<?oo
00<00?ooool0cooo000Kool00`00ool0000`ool00`00ooooo`3?ool001_oo`<002goo`@002ooo`@0
033oo`80037oo`03003ooooo02koo`<000Ooo`006ooo00<00?oo0000;ooo0P00;ooo00<00?ooool0
<?oo00@00?ooool0033oo`03003ooooo037oo`03003ooooo00Coo`006ooo0P00<?oo0P00<?oo00<0
0?ooool0<_oo00<00?ooool0:ooo1@00<_oo00<00?ooool01?oo000Kool2000`ool2000aool00`00
ooooo`0_ool2000^ool01000ooooo`00<?oo0`001ooo000Kool2000`ool2000bool00`00ooooo`0`
ool00`00ooooo`0/ool00`00ool0000`ool00`00ooooo`07ool001_oo`8002koo`@002ooo`04003o
oooo000_ool01000ooooo`00;ooo0P00<?oo00<00?ooool01ooo000Kool2000`ool2000`ool2000a
ool2000aool00`00ooooo`0^ool40006ool001_oo`80037oo`03003ooooo0<ooo`006ooo00<00?oo
ool0<?oo00<00?ooool0cooo000Kool00`00ooooo`0`ool00`00ooooo`3?ool001_oo`03003ooooo
033oo`03003ooooo0<ooo`005?ooo`003@00000Kool00`00ooooo`07ool00`00ooooo`07ool00`00
ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`07ool00`00
ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00
ooooo`08ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00
ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`07ool00`00
ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`05ool001_oo`03003ooooo0?oo
o`?oo`006ooo00<00?ooool0oooo0ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?oo
ob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.539323, -0.636008, \
0.0197364, 0.0037855}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[BoxData[
    \(\(aha!\)\ Mathematica\ somehow\ forgets\ the\ delta\ \(contrib!\)\)], \
"Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[eta], 
        Abs[Im[betakx0korr[eta, 10.0]] - 
            Sqrt[11] ImbetaA[1, 1, 1 - 10^\((\(-6\))\), eta, 10]]}, {eta, 
        0.01, 1, 0.05}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, 3.316830494538381`*^-9}, {0.060000000000000005`, 
        5.982895099915925`*^-7}, {0.11`, 2.4062104213712354`*^-6}, {0.16`, 
        3.832161068334505`*^-6}, {0.21000000000000002`, 
        3.4816056448949695`*^-6}, {0.26`, 1.6619540182949422`*^-6}, {0.31`, 
        5.629282939789704`*^-7}, {0.36000000000000004`, 
        2.40938788376166`*^-6}, {0.41000000000000003`, 
        3.5976810058846453`*^-6}, {0.46`, 4.168125264936151`*^-6}, {0.51`, 
        4.279123371198068`*^-6}, {0.56`, 4.095061492570551`*^-6}, {0.61`, 
        3.745447748282338`*^-6}, {0.66`, 
        3.319541215796029`*^-6}, {0.7100000000000001`, 
        2.8736488848490893`*^-6}, {0.76`, 2.4406195259230046`*^-6}, {0.81`, 
        2.037862120052303`*^-6}, {0.8600000000000001`, 
        1.6731722353169332`*^-6}, {0.91`, 
        1.348644574350466`*^-6}, {0.9600000000000001`, 
        1.0631827950247086`*^-6}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(looks\ o . k . \)], "Input"],

Cell[BoxData[
    \(Testing\ Mathematica\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NIntegrate[\(1/\((eta - xi + I\ eps)\) /. eta \[Rule] 0.5\) /. 
        eps \[Rule] 10^\((\(-10\))\), {xi, \(-1\), 0.5, 1}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.49999999999805766`\)."\
\)], "Message"],

Cell[BoxData[
    \(\(\(1.0986277812671152`\)\(\[InvisibleSpace]\)\) - 
      3.141596848366013`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[1/\((eta - xi)\) /. eta \[Rule] 0.5, {xi, \(-1\), 1}, 
      PrincipalValue \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(\(\(1.0986122886681096`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["gamma", "Section"],

Cell[CellGroupData[{

Cell["Exact results -numerical ", "Subsection"],

Cell[CellGroupData[{

Cell["Preliminaries", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(s = Sqrt[\((1 - om^2/k^2)\)*\((1 - kz^2/k^2)\)]\)], "Input"],

Cell[BoxData[
    \(\@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi1[k_, kz_, om_] = om*kz/k^2 + s\)], "Input"],

Cell[BoxData[
    \(\(kz\ om\)\/k\^2 + \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi2[k_, kz_, om_] = om*kz/k^2 - s\)], "Input"],

Cell[BoxData[
    \(\(kz\ om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Imaginary part:", "Subsubsection"],

Cell[BoxData[
    \(Imgamma[md2_, k_, kz_, om_, a_] := 
      If[om > k, 
        0, \(\(\(-md2\)/2\)/k\)/\((1 - kz^2/k^2)\)*
          NIntegrate[
            1/\((1 + a\ xi^2)\)^2*\((om + 
                  a\ xi\ kz)\)*\((2 \((xi - om\ kz/k^2)\)^2 - \((1 - 
                          kz^2/k^2)\)\ \((1 - om^2/k^2)\))\)/
                Sqrt[\((1 - kz^2/k^2)\)\ \((1 - om^2/k^2)\) - \((xi - 
                          om\ kz/k^2)\)^2], {xi, 
              xi2[k, kz, om]*
                If[xi2[k, kz, om] > 
                    0, \((1 + 10^\((\(-50\))\))\), \((1 - 
                      10^\((\(-50\))\))\)], xi1[k, kz, om]}, 
            WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30]]\)], "Input",\

  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Real part:", "Subsubsection"],

Cell[BoxData[
    \(Regamma[md2_, k_, kz_, om_, a_] := 
      Module[{res, mxi1, mxi2, wp}, 
        If[logging \[Equal] 0, 
          Print["\<gamma:\>", om]]; \[IndentingNewLine]mxi1 = 
          If[k > om, xi1[k, kz, om], 0]; 
        mxi2 = If[k > om, xi2[k, kz, om], 0]; \ 
        If[kz/k < 1 - 10^\((\(-10\))\), wp = 30, 
          wp = 50]; \[IndentingNewLine]If[om < k, 
          res = md2/
                2*\((1/k^2*
                    NIntegrate[\((om^2 + a\ k^2\ xi^2 - 
                            2\ k^2/\((k^2 - kz^2)\)\ \((om^2 - 
                                  a\ kz^2\ xi^2)\))\)/\((1 + 
                              a\ xi^2)\)^2, {xi, \(-1\), 1}, 
                      WorkingPrecision \[Rule] wp, MaxRecursion \[Rule] 30] + 
                  k/\((k^2 - 
                          kz^2)\)*\((NIntegrate[\((om + 
                                  a\ kz\ xi)\)/\((1 + 
                                    a\ xi^2)\)^2\ \((2\ \((xi - 
                                        om\ kz/k^2)\)^2 - \((1 - 
                                        om^2/k^2)\)\ \((1 - kz^2/k^2)\))\)/
                              Sqrt[\((xi - om\ kz/k^2)\)^2 - \((1 - 
                                        om^2/k^2)\)\ \((1 - 
                                        kz^2/k^2)\)], {xi, \(-1\), 
                            mxi2*If[
                                mxi2 < 
                                  0, \((1 + 10^\((\(-50\))\))\), \((1 - 
                                    10^\((\(-50\))\))\)]}, 
                          WorkingPrecision \[Rule] wp, 
                          MaxRecursion \[Rule] 30] + 
                        Sign[om - kz]*
                          NIntegrate[\((om + 
                                    a\ kz\ xi)\)/\((1 + 
                                      a\ xi^2)\)^2\ \((2\ \((xi - 
                                        om\ kz/k^2)\)^2 - \((1 - 
                                        om^2/k^2)\)\ \((1 - kz^2/k^2)\))\)/
                                Sqrt[\((xi - om\ kz/k^2)\)^2 - \((1 - 
                                        om^2/k^2)\)\ \((1 - 
                                        kz^2/k^2)\)], {xi, 
                              mxi1*\((1 + 10^\((\(-50\))\))\), 1}, 
                            WorkingPrecision \[Rule] wp, 
                            MaxRecursion \[Rule] 30])\))\)]; 
        If[om > k, \(res = 
              md2/2*\((1/k^2*
                      NIntegrate[\((om^2 + a\ k^2\ xi^2 - 
                              2\ k^2/\((k^2 - kz^2)\)*\ \((om^2 - 
                                    a\ kz^2\ xi^2)\))\)/\((1 + 
                                a\ xi^2)\)^2, {xi, \(-1\), 1}, 
                        WorkingPrecision \[Rule] wp, 
                        MaxRecursion \[Rule] 30] + 
                    k/\((k^2 - 
                            kz^2)\)*\((NIntegrate[\((om + 
                                  a\ kz\ xi)\)/\((1 + 
                                    a\ xi^2)\)^2\ *\((2\ \((xi - 
                                        om\ kz/k^2)\)^2 - \((1 - 
                                      om^2/k^2)\)\ \((1 - kz^2/k^2)\))\)*
                            Sign[om - kz\ xi]/
                              Sqrt[\((xi - om\ kz/k^2)\)^2 - \((1 - 
                                        om^2/k^2)\)\ \((1 - 
                                        kz^2/k^2)\)], {xi, \(-1\), 1}, 
                          WorkingPrecision \[Rule] wp, 
                          MaxRecursion \[Rule] 30])\))\);\)]; 
        If[logging \[Equal] 0, Print[res]]; res]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      Integrate[\((1 - xi^2)\)/\((1 + a\ xi^2)\)^2, {xi, \(-1\), 
          1}]]\)], "Input"],

Cell[BoxData[
    \(\(\@a + \((\(-1\) + a)\)\ ArcTan[\@a]\)\/a\^\(3/2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% /. a \[Rule] 1\)], "Input"],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["for imaginary om", "Subsection"],

Cell[BoxData[
    \(gammaspecial[md2_, k_, kz_, om_, a_] := 
      md2/2*\((1/k^2*
              NIntegrate[\((om^2 + a\ k^2\ xi^2 - 
                      2\ k^2/\((k^2 - kz^2)\)*\ \((om^2 - 
                            a\ kz^2\ xi^2)\))\)/\((1 + 
                        a\ xi^2)\)^2, {xi, \(-1\), 1}, 
                WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30] + 
            k^2/\((k^2 - 
                    kz^2)\)*\((NIntegrate[\((om + 
                          a\ kz\ xi)\)/\((1 + 
                            a\ xi^2)\)^2\ *\((2\ \((xi - 
                                om\ kz/k^2)\)^2 - \((1 - om^2/k^2)\)\ \((1 - 
                              kz^2/k^2)\))\)*
                    R[om - kz\ xi, 
                      Sqrt[k^2 - kz^2]\ Sqrt[1 - xi^2]], {xi, \(-1\), 1}, 
                  WorkingPrecision \[Rule] 30, 
                  MaxRecursion \[Rule] 30])\))\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(gammaspecial[1, 1, 1/2, I, 1]\)], "Input"],

Cell[BoxData[
    \(0.039214686147775154574468060816501`18.7338 + 
      0``21.7327\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["small a", "Subsection"],

Cell[BoxData[
    \(\(Intexp = 
        Series[\(1/\((1 + a\ \((vn^2)\))\)^2\)/\((om - 
                  k\ xi)\)\ \ \((om\ \((nz^2\ k^2\ k^2\ xi^2 + 
                      k^2\ \((k^2\ nz^2 + k^2\ xi^2 - 
                            4\ nz\ k\ k\ xi\ vn)\) + 
                      k^4 \((\(-1\) + 2\ \((vn)\)^2)\))\) + 
                a\ \((vn)\)\ \((om\ k^3\ nz^3\ k\ xi + 
                      k^2\ k\ nz\ \((k^2\ nz^2 + k\ xi\ \((2\ k\ xi - om)\) - 
                            nz\ k\ \((om + 3\ k\ xi)\)\ vn)\) + 
                      k^4\ \((\((om - k\ xi)\)\ vn + 
                            nz\ k\ \((\(-1\) + 2\ vn^2)\))\))\))\), {a, 0, 
            toorder}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      1/k^4*\(1/2\)/\((1 - nz^2)\)*\(1/2\)/Pi*
        Integrate[\(Intexp[\([3]\)]\)[\([1]\)], {phi, 0, 2*Pi}]]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      1/k^4*\(1/2\)/\((1 - nz^2)\)*\(1/2\)/Pi*
        Integrate[\(Intexp[\([3]\)]\)[\([2]\)], {phi, 0, 2*Pi}]]\)], "Input"],

Cell[BoxData[
    \(\(\((\(-1\) + nz\^2)\)\ xi\ \((\(-k\) + om\ xi)\)\ \((\(-1\) + xi\^2)\)\
\)\/\(4\ \((om - k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apart[%]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\((\(-1\) + 
                  nz\^2)\)\ \((k\^2 - om\^2)\)\^2\)\/\(4\ k\^4\)\)\) + \
\(\((\(-1\) + nz\^2)\)\ om\ \((2\ k\^2 - om\^2)\)\ xi\)\/\(4\ k\^3\) + \
\(\((\(-1\) + nz\^2)\)\ \((k\^2 - om\^2)\)\ xi\^2\)\/\(4\ k\^2\) - \
\(\((\(-1\) + nz\^2)\)\ om\ xi\^3\)\/\(4\ k\) + \(k\^4\ om - k\^4\ nz\^2\ om \
- 2\ k\^2\ om\^3 + 2\ k\^2\ nz\^2\ om\^3 + om\^5 - nz\^2\ om\^5\)\/\(4\ k\^4\ \
\((\(-om\) + k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[\((k\^4\ om - k\^4\ nz\^2\ om - 2\ k\^2\ om\^3 + 
            2\ k\^2\ nz\^2\ om\^3 + om\^5 - nz\^2\ om\^5)\)/om]\)], "Input"],

Cell[BoxData[
    \(\(-\((\(-1\) + nz\^2)\)\)\ \((k\^2 - om\^2)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      2\ k\^2\ om\^2 - 8\ k\^2\ nz\^2\ om\^2 - om\^4 + 
        5\ nz\^2\ om\^4]\)], "Input"],

Cell[BoxData[
    \(om\^2\ \((k\^2\ \((2 - 8\ nz\^2)\) + \((\(-1\) + 
                5\ nz\^2)\)\ om\^2)\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Fast gamma", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imarg2 = 
      Compile[{md2, k, kz, om, a, xi}, 
        1/\((1 + a\ xi^2)\)^2*\((om + 
              a\ xi\ kz)\)*\((2 \((xi - om\ kz/k^2)\)^2 - \((1 - 
                      kz^2/k^2)\)\ \((1 - om^2/k^2)\))\)/
            Sqrt[\((1 - kz^2/k^2)\)\ \((1 - om^2/k^2)\) - \((xi - 
                      om\ kz/k^2)\)^2]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(\((om + a\ xi\ kz)\)\ \((2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \
\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\))\)\)\/\(\((1 + a\ xi\^2)\)\^2\ \
\@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\) - \((xi - \(om\ \
kz\)\/k\^2)\)\^2\)\), "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(FastImgamma[md2_, k_, kz_, om_, a_] := 
      If[om > k, 
        0, \(\(\(-md2\)/2\)/k\)/\((1 - kz^2/k^2)\)*
          NIntegrate[
            Imarg2[md2, k, kz, om, a, xi], {xi, 
              xi2[k, kz, om]*
                If[xi2[k, kz, om] > 
                    0, \((1 + 10^\((\(-13\))\))\), \((1 - 
                      10^\((\(-13\))\))\)], 
              xi1[k, kz, om]*\((1 - 10^\((\(-13\))\))\)}]]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Plot[FastImgamma[1, 1, 8/10, om, 1], {om, 1/10, 1}]]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(67\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(67\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"rnderr" \(\(:\)\(\ \)\) 
      "Numerical approximation has caused \!\({xi}\) to take the value \
\!\({\(-0.011081959403953946`\)}\) where the integrand is singular."\)], \
"Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(67\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(Power :: \"infy\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"rnderr" \(\(:\)\(\ \)\) 
      "Numerical approximation has caused \!\({xi}\) to take the value \
\!\({\(-0.011081959403953946`\)}\) where the integrand is singular."\)], \
"Message"],

Cell[BoxData[
    \(NIntegrate::"rnderr" \(\(:\)\(\ \)\) 
      "Numerical approximation has caused \!\({xi}\) to take the value \
\!\({\(-0.011081959403953946`\)}\) where the integrand is singular."\)], \
"Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"rnderr\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.355208611077611`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.7998356532343436`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.7233514325711008`\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.01991 1.60463 [
[.21429 .00741 -9 -9 ]
[.21429 .00741 9 0 ]
[.40476 .00741 -9 -9 ]
[.40476 .00741 9 0 ]
[.59524 .00741 -9 -9 ]
[.59524 .00741 9 0 ]
[.78571 .00741 -9 -9 ]
[.78571 .00741 9 0 ]
[.97619 .00741 -3 -9 ]
[.97619 .00741 3 0 ]
[.01131 .10014 -24 -4.5 ]
[.01131 .10014 0 4.5 ]
[.01131 .18037 -18 -4.5 ]
[.01131 .18037 0 4.5 ]
[.01131 .2606 -24 -4.5 ]
[.01131 .2606 0 4.5 ]
[.01131 .34084 -18 -4.5 ]
[.01131 .34084 0 4.5 ]
[.01131 .42107 -24 -4.5 ]
[.01131 .42107 0 4.5 ]
[.01131 .5013 -18 -4.5 ]
[.01131 .5013 0 4.5 ]
[.01131 .58153 -24 -4.5 ]
[.01131 .58153 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .01991 m
.21429 .02616 L
s
[(0.2)] .21429 .00741 0 1 Mshowa
.40476 .01991 m
.40476 .02616 L
s
[(0.4)] .40476 .00741 0 1 Mshowa
.59524 .01991 m
.59524 .02616 L
s
[(0.6)] .59524 .00741 0 1 Mshowa
.78571 .01991 m
.78571 .02616 L
s
[(0.8)] .78571 .00741 0 1 Mshowa
.97619 .01991 m
.97619 .02616 L
s
[(1)] .97619 .00741 0 1 Mshowa
.125 Mabswid
.07143 .01991 m
.07143 .02366 L
s
.11905 .01991 m
.11905 .02366 L
s
.16667 .01991 m
.16667 .02366 L
s
.2619 .01991 m
.2619 .02366 L
s
.30952 .01991 m
.30952 .02366 L
s
.35714 .01991 m
.35714 .02366 L
s
.45238 .01991 m
.45238 .02366 L
s
.5 .01991 m
.5 .02366 L
s
.54762 .01991 m
.54762 .02366 L
s
.64286 .01991 m
.64286 .02366 L
s
.69048 .01991 m
.69048 .02366 L
s
.7381 .01991 m
.7381 .02366 L
s
.83333 .01991 m
.83333 .02366 L
s
.88095 .01991 m
.88095 .02366 L
s
.92857 .01991 m
.92857 .02366 L
s
.25 Mabswid
0 .01991 m
1 .01991 L
s
.02381 .10014 m
.03006 .10014 L
s
[(0.05)] .01131 .10014 1 0 Mshowa
.02381 .18037 m
.03006 .18037 L
s
[(0.1)] .01131 .18037 1 0 Mshowa
.02381 .2606 m
.03006 .2606 L
s
[(0.15)] .01131 .2606 1 0 Mshowa
.02381 .34084 m
.03006 .34084 L
s
[(0.2)] .01131 .34084 1 0 Mshowa
.02381 .42107 m
.03006 .42107 L
s
[(0.25)] .01131 .42107 1 0 Mshowa
.02381 .5013 m
.03006 .5013 L
s
[(0.3)] .01131 .5013 1 0 Mshowa
.02381 .58153 m
.03006 .58153 L
s
[(0.35)] .01131 .58153 1 0 Mshowa
.125 Mabswid
.02381 .03596 m
.02756 .03596 L
s
.02381 .052 m
.02756 .052 L
s
.02381 .06805 m
.02756 .06805 L
s
.02381 .0841 m
.02756 .0841 L
s
.02381 .11619 m
.02756 .11619 L
s
.02381 .13223 m
.02756 .13223 L
s
.02381 .14828 m
.02756 .14828 L
s
.02381 .16433 m
.02756 .16433 L
s
.02381 .19642 m
.02756 .19642 L
s
.02381 .21247 m
.02756 .21247 L
s
.02381 .22851 m
.02756 .22851 L
s
.02381 .24456 m
.02756 .24456 L
s
.02381 .27665 m
.02756 .27665 L
s
.02381 .2927 m
.02756 .2927 L
s
.02381 .30874 m
.02756 .30874 L
s
.02381 .32479 m
.02756 .32479 L
s
.02381 .35688 m
.02756 .35688 L
s
.02381 .37293 m
.02756 .37293 L
s
.02381 .38898 m
.02756 .38898 L
s
.02381 .40502 m
.02756 .40502 L
s
.02381 .43711 m
.02756 .43711 L
s
.02381 .45316 m
.02756 .45316 L
s
.02381 .46921 m
.02756 .46921 L
s
.02381 .48525 m
.02756 .48525 L
s
.02381 .51735 m
.02756 .51735 L
s
.02381 .53339 m
.02756 .53339 L
s
.02381 .54944 m
.02756 .54944 L
s
.02381 .56549 m
.02756 .56549 L
s
.02381 .00386 m
.02756 .00386 L
s
.02381 .59758 m
.02756 .59758 L
s
.02381 .61362 m
.02756 .61362 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.11905 .28559 m
.15382 .37074 L
.19174 .45153 L
.22736 .5135 L
.26161 .55862 L
.27922 .5759 L
.29806 .58977 L
.30726 .59478 L
.3121 .59696 L
.31724 .59892 L
.32172 .60034 L
.32666 .60158 L
.32883 .60202 L
.33116 .60242 L
.33336 .60273 L
.33537 .60296 L
.33765 .60315 L
.3389 .60322 L
.34008 .60327 L
.34118 .6033 L
.34218 .60332 L
.34329 .60332 L
.34446 .6033 L
.34564 .60326 L
.3463 .60323 L
.34691 .6032 L
.34919 .60304 L
.35047 .60292 L
.35186 .60276 L
.35439 .60241 L
.35894 .60156 L
.36378 .60037 L
.37247 .59745 L
.3809 .5937 L
.38999 .58865 L
.40644 .57697 L
.42448 .56057 L
.44347 .53955 L
.47777 .49305 L
.51427 .43408 L
.54941 .37139 L
.58676 .30234 L
.62274 .23728 L
.65736 .1793 L
.69419 .12548 L
.72965 .08333 L
.76375 .05273 L
.78125 .04083 L
.80006 .03083 L
.80917 .02696 L
Mistroke
.81911 .02343 L
.82861 .02069 L
.83721 .01871 L
.84635 .01708 L
.85495 .01598 L
.85968 .01553 L
.86403 .01521 L
.8688 .01496 L
.8712 .01486 L
.87257 .01482 L
.87383 .01479 L
.87491 .01477 L
.8761 .01475 L
.8773 .01473 L
.87795 .01472 L
.87857 .01472 L
.87969 .01472 L
.88072 .01472 L
.88185 .01472 L
.88304 .01473 L
.88405 .01474 L
.88512 .01475 L
.88705 .01479 L
.88934 .01484 L
.89143 .01491 L
.89608 .01509 L
.90112 .01534 L
.91026 .01592 L
.92783 .01728 L
.94714 .01877 L
.95438 .01923 L
.95837 .01944 L
.96216 .01961 L
.96567 .01974 L
.96752 .01979 L
.96949 .01984 L
.97115 .01987 L
.97202 .01988 L
.97297 .01989 L
.97384 .0199 L
.97463 .01991 L
.97539 .01991 L
.97619 .01991 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00AP3oool2000000@0oooo00<0
00000?ooo`3oool00P3oool4000002@0oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool08`3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000002@0oooo0P00
00040?ooo`030000003oool0oooo00<0oooo0P00000Z0?ooo`@000001P3oool004D0oooo00@00000
0?ooo`3oool00000203oool00`000000oooo0?ooo`0T0?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool08P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
0280oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000[0?ooo`030000003o
ool0oooo00D0oooo00150?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool08`3o
ool010000000oooo0?ooo`0000070?ooo`D000008`3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo00000280oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
000[0?ooo`030000003oool0oooo00D0oooo00150?ooo`040000003oool0oooo000000X0oooo00<0
00000?ooo`3oool08P3oool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo000002@0
oooo00@000000?ooo`3oool00000203oool3000002<0oooo00@000000?ooo`3oool000002@3oool2
000002`0oooo00<000000?ooo`3oool01@3oool004D0oooo00@000000?ooo`3oool000002`3oool0
0`000000oooo0?ooo`0Q0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`000000903o
ool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo02<0oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`00000[0?ooo`030000003oool0oooo00D0oooo000K0?ooo`03
0000003oool0oooo02L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000S
0?ooo`040000003oool0oooo000000T0oooo0P00000T0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool08`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
02T0oooo0`0000070?ooo`006`3oool2000002T0oooo0P00000:0?ooo`8000009@3oool2000000/0
oooo00<000000?ooo`3oool08`3oool2000000X0oooo0`00000S0?ooo`8000002P3oool2000002`0
oooo00<000000?ooo`3oool01@3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3o
ool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0=/0oooo3@00000J
0?ooo`006`3oool00`000000oooo0?ooo`3D0?ooo`L000003@3oool=000000d0oooo000D0?ooool0
00003@00000001/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`050?ooo`<00000
0P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`050?ooo`006`3oool00`000000oooo0?ooo`3=0?ooo`800000<`3o
ool001/0oooo00<000000?ooo`3oool0bP3oool3000003D0oooo000K0?ooo`800000b@3oool20000
03P0oooo000K0?ooo`030000003oool0oooo0<H0oooo0P00000j0?ooo`006`3oool00`000000oooo
0?ooo`340?ooo`800000?03oool001/0oooo00<000000?ooo`3oool0``3oool00`000000oooo0?oo
o`0l0?ooo`006`3oool200000<<0oooo00<000000?ooo`3oool0?@3oool001/0oooo00<000000?oo
o`3oool0`@3oool00`000000oooo0?ooo`0n0?ooo`006`3oool00`000000oooo0?ooo`300?ooo`03
0000003oool0oooo03l0oooo000K0?ooo`030000003oool0oooo0;l0oooo00<000000?ooo`3oool0
@03oool001/0oooo0P00002o0?ooo`030000003oool0oooo0440oooo000K0?ooo`030000003oool0
oooo0;d0oooo00<000000?ooo`3oool0@P3oool001/0oooo00<000000?ooo`3oool0_03oool00`00
0000oooo0?ooo`130?ooo`006`3oool00`000000oooo0?ooo`2k0?ooo`030000003oool0oooo04@0
oooo000K0?ooo`030000003oool0oooo0;X0oooo00<000000?ooo`3oool0A@3oool001/0oooo0P00
002j0?ooo`030000003oool0oooo04H0oooo000K0?ooo`030000003oool0oooo0;P0oooo00<00000
0?ooo`3oool0A`3oool00080oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?oo
o`<000001@3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0oooo04P0oooo00001@3oool0
00000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0;H0oooo00<000000?ooo`3oool0B@3oool000050?ooo`000000oooo
0?ooo`000000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo0P00
002f0?ooo`030000003oool0oooo04X0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@0
00000?ooo`3oool000000P3oool3000000D0oooo00<000000?ooo`3oool0]03oool00`000000oooo
0?ooo`1;0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2d0?ooo`030000003oool0oooo04/0
oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool0C03oool00080
oooo0P00000:0?ooo`8000000`3oool4000000@0oooo0P00002c0?ooo`030000003oool0oooo04d0
oooo000K0?ooo`030000003oool0oooo0;40oooo00<000000?ooo`3oool0CP3oool001/0oooo00<0
00000?ooo`3oool0/03oool00`000000oooo0?ooo`1?0?ooo`006`3oool00`000000oooo0?ooo`2_
0?ooo`030000003oool0oooo0500oooo000K0?ooo`030000003oool0oooo0:h0oooo00<000000?oo
o`3oool0D@3oool001/0oooo0P00002_0?ooo`030000003oool0oooo0540oooo000K0?ooo`030000
003oool0oooo0:d0oooo00<000000?ooo`3oool0DP3oool001/0oooo00<000000?ooo`3oool0[03o
ool00`000000oooo0?ooo`1C0?ooo`006`3oool00`000000oooo0?ooo`2[0?ooo`030000003oool0
oooo05@0oooo000K0?ooo`800000[03oool00`000000oooo0?ooo`1D0?ooo`006`3oool00`000000
oooo0?ooo`2Z0?ooo`030000003oool0oooo05D0oooo000K0?ooo`030000003oool0oooo0:T0oooo
00<000000?ooo`3oool0EP3oool001/0oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?oo
o`1F0?ooo`006`3oool200000:T0oooo00<000000?ooo`3oool0E`3oool001/0oooo00<000000?oo
o`3oool0Y`3oool00`000000oooo0?ooo`1H0?ooo`00203oool2000000@0oooo00<000000?ooo`3o
ool00P3oool4000000@0oooo00<000000?ooo`3oool0YP3oool00`000000oooo0?ooo`1I0?ooo`00
1`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0YP3oool00`000000oooo0?ooo`1I0?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`03
0000003oool0oooo00<0oooo0P00002V0?ooo`030000003oool0oooo05X0oooo00070?ooo`040000
003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2T0?ooo`03
0000003oool0oooo05/0oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`2T0?ooo`030000003oool0oooo05/0oooo00070?ooo`040000
003oool0oooo000000P0oooo0`0000050?ooo`030000003oool0oooo0:<0oooo00<000000?ooo`3o
ool0G03oool000P0oooo0P00000;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
X`3oool00`000000oooo0?ooo`1L0?ooo`006`3oool200000:<0oooo00<000000?ooo`3oool0G@3o
ool001/0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`1N0?ooo`006`3oool00`00
0000oooo0?ooo`2Q0?ooo`030000003oool0oooo05h0oooo000K0?ooo`030000003oool0oooo0:00
oooo00<000000?ooo`3oool0G`3oool001/0oooo0P00002Q0?ooo`030000003oool0oooo05l0oooo
000K0?ooo`030000003oool0oooo09l0oooo00<000000?ooo`3oool0H03oool001/0oooo00<00000
0?ooo`3oool0WP3oool00`000000oooo0?ooo`1Q0?ooo`006`3oool00`000000oooo0?ooo`2N0?oo
o`030000003oool0oooo0640oooo000K0?ooo`800000WP3oool00`000000oooo0?ooo`1R0?ooo`00
6`3oool00`000000oooo0?ooo`2M0?ooo`030000003oool0oooo0680oooo000K0?ooo`030000003o
ool0oooo09`0oooo00<000000?ooo`3oool0H`3oool001/0oooo00<000000?ooo`3oool0W03oool0
0`000000oooo0?ooo`1S0?ooo`006`3oool2000009`0oooo00<000000?ooo`3oool0I03oool001/0
oooo00<000000?ooo`3oool0V`3oool00`000000oooo0?ooo`1T0?ooo`006`3oool00`000000oooo
0?ooo`2J0?ooo`030000003oool0oooo06D0oooo00020?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`@000000P3oool3000000D0oooo00<000000?ooo`3oool0VP3oool00`000000oooo0?oo
o`1U0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`2I0?ooo`030000003oool0oooo06H0oooo0000
1@3oool000000?ooo`3oool0000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`020?ooo`800000VP3oool00`000000oooo0?ooo`1V0?ooo`0000D0oooo0000003oool0oooo0000
000:0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`2H0?ooo`030000
003oool0oooo06L0oooo00001@3oool000000?ooo`3oool0000000X0oooo00D000000?ooo`3oool0
oooo000000070?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool0I`3oool000050?oo
o`000000oooo0?ooo`000000203oool3000000<0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`2G0?ooo`030000003oool0oooo06P0oooo00020?ooo`8000002`3oool010000000oooo
0?ooo`3oool4000000@0oooo0P00002H0?ooo`030000003oool0oooo06P0oooo000K0?ooo`030000
003oool0oooo09H0oooo00<000000?ooo`3oool0J@3oool001/0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`1m0?ooo`030000003oool0oooo06T0oooo000K0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0O03oool00`000000oooo0?ooo`1Z0?ooo`006`3oool20000
01P0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`1Z0?ooo`006`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo07X0oooo00<000000?ooo`3oool0J`3oool001/0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo06/0oooo
000K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0N03oool00`000000oooo0?oo
o`1/0?ooo`006`3oool2000001T0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`1]
0?ooo`006`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo07H0oooo00<000000?oo
o`3oool0K@3oool001/0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1e0?ooo`03
0000003oool0oooo06h0oooo000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
M03oool00`000000oooo0?ooo`1^0?ooo`006`3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo07<0oooo00<000000?ooo`3oool0K`3oool001/0oooo0P00000K0?ooo`030000003oool0
oooo07<0oooo00<000000?ooo`3oool0K`3oool001/0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`1a0?ooo`030000003oool0oooo0700oooo00080?ooo`800000103oool00`000000
oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0740
oooo00<000000?ooo`3oool0L03oool000L0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0L03oool00`000000
oooo0?ooo`1a0?ooo`001`3oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo00@0
oooo0P00000M0?ooo`030000003oool0oooo06h0oooo00<000000?ooo`3oool0LP3oool000L0oooo
00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0KP3oool00`000000oooo0?ooo`1b0?ooo`001`3oool010000000oooo
0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`1/0?ooo`030000003oool0oooo07<0oooo00070?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
06`0oooo00<000000?ooo`3oool0L`3oool000P0oooo0P00000:0?ooo`8000001@3oool2000001h0
oooo00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`1d0?ooo`006`3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool0M03oool001/0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo07D0oooo000K
0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`1e
0?ooo`006`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo06L0oooo00<000000?oo
o`3oool0MP3oool001/0oooo0P00000P0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3o
ool0M`3oool001/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`1U0?ooo`030000
003oool0oooo07L0oooo000K0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0I03o
ool00`000000oooo0?ooo`1h0?ooo`006`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo06<0oooo00<000000?ooo`3oool0N03oool001/0oooo0P00000R0?ooo`030000003oool0oooo
0680oooo00<000000?ooo`3oool0N@3oool001/0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`1P0?ooo`030000003oool0oooo07X0oooo000K0?ooo`030000003oool0oooo0280oooo
00<000000?ooo`3oool0H03oool00`000000oooo0?ooo`1j0?ooo`006`3oool00`000000oooo0?oo
o`0S0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0N`3oool001/0oooo0P00000T
0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0N`3oool001/0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo07`0oooo00020?ooo`80
0000103oool00`000000oooo0?ooo`020?ooo`@000000P3oool3000000D0oooo00<000000?ooo`3o
ool0903oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo07`0oooo00001@3oool00000
0?ooo`3oool0000000P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo02D0oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`1m0?ooo`00
00D0oooo0000003oool0oooo000000090?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool00P3oool2000002H0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`1n0?ooo`00
00D0oooo0000003oool0oooo0000000:0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool0OP3oool00005
0?ooo`000000oooo0?ooo`0000002`3oool010000000oooo0?ooo`0000070?ooo`030000003oool0
oooo02H0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`1o0?ooo`0000D0oooo0000
003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0O`3o
ool00080oooo0P00000:0?ooo`8000000`3oool4000000@0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`1E0?ooo`030000003oool0oooo0800oooo000K0?ooo`800000:@3oool00`00
0000oooo0?ooo`1C0?ooo`030000003oool0oooo0840oooo000K0?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`210?ooo`006`3oool00`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo0540oooo00<000000?ooo`3oool0PP3oool001/0oooo00<0
00000?ooo`3oool0:@3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo0880oooo000K
0?ooo`800000:`3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo08<0oooo000K0?oo
o`030000003oool0oooo02X0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`240?oo
o`006`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3o
ool0Q03oool001/0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`1<0?ooo`030000
003oool0oooo08D0oooo000K0?ooo`800000;@3oool00`000000oooo0?ooo`1;0?ooo`030000003o
ool0oooo08D0oooo000K0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0B@3oool0
0`000000oooo0?ooo`260?ooo`006`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo
04P0oooo00<000000?ooo`3oool0Q`3oool001/0oooo00<000000?ooo`3oool0;P3oool00`000000
oooo0?ooo`170?ooo`030000003oool0oooo08L0oooo000K0?ooo`800000;`3oool00`000000oooo
0?ooo`160?ooo`030000003oool0oooo08P0oooo000K0?ooo`030000003oool0oooo02l0oooo00<0
00000?ooo`3oool0A@3oool00`000000oooo0?ooo`280?ooo`006`3oool00`000000oooo0?ooo`0_
0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0R@3oool000P0oooo0P0000040?oo
o`030000003oool0oooo00<0oooo0P0000050?ooo`030000003oool0oooo0300oooo00<000000?oo
o`3oool0@P3oool00`000000oooo0?ooo`2:0?ooo`001`3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`10
0?ooo`030000003oool0oooo08/0oooo00070?ooo`040000003oool0oooo000000/0oooo00<00000
0?ooo`3oool00P3oool200000380oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`2;
0?ooo`001`3oool010000000oooo0?ooo`0000090?ooo`8000001@3oool00`000000oooo0?ooo`0b
0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0S03oool000L0oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0380oooo00<00000
0?ooo`3oool0?@3oool00`000000oooo0?ooo`2=0?ooo`001`3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0k0?ooo`030000003oool0oooo08h0oooo00080?ooo`8000002P3oool2000000D0oooo0P00000e
0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0SP3oool001/0oooo00<000000?oo
o`3oool0=03oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo08l0oooo000K0?ooo`03
0000003oool0oooo03D0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`2@0?ooo`00
6`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0
T@3oool001/0oooo0P00000h0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0T@3o
ool001/0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo0980oooo000K0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0<@3oool00`00
0000oooo0?ooo`2C0?ooo`006`3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo02l0
oooo00<000000?ooo`3oool0U03oool001/0oooo0P00000k0?ooo`030000003oool0oooo02d0oooo
00<000000?ooo`3oool0U@3oool001/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?oo
o`0/0?ooo`030000003oool0oooo09H0oooo000K0?ooo`030000003oool0oooo03/0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`2G0?ooo`006`3oool00`000000oooo0?ooo`0l0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0V03oool001/0oooo00<000000?ooo`3o
ool0?@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo09T0oooo000K0?ooo`800000
?`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo09X0oooo000K0?ooo`030000003o
ool0oooo03l0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`2J0?ooo`000P3oool2
000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo0`0000050?ooo`030000003oool0
oooo0400oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`2K0?ooo`0000D0oooo0000
003oool0oooo000000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`110?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0W03o
ool000050?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0080oooo0P0000130?ooo`8000007@3oool2000009l0oooo00001@3oool000000?ooo`3o
ool0000000T0oooo0P0000030?ooo`<000001@3oool00`000000oooo0?ooo`140?ooo`8000006@3o
ool200000:40oooo00001@3oool000000?ooo`3oool0000000/0oooo00@000000?ooo`3oool00000
1`3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo01@0oooo0P00002S0?ooo`0000D0
oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0140oooo0P00002U0?ooo`00
0P3oool2000000X0oooo0P0000030?ooo`@00000103oool2000004T0oooo0`00000>0?ooo`800000
Y`3oool001/0oooo00<000000?ooo`3oool0B`3oool4000000L0oooo0`00002Y0?ooo`006`3oool0
0`000000oooo0?ooo`1?0?ooo`L00000[03oool001/0oooo00<000000?ooo`3oool0o`3oool30?oo
o`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`800000o`3oool40?ooo`006`3o
ool00`000000oooo0?ooo`3o0?ooo`<0oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.107864, -0.0353283, \
0.00394728, 0.00234279}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{\(2.2900000000000205`\ Second\), ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Plot[Imgamma[1, 1.0, 8/10, om, 1], {om, 1/10, 1}]]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\(0.1000000375` \
\[InvisibleSpace]\)\) + \(\(xi\\ 0.8`\)\))\)\)\\ \(\((\(\(2\\ \((xi - \(\(\
\[LeftSkeleton] 1 \[RightSkeleton]\)\))\)\^2\)\) - \(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\))\)\)\)\/\(\((1 + \(\(1\\ xi\^2\)\))\)\^2\\ \
\@\(\(\(\(\((1 - 0.8`\^2\/1.`\^2)\)\)\\ \(\((1 - \(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/\(\[LeftSkeleton] 3 \[RightSkeleton]\)\^2)\)\)\)\) - \
\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\^2\)\)\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((\(\(0.1365102924156242` \[InvisibleSpace]\)\) + \(\(xi\\ \
0.8`\)\))\)\)\\ \(\((\(\(2\\ \((\(\[LeftSkeleton] 1 \
\[RightSkeleton]\))\)\^2\)\) - \(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\))\)\)\)\/\(\((1 + \(\(1\\ xi\^2\)\))\)\^2\\ \
\@\(\(\(\(\((1 - 0.8`\^2\/1.`\^2)\)\)\\ \(\((1 - \(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/\(\[LeftSkeleton] 3 \[RightSkeleton]\)\^2)\)\)\)\) - \
\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\^2\)\)\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.4851749576394496`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((\(\(0.1365102924156242` \[InvisibleSpace]\)\) + \(xi\\ \
4\)\/5)\)\)\\ \(\((\(\(2\\ \((\(\[LeftSkeleton] 1 \
\[RightSkeleton]\))\)\^2\)\) - \(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\))\)\)\)\/\(\((1 + \(\(1\\ xi\^2\)\))\)\^2\\ \
\@\(\(\(\(\((1 - \((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\^2\/1.`\^2)\)\)\
\\ \(\((1 - \(\[LeftSkeleton] 1 \[RightSkeleton]\)\/\(\[LeftSkeleton] 3 \
\[RightSkeleton]\)\^2)\)\)\)\) - \(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\^2\)\)\)) is less than WorkingPrecision (\!\(30\))."\)], \
"Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.7035914255044484`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.4851749576394496`\)."\
\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(Plot::"plnr" \(\(:\)\(\ \)\) 
      "\!\(Imgamma[\(\(1, 1.`, 8\/10, om, 1\)\)]\) is not a machine-size real \
number at \!\(om\) = \!\(0.1365102924156242`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(Plot::"plnr" \(\(:\)\(\ \)\) 
      "\!\(Imgamma[\(\(1, 1.`, 8\/10, om, 1\)\)]\) is not a machine-size real \
number at \!\(om\) = \!\(0.11777335368671815`\)."\)], "Message"],

Cell[BoxData[
    \(Plot::"plnr" \(\(:\)\(\ \)\) 
      "\!\(Imgamma[\(\(1, 1.`, 8\/10, om, 1\)\)]\) is not a machine-size real \
number at \!\(om\) = \!\(0.10848242816171107`\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(Plot :: \"plnr\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imgamma[1, 1.0, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((1\/2 + \(xi\\ \
4\)\/5)\)\)\\ \(\((\(\(2\\ \((xi - 4\/\(\[LeftSkeleton] 1 \[RightSkeleton]\))\
\)\^2\)\) - \(\(\(\((1 - \(\[LeftSkeleton] 1 \[RightSkeleton]\)\^2\/\(\
\[LeftSkeleton] 3 \[RightSkeleton]\)\^2)\)\)\\ \(\((1 - \(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\))\)\)\)\))\)\)\)\/\(\((1 + \(\(1\\ xi\^2\)\))\)\^2\\ \@\(\
\(\(\(\((1 - \((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\^2\/1.`\^2)\)\)\\ \
\(\((1 - \(\[LeftSkeleton] 1 \[RightSkeleton]\)\^2\/\(\[LeftSkeleton] 3 \
\[RightSkeleton]\)\^2)\)\)\)\) - \(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\^2\)\)\)) is less than WorkingPrecision (\!\(30\))."\)], \
"Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.9196152422706632`\)."\)], "Message"],

Cell[BoxData[
    \(\(\(0.27306621809872256`\)\(\[InvisibleSpace]\)\) + 
      1.7809449902248383`*^-8\ \[ImaginaryI]\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImgamma[1, 1.0, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(0.27306624713288663`\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImgamma[1, 1.0, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(0.2730664113356185`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rearg7 = 
      Compile[{md2, k, kz, om, a, 
          xi}, \((om^2 + a\ k^2\ xi^2 - 
              2\ k^2/\((k^2 - kz^2)\)\ \((om^2 - a\ kz^2\ xi^2)\))\)/\((1 + 
                a\ xi^2)\)^2]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(om\^2 + a\ k\^2\ xi\^2 - \(2\ k\^2\ \((om\^2 - a\ kz\^2\ \
xi\^2)\)\)\/\(k\^2 - kz\^2\)\)\/\((1 + a\ xi\^2)\)\^2, "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rearg8 = 
      Compile[{md2, k, kz, om, a, 
          xi}, \((om + 
                a\ kz\ xi)\)/\((1 + 
                  a\ xi^2)\)^2\ *\((2\ \((xi - om\ kz/k^2)\)^2 - \((1 - 
                    om^2/k^2)\)\ \((1 - kz^2/k^2)\))\)*
          Sign[om - kz\ xi]/
            Sqrt[\((xi - om\ kz/k^2)\)^2 - \((1 - om^2/k^2)\)\ \((1 - 
                      kz^2/k^2)\)]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(\((om + a\ kz\ xi)\)\ \((2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \
\((1 - om\^2\/k\^2)\)\ \((1 - kz\^2\/k\^2)\))\)\ Sign[om - kz\ xi]\)\/\(\((1 \
+ a\ xi\^2)\)\^2\ \@\(\((xi - \(om\ kz\)\/k\^2)\)\^2 - \((1 - om\^2\/k\^2)\)\ \
\((1 - kz\^2\/k\^2)\)\)\), "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(FastRegamma[md2_, k_, kz_, om_, a_] := 
      Module[{res, mxi1, mxi2, wp}, 
        If[logging \[Equal] 0, 
          Print["\<gamma:\>", om]]; \[IndentingNewLine]mxi1 = 
          If[k > om, xi1[k, kz, om], 0]; 
        mxi2 = If[k > om, xi2[k, kz, om], 0]; \ \[IndentingNewLine]If[om < k, 
          res = md2/
                2*\((1/k^2*
                    NIntegrate[
                      Rearg7[md2, k, kz, om, a, xi], {xi, \(-1\), 1}] + 
                  k/\((k^2 - kz^2)\)*\((NIntegrate[
                          Rearg8[md2, k, kz, om, a, xi], {xi, \(-1\), 
                            mxi2*If[
                                mxi2 < 
                                  0, \((1 + 10^\((\(-13\))\))\), \((1 - 
                                    10^\((\(-13\))\))\)]}, 
                          WorkingPrecision \[Rule] 20] + 
                        NIntegrate[
                          Rearg8[md2, k, kz, om, a, xi], {xi, 
                            mxi1*\((1 + 10^\((\(-15\))\))\), 1}, 
                          WorkingPrecision \[Rule] 20])\))\), 
          res = md2/
                2*\((1/k^2*
                    NIntegrate[
                      Rearg7[md2, k, kz, om, a, xi], {xi, \(-1\), 1}, 
                      WorkingPrecision \[Rule] 20] + 
                  k/\((k^2 - kz^2)\)*\((NIntegrate[
                        Rearg8[md2, k, kz, om, a, xi], {xi, \(-1\), 
                          1}])\))\), WorkingPrecision \[Rule] 20]; 
        res]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRegamma[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.01581009481364061`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Regamma[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.015810095839027834934103187500932`18.8726\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRegamma[1, 1, 1 - 10^\((\(-4\))\), 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(0.00012849225367972394`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Regamma[1, 1, 1 - 10^\((\(-4\))\), 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(0.0001284831592972693169986101`13.3448\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fgamma[1, 1, 1 - 10^\((\(-4\))\), 100, 1]\)], "Input"],

Cell[BoxData[
    \(8.38935375213623`*^-6\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fgamma[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message",
  GeneratedCell->False,
  CellAutoOverwrite->False],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message",
  GeneratedCell->False,
  CellAutoOverwrite->False],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message",
  GeneratedCell->False,
  CellAutoOverwrite->False],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message",
  GeneratedCell->False,
  CellAutoOverwrite->False],

Cell[BoxData[
    \(\(-0.01581009481364061`\) + 
      0.27306625665819584`\ \[ImaginaryI]\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fgamma[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(-0.11961524227066439`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.9196152422707682`\)."\)], "Message"],

Cell[BoxData[
    \(\(-0.01581009785604144`\) + 
      0.27306641497217743`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Fgamma[md2_, k_, kz_, om_, a_] := 
      FastRegamma[md2, k, kz, om, a] + 
        I*FastImgamma[md2, k, kz, om, a]\)], "Input",
  InitializationCell->True]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["delta", "Section"],

Cell[CellGroupData[{

Cell["small a", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(toorder = 5\)], "Input"],

Cell[BoxData[
    \(5\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(Intexpdelta = 
        Series[1/\((1 + a\ \((vn^2)\))\)^2\ \((k\ xi\  + \ 
                a\ vn\ k\ nz)\)\ \((k^2\ vn\  - \ 
                nz\ k\ k\ xi)\)\ om/\((om - k\ xi)\), {a, 0, 
            toorder}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(a0delta = 
      Simplify[\(md2/k^4\)/\((1 - nz^2)\)*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexpdelta[\([3]\)]\)[\([1]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a1delta = 
      Simplify[\(md2/k^4\)/\((1 - nz^2)\)*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexpdelta[\([3]\)]\)[\([2]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(md2\ nz\ om\ \((\(-22\)\ k\^3\ om + 24\ k\ om\^3 + 
                3\ \((k\^4 - 5\ k\^2\ om\^2 + 4\ om\^4)\)\ Log[\(-k\) + om] - 
                3\ \((k\^4 - 5\ k\^2\ om\^2 + 4\ om\^4)\)\ Log[
                    k + om])\)\)\/\(12\ k\^6\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[\(1/k^4\)/\((1 - nz^2)\)*1/2*\(1/2\)/Pi*
        Integrate[\(Intexpdelta[\([3]\)]\)[\([2]\)], {phi, 0, 
            2*Pi}]]\)], "Input"],

Cell[BoxData[
    \(\(nz\ om\ \((1 - 5\ xi\^2 + 4\ xi\^4)\)\)\/\(4\ k\ \((om - k\ \
xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apart[%]\)], "Input"],

Cell[BoxData[
    \(\(nz\ om\^2\ \((5\ k\^2 - 4\ om\^2)\)\)\/\(4\ k\^5\) + \(nz\ om\ \((5\ \
k\^2 - 4\ om\^2)\)\ xi\)\/\(4\ k\^4\) - \(nz\ om\^2\ xi\^2\)\/k\^3 - \(nz\ om\
\ xi\^3\)\/k\^2 + \(\(-k\^4\)\ nz\ om + 5\ k\^2\ nz\ om\^3 - 4\ nz\ om\^5\)\/\
\(4\ k\^5\ \((\(-om\) + k\ xi)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a2delta = 
      Simplify[\(md2/k^4\)/\((1 - nz^2)\)*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexpdelta[\([3]\)]\)[\([3]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(40\ k\^8\)\)\((md2\ nz\ om\ \((2\ k\ om\ \((k\^4\ \((\(-73\) + 
                        97\ nz\^2)\) - 
                  5\ k\^2\ \((\(-33\) + 61\ nz\^2)\)\ om\^2 + 
                  30\ \((\(-3\) + 7\ nz\^2)\)\ om\^4)\) - 
            15\ \((k\^2 - om\^2)\)\ \((k\^4\ \((\(-1\) + nz\^2)\) + 
                  k\^2\ \((7 - 11\ nz\^2)\)\ om\^2 + 
                  2\ \((\(-3\) + 7\ nz\^2)\)\ om\^4)\)\ Log[\(-k\) + om] + 
            15\ \((k\^2 - om\^2)\)\ \((k\^4\ \((\(-1\) + nz\^2)\) + 
                  k\^2\ \((7 - 11\ nz\^2)\)\ om\^2 + 
                  2\ \((\(-3\) + 7\ nz\^2)\)\ om\^4)\)\ Log[
                k + om])\))\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a3delta = 
      Simplify[\(md2/k^4\)/\((1 - nz^2)\)*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexpdelta[\([3]\)]\)[\([4]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(1120\ k\^10\)\)\((md2\ nz\ om\ \((2\ k\ om\ \((k\^6\ \
\((\(-3075\) + 8230\ nz\^2 - 4979\ nz\^4)\) + 
                    28\ k\^4\ \((380 - 1455\ nz\^2 + 1143\ nz\^4)\)\ om\^2 - 
                    35\ k\^2\ \((335 - 1650\ nz\^2 + 1563\ nz\^4)\)\ om\^4 + 
                    840\ \((5 - 30\ nz\^2 + 33\ nz\^4)\)\ om\^6)\) + 
              105\ \((k\^2 - om\^2)\)\ \((5\ k\^6\ \((\(-1\) + nz\^2)\)\^2 - 
                    10\ k\^4\ \((5 - 16\ nz\^2 + 11\ nz\^4)\)\ om\^2 + 
                    5\ k\^2\ \((17 - 78\ nz\^2 + 69\ nz\^4)\)\ om\^4 - 
                    8\ \((5 - 30\ nz\^2 + 
                          33\ nz\^4)\)\ om\^6)\)\ Log[\(-k\) + om] - 
              105\ \((k\^2 - om\^2)\)\ \((5\ k\^6\ \((\(-1\) + nz\^2)\)\^2 - 
                    10\ k\^4\ \((5 - 16\ nz\^2 + 11\ nz\^4)\)\ om\^2 + 
                    5\ k\^2\ \((17 - 78\ nz\^2 + 69\ nz\^4)\)\ om\^4 - 
                    8\ \((5 - 30\ nz\^2 + 33\ nz\^4)\)\ om\^6)\)\ Log[
                  k + om])\))\)\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a4delta = 
      Simplify[\(md2/k^4\)/\((1 - nz^2)\)*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexpdelta[\([3]\)]\)[\([5]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(20160\ k\^12\)\)\((md2\ nz\ om\ \((2\ k\ om\ \((k\^8\ \
\((\(-74095\) + 298445\ nz\^2 - 361165\ nz\^4 + 138479\ nz\^6)\) - 
                  3\ k\^6\ \((\(-115535\) + 675045\ nz\^2 - 1070685\ nz\^4 + 
                        508743\ nz\^6)\)\ om\^2 + 
                  105\ k\^4\ \((\(-5495\) + 41685\ nz\^2 - 80661\ nz\^4 + 
                        45111\ nz\^6)\)\ om\^4 - 
                  105\ k\^2\ \((\(-3955\) + 36785\ nz\^2 - 83545\ nz\^4 + 
                        53339\ nz\^6)\)\ om\^6 + 
                  3150\ \((\(-35\) + 385\ nz\^2 - 1001\ nz\^4 + 
                        715\ nz\^6)\)\ om\^8)\) - 
            315\ \((k\^2 - om\^2)\)\ \((35\ k\^8\ \((\(-1\) + nz\^2)\)\^3 - 
                  35\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((\(-13\) + 
                        37\ nz\^2)\)\ om\^2 + 
                  35\ k\^4\ \((\(-33\) + 227\ nz\^2 - 403\ nz\^4 + 
                        209\ nz\^6)\)\ om\^4 - 
                  7\ k\^2\ \((\(-155\) + 1385\ nz\^2 - 3025\ nz\^4 + 
                        1859\ nz\^6)\)\ om\^6 + 
                  10\ \((\(-35\) + 385\ nz\^2 - 1001\ nz\^4 + 
                        715\ nz\^6)\)\ om\^8)\)\ Log[\(-k\) + om] + 
            315\ \((k\^2 - om\^2)\)\ \((35\ k\^8\ \((\(-1\) + nz\^2)\)\^3 - 
                  35\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((\(-13\) + 
                        37\ nz\^2)\)\ om\^2 + 
                  35\ k\^4\ \((\(-33\) + 227\ nz\^2 - 403\ nz\^4 + 
                        209\ nz\^6)\)\ om\^4 - 
                  7\ k\^2\ \((\(-155\) + 1385\ nz\^2 - 3025\ nz\^4 + 
                        1859\ nz\^6)\)\ om\^6 + 
                  10\ \((\(-35\) + 385\ nz\^2 - 1001\ nz\^4 + 
                        715\ nz\^6)\)\ om\^8)\)\ Log[
                k + om])\))\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a5delta = 
      Simplify[\(md2/k^4\)/\((1 - nz^2)\)*1/2*
          Integrate[\(1/2\)/Pi*
              Integrate[\(Intexpdelta[\([3]\)]\)[\([6]\)], {phi, 0, 
                  2*Pi}], {xi, \(-1\), \ 1}]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(118272\ k\^14\)\)\((md2\ nz\ om\ \((\(-2\)\ k\ om\ \((k\^10\
\ \((545475 - 2934540\ nz\^2 + 5326482\ nz\^4 - 4083468\ nz\^6 + 
                          1139651\ nz\^8)\) - 
                    22\ k\^8\ \((146475 - 1152480\ nz\^2 + 2755998\ nz\^4 - 
                          2626632\ nz\^6 + 877535\ nz\^8)\)\ om\^2 + 
                    792\ k\^6\ \((9079 - 93261\ nz\^2 + 273777\ nz\^4 - 
                          308823\ nz\^6 + 119132\ nz\^8)\)\ om\^4 - 
                    462\ k\^4\ \((16863 - 213192\ nz\^2 + 738654\ nz\^4 - 
                          957216\ nz\^6 + 416299\ nz\^8)\)\ om\^6 + 
                    1155\ k\^2\ \((3591 - 53844\ nz\^2 + 214578\ nz\^4 - 
                          313092\ nz\^6 + 150943\ nz\^8)\)\ om\^8 - 
                    13860\ \((63 - 1092\ nz\^2 + 4914\ nz\^4 - 7956\ nz\^6 + 
                          4199\ nz\^8)\)\ om\^10)\) + 
              3465\ \((k\^2 - 
                    om\^2)\)\ \((21\ k\^10\ \((\(-1\) + nz\^2)\)\^4 - 
                    168\ k\^8\ \((\(-1\) + nz\^2)\)\^3\ \((\(-2\) + 
                          7\ nz\^2)\)\ om\^2 + 
                    42\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((27 - 194\ nz\^2 + 
                          247\ nz\^4)\)\ om\^4 - 
                    84\ k\^4\ \((19 - 226\ nz\^2 + 740\ nz\^4 - 910\ nz\^6 + 
                          377\ nz\^8)\)\ om\^6 + 
                    3\ k\^2\ \((343 - 5012\ nz\^2 + 19474\ nz\^4 - 
                          27716\ nz\^6 + 13039\ nz\^8)\)\ om\^8 - 
                    4\ \((63 - 1092\ nz\^2 + 4914\ nz\^4 - 7956\ nz\^6 + 
                          4199\ nz\^8)\)\ om\^10)\)\ Log[\(-k\) + om] - 
              3465\ \((k\^2 - 
                    om\^2)\)\ \((21\ k\^10\ \((\(-1\) + nz\^2)\)\^4 - 
                    168\ k\^8\ \((\(-1\) + nz\^2)\)\^3\ \((\(-2\) + 
                          7\ nz\^2)\)\ om\^2 + 
                    42\ k\^6\ \((\(-1\) + nz\^2)\)\^2\ \((27 - 194\ nz\^2 + 
                          247\ nz\^4)\)\ om\^4 - 
                    84\ k\^4\ \((19 - 226\ nz\^2 + 740\ nz\^4 - 910\ nz\^6 + 
                          377\ nz\^8)\)\ om\^6 + 
                    3\ k\^2\ \((343 - 5012\ nz\^2 + 19474\ nz\^4 - 
                          27716\ nz\^6 + 13039\ nz\^8)\)\ om\^8 - 
                    4\ \((63 - 1092\ nz\^2 + 4914\ nz\^4 - 7956\ nz\^6 + 
                          4199\ nz\^8)\)\ om\^10)\)\ Log[
                  k + om])\))\)\)\)\)], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exact results -numerical ", "Subsection"],

Cell[CellGroupData[{

Cell["Preliminaries", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(s = Sqrt[\((1 - om^2/k^2)\)*\((1 - kz^2/k^2)\)]\)], "Input"],

Cell[BoxData[
    \(\@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi1[k_, kz_, om_] = om*kz/k^2 + s\)], "Input"],

Cell[BoxData[
    \(\(kz\ om\)\/k\^2 + \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi2[k_, kz_, om_] = om*kz/k^2 - s\)], "Input"],

Cell[BoxData[
    \(\(kz\ om\)\/k\^2 - \@\(\((1 - kz\^2\/k\^2)\)\ \((1 - \
om\^2\/k\^2)\)\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Imaginary part:", "Subsubsection"],

Cell[BoxData[
    \(Imdelta[md2_, k_, kz_, om_, a_] := 
      If[om > k, 
        0, \(\(-md2\)/k^4\)/\((1 - kz^2/k^2)\)*om/2*
          NIntegrate[
            1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*\((k^2\ xi - kz\ om)\)/
                Sqrt[\(-\((k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                              om^2)\)*\((1 - kz^2/k^2)\))\)\)], {xi, 
              xi2[k, kz, om]*
                If[xi2[k, kz, om] > 
                    0, \((1 + 10^\((\(-50\))\))\), \((1 - 
                      10^\((\(-50\))\))\)], xi1[k, kz, om]}, 
            WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30]]\)], "Input",\

  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imdelta[1, 1, 1 - 10^\((\(-15\))\), 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(\(-0.00007853980770036501002924503325887589`19.8954\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imdelta[1, 1, 1/10, 1/2, 0]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(0``19.5658\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imdelta[1, 1, 9/10, 1/2, 1/10]\)], "Input"],

Cell[BoxData[
    \(0.00119374538930514031473`10.0003\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[xi2[1, 9/10, 1/2]]\)], "Input"],

Cell[BoxData[
    \(0.0725082782364625`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImTable1delta = 
      Table[{N[a], Imdelta[1, 1, 1/10, 1/2, a]}, {a, \(-1\), 1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \({{\(-1.`\), \(-0.573305951027966965115584`10.0631\)}, {\(-0.9`\), \
\(-0.22453743895633585250091`10.082\)}, {\(-0.8`\), \
\(-0.10289988748188690561343`10.2332\)}, {\(-0.7`\), \
\(-0.050764878453578466998382`10.0619\)}, {\(-0.6`\), \
\(-0.025765151606504286684726`10.3369\)}, {\(-0.5`\), \
\(-0.012992211075767218011443`10.1431\)}, {\(-0.4`\), \
\(-0.006269319395815941645347`10.0864\)}, {\(-0.3`\), \
\(-0.002738506661713966974069`10.0581\)}, {\(-0.2`\), \
\(-0.000967914505709100228476`10.0691\)}, {\(-0.1`\), \
\(-0.000196248201217952608296`10.1031\)}, {0.`, 
        0``19.5658}, {0.1`, \(-0.000135291775437107671427`10.0727\)}, {0.2`, \
\(-0.000458007291372561536687`10.0106\)}, {0.3`, \
\(-0.000881416805497722646328`10.0129\)}, {0.4`, \
\(-0.001352755714547056081341`10.1611\)}, {0.5`, \
\(-0.00183982898655162182393`10.1052\)}, {0.6`, \
\(-0.00232312593852185095346`10.0937\)}, {0.7`, \
\(-0.00279106448907672631241`10.2164\)}, {0.8`, \
\(-0.003237063606484296431047`10.3217\)}, {0.9`, \
\(-0.00365771017226861495215`10.414\)}, {1.`, \
\(-0.004051595024487453670138`10.0572\)}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImPlot1delta = 
      ListPlot[ImTable1delta, PlotJoined \[Rule] True, 
        PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.47619 0.603319 4.66312 [
[.02381 .59082 -6 -9 ]
[.02381 .59082 6 0 ]
[.2619 .59082 -12 -9 ]
[.2619 .59082 12 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.4875 .04374 -30 -4.5 ]
[.4875 .04374 0 4.5 ]
[.4875 .13701 -24 -4.5 ]
[.4875 .13701 0 4.5 ]
[.4875 .23027 -30 -4.5 ]
[.4875 .23027 0 4.5 ]
[.4875 .32353 -30 -4.5 ]
[.4875 .32353 0 4.5 ]
[.4875 .41679 -30 -4.5 ]
[.4875 .41679 0 4.5 ]
[.4875 .51006 -30 -4.5 ]
[.4875 .51006 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60332 m
.02381 .60957 L
s
[(-1)] .02381 .59082 0 1 Mshowa
.2619 .60332 m
.2619 .60957 L
s
[(-0.5)] .2619 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.5)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(1)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.5 .04374 m
.50625 .04374 L
s
[(-0.12)] .4875 .04374 1 0 Mshowa
.5 .13701 m
.50625 .13701 L
s
[(-0.1)] .4875 .13701 1 0 Mshowa
.5 .23027 m
.50625 .23027 L
s
[(-0.08)] .4875 .23027 1 0 Mshowa
.5 .32353 m
.50625 .32353 L
s
[(-0.06)] .4875 .32353 1 0 Mshowa
.5 .41679 m
.50625 .41679 L
s
[(-0.04)] .4875 .41679 1 0 Mshowa
.5 .51006 m
.50625 .51006 L
s
[(-0.02)] .4875 .51006 1 0 Mshowa
.125 Mabswid
.5 .06706 m
.50375 .06706 L
s
.5 .09038 m
.50375 .09038 L
s
.5 .11369 m
.50375 .11369 L
s
.5 .16032 m
.50375 .16032 L
s
.5 .18364 m
.50375 .18364 L
s
.5 .20695 m
.50375 .20695 L
s
.5 .25358 m
.50375 .25358 L
s
.5 .2769 m
.50375 .2769 L
s
.5 .30022 m
.50375 .30022 L
s
.5 .34685 m
.50375 .34685 L
s
.5 .37016 m
.50375 .37016 L
s
.5 .39348 m
.50375 .39348 L
s
.5 .44011 m
.50375 .44011 L
s
.5 .46343 m
.50375 .46343 L
s
.5 .48674 m
.50375 .48674 L
s
.5 .53337 m
.50375 .53337 L
s
.5 .55669 m
.50375 .55669 L
s
.5 .58 m
.50375 .58 L
s
.5 .02043 m
.50375 .02043 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.10868 0 m
.11905 .12348 L
.16667 .3666 L
.21429 .48317 L
.2619 .54273 L
.30952 .57408 L
.35714 .59055 L
.40476 .59881 L
.45238 .6024 L
.5 .60332 L
.54762 .60269 L
.59524 .60118 L
.64286 .59921 L
.69048 .59701 L
.7381 .59474 L
.78571 .59249 L
.83333 .5903 L
.88095 .58822 L
.92857 .58626 L
.97619 .58443 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003o7`3oool0
0`000?l0oooo0?ooo`1]0?ooo`030000003oool0oooo08h0oooo000O0?ooo`030000o`3oool0oooo
06d0oooo00<000000?ooo`3oool0SP3oool001l0oooo00<0003o0?ooo`3oool0K@3oool00`000000
oooo0?ooo`2>0?ooo`007`3oool00`000?l0oooo0?ooo`1]0?ooo`030000003oool0oooo08h0oooo
000O0?ooo`030000o`3oool0oooo06d0oooo00<000000?ooo`3oool0SP3oool001l0oooo00<0003o
0?ooo`3oool0K@3oool2000008l0oooo000P0?ooo`030000o`3oool0oooo06`0oooo00<000000?oo
o`3oool0SP3oool00200oooo00<0003o0?ooo`3oool0K03oool00`000000oooo0?ooo`2>0?ooo`00
803oool00`000?l0oooo0?ooo`1/0?ooo`030000003oool0oooo08h0oooo000P0?ooo`030000o`3o
ool0oooo06`0oooo00<000000?ooo`3oool0SP3oool00200oooo00<0003o0?ooo`3oool0DP3oool2
000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo100000050?ooo`030000003oool0
oooo08h0oooo000P0?ooo`030000o`3oool0oooo0540oooo00@000000?ooo`3oool000002P3oool0
1@000000oooo0?ooo`3oool0000000P0oooo00<000000?ooo`3oool0SP3oool00200oooo00<0003o
0?ooo`3oool0D@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool01@3oool3000008h0oooo000P0?ooo`030000o`3oool0oooo04/0oooo10000002
0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo08h0oooo000P0?ooo`030000o`3oool0oooo0540oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0SP3oool00200oooo00<0003o0?ooo`3oool0D@3oool010000000oooo0?ooo`00
00080?ooo`<000000`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo
000P0?ooo`030000o`3oool0oooo0580oooo0P00000;0?ooo`030000003oool0oooo0080oooo0P00
00060?ooo`030000003oool0oooo08h0oooo000P0?ooo`030000o`3oool0oooo06`0oooo00<00000
0?ooo`3oool0SP3oool00240oooo00<0003o0?ooo`3oool0J`3oool00`000000oooo0?ooo`2>0?oo
o`008@3oool00`000?l0oooo0?ooo`1[0?ooo`800000S`3oool00240oooo00<0003o0?ooo`3oool0
J`3oool00`000000oooo0?ooo`2>0?ooo`008@3oool00`000?l0oooo0?ooo`1[0?ooo`030000003o
ool0oooo08h0oooo000Q0?ooo`030000o`3oool0oooo06/0oooo00<000000?ooo`3oool0SP3oool0
0240oooo00<0003o0?ooo`3oool0J`3oool00`000000oooo0?ooo`2>0?ooo`008@3oool00`000?l0
oooo0?ooo`1[0?ooo`030000003oool0oooo08h0oooo000Q0?ooo`030000o`3oool0oooo06/0oooo
0P00002?0?ooo`008@3oool00`000?l0oooo0?ooo`1[0?ooo`030000003oool0oooo08h0oooo000Q
0?ooo`030000o`3oool0oooo06/0oooo00<000000?ooo`3oool0SP3oool00240oooo00<0003o0?oo
o`3oool0J`3oool00`000000oooo0?ooo`2>0?ooo`008@3oool00`000?l0oooo0?ooo`1[0?ooo`03
0000003oool0oooo08h0oooo000R0?ooo`030000o`3oool0oooo06X0oooo00<000000?ooo`3oool0
SP3oool00280oooo00<0003o0?ooo`3oool0JP3oool00`000000oooo0?ooo`2>0?ooo`008P3oool0
0`000?l0oooo0?ooo`1Z0?ooo`800000S`3oool00280oooo00<0003o0?ooo`3oool0JP3oool00`00
0000oooo0?ooo`2>0?ooo`008P3oool00`000?l0oooo0?ooo`1Z0?ooo`030000003oool0oooo08h0
oooo000R0?ooo`030000o`3oool0oooo06X0oooo00<000000?ooo`3oool0SP3oool00280oooo00<0
003o0?ooo`3oool0JP3oool00`000000oooo0?ooo`2>0?ooo`008P3oool00`000?l0oooo0?ooo`1F
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000001@3oool00`000000oooo0?ooo`2>
0?ooo`008`3oool00`000?l0oooo0?ooo`1D0?ooo`040000003oool0oooo000000X0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`008`3oool00`000?l0oooo0?ooo`1D0?oo
o`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool3000008h0oooo000S0?oo
o`030000o`3oool0oooo04h0oooo100000020?ooo`040000003oool0oooo000000X0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`008`3oool00`000?l0oooo0?ooo`1D0?oo
o`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>
0?ooo`008`3oool00`000?l0oooo0?ooo`1D0?ooo`040000003oool0oooo000000P0oooo0`000006
0?ooo`030000003oool0oooo08h0oooo000T0?ooo`030000o`3oool0oooo05@0oooo0P00000;0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool002@0oooo00<0003o0?ooo`3o
ool0J03oool00`000000oooo0?ooo`2>0?ooo`00903oool00`000?l0oooo0?ooo`1X0?ooo`800000
S`3oool002@0oooo00<0003o0?ooo`3oool0J03oool00`000000oooo0?ooo`2>0?ooo`00903oool0
0`000?l0oooo0?ooo`1X0?ooo`030000003oool0oooo08h0oooo000U0?ooo`030000o`3oool0oooo
06L0oooo00<000000?ooo`3oool0SP3oool002D0oooo00<0003o0?ooo`3oool0I`3oool00`000000
oooo0?ooo`2>0?ooo`009@3oool00`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo08h0oooo
000U0?ooo`030000o`3oool0oooo06L0oooo00<000000?ooo`3oool0SP3oool002D0oooo00<0003o
0?ooo`3oool0I`3oool2000008l0oooo000V0?ooo`030000o`3oool0oooo06H0oooo00<000000?oo
o`3oool0SP3oool002H0oooo00<0003o0?ooo`3oool0IP3oool00`000000oooo0?ooo`2>0?ooo`00
9P3oool00`000?l0oooo0?ooo`1V0?ooo`030000003oool0oooo08h0oooo000V0?ooo`030000o`3o
ool0oooo06H0oooo00<000000?ooo`3oool0SP3oool002H0oooo00<0003o0?ooo`3oool0IP3oool0
0`000000oooo0?ooo`2>0?ooo`009`3oool00`000?l0oooo0?ooo`1U0?ooo`030000003oool0oooo
08h0oooo000W0?ooo`030000o`3oool0oooo06D0oooo0P00002?0?ooo`009`3oool00`000?l0oooo
0?ooo`1U0?ooo`030000003oool0oooo08h0oooo000W0?ooo`030000o`3oool0oooo06D0oooo00<0
00000?ooo`3oool0SP3oool002L0oooo00<0003o0?ooo`3oool0I@3oool00`000000oooo0?ooo`2>
0?ooo`00:03oool00`000?l0oooo0?ooo`1:0?ooo`800000103oool00`000000oooo0?ooo`030?oo
o`800000103oool2000000H0oooo00<000000?ooo`3oool0SP3oool002P0oooo00<0003o0?ooo`3o
ool0B@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@00000
0?ooo`3oool000001@3oool00`000000oooo0?ooo`2>0?ooo`00:03oool00`000?l0oooo0?ooo`19
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`0000050?ooo`<00000SP3oool002P0oooo00<0003o0?ooo`3oool0@`3oool400000080oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?ooo`8000001P3oool00`00
0000oooo0?ooo`2>0?ooo`00:03oool00`000?l0oooo0?ooo`190?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000003o
ool0oooo08h0oooo000Y0?ooo`030000o`3oool0oooo04P0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3o
ool0SP3oool002T0oooo00<0003o0?ooo`3oool0B@3oool2000000X0oooo0P0000040?ooo`800000
1P3oool00`000000oooo0?ooo`2>0?ooo`00:@3oool00`000?l0oooo0?ooo`1S0?ooo`030000003o
ool0oooo08h0oooo000Y0?ooo`030000o`3oool0oooo06<0oooo00<000000?ooo`3oool0SP3oool0
02T0oooo00<0003o0?ooo`3oool0H`3oool2000008l0oooo000Z0?ooo`030000o`3oool0oooo0680
oooo00<000000?ooo`3oool0SP3oool002X0oooo00<0003o0?ooo`3oool0HP3oool00`000000oooo
0?ooo`2>0?ooo`00:P3oool00`000?l0oooo0?ooo`1R0?ooo`030000003oool0oooo08h0oooo000Z
0?ooo`030000o`3oool0oooo0680oooo00<000000?ooo`3oool0SP3oool002X0oooo00<0003o0?oo
o`3oool0HP3oool00`000000oooo0?ooo`2>0?ooo`00:`3oool00`000?l0oooo0?ooo`1Q0?ooo`03
0000003oool0oooo08h0oooo000[0?ooo`030000o`3oool0oooo0640oooo0P00002?0?ooo`00:`3o
ool00`000?l0oooo0?ooo`1Q0?ooo`030000003oool0oooo08h0oooo000[0?ooo`030000o`3oool0
oooo0640oooo00<000000?ooo`3oool0SP3oool002/0oooo00<0003o0?ooo`3oool0H@3oool00`00
0000oooo0?ooo`2>0?ooo`00;03oool00`000?l0oooo0?ooo`1P0?ooo`030000003oool0oooo08h0
oooo000/0?ooo`030000o`3oool0oooo0600oooo00<000000?ooo`3oool0SP3oool002`0oooo00<0
003o0?ooo`3oool0H03oool00`000000oooo0?ooo`2>0?ooo`00;03oool00`000?l0oooo0?ooo`1P
0?ooo`800000S`3oool002`0oooo00<0003o0?ooo`3oool0H03oool00`000000oooo0?ooo`2>0?oo
o`00;@3oool00`000?l0oooo0?ooo`1O0?ooo`030000003oool0oooo08h0oooo000]0?ooo`030000
o`3oool0oooo05l0oooo00<000000?ooo`3oool0SP3oool002d0oooo00<0003o0?ooo`3oool0A@3o
ool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P0000060?ooo`030000003o
ool0oooo08h0oooo000]0?ooo`030000o`3oool0oooo04@0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3o
ool0SP3oool002d0oooo00<0003o0?ooo`3oool0A03oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool3000008h0oooo000^0?oo
o`030000o`3oool0oooo03d0oooo100000020?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool000000P3oool3000000H0oooo00<000000?ooo`3oool0SP3oool002h0oooo00<0003o
0?ooo`3oool0@`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2>0?ooo`00;P3oool00`000?l0oooo0?oo
o`130?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo000^0?ooo`030000o`3oool0oooo04@0oooo
0P00000:0?ooo`800000103oool3000000D0oooo00<000000?ooo`3oool0SP3oool002h0oooo00<0
003o0?ooo`3oool0GP3oool00`000000oooo0?ooo`2>0?ooo`00;`3oool00`000?l0oooo0?ooo`1M
0?ooo`030000003oool0oooo08h0oooo000_0?ooo`030000o`3oool0oooo05d0oooo0P00002?0?oo
o`00;`3oool00`000?l0oooo0?ooo`1M0?ooo`030000003oool0oooo08h0oooo000_0?ooo`030000
o`3oool0oooo05d0oooo00<000000?ooo`3oool0SP3oool002l0oooo00<0003o0?ooo`3oool0G@3o
ool00`000000oooo0?ooo`2>0?ooo`00<03oool00`000?l0oooo0?ooo`1L0?ooo`030000003oool0
oooo08h0oooo000`0?ooo`030000o`3oool0oooo05`0oooo00<000000?ooo`3oool0SP3oool00300
oooo00<0003o0?ooo`3oool0G03oool00`000000oooo0?ooo`2>0?ooo`00<03oool00`000?l0oooo
0?ooo`1L0?ooo`800000S`3oool00340oooo00<0003o0?ooo`3oool0F`3oool00`000000oooo0?oo
o`2>0?ooo`00<@3oool00`000?l0oooo0?ooo`1K0?ooo`030000003oool0oooo08h0oooo000b0?oo
o`030000o`3oool0oooo05X0oooo00<000000?ooo`3oool0SP3oool00380oooo00<0003o0?ooo`3o
ool0FP3oool00`000000oooo0?ooo`2>0?ooo`00<P3oool00`000?l0oooo0?ooo`1J0?ooo`030000
003oool0oooo08h0oooo000c0?ooo`030000o`3oool0oooo05T0oooo0P00002?0?ooo`00<`3oool0
0`000?l0oooo0?ooo`1I0?ooo`030000003oool0oooo08h0oooo000d0?ooo`030000o`3oool0oooo
05P0oooo00<000000?ooo`3oool0SP3oool003@0oooo00<0003o0?ooo`3oool0F03oool00`000000
oooo0?ooo`2>0?ooo`00=03oool00`000?l0oooo0?ooo`1H0?ooo`030000003oool0oooo08h0oooo
000e0?ooo`030000o`3oool0oooo03d0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P00
00050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool003D0oooo00<0003o
0?ooo`3oool0?03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`00=P3oool00`000?l0oooo0?oo
o`0k0?ooo`040000003oool0oooo000000P0oooo00D000000?ooo`3oool000000?ooo`05000000D0
oooo0`00002>0?ooo`00=P3oool00`000?l0oooo0?ooo`0e0?ooo`@000000P3oool010000000oooo
0?ooo`0000080?ooo`060000003oool0oooo0000003oool000000P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo08h0oooo000f0?ooo`030000o`3oool0oooo03/0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0000000H0oooo00<0
00000?ooo`3oool0SP3oool003L0oooo00<0003o0?ooo`3oool0>P3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo000000<0oooo0P0000060?ooo`030000003oool0oooo08h0oooo
000g0?ooo`030000o`3oool0oooo03/0oooo0P00000:0?ooo`8000001@3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo08h0oooo000g0?ooo`030000o`3oool0oooo05D0oooo00<00000
0?ooo`3oool0SP3oool003P0oooo00<0003o0?ooo`3oool0E03oool00`000000oooo0?ooo`2>0?oo
o`00>03oool00`000?l0oooo0?ooo`1D0?ooo`800000S`3oool003T0oooo00<0003o0?ooo`3oool0
D`3oool00`000000oooo0?ooo`2>0?ooo`00>@3oool00`000?l0oooo0?ooo`1C0?ooo`030000003o
ool0oooo08h0oooo000i0?ooo`030000o`3oool0oooo05<0oooo00<000000?ooo`3oool0SP3oool0
03X0oooo00<0003o0?ooo`3oool0DP3oool00`000000oooo0?ooo`2>0?ooo`00>P3oool00`000?l0
oooo0?ooo`1B0?ooo`030000003oool0oooo08h0oooo000k0?ooo`030000o`3oool0oooo0540oooo
0P00002?0?ooo`00>`3oool00`000?l0oooo0?ooo`1A0?ooo`030000003oool0oooo08h0oooo000k
0?ooo`030000o`3oool0oooo0540oooo00<000000?ooo`3oool0SP3oool003`0oooo00<0003o0?oo
o`3oool0D03oool00`000000oooo0?ooo`2>0?ooo`00?03oool00`000?l0oooo0?ooo`1@0?ooo`03
0000003oool0oooo08h0oooo000m0?ooo`030000o`3oool0oooo04l0oooo00<000000?ooo`3oool0
SP3oool003d0oooo00<0003o0?ooo`3oool0C`3oool00`000000oooo0?ooo`2>0?ooo`00?P3oool0
0`000?l0oooo0?ooo`1>0?ooo`800000S`3oool003l0oooo00<0003o0?ooo`3oool0C@3oool00`00
0000oooo0?ooo`2>0?ooo`00?`3oool00`000?l0oooo0?ooo`1=0?ooo`030000003oool0oooo08h0
oooo00100?ooo`030000o`3oool0oooo04`0oooo00<000000?ooo`3oool0SP3oool00440oooo00<0
003o0?ooo`3oool0B`3oool00`000000oooo0?ooo`2>0?ooo`00@P3oool00`000?l0oooo0?ooo`0`
0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool4000000D0oooo00<00000
0?ooo`3oool0SP3oool004<0oooo00<0003o0?ooo`3oool0;P3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`2>0?ooo`00A03oool00`000?l0oooo0?ooo`0]0?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool000000`3oool00`000000oooo0?ooo`050?ooo`<00000SP3oool004@0oooo00<0
003o0?ooo`3oool09`3oool400000080oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool004D0oooo
00<0003o0?ooo`3oool0;03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00AP3oool00`000?l0
oooo0?ooo`0[0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo00170?ooo`030000o`3oool0
oooo02/0oooo0P00000:0?ooo`800000103oool2000000H0oooo00<000000?ooo`3oool0SP3oool0
04P0oooo00<0003o0?ooo`3oool0A03oool00`000000oooo0?ooo`2>0?ooo`00B@3oool00`000?l0
oooo0?ooo`130?ooo`800000S`3oool004T0oooo00<0003o0?ooo`3oool0@`3oool00`000000oooo
0?ooo`2>0?ooo`00BP3oool00`000?l0oooo0?ooo`120?ooo`030000003oool0oooo08h0oooo001;
0?ooo`030000o`3oool0oooo0440oooo00<000000?ooo`3oool0SP3oool004`0oooo0P000?m10?oo
o`030000003oool0oooo08h0oooo001>0?ooo`030000o`3oool0oooo03h0oooo00<000000?ooo`3o
ool0SP3oool004l0oooo0P000?ln0?ooo`030000003oool0oooo08h0oooo001A0?ooo`030000o`3o
ool0oooo03/0oooo0P00002?0?ooo`00DP3oool00`000?l0oooo0?ooo`0j0?ooo`030000003oool0
oooo08h0oooo001C0?ooo`80003o>P3oool00`000000oooo0?ooo`2>0?ooo`00203oool4000003/0
oooo0P0000040?ooo`030000003oool0oooo0080oooo0`0000000`000?l0oooo0?ooo`0g0?ooo`03
0000003oool0oooo03X0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00000k0?ooo`@0
00001`3oool000X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00000;0?ooo`03
000000000?l0003o03L0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00000;0?oo
o`030000003oool0oooo03X0oooo00<000000?ooo`3oool01P3oool000X0oooo00<000000?ooo`3o
ool0>@3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0080003o=@3oool00`00
0000oooo0?ooo`0i0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0>P3oool0
0`000000oooo0?ooo`060?ooo`000P3oool4000000@0oooo00<000000?ooo`3oool0<`3oool40000
0080oooo00@000000?ooo`3oool00000203oool3000000D0oooo0`000?lb0?ooo`030000003oool0
oooo03T0oooo00@000000?ooo`3oool00000203oool3000003d0oooo00<000000?ooo`3oool01P3o
ool000X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`0000080?ooo`030000003o
ool0oooo00P0oooo0P000?l`0?ooo`800000>P3oool010000000oooo0?ooo`0000080?ooo`030000
003oool0oooo03d0oooo00<000000?ooo`3oool01P3oool000P0oooo0`00000k0?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`3oool02P3oool30000obd0oooo00<000000?ooo`3oool0
>@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo02T0oooo5@000?l80?ooo`00
2P3oool00`000000oooo0?ooo`0j0?ooo`8000002@3oool4000000`0oooo0`000?lZ0?ooo`030000
003oool0oooo03X0oooo0P0000090?ooo`@000006P3oool>0000oa@0oooo00<000000?ooo`3oool0
1P3oool006D0oooo1@000?lU0?ooo`030000003oool0oooo04P0oooo6`000?l[0?ooo`00JP3oool7
0000oah0oooo00<000000?ooo`3oool0;@3ooolK0000odH0oooo001a0?ooo`X0003o503oool00`00
0000oooo0?ooo`0O0?ooo`h0003oH@3oool007/00000=P000?m^00000040oooo00070?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?ooo`001`3o
ool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0@@3o
ool00`000000oooo0?ooo`110?ooo`030000003oool0oooo00H0oooo002?0?ooo`030000003oool0
oooo08h0oooo002?0?ooo`030000003oool0oooo08h0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05261, -0.129382, \
0.00733523, 0.000749061}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImTable2delta = 
      Table[{N[a], Imdelta[1, 1, 1/2, 1/2, a]}, {a, \(-999\)/1000, 1.1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \({{\(-0.999`\), \(-5.068057158561567508128689`10.0484\)}, {\(-0.899`\), \
\(-0.252480277265721809380576`10.1628\)}, {\(-0.799`\), \
\(-0.114921634623443799874828`10.0145\)}, {\(-0.699`\), \
\(-0.061440841385741123302624`10.0396\)}, {\(-0.599`\), \
\(-0.034280790009428913232775`10.148\)}, {\(-0.499`\), \
\(-0.018969776881217402006444`10.3873\)}, {\(-0.399`\), \
\(-0.009983209079879421048856`10.1663\)}, {\(-0.299`\), \
\(-0.004720540922412997554485`10.0579\)}, {\(-0.199`\), \
\(-0.001791151506645128789015`10.017\)}, {\(-0.099`\), \
\(-0.000384888750095464794299`10.066\)}, {0.001`, \
\(-3.44712517894380855449`10.0062*^-8\)}, {0.101`, \
\(-0.000311486726523294225104`10.0624\)}, {0.201`, \
\(-0.001101209405541934550486`10.0702\)}, {0.301`, \
\(-0.00221903628102012687775`10.2189\)}, {0.401`, \
\(-0.003559391941735329933222`10.0644\)}, {0.501`, \
\(-0.00504686318660202669405`10.2534\)}, {0.601`, \
\(-0.006626934606425230664328`10.4086\)}, {0.701`, \
\(-0.00825986123996656367405`10.1117\)}, {0.801`, \
\(-0.009916513010086937956103`10.2379\)}, {0.901`, \
\(-0.01157549523926776252176`10.0805\)}, {1.001`, \
\(-0.01322111612726972635469`10.175\)}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImTable3delta = 
      Table[{N[a], Imdelta[1, 1, 9/10, 1/2, a]}, {a, \(-1\), 1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \({{\(-1.`\), 0.820210135563342326414215`10.1153}, {\(-0.9`\), 
        0.488674134874421248653405`10.1515}, {\(-0.8`\), 
        0.298462384992532560811732`10.0783}, {\(-0.7`\), 
        0.183055869787222760344965`10.1396}, {\(-0.6`\), 
        0.110677104782395909670159`10.0924}, {\(-0.5`\), 
        0.064594747527148408888591`10.1006}, {\(-0.4`\), 
        0.035335883953918022107408`10.2363}, {\(-0.3`\), 
        0.017226678285726686480606`10.3301}, {\(-0.2`\), 
        0.006713024624069859767425`10.0625}, {\(-0.1`\), 
        0.001486041651767416046064`10.047}, {0.`, 0``18.8737}, {0.1`, 
        0.00119374538930514031473`10.0003}, {0.2`, 
        0.004323194299608826278374`10.088}, {0.3`, 
        0.008857142942322579145894`10.1559}, {0.4`, 
        0.014410533549097122792983`10.2073}, {0.5`, 
        0.020700607504377655857506`10.365}, {0.6`, 
        0.027517443074165365626826`10.0412}, {0.7`, 
        0.034703762120755100273567`10.1477}, {0.8`, 
        0.042140843236956041078785`10.238}, {0.9`, 
        0.04973853828791614909181`10.0656}, {1.`, 
        0.057428095932515759076704`10.1296}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImPlot3delta = 
      ListPlot[ImTable3delta, PlotJoined \[Rule] True, 
        PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.47619 0.0147151 1.30466 [
[.02381 .00222 -6 -9 ]
[.02381 .00222 6 0 ]
[.2619 .00222 -12 -9 ]
[.2619 .00222 12 0 ]
[.7381 .00222 -9 -9 ]
[.7381 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.4875 .14518 -18 -4.5 ]
[.4875 .14518 0 4.5 ]
[.4875 .27565 -18 -4.5 ]
[.4875 .27565 0 4.5 ]
[.4875 .40611 -18 -4.5 ]
[.4875 .40611 0 4.5 ]
[.4875 .53658 -18 -4.5 ]
[.4875 .53658 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .01472 m
.02381 .02097 L
s
[(-1)] .02381 .00222 0 1 Mshowa
.2619 .01472 m
.2619 .02097 L
s
[(-0.5)] .2619 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(0.5)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(1)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.5 .14518 m
.50625 .14518 L
s
[(0.1)] .4875 .14518 1 0 Mshowa
.5 .27565 m
.50625 .27565 L
s
[(0.2)] .4875 .27565 1 0 Mshowa
.5 .40611 m
.50625 .40611 L
s
[(0.3)] .4875 .40611 1 0 Mshowa
.5 .53658 m
.50625 .53658 L
s
[(0.4)] .4875 .53658 1 0 Mshowa
.125 Mabswid
.5 .04081 m
.50375 .04081 L
s
.5 .0669 m
.50375 .0669 L
s
.5 .09299 m
.50375 .09299 L
s
.5 .11909 m
.50375 .11909 L
s
.5 .17127 m
.50375 .17127 L
s
.5 .19737 m
.50375 .19737 L
s
.5 .22346 m
.50375 .22346 L
s
.5 .24955 m
.50375 .24955 L
s
.5 .30174 m
.50375 .30174 L
s
.5 .32783 m
.50375 .32783 L
s
.5 .35393 m
.50375 .35393 L
s
.5 .38002 m
.50375 .38002 L
s
.5 .43221 m
.50375 .43221 L
s
.5 .4583 m
.50375 .4583 L
s
.5 .48439 m
.50375 .48439 L
s
.5 .51049 m
.50375 .51049 L
s
.5 .56267 m
.50375 .56267 L
s
.5 .58877 m
.50375 .58877 L
s
.5 .61486 m
.50375 .61486 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.078 .61803 m
.11905 .40411 L
.16667 .25354 L
.21429 .15911 L
.2619 .09899 L
.30952 .06082 L
.35714 .03719 L
.40476 .02347 L
.45238 .01665 L
.5 .01472 L
.54762 .01627 L
.59524 .02036 L
.64286 .02627 L
.69048 .03352 L
.7381 .04172 L
.78571 .05062 L
.83333 .05999 L
.88095 .06969 L
.92857 .07961 L
.97619 .08964 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003P3oool4000003P0oooo0P0000040?ooo`030000003oool0oooo0080
oooo0`00001a0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<00000>03oool4000000d0
oooo000@0?ooo`030000003oool0oooo03H0oooo00@000000?ooo`3oool000002`3oool00`000000
oooo0?ooo`1]0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0=`3oool00`00
0000oooo0?ooo`0<0?ooo`00403oool00`000000oooo0?ooo`0f0?ooo`040000003oool0oooo0000
00/0oooo00<000000?ooo`3oool0K@3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0
oooo03L0oooo00<000000?ooo`3oool0303oool000P0oooo100000040?ooo`030000003oool0oooo
0300oooo100000020?ooo`040000003oool0oooo000000P0oooo0`00001`0?ooo`040000003oool0
oooo000000P0oooo0`00000j0?ooo`030000003oool0oooo00`0oooo000@0?ooo`030000003oool0
oooo03H0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`1`0?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0<0?ooo`003P3o
ool3000003P0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`1`0?ooo`040000
003oool0oooo000000P0oooo00<000000?ooo`3oool0>03oool3000000h0oooo000@0?ooo`030000
003oool0oooo03L0oooo0P0000090?ooo`@00000=P3oool00`000000oooo0?ooo`0g0?ooo`800000
2@3oool4000003T0oooo00<000000?ooo`3oool0303oool008l0oooo00<000000?ooo`3oool0SP3o
ool008l0oooo00<000000?ooo`3oool0SP3oool008l0oooo00<000000?ooo`3oool0SP3oool000L0
ooooN000000Q0000ogP00000203oool000d0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`040000003oool0oooo0?ooo`H0003o0`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00X0oooo00@000000?ooo`3oool0oooo1P000?l30?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`003@3oool00`000000
oooo0?ooo`0n0?ooo`030000003oool0oooo02<0oooo1@000?lF0?ooo`030000003oool0oooo01@0
oooo2P000?lP0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0303oool00740oooo
0`000?lK0?ooo`030000003oool0oooo01h0oooo2P000?mV0?ooo`00K@3oool40000oah0oooo00<0
00000?ooo`3oool0:03oool60000of00oooo001Z0?ooo`<0003o8P3oool00`000000oooo0?ooo`0^
0?ooo`L0003oF@3oool006P0oooo0P000?lU0?ooo`030000003oool0oooo03D0oooo1P000?mC0?oo
o`00IP3oool20000obL0oooo0P00000l0?ooo`H0003oC@3oool006@0oooo0P000?lY0?ooo`030000
003oool0oooo0440oooo10000?m90?ooo`00HP3oool20000ob/0oooo00<000000?ooo`3oool0A@3o
ool40000odD0oooo001P0?ooo`80003o;@3oool00`000000oooo0?ooo`190?ooo`H0003o?`3oool0
05h0oooo0P000?l_0?ooo`030000003oool0oooo04l0oooo1P000?li0?ooo`00G03oool20000oc40
oooo00<000000?ooo`3oool0E@3oool60000oc<0oooo001K0?ooo`030000o`3oool0oooo0340oooo
00<000000?ooo`3oool0F`3oool40000obl0oooo001J0?ooo`030000o`3oool0oooo0380oooo0P00
001P0?ooo`@0003o:`3oool005P0oooo0P000?le0?ooo`030000003oool0oooo06<0oooo1@000?lV
0?ooo`00E`3oool00`000?l0oooo0?ooo`0e0?ooo`030000003oool0oooo06P0oooo10000?lR0?oo
o`00EP3oool00`000?l0oooo0?ooo`0f0?ooo`030000003oool0oooo06`0oooo10000?lN0?ooo`00
E03oool20000ocT0oooo00<000000?ooo`3oool0L03oool60000oaP0oooo001C0?ooo`030000o`3o
ool0oooo03T0oooo00<000000?ooo`3oool0MP3oool60000oa80oooo001B0?ooo`030000o`3oool0
oooo03X0oooo00<000000?ooo`3oool0O03oool40000o`h0oooo001@0?ooo`80003o?@3oool20000
08l0oooo001?0?ooo`030000o`3oool0oooo03d0oooo00<000000?ooo`3oool0SP3oool004h0oooo
00<0003o0?ooo`3oool0?P3oool00`000000oooo0?ooo`2>0?ooo`00C@3oool00`000?l0oooo0?oo
o`0o0?ooo`030000003oool0oooo08h0oooo001<0?ooo`030000o`3oool0oooo0400oooo00<00000
0?ooo`3oool0SP3oool004`0oooo00<0003o0?ooo`3oool0@03oool00`000000oooo0?ooo`2>0?oo
o`00B`3oool00`000?l0oooo0?ooo`110?ooo`030000003oool0oooo08h0oooo001:0?ooo`030000
o`3oool0oooo0480oooo0P00002?0?ooo`00B@3oool00`000?l0oooo0?ooo`130?ooo`030000003o
ool0oooo08h0oooo00180?ooo`030000o`3oool0oooo04@0oooo00<000000?ooo`3oool0SP3oool0
04P0oooo00<0003o0?ooo`3oool0A03oool00`000000oooo0?ooo`2>0?ooo`00A`3oool00`000?l0
oooo0?ooo`150?ooo`030000003oool0oooo08h0oooo00160?ooo`030000o`3oool0oooo04H0oooo
00<000000?ooo`3oool0SP3oool004D0oooo00<0003o0?ooo`3oool0=03oool2000000@0oooo00<0
00000?ooo`3oool00P3oool4000000@0oooo00<000000?ooo`3oool0SP3oool004@0oooo00<0003o
0?ooo`3oool0=03oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0SP3oool004<0oooo00<0003o0?ooo`3oool0=@3oool010000000oooo0?ooo`00
000:0?ooo`030000003oool0oooo00<0oooo0`00002>0?ooo`00@`3oool00`000?l0oooo0?ooo`0e
0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`2>0?ooo`00@P3oool00`000?l0oooo0?ooo`0f0?ooo`040000003oool0oooo000000X0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00@@3oool00`000?l0oooo0?ooo`0g
0?ooo`040000003oool0oooo000000P0oooo0`0000050?ooo`030000003oool0oooo08h0oooo0011
0?ooo`030000o`3oool0oooo03P0oooo0P00000;0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0SP3oool00400oooo00<0003o0?ooo`3oool0C03oool00`000000oooo0?ooo`2>0?oo
o`00@03oool00`000?l0oooo0?ooo`1<0?ooo`030000003oool0oooo08h0oooo000o0?ooo`030000
o`3oool0oooo04d0oooo0P00002?0?ooo`00?`3oool00`000?l0oooo0?ooo`1=0?ooo`030000003o
ool0oooo08h0oooo000n0?ooo`030000o`3oool0oooo04h0oooo00<000000?ooo`3oool0SP3oool0
03h0oooo00<0003o0?ooo`3oool0CP3oool00`000000oooo0?ooo`2>0?ooo`00?@3oool00`000?l0
oooo0?ooo`1?0?ooo`030000003oool0oooo08h0oooo000m0?ooo`030000o`3oool0oooo04l0oooo
00<000000?ooo`3oool0SP3oool003`0oooo00<0003o0?ooo`3oool0D03oool00`000000oooo0?oo
o`2>0?ooo`00?03oool00`000?l0oooo0?ooo`1@0?ooo`800000S`3oool003/0oooo00<0003o0?oo
o`3oool0D@3oool00`000000oooo0?ooo`2>0?ooo`00>`3oool00`000?l0oooo0?ooo`1A0?ooo`03
0000003oool0oooo08h0oooo000j0?ooo`030000o`3oool0oooo0580oooo00<000000?ooo`3oool0
SP3oool003X0oooo00<0003o0?ooo`3oool0DP3oool00`000000oooo0?ooo`2>0?ooo`00>@3oool0
0`000?l0oooo0?ooo`1C0?ooo`030000003oool0oooo08h0oooo000i0?ooo`030000o`3oool0oooo
05<0oooo00<000000?ooo`3oool0SP3oool003P0oooo00<0003o0?ooo`3oool0E03oool2000008l0
oooo000h0?ooo`030000o`3oool0oooo05@0oooo00<000000?ooo`3oool0SP3oool003L0oooo00<0
003o0?ooo`3oool0E@3oool00`000000oooo0?ooo`2>0?ooo`00=`3oool00`000?l0oooo0?ooo`1E
0?ooo`030000003oool0oooo08h0oooo000f0?ooo`030000o`3oool0oooo05H0oooo00<000000?oo
o`3oool0SP3oool003H0oooo00<0003o0?ooo`3oool0EP3oool00`000000oooo0?ooo`2>0?ooo`00
=@3oool00`000?l0oooo0?ooo`1G0?ooo`030000003oool0oooo08h0oooo000e0?ooo`030000o`3o
ool0oooo05L0oooo0P00002?0?ooo`00=03oool00`000?l0oooo0?ooo`1H0?ooo`030000003oool0
oooo08h0oooo000d0?ooo`030000o`3oool0oooo05P0oooo00<000000?ooo`3oool0SP3oool003<0
oooo00<0003o0?ooo`3oool0F@3oool00`000000oooo0?ooo`2>0?ooo`00<`3oool00`000?l0oooo
0?ooo`1I0?ooo`030000003oool0oooo08h0oooo000c0?ooo`030000o`3oool0oooo04H0oooo0P00
00040?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo08h0oooo000b
0?ooo`030000o`3oool0oooo04H0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo08h0oooo000b0?ooo`030000o`3oool0oooo04H0oooo00@00000
0?ooo`3oool000002@3oool00`000000oooo0?ooo`040?ooo`<00000SP3oool00380oooo00<0003o
0?ooo`3oool0AP3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0SP3oool00340oooo00<0003o0?ooo`3oool0A`3oool010000000oooo0?ooo`00
000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0SP3oool00340oooo00<0003o
0?ooo`3oool0A`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo
00<000000?ooo`3oool0SP3oool00340oooo00<0003o0?ooo`3oool0B03oool2000000X0oooo0P00
00050?ooo`030000003oool0oooo08h0oooo000a0?ooo`030000o`3oool0oooo05/0oooo00<00000
0?ooo`3oool0SP3oool00300oooo00<0003o0?ooo`3oool0G03oool00`000000oooo0?ooo`2>0?oo
o`00<03oool00`000?l0oooo0?ooo`1L0?ooo`800000S`3oool00300oooo00<0003o0?ooo`3oool0
G03oool00`000000oooo0?ooo`2>0?ooo`00;`3oool00`000?l0oooo0?ooo`1M0?ooo`030000003o
ool0oooo08h0oooo000_0?ooo`030000o`3oool0oooo05d0oooo00<000000?ooo`3oool0SP3oool0
02l0oooo00<0003o0?ooo`3oool0G@3oool00`000000oooo0?ooo`2>0?ooo`00;P3oool00`000?l0
oooo0?ooo`1N0?ooo`030000003oool0oooo08h0oooo000^0?ooo`030000o`3oool0oooo05h0oooo
00<000000?ooo`3oool0SP3oool002h0oooo00<0003o0?ooo`3oool0GP3oool2000008l0oooo000]
0?ooo`030000o`3oool0oooo05l0oooo00<000000?ooo`3oool0SP3oool002d0oooo00<0003o0?oo
o`3oool0G`3oool00`000000oooo0?ooo`2>0?ooo`00;@3oool00`000?l0oooo0?ooo`1O0?ooo`03
0000003oool0oooo08h0oooo000/0?ooo`030000o`3oool0oooo0600oooo00<000000?ooo`3oool0
SP3oool002`0oooo00<0003o0?ooo`3oool0H03oool00`000000oooo0?ooo`2>0?ooo`00;03oool0
0`000?l0oooo0?ooo`1P0?ooo`030000003oool0oooo08h0oooo000[0?ooo`030000o`3oool0oooo
0640oooo0P00002?0?ooo`00:`3oool00`000?l0oooo0?ooo`1Q0?ooo`030000003oool0oooo08h0
oooo000[0?ooo`030000o`3oool0oooo0640oooo00<000000?ooo`3oool0SP3oool002X0oooo00<0
003o0?ooo`3oool0HP3oool00`000000oooo0?ooo`2>0?ooo`00:P3oool00`000?l0oooo0?ooo`1R
0?ooo`030000003oool0oooo08h0oooo000Z0?ooo`030000o`3oool0oooo0680oooo00<000000?oo
o`3oool0SP3oool002X0oooo00<0003o0?ooo`3oool0HP3oool00`000000oooo0?ooo`2>0?ooo`00
:@3oool00`000?l0oooo0?ooo`1S0?ooo`030000003oool0oooo08h0oooo000Y0?ooo`030000o`3o
ool0oooo06<0oooo0P00002?0?ooo`00:@3oool00`000?l0oooo0?ooo`1S0?ooo`030000003oool0
oooo08h0oooo000X0?ooo`030000o`3oool0oooo06@0oooo00<000000?ooo`3oool0SP3oool002P0
oooo00<0003o0?ooo`3oool0I03oool00`000000oooo0?ooo`2>0?ooo`00:03oool00`000?l0oooo
0?ooo`1T0?ooo`030000003oool0oooo08h0oooo000W0?ooo`030000o`3oool0oooo0580oooo0P00
00040?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`030000003oool0oooo08h0oooo000W
0?ooo`030000o`3oool0oooo0540oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo08h0oooo000W0?ooo`030000o`3oool0oooo0540oooo00@0
00000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?ooo`<00000SP3oool002L0oooo00<0
003o0?ooo`3oool0D@3oool010000000oooo0?ooo`0000090?ooo`8000001@3oool00`000000oooo
0?ooo`2>0?ooo`009P3oool00`000?l0oooo0?ooo`1B0?ooo`040000003oool0oooo000000/0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2>0?ooo`009P3oool00`000?l0oooo0?oo
o`1B0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`2>0?ooo`009P3oool00`000?l0oooo0?ooo`1C0?ooo`8000002P3oool2000000D0oooo
00<000000?ooo`3oool0SP3oool002H0oooo00<0003o0?ooo`3oool0IP3oool00`000000oooo0?oo
o`2>0?ooo`009P3oool00`000?l0oooo0?ooo`1V0?ooo`030000003oool0oooo08h0oooo000U0?oo
o`030000o`3oool0oooo06L0oooo0P00002?0?ooo`009@3oool00`000?l0oooo0?ooo`1W0?ooo`03
0000003oool0oooo08h0oooo000U0?ooo`030000o`3oool0oooo06L0oooo00<000000?ooo`3oool0
SP3oool002D0oooo00<0003o0?ooo`3oool0I`3oool00`000000oooo0?ooo`2>0?ooo`009@3oool0
0`000?l0oooo0?ooo`1W0?ooo`030000003oool0oooo08h0oooo000U0?ooo`030000o`3oool0oooo
06L0oooo00<000000?ooo`3oool0SP3oool002@0oooo00<0003o0?ooo`3oool0J03oool00`000000
oooo0?ooo`2>0?ooo`00903oool00`000?l0oooo0?ooo`1X0?ooo`800000S`3oool002@0oooo00<0
003o0?ooo`3oool0J03oool00`000000oooo0?ooo`2>0?ooo`00903oool00`000?l0oooo0?ooo`1X
0?ooo`030000003oool0oooo08h0oooo000T0?ooo`030000o`3oool0oooo06P0oooo00<000000?oo
o`3oool0SP3oool002<0oooo00<0003o0?ooo`3oool0J@3oool00`000000oooo0?ooo`2>0?ooo`00
8`3oool00`000?l0oooo0?ooo`1Y0?ooo`030000003oool0oooo08h0oooo000S0?ooo`030000o`3o
ool0oooo06T0oooo00<000000?ooo`3oool0SP3oool002<0oooo00<0003o0?ooo`3oool0J@3oool2
000008l0oooo000S0?ooo`030000o`3oool0oooo06T0oooo00<000000?ooo`3oool0SP3oool00280
oooo00<0003o0?ooo`3oool0JP3oool00`000000oooo0?ooo`2>0?ooo`008P3oool00`000?l0oooo
0?ooo`1Z0?ooo`030000003oool0oooo08h0oooo000R0?ooo`030000o`3oool0oooo06X0oooo00<0
00000?ooo`3oool0SP3oool00280oooo00<0003o0?ooo`3oool0JP3oool00`000000oooo0?ooo`2>
0?ooo`008P3oool00`000?l0oooo0?ooo`1Z0?ooo`030000003oool0oooo08h0oooo000R0?ooo`03
0000o`3oool0oooo06X0oooo0P00002?0?ooo`008@3oool00`000?l0oooo0?ooo`1[0?ooo`030000
003oool0oooo08h0oooo000Q0?ooo`030000o`3oool0oooo06/0oooo00<000000?ooo`3oool0SP3o
ool00240oooo00<0003o0?ooo`3oool0J`3oool00`000000oooo0?ooo`2>0?ooo`008@3oool00`00
0?l0oooo0?ooo`1[0?ooo`030000003oool0oooo08h0oooo000Q0?ooo`030000o`3oool0oooo05P0
oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`2>0?ooo`00803oool00`000?l0oooo0?ooo`1H0?ooo`040000003oool0oooo000000X0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00803oool00`000?l0oooo
0?ooo`1H0?ooo`040000003oool0oooo000000L0oooo1@0000040?ooo`<00000SP3oool00200oooo
00<0003o0?ooo`3oool0F03oool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo0000
00D0oooo00<000000?ooo`3oool0SP3oool00200oooo00<0003o0?ooo`3oool0F03oool010000000
oooo0?ooo`0000080?ooo`030000003oool0000000D0oooo00<000000?ooo`3oool0SP3oool00200
oooo00<0003o0?ooo`3oool0F03oool010000000oooo0?ooo`0000090?ooo`8000001@3oool00`00
0000oooo0?ooo`2>0?ooo`007`3oool00`000?l0oooo0?ooo`1J0?ooo`8000002`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000O0?ooo`030000o`3oool0oooo06d0oooo
00<000000?ooo`3oool0SP3oool001l0oooo00<0003o0?ooo`3oool0K@3oool00`000000oooo0?oo
o`2>0?ooo`007`3oool00`000?l0oooo0?ooo`1]0?ooo`800000S`3oool001l0oooo00<0003o0?oo
o`3oool0K@3oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000?l0oooo0?ooo`1^0?ooo`03
0000003oool0oooo08h0oooo000N0?ooo`030000o`3oool0oooo06h0oooo00<000000?ooo`3oool0
SP3oool001h0oooo00<0003o0?ooo`3oool0KP3oool00`000000oooo0?ooo`2>0?ooo`007P3oool0
0`000?l0oooo0?ooo`1^0?ooo`030000003oool0oooo08h0oooo000N0?ooo`030000o`3oool0oooo
06h0oooo00<000000?ooo`3oool0SP3oool001h0oooo00<0003o0?ooo`3oool0KP3oool00`000000
oooo0?ooo`2>0?ooo`007@3oool00`000?l0oooo0?ooo`1_0?ooo`800000S`3oool001d0oooo00<0
003o0?ooo`3oool0K`3oool00`000000oooo0?ooo`2>0?ooo`007@3oool00`000?l0oooo0?ooo`1_
0?ooo`030000003oool0oooo08h0oooo000M0?ooo`030000o`3oool0oooo06l0oooo00<000000?oo
o`3oool0SP3oool001d0oooo00<0003o0?ooo`3oool0K`3oool00`000000oooo0?ooo`2>0?ooo`00
703oool00`000?l0oooo0?ooo`1`0?ooo`030000003oool0oooo08h0oooo000L0?ooo`030000o`3o
ool0oooo0700oooo00<000000?ooo`3oool0SP3oool001`0oooo00<0003o0?ooo`3oool0L03oool2
000008l0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.10503, -0.0348755, \
0.00770057, 0.00281065}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Real part:", "Subsubsection"],

Cell[BoxData[
    \(Redelta[md2_, k_, kz_, om_, a_] := 
      Module[{res, mxi1, mxi2, wp}, 
        If[logging \[Equal] 0, Print["\<delta\>"]]; \[IndentingNewLine]mxi1 = 
          If[k > om, xi1[k, kz, om], 0]; 
        mxi2 = If[k > om, xi2[k, kz, om], 0]; 
        If[kz/k < 1 - 10^\((\(-10\))\), wp = 30, 
          wp = 50]; \[IndentingNewLine]If[om < k, 
          res = \(md2/k^4\)/\((1 - kz^2/k^2)\)*
              om/2*\((NIntegrate[
                    1/\((1 + a\ xi^2)\)^2*om*kz, {xi, \(-1\), 1}, 
                    WorkingPrecision \[Rule] wp, MaxRecursion \[Rule] 30] + 
                  NIntegrate[
                    1/\((1 + a\ xi^2)\)^2*\((om + 
                          a*kz*xi)\)*\((k^2\ xi - kz\ om)\)/
                        Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                  om^2)\)*\((1 - kz^2/k^2)\)], {xi, \(-1\), 
                      mxi2*If[
                          mxi2 < 
                            0, \((1 + 10^\((\(-50\))\))\), \((1 - 
                              10^\((\(-50\))\))\)]}, 
                    WorkingPrecision \[Rule] wp, MaxRecursion \[Rule] 30] + 
                  Sign[om - kz]*
                    NIntegrate[
                      1/\((1 + a\ xi^2)\)^2*\((om + 
                            a*kz*xi)\)*\((k^2\ xi - kz\ om)\)/
                          Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                    om^2)\)*\((1 - kz^2/k^2)\)], {xi, 
                        mxi1*\((1 + 10^\((\(-50\))\))\), 1}, 
                      WorkingPrecision \[Rule] wp, 
                      MaxRecursion \[Rule] 30])\)]; 
        If[om > k, 
          res = \(md2/k^4\)/\((1 - kz^2/k^2)\)*
              om/2*\((NIntegrate[
                    1/\((1 + a\ xi^2)\)^2*om*kz, {xi, \(-1\), 1}, 
                    WorkingPrecision \[Rule] wp, MaxRecursion \[Rule] 30] + 
                  NIntegrate[
                    1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
                      Sign[om - kz\ xi]*\((k^2\ xi - kz\ om)\)/
                        Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - 
                                  om^2)\)*\((1 - kz^2/k^2)\)], {xi, \(-1\), 
                      If[om\ kz/k^2 < 1, om\ kz/k^2, 0], 1}, 
                    WorkingPrecision \[Rule] wp, 
                    MaxRecursion \[Rule] 30])\)]; res]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Redelta[1, 1, 1/10, 1/2, 0]\)], "Input"],

Cell[BoxData[
    \(\(-8.54752016819`0*^-17\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Redelta[1, 1, 1/10, 1/2, \(-999999\)/1000000]\)], "Input"],

Cell[BoxData[
    \(2.1122761324516012849`5.0257\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ReTable1delta = 
      Table[{N[a], Redelta[1, 1, 1/10, 1/2, a]}, {a, \(-99999\)/100000, 1.1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \({{\(-0.99999`\), 
        1.677382620151046666`6.0017}, {\(-0.89999`\), \
\(-0.069923127952532485341621`8.5186\)}, {\(-0.79999`\), \
\(-0.083419537504978454259675`8.8537\)}, {\(-0.69999`\), \
\(-0.065961578806783293396849`8.96\)}, {\(-0.59999`\), \
\(-0.048263534208337426640169`9.0529\)}, {\(-0.49999`\), \
\(-0.034077838911527482548629`8.9957\)}, {\(-0.39999`\), \
\(-0.023217072229225224321296`9.0648\)}, {\(-0.29999`\), \
\(-0.014956562160853697455044`8.9262\)}, {\(-0.19999`\), \
\(-0.008643596651440041287113`8.8012\)}, {\(-0.09999`\), \
\(-0.003779664952113484952738`8.5106\)}, {0.00001`, 
        3.33329269582098011`4.5263*^-7}, {0.10001`, 
        0.002961730778455437545658`8.5365}, {0.20001`, 
        0.005298224973145509425335`8.8135}, {0.30001`, 
        0.007152527086905086976484`9.0323}, {0.40001`, 
        0.008630933768804325312046`9.1652}, {0.50001`, 
        0.009813512862130528260408`9.2685}, {0.60001`, 
        0.010761240462360937108328`9.3514}, {0.70001`, 
        0.011521040271502902622687`9.4166}, {0.80001`, 
        0.012129377366486524949065`9.4617}, {0.90001`, 
        0.012614848916148735337092`9.4826}, {1.00001`, 
        0.013000072387946828993718`9.4776}}\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(RePlot1delta = 
      ListPlot[ReTable1delta, PlotJoined \[Rule] True, 
        PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.499995 0.47619 0.278503 3.16219 [
[.0238 .266 -6 -9 ]
[.0238 .266 6 0 ]
[.2619 .266 -12 -9 ]
[.2619 .266 12 0 ]
[.73809 .266 -9 -9 ]
[.73809 .266 9 0 ]
[.97619 .266 -3 -9 ]
[.97619 .266 3 0 ]
[.4875 .04134 -36 -4.5 ]
[.4875 .04134 0 4.5 ]
[.4875 .12039 -30 -4.5 ]
[.4875 .12039 0 4.5 ]
[.4875 .19945 -36 -4.5 ]
[.4875 .19945 0 4.5 ]
[.4875 .35756 -30 -4.5 ]
[.4875 .35756 0 4.5 ]
[.4875 .43661 -24 -4.5 ]
[.4875 .43661 0 4.5 ]
[.4875 .51567 -30 -4.5 ]
[.4875 .51567 0 4.5 ]
[.4875 .59472 -18 -4.5 ]
[.4875 .59472 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.0238 .2785 m
.0238 .28475 L
s
[(-1)] .0238 .266 0 1 Mshowa
.2619 .2785 m
.2619 .28475 L
s
[(-0.5)] .2619 .266 0 1 Mshowa
.73809 .2785 m
.73809 .28475 L
s
[(0.5)] .73809 .266 0 1 Mshowa
.97619 .2785 m
.97619 .28475 L
s
[(1)] .97619 .266 0 1 Mshowa
.125 Mabswid
.07142 .2785 m
.07142 .28225 L
s
.11904 .2785 m
.11904 .28225 L
s
.16666 .2785 m
.16666 .28225 L
s
.21428 .2785 m
.21428 .28225 L
s
.30952 .2785 m
.30952 .28225 L
s
.35714 .2785 m
.35714 .28225 L
s
.40476 .2785 m
.40476 .28225 L
s
.45238 .2785 m
.45238 .28225 L
s
.54761 .2785 m
.54761 .28225 L
s
.59523 .2785 m
.59523 .28225 L
s
.64285 .2785 m
.64285 .28225 L
s
.69047 .2785 m
.69047 .28225 L
s
.78571 .2785 m
.78571 .28225 L
s
.83333 .2785 m
.83333 .28225 L
s
.88095 .2785 m
.88095 .28225 L
s
.92857 .2785 m
.92857 .28225 L
s
.25 Mabswid
0 .2785 m
1 .2785 L
s
.5 .04134 m
.50625 .04134 L
s
[(-0.075)] .4875 .04134 1 0 Mshowa
.5 .12039 m
.50625 .12039 L
s
[(-0.05)] .4875 .12039 1 0 Mshowa
.5 .19945 m
.50625 .19945 L
s
[(-0.025)] .4875 .19945 1 0 Mshowa
.5 .35756 m
.50625 .35756 L
s
[(0.025)] .4875 .35756 1 0 Mshowa
.5 .43661 m
.50625 .43661 L
s
[(0.05)] .4875 .43661 1 0 Mshowa
.5 .51567 m
.50625 .51567 L
s
[(0.075)] .4875 .51567 1 0 Mshowa
.5 .59472 m
.50625 .59472 L
s
[(0.1)] .4875 .59472 1 0 Mshowa
.125 Mabswid
.5 .05715 m
.50375 .05715 L
s
.5 .07296 m
.50375 .07296 L
s
.5 .08877 m
.50375 .08877 L
s
.5 .10458 m
.50375 .10458 L
s
.5 .1362 m
.50375 .1362 L
s
.5 .15202 m
.50375 .15202 L
s
.5 .16783 m
.50375 .16783 L
s
.5 .18364 m
.50375 .18364 L
s
.5 .21526 m
.50375 .21526 L
s
.5 .23107 m
.50375 .23107 L
s
.5 .24688 m
.50375 .24688 L
s
.5 .26269 m
.50375 .26269 L
s
.5 .29431 m
.50375 .29431 L
s
.5 .31013 m
.50375 .31013 L
s
.5 .32594 m
.50375 .32594 L
s
.5 .34175 m
.50375 .34175 L
s
.5 .37337 m
.50375 .37337 L
s
.5 .38918 m
.50375 .38918 L
s
.5 .40499 m
.50375 .40499 L
s
.5 .4208 m
.50375 .4208 L
s
.5 .45242 m
.50375 .45242 L
s
.5 .46823 m
.50375 .46823 L
s
.5 .48405 m
.50375 .48405 L
s
.5 .49986 m
.50375 .49986 L
s
.5 .53148 m
.50375 .53148 L
s
.5 .54729 m
.50375 .54729 L
s
.5 .5631 m
.50375 .5631 L
s
.5 .57891 m
.50375 .57891 L
s
.5 .02553 m
.50375 .02553 L
s
.5 .00972 m
.50375 .00972 L
s
.5 .61053 m
.50375 .61053 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.0666 .61803 m
.07143 .05739 L
.11905 .01472 L
.16667 .06992 L
.21429 .12588 L
.2619 .17074 L
.30952 .20509 L
.35714 .23121 L
.40476 .25117 L
.45238 .26655 L
.5 .2785 L
.54762 .28787 L
.59524 .29526 L
.64286 .30112 L
.69048 .3058 L
.7381 .30954 L
.78571 .31253 L
.83333 .31493 L
.88095 .31686 L
.92857 .31839 L
.97619 .31961 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool2000008l0
oooo002?0?ooo`030000003oool0oooo08h0oooo000R0?ooo`030000o`3oool0oooo06X0oooo00<0
00000?ooo`3oool0SP3oool00240oooo00<0003o0?ooo`000?l0J`3oool00`000000oooo0?ooo`2>
0?ooo`00803oool01@000?l0oooo0?ooo`3oool0003o06X0oooo00<000000?ooo`3oool0SP3oool0
01h0oooo0P000?l50?ooo`030000o`3oool0oooo06L0oooo0P00002?0?ooo`007@3oool00`000?l0
oooo0?ooo`050?ooo`030000o`3oool0oooo06L0oooo00<000000?ooo`3oool0SP3oool001`0oooo
00<0003o0?ooo`3oool01`3oool00`000?l0oooo0?ooo`160?ooo`800000103oool00`000000oooo
0?ooo`030?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<000001P3oool00`000000oooo
0?ooo`2>0?ooo`006`3oool00`000?l0oooo0?ooo`090?ooo`030000o`3oool0oooo04@0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`006P3oool00`000?l0oooo
0?ooo`0;0?ooo`030000o`3oool0oooo04<0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3oool30000
08h0oooo000I0?ooo`030000o`3oool0oooo00d0oooo00<0003o0?ooo`3oool0?03oool400000080
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`040000003oool0
oooo0?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`005`3oool20000oa40oooo00<0003o
0?ooo`3oool0@@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo
0?ooo`0B0?ooo`030000o`3oool0oooo0400oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo0?ooo`0D0?ooo`030000o`3oool0oooo
0400oooo0P00000:0?ooo`8000000`3oool400000080oooo100000050?ooo`030000003oool0oooo
08h0oooo000D0?ooo`030000o`3oool0oooo01D0oooo00<0003o0?ooo`3oool0H03oool2000008l0
oooo000D0?ooo`030000o`3oool0oooo01H0oooo00<0003o0?ooo`3oool0G`3oool00`000000oooo
0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0G0?ooo`030000o`3oool0oooo05h0oooo00<0
00000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0603oool00`000?l0oooo0?ooo`1M
0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo01T0oooo00<0003o0?oo
o`3oool0G03oool2000008l0oooo000D0?ooo`030000o`3oool0oooo01X0oooo00<0003o0?ooo`3o
ool0F`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0K0?ooo`030000
o`3oool0oooo05X0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool06`3o
ool00`000?l0oooo0?ooo`1J0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0
oooo01`0oooo00<0003o0?ooo`3oool0F@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`00
0?l0oooo0?ooo`0M0?ooo`030000o`3oool0oooo05P0oooo0P00002?0?ooo`00503oool00`000?l0
oooo0?ooo`0N0?ooo`030000o`3oool0oooo05L0oooo00<000000?ooo`3oool0SP3oool001@0oooo
00<0003o0?ooo`3oool07`3oool00`000?l0oooo0?ooo`1F0?ooo`030000003oool0oooo08h0oooo
000D0?ooo`030000o`3oool0oooo01l0oooo00<0003o0?ooo`3oool0EP3oool00`000000oooo0?oo
o`2>0?ooo`00503oool00`000?l0oooo0?ooo`0P0?ooo`030000o`3oool0oooo05D0oooo00<00000
0?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool08@3oool00`000?l0oooo0?ooo`1D0?oo
o`800000S`3oool001@0oooo00<0003o0?ooo`3oool08P3oool00`000?l0oooo0?ooo`1C0?ooo`03
0000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo02<0oooo00<0003o0?ooo`3oool0
>03oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo0`0000060?ooo`030000
003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo02@0oooo00<0003o0?ooo`3oool0=P3o
ool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0T0?ooo`030000
o`3oool0oooo03H0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?oo
o`030000003oool0oooo00<0oooo0`00002>0?ooo`00503oool00`000?l0oooo0?ooo`0U0?ooo`03
0000o`3oool0oooo02l0oooo100000020?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool000000P3oool3000000H0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?oo
o`3oool09P3oool00`000?l0oooo0?ooo`0d0?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo000D
0?ooo`030000o`3oool0oooo02L0oooo00<0003o0?ooo`3oool0<`3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0X0?ooo`030000o`3oool0oooo03<0oooo0P00
000:0?ooo`8000000`3oool4000000D0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o
0?ooo`3oool0:@3oool20000odd0oooo0P00002?0?ooo`00503oool00`000?l0oooo0?ooo`0[0?oo
o`030000o`3oool0oooo04X0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3o
ool0;03oool00`000?l0oooo0?ooo`190?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000
o`3oool0oooo02d0oooo00<0003o0?ooo`3oool0B03oool00`000000oooo0?ooo`2>0?ooo`00503o
ool00`000?l0oooo0?ooo`0^0?ooo`030000o`3oool0oooo04L0oooo0P00002?0?ooo`00503oool0
0`000?l0oooo0?ooo`0_0?ooo`030000o`3oool0oooo04H0oooo00<000000?ooo`3oool0SP3oool0
01@0oooo00<0003o0?ooo`3oool0<03oool20000odH0oooo00<000000?ooo`3oool0SP3oool001@0
oooo00<0003o0?ooo`3oool0<P3oool00`000?l0oooo0?ooo`130?ooo`030000003oool0oooo08h0
oooo000D0?ooo`030000o`3oool0oooo03<0oooo00<0003o0?ooo`3oool0@P3oool00`000000oooo
0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0d0?ooo`030000o`3oool0oooo0440oooo0P00
002?0?ooo`00503oool00`000?l0oooo0?ooo`0e0?ooo`030000o`3oool0oooo0400oooo00<00000
0?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0=P3oool20000od00oooo00<000000?oo
o`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0>03oool00`000?l0oooo0?ooo`0m0?ooo`03
0000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo03T0oooo00<0003o0?ooo`3oool0
?03oool2000008l0oooo000D0?ooo`030000o`3oool0oooo03X0oooo0P000?ll0?ooo`030000003o
ool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo03`0oooo00<0003o0?ooo`3oool0>@3oool0
0`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0m0?ooo`030000o`3oool0oooo
01P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`@000000P3oool30000
00H0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0?P3oool20000oaL0
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`00
0?l0oooo0?ooo`100?ooo`030000o`3oool0oooo01@0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000030?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00`3o
ool3000008h0oooo000D0?ooo`030000o`3oool0oooo0440oooo00<0003o0?ooo`3oool03@3oool4
00000080oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`040000
003oool0oooo0?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo
0?ooo`120?ooo`80003o4`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00D0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`2>0?ooo`00503oool00`00
0?l0oooo0?ooo`140?ooo`80003o4@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo04H0oooo0P000?l@0?ooo`8000002P3o
ool2000000@0oooo0P0000030?ooo`@000001@3oool2000008l0oooo000D0?ooo`030000o`3oool0
oooo04P0oooo00<0003o0?ooo`3oool0;@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`00
0?l0oooo0?ooo`190?ooo`80003o;@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0
oooo0?ooo`1;0?ooo`80003o:`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo
0?ooo`1=0?ooo`80003o:@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?oo
o`1?0?ooo`80003o9`3oool2000008l0oooo000D0?ooo`030000o`3oool0oooo0540oooo0`000?lT
0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo05@0oooo0P000?lR0?oo
o`030000003oool0oooo08h0oooo00080?ooo`@00000203oool00`000?l0oooo0?ooo`0`0?ooo`80
0000103oool00`000000oooo0?ooo`020?ooo`<00000603oool30000oal0oooo00<000000?ooo`3o
ool0>P3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000003/0oooo100000070?ooo`00
2P3oool00`000000oooo0?ooo`070?ooo`030000o`3oool0oooo02l0oooo00@000000?ooo`3oool0
00002`3oool00`000000oooo0?ooo`0H0?ooo`<0003o703oool2000003X0oooo00@000000?ooo`3o
ool000002`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo00H0oooo000:0?ooo`03
0000003oool0oooo00L0oooo00<0003o0?ooo`3oool0;`3oool010000000oooo0?ooo`00000;0?oo
o`030000003oool0oooo01/0oooo0`000?lI0?ooo`030000003oool0oooo03T0oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo00H0oooo00020?oo
o`@00000103oool00`000000oooo0?ooo`070?ooo`030000o`3oool0oooo02T0oooo100000020?oo
o`040000003oool0oooo000000P0oooo0`00000Q0?ooo`80003o5`3oool00`000000oooo0?ooo`0i
0?ooo`040000003oool0oooo000000P0oooo0`00000m0?ooo`030000003oool0oooo00H0oooo000:
0?ooo`030000003oool0oooo00L0oooo00<0003o0?ooo`3oool0;`3oool010000000oooo0?ooo`00
00080?ooo`030000003oool0oooo02<0oooo0`000?lD0?ooo`030000003oool0oooo03T0oooo00@0
00000?ooo`3oool00000203oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00H0oooo
00080?ooo`<000002@3oool00`000?l0oooo0?ooo`0_0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool09P3oool30000oa40oooo00<000000?ooo`3oool0>@3oool010000000oooo
0?ooo`0000080?ooo`030000003oool0oooo03/0oooo0`0000080?ooo`002P3oool00`000000oooo
0?ooo`070?ooo`030000o`3oool0oooo0300oooo0P0000090?ooo`@00000:03oool20000o`l0oooo
0P00000k0?ooo`8000002@3oool4000003`0oooo00<000000?ooo`3oool01P3oool001@0oooo00<0
003o0?ooo`3oool0J@3oool40000o`/0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o
0?ooo`3oool0K@3oool40000o`L0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?oo
o`3oool0L@3oool50000o`80oooo00<000000?ooo`3oool0SP3oool001@0000000<0003o00000000
0000MP0000050000ohd000000@3oool000L0oooo00<000000?ooo`3oool02P3oool00`000?l0oooo
0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`040000o`L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool01P3oool000L0oooo00<000000?ooo`3oool02P3o
ool00`000?l0oooo0?ooo`0d0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0103o
ool50000ocP0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`060?ooo`00503oool0
0`000?l0oooo0?ooo`1h0?ooo`030000003oool0oooo00T0oooo1P000?mo0?ooo`00503oool00`00
0?l0oooo0?ooo`1h0?ooo`030000003oool0oooo00l0oooo1P000?mi0?ooo`00503oool00`000?l0
oooo0?ooo`1h0?ooo`8000005P3oool70000og80oooo000D0?ooo`030000o`3oool0oooo07P0oooo
00<000000?ooo`3oool0703oool70000of/0oooo000D0?ooo`030000o`3oool0oooo07P0oooo00<0
00000?ooo`3oool08`3oool:0000of40oooo000D0?ooo`030000o`3oool0oooo07P0oooo00<00000
0?ooo`3oool0;@3oool>0000oe<0oooo000D0?ooo`030000o`3oool0oooo07P0oooo0P00000l0?oo
o`d0003oAP3oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`180?ooo`h0
003o>03oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`1F0?oooa/0003o
7@3oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`1a0?oooaD0003o203o
ool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`2>0?ooo`00503oool00`00
0?l0oooo0?ooo`1h0?ooo`800000S`3oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000
oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`1h0?ooo`030000003oool0oooo08h0oooo
000D0?ooo`030000o`3oool0oooo07P0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o
0?ooo`3oool0N@3oool2000008l0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?oo
o`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`00
4`3oool00`000?l0oooo0?ooo`1I0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000
0`3oool400000080oooo0`0000060?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3o
ool0oooo05P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00
4`3oool00`000?l0oooo0?ooo`1H0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000000`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo0`00002>0?oo
o`004`3oool00`000?l0oooo0?ooo`1H0?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool00000103oool010000000oooo0?ooo`3oool3000000H0oooo00<000000?ooo`3oool0SP3o
ool001<0oooo00<0003o0?ooo`3oool0F03oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000000D0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`2>0?ooo`00
4`3oool00`000?l0oooo0?ooo`1H0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0F@3oool2000000X0oooo0P0000040?oo
o`8000000`3oool4000000D0oooo0P00002?0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`03
0000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0
SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool0
0`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo
07T0oooo0P00002?0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0
oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0
003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo0P00002?0?oo
o`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000
o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3o
ool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000S`3oool0
01<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0
oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo05l0oooo
0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`<000001P3oool00`000000oooo
0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1N0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08h0
oooo000C0?ooo`030000o`3oool0oooo05h0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000050?ooo`030000003oool0oooo00<0oooo0`00002>0?ooo`004`3oool00`000?l0
oooo0?ooo`1N0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool3
000000H0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0GP3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1N0?ooo`040000003oool0
oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo05l0oooo0P00000:0?ooo`8000000`3o
ool4000000D0oooo0P00002?0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0
oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0
oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo
0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo0P00
002?0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?oo
o`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3o
ool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000
S`3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool0
0`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo
07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000
oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000S`3oool001<0oooo00<0
003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1I
0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000103oool00`000000oooo0?ooo`02
0?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1H0?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o
0?ooo`3oool0F03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`<00000SP3oool001<0oooo00<0
003o0?ooo`3oool0F03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0
oooo00@000000?ooo`3oool0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000C0?ooo`03
0000o`3oool0oooo05P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000005
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o
0?ooo`3oool0F03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo08h0
oooo000C0?ooo`030000o`3oool0oooo05T0oooo0P00000:0?ooo`8000000`3oool400000080oooo
100000050?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo0P00
002?0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?oo
o`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3o
ool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000
S`3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool0
0`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo
07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000
oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000S`3oool001<0oooo00<0
003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?oo
o`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool2000008l0oooo000C0?ooo`030000
o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3o
ool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1U0?ooo`800000103oool0
0`000000oooo0?ooo`020?ooo`@000001@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`00
0?l0oooo0?ooo`1T0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1T0?ooo`040000003oool0oooo
000000X0oooo00<000000?ooo`3oool0103oool3000008h0oooo000C0?ooo`030000o`3oool0oooo
06@0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo08h0oooo000C0?ooo`030000o`3oool0oooo06@0oooo00@000000?ooo`3oool000002P3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo
06@0oooo00@000000?ooo`3oool00000203oool3000000H0oooo00<000000?ooo`3oool0SP3oool0
01<0oooo00<0003o0?ooo`3oool0I@3oool2000000/0oooo00<000000?ooo`3oool0103oool20000
08l0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo
00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.0526, -0.0880738, \
0.00733523, 0.0011046}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ReTable3delta = 
      Table[{N[a], Redelta[1, 1, 9/10, 1/2, a]}, {a, \(-99999\)/100000, 1.1, 
          1/10}]\)], "Input"],

Cell[BoxData[
    \({{\(-0.99999`\), 
        1.3468037042150875493`5.0352}, {\(-0.89999`\), \
\(-0.2918324833062537014`8.1995\)}, {\(-0.79999`\), \
\(-0.3267461884680342556`8.5671\)}, {\(-0.69999`\), \
\(-0.2946615044451756921`8.7328\)}, {\(-0.59999`\), \
\(-0.246609194572547222245543`8.9894\)}, {\(-0.49999`\), \
\(-0.196662518798075562601762`8.8456\)}, {\(-0.39999`\), \
\(-0.149346177747400271759379`9.104\)}, {\(-0.29999`\), \
\(-0.106014009616067291655122`8.7772\)}, {\(-0.19999`\), \
\(-0.066858898530729820938062`8.8617\)}, {\(-0.09999`\), \
\(-0.03164521492713833493752`8.5206\)}, {0.00001`, 
        2.99998418214185989`4.5129*^-6}, {0.10001`, 
        0.028485309806130019452067`8.4835}, {0.20001`, 
        0.05417875867605390050822`8.7469}, {0.30001`, 
        0.077421856806913227368704`9.0512}, {0.40001`, 
        0.098511475085119881261704`9.152}, {0.50001`, 
        0.117705070911003759293262`9.2227}, {0.60001`, 
        0.135224849263437800565282`9.41}, {0.70001`, 
        0.151262346429399764752787`9.4277}, {0.80001`, 
        0.165982791168825768790074`9.4098}, {0.90001`, 
        0.179529000413557679322594`9.3558}, {1.00001`, 
        0.192024750901700399843548`9.2763}}\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(RePlot3delta = 
      ListPlot[ReTable3delta, PlotJoined \[Rule] True, 
        PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.499995 0.47619 0.187728 0.529502 [
[.0238 .17523 -6 -9 ]
[.0238 .17523 6 0 ]
[.2619 .17523 -12 -9 ]
[.2619 .17523 12 0 ]
[.73809 .17523 -9 -9 ]
[.73809 .17523 9 0 ]
[.97619 .17523 -3 -9 ]
[.97619 .17523 3 0 ]
[.4875 .08183 -24 -4.5 ]
[.4875 .08183 0 4.5 ]
[.4875 .29363 -18 -4.5 ]
[.4875 .29363 0 4.5 ]
[.4875 .39953 -18 -4.5 ]
[.4875 .39953 0 4.5 ]
[.4875 .50543 -18 -4.5 ]
[.4875 .50543 0 4.5 ]
[.4875 .61133 -18 -4.5 ]
[.4875 .61133 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.0238 .18773 m
.0238 .19398 L
s
[(-1)] .0238 .17523 0 1 Mshowa
.2619 .18773 m
.2619 .19398 L
s
[(-0.5)] .2619 .17523 0 1 Mshowa
.73809 .18773 m
.73809 .19398 L
s
[(0.5)] .73809 .17523 0 1 Mshowa
.97619 .18773 m
.97619 .19398 L
s
[(1)] .97619 .17523 0 1 Mshowa
.125 Mabswid
.07142 .18773 m
.07142 .19148 L
s
.11904 .18773 m
.11904 .19148 L
s
.16666 .18773 m
.16666 .19148 L
s
.21428 .18773 m
.21428 .19148 L
s
.30952 .18773 m
.30952 .19148 L
s
.35714 .18773 m
.35714 .19148 L
s
.40476 .18773 m
.40476 .19148 L
s
.45238 .18773 m
.45238 .19148 L
s
.54761 .18773 m
.54761 .19148 L
s
.59523 .18773 m
.59523 .19148 L
s
.64285 .18773 m
.64285 .19148 L
s
.69047 .18773 m
.69047 .19148 L
s
.78571 .18773 m
.78571 .19148 L
s
.83333 .18773 m
.83333 .19148 L
s
.88095 .18773 m
.88095 .19148 L
s
.92857 .18773 m
.92857 .19148 L
s
.25 Mabswid
0 .18773 m
1 .18773 L
s
.5 .08183 m
.50625 .08183 L
s
[(-0.2)] .4875 .08183 1 0 Mshowa
.5 .29363 m
.50625 .29363 L
s
[(0.2)] .4875 .29363 1 0 Mshowa
.5 .39953 m
.50625 .39953 L
s
[(0.4)] .4875 .39953 1 0 Mshowa
.5 .50543 m
.50625 .50543 L
s
[(0.6)] .4875 .50543 1 0 Mshowa
.5 .61133 m
.50625 .61133 L
s
[(0.8)] .4875 .61133 1 0 Mshowa
.125 Mabswid
.5 .1083 m
.50375 .1083 L
s
.5 .13478 m
.50375 .13478 L
s
.5 .16125 m
.50375 .16125 L
s
.5 .2142 m
.50375 .2142 L
s
.5 .24068 m
.50375 .24068 L
s
.5 .26715 m
.50375 .26715 L
s
.5 .3201 m
.50375 .3201 L
s
.5 .34658 m
.50375 .34658 L
s
.5 .37305 m
.50375 .37305 L
s
.5 .426 m
.50375 .426 L
s
.5 .45248 m
.50375 .45248 L
s
.5 .47895 m
.50375 .47895 L
s
.5 .5319 m
.50375 .5319 L
s
.5 .55838 m
.50375 .55838 L
s
.5 .58485 m
.50375 .58485 L
s
.5 .05535 m
.50375 .05535 L
s
.5 .02888 m
.50375 .02888 L
s
.5 .0024 m
.50375 .0024 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.03933 .61803 m
.07143 .0332 L
.11905 .01472 L
.16667 .0317 L
.21429 .05715 L
.2619 .08359 L
.30952 .10865 L
.35714 .13159 L
.40476 .15233 L
.45238 .17097 L
.5 .18773 L
.54762 .20281 L
.59524 .21642 L
.64286 .22872 L
.69048 .23989 L
.7381 .25005 L
.78571 .25933 L
.83333 .26782 L
.88095 .27562 L
.92857 .28279 L
.97619 .28941 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool2
000008l0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0
oooo002?0?ooo`030000003oool0oooo08h0oooo000S0?ooo`<0003oJ@3oool00`000000oooo0?oo
o`2>0?ooo`00803oool30000o`<0oooo0P000?mW0?ooo`030000003oool0oooo08h0oooo000M0?oo
o`<0003o203oool30000of@0oooo00<000000?ooo`3oool0SP3oool001/0oooo0P000?l>0?ooo`<0
003oH@3oool00`000000oooo0?ooo`2>0?ooo`00603oool30000oa<0oooo0P000?mO0?ooo`800000
S`3oool001H0oooo0P000?lH0?ooo`80003oG@3oool00`000000oooo0?ooo`2>0?ooo`005P3oool0
0`000?l0oooo0?ooo`0I0?ooo`80003oF`3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`00
0?l0oooo0?ooo`0K0?ooo`80003oF@3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0
oooo0?ooo`0M0?ooo`80003oE`3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo
0?ooo`0O0?ooo`80003oE@3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo0?oo
o`0Q0?ooo`80003oD`3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo0?ooo`0S
0?ooo`80003oD@3oool2000008l0oooo000F0?ooo`030000o`3oool0oooo02D0oooo00<0003o0?oo
o`3oool0CP3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo0?ooo`0V0?ooo`80
003oCP3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo0?ooo`0X0?ooo`80003o
C03oool00`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo0?ooo`0[0?ooo`80003oBP3o
ool00`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo0?ooo`0]0?ooo`80003oB03oool0
0`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo0?ooo`0_0?ooo`80003o<P3oool20000
00@0oooo00<000000?ooo`3oool00P3oool4000000D0oooo00<000000?ooo`3oool0SP3oool001D0
oooo00<0003o0?ooo`3oool0<@3oool20000obl0oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo03<0
oooo0P000?l]0?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool01@3oool30000
08h0oooo000E0?ooo`030000o`3oool0oooo03D0oooo0P000?lU0?ooo`@000000P3oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool001D0
oooo00<0003o0?ooo`3oool0=`3oool20000obT0oooo00@000000?ooo`3oool000002`3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo03T0
oooo0P000?lW0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool0
0`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo0?ooo`0k0?ooo`80003o9P3oool20000
00X0oooo0P0000060?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo03d0
oooo0P000?lh0?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo03l0oooo
0P000?lf0?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo0440oooo00<0
003o0?ooo`3oool0<`3oool2000008l0oooo000E0?ooo`030000o`3oool0oooo0480oooo0P000?lc
0?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo04@0oooo0P000?la0?oo
o`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo04H0oooo0P000?l_0?ooo`03
0000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo04P0oooo0P000?l]0?ooo`030000
003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo04X0oooo0P000?l[0?ooo`030000003o
ool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo04`0oooo0P000?lY0?ooo`030000003oool0
oooo08h0oooo000D0?ooo`030000o`3oool0oooo04l0oooo0`000?lV0?ooo`030000003oool0oooo
08h0oooo000D0?ooo`030000o`3oool0oooo0580oooo0P000?lT0?ooo`800000S`3oool001@0oooo
00<0003o0?ooo`3oool0E03oool20000ob80oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0
003o0?ooo`3oool0EP3oool20000ob00oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o
0?ooo`3oool0F03oool20000oah0oooo00<000000?ooo`3oool0SP3oool000X0oooo100000060?oo
o`030000o`3oool0oooo0340oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00000K0?oo
o`80003o703oool00`000000oooo0?ooo`0i0?ooo`800000103oool00`000000oooo0?ooo`020?oo
o`<00000>P3oool4000000T0oooo000<0?ooo`030000003oool0oooo00D0oooo00<0003o0?ooo`3o
ool0<03oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo01X0oooo0`000?lI0?oo
o`030000003oool0oooo03P0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0i
0?ooo`030000003oool0oooo00P0oooo000<0?ooo`030000003oool0oooo00D0oooo00<0003o0?oo
o`3oool0<03oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo01d0oooo0`000?lF
0?ooo`030000003oool0oooo03P0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?oo
o`0i0?ooo`030000003oool0oooo00P0oooo00040?ooo`@00000103oool00`000000oooo0?ooo`05
0?ooo`030000o`3oool0oooo02X0oooo100000020?ooo`040000003oool0oooo000000P0oooo0`00
000S0?ooo`80003o503oool2000003T0oooo00@000000?ooo`3oool00000203oool3000003`0oooo
00<000000?ooo`3oool0203oool000`0oooo00<000000?ooo`3oool01@3oool00`000?l0oooo0?oo
o`0`0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool09@3oool30000oa40oooo
00<000000?ooo`3oool0>03oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo03`0
oooo00<000000?ooo`3oool0203oool000X0oooo0`0000070?ooo`030000o`3oool0oooo0300oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0X0?ooo`<0003o3P3oool00`000000
oooo0?ooo`0h0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0>P3oool30000
00X0oooo000<0?ooo`030000003oool0oooo00D0oooo00<0003o0?ooo`3oool0<@3oool2000000T0
oooo1000000Z0?ooo`<0003o2`3oool00`000000oooo0?ooo`0i0?ooo`8000002@3oool4000003/0
oooo00<000000?ooo`3oool0203oool001@0oooo00<0003o0?ooo`3oool0K@3oool20000o`T0oooo
00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0K`3oool30000o`H0oooo00<0
00000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0LP3oool30000o`<0oooo00<00000
0?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0M@3oool20000o`030?ooo`000000oooo
08l0oooo00020?oooa80000000<0003o000000000000M`0000030000oh`000000`3oool000T0oooo
00<000000?ooo`3oool0203oool00`000?l0oooo0000000=0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0000o`020000o`P0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
203oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`020?ooo`@0003oR03o
ool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`060?ooo`<0003oQ@3oool0
01<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`090?ooo`<0003oPP3oool001<0
oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`0<0?ooo`@0003oOP3oool001<0oooo
00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`0@0?ooo`<0003oN`3oool001<0oooo00<0
003o0?ooo`3oool0N@3oool2000001@0oooo10000?mg0?ooo`004`3oool00`000?l0oooo0?ooo`1i
0?ooo`030000003oool0oooo01L0oooo10000?mc0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?oo
o`030000003oool0oooo01/0oooo10000?m_0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`03
0000003oool0oooo01l0oooo10000?m[0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000
003oool0oooo02<0oooo1@000?mV0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003o
ool0oooo02P0oooo10000?mR0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0
oooo02`0oooo1@000?mM0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000<P3oool50000
oeP0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0=P3oool40000oe@0
oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0>P3oool40000oe00oooo
000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0?P3oool50000od/0oooo000C
0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0@`3oool40000odL0oooo000C0?oo
o`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0A`3oool50000od80oooo000C0?ooo`03
0000o`3oool0oooo07T0oooo00<000000?ooo`3oool0C03oool60000oc`0oooo000B0?ooo`030000
o`3oool0oooo07X0oooo00<000000?ooo`3oool0DP3oool60000ocH0oooo000B0?ooo`030000o`3o
ool0oooo07X0oooo0P00001I0?ooo`L0003o;`3oool00180oooo00<0003o0?ooo`3oool0NP3oool0
0`000000oooo0?ooo`1O0?ooo`H0003o:@3oool00180oooo00<0003o0?ooo`3oool0NP3oool00`00
0000oooo0?ooo`1U0?ooo`L0003o8P3oool00180oooo00<0003o0?ooo`3oool0NP3oool00`000000
oooo0?ooo`1/0?ooo`L0003o6`3oool00180oooo00<0003o0?ooo`3oool0NP3oool00`000000oooo
0?ooo`1c0?ooo`L0003o503oool00180oooo00<0003o0?ooo`3oool0IP3oool2000000@0oooo00<0
00000?ooo`3oool00P3oool4000000D0oooo00<000000?ooo`3oool0NP3oool60000o`h0oooo000B
0?ooo`030000o`3oool0oooo06D0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo0800oooo10000?l:0?ooo`004P3oool00`000?l0oooo0?ooo`1U
0?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool01@3oool3000008h0oooo000B
0?ooo`030000o`3oool0oooo06D0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000o`3oool0oooo06D0oooo00@00000
0?ooo`3oool000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000B
0?ooo`030000o`3oool0oooo06D0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000050?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000o`3oool0oooo06H0oooo0P00
000:0?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`004P3oool00`000?l0oooo0?ooo`1j
0?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000o`3oool0oooo07X0oooo00<000000?oo
o`3oool0SP3oool00180oooo00<0003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`2>0?ooo`00
4P3oool00`000?l0oooo0?ooo`1j0?ooo`800000S`3oool00180oooo00<0003o0?ooo`3oool0NP3o
ool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`030000003oool0
oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3oool00140
oooo00<0003o0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo
0?ooo`1k0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo00<0
00000?ooo`3oool0SP3oool00140oooo00<0003o0?ooo`3oool0N`3oool2000008l0oooo000A0?oo
o`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3oool00140oooo00<0003o0?ooo`3o
ool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`030000
003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3o
ool00140oooo00<0003o0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`00
0?l0oooo0?ooo`1k0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0
oooo00<000000?ooo`3oool0SP3oool00140oooo00<0003o0?ooo`3oool0N`3oool2000008l0oooo
000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3oool00140oooo00<0003o
0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?oo
o`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3o
ool0SP3oool00100oooo00<0003o0?ooo`3oool0J03oool2000000@0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3o
ool0oooo06L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo06L0oooo00@000000?ooo`3oool00000
1`3oool5000000D0oooo0`00002>0?ooo`00403oool00`000?l0oooo0?ooo`1W0?ooo`040000003o
ool0oooo000000L0oooo00@000000?ooo`3oool000001P3oool00`000000oooo0?ooo`2>0?ooo`00
403oool00`000?l0oooo0?ooo`1W0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`00
00001P3oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1W0?ooo`040000
003oool0oooo000000T0oooo0P0000060?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000
o`3oool0oooo06P0oooo0P00000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
SP3oool00100oooo00<0003o0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool0
0`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo
07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool2000008l0
oooo000@0?ooo`030000o`3oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0
003o0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l
0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo07`0oooo00<000000?oo
o`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00
403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3o
ool0oooo07`0oooo0P00002?0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0
oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0
oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo
0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<0
00000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>
0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`03
0000o`3oool0oooo07d0oooo0P00002?0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000
003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3o
ool000l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`00
0?l0oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo06T0
oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000060?ooo`030000003oool0oooo08h0
oooo000?0?ooo`030000o`3oool0oooo06P0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo06P0
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?ooo`<00000SP3oool0
00l0oooo00<0003o0?ooo`3oool0J03oool010000000oooo0?ooo`0000080?ooo`<000001P3oool0
0`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1X0?ooo`040000003oool0oooo
000000P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`00
0?l0oooo0?ooo`1X0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1Y0?ooo`8000002P3oool30000
00D0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`000000
oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo
000>0?ooo`030000o`3oool0oooo07h0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o
0?ooo`3oool0OP3oool2000008l0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<000000?oo
o`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`000000oooo0?ooo`2>0?ooo`00
3P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000o`3o
ool0oooo07h0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool0
0`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo
08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo0P00002?0?ooo`003P3oool00`000?l0oooo
0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<0
00000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`000000oooo0?ooo`2>
0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000>0?ooo`03
0000o`3oool0oooo07h0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0
OP3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003o
ool0oooo08h0oooo000=0?ooo`030000o`3oool0oooo07l0oooo0P00002?0?ooo`003@3oool00`00
0?l0oooo0?ooo`1o0?ooo`030000003oool0oooo08h0oooo000=0?ooo`030000o`3oool0oooo07l0
oooo00<000000?ooo`3oool0SP3oool000d0oooo00<0003o0?ooo`3oool0O`3oool00`000000oooo
0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1o0?ooo`030000003oool0oooo08h0oooo000=
0?ooo`030000o`3oool0oooo06/0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000006
0?ooo`030000003oool0oooo08h0oooo000=0?ooo`030000o`3oool0oooo06X0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo000=
0?ooo`030000o`3oool0oooo06X0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000050?ooo`<00000SP3oool000d0oooo00<0003o0?ooo`3oool0JP3oool010000000oooo0?oo
o`0000090?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?oo
o`1Z0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`000000
oooo0?ooo`2>0?ooo`00NP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
09H0oooo001k0?ooo`8000002P3oool2000009L0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.06836, -0.354543, \
0.00744506, 0.00669546}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(fulldelta[md2_, k_, kz_, om_, a_] := 
      Redelta[md2, k, kz, om, a] + I\ Imdelta[md2, k, kz, om, a]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fulldelta[1, 1, 99999/100000, 1/100, 1]\)], "Input"],

Cell[BoxData[
    \(0.0003318879035348566`4.4385 - 
      0.007845031731751109272147`10.1133\ \[ImaginaryI]\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[""], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Comparison to small a:", "Subsection"],

Cell[BoxData[
    \(choose\ order\ to\ compare\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(toorder = 5\)], "Input"],

Cell[BoxData[
    \(5\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(delta[md2_, k_, kz_, om_, a_] = 
        Normal[Series[
              a0delta + a1delta*a + a2delta*a^2 + a3delta*a^3 + a4delta*a^4 + 
                a5delta*a^5, {a, 0, toorder}]] /. 
          nz \[Rule] kz/k;\)\)], "Input"],

Cell[BoxData[
    \(Imaginary\ part\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Im[delta[1, 1, 1/10, 1/2, a]], {a, \(-1\), 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 0.382736 107.755 [
[.02381 .37024 -6 -9 ]
[.02381 .37024 6 0 ]
[.2619 .37024 -12 -9 ]
[.2619 .37024 12 0 ]
[.7381 .37024 -9 -9 ]
[.7381 .37024 9 0 ]
[.97619 .37024 -3 -9 ]
[.97619 .37024 3 0 ]
[.4875 .05947 -36 -4.5 ]
[.4875 .05947 0 4.5 ]
[.4875 .16723 -36 -4.5 ]
[.4875 .16723 0 4.5 ]
[.4875 .27498 -36 -4.5 ]
[.4875 .27498 0 4.5 ]
[.4875 .49049 -30 -4.5 ]
[.4875 .49049 0 4.5 ]
[.4875 .59825 -30 -4.5 ]
[.4875 .59825 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .38274 m
.02381 .38899 L
s
[(-1)] .02381 .37024 0 1 Mshowa
.2619 .38274 m
.2619 .38899 L
s
[(-0.5)] .2619 .37024 0 1 Mshowa
.7381 .38274 m
.7381 .38899 L
s
[(0.5)] .7381 .37024 0 1 Mshowa
.97619 .38274 m
.97619 .38899 L
s
[(1)] .97619 .37024 0 1 Mshowa
.125 Mabswid
.07143 .38274 m
.07143 .38649 L
s
.11905 .38274 m
.11905 .38649 L
s
.16667 .38274 m
.16667 .38649 L
s
.21429 .38274 m
.21429 .38649 L
s
.30952 .38274 m
.30952 .38649 L
s
.35714 .38274 m
.35714 .38649 L
s
.40476 .38274 m
.40476 .38649 L
s
.45238 .38274 m
.45238 .38649 L
s
.54762 .38274 m
.54762 .38649 L
s
.59524 .38274 m
.59524 .38649 L
s
.64286 .38274 m
.64286 .38649 L
s
.69048 .38274 m
.69048 .38649 L
s
.78571 .38274 m
.78571 .38649 L
s
.83333 .38274 m
.83333 .38649 L
s
.88095 .38274 m
.88095 .38649 L
s
.92857 .38274 m
.92857 .38649 L
s
.25 Mabswid
0 .38274 m
1 .38274 L
s
.5 .05947 m
.50625 .05947 L
s
[(-0.003)] .4875 .05947 1 0 Mshowa
.5 .16723 m
.50625 .16723 L
s
[(-0.002)] .4875 .16723 1 0 Mshowa
.5 .27498 m
.50625 .27498 L
s
[(-0.001)] .4875 .27498 1 0 Mshowa
.5 .49049 m
.50625 .49049 L
s
[(0.001)] .4875 .49049 1 0 Mshowa
.5 .59825 m
.50625 .59825 L
s
[(0.002)] .4875 .59825 1 0 Mshowa
.125 Mabswid
.5 .08102 m
.50375 .08102 L
s
.5 .10257 m
.50375 .10257 L
s
.5 .12412 m
.50375 .12412 L
s
.5 .14568 m
.50375 .14568 L
s
.5 .18878 m
.50375 .18878 L
s
.5 .21033 m
.50375 .21033 L
s
.5 .23188 m
.50375 .23188 L
s
.5 .25343 m
.50375 .25343 L
s
.5 .29653 m
.50375 .29653 L
s
.5 .31808 m
.50375 .31808 L
s
.5 .33963 m
.50375 .33963 L
s
.5 .36118 m
.50375 .36118 L
s
.5 .40429 m
.50375 .40429 L
s
.5 .42584 m
.50375 .42584 L
s
.5 .44739 m
.50375 .44739 L
s
.5 .46894 m
.50375 .46894 L
s
.5 .51204 m
.50375 .51204 L
s
.5 .53359 m
.50375 .53359 L
s
.5 .55514 m
.50375 .55514 L
s
.5 .57669 m
.50375 .57669 L
s
.5 .03792 m
.50375 .03792 L
s
.5 .01637 m
.50375 .01637 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.34176 0 m
.34251 .00562 L
.36282 .12219 L
.38186 .20553 L
.4021 .27164 L
.42063 .31544 L
.43047 .33326 L
.43961 .34685 L
.45028 .35949 L
.46034 .36855 L
.4702 .37504 L
.47575 .37776 L
.48086 .3797 L
.48517 .38094 L
.48765 .3815 L
.48991 .38192 L
.49229 .38227 L
.49352 .3824 L
.49485 .38253 L
.49608 .38262 L
.49722 .38268 L
.49831 .38271 L
.49947 .38273 L
.5006 .38273 L
.50185 .38271 L
.50315 .38266 L
.50436 .38259 L
.50549 .38251 L
.50652 .38242 L
.50884 .38216 L
.51133 .38179 L
.51403 .3813 L
.51894 .38017 L
.52845 .37715 L
.53722 .37349 L
.55598 .3632 L
.57662 .34876 L
.61696 .31438 L
.65578 .27885 L
.67553 .26224 L
.68596 .25445 L
.69706 .24723 L
.70789 .24155 L
.713 .23942 L
.71769 .23783 L
.72243 .23661 L
.72492 .23613 L
.72619 .23593 L
.72755 .23576 L
.72875 .23563 L
Mistroke
.72984 .23554 L
.73094 .23548 L
.73197 .23544 L
.73318 .23543 L
.7343 .23544 L
.73552 .2355 L
.73682 .23559 L
.73796 .2357 L
.73921 .23586 L
.74173 .23629 L
.74405 .23684 L
.74622 .23747 L
.75111 .23938 L
.75633 .24218 L
.76127 .24561 L
.76592 .24959 L
.77462 .25914 L
.78437 .27345 L
.79468 .29327 L
.81314 .34284 L
.82261 .37634 L
.83276 .4193 L
.85108 .51785 L
Mfstroke
.85108 .51785 m
.86489 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oHP3oool0
0`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo08h0oooo001R0?ooo`030000003oool0oooo
02X0oooo00<000000?ooo`3oool0SP3oool00680oooo00<000000?ooo`3oool0:P3oool00`000000
oooo0?ooo`2>0?ooo`00HP3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo08h0oooo
001S0?ooo`030000003oool0oooo02T0oooo0P00002?0?ooo`00H`3oool00`000000oooo0?ooo`0Y
0?ooo`030000003oool0oooo08h0oooo001S0?ooo`030000003oool0oooo02T0oooo00<000000?oo
o`3oool0SP3oool006<0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`2>0?ooo`00
H`3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo08h0oooo001S0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0SP3oool006@0oooo00<000000?ooo`3oool0:03oool2
000008l0oooo001T0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0SP3oool006@0
oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`2>0?ooo`00I03oool00`000000oooo
0?ooo`0X0?ooo`030000003oool0oooo08h0oooo001T0?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool0SP3oool006@0oooo00<000000?ooo`3oool0203oool2000000@0oooo00<00000
0?ooo`3oool00`3oool2000000@0oooo0P0000040?ooo`8000001P3oool00`000000oooo0?ooo`2>
0?ooo`00I@3oool00`000000oooo0?ooo`060?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool0SP3oool006D0oooo00<000000?ooo`3oool01P3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool0
0`000000oooo0?ooo`030?ooo`<00000SP3oool006D0oooo00<000000?ooo`3oool0100000020?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?oo
o`0000030?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`00I@3oool00`000000oooo0?oo
o`060?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000
oooo0?ooo`0000050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0SP3oool006D0
oooo00<000000?ooo`3oool01P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000
003oool0oooo08h0oooo001V0?ooo`030000003oool0oooo00H0oooo0P00000:0?ooo`800000103o
ool2000000@0oooo0P0000060?ooo`030000003oool0oooo08h0oooo001V0?ooo`030000003oool0
oooo02H0oooo00<000000?ooo`3oool0SP3oool006H0oooo00<000000?ooo`3oool09P3oool20000
08l0oooo001V0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0SP3oool006H0oooo
00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`2>0?ooo`00IP3oool00`000000oooo0?oo
o`0V0?ooo`030000003oool0oooo08h0oooo001W0?ooo`030000003oool0oooo02D0oooo00<00000
0?ooo`3oool0SP3oool006L0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`2>0?oo
o`00I`3oool00`000000oooo0?ooo`0U0?ooo`800000S`3oool006L0oooo00<000000?ooo`3oool0
9@3oool00`000000oooo0?ooo`2>0?ooo`00I`3oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0oooo08h0oooo001W0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0SP3oool0
06P0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`2>0?ooo`00J03oool00`000000
oooo0?ooo`0T0?ooo`030000003oool0oooo08h0oooo001X0?ooo`030000003oool0oooo02@0oooo
0P00002?0?ooo`00J03oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo08h0oooo001X
0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0SP3oool006T0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`2>0?ooo`00J@3oool00`000000oooo0?ooo`0S0?ooo`03
0000003oool0oooo08h0oooo001Y0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
SP3oool006T0oooo00<000000?ooo`3oool08`3oool2000008l0oooo001Z0?ooo`030000003oool0
oooo0280oooo00<000000?ooo`3oool0SP3oool006X0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`2>0?ooo`00JP3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo08h0
oooo001Z0?ooo`030000003oool0oooo0080oooo0P0000040?ooo`030000003oool0oooo00<0oooo
0P0000040?ooo`8000000`3oool4000000D0oooo00<000000?ooo`3oool0SP3oool006X0oooo00D0
00000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00H0oooo00@000000?ooo`3oool0
00000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0SP3oool006/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?oo
o`050?ooo`<00000SP3oool006P0oooo100000020?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0SP3oool006/0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`060?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo001[0?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool01P3oool010000000oooo0?ooo`0000020?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`2>0?ooo`00
J`3oool010000000oooo0?ooo`3oool2000000X0oooo0P0000040?ooo`800000103oool2000000H0
oooo00<000000?ooo`3oool0SP3oool006`0oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`2>0?ooo`00K03oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo08h0oooo001/
0?ooo`030000003oool0oooo0200oooo0P00002?0?ooo`00K03oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo08h0oooo001]0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3o
ool0SP3oool006d0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`2>0?ooo`00K@3o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo08h0oooo001]0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool0SP3oool006h0oooo00<000000?ooo`3oool07P3oool20000
08l0oooo001^0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0SP3oool006h0oooo
00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`2>0?ooo`00K`3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo08h0oooo001_0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool0SP3oool006l0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`2>0?oo
o`00L03oool00`000000oooo0?ooo`0L0?ooo`800000S`3oool00700oooo00<000000?ooo`3oool0
703oool00`000000oooo0?ooo`0l0?ooo`P00000BP3oool00700oooo00<000000?ooo`3oool0703o
ool00`000000oooo0?ooo`0j0?ooo`8000001`3oool3000004P0oooo001`0?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool0=`3oool3000000`0oooo00<000000?ooo`3oool0A@3oool0
0740oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0e0?ooo`800000403oool20000
04D0oooo001a0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0<`3oool2000001@0
oooo00<000000?ooo`3oool0@P3oool00740oooo00<000000?ooo`3oool06`3oool2000003<0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`110?ooo`00LP3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`100?ooo`00LP3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0300oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0o0?ooo`00LP3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo02l0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0o0?oo
o`00K`3oool200000080oooo00<000000?ooo`0000001@3oool2000000@0oooo0P0000030?ooo`@0
00001@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool0?P3oool006h0oooo00H000000?ooo`3oool000000?ooo`0000060?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo02`0oooo0P00000O0?ooo`030000003oool0oooo03h0oooo001^0?ooo`060000003oool0
oooo0000003oool000001P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0000
00@0oooo00<000000?ooo`3oool0103oool3000002/0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0m0?ooo`00J03oool400000080oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`0m0?ooo`00KP3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo00<0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0S0?ooo`03
0000003oool0oooo03`0oooo001^0?ooo`040000003oool0oooo000000<0oooo00<000000?ooo`3o
ool00P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo0`000006
0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0l
0?ooo`00K`3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P0000050?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0V0?oo
o`030000003oool0oooo03/0oooo001e0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool09P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo03/0oooo001f0?ooo`030000
003oool0oooo01H0oooo0P00000U0?ooo`800000:`3oool00`000000oooo0?ooo`0j0?ooo`00MP3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0:`3o
ool00`000000oooo0?ooo`0j0?ooo`00MP3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo0280oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0i0?ooo`00M`3oool00`00
0000oooo0?ooo`0E0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`0i0?ooo`00M`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0h0?ooo`00N03oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0<@3oool00`000000oooo
0?ooo`0h0?ooo`00N03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01h0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`0g0?ooo`00N@3oool00`000000oooo0?ooo`0C
0?ooo`8000007P3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo03L0oooo001i0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool06`3oool2000003L0oooo00<000000?oo
o`3oool0=`3oool007X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo03P0oooo00<000000?ooo`3oool0=P3oool007X0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0
=P3oool007/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo03/0oooo00<000000?ooo`3oool0=@3oool007`0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0=@3oool0
07`0oooo00<000000?ooo`3oool0403oool2000001H0oooo0P0000100?ooo`030000003oool0oooo
03@0oooo001m0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`100?ooo`030000003oool0oooo03@0oooo00080?ooo`@00000>`3oool2000000@0oooo
00<000000?ooo`3oool00P3oool3000002T0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo02@0oooo0P0000040?ooo`030000003oool0oooo0080oooo
0`00000?0?ooo`030000003oool0oooo02T0oooo100000070?ooo`002P3oool00`000000oooo0?oo
o`0i0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool09`3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0903oool010000000oooo
0?ooo`00000;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`060?ooo`002P3oool00`000000oooo0?ooo`0i0?ooo`040000003oool0oooo000000/0
oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0100
oooo0P00000W0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo00H0oooo00020?ooo`@00000103oool00`000000
oooo0?ooo`0c0?ooo`@000000P3oool010000000oooo0?ooo`0000080?ooo`<00000;03oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool09`3oool01000
0000oooo0?ooo`0000080?ooo`<00000403oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0
oooo00H0oooo000:0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo0?ooo`0]0?ooo`030000003oool0oooo00X0oooo0P00000?0?ooo`030000003oool0
oooo02P0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool01P3oool000P0oooo0`00000k0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00`0oooo0P00000[0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0X0?ooo`<00000203oool000X0oooo00<000000?ooo`3oool0>P3o
ool2000000T0oooo1000000^0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02P3o
ool2000002h0oooo0P0000090?ooo`@00000403oool00`000000oooo0?ooo`0Y0?ooo`030000003o
ool0oooo00H0oooo00250?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0203oool2
000004l0oooo00<000000?ooo`3oool0<P3oool008H0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`060?ooo`800000D@3oool00`000000oooo0?ooo`0b0?ooo`00Q`3oool4000000@0
oooo00<000000?ooo`3oool00P3oool4000005<0oooo00<000000?ooo`3oool0<P3oool00?l00000
800000010?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`0000003@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool01P3oool000L0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`11
0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo00H0oooo002?0?ooo`030000003oool0oooo05X0oooo00<000000?oo
o`3oool0<@3oool008l0oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`0a0?ooo`00
S`3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo0300oooo002?0?ooo`800000G03o
ool00`000000oooo0?ooo`0`0?ooo`00S`3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0
oooo0300oooo002?0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0<03oool008l0
oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`0_0?ooo`00S`3oool00`000000oooo
0?ooo`1L0?ooo`030000003oool0oooo02l0oooo002?0?ooo`030000003oool0oooo05`0oooo00<0
00000?ooo`3oool0;`3oool008l0oooo0P00001M0?ooo`030000003oool0oooo02l0oooo002?0?oo
o`030000003oool0oooo05`0oooo00<000000?ooo`3oool0;`3oool008l0oooo00<000000?ooo`3o
ool0G@3oool00`000000oooo0?ooo`0^0?ooo`00S`3oool00`000000oooo0?ooo`1M0?ooo`030000
003oool0oooo02h0oooo002?0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool0;P3o
ool008l0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`0^0?ooo`00S`3oool00`00
0000oooo0?ooo`1M0?ooo`030000003oool0oooo02h0oooo002?0?ooo`800000G`3oool00`000000
oooo0?ooo`0]0?ooo`00S`3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo02d0oooo
002?0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0;@3oool008l0oooo00<00000
0?ooo`3oool0GP3oool00`000000oooo0?ooo`0]0?ooo`00S`3oool00`000000oooo0?ooo`1O0?oo
o`030000003oool0oooo02`0oooo002?0?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3o
ool0;03oool008l0oooo0P00001P0?ooo`030000003oool0oooo02`0oooo002?0?ooo`030000003o
ool0oooo05l0oooo00<000000?ooo`3oool0;03oool008l0oooo00<000000?ooo`3oool0G`3oool0
0`000000oooo0?ooo`0/0?ooo`00S`3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0oooo
02/0oooo001_0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000103oool2000000<0
oooo100000050?ooo`030000003oool0oooo0600oooo00<000000?ooo`3oool0:`3oool006h0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1P0?ooo`030000003oool0
oooo02/0oooo001^0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3o
ool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00@0oooo0`00001P0?ooo`030000
003oool0oooo02/0oooo001^0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
00000P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0H03oool00`000000oooo0?ooo`0[0?ooo`00KP3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0:P3oool006h0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080
oooo0`0000060?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0:P3oool006l0oooo
0P00000:0?ooo`800000103oool2000000D0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`1Q0?ooo`030000003oool0oooo02X0oooo002?0?ooo`030000003oool0oooo0640oooo00<0
00000?ooo`3oool0:P3oool008l0oooo0P00001S0?ooo`030000003oool0oooo02T0oooo002?0?oo
o`030000003oool0oooo0680oooo00<000000?ooo`3oool0:@3oool008l0oooo00<000000?ooo`3o
ool0HP3oool00`000000oooo0?ooo`0Y0?ooo`00S`3oool00`000000oooo0?ooo`1R0?ooo`030000
003oool0oooo02T0oooo002?0?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool0:@3o
ool008l0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`0Y0?ooo`00S`3oool20000
06<0oooo00<000000?ooo`3oool0:@3oool008l0oooo00<000000?ooo`3oool0H`3oool00`000000
oooo0?ooo`0X0?ooo`00S`3oool00`000000oooo0?ooo`1S0?ooo`030000003oool0oooo02P0oooo
002?0?ooo`030000003oool0oooo06<0oooo00<000000?ooo`3oool0:03oool008l0oooo00<00000
0?ooo`3oool0H`3oool00`000000oooo0?ooo`0X0?ooo`00S`3oool00`000000oooo0?ooo`1S0?oo
o`030000003oool0oooo02P0oooo002?0?ooo`800000I03oool00`000000oooo0?ooo`0X0?ooo`00
S`3oool00`000000oooo0?ooo`1S0?ooo`030000003oool0oooo02P0oooo002?0?ooo`030000003o
ool0oooo06<0oooo00<000000?ooo`3oool0:03oool008l0oooo00<000000?ooo`3oool0H`3oool0
0`000000oooo0?ooo`0X0?ooo`00S`3oool00`000000oooo0?ooo`1S0?ooo`030000003oool0oooo
02P0oooo002?0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool09`3oool008l0oooo
00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`0W0?ooo`00S`3oool2000006D0oooo00<0
00000?ooo`3oool09`3oool008l0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`0W
0?ooo`00S`3oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo02L0oooo002?0?ooo`03
0000003oool0oooo06@0oooo00<000000?ooo`3oool09`3oool006l0oooo0P0000040?ooo`030000
003oool0oooo00<0oooo0P0000040?ooo`8000000`3oool4000000D0oooo00<000000?ooo`3oool0
I03oool00`000000oooo0?ooo`0W0?ooo`00KP3oool010000000oooo0?ooo`0000080?ooo`040000
003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo06@0oooo00<000000?ooo`3oool09`3oool006h0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000<0oooo00<0
00000?ooo`3oool01@3oool3000006@0oooo00<000000?ooo`3oool09`3oool006h0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo02H0
oooo001^0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`0000050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0I@3oool0
0`000000oooo0?ooo`0V0?ooo`00KP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`03
0000003oool0oooo06D0oooo00<000000?ooo`3oool09P3oool006l0oooo0P00000:0?ooo`800000
103oool2000000@0oooo0P0000060?ooo`030000003oool0oooo06D0oooo00<000000?ooo`3oool0
9P3oool008l0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`0V0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.0526, -0.00355194, \
0.00733521, 3.24158*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, ImPlot1delta]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.47619 0.382712 40.2629 [
[.02381 .37021 -6 -9 ]
[.02381 .37021 6 0 ]
[.2619 .37021 -12 -9 ]
[.2619 .37021 12 0 ]
[.7381 .37021 -9 -9 ]
[.7381 .37021 9 0 ]
[.97619 .37021 -3 -9 ]
[.97619 .37021 3 0 ]
[.4875 .06061 -36 -4.5 ]
[.4875 .06061 0 4.5 ]
[.4875 .14113 -36 -4.5 ]
[.4875 .14113 0 4.5 ]
[.4875 .22166 -36 -4.5 ]
[.4875 .22166 0 4.5 ]
[.4875 .30219 -36 -4.5 ]
[.4875 .30219 0 4.5 ]
[.4875 .46324 -30 -4.5 ]
[.4875 .46324 0 4.5 ]
[.4875 .54376 -30 -4.5 ]
[.4875 .54376 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .38271 m
.02381 .38896 L
s
[(-1)] .02381 .37021 0 1 Mshowa
.2619 .38271 m
.2619 .38896 L
s
[(-0.5)] .2619 .37021 0 1 Mshowa
.7381 .38271 m
.7381 .38896 L
s
[(0.5)] .7381 .37021 0 1 Mshowa
.97619 .38271 m
.97619 .38896 L
s
[(1)] .97619 .37021 0 1 Mshowa
.125 Mabswid
.07143 .38271 m
.07143 .38646 L
s
.11905 .38271 m
.11905 .38646 L
s
.16667 .38271 m
.16667 .38646 L
s
.21429 .38271 m
.21429 .38646 L
s
.30952 .38271 m
.30952 .38646 L
s
.35714 .38271 m
.35714 .38646 L
s
.40476 .38271 m
.40476 .38646 L
s
.45238 .38271 m
.45238 .38646 L
s
.54762 .38271 m
.54762 .38646 L
s
.59524 .38271 m
.59524 .38646 L
s
.64286 .38271 m
.64286 .38646 L
s
.69048 .38271 m
.69048 .38646 L
s
.78571 .38271 m
.78571 .38646 L
s
.83333 .38271 m
.83333 .38646 L
s
.88095 .38271 m
.88095 .38646 L
s
.92857 .38271 m
.92857 .38646 L
s
.25 Mabswid
0 .38271 m
1 .38271 L
s
.5 .06061 m
.50625 .06061 L
s
[(-0.008)] .4875 .06061 1 0 Mshowa
.5 .14113 m
.50625 .14113 L
s
[(-0.006)] .4875 .14113 1 0 Mshowa
.5 .22166 m
.50625 .22166 L
s
[(-0.004)] .4875 .22166 1 0 Mshowa
.5 .30219 m
.50625 .30219 L
s
[(-0.002)] .4875 .30219 1 0 Mshowa
.5 .46324 m
.50625 .46324 L
s
[(0.002)] .4875 .46324 1 0 Mshowa
.5 .54376 m
.50625 .54376 L
s
[(0.004)] .4875 .54376 1 0 Mshowa
.125 Mabswid
.5 .08074 m
.50375 .08074 L
s
.5 .10087 m
.50375 .10087 L
s
.5 .121 m
.50375 .121 L
s
.5 .16127 m
.50375 .16127 L
s
.5 .1814 m
.50375 .1814 L
s
.5 .20153 m
.50375 .20153 L
s
.5 .24179 m
.50375 .24179 L
s
.5 .26192 m
.50375 .26192 L
s
.5 .28205 m
.50375 .28205 L
s
.5 .32232 m
.50375 .32232 L
s
.5 .34245 m
.50375 .34245 L
s
.5 .36258 m
.50375 .36258 L
s
.5 .40284 m
.50375 .40284 L
s
.5 .42297 m
.50375 .42297 L
s
.5 .44311 m
.50375 .44311 L
s
.5 .48337 m
.50375 .48337 L
s
.5 .5035 m
.50375 .5035 L
s
.5 .52363 m
.50375 .52363 L
s
.5 .04048 m
.50375 .04048 L
s
.5 .02035 m
.50375 .02035 L
s
.5 .00021 m
.50375 .00021 L
s
.5 .56389 m
.50375 .56389 L
s
.5 .58403 m
.50375 .58403 L
s
.5 .60416 m
.50375 .60416 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.27734 0 m
.28086 .01949 L
.30164 .11251 L
.32109 .18189 L
.34251 .2418 L
.36282 .28536 L
.38186 .3165 L
.4021 .3412 L
.42063 .35757 L
.43047 .36423 L
.43961 .3693 L
.45028 .37403 L
.46034 .37741 L
.4702 .37984 L
.47575 .38085 L
.48086 .38158 L
.48517 .38204 L
.48765 .38225 L
.48991 .38241 L
.49229 .38254 L
.49352 .38259 L
.49485 .38263 L
.49608 .38267 L
.49722 .38269 L
.49831 .3827 L
.49947 .38271 L
.5006 .38271 L
.50185 .3827 L
.50315 .38268 L
.50436 .38266 L
.50549 .38263 L
.50652 .38259 L
.50884 .38249 L
.51133 .38236 L
.51403 .38218 L
.51894 .38175 L
.52845 .38063 L
.53722 .37926 L
.55598 .37541 L
.57662 .37002 L
.61696 .35717 L
.65578 .34389 L
.67553 .33769 L
.68596 .33478 L
.69706 .33208 L
.70789 .32996 L
.713 .32916 L
.71769 .32857 L
.72243 .32811 L
.72492 .32793 L
Mistroke
.72619 .32786 L
.72755 .32779 L
.72875 .32775 L
.72984 .32771 L
.73094 .32769 L
.73197 .32767 L
.73318 .32767 L
.7343 .32768 L
.73552 .32769 L
.73682 .32773 L
.73796 .32777 L
.73921 .32783 L
.74173 .32799 L
.74405 .3282 L
.74622 .32843 L
.75111 .32915 L
.75633 .33019 L
.76127 .33147 L
.76592 .33296 L
.77462 .33653 L
.78437 .34188 L
.79468 .34928 L
.81314 .3678 L
.82261 .38032 L
.83276 .39637 L
.85108 .4332 L
.8723 .49073 L
.89241 .56311 L
Mfstroke
.89241 .56311 m
.90428 .61803 L
s
0 0 1 r
.2866 0 m
.30952 .13029 L
.35714 .27245 L
.40476 .34374 L
.45238 .37481 L
.5 .38271 L
.54762 .37726 L
.59524 .36427 L
.64286 .34722 L
.69048 .32825 L
.7381 .30863 L
.78571 .28918 L
.83333 .27034 L
.88095 .25238 L
.92857 .23544 L
.97619 .21958 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oC`3oool0
10000000oooo0?ooo`000?ll0?ooo`800000S`3oool004l0oooo00@000000?ooo`3oool0003o?03o
ool00`000000oooo0?ooo`2>0?ooo`00C`3oool010000000oooo0?ooo`000?ll0?ooo`030000003o
ool0oooo08h0oooo001@0?ooo`030000003oool0003o03`0oooo00<000000?ooo`3oool0SP3oool0
0500oooo00@000000?ooo`3oool0003o>`3oool00`000000oooo0?ooo`2>0?ooo`00D03oool01000
0000oooo0?ooo`000?lk0?ooo`800000S`3oool00500oooo00@000000?ooo`3oool0003o>`3oool0
0`000000oooo0?ooo`2>0?ooo`00D03oool010000000oooo0?ooo`000?lk0?ooo`030000003oool0
oooo08h0oooo001A0?ooo`030000003oool0003o03/0oooo00<000000?ooo`3oool0SP3oool00540
oooo00<000000?ooo`000?l0>`3oool00`000000oooo0?ooo`2>0?ooo`00D@3oool010000000oooo
0?ooo`000?lj0?ooo`030000003oool0oooo08h0oooo001A0?ooo`040000003oool0oooo0000ocX0
oooo0P00002?0?ooo`00DP3oool00`000000oooo0000o`0j0?ooo`030000003oool0oooo08h0oooo
001B0?ooo`030000003oool0003o03X0oooo00<000000?ooo`3oool0SP3oool00580oooo00<00000
0?ooo`000?l0>P3oool00`000000oooo0?ooo`2>0?ooo`00DP3oool00`000000oooo0000o`0J0?oo
o`800000103oool00`000000oooo0?ooo`030?ooo`800000103oool2000000@0oooo0P0000060?oo
o`030000003oool0oooo08h0oooo001B0?ooo`040000003oool0oooo0000oaP0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`2>0?ooo`00D`3oool00`000000oooo
0000o`0H0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo0`00002>0?ooo`00D`3oool0
0`000000oooo0000o`0B0?ooo`@000000P3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000000`3oool2000000H0oooo00<000000?ooo`3o
ool0SP3oool005<0oooo00<000000?ooo`000?l0603oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`00
00050?ooo`030000003oool0oooo08h0oooo001C0?ooo`030000003oool0003o01P0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080
oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`2>0?ooo`00E03oool00`000000
003o0?ooo`0H0?ooo`8000002P3oool2000000@0oooo0P0000040?ooo`8000001P3oool00`000000
oooo0?ooo`2>0?ooo`00E03oool00`000000oooo0000o`0h0?ooo`030000003oool0oooo08h0oooo
001D0?ooo`030000003oool0003o03P0oooo0P00002?0?ooo`00E03oool00`000000oooo0000o`0h
0?ooo`030000003oool0oooo08h0oooo001D0?ooo`030000003oool0003o03P0oooo00<000000?oo
o`3oool0SP3oool005D0oooo00<000000000o`3oool0=`3oool00`000000oooo0?ooo`2>0?ooo`00
E@3oool00`000000003o0?ooo`0g0?ooo`030000003oool0oooo08h0oooo001E0?ooo`030000003o
ool0003o03L0oooo0P00002?0?ooo`00E@3oool00`000000oooo0000o`0g0?ooo`030000003oool0
oooo08h0oooo001F0?ooo`03000000000?l0oooo03H0oooo00<000000?ooo`3oool0SP3oool005H0
oooo00<000000000o`3oool0=P3oool00`000000oooo0?ooo`2>0?ooo`00EP3oool00`000000003o
0?ooo`0f0?ooo`030000003oool0oooo08h0oooo001F0?ooo`03000000000?l0oooo03H0oooo00<0
00000?ooo`3oool0SP3oool005L0oooo00<000000000o`3oool0=@3oool2000008l0oooo001G0?oo
o`03000000000?l0oooo03D0oooo00<000000?ooo`3oool0SP3oool005L0oooo00<000000000o`3o
ool0=@3oool00`000000oooo0?ooo`2>0?ooo`00E`3oool00`000000003o0?ooo`0e0?ooo`030000
003oool0oooo08h0oooo001H0?ooo`030000o`3oool0oooo01@0oooo0P0000040?ooo`030000003o
ool0oooo00<0oooo0P0000040?ooo`800000103oool2000000H0oooo00<000000?ooo`3oool0SP3o
ool005P0oooo00<000000000o`3oool04`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?oo
o`030000003oool0oooo08h0oooo001H0?ooo`03000000000?l0oooo01<0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool000001@3oool3000008h0oooo001I0?ooo`030000o`3oool0oooo00`0oooo1000
00020?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000
oooo0?ooo`0000020?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`00F@3oool00`000000
003o0?ooo`0B0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3o
ool005T0oooo00<000000000o`3oool04P3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo08h0oooo001J0?ooo`030000o`3oool0oooo0180oooo0P00000:0?ooo`800000
103oool2000000@0oooo0`0000050?ooo`030000003oool0oooo08h0oooo001J0?ooo`0300000000
0?l0oooo0380oooo00<000000?ooo`3oool0SP3oool005X0oooo00<000000000o`3oool0<P3oool2
000008l0oooo001J0?ooo`03000000000?l0oooo0380oooo00<000000?ooo`3oool0SP3oool005/0
oooo00<000000000o`3oool0<@3oool00`000000oooo0?ooo`2>0?ooo`00F`3oool00`000000003o
0?ooo`0a0?ooo`030000003oool0oooo08h0oooo001K0?ooo`03000000000?l0oooo0340oooo00<0
00000?ooo`3oool0SP3oool005`0oooo00<000000000o`3oool0<03oool2000008l0oooo001L0?oo
o`03000000000?l0oooo0300oooo00<000000?ooo`3oool0SP3oool005`0oooo00<000000000o`3o
ool0<03oool00`000000oooo0?ooo`2>0?ooo`00G@3oool00`000000003o0?ooo`0_0?ooo`030000
003oool0oooo08h0oooo001M0?ooo`03000000000?l0oooo02l0oooo00<000000?ooo`3oool0SP3o
ool005d0oooo00<000000000o`3oool0;`3oool00`000000oooo0?ooo`2>0?ooo`00GP3oool00`00
0000003o0?ooo`0^0?ooo`800000S`3oool005h0oooo00<000000000o`3oool0;P3oool00`000000
oooo0?ooo`2>0?ooo`00GP3oool00`000000oooo0000o`0^0?ooo`030000003oool0oooo08h0oooo
001O0?ooo`03000000000?l0oooo02d0oooo00<000000?ooo`3oool0SP3oool005l0oooo00<00000
0000o`3oool03@3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P000005
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool00600oooo00<000000000
o`3oool02`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08@0oooo
0P000?l80?ooo`00H03oool00`000000003o0?ooo`0;0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool01@000000oooo0?ooo`000000oooo00D000001@3oool30000
0880oooo0P000?l:0?ooo`00H03oool00`000000003o0?ooo`050?ooo`@000000P3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00H000000?ooo`3oool000000?oo
o`0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0O`3oool30000o``0oooo
001Q0?ooo`03000000000?l0oooo00X0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`0000001P3oool00`00
0000oooo0?ooo`1l0?ooo`<0003o3`3oool00640oooo00<000000000o`3oool02P3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000`3o
ool2000000H0oooo00<000000?ooo`3oool0NP3oool20000oa80oooo001Q0?ooo`03000000000?l0
oooo00/0oooo0P00000:0?ooo`800000103oool2000000D0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`1g0?ooo`<0003o503oool00680oooo00<000000000o`3oool0:P3oool00`00
0000oooo0?ooo`1d0?ooo`<0003o5`3oool00680oooo00<000000000o`3oool0:P3oool200000780
oooo0`000?lJ0?ooo`00HP3oool00`000000003o0?ooo`0Z0?ooo`030000003oool0oooo06l0oooo
0P000?lM0?ooo`00H`3oool00`000000003o0?ooo`0Y0?ooo`030000003oool0oooo06`0oooo0`00
0?lO0?ooo`00H`3oool00`000000003o0?ooo`0Y0?ooo`030000003oool0oooo06T0oooo0`000?lR
0?ooo`00I03oool00`000?l0oooo0?ooo`0X0?ooo`030000003oool0oooo06H0oooo0`000?lU0?oo
o`00I03oool00`000000003o0?ooo`0X0?ooo`030000003oool0oooo06<0oooo0`000?lX0?ooo`00
I@3oool00`000?l0oooo0?ooo`0W0?ooo`800000HP3oool20000ob/0oooo001U0?ooo`030000o`3o
ool0oooo02L0oooo00<000000?ooo`3oool0GP3oool30000obd0oooo001V0?ooo`030000o`3oool0
oooo02H0oooo00<000000?ooo`3oool0F`3oool30000oc00oooo001V0?ooo`030000o`3oool0oooo
02H0oooo00<000000?ooo`3oool0F@3oool20000oc<0oooo001W0?ooo`030000o`3oool0oooo02D0
oooo00<000000?ooo`3oool0EP3oool30000ocD0oooo001W0?ooo`030000o`3oool0oooo02D0oooo
0P00001D0?ooo`<0003o>03oool006P0oooo00<0003o0?ooo`3oool0903oool00`000000oooo0?oo
o`1A0?ooo`80003o>`3oool006T0oooo00<0003o0?ooo`3oool08`3oool00`000000oooo0?ooo`1>
0?ooo`<0003o?@3oool006T0oooo00<0003o0?ooo`3oool08`3oool00`000000oooo0?ooo`1<0?oo
o`80003o@03oool006X0oooo00<0003o0?ooo`3oool00P3oool2000000@0oooo00<000000?ooo`3o
ool00`3oool2000000@0oooo0P0000030?ooo`@000001@3oool00`000000oooo0?ooo`1:0?ooo`80
003o@P3oool006X0oooo00D000000000o`3oool0oooo000000020?ooo`030000003oool0oooo00H0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool0A`3oool30000od@0oooo001[0?ooo`04000000000?l0oooo
00000080oooo00<000000?ooo`3oool01P3oool010000000oooo0?ooo`0000020?ooo`040000003o
ool0oooo000000<0oooo00<000000?ooo`3oool01@3oool3000004D0oooo0P000?m70?ooo`00J03o
ool4000000030000o`3oool000000080oooo00<000000?ooo`3oool01P3oool010000000oooo0?oo
o`0000020?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`130?ooo`80003oB@3oool006`0oooo00<000000000o`0000000P3oool00`000000oooo
0?ooo`060?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0400oooo0`000?m;0?ooo`00K03oool00`000000
oooo0000o`020?ooo`030000003oool0oooo00H0oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0?P3o
ool20000odh0oooo001]0?ooo`04000000000?l00000000000X0oooo0P0000040?ooo`800000103o
ool2000000H0oooo00<000000?ooo`3oool0?03oool20000oe00oooo001^0?ooo`03000000000?l0
oooo01h0oooo00<000000?ooo`3oool0>P3oool20000oe80oooo001_0?ooo`03000000000?l0oooo
01d0oooo0P00000i0?ooo`80003oE03oool00700oooo00<0003o0?ooo`3oool0703oool00`000000
oooo0?ooo`0f0?ooo`80003o103oool7000004/0oooo001`0?ooo`03000000000?l0oooo01`0oooo
00<000000?ooo`3oool0<`3oool30000o`030?ooo`000000000000<000001`3oool6000004D0oooo
001a0?ooo`03000000000?l0oooo01/0oooo00<000000?ooo`3oool0<03oool30000o`@000004P3o
ool2000004<0oooo001b0?ooo`03000000000?l0oooo01X0oooo00<000000?ooo`3oool0;@3oool3
0000o`030000003oool0oooo01P0oooo0P0000110?ooo`00L`3oool00`000?l0oooo0?ooo`0I0?oo
o`030000003oool0oooo02/0oooo0P000?l00`000000oooo0?ooo`0M0?ooo`800000?`3oool007@0
oooo00<0003o0?ooo`3oool0603oool2000002T0oooo0`000?l00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo03`0oooo00080?ooo`@00000>`3oool2000000@0oooo00<000000?ooo`3oool0
0P3oool300000200oooo0P000?lH0?ooo`030000003oool0oooo02D0oooo0`000?l00`000000oooo
0?ooo`0?0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<00000203oool00`000000oooo
0?ooo`0`0?ooo`@000001`3oool000X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo01h0oooo00<000000000o`3oool05P3oool00`000000oooo
0?ooo`0R0?ooo`<0003o00<000000?ooo`3oool04@3oool010000000oooo0?ooo`00000;0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`060?ooo`00
2P3oool00`000000oooo0?ooo`0i0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3o
ool07`3oool00`000000003o0000o`0E0?ooo`030000003oool0oooo01l0oooo0`000?l00`000000
oooo0?ooo`0D0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0`0?ooo`030000003oool0oooo00H0oooo00020?ooo`@00000103oool00`000000
oooo0?ooo`0c0?ooo`@000000P3oool010000000oooo0?ooo`0000080?ooo`<000008`3oool00`00
0000oooo0000o`0D0?ooo`030000003oool0oooo01d0oooo0P000?l00`000000oooo0?ooo`0G0?oo
o`040000003oool0oooo000000P0oooo0`00000;0?ooo`030000003oool0oooo02l0oooo00<00000
0?ooo`3oool01P3oool000X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`000008
0?ooo`030000003oool0oooo02@0oooo0P0000000`000?l0oooo0?ooo`0A0?ooo`8000006P3oool0
10000000003o0000o`000?lL0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0
303oool2000002l0oooo00<000000?ooo`3oool01P3oool000P0oooo0`00000k0?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`3oool09P3oool00`000000003o0000o`0A0?ooo`030000
003oool0oooo01D0oooo0P0000030000oal0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo02X0oooo0`0000080?ooo`002P3oool00`000000oooo
0?ooo`0j0?ooo`8000002@3oool4000002L0oooo00<000000000o`3oool03`3oool00`000000oooo
0?ooo`0B0?ooo`8000000`000?lS0?ooo`8000002@3oool4000000h0oooo00<000000?ooo`3oool0
:`3oool00`000000oooo0?ooo`060?ooo`00O`3oool20000o`h0oooo00<000000?ooo`3oool03`3o
ool00`000000003o0000o`020000od@0oooo00<000000?ooo`3oool0<`3oool00840oooo10000?l:
0?ooo`030000003oool0oooo00/0oooo0P0000030000odP0oooo00<000000?ooo`3oool0<`3oool0
08<0oooo0P0000070000o`<0oooo00<000000?ooo`3oool0103oool90000od`0oooo00<000000?oo
o`3oool0<P3oool008`000002P000?n900000040oooo00070?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0203oool00`000000oooo0000000=0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?ooo`001`3oool00`000000oooo0?oo
o`110?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool01P3oool008l0oooo00<00000
0?ooo`3oool0F`3oool00`000000oooo0?ooo`0`0?ooo`00S`3oool00`000000oooo0?ooo`1L0?oo
o`030000003oool0oooo02l0oooo002?0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3o
ool0;P3oool008l0oooo0P00001N0?ooo`030000003oool0oooo02h0oooo002?0?ooo`030000003o
ool0oooo05h0oooo00<000000?ooo`3oool0;@3oool008l0oooo00<000000?ooo`3oool0GP3oool0
0`000000oooo0?ooo`0]0?ooo`00S`3oool00`000000oooo0?ooo`1O0?ooo`030000003oool0oooo
02`0oooo002?0?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3oool0;03oool008l0oooo
00<000000?ooo`3oool0H03oool00`000000oooo0?ooo`0[0?ooo`00S`3oool200000640oooo00<0
00000?ooo`3oool0:`3oool008l0oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?ooo`0Z
0?ooo`00S`3oool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo02X0oooo002?0?ooo`03
0000003oool0oooo0680oooo00<000000?ooo`3oool0:@3oool008l0oooo00<000000?ooo`3oool0
HP3oool00`000000oooo0?ooo`0Y0?ooo`00S`3oool2000006@0oooo00<000000?ooo`3oool0:03o
ool008l0oooo00<000000?ooo`3oool0H`3oool00`000000oooo0?ooo`0X0?ooo`00S`3oool00`00
0000oooo0?ooo`1S0?ooo`030000003oool0oooo02P0oooo002?0?ooo`030000003oool0oooo06@0
oooo00<000000?ooo`3oool09`3oool006l0oooo0P0000040?ooo`030000003oool0oooo00<0oooo
0P0000040?ooo`8000000`3oool4000000D0oooo00<000000?ooo`3oool0I03oool00`000000oooo
0?ooo`0W0?ooo`00KP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080
oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
06D0oooo00<000000?ooo`3oool09P3oool006h0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo000000<0oooo00<000000?ooo`3oool01@3o
ool3000006D0oooo00<000000?ooo`3oool09P3oool006h0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo02H0oooo001^0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`0U
0?ooo`00KP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo06H0
oooo00<000000?ooo`3oool09@3oool006l0oooo0P00000:0?ooo`800000103oool2000000@0oooo
0P0000060?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool09@3oool008l0oooo00<0
00000?ooo`3oool0I`3oool00`000000oooo0?ooo`0T0?ooo`00S`3oool2000006P0oooo00<00000
0?ooo`3oool0903oool008l0oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`0S0?oo
o`00S`3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo02<0oooo002?0?ooo`030000
003oool0oooo06P0oooo00<000000?ooo`3oool08`3oool008l0oooo00<000000?ooo`3oool0J03o
ool00`000000oooo0?ooo`0S0?ooo`00S`3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0
oooo0280oooo002?0?ooo`800000JP3oool00`000000oooo0?ooo`0R0?ooo`00S`3oool00`000000
oooo0?ooo`1Y0?ooo`030000003oool0oooo0280oooo002?0?ooo`030000003oool0oooo06T0oooo
00<000000?ooo`3oool08P3oool008l0oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?oo
o`0Q0?ooo`00S`3oool00`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo0240oooo002?0?oo
o`800000J`3oool00`000000oooo0?ooo`0Q0?ooo`00S`3oool00`000000oooo0?ooo`1Z0?ooo`03
0000003oool0oooo0240oooo002?0?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0
803oool008l0oooo00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`0P0?ooo`00K`3oool2
000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P0000050?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`0P0?ooo`00KP3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000
103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0
803oool006h0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`05
0000003oool0oooo0000003oool01@0000050?ooo`<00000K03oool00`000000oooo0?ooo`0O0?oo
o`00KP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00H00000
0?ooo`3oool000000?ooo`0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
K03oool00`000000oooo0?ooo`0O0?ooo`00KP3oool010000000oooo0?ooo`0000080?ooo`040000
003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo000000060?oo
o`030000003oool0oooo06`0oooo00<000000?ooo`3oool07`3oool006h0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000<0oooo0P00
00060?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool07`3oool006l0oooo0P00000:
0?ooo`800000103oool2000000D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1]
0?ooo`030000003oool0oooo01h0oooo002?0?ooo`030000003oool0oooo06d0oooo00<000000?oo
o`3oool07P3oool008l0oooo0P00001^0?ooo`030000003oool0oooo01h0oooo002?0?ooo`030000
003oool0oooo06d0oooo00<000000?ooo`3oool07P3oool008l0oooo00<000000?ooo`3oool0KP3o
ool00`000000oooo0?ooo`0M0?ooo`00S`3oool00`000000oooo0?ooo`1^0?ooo`030000003oool0
oooo01d0oooo002?0?ooo`030000003oool0oooo06h0oooo00<000000?ooo`3oool07@3oool008l0
oooo00<000000?ooo`3oool0KP3oool00`000000oooo0?ooo`0M0?ooo`00S`3oool200000700oooo
00<000000?ooo`3oool0703oool008l0oooo00<000000?ooo`3oool0K`3oool00`000000oooo0?oo
o`0L0?ooo`00S`3oool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo01`0oooo002?0?oo
o`030000003oool0oooo06l0oooo00<000000?ooo`3oool0703oool008l0oooo00<000000?ooo`3o
ool0L03oool00`000000oooo0?ooo`0K0?ooo`00S`3oool00`000000oooo0?ooo`1`0?ooo`030000
003oool0oooo01/0oooo002?0?ooo`800000L@3oool00`000000oooo0?ooo`0K0?ooo`00S`3oool0
0`000000oooo0?ooo`1`0?ooo`030000003oool0oooo01/0oooo002?0?ooo`030000003oool0oooo
0740oooo00<000000?ooo`3oool06P3oool008l0oooo00<000000?ooo`3oool0L@3oool00`000000
oooo0?ooo`0J0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05261, -0.0095054, \
0.00733523, 8.67539*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Real\ \(\(part\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[delta[1, 1, 1/10, 1/2, a]], {a, \(-1\), 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 0.536191 3.2481 [
[.02381 .52369 -6 -9 ]
[.02381 .52369 6 0 ]
[.2619 .52369 -12 -9 ]
[.2619 .52369 12 0 ]
[.7381 .52369 -9 -9 ]
[.7381 .52369 9 0 ]
[.97619 .52369 -3 -9 ]
[.97619 .52369 3 0 ]
[.4875 .04898 -30 -4.5 ]
[.4875 .04898 0 4.5 ]
[.4875 .13018 -36 -4.5 ]
[.4875 .13018 0 4.5 ]
[.4875 .21138 -24 -4.5 ]
[.4875 .21138 0 4.5 ]
[.4875 .29258 -36 -4.5 ]
[.4875 .29258 0 4.5 ]
[.4875 .37379 -30 -4.5 ]
[.4875 .37379 0 4.5 ]
[.4875 .45499 -36 -4.5 ]
[.4875 .45499 0 4.5 ]
[.4875 .61739 -30 -4.5 ]
[.4875 .61739 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .53619 m
.02381 .54244 L
s
[(-1)] .02381 .52369 0 1 Mshowa
.2619 .53619 m
.2619 .54244 L
s
[(-0.5)] .2619 .52369 0 1 Mshowa
.7381 .53619 m
.7381 .54244 L
s
[(0.5)] .7381 .52369 0 1 Mshowa
.97619 .53619 m
.97619 .54244 L
s
[(1)] .97619 .52369 0 1 Mshowa
.125 Mabswid
.07143 .53619 m
.07143 .53994 L
s
.11905 .53619 m
.11905 .53994 L
s
.16667 .53619 m
.16667 .53994 L
s
.21429 .53619 m
.21429 .53994 L
s
.30952 .53619 m
.30952 .53994 L
s
.35714 .53619 m
.35714 .53994 L
s
.40476 .53619 m
.40476 .53994 L
s
.45238 .53619 m
.45238 .53994 L
s
.54762 .53619 m
.54762 .53994 L
s
.59524 .53619 m
.59524 .53994 L
s
.64286 .53619 m
.64286 .53994 L
s
.69048 .53619 m
.69048 .53994 L
s
.78571 .53619 m
.78571 .53994 L
s
.83333 .53619 m
.83333 .53994 L
s
.88095 .53619 m
.88095 .53994 L
s
.92857 .53619 m
.92857 .53994 L
s
.25 Mabswid
0 .53619 m
1 .53619 L
s
.5 .04898 m
.50625 .04898 L
s
[(-0.15)] .4875 .04898 1 0 Mshowa
.5 .13018 m
.50625 .13018 L
s
[(-0.125)] .4875 .13018 1 0 Mshowa
.5 .21138 m
.50625 .21138 L
s
[(-0.1)] .4875 .21138 1 0 Mshowa
.5 .29258 m
.50625 .29258 L
s
[(-0.075)] .4875 .29258 1 0 Mshowa
.5 .37379 m
.50625 .37379 L
s
[(-0.05)] .4875 .37379 1 0 Mshowa
.5 .45499 m
.50625 .45499 L
s
[(-0.025)] .4875 .45499 1 0 Mshowa
.5 .61739 m
.50625 .61739 L
s
[(0.025)] .4875 .61739 1 0 Mshowa
.125 Mabswid
.5 .06522 m
.50375 .06522 L
s
.5 .08146 m
.50375 .08146 L
s
.5 .0977 m
.50375 .0977 L
s
.5 .11394 m
.50375 .11394 L
s
.5 .14642 m
.50375 .14642 L
s
.5 .16266 m
.50375 .16266 L
s
.5 .1789 m
.50375 .1789 L
s
.5 .19514 m
.50375 .19514 L
s
.5 .22762 m
.50375 .22762 L
s
.5 .24386 m
.50375 .24386 L
s
.5 .2601 m
.50375 .2601 L
s
.5 .27634 m
.50375 .27634 L
s
.5 .30882 m
.50375 .30882 L
s
.5 .32506 m
.50375 .32506 L
s
.5 .3413 m
.50375 .3413 L
s
.5 .35755 m
.50375 .35755 L
s
.5 .39003 m
.50375 .39003 L
s
.5 .40627 m
.50375 .40627 L
s
.5 .42251 m
.50375 .42251 L
s
.5 .43875 m
.50375 .43875 L
s
.5 .47123 m
.50375 .47123 L
s
.5 .48747 m
.50375 .48747 L
s
.5 .50371 m
.50375 .50371 L
s
.5 .51995 m
.50375 .51995 L
s
.5 .55243 m
.50375 .55243 L
s
.5 .56867 m
.50375 .56867 L
s
.5 .58491 m
.50375 .58491 L
s
.5 .60115 m
.50375 .60115 L
s
.5 .03274 m
.50375 .03274 L
s
.5 .0165 m
.50375 .0165 L
s
.5 .00025 m
.50375 .00025 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .12094 L
.10458 .21506 L
.14415 .28625 L
.18221 .3417 L
.22272 .38921 L
.26171 .42579 L
.30316 .45677 L
.34309 .48048 L
.3815 .49878 L
.42237 .51443 L
.46172 .52657 L
.49955 .53609 L
.53984 .54439 L
.57861 .55095 L
.61984 .55669 L
.65954 .56127 L
.69774 .56503 L
.73838 .56856 L
.77751 .5718 L
.81909 .57547 L
.85916 .57975 L
.89771 .58515 L
.93871 .59312 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool2
000008l0oooo002?0?ooo`030000003oool0oooo08h0oooo002?0?ooo`030000003oool0oooo08h0
oooo002?0?ooo`030000003oool0oooo08h0oooo000:0?ooo`030000003oool0oooo0880oooo0P00
002?0?ooo`002P3oool00`000000oooo0?ooo`220?ooo`030000003oool0oooo08h0oooo000;0?oo
o`030000003oool0oooo0840oooo00<000000?ooo`3oool0SP3oool000/0oooo00<000000?ooo`3o
ool0P@3oool00`000000oooo0?ooo`2>0?ooo`00303oool00`000000oooo0?ooo`200?ooo`030000
003oool0oooo08h0oooo000<0?ooo`030000003oool0oooo0800oooo0P00002?0?ooo`00303oool0
0`000000oooo0?ooo`200?ooo`030000003oool0oooo08h0oooo000=0?ooo`030000003oool0oooo
06H0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?ooo`<000001@3oool00`00
0000oooo0?ooo`2>0?ooo`003@3oool00`000000oooo0?ooo`1U0?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
08h0oooo000=0?ooo`030000003oool0oooo06D0oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo0`00002>0?ooo`003P3oool00`000000
oooo0?ooo`1N0?ooo`@000000P3oool010000000oooo0?ooo`00000:0?ooo`040000003oool0oooo
0?ooo`<000001@3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000000oooo0?ooo`1T0?oo
o`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo000000070?ooo`030000003o
ool0oooo08h0oooo000?0?ooo`030000003oool0oooo06<0oooo00@000000?ooo`3oool00000203o
ool3000000<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool0
0`000000oooo0?ooo`1T0?ooo`8000002`3oool010000000oooo0?ooo`3oool4000000@0oooo00<0
00000?ooo`3oool0SP3oool000l0oooo00<000000?ooo`3oool0O@3oool2000008l0oooo000@0?oo
o`030000003oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<000000?ooo`3o
ool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000000oooo0?ooo`1l0?ooo`030000
003oool0oooo08h0oooo000A0?ooo`030000003oool0oooo07/0oooo0P00002?0?ooo`004@3oool0
0`000000oooo0?ooo`1k0?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000003oool0oooo
07X0oooo00<000000?ooo`3oool0SP3oool00180oooo00<000000?ooo`3oool0NP3oool00`000000
oooo0?ooo`2>0?ooo`004P3oool00`000000oooo0?ooo`1j0?ooo`030000003oool0oooo08h0oooo
000C0?ooo`030000003oool0oooo07T0oooo0P00002?0?ooo`004`3oool00`000000oooo0?ooo`1i
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000003oool0oooo07T0oooo00<000000?oo
o`3oool0SP3oool001@0oooo00<000000?ooo`3oool0N03oool00`000000oooo0?ooo`2>0?ooo`00
503oool00`000000oooo0?ooo`1h0?ooo`800000S`3oool001D0oooo00<000000?ooo`3oool0M`3o
ool00`000000oooo0?ooo`2>0?ooo`005@3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0
oooo08h0oooo000E0?ooo`030000003oool0oooo05P0oooo0P0000040?ooo`030000003oool0oooo
0080oooo100000020?ooo`@000000P3oool3000000D0oooo00<000000?ooo`3oool0SP3oool001H0
oooo00<000000?ooo`3oool0EP3oool010000000oooo0?ooo`00000:0?ooo`050000003oool0oooo
0?ooo`000000203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo08h0oooo000F0?oo
o`030000003oool0oooo05H0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool3000008h0oooo000G0?oo
o`030000003oool0oooo04l0oooo100000020?ooo`040000003oool0oooo000000X0oooo00<00000
0?ooo`3oool00`3oool010000000oooo0?ooo`3oool3000000D0oooo00<000000?ooo`3oool0SP3o
ool001L0oooo00<000000?ooo`3oool0E@3oool010000000oooo0?ooo`00000:0?ooo`030000003o
ool0oooo00@0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`2>0?ooo`00603o
ool00`000000oooo0?ooo`1D0?ooo`040000003oool0oooo000000P0oooo0`0000030?ooo`040000
003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2>0?ooo`00
603oool00`000000oooo0?ooo`1E0?ooo`8000002`3oool00`000000oooo0?ooo`020?ooo`800000
0`3oool4000000@0oooo0P00002?0?ooo`006@3oool00`000000oooo0?ooo`1c0?ooo`030000003o
ool0oooo08h0oooo000I0?ooo`030000003oool0oooo07<0oooo00<000000?ooo`3oool0SP3oool0
01T0oooo00<000000?ooo`3oool0L`3oool00`000000oooo0?ooo`2>0?ooo`006P3oool00`000000
oooo0?ooo`1b0?ooo`030000003oool0oooo08h0oooo000J0?ooo`030000003oool0oooo0780oooo
0P00002?0?ooo`006`3oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo08h0oooo000K
0?ooo`030000003oool0oooo0740oooo00<000000?ooo`3oool0SP3oool001`0oooo00<000000?oo
o`3oool0L03oool00`000000oooo0?ooo`2>0?ooo`00703oool00`000000oooo0?ooo`1`0?ooo`03
0000003oool0oooo08h0oooo000M0?ooo`030000003oool0oooo06l0oooo0P00002?0?ooo`007@3o
ool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo08h0oooo000M0?ooo`030000003oool0
oooo06l0oooo00<000000?ooo`3oool0SP3oool001h0oooo00<000000?ooo`3oool0KP3oool00`00
0000oooo0?ooo`2>0?ooo`007P3oool00`000000oooo0?ooo`1^0?ooo`800000S`3oool001l0oooo
00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`2>0?ooo`007`3oool00`000000oooo0?oo
o`1]0?ooo`030000003oool0oooo08h0oooo000P0?ooo`030000003oool0oooo05T0oooo0P000004
0?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo08h0oooo000P0?oo
o`030000003oool0oooo05P0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo08h0oooo000Q0?ooo`030000003oool0oooo05L0oooo00@000000?oo
o`3oool000002P3oool00`000000oooo0?ooo`030?ooo`<00000SP3oool00240oooo00<000000?oo
o`3oool0D@3oool400000080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo08h0oooo000R0?ooo`030000003oool0oooo05H0oooo00@000000?oo
o`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08h0oooo000R0?oo
o`030000003oool0oooo05H0oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<00000
0?ooo`3oool0SP3oool002<0oooo00<000000?ooo`3oool0EP3oool2000000/0oooo00<000000?oo
o`3oool00`3oool2000008l0oooo000S0?ooo`030000003oool0oooo06T0oooo00<000000?ooo`3o
ool0SP3oool002@0oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`2>0?ooo`00903o
ool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo08h0oooo000U0?ooo`030000003oool0
oooo06L0oooo00<000000?ooo`3oool0SP3oool002D0oooo00<000000?ooo`3oool0I`3oool20000
08l0oooo000V0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0SP3oool002H0oooo
00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`2>0?ooo`009`3oool00`000000oooo0?oo
o`1U0?ooo`030000003oool0oooo08h0oooo000X0?ooo`030000003oool0oooo06@0oooo0P00002?
0?ooo`00:03oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo08h0oooo000Y0?ooo`03
0000003oool0oooo06<0oooo00<000000?ooo`3oool0SP3oool002T0oooo00<000000?ooo`3oool0
H`3oool00`000000oooo0?ooo`2>0?ooo`00:P3oool00`000000oooo0?ooo`1R0?ooo`030000003o
ool0oooo08h0oooo000Z0?ooo`030000003oool0oooo0680oooo0P00002?0?ooo`00:`3oool00`00
0000oooo0?ooo`1Q0?ooo`030000003oool0oooo08h0oooo000[0?ooo`030000003oool0oooo0480
oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000040?ooo`030000003oool0oooo0080
oooo0`0000050?ooo`030000003oool0oooo08h0oooo000/0?ooo`030000003oool0oooo0400oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2>0?ooo`00;@3oool00`000000
oooo0?ooo`0o0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo0`00002>0?ooo`00;@3oool00`00
0000oooo0?ooo`0i0?ooo`@000000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo000000@0oooo00@000000?ooo`3oool0oooo0`0000050?ooo`030000003oool0oooo08h0oooo
000^0?ooo`030000003oool0oooo03h0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000050?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0SP3oool002l0
oooo00<000000?ooo`3oool0?@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo08h0oooo000_0?ooo`030000003oool0oooo03h0oooo0P00000:0?ooo`8000000`3oool4
00000080oooo100000040?ooo`030000003oool0oooo08h0oooo000`0?ooo`030000003oool0oooo
05`0oooo0P00002?0?ooo`00<@3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo08h0
oooo000a0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0SP3oool00380oooo00<0
00000?ooo`3oool0FP3oool00`000000oooo0?ooo`2>0?ooo`00<`3oool00`000000oooo0?ooo`1I
0?ooo`800000S`3oool003<0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`2>0?oo
o`00=03oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo08h0oooo000e0?ooo`030000
003oool0oooo05L0oooo00<000000?ooo`3oool0SP3oool003D0oooo00<000000?ooo`3oool0E`3o
ool00`000000oooo0?ooo`2>0?ooo`00=P3oool00`000000oooo0?ooo`1F0?ooo`800000S`3oool0
03L0oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`2>0?ooo`00>03oool00`000000
oooo0?ooo`1D0?ooo`030000003oool0oooo08h0oooo000i0?ooo`030000003oool0oooo05<0oooo
00<000000?ooo`3oool0SP3oool003X0oooo00<000000?ooo`3oool0DP3oool2000008l0oooo000k
0?ooo`030000003oool0oooo0540oooo00<000000?ooo`3oool0SP3oool003`0oooo00<000000?oo
o`3oool0D03oool00`000000oooo0?ooo`2>0?ooo`00?03oool00`000000oooo0?ooo`0g0?ooo`80
0000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool3000000D0oooo00<000000?ooo`3o
ool0SP3oool003d0oooo00<000000?ooo`3oool0=@3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2>0?oo
o`00?P3oool00`000000oooo0?ooo`0d0?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool000001@3oool00`000000oooo0?ooo`020?ooo`<00000SP3oool003l0oooo00<000000?oo
o`3oool0;@3oool400000080oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00020?ooo`<000001@3oool00`000000oooo0?ooo`2>0?ooo`00@03oool00`000000oooo0?ooo`0b
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo08h0oooo00110?ooo`030000003oool0oooo0340oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0SP3oool00480oooo00<000000?ooo`3oool0<@3oool2000000X0oooo
0P0000030?ooo`@00000103oool2000008l0oooo00130?ooo`030000003oool0oooo04T0oooo00<0
00000?ooo`3oool0SP3oool004@0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`2>
0?ooo`00A@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo08h0oooo00160?ooo`03
0000003oool0oooo04H0oooo00<000000?ooo`3oool0SP3oool004L0oooo00<000000?ooo`3oool0
A@3oool2000008l0oooo00180?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0SP3o
ool004T0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`2>0?ooo`00BP3oool00`00
0000oooo0?ooo`120?ooo`030000003oool0oooo08h0oooo001;0?ooo`030000003oool0oooo0440
oooo00<000000?ooo`3oool0SP3oool004`0oooo00<000000?ooo`3oool0@03oool2000008l0oooo
001=0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0SP3oool004h0oooo0P00000o
0?ooo`030000003oool0oooo08h0oooo001@0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool0SP3oool00540oooo00<000000?ooo`3oool0>`3oool2000008l0oooo001B0?ooo`800000
>`3oool00`000000oooo0?ooo`2>0?ooo`00E03oool00`000000oooo0?ooo`0h0?ooo`030000003o
ool0oooo08h0oooo001E0?ooo`030000003oool0oooo01P0oooo0P0000040?ooo`030000003oool0
oooo00<0oooo0P0000030?ooo`@000000P3oool3000000D0oooo00<000000?ooo`3oool0SP3oool0
05H0oooo0P00000G0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0SP3o
ool005P0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000000<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`<00000
SP3oool005T0oooo0P00000>0?ooo`@000000P3oool010000000oooo0?ooo`0000080?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool0oooo0`0000050?ooo`030000003oool0oooo
08h0oooo001K0?ooo`030000003oool0oooo0140oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000050?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0SP3o
ool005`0oooo0P00000A0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0SP3oool005h0oooo0P00000@0?ooo`8000002P3oool2000000@0oooo0P0000030?ooo`@00000
103oool2000008l0oooo001P0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0SP3o
ool00640oooo0P00000/0?ooo`030000003oool0oooo08h0oooo001S0?ooo`800000:P3oool00`00
0000oooo0?ooo`2>0?ooo`00I@3oool2000002P0oooo00<000000?ooo`3oool0SP3oool006L0oooo
0P00000V0?ooo`800000S`3oool006T0oooo0P00000T0?ooo`030000003oool0oooo08h0oooo001[
0?ooo`8000008P3oool00`000000oooo0?ooo`2>0?ooo`002`3oool4000003X0oooo0P0000040?oo
o`030000003oool0oooo0080oooo0`00000F0?ooo`<000007`3oool00`000000oooo0?ooo`0i0?oo
o`800000103oool00`000000oooo0?ooo`020?ooo`<00000>@3oool4000000X0oooo000=0?ooo`03
0000003oool0oooo03P0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0F0?oo
o`<00000703oool2000003T0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0h
0?ooo`030000003oool0oooo00T0oooo000=0?ooo`030000003oool0oooo03P0oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`0I0?ooo`8000006P3oool00`000000oooo0?ooo`0h
0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?oo
o`090?ooo`001@3oool4000000@0oooo00<000000?ooo`3oool0<P3oool400000080oooo00@00000
0?ooo`3oool00000203oool3000001h0oooo0`00000G0?ooo`030000003oool0oooo03P0oooo00@0
00000?ooo`3oool00000203oool3000003/0oooo00<000000?ooo`3oool02@3oool000d0oooo00<0
00000?ooo`3oool0>03oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo0240oooo
0`00000D0?ooo`030000003oool0oooo03P0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`0k0?ooo`030000003oool0oooo00T0oooo000;0?ooo`<00000>P3oool010000000oooo
0?ooo`0000080?ooo`030000003oool0oooo02@0oooo0`00000A0?ooo`030000003oool0oooo03P0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0i0?ooo`<000002`3oool000d0
oooo00<000000?ooo`3oool0>@3oool2000000T0oooo1000000V0?ooo`8000003`3oool2000003X0
oooo0P0000090?ooo`@00000>P3oool00`000000oooo0?ooo`090?ooo`00P03oool3000000`0oooo
00<000000?ooo`3oool0SP3oool008<0oooo100000080?ooo`030000003oool0oooo08h0oooo0027
0?ooo`H000000P3oool00`000000oooo0?ooo`2>0?ooo`000`3ooooo000001X00000103oool000X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool02P3oool00`000000oooo00000003000000L0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool02@3oool000X0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0o0?ooo`030000
003oool0oooo00<0oooo1000000i0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0
2@3oool008l0oooo00<000000?ooo`3oool01`3oool700000800oooo002?0?ooo`030000003oool0
oooo00h0oooo2000001h0?ooo`00S`3oool2000001L0oooo1P00001b0?ooo`00S`3oool00`000000
oooo0?ooo`0L0?ooo`P00000JP3oool008l0oooo00<000000?ooo`3oool0903oool;000005l0oooo
002?0?ooo`030000003oool0oooo02l0oooo2`00001D0?ooo`00S`3oool2000003/0oooo30000018
0?ooo`00S`3oool00`000000oooo0?ooo`160?ooo`/00000?@3oool008l0oooo00<000000?ooo`3o
ool0D@3oool;00000380oooo002?0?ooo`030000003oool0oooo05`0oooo2000000Z0?ooo`00S`3o
ool00`000000oooo0?ooo`1T0?ooo`H00000903oool008l0oooo0P00001[0?ooo`D000007`3oool0
08l0oooo00<000000?ooo`3oool0K`3oool6000001T0oooo002?0?ooo`030000003oool0oooo07D0
oooo1@00000D0?ooo`00S`3oool00`000000oooo0?ooo`1j0?ooo`<000004@3oool008l0oooo0P00
001n0?ooo`@000003@3oool008l0oooo00<000000?ooo`3oool0P@3oool2000000/0oooo002?0?oo
o`030000003oool0oooo08h0oooo001`0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`80
00000`3oool400000080oooo0`0000050?ooo`030000003oool0oooo08h0oooo001_0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0SP3oool006l0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000030?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool00P3oool3000008h0oooo001_0?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool010000000oooo0?ooo`3oool3000009H0oooo001_0?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool010000000oooo0?ooo`00002H0?oo
o`00K`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`2F0?ooo`00L03oool2000000X0oooo0P000004
0?ooo`8000000`3oool4000009D0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.07895, -0.165079, \
0.00751884, 0.00110231}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, RePlot1delta]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.499998 0.476188 0.373784 2.23652 [
[.02381 .36128 -6 -9 ]
[.02381 .36128 6 0 ]
[.2619 .36128 -12 -9 ]
[.2619 .36128 12 0 ]
[.73809 .36128 -9 -9 ]
[.73809 .36128 9 0 ]
[.97619 .36128 -3 -9 ]
[.97619 .36128 3 0 ]
[.4875 .03831 -30 -4.5 ]
[.4875 .03831 0 4.5 ]
[.4875 .15013 -24 -4.5 ]
[.4875 .15013 0 4.5 ]
[.4875 .26196 -30 -4.5 ]
[.4875 .26196 0 4.5 ]
[.4875 .48561 -24 -4.5 ]
[.4875 .48561 0 4.5 ]
[.4875 .59744 -18 -4.5 ]
[.4875 .59744 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .37378 m
.02381 .38003 L
s
[(-1)] .02381 .36128 0 1 Mshowa
.2619 .37378 m
.2619 .38003 L
s
[(-0.5)] .2619 .36128 0 1 Mshowa
.73809 .37378 m
.73809 .38003 L
s
[(0.5)] .73809 .36128 0 1 Mshowa
.97619 .37378 m
.97619 .38003 L
s
[(1)] .97619 .36128 0 1 Mshowa
.125 Mabswid
.07143 .37378 m
.07143 .37753 L
s
.11905 .37378 m
.11905 .37753 L
s
.16667 .37378 m
.16667 .37753 L
s
.21428 .37378 m
.21428 .37753 L
s
.30952 .37378 m
.30952 .37753 L
s
.35714 .37378 m
.35714 .37753 L
s
.40476 .37378 m
.40476 .37753 L
s
.45238 .37378 m
.45238 .37753 L
s
.54762 .37378 m
.54762 .37753 L
s
.59524 .37378 m
.59524 .37753 L
s
.64285 .37378 m
.64285 .37753 L
s
.69047 .37378 m
.69047 .37753 L
s
.78571 .37378 m
.78571 .37753 L
s
.83333 .37378 m
.83333 .37753 L
s
.88095 .37378 m
.88095 .37753 L
s
.92857 .37378 m
.92857 .37753 L
s
.25 Mabswid
0 .37378 m
1 .37378 L
s
.5 .03831 m
.50625 .03831 L
s
[(-0.15)] .4875 .03831 1 0 Mshowa
.5 .15013 m
.50625 .15013 L
s
[(-0.1)] .4875 .15013 1 0 Mshowa
.5 .26196 m
.50625 .26196 L
s
[(-0.05)] .4875 .26196 1 0 Mshowa
.5 .48561 m
.50625 .48561 L
s
[(0.05)] .4875 .48561 1 0 Mshowa
.5 .59744 m
.50625 .59744 L
s
[(0.1)] .4875 .59744 1 0 Mshowa
.125 Mabswid
.5 .06067 m
.50375 .06067 L
s
.5 .08304 m
.50375 .08304 L
s
.5 .1054 m
.50375 .1054 L
s
.5 .12777 m
.50375 .12777 L
s
.5 .1725 m
.50375 .1725 L
s
.5 .19486 m
.50375 .19486 L
s
.5 .21723 m
.50375 .21723 L
s
.5 .23959 m
.50375 .23959 L
s
.5 .28432 m
.50375 .28432 L
s
.5 .30669 m
.50375 .30669 L
s
.5 .32905 m
.50375 .32905 L
s
.5 .35142 m
.50375 .35142 L
s
.5 .39615 m
.50375 .39615 L
s
.5 .41851 m
.50375 .41851 L
s
.5 .44088 m
.50375 .44088 L
s
.5 .46324 m
.50375 .46324 L
s
.5 .50797 m
.50375 .50797 L
s
.5 .53034 m
.50375 .53034 L
s
.5 .55271 m
.50375 .55271 L
s
.5 .57507 m
.50375 .57507 L
s
.5 .01594 m
.50375 .01594 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .08786 L
.10458 .15267 L
.14415 .20168 L
.18221 .23986 L
.22272 .27258 L
.26171 .29777 L
.30315 .3191 L
.34308 .33542 L
.3815 .34803 L
.42236 .3588 L
.46171 .36716 L
.49955 .37371 L
.53984 .37943 L
.57861 .38395 L
.61983 .3879 L
.65954 .39106 L
.69773 .39364 L
.73838 .39607 L
.77751 .3983 L
.81909 .40083 L
.85916 .40378 L
.89771 .4075 L
.93871 .41298 L
.97619 .42001 L
s
0 0 1 r
.06655 .61803 m
.07143 .2174 L
.11905 .18721 L
.16667 .22626 L
.21429 .26584 L
.26191 .29757 L
.30953 .32186 L
.35715 .34033 L
.40476 .35445 L
.45238 .36533 L
.5 .37378 L
.54762 .38041 L
.59524 .38563 L
.64286 .38978 L
.69048 .39309 L
.7381 .39573 L
.78572 .39785 L
.83333 .39955 L
.88095 .40091 L
.92857 .402 L
.97619 .40286 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`001`3oool00`000000oooo0?oo
o`250?ooo`800000S`3oool000P0oooo00<000000?ooo`3oool0Q03oool00`000000oooo0?ooo`2>
0?ooo`00203oool00`000000oooo0?ooo`240?ooo`030000003oool0oooo08h0oooo00090?ooo`03
0000003oool0oooo08<0oooo00<000000?ooo`3oool0SP3oool000T0oooo00<000000?ooo`3oool0
P`3oool00`000000oooo0?ooo`2>0?ooo`002P3oool00`000000oooo0?ooo`1X0?ooo`800000103o
ool00`000000oooo0?ooo`020?ooo`@000000P3oool3000000H0oooo00<000000?ooo`3oool0SP3o
ool000X0oooo00<000000?ooo`3oool0I`3oool010000000oooo0?ooo`00000:0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`002`3oool0
0`000000oooo0?ooo`1V0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`030?ooo`<00000SP3oool000/0oooo00<000000?ooo`3oool0H03oool4
00000080oooo00@000000?ooo`3oool000002P3oool010000000oooo0?ooo`3oool3000000H0oooo
00<000000?ooo`3oool0SP3oool000`0oooo00<000000?ooo`3oool0I@3oool010000000oooo0?oo
o`00000:0?ooo`050000003oool0oooo0?ooo`000000203oool00`000000oooo0?ooo`2>0?ooo`00
303oool00`000000oooo0?ooo`1U0?ooo`040000003oool0oooo000000P0oooo0`0000030?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool000d0oooo00<000000?ooo`3oool0
I@3oool2000000/0oooo00@000000?ooo`3oool0oooo100000050?ooo`030000003oool0oooo08h0
oooo000=0?ooo`030000003oool0oooo07l0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0
00000?ooo`3oool0OP3oool2000008l0oooo000>0?ooo`030000003oool0oooo07h0oooo00<00000
0?ooo`3oool0SP3oool000l0oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`2>0?oo
o`003`3oool00`000000oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000
003oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<000000?ooo`3oool0O03o
ool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000000oooo0?ooo`1k0?ooo`800000S`3oool0
0140oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004P3oool00`000000
oooo0?ooo`1j0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000003oool0oooo07T0oooo
00<000000?ooo`3oool0SP3oool001<0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?oo
o`2>0?ooo`00503oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo08h0oooo000E0?oo
o`030000003oool0oooo07L0oooo00<000000?ooo`3oool0SP3oool001D0oooo00<000000?ooo`3o
ool0M`3oool2000008l0oooo000F0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool0
SP3oool001L0oooo00<000000?ooo`3oool0M@3oool00`000000oooo0?ooo`2>0?ooo`005`3oool0
0`000000oooo0?ooo`1e0?ooo`030000003oool0oooo08h0oooo000H0?ooo`030000003oool0oooo
07@0oooo00<000000?ooo`3oool0SP3oool001T0oooo00<000000?ooo`3oool0L`3oool00`000000
oooo0?ooo`2>0?ooo`006@3oool00`000000oooo0?ooo`1c0?ooo`800000S`3oool001X0oooo00<0
00000?ooo`3oool0LP3oool00`000000oooo0?ooo`2>0?ooo`006`3oool00`000000oooo0?ooo`1a
0?ooo`030000003oool0oooo08h0oooo000K0?ooo`030000003oool0oooo0740oooo00<000000?oo
o`3oool0SP3oool001`0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`2>0?ooo`00
7@3oool00`000000oooo0?ooo`1K0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000
1@3oool00`000000oooo0?ooo`2>0?ooo`007@3oool00`000000oooo0?ooo`1J0?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`007P3o
ool00`000000oooo0?ooo`1I0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
103oool3000008h0oooo000O0?ooo`030000003oool0oooo0580oooo100000020?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`00803o
ool00`000000oooo0?ooo`1G0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`2>0?ooo`00803oool00`000000oooo0?ooo`1G0?ooo`040000003o
ool0oooo000000P0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000Q0?ooo`030000003o
ool0oooo05L0oooo0P00000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3o
ool00280oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?ooo`2>0?ooo`008`3oool00`00
0000oooo0?ooo`1Y0?ooo`800000S`3oool002<0oooo00<000000?ooo`3oool0J@3oool00`000000
oooo0?ooo`2>0?ooo`00903oool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo08h0oooo
000U0?ooo`030000003oool0oooo06L0oooo00<000000?ooo`3oool0SP3oool00280oooo00D0003o
0?ooo`3oool0oooo0000001X0?ooo`030000003oool0oooo08h0oooo000P0?ooo`80003o00<0oooo
0000o`3oool00P3oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo08h0oooo000O0?oo
o`030000o`3oool0oooo0080oooo0P000?l00`3oool000000?ooo`1V0?ooo`800000S`3oool001d0
oooo0P000?l70?ooo`030000o`3oool0000006H0oooo00<000000?ooo`3oool0SP3oool001/0oooo
0P000?l:0?ooo`030000o`3oool0000006D0oooo00<000000?ooo`3oool0SP3oool001X0oooo00<0
003o0?ooo`3oool02`3oool00`000?l0oooo0000001T0?ooo`030000003oool0oooo08h0oooo000H
0?ooo`80003o3`3oool20000o`030000003oool0oooo0640oooo00<000000?ooo`3oool0SP3oool0
01L0oooo00<0003o0?ooo`3oool04@3oool00`000?l000000?ooo`1Q0?ooo`030000003oool0oooo
08h0oooo000E0?ooo`80003o5@3oool00`000?l000000?ooo`1P0?ooo`030000003oool0oooo08h0
oooo000D0?ooo`030000o`3oool0oooo01H0oooo0P000?mP0?ooo`800000S`3oool001@0oooo00<0
003o0?ooo`3oool0603oool00`000?l0oooo0?ooo`1M0?ooo`030000003oool0oooo08h0oooo000D
0?ooo`030000o`3oool0oooo01T0oooo00<0003o0?ooo`3oool0G03oool00`000000oooo0?ooo`2>
0?ooo`00503oool00`000?l0oooo0?ooo`0J0?ooo`030000o`3oool0oooo05/0oooo00<000000?oo
o`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool06`3oool00`000?l0oooo0?ooo`1J0?ooo`03
0000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo01`0oooo00<0003o0?ooo`3oool0
F@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0M0?ooo`030000o`3o
ool0oooo05P0oooo0P00002?0?ooo`00503oool00`000?l0oooo0?ooo`0N0?ooo`030000o`3oool0
oooo05L0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool07`3oool20000
oeL0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0803oool00`000000
003o0?ooo`1E0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo0280oooo
00<0003o0?ooo`3oool0D`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?oo
o`0S0?ooo`030000o`3oool0oooo03P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P00
00030?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0T
0?ooo`030000o`3oool0oooo03H0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0
003o0?ooo`3oool09@3oool00`000?l0oooo0?ooo`0e0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`030?ooo`<00000SP3oool001@0oooo
00<0003o0?ooo`3oool09P3oool00`000?l000000?ooo`0^0?ooo`@000000P3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`0000060?ooo`030000003oool0oooo
08h0oooo000D0?ooo`030000o`3oool0oooo02L0oooo0P000?ld0?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo08h0oooo000D0?ooo`030000o`3oool0oooo02T0oooo00<0003o0?ooo`3oool0<@3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`0Z0?ooo`80003o<P3oool2
000000X0oooo0P0000030?ooo`@000001@3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`00
0?l0oooo0?ooo`0/0?ooo`030000o`3oool0oooo04T0oooo00<000000?ooo`3oool0SP3oool001@0
oooo00<0003o0?ooo`3oool0;@3oool20000odT0oooo0P00002?0?ooo`00503oool00`000?l0oooo
0?ooo`0_0?ooo`80003oA`3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?oo
o`0a0?ooo`030000o`3oool0oooo04@0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o
0?ooo`3oool0<P3oool20000od@0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?oo
o`3oool0=03oool00`000?l0oooo0?ooo`110?ooo`030000003oool0oooo08h0oooo000D0?ooo`03
0000o`3oool0oooo03D0oooo0P000?m10?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000
o`3oool0oooo03L0oooo0P000?lo0?ooo`800000S`3oool001@0oooo00<0003o0?ooo`3oool0>@3o
ool20000ocd0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0>`3oool2
0000oc/0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0?@3oool20000
ocT0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0?`3oool20000o`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o0?ooo`3oool0
@@3oool20000o`030000003oool0oooo0380oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0
003o0?ooo`3oool0@`3oool30000oc80oooo00<000000?ooo`3oool0SP3oool001@0oooo00<0003o
0?ooo`3oool0AP3oool20000oc00oooo0P00002?0?ooo`00503oool00`000?l0oooo0?ooo`180?oo
o`<0003o;@3oool00`000000oooo0?ooo`2>0?ooo`00203oool4000000P0oooo00<0003o0?ooo`3o
ool0<03oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000`0oooo00@000000000o`00
0?l0003o:P3oool00`000000oooo0?ooo`0j0?ooo`800000103oool00`000000oooo0?ooo`020?oo
o`<00000>`3oool4000000L0oooo000:0?ooo`030000003oool0oooo00L0oooo00<0003o0?ooo`3o
ool0;`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo00`0oooo00@000000000
o`000?l0003o9`3oool00`000000oooo0?ooo`0i0?ooo`040000003oool0oooo000000/0oooo00<0
00000?ooo`3oool0>P3oool00`000000oooo0?ooo`060?ooo`002P3oool00`000000oooo0?ooo`07
0?ooo`030000o`3oool0oooo02l0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?oo
o`0@0?ooo`@0003o8`3oool00`000000oooo0?ooo`0i0?ooo`040000003oool0oooo000000/0oooo
00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`060?ooo`000P3oool4000000@0oooo00<0
00000?ooo`3oool01`3oool00`000?l0oooo0?ooo`0Y0?ooo`@000000P3oool010000000oooo0?oo
o`0000080?ooo`<000005`3oool30000ob00oooo00<000000?ooo`3oool0>@3oool010000000oooo
0?ooo`0000080?ooo`<00000?@3oool00`000000oooo0?ooo`060?ooo`002P3oool00`000000oooo
0?ooo`070?ooo`030000o`3oool0oooo02l0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`0J0?ooo`@0003o703oool2000003X0oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`0m0?ooo`030000003oool0oooo00H0oooo00080?ooo`<000002@3oool00`000?l0
oooo0?ooo`0_0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool07P3oool40000
oaP0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0
oooo03/0oooo0`0000080?ooo`002P3oool00`000000oooo0?ooo`070?ooo`030000o`3oool0oooo
0300oooo0P0000090?ooo`@000008@3oool40000oa@0oooo00<000000?ooo`3oool0>P3oool20000
00T0oooo1000000l0?ooo`030000003oool0oooo00H0oooo000D0?ooo`030000o`3oool0oooo06@0
oooo10000?l@0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo06P0oooo
1@000?l;0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo06`0oooo00<0
00000000o`000?l00P000?l70?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0
oooo0700oooo00<000000000o`000?l00`000?l20?ooo`030000003oool0oooo08h0oooo000D0000
00030000o`000000000007H000002@000?n900000040oooo00070?ooo`030000003oool0oooo00X0
oooo00<0003o0?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool00`3oool00`000000003o0000o`090000o`T0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00H0oooo000D0?ooo`030000
o`3oool0oooo07P0oooo00<000000?ooo`3oool03P3oool00`000000003o0000o`040000o`800000
M`3oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`0E0?ooo`X0003oK`3o
ool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`0M0?ooo`8000003P000?mQ
0?ooo`00503oool00`000?l0oooo0?ooo`1h0?ooo`030000003oool0oooo02D0oooo2000000>0000
o`030000003oool0oooo0500oooo000D0?ooo`030000o`3oool0oooo07P0oooo0P00000l0?oooa/0
003o>03oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`170?ooo`l00000
6`000?lM0?ooo`00503oool00`000?l0oooo0?ooo`1h0?ooo`030000003oool0oooo05h0oooo3000
00070?oooaD0003o203oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`1Z
0?ooo`P00000703oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`1b0?oo
o`H000005P3oool001@0oooo00<0003o0?ooo`3oool0N03oool00`000000oooo0?ooo`1h0?ooo`H0
0000403oool001@0oooo00<0003o0?ooo`3oool0N03oool2000007l0oooo1@00000;0?ooo`004`3o
ool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08<0oooo0`0000080?ooo`004`3oool0
0`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo
07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000
oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo
000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o
0?ooo`3oool0N@3oool2000008l0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?oo
o`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`00
4`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3o
ool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool0
0`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000S`3oool001<0
oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo
0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<0
00000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>
0?ooo`004`3oool00`000?l0oooo0?ooo`1O0?ooo`800000103oool00`000000oooo0?ooo`030?oo
o`8000000`3oool3000000H0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3o
ool0GP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1N0?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?oo
o`030?ooo`<00000SP3oool001<0oooo00<0003o0?ooo`3oool0GP3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo0`0000060?ooo`030000003oool0oooo08h0oooo
000C0?ooo`030000o`3oool0oooo05h0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool001<0oooo
00<0003o0?ooo`3oool0GP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0
oooo0?ooo`1O0?ooo`8000002P3oool2000000<0oooo100000050?ooo`030000003oool0oooo08h0
oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0
003o0?ooo`3oool0N@3oool2000008l0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<00000
0?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?oo
o`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000
o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3o
ool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000S`3oool0
01<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0
oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo
00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?oo
o`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?oo
o`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3o
ool0N@3oool2000008l0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0
SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool0
0`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo
07T0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000
oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000S`3oool001<0oooo00<0
003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1i
0?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?oo
o`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`2>0?ooo`00
4`3oool00`000?l0oooo0?ooo`1U0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000
1@3oool00`000000oooo0?ooo`2>0?ooo`004`3oool00`000?l0oooo0?ooo`1T0?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`004`3o
ool00`000?l0oooo0?ooo`1T0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
103oool3000008h0oooo000C0?ooo`030000o`3oool0oooo06@0oooo00@000000?ooo`3oool00000
2P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000C0?ooo`030000o`3o
ool0oooo06@0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo08h0oooo000C0?ooo`030000o`3oool0oooo06@0oooo00@000000?ooo`3oool00000
203oool3000000H0oooo00<000000?ooo`3oool0SP3oool001<0oooo00<0003o0?ooo`3oool0I@3o
ool2000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`004`3oool0
0`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05261, -0.167129, \
0.00733526, 0.00156178}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[delta[1, 1, 9/10, 1/2, a]], {a, \(-1\), 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 0.452946 0.849537 [
[.02381 .44045 -6 -9 ]
[.02381 .44045 6 0 ]
[.2619 .44045 -12 -9 ]
[.2619 .44045 12 0 ]
[.7381 .44045 -9 -9 ]
[.7381 .44045 9 0 ]
[.97619 .44045 -3 -9 ]
[.97619 .44045 3 0 ]
[.4875 .02818 -24 -4.5 ]
[.4875 .02818 0 4.5 ]
[.4875 .11313 -24 -4.5 ]
[.4875 .11313 0 4.5 ]
[.4875 .19808 -24 -4.5 ]
[.4875 .19808 0 4.5 ]
[.4875 .28304 -24 -4.5 ]
[.4875 .28304 0 4.5 ]
[.4875 .36799 -24 -4.5 ]
[.4875 .36799 0 4.5 ]
[.4875 .5379 -18 -4.5 ]
[.4875 .5379 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .45295 m
.02381 .4592 L
s
[(-1)] .02381 .44045 0 1 Mshowa
.2619 .45295 m
.2619 .4592 L
s
[(-0.5)] .2619 .44045 0 1 Mshowa
.7381 .45295 m
.7381 .4592 L
s
[(0.5)] .7381 .44045 0 1 Mshowa
.97619 .45295 m
.97619 .4592 L
s
[(1)] .97619 .44045 0 1 Mshowa
.125 Mabswid
.07143 .45295 m
.07143 .4567 L
s
.11905 .45295 m
.11905 .4567 L
s
.16667 .45295 m
.16667 .4567 L
s
.21429 .45295 m
.21429 .4567 L
s
.30952 .45295 m
.30952 .4567 L
s
.35714 .45295 m
.35714 .4567 L
s
.40476 .45295 m
.40476 .4567 L
s
.45238 .45295 m
.45238 .4567 L
s
.54762 .45295 m
.54762 .4567 L
s
.59524 .45295 m
.59524 .4567 L
s
.64286 .45295 m
.64286 .4567 L
s
.69048 .45295 m
.69048 .4567 L
s
.78571 .45295 m
.78571 .4567 L
s
.83333 .45295 m
.83333 .4567 L
s
.88095 .45295 m
.88095 .4567 L
s
.92857 .45295 m
.92857 .4567 L
s
.25 Mabswid
0 .45295 m
1 .45295 L
s
.5 .02818 m
.50625 .02818 L
s
[(-0.5)] .4875 .02818 1 0 Mshowa
.5 .11313 m
.50625 .11313 L
s
[(-0.4)] .4875 .11313 1 0 Mshowa
.5 .19808 m
.50625 .19808 L
s
[(-0.3)] .4875 .19808 1 0 Mshowa
.5 .28304 m
.50625 .28304 L
s
[(-0.2)] .4875 .28304 1 0 Mshowa
.5 .36799 m
.50625 .36799 L
s
[(-0.1)] .4875 .36799 1 0 Mshowa
.5 .5379 m
.50625 .5379 L
s
[(0.1)] .4875 .5379 1 0 Mshowa
.125 Mabswid
.5 .04517 m
.50375 .04517 L
s
.5 .06216 m
.50375 .06216 L
s
.5 .07915 m
.50375 .07915 L
s
.5 .09614 m
.50375 .09614 L
s
.5 .13012 m
.50375 .13012 L
s
.5 .14711 m
.50375 .14711 L
s
.5 .1641 m
.50375 .1641 L
s
.5 .18109 m
.50375 .18109 L
s
.5 .21508 m
.50375 .21508 L
s
.5 .23207 m
.50375 .23207 L
s
.5 .24906 m
.50375 .24906 L
s
.5 .26605 m
.50375 .26605 L
s
.5 .30003 m
.50375 .30003 L
s
.5 .31702 m
.50375 .31702 L
s
.5 .33401 m
.50375 .33401 L
s
.5 .351 m
.50375 .351 L
s
.5 .38498 m
.50375 .38498 L
s
.5 .40197 m
.50375 .40197 L
s
.5 .41896 m
.50375 .41896 L
s
.5 .43596 m
.50375 .43596 L
s
.5 .46994 m
.50375 .46994 L
s
.5 .48693 m
.50375 .48693 L
s
.5 .50392 m
.50375 .50392 L
s
.5 .52091 m
.50375 .52091 L
s
.5 .01119 m
.50375 .01119 L
s
.5 .55489 m
.50375 .55489 L
s
.5 .57188 m
.50375 .57188 L
s
.5 .58887 m
.50375 .58887 L
s
.5 .60586 m
.50375 .60586 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .06591 L
.10458 .11861 L
.14415 .16498 L
.18221 .20668 L
.22272 .24795 L
.26171 .28467 L
.30316 .3206 L
.34309 .35232 L
.3815 .38032 L
.42237 .40758 L
.46172 .43156 L
.49955 .45271 L
.53984 .47336 L
.57861 .4916 L
.61984 .50941 L
.65954 .52516 L
.69774 .53912 L
.73838 .55275 L
.77751 .56469 L
.81909 .57605 L
.85916 .5856 L
.89771 .59329 L
.91726 .59656 L
.93871 .59959 L
.9581 .60179 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00S`3oool2000008l0oooo00070?ooo`030000003oool0oooo08D0oooo00<0
00000?ooo`3oool0SP3oool000P0oooo00<000000?ooo`3oool0Q03oool00`000000oooo0?ooo`2>
0?ooo`002@3oool00`000000oooo0?ooo`1_0?ooo`800000103oool00`000000oooo0?ooo`020?oo
o`<000001P3oool00`000000oooo0?ooo`2>0?ooo`002@3oool00`000000oooo0?ooo`1^0?ooo`04
0000003oool0oooo000000/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?oo
o`002P3oool00`000000oooo0?ooo`1]0?ooo`040000003oool0oooo000000/0oooo00<000000?oo
o`3oool00`3oool3000008h0oooo000;0?ooo`030000003oool0oooo06H0oooo100000020?ooo`04
0000003oool0oooo000000P0oooo0`0000060?ooo`030000003oool0oooo08h0oooo000<0?ooo`03
0000003oool0oooo06/0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo08h0oooo000<0?ooo`030000003oool0oooo06/0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo000=0?ooo`03
0000003oool0oooo06/0oooo0P0000090?ooo`@000001@3oool2000008l0oooo000>0?ooo`030000
003oool0oooo07h0oooo00<000000?ooo`3oool0SP3oool000l0oooo00<000000?ooo`3oool0O@3o
ool00`000000oooo0?ooo`2>0?ooo`00403oool00`000000oooo0?ooo`1l0?ooo`030000003oool0
oooo08h0oooo000@0?ooo`030000003oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00140
oooo00<000000?ooo`3oool0N`3oool2000008l0oooo000B0?ooo`030000003oool0oooo07X0oooo
00<000000?ooo`3oool0SP3oool001<0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?oo
o`2>0?ooo`004`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo08h0oooo000D0?oo
o`030000003oool0oooo07P0oooo00<000000?ooo`3oool0SP3oool001D0oooo00<000000?ooo`3o
ool0M`3oool2000008l0oooo000F0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool0
SP3oool001H0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`2>0?ooo`005`3oool0
0`000000oooo0?ooo`1e0?ooo`030000003oool0oooo08h0oooo000H0?ooo`030000003oool0oooo
07@0oooo00<000000?ooo`3oool0SP3oool001T0oooo00<000000?ooo`3oool0L`3oool2000008l0
oooo000I0?ooo`030000003oool0oooo07<0oooo00<000000?ooo`3oool0SP3oool001X0oooo00<0
00000?ooo`3oool0LP3oool00`000000oooo0?ooo`2>0?ooo`006`3oool00`000000oooo0?ooo`1M
0?ooo`800000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0SP3oool001`0oooo00<000000?ooo`3oool0F`3oool010000000oooo0?ooo`00000:
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool001`0oooo00<000000?oo
o`3oool0F`3oool010000000oooo0?ooo`0000070?ooo`D000001@3oool3000008h0oooo000M0?oo
o`030000003oool0oooo05@0oooo100000020?ooo`040000003oool0oooo000000L0oooo00@00000
0?ooo`3oool000001P3oool00`000000oooo0?ooo`2>0?ooo`007P3oool00`000000oooo0?ooo`1I
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`0000001P3oool00`000000oooo0?oo
o`2>0?ooo`007`3oool00`000000oooo0?ooo`1H0?ooo`040000003oool0oooo000000T0oooo0P00
00060?ooo`030000003oool0oooo08h0oooo000P0?ooo`030000003oool0oooo05P0oooo0P00000;
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0SP3oool00240oooo00<000000?oo
o`3oool0J`3oool2000008l0oooo000Q0?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3o
ool0SP3oool00280oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?ooo`2>0?ooo`008`3o
ool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo08h0oooo000T0?ooo`030000003oool0
oooo06P0oooo00<000000?ooo`3oool0SP3oool002D0oooo00<000000?ooo`3oool0I`3oool20000
08l0oooo000V0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0SP3oool002H0oooo
00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`2>0?ooo`009`3oool00`000000oooo0?oo
o`1U0?ooo`030000003oool0oooo08h0oooo000X0?ooo`030000003oool0oooo06@0oooo00<00000
0?ooo`3oool0SP3oool002T0oooo00<000000?ooo`3oool0H`3oool2000008l0oooo000Z0?ooo`03
0000003oool0oooo0680oooo00<000000?ooo`3oool0SP3oool002/0oooo00<000000?ooo`3oool0
H@3oool00`000000oooo0?ooo`2>0?ooo`00;03oool00`000000oooo0?ooo`1P0?ooo`030000003o
ool0oooo08h0oooo000]0?ooo`030000003oool0oooo05l0oooo0P00002?0?ooo`00;P3oool00`00
0000oooo0?ooo`1N0?ooo`030000003oool0oooo08h0oooo000^0?ooo`030000003oool0oooo05h0
oooo00<000000?ooo`3oool0SP3oool002l0oooo00<000000?ooo`3oool0B@3oool2000000@0oooo
00<000000?ooo`3oool00`3oool2000000H0oooo00<000000?ooo`3oool0SP3oool00300oooo00<0
00000?ooo`3oool0A`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool0SP3oool00340oooo00<000000?ooo`3oool0AP3oool010000000oooo
0?ooo`00000;0?ooo`030000003oool0oooo00<0oooo0`00002>0?ooo`00<P3oool00`000000oooo
0?ooo`0o0?ooo`@000000P3oool010000000oooo0?ooo`0000090?ooo`8000001P3oool00`000000
oooo0?ooo`2>0?ooo`00<`3oool00`000000oooo0?ooo`140?ooo`040000003oool0oooo000000/0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00=03oool00`000000oooo
0?ooo`130?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`00
0000oooo0?ooo`2>0?ooo`00=@3oool00`000000oooo0?ooo`130?ooo`8000002P3oool2000000H0
oooo00<000000?ooo`3oool0SP3oool003H0oooo00<000000?ooo`3oool0EP3oool2000008l0oooo
000g0?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool0SP3oool003P0oooo00<00000
0?ooo`3oool0E03oool00`000000oooo0?ooo`2>0?ooo`00>@3oool00`000000oooo0?ooo`1C0?oo
o`030000003oool0oooo08h0oooo000j0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3o
ool0SP3oool003/0oooo00<000000?ooo`3oool0D@3oool2000008l0oooo000l0?ooo`030000003o
ool0oooo0500oooo00<000000?ooo`3oool0SP3oool003d0oooo00<000000?ooo`3oool0C`3oool0
0`000000oooo0?ooo`2>0?ooo`00?P3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo
08h0oooo000o0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0SP3oool00400oooo
00<000000?ooo`3oool0C03oool2000008l0oooo00110?ooo`030000003oool0oooo04/0oooo00<0
00000?ooo`3oool0SP3oool00480oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`2>
0?ooo`00@`3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo08h0oooo00140?ooo`03
0000003oool0oooo04P0oooo00<000000?ooo`3oool0SP3oool004D0oooo0P0000180?ooo`800000
S`3oool004L0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`2>0?ooo`00B03oool0
0`000000oooo0?ooo`140?ooo`030000003oool0oooo08h0oooo00190?ooo`030000003oool0oooo
02l0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000050?ooo`030000003oool0oooo
08h0oooo001:0?ooo`030000003oool0oooo02d0oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo001;0?ooo`030000003oool0oooo02`0
oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`050?ooo`<00000SP3oool004`0
oooo00<000000?ooo`3oool09@3oool400000080oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo001=0?ooo`030000003oool0oooo02X0
oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
08h0oooo001>0?ooo`800000:P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo
000000D0oooo00<000000?ooo`3oool0SP3oool00500oooo00<000000?ooo`3oool0:03oool20000
00X0oooo0P0000060?ooo`800000S`3oool00540oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`2>0?ooo`00DP3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo08h0oooo
001C0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0SP3oool005@0oooo0P00000i
0?ooo`030000003oool0oooo08h0oooo001F0?ooo`030000003oool0oooo03H0oooo0P00002?0?oo
o`00E`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo08h0oooo001H0?ooo`030000
003oool0oooo03@0oooo00<000000?ooo`3oool0SP3oool005T0oooo0P00000d0?ooo`030000003o
ool0oooo08h0oooo001K0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0SP3oool0
05`0oooo00<000000?ooo`3oool0<03oool2000008l0oooo001M0?ooo`030000003oool0oooo02l0
oooo00<000000?ooo`3oool0SP3oool005h0oooo00<000000?ooo`3oool0;P3oool00`000000oooo
0?ooo`2>0?ooo`00G`3oool2000002h0oooo00<000000?ooo`3oool0SP3oool00640oooo00<00000
0?ooo`3oool0:`3oool00`000000oooo0?ooo`2>0?ooo`00HP3oool00`000000oooo0?ooo`0Z0?oo
o`800000S`3oool006<0oooo0P00000Z0?ooo`030000003oool0oooo08h0oooo001U0?ooo`030000
003oool0oooo02L0oooo00<000000?ooo`3oool0SP3oool006H0oooo00<000000?ooo`3oool04P3o
ool2000000@0oooo00<000000?ooo`3oool00P3oool4000000D0oooo00<000000?ooo`3oool0SP3o
ool006L0oooo0P00000A0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`2>0?ooo`00J@3oool00`000000oooo0?ooo`0>0?ooo`040000003oool0
oooo000000X0oooo00<000000?ooo`3oool0103oool3000008h0oooo001Z0?ooo`030000003oool0
oooo00L0oooo100000020?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`2>0?ooo`00J`3oool2000000d0oooo00@000000?ooo`3oool000002P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo001]0?ooo`030000003oool0
oooo00X0oooo00@000000?ooo`3oool00000203oool3000000H0oooo00<000000?ooo`3oool0SP3o
ool006h0oooo0P00000;0?ooo`8000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo08h0oooo001`0?ooo`030000003oool0oooo01`0oooo0P00002?0?ooo`00L@3oool2000001`0
oooo00<000000?ooo`3oool0SP3oool007<0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`2>0?ooo`00M03oool2000001T0oooo00<000000?ooo`3oool0SP3oool007H0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`2>0?ooo`00M`3oool2000001H0oooo0P00002?0?oo
o`00N@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo08h0oooo001j0?ooo`800000
4`3oool00`000000oooo0?ooo`2>0?ooo`00O03oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo08h0oooo00080?ooo`@00000>`3oool2000000@0oooo00<000000?ooo`3oool00P3oool3
000002P0oooo0P00000@0?ooo`030000003oool0oooo03X0oooo0P0000040?ooo`030000003oool0
oooo0080oooo0`00000k0?ooo`@000001`3oool000X0oooo00<000000?ooo`3oool0>@3oool01000
0000oooo0?ooo`00000;0?ooo`030000003oool0oooo02L0oooo0P00000>0?ooo`800000>P3oool0
10000000oooo0?ooo`00000;0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool01P3o
ool000X0oooo00<000000?ooo`3oool0>@3oool010000000oooo0?ooo`00000;0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0i0?ooo`040000003o
ool0oooo000000/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`060?ooo`000P3o
ool4000000@0oooo00<000000?ooo`3oool0<`3oool400000080oooo00@000000?ooo`3oool00000
203oool3000002d0oooo0P00000;0?ooo`030000003oool0oooo03T0oooo00@000000?ooo`3oool0
0000203oool3000003d0oooo00<000000?ooo`3oool01P3oool000X0oooo00<000000?ooo`3oool0
>@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`0i0?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`3oool0?@3oool00`000000oooo0?ooo`060?ooo`00203oool3000003/0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0?ooo`0`0?ooo`800000203oool2000003X0oooo00@000000?oo
o`3oool00000203oool00`000000oooo0?ooo`0k0?ooo`<00000203oool000X0oooo00<000000?oo
o`3oool0>P3oool2000000T0oooo1000000a0?ooo`8000001P3oool00`000000oooo0?ooo`0j0?oo
o`8000002@3oool4000003`0oooo00<000000?ooo`3oool01P3oool008T0oooo0P0000040?ooo`03
0000003oool0oooo08h0oooo002;0?ooo`8000000P3oool00`000000oooo0?ooo`2>0?ooo`00S@3o
ool300000900oooo003o0000020000000@3oool000L0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool30000
00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool01P3oool000L0oooo00<000000?ooo`3oool0@@3oool00`000000
oooo0?ooo`110?ooo`030000003oool0oooo00800000?`3oool00`000000oooo0?ooo`110?ooo`03
0000003oool0oooo00H0oooo002?0?ooo`030000003oool0oooo0080oooo0P00002:0?ooo`00S`3o
ool00`000000oooo0?ooo`040?ooo`800000R03oool008l0oooo0P0000070?ooo`800000QP3oool0
08l0oooo00<000000?ooo`3oool0203oool2000008@0oooo002?0?ooo`030000003oool0oooo00X0
oooo0P0000220?ooo`00S`3oool00`000000oooo0?ooo`0<0?ooo`800000P03oool008l0oooo00<0
00000?ooo`3oool03P3oool3000007d0oooo002?0?ooo`8000004P3oool2000007/0oooo002?0?oo
o`030000003oool0oooo01<0oooo0`00001h0?ooo`00S`3oool00`000000oooo0?ooo`0F0?ooo`80
0000MP3oool008l0oooo00<000000?ooo`3oool0603oool2000007@0oooo002?0?ooo`030000003o
ool0oooo01X0oooo0P00001b0?ooo`00S`3oool2000001d0oooo0P00001`0?ooo`00S`3oool00`00
0000oooo0?ooo`0N0?ooo`<00000K@3oool008l0oooo00<000000?ooo`3oool08@3oool2000006/0
oooo002?0?ooo`030000003oool0oooo02<0oooo0P00001Y0?ooo`00S`3oool00`000000oooo0?oo
o`0U0?ooo`<00000IP3oool008l0oooo0P00000Y0?ooo`800000I03oool008l0oooo00<000000?oo
o`3oool0:P3oool300000640oooo002?0?ooo`030000003oool0oooo02d0oooo0`00001N0?ooo`00
N`3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000D0oooo00<000000?ooo`3oool0
<03oool2000005`0oooo001j0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0b0?ooo`<00000F@3oool007X0oooo00@000000?ooo`3oool00000
2P3oool00`000000oooo0?ooo`040?ooo`<00000=@3oool3000005H0oooo001j0?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0h0?ooo`<00000
D`3oool007X0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo03/0oooo0P00001A0?ooo`00NP3oool010000000oooo0?ooo`0000080?ooo`<00000
1P3oool00`000000oooo0?ooo`0m0?ooo`<00000CP3oool007/0oooo0P00000;0?ooo`030000003o
ool0oooo00@0oooo0P0000110?ooo`<00000B`3oool008l0oooo00<000000?ooo`3oool0@`3oool4
000004L0oooo002?0?ooo`030000003oool0oooo04L0oooo100000130?ooo`00S`3oool00`000000
oooo0?ooo`1;0?ooo`<00000@03oool008l0oooo00<000000?ooo`3oool0CP3oool4000003`0oooo
002?0?ooo`800000D`3oool4000003P0oooo002?0?ooo`030000003oool0oooo05H0oooo1000000d
0?ooo`00S`3oool00`000000oooo0?ooo`1J0?ooo`@00000<03oool008l0oooo00<000000?ooo`3o
ool0GP3oool4000002`0oooo002?0?ooo`030000003oool0oooo0680oooo1@00000W0?ooo`00S`3o
ool2000006P0oooo1P00000Q0?ooo`00S`3oool00`000000oooo0?ooo`1]0?ooo`D00000703oool0
08l0oooo00<000000?ooo`3oool0LP3oool6000001H0oooo002?0?ooo`030000003oool0oooo07P0
oooo1P00000@0?ooo`00S`3oool00`000000oooo0?ooo`1n0?ooo`P00000203oool008l0oooo0P00
002?0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?oo
o`00S`3oool00`000000oooo0?ooo`2>0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.0526, -0.533172, \
0.00733521, 0.00411161}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, RePlot3delta]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.499998 0.476188 0.248496 0.453199 [
[.02381 .236 -6 -9 ]
[.02381 .236 6 0 ]
[.2619 .236 -12 -9 ]
[.2619 .236 12 0 ]
[.73809 .236 -9 -9 ]
[.73809 .236 9 0 ]
[.97619 .236 -3 -9 ]
[.97619 .236 3 0 ]
[.4875 .06722 -24 -4.5 ]
[.4875 .06722 0 4.5 ]
[.4875 .15786 -24 -4.5 ]
[.4875 .15786 0 4.5 ]
[.4875 .33914 -18 -4.5 ]
[.4875 .33914 0 4.5 ]
[.4875 .42978 -18 -4.5 ]
[.4875 .42978 0 4.5 ]
[.4875 .52042 -18 -4.5 ]
[.4875 .52042 0 4.5 ]
[.4875 .61106 -18 -4.5 ]
[.4875 .61106 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .2485 m
.02381 .25475 L
s
[(-1)] .02381 .236 0 1 Mshowa
.2619 .2485 m
.2619 .25475 L
s
[(-0.5)] .2619 .236 0 1 Mshowa
.73809 .2485 m
.73809 .25475 L
s
[(0.5)] .73809 .236 0 1 Mshowa
.97619 .2485 m
.97619 .25475 L
s
[(1)] .97619 .236 0 1 Mshowa
.125 Mabswid
.07143 .2485 m
.07143 .25225 L
s
.11905 .2485 m
.11905 .25225 L
s
.16667 .2485 m
.16667 .25225 L
s
.21428 .2485 m
.21428 .25225 L
s
.30952 .2485 m
.30952 .25225 L
s
.35714 .2485 m
.35714 .25225 L
s
.40476 .2485 m
.40476 .25225 L
s
.45238 .2485 m
.45238 .25225 L
s
.54762 .2485 m
.54762 .25225 L
s
.59524 .2485 m
.59524 .25225 L
s
.64285 .2485 m
.64285 .25225 L
s
.69047 .2485 m
.69047 .25225 L
s
.78571 .2485 m
.78571 .25225 L
s
.83333 .2485 m
.83333 .25225 L
s
.88095 .2485 m
.88095 .25225 L
s
.92857 .2485 m
.92857 .25225 L
s
.25 Mabswid
0 .2485 m
1 .2485 L
s
.5 .06722 m
.50625 .06722 L
s
[(-0.4)] .4875 .06722 1 0 Mshowa
.5 .15786 m
.50625 .15786 L
s
[(-0.2)] .4875 .15786 1 0 Mshowa
.5 .33914 m
.50625 .33914 L
s
[(0.2)] .4875 .33914 1 0 Mshowa
.5 .42978 m
.50625 .42978 L
s
[(0.4)] .4875 .42978 1 0 Mshowa
.5 .52042 m
.50625 .52042 L
s
[(0.6)] .4875 .52042 1 0 Mshowa
.5 .61106 m
.50625 .61106 L
s
[(0.8)] .4875 .61106 1 0 Mshowa
.125 Mabswid
.5 .08988 m
.50375 .08988 L
s
.5 .11254 m
.50375 .11254 L
s
.5 .1352 m
.50375 .1352 L
s
.5 .18052 m
.50375 .18052 L
s
.5 .20318 m
.50375 .20318 L
s
.5 .22584 m
.50375 .22584 L
s
.5 .27116 m
.50375 .27116 L
s
.5 .29382 m
.50375 .29382 L
s
.5 .31648 m
.50375 .31648 L
s
.5 .3618 m
.50375 .3618 L
s
.5 .38446 m
.50375 .38446 L
s
.5 .40712 m
.50375 .40712 L
s
.5 .45244 m
.50375 .45244 L
s
.5 .4751 m
.50375 .4751 L
s
.5 .49776 m
.50375 .49776 L
s
.5 .54308 m
.50375 .54308 L
s
.5 .56574 m
.50375 .56574 L
s
.5 .5884 m
.50375 .5884 L
s
.5 .04456 m
.50375 .04456 L
s
.5 .0219 m
.50375 .0219 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .04203 L
.10458 .07014 L
.14415 .09488 L
.18221 .11712 L
.22272 .13914 L
.26171 .15873 L
.30315 .1779 L
.34308 .19482 L
.3815 .20975 L
.42236 .22429 L
.46171 .23709 L
.49955 .24837 L
.53984 .25939 L
.57861 .26912 L
.61983 .27862 L
.65954 .28702 L
.69773 .29447 L
.73838 .30174 L
.77751 .30811 L
.81909 .31417 L
.85916 .31926 L
.89771 .32337 L
.91725 .32511 L
.93871 .32673 L
.95809 .3279 L
.97619 .32872 L
s
0 0 1 r
.03926 .61803 m
.07143 .11624 L
.11905 .10042 L
.16667 .11496 L
.21429 .13673 L
.26191 .15937 L
.30953 .18081 L
.35715 .20045 L
.40476 .2182 L
.45238 .23415 L
.5 .2485 L
.54762 .26141 L
.59524 .27305 L
.64286 .28358 L
.69048 .29314 L
.7381 .30184 L
.78572 .30978 L
.83333 .31705 L
.88095 .32372 L
.92857 .32986 L
.97619 .33552 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`002@3oool00`000000oooo0?oo
o`230?ooo`030000003oool0oooo08h0oooo000:0?ooo`800000P`3oool00`000000oooo0?ooo`2>
0?ooo`00303oool00`000000oooo0?ooo`200?ooo`800000S`3oool000d0oooo0P0000200?ooo`03
0000003oool0oooo08h0oooo000?0?ooo`800000OP3oool00`000000oooo0?ooo`2>0?ooo`004@3o
ool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo08h0oooo000B0?ooo`800000N`3oool0
0`000000oooo0?ooo`2>0?ooo`00503oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo
08h0oooo000E0?ooo`800000N03oool2000008l0oooo000G0?ooo`030000003oool0oooo07D0oooo
00<000000?ooo`3oool0SP3oool001P0oooo00<000000?ooo`3oool0M03oool00`000000oooo0?oo
o`2>0?ooo`006@3oool2000007@0oooo00<000000?ooo`3oool0SP3oool001/0oooo00<000000?oo
o`3oool0L@3oool00`000000oooo0?ooo`2>0?ooo`00703oool00`000000oooo0?ooo`1L0?ooo`80
0000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0SP3oool001d0oooo0P00001K0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`2>0?ooo`007`3oool00`000000oooo0?ooo`1H0?ooo`040000
003oool0oooo000000L0oooo1@0000050?ooo`<00000SP3oool00200oooo0P00001B0?ooo`@00000
0P3oool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo000000H0oooo00<000000?oo
o`3oool0SP3oool00280oooo0P00001F0?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`0000001P3oool00`000000oooo0?ooo`2>0?ooo`00903oool00`000000oooo0?ooo`1C0?ooo`04
0000003oool0oooo000000T0oooo0P0000060?ooo`030000003oool0oooo08h0oooo000U0?ooo`80
0000E03oool2000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?ooo`00
9`3oool2000006H0oooo00<000000?ooo`3oool0SP3oool002T0oooo0P00001T0?ooo`800000S`3o
ool002/0oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?ooo`2>0?ooo`00;03oool20000
0640oooo00<000000?ooo`3oool0SP3oool002<0oooo0`000?l80?ooo`030000003oool0oooo05h0
oooo00<000000?ooo`3oool0SP3oool001l0oooo10000?l30?ooo`<0003o1P3oool00`000000oooo
0?ooo`1M0?ooo`030000003oool0oooo08h0oooo000L0?ooo`<0003o2P3oool40000o`<0oooo0P00
001M0?ooo`030000003oool0oooo08h0oooo000H0?ooo`@0003o4@3oool30000o`80oooo00<00000
0?ooo`3oool0FP3oool2000008l0oooo000F0?ooo`80003o603oool30000o`800000FP3oool00`00
0000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo0?ooo`0J0?ooo`80003o00<000000?ooo`3o
ool0E`3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo0?ooo`0L0?ooo`80003o
00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo0?oo
o`0N0?ooo`80003o00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`2>0?ooo`005P3oool0
0`000?l0oooo0?ooo`0P0?ooo`80003o00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`2>
0?ooo`005P3oool00`000?l0oooo0?ooo`0R0?ooo`80003o00<000000?ooo`3oool0C`3oool00`00
0000oooo0?ooo`2>0?ooo`005P3oool00`000?l0oooo0?ooo`0T0?ooo`80003o00<000000?ooo`3o
ool0C@3oool2000008l0oooo000F0?ooo`030000o`3oool0oooo02H0oooo0P000?l2000004`0oooo
00<000000?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0:@3oool20000o`800000BP3o
ool00`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo0?ooo`0[0?ooo`80003o0P000018
0?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo02d0oooo0P000?l20000
0380oooo0P0000040?ooo`030000003oool0oooo0080oooo100000050?ooo`030000003oool0oooo
08h0oooo000E0?ooo`030000o`3oool0oooo02l0oooo0P000?l2000002l0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo000E0?ooo`03
0000o`3oool0oooo0340oooo0P000?l2000002d0oooo00@000000?ooo`3oool000002@3oool00`00
0000oooo0?ooo`050?ooo`<00000SP3oool001D0oooo00<0003o0?ooo`3oool0<`3oool30000o`03
0000003oool0oooo02<0oooo100000020?ooo`040000003oool0oooo000000X0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`2>0?ooo`005@3oool00`000?l0oooo0?ooo`0f0?ooo`80
003o00<000000?ooo`3oool09`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0>03oool20000o`03
0000003oool0oooo02D0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo08h0oooo000E0?ooo`030000o`3oool0oooo03X0oooo0P000?l00`00
0000oooo0?ooo`0T0?ooo`8000002P3oool2000000H0oooo00<000000?ooo`3oool0SP3oool001D0
oooo00<0003o0?ooo`3oool0?03oool20000o`030000003oool0oooo03H0oooo00<000000?ooo`3o
ool0SP3oool001D0oooo00<0003o0?ooo`3oool0?P3oool20000o`030000003oool0oooo03@0oooo
00<000000?ooo`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0@03oool30000oc@0oooo0P00
002?0?ooo`005@3oool00`000?l0oooo0?ooo`130?ooo`<0003o<@3oool00`000000oooo0?ooo`2>
0?ooo`005@3oool00`000?l0oooo0?ooo`150?ooo`03000000000?l0003o02l0oooo00<000000?oo
o`3oool0SP3oool001D0oooo00<0003o0?ooo`3oool0B03oool30000ob`0oooo00<000000?ooo`3o
ool0SP3oool001D0oooo00<0003o0?ooo`3oool0BP3oool010000000003o0000o`000?lY0?ooo`03
0000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo04h0oooo00@000000000o`000?l0
003o9P3oool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`1A0?ooo`030000
00000?l0003o02@0oooo0P00002?0?ooo`00503oool00`000?l0oooo0?ooo`1C0?ooo`0400000000
0?l0003o0000ob40oooo00<000000?ooo`3oool0SP3oool000X0oooo100000060?ooo`030000o`3o
ool0oooo0340oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00000G0?ooo`0400000000
0?l0003o0000oah0oooo00<000000?ooo`3oool0>@3oool2000000@0oooo00<000000?ooo`3oool0
0P3oool3000003X0oooo100000090?ooo`00303oool00`000000oooo0?ooo`050?ooo`030000o`3o
ool0oooo0300oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0G0?ooo`030000
00000?l0003o01`0oooo00<000000?ooo`3oool0>03oool010000000oooo0?ooo`00000;0?ooo`03
0000003oool0oooo03T0oooo00<000000?ooo`3oool0203oool000`0oooo00<000000?ooo`3oool0
1@3oool00`000?l0oooo0?ooo`0`0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3o
ool06P3oool30000oaT0oooo00<000000?ooo`3oool0>03oool010000000oooo0?ooo`00000;0?oo
o`030000003oool0oooo03T0oooo00<000000?ooo`3oool0203oool000@0oooo100000040?ooo`03
0000003oool0oooo00D0oooo00<0003o0?ooo`3oool0:P3oool400000080oooo00@000000?ooo`3o
ool00000203oool3000001l0oooo00@000000000o`000?l0003o5P3oool00`000000oooo0?ooo`0h
0?ooo`040000003oool0oooo000000P0oooo0`00000l0?ooo`030000003oool0oooo00P0oooo000<
0?ooo`030000003oool0oooo00D0oooo00<0003o0?ooo`3oool0<03oool010000000oooo0?ooo`00
00080?ooo`030000003oool0oooo0280oooo00<000000000o`000?l0503oool2000003T0oooo00@0
00000?ooo`3oool00000203oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00P0oooo
000:0?ooo`<000001`3oool00`000?l0oooo0?ooo`0`0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool09@3oool30000o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
>03oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo03X0oooo0`00000:0?ooo`00
303oool00`000000oooo0?ooo`050?ooo`030000o`3oool0oooo0340oooo0P0000090?ooo`@00000
9`3oool30000o`800000303oool00`000000oooo0?ooo`0i0?ooo`8000002@3oool4000003/0oooo
00<000000?ooo`3oool0203oool001@0oooo00<0003o0?ooo`3oool0JP3oool30000o`8000002@3o
ool00`000000oooo0?ooo`2>0?ooo`00503oool00`000?l0oooo0?ooo`1]0?ooo`<0003o0P000006
0?ooo`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo0700oooo10000?l40?oo
o`030000003oool0oooo08h0oooo000D0?ooo`030000o`3oool0oooo07@0oooo0`000?l00`3oool0
00000?ooo`2?0?ooo`000P3ooolB000000030000o`000000000007L0000010000?n;000000<0oooo
00090?ooo`030000003oool0oooo00P0oooo00<0003o0?ooo`0000003@3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`00000010000?l60?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00P0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool00`3oool0
0`000000003o0000o`020000ohH0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?oo
o`3oool01`3oool00`000000003o0000o`020000oh80oooo000C0?ooo`030000o`3oool0oooo07T0
oooo00<000000?ooo`3oool02P3oool2000000@0003oOP3oool001<0oooo00<0003o0?ooo`3oool0
N@3oool00`000000oooo0?ooo`0>0?ooo`8000000`000?mk0?ooo`004`3oool00`000?l0oooo0?oo
o`1i0?ooo`8000004`3oool00`000000003o0000o`020000ogL0oooo000C0?ooo`030000o`3oool0
oooo07T0oooo00<000000?ooo`3oool05P3oool00`000000003o0000o`020000o`030000003oool0
oooo0700oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool06`3oool40000
o`800000K@3oool001<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`0O0?ooo`@0
003o0P00001Y0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo02<0oooo
1P000?mU0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo02T0oooo1`00
0?mN0?ooo`004`3oool00`000?l0oooo0?ooo`1i0?ooo`800000;P3oool3000000H0003oF03oool0
01<0oooo00<0003o0?ooo`3oool0N@3oool00`000000oooo0?ooo`0c0?ooo`<0000010000?mD0?oo
o`004`3oool00`000?l0oooo0?ooo`1i0?ooo`030000003oool0oooo03P0oooo0P0000040000oe00
oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0?P3oool60000odX0oooo
000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0@`3oool00`000000003o0000
o`050000od<0oooo000C0?ooo`030000o`3oool0oooo07T0oooo00<000000?ooo`3oool0B@3oool2
000000L0003o?03oool00180oooo00<0003o0?ooo`3oool0NP3oool00`000000oooo0?ooo`1A0?oo
o`03000000000?l0003o00@0003o0P00000d0?ooo`004P3oool00`000?l0oooo0?ooo`1j0?ooo`80
0000F@3oool70000o`030000003oool0oooo02`0oooo000B0?ooo`030000o`3oool0oooo07X0oooo
00<000000?ooo`3oool0G`3oool70000o`8000009P3oool00180oooo00<0003o0?ooo`3oool0NP3o
ool00`000000oooo0?ooo`1V0?ooo`L0003o2@00000H0?ooo`004P3oool00`000?l0oooo0?ooo`1j
0?ooo`030000003oool0oooo06d0oooo1P000?l30?ooo`h000002P3oool00180oooo00<0003o0?oo
o`3oool0IP3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000D0oooo00<000000?oo
o`3oool0L`3oool:0000oa40oooo000B0?ooo`030000o`3oool0oooo06D0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo07d0oooo1`000?l:0?oo
o`004P3oool00`000?l0oooo0?ooo`1U0?ooo`040000003oool0oooo000000T0oooo00<000000?oo
o`3oool01@3oool3000008h0oooo000B0?ooo`030000o`3oool0oooo06D0oooo00@000000?ooo`3o
ool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08h0oooo000B0?ooo`03
0000o`3oool0oooo06D0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo08h0oooo000B0?ooo`030000o`3oool0oooo06D0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo000B0?oo
o`030000o`3oool0oooo06H0oooo0P00000:0?ooo`8000001P3oool00`000000oooo0?ooo`2>0?oo
o`004P3oool00`000?l0oooo0?ooo`1j0?ooo`030000003oool0oooo08h0oooo000B0?ooo`030000
o`3oool0oooo07X0oooo00<000000?ooo`3oool0SP3oool00180oooo00<0003o0?ooo`3oool0NP3o
ool2000008l0oooo000B0?ooo`030000o`3oool0oooo07X0oooo00<000000?ooo`3oool0SP3oool0
0140oooo00<0003o0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0
oooo0?ooo`1k0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo
00<000000?ooo`3oool0SP3oool00140oooo00<0003o0?ooo`3oool0N`3oool00`000000oooo0?oo
o`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`800000S`3oool00140oooo00<0003o0?oo
o`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`03
0000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0
SP3oool00140oooo00<0003o0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool0
0`000?l0oooo0?ooo`1k0?ooo`030000003oool0oooo08h0oooo000A0?ooo`030000o`3oool0oooo
07/0oooo0P00002?0?ooo`004@3oool00`000?l0oooo0?ooo`1k0?ooo`030000003oool0oooo08h0
oooo000A0?ooo`030000o`3oool0oooo07/0oooo00<000000?ooo`3oool0SP3oool00140oooo00<0
003o0?ooo`3oool0N`3oool00`000000oooo0?ooo`2>0?ooo`004@3oool00`000?l0oooo0?ooo`1k
0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo06P0oooo0P0000040?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2>0?oo
o`00403oool00`000?l0oooo0?ooo`1W0?ooo`040000003oool0oooo000000X0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1W0?ooo`04
0000003oool0oooo000000L0oooo1@0000050?ooo`<00000SP3oool00100oooo00<0003o0?ooo`3o
ool0I`3oool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo000000H0oooo00<00000
0?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0I`3oool010000000oooo0?ooo`000008
0?ooo`030000003oool0000000H0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?oo
o`3oool0I`3oool010000000oooo0?ooo`0000090?ooo`8000001P3oool00`000000oooo0?ooo`2>
0?ooo`00403oool00`000?l0oooo0?ooo`1X0?ooo`8000002`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo07`0oooo00<000000?ooo`3o
ool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool2000008l0oooo000@0?ooo`030000o`3o
ool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo00<0003o0?ooo`3oool0O03oool0
0`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?ooo`1l0?ooo`030000003oool0oooo
08h0oooo000@0?ooo`030000o`3oool0oooo07`0oooo00<000000?ooo`3oool0SP3oool00100oooo
00<0003o0?ooo`3oool0O03oool00`000000oooo0?ooo`2>0?ooo`00403oool00`000?l0oooo0?oo
o`1l0?ooo`030000003oool0oooo08h0oooo000@0?ooo`030000o`3oool0oooo07`0oooo0P00002?
0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000?0?ooo`03
0000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0
O@3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003o
ool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo07d0oooo00<000000?ooo`3oool0SP3oool0
00l0oooo00<0003o0?ooo`3oool0O@3oool2000008l0oooo000?0?ooo`030000o`3oool0oooo07d0
oooo00<000000?ooo`3oool0SP3oool000l0oooo00<0003o0?ooo`3oool0O@3oool00`000000oooo
0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1m0?ooo`030000003oool0oooo08h0oooo000?
0?ooo`030000o`3oool0oooo06T0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000006
0?ooo`030000003oool0oooo08h0oooo000?0?ooo`030000o`3oool0oooo06P0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08h0oooo000?
0?ooo`030000o`3oool0oooo06P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000050?ooo`<00000SP3oool000l0oooo00<0003o0?ooo`3oool0J03oool010000000oooo0?oo
o`0000080?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?oo
o`1X0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?ooo`1X0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2>0?ooo`003`3oool00`000?l0oooo0?oo
o`1Y0?ooo`8000002P3oool3000000D0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o
0?ooo`3oool0OP3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?oo
o`030000003oool0oooo08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo0P00002?0?ooo`00
3P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000>0?ooo`030000o`3o
ool0oooo07h0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool0
0`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo
08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<000000?ooo`3oool0SP3oool000h0oooo
00<0003o0?ooo`3oool0OP3oool2000008l0oooo000>0?ooo`030000o`3oool0oooo07h0oooo00<0
00000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0OP3oool00`000000oooo0?ooo`2>
0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003oool0oooo08h0oooo000>0?ooo`03
0000o`3oool0oooo07h0oooo00<000000?ooo`3oool0SP3oool000h0oooo00<0003o0?ooo`3oool0
OP3oool00`000000oooo0?ooo`2>0?ooo`003P3oool00`000?l0oooo0?ooo`1n0?ooo`030000003o
ool0oooo08h0oooo000>0?ooo`030000o`3oool0oooo07h0oooo0P00002?0?ooo`003@3oool00`00
0?l0oooo0?ooo`1o0?ooo`030000003oool0oooo08h0oooo000=0?ooo`030000o`3oool0oooo07l0
oooo00<000000?ooo`3oool0SP3oool000d0oooo00<0003o0?ooo`3oool0O`3oool00`000000oooo
0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1[0?ooo`800000103oool00`000000oooo0?oo
o`030?ooo`8000001P3oool00`000000oooo0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1Z
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo
0?ooo`2>0?ooo`003@3oool00`000?l0oooo0?ooo`1Z0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000001@3oool3000008h0oooo000=0?ooo`030000o`3oool0oooo06X0oooo
00@000000?ooo`3oool000002@3oool2000000H0oooo00<000000?ooo`3oool0SP3oool000d0oooo
00<0003o0?ooo`3oool0JP3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00D0oooo00<000000?ooo`3oool0SP3oool007X0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`00002F0?ooo`00N`3oool2000000X0oooo0P00002G0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.06789, -0.548322, \
0.0074418, 0.00781929}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["for imaginary omega", "Subsection"],

Cell[BoxData[
    \(deltaspecial[md2_, k_, kz_, om_, a_] := \(md2/k^4\)/\((1 - kz^2/k^2)\)*
        om/2*\((NIntegrate[1/\((1 + a\ xi^2)\)^2*om*kz, {xi, \(-1\), 1}, 
              WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30] + 
            NIntegrate[
              1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*\((k^2\ xi - kz\ om)\)*
                R[om - kz\ xi, 
                  Sqrt[k^2 - kz^2]\ Sqrt[1 - xi^2]], {xi, \(-1\), 1}, 
              WorkingPrecision \[Rule] 30, 
              MaxRecursion \[Rule] 30])\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltaspecial[1, 1, 1/2, I, 1]\)], "Input"],

Cell[BoxData[
    \(0.023865550010996915771868216425177`18.9177 + 
      0``20.5399\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exact-analytical", "Subsection"],

Cell[BoxData[
    \(Clear[s]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apart[\((1 - 
            y^4)\)/\((\((1 + y^2)\) + 
              a \((xxxi1 + xxxi2\ y^2)\)^2)\)^2]\)], "Input"],

Cell[BoxData[
    \(\(1 + a\ xxxi1\^2 + a\ xxxi2\^2 + y\^2 + 2\ a\ xxxi1\ xxxi2\ \
y\^2\)\/\(a\ xxxi2\^2\ \((1 + a\ xxxi1\^2 + y\^2 + 2\ a\ xxxi1\ xxxi2\ y\^2 + \
a\ xxxi2\^2\ y\^4)\)\^2\) - 
      1\/\(a\ xxxi2\^2\ \((1 + a\ xxxi1\^2 + y\^2 + 2\ a\ xxxi1\ xxxi2\ y\^2 \
+ a\ xxxi2\^2\ y\^4)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\(1 + a\ xxxi1\^2 + a\ xxxi2\^2 + y\^2 + 2\ a\ xxxi1\ xxxi2\ \
y\^2\)\/\(a\ xxxi2\^2\ \((1 + a\ xxxi1\^2 + y\^2 + 2\ a\ xxxi1\ xxxi2\ y\^2 + \
a\ xxxi2\^2\ y\^4)\)\^2\), {y, 0, Infinity}]\)], "Input"],

Cell[BoxData[
    \(\(-\(\((\[ImaginaryI]\ \((\(-5\)\ \@2\ \@a\ xxxi2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                5\ \@2\ a\^\(3/2\)\ xxxi1\^2\ xxxi2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                20\ \@2\ a\^\(3/2\)\ xxxi1\ xxxi2\^2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                20\ \@2\ a\^\(5/2\)\ xxxi1\^3\ xxxi2\^2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                11\ \@2\ a\^\(3/2\)\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                4\ \@2\ a\^\(5/2\)\ xxxi1\^2\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                8\ \@2\ a\^\(7/2\)\ xxxi1\^4\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                4\ \@2\ a\^\(5/2\)\ xxxi1\ xxxi2\^4\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                12\ \@2\ a\^\(5/2\)\ xxxi2\^5\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                8\ \@2\ a\^\(7/2\)\ xxxi1\^2\ xxxi2\^5\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + \@2\ \
\@a\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 \
+ Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 \
+ 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + \@2\ a\^\(3/2\)\ xxxi1\^2\
\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                2\ \@2\ a\^\(3/2\)\ xxxi1\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                2\ \@2\ a\^\(5/2\)\ xxxi1\^3\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - \@2\ \
a\^\(3/2\)\ xxxi2\^3\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + \
2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \
\((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                2\ \@2\ a\^\(5/2\)\ xxxi1\ xxxi2\^4\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                5\ \@2\ \@a\ xxxi2\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\
\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                5\ \@2\ a\^\(3/2\)\ xxxi1\^2\ xxxi2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                20\ \@2\ a\^\(3/2\)\ xxxi1\ xxxi2\^2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                20\ \@2\ a\^\(5/2\)\ xxxi1\^3\ xxxi2\^2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                11\ \@2\ a\^\(3/2\)\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                4\ \@2\ a\^\(5/2\)\ xxxi1\^2\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                8\ \@2\ a\^\(7/2\)\ xxxi1\^4\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                4\ \@2\ a\^\(5/2\)\ xxxi1\ xxxi2\^4\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                12\ \@2\ a\^\(5/2\)\ xxxi2\^5\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                8\ \@2\ a\^\(7/2\)\ xxxi1\^2\ xxxi2\^5\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - \@2\ \@a\ xxxi2\
\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - \@2\ a\^\(3/2\)\
\ xxxi1\^2\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ \
a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \
\((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                2\ \@2\ a\^\(3/2\)\ xxxi1\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                2\ \@2\ a\^\(5/2\)\ xxxi1\^3\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + \@2\ a\^\(3/2\)\
\ xxxi2\^3\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 \
+ Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ \
a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                2\ \@2\ a\^\(5/2\)\ xxxi1\ xxxi2\^4\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                5\ \@2\ \@a\ xxxi2\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\
\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                5\ \@2\ a\^\(3/2\)\ xxxi1\^2\ xxxi2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                20\ \@2\ a\^\(3/2\)\ xxxi1\ xxxi2\^2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                20\ \@2\ a\^\(5/2\)\ xxxi1\^3\ xxxi2\^2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                11\ \@2\ a\^\(3/2\)\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                4\ \@2\ a\^\(5/2\)\ xxxi1\^2\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                8\ \@2\ a\^\(7/2\)\ xxxi1\^4\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                4\ \@2\ a\^\(5/2\)\ xxxi1\ xxxi2\^4\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                12\ \@2\ a\^\(5/2\)\ xxxi2\^5\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                8\ \@2\ a\^\(7/2\)\ xxxi1\^2\ xxxi2\^5\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + \@2\ \
\@a\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 \
+ Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 \
+ 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + \@2\ a\^\(3/2\)\ xxxi1\^2\
\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                2\ \@2\ a\^\(3/2\)\ xxxi1\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) + 
                2\ \@2\ a\^\(5/2\)\ xxxi1\^3\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - \@2\ \
a\^\(3/2\)\ xxxi2\^3\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + \
2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \
\((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                2\ \@2\ a\^\(5/2\)\ xxxi1\ xxxi2\^4\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\)\)])\) - 
                5\ \@2\ \@a\ xxxi2\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\
\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                
                5\ \@2\ a\^\(3/2\)\ xxxi1\^2\ xxxi2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                20\ \@2\ a\^\(3/2\)\ xxxi1\ xxxi2\^2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                20\ \@2\ a\^\(5/2\)\ xxxi1\^3\ xxxi2\^2\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                11\ \@2\ a\^\(3/2\)\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                4\ \@2\ a\^\(5/2\)\ xxxi1\^2\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                8\ \@2\ a\^\(7/2\)\ xxxi1\^4\ xxxi2\^3\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                4\ \@2\ a\^\(5/2\)\ xxxi1\ xxxi2\^4\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                12\ \@2\ a\^\(5/2\)\ xxxi2\^5\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                8\ \@2\ a\^\(7/2\)\ xxxi1\^2\ xxxi2\^5\ \@\(1 + 2\ a\ xxxi1\ \
xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - \@2\ \@a\ xxxi2\
\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - \@2\ a\^\(3/2\)\
\ xxxi1\^2\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ \
a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \
\((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                2\ \@2\ a\^\(3/2\)\ xxxi1\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) - 
                2\ \@2\ a\^\(5/2\)\ xxxi1\^3\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + \@2\ a\^\(3/2\)\
\ xxxi2\^3\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ \
xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 \
+ Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ \
a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\) + 
                2\ \@2\ a\^\(5/2\)\ xxxi1\ xxxi2\^4\ \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ \
xxxi2 - 4\ a\ xxxi2\^2\)\)\ \((Log[2]\/2 + 
                      Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\))\))\)/\((8\ a\ \
\((1 + a\ xxxi1\^2)\)\ xxxi2\^2\ \((1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2)\)\^\(3/2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 \
- 4\ a\ xxxi2\^2\)\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 \
- 4\ a\ xxxi2\^2\)\))\)\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[
      1\/\(a\ xxxi2\^2\ \((1 + a\ xxxi1\^2 + y\^2 + 2\ a\ xxxi1\ xxxi2\ y\^2 \
+ a\ xxxi2\^2\ y\^4)\)\), {y, 0, Infinity}]\)], "Input"],

Cell[BoxData[
    \(\((\[ImaginaryI]\ \((\@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\)\)\ Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 \
+ 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\)\)] - \@\
\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\ Log[\
\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\)\)] + \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\)\)\ \((\(-Log[\(-\(\(\[ImaginaryI]\ \@a\ \
xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ \
xxxi2\)\)\)\)]\) + 
                    Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ \
xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)])\))\))\)/\((\@2\ \@a\ \
xxxi2\ \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 \
- \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \
\@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\))\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%66 - %65]\)], "Input"],

Cell[BoxData[
    \(\(-\(\((\[ImaginaryI]\ \((\(\(1\/\(1 + 
                      a\ xxxi1\^2\)\)\((\@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + \
4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\ \((\(-5\) + \@\(1 + 4\ a\ \((xxxi1 - \
xxxi2)\)\ xxxi2\) + a\^3\ \((\(-8\)\ xxxi1\^4\ xxxi2\^2 + 
                                8\ xxxi1\^2\ xxxi2\^4)\) + 
                          a\ \((\(-xxxi2\^2\)\ \((\(-11\) + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\))\) + 
                                2\ xxxi1\ xxxi2\ \((\(-10\) + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\))\) + 
                                xxxi1\^2\ \((\(-5\) + \@\(1 + 4\ a\ \((xxxi1 \
- xxxi2)\)\ xxxi2\))\))\) + 
                          2\ a\^2\ xxxi2\ \((2\ xxxi1\^2\ xxxi2 + 
                                6\ xxxi2\^3 + 
                                xxxi1\^3\ \((\(-10\) + \@\(1 + 4\ a\ \((xxxi1 \
- xxxi2)\)\ xxxi2\))\) - 
                                xxxi1\ xxxi2\^2\ \((2 + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\))\))\))\)\ Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\
\/\@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\)\
\)] - \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\
\)\ \((\(-5\) + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\) + 
                          a\^3\ \((\(-8\)\ xxxi1\^4\ xxxi2\^2 + 
                                8\ xxxi1\^2\ xxxi2\^4)\) + 
                          a\ \((\(-xxxi2\^2\)\ \((\(-11\) + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\))\) + 
                                2\ xxxi1\ xxxi2\ \((\(-10\) + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\))\) + 
                                xxxi1\^2\ \((\(-5\) + \@\(1 + 4\ a\ \((xxxi1 \
- xxxi2)\)\ xxxi2\))\))\) + 
                          2\ a\^2\ xxxi2\ \((2\ xxxi1\^2\ xxxi2 + 
                                6\ xxxi2\^3 + 
                                xxxi1\^3\ \((\(-10\) + \@\(1 + 4\ a\ \((xxxi1 \
- xxxi2)\)\ xxxi2\))\) - 
                                xxxi1\ xxxi2\^2\ \((2 + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\))\))\))\)\ Log[\(\[ImaginaryI]\ \@a\ \
xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ \
xxxi2\)\)] + \@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ \
xxxi2\)\)\ \((5 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\) + 
                          8\ a\^3\ xxxi1\^2\ xxxi2\^2\ \((xxxi1\^2 - 
                                xxxi2\^2)\) + 
                          2\ a\^2\ xxxi2\ \((\(-2\)\ xxxi1\^2\ xxxi2 - 
                                6\ xxxi2\^3 - 
                                xxxi1\ xxxi2\^2\ \((\(-2\) + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\))\) + 
                                xxxi1\^3\ \((10 + \@\(1 + 4\ a\ \((xxxi1 - \
xxxi2)\)\ xxxi2\))\))\) + 
                          a\ \((xxxi1\^2\ \((5 + \@\(1 + 4\ a\ \((xxxi1 - \
xxxi2)\)\ xxxi2\))\) + 
                                2\ xxxi1\ xxxi2\ \((10 + \@\(1 + 4\ a\ \
\((xxxi1 - xxxi2)\)\ xxxi2\))\) - 
                                xxxi2\^2\ \((11 + \@\(1 + 4\ a\ \((xxxi1 - \
xxxi2)\)\ xxxi2\))\))\))\)\ \((Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 \
+ 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\)\)] - 
                          Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)])\))\)\) + 
                4\ \((1 + 
                      4\ a\ \((xxxi1 - 
                            xxxi2)\)\ xxxi2)\)\ \((\@\(1 + 2\ a\ xxxi1\ xxxi2 \
+ \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\ Log[\(-\(\(\[ImaginaryI]\ \@a\
\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ \
xxxi2\)\)\)\)] - \@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\
\)\ xxxi2\)\)\ Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 \
- \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)] + \@\(1 + 2\ a\ xxxi1\ xxxi2 \
- \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\ \
\((\(-Log[\(-\(\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\
\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\)\)]\) + 
                            Log[\(\[ImaginaryI]\ \@a\ xxxi2\)\/\@\(1 + 2\ a\ \
xxxi1\ xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)])\))\))\))\)/\((4\
\ \@2\ \@a\ xxxi2\ \((1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2)\)\^\(3/2\)\ \@\(1 \
+ 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\)\ \@\(1 + \
2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ \((xxxi1 - xxxi2)\)\ xxxi2\)\))\)\)\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\((1 - 
            y^4)\)\ /\((\((1 + y^2)\) + 
              a \((xxxi1 + xxxi2\ y^2)\)^2)\)^2, {y, 0, Infinity}]\)], "Input"],

Cell[BoxData[
    \(\((\(-y\) - 4\ a\ xxxi1\^2\ y - 2\ a\^2\ xxxi1\^4\ y + 
            4\ a\ xxxi1\ xxxi2\ y - 2\ a\ xxxi2\^2\ y + 
            2\ a\^2\ xxxi1\^2\ xxxi2\^2\ y - y\^3 - a\ xxxi1\^2\ y\^3 - 
            2\ a\ xxxi1\ xxxi2\ y\^3 - 2\ a\^2\ xxxi1\^3\ xxxi2\ y\^3 + 
            a\ xxxi2\^2\ y\^3 + 
            2\ a\^2\ xxxi1\ xxxi2\^3\ y\^3)\)/\((2\ \((1 + 
                a\ xxxi1\^2)\)\ \((1 + 4\ a\ xxxi1\ xxxi2 - 
                4\ a\ xxxi2\^2)\)\ \((1 + a\ xxxi1\^2 + y\^2 + 
                2\ a\ xxxi1\ xxxi2\ y\^2 + 
                a\ xxxi2\^2\ y\^4)\))\) - \((\((\(-1\) - a\ xxxi1\^2 - 
                4\ a\ xxxi1\ xxxi2 - 4\ a\^2\ xxxi1\^3\ xxxi2 - 
                5\ a\ xxxi2\^2 - 12\ a\^2\ xxxi1\^2\ xxxi2\^2 - 
                8\ a\^3\ xxxi1\^4\ xxxi2\^2 - 4\ a\^2\ xxxi1\ xxxi2\^3 + 
                12\ a\^2\ xxxi2\^4 + 
                8\ a\^3\ xxxi1\^2\ xxxi2\^4 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ \
a\ xxxi2\^2\) + a\ xxxi1\^2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) + 
                2\ a\ xxxi1\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\) + 
                2\ a\^2\ xxxi1\^3\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\) - a\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) - 
                2\ a\^2\ xxxi1\ xxxi2\^3\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\))\)\ ArcTan[\(\@2\ \@a\ xxxi2\ y\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 - \@\
\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\)/\((2\ \@2\ \@a\ xxxi2\ \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \((1 + a\ xxxi1\^2 + 
                4\ a\ xxxi1\ xxxi2 + 4\ a\^2\ xxxi1\^3\ xxxi2 - 
                4\ a\ xxxi2\^2 - 
                4\ a\^2\ xxxi1\^2\ xxxi2\^2)\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 - \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\))\) - \((\((1 + a\ xxxi1\^2 + 
                4\ a\ xxxi1\ xxxi2 + 4\ a\^2\ xxxi1\^3\ xxxi2 + 
                5\ a\ xxxi2\^2 + 12\ a\^2\ xxxi1\^2\ xxxi2\^2 + 
                8\ a\^3\ xxxi1\^4\ xxxi2\^2 + 4\ a\^2\ xxxi1\ xxxi2\^3 - 
                12\ a\^2\ xxxi2\^4 - 
                8\ a\^3\ xxxi1\^2\ xxxi2\^4 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ \
a\ xxxi2\^2\) + a\ xxxi1\^2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) + 
                2\ a\ xxxi1\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\) + 
                2\ a\^2\ xxxi1\^3\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\) - a\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) - 
                2\ a\^2\ xxxi1\ xxxi2\^3\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\))\)\ ArcTan[\(\@2\ \@a\ xxxi2\ y\)\/\@\(1 + 2\ a\ xxxi1\ xxxi2 + \@\
\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\)])\)/\((2\ \@2\ \@a\ xxxi2\ \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\ \((1 + a\ xxxi1\^2 + 
                4\ a\ xxxi1\ xxxi2 + 4\ a\^2\ xxxi1\^3\ xxxi2 - 
                4\ a\ xxxi2\^2 - 
                4\ a\^2\ xxxi1\^2\ xxxi2\^2)\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% /. y \[Rule] Infinity\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"-", 
        RowBox[{
          RowBox[{"(", 
            
            RowBox[{\((\(-1\) - a\ xxxi1\^2 - 4\ a\ xxxi1\ xxxi2 - 
                  4\ a\^2\ xxxi1\^3\ xxxi2 - 5\ a\ xxxi2\^2 - 
                  12\ a\^2\ xxxi1\^2\ xxxi2\^2 - 
                  8\ a\^3\ xxxi1\^4\ xxxi2\^2 - 4\ a\^2\ xxxi1\ xxxi2\^3 + 
                  12\ a\^2\ xxxi2\^4 + 
                  8\ a\^3\ xxxi1\^2\ xxxi2\^4 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - \
4\ a\ xxxi2\^2\) + 
                  a\ xxxi1\^2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) \
+ 2\ a\ xxxi1\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) + 
                  2\ a\^2\ xxxi1\^3\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\
\ xxxi2\^2\) - a\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) - 
                  2\ a\^2\ xxxi1\ xxxi2\^3\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\
\ xxxi2\^2\))\), " ", 
              RowBox[{"ArcTan", "[", 
                FractionBox[
                  RowBox[{"xxxi2", " ", 
                    InterpretationBox["\[Infinity]",
                      DirectedInfinity[ 1]], 
                    " ", \(\@Sign[a]\)}], \(\@Sign[
                        1 + 2\ a\ xxxi1\ xxxi2 - \@\(1 + 4\ a\ xxxi1\ xxxi2 - \
4\ a\ xxxi2\^2\)]\)], "]"}]}], ")"}], 
          "/", \((2\ \@2\ \@a\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\)\ \((1 + a\ xxxi1\^2 + 4\ a\ xxxi1\ xxxi2 + 
                  4\ a\^2\ xxxi1\^3\ xxxi2 - 4\ a\ xxxi2\^2 - 
                  4\ a\^2\ xxxi1\^2\ xxxi2\^2)\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 \
- \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\))\)}]}], "-", 
      RowBox[{
        RowBox[{"(", 
          
          RowBox[{\((1 + a\ xxxi1\^2 + 4\ a\ xxxi1\ xxxi2 + 
                4\ a\^2\ xxxi1\^3\ xxxi2 + 5\ a\ xxxi2\^2 + 
                12\ a\^2\ xxxi1\^2\ xxxi2\^2 + 8\ a\^3\ xxxi1\^4\ xxxi2\^2 + 
                4\ a\^2\ xxxi1\ xxxi2\^3 - 12\ a\^2\ xxxi2\^4 - 
                8\ a\^3\ xxxi1\^2\ xxxi2\^4 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ \
a\ xxxi2\^2\) + a\ xxxi1\^2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) + 
                2\ a\ xxxi1\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\) + 
                2\ a\^2\ xxxi1\^3\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\) - a\ xxxi2\^2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\) - 
                2\ a\^2\ xxxi1\ xxxi2\^3\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\))\), " ", 
            RowBox[{"ArcTan", "[", 
              FractionBox[
                RowBox[{"xxxi2", " ", 
                  InterpretationBox["\[Infinity]",
                    DirectedInfinity[ 1]], 
                  " ", \(\@Sign[a]\)}], \(\@Sign[
                      1 + 2\ a\ xxxi1\ xxxi2 + \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\
\ a\ xxxi2\^2\)]\)], "]"}]}], ")"}], 
        "/", \((2\ \@2\ \@a\ xxxi2\ \@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ \
xxxi2\^2\)\ \((1 + a\ xxxi1\^2 + 4\ a\ xxxi1\ xxxi2 + 
                4\ a\^2\ xxxi1\^3\ xxxi2 - 4\ a\ xxxi2\^2 - 
                4\ a\^2\ xxxi1\^2\ xxxi2\^2)\)\ \@\(1 + 2\ a\ xxxi1\ xxxi2 + \
\@\(1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2\)\))\)}], "+", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            InterpretationBox[\(-\[Infinity]\),
              DirectedInfinity[ -1]], "+", 
            RowBox[{"a", " ", "xxxi1", " ", "xxxi2", " ", 
              RowBox[{"(", 
                InterpretationBox[\(-\[Infinity]\),
                  DirectedInfinity[ -1]], ")"}]}], "+", 
            RowBox[{"a", " ", "xxxi1", " ", "xxxi2", " ", 
              InterpretationBox["\[Infinity]",
                DirectedInfinity[ 1]]}], "+", 
            RowBox[{"a", " ", \(xxxi2\^2\), " ", 
              InterpretationBox["\[Infinity]",
                DirectedInfinity[ 1]]}], "+", 
            RowBox[{"a", " ", 
              RowBox[{"(", 
                InterpretationBox[\(-\[Infinity]\),
                  DirectedInfinity[ -1]], ")"}], " ", \(Sign[xxxi1]\^2\)}], 
            "+", 
            RowBox[{"xxxi2", " ", 
              RowBox[{"(", 
                InterpretationBox[\(-\[Infinity]\),
                  DirectedInfinity[ -1]], ")"}], " ", \(Sign[a]\^2\), 
              " ", \(Sign[xxxi1]\^3\)}], "+", 
            RowBox[{
              InterpretationBox[\(-\[Infinity]\),
                DirectedInfinity[ -1]], " ", \(Sign[a]\^2\), 
              " ", \(Sign[xxxi1]\^4\)}], "+", 
            RowBox[{"a", " ", 
              RowBox[{"(", 
                InterpretationBox[\(-\[Infinity]\),
                  DirectedInfinity[ -1]], ")"}], " ", \(Sign[xxxi2]\^2\)}], 
            "+", 
            RowBox[{
              InterpretationBox["\[Infinity]",
                DirectedInfinity[ 1]], " ", \(Sign[a]\^2\), 
              " ", \(Sign[xxxi1]\^2\), " ", \(Sign[xxxi2]\^2\)}], "+", 
            RowBox[{"xxxi1", " ", 
              InterpretationBox["\[Infinity]",
                DirectedInfinity[ 1]], " ", \(Sign[a]\^2\), 
              " ", \(Sign[xxxi2]\^3\)}]}], ")"}], "/", 
        RowBox[{"(", 
          RowBox[{
          "2", " ", \((1 + a\ xxxi1\^2)\), 
            " ", \((1 + 4\ a\ xxxi1\ xxxi2 - 4\ a\ xxxi2\^2)\), " ", 
            RowBox[{"(", 
              RowBox[{"1", "+", \(a\ xxxi1\^2\), "+", 
                InterpretationBox["\[Infinity]",
                  DirectedInfinity[ 1]], "+", 
                RowBox[{"a", " ", "xxxi1", " ", "xxxi2", " ", 
                  InterpretationBox["\[Infinity]",
                    DirectedInfinity[ 1]]}], "+", 
                RowBox[{"a", " ", \(xxxi2\^2\), " ", 
                  InterpretationBox["\[Infinity]",
                    DirectedInfinity[ 1]]}]}], ")"}]}], ")"}]}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[x^2/Sqrt[s^2 - x^2], {x, \(-s\), s}, 
      Assumptions \[Rule] {s > 0}]\)], "Input"],

Cell[BoxData[
    \(\(\[Pi]\ s\^2\)\/2\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(ImdeltaA[md2_, k_, kz_, om_, a_] := \(-md2\)*a*om*
        kz/k^3*\((1 - om^2/k^2)\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fast delta", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imarg3 = 
      Compile[{md2, k, kz, om, a, xi}, 
        1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*\((k^2\ xi - kz\ om)\)/
            Sqrt[\(-\((k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - om^2)\)*\((1 - 
                          kz^2/k^2)\))\)\)]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(\((om + a\ kz\ xi)\)\ \((k\^2\ xi - kz\ om)\)\)\/\(\((1 + a\ \
xi\^2)\)\^2\ \@\(-\((k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - \
om\^2)\)\ \((1 - kz\^2\/k\^2)\))\)\)\), "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(FastImdelta[md2_, k_, kz_, om_, a_] := 
      If[om > k, 
        0, \(\(-md2\)/k^4\)/\((1 - kz^2/k^2)\)*om/2*
          NIntegrate[
            Imarg3[md2, k, kz, om, a, xi], {xi, 
              xi2[k, kz, om]*
                If[xi2[k, kz, om] > 
                    0, \((1 + 10^\((\(-13\))\))\), \((1 - 
                      10^\((\(-13\))\))\)], 
              xi1[k, kz, om]*\((1 - 10^\((\(-15\))\))\)}]]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Plot[FastImdelta[1, 1, 8/10, om, 1], {om, 1/10, 1}]]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"rnderr" \(\(:\)\(\ \)\) 
      "Numerical approximation has caused \!\({xi}\) to take the value \
\!\({\(-0.019711511703358255`\)}\) where the integrand is singular."\)], \
"Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(Power :: \"infy\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"rnderr" \(\(:\)\(\ \)\) 
      "Numerical approximation has caused \!\({xi}\) to take the value \
\!\({\(-0.019711511703358255`\)}\) where the integrand is singular."\)], \
"Message"],

Cell[BoxData[
    \(NIntegrate::"rnderr" \(\(:\)\(\ \)\) 
      "Numerical approximation has caused \!\({xi}\) to take the value \
\!\({\(-0.019711511703358255`\)}\) where the integrand is singular."\)], \
"Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"rnderr\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.7998356532343436`\)."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.185983 2.61233 [
[.21429 .17348 -9 -9 ]
[.21429 .17348 9 0 ]
[.40476 .17348 -9 -9 ]
[.40476 .17348 9 0 ]
[.59524 .17348 -9 -9 ]
[.59524 .17348 9 0 ]
[.78571 .17348 -9 -9 ]
[.78571 .17348 9 0 ]
[.97619 .17348 -3 -9 ]
[.97619 .17348 3 0 ]
[.01131 .05537 -30 -4.5 ]
[.01131 .05537 0 4.5 ]
[.01131 .3166 -24 -4.5 ]
[.01131 .3166 0 4.5 ]
[.01131 .44722 -18 -4.5 ]
[.01131 .44722 0 4.5 ]
[.01131 .57783 -24 -4.5 ]
[.01131 .57783 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .18598 m
.21429 .19223 L
s
[(0.2)] .21429 .17348 0 1 Mshowa
.40476 .18598 m
.40476 .19223 L
s
[(0.4)] .40476 .17348 0 1 Mshowa
.59524 .18598 m
.59524 .19223 L
s
[(0.6)] .59524 .17348 0 1 Mshowa
.78571 .18598 m
.78571 .19223 L
s
[(0.8)] .78571 .17348 0 1 Mshowa
.97619 .18598 m
.97619 .19223 L
s
[(1)] .97619 .17348 0 1 Mshowa
.125 Mabswid
.07143 .18598 m
.07143 .18973 L
s
.11905 .18598 m
.11905 .18973 L
s
.16667 .18598 m
.16667 .18973 L
s
.2619 .18598 m
.2619 .18973 L
s
.30952 .18598 m
.30952 .18973 L
s
.35714 .18598 m
.35714 .18973 L
s
.45238 .18598 m
.45238 .18973 L
s
.5 .18598 m
.5 .18973 L
s
.54762 .18598 m
.54762 .18973 L
s
.64286 .18598 m
.64286 .18973 L
s
.69048 .18598 m
.69048 .18973 L
s
.7381 .18598 m
.7381 .18973 L
s
.83333 .18598 m
.83333 .18973 L
s
.88095 .18598 m
.88095 .18973 L
s
.92857 .18598 m
.92857 .18973 L
s
.25 Mabswid
0 .18598 m
1 .18598 L
s
.02381 .05537 m
.03006 .05537 L
s
[(-0.05)] .01131 .05537 1 0 Mshowa
.02381 .3166 m
.03006 .3166 L
s
[(0.05)] .01131 .3166 1 0 Mshowa
.02381 .44722 m
.03006 .44722 L
s
[(0.1)] .01131 .44722 1 0 Mshowa
.02381 .57783 m
.03006 .57783 L
s
[(0.15)] .01131 .57783 1 0 Mshowa
.125 Mabswid
.02381 .08149 m
.02756 .08149 L
s
.02381 .10761 m
.02756 .10761 L
s
.02381 .13374 m
.02756 .13374 L
s
.02381 .15986 m
.02756 .15986 L
s
.02381 .21211 m
.02756 .21211 L
s
.02381 .23823 m
.02756 .23823 L
s
.02381 .26435 m
.02756 .26435 L
s
.02381 .29048 m
.02756 .29048 L
s
.02381 .34272 m
.02756 .34272 L
s
.02381 .36885 m
.02756 .36885 L
s
.02381 .39497 m
.02756 .39497 L
s
.02381 .42109 m
.02756 .42109 L
s
.02381 .47334 m
.02756 .47334 L
s
.02381 .49946 m
.02756 .49946 L
s
.02381 .52559 m
.02756 .52559 L
s
.02381 .55171 m
.02756 .55171 L
s
.02381 .02924 m
.02756 .02924 L
s
.02381 .00312 m
.02756 .00312 L
s
.02381 .60396 m
.02756 .60396 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.11905 .08834 m
.15382 .05949 L
.17192 .0468 L
.19174 .03504 L
.21018 .02631 L
.22736 .02027 L
.2368 .01786 L
.24144 .01692 L
.24573 .01619 L
.24955 .01567 L
.25375 .01522 L
.25593 .01505 L
.25797 .01491 L
.26013 .01481 L
.26134 .01477 L
.26245 .01474 L
.26361 .01472 L
.2647 .01472 L
.26568 .01472 L
.26674 .01473 L
.2679 .01475 L
.26848 .01476 L
.26912 .01478 L
.27131 .01488 L
.27336 .015 L
.27556 .01517 L
.27954 .01558 L
.28455 .01628 L
.28929 .01714 L
.29819 .01927 L
.30688 .02199 L
.31611 .0256 L
.33268 .03395 L
.35021 .04537 L
.36666 .05851 L
.40148 .09385 L
.4385 .14193 L
.47417 .19706 L
.50846 .25636 L
.54497 .32363 L
.58011 .38932 L
.61746 .4562 L
.65344 .51362 L
.68806 .5584 L
.70595 .5763 L
.71508 .58385 L
.72489 .59067 L
.73409 .5958 L
.7425 .59937 L
.74655 .60069 L
Mistroke
.75096 .60183 L
.75298 .60224 L
.75512 .60261 L
.75697 .60287 L
.75899 .60308 L
.7602 .60318 L
.76134 .60325 L
.76197 .60328 L
.76254 .60329 L
.7632 .60331 L
.76382 .60332 L
.76493 .60332 L
.76595 .6033 L
.76708 .60325 L
.76764 .60322 L
.76827 .60318 L
.76957 .60308 L
.77076 .60296 L
.77344 .6026 L
.77581 .60217 L
.77829 .60162 L
.78279 .60034 L
.78697 .59882 L
.79646 .5942 L
.80595 .58791 L
.8149 .58042 L
.83164 .5623 L
.84969 .53668 L
.86631 .50749 L
.90182 .42728 L
.93597 .32811 L
.97232 .20076 L
.97619 .18598 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3o
ool200000?d0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo
0?`0oooo000Q0?ooo`030000003oool0oooo03<0oooo4000002i0?ooo`008@3oool00`000000oooo
0?ooo`0a0?ooo`800000403oool500000;@0oooo000Q0?ooo`030000003oool0oooo02l0oooo0P00
000G0?ooo`800000/P3oool00240oooo0P00000]0?ooo`<000006`3oool00`000000oooo0?ooo`2_
0?ooo`008@3oool00`000000oooo0?ooo`0Z0?ooo`8000007`3oool200000:l0oooo000Q0?ooo`03
0000003oool0oooo02P0oooo0P00000S0?ooo`800000[@3oool00240oooo00<000000?ooo`3oool0
9P3oool2000002L0oooo0P00002[0?ooo`008@3oool00`000000oooo0?ooo`0T0?ooo`800000:`3o
ool200000:T0oooo00080?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool3
000000D0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0
oooo0:H0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0240oooo0P00000a0?ooo`030000003o
ool0oooo0:D0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
1@3oool00`000000oooo0?ooo`020?ooo`8000008@3oool00`000000oooo0?ooo`0b0?ooo`030000
003oool0oooo0:@0oooo00000`3oool000000000000200000080oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000020?ooo`<000001@3oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool0X`3oool000L0oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0:80oooo00070?ooo`04
0000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`2Q
0?ooo`00203oool2000000X0oooo0P0000030?ooo`@00000103oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo03X0oooo00<000000?ooo`3oool0X03oool00240oooo00<000000?ooo`3o
ool06P3oool2000003h0oooo00<000000?ooo`3oool0W`3oool00240oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo09h0oooo000Q0?ooo`8000006@3o
ool00`000000oooo0?ooo`110?ooo`030000003oool0oooo09d0oooo000Q0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`2L0?ooo`008@3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0V`3oool00240
oooo00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`2J0?ooo`008@3oool00`000000oooo
0?ooo`1P0?ooo`030000003oool0oooo09T0oooo000Q0?ooo`030000003oool0oooo0640oooo00<0
00000?ooo`3oool0V03oool00240oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?ooo`2H
0?ooo`008@3oool2000006<0oooo00<000000?ooo`3oool0U`3oool00240oooo00<000000?ooo`3o
ool0H`3oool00`000000oooo0?ooo`2F0?ooo`008@3oool00`000000oooo0?ooo`1T0?ooo`030000
003oool0oooo09D0oooo000Q0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool0U@3o
ool00240oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`2D0?ooo`008@3oool00`00
0000oooo0?ooo`1V0?ooo`030000003oool0oooo09<0oooo000Q0?ooo`800000J03oool00`000000
oooo0?ooo`2B0?ooo`008@3oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo0980oooo
000Q0?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool0T@3oool00240oooo00<00000
0?ooo`3oool0J@3oool00`000000oooo0?ooo`2@0?ooo`008@3oool00`000000oooo0?ooo`0W0?oo
o`800000103oool00`000000oooo0?ooo`020?ooo`@000008`3oool2000000@0oooo00<000000?oo
o`3oool0103oool01@000000oooo0?ooo`3oool0000001l0oooo0P0000040?ooo`030000003oool0
oooo00<0oooo0P00000T0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000:03oool4
000000H0oooo000Q0?ooo`030000003oool0oooo02H0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo0?ooo`0S0?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo
0000000N0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000008P3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000002T0oooo00<000000?ooo`3oool01@3o
ool00240oooo00<000000?ooo`3oool09P3oool010000000oooo0?ooo`0000090?ooo`030000003o
ool0oooo0280oooo00@000000?ooo`3oool000001`3oool5000000<0oooo00<000000?ooo`3oool0
6`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000280oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`00000Y0?ooo`030000003oool0oooo00D0oooo000Q
0?ooo`8000009`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0240oooo00@0
00000?ooo`3oool000001`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo01X0
oooo00@000000?ooo`3oool00000203oool3000002<0oooo00@000000?ooo`3oool000002@3oool2
000002X0oooo00<000000?ooo`3oool01@3oool00240oooo00<000000?ooo`3oool09P3oool01000
0000oooo0?ooo`00000;0?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool00000203o
ool00`000000oooo000000050?ooo`030000003oool0oooo01X0oooo00@000000?ooo`3oool00000
203oool00`000000oooo0?ooo`0S0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool00000:@3oool00`000000oooo0?ooo`050?ooo`008@3oool00`000000oooo0?ooo`0V0?ooo`04
0000003oool0oooo000000P0oooo00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`00
00090?ooo`8000001P3oool00`000000oooo0?ooo`0I0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool08`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
02L0oooo0`0000070?ooo`008@3oool00`000000oooo0?ooo`0W0?ooo`8000002P3oool2000002@0
oooo0P00000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool2000000X0
oooo0`00000S0?ooo`8000002P3oool2000002X0oooo00<000000?ooo`3oool01@3oool00240oooo
00<000000?ooo`3oool0KP3oool00`000000oooo0?ooo`2;0?ooo`008@3oool00`000000oooo0?oo
o`1_0?ooo`030000003oool0oooo08X0oooo000Q0?ooo`030000003oool0oooo06l0oooo00<00000
0?ooo`3oool0RP3oool001/0ooooo`000006000000008@3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`050?ooo`00
8@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
;P3oool00`000000oooo0?ooo`050?ooo`008@3oool00`000000oooo0?ooo`1b0?ooo`030000003o
ool0oooo07h0oooo00<000000?ooo`3oool01P3oool00240oooo00<000000?ooo`3oool0L`3oool0
0`000000oooo0?ooo`1m0?ooo`030000003oool0oooo00H0oooo000Q0?ooo`030000003oool0oooo
07<0oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`060?ooo`008@3oool00`000000
oooo0?ooo`1d0?ooo`030000003oool0oooo07/0oooo00<000000?ooo`3oool01`3oool00240oooo
0P00001e0?ooo`030000003oool0oooo07/0oooo00<000000?ooo`3oool01`3oool00240oooo00<0
00000?ooo`3oool0M@3oool00`000000oooo0?ooo`1j0?ooo`030000003oool0oooo00L0oooo000Q
0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?ooo`07
0?ooo`008@3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo07P0oooo00<000000?oo
o`3oool0203oool00240oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`1g0?ooo`03
0000003oool0oooo00P0oooo000Q0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool0
M`3oool00`000000oooo0?ooo`080?ooo`008@3oool00`000000oooo0?ooo`1h0?ooo`030000003o
ool0oooo07H0oooo00<000000?ooo`3oool0203oool00240oooo0P00001j0?ooo`030000003oool0
oooo07@0oooo00<000000?ooo`3oool02@3oool00240oooo00<000000?ooo`3oool0N@3oool00`00
0000oooo0?ooo`1d0?ooo`030000003oool0oooo00T0oooo000Q0?ooo`030000003oool0oooo07X0
oooo00<000000?ooo`3oool0L`3oool00`000000oooo0?ooo`090?ooo`008@3oool00`000000oooo
0?ooo`1j0?ooo`030000003oool0oooo0780oooo00<000000?ooo`3oool02P3oool00240oooo00<0
00000?ooo`3oool0N`3oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo00X0oooo000Q
0?ooo`030000003oool0oooo07/0oooo00<000000?ooo`3oool0L@3oool00`000000oooo0?ooo`0:
0?ooo`008@3oool2000007d0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`0:0?oo
o`008@3oool00`000000oooo0?ooo`1l0?ooo`030000003oool0oooo06l0oooo00<000000?ooo`3o
ool02`3oool00240oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`1^0?ooo`030000
003oool0oooo00/0oooo000Q0?ooo`030000003oool0oooo07d0oooo00<000000?ooo`3oool0KP3o
ool00`000000oooo0?ooo`0;0?ooo`008@3oool00`000000oooo0?ooo`1n0?ooo`030000003oool0
oooo06d0oooo00<000000?ooo`3oool02`3oool00240oooo00<000000?ooo`3oool0OP3oool00`00
0000oooo0?ooo`1/0?ooo`030000003oool0oooo00`0oooo000Q0?ooo`030000003oool0oooo07l0
oooo00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`0<0?ooo`008@3oool200000800oooo
00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`0<0?ooo`008@3oool00`000000oooo0?oo
o`200?ooo`030000003oool0oooo06T0oooo00<000000?ooo`3oool03@3oool00240oooo00<00000
0?ooo`3oool0P03oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo00d0oooo000Q0?oo
o`030000003oool0oooo0840oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`0=0?oo
o`008@3oool00`000000oooo0?ooo`210?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3o
ool03@3oool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`<00000
1@3oool00`000000oooo0?ooo`220?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0
3P3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0PP3oool00`000000oooo0?ooo`1V0?ooo`03
0000003oool0oooo00h0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000001@3oool00`000000oooo0?ooo`020?ooo`800000Q03oool00`000000oooo0?ooo`1U0?oo
o`030000003oool0oooo00h0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool000000P3oool3000000D0oooo00<000000?ooo`3oool0P`3oool00`000000oooo0?ooo`1U
0?ooo`030000003oool0oooo00h0oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo08@0oooo00<0
00000?ooo`3oool0H`3oool00`000000oooo0?ooo`0?0?ooo`001`3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`250?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool03`3oool000P0oooo0P00
000:0?ooo`8000000`3oool4000000@0oooo00<000000?ooo`3oool0Q@3oool00`000000oooo0?oo
o`1R0?ooo`030000003oool0oooo00l0oooo000Q0?ooo`030000003oool0oooo08H0oooo00<00000
0?ooo`3oool0H03oool00`000000oooo0?ooo`0@0?ooo`008@3oool00`000000oooo0?ooo`260?oo
o`030000003oool0oooo0600oooo00<000000?ooo`3oool0403oool00240oooo0P0000280?ooo`03
0000003oool0oooo05l0oooo00<000000?ooo`3oool0403oool00240oooo00<000000?ooo`3oool0
Q`3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo0140oooo000Q0?ooo`030000003o
ool0oooo08P0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`0A0?ooo`008@3oool0
0`000000oooo0?ooo`280?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool04@3oool0
0240oooo00<000000?ooo`3oool0R@3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo
0180oooo000Q0?ooo`030000003oool0oooo08T0oooo00<000000?ooo`3oool0F`3oool00`000000
oooo0?ooo`0B0?ooo`008@3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo05X0oooo
00<000000?ooo`3oool04P3oool00240oooo0P00002;0?ooo`030000003oool0oooo05T0oooo00<0
00000?ooo`3oool04`3oool00240oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1H
0?ooo`030000003oool0oooo01<0oooo000Q0?ooo`030000003oool0oooo08/0oooo00<000000?oo
o`3oool0F03oool00`000000oooo0?ooo`0C0?ooo`008@3oool00`000000oooo0?ooo`2<0?ooo`03
0000003oool0oooo05H0oooo00<000000?ooo`3oool0503oool00240oooo00<000000?ooo`3oool0
S03oool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo01@0oooo000Q0?ooo`030000003o
ool0oooo08d0oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`0E0?ooo`008@3oool2
000008l0oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`0E0?ooo`008@3oool00`00
0000oooo0?ooo`2>0?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool05@3oool00240
oooo00<000000?ooo`3oool0S`3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo01H0
oooo000Q0?ooo`030000003oool0oooo08l0oooo00<000000?ooo`3oool0D@3oool00`000000oooo
0?ooo`0F0?ooo`008@3oool00`000000oooo0?ooo`2@0?ooo`030000003oool0oooo0500oooo00<0
00000?ooo`3oool05P3oool00240oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`1>
0?ooo`030000003oool0oooo01L0oooo000Q0?ooo`030000003oool0oooo0940oooo00<000000?oo
o`3oool0CP3oool00`000000oooo0?ooo`0G0?ooo`008@3oool2000009<0oooo00<000000?ooo`3o
ool0C@3oool00`000000oooo0?ooo`0G0?ooo`008@3oool00`000000oooo0?ooo`2B0?ooo`030000
003oool0oooo04`0oooo00<000000?ooo`3oool0603oool00240oooo00<000000?ooo`3oool0T`3o
ool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo01P0oooo000Q0?ooo`030000003oool0
oooo09<0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`0H0?ooo`008@3oool00`00
0000oooo0?ooo`2D0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool06@3oool000h0
oooo0P0000040?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo09D0
oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`0I0?ooo`003@3oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0U@3oool00`000000
oooo0?ooo`170?ooo`030000003oool0oooo01X0oooo000=0?ooo`040000003oool0oooo000000X0
oooo00<000000?ooo`3oool00`3oool2000009L0oooo00<000000?ooo`3oool0AP3oool00`000000
oooo0?ooo`0J0?ooo`003@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0UP3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo01/0
oooo000=0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`2G0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool06`3oool000d0oooo
00@000000?ooo`3oool00000203oool3000000D0oooo00<000000?ooo`3oool0V03oool00`000000
oooo0?ooo`130?ooo`030000003oool0oooo01/0oooo000>0?ooo`8000002`3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool0@P3oool00`000000oooo
0?ooo`0L0?ooo`008@3oool00`000000oooo0?ooo`2I0?ooo`030000003oool0oooo0440oooo00<0
00000?ooo`3oool0703oool00240oooo00<000000?ooo`3oool0V@3oool00`000000oooo0?ooo`10
0?ooo`030000003oool0oooo01d0oooo000Q0?ooo`800000V`3oool00`000000oooo0?ooo`0o0?oo
o`030000003oool0oooo01d0oooo000Q0?ooo`030000003oool0oooo09/0oooo00<000000?ooo`3o
ool0?@3oool00`000000oooo0?ooo`0N0?ooo`008@3oool00`000000oooo0?ooo`2K0?ooo`030000
003oool0oooo03d0oooo00<000000?ooo`3oool07P3oool00240oooo00<000000?ooo`3oool0W03o
ool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo01h0oooo000Q0?ooo`030000003oool0
oooo09`0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0O0?ooo`008@3oool00`00
0000oooo0?ooo`2M0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool07`3oool00240
oooo00<000000?ooo`3oool0WP3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo0200
oooo000Q0?ooo`800000W`3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo0200oooo
000Q0?ooo`030000003oool0oooo09l0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?oo
o`0Q0?ooo`008@3oool00`000000oooo0?ooo`2O0?ooo`030000003oool0oooo03H0oooo00<00000
0?ooo`3oool08@3oool00240oooo00<000000?ooo`3oool0X03oool00`000000oooo0?ooo`0d0?oo
o`030000003oool0oooo0280oooo000Q0?ooo`030000003oool0oooo0:40oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0R0?ooo`008@3oool00`000000oooo0?ooo`2Q0?ooo`030000
003oool0oooo0380oooo00<000000?ooo`3oool08`3oool00240oooo0P00002S0?ooo`030000003o
ool0oooo0300oooo00<000000?ooo`3oool0903oool00240oooo00<000000?ooo`3oool0X`3oool0
0`000000oooo0?ooo`0^0?ooo`030000003oool0oooo02D0oooo000Q0?ooo`030000003oool0oooo
0:@0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0U0?ooo`008@3oool00`000000
oooo0?ooo`2T0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool09P3oool00240oooo
00<000000?ooo`3oool0Y@3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02L0oooo
000Q0?ooo`030000003oool0oooo0:H0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?oo
o`0W0?ooo`008@3oool00`000000oooo0?ooo`2W0?ooo`030000003oool0oooo02L0oooo00<00000
0?ooo`3oool0:03oool00240oooo0P00002X0?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool0:03oool00240oooo00<000000?ooo`3oool0Z03oool00`000000oooo0?ooo`0U0?ooo`03
0000003oool0oooo02T0oooo000Q0?ooo`030000003oool0oooo0:T0oooo00<000000?ooo`3oool0
903oool00`000000oooo0?ooo`0Y0?ooo`008@3oool00`000000oooo0?ooo`2Z0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool0:P3oool00240oooo00<000000?ooo`3oool0Z`3oool0
0`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02/0oooo00080?ooo`800000103oool00`00
0000oooo0?ooo`020?ooo`@000000P3oool3000000D0oooo00<000000?ooo`3oool0[03oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo02`0oooo00070?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0:d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0]0?ooo`001`3oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool20000
0:l0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0^0?ooo`001`3oool010000000
oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`2_
0?ooo`8000006@3oool00`000000oooo0?ooo`0_0?ooo`001`3oool010000000oooo0?ooo`00000:
0?ooo`050000003oool0oooo0?ooo`0000001`3oool00`000000oooo0?ooo`2a0?ooo`8000005P3o
ool00`000000oooo0?ooo`0`0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`<000000`3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool04@3o
ool2000003<0oooo00080?ooo`8000002`3oool010000000oooo0?ooo`3oool4000000@0oooo00<0
00000?ooo`3oool0]03oool00`000000oooo0?ooo`0>0?ooo`800000=@3oool00240oooo00<00000
0?ooo`3oool0]@3oool2000000`0oooo0P00000g0?ooo`008@3oool00`000000oooo0?ooo`2g0?oo
o`H0000000<0oooo0000000000000`00000i0?ooo`008@3oool200000;d0oooo0`00000m0?ooo`00
8@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool0
0`000000oooo0?ooo`3l0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3o
oolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.134313, -0.083188, \
0.00403944, 0.00147266}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{\(1.419999999999959`\ Second\), ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Plot[Imdelta[1, 1.0, 8/10, om, 1], {om, 1/10, 1}]]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\(0.1000000375` \
\[InvisibleSpace]\)\) + \(\(0.8`\\ xi\)\))\)\)\\ \(\((\(\(1.`\^2\\ xi\)\) - \
\(\(0.8`\\ \(\(\[LeftSkeleton] 13 \[RightSkeleton]\)\)\)\))\)\)\)\/\(\((1 + \
\(\(1\\ xi\^2\)\))\)\^2\\ \@\(-\(\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\
\)\)\)\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((\(\(0.1365102924156242` \[InvisibleSpace]\)\) + \(\(0.8`\\ \
xi\)\))\)\)\\ \(\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\(\((1 + \(\
\(1\\ xi\^2\)\))\)\^2\\ \@\(-\(\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\
\)\)\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.4851749576394496`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((\(\(0.1365102924156242` \[InvisibleSpace]\)\) + \(4\\ \
xi\)\/5)\)\)\\ \(\((\(\(1.`\^2\\ xi\)\) - \(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/5)\)\)\)\/\(\((1 + \(\(1\\ xi\^2\)\))\)\^2\\ \@\(-\(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\)\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.7035914255044484`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.4851749576394496`\)."\
\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(Plot::"plnr" \(\(:\)\(\ \)\) 
      "\!\(Imdelta[\(\(1, 1.`, 8\/10, om, 1\)\)]\) is not a machine-size real \
number at \!\(om\) = \!\(0.1365102924156242`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(Plot::"plnr" \(\(:\)\(\ \)\) 
      "\!\(Imdelta[\(\(1, 1.`, 8\/10, om, 1\)\)]\) is not a machine-size real \
number at \!\(om\) = \!\(0.11777335368671815`\)."\)], "Message"],

Cell[BoxData[
    \(Plot::"plnr" \(\(:\)\(\ \)\) 
      "\!\(Imdelta[\(\(1, 1.`, 8\/10, om, 1\)\)]\) is not a machine-size real \
number at \!\(om\) = \!\(0.10848242816171107`\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(Plot :: \"plnr\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImdelta[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(0.021165751356999385`\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imdelta[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(0.021165744731208605200745516300557`20.0808\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rearg10 = 
      Compile[{md2, k, kz, om, a, xi}, 
        1/\((1 + a\ xi^2)\)^2*om*kz]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(om\ kz\)\/\((1 + a\ xi\^2)\)\^2, "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rearg11 = 
      Compile[{md2, k, kz, om, a, xi}, 
        1/\((1 + a\ xi^2)\)^2*\((om + a*kz*xi)\)*
          Sign[om - kz\ xi]*\((k^2\ xi - kz\ om)\)/
            Sqrt[k^2*\((xi - om*kz/k^2)\)^2 - \((k^2 - om^2)\)*\((1 - 
                      kz^2/k^2)\)]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    TagBox[\(CompiledFunction[{md2, k, kz, om, a, 
          xi}, \(\((om + a\ kz\ xi)\)\ Sign[om - kz\ xi]\ \((k\^2\ xi - kz\ \
om)\)\)\/\(\((1 + a\ xi\^2)\)\^2\ \@\(k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \
\((k\^2 - om\^2)\)\ \((1 - kz\^2\/k\^2)\)\)\), "-CompiledCode-"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(FastRedelta[md2_, k_, kz_, om_, a_] := 
      Module[{res, mxi1, mxi2, wp}, 
        If[logging \[Equal] 0, Print["\<delta\>"]]; \[IndentingNewLine]mxi1 = 
          If[k > om, xi1[k, kz, om], 0]; 
        mxi2 = If[k > om, xi2[k, kz, om], 0]; \[IndentingNewLine]If[om < k, 
          res = \(md2/k^4\)/\((1 - kz^2/k^2)\)*
              om/2*\((NIntegrate[
                    Rearg10[md2, k, kz, om, a, xi], {xi, \(-1\), 1}] + 
                  NIntegrate[
                    Rearg11[md2, k, kz, om, a, xi], {xi, \(-1\), 
                      mxi2*If[
                          mxi2 < 
                            0, \((1 + 10^\((\(-13\))\))\), \((1 - 
                              10^\((\(-13\))\))\)]}] + 
                  NIntegrate[
                    Rearg11[md2, k, kz, om, a, xi], {xi, 
                      mxi1*\((1 + 10^\((\(-15\))\))\), 1}, 
                    WorkingPrecision \[Rule] 20])\), 
          res = \(md2/k^4\)/\((1 - kz^2/k^2)\)*
              om/2*\((NIntegrate[
                    Rearg10[md2, k, kz, om, a, xi], {xi, \(-1\), 1}, 
                    WorkingPrecision \[Rule] 20] + 
                  NIntegrate[
                    Rearg11[md2, k, kz, om, a, xi], {xi, \(-1\), 
                      If[om\ kz/k^2 < 1, om\ kz/k^2, 0], 1}, 
                    WorkingPrecision \[Rule] 20])\)]; res]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRedelta[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(0.16168646866450048`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Redelta[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(0.161686454605860461120644070705476`19.5448\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Fdelta[md2_, k_, kz_, om_, a_] := 
      FastRedelta[md2, k, kz, om, a] + 
        I*FastImdelta[md2, k, kz, om, a]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fdelta[1, 1, 8/10, 5/10, 1]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(\(0.16168646866450048`\)\(\[InvisibleSpace]\)\) + 
      0.021165751356999385`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fdelta[1, 2, 2*8/10, 2*5/10, 1]*2\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(\(0.16168646866450048`\)\(\[InvisibleSpace]\)\) + 
      0.021165751356999385`\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["delta kx=0", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\(\((1 - xi^2)\)/\((1 + a\ xi^2)\)^3\)/\((eta - xi + 
              I\ eps)\)\ \((1 - 3\ xi^2\ a\  - 4\ xi^2)\), {xi, \(-1\), 
        1}]\)], "Input"],

Cell[BoxData[
    \(\((\(-2\)\ \[ImaginaryI]\ \@a\ eps - 
          8\ \[ImaginaryI]\ a\^\(3/2\)\ eps - 
          4\ \[ImaginaryI]\ a\^\(3/2\)\ eps\^3 + 
          8\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^3 + 
          6\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^5 - 2\ \@a\ eta - 
          8\ a\^\(3/2\)\ eta - 12\ a\^\(3/2\)\ eps\^2\ eta + 
          24\ a\^\(5/2\)\ eps\^2\ eta + 30\ a\^\(5/2\)\ eps\^4\ eta + 
          12\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^2 - 
          24\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\^2 - 
          60\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^3\ eta\^2 + 
          4\ a\^\(3/2\)\ eta\^3 - 8\ a\^\(5/2\)\ eta\^3 - 
          60\ a\^\(5/2\)\ eps\^2\ eta\^3 + 
          30\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\^4 + 
          6\ a\^\(5/2\)\ eta\^5 + 
          eps\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - 
          3\ a\ eps\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - 
          6\ a\^2\ eps\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - 
          6\ a\ eps\^3\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - 
          12\ a\^2\ eps\^3\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] - 
          2\ a\^3\ eps\^3\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] \
- 3\ a\^2\ eps\^5\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - 
          a\^3\ eps\^5\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] - \
\[ImaginaryI]\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] + 
          3\ \[ImaginaryI]\ a\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) \
+ \@a)\)] + 
          6\ \[ImaginaryI]\ a\^2\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\
\) + \@a)\)] + 
          18\ \[ImaginaryI]\ a\ eps\^2\ eta\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
          36\ \[ImaginaryI]\ a\^2\ eps\^2\ eta\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
          6\ \[ImaginaryI]\ a\^3\ eps\^2\ eta\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
          15\ \[ImaginaryI]\ a\^2\ eps\^4\ eta\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
          5\ \[ImaginaryI]\ a\^3\ eps\^4\ eta\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
          18\ a\ eps\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\
\)] + 36\ a\^2\ eps\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] + 
          6\ a\^3\ eps\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] + 
          30\ a\^2\ eps\^3\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) \
+ \@a)\)] + 
          10\ a\^3\ eps\^3\ eta\^2\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) \
+ \@a)\)] - 
          6\ \[ImaginaryI]\ a\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\
\) + \@a)\)] - 
          12\ \[ImaginaryI]\ a\^2\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - 
          2\ \[ImaginaryI]\ a\^3\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - 
          30\ \[ImaginaryI]\ a\^2\ eps\^2\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - 
          10\ \[ImaginaryI]\ a\^3\ eps\^2\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - 
          15\ a\^2\ eps\ eta\^4\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] - 
          5\ a\^3\ eps\ eta\^4\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] + 
          3\ \[ImaginaryI]\ a\^2\ eta\^5\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\^3\ eta\^5\ Log[\[ImaginaryI]\ \
\((\(-\[ImaginaryI]\) + \@a)\)] - 
          eps\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + 
          3\ a\ eps\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + 
          6\ a\^2\ eps\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + 
          6\ a\ eps\^3\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + 
          12\ a\^2\ eps\^3\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] + 
          2\ a\^3\ eps\^3\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] \
+ 3\ a\^2\ eps\^5\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + 
          a\^3\ eps\^5\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] + \
\[ImaginaryI]\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - 
          3\ \[ImaginaryI]\ a\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] \
+ \@a)\)] - 
          6\ \[ImaginaryI]\ a\^2\ eta\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
          18\ \[ImaginaryI]\ a\ eps\^2\ eta\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
          36\ \[ImaginaryI]\ a\^2\ eps\^2\ eta\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
          6\ \[ImaginaryI]\ a\^3\ eps\^2\ eta\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
          15\ \[ImaginaryI]\ a\^2\ eps\^4\ eta\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
          5\ \[ImaginaryI]\ a\^3\ eps\^4\ eta\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
          18\ a\ eps\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\
\)] - 36\ a\^2\ eps\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] - 
          6\ a\^3\ eps\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] - 
          30\ a\^2\ eps\^3\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] \
+ \@a)\)] - 
          10\ a\^3\ eps\^3\ eta\^2\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] \
+ \@a)\)] + 
          6\ \[ImaginaryI]\ a\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] + 
          12\ \[ImaginaryI]\ a\^2\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] + 
          2\ \[ImaginaryI]\ a\^3\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] + 
          30\ \[ImaginaryI]\ a\^2\ eps\^2\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \
\((\[ImaginaryI] + \@a)\)] + 
          10\ \[ImaginaryI]\ a\^3\ eps\^2\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \
\((\[ImaginaryI] + \@a)\)] + 
          15\ a\^2\ eps\ eta\^4\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] + 
          5\ a\^3\ eps\ eta\^4\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] - 
          3\ \[ImaginaryI]\ a\^2\ eta\^5\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - \[ImaginaryI]\ a\^3\ eta\^5\ Log[\(-\[ImaginaryI]\)\
\ \((\[ImaginaryI] + \@a)\)] - 
          a\^\(3/2\)\ Log[\(\(-1\) - \[ImaginaryI]\ eps + eta\)\/\(\(-1\) + \
eta\)] - 5\ a\^\(3/2\)\ eps\^2\ Log[\(\(-1\) - \[ImaginaryI]\ eps + eta\)\/\(\
\(-1\) + eta\)] - 
          3\ a\^\(5/2\)\ eps\^2\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          4\ a\^\(3/2\)\ eps\^4\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          
          3\ a\^\(5/2\)\ eps\^4\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          10\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          6\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ eta\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^3\ eta\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          5\ a\^\(3/2\)\ eta\^2\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          3\ a\^\(5/2\)\ eta\^2\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          24\ a\^\(3/2\)\ eps\^2\ eta\^2\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          18\ a\^\(5/2\)\ eps\^2\ eta\^2\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^3\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\^3\ Log[\(\(-1\) - \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          4\ a\^\(3/2\)\ eta\^4\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          3\ a\^\(5/2\)\ eta\^4\ Log[\(\(-1\) - \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          a\^\(3/2\)\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] + 
          5\ a\^\(3/2\)\ eps\^2\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + 3\ a\^\(5/2\)\ eps\^2\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + 4\ a\^\(3/2\)\ eps\^4\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + 3\ a\^\(5/2\)\ eps\^4\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] - 10\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\ Log[\(1 - \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] - 
          6\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\ Log[\(1 - \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] - 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ eta\ Log[\(1 - \[ImaginaryI]\
\ eps + eta\)\/\(1 + eta\)] - 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^3\ eta\ Log[\(1 - \[ImaginaryI]\
\ eps + eta\)\/\(1 + eta\)] - 
          5\ a\^\(3/2\)\ eta\^2\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] - 3\ a\^\(5/2\)\ eta\^2\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] - 24\ a\^\(3/2\)\ eps\^2\ eta\^2\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\
\/\(1 + eta\)] - 
          18\ a\^\(5/2\)\ eps\^2\ eta\^2\ Log[\(1 - \[ImaginaryI]\ eps + \
eta\)\/\(1 + eta\)] + 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^3\ Log[\(1 - \[ImaginaryI]\
\ eps + eta\)\/\(1 + eta\)] + 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\^3\ Log[\(1 - \[ImaginaryI]\
\ eps + eta\)\/\(1 + eta\)] + 
          4\ a\^\(3/2\)\ eta\^4\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + 3\ a\^\(5/2\)\ eta\^4\ Log[\(1 - \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + a\^\(3/2\)\ Log[\(\(-1\) + \[ImaginaryI]\ eps + eta\)\/\(\(-1\) + \
eta\)] + 5\ a\^\(3/2\)\ eps\^2\ Log[\(\(-1\) + \[ImaginaryI]\ eps + eta\)\/\(\
\(-1\) + eta\)] + 
          3\ a\^\(5/2\)\ eps\^2\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          4\ a\^\(3/2\)\ eps\^4\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          3\ a\^\(5/2\)\ eps\^4\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          10\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          6\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ eta\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^3\ eta\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] - 
          5\ a\^\(3/2\)\ eta\^2\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          3\ a\^\(5/2\)\ eta\^2\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          24\ a\^\(3/2\)\ eps\^2\ eta\^2\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          18\ a\^\(5/2\)\ eps\^2\ eta\^2\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^3\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\^3\ Log[\(\(-1\) + \
\[ImaginaryI]\ eps + eta\)\/\(\(-1\) + eta\)] + 
          4\ a\^\(3/2\)\ eta\^4\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] + 
          3\ a\^\(5/2\)\ eta\^4\ Log[\(\(-1\) + \[ImaginaryI]\ eps + \
eta\)\/\(\(-1\) + eta\)] - 
          a\^\(3/2\)\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + eta\)] - 
          5\ a\^\(3/2\)\ eps\^2\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] - 3\ a\^\(5/2\)\ eps\^2\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] - 4\ a\^\(3/2\)\ eps\^4\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] - 3\ a\^\(5/2\)\ eps\^4\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + 10\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\ Log[\(1 + \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] + 
          6\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\ Log[\(1 + \[ImaginaryI]\ \
eps + eta\)\/\(1 + eta\)] + 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ eta\ Log[\(1 + \[ImaginaryI]\
\ eps + eta\)\/\(1 + eta\)] + 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^3\ eta\ Log[\(1 + \[ImaginaryI]\
\ eps + eta\)\/\(1 + eta\)] + 
          5\ a\^\(3/2\)\ eta\^2\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + 3\ a\^\(5/2\)\ eta\^2\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + 24\ a\^\(3/2\)\ eps\^2\ eta\^2\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\
\/\(1 + eta\)] + 
          18\ a\^\(5/2\)\ eps\^2\ eta\^2\ Log[\(1 + \[ImaginaryI]\ eps + \
eta\)\/\(1 + eta\)] - 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^3\ Log[\(1 + \[ImaginaryI]\
\ eps + eta\)\/\(1 + eta\)] - 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\^3\ Log[\(1 + \[ImaginaryI]\
\ eps + eta\)\/\(1 + eta\)] - 
          4\ a\^\(3/2\)\ eta\^4\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] - 3\ a\^\(5/2\)\ eta\^4\ Log[\(1 + \[ImaginaryI]\ eps + eta\)\/\(1 + \
eta\)] + a\^\(3/2\)\ Log[1 + eps\^2 - 2\ eta + eta\^2] + 
          5\ a\^\(3/2\)\ eps\^2\ Log[1 + eps\^2 - 2\ eta + eta\^2] + 
          3\ a\^\(5/2\)\ eps\^2\ Log[1 + eps\^2 - 2\ eta + eta\^2] + 
          4\ a\^\(3/2\)\ eps\^4\ Log[1 + eps\^2 - 2\ eta + eta\^2] + 
          3\ a\^\(5/2\)\ eps\^4\ Log[1 + eps\^2 - 2\ eta + eta\^2] - 
          10\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - 
          6\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^3\ eta\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] - 
          5\ a\^\(3/2\)\ eta\^2\ Log[1 + eps\^2 - 2\ eta + eta\^2] - 
          3\ a\^\(5/2\)\ eta\^2\ Log[1 + eps\^2 - 2\ eta + eta\^2] - 
          24\ a\^\(3/2\)\ eps\^2\ eta\^2\ Log[1 + eps\^2 - 2\ eta + eta\^2] - 
          18\ a\^\(5/2\)\ eps\^2\ eta\^2\ Log[1 + eps\^2 - 2\ eta + eta\^2] + 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^3\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] + 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\^3\ Log[
              1 + eps\^2 - 2\ eta + eta\^2] + 
          4\ a\^\(3/2\)\ eta\^4\ Log[1 + eps\^2 - 2\ eta + eta\^2] + 
          3\ a\^\(5/2\)\ eta\^4\ Log[1 + eps\^2 - 2\ eta + eta\^2] - 
          a\^\(3/2\)\ Log[1 + eps\^2 + 2\ eta + eta\^2] - 
          5\ a\^\(3/2\)\ eps\^2\ Log[1 + eps\^2 + 2\ eta + eta\^2] - 
          3\ a\^\(5/2\)\ eps\^2\ Log[1 + eps\^2 + 2\ eta + eta\^2] - 
          4\ a\^\(3/2\)\ eps\^4\ Log[1 + eps\^2 + 2\ eta + eta\^2] - 
          3\ a\^\(5/2\)\ eps\^4\ Log[1 + eps\^2 + 2\ eta + eta\^2] + 
          10\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + 
          6\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\^3\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\^3\ eta\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] + 
          5\ a\^\(3/2\)\ eta\^2\ Log[1 + eps\^2 + 2\ eta + eta\^2] + 
          3\ a\^\(5/2\)\ eta\^2\ Log[1 + eps\^2 + 2\ eta + eta\^2] + 
          24\ a\^\(3/2\)\ eps\^2\ eta\^2\ Log[1 + eps\^2 + 2\ eta + eta\^2] + 
          18\ a\^\(5/2\)\ eps\^2\ eta\^2\ Log[1 + eps\^2 + 2\ eta + eta\^2] - 
          16\ \[ImaginaryI]\ a\^\(3/2\)\ eps\ eta\^3\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] - 
          12\ \[ImaginaryI]\ a\^\(5/2\)\ eps\ eta\^3\ Log[
              1 + eps\^2 + 2\ eta + eta\^2] - 
          4\ a\^\(3/2\)\ eta\^4\ Log[1 + eps\^2 + 2\ eta + eta\^2] - 
          3\ a\^\(5/2\)\ eta\^4\ Log[
              1 + eps\^2 + 2\ eta + 
                eta\^2])\)/\((2\ a\^\(3/2\)\ \((\(-1\) + a\ eps\^2 - 2\ \
\[ImaginaryI]\ a\ eps\ eta - a\ eta\^2)\)\^3)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Limit[%, eps \[Rule] 0]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(2\ a\^\(3/2\)\ \((\(-1\) - a\ eta\^2)\)\^3\)\)\((\(-2\)\ \@a\ \
eta - 8\ a\^\(3/2\)\ eta + 4\ a\^\(3/2\)\ eta\^3 - 8\ a\^\(5/2\)\ eta\^3 + 
        6\ a\^\(5/2\)\ eta\^5 - \[ImaginaryI]\ eta\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] + 
        3\ \[ImaginaryI]\ a\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \
\@a)\)] + 
        6\ \[ImaginaryI]\ a\^2\ eta\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) \
+ \@a)\)] - 
        6\ \[ImaginaryI]\ a\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\[ImaginaryI]\) \
+ \@a)\)] - 
        12\ \[ImaginaryI]\ a\^2\ eta\^3\ Log[\[ImaginaryI]\ \((\(-\
\[ImaginaryI]\) + \@a)\)] - 
        2\ \[ImaginaryI]\ a\^3\ eta\^3\ Log[\[ImaginaryI]\ \
\((\(-\[ImaginaryI]\) + \@a)\)] + 
        3\ \[ImaginaryI]\ a\^2\ eta\^5\ Log[\[ImaginaryI]\ \
\((\(-\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ a\^3\ eta\^5\ Log[\
\[ImaginaryI]\ \((\(-\[ImaginaryI]\) + \@a)\)] + \[ImaginaryI]\ eta\ Log[\(-\
\[ImaginaryI]\)\ \((\[ImaginaryI] + \@a)\)] - 
        3\ \[ImaginaryI]\ a\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] + \
\@a)\)] - 
        6\ \[ImaginaryI]\ a\^2\ eta\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] \
+ \@a)\)] + 
        6\ \[ImaginaryI]\ a\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \((\[ImaginaryI] \
+ \@a)\)] + 
        12\ \[ImaginaryI]\ a\^2\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] + 
        2\ \[ImaginaryI]\ a\^3\ eta\^3\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - 
        3\ \[ImaginaryI]\ a\^2\ eta\^5\ Log[\(-\[ImaginaryI]\)\ \((\
\[ImaginaryI] + \@a)\)] - \[ImaginaryI]\ a\^3\ eta\^5\ Log[\(-\[ImaginaryI]\)\
\ \((\[ImaginaryI] + \@a)\)] + a\^\(3/2\)\ Log[1 - 2\ eta + eta\^2] - 
        5\ a\^\(3/2\)\ eta\^2\ Log[1 - 2\ eta + eta\^2] - 
        3\ a\^\(5/2\)\ eta\^2\ Log[1 - 2\ eta + eta\^2] + 
        4\ a\^\(3/2\)\ eta\^4\ Log[1 - 2\ eta + eta\^2] + 
        3\ a\^\(5/2\)\ eta\^4\ Log[1 - 2\ eta + eta\^2] - 
        a\^\(3/2\)\ Log[1 + 2\ eta + eta\^2] + 
        5\ a\^\(3/2\)\ eta\^2\ Log[1 + 2\ eta + eta\^2] + 
        3\ a\^\(5/2\)\ eta\^2\ Log[1 + 2\ eta + eta\^2] - 
        4\ a\^\(3/2\)\ eta\^4\ Log[1 + 2\ eta + eta\^2] - 
        3\ a\^\(5/2\)\ eta\^4\ Log[1 + 2\ eta + eta\^2])\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltakx0[eta_, a_] = 
      a/4\ \ eta\ \((1 + a)\)^\((1/2)\) FullSimplify[%]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^3\)\)\((\@\(1 + a\)\ eta\ \((\
\[ImaginaryI]\ eta\ \((\(-1\) + 
                  a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                        a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ eta\ \((\(-1\) + 
                  a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                        a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ eta\ \((1 + 
                        a\ eta\^2)\)\ \((\(-1\) - 4\ a + 3\ a\ eta\^2)\) + 
                  a\ \((\(-1\) + 
                        eta\^2)\)\ \((\(-1\) + \((4 + 
                              3\ a)\)\ eta\^2)\)\ \((\(-Log[\((\(-1\) + \
eta)\)\^2]\) + Log[\((1 + eta)\)\^2])\))\))\))\)\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltakx0[0.5, 10]\)], "Input"],

Cell[BoxData[
    \(\(-0.233183350899665`\) + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRedelta[1, 1, 1 - 10^\((\(-5\))\), 1/2, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.5038679736643799`\)."\)], "Message"],

Cell[BoxData[
    \(\(-0.2325051599879824`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImdelta[1, 1, 1 - 10^\((\(-5\))\), 1/2, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(36\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.5038679736637265`\)."\)], "Message"],

Cell[BoxData[
    \(1.7081681885070317`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltakx0korr[eta_, a_] = 
      deltakx0[eta, a] - 
        I\ \ Pi/4\ a\ eta\ \((1 + a)\)^\((1/2)\) \((1 - 
                eta^2)\)/\((1 + a\ eta^2)\)^3\ \((1 - 3\ a\ eta^2 - 
              4\ eta^2)\)\ UnitStep[1 - eta]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^3\)\)\((\@\(1 + a\)\ eta\ \((\
\[ImaginaryI]\ eta\ \((\(-1\) + 
                    a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                          a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                  1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ eta\ \((\(-1\) + 
                    a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                          a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                  1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ eta\ \((1 + 
                          a\ eta\^2)\)\ \((\(-1\) - 4\ a + 3\ a\ eta\^2)\) + 
                    a\ \((\(-1\) + 
                          eta\^2)\)\ \((\(-1\) + \((4 + 
                                3\ a)\)\ eta\^2)\)\ \((\(-Log[\((\(-1\) + \
eta)\)\^2]\) + 
                          Log[\((1 + eta)\)\^2])\))\))\))\)\) - \(\
\[ImaginaryI]\ a\ \@\(1 + a\)\ eta\ \((1 - eta\^2)\)\ \((1 - 4\ eta\^2 - 3\ a\
\ eta\^2)\)\ \[Pi]\ UnitStep[1 - eta]\)\/\(4\ \((1 + a\ eta\^2)\)\^3\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltakx0korr[eta_, 
        a_] = \(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^3\)\) \((\@\(1 + a\)\ eta\ \
\((\[ImaginaryI]\ eta\ \((\(-1\) + 
                        a\ \((3 + 6\ a - 
                              2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                              a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                      1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ eta\ \((\(-1\) \
+ a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                              a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                      1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ eta\ \((1 + 
                              a\ eta\^2)\)\ \((\(-1\) - 4\ a + 
                              3\ a\ eta\^2)\) + 
                        a\ \((\(-1\) + 
                              eta\^2)\)\ \((\(-1\) + \((4 + 
                                    3\ a)\)\ eta\^2)\)\ \((\(-Log[\((\(-1\) + \
eta)\)\^2]\) + 
                              Log[\((1 + eta)\)\^2])\))\))\))\) - \(\
\[ImaginaryI]\ a\ \@\(1 + a\)\ eta\ \((1 - eta\^2)\)\ \((1 - 4\ eta\^2 - 3\ a\
\ eta\^2)\)\ \[Pi]\ UnitStep[1 - Re[eta]]\)\/\(4\ \((1 + a\ \
eta\^2)\)\^3\)\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^3\)\)\((\@\(1 + a\)\ eta\ \((\
\[ImaginaryI]\ eta\ \((\(-1\) + 
                    a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                          a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                  1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ eta\ \((\(-1\) + 
                    a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                          a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                  1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ eta\ \((1 + 
                          a\ eta\^2)\)\ \((\(-1\) - 4\ a + 3\ a\ eta\^2)\) + 
                    a\ \((\(-1\) + 
                          eta\^2)\)\ \((\(-1\) + \((4 + 
                                3\ a)\)\ eta\^2)\)\ \((\(-Log[\((\(-1\) + \
eta)\)\^2]\) + 
                          Log[\((1 + eta)\)\^2])\))\))\))\)\) - \(\
\[ImaginaryI]\ a\ \@\(1 + a\)\ eta\ \((1 - eta\^2)\)\ \((1 - 4\ eta\^2 - 3\ a\
\ eta\^2)\)\ \[Pi]\ UnitStep[1 - Re[eta]]\)\/\(4\ \((1 + a\ \
eta\^2)\)\^3\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltakx0korr[0.5, 10]\)], "Input"],

Cell[BoxData[
    \(\(-0.233183350899665`\) + 1.708734633470277`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(yop!\)\)], "Input"],

Cell[BoxData[
    \(modifying\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltakx0korr[eta_, 
        a_] = \(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^3\)\) \((\@\(1 + a\)\ eta\ \
\((\[ImaginaryI]\ eta\ \((\(-1\) + 
                      a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                            a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                    1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ eta\ \((\(-1\) + 
                      a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                            a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                    1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ eta\ \((1 + 
                            a\ eta\^2)\)\ \((\(-1\) - 4\ a + 
                            3\ a\ eta\^2)\) + 
                      a\ \((\(-1\) + 
                            eta\^2)\)\ \((\(-1\) + \((4 + 
                                  3\ a)\)\ eta\^2)\)\ 2\ \((\(-Log[\((\(-1\) \
+ eta)\)]\) + Log[\((1 + eta)\)])\))\))\))\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^3\)\)\((\@\(1 + a\)\ eta\ \((\
\[ImaginaryI]\ eta\ \((\(-1\) + 
                  a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                        a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ eta\ \((\(-1\) + 
                  a\ \((3 + 6\ a - 2\ \((3 + a\ \((6 + a)\))\)\ eta\^2 + 
                        a\ \((3 + a)\)\ eta\^4)\))\)\ Log[
                1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ eta\ \((1 + 
                        a\ eta\^2)\)\ \((\(-1\) - 4\ a + 3\ a\ eta\^2)\) + 
                  2\ a\ \((\(-1\) + 
                        eta\^2)\)\ \((\(-1\) + \((4 + 
                              3\ a)\)\ eta\^2)\)\ \((\(-Log[\(-1\) + eta]\) + 
                        Log[1 + eta])\))\))\))\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltakx0korr[0.5, 10]\)], "Input"],

Cell[BoxData[
    \(\(-0.23318335089966505`\) + 
      1.7087346334702769`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NIntegrate[\(\(Sqrt[1 + a]/
                4 \((1 - xi^2)\)/\((1 + a\ xi^2)\)^2\ \((eta + 
                    a\ xi)\)/\((eta - xi + I\ eps)\) /. 
            eps \[Rule] 10^\((\(-10\))\)\) /. a \[Rule] 10\) /. 
        eta \[Rule] 0.4, {xi, \(-1\), \(-0.5\), 0.1, 0.399, 0.4, 0.401, 
        1}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.40000000011312065`\)."\
\)], "Message"],

Cell[BoxData[
    \(\(\(1.051268136492134`\)\(\[InvisibleSpace]\)\) - 
      1.4242016709780079`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRedelta[1, 1, 1 - 10^\((\(-4\))\), 3/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3134604002914663`\)."\)], "Message"],

Cell[BoxData[
    \(1.8799830354672142`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRegamma[1, 1, 1 - 10^\((\(-6\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(-0.000032104998685727394`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphakx0[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.0512681376293807`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImalpha[1, 1, 1 - 10^\((\(-4\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(32\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(32\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(32\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(-1.4246952005650197`\) - 
      2.3457600923356023`*^-43\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImgamma[1, 1, 1 - 10^\((\(-5\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3958972299406299`\)."\)], "Message"],

Cell[BoxData[
    \(\(-0.00007351515545636133`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NIntegrate[\(\(eta^2\ \((1 + a)\)^\((3/2)\)/
                2 \( xi/\((1 + a\ xi^2)\)^2\)\ /\((eta - xi + I\ eps)\) /. 
            eps \[Rule] 10^\((\(-10\))\)\) /. a \[Rule] 10\) /. 
        eta \[Rule] 0.4, {xi, \(-1\), 0, 0.4, 1}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3999999992222327`\)."\)], "Message"],

Cell[BoxData[
    \(\(\(0.14786908419641076`\)\(\[InvisibleSpace]\)\) - 
      0.5425509355174426`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0[0.4, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.14786621025190982`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(aha!\)\)], "Input"],

Cell[BoxData[
    \(also\ numerische\ Integration\ sollte\ schon\ \(\(stimmen\)\(.\)\)\)], \
"Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[alphakx0korr[eta, 10.0]], {eta, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.402135 0.178288 [
[.15079 .38963 -9 -9 ]
[.15079 .38963 9 0 ]
[.27778 .38963 -9 -9 ]
[.27778 .38963 9 0 ]
[.40476 .38963 -9 -9 ]
[.40476 .38963 9 0 ]
[.53175 .38963 -9 -9 ]
[.53175 .38963 9 0 ]
[.65873 .38963 -3 -9 ]
[.65873 .38963 3 0 ]
[.78571 .38963 -9 -9 ]
[.78571 .38963 9 0 ]
[.9127 .38963 -9 -9 ]
[.9127 .38963 9 0 ]
[.01131 .04556 -12 -4.5 ]
[.01131 .04556 0 4.5 ]
[.01131 .1347 -24 -4.5 ]
[.01131 .1347 0 4.5 ]
[.01131 .22385 -12 -4.5 ]
[.01131 .22385 0 4.5 ]
[.01131 .31299 -24 -4.5 ]
[.01131 .31299 0 4.5 ]
[.01131 .49128 -18 -4.5 ]
[.01131 .49128 0 4.5 ]
[.01131 .58042 -6 -4.5 ]
[.01131 .58042 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .40213 m
.15079 .40838 L
s
[(0.2)] .15079 .38963 0 1 Mshowa
.27778 .40213 m
.27778 .40838 L
s
[(0.4)] .27778 .38963 0 1 Mshowa
.40476 .40213 m
.40476 .40838 L
s
[(0.6)] .40476 .38963 0 1 Mshowa
.53175 .40213 m
.53175 .40838 L
s
[(0.8)] .53175 .38963 0 1 Mshowa
.65873 .40213 m
.65873 .40838 L
s
[(1)] .65873 .38963 0 1 Mshowa
.78571 .40213 m
.78571 .40838 L
s
[(1.2)] .78571 .38963 0 1 Mshowa
.9127 .40213 m
.9127 .40838 L
s
[(1.4)] .9127 .38963 0 1 Mshowa
.125 Mabswid
.05556 .40213 m
.05556 .40588 L
s
.0873 .40213 m
.0873 .40588 L
s
.11905 .40213 m
.11905 .40588 L
s
.18254 .40213 m
.18254 .40588 L
s
.21429 .40213 m
.21429 .40588 L
s
.24603 .40213 m
.24603 .40588 L
s
.30952 .40213 m
.30952 .40588 L
s
.34127 .40213 m
.34127 .40588 L
s
.37302 .40213 m
.37302 .40588 L
s
.43651 .40213 m
.43651 .40588 L
s
.46825 .40213 m
.46825 .40588 L
s
.5 .40213 m
.5 .40588 L
s
.56349 .40213 m
.56349 .40588 L
s
.59524 .40213 m
.59524 .40588 L
s
.62698 .40213 m
.62698 .40588 L
s
.69048 .40213 m
.69048 .40588 L
s
.72222 .40213 m
.72222 .40588 L
s
.75397 .40213 m
.75397 .40588 L
s
.81746 .40213 m
.81746 .40588 L
s
.84921 .40213 m
.84921 .40588 L
s
.88095 .40213 m
.88095 .40588 L
s
.94444 .40213 m
.94444 .40588 L
s
.97619 .40213 m
.97619 .40588 L
s
.25 Mabswid
0 .40213 m
1 .40213 L
s
.02381 .04556 m
.03006 .04556 L
s
[(-2)] .01131 .04556 1 0 Mshowa
.02381 .1347 m
.03006 .1347 L
s
[(-1.5)] .01131 .1347 1 0 Mshowa
.02381 .22385 m
.03006 .22385 L
s
[(-1)] .01131 .22385 1 0 Mshowa
.02381 .31299 m
.03006 .31299 L
s
[(-0.5)] .01131 .31299 1 0 Mshowa
.02381 .49128 m
.03006 .49128 L
s
[(0.5)] .01131 .49128 1 0 Mshowa
.02381 .58042 m
.03006 .58042 L
s
[(1)] .01131 .58042 1 0 Mshowa
.125 Mabswid
.02381 .06339 m
.02756 .06339 L
s
.02381 .08122 m
.02756 .08122 L
s
.02381 .09904 m
.02756 .09904 L
s
.02381 .11687 m
.02756 .11687 L
s
.02381 .15253 m
.02756 .15253 L
s
.02381 .17036 m
.02756 .17036 L
s
.02381 .18819 m
.02756 .18819 L
s
.02381 .20602 m
.02756 .20602 L
s
.02381 .24168 m
.02756 .24168 L
s
.02381 .2595 m
.02756 .2595 L
s
.02381 .27733 m
.02756 .27733 L
s
.02381 .29516 m
.02756 .29516 L
s
.02381 .33082 m
.02756 .33082 L
s
.02381 .34865 m
.02756 .34865 L
s
.02381 .36648 m
.02756 .36648 L
s
.02381 .38431 m
.02756 .38431 L
s
.02381 .41996 m
.02756 .41996 L
s
.02381 .43779 m
.02756 .43779 L
s
.02381 .45562 m
.02756 .45562 L
s
.02381 .47345 m
.02756 .47345 L
s
.02381 .50911 m
.02756 .50911 L
s
.02381 .52694 m
.02756 .52694 L
s
.02381 .54477 m
.02756 .54477 L
s
.02381 .56259 m
.02756 .56259 L
s
.02381 .02773 m
.02756 .02773 L
s
.02381 .0099 m
.02756 .0099 L
s
.02381 .59825 m
.02756 .59825 L
s
.02381 .61608 m
.02756 .61608 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.09769 0 m
.11039 .07114 L
.14855 .28165 L
.16787 .3678 L
.18914 .44367 L
.20932 .49823 L
.22823 .53598 L
.23854 .55193 L
.24834 .56447 L
.26674 .5821 L
.27627 .58866 L
.28642 .59405 L
.296 .59782 L
.30468 .60028 L
.30952 .6013 L
.31464 .60213 L
.31901 .60265 L
.32149 .60288 L
.3238 .60305 L
.32509 .60312 L
.32648 .60319 L
.3278 .60324 L
.32902 .60327 L
.33012 .6033 L
.33133 .60331 L
.33259 .60332 L
.33378 .60331 L
.33509 .6033 L
.33575 .60329 L
.33647 .60327 L
.33774 .60324 L
.33893 .60319 L
.34162 .60307 L
.34448 .60289 L
.34978 .60245 L
.35466 .60192 L
.36569 .60035 L
.3857 .59645 L
.42484 .58639 L
.46248 .57539 L
.50255 .56348 L
.54111 .5524 L
.58212 .54122 L
.62161 .53088 L
.6596 .52 L
.68082 .51446 L
.70003 .51076 L
.73895 .50495 L
.7788 .50042 L
.81715 .49695 L
Mistroke
.85793 .49393 L
.8972 .49151 L
.93497 .48954 L
.97518 .48774 L
.97619 .48769 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool04?oo00<00?ooool0
kooo000Kool00`00ooooo`0@ool00`00ooooo`3_ool001_oo`80017oo`03003ooooo0>ooo`006ooo
00<00?ooool04Ooo00<00?ooool0k_oo000Kool00`00ooooo`0Aool00`00ooooo`3^ool001_oo`03
003ooooo017oo`03003ooooo0>koo`006ooo00<00?ooool04Ooo00<00?ooool0k_oo000Kool2000B
ool00`00ooooo`3^ool001_oo`03003ooooo01;oo`03003ooooo0>goo`006ooo00<00?ooool04_oo
00<00?ooool0kOoo000Cool40004ool00`00ooooo`0Bool00`00ooooo`3]ool001?oo`03003ooooo
00Goo`03003ooooo01;oo`03003ooooo0>goo`005?oo00<00?ooool01?oo0P005?oo00<00?ooool0
k?oo000=ool40004ool00`00ooooo`03ool00`00ooooo`0Cool00`00ooooo`3/ool001Koo`03003o
oooo00;oo`03003ooooo01?oo`03003ooooo0>coo`004ooo00@00?ooool000Coo`03003ooooo01?o
o`03003ooooo0>coo`005?oo0P001Ooo00<00?ooool04ooo00<00?ooool0k?oo000Kool2000Eool0
0`00ooooo`3[ool001_oo`03003ooooo01Coo`03003ooooo0>_oo`006ooo00<00?ooool05?oo00<0
0?ooool0jooo000Kool00`00ooooo`0Dool00`00ooooo`3[ool001_oo`8001Goo`03003ooooo0>_o
o`006ooo00<00?ooool05Ooo00<00?ooool0j_oo000Kool00`00ooooo`0Eool00`00ooooo`3Zool0
01_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo00<00?ooool05Ooo00<00?ooool0j_oo000K
ool2000Fool00`00ooooo`3Zool001_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo00<00?oo
ool05_oo00<00?ooool0jOoo000Kool00`00ooooo`0Fool00`00ooooo`3Yool001_oo`03003ooooo
01Koo`03003ooooo0>Woo`006ooo0P005ooo00<00?ooool0jOoo000Kool00`00ooooo`0Fool00`00
ooooo`3Yool001_oo`03003ooooo01Koo`03003ooooo0>Woo`001ooo10000ooo00<00?ooool00_oo
0`001Ooo00<00?ooool05ooo00<00?ooool0j?oo0009ool00`00ooooo`0:ool00`00ooooo`02ool0
0`00ooooo`0Gool00`00ooooo`3Xool000Woo`03003ooooo00[oo`03003ooooo00;oo`8001Soo`03
003ooooo0>Soo`0000?oo`0000000P001?oo00<00?ooool01ooo0`001Ooo00<00?ooool05ooo00<0
0?ooool0j?oo0009ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`0Gool00`00ooooo`3X
ool000Ooo`<000Woo`03003ooooo00Goo`03003ooooo01Soo`03003ooooo0>Ooo`002Ooo00<00?oo
ool01ooo10001?oo0P006Ooo00<00?ooool0iooo000Kool00`00ooooo`0Hool00`00ooooo`3Wool0
01_oo`03003ooooo01Soo`03003ooooo0>Ooo`006ooo00<00?ooool06?oo00<00?ooool0iooo000K
ool00`00ooooo`0Hool00`00ooooo`3Wool001_oo`8001[oo`03003ooooo0>Koo`006ooo00<00?oo
ool06Ooo00<00?ooool0i_oo000Kool00`00ooooo`0Iool00`00ooooo`3Vool001_oo`03003ooooo
01Woo`03003ooooo0>Koo`006ooo00<00?ooool06Ooo00<00?ooool0i_oo000Kool2000Kool00`00
ooooo`3Uool001_oo`03003ooooo01[oo`03003ooooo0>Goo`006ooo00<00?ooool06_oo00<00?oo
ool0iOoo000Kool00`00ooooo`0Jool00`00ooooo`3Uool001_oo`03003ooooo01[oo`03003ooooo
0>Goo`006ooo0P006ooo00<00?ooool0iOoo000Kool00`00ooooo`0Kool00`00ooooo`3Tool001?o
o`@000Coo`03003ooooo01_oo`03003ooooo0>Coo`005Ooo00<00?ooool00ooo00<00?ooool06ooo
00<00?ooool0i?oo000Eool00`00ooooo`03ool2000Lool00`00ooooo`3Tool000goo`@000Coo`03
003ooooo00?oo`03003ooooo01_oo`03003ooooo0>Coo`005Ooo00<00?ooool00ooo00<00?ooool0
6ooo00<00?ooool0i?oo000Cool30005ool00`00ooooo`0Lool00`00ooooo`3Sool001Goo`03003o
oooo00?oo`03003ooooo01coo`03003ooooo0>?oo`006ooo0P007Ooo00<00?ooool0hooo000Kool0
0`00ooooo`0Lool00`00ooooo`3Sool001_oo`03003ooooo01coo`03003ooooo0>?oo`006ooo00<0
0?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Mool00`00ooooo`3Rool001_oo`8001ko
o`03003ooooo0>;oo`006ooo00<00?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Mool0
0`00ooooo`3Rool001_oo`03003ooooo01goo`03003ooooo0>;oo`006ooo00<00?ooool07_oo00<0
0?ooool0hOoo000Kool2000Oool00`00ooooo`3Qool001_oo`03003ooooo01koo`03003ooooo0>7o
o`006ooo00<00?ooool07_oo00<00?ooool0hOoo000Kool00`00ooooo`0Nool00`00ooooo`3Qool0
01_oo`80023oo`03003ooooo0>3oo`006ooo00<00?ooool07ooo00<00?ooool0h?oo000Kool00`00
ooooo`0Oool00`00ooooo`3Pool000Soo`8000Coo`03003ooooo00;oo`<000Goo`03003ooooo01oo
o`03003ooooo0>3oo`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo023oo`03003o
oooo0=ooo`001ooo00@00?ooool000_oo`03003ooooo00;oo`80027oo`03003ooooo0=ooo`0000?o
o`0000000P000_oo00@00?ooool000Soo`<000Goo`03003ooooo023oo`03003ooooo0=ooo`001ooo
00@00?ooool000Soo`03003ooooo00Goo`03003ooooo023oo`03003ooooo0=ooo`001ooo00@00?oo
ool000Soo`03003ooooo00Goo`03003ooooo023oo`03003ooooo0=ooo`002?oo0P002Ooo10001?oo
00<00?ooool08Ooo00<00?ooool0g_oo000Kool2000Rool00`00ooooo`3Nool001_oo`03003ooooo
027oo`03003ooooo0=koo`006ooo00<00?ooool08Ooo00<00?ooool0g_oo000Kool00`00ooooo`0Q
ool00`00ooooo`3Nool001_oo`03003ooooo02;oo`03003ooooo0=goo`006ooo0P008ooo00<00?oo
ool0gOoo000Kool00`00ooooo`0Rool00`00ooooo`3Mool001_oo`03003ooooo02;oo`03003ooooo
0=goo`006ooo00<00?ooool06?oo0P001?oo00<00?ooool00_oo10004_oo0P001?oo00<00?ooool0
1?oo00<00?ooool04_oo0P001?oo00<00?ooool00ooo0P005?oo0P001?oo00<00?ooool00ooo0P00
6Ooo10005ooo10000ooo00<00?ooool00_oo10004_oo10000ooo00<00?ooool01?oo00<00?ooool0
4?oo000Kool2000Hool01000ooooo`002?oo00<00?ooool04_oo00@00?ooool000[oo`03003ooooo
017oo`04003ooooo0008ool01000ooooo`004_oo00@00?ooool000Soo`04003ooooo000Jool00`00
ooooo`0Hool00`00ooooo`07ool00`00ooooo`0Eool00`00ooooo`09ool00`00ooooo`0@ool001_o
o`03003ooooo01Ooo`04003ooooo0008ool2000Cool01000ooooo`001ooo1@004_oo00@00?ooool0
00Soo`04003ooooo000Bool01000ooooo`002?oo00@00?ooool001[oo`03003ooooo01Soo`03003o
oooo00Soo`03003ooooo01Coo`03003ooooo00Koo`D0017oo`006ooo00<00?ooool05ooo00@00?oo
ool000Soo`03003oo`0001;oo`04003ooooo0007ool01000ooooo`004ooo00@00?ooool000Soo`<0
01?oo`04003ooooo0009ool2000Kool00`00ooooo`0Hool00`00ooooo`09ool00`00ooooo`0Cool0
0`00ooooo`06ool01000ooooo`004_oo000Kool00`00ooooo`0Gool01000ooooo`002Ooo00<00?oo
00004Ooo00@00?ooool000Soo`03003oo`0001?oo`04003ooooo0008ool00`00ooooo`0Cool01000
ooooo`002?oo00@00?ooool001[oo`03003ooooo01Soo`03003ooooo00[oo`03003ooooo01;oo`03
003ooooo00Ooo`03003oo`0001;oo`006ooo00<00?ooool05ooo00@00?ooool000Soo`800003ool0
0?oo013oo`04003ooooo0009ool2000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool0
00Soo`04003ooooo000Hool3000Hool30009ool01000ooooo`004_oo0`002_oo0P004_oo000Kool2
000Iool2000:ool2000Cool2000;ool00`00ooooo`0Bool2000:ool3000Cool2000:ool2000Kool0
0`00ooooo`0Hool00`00ooooo`08ool2000Eool00`00ooooo`09ool00`00ooooo`0@ool001_oo`03
003ooooo02Coo`03003ooooo0=_oo`006ooo00<00?ooool09Ooo00<00?ooool0f_oo000Kool00`00
ooooo`0Uool00`00ooooo`3Jool001_oo`03003ooooo02Goo`03003ooooo0=[oo`005?ooo`003@00
000Kool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`04ool20007ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`05ool001_oo`03003ooooo02Koo`03003ooooo0=Woo`006ooo00<00?ooool09ooo00<0
0?ooool0f?oo000Kool00`00ooooo`0Wool00`00ooooo`3Hool001_oo`8002Soo`03003ooooo0=So
o`006ooo00<00?ooool09ooo00<00?ooool0f?oo000Kool00`00ooooo`0Xool00`00ooooo`3Gool0
01_oo`03003ooooo02Soo`03003ooooo0=Ooo`006ooo0P00:Ooo00<00?ooool0eooo000Kool00`00
ooooo`0Yool00`00ooooo`3Fool001_oo`03003ooooo02Woo`03003ooooo0=Koo`006ooo00<00?oo
ool0:Ooo00<00?ooool0e_oo000Kool00`00ooooo`0Zool00`00ooooo`3Eool001_oo`8002_oo`03
003ooooo0=Goo`006ooo00<00?ooool0:_oo00<00?ooool0eOoo000Kool00`00ooooo`0[ool00`00
ooooo`3Dool001_oo`03003ooooo02_oo`03003ooooo0=Coo`006ooo00<00?ooool0:ooo00<00?oo
ool0e?oo000Kool2000]ool00`00ooooo`3Cool001_oo`03003ooooo02coo`03003ooooo0=?oo`00
6ooo00<00?ooool0;Ooo00<00?ooool0d_oo0008ool20004ool00`00ooooo`02ool30005ool00`00
ooooo`0]ool00`00ooooo`3Bool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0]
ool00`00ooooo`2joolA0007ool000Ooo`04003ooooo000;ool00`00ooooo`02ool2000_ool00`00
ooooo`2UoolD000Hool000Ooo`04003ooooo0008ool30005ool00`00ooooo`0^ool00`00ooooo`2J
ool;000/ool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0^ool00`00ooooo`2@
ool:000gool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0_ool00`00ooooo`27
ool80011ool000Soo`8000Woo`@000Coo`80033oo`03003ooooo08;oo`D004Woo`006ooo00<00?oo
ool0<?oo00<00?ooool0O?oo1@00C_oo000Kool00`00ooooo`0`ool00`00ooooo`1fool6001Cool0
01_oo`03003ooooo037oo`03003ooooo077oo`@005Woo`006ooo00<00?ooool0<Ooo00<00?ooool0
KOoo1000GOoo000Kool2000cool00`00ooooo`1Yool3001Qool001_oo`03003ooooo03;oo`03003o
oooo06Koo`<006Coo`006ooo00<00?ooool0<ooo00<00?ooool0HOoo1000Iooo000Kool00`00oooo
o`0dool00`00ooooo`1Lool4001[ool001_oo`03003ooooo03Coo`03003ooooo05Woo`<006ooo`00
6ooo0P00=_oo00<00?ooool0E?oo1000L_oo000Kool00`00ooooo`0eool00`00ooooo`1@ool4001f
ool001_oo`03003ooooo03Koo`03003ooooo04coo`<007[oo`006ooo00<00?ooool0=ooo00<00?oo
ool0Aooo1000OOoo000Kool2000iool00`00ooooo`13ool30021ool001_oo`03003ooooo03Woo`03
003ooooo03ooo`<008Coo`006ooo00<00?ooool0>_oo00<00?ooool0>_oo1000Qooo000Cool40004
ool00`00ooooo`0kool00`00ooooo`0fool3002;ool001Goo`03003ooooo00?oo`03003ooooo03_o
o`03003ooooo03?oo`<008koo`005Ooo00<00?ooool00ooo0P00?Ooo00<00?ooool0;_oo1000TOoo
000Eool00`00ooooo`03ool00`00ooooo`0mool2000[ool3002Eool001Goo`03003ooooo00?oo`03
003ooooo03ooo`8002Goo`@009Soo`004ooo0`001Ooo00<00?ooool0@Ooo0P007Ooo1P00W?oo000E
ool00`00ooooo`03ool00`00ooooo`13ool00`00ooooo`0Fool4002Rool001_oo`8004Goo`<001?o
o`800:Koo`006ooo00<00?ooool0Aooo4`00Z?oo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, -2.38772, \
0.00592092, 0.0210856}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Im[alphakx0korr[eta, 10.0]], {eta, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.603319 0.206619 [
[.15079 .59082 -9 -9 ]
[.15079 .59082 9 0 ]
[.27778 .59082 -9 -9 ]
[.27778 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.53175 .59082 -9 -9 ]
[.53175 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.9127 .59082 -9 -9 ]
[.9127 .59082 9 0 ]
[.01131 .08677 -24 -4.5 ]
[.01131 .08677 0 4.5 ]
[.01131 .19008 -12 -4.5 ]
[.01131 .19008 0 4.5 ]
[.01131 .29339 -24 -4.5 ]
[.01131 .29339 0 4.5 ]
[.01131 .3967 -12 -4.5 ]
[.01131 .3967 0 4.5 ]
[.01131 .50001 -24 -4.5 ]
[.01131 .50001 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .60332 m
.15079 .60957 L
s
[(0.2)] .15079 .59082 0 1 Mshowa
.27778 .60332 m
.27778 .60957 L
s
[(0.4)] .27778 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(0.6)] .40476 .59082 0 1 Mshowa
.53175 .60332 m
.53175 .60957 L
s
[(0.8)] .53175 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(1)] .65873 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(1.2)] .78571 .59082 0 1 Mshowa
.9127 .60332 m
.9127 .60957 L
s
[(1.4)] .9127 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.18254 .60332 m
.18254 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.34127 .60332 m
.34127 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.81746 .60332 m
.81746 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.97619 .60332 m
.97619 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .08677 m
.03006 .08677 L
s
[(-2.5)] .01131 .08677 1 0 Mshowa
.02381 .19008 m
.03006 .19008 L
s
[(-2)] .01131 .19008 1 0 Mshowa
.02381 .29339 m
.03006 .29339 L
s
[(-1.5)] .01131 .29339 1 0 Mshowa
.02381 .3967 m
.03006 .3967 L
s
[(-1)] .01131 .3967 1 0 Mshowa
.02381 .50001 m
.03006 .50001 L
s
[(-0.5)] .01131 .50001 1 0 Mshowa
.125 Mabswid
.02381 .10743 m
.02756 .10743 L
s
.02381 .1281 m
.02756 .1281 L
s
.02381 .14876 m
.02756 .14876 L
s
.02381 .16942 m
.02756 .16942 L
s
.02381 .21074 m
.02756 .21074 L
s
.02381 .23141 m
.02756 .23141 L
s
.02381 .25207 m
.02756 .25207 L
s
.02381 .27273 m
.02756 .27273 L
s
.02381 .31405 m
.02756 .31405 L
s
.02381 .33471 m
.02756 .33471 L
s
.02381 .35538 m
.02756 .35538 L
s
.02381 .37604 m
.02756 .37604 L
s
.02381 .41736 m
.02756 .41736 L
s
.02381 .43802 m
.02756 .43802 L
s
.02381 .45869 m
.02756 .45869 L
s
.02381 .47935 m
.02756 .47935 L
s
.02381 .52067 m
.02756 .52067 L
s
.02381 .54133 m
.02756 .54133 L
s
.02381 .562 m
.02756 .562 L
s
.02381 .58266 m
.02756 .58266 L
s
.02381 .06611 m
.02756 .06611 L
s
.02381 .04545 m
.02756 .04545 L
s
.02381 .02479 m
.02756 .02479 L
s
.02381 .00412 m
.02756 .00412 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03016 .54424 m
.04884 .37735 L
.06854 .22664 L
.07819 .16643 L
.08851 .11332 L
.09444 .08823 L
.09984 .06882 L
.10496 .05335 L
.11039 .03999 L
.1151 .03085 L
.11759 .02688 L
.12028 .02327 L
.12274 .02055 L
.12499 .01854 L
.12726 .01696 L
.12941 .01585 L
.13071 .01537 L
.13189 .01504 L
.13254 .01491 L
.13325 .01481 L
.13453 .01472 L
.13573 .01474 L
.137 .01488 L
.13809 .0151 L
.13928 .01543 L
.14069 .01594 L
.14197 .01653 L
.14485 .01824 L
.15005 .02262 L
.15483 .02797 L
.1599 .0349 L
.16901 .05014 L
.18963 .09412 L
.22713 .18859 L
.26707 .28556 L
.3055 .36389 L
.34636 .42929 L
.38572 .47714 L
.42358 .51203 L
.46387 .54002 L
.50265 .56027 L
.54387 .57647 L
.58359 .58822 L
.6218 .59685 L
.64154 .60051 L
.65162 .6022 L
.65452 .60266 L
.65595 .60289 L
.65728 .60309 L
.65847 .60328 L
Mistroke
.65975 .60332 L
.66048 .60332 L
.66116 .60332 L
.66245 .60332 L
.68189 .60332 L
.70008 .60332 L
.73843 .60332 L
.77921 .60332 L
.81848 .60332 L
.85625 .60332 L
.89646 .60332 L
.93516 .60332 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool2
003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`0Hool5003Uool001_oo`03003ooooo01Ooo`8000Coo`03003ooooo0>;oo`006ooo0P005ooo00<0
0?ooool01Ooo00<00?ooool0hOoo000Kool00`00ooooo`0Fool00`00ooooo`06ool2003Qool001_o
o`03003ooooo01Goo`03003ooooo00Soo`03003ooooo0=ooo`006ooo00<00?ooool05Ooo00<00?oo
ool02Ooo00<00?ooool0g_oo000Kool00`00ooooo`0Dool00`00ooooo`0:ool00`00ooooo`3Nool0
01_oo`03003ooooo01Coo`03003ooooo00_oo`03003ooooo0=goo`006ooo0P005?oo00<00?ooool0
3?oo00<00?ooool0gOoo000Kool00`00ooooo`0Cool00`00ooooo`0=ool00`00ooooo`3Lool001_o
o`03003ooooo01;oo`03003ooooo00koo`03003ooooo0=coo`006ooo00<00?ooool04_oo00<00?oo
ool03ooo00<00?ooool0fooo000Kool00`00ooooo`0Bool00`00ooooo`0?ool00`00ooooo`3Kool0
01_oo`8001;oo`03003ooooo017oo`03003ooooo0=[oo`006ooo00<00?ooool04Ooo00<00?ooool0
4Ooo00<00?ooool0f_oo000Kool00`00ooooo`0Aool00`00ooooo`0Bool00`00ooooo`3Iool001_o
o`03003ooooo017oo`03003ooooo01;oo`03003ooooo0=Woo`001ooo10000ooo00<00?ooool00_oo
0`001Ooo00<00?ooool04?oo00<00?ooool05?oo00<00?ooool0f?oo0007ool00`00ooooo`0<ool0
0`00ooooo`02ool00`00ooooo`0@ool00`00ooooo`0Dool00`00ooooo`3Hool000Soo`03003ooooo
00_oo`03003ooooo00;oo`80017oo`03003ooooo01Goo`03003ooooo0=Ooo`0000?oo`0000000P00
1?oo00<00?ooool01ooo0`001Ooo00<00?ooool04?oo00<00?ooool05Ooo00<00?ooool0eooo000:
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`0?ool00`00ooooo`0Gool00`00ooooo`3F
ool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0?ool00`00ooooo`0Gool00`00
ooooo`3Fool000Soo`8000Woo`@000Coo`03003ooooo00ooo`03003ooooo01Soo`03003ooooo0=Go
o`006ooo0P004?oo00<00?ooool06?oo00<00?ooool0eOoo000Kool00`00ooooo`0>ool00`00oooo
o`0Jool00`00ooooo`3Dool001_oo`03003ooooo00koo`03003ooooo01[oo`03003ooooo0=Coo`00
6ooo00<00?ooool03_oo00<00?ooool06_oo00<00?ooool0e?oo000Kool00`00ooooo`0>ool00`00
ooooo`0Kool00`00ooooo`3Cool001_oo`03003ooooo00goo`03003ooooo01coo`03003ooooo0=?o
o`006ooo0P003_oo00<00?ooool07Ooo00<00?ooool0d_oo000Kool00`00ooooo`0=ool00`00oooo
o`0Mool00`00ooooo`3Bool001_oo`03003ooooo00goo`03003ooooo01goo`03003ooooo0=;oo`00
6ooo00<00?ooool03Ooo00<00?ooool07_oo00<00?ooool0dOoo000Kool00`00ooooo`0<ool00`00
ooooo`0Oool00`00ooooo`3Aool001_oo`8000goo`03003ooooo023oo`03003ooooo0=3oo`006ooo
00<00?ooool03?oo00<00?ooool08?oo00<00?ooool0d?oo000Kool00`00ooooo`0<ool00`00oooo
o`0Pool00`00ooooo`3@ool001_oo`03003ooooo00_oo`03003ooooo02;oo`03003ooooo0<ooo`00
6ooo00<00?ooool02ooo00<00?ooool08_oo00<00?ooool0cooo000Kool00`00ooooo`0;ool00`00
ooooo`0Sool00`00ooooo`3>ool001_oo`8000coo`03003ooooo02?oo`03003ooooo0<koo`006ooo
00<00?ooool02ooo00<00?ooool08ooo00<00?ooool0c_oo000Kool00`00ooooo`0;ool00`00oooo
o`0Tool00`00ooooo`3=ool001?oo`@000Coo`03003ooooo00_oo`03003ooooo02Coo`03003ooooo
0<goo`004ooo00<00?ooool01Ooo00<00?ooool02_oo00<00?ooool09_oo00<00?ooool0c?oo000D
ool00`00ooooo`04ool2000;ool00`00ooooo`0Vool00`00ooooo`3<ool000goo`@000Coo`03003o
oooo00?oo`03003ooooo00[oo`03003ooooo02Koo`03003ooooo0<coo`005_oo00<00?ooool00_oo
00<00?ooool02_oo00<00?ooool09ooo00<00?ooool0booo000Cool01000ooooo`001?oo00<00?oo
ool02_oo00<00?ooool09ooo00<00?ooool0booo000Dool20005ool00`00ooooo`0:ool00`00oooo
o`0Xool00`00ooooo`3:ool001_oo`03003ooooo00[oo`03003ooooo02Soo`03003ooooo0<[oo`00
6ooo0P002ooo00<00?ooool0:Ooo00<00?ooool0bOoo000Kool00`00ooooo`09ool00`00ooooo`0Z
ool00`00ooooo`39ool001_oo`03003ooooo00Woo`03003ooooo02[oo`03003ooooo0<Woo`006ooo
00<00?ooool02Ooo00<00?ooool0:ooo00<00?ooool0b?oo000Kool00`00ooooo`09ool00`00oooo
o`0[ool00`00ooooo`38ool001_oo`8000[oo`03003ooooo02coo`03003ooooo0<Ooo`006ooo00<0
0?ooool02Ooo00<00?ooool0;?oo00<00?ooool0aooo000Kool00`00ooooo`09ool00`00ooooo`0/
ool00`00ooooo`37ool001_oo`03003ooooo00Soo`03003ooooo02koo`03003ooooo0<Koo`006ooo
00<00?ooool02?oo00<00?ooool0;_oo00<00?ooool0a_oo000Kool00`00ooooo`08ool00`00oooo
o`0_ool00`00ooooo`35ool001_oo`8000Woo`03003ooooo02ooo`03003ooooo0<Goo`006ooo00<0
0?ooool02?oo00<00?ooool0<?oo00<00?ooool0a?oo000Kool00`00ooooo`08ool00`00ooooo`0`
ool00`00ooooo`34ool001_oo`03003ooooo00Soo`03003ooooo033oo`03003ooooo0<Coo`006ooo
00<00?ooool01ooo00<00?ooool0<_oo00<00?ooool0`ooo000Kool20008ool00`00ooooo`0bool0
0`00ooooo`33ool001_oo`03003ooooo00Ooo`03003ooooo03?oo`03003ooooo0<;oo`006ooo00<0
0?ooool01ooo00<00?ooool0<ooo00<00?ooool0`_oo000Kool00`00ooooo`07ool00`00ooooo`0d
ool00`00ooooo`31ool000Ooo`@000?oo`03003ooooo00;oo`<000Goo`03003ooooo00Ooo`03003o
oooo03Coo`03003ooooo0<7oo`002Ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool01_oo
00<00?ooool0=Ooo00<00?ooool0`Ooo0009ool00`00ooooo`0:ool00`00ooooo`02ool20007ool0
0`00ooooo`0fool00`00ooooo`30ool00003ool00000008000Coo`03003ooooo00Ooo`<000Goo`03
003ooooo00Koo`03003ooooo03Koo`03003ooooo0<3oo`002Ooo00<00?ooool01ooo00<00?ooool0
1Ooo00<00?ooool01_oo00<00?ooool0=ooo00<00?ooool0_ooo0007ool30009ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`0gool00`00ooooo`2oool000Woo`03003ooooo00Ooo`@000Co
o`03003ooooo00Koo`03003ooooo03Soo`03003ooooo0;koo`006ooo0P001ooo00<00?ooool0>?oo
00<00?ooool0__oo000Kool00`00ooooo`05ool00`00ooooo`0jool00`00ooooo`2mool001_oo`03
003ooooo00Goo`03003ooooo03[oo`03003ooooo0;goo`006ooo00<00?ooool01Ooo00<00?ooool0
>ooo00<00?ooool0_?oo000Kool00`00ooooo`05ool00`00ooooo`0kool00`00ooooo`2lool001_o
o`03003ooooo00Goo`03003ooooo03coo`03003ooooo0;_oo`006ooo0P001_oo00<00?ooool0??oo
00<00?ooool0^ooo000Kool00`00ooooo`05ool00`00ooooo`0mool00`00ooooo`2jool001_oo`03
003ooooo00Coo`03003ooooo03koo`03003ooooo0;[oo`006ooo00<00?ooool01?oo00<00?ooool0
?ooo00<00?ooool0^Ooo000Kool00`00ooooo`04ool00`00ooooo`0oool00`00ooooo`2iool001_o
o`8000Goo`03003ooooo043oo`03003ooooo0;Soo`006ooo00<00?ooool01?oo00<00?ooool0@?oo
00<00?ooool0^?oo000Kool00`00ooooo`04ool00`00ooooo`11ool00`00ooooo`2gool001_oo`03
003ooooo00?oo`03003ooooo04;oo`03003ooooo0;Ooo`006ooo00<00?ooool00ooo00<00?ooool0
@ooo00<00?ooool0]_oo000Kool00`00ooooo`03ool00`00ooooo`13ool00`00ooooo`2fool001_o
o`8000Coo`03003ooooo04Coo`03003ooooo0;Goo`006ooo00<00?ooool00ooo00<00?ooool0AOoo
00<00?ooool0]?oo000Kool00`00ooooo`03ool00`00ooooo`15ool00`00ooooo`2dool001?oo`@0
00Coo`03003ooooo00?oo`03003ooooo04Koo`03003ooooo0;?oo`005Ooo00<00?ooool00ooo00<0
0?ooool00ooo00<00?ooool0Aooo00<00?ooool0/_oo000Eool00`00ooooo`03ool20004ool00`00
ooooo`17ool00`00ooooo`2bool000goo`@000Coo`03003ooooo00?oo`03003ooooo00;oo`03003o
oooo04Woo`03003ooooo0;7oo`005Ooo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0BOoo
00<00?ooool0/Ooo000Cool30005ool00`00ooooo`02ool00`00ooooo`1:ool00`00ooooo`2`ool0
01Goo`03003ooooo00?oo`03003ooooo00;oo`03003ooooo04_oo`03003ooooo0:ooo`006ooo00<0
0?ooool00_oo00<00?ooool0Booo00<00?ooool0[ooo000Kool20003ool00`00ooooo`1<ool00`00
ooooo`2^ool001_oo`03003ooooo00;oo`03003ooooo04goo`03003ooooo0:goo`006ooo00<00?oo
ool00_oo00<00?ooool0COoo00<00?ooool0[Ooo000Kool00`00ooooo`02ool00`00ooooo`1>ool0
0`00ooooo`2/ool001_oo`03003ooooo00;oo`03003ooooo04ooo`03003ooooo0:_oo`006ooo0P00
0ooo00<00?ooool0D?oo00<00?ooool0Z_oo000Kool01@00oooooooo0000Dooo00<00?ooool0Z_oo
000Kool01@00oooooooo0000E?oo00<00?ooool0ZOoo000Kool01@00oooooooo0000EOoo00<00?oo
ool0Z?oo000Kool01@00oooooooo0000E_oo00<00?ooool0Yooo000Kool01@00oooooooo0000E_oo
00<00?ooool0Yooo000Kool20002ool00`00ooooo`1Eool00`00ooooo`2Vool001_oo`05003ooooo
ool0001Hool00`00ooooo`2Uool001_oo`05003oooooool0001Iool00`00ooooo`2Tool001_oo`05
003oooooool0001Iool00`00ooooo`2Tool001_oo`05003oooooool0001Jool00`00ooooo`2Sool0
01_oo`8000;oo`03003ooooo05Woo`03003ooooo0:;oo`006ooo00D00?ooooooo`0005coo`03003o
oooo0:7oo`006ooo00@00?ooool005koo`03003ooooo0:3oo`006ooo00@00?ooool005ooo`03003o
oooo09ooo`002?oo0P001?oo00<00?ooool00_oo0`001Ooo00@00?ooool0063oo`03003ooooo09ko
o`001ooo00@00?ooool000_oo`03003ooooo00;oo`04003ooooo001Qool2002Nool000Ooo`04003o
oooo000;ool00`00ooooo`02ool200000ooo003oo`1Rool00`00ooooo`2Kool00003ool000000080
00;oo`04003ooooo0008ool30005ool01000ooooo`00I?oo00<00?ooool0V_oo0007ool01000oooo
o`002?oo00<00?ooool01Ooo00@00?ooool006Goo`03003ooooo09Woo`001ooo00@00?ooool000So
o`03003ooooo00Goo`04003ooooo001Vool00`00ooooo`2Hool000Soo`8000Woo`@000Coo`04003o
oooo001Wool2002Hool001_oo`800003ool00?oo06Soo`03003ooooo09Goo`006ooo00@00?ooool0
06[oo`8009Goo`006ooo00<00?oo0000KOoo0P00Tooo000Kool00`00ool0001_ool00`00ooooo`2@
ool001_oo`03003oo`00073oo`80093oo`006ooo00<00?oo0000L_oo00<00?ooool0SOoo000Kool3
001cool2002=ool001_oo`03003oo`0007Goo`8008_oo`006ooo00<00?ooool0Mooo00<00?ooool0
R?oo000Kool00`00ooooo`1hool20028ool001_oo`03003ooooo07[oo`8008Koo`006ooo0P00OOoo
0P00Q?oo000Kool00`00ooooo`0Hool20004ool00`00ooooo`02ool4000Bool20004ool00`00oooo
o`04ool00`00ooooo`0Bool20004ool00`00ooooo`03ool2000Dool40002ool00`00ooooo`03ool2
000Iool4000Gool40003ool00`00ooooo`02ool4000Bool40003ool00`00ooooo`04ool00`00oooo
o`0@ool001_oo`03003ooooo01Ooo`04003ooooo0008ool00`00ooooo`0Bool01000ooooo`002_oo
00<00?ooool04Ooo00@00?ooool000Soo`04003ooooo000Bool01@00ooooo`00ool00P001Ooo00@0
0?ooool001[oo`03003ooooo01Soo`03003ooooo00Ooo`03003ooooo01Goo`03003ooooo00Woo`03
003ooooo013oo`006ooo00<00?ooool05ooo00@00?ooool000Woo`03003ooooo017oo`04003ooooo
0007ool5000Bool01000ooooo`002?oo00@00?ooool001;oo`04003ooooo0003ool30002ool01000
ooooo`006_oo00<00?ooool06?oo00<00?ooool02?oo00<00?ooool05?oo00<00?ooool01_oo1@00
4Ooo000Kool00`00ooooo`0Gool01000ooooo`002_oo00<00?ooool04?oo00@00?ooool000Ooo`04
003ooooo000Cool01000ooooo`002?oo0`004ooo00@00?ooool000Koo`D001_oo`03003ooooo01So
o`03003ooooo00Woo`03003ooooo01?oo`03003ooooo00Koo`04003ooooo000Bool001_oo`03003o
oooo01Ooo`04003ooooo000;ool00`00ooooo`0?ool01000ooooo`002?oo00<00?oo00004ooo00@0
0?ooool000Soo`03003ooooo01?oo`04003ooooo0008ool5000Iool00`00ooooo`0Hool00`00oooo
o`0:ool00`00ooooo`0Bool00`00ooooo`07ool00`00ool0000Bool001_oo`8001Soo`04003ooooo
0008ool01000ooooo`004Ooo00@00?ooool000Woo`8001?oo`04003ooooo0008ool00`00ooooo`0C
ool01000ooooo`002?oo00D00?ooool00?oo00@001?oo`<001Soo`<000Woo`04003ooooo000Bool3
000:ool2000Bool001_oo`03003ooooo01Soo`8000[oo`8001?oo`8000_oo`03003ooooo01;oo`80
00[oo`<001?oo`8000[oo`8000Koo`<001;oo`03003ooooo01Soo`03003ooooo00Soo`8001Goo`03
003ooooo00Woo`03003ooooo013oo`006ooo00<00?ooool0U?oo0`00Jooo000Kool00`00ooooo`2G
ool4001Wool001_oo`03003ooooo09_oo`@006?oo`005?ooo`003@00000Kool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool0
01_oo`03003ooooo01ooo`03003ooooo01koo`03003ooooo01ooo`03003ooooo01ooo`03003ooooo
01ooo`03003ooooo01koo`03003ooooo01ooo`03003ooooo01Koo`006ooo00<00?ooool0oooo0ooo
000Kool00`00ooooo`3oool3ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo
003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, -3.03402, \
0.00592092, 0.0181944}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(diffs\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[eta], 
        Abs[Re[alphakx0korr[eta, 10.0]] - 
            Sqrt[11] FastRealpha[1, 1, 1 - 10^\((\(-6\))\), eta, 10]]}, {eta, 
        0.01, 1.2, 0.01}]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(37\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.01`, 0.002182302930692437`}, {0.02`, 
        0.020309602000534888`}, {0.03`, 0.00006752236381712962`}, {0.04`, 
        0.0005983966194200718`}, {0.05`, 0.012427656086924088`}, {0.06`, 
        0.000047040413460042174`}, {0.07`, 0.00008674163956934322`}, {0.08`, 
        0.0001297168835914242`}, {0.09`, 0.0001750206983825997`}, {0.1`, 
        0.00022172594322755756`}, {0.11`, 0.00026895325392350955`}, {0.12`, 
        0.00031589507891505164`}, {0.13`, 0.6074009980310788`}, {0.14`, 
        0.0004061447638050719`}, {0.15`, 0.0004483174340061957`}, {0.16`, 
        0.0005545842710867888`}, {0.17`, 0.0005433763982689577`}, {0.18`, 
        0.0005299414241854983`}, {0.19`, 0.0005146968845608502`}, {0.2`, 
        0.0004980217773822293`}, {0.21`, 0.0004802565829584715`}, {0.22`, 
        0.0004617027388378325`}, {0.23`, 0.00044262489442675745`}, {0.24`, 
        0.00042325114542280523`}, {0.25`, 0.00040377653697031`}, {0.26`, 
        0.0003843640876117216`}, {0.27`, 0.2053113596511924`}, {0.28`, 
        0.20133728311230484`}, {0.29`, 0.19708896374376428`}, {0.3`, 
        0.19261957748567515`}, {0.31`, 0.00029237836469631073`}, {0.32`, 
        0.00027552099714434064`}, {0.33`, 0.0007173210181935508`}, {0.34`, 
        0.0007114137254045083`}, {0.35000000000000003`, 
        0.0002299174994142561`}, {0.36`, 0.00021585079766106574`}, {0.37`, 
        0.00020245804841145798`}, {0.38`, 0.00018973149840206815`}, {0.39`, 
        0.00017765880910380893`}, {0.4`, 
        0.0001662245286073727`}, {0.41000000000000003`, 
        0.00015846343026248988`}, {0.42`, 0.00014519584627215032`}, {0.43`, 
        0.000135559262253393`}, {0.44`, 0.00012647789739905768`}, {0.45`, 
        0.00011792812601507663`}, {0.46`, 
        0.00010988639717446347`}, {0.47000000000000003`, 
        0.00010232878605531681`}, {0.48`, 0.00009523181340820308`}, {0.49`, 
        0.00008857233397940867`}, {0.5`, 0.00008232749223191504`}, {0.51`, 
        0.00007647535756327706`}, {0.52`, 0.0000709947140733469`}, {0.53`, 
        0.0000658647701325954`}, {0.54`, 0.00006106591039856113`}, {0.55`, 
        0.08467922044382781`}, {0.56`, 
        0.08157111311179066`}, {0.5700000000000001`, 
        0.07856104994262925`}, {0.58`, 0.07564627403679558`}, {0.59`, 
        0.07282394176894269`}, {0.6`, 0.07009114968778352`}, {0.61`, 
        0.06744495842143916`}, {0.62`, 0.06488241179628507`}, {0.63`, 
        0.062400553363119005`}, {0.64`, 0.05999643963587542`}, {0.65`, 
        0.000025225303392550913`}, {0.66`, 0.000023130315236263854`}, {0.67`, 
        0.000021195295686671223`}, {0.68`, 
        0.00001939725069211029`}, {0.6900000000000001`, 
        0.0000177344258769363`}, {0.7000000000000001`, 
        0.000016185064189655485`}, {0.71`, 0.000014748569758249452`}, {0.72`, 
        0.00001341768437745916`}, {0.73`, 0.000012185887582671917`}, {0.74`, 
        0.000011054142167177261`}, {0.75`, 0.000010002501601658054`}, {0.76`, 
        9.032165191436015`*^-6}, {0.77`, 8.136550438520018`*^-6}, {0.78`, 
        7.311809200993835`*^-6}, {0.79`, 6.37441568884789`*^-6}, {0.8`, 
        5.857407411657967`*^-6}, {0.81`, 
        5.220025116270399`*^-6}, {0.8200000000000001`, 
        4.637080605940369`*^-6}, {0.8300000000000001`, 
        4.105307360169341`*^-6}, {0.84`, 3.6213203760127044`*^-6}, {0.85`, 
        3.1823144688303273`*^-6}, {0.86`, 2.777185238556079`*^-6}, {0.87`, 
        0.004743889788025202`}, {0.88`, 0.009919970382346044`}, {0.89`, 
        0.009307377659422578`}, {0.9`, 0.00869132812173723`}, {0.91`, 
        0.014615776958428306`}, {0.92`, 0.00743556907324594`}, {0.93`, 
        0.011953407760167845`}, {0.9400000000000001`, 
        0.010594993636641958`}, {0.9500000000000001`, 
        0.009202886305519908`}, {0.96`, 0.007759253978918079`}, {0.97`, 
        0.006236344204728184`}, {0.98`, 0.004585129256444831`}, {0.99`, 
        0.0026965161613335953`}, {1.`, Indeterminate}, {1.01`, 
        3.3856242809271464`*^-7}, {1.02`, 7.690140647609311`*^-7}, {1.03`, 
        3.3645516006597376`*^-7}, {1.04`, 1.628890380356296`*^-7}, {1.05`, 
        1.0432978136964977`*^-7}, {1.06`, 8.571520759836915`*^-8}, {1.07`, 
        8.108339843992951`*^-8}, {1.08`, 8.120643979481201`*^-8}, {1.09`, 
        8.30780267913056`*^-8}, {1.1`, 8.533794737530798`*^-8}, {1.11`, 
        8.766491055300918`*^-8}, {1.12`, 
        8.994062450362605`*^-8}, {1.1300000000000001`, 
        9.178154181110898`*^-8}, {1.1400000000000001`, 
        9.376720000453531`*^-8}, {1.1500000000000001`, 
        9.534269795885564`*^-8}, {1.16`, 9.672814060834867`*^-8}, {1.17`, 
        9.80863067434612`*^-8}, {1.18`, 9.914994536508459`*^-8}, {1.19`, 
        1.0018349072549171`*^-7}, {1.2`, 7.804566193314599`*^-8}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotRange \[Rule] All]\)], "Input"],

Cell[BoxData[
    \(Graphics::"gptn" \(\(:\)\(\ \)\) 
      "Coordinate \!\(Indeterminate\) in \!\({1.`, Indeterminate}\) is not a \
floating-point number."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0158063 0.80032 0.014715 0.969053 [
[.17587 .00222 -9 -9 ]
[.17587 .00222 9 0 ]
[.33593 .00222 -9 -9 ]
[.33593 .00222 9 0 ]
[.496 .00222 -9 -9 ]
[.496 .00222 9 0 ]
[.65606 .00222 -9 -9 ]
[.65606 .00222 9 0 ]
[.81613 .00222 -3 -9 ]
[.81613 .00222 3 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.00331 .11162 -18 -4.5 ]
[.00331 .11162 0 4.5 ]
[.00331 .20853 -18 -4.5 ]
[.00331 .20853 0 4.5 ]
[.00331 .30543 -18 -4.5 ]
[.00331 .30543 0 4.5 ]
[.00331 .40234 -18 -4.5 ]
[.00331 .40234 0 4.5 ]
[.00331 .49924 -18 -4.5 ]
[.00331 .49924 0 4.5 ]
[.00331 .59615 -18 -4.5 ]
[.00331 .59615 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17587 .01472 m
.17587 .02097 L
s
[(0.2)] .17587 .00222 0 1 Mshowa
.33593 .01472 m
.33593 .02097 L
s
[(0.4)] .33593 .00222 0 1 Mshowa
.496 .01472 m
.496 .02097 L
s
[(0.6)] .496 .00222 0 1 Mshowa
.65606 .01472 m
.65606 .02097 L
s
[(0.8)] .65606 .00222 0 1 Mshowa
.81613 .01472 m
.81613 .02097 L
s
[(1)] .81613 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(1.2)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05582 .01472 m
.05582 .01847 L
s
.09584 .01472 m
.09584 .01847 L
s
.13585 .01472 m
.13585 .01847 L
s
.21589 .01472 m
.21589 .01847 L
s
.2559 .01472 m
.2559 .01847 L
s
.29592 .01472 m
.29592 .01847 L
s
.37595 .01472 m
.37595 .01847 L
s
.41597 .01472 m
.41597 .01847 L
s
.45598 .01472 m
.45598 .01847 L
s
.53601 .01472 m
.53601 .01847 L
s
.57603 .01472 m
.57603 .01847 L
s
.61605 .01472 m
.61605 .01847 L
s
.69608 .01472 m
.69608 .01847 L
s
.73609 .01472 m
.73609 .01847 L
s
.77611 .01472 m
.77611 .01847 L
s
.85614 .01472 m
.85614 .01847 L
s
.89616 .01472 m
.89616 .01847 L
s
.93617 .01472 m
.93617 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.01581 .11162 m
.02206 .11162 L
s
[(0.1)] .00331 .11162 1 0 Mshowa
.01581 .20853 m
.02206 .20853 L
s
[(0.2)] .00331 .20853 1 0 Mshowa
.01581 .30543 m
.02206 .30543 L
s
[(0.3)] .00331 .30543 1 0 Mshowa
.01581 .40234 m
.02206 .40234 L
s
[(0.4)] .00331 .40234 1 0 Mshowa
.01581 .49924 m
.02206 .49924 L
s
[(0.5)] .00331 .49924 1 0 Mshowa
.01581 .59615 m
.02206 .59615 L
s
[(0.6)] .00331 .59615 1 0 Mshowa
.125 Mabswid
.01581 .0341 m
.01956 .0341 L
s
.01581 .05348 m
.01956 .05348 L
s
.01581 .07286 m
.01956 .07286 L
s
.01581 .09224 m
.01956 .09224 L
s
.01581 .131 m
.01956 .131 L
s
.01581 .15038 m
.01956 .15038 L
s
.01581 .16976 m
.01956 .16976 L
s
.01581 .18914 m
.01956 .18914 L
s
.01581 .22791 m
.01956 .22791 L
s
.01581 .24729 m
.01956 .24729 L
s
.01581 .26667 m
.01956 .26667 L
s
.01581 .28605 m
.01956 .28605 L
s
.01581 .32481 m
.01956 .32481 L
s
.01581 .34419 m
.01956 .34419 L
s
.01581 .36357 m
.01956 .36357 L
s
.01581 .38296 m
.01956 .38296 L
s
.01581 .42172 m
.01956 .42172 L
s
.01581 .4411 m
.01956 .4411 L
s
.01581 .46048 m
.01956 .46048 L
s
.01581 .47986 m
.01956 .47986 L
s
.01581 .51862 m
.01956 .51862 L
s
.01581 .538 m
.01956 .538 L
s
.01581 .55738 m
.01956 .55738 L
s
.01581 .57677 m
.01956 .57677 L
s
.01581 .61553 m
.01956 .61553 L
s
.25 Mabswid
.01581 0 m
.01581 .61803 L
s
.008 w
.02381 .01683 Mdot
.03181 .0344 Mdot
.03982 .01478 Mdot
.04782 .01529 Mdot
.05582 .02676 Mdot
.06383 .01476 Mdot
.07183 .0148 Mdot
.07983 .01484 Mdot
.08784 .01488 Mdot
.09584 .01493 Mdot
.10384 .01498 Mdot
.11184 .01502 Mdot
.11985 .60332 Mdot
.12785 .01511 Mdot
.13585 .01515 Mdot
.14386 .01525 Mdot
.15186 .01524 Mdot
.15986 .01523 Mdot
.16787 .01521 Mdot
.17587 .0152 Mdot
.18387 .01518 Mdot
.19188 .01516 Mdot
.19988 .01514 Mdot
.20788 .01513 Mdot
.21589 .01511 Mdot
.22389 .01509 Mdot
.23189 .21367 Mdot
.2399 .20982 Mdot
.2479 .2057 Mdot
.2559 .20137 Mdot
.26391 .015 Mdot
.27191 .01498 Mdot
.27991 .01541 Mdot
.28792 .0154 Mdot
.29592 .01494 Mdot
.30392 .01492 Mdot
.31192 .01491 Mdot
.31993 .0149 Mdot
.32793 .01489 Mdot
.33593 .01488 Mdot
.34394 .01487 Mdot
.35194 .01486 Mdot
.35994 .01485 Mdot
.36795 .01484 Mdot
.37595 .01483 Mdot
.38395 .01482 Mdot
.39196 .01481 Mdot
.39996 .01481 Mdot
.40796 .0148 Mdot
.41597 .01479 Mdot
.42397 .01479 Mdot
.43197 .01478 Mdot
.43998 .01478 Mdot
.44798 .01477 Mdot
.45598 .09677 Mdot
.46399 .09376 Mdot
.47199 .09084 Mdot
.47999 .08802 Mdot
.488 .08529 Mdot
.496 .08264 Mdot
.504 .08007 Mdot
.512 .07759 Mdot
.52001 .07518 Mdot
.52801 .07285 Mdot
.53601 .01474 Mdot
.54402 .01474 Mdot
.55202 .01474 Mdot
.56002 .01473 Mdot
.56803 .01473 Mdot
.57603 .01473 Mdot
.58403 .01473 Mdot
.59204 .01473 Mdot
.60004 .01473 Mdot
.60804 .01473 Mdot
.61605 .01472 Mdot
.62405 .01472 Mdot
.63205 .01472 Mdot
.64006 .01472 Mdot
.64806 .01472 Mdot
.65606 .01472 Mdot
.66407 .01472 Mdot
.67207 .01472 Mdot
.68007 .01472 Mdot
.68808 .01472 Mdot
.69608 .01472 Mdot
.70408 .01472 Mdot
.71208 .01931 Mdot
.72009 .02433 Mdot
.72809 .02373 Mdot
.73609 .02314 Mdot
.7441 .02888 Mdot
.7521 .02192 Mdot
.7601 .0263 Mdot
.76811 .02498 Mdot
.77611 .02363 Mdot
.78411 .02223 Mdot
.79212 .02076 Mdot
.80012 .01916 Mdot
.80812 .01733 Mdot
.82413 .01472 Mdot
.83213 .01472 Mdot
.84014 .01472 Mdot
.84814 .01472 Mdot
.85614 .01472 Mdot
.86415 .01472 Mdot
.87215 .01472 Mdot
.88015 .01472 Mdot
.88816 .01472 Mdot
.89616 .01472 Mdot
.90416 .01472 Mdot
.91216 .01472 Mdot
.92017 .01472 Mdot
.92817 .01472 Mdot
.93617 .01472 Mdot
.94418 .01472 Mdot
.95218 .01472 Mdot
.96018 .01472 Mdot
.96819 .01472 Mdot
.97619 .01472 Mdot
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00>Ooo0P001?oo00<00?ooool00_oo10006ooo0P001?oo00<00?ooool01?oo00<0
0?ooool06ooo0P001?oo00<00?ooool00ooo0P007Ooo0P001?oo00<00?ooool00ooo0P008_oo1000
8Ooo10000ooo00<00?ooool00_oo10000_oo000hool01000ooooo`002?oo00<00?ooool06ooo00@0
0?ooool000[oo`03003ooooo01[oo`04003ooooo0008ool01000ooooo`006ooo00@00?ooool000So
o`04003ooooo000Sool00`00ooooo`0Rool00`00ooooo`07ool00`00ooooo`03ool003Soo`04003o
oooo0009ool00`00ooooo`0Jool01000ooooo`001ooo1@006ooo00@00?ooool000Soo`04003ooooo
000Kool01000ooooo`002?oo00@00?ooool002?oo`03003ooooo02;oo`03003ooooo00Soo`03003o
oooo00;oo`00>?oo00@00?ooool000[oo`03003ooooo01Woo`04003ooooo0007ool01000ooooo`00
7?oo00@00?ooool000Soo`<001coo`04003ooooo0009ool2000Tool00`00ooooo`0Rool00`00oooo
o`09ool00`00ooooo`01ool003Soo`04003ooooo000;ool00`00ooooo`0Hool01000ooooo`002?oo
00<00?oo00007?oo00@00?ooool000Soo`03003ooooo01coo`04003ooooo0008ool01000ooooo`00
8ooo00<00?ooool08_oo00<00?ooool02_oo0@000Ooo0Ooo000hool01000ooooo`002?oo00@00?oo
ool001[oo`04003ooooo0009ool2000Lool01000ooooo`002?oo00<00?ooool07?oo00@00?ooool0
00Soo`04003ooooo000Qool3000Rool30009ool01000ooooo`000_oo000iool2000:ool2000Lool2
000;ool00`00ooooo`0Kool2000:ool3000Lool2000:ool2000Tool00`00ooooo`0Rool00`00oooo
o`08ool20003ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`3oool9ool001Goo`03003ooooo00?oo`@000;oo`H00003ool0000000H000;oo`@00003ool0
000000h00003ool0000000@000Woo`h00003ool0000000`00003ool0000000h00003ool000000080
01Goo`@00003ool0000000`00003ool0000000`00003ool0000000X001koo`@00003ool0000000`0
0003ool0000000`00003ool0000000H000Woo`004Oooo`003@000ooo000Eool00`00ool00008ool0
0`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool0
0`00ooooo`08ool01@00oooooooo00001Ooo0P000_oo0P001Ooo00<00?oo00000`0000Koo`00ool0
0?oo0009ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool001Go
o`03003ooooo00Ooo`800:ooo`800005ool00000ool00003ool70002ool00`00ool0000jool001Go
o`03003ooooo00Soo`03003ooooo0:koo`04003ooooo0002ool20003ool01@00ool00?oo0000?ooo
000Eool01000oooooooo0P00_Ooo00<00?ooool0AOoo000Eool20003ool00`00ooooo`3oool4ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03
003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`80
08Coo`@0087oo`005Ooo00<00?ooool0POoo0P0000Coo`00ool0087oo`005Ooo00<00?ooool0OOoo
100000?oo`00ool0Q?oo000Eool00`00ooooo`1jool20002ool00`00ool00027ool001Goo`03003o
oooo07Koo`@00003ool00?oo08_oo`005Ooo0P00MOoo0P0000Coo`00ool008koo`005Ooo00<00?oo
ool0L_oo0P0000?oo`00ool0TOoo000Eool00`00ooooo`1cool00`00ooooo`2Bool000;oo`8000Co
o`03003ooooo00;oo`@000Coo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000[oo`03003ooooo
00?oo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`<00?ooo`Woo`00
00Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000[o
o`03003ooooo00?oo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000Soo`<000Goo`03003ooooo
0?ooo`Woo`000_oo0P002ooo00<00?ooool00ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3o
ool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03
003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03
003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[o
o`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool000;oo`8000Coo`03003ooooo
00;oo`@000Coo`03003ooooo03coo`800<[oo`0000Goo`00ooooo`0000Soo`03003ooooo00Goo`03
003ooooo03[oo`800003ool00?oo0<Woo`0000Goo`00ooooo`0000Woo`03003ooooo00Coo`<003So
o`800003ool00?oo0<_oo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo03Koo`80
0003ool00?oo0<goo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003ooooo03Ooo`03003o
oooo0<koo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3oool9ool000;oo`80
00[oo`8000Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03
003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`80
0?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Wo
o`005Ooo00<00?ooool0oooo2Ooo0002ool20004ool00`00ooooo`03ool20005ool00`00ooooo`3o
ool9ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0oooo2Ooo00001Ooo003o
oooo00002ooo00<00?ooool00_oo0`00oooo2Ooo00001Ooo003ooooo00002Ooo0P001Ooo00<00?oo
ool0oooo2Ooo00001Ooo003ooooo00002ooo00<00?ooool00_oo00<00?ooool0oooo2Ooo00001Ooo
003ooooo00002?oo00@00?ooool000Coo`03003ooooo0?ooo`Woo`000_oo0P002_oo0P001Ooo00<0
0?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo
000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool0
0`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool0
0`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00oooo
o`3oool9ool000;oo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo0?ooo`Woo`00
00Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000Oo
o`D000Coo`<00?ooo`Woo`0000Goo`00ooooo`0000Ooo`04003ooooo0005ool00`00ooooo`3oool9
ool00005ool00?ooool00008ool00`00ool00005ool00`00ooooo`3oool9ool00005ool00?ooool0
0009ool20005ool00`00ooooo`3oool9ool000;oo`8000_oo`03003ooooo00?oo`03003ooooo0?oo
o`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo
0002ool20004ool00`00ooooo`02ool30005ool00`00ooooo`3oool9ool00005ool00?ooool0000;
ool00`00ooooo`02ool00`00ooooo`3oool9ool00005ool00?ooool0000;ool00`00ooooo`02ool3
003oool9ool00005ool00?ooool00008ool30005ool00`00ooooo`3oool9ool00005ool00?ooool0
0008ool00`00ooooo`05ool00`00ooooo`3oool9ool00005ool00?ooool00008ool00`00ooooo`05
ool00`00ooooo`3oool9ool000;oo`8000Woo`@000Coo`03003ooooo0?ooo`Woo`005Ooo0P00oooo
2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo
2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000E
ool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000E
ool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003o
ool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo0002ool20004ool00`00
ooooo`03ool20005ool00`00ooooo`3oool9ool00005ool00?ooool00008ool01000ooooo`001?oo
00<00?ooool0oooo2Ooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`<00?ooo`Woo`0000Go
o`00ooooo`0000Soo`<000Goo`03003ooooo01Soo`800>koo`0000Goo`00ooooo`0000Soo`03003o
oooo00Goo`03003ooooo01Woo`03003ooooo0>coo`0000Goo`00ooooo`0000Soo`03003ooooo00Go
o`03003ooooo0?ooo`Woo`000_oo0P002_oo0`001?oo00<00?ooool0oooo2Ooo000Eool2003oool:
ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0997789, -0.0541438, \
0.00467548, 0.00386138}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastRedelta[1, 1, 1 - 10^\((\(-6\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(39\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(39\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(39\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.40129574781642013`\)."\
\)], "Message"],

Cell[BoxData[
    \(0.5396538677162888`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FastImdelta[1, 1, 1 - 10^\((\(-6\))\), 4/10, 10]*Sqrt[11]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.40129574781564076`\)."\
\)], "Message"],

Cell[BoxData[
    \(2.209939429344497`\)], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Comparison to Mike", "Section"],

Cell[CellGroupData[{

Cell["Importing data", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Imdata1t0 = 
        ReadList["\<mike/data_im_1_0.000000.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(ReadList::"nffil" \(\(:\)\(\ \)\) 
      "File not found during \
\!\(ReadList[\(\(\"mike/data_im_1_0.000000.txt\", \(\({Number, Number, \
Number, Number, Number}\)\)\)\)]\)."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(\(Imdata1tPi4 = 
        ReadList["\<mike/data_im_1_0.785398.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Imdata1tPi2 = 
        ReadList["\<mike/data_im_1_1.570796.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Imdata2t0 = 
        ReadList["\<mike/data_im_2_0.000000.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Imdata2tPi4 = 
        ReadList["\<mike/data_im_2_0.785398.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Imdata2tPi2 = 
        ReadList["\<mike/data_im_2_1.570796.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Imdata5t0 = 
        ReadList["\<mike/data_im_5_0.000000.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Imdata5tPi4 = 
        ReadList["\<mike/data_im_5_0.785398.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Imdata5tPi2 = 
        ReadList["\<mike/data_im_5_1.570796.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata1t0 = 
        ReadList["\<mike/data_re_1_0.000000.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata1tPi4 = 
        ReadList["\<mike/data_re_1_0.785398.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata1tPi2 = 
        ReadList["\<mike/data_re_1_1.570796.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata2t0 = 
        ReadList["\<mike/data_re_2_0.000000.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata2tPi4 = 
        ReadList["\<mike/data_re_2_0.785398.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata2tPi2 = 
        ReadList["\<mike/data_re_2_1.570796.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata5t0 = 
        ReadList["\<mike/data_re_5_0.000000.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata5tPi4 = 
        ReadList["\<mike/data_re_5_0.785398.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"],

Cell[BoxData[
    \(\(Redata5tPi2 = 
        ReadList["\<mike/data_re_5_1.570796.txt\>", {Number, Number, Number, 
            Number, Number}];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["alpha-Im", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[dat_, pos_, a_, t_] := 
      Module[{res}, 
        res = Table[{\(dat[\([i]\)]\)[\([1]\)], \((Imalpha[1, 1, Cos[t], 
                      IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, 
                      a] + \(dat[\([i]\)]\)[\([pos]\)])\)/
                If[\(dat[\([i]\)]\)[\([pos]\)] \[Equal] 0, 
                  1, \(dat[\([i]\)]\)[\([pos]\)]]}, {i, 1, Length[dat]}]; \ 
        res]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(aImtablegen\)\" is \
similar to existing symbol \"\!\(Imtablegen\)\"."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata1t0, 2, 1, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, \(-1.7804413016696888`*^-7\)}, {0.00019952620000000001`, 
        0.00003112265274122345`}, {0.00039810720000000004`, 
        0.0000180374328584289`}, {0.0007943282`, 
        0.000010758302334220912`}, {0.0015848930000000002`, 
        2.1569537999990572`*^-6}, {0.003162278`, 
        2.3987469217254124`*^-6}, {0.006309573000000001`, 
        5.286107874014263`*^-7}, {0.012589250000000001`, 
        1.1650075171022778`*^-7}, {0.02511886`, 
        1.2193280383500385`*^-7}, {0.050118720000000005`, 
        1.0602638389220582`*^-7}, {0.1`, 
        1.3615337108244583`*^-8}, {0.19952620000000001`, 
        7.59006655326146`*^-8}, {0.3981072`, \(-9.521728298943535`*^-8\)}, \
{0.7943282`, \(-4.508545837856946`*^-7\)}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata2t0, 2, 2, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, \(-1.5804412727079074`*^-7\)}, {0.00019952620000000001`, 
        0.000031202266716351035`}, {0.00039810720000000004`, 
        0.000018034576546058603`}, {0.0007943282`, 
        0.00001041727596615391`}, {0.0015848930000000002`, 
        2.09269140694525`*^-6}, {0.003162278`, 
        2.4671848642299364`*^-6}, {0.006309573000000001`, 
        3.226136226934549`*^-7}, {0.012589250000000001`, 
        4.2866659139391416`*^-7}, {0.02511886`, 
        1.6719937361614862`*^-7}, {0.050118720000000005`, 
        3.8301162937869373`*^-7}, {0.1`, 
        1.8704596130690247`*^-7}, {0.19952620000000001`, 
        1.3991862335699848`*^-8}, {0.3981072`, 
        6.513142372130221`*^-8}, {0.7943282`, 
        6.396454450823855`*^-8}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata5t0, 2, 5, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, \(-9.804412168826728`*^-8\)}, {0.00019952620000000001`, 
        0.00003122840470979165`}, {0.00039810720000000004`, 
        0.000018239218715784056`}, {0.0007943282`, 
        0.000010462782324910815`}, {0.0015848930000000002`, 
        2.033569912011651`*^-6}, {0.003162278`, 
        2.1991252929576496`*^-6}, {0.006309573000000001`, 
        6.231395280793624`*^-7}, {0.012589250000000001`, 
        2.9331384474092957`*^-7}, {0.02511886`, 
        1.8432700909245313`*^-7}, {0.050118720000000005`, 
        2.1109903278390839`*^-7}, {0.1`, 
        8.98537495039446`*^-9}, {0.19952620000000001`, \
\(-1.8285507433751094`*^-8\)}, {0.3981072`, 
        1.628134059492215`*^-8}, {0.7943282`, \(-3.0017009875144613`*^-7\)}, \
{1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 0.`}, {12.58925`, 
        0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata1tPi4, 2, 1, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-1.1294887117026305`*^-8\)}, {0.00019952620000000001`, 
        0.000030950632583868446`}, {0.00039810720000000004`, 
        0.00001804084281769294`}, {0.0007943282`, 
        0.000010360117884943768`}, {0.0015848930000000002`, 
        1.8892178827638244`*^-6}, {0.003162278`, 
        2.2730422094534864`*^-6}, {0.006309573000000001`, 
        4.72192479599687`*^-7}, {0.012589250000000001`, 
        7.583831332538805`*^-7}, {0.02511886`, 
        1.4055697838164296`*^-7}, {0.050118720000000005`, 
        4.9188445566699224`*^-8}, {0.1`, 
        1.0993227444101903`*^-8}, {0.19952620000000001`, 
        4.3108610519170355`*^-7}, {0.3981072`, \(-5.939194100769573`*^-8\)}, \
{0.7943282`, \(-3.171345002201838`*^-7\)}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata2tPi4, 2, 2, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-4.985710471965477`*^-8\)}, {0.00019952620000000001`, 
        0.000031024736928384966`}, {0.00039810720000000004`, 
        0.000018139621280118528`}, {0.0007943282`, 
        0.000010426498020154805`}, {0.0015848930000000002`, 
        2.3420844414117315`*^-6}, {0.003162278`, 
        2.4647746755416905`*^-6}, {0.006309573000000001`, 
        5.213436252658146`*^-7}, {0.012589250000000001`, 
        4.032507418028381`*^-7}, {0.02511886`, 
        3.2714563698640625`*^-7}, {0.050118720000000005`, 
        2.2078872973920535`*^-8}, {0.1`, \(-3.310738857731825`*^-10\)}, \
{0.19952620000000001`, 4.1342469531921624`*^-8}, {0.3981072`, 
        1.1387468090529738`*^-7}, {0.7943282`, \(-4.1251488379805116`*^-7\)}, \
{1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 0.`}, {12.58925`, 
        0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata5tPi4, 2, 5, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 2.2829268728691984`*^-8}, {0.00019952620000000001`, 
        0.00003114871583852363`}, {0.00039810720000000004`, 
        0.000018049174684344226`}, {0.0007943282`, 
        0.00001041624791178841`}, {0.0015848930000000002`, 
        2.2909328410256977`*^-6}, {0.003162278`, 
        2.4480692424409365`*^-6}, {0.006309573000000001`, 
        5.290825094222841`*^-7}, {0.012589250000000001`, 
        3.633325728103643`*^-7}, {0.02511886`, 
        3.9589362584351956`*^-7}, {0.050118720000000005`, \
\(-4.223751251166933`*^-8\)}, {0.1`, 
        4.2097826667287125`*^-8}, {0.19952620000000001`, \
\(-1.6873545116651443`*^-7\)}, {0.3981072`, \(-1.9453714999473204`*^-7\)}, \
{0.7943282`, 3.6598544025767506`*^-8}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata1tPi2, 2, 1, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 6.643506986993642`*^-8}, {0.00019952620000000001`, 
        0.000031611426495522566`}, {0.00039810720000000004`, 
        0.000017929427529573064`}, {0.0007943282`, 
        0.000010341018835998316`}, {0.0015848930000000002`, 
        2.0060615699434926`*^-6}, {0.003162278`, 
        2.700647643658921`*^-6}, {0.006309573000000001`, 
        4.3189284247438257`*^-7}, {0.012589250000000001`, 
        3.7051637566107337`*^-7}, {0.02511886`, 
        2.1164331870569303`*^-7}, {0.050118720000000005`, 
        1.2612372730436688`*^-7}, {0.1`, 
        7.889215026878516`*^-8}, {0.19952620000000001`, 
        2.0569387491441125`*^-7}, {0.3981072`, \(-1.413510202931479`*^-7\)}, \
{0.7943282`, 1.7108414028499168`*^-8}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata2tPi2, 2, 2, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-8.388041088588464`*^-8\)}, {0.00019952620000000001`, 
        0.000031280559783299255`}, {0.00039810720000000004`, 
        0.000017933296588408627`}, {0.0007943282`, 
        0.000010477182960729923`}, {0.0015848930000000002`, 
        2.0661486514128743`*^-6}, {0.003162278`, 
        2.601692703328364`*^-6}, {0.006309573000000001`, 
        4.967788432754683`*^-7}, {0.012589250000000001`, 
        3.1172935249250766`*^-7}, {0.02511886`, \(-1.045763823121219`*^-7\)}, \
{0.050118720000000005`, 2.469010529715609`*^-7}, {0.1`, 
        1.0910720636331859`*^-7}, {0.19952620000000001`, 
        1.0573935879447845`*^-7}, {0.3981072`, 
        3.3702573223606895`*^-9}, {0.7943282`, \(-3.324753120364249`*^-7\)}, \
{1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 0.`}, {12.58925`, 
        0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aImtablegen[Imdata5tPi2, 2, 5, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 1.3825556639723286`*^-7}, {0.00019952620000000001`, 
        0.000031198733747836065`}, {0.00039810720000000004`, 
        0.000018331286058251462`}, {0.0007943282`, 
        0.00001039872533663837`}, {0.0015848930000000002`, 
        2.0554486495283686`*^-6}, {0.003162278`, 
        2.664651035975694`*^-6}, {0.006309573000000001`, 
        7.411630382218035`*^-7}, {0.012589250000000001`, 
        2.71056267385258`*^-7}, {0.02511886`, 
        1.80551540619969`*^-7}, {0.050118720000000005`, 
        2.2787983477617844`*^-7}, {0.1`, \(-1.1525655587570917`*^-7\)}, \
{0.19952620000000001`, 1.236360008408395`*^-7}, {0.3981072`, 
        5.6640606938295676`*^-8}, {0.7943282`, 
        1.0826895564898317`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["alpha-Re", "Subsection"],

Cell[BoxData[
    \(aRetablegen[dat_, pos_, a_, t_] := 
      Module[{res}, 
        res = Table[{\(dat[\([i]\)]\)[\([1]\)], \((Realpha[1, 1, Cos[t], 
                      IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, 
                      a] - \(dat[\([i]\)]\)[\([pos]\)])\)/
                If[\(dat[\([i]\)]\)[\([pos]\)] \[Equal] 0, 
                  1, \(dat[\([i]\)]\)[\([pos]\)]]}, {i, 1, Length[dat]}]; \ 
        res]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata1t0, 2, 1, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.000019877565700643284`}, {0.00019952620000000001`, 
        0.00001991143436248929`}, {0.00039810720000000004`, 
        0.000020053758266782947`}, {0.0007943282`, 
        0.000019850552626606393`}, {0.0015848930000000002`, 
        0.000019947330322023624`}, {0.003162278`, 
        0.000019974117558646464`}, {0.006309573000000001`, 
        0.000019848207127157575`}, {0.012589250000000001`, 
        0.000019920434186753536`}, {0.02511886`, 
        0.0000200280946016489`}, {0.050118720000000005`, 
        0.000020068996937685604`}, {0.1`, 
        0.000019437377264110624`}, {0.19952620000000001`, 
        0.00001712900695094354`}, {0.3981072`, 
        0.000024404374502666714`}, {0.7943282`, 
        0.000014252880064338575`}, {1.584893`, \(-6.321386854925451`*^-8\)}, \
{3.162278`, 1.5785944261912004`*^-7}, {6.309573`, 
        1.7283164832037337`*^-7}, {12.58925`, \(-1.4952867263164976`*^-8\)}, \
{25.11886`, \(-1.2919227439817007`*^-7\)}, {50.11872`, \
\(-4.59793166273163`*^-8\)}, {100.`, \(-8.046398691267909`*^-8\)}}\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata2t0, 2, 2, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.000023770631376121474`}, {0.00019952620000000001`, 
        0.000023835624256827408`}, {0.00039810720000000004`, 
        0.000023864567648834364`}, {0.0007943282`, 
        0.000023772620091166856`}, {0.0015848930000000002`, 
        0.00002376711099204167`}, {0.003162278`, 
        0.000023764586937196495`}, {0.006309573000000001`, 
        0.000023861234938940204`}, {0.012589250000000001`, 
        0.000023894954904062688`}, {0.02511886`, 
        0.000023666305197859396`}, {0.050118720000000005`, 
        0.000023451937227985002`}, {0.1`, 
        0.000022209935551527914`}, {0.19952620000000001`, 
        0.000016714963937117846`}, {0.3981072`, 
        0.00003712029313439846`}, {0.7943282`, 
        0.000013597203709570663`}, {1.584893`, \(-6.046364740153988`*^-8\)}, \
{3.162278`, \(-1.0532766553301808`*^-7\)}, {6.309573`, \
\(-7.476327384392123`*^-8\)}, {12.58925`, 
        2.3493972218154788`*^-8}, {25.11886`, 
        4.7141272527770783`*^-8}, {50.11872`, 
        2.0740582047002968`*^-7}, {100.`, 
        1.7532607309798053`*^-7}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata5t0, 2, 5, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.00003274847929026286`}, {0.00019952620000000001`, 
        0.00003274569262556896`}, {0.00039810720000000004`, 
        0.0000327468367197642`}, {0.0007943282`, 
        0.00003267002308128202`}, {0.0015848930000000002`, 
        0.000032690826387827825`}, {0.003162278`, 
        0.000032657488040330685`}, {0.006309573000000001`, 
        0.00003271867824937393`}, {0.012589250000000001`, 
        0.00003256815890082836`}, {0.02511886`, 
        0.00003239980072977564`}, {0.050118720000000005`, 
        0.000031396462360483954`}, {0.1`, 
        0.0000275413886668202`}, {0.19952620000000001`, 
        6.7727315633284224`*^-6}, {0.3981072`, 
        0.00004514003711103723`}, {0.7943282`, 
        0.000011128779705324301`}, {1.584893`, 
        4.217332271343003`*^-8}, {3.162278`, \(-2.2856146046621186`*^-7\)}, \
{6.309573`, 8.129875447284852`*^-8}, {12.58925`, 
        2.9756964200610675`*^-7}, {25.11886`, \(-2.6196795782227985`*^-7\)}, \
{50.11872`, 
        2.234544579534749`*^-8}, {100.`, \(-3.1278858174085003`*^-7\)}}\)], \
"Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata1tPi4, 2, 1, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.0000151089210300647`}, {0.00019952620000000001`, 
        0.000015050166095990623`}, {0.00039810720000000004`, 
        0.000015043700469576956`}, {0.0007943282`, 
        0.000015047205382975668`}, {0.0015848930000000002`, 
        0.00001506739729589582`}, {0.003162278`, 
        0.000015062671128418472`}, {0.006309573000000001`, 
        0.000015024936078136176`}, {0.012589250000000001`, 
        0.000015070751853024995`}, {0.02511886`, 
        0.000015115843641430236`}, {0.050118720000000005`, 
        0.000015172332752075127`}, {0.1`, 
        0.00001549167193693536`}, {0.19952620000000001`, 
        0.000018088634743864034`}, {0.3981072`, 
        0.000011381164180112356`}, {0.7943282`, 
        0.000014991204529324633`}, {1.584893`, 
        1.260035668550748`*^-7}, {3.162278`, \(-8.598181441473446`*^-9\)}, \
{6.309573`, 1.8309867966977194`*^-7}, {12.58925`, 
        2.6904155340807247`*^-8}, {25.11886`, \(-7.750039380516491`*^-8\)}, \
{50.11872`, \(-9.773307253659823`*^-8\)}, {100.`, 
        9.035979382077614`*^-8}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata2tPi4, 2, 2, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.000014816507623091855`}, {0.00019952620000000001`, 
        0.000014533627442817988`}, {0.00039810720000000004`, 
        0.000015072026877126332`}, {0.0007943282`, 
        0.00001474994574544565`}, {0.0015848930000000002`, 
        0.000014378501835900775`}, {0.003162278`, 
        0.000014895943021032226`}, {0.006309573000000001`, 
        0.000014982944095999867`}, {0.012589250000000001`, 
        0.000014906609449605131`}, {0.02511886`, 
        0.000015014241455431762`}, {0.050118720000000005`, 
        0.000014613151424158923`}, {0.1`, 
        0.000015128633746911178`}, {0.19952620000000001`, 
        0.000016590008840143023`}, {0.3981072`, 
        9.680288320563558`*^-6}, {0.7943282`, 
        0.00001632035143526522`}, {1.584893`, 
        9.062201780444666`*^-8}, {3.162278`, \(-2.1073299221071208`*^-7\)}, \
{6.309573`, \(-1.0578871129181392`*^-7\)}, {12.58925`, \
\(-9.147835150736279`*^-8\)}, {25.11886`, 
        1.481519187380009`*^-7}, {50.11872`, 1.951470235064995`*^-9}, {100.`, 
        9.634970724092801`*^-8}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata5tPi4, 2, 5, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.000013832399274108256`}, {0.00019952620000000001`, 
        0.000013636780635017909`}, {0.00039810720000000004`, 
        0.000013579476015966678`}, {0.0007943282`, 
        0.000014086476444604357`}, {0.0015848930000000002`, 
        0.000014008414628138811`}, {0.003162278`, 
        0.000013930095214671365`}, {0.006309573000000001`, 
        0.000013823717126883629`}, {0.012589250000000001`, 
        0.000013482977619762845`}, {0.02511886`, 
        0.000013702670990924658`}, {0.050118720000000005`, 
        0.00001354399287546186`}, {0.1`, 
        0.00001390342316325845`}, {0.19952620000000001`, 
        0.00001605512638106227`}, {0.3981072`, \(-0.00004580175793971603`\)}, \
{0.7943282`, 0.000019378865769623463`}, {1.584893`, 
        4.508741611982097`*^-8}, {3.162278`, 
        1.876678290116037`*^-7}, {6.309573`, \(-1.8778798198529723`*^-7\)}, \
{12.58925`, 1.7065287894160028`*^-7}, {25.11886`, 
        1.0542868437613093`*^-7}, {50.11872`, 
        2.3873597198894355`*^-7}, {100.`, \(-1.44434358139198`*^-7\)}}\)], \
"Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata1tPi2, 2, 1, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        8.216767286137652`*^-6}, {0.00019952620000000001`, \
\(-0.000054209509331192404`\)}, {0.00039810720000000004`, \
\(-0.000027797751910203044`\)}, {0.0007943282`, \
\(-0.000012416705179047564`\)}, {0.0015848930000000002`, 
        4.3814738630668055`*^-6}, {0.003162278`, 
        3.4770538109554422`*^-6}, {0.006309573000000001`, 
        7.085036203843731`*^-6}, {0.012589250000000001`, 
        7.65640541765335`*^-6}, {0.02511886`, 
        7.838687462072962`*^-6}, {0.050118720000000005`, 
        8.238018164735701`*^-6}, {0.1`, 
        8.284129521388625`*^-6}, {0.19952620000000001`, 
        8.027068166124087`*^-6}, {0.3981072`, 
        9.172597854314535`*^-6}, {0.7943282`, 
        0.000013261419165885053`}, {1.584893`, 
        1.6690007107753225`*^-7}, {3.162278`, \(-1.335775457400034`*^-7\)}, \
{6.309573`, \(-7.20751902151295`*^-8\)}, {12.58925`, \
\(-1.224156180323182`*^-7\)}, {25.11886`, \(-3.784655195412126`*^-8\)}, \
{50.11872`, \(-1.5024782401398547`*^-7\)}, {100.`, \(-1.3885458217419667`*^-7\
\)}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata2tPi2, 2, 2, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        7.723203318669806`*^-6}, {0.00019952620000000001`, \
\(-0.00005472958626554718`\)}, {0.00039810720000000004`, \
\(-0.000028346741425037026`\)}, {0.0007943282`, \
\(-0.000013156856587449628`\)}, {0.0015848930000000002`, 
        3.6942716298402358`*^-6}, {0.003162278`, 
        2.739685518173266`*^-6}, {0.006309573000000001`, 
        6.708531874671579`*^-6}, {0.012589250000000001`, 
        7.028512692193443`*^-6}, {0.02511886`, 
        7.432404543289149`*^-6}, {0.050118720000000005`, 
        7.680568934601986`*^-6}, {0.1`, 
        7.7164079138567`*^-6}, {0.19952620000000001`, 
        7.66851823936131`*^-6}, {0.3981072`, 
        8.558422416335805`*^-6}, {0.7943282`, 
        0.000012980624488275688`}, {1.584893`, 
        6.748381610636028`*^-8}, {3.162278`, \(-4.391990348285105`*^-8\)}, \
{6.309573`, \(-1.5749614407651266`*^-7\)}, {12.58925`, \
\(-2.3730620138176676`*^-7\)}, {25.11886`, 
        2.1720315977956218`*^-7}, {50.11872`, \(-2.055292210285759`*^-7\)}, \
{100.`, 1.7245188938063797`*^-8}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(aRetablegen[Redata5tPi2, 2, 5, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        6.853097526425594`*^-6}, {0.00019952620000000001`, \
\(-0.00005575185064539352`\)}, {0.00039810720000000004`, \
\(-0.00002897577609380243`\)}, {0.0007943282`, \
\(-0.000013555090042436245`\)}, {0.0015848930000000002`, 
        2.9604134088513503`*^-6}, {0.003162278`, 
        2.0303792108541703`*^-6}, {0.006309573000000001`, 
        6.012436959769686`*^-6}, {0.012589250000000001`, 
        6.349089876685645`*^-6}, {0.02511886`, 
        6.441992459059434`*^-6}, {0.050118720000000005`, 
        6.865851348002789`*^-6}, {0.1`, 
        7.142848703453958`*^-6}, {0.19952620000000001`, 
        7.187484393497755`*^-6}, {0.3981072`, 
        7.840138231599501`*^-6}, {0.7943282`, 
        0.000012351736893795225`}, {1.584893`, \(-3.103294211814249`*^-8\)}, \
{3.162278`, \(-2.16965235743322`*^-7\)}, {6.309573`, \
\(-2.8002566691865027`*^-7\)}, {12.58925`, \(-6.805039820289803`*^-8\)}, \
{25.11886`, \(-2.7026590518085505`*^-7\)}, {50.11872`, \
\(-2.045284983491576`*^-7\)}, {100.`, 2.3567797804107797`*^-8}}\)], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["beta-Im", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra1t0 = 
      Table[{\(Imdata1t0[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  1 - 10^\((\(-10\))\), \(Imdata1t0[\([i]\)]\)[\([1]\)], 
                  1] + \(Imdata1t0[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata1t0[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata1t0[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata1t0]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 1.3076554089149952`*^-7}, {0.00019952620000000001`, 
        6.551812248011325`*^-7}, {0.00039810720000000004`, \
\(-3.877935300306326`*^-7\)}, {0.0007943282`, 
        3.998589641089275`*^-8}, {0.0015848930000000002`, 
        1.4157268089359329`*^-7}, {0.003162278`, \
\(-2.7931008720455044`*^-7\)}, {0.006309573000000001`, 
        2.5267496503946036`*^-7}, {0.012589250000000001`, 
        9.83937723895395`*^-7}, {0.02511886`, 
        6.026028345375857`*^-7}, {0.050118720000000005`, 
        1.3445209989826058`*^-7}, {0.1`, 
        4.3610602057101546`*^-8}, {0.19952620000000001`, 
        3.743341319573853`*^-7}, {0.3981072`, \(-2.071252840447761`*^-7\)}, \
{0.7943282`, 1.4338762763027913`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra1tPi4 = 
      Table[{\(Imdata1tPi4[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  Cos[Pi/4], \(Imdata1tPi4[\([i]\)]\)[\([1]\)], 
                  1] + \(Imdata1tPi4[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata1tPi4[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata1tPi4[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata1tPi4]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 1.2896506836040595`*^-7}, {0.00019952620000000001`, 
        9.088480293146882`*^-7}, {0.00039810720000000004`, \
\(-1.9636548559931695`*^-7\)}, {0.0007943282`, 
        6.647910603502006`*^-8}, {0.0015848930000000002`, 
        3.10780176796436`*^-7}, {0.003162278`, \(-2.473942312835581`*^-7\)}, \
{0.006309573000000001`, 2.581950031789166`*^-7}, {0.012589250000000001`, 
        1.1679925620617769`*^-6}, {0.02511886`, 
        7.193442588368605`*^-7}, {0.050118720000000005`, 
        2.1511714509504783`*^-8}, {0.1`, 
        2.528701680125926`*^-7}, {0.19952620000000001`, 
        9.554037092241133`*^-7}, {0.3981072`, \(-2.5617774442530814`*^-7\)}, \
{0.7943282`, 6.629722183475828`*^-8}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra1tPi2 = 
      Table[{\(Imdata1tPi2[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  0, \(Imdata1tPi2[\([i]\)]\)[\([1]\)], 
                  1] + \(Imdata1tPi2[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata1tPi2[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata1tPi2[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata1tPi2]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 5.4768403543203894`*^-8}, {0.00019952620000000001`, 
        4.1126640231257657`*^-7}, {0.00039810720000000004`, \
\(-2.0012077809239916`*^-7\)}, {0.0007943282`, 
        1.3361279867970273`*^-7}, {0.0015848930000000002`, 
        4.735262972094978`*^-7}, {0.003162278`, \
\(-1.3276969588357765`*^-7\)}, {0.006309573000000001`, 
        2.270403531433025`*^-7}, {0.012589250000000001`, 
        8.670839943856342`*^-7}, {0.02511886`, 
        8.838977985389877`*^-7}, {0.050118720000000005`, 
        2.0587796453949732`*^-7}, {0.1`, 
        3.09505760008146`*^-8}, {0.19952620000000001`, 
        4.591130259016827`*^-7}, {0.3981072`, \(-2.2990303156292587`*^-7\)}, \
{0.7943282`, 1.6106993862394333`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra2t0 = 
      Table[{\(Imdata2t0[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  1 - 10^\((\(-10\))\), \(Imdata2t0[\([i]\)]\)[\([1]\)], 
                  2] + \(Imdata2t0[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata2t0[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata2t0[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata2t0]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 4.5229104896345486`*^-8}, {0.00019952620000000001`, 
        4.6821816319397536`*^-7}, {0.00039810720000000004`, \
\(-7.024693012946642`*^-8\)}, {0.0007943282`, 
        3.223978415795271`*^-8}, {0.0015848930000000002`, 
        3.685069770155076`*^-7}, {0.003162278`, \(-7.578600362959433`*^-8\)}, \
{0.006309573000000001`, 1.1035208625991886`*^-7}, {0.012589250000000001`, 
        9.578962533714865`*^-7}, {0.02511886`, 
        4.780817970698872`*^-7}, {0.050118720000000005`, 
        2.1799940664249957`*^-7}, {0.1`, \(-3.72071294018756`*^-8\)}, \
{0.19952620000000001`, 5.525935065984962`*^-7}, {0.3981072`, 
        1.1389434123687159`*^-7}, {0.7943282`, \(-3.060949105135388`*^-8\)}, \
{1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 0.`}, {12.58925`, 
        0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra2tPi4 = 
      Table[{\(Imdata2tPi4[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  Cos[Pi/4], \(Imdata2tPi4[\([i]\)]\)[\([1]\)], 
                  2] + \(Imdata2tPi4[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata2tPi4[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata2tPi4[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata1tPi2]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-5.3422984762398364`*^-8\)}, {0.00019952620000000001`, 
        5.197112671528583`*^-7}, {0.00039810720000000004`, \
\(-2.569034858428191`*^-7\)}, {0.0007943282`, 
        2.0995402128096557`*^-7}, {0.0015848930000000002`, 
        4.3510694726239283`*^-7}, {0.003162278`, \
\(-1.7011623452130816`*^-7\)}, {0.006309573000000001`, 
        3.009881756100817`*^-7}, {0.012589250000000001`, 
        1.1371084128561448`*^-6}, {0.02511886`, 
        5.794347591979011`*^-7}, {0.050118720000000005`, 
        3.3762256789538013`*^-7}, {0.1`, 
        3.618307166027964`*^-8}, {0.19952620000000001`, 
        4.3535287828838433`*^-7}, {0.3981072`, \(-3.0350578498214397`*^-7\)}, \
{0.7943282`, 1.600452724878707`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra2tPi2 = 
      Table[{\(Imdata2tPi2[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  0, \(Imdata2tPi2[\([i]\)]\)[\([1]\)], 
                  2] + \(Imdata2tPi2[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata2tPi2[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata2tPi2[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata2tPi2]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 1.4718708607529796`*^-8}, {0.00019952620000000001`, 
        4.903615241271717`*^-7}, {0.00039810720000000004`, \
\(-2.4021980849340385`*^-7\)}, {0.0007943282`, 
        1.61394430092245`*^-7}, {0.0015848930000000002`, 
        3.0925827011084636`*^-7}, {0.003162278`, \(-3.02940785954366`*^-7\)}, \
{0.006309573000000001`, 5.6783161296740663`*^-8}, {0.012589250000000001`, 
        1.1649398219734585`*^-6}, {0.02511886`, 
        4.994063341951396`*^-7}, {0.050118720000000005`, 
        2.0537605738164157`*^-7}, {0.1`, 
        3.079938156873764`*^-8}, {0.19952620000000001`, 
        5.308571473076846`*^-7}, {0.3981072`, \(-1.5757131135764263`*^-7\)}, \
{0.7943282`, 1.2655164449682811`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra5t0 = 
      Table[{\(Imdata5t0[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  1 - 10^\((\(-10\))\), \(Imdata5t0[\([i]\)]\)[\([1]\)], 
                  5] + \(Imdata5t0[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata5t0[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata5t0[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata5t0]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 8.931970662671753`*^-10}, {0.00019952620000000001`, 
        4.416954802401408`*^-7}, {0.00039810720000000004`, \
\(-2.9468282719154446`*^-7\)}, {0.0007943282`, 
        2.20760487415525`*^-7}, {0.0015848930000000002`, 
        2.4958030550099154`*^-7}, {0.003162278`, \(-4.75386778185531`*^-7\)}, \
{0.006309573000000001`, 2.178575721826251`*^-7}, {0.012589250000000001`, 
        1.2005725509264359`*^-6}, {0.02511886`, 
        3.2873707428480455`*^-7}, {0.050118720000000005`, 
        4.5710205941938474`*^-7}, {0.1`, \(-4.8884434666056684`*^-8\)}, \
{0.19952620000000001`, 3.0815710106647424`*^-7}, {0.3981072`, 
        3.551489866301384`*^-8}, {0.7943282`, 
        1.1812879153291239`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra5tPi4 = 
      Table[{\(Imdata5tPi4[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  Cos[Pi/4], \(Imdata5tPi4[\([i]\)]\)[\([1]\)], 
                  5] + \(Imdata5tPi4[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata5tPi4[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata5tPi4[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata5tPi2]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 1.532159193752475`*^-8}, {0.00019952620000000001`, 
        4.89098035692997`*^-7}, {0.00039810720000000004`, \
\(-7.009381819903892`*^-8\)}, {0.0007943282`, 
        1.4081057607176775`*^-7}, {0.0015848930000000002`, 
        1.8323606113611335`*^-7}, {0.003162278`, \
\(-3.9924527749893316`*^-7\)}, {0.006309573000000001`, \
\(-7.446591143314566`*^-8\)}, {0.012589250000000001`, 
        9.387972512507627`*^-7}, {0.02511886`, 
        5.51607702775975`*^-7}, {0.050118720000000005`, 
        1.8213603211146156`*^-7}, {0.1`, 
        1.9127706218212677`*^-8}, {0.19952620000000001`, 
        4.429049136810043`*^-7}, {0.3981072`, \(-1.73532849553974`*^-7\)}, \
{0.7943282`, 3.2985763621874865`*^-9}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(erra5tPi2 = 
      Table[{\(Imdata5tPi2[\([i]\)]\)[\([1]\)], \((ImbetaA[1, 1, 
                  Cos[Pi/2], \(Imdata5tPi2[\([i]\)]\)[\([1]\)], 
                  5] + \(Imdata5tPi2[\([i]\)]\)[\([3]\)])\)/
            If[\(Imdata5tPi2[\([i]\)]\)[\([3]\)] \[Equal] 0, 
              1, \(Imdata5tPi2[\([i]\)]\)[\([3]\)]]}, {i, 1, 
          Length[Imdata5tPi2]}]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-9.570541425140678`*^-8\)}, {0.00019952620000000001`, 
        6.208315594294232`*^-7}, {0.00039810720000000004`, \
\(-1.0291040743562545`*^-7\)}, {0.0007943282`, 
        1.4659319840076914`*^-7}, {0.0015848930000000002`, 
        2.927997727564669`*^-8}, {0.003162278`, \
\(-1.4010375859118748`*^-7\)}, {0.006309573000000001`, 
        2.4769123155058696`*^-7}, {0.012589250000000001`, 
        9.979048566375285`*^-7}, {0.02511886`, 
        4.490933140735577`*^-7}, {0.050118720000000005`, 
        1.6388886943646265`*^-7}, {0.1`, 
        1.254453131153039`*^-7}, {0.19952620000000001`, 
        3.552983642147076`*^-7}, {0.3981072`, \(-2.4010636104544065`*^-7\)}, \
{0.7943282`, 3.607705316755195`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["beta-Re", "Subsection"],

Cell[BoxData[
    \(Retablegen[dat_, pos_, a_, t_] := 
      Module[{res}, 
        res = Table[{\(dat[\([i]\)]\)[\([1]\)], \((Rebeta[1, 1, Cos[t], 
                      IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, 
                      a] - \(dat[\([i]\)]\)[\([pos]\)])\)/
                If[\(dat[\([i]\)]\)[\([pos]\)] \[Equal] 0, 
                  1, \(dat[\([i]\)]\)[\([pos]\)]]}, {i, 1, Length[dat]}]; \ 
        res]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata1t0, 3, 1, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        0.000025762982113331146`}, {0.00019952620000000001`, \
\(-0.00003129606025399159`\)}, {0.00039810720000000004`, \
\(-4.814317788496657`*^-6\)}, {0.0007943282`, 
        0.000010435077137985546`}, {0.0015848930000000002`, 
        0.00002711916708210646`}, {0.003162278`, 
        0.000025966745575963556`}, {0.006309573000000001`, 
        0.000030112709221729172`}, {0.012589250000000001`, 
        0.000030431009719886338`}, {0.02511886`, 
        0.00003077114244297755`}, {0.050118720000000005`, 
        0.0000308986081349647`}, {0.1`, 
        0.000030968583605346916`}, {0.19952620000000001`, 
        0.00003059726540355516`}, {0.3981072`, 
        0.000032424317326510615`}, {0.7943282`, \
\(-0.000014859419221952552`\)}, {1.584893`, \(-5.3888305367341454`*^-8\)}, \
{3.162278`, \(-5.288141821722688`*^-8\)}, {6.309573`, 
        9.098522271803766`*^-9}, {12.58925`, \(-1.3488318245411192`*^-7\)}, \
{25.11886`, 5.3849973816842903`*^-8}, {50.11872`, 
        9.899204036370644`*^-8}, {100.`, 
        1.5726386484538987`*^-7}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata1tPi4, 3, 1, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        0.000025689730612653928`}, {0.00019952620000000001`, \
\(-0.000042659391216001244`\)}, {0.00039810720000000004`, \
\(-0.000016172341631950645`\)}, {0.0007943282`, \
\(-1.0570635149178466`*^-6\)}, {0.0015848930000000002`, 
        0.000015757586413180132`}, {0.003162278`, 
        0.000014951217313818383`}, {0.006309573000000001`, 
        0.000018784875140974365`}, {0.012589250000000001`, 
        0.000019468278913494955`}, {0.02511886`, 
        0.000019394135021795872`}, {0.050118720000000005`, 
        0.000019930540274776303`}, {0.1`, 
        0.00002001323033350485`}, {0.19952620000000001`, 
        0.00002078074682580404`}, {0.3981072`, 
        0.00002496324184891577`}, {0.7943282`, 
        0.0005331893754679114`}, {1.584893`, 
        5.61932991176115`*^-8}, {3.162278`, \(-1.1941262634468445`*^-7\)}, \
{6.309573`, 
        1.1162406355839153`*^-7}, {12.58925`, \(-5.630795335994283`*^-8\)}, \
{25.11886`, 1.388489915484115`*^-7}, {50.11872`, 
        1.4195989144557415`*^-7}, {100.`, \(-1.2589876274279677`*^-7\)}}\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata1tPi2, 3, 1, Pi/2]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 
        0.000015060461124993053`}, {0.00019952620000000001`, \
\(-0.00004601262815491025`\)}, {0.00039810720000000004`, \
\(-0.00001998509675520499`\)}, {0.0007943282`, \(-4.195900927136498`*^-6\)}, \
{0.0015848930000000002`, 0.00001240228650120933`}, {0.003162278`, 
        0.000011619993713454206`}, {0.006309573000000001`, 
        0.000015521165146198402`}, {0.012589250000000001`, 
        0.000015636239529559735`}, {0.02511886`, 
        0.000016223126783904122`}, {0.050118720000000005`, 
        0.000016445163637294672`}, {0.1`, 
        0.000016657051982675356`}, {0.19952620000000001`, 
        0.0000172227042739934`}, {0.3981072`, 
        0.00002063869719942507`}, {0.7943282`, 
        0.00007750880023260229`}, {1.584893`, 
        7.770119891451216`*^-8}, {3.162278`, \(-4.277804221949868`*^-8\)}, \
{6.309573`, \(-7.759376116857736`*^-8\)}, {12.58925`, 
        1.2046409646889856`*^-7}, {25.11886`, \(-3.104213640288526`*^-8\)}, \
{50.11872`, \(-2.0374148924143464`*^-8\)}, {100.`, \(-1.4654174945289667`*^-8\
\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata2t0, 3, 2, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        0.00003316610985236621`}, {0.00019952620000000001`, \
\(-0.000022790027780224258`\)}, {0.00039810720000000004`, 
        3.548764708756893`*^-6}, {0.0007943282`, 
        0.00001892559777734528`}, {0.0015848930000000002`, 
        0.00003555608697797949`}, {0.003162278`, 
        0.000034475623972481144`}, {0.006309573000000001`, 
        0.000038476787736224045`}, {0.012589250000000001`, 
        0.00003895090193492184`}, {0.02511886`, 
        0.00003926779206252561`}, {0.050118720000000005`, 
        0.000039361103415743896`}, {0.1`, 
        0.00003944631453572698`}, {0.19952620000000001`, 
        0.000037644209490362856`}, {0.3981072`, 
        0.000036744476268698585`}, {0.7943282`, 
        4.022592090013087`*^-6}, {1.584893`, 
        8.159725431873743`*^-8}, {3.162278`, \(-8.78631991670101`*^-8\)}, \
{6.309573`, \(-1.588475786595482`*^-7\)}, {12.58925`, \
\(-2.3984808989048114`*^-8\)}, {25.11886`, \(-1.376961235298784`*^-7\)}, \
{50.11872`, \(-1.647427389001571`*^-7\)}, {100.`, 
        1.9434803394075362`*^-7}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata2tPi4, 3, 2, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        0.0000177969398956992`}, {0.00019952620000000001`, \
\(-0.00004518601891440921`\)}, {0.00039810720000000004`, \
\(-0.000018068254858421687`\)}, {0.0007943282`, \
\(-2.8875528294727075`*^-6\)}, {0.0015848930000000002`, 
        0.00001374084247126672`}, {0.003162278`, 
        0.000012853659427214802`}, {0.006309573000000001`, 
        0.00001683233210433052`}, {0.012589250000000001`, 
        0.000016859232388187207`}, {0.02511886`, 
        0.00001743982366971563`}, {0.050118720000000005`, 
        0.000017922293037171984`}, {0.1`, 
        0.000018140801173870077`}, {0.19952620000000001`, 
        0.000019169887886558664`}, {0.3981072`, 
        0.000025346827513296003`}, {0.7943282`, \(-0.0003802467744923348`\)}, \
{1.584893`, 
        1.0106046853775362`*^-7}, {3.162278`, \(-1.0354554634884946`*^-7\)}, \
{6.309573`, \(-8.788832396207787`*^-8\)}, {12.58925`, \
\(-2.6449680088873493`*^-9\)}, {25.11886`, 
        1.7896894784643832`*^-7}, {50.11872`, 
        8.896968431088664`*^-8}, {100.`, 
        1.7852753451341478`*^-7}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata2tPi2, 3, 2, Pi/2]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 
        0.000013867514226870237`}, {0.00019952620000000001`, \
\(-0.000047764087195720966`\)}, {0.00039810720000000004`, \
\(-0.00002073991690393424`\)}, {0.0007943282`, \(-5.394496849742958`*^-6\)}, \
{0.0015848930000000002`, 0.000011217894316298562`}, {0.003162278`, 
        0.00001039068951605273`}, {0.006309573000000001`, 
        0.000014307571999614097`}, {0.012589250000000001`, 
        0.000014645715173924242`}, {0.02511886`, 
        0.00001492260261329873`}, {0.050118720000000005`, 
        0.000015499491445625587`}, {0.1`, 
        0.000015423786861126994`}, {0.19952620000000001`, 
        0.000015769420618710063`}, {0.3981072`, 
        0.000019122615602081092`}, {0.7943282`, 
        0.0000660639749960852`}, {1.584893`, 
        9.442024150152413`*^-8}, {3.162278`, 
        1.9074135171067928`*^-7}, {6.309573`, 
        1.922299155192093`*^-7}, {12.58925`, 
        1.7397484502470355`*^-7}, {25.11886`, 
        1.3602834649859138`*^-7}, {50.11872`, \(-1.7727906534011087`*^-7\)}, \
{100.`, \(-1.649168710510947`*^-8\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata5t0, 3, 5, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        0.00005877608924361077`}, {0.00019952620000000001`, \
\(-3.7503486339745064`*^-6\)}, {0.00039810720000000004`, 
        0.00002255408193894849`}, {0.0007943282`, 
        0.000037970545413357`}, {0.0015848930000000002`, 
        0.00005465074333531409`}, {0.003162278`, 
        0.000053870966680977634`}, {0.006309573000000001`, 
        0.000057625186728658135`}, {0.012589250000000001`, 
        0.00005796544188599472`}, {0.02511886`, 
        0.00005811639224327969`}, {0.050118720000000005`, 
        0.00005785645645366657`}, {0.1`, 
        0.00005641241060304612`}, {0.19952620000000001`, 
        0.00005020104810890706`}, {0.3981072`, 
        0.00006491810545193919`}, {0.7943282`, 
        9.442155225644211`*^-6}, {1.584893`, 
        1.5894259477847585`*^-7}, {3.162278`, 
        2.0192829312570454`*^-7}, {6.309573`, 
        2.1393029798044233`*^-7}, {12.58925`, \(-1.0852712092431393`*^-7\)}, \
{25.11886`, \(-2.2209901726782715`*^-7\)}, {50.11872`, \
\(-1.4297060737921302`*^-7\)}, {100.`, \(-1.8139583646384758`*^-7\)}}\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata5tPi4, 3, 5, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        0.000012994839985675357`}, {0.00019952620000000001`, \
\(-0.00004932400798364942`\)}, {0.00039810720000000004`, \
\(-0.000022956136494029175`\)}, {0.0007943282`, \(-7.589638303896231`*^-6\)}, \
{0.0015848930000000002`, 9.172678357599866`*^-6}, {0.003162278`, 
        8.136795132447487`*^-6}, {0.006309573000000001`, 
        0.000011847091789087883`}, {0.012589250000000001`, 
        0.000012434991335087334`}, {0.02511886`, 
        0.00001287871738872582`}, {0.050118720000000005`, 
        0.000013084184216996728`}, {0.1`, 
        0.000013554760099478588`}, {0.19952620000000001`, 
        0.000014545411083672454`}, {0.3981072`, 
        0.00002254889930259442`}, {0.7943282`, \
\(-0.000027652848002420395`\)}, {1.584893`, 
        8.26644564257841`*^-8}, {3.162278`, \(-1.1488280083385335`*^-7\)}, \
{6.309573`, 5.381767779925422`*^-8}, {12.58925`, 
        1.5134960635652603`*^-7}, {25.11886`, \(-2.45683470001442`*^-7\)}, \
{50.11872`, \(-1.819940496128483`*^-7\)}, {100.`, 
        2.2916169366085647`*^-8}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Retablegen[Redata5tPi2, 3, 5, Pi/2]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 
        0.000013019357548132969`}, {0.00019952620000000001`, \
\(-0.00004985331633963557`\)}, {0.00039810720000000004`, \
\(-0.00002212907966933977`\)}, {0.0007943282`, \(-6.608302507752787`*^-6\)}, \
{0.0015848930000000002`, 9.955128698840949`*^-6}, {0.003162278`, 
        9.12163504550986`*^-6}, {0.006309573000000001`, 
        0.000012698986706268321`}, {0.012589250000000001`, 
        0.000013292035596560277`}, {0.02511886`, 
        0.000013514944643971105`}, {0.050118720000000005`, 
        0.000013936081737032089`}, {0.1`, 
        0.000014090653642240308`}, {0.19952620000000001`, 
        0.00001422052672643502`}, {0.3981072`, 
        0.000017273414570043176`}, {0.7943282`, 
        0.000054691387894602115`}, {1.584893`, 
        1.8645921392918553`*^-7}, {3.162278`, \(-2.77205746844216`*^-7\)}, \
{6.309573`, 
        1.6338258298923416`*^-7}, {12.58925`, \(-9.52642185831235`*^-8\)}, \
{25.11886`, 
        2.580096046552797`*^-7}, {50.11872`, \(-5.836812864966284`*^-8\)}, \
{100.`, \(-6.052119626616783`*^-8\)}}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["gamma-Im", "Subsection"],

Cell[BoxData[
    \(gImtablegen[dat_, pos_, a_, t_] := 
      Module[{res}, 
        res = Table[{\(dat[\([i]\)]\)[\([1]\)], \((Imgamma[1, 1, Cos[t], 
                      IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, 
                      a] + \(dat[\([i]\)]\)[\([pos]\)])\)/
                If[\(dat[\([i]\)]\)[\([pos]\)] \[Equal] 0, 
                  1, \(dat[\([i]\)]\)[\([pos]\)]]}, {i, 1, Length[dat]}]; \ 
        res]\)], "Input"],

Cell[BoxData[
    \(Imgamma\ fuer\ t = 0\ \(verschwindet!\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(gImtablegen[Imdata1tPi4, 4, 1, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 5.742150171583162`*^-9}, {0.00019952620000000001`, 
        0.00003125290911361291`}, {0.00039810720000000004`, 
        0.000017723304433854488`}, {0.0007943282`, 
        0.000010404399657139789`}, {0.0015848930000000002`, 
        2.036272949414724`*^-6}, {0.003162278`, 
        2.666884706498487`*^-6}, {0.006309573000000001`, 
        4.526926454642526`*^-7}, {0.012589250000000001`, 
        3.8694261460799877`*^-7}, {0.02511886`, 
        6.254093176174461`*^-7}, {0.050118720000000005`, 
        3.153076919306579`*^-8}, {0.1`, \(-5.686803148936104`*^-8\)}, \
{0.19952620000000001`, 
        1.6530991606880489`*^-7}, {0.3981072`, \(-3.3903825184856423`*^-7\)}, \
{0.7943282`, \(-3.400191506194808`*^-7\)}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gImtablegen[Imdata2tPi4, 4, 2, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 8.518506866050621`*^-8}, {0.00019952620000000001`, 
        0.00003132509436853535`}, {0.00039810720000000004`, 
        0.000018136120920662627`}, {0.0007943282`, 
        0.000010345270709610305`}, {0.0015848930000000002`, 
        1.7168862533304195`*^-6}, {0.003162278`, 
        2.43220569857969`*^-6}, {0.006309573000000001`, 
        5.345109515925593`*^-7}, {0.012589250000000001`, 
        3.090419085371851`*^-7}, {0.02511886`, 
        2.436082736431286`*^-7}, {0.050118720000000005`, 
        1.509982751835219`*^-7}, {0.1`, \(-3.948069461641207`*^-8\)}, \
{0.19952620000000001`, \(-9.086156250183143`*^-9\)}, {0.3981072`, \
\(-1.5522174632115028`*^-7\)}, {0.7943282`, \(-3.1262201222894504`*^-6\)}, \
{1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 0.`}, {12.58925`, 
        0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gImtablegen[Imdata5tPi4, 4, 5, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 2.151782218565792`*^-8}, {0.00019952620000000001`, 
        0.00003150025946263787`}, {0.00039810720000000004`, 
        0.000017890288259663843`}, {0.0007943282`, 
        0.000010417694073666342`}, {0.0015848930000000002`, 
        1.9016196297529406`*^-6}, {0.003162278`, 
        2.262433723720381`*^-6}, {0.006309573000000001`, 
        5.932262375756601`*^-7}, {0.012589250000000001`, 
        1.744193747310496`*^-7}, {0.02511886`, 
        3.781683043341124`*^-7}, {0.050118720000000005`, 
        1.5991862359022312`*^-7}, {0.1`, 
        2.3225922882487646`*^-8}, {0.19952620000000001`, 
        3.853830545261782`*^-7}, {0.3981072`, 
        8.255784655484727`*^-8}, {0.7943282`, 
        2.2319069474631414`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gImtablegen[Imdata1tPi2, 4, 1, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 7.143506954416794`*^-8}, {0.00019952620000000001`, 
        0.000031270359761106726`}, {0.00039810720000000004`, 
        0.00001846095812865273`}, {0.0007943282`, 
        0.000010203119768498843`}, {0.0015848930000000002`, 
        2.125876665458623`*^-6}, {0.003162278`, 
        2.462037663089258`*^-6}, {0.006309573000000001`, 
        3.6035289421392853`*^-7}, {0.012589250000000001`, 
        3.7373677224129465`*^-7}, {0.02511886`, 
        2.2873903732783915`*^-7}, {0.050118720000000005`, 
        3.368395723313101`*^-7}, {0.1`, \(-1.3735395816792794`*^-7\)}, \
{0.19952620000000001`, 
        9.103658593512448`*^-8}, {0.3981072`, \(-2.2791297417026604`*^-8\)}, \
{0.7943282`, \(-2.600550086818947`*^-7\)}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gImtablegen[Imdata2tPi2, 4, 2, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 2.9718708568747412`*^-8}, {0.00019952620000000001`, 
        0.00003129382877873182`}, {0.00039810720000000004`, 
        0.000017709153256329193`}, {0.0007943282`, 
        0.000010409865621102693`}, {0.0015848930000000002`, 
        1.9294186822913238`*^-6}, {0.003162278`, 
        2.3783605225842674`*^-6}, {0.006309573000000001`, 
        3.105151860298515`*^-7}, {0.012589250000000001`, 
        3.263745014841524`*^-7}, {0.02511886`, 
        1.206354698872442`*^-7}, {0.050118720000000005`, \
\(-3.402033829598314`*^-8\)}, {0.1`, 
        1.6292910144541914`*^-7}, {0.19952620000000001`, 
        2.161276343036133`*^-7}, {0.3981072`, \(-3.514244287763823`*^-7\)}, \
{0.7943282`, \(-1.4475491955579128`*^-7\)}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gImtablegen[Imdata5tPi2, 4, 5, Pi/2]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 7.754650164838537`*^-8}, {0.00019952620000000001`, 
        0.000031142846336618376`}, {0.00039810720000000004`, 
        0.00001820102161467785`}, {0.0007943282`, 
        0.000010268304787486454`}, {0.0015848930000000002`, 
        2.001817783844597`*^-6}, {0.003162278`, 
        2.4771687346997`*^-6}, {0.006309573000000001`, 
        3.5735004131037494`*^-7}, {0.012589250000000001`, 
        2.785193504745722`*^-7}, {0.02511886`, 
        2.170149324950526`*^-7}, {0.050118720000000005`, \
\(-2.95317166419869`*^-8\)}, {0.1`, 
        5.5859987196482796`*^-8}, {0.19952620000000001`, 
        1.5978934390786146`*^-7}, {0.3981072`, \(-6.306353807580183`*^-8\)}, \
{0.7943282`, \(-1.3627658219028231`*^-7\)}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["gamma-Re", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(gRetablegen[dat_, pos_, a_, t_] := 
      Module[{res}, 
        res = Table[{\(dat[\([i]\)]\)[\([1]\)], \((Regamma[1, 1, Cos[t], 
                      IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, 
                      a] - \(dat[\([i]\)]\)[\([pos]\)])\)/
                If[\(dat[\([i]\)]\)[\([pos]\)] \[Equal] 0, 
                  1, \(dat[\([i]\)]\)[\([pos]\)]]}, {i, 1, Length[dat]}]; \ 
        res]\)], "Input"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(gRetablegen\)\" is \
similar to existing symbols \!\({aRetablegen, Retablegen}\)."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Regamma[1, 1, 1 - 10^\((\(-20\))\), 3/2, 1]\)], "Input"],

Cell[BoxData[
    \(4.06068599858`-1.7385*^-22\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Regamma[1, 1, 1 - 10^\((\(-20\))\), 3/2, 1]\)], "Input"],

Cell[BoxData[
    \(4.06068599857`-1.4481*^-22\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Regamma[1, 1, Cos[Pi/4], 10^\((2)\), 1]\)], "Input"],

Cell[BoxData[
    \(0.0176978233880099523622970554`13.7096\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gRetablegen[Redata1tPi4, 4, 1, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.00001925174558081939`}, {0.00019952620000000001`, 
        0.000019782139092043535`}, {0.00039810720000000004`, 
        0.000019755007905607226`}, {0.0007943282`, 
        0.00001966054389279654`}, {0.0015848930000000002`, 
        0.000019338626058401966`}, {0.003162278`, 
        0.00001971010539051415`}, {0.006309573000000001`, 
        0.000019224712610687945`}, {0.012589250000000001`, 
        0.000019551235155004884`}, {0.02511886`, 
        0.00001958040069914481`}, {0.050118720000000005`, 
        0.000019608039779167105`}, {0.1`, 
        0.000019275651689558526`}, {0.19952620000000001`, 
        0.000018363725664200723`}, {0.3981072`, \(-7.843510435496787`*^-7\)}, \
{0.7943282`, 0.00003953605335575324`}, {1.584893`, 
        2.677438721139351`*^-8}, {3.162278`, 
        1.3807533342374564`*^-7}, {6.309573`, 
        6.493832040384328`*^-8}, {12.58925`, 
        2.5734390649986647`*^-7}, {25.11886`, 
        8.091768244030526`*^-8}, {50.11872`, 
        1.0742795160066024`*^-7}, {100.`, 
        1.9143656972715226`*^-7}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gRetablegen[Redata2tPi4, 4, 2, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.000020204431386619628`}, {0.00019952620000000001`, 
        0.000020015886040486455`}, {0.00039810720000000004`, 
        0.00002017745255289045`}, {0.0007943282`, 
        0.000020381784827938786`}, {0.0015848930000000002`, 
        0.000020343282638581063`}, {0.003162278`, 
        0.000019959124552196786`}, {0.006309573000000001`, 
        0.000020237783230589255`}, {0.012589250000000001`, 
        0.000020022637092785675`}, {0.02511886`, 
        0.00002019169810970634`}, {0.050118720000000005`, 
        0.00002003828339678803`}, {0.1`, 
        0.000019841130266007537`}, {0.19952620000000001`, 
        0.000019123392179698347`}, {0.3981072`, \(-2.619995517644204`*^-6\)}, \
{0.7943282`, 
        0.00003391333581760589`}, {1.584893`, \(-3.75694650203948`*^-7\)}, \
{3.162278`, \(-1.1781141854164896`*^-7\)}, {6.309573`, \
\(-1.0408724974477098`*^-7\)}, {12.58925`, 
        1.887866703486603`*^-8}, {25.11886`, 
        1.226249302661169`*^-8}, {50.11872`, 8.150415290873155`*^-8}, {100.`, 
        7.775261386644027`*^-8}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gRetablegen[Redata5tPi4, 4, 5, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 0.0000196871866983722`}, {0.00019952620000000001`, 
        0.000019526497505048787`}, {0.00039810720000000004`, 
        0.000019538587290061118`}, {0.0007943282`, 
        0.000019599026798233557`}, {0.0015848930000000002`, 
        0.00001956290141210332`}, {0.003162278`, 
        0.000019610646905383056`}, {0.006309573000000001`, 
        0.000019587827866154977`}, {0.012589250000000001`, 
        0.000019617443130949847`}, {0.02511886`, 
        0.00001964945036653711`}, {0.050118720000000005`, 
        0.000019553466855372568`}, {0.1`, 
        0.000019953635579702983`}, {0.19952620000000001`, 
        0.000020444208362181796`}, {0.3981072`, 
        0.000012563394330434125`}, {0.7943282`, 
        0.00002063426622409906`}, {1.584893`, \(-1.8610295825627378`*^-7\)}, \
{3.162278`, \(-1.056270034647383`*^-7\)}, {6.309573`, \
\(-2.6926933412005083`*^-8\)}, {12.58925`, 
        1.4094492859888477`*^-7}, {25.11886`, \(-7.96127108951661`*^-8\)}, \
{50.11872`, 
        4.4850744942662597`*^-8}, {100.`, \(-5.1551194395328235`*^-8\)}}\)], \
"Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gRetablegen[Redata1tPi2, 4, 1, Pi/2]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 0.000012458341772074534`}, {0.00019952620000000001`, 
        0.000012376310335841954`}, {0.00039810720000000004`, 
        0.000012049718952053093`}, {0.0007943282`, 
        0.000012151105790250106`}, {0.0015848930000000002`, 
        0.000012581182936356854`}, {0.003162278`, 
        0.000012297300357757427`}, {0.006309573000000001`, 
        0.000012253483929973786`}, {0.012589250000000001`, 
        0.000012250252924838057`}, {0.02511886`, 
        0.00001243795689426589`}, {0.050118720000000005`, 
        0.000012130157939036468`}, {0.1`, 
        0.000012425173523013856`}, {0.19952620000000001`, 
        0.00001233090731893383`}, {0.3981072`, 
        0.000011479849817143293`}, {0.7943282`, 
        0.00006523430769259416`}, {1.584893`, 
        8.545614628492512`*^-8}, {3.162278`, \(-1.1237763816043181`*^-7\)}, \
{6.309573`, \(-5.53599481331659`*^-8\)}, {12.58925`, \
\(-9.366148972354871`*^-8\)}, {25.11886`, \(-4.808068539451866`*^-8\)}, \
{50.11872`, \(-1.1456976694961122`*^-7\)}, {100.`, \(-5.3687684828180266`*^-8\
\)}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gRetablegen[Redata2tPi2, 4, 2, Pi/2]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 0.000011449324222968436`}, {0.00019952620000000001`, 
        0.000011380798162982613`}, {0.00039810720000000004`, 
        0.000011107975665231909`}, {0.0007943282`, 
        0.00001119084769527775`}, {0.0015848930000000002`, 
        0.000010958316436542097`}, {0.003162278`, 
        0.000011279212795526058`}, {0.006309573000000001`, 
        0.00001113537900486541`}, {0.012589250000000001`, 
        0.000011283137675154365`}, {0.02511886`, 
        0.000011354137357782116`}, {0.050118720000000005`, 
        0.00001137261159906024`}, {0.1`, 
        0.000011300683164810833`}, {0.19952620000000001`, 
        0.000011160009812929693`}, {0.3981072`, 
        0.00001149600919270947`}, {0.7943282`, 
        0.00007964071232929451`}, {1.584893`, 
        4.867415398674228`*^-8}, {3.162278`, 
        9.61906714585583`*^-8}, {6.309573`, 
        2.35397364927916`*^-9}, {12.58925`, \(-3.896051017588135`*^-8\)}, \
{25.11886`, \(-3.840532016903814`*^-8\)}, {50.11872`, 
        7.022291855273895`*^-8}, {100.`, 7.758500754275262`*^-9}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gRetablegen[Redata5tPi2, 4, 5, Pi/2]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 9.570488993450676`*^-6}, {0.00019952620000000001`, 
        9.516869592074977`*^-6}, {0.00039810720000000004`, 
        9.87853830078667`*^-6}, {0.0007943282`, 
        9.60381357100614`*^-6}, {0.0015848930000000002`, 
        9.671231280861236`*^-6}, {0.003162278`, 
        0.000010005053355692953`}, {0.006309573000000001`, 
        9.869766318308934`*^-6}, {0.012589250000000001`, 
        9.771733109162312`*^-6}, {0.02511886`, 
        0.000010019173573703147`}, {0.050118720000000005`, 
        0.000010052151285042912`}, {0.1`, 
        9.715614534769746`*^-6}, {0.19952620000000001`, 
        0.000010025254194905367`}, {0.3981072`, 
        0.000010532349557210487`}, {0.7943282`, \(-0.0004448423289448457`\)}, \
{1.584893`, \(-9.142319308423741`*^-8\)}, {3.162278`, \
\(-6.702889265112173`*^-8\)}, {6.309573`, \(-6.492092440923022`*^-8\)}, \
{12.58925`, 6.552827449551908`*^-8}, {25.11886`, 
        7.395132664475627`*^-8}, {50.11872`, 5.315737234089139`*^-8}, {100.`, 
        4.225984165036179`*^-8}}\)], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["delta-Im", "Subsection"],

Cell[BoxData[
    \(Imtablegen[dat_, pos_, a_, t_] := 
      Module[{res}, 
        res = Table[{\(dat[\([i]\)]\)[\([1]\)], \((Imdelta[1, 1, Cos[t], 
                      IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, 
                      a] + \(dat[\([i]\)]\)[\([pos]\)])\)/
                If[\(dat[\([i]\)]\)[\([pos]\)] \[Equal] 0, 
                  1, \(dat[\([i]\)]\)[\([pos]\)]]}, {i, 1, Length[dat]}]; \ 
        res]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imdelta[1, 1, Cos[10^\((\(-5\))\)], 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.000078539807684661378395546`10.3468\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[ImdeltaA[1, 1, Cos[10^\((\(-5\))\)], 10^\((\(-4\))\), 1]]\)], "Input"],

Cell[BoxData[
    \(\(-0.000099999998995`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imtablegen[Imdata1t0, 5, 1, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, \(-0.0005647165972528099`\)}, {0.00019952620000000001`, \
\(-0.0010960956134228605`\)}, {0.00039810720000000004`, \
\(-0.002234076633312061`\)}, {0.0007943282`, \(-0.004493330939889114`\)}, \
{0.0015848930000000002`, \(-0.009024217430537499`\)}, {0.003162278`, \
\(-0.01817221911207056`\)}, {0.006309573000000001`, \
\(-0.03694093771963058`\)}, {0.012589250000000001`, \
\(-0.07660351899657965`\)}, {0.02511886`, \(-0.16624036222078747`\)}, \
{0.050118720000000005`, \(-0.4065390894097305`\)}, {0.1`, \
\(-1.6046706790403191`\) + 0.`\ \[ImaginaryI]}, {0.19952620000000001`, 
        2.5226618384403485`}, {0.3981072`, 0.9628713799094838`}, {0.7943282`, 
        0.7789027753511889`}, {1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 
        0.`}, {12.58925`, 0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 
        0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imtablegen[Imdata2t0, 5, 2, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, \(-0.00042356959994877533`\)}, {0.00019952620000000001`, \
\(-0.0008142134660097363`\)}, {0.00039810720000000004`, \
\(-0.0016701107039661296`\)}, {0.0007943282`, \(-0.0033634832226222956`\)}, \
{0.0015848930000000002`, \(-0.006752880470152425`\)}, {0.003162278`, \
\(-0.013567612152145468`\)}, {0.006309573000000001`, \(-0.027455294517040985`\
\)}, {0.012589250000000001`, \(-0.05640939214695933`\)}, {0.02511886`, \
\(-0.12004468964368238`\)}, {0.050118720000000005`, \
\(-0.2804101394851844`\)}, {0.1`, \(-0.9311459066188317`\) + 
          0.`\ \[ImaginaryI]}, {0.19952620000000001`, 
        2.831176886373153`}, {0.3981072`, 0.8053212198160102`}, {0.7943282`, 
        0.7021990140305032`}, {1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 
        0.`}, {12.58925`, 0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 
        0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imtablegen[Imdata5t0, 5, 5, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, \(-0.00033890935016017275`\)}, {0.00019952620000000001`, \
\(-0.0006449700727830267`\)}, {0.00039810720000000004`, \
\(-0.0013318664866529273`\)}, {0.0007943282`, \(-0.0026868656756715154`\)}, \
{0.0015848930000000002`, \(-0.005394638360178832`\)}, {0.003162278`, \
\(-0.010825702851462272`\)}, {0.006309573000000001`, \
\(-0.02185508888324551`\)}, {0.012589250000000001`, \
\(-0.044713403042225555`\)}, {0.02511886`, \(-0.09457127009510205`\)}, \
{0.050118720000000005`, \(-0.22055997180467085`\)}, {0.1`, \
\(-0.7895685068989138`\) + 0.`\ \[ImaginaryI]}, {0.19952620000000001`, 
        1.417662083511931`}, {0.3981072`, 0.567057333845921`}, {0.7943282`, 
        0.6260804037053117`}, {1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 
        0.`}, {12.58925`, 0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 
        0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imtablegen[Imdata1tPi4, 5, 1, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 6.638537432271382`*^-8}, {0.00019952620000000001`, 
        0.000031274001511608715`}, {0.00039810720000000004`, 
        0.000018290249580505262`}, {0.0007943282`, 
        0.00001022463358376234`}, {0.0015848930000000002`, 
        1.9559830884034944`*^-6}, {0.003162278`, 
        2.4321072406839202`*^-6}, {0.006309573000000001`, 
        5.306184092472875`*^-7}, {0.012589250000000001`, 
        4.405468810073044`*^-7}, {0.02511886`, 
        2.2777935796440327`*^-7}, {0.050118720000000005`, 
        2.2299966065391066`*^-7}, {0.1`, \(-7.103474310950165`*^-8\)}, \
{0.19952620000000001`, 1.5921825209582707`*^-7}, {0.3981072`, 
        2.409208540593867`*^-7}, {0.7943282`, \(-1.5361747765594198`*^-8\)}, \
{1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 0.`}, {12.58925`, 
        0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imtablegen[Imdata2tPi4, 5, 2, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 8.535381251567859`*^-8}, {0.00019952620000000001`, 
        0.00003128510619020305`}, {0.00039810720000000004`, 
        0.000017856285993183996`}, {0.0007943282`, 
        0.000010348636511039071`}, {0.0015848930000000002`, 
        1.9448044191226754`*^-6}, {0.003162278`, 
        2.090063711551148`*^-6}, {0.006309573000000001`, 
        3.6148095807200374`*^-7}, {0.012589250000000001`, 
        2.9373403652453796`*^-7}, {0.02511886`, 
        1.268295330025634`*^-7}, {0.050118720000000005`, 
        2.842861473325207`*^-7}, {0.1`, \(-6.952015774265342`*^-8\)}, \
{0.19952620000000001`, 8.326435437778597`*^-8}, {0.3981072`, 
        9.632838462145169`*^-8}, {0.7943282`, \(-1.70571678241569`*^-7\)}, \
{1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 0.`}, {12.58925`, 
        0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imtablegen[Imdata5tPi4, 5, 5, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-8.848464177595584`*^-8\)}, {0.00019952620000000001`, 
        0.00003125037991420031`}, {0.00039810720000000004`, 
        0.000018030413100295015`}, {0.0007943282`, 
        0.00001040900315907101`}, {0.0015848930000000002`, 
        1.97913535932747`*^-6}, {0.003162278`, 
        2.229744251902366`*^-6}, {0.006309573000000001`, 
        4.947007051685712`*^-7}, {0.012589250000000001`, 
        3.7043678470871676`*^-7}, {0.02511886`, 
        5.3276178631318634`*^-8}, {0.050118720000000005`, 
        6.009078302274758`*^-8}, {0.1`, 
        4.158012797381813`*^-8}, {0.19952620000000001`, 
        8.065682671999188`*^-8}, {0.3981072`, 
        1.3592457620458238`*^-7}, {0.7943282`, 
        1.3099075671990632`*^-7}, {1.584893`, 0.`}, {3.162278`, 
        0.`}, {6.309573`, 0.`}, {12.58925`, 0.`}, {25.11886`, 
        0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imtablegen[Imdata1tPi2, 5, 1, 
      Pi/2*\((1 - 10^\((\(-10\))\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 158.02199691143062`}, {0.00019952620000000001`, 
        157.96408046616722`}, {0.00039810720000000004`, 
        157.8604468413922`}, {0.0007943282`, 
        157.65132642076176`}, {0.0015848930000000002`, 
        157.23450711079562`}, {0.003162278`, 
        156.40679360773257`}, {0.006309573000000001`, 
        154.78154016433402`}, {0.012589250000000001`, 
        151.6351631792907`}, {0.02511886`, 
        145.71196441411712`}, {0.050118720000000005`, 
        135.07834574958252`}, {0.1`, 
        117.28547726284663`}, {0.19952620000000001`, 
        89.42739517447733`}, {0.3981072`, 
        34.685543239125366`}, {0.7943282`, \(-25.777458178156504`\)}, \
{1.584893`, 0.`}, {3.162278`, 0.`}, {6.309573`, 0.`}, {12.58925`, 
        0.`}, {25.11886`, 0.`}, {50.11872`, 0.`}, {100.`, 0.`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imdelta[1, 1, Cos[Pi/2], 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(0``41.1157\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imdelta[1, 1, Cos[Pi/2], 10^\((\(-1\))\), 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(0``36.6342\)], "Output"]
}, Closed]],

Cell[BoxData[
    \(Imdelta\ of\ t = Pi/2 \((kz \[Rule] 0)\)\ is\ \(\(0!!\)!!\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["delta-Re", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(dRetablegen[dat_, pos_, a_, t_] := 
      Module[{res}, 
        res = Table[{\(dat[\([i]\)]\)[\([1]\)], \((Redelta[1, 1, Cos[t], 
                      IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, 
                      a] - \(dat[\([i]\)]\)[\([pos]\)])\)/
                If[\(dat[\([i]\)]\)[\([pos]\)] \[Equal] 0, 
                  1, \(dat[\([i]\)]\)[\([pos]\)]]}, {i, 1, Length[dat]}]; \ 
        res]\)], "Input"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(dRetablegen\)\" is \
similar to existing symbols \!\({aRetablegen, gRetablegen, \
Retablegen}\)."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Redelta[1, 1, 1 - 10^\((\(-16\))\), 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(3.32079615456776943800540898114886456`22.1222*^-8\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% - 1.56766*10^\((\(-4\))\)\)], "Input"],

Cell[BoxData[
    \(\(-0.00015673279203845433`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%226/\((\(-1.56766\)*10^\((\(-4\))\))\)\)], "Input"],

Cell[BoxData[
    \(0.9997881685981294`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(dRetablegen[Redata1t0, 5, 1, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-0.9997881690035059`\)}, {0.00019952620000000001`, \
\(-0.9995774572319206`\)}, {0.00039810720000000004`, \
\(-0.999157234265239`\)}, {0.0007943282`, \(-0.9983198827553096`\)}, \
{0.0015848930000000002`, \(-0.9966531942431971`\)}, {0.003162278`, \
\(-0.993344111724979`\)}, {0.006309573000000001`, \(-0.9868081659316851`\)}, \
{0.012589250000000001`, \(-0.9740173222343388`\)}, {0.02511886`, \
\(-0.949474010726578`\)}, {0.050118720000000005`, \(-0.904041236768688`\)}, \
{0.1`, \(-0.825365879480423`\)}, {0.19952620000000001`, \
\(-0.7032656631647647`\)}, {0.3981072`, \(-0.5175462879330898`\)}, \
{0.7943282`, \(-0.9388669170535647`\)}, {1.584893`, \
\(-1.0964770475487855`\)}, {3.162278`, \(-1.5602368372134994`\)}, {6.309573`, \
\(-5.054561417879838`\)}, {12.58925`, 1.5558789113803813`}, {25.11886`, 
        0.43510920996525`}, {50.11872`, 0.1789139532195343`}, {100.`, 
        0.08234727048106798`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(dRetablegen[Redata2t0, 5, 2, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-0.9995757323751339`\)}, {0.00019952620000000001`, \
\(-0.9991538743755636`\)}, {0.00039810720000000004`, \
\(-0.9983131429086123`\)}, {0.0007943282`, \(-0.9966398681454637`\)}, \
{0.0015848930000000002`, \(-0.993317850491258`\)}, {0.003162278`, \
\(-0.9867555548949742`\)}, {0.006309573000000001`, \(-0.9739180640562165`\)}, \
{0.012589250000000001`, \(-0.9492787971708154`\)}, {0.02511886`, \
\(-0.9036836562577008`\)}, {0.050118720000000005`, \(-0.8247785277302141`\)}, \
{0.1`, \(-0.7029401209894567`\)}, {0.19952620000000001`, \
\(-0.5432882348418949`\)}, {0.3981072`, \(-0.314054717180931`\)}, \
{0.7943282`, \(-0.8406136403818578`\)}, {1.584893`, \(-1.129478375246483`\)}, \
{3.162278`, \(-1.9850053125623308`\)}, {6.309573`, 
        8.609752661581439`}, {12.58925`, 0.7756225795297854`}, {25.11886`, 
        0.2779850892830769`}, {50.11872`, 0.12217304814556686`}, {100.`, 
        0.0577333978126706`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(dRetablegen[Redata1t0, 5, 1, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-0.9999435937865448`\)}, {0.00019952620000000001`, \
\(-0.9998874855872016`\)}, {0.00039810720000000004`, \
\(-0.9997755887916614`\)}, {0.0007943282`, \(-0.9995526185028487`\)}, \
{0.0015848930000000002`, \(-0.999108806642768`\)}, {0.003162278`, \
\(-0.9982276094870449`\)}, {0.006309573000000001`, \(-0.9964867732031212`\)}, \
{0.012589250000000001`, \(-0.9930773168544382`\)}, {0.02511886`, \
\(-0.9865147263705246`\)}, {0.050118720000000005`, \(-0.9742112689376355`\)}, \
{0.1`, \(-0.9517637144238869`\)}, {0.19952620000000001`, \
\(-0.9087369651082151`\)}, {0.3981072`, \(-0.7769945061471666`\)}, \
{0.7943282`, \(-1.0812498307917155`\)}, {1.584893`, \
\(-1.0781490096326976`\)}, {3.162278`, \(-1.4047539522790211`\)}, {6.309573`, \
\(-3.881429651756068`\)}, {12.58925`, 0.8095202104957261`}, {25.11886`, 
        0.015090046426511105`}, {50.11872`, \(-0.16631710962855925`\)}, \
{100.`, \(-0.23464995692241025`\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(dRetablegen[Redata2t0, 5, 2, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, \(-0.999943504080287`\)}, {0.00019952620000000001`, \
\(-0.9998873290031675`\)}, {0.00039810720000000004`, \
\(-0.9997753760947409`\)}, {0.0007943282`, \(-0.999552559215549`\)}, \
{0.0015848930000000002`, \(-0.9991101810115446`\)}, {0.003162278`, \
\(-0.9982362214305601`\)}, {0.006309573000000001`, \(-0.9965258473388793`\)}, \
{0.012589250000000001`, \(-0.9932377405813434`\)}, {0.02511886`, \
\(-0.9871125502032805`\)}, {0.050118720000000005`, \(-0.9762166595980339`\)}, \
{0.1`, \(-0.9573303837238594`\)}, {0.19952620000000001`, \
\(-0.9182817696636862`\)}, {0.3981072`, \(-0.725404966488068`\)}, \
{0.7943282`, \(-1.0755390685837065`\)}, {1.584893`, \
\(-1.1133684125220302`\)}, {3.162278`, \(-1.7202179401079172`\)}, {6.309573`, 
        5.848464187772339`}, {12.58925`, 
        0.2579821220249603`}, {25.11886`, \(-0.09589136516402957`\)}, \
{50.11872`, \(-0.20640762229064213`\)}, {100.`, \
\(-0.25204677058761216`\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(dRetablegen[Redata1t0, 5, 1, 
      Pi/2*\((1 - 10^\((\(-10\))\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, \(-0.9999999999999936`\)}, {0.00019952620000000001`, \
\(-0.9999999999999871`\)}, {0.00039810720000000004`, \
\(-0.9999999999999742`\)}, {0.0007943282`, \(-0.9999999999999488`\)}, \
{0.0015848930000000002`, \(-0.9999999999998982`\)}, {0.003162278`, \
\(-0.9999999999997976`\)}, {0.006309573000000001`, \(-0.9999999999995988`\)}, \
{0.012589250000000001`, \(-0.9999999999992095`\)}, {0.02511886`, \
\(-0.9999999999984596`\)}, {0.050118720000000005`, \(-0.9999999999970494`\)}, \
{0.1`, \(-0.9999999999944449`\)}, {0.19952620000000001`, \
\(-0.9999999999892`\)}, {0.3981072`, \(-0.999999999969987`\)}, {0.7943282`, \
\(-1.0000000000448364`\)}, {1.584893`, \(-1.0000000000198657`\)}, {3.162278`, \
\(-1.000000000091875`\)}, {6.309573`, \(-1.000000000643316`\)}, {12.58925`, \
\(-0.9999999995975272`\)}, {25.11886`, \(-0.9999999997744339`\)}, {50.11872`, \
\(-0.9999999998147878`\)}, {100.`, \(-0.9999999998299787`\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Redelta[1, 1, Cos[Pi/2*\((1 - 10^\((\(-15\))\))\)], 10^\((\(-4\))\), 
      1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(1.00954935022182432375`9.4368*^-23\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(dfulltablegen[dat_, a_, t_] := 
      Module[{res}, 
        res = Table[{\(dat[\([i]\)]\)[\([1]\)], 
              Redelta[1, 1, Cos[t], 
                IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, a], 
              Imdelta[1, 1, Cos[t], 
                IntegerPart[10^8*\(dat[\([i]\)]\)[\([1]\)]]/10^8, a]}, {i, 1, 
              Length[dat]}]; \ res]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(dfulltablegen[Redata1t0, 1, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 
        3.320796154570598139923412552739`17.2503*^-8, \
\(-0.000078539807700365338752194282199`14.5414\)}, {0.00019952620000000001`, 
        1.321949979959357963117555829103921407`17.5847*^-7, \
\(-0.000156702572942464704618429540253`14.1427\)}, {0.00039810720000000004`, 
        5.2629135736243778224663966792708`17.9624*^-7, \
\(-0.000312666463770256024469789301975`13.473\)}, {0.0007943282`, 
        2.09523053280504252368939361771784`18.0374*^-6, \
\(-0.000623853139346854825197520124227`12.7146\)}, {0.0015848930000000002`, 
        8.34132096421268338675774013204297`18.7782*^-6, \
\(-0.00124473530169496333772566341375`12.1863\)}, {0.003162278`, 
        0.00003320608017453733436538312203815456`18.7195, \
\(-0.0024833678601642002669469509`11.6311\)}, {0.006309573000000001`, 
        0.0001321758452841624475351875736139845`18.9891, \
\(-0.0049533548698654019469850231`11.3317\)}, {0.012589250000000001`, 
        0.00052587796559876546719199568605060852`19.3337, \
\(-0.0098703452029049211315408989`11.0314\)}, {0.02511886`, 
        0.00208843467224902033944033136139792696`19.7279, \
\(-0.019591671844947006100345`10.7291\)}, {0.050118720000000005`, 
        0.0082334883479277929938648789517373078`20.0743, \
\(-0.0382846578015979259817371495`10.4202\)}, {0.1`, 
        0.03152888070390575992343149211246116666`20.7388, \
\(-0.070184935183284637378197`8.5412\)}, {0.19952620000000001`, 
        0.107487473483210407069934391986423`20.744, \
\(-0.096541678754856230470612`8.1117\)}, {0.3981072`, 
        0.22729590860678063163180515625411591382`20.2605, 
        0.018517891844420931311021`6.6219}, {0.7943282`, \
\(-0.02719790075038700240616522426157121166`18.759\), 
        0.181319998827525340578189`7.4879}, {1.584893`, 
        0.01814307800612965266727213707687881968`18.0084, 0}, {3.162278`, 
        0.031875403161150422450695529795344`17.6566, 0}, {6.309573`, 
        0.0345522993459766184625839269`17.3857, 0}, {12.58925`, 
        0.035188011794851167807875891725206`16.7937, 0}, {25.11886`, 
        0.035345519998615638374225431165166`16.1956, 0}, {50.11872`, 
        0.0353849490470704041993437408`15.5961, 0}, {100.`, 
        0.035394844674962949430423551`14.9962, 0}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<data_delta_1_0.0000.dat\>", %%, "\<Table\>"]\)], "Input"],

Cell[BoxData[
    \("data_delta_1_0.0000.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(dfulltablegen[Redata1t0, 1, Pi/4]\)], "Input"],

Cell[BoxData[
    \({{0.0001`, 
        8.84259338036354905225961599934`16.5613*^-9, \
\(-0.00003022998799317079714469311543548925`20.0034\)}, \
{0.00019952620000000001`, 
        3.52007978789823038465770212348302992`16.8612*^-8, \
\(-0.0000603148636538693241347423651108352`20.0034\)}, \
{0.00039810720000000004`, 
        1.4014058067665168003147179128933`17.1609*^-7, \
\(-0.00012034549881053054848238159985603929`20.0034\)}, {0.0007943282`, 
        5.5791783317840241673598605047763`17.4603*^-7, \
\(-0.00024012214481395054788554337181335979`20.0034\)}, \
{0.0015848930000000002`, 
        2.22114170167659618470948842154433`17.7592*^-6, \
\(-0.000479106462874028064956339321013`20.0034\)}, {0.003162278`, 
        8.84241727654209776650088483013591`18.0569*^-6, \
\(-0.000955909275120476163379190249199`20.0033\)}, {0.006309573000000001`, 
        0.00003520084615586361878723648197752681`18.3524, \
\(-0.001907027988095305728228059955831`20.0033\)}, {0.012589250000000001`, 
        0.00014011206088558638486364563546324168`18.6436, \
\(-0.00380291432463721789752658533256609871`20.0033\)}, {0.02511886`, 
        0.00055739854711516975810775633192702982`18.927, \
\(-0.007571075275464941898224944572216`20.0031\)}, {0.050118720000000005`, 
        0.00221273398655617905475105801391024694`19.1969, \
\(-0.014973446660925731230463905481053`20.0024\)}, {0.1`, 
        0.008708699588625405872485941337855`19.4467, \
\(-0.028824502047540950920420891261039`20.1404\)}, {0.19952620000000001`, 
        0.033058638065115209174343834376589`19.6739, \
\(-0.049285802152799484614507987668065`20.0508\)}, {0.3981072`, 
        0.10506341869031739347824961782477`19.8073, \
\(-0.03801078084240800898428366934831364428`20.1935\)}, {0.7943282`, 
        0.036147773469809623029603205026347`18.6066, 
        0.139515802143206519362675119077782`20.1191}, {1.584893`, 
        0.01469638234058562633043272879485509674`17.7932, 0}, {3.162278`, 
        0.023029002295052864518314565962488`17.4473, 0}, {6.309573`, 
        0.024555065174943027798110593`16.8751, 0}, {12.58925`, 
        0.0249125333075956998233306352`16.2814, 0}, {25.11886`, 
        0.025000805016945507311453349582655`15.683, 0}, {50.11872`, 
        0.0250228835757230445223223482`15.0833, 0}, {100.`, 
        0.0250284235342203210470410901`14.4834, 0}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<data_delta_1_Pi4.dat\>", %%, "\<Table\>"]\)], "Input"],

Cell[BoxData[
    \("data_delta_1_Pi4.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(dfulltablegen[Redata1t0, 1, Pi/2]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.0001`, 0``32.4326, 0``37.2407}, {0.00019952620000000001`, 
        0``31.536, 0``36.6407}, {0.00039810720000000004`, 0``30.6335, 
        0``36.0407}, {0.0007943282`, 0``29.9696, 
        0``35.4407}, {0.0015848930000000002`, 0``29.0715, 
        0``34.8407}, {0.003162278`, 0``28.1734, 
        0``34.2407}, {0.006309573000000001`, 0``27.2735, 
        0``33.6407}, {0.012589250000000001`, 0``26.3735, 
        0``33.0407}, {0.02511886`, 0``25.4734, 
        0``32.4406}, {0.050118720000000005`, 0``24.5728, 0``31.8403}, {0.1`, 
        0``23.67, 0``46.4749}, {0.19952620000000001`, 0``22.7586, 
        0``30.6342}, {0.3981072`, 0``21.8119, 0``40.3894}, {0.7943282`, 
        0``20.7058, 0``29.3664}, {1.584893`, 0``23.2413, 0}, {3.162278`, 
        0``23.1908, 0}, {6.309573`, 0``22.9257, 0}, {12.58925`, 0``22.6336, 
        0}, {25.11886`, 0``22.3355, 0}, {50.11872`, 0``22.036, 0}, {100.`, 
        0``21.7362, 0}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Export["\<data_delta_1_Pi4.dat\>", %%, "\<Table\>"]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fast", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Falpha\)\)], "Input"],

Cell["Global`Falpha", "Print",
  CellTags->"Info3267953076-4906506"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(Falpha[md2_, k_, kz_, om_, a_] := 
                    FastRealpha[md2, k, kz, om, 
                        a] + \[ImaginaryI]\ FastImalpha[md2, k, kz, om, a]\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "Falpha"],
      Editable->False]], "Print",
  CellTags->"Info3267953076-4906506"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[6/20]\)], "Input"],

Cell[BoxData[
    \(0.3`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Falpha[1, 1, 0.999999, 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.2498756579219836`\) - 
      0.00015707915672738337`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{10^pom, 
        Re[Falpha[1, 1, \((1 - 10^\((\(-10\))\))\) Cos[0], 10^pom, 1]], 
        Im[Falpha[1, 1, \((1 - 10^\((\(-10\))\))\) Cos[0], 10^pom, 
            1]]}, {pom, \(-4\), 2, 0.3}]\)], "Input"],

Cell[BoxData[
    \({{1\/10000, \(-0.24999780025154342`\), \(-0.0001570796409231818`\)}, \
{0.00019952623149688788`, \(-0.2499976614736653`\), \
\(-0.00031341505995348407`\)}, {0.00039810717055349735`, \
\(-0.24999731452897012`\), \(-0.0006253450354236429`\)}, \
{0.0007943282347242813`, \(-0.24999634308382349`\), \(-0.0012477256145012833`\
\)}, {0.001584893192461114`, \(-0.2499925960811152`\), \
\(-0.0024895258501387713`\)}, {0.0031622776601683794`, \
\(-0.24997698356983064`\), \(-0.004967145526252732`\)}, \
{0.00630957344480193`, \(-0.24991494985832663`\), \
\(-0.009909871988055272`\)}, {0.012589254117941668`, \
\(-0.2496678558463961`\), \(-0.019765755786803934`\)}, \
{0.025118864315095794`, \(-0.24868607174788318`\), \
\(-0.039382015186056704`\)}, {0.05011872336272722`, \(-0.2447930053229014`\), \
\(-0.07813553332411537`\)}, {0.1`, \(-0.22954693702541315`\), \
\(-0.15244471050032235`\)}, {0.1995262314968879`, \(-0.11181271187270037`\), \
\(-0.2783352354111937`\)}, {0.39810717055349715`, 
        0.17619936760519572`, \(-0.3920987675347777`\)}, \
{0.7943282347242813`, 
        0.39255325554618975`, \(-0.17310566236692876`\)}, \
{1.584893192461113`, 0.28318902779739663`, 0}, {3.162277660168378`, 
        0.25734969712097877`, 0}, {6.30957344480193`, 0.25178081842918054`, 
        0}, {12.589254117941667`, 0.25039526009437907`, 
        0}, {25.118864315095795`, 0.2506794771886973`, 
        0}, {50.118723362727195`, 0.2501110430000608`, 
        0}, {99.99999999999994`, 0.2501110430000608`, 0}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{10^pom, 
        Re[Fbeta[1, 1, \((1 - 10^\((\(-10\))\))\) Cos[0], 10^pom, 1]], 
        Im[Fbeta[1, 1, \((1 - 10^\((\(-10\))\))\) Cos[0], 10^pom, 
            1]]}, {pom, \(-4\), 2, 0.3}]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(42\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.003148135593609172`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.003176419726087224`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.003148135593609172`\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{1\/10000, \(-1.2853981149335407`*^-8\), \
\(-3.1415925891871447`*^-12\)}, {0.00019952623149688788`, \
\(-5.117261345931659`*^-8\), \(-2.4954555468095496`*^-11\)}, \
{0.00039810717055349735`, \(-2.037222528632779`*^-7\), \
\(-1.9822103288381562`*^-10\)}, {0.0007943282347242813`, \
\(-8.110294799787896`*^-7\), \(-1.5745241435333567`*^-9\)}, \
{0.001584893192461114`, \(-3.228743569138075`*^-6\), \
\(-1.2506842785653477`*^-8\)}, {0.0031622776601683794`, \
\(-0.000012853496051879335`\), \(-9.934389572044112`*^-8\)}, \
{0.00630957344480193`, \(-0.00005116492690652759`\), \
\(-7.890695674925894`*^-7\)}, {0.012589254117941668`, \
\(-0.00020359993386028736`\), \(-6.266314982164927`*^-6\)}, \
{0.025118864315095794`, \(-0.0008091001356821632`\), \
\(-0.00004972811564008347`\)}, {0.05011872336272722`, \
\(-0.0031982571225341308`\), \(-0.00039352362603596824`\)}, {0.1`, \
\(-0.012376013043679183`\), \(-0.003079690865692821`\)}, \
{0.1995262314968879`, \(-0.043939265618228036`\), \
\(-0.023080291732541097`\)}, {0.39810717055349715`, \
\(-0.10682998596057057`\), \(-0.14769500379607312`\)}, {0.7943282347242813`, 
        0.14594298716142085`, \(-0.5919222527193342`\)}, {1.584893192461113`, 
        0.3638817538683114`, 0.`}, {3.162277660168378`, 0.3007900876268623`, 
        0.`}, {6.30957344480193`, 0.2890706020962805`, 
        0.`}, {12.589254117941667`, 0.2863092607756653`, 
        0.`}, {25.118864315095795`, 0.28562631529506854`, 
        0.`}, {50.118723362727195`, 0.2854554282422971`, 
        0.`}, {99.99999999999994`, 0.28541254487777207`, 0.`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Imgamma\)\)], "Input"],

Cell["Global`Imgamma", "Print",
  CellTags->"Info3267953430-8404111"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(Imgamma[md2_, k_, kz_, om_, a_] := 
                    If[om > k, 
                      0, \(-\(\(1\/\(k\ \((1 - 
                                  kz\^2\/k\^2)\)\)\)\((md2\ 2\ \
NIntegrate[\(\((om + a\ xi\ kz)\)\ \((2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \
\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\))\)\)\/\(\((1 + a\ xi\^2)\)\^2\ \
\@\(\((1 - kz\^2\/k\^2)\)\ \((1 - om\^2\/k\^2)\) - \((xi - \(om\ \
kz\)\/k\^2)\)\^2\)\), {xi, xi2[k, kz, om]\ If[xi2[k, kz, om] > 0, 
                                    1 + 1\/10\^50, 1 - 1\/10\^50], 
                                xi1[k, kz, om]}, WorkingPrecision \[Rule] 30, 
                              MaxRecursion \[Rule] 30])\)\)\)]\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "Imgamma"],
      Editable->False]], "Print",
  CellTags->"Info3267953430-8404111"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Imgamma[1, 1, 1 - 10^\((\(-5\))\), 10^\((\(-1\))\), 1]\)], "Input"],

Cell[BoxData[
    \(0.00002319659439842990016164860338602021`20.0479\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi1[1, 1 - 10^\((\(-6\))\), 0.1] - 
      xi2[1, 1 - 10^\((\(-6\))\), 0.1]\)], "Input"],

Cell[BoxData[
    \(0.002814248752331605`\)], "Output"]
}, Open  ]],

Cell[BoxData[""], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Im[Fgamma[1, 1, 1 - 10^\((\(-5\))\), 10^\((\(-1\))\), 1]]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.09554929203207531`\)."\
\)], "Message"],

Cell[BoxData[
    \(0.00002296387714200131`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Im[Fgamma[1, 1, 1 - 10^\((\(-5\))\), 10^\((\(-1\))\), 1]]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.09554929203206527`\)."\
\)], "Message"],

Cell[BoxData[
    \(0.000021327780194275621434`5.1041\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Im[Fgamma[1, 1, 1 - 10^\((\(-6\))\), 10^\((\(-1\))\), 1]]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.10140702437616206`\)."\
\)], "Message"],

Cell[BoxData[
    \(0.000019430807582273788`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{10^pom, 
        Re[Fgamma[1, 1, \((1 - 10^\((\(-5\))\))\) Cos[0], 10^pom, 1]], 
        Im[Fgamma[1, 1, \((1 - 10^\((\(-5\))\))\) Cos[0], 10^pom, 
            1]]}, {pom, \(-4\), 2, 0.3}]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.004572123752273506`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.004572123752273506`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.004671648921849454`\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(Less::"nord" \(\(:\)\(\ \)\) 
      "Invalid comparison with \!\(\(\(0.` \[InvisibleSpace]\)\) + \
\(\(0.01099774112107544`\\ \[ImaginaryI]\)\)\) attempted."\)], "Message"],

Cell[BoxData[
    \(Less::"nord" \(\(:\)\(\ \)\) 
      "Invalid comparison with \!\(\(\(0.` \[InvisibleSpace]\)\) + \
\(\(0.01099774112107544`\\ \[ImaginaryI]\)\)\) attempted."\)], "Message"],

Cell[BoxData[
    \(Less::"nord" \(\(:\)\(\ \)\) 
      "Invalid comparison with \!\(\(\(0.` \[InvisibleSpace]\)\) + \
\(\(0.02683274864787429`\\ \[ImaginaryI]\)\)\) attempted."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(Less :: \"nord\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \({{1\/10000, 
        0.000012940197848365642`, \(-3.991938663114092`*^-8\)}, \
{0.00019952623149688788`, 
        0.000012940188753418624`, \(-1.7761276543515848`*^-8\)}, \
{0.00039810717055349735`, 0.000012940165106556378`, 
        2.6299276704194626`*^-8}, {0.0007943282347242813`, 
        0.00001294006870011799`, 
        1.1403787624600908`*^-7}, {0.001584893192461114`, 
        0.000012939681255375035`, 
        2.8944349431881277`*^-7}, {0.0031622776601683794`, 
        0.000012938175132148899`, 
        6.381622439934173`*^-7}, {0.00630957344480193`, 
        0.000012932179743074812`, 
        1.3337989183429661`*^-6}, {0.012589254117941668`, 
        0.000012908200005767867`, 
        2.7379359808581614`*^-6}, {0.025118864315095794`, 
        0.000012812844943255186`, 
        5.550156333600108`*^-6}, {0.05011872336272722`, 
        0.000012482771126087755`, 0.000012147597328723135`}, {0.1`, 
        0.00001099902692658361`, 
        0.00002296772189724725`}, {0.1995262314968879`, 
        6.30003705737181`*^-6, 
        0.00003376005469353826`}, {0.39810717055349715`, \
\(-2.2056992747820914`*^-6\), 
        0.000025867826342028632`}, {0.7943282347242813`, \
\(-1.1919873941224068`*^-6\), 9.283928904053066`*^-7}, {1.584893192461113`, 
        Re[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.01099774112107544`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 1.584893192461113`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 1.584893192461113`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 1.584893192461113`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  1.584893192461113`, 1]]], 
        Im[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.01099774112107544`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 1.584893192461113`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 1.584893192461113`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 1.584893192461113`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  1.584893192461113`, 1]]]}, {3.162277660168378`, 
        Re[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.02683274864787429`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 3.162277660168378`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 3.162277660168378`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 3.162277660168378`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  3.162277660168378`, 1]]], 
        Im[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.02683274864787429`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 3.162277660168378`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 3.162277660168378`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 3.162277660168378`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  3.162277660168378`, 1]]]}, {6.30957344480193`, 
        Re[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.055721107671520255`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 6.30957344480193`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 6.30957344480193`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 6.30957344480193`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  6.30957344480193`, 1]]], 
        Im[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.055721107671520255`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 6.30957344480193`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 6.30957344480193`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 6.30957344480193`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  6.30957344480193`, 1]]]}, {12.589254117941667`, 
        Re[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.11224563485481828`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 12.589254117941667`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 12.589254117941667`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 12.589254117941667`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  12.589254117941667`, 1]]], 
        Im[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.11224563485481828`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 12.589254117941667`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 12.589254117941667`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 12.589254117941667`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  12.589254117941667`, 1]]]}, {25.118864315095795`, 
        Re[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.2244912817360123`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 25.118864315095795`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 25.118864315095795`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 25.118864315095795`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  25.118864315095795`, 1]]], 
        Im[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.2244912817360123`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 25.118864315095795`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 25.118864315095795`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 25.118864315095795`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  25.118864315095795`, 1]]]}, {50.118723362727195`, 
        Re[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.4481851293452222`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 50.118723362727195`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 50.118723362727195`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 50.118723362727195`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  50.118723362727195`, 1]]], 
        Im[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.4481851293452222`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 50.118723362727195`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 50.118723362727195`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 50.118723362727195`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  50.118723362727195`, 1]]]}, {99.99999999999994`, 
        Re[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.8943802325633092`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 99.99999999999994`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 99.99999999999994`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 99.99999999999994`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  99.99999999999994`, 1]]], 
        Im[If[\(\(0.`\)\(\[InvisibleSpace]\)\) + 
                0.8943802325633092`\ \[ImaginaryI] < 3\/1000, 
            Regamma[1, 1, 99999\/100000, 99.99999999999994`, 
                1] + \[ImaginaryI]\ Re[
                  Imgamma[1, 1, 99999\/100000, 99.99999999999994`, 1]], 
            FastRegamma[1, 1, 99999\/100000, 99.99999999999994`, 
                1] + \[ImaginaryI]\ FastImgamma[1, 1, 99999\/100000, 
                  99.99999999999994`, 1]]]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{10^pom, 
        Re[Fgamma[1, 1, \((1 - 10^\((\(-4\))\))\) Cos[0], 10^pom, 1]], 
        Im[Fgamma[1, 1, \((1 - 10^\((\(-4\))\))\) Cos[0], 10^pom, 
            1]]}, {pom, \(-4\), 2, 0.3}]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(66\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(-0.01404179199517389`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(-0.01404179199517389`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(-0.013942275505511829`\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{1\/10000, 0.00012849225367972394`, 
        2.5274773707691523`*^-7}, {0.00019952623149688788`, 
        0.00012849219251620525`, 
        5.029168471888499`*^-7}, {0.00039810717055349735`, 
        0.00012849194376940432`, 
        1.0017945655499552`*^-6}, {0.0007943282347242813`, 
        0.00012849097151956812`, 
        1.9974565516411683`*^-6}, {0.001584893192461114`, 
        0.00012848714459323674`, 
        3.98383384916554`*^-6}, {0.0031622776601683794`, 
        0.00012847198240706348`, 
        7.946728774774772`*^-6}, {0.00630957344480193`, 
        0.0001284139016206609`, 
        0.000015829259853947534`}, {0.012589254117941668`, 
        0.00012817557990274508`, 
        0.000031562354467146757`}, {0.025118864315095794`, 
        0.0001272243309813348`, 
        0.00006271667377330625`}, {0.05011872336272722`, 
        0.00012348615859991696`, 0.0001233981071353624`}, {0.1`, 
        0.00010932745954050915`, 
        0.0002320488637274928`}, {0.1995262314968879`, 
        0.00006284599157879711`, 
        0.0003647926760003039`}, {0.39810717055349715`, \
\(-0.00002142945254490769`\), 
        0.00027095942616696996`}, {0.7943282347242813`, \
\(-7.969531452545198`*^-6\), \(-8.32080781419898`*^-6\)}, \
{1.584893192461113`, 4.390392859932035`*^-6, 0}, {3.162277660168378`, 
        6.4338892116211355`*^-6, 0}, {6.30957344480193`, 
        6.925954949110746`*^-6, 0}, {12.589254117941667`, 
        7.063033990561962`*^-6, 0}, {25.118864315095795`, 
        7.1553513407707214`*^-6, 0}, {50.118723362727195`, 
        7.417052984237671`*^-6, 0}, {99.99999999999994`, 
        8.404254913330078`*^-6, 0}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fgamma[1, 1, 1 - 10^\((\(-5\))\), 100, 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.002944350242614746`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Regamma[1, 1, 1 - 10^\((\(-5\))\), 100, 1]\)], "Input"],

Cell[BoxData[
    \(7.07895570436413781`4.7846*^-7\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{10^pom, 
        Re[Fdelta[1, 1, \((1 - 10^\((\(-4\))\))\) Cos[0], 10^pom, 1]], 
        Im[Fdelta[1, 1, \((1 - 10^\((\(-4\))\))\) Cos[0], 10^pom, 
            1]]}, {pom, \(-4\), 2, 0.3}]\)], "Input"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfn" \(\(:\)\(\ \)\) 
      "Numerical error encountered at instruction \!\(41\); proceeding with \
uncompiled evaluation."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(CompiledFunction :: \"cfn\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.06423772111342389`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.06423772111342389`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.11406089549385438`\)."\
\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{1\/10000, 
        3.2488348589316825`*^-8, \(-0.00007850840695494769`\)}, \
{0.00019952623149688788`, 
        1.3075019955874664`*^-7, \(-0.0001566448144454324`\)}, \
{0.00039810717055349735`, 
        5.2334255577572`*^-7, \(-0.00031254708845674046`\)}, \
{0.0007943282347242813`, 
        2.0890799274369214`*^-6, \(-0.0006236101898830333`\)}, \
{0.001584893192461114`, 
        8.327910303158753`*^-6, \(-0.001244240189411566`\)}, \
{0.0031622776601683794`, 
        0.00003317509945321578`, \(-0.0024823812189420356`\)}, \
{0.00630957344480193`, 
        0.00013209671093209103`, \(-0.0049513784392217685`\)}, \
{0.012589254117941668`, 
        0.0005256523945591713`, \(-0.009866410117694638`\)}, \
{0.025118864315095794`, 
        0.002087723764101124`, \(-0.019583904836936603`\)}, \
{0.05011872336272722`, 
        0.008231140356437563`, \(-0.03826986057976823`\)}, {0.1`, 
        0.03151232923296267`, \(-0.07016047955361177`\)}, \
{0.1995262314968879`, 
        0.10744068651726492`, \(-0.0965237785256691`\)}, \
{0.39810717055349715`, 0.22726358964078672`, 
        0.018460566381486014`}, {0.7943282347242813`, \
\(-0.027142516665058822`\), 0.1813028910429624`}, {1.584893192461113`, 
        0.018142261056145536`, 0}, {3.162277660168378`, 
        0.031872488288752984`, 0}, {6.30957344480193`, 0.034548913508403375`, 
        0}, {12.589254117941667`, 0.03518451062784103`, 
        0}, {25.118864315095795`, 0.03534198948742613`, 
        0}, {50.118723362727195`, 0.0353814134641286`, 
        0}, {99.99999999999994`, 0.03539130975269698`, 0}}\)], "Output"]
}, Closed]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Dispersion-relations", "Section"],

Cell[CellGroupData[{

Cell["prelim", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[xi^2/\((1 + a\ xi^2)\)^2, {xi, 0, 1}]\)], "Input"],

Cell[BoxData[
    \(\(\(-\@a\) + ArcTan[\@a] + a\ ArcTan[\@a]\)\/\(2\ a\^\(3/2\)\ \((1 + a)\
\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[
      xi*Sqrt[xi^2 - kt^2]/\((1 + a\ xi^2)\)^2, {xi, kt, 1}]\)], "Input"],

Cell[BoxData[
    \(\(\(-\@a\)\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - ArcTanh[\(\@a\ \
\@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)] - a\ ArcTanh[\(\@a\ \@\(1 - kt\^2\
\)\)\/\@\(\(-1\) - a\ kt\^2\)]\)\/\(2\ a\^\(3/2\)\ \((1 + a)\)\ \@\(\(-1\) - \
a\ kt\^2\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input"],

Cell[BoxData[
    \(\(\(-\@a\)\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - \((1 + a)\)\ \
ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)]\)\/\(2\ a\^\(3/2\)\
\ \((1 + a)\)\ \@\(\(-1\) - a\ kt\^2\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Series[ArcTan[x*\((1 + eps)\)], {eps, 0, 2}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(ArcTan[x]\), "+", \(\(x\ eps\)\/\(1 + x\^2\)\), 
        "-", \(\(x\^3\ eps\^2\)\/\((1 + x\^2)\)\^2\), "+", 
        InterpretationBox[\(O[eps]\^3\),
          SeriesData[ eps, 0, {}, 0, 3, 1]]}],
      SeriesData[ eps, 0, {
        ArcTan[ x], 
        Times[ x, 
          Power[ 
            Plus[ 1, 
              Power[ x, 2]], -1]], 
        Times[ -1, 
          Power[ x, 3], 
          Power[ 
            Plus[ 1, 
              Power[ x, 2]], -2]]}, 0, 3, 1]]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(mMag2[mD2_, 
        a_] := \(mD2/4\)/
          Sqrt[a]*\((\((a - 1)\)*ArcTan[Sqrt[a]] + Sqrt[a])\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[mMag2[1, a], {a, \(-1\), 20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.4151 0.11172 [
[.29592 .4026 -3 -9 ]
[.29592 .4026 3 0 ]
[.52268 .4026 -6 -9 ]
[.52268 .4026 6 0 ]
[.74943 .4026 -6 -9 ]
[.74943 .4026 6 0 ]
[.97619 .4026 -6 -9 ]
[.97619 .4026 6 0 ]
[.05666 .07994 -12 -4.5 ]
[.05666 .07994 0 4.5 ]
[.05666 .19166 -12 -4.5 ]
[.05666 .19166 0 4.5 ]
[.05666 .30338 -12 -4.5 ]
[.05666 .30338 0 4.5 ]
[.05666 .52682 -6 -4.5 ]
[.05666 .52682 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .4151 m
.29592 .42135 L
s
[(5)] .29592 .4026 0 1 Mshowa
.52268 .4151 m
.52268 .42135 L
s
[(10)] .52268 .4026 0 1 Mshowa
.74943 .4151 m
.74943 .42135 L
s
[(15)] .74943 .4026 0 1 Mshowa
.97619 .4151 m
.97619 .42135 L
s
[(20)] .97619 .4026 0 1 Mshowa
.125 Mabswid
.11451 .4151 m
.11451 .41885 L
s
.15986 .4151 m
.15986 .41885 L
s
.20522 .4151 m
.20522 .41885 L
s
.25057 .4151 m
.25057 .41885 L
s
.34127 .4151 m
.34127 .41885 L
s
.38662 .4151 m
.38662 .41885 L
s
.43197 .4151 m
.43197 .41885 L
s
.47732 .4151 m
.47732 .41885 L
s
.56803 .4151 m
.56803 .41885 L
s
.61338 .4151 m
.61338 .41885 L
s
.65873 .4151 m
.65873 .41885 L
s
.70408 .4151 m
.70408 .41885 L
s
.79478 .4151 m
.79478 .41885 L
s
.84014 .4151 m
.84014 .41885 L
s
.88549 .4151 m
.88549 .41885 L
s
.93084 .4151 m
.93084 .41885 L
s
.02381 .4151 m
.02381 .41885 L
s
.25 Mabswid
0 .4151 m
1 .4151 L
s
.06916 .07994 m
.07541 .07994 L
s
[(-3)] .05666 .07994 1 0 Mshowa
.06916 .19166 m
.07541 .19166 L
s
[(-2)] .05666 .19166 1 0 Mshowa
.06916 .30338 m
.07541 .30338 L
s
[(-1)] .05666 .30338 1 0 Mshowa
.06916 .52682 m
.07541 .52682 L
s
[(1)] .05666 .52682 1 0 Mshowa
.125 Mabswid
.06916 .10228 m
.07291 .10228 L
s
.06916 .12463 m
.07291 .12463 L
s
.06916 .14697 m
.07291 .14697 L
s
.06916 .16932 m
.07291 .16932 L
s
.06916 .214 m
.07291 .214 L
s
.06916 .23635 m
.07291 .23635 L
s
.06916 .25869 m
.07291 .25869 L
s
.06916 .28104 m
.07291 .28104 L
s
.06916 .32572 m
.07291 .32572 L
s
.06916 .34807 m
.07291 .34807 L
s
.06916 .37041 m
.07291 .37041 L
s
.06916 .39276 m
.07291 .39276 L
s
.06916 .43744 m
.07291 .43744 L
s
.06916 .45979 m
.07291 .45979 L
s
.06916 .48213 m
.07291 .48213 L
s
.06916 .50448 m
.07291 .50448 L
s
.06916 .0576 m
.07291 .0576 L
s
.06916 .03525 m
.07291 .03525 L
s
.06916 .01291 m
.07291 .01291 L
s
.06916 .54916 m
.07291 .54916 L
s
.06916 .57151 m
.07291 .57151 L
s
.06916 .59385 m
.07291 .59385 L
s
.06916 .6162 m
.07291 .6162 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .30265 L
.02605 .32103 L
.02729 .33362 L
.02846 .34202 L
.03053 .3529 L
.03279 .36157 L
.0379 .37546 L
.04262 .38468 L
.05224 .39844 L
.06244 .40922 L
.0825 .42496 L
.10458 .43802 L
.14487 .45623 L
.18364 .47007 L
.22486 .48249 L
.26457 .49298 L
.30277 .50207 L
.34341 .51094 L
.38254 .51884 L
.42412 .52669 L
.46419 .53381 L
.50274 .54031 L
.54374 .54689 L
.58323 .55295 L
.6212 .55856 L
.66163 .5643 L
.70054 .56964 L
.7419 .57513 L
.78174 .58025 L
.82007 .58504 L
.86085 .58999 L
.90012 .59464 L
.93787 .59899 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003o503oool0
0`000000oooo0?ooo`3o0?ooo`X0oooo000D0?ooo`030000003oool0oooo0?l0oooo2P3oool001@0
oooo00<000000?ooo`3oool0o`3oool:0?ooo`00503oool200000?l0oooo2`3oool000L0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3o
ool000L0oooo00<000000?ooo`3oool02P3oool200000?l0oooo2`3oool000L0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:
0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`800000o`3oool;0?ooo`001`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`020?ooo`8000001P3oool0
0`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`050000003oool0oooo0?ooo`0000000P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`030?ooo`800000o`3oool;0?ooo`001@3oool4000000<0
oooo0P0000060?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00D0
00000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`000000oooo0?oo
o`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:
0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3oool000L0
oooo00<000000?ooo`3oool02P3oool200000?l0oooo2`3oool000L0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?oo
o`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:0?ooo`00
1`3oool00`000000oooo0?ooo`0:0?ooo`800000o`3oool;0?ooo`001`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0
oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo
00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool0
0`000000oooo0?ooo`0:0?ooo`800000o`3oool;0?ooo`001`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3o
ool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?oo
o`030000003oool0oooo00X0oooo0P00003o0?ooo`/0oooo00070?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`3o0?ooo`X0oooo00070?ooo`040000003oool0oooo0?ooo`@000001@3oool00`000000oooo0?oo
o`3o0?ooo`X0oooo00070?ooo`050000003oool0oooo0?ooo`000000203oool00`000000oooo0?oo
o`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01@3oool2
00000?l0oooo2`3oool000D0oooo100000040?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool01@000000oooo0?ooo`3oool000000080
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000
003oool0oooo0080oooo0P0000060?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?oo
o`800000o`3oool;0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0
oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo
00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0
oooo0P00003o0?ooo`/0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3o
ool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo0P00
003o0?ooo`/0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:
0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3oool000L0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool2
00000?l0oooo2`3oool000L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3o0?oo
o`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0o`3oool:0?ooo`00
1`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00@0
00000?ooo`3oool0oooo100000050?ooo`030000003oool0oooo0?l0oooo2P3oool000L0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo2P3o
ool000L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`800000o`3oool;
0?ooo`001@3oool4000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?oo
o`X0oooo00070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`3o0?ooo`X0oooo00070?ooo`040000003oool0oooo0?ooo`<000001P3oool00`000000
oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`3o0?ooo`X0oooo00070?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool0o`3oool:0?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo0?l0oooo2P3oool000L0oooo00<000000?ooo`3oool02P3oool200000?l0oooo2`3oool000P0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00080?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool:0?ooo`00203oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo0?l0oooo2P3oool000P0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`3o0?ooo`X0oooo00080?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0o`3oool:0?ooo`00203oool00`000000oooo0?ooo`090?ooo`800000o`3oool;0?ooo`00
2@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?l0oooo2P3oool000T0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`3o0?ooo`X0oooo00090?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool0o`3oool:0?ooo`002@3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0?l0oooo2P3oool000T0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`3o0?ooo`X0oooo000:0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
o`3oool:0?ooo`002P3oool00`000000oooo0?ooo`070?ooo`800000o`3oool;0?ooo`002`3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo2P3oool000/0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`0l0?ooo`<00000>P3oool4000000<0oooo0P00000h0?oo
o`@000000P3oool3000003P0oooo100000030?ooo`8000001@3oool000`0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03T0oooo00D000000?ooo`3o
ool0oooo000000020?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0e0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool00000103oool0
00`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo
03T0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo03L0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0080oooo00@00000
0?ooo`3oool00000103oool000d0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0l
0?ooo`<00000?03oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0=`3o
ool010000000oooo0?ooo`3oool3000003X0oooo00D000000?ooo`3oool0oooo000000020?ooo`03
0000003oool0oooo0080oooo000=0?ooo`030000003oool0oooo00@0oooo0P00000m0?ooo`030000
003oool0oooo03`0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo03L0
oooo00D000000?ooo`3oool0oooo0000000m0?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool000h0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0l0?oo
o`030000003oool0oooo03X0oooo0`0000030?ooo`040000003oool0oooo000003L0oooo0`000003
0?ooo`030000003oool0oooo03P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?oo
o`0000040?ooo`003`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo03`0oooo1000
000k0?ooo`030000003oool0oooo0080oooo0P00000j0?ooo`040000003oool0oooo0?ooo`@00000
>03oool2000000@0oooo0P0000050?ooo`00403oool01@000000oooo0?ooo`3oool000000?l0oooo
303oool00140oooo00@000000?ooo`3oool00000o`3oool<0?ooo`004P3oool00`000000oooo0000
003o0?ooo``0oooo003o0000020000000@3oool000L0oooo00<000000?ooo`3oool02P3oool20000
00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?ooo`00503oool00`000000
oooo0000000n0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0?P3oool00`000000
oooo0?ooo`0n0?ooo`030000003oool0oooo00H0oooo000D0?ooo`040000003oool0oooo00000?l0
oooo2@3oool001@0oooo00@000000?ooo`3oool0oooo0P00003o0?ooo`L0oooo000D0?ooo`030000
003oool0oooo00<0oooo0P00003o0?ooo`D0oooo000D0?ooo`030000003oool0oooo00D0oooo0P00
003o0?ooo`<0oooo000D0?ooo`800000203oool200000?l0oooo0@3oool001@0oooo00<000000?oo
o`3oool02@3oool200000?h0oooo000D0?ooo`030000003oool0oooo00/0oooo0P00003l0?ooo`00
503oool00`000000oooo0?ooo`0=0?ooo`800000nP3oool001@0oooo00<000000?ooo`3oool03`3o
ool200000?P0oooo000D0?ooo`030000003oool0oooo0140oooo0`00003e0?ooo`00503oool20000
01D0oooo0`00003b0?ooo`00503oool00`000000oooo0?ooo`0G0?ooo`800000l03oool001@0oooo
00<000000?ooo`3oool06@3oool300000>d0oooo000D0?ooo`030000003oool0oooo01`0oooo0`00
003Z0?ooo`00503oool00`000000oooo0?ooo`0O0?ooo`<00000i`3oool001@0oooo00<000000?oo
o`3oool08P3oool300000>@0oooo000D0?ooo`030000003oool0oooo02D0oooo0`00003Q0?ooo`00
503oool2000002T0oooo0`00003N0?ooo`00503oool00`000000oooo0?ooo`0[0?ooo`@00000fP3o
ool001@0oooo00<000000?ooo`3oool0;`3oool400000=H0oooo000D0?ooo`030000003oool0oooo
03<0oooo1@00003A0?ooo`00503oool00`000000oooo0?ooo`0h0?ooo`H00000b`3oool001@0oooo
00<000000?ooo`3oool0?P3oool400000<L0oooo000D0?ooo`800000@`3oool400000<<0oooo000D
0?ooo`030000003oool0oooo04H0oooo1000002o0?ooo`00503oool00`000000oooo0?ooo`1:0?oo
o`@00000^`3oool001@0oooo00<000000?ooo`3oool0CP3oool600000;D0oooo000;0?ooo`@00000
1@3oool00`000000oooo0?ooo`1D0?ooo`D00000/03oool000d0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`1I0?ooo`H00000ZP3oool000d0oooo00<000000?ooo`3oool0103oool2
00000600oooo1P00002T0?ooo`003@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
06D0oooo1P00002N0?ooo`003@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo06/0
oooo1P00002H0?ooo`002`3oool3000000H0oooo00<000000?ooo`3oool0L@3oool600000980oooo
000=0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0M`3oool5000008d0oooo000D
0?ooo`030000003oool0oooo07`0oooo1P0000270?ooo`00503oool00`000000oooo0?ooo`220?oo
o`H00000P@3oool001@0oooo0P0000290?ooo`H00000N`3oool001@0oooo00<000000?ooo`3oool0
SP3oool8000007<0oooo000D0?ooo`030000003oool0oooo09H0oooo2000001[0?ooo`00503oool0
0`000000oooo0?ooo`2N0?ooo`H00000I@3oool001@0oooo00<000000?ooo`3oool0Y03oool50000
0600oooo000D0?ooo`030000003oool0oooo0:T0oooo1P00001J0?ooo`00503oool200000;00oooo
2000001B0?ooo`00503oool00`000000oooo0?ooo`2g0?ooo`T00000B@3oool001@0oooo00<00000
0?ooo`3oool0`03oool6000004<0oooo000D0?ooo`030000003oool0oooo0<H0oooo2@00000j0?oo
o`00503oool00`000000oooo0?ooo`3?0?ooo`/00000;`3oool001@0oooo00<000000?ooo`3oool0
fP3oool8000002L0oooo000D0?ooo`030000003oool0oooo0>80oooo1P00000Q0?ooo`00503oool2
00000>T0oooo2000000I0?ooo`00503oool00`000000oooo0?ooo`3`0?ooo`/000003P3oool001@0
oooo00<000000?ooo`3oool0n`3oool6000000P0oooo000D0?ooo`030000003oool0oooo0?l0oooo
2P3oool001@0oooo00<000000?ooo`3oool0o`3oool:0?ooo`00503oool00`000000oooo0?ooo`3o
0?ooo`X0oooo000D0?ooo`800000o`3oool;0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.55235, -3.71557, \
0.0770198, 0.0312653}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(alomz[mD2_, kz_, k_, 
        a_] := \(\(\(-mD2\)/2\)/
            Sqrt[a]\)/\((k^2 - kz^2)\)*\((kz^2*ArcTan[Sqrt[a]] - 
            kz*k/Sqrt[1 + a*\((1 - kz^2/k^2)\)]*
              ArcTan[Sqrt[a]*\(kz/k\)/
                    Sqrt[1 + a*\((1 - kz^2/k^2)\)]])\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[alomz[Sqrt[3], Cos[th], 1, 10], {th, 0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.603319 0.295831 [
[.2619 .59082 -9 -9 ]
[.2619 .59082 9 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .01166 -12 -4.5 ]
[.01131 .01166 0 4.5 ]
[.01131 .15957 -24 -4.5 ]
[.01131 .15957 0 4.5 ]
[.01131 .30749 -12 -4.5 ]
[.01131 .30749 0 4.5 ]
[.01131 .4554 -24 -4.5 ]
[.01131 .4554 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(0.2)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(0.4)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.6)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(0.8)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.08333 .60332 m
.08333 .60707 L
s
.14286 .60332 m
.14286 .60707 L
s
.20238 .60332 m
.20238 .60707 L
s
.32143 .60332 m
.32143 .60707 L
s
.38095 .60332 m
.38095 .60707 L
s
.44048 .60332 m
.44048 .60707 L
s
.55952 .60332 m
.55952 .60707 L
s
.61905 .60332 m
.61905 .60707 L
s
.67857 .60332 m
.67857 .60707 L
s
.79762 .60332 m
.79762 .60707 L
s
.85714 .60332 m
.85714 .60707 L
s
.91667 .60332 m
.91667 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .01166 m
.03006 .01166 L
s
[(-2)] .01131 .01166 1 0 Mshowa
.02381 .15957 m
.03006 .15957 L
s
[(-1.5)] .01131 .15957 1 0 Mshowa
.02381 .30749 m
.03006 .30749 L
s
[(-1)] .01131 .30749 1 0 Mshowa
.02381 .4554 m
.03006 .4554 L
s
[(-0.5)] .01131 .4554 1 0 Mshowa
.125 Mabswid
.02381 .04124 m
.02756 .04124 L
s
.02381 .07082 m
.02756 .07082 L
s
.02381 .10041 m
.02756 .10041 L
s
.02381 .12999 m
.02756 .12999 L
s
.02381 .18916 m
.02756 .18916 L
s
.02381 .21874 m
.02756 .21874 L
s
.02381 .24832 m
.02756 .24832 L
s
.02381 .2779 m
.02756 .2779 L
s
.02381 .33707 m
.02756 .33707 L
s
.02381 .36665 m
.02756 .36665 L
s
.02381 .39624 m
.02756 .39624 L
s
.02381 .42582 m
.02756 .42582 L
s
.02381 .48499 m
.02756 .48499 L
s
.02381 .51457 m
.02756 .51457 L
s
.02381 .54415 m
.02756 .54415 L
s
.02381 .57374 m
.02756 .57374 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .01472 m
.0403 .01518 L
.04458 .01575 L
.05429 .01752 L
.06375 .01987 L
.07387 .02305 L
.09565 .03214 L
.11547 .04285 L
.15491 .06998 L
.19284 .10152 L
.2332 .13851 L
.27205 .17544 L
.30942 .21072 L
.3492 .24694 L
.38749 .27982 L
.4282 .31225 L
.46741 .34089 L
.50512 .36604 L
.54526 .39032 L
.5839 .41145 L
.62496 .43167 L
.66453 .44919 L
.7026 .46439 L
.74309 .47898 L
.78208 .49164 L
.81958 .50266 L
.8595 .51328 L
.89792 .52253 L
.93877 .53143 L
.97619 .53883 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000C0?ooo`@00000
103oool00`000000oooo0?ooo`3o0?ooo`<0oooo000C0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0o`3oool30?ooo`00503oool00`000000oooo0?ooo`040?ooo`800000o`3oool4
0?ooo`003@3oool4000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020000
0?l0oooo0@3oool001H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`@0
0000o03oool001<0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`060?ooo`D0
0000m`3oool001@0oooo0P0000050?ooo`030000003oool0oooo00/0oooo0P00003e0?ooo`006`3o
ool00`000000oooo0?ooo`0=0?ooo`800000l`3oool001/0oooo00<000000?ooo`3oool03`3oool2
00000?40oooo000K0?ooo`030000003oool0oooo0140oooo0P00003_0?ooo`006`3oool2000001@0
oooo0P00003]0?ooo`006`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0>X0oooo
000K0?ooo`030000003oool0oooo01H0oooo0P00003Z0?ooo`006`3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo0>L0oooo000K0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0iP3oool001/0oooo00<000000?ooo`3oool06P3oool200000>H0oooo000K0?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool0h`3oool001/0oooo00<000000?ooo`3oool07@3o
ool200000><0oooo000K0?ooo`800000803oool00`000000oooo0?ooo`3P0?ooo`006`3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo0=l0oooo000K0?ooo`030000003oool0oooo0240
oooo0P00003O0?ooo`006`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0=`0oooo
000K0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0f`3oool001/0oooo00<00000
0?ooo`3oool09@3oool00`000000oooo0?ooo`3J0?ooo`006`3oool00`000000oooo0?ooo`0V0?oo
o`800000fP3oool001/0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`3G0?ooo`00
6`3oool2000002X0oooo00<000000?ooo`3oool0eP3oool001/0oooo00<000000?ooo`3oool0:P3o
ool00`000000oooo0?ooo`3E0?ooo`006`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo0=@0oooo000K0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0d`3oool001/0
oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`3B0?ooo`006`3oool00`000000oooo
0?ooo`0^0?ooo`800000dP3oool001/0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?oo
o`3?0?ooo`006`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0<h0oooo000K0?oo
o`800000<`3oool00`000000oooo0?ooo`3=0?ooo`006`3oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo0<`0oooo000K0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0
b`3oool001/0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`3:0?ooo`006`3oool0
0`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0<T0oooo00070?ooo`@000000`3oool00`00
0000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo
0<P0oooo00090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0h0?ooo`030000003oool0oooo0<L0oooo00090?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool00P3oool2000003X0oooo00<000000?ooo`3oool0aP3oool000030?ooo`00
0000000000800000103oool00`000000oooo0?ooo`070?ooo`<000001@3oool00`000000oooo0?oo
o`0j0?ooo`030000003oool0oooo0<D0oooo00090?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0<@0oooo00070?oo
o`<000002@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool0``3oool000T0oooo00<000000?ooo`3oool01`3oool4000000@0oooo00<000000?ooo`3o
ool0?@3oool00`000000oooo0?ooo`320?ooo`006`3oool00`000000oooo0?ooo`0n0?ooo`030000
003oool0oooo0<40oooo000K0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0`03o
ool001/0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`2o0?ooo`006`3oool20000
0480oooo00<000000?ooo`3oool0_P3oool001/0oooo00<000000?ooo`3oool0@P3oool00`000000
oooo0?ooo`2m0?ooo`006`3oool00`000000oooo0?ooo`130?ooo`800000_@3oool001/0oooo00<0
00000?ooo`3oool0A@3oool00`000000oooo0?ooo`2j0?ooo`006`3oool00`000000oooo0?ooo`16
0?ooo`030000003oool0oooo0;T0oooo000K0?ooo`030000003oool0oooo04L0oooo00<000000?oo
o`3oool0^03oool001/0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`2g0?ooo`00
6`3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo0;H0oooo000K0?ooo`800000B`3o
ool00`000000oooo0?ooo`2e0?ooo`006`3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0
oooo0;@0oooo000K0?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0/`3oool001/0
oooo00<000000?ooo`3oool0C@3oool200000;<0oooo000K0?ooo`030000003oool0oooo04l0oooo
00<000000?ooo`3oool0/03oool001/0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?oo
o`2_0?ooo`006`3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo0:h0oooo000K0?oo
o`030000003oool0oooo0580oooo00<000000?ooo`3oool0[@3oool001/0oooo0P00001D0?ooo`03
0000003oool0oooo0:`0oooo000K0?ooo`030000003oool0oooo05@0oooo0P00002/0?ooo`006`3o
ool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo0:T0oooo000K0?ooo`030000003oool0
oooo05L0oooo00<000000?ooo`3oool0Z03oool001/0oooo00<000000?ooo`3oool0F03oool00`00
0000oooo0?ooo`2W0?ooo`006`3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo0:H0
oooo000K0?ooo`030000003oool0oooo05X0oooo0P00002V0?ooo`006`3oool00`000000oooo0?oo
o`1L0?ooo`030000003oool0oooo0:<0oooo000K0?ooo`800000GP3oool00`000000oooo0?ooo`2R
0?ooo`006`3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo0:40oooo000K0?ooo`03
0000003oool0oooo05l0oooo00<000000?ooo`3oool0X03oool001/0oooo00<000000?ooo`3oool0
H03oool00`000000oooo0?ooo`2O0?ooo`006`3oool00`000000oooo0?ooo`1Q0?ooo`030000003o
ool0oooo09h0oooo000C0?ooo`@00000103oool00`000000oooo0?ooo`1R0?ooo`800000WP3oool0
01D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo
09/0oooo000E0?ooo`030000003oool0oooo00<0oooo0P00001V0?ooo`030000003oool0oooo09X0
oooo000=0?ooo`@00000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo06H0oooo
00<000000?ooo`3oool0V@3oool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`1W0?ooo`030000003oool0oooo09P0oooo000C0?ooo`<000001@3oool00`000000oooo0?ooo`1X
0?ooo`800000V03oool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1Z0?oo
o`030000003oool0oooo09D0oooo000K0?ooo`030000003oool0oooo06/0oooo0P00002E0?ooo`00
6`3oool00`000000oooo0?ooo`1]0?ooo`800000T`3oool001/0oooo00<000000?ooo`3oool0K`3o
ool00`000000oooo0?ooo`2@0?ooo`006`3oool200000740oooo0P00002@0?ooo`006`3oool00`00
0000oooo0?ooo`1b0?ooo`030000003oool0oooo08d0oooo000K0?ooo`030000003oool0oooo07<0
oooo00<000000?ooo`3oool0S03oool001/0oooo00<000000?ooo`3oool0M03oool2000008`0oooo
000K0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool0R@3oool001/0oooo00<00000
0?ooo`3oool0M`3oool00`000000oooo0?ooo`280?ooo`006`3oool00`000000oooo0?ooo`1h0?oo
o`800000R03oool001/0oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`250?ooo`00
6`3oool2000007`0oooo00<000000?ooo`3oool0Q03oool001/0oooo00<000000?ooo`3oool0O03o
ool2000008@0oooo000K0?ooo`030000003oool0oooo07h0oooo0P0000220?ooo`006`3oool00`00
0000oooo0?ooo`200?ooo`800000P03oool001/0oooo00<000000?ooo`3oool0PP3oool2000007h0
oooo000K0?ooo`030000003oool0oooo08@0oooo0P00001l0?ooo`006`3oool00`000000oooo0?oo
o`260?ooo`030000003oool0oooo07T0oooo000K0?ooo`030000003oool0oooo08L0oooo0P00001i
0?ooo`006`3oool2000008X0oooo0P00001g0?ooo`006`3oool00`000000oooo0?ooo`2;0?ooo`03
0000003oool0oooo07@0oooo000K0?ooo`030000003oool0oooo08`0oooo0P00001d0?ooo`006`3o
ool00`000000oooo0?ooo`2>0?ooo`800000LP3oool001/0oooo00<000000?ooo`3oool0T03oool2
00000700oooo000K0?ooo`030000003oool0oooo0980oooo0P00001^0?ooo`006`3oool00`000000
oooo0?ooo`2D0?ooo`800000K03oool001/0oooo0P00002G0?ooo`800000JP3oool001/0oooo00<0
00000?ooo`3oool0V03oool2000006P0oooo000K0?ooo`030000003oool0oooo09X0oooo0P00001V
0?ooo`006`3oool00`000000oooo0?ooo`2L0?ooo`800000I03oool001/0oooo00<000000?ooo`3o
ool0WP3oool200000680oooo000K0?ooo`030000003oool0oooo0:00oooo0P00001P0?ooo`00203o
ool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0XP3o
ool2000005h0oooo00070?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`2T0?ooo`<00000F`3oool000L0oooo00@000000?ooo`3oool000002`3o
ool00`000000oooo0?ooo`020?ooo`800000Z03oool2000005T0oooo00000`3oool0000000000002
00000080oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<000000?ooo`3oool0Z@3o
ool3000005H0oooo00070?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`2/0?ooo`800000E03oool000L0oooo00@000000?ooo`3oool00000203o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0:h0oooo0`00001A0?ooo`00203oool2
000000T0oooo100000040?ooo`030000003oool0oooo0;40oooo1000001=0?ooo`006`3oool00`00
0000oooo0?ooo`2e0?ooo`@00000B@3oool001/0oooo00<000000?ooo`3oool0^@3oool3000004H0
oooo000K0?ooo`030000003oool0oooo0;`0oooo0P0000140?ooo`006`3oool200000;l0oooo0`00
00110?ooo`006`3oool00`000000oooo0?ooo`310?ooo`800000?`3oool001/0oooo00<000000?oo
o`3oool0``3oool3000003`0oooo000K0?ooo`030000003oool0oooo0<H0oooo0`00000i0?ooo`00
6`3oool00`000000oooo0?ooo`390?ooo`@00000=@3oool001/0oooo00<000000?ooo`3oool0c@3o
ool300000380oooo000K0?ooo`030000003oool0oooo0=00oooo1000000^0?ooo`006`3oool00`00
0000oooo0?ooo`3D0?ooo`@00000:P3oool001/0oooo0P00003I0?ooo`@000009P3oool001/0oooo
00<000000?ooo`3oool0g03oool500000240oooo000K0?ooo`030000003oool0oooo0>40oooo1@00
000L0?ooo`006`3oool00`000000oooo0?ooo`3V0?ooo`H000005P3oool001/0oooo00<000000?oo
o`3oool0k03oool500000140oooo000K0?ooo`030000003oool0oooo0?40oooo1@00000<0?ooo`00
6`3oool00`000000oooo0?ooo`3f0?ooo`<000002@3oool001/0oooo00<000000?ooo`3oool0o`3o
ool30?ooo`006`3oool200000?l0oooo103oool001/0oooo00<000000?ooo`3oool0o`3oool30?oo
o`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo
0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`0d
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000<03oool2000000@0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0_0?ooo`800000103oool00`000000oooo0?ooo`03
0?ooo`800000<@3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo000K0?oo
o`030000003oool0oooo03<0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0`
0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0;P3oool010000000oooo0?oo
o`0000080?ooo`040000003oool0oooo000002l0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000020?ooo`006`3oool2000003@0oooo00@000000?ooo`3oool000002@3oool0
0`000000oooo0?ooo`0_0?ooo`040000003oool0oooo000000L0oooo1@00000_0?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool00000;`3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo00000080oooo000K0?ooo`030000003oool0oooo03<0oooo00@000000?oo
o`3oool000002P3oool00`000000oooo0?ooo`0^0?ooo`040000003oool0oooo000000L0oooo00@0
00000?ooo`3oool00000<03oool010000000oooo0?ooo`0000080?ooo`<00000<03oool010000000
oooo0?ooo`0000090?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0<`3oool010000000
oooo0?ooo`00000;0?ooo`030000003oool0oooo02d0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo0000000`0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0<03o
ool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo000K0?ooo`030000
003oool0oooo03<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000_0?oo
o`040000003oool0oooo000000T0oooo0P00000`0?ooo`040000003oool0oooo000000P0oooo00<0
00000?ooo`3oool0<03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080
oooo000K0?ooo`030000003oool0oooo03@0oooo0P00000:0?ooo`800000<@3oool2000000/0oooo
00<000000?ooo`3oool0;`3oool2000000X0oooo0`00000`0?ooo`8000002P3oool2000000<0oooo
000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000E0?ooool000002@0000030?ooo`00
6`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00L0oooo000K0?ooo`030000003o
ool0oooo03/0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0l0?ooo`030000003o
ool0oooo03`0oooo00<000000?ooo`3oool01`3oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo003o0?ooob40oooo003o0?ooob40oooo
003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0870867, -2.14005, \
0.00319096, 0.012841}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-alomz[1, 0.5, 1, a]\), {a, \(-1\), 20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.480033 2.299 [
[.29592 .46753 -3 -9 ]
[.29592 .46753 3 0 ]
[.52268 .46753 -6 -9 ]
[.52268 .46753 6 0 ]
[.74943 .46753 -6 -9 ]
[.74943 .46753 6 0 ]
[.97619 .46753 -6 -9 ]
[.97619 .46753 6 0 ]
[.05666 .02023 -24 -4.5 ]
[.05666 .02023 0 4.5 ]
[.05666 .13518 -30 -4.5 ]
[.05666 .13518 0 4.5 ]
[.05666 .25013 -24 -4.5 ]
[.05666 .25013 0 4.5 ]
[.05666 .36508 -30 -4.5 ]
[.05666 .36508 0 4.5 ]
[.05666 .59498 -24 -4.5 ]
[.05666 .59498 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .48003 m
.29592 .48628 L
s
[(5)] .29592 .46753 0 1 Mshowa
.52268 .48003 m
.52268 .48628 L
s
[(10)] .52268 .46753 0 1 Mshowa
.74943 .48003 m
.74943 .48628 L
s
[(15)] .74943 .46753 0 1 Mshowa
.97619 .48003 m
.97619 .48628 L
s
[(20)] .97619 .46753 0 1 Mshowa
.125 Mabswid
.11451 .48003 m
.11451 .48378 L
s
.15986 .48003 m
.15986 .48378 L
s
.20522 .48003 m
.20522 .48378 L
s
.25057 .48003 m
.25057 .48378 L
s
.34127 .48003 m
.34127 .48378 L
s
.38662 .48003 m
.38662 .48378 L
s
.43197 .48003 m
.43197 .48378 L
s
.47732 .48003 m
.47732 .48378 L
s
.56803 .48003 m
.56803 .48378 L
s
.61338 .48003 m
.61338 .48378 L
s
.65873 .48003 m
.65873 .48378 L
s
.70408 .48003 m
.70408 .48378 L
s
.79478 .48003 m
.79478 .48378 L
s
.84014 .48003 m
.84014 .48378 L
s
.88549 .48003 m
.88549 .48378 L
s
.93084 .48003 m
.93084 .48378 L
s
.02381 .48003 m
.02381 .48378 L
s
.25 Mabswid
0 .48003 m
1 .48003 L
s
.06916 .02023 m
.07541 .02023 L
s
[(-0.2)] .05666 .02023 1 0 Mshowa
.06916 .13518 m
.07541 .13518 L
s
[(-0.15)] .05666 .13518 1 0 Mshowa
.06916 .25013 m
.07541 .25013 L
s
[(-0.1)] .05666 .25013 1 0 Mshowa
.06916 .36508 m
.07541 .36508 L
s
[(-0.05)] .05666 .36508 1 0 Mshowa
.06916 .59498 m
.07541 .59498 L
s
[(0.05)] .05666 .59498 1 0 Mshowa
.125 Mabswid
.06916 .04322 m
.07291 .04322 L
s
.06916 .06621 m
.07291 .06621 L
s
.06916 .0892 m
.07291 .0892 L
s
.06916 .11219 m
.07291 .11219 L
s
.06916 .15817 m
.07291 .15817 L
s
.06916 .18116 m
.07291 .18116 L
s
.06916 .20415 m
.07291 .20415 L
s
.06916 .22714 m
.07291 .22714 L
s
.06916 .27312 m
.07291 .27312 L
s
.06916 .29611 m
.07291 .29611 L
s
.06916 .3191 m
.07291 .3191 L
s
.06916 .34209 m
.07291 .34209 L
s
.06916 .38807 m
.07291 .38807 L
s
.06916 .41106 m
.07291 .41106 L
s
.06916 .43405 m
.07291 .43405 L
s
.06916 .45704 m
.07291 .45704 L
s
.06916 .50302 m
.07291 .50302 L
s
.06916 .52601 m
.07291 .52601 L
s
.06916 .549 m
.07291 .549 L
s
.06916 .57199 m
.07291 .57199 L
s
.06916 .61797 m
.07291 .61797 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03484 0 m
.03506 .01084 L
.03756 .103 L
.04017 .17663 L
.04262 .2312 L
.04761 .31395 L
.05217 .36785 L
.05744 .41385 L
.06244 .44682 L
.0675 .47276 L
.07289 .49458 L
.08259 .52352 L
.08853 .53667 L
.09412 .54681 L
.10458 .56145 L
.11434 .57148 L
.12466 .57944 L
.13346 .58467 L
.14313 .58917 L
.15334 .59285 L
.16264 .59546 L
.17338 .59779 L
.18355 .59947 L
.19321 .60068 L
.20346 .60165 L
.21222 .60226 L
.22188 .60275 L
.2268 .60293 L
.23198 .60308 L
.2364 .60317 L
.23888 .60322 L
.24123 .60325 L
.24397 .60328 L
.24646 .6033 L
.24785 .60331 L
.24913 .60331 L
.2498 .60331 L
.25054 .60332 L
.25131 .60332 L
.25204 .60332 L
.25333 .60332 L
.25457 .60332 L
.25567 .60332 L
.25688 .60331 L
.25819 .60331 L
.25958 .6033 L
.26209 .60328 L
.2647 .60325 L
.26746 .60322 L
.2724 .60315 L
Mistroke
.28354 .60293 L
.29317 .60268 L
.30372 .60236 L
.3442 .60079 L
.38316 .59897 L
.42457 .5969 L
.46447 .59487 L
.50285 .59293 L
.54369 .59092 L
.583 .58904 L
.62081 .5873 L
.66106 .58552 L
.6998 .58388 L
.74099 .5822 L
.78067 .58066 L
.81883 .57923 L
.85944 .57778 L
.89854 .57644 L
.94009 .57507 L
.97619 .57393 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00603oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo0?`0oooo000H0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0o03oool001P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3l0?oo
o`003P3oool2000000@0oooo00@000000?ooo`3oool0oooo1@0000040?ooo`030000003oool0oooo
0?`0oooo000=0?ooo`040000003oool0oooo000000L0oooo0P0000070?ooo`030000003oool0oooo
0?`0oooo000=0?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`0000001P3oool30000
0?`0oooo00070?ooo`@000000P3oool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo
000000D0oooo00<000000?ooo`3oool0o03oool000d0oooo00@000000?ooo`3oool000001`3oool0
1@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool0o03oool000d0oooo00@00000
0?ooo`3oool000001`3oool200000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`3l0?ooo`003P3oool2000000P0oooo00@000000?ooo`00000000001@3oool00`000000oooo0?oo
o`3l0?ooo`00603oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?`0oooo000H0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o03oool001P0oooo00<000000?ooo`3o
ool01P3oool200000?d0oooo000H0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
o03oool001P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3l0?ooo`00603oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo0?`0oooo000H0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0o03oool001P0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`3l0?ooo`00603oool00`000000oooo0?ooo`060?ooo`800000o@3oool001P0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`3l0?ooo`00603oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo0?`0oooo000H0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0o03oool001P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3l0?ooo`00
603oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?`0oooo000H0?ooo`030000003o
ool0oooo00H0oooo0P00003m0?ooo`00603oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo0?`0oooo000H0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o03oool001P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3l0?ooo`00603oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0?`0oooo000H0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0o03oool001P0oooo00<000000?ooo`3oool01P3oool200000?d0oooo000H0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o03oool001P0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`3l0?ooo`00603oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo0?`0oooo000H0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o03o
ool000P0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000000`3oool0000000000002
000000D0oooo00<000000?ooo`3oool0o03oool000L0oooo00@000000?ooo`3oool000002P3oool0
10000000oooo0?ooo`0000030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o03o
ool000L0oooo00@000000?ooo`3oool000002P3oool010000000oooo0?ooo`0000030?ooo`030000
003oool0oooo0080oooo0`00003l0?ooo`0000<0oooo0000000000000P0000020?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0100000050?ooo`030000003oool0oooo0?`0oooo
00070?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo000000070?ooo`03
0000003oool0oooo0?`0oooo00070?ooo`040000003oool0oooo000000P0oooo0`0000030?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0o03oool000P0oooo0P00000;0?ooo`040000
003oool0oooo0?ooo`@00000103oool00`000000oooo0?ooo`3l0?ooo`006@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo0?`0oooo000I0?ooo`030000003oool0oooo00D0oooo0P00
003m0?ooo`006@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?`0oooo000I0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o03oool001T0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`3l0?ooo`006@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo0?`0oooo000I0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o03o
ool001T0oooo00<000000?ooo`3oool01@3oool200000?d0oooo000I0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0o03oool001T0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`3l0?ooo`006@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?`0oooo
000I0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o03oool001T0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`3l0?ooo`006@3oool00`000000oooo0?ooo`050?oo
o`800000o@3oool001X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3l0?ooo`00
6P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?`0oooo000J0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0o03oool001X0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`3l0?ooo`006P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0?`0oooo000J0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o03oool001X0oooo
00<000000?ooo`3oool0103oool200000?d0oooo000J0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0o03oool001X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3l
0?ooo`006P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?`0oooo000>0?ooo`80
0000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`3l0?ooo`00
3@3oool010000000oooo0?ooo`0000090?ooo`8000001@3oool00`000000oooo0?ooo`3l0?ooo`00
3@3oool010000000oooo0?ooo`0000090?ooo`8000001@3oool300000?`0oooo00070?ooo`@00000
0P3oool010000000oooo0?ooo`0000090?ooo`8000001@3oool00`000000oooo0?ooo`3l0?ooo`00
3@3oool010000000oooo0?ooo`0000090?ooo`8000001@3oool00`000000oooo0?ooo`3l0?ooo`00
3@3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000oooo0?ooo`3l0?ooo`00
3P3oool2000000X0oooo0P0000050?ooo`030000003oool0oooo0?`0oooo000J0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0o03oool001X0oooo00<000000?ooo`3oool0103oool2
00000?d0oooo000K0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o03oool001/0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3l0?ooo`006`3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0?`0oooo000K0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0o03oool001/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3l
0?ooo`006`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?`0oooo000K0?ooo`03
0000003oool0oooo00<0oooo0P00003m0?ooo`006`3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo0?`0oooo000K0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o03o
ool001/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3l0?ooo`006`3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0?`0oooo000K0?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0o03oool001/0oooo00<000000?ooo`3oool00`3oool200000?d0oooo
000K0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o03oool001/0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`3l0?ooo`006`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0?`0oooo000K0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0o03oool001/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3l0?ooo`00703o
ool00`000000oooo0?ooo`020?ooo`800000o@3oool001`0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`3l0?ooo`00703oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0?`0oooo000L0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o03oool000P0oooo
0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`@00000103oool00`000000oooo
0?ooo`3l0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3l0?ooo`001`3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3oool30000
0?`0oooo00000`3oool000000000000200000080oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000020?ooo`@00000103oool00`000000oooo0?ooo`3l0?ooo`001`3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000
103oool00`000000oooo0?ooo`3l0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000
003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3l0?oo
o`00203oool2000000X0oooo0P0000030?ooo`@00000103oool00`000000oooo0?ooo`3l0?ooo`00
7@3oool01@000000oooo0?ooo`3oool000000?h0oooo000M0?ooo`050000003oool0oooo0?ooo`00
0000oP3oool001d0oooo00@000000?ooo`3oool0oooo0P00003m0?ooo`007@3oool01@000000oooo
0?ooo`3oool000000?h0oooo000M0?ooo`050000003oool0oooo0?ooo`000000oP3oool001d0oooo
00D000000?ooo`3oool0oooo0000003n0?ooo`007P3oool010000000oooo0?ooo`00003n0?ooo`00
7P3oool010000000oooo0?ooo`00003n0?ooo`007P3oool00`000000oooo0?ooo`0200000?d0oooo
000N0?ooo`040000003oool0oooo00000?h0oooo000N0?ooo`040000003oool0oooo00000?h0oooo
000N0?ooo`040000003oool0oooo00000?h0oooo000N0?ooo`040000003oool0oooo00000?h0oooo
000N0?ooo`040000003oool0oooo00000?h0oooo000O0?ooo`040000003oool0000000000?d0oooo
000O0?ooo`030000003oool000000?h0oooo000O0?ooo`030000003oool0000003/0oooo0`00000g
0?ooo`@000000`3oool2000003D0oooo100000020?ooo`<00000=@3oool4000000<0oooo0P000004
0?ooo`007`3oool00`000000oooo0000000n0?ooo`030000003oool0oooo03H0oooo00D000000?oo
o`3oool0oooo000000020?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0b0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool000000`3o
ool001l0oooo00<000000?ooo`000000?P3oool00`000000oooo0?ooo`0f0?ooo`050000003oool0
oooo0?ooo`0000000P3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000<0oooo
000O0?ooo`030000003oool0000003/0oooo0`00000i0?ooo`050000003oool0oooo0?ooo`000000
0P3oool00`000000oooo0?ooo`0d0?ooo`040000003oool0oooo0?ooo`<00000=`3oool01@000000
oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00@3oool001l0oooo00@000000?ooo`00
00000000>P3oool00`000000oooo0?ooo`0i0?ooo`050000003oool0oooo0?ooo`0000000P3oool0
0`000000oooo0?ooo`0d0?ooo`050000003oool0oooo0?ooo`000000>P3oool010000000oooo0?oo
o`0000020?ooo`030000003oool0oooo0040oooo000O0?ooo`030000003oool0000003/0oooo00<0
00000?ooo`3oool0=`3oool3000000<0oooo00@000000?ooo`3oool00000=03oool3000000<0oooo
00<000000?ooo`3oool0=@3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0000
00<0oooo000P0?ooo`800000>`3oool4000003P0oooo00<000000?ooo`3oool00P3oool2000003L0
oooo00@000000?ooo`3oool0oooo1000000e0?ooo`800000103oool2000000@0oooo000P0?ooo`80
0000oP3oool00200oooo0P00003n0?ooo`00803oool200000?h0oooo000P0?ooo`800000oP3oool0
00h0ooooo`00000C000000005@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`050?ooo`008@3oool00`000000oooo
0?ooo`3l0?ooo`008@3oool200000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?oo
o`008@3oool300000?`0oooo000Q0?ooo`030000003oool000000?`0oooo000Q0?ooo`040000003o
ool0oooo00000?/0oooo000Q0?ooo`040000003oool0oooo00000?/0oooo000Q0?ooo`050000003o
ool0oooo0?ooo`000000nP3oool00240oooo00D000000?ooo`3oool0oooo0000003j0?ooo`008@3o
ool200000080oooo00<000000?ooo`3oool0n03oool00240oooo00D000000?ooo`3oool0oooo0000
003j0?ooo`008@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?L0oooo000Q0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0m`3oool00240oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`3f0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo0?H0oooo000Q0?ooo`8000001@3oool00`000000oooo0?ooo`3e0?ooo`008@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0?@0oooo000Q0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0m03oool00240oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`3c0?ooo`008@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?80oooo
000Q0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0l@3oool00240oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`3a0?ooo`008@3oool2000000X0oooo00<000000?oo
o`3oool0h`3oool6000000L0oooo000Q0?ooo`030000003oool0oooo00X0oooo0P0000330?ooob00
00003@3oool00240oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`2P0?ooob000000
;@3oool00240oooo00<000000?ooo`3oool03@3oool3000008T0oooo5P00001=0?ooo`00203oool2
000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo0`0000050?ooo`030000003oool0
oooo0100oooo0`00001a0?oooaD00000H`3oool000L0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3o
ool2000005T0oooo5P00001h0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000000D0oooo00<000000?ooo`3oool00P3oool3000001D0oooo0`0000100?oooaH00000
SP3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`<0
00001@3oool00`000000oooo0?ooo`0H0?ooo`P00000;@3oool;00000:@0oooo00070?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0200oooo;@00002_0?ooo`001`3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3l
0?ooo`00203oool2000000X0oooo0P0000030?ooo`@00000103oool00`000000oooo0?ooo`3l0?oo
o`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool200000?d0oooo003o0?ooob40oooo003o
0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.70386, -0.215639, \
0.0809371, 0.00159662}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{th, 
        Re[N[alomz[1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 
              1, \(-9\)/10]]]}, {th, 0.01, Pi/2, 0.1}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.6604701058125517`}, {0.11`, 
        0.6586560456457287`}, {0.21000000000000002`, 
        0.6537490370141033`}, {0.31`, 
        0.6455540225049472`}, {0.41000000000000003`, 
        0.6337363237114501`}, {0.51`, 0.6177942854354646`}, {0.61`, 
        0.597017484166324`}, {0.7100000000000001`, 
        0.5704272735223761`}, {0.81`, 0.5366981967542259`}, {0.91`, 
        0.4940685156628228`}, {1.01`, 0.44028353334747766`}, {1.11`, 
        0.37272949018129353`}, {1.21`, 0.28925087869931243`}, {1.31`, 
        0.19095705836975105`}, {1.4100000000000001`, 
        0.08921522010616728`}, {1.51`, 0.014575646800141034`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<malpha.dat\>", %]\)], "Input"],

Cell[BoxData[
    \("malpha.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %53]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.268574 0.198693 [
[.29592 .25607 -3 -9 ]
[.29592 .25607 3 0 ]
[.52268 .25607 -6 -9 ]
[.52268 .25607 6 0 ]
[.74943 .25607 -6 -9 ]
[.74943 .25607 6 0 ]
[.97619 .25607 -6 -9 ]
[.97619 .25607 6 0 ]
[.05666 .06988 -12 -4.5 ]
[.05666 .06988 0 4.5 ]
[.05666 .16923 -24 -4.5 ]
[.05666 .16923 0 4.5 ]
[.05666 .36792 -18 -4.5 ]
[.05666 .36792 0 4.5 ]
[.05666 .46727 -6 -4.5 ]
[.05666 .46727 0 4.5 ]
[.05666 .56661 -18 -4.5 ]
[.05666 .56661 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .26857 m
.29592 .27482 L
s
[(5)] .29592 .25607 0 1 Mshowa
.52268 .26857 m
.52268 .27482 L
s
[(10)] .52268 .25607 0 1 Mshowa
.74943 .26857 m
.74943 .27482 L
s
[(15)] .74943 .25607 0 1 Mshowa
.97619 .26857 m
.97619 .27482 L
s
[(20)] .97619 .25607 0 1 Mshowa
.125 Mabswid
.11451 .26857 m
.11451 .27232 L
s
.15986 .26857 m
.15986 .27232 L
s
.20522 .26857 m
.20522 .27232 L
s
.25057 .26857 m
.25057 .27232 L
s
.34127 .26857 m
.34127 .27232 L
s
.38662 .26857 m
.38662 .27232 L
s
.43197 .26857 m
.43197 .27232 L
s
.47732 .26857 m
.47732 .27232 L
s
.56803 .26857 m
.56803 .27232 L
s
.61338 .26857 m
.61338 .27232 L
s
.65873 .26857 m
.65873 .27232 L
s
.70408 .26857 m
.70408 .27232 L
s
.79478 .26857 m
.79478 .27232 L
s
.84014 .26857 m
.84014 .27232 L
s
.88549 .26857 m
.88549 .27232 L
s
.93084 .26857 m
.93084 .27232 L
s
.02381 .26857 m
.02381 .27232 L
s
.25 Mabswid
0 .26857 m
1 .26857 L
s
.06916 .06988 m
.07541 .06988 L
s
[(-1)] .05666 .06988 1 0 Mshowa
.06916 .16923 m
.07541 .16923 L
s
[(-0.5)] .05666 .16923 1 0 Mshowa
.06916 .36792 m
.07541 .36792 L
s
[(0.5)] .05666 .36792 1 0 Mshowa
.06916 .46727 m
.07541 .46727 L
s
[(1)] .05666 .46727 1 0 Mshowa
.06916 .56661 m
.07541 .56661 L
s
[(1.5)] .05666 .56661 1 0 Mshowa
.125 Mabswid
.06916 .08975 m
.07291 .08975 L
s
.06916 .10962 m
.07291 .10962 L
s
.06916 .12949 m
.07291 .12949 L
s
.06916 .14936 m
.07291 .14936 L
s
.06916 .1891 m
.07291 .1891 L
s
.06916 .20897 m
.07291 .20897 L
s
.06916 .22884 m
.07291 .22884 L
s
.06916 .2487 m
.07291 .2487 L
s
.06916 .28844 m
.07291 .28844 L
s
.06916 .30831 m
.07291 .30831 L
s
.06916 .32818 m
.07291 .32818 L
s
.06916 .34805 m
.07291 .34805 L
s
.06916 .38779 m
.07291 .38779 L
s
.06916 .40766 m
.07291 .40766 L
s
.06916 .42753 m
.07291 .42753 L
s
.06916 .4474 m
.07291 .4474 L
s
.06916 .48714 m
.07291 .48714 L
s
.06916 .50701 m
.07291 .50701 L
s
.06916 .52687 m
.07291 .52687 L
s
.06916 .54674 m
.07291 .54674 L
s
.06916 .05001 m
.07291 .05001 L
s
.06916 .03014 m
.07291 .03014 L
s
.06916 .01027 m
.07291 .01027 L
s
.06916 .58648 m
.07291 .58648 L
s
.06916 .60635 m
.07291 .60635 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02472 0 m
.02499 .11517 L
.02605 .14929 L
.02729 .17219 L
.02846 .18697 L
.02954 .19734 L
.03053 .20508 L
.03279 .21837 L
.03506 .22802 L
.03756 .23599 L
.04017 .24235 L
.04262 .24707 L
.04761 .25422 L
.05217 .25888 L
.05744 .26285 L
.06244 .2657 L
.0675 .26795 L
.07289 .26983 L
.08259 .27233 L
.08853 .27347 L
.09412 .27434 L
.10458 .27561 L
.11434 .27648 L
.12466 .27717 L
.13346 .27762 L
.14313 .27801 L
.15334 .27832 L
.16264 .27855 L
.17338 .27875 L
.18355 .2789 L
.19321 .279 L
.20346 .27908 L
.21222 .27914 L
.22188 .27918 L
.2268 .2792 L
.23198 .27921 L
.2364 .27922 L
.23888 .27922 L
.24123 .27922 L
.24397 .27923 L
.24646 .27923 L
.24785 .27923 L
.24913 .27923 L
.2498 .27923 L
.25054 .27923 L
.25131 .27923 L
.25204 .27923 L
.25333 .27923 L
.25457 .27923 L
.25567 .27923 L
Mistroke
.25688 .27923 L
.25819 .27923 L
.25958 .27923 L
.26209 .27923 L
.2647 .27922 L
.26746 .27922 L
.2724 .27921 L
.28354 .2792 L
.29317 .27917 L
.30372 .27915 L
.3442 .27901 L
.38316 .27885 L
.42457 .27867 L
.46447 .2785 L
.50285 .27833 L
.54369 .27816 L
.583 .278 L
.62081 .27784 L
.66106 .27769 L
.6998 .27755 L
.74099 .2774 L
.78067 .27727 L
.81883 .27715 L
.85944 .27702 L
.89854 .27691 L
.94009 .27679 L
.97619 .27669 L
Mfstroke
.02483 0 m
.02499 .06858 L
.02605 .10127 L
.02729 .12366 L
.02846 .13861 L
.03053 .15795 L
.03279 .17337 L
.0379 .19807 L
.04262 .21447 L
.05224 .23894 L
.06244 .25812 L
.0825 .28611 L
.10458 .30934 L
.14487 .34172 L
.18364 .36633 L
.22486 .38843 L
.26457 .40708 L
.30277 .42325 L
.34341 .43902 L
.38254 .45308 L
.42412 .46704 L
.46419 .4797 L
.50274 .49125 L
.54374 .50296 L
.58323 .51374 L
.6212 .52371 L
.66163 .53393 L
.70054 .54342 L
.7419 .55318 L
.78174 .56229 L
.82007 .5708 L
.86085 .57961 L
.90012 .58788 L
.93787 .59563 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0
oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo
000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool0
0`000000oooo0?ooo`090?ooo`800000o`3oool40?ooo`003`3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3o
ool000l0oooo00<000000?ooo`3oool02@3oool200000?l0oooo103oool000l0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`003`3oool00`000000oooo0?ooo`090?ooo`800000o`3oool40?ooo`003`3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`040000003oool0oooo0?ooo`@00000
103oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool00`3oool300000?l0oooo0`3oool000d0oooo100000040?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
3`3oool010000000oooo0?ooo`3oool3000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
3`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3o
ool000l0oooo00<000000?ooo`3oool02@3oool200000?l0oooo103oool000l0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo0P00003o0?ooo`@0oooo000?0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool200000?l0oooo103oool000l0
oooo0P00000:0?ooo`030000003oool0oooo0?l0oooo0`3oool000l0oooo0P00000:0?ooo`030000
003oool0oooo0?l0oooo0`3oool000l0oooo0P00000:0?ooo`030000003oool0oooo0?l0oooo0`3o
ool000l0oooo0P00000:0?ooo`030000003oool0oooo0?l0oooo0`3oool000l0oooo0P00000:0?oo
o`030000003oool0oooo0?l0oooo0`3oool000l0oooo0P00000:0?ooo`800000o`3oool40?ooo`00
3`3oool2000000X0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool2000000X0oooo00<0
00000?ooo`3oool0o`3oool30?ooo`00203oool2000000@0oooo0`0000020?ooo`<000001@3oool0
0`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000@0oooo0P000005
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool30?ooo`001`3oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0000000@0oooo00<000000?ooo`3oool00P3oool3
00000?l0oooo0`3oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`000004
0?ooo`040000003oool000000?ooo`<000001@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo0007
0?ooo`040000003oool0oooo000000@0oooo00D000000?ooo`000000oooo000000070?ooo`030000
003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool00000103oool01@000000oooo
0000003oool0000000L0oooo00<000000?ooo`3oool0o`3oool30?ooo`00203oool2000000H0oooo
0P0000000`3oool0000000000002000000@0oooo00<000000?ooo`3oool0o`3oool30?ooo`00403o
ool00`000000oooo000000080?ooo`030000003oool0oooo0?l0oooo0`3oool00100oooo00<00000
0?ooo`000000203oool200000?l0oooo103oool00100oooo00<000000?ooo`000000203oool00`00
0000oooo0?ooo`3o0?ooo`<0oooo000@0?ooo`030000003oool0000000P0oooo00<000000?ooo`3o
ool0o`3oool30?ooo`00403oool00`000000oooo000000080?ooo`030000003oool0oooo0?l0oooo
0`3oool00100oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo
000@0?ooo`040000003oool0oooo000000L0oooo0P00003o0?ooo`@0oooo000@0?ooo`050000003o
ool0oooo0?ooo`0000001P3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000A0?ooo`040000003o
ool0oooo000000H0oooo00<000000?ooo`3oool0o`3oool30?ooo`004@3oool010000000oooo0?oo
o`0000060?ooo`030000003oool0oooo0?l0oooo0`3oool00140oooo00D000000?ooo`3oool0oooo
000000050?ooo`030000003oool0oooo0?l0oooo0`3oool00140oooo00D000000?ooo`3oool0oooo
000000050?ooo`030000003oool0oooo0?l0oooo0`3oool00140oooo00D000000?ooo`3oool0oooo
000000050?ooo`800000>`3oool3000003T0oooo100000030?ooo`800000=`3oool400000080oooo
0`00000f0?ooo`@000000`3oool2000000@0oooo000B0?ooo`050000003oool0oooo0?ooo`000000
103oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo03P0oooo00D000000?ooo`3oool0
oooo000000020?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool000000`3oool00180
oooo00D000000?ooo`3oool0oooo000000040?ooo`030000003oool0oooo03d0oooo00<000000?oo
o`3oool0>03oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0=P3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool00P3oool0
10000000oooo0?ooo`0000030?ooo`004`3oool010000000oooo0?ooo`0000040?ooo`030000003o
ool0oooo03X0oooo0`00000k0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo
0?ooo`0f0?ooo`040000003oool0oooo0?ooo`<00000>03oool01@000000oooo0?ooo`3oool00000
0080oooo00<000000?ooo`3oool00@3oool001<0oooo00D000000?ooo`3oool0oooo000000030?oo
o`030000003oool0oooo03X0oooo00<000000?ooo`3oool0>`3oool01@000000oooo0?ooo`3oool0
00000080oooo00<000000?ooo`3oool0=P3oool01@000000oooo0?ooo`3oool0000003/0oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo0?ooo`010?ooo`00503oool010000000oooo0?oo
o`0000030?ooo`800000>`3oool00`000000oooo0?ooo`0i0?ooo`<000000`3oool010000000oooo
0?ooo`00000f0?ooo`<000000`3oool00`000000oooo0?ooo`0f0?ooo`040000003oool0oooo0000
0080oooo00@000000?ooo`3oool000000`3oool001@0oooo00D000000?ooo`3oool0oooo00000002
0?ooo`030000003oool0oooo03X0oooo1000000j0?ooo`030000003oool0oooo0080oooo0P00000i
0?ooo`040000003oool0oooo0?ooo`@00000=P3oool2000000@0oooo0P0000040?ooo`005@3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0?l0oooo0`3oool001H0oooo00H00000
0?ooo`3oool000000?ooo`00003o0?ooo`D0oooo000G0?ooo`80000000<0oooo000000000000o`3o
ool50?ooo`006@3oool300000?l0oooo1@3oool000P0ooooo`00000I000000003P3oool00`000000
oooo0?ooo`0:0?ooo`80000000<0oooo000000000000100000040?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool01@3oool001/0oooo00<000000?ooo`0000001P3oool@000005/0oooo
RP0000070?ooo`006`3oool00`000000oooo0000000F0?oooe/00000T@3oool001/0oooo00@00000
0?ooo`3oool00000o`3oool20?ooo`006`3oool200000080oooo00<000000?ooo`3oool0oP3oool0
01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3m0?ooo`006`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?`0oooo000K0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0n`3oool001/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`3j0?ooo`006`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?T0oooo000K0?oo
o`800000203oool00`000000oooo0?ooo`3h0?ooo`006`3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo0?L0oooo000K0?ooo`030000003oool0oooo00T0oooo0P00003g0?ooo`006`3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?@0oooo000K0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0l`3oool001/0oooo0P00000>0?ooo`030000003oool0oooo
0?80oooo000K0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0l@3oool001/0oooo
00<000000?ooo`3oool03`3oool200000?40oooo000K0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool0kP3oool001/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3]
0?ooo`006`3oool00`000000oooo0?ooo`0C0?ooo`800000k@3oool001/0oooo0P00000F0?ooo`03
0000003oool0oooo0>X0oooo000K0?ooo`030000003oool0oooo01H0oooo0P00003Z0?ooo`006`3o
ool00`000000oooo0?ooo`0H0?ooo`800000j03oool000P0oooo0P0000040?ooo`030000003oool0
oooo0080oooo0`0000050?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0i@3oool0
00L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo01/0oooo0P00003U0?ooo`001`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0
oooo0080oooo0`00000M0?ooo`030000003oool0oooo0>80oooo00070?ooo`040000003oool0oooo
000000P0oooo0`0000050?ooo`030000003oool0oooo01h0oooo0P00003R0?ooo`001`3oool01000
0000oooo0?ooo`0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0803oool2
00000>00oooo00070?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0R0?ooo`800000gP3oool000P0oooo0P0000090?ooo`@00000103oool00`00
0000oooo0?ooo`0T0?ooo`800000g03oool001/0oooo00<000000?ooo`3oool09P3oool200000=X0
oooo000K0?ooo`800000:@3oool200000=P0oooo000K0?ooo`030000003oool0oooo02X0oooo0P00
003F0?ooo`006`3oool00`000000oooo0?ooo`0/0?ooo`800000e03oool001/0oooo00<000000?oo
o`3oool0;P3oool200000=80oooo000K0?ooo`030000003oool0oooo0300oooo0P00003@0?ooo`00
6`3oool00`000000oooo0?ooo`0b0?ooo`<00000c@3oool001/0oooo0P00000f0?ooo`800000b`3o
ool001/0oooo00<000000?ooo`3oool0=`3oool200000<T0oooo000K0?ooo`030000003oool0oooo
03T0oooo0P0000370?ooo`006`3oool00`000000oooo0?ooo`0k0?ooo`800000a@3oool001/0oooo
00<000000?ooo`3oool0?@3oool300000<80oooo000K0?ooo`800000@@3oool300000;l0oooo000K
0?ooo`030000003oool0oooo04<0oooo0P00002m0?ooo`006`3oool00`000000oooo0?ooo`150?oo
o`<00000^P3oool001/0oooo00<000000?ooo`3oool0B03oool300000;L0oooo000K0?ooo`030000
003oool0oooo04/0oooo0`00002d0?ooo`006`3oool00`000000oooo0?ooo`1>0?ooo`800000/P3o
ool001/0oooo0P00001A0?ooo`<00000[`3oool001/0oooo00<000000?ooo`3oool0D`3oool30000
0:`0oooo000K0?ooo`030000003oool0oooo05H0oooo0`00002Y0?ooo`004`3oool4000000@0oooo
00<000000?ooo`3oool0F@3oool300000:H0oooo000E0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0G03oool300000:<0oooo000E0?ooo`030000003oool0oooo00<0oooo0`00001O
0?ooo`<00000X03oool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1R0?oo
o`<00000W@3oool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1U0?ooo`80
0000V`3oool001<0oooo0`0000050?ooo`030000003oool0oooo06L0oooo0`00002H0?ooo`005@3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo06X0oooo0`00002E0?ooo`006`3oool0
0`000000oooo0?ooo`1]0?ooo`@00000T@3oool001/0oooo0P00001b0?ooo`@00000S@3oool001/0
oooo00<000000?ooo`3oool0M@3oool3000008X0oooo000K0?ooo`030000003oool0oooo07P0oooo
100000260?ooo`006`3oool00`000000oooo0?ooo`1l0?ooo`@00000PP3oool001/0oooo00<00000
0?ooo`3oool0P03oool3000007l0oooo000K0?ooo`800000Q03oool4000007/0oooo000K0?ooo`03
0000003oool0oooo08L0oooo1000001g0?ooo`006`3oool00`000000oooo0?ooo`2;0?ooo`<00000
M03oool001/0oooo00<000000?ooo`3oool0SP3oool400000700oooo000K0?ooo`030000003oool0
oooo0980oooo1000001/0?ooo`006`3oool00`000000oooo0?ooo`2F0?ooo`<00000J@3oool001/0
oooo0P00002J0?ooo`@00000I@3oool001/0oooo00<000000?ooo`3oool0W@3oool400000640oooo
000K0?ooo`030000003oool0oooo0:40oooo1000001M0?ooo`006`3oool00`000000oooo0?ooo`2U
0?ooo`H00000E`3oool001/0oooo00<000000?ooo`3oool0Z`3oool4000005<0oooo000K0?ooo`80
0000/03oool4000004l0oooo000K0?ooo`030000003oool0oooo0;<0oooo1000001;0?ooo`006`3o
ool00`000000oooo0?ooo`2g0?ooo`@00000A`3oool001/0oooo00<000000?ooo`3oool0^`3oool4
000004<0oooo00070?ooo`@000000`3oool00`000000oooo0?ooo`020?ooo`<000001@3oool00`00
0000oooo0?ooo`2o0?ooo`@00000?`3oool000T0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0<<0oooo1000000k0?ooo`002@3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo0080oooo0`0000370?ooo`D00000=P3oool000T0oooo00<0
00000?ooo`3oool01`3oool3000000D0oooo00<000000?ooo`3oool0c03oool500000340oooo0009
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3A
0?ooo`H00000:`3oool000L0oooo0`0000090?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool0e`3oool5000002H0oooo00090?ooo`030000003oool0oooo00L0oooo100000040?ooo`03
0000003oool0oooo0=`0oooo1000000R0?ooo`006`3oool200000>40oooo1000000N0?ooo`006`3o
ool00`000000oooo0?ooo`3T0?ooo`@000006P3oool001/0oooo00<000000?ooo`3oool0j03oool5
000001D0oooo000K0?ooo`030000003oool0oooo0>d0oooo1@00000@0?ooo`006`3oool00`000000
oooo0?ooo`3b0?ooo`H000002P3oool001/0oooo00<000000?ooo`3oool0n03oool3000000L0oooo
000K0?ooo`800000o`3oool40?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?oo
o`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.17393, -1.39354, \
0.0790907, 0.0180524}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(\(aha!\)\ changes\ \(much!\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-alomz[1, 0.9, 1, a]\), {a, \(-1\), 20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.39104 0.531405 [
[.29592 .37854 -3 -9 ]
[.29592 .37854 3 0 ]
[.52268 .37854 -6 -9 ]
[.52268 .37854 6 0 ]
[.74943 .37854 -6 -9 ]
[.74943 .37854 6 0 ]
[.97619 .37854 -6 -9 ]
[.97619 .37854 6 0 ]
[.05666 .0722 -24 -4.5 ]
[.05666 .0722 0 4.5 ]
[.05666 .17848 -24 -4.5 ]
[.05666 .17848 0 4.5 ]
[.05666 .28476 -24 -4.5 ]
[.05666 .28476 0 4.5 ]
[.05666 .49732 -18 -4.5 ]
[.05666 .49732 0 4.5 ]
[.05666 .6036 -18 -4.5 ]
[.05666 .6036 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .39104 m
.29592 .39729 L
s
[(5)] .29592 .37854 0 1 Mshowa
.52268 .39104 m
.52268 .39729 L
s
[(10)] .52268 .37854 0 1 Mshowa
.74943 .39104 m
.74943 .39729 L
s
[(15)] .74943 .37854 0 1 Mshowa
.97619 .39104 m
.97619 .39729 L
s
[(20)] .97619 .37854 0 1 Mshowa
.125 Mabswid
.11451 .39104 m
.11451 .39479 L
s
.15986 .39104 m
.15986 .39479 L
s
.20522 .39104 m
.20522 .39479 L
s
.25057 .39104 m
.25057 .39479 L
s
.34127 .39104 m
.34127 .39479 L
s
.38662 .39104 m
.38662 .39479 L
s
.43197 .39104 m
.43197 .39479 L
s
.47732 .39104 m
.47732 .39479 L
s
.56803 .39104 m
.56803 .39479 L
s
.61338 .39104 m
.61338 .39479 L
s
.65873 .39104 m
.65873 .39479 L
s
.70408 .39104 m
.70408 .39479 L
s
.79478 .39104 m
.79478 .39479 L
s
.84014 .39104 m
.84014 .39479 L
s
.88549 .39104 m
.88549 .39479 L
s
.93084 .39104 m
.93084 .39479 L
s
.02381 .39104 m
.02381 .39479 L
s
.25 Mabswid
0 .39104 m
1 .39104 L
s
.06916 .0722 m
.07541 .0722 L
s
[(-0.6)] .05666 .0722 1 0 Mshowa
.06916 .17848 m
.07541 .17848 L
s
[(-0.4)] .05666 .17848 1 0 Mshowa
.06916 .28476 m
.07541 .28476 L
s
[(-0.2)] .05666 .28476 1 0 Mshowa
.06916 .49732 m
.07541 .49732 L
s
[(0.2)] .05666 .49732 1 0 Mshowa
.06916 .6036 m
.07541 .6036 L
s
[(0.4)] .05666 .6036 1 0 Mshowa
.125 Mabswid
.06916 .09877 m
.07291 .09877 L
s
.06916 .12534 m
.07291 .12534 L
s
.06916 .15191 m
.07291 .15191 L
s
.06916 .20505 m
.07291 .20505 L
s
.06916 .23162 m
.07291 .23162 L
s
.06916 .25819 m
.07291 .25819 L
s
.06916 .31133 m
.07291 .31133 L
s
.06916 .3379 m
.07291 .3379 L
s
.06916 .36447 m
.07291 .36447 L
s
.06916 .41761 m
.07291 .41761 L
s
.06916 .44418 m
.07291 .44418 L
s
.06916 .47075 m
.07291 .47075 L
s
.06916 .52389 m
.07291 .52389 L
s
.06916 .55046 m
.07291 .55046 L
s
.06916 .57703 m
.07291 .57703 L
s
.06916 .04563 m
.07291 .04563 L
s
.06916 .01906 m
.07291 .01906 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02687 0 m
.02729 .02055 L
.02846 .06087 L
.03053 .11296 L
.03279 .15425 L
.03527 .18948 L
.0379 .21957 L
.04262 .26196 L
.05205 .32193 L
.05742 .34751 L
.06244 .36787 L
.08274 .42801 L
.09415 .45206 L
.10458 .47018 L
.12357 .49638 L
.14429 .518 L
.16396 .53388 L
.18493 .54725 L
.20491 .55744 L
.22652 .56635 L
.26658 .57862 L
.28652 .58321 L
.30759 .58723 L
.34556 .59278 L
.36507 .59497 L
.38598 .59691 L
.40579 .59842 L
.42734 .59976 L
.44628 .6007 L
.46719 .60154 L
.48696 .60215 L
.50797 .60264 L
.52732 .60296 L
.5382 .60309 L
.54817 .60318 L
.55291 .60322 L
.55798 .60325 L
.56235 .60327 L
.5671 .60329 L
.57 .6033 L
.57268 .60331 L
.57523 .60331 L
.57654 .60331 L
.57794 .60332 L
.57921 .60332 L
.58037 .60332 L
.58153 .60332 L
.58264 .60332 L
.58392 .60332 L
.58511 .60332 L
Mistroke
.58641 .60332 L
.58714 .60332 L
.5878 .60332 L
.59013 .60331 L
.59145 .60331 L
.59269 .60331 L
.59785 .6033 L
.60231 .60328 L
.60708 .60327 L
.61799 .60321 L
.62792 .60314 L
.64704 .60296 L
.66781 .60272 L
.70865 .60209 L
.74797 .60134 L
.78577 .60051 L
.82602 .59955 L
.86476 .59855 L
.90596 .59743 L
.94563 .59632 L
.97619 .59544 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0
oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo
000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0
oooo0P00003o0?ooo`@0oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3o
ool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000@0?ooo`030000003oool0oooo00P0oooo0P00
003o0?ooo`@0oooo000@0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`00403oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?l0oooo0`3oool00100
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3o0?ooo`<0oooo000@0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool0o`3oool30?ooo`00403oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo0?l0oooo0`3oool000P0oooo0P0000040?ooo`030000003oool0
000000<0oooo0P0000050?ooo`030000003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?oo
o`3oool000001@3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0o`3oool30?ooo`001`3oool010000000oooo0?ooo`0000050?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool00P3oool300000?l0oooo0`3oool000030?ooo`00
00000000008000000P3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo00<00000
1@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000D0oooo
00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000
003oool0oooo000000D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo00080?ooo`8000001P3oool010000000oooo0?ooo`3oool3000000@0oooo00<000000?oo
o`3oool0o`3oool30?ooo`00403oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?l0
oooo0`3oool00100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3o0?ooo`<0oooo
000@0?ooo`030000003oool0oooo00P0oooo0P00003o0?ooo`@0oooo000@0?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool0o`3oool30?ooo`00403oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0?l0oooo0`3oool00100oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`3o0?ooo`<0oooo000@0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
o`3oool30?ooo`00403oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?l0oooo0`3o
ool00100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3o0?ooo`<0oooo000@0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool0o`3oool30?ooo`00403oool00`000000
oooo0?ooo`080?ooo`800000o`3oool40?ooo`00403oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo0?l0oooo0`3oool00100oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000A0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`004@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo0`3oool00140
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000A0?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0o`3oool30?ooo`004@3oool00`000000oooo0?oo
o`070?ooo`800000o`3oool40?ooo`004@3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo0?l0oooo0`3oool00140oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo000A0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
4@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo0`3oool000P0oooo0P00
00040?ooo`040000003oool0oooo000000<0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool0
00001@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000L0
oooo1@0000040?ooo`<00000o`3oool30?ooo`0000<0oooo0000000000000P0000020?ooo`040000
003oool0oooo000000L0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo00070?ooo`040000003oool0oooo000000L0oooo0P0000000`3oool000000?ooo`040?oo
o`030000003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool000001`3oool01000
0000oooo0000000000050?ooo`030000003oool0oooo0?l0oooo0`3oool000P0oooo0P0000080?oo
o`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`004P3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo0`3oool00180oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000B0?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0o`3oool30?ooo`004P3oool00`000000oooo0?ooo`060?ooo`800000o`3o
ool40?ooo`004P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo0`3oool0
0180oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000B0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3oool30?ooo`004P3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0?l0oooo0`3oool00180oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`3o0?ooo`<0oooo000B0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0o`3oool30?ooo`004P3oool00`000000oooo0?ooo`060?ooo`800000o`3oool40?oo
o`004`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?l0oooo0`3oool001<0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000C0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0o`3oool30?ooo`004`3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo0?l0oooo0`3oool001<0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`3o0?ooo`<0oooo000C0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0o`3oool30?ooo`00503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo
0`3oool001@0oooo00<000000?ooo`3oool0103oool200000?l0oooo103oool001@0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`<0oooo000D0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0o`3oool30?ooo`00503oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?l0oooo0`3oool001@0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`3o0?ooo`<0oooo00080?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103o
ool00`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000P0oooo00<0
00000?ooo`0000001@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0
oooo000000T0oooo0P0000050?ooo`<00000o`3oool30?ooo`0000<0oooo0000000000000P000002
0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000X0oooo0P0000040?ooo`030000003o
ool0oooo0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0000
000000040?ooo`030000003oool0oooo0?l0oooo0`3oool000P0oooo0P00000:0?ooo`8000001@3o
ool00`000000oooo0?ooo`3o0?ooo`<0oooo000E0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0o`3oool30?ooo`005@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
0?l0oooo0`3oool001H0oooo00<000000?ooo`3oool00P3oool200000?l0oooo103oool001H0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000F0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0o`3oool30?ooo`005P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0?l0oooo0`3oool001H0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`3o0?ooo`<0oooo000F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0o`3oool30?ooo`005`3oool01@000000oooo0?ooo`3oool000000?l0oooo1@3oool001L0oooo
00D000000?ooo`3oool0oooo0000003o0?ooo`D0oooo000G0?ooo`040000003oool0oooo0?ooo`80
0000o`3oool40?ooo`005`3oool01@000000oooo0?ooo`3oool000000?l0oooo1@3oool001P0oooo
00@000000?ooo`3oool00000o`3oool50?ooo`00603oool010000000oooo0?ooo`00003o0?ooo`D0
oooo000H0?ooo`040000003oool0oooo000003`0oooo0`00000i0?ooo`@000000`3oool2000003L0
oooo100000020?ooo`<00000=P3oool4000000<0oooo0P0000040?ooo`00603oool010000000oooo
0?ooo`00000o0?ooo`030000003oool0oooo03P0oooo00D000000?ooo`3oool0oooo000000020?oo
o`030000003oool0oooo03H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo00<0oooo00@000000?ooo`3oool000000`3oool001T0oooo00<000000?oo
o`000000?`3oool00`000000oooo0?ooo`0h0?ooo`050000003oool0oooo0?ooo`0000000P3oool0
0`000000oooo0?ooo`0f0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0=03oool0
0`000000oooo0?ooo`020?ooo`040000003oool0oooo000000<0oooo000I0?ooo`040000003oool0
0000000003/0oooo0`00000k0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo
0?ooo`0f0?ooo`040000003oool0oooo0?ooo`<00000>03oool01@000000oooo0?ooo`3oool00000
0080oooo00<000000?ooo`3oool00@3oool001T0oooo00<000000?ooo`000000?03oool00`000000
oooo0?ooo`0k0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`0f0?oo
o`050000003oool0oooo0?ooo`000000>`3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0040oooo000I0?ooo`030000003oool0000003`0oooo00<000000?ooo`3oool0>@3oool3
000000<0oooo00@000000?ooo`3oool00000=P3oool3000000<0oooo00<000000?ooo`3oool0=P3o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000<0oooo000J0?ooo`800000
?03oool4000003X0oooo00<000000?ooo`3oool00P3oool2000003T0oooo00@000000?ooo`3oool0
oooo1000000f0?ooo`800000103oool2000000@0oooo000J0?ooo`800000o`3oool50?ooo`006P3o
ool200000?l0oooo1@3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`00
0000oooo0?ooo`3o0?ooo`<0oooo00080?ooool000006@00000000h0oooo00<000000?ooo`3oool0
2P3oool2000000/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`050?ooo`006`3o
ool200000?l0oooo103oool001/0oooo00<000000?ooo`000000o`3oool30?ooo`006`3oool00`00
0000oooo0000003o0?ooo`<0oooo000K0?ooo`030000003oool000000?l0oooo0`3oool001/0oooo
00@000000?ooo`3oool00000o`3oool20?ooo`006`3oool2000000030?ooo`000000oooo0?l0oooo
0@3oool001/0oooo00@000000?ooo`3oool00000o`3oool20?ooo`006`3oool01@000000oooo0?oo
o`3oool000000?l0oooo0@3oool001/0oooo00D000000?ooo`3oool0oooo0000003o0?ooo`40oooo
000K0?ooo`050000003oool0oooo0?ooo`000000o`3oool10?ooo`006`3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0?d0oooo000K0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0o@3oool001/0oooo0P0000040?ooo`030000003oool0oooo0?`0oooo000K0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0o03oool001/0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`3k0?ooo`006`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0?/0oooo000K0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0nP3oool0
01/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3j0?ooo`006`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo0?T0oooo000K0?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0n@3oool001/0oooo0P0000080?ooo`030000003oool0oooo0?P0oooo000K
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0m`3oool001/0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`3g0?ooo`006`3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo0?H0oooo000K0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
m@3oool000P0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0m03oool000L0oooo00@000000?ooo`3oool00000203o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0m03o
ool000L0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`040?ooo`<00000303o
ool00`000000oooo0?ooo`3c0?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3b0?ooo`001`3oool0
10000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`3a0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000000@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3`0?ooo`00203o
ool2000000X0oooo0P0000050?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0k`3o
ool001/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3^0?ooo`006`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo0>d0oooo000K0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool0k03oool001/0oooo0P00000E0?ooo`800000k03oool001/0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`3Y0?ooo`006`3oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo0>P0oooo000K0?ooo`030000003oool0oooo01P0oooo0P00003X0?oo
o`006`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>D0oooo000K0?ooo`030000
003oool0oooo01/0oooo0P00003U0?ooo`006`3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo0>80oooo000K0?ooo`8000007`3oool200000>80oooo000K0?ooo`030000003oool0oooo
0200oooo0P00003P0?ooo`006`3oool00`000000oooo0?ooo`0R0?ooo`800000gP3oool001/0oooo
00<000000?ooo`3oool0903oool200000=`0oooo000K0?ooo`030000003oool0oooo02H0oooo0P00
003J0?ooo`006`3oool00`000000oooo0?ooo`0X0?ooo`<00000e`3oool001/0oooo00<000000?oo
o`3oool0:`3oool400000=<0oooo000K0?ooo`030000003oool0oooo02l0oooo1000003?0?ooo`00
6`3oool2000003@0oooo1000003;0?ooo`006`3oool00`000000oooo0?ooo`0g0?ooo`H00000a@3o
ool001/0oooo00<000000?ooo`3oool0?@3oool500000<00oooo000K0?ooo`030000003oool0oooo
0480oooo1P00002j0?ooo`006`3oool00`000000oooo0?ooo`180?ooo`D00000]@3oool000P0oooo
0P0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`1=0?oooa400000P`3ooolJ000000L0oooo00070?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1N0?oooa400000>`3ooolg00000240oooo
00070?ooo`040000003oool0oooo000000L0oooo1@0000040?ooo`<00000K`3ooolk000005P0oooo
00070?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool000001@3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`00
00001@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000T0
oooo0P0000050?ooo`030000003oool0oooo0?l0oooo0`3oool000P0oooo0P00000;0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool30?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.17393, -0.749892, \
0.0790907, 0.0067498}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %53]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.306205 0.176356 [
[.29592 .2937 -3 -9 ]
[.29592 .2937 3 0 ]
[.52268 .2937 -6 -9 ]
[.52268 .2937 6 0 ]
[.74943 .2937 -6 -9 ]
[.74943 .2937 6 0 ]
[.97619 .2937 -6 -9 ]
[.97619 .2937 6 0 ]
[.05666 .04167 -24 -4.5 ]
[.05666 .04167 0 4.5 ]
[.05666 .12985 -12 -4.5 ]
[.05666 .12985 0 4.5 ]
[.05666 .21803 -24 -4.5 ]
[.05666 .21803 0 4.5 ]
[.05666 .39438 -18 -4.5 ]
[.05666 .39438 0 4.5 ]
[.05666 .48256 -6 -4.5 ]
[.05666 .48256 0 4.5 ]
[.05666 .57074 -18 -4.5 ]
[.05666 .57074 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .3062 m
.29592 .31245 L
s
[(5)] .29592 .2937 0 1 Mshowa
.52268 .3062 m
.52268 .31245 L
s
[(10)] .52268 .2937 0 1 Mshowa
.74943 .3062 m
.74943 .31245 L
s
[(15)] .74943 .2937 0 1 Mshowa
.97619 .3062 m
.97619 .31245 L
s
[(20)] .97619 .2937 0 1 Mshowa
.125 Mabswid
.11451 .3062 m
.11451 .30995 L
s
.15986 .3062 m
.15986 .30995 L
s
.20522 .3062 m
.20522 .30995 L
s
.25057 .3062 m
.25057 .30995 L
s
.34127 .3062 m
.34127 .30995 L
s
.38662 .3062 m
.38662 .30995 L
s
.43197 .3062 m
.43197 .30995 L
s
.47732 .3062 m
.47732 .30995 L
s
.56803 .3062 m
.56803 .30995 L
s
.61338 .3062 m
.61338 .30995 L
s
.65873 .3062 m
.65873 .30995 L
s
.70408 .3062 m
.70408 .30995 L
s
.79478 .3062 m
.79478 .30995 L
s
.84014 .3062 m
.84014 .30995 L
s
.88549 .3062 m
.88549 .30995 L
s
.93084 .3062 m
.93084 .30995 L
s
.02381 .3062 m
.02381 .30995 L
s
.25 Mabswid
0 .3062 m
1 .3062 L
s
.06916 .04167 m
.07541 .04167 L
s
[(-1.5)] .05666 .04167 1 0 Mshowa
.06916 .12985 m
.07541 .12985 L
s
[(-1)] .05666 .12985 1 0 Mshowa
.06916 .21803 m
.07541 .21803 L
s
[(-0.5)] .05666 .21803 1 0 Mshowa
.06916 .39438 m
.07541 .39438 L
s
[(0.5)] .05666 .39438 1 0 Mshowa
.06916 .48256 m
.07541 .48256 L
s
[(1)] .05666 .48256 1 0 Mshowa
.06916 .57074 m
.07541 .57074 L
s
[(1.5)] .05666 .57074 1 0 Mshowa
.125 Mabswid
.06916 .05931 m
.07291 .05931 L
s
.06916 .07694 m
.07291 .07694 L
s
.06916 .09458 m
.07291 .09458 L
s
.06916 .11221 m
.07291 .11221 L
s
.06916 .14748 m
.07291 .14748 L
s
.06916 .16512 m
.07291 .16512 L
s
.06916 .18276 m
.07291 .18276 L
s
.06916 .20039 m
.07291 .20039 L
s
.06916 .23566 m
.07291 .23566 L
s
.06916 .2533 m
.07291 .2533 L
s
.06916 .27093 m
.07291 .27093 L
s
.06916 .28857 m
.07291 .28857 L
s
.06916 .32384 m
.07291 .32384 L
s
.06916 .34148 m
.07291 .34148 L
s
.06916 .35911 m
.07291 .35911 L
s
.06916 .37675 m
.07291 .37675 L
s
.06916 .41202 m
.07291 .41202 L
s
.06916 .42965 m
.07291 .42965 L
s
.06916 .44729 m
.07291 .44729 L
s
.06916 .46493 m
.07291 .46493 L
s
.06916 .5002 m
.07291 .5002 L
s
.06916 .51783 m
.07291 .51783 L
s
.06916 .53547 m
.07291 .53547 L
s
.06916 .5531 m
.07291 .5531 L
s
.06916 .02403 m
.07291 .02403 L
s
.06916 .0064 m
.07291 .0064 L
s
.06916 .58837 m
.07291 .58837 L
s
.06916 .60601 m
.07291 .60601 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02464 0 m
.02499 .13395 L
.02605 .16319 L
.02729 .18325 L
.02846 .19663 L
.03053 .21392 L
.03279 .22762 L
.03527 .23931 L
.0379 .2493 L
.04262 .26337 L
.05205 .28327 L
.05742 .29176 L
.06244 .29852 L
.08274 .31847 L
.09415 .32646 L
.10458 .33247 L
.12357 .34116 L
.14429 .34834 L
.16396 .35361 L
.18493 .35805 L
.20491 .36143 L
.22652 .36438 L
.26658 .36846 L
.28652 .36998 L
.30759 .37131 L
.34556 .37316 L
.36507 .37388 L
.38598 .37453 L
.40579 .37503 L
.42734 .37547 L
.44628 .37579 L
.46719 .37606 L
.48696 .37627 L
.50797 .37643 L
.52732 .37653 L
.5382 .37658 L
.54817 .37661 L
.55291 .37662 L
.55798 .37663 L
.56235 .37664 L
.5671 .37664 L
.57 .37665 L
.57268 .37665 L
.57523 .37665 L
.57654 .37665 L
.57794 .37665 L
.57921 .37665 L
.58037 .37665 L
.58153 .37665 L
.58264 .37665 L
Mistroke
.58392 .37665 L
.58511 .37665 L
.58641 .37665 L
.58714 .37665 L
.5878 .37665 L
.59013 .37665 L
.59145 .37665 L
.59269 .37665 L
.59785 .37665 L
.60231 .37664 L
.60708 .37664 L
.61799 .37662 L
.62792 .37659 L
.64704 .37653 L
.66781 .37645 L
.70865 .37625 L
.74797 .376 L
.78577 .37572 L
.82602 .3754 L
.86476 .37507 L
.90596 .3747 L
.94563 .37433 L
.97619 .37404 L
Mfstroke
.02465 0 m
.02499 .12869 L
.02605 .15771 L
.02729 .17758 L
.02846 .19085 L
.03053 .20802 L
.03279 .2217 L
.0379 .24363 L
.04262 .25818 L
.05224 .27991 L
.06244 .29693 L
.0825 .32177 L
.10458 .34239 L
.14487 .37113 L
.18364 .39297 L
.22486 .41259 L
.26457 .42914 L
.30277 .44349 L
.34341 .45749 L
.38254 .46997 L
.42412 .48236 L
.46419 .49359 L
.50274 .50385 L
.54374 .51424 L
.58323 .52381 L
.6212 .53266 L
.66163 .54173 L
.70054 .55015 L
.7419 .55882 L
.78174 .5669 L
.82007 .57446 L
.86085 .58228 L
.90012 .58961 L
.93787 .59649 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0
oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo
000?0?ooo`030000003oool0oooo00T0oooo0P00003o0?ooo`@0oooo000?0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`800000o`3oool40?ooo`003`3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000L0oooo100000030?oo
o`8000000`3oool3000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`002@3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`020?ooo`<00000o`3oool30?ooo`0000<0oooo0000000000000P00
00040?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool0oooo0`0000050?ooo`030000
003oool0oooo0?l0oooo0`3oool000T0oooo00<000000?ooo`3oool00`3oool01@000000oooo0?oo
o`3oool0000000L0oooo00<000000?ooo`3oool0o`3oool30?ooo`001`3oool3000000D0oooo00D0
00000?ooo`3oool0oooo000000070?ooo`030000003oool0oooo0?l0oooo0`3oool000T0oooo00<0
00000?ooo`3oool00`3oool010000000oooo0?ooo`3oool4000000@0oooo00<000000?ooo`3oool0
o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`800000o`3oool40?ooo`003`3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool200000?l0oooo103o
ool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo0P00
003o0?ooo`@0oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000l0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?oo
o`090?ooo`800000o`3oool40?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo000?0?ooo`040000003oool0oooo0?ooo`@00000103oool00`000000oooo0?ooo`3o0?oo
o`<0oooo000?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
0`3oool300000?l0oooo0`3oool000d0oooo100000040?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool010000000oooo0?ooo`3oool3
000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000
oooo0?ooo`090?ooo`800000o`3oool40?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo0`3oool000l0
oooo00<000000?ooo`3oool02@3oool200000?l0oooo103oool000l0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo0?l0oooo0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo000?0?ooo`030000003oool0oooo00T0oooo0P00003o0?ooo`@0oooo000?0?ooo`800000
2P3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?ooo`8000002P3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000@0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`00403oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?l0oooo0`3oool00100
oooo00<000000?ooo`3oool0203oool200000?l0oooo103oool00100oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`3o0?ooo`<0oooo000@0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool0o`3oool30?ooo`00203oool2000000@0oooo00<000000?ooo`0000000P3oool3
000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`001`3oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o
0?ooo`<0oooo00070?ooo`040000003oool0oooo000000D0oooo0P0000040?ooo`030000003oool0
oooo0080oooo0`00003o0?ooo`<0oooo00000`3oool000000000000200000080oooo00@000000?oo
o`3oool000001P3oool00`000000oooo00000002000000D0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`001`3oool010000000oooo0?ooo`0000060?ooo`030000003oool0000000L0oooo00<00000
0?ooo`3oool0o`3oool30?ooo`001`3oool010000000oooo0?ooo`0000060?ooo`030000003oool0
000000L0oooo00<000000?ooo`3oool0o`3oool30?ooo`00203oool2000000L0oooo00<000000?oo
o`0000000`0000040?ooo`030000003oool0oooo0?l0oooo0`3oool00180oooo00<000000?ooo`3o
ool01P3oool200000?l0oooo103oool00180oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`3o0?ooo`<0oooo000B0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3o
ool30?ooo`004P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo0`3oool0
0180oooo0P0000070?ooo`800000o`3oool40?ooo`004`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo0?l0oooo0`3oool001<0oooo0P0000060?ooo`030000003oool0oooo0?l0oooo
0`3oool001@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`<0oooo000D
0?ooo`8000001@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000E0?ooo`030000003oool0oooo
00<0oooo0P00000k0?ooo`<00000>@3oool4000000<0oooo0P00000g0?ooo`@000000P3oool30000
03H0oooo100000030?ooo`800000103oool001D0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0m0?ooo`030000003oool0oooo03P0oooo00D000000?ooo`3oool0oooo000000020?oo
o`030000003oool0oooo03H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo00<0oooo00@000000?ooo`3oool000000`3oool001D0oooo0P0000040?oo
o`030000003oool0oooo03d0oooo00<000000?ooo`3oool0>03oool01@000000oooo0?ooo`3oool0
00000080oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo03@0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000030?ooo`005P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo03X0oooo0`00000k0?ooo`050000003oool0
oooo0?ooo`0000000P3oool00`000000oooo0?ooo`0f0?ooo`040000003oool0oooo0?ooo`<00000
>03oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00@3oool001H0oooo
0P0000030?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0>`3oool01@000000oooo
0?ooo`3oool000000080oooo00<000000?ooo`3oool0=P3oool01@000000oooo0?ooo`3oool00000
03/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`010?ooo`005`3oool01000
0000oooo0?ooo`3oool2000003/0oooo00<000000?ooo`3oool0>@3oool3000000<0oooo00@00000
0?ooo`3oool00000=P3oool3000000<0oooo00<000000?ooo`3oool0=P3oool010000000oooo0?oo
o`0000020?ooo`040000003oool0oooo000000<0oooo000H0?ooo`040000003oool0oooo000003`0
oooo1000000j0?ooo`030000003oool0oooo0080oooo0P00000i0?ooo`040000003oool0oooo0?oo
o`@00000=P3oool2000000@0oooo0P0000040?ooo`00603oool010000000oooo0?ooo`00003o0?oo
o`D0oooo000I0?ooo`030000003oool000000?l0oooo1@3oool001X0oooo0P00003o0?ooo`D0oooo
00080?ooool000006@00000000h0oooo00<000000?ooo`3oool02P3oool2000000/0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`050?ooo`006`3oool3000003`0oooo00<000000?oo
o`3oool0?03oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool01@3oool001/0oooo00@000000?ooo`0000000000o`3oool20?ooo`006`3oool00`000000
oooo0?ooo`0200000?l0oooo0@3oool001/0oooo0P0000020?ooo`<00000oP3oool001/0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0000003m0?ooo`006`3oool00`000000oooo0?ooo`03
0?ooo`040000003oool0000000000?/0oooo000K0?ooo`030000003oool0oooo00@0oooo0P000000
0`3oool000000000003i0?ooo`006`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00800000m`3oool001/0oooo0P0000080?ooo`040000003oool0oooo0?ooo`<00000m03oool001/0
oooo00<000000?ooo`3oool0203oool2000000@0oooo0`00003a0?ooo`006`3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00@0oooo1000003]0?ooo`006`3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00L0oooo1000003Y0?ooo`006`3oool00`000000oooo0?ooo`0<
0?ooo`8000002`3oool300000>H0oooo000K0?ooo`8000003`3oool00`000000oooo0?ooo`0;0?oo
o`@00000hP3oool001/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo``0
0000eP3oool001/0oooo00<000000?ooo`3oool0403oool2000001X0oooo1P00003@0?ooo`006`3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01d0oooo4`00002m0?ooo`006`3oool0
0`000000oooo0?ooo`0C0?ooo`800000<03ooolI000007P0oooo9@0000070?ooo`006`3oool20000
01H0oooo0P0000170?ooogP00000;03oool001/0oooo00<000000?ooo`3oool05`3oool200000>T0
oooo000K0?ooo`030000003oool0oooo01T0oooo0P00003W0?ooo`00203oool2000000@0oooo00<0
00000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool06`3oool200000>D0oooo0007
0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo0>80oooo00070?ooo`040000003oool0oooo000000/0oooo00<0
00000?ooo`3oool00P3oool3000001h0oooo0P00003R0?ooo`001`3oool010000000oooo0?ooo`00
00080?ooo`<000001@3oool00`000000oooo0?ooo`0P0?ooo`800000h03oool000L0oooo00@00000
0?ooo`3oool00000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0280oooo0P00
003N0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0903oool200000=`0oooo00080?ooo`8000002@3oool4000000@0oooo00<00000
0?ooo`3oool09P3oool200000=X0oooo000K0?ooo`800000:@3oool200000=P0oooo000K0?ooo`03
0000003oool0oooo02X0oooo0P00003F0?ooo`006`3oool00`000000oooo0?ooo`0/0?ooo`800000
e03oool001/0oooo00<000000?ooo`3oool0;P3oool200000=80oooo000K0?ooo`030000003oool0
oooo0300oooo0P00003@0?ooo`006`3oool2000003<0oooo0`00003=0?ooo`006`3oool00`000000
oooo0?ooo`0e0?ooo`<00000bP3oool001/0oooo00<000000?ooo`3oool0>03oool200000<P0oooo
000K0?ooo`030000003oool0oooo03X0oooo0`0000350?ooo`006`3oool00`000000oooo0?ooo`0m
0?ooo`<00000`P3oool001/0oooo0P0000110?ooo`@00000_P3oool001/0oooo00<000000?ooo`3o
ool0A03oool400000;X0oooo000K0?ooo`030000003oool0oooo04P0oooo0`00002g0?ooo`006`3o
ool00`000000oooo0?ooo`1;0?ooo`<00000]03oool001/0oooo0P00001?0?ooo`800000/P3oool0
01/0oooo00<000000?ooo`3oool0D03oool300000:l0oooo000K0?ooo`030000003oool0oooo05<0
oooo0`00002/0?ooo`004`3oool4000000@0oooo00<000000?ooo`3oool0EP3oool400000:P0oooo
000E0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0FP3oool400000:@0oooo000E
0?ooo`030000003oool0oooo00<0oooo0`00001N0?ooo`@00000X03oool001D0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`1R0?ooo`@00000W03oool001D0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`1V0?ooo`@00000V03oool001<0oooo0`0000050?ooo`030000
003oool0oooo06X0oooo0`00002E0?ooo`005@3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo06d0oooo1000002A0?ooo`006`3oool200000780oooo1000002=0?ooo`006`3oool00`00
0000oooo0?ooo`1e0?ooo`<00000RP3oool001/0oooo00<000000?ooo`3oool0N03oool4000008H0
oooo000K0?ooo`030000003oool0oooo07`0oooo100000220?ooo`006`3oool00`000000oooo0?oo
o`200?ooo`<00000O`3oool001/0oooo0P0000240?ooo`@00000N`3oool001/0oooo00<000000?oo
o`3oool0Q`3oool4000007L0oooo000K0?ooo`030000003oool0oooo08/0oooo1000001c0?ooo`00
6`3oool00`000000oooo0?ooo`2?0?ooo`H00000K@3oool001/0oooo00<000000?ooo`3oool0U@3o
ool4000006T0oooo000K0?ooo`800000VP3oool4000006D0oooo000K0?ooo`030000003oool0oooo
09d0oooo1000001Q0?ooo`006`3oool00`000000oooo0?ooo`2Q0?ooo`@00000G@3oool001/0oooo
00<000000?ooo`3oool0Y@3oool6000005L0oooo000K0?ooo`030000003oool0oooo0:/0oooo1000
001C0?ooo`006`3oool200000;00oooo1000001?0?ooo`006`3oool00`000000oooo0?ooo`2c0?oo
o`@00000B`3oool001/0oooo00<000000?ooo`3oool0]`3oool5000004H0oooo00070?ooo`@00000
0`3oool00`000000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`2l0?ooo`H00000
@03oool000T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0<80oooo1@00000k0?ooo`002@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo0080oooo0`0000370?ooo`D00000=P3oool000T0oooo00<000000?ooo`3oool01`3oool30000
00D0oooo00<000000?ooo`3oool0c03oool500000340oooo00090?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3A0?ooo`H00000:`3oool000L0oooo
0`0000090?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0e`3oool6000002D0oooo
00090?ooo`030000003oool0oooo00L0oooo100000040?ooo`030000003oool0oooo0=d0oooo1P00
000O0?ooo`006`3oool200000>@0oooo1@00000J0?ooo`006`3oool00`000000oooo0?ooo`3X0?oo
o`D000005@3oool001/0oooo00<000000?ooo`3oool0k@3oool500000100oooo000K0?ooo`030000
003oool0oooo0?80oooo1P00000:0?ooo`006`3oool00`000000oooo0?ooo`3h0?ooo`<000001`3o
ool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0
oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo
003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.17393, -1.78342, \
0.0790907, 0.0203389}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(Look\ at\ almoz/mMag^2\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-alomz[1, 0.99, 1, a]\)/mMag[1, a], {a, \(-1\), 20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 -1.83112 2.43615 [
[.29592 .59253 -3 -9 ]
[.29592 .59253 3 0 ]
[.52268 .59253 -6 -9 ]
[.52268 .59253 6 0 ]
[.74943 .59253 -6 -9 ]
[.74943 .59253 6 0 ]
[.97619 .59253 -6 -9 ]
[.97619 .59253 6 0 ]
[.05666 .1178 -18 -4.5 ]
[.05666 .1178 0 4.5 ]
[.05666 .23961 -24 -4.5 ]
[.05666 .23961 0 4.5 ]
[.05666 .36142 -18 -4.5 ]
[.05666 .36142 0 4.5 ]
[.05666 .48322 -24 -4.5 ]
[.05666 .48322 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .60503 m
.29592 .61128 L
s
[(5)] .29592 .59253 0 1 Mshowa
.52268 .60503 m
.52268 .61128 L
s
[(10)] .52268 .59253 0 1 Mshowa
.74943 .60503 m
.74943 .61128 L
s
[(15)] .74943 .59253 0 1 Mshowa
.97619 .60503 m
.97619 .61128 L
s
[(20)] .97619 .59253 0 1 Mshowa
.125 Mabswid
.11451 .60503 m
.11451 .60878 L
s
.15986 .60503 m
.15986 .60878 L
s
.20522 .60503 m
.20522 .60878 L
s
.25057 .60503 m
.25057 .60878 L
s
.34127 .60503 m
.34127 .60878 L
s
.38662 .60503 m
.38662 .60878 L
s
.43197 .60503 m
.43197 .60878 L
s
.47732 .60503 m
.47732 .60878 L
s
.56803 .60503 m
.56803 .60878 L
s
.61338 .60503 m
.61338 .60878 L
s
.65873 .60503 m
.65873 .60878 L
s
.70408 .60503 m
.70408 .60878 L
s
.79478 .60503 m
.79478 .60878 L
s
.84014 .60503 m
.84014 .60878 L
s
.88549 .60503 m
.88549 .60878 L
s
.93084 .60503 m
.93084 .60878 L
s
.02381 .60503 m
.02381 .60878 L
s
.25 Mabswid
0 .60503 m
1 .60503 L
s
.06916 .1178 m
.07541 .1178 L
s
[(0.8)] .05666 .1178 1 0 Mshowa
.06916 .23961 m
.07541 .23961 L
s
[(0.85)] .05666 .23961 1 0 Mshowa
.06916 .36142 m
.07541 .36142 L
s
[(0.9)] .05666 .36142 1 0 Mshowa
.06916 .48322 m
.07541 .48322 L
s
[(0.95)] .05666 .48322 1 0 Mshowa
.125 Mabswid
.06916 .02036 m
.07291 .02036 L
s
.06916 .04472 m
.07291 .04472 L
s
.06916 .06908 m
.07291 .06908 L
s
.06916 .09344 m
.07291 .09344 L
s
.06916 .14216 m
.07291 .14216 L
s
.06916 .16653 m
.07291 .16653 L
s
.06916 .19089 m
.07291 .19089 L
s
.06916 .21525 m
.07291 .21525 L
s
.06916 .26397 m
.07291 .26397 L
s
.06916 .28833 m
.07291 .28833 L
s
.06916 .31269 m
.07291 .31269 L
s
.06916 .33706 m
.07291 .33706 L
s
.06916 .38578 m
.07291 .38578 L
s
.06916 .41014 m
.07291 .41014 L
s
.06916 .4345 m
.07291 .4345 L
s
.06916 .45886 m
.07291 .45886 L
s
.06916 .50759 m
.07291 .50759 L
s
.06916 .53195 m
.07291 .53195 L
s
.06916 .55631 m
.07291 .55631 L
s
.06916 .58067 m
.07291 .58067 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .56222 L
.10458 .52771 L
.14415 .49691 L
.18221 .46836 L
.22272 .43895 L
.26171 .41151 L
.30316 .38319 L
.34309 .35669 L
.3815 .33187 L
.42237 .30617 L
.46172 .28207 L
.49955 .25947 L
.53984 .23599 L
.57861 .21395 L
.61984 .19108 L
.65954 .16959 L
.69774 .14939 L
.73838 .12838 L
.77751 .10862 L
.81909 .0881 L
.85916 .06878 L
.89771 .05059 L
.93871 .03165 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool0
0`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?T0oooo0P0000070?oo
o`006`3oool00`000000oooo0?ooo`3g0?ooo`8000002@3oool001/0oooo0P00003f0?ooo`800000
2`3oool001/0oooo00<000000?ooo`3oool0l`3oool2000000d0oooo000K0?ooo`030000003oool0
oooo0?40oooo0P00000?0?ooo`006`3oool00`000000oooo0?ooo`3^0?ooo`<000004@3oool001/0
oooo00<000000?ooo`3oool0k03oool2000001@0oooo000K0?ooo`030000003oool0oooo0>X0oooo
0P00000F0?ooo`006`3oool200000>T0oooo0P00000H0?ooo`006`3oool00`000000oooo0?ooo`3V
0?ooo`8000006P3oool001/0oooo00<000000?ooo`3oool0h`3oool3000001`0oooo000K0?ooo`03
0000003oool0oooo0>40oooo0P00000O0?ooo`006`3oool00`000000oooo0?ooo`3O0?ooo`800000
8@3oool001/0oooo00<000000?ooo`3oool0g@3oool2000002<0oooo000K0?ooo`030000003oool0
oooo0=/0oooo0P00000U0?ooo`006`3oool200000=T0oooo0`00000W0?ooo`006`3oool00`000000
oooo0?ooo`3F0?ooo`800000:P3oool001/0oooo00<000000?ooo`3oool0e03oool2000002`0oooo
000K0?ooo`030000003oool0oooo0=80oooo0P00000^0?ooo`006`3oool00`000000oooo0?ooo`3@
0?ooo`800000<03oool001/0oooo00<000000?ooo`3oool0c@3oool300000380oooo000K0?ooo`03
0000003oool0oooo0</0oooo0P00000e0?ooo`006`3oool200000<X0oooo0P00000g0?ooo`006`3o
ool00`000000oooo0?ooo`370?ooo`800000>@3oool001/0oooo00<000000?ooo`3oool0a@3oool2
000003/0oooo000K0?ooo`030000003oool0oooo0<<0oooo0P00000m0?ooo`006`3oool00`000000
oooo0?ooo`320?ooo`030000003oool0oooo03d0oooo00080?ooo`800000103oool00`000000oooo
0?ooo`030?ooo`8000001@3oool00`000000oooo0?ooo`300?ooo`800000@03oool000L0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0;h0
oooo0P0000120?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00@0oooo0`00002m0?ooo`030000003oool0oooo0480oooo00070?ooo`040000003oool0oooo0000
00T0oooo0P0000050?ooo`030000003oool0oooo0;/0oooo0P0000150?ooo`001`3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0^@3oool2
000004L0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`2g0?ooo`800000B@3oool000P0oooo0P00000:0?ooo`8000001@3oool0
0`000000oooo0?ooo`2e0?ooo`800000B`3oool001/0oooo00<000000?ooo`3oool0/`3oool20000
04d0oooo000K0?ooo`030000003oool0oooo0;40oooo0P00001?0?ooo`006`3oool200000;00oooo
0P00001A0?ooo`006`3oool00`000000oooo0?ooo`2]0?ooo`800000D`3oool001/0oooo00<00000
0?ooo`3oool0Z`3oool2000005D0oooo000K0?ooo`030000003oool0oooo0:T0oooo0P00001G0?oo
o`006`3oool00`000000oooo0?ooo`2W0?ooo`800000F@3oool001/0oooo00<000000?ooo`3oool0
Y@3oool2000005/0oooo000K0?ooo`800000Y03oool2000005d0oooo000K0?ooo`030000003oool0
oooo0:80oooo00<000000?ooo`3oool0G@3oool001/0oooo00<000000?ooo`3oool0X03oool20000
0600oooo000K0?ooo`030000003oool0oooo09h0oooo0P00001R0?ooo`006`3oool00`000000oooo
0?ooo`2L0?ooo`800000I03oool001/0oooo00<000000?ooo`3oool0VP3oool2000006H0oooo000K
0?ooo`030000003oool0oooo09P0oooo0P00001X0?ooo`006`3oool2000009P0oooo00<000000?oo
o`3oool0J03oool001/0oooo00<000000?ooo`3oool0U@3oool2000006/0oooo000K0?ooo`030000
003oool0oooo09<0oooo0P00001]0?ooo`006`3oool00`000000oooo0?ooo`2A0?ooo`800000K`3o
ool001/0oooo00<000000?ooo`3oool0T03oool00`000000oooo0?ooo`1_0?ooo`006`3oool00`00
0000oooo0?ooo`2>0?ooo`800000LP3oool001/0oooo00<000000?ooo`3oool0S03oool2000007@0
oooo000K0?ooo`800000S03oool00`000000oooo0?ooo`1d0?ooo`006`3oool00`000000oooo0?oo
o`290?ooo`800000M`3oool001/0oooo00<000000?ooo`3oool0Q`3oool2000007T0oooo000K0?oo
o`030000003oool0oooo08D0oooo0P00001k0?ooo`006`3oool00`000000oooo0?ooo`230?ooo`80
0000O@3oool00080oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`<00000
1@3oool00`000000oooo0?ooo`210?ooo`800000O`3oool000050?ooo`000000oooo0?ooo`000000
203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0P03oool00`000000oooo0?ooo`1o0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`04
0000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3oool3000007h0oooo0P0000220?oo
o`0000D0oooo0000003oool0oooo000000090?ooo`8000000`3oool3000000D0oooo00<000000?oo
o`3oool0O03oool2000008@0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?oo
o`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo07X0oooo0P000026
0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`1h0?ooo`800000R03oool00080oooo0P00000:
0?ooo`8000000`3oool4000000@0oooo00<000000?ooo`3oool0MP3oool2000008X0oooo000K0?oo
o`030000003oool0oooo07D0oooo00<000000?ooo`3oool0RP3oool001/0oooo0P00001d0?ooo`80
0000S@3oool001/0oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`2=0?ooo`006`3o
ool00`000000oooo0?ooo`1`0?ooo`800000T03oool001/0oooo00<000000?ooo`3oool0KP3oool2
00000980oooo000K0?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0TP3oool001/0
oooo00<000000?ooo`3oool0J`3oool2000009D0oooo000K0?ooo`030000003oool0oooo06X0oooo
00<000000?ooo`3oool0U@3oool001/0oooo0P00001Y0?ooo`800000V03oool001/0oooo00<00000
0?ooo`3oool0IP3oool2000009X0oooo000K0?ooo`030000003oool0oooo06@0oooo0P00002L0?oo
o`006`3oool00`000000oooo0?ooo`1R0?ooo`800000WP3oool001/0oooo00<000000?ooo`3oool0
H03oool200000:00oooo000K0?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3oool0X03o
ool001/0oooo00<000000?ooo`3oool0G@3oool200000:<0oooo000K0?ooo`800000G@3oool00`00
0000oooo0?ooo`2S0?ooo`006`3oool00`000000oooo0?ooo`1J0?ooo`800000YP3oool001/0oooo
00<000000?ooo`3oool0F03oool200000:P0oooo000K0?ooo`030000003oool0oooo05L0oooo00<0
00000?ooo`3oool0Z03oool001/0oooo00<000000?ooo`3oool0E@3oool200000:/0oooo000K0?oo
o`030000003oool0oooo05@0oooo00<000000?ooo`3oool0Z`3oool001/0oooo00<000000?ooo`3o
ool0DP3oool200000:h0oooo000K0?ooo`800000DP3oool00`000000oooo0?ooo`2^0?ooo`006`3o
ool00`000000oooo0?ooo`1?0?ooo`800000/@3oool001/0oooo00<000000?ooo`3oool0C@3oool2
00000;<0oooo000K0?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0/`3oool001/0
oooo00<000000?ooo`3oool0BP3oool200000;H0oooo00080?ooo`800000103oool00`000000oooo
0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo0;H0oooo
00070?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`170?ooo`800000^@3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo
0?ooo`020?ooo`<00000AP3oool00`000000oooo0?ooo`2i0?ooo`001`3oool010000000oooo0?oo
o`0000090?ooo`<00000103oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo0;X0oooo
00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`130?ooo`800000_@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0_@3oool000P0
oooo0P00000:0?ooo`8000001@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0;h0
oooo000K0?ooo`030000003oool0oooo03l0oooo0P0000310?ooo`006`3oool2000003l0oooo00<0
00000?ooo`3oool0`@3oool001/0oooo00<000000?ooo`3oool0?03oool200000<@0oooo000K0?oo
o`030000003oool0oooo03/0oooo00<000000?ooo`3oool0a03oool001/0oooo00<000000?ooo`3o
ool0>P3oool00`000000oooo0?ooo`350?ooo`006`3oool00`000000oooo0?ooo`0h0?ooo`800000
b03oool001/0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`380?ooo`006`3oool0
0`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0<T0oooo000K0?ooo`800000=@3oool20000
0<`0oooo000K0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0c03oool001/0oooo
00<000000?ooo`3oool0<@3oool200000<l0oooo000K0?ooo`030000003oool0oooo0300oooo00<0
00000?ooo`3oool0c`3oool001/0oooo00<000000?ooo`3oool0;P3oool200000=80oooo000K0?oo
o`030000003oool0oooo02`0oooo0P00003D0?ooo`006`3oool00`000000oooo0?ooo`0[0?ooo`03
0000003oool0oooo0=@0oooo000K0?ooo`800000:P3oool200000=L0oooo000K0?ooo`030000003o
ool0oooo02P0oooo00<000000?ooo`3oool0e`3oool001/0oooo00<000000?ooo`3oool09P3oool2
00000=X0oooo000K0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0fP3oool001/0
oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`3K0?ooo`006`3oool00`000000oooo
0?ooo`0R0?ooo`800000gP3oool001/0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`3N0?ooo`006`3oool200000240oooo00<000000?ooo`3oool0g`3oool001/0oooo00<000000?oo
o`3oool07P3oool200000>80oooo000K0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3o
ool0hP3oool001/0oooo00<000000?ooo`3oool06`3oool200000>D0oooo000K0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0i@3oool00080oooo0P0000040?ooo`030000003oool0
oooo0080oooo0`0000030?ooo`<000001@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo0>H0oooo00001@3oool000000?ooo`3oool0000000/0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo01L0oooo0P00003Y0?ooo`0000D0oooo0000
003oool0oooo0000000;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool3
000001H0oooo00<000000?ooo`3oool0j@3oool000050?ooo`000000oooo0?ooo`0000002@3oool3
00000080oooo0`0000050?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0jP3oool0
00050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool04`3oool200000>d0oooo00001@3oool000000?ooo`3o
ool0000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool0k@3oool00080oooo0P00000:0?ooo`8000000`3o
ool4000000@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3^0?ooo`006`3oool0
0`000000oooo0?ooo`0?0?ooo`800000l@3oool001/0oooo0P00000?0?ooo`030000003oool0oooo
0?40oooo000K0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0lP3oool001/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3c0?ooo`006`3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo0?@0oooo000K0?ooo`030000003oool0oooo00T0oooo0P00003g
0?ooo`006`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?L0oooo000K0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0n03oool001/0oooo0P0000070?ooo`030000
003oool0oooo0?T0oooo000K0?ooo`030000003oool0oooo00@0oooo0P00003l0?ooo`006`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0?`0oooo000K0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0o@3oool001/0oooo00D000000?ooo`3oool0oooo0000003o0?oo
o`40oooo000K0?ooo`040000003oool0oooo00000?l0oooo0P3oool001/0oooo0`00003o0?ooo`<0
oooo000K0?ooo`800000o`3oool40?ooo`006P3oool200000?l0oooo1@3oool001T0oooo00<00000
0?ooo`000000o`3oool50?ooo`00603oool010000000oooo0?ooo`00000l0?ooo`<00000>@3oool4
000000<0oooo0P00000g0?ooo`@000000P3oool3000003H0oooo100000030?ooo`800000103oool0
01L0oooo00D000000?ooo`3oool0oooo0000000o0?ooo`030000003oool0oooo03P0oooo00D00000
0?ooo`3oool0oooo000000020?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool00000
0`3oool001H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0m0?ooo`030000003o
ool0oooo03P0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo03H0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo0080oooo
00@000000?ooo`3oool000000`3oool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0j0?ooo`<00000>`3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3o
ool0=P3oool010000000oooo0?ooo`3oool3000003P0oooo00D000000?ooo`3oool0oooo00000002
0?ooo`030000003oool0oooo0040oooo000D0?ooo`030000003oool0oooo00@0oooo0P00000k0?oo
o`030000003oool0oooo03/0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0
oooo03H0oooo00D000000?ooo`3oool0oooo0000000k0?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool00@3oool001<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`0j0?ooo`030000003oool0oooo03T0oooo0`0000030?ooo`040000003oool0oooo000003H0oooo
0`0000030?ooo`030000003oool0oooo03H0oooo00@000000?ooo`3oool000000P3oool010000000
oooo0?ooo`0000030?ooo`004P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo03X0
oooo1000000j0?ooo`030000003oool0oooo0080oooo0P00000i0?ooo`040000003oool0oooo0?oo
o`@00000=P3oool2000000@0oooo0P0000040?ooo`004@3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo0?l0oooo0`3oool00100oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`3o0?ooo`<0oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3o
ool30?ooo`003P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo0`3oool0
00P0ooooo`00000I000000003P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`050?ooo`006`3oool00`000000oooo
0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool00?l0oooo8@3oool0
0?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.17393, 0.748233, \
0.0790907, 0.00147236}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(klim[th_, a_] := 
      k /. FindRoot[
          k^2 + Sqrt[1 + a] 
                alomz[1, IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\)*Cos[th], 
                  IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), a] \[Equal] 
            0, {k, 0.1, 0.5}]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(0.35755245591872487`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(0.3006645987164847`\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/8, 1]\)], "Input"],

Cell[BoxData[
    \(0.5204693420090241`\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Damped mode 1/a", "Subsection"],

Cell[BoxData[
    \(omspeca[k_, th_, a_] := 
      gam /. FindRoot[
          Sqrt[1 + a] Re[alphaspecial[1, k, k*Cos[th], I*gam, a]] + k^2 + 
              gam^2 \[Equal] 0, {gam, 0, klim[th, a]}]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/8, 10]\)], "Input"],

Cell[BoxData[
    \(1.2624463191913442`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphaspecial[1, 1/5, 1/5*Cos[Pi/4], I, 1]\)], "Input"],

Cell[BoxData[
    \(0.2475504204004157530659978442792826265`23.1105 + 
      0``23.7169\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      alphaspecial[1, 1/5, 1/5*Cos[Pi/16], 
        I*IntegerPart[om*10^\((16)\)]*10^\((\(-16\))\), 1], {om, 1/20, 
        5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 -0.486457 4.36106 [
[.21429 .37325 -3 -9 ]
[.21429 .37325 3 0 ]
[.40476 .37325 -3 -9 ]
[.40476 .37325 3 0 ]
[.59524 .37325 -3 -9 ]
[.59524 .37325 3 0 ]
[.78571 .37325 -3 -9 ]
[.78571 .37325 3 0 ]
[.97619 .37325 -3 -9 ]
[.97619 .37325 3 0 ]
[.01131 .03687 -24 -4.5 ]
[.01131 .03687 0 4.5 ]
[.01131 .12409 -24 -4.5 ]
[.01131 .12409 0 4.5 ]
[.01131 .21131 -24 -4.5 ]
[.01131 .21131 0 4.5 ]
[.01131 .29853 -24 -4.5 ]
[.01131 .29853 0 4.5 ]
[.01131 .47298 -24 -4.5 ]
[.01131 .47298 0 4.5 ]
[.01131 .5602 -24 -4.5 ]
[.01131 .5602 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .38575 m
.21429 .392 L
s
[(1)] .21429 .37325 0 1 Mshowa
.40476 .38575 m
.40476 .392 L
s
[(2)] .40476 .37325 0 1 Mshowa
.59524 .38575 m
.59524 .392 L
s
[(3)] .59524 .37325 0 1 Mshowa
.78571 .38575 m
.78571 .392 L
s
[(4)] .78571 .37325 0 1 Mshowa
.97619 .38575 m
.97619 .392 L
s
[(5)] .97619 .37325 0 1 Mshowa
.125 Mabswid
.0619 .38575 m
.0619 .3895 L
s
.1 .38575 m
.1 .3895 L
s
.1381 .38575 m
.1381 .3895 L
s
.17619 .38575 m
.17619 .3895 L
s
.25238 .38575 m
.25238 .3895 L
s
.29048 .38575 m
.29048 .3895 L
s
.32857 .38575 m
.32857 .3895 L
s
.36667 .38575 m
.36667 .3895 L
s
.44286 .38575 m
.44286 .3895 L
s
.48095 .38575 m
.48095 .3895 L
s
.51905 .38575 m
.51905 .3895 L
s
.55714 .38575 m
.55714 .3895 L
s
.63333 .38575 m
.63333 .3895 L
s
.67143 .38575 m
.67143 .3895 L
s
.70952 .38575 m
.70952 .3895 L
s
.74762 .38575 m
.74762 .3895 L
s
.82381 .38575 m
.82381 .3895 L
s
.8619 .38575 m
.8619 .3895 L
s
.9 .38575 m
.9 .3895 L
s
.9381 .38575 m
.9381 .3895 L
s
.25 Mabswid
0 .38575 m
1 .38575 L
s
.02381 .03687 m
.03006 .03687 L
s
[(0.12)] .01131 .03687 1 0 Mshowa
.02381 .12409 m
.03006 .12409 L
s
[(0.14)] .01131 .12409 1 0 Mshowa
.02381 .21131 m
.03006 .21131 L
s
[(0.16)] .01131 .21131 1 0 Mshowa
.02381 .29853 m
.03006 .29853 L
s
[(0.18)] .01131 .29853 1 0 Mshowa
.02381 .47298 m
.03006 .47298 L
s
[(0.22)] .01131 .47298 1 0 Mshowa
.02381 .5602 m
.03006 .5602 L
s
[(0.24)] .01131 .5602 1 0 Mshowa
.125 Mabswid
.02381 .05868 m
.02756 .05868 L
s
.02381 .08048 m
.02756 .08048 L
s
.02381 .10229 m
.02756 .10229 L
s
.02381 .1459 m
.02756 .1459 L
s
.02381 .1677 m
.02756 .1677 L
s
.02381 .18951 m
.02756 .18951 L
s
.02381 .23312 m
.02756 .23312 L
s
.02381 .25492 m
.02756 .25492 L
s
.02381 .27673 m
.02756 .27673 L
s
.02381 .32034 m
.02756 .32034 L
s
.02381 .34214 m
.02756 .34214 L
s
.02381 .36395 m
.02756 .36395 L
s
.02381 .40756 m
.02756 .40756 L
s
.02381 .42937 m
.02756 .42937 L
s
.02381 .45117 m
.02756 .45117 L
s
.02381 .49478 m
.02756 .49478 L
s
.02381 .51659 m
.02756 .51659 L
s
.02381 .53839 m
.02756 .53839 L
s
.02381 .01506 m
.02756 .01506 L
s
.02381 .582 m
.02756 .582 L
s
.02381 .60381 m
.02756 .60381 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.0421 0 m
.04222 .00479 L
.04484 .08658 L
.04732 .15075 L
.05195 .2448 L
.05685 .31738 L
.06137 .36752 L
.06632 .40948 L
.07158 .44332 L
.07652 .46795 L
.08183 .4888 L
.09143 .51629 L
.09729 .52858 L
.10285 .53806 L
.1133 .55167 L
.12279 .56073 L
.1329 .56804 L
.14245 .57338 L
.15111 .57725 L
.17016 .58357 L
.1809 .58619 L
.19077 .58817 L
.20941 .59111 L
.22987 .59348 L
.24918 .59516 L
.26989 .59654 L
.29091 .59763 L
.31083 .59845 L
.34878 .59962 L
.36836 .60008 L
.38915 .60049 L
.42653 .60108 L
.46633 .60155 L
.50463 .60189 L
.54536 .60218 L
.58458 .6024 L
.62231 .60257 L
.66247 .60272 L
.70112 .60284 L
.7422 .60295 L
.78179 .60304 L
.81987 .60311 L
.86038 .60317 L
.89939 .60323 L
.9369 .60328 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`006`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?d0oooo000K0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool001/0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`3m0?ooo`006`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0?d0oooo000K0?ooo`8000000`3oool00`000000oooo0?ooo`3m0?ooo`006`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0?d0oooo000K0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0o@3oool001/0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`3m0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo1000
00040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool000050?ooo`000000
oooo0?ooo`0000002P3oool01@000000oooo0?ooo`3oool0000000L0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`3m0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0103oool2000000<0oooo00<000000?ooo`3oool0
o@3oool000050?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3m0?ooo`0000D0
oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?d0oooo00001@3oool000000?oo
o`3oool0000000P0oooo0`0000030?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`3m0?ooo`000P3oool2000000/0oooo00<000000?ooo`3oool0
0P3oool2000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3m0?ooo`006`3o
ool2000000<0oooo00<000000?ooo`3oool0o@3oool001/0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`3m0?ooo`006`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0?d0oooo000K0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool001/0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3m0?ooo`006`3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0?d0oooo000K0?ooo`8000000`3oool00`000000oooo0?ooo`3m
0?ooo`006`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?d0oooo000K0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool001/0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`3m0?ooo`006`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0?d0oooo000K0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool0
01/0oooo0P0000030?ooo`030000003oool0oooo0?d0oooo000K0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0o@3oool001/0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`3m0?ooo`006`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?d0oooo0002
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool000050?ooo`000000oooo
0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`3l0?ooo`0000D0oooo0000003oool0oooo0000000:
0?ooo`030000003oool0oooo00D00000103oool2000000@0oooo00<000000?ooo`3oool0o03oool0
00050?ooo`000000oooo0?ooo`0000002P3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3l0?ooo`0000D0oooo
0000003oool0oooo0000000:0?ooo`070000003oool0oooo0?ooo`000000oooo000000050?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0o03oool000050?ooo`000000oooo0?ooo`00
0000203oool3000000@0oooo0P0000050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0o03oool00080oooo0P00000;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?`0oooo000K0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0o03oool001/0oooo0P0000040?ooo`030000003oool0
oooo0?`0oooo000K0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o03oool001/0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3l0?ooo`006`3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0?`0oooo000K0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0o03oool001/0oooo0P0000040?ooo`030000003oool0oooo0?`0oooo000K0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o03oool001/0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`3l0?ooo`006`3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo0?`0oooo000K0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o03o
ool001/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3l0?ooo`006`3oool20000
00@0oooo00<000000?ooo`3oool0o03oool001/0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`3l0?ooo`006`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?`0oooo
000K0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0n`3oool00080oooo0P000004
0?ooo`030000003oool0oooo0080oooo100000030?ooo`8000001@3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0?/0oooo00001@3oool000000?ooo`3oool0000000X0oooo00D00000
0?ooo`3oool0oooo000000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`3k0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`050000003o
ool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`020?ooo`8000001@3oool00`000000oooo
0?ooo`3k0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`040000003oool0oooo0?ooo`<0
00001@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?/0oooo00001@3oool00000
0?ooo`3oool0000000X0oooo00D000000?ooo`3oool0oooo000000070?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0n`3oool000050?ooo`000000oooo0?ooo`000000203oool30000
00<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0?/0oooo00020?ooo`8000002`3oool00`000000oooo0?ooo`020?ooo`<00000103oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0?/0oooo000K0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0n`3oool001/0oooo0P0000050?ooo`030000003oool0oooo0?/0oooo000K
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0n`3oool001/0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`3k0?ooo`006`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?/0oooo000K0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
n`3oool001/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3k0?ooo`006`3oool2
000000D0oooo00<000000?ooo`3oool0n`3oool001/0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`3k0?ooo`006`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?X0
oooo000K0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0nP3oool001/0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`3j0?ooo`006`3oool2000000H0oooo00<00000
0?ooo`3oool0nP3oool001/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3j0?oo
o`006`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?X0oooo000K0?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool0nP3oool00080oooo0P0000040?ooo`030000003o
ool0oooo0080oooo100000030?ooo`8000001@3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo0?X0oooo00001@3oool000000?ooo`3oool0000000X0oooo00D000000?ooo`3oool0oooo
000000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`3j0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`050000003oool0oooo0?ooo`00
00000P3oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`000000oooo0?ooo`3j0?ooo`00
00D0oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo0080oooo0P0000050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0n@3oool000050?ooo`000000oooo0?ooo`00
00002P3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo0?T0oooo00001@3oool000000?ooo`3oool00000
00P0oooo0`0000030?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`3i0?ooo`000P3oool2000000/0oooo00<000000?ooo`3oool00P3oool20000
00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3i0?ooo`006`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo0?T0oooo000K0?ooo`8000001`3oool00`000000oooo
0?ooo`3i0?ooo`006`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?T0oooo000K
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0n@3oool001/0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`3i0?ooo`006`3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo0?T0oooo000K0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
n@3oool001/0oooo0P0000070?ooo`030000003oool0oooo0?T0oooo000K0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool08`3oool4000002l0oooo1000000`0?ooo`800000<@3oool0
0`000000oooo0?ooo`0^0?ooo`<000001`3oool001/0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0U0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0<03oool01000
0000oooo0?ooo`00000`0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0103oool0
01/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo
02l0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0[0?ooo`D00000<P3oool00`00
0000oooo0?ooo`040?ooo`006`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo02D0
oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0_0?ooo`800000;P3oool010000000
oooo0?ooo`00000`0?ooo`<000001`3oool001/0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0U0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0<03oool00`000000
oooo0?ooo`0/0?ooo`030000003oool000000300oooo00<000000?ooo`3oool01`3oool001/0oooo
0P0000080?ooo`030000003oool0oooo02<0oooo0`00000`0?ooo`040000003oool0oooo000002l0
oooo00@000000?ooo`3oool00000;`3oool200000300oooo00<000000?ooo`3oool01`3oool001/0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02l0
oooo0P00000a0?ooo`800000<@3oool00`000000oooo0?ooo`0^0?ooo`@000001P3oool001/0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3h0?ooo`006`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo0?P0oooo000K0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool0n03oool001@0ooooo`00000=000000006`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00D0oooo000K0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
903oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0
;`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo00D0oooo000K0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0m`3oool001/0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`3g0?ooo`006`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
0?L0oooo000K0?ooo`8000002@3oool00`000000oooo0?ooo`3g0?ooo`006`3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo0?L0oooo000K0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool0m`3oool001/0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3g
0?ooo`006`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?H0oooo000K0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0mP3oool001/0oooo0P00000:0?ooo`030000
003oool0oooo0?H0oooo000K0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0mP3o
ool001/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3e0?ooo`006`3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo0?D0oooo000K0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0m@3oool001/0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`3e0?ooo`006`3oool2000000/0oooo00<000000?ooo`3oool0m@3oool001/0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`3e0?ooo`006`3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo0?@0oooo000K0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool0m03oool00080oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?ooo`@00000
103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?@0oooo00001@3oool000000?oo
o`3oool0000000P0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool0m03oool000050?ooo`000000oooo0?ooo`000000
2@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00@0oooo0P00000<0?ooo`030000
003oool0oooo0?@0oooo00001@3oool000000?ooo`3oool0000000X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
m03oool000050?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3c0?ooo`0000D0
oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3o
ool00000103oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?<0oooo00020?ooo`80
00002P3oool2000000@0oooo0P0000050?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool0l`3oool001/0oooo0P00000=0?ooo`030000003oool0oooo0?<0oooo000K0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0lP3oool001/0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`3b0?ooo`006`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
0?40oooo000K0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0l@3oool001/0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3`0?ooo`006`3oool200000100oooo00<0
00000?ooo`3oool0l03oool001/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3`
0?ooo`006`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0>l0oooo000K0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0k`3oool001/0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`3^0?ooo`006`3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo0>h0oooo000K0?ooo`8000004`3oool00`000000oooo0?ooo`3]0?ooo`006`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo0>`0oooo000K0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool0k03oool001/0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`3[0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000@0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0>X0oooo0000
1@3oool000000?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0j@3oool000050?ooo`000000
oooo0?ooo`0000002@3oool010000000oooo0?ooo`3oool5000000@0oooo0P00000H0?ooo`030000
003oool0oooo0>P0oooo00001@3oool000000?ooo`3oool0000000X0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01P0oooo0P00003X0?ooo`00
00D0oooo0000003oool0oooo0000000;0?ooo`060000003oool0oooo0000003oool000001@3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>D0oooo00001@3oool000000?ooo`3oool0
000000P0oooo00@000000?ooo`3oool000000`3oool2000000D0oooo00<000000?ooo`3oool06`3o
ool300000>@0oooo00020?ooo`8000002P3oool2000000D0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0N0?ooo`<00000h@3oool001/0oooo00<000000?ooo`3oool08@3oool20000
0=l0oooo000K0?ooo`800000903oool300000=`0oooo000K0?ooo`030000003oool0oooo02H0oooo
1P00003F0?ooo`006`3oool00`000000oooo0?ooo`0/0?ooo`D00000d@3oool001/0oooo00<00000
0?ooo`3oool0<@3oool;00000<H0oooo000K0?ooo`030000003oool0oooo03`0oooo7`00002W0?oo
o`006`3oool2000005`0ooooX00000070?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo
000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo003o0?ooob40oooo003o0?ooob40oooo
003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.539323, 0.106142, \
0.0197364, 0.000862019}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      alphaspecial[1, 1/5, 1/5*Cos[Pi/16], 
        I*IntegerPart[om*10^\((16)\)]*10^\((\(-16\))\), 1], {om, 1/2000, 
        1/20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 19.0476 0.576641 2.44501 [
[.21429 .56414 -12 -9 ]
[.21429 .56414 12 0 ]
[.40476 .56414 -12 -9 ]
[.40476 .56414 12 0 ]
[.59524 .56414 -12 -9 ]
[.59524 .56414 12 0 ]
[.78571 .56414 -12 -9 ]
[.78571 .56414 12 0 ]
[.97619 .56414 -12 -9 ]
[.97619 .56414 12 0 ]
[.01131 .08764 -24 -4.5 ]
[.01131 .08764 0 4.5 ]
[.01131 .20989 -30 -4.5 ]
[.01131 .20989 0 4.5 ]
[.01131 .33214 -24 -4.5 ]
[.01131 .33214 0 4.5 ]
[.01131 .45439 -30 -4.5 ]
[.01131 .45439 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .57664 m
.21429 .58289 L
s
[(0.01)] .21429 .56414 0 1 Mshowa
.40476 .57664 m
.40476 .58289 L
s
[(0.02)] .40476 .56414 0 1 Mshowa
.59524 .57664 m
.59524 .58289 L
s
[(0.03)] .59524 .56414 0 1 Mshowa
.78571 .57664 m
.78571 .58289 L
s
[(0.04)] .78571 .56414 0 1 Mshowa
.97619 .57664 m
.97619 .58289 L
s
[(0.05)] .97619 .56414 0 1 Mshowa
.125 Mabswid
.0619 .57664 m
.0619 .58039 L
s
.1 .57664 m
.1 .58039 L
s
.1381 .57664 m
.1381 .58039 L
s
.17619 .57664 m
.17619 .58039 L
s
.25238 .57664 m
.25238 .58039 L
s
.29048 .57664 m
.29048 .58039 L
s
.32857 .57664 m
.32857 .58039 L
s
.36667 .57664 m
.36667 .58039 L
s
.44286 .57664 m
.44286 .58039 L
s
.48095 .57664 m
.48095 .58039 L
s
.51905 .57664 m
.51905 .58039 L
s
.55714 .57664 m
.55714 .58039 L
s
.63333 .57664 m
.63333 .58039 L
s
.67143 .57664 m
.67143 .58039 L
s
.70952 .57664 m
.70952 .58039 L
s
.74762 .57664 m
.74762 .58039 L
s
.82381 .57664 m
.82381 .58039 L
s
.8619 .57664 m
.8619 .58039 L
s
.9 .57664 m
.9 .58039 L
s
.9381 .57664 m
.9381 .58039 L
s
.25 Mabswid
0 .57664 m
1 .57664 L
s
.02381 .08764 m
.03006 .08764 L
s
[(-0.2)] .01131 .08764 1 0 Mshowa
.02381 .20989 m
.03006 .20989 L
s
[(-0.15)] .01131 .20989 1 0 Mshowa
.02381 .33214 m
.03006 .33214 L
s
[(-0.1)] .01131 .33214 1 0 Mshowa
.02381 .45439 m
.03006 .45439 L
s
[(-0.05)] .01131 .45439 1 0 Mshowa
.125 Mabswid
.02381 .11209 m
.02756 .11209 L
s
.02381 .13654 m
.02756 .13654 L
s
.02381 .16099 m
.02756 .16099 L
s
.02381 .18544 m
.02756 .18544 L
s
.02381 .23434 m
.02756 .23434 L
s
.02381 .25879 m
.02756 .25879 L
s
.02381 .28324 m
.02756 .28324 L
s
.02381 .30769 m
.02756 .30769 L
s
.02381 .35659 m
.02756 .35659 L
s
.02381 .38104 m
.02756 .38104 L
s
.02381 .40549 m
.02756 .40549 L
s
.02381 .42994 m
.02756 .42994 L
s
.02381 .47884 m
.02756 .47884 L
s
.02381 .50329 m
.02756 .50329 L
s
.02381 .52774 m
.02756 .52774 L
s
.02381 .55219 m
.02756 .55219 L
s
.02381 .06319 m
.02756 .06319 L
s
.02381 .03874 m
.02756 .03874 L
s
.02381 .01429 m
.02756 .01429 L
s
.02381 .60109 m
.02756 .60109 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03333 .01472 m
.07158 .05016 L
.1133 .08732 L
.15247 .12087 L
.19015 .15195 L
.23025 .18382 L
.26885 .21335 L
.30989 .24357 L
.34942 .2716 L
.38745 .29759 L
.4279 .32424 L
.46686 .34897 L
.50432 .37192 L
.5442 .3955 L
.58259 .41739 L
.6234 .43985 L
.66271 .4607 L
.70052 .48008 L
.74076 .49999 L
.7795 .5185 L
.82067 .53748 L
.86033 .55513 L
.8985 .57154 L
.93909 .58841 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool0
0`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000
oooo0?ooo`3l0?ooo`008@3oool300000?`0oooo000Q0?ooo`040000003oool0oooo00000?/0oooo
000Q0?ooo`050000003oool0oooo0?ooo`000000nP3oool00240oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`3g0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0?H0oooo000Q0?ooo`030000003oool0oooo00@0oooo0P00003f0?ooo`008@3oool2000000L0
oooo00<000000?ooo`3oool0l`3oool00240oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`3b0?ooo`008@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?40oooo000Q
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0l03oool00240oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`3_0?ooo`008@3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo0>h0oooo000Q0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
k@3oool00240oooo0P00000>0?ooo`030000003oool0oooo0>`0oooo000Q0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0j`3oool00240oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`3Z0?ooo`008@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0>T0
oooo000>0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0>P0oooo000=0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0>L0oooo
000=0?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool0103oool2000001@0oooo
00<000000?ooo`3oool0iP3oool000L0oooo100000020?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0>D0oooo
000=0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0E0?ooo`800000i@3oool000d0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0hP3oool000h0oooo
0P00000:0?ooo`8000001@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0>40oooo
000Q0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0h03oool00240oooo0P00000K
0?ooo`800000h03oool00240oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3M0?oo
o`008@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0=`0oooo000Q0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool0f`3oool00240oooo00<000000?ooo`3oool07`3o
ool200000=/0oooo000Q0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0f03oool0
0240oooo0P00000S0?ooo`030000003oool0oooo0=L0oooo000Q0?ooo`030000003oool0oooo02<0
oooo00<000000?ooo`3oool0eP3oool00240oooo00<000000?ooo`3oool0903oool200000=H0oooo
000Q0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0d`3oool00240oooo00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`3B0?ooo`008@3oool00`000000oooo0?ooo`0X0?oo
o`030000003oool0oooo0=40oooo000Q0?ooo`800000:P3oool200000=40oooo000Q0?ooo`030000
003oool0oooo02/0oooo00<000000?ooo`3oool0cP3oool00240oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`3=0?ooo`008@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0
oooo0<`0oooo000Q0?ooo`030000003oool0oooo02h0oooo0P00003<0?ooo`008@3oool00`000000
oooo0?ooo`0`0?ooo`030000003oool0oooo0<T0oooo000Q0?ooo`030000003oool0oooo0340oooo
00<000000?ooo`3oool0b03oool00240oooo0P00000c0?ooo`030000003oool0oooo0<L0oooo000Q
0?ooo`030000003oool0oooo03<0oooo0P0000370?ooo`008@3oool00`000000oooo0?ooo`0e0?oo
o`030000003oool0oooo0<@0oooo000Q0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3o
ool0``3oool000P0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?ooo`<00000
1@3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0<80oooo00070?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo03P0oooo0P0000320?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool2000003/0oooo00<000000?ooo`3oool0
_`3oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`00000:0?ooo`040000
003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0;h0
oooo00070?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo000000070?oo
o`030000003oool0oooo03`0oooo0P00002n0?ooo`001`3oool010000000oooo0?ooo`0000080?oo
o`<000000`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03h0oooo00<000000?oo
o`3oool0^`3oool000P0oooo0P00000;0?ooo`040000003oool0oooo0?ooo`@00000103oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo0;X0oooo000Q0?ooo`030000003oool0oooo0400
oooo0P00002j0?ooo`008@3oool2000004<0oooo00<000000?ooo`3oool0]`3oool00240oooo00<0
00000?ooo`3oool0@`3oool200000;L0oooo000Q0?ooo`030000003oool0oooo04D0oooo00<00000
0?ooo`3oool0]03oool00240oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`2c0?oo
o`008@3oool00`000000oooo0?ooo`170?ooo`800000/`3oool00240oooo00<000000?ooo`3oool0
B@3oool00`000000oooo0?ooo`2`0?ooo`008@3oool2000004/0oooo00<000000?ooo`3oool0[`3o
ool00240oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`2^0?ooo`008@3oool00`00
0000oooo0?ooo`1<0?ooo`800000[P3oool00240oooo00<000000?ooo`3oool0CP3oool00`000000
oooo0?ooo`2[0?ooo`008@3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo0:X0oooo
000Q0?ooo`030000003oool0oooo0500oooo0P00002Z0?ooo`008@3oool2000005<0oooo0P00002X
0?ooo`008@3oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo0:D0oooo000Q0?ooo`03
0000003oool0oooo05D0oooo0P00002U0?ooo`008@3oool00`000000oooo0?ooo`1G0?ooo`800000
X`3oool00240oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`2P0?ooo`008@3oool0
0`000000oooo0?ooo`1J0?ooo`800000X03oool00240oooo00<000000?ooo`3oool0G03oool00`00
0000oooo0?ooo`2M0?ooo`008@3oool2000005h0oooo00<000000?ooo`3oool0W03oool00240oooo
00<000000?ooo`3oool0GP3oool2000009`0oooo000Q0?ooo`030000003oool0oooo0600oooo00<0
00000?ooo`3oool0V@3oool00240oooo00<000000?ooo`3oool0H@3oool2000009T0oooo000>0?oo
o`800000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`1S0?oo
o`030000003oool0oooo09H0oooo000=0?ooo`040000003oool0oooo000000X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`1T0?ooo`800000UP3oool000d0oooo00@000000?ooo`3o
ool000002P3oool00`000000oooo0?ooo`030?ooo`800000I`3oool2000009@0oooo00070?ooo`@0
00000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0J03oool00`000000oooo0?ooo`2A0?ooo`003@3oool010000000oooo0?ooo`00000:0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0J@3oool200000940oooo000=0?ooo`04
0000003oool0oooo000000P0oooo0`0000050?ooo`030000003oool0oooo06/0oooo0P00002?0?oo
o`003P3oool2000000/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1]0?ooo`03
0000003oool0oooo08`0oooo000Q0?ooo`030000003oool0oooo06h0oooo0P00002<0?ooo`008@3o
ool200000740oooo0P00002:0?ooo`008@3oool00`000000oooo0?ooo`1b0?ooo`030000003oool0
oooo08L0oooo000Q0?ooo`030000003oool0oooo07<0oooo0P0000270?ooo`008@3oool00`000000
oooo0?ooo`1e0?ooo`800000Q@3oool00240oooo00<000000?ooo`3oool0M`3oool00`000000oooo
0?ooo`220?ooo`008@3oool00`000000oooo0?ooo`1h0?ooo`800000PP3oool00240oooo0P00001k
0?ooo`800000P03oool00240oooo00<000000?ooo`3oool0O03oool00`000000oooo0?ooo`1m0?oo
o`008@3oool00`000000oooo0?ooo`1m0?ooo`800000O@3oool00240oooo00<000000?ooo`3oool0
O`3oool2000007/0oooo000Q0?ooo`030000003oool0oooo0840oooo00<000000?ooo`3oool0N03o
ool00240oooo00<000000?ooo`3oool0PP3oool2000007P0oooo000Q0?ooo`030000003oool0oooo
08@0oooo0P00001f0?ooo`008@3oool2000008L0oooo00<000000?ooo`3oool0L`3oool00240oooo
00<000000?ooo`3oool0Q`3oool2000007<0oooo000Q0?ooo`030000003oool0oooo08T0oooo0P00
001a0?ooo`008@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo06h0oooo000Q0?oo
o`030000003oool0oooo08`0oooo0P00001^0?ooo`008@3oool00`000000oooo0?ooo`2>0?ooo`80
0000K03oool00240oooo0P00002A0?ooo`800000JP3oool00240oooo00<000000?ooo`3oool0TP3o
ool2000006P0oooo000Q0?ooo`030000003oool0oooo09@0oooo0P00001V0?ooo`008@3oool00`00
0000oooo0?ooo`2F0?ooo`800000I03oool000P0oooo0P0000040?ooo`030000003oool0oooo00<0
oooo0P0000030?ooo`<000001@3oool00`000000oooo0?ooo`2H0?ooo`800000HP3oool000L0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0VP3oool00`000000oooo0?ooo`1O0?ooo`001`3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3oool2
000009`0oooo0P00001O0?ooo`0000<0oooo0000000000000P0000020?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool000000P3oool3000000D0oooo00<000000?ooo`3oool0W@3o
ool2000005d0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo09l0oooo0P00001K0?ooo`001`3o
ool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`2Q0?ooo`800000F@3oool000P0oooo0P00000:0?ooo`800000
0`3oool4000000@0oooo00<000000?ooo`3oool0X`3oool3000005H0oooo000Q0?ooo`030000003o
ool0oooo0:H0oooo0P00001D0?ooo`008@3oool200000:T0oooo0P00001B0?ooo`008@3oool00`00
0000oooo0?ooo`2Z0?ooo`800000D03oool00240oooo00<000000?ooo`3oool0[03oool2000004h0
oooo000Q0?ooo`030000003oool0oooo0:h0oooo00<000000?ooo`3oool0B`3oool00240oooo00<0
00000?ooo`3oool0[`3oool2000004/0oooo000Q0?ooo`030000003oool0oooo0;40oooo0P000019
0?ooo`008@3oool200000;@0oooo0P0000170?ooo`008@3oool00`000000oooo0?ooo`2e0?ooo`80
0000A@3oool00240oooo00<000000?ooo`3oool0]`3oool300000480oooo000Q0?ooo`030000003o
ool0oooo0;X0oooo0P0000100?ooo`008@3oool00`000000oooo0?ooo`2l0?ooo`800000?P3oool0
0240oooo00<000000?ooo`3oool0_P3oool2000003`0oooo000Q0?ooo`030000003oool0oooo0<00
oooo0P00000j0?ooo`008@3oool200000<<0oooo0P00000h0?ooo`008@3oool00`000000oooo0?oo
o`340?ooo`800000=P3oool00240oooo00<000000?ooo`3oool08`3oool2000000@0oooo00<00000
0?ooo`3oool00`3oool2000000<0oooo1000000K0?ooo`800000103oool00`000000oooo0?ooo`03
0?ooo`8000000`3oool4000001`0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P000004
0?ooo`800000703oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P00000M
0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool3000000<0oooo000Q0?oo
o`030000003oool0oooo0280oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00040?ooo`030000003oool0oooo01T0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo01`0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo000001X0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000040?ooo`<000006P3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo000000D0oooo0@0000010?ooo`40oooo000Q0?ooo`030000003oool0oooo
0280oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003o
ool0oooo01T0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?ooo`03
0000003oool0oooo01/0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool00000203oool01@000000oooo0?oo
o`000000oooo00D0000000<0oooo000000000000603oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo000000D0oooo0@0000010?ooo`40oooo000Q0?ooo`030000003oool0oooo0280
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0
oooo01T0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo01X0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?oo
o`8000006`3oool010000000oooo0?ooo`0000080?ooo`060000003oool0oooo0000003oool00000
0P3oool00`000000oooo0?ooo`020?ooo`8000005P3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo00000080oooo0`0000030?ooo`008@3oool2000002<0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo01T0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo01T0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo
01P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0000000H0oooo0P00000D0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`030?ooo`008@3oool00`000000oooo0?ooo`0R0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool3000001/0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000001/0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo
000001X0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?ooo`800000
203oool300000140oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?oo
o`030000003oool0oooo00<0oooo000Q0?ooo`030000003oool0oooo02<0oooo0P00000:0?ooo`80
00001@3oool00`000000oooo0?ooo`0J0?ooo`8000002P3oool2000000@0oooo0P00000M0?ooo`80
00002P3oool2000000@0oooo0P00000L0?ooo`8000002P3oool2000000D0oooo00<000000?ooo`3o
ool02@3oool200000100oooo0P00000:0?ooo`8000000`3oool400000080oooo000Q0?ooo`030000
003oool0oooo0=D0oooo0`00000T0?ooo`008@3oool00`000000oooo0?ooo`3H0?ooo`8000008P3o
ool00240oooo00<000000?ooo`3oool0fP3oool3000001l0oooo000K0?ooool000001P3oool00240
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`040000003oool0oooo0?oo
o`<000000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00X0oooo000Q0?ooo`03
0000003oool0oooo02d0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0^0?ooo`03
0000003oool0oooo02d0oooo00<000000?ooo`3oool08@3oool2000000/0oooo00<000000?ooo`3o
ool02P3oool00240oooo00<000000?ooo`3oool0i03oool3000001D0oooo000Q0?ooo`030000003o
ool0oooo0>L0oooo0P00000C0?ooo`008@3oool00`000000oooo0?ooo`3Y0?ooo`<00000403oool0
0240oooo0P00003]0?ooo`8000003P3oool00240oooo00<000000?ooo`3oool0kP3oool2000000`0
oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q
0?ooo`030000003oool0oooo0?`0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo
003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo
003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.00685935, -0.25196, \
0.000206762, 0.00161076}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(omspeca[1/10, Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.`\\ \[ImaginaryI]\)\))\)\) + \
\(\(0.07071067811865477`\\ xi\)\))\)\)\\ \(\((\(\[LeftSkeleton] 1 \
\[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.7071067807264626`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.3006645987164847`\\ \[ImaginaryI]\)\))\)\) + \(\
\(\(\(\[LeftSkeleton] 20 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.05698460196119841`\\ \[ImaginaryI]\)\))\)\) + \
\(\(\(\(\[LeftSkeleton] 20 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(0.010880558102556098`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/8, 10]\)], "Input"],

Cell[BoxData[
    \(1.2624463191913442`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omspeca[IntegerPart[T*10^\((16)\)]*10^\((\(-16\))\), Pi/8, 10]}, {T, 
        0.001, 1.262, 0.05}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/\((1 + \(\(10\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3826834321953356`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/\((1 + \(\(10\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3826834321953356`\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(1.2624463191913442`\\ \[ImaginaryI]\)\))\)\) + \(\
\(10\\ \(\(\[LeftSkeleton] 22 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(10\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(0.3826834321953356`\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.001`, \(\(0.9477284677847589`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.051000000000000004`, \(-0.8368112069849504`\
\) + 0.`\ \[ImaginaryI]}, {0.101`, \(-1.909953662010369`\) + 
          0.`\ \[ImaginaryI]}, {0.151`, \(-0.0761696657825397`\) + 
          0.`\ \[ImaginaryI]}, {0.201`, \(-0.09731696165640852`\) + 
          0.`\ \[ImaginaryI]}, {0.251`, \(\(0.11564234495290415`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.301`, \(\(0.1310057283209336`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.35100000000000003`, \
\(\(0.14341094041027883`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.401`, \(\(0.15295008017236425`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.451`, \(\(0.15976127889993696`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.501`, \(\(0.16400210052011602`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.551`, \(\(0.1658331452669902`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.601`, \(\(0.1654101801701735`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.651`, \(\(0.1628775550903669`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.7010000000000001`, \(\(0.15836845914474781`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.751`, \(\(0.15200306880309397`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.801`, \(\(0.14388967756156135`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.8510000000000001`, \(\(0.13412473805204952`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.901`, \(\(0.12279385577327304`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.9510000000000001`, \(\(0.10997279224610063`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.0010000000000001`, \(\(0.09572775269067024`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.0510000000000002`, \(\(0.08011650980925585`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.101`, \(\(0.06318885007861312`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.151`, \(\(0.04498698302197072`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.201`, \(\(0.025546181915369257`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.2510000000000001`, \
\(\(0.0048953309130766905`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.761296 0.230882 0.93144 [
[.17607 .21838 -9 -9 ]
[.17607 .21838 9 0 ]
[.32833 .21838 -9 -9 ]
[.32833 .21838 9 0 ]
[.48059 .21838 -9 -9 ]
[.48059 .21838 9 0 ]
[.63285 .21838 -9 -9 ]
[.63285 .21838 9 0 ]
[.78511 .21838 -3 -9 ]
[.78511 .21838 3 0 ]
[.93736 .21838 -9 -9 ]
[.93736 .21838 9 0 ]
[.01131 .04459 -24 -4.5 ]
[.01131 .04459 0 4.5 ]
[.01131 .13774 -24 -4.5 ]
[.01131 .13774 0 4.5 ]
[.01131 .32403 -18 -4.5 ]
[.01131 .32403 0 4.5 ]
[.01131 .41717 -18 -4.5 ]
[.01131 .41717 0 4.5 ]
[.01131 .51031 -18 -4.5 ]
[.01131 .51031 0 4.5 ]
[.01131 .60346 -18 -4.5 ]
[.01131 .60346 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17607 .23088 m
.17607 .23713 L
s
[(0.2)] .17607 .21838 0 1 Mshowa
.32833 .23088 m
.32833 .23713 L
s
[(0.4)] .32833 .21838 0 1 Mshowa
.48059 .23088 m
.48059 .23713 L
s
[(0.6)] .48059 .21838 0 1 Mshowa
.63285 .23088 m
.63285 .23713 L
s
[(0.8)] .63285 .21838 0 1 Mshowa
.78511 .23088 m
.78511 .23713 L
s
[(1)] .78511 .21838 0 1 Mshowa
.93736 .23088 m
.93736 .23713 L
s
[(1.2)] .93736 .21838 0 1 Mshowa
.125 Mabswid
.06187 .23088 m
.06187 .23463 L
s
.09994 .23088 m
.09994 .23463 L
s
.138 .23088 m
.138 .23463 L
s
.21413 .23088 m
.21413 .23463 L
s
.2522 .23088 m
.2522 .23463 L
s
.29026 .23088 m
.29026 .23463 L
s
.36639 .23088 m
.36639 .23463 L
s
.40446 .23088 m
.40446 .23463 L
s
.44252 .23088 m
.44252 .23463 L
s
.51865 .23088 m
.51865 .23463 L
s
.55672 .23088 m
.55672 .23463 L
s
.59478 .23088 m
.59478 .23463 L
s
.67091 .23088 m
.67091 .23463 L
s
.70898 .23088 m
.70898 .23463 L
s
.74704 .23088 m
.74704 .23463 L
s
.82317 .23088 m
.82317 .23463 L
s
.86123 .23088 m
.86123 .23463 L
s
.8993 .23088 m
.8993 .23463 L
s
.97543 .23088 m
.97543 .23463 L
s
.25 Mabswid
0 .23088 m
1 .23088 L
s
.02381 .04459 m
.03006 .04459 L
s
[(-0.2)] .01131 .04459 1 0 Mshowa
.02381 .13774 m
.03006 .13774 L
s
[(-0.1)] .01131 .13774 1 0 Mshowa
.02381 .32403 m
.03006 .32403 L
s
[(0.1)] .01131 .32403 1 0 Mshowa
.02381 .41717 m
.03006 .41717 L
s
[(0.2)] .01131 .41717 1 0 Mshowa
.02381 .51031 m
.03006 .51031 L
s
[(0.3)] .01131 .51031 1 0 Mshowa
.02381 .60346 m
.03006 .60346 L
s
[(0.4)] .01131 .60346 1 0 Mshowa
.125 Mabswid
.02381 .06322 m
.02756 .06322 L
s
.02381 .08185 m
.02756 .08185 L
s
.02381 .10048 m
.02756 .10048 L
s
.02381 .11911 m
.02756 .11911 L
s
.02381 .15637 m
.02756 .15637 L
s
.02381 .175 m
.02756 .175 L
s
.02381 .19362 m
.02756 .19362 L
s
.02381 .21225 m
.02756 .21225 L
s
.02381 .24951 m
.02756 .24951 L
s
.02381 .26814 m
.02756 .26814 L
s
.02381 .28677 m
.02756 .28677 L
s
.02381 .3054 m
.02756 .3054 L
s
.02381 .34265 m
.02756 .34265 L
s
.02381 .36128 m
.02756 .36128 L
s
.02381 .37991 m
.02756 .37991 L
s
.02381 .39854 m
.02756 .39854 L
s
.02381 .4358 m
.02756 .4358 L
s
.02381 .45443 m
.02756 .45443 L
s
.02381 .47306 m
.02756 .47306 L
s
.02381 .49168 m
.02756 .49168 L
s
.02381 .52894 m
.02756 .52894 L
s
.02381 .54757 m
.02756 .54757 L
s
.02381 .5662 m
.02756 .5662 L
s
.02381 .58483 m
.02756 .58483 L
s
.02381 .02597 m
.02756 .02597 L
s
.02381 .00734 m
.02756 .00734 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.13877 .15993 Mdot
.17683 .14024 Mdot
.21489 .3386 Mdot
.25296 .35291 Mdot
.29102 .36446 Mdot
.32909 .37335 Mdot
.36715 .37969 Mdot
.40522 .38364 Mdot
.44328 .38535 Mdot
.48135 .38495 Mdot
.51941 .38259 Mdot
.55748 .37839 Mdot
.59554 .37246 Mdot
.63361 .36491 Mdot
.67167 .35581 Mdot
.70974 .34526 Mdot
.7478 .33331 Mdot
.78587 .32005 Mdot
.82393 .30551 Mdot
.862 .28974 Mdot
.90006 .27278 Mdot
.93813 .25468 Mdot
.97619 .23544 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool2
003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00ooooo`3oool3
ool001_oo`03003ooooo0?ooo`?oo`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool0
oooo0ooo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0oooo0ooo0007ool01000
ooooo`002Ooo00<00?ooool01?oo0P00oooo1?oo00000ooo000000020002ool01000ooooo`002_oo
00<00?ooool00ooo00<00?ooool0oooo0ooo0007ool01000ooooo`002ooo00<00?ooool00_oo00<0
0?ooool0oooo0ooo0007ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo0?ooo`?oo`00
2?oo0P002_oo0P001Ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03003ooooo0?oo
o`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?oo
o`?oo`006ooo0P00oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`00
6ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<0
0?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<0
0?ooool0oooo0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0
oooo0ooo0008ool20004ool00`00ooooo`02ool40004ool00`00ooooo`3oool3ool000Ooo`04003o
oooo000:ool00`00ooooo`03ool00`00ooooo`3oool3ool000Ooo`04003ooooo000:ool00`00oooo
o`03ool2000Vool2003Kool00003ool00000008000;oo`04003ooooo000:ool00`00ooooo`03ool0
0`00ooooo`0Vool00`00ooooo`3Iool000Ooo`04003ooooo000:ool00`00ooooo`03ool00`00oooo
o`3oool3ool000Ooo`04003ooooo0008ool30005ool00`00ooooo`3oool3ool000Soo`8000_oo`03
003ooooo00?oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00ooooo`0Kool2003U
ool001_oo`03003ooooo01coo`03003ooooo0>?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2000Oool20004
ool00`00ooooo`02ool4000Jool20004ool00`00ooooo`04ool00`00ooooo`0Hool20004ool00`00
ooooo`03ool2000Kool20004ool00`00ooooo`03ool2000Oool4000Oool40003ool00`00ooooo`02
ool4000:ool001_oo`03003ooooo01goo`04003ooooo0008ool00`00ooooo`0Jool01000ooooo`00
2_oo00<00?ooool05ooo00@00?ooool000Soo`04003ooooo000Iool01000ooooo`002?oo00@00?oo
ool0023oo`03003ooooo023oo`03003ooooo00Ooo`03003ooooo00_oo`006ooo00<00?ooool07Ooo
00@00?ooool000Woo`03003ooooo01Woo`04003ooooo0007ool5000Hool01000ooooo`002?oo00@0
0?ooool001Woo`04003ooooo0008ool01000ooooo`008?oo00<00?ooool08?oo00<00?ooool02?oo
00<00?ooool02_oo000Kool00`00ooooo`0Mool01000ooooo`002_oo00<00?ooool06?oo00@00?oo
ool000Ooo`04003ooooo000Iool01000ooooo`002?oo0`006_oo00@00?ooool000Woo`80027oo`03
003ooooo023oo`03003ooooo00Woo`03003ooooo00Woo`006ooo00<00?ooool07Ooo00@00?ooool0
00_oo`03003ooooo01Ooo`04003ooooo0008ool00`00ool0000Iool01000ooooo`002?oo00<00?oo
ool06_oo00@00?ooool000Soo`04003ooooo000Pool00`00ooooo`0Pool00`00ooooo`0:ool00`00
ooooo`08ool001_oo`8001koo`04003ooooo0008ool01000ooooo`006Ooo00@00?ooool000Woo`80
01Woo`04003ooooo0008ool00`00ooooo`0Jool01000ooooo`002?oo00@00?ooool001koo`<0023o
o`<000Woo`04003ooooo000:ool001_oo`03003ooooo01koo`8000[oo`8001_oo`8000_oo`03003o
oooo01Soo`8000[oo`<001[oo`8000[oo`80027oo`03003ooooo023oo`03003ooooo00Soo`8000_o
o`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?o
o`005?ooo`003@00000Kool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00oooo
o`07ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`07ool00`00oooo
o`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00oooo
o`08ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool00`00oooo
o`07ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`07ool00`00oooo
o`07ool00`00ooooo`07ool20006ool001_oo`03003ooooo02Goo`03003ooooo02Koo`03003ooooo
02Goo`03003ooooo02Koo`03003ooooo02Goo`03003ooooo02Koo`03003ooooo00Soo`03003ooooo
00Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`00
6ooo00<00?ooool0kooo0P004Ooo000Kool00`00ooooo`3`ool00`00ooooo`0?ool001_oo`03003o
oooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03003ooooo0>Go
o`8001_oo`006ooo00<00?ooool0i_oo00<00?ooool06Ooo000Kool00`00ooooo`3oool3ool001_o
o`03003ooooo0?ooo`?oo`006ooo0P00g?oo0P009Ooo000Kool00`00ooooo`3Lool00`00ooooo`0S
ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3Aool2
000_ool001_oo`800=?oo`03003ooooo02goo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`3oool3ool000Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo0<Ooo`8003Woo`001ooo
00@00?ooool000[oo`03003ooooo00?oo`03003ooooo0<Soo`03003ooooo03Ooo`001ooo00@00?oo
ool000[oo`03003ooooo00?oo`800?ooo`Coo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03
003ooooo0?ooo`?oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo0;goo`8004?o
o`001ooo00@00?ooool000Soo`<000Goo`03003ooooo02ooo`8008goo`03003ooooo047oo`002?oo
0P002ooo00<00?ooool00ooo00<00?ooool0<?oo00<00?ooool0cooo000Kool2002dool2001=ool0
01_oo`03003ooooo0;Coo`03003ooooo04_oo`006ooo00<00?ooool0>Ooo0P00aooo000Kool00`00
ooooo`0jool00`00ooooo`1/ool2001Gool001_oo`03003ooooo0:[oo`03003ooooo05Goo`006ooo
0P00A?oo0P00FOoo0P00H_oo000Kool00`00ooooo`14ool00`00ooooo`1Hool00`00ooooo`1Pool0
01_oo`03003ooooo04goo`8004Goo`8006coo`006ooo00<00?ooool0C_oo00<00?ooool0A?oo00<0
0?ooool0J_oo000Kool00`00ooooo`1Hool2000`ool2001fool001_oo`8005[oo`03003ooooo00Ko
o`8001;oo`8000Soo`8000Woo`03003ooooo07Coo`006ooo00<00?ooool0Hooo00<00?ooool01_oo
0P002Ooo00<00?ooool01ooo00<00?ooool0O_oo000Kool00`00ooooo`1]ool00`00ooooo`2Bool0
01_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03
003ooooo0?ooo`?oo`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool0oooo0ooo0007
ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0oooo0ooo0007ool01000ooooo`002Ooo
00<00?ooool01?oo0P00oooo1?oo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool0
oooo0ooo0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0oooo0ooo0007ool01000
ooooo`002?oo00@00?ooool000Coo`03003ooooo0?ooo`?oo`002?oo0P002_oo0P001Ooo00<00?oo
ool0oooo0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo
0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003o
ool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo0008ool20004ool00`00
ooooo`03ool20005ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool01000ooooo`001?oo
00<00?ooool0oooo0ooo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00oooo1?oo0007ool0
1000ooooo`002Ooo0P001Ooo00<00?ooool0oooo0ooo0007ool01000ooooo`002ooo00<00?ooool0
0_oo00<00?ooool0oooo0ooo0007ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo0?oo
o`?oo`002?oo0P002_oo0P001Ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03003o
oooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?oo
o`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`00
6ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`00
6ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<0
0?ooool0oooo0ooo0008ool20004ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`3oool3
ool000Ooo`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`3oool3ool000Ooo`04003ooooo
0007ool50004ool2003oool4ool000Ooo`04003ooooo0007ool01000ooooo`001Ooo00<00?ooool0
oooo0ooo0007ool01000ooooo`002?oo00<00?oo00001Ooo00<00?ooool0oooo0ooo0007ool01000
ooooo`002Ooo0P001Ooo00<00?ooool0oooo0ooo0008ool2000;ool00`00ooooo`03ool00`00oooo
o`3oool3ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00001
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.134938, -0.271918, \
0.00493805, 0.00403603}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(omspeca[k_, th_, a_] := 
      gam /. FindRoot[
          Sqrt[1 + a] Re[alphaspecial[1, k, k*Cos[th], I*gam, a]] + k^2 + 
              gam^2 \[Equal] 0, {gam, 0.001, 0.1}]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omspeca[IntegerPart[T*10^\((16)\)]*10^\((\(-16\))\), Pi/8, 10]}, {T, 
        0.001, 0.201, 0.05}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/\((1 + \(\(10\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/\((1 + \(\(10\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\/\((1 + \(\(10\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \({{0.001`, \(\(0.02214886056541065`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.051000000000000004`, \
\(-0.027053773789956343`\) + 
          0.`\ \[ImaginaryI]}, {0.101`, \(\(0.05254813012009045`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.151`, \(\(0.07616966578352233`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.201`, \(\(0.09731696202551249`\)\(\
\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.001`, 
        0}, {0.101`, \(\(0.05254813012009045`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.151`, \(\(0.07616966578352233`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.201`, \(\(0.09731696202551249`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.251`, \(\(0.11564234495290415`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.301`, \(\(0.1310057283209336`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.35100000000000003`, \
\(\(0.14341094041027883`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.401`, \(\(0.15295008017236425`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.451`, \(\(0.15976127889993696`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.501`, \(\(0.16400210052011602`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.551`, \(\(0.1658331452669902`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.601`, \(\(0.1654101801701735`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.651`, \(\(0.1628775550903669`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.7010000000000001`, \(\(0.15836845914474781`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.751`, \(\(0.15200306880309397`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.801`, \(\(0.14388967756156135`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.8510000000000001`, \(\(0.13412473805204952`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.901`, \(\(0.12279385577327304`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.9510000000000001`, \(\(0.10997279224610063`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.0010000000000001`, \(\(0.09572775269067024`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.0510000000000002`, \(\(0.08011650980925585`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.101`, \(\(0.06318885007861312`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.151`, \(\(0.04498698302197072`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.201`, \(\(0.025546181915369257`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.2510000000000001`, \
\(\(0.0048953309130766905`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}}\)], "Input"],

Cell[BoxData[
    \({{0.001`, 
        0}, {0.101`, \(\(0.05254813012009045`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.151`, \(\(0.07616966578352233`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.201`, \(\(0.09731696202551249`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.251`, \(\(0.11564234495290415`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.301`, \(\(0.1310057283209336`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.35100000000000003`, \
\(\(0.14341094041027883`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.401`, \(\(0.15295008017236425`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.451`, \(\(0.15976127889993696`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.501`, \(\(0.16400210052011602`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.551`, \(\(0.1658331452669902`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.601`, \(\(0.1654101801701735`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.651`, \(\(0.1628775550903669`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.7010000000000001`, \(\(0.15836845914474781`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.751`, \(\(0.15200306880309397`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.801`, \(\(0.14388967756156135`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.8510000000000001`, \(\(0.13412473805204952`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.901`, \(\(0.12279385577327304`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0.9510000000000001`, \(\(0.10997279224610063`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.0010000000000001`, \(\(0.09572775269067024`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.0510000000000002`, \(\(0.08011650980925585`\
\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.101`, \(\(0.06318885007861312`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.151`, \(\(0.04498698302197072`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.201`, \(\(0.025546181915369257`\)\(\
\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {1.2510000000000001`, \
\(\(0.0048953309130766905`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True, 
      PlotStyle \[Rule] {Dashing[{0.002, 0.01}]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.761296 0.0147151 3.54937 [
[.17607 .00222 -9 -9 ]
[.17607 .00222 9 0 ]
[.32833 .00222 -9 -9 ]
[.32833 .00222 9 0 ]
[.48059 .00222 -9 -9 ]
[.48059 .00222 9 0 ]
[.63285 .00222 -9 -9 ]
[.63285 .00222 9 0 ]
[.78511 .00222 -3 -9 ]
[.78511 .00222 3 0 ]
[.93736 .00222 -9 -9 ]
[.93736 .00222 9 0 ]
[.01131 .10345 -30 -4.5 ]
[.01131 .10345 0 4.5 ]
[.01131 .19218 -24 -4.5 ]
[.01131 .19218 0 4.5 ]
[.01131 .28092 -30 -4.5 ]
[.01131 .28092 0 4.5 ]
[.01131 .36965 -18 -4.5 ]
[.01131 .36965 0 4.5 ]
[.01131 .45839 -30 -4.5 ]
[.01131 .45839 0 4.5 ]
[.01131 .54712 -24 -4.5 ]
[.01131 .54712 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17607 .01472 m
.17607 .02097 L
s
[(0.2)] .17607 .00222 0 1 Mshowa
.32833 .01472 m
.32833 .02097 L
s
[(0.4)] .32833 .00222 0 1 Mshowa
.48059 .01472 m
.48059 .02097 L
s
[(0.6)] .48059 .00222 0 1 Mshowa
.63285 .01472 m
.63285 .02097 L
s
[(0.8)] .63285 .00222 0 1 Mshowa
.78511 .01472 m
.78511 .02097 L
s
[(1)] .78511 .00222 0 1 Mshowa
.93736 .01472 m
.93736 .02097 L
s
[(1.2)] .93736 .00222 0 1 Mshowa
.125 Mabswid
.06187 .01472 m
.06187 .01847 L
s
.09994 .01472 m
.09994 .01847 L
s
.138 .01472 m
.138 .01847 L
s
.21413 .01472 m
.21413 .01847 L
s
.2522 .01472 m
.2522 .01847 L
s
.29026 .01472 m
.29026 .01847 L
s
.36639 .01472 m
.36639 .01847 L
s
.40446 .01472 m
.40446 .01847 L
s
.44252 .01472 m
.44252 .01847 L
s
.51865 .01472 m
.51865 .01847 L
s
.55672 .01472 m
.55672 .01847 L
s
.59478 .01472 m
.59478 .01847 L
s
.67091 .01472 m
.67091 .01847 L
s
.70898 .01472 m
.70898 .01847 L
s
.74704 .01472 m
.74704 .01847 L
s
.82317 .01472 m
.82317 .01847 L
s
.86123 .01472 m
.86123 .01847 L
s
.8993 .01472 m
.8993 .01847 L
s
.97543 .01472 m
.97543 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .10345 m
.03006 .10345 L
s
[(0.025)] .01131 .10345 1 0 Mshowa
.02381 .19218 m
.03006 .19218 L
s
[(0.05)] .01131 .19218 1 0 Mshowa
.02381 .28092 m
.03006 .28092 L
s
[(0.075)] .01131 .28092 1 0 Mshowa
.02381 .36965 m
.03006 .36965 L
s
[(0.1)] .01131 .36965 1 0 Mshowa
.02381 .45839 m
.03006 .45839 L
s
[(0.125)] .01131 .45839 1 0 Mshowa
.02381 .54712 m
.03006 .54712 L
s
[(0.15)] .01131 .54712 1 0 Mshowa
.125 Mabswid
.02381 .03246 m
.02756 .03246 L
s
.02381 .05021 m
.02756 .05021 L
s
.02381 .06796 m
.02756 .06796 L
s
.02381 .0857 m
.02756 .0857 L
s
.02381 .1212 m
.02756 .1212 L
s
.02381 .13894 m
.02756 .13894 L
s
.02381 .15669 m
.02756 .15669 L
s
.02381 .17444 m
.02756 .17444 L
s
.02381 .20993 m
.02756 .20993 L
s
.02381 .22768 m
.02756 .22768 L
s
.02381 .24542 m
.02756 .24542 L
s
.02381 .26317 m
.02756 .26317 L
s
.02381 .29866 m
.02756 .29866 L
s
.02381 .31641 m
.02756 .31641 L
s
.02381 .33416 m
.02756 .33416 L
s
.02381 .35191 m
.02756 .35191 L
s
.02381 .3874 m
.02756 .3874 L
s
.02381 .40515 m
.02756 .40515 L
s
.02381 .42289 m
.02756 .42289 L
s
.02381 .44064 m
.02756 .44064 L
s
.02381 .47613 m
.02756 .47613 L
s
.02381 .49388 m
.02756 .49388 L
s
.02381 .51163 m
.02756 .51163 L
s
.02381 .52937 m
.02756 .52937 L
s
.02381 .56487 m
.02756 .56487 L
s
.02381 .58261 m
.02756 .58261 L
s
.02381 .60036 m
.02756 .60036 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .002 .01 ] 0 setdash
.02457 .01472 m
.1007 .20123 L
.13877 .28507 L
.17683 .36013 L
.21489 .42517 L
.25296 .4797 L
.29102 .52373 L
.32909 .55759 L
.36715 .58177 L
.40522 .59682 L
.44328 .60332 L
.48135 .60182 L
.51941 .59283 L
.55748 .57682 L
.59554 .55423 L
.63361 .52543 L
.67167 .49077 L
.70974 .45056 L
.7478 .40505 L
.78587 .35449 L
.82393 .29908 L
.862 .239 L
.90006 .17439 L
.93813 .10539 L
.97619 .03209 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool0047oo`8000Coo`03003ooooo00;oo`@001Woo`8000Co
o`03003ooooo00Coo`03003ooooo01Ooo`8000Coo`03003ooooo00?oo`8001[oo`8000Coo`03003o
oooo00?oo`8001ooo`@001goo`@000?oo`03003ooooo00;oo`@000[oo`00@?oo00@00?ooool000So
o`03003ooooo01Woo`04003ooooo000:ool00`00ooooo`0Fool01000ooooo`002?oo00@00?ooool0
01Soo`04003ooooo0008ool01000ooooo`008?oo00<00?ooool07_oo00<00?ooool01ooo00<00?oo
ool02ooo0010ool01000ooooo`002Ooo00<00?ooool06?oo00@00?ooool000Ooo`D001Ooo`04003o
oooo0008ool01000ooooo`006?oo00@00?ooool000Soo`04003ooooo000Pool00`00ooooo`0Nool0
0`00ooooo`08ool00`00ooooo`0:ool0043oo`04003ooooo000:ool00`00ooooo`0Gool01000oooo
o`001ooo00@00?ooool001Soo`04003ooooo0008ool3000Iool01000ooooo`002Ooo0P008Ooo00<0
0?ooool07_oo00<00?ooool02Ooo00<00?ooool02Ooo0010ool01000ooooo`002ooo00<00?ooool0
5_oo00@00?ooool000Soo`03003oo`0001Soo`04003ooooo0008ool00`00ooooo`0Iool01000oooo
o`002?oo00@00?ooool0023oo`03003ooooo01koo`03003ooooo00[oo`03003ooooo00Soo`00@?oo
00@00?ooool000Soo`04003ooooo000Hool01000ooooo`002Ooo0P006?oo00@00?ooool000Soo`03
003ooooo01Woo`04003ooooo0008ool01000ooooo`007_oo0`007_oo0`002Ooo00@00?ooool000[o
o`00@Ooo0P002_oo0P006_oo0P002ooo00<00?ooool05ooo0P002_oo0`006Ooo0P002_oo0P008Ooo
00<00?ooool07_oo00<00?ooool02?oo0P002ooo000Qool00`00ooooo`3lool0027oo`03003ooooo
0?coo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool001_oool000H000008Ooo00<00?oo
ool01ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01Ooo000Qool00`00ooooo`3lool0027oo`800?goo`008Ooo0P00oOoo000Qool00`00ooooo`3l
ool0027oo`03003oo`000??oo`8000Ooo`008Ooo00<00?ooool0o?oo000Qool01000ooooo`00lOoo
00<00?ooool01ooo000Qool20002ool00`00ooooo`3^ool00`00ooooo`07ool0027oo`03003ooooo
0?coo`008Ooo00<00?ooool00_oo00<00?ooool0k?oo00<00?ooool02?oo000Qool00`00ooooo`02
ool00`00ooooo`3[ool00`00ooooo`09ool0027oo`03003ooooo0?coo`008Ooo0P001?oo00<00?oo
ool0jOoo00<00?ooool02_oo000Qool00`00ooooo`03ool00`00ooooo`3fool0027oo`03003ooooo
0?coo`008Ooo00<00?ooool01?oo00<00?ooool0iooo00<00?ooool02ooo000Qool20005ool00`00
ooooo`3eool0027oo`03003ooooo0?coo`008Ooo00<00?ooool01Ooo00<00?ooool0i?oo00<00?oo
ool03Ooo0002ool20004ool00`00ooooo`03ool20003ool40002ool30005ool00`00ooooo`3lool0
0005ool00?ooool00008ool01000ooooo`000_oo00<00?ooool01_oo00<00?ooool00_oo00<00?oo
ool0jooo00<00?ooool03_oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo00Go
o`03003ooooo00;oo`8000Ooo`800>?oo`03003ooooo00koo`0000Goo`00ooooo`0000Soo`04003o
oooo0004ool01000oooooooo0`001Ooo00<00?ooool0o?oo00001Ooo003ooooo00002?oo00@00?oo
ool000Goo`04003ooooo0007ool00`00ooooo`3Yool00`00ooooo`0@ool00005ool00?ooool00008
ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo00Soo`03003ooooo
0?7oo`000_oo0P002_oo0P001?oo0P000ooo10001?oo0P00oOoo000Qool00`00ooooo`3Xool00`00
ooooo`0Aool0027oo`03003ooooo00Woo`03003ooooo0?3oo`008Ooo00<00?ooool0i_oo00<00?oo
ool04ooo000Qool00`00ooooo`3Vool00`00ooooo`0Cool0027oo`8000_oo`03003ooooo0>ooo`00
8Ooo00<00?ooool0iOoo00<00?ooool05?oo000Qool00`00ooooo`0;ool00`00ooooo`3Gool00`00
ooooo`0Dool0027oo`03003ooooo00_oo`03003ooooo0>koo`008Ooo00<00?ooool0hooo00<00?oo
ool05_oo000Qool2000=ool00`00ooooo`3]ool0027oo`03003ooooo00coo`03003ooooo0>goo`00
8Ooo00<00?ooool0h_oo00<00?ooool05ooo000Qool00`00ooooo`0=ool00`00ooooo`3/ool0027o
o`8000ooo`03003ooooo0<ooo`03003ooooo01Woo`008Ooo00<00?ooool0h?oo00<00?ooool06Ooo
000Qool00`00ooooo`0?ool00`00ooooo`3Zool000Soo`8000Coo`03003ooooo00?oo`8000?oo`<0
00Goo`03003ooooo00ooo`03003ooooo0<goo`03003ooooo01[oo`001ooo00@00?ooool000Soo`04
003ooooo0005ool00`00ooooo`02ool00`00ooooo`3Oool00`00ooooo`0Jool000Ooo`04003ooooo
0008ool01000ooooo`001Ooo00<00?ooool00_oo0P004Ooo00<00?ooool0jOoo0007ool01000oooo
o`002?oo00@00?ooool000;oo`<000Goo`03003ooooo0=goo`03003ooooo01coo`001ooo00@00?oo
ool000Soo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`3lool000Ooo`04003ooooo0008
ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool04Ooo00<00?ooool0b?oo00<00?ooool0
7Ooo0008ool2000:ool20003ool40004ool2003Lool00`00ooooo`0Nool0027oo`03003ooooo0?co
o`008Ooo00<00?ooool04_oo00<00?ooool0aOoo00<00?ooool07ooo000Qool00`00ooooo`3Jool0
0`00ooooo`0Oool0027oo`03003ooooo0?coo`008Ooo0P005Ooo00<00?ooool0`Ooo0P008_oo000Q
ool00`00ooooo`3lool0027oo`03003ooooo01Goo`03003ooooo0>Coo`008Ooo00<00?ooool05Ooo
00<00?ooool0_ooo00<00?ooool08_oo000Qool00`00ooooo`3lool0027oo`8001Ooo`03003ooooo
0;coo`03003ooooo02Coo`008Ooo00<00?ooool05_oo00<00?ooool0_?oo00<00?ooool09?oo000Q
ool00`00ooooo`3lool0027oo`03003ooooo01Ooo`03003ooooo0;Woo`8002Ooo`008Ooo0P006Ooo
00<00?ooool0hOoo000Qool00`00ooooo`3Bool00`00ooooo`0Wool0027oo`03003ooooo01Woo`03
003ooooo0;Koo`03003ooooo02Ooo`000_oo0P001?oo00<00?ooool00ooo0P001?oo00<00?ooool0
0_oo0`001Ooo00<00?ooool0o?oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo
00Goo`03003ooooo00;oo`03003ooooo0=3oo`03003ooooo02Woo`0000Goo`00ooooo`0000Soo`04
003ooooo0004ool00`00ooooo`04ool00`00ooooo`02ool2000Kool00`00ooooo`2cool00`00oooo
o`0Yool00005ool00?ooool00008ool01000ooooo`001?oo00@00?ooooooo`<000Goo`03003ooooo
0?coo`0000Goo`00ooooo`0000Soo`04003ooooo0005ool01000ooooo`001ooo00<00?ooool0c_oo
0P00;?oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`05
ool00`00ooooo`0Kool2003Oool000;oo`8000[oo`8000?oo`@000;oo`@000Coo`03003ooooo0<go
o`03003ooooo02coo`008Ooo0P007_oo00<00?ooool0[Ooo00<00?ooool0;?oo000Qool00`00oooo
o`0Mool00`00ooooo`3Lool0027oo`03003ooooo0<_oo`03003ooooo02koo`008Ooo00<00?ooool0
7_oo00<00?ooool0fooo000Qool2000Oool00`00ooooo`3Kool0027oo`03003ooooo0<Woo`03003o
oooo033oo`008Ooo00<00?ooool08?oo00<00?ooool0fOoo000Qool00`00ooooo`38ool00`00oooo
o`0aool0027oo`03003ooooo0<Ooo`03003ooooo03;oo`008Ooo0P008_oo00<00?ooool0f?oo000Q
ool00`00ooooo`36ool00`00ooooo`0cool0027oo`03003ooooo02;oo`03003ooooo0=Ooo`008Ooo
00<00?ooool08ooo00<00?ooool0W_oo00<00?ooool0=Ooo000Qool20035ool00`00ooooo`0eool0
027oo`03003ooooo02Coo`03003ooooo0=Goo`008Ooo00<00?ooool09?oo00<00?ooool0Vooo00<0
0?ooool0=ooo000>ool20004ool00`00ooooo`02ool40004ool00`00ooooo`3lool000goo`04003o
oooo000:ool00`00ooooo`03ool00`00ooooo`0Uool2002Iool00`00ooooo`0iool000goo`04003o
oooo000:ool00`00ooooo`03ool20031ool00`00ooooo`0iool000goo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`3lool000goo`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`0W
ool00`00ooooo`2Dool2000lool000goo`04003ooooo0008ool30005ool00`00ooooo`3lool000ko
o`8000_oo`03003ooooo00?oo`03003ooooo02Soo`03003ooooo09;oo`03003ooooo03coo`008Ooo
0P00:_oo00<00?ooool0T?oo00<00?ooool0?Ooo000Qool00`00ooooo`3lool0027oo`03003ooooo
02[oo`03003ooooo08goo`80043oo`008Ooo00<00?ooool0o?oo000Qool2002jool00`00ooooo`10
ool0027oo`03003ooooo02coo`03003ooooo08[oo`03003ooooo043oo`008Ooo00<00?ooool0o?oo
000Qool00`00ooooo`0]ool00`00ooooo`27ool00`00ooooo`12ool0027oo`03003ooooo02goo`03
003ooooo0<coo`008Ooo0P00]_oo00<00?ooool0A?oo000Qool00`00ooooo`0_ool00`00ooooo`3:
ool0027oo`03003ooooo02ooo`03003ooooo087oo`03003ooooo04Koo`008Ooo00<00?ooool0/ooo
00<00?ooool0A_oo000Qool2000bool00`00ooooo`38ool0027oo`03003ooooo0;7oo`03003ooooo
04Soo`008Ooo00<00?ooool0<_oo00<00?ooool0aooo0002ool20004ool00`00ooooo`02ool40002
ool40002ool30005ool00`00ooooo`0cool00`00ooooo`1iool00`00ooooo`1:ool00005ool00?oo
ool0000:ool01@00oooooooo00002?oo00<00?ooool00_oo00<00?ooool0[ooo00<00?ooool0B_oo
00001Ooo003ooooo00002_oo00<00?ooool00_oo00<00?ooool01Ooo00<00?ooool00_oo0P00=Ooo
00<00?ooool0M_oo00<00?ooool0C?oo00001Ooo003ooooo00002_oo00<00?ooool00ooo00@00?oo
ooooo`<000Goo`03003ooooo0:coo`03003ooooo04goo`0000Goo`00ooooo`0000[oo`03003ooooo
00Coo`04003ooooo0007ool00`00ooooo`0fool00`00ooooo`33ool00005ool00?ooool00008ool3
0003ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0=_oo00<00?ooool0LOoo0P00D?oo
0002ool2000;ool00`00ooooo`02ool20003ool40004ool00`00ooooo`3lool0027oo`8003Woo`03
003ooooo06goo`03003ooooo057oo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`0jool00`00
ooooo`1Yool00`00ooooo`1Cool0027oo`03003ooooo03[oo`03003ooooo06Woo`03003ooooo05?o
o`008Ooo0P00YOoo00<00?ooool0EOoo000Qool00`00ooooo`0lool00`00ooooo`1Uool00`00oooo
o`1Eool0027oo`03003ooooo0?coo`008Ooo00<00?ooool0?_oo00<00?ooool0H?oo0P00FOoo000Q
ool00`00ooooo`3lool0027oo`80047oo`03003ooooo05coo`03003ooooo05[oo`008Ooo00<00?oo
ool0@?oo00<00?ooool0^Ooo000Qool00`00ooooo`2Mool00`00ooooo`1Lool0027oo`03003ooooo
04;oo`03003ooooo0;Ooo`008Ooo00<00?ooool0V_oo0P00H?oo000Qool20015ool2002fool0027o
o`03003ooooo09Soo`03003ooooo067oo`002?oo0P001?oo00<00?ooool00_oo10000_oo0`001Ooo
00<00?ooool0Aooo00<00?ooool0/_oo0007ool01000ooooo`002_oo00<00?ooool01?oo00<00?oo
ool00_oo00<00?ooool0UOoo0P00IOoo0007ool01000ooooo`002_oo00<00?ooool01?oo00<00?oo
ool00_oo0P00B_oo00<00?ooool0Aooo00<00?ooool0I_oo0007ool01000ooooo`002_oo00@00?oo
ooooo`<000Goo`03003ooooo09?oo`03003ooooo06Koo`001ooo00@00?ooool000[oo`05003ooooo
ool00007ool00`00ooooo`1;ool20014ool00`00ooooo`1Xool000Ooo`04003ooooo0008ool30003
ool00`00ooooo`05ool00`00ooooo`2@ool00`00ooooo`1Yool000Soo`8000_oo`04003oooooool4
0004ool00`00ooooo`1>ool00`00ooooo`0mool00`00ooooo`1[ool0027oo`8008coo`03003ooooo
06koo`008Ooo00<00?ooool0D?oo0P00>Ooo00<00?ooool0K_oo000Qool00`00ooooo`1Cool00`00
ooooo`0bool2001bool0027oo`03003ooooo05?oo`03003ooooo033oo`03003ooooo07?oo`008Ooo
0P00Eooo00<00?ooool0;Ooo00<00?ooool0Looo000Qool00`00ooooo`1Hool2000Yool00`00oooo
o`1fool0027oo`03003ooooo05_oo`8002?oo`03003ooooo07Woo`008Ooo00<00?ooool0G_oo00<0
0?ooool07?oo00<00?ooool0O?oo000Qool00`00ooooo`1Oool01000ool000005ooo00<00?ooool0
Oooo000Qool2001Uool200000ooo00000008ool01000ooooo`000_oo00<00?ooool0P_oo000Qool0
0`00ooooo`1Zool20002ool00`00ooooo`2;ool0027oo`03003ooooo0?coo`008Ooo00<00?ooool0
o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`00oooo8Ooo003ooolQool00?ooob7o
o`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.168025, -0.0175379, \
0.00505333, 0.00108388}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["prelim", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[xi^2/\((1 + a\ xi^2)\)^2, {xi, 0, 1}]\)], "Input"],

Cell[BoxData[
    \(\(\(-\@a\) + ArcTan[\@a] + a\ ArcTan[\@a]\)\/\(2\ a\^\(3/2\)\ \((1 + a)\
\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[
      xi*Sqrt[xi^2 - kt^2]/\((1 + a\ xi^2)\)^2, {xi, kt, 1}]\)], "Input"],

Cell[BoxData[
    \(\(\(-\@a\)\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - ArcTanh[\(\@a\ \
\@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)] - a\ ArcTanh[\(\@a\ \@\(1 - kt\^2\
\)\)\/\@\(\(-1\) - a\ kt\^2\)]\)\/\(2\ a\^\(3/2\)\ \((1 + a)\)\ \@\(\(-1\) - \
a\ kt\^2\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input"],

Cell[BoxData[
    \(\(\(-\@a\)\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - \((1 + a)\)\ \
ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)]\)\/\(2\ a\^\(3/2\)\
\ \((1 + a)\)\ \@\(\(-1\) - a\ kt\^2\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Series[ArcTan[x*\((1 + eps)\)], {eps, 0, 2}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(ArcTan[x]\), "+", \(\(x\ eps\)\/\(1 + x\^2\)\), 
        "-", \(\(x\^3\ eps\^2\)\/\((1 + x\^2)\)\^2\), "+", 
        InterpretationBox[\(O[eps]\^3\),
          SeriesData[ eps, 0, {}, 0, 3, 1]]}],
      SeriesData[ eps, 0, {
        ArcTan[ x], 
        Times[ x, 
          Power[ 
            Plus[ 1, 
              Power[ x, 2]], -1]], 
        Times[ -1, 
          Power[ x, 3], 
          Power[ 
            Plus[ 1, 
              Power[ x, 2]], -2]]}, 0, 3, 1]]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(mMag2[mD2_, 
        a_] := \(mD2/4\)/
          Sqrt[a]*\((\((a - 1)\)*ArcTan[Sqrt[a]] + Sqrt[a])\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[mMag2[1, a], {a, \(-1\), 20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.4151 0.11172 [
[.29592 .4026 -3 -9 ]
[.29592 .4026 3 0 ]
[.52268 .4026 -6 -9 ]
[.52268 .4026 6 0 ]
[.74943 .4026 -6 -9 ]
[.74943 .4026 6 0 ]
[.97619 .4026 -6 -9 ]
[.97619 .4026 6 0 ]
[.05666 .07994 -12 -4.5 ]
[.05666 .07994 0 4.5 ]
[.05666 .19166 -12 -4.5 ]
[.05666 .19166 0 4.5 ]
[.05666 .30338 -12 -4.5 ]
[.05666 .30338 0 4.5 ]
[.05666 .52682 -6 -4.5 ]
[.05666 .52682 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .4151 m
.29592 .42135 L
s
[(5)] .29592 .4026 0 1 Mshowa
.52268 .4151 m
.52268 .42135 L
s
[(10)] .52268 .4026 0 1 Mshowa
.74943 .4151 m
.74943 .42135 L
s
[(15)] .74943 .4026 0 1 Mshowa
.97619 .4151 m
.97619 .42135 L
s
[(20)] .97619 .4026 0 1 Mshowa
.125 Mabswid
.11451 .4151 m
.11451 .41885 L
s
.15986 .4151 m
.15986 .41885 L
s
.20522 .4151 m
.20522 .41885 L
s
.25057 .4151 m
.25057 .41885 L
s
.34127 .4151 m
.34127 .41885 L
s
.38662 .4151 m
.38662 .41885 L
s
.43197 .4151 m
.43197 .41885 L
s
.47732 .4151 m
.47732 .41885 L
s
.56803 .4151 m
.56803 .41885 L
s
.61338 .4151 m
.61338 .41885 L
s
.65873 .4151 m
.65873 .41885 L
s
.70408 .4151 m
.70408 .41885 L
s
.79478 .4151 m
.79478 .41885 L
s
.84014 .4151 m
.84014 .41885 L
s
.88549 .4151 m
.88549 .41885 L
s
.93084 .4151 m
.93084 .41885 L
s
.02381 .4151 m
.02381 .41885 L
s
.25 Mabswid
0 .4151 m
1 .4151 L
s
.06916 .07994 m
.07541 .07994 L
s
[(-3)] .05666 .07994 1 0 Mshowa
.06916 .19166 m
.07541 .19166 L
s
[(-2)] .05666 .19166 1 0 Mshowa
.06916 .30338 m
.07541 .30338 L
s
[(-1)] .05666 .30338 1 0 Mshowa
.06916 .52682 m
.07541 .52682 L
s
[(1)] .05666 .52682 1 0 Mshowa
.125 Mabswid
.06916 .10228 m
.07291 .10228 L
s
.06916 .12463 m
.07291 .12463 L
s
.06916 .14697 m
.07291 .14697 L
s
.06916 .16932 m
.07291 .16932 L
s
.06916 .214 m
.07291 .214 L
s
.06916 .23635 m
.07291 .23635 L
s
.06916 .25869 m
.07291 .25869 L
s
.06916 .28104 m
.07291 .28104 L
s
.06916 .32572 m
.07291 .32572 L
s
.06916 .34807 m
.07291 .34807 L
s
.06916 .37041 m
.07291 .37041 L
s
.06916 .39276 m
.07291 .39276 L
s
.06916 .43744 m
.07291 .43744 L
s
.06916 .45979 m
.07291 .45979 L
s
.06916 .48213 m
.07291 .48213 L
s
.06916 .50448 m
.07291 .50448 L
s
.06916 .0576 m
.07291 .0576 L
s
.06916 .03525 m
.07291 .03525 L
s
.06916 .01291 m
.07291 .01291 L
s
.06916 .54916 m
.07291 .54916 L
s
.06916 .57151 m
.07291 .57151 L
s
.06916 .59385 m
.07291 .59385 L
s
.06916 .6162 m
.07291 .6162 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .30265 L
.02605 .32103 L
.02729 .33362 L
.02846 .34202 L
.03053 .3529 L
.03279 .36157 L
.0379 .37546 L
.04262 .38468 L
.05224 .39844 L
.06244 .40922 L
.0825 .42496 L
.10458 .43802 L
.14487 .45623 L
.18364 .47007 L
.22486 .48249 L
.26457 .49298 L
.30277 .50207 L
.34341 .51094 L
.38254 .51884 L
.42412 .52669 L
.46419 .53381 L
.50274 .54031 L
.54374 .54689 L
.58323 .55295 L
.6212 .55856 L
.66163 .5643 L
.70054 .56964 L
.7419 .57513 L
.78174 .58025 L
.82007 .58504 L
.86085 .58999 L
.90012 .59464 L
.93787 .59899 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O5?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo
0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?oo
o`[oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00oooo
o`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo00<0
0?ooool0oooo2_oo0007ool00`00ooooo`0:ool2003oool;ool000Ooo`03003ooooo00[oo`03003o
oooo0?ooo`[oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool0
0`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool0
2_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool2003oool;ool000Ooo`03003ooooo00[o
o`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00oooo
o`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00;oo`8000Koo`03003ooooo0?ooo`[oo`00
1ooo00D00?ooooooo`0000;oo`03003ooooo00?oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool0
1?oo00<00?ooool00ooo0P00oooo2ooo0005ool40003ool20006ool00`00ooooo`3oool:ool000Oo
o`03003ooooo00Coo`03003ooooo00?oo`03003ooooo0?ooo`[oo`001ooo00D00?ooooooo`0000;o
o`03003ooooo00?oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool00_oo0P001_oo00<00?ooool0
oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`03003o
oooo0?ooo`[oo`001ooo00<00?ooool02_oo0P00oooo2ooo0007ool00`00ooooo`0:ool00`00oooo
o`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo00<0
0?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00[o
o`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo0P00oooo2ooo0007ool00`00ooooo`0:ool0
0`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool0
2_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Ooo`03003o
oooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool0
0`00ooooo`0:ool2003oool;ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<0
0?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Oo
o`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo
0007ool00`00ooooo`0:ool2003oool;ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`00
1ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:
ool000Ooo`04003oooooool40005ool00`00ooooo`3oool:ool000Ooo`05003oooooool00008ool0
0`00ooooo`3oool:ool000Ooo`03003ooooo00;oo`03003ooooo00Goo`800?ooo`_oo`001Ooo1000
1?oo00<00?ooool01?oo00<00?ooool0oooo2_oo0007ool00`00ooooo`04ool00`00ooooo`03ool0
0`00ooooo`3oool:ool000Ooo`05003oooooool00002ool00`00ooooo`03ool00`00ooooo`3oool:
ool000Ooo`03003ooooo00;oo`8000Koo`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo00<0
0?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00[o
o`800?ooo`_oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool0
0`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool0
2_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Ooo`03003o
oooo00[oo`800?ooo`_oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00oooo
o`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<0
0?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Oo
o`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo0P00oooo2ooo0007ool0
0`00ooooo`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`00
1ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00ooooo`0:ool00`00ooooo`3oool:
ool000Ooo`03003ooooo00[oo`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo0P00oooo2ooo
0007ool00`00ooooo`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`03003ooooo0?oo
o`[oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool01000oooooooo10001Ooo00<0
0?ooool0oooo2_oo0007ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`3oool:ool000Oo
o`03003ooooo00?oo`03003ooooo00Coo`800?ooo`_oo`001Ooo10001?oo00<00?ooool01?oo00<0
0?ooool0oooo2_oo0007ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`3oool:ool000Oo
o`04003oooooool30006ool00`00ooooo`3oool:ool000Ooo`03003ooooo00?oo`03003ooooo00Co
o`03003ooooo0?ooo`[oo`001ooo00<00?ooool02_oo00<00?ooool0oooo2_oo0007ool00`00oooo
o`0:ool00`00ooooo`3oool:ool000Ooo`03003ooooo00[oo`800?ooo`_oo`002?oo00<00?ooool0
2Ooo00<00?ooool0oooo2_oo0008ool00`00ooooo`09ool00`00ooooo`3oool:ool000Soo`03003o
oooo00Woo`03003ooooo0?ooo`[oo`002?oo00<00?ooool02Ooo00<00?ooool0oooo2_oo0008ool0
0`00ooooo`09ool00`00ooooo`3oool:ool000Soo`03003ooooo00Woo`800?ooo`_oo`002Ooo00<0
0?ooool02?oo00<00?ooool0oooo2_oo0009ool00`00ooooo`08ool00`00ooooo`3oool:ool000Wo
o`03003ooooo00Soo`03003ooooo0?ooo`[oo`002Ooo00<00?ooool02?oo00<00?ooool0oooo2_oo
0009ool00`00ooooo`08ool00`00ooooo`3oool:ool000[oo`03003ooooo00Ooo`03003ooooo0?oo
o`[oo`002_oo00<00?ooool01ooo0P00oooo2ooo000;ool00`00ooooo`06ool00`00ooooo`3oool:
ool000_oo`03003ooooo00Koo`03003ooooo03coo`<003[oo`@000?oo`8003Soo`@000;oo`<003So
o`@000?oo`8000Goo`003?oo00<00?ooool01Ooo00<00?ooool0?ooo00<00?ooool0>Ooo00D00?oo
ooooo`0000;oo`03003ooooo03Ooo`03003ooooo00Coo`03003ooooo03Goo`03003ooooo00?oo`04
003ooooo0004ool000coo`03003ooooo00Goo`03003ooooo03ooo`03003ooooo03Woo`05003ooooo
ool00002ool00`00ooooo`0gool00`00ooooo`04ool00`00ooooo`0fool00`00ooooo`02ool01000
ooooo`001?oo000=ool00`00ooooo`04ool00`00ooooo`0lool3000lool01@00oooooooo00000_oo
00<00?ooool0=ooo00@00?ooooooo`<003[oo`05003oooooool00002ool00`00ooooo`02ool000go
o`03003ooooo00Coo`8003goo`03003ooooo03coo`05003oooooool00002ool00`00ooooo`0gool0
1@00oooooooo0000?Ooo00@00?ooool000;oo`03003ooooo00;oo`003_oo00<00?ooool00ooo00<0
0?ooool0??oo00<00?ooool0>_oo0`000ooo00@00?ooool003Ooo`<000?oo`03003ooooo03Soo`04
003ooooo0002ool01000ooooo`001?oo000?ool00`00ooooo`02ool00`00ooooo`0lool4000kool0
0`00ooooo`02ool2000jool01000oooooooo1000>?oo0P001?oo0P001Ooo000@ool01@00oooooooo
0000oooo3?oo000Aool01000ooooo`00oooo3?oo000Bool00`00ool0003oool<ool00?l00200007o
o`001ooo00<00?ooool02_oo0P002ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo
00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo
00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo
00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo
00<00?ooool02_oo00<00?ooool01_oo000Dool00`00ool0000nool00`00ooooo`0mool00`00oooo
o`0nool00`00ooooo`0nool00`00ooooo`06ool001Coo`04003ooooo003oool9ool001Coo`04003o
ooooool2003oool7ool001Coo`03003ooooo00?oo`800?ooo`Goo`005?oo00<00?ooool01Ooo0P00
oooo0ooo000Dool20008ool2003oool1ool001Coo`03003ooooo00Woo`800?koo`005?oo00<00?oo
ool02ooo0P00o?oo000Dool00`00ooooo`0=ool2003jool001Coo`03003ooooo00ooo`800?Soo`00
5?oo00<00?ooool04Ooo0`00mOoo000Dool2000Eool3003bool001Coo`03003ooooo01Ooo`800?3o
o`005?oo00<00?ooool06Ooo0`00kOoo000Dool00`00ooooo`0Lool3003Zool001Coo`03003ooooo
01ooo`<00>Ooo`005?oo00<00?ooool08_oo0`00i?oo000Dool00`00ooooo`0Uool3003Qool001Co
o`8002Woo`<00=koo`005?oo00<00?ooool0:ooo1000f_oo000Dool00`00ooooo`0_ool4003Fool0
01Coo`03003ooooo03?oo`D00=7oo`005?oo00<00?ooool0>?oo1P00booo000Dool00`00ooooo`0n
ool40037ool001Coo`8004?oo`@00<?oo`005?oo00<00?ooool0A_oo1000_ooo000Dool00`00oooo
o`1:ool4002kool001Coo`03003ooooo04koo`H00;Goo`002ooo10001Ooo00<00?ooool0E?oo1@00
/?oo000=ool00`00ooooo`04ool00`00ooooo`1Iool6002Zool000goo`03003ooooo00Coo`80063o
o`H00:Coo`003Ooo00<00?ooool01?oo00<00?ooool0IOoo1P00W_oo000=ool00`00ooooo`04ool0
0`00ooooo`1[ool6002Hool000_oo`<000Koo`03003ooooo077oo`H009;oo`003Ooo00<00?ooool0
1?oo00<00?ooool0Mooo1@00SOoo000Dool00`00ooooo`1lool60027ool001Coo`03003ooooo08;o
o`H0087oo`005?oo0P00ROoo1P00Nooo000Dool00`00ooooo`2>ool8001cool001Coo`03003ooooo
09Koo`P006_oo`005?oo00<00?ooool0W_oo1P00IOoo000Dool00`00ooooo`2Tool5001Pool001Co
o`03003ooooo0:Woo`H005[oo`005?oo0P00/?oo2000D_oo000Dool00`00ooooo`2gool90019ool0
01Coo`03003ooooo0<3oo`H004?oo`005?oo00<00?ooool0a_oo2@00>_oo000Dool00`00ooooo`3?
ool;000_ool001Coo`03003ooooo0=[oo`P002Ooo`005?oo00<00?ooool0h_oo1P008Ooo000Dool2
003Yool8000Iool001Coo`03003ooooo0?3oo`/000koo`005?oo00<00?ooool0nooo1P002?oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool2003oool;ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.55235, -3.71557, \
0.0770198, 0.0312653}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(alomz[mD2_, kz_, k_, 
        a_] := \(\(\(-mD2\)/2\)/
            Sqrt[a]\)/\((k^2 - kz^2)\)*\((kz^2*ArcTan[Sqrt[a]] - 
            kz*k/Sqrt[1 + a*\((1 - kz^2/k^2)\)]*
              ArcTan[Sqrt[a]*\(kz/k\)/
                    Sqrt[1 + a*\((1 - kz^2/k^2)\)]])\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[alomz[Sqrt[3], Cos[th], 1, 10], {th, 0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.603319 0.295831 [
[.2619 .59082 -9 -9 ]
[.2619 .59082 9 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .01166 -12 -4.5 ]
[.01131 .01166 0 4.5 ]
[.01131 .15957 -24 -4.5 ]
[.01131 .15957 0 4.5 ]
[.01131 .30749 -12 -4.5 ]
[.01131 .30749 0 4.5 ]
[.01131 .4554 -24 -4.5 ]
[.01131 .4554 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(0.2)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(0.4)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.6)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(0.8)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.08333 .60332 m
.08333 .60707 L
s
.14286 .60332 m
.14286 .60707 L
s
.20238 .60332 m
.20238 .60707 L
s
.32143 .60332 m
.32143 .60707 L
s
.38095 .60332 m
.38095 .60707 L
s
.44048 .60332 m
.44048 .60707 L
s
.55952 .60332 m
.55952 .60707 L
s
.61905 .60332 m
.61905 .60707 L
s
.67857 .60332 m
.67857 .60707 L
s
.79762 .60332 m
.79762 .60707 L
s
.85714 .60332 m
.85714 .60707 L
s
.91667 .60332 m
.91667 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .01166 m
.03006 .01166 L
s
[(-2)] .01131 .01166 1 0 Mshowa
.02381 .15957 m
.03006 .15957 L
s
[(-1.5)] .01131 .15957 1 0 Mshowa
.02381 .30749 m
.03006 .30749 L
s
[(-1)] .01131 .30749 1 0 Mshowa
.02381 .4554 m
.03006 .4554 L
s
[(-0.5)] .01131 .4554 1 0 Mshowa
.125 Mabswid
.02381 .04124 m
.02756 .04124 L
s
.02381 .07082 m
.02756 .07082 L
s
.02381 .10041 m
.02756 .10041 L
s
.02381 .12999 m
.02756 .12999 L
s
.02381 .18916 m
.02756 .18916 L
s
.02381 .21874 m
.02756 .21874 L
s
.02381 .24832 m
.02756 .24832 L
s
.02381 .2779 m
.02756 .2779 L
s
.02381 .33707 m
.02756 .33707 L
s
.02381 .36665 m
.02756 .36665 L
s
.02381 .39624 m
.02756 .39624 L
s
.02381 .42582 m
.02756 .42582 L
s
.02381 .48499 m
.02756 .48499 L
s
.02381 .51457 m
.02756 .51457 L
s
.02381 .54415 m
.02756 .54415 L
s
.02381 .57374 m
.02756 .57374 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .01472 m
.0403 .01518 L
.04458 .01575 L
.05429 .01752 L
.06375 .01987 L
.07387 .02305 L
.09565 .03214 L
.11547 .04285 L
.15491 .06998 L
.19284 .10152 L
.2332 .13851 L
.27205 .17544 L
.30942 .21072 L
.3492 .24694 L
.38749 .27982 L
.4282 .31225 L
.46741 .34089 L
.50512 .36604 L
.54526 .39032 L
.5839 .41145 L
.62496 .43167 L
.66453 .44919 L
.7026 .46439 L
.74309 .47898 L
.78208 .49164 L
.81958 .50266 L
.8595 .51328 L
.89792 .52253 L
.93877 .53143 L
.97619 .53883 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo000Kool00`00ooooo`3oool3
ool001?oo`@000Coo`03003ooooo0?ooo`?oo`004ooo00<00?ooool01Ooo00<00?ooool0oooo0ooo
000Dool00`00ooooo`04ool2003oool4ool000goo`@000Coo`03003ooooo00?oo`03003ooooo0080
0?ooo`7oo`005_oo00<00?ooool00_oo00<00?ooool00_oo1000o?oo000Cool01000ooooo`001?oo
00<00?ooool01_oo1@00mooo000Dool20005ool00`00ooooo`0;ool2003eool001_oo`03003ooooo
00goo`800??oo`006ooo00<00?ooool03ooo0P00lOoo000Kool00`00ooooo`0Aool2003_ool001_o
o`8001Coo`800>goo`006ooo00<00?ooool05Ooo00<00?ooool0j_oo000Kool00`00ooooo`0Fool2
003Zool001_oo`03003ooooo01Soo`03003ooooo0>Ooo`006ooo00<00?ooool06Ooo00<00?ooool0
i_oo000Kool00`00ooooo`0Jool2003Vool001_oo`03003ooooo01coo`03003ooooo0>?oo`006ooo
00<00?ooool07Ooo0P00hooo000Kool2000Pool00`00ooooo`3Pool001_oo`03003ooooo023oo`03
003ooooo0=ooo`006ooo00<00?ooool08Ooo0P00gooo000Kool00`00ooooo`0Sool00`00ooooo`3L
ool001_oo`03003ooooo02Coo`03003ooooo0=_oo`006ooo00<00?ooool09Ooo00<00?ooool0f_oo
000Kool00`00ooooo`0Vool2003Jool001_oo`03003ooooo02Soo`03003ooooo0=Ooo`006ooo0P00
:_oo00<00?ooool0e_oo000Kool00`00ooooo`0Zool00`00ooooo`3Eool001_oo`03003ooooo02_o
o`03003ooooo0=Coo`006ooo00<00?ooool0;?oo00<00?ooool0dooo000Kool00`00ooooo`0]ool0
0`00ooooo`3Bool001_oo`03003ooooo02koo`800=;oo`006ooo00<00?ooool0<?oo00<00?ooool0
cooo000Kool00`00ooooo`0aool00`00ooooo`3>ool001_oo`8003?oo`03003ooooo0<goo`006ooo
00<00?ooool0<ooo00<00?ooool0c?oo000Kool00`00ooooo`0dool00`00ooooo`3;ool001_oo`03
003ooooo03Goo`03003ooooo0<[oo`006ooo00<00?ooool0=_oo00<00?ooool0bOoo0007ool40003
ool00`00ooooo`02ool30005ool00`00ooooo`0gool00`00ooooo`38ool000Woo`03003ooooo00[o
o`03003ooooo00;oo`03003ooooo03Soo`03003ooooo0<Ooo`002Ooo00<00?ooool02_oo00<00?oo
ool00_oo0P00>_oo00<00?ooool0a_oo00000ooo000000020004ool00`00ooooo`07ool30005ool0
0`00ooooo`0jool00`00ooooo`35ool000Woo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo
03_oo`03003ooooo0<Coo`001ooo0`002Ooo00<00?ooool01Ooo00<00?ooool0??oo00<00?ooool0
`ooo0009ool00`00ooooo`07ool40004ool00`00ooooo`0mool00`00ooooo`32ool001_oo`03003o
oooo03koo`03003ooooo0<7oo`006ooo00<00?ooool0?ooo00<00?ooool0`?oo000Kool00`00oooo
o`10ool00`00ooooo`2oool001_oo`8004;oo`03003ooooo0;koo`006ooo00<00?ooool0@_oo00<0
0?ooool0_Ooo000Kool00`00ooooo`13ool2002mool001_oo`03003ooooo04Goo`03003ooooo0;[o
o`006ooo00<00?ooool0A_oo00<00?ooool0^Ooo000Kool00`00ooooo`17ool00`00ooooo`2hool0
01_oo`03003ooooo04Soo`03003ooooo0;Ooo`006ooo00<00?ooool0BOoo00<00?ooool0]_oo000K
ool2001;ool00`00ooooo`2eool001_oo`03003ooooo04_oo`03003ooooo0;Coo`006ooo00<00?oo
ool0C?oo00<00?ooool0/ooo000Kool00`00ooooo`1=ool2002cool001_oo`03003ooooo04ooo`03
003ooooo0;3oo`006ooo00<00?ooool0D?oo00<00?ooool0[ooo000Kool00`00ooooo`1Aool00`00
ooooo`2^ool001_oo`03003ooooo05;oo`03003ooooo0:goo`006ooo0P00E?oo00<00?ooool0[?oo
000Kool00`00ooooo`1Dool2002/ool001_oo`03003ooooo05Koo`03003ooooo0:Woo`006ooo00<0
0?ooool0Eooo00<00?ooool0Z?oo000Kool00`00ooooo`1Hool00`00ooooo`2Wool001_oo`03003o
oooo05Woo`03003ooooo0:Koo`006ooo00<00?ooool0F_oo0P00Y_oo000Kool00`00ooooo`1Lool0
0`00ooooo`2Sool001_oo`8005koo`03003ooooo0:;oo`006ooo00<00?ooool0G_oo00<00?ooool0
XOoo000Kool00`00ooooo`1Oool00`00ooooo`2Pool001_oo`03003ooooo063oo`03003ooooo09oo
o`006ooo00<00?ooool0HOoo00<00?ooool0W_oo000Cool40004ool00`00ooooo`1Rool2002Nool0
01Goo`03003ooooo00?oo`03003ooooo06Coo`03003ooooo09_oo`005Ooo00<00?ooool00ooo0P00
I_oo00<00?ooool0V_oo000=ool40004ool00`00ooooo`03ool00`00ooooo`1Vool00`00ooooo`2I
ool001Goo`03003ooooo00?oo`03003ooooo06Ooo`03003ooooo09Soo`004ooo0`001Ooo00<00?oo
ool0J?oo0P00V?oo000Eool00`00ooooo`03ool00`00ooooo`1Zool00`00ooooo`2Eool001_oo`03
003ooooo06_oo`8009Goo`006ooo00<00?ooool0KOoo0P00Tooo000Kool00`00ooooo`1_ool00`00
ooooo`2@ool001_oo`80077oo`80093oo`006ooo00<00?ooool0L_oo00<00?ooool0SOoo000Kool0
0`00ooooo`1cool00`00ooooo`2<ool001_oo`03003ooooo07Coo`8008coo`006ooo00<00?ooool0
M_oo00<00?ooool0ROoo000Kool00`00ooooo`1gool00`00ooooo`28ool001_oo`03003ooooo07So
o`8008Soo`006ooo00<00?ooool0N_oo00<00?ooool0QOoo000Kool2001lool00`00ooooo`24ool0
01_oo`03003ooooo07coo`8008Coo`006ooo00<00?ooool0O_oo0P00P_oo000Kool00`00ooooo`20
ool20020ool001_oo`03003ooooo08;oo`8007koo`006ooo00<00?ooool0Q?oo0P00O?oo000Kool0
0`00ooooo`26ool00`00ooooo`1iool001_oo`03003ooooo08Ooo`8007Woo`006ooo0P00R_oo0P00
Mooo000Kool00`00ooooo`2;ool00`00ooooo`1dool001_oo`03003ooooo08coo`8007Coo`006ooo
00<00?ooool0S_oo0P00L_oo000Kool00`00ooooo`2@ool2001`ool001_oo`03003ooooo09;oo`80
06koo`006ooo00<00?ooool0U?oo0P00K?oo000Kool2002Gool2001Zool001_oo`03003ooooo09So
o`8006Soo`006ooo00<00?ooool0V_oo0P00I_oo000Kool00`00ooooo`2Lool2001Tool001_oo`03
003ooooo09koo`8006;oo`006ooo00<00?ooool0X?oo0P00H?oo0008ool20004ool00`00ooooo`02
ool30005ool00`00ooooo`2Rool2001Nool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00
ooooo`2Tool3001Kool000Ooo`04003ooooo000;ool00`00ooooo`02ool2002Xool2001Iool00003
ool00000008000;oo`04003ooooo0008ool30005ool00`00ooooo`2Yool3001Fool000Ooo`04003o
oooo0008ool00`00ooooo`05ool00`00ooooo`2/ool2001Dool000Ooo`04003ooooo0008ool00`00
ooooo`05ool00`00ooooo`2^ool3001Aool000Soo`8000Woo`@000Coo`03003ooooo0;7oo`@004go
o`006ooo00<00?ooool0]Ooo1000BOoo000Kool00`00ooooo`2iool30016ool001_oo`03003ooooo
0;coo`8004Coo`006ooo0P00_ooo0`00@Ooo000Kool00`00ooooo`31ool2000oool001_oo`03003o
oooo0<?oo`<003coo`006ooo00<00?ooool0a_oo0`00>Ooo000Kool00`00ooooo`39ool4000eool0
01_oo`03003ooooo0<goo`<003;oo`006ooo00<00?ooool0d?oo1000;_oo000Kool00`00ooooo`3D
ool4000Zool001_oo`800=Woo`@002Koo`006ooo00<00?ooool0g?oo1@008Ooo000Kool00`00oooo
o`3Qool5000Lool001_oo`03003ooooo0>Koo`H001Koo`006ooo00<00?ooool0k?oo1@004Ooo000K
ool00`00ooooo`3aool5000<ool001_oo`03003ooooo0?Koo`<000Woo`006ooo00<00?ooool0oooo
0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0=?oo0P001?oo
00<00?ooool00_oo1000<?oo0P001?oo00<00?ooool01?oo00<00?ooool0;ooo0P001?oo00<00?oo
ool00ooo0P00<Ooo0P001?oo00<00?ooool00ooo0P000ooo000Kool00`00ooooo`0cool01000oooo
o`002?oo00<00?ooool0<?oo00@00?ooool000[oo`03003ooooo02koo`04003ooooo0008ool01000
ooooo`00;ooo00@00?ooool000Soo`04003ooooo0002ool001_oo`8003Coo`04003ooooo0009ool0
0`00ooooo`0_ool01000ooooo`001ooo1@00;ooo00@00?ooool000Soo`04003ooooo000_ool01000
ooooo`002?oo00@00?ooool000;oo`006ooo00<00?ooool0<ooo00@00?ooool000[oo`03003ooooo
02koo`04003ooooo0007ool01000ooooo`00<?oo00@00?ooool000Soo`<0033oo`04003ooooo0009
ool20003ool001_oo`03003ooooo03?oo`04003ooooo000;ool00`00ooooo`0]ool01000ooooo`00
2?oo00<00?oo0000<?oo00@00?ooool000Soo`03003ooooo033oo`04003ooooo0008ool01000oooo
o`000_oo000Kool00`00ooooo`0cool01000ooooo`002?oo00@00?ooool002ooo`04003ooooo0009
ool2000`ool01000ooooo`002?oo00<00?ooool0<?oo00@00?ooool000Soo`04003ooooo0002ool0
01_oo`03003ooooo03Coo`8000[oo`80037oo`8000_oo`03003ooooo02ooo`8000[oo`<0033oo`80
00[oo`8000?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0?ooo`?oo`005Oooo`002@000ooo000Kool00`00ooooo`0<ool00`00ooooo`0=ool00`00oooo
o`0=ool00`00ooooo`0<ool00`00ooooo`0=ool00`00ooooo`0=ool00`00ooooo`0<ool00`00oooo
o`0=ool00`00ooooo`0=ool00`00ooooo`0<ool00`00ooooo`0=ool00`00ooooo`0=ool00`00oooo
o`0<ool00`00ooooo`0=ool00`00ooooo`0=ool00`00ooooo`0=ool00`00ooooo`07ool001_oo`03
003ooooo03_oo`03003ooooo03coo`03003ooooo03coo`03003ooooo03coo`03003ooooo00Ooo`00
6ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool00?ooob7oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0870867, -2.14005, \
0.00319096, 0.012841}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-alomz[1, 0.5, 1, a]\), {a, \(-1\), 20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.480033 2.299 [
[.29592 .46753 -3 -9 ]
[.29592 .46753 3 0 ]
[.52268 .46753 -6 -9 ]
[.52268 .46753 6 0 ]
[.74943 .46753 -6 -9 ]
[.74943 .46753 6 0 ]
[.97619 .46753 -6 -9 ]
[.97619 .46753 6 0 ]
[.05666 .02023 -24 -4.5 ]
[.05666 .02023 0 4.5 ]
[.05666 .13518 -30 -4.5 ]
[.05666 .13518 0 4.5 ]
[.05666 .25013 -24 -4.5 ]
[.05666 .25013 0 4.5 ]
[.05666 .36508 -30 -4.5 ]
[.05666 .36508 0 4.5 ]
[.05666 .59498 -24 -4.5 ]
[.05666 .59498 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .48003 m
.29592 .48628 L
s
[(5)] .29592 .46753 0 1 Mshowa
.52268 .48003 m
.52268 .48628 L
s
[(10)] .52268 .46753 0 1 Mshowa
.74943 .48003 m
.74943 .48628 L
s
[(15)] .74943 .46753 0 1 Mshowa
.97619 .48003 m
.97619 .48628 L
s
[(20)] .97619 .46753 0 1 Mshowa
.125 Mabswid
.11451 .48003 m
.11451 .48378 L
s
.15986 .48003 m
.15986 .48378 L
s
.20522 .48003 m
.20522 .48378 L
s
.25057 .48003 m
.25057 .48378 L
s
.34127 .48003 m
.34127 .48378 L
s
.38662 .48003 m
.38662 .48378 L
s
.43197 .48003 m
.43197 .48378 L
s
.47732 .48003 m
.47732 .48378 L
s
.56803 .48003 m
.56803 .48378 L
s
.61338 .48003 m
.61338 .48378 L
s
.65873 .48003 m
.65873 .48378 L
s
.70408 .48003 m
.70408 .48378 L
s
.79478 .48003 m
.79478 .48378 L
s
.84014 .48003 m
.84014 .48378 L
s
.88549 .48003 m
.88549 .48378 L
s
.93084 .48003 m
.93084 .48378 L
s
.02381 .48003 m
.02381 .48378 L
s
.25 Mabswid
0 .48003 m
1 .48003 L
s
.06916 .02023 m
.07541 .02023 L
s
[(-0.2)] .05666 .02023 1 0 Mshowa
.06916 .13518 m
.07541 .13518 L
s
[(-0.15)] .05666 .13518 1 0 Mshowa
.06916 .25013 m
.07541 .25013 L
s
[(-0.1)] .05666 .25013 1 0 Mshowa
.06916 .36508 m
.07541 .36508 L
s
[(-0.05)] .05666 .36508 1 0 Mshowa
.06916 .59498 m
.07541 .59498 L
s
[(0.05)] .05666 .59498 1 0 Mshowa
.125 Mabswid
.06916 .04322 m
.07291 .04322 L
s
.06916 .06621 m
.07291 .06621 L
s
.06916 .0892 m
.07291 .0892 L
s
.06916 .11219 m
.07291 .11219 L
s
.06916 .15817 m
.07291 .15817 L
s
.06916 .18116 m
.07291 .18116 L
s
.06916 .20415 m
.07291 .20415 L
s
.06916 .22714 m
.07291 .22714 L
s
.06916 .27312 m
.07291 .27312 L
s
.06916 .29611 m
.07291 .29611 L
s
.06916 .3191 m
.07291 .3191 L
s
.06916 .34209 m
.07291 .34209 L
s
.06916 .38807 m
.07291 .38807 L
s
.06916 .41106 m
.07291 .41106 L
s
.06916 .43405 m
.07291 .43405 L
s
.06916 .45704 m
.07291 .45704 L
s
.06916 .50302 m
.07291 .50302 L
s
.06916 .52601 m
.07291 .52601 L
s
.06916 .549 m
.07291 .549 L
s
.06916 .57199 m
.07291 .57199 L
s
.06916 .61797 m
.07291 .61797 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03484 0 m
.03506 .01084 L
.03756 .103 L
.04017 .17663 L
.04262 .2312 L
.04761 .31395 L
.05217 .36785 L
.05744 .41385 L
.06244 .44682 L
.0675 .47276 L
.07289 .49458 L
.08259 .52352 L
.08853 .53667 L
.09412 .54681 L
.10458 .56145 L
.11434 .57148 L
.12466 .57944 L
.13346 .58467 L
.14313 .58917 L
.15334 .59285 L
.16264 .59546 L
.17338 .59779 L
.18355 .59947 L
.19321 .60068 L
.20346 .60165 L
.21222 .60226 L
.22188 .60275 L
.2268 .60293 L
.23198 .60308 L
.2364 .60317 L
.23888 .60322 L
.24123 .60325 L
.24397 .60328 L
.24646 .6033 L
.24785 .60331 L
.24913 .60331 L
.2498 .60331 L
.25054 .60332 L
.25131 .60332 L
.25204 .60332 L
.25333 .60332 L
.25457 .60332 L
.25567 .60332 L
.25688 .60331 L
.25819 .60331 L
.25958 .6033 L
.26209 .60328 L
.2647 .60325 L
.26746 .60322 L
.2724 .60315 L
Mistroke
.28354 .60293 L
.29317 .60268 L
.30372 .60236 L
.3442 .60079 L
.38316 .59897 L
.42457 .5969 L
.46447 .59487 L
.50285 .59293 L
.54369 .59092 L
.583 .58904 L
.62081 .5873 L
.66106 .58552 L
.6998 .58388 L
.74099 .5822 L
.78067 .58066 L
.81883 .57923 L
.85944 .57778 L
.89854 .57644 L
.94009 .57507 L
.97619 .57393 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Hool00`00ooooo`06ool00`00ooooo`3lool001Soo`03003ooooo
00Koo`03003ooooo0?coo`006?oo00<00?ooool01_oo00<00?ooool0o?oo000>ool20004ool01000
oooooooo1@001?oo00<00?ooool0o?oo000=ool01000ooooo`001ooo0P001ooo00<00?ooool0o?oo
000=ool01000ooooo`001ooo00<00?oo00001_oo0`00o?oo0007ool40002ool01000ooooo`001ooo
00@00?ooool000Goo`03003ooooo0?coo`003Ooo00@00?ooool000Ooo`05003oooooool00004ool0
0`00ooooo`3lool000goo`04003ooooo0007ool20002ool00`00ooooo`02ool00`00ooooo`3lool0
00koo`8000Soo`04003oo`000005ool00`00ooooo`3lool001Soo`03003ooooo00Koo`03003ooooo
0?coo`006?oo00<00?ooool01_oo00<00?ooool0o?oo000Hool00`00ooooo`06ool2003mool001So
o`03003ooooo00Koo`03003ooooo0?coo`006?oo00<00?ooool01_oo00<00?ooool0o?oo000Hool0
0`00ooooo`06ool00`00ooooo`3lool001Soo`03003ooooo00Koo`03003ooooo0?coo`006?oo00<0
0?ooool01_oo00<00?ooool0o?oo000Hool00`00ooooo`06ool2003mool001Soo`03003ooooo00Ko
o`03003ooooo0?coo`006?oo00<00?ooool01_oo00<00?ooool0o?oo000Hool00`00ooooo`06ool0
0`00ooooo`3lool001Soo`03003ooooo00Koo`03003ooooo0?coo`006?oo00<00?ooool01_oo00<0
0?ooool0o?oo000Hool00`00ooooo`06ool2003mool001Soo`03003ooooo00Koo`03003ooooo0?co
o`006?oo00<00?ooool01_oo00<00?ooool0o?oo000Hool00`00ooooo`06ool00`00ooooo`3lool0
01Soo`03003ooooo00Koo`03003ooooo0?coo`006?oo00<00?ooool01_oo00<00?ooool0o?oo000H
ool00`00ooooo`06ool2003mool001Soo`03003ooooo00Koo`03003ooooo0?coo`006?oo00<00?oo
ool01_oo00<00?ooool0o?oo000Hool00`00ooooo`06ool00`00ooooo`3lool001Soo`03003ooooo
00Koo`03003ooooo0?coo`002?oo0P001?oo00<00?ooool00_oo100000?oo`0000000P001Ooo00<0
0?ooool0o?oo0007ool01000ooooo`002_oo00@00?ooool000?oo`03003ooooo00;oo`03003ooooo
0?coo`001ooo00@00?ooool000[oo`04003ooooo0003ool00`00ooooo`02ool3003lool00003ool0
0000008000;oo`04003ooooo000:ool00`00ooooo`040005ool00`00ooooo`3lool000Ooo`04003o
oooo000:ool01@00oooooooo00001ooo00<00?ooool0o?oo0007ool01000ooooo`002?oo0`000ooo
00<00?ooool01Ooo00<00?ooool0o?oo0008ool2000;ool01000oooooooo10001?oo00<00?ooool0
o?oo000Iool00`00ooooo`05ool00`00ooooo`3lool001Woo`03003ooooo00Goo`800?goo`006Ooo
00<00?ooool01Ooo00<00?ooool0o?oo000Iool00`00ooooo`05ool00`00ooooo`3lool001Woo`03
003ooooo00Goo`03003ooooo0?coo`006Ooo00<00?ooool01Ooo00<00?ooool0o?oo000Iool00`00
ooooo`05ool00`00ooooo`3lool001Woo`03003ooooo00Goo`800?goo`006Ooo00<00?ooool01Ooo
00<00?ooool0o?oo000Iool00`00ooooo`05ool00`00ooooo`3lool001Woo`03003ooooo00Goo`03
003ooooo0?coo`006Ooo00<00?ooool01Ooo00<00?ooool0o?oo000Iool00`00ooooo`05ool00`00
ooooo`3lool001Woo`03003ooooo00Goo`800?goo`006_oo00<00?ooool01?oo00<00?ooool0o?oo
000Jool00`00ooooo`04ool00`00ooooo`3lool001[oo`03003ooooo00Coo`03003ooooo0?coo`00
6_oo00<00?ooool01?oo00<00?ooool0o?oo000Jool00`00ooooo`04ool00`00ooooo`3lool001[o
o`03003ooooo00Coo`03003ooooo0?coo`006_oo00<00?ooool01?oo0P00oOoo000Jool00`00oooo
o`04ool00`00ooooo`3lool001[oo`03003ooooo00Coo`03003ooooo0?coo`006_oo00<00?ooool0
1?oo00<00?ooool0o?oo000>ool20004ool00`00ooooo`02ool40004ool00`00ooooo`3lool000go
o`04003ooooo0009ool20005ool00`00ooooo`3lool000goo`04003ooooo0009ool20005ool3003l
ool000Ooo`@000;oo`04003ooooo0009ool20005ool00`00ooooo`3lool000goo`04003ooooo0009
ool20005ool00`00ooooo`3lool000goo`04003ooooo0008ool30005ool00`00ooooo`3lool000ko
o`8000[oo`8000Goo`03003ooooo0?coo`006_oo00<00?ooool01?oo00<00?ooool0o?oo000Jool0
0`00ooooo`04ool2003mool001_oo`03003ooooo00?oo`03003ooooo0?coo`006ooo00<00?ooool0
0ooo00<00?ooool0o?oo000Kool00`00ooooo`03ool00`00ooooo`3lool001_oo`03003ooooo00?o
o`03003ooooo0?coo`006ooo00<00?ooool00ooo00<00?ooool0o?oo000Kool00`00ooooo`03ool0
0`00ooooo`3lool001_oo`03003ooooo00?oo`800?goo`006ooo00<00?ooool00ooo00<00?ooool0
o?oo000Kool00`00ooooo`03ool00`00ooooo`3lool001_oo`03003ooooo00?oo`03003ooooo0?co
o`006ooo00<00?ooool00ooo00<00?ooool0o?oo000Kool00`00ooooo`03ool00`00ooooo`3lool0
01_oo`03003ooooo00?oo`800?goo`006ooo00<00?ooool00ooo00<00?ooool0o?oo000Kool00`00
ooooo`03ool00`00ooooo`3lool001_oo`03003ooooo00?oo`03003ooooo0?coo`006ooo00<00?oo
ool00ooo00<00?ooool0o?oo000Kool00`00ooooo`03ool00`00ooooo`3lool001coo`03003ooooo
00;oo`800?goo`007?oo00<00?ooool00_oo00<00?ooool0o?oo000Lool00`00ooooo`02ool00`00
ooooo`3lool001coo`03003ooooo00;oo`03003ooooo0?coo`002?oo0P001?oo00<00?ooool00ooo
0P000ooo10001?oo00<00?ooool0o?oo0007ool01000ooooo`002?oo00@00?ooool000Goo`03003o
oooo00;oo`03003ooooo0?coo`001ooo00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`02
ool3003lool00003ool00000008000;oo`04003ooooo0008ool01000ooooo`000_oo10001?oo00<0
0?ooool0o?oo0007ool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0004ool00`00oooo
o`3lool000Ooo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo0?co
o`002?oo0P002_oo0P000ooo10001?oo00<00?ooool0o?oo000Mool01@00oooooooo0000o_oo000M
ool01@00oooooooo0000o_oo000Mool01000oooooooo0P00oOoo000Mool01@00oooooooo0000o_oo
000Mool01@00oooooooo0000o_oo000Mool01@00oooooooo0000o_oo000Nool01000ooooo`00o_oo
000Nool01000ooooo`00o_oo000Nool00`00ooooo`02003mool001koo`04003ooooo003nool001ko
o`04003ooooo003nool001koo`04003ooooo003nool001koo`04003ooooo003nool001koo`04003o
oooo003nool001ooo`04003oo`00003mool001ooo`03003oo`000?koo`007ooo00<00?oo0000>ooo
0`00=ooo10000ooo0P00=Ooo10000_oo0`00=Ooo10000ooo0P001?oo000Oool00`00ool0000nool0
0`00ooooo`0fool01@00oooooooo00000_oo00<00?ooool0=?oo00<00?ooool01?oo00<00?ooool0
<_oo00<00?ooool00ooo00@00?ooool000?oo`007ooo00<00?oo0000?_oo00<00?ooool0=_oo00D0
0?ooooooo`0000;oo`03003ooooo03Coo`03003ooooo00Coo`03003ooooo03?oo`03003ooooo00;o
o`04003ooooo0003ool001ooo`03003oo`0003_oo`<003Woo`05003oooooool00002ool00`00oooo
o`0dool01000oooooooo0`00=ooo00D00?ooooooo`0000;oo`03003ooooo007oo`007ooo00@00?oo
000003[oo`03003ooooo03Woo`05003oooooool00002ool00`00ooooo`0dool01@00oooooooo0000
>_oo00@00?ooool000;oo`03003ooooo007oo`007ooo00<00?oo0000>ooo00<00?ooool0=ooo0`00
0ooo00@00?ooool003Coo`<000?oo`03003ooooo03Goo`04003ooooo0002ool01000ooooo`000ooo
000Pool2000kool4000hool00`00ooooo`02ool2000gool01000oooooooo1000=Ooo0P001?oo0P00
1?oo000Pool2003nool0023oo`800?koo`008?oo0P00o_oo000Pool2003nool000koool001<00000
5Ooo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool0
2Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool0
2Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool0
2Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool0
2_oo00<00?ooool02Ooo00<00?ooool01Ooo000Qool00`00ooooo`3lool0027oo`800?goo`008Ooo
0P00oOoo000Qool2003mool0027oo`<00?coo`008Ooo00<00?oo0000o?oo000Qool01000ooooo`00
nooo000Qool01000ooooo`00nooo000Qool01@00oooooooo0000n_oo000Qool01@00oooooooo0000
n_oo000Qool20002ool00`00ooooo`3hool0027oo`05003oooooool0003jool0027oo`03003ooooo
00;oo`03003ooooo0?Ooo`008Ooo00<00?ooool00_oo00<00?ooool0mooo000Qool00`00ooooo`03
ool00`00ooooo`3fool0027oo`03003ooooo00?oo`03003ooooo0?Koo`008Ooo0P001Ooo00<00?oo
ool0mOoo000Qool00`00ooooo`05ool00`00ooooo`3dool0027oo`03003ooooo00Goo`03003ooooo
0?Coo`008Ooo00<00?ooool01_oo00<00?ooool0looo000Qool00`00ooooo`07ool00`00ooooo`3b
ool0027oo`03003ooooo00Soo`03003ooooo0?7oo`008Ooo00<00?ooool02?oo00<00?ooool0lOoo
000Qool2000:ool00`00ooooo`3Sool60007ool0027oo`03003ooooo00[oo`800<?oob0000goo`00
8Ooo00<00?ooool03?oo00<00?ooool0X?oo8000;Ooo000Qool00`00ooooo`0=ool30029oolF001=
ool000Soo`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo013oo`<0077ooaD006?o
o`001ooo00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`02ool00`00ooooo`0Cool2001I
oolF001hool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool00_oo0`005Ooo0`00
@?oo5P00S_oo0007ool01000ooooo`002?oo00@00?ooool000;oo`<000Goo`03003ooooo01Soo`P0
02goo`/00:Coo`001ooo00@00?ooool000Soo`04003ooooo0002ool00`00ooooo`05ool00`00oooo
o`0Pool]002_ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?oo
ool0o?oo0008ool2000:ool20003ool40004ool00`00ooooo`3lool0027oo`03003ooooo0?coo`00
8Ooo0P00oOoo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.70386, -0.215639, \
0.0809371, 0.00159662}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{th, 
        Re[N[alomz[1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 
              1, \(-9\)/10]]]}, {th, 0.01, Pi/2, 0.1}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.6604701058125517`}, {0.11`, 
        0.6586560456457287`}, {0.21000000000000002`, 
        0.6537490370141033`}, {0.31`, 
        0.6455540225049472`}, {0.41000000000000003`, 
        0.6337363237114501`}, {0.51`, 0.6177942854354646`}, {0.61`, 
        0.597017484166324`}, {0.7100000000000001`, 
        0.5704272735223761`}, {0.81`, 0.5366981967542259`}, {0.91`, 
        0.4940685156628228`}, {1.01`, 0.44028353334747766`}, {1.11`, 
        0.37272949018129353`}, {1.21`, 0.28925087869931243`}, {1.31`, 
        0.19095705836975105`}, {1.4100000000000001`, 
        0.08921522010616728`}, {1.51`, 0.014575646800141034`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<malpha.dat\>", %]\)], "Input"],

Cell[BoxData[
    \("malpha.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %53]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.268574 0.198693 [
[.29592 .25607 -3 -9 ]
[.29592 .25607 3 0 ]
[.52268 .25607 -6 -9 ]
[.52268 .25607 6 0 ]
[.74943 .25607 -6 -9 ]
[.74943 .25607 6 0 ]
[.97619 .25607 -6 -9 ]
[.97619 .25607 6 0 ]
[.05666 .06988 -12 -4.5 ]
[.05666 .06988 0 4.5 ]
[.05666 .16923 -24 -4.5 ]
[.05666 .16923 0 4.5 ]
[.05666 .36792 -18 -4.5 ]
[.05666 .36792 0 4.5 ]
[.05666 .46727 -6 -4.5 ]
[.05666 .46727 0 4.5 ]
[.05666 .56661 -18 -4.5 ]
[.05666 .56661 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .26857 m
.29592 .27482 L
s
[(5)] .29592 .25607 0 1 Mshowa
.52268 .26857 m
.52268 .27482 L
s
[(10)] .52268 .25607 0 1 Mshowa
.74943 .26857 m
.74943 .27482 L
s
[(15)] .74943 .25607 0 1 Mshowa
.97619 .26857 m
.97619 .27482 L
s
[(20)] .97619 .25607 0 1 Mshowa
.125 Mabswid
.11451 .26857 m
.11451 .27232 L
s
.15986 .26857 m
.15986 .27232 L
s
.20522 .26857 m
.20522 .27232 L
s
.25057 .26857 m
.25057 .27232 L
s
.34127 .26857 m
.34127 .27232 L
s
.38662 .26857 m
.38662 .27232 L
s
.43197 .26857 m
.43197 .27232 L
s
.47732 .26857 m
.47732 .27232 L
s
.56803 .26857 m
.56803 .27232 L
s
.61338 .26857 m
.61338 .27232 L
s
.65873 .26857 m
.65873 .27232 L
s
.70408 .26857 m
.70408 .27232 L
s
.79478 .26857 m
.79478 .27232 L
s
.84014 .26857 m
.84014 .27232 L
s
.88549 .26857 m
.88549 .27232 L
s
.93084 .26857 m
.93084 .27232 L
s
.02381 .26857 m
.02381 .27232 L
s
.25 Mabswid
0 .26857 m
1 .26857 L
s
.06916 .06988 m
.07541 .06988 L
s
[(-1)] .05666 .06988 1 0 Mshowa
.06916 .16923 m
.07541 .16923 L
s
[(-0.5)] .05666 .16923 1 0 Mshowa
.06916 .36792 m
.07541 .36792 L
s
[(0.5)] .05666 .36792 1 0 Mshowa
.06916 .46727 m
.07541 .46727 L
s
[(1)] .05666 .46727 1 0 Mshowa
.06916 .56661 m
.07541 .56661 L
s
[(1.5)] .05666 .56661 1 0 Mshowa
.125 Mabswid
.06916 .08975 m
.07291 .08975 L
s
.06916 .10962 m
.07291 .10962 L
s
.06916 .12949 m
.07291 .12949 L
s
.06916 .14936 m
.07291 .14936 L
s
.06916 .1891 m
.07291 .1891 L
s
.06916 .20897 m
.07291 .20897 L
s
.06916 .22884 m
.07291 .22884 L
s
.06916 .2487 m
.07291 .2487 L
s
.06916 .28844 m
.07291 .28844 L
s
.06916 .30831 m
.07291 .30831 L
s
.06916 .32818 m
.07291 .32818 L
s
.06916 .34805 m
.07291 .34805 L
s
.06916 .38779 m
.07291 .38779 L
s
.06916 .40766 m
.07291 .40766 L
s
.06916 .42753 m
.07291 .42753 L
s
.06916 .4474 m
.07291 .4474 L
s
.06916 .48714 m
.07291 .48714 L
s
.06916 .50701 m
.07291 .50701 L
s
.06916 .52687 m
.07291 .52687 L
s
.06916 .54674 m
.07291 .54674 L
s
.06916 .05001 m
.07291 .05001 L
s
.06916 .03014 m
.07291 .03014 L
s
.06916 .01027 m
.07291 .01027 L
s
.06916 .58648 m
.07291 .58648 L
s
.06916 .60635 m
.07291 .60635 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02472 0 m
.02499 .11517 L
.02605 .14929 L
.02729 .17219 L
.02846 .18697 L
.02954 .19734 L
.03053 .20508 L
.03279 .21837 L
.03506 .22802 L
.03756 .23599 L
.04017 .24235 L
.04262 .24707 L
.04761 .25422 L
.05217 .25888 L
.05744 .26285 L
.06244 .2657 L
.0675 .26795 L
.07289 .26983 L
.08259 .27233 L
.08853 .27347 L
.09412 .27434 L
.10458 .27561 L
.11434 .27648 L
.12466 .27717 L
.13346 .27762 L
.14313 .27801 L
.15334 .27832 L
.16264 .27855 L
.17338 .27875 L
.18355 .2789 L
.19321 .279 L
.20346 .27908 L
.21222 .27914 L
.22188 .27918 L
.2268 .2792 L
.23198 .27921 L
.2364 .27922 L
.23888 .27922 L
.24123 .27922 L
.24397 .27923 L
.24646 .27923 L
.24785 .27923 L
.24913 .27923 L
.2498 .27923 L
.25054 .27923 L
.25131 .27923 L
.25204 .27923 L
.25333 .27923 L
.25457 .27923 L
.25567 .27923 L
Mistroke
.25688 .27923 L
.25819 .27923 L
.25958 .27923 L
.26209 .27923 L
.2647 .27922 L
.26746 .27922 L
.2724 .27921 L
.28354 .2792 L
.29317 .27917 L
.30372 .27915 L
.3442 .27901 L
.38316 .27885 L
.42457 .27867 L
.46447 .2785 L
.50285 .27833 L
.54369 .27816 L
.583 .278 L
.62081 .27784 L
.66106 .27769 L
.6998 .27755 L
.74099 .2774 L
.78067 .27727 L
.81883 .27715 L
.85944 .27702 L
.89854 .27691 L
.94009 .27679 L
.97619 .27669 L
Mfstroke
.02483 0 m
.02499 .06858 L
.02605 .10127 L
.02729 .12366 L
.02846 .13861 L
.03053 .15795 L
.03279 .17337 L
.0379 .19807 L
.04262 .21447 L
.05224 .23894 L
.06244 .25812 L
.0825 .28611 L
.10458 .30934 L
.14487 .34172 L
.18364 .36633 L
.22486 .38843 L
.26457 .40708 L
.30277 .42325 L
.34341 .43902 L
.38254 .45308 L
.42412 .46704 L
.46419 .4797 L
.50274 .49125 L
.54374 .50296 L
.58323 .51374 L
.6212 .52371 L
.66163 .53393 L
.70054 .54342 L
.7419 .55318 L
.78174 .56229 L
.82007 .5708 L
.86085 .57961 L
.90012 .58788 L
.93787 .59563 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0
oooo0ooo000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000ooo`03003ooooo00Woo`800?oo
o`Coo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool00`00oooo
o`3oool3ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<0
0?ooool0oooo0ooo000?ool00`00ooooo`09ool2003oool4ool000ooo`03003ooooo00Woo`03003o
oooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool0
0`00ooooo`3oool3ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool0
2Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool2003oool4ool000ooo`03003ooooo00Wo
o`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool01000oooo
oooo10001?oo00<00?ooool0oooo0ooo000?ool00`00ooooo`03ool00`00ooooo`03ool00`00oooo
o`3oool3ool000ooo`03003ooooo00?oo`03003ooooo00?oo`<00?ooo`?oo`003Ooo10001?oo00<0
0?ooool00ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`03ool00`00ooooo`03ool00`00oooo
o`3oool3ool000ooo`04003oooooool30005ool00`00ooooo`3oool3ool000ooo`03003ooooo00?o
o`03003ooooo00?oo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo
000?ool00`00ooooo`09ool2003oool4ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`00
3ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool00`00ooooo`3oool3
ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo0P00oooo1?oo
000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000ooo`03003ooooo00Woo`03003ooooo0?oo
o`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool00`00oooo
o`3oool3ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo0P00
oooo1?oo000?ool2000:ool00`00ooooo`3oool3ool000ooo`8000[oo`03003ooooo0?ooo`?oo`00
3ooo0P002_oo00<00?ooool0oooo0ooo000?ool2000:ool00`00ooooo`3oool3ool000ooo`8000[o
o`03003ooooo0?ooo`?oo`003ooo0P002_oo0P00oooo1?oo000?ool2000:ool00`00ooooo`3oool3
ool000ooo`8000[oo`03003ooooo0?ooo`?oo`002?oo0P001?oo0`000_oo0`001Ooo00<00?ooool0
oooo0ooo0007ool01000ooooo`001?oo0P001Ooo00<00?ooool00_oo00<00?ooool0oooo0ooo0007
ool01000ooooo`001?oo00<00?oo00001?oo00<00?ooool00_oo0`00oooo0ooo00000ooo00000002
0002ool01000ooooo`001?oo00@00?oo003oo`<000Goo`03003ooooo0?ooo`?oo`001ooo00@00?oo
ool000Coo`05003oo`00ool00007ool00`00ooooo`3oool3ool000Ooo`04003ooooo0004ool01@00
ool00?oo00001ooo00<00?ooool0oooo0ooo0008ool20006ool200000ooo000000020004ool00`00
ooooo`3oool3ool0013oo`03003oo`0000Soo`03003ooooo0?ooo`?oo`004?oo00<00?oo00002?oo
0P00oooo1?oo000@ool00`00ool00008ool00`00ooooo`3oool3ool0013oo`03003oo`0000Soo`03
003ooooo0?ooo`?oo`004?oo00<00?oo00002?oo00<00?ooool0oooo0ooo000@ool01000ooooo`00
1ooo00<00?ooool0oooo0ooo000@ool01000ooooo`001ooo0P00oooo1?oo000@ool01@00oooooooo
00001_oo00<00?ooool0oooo0ooo000Aool01000ooooo`001_oo00<00?ooool0oooo0ooo000Aool0
1000ooooo`001_oo00<00?ooool0oooo0ooo000Aool01@00oooooooo00001Ooo00<00?ooool0oooo
0ooo000Aool01@00oooooooo00001Ooo00<00?ooool0oooo0ooo000Aool01@00oooooooo00001Ooo
0P00>ooo0`00>Ooo10000ooo0P00=ooo10000_oo0`00=_oo10000ooo0P001?oo000Bool01@00oooo
oooo00001?oo00<00?ooool0?Ooo00<00?ooool0>?oo00D00?ooooooo`0000;oo`03003ooooo03Ko
o`03003ooooo00Coo`03003ooooo03?oo`03003ooooo00?oo`04003ooooo0003ool001;oo`05003o
ooooool00004ool00`00ooooo`0mool00`00ooooo`0hool01@00oooooooo00000_oo00<00?ooool0
=_oo00<00?ooool01?oo00<00?ooool0=?oo00<00?ooool00_oo00@00?ooool000?oo`004ooo00@0
0?ooool000Coo`03003ooooo03[oo`<003_oo`05003oooooool00002ool00`00ooooo`0fool01000
oooooooo0`00>?oo00D00?ooooooo`0000;oo`03003ooooo007oo`004ooo00D00?ooooooo`0000?o
o`03003ooooo03[oo`03003ooooo03_oo`05003oooooool00002ool00`00ooooo`0fool01@00oooo
oooo0000>ooo00@00?ooool000;oo`03003ooooo007oo`005?oo00@00?ooool000?oo`8003_oo`03
003ooooo03Woo`<000?oo`04003ooooo000fool30003ool00`00ooooo`0fool01000ooooo`000_oo
00@00?ooool000?oo`005?oo00D00?ooooooo`0000;oo`03003ooooo03[oo`@003[oo`03003ooooo
00;oo`8003Woo`04003oooooool4000fool20004ool20004ool001Goo`04003ooooo0002ool00`00
ooooo`3oool3ool001Koo`06003ooooo003oo`00oooo1Ooo000Gool200000ooo0000003oool5ool0
01Woo`<00?ooo`Goo`002?ooo`006@00000>ool00`00ooooo`0:ool200000ooo000000040004ool0
0`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool0
0`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool0
0`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool0
0`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`05ool0
01_oo`03003oo`0000Kooa0005_oohX000Ooo`006ooo00<00?oo00005_ooF`00TOoo000Kool01000
ooooo`00oooo0_oo000Kool20002ool00`00ooooo`3nool001_oo`03003ooooo00;oo`03003ooooo
0?goo`006ooo00<00?ooool00ooo00<00?ooool0o?oo000Kool00`00ooooo`04ool00`00ooooo`3k
ool001_oo`03003ooooo00Goo`03003ooooo0?[oo`006ooo00<00?ooool01_oo00<00?ooool0nOoo
000Kool20008ool00`00ooooo`3hool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<0
0?ooool02Ooo0P00mooo000Kool00`00ooooo`0;ool00`00ooooo`3dool001_oo`03003ooooo00co
o`03003ooooo0??oo`006ooo0P003_oo00<00?ooool0l_oo000Kool00`00ooooo`0>ool00`00oooo
o`3aool001_oo`03003ooooo00ooo`800?7oo`006ooo00<00?ooool04Ooo00<00?ooool0k_oo000K
ool00`00ooooo`0Bool00`00ooooo`3]ool001_oo`03003ooooo01?oo`800>goo`006ooo0P005_oo
00<00?ooool0j_oo000Kool00`00ooooo`0Fool2003Zool001_oo`03003ooooo01Soo`800>Soo`00
2?oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool06_oo00<00?ooool0iOoo0007ool01000
ooooo`002ooo00<00?ooool00_oo00<00?ooool06ooo0P00iOoo0007ool01000ooooo`002ooo00<0
0?ooool00_oo0`007Ooo00<00?ooool0h_oo0007ool01000ooooo`002?oo0`001Ooo00<00?ooool0
7_oo0P00h_oo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool08?oo0P00h?oo0007
ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool08_oo0P00g_oo0008ool20009ool40004
ool00`00ooooo`0Tool2003Lool001_oo`03003ooooo02Koo`800=[oo`006ooo0P00:Ooo0P00f?oo
000Kool00`00ooooo`0Zool2003Fool001_oo`03003ooooo02coo`800=Coo`006ooo00<00?ooool0
;_oo0P00d_oo000Kool00`00ooooo`0`ool2003@ool001_oo`03003ooooo03;oo`<00<goo`006ooo
0P00=_oo0P00booo000Kool00`00ooooo`0gool20039ool001_oo`03003ooooo03Woo`800<Ooo`00
6ooo00<00?ooool0>ooo0P00aOoo000Kool00`00ooooo`0mool30032ool001_oo`80047oo`<00;oo
o`006ooo00<00?ooool0@ooo0P00_Ooo000Kool00`00ooooo`15ool3002jool001_oo`03003ooooo
04Soo`<00;Ooo`006ooo00<00?ooool0Booo0`00]?oo000Kool00`00ooooo`1>ool2002bool001_o
o`80057oo`<00:ooo`006ooo00<00?ooool0Dooo0`00[?oo000Kool00`00ooooo`1Fool3002Yool0
01?oo`@000Coo`03003ooooo05Woo`<00:Koo`005Ooo00<00?ooool00ooo00<00?ooool0G?oo0`00
Xooo000Eool00`00ooooo`03ool3001Oool3002Pool001Goo`03003ooooo00?oo`03003ooooo06;o
o`<009goo`005Ooo00<00?ooool00ooo00<00?ooool0IOoo0P00Vooo000Cool30005ool00`00oooo
o`1Wool3002Hool001Goo`03003ooooo00?oo`03003ooooo06[oo`<009Goo`006ooo00<00?ooool0
KOoo1000TOoo000Kool2001bool4002=ool001_oo`03003ooooo07Goo`<008[oo`006ooo00<00?oo
ool0N?oo1000Q_oo000Kool00`00ooooo`1lool40022ool001_oo`03003ooooo083oo`<007ooo`00
6ooo0P00Q?oo1000Nooo000Kool00`00ooooo`27ool4001gool001_oo`03003ooooo08_oo`<007Co
o`006ooo00<00?ooool0S_oo1000L?oo000Kool00`00ooooo`2Bool4001/ool001_oo`03003ooooo
09Koo`<006Woo`006ooo0P00V_oo1000IOoo000Kool00`00ooooo`2Mool4001Qool001_oo`03003o
oooo0:7oo`@005goo`006ooo00<00?ooool0YOoo1P00Eooo000Kool00`00ooooo`2[ool4001Cool0
01_oo`800;3oo`@004ooo`006ooo00<00?ooool0/ooo1000Booo000Kool00`00ooooo`2gool40017
ool001_oo`03003ooooo0;_oo`@004?oo`001ooo10000ooo00<00?ooool00_oo0`001Ooo00<00?oo
ool0_ooo1000?ooo0009ool00`00ooooo`0:ool00`00ooooo`02ool00`00ooooo`33ool4000kool0
00Woo`03003ooooo00[oo`03003ooooo00;oo`<00<Ooo`D003Koo`002Ooo00<00?ooool01ooo0`00
1Ooo00<00?ooool0c?oo1@00<Ooo0009ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`3A
ool6000[ool000Ooo`<000Woo`03003ooooo00Goo`03003ooooo0=Ooo`D002Koo`002Ooo00<00?oo
ool01ooo10001?oo00<00?ooool0g?oo10008_oo000Kool2003Qool4000Nool001_oo`03003ooooo
0>Coo`@001[oo`006ooo00<00?ooool0j?oo1@005Ooo000Kool00`00ooooo`3]ool5000@ool001_o
o`03003ooooo0?;oo`H000[oo`006ooo00<00?ooool0n?oo0`001ooo000Kool2003oool4ool001_o
o`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool00?oo
ob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.17393, -1.39354, \
0.0790907, 0.0180524}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(\(aha!\)\ changes\ \(much!\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-alomz[1, 0.9, 1, a]\), {a, \(-1\), 20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.39104 0.531405 [
[.29592 .37854 -3 -9 ]
[.29592 .37854 3 0 ]
[.52268 .37854 -6 -9 ]
[.52268 .37854 6 0 ]
[.74943 .37854 -6 -9 ]
[.74943 .37854 6 0 ]
[.97619 .37854 -6 -9 ]
[.97619 .37854 6 0 ]
[.05666 .0722 -24 -4.5 ]
[.05666 .0722 0 4.5 ]
[.05666 .17848 -24 -4.5 ]
[.05666 .17848 0 4.5 ]
[.05666 .28476 -24 -4.5 ]
[.05666 .28476 0 4.5 ]
[.05666 .49732 -18 -4.5 ]
[.05666 .49732 0 4.5 ]
[.05666 .6036 -18 -4.5 ]
[.05666 .6036 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .39104 m
.29592 .39729 L
s
[(5)] .29592 .37854 0 1 Mshowa
.52268 .39104 m
.52268 .39729 L
s
[(10)] .52268 .37854 0 1 Mshowa
.74943 .39104 m
.74943 .39729 L
s
[(15)] .74943 .37854 0 1 Mshowa
.97619 .39104 m
.97619 .39729 L
s
[(20)] .97619 .37854 0 1 Mshowa
.125 Mabswid
.11451 .39104 m
.11451 .39479 L
s
.15986 .39104 m
.15986 .39479 L
s
.20522 .39104 m
.20522 .39479 L
s
.25057 .39104 m
.25057 .39479 L
s
.34127 .39104 m
.34127 .39479 L
s
.38662 .39104 m
.38662 .39479 L
s
.43197 .39104 m
.43197 .39479 L
s
.47732 .39104 m
.47732 .39479 L
s
.56803 .39104 m
.56803 .39479 L
s
.61338 .39104 m
.61338 .39479 L
s
.65873 .39104 m
.65873 .39479 L
s
.70408 .39104 m
.70408 .39479 L
s
.79478 .39104 m
.79478 .39479 L
s
.84014 .39104 m
.84014 .39479 L
s
.88549 .39104 m
.88549 .39479 L
s
.93084 .39104 m
.93084 .39479 L
s
.02381 .39104 m
.02381 .39479 L
s
.25 Mabswid
0 .39104 m
1 .39104 L
s
.06916 .0722 m
.07541 .0722 L
s
[(-0.6)] .05666 .0722 1 0 Mshowa
.06916 .17848 m
.07541 .17848 L
s
[(-0.4)] .05666 .17848 1 0 Mshowa
.06916 .28476 m
.07541 .28476 L
s
[(-0.2)] .05666 .28476 1 0 Mshowa
.06916 .49732 m
.07541 .49732 L
s
[(0.2)] .05666 .49732 1 0 Mshowa
.06916 .6036 m
.07541 .6036 L
s
[(0.4)] .05666 .6036 1 0 Mshowa
.125 Mabswid
.06916 .09877 m
.07291 .09877 L
s
.06916 .12534 m
.07291 .12534 L
s
.06916 .15191 m
.07291 .15191 L
s
.06916 .20505 m
.07291 .20505 L
s
.06916 .23162 m
.07291 .23162 L
s
.06916 .25819 m
.07291 .25819 L
s
.06916 .31133 m
.07291 .31133 L
s
.06916 .3379 m
.07291 .3379 L
s
.06916 .36447 m
.07291 .36447 L
s
.06916 .41761 m
.07291 .41761 L
s
.06916 .44418 m
.07291 .44418 L
s
.06916 .47075 m
.07291 .47075 L
s
.06916 .52389 m
.07291 .52389 L
s
.06916 .55046 m
.07291 .55046 L
s
.06916 .57703 m
.07291 .57703 L
s
.06916 .04563 m
.07291 .04563 L
s
.06916 .01906 m
.07291 .01906 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02687 0 m
.02729 .02055 L
.02846 .06087 L
.03053 .11296 L
.03279 .15425 L
.03527 .18948 L
.0379 .21957 L
.04262 .26196 L
.05205 .32193 L
.05742 .34751 L
.06244 .36787 L
.08274 .42801 L
.09415 .45206 L
.10458 .47018 L
.12357 .49638 L
.14429 .518 L
.16396 .53388 L
.18493 .54725 L
.20491 .55744 L
.22652 .56635 L
.26658 .57862 L
.28652 .58321 L
.30759 .58723 L
.34556 .59278 L
.36507 .59497 L
.38598 .59691 L
.40579 .59842 L
.42734 .59976 L
.44628 .6007 L
.46719 .60154 L
.48696 .60215 L
.50797 .60264 L
.52732 .60296 L
.5382 .60309 L
.54817 .60318 L
.55291 .60322 L
.55798 .60325 L
.56235 .60327 L
.5671 .60329 L
.57 .6033 L
.57268 .60331 L
.57523 .60331 L
.57654 .60331 L
.57794 .60332 L
.57921 .60332 L
.58037 .60332 L
.58153 .60332 L
.58264 .60332 L
.58392 .60332 L
.58511 .60332 L
Mistroke
.58641 .60332 L
.58714 .60332 L
.5878 .60332 L
.59013 .60331 L
.59145 .60331 L
.59269 .60331 L
.59785 .6033 L
.60231 .60328 L
.60708 .60327 L
.61799 .60321 L
.62792 .60314 L
.64704 .60296 L
.66781 .60272 L
.70865 .60209 L
.74797 .60134 L
.78577 .60051 L
.82602 .59955 L
.86476 .59855 L
.90596 .59743 L
.94563 .59632 L
.97619 .59544 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0
oooo0ooo000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000ooo`03003ooooo00Woo`03003o
oooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool2
003oool4ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<0
0?ooool0oooo0ooo000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000ooo`03003ooooo00Wo
o`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00oooo
o`09ool00`00ooooo`3oool3ool0013oo`03003ooooo00Soo`800?ooo`Coo`004?oo00<00?ooool0
2?oo00<00?ooool0oooo0ooo000@ool00`00ooooo`08ool00`00ooooo`3oool3ool0013oo`03003o
oooo00Soo`03003ooooo0?ooo`?oo`004?oo00<00?ooool02?oo00<00?ooool0oooo0ooo000@ool0
0`00ooooo`08ool00`00ooooo`3oool3ool000Soo`8000Coo`03003oo`0000?oo`8000Goo`03003o
oooo0?ooo`?oo`001ooo00@00?ooool000Goo`04003ooooo0002ool00`00ooooo`02ool00`00oooo
o`3oool3ool000Ooo`04003ooooo0005ool01000ooooo`000_oo00<00?ooool00_oo0`00oooo0ooo
00000ooo000000020002ool01000ooooo`001Ooo00<00?ooool00`001Ooo00<00?ooool0oooo0ooo
0007ool01000ooooo`001Ooo00@00?ooool000Ooo`03003ooooo0?ooo`?oo`001ooo00@00?ooool0
00Goo`04003ooooo0007ool00`00ooooo`3oool3ool000Soo`8000Koo`04003oooooool30004ool0
0`00ooooo`3oool3ool0013oo`03003ooooo00Soo`03003ooooo0?ooo`?oo`004?oo00<00?ooool0
2?oo00<00?ooool0oooo0ooo000@ool00`00ooooo`08ool2003oool4ool0013oo`03003ooooo00So
o`03003ooooo0?ooo`?oo`004?oo00<00?ooool02?oo00<00?ooool0oooo0ooo000@ool00`00oooo
o`08ool00`00ooooo`3oool3ool0013oo`03003ooooo00Soo`03003ooooo0?ooo`?oo`004?oo00<0
0?ooool02?oo00<00?ooool0oooo0ooo000@ool00`00ooooo`08ool00`00ooooo`3oool3ool0013o
o`03003ooooo00Soo`03003ooooo0?ooo`?oo`004?oo00<00?ooool02?oo0P00oooo1?oo000@ool0
0`00ooooo`08ool00`00ooooo`3oool3ool0013oo`03003ooooo00Soo`03003ooooo0?ooo`?oo`00
4Ooo00<00?ooool01ooo00<00?ooool0oooo0ooo000Aool00`00ooooo`07ool00`00ooooo`3oool3
ool0017oo`03003ooooo00Ooo`03003ooooo0?ooo`?oo`004Ooo00<00?ooool01ooo00<00?ooool0
oooo0ooo000Aool00`00ooooo`07ool2003oool4ool0017oo`03003ooooo00Ooo`03003ooooo0?oo
o`?oo`004Ooo00<00?ooool01ooo00<00?ooool0oooo0ooo000Aool00`00ooooo`07ool00`00oooo
o`3oool3ool0017oo`03003ooooo00Ooo`03003ooooo0?ooo`?oo`002?oo0P001?oo00@00?ooool0
00?oo`03003ooooo00?oo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Ooo`04003ooooo0005
ool00`00ooooo`3oool3ool000Ooo`04003ooooo0007ool50004ool3003oool3ool00003ool00000
008000;oo`04003ooooo0007ool01000ooooo`001Ooo00<00?ooool0oooo0ooo0007ool01000oooo
o`001ooo0P0000?oo`00ool01?oo00<00?ooool0oooo0ooo0007ool01000ooooo`001ooo00@00?oo
000000Goo`03003ooooo0?ooo`?oo`002?oo0P002?oo00@00?ooool000Goo`03003ooooo0?ooo`?o
o`004_oo00<00?ooool01_oo00<00?ooool0oooo0ooo000Bool00`00ooooo`06ool00`00ooooo`3o
ool3ool001;oo`03003ooooo00Koo`03003ooooo0?ooo`?oo`004_oo00<00?ooool01_oo0P00oooo
1?oo000Bool00`00ooooo`06ool00`00ooooo`3oool3ool001;oo`03003ooooo00Koo`03003ooooo
0?ooo`?oo`004_oo00<00?ooool01_oo00<00?ooool0oooo0ooo000Bool00`00ooooo`06ool00`00
ooooo`3oool3ool001;oo`03003ooooo00Koo`03003ooooo0?ooo`?oo`004_oo00<00?ooool01_oo
00<00?ooool0oooo0ooo000Bool00`00ooooo`06ool2003oool4ool001?oo`03003ooooo00Goo`03
003ooooo0?ooo`?oo`004ooo00<00?ooool01Ooo00<00?ooool0oooo0ooo000Cool00`00ooooo`05
ool00`00ooooo`3oool3ool001?oo`03003ooooo00Goo`03003ooooo0?ooo`?oo`004ooo00<00?oo
ool01Ooo00<00?ooool0oooo0ooo000Cool00`00ooooo`05ool00`00ooooo`3oool3ool001Coo`03
003ooooo00Coo`03003ooooo0?ooo`?oo`005?oo00<00?ooool01?oo0P00oooo1?oo000Dool00`00
ooooo`04ool00`00ooooo`3oool3ool001Coo`03003ooooo00Coo`03003ooooo0?ooo`?oo`005?oo
00<00?ooool01?oo00<00?ooool0oooo0ooo000Dool00`00ooooo`04ool00`00ooooo`3oool3ool0
00Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000So
o`03003oo`0000Goo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Woo`8000Goo`<00?ooo`?o
o`0000?oo`0000000P000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo0?ooo`?oo`00
1ooo00@00?ooool000[oo`8000Coo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Soo`04003o
o`000004ool00`00ooooo`3oool3ool000Soo`8000[oo`8000Goo`03003ooooo0?ooo`?oo`005Ooo
00<00?ooool00ooo00<00?ooool0oooo0ooo000Eool00`00ooooo`03ool00`00ooooo`3oool3ool0
01Koo`03003ooooo00;oo`800?ooo`Coo`005_oo00<00?ooool00_oo00<00?ooool0oooo0ooo000F
ool00`00ooooo`02ool00`00ooooo`3oool3ool001Koo`03003ooooo00;oo`03003ooooo0?ooo`?o
o`005_oo00<00?ooool00_oo00<00?ooool0oooo0ooo000Fool00`00ooooo`02ool00`00ooooo`3o
ool3ool001Ooo`05003oooooool0003oool5ool001Ooo`05003oooooool0003oool5ool001Ooo`04
003oooooool2003oool4ool001Ooo`05003oooooool0003oool5ool001Soo`04003ooooo003oool5
ool001Soo`04003ooooo003oool5ool001Soo`04003ooooo000lool3000iool40003ool2000gool4
0002ool3000fool40003ool20004ool001Soo`04003ooooo000oool00`00ooooo`0hool01@00oooo
oooo00000_oo00<00?ooool0=_oo00<00?ooool01?oo00<00?ooool0<ooo00<00?ooool00ooo00@0
0?ooool000?oo`006Ooo00<00?oo0000?ooo00<00?ooool0>?oo00D00?ooooooo`0000;oo`03003o
oooo03Koo`03003ooooo00Coo`03003ooooo03Coo`03003ooooo00;oo`04003ooooo0003ool001Wo
o`04003oo`00000kool3000kool01@00oooooooo00000_oo00<00?ooool0=_oo00@00?ooooooo`<0
03Soo`05003oooooool00002ool00`00ooooo`01ool001Woo`03003oo`0003coo`03003ooooo03_o
o`05003oooooool00002ool00`00ooooo`0fool01@00oooooooo0000>ooo00@00?ooool000;oo`03
003ooooo007oo`006Ooo00<00?oo0000??oo00<00?ooool0>Ooo0`000ooo00@00?ooool003Koo`<0
00?oo`03003ooooo03Koo`04003ooooo0002ool01000ooooo`000ooo000Jool2000lool4000jool0
0`00ooooo`02ool2000iool01000oooooooo1000=_oo0P001?oo0P001?oo000Jool2003oool5ool0
01[oo`800?ooo`Goo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool000Soool0
01T000003_oo00<00?ooool02_oo0P002ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool0
2_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool0
2_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool0
2_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool0
2Ooo00<00?ooool02_oo00<00?ooool01Ooo000Kool2003oool4ool001_oo`03003oo`000?ooo`?o
o`006ooo00<00?oo0000oooo0ooo000Kool00`00ool0003oool3ool001_oo`04003ooooo003oool2
ool001_oo`800003ool00?oo0?ooo`7oo`006ooo00@00?ooool00?ooo`;oo`006ooo00D00?oooooo
o`000?ooo`7oo`006ooo00D00?ooooooo`000?ooo`7oo`006ooo00D00?ooooooo`000?ooo`7oo`00
6ooo00<00?ooool00_oo00<00?ooool0oOoo000Kool00`00ooooo`02ool00`00ooooo`3mool001_o
o`8000Coo`03003ooooo0?coo`006ooo00<00?ooool00ooo00<00?ooool0o?oo000Kool00`00oooo
o`04ool00`00ooooo`3kool001_oo`03003ooooo00Coo`03003ooooo0?_oo`006ooo00<00?ooool0
1Ooo00<00?ooool0n_oo000Kool00`00ooooo`05ool00`00ooooo`3jool001_oo`03003ooooo00Ko
o`03003ooooo0?Woo`006ooo00<00?ooool01_oo00<00?ooool0nOoo000Kool20008ool00`00oooo
o`3hool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0
mooo000Kool00`00ooooo`09ool00`00ooooo`3fool001_oo`03003ooooo00[oo`03003ooooo0?Go
o`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool02ooo00<00?ooool0m?oo0007ool0
1000ooooo`002?oo00<00?ooool01Ooo00<00?ooool02ooo00<00?ooool0m?oo0007ool01000oooo
o`002Ooo00<00?ooool01?oo0`003?oo00<00?ooool0looo0007ool01000ooooo`002_oo00<00?oo
ool00ooo00<00?ooool03Ooo00<00?ooool0l_oo0007ool01000ooooo`002ooo00<00?ooool00_oo
00<00?ooool03_oo00<00?ooool0lOoo0007ool01000ooooo`002?oo00@00?ooool000Coo`03003o
oooo00ooo`03003ooooo0?3oo`002?oo0P002_oo0P001Ooo00<00?ooool04?oo00<00?ooool0kooo
000Kool00`00ooooo`0Aool00`00ooooo`3^ool001_oo`03003ooooo01;oo`03003ooooo0>goo`00
6ooo00<00?ooool04ooo00<00?ooool0k?oo000Kool2000Eool2003/ool001_oo`03003ooooo01Ko
o`03003ooooo0>Woo`006ooo00<00?ooool05ooo00<00?ooool0j?oo000Kool00`00ooooo`0Hool2
003Xool001_oo`03003ooooo01[oo`03003ooooo0>Goo`006ooo00<00?ooool06ooo0P00iOoo000K
ool00`00ooooo`0Mool00`00ooooo`3Rool001_oo`8001ooo`800>;oo`006ooo00<00?ooool08?oo
0P00h?oo000Kool00`00ooooo`0Rool2003Nool001_oo`03003ooooo02Coo`800=coo`006ooo00<0
0?ooool09_oo0P00f_oo000Kool00`00ooooo`0Xool3003Gool001_oo`03003ooooo02_oo`@00=?o
o`006ooo00<00?ooool0;ooo1000cooo000Kool2000dool4003;ool001_oo`03003ooooo03Ooo`H0
0<Goo`006ooo00<00?ooool0?Ooo1@00`?oo000Kool00`00ooooo`12ool6002jool001_oo`03003o
oooo04Soo`D00;Goo`002?oo0P001?oo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0COoo
4@00Pooo6P001ooo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool0G_oo4@00>ooo
=`008Ooo0007ool01000ooooo`001ooo1@001?oo0`00Kooo>`00F?oo0007ool01000ooooo`001ooo
00@00?ooool000Goo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Soo`03003oo`0000Goo`03
003ooooo0?ooo`?oo`001ooo00@00?ooool000Woo`8000Goo`03003ooooo0?ooo`?oo`002?oo0P00
2ooo00<00?ooool00ooo00<00?ooool0oooo0ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.17393, -0.749892, \
0.0790907, 0.0067498}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %53]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 0.306205 0.176356 [
[.29592 .2937 -3 -9 ]
[.29592 .2937 3 0 ]
[.52268 .2937 -6 -9 ]
[.52268 .2937 6 0 ]
[.74943 .2937 -6 -9 ]
[.74943 .2937 6 0 ]
[.97619 .2937 -6 -9 ]
[.97619 .2937 6 0 ]
[.05666 .04167 -24 -4.5 ]
[.05666 .04167 0 4.5 ]
[.05666 .12985 -12 -4.5 ]
[.05666 .12985 0 4.5 ]
[.05666 .21803 -24 -4.5 ]
[.05666 .21803 0 4.5 ]
[.05666 .39438 -18 -4.5 ]
[.05666 .39438 0 4.5 ]
[.05666 .48256 -6 -4.5 ]
[.05666 .48256 0 4.5 ]
[.05666 .57074 -18 -4.5 ]
[.05666 .57074 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .3062 m
.29592 .31245 L
s
[(5)] .29592 .2937 0 1 Mshowa
.52268 .3062 m
.52268 .31245 L
s
[(10)] .52268 .2937 0 1 Mshowa
.74943 .3062 m
.74943 .31245 L
s
[(15)] .74943 .2937 0 1 Mshowa
.97619 .3062 m
.97619 .31245 L
s
[(20)] .97619 .2937 0 1 Mshowa
.125 Mabswid
.11451 .3062 m
.11451 .30995 L
s
.15986 .3062 m
.15986 .30995 L
s
.20522 .3062 m
.20522 .30995 L
s
.25057 .3062 m
.25057 .30995 L
s
.34127 .3062 m
.34127 .30995 L
s
.38662 .3062 m
.38662 .30995 L
s
.43197 .3062 m
.43197 .30995 L
s
.47732 .3062 m
.47732 .30995 L
s
.56803 .3062 m
.56803 .30995 L
s
.61338 .3062 m
.61338 .30995 L
s
.65873 .3062 m
.65873 .30995 L
s
.70408 .3062 m
.70408 .30995 L
s
.79478 .3062 m
.79478 .30995 L
s
.84014 .3062 m
.84014 .30995 L
s
.88549 .3062 m
.88549 .30995 L
s
.93084 .3062 m
.93084 .30995 L
s
.02381 .3062 m
.02381 .30995 L
s
.25 Mabswid
0 .3062 m
1 .3062 L
s
.06916 .04167 m
.07541 .04167 L
s
[(-1.5)] .05666 .04167 1 0 Mshowa
.06916 .12985 m
.07541 .12985 L
s
[(-1)] .05666 .12985 1 0 Mshowa
.06916 .21803 m
.07541 .21803 L
s
[(-0.5)] .05666 .21803 1 0 Mshowa
.06916 .39438 m
.07541 .39438 L
s
[(0.5)] .05666 .39438 1 0 Mshowa
.06916 .48256 m
.07541 .48256 L
s
[(1)] .05666 .48256 1 0 Mshowa
.06916 .57074 m
.07541 .57074 L
s
[(1.5)] .05666 .57074 1 0 Mshowa
.125 Mabswid
.06916 .05931 m
.07291 .05931 L
s
.06916 .07694 m
.07291 .07694 L
s
.06916 .09458 m
.07291 .09458 L
s
.06916 .11221 m
.07291 .11221 L
s
.06916 .14748 m
.07291 .14748 L
s
.06916 .16512 m
.07291 .16512 L
s
.06916 .18276 m
.07291 .18276 L
s
.06916 .20039 m
.07291 .20039 L
s
.06916 .23566 m
.07291 .23566 L
s
.06916 .2533 m
.07291 .2533 L
s
.06916 .27093 m
.07291 .27093 L
s
.06916 .28857 m
.07291 .28857 L
s
.06916 .32384 m
.07291 .32384 L
s
.06916 .34148 m
.07291 .34148 L
s
.06916 .35911 m
.07291 .35911 L
s
.06916 .37675 m
.07291 .37675 L
s
.06916 .41202 m
.07291 .41202 L
s
.06916 .42965 m
.07291 .42965 L
s
.06916 .44729 m
.07291 .44729 L
s
.06916 .46493 m
.07291 .46493 L
s
.06916 .5002 m
.07291 .5002 L
s
.06916 .51783 m
.07291 .51783 L
s
.06916 .53547 m
.07291 .53547 L
s
.06916 .5531 m
.07291 .5531 L
s
.06916 .02403 m
.07291 .02403 L
s
.06916 .0064 m
.07291 .0064 L
s
.06916 .58837 m
.07291 .58837 L
s
.06916 .60601 m
.07291 .60601 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02464 0 m
.02499 .13395 L
.02605 .16319 L
.02729 .18325 L
.02846 .19663 L
.03053 .21392 L
.03279 .22762 L
.03527 .23931 L
.0379 .2493 L
.04262 .26337 L
.05205 .28327 L
.05742 .29176 L
.06244 .29852 L
.08274 .31847 L
.09415 .32646 L
.10458 .33247 L
.12357 .34116 L
.14429 .34834 L
.16396 .35361 L
.18493 .35805 L
.20491 .36143 L
.22652 .36438 L
.26658 .36846 L
.28652 .36998 L
.30759 .37131 L
.34556 .37316 L
.36507 .37388 L
.38598 .37453 L
.40579 .37503 L
.42734 .37547 L
.44628 .37579 L
.46719 .37606 L
.48696 .37627 L
.50797 .37643 L
.52732 .37653 L
.5382 .37658 L
.54817 .37661 L
.55291 .37662 L
.55798 .37663 L
.56235 .37664 L
.5671 .37664 L
.57 .37665 L
.57268 .37665 L
.57523 .37665 L
.57654 .37665 L
.57794 .37665 L
.57921 .37665 L
.58037 .37665 L
.58153 .37665 L
.58264 .37665 L
Mistroke
.58392 .37665 L
.58511 .37665 L
.58641 .37665 L
.58714 .37665 L
.5878 .37665 L
.59013 .37665 L
.59145 .37665 L
.59269 .37665 L
.59785 .37665 L
.60231 .37664 L
.60708 .37664 L
.61799 .37662 L
.62792 .37659 L
.64704 .37653 L
.66781 .37645 L
.70865 .37625 L
.74797 .376 L
.78577 .37572 L
.82602 .3754 L
.86476 .37507 L
.90596 .3747 L
.94563 .37433 L
.97619 .37404 L
Mfstroke
.02465 0 m
.02499 .12869 L
.02605 .15771 L
.02729 .17758 L
.02846 .19085 L
.03053 .20802 L
.03279 .2217 L
.0379 .24363 L
.04262 .25818 L
.05224 .27991 L
.06244 .29693 L
.0825 .32177 L
.10458 .34239 L
.14487 .37113 L
.18364 .39297 L
.22486 .41259 L
.26457 .42914 L
.30277 .44349 L
.34341 .45749 L
.38254 .46997 L
.42412 .48236 L
.46419 .49359 L
.50274 .50385 L
.54374 .51424 L
.58323 .52381 L
.6212 .53266 L
.66163 .54173 L
.70054 .55015 L
.7419 .55882 L
.78174 .5669 L
.82007 .57446 L
.86085 .58228 L
.90012 .58961 L
.93787 .59649 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0
oooo0ooo000?ool00`00ooooo`09ool2003oool4ool000ooo`03003ooooo00Woo`03003ooooo0?oo
o`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool00`00oooo
o`3oool3ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo0P00
oooo1?oo000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000Ooo`@000?oo`8000?oo`<000Go
o`03003ooooo0?ooo`?oo`002Ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool00_oo00<0
0?ooool0oooo0ooo0009ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`02ool3003oool3
ool00003ool00000008000Coo`03003ooooo00?oo`04003oooooool30005ool00`00ooooo`3oool3
ool000Woo`03003ooooo00?oo`05003oooooool00007ool00`00ooooo`3oool3ool000Ooo`<000Go
o`05003oooooool00007ool00`00ooooo`3oool3ool000Woo`03003ooooo00?oo`04003oooooool4
0004ool00`00ooooo`3oool3ool000ooo`03003ooooo00Woo`800?ooo`Coo`003ooo00<00?ooool0
2Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000ooo`03003o
oooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool0
0`00ooooo`09ool2003oool4ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<0
0?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000oo
o`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo0P00oooo1?oo000?ool0
0`00ooooo`09ool00`00ooooo`3oool3ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`00
3ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool00`00ooooo`3oool3
ool000ooo`03003ooooo00Woo`800?ooo`Coo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo
000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000ooo`04003oooooool40004ool00`00oooo
o`3oool3ool000ooo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo0?ooo`?oo`003ooo00<0
0?ooool00ooo00<00?ooool00ooo0`00oooo0ooo000=ool40004ool00`00ooooo`03ool00`00oooo
o`3oool3ool000ooo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo0?ooo`?oo`003ooo00@0
0?ooooooo`<000Goo`03003ooooo0?ooo`?oo`003ooo00<00?ooool00ooo00<00?ooool00ooo00<0
0?ooool0oooo0ooo000?ool00`00ooooo`09ool2003oool4ool000ooo`03003ooooo00Woo`03003o
oooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00ooooo`09ool0
0`00ooooo`3oool3ool000ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003ooo00<00?ooool0
2Ooo0P00oooo1?oo000?ool00`00ooooo`09ool00`00ooooo`3oool3ool000ooo`03003ooooo00Wo
o`03003ooooo0?ooo`?oo`003ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo000?ool00`00oooo
o`09ool00`00ooooo`3oool3ool000ooo`03003ooooo00Woo`800?ooo`Coo`003ooo0P002_oo00<0
0?ooool0oooo0ooo000?ool2000:ool00`00ooooo`3oool3ool0013oo`03003ooooo00Soo`03003o
oooo0?ooo`?oo`004?oo00<00?ooool02?oo00<00?ooool0oooo0ooo000@ool00`00ooooo`08ool2
003oool4ool0013oo`03003ooooo00Soo`03003ooooo0?ooo`?oo`004?oo00<00?ooool02?oo00<0
0?ooool0oooo0ooo0008ool20004ool00`00ool00002ool30005ool00`00ooooo`3oool3ool000Oo
o`04003ooooo0005ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`3oool3ool000Ooo`04
003ooooo0005ool20004ool00`00ooooo`02ool3003oool3ool00003ool00000008000;oo`04003o
oooo0006ool00`00ool000020005ool00`00ooooo`3oool3ool000Ooo`04003ooooo0006ool00`00
ool00007ool00`00ooooo`3oool3ool000Ooo`04003ooooo0006ool00`00ool00007ool00`00oooo
o`3oool3ool000Soo`8000Ooo`03003oo`0000<000Coo`03003ooooo0?ooo`?oo`004_oo00<00?oo
ool01_oo0P00oooo1?oo000Bool00`00ooooo`06ool00`00ooooo`3oool3ool001;oo`03003ooooo
00Koo`03003ooooo0?ooo`?oo`004_oo00<00?ooool01_oo00<00?ooool0oooo0ooo000Bool20007
ool2003oool4ool001?oo`03003ooooo00Goo`03003ooooo0?ooo`?oo`004ooo0P001_oo00<00?oo
ool0oooo0ooo000Dool00`00ooooo`04ool00`00ooooo`3oool3ool001Coo`8000Goo`03003ooooo
0?ooo`?oo`005Ooo00<00?ooool00ooo0P00>ooo0`00>Ooo10000ooo0P00=ooo10000_oo0`00=_oo
10000ooo0P001?oo000Eool00`00ooooo`03ool00`00ooooo`0mool00`00ooooo`0hool01@00oooo
oooo00000_oo00<00?ooool0=_oo00<00?ooool01?oo00<00?ooool0<ooo00<00?ooool00ooo00@0
0?ooool000?oo`005Ooo0P001?oo00<00?ooool0?Ooo00<00?ooool0>?oo00D00?ooooooo`0000;o
o`03003ooooo03Koo`03003ooooo00Coo`03003ooooo03Coo`03003ooooo00;oo`04003ooooo0003
ool001Koo`03003ooooo00;oo`03003ooooo03[oo`<003_oo`05003oooooool00002ool00`00oooo
o`0fool01000oooooooo0`00>?oo00D00?ooooooo`0000;oo`03003ooooo007oo`005_oo0P000ooo
00<00?ooool0>_oo00<00?ooool0>ooo00D00?ooooooo`0000;oo`03003ooooo03Koo`05003ooooo
ool0000kool01000ooooo`000_oo00<00?ooool00Ooo000Gool01000oooooooo0P00>ooo00<00?oo
ool0>Ooo0`000ooo00@00?ooool003Koo`<000?oo`03003ooooo03Koo`04003ooooo0002ool01000
ooooo`000ooo000Hool01000ooooo`00??oo1000>_oo00<00?ooool00_oo0P00>Ooo00@00?oooooo
o`@003Koo`8000Coo`8000Coo`006?oo00@00?ooool00?ooo`Goo`006Ooo00<00?oo0000oooo1Ooo
000Jool2003oool5ool000Soool001T000003_oo00<00?ooool02_oo0P002ooo00<00?ooool02Ooo
00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo
00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02_oo
00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo
00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool01Ooo000Kool3000lool0
0`00ooooo`0lool00`00ooooo`0mool00`00ooooo`0lool00`00ooooo`05ool001_oo`04003oo`00
003oool2ool001_oo`03003ooooo00800?ooo`7oo`006ooo0P000_oo0`00o_oo000Kool00`00oooo
o`02ool00`00ool0003mool001_oo`03003ooooo00?oo`04003oo`00003kool001_oo`03003ooooo
00Coo`800003ool000000?Woo`006ooo00<00?ooool01_oo00<00?ooool00P00mooo000Kool20008
ool01000oooooooo0`00m?oo000Kool00`00ooooo`08ool20004ool3003aool001_oo`03003ooooo
00[oo`03003ooooo00Coo`@00>goo`006ooo00<00?ooool02ooo00<00?ooool01ooo1000jOoo000K
ool00`00ooooo`0<ool2000;ool3003Vool001_oo`8000ooo`03003ooooo00_oo`@00>;oo`006ooo
00<00?ooool03ooo00<00?ooool03_oo3000e_oo000Kool00`00ooooo`0@ool2000Jool6003@ool0
01_oo`03003ooooo01;oo`03003ooooo01gooa<00;goo`006ooo00<00?ooool04ooo0P00<?oo6@00
N?oo9@001ooo000Kool2000Fool20017oomh000/ool001_oo`03003ooooo01Ooo`800>Woo`006ooo
00<00?ooool06Ooo0P00iooo0008ool20004ool00`00ooooo`02ool30005ool00`00ooooo`0Kool2
003Uool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0Mool00`00ooooo`3Rool0
00Ooo`04003ooooo000;ool00`00ooooo`02ool3000Nool2003Rool000Ooo`04003ooooo0008ool3
0005ool00`00ooooo`0Pool2003Pool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00oooo
o`0Rool2003Nool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0Tool2003Lool0
00Soo`8000Woo`@000Coo`03003ooooo02Koo`800=[oo`006ooo0P00:Ooo0P00f?oo000Kool00`00
ooooo`0Zool2003Fool001_oo`03003ooooo02coo`800=Coo`006ooo00<00?ooool0;_oo0P00d_oo
000Kool00`00ooooo`0`ool2003@ool001_oo`8003?oo`<00<goo`006ooo00<00?ooool0=Ooo0`00
b_oo000Kool00`00ooooo`0hool20038ool001_oo`03003ooooo03[oo`<00<Goo`006ooo00<00?oo
ool0?Ooo0`00`_oo000Kool20011ool4002nool001_oo`03003ooooo04Coo`@00;[oo`006ooo00<0
0?ooool0B?oo0`00]ooo000Kool00`00ooooo`1;ool3002dool001_oo`8004ooo`800;;oo`006ooo
00<00?ooool0D?oo0`00[ooo000Kool00`00ooooo`1Cool3002/ool001?oo`@000Coo`03003ooooo
05Koo`@00:Soo`005Ooo00<00?ooool00ooo00<00?ooool0F_oo1000Y?oo000Eool00`00ooooo`03
ool3001Nool4002Pool001Goo`03003ooooo00?oo`03003ooooo06;oo`@009coo`005Ooo00<00?oo
ool00ooo00<00?ooool0I_oo1000V?oo000Cool30005ool00`00ooooo`1Zool3002Eool001Goo`03
003ooooo00?oo`03003ooooo06goo`@0097oo`006ooo0P00L_oo1000SOoo000Kool00`00ooooo`1e
ool3002:ool001_oo`03003ooooo07Soo`@008Koo`006ooo00<00?ooool0O?oo1000P_oo000Kool0
0`00ooooo`20ool3001oool001_oo`8008Coo`@007_oo`006ooo00<00?ooool0Qooo1000Mooo000K
ool00`00ooooo`2;ool4001cool001_oo`03003ooooo08ooo`H006goo`006ooo00<00?ooool0UOoo
1000JOoo000Kool2002Jool4001Uool001_oo`03003ooooo09goo`@0067oo`006ooo00<00?ooool0
XOoo1000GOoo000Kool00`00ooooo`2Uool6001Gool001_oo`03003ooooo0:_oo`@005?oo`006ooo
0P00/?oo1000Cooo000Kool00`00ooooo`2cool4001;ool001_oo`03003ooooo0;Ooo`D004Koo`00
1ooo10000ooo00<00?ooool00_oo0`001Ooo00<00?ooool0_?oo1P00@?oo0009ool00`00ooooo`0:
ool00`00ooooo`02ool00`00ooooo`32ool5000kool000Woo`03003ooooo00[oo`03003ooooo00;o
o`<00<Ooo`D003Koo`002Ooo00<00?ooool01ooo0`001Ooo00<00?ooool0c?oo1@00<Ooo0009ool0
0`00ooooo`07ool00`00ooooo`05ool00`00ooooo`3Aool6000[ool000Ooo`<000Woo`03003ooooo
00Goo`03003ooooo0=Ooo`H002Goo`002Ooo00<00?ooool01ooo10001?oo00<00?ooool0gOoo1P00
7ooo000Kool2003Tool5000Jool001_oo`03003ooooo0>Soo`D001Goo`006ooo00<00?ooool0kOoo
1@004?oo000Kool00`00ooooo`3bool6000:ool001_oo`03003ooooo0?Soo`<000Ooo`006ooo0P00
oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0
oooo0ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.17393, -1.78342, \
0.0790907, 0.0203389}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(Look\ at\ almoz/mMag^2\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-alomz[1, 0.99, 1, a]\)/mMag[1, a], {a, \(-1\), 20}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 -1.83112 2.43615 [
[.29592 .59253 -3 -9 ]
[.29592 .59253 3 0 ]
[.52268 .59253 -6 -9 ]
[.52268 .59253 6 0 ]
[.74943 .59253 -6 -9 ]
[.74943 .59253 6 0 ]
[.97619 .59253 -6 -9 ]
[.97619 .59253 6 0 ]
[.05666 .1178 -18 -4.5 ]
[.05666 .1178 0 4.5 ]
[.05666 .23961 -24 -4.5 ]
[.05666 .23961 0 4.5 ]
[.05666 .36142 -18 -4.5 ]
[.05666 .36142 0 4.5 ]
[.05666 .48322 -24 -4.5 ]
[.05666 .48322 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .60503 m
.29592 .61128 L
s
[(5)] .29592 .59253 0 1 Mshowa
.52268 .60503 m
.52268 .61128 L
s
[(10)] .52268 .59253 0 1 Mshowa
.74943 .60503 m
.74943 .61128 L
s
[(15)] .74943 .59253 0 1 Mshowa
.97619 .60503 m
.97619 .61128 L
s
[(20)] .97619 .59253 0 1 Mshowa
.125 Mabswid
.11451 .60503 m
.11451 .60878 L
s
.15986 .60503 m
.15986 .60878 L
s
.20522 .60503 m
.20522 .60878 L
s
.25057 .60503 m
.25057 .60878 L
s
.34127 .60503 m
.34127 .60878 L
s
.38662 .60503 m
.38662 .60878 L
s
.43197 .60503 m
.43197 .60878 L
s
.47732 .60503 m
.47732 .60878 L
s
.56803 .60503 m
.56803 .60878 L
s
.61338 .60503 m
.61338 .60878 L
s
.65873 .60503 m
.65873 .60878 L
s
.70408 .60503 m
.70408 .60878 L
s
.79478 .60503 m
.79478 .60878 L
s
.84014 .60503 m
.84014 .60878 L
s
.88549 .60503 m
.88549 .60878 L
s
.93084 .60503 m
.93084 .60878 L
s
.02381 .60503 m
.02381 .60878 L
s
.25 Mabswid
0 .60503 m
1 .60503 L
s
.06916 .1178 m
.07541 .1178 L
s
[(0.8)] .05666 .1178 1 0 Mshowa
.06916 .23961 m
.07541 .23961 L
s
[(0.85)] .05666 .23961 1 0 Mshowa
.06916 .36142 m
.07541 .36142 L
s
[(0.9)] .05666 .36142 1 0 Mshowa
.06916 .48322 m
.07541 .48322 L
s
[(0.95)] .05666 .48322 1 0 Mshowa
.125 Mabswid
.06916 .02036 m
.07291 .02036 L
s
.06916 .04472 m
.07291 .04472 L
s
.06916 .06908 m
.07291 .06908 L
s
.06916 .09344 m
.07291 .09344 L
s
.06916 .14216 m
.07291 .14216 L
s
.06916 .16653 m
.07291 .16653 L
s
.06916 .19089 m
.07291 .19089 L
s
.06916 .21525 m
.07291 .21525 L
s
.06916 .26397 m
.07291 .26397 L
s
.06916 .28833 m
.07291 .28833 L
s
.06916 .31269 m
.07291 .31269 L
s
.06916 .33706 m
.07291 .33706 L
s
.06916 .38578 m
.07291 .38578 L
s
.06916 .41014 m
.07291 .41014 L
s
.06916 .4345 m
.07291 .4345 L
s
.06916 .45886 m
.07291 .45886 L
s
.06916 .50759 m
.07291 .50759 L
s
.06916 .53195 m
.07291 .53195 L
s
.06916 .55631 m
.07291 .55631 L
s
.06916 .58067 m
.07291 .58067 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .56222 L
.10458 .52771 L
.14415 .49691 L
.18221 .46836 L
.22272 .43895 L
.26171 .41151 L
.30316 .38319 L
.34309 .35669 L
.3815 .33187 L
.42237 .30617 L
.46172 .28207 L
.49955 .25947 L
.53984 .23599 L
.57861 .21395 L
.61984 .19108 L
.65954 .16959 L
.69774 .14939 L
.73838 .12838 L
.77751 .10862 L
.81909 .0881 L
.85916 .06878 L
.89771 .05059 L
.93871 .03165 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3
ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0nOoo0P001ooo000Kool00`00ooooo`3g
ool20009ool001_oo`800?Koo`8000_oo`006ooo00<00?ooool0looo0P003Ooo000Kool00`00oooo
o`3aool2000?ool001_oo`03003ooooo0>koo`<0017oo`006ooo00<00?ooool0k?oo0P005?oo000K
ool00`00ooooo`3Zool2000Fool001_oo`800>Woo`8001Soo`006ooo00<00?ooool0i_oo0P006_oo
000Kool00`00ooooo`3Sool3000Lool001_oo`03003ooooo0>7oo`8001ooo`006ooo00<00?ooool0
gooo0P008Ooo000Kool00`00ooooo`3Mool2000Sool001_oo`03003ooooo0=_oo`8002Goo`006ooo
0P00fOoo0`009ooo000Kool00`00ooooo`3Fool2000Zool001_oo`03003ooooo0=Coo`8002coo`00
6ooo00<00?ooool0d_oo0P00;_oo000Kool00`00ooooo`3@ool2000`ool001_oo`03003ooooo0<go
o`<003;oo`006ooo00<00?ooool0booo0P00=Ooo000Kool2003:ool2000gool001_oo`03003ooooo
0<Ooo`8003Woo`006ooo00<00?ooool0aOoo0P00>ooo000Kool00`00ooooo`33ool2000mool001_o
o`03003ooooo0<;oo`03003ooooo03goo`002?oo0P001?oo00<00?ooool00ooo0P001Ooo00<00?oo
ool0`?oo0P00@?oo0007ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo0;koo`8004;o
o`001ooo00@00?ooool000Soo`04003ooooo0004ool3002mool00`00ooooo`12ool000Ooo`04003o
oooo0009ool20005ool00`00ooooo`2kool20015ool000Ooo`04003ooooo0008ool01000ooooo`00
1?oo00<00?ooool0^Ooo0P00Aooo0007ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo
0;Ooo`8004Woo`002?oo0P002_oo0P001Ooo00<00?ooool0]Ooo0P00Booo000Kool00`00ooooo`2c
ool2001=ool001_oo`03003ooooo0;7oo`8004ooo`006ooo0P00/?oo0P00DOoo000Kool00`00oooo
o`2]ool2001Cool001_oo`03003ooooo0:_oo`8005Goo`006ooo00<00?ooool0ZOoo0P00Eooo000K
ool00`00ooooo`2Wool2001Iool001_oo`03003ooooo0:Goo`8005_oo`006ooo0P00Y?oo0P00GOoo
000Kool00`00ooooo`2Rool00`00ooooo`1Mool001_oo`03003ooooo0:3oo`80063oo`006ooo00<0
0?ooool0W_oo0P00H_oo000Kool00`00ooooo`2Lool2001Tool001_oo`03003ooooo09[oo`8006Ko
o`006ooo00<00?ooool0V?oo0P00J?oo000Kool2002Hool00`00ooooo`1Xool001_oo`03003ooooo
09Goo`8006_oo`006ooo00<00?ooool0Tooo0P00KOoo000Kool00`00ooooo`2Aool2001_ool001_o
o`03003ooooo093oo`03003ooooo06ooo`006ooo00<00?ooool0S_oo0P00L_oo000Kool00`00oooo
o`2<ool2001dool001_oo`8008coo`03003ooooo07Coo`006ooo00<00?ooool0ROoo0P00Mooo000K
ool00`00ooooo`27ool2001iool001_oo`03003ooooo08Goo`8007_oo`006ooo00<00?ooool0Pooo
0P00OOoo0002ool20004ool00`00ooooo`03ool20003ool30005ool00`00ooooo`21ool2001oool0
0005ool00?ooool00008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool0P?oo00<00?oo
ool0Oooo00001Ooo003ooooo00002?oo00@00?ooool000Goo`03003ooooo00;oo`<007koo`8008;o
o`0000Goo`00ooooo`0000Woo`8000?oo`<000Goo`03003ooooo07coo`8008Coo`0000Goo`00oooo
o`0000Soo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`1jool20026ool00005ool00?oo
ool00008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0N?oo0P00R?oo0002ool2000:
ool20003ool40004ool00`00ooooo`1fool2002:ool001_oo`03003ooooo07Goo`03003ooooo08[o
o`006ooo0P00M?oo0P00SOoo000Kool00`00ooooo`1bool00`00ooooo`2=ool001_oo`03003ooooo
073oo`80093oo`006ooo00<00?ooool0K_oo0P00T_oo000Kool00`00ooooo`1]ool00`00ooooo`2B
ool001_oo`03003ooooo06_oo`8009Goo`006ooo00<00?ooool0J_oo00<00?ooool0UOoo000Kool2
001Yool2002Hool001_oo`03003ooooo06Koo`8009[oo`006ooo00<00?ooool0I?oo0P00W?oo000K
ool00`00ooooo`1Rool2002Nool001_oo`03003ooooo063oo`800:3oo`006ooo00<00?ooool0Gooo
00<00?ooool0X?oo000Kool00`00ooooo`1Mool2002Sool001_oo`8005goo`03003ooooo0:?oo`00
6ooo00<00?ooool0F_oo0P00Y_oo000Kool00`00ooooo`1Hool2002Xool001_oo`03003ooooo05Oo
o`03003ooooo0:Soo`006ooo00<00?ooool0EOoo0P00Zooo000Kool00`00ooooo`1Dool00`00oooo
o`2[ool001_oo`03003ooooo05;oo`800:koo`006ooo0P00D_oo00<00?ooool0[_oo000Kool00`00
ooooo`1?ool2002aool001_oo`03003ooooo04goo`800;?oo`006ooo00<00?ooool0C?oo00<00?oo
ool0/ooo000Kool00`00ooooo`1:ool2002fool000Soo`8000Coo`03003ooooo00;oo`<000Goo`03
003ooooo04Woo`03003ooooo0;Koo`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo
04Ooo`800;Woo`001ooo00@00?ooool000_oo`03003ooooo00;oo`<004Koo`03003ooooo0;Woo`00
1ooo00@00?ooool000Woo`<000Coo`03003ooooo04Goo`03003ooooo0;[oo`001ooo00@00?ooool0
00Soo`04003ooooo0004ool00`00ooooo`13ool2002mool000Ooo`04003ooooo0008ool01000oooo
o`001?oo00<00?ooool0@_oo00<00?ooool0_Ooo0008ool2000:ool20005ool00`00ooooo`11ool0
0`00ooooo`2nool001_oo`03003ooooo03ooo`800<7oo`006ooo0P00?ooo00<00?ooool0`Ooo000K
ool00`00ooooo`0lool20034ool001_oo`03003ooooo03_oo`03003ooooo0<Coo`006ooo00<00?oo
ool0>_oo00<00?ooool0aOoo000Kool00`00ooooo`0hool20038ool001_oo`03003ooooo03Ooo`03
003ooooo0<Soo`006ooo00<00?ooool0=_oo00<00?ooool0bOoo000Kool2000eool2003<ool001_o
o`03003ooooo03?oo`03003ooooo0<coo`006ooo00<00?ooool0<Ooo0P00cooo000Kool00`00oooo
o`0`ool00`00ooooo`3?ool001_oo`03003ooooo02koo`800=;oo`006ooo00<00?ooool0;?oo0P00
e?oo000Kool00`00ooooo`0[ool00`00ooooo`3Dool001_oo`8002[oo`800=Ooo`006ooo00<00?oo
ool0:?oo00<00?ooool0eooo000Kool00`00ooooo`0Vool2003Jool001_oo`03003ooooo02Goo`03
003ooooo0=[oo`006ooo00<00?ooool09?oo00<00?ooool0fooo000Kool00`00ooooo`0Rool2003N
ool001_oo`03003ooooo027oo`03003ooooo0=koo`006ooo0P008Ooo00<00?ooool0gooo000Kool0
0`00ooooo`0Nool2003Rool001_oo`03003ooooo01goo`03003ooooo0>;oo`006ooo00<00?ooool0
6ooo0P00iOoo000Kool00`00ooooo`0Jool00`00ooooo`3Uool000;oo`8000Coo`03003ooooo00;o
o`<000?oo`<000Goo`03003ooooo01Woo`03003ooooo0>Koo`0000Goo`00ooooo`0000_oo`03003o
oooo00?oo`03003ooooo00;oo`03003ooooo01Ooo`800>Woo`0000Goo`00ooooo`0000_oo`03003o
oooo00?oo`03003ooooo00;oo`<001Koo`03003ooooo0>Woo`0000Goo`00ooooo`0000Woo`<000;o
o`<000Goo`03003ooooo01Goo`03003ooooo0>[oo`0000Goo`00ooooo`0000Soo`04003ooooo0002
ool00`00ooooo`05ool00`00ooooo`0Cool2003]ool00005ool00?ooool00008ool01000ooooo`00
0_oo00<00?ooool01Ooo00<00?ooool04_oo00<00?ooool0kOoo0002ool2000:ool20003ool40004
ool00`00ooooo`0Aool00`00ooooo`3^ool001_oo`03003ooooo00ooo`800?7oo`006ooo0P003ooo
00<00?ooool0lOoo000Kool00`00ooooo`0=ool00`00ooooo`3bool001_oo`03003ooooo00coo`03
003ooooo0??oo`006ooo00<00?ooool02ooo00<00?ooool0m?oo000Kool00`00ooooo`09ool2003g
ool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool01ooo00<00?ooool0n?oo
000Kool20007ool00`00ooooo`3iool001_oo`03003ooooo00Coo`800?coo`006ooo00<00?ooool0
0ooo00<00?ooool0o?oo000Kool00`00ooooo`02ool00`00ooooo`3mool001_oo`05003oooooool0
003oool1ool001_oo`04003ooooo003oool2ool001_oo`<00?ooo`?oo`006ooo0P00oooo1?oo000J
ool2003oool5ool001Woo`03003oo`000?ooo`Goo`006?oo00@00?ooool003coo`<003Woo`@000?o
o`8003Ooo`@000;oo`<003Koo`@000?oo`8000Coo`005ooo00D00?ooooooo`0003ooo`03003ooooo
03Soo`05003oooooool00002ool00`00ooooo`0fool00`00ooooo`04ool00`00ooooo`0cool00`00
ooooo`03ool01000ooooo`000ooo000Fool00`00ooooo`02ool00`00ooooo`0mool00`00ooooo`0h
ool01@00oooooooo00000_oo00<00?ooool0=_oo00<00?ooool01?oo00<00?ooool0=?oo00<00?oo
ool00_oo00@00?ooool000?oo`005Ooo00<00?ooool00ooo00<00?ooool0>_oo0`00>ooo00D00?oo
ooooo`0000;oo`03003ooooo03Koo`04003oooooool3000hool01@00oooooooo00000_oo00<00?oo
ool00Ooo000Dool00`00ooooo`04ool2000kool00`00ooooo`0kool01@00oooooooo00000_oo00<0
0?ooool0=_oo00D00?ooooooo`0003_oo`04003ooooo0002ool00`00ooooo`01ool001?oo`03003o
oooo00Goo`03003ooooo03[oo`03003ooooo03Woo`<000?oo`04003ooooo000fool30003ool00`00
ooooo`0fool01000ooooo`000_oo00@00?ooool000?oo`004_oo00<00?ooool01_oo00<00?ooool0
>_oo1000>_oo00<00?ooool00_oo0P00>Ooo00@00?ooooooo`@003Koo`8000Coo`8000Coo`004Ooo
00<00?ooool01ooo00<00?ooool0oooo0ooo000@ool00`00ooooo`08ool00`00ooooo`3oool3ool0
00ooo`03003ooooo00Woo`03003ooooo0?ooo`?oo`003_oo00<00?ooool02_oo00<00?ooool0oooo
0ooo0008oooo000I000000koo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00[oo`03003o
oooo00Woo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Goo`006ooo00<00?ooool0oooo
0ooo000Kool00`00ooooo`3oool3ool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-2.17393, 0.748233, \
0.0790907, 0.00147236}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(klim[th_, a_] := 
      k /. FindRoot[
          k^2 + Sqrt[1 + a] 
                alomz[1, IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\)*Cos[th], 
                  IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), a] \[Equal] 
            0, {k, 0.1, 0.5}]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(0.35755245591872487`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(0.3006645987164847`\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/8, 1]\)], "Input"],

Cell[BoxData[
    \(0.5204693420090241`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omspeca[IntegerPart[T*10^\((16)\)]*10^\((\(-16\))\), Pi/4, 1]}, {T, 
        0.005, 0.3, 0.005}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.`\\ \[ImaginaryI]\)\))\)\) + \
\(\(0.003535533905932738`\\ xi\)\))\)\)\\ \(\((\(\[LeftSkeleton] 1 \
\[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.7071067807264626`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.3006645987164847`\\ \[ImaginaryI]\)\))\)\) + \(\
\(\(\(\[LeftSkeleton] 21 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.06307684728935363`\\ \[ImaginaryI]\)\))\)\) + \
\(\(\(\(\[LeftSkeleton] 21 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.7071067807264626`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.7071067807264626`\)."\
\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.005`, 0.0006257531059251438`}, {0.01`, 
        0.001250243309001515`}, {0.015`, 0.001872220638184215`}, {0.02`, 
        0.002490403452588587`}, {0.025`, 0.00310356259704492`}, {0.03`, 
        0.0037104477922098446`}, {0.035`, 0.004309822404612067`}, {0.04`, 
        0.004900460387553068`}, {0.045`, 0.0054811474738197`}, {0.05`, 
        0.006050682618679738`}, {0.055`, 0.006607879429526019`}, {0.06`, 
        0.007151567536593173`}, {0.065`, 0.0076805986093293165`}, {0.07`, 
        0.008193826627916862`}, {0.075`, 0.008690144550630422`}, {0.08`, 
        0.0091684578585202`}, {0.085`, 0.009627693966981125`}, {0.09`, 
        0.010066802992084643`}, {0.095`, 0.010484758485042309`}, {0.1`, 
        0.010880558102556098`}, {0.105`, 0.011253224199142136`}, {0.11`, 
        0.011601804335682524`}, {0.115`, 0.011925371702406916`}, {0.12`, 
        0.012223025456599144`}, {0.125`, 0.012493890976555148`}, {0.13`, 
        0.012737120034118744`}, {0.135`, 0.012951890888686701`}, {0.14`, 
        0.013137408305996716`}, {0.145`, 0.013292924844556148`}, {0.15`, 
        0.013417649363418315`}, {0.155`, 0.0135108945257446`}, {0.16`, 
        0.013571969529413863`}, {0.165`, 0.013600209279024826`}, {0.17`, 
        0.01359497401259337`}, {0.17500000000000002`, 
        0.01355564888757589`}, {0.18`, 0.013481643529073889`}, {0.185`, 
        0.013372391543401259`}, {0.19`, 0.013227350000380857`}, {0.195`, 
        0.013045998887820442`}, {0.2`, 
        0.012827840541625238`}, {0.20500000000000002`, 
        0.012572399054955755`}, {0.21`, 0.012279219669749254`}, {0.215`, 
        0.011947868153801486`}, {0.22`, 0.011577930166460816`}, {0.225`, 
        0.011169010615826522`}, {0.23`, 
        0.010720733010170666`}, {0.23500000000000001`, 
        0.010232738806124268`}, {0.24`, 0.009704686755985636`}, {0.245`, 
        0.009136252256325115`}, {0.25`, 0.008527126699878183`}, {0.255`, 
        0.007877016832539368`}, {0.26`, 0.007185644117095007`}, {0.265`, 
        0.006452744105163739`}, {0.27`, 0.005678065818651241`}, {0.275`, 
        0.0048613711418711665`}, {0.28`, 
        0.0040024342253367875`}, {0.28500000000000003`, 
        0.0031010408986327422`}, {0.29`, 0.002156988116954923`}, {0.295`, 
        0.0011700833717408234`}, {0.3`, 0.00014014424093272994`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphaspecial[1, 10^\((\(-6\))\), 10^\((\(-6\))\)*Cos[Pi/4], 
      I*10^\((\(-7\))\), 1]\)], "Input"],

Cell[BoxData[
    \(\(-0.01574945674924121783133474009899539502`20.2929\) + 
      0``34\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(alphaspecial[1, k, k*Cos[th], I*gam, a]\)\(]\)\) + k^2 + 
      gam^2\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<instamode.dat\>", %217]\)], "Input"],

Cell[BoxData[
    \("instamode.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%217, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 3.1746 0.0147151 43.279 [
[.18254 .00222 -12 -9 ]
[.18254 .00222 12 0 ]
[.34127 .00222 -9 -9 ]
[.34127 .00222 9 0 ]
[.5 .00222 -12 -9 ]
[.5 .00222 12 0 ]
[.65873 .00222 -9 -9 ]
[.65873 .00222 9 0 ]
[.81746 .00222 -12 -9 ]
[.81746 .00222 12 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .10127 -30 -4.5 ]
[.01131 .10127 0 4.5 ]
[.01131 .18783 -30 -4.5 ]
[.01131 .18783 0 4.5 ]
[.01131 .27439 -30 -4.5 ]
[.01131 .27439 0 4.5 ]
[.01131 .36095 -30 -4.5 ]
[.01131 .36095 0 4.5 ]
[.01131 .44751 -24 -4.5 ]
[.01131 .44751 0 4.5 ]
[.01131 .53406 -30 -4.5 ]
[.01131 .53406 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.05)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(0.1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(0.15)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(0.2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(0.25)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(0.3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .10127 m
.03006 .10127 L
s
[(0.002)] .01131 .10127 1 0 Mshowa
.02381 .18783 m
.03006 .18783 L
s
[(0.004)] .01131 .18783 1 0 Mshowa
.02381 .27439 m
.03006 .27439 L
s
[(0.006)] .01131 .27439 1 0 Mshowa
.02381 .36095 m
.03006 .36095 L
s
[(0.008)] .01131 .36095 1 0 Mshowa
.02381 .44751 m
.03006 .44751 L
s
[(0.01)] .01131 .44751 1 0 Mshowa
.02381 .53406 m
.03006 .53406 L
s
[(0.012)] .01131 .53406 1 0 Mshowa
.125 Mabswid
.02381 .03635 m
.02756 .03635 L
s
.02381 .05799 m
.02756 .05799 L
s
.02381 .07963 m
.02756 .07963 L
s
.02381 .12291 m
.02756 .12291 L
s
.02381 .14455 m
.02756 .14455 L
s
.02381 .16619 m
.02756 .16619 L
s
.02381 .20947 m
.02756 .20947 L
s
.02381 .23111 m
.02756 .23111 L
s
.02381 .25275 m
.02756 .25275 L
s
.02381 .29603 m
.02756 .29603 L
s
.02381 .31767 m
.02756 .31767 L
s
.02381 .33931 m
.02756 .33931 L
s
.02381 .38259 m
.02756 .38259 L
s
.02381 .40423 m
.02756 .40423 L
s
.02381 .42587 m
.02756 .42587 L
s
.02381 .46914 m
.02756 .46914 L
s
.02381 .49078 m
.02756 .49078 L
s
.02381 .51242 m
.02756 .51242 L
s
.02381 .5557 m
.02756 .5557 L
s
.02381 .57734 m
.02756 .57734 L
s
.02381 .59898 m
.02756 .59898 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03968 .0418 m
.05556 .06882 L
.07143 .09574 L
.0873 .1225 L
.10317 .14903 L
.11905 .1753 L
.13492 .20124 L
.15079 .2268 L
.16667 .25193 L
.18254 .27658 L
.19841 .3007 L
.21429 .32423 L
.23016 .34712 L
.24603 .36934 L
.2619 .39082 L
.27778 .41152 L
.29365 .43139 L
.30952 .4504 L
.3254 .46849 L
.34127 .48562 L
.35714 .50174 L
.37302 .51683 L
.38889 .53083 L
.40476 .54372 L
.42063 .55544 L
.43651 .56597 L
.45238 .57526 L
.46825 .58329 L
.48413 .59002 L
.5 .59542 L
.51587 .59945 L
.53175 .6021 L
.54762 .60332 L
.56349 .60309 L
.57937 .60139 L
.59524 .59819 L
.61111 .59346 L
.62698 .58718 L
.64286 .57933 L
.65873 .56989 L
.6746 .55884 L
.69048 .54615 L
.70635 .53181 L
.72222 .5158 L
.7381 .4981 L
.75397 .4787 L
.76984 .45758 L
.78571 .43472 L
.80159 .41012 L
.81746 .38376 L
Mistroke
.83333 .35562 L
.84921 .3257 L
.86508 .29398 L
.88095 .26046 L
.89683 .22511 L
.9127 .18794 L
.92857 .14893 L
.94444 .10807 L
.96032 .06536 L
.97619 .02078 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00@03oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo0`00000G0?oo
o`800000103oool00`000000oooo0?ooo`020?ooo`@000005`3oool2000000@0oooo00<000000?oo
o`3oool00P3oool400000080oooo0`00000H0?ooo`800000103oool00`000000oooo0?ooo`020?oo
o`@000005`3oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`00000H0?oo
o`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool003l0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo01<0oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool05`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo000o0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
1@3oool00`000000oooo0?ooo`0C0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool05@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0503oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo01H0oooo00@00000
0?ooo`3oool000002@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo00@0
00000?ooo`3oool000002`3oool100000040oooo0@3oool003l0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000020?ooo`<000005P3oool010000000oooo0?ooo`00000:0?oo
o`030000003oool0oooo01D0oooo00@000000?ooo`3oool000002P3oool010000000oooo0?ooo`3o
ool3000001L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0E0?ooo`040000
003oool0oooo000000X0oooo00@000000?ooo`3oool0oooo0`00000G0?ooo`040000003oool0oooo
000000T0oooo0P0000030?ooo`00?`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo01D0oooo00@000000?ooo`3oool000002P3oool01@000000oooo0?ooo`3oool00000
01T0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0D0?ooo`040000003oool0
oooo000000/0oooo00@000000?ooo`3oool000006@3oool010000000oooo0?ooo`00000;0?ooo`40
00000@3oool10?ooo`00?`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`0000080?ooo`<000005`3oool0
10000000oooo0?ooo`0000080?ooo`<000000`3oool00`000000oooo0?ooo`0G0?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool000005P3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool05`3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo00100?ooo`8000002P3oool2000000<0oooo1000
000F0?ooo`8000002`3oool00`000000oooo0?ooo`0F0?ooo`8000002`3oool010000000oooo0?oo
o`3oool4000001L0oooo0P00000:0?ooo`800000603oool2000000X0oooo0P0000030?ooo`@00000
5`3oool2000000X0oooo0P0000030?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool0
0`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000
oooo0?ooo`3l0?ooo`006`3ooooo000000<000000`3oool00240oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3o
ool00240oooo00<000000?ooo`3oool0lP3oool00`000000oooo0?ooo`070?ooo`008@3oool00`00
0000oooo0?ooo`3a0?ooo`030000003oool0oooo00P0oooo000Q0?ooo`030000003oool0oooo0?40
oooo00<000000?ooo`3oool0203oool00240oooo0P00003a0?ooo`030000003oool0oooo00T0oooo
000Q0?ooo`030000003oool0oooo0?00oooo00<000000?ooo`3oool02@3oool00240oooo00D00000
0?ooo`3oool0oooo0000003^0?ooo`030000003oool0oooo00T0oooo000Q0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0jP3oool00`000000oooo0?ooo`0:0?ooo`008@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0>X0oooo00<000000?ooo`3oool02P3oool00240
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3X0?ooo`030000003oool0oooo00/0
oooo000Q0?ooo`800000103oool00`000000oooo0?ooo`3X0?ooo`030000003oool0oooo00/0oooo
000Q0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0iP3oool00`000000oooo0?oo
o`0<0?ooo`008@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0>H0oooo00<00000
0?ooo`3oool0303oool00240oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3U0?oo
o`030000003oool0oooo00`0oooo000Q0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool0h`3oool00`000000oooo0?ooo`0=0?ooo`008@3oool2000000L0oooo00<000000?ooo`3oool0
h`3oool00`000000oooo0?ooo`0=0?ooo`008@3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo0>80oooo00<000000?ooo`3oool03@3oool00240oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo000Q0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0h03oool00`000000oooo0?ooo`0>0?ooo`000P3oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P0000030?ooo`@00000103oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0=h0oooo00<000000?ooo`3oool03`3oool000050?oo
o`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo0=d0oooo00<000000?ooo`3oool03`3oool000050?ooo`000000oooo0?ooo`000000203oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000<0oooo00<000000?ooo`3oool0
103oool2000000/0oooo00<000000?ooo`3oool0g@3oool00`000000oooo0?ooo`0?0?ooo`0000D0
oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3o
ool00000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0f`3oool00`000000oooo0?ooo`0@0?ooo`0000D0oooo0000003oool0oooo000000080?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0f`3oool00`000000oooo0?oo
o`0@0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool0fP3oool00`000000oooo0?ooo`0@0?ooo`000P3oool2000000X0
oooo0P0000040?ooo`800000103oool2000000D0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`3I0?ooo`030000003oool0oooo0140oooo000Q0?ooo`8000003P3oool00`000000oooo
0?ooo`3H0?ooo`030000003oool0oooo0140oooo000Q0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0eP3oool00`000000oooo0?ooo`0B0?ooo`008@3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo0=H0oooo00<000000?ooo`3oool04P3oool00240oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`3E0?ooo`030000003oool0oooo0180oooo000Q0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool0e03oool00`000000oooo0?ooo`0C0?ooo`00
8@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0=<0oooo00<000000?ooo`3oool0
4`3oool00240oooo0P00000A0?ooo`030000003oool0oooo0=80oooo00<000000?ooo`3oool0503o
ool00240oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3A0?ooo`030000003oool0
oooo01@0oooo000Q0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0d03oool00`00
0000oooo0?ooo`0D0?ooo`008@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0<l0
oooo00<000000?ooo`3oool05@3oool00240oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`3>0?ooo`030000003oool0oooo01D0oooo000Q0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0c@3oool00`000000oooo0?ooo`0F0?ooo`008@3oool2000001D0oooo00<00000
0?ooo`3oool0c03oool00`000000oooo0?ooo`0F0?ooo`008@3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo0<`0oooo00<000000?ooo`3oool05P3oool00240oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`3:0?ooo`030000003oool0oooo01L0oooo00020?ooo`800000
103oool00`000000oooo0?ooo`030?ooo`800000103oool2000000D0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0<T0oooo00<000000?ooo`3oool0
5`3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo0<P0oooo00<000000?ooo`3oool0603oool000050?ooo`000000oooo0?oo
o`000000203oool010000000oooo0?ooo`0000020?ooo`050000003oool0oooo0000003oool01@00
00040?ooo`800000603oool00`000000oooo0?ooo`370?ooo`030000003oool0oooo01P0oooo0000
1@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool01P000000oooo
0?ooo`000000oooo00000080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0603oool000050?ooo`000000oooo0?oo
o`000000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<0
00000?ooo`0000001@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0<D0oooo00<0
00000?ooo`3oool06@3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?oo
o`0000020?ooo`040000003oool0oooo000000<0oooo0P0000050?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0a@3oool00`000000oooo0?ooo`0I0?ooo`000P3oool2000000X0oooo
0P0000040?ooo`8000001@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool0``3oool00`000000oooo0?ooo`0J0?ooo`008@3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo0<80oooo00<000000?ooo`3oool06P3oool00240oooo0P00000K
0?ooo`030000003oool0oooo0<80oooo00<000000?ooo`3oool06P3oool00240oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`300?ooo`030000003oool0oooo01/0oooo000Q0?ooo`03
0000003oool0oooo01`0oooo00<000000?ooo`3oool0_`3oool00`000000oooo0?ooo`0K0?ooo`00
8@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
703oool00240oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`2m0?ooo`030000003o
ool0oooo01`0oooo000Q0?ooo`8000007`3oool00`000000oooo0?ooo`2l0?ooo`030000003oool0
oooo01`0oooo000Q0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0^`3oool00`00
0000oooo0?ooo`0M0?ooo`008@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0;X0
oooo00<000000?ooo`3oool07@3oool00240oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`2i0?ooo`030000003oool0oooo01h0oooo000Q0?ooo`030000003oool0oooo0200oooo00<0
00000?ooo`3oool0^03oool00`000000oooo0?ooo`0N0?ooo`008@3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo0;L0oooo00<000000?ooo`3oool07`3oool00240oooo0P00000R0?oo
o`030000003oool0oooo0;H0oooo00<000000?ooo`3oool07`3oool00240oooo00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`2d0?ooo`030000003oool0oooo0200oooo000Q0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool0/`3oool00`000000oooo0?ooo`0P0?ooo`000P3o
ool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P0000040?ooo`8000001@3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool0803o
ool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`040000
003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo0;40oooo00<000000?ooo`3oool08@3oool000050?ooo`000000oooo0?oo
o`000000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool00000103oool2000002H0oooo00<000000?ooo`3oool0/03oool00`000000oooo
0?ooo`0Q0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080
oooo00@000000?ooo`3oool000000P3oool3000000D0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`2^0?ooo`030000003oool0oooo0280oooo00001@3oool000000?ooo`3oool00000
00P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`2]0?ooo`030000003o
ool0oooo0280oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`2/0?ooo`030000003oool0oooo02<0oooo00020?ooo`800000
2P3oool2000000@0oooo0P0000040?ooo`<00000103oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo0:/0oooo00<000000?ooo`3oool08`3oool00240oooo00<000000?ooo`3oool0:@3o
ool00`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo02@0oooo000Q0?ooo`800000:P3oool0
0`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo02@0oooo000Q0?ooo`030000003oool0oooo
02X0oooo00<000000?ooo`3oool0Y`3oool00`000000oooo0?ooo`0U0?ooo`008@3oool00`000000
oooo0?ooo`0[0?ooo`030000003oool0oooo0:H0oooo00<000000?ooo`3oool09@3oool00240oooo
00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`2U0?ooo`030000003oool0oooo02H0oooo
000Q0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0Y03oool00`000000oooo0?oo
o`0V0?ooo`008@3oool2000002h0oooo00<000000?ooo`3oool0XP3oool00`000000oooo0?ooo`0W
0?ooo`008@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0:80oooo00<000000?oo
o`3oool09`3oool00240oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`2P0?ooo`03
0000003oool0oooo02P0oooo000Q0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
W`3oool00`000000oooo0?ooo`0X0?ooo`008@3oool00`000000oooo0?ooo`0_0?ooo`030000003o
ool0oooo09h0oooo00<000000?ooo`3oool0:@3oool00240oooo00<000000?ooo`3oool0<03oool0
0`000000oooo0?ooo`2M0?ooo`030000003oool0oooo02T0oooo000Q0?ooo`800000<P3oool00`00
0000oooo0?ooo`2K0?ooo`030000003oool0oooo02X0oooo000Q0?ooo`030000003oool0oooo0340
oooo00<000000?ooo`3oool0V`3oool00`000000oooo0?ooo`0Z0?ooo`008@3oool00`000000oooo
0?ooo`0b0?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3oool0:`3oool00240oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo02/0oooo0002
0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000103oool2000000@0oooo0P000005
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`0/
0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo03@0
oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`0]0?ooo`0000D0oooo0000003oool0
oooo000000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000040?ooo`800000=P3oool00`000000oooo0?ooo`2D0?ooo`030000003o
ool0oooo02d0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000030?ooo`8000001@3oool00`000000oooo0?ooo`0e0?ooo`03
0000003oool0oooo09<0oooo00<000000?ooo`3oool0;P3oool000050?ooo`000000oooo0?ooo`00
0000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0980oooo00<0
00000?ooo`3oool0;P3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?oo
o`0000020?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`0g0?ooo`030000003oool0oooo0900oooo00<000000?ooo`3oool0;`3oool00080
oooo0P00000:0?ooo`800000103oool2000000@0oooo0P0000050?ooo`030000003oool0oooo03P0
oooo00<000000?ooo`3oool0S`3oool00`000000oooo0?ooo`0_0?ooo`008@3oool2000003T0oooo
00<000000?ooo`3oool0SP3oool00`000000oooo0?ooo`0`0?ooo`008@3oool00`000000oooo0?oo
o`0i0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool0<@3oool00240oooo00<00000
0?ooo`3oool0>P3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0340oooo000Q0?oo
o`030000003oool0oooo03/0oooo00<000000?ooo`3oool0R@3oool00`000000oooo0?ooo`0b0?oo
o`008@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo08T0oooo00<000000?ooo`3o
ool0<P3oool00240oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`270?ooo`030000
003oool0oooo03<0oooo000Q0?ooo`800000?P3oool00`000000oooo0?ooo`260?ooo`030000003o
ool0oooo03<0oooo000Q0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0Q@3oool0
0`000000oooo0?ooo`0d0?ooo`008@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo
08<0oooo00<000000?ooo`3oool0=@3oool00240oooo00<000000?ooo`3oool0?`3oool00`000000
oooo0?ooo`220?ooo`030000003oool0oooo03D0oooo000Q0?ooo`030000003oool0oooo0400oooo
00<000000?ooo`3oool0P03oool00`000000oooo0?ooo`0f0?ooo`008@3oool200000440oooo00<0
00000?ooo`3oool0P03oool00`000000oooo0?ooo`0f0?ooo`008@3oool00`000000oooo0?ooo`11
0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3oool0=`3oool00240oooo00<000000?oo
o`3oool0@P3oool00`000000oooo0?ooo`1m0?ooo`030000003oool0oooo03L0oooo000Q0?ooo`03
0000003oool0oooo04<0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`0h0?ooo`00
203oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo100000040?ooo`030000
003oool0oooo04@0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?ooo`0i0?ooo`001`3o
ool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo07P0oooo00<000000?ooo`3o
ool0>P3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo00<0oooo0P0000160?ooo`030000003oool0oooo07H0oooo00<000000?oo
o`3oool0>`3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`1d
0?ooo`030000003oool0oooo03`0oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo04L0oooo00<0
00000?ooo`3oool0LP3oool00`000000oooo0?ooo`0m0?ooo`001`3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo0`0000050?ooo`030000003oool0oooo04P0oooo
00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`0n0?ooo`00203oool2000000X0oooo0P00
00050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?oo
o`1_0?ooo`030000003oool0oooo03h0oooo000Q0?ooo`800000B`3oool00`000000oooo0?ooo`1]
0?ooo`030000003oool0oooo03l0oooo000Q0?ooo`030000003oool0oooo04/0oooo00<000000?oo
o`3oool0J`3oool00`000000oooo0?ooo`100?ooo`008@3oool00`000000oooo0?ooo`1<0?ooo`03
0000003oool0oooo06X0oooo00<000000?ooo`3oool0@03oool00240oooo00<000000?ooo`3oool0
C03oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo0440oooo000Q0?ooo`030000003o
ool0oooo04d0oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`120?ooo`008@3oool0
0`000000oooo0?ooo`1>0?ooo`030000003oool0oooo06D0oooo00<000000?ooo`3oool0@`3oool0
0240oooo0P00001@0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool0@`3oool00240
oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo04@0
oooo000Q0?ooo`030000003oool0oooo0540oooo00<000000?ooo`3oool0H03oool00`000000oooo
0?ooo`150?ooo`008@3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo05h0oooo00<0
00000?ooo`3oool0AP3oool00240oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`1L
0?ooo`030000003oool0oooo04L0oooo000Q0?ooo`030000003oool0oooo05@0oooo00<000000?oo
o`3oool0FP3oool00`000000oooo0?ooo`180?ooo`008@3oool2000005H0oooo00<000000?ooo`3o
ool0F03oool00`000000oooo0?ooo`190?ooo`008@3oool00`000000oooo0?ooo`1F0?ooo`030000
003oool0oooo05H0oooo00<000000?ooo`3oool0BP3oool00240oooo00<000000?ooo`3oool0E`3o
ool2000005D0oooo00<000000?ooo`3oool0B`3oool00080oooo0P0000040?ooo`030000003oool0
oooo00<0oooo0P0000030?ooo`@000000P3oool4000000@0oooo00<000000?ooo`3oool0F@3oool2
000005<0oooo00<000000?ooo`3oool0B`3oool000050?ooo`000000oooo0?ooo`000000203oool0
10000000oooo0?ooo`0000040?ooo`050000003oool0oooo0?ooo`0000001`3oool00`000000oooo
0?ooo`1K0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0C03oool000050?ooo`00
0000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0103oool2000005d0oooo00<000000?ooo`3oool0C@3oool00`000000
oooo0?ooo`1=0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo0000
00@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
05d0oooo00<000000?ooo`3oool0BP3oool200000500oooo00001@3oool000000?ooo`3oool00000
00P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`180?ooo`030000003oool0
oooo0500oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3o
ool3000000<0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`1O0?ooo`030000
003oool0oooo04H0oooo00<000000?ooo`3oool0D@3oool00080oooo0P00000:0?ooo`8000001@3o
ool00`000000oooo0?ooo`020?ooo`8000001@3oool00`000000oooo0?ooo`1P0?ooo`030000003o
ool0oooo04<0oooo0P00001D0?ooo`008@3oool00`000000oooo0?ooo`1Q0?ooo`800000@P3oool0
0`000000oooo0?ooo`1D0?ooo`008@3oool2000006@0oooo00<000000?ooo`3oool0?P3oool00`00
0000oooo0?ooo`1E0?ooo`008@3oool00`000000oooo0?ooo`1T0?ooo`800000?03oool2000005P0
oooo000Q0?ooo`030000003oool0oooo06H0oooo0P00000i0?ooo`030000003oool0oooo05P0oooo
000Q0?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?oo
o`1I0?ooo`008@3oool00`000000oooo0?ooo`1Y0?ooo`800000<`3oool2000005`0oooo000Q0?oo
o`800000K03oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo05`0oooo000Q0?ooo`03
0000003oool0oooo06`0oooo0P00000]0?ooo`800000G`3oool00240oooo00<000000?ooo`3oool0
KP3oool3000002P0oooo0P00001Q0?ooo`008@3oool00`000000oooo0?ooo`1a0?ooo`<000008P3o
ool3000006<0oooo000Q0?ooo`030000003oool0oooo07@0oooo0P00000M0?ooo`<00000IP3oool0
0240oooo00<000000?ooo`3oool0MP3oool3000001P0oooo0P00001Y0?ooo`008@3oool2000007X0
oooo1000000=0?ooo`L00000J`3oool00240oooo00<000000?ooo`3oool0O@3oool=00000780oooo
000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?oo
o`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo003o0?ooob40oooo
003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0407059, -0.00153197, \
0.00122557, 8.98977*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[omspeca[k, Pi/4, 1], {k, 1/100, klim[Pi/4, 1]}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.`\\ \[ImaginaryI]\)\))\)\) + \
\(\(0.0070710763756533426`\\ xi\)\))\)\)\\ \(\((\(\[LeftSkeleton] 1 \
\[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.7071067807264626`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.3006645987164847`\\ \[ImaginaryI]\)\))\)\) + \(\
\(\(\(\[LeftSkeleton] 22 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(\(\((\(\((\(\(0.` \
\[InvisibleSpace]\)\) + \(\(0.06303205014768996`\\ \[ImaginaryI]\)\))\)\) + \
\(\(\(\(\[LeftSkeleton] 22 \[RightSkeleton]\)\)\\ xi\)\))\)\)\\ \(\((\(\
\[LeftSkeleton] 1 \[RightSkeleton]\))\)\)\)\/\((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.7071067807264626`\)."\
\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(-0.7071067807264626`\)."\
\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ncvb\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 3.16759 0.0147153 43.2726 [
[.18219 .00222 -12 -9 ]
[.18219 .00222 12 0 ]
[.34057 .00222 -9 -9 ]
[.34057 .00222 9 0 ]
[.49895 .00222 -12 -9 ]
[.49895 .00222 12 0 ]
[.65733 .00222 -9 -9 ]
[.65733 .00222 9 0 ]
[.81571 .00222 -12 -9 ]
[.81571 .00222 12 0 ]
[.97409 .00222 -9 -9 ]
[.97409 .00222 9 0 ]
[.01131 .10126 -30 -4.5 ]
[.01131 .10126 0 4.5 ]
[.01131 .18781 -30 -4.5 ]
[.01131 .18781 0 4.5 ]
[.01131 .27435 -30 -4.5 ]
[.01131 .27435 0 4.5 ]
[.01131 .3609 -30 -4.5 ]
[.01131 .3609 0 4.5 ]
[.01131 .44744 -24 -4.5 ]
[.01131 .44744 0 4.5 ]
[.01131 .53399 -30 -4.5 ]
[.01131 .53399 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18219 .01472 m
.18219 .02097 L
s
[(0.05)] .18219 .00222 0 1 Mshowa
.34057 .01472 m
.34057 .02097 L
s
[(0.1)] .34057 .00222 0 1 Mshowa
.49895 .01472 m
.49895 .02097 L
s
[(0.15)] .49895 .00222 0 1 Mshowa
.65733 .01472 m
.65733 .02097 L
s
[(0.2)] .65733 .00222 0 1 Mshowa
.81571 .01472 m
.81571 .02097 L
s
[(0.25)] .81571 .00222 0 1 Mshowa
.97409 .01472 m
.97409 .02097 L
s
[(0.3)] .97409 .00222 0 1 Mshowa
.125 Mabswid
.05549 .01472 m
.05549 .01847 L
s
.08716 .01472 m
.08716 .01847 L
s
.11884 .01472 m
.11884 .01847 L
s
.15051 .01472 m
.15051 .01847 L
s
.21386 .01472 m
.21386 .01847 L
s
.24554 .01472 m
.24554 .01847 L
s
.27722 .01472 m
.27722 .01847 L
s
.30889 .01472 m
.30889 .01847 L
s
.37224 .01472 m
.37224 .01847 L
s
.40392 .01472 m
.40392 .01847 L
s
.4356 .01472 m
.4356 .01847 L
s
.46727 .01472 m
.46727 .01847 L
s
.53062 .01472 m
.53062 .01847 L
s
.5623 .01472 m
.5623 .01847 L
s
.59398 .01472 m
.59398 .01847 L
s
.62565 .01472 m
.62565 .01847 L
s
.689 .01472 m
.689 .01847 L
s
.72068 .01472 m
.72068 .01847 L
s
.75235 .01472 m
.75235 .01847 L
s
.78403 .01472 m
.78403 .01847 L
s
.84738 .01472 m
.84738 .01847 L
s
.87906 .01472 m
.87906 .01847 L
s
.91073 .01472 m
.91073 .01847 L
s
.94241 .01472 m
.94241 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .10126 m
.03006 .10126 L
s
[(0.002)] .01131 .10126 1 0 Mshowa
.02381 .18781 m
.03006 .18781 L
s
[(0.004)] .01131 .18781 1 0 Mshowa
.02381 .27435 m
.03006 .27435 L
s
[(0.006)] .01131 .27435 1 0 Mshowa
.02381 .3609 m
.03006 .3609 L
s
[(0.008)] .01131 .3609 1 0 Mshowa
.02381 .44744 m
.03006 .44744 L
s
[(0.01)] .01131 .44744 1 0 Mshowa
.02381 .53399 m
.03006 .53399 L
s
[(0.012)] .01131 .53399 1 0 Mshowa
.125 Mabswid
.02381 .03635 m
.02756 .03635 L
s
.02381 .05799 m
.02756 .05799 L
s
.02381 .07962 m
.02756 .07962 L
s
.02381 .1229 m
.02756 .1229 L
s
.02381 .14453 m
.02756 .14453 L
s
.02381 .16617 m
.02756 .16617 L
s
.02381 .20944 m
.02756 .20944 L
s
.02381 .23108 m
.02756 .23108 L
s
.02381 .25271 m
.02756 .25271 L
s
.02381 .29599 m
.02756 .29599 L
s
.02381 .31762 m
.02756 .31762 L
s
.02381 .33926 m
.02756 .33926 L
s
.02381 .38253 m
.02756 .38253 L
s
.02381 .40417 m
.02756 .40417 L
s
.02381 .42581 m
.02756 .42581 L
s
.02381 .46908 m
.02756 .46908 L
s
.02381 .49071 m
.02756 .49071 L
s
.02381 .51235 m
.02756 .51235 L
s
.02381 .55562 m
.02756 .55562 L
s
.02381 .57726 m
.02756 .57726 L
s
.02381 .5989 m
.02756 .5989 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05549 .06882 m
.09284 .13202 L
.13357 .19941 L
.17183 .26047 L
.20862 .31645 L
.24778 .37236 L
.28547 .42192 L
.32554 .46932 L
.36414 .50917 L
.40128 .54157 L
.44079 .56907 L
.46043 .57989 L
.47883 .58825 L
.49645 .59457 L
.50628 .59737 L
.5154 .5995 L
.52517 .60126 L
.53033 .60197 L
.53584 .60257 L
.53836 .60279 L
.54108 .60298 L
.54365 .60312 L
.54488 .60317 L
.54602 .60322 L
.54714 .60325 L
.54816 .60328 L
.5493 .6033 L
.55051 .60331 L
.55168 .60332 L
.55295 .60332 L
.55415 .6033 L
.55526 .60329 L
.5564 .60326 L
.55759 .60323 L
.55974 .60314 L
.56204 .60302 L
.56456 .60285 L
.56909 .60246 L
.5733 .60198 L
.58293 .60049 L
.59322 .59829 L
.60312 .59556 L
.61385 .59192 L
.63334 .58349 L
.65255 .57283 L
.67289 .55896 L
.70952 .52718 L
.74852 .48348 L
.78605 .43162 L
.82596 .3657 L
Mistroke
.86439 .29152 L
.90137 .21011 L
.94071 .11252 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00@03oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo0`00000G0?oo
o`800000103oool00`000000oooo0?ooo`020?ooo`@000005`3oool2000000@0oooo00<000000?oo
o`3oool00P3oool400000080oooo0`00000H0?ooo`800000103oool00`000000oooo0?ooo`020?oo
o`@000005`3oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`00000H0?oo
o`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool003l0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo01<0oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool05`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo000o0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
1@3oool00`000000oooo0?ooo`0C0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool05@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0503oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo01H0oooo00@00000
0?ooo`3oool000002@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo00@0
00000?ooo`3oool000002`3oool100000040oooo0@3oool003l0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000020?ooo`<000005P3oool010000000oooo0?ooo`00000:0?oo
o`030000003oool0oooo01D0oooo00@000000?ooo`3oool000002P3oool010000000oooo0?ooo`3o
ool3000001L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0E0?ooo`040000
003oool0oooo000000X0oooo00@000000?ooo`3oool0oooo0`00000G0?ooo`040000003oool0oooo
000000T0oooo0P0000030?ooo`00?`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo01D0oooo00@000000?ooo`3oool000002P3oool01@000000oooo0?ooo`3oool00000
01T0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0D0?ooo`040000003oool0
oooo000000/0oooo00@000000?ooo`3oool000006@3oool010000000oooo0?ooo`00000;0?ooo`40
00000@3oool10?ooo`00?`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`0000080?ooo`<000005`3oool0
10000000oooo0?ooo`0000080?ooo`<000000`3oool00`000000oooo0?ooo`0G0?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool000005P3oool010000000oooo0?ooo`0000080?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool05`3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo00100?ooo`8000002P3oool2000000<0oooo1000
000F0?ooo`8000002`3oool00`000000oooo0?ooo`0F0?ooo`8000002`3oool010000000oooo0?oo
o`3oool4000001L0oooo0P00000:0?ooo`800000603oool2000000X0oooo0P0000030?ooo`@00000
5`3oool2000000X0oooo0P0000030?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool0
0`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`006`3ooooo000000<0
00000`3oool00240oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00240oooo00<000000?ooo`3oool09P3o
ool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool09P3o
ool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02D0oooo0P0000090?ooo`008@3oool0
0`000000oooo0?ooo`3a0?ooo`030000003oool0oooo00P0oooo000Q0?ooo`030000003oool0oooo
0?40oooo00<000000?ooo`3oool0203oool00240oooo00<000000?ooo`3oool0l03oool00`000000
oooo0?ooo`090?ooo`008@3oool200000?40oooo00<000000?ooo`3oool02@3oool00240oooo00<0
00000?ooo`3oool0l03oool00`000000oooo0?ooo`090?ooo`008@3oool00`000000oooo0?ooo`3_
0?ooo`030000003oool0oooo00X0oooo000Q0?ooo`030000003oool0oooo0>l0oooo00<000000?oo
o`3oool02P3oool00240oooo00<000000?ooo`3oool0k`3oool00`000000oooo0?ooo`0:0?ooo`00
8@3oool00`000000oooo0?ooo`3^0?ooo`030000003oool0oooo00/0oooo000Q0?ooo`800000k`3o
ool00`000000oooo0?ooo`0;0?ooo`008@3oool00`000000oooo0?ooo`3^0?ooo`030000003oool0
oooo00/0oooo000Q0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0i@3oool00`00
0000oooo0?ooo`0<0?ooo`008@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0>@0
oooo00<000000?ooo`3oool0303oool00240oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`3S0?ooo`030000003oool0oooo00d0oooo000Q0?ooo`800000203oool00`000000oooo0?oo
o`3R0?ooo`030000003oool0oooo00d0oooo000Q0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool0hP3oool00`000000oooo0?ooo`0=0?ooo`008@3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00240oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`3O0?ooo`030000003oool0oooo00h0oooo00020?ooo`800000
103oool00`000000oooo0?ooo`030?ooo`800000103oool2000000<0oooo100000040?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool0g`3oool00`000000oooo0?ooo`0>0?ooo`0000D0
oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool0g@3oool00`000000oooo0?ooo`0?0?ooo`0000D0oooo0000003oool0oooo000000080?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?oo
o`040?ooo`8000002`3oool00`000000oooo0?ooo`3M0?ooo`030000003oool0oooo00l0oooo0000
1@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`3L0?ooo`030000003oool0oooo00l0oooo00001@3oool000000?ooo`3oool0000000P0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3K0?ooo`030000003oool0
oooo0100oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`3J0?ooo`030000003oool0oooo0100oooo00020?ooo`800000
2P3oool2000000@0oooo0P0000040?ooo`8000001@3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo0=T0oooo00<000000?ooo`3oool0403oool00240oooo0P00000>0?ooo`030000003o
ool0oooo0=P0oooo00<000000?ooo`3oool04@3oool00240oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`3G0?ooo`030000003oool0oooo0140oooo000Q0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0eP3oool00`000000oooo0?ooo`0B0?ooo`008@3oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo0=D0oooo00<000000?ooo`3oool04P3oool00240oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo0180oooo
000Q0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0d`3oool00`000000oooo0?oo
o`0C0?ooo`008@3oool200000180oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`0C
0?ooo`008@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0=40oooo00<000000?oo
o`3oool0503oool00240oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3@0?ooo`03
0000003oool0oooo01@0oooo000Q0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
c`3oool00`000000oooo0?ooo`0D0?ooo`008@3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo0<h0oooo00<000000?ooo`3oool05@3oool00240oooo0P00000E0?ooo`030000003oool0
oooo0<d0oooo00<000000?ooo`3oool05@3oool00240oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`3<0?ooo`030000003oool0oooo01H0oooo000Q0?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool0b`3oool00`000000oooo0?ooo`0F0?ooo`008@3oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo0</0oooo00<000000?ooo`3oool05P3oool00080oooo0P00
00040?ooo`030000003oool0oooo00<0oooo0P0000040?ooo`8000001@3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0b@3oool00`000000oooo0?oo
o`0G0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`0G0?ooo`0000D0oooo0000003oool0
oooo000000080?ooo`040000003oool0oooo00000080oooo00D000000?ooo`3oool000000?ooo`05
000000@0oooo0P00000H0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0603oool0
00050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`060000003o
ool0oooo0000003oool000000P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0aP3oool00`000000oooo0?ooo`0H0?ooo`0000D0oooo0000003oool0
oooo000000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo000000050?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0aP3oool0
0`000000oooo0?ooo`0H0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0
oooo00000080oooo00@000000?ooo`3oool000000`3oool2000000D0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`340?ooo`030000003oool0oooo01T0oooo00020?ooo`8000002P3o
ool2000000@0oooo0P0000050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`330?ooo`030000003oool0oooo01T0oooo000Q0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0J0?ooo`008@3oool20000
01`0oooo00<000000?ooo`3oool0`@3oool00`000000oooo0?ooo`0J0?ooo`008@3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo0<40oooo00<000000?ooo`3oool06P3oool00240oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`2o0?ooo`030000003oool0oooo01/0oooo
000Q0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0_P3oool00`000000oooo0?oo
o`0K0?ooo`008@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0;d0oooo00<00000
0?ooo`3oool0703oool00240oooo0P00000O0?ooo`030000003oool0oooo0;`0oooo00<000000?oo
o`3oool0703oool00240oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`2j0?ooo`03
0000003oool0oooo01d0oooo000Q0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
^P3oool00`000000oooo0?ooo`0M0?ooo`008@3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo0;P0oooo00<000000?ooo`3oool07P3oool00240oooo00<000000?ooo`3oool0803oool0
0`000000oooo0?ooo`2h0?ooo`030000003oool0oooo01h0oooo000Q0?ooo`030000003oool0oooo
0240oooo00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`0O0?ooo`008@3oool2000002<0
oooo00<000000?ooo`3oool0]@3oool00`000000oooo0?ooo`0O0?ooo`008@3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo0;@0oooo00<000000?ooo`3oool0803oool00240oooo00<0
00000?ooo`3oool08`3oool00`000000oooo0?ooo`2c0?ooo`030000003oool0oooo0200oooo0002
0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000103oool2000000@0oooo0P000005
0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0Q
0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02@0
oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0Q0?ooo`0000D0oooo0000003oool0
oooo000000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000040?ooo`8000009P3oool00`000000oooo0?ooo`2_0?ooo`030000003o
ool0oooo0280oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`<000001@3oool00`000000oooo0?ooo`0V0?ooo`03
0000003oool0oooo0:h0oooo00<000000?ooo`3oool08P3oool000050?ooo`000000oooo0?ooo`00
0000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0:d0oooo00<00000
0?ooo`3oool08`3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`00
00020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo0:`0oooo00<000000?ooo`3oool08`3oool00080oooo0P00
000:0?ooo`800000103oool2000000@0oooo0`0000040?ooo`030000003oool0oooo02L0oooo00<0
00000?ooo`3oool0Z`3oool00`000000oooo0?ooo`0T0?ooo`008@3oool00`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo0:X0oooo00<000000?ooo`3oool0903oool00240oooo0P00000Z0?oo
o`030000003oool0oooo0:P0oooo00<000000?ooo`3oool09@3oool00240oooo00<000000?ooo`3o
ool0:@3oool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo02D0oooo000Q0?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool0YP3oool00`000000oooo0?ooo`0V0?ooo`008@3o
ool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool09P3o
ool00240oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`2T0?ooo`030000003oool0
oooo02L0oooo000Q0?ooo`800000;@3oool00`000000oooo0?ooo`2S0?ooo`030000003oool0oooo
02L0oooo000Q0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0X@3oool00`000000
oooo0?ooo`0X0?ooo`008@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0:40oooo
00<000000?ooo`3oool0:03oool00240oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?oo
o`2O0?ooo`030000003oool0oooo02T0oooo000Q0?ooo`030000003oool0oooo02l0oooo00<00000
0?ooo`3oool0WP3oool00`000000oooo0?ooo`0Y0?ooo`008@3oool00`000000oooo0?ooo`0_0?oo
o`030000003oool0oooo09d0oooo00<000000?ooo`3oool0:P3oool00240oooo0P00000a0?ooo`03
0000003oool0oooo09`0oooo00<000000?ooo`3oool0:P3oool00240oooo00<000000?ooo`3oool0
<@3oool00`000000oooo0?ooo`2J0?ooo`030000003oool0oooo02/0oooo000Q0?ooo`030000003o
ool0oooo0340oooo00<000000?ooo`3oool0VP3oool00`000000oooo0?ooo`0[0?ooo`008@3oool0
0`000000oooo0?ooo`0b0?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool0;03oool0
0080oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000040?ooo`800000103oool20000
00D0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`2G0?ooo`030000003oool0oooo
02`0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`2F0?ooo`030000003oool0oooo02d0oooo00001@3oool000000?oo
o`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo000000@0oooo0P00000e0?ooo`030000003oool0oooo09D0oooo00<00000
0?ooo`3oool0;@3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`00
00020?ooo`040000003oool0oooo000000<0oooo0P0000050?ooo`030000003oool0oooo03D0oooo
00<000000?ooo`3oool0T`3oool00`000000oooo0?ooo`0^0?ooo`0000D0oooo0000003oool0oooo
000000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0TP3oool0
0`000000oooo0?ooo`0_0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0
oooo00000080oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`03
0000003oool0oooo03H0oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`0_0?ooo`00
0P3oool2000000X0oooo0P0000040?ooo`800000103oool2000000D0oooo00<000000?ooo`3oool0
=`3oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo0300oooo000Q0?ooo`800000>@3o
ool00`000000oooo0?ooo`2>0?ooo`030000003oool0oooo0300oooo000Q0?ooo`030000003oool0
oooo03P0oooo00<000000?ooo`3oool0S@3oool00`000000oooo0?ooo`0a0?ooo`008@3oool00`00
0000oooo0?ooo`0i0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0<P3oool00240
oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo0380
oooo000Q0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0R03oool00`000000oooo
0?ooo`0c0?ooo`008@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo08L0oooo00<0
00000?ooo`3oool0<`3oool00240oooo0P00000n0?ooo`030000003oool0oooo08D0oooo00<00000
0?ooo`3oool0=03oool00240oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`250?oo
o`030000003oool0oooo03@0oooo000Q0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3o
ool0P`3oool00`000000oooo0?ooo`0e0?ooo`008@3oool00`000000oooo0?ooo`0o0?ooo`030000
003oool0oooo0840oooo00<000000?ooo`3oool0=P3oool00240oooo00<000000?ooo`3oool0@03o
ool00`000000oooo0?ooo`200?ooo`030000003oool0oooo03H0oooo000Q0?ooo`800000@P3oool0
0`000000oooo0?ooo`1n0?ooo`030000003oool0oooo03L0oooo000Q0?ooo`030000003oool0oooo
0480oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`0g0?ooo`008@3oool00`000000
oooo0?ooo`120?ooo`030000003oool0oooo07`0oooo00<000000?ooo`3oool0>03oool00240oooo
00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`1j0?ooo`030000003oool0oooo03T0oooo
00080?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool4000000@0oooo00<0
00000?ooo`3oool0A03oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo03X0oooo0007
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0M`3oool00`000000oooo
0?ooo`0j0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool00`3oool2000004H0oooo00<000000?ooo`3oool0MP3oool00`000000
oooo0?ooo`0k0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo
07@0oooo00<000000?ooo`3oool0?03oool000L0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0A`3oool0
0`000000oooo0?ooo`1b0?ooo`030000003oool0oooo03d0oooo00070?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool000000P3oool3000000D0oooo00<000000?ooo`3oool0B03o
ool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo03d0oooo00080?ooo`8000002P3oool2
000000D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`180?ooo`030000003oool0
oooo0700oooo00<000000?ooo`3oool0?P3oool00240oooo0P00001:0?ooo`030000003oool0oooo
06h0oooo00<000000?ooo`3oool0?`3oool00240oooo00<000000?ooo`3oool0BP3oool00`000000
oooo0?ooo`1/0?ooo`030000003oool0oooo0400oooo000Q0?ooo`030000003oool0oooo04/0oooo
00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`100?ooo`008@3oool00`000000oooo0?oo
o`1<0?ooo`030000003oool0oooo06T0oooo00<000000?ooo`3oool0@@3oool00240oooo00<00000
0?ooo`3oool0C@3oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo0480oooo000Q0?oo
o`030000003oool0oooo04h0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`130?oo
o`008@3oool200000500oooo00<000000?ooo`3oool0H`3oool00`000000oooo0?ooo`140?ooo`00
8@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0
A@3oool00240oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`1O0?ooo`030000003o
ool0oooo04H0oooo000Q0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0G@3oool0
0`000000oooo0?ooo`170?ooo`008@3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo
05`0oooo00<000000?ooo`3oool0A`3oool00240oooo00<000000?ooo`3oool0E03oool00`000000
oooo0?ooo`1J0?ooo`030000003oool0oooo04P0oooo000Q0?ooo`800000EP3oool00`000000oooo
0?ooo`1H0?ooo`030000003oool0oooo04T0oooo000Q0?ooo`030000003oool0oooo05H0oooo0P00
001G0?ooo`030000003oool0oooo04X0oooo000Q0?ooo`030000003oool0oooo05P0oooo00<00000
0?ooo`3oool0D`3oool00`000000oooo0?ooo`1;0?ooo`000P3oool2000000@0oooo00<000000?oo
o`3oool00`3oool2000000<0oooo100000020?ooo`@00000103oool00`000000oooo0?ooo`1I0?oo
o`030000003oool0oooo0540oooo00<000000?ooo`3oool0C03oool000050?ooo`000000oooo0?oo
o`000000203oool010000000oooo0?ooo`0000040?ooo`050000003oool0oooo0?ooo`0000001`3o
ool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0C@3o
ool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0103oool2000005`0oooo0P00001>0?ooo`030000
003oool0oooo04h0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool0
0000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0G@3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo04l0oooo00001@3oool00000
0?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`180?oo
o`030000003oool0oooo0500oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?oo
o`3oool000000P3oool3000000<0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`1O0?ooo`800000A`3oool00`000000oooo0?ooo`1A0?ooo`000P3oool2000000X0oooo0P000005
0?ooo`030000003oool0oooo0080oooo0P0000050?ooo`030000003oool0oooo0640oooo00<00000
0?ooo`3oool0@`3oool00`000000oooo0?ooo`1B0?ooo`008@3oool00`000000oooo0?ooo`1R0?oo
o`030000003oool0oooo0440oooo00<000000?ooo`3oool0D`3oool00240oooo0P00001T0?ooo`80
0000@03oool00`000000oooo0?ooo`1D0?ooo`008@3oool00`000000oooo0?ooo`1U0?ooo`030000
003oool0oooo03`0oooo00<000000?ooo`3oool0E@3oool00240oooo00<000000?ooo`3oool0IP3o
ool2000003X0oooo0P00001H0?ooo`008@3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0
oooo03D0oooo0P00001J0?ooo`008@3oool00`000000oooo0?ooo`1Y0?ooo`800000=03oool00`00
0000oooo0?ooo`1J0?ooo`008@3oool2000006`0oooo0P00000`0?ooo`800000G@3oool00240oooo
00<000000?ooo`3oool0K@3oool2000002`0oooo0P00001O0?ooo`008@3oool00`000000oooo0?oo
o`1_0?ooo`800000:03oool200000640oooo000Q0?ooo`030000003oool0oooo0740oooo0P00000T
0?ooo`800000H`3oool00240oooo00<000000?ooo`3oool0L`3oool2000001l0oooo0`00001U0?oo
o`008@3oool00`000000oooo0?ooo`1e0?ooo`<000006@3oool3000006P0oooo000Q0?ooo`800000
N@3oool5000000l0oooo1@00001[0?ooo`008@3oool00`000000oooo0?ooo`1m0?ooo`l00000L03o
ool00240oooo00<000000?ooo`3oool0o03oool00240oooo00<000000?ooo`3oool0o03oool00240
oooo00<000000?ooo`3oool0o03oool00240oooo00<000000?ooo`3oool0o03oool00?l0oooo8@3o
ool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0407158, -0.00151401, \
0.00122561, 8.97153*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fullalpha[1, 1/2, 1/2*Cos[Pi/4], 3/5, 1]\)], "Input"],

Cell[BoxData[
    \(0.313604706141335621547708054797154`20.5177\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      klim[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 1], {th, 0.001, 
        Pi}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.303152 0.0147151 1.17721 [
[.17539 .00222 -9 -9 ]
[.17539 .00222 9 0 ]
[.32696 .00222 -3 -9 ]
[.32696 .00222 3 0 ]
[.47854 .00222 -9 -9 ]
[.47854 .00222 9 0 ]
[.63011 .00222 -3 -9 ]
[.63011 .00222 3 0 ]
[.78169 .00222 -9 -9 ]
[.78169 .00222 9 0 ]
[.93327 .00222 -3 -9 ]
[.93327 .00222 3 0 ]
[.01131 .13244 -18 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -18 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -18 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .01472 m
.17539 .02097 L
s
[(0.5)] .17539 .00222 0 1 Mshowa
.32696 .01472 m
.32696 .02097 L
s
[(1)] .32696 .00222 0 1 Mshowa
.47854 .01472 m
.47854 .02097 L
s
[(1.5)] .47854 .00222 0 1 Mshowa
.63011 .01472 m
.63011 .02097 L
s
[(2)] .63011 .00222 0 1 Mshowa
.78169 .01472 m
.78169 .02097 L
s
[(2.5)] .78169 .00222 0 1 Mshowa
.93327 .01472 m
.93327 .02097 L
s
[(3)] .93327 .00222 0 1 Mshowa
.125 Mabswid
.05412 .01472 m
.05412 .01847 L
s
.08444 .01472 m
.08444 .01847 L
s
.11476 .01472 m
.11476 .01847 L
s
.14507 .01472 m
.14507 .01847 L
s
.2057 .01472 m
.2057 .01847 L
s
.23602 .01472 m
.23602 .01847 L
s
.26633 .01472 m
.26633 .01847 L
s
.29665 .01472 m
.29665 .01847 L
s
.35728 .01472 m
.35728 .01847 L
s
.38759 .01472 m
.38759 .01847 L
s
.41791 .01472 m
.41791 .01847 L
s
.44822 .01472 m
.44822 .01847 L
s
.50885 .01472 m
.50885 .01847 L
s
.53917 .01472 m
.53917 .01847 L
s
.56948 .01472 m
.56948 .01847 L
s
.5998 .01472 m
.5998 .01847 L
s
.66043 .01472 m
.66043 .01847 L
s
.69074 .01472 m
.69074 .01847 L
s
.72106 .01472 m
.72106 .01847 L
s
.75138 .01472 m
.75138 .01847 L
s
.81201 .01472 m
.81201 .01847 L
s
.84232 .01472 m
.84232 .01847 L
s
.87264 .01472 m
.87264 .01847 L
s
.90295 .01472 m
.90295 .01847 L
s
.96358 .01472 m
.96358 .01847 L
s
.9939 .01472 m
.9939 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(0.1)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.2)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(0.3)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(0.4)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(0.5)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03826 m
.02756 .03826 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .08535 m
.02756 .08535 L
s
.02381 .10889 m
.02756 .10889 L
s
.02381 .15598 m
.02756 .15598 L
s
.02381 .17952 m
.02756 .17952 L
s
.02381 .20307 m
.02756 .20307 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .2737 m
.02756 .2737 L
s
.02381 .29725 m
.02756 .29725 L
s
.02381 .32079 m
.02756 .32079 L
s
.02381 .34433 m
.02756 .34433 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .41497 m
.02756 .41497 L
s
.02381 .43851 m
.02756 .43851 L
s
.02381 .46205 m
.02756 .46205 L
s
.02381 .50914 m
.02756 .50914 L
s
.02381 .53269 m
.02756 .53269 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57978 m
.02756 .57978 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02411 .60332 m
.02529 .60331 L
.02636 .60328 L
.02759 .60324 L
.02876 .60318 L
.03083 .60304 L
.03309 .60283 L
.03557 .60253 L
.0382 .60214 L
.04291 .60124 L
.05235 .59869 L
.06274 .59475 L
.07321 .58962 L
.08448 .58284 L
.10486 .56752 L
.14607 .5263 L
.18576 .47703 L
.22395 .42415 L
.26458 .36469 L
.30369 .30615 L
.34526 .24371 L
.38532 .18384 L
.42386 .12669 L
.46485 .06631 L
.4747 .05185 L
.48526 .03634 L
.49038 .02883 L
.49523 .02172 L
.49734 .01875 L
.4985 .01702 L
.49959 .016 L
.50075 .01622 L
.50205 .01775 L
.50432 .02107 L
.54228 .07679 L
.58096 .1338 L
.62208 .19487 L
.66169 .25415 L
.69979 .31139 L
.74034 .37199 L
.77937 .42889 L
.82086 .48574 L
.86083 .53402 L
.89928 .57094 L
.91915 .58516 L
.92929 .59095 L
.94019 .59598 L
.94991 .59939 L
.95451 .60064 L
.95879 .60159 L
Mistroke
.96306 .60234 L
.9655 .60267 L
.96771 .60291 L
.96983 .60309 L
.97182 .60321 L
.97291 .60326 L
.97392 .60329 L
.97503 .60331 L
.97619 .60235 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00=`3oool2000000@0oooo00<000000?ooo`3oool00P3oool300000200
oooo1000000P0?ooo`@000000`3oool00`000000oooo0?ooo`020?ooo`<00000803oool4000001l0
oooo100000030?ooo`030000003oool0oooo0080oooo0`00000Q0?ooo`8000004`3oool003H0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0M0?ooo`040000003oool0oooo0000
0180oooo000f0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0@0?ooo`00=P3oool010000000oooo0?ooo`0000080?ooo`<000008P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo00L0oooo0`00000R0?ooo`030000003oool0oooo
0200oooo00<000000?ooo`3oool01`3oool300000240oooo0P00000C0?ooo`00=P3oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0403oool003H0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0P0?ooo`<000008@3oool30000
00T0oooo00<000000?ooo`3oool0803oool010000000oooo0?ooo`00000O0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool0803oool010000000oooo0?ooo`00000B0?ooo`005@3o
ool00`000000oooo0?ooo`0O0?ooo`8000002@3oool400000240oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`070?ooo`@00000803oool200000240oooo0P0000090?ooo`@00000803o
ool2000001<0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?oo
o`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000?0?ooool00000
4@0000010?ooo`005@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`020?ooo`80000000<0oooo0000003oool01P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool100000040oooo0@3oool001D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo02L0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0V0?ooo`030000003o
ool0oooo0140oooo000E0?ooo`030000003oool0oooo07d0oooo00D000000?ooo`3oool0oooo0000
00260?ooo`005@3oool00`000000oooo0?ooo`1l0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0Q03oool001D0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo08<0oooo000E0?ooo`800000N`3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo0880oooo000E0?ooo`030000003oool0oooo07X0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`220?ooo`005@3oool00`000000oooo0?ooo`1i0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0P@3oool001D0oooo00<000000?ooo`3oool0N@3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0800oooo000E0?ooo`030000003oool0oooo
07P0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`200?ooo`005@3oool00`000000
oooo0?ooo`1g0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0O`3oool001D0oooo
00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo07h0oooo
000E0?ooo`800000M`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo07h0oooo000E
0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`1m
0?ooo`005@3oool00`000000oooo0?ooo`1d0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool0O03oool001D0oooo00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo07`0oooo000E0?ooo`030000003oool0oooo07<0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`1k0?ooo`005@3oool00`000000oooo0?ooo`1b0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool0NP3oool001D0oooo0P00001c0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0NP3oool001D0oooo00<000000?ooo`3oool0L@3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo07T0oooo000E0?ooo`030000003oool0oooo0700
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1h0?ooo`005@3oool00`000000oooo
0?ooo`1`0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0N03oool001D0oooo00<0
00000?ooo`3oool0K`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo07L0oooo000E
0?ooo`030000003oool0oooo06h0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`1f
0?ooo`005@3oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo01l0oooo00<000000?oo
o`3oool0MP3oool001D0oooo0P00001^0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3o
ool0M@3oool001D0oooo00<000000?ooo`3oool0K03oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo07@0oooo000E0?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`1d0?ooo`005@3oool00`000000oooo0?ooo`1[0?ooo`030000003oool0
oooo02@0oooo00<000000?ooo`3oool0L`3oool00080oooo0P0000040?ooo`030000003oool0oooo
0080oooo100000040?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`1b0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo
0780oooo00001@3oool000000?ooo`3oool0000000X0oooo00<000000?ooo`3oool00`3oool30000
06T0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1a0?ooo`0000D0oooo0000003o
ool0oooo0000000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0J03oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo0700oooo00001@3oool000000?ooo`3oool00000
00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo
02/0oooo00<000000?ooo`3oool0L03oool000050?ooo`000000oooo0?ooo`000000203oool30000
00D0oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo
06l0oooo00020?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo06H0
oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`1^0?ooo`005@3oool00`000000oooo
0?ooo`1U0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0K@3oool001D0oooo0P00
001V0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0K@3oool001D0oooo00<00000
0?ooo`3oool0I03oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo06`0oooo000E0?oo
o`030000003oool0oooo06<0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`1[0?oo
o`005@3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3o
ool0J`3oool001D0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`0f0?ooo`030000
003oool0oooo06X0oooo000E0?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0>03o
ool00`000000oooo0?ooo`1Y0?ooo`005@3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0
oooo03T0oooo00<000000?ooo`3oool0J@3oool001D0oooo0P00001Q0?ooo`030000003oool0oooo
03X0oooo00<000000?ooo`3oool0J03oool001D0oooo00<000000?ooo`3oool0G`3oool00`000000
oooo0?ooo`0l0?ooo`030000003oool0oooo06L0oooo000E0?ooo`030000003oool0oooo05h0oooo
00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`1V0?ooo`005@3oool00`000000oooo0?oo
o`1N0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0IP3oool001D0oooo00<00000
0?ooo`3oool0G@3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo06D0oooo000E0?oo
o`030000003oool0oooo05`0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1T0?oo
o`005@3oool2000005d0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1T0?ooo`00
5@3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0
H`3oool001D0oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`150?ooo`030000003o
ool0oooo06<0oooo000E0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool0AP3oool0
0`000000oooo0?ooo`1R0?ooo`005@3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo
04P0oooo00<000000?ooo`3oool0H@3oool001D0oooo00<000000?ooo`3oool0F03oool00`000000
oooo0?ooo`190?ooo`030000003oool0oooo0640oooo000E0?ooo`030000003oool0oooo05L0oooo
00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`1P0?ooo`005@3oool2000005P0oooo00<0
00000?ooo`3oool0C03oool00`000000oooo0?ooo`1O0?ooo`005@3oool00`000000oooo0?ooo`1F
0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0G`3oool001D0oooo00<000000?oo
o`3oool0E@3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo05h0oooo000E0?ooo`03
0000003oool0oooo05D0oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`1N0?ooo`00
0P3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000@0oooo00<000000?ooo`3oool0
E03oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo05d0oooo00001@3oool000000?oo
o`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`1C0?ooo`030000
003oool0oooo05<0oooo00<000000?ooo`3oool0G03oool000050?ooo`000000oooo0?ooo`000000
2@3oool00`000000oooo0?ooo`040?ooo`<00000D`3oool00`000000oooo0?ooo`1C0?ooo`030000
003oool0oooo05`0oooo00001@3oool000000?ooo`3oool0000000X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool0
F`3oool000050?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0540oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`1J0?ooo`0000D0
oooo0000003oool0oooo000000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool0D@3oool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo05X0oooo00020?ooo`800000
2P3oool2000000D0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`1I0?ooo`030000
003oool0oooo05T0oooo000E0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0F`3o
ool00`000000oooo0?ooo`1H0?ooo`005@3oool200000500oooo00<000000?ooo`3oool0F`3oool0
0`000000oooo0?ooo`1H0?ooo`005@3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo
05d0oooo00<000000?ooo`3oool0E`3oool001D0oooo00<000000?ooo`3oool0CP3oool00`000000
oooo0?ooo`1N0?ooo`030000003oool0oooo05H0oooo000E0?ooo`030000003oool0oooo04d0oooo
00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`1F0?ooo`005@3oool00`000000oooo0?oo
o`1<0?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0E@3oool001D0oooo00<00000
0?ooo`3oool0C03oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo05@0oooo000E0?oo
o`030000003oool0oooo04/0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`1C0?oo
o`005@3oool2000004/0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`1C0?ooo`00
5@3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0
DP3oool001D0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`1X0?ooo`030000003o
ool0oooo0540oooo000E0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0J@3oool0
0`000000oooo0?ooo`1A0?ooo`005@3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo
06X0oooo00<000000?ooo`3oool0D03oool001D0oooo00<000000?ooo`3oool0A`3oool00`000000
oooo0?ooo`1/0?ooo`030000003oool0oooo04l0oooo000E0?ooo`800000A`3oool00`000000oooo
0?ooo`1]0?ooo`030000003oool0oooo04l0oooo000E0?ooo`030000003oool0oooo04H0oooo00<0
00000?ooo`3oool0KP3oool00`000000oooo0?ooo`1>0?ooo`005@3oool00`000000oooo0?ooo`15
0?ooo`030000003oool0oooo0700oooo00<000000?ooo`3oool0C@3oool001D0oooo00<000000?oo
o`3oool0A03oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo04d0oooo000E0?ooo`03
0000003oool0oooo04@0oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`1<0?ooo`00
5@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo07@0oooo00<000000?ooo`3oool0
B`3oool001D0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1e0?ooo`030000003o
ool0oooo04/0oooo000E0?ooo`800000@P3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0
oooo04X0oooo000E0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0N03oool00`00
0000oooo0?ooo`190?ooo`005@3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo07X0
oooo00<000000?ooo`3oool0B03oool001D0oooo00<000000?ooo`3oool0?`3oool00`000000oooo
0?ooo`1k0?ooo`030000003oool0oooo04P0oooo00020?ooo`800000103oool00`000000oooo0?oo
o`030?ooo`8000001@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo07`0oooo00<0
00000?ooo`3oool0A`3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0OP3oool00`000000oooo
0?ooo`160?ooo`0000D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo
0`00000m0?ooo`030000003oool0oooo07l0oooo00<000000?ooo`3oool0AP3oool000050?ooo`00
0000oooo0?ooo`0000002@3oool2000000D0oooo00<000000?ooo`3oool0?@3oool00`000000oooo
0?ooo`200?ooo`030000003oool0oooo04D0oooo00001@3oool000000?ooo`3oool0000000/0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0880oooo
00<000000?ooo`3oool0A03oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0P`3oool00`000000
oooo0?ooo`140?ooo`000P3oool2000000X0oooo0P0000050?ooo`030000003oool0oooo03/0oooo
00<000000?ooo`3oool0Q03oool00`000000oooo0?ooo`130?ooo`005@3oool00`000000oooo0?oo
o`0j0?ooo`030000003oool0oooo08D0oooo00<000000?ooo`3oool0@`3oool001D0oooo0P00000j
0?ooo`030000003oool0oooo08L0oooo00<000000?ooo`3oool0@P3oool001D0oooo00<000000?oo
o`3oool0>@3oool00`000000oooo0?ooo`280?ooo`030000003oool0oooo0440oooo000E0?ooo`03
0000003oool0oooo03P0oooo00<000000?ooo`3oool0R@3oool00`000000oooo0?ooo`110?ooo`00
5@3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0
@03oool001D0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`2=0?ooo`030000003o
ool0oooo03l0oooo000E0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0S@3oool0
0`000000oooo0?ooo`0o0?ooo`005@3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo
08l0oooo00<000000?ooo`3oool0?P3oool001D0oooo0P00000e0?ooo`030000003oool0oooo0900
oooo00<000000?ooo`3oool0?P3oool001D0oooo00<000000?ooo`3oool0=03oool00`000000oooo
0?ooo`2A0?ooo`030000003oool0oooo03d0oooo000E0?ooo`030000003oool0oooo03<0oooo00<0
00000?ooo`3oool0T`3oool00`000000oooo0?ooo`0l0?ooo`005@3oool00`000000oooo0?ooo`0b
0?ooo`030000003oool0oooo09@0oooo00<000000?ooo`3oool0?03oool001D0oooo00<000000?oo
o`3oool0<P3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo03/0oooo000E0?ooo`03
0000003oool0oooo0340oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`0j0?ooo`00
5@3oool200000340oooo00<000000?ooo`3oool0V@3oool00`000000oooo0?ooo`0i0?ooo`005@3o
ool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3oool0>@3o
ool001D0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`2K0?ooo`030000003oool0
oooo03P0oooo000E0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0W@3oool00`00
0000oooo0?ooo`0g0?ooo`005@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo09h0
oooo00<000000?ooo`3oool0=P3oool001D0oooo00<000000?ooo`3oool0;@3oool00`000000oooo
0?ooo`2P0?ooo`030000003oool0oooo03D0oooo000E0?ooo`030000003oool0oooo02`0oooo00<0
00000?ooo`3oool0X@3oool00`000000oooo0?ooo`0e0?ooo`005@3oool2000002d0oooo00<00000
0?ooo`3oool0XP3oool00`000000oooo0?ooo`0d0?ooo`005@3oool00`000000oooo0?ooo`0[0?oo
o`030000003oool0oooo0:@0oooo00<000000?ooo`3oool0<`3oool001D0oooo00<000000?ooo`3o
ool0:P3oool00`000000oooo0?ooo`2V0?ooo`030000003oool0oooo0380oooo000E0?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool0Y`3oool00`000000oooo0?ooo`0a0?ooo`000P3o
ool2000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0Z03oool00`000000oooo0?ooo`0a0?ooo`0000D0oooo
0000003oool0oooo0000000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0:03o
ool00`000000oooo0?ooo`2Z0?ooo`030000003oool0oooo0300oooo00001@3oool000000?ooo`3o
ool0000000L0oooo1@0000040?ooo`<000009`3oool00`000000oooo0?ooo`2/0?ooo`030000003o
ool0oooo02l0oooo00001@3oool000000?ooo`3oool0000000L0oooo00@000000?ooo`3oool00000
1@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0:h0oooo00<000000?ooo`3oool0
;P3oool000050?ooo`000000oooo0?ooo`000000203oool00`000000oooo000000050?ooo`030000
003oool0oooo02H0oooo00<000000?ooo`3oool0[`3oool00`000000oooo0?ooo`0]0?ooo`0000D0
oooo0000003oool0oooo000000090?ooo`8000001@3oool00`000000oooo0?ooo`0U0?ooo`030000
003oool0oooo0;40oooo00<000000?ooo`3oool0;03oool00080oooo0P00000;0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`2b0?ooo`030000003o
ool0oooo02`0oooo000E0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0]03oool0
0`000000oooo0?ooo`0[0?ooo`005@3oool2000002<0oooo00<000000?ooo`3oool0]P3oool00`00
0000oooo0?ooo`0Z0?ooo`005@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;P0
oooo00<000000?ooo`3oool0:@3oool001D0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`2i0?ooo`030000003oool0oooo02P0oooo000E0?ooo`030000003oool0oooo0200oooo00<0
00000?ooo`3oool0^`3oool00`000000oooo0?ooo`0W0?ooo`005@3oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo0;`0oooo00<000000?ooo`3oool09`3oool001D0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`2n0?ooo`030000003oool0oooo02H0oooo000E0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool0`03oool00`000000oooo0?ooo`0U0?ooo`00
5@3oool2000001d0oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0T0?ooo`005@3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0<@0oooo00<000000?ooo`3oool08`3o
ool001D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`360?ooo`030000003oool0
oooo0280oooo000E0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0b03oool00`00
0000oooo0?ooo`0Q0?ooo`005@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0<X0
oooo00<000000?ooo`3oool0803oool001D0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`3<0?ooo`030000003oool0oooo01l0oooo000E0?ooo`8000005`3oool00`000000oooo0?oo
o`3>0?ooo`030000003oool0oooo01h0oooo000E0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0d03oool00`000000oooo0?ooo`0M0?ooo`005@3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo0=80oooo00<000000?ooo`3oool0703oool001D0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo01/0oooo000E0?ooo`030000
003oool0oooo0140oooo0P00003H0?ooo`030000003oool0oooo01X0oooo000E0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool0f@3oool00`000000oooo0?ooo`0I0?ooo`005@3oool0
0`000000oooo0?ooo`0>0?ooo`800000g@3oool2000001T0oooo000E0?ooo`8000003P3oool00`00
0000oooo0?ooo`3O0?ooo`030000003oool0oooo01H0oooo000E0?ooo`030000003oool0oooo00/0
oooo0P00003S0?ooo`030000003oool0oooo01D0oooo000E0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool0i03oool2000001D0oooo000E0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0i`3oool2000001<0oooo00020?ooo`800000103oool00`000000oooo0?ooo`02
0?ooo`<000001@3oool00`000000oooo0?ooo`070?ooo`800000k03oool300000100oooo00001@3o
ool000000?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`D00000l@3oool3000000d0oooo00001@3oool000000?ooo`3oool0000000/0oooo00<00000
0?ooo`3oool00P3oool500000?T0oooo1P0000070?ooo`0000D0oooo0000003oool0oooo00000008
0?ooo`<000001@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo00001@3oool000000?ooo`3oool0
000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo00001@3o
ool000000?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o
0?ooo`T0oooo00020?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0o`3oool90?oo
o`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.260706, -0.0387349, \
0.0121322, 0.00312425}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      klim[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 10], {th, 0.001, 
        Pi}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.303152 0.0147151 0.548945 [
[.17539 .00222 -9 -9 ]
[.17539 .00222 9 0 ]
[.32696 .00222 -3 -9 ]
[.32696 .00222 3 0 ]
[.47854 .00222 -9 -9 ]
[.47854 .00222 9 0 ]
[.63011 .00222 -3 -9 ]
[.63011 .00222 3 0 ]
[.78169 .00222 -9 -9 ]
[.78169 .00222 9 0 ]
[.93327 .00222 -3 -9 ]
[.93327 .00222 3 0 ]
[.01131 .1245 -18 -4.5 ]
[.01131 .1245 0 4.5 ]
[.01131 .23429 -18 -4.5 ]
[.01131 .23429 0 4.5 ]
[.01131 .34408 -18 -4.5 ]
[.01131 .34408 0 4.5 ]
[.01131 .45387 -18 -4.5 ]
[.01131 .45387 0 4.5 ]
[.01131 .56366 -6 -4.5 ]
[.01131 .56366 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .01472 m
.17539 .02097 L
s
[(0.5)] .17539 .00222 0 1 Mshowa
.32696 .01472 m
.32696 .02097 L
s
[(1)] .32696 .00222 0 1 Mshowa
.47854 .01472 m
.47854 .02097 L
s
[(1.5)] .47854 .00222 0 1 Mshowa
.63011 .01472 m
.63011 .02097 L
s
[(2)] .63011 .00222 0 1 Mshowa
.78169 .01472 m
.78169 .02097 L
s
[(2.5)] .78169 .00222 0 1 Mshowa
.93327 .01472 m
.93327 .02097 L
s
[(3)] .93327 .00222 0 1 Mshowa
.125 Mabswid
.05412 .01472 m
.05412 .01847 L
s
.08444 .01472 m
.08444 .01847 L
s
.11476 .01472 m
.11476 .01847 L
s
.14507 .01472 m
.14507 .01847 L
s
.2057 .01472 m
.2057 .01847 L
s
.23602 .01472 m
.23602 .01847 L
s
.26633 .01472 m
.26633 .01847 L
s
.29665 .01472 m
.29665 .01847 L
s
.35728 .01472 m
.35728 .01847 L
s
.38759 .01472 m
.38759 .01847 L
s
.41791 .01472 m
.41791 .01847 L
s
.44822 .01472 m
.44822 .01847 L
s
.50885 .01472 m
.50885 .01847 L
s
.53917 .01472 m
.53917 .01847 L
s
.56948 .01472 m
.56948 .01847 L
s
.5998 .01472 m
.5998 .01847 L
s
.66043 .01472 m
.66043 .01847 L
s
.69074 .01472 m
.69074 .01847 L
s
.72106 .01472 m
.72106 .01847 L
s
.75138 .01472 m
.75138 .01847 L
s
.81201 .01472 m
.81201 .01847 L
s
.84232 .01472 m
.84232 .01847 L
s
.87264 .01472 m
.87264 .01847 L
s
.90295 .01472 m
.90295 .01847 L
s
.96358 .01472 m
.96358 .01847 L
s
.9939 .01472 m
.9939 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .1245 m
.03006 .1245 L
s
[(0.2)] .01131 .1245 1 0 Mshowa
.02381 .23429 m
.03006 .23429 L
s
[(0.4)] .01131 .23429 1 0 Mshowa
.02381 .34408 m
.03006 .34408 L
s
[(0.6)] .01131 .34408 1 0 Mshowa
.02381 .45387 m
.03006 .45387 L
s
[(0.8)] .01131 .45387 1 0 Mshowa
.02381 .56366 m
.03006 .56366 L
s
[(1)] .01131 .56366 1 0 Mshowa
.125 Mabswid
.02381 .04216 m
.02756 .04216 L
s
.02381 .06961 m
.02756 .06961 L
s
.02381 .09706 m
.02756 .09706 L
s
.02381 .15195 m
.02756 .15195 L
s
.02381 .1794 m
.02756 .1794 L
s
.02381 .20685 m
.02756 .20685 L
s
.02381 .26174 m
.02756 .26174 L
s
.02381 .28919 m
.02756 .28919 L
s
.02381 .31663 m
.02756 .31663 L
s
.02381 .37153 m
.02756 .37153 L
s
.02381 .39898 m
.02756 .39898 L
s
.02381 .42642 m
.02756 .42642 L
s
.02381 .48132 m
.02756 .48132 L
s
.02381 .50877 m
.02756 .50877 L
s
.02381 .53621 m
.02756 .53621 L
s
.02381 .59111 m
.02756 .59111 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02411 .60332 m
.02529 .60326 L
.02636 .60314 L
.02759 .60293 L
.02876 .60265 L
.03083 .60196 L
.03309 .60096 L
.03557 .59954 L
.0382 .59769 L
.04291 .5935 L
.05235 .58205 L
.06274 .56545 L
.10486 .47658 L
.14513 .39073 L
.18389 .32078 L
.2251 .25982 L
.2648 .21118 L
.30298 .17117 L
.34361 .13379 L
.38273 .10135 L
.4243 .06943 L
.46435 .04017 L
.48272 .02702 L
.48776 .02344 L
.49315 .0196 L
.49577 .01773 L
.49706 .01685 L
.49825 .01599 L
.49933 .0154 L
.50047 .01533 L
.50162 .0159 L
.50289 .01681 L
.52282 .03099 L
.54388 .04611 L
.58407 .0757 L
.62275 .10573 L
.66387 .14035 L
.70348 .17757 L
.74158 .21845 L
.78213 .26965 L
.82116 .32916 L
.86265 .40638 L
.90262 .49333 L
.92129 .53425 L
.93151 .55486 L
.94107 .57195 L
.94981 .585 L
.95779 .5942 L
.96234 .5981 L
.96666 .60083 L
Mistroke
.96903 .60191 L
.97122 .60264 L
.97248 .60294 L
.97363 .60314 L
.97495 .60328 L
.97619 .60317 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00=`3oool2000000@0oooo00<000000?ooo`3oool00P3oool300000200
oooo1000000P0?ooo`@000000`3oool00`000000oooo0?ooo`020?ooo`<00000803oool4000001l0
oooo100000030?ooo`030000003oool0oooo0080oooo0`00000Q0?ooo`8000004`3oool003H0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0M0?ooo`040000003oool0oooo0000
0180oooo000f0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0@0?ooo`00=P3oool010000000oooo0?ooo`0000080?ooo`<000008P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo00L0oooo0`00000R0?ooo`030000003oool0oooo
0200oooo00<000000?ooo`3oool01`3oool300000240oooo0P00000C0?ooo`00=P3oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0403oool003H0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0P0?ooo`<000008@3oool30000
00T0oooo00<000000?ooo`3oool0803oool010000000oooo0?ooo`00000O0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool0803oool010000000oooo0?ooo`00000B0?ooo`005@3o
ool00`000000oooo0?ooo`0O0?ooo`8000002@3oool400000240oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`070?ooo`@00000803oool200000240oooo0P0000090?ooo`@00000803o
ool2000001<0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?oo
o`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000>0?ooool00000
4`00000001D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00@000000?ooo`3oool0oooo0`0000000`3oool000000?ooo`060?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`400000
0@3oool10?ooo`005@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02H0oooo00<0
00000?ooo`3oool09`3oool010000000oooo0?ooo`0000030?ooo`800000803oool00`000000oooo
0?ooo`0V0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool04@3oool001D0oooo00<0
00000?ooo`3oool0N`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo08<0oooo000E
0?ooo`030000003oool0oooo07X0oooo00<000000?ooo`3oool01P3oool2000008<0oooo000E0?oo
o`030000003oool0oooo07P0oooo0P00000;0?ooo`030000003oool0oooo0800oooo000E0?ooo`03
0000003oool0oooo07H0oooo0P00000>0?ooo`030000003oool0oooo07l0oooo000E0?ooo`030000
003oool0oooo07D0oooo00<000000?ooo`3oool03`3oool2000007l0oooo000E0?ooo`800000M03o
ool2000001@0oooo00<000000?ooo`3oool0O03oool001D0oooo00<000000?ooo`3oool0LP3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo07/0oooo000E0?ooo`030000003oool0oooo
0740oooo00<000000?ooo`3oool05`3oool2000007/0oooo000E0?ooo`030000003oool0oooo06l0
oooo0P00000L0?ooo`030000003oool0oooo07P0oooo000E0?ooo`030000003oool0oooo06h0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`1g0?ooo`005@3oool00`000000oooo0?oo
o`1]0?ooo`030000003oool0oooo01l0oooo0P00001g0?ooo`005@3oool00`000000oooo0?ooo`1[
0?ooo`800000903oool00`000000oooo0?ooo`1d0?ooo`005@3oool2000006/0oooo00<000000?oo
o`3oool09@3oool00`000000oooo0?ooo`1c0?ooo`005@3oool00`000000oooo0?ooo`1Y0?ooo`03
0000003oool0oooo02L0oooo0P00001c0?ooo`005@3oool00`000000oooo0?ooo`1W0?ooo`800000
;03oool00`000000oooo0?ooo`1`0?ooo`005@3oool00`000000oooo0?ooo`1V0?ooo`030000003o
ool0oooo02d0oooo0P00001`0?ooo`005@3oool00`000000oooo0?ooo`1U0?ooo`030000003oool0
oooo0300oooo00<000000?ooo`3oool0K@3oool001D0oooo00<000000?ooo`3oool0I03oool00`00
0000oooo0?ooo`0b0?ooo`030000003oool0oooo06`0oooo000E0?ooo`030000003oool0oooo06<0
oooo00<000000?ooo`3oool0=03oool2000006`0oooo000E0?ooo`030000003oool0oooo0640oooo
0P00000i0?ooo`030000003oool0oooo06T0oooo000E0?ooo`800000H@3oool00`000000oooo0?oo
o`0j0?ooo`030000003oool0oooo06P0oooo000E0?ooo`030000003oool0oooo05l0oooo00<00000
0?ooo`3oool0?03oool2000006P0oooo000E0?ooo`030000003oool0oooo05h0oooo00<000000?oo
o`3oool0?`3oool00`000000oooo0?ooo`1U0?ooo`005@3oool00`000000oooo0?ooo`1L0?ooo`80
0000@`3oool00`000000oooo0?ooo`1T0?ooo`005@3oool00`000000oooo0?ooo`1K0?ooo`030000
003oool0oooo04@0oooo0P00001T0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00P3o
ool4000000@0oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`170?ooo`030000003o
ool0oooo0640oooo00001@3oool000000?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0H03o
ool000050?ooo`000000oooo0?ooo`0000002@3oool00`000000oooo0?ooo`040?ooo`<00000F03o
ool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo05l0oooo00001@3oool000000?ooo`3o
ool0000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1F0?ooo`800000C`3o
ool00`000000oooo0?ooo`1N0?ooo`0000D0oooo0000003oool0oooo0000000;0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`1@0?ooo`800000GP3o
ool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo05@0oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`1K0?ooo`000P3o
ool2000000X0oooo0P0000050?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool0E@3o
ool00`000000oooo0?ooo`1J0?ooo`005@3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0
oooo05L0oooo00<000000?ooo`3oool0F@3oool001D0oooo00<000000?ooo`3oool0D@3oool00`00
0000oooo0?ooo`1I0?ooo`030000003oool0oooo05P0oooo000E0?ooo`800000D@3oool00`000000
oooo0?ooo`1K0?ooo`030000003oool0oooo05L0oooo000E0?ooo`030000003oool0oooo04h0oooo
0P00001O0?ooo`030000003oool0oooo05H0oooo000E0?ooo`030000003oool0oooo04d0oooo00<0
00000?ooo`3oool0H03oool2000005H0oooo000E0?ooo`030000003oool0oooo04`0oooo00<00000
0?ooo`3oool0H`3oool00`000000oooo0?ooo`1C0?ooo`005@3oool00`000000oooo0?ooo`1;0?oo
o`030000003oool0oooo06D0oooo00<000000?ooo`3oool0DP3oool001D0oooo00<000000?ooo`3o
ool0BP3oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo0540oooo000E0?ooo`030000
003oool0oooo04T0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`1@0?ooo`005@3o
ool00`000000oooo0?ooo`180?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0C`3o
ool001D0oooo0P0000180?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0CP3oool0
01D0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo
04d0oooo000E0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0L03oool00`000000
oooo0?ooo`1=0?ooo`005@3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo0780oooo
00<000000?ooo`3oool0C03oool001D0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?oo
o`1d0?ooo`030000003oool0oooo04/0oooo000E0?ooo`030000003oool0oooo0480oooo00<00000
0?ooo`3oool0MP3oool00`000000oooo0?ooo`1:0?ooo`005@3oool00`000000oooo0?ooo`110?oo
o`030000003oool0oooo07P0oooo00<000000?ooo`3oool0B@3oool001D0oooo0P0000110?ooo`03
0000003oool0oooo07X0oooo00<000000?ooo`3oool0B03oool001D0oooo00<000000?ooo`3oool0
?`3oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo04P0oooo000E0?ooo`030000003o
ool0oooo03h0oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`170?ooo`005@3oool0
0`000000oooo0?ooo`0m0?ooo`030000003oool0oooo07l0oooo00<000000?ooo`3oool0AP3oool0
01D0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`210?ooo`030000003oool0oooo
04D0oooo000E0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0PP3oool00`000000
oooo0?ooo`140?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0Q03oool00`000000oooo
0?ooo`130?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`250?ooo`030000003oool0oooo04<0oooo
00001@3oool000000?ooo`3oool0000000L0oooo1@0000040?ooo`<00000>@3oool00`000000oooo
0?ooo`270?ooo`030000003oool0oooo0480oooo00001@3oool000000?ooo`3oool0000000L0oooo
00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo08P0
oooo00<000000?ooo`3oool0@@3oool000050?ooo`000000oooo0?ooo`000000203oool00`000000
oooo000000050?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0RP3oool00`000000
oooo0?ooo`100?ooo`0000D0oooo0000003oool0oooo000000090?ooo`8000001@3oool00`000000
oooo0?ooo`0g0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0@03oool00080oooo
0P00000;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0=P3oool00`000000oooo
0?ooo`2=0?ooo`030000003oool0oooo03l0oooo000E0?ooo`030000003oool0oooo03H0oooo00<0
00000?ooo`3oool0SP3oool00`000000oooo0?ooo`0n0?ooo`005@3oool00`000000oooo0?ooo`0e
0?ooo`030000003oool0oooo0900oooo00<000000?ooo`3oool0?@3oool001D0oooo0P00000e0?oo
o`030000003oool0oooo0980oooo00<000000?ooo`3oool0?03oool001D0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`2C0?ooo`030000003oool0oooo03`0oooo000E0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool0U03oool00`000000oooo0?ooo`0k0?ooo`005@3o
ool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo09H0oooo00<000000?ooo`3oool0>P3o
ool001D0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0
oooo03T0oooo000E0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0V@3oool00`00
0000oooo0?ooo`0i0?ooo`005@3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo09X0
oooo00<000000?ooo`3oool0>03oool001D0oooo00<000000?ooo`3oool0;`3oool00`000000oooo
0?ooo`2L0?ooo`030000003oool0oooo03L0oooo000E0?ooo`800000;`3oool00`000000oooo0?oo
o`2M0?ooo`030000003oool0oooo03L0oooo000E0?ooo`030000003oool0oooo02d0oooo00<00000
0?ooo`3oool0W`3oool00`000000oooo0?ooo`0f0?ooo`005@3oool00`000000oooo0?ooo`0]0?oo
o`030000003oool0oooo0:00oooo00<000000?ooo`3oool0=@3oool001D0oooo00<000000?ooo`3o
ool0;03oool00`000000oooo0?ooo`2Q0?ooo`030000003oool0oooo03D0oooo000E0?ooo`030000
003oool0oooo02/0oooo00<000000?ooo`3oool0X`3oool00`000000oooo0?ooo`0d0?ooo`005@3o
ool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool0<`3o
ool001D0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`2V0?ooo`030000003oool0
oooo0380oooo000E0?ooo`800000:P3oool00`000000oooo0?ooo`2W0?ooo`030000003oool0oooo
0380oooo000E0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0Z@3oool00`000000
oooo0?ooo`0a0?ooo`005@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0:/0oooo
00<000000?ooo`3oool0<03oool001D0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`2[0?ooo`030000003oool0oooo0300oooo000E0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool0[@3oool00`000000oooo0?ooo`0_0?ooo`005@3oool00`000000oooo0?ooo`0V0?oo
o`030000003oool0oooo0:h0oooo00<000000?ooo`3oool0;P3oool00080oooo0P0000040?ooo`03
0000003oool0oooo00<0oooo0P0000050?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3o
ool0[`3oool00`000000oooo0?ooo`0^0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`2`0?oo
o`030000003oool0oooo02d0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?oo
o`3oool00000103oool3000002@0oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0]
0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo0;80oooo00<000000?ooo`3oool0;03oool000050?ooo`000000oooo
0?ooo`000000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02<0oooo00<00000
0?ooo`3oool0/`3oool00`000000oooo0?ooo`0/0?ooo`0000D0oooo0000003oool0oooo00000008
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`2d
0?ooo`030000003oool0oooo02/0oooo00020?ooo`8000002P3oool3000000@0oooo00<000000?oo
o`3oool08P3oool00`000000oooo0?ooo`2e0?ooo`030000003oool0oooo02/0oooo000E0?ooo`03
0000003oool0oooo0280oooo00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`0Z0?ooo`00
5@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;L0oooo00<000000?ooo`3oool0
:P3oool001D0oooo0P00000R0?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3oool0:@3o
ool001D0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`2i0?ooo`030000003oool0
oooo02T0oooo000E0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0^P3oool00`00
0000oooo0?ooo`0X0?ooo`005@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0;/0
oooo00<000000?ooo`3oool0:03oool001D0oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`2l0?ooo`030000003oool0oooo02L0oooo000E0?ooo`030000003oool0oooo01h0oooo00<0
00000?ooo`3oool0_@3oool00`000000oooo0?ooo`0W0?ooo`005@3oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool09P3oool001D0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`2o0?ooo`030000003oool0oooo02H0oooo000E0?ooo`80
00007P3oool00`000000oooo0?ooo`300?ooo`030000003oool0oooo02D0oooo000E0?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool0`@3oool00`000000oooo0?ooo`0U0?ooo`005@3o
ool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0<80oooo00<000000?ooo`3oool0903o
ool001D0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`330?ooo`030000003oool0
oooo02@0oooo000E0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0a03oool00`00
0000oooo0?ooo`0S0?ooo`005@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0<D0
oooo00<000000?ooo`3oool08`3oool001D0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`360?ooo`030000003oool0oooo0280oooo000E0?ooo`8000006P3oool00`000000oooo0?oo
o`370?ooo`030000003oool0oooo0280oooo000E0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0b03oool00`000000oooo0?ooo`0Q0?ooo`005@3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo0<P0oooo00<000000?ooo`3oool08@3oool001D0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`3:0?ooo`030000003oool0oooo0200oooo000E0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool0bP3oool00`000000oooo0?ooo`0P0?ooo`005@3o
ool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0<`0oooo00<000000?ooo`3oool07`3o
ool00080oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool0c03oool00`000000oooo0?ooo`0O0?ooo`0000D0oooo0000
003oool0oooo000000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`3>0?ooo`030000003oool0oooo01h0oooo00001@3oool000000?ooo`3o
ool0000000P0oooo00@000000?ooo`3oool00000103oool3000001H0oooo00<000000?ooo`3oool0
cP3oool00`000000oooo0?ooo`0N0?ooo`0000D0oooo0000003oool0oooo000000090?ooo`800000
1@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0=00oooo00<000000?ooo`3oool0
7@3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool0d03oool00`000000oooo0?ooo`0M0?ooo`00
00D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo000000@0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`3B0?ooo`030000003oool0oooo01`0oooo00020?ooo`80
00002P3oool2000000D0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`3B0?ooo`03
0000003oool0oooo01`0oooo000E0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
e03oool00`000000oooo0?ooo`0K0?ooo`005@3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo0=@0oooo00<000000?ooo`3oool06`3oool001D0oooo0P00000D0?ooo`030000003oool0
oooo0=D0oooo00<000000?ooo`3oool06P3oool001D0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`3F0?ooo`030000003oool0oooo01X0oooo000E0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool0e`3oool00`000000oooo0?ooo`0I0?ooo`005@3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0=P0oooo00<000000?ooo`3oool06@3oool001D0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`3H0?ooo`030000003oool0oooo01T0oooo000E
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0fP3oool00`000000oooo0?ooo`0H
0?ooo`005@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0=X0oooo00<000000?oo
o`3oool0603oool001D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3L0?ooo`03
0000003oool0oooo01L0oooo000E0?ooo`800000403oool00`000000oooo0?ooo`3L0?ooo`030000
003oool0oooo01L0oooo000E0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0g@3o
ool00`000000oooo0?ooo`0G0?ooo`005@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo0=h0oooo00<000000?ooo`3oool05P3oool001D0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`3O0?ooo`030000003oool0oooo01H0oooo000E0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool0h03oool00`000000oooo0?ooo`0E0?ooo`005@3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo0>40oooo00<000000?ooo`3oool05@3oool001D0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`3R0?ooo`030000003oool0oooo01@0oooo000E
0?ooo`800000303oool00`000000oooo0?ooo`3S0?ooo`030000003oool0oooo01@0oooo000E0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0i03oool00`000000oooo0?ooo`0C0?oo
o`005@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0>D0oooo00<000000?ooo`3o
ool04`3oool001D0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3V0?ooo`030000
003oool0oooo0180oooo000E0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0i`3o
ool00`000000oooo0?ooo`0B0?ooo`005@3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo0>P0oooo00<000000?ooo`3oool04@3oool000d0oooo100000040?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0j@3oool00`000000oooo0?ooo`0A0?ooo`003`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0jP3oool00`000000
oooo0?ooo`0@0?ooo`003`3oool00`000000oooo0?ooo`030?ooo`<000001`3oool00`000000oooo
0?ooo`3[0?ooo`030000003oool0oooo0100oooo000?0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`3/0?ooo`030000003oool0oooo00l0oooo000?
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3]
0?ooo`030000003oool0oooo00l0oooo000=0?ooo`<000001@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo0>h0oooo00<000000?ooo`3oool03P3oool000l0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?00oooo00<000000?ooo`3o
ool03@3oool001D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3a0?ooo`030000
003oool0oooo00`0oooo000E0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0l`3o
ool00`000000oooo0?ooo`0;0?ooo`005@3oool2000000@0oooo00<000000?ooo`3oool0m03oool0
0`000000oooo0?ooo`0;0?ooo`005@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0?H0oooo00<000000?ooo`3oool02P3oool001D0oooo00<000000?ooo`3oool00P00003j0?ooo`03
0000003oool0oooo00T0oooo000E0?ooo`<00000o@3oool4000000L0oooo000E0?ooo`030000003o
ool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`00
0000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.259483, -0.0843316, \
0.0121242, 0.00669551}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      klim[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 100], {th, 0.001, 
        Pi}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.303152 0.0147151 0.298405 [
[.17539 .00222 -9 -9 ]
[.17539 .00222 9 0 ]
[.32696 .00222 -3 -9 ]
[.32696 .00222 3 0 ]
[.47854 .00222 -9 -9 ]
[.47854 .00222 9 0 ]
[.63011 .00222 -3 -9 ]
[.63011 .00222 3 0 ]
[.78169 .00222 -9 -9 ]
[.78169 .00222 9 0 ]
[.93327 .00222 -3 -9 ]
[.93327 .00222 3 0 ]
[.01131 .16392 -18 -4.5 ]
[.01131 .16392 0 4.5 ]
[.01131 .31312 -6 -4.5 ]
[.01131 .31312 0 4.5 ]
[.01131 .46232 -18 -4.5 ]
[.01131 .46232 0 4.5 ]
[.01131 .61153 -6 -4.5 ]
[.01131 .61153 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .01472 m
.17539 .02097 L
s
[(0.5)] .17539 .00222 0 1 Mshowa
.32696 .01472 m
.32696 .02097 L
s
[(1)] .32696 .00222 0 1 Mshowa
.47854 .01472 m
.47854 .02097 L
s
[(1.5)] .47854 .00222 0 1 Mshowa
.63011 .01472 m
.63011 .02097 L
s
[(2)] .63011 .00222 0 1 Mshowa
.78169 .01472 m
.78169 .02097 L
s
[(2.5)] .78169 .00222 0 1 Mshowa
.93327 .01472 m
.93327 .02097 L
s
[(3)] .93327 .00222 0 1 Mshowa
.125 Mabswid
.05412 .01472 m
.05412 .01847 L
s
.08444 .01472 m
.08444 .01847 L
s
.11476 .01472 m
.11476 .01847 L
s
.14507 .01472 m
.14507 .01847 L
s
.2057 .01472 m
.2057 .01847 L
s
.23602 .01472 m
.23602 .01847 L
s
.26633 .01472 m
.26633 .01847 L
s
.29665 .01472 m
.29665 .01847 L
s
.35728 .01472 m
.35728 .01847 L
s
.38759 .01472 m
.38759 .01847 L
s
.41791 .01472 m
.41791 .01847 L
s
.44822 .01472 m
.44822 .01847 L
s
.50885 .01472 m
.50885 .01847 L
s
.53917 .01472 m
.53917 .01847 L
s
.56948 .01472 m
.56948 .01847 L
s
.5998 .01472 m
.5998 .01847 L
s
.66043 .01472 m
.66043 .01847 L
s
.69074 .01472 m
.69074 .01847 L
s
.72106 .01472 m
.72106 .01847 L
s
.75138 .01472 m
.75138 .01847 L
s
.81201 .01472 m
.81201 .01847 L
s
.84232 .01472 m
.84232 .01847 L
s
.87264 .01472 m
.87264 .01847 L
s
.90295 .01472 m
.90295 .01847 L
s
.96358 .01472 m
.96358 .01847 L
s
.9939 .01472 m
.9939 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .16392 m
.03006 .16392 L
s
[(0.5)] .01131 .16392 1 0 Mshowa
.02381 .31312 m
.03006 .31312 L
s
[(1)] .01131 .31312 1 0 Mshowa
.02381 .46232 m
.03006 .46232 L
s
[(1.5)] .01131 .46232 1 0 Mshowa
.02381 .61153 m
.03006 .61153 L
s
[(2)] .01131 .61153 1 0 Mshowa
.125 Mabswid
.02381 .04456 m
.02756 .04456 L
s
.02381 .0744 m
.02756 .0744 L
s
.02381 .10424 m
.02756 .10424 L
s
.02381 .13408 m
.02756 .13408 L
s
.02381 .19376 m
.02756 .19376 L
s
.02381 .2236 m
.02756 .2236 L
s
.02381 .25344 m
.02756 .25344 L
s
.02381 .28328 m
.02756 .28328 L
s
.02381 .34296 m
.02756 .34296 L
s
.02381 .3728 m
.02756 .3728 L
s
.02381 .40264 m
.02756 .40264 L
s
.02381 .43248 m
.02756 .43248 L
s
.02381 .49216 m
.02756 .49216 L
s
.02381 .522 m
.02756 .522 L
s
.02381 .55184 m
.02756 .55184 L
s
.02381 .58169 m
.02756 .58169 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02411 .60332 m
.06274 .41404 L
.08284 .32449 L
.09424 .2869 L
.10486 .25822 L
.12397 .21777 L
.14442 .18545 L
.16572 .15966 L
.18585 .14028 L
.22425 .11202 L
.2651 .08974 L
.30444 .07281 L
.34227 .05917 L
.38254 .04649 L
.4213 .03544 L
.46251 .02443 L
.4831 .01909 L
.48826 .01775 L
.49312 .01649 L
.49524 .01596 L
.49748 .01538 L
.49866 .01514 L
.49994 .01501 L
.50111 .0151 L
.50221 .01531 L
.52069 .02006 L
.54039 .02519 L
.57928 .0356 L
.62063 .04744 L
.66046 .06009 L
.69878 .07407 L
.73955 .09199 L
.77881 .11395 L
.81654 .14238 L
.83588 .16138 L
.85673 .18704 L
.87677 .21912 L
.89786 .26506 L
.9074 .29183 L
.9176 .32608 L
.93596 .40716 L
.97619 .59202 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00>03oool2000000@0oooo00<000000?ooo`3oool00P3oool300000200
oooo1000000O0?ooo`@000000`3oool00`000000oooo0?ooo`020?ooo`<000007`3oool4000001l0
oooo100000030?ooo`030000003oool0oooo0080oooo0`00000Q0?ooo`800000503oool003L0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0M0?ooo`040000003oool0oooo0000
01<0oooo000g0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0A0?ooo`00=`3oool010000000oooo0?ooo`0000080?ooo`<000008P3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo00L0oooo0`00000Q0?ooo`030000003oool0oooo
0200oooo00<000000?ooo`3oool01`3oool300000240oooo0P00000D0?ooo`00=`3oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool04@3oool003L0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0P0?ooo`<00000803oool30000
00T0oooo00<000000?ooo`3oool07`3oool010000000oooo0?ooo`00000O0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool0803oool010000000oooo0?ooo`00000C0?ooo`005`3o
ool00`000000oooo0?ooo`0N0?ooo`8000002@3oool400000240oooo00<000000?ooo`3oool0803o
ool00`000000oooo0?ooo`070?ooo`@000007`3oool200000240oooo0P0000090?ooo`@00000803o
ool2000001@0oooo000G0?ooo`030000003oool0oooo0?l0oooo1`3oool001L0oooo00<000000?oo
o`3oool0o`3oool70?ooo`005`3oool00`000000oooo0?ooo`3o0?ooo`L0oooo000@0?ooool00000
3`0000020?ooo`005`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool3000000D0oooo1@0000030?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo000G
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0S
0?ooo`@00000303oool2000001T0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo0180oooo000G0?ooo`030000003oool0oooo0700oooo1000000B0?oo
o`<00000O@3oool001L0oooo00<000000?ooo`3oool0JP3oool6000001T0oooo1000001i0?ooo`00
5`3oool00`000000oooo0?ooo`1V0?ooo`@000008`3oool4000007D0oooo000G0?ooo`030000003o
ool0oooo0680oooo1000000[0?ooo`<00000LP3oool001L0oooo00<000000?ooo`3oool0G`3oool3
00000380oooo1000001^0?ooo`005`3oool2000005d0oooo0`00000i0?ooo`@00000JP3oool001L0
oooo00<000000?ooo`3oool0F@3oool300000400oooo0`00001W0?ooo`005`3oool00`000000oooo
0?ooo`1G0?ooo`800000AP3oool4000006<0oooo000G0?ooo`030000003oool0oooo05@0oooo0`00
001<0?ooo`@00000G`3oool001L0oooo00<000000?ooo`3oool0D@3oool3000005<0oooo0`00001L
0?ooo`005`3oool00`000000oooo0?ooo`1=0?ooo`@00000F@3oool2000005X0oooo000G0?ooo`03
0000003oool0oooo04X0oooo0`00001O0?ooo`<00000E`3oool001L0oooo00<000000?ooo`3oool0
A`3oool3000006D0oooo0P00001E0?ooo`005`3oool2000004H0oooo0P00001Z0?ooo`<00000DP3o
ool001L0oooo00<000000?ooo`3oool0@`3oool2000006l0oooo0P00001@0?ooo`005`3oool00`00
0000oooo0?ooo`110?ooo`800000L`3oool2000004h0oooo000G0?ooo`030000003oool0oooo03l0
oooo0P00001g0?ooo`800000C03oool001L0oooo00<000000?ooo`3oool0?@3oool2000007/0oooo
0P00001:0?ooo`005`3oool00`000000oooo0?ooo`0k0?ooo`800000O`3oool2000004P0oooo000G
0?ooo`030000003oool0oooo03T0oooo0P0000230?ooo`800000AP3oool001L0oooo00<000000?oo
o`3oool0>03oool00`000000oooo0?ooo`250?ooo`800000A03oool001L0oooo0P00000g0?ooo`80
0000RP3oool00`000000oooo0?ooo`110?ooo`005`3oool00`000000oooo0?ooo`0d0?ooo`800000
S@3oool200000440oooo000G0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0S`3o
ool2000003l0oooo000G0?ooo`030000003oool0oooo0340oooo0P00002D0?ooo`030000003oool0
oooo03`0oooo000G0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0U@3oool20000
03`0oooo000G0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0V03oool00`000000
oooo0?ooo`0i0?ooo`005`3oool00`000000oooo0?ooo`0]0?ooo`800000W03oool00`000000oooo
0?ooo`0h0?ooo`005`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo09d0oooo0P00
000h0?ooo`005`3oool2000002`0oooo00<000000?ooo`3oool0X03oool00`000000oooo0?ooo`0e
0?ooo`005`3oool00`000000oooo0?ooo`0Y0?ooo`800000Y03oool2000003D0oooo000G0?ooo`03
0000003oool0oooo02P0oooo00<000000?ooo`3oool0YP3oool00`000000oooo0?ooo`0b0?ooo`00
5`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0:P0oooo00<000000?ooo`3oool0
<@3oool001L0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`2Z0?ooo`030000003o
ool0oooo0300oooo000G0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0[03oool2
00000300oooo00030?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<000001P3oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo0:l0oooo00<000000?ooo`3oool0;@3oool00080
oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
02<0oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0/0?ooo`000P3oool010000000
oooo0?ooo`00000;0?ooo`030000003oool0oooo00<0oooo0P00000S0?ooo`030000003oool0oooo
0;<0oooo00<000000?ooo`3oool0:`3oool00080oooo00@000000?ooo`3oool00000203oool30000
00H0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`2d0?ooo`030000003oool0oooo
02/0oooo00020?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo0;H0oooo00<000000?ooo`3oool0:P3oool00080
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
0200oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?ooo`0Y0?ooo`000`3oool2000000T0
oooo100000050?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0^@3oool00`000000
oooo0?ooo`0X0?ooo`005`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0;X0oooo
00<000000?ooo`3oool0:03oool001L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`2l0?ooo`030000003oool0oooo02L0oooo000G0?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool0_P3oool00`000000oooo0?ooo`0V0?ooo`005`3oool2000001d0oooo00<000000?oo
o`3oool0_P3oool00`000000oooo0?ooo`0V0?ooo`005`3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0<00oooo00<000000?ooo`3oool09@3oool001L0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo02@0oooo000G0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0T0?ooo`005`3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0<@0oooo00<000000?ooo`3oool08`3oool0
01L0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`350?ooo`030000003oool0oooo
0280oooo000G0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0aP3oool00`000000
oooo0?ooo`0R0?ooo`005`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0<P0oooo
00<000000?ooo`3oool08@3oool001L0oooo0P00000H0?ooo`030000003oool0oooo0<P0oooo00<0
00000?ooo`3oool08@3oool001L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`3:
0?ooo`030000003oool0oooo0200oooo000G0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool0bP3oool00`000000oooo0?ooo`0P0?ooo`005`3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo0<`0oooo00<000000?ooo`3oool07`3oool001L0oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`3<0?ooo`030000003oool0oooo01l0oooo000G0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0c@3oool00`000000oooo0?ooo`0O0?ooo`005`3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0<h0oooo00<000000?ooo`3oool07P3oool0
01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3?0?ooo`030000003oool0oooo
01h0oooo000G0?ooo`800000503oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo01d0
oooo000G0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0d@3oool00`000000oooo
0?ooo`0M0?ooo`005`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0=80oooo00<0
00000?ooo`3oool0703oool001L0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3B
0?ooo`030000003oool0oooo01`0oooo000G0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool0d`3oool00`000000oooo0?ooo`0L0?ooo`005`3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo0=@0oooo00<000000?ooo`3oool06`3oool001L0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo01/0oooo000G0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0e@3oool00`000000oooo0?ooo`0J0?ooo`005`3oool2
00000140oooo00<000000?ooo`3oool0eP3oool00`000000oooo0?ooo`0J0?ooo`005`3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo0=L0oooo00<000000?ooo`3oool06@3oool001L0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3G0?ooo`030000003oool0oooo01T0
oooo000G0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0f03oool00`000000oooo
0?ooo`0I0?ooo`005`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0=P0oooo00<0
00000?ooo`3oool06@3oool001L0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3I
0?ooo`030000003oool0oooo01P0oooo000>0?ooo`@000001@3oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo0=T0oooo00<000000?ooo`3oool0603oool00100oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0=X0oooo00<000000?ooo`3o
ool0603oool00100oooo00<000000?ooo`3oool0103oool2000000l0oooo00<000000?ooo`3oool0
fP3oool00`000000oooo0?ooo`0H0?ooo`00403oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0f`3oool00`000000oooo0?ooo`0G0?ooo`00403oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0g03oool0
0`000000oooo0?ooo`0G0?ooo`003P3oool3000000H0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`3L0?ooo`030000003oool0oooo01L0oooo000@0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3L0?ooo`030000003oool0oooo01L0
oooo000G0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0g03oool00`000000oooo
0?ooo`0G0?ooo`005`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0=h0oooo00<0
00000?ooo`3oool05P3oool001L0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3N
0?ooo`030000003oool0oooo01H0oooo000G0?ooo`8000003@3oool00`000000oooo0?ooo`3N0?oo
o`030000003oool0oooo01H0oooo000G0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool0gP3oool00`000000oooo0?ooo`0F0?ooo`005`3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool05@3oool001L0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`3P0?ooo`030000003oool0oooo01D0oooo000G0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool0h03oool00`000000oooo0?ooo`0E0?ooo`005`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool05@3oool001L0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3Q0?ooo`030000003oool0oooo01D0
oooo000G0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0hP3oool00`000000oooo
0?ooo`0D0?ooo`005`3oool2000000/0oooo00<000000?ooo`3oool0hP3oool00`000000oooo0?oo
o`0D0?ooo`005`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0>80oooo00<00000
0?ooo`3oool0503oool001L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3S0?oo
o`030000003oool0oooo01@0oooo000G0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool0i03oool00`000000oooo0?ooo`0C0?ooo`005`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0>@0oooo00<000000?ooo`3oool04`3oool001L0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`3T0?ooo`030000003oool0oooo01<0oooo000G0?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool0i@3oool00`000000oooo0?ooo`0C0?ooo`005`3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool04P3oool001L0
oooo0P0000090?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool04P3oool001L0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3V0?ooo`030000003oool0oooo0180oooo
000G0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0i`3oool00`000000oooo0?oo
o`0B0?ooo`005`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0>L0oooo00<00000
0?ooo`3oool04P3oool001L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3X0?oo
o`030000003oool0oooo0140oooo000G0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0j03oool00`000000oooo0?ooo`0A0?ooo`005`3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo0>T0oooo00<000000?ooo`3oool04@3oool001L0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`3Y0?ooo`030000003oool0oooo0140oooo000G0?ooo`8000001`3oool0
0`000000oooo0?ooo`3Z0?ooo`030000003oool0oooo0100oooo000G0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0jP3oool00`000000oooo0?ooo`0@0?ooo`005`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo0>X0oooo00<000000?ooo`3oool0403oool001L0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3[0?ooo`030000003oool0oooo0100oooo
000G0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0j`3oool00`000000oooo0?oo
o`0@0?ooo`005`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0>`0oooo00<00000
0?ooo`3oool03`3oool00080oooo100000030?ooo`030000003oool0oooo0080oooo0`0000060?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0k03oool00`000000oooo0?ooo`0?0?oo
o`00103oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`3/0?ooo`030000003oool0oooo00l0oooo00040?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3oool0
k@3oool00`000000oooo0?ooo`0?0?ooo`00103oool00`000000oooo0?ooo`070?ooo`<000001P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0>h0oooo00<000000?ooo`3oool03P3o
ool000@0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0kP3oool00`000000oooo0?ooo`0>0?ooo`000P3oool30000
00T0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0>h0oooo00<000000?ooo`3oool03P3oool000@0oooo00<000000?ooo`3oool01`3oool4000000D0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3^0?ooo`030000003oool0oooo00h0
oooo000G0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0k`3oool00`000000oooo
0?ooo`0>0?ooo`005`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?00oooo00<0
00000?ooo`3oool03@3oool001L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3`
0?ooo`030000003oool0oooo00d0oooo000G0?ooo`800000103oool00`000000oooo0?ooo`3`0?oo
o`030000003oool0oooo00d0oooo000G0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0l03oool00`000000oooo0?ooo`0=0?ooo`005`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0?40oooo00<000000?ooo`3oool03@3oool001L0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`3b0?ooo`030000003oool0oooo00`0oooo000G0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0lP3oool00`000000oooo0?ooo`0<0?ooo`005`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0?80oooo00<000000?ooo`3oool0303oool001L0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3b0?ooo`030000003oool0oooo00`0
oooo000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0l`3oool00`000000oooo
0?ooo`0;0?ooo`005`3oool200000080oooo00<000000?ooo`3oool0m03oool00`000000oooo0?oo
o`0;0?ooo`005`3oool01@000000oooo0?ooo`3oool000000?H0oooo00<000000?ooo`3oool02`3o
ool001L0oooo00D000000?ooo`3oool0oooo0000003f0?ooo`030000003oool0oooo00/0oooo000G
0?ooo`050000003oool0oooo0?ooo`000000mP3oool00`000000oooo0?ooo`0;0?ooo`005`3oool0
1@000000oooo0?ooo`3oool000000?L0oooo00<000000?ooo`3oool02P3oool001L0oooo00@00000
0?ooo`3oool00000n03oool00`000000oooo0?ooo`0:0?ooo`005`3oool010000000oooo0?ooo`00
003h0?ooo`030000003oool0oooo00X0oooo000G0?ooo`040000003oool0oooo00000?P0oooo00<0
00000?ooo`3oool02P3oool001L0oooo0P0000000`3oool000000?ooo`3h0?ooo`030000003oool0
oooo00T0oooo000G0?ooo`040000003oool0oooo00000?T0oooo00<000000?ooo`3oool02@3oool0
01L0oooo00<000000?ooo`000000nP3oool00`000000oooo0?ooo`090?ooo`005`3oool00`000000
oooo0000003j0?ooo`030000003oool0oooo00T0oooo000G0?ooo`030000003oool000000?X0oooo
00<000000?ooo`3oool02@3oool001L0oooo00<000000?ooo`000000n`3oool00`000000oooo0?oo
o`080?ooo`005`3oool00`000000oooo0000003k0?ooo`030000003oool0oooo00P0oooo000G0?oo
o`800000o03oool00`000000oooo0?ooo`080?ooo`005`3oool200000?`0oooo00<000000?ooo`3o
ool0203oool001L0oooo0P00003m0?ooo`030000003oool0oooo00L0oooo000G0?ooo`800000o@3o
ool00`000000oooo0?ooo`070?ooo`005`3oool200000?d0oooo00<000000?ooo`3oool01`3oool0
01L0oooo00<000000?ooo`3oool0o`3oool70?ooo`005`3oool00`000000oooo0?ooo`3o0?ooo`L0
oooo000>0?ooo`@000001@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo000>0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool0o`3oool70?ooo`003`3oool00`000000oooo0?ooo`05
0?ooo`800000o`3oool80?ooo`00403oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0?l0oooo1`3oool00140oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`L0
oooo000>0?ooo`040000003oool0oooo00000?l0oooo3`3oool000l0oooo0P00003o0?oooa00oooo
0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.285976, -0.154325, \
0.0122979, 0.0124935}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["prop mode 1/a", "Subsection"],

Cell[BoxData[
    \(oma[k_, th_, a_] := 
      om /. FindRoot[\(-om^2\) + k^2 + 
              Sqrt[1 + a] 
                Realpha[1, k, k*Cos[th], 
                  IntegerPart[om*10^16]*10^\((\(-16\))\), a] \[Equal] 0, {om, 
            k + 10^\((\(-5\))\), 3/2*k}, MaxIterations \[Rule] 15]\)], "Input"],

Cell[BoxData[
    \(oma[k_, th_, a_] := 
      om /. FindRoot[\(-om^2\) + k^2 + 
              Realpha[1, k, k*Cos[th], 
                IntegerPart[om*10^16]*10^\((\(-16\))\), a] \[Equal] 0, {om, 
            k + 10^\((\(-5\))\), 3/2*k}, MaxIterations \[Rule] 15]\)], "Input"],

Cell[BoxData[
    \(\(\(old\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/1000, 1/100000, 0]\)], "Input"],

Cell[BoxData[
    \(0.5773510772466119`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(1.0000013995957555`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/1000, 1/100000, 1]\)], "Input"],

Cell[BoxData[
    \(0.5000012786321784`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%*Sqrt[Sqrt[2]]\)], "Input"],

Cell[BoxData[
    \(0.5946050780598445`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(1.029886205638109`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/100, 1/100000, 10]\)], "Input"],

Cell[BoxData[
    \(0.3392997801352588`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%*Sqrt[Sqrt[11]]\)], "Input"],

Cell[BoxData[
    \(0.6179192849151548`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(1.070267596449677`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/10, Pi/4, 10]\)], "Input"],

Cell[BoxData[
    \(0.35845731774074935`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(new\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/10*Sqrt[Sqrt[2]], 1/100000, 1]/Sqrt[Sqrt[2]]\)], "Input"],

Cell[BoxData[
    \(0.5125735438242647`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.3392803127062247*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(0.5876507396150377`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/100000, Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(0.4999999700282049`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.6285227822590245`*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(1.088633392587181`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Realpha[1, 1/1000, 1/1000, 0.339^2]\)], "Input"],

Cell[BoxData[
    \(Realpha[1, 1\/1000, 1\/1000, 0.11492100000000001`]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Realpha\)\)], "Input"],

Cell["Global`Realpha", "Print",
  CellTags->"Info3275041079-1935256"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(Realpha[md2_, k_, kz_, om_, a_] := 
                    Module[{res, mxi1, mxi2}, 
                      If[logging == 0, Print["alpha"]]; 
                      mxi1 = If[k > om, xi1[k, kz, om]]; 
                      mxi2 = If[k > om, xi2[k, kz, om]]; 
                      If[om < k, 
                        If[kz\/k < 1 - 1\/10\^10, 
                          res = \(\(1\/\(\((k\^2 - 
                                      kz\^2)\)\ 2\)\)\((md2\ \
\((NIntegrate[\(om\^2 - a\ kz\^2\ xi\^2\)\/\((1 + a\ xi\^2)\)\^2, {xi, \
\(-1\), 1}, WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30] - 
                                    NIntegrate[\(\((om + a\ kz\ xi)\)\ \
\@\(k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - om\^2)\)\ \((1 - \
kz\^2\/k\^2)\)\)\)\/\((1 + a\ xi\^2)\)\^2, {xi, \(-1\), 
                                        mxi2\ If[mxi2 < 0, 1 + 1\/10\^50, 
                                        1 - 1\/10\^50]}, 
                                      WorkingPrecision \[Rule] 30, 
                                      MaxRecursion \[Rule] 30] - 
                                    Sign[om - 
                                        kz]\ NIntegrate[\(\((om + a\ kz\ \
xi)\)\ \@\(k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - om\^2)\)\ \((1 - \
kz\^2\/k\^2)\)\)\)\/\((1 + a\ xi\^2)\)\^2, {xi, mxi1\ \((1 + 1\/10\^50)\), 
                                        1}, WorkingPrecision \[Rule] 30, 
                                        MaxRecursion \[Rule] 30])\))\)\), 
                          res = \(\(1\/\(\((k\^2 - 
                                      kz\^2)\)\ 2\)\)\((md2\ \
\((NIntegrate[\(om\^2 - a\ kz\^2\ xi\^2\)\/\((1 + a\ xi\^2)\)\^2, {xi, \
\(-1\), 1}, WorkingPrecision \[Rule] 50, MaxRecursion \[Rule] 30] - 
                                    NIntegrate[\(\((om + a\ kz\ xi)\)\ \
\@\(k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - om\^2)\)\ \((1 - \
kz\^2\/k\^2)\)\)\)\/\((1 + a\ xi\^2)\)\^2, {xi, \(-1\), 
                                        mxi2\ If[mxi2 < 0, 1 + 1\/10\^50, 
                                        1 - 1\/10\^50]}, 
                                      WorkingPrecision \[Rule] 50, 
                                      MaxRecursion \[Rule] 30] - 
                                    Sign[om - 
                                        kz]\ NIntegrate[\(\((om + a\ kz\ \
xi)\)\ \@\(k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - om\^2)\)\ \((1 - \
kz\^2\/k\^2)\)\)\)\/\((1 + a\ xi\^2)\)\^2, {xi, mxi1\ \((1 + 1\/10\^50)\), 
                                        1}, WorkingPrecision \[Rule] 50, 
                                        MaxRecursion \[Rule] 30])\))\)\)]]; 
                      If[om > k, 
                        If[kz\/k < 1 - 1\/10\^10, 
                          res = \(\(1\/\(\((k\^2 - 
                                      kz\^2)\)\ 2\)\)\((md2\ \
\((NIntegrate[\(om\^2 - a\ kz\^2\ xi\^2\)\/\((1 + a\ xi\^2)\)\^2, {xi, \
\(-1\), 1}, WorkingPrecision \[Rule] 30, MaxRecursion \[Rule] 30] - 
                                    NIntegrate[\(\((om + a\ kz\ xi)\)\ \
Sign[om - kz\ xi]\ \@\(k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - \
om\^2)\)\ \((1 - kz\^2\/k\^2)\)\)\)\/\((1 + a\ xi\^2)\)\^2, {xi, \(-1\), 1}, 
                                      WorkingPrecision \[Rule] 30, 
                                      MaxRecursion \[Rule] 30])\))\)\), 
                          res = \(\(1\/\(\((k\^2 - 
                                      kz\^2)\)\ 2\)\)\((md2\ \
\((NIntegrate[\(om\^2 - a\ kz\^2\ xi\^2\)\/\((1 + a\ xi\^2)\)\^2, {xi, \
\(-1\), 1}, WorkingPrecision \[Rule] 50, MaxRecursion \[Rule] 30] - 
                                    NIntegrate[\(\((om + a\ kz\ xi)\)\ \
Sign[om - kz\ xi]\ \@\(k\^2\ \((xi - \(om\ kz\)\/k\^2)\)\^2 - \((k\^2 - \
om\^2)\)\ \((1 - kz\^2\/k\^2)\)\)\)\/\((1 + a\ xi\^2)\)\^2, {xi, \(-1\), 1}, 
                                      WorkingPrecision \[Rule] 50, 
                                      MaxRecursion \[Rule] 30])\))\)\)]]; 
                      res]\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "Realpha"],
      Editable->False]], "Print",
  CellTags->"Info3275041079-1935256"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, oma[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, 1]}, {T, 
        1/10, 3, 1/10}]\)], "Input"],

Cell[BoxData[
    \({{1\/10, 0.5122581958767373`}, {1\/5, 0.5466392926136814`}, {3\/10, 
        0.5979209674740262`}, {2\/5, 0.6610621781886858`}, {1\/2, 
        0.7323852352786566`}, {3\/5, 0.8094519462508853`}, {7\/10, 
        0.8906731974090144`}, {4\/5, 0.9749925725551474`}, {9\/10, 
        1.0616866081863456`}, {1, 1.1502443871615995`}, {11\/10, 
        1.240294493613944`}, {6\/5, 1.3315599189115386`}, {13\/10, 
        1.423829750388172`}, {7\/5, 1.5169391226751445`}, {3\/2, 
        1.6107585244775722`}, {8\/5, 1.705183867412974`}, {17\/10, 
        1.8001306191954072`}, {9\/5, 1.8955293595409881`}, {19\/10, 
        1.9913225112812618`}, {2, 2.087461895298995`}, {21\/10, 
        2.1839068740488163`}, {11\/5, 2.2806229228851715`}, {23\/10, 
        2.377580319129595`}, {12\/5, 2.4747540784023596`}, {5\/2, 
        2.5721220059594447`}, {13\/5, 2.6696649944194966`}, {27\/10, 
        2.7673663505940858`}, {14\/5, 2.8652114261071135`}, {29\/10, 
        2.963187314004124`}, {3, 3.0612825979274025`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.6285227822590245`*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(1.088633392587181`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, oma[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, 10]}, {T, 
        1/10, 2, 1/20}]\)], "Input"],

Cell[BoxData[
    \({{1\/10, 0.6285227822590245`}, {3\/20, 0.6418253014660881`}, {1\/5, 
        0.6596825224651927`}, {1\/4, 0.6815116301902829`}, {3\/10, 
        0.7067348966272706`}, {7\/20, 0.7348264786924974`}, {2\/5, 
        0.7653325013768894`}, {9\/20, 0.7978710852258526`}, {1\/2, 
        0.8321281601133856`}, {11\/20, 0.8678467409571169`}, {3\/5, 
        0.9048177936310859`}, {13\/20, 0.9428682814435823`}, {7\/10, 
        0.981858744056064`}, {3\/4, 1.0216721376697449`}, {4\/5, 
        1.0622114116059376`}, {17\/20, 1.1033951670318816`}, {9\/10, 
        1.1451545811217314`}, {19\/20, 1.187430995860855`}, {1, 
        1.2301740400218613`}, {21\/20, 1.2733402070004838`}, {11\/10, 
        1.3168916560598611`}, {23\/20, 1.3607952707632236`}, {6\/5, 
        1.4050219023314328`}, {5\/4, 1.4495457569302108`}, {13\/10, 
        1.4943438962298243`}, {27\/20, 1.5393958275905155`}, {7\/5, 
        1.5846831655711444`}, {29\/20, 1.6301896287602424`}, {3\/2, 
        1.675899710513082`}, {31\/20, 1.7218000882649886`}, {8\/5, 
        1.7678784098362217`}, {33\/20, 1.81412340493137`}, {17\/10, 
        1.8605247628572563`}, {7\/4, 1.9070730273336918`}, {9\/5, 
        1.953759505571722`}, {37\/20, 2.00057618932451`}, {19\/10, 
        2.04751568603679`}, {39\/20, 2.0945711585543614`}, {2, 
        2.1417362721240307`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[IntegerPart[0.001*10^16]*10^\((\(-16\))\), Pi/4, 10]\)], "Input"],

Cell[BoxData[
    \(0.617512070403812`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.001, 0.617512070403812}, {1\/10, 0.6285227822590245`}, {3\/20, 
        0.6418253014660881`}, {1\/5, 0.6596825224651927`}, {1\/4, 
        0.6815116301902829`}, {3\/10, 0.7067348966272706`}, {7\/20, 
        0.7348264786924974`}, {2\/5, 0.7653325013768894`}, {9\/20, 
        0.7978710852258526`}, {1\/2, 0.8321281601133856`}, {11\/20, 
        0.8678467409571169`}, {3\/5, 0.9048177936310859`}, {13\/20, 
        0.9428682814435823`}, {7\/10, 0.981858744056064`}, {3\/4, 
        1.0216721376697449`}, {4\/5, 1.0622114116059376`}, {17\/20, 
        1.1033951670318816`}, {9\/10, 1.1451545811217314`}, {19\/20, 
        1.187430995860855`}, {1, 1.2301740400218613`}, {21\/20, 
        1.2733402070004838`}, {11\/10, 1.3168916560598611`}, {23\/20, 
        1.3607952707632236`}, {6\/5, 1.4050219023314328`}, {5\/4, 
        1.4495457569302108`}, {13\/10, 1.4943438962298243`}, {27\/20, 
        1.5393958275905155`}, {7\/5, 1.5846831655711444`}, {29\/20, 
        1.6301896287602424`}, {3\/2, 1.675899710513082`}, {31\/20, 
        1.7218000882649886`}, {8\/5, 1.7678784098362217`}, {33\/20, 
        1.81412340493137`}, {17\/10, 1.8605247628572563`}, {7\/4, 
        1.9070730273336918`}, {9\/5, 1.953759505571722`}, {37\/20, 
        2.00057618932451`}, {19\/10, 2.04751568603679`}, {39\/20, 
        2.0945711585543614`}, {2, 2.1417362721240307`}}\)], "Input"],

Cell[BoxData[
    \({{0.001`, 0.617512070403812`}, {1\/10, 0.6285227822590245`}, {3\/20, 
        0.6418253014660881`}, {1\/5, 0.6596825224651927`}, {1\/4, 
        0.6815116301902829`}, {3\/10, 0.7067348966272706`}, {7\/20, 
        0.7348264786924974`}, {2\/5, 0.7653325013768894`}, {9\/20, 
        0.7978710852258526`}, {1\/2, 0.8321281601133856`}, {11\/20, 
        0.8678467409571169`}, {3\/5, 0.9048177936310859`}, {13\/20, 
        0.9428682814435823`}, {7\/10, 0.981858744056064`}, {3\/4, 
        1.0216721376697449`}, {4\/5, 1.0622114116059376`}, {17\/20, 
        1.1033951670318816`}, {9\/10, 1.1451545811217314`}, {19\/20, 
        1.187430995860855`}, {1, 1.2301740400218613`}, {21\/20, 
        1.2733402070004838`}, {11\/10, 1.3168916560598611`}, {23\/20, 
        1.3607952707632236`}, {6\/5, 1.4050219023314328`}, {5\/4, 
        1.4495457569302108`}, {13\/10, 1.4943438962298243`}, {27\/20, 
        1.5393958275905155`}, {7\/5, 1.5846831655711444`}, {29\/20, 
        1.6301896287602424`}, {3\/2, 1.675899710513082`}, {31\/20, 
        1.7218000882649886`}, {8\/5, 1.7678784098362217`}, {33\/20, 
        1.81412340493137`}, {17\/10, 1.8605247628572563`}, {7\/4, 
        1.9070730273336918`}, {9\/5, 1.953759505571722`}, {37\/20, 
        2.00057618932451`}, {19\/10, 2.04751568603679`}, {39\/20, 
        2.0945711585543614`}, {2, 2.1417362721240307`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True, 
      PlotStyle \[Rule] {Dashing[{0.002, 0.01}]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.223747 0.386166 [
[.2619 .14992 -9 -9 ]
[.2619 .14992 9 0 ]
[.5 .14992 -3 -9 ]
[.5 .14992 3 0 ]
[.7381 .14992 -9 -9 ]
[.7381 .14992 9 0 ]
[.97619 .14992 -3 -9 ]
[.97619 .14992 3 0 ]
[.01131 .06588 -24 -4.5 ]
[.01131 .06588 0 4.5 ]
[.01131 .25896 -24 -4.5 ]
[.01131 .25896 0 4.5 ]
[.01131 .3555 -18 -4.5 ]
[.01131 .3555 0 4.5 ]
[.01131 .45204 -24 -4.5 ]
[.01131 .45204 0 4.5 ]
[.01131 .54859 -6 -4.5 ]
[.01131 .54859 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .16242 m
.2619 .16867 L
s
[(0.5)] .2619 .14992 0 1 Mshowa
.5 .16242 m
.5 .16867 L
s
[(1)] .5 .14992 0 1 Mshowa
.7381 .16242 m
.7381 .16867 L
s
[(1.5)] .7381 .14992 0 1 Mshowa
.97619 .16242 m
.97619 .16867 L
s
[(2)] .97619 .14992 0 1 Mshowa
.125 Mabswid
.07143 .16242 m
.07143 .16617 L
s
.11905 .16242 m
.11905 .16617 L
s
.16667 .16242 m
.16667 .16617 L
s
.21429 .16242 m
.21429 .16617 L
s
.30952 .16242 m
.30952 .16617 L
s
.35714 .16242 m
.35714 .16617 L
s
.40476 .16242 m
.40476 .16617 L
s
.45238 .16242 m
.45238 .16617 L
s
.54762 .16242 m
.54762 .16617 L
s
.59524 .16242 m
.59524 .16617 L
s
.64286 .16242 m
.64286 .16617 L
s
.69048 .16242 m
.69048 .16617 L
s
.78571 .16242 m
.78571 .16617 L
s
.83333 .16242 m
.83333 .16617 L
s
.88095 .16242 m
.88095 .16617 L
s
.92857 .16242 m
.92857 .16617 L
s
.25 Mabswid
0 .16242 m
1 .16242 L
s
.02381 .06588 m
.03006 .06588 L
s
[(0.75)] .01131 .06588 1 0 Mshowa
.02381 .25896 m
.03006 .25896 L
s
[(1.25)] .01131 .25896 1 0 Mshowa
.02381 .3555 m
.03006 .3555 L
s
[(1.5)] .01131 .3555 1 0 Mshowa
.02381 .45204 m
.03006 .45204 L
s
[(1.75)] .01131 .45204 1 0 Mshowa
.02381 .54859 m
.03006 .54859 L
s
[(2)] .01131 .54859 1 0 Mshowa
.125 Mabswid
.02381 .08519 m
.02756 .08519 L
s
.02381 .10449 m
.02756 .10449 L
s
.02381 .1238 m
.02756 .1238 L
s
.02381 .14311 m
.02756 .14311 L
s
.02381 .18173 m
.02756 .18173 L
s
.02381 .20104 m
.02756 .20104 L
s
.02381 .22034 m
.02756 .22034 L
s
.02381 .23965 m
.02756 .23965 L
s
.02381 .27827 m
.02756 .27827 L
s
.02381 .29758 m
.02756 .29758 L
s
.02381 .31689 m
.02756 .31689 L
s
.02381 .33619 m
.02756 .33619 L
s
.02381 .37481 m
.02756 .37481 L
s
.02381 .39412 m
.02756 .39412 L
s
.02381 .41343 m
.02756 .41343 L
s
.02381 .43274 m
.02756 .43274 L
s
.02381 .47135 m
.02756 .47135 L
s
.02381 .49066 m
.02756 .49066 L
s
.02381 .50997 m
.02756 .50997 L
s
.02381 .52928 m
.02756 .52928 L
s
.02381 .04657 m
.02756 .04657 L
s
.02381 .02726 m
.02756 .02726 L
s
.02381 .00795 m
.02756 .00795 L
s
.02381 .56789 m
.02756 .56789 L
s
.02381 .5872 m
.02756 .5872 L
s
.02381 .60651 m
.02756 .60651 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .002 .01 ] 0 setdash
.02429 .01472 m
.07143 .01897 L
.09524 .0241 L
.11905 .031 L
.14286 .03943 L
.16667 .04917 L
.19048 .06002 L
.21429 .0718 L
.2381 .08436 L
.2619 .09759 L
.28571 .11139 L
.30952 .12566 L
.33333 .14036 L
.35714 .15541 L
.38095 .17079 L
.40476 .18644 L
.42857 .20235 L
.45238 .21847 L
.47619 .2348 L
.5 .2513 L
.52381 .26797 L
.54762 .28479 L
.57143 .30175 L
.59524 .31882 L
.61905 .33602 L
.64286 .35332 L
.66667 .37072 L
.69048 .3882 L
.71429 .40578 L
.7381 .42343 L
.7619 .44115 L
.78571 .45895 L
.80952 .47681 L
.83333 .49472 L
.85714 .5127 L
.88095 .53073 L
.90476 .54881 L
.92857 .56693 L
.95238 .58511 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`0200000ooo0000003mool001_oo`03003ooooo00Koo`8000;oo`03003ooooo0?Goo`006ooo00<0
0?ooool03Ooo00@00?ooool00?7oo`006ooo0P005?oo00<00?ooool0k?oo000Kool00`00ooooo`0F
ool2003Zool001_oo`03003ooooo01Woo`800>Ooo`006ooo00<00?ooool07?oo0P00i?oo000Kool0
0`00ooooo`0Oool00`00ooooo`3Pool001_oo`80023oo`03003ooooo0>3oo`006ooo00<00?ooool0
8_oo00<00?ooool0gOoo000Kool00`00ooooo`0Uool00`00ooooo`3Jool000;oo`8000Coo`03003o
oooo00?oo`03003ooooo00;oo`<000Goo`03003ooooo02Soo`03003ooooo0=Ooo`0000Goo`00oooo
o`0000Woo`03003ooooo00Goo`03003ooooo00;oo`03003ooooo0?ooo`?oo`0000Goo`00ooooo`00
00[oo`03003ooooo00Coo`03003ooooo00;oo`8002coo`03003ooooo0=Coo`0000Goo`00ooooo`00
00[oo`04003oooooool30005ool00`00ooooo`0^ool00`00ooooo`3Aool00005ool00?ooool0000;
ool01000ooooo`001ooo00<00?ooool0<?oo00<00?ooool0cooo00001Ooo003ooooo00002?oo00@0
0?ooool000;oo`03003ooooo00Goo`03003ooooo037oo`03003ooooo0<koo`000_oo0P002Ooo1000
0_oo10001?oo00<00?ooool0<ooo0P00cOoo000Kool2000gool00`00ooooo`39ool001_oo`03003o
oooo03Koo`03003ooooo0<Woo`006ooo00<00?ooool0>Ooo00<00?ooool0a_oo000Kool00`00oooo
o`3oool3ool001_oo`03003ooooo03coo`03003ooooo0<?oo`006ooo00<00?ooool0?_oo0P00`_oo
000Kool2003oool4ool001_oo`03003ooooo047oo`800;ooo`006ooo00<00?ooool0A?oo00<00?oo
ool0^ooo000Kool00`00ooooo`14ool00`00ooooo`2kool001_oo`03003ooooo04Ooo`03003ooooo
0;Soo`006ooo0P00=_oo0P001?oo00<00?ooool00_oo0`00=_oo1000=_oo10000ooo00<00?ooool0
0_oo0`00=_oo10001_oo000Kool00`00ooooo`0dool01000ooooo`002ooo00<00?ooool00ooo0P00
<?oo00<00?ooool0=ooo00<00?ooool02_oo00<00?ooool0<ooo00<00?ooool01ooo000Kool00`00
ooooo`0dool01000ooooo`002ooo00<00?ooool01_oo00<00?ooool0;?oo00<00?ooool0=ooo00<0
0?ooool02_oo00<00?ooool0=?oo00<00?ooool01_oo000Kool00`00ooooo`0dool01000ooooo`00
2?oo0`002_oo00<00?ooool0:ooo00<00?ooool0=ooo00<00?ooool01ooo0`00>?oo00<00?ooool0
1Ooo000Kool00`00ooooo`0dool01000ooooo`002?oo00<00?ooool03?oo00<00?ooool0:Ooo00<0
0?ooool0=ooo00<00?ooool01ooo00<00?ooool0>Ooo00<00?ooool01?oo000Kool2000eool01000
ooooo`002?oo00<00?ooool0=_oo0`00=ooo0`002Ooo00<00?ooool0=_oo00@00?ooool000Koo`00
6ooo00<00?ooool0=Ooo0P002Ooo10003Ooo0P00:?oo00<00?ooool0=ooo00<00?ooool01ooo1000
=_oo0P001ooo000Kool00`00ooooo`1Dool00`00ooooo`2[ool001_oo`03003ooooo05Goo`03003o
oooo0:[oo`006ooo00<00?ooool0Eooo00<00?ooool0Z?oo000Doooo000=000001_oo`03003ooooo
00Woo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo
00[oo`03003ooooo00Woo`03003ooooo00;oo`03003ooooo00Goo`03003ooooo00[oo`03003ooooo
00Woo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo
00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo
00[oo`03003ooooo00Goo`006ooo00<00?ooool0??oo00<00?ooool0??oo00<00?ooool0?Ooo00<0
0?ooool0??oo00<00?ooool01Ooo000Kool00`00ooooo`1Lool2002Tool001_oo`03003ooooo05oo
o`03003ooooo0:3oo`006ooo0P00H?oo00<00?ooool0X?oo000Kool00`00ooooo`1Rool00`00oooo
o`2Mool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0I?oo0P00W?oo000Kool00`00oooo
o`3oool3ool001_oo`8006Soo`03003ooooo09Soo`006ooo00<00?ooool0J_oo00<00?ooool0UOoo
000Kool00`00ooooo`1Zool00`00ooooo`2Eool001_oo`03003ooooo06coo`03003ooooo09?oo`00
6ooo00<00?ooool0KOoo00<00?ooool0T_oo000Kool2001`ool00`00ooooo`2@ool001_oo`03003o
oooo0?ooo`?oo`006ooo00<00?ooool0LOoo0P00Sooo000Kool00`00ooooo`3oool3ool001_oo`03
003ooooo07Coo`8008coo`006ooo00<00?ooool0Mooo00<00?ooool0R?oo000Kool2001hool00`00
ooooo`28ool001_oo`03003ooooo07Woo`03003ooooo08Koo`006ooo00<00?ooool0N_oo00<00?oo
ool0QOoo00000ooo000000020003ool00`00ooooo`02ool40002ool30005ool00`00ooooo`1lool0
0`00ooooo`23ool000?oo`03003ooooo00Ooo`03003ooooo00Koo`03003ooooo00;oo`03003ooooo
0?ooo`?oo`000ooo00<00?ooool02?oo00<00?ooool01Ooo00<00?ooool00_oo0P00P?oo00<00?oo
ool0P?oo0003ool00`00ooooo`09ool01000oooooooo0`001Ooo00<00?ooool0oooo0ooo0003ool0
0`00ooooo`0:ool01000ooooo`001ooo00<00?ooool0POoo0P00Oooo00001?oo000000002Ooo00@0
0?ooool000;oo`03003ooooo00Goo`03003ooooo0?ooo`?oo`000ooo00<00?ooool02?oo0P000ooo
10001?oo00<00?ooool0Q?oo00<00?ooool0Nooo000Kool20027ool00`00ooooo`1iool001_oo`03
003ooooo08Ooo`03003ooooo07Soo`006ooo00<00?ooool0ROoo00<00?ooool0M_oo000Kool00`00
ooooo`29ool00`00ooooo`1fool001_oo`03003ooooo08coo`03003ooooo07?oo`006ooo0P00oooo
1?oo000Kool00`00ooooo`2>ool2001bool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0
TOoo00<00?ooool0K_oo000Kool00`00ooooo`3oool3ool001_oo`8009Coo`8006goo`006ooo00<0
0?ooool0oooo0ooo000Kool00`00ooooo`2Fool00`00ooooo`1Yool001_oo`03003ooooo09Woo`03
003ooooo06Koo`006ooo00<00?ooool0VOoo00<00?ooool0I_oo000Kool2002Lool00`00ooooo`1T
ool001_oo`03003ooooo09coo`03003ooooo06?oo`006ooo00<00?ooool0W_oo00<00?ooool0HOoo
0007ool40003ool00`00ooooo`02ool30005ool00`00ooooo`3oool3ool000Woo`03003ooooo00[o
o`03003ooooo00;oo`03003ooooo0:3oo`80063oo`002Ooo00<00?ooool02_oo00<00?ooool00_oo
0P00oooo1?oo0009ool00`00ooooo`07ool30005ool00`00ooooo`2Sool00`00ooooo`1Lool000Wo
o`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0?ooo`?oo`001ooo0`002Ooo00<00?ooool0
1Ooo00<00?ooool0YOoo0P00Fooo0009ool00`00ooooo`07ool40004ool00`00ooooo`3oool3ool0
01_oo`03003ooooo0:Soo`03003ooooo05Ooo`006ooo0P00oooo1?oo000Kool00`00ooooo`2[ool0
0`00ooooo`1Dool001_oo`03003ooooo0:goo`03003ooooo05;oo`006ooo00<00?ooool0[_oo00<0
0?ooool0DOoo000Kool00`00ooooo`2`ool00`00ooooo`1?ool001_oo`800;7oo`03003ooooo04oo
o`006ooo00<00?ooool0/_oo00<00?ooool0COoo000Kool00`00ooooo`2cool00`00ooooo`1<ool0
01_oo`03003ooooo0;Goo`03003ooooo04[oo`006ooo00<00?ooool0oooo0ooo000Kool2002hool2
0019ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0^_oo00<00?ooool0AOoo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0;goo`03003ooooo04;oo`006ooo0P00oooo1?oo000K
ool00`00ooooo`2oool20011ool001_oo`03003ooooo0?ooo`?oo`0000?oo`0000000P000ooo00<0
0?ooool00ooo00<00?ooool00_oo0`001Ooo00<00?ooool0`_oo00<00?ooool0?Ooo0003ool00`00
ooooo`08ool00`00ooooo`05ool00`00ooooo`02ool00`00ooooo`3oool3ool000?oo`03003ooooo
00Woo`03003ooooo00Coo`03003ooooo00;oo`800<Goo`8003coo`000ooo00<00?ooool02Ooo00@0
0?ooooooo`<000Goo`03003ooooo0<Ooo`03003ooooo03Soo`000ooo00<00?ooool02_oo00@00?oo
ool000Ooo`03003ooooo0<Ooo`03003ooooo03Soo`0000Coo`00000000Woo`04003ooooo0002ool0
0`00ooooo`05ool00`00ooooo`39ool00`00ooooo`0fool000?oo`03003ooooo00Ooo`@000;oo`@0
00Coo`03003ooooo0<[oo`03003ooooo03Goo`006ooo0P00cOoo00<00?ooool0<ooo000Kool00`00
ooooo`3<ool00`00ooooo`0cool001_oo`03003ooooo0<koo`03003ooooo037oo`006ooo00<00?oo
ool0cooo00<00?ooool0<?oo000Kool00`00ooooo`3Aool00`00ooooo`0^ool001_oo`800?ooo`Co
o`006ooo00<00?ooool0e?oo00<00?ooool0:ooo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0=Koo`8002[oo`006ooo00<00?ooool0oooo0ooo000Kool2003Jool00`00ooooo`0Vool001_o
o`03003ooooo0?ooo`?oo`006ooo00<00?ooool0fooo0P009Ooo000Kool00`00ooooo`3oool3ool0
01_oo`03003ooooo0=koo`03003ooooo027oo`006ooo00<00?ooool0oooo0ooo000Kool2003Qool2
000Pool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0hooo00<00?ooool07?oo000Cool4
0004ool00`00ooooo`3oool3ool001?oo`03003ooooo00Goo`03003ooooo0>Goo`8001_oo`005?oo
00<00?ooool01?oo0P00jOoo00<00?ooool05ooo000Eool00`00ooooo`03ool00`00ooooo`3Xool0
0`00ooooo`0Gool001Koo`03003ooooo00;oo`03003ooooo0>[oo`03003ooooo01Goo`004ooo00@0
0?ooool000Coo`03003ooooo0>_oo`03003ooooo01Coo`005?oo0P001Ooo00<00?ooool0kOoo00<0
0?ooool04_oo000Kool2003^ool00`00ooooo`0Bool001_oo`03003ooooo0?3oo`03003ooooo00oo
o`006ooo00<00?ooool0l?oo00<00?ooool03ooo000Kool00`00ooooo`3bool00`00ooooo`0=ool0
01_oo`03003ooooo0??oo`03003ooooo00coo`006ooo0P00m_oo00<00?ooool02_oo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?Ooo`8000Woo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3jool00`00ooooo`05ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.215729, 0.518378, \
0.00789457, 0.00973497}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        oma[IntegerPart[T*10^16]*10^\((\(-16\))\), 10^\((\(-5\))\), 10]}, {T, 
        1/10, 2, 1/20}]\)], "Input"],

Cell[BoxData[
    \({{1\/10, 0.6296051634586106`}, {3\/20, 0.6440606264946983`}, {1\/5, 
        0.6632423749281585`}, {1\/4, 0.6864035761114252`}, {3\/10, 
        0.7128535701926363`}, {7\/20, 0.7420035046742031`}, {2\/5, 
        0.773374429996941`}, {9\/20, 0.8065852911289606`}, {1\/2, 
        0.8413360046795553`}, {11\/20, 0.877391806409354`}, {3\/5, 
        0.9145648581849953`}, {13\/20, 0.9527062788247888`}, {7\/10, 
        0.9916952180132482`}, {3\/4, 1.03143409342898`}, {4\/5, 
        1.0718402730013918`}, {17\/20, 1.1128456802229016`}, {9\/10, 
        1.1543923249225678`}, {19\/20, 1.1964305256229273`}, {1, 
        1.2389173536477835`}, {21\/20, 1.281815230711901`}, {11\/10, 
        1.3250911847978508`}, {23\/20, 1.36871598034328`}, {6\/5, 
        1.4126635527521552`}, {5\/4, 1.456910535157138`}, {13\/10, 
        1.5014358761727018`}, {27\/20, 1.546220527963972`}, {7\/5, 
        1.5912471891573159`}, {29\/20, 1.636500090926236`}, {3\/2, 
        1.6819648173885609`}, {31\/20, 1.7276281535260862`}, {8\/5, 
        1.7734779553844942`}, {33\/20, 1.8195030384720534`}, {17\/10, 
        1.8656930811527754`}, {7\/4, 1.9120385404973934`}, {9\/5, 
        1.9585305785674652`}, {37\/20, 2.005160997503284`}, {19\/10, 
        2.051922182093907`}, {39\/20, 2.098807048748694`}, {2, 
        2.145808999980101`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/1000, 10^\((\(-5\))\), 10]\)], "Input"],

Cell[BoxData[
    \(0.6175114910318721`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.001, 0.6175114910318721`}, {1\/10, 0.6296051634586106`}, {3\/20, 
        0.6440606264946983`}, {1\/5, 0.6632423749281585`}, {1\/4, 
        0.6864035761114252`}, {3\/10, 0.7128535701926363`}, {7\/20, 
        0.7420035046742031`}, {2\/5, 0.773374429996941`}, {9\/20, 
        0.8065852911289606`}, {1\/2, 0.8413360046795553`}, {11\/20, 
        0.877391806409354`}, {3\/5, 0.9145648581849953`}, {13\/20, 
        0.9527062788247888`}, {7\/10, 0.9916952180132482`}, {3\/4, 
        1.03143409342898`}, {4\/5, 1.0718402730013918`}, {17\/20, 
        1.1128456802229016`}, {9\/10, 1.1543923249225678`}, {19\/20, 
        1.1964305256229273`}, {1, 1.2389173536477835`}, {21\/20, 
        1.281815230711901`}, {11\/10, 1.3250911847978508`}, {23\/20, 
        1.36871598034328`}, {6\/5, 1.4126635527521552`}, {5\/4, 
        1.456910535157138`}, {13\/10, 1.5014358761727018`}, {27\/20, 
        1.546220527963972`}, {7\/5, 1.5912471891573159`}, {29\/20, 
        1.636500090926236`}, {3\/2, 1.6819648173885609`}, {31\/20, 
        1.7276281535260862`}, {8\/5, 1.7734779553844942`}, {33\/20, 
        1.8195030384720534`}, {17\/10, 1.8656930811527754`}, {7\/4, 
        1.9120385404973934`}, {9\/5, 1.9585305785674652`}, {37\/20, 
        2.005160997503284`}, {19\/10, 2.051922182093907`}, {39\/20, 
        2.098807048748694`}, {2, 2.145808999980101`}}\)], "Input"],

Cell[BoxData[
    \({{0.001`, 0.6175114910318721`}, {1\/10, 0.6296051634586106`}, {3\/20, 
        0.6440606264946983`}, {1\/5, 0.6632423749281585`}, {1\/4, 
        0.6864035761114252`}, {3\/10, 0.7128535701926363`}, {7\/20, 
        0.7420035046742031`}, {2\/5, 0.773374429996941`}, {9\/20, 
        0.8065852911289606`}, {1\/2, 0.8413360046795553`}, {11\/20, 
        0.877391806409354`}, {3\/5, 0.9145648581849953`}, {13\/20, 
        0.9527062788247888`}, {7\/10, 0.9916952180132482`}, {3\/4, 
        1.03143409342898`}, {4\/5, 1.0718402730013918`}, {17\/20, 
        1.1128456802229016`}, {9\/10, 1.1543923249225678`}, {19\/20, 
        1.1964305256229273`}, {1, 1.2389173536477835`}, {21\/20, 
        1.281815230711901`}, {11\/10, 1.3250911847978508`}, {23\/20, 
        1.36871598034328`}, {6\/5, 1.4126635527521552`}, {5\/4, 
        1.456910535157138`}, {13\/10, 1.5014358761727018`}, {27\/20, 
        1.546220527963972`}, {7\/5, 1.5912471891573159`}, {29\/20, 
        1.636500090926236`}, {3\/2, 1.6819648173885609`}, {31\/20, 
        1.7276281535260862`}, {8\/5, 1.7734779553844942`}, {33\/20, 
        1.8195030384720534`}, {17\/10, 1.8656930811527754`}, {7\/4, 
        1.9120385404973934`}, {9\/5, 1.9585305785674652`}, {37\/20, 
        2.005160997503284`}, {19\/10, 2.051922182093907`}, {39\/20, 
        2.098807048748694`}, {2, 2.145808999980101`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True, 
      PlotStyle \[Rule] {Dashing[{0.002, 0.01}]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.223111 0.385137 [
[.2619 .14953 -9 -9 ]
[.2619 .14953 9 0 ]
[.5 .14953 -3 -9 ]
[.5 .14953 3 0 ]
[.7381 .14953 -9 -9 ]
[.7381 .14953 9 0 ]
[.97619 .14953 -3 -9 ]
[.97619 .14953 3 0 ]
[.01131 .06574 -24 -4.5 ]
[.01131 .06574 0 4.5 ]
[.01131 .25831 -24 -4.5 ]
[.01131 .25831 0 4.5 ]
[.01131 .35459 -18 -4.5 ]
[.01131 .35459 0 4.5 ]
[.01131 .45088 -24 -4.5 ]
[.01131 .45088 0 4.5 ]
[.01131 .54716 -6 -4.5 ]
[.01131 .54716 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .16203 m
.2619 .16828 L
s
[(0.5)] .2619 .14953 0 1 Mshowa
.5 .16203 m
.5 .16828 L
s
[(1)] .5 .14953 0 1 Mshowa
.7381 .16203 m
.7381 .16828 L
s
[(1.5)] .7381 .14953 0 1 Mshowa
.97619 .16203 m
.97619 .16828 L
s
[(2)] .97619 .14953 0 1 Mshowa
.125 Mabswid
.07143 .16203 m
.07143 .16578 L
s
.11905 .16203 m
.11905 .16578 L
s
.16667 .16203 m
.16667 .16578 L
s
.21429 .16203 m
.21429 .16578 L
s
.30952 .16203 m
.30952 .16578 L
s
.35714 .16203 m
.35714 .16578 L
s
.40476 .16203 m
.40476 .16578 L
s
.45238 .16203 m
.45238 .16578 L
s
.54762 .16203 m
.54762 .16578 L
s
.59524 .16203 m
.59524 .16578 L
s
.64286 .16203 m
.64286 .16578 L
s
.69048 .16203 m
.69048 .16578 L
s
.78571 .16203 m
.78571 .16578 L
s
.83333 .16203 m
.83333 .16578 L
s
.88095 .16203 m
.88095 .16578 L
s
.92857 .16203 m
.92857 .16578 L
s
.25 Mabswid
0 .16203 m
1 .16203 L
s
.02381 .06574 m
.03006 .06574 L
s
[(0.75)] .01131 .06574 1 0 Mshowa
.02381 .25831 m
.03006 .25831 L
s
[(1.25)] .01131 .25831 1 0 Mshowa
.02381 .35459 m
.03006 .35459 L
s
[(1.5)] .01131 .35459 1 0 Mshowa
.02381 .45088 m
.03006 .45088 L
s
[(1.75)] .01131 .45088 1 0 Mshowa
.02381 .54716 m
.03006 .54716 L
s
[(2)] .01131 .54716 1 0 Mshowa
.125 Mabswid
.02381 .085 m
.02756 .085 L
s
.02381 .10426 m
.02756 .10426 L
s
.02381 .12351 m
.02756 .12351 L
s
.02381 .14277 m
.02756 .14277 L
s
.02381 .18128 m
.02756 .18128 L
s
.02381 .20054 m
.02756 .20054 L
s
.02381 .2198 m
.02756 .2198 L
s
.02381 .23905 m
.02756 .23905 L
s
.02381 .27757 m
.02756 .27757 L
s
.02381 .29682 m
.02756 .29682 L
s
.02381 .31608 m
.02756 .31608 L
s
.02381 .33534 m
.02756 .33534 L
s
.02381 .37385 m
.02756 .37385 L
s
.02381 .39311 m
.02756 .39311 L
s
.02381 .41236 m
.02756 .41236 L
s
.02381 .43162 m
.02756 .43162 L
s
.02381 .47014 m
.02756 .47014 L
s
.02381 .48939 m
.02756 .48939 L
s
.02381 .50865 m
.02756 .50865 L
s
.02381 .52791 m
.02756 .52791 L
s
.02381 .04648 m
.02756 .04648 L
s
.02381 .02723 m
.02756 .02723 L
s
.02381 .00797 m
.02756 .00797 L
s
.02381 .56642 m
.02756 .56642 L
s
.02381 .58568 m
.02756 .58568 L
s
.02381 .60493 m
.02756 .60493 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .002 .01 ] 0 setdash
.02429 .01472 m
.07143 .01937 L
.09524 .02494 L
.11905 .03233 L
.14286 .04125 L
.16667 .05143 L
.19048 .06266 L
.21429 .07474 L
.2381 .08753 L
.2619 .10092 L
.28571 .1148 L
.30952 .12912 L
.33333 .14381 L
.35714 .15883 L
.38095 .17413 L
.40476 .18969 L
.42857 .20549 L
.45238 .22149 L
.47619 .23768 L
.5 .25404 L
.52381 .27056 L
.54762 .28723 L
.57143 .30403 L
.59524 .32096 L
.61905 .338 L
.64286 .35515 L
.66667 .3724 L
.69048 .38974 L
.71429 .40717 L
.7381 .42468 L
.7619 .44226 L
.78571 .45992 L
.80952 .47765 L
.83333 .49544 L
.85714 .51329 L
.88095 .53119 L
.90476 .54915 L
.92857 .56716 L
.95238 .58522 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`0200000ooo0000003mool001_oo`03003ooooo00Koo`8000;oo`03003ooooo0?Goo`006ooo00<0
0?ooool03Ooo00<00?ooool0l_oo000Kool2000Aool01000ooooo`00k_oo000Kool00`00ooooo`0C
ool01000ooooo`00jooo000Kool00`00ooooo`0Fool01000ooooo`00j?oo000Kool00`00ooooo`0I
ool00`00ooooo`3Vool001_oo`03003ooooo01coo`03003ooooo0>?oo`006ooo0P008?oo00<00?oo
ool0h?oo000Kool00`00ooooo`0Rool00`00ooooo`3Mool001_oo`03003ooooo02Goo`03003ooooo
0=[oo`000_oo0P001?oo00<00?ooool00ooo00<00?ooool00_oo0`001Ooo00<00?ooool09Ooo00<0
0?ooool0f_oo00001Ooo003ooooo00002Ooo00<00?ooool01Ooo00<00?ooool00_oo00<00?ooool0
:?oo00<00?ooool0eooo00001Ooo003ooooo00002_oo00<00?ooool01?oo00<00?ooool00_oo0P00
:ooo0P00e_oo00001Ooo003ooooo00002_oo00@00?ooooooo`<000Goo`03003ooooo0?ooo`?oo`00
00Goo`00ooooo`0000_oo`04003ooooo0007ool00`00ooooo`0]ool2003Cool00005ool00?ooool0
0008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0<?oo0P00d?oo0002ool20009ool4
0002ool40004ool00`00ooooo`3oool3ool001_oo`8003Coo`03003ooooo0<coo`006ooo00<00?oo
ool0=_oo00<00?ooool0bOoo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo03Woo`03003o
oooo0<Koo`006ooo00<00?ooool0>ooo00<00?ooool0a?oo000Kool00`00ooooo`0lool00`00oooo
o`33ool001_oo`8003ooo`800<;oo`006ooo00<00?ooool0@Ooo00<00?ooool0__oo000Kool00`00
ooooo`11ool00`00ooooo`2nool001_oo`03003ooooo04Coo`03003ooooo0;_oo`006ooo00<00?oo
ool0oooo0ooo000Kool2000fool20004ool00`00ooooo`02ool30003ool2000aool4000fool40003
ool00`00ooooo`02ool3000fool40006ool001_oo`03003ooooo03Coo`04003ooooo000;ool00`00
ooooo`03ool00`00ooooo`0_ool00`00ooooo`0gool00`00ooooo`0:ool00`00ooooo`0cool00`00
ooooo`07ool001_oo`03003ooooo03Coo`04003ooooo000;ool00`00ooooo`04ool00`00ooooo`0^
ool00`00ooooo`0gool00`00ooooo`0:ool00`00ooooo`0dool00`00ooooo`06ool001_oo`03003o
oooo03Coo`04003ooooo0008ool30009ool00`00ooooo`0/ool00`00ooooo`0gool00`00ooooo`07
ool3000hool00`00ooooo`05ool001_oo`03003ooooo03Coo`04003ooooo0008ool00`00ooooo`0h
ool00`00ooooo`0gool00`00ooooo`07ool00`00ooooo`0iool00`00ooooo`04ool001_oo`8003Go
o`04003ooooo0008ool00`00ooooo`0<ool00`00ooooo`0Wool3000gool30009ool00`00ooooo`0f
ool01000ooooo`001_oo000Kool00`00ooooo`0eool20009ool4000=ool00`00ooooo`0Wool00`00
ooooo`0gool00`00ooooo`07ool4000fool20007ool001_oo`03003ooooo05;oo`03003ooooo0:go
o`006ooo00<00?ooool0E?oo00<00?ooool0Zooo000Kool00`00ooooo`3oool3ool001Coool000d0
00006ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?oo
ool02_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?oo
ool02Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?oo
ool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?oo
ool02_oo00<00?ooool01Ooo000Kool00`00ooooo`0lool00`00ooooo`0Jool2000Pool00`00oooo
o`0mool00`00ooooo`0lool00`00ooooo`05ool001_oo`03003ooooo05coo`03003ooooo0:?oo`00
6ooo00<00?ooool0G?oo00<00?ooool0Xooo000Kool2001Pool00`00ooooo`2Pool001_oo`03003o
oooo0?ooo`?oo`006ooo00<00?ooool0HOoo0P00Wooo000Kool00`00ooooo`3oool3ool001_oo`03
003ooooo06Coo`03003ooooo09_oo`006ooo0P00J?oo00<00?ooool0V?oo000Kool00`00ooooo`1W
ool00`00ooooo`2Hool001_oo`03003ooooo06Woo`8009Ooo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`1/ool00`00ooooo`2Cool001_oo`800?ooo`Coo`006ooo00<00?ooool0Kooo00<0
0?ooool0T?oo000Kool00`00ooooo`1aool00`00ooooo`2>ool001_oo`03003ooooo07;oo`03003o
oooo08goo`006ooo00<00?ooool0M?oo00<00?ooool0Rooo000Kool2001eool00`00ooooo`2;ool0
01_oo`03003ooooo07Ooo`03003ooooo08Soo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`1iool20027ool00003ool00000008000?oo`03003ooooo00;oo`@000;oo`<000Goo`03003ooooo
0?ooo`?oo`000ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool00_oo00<00?ooool0O?oo
00<00?ooool0Pooo0003ool00`00ooooo`08ool00`00ooooo`05ool00`00ooooo`02ool2001oool0
0`00ooooo`21ool000?oo`03003ooooo00Woo`04003oooooool30005ool00`00ooooo`1oool00`00
ooooo`20ool000?oo`03003ooooo00[oo`04003ooooo0007ool00`00ooooo`21ool00`00ooooo`1n
ool00004ool000000009ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0POoo00<00?oo
ool0O_oo0003ool00`00ooooo`08ool20003ool40004ool00`00ooooo`24ool00`00ooooo`1kool0
01_oo`800?ooo`Coo`006ooo00<00?ooool0Q_oo0P00N_oo000Kool00`00ooooo`3oool3ool001_o
o`03003ooooo08Woo`03003ooooo07Koo`006ooo00<00?ooool0oooo0ooo000Kool2002<ool2001e
ool001_oo`03003ooooo08koo`03003ooooo077oo`006ooo00<00?ooool0S_oo0P00L_oo000Kool0
0`00ooooo`2Aool00`00ooooo`1^ool001_oo`03003ooooo097oo`03003ooooo06koo`006ooo0P00
U?oo0P00KOoo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo09Koo`03003ooooo06Woo`00
6ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`2Hool2001Xool001_oo`800?ooo`Coo`006ooo
00<00?ooool0Vooo0P00IOoo000Kool00`00ooooo`3oool3ool000Ooo`@000?oo`03003ooooo00;o
o`<000Goo`03003ooooo09koo`03003ooooo067oo`002Ooo00<00?ooool02_oo00<00?ooool00_oo
00<00?ooool0X?oo00<00?ooool0Gooo0009ool00`00ooooo`0:ool00`00ooooo`02ool2002Rool0
0`00ooooo`1Nool000Woo`03003ooooo00Ooo`<000Goo`03003ooooo0:?oo`03003ooooo05coo`00
2Ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0Xooo00<00?ooool0G?oo0007ool30009
ool00`00ooooo`05ool00`00ooooo`2Uool00`00ooooo`1Jool000Woo`03003ooooo00Ooo`@000Co
o`03003ooooo0:Koo`03003ooooo05Woo`006ooo0P00ZOoo00<00?ooool0Eooo000Kool00`00oooo
o`3oool3ool001_oo`03003ooooo0:_oo`03003ooooo05Coo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`2]ool2001Cool001_oo`800?ooo`Coo`006ooo00<00?ooool0/?oo00<00?ooool0
Cooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0;;oo`8004koo`006ooo00<00?ooool0
]Ooo00<00?ooool0B_oo000Kool00`00ooooo`2eool00`00ooooo`1:ool001_oo`800;Soo`03003o
oooo04Soo`006ooo00<00?ooool0^?oo00<00?ooool0Aooo000Kool00`00ooooo`2jool00`00oooo
o`15ool001_oo`03003ooooo0;[oo`03003ooooo04Goo`006ooo00<00?ooool0_Ooo00<00?ooool0
@_oo000Kool2003oool4ool001_oo`03003ooooo0;ooo`80047oo`006ooo00<00?ooool0oooo0ooo
00000ooo000000020003ool00`00ooooo`03ool00`00ooooo`02ool30005ool00`00ooooo`32ool0
0`00ooooo`0mool000?oo`03003ooooo00Soo`03003ooooo00Goo`03003ooooo00;oo`03003ooooo
0?ooo`?oo`000ooo00<00?ooool02Ooo00<00?ooool01?oo00<00?ooool00_oo0P00aOoo0P00??oo
0003ool00`00ooooo`09ool01000oooooooo0`001Ooo00<00?ooool0oooo0ooo0003ool00`00oooo
o`0:ool01000ooooo`001ooo00<00?ooool0aooo00<00?ooool0>?oo00001?oo000000002Ooo00@0
0?ooool000;oo`03003ooooo00Goo`03003ooooo0?ooo`?oo`000ooo00<00?ooool01ooo10000_oo
10001?oo00<00?ooool0bOoo0P00=ooo000Kool2003oool4ool001_oo`03003ooooo0<coo`03003o
oooo03?oo`006ooo00<00?ooool0cooo00<00?ooool0<?oo000Kool00`00ooooo`3?ool00`00oooo
o`0`ool001_oo`03003ooooo0=7oo`03003ooooo02koo`006ooo0P00dooo00<00?ooool0;Ooo000K
ool00`00ooooo`3Dool00`00ooooo`0[ool001_oo`03003ooooo0=Coo`03003ooooo02_oo`006ooo
00<00?ooool0e_oo00<00?ooool0:Ooo000Kool00`00ooooo`3Gool00`00ooooo`0Xool001_oo`80
0=[oo`03003ooooo02Koo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3Kool2000Uool0
01_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0g_oo00<00?ooool08Ooo000Kool2003oool4
ool001_oo`03003ooooo0>3oo`80023oo`006ooo00<00?ooool0oooo0ooo000Cool40004ool00`00
ooooo`3Sool00`00ooooo`0Lool001?oo`03003ooooo00Goo`03003ooooo0?ooo`?oo`005?oo00<0
0?ooool01?oo0P00iooo00<00?ooool06Ooo000Eool00`00ooooo`03ool00`00ooooo`3oool3ool0
01Koo`03003ooooo00;oo`03003ooooo0>Soo`8001Soo`004ooo00@00?ooool000Coo`03003ooooo
0?ooo`?oo`005?oo0P001Ooo00<00?ooool0jooo00<00?ooool05?oo000Kool00`00ooooo`3oool3
ool001_oo`800>koo`8001?oo`006ooo00<00?ooool0l?oo00<00?ooool03ooo000Kool00`00oooo
o`3`ool00`00ooooo`0?ool001_oo`03003ooooo0?;oo`03003ooooo00goo`006ooo00<00?ooool0
looo00<00?ooool03?oo000Kool2003fool00`00ooooo`0:ool001_oo`03003ooooo0?Goo`03003o
oooo00[oo`006ooo00<00?ooool0mooo00<00?ooool02?oo000Kool00`00ooooo`3hool00`00oooo
o`07ool001_oo`03003ooooo0?[oo`03003ooooo00Goo`006ooo0P00oooo1?oo000Kool00`00oooo
o`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo003ooolQool00?oo
ob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.215729, 0.518112, \
0.00789457, 0.00976098}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        oma[IntegerPart[T*10^16]*10^\((\(-16\))\), 
          Pi/2 \((1 - 10^\((\(-5\))\))\), 10]}, {T, 1/10, 2, 
        1/20}]\)], "Input"],

Cell[BoxData[
    \({{1\/10, 0.6274183243559516`}, {3\/20, 0.6394919360514812`}, {1\/5, 
        0.6558653478081127`}, {1\/4, 0.6761055407183231`}, {3\/10, 
        0.6997581154813187`}, {7\/20, 0.7263833559070885`}, {2\/5, 
        0.7555772074774438`}, {9\/20, 0.7869821283978216`}, {1\/2, 
        0.8202881802735569`}, {11\/20, 0.8552312390177729`}, {3\/5, 
        0.8915882070775523`}, {13\/20, 0.9291705926943987`}, {7\/10, 
        0.9678200756248465`}, {3\/4, 1.007403106998531`}, {4\/5, 
        1.0478068133751808`}, {17\/20, 1.0889354419135884`}, {9\/10, 
        1.1307074189049222`}, {19\/20, 1.1730529341311553`}, {1, 
        1.2159119558128613`}, {21\/20, 1.2592326263559392`}, {11\/10, 
        1.302969931891037`}, {23\/20, 1.3470846222130037`}, {6\/5, 
        1.3915423212329803`}, {5\/4, 1.4363127934796853`}, {13\/10, 
        1.4813693369059298`}, {27\/20, 1.526688434385867`}, {7\/5, 
        1.5722485949740432`}, {29\/20, 1.6180313458226074`}, {3\/2, 
        1.6640197981027602`}, {31\/20, 1.7101987544906645`}, {8\/5, 
        1.7565545563213687`}, {33\/20, 1.8030748562681005`}, {17\/10, 
        1.849748460292027`}, {7\/4, 1.896565193581905`}, {9\/5, 
        1.9435157849475122`}, {37\/20, 1.9905920222190774`}, {19\/10, 
        2.0377856906061167`}, {39\/20, 2.0850898874313737`}, {2, 
        2.132498076722828`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/1000, Pi/2 \((1 - 10^\((\(-5\))\))\), 10]\)], "Input"],

Cell[BoxData[
    \(0.617511481068335`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.001, 0.617511481068335`}, {1\/10, 0.6274183243559516`}, {3\/20, 
        0.6394919360514812`}, {1\/5, 0.6558653478081127`}, {1\/4, 
        0.6761055407183231`}, {3\/10, 0.6997581154813187`}, {7\/20, 
        0.7263833559070885`}, {2\/5, 0.7555772074774438`}, {9\/20, 
        0.7869821283978216`}, {1\/2, 0.8202881802735569`}, {11\/20, 
        0.8552312390177729`}, {3\/5, 0.8915882070775523`}, {13\/20, 
        0.9291705926943987`}, {7\/10, 0.9678200756248465`}, {3\/4, 
        1.007403106998531`}, {4\/5, 1.0478068133751808`}, {17\/20, 
        1.0889354419135884`}, {9\/10, 1.1307074189049222`}, {19\/20, 
        1.1730529341311553`}, {1, 1.2159119558128613`}, {21\/20, 
        1.2592326263559392`}, {11\/10, 1.302969931891037`}, {23\/20, 
        1.3470846222130037`}, {6\/5, 1.3915423212329803`}, {5\/4, 
        1.4363127934796853`}, {13\/10, 1.4813693369059298`}, {27\/20, 
        1.526688434385867`}, {7\/5, 1.5722485949740432`}, {29\/20, 
        1.6180313458226074`}, {3\/2, 1.6640197981027602`}, {31\/20, 
        1.7101987544906645`}, {8\/5, 1.7565545563213687`}, {33\/20, 
        1.8030748562681005`}, {17\/10, 1.849748460292027`}, {7\/4, 
        1.896565193581905`}, {9\/5, 1.9435157849475122`}, {37\/20, 
        1.9905920222190774`}, {19\/10, 2.0377856906061167`}, {39\/20, 
        2.0850898874313737`}, {2, 2.132498076722828`}}\)], "Input"],

Cell[BoxData[
    \({{0.001`, 0.617511481068335`}, {1\/10, 0.6274183243559516`}, {3\/20, 
        0.6394919360514812`}, {1\/5, 0.6558653478081127`}, {1\/4, 
        0.6761055407183231`}, {3\/10, 0.6997581154813187`}, {7\/20, 
        0.7263833559070885`}, {2\/5, 0.7555772074774438`}, {9\/20, 
        0.7869821283978216`}, {1\/2, 0.8202881802735569`}, {11\/20, 
        0.8552312390177729`}, {3\/5, 0.8915882070775523`}, {13\/20, 
        0.9291705926943987`}, {7\/10, 0.9678200756248465`}, {3\/4, 
        1.007403106998531`}, {4\/5, 1.0478068133751808`}, {17\/20, 
        1.0889354419135884`}, {9\/10, 1.1307074189049222`}, {19\/20, 
        1.1730529341311553`}, {1, 1.2159119558128613`}, {21\/20, 
        1.2592326263559392`}, {11\/10, 1.302969931891037`}, {23\/20, 
        1.3470846222130037`}, {6\/5, 1.3915423212329803`}, {5\/4, 
        1.4363127934796853`}, {13\/10, 1.4813693369059298`}, {27\/20, 
        1.526688434385867`}, {7\/5, 1.5722485949740432`}, {29\/20, 
        1.6180313458226074`}, {3\/2, 1.6640197981027602`}, {31\/20, 
        1.7101987544906645`}, {8\/5, 1.7565545563213687`}, {33\/20, 
        1.8030748562681005`}, {17\/10, 1.849748460292027`}, {7\/4, 
        1.896565193581905`}, {9\/5, 1.9435157849475122`}, {37\/20, 
        1.9905920222190774`}, {19\/10, 2.0377856906061167`}, {39\/20, 
        2.0850898874313737`}, {2, 2.132498076722828`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True, 
      PlotStyle \[Rule] {Dashing[{0.002, 0.01}]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.225201 0.388521 [
[.2619 .15082 -9 -9 ]
[.2619 .15082 9 0 ]
[.5 .15082 -3 -9 ]
[.5 .15082 3 0 ]
[.7381 .15082 -9 -9 ]
[.7381 .15082 9 0 ]
[.97619 .15082 -3 -9 ]
[.97619 .15082 3 0 ]
[.01131 .00791 -18 -4.5 ]
[.01131 .00791 0 4.5 ]
[.01131 .08562 -18 -4.5 ]
[.01131 .08562 0 4.5 ]
[.01131 .24102 -18 -4.5 ]
[.01131 .24102 0 4.5 ]
[.01131 .31873 -18 -4.5 ]
[.01131 .31873 0 4.5 ]
[.01131 .39643 -18 -4.5 ]
[.01131 .39643 0 4.5 ]
[.01131 .47414 -18 -4.5 ]
[.01131 .47414 0 4.5 ]
[.01131 .55184 -6 -4.5 ]
[.01131 .55184 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .16332 m
.2619 .16957 L
s
[(0.5)] .2619 .15082 0 1 Mshowa
.5 .16332 m
.5 .16957 L
s
[(1)] .5 .15082 0 1 Mshowa
.7381 .16332 m
.7381 .16957 L
s
[(1.5)] .7381 .15082 0 1 Mshowa
.97619 .16332 m
.97619 .16957 L
s
[(2)] .97619 .15082 0 1 Mshowa
.125 Mabswid
.07143 .16332 m
.07143 .16707 L
s
.11905 .16332 m
.11905 .16707 L
s
.16667 .16332 m
.16667 .16707 L
s
.21429 .16332 m
.21429 .16707 L
s
.30952 .16332 m
.30952 .16707 L
s
.35714 .16332 m
.35714 .16707 L
s
.40476 .16332 m
.40476 .16707 L
s
.45238 .16332 m
.45238 .16707 L
s
.54762 .16332 m
.54762 .16707 L
s
.59524 .16332 m
.59524 .16707 L
s
.64286 .16332 m
.64286 .16707 L
s
.69048 .16332 m
.69048 .16707 L
s
.78571 .16332 m
.78571 .16707 L
s
.83333 .16332 m
.83333 .16707 L
s
.88095 .16332 m
.88095 .16707 L
s
.92857 .16332 m
.92857 .16707 L
s
.25 Mabswid
0 .16332 m
1 .16332 L
s
.02381 .00791 m
.03006 .00791 L
s
[(0.6)] .01131 .00791 1 0 Mshowa
.02381 .08562 m
.03006 .08562 L
s
[(0.8)] .01131 .08562 1 0 Mshowa
.02381 .24102 m
.03006 .24102 L
s
[(1.2)] .01131 .24102 1 0 Mshowa
.02381 .31873 m
.03006 .31873 L
s
[(1.4)] .01131 .31873 1 0 Mshowa
.02381 .39643 m
.03006 .39643 L
s
[(1.6)] .01131 .39643 1 0 Mshowa
.02381 .47414 m
.03006 .47414 L
s
[(1.8)] .01131 .47414 1 0 Mshowa
.02381 .55184 m
.03006 .55184 L
s
[(2)] .01131 .55184 1 0 Mshowa
.125 Mabswid
.02381 .02734 m
.02756 .02734 L
s
.02381 .04676 m
.02756 .04676 L
s
.02381 .06619 m
.02756 .06619 L
s
.02381 .10504 m
.02756 .10504 L
s
.02381 .12447 m
.02756 .12447 L
s
.02381 .14389 m
.02756 .14389 L
s
.02381 .18275 m
.02756 .18275 L
s
.02381 .20217 m
.02756 .20217 L
s
.02381 .2216 m
.02756 .2216 L
s
.02381 .26045 m
.02756 .26045 L
s
.02381 .27988 m
.02756 .27988 L
s
.02381 .2993 m
.02756 .2993 L
s
.02381 .33815 m
.02756 .33815 L
s
.02381 .35758 m
.02756 .35758 L
s
.02381 .37701 m
.02756 .37701 L
s
.02381 .41586 m
.02756 .41586 L
s
.02381 .43528 m
.02756 .43528 L
s
.02381 .45471 m
.02756 .45471 L
s
.02381 .49356 m
.02756 .49356 L
s
.02381 .51299 m
.02756 .51299 L
s
.02381 .53241 m
.02756 .53241 L
s
.02381 .57127 m
.02756 .57127 L
s
.02381 .59069 m
.02756 .59069 L
s
.02381 .61012 m
.02756 .61012 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .002 .01 ] 0 setdash
.02429 .01472 m
.07143 .01856 L
.09524 .02325 L
.11905 .02962 L
.14286 .03748 L
.16667 .04667 L
.19048 .05701 L
.21429 .06836 L
.2381 .08056 L
.2619 .0935 L
.28571 .10707 L
.30952 .1212 L
.33333 .1358 L
.35714 .15082 L
.38095 .1662 L
.40476 .18189 L
.42857 .19787 L
.45238 .2141 L
.47619 .23055 L
.5 .24721 L
.52381 .26404 L
.54762 .28103 L
.57143 .29817 L
.59524 .31544 L
.61905 .33284 L
.64286 .35034 L
.66667 .36795 L
.69048 .38565 L
.71429 .40344 L
.7381 .42131 L
.7619 .43925 L
.78571 .45726 L
.80952 .47533 L
.83333 .49346 L
.85714 .51165 L
.88095 .5299 L
.90476 .54819 L
.92857 .56652 L
.95238 .5849 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003o
oooo0?ooo`Woo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3oool9ool00005
ool00?ooool00008ool01000ooooo`001?oo0`00oooo2Ooo00001Ooo003ooooo00002?oo0`001Ooo
00<00?ooool0oooo2Ooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo0P0000?oo`000000oooo
1ooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool01?oo00@00?ooool000;oo`80
0?coo`000_oo0P002_oo0`001?oo00<00?ooool03Ooo0P00nOoo000Eool00`00ooooo`0@ool20002
ool00`00ooooo`3aool001Goo`8001Soo`03003ooooo0>koo`005Ooo00<00?ooool06_oo00<00?oo
ool0jooo000Eool00`00ooooo`0Mool00`00ooooo`3Xool001Goo`03003ooooo023oo`03003ooooo
0>Goo`005Ooo00<00?ooool08?oo00<00?ooool0iOoo000Eool2000Tool2003Sool001Goo`03003o
oooo02Koo`800>3oo`005Ooo00<00?ooool0:Ooo0P00gOoo000Eool00`00ooooo`3oool9ool001Go
o`03003ooooo02coo`03003ooooo0=Woo`005Ooo0P00<?oo00<00?ooool0e_oo000Eool00`00oooo
o`0bool00`00ooooo`3Cool001Goo`03003ooooo03;oo`03003ooooo0=?oo`000_oo0P001?oo00<0
0?ooool00ooo0P001Ooo00<00?ooool0=Ooo00<00?ooool0d?oo00001Ooo003ooooo00002?oo00@0
0?ooool000Coo`03003ooooo03Soo`03003ooooo0<goo`0000Goo`00ooooo`0000Soo`04003ooooo
0004ool3000hool00`00ooooo`3=ool00005ool00?ooool00009ool20005ool00`00ooooo`0kool0
0`00ooooo`3:ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0?Ooo00<00?oo
ool0b?oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo03koo`03003ooooo0<Oo
o`000_oo0P002_oo0P001Ooo00<00?ooool0@?oo0P00a_oo000Eool00`00ooooo`3oool9ool001Go
o`8004Coo`800<?oo`005Ooo00<00?ooool0A_oo00<00?ooool0_ooo000Eool00`00ooooo`16ool0
0`00ooooo`2oool001Goo`03003ooooo04Woo`03003ooooo0;coo`005Ooo00<00?ooool0oooo2Ooo
000Eool2000hool20004ool00`00ooooo`02ool30007ool00`00ooooo`0]ool4000gool40003ool0
0`00ooooo`02ool3000hool40006ool001Goo`03003ooooo03Koo`04003ooooo000;ool00`00oooo
o`06ool00`00ooooo`0]ool00`00ooooo`0hool00`00ooooo`0:ool00`00ooooo`0eool00`00oooo
o`07ool001Goo`03003ooooo03Koo`04003ooooo000;ool00`00ooooo`07ool00`00ooooo`0/ool0
0`00ooooo`0hool00`00ooooo`0:ool00`00ooooo`0fool00`00ooooo`06ool001Goo`03003ooooo
03Koo`04003ooooo0008ool3000<ool2000[ool00`00ooooo`0hool00`00ooooo`07ool3000jool0
0`00ooooo`05ool001Goo`03003ooooo03Koo`04003ooooo0008ool00`00ooooo`0iool00`00oooo
o`0hool00`00ooooo`07ool00`00ooooo`0kool00`00ooooo`04ool001Goo`8003Ooo`04003ooooo
0008ool00`00ooooo`0?ool00`00ooooo`0Uool3000hool30009ool00`00ooooo`0hool01000oooo
o`001_oo000Eool00`00ooooo`0gool20009ool4000hool00`00ooooo`0hool00`00ooooo`07ool4
000hool20007ool001Goo`03003ooooo05Ooo`03003ooooo0:koo`005Ooo00<00?ooool0FOoo00<0
0?ooool0[?oo000Eool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`03003ooooo05coo`800:[o
o`003_ooo`004`00000Eool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00oooo
o`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`04ool00`00oooo
o`03ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00oooo
o`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00oooo
o`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`05ool001Goo`03003ooooo0?ooo`Wo
o`005Ooo00<00?ooool0H_oo00<00?ooool0Xooo000Eool00`00ooooo`1Tool00`00ooooo`2Qool0
01Goo`8006Koo`03003ooooo0:3oo`005Ooo00<00?ooool0Iooo00<00?ooool0W_oo000Eool00`00
ooooo`1Wool00`00ooooo`2Nool001Goo`03003ooooo06[oo`03003ooooo09_oo`005Ooo00<00?oo
ool0oooo2Ooo000Eool2001]ool2002Jool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
Kooo0P00Uooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo07;oo`03003ooooo09?oo`00
5Ooo0P00MOoo00<00?ooool0TOoo000Eool00`00ooooo`1eool00`00ooooo`2@ool001Goo`03003o
oooo07Ooo`03003ooooo08koo`005Ooo00<00?ooool0N?oo00<00?ooool0SOoo00000ooo00000002
0003ool00`00ooooo`02ool40004ool00`00ooooo`1jool00`00ooooo`2;ool000?oo`03003ooooo
00Ooo`03003ooooo00Goo`03003ooooo07[oo`03003ooooo08_oo`000ooo00<00?ooool02?oo00<0
0?ooool01?oo0`00OOoo00<00?ooool0R?oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00
ooooo`3oool9ool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo07ooo`8008Ooo`00
00Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3oool9ool000?oo`03003ooooo00Soo`80
00Goo`03003ooooo08;oo`03003ooooo08?oo`005Ooo0P00oooo2_oo000Eool00`00ooooo`25ool0
0`00ooooo`20ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0Qooo0P00Oooo000Eool0
0`00ooooo`3oool9ool001Goo`8008_oo`03003ooooo07_oo`005Ooo00<00?ooool0SOoo00<00?oo
ool0N?oo000Eool00`00ooooo`2=ool00`00ooooo`1hool001Goo`03003ooooo08ooo`03003ooooo
07Koo`005Ooo00<00?ooool0T?oo00<00?ooool0MOoo000Eool00`00ooooo`2Bool00`00ooooo`1c
ool001Goo`8009?oo`03003ooooo07?oo`005Ooo00<00?ooool0U?oo0P00L_oo000Eool00`00oooo
o`3oool9ool00003ool00000008000?oo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo09Oo
o`8006ooo`000ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0oooo2Ooo0003ool00`00
ooooo`06ool50004ool3002Jool00`00ooooo`1[ool000?oo`03003ooooo00Koo`04003ooooo0005
ool00`00ooooo`3oool9ool000?oo`03003ooooo00Ooo`03003oo`0000Goo`03003ooooo09coo`80
06[oo`0000Coo`00000000[oo`8000Goo`03003ooooo0?ooo`Woo`000ooo00<00?ooool02Ooo00<0
0?ooool00ooo00<00?ooool0Wooo00<00?ooool0I_oo000Eool2003oool:ool001Goo`03003ooooo
0:;oo`03003ooooo06?oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`2Tool2001Rool0
01Goo`03003ooooo0?ooo`Woo`005Ooo0P00Z?oo00<00?ooool0G_oo000Eool00`00ooooo`3oool9
ool001Goo`03003ooooo0:[oo`03003ooooo05_oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`2/ool2001Jool001Goo`03003ooooo0:ooo`03003ooooo05Koo`005Ooo0P00/?oo00<00?oo
ool0E_oo000Eool00`00ooooo`2aool00`00ooooo`1Dool001Goo`03003ooooo0;;oo`03003ooooo
05?oo`0000?oo`0000000P000ooo00<00?ooool00ooo0P001Ooo00<00?ooool0]?oo00<00?ooool0
DOoo0003ool00`00ooooo`07ool01000ooooo`001?oo00<00?ooool0]?oo00<00?ooool0DOoo0003
ool00`00ooooo`07ool01000ooooo`001?oo0`00]ooo00<00?ooool0C_oo0003ool00`00ooooo`07
ool30005ool00`00ooooo`2gool00`00ooooo`1>ool000?oo`03003ooooo00Ooo`03003ooooo00Go
o`03003ooooo0;Woo`03003ooooo04coo`0000Coo`00000000Woo`03003ooooo00Goo`03003ooooo
0;[oo`03003ooooo04_oo`000ooo00<00?ooool02?oo0`001?oo00<00?ooool0_?oo00<00?ooool0
BOoo000Eool2002mool00`00ooooo`19ool001Goo`03003ooooo0;koo`03003ooooo04Ooo`005Ooo
00<00?ooool0_ooo00<00?ooool0A_oo000Eool00`00ooooo`31ool00`00ooooo`14ool001Goo`03
003ooooo0<7oo`03003ooooo04Coo`005Ooo00<00?ooool0a?oo00<00?ooool0@Ooo000Eool2003o
ool:ool001Goo`03003ooooo0<Koo`80043oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00oooo
o`39ool00`00ooooo`0lool001Goo`03003ooooo0?ooo`Woo`005Ooo0P00c?oo0P00>ooo000Eool0
0`00ooooo`3oool9ool001Goo`03003ooooo0<koo`8003Soo`0000?oo`0000000P000ooo00<00?oo
ool00ooo0P001Ooo00<00?ooool0oooo2Ooo0003ool00`00ooooo`07ool01000ooooo`001?oo00<0
0?ooool0dOoo00<00?ooool0=?oo0003ool00`00ooooo`07ool01000ooooo`001?oo0`00oooo2Ooo
0003ool00`00ooooo`08ool20005ool00`00ooooo`3Cool2000cool000?oo`03003ooooo00Ooo`04
003ooooo0004ool00`00ooooo`3oool9ool00004ool000000009ool01000ooooo`001?oo00<00?oo
ool0e_oo00<00?ooool0;ooo0003ool00`00ooooo`08ool20005ool00`00ooooo`3oool9ool001Go
o`800=Woo`8002koo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3Kool2000[ool001Go
o`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0g_oo00<00?ooool09ooo000Eool00`00ooooo`3o
ool9ool001Goo`800>7oo`8002Koo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3Sool0
0`00ooooo`0Rool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0iOoo0P008Ooo000Eool2
003oool:ool001Goo`03003ooooo0>Soo`03003ooooo01goo`005Ooo00<00?ooool0oooo2Ooo000=
ool40004ool00`00ooooo`3[ool00`00ooooo`0Jool000goo`03003ooooo00Goo`03003ooooo0?oo
o`Woo`003_oo00<00?ooool01?oo0`00kOoo0P006Ooo000?ool00`00ooooo`03ool00`00ooooo`3o
ool9ool0013oo`03003ooooo00;oo`03003ooooo0?3oo`03003ooooo01Goo`003Ooo00@00?ooool0
00Coo`03003ooooo0?ooo`Woo`003_oo0P001Ooo00<00?ooool0l_oo0P005?oo000Eool00`00oooo
o`3oool9ool001Goo`800?Koo`03003ooooo013oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`3hool00`00ooooo`0=ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0n_oo0P00
3?oo000Eool2003oool:ool001Goo`03003ooooo0?goo`03003ooooo00Soo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool20007ool001Goo`03003ooooo0?ooo`Woo`005Ooo0P00oooo
2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo003ooolQool0
0?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.165192, 0.526966, \
0.00771848, 0.00946014}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<amode.dat\>", N[%329]]\)], "Input"],

Cell[BoxData[
    \("amode.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        oma[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, \(-9\)/10]}, {T, 
        0.1, 3, 0.1}]\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.8630134213237192`}, {0.2`, 
        0.8817439765081156`}, {0.30000000000000004`, 
        0.9119745448212755`}, {0.4`, 0.952452737601105`}, {0.5`, 
        1.001774280510205`}, {0.6000000000000001`, 
        1.0585587523651894`}, {0.7000000000000001`, 
        1.1215564497626147`}, {0.8`, 1.1896930571554438`}, {0.9`, 
        1.262075519772663`}, {1.`, 1.3379720551635317`}, {1.1`, 
        1.4167911403210287`}, {1.2000000000000002`, 
        1.498054869760289`}, {1.3`, 
        1.581376976394623`}, {1.4000000000000001`, 
        1.6664442632371115`}, {1.5`, 1.7530011791619`}, {1.6`, 
        1.8408382806519088`}, {1.7000000000000002`, 
        1.929782627949666`}, {1.8`, 
        2.0196905033872876`}, {1.9000000000000001`, 
        2.110441679666996`}, {2.`, 2.2019349266543764`}, {2.1`, 
        2.294084474643645`}, {2.2`, 
        2.3868172164097805`}, {2.3000000000000003`, 
        2.480070481257107`}, {2.4000000000000004`, 
        2.5737902536084674`}, {2.5`, 2.667929738704867`}, {2.6`, 
        2.762448200767134`}, {2.7`, 
        2.8573100162209952`}, {2.8000000000000003`, 
        2.952483897655422`}, {2.9000000000000004`, 3.047942254107298`}, {3.`, 
        3.1436606608262356`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<Propa.dat\>", %32]\)], "Input"],

Cell[BoxData[
    \("Propa.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(oma[1/2, Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(0.7323852352786566`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      oma[IntegerPart[k*10^16]*10^\((\(-16\))\), Pi/4, 1], {k, 1/100, 
        3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147151 0.192274 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .11085 -18 -4.5 ]
[.01131 .11085 0 4.5 ]
[.01131 .20699 -6 -4.5 ]
[.01131 .20699 0 4.5 ]
[.01131 .30313 -18 -4.5 ]
[.01131 .30313 0 4.5 ]
[.01131 .39926 -6 -4.5 ]
[.01131 .39926 0 4.5 ]
[.01131 .4954 -18 -4.5 ]
[.01131 .4954 0 4.5 ]
[.01131 .59154 -6 -4.5 ]
[.01131 .59154 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11085 m
.03006 .11085 L
s
[(0.5)] .01131 .11085 1 0 Mshowa
.02381 .20699 m
.03006 .20699 L
s
[(1)] .01131 .20699 1 0 Mshowa
.02381 .30313 m
.03006 .30313 L
s
[(1.5)] .01131 .30313 1 0 Mshowa
.02381 .39926 m
.03006 .39926 L
s
[(2)] .01131 .39926 1 0 Mshowa
.02381 .4954 m
.03006 .4954 L
s
[(2.5)] .01131 .4954 1 0 Mshowa
.02381 .59154 m
.03006 .59154 L
s
[(3)] .01131 .59154 1 0 Mshowa
.125 Mabswid
.02381 .03394 m
.02756 .03394 L
s
.02381 .05317 m
.02756 .05317 L
s
.02381 .0724 m
.02756 .0724 L
s
.02381 .09162 m
.02756 .09162 L
s
.02381 .13008 m
.02756 .13008 L
s
.02381 .14931 m
.02756 .14931 L
s
.02381 .16853 m
.02756 .16853 L
s
.02381 .18776 m
.02756 .18776 L
s
.02381 .22622 m
.02756 .22622 L
s
.02381 .24544 m
.02756 .24544 L
s
.02381 .26467 m
.02756 .26467 L
s
.02381 .2839 m
.02756 .2839 L
s
.02381 .32235 m
.02756 .32235 L
s
.02381 .34158 m
.02756 .34158 L
s
.02381 .36081 m
.02756 .36081 L
s
.02381 .38003 m
.02756 .38003 L
s
.02381 .41849 m
.02756 .41849 L
s
.02381 .43772 m
.02756 .43772 L
s
.02381 .45694 m
.02756 .45694 L
s
.02381 .47617 m
.02756 .47617 L
s
.02381 .51463 m
.02756 .51463 L
s
.02381 .53385 m
.02756 .53385 L
s
.02381 .55308 m
.02756 .55308 L
s
.02381 .57231 m
.02756 .57231 L
s
.02381 .61076 m
.02756 .61076 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02698 .11088 m
.02816 .1109 L
.02922 .11092 L
.03162 .111 L
.03383 .11109 L
.03593 .1112 L
.04063 .11152 L
.04573 .11199 L
.056 .11327 L
.06549 .11486 L
.08588 .11944 L
.10749 .12578 L
.14519 .13955 L
.18535 .1568 L
.22399 .17505 L
.26508 .19563 L
.30465 .21627 L
.34272 .23666 L
.38323 .25881 L
.42223 .28048 L
.46367 .30379 L
.5036 .32648 L
.54203 .34848 L
.58289 .37203 L
.62225 .39484 L
.66009 .41687 L
.70038 .44042 L
.73916 .46316 L
.78039 .48741 L
.8201 .51082 L
.8583 .5334 L
.89895 .55747 L
.93808 .58068 L
.97571 .60303 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00>@3oool2000000@0oooo00<000000?ooo`3oool00P3oool300000280
oooo1000000Q0?ooo`@000000`3oool00`000000oooo0?ooo`020?ooo`<000008`3oool400000240
oooo100000030?ooo`030000003oool0oooo0080oooo0`00000S0?ooo`8000001`3oool003P0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0O0?ooo`040000003oool0oooo0000
00H0oooo000h0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`040?ooo`00>03oool010000000oooo0?ooo`0000080?ooo`<00000903oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo0`00000U0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool01`3oool3000002<0oooo0P0000070?ooo`00>03oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0103oool003P0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0R0?ooo`<000008P3oool30000
00T0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`00000Q0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool08P3oool010000000oooo0?ooo`0000060?ooo`005@3o
ool00`000000oooo0?ooo`0Q0?ooo`8000002@3oool4000002<0oooo00<000000?ooo`3oool08P3o
ool00`000000oooo0?ooo`070?ooo`@000008`3oool2000002<0oooo0P0000090?ooo`@000008P3o
ool2000000L0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?oo
o`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000>0?ooool00000
4`00000001D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool001D0oooo00<000000?ooo`3oool0:03oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool01@3oool0
01D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0
oooo000E0?ooo`800000o`3oool:0?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E
0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?oo
o`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo
2@3oool001D0oooo0P00003o0?ooo`X0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool0
01D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0
oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo0P00003o0?ooo`X0oooo000E
0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?oo
o`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo
2@3oool001D0oooo0P00003o0?ooo`X0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool0
01D0oooo00<000000?ooo`3oool0o`3oool90?ooo`000P3oool2000000@0oooo00<000000?ooo`3o
ool00P3oool3000000D0oooo00<000000?ooo`3oool0o`3oool90?ooo`0000D0oooo0000003oool0
oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool90?ooo`00
00D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo1@00003o0?ooo`L0
oooo00001@3oool000000?ooo`3oool0000000P0oooo0`0000050?ooo`030000003oool0oooo0080
oooo2000003n0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool02P3oool300000?/0oooo00001@3oool000000?ooo`3oool0000000P0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0=0?ooo`@00000m`3oool00080oooo
0P0000090?ooo`@00000103oool00`000000oooo0?ooo`0A0?ooo`D00000lP3oool001D0oooo00<0
00000?ooo`3oool05P3oool200000?00oooo000E0?ooo`8000006@3oool300000>d0oooo000E0?oo
o`030000003oool0oooo01/0oooo0P00003[0?ooo`005@3oool00`000000oooo0?ooo`0M0?ooo`<0
0000j03oool001D0oooo00<000000?ooo`3oool0803oool200000>H0oooo000E0?ooo`030000003o
ool0oooo0280oooo0P00003T0?ooo`005@3oool2000002D0oooo0P00003R0?ooo`005@3oool00`00
0000oooo0?ooo`0V0?ooo`800000h03oool001D0oooo00<000000?ooo`3oool0:03oool200000=h0
oooo000E0?ooo`030000003oool0oooo02X0oooo0P00003L0?ooo`005@3oool00`000000oooo0?oo
o`0/0?ooo`800000fP3oool001D0oooo0P00000_0?ooo`800000f03oool001D0oooo00<000000?oo
o`3oool0<03oool200000=H0oooo000E0?ooo`030000003oool0oooo0380oooo0`00003C0?ooo`00
5@3oool00`000000oooo0?ooo`0e0?ooo`800000d@3oool001D0oooo00<000000?ooo`3oool0=`3o
ool200000<l0oooo000E0?ooo`800000>P3oool300000<`0oooo000E0?ooo`030000003oool0oooo
03`0oooo0P00003:0?ooo`005@3oool00`000000oooo0?ooo`0n0?ooo`800000b03oool000d0oooo
100000040?ooo`030000003oool0oooo0400oooo0P0000360?ooo`003`3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0480oooo0P0000340?ooo`003`3oool00`000000oooo0?ooo`03
0?ooo`<00000A03oool00`000000oooo0?ooo`310?ooo`003`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo04D0oooo0P0000310?ooo`003`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo04L0oooo0P00002o0?ooo`003@3oool3000000D0oooo00<000000?ooo`3oool0
B@3oool00`000000oooo0?ooo`2l0?ooo`003`3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo04X0oooo0P00002l0?ooo`005@3oool00`000000oooo0?ooo`1<0?ooo`800000^P3oool0
01D0oooo0P00001?0?ooo`800000^03oool001D0oooo00<000000?ooo`3oool0D03oool200000;H0
oooo000E0?ooo`030000003oool0oooo0580oooo0P00002d0?ooo`005@3oool00`000000oooo0?oo
o`1D0?ooo`030000003oool0oooo0;40oooo000E0?ooo`030000003oool0oooo05D0oooo0P00002a
0?ooo`005@3oool2000005P0oooo0P00002_0?ooo`005@3oool00`000000oooo0?ooo`1I0?ooo`80
0000[@3oool001D0oooo00<000000?ooo`3oool0F`3oool200000:/0oooo000E0?ooo`030000003o
ool0oooo05d0oooo0P00002Y0?ooo`005@3oool00`000000oooo0?ooo`1O0?ooo`030000003oool0
oooo0:H0oooo000E0?ooo`800000H@3oool200000:H0oooo000E0?ooo`030000003oool0oooo0680
oooo0P00002T0?ooo`005@3oool00`000000oooo0?ooo`1T0?ooo`800000XP3oool001D0oooo00<0
00000?ooo`3oool0IP3oool200000:00oooo000E0?ooo`030000003oool0oooo06P0oooo0P00002N
0?ooo`005@3oool2000006/0oooo0P00002L0?ooo`005@3oool00`000000oooo0?ooo`1/0?ooo`80
0000VP3oool001D0oooo00<000000?ooo`3oool0KP3oool00`000000oooo0?ooo`2G0?ooo`005@3o
ool00`000000oooo0?ooo`1_0?ooo`800000U`3oool000030?ooo`0000000000008000000`3oool0
0`000000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`1a0?ooo`800000U@3oool0
00<0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
07<0oooo0P00002C0?ooo`000`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080
oooo0`00001e0?ooo`030000003oool0oooo0900oooo00030?ooo`030000003oool0oooo00L0oooo
0`0000050?ooo`030000003oool0oooo07H0oooo0P00002@0?ooo`000`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0N03oool2000008h0oooo0000
103oool000000000000000090?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0NP3o
ool00`000000oooo0?ooo`2;0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`@00000103oool0
0`000000oooo0?ooo`1k0?ooo`800000R`3oool001D0oooo0P00001n0?ooo`800000R@3oool001D0
oooo00<000000?ooo`3oool0O`3oool00`000000oooo0?ooo`260?ooo`005@3oool00`000000oooo
0?ooo`200?ooo`800000QP3oool001D0oooo00<000000?ooo`3oool0PP3oool2000008@0oooo000E
0?ooo`030000003oool0oooo08@0oooo0P0000220?ooo`005@3oool2000008L0oooo0P0000200?oo
o`005@3oool00`000000oooo0?ooo`280?ooo`800000OP3oool001D0oooo00<000000?ooo`3oool0
RP3oool00`000000oooo0?ooo`1k0?ooo`005@3oool00`000000oooo0?ooo`2;0?ooo`800000N`3o
ool001D0oooo00<000000?ooo`3oool0S@3oool2000007T0oooo000E0?ooo`800000T03oool20000
07L0oooo000E0?ooo`030000003oool0oooo0940oooo0P00001e0?ooo`005@3oool00`000000oooo
0?ooo`2C0?ooo`800000L`3oool001D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?oo
o`1`0?ooo`005@3oool00`000000oooo0?ooo`2F0?ooo`800000L03oool001D0oooo00<000000?oo
o`3oool0V03oool00`000000oooo0?ooo`1]0?ooo`005@3oool2000009X0oooo0P00001]0?ooo`00
5@3oool00`000000oooo0?ooo`2K0?ooo`800000J`3oool001D0oooo00<000000?ooo`3oool0W@3o
ool00`000000oooo0?ooo`1X0?ooo`003@3oool4000000@0oooo00<000000?ooo`3oool0WP3oool2
000006P0oooo000=0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0X03oool00`00
0000oooo0?ooo`1U0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`<00000X@3oool2000006D0
oooo000?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0X`3oool2000006<0oooo
000@0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0Y@3oool00`000000oooo0?oo
o`1P0?ooo`003@3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0:H0oooo0P00
001P0?ooo`003P3oool2000000D0oooo00<000000?ooo`3oool0Z03oool2000005h0oooo000E0?oo
o`800000Z`3oool00`000000oooo0?ooo`1K0?ooo`005@3oool00`000000oooo0?ooo`2[0?ooo`80
0000F`3oool001D0oooo00<000000?ooo`3oool0[@3oool2000005T0oooo000E0?ooo`030000003o
ool0oooo0:l0oooo0P00001G0?ooo`005@3oool00`000000oooo0?ooo`2a0?ooo`800000E@3oool0
01D0oooo0P00002d0?ooo`800000D`3oool001D0oooo00<000000?ooo`3oool0]@3oool00`000000
oooo0?ooo`1@0?ooo`005@3oool00`000000oooo0?ooo`2f0?ooo`800000D03oool001D0oooo00<0
00000?ooo`3oool0^03oool2000004h0oooo000E0?ooo`030000003oool0oooo0;X0oooo0P00001<
0?ooo`005@3oool200000;d0oooo0P00001:0?ooo`005@3oool00`000000oooo0?ooo`2n0?ooo`80
0000B03oool001D0oooo00<000000?ooo`3oool0`03oool00`000000oooo0?ooo`150?ooo`005@3o
ool00`000000oooo0?ooo`310?ooo`800000A@3oool001D0oooo00<000000?ooo`3oool0``3oool0
0`000000oooo0?ooo`120?ooo`005@3oool00`000000oooo0?ooo`340?ooo`800000@P3oool001D0
oooo0P0000370?ooo`800000@03oool001D0oooo00<000000?ooo`3oool0b03oool00`000000oooo
0?ooo`0m0?ooo`005@3oool00`000000oooo0?ooo`390?ooo`800000?@3oool000030?ooo`000000
0000008000000`3oool00`000000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`3;
0?ooo`030000003oool0oooo03X0oooo00000`3oool000000?ooo`0=0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0c03oool2000003X0oooo00020?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool00P3oool300000<h0oooo0P00000h0?ooo`000`3oool00`000000oooo
0?ooo`070?ooo`<000001@3oool00`000000oooo0?ooo`3@0?ooo`800000=P3oool000@0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0=80oooo0P00
000d0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0e03oool200000380oooo00020?ooo`8000002@3oool4000000@0oooo00<00000
0?ooo`3oool0eP3oool00`000000oooo0?ooo`0_0?ooo`005@3oool200000=P0oooo0P00000_0?oo
o`005@3oool00`000000oooo0?ooo`3I0?ooo`800000;@3oool001D0oooo00<000000?ooo`3oool0
f`3oool00`000000oooo0?ooo`0Z0?ooo`005@3oool00`000000oooo0?ooo`3L0?ooo`800000:P3o
ool001D0oooo00<000000?ooo`3oool0gP3oool2000002P0oooo000E0?ooo`800000h@3oool00`00
0000oooo0?ooo`0U0?ooo`005@3oool00`000000oooo0?ooo`3Q0?ooo`8000009@3oool001D0oooo
00<000000?ooo`3oool0h`3oool00`000000oooo0?ooo`0R0?ooo`005@3oool00`000000oooo0?oo
o`3T0?ooo`8000008P3oool001D0oooo00<000000?ooo`3oool0iP3oool200000200oooo000E0?oo
o`030000003oool0oooo0>P0oooo00<000000?ooo`3oool07@3oool001D0oooo0P00003Z0?ooo`80
00007@3oool001D0oooo00<000000?ooo`3oool0j`3oool00`000000oooo0?ooo`0J0?ooo`005@3o
ool00`000000oooo0?ooo`3/0?ooo`8000006P3oool001D0oooo00<000000?ooo`3oool0kP3oool2
000001P0oooo000E0?ooo`030000003oool0oooo0?00oooo0P00000F0?ooo`005@3oool200000?<0
oooo0P00000D0?ooo`005@3oool00`000000oooo0?ooo`3d0?ooo`8000004P3oool001D0oooo00<0
00000?ooo`3oool0mP3oool00`000000oooo0?ooo`0?0?ooo`003P3oool2000000D0oooo00<00000
0?ooo`3oool0m`3oool2000000l0oooo000=0?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0n@3oool2000000d0oooo000@0?ooo`030000003oool0oooo0080oooo0`00003k0?oo
o`030000003oool0oooo00X0oooo000>0?ooo`8000001@3oool00`000000oooo0?ooo`3l0?ooo`80
00002P3oool00100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3n0?ooo`800000
203oool000d0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3o0?ooo`40oooo
00<000000?ooo`3oool01@3oool000h0oooo0P0000050?ooo`030000003oool0oooo0?l0oooo2@3o
ool001D0oooo0P00003o0?ooo`X0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0
oooo00<000000?ooo`3oool0o`3oool90?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -0.240768, \
0.0115777, 0.0191158}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim[Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(0.3006645987164847`\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[oma[k, Pi/4, 1], {k, 1/100, 3}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(0.010010124583333333`\^2 \
- \(\(\(\[LeftSkeleton] 22 \[RightSkeleton]\)\^2\\ xi\^2\)\)\)\/\((1 + \
\(\(1\\ xi\^2\)\))\)\^2\)) is less than WorkingPrecision (\!\(30\))."\)], \
"Message",
  GeneratedCell->False,
  CellAutoOverwrite->False],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((\(\(0.010010124583333333` \[InvisibleSpace]\)\) + \(\(\
\[LeftSkeleton] 1 \[RightSkeleton]\)\))\)\)\\ \(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\)\)\/\((1 + \(\(1\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message",
  GeneratedCell->False,
  CellAutoOverwrite->False],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function \
(\!\(\(\(\((\(\(0.010010124583333333` \[InvisibleSpace]\)\) + \(\(\
\[LeftSkeleton] 1 \[RightSkeleton]\)\))\)\)\\ \(\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\)\)\/\((1 + \(\(1\\ xi\^2\)\))\)\^2\)) is less than \
WorkingPrecision (\!\(30\))."\)], "Message",
  GeneratedCell->False,
  CellAutoOverwrite->False],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(for\ \(\(gamma\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\(xi^3/Sqrt[xi^2 - kt^2]\)/\((1 + a\ xi^2)\)^2, {xi, kt, 
        1}]\)], "Input"],

Cell[BoxData[
    \(\((\(-\@a\)\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - 
          ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)] - 
          a\ ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)] - 
          2\ a\ kt\^2\ ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ \
kt\^2\)] - 
          2\ a\^2\ kt\^2\ ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ \
kt\^2\)])\)/\((2\ a\^\(3/2\)\ \((1 + a)\)\ \@\(\(-1\) - a\ kt\^2\)\ \((1 + 
              a\ kt\^2)\))\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\(-\@a\)\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - \((1 + a + 
                  2\ a\ kt\^2 + 
                  2\ a\^2\ kt\^2)\)\ ArcTanh[\(\@a\ \@\(1 - \
kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)]\)\/\(2\ a\^\(3/2\)\ \((1 + 
                a)\)\ \((\(-1\) - a\ kt\^2)\)\^\(3/2\)\)\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% /. kt^2 \[Rule] \((1 - kz^2/k^2)\)\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\(-\@a\)\ \@\(kz\^2\/k\^2\)\ \@\(\(-1\) - a\ \((1 - kz\^2\/k\^2)\
\)\) - \((1 + a + 2\ a\ \((1 - kz\^2\/k\^2)\) + 
                  2\ a\^2\ \((1 - 
                        kz\^2\/k\^2)\))\)\ ArcTanh[\(\@a\ \@\(kz\^2\/k\^2\)\)\
\/\@\(\(-1\) - a\ \((1 - kz\^2\/k\^2)\)\)]\)\/\(2\ a\^\(3/2\)\ \((1 + 
                a)\)\ \((\(-1\) - a\ \((1 - \
kz\^2\/k\^2)\))\)\^\(3/2\)\)\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(three = Simplify[%96, {k > 0, kz > 0}]\)], "Input"],

Cell[BoxData[
    \(\(\@a\ k\ kz\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\) + \((1 + \
a)\)\ \((\((1 + 2\ a)\)\ k\^2 - 2\ a\ kz\^2)\)\ ArcTanh[\(\@a\ \
kz\)\/\@\(\(-\((1 + a)\)\)\ k\^2 + a\ kz\^2\)]\)\/\(2\ a\^\(3/2\)\ \((1 + \
a)\)\ k\^2\ \((\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\))\)\^\(3/2\)\)\)], \
"Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(two = 
      Simplify[%44 /. kt^2 \[Rule] \((1 - kz^2/k^2)\), {k > 0, 
          kz > 0}]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\@a\ kz\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\) + \((1 + 
                  a)\)\ k\ ArcTanh[\(\@a\ kz\)\/\@\(\(-\((1 + a)\)\)\ k\^2 + \
a\ kz\^2\)]\)\/\(2\ a\^\(3/2\)\ \((1 + 
                a)\)\ k\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\)\)\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(one = %27\)], "Input"],

Cell[BoxData[
    \(\(\(-\@a\) + ArcTan[\@a] + a\ ArcTan[\@a]\)\/\(2\ a\^\(3/2\)\ \((1 + a)\
\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[two + three]\)], "Input"],

Cell[BoxData[
    \(\((\@a\ kz\ \((\(-\((2 + a)\)\)\ k\^2 + 
                a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\) + \
\((\(-\((2 + 5\ a + 3\ a\^2)\)\)\ k\^3 + 
                3\ a\ \((1 + 
                      a)\)\ k\ kz\^2)\)\ ArcTanh[\(\@a\ kz\)\/\@\(\(-\((1 + \
a)\)\)\ k\^2 + a\ kz\^2\)])\)/\((2\ a\^\(3/2\)\ \((1 + 
              a)\)\ k\ \((\((1 + a)\)\ k\^2 - 
              a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\))\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[\((1 + 2*kz^2/\((k^2 - kz^2)\))\)*one - 
        kz*k/\((1 - kz^2/k^2)\)*\((two + three)\)]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(2\ a\^\(3/2\)\ \((1 + a)\)\ \((k\^2 - kz\^2)\)\)\) \((\((k\^2 + 
                kz\^2)\)\ \((\(-\@a\) + \((1 + 
                      a)\)\ ArcTan[\@a])\) + \((k\^2\ kz\ \((\@a\ kz\ \((\((2 \
+ a)\)\ k\^2 - 
                          a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\) + \((1 + a)\)\ k\ \((\((2 + 3\ a)\)\ k\^2 - 
                          3\ a\ kz\^2)\)\ ArcTanh[\(\@a\ kz\)\/\@\(\(-\((1 + \
a)\)\)\ k\^2 + a\ kz\^2\)])\))\)/\((\((\((1 + a)\)\ k\^2 - 
                    a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\))\))\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(gamoz[md2_, kz_, k_, a_] := Evaluate[a*md2*Simplify[%114]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(gamoz[1, 0.99, 1, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.012305644634840747`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gamoz[1, 0.99, 1, 0.00001]\)], "Input"],

Cell[BoxData[
    \(6.633409108117785`*^-8 + 0.`\ \[ImaginaryI]\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-\((alomz[1, 0.99, 1, a] + gamoz[1, 0.99, 1, a])\)\)/
        mMag[1, a], {a, \(-1\), 20}, 
      PlotStyle \[Rule] {RGBColor[1, 0, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 -0.375953 0.961805 [
[.29592 .57335 -3 -9 ]
[.29592 .57335 3 0 ]
[.52268 .57335 -6 -9 ]
[.52268 .57335 6 0 ]
[.74943 .57335 -6 -9 ]
[.74943 .57335 6 0 ]
[.97619 .57335 -6 -9 ]
[.97619 .57335 6 0 ]
[.05666 .00877 -18 -4.5 ]
[.05666 .00877 0 4.5 ]
[.05666 .10495 -18 -4.5 ]
[.05666 .10495 0 4.5 ]
[.05666 .20113 -18 -4.5 ]
[.05666 .20113 0 4.5 ]
[.05666 .29731 -18 -4.5 ]
[.05666 .29731 0 4.5 ]
[.05666 .39349 -18 -4.5 ]
[.05666 .39349 0 4.5 ]
[.05666 .48967 -18 -4.5 ]
[.05666 .48967 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .58585 m
.29592 .5921 L
s
[(5)] .29592 .57335 0 1 Mshowa
.52268 .58585 m
.52268 .5921 L
s
[(10)] .52268 .57335 0 1 Mshowa
.74943 .58585 m
.74943 .5921 L
s
[(15)] .74943 .57335 0 1 Mshowa
.97619 .58585 m
.97619 .5921 L
s
[(20)] .97619 .57335 0 1 Mshowa
.125 Mabswid
.11451 .58585 m
.11451 .5896 L
s
.15986 .58585 m
.15986 .5896 L
s
.20522 .58585 m
.20522 .5896 L
s
.25057 .58585 m
.25057 .5896 L
s
.34127 .58585 m
.34127 .5896 L
s
.38662 .58585 m
.38662 .5896 L
s
.43197 .58585 m
.43197 .5896 L
s
.47732 .58585 m
.47732 .5896 L
s
.56803 .58585 m
.56803 .5896 L
s
.61338 .58585 m
.61338 .5896 L
s
.65873 .58585 m
.65873 .5896 L
s
.70408 .58585 m
.70408 .5896 L
s
.79478 .58585 m
.79478 .5896 L
s
.84014 .58585 m
.84014 .5896 L
s
.88549 .58585 m
.88549 .5896 L
s
.93084 .58585 m
.93084 .5896 L
s
.02381 .58585 m
.02381 .5896 L
s
.25 Mabswid
0 .58585 m
1 .58585 L
s
.06916 .00877 m
.07541 .00877 L
s
[(0.4)] .05666 .00877 1 0 Mshowa
.06916 .10495 m
.07541 .10495 L
s
[(0.5)] .05666 .10495 1 0 Mshowa
.06916 .20113 m
.07541 .20113 L
s
[(0.6)] .05666 .20113 1 0 Mshowa
.06916 .29731 m
.07541 .29731 L
s
[(0.7)] .05666 .29731 1 0 Mshowa
.06916 .39349 m
.07541 .39349 L
s
[(0.8)] .05666 .39349 1 0 Mshowa
.06916 .48967 m
.07541 .48967 L
s
[(0.9)] .05666 .48967 1 0 Mshowa
.125 Mabswid
.06916 .02801 m
.07291 .02801 L
s
.06916 .04724 m
.07291 .04724 L
s
.06916 .06648 m
.07291 .06648 L
s
.06916 .08571 m
.07291 .08571 L
s
.06916 .12419 m
.07291 .12419 L
s
.06916 .14342 m
.07291 .14342 L
s
.06916 .16266 m
.07291 .16266 L
s
.06916 .18189 m
.07291 .18189 L
s
.06916 .22037 m
.07291 .22037 L
s
.06916 .2396 m
.07291 .2396 L
s
.06916 .25884 m
.07291 .25884 L
s
.06916 .27807 m
.07291 .27807 L
s
.06916 .31655 m
.07291 .31655 L
s
.06916 .33578 m
.07291 .33578 L
s
.06916 .35502 m
.07291 .35502 L
s
.06916 .37425 m
.07291 .37425 L
s
.06916 .41273 m
.07291 .41273 L
s
.06916 .43196 m
.07291 .43196 L
s
.06916 .4512 m
.07291 .4512 L
s
.06916 .47044 m
.07291 .47044 L
s
.06916 .50891 m
.07291 .50891 L
s
.06916 .52814 m
.07291 .52814 L
s
.06916 .54738 m
.07291 .54738 L
s
.06916 .56662 m
.07291 .56662 L
s
.06916 .60509 m
.07291 .60509 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.02381 .60332 m
.06244 .55425 L
.10458 .51393 L
.14415 .47868 L
.18221 .44661 L
.22272 .41419 L
.26171 .3845 L
.30316 .35442 L
.34309 .3268 L
.3815 .30139 L
.42237 .27553 L
.46172 .25171 L
.49955 .22974 L
.53984 .2073 L
.57861 .18659 L
.61984 .16547 L
.65954 .14594 L
.69774 .12788 L
.73838 .10941 L
.77751 .09231 L
.81909 .07484 L
.85916 .05865 L
.89771 .04364 L
.93871 .02826 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo203oool00080oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo203oool00080oooo
00@000000?ooo`3oool000001`3oool5000000@0oooo0`00003o0?ooo`P0oooo00020?ooo`040000
003oool0oooo000000L0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3o0?oo
o`P0oooo00020?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`0000001@3oool00`00
0000oooo0?ooo`3m0?ooo`80o`00203oool00080oooo00@000000?ooo`3oool000002@3oool20000
00D0oooo00<000000?ooo`3oool0nP3oool30?l000X0oooo00030?ooo`8000002`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?P0oooo0P3o000=0?ooo`005P3oool00`000000oooo
0?ooo`3e0?ooo`<0o`003`3oool001H0oooo0P00003c0?ooo`<0o`004P3oool001H0oooo00<00000
0?ooo`3oool0k`3oool30?l001D0oooo000F0?ooo`030000003oool0oooo0>d0oooo0P3o000H0?oo
o`005P3oool00`000000oooo0?ooo`3Z0?ooo`<0o`006P3oool001H0oooo00<000000?ooo`3oool0
i`3oool30?l001d0oooo000F0?ooo`800000i@3oool30?l00200oooo000F0?ooo`030000003oool0
oooo0>80oooo0P3o000S0?ooo`005P3oool00`000000oooo0?ooo`3O0?ooo`<0o`009@3oool001H0
oooo00<000000?ooo`3oool0g03oool30?l002P0oooo000F0?ooo`030000003oool0oooo0=X0oooo
0P3o000[0?ooo`005P3oool200000=P0oooo0`3o000]0?ooo`005P3oool00`000000oooo0?ooo`3E
0?ooo`80o`00<03oool001H0oooo00<000000?ooo`3oool0d`3oool20?l00380oooo000F0?ooo`03
0000003oool0oooo0=00oooo0`3o000d0?ooo`005P3oool00`000000oooo0?ooo`3>0?ooo`80o`00
=`3oool001H0oooo00<000000?ooo`3oool0c03oool20?l003T0oooo000F0?ooo`800000b`3oool2
0?l003/0oooo000F0?ooo`030000003oool0oooo0<P0oooo0P3o000m0?ooo`005P3oool00`000000
oooo0?ooo`350?ooo`<0o`00?`3oool000<0oooo0P0000040?ooo`030000003oool0oooo0080oooo
0`0000050?ooo`030000003oool0oooo0<<0oooo0P3o00120?ooo`000P3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0`03oool30?l004@0oooo
00020?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool300000;h0oooo
0P3o00170?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000oooo
0?ooo`2l0?ooo`80o`00B@3oool00080oooo00@000000?ooo`3oool00000203oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo0;T0oooo0`3o001;0?ooo`000P3oool010000000oooo0?oo
o`0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0]`3oool20?l004h0oooo
00030?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0]@3oool20?l00500oooo000F
0?ooo`030000003oool0oooo0;<0oooo0P3o001B0?ooo`005P3oool200000;80oooo0P3o001D0?oo
o`005P3oool00`000000oooo0?ooo`2^0?ooo`<0o`00EP3oool001H0oooo00<000000?ooo`3oool0
[03oool20?l005T0oooo000F0?ooo`030000003oool0oooo0:X0oooo0P3o001K0?ooo`005P3oool0
0`000000oooo0?ooo`2X0?ooo`80o`00G@3oool001H0oooo0P00002W0?ooo`80o`00G`3oool001H0
oooo00<000000?ooo`3oool0X`3oool30?l00640oooo000F0?ooo`030000003oool0oooo0:40oooo
0P3o001T0?ooo`005P3oool00`000000oooo0?ooo`2O0?ooo`80o`00IP3oool001H0oooo00<00000
0?ooo`3oool0W@3oool20?l006P0oooo000F0?ooo`030000003oool0oooo09/0oooo0P3o001Z0?oo
o`005P3oool2000009T0oooo0`3o001/0?ooo`005P3oool00`000000oooo0?ooo`2F0?ooo`80o`00
K`3oool001H0oooo00<000000?ooo`3oool0U03oool20?l00740oooo000F0?ooo`030000003oool0
oooo0980oooo0P3o001c0?ooo`005P3oool00`000000oooo0?ooo`2@0?ooo`80o`00M@3oool001H0
oooo0P00002?0?ooo`80o`00M`3oool001H0oooo00<000000?ooo`3oool0S@3oool00`3o0000oooo
0?ooo`1g0?ooo`005P3oool00`000000oooo0?ooo`2;0?ooo`80o`00NP3oool001H0oooo00<00000
0?ooo`3oool0R@3oool20?l007`0oooo00030?ooo`800000103oool00`000000oooo0?ooo`030?oo
o`8000001@3oool00`000000oooo0?ooo`270?ooo`80o`00OP3oool00080oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo08D0oooo0P3o0020
0?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo0`00
00230?ooo`80o`00PP3oool00080oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<0
00000?ooo`3oool0PP3oool00`3o0000oooo0?ooo`220?ooo`000P3oool010000000oooo0?ooo`00
00080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0P03oool20?l008D0oooo0002
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`1o0?ooo`030?l0003oool0oooo08D0oooo00030?ooo`8000002P3oool3000000@0oooo00<00000
0?ooo`3oool0O@3oool20?l008P0oooo000F0?ooo`800000O03oool20?l008X0oooo000F0?ooo`03
0000003oool0oooo07X0oooo00<0o`000?ooo`3oool0RP3oool001H0oooo00<000000?ooo`3oool0
N03oool20?l008d0oooo000F0?ooo`030000003oool0oooo07L0oooo00<0o`000?ooo`3oool0S@3o
ool001H0oooo00<000000?ooo`3oool0M@3oool20?l00900oooo000F0?ooo`800000M03oool20?l0
0980oooo000F0?ooo`030000003oool0oooo0740oooo0P3o002D0?ooo`005P3oool00`000000oooo
0?ooo`1_0?ooo`80o`00UP3oool001H0oooo00<000000?ooo`3oool0K@3oool20?l009P0oooo000F
0?ooo`030000003oool0oooo06`0oooo00<0o`000?ooo`3oool0V03oool001H0oooo00<000000?oo
o`3oool0JP3oool20?l009/0oooo000F0?ooo`800000JP3oool00`3o0000oooo0?ooo`2K0?ooo`00
5P3oool00`000000oooo0?ooo`1W0?ooo`80o`00WP3oool001H0oooo00<000000?ooo`3oool0I@3o
ool20?l00:00oooo000F0?ooo`030000003oool0oooo06@0oooo00<0o`000?ooo`3oool0X03oool0
01H0oooo00<000000?ooo`3oool0HP3oool20?l00:<0oooo000F0?ooo`800000HP3oool00`3o0000
oooo0?ooo`2S0?ooo`005P3oool00`000000oooo0?ooo`1O0?ooo`80o`00YP3oool001H0oooo00<0
00000?ooo`3oool0G@3oool20?l00:P0oooo000F0?ooo`030000003oool0oooo05/0oooo0P3o002Z
0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo05X0oooo00<0o`000?ooo`3oool0ZP3oool00080oooo00@000000?ooo`3o
ool000002@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo05P0oooo0P3o002]0?oo
o`000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo0`00001F0?oo
o`80o`00[`3oool00080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo05D0oooo00<0o`000?ooo`3oool0[`3oool00080oooo00@000000?ooo`3o
ool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo05<0oooo0P3o002b0?oo
o`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0DP3oool00`3o0000oooo0?ooo`2b0?ooo`000`3oool2000000T0oooo100000040?oo
o`030000003oool0oooo0500oooo0P3o002e0?ooo`005P3oool200000500oooo00<0o`000?ooo`3o
ool0]@3oool001H0oooo00<000000?ooo`3oool0CP3oool00`3o0000oooo0?ooo`2f0?ooo`005P3o
ool00`000000oooo0?ooo`1<0?ooo`80o`00^@3oool001H0oooo00<000000?ooo`3oool0B`3oool0
0`3o0000oooo0?ooo`2i0?ooo`005P3oool00`000000oooo0?ooo`190?ooo`80o`00_03oool001H0
oooo00<000000?ooo`3oool0B03oool00`3o0000oooo0?ooo`2l0?ooo`005P3oool2000004L0oooo
0P3o002o0?ooo`005P3oool00`000000oooo0?ooo`150?ooo`030?l0003oool0oooo0;l0oooo000F
0?ooo`030000003oool0oooo04<0oooo0P3o00320?ooo`005P3oool00`000000oooo0?ooo`120?oo
o`030?l0003oool0oooo0<80oooo000F0?ooo`030000003oool0oooo0400oooo0P3o00350?ooo`00
5P3oool200000400oooo00<0o`000?ooo`3oool0a@3oool001H0oooo00<000000?ooo`3oool0?@3o
ool20?l00<P0oooo000F0?ooo`030000003oool0oooo03`0oooo00<0o`000?ooo`3oool0b03oool0
01H0oooo00<000000?ooo`3oool0>`3oool00`3o0000oooo0?ooo`390?ooo`005P3oool00`000000
oooo0?ooo`0i0?ooo`80o`00c03oool001H0oooo00<000000?ooo`3oool0>03oool00`3o0000oooo
0?ooo`3<0?ooo`005P3oool2000003P0oooo00<0o`000?ooo`3oool0c@3oool001H0oooo00<00000
0?ooo`3oool0=@3oool20?l00=00oooo000F0?ooo`030000003oool0oooo03@0oooo00<0o`000?oo
o`3oool0d03oool000<0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`03
0000003oool0oooo03<0oooo00<0o`000?ooo`3oool0d@3oool00080oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0340oooo0P3o003D0?oo
o`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo0`00000`
0?ooo`030?l0003oool0oooo0=@0oooo00020?ooo`040000003oool0oooo000000T0oooo0P000005
0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool0e@3oool00080oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02h0oooo00<0
o`000?ooo`3oool0eP3oool00080oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo02d0oooo00<0o`000?ooo`3oool0e`3oool000<0oooo0P00
000:0?ooo`8000001@3oool00`000000oooo0?ooo`0[0?ooo`80o`00fP3oool001H0oooo0P00000[
0?ooo`030?l0003oool0oooo0=X0oooo000F0?ooo`030000003oool0oooo02T0oooo00<0o`000?oo
o`3oool0f`3oool001H0oooo00<000000?ooo`3oool09`3oool20?l00=h0oooo000F0?ooo`030000
003oool0oooo02H0oooo00<0o`000?ooo`3oool0gP3oool001H0oooo00<000000?ooo`3oool09@3o
ool00`3o0000oooo0?ooo`3O0?ooo`005P3oool00`000000oooo0?ooo`0S0?ooo`80o`00hP3oool0
01H0oooo0P00000S0?ooo`030?l0003oool0oooo0>80oooo000F0?ooo`030000003oool0oooo0240
oooo00<0o`000?ooo`3oool0h`3oool001H0oooo00<000000?ooo`3oool07`3oool20?l00>H0oooo
000F0?ooo`030000003oool0oooo01h0oooo00<0o`000?ooo`3oool0iP3oool001H0oooo00<00000
0?ooo`3oool07@3oool00`3o0000oooo0?ooo`3W0?ooo`005P3oool2000001d0oooo00<0o`000?oo
o`3oool0j03oool001H0oooo00<000000?ooo`3oool06P3oool20?l00>/0oooo000F0?ooo`030000
003oool0oooo01T0oooo00<0o`000?ooo`3oool0j`3oool001H0oooo00<000000?ooo`3oool0603o
ool00`3o0000oooo0?ooo`3/0?ooo`005P3oool00`000000oooo0?ooo`0G0?ooo`030?l0003oool0
oooo0>d0oooo000F0?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool0kP3oool001H0
oooo0P00000E0?ooo`80o`00l@3oool001H0oooo00<000000?ooo`3oool04`3oool00`3o0000oooo
0?ooo`3a0?ooo`005P3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo0?80oooo0003
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`0A
0?ooo`030?l0003oool0oooo0?<0oooo00020?ooo`040000003oool0oooo000000/0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`030?l0003oool0oooo0?@0oooo00020?oo
o`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool3000000l0oooo00<0o`00
0?ooo`3oool0m@3oool00080oooo00@000000?ooo`3oool000002@3oool3000000@0oooo00<00000
0?ooo`3oool03P3oool00`3o0000oooo0?ooo`3f0?ooo`000P3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0303oool20?l00?T0oooo0002
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo
0?ooo`0;0?ooo`030?l0003oool0oooo0?T0oooo00030?ooo`8000002P3oool2000000D0oooo00<0
00000?ooo`3oool02P3oool00`3o0000oooo0?ooo`3j0?ooo`005P3oool00`000000oooo0?ooo`09
0?ooo`030?l0003oool0oooo0?/0oooo000F0?ooo`8000002@3oool00`3o0000oooo0?ooo`3l0?oo
o`005P3oool00`000000oooo0?ooo`070?ooo`030?l0003oool0oooo0?d0oooo000F0?ooo`030000
003oool0oooo00H0oooo00<0o`000?ooo`3oool0oP3oool001H0oooo00<000000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`3o0?ooo`005P3oool00`000000oooo0?ooo`040?ooo`030?l0003oool0
oooo0?l0oooo0@3oool001H0oooo0P0000040?ooo`030?l0003oool0oooo0?l0oooo0P3oool001H0
oooo00@000000?ooo`3oool0oooo0P3o003o0?ooo`D0oooo000F0?ooo`040000003oool0oooo0?l0
0?l0oooo1`3oool001H0oooo00<000000?ooo`3o0000o`3oool80?ooo`005P3oool00`000000o`00
0?ooo`3o0?ooo`P0oooo000F0?ooo`030?l0003oool0oooo0?l0oooo203oool001D0oooo00<0o`00
000000000000?03oool3000003X0oooo100000030?ooo`800000>03oool400000080oooo0`00000g
0?ooo`@000000`3oool2000000D0oooo000D0?ooo`030?l0003oool000000400oooo00<000000?oo
o`3oool0>@3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0=`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool00`3oool0
10000000oooo0?ooo`0000040?ooo`004`3oool0103o0000oooo0?ooo`0000100?ooo`030000003o
ool0oooo03T0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo03L0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo0080oooo
00@000000?ooo`3oool00000103oool00180oooo00D0o`000?ooo`3oool0oooo0000000m0?ooo`<0
0000?03oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0=`3oool01000
0000oooo0?ooo`3oool3000003T0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003o
ool0oooo0080oooo000B0?ooo`050?l0003oool0oooo0?ooo`000000?@3oool00`000000oooo0?oo
o`0l0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`0g0?ooo`050000
003oool0oooo0?ooo`000000?03oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
0080oooo000A0?ooo`030?l0003oool0oooo0080oooo0P00000l0?ooo`030000003oool0oooo03X0
oooo0`0000030?ooo`040000003oool0oooo000003L0oooo0`0000030?ooo`030000003oool0oooo
03L0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`00403oool0
0`3o0000oooo0?ooo`030?ooo`030000003oool0oooo03/0oooo1000000k0?ooo`030000003oool0
oooo0080oooo0P00000j0?ooo`040000003oool0oooo0?ooo`@00000=`3oool2000000@0oooo0P00
00050?ooo`003`3oool00`3o0000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool0
00l0oooo00<0o`000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo000>0?ooo`03
0?l0003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool80?ooo`000P3oool;000000030?l0
0000000000000?l000003`0000020?ooo`002@3oool010000000oooo0?ooo`3o00090?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?ooo`002`3o
ool00`3o0000oooo0?ooo`080?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool01P3o
ool000/0oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?ooo`3o0?ooo`P0oooo000:0?oo
o`030?l0003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool80?ooo`002@3oool00`3o0000
oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo203oool001H0oooo0P00003o0?ooo`T0oooo
000F0?ooo`030000003oool0oooo0?l0oooo203oool001H0oooo00<000000?ooo`3oool0o`3oool8
0?ooo`005P3oool00`000000oooo0?ooo`3o0?ooo`P0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.75634, 0.383467, \
0.0779083, 0.00367357}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(compare\ to\ original\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %70]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 -0.375953 0.961805 [
[.29592 .57335 -3 -9 ]
[.29592 .57335 3 0 ]
[.52268 .57335 -6 -9 ]
[.52268 .57335 6 0 ]
[.74943 .57335 -6 -9 ]
[.74943 .57335 6 0 ]
[.97619 .57335 -6 -9 ]
[.97619 .57335 6 0 ]
[.05666 .00877 -18 -4.5 ]
[.05666 .00877 0 4.5 ]
[.05666 .10495 -18 -4.5 ]
[.05666 .10495 0 4.5 ]
[.05666 .20113 -18 -4.5 ]
[.05666 .20113 0 4.5 ]
[.05666 .29731 -18 -4.5 ]
[.05666 .29731 0 4.5 ]
[.05666 .39349 -18 -4.5 ]
[.05666 .39349 0 4.5 ]
[.05666 .48967 -18 -4.5 ]
[.05666 .48967 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .58585 m
.29592 .5921 L
s
[(5)] .29592 .57335 0 1 Mshowa
.52268 .58585 m
.52268 .5921 L
s
[(10)] .52268 .57335 0 1 Mshowa
.74943 .58585 m
.74943 .5921 L
s
[(15)] .74943 .57335 0 1 Mshowa
.97619 .58585 m
.97619 .5921 L
s
[(20)] .97619 .57335 0 1 Mshowa
.125 Mabswid
.11451 .58585 m
.11451 .5896 L
s
.15986 .58585 m
.15986 .5896 L
s
.20522 .58585 m
.20522 .5896 L
s
.25057 .58585 m
.25057 .5896 L
s
.34127 .58585 m
.34127 .5896 L
s
.38662 .58585 m
.38662 .5896 L
s
.43197 .58585 m
.43197 .5896 L
s
.47732 .58585 m
.47732 .5896 L
s
.56803 .58585 m
.56803 .5896 L
s
.61338 .58585 m
.61338 .5896 L
s
.65873 .58585 m
.65873 .5896 L
s
.70408 .58585 m
.70408 .5896 L
s
.79478 .58585 m
.79478 .5896 L
s
.84014 .58585 m
.84014 .5896 L
s
.88549 .58585 m
.88549 .5896 L
s
.93084 .58585 m
.93084 .5896 L
s
.02381 .58585 m
.02381 .5896 L
s
.25 Mabswid
0 .58585 m
1 .58585 L
s
.06916 .00877 m
.07541 .00877 L
s
[(0.4)] .05666 .00877 1 0 Mshowa
.06916 .10495 m
.07541 .10495 L
s
[(0.5)] .05666 .10495 1 0 Mshowa
.06916 .20113 m
.07541 .20113 L
s
[(0.6)] .05666 .20113 1 0 Mshowa
.06916 .29731 m
.07541 .29731 L
s
[(0.7)] .05666 .29731 1 0 Mshowa
.06916 .39349 m
.07541 .39349 L
s
[(0.8)] .05666 .39349 1 0 Mshowa
.06916 .48967 m
.07541 .48967 L
s
[(0.9)] .05666 .48967 1 0 Mshowa
.125 Mabswid
.06916 .02801 m
.07291 .02801 L
s
.06916 .04724 m
.07291 .04724 L
s
.06916 .06648 m
.07291 .06648 L
s
.06916 .08571 m
.07291 .08571 L
s
.06916 .12419 m
.07291 .12419 L
s
.06916 .14342 m
.07291 .14342 L
s
.06916 .16266 m
.07291 .16266 L
s
.06916 .18189 m
.07291 .18189 L
s
.06916 .22037 m
.07291 .22037 L
s
.06916 .2396 m
.07291 .2396 L
s
.06916 .25884 m
.07291 .25884 L
s
.06916 .27807 m
.07291 .27807 L
s
.06916 .31655 m
.07291 .31655 L
s
.06916 .33578 m
.07291 .33578 L
s
.06916 .35502 m
.07291 .35502 L
s
.06916 .37425 m
.07291 .37425 L
s
.06916 .41273 m
.07291 .41273 L
s
.06916 .43196 m
.07291 .43196 L
s
.06916 .4512 m
.07291 .4512 L
s
.06916 .47044 m
.07291 .47044 L
s
.06916 .50891 m
.07291 .50891 L
s
.06916 .52814 m
.07291 .52814 L
s
.06916 .54738 m
.07291 .54738 L
s
.06916 .56662 m
.07291 .56662 L
s
.06916 .60509 m
.07291 .60509 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.02381 .60332 m
.06244 .55425 L
.10458 .51393 L
.14415 .47868 L
.18221 .44661 L
.22272 .41419 L
.26171 .3845 L
.30316 .35442 L
.34309 .3268 L
.3815 .30139 L
.42237 .27553 L
.46172 .25171 L
.49955 .22974 L
.53984 .2073 L
.57861 .18659 L
.61984 .16547 L
.65954 .14594 L
.69774 .12788 L
.73838 .10941 L
.77751 .09231 L
.81909 .07484 L
.85916 .05865 L
.89771 .04364 L
.93871 .02826 L
.97619 .01472 L
s
0 g
.02381 .58518 m
.06244 .56895 L
.10458 .55532 L
.14415 .54316 L
.18221 .53189 L
.22272 .52028 L
.26171 .50945 L
.30316 .49827 L
.34309 .4878 L
.3815 .47801 L
.42237 .46786 L
.46172 .45834 L
.49955 .44942 L
.53984 .44015 L
.57861 .43145 L
.61984 .42242 L
.65954 .41394 L
.69774 .40596 L
.73838 .39767 L
.77751 .38987 L
.81909 .38176 L
.85916 .37414 L
.89771 .36695 L
.93871 .35948 L
.97619 .35279 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo203oool00080oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo203oool00080oooo
00@000000?ooo`3oool000001`3oool5000000@0oooo0`00003o0?ooo`P0oooo00020?ooo`040000
003oool0oooo000000L0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3o0?oo
o`P0oooo00020?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`0000001@3oool00`00
0000oooo0?ooo`3m0?ooo`80o`00203oool00080oooo00@000000?ooo`3oool000002@3oool20000
00D0oooo00<000000?ooo`3oool0nP3oool30?l000X0oooo00030?ooo`8000002`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?P0oooo0P3o000=0?ooo`005P3oool00`000000oooo
0?ooo`3e0?ooo`<0o`003`3oool001H0oooo0P00003c0?ooo`<0o`004P3oool001H0oooo00<00000
0?ooo`3oool0k`3oool30?l001D0oooo000F0?ooo`030000003oool0oooo0>d0oooo0P3o000H0?oo
o`005P3oool00`000000oooo0?ooo`3Z0?ooo`<0o`006P3oool001H0oooo00<000000?ooo`3oool0
i`3oool30?l001d0oooo000F0?ooo`800000i@3oool30?l00200oooo000F0?ooo`030000003oool0
oooo0>80oooo0P3o000S0?ooo`005P3oool00`000000oooo0?ooo`3O0?ooo`<0o`009@3oool001H0
oooo00<000000?ooo`3oool0g03oool30?l002P0oooo000F0?ooo`030000003oool0oooo0=X0oooo
0P3o000[0?ooo`005P3oool200000=P0oooo0`3o000]0?ooo`005P3oool00`000000oooo0?ooo`3E
0?ooo`80o`00<03oool001H0oooo00<000000?ooo`3oool0d`3oool20?l00380oooo000F0?ooo`03
0000003oool0oooo0=00oooo0`3o000d0?ooo`005P3oool00`000000oooo0?ooo`3>0?ooo`80o`00
=`3oool001H0oooo00<000000?ooo`3oool0c03oool20?l003T0oooo000F0?ooo`800000b`3oool2
0?l003/0oooo000F0?ooo`030000003oool0oooo0<P0oooo0P3o000m0?ooo`005P3oool00`000000
oooo0?ooo`350?ooo`<0o`00?`3oool000<0oooo0P0000040?ooo`030000003oool0oooo0080oooo
0`0000050?ooo`030000003oool0oooo0<<0oooo0P3o00120?ooo`000P3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0`03oool30?l004@0oooo
00020?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool300000;h0oooo
0P3o00170?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000oooo
0?ooo`2l0?ooo`80o`00B@3oool00080oooo00@000000?ooo`3oool00000203oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo0;T0oooo0`3o001;0?ooo`000P3oool010000000oooo0?oo
o`0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0]`3oool20?l004h0oooo
00030?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0]@3oool20?l00500oooo000F
0?ooo`030000003oool0oooo0;<0oooo0P3o001B0?ooo`005P3oool200000;80oooo0P3o001D0?oo
o`005P3oool00`000000oooo0?ooo`2^0?ooo`<0o`00EP3oool001H0oooo00<000000?ooo`3oool0
[03oool20?l005T0oooo000F0?ooo`030000003oool0oooo0:X0oooo0P3o001K0?ooo`005P3oool0
0`000000oooo0?ooo`2X0?ooo`80o`00G@3oool001H0oooo0P00002W0?ooo`80o`00G`3oool001H0
oooo00<000000?ooo`3oool0X`3oool30?l00640oooo000F0?ooo`030000003oool0oooo0:40oooo
0P3o001T0?ooo`005P3oool00`000000oooo0?ooo`2O0?ooo`80o`00IP3oool001H0oooo00<00000
0?ooo`3oool0W@3oool20?l006P0oooo000F0?ooo`030000003oool0oooo09/0oooo0P3o001Z0?oo
o`005P3oool2000009T0oooo0`3o001/0?ooo`005P3oool00`000000oooo0?ooo`2F0?ooo`80o`00
K`3oool001H0oooo00<000000?ooo`3oool0U03oool20?l00740oooo000F0?ooo`030000003oool0
oooo0980oooo0P3o001c0?ooo`005P3oool00`000000oooo0?ooo`2@0?ooo`80o`00M@3oool001H0
oooo0P00002?0?ooo`80o`00M`3oool001H0oooo00<000000?ooo`3oool0S@3oool00`3o0000oooo
0?ooo`1g0?ooo`005P3oool00`000000oooo0?ooo`2;0?ooo`80o`00NP3oool001H0oooo00<00000
0?ooo`3oool0R@3oool20?l007`0oooo00030?ooo`800000103oool00`000000oooo0?ooo`030?oo
o`8000001@3oool00`000000oooo0?ooo`270?ooo`80o`00OP3oool00080oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo08D0oooo0P3o0020
0?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo0`00
00230?ooo`80o`00PP3oool00080oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<0
00000?ooo`3oool0PP3oool00`3o0000oooo0?ooo`220?ooo`000P3oool010000000oooo0?ooo`00
00080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0P03oool20?l008D0oooo0002
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`1o0?ooo`030?l0003oool0oooo08D0oooo00030?ooo`8000002P3oool3000000@0oooo00<00000
0?ooo`3oool0O@3oool20?l008P0oooo000F0?ooo`800000O03oool20?l008X0oooo000F0?ooo`03
0000003oool0oooo07X0oooo00<0o`000?ooo`3oool0RP3oool001H0oooo00<000000?ooo`3oool0
N03oool20?l008d0oooo000F0?ooo`030000003oool0oooo07L0oooo00<0o`000?ooo`3oool0S@3o
ool001H0oooo00<000000?ooo`3oool0M@3oool20?l00900oooo000F0?ooo`800000M03oool20?l0
0980oooo000F0?ooo`030000003oool0oooo0740oooo0P3o002D0?ooo`005P3oool00`000000oooo
0?ooo`1_0?ooo`80o`00UP3oool001H0oooo00<000000?ooo`3oool0K@3oool20?l009P0oooo000F
0?ooo`030000003oool0oooo06`0oooo00<0o`000?ooo`3oool0V03oool001H0oooo00<000000?oo
o`3oool0JP3oool20?l009/0oooo000F0?ooo`800000JP3oool00`3o0000oooo0?ooo`2K0?ooo`00
5P3oool00`000000oooo0?ooo`1W0?ooo`80o`00WP3oool001H0oooo00<000000?ooo`3oool0I@3o
ool20?l00:00oooo000F0?ooo`030000003oool0oooo06@0oooo00<0o`000?ooo`3oool0X03oool0
01H0oooo00<000000?ooo`3oool0HP3oool20?l00:<0oooo000F0?ooo`800000HP3oool00`3o0000
oooo0?ooo`2S0?ooo`005P3oool00`000000oooo0?ooo`1O0?ooo`80o`00YP3oool001H0oooo00<0
00000?ooo`3oool0G@3oool20?l00:P0oooo000F0?ooo`030000003oool0oooo05/0oooo0P3o002Z
0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo05X0oooo00<0o`000?ooo`3oool0ZP3oool00080oooo00@000000?ooo`3o
ool000002@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo05P0oooo0P3o002]0?oo
o`000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo0`00001F0?oo
o`80o`00[`3oool00080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo05D0oooo00<0o`000?ooo`3oool0[`3oool00080oooo00@000000?ooo`3o
ool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo05<0oooo0P3o002b0?oo
o`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0DP3oool00`3o0000oooo0?ooo`2b0?ooo`000`3oool2000000T0oooo100000040?oo
o`030000003oool0oooo0500oooo0P3o002e0?ooo`005P3oool200000500oooo00<0o`000?ooo`3o
ool0]@3oool001H0oooo00<000000?ooo`3oool0CP3oool00`3o0000oooo0?ooo`2f0?ooo`005P3o
ool00`000000oooo0?ooo`1<0?ooo`80o`00^@3oool001H0oooo00<000000?ooo`3oool0B`3oool0
0`3o0000oooo0?ooo`2i0?ooo`005P3oool00`000000oooo0?ooo`190?ooo`80o`00_03oool001H0
oooo00<000000?ooo`3oool0B03oool00`3o0000oooo0?ooo`2l0?ooo`005P3oool2000004L0oooo
0P3o002o0?ooo`005P3oool00`000000oooo0?ooo`150?ooo`030?l0003oool0oooo0;l0oooo000F
0?ooo`030000003oool0oooo04<0oooo0P3o00320?ooo`005P3oool00`000000oooo0?ooo`120?oo
o`030?l0003oool0oooo0<80oooo000F0?ooo`030000003oool0oooo0400oooo0P3o002j0?ooo`<0
0000203oool001H0oooo0P0000100?ooo`030?l0003oool0oooo0;@0oooo1P00000;0?ooo`005P3o
ool00`000000oooo0?ooo`0m0?ooo`80o`00/P3oool500000140oooo000F0?ooo`030000003oool0
oooo03`0oooo00<0o`000?ooo`3oool0[03oool6000001H0oooo000F0?ooo`030000003oool0oooo
03/0oooo00<0o`000?ooo`3oool0Z03oool5000001`0oooo000F0?ooo`030000003oool0oooo03T0
oooo0P3o002U0?ooo`H000008@3oool001H0oooo00<000000?ooo`3oool0>03oool00`3o0000oooo
0?ooo`2P0?ooo`D000009`3oool001H0oooo0P00000h0?ooo`030?l0003oool0oooo09d0oooo1000
000/0?ooo`005P3oool00`000000oooo0?ooo`0e0?ooo`80o`00W03oool400000300oooo000F0?oo
o`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool0V03oool4000003@0oooo00030?ooo`80
0000103oool00`000000oooo0?ooo`030?ooo`8000001@3oool00`000000oooo0?ooo`0c0?ooo`03
0?l0003oool0oooo09<0oooo1P00000h0?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0<@3oool20?l00900oooo1P00000n0?oo
o`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo0`00000`
0?ooo`030?l0003oool0oooo08X0oooo1P0000140?ooo`000P3oool010000000oooo0?ooo`000009
0?ooo`8000001@3oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0oooo08H0oooo1@00001:
0?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0;P3oool00`3o0000oooo0?ooo`210?ooo`H00000C`3oool00080oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02d0oooo
00<0o`000?ooo`3oool0OP3oool4000005D0oooo00030?ooo`8000002P3oool2000000D0oooo00<0
00000?ooo`3oool0:`3oool20?l007d0oooo1000001I0?ooo`005P3oool2000002/0oooo00<0o`00
0?ooo`3oool0N@3oool4000005d0oooo000F0?ooo`030000003oool0oooo02T0oooo00<0o`000?oo
o`3oool0MP3oool400000640oooo000F0?ooo`030000003oool0oooo02L0oooo0P3o001c0?ooo`H0
0000I@3oool001H0oooo00<000000?ooo`3oool09P3oool00`3o0000oooo0?ooo`1^0?ooo`D00000
J`3oool001H0oooo00<000000?ooo`3oool09@3oool00`3o0000oooo0?ooo`1[0?ooo`@00000L03o
ool001H0oooo00<000000?ooo`3oool08`3oool20?l006X0oooo1000001d0?ooo`005P3oool20000
02<0oooo00<0o`000?ooo`3oool0IP3oool4000007P0oooo000F0?ooo`030000003oool0oooo0240
oooo00<0o`000?ooo`3oool0H@3oool6000007`0oooo000F0?ooo`030000003oool0oooo01l0oooo
0P3o001P0?ooo`@00000PP3oool001H0oooo00<000000?ooo`3oool07P3oool00`3o0000oooo0?oo
o`1L0?ooo`@00000QP3oool001H0oooo00<000000?ooo`3oool07@3oool00`3o0000oooo0?ooo`1I
0?ooo`@00000RP3oool001H0oooo0P00000M0?ooo`030?l0003oool0oooo05H0oooo1000002>0?oo
o`005P3oool00`000000oooo0?ooo`0J0?ooo`80o`00D`3oool600000980oooo000F0?ooo`030000
003oool0oooo01T0oooo00<0o`000?ooo`3oool0C`3oool4000009P0oooo000F0?ooo`030000003o
ool0oooo01P0oooo00<0o`000?ooo`3oool0C03oool4000009`0oooo000F0?ooo`030000003oool0
oooo01L0oooo00<0o`000?ooo`3oool0B@3oool400000:00oooo000F0?ooo`030000003oool0oooo
01H0oooo00<0o`000?ooo`3oool0AP3oool400000:@0oooo000F0?ooo`8000005@3oool20?l004D0
oooo1000002X0?ooo`005P3oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo0440oooo
1000002/0?ooo`005P3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo03l0oooo0`00
002`0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<00000
0?ooo`3oool04@3oool00`3o0000oooo0?ooo`0l0?ooo`@00000/`3oool00080oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo00<0o`00
0?ooo`3oool0>P3oool300000;L0oooo00020?ooo`040000003oool0oooo000000/0oooo00<00000
0?ooo`3oool00P3oool3000000l0oooo00<0o`000?ooo`3oool0=`3oool400000;X0oooo00020?oo
o`040000003oool0oooo000000T0oooo0`0000040?ooo`030000003oool0oooo00h0oooo00<0o`00
0?ooo`3oool0=03oool400000;h0oooo00020?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0<0?ooo`80o`00<`3oool400000<80oooo0002
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo
0?ooo`0;0?ooo`030?l0003oool0oooo0300oooo0`0000360?ooo`000`3oool2000000X0oooo0P00
00050?ooo`030000003oool0oooo00X0oooo00<0o`000?ooo`3oool0;@3oool400000<T0oooo000F
0?ooo`030000003oool0oooo00T0oooo00<0o`000?ooo`3oool0:P3oool400000<d0oooo000F0?oo
o`8000002@3oool00`3o0000oooo0?ooo`0X0?ooo`<00000d@3oool001H0oooo00<000000?ooo`3o
ool01`3oool00`3o0000oooo0?ooo`0U0?ooo`@00000e03oool001H0oooo00<000000?ooo`3oool0
1P3oool00`3o0000oooo0?ooo`0R0?ooo`@00000f03oool001H0oooo00<000000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`0O0?ooo`@00000g03oool001H0oooo00<000000?ooo`3oool0103oool0
0`3o0000oooo0?ooo`0L0?ooo`@00000h03oool001H0oooo0P0000040?ooo`030?l0003oool0oooo
01T0oooo1000003T0?ooo`005P3oool010000000oooo0?ooo`3oool20?l001T0oooo0`00003X0?oo
o`005P3oool010000000oooo0?ooo`3o000G0?ooo`@00000j`3oool001H0oooo00<000000?ooo`3o
0000503oool400000>l0oooo000F0?ooo`030000003o0000oooo0140oooo0`00003c0?ooo`005P3o
ool00`3o0000oooo0?ooo`0>0?ooo`<00000mP3oool001D0oooo00<0o`000000000000003@3oool2
000002d0oooo0`00000j0?ooo`@000000`3oool2000003P0oooo100000020?ooo`<00000=`3oool4
000000<0oooo0P0000050?ooo`00503oool00`3o0000oooo0000000;0?ooo`<00000<P3oool00`00
0000oooo0?ooo`0i0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`0g
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`03
0?ooo`040000003oool0oooo000000@0oooo000C0?ooo`040?l0003oool0oooo000000P0oooo0`00
000e0?ooo`030000003oool0oooo03T0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000
003oool0oooo03L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0e0?ooo`030000
003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00180oooo00D0o`000?ooo`3oool0
oooo000000050?ooo`<00000=@3oool3000003`0oooo00D000000?ooo`3oool0oooo000000020?oo
o`030000003oool0oooo03L0oooo00@000000?ooo`3oool0oooo0`00000i0?ooo`050000003oool0
oooo0?ooo`0000000P3oool00`000000oooo0?ooo`020?ooo`004P3oool01@3o0000oooo0?ooo`3o
ool000000080oooo0`00000h0?ooo`030000003oool0oooo03`0oooo00D000000?ooo`3oool0oooo
000000020?ooo`030000003oool0oooo03L0oooo00D000000?ooo`3oool0oooo0000000l0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00140oooo00<0o`000?ooo`3o
ool00P3oool3000003/0oooo00<000000?ooo`3oool0>P3oool3000000<0oooo00@000000?ooo`3o
ool00000=`3oool3000000<0oooo00<000000?ooo`3oool0=`3oool010000000oooo0?ooo`000002
0?ooo`040000003oool0oooo000000@0oooo000@0?ooo`030?l0003oool0oooo00@00000?@3oool4
000003/0oooo00<000000?ooo`3oool00P3oool2000003X0oooo00@000000?ooo`3oool0oooo1000
000g0?ooo`800000103oool2000000D0oooo000?0?ooo`040?l0000000000000000000<0oooo00<0
00000?ooo`3oool0o`3oool80?ooo`003P3oool2000000H0oooo00<000000?ooo`3oool0o`3oool8
0?ooo`002`3oool3000000030?l0003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool80?oo
o`000P3oool;000000030?l00000000000000?l000003`0000020?ooo`002@3oool010000000oooo
0?ooo`3o00090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`060?ooo`002`3oool00`3o0000oooo0?ooo`080?ooo`030000003oool0oooo03d0oooo
00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03d0oooo
00<000000?ooo`3oool01P3oool000/0oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?oo
o`3o0?ooo`P0oooo000:0?ooo`030?l0003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool8
0?ooo`002@3oool00`3o0000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo203oool001H0
oooo0P00003o0?ooo`T0oooo000F0?ooo`030000003oool0oooo0?l0oooo203oool001H0oooo00<0
00000?ooo`3oool0o`3oool80?ooo`005P3oool00`000000oooo0?ooo`3o0?ooo`P0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.75634, 0.383467, \
0.0779083, 0.00367357}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(lowers\ much\ further\)], "Input"],

Cell[BoxData[
    \(Does\ it\ get\ \(\(negative\)\(?\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-\((alomz[1, 0.9, 1, a] + gamoz[1, 0.9, 1, a])\)\), {a, \(-1\), 
        10}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.11039 0.0865801 0.531406 0.768442 [
[.28355 .51891 -3 -9 ]
[.28355 .51891 3 0 ]
[.45671 .51891 -3 -9 ]
[.45671 .51891 3 0 ]
[.62987 .51891 -3 -9 ]
[.62987 .51891 3 0 ]
[.80303 .51891 -3 -9 ]
[.80303 .51891 3 0 ]
[.97619 .51891 -6 -9 ]
[.97619 .51891 6 0 ]
[.09789 .07034 -24 -4.5 ]
[.09789 .07034 0 4.5 ]
[.09789 .22403 -24 -4.5 ]
[.09789 .22403 0 4.5 ]
[.09789 .37772 -24 -4.5 ]
[.09789 .37772 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.28355 .53141 m
.28355 .53766 L
s
[(2)] .28355 .51891 0 1 Mshowa
.45671 .53141 m
.45671 .53766 L
s
[(4)] .45671 .51891 0 1 Mshowa
.62987 .53141 m
.62987 .53766 L
s
[(6)] .62987 .51891 0 1 Mshowa
.80303 .53141 m
.80303 .53766 L
s
[(8)] .80303 .51891 0 1 Mshowa
.97619 .53141 m
.97619 .53766 L
s
[(10)] .97619 .51891 0 1 Mshowa
.125 Mabswid
.15368 .53141 m
.15368 .53516 L
s
.19697 .53141 m
.19697 .53516 L
s
.24026 .53141 m
.24026 .53516 L
s
.32684 .53141 m
.32684 .53516 L
s
.37013 .53141 m
.37013 .53516 L
s
.41342 .53141 m
.41342 .53516 L
s
.5 .53141 m
.5 .53516 L
s
.54329 .53141 m
.54329 .53516 L
s
.58658 .53141 m
.58658 .53516 L
s
.67316 .53141 m
.67316 .53516 L
s
.71645 .53141 m
.71645 .53516 L
s
.75974 .53141 m
.75974 .53516 L
s
.84632 .53141 m
.84632 .53516 L
s
.88961 .53141 m
.88961 .53516 L
s
.9329 .53141 m
.9329 .53516 L
s
.0671 .53141 m
.0671 .53516 L
s
.02381 .53141 m
.02381 .53516 L
s
.25 Mabswid
0 .53141 m
1 .53141 L
s
.11039 .07034 m
.11664 .07034 L
s
[(-0.6)] .09789 .07034 1 0 Mshowa
.11039 .22403 m
.11664 .22403 L
s
[(-0.4)] .09789 .22403 1 0 Mshowa
.11039 .37772 m
.11664 .37772 L
s
[(-0.2)] .09789 .37772 1 0 Mshowa
.125 Mabswid
.11039 .10876 m
.11414 .10876 L
s
.11039 .14719 m
.11414 .14719 L
s
.11039 .18561 m
.11414 .18561 L
s
.11039 .26245 m
.11414 .26245 L
s
.11039 .30087 m
.11414 .30087 L
s
.11039 .3393 m
.11414 .3393 L
s
.11039 .41614 m
.11414 .41614 L
s
.11039 .45456 m
.11414 .45456 L
s
.11039 .49298 m
.11414 .49298 L
s
.11039 .03192 m
.11414 .03192 L
s
.11039 .56983 m
.11414 .56983 L
s
.11039 .60825 m
.11414 .60825 L
s
.25 Mabswid
.11039 0 m
.11039 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.0327 0 m
.03279 .00277 L
.03527 .06473 L
.0379 .1173 L
.04262 .19067 L
.04749 .24865 L
.05205 .2924 L
.06244 .36781 L
.07305 .42288 L
.08274 .46099 L
.09394 .49497 L
.10458 .52004 L
.11417 .53805 L
.12439 .55353 L
.14277 .57399 L
.15306 .58225 L
.16418 .58918 L
.17469 .59414 L
.18435 .59756 L
.18905 .59889 L
.19409 .60009 L
.19844 .60095 L
.20318 .60171 L
.20873 .60241 L
.21128 .60265 L
.21399 .60287 L
.21642 .60303 L
.21771 .6031 L
.21911 .60316 L
.22038 .60321 L
.22155 .60325 L
.22267 .60327 L
.22384 .6033 L
.22508 .60331 L
.22577 .60332 L
.22641 .60332 L
.22759 .60332 L
.22884 .60331 L
.22996 .60329 L
.23101 .60328 L
.23215 .60325 L
.23339 .60321 L
.23612 .60311 L
.23872 .60298 L
.24358 .60267 L
.25237 .60186 L
.26195 .60068 L
.28166 .59744 L
.30265 .59304 L
.34428 .58245 L
Mistroke
.3844 .57093 L
.423 .55936 L
.46406 .547 L
.5036 .53532 L
.54162 .52443 L
.5821 .51332 L
.62106 .50313 L
.66247 .49286 L
.70237 .4835 L
.74075 .475 L
.78159 .46648 L
.82091 .45874 L
.85871 .45173 L
.89896 .44469 L
.9377 .4383 L
.97619 .43232 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003o2@3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0?h0oooo00090?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool0oP3oool000T0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`3n0?ooo`002@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0?h0oooo
00090?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0oP3oool000T0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`3n0?ooo`002@3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0?h0oooo00090?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0oP3oool000T0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3n0?ooo`002@3o
ool00`000000oooo0?ooo`0C0?ooo`<00000oP3oool000X0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`3n0?ooo`002P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
0?h0oooo000:0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0oP3oool000X0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3n0?ooo`002P3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo0?h0oooo000:0?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool0oP3oool000X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3n0?oo
o`002P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0?h0oooo000:0?ooo`040000
003oool00000000000@0oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3o
ool0oP3oool000X0oooo0P0000020?ooo`030000003oool0oooo00H0oooo00@000000?ooo`3oool0
0000103oool00`000000oooo0?ooo`3n0?ooo`002P3oool200000080oooo00<000000?ooo`3oool0
1P3oool010000000oooo0?ooo`0000040?ooo`<00000oP3oool000D0oooo100000000`3oool00000
000000020?ooo`030000003oool0oooo00H0oooo0`0000050?ooo`030000003oool0oooo0?h0oooo
000:0?ooo`8000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0oP3oool000X0oooo0P0000020?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`3n0?ooo`002P3oool010000000oooo00000000000:
0?ooo`<00000103oool00`000000oooo0?ooo`3n0?ooo`002P3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo0?h0oooo000;0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool0oP3oool000/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3n0?ooo`002`3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0?h0oooo000;0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool0oP3oool000/0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`3n0?ooo`002`3oool00`000000oooo0?ooo`0A0?ooo`<00000oP3oool000/0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3n0?ooo`002`3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo0?h0oooo000;0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool0oP3oool000/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3n0?oo
o`002`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0?h0oooo000;0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool0oP3oool000/0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`3n0?ooo`002`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0?h0oooo000;0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0oP3oool000/0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3n0?ooo`002`3oool00`000000oooo
0?ooo`0A0?ooo`<00000oP3oool000/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0?h0oooo000<0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0oP3oool000`0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0?h0oooo000<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0oP3o
ool000`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3n0?ooo`00303oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo0?h0oooo000<0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0oP3oool000`0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?ooo`<00000oP3oool000`0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0?h0oooo000<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0oP3oool000`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3n0?ooo`00303o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0?h0oooo000<0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0oP3oool000`0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0?h0
oooo000<0?ooo`800000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0oP3oool000/0oooo00@000000?ooo`00000000002P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0?h0oooo000;0?ooo`040000003oool00000000000L0oooo
1@0000040?ooo`<00000oP3oool000D0oooo100000020?ooo`040000003oool00000000000L0oooo
00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3n0?ooo`002`3oool010000000oooo
0000000000080?ooo`030000003oool0000000D0oooo00<000000?ooo`3oool0oP3oool000/0oooo
00@000000?ooo`00000000002@3oool2000000D0oooo00<000000?ooo`3oool0oP3oool000`0oooo
0P00000;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0oP3oool000d0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`3n0?ooo`003@3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo0?h0oooo000=0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0oP3oool000d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3n0?ooo`00
3@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0?h0oooo000=0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0oP3oool000h0oooo00<000000?ooo`3oool03P3oool3
00000?h0oooo000>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0oP3oool000h0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3n0?ooo`003P3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo0?h0oooo000>0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0oP3oool000h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3n
0?ooo`003`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?h0oooo000?0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool0oP3oool000l0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`3n0?ooo`003`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo0?h0oooo000?0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0oP3oool0
00l0oooo00<000000?ooo`3oool03@3oool300000?h0oooo000@0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0oP3oool00100oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`3n0?ooo`00403oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?h0oooo000@
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0oP3oool00100oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`3n0?ooo`00403oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo0?h0oooo000@0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
oP3oool00100oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3n0?ooo`004@3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?h0oooo000A0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0oP3oool00140oooo00<000000?ooo`3oool02`3oool300000?h0
oooo000A0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0oP3oool00140oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`3n0?ooo`004@3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0?h0oooo000A0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0oP3oool00180oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3n0?ooo`00
4P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?h0oooo000B0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0oP3oool00180oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`3n0?ooo`00303oool2000000@0oooo00<000000?ooo`3oool00P3oool40000
00@0oooo00<000000?ooo`3oool0oP3oool000/0oooo00@000000?ooo`3oool000000`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0oP3oool000/0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00@0oooo0`00003n0?ooo`001@3oool400000080oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0oP3oool000/0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0oP3oool000/0oooo00@000000?ooo`3oool00000103oool01@00
0000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3n
0?ooo`00303oool2000000D0oooo00<000000?ooo`3oool00P3oool2000000D0oooo00<000000?oo
o`3oool0oP3oool001<0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3n0?ooo`00
503oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?h0oooo000D0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0oP3oool001@0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`3n0?ooo`00503oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
0?h0oooo000D0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0oP3oool001D0oooo
00<000000?ooo`3oool01`3oool300000?h0oooo000E0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0oP3oool001D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3n
0?ooo`005@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?h0oooo000F0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0l`3oool3000000P0oooo000F0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0k@3oool6000000/0oooo000F0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool0j03oool500000140oooo000F0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool0hP3oool6000001H0oooo000G0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0g@3oool5000001`0oooo000G0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0e`3oool600000240oooo000G0?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0dP3oool5000002L0oooo000G0?ooo`030000003oool0oooo00D0oooo0`00
003<0?ooo`H00000;03oool001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`37
0?ooo`D00000<P3oool001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`310?oo
o`H00000=`3oool001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2l0?ooo`D0
0000?@3oool001T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2h0?ooo`@00000
@P3oool001T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2d0?ooo`@00000AP3o
ool001T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2`0?ooo`@00000BP3oool0
01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2Z0?ooo`H00000CP3oool001X0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2V0?ooo`@00000E03oool001X0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2R0?ooo`@00000F03oool001/0oooo00D0
00000?ooo`3oool0oooo0000002P0?ooo`@00000G03oool001/0oooo00@000000?ooo`3oool0oooo
0`00000]0?ooo`@00000<03oool00`000000oooo0?ooo`0]0?ooo`8000001`3oool4000002D0oooo
0P00000[0?ooo`@000000`3oool2000000D0oooo000K0?ooo`050000003oool0oooo0?ooo`000000
;`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo02`0oooo00@000000?ooo`3oool0
00000P3oool4000002P0oooo00@000000?ooo`3oool00000;03oool01@000000oooo0?ooo`3oool0
00000080oooo00<000000?ooo`3oool00P3oool001`0oooo00@000000?ooo`3oool00000<03oool0
0`000000oooo0?ooo`0]0?ooo`D00000;@3oool00`000000oooo00000003000002`0oooo00@00000
0?ooo`3oool00000;03oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0
0P3oool001`0oooo00@000000?ooo`3oool00000<@3oool00`000000oooo0?ooo`0/0?ooo`040000
003oool0oooo000002d0oooo1000000`0?ooo`800000;@3oool01@000000oooo0?ooo`3oool00000
0080oooo00<000000?ooo`3oool00P3oool001d0oooo00<000000?ooo`000000<P3oool00`000000
oooo0?ooo`0/0?ooo`030000003oool0000002T0oooo100000000`3oool000000?ooo`0`0?ooo`04
0000003oool0oooo000002`0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0
oooo0080oooo000M0?ooo`030000003oool0000002l0oooo00@000000?ooo`3oool00000;`3oool2
000002D0oooo100000050?ooo`030000003oool0oooo02l0oooo00@000000?ooo`3oool00000:P3o
ool3000000<0oooo00@000000?ooo`3oool00000103oool001h0oooo0P00000`0?ooo`800000<@3o
ool00`000000oooo0?ooo`0O0?ooo`@000002P3oool3000002l0oooo0P00000]0?ooo`030000003o
ool0oooo0080oooo0P0000050?ooo`007P3oool200000840oooo1000001k0?ooo`007P3oool20000
07d0oooo1000001o0?ooo`007`3oool00`000000oooo0?ooo`1h0?ooo`<00000P`3oool001l0oooo
00<000000?ooo`3oool0M03oool4000008H0oooo003o0000020000000@3oool000L0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`8000002`3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00T0oooo0`0000090?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool01P3oool001l0oooo00<000000?ooo`000000J@3oool400000940oooo000O0?ooo`03
0000003oool0000006D0oooo1000002E0?ooo`007`3oool010000000oooo0?ooo`00001Q0?ooo`<0
0000V@3oool001l0oooo00@000000?ooo`3oool00000GP3oool3000009`0oooo000O0?ooo`050000
003oool0oooo0?ooo`000000FP3oool3000009l0oooo000O0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0E03oool300000:80oooo000O0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0D03oool300000:D0oooo000O0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0B`3oool400000:P0oooo000O0?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool0AP3oool400000:`0oooo000O0?ooo`<000001P3oool00`000000oooo0?ooo`110?ooo`@0
0000/03oool001l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0l0?ooo`@00000
]03oool001l0oooo00<000000?ooo`3oool0203oool2000003P0oooo1000002h0?ooo`007`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0380oooo0`00002l0?ooo`007`3oool00`00
0000oooo0?ooo`0;0?ooo`800000;P3oool400000;l0oooo000O0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool09`3oool400000<<0oooo000O0?ooo`030000003oool0oooo00h0oooo
0P00000T0?ooo`<00000a`3oool001l0oooo00<000000?ooo`3oool0403oool2000001h0oooo1000
003:0?ooo`007`3oool00`000000oooo0?ooo`0B0?ooo`@000005P3oool400000<h0oooo000O0?oo
o`030000003oool0oooo01H0oooo5P00003B0?ooo`007`3oool00`000000oooo0?ooo`3n0?ooo`00
7`3oool300000?h0oooo000O0?ooo`030000003oool0oooo0?h0oooo000O0?ooo`030000003oool0
oooo0?h0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.28933, -0.691541, \
0.0403437, 0.00454551}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(\(YES!\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-\((alomz[1, 0.9, 1, a])\)\), {a, \(-1\), 1000}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0247609 0.00095143 0.272727 0.827694 [
[.21505 .26023 -9 -9 ]
[.21505 .26023 9 0 ]
[.40533 .26023 -9 -9 ]
[.40533 .26023 9 0 ]
[.59562 .26023 -9 -9 ]
[.59562 .26023 9 0 ]
[.7859 .26023 -9 -9 ]
[.7859 .26023 9 0 ]
[.97619 .26023 -12 -9 ]
[.97619 .26023 12 0 ]
[.01226 .02442 -24 -4.5 ]
[.01226 .02442 0 4.5 ]
[.01226 .10719 -24 -4.5 ]
[.01226 .10719 0 4.5 ]
[.01226 .18996 -24 -4.5 ]
[.01226 .18996 0 4.5 ]
[.01226 .3555 -18 -4.5 ]
[.01226 .3555 0 4.5 ]
[.01226 .43827 -18 -4.5 ]
[.01226 .43827 0 4.5 ]
[.01226 .52103 -18 -4.5 ]
[.01226 .52103 0 4.5 ]
[.01226 .6038 -18 -4.5 ]
[.01226 .6038 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21505 .27273 m
.21505 .27898 L
s
[(200)] .21505 .26023 0 1 Mshowa
.40533 .27273 m
.40533 .27898 L
s
[(400)] .40533 .26023 0 1 Mshowa
.59562 .27273 m
.59562 .27898 L
s
[(600)] .59562 .26023 0 1 Mshowa
.7859 .27273 m
.7859 .27898 L
s
[(800)] .7859 .26023 0 1 Mshowa
.97619 .27273 m
.97619 .27898 L
s
[(1000)] .97619 .26023 0 1 Mshowa
.125 Mabswid
.07233 .27273 m
.07233 .27648 L
s
.1199 .27273 m
.1199 .27648 L
s
.16748 .27273 m
.16748 .27648 L
s
.26262 .27273 m
.26262 .27648 L
s
.31019 .27273 m
.31019 .27648 L
s
.35776 .27273 m
.35776 .27648 L
s
.4529 .27273 m
.4529 .27648 L
s
.50048 .27273 m
.50048 .27648 L
s
.54805 .27273 m
.54805 .27648 L
s
.64319 .27273 m
.64319 .27648 L
s
.69076 .27273 m
.69076 .27648 L
s
.73833 .27273 m
.73833 .27648 L
s
.83348 .27273 m
.83348 .27648 L
s
.88105 .27273 m
.88105 .27648 L
s
.92862 .27273 m
.92862 .27648 L
s
.25 Mabswid
0 .27273 m
1 .27273 L
s
.02476 .02442 m
.03101 .02442 L
s
[(-0.3)] .01226 .02442 1 0 Mshowa
.02476 .10719 m
.03101 .10719 L
s
[(-0.2)] .01226 .10719 1 0 Mshowa
.02476 .18996 m
.03101 .18996 L
s
[(-0.1)] .01226 .18996 1 0 Mshowa
.02476 .3555 m
.03101 .3555 L
s
[(0.1)] .01226 .3555 1 0 Mshowa
.02476 .43827 m
.03101 .43827 L
s
[(0.2)] .01226 .43827 1 0 Mshowa
.02476 .52103 m
.03101 .52103 L
s
[(0.3)] .01226 .52103 1 0 Mshowa
.02476 .6038 m
.03101 .6038 L
s
[(0.4)] .01226 .6038 1 0 Mshowa
.125 Mabswid
.02476 .04097 m
.02851 .04097 L
s
.02476 .05753 m
.02851 .05753 L
s
.02476 .07408 m
.02851 .07408 L
s
.02476 .09063 m
.02851 .09063 L
s
.02476 .12374 m
.02851 .12374 L
s
.02476 .1403 m
.02851 .1403 L
s
.02476 .15685 m
.02851 .15685 L
s
.02476 .1734 m
.02851 .1734 L
s
.02476 .20651 m
.02851 .20651 L
s
.02476 .22307 m
.02851 .22307 L
s
.02476 .23962 m
.02851 .23962 L
s
.02476 .25617 m
.02851 .25617 L
s
.02476 .28928 m
.02851 .28928 L
s
.02476 .30583 m
.02851 .30583 L
s
.02476 .32239 m
.02851 .32239 L
s
.02476 .33894 m
.02851 .33894 L
s
.02476 .37205 m
.02851 .37205 L
s
.02476 .3886 m
.02851 .3886 L
s
.02476 .40516 m
.02851 .40516 L
s
.02476 .42171 m
.02851 .42171 L
s
.02476 .45482 m
.02851 .45482 L
s
.02476 .47137 m
.02851 .47137 L
s
.02476 .48793 m
.02851 .48793 L
s
.02476 .50448 m
.02851 .50448 L
s
.02476 .53759 m
.02851 .53759 L
s
.02476 .55414 m
.02851 .55414 L
s
.02476 .5707 m
.02851 .5707 L
s
.02476 .58725 m
.02851 .58725 L
s
.02476 .00786 m
.02851 .00786 L
s
.25 Mabswid
.02476 0 m
.02476 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02481 0 m
.02499 .31941 L
.02605 .44968 L
.02729 .52012 L
.02846 .55566 L
.02954 .57523 L
.03053 .5867 L
.03163 .59467 L
.03279 .59966 L
.03395 .60228 L
.0352 .60332 L
.03638 .60314 L
.03746 .60228 L
.03873 .60069 L
.04011 .59846 L
.04262 .59357 L
.06244 .55211 L
.08429 .51884 L
.10458 .49675 L
.1251 .47971 L
.14371 .46734 L
.18378 .44705 L
.22233 .43272 L
.26333 .42091 L
.30282 .41178 L
.34079 .40448 L
.38121 .39788 L
.42012 .39242 L
.46148 .38735 L
.50133 .38305 L
.53966 .37935 L
.58044 .37581 L
.61971 .37272 L
.65746 .37001 L
.69766 .36736 L
.73635 .365 L
.77749 .36269 L
.81711 .36062 L
.85522 .35876 L
.89578 .35691 L
.93483 .35524 L
.97619 .35358 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`00203oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?oo
o`3oool0o`3oool30?ooo`001`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo
0080oooo0P00003o0?ooo`@0oooo00000`3oool000000000000200000080oooo00@000000?ooo`3o
ool000002@3oool2000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`001`3oool010000000
oooo0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool30?oo
o`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0o`3oool30?ooo`00203oool2000000X0oooo0P0000050?ooo`800000o`3oool40?oo
o`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo
0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool200000?l0oooo103oool0
01/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0
oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K
0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?oo
o`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo
0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool0
00P0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo
0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool000002@3oool00`00
0000oooo0?ooo`040?ooo`800000o`3oool40?ooo`0000<0oooo0000000000000P0000020?ooo`04
0000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo00070?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`3o0?ooo`<0oooo00080?ooo`8000002P3oool20000
00D0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo
00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K
0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`03
0000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3o
ool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`800000o`3oool40?ooo`006`3oool00`00
0000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo
00<000000?ooo`3oool0o`3oool30?ooo`006`3oool200000?l0oooo103oool001/0oooo00<00000
0?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00080?ooo`80
0000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`3o0?ooo`<0
oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool00`3oool200000?l0oooo103oool000030?ooo`0000000000008000000P3oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
1`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0o`3oool30?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo00080?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`800000
o`3oool40?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool200000?l0
oooo103oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo02H0oooo100000030?ooo`800000103oool2
000002D0oooo00<000000?ooo`3oool00P3oool2000000@0oooo0P00000S0?ooo`800000103oool2
000000@0oooo0P00000T0?ooo`800000103oool2000000@0oooo0P00000P0?ooo`@000000`3oool2
000000@0oooo0P0000040?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000903o
ool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`0O0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`00000R0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0
00000P3oool010000000oooo0?ooo`00000Q0?ooo`050000003oool0oooo0?ooo`0000000P3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo0@0000010?ooo`40oooo
000K0?ooo`800000:03oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool000008@3oool500000080oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`00000Q0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`00000R0?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool000000P3oool010000000oooo0?ooo`00000Q0?ooo`050000003oool0oooo0?ooo`000000
0P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo0@0000010?oo
o`40oooo000K0?ooo`030000003oool0oooo02P0oooo00D000000?ooo`3oool0oooo000000020?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool07`3oool010000000oooo0?ooo`00
00030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000008@3oool3000000<0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`00000S0?ooo`8000000`3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000240oooo00D000000?ooo`3oool0
oooo000000020?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool1
00000040oooo0@3oool001/0oooo00<000000?ooo`3oool0:@3oool010000000oooo0?ooo`000002
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0803oool00`000000oooo0000
00030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000008@3oool00`000000
oooo0?ooo`030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000008P3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0
00008@3oool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000020?ooo`4000000@3oool10?ooo`006`3oool00`000000oooo0?ooo`0V
0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`00000S0?ooo`8000000`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo
00000240oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`0000020?ooo`040000003o
ool0oooo00000280oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo000001l0oooo0`0000030?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`006`3oool2000002P0oooo
0P0000040?ooo`800000103oool2000002D0oooo00<000000?ooo`3oool00P3oool2000000@0oooo
0P00000S0?ooo`<000000`3oool2000000@0oooo0P00000T0?ooo`800000103oool2000000@0oooo
0P00000R0?ooo`030000003oool0oooo0080oooo0P0000040?ooo`800000103oool2000000<0oooo
000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000D0?ooool000001`0000060?ooo`00
6`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool001/0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0_0?ooo`030000003o
ool0oooo02h0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0_0?ooo`030000003o
ool0oooo00X0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?oo
o`@0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0
o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0oooo0?l0
oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000K0?ooo`800000o`3oool40?ooo`006`3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0
o`3oool30?ooo`006`3oool200000?l0oooo103oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00080?ooo`800000103oool00`000000
oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000
003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3`0?ooo`H0
0000303oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`80
0000g03ooolE00000180oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`360?oooaD000009`3oool000L0oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0;40oooo5@00000l0?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000oooo0?ooo`2W0?ooo`X0
0000D@3oool000P0oooo0P00000;0?ooo`030000003oool0oooo00<0oooo0P00002D0?oooa@00000
F`3oool001/0oooo00<000000?ooo`3oool0R03oool;000006l0oooo000K0?ooo`030000003oool0
oooo07h0oooo2P00001j0?ooo`006`3oool00`000000oooo0?ooo`1d0?ooo`X00000Q03oool001/0
oooo0P00001Z0?ooo`/00000SP3oool001/0oooo00<000000?ooo`3oool0HP3oool7000009T0oooo
000K0?ooo`030000003oool0oooo05d0oooo1@00002P0?ooo`006`3oool00`000000oooo0?ooo`1E
0?ooo`P00000Y@3oool001/0oooo00<000000?ooo`3oool0C@3oool800000:d0oooo000K0?ooo`80
0000B@3oool500000;D0oooo000K0?ooo`030000003oool0oooo04@0oooo1000002j0?ooo`006`3o
ool00`000000oooo0?ooo`100?ooo`@00000_P3oool001/0oooo00<000000?ooo`3oool0?@3oool3
00000<80oooo000K0?ooo`800000>`3oool300000<D0oooo000K0?ooo`030000003oool0oooo03H0
oooo100000380?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000@0oooo
00<000000?ooo`3oool0<P3oool400000<`0oooo00070?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0_0?ooo`<00000d03oool000L0oooo00@0
00000?ooo`3oool000002@3oool00`000000oooo0?ooo`040?ooo`800000;P3oool200000=<0oooo
00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0Z0?ooo`<00000e@3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo02P0oooo0P00003H0?ooo`001`3oool010000000oooo0?oo
o`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool09@3oool300000=X0
oooo00080?ooo`8000002P3oool2000000D0oooo00<000000?ooo`3oool08`3oool200000=d0oooo
000K0?ooo`8000008P3oool200000=l0oooo000K0?ooo`030000003oool0oooo01l0oooo0P00003Q
0?ooo`006`3oool00`000000oooo0?ooo`0M0?ooo`800000h`3oool001/0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`3S0?ooo`006`3oool2000001/0oooo0P00003V0?ooo`006`3o
ool00`000000oooo0?ooo`0H0?ooo`800000j03oool001/0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`3X0?ooo`006`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
0>T0oooo000K0?ooo`8000005P3oool00`000000oooo0?ooo`3Z0?ooo`006`3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo0>/0oooo000K0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0k03oool001/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3]
0?ooo`006`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0>h0oooo000K0?ooo`80
0000403oool200000?40oooo000K0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
l@3oool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0lP3oool000L0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
l`3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?ooo`800000
303oool00`000000oooo0?ooo`3d0?ooo`001`3oool010000000oooo0?ooo`0000090?ooo`800000
1@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?@0oooo00070?ooo`040000003o
ool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo0?D0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
0000103oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?D0oooo00080?ooo`800000
2P3oool2000000D0oooo0P00000:0?ooo`030000003oool0oooo0?H0oooo000K0?ooo`8000002P3o
ool00`000000oooo0?ooo`3f0?ooo`006`3oool2000000T0oooo00<000000?ooo`3oool0m`3oool0
01/0oooo0P0000090?ooo`030000003oool0oooo0?L0oooo000K0?ooo`800000203oool00`000000
oooo0?ooo`3h0?ooo`006`3oool2000000L0oooo00<000000?ooo`3oool0n@3oool001/0oooo0P00
00070?ooo`030000003oool0oooo0?T0oooo000K0?ooo`8000001P3oool00`000000oooo0?ooo`3j
0?ooo`006`3oool2000000H0oooo00<000000?ooo`3oool0nP3oool001/0oooo0P0000050?ooo`03
0000003oool0oooo0?/0oooo000K0?ooo`8000001@3oool00`000000oooo0?ooo`3k0?ooo`006`3o
ool2000000@0oooo00<000000?ooo`3oool0o03oool001/0oooo0P0000040?ooo`030000003oool0
oooo0?`0oooo000K0?ooo`8000000`3oool00`000000oooo0?ooo`3m0?ooo`006`3oool2000000<0
oooo00<000000?ooo`3oool0o@3oool000P0oooo0P0000040?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool00`3oool01@000000oooo0000003oool000000?l0oooo0@3oool000L0oooo
00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`050000003oool000000?oo
o`000000o`3oool10?ooo`001`3oool010000000oooo0?ooo`0000070?ooo`D00000103oool40000
0?l0oooo0P3oool000L0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo000000050?ooo`030000003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?oo
o`3oool000002@3oool2000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`00203oool20000
00/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo003o0?ooob40
oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40
oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-110.019, -0.364798, \
4.03645, 0.00463988}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(\(never!\)\)], "Input"],

Cell[BoxData[
    \(\(\(=\)\(\(>\)\(\ \)\(The\ second\ mode\ has\ instabilities\ only\ for\ \
certain\ values\ of\ \(kx!\)\)\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-\((alomz[1, Cos[Pi/4], 1, a] + 
            gamoz[1, Cos[Pi/4], 1, a])\)\), {a, 0, 200}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0047619 0.603319 3.0359 [
[.2619 .59082 -6 -9 ]
[.2619 .59082 6 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .14793 -30 -4.5 ]
[.01131 .14793 0 4.5 ]
[.01131 .29973 -24 -4.5 ]
[.01131 .29973 0 4.5 ]
[.01131 .45152 -30 -4.5 ]
[.01131 .45152 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(50)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(100)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(150)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(200)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .14793 m
.03006 .14793 L
s
[(-0.15)] .01131 .14793 1 0 Mshowa
.02381 .29973 m
.03006 .29973 L
s
[(-0.1)] .01131 .29973 1 0 Mshowa
.02381 .45152 m
.03006 .45152 L
s
[(-0.05)] .01131 .45152 1 0 Mshowa
.125 Mabswid
.02381 .0265 m
.02756 .0265 L
s
.02381 .05686 m
.02756 .05686 L
s
.02381 .08722 m
.02756 .08722 L
s
.02381 .11757 m
.02756 .11757 L
s
.02381 .17829 m
.02756 .17829 L
s
.02381 .20865 m
.02756 .20865 L
s
.02381 .23901 m
.02756 .23901 L
s
.02381 .26937 m
.02756 .26937 L
s
.02381 .33009 m
.02756 .33009 L
s
.02381 .36045 m
.02756 .36045 L
s
.02381 .39081 m
.02756 .39081 L
s
.02381 .42116 m
.02756 .42116 L
s
.02381 .48188 m
.02756 .48188 L
s
.02381 .51224 m
.02756 .51224 L
s
.02381 .5426 m
.02756 .5426 L
s
.02381 .57296 m
.02756 .57296 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.02846 .45889 L
.03279 .31709 L
.03527 .25604 L
.03754 .21129 L
.04018 .16942 L
.04262 .1386 L
.0475 .09343 L
.05008 .07619 L
.05281 .06158 L
.05739 .04349 L
.06 .03597 L
.06244 .03036 L
.06504 .02563 L
.06788 .0217 L
.07056 .01896 L
.07184 .01794 L
.07305 .01713 L
.07423 .01646 L
.07551 .01588 L
.07667 .01546 L
.07774 .01517 L
.0789 .01493 L
.08017 .01478 L
.08081 .01473 L
.0815 .01472 L
.08274 .01474 L
.08345 .0148 L
.0842 .01488 L
.08484 .01497 L
.08555 .01509 L
.08713 .01544 L
.08861 .01585 L
.09417 .01803 L
.09918 .02068 L
.10458 .02406 L
.14572 .05665 L
.18534 .08807 L
.22345 .11477 L
.26401 .13938 L
.30306 .15994 L
.34456 .17901 L
.38454 .19519 L
.423 .20907 L
.46392 .22232 L
.50333 .23385 L
.54121 .24397 L
.58155 .25383 L
.62038 .26256 L
.66165 .27113 L
Mistroke
.70141 .27878 L
.73966 .28563 L
.78036 .29245 L
.81954 .29859 L
.86117 .30472 L
.90129 .31028 L
.9399 .31533 L
.97619 .31983 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool00`000000oooo0?oo
o`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?oo
o`008@3oool00`000000oooo0?ooo`0;0?ooo`<00000kP3oool00240oooo00<000000?ooo`3oool0
2@3oool3000000040?ooo`000000000000000>`0oooo000Q0?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3Y0?ooo`008@3oool2000000P0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`3X0?ooo`008@3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00P0oooo0P00003X0?ooo`008@3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0i@3oool00240oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0>@0oooo000Q0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool03@3oool200000>@0oooo000Q0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3Q0?ooo`008@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0h03oool0
0240oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0B0?ooo`800000h03oool00240
oooo0P0000050?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0g@3oool00240oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0=`0oooo
000Q0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool05`3oool200000=`0oooo000Q
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`3I
0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool0f03oool00240oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo0=L0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
7@3oool200000=L0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`3D0?ooo`008@3oool2000000@0oooo00<000000?ooo`3oool0803oool0
0`000000oooo0?ooo`3C0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
0240oooo0P00003C0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02<0
oooo00<000000?ooo`3oool0d03oool00240oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0T0?ooo`030000003oool0oooo0<l0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool09@3oool200000<l0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`3<0?ooo`008@3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo02P0oooo0P00003<0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo02X0oooo00<000000?ooo`3oool0b@3oool00240oooo0P0000030?ooo`030000
003oool0oooo02`0oooo0P0000390?ooo`008@3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo02h0oooo0P0000370?ooo`008@3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0300oooo00<000000?ooo`3oool0a03oool00240oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0a0?ooo`800000a03oool00240oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0c0?ooo`800000`P3oool00240oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0e0?ooo`030000003oool0oooo0;l0oooo00080?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`@000000P3oool3000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0f0?ooo`800000_`3oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0h0?ooo`800000_@3oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0080oooo0P0000030?ooo`030000003oool0oooo03X0oooo00<0
00000?ooo`3oool0^P3oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`00
000:0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo03/0oooo0P00002j0?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`050000
003oool0oooo0?ooo`0000001`3oool01@000000oooo0?ooo`3oool000000400oooo0P00002h0?oo
o`001`3oool010000000oooo0?ooo`0000080?ooo`<000000`3oool00`000000oooo0?ooo`050?oo
o`050000003oool0oooo0?ooo`000000@P3oool200000;H0oooo00080?ooo`8000002`3oool01000
0000oooo0?ooo`3oool4000000@0oooo00D000000?ooo`3oool0oooo000000140?ooo`800000]03o
ool00240oooo00D000000?ooo`3oool0oooo000000160?ooo`800000/P3oool00240oooo00D00000
0?ooo`3oool0oooo000000180?ooo`800000/03oool00240oooo0P0000020?ooo`030000003oool0
oooo04P0oooo0P00002^0?ooo`008@3oool01@000000oooo0?ooo`3oool0000004`0oooo0`00002[
0?ooo`008@3oool01@000000oooo0?ooo`3oool0000004l0oooo0P00002Y0?ooo`008@3oool01@00
0000oooo0?ooo`3oool000000540oooo0`00002V0?ooo`008@3oool010000000oooo0?ooo`00001E
0?ooo`800000Y03oool00240oooo00@000000?ooo`3oool00000E`3oool300000:40oooo000Q0?oo
o`040000003oool0oooo000005X0oooo0`00002N0?ooo`008@3oool010000000oooo0?ooo`00001M
0?ooo`@00000VP3oool00240oooo0P0000000`3oool000000?ooo`1P0?ooo`<00000U`3oool00240
oooo00@000000?ooo`3oool00000I03oool3000009@0oooo000Q0?ooo`040000003oool0oooo0000
06L0oooo0P00002B0?ooo`008@3oool010000000oooo0?ooo`00001Y0?ooo`<00000S`3oool00240
oooo00@000000?ooo`3oool00000K03oool3000008`0oooo000Q0?ooo`040000003oool0oooo0000
06l0oooo0`0000290?ooo`008@3oool010000000oooo0?ooo`00001b0?ooo`@00000Q@3oool00240
oooo00@000000?ooo`3oool00000MP3oool400000840oooo000Q0?ooo`80000000<0oooo0000003o
ool0N@3oool5000007`0oooo000Q0?ooo`040000003oool0oooo000007l0oooo1000001h0?ooo`00
8@3oool010000000oooo0?ooo`0000230?ooo`<00000M@3oool00240oooo00@000000?ooo`3oool0
0000QP3oool400000740oooo000Q0?ooo`040000003oool0oooo000008X0oooo1000001]0?ooo`00
8@3oool010000000oooo0?ooo`00002>0?ooo`D00000J03oool00240oooo00@000000?ooo`3oool0
0000T`3oool5000006<0oooo000Q0?ooo`040000003oool0oooo000009P0oooo1P00001M0?ooo`00
8@3oool2000000030?ooo`000000oooo09d0oooo1@00001H0?ooo`008@3oool010000000oooo0?oo
o`00002S0?ooo`D00000D`3oool00240oooo00@000000?ooo`3oool00000Z03oool5000004h0oooo
000Q0?ooo`040000003oool0oooo00000:d0oooo1@0000190?ooo`008@3oool010000000oooo0?oo
o`00002b0?ooo`D00000A03oool00240oooo00<000000?ooo`000000^03oool5000003l0oooo000>
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`000000oooo0000002m
0?ooo`L00000>03oool000d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool000000<@0oooo2000000`0?ooo`003@3oool010000000oooo0?ooo`00000:
0?ooo`030000003oool0oooo00<0oooo0`00003<0?ooo`H00000:P3oool000L0oooo100000020?oo
o`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0000003B
0?ooo`L000008`3oool000d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool000000=T0oooo2000000K0?ooo`003@3oool010000000oooo0?ooo`000008
0?ooo`<000001@3oool00`000000oooo0000003Q0?ooo`D000005P3oool000h0oooo0P00000;0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`000000iP3oool7000000l0oooo000Q0?ooo`03
0000003oool000000>d0oooo1P0000090?ooo`008@3oool00`000000oooo0000003l0?ooo`008@3o
ool300000?`0oooo000Q0?ooo`030000003oool000000?`0oooo000Q0?ooo`030000003oool00000
0?`0oooo000Q0?ooo`030000003oool000000?`0oooo000Q0?ooo`030000003oool000000?`0oooo
000Q0?ooo`030000003oool000000?`0oooo000Q0?ooo`030000003oool000000?`0oooo000Q0?oo
o`030000003oool000000?`0oooo000Q0?ooo`<00000o03oool00240oooo00<000000?ooo`000000
o03oool00240oooo00<000000?ooo`000000o03oool00240oooo00<000000?ooo`000000o03oool0
0240oooo00<000000?ooo`000000o03oool00240oooo00<000000?ooo`000000o03oool00240oooo
00<000000?ooo`000000o03oool00240oooo00<000000?ooo`000000o03oool00240oooo0P00003m
0?ooo`008@3oool200000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3o
ool200000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0
oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo000Q0?oo
o`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo00080?ooo`800000103o
ool00`000000oooo0?ooo`030?ooo`8000000`3oool3000000D0oooo0P00003m0?ooo`001`3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0
0P3oool200000?d0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
00001@3oool00`000000oooo0?ooo`020?ooo`800000o@3oool000030?ooo`000000000000800000
0P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`0000050?oo
o`800000o@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo00D0oooo0P00003m0?ooo`001`3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool200000?d0oooo0008
0?ooo`8000002P3oool2000000<0oooo100000040?ooo`800000o@3oool00240oooo0P00003m0?oo
o`008@3oool200000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool2
00000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo
000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo000Q0?ooo`80
0000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo000Q0?ooo`800000o@3oool0
0240oooo0P00003m0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo
0?ooo`3l0?ooo`008@3oool200000?d0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?oo
o`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?ooo`030000
003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo03D0oooo0`0000040?ooo`800000<@3o
ool4000000<0oooo0P0000040?ooo`800000;P3oool400000080oooo0`0000040?ooo`800000;`3o
ool4000000<0oooo0P0000040?ooo`8000000`3oool00240oooo00<000000?ooo`3oool0>03oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0300oooo00D000000?ooo`3oool0oooo
000000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0;@3oool00`000000
oooo0?ooo`040?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0;03oool00`00
0000oooo0?ooo`030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3o
ool00240oooo00<000000?ooo`3oool0>03oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0300oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool000000P3oool00240oooo0P00000f0?ooo`<000000`3oool0
10000000oooo0?ooo`00000b0?ooo`050000003oool0oooo0?ooo`0000000P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo02d0oooo00@000000?ooo`3oool0oooo0`0000030?oo
o`040000003oool0oooo00000300oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003o
ool0oooo00000080oooo0@0000010?ooo`40oooo000Q0?ooo`030000003oool0oooo03D0oooo00<0
00000?ooo`3oool00`3oool010000000oooo0?ooo`00000b0?ooo`050000003oool0oooo0?ooo`00
00000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo02d0oooo00D000000?oo
o`3oool0oooo000000050?ooo`040000003oool0oooo00000340oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`4000000@3oool10?ooo`008@3oool00`000000oooo
0?ooo`0e0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool00000<03oool3000000<0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`00000]0?ooo`<000000`3oool0
0`000000oooo0?ooo`030?ooo`040000003oool0oooo000002h0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo000Q0?ooo`03
0000003oool0oooo03D0oooo100000030?ooo`800000<`3oool00`000000oooo0?ooo`020?ooo`80
0000103oool200000300oooo00@000000?ooo`3oool0oooo100000030?ooo`800000<03oool20000
00@0oooo0P0000040?ooo`8000000`3oool00240oooo00<000000?ooo`3oool0o03oool00240oooo
00<000000?ooo`3oool0o03oool00240oooo00<000000?ooo`3oool0o03oool00240oooo00<00000
0?ooo`3oool0o03oool001/0ooooo`000003000000<0oooo000Q0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`070?ooo`008@3oool00`000000oooo
0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?oo
o`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?oo
o`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-27.1372, -0.210319, \
0.817044, 0.00128156}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[""], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["prop mode II", "Subsection"],

Cell[BoxData[
    \(testfunc[om_, k_, th_, 
        a_] := \((om^2 - k^2 - 
              Sqrt[1 + a] 
                FastRealpha[1, k, k*Cos[th], 
                  IntegerPart[om*10^16]*10^\((\(-16\))\), a] - 
              Sqrt[1 + a] 
                FastRegamma[1, k, k*Cos[th], 
                  IntegerPart[om*10^16]*10^\((\(-16\))\), a])\)*\((1 - 
              Sqrt[1 + a] 
                FastRebeta[1, k, k*Cos[th], 
                    IntegerPart[om*10^16]*10^\((\(-16\))\), a]/
                  om^2)\) - \((1 + a)\) 
          FastRedelta[1, k, k*Cos[th], 
                IntegerPart[om*10^16]*10^\((\(-16\))\), a]^2/
            om^2*\((1 - Cos[th]^2)\)*k^2\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rebeta[1, 14/10, 14/10*Cos[Pi/4], 15/10, 1]\)], "Input"],

Cell[BoxData[
    \(0.708614521859252877752748`9.7426\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(omtwo[k_, th_, a_] := 
      om /. FindRoot[
          testfunc[om, IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), th, 
              a] \[Equal] 0, {om, k + 10^\((\(-10\))\), k + 10^\((\(-5\))\)}, 
          MaxIterations \[Rule] 25]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, omtwo[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, 10]}, {T, 
        0.001, 2, 1/20}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(25\) iterations."\)], "Message"],

Cell[BoxData[
    \({{0.001`, 0.23434838204376904`}, {0.051000000000000004`, 
        0.6198593540011474`}, {0.101`, 
        0.6266148666694963`}, {0.15100000000000002`, 
        0.63747366580586`}, {0.201`, 0.6519730801048738`}, {0.251`, 
        0.6695405983989737`}, {0.301`, 0.6895344517469927`}, {0.351`, 
        0.7112950123789659`}, {0.40099999999999997`, 
        0.7341994281665168`}, {0.45099999999999996`, 
        0.7577558059530454`}, {0.501`, 0.7816767090459024`}, {0.551`, 
        0.8059180715712794`}, {0.6010000000000001`, 
        0.830620157084095`}, {0.6510000000000001`, 
        0.8560063857241433`}, {0.7010000000000002`, 
        0.8823109586764857`}, {0.7510000000000002`, 
        0.9097334244727813`}, {0.8010000000000003`, 
        0.9384363708841587`}, {0.8510000000000003`, 
        0.968548237934262`}, {0.9010000000000004`, 
        1.0001702565525605`}, {0.9510000000000004`, 
        1.033383975516456`}, {1.0010000000000003`, 
        1.0682544298193968`}, {1.0510000000000004`, 
        1.104829212332863`}, {1.1010000000000004`, 
        1.1431353913634814`}, {1.1510000000000005`, 
        1.1831729715272208`}, {1.2010000000000005`, 
        1.2249041173475486`}, {1.2510000000000006`, 
        1.2682457039939228`}, {1.3010000000000006`, 
        1.3130638706354913`}, {1.3510000000000006`, 
        1.3591781424207892`}, {1.4010000000000007`, 
        1.4063746040506127`}, {1.4510000000000007`, 
        1.4544283794832769`}, {1.5010000000000008`, 
        1.503126361188498`}, {1.5510000000000008`, 
        1.5522847966160302`}, {1.6010000000000009`, 
        1.6017576592971392`}, {1.651000000000001`, 
        1.6514367537663635`}, {1.701000000000001`, 
        1.7012464110123222`}, {1.751000000000001`, 
        1.7511361882155425`}, {1.801000000000001`, 
        1.8010737823188154`}, {1.851000000000001`, 
        1.8510392019884827`}, {1.9010000000000011`, 
        1.9010204323421451`}, {1.9510000000000012`, 
        1.9510104488113773`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.051000000000000004`, 0.6198593540011474`}, {0.101`, 
        0.6266148666694963`}, {0.15100000000000002`, 
        0.63747366580586`}, {0.201`, 0.6519730801048738`}, {0.251`, 
        0.6695405983989737`}, {0.301`, 0.6895344517469927`}, {0.351`, 
        0.7112950123789659`}, {0.40099999999999997`, 
        0.7341994281665168`}, {0.45099999999999996`, 
        0.7577558059530454`}, {0.501`, 0.7816767090459024`}, {0.551`, 
        0.8059180715712794`}, {0.6010000000000001`, 
        0.830620157084095`}, {0.6510000000000001`, 
        0.8560063857241433`}, {0.7010000000000002`, 
        0.8823109586764857`}, {0.7510000000000002`, 
        0.9097334244727813`}, {0.8010000000000003`, 
        0.9384363708841587`}, {0.8510000000000003`, 
        0.968548237934262`}, {0.9010000000000004`, 
        1.0001702565525605`}, {0.9510000000000004`, 
        1.033383975516456`}, {1.0010000000000003`, 
        1.0682544298193968`}, {1.0510000000000004`, 
        1.104829212332863`}, {1.1010000000000004`, 
        1.1431353913634814`}, {1.1510000000000005`, 
        1.1831729715272208`}, {1.2010000000000005`, 
        1.2249041173475486`}, {1.2510000000000006`, 
        1.2682457039939228`}, {1.3010000000000006`, 
        1.3130638706354913`}, {1.3510000000000006`, 
        1.3591781424207892`}, {1.4010000000000007`, 
        1.4063746040506127`}, {1.4510000000000007`, 
        1.4544283794832769`}, {1.5010000000000008`, 
        1.503126361188498`}, {1.5510000000000008`, 
        1.5522847966160302`}, {1.6010000000000009`, 
        1.6017576592971392`}, {1.651000000000001`, 
        1.6514367537663635`}, {1.701000000000001`, 
        1.7012464110123222`}, {1.751000000000001`, 
        1.7511361882155425`}, {1.801000000000001`, 
        1.8010737823188154`}, {1.851000000000001`, 
        1.8510392019884827`}, {1.9010000000000011`, 
        1.9010204323421451`}, {1.9510000000000012`, 
        1.9510104488113773`}}\)], "Input"],

Cell[BoxData[
    \({{0.051000000000000004`, 0.6198593540011474`}, {0.101`, 
        0.6266148666694963`}, {0.15100000000000002`, 
        0.63747366580586`}, {0.201`, 0.6519730801048738`}, {0.251`, 
        0.6695405983989737`}, {0.301`, 0.6895344517469927`}, {0.351`, 
        0.7112950123789659`}, {0.40099999999999997`, 
        0.7341994281665168`}, {0.45099999999999996`, 
        0.7577558059530454`}, {0.501`, 0.7816767090459024`}, {0.551`, 
        0.8059180715712794`}, {0.6010000000000001`, 
        0.830620157084095`}, {0.6510000000000001`, 
        0.8560063857241433`}, {0.7010000000000002`, 
        0.8823109586764857`}, {0.7510000000000002`, 
        0.9097334244727813`}, {0.8010000000000003`, 
        0.9384363708841587`}, {0.8510000000000003`, 
        0.968548237934262`}, {0.9010000000000004`, 
        1.0001702565525605`}, {0.9510000000000004`, 
        1.033383975516456`}, {1.0010000000000003`, 
        1.0682544298193968`}, {1.0510000000000004`, 
        1.104829212332863`}, {1.1010000000000004`, 
        1.1431353913634814`}, {1.1510000000000005`, 
        1.1831729715272208`}, {1.2010000000000005`, 
        1.2249041173475486`}, {1.2510000000000006`, 
        1.2682457039939228`}, {1.3010000000000006`, 
        1.3130638706354913`}, {1.3510000000000006`, 
        1.3591781424207892`}, {1.4010000000000007`, 
        1.4063746040506127`}, {1.4510000000000007`, 
        1.4544283794832769`}, {1.5010000000000008`, 
        1.503126361188498`}, {1.5510000000000008`, 
        1.5522847966160302`}, {1.6010000000000009`, 
        1.6017576592971392`}, {1.651000000000001`, 
        1.6514367537663635`}, {1.701000000000001`, 
        1.7012464110123222`}, {1.751000000000001`, 
        1.7511361882155425`}, {1.801000000000001`, 
        1.8010737823188154`}, {1.851000000000001`, 
        1.8510392019884827`}, {1.9010000000000011`, 
        1.9010204323421451`}, {1.9510000000000012`, 
        1.9510104488113773`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True, 
      PlotStyle \[Rule] {Dashing[{0.02, 0.02}]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.48815 -0.259372 0.442177 [
[.26788 .1703 -9 -9 ]
[.26788 .1703 9 0 ]
[.51196 .1703 -3 -9 ]
[.51196 .1703 3 0 ]
[.75603 .1703 -9 -9 ]
[.75603 .1703 9 0 ]
[.01131 .00593 -18 -4.5 ]
[.01131 .00593 0 4.5 ]
[.01131 .09437 -18 -4.5 ]
[.01131 .09437 0 4.5 ]
[.01131 .27124 -18 -4.5 ]
[.01131 .27124 0 4.5 ]
[.01131 .35967 -18 -4.5 ]
[.01131 .35967 0 4.5 ]
[.01131 .44811 -18 -4.5 ]
[.01131 .44811 0 4.5 ]
[.01131 .53655 -18 -4.5 ]
[.01131 .53655 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26788 .1828 m
.26788 .18905 L
s
[(0.5)] .26788 .1703 0 1 Mshowa
.51196 .1828 m
.51196 .18905 L
s
[(1)] .51196 .1703 0 1 Mshowa
.75603 .1828 m
.75603 .18905 L
s
[(1.5)] .75603 .1703 0 1 Mshowa
.125 Mabswid
.07262 .1828 m
.07262 .18655 L
s
.12144 .1828 m
.12144 .18655 L
s
.17025 .1828 m
.17025 .18655 L
s
.21907 .1828 m
.21907 .18655 L
s
.3167 .1828 m
.3167 .18655 L
s
.36551 .1828 m
.36551 .18655 L
s
.41433 .1828 m
.41433 .18655 L
s
.46314 .1828 m
.46314 .18655 L
s
.56077 .1828 m
.56077 .18655 L
s
.60959 .1828 m
.60959 .18655 L
s
.6584 .1828 m
.6584 .18655 L
s
.70722 .1828 m
.70722 .18655 L
s
.80485 .1828 m
.80485 .18655 L
s
.85366 .1828 m
.85366 .18655 L
s
.90248 .1828 m
.90248 .18655 L
s
.95129 .1828 m
.95129 .18655 L
s
.25 Mabswid
0 .1828 m
1 .1828 L
s
.02381 .00593 m
.03006 .00593 L
s
[(0.6)] .01131 .00593 1 0 Mshowa
.02381 .09437 m
.03006 .09437 L
s
[(0.8)] .01131 .09437 1 0 Mshowa
.02381 .27124 m
.03006 .27124 L
s
[(1.2)] .01131 .27124 1 0 Mshowa
.02381 .35967 m
.03006 .35967 L
s
[(1.4)] .01131 .35967 1 0 Mshowa
.02381 .44811 m
.03006 .44811 L
s
[(1.6)] .01131 .44811 1 0 Mshowa
.02381 .53655 m
.03006 .53655 L
s
[(1.8)] .01131 .53655 1 0 Mshowa
.125 Mabswid
.02381 .02804 m
.02756 .02804 L
s
.02381 .05015 m
.02756 .05015 L
s
.02381 .07226 m
.02756 .07226 L
s
.02381 .11648 m
.02756 .11648 L
s
.02381 .13859 m
.02756 .13859 L
s
.02381 .1607 m
.02756 .1607 L
s
.02381 .20491 m
.02756 .20491 L
s
.02381 .22702 m
.02756 .22702 L
s
.02381 .24913 m
.02756 .24913 L
s
.02381 .29335 m
.02756 .29335 L
s
.02381 .31546 m
.02756 .31546 L
s
.02381 .33757 m
.02756 .33757 L
s
.02381 .38178 m
.02756 .38178 L
s
.02381 .40389 m
.02756 .40389 L
s
.02381 .426 m
.02756 .426 L
s
.02381 .47022 m
.02756 .47022 L
s
.02381 .49233 m
.02756 .49233 L
s
.02381 .51444 m
.02756 .51444 L
s
.02381 .55865 m
.02756 .55865 L
s
.02381 .58076 m
.02756 .58076 L
s
.02381 .60287 m
.02756 .60287 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .02 .02 ] 0 setdash
.04871 .01472 m
.07311 .0177 L
.09752 .0225 L
.12193 .02892 L
.14634 .03668 L
.17074 .04552 L
.19515 .05515 L
.21956 .06527 L
.24397 .07569 L
.26837 .08627 L
.29278 .09699 L
.31719 .10791 L
.3416 .11913 L
.366 .13077 L
.39041 .14289 L
.41482 .15558 L
.43923 .1689 L
.46363 .18288 L
.48804 .19757 L
.51245 .21298 L
.53686 .22916 L
.56126 .2461 L
.58567 .2638 L
.61008 .28225 L
.63449 .30142 L
.65889 .32123 L
.6833 .34162 L
.70771 .36249 L
.73212 .38374 L
.75652 .40528 L
.78093 .42701 L
.80534 .44889 L
.82975 .47085 L
.85415 .49288 L
.87856 .51494 L
.90297 .53702 L
.92738 .55911 L
.95178 .58121 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003o
oooo0?ooo`Woo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3oool9ool00005
ool00?ooool00008ool01000ooooo`001?oo0`00oooo2Ooo00001Ooo003ooooo00002?oo0`001Ooo
00<00?ooool0oooo2Ooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool01?oo0`00
oooo0_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool01ooo0`00o_oo0002ool2
000:ool30004ool00`00ooooo`0>ool2003hool001Goo`03003ooooo013oo`D00??oo`005Ooo0P00
oooo2_oo000Eool00`00ooooo`0Iool3003/ool001Goo`03003ooooo01coo`<00>Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`0Sool00`00ooooo`3Rool001Goo`03003ooooo02Coo`<0
0>7oo`005Ooo0P00:?oo0P00gooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo02goo`80
0=Woo`005Ooo00<00?ooool0;ooo0P00eooo000Eool00`00ooooo`0aool2003Eool001Goo`03003o
oooo0?ooo`Woo`005Ooo0P00>?oo00<00?ooool0c_oo000Eool00`00ooooo`0hool2003>ool001Go
o`03003ooooo03[oo`800<coo`005Ooo00<00?ooool0??oo00<00?ooool0bOoo0002ool20004ool0
0`00ooooo`03ool20005ool00`00ooooo`3oool9ool00005ool00?ooool00008ool01000ooooo`00
1?oo00<00?ooool0@Ooo0P00aOoo00001Ooo003ooooo00002?oo00@00?ooool000Coo`<004?oo`80
0<?oo`0000Goo`00ooooo`0000Woo`8000Goo`03003ooooo04Goo`800<7oo`0000Goo`00ooooo`00
00Soo`04003ooooo0004ool00`00ooooo`3oool9ool00005ool00?ooool00008ool01000ooooo`00
1?oo00<00?ooool0Booo00<00?ooool0^_oo0002ool2000:ool20005ool00`00ooooo`1<ool3002i
ool001Goo`03003ooooo04ooo`800;Ooo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo05Goo`800;7oo`005Ooo00<00?ooool0Eooo0P00[ooo000Eool00`00ooooo`1I
ool2002]ool001Goo`03003ooooo0?ooo`Woo`005Ooo0P00H?oo00<00?ooool0Y_oo000Eool00`00
ooooo`1Pool2002Vool001Goo`03003ooooo03Soo`8000Coo`03003ooooo00;oo`<001coo`8001co
o`@003Soo`@000?oo`03003ooooo00;oo`<003goo`005Ooo00<00?ooool0=ooo00@00?ooool000_o
o`03003ooooo01_oo`03003ooooo01_oo`03003ooooo03Woo`03003ooooo00[oo`03003ooooo03[o
o`005Ooo00<00?ooool0=ooo00@00?ooool000_oo`03003ooooo03Woo`03003ooooo03Woo`03003o
oooo00[oo`03003ooooo03[oo`005Ooo00<00?ooool0=ooo00@00?ooool000Soo`<003coo`03003o
oooo03Woo`03003ooooo00Ooo`<003goo`005Ooo0P00>?oo00@00?ooool000Soo`03003ooooo02?o
o`03003ooooo01Koo`03003ooooo03Woo`03003ooooo00Ooo`03003ooooo03goo`005Ooo00<00?oo
ool0=ooo00@00?ooool000Soo`03003ooooo02Coo`8001Coo`<003Woo`<000Woo`03003ooooo03go
o`005Ooo00<00?ooool0>?oo0P002Ooo10009Ooo0P005?oo00<00?ooool0>Ooo00<00?ooool01ooo
1000??oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
L_oo0P00U?oo000>oooo000C000001Goo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_o
o`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00[o
o`03003ooooo00[oo`03003oo`0000_oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_o
o`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[o
o`03003ooooo00[oo`03003ooooo00coo`005Ooo00<00?ooool0?ooo00<00?ooool0=Ooo00<00?oo
ool02?oo00<00?ooool0?ooo00<00?ooool0@Ooo000Eool00`00ooooo`3oool9ool001Goo`03003o
oooo07_oo`03003ooooo08[oo`005Ooo00<00?ooool0O?oo0P00R_oo000Eool2001oool20028ool0
01Goo`03003ooooo083oo`03003ooooo08Goo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00oooo
o`3oool9ool001Goo`03003ooooo08Goo`03003ooooo083oo`005Ooo00<00?ooool0Q_oo00<00?oo
ool0Oooo000Eool20028ool00`00ooooo`1nool001Goo`03003ooooo08Soo`8007koo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo08koo`03003ooooo07Oo
o`005Ooo00<00?ooool0Sooo00<00?ooool0M_oo000Eool2002Aool00`00ooooo`1eool001Goo`03
003ooooo097oo`03003ooooo07Coo`005Ooo00<00?ooool0T_oo00<00?ooool0Looo000Eool00`00
ooooo`3oool9ool00003ool00000008000?oo`03003ooooo00;oo`@000Coo`03003ooooo0?ooo`Wo
o`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0U_oo00<00?ooool0Kooo0003ool0
0`00ooooo`08ool00`00ooooo`04ool3002Gool2001_ool000?oo`03003ooooo00Woo`03003ooooo
00?oo`03003ooooo09Woo`8006goo`000ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0
Vooo00<00?ooool0J_oo00001?oo000000002Ooo00@00?ooool000Coo`03003ooooo0?ooo`Woo`00
0ooo00<00?ooool02?oo0P001Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`2Oool00`00oooo
o`1Vool001Goo`800:7oo`03003ooooo06Goo`005Ooo00<00?ooool0XOoo0P00IOoo000Eool00`00
ooooo`2Sool00`00ooooo`1Rool001Goo`03003ooooo0:?oo`03003ooooo06;oo`005Ooo00<00?oo
ool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800:Woo`03003ooooo05goo`005Ooo00<0
0?ooool0ZOoo00<00?ooool0G?oo000Eool00`00ooooo`2Zool00`00ooooo`1Kool001Goo`03003o
oooo0:_oo`03003ooooo05[oo`005Ooo00<00?ooool0[?oo00<00?ooool0FOoo000Eool00`00oooo
o`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0/?oo00<00?ooool0EOoo000Eool00`00
ooooo`2aool00`00ooooo`1Dool001Goo`03003ooooo0;;oo`03003ooooo05?oo`0000?oo`000000
0P000ooo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0/ooo00<00?ooool0D_oo0003ool0
0`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2dool00`00ooooo`1Aool000?oo`03003ooooo
00Koo`D000Coo`<00?ooo`Woo`000ooo00<00?ooool01_oo00@00?ooool000Goo`03003ooooo0?oo
o`Woo`000ooo00<00?ooool01ooo00<00?oo00001Ooo00<00?ooool0^?oo00<00?ooool0COoo0000
1?oo000000002_oo0P001Ooo00<00?ooool0^Ooo00<00?ooool0C?oo0003ool00`00ooooo`09ool0
0`00ooooo`03ool00`00ooooo`2jool00`00ooooo`1;ool001Goo`03003ooooo0;_oo`03003ooooo
04[oo`005Ooo0P00_Ooo00<00?ooool0BOoo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`005Ooo00<00?ooool0`?oo00<00?ooool0AOoo000Eool00`00ooooo`31ool00`00oooo
o`14ool001Goo`03003ooooo0<;oo`03003ooooo04?oo`005Ooo0P00a?oo00<00?ooool0@_oo000E
ool00`00ooooo`34ool20012ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo
000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0<Woo`03003ooooo03coo`005Ooo0P00booo
00<00?ooool0>ooo000Eool00`00ooooo`3;ool00`00ooooo`0jool001Goo`03003ooooo0<coo`80
03[oo`005Ooo00<00?ooool0oooo2Ooo00000ooo000000020003ool00`00ooooo`03ool20005ool0
0`00ooooo`3oool9ool000?oo`03003ooooo00Ooo`04003ooooo0004ool00`00ooooo`3oool9ool0
00?oo`03003ooooo00Ooo`04003ooooo0004ool3003Aool00`00ooooo`0dool000?oo`03003ooooo
00Ooo`<000Goo`03003ooooo0=;oo`03003ooooo03?oo`000ooo00<00?ooool01ooo00<00?ooool0
1Ooo00<00?ooool0dooo0P00<ooo00001?oo000000002Ooo00<00?ooool01Ooo00<00?ooool0eOoo
00<00?ooool0<?oo0003ool00`00ooooo`08ool30004ool00`00ooooo`3oool9ool001Goo`03003o
oooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3Iool00`00ooooo`0/ool001Goo`03
003ooooo0=[oo`03003ooooo02_oo`005Ooo00<00?ooool0fooo00<00?ooool0:_oo000Eool00`00
ooooo`3Lool00`00ooooo`0Yool001Goo`03003ooooo0=goo`03003ooooo02Soo`005Ooo0P00oooo
2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0>7oo`03003ooooo02Coo`005Ooo00<0
0?ooool0h_oo00<00?ooool08ooo000Eool00`00ooooo`3Sool00`00ooooo`0Rool001Goo`03003o
oooo0>Coo`03003ooooo027oo`005Ooo0P00i_oo00<00?ooool08?oo000Eool00`00ooooo`3oool9
ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo00000ooo000000020003ool0
0`00ooooo`03ool20005ool00`00ooooo`3Yool00`00ooooo`0Lool000?oo`03003ooooo00Ooo`04
003ooooo0004ool00`00ooooo`3Zool00`00ooooo`0Kool000?oo`03003ooooo00Ooo`04003ooooo
0004ool3003[ool00`00ooooo`0Jool000?oo`03003ooooo00Soo`8000Goo`03003ooooo0>coo`80
01[oo`000ooo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo0?ooo`Woo`0000Coo`000000
00Woo`04003ooooo0004ool00`00ooooo`3oool9ool000?oo`03003ooooo00Soo`8000Goo`03003o
oooo0?ooo`Woo`005Ooo00<00?ooool0lOoo00<00?ooool05?oo000Eool2003cool00`00ooooo`0C
ool001Goo`03003ooooo0??oo`03003ooooo01;oo`005Ooo00<00?ooool0m?oo00<00?ooool04Ooo
000Eool00`00ooooo`3eool00`00ooooo`0@ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?oo
ool0oooo2Ooo000Eool2003jool00`00ooooo`0<ool001Goo`03003ooooo0?Woo`03003ooooo00co
o`005Ooo00<00?ooool0n_oo00<00?ooool02ooo000Eool00`00ooooo`3kool2000;ool001Goo`03
003ooooo0?goo`03003ooooo00Soo`005Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool001Go
o`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Go
o`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161145, 0.538062, \
0.00752937, 0.0083122}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omtwo[IntegerPart[T*10^16]*10^\((\(-16\))\), 10^\((\(-5\))\), 
          10]}, {T, 0.001, 2, 1/20}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(25\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frmp\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.001`, 0.2766978558795718`}, {0.051000000000000004`, 
        0.6207050562440884`}, {0.101`, 
        0.629842436675907`}, {0.15100000000000002`, 
        0.6443995516926704`}, {0.201`, 0.6636689960837526`}, {0.251`, 
        0.6869010612977935`}, {0.301`, 0.7134102417565151`}, {0.351`, 
        0.7426105610774835`}, {0.40099999999999997`, 
        0.7740133521456564`}, {0.45099999999999996`, 
        0.7921646844892605`}, {0.501`, 0.8026222607723815`}, {0.551`, 
        0.8145610481049163`}, {0.6010000000000001`, 
        0.8281290771717661`}, {0.6510000000000001`, 
        0.8435109610139152`}, {0.7010000000000002`, 
        0.8609280152799419`}, {0.7510000000000002`, 
        0.8806507467300143`}, {0.8010000000000003`, 
        0.9029987445023746`}, {0.8510000000000003`, 
        0.9283368571351782`}, {0.9010000000000004`, 
        0.9570479604053281`}, {0.9510000000000004`, 
        0.9894726841809043`}, {1.0010000000000003`, 
        1.0257960556286083`}, {1.0510000000000004`, 
        1.0659153988031542`}, {1.1010000000000004`, 
        1.1093640082600102`}, {1.1510000000000005`, 
        1.1553909213203097`}, {1.2010000000000005`, 
        1.2031744371863011`}, {1.2510000000000006`, 
        1.2520238905820207`}, {1.3010000000000006`, 
        1.3014613584285821`}, {1.3510000000000006`, 
        1.3511997855283253`}, {1.4010000000000007`, 
        1.4010833609044184`}, {1.4510000000000007`, 
        1.451033561105223`}, {1.5010000000000008`, 
        1.5010130481753106`}, {1.5510000000000008`, 
        1.5510048977246345`}, {1.6010000000000009`, 
        1.601001798340138`}, {1.651000000000001`, 
        1.6510006237331216`}, {1.701000000000001`, 
        1.701000195410548`}, {1.751000000000001`, 
        1.7510000782242199`}, {1.801000000000001`, 
        1.8010000415538094`}, {1.851000000000001`, 
        1.851000051859109`}, {1.9010000000000011`, 
        1.9010000001000011`}, {1.9510000000000012`, 
        1.9510000001000012`}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(omtwo[k_, th_, a_] := 
      om /. FindRoot[
          testfunc[om, IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), th, 
              a] \[Equal] 0, {om, k + 8/10, k + 9/10}, 
          MaxIterations \[Rule] 25]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(omtwo[0.4, 10^\((\(-5\))\), 10]\)], "Input"],

Cell[BoxData[
    \(0.7829138917327014`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omtwo[IntegerPart[T*10^16]*10^\((\(-16\))\), 10^\((\(-5\))\), 
          10]}, {T, 0.001, 0.36, 1/20}]\)], "Input"],

Cell[BoxData[
    \({{0.001`, 0.7507332083329381`}, {0.051000000000000004`, 
        0.751234785897761`}, {0.101`, 
        0.7526983999163901`}, {0.15100000000000002`, 
        0.7551424823907917`}, {0.201`, 0.7585816171700839`}, {0.251`, 
        0.763051424667609`}, {0.301`, 0.7685846374439593`}, {0.351`, 
        0.7752374061515549`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.7507332083329381`/Sqrt[Sqrt[11]]*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(0.7139997885741284`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \({0.001`, 0.7507332083329381`}, {0.051000000000000004`, 
      0.751234785897761`}, {0.101`, 
      0.7526983999163901`}, {0.15100000000000002`, 
      0.7551424823907917`}, {0.201`, 0.7585816171700839`}, {0.251`, 
      0.763051424667609`}, {0.301`, 0.7685846374439593`}, {0.351`, 
      0.7752374061515549`}, {0.351`, 0.7752362962013227`}, {0.4, 
      0.7829138917327014`}, {0.45099999999999996`, 
      0.7921646844892605`}, \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.01`, 0.6176340793990494`}, {0.060000000000000005`, 
        0.6219234245061466`}, {0.11000000000000001`, 
        0.6320828923254037`}, {0.31`, 0.7184803646230286`}, {0.36`, 
        0.7481075164815932`}, {0.41`, 0.7798494540348498`}, {0.501`, 
        0.8026222607723815`}, {0.551`, 
        0.8145610481049163`}, {0.6010000000000001`, 
        0.8281290771717661`}, {0.6510000000000001`, 
        0.8435109610139152`}, {0.7010000000000002`, 
        0.8609280152799419`}, {0.7510000000000002`, 
        0.8806507467300143`}, {0.8010000000000003`, 
        0.9029987445023746`}, {0.8510000000000003`, 
        0.9283368571351782`}, {0.9010000000000004`, 
        0.9570479604053281`}, {0.9510000000000004`, 
        0.9894726841809043`}, {1.0010000000000003`, 
        1.0257960556286083`}, {1.0510000000000004`, 
        1.0659153988031542`}, {1.1010000000000004`, 
        1.1093640082600102`}, {1.1510000000000005`, 
        1.1553909213203097`}, {1.2010000000000005`, 
        1.2031744371863011`}, {1.2510000000000006`, 
        1.2520238905820207`}, {1.3010000000000006`, 
        1.3014613584285821`}, {1.3510000000000006`, 
        1.3511997855283253`}, {1.4010000000000007`, 
        1.4010833609044184`}, {1.4510000000000007`, 
        1.451033561105223`}, {1.5010000000000008`, 
        1.5010130481753106`}, {1.5510000000000008`, 
        1.5510048977246345`}, {1.6010000000000009`, 
        1.601001798340138`}, {1.651000000000001`, 
        1.6510006237331216`}, {1.701000000000001`, 
        1.701000195410548`}, {1.751000000000001`, 
        1.7510000782242199`}, {1.801000000000001`, 
        1.8010000415538094`}, {1.851000000000001`, 
        1.851000051859109`}, {1.9010000000000011`, 
        1.9010000001000011`}, {1.9510000000000012`, 
        1.9510000001000012`}}\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.6176340793990494`}, {0.060000000000000005`, 
        0.6219234245061466`}, {0.11000000000000001`, 
        0.6320828923254037`}, {0.31`, 0.7184803646230286`}, {0.36`, 
        0.7481075164815932`}, {0.41`, 0.7798494540348498`}, {0.501`, 
        0.8026222607723815`}, {0.551`, 
        0.8145610481049163`}, {0.6010000000000001`, 
        0.8281290771717661`}, {0.6510000000000001`, 
        0.8435109610139152`}, {0.7010000000000002`, 
        0.8609280152799419`}, {0.7510000000000002`, 
        0.8806507467300143`}, {0.8010000000000003`, 
        0.9029987445023746`}, {0.8510000000000003`, 
        0.9283368571351782`}, {0.9010000000000004`, 
        0.9570479604053281`}, {0.9510000000000004`, 
        0.9894726841809043`}, {1.0010000000000003`, 
        1.0257960556286083`}, {1.0510000000000004`, 
        1.0659153988031542`}, {1.1010000000000004`, 
        1.1093640082600102`}, {1.1510000000000005`, 
        1.1553909213203097`}, {1.2010000000000005`, 
        1.2031744371863011`}, {1.2510000000000006`, 
        1.2520238905820207`}, {1.3010000000000006`, 
        1.3014613584285821`}, {1.3510000000000006`, 
        1.3511997855283253`}, {1.4010000000000007`, 
        1.4010833609044184`}, {1.4510000000000007`, 
        1.451033561105223`}, {1.5010000000000008`, 
        1.5010130481753106`}, {1.5510000000000008`, 
        1.5510048977246345`}, {1.6010000000000009`, 
        1.601001798340138`}, {1.651000000000001`, 
        1.6510006237331216`}, {1.701000000000001`, 
        1.701000195410548`}, {1.751000000000001`, 
        1.7510000782242199`}, {1.801000000000001`, 
        1.8010000415538094`}, {1.851000000000001`, 
        1.851000051859109`}, {1.9010000000000011`, 
        1.9010000001000011`}, {1.9510000000000012`, 
        1.9510000001000012`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True, 
      PlotStyle \[Rule] {Dashing[{0.02, 0.02}]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.48815 -0.257935 0.441442 [
[.26788 .17101 -9 -9 ]
[.26788 .17101 9 0 ]
[.51196 .17101 -3 -9 ]
[.51196 .17101 3 0 ]
[.75603 .17101 -9 -9 ]
[.75603 .17101 9 0 ]
[.01131 .00693 -18 -4.5 ]
[.01131 .00693 0 4.5 ]
[.01131 .09522 -18 -4.5 ]
[.01131 .09522 0 4.5 ]
[.01131 .2718 -18 -4.5 ]
[.01131 .2718 0 4.5 ]
[.01131 .36008 -18 -4.5 ]
[.01131 .36008 0 4.5 ]
[.01131 .44837 -18 -4.5 ]
[.01131 .44837 0 4.5 ]
[.01131 .53666 -18 -4.5 ]
[.01131 .53666 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26788 .18351 m
.26788 .18976 L
s
[(0.5)] .26788 .17101 0 1 Mshowa
.51196 .18351 m
.51196 .18976 L
s
[(1)] .51196 .17101 0 1 Mshowa
.75603 .18351 m
.75603 .18976 L
s
[(1.5)] .75603 .17101 0 1 Mshowa
.125 Mabswid
.07262 .18351 m
.07262 .18726 L
s
.12144 .18351 m
.12144 .18726 L
s
.17025 .18351 m
.17025 .18726 L
s
.21907 .18351 m
.21907 .18726 L
s
.3167 .18351 m
.3167 .18726 L
s
.36551 .18351 m
.36551 .18726 L
s
.41433 .18351 m
.41433 .18726 L
s
.46314 .18351 m
.46314 .18726 L
s
.56077 .18351 m
.56077 .18726 L
s
.60959 .18351 m
.60959 .18726 L
s
.6584 .18351 m
.6584 .18726 L
s
.70722 .18351 m
.70722 .18726 L
s
.80485 .18351 m
.80485 .18726 L
s
.85366 .18351 m
.85366 .18726 L
s
.90248 .18351 m
.90248 .18726 L
s
.95129 .18351 m
.95129 .18726 L
s
.25 Mabswid
0 .18351 m
1 .18351 L
s
.02381 .00693 m
.03006 .00693 L
s
[(0.6)] .01131 .00693 1 0 Mshowa
.02381 .09522 m
.03006 .09522 L
s
[(0.8)] .01131 .09522 1 0 Mshowa
.02381 .2718 m
.03006 .2718 L
s
[(1.2)] .01131 .2718 1 0 Mshowa
.02381 .36008 m
.03006 .36008 L
s
[(1.4)] .01131 .36008 1 0 Mshowa
.02381 .44837 m
.03006 .44837 L
s
[(1.6)] .01131 .44837 1 0 Mshowa
.02381 .53666 m
.03006 .53666 L
s
[(1.8)] .01131 .53666 1 0 Mshowa
.125 Mabswid
.02381 .029 m
.02756 .029 L
s
.02381 .05107 m
.02756 .05107 L
s
.02381 .07315 m
.02756 .07315 L
s
.02381 .11729 m
.02756 .11729 L
s
.02381 .13936 m
.02756 .13936 L
s
.02381 .16144 m
.02756 .16144 L
s
.02381 .20558 m
.02756 .20558 L
s
.02381 .22765 m
.02756 .22765 L
s
.02381 .24972 m
.02756 .24972 L
s
.02381 .29387 m
.02756 .29387 L
s
.02381 .31594 m
.02756 .31594 L
s
.02381 .33801 m
.02756 .33801 L
s
.02381 .38216 m
.02756 .38216 L
s
.02381 .40423 m
.02756 .40423 L
s
.02381 .4263 m
.02756 .4263 L
s
.02381 .47044 m
.02756 .47044 L
s
.02381 .49252 m
.02756 .49252 L
s
.02381 .51459 m
.02756 .51459 L
s
.02381 .55873 m
.02756 .55873 L
s
.02381 .58081 m
.02756 .58081 L
s
.02381 .60288 m
.02756 .60288 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .02 .02 ] 0 setdash
.02869 .01472 m
.0531 .01661 L
.07751 .02109 L
.17514 .05923 L
.19954 .07231 L
.22395 .08632 L
.26837 .09638 L
.29278 .10165 L
.31719 .10764 L
.3416 .11443 L
.366 .12212 L
.39041 .13082 L
.41482 .14069 L
.43923 .15187 L
.46363 .16455 L
.48804 .17886 L
.51245 .19489 L
.53686 .21261 L
.56126 .23179 L
.58567 .2521 L
.61008 .2732 L
.63449 .29476 L
.65889 .31659 L
.6833 .33854 L
.70771 .36056 L
.73212 .38261 L
.75652 .40468 L
.78093 .42674 L
.80534 .44881 L
.82975 .47089 L
.85415 .49296 L
.87856 .51503 L
.90297 .5371 L
.92738 .55917 L
.95178 .58125 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003o
oooo0?ooo`Woo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3oool9ool00005
ool00?ooool00008ool01000ooooo`001?oo0`00oooo2Ooo00001Ooo003ooooo00002?oo0`001Ooo
00<00?ooool0oooo2Ooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo1000oooo2?oo00001Ooo
003ooooo00002?oo00<00?ooool01Ooo00@00?ooooooo`@00?ooo`Coo`000_oo0P002_oo0`001?oo
00<00?ooool02Ooo1000nooo000Eool00`00ooooo`0=ool2003iool001Goo`800?ooo`[oo`005Ooo
00<00?ooool04ooo0P00looo000Eool00`00ooooo`0Eool2003aool001Goo`03003ooooo01Ooo`80
0>ooo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`0Mool2003Yool001Goo`80023oo`80
0>Ooo`005Ooo00<00?ooool08Ooo0P00iOoo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
02Ooo`03003ooooo0=koo`005Ooo00<00?ooool0:?oo0P00g_oo000Eool00`00ooooo`0Zool2003L
ool001Goo`8002goo`03003ooooo0=Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`0a
ool00`00ooooo`3Dool001Goo`03003ooooo03;oo`03003ooooo0=?oo`000_oo0P001?oo00<00?oo
ool00ooo0P001Ooo00<00?ooool0<ooo1000dOoo00001Ooo003ooooo00002?oo00@00?ooool000Co
o`03003ooooo0?ooo`Woo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3000kool50038ool0
0005ool00?ooool00009ool20005ool00`00ooooo`10ool20036ool00005ool00?ooool00008ool0
1000ooooo`001?oo00<00?ooool0A_oo0`00_ooo00001Ooo003ooooo00002?oo00@00?ooool000Co
o`03003ooooo04Woo`<00;coo`000_oo0P002_oo0P001Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`1@ool5002cool001Goo`8005Koo`800;7oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`1Kool2002[ool001Goo`03003ooooo05goo`800:Woo`005Ooo00<00?ooool0Gooo0P00Yooo
000Eool00`00ooooo`3oool9ool001Goo`8006Koo`03003ooooo0:3oo`005Ooo00<00?ooool0I_oo
0`00Wooo000Eool00`00ooooo`0hool20004ool00`00ooooo`02ool3000Sool2000Eool4000hool4
0003ool00`00ooooo`02ool3000mool001Goo`03003ooooo03Ooo`04003ooooo000;ool00`00oooo
o`0iool00`00ooooo`0iool00`00ooooo`0:ool00`00ooooo`0jool001Goo`03003ooooo03Ooo`04
003ooooo000;ool00`00ooooo`0Vool00`00ooooo`0@ool00`00ooooo`0iool00`00ooooo`0:ool0
0`00ooooo`0jool001Goo`03003ooooo03Ooo`04003ooooo0008ool3000Zool2000@ool00`00oooo
o`0iool00`00ooooo`07ool3000mool001Goo`8003Soo`04003ooooo0008ool00`00ooooo`0/ool2
000>ool00`00ooooo`0iool00`00ooooo`07ool00`00ooooo`0mool001Goo`03003ooooo03Ooo`04
003ooooo0008ool00`00ooooo`0^ool00`00ooooo`09ool3000iool30009ool00`00ooooo`0mool0
01Goo`03003ooooo03Soo`8000Woo`@003_oo`03003ooooo03Woo`03003ooooo00Ooo`@003coo`00
5Ooo00<00?ooool0NOoo00<00?ooool0S?oo000Eool00`00ooooo`1jool00`00ooooo`2;ool001Go
o`03003ooooo07_oo`03003ooooo08[oo`003_ooo`004`00000Eool00`00ooooo`0:ool00`00oooo
o`0:ool00`00ooooo`0;ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0;ool00`00oooo
o`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0;ool00`00ooooo`0:ool00`00oooo
o`0:ool00`00ooooo`0;ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00oooo
o`0;ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0<ool001Goo`03003ooooo03ooo`03
003ooooo043oo`03003ooooo03ooo`03003ooooo047oo`005Ooo00<00?ooool0P_oo00<00?ooool0
Pooo000Eool00`00ooooo`23ool00`00ooooo`22ool001Goo`03003ooooo08Coo`8008;oo`005Ooo
0P00Qooo00<00?ooool0Oooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`2:ool00`00ooooo`1kool001Goo`03003ooooo
08_oo`8007_oo`005Ooo0P00S_oo0P00NOoo000Eool00`00ooooo`2?ool00`00ooooo`1fool001Go
o`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Go
o`03003ooooo09?oo`03003ooooo07;oo`005Ooo0P00UOoo0P00L_oo000Eool00`00ooooo`2Fool0
0`00ooooo`1_ool001Goo`03003ooooo09Ooo`03003ooooo06koo`005Ooo00<00?ooool0oooo2Ooo
00000ooo000000020003ool00`00ooooo`02ool40004ool00`00ooooo`3oool9ool000?oo`03003o
oooo00Ooo`03003ooooo00Goo`03003ooooo0?ooo`Woo`000ooo00<00?ooool02?oo00<00?ooool0
1?oo0`00Vooo00<00?ooool0J_oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2L
ool00`00ooooo`1Yool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo09goo`8006Wo
o`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`2Oool00`00ooooo`1Vool000?oo`03
003ooooo00Soo`8000Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2003o
ool:ool001Goo`03003ooooo0:?oo`03003ooooo06;oo`005Ooo00<00?ooool0Y?oo00<00?ooool0
HOoo000Eool00`00ooooo`2Uool2001Qool001Goo`03003ooooo0:Ooo`03003ooooo05koo`005Ooo
00<00?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?oo
ool0Zooo00<00?ooool0F_oo000Eool00`00ooooo`2/ool00`00ooooo`1Iool001Goo`03003ooooo
0:goo`8005Woo`005Ooo00<00?ooool0[ooo00<00?ooool0E_oo000Eool2002aool00`00ooooo`1E
ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`2dool0
0`00ooooo`1Aool00003ool00000008000?oo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo
0;Goo`03003ooooo053oo`000ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0]_oo00<0
0?ooool0Cooo0003ool00`00ooooo`06ool50004ool3002gool00`00ooooo`1>ool000?oo`03003o
oooo00Koo`04003ooooo0005ool00`00ooooo`2hool00`00ooooo`1=ool000?oo`03003ooooo00Oo
o`03003oo`0000Goo`03003ooooo0?ooo`Woo`0000Coo`00000000[oo`8000Goo`03003ooooo0?oo
o`Woo`000ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0oooo2Ooo000Eool00`00oooo
o`2lool00`00ooooo`19ool001Goo`800;koo`8004Woo`005Ooo00<00?ooool0_ooo00<00?ooool0
A_oo000Eool00`00ooooo`30ool00`00ooooo`15ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800<Goo`03003ooooo047oo`005Ooo
00<00?ooool0aOoo00<00?ooool0@?oo000Eool00`00ooooo`36ool00`00ooooo`0oool001Goo`03
003ooooo0<Ooo`03003ooooo03koo`005Ooo00<00?ooool0b?oo00<00?ooool0?Ooo000Eool00`00
ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0c?oo00<00?ooool0>Ooo000Eool0
0`00ooooo`3=ool00`00ooooo`0hool001Goo`03003ooooo0<koo`03003ooooo03Ooo`0000?oo`00
00000P000ooo00<00?ooool00ooo0P001Ooo00<00?ooool0cooo00<00?ooool0=_oo0003ool00`00
ooooo`07ool01000ooooo`001?oo00<00?ooool0d?oo00<00?ooool0=Ooo0003ool00`00ooooo`07
ool01000ooooo`001?oo0`00oooo2Ooo0003ool00`00ooooo`07ool30005ool00`00ooooo`3oool9
ool000?oo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0?ooo`Woo`0000Coo`00000000Wo
o`03003ooooo00Goo`03003ooooo0=Coo`03003ooooo037oo`000ooo00<00?ooool02?oo0`001?oo
00<00?ooool0eOoo00<00?ooool0<?oo000Eool00`00ooooo`3Fool2000`ool001Goo`800=Woo`03
003ooooo02goo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003o
oooo0?ooo`Woo`005Ooo00<00?ooool0g?oo00<00?ooool0:Ooo000Eool00`00ooooo`3Mool2000Y
ool001Goo`800>3oo`03003ooooo02Koo`005Ooo00<00?ooool0h?oo00<00?ooool09Ooo000Eool0
0`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool0
0`00ooooo`3Tool00`00ooooo`0Qool001Goo`800>Koo`03003ooooo023oo`005Ooo00<00?ooool0
i_oo00<00?ooool07ooo000Eool00`00ooooo`3Wool00`00ooooo`0Nool001Goo`03003ooooo0>So
o`03003ooooo01goo`0000?oo`0000000P000ooo00<00?ooool00ooo0P001Ooo00<00?ooool0oooo
2Ooo0003ool00`00ooooo`07ool01000ooooo`001?oo00<00?ooool0oooo2Ooo0003ool00`00oooo
o`07ool01000ooooo`001?oo0`00k?oo00<00?ooool06Ooo0003ool00`00ooooo`08ool20005ool0
0`00ooooo`3]ool00`00ooooo`0Hool000?oo`03003ooooo00Ooo`04003ooooo0004ool00`00oooo
o`3^ool00`00ooooo`0Gool00004ool000000009ool01000ooooo`001?oo00<00?ooool0kooo00<0
0?ooool05_oo0003ool00`00ooooo`08ool20005ool00`00ooooo`3`ool00`00ooooo`0Eool001Go
o`03003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03003o
oooo0?Coo`03003ooooo017oo`005Ooo00<00?ooool0mOoo00<00?ooool04?oo000Eool00`00oooo
o`3fool2000@ool001Goo`03003ooooo0?Soo`03003ooooo00goo`005Ooo0P00oooo2_oo000Eool0
0`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0o?oo00<00?ooool0
2Ooo000Eool00`00ooooo`3mool00`00ooooo`08ool001Goo`03003ooooo0?koo`8000Soo`005Ooo
0P00oooo0_oo00<00?ooool01Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Wo
o`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool00?ooob7oo`00oooo8Ooo003o
oolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161145, 0.536835, \
0.00752937, 0.00832604}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omtwo[IntegerPart[T*10^16]*10^\((\(-16\))\), 
          Pi/2 \((1 - 10^\((\(-5\))\))\), 10]}, {T, 0.001, 2, 
        1/20}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(25\) iterations."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(25\) iterations."\)], "Message"],

Cell[BoxData[
    \({{0.001`, 0.16416133638101826`}, {0.051000000000000004`, 
        0.6189070569258497`}, {0.101`, 
        0.6233931056001559`}, {0.15100000000000002`, 
        0.63062212127032`}, {0.201`, 0.6406451376155898`}, {0.251`, 
        0.6533956482723087`}, {0.301`, 0.6687852923360084`}, {0.351`, 
        0.6867158792179943`}, {0.40099999999999997`, 
        0.7070759544838557`}, {0.45099999999999996`, 
        0.7297465083843034`}, {0.501`, 0.7546012553254112`}, {0.551`, 
        0.781512161140522`}, {0.6010000000000001`, 
        0.810347881316272`}, {0.6510000000000001`, 
        0.8409787585899607`}, {0.7010000000000002`, 
        0.8732780302513268`}, {0.7510000000000002`, 
        0.9071216068482567`}, {0.8010000000000003`, 
        0.9423905948367407`}, {0.8510000000000003`, 
        0.9789727650025765`}, {0.9010000000000004`, 
        1.0167604525362153`}, {0.9510000000000004`, 
        1.0556534262091088`}, {1.0010000000000003`, 
        1.0955577499107925`}, {1.0510000000000004`, 
        1.1363868691173122`}, {1.1010000000000004`, 
        1.1780596623844153`}, {1.1510000000000005`, 
        1.2205018830936312`}, {1.2010000000000005`, 
        1.2636452093064579`}, {1.2510000000000006`, 
        1.3074268777536522`}, {1.3010000000000006`, 
        1.3517898136088196`}, {1.3510000000000006`, 
        1.3966814076959388`}, {1.4010000000000007`, 
        1.4420539098536462`}, {1.4510000000000007`, 
        1.4878637804378032`}, {1.5010000000000008`, 
        1.534071231116833`}, {1.5510000000000008`, 
        1.580640524733995`}, {1.6010000000000009`, 
        1.6275385408431617`}, {1.651000000000001`, 
        1.67473544337243`}, {1.701000000000001`, 
        1.7222040690295164`}, {1.751000000000001`, 
        1.7699196957099494`}, {1.801000000000001`, 
        1.8178598220078805`}, {1.851000000000001`, 
        1.8660039084575548`}, {1.9010000000000011`, 
        1.914333440610409`}, {1.9510000000000012`, 
        1.9628313121646601`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.051000000000000004`, 0.6189070569258497`}, {0.101`, 
        0.6233931056001559`}, {0.15100000000000002`, 
        0.63062212127032`}, {0.201`, 0.6406451376155898`}, {0.251`, 
        0.6533956482723087`}, {0.301`, 0.6687852923360084`}, {0.351`, 
        0.6867158792179943`}, {0.40099999999999997`, 
        0.7070759544838557`}, {0.45099999999999996`, 
        0.7297465083843034`}, {0.501`, 0.7546012553254112`}, {0.551`, 
        0.781512161140522`}, {0.6010000000000001`, 
        0.810347881316272`}, {0.6510000000000001`, 
        0.8409787585899607`}, {0.7010000000000002`, 
        0.8732780302513268`}, {0.7510000000000002`, 
        0.9071216068482567`}, {0.8010000000000003`, 
        0.9423905948367407`}, {0.8510000000000003`, 
        0.9789727650025765`}, {0.9010000000000004`, 
        1.0167604525362153`}, {0.9510000000000004`, 
        1.0556534262091088`}, {1.0010000000000003`, 
        1.0955577499107925`}, {1.0510000000000004`, 
        1.1363868691173122`}, {1.1010000000000004`, 
        1.1780596623844153`}, {1.1510000000000005`, 
        1.2205018830936312`}, {1.2010000000000005`, 
        1.2636452093064579`}, {1.2510000000000006`, 
        1.3074268777536522`}, {1.3010000000000006`, 
        1.3517898136088196`}, {1.3510000000000006`, 
        1.3966814076959388`}, {1.4010000000000007`, 
        1.4420539098536462`}, {1.4510000000000007`, 
        1.4878637804378032`}, {1.5010000000000008`, 
        1.534071231116833`}, {1.5510000000000008`, 
        1.580640524733995`}, {1.6010000000000009`, 
        1.6275385408431617`}, {1.651000000000001`, 
        1.67473544337243`}, {1.701000000000001`, 
        1.7222040690295164`}, {1.751000000000001`, 
        1.7699196957099494`}, {1.801000000000001`, 
        1.8178598220078805`}, {1.851000000000001`, 
        1.8660039084575548`}, {1.9010000000000011`, 
        1.914333440610409`}, {1.9510000000000012`, 
        1.9628313121646601`}}\)], "Input"],

Cell[BoxData[
    \({{0.051000000000000004`, 0.6189070569258497`}, {0.101`, 
        0.6233931056001559`}, {0.15100000000000002`, 
        0.63062212127032`}, {0.201`, 0.6406451376155898`}, {0.251`, 
        0.6533956482723087`}, {0.301`, 0.6687852923360084`}, {0.351`, 
        0.6867158792179943`}, {0.40099999999999997`, 
        0.7070759544838557`}, {0.45099999999999996`, 
        0.7297465083843034`}, {0.501`, 0.7546012553254112`}, {0.551`, 
        0.781512161140522`}, {0.6010000000000001`, 
        0.810347881316272`}, {0.6510000000000001`, 
        0.8409787585899607`}, {0.7010000000000002`, 
        0.8732780302513268`}, {0.7510000000000002`, 
        0.9071216068482567`}, {0.8010000000000003`, 
        0.9423905948367407`}, {0.8510000000000003`, 
        0.9789727650025765`}, {0.9010000000000004`, 
        1.0167604525362153`}, {0.9510000000000004`, 
        1.0556534262091088`}, {1.0010000000000003`, 
        1.0955577499107925`}, {1.0510000000000004`, 
        1.1363868691173122`}, {1.1010000000000004`, 
        1.1780596623844153`}, {1.1510000000000005`, 
        1.2205018830936312`}, {1.2010000000000005`, 
        1.2636452093064579`}, {1.2510000000000006`, 
        1.3074268777536522`}, {1.3010000000000006`, 
        1.3517898136088196`}, {1.3510000000000006`, 
        1.3966814076959388`}, {1.4010000000000007`, 
        1.4420539098536462`}, {1.4510000000000007`, 
        1.4878637804378032`}, {1.5010000000000008`, 
        1.534071231116833`}, {1.5510000000000008`, 
        1.580640524733995`}, {1.6010000000000009`, 
        1.6275385408431617`}, {1.651000000000001`, 
        1.67473544337243`}, {1.701000000000001`, 
        1.7222040690295164`}, {1.751000000000001`, 
        1.7699196957099494`}, {1.801000000000001`, 
        1.8178598220078805`}, {1.851000000000001`, 
        1.8660039084575548`}, {1.9010000000000011`, 
        1.914333440610409`}, {1.9510000000000012`, 
        1.9628313121646601`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.6189070569258497/Sqrt[Sqrt[11]]*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(0.5886238984597008`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True, 
      PlotStyle \[Rule] {Dashing[{0.02, 0.02}]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.48815 -0.25635 0.437974 [
[.26788 .16912 -9 -9 ]
[.26788 .16912 9 0 ]
[.51196 .16912 -3 -9 ]
[.51196 .16912 3 0 ]
[.75603 .16912 -9 -9 ]
[.75603 .16912 9 0 ]
[.01131 .00643 -18 -4.5 ]
[.01131 .00643 0 4.5 ]
[.01131 .09403 -18 -4.5 ]
[.01131 .09403 0 4.5 ]
[.01131 .26922 -18 -4.5 ]
[.01131 .26922 0 4.5 ]
[.01131 .35681 -18 -4.5 ]
[.01131 .35681 0 4.5 ]
[.01131 .44441 -18 -4.5 ]
[.01131 .44441 0 4.5 ]
[.01131 .532 -18 -4.5 ]
[.01131 .532 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26788 .18162 m
.26788 .18787 L
s
[(0.5)] .26788 .16912 0 1 Mshowa
.51196 .18162 m
.51196 .18787 L
s
[(1)] .51196 .16912 0 1 Mshowa
.75603 .18162 m
.75603 .18787 L
s
[(1.5)] .75603 .16912 0 1 Mshowa
.125 Mabswid
.07262 .18162 m
.07262 .18537 L
s
.12144 .18162 m
.12144 .18537 L
s
.17025 .18162 m
.17025 .18537 L
s
.21907 .18162 m
.21907 .18537 L
s
.3167 .18162 m
.3167 .18537 L
s
.36551 .18162 m
.36551 .18537 L
s
.41433 .18162 m
.41433 .18537 L
s
.46314 .18162 m
.46314 .18537 L
s
.56077 .18162 m
.56077 .18537 L
s
.60959 .18162 m
.60959 .18537 L
s
.6584 .18162 m
.6584 .18537 L
s
.70722 .18162 m
.70722 .18537 L
s
.80485 .18162 m
.80485 .18537 L
s
.85366 .18162 m
.85366 .18537 L
s
.90248 .18162 m
.90248 .18537 L
s
.95129 .18162 m
.95129 .18537 L
s
.25 Mabswid
0 .18162 m
1 .18162 L
s
.02381 .00643 m
.03006 .00643 L
s
[(0.6)] .01131 .00643 1 0 Mshowa
.02381 .09403 m
.03006 .09403 L
s
[(0.8)] .01131 .09403 1 0 Mshowa
.02381 .26922 m
.03006 .26922 L
s
[(1.2)] .01131 .26922 1 0 Mshowa
.02381 .35681 m
.03006 .35681 L
s
[(1.4)] .01131 .35681 1 0 Mshowa
.02381 .44441 m
.03006 .44441 L
s
[(1.6)] .01131 .44441 1 0 Mshowa
.02381 .532 m
.03006 .532 L
s
[(1.8)] .01131 .532 1 0 Mshowa
.125 Mabswid
.02381 .02833 m
.02756 .02833 L
s
.02381 .05023 m
.02756 .05023 L
s
.02381 .07213 m
.02756 .07213 L
s
.02381 .11593 m
.02756 .11593 L
s
.02381 .13783 m
.02756 .13783 L
s
.02381 .15973 m
.02756 .15973 L
s
.02381 .20352 m
.02756 .20352 L
s
.02381 .22542 m
.02756 .22542 L
s
.02381 .24732 m
.02756 .24732 L
s
.02381 .29112 m
.02756 .29112 L
s
.02381 .31302 m
.02756 .31302 L
s
.02381 .33491 m
.02756 .33491 L
s
.02381 .37871 m
.02756 .37871 L
s
.02381 .40061 m
.02756 .40061 L
s
.02381 .42251 m
.02756 .42251 L
s
.02381 .46631 m
.02756 .46631 L
s
.02381 .48821 m
.02756 .48821 L
s
.02381 .5101 m
.02756 .5101 L
s
.02381 .5539 m
.02756 .5539 L
s
.02381 .5758 m
.02756 .5758 L
s
.02381 .5977 m
.02756 .5977 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .02 .02 ] 0 setdash
.04871 .01472 m
.07311 .01668 L
.09752 .01985 L
.12193 .02424 L
.14634 .02982 L
.17074 .03656 L
.19515 .04441 L
.21956 .05333 L
.24397 .06326 L
.26837 .07415 L
.29278 .08593 L
.31719 .09856 L
.3416 .11198 L
.366 .12612 L
.39041 .14095 L
.41482 .15639 L
.43923 .17241 L
.46363 .18896 L
.48804 .206 L
.51245 .22348 L
.53686 .24136 L
.56126 .25961 L
.58567 .2782 L
.61008 .29709 L
.63449 .31627 L
.65889 .3357 L
.6833 .35536 L
.70771 .37523 L
.73212 .3953 L
.75652 .41553 L
.78093 .43593 L
.80534 .45647 L
.82975 .47714 L
.85415 .49793 L
.87856 .51883 L
.90297 .53983 L
.92738 .56091 L
.95178 .58208 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003o
oooo0?ooo`Woo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3oool9ool00005
ool00?ooool00008ool01000ooooo`001?oo0`00oooo2Ooo00001Ooo003ooooo00002?oo0`001Ooo
00<00?ooool0oooo2Ooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool01?oo0`00
oooo0_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool01ooo0`001?oo0P00n?oo
0002ool2000:ool30004ool00`00ooooo`0@ool5003cool001Goo`03003ooooo01Woo`<00>coo`00
5Ooo0P007Ooo0`00jOoo000Eool00`00ooooo`0Nool00`00ooooo`3Wool001Goo`03003ooooo02Co
o`800>;oo`005Ooo00<00?ooool09_oo0P00h?oo000Eool00`00ooooo`0Xool2003Nool001Goo`03
003ooooo02koo`03003ooooo0=Ooo`005Ooo0P00<?oo0P00eooo000Eool00`00ooooo`0aool3003D
ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0>?oo00<00?ooool0cOoo000Eool00`00
ooooo`0iool00`00ooooo`3<ool001Goo`03003ooooo03[oo`800<coo`005Ooo0P00?Ooo0P00b_oo
000Eool00`00ooooo`3oool9ool001Goo`03003ooooo04;oo`03003ooooo0<?oo`005Ooo00<00?oo
ool0@ooo0P00`ooo0002ool20004ool00`00ooooo`03ool20005ool00`00ooooo`15ool30030ool0
0005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0oooo2Ooo00001Ooo003ooooo0000
2?oo00@00?ooool000Coo`<00?ooo`Woo`0000Goo`00ooooo`0000Woo`8000Goo`03003ooooo04co
o`800;[oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1>ool2002hool00005
ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0D?oo0P00]_oo0002ool2000:ool20005
ool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo0P00E_oo00<00?ooool0/?oo
000Eool00`00ooooo`1Fool2002`ool001Goo`03003ooooo05Soo`800:koo`005Ooo00<00?ooool0
F_oo00<00?ooool0Zooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo
0P00H?oo00<00?ooool0Y_oo000Eool00`00ooooo`0hool20004ool00`00ooooo`02ool3000Jool2
000Nool4000hool40003ool00`00ooooo`02ool3000mool001Goo`03003ooooo03Ooo`04003ooooo
000;ool00`00ooooo`0Iool2000Nool00`00ooooo`0iool00`00ooooo`0:ool00`00ooooo`0jool0
01Goo`03003ooooo03Ooo`04003ooooo000;ool00`00ooooo`0iool00`00ooooo`0iool00`00oooo
o`0:ool00`00ooooo`0jool001Goo`03003ooooo03Ooo`04003ooooo0008ool3000lool00`00oooo
o`0iool00`00ooooo`07ool3000mool001Goo`03003ooooo03Ooo`04003ooooo0008ool00`00oooo
o`0Rool00`00ooooo`0Gool00`00ooooo`0iool00`00ooooo`07ool00`00ooooo`0mool001Goo`80
03Soo`04003ooooo0008ool00`00ooooo`0Sool00`00ooooo`0Dool3000iool30009ool00`00oooo
o`0mool001Goo`03003ooooo03Soo`8000Woo`@002?oo`8001Koo`03003ooooo03Woo`03003ooooo
00Ooo`@003coo`005Ooo00<00?ooool0K?oo00<00?ooool0VOoo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0LOoo00<00?ooool0U?oo000>oooo000C0000
01Goo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo
00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`8000goo`03
003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00[oo`03
003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00coo`00
5Ooo00<00?ooool0MOoo00<00?ooool0T?oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool2001kool2002<ool001Goo`03003ooooo07co
o`03003ooooo08Woo`005Ooo00<00?ooool0OOoo00<00?ooool0R?oo000Eool00`00ooooo`1nool0
0`00ooooo`27ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool20024
ool00`00ooooo`22ool001Goo`03003ooooo08Coo`03003ooooo087oo`005Ooo00<00?ooool0QOoo
0P00POoo000Eool00`00ooooo`27ool00`00ooooo`1nool001Goo`03003ooooo0?ooo`Woo`005Ooo
00<00?ooool0oooo2Ooo000Eool2002=ool00`00ooooo`1iool001Goo`03003ooooo08goo`03003o
oooo07Soo`005Ooo00<00?ooool0S_oo00<00?ooool0Mooo000Eool00`00ooooo`2?ool00`00oooo
o`1fool00003ool00000008000?oo`03003ooooo00;oo`@000Coo`03003ooooo093oo`03003ooooo
07Goo`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0oooo2Ooo0003ool00`00oooo
o`08ool00`00ooooo`04ool3003oool9ool000?oo`03003ooooo00Woo`03003ooooo00?oo`03003o
oooo09Coo`03003ooooo077oo`000ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0UOoo
0P00LOoo00001?oo000000002Ooo00@00?ooool000Coo`03003ooooo09Ooo`8006ooo`000ooo00<0
0?ooool02?oo0P001Ooo00<00?ooool0VOoo00<00?ooool0K?oo000Eool00`00ooooo`3oool9ool0
01Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`2Mool00`00ooooo`1X
ool001Goo`03003ooooo09koo`03003ooooo06Ooo`005Ooo00<00?ooool0Wooo0P00Iooo000Eool2
002Rool00`00ooooo`1Tool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000E
ool00`00ooooo`2Uool00`00ooooo`1Pool001Goo`03003ooooo0:Koo`03003ooooo05ooo`005Ooo
00<00?ooool0Yooo0P00Gooo000Eool2002Zool00`00ooooo`1Lool001Goo`03003ooooo0:[oo`03
003ooooo05_oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool00003ool00000
008000?oo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo0:koo`03003ooooo05Ooo`000ooo
00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0[ooo00<00?ooool0E_oo0003ool00`00oooo
o`06ool50004ool3002`ool00`00ooooo`1Eool000?oo`03003ooooo00Koo`04003ooooo0005ool0
0`00ooooo`2aool00`00ooooo`1Dool000?oo`03003ooooo00Ooo`03003oo`0000Goo`03003ooooo
0;;oo`03003ooooo05?oo`0000Coo`00000000[oo`8000Goo`03003ooooo0?ooo`Woo`000ooo00<0
0?ooool02Ooo00<00?ooool00ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`2fool2001@ool0
01Goo`800;Woo`03003ooooo04goo`005Ooo00<00?ooool0^Ooo0P00COoo000Eool00`00ooooo`2k
ool00`00ooooo`1:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool0
0`00ooooo`3oool9ool001Goo`800<3oo`03003ooooo04Koo`005Ooo00<00?ooool0`?oo00<00?oo
ool0AOoo000Eool00`00ooooo`31ool20015ool001Goo`03003ooooo0<?oo`03003ooooo04;oo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo00<0
0?ooool0aooo00<00?ooool0?_oo000Eool00`00ooooo`38ool00`00ooooo`0mool001Goo`03003o
oooo0<Woo`8003goo`0000?oo`0000000P000ooo00<00?ooool00ooo0P001Ooo00<00?ooool0booo
00<00?ooool0>_oo0003ool00`00ooooo`07ool01000ooooo`001?oo00<00?ooool0oooo2Ooo0003
ool00`00ooooo`07ool01000ooooo`001?oo0`00oooo2Ooo0003ool00`00ooooo`07ool30005ool0
0`00ooooo`3oool9ool000?oo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0<ooo`03003o
oooo03Koo`0000Coo`00000000Woo`03003ooooo00Goo`03003ooooo0=3oo`8003Koo`000ooo00<0
0?ooool02?oo0`001?oo00<00?ooool0d_oo00<00?ooool0<ooo000Eool00`00ooooo`3Cool2000c
ool001Goo`800?ooo`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Go
o`03003ooooo0=Soo`03003ooooo02goo`005Ooo00<00?ooool0fOoo00<00?ooool0;?oo000Eool0
0`00ooooo`3Jool2000/ool001Goo`800=goo`03003ooooo02Woo`005Ooo00<00?ooool0oooo2Ooo
000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0h?oo00<0
0?ooool09Ooo000Eool00`00ooooo`3Qool00`00ooooo`0Tool001Goo`800>?oo`8002Coo`005Ooo
00<00?ooool0i?oo00<00?ooool08Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?oo
o`Woo`0000?oo`0000000P000ooo00<00?ooool00ooo0P001Ooo00<00?ooool0oooo2Ooo0003ool0
0`00ooooo`07ool01000ooooo`001?oo00<00?ooool0j?oo00<00?ooool07Ooo0003ool00`00oooo
o`07ool01000ooooo`001?oo0`00jOoo00<00?ooool07?oo0003ool00`00ooooo`08ool20005ool0
0`00ooooo`3Zool2000Lool000?oo`03003ooooo00Ooo`04003ooooo0004ool00`00ooooo`3/ool0
0`00ooooo`0Iool00004ool000000009ool01000ooooo`001?oo00<00?ooool0k?oo00<00?ooool0
6Ooo0003ool00`00ooooo`08ool20005ool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Wo
o`005Ooo0P00l_oo00<00?ooool05?oo000Eool00`00ooooo`3bool00`00ooooo`0Cool001Goo`03
003ooooo0??oo`03003ooooo01;oo`005Ooo00<00?ooool0m?oo00<00?ooool04Ooo000Eool00`00
ooooo`3eool00`00ooooo`0@ool001Goo`03003ooooo0?ooo`Woo`005Ooo0P00oooo2_oo000Eool0
0`00ooooo`3iool00`00ooooo`0<ool001Goo`03003ooooo0?Woo`03003ooooo00coo`005Ooo00<0
0?ooool0n_oo00<00?ooool02ooo000Eool00`00ooooo`3kool2000;ool001Goo`03003ooooo0?go
o`03003ooooo00Soo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161145, 0.536896, \
0.00752937, 0.00839197}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omthree[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, 10]}, {T, 0.01, 
        2, 1/20}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.61760123175664`}, {0.060000000000000005`, 
        0.6207566195804275`}, {0.11000000000000001`, 
        0.6282786072649869`}, {0.16000000000000003`, 
        0.7621346927435746`}, {0.21000000000000002`, 
        0.7705552084144212`}, {0.26`, 0.7814929072106338`}, {0.31`, 
        0.693340005010564`}, {0.36`, 0.7153500957276302`}, {0.41`, 
        0.7384040207975904`}, {0.45999999999999996`, 
        0.762040846867269`}, {0.51`, 0.8804932664149454`}, {0.56`, 
        0.9095730735009533`}, {0.6100000000000001`, 
        0.9412226038914596`}, {0.6600000000000001`, 
        0.9750357008226401`}, {0.7100000000000002`, 
        1.0106410159247183`}, {0.7600000000000002`, 
        1.0477274775311103`}, {0.8100000000000003`, 
        1.0860474881901534`}, {0.8600000000000003`, 
        1.1254051745533107`}, {0.9100000000000004`, 
        1.1656462503161418`}, {0.9600000000000004`, 
        1.2066480155517205`}, {1.0100000000000005`, 
        1.2483117281072993`}, {1.0600000000000005`, 
        1.2905568902801523`}, {1.1100000000000005`, 
        1.333317033739176`}, {1.1600000000000006`, 
        1.3765366120471425`}, {1.2100000000000006`, 
        1.4201686937337965`}, {1.2600000000000007`, 
        1.464173231097893`}, {1.3100000000000007`, 
        1.5085157448227655`}, {1.3600000000000008`, 
        1.5531663116396912`}, {1.4100000000000008`, 
        1.5980987753744877`}, {1.4600000000000009`, 
        1.643290124681043`}, {1.510000000000001`, 
        1.6887199966987236`}, {1.560000000000001`, 
        1.734370276988184`}, {1.610000000000001`, 
        1.7802247739279686`}, {1.660000000000001`, 
        1.8262689513248909`}, {1.710000000000001`, 
        1.8724897070004805`}, {1.7600000000000011`, 
        1.9188751880346069`}, {1.8100000000000012`, 
        1.9654146354962887`}, {1.8600000000000012`, 
        2.0120982530910445`}, {1.9100000000000013`, 
        2.0589170953568696`}, {1.9600000000000013`, 
        2.1058629719543744`}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(omthree[k_, th_, a_] := 
      om /. FindRoot[
          testfunc[om, IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), th, 
              a] \[Equal] 0, {om, k + 7/10, k + 8/10}, 
          MaxIterations \[Rule] 25]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(omthree[0.01, Pi/4, 10]\)], "Input"],

Cell[BoxData[
    \(0.7507769815966245`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(omthree[0.4, Pi/4, 10]\)], "Input"],

Cell[BoxData[
    \(0.8272073402727343`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.01`, 0.7507769815966245`}, {0.1, 
        0.7551522693859802`}, {0.16000000000000003`, 
        0.7621346927435746`}, {0.21000000000000002`, 
        0.7705552084144212`}, {0.26`, 0.7814929072106338`}, {0.4, 
        0.8272073402727343`}, {0.51`, 0.8804932664149454`}, {0.56`, 
        0.9095730735009533`}, {0.6100000000000001`, 
        0.9412226038914596`}, {0.6600000000000001`, 
        0.9750357008226401`}, {0.7100000000000002`, 
        1.0106410159247183`}, {0.7600000000000002`, 
        1.0477274775311103`}, {0.8100000000000003`, 
        1.0860474881901534`}, {0.8600000000000003`, 
        1.1254051745533107`}, {0.9100000000000004`, 
        1.1656462503161418`}, {0.9600000000000004`, 
        1.2066480155517205`}, {1.0100000000000005`, 
        1.2483117281072993`}, {1.0600000000000005`, 
        1.2905568902801523`}, {1.1100000000000005`, 
        1.333317033739176`}, {1.1600000000000006`, 
        1.3765366120471425`}, {1.2100000000000006`, 
        1.4201686937337965`}, {1.2600000000000007`, 
        1.464173231097893`}, {1.3100000000000007`, 
        1.5085157448227655`}, {1.3600000000000008`, 
        1.5531663116396912`}, {1.4100000000000008`, 
        1.5980987753744877`}, {1.4600000000000009`, 
        1.643290124681043`}, {1.510000000000001`, 
        1.6887199966987236`}, {1.560000000000001`, 
        1.734370276988184`}, {1.610000000000001`, 
        1.7802247739279686`}, {1.660000000000001`, 
        1.8262689513248909`}, {1.710000000000001`, 
        1.8724897070004805`}, {1.7600000000000011`, 
        1.9188751880346069`}, {1.8100000000000012`, 
        1.9654146354962887`}, {1.8600000000000012`, 
        2.0120982530910445`}, {1.9100000000000013`, 
        2.0589170953568696`}, {1.9600000000000013`, 
        2.1058629719543744`}}\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.7507769815966245`}, {0.1`, 
        0.7551522693859802`}, {0.16000000000000003`, 
        0.7621346927435746`}, {0.21000000000000002`, 
        0.7705552084144212`}, {0.26`, 0.7814929072106338`}, {0.4`, 
        0.8272073402727343`}, {0.51`, 0.8804932664149454`}, {0.56`, 
        0.9095730735009533`}, {0.6100000000000001`, 
        0.9412226038914596`}, {0.6600000000000001`, 
        0.9750357008226401`}, {0.7100000000000002`, 
        1.0106410159247183`}, {0.7600000000000002`, 
        1.0477274775311103`}, {0.8100000000000003`, 
        1.0860474881901534`}, {0.8600000000000003`, 
        1.1254051745533107`}, {0.9100000000000004`, 
        1.1656462503161418`}, {0.9600000000000004`, 
        1.2066480155517205`}, {1.0100000000000005`, 
        1.2483117281072993`}, {1.0600000000000005`, 
        1.2905568902801523`}, {1.1100000000000005`, 
        1.333317033739176`}, {1.1600000000000006`, 
        1.3765366120471425`}, {1.2100000000000006`, 
        1.4201686937337965`}, {1.2600000000000007`, 
        1.464173231097893`}, {1.3100000000000007`, 
        1.5085157448227655`}, {1.3600000000000008`, 
        1.5531663116396912`}, {1.4100000000000008`, 
        1.5980987753744877`}, {1.4600000000000009`, 
        1.643290124681043`}, {1.510000000000001`, 
        1.6887199966987236`}, {1.560000000000001`, 
        1.734370276988184`}, {1.610000000000001`, 
        1.7802247739279686`}, {1.660000000000001`, 
        1.8262689513248909`}, {1.710000000000001`, 
        1.8724897070004805`}, {1.7600000000000011`, 
        1.9188751880346069`}, {1.8100000000000012`, 
        1.9654146354962887`}, {1.8600000000000012`, 
        2.0120982530910445`}, {1.9100000000000013`, 
        2.0589170953568696`}, {1.9600000000000013`, 
        2.1058629719543744`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.485909 -0.311397 0.434366 [
[.26676 .11047 -9 -9 ]
[.26676 .11047 9 0 ]
[.50972 .11047 -3 -9 ]
[.50972 .11047 3 0 ]
[.75267 .11047 -9 -9 ]
[.75267 .11047 9 0 ]
[.99563 .11047 -3 -9 ]
[.99563 .11047 3 0 ]
[.01131 .0361 -18 -4.5 ]
[.01131 .0361 0 4.5 ]
[.01131 .20984 -18 -4.5 ]
[.01131 .20984 0 4.5 ]
[.01131 .29672 -18 -4.5 ]
[.01131 .29672 0 4.5 ]
[.01131 .38359 -18 -4.5 ]
[.01131 .38359 0 4.5 ]
[.01131 .47046 -18 -4.5 ]
[.01131 .47046 0 4.5 ]
[.01131 .55734 -6 -4.5 ]
[.01131 .55734 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26676 .12297 m
.26676 .12922 L
s
[(0.5)] .26676 .11047 0 1 Mshowa
.50972 .12297 m
.50972 .12922 L
s
[(1)] .50972 .11047 0 1 Mshowa
.75267 .12297 m
.75267 .12922 L
s
[(1.5)] .75267 .11047 0 1 Mshowa
.99563 .12297 m
.99563 .12922 L
s
[(2)] .99563 .11047 0 1 Mshowa
.125 Mabswid
.0724 .12297 m
.0724 .12672 L
s
.12099 .12297 m
.12099 .12672 L
s
.16958 .12297 m
.16958 .12672 L
s
.21817 .12297 m
.21817 .12672 L
s
.31535 .12297 m
.31535 .12672 L
s
.36395 .12297 m
.36395 .12672 L
s
.41254 .12297 m
.41254 .12672 L
s
.46113 .12297 m
.46113 .12672 L
s
.55831 .12297 m
.55831 .12672 L
s
.6069 .12297 m
.6069 .12672 L
s
.65549 .12297 m
.65549 .12672 L
s
.70408 .12297 m
.70408 .12672 L
s
.80126 .12297 m
.80126 .12672 L
s
.84985 .12297 m
.84985 .12672 L
s
.89845 .12297 m
.89845 .12672 L
s
.94704 .12297 m
.94704 .12672 L
s
.25 Mabswid
0 .12297 m
1 .12297 L
s
.02381 .0361 m
.03006 .0361 L
s
[(0.8)] .01131 .0361 1 0 Mshowa
.02381 .20984 m
.03006 .20984 L
s
[(1.2)] .01131 .20984 1 0 Mshowa
.02381 .29672 m
.03006 .29672 L
s
[(1.4)] .01131 .29672 1 0 Mshowa
.02381 .38359 m
.03006 .38359 L
s
[(1.6)] .01131 .38359 1 0 Mshowa
.02381 .47046 m
.03006 .47046 L
s
[(1.8)] .01131 .47046 1 0 Mshowa
.02381 .55734 m
.03006 .55734 L
s
[(2)] .01131 .55734 1 0 Mshowa
.125 Mabswid
.02381 .05781 m
.02756 .05781 L
s
.02381 .07953 m
.02756 .07953 L
s
.02381 .10125 m
.02756 .10125 L
s
.02381 .14469 m
.02756 .14469 L
s
.02381 .16641 m
.02756 .16641 L
s
.02381 .18812 m
.02756 .18812 L
s
.02381 .23156 m
.02756 .23156 L
s
.02381 .25328 m
.02756 .25328 L
s
.02381 .275 m
.02756 .275 L
s
.02381 .31843 m
.02756 .31843 L
s
.02381 .34015 m
.02756 .34015 L
s
.02381 .36187 m
.02756 .36187 L
s
.02381 .40531 m
.02756 .40531 L
s
.02381 .42703 m
.02756 .42703 L
s
.02381 .44874 m
.02756 .44874 L
s
.02381 .49218 m
.02756 .49218 L
s
.02381 .5139 m
.02756 .5139 L
s
.02381 .53562 m
.02756 .53562 L
s
.02381 .01438 m
.02756 .01438 L
s
.02381 .57905 m
.02756 .57905 L
s
.02381 .60077 m
.02756 .60077 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02867 .01472 m
.0724 .01662 L
.10155 .01965 L
.12585 .02331 L
.15015 .02806 L
.21817 .04791 L
.27162 .07106 L
.29592 .08369 L
.32021 .09744 L
.34451 .11213 L
.3688 .12759 L
.3931 .1437 L
.4174 .16035 L
.44169 .17744 L
.46599 .19492 L
.49028 .21273 L
.51458 .23083 L
.53887 .24918 L
.56317 .26775 L
.58746 .28652 L
.61176 .30548 L
.63605 .32459 L
.66035 .34385 L
.68465 .36325 L
.70894 .38276 L
.73324 .40239 L
.75753 .42213 L
.78183 .44195 L
.80612 .46187 L
.83042 .48187 L
.85471 .50195 L
.87901 .5221 L
.9033 .54231 L
.9276 .56259 L
.9519 .58293 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo4@00n_oo000Eool00`00
ooooo`0>ool7003cool001Goo`03003ooooo01Goo`L00>coo`000_oo0P001?oo00<00?ooool00ooo
0P001Ooo00<00?ooool07?oo1@00iooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003o
oooo027oo`@00>?oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3000Uool3003Pool00005
ool00?ooool00009ool20005ool00`00ooooo`0Xool4003Lool00005ool00?ooool00008ool01000
ooooo`001?oo00<00?ooool0;?oo1000f?oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03
003ooooo033oo`<00=Goo`000_oo0P002_oo0P001Ooo00<00?ooool0<ooo0P00dooo000Eool00`00
ooooo`0eool2003Aool001Goo`8003Soo`800<ooo`005Ooo00<00?ooool0>Ooo0P00cOoo000Eool0
0`00ooooo`0kool2003;ool001Goo`03003ooooo03goo`800<Woo`005Ooo00<00?ooool0?ooo0P00
aooo000Eool00`00ooooo`11ool20035ool001Goo`8004Coo`800<?oo`005Ooo00<00?ooool0>?oo
0P001?oo00<00?ooool00_oo1000=ooo1000>?oo10000ooo00<00?ooool00_oo0`00>Ooo10000_oo
000Eool00`00ooooo`0gool01000ooooo`002ooo0`00=ooo00<00?ooool0>Ooo00<00?ooool02_oo
00<00?ooool0=_oo00<00?ooool00ooo000Eool00`00ooooo`0gool01000ooooo`002ooo00<00?oo
ool00P00=Ooo00<00?ooool0>Ooo00<00?ooool02_oo00<00?ooool0=ooo00<00?ooool00_oo000E
ool00`00ooooo`0gool01000ooooo`002?oo0`001Ooo0P00<ooo00<00?ooool0>Ooo00<00?ooool0
1ooo0`00>ooo00<00?ooool00Ooo000Eool00`00ooooo`0gool01000ooooo`002?oo00<00?ooool0
1ooo00<00?ooool0<?oo00<00?ooool0>Ooo00<00?ooool01ooo00<00?ooool0??oo0@000Ooo0Ooo
000Eool2000hool01000ooooo`002?oo00<00?ooool02?oo0P00;_oo0`00>Ooo0`002Ooo00<00?oo
ool0>Ooo00@00?ooool000;oo`005Ooo00<00?ooool0>?oo0P002Ooo10002Ooo0P00;_oo00<00?oo
ool0>Ooo00<00?ooool01ooo1000>Ooo0P000ooo000Eool00`00ooooo`1Bool2002dool001Goo`03
003ooooo05Coo`03003ooooo0;7oo`005Ooo00<00?ooool0EOoo0P00/Ooo000Eool00`00ooooo`1G
ool00`00ooooo`2^ool000koool0014000;oo`005Ooo00<00?ooool02_oo00<00?ooool02_oo00<0
0?ooool02_oo00<00?ooool02_oo00<00?ooool02ooo00<00?ooool02_oo00<00?ooool02_oo0P00
2ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool0
2ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool0
2_oo00<00?ooool02_oo00<00?ooool02ooo00<00?ooool00Ooo000Eool00`00ooooo`1Kool2002[
ool001Goo`03003ooooo05goo`800:Woo`005Ooo00<00?ooool0Gooo0P00Yooo000Eool00`00oooo
o`1Qool00`00ooooo`2Tool001Goo`8006?oo`03003ooooo0:?oo`005Ooo00<00?ooool0Hooo00<0
0?ooool0X_oo000Eool00`00ooooo`1Tool2002Rool001Goo`03003ooooo06Koo`03003ooooo09oo
o`005Ooo00<00?ooool0Iooo00<00?ooool0W_oo000Eool2001Yool2002Nool001Goo`03003ooooo
06[oo`8009coo`005Ooo00<00?ooool0K?oo0P00V_oo000Eool00`00ooooo`1^ool00`00ooooo`2G
ool001Goo`03003ooooo06ooo`03003ooooo09Koo`005Ooo00<00?ooool0L?oo00<00?ooool0UOoo
000Eool2001bool2002Eool001Goo`03003ooooo07?oo`03003ooooo09;oo`005Ooo00<00?ooool0
M?oo00<00?ooool0TOoo000Eool00`00ooooo`1eool2002Aool00003ool00000008000?oo`03003o
oooo00;oo`@000Coo`03003ooooo07Ooo`03003ooooo08koo`000ooo00<00?ooool01ooo00<00?oo
ool01Ooo00<00?ooool0N?oo00<00?ooool0SOoo0003ool00`00ooooo`08ool00`00ooooo`04ool3
001iool2002=ool000?oo`03003ooooo00Woo`03003ooooo00?oo`03003ooooo07_oo`03003ooooo
08[oo`000ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0O?oo0P00R_oo00001?oo0000
00002Ooo00@00?ooool000Coo`03003ooooo07koo`03003ooooo08Ooo`000ooo00<00?ooool02?oo
0P001Ooo00<00?ooool0Oooo00<00?ooool0Q_oo000Eool00`00ooooo`20ool20026ool001Goo`80
08?oo`03003ooooo08?oo`005Ooo00<00?ooool0Pooo00<00?ooool0P_oo000Eool00`00ooooo`24
ool00`00ooooo`21ool001Goo`03003ooooo08Goo`80087oo`005Ooo00<00?ooool0Qooo00<00?oo
ool0O_oo000Eool00`00ooooo`28ool00`00ooooo`1mool001Goo`8008[oo`8007goo`005Ooo00<0
0?ooool0Rooo00<00?ooool0N_oo000Eool00`00ooooo`2<ool00`00ooooo`1iool001Goo`03003o
oooo08goo`8007Woo`005Ooo00<00?ooool0Sooo00<00?ooool0M_oo000Eool00`00ooooo`2@ool0
0`00ooooo`1eool001Goo`8009;oo`03003ooooo07Coo`005Ooo00<00?ooool0T_oo0P00M?oo000E
ool00`00ooooo`2Dool00`00ooooo`1aool001Goo`03003ooooo09Goo`03003ooooo073oo`0000?o
o`0000000P000ooo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0U_oo0P00L?oo0003ool0
0`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2Hool00`00ooooo`1]ool000?oo`03003ooooo
00Koo`D000Coo`<009Woo`03003ooooo06coo`000ooo00<00?ooool01_oo00@00?ooool000Goo`03
003ooooo09[oo`8006coo`000ooo00<00?ooool01ooo00<00?oo00001Ooo00<00?ooool0W?oo00<0
0?ooool0JOoo00001?oo000000002_oo0P001Ooo00<00?ooool0WOoo00<00?ooool0J?oo0003ool0
0`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2Nool00`00ooooo`1Wool001Goo`03003ooooo
09ooo`8006Ooo`005Ooo0P00X_oo00<00?ooool0I?oo000Eool00`00ooooo`2Rool00`00ooooo`1S
ool001Goo`03003ooooo0:?oo`8006?oo`005Ooo00<00?ooool0YOoo00<00?ooool0H?oo000Eool0
0`00ooooo`2Vool00`00ooooo`1Oool001Goo`800:Soo`8005ooo`005Ooo00<00?ooool0ZOoo00<0
0?ooool0G?oo000Eool00`00ooooo`2Zool00`00ooooo`1Kool001Goo`03003ooooo0:_oo`03003o
oooo05[oo`005Ooo00<00?ooool0[?oo00<00?ooool0FOoo000Eool00`00ooooo`2]ool00`00oooo
o`1Hool001Goo`800:ooo`03003ooooo05Ooo`005Ooo00<00?ooool0[ooo00<00?ooool0E_oo000E
ool00`00ooooo`2`ool2001Fool001Goo`03003ooooo0;;oo`03003ooooo05?oo`0000?oo`000000
0P000ooo00<00?ooool00ooo0P001Ooo00<00?ooool0/ooo00<00?ooool0D_oo0003ool00`00oooo
o`07ool01000ooooo`001?oo00<00?ooool0]?oo0P00D_oo0003ool00`00ooooo`07ool01000oooo
o`001?oo0`00]_oo00<00?ooool0Cooo0003ool00`00ooooo`07ool30005ool00`00ooooo`2gool2
001?ool000?oo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0;Woo`03003ooooo04coo`00
00Coo`00000000Woo`03003ooooo00Goo`03003ooooo0;[oo`03003ooooo04_oo`000ooo00<00?oo
ool02?oo0`001?oo00<00?ooool0^ooo0P00Booo000Eool00`00ooooo`2mool00`00ooooo`18ool0
01Goo`800;ooo`03003ooooo04Ooo`005Ooo00<00?ooool0_ooo00<00?ooool0A_oo000Eool00`00
ooooo`30ool00`00ooooo`15ool001Goo`03003ooooo0<7oo`03003ooooo04Coo`005Ooo00<00?oo
ool0`_oo00<00?ooool0@ooo000Eool00`00ooooo`33ool00`00ooooo`12ool001Goo`800<Goo`80
04;oo`005Ooo00<00?ooool0a_oo00<00?ooool0?ooo000Eool00`00ooooo`37ool00`00ooooo`0n
ool001Goo`03003ooooo0<Soo`8003koo`005Ooo00<00?ooool0b_oo00<00?ooool0>ooo000Eool0
0`00ooooo`3;ool00`00ooooo`0jool001Goo`800<goo`03003ooooo03Woo`005Ooo00<00?ooool0
cOoo0P00>Ooo000Eool00`00ooooo`3?ool00`00ooooo`0fool001Goo`03003ooooo0=3oo`03003o
oooo03Goo`0000?oo`0000000P000ooo00<00?ooool00ooo0P001Ooo00<00?ooool0dOoo00<00?oo
ool0=?oo0003ool00`00ooooo`07ool01000ooooo`001?oo00<00?ooool0d_oo00<00?ooool0<ooo
0003ool00`00ooooo`07ool01000ooooo`001?oo0`00dooo0P00<ooo0003ool00`00ooooo`08ool2
0005ool00`00ooooo`3Eool00`00ooooo`0`ool000?oo`03003ooooo00Ooo`04003ooooo0004ool0
0`00ooooo`3Fool00`00ooooo`0_ool00004ool000000009ool01000ooooo`001?oo00<00?ooool0
eooo00<00?ooool0;_oo0003ool00`00ooooo`08ool20005ool00`00ooooo`3Hool00`00ooooo`0]
ool001Goo`03003ooooo0=Woo`03003ooooo02coo`005Ooo0P00fooo0P00;?oo000Eool00`00oooo
o`3Lool00`00ooooo`0Yool001Goo`03003ooooo0=goo`03003ooooo02Soo`005Ooo00<00?ooool0
g_oo00<00?ooool09ooo000Eool00`00ooooo`3Oool00`00ooooo`0Vool001Goo`800>7oo`8002Ko
o`005Ooo00<00?ooool0h_oo00<00?ooool08ooo000Eool00`00ooooo`3Sool00`00ooooo`0Rool0
01Goo`03003ooooo0>Coo`03003ooooo027oo`005Ooo00<00?ooool0iOoo0P008Ooo000Eool00`00
ooooo`3Wool00`00ooooo`0Nool001Goo`800>Woo`03003ooooo01goo`005Ooo00<00?ooool0jOoo
0P007Ooo000Eool00`00ooooo`3[ool00`00ooooo`0Jool001Goo`03003ooooo0>coo`03003ooooo
01Woo`003Ooo10001?oo00<00?ooool0kOoo00<00?ooool06?oo000=ool00`00ooooo`05ool00`00
ooooo`3^ool00`00ooooo`0Gool000koo`03003ooooo00Coo`<00>ooo`03003ooooo01Koo`003ooo
00<00?ooool00ooo00<00?ooool0l?oo00<00?ooool05Ooo000@ool00`00ooooo`02ool00`00oooo
o`3aool00`00ooooo`0Dool000goo`04003ooooo0004ool00`00ooooo`3bool2000Dool000koo`80
00Goo`03003ooooo0?Coo`03003ooooo017oo`005Ooo00<00?ooool0mOoo00<00?ooool04?oo000E
ool2003gool2000@ool001Goo`03003ooooo0?Soo`03003ooooo00goo`005Ooo00<00?ooool0nOoo
00<00?ooool03?oo000Eool00`00ooooo`3jool00`00ooooo`0;ool001Goo`03003ooooo0?_oo`03
003ooooo00[oo`005Ooo00<00?ooool0o?oo00<00?ooool02Ooo000Eool2003nool00`00ooooo`08
ool001Goo`03003ooooo0?koo`03003ooooo00Ooo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo003ooolQool0
0?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.162815, 0.674626, \
0.0076156, 0.00851929}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%89, %87, %85, %91, PlotRange \[Rule] {0.5, 2}, 
      Frame \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.206011 0.412023 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.2619 -0.0125 -9 -9 ]
[.2619 -0.0125 9 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -9 -9 ]
[.7381 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .0412 -18 -4.5 ]
[-0.0125 .0412 0 4.5 ]
[-0.0125 .12361 -18 -4.5 ]
[-0.0125 .12361 0 4.5 ]
[-0.0125 .20601 -6 -4.5 ]
[-0.0125 .20601 0 4.5 ]
[-0.0125 .28842 -18 -4.5 ]
[-0.0125 .28842 0 4.5 ]
[-0.0125 .37082 -18 -4.5 ]
[-0.0125 .37082 0 4.5 ]
[-0.0125 .45322 -18 -4.5 ]
[-0.0125 .45322 0 4.5 ]
[-0.0125 .53563 -18 -4.5 ]
[-0.0125 .53563 0 4.5 ]
[-0.0125 .61803 -6 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.2619 0 m
.2619 .00625 L
s
[(0.5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(1)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(1.5)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(2)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0412 m
.00625 .0412 L
s
[(0.6)] -0.0125 .0412 1 0 Mshowa
0 .12361 m
.00625 .12361 L
s
[(0.8)] -0.0125 .12361 1 0 Mshowa
0 .20601 m
.00625 .20601 L
s
[(1)] -0.0125 .20601 1 0 Mshowa
0 .28842 m
.00625 .28842 L
s
[(1.2)] -0.0125 .28842 1 0 Mshowa
0 .37082 m
.00625 .37082 L
s
[(1.4)] -0.0125 .37082 1 0 Mshowa
0 .45322 m
.00625 .45322 L
s
[(1.6)] -0.0125 .45322 1 0 Mshowa
0 .53563 m
.00625 .53563 L
s
[(1.8)] -0.0125 .53563 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(2)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .10301 m
.00375 .10301 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .30902 m
.00375 .30902 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .39142 m
.00375 .39142 L
s
0 .41202 m
.00375 .41202 L
s
0 .43262 m
.00375 .43262 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .51503 m
.00375 .51503 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
0 .0206 m
.00375 .0206 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.2619 .61178 m
.2619 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.7381 .61178 m
.7381 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0412 m
1 .0412 L
s
.99375 .12361 m
1 .12361 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .28842 m
1 .28842 L
s
.99375 .37082 m
1 .37082 L
s
.99375 .45322 m
1 .45322 L
s
.99375 .53563 m
1 .53563 L
s
.125 Mabswid
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .10301 m
1 .10301 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .30902 m
1 .30902 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .41202 m
1 .41202 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .51503 m
1 .51503 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.99625 .0206 m
1 .0206 L
s
.99625 0 m
1 0 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02857 .10333 m
.07143 .10513 L
.1 .10801 L
.12381 .11147 L
.14762 .11598 L
.21429 .13482 L
.26667 .15677 L
.29048 .16875 L
.31429 .18179 L
.3381 .19573 L
.3619 .2104 L
.38571 .22568 L
.40952 .24146 L
.43333 .25768 L
.45714 .27426 L
.48095 .29115 L
.50476 .30832 L
.52857 .32573 L
.55238 .34335 L
.57619 .36115 L
.6 .37913 L
.62381 .39726 L
.64762 .41553 L
.67143 .43393 L
.69524 .45244 L
.71905 .47106 L
.74286 .48978 L
.76667 .50859 L
.79048 .52748 L
.81429 .54645 L
.8381 .5655 L
.8619 .58461 L
.88571 .60378 L
s
.88571 .60378 m
.90335 .61803 L
s
[ .02 .02 ] 0 setdash
.0481 .04938 m
.0719 .05217 L
.09571 .05664 L
.11952 .06262 L
.14333 .06985 L
.16714 .07809 L
.19095 .08706 L
.21476 .0965 L
.23857 .1062 L
.26238 .11606 L
.28619 .12605 L
.31 .13622 L
.33381 .14668 L
.35762 .15752 L
.38143 .16882 L
.40524 .18065 L
.42905 .19305 L
.45286 .20608 L
.47667 .21977 L
.50048 .23413 L
.52429 .2492 L
.5481 .26499 L
.5719 .28148 L
.59571 .29868 L
.61952 .31653 L
.64333 .335 L
.66714 .354 L
.69095 .37345 L
.71476 .39325 L
.73857 .41331 L
.76238 .43357 L
.78619 .45395 L
.81 .47442 L
.83381 .49494 L
.85762 .5155 L
.88143 .53607 L
.90524 .55666 L
.92905 .57725 L
.95286 .59785 L
s
[ .002 .01 ] 0 setdash
.02429 .04842 m
.07143 .05295 L
.09524 .05844 L
.11905 .06579 L
.14286 .07479 L
.16667 .08518 L
.19048 .09675 L
.21429 .10932 L
.2381 .12273 L
.2619 .13684 L
.28571 .15156 L
.30952 .16679 L
.33333 .18247 L
.35714 .19854 L
.38095 .21494 L
.40476 .23164 L
.42857 .24861 L
.45238 .26582 L
.47619 .28324 L
.5 .30085 L
.52381 .31863 L
.54762 .33658 L
.57143 .35467 L
.59524 .37289 L
.61905 .39123 L
.64286 .40969 L
.66667 .42825 L
.69048 .44691 L
.71429 .46566 L
.7381 .4845 L
.7619 .50341 L
.78571 .52239 L
.80952 .54145 L
.83333 .56057 L
.85714 .57975 L
.88095 .59898 L
s
.88095 .59898 m
.90447 .61803 L
s
.8 g
[ ] 0 setdash
.2619 0 m
.30316 .03569 L
.34309 .07024 L
.3815 .10348 L
.42237 .13884 L
.46172 .17289 L
.49955 .20562 L
.53984 .24048 L
.57861 .27403 L
.61984 .3097 L
.65954 .34406 L
.69774 .3771 L
.73838 .41227 L
.77751 .44613 L
.81909 .48211 L
.85916 .51677 L
.89771 .55013 L
.93871 .58561 L
.97619 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{405, 250.125},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006E0000nQ000`40n007h00OooooU_oo003ooonF
ool002;oo`80057oo`8000Coo`03003ooooo00;oo`<0053oo`@004ooo`@000?oo`03003ooooo00;o
o`<0053oo`@000coo`008Ooo00@00?ooool004ooo`04003ooooo000;ool00`00ooooo`1?ool00`00
ooooo`1@ool00`00ooooo`0:ool00`00ooooo`1=ool00`00ooooo`0=ool0027oo`04003ooooo001?
ool01000ooooo`002ooo00<00?ooool0Cooo00<00?ooool0D?oo00<00?ooool02_oo00<00?ooool0
C_oo00<00?ooool03?oo000Qool01000ooooo`00Cooo00@00?ooool000Soo`<005;oo`03003ooooo
053oo`03003ooooo00Ooo`<005;oo`03003ooooo00_oo`008Ooo00@00?ooool004ooo`04003ooooo
0008ool00`00ooooo`1Bool00`00ooooo`1@ool00`00ooooo`07ool00`00ooooo`1Cool00`00oooo
o`0:ool0027oo`04003ooooo001?ool01000ooooo`002?oo00<00?ooool0D?oo0`00D?oo0`002Ooo
00<00?ooool0D?oo00@00?ooool000coo`008_oo0P00DOoo0P002Ooo1000DOoo00<00?ooool0D?oo
00<00?ooool01ooo1000D?oo0P003Ooo003ooonFool00?oooiKoo`00ooooU_oo003ooonFool00?oo
oiKoo`006_ooHP0000?>N@000000o`004`001?oo000Jool00`00ooooo`06ool00`00ooooo`0?ool0
0`00ooooo`0>ool00`00ooooo`0?ool00`00ooooo`0?ool00`00ooooo`0?ool00`00cWWoo`0?ool0
0`00ooooo`0>ool00`00ooooo`0?ool00`00ooooo`0?ool00`00ooooo`0?ool00`00ooooo`0?ool0
0`00ooooo`0>ool00`00ooooo`0?ool00`00ooooo`0?ool00`00ooooo`0?ool00`00ooooo`0?ool0
0`00ooooo`0>ool00`00ooooo`0?ool00`00ooooo`0?ool00`00ooooo`0?ool00`00ooooo`06ool0
0`00ooooo`02ool001[oo`03003ooooo00Koo`03003ooooo05Koo`03003oolii05Koo`03003ooooo
05Koo`03003ooooo05Koo`03003ooooo00Koo`03003ooooo00;oo`006_oo00<00?ooool0H_oo0/ii
oooo4?oo00<00?ooool00_oo000Jool00`00ooooo`1Tool00liiooooo`3oool=ool00`00ooooo`02
ool001[oo`03003ooooo06Goo`03cWWooooo0?ooo`coo`03003ooooo00;oo`006_oo00<00?ooool0
I_oo00?>NOooool0oooo2ooo00<00?ooool00_oo000Jool00`00ooooo`1Wool00liiooooo`3oool:
ool00`00ooooo`02ool001[oo`8006Woo`03cWWooooo0?ooo`Ooo`<000Coo`006_oo00<00?ooool0
JOoo00?>NOooool0oooo2?oo00<00?ooool00_oo000Jool00`00ooooo`1Zool2cWWoool8ool00`00
ooooo`02ool001[oo`03003ooooo06coo`03cWWooooo0?ooo`Goo`03003ooooo00;oo`006_oo00<0
0?ooool0KOoo00?>NOooool0oooo1?oo00<00?ooool00_oo0005ool20004ool00`00ooooo`03ool2
0007ool00`00ooooo`1^ool00liiooooo`3oool3ool00`00ooooo`02ool000Coo`04003ooooo0008
ool01000ooooo`001_oo00<00?ooool0Kooo00?>NOooool0oooo0_oo00<00?ooool00_oo0004ool0
1000ooooo`002?oo00@00?ooool000Koo`<0073oo`03cWWooooo0?goo`@000Coo`001?oo00@00?oo
ool000Soo`<000Ooo`03003ooooo077oo`;>NOooo`7oo`03003ooooo00;oo`001?oo00@00?ooool0
00Soo`03003ooooo00Ooo`03003ooooo07?oo`03cWWooooo0?goo`03003ooooo00;oo`001?oo00@0
0?ooool000Soo`03003ooooo00Ooo`03003ooooo00Koo`8000Ooo`@0067oo`03cWWooooo0?coo`03
003ooooo00;oo`001Ooo0P002_oo0`001_oo00<00?ooool02_oo0P000ooo0P000_oo1000G_oo00?>
NOooool0nooo00<00?ooool00_oo000Jool00`00ooooo`0Dool01000oooooooo0P00G?oo00?>NOoo
ool0n_oo00<00?ooool00_oo000Jool00`00ooooo`0Lool7001Dool00liiooooo`3iool00`00oooo
o`02ool001[oo`03003ooooo027oo`03003oo`00008005;oo`03cWWooooo0?Soo`03003ooooo00;o
o`006_oo0P009_oo00<00?ooool0DOoo0/iim_oo0`001?oo000Jool00`00ooooo`0Vool00`00oooo
o`03ool2001=ool00liiooooo`3eool00`00ooooo`02ool001[oo`03003ooooo02Woo`8000?oo`<0
04_oo`03cWWooooo0?Coo`03003ooooo00;oo`006_oo00<00?ooool0;Ooo0P000_oo0`00BOoo00?>
NOooool0looo00<00?ooool00_oo000Jool00`00ooooo`1nool00liiooooo`3bool00`00ooooo`02
ool001[oo`03003ooooo03;oo`03003ooooo04[oo`03cWWooooo0?7oo`03003ooooo00;oo`006_oo
00<00?ooool0=_oo00@00?ooooooo`8004Coo`03cWWooooo0?3oo`03003ooooo00;oo`006_oo00<0
0?ooool0=_oo00<00?ooool00ooo0P00@ooo00?>NOooool0kooo00<00?ooool00_oo000Jool2000k
ool01000oooooooo0P00@_oo00?>NOooool0k?oo0`001?oo000Jool00`00ooooo`0jool00`00oooo
o`03ool20011ool00liiooooo`3]ool00`00ooooo`02ool001[oo`03003ooooo03koo`03003ooooo
04?oo`;>NNgoo`03003ooooo00;oo`006_oo00<00?ooool0?ooo00<00?ooool01_oo0P00??oo00?>
NOooool0j_oo00<00?ooool00_oo000Jool00`00ooooo`12ool00`00ooooo`05ool2000kool00lii
ooooo`3Yool00`00ooooo`02ool001[oo`03003ooooo04?oo`03003ooooo00Koo`8003[oo`03cWWo
oooo0>Soo`03003ooooo00;oo`006_oo00<00?ooool0A_oo00<00?ooool01Ooo0P00>Ooo00?>NOoo
ool0iooo00<00?ooool00_oo000Jool00`00ooooo`17ool00`00ooooo`10ool00liiooooo`3Vool0
0`00ooooo`02ool001[oo`8000WooaD002goo`03003ooooo03koo`03cWWooooo0>?oo`<000Coo`00
6_oo00<00?ooool07Ooo20009_oo00<00?ooool02?oo0P00=?oo00?>NOooool0i?oo00<00?ooool0
0_oo000Jool00`00ooooo`0Uool4000Uool00`00ooooo`07ool2000cool2cWWTool00`00ooooo`02
ool001[oo`03003ooooo02Woo`L001koo`03003ooooo00Woo`8003?oo`03cWWooooo0>7oo`03003o
oooo00;oo`006_oo00<00?ooool0<?oo1P009_oo0P00<_oo00?>NOooool0h?oo00<00?ooool00_oo
0005ool20004ool00`00ooooo`03ool20007ool00`00ooooo`0fool4000Hool00`00ooooo`0lool0
0liiooooo`3Oool00`00ooooo`02ool000Coo`04003ooooo0008ool01000ooooo`001_oo00<00?oo
ool0>_oo0`00EOoo00?>NOooool0g_oo00<00?ooool00_oo0004ool01000ooooo`002?oo00@00?oo
ool000Koo`<003goo`@001Goo`03003ooooo00_oo`8002goo`;>NM_oo`@000Coo`001?oo00@00?oo
ool000Woo`8000Ooo`03003ooooo047oo`@0027oo`@002_oo`03cWWooooo0=_oo`03003ooooo00;o
o`001?oo00@00?ooool000Soo`04003ooooo0006ool00`00ooooo`15ool3000Aool2000?ool2000Z
ool00liiooooo`3Jool00`00ooooo`02ool000Coo`04003ooooo0008ool01000ooooo`001_oo00<0
0?ooool0B?oo1000Booo00?>NOooool0fOoo00<00?ooool00_oo0005ool2000:ool20007ool00`00
ooooo`1<ool3000>ool00`00ooooo`0hool2cWWIool00`00ooooo`02ool001[oo`03003ooooo04oo
o`8000goo`03003ooooo017oo`8002Koo`03cWWooooo0=Koo`03003ooooo00;oo`006_oo00<00?oo
ool0DOoo0P008Ooo0P009Ooo00?>NOooool0eOoo00<00?ooool00_oo000Jool00`00ooooo`1Cool2
000<ool2000Cool2000Tool00liiooooo`3Dool00`00ooooo`02ool001[oo`8005Koo`80027oo`80
02?oo`03cWWooooo0=7oo`<000Coo`006_oo00<00?ooool0Eooo0`002ooo0P00=ooo00?>NOooool0
d_oo00<00?ooool00_oo000Jool00`00ooooo`1Jool20013ool00liiooooo`3Aool00`00ooooo`02
ool001[oo`03003ooooo05coo`8000_oo`03003ooooo01?oo`03003ooooo01koo`03cWWooooo0=3o
o`03003ooooo00;oo`006_oo00<00?ooool0G_oo0P002Ooo00<00?ooool05?oo0P007ooo00?>NOoo
ool0cooo00<00?ooool00_oo000Jool00`00ooooo`1Pool2000Pool2000Nool2cWW?ool00`00oooo
o`02ool001[oo`03003ooooo06;oo`8000Woo`03003ooooo01Coo`8001koo`03cWWooooo0<coo`03
003ooooo00;oo`006_oo00<00?ooool0I?oo0P008?oo00<00?ooool07?oo00?>NOooool0booo00<0
0?ooool00_oo000Jool2001Wool20008ool2000dool00liiooooo`38ool30004ool001[oo`03003o
oooo06Soo`8000Ooo`03003ooooo03?oo`03cWWooooo0<Woo`03003ooooo00;oo`006_oo00<00?oo
ool0J_oo0P008Ooo0P006Ooo00?>NOooool0b?oo00<00?ooool00_oo000Jool00`00ooooo`1/ool2
0006ool2000Iool2000Hool00liiooooo`37ool00`00ooooo`02ool001[oo`03003ooooo06koo`80
027oo`8001Ooo`03cWWooooo0<Koo`03003ooooo00;oo`006_oo00<00?ooool0L?oo0P001_oo00<0
0?ooool06?oo0P005_oo0/iia_oo00<00?ooool00_oo000Jool00`00ooooo`1bool00`00ooooo`03
ool00`00ooooo`0bool00liiooooo`33ool00`00ooooo`02ool001[oo`8007Coo`8003Woo`03cWWo
oooo0<3oo`<000Coo`006_oo00<00?ooool0MOoo0P001Ooo00<00?ooool0<?oo00?>NOooool0`Ooo
00<00?ooool00_oo000Jool00`00ooooo`1gool2000Qool2000Dool2cWW1ool00`00ooooo`02ool0
01[oo`03003ooooo07Woo`8000Coo`03003ooooo01[oo`8001Coo`03cWWooooo0;koo`03003ooooo
00;oo`006_oo00<00?ooool0Nooo00<00?ooool00_oo00<00?ooool06ooo0P004ooo00?>NOooool0
_Ooo00<00?ooool00_oo000Jool00`00ooooo`1lool2000Rool2000Bool00liiooooo`2lool00`00
ooooo`02ool0013oo`@000Koo`03003ooooo07koo`03003ooooo00;oo`80033oo`;>NKcoo`03003o
oooo00;oo`004_oo00<00?ooool01Ooo00<00?ooool0Oooo0P00=_oo00?>NOooool0^Ooo00<00?oo
ool00_oo000Bool00`00ooooo`05ool30021ool00`00ooooo`03ool00`00ooooo`0^ool00liioooo
o`2eool40004ool001;oo`03003ooooo00Goo`03003ooooo08;oo`8000?oo`03003ooooo01goo`03
003ooooo00ooo`03cWWooooo0;Ooo`03003ooooo00;oo`004_oo00<00?ooool01Ooo00<00?ooool0
Q?oo00<00?ooool08Ooo0P004?oo00?>NOooool0]_oo00<00?ooool00_oo000@ool30007ool00`00
ooooo`25ool20003ool2000Nool2000?ool2cWVfool00`00ooooo`02ool001;oo`03003ooooo00Go
o`03003ooooo08Ooo`8002?oo`8000ooo`03cWWooooo0;?oo`03003ooooo00;oo`006_oo00<00?oo
ool0ROoo0P000ooo00<00?ooool07Ooo00<00?ooool03Ooo00?>NOooool0/_oo00<00?ooool00_oo
000Jool00`00ooooo`2;ool20002ool00`00ooooo`0]ool00liiooooo`2aool00`00ooooo`02ool0
01[oo`03003ooooo08goo`03003ooooo033oo`03cWWooooo0;3oo`03003ooooo00;oo`006_oo0P00
Sooo0P000_oo00<00?ooool07ooo00<00?ooool02_oo00?>NOooool0[Ooo0`001?oo000Jool00`00
ooooo`2@ool00`00ooooo`0Rool00`00ooooo`0:ool00liiooooo`2^ool00`00ooooo`02ool001[o
o`03003ooooo097oo`8002?oo`8000_oo`;>NJkoo`03003ooooo00;oo`006_oo00<00?ooool0Tooo
00@00?oo0000027oo`8000_oo`03cWWooooo0:_oo`03003ooooo00;oo`006_oo00<00?ooool0U?oo
0P009?oo0P002_oo00?>NOooool0Z_oo00<00?ooool00_oo000Jool00`00ooooo`2Fool01000oooo
o`00;Ooo00?>NOooool0ZOoo00<00?ooool00_oo000Jool00`00ooooo`2Gool200000ooo003oo`0/
ool00liiooooo`2Xool00`00ooooo`02ool001[oo`8009[oo`03003ooooo02goo`03cWWooooo0:Go
o`<000Coo`006_oo00<00?ooool0V_oo0P0000?oo`00ool08_oo00<00?ooool01_oo0/iiYooo00<0
0?ooool00_oo000Jool00`00ooooo`2Lool00`00ooooo`0Sool20008ool00liiooooo`2Tool00`00
ooooo`02ool001[oo`03003ooooo09goo`800003ool00?oo02;oo`03003ooooo00Koo`03cWWooooo
0:?oo`03003ooooo00;oo`006_oo00<00?ooool0Wooo00<00?oo00008ooo0P001ooo00?>NOooool0
X_oo00<00?ooool00_oo000Jool00`00ooooo`2Pool00`00ooooo`0Tool00`00ooooo`05ool00lii
ooooo`2Qool00`00ooooo`02ool001[oo`03003ooooo0:7oo`800003ool0000002[oo`03cWWooooo
0:3oo`03003ooooo00;oo`006_oo00<00?ooool0Xooo00<00?ooool0:ooo0/iiX?oo00<00?ooool0
0_oo000Jool2002Uool00`00ooooo`0/ool00liiooooo`2Kool30004ool001[oo`03003ooooo0:Go
o`800003ool00?oo02[oo`03cWWooooo09coo`03003ooooo00;oo`006_oo00<00?ooool0Yooo00<0
0?ooool08ooo00<00?ooool01Ooo00?>NOooool0Vooo00<00?ooool00_oo000Jool00`00ooooo`2X
ool01000ooooo`008_oo0P001_oo00?>NOooool0V_oo00<00?ooool00_oo000Jool00`00ooooo`2Y
ool200000ooo003oo`0Rool20005ool00liiooooo`2Iool00`00ooooo`02ool001[oo`03003ooooo
0:_oo`03003ooooo02Coo`8000Coo`03cWWooooo09Soo`03003ooooo00;oo`001?oo10000ooo00<0
0?ooool00_oo10001_oo00<00?ooool0[?oo0P0000?oo`00ool08_oo00<00?ooool00ooo0/iiV?oo
00<00?ooool00_oo0006ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`2^ool00`00oooo
o`0Zool00liiooooo`2Eool00`00ooooo`02ool000Koo`03003ooooo00Soo`03003ooooo00Koo`<0
0:ooo`8002_oo`03cWWooooo097oo`@000Coo`001_oo00<00?ooool02Ooo00<00?ooool01Ooo00<0
0?ooool0/Ooo0`00:Ooo00?>NOooool0Tooo00<00?ooool00_oo0006ool00`00ooooo`0:ool00`00
ooooo`04ool00`00ooooo`2bool2000Vool01@00oooooooocWT0U?oo00<00?ooool00_oo0004ool3
0009ool01000ooooo`001_oo00<00?ooool0]?oo00<00?oo00009?oo00D00?ooooooolii09?oo`03
003ooooo00;oo`001_oo00<00?ooool02?oo0P001ooo00<00?ooool0]Ooo0`009?oo00D00?oooooo
olii09;oo`03003ooooo00;oo`006_oo00<00?ooool0]ooo00<00?ooool08ooo00@00?ooooooo`;>
NI3oo`03003ooooo00;oo`006_oo00<00?ooool0^?oo0`008ooo0P000ooo00?>NOooool0SOoo00<0
0?ooool00_oo000Jool00`00ooooo`2jool00`00ooooo`0Sool01@00oooooooocWT0S_oo00<00?oo
ool00_oo000Jool2002lool00`00ooooo`0Wool00liiooooo`29ool30004ool001[oo`03003ooooo
0;coo`<002Ooo`03cWWooooo08[oo`03003ooooo00;oo`006_oo00<00?ooool0_Ooo00<00?ooool0
9ooo00?>NOooool0ROoo00<00?ooool00_oo000Jool00`00ooooo`2nool200000ooo003oo`0Sool0
0`00ooo>N@2:ool00`00ooooo`02ool001[oo`03003ooooo0<3oo`8002Goo`03003oolii08Woo`03
003ooooo00;oo`006_oo00<00?ooool0`Ooo00<00?ooool09?oo00<00?oocWT0R?oo00<00?ooool0
0_oo000Jool00`00ooooo`32ool01000ool000008ooo00<00?oocWT0Qooo00<00?ooool00_oo000J
ool20034ool2000Uool01000ooo>NLiiPooo0`001?oo000Jool00`00ooooo`35ool00`00ooooo`0S
ool01000oooooliiQ?oo00<00?ooool00_oo000Jool00`00ooooo`36ool4000Uool00liiooooo`21
ool00`00ooooo`02ool001[oo`03003ooooo0<Soo`03003ooooo02Goo`03cWWooooo083oo`03003o
oooo00;oo`006_oo00<00?ooool0bOoo0P0000?oo`00ool08ooo00?>NOooool0Oooo00<00?ooool0
0_oo000Jool00`00ooooo`3;ool2000Tool00`00cWWoo`1oool00`00ooooo`02ool001[oo`03003o
oooo0<coo`03003ooooo02?oo`03003>NOoo07koo`03003ooooo00;oo`006_oo00<00?ooool0cOoo
0`008ooo00<00<iiool0OOoo00<00?ooool00_oo000Jool2003@ool2000Tool2cWUjool30004ool0
01[oo`03003ooooo0=3oo`03003ooooo02?oo`03003>NOoo07[oo`03003ooooo00;oo`006_oo00<0
0?ooool0dOoo0`008ooo00<00<iiool0NOoo00<00?ooool00_oo000Jool00`00ooooo`3Cool00`00
ooooo`0Sool00liiooooo`1gool00`00ooooo`02ool001[oo`03003ooooo0=Coo`03003ooooo02?o
o`;>NGOoo`03003ooooo00;oo`006_oo00<00?ooool0eOoo0`009?oo00?>NOooool0M?oo00<00?oo
ool00_oo0004ool40003ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`3Fool2000Uool0
0liiooooo`1cool00`00ooooo`02ool000Koo`03003ooooo00Woo`03003ooooo00Goo`03003ooooo
0=Soo`03003oo`0002;oo`03003>NOoo07?oo`03003ooooo00;oo`001_oo00<00?ooool01_oo1@00
1_oo0`00fOoo0P008ooo00<00<iicWT0Kooo10001?oo0006ool00`00ooooo`06ool01000ooooo`00
1ooo00<00?ooool0f_oo0P008ooo00<00?oocWT0LOoo00<00?ooool00_oo0006ool00`00ooooo`07
ool00`00ool00007ool00`00ooooo`3Lool3000Qool00`00ooo>N@1`ool00`00ooooo`02ool000Co
o`<000[oo`8000Ooo`03003ooooo0=goo`03003ooooo027oo`800003cWWooooo06goo`03003ooooo
00;oo`001_oo00<00?ooool02Ooo00<00?ooool01Ooo00<00?ooool0g_oo0P008ooo00<00<iiool0
KOoo00<00?ooool00_oo000Jool00`00ooooo`3Pool3000Rool2cWU/ool00`00ooooo`02ool001[o
o`03003ooooo0>7oo`8002Coo`03cWWooooo06Woo`03003ooooo00;oo`006_oo0P00i?oo00<00?oo
00008_oo00?>NOooool0I_oo0`001?oo000Jool00`00ooooo`3Tool2000Rool00`00cWWoo`1Xool0
0`00ooooo`02ool001[oo`03003ooooo0>Koo`03003ooooo023oo`03003>NOoo06Ooo`03003ooooo
00;oo`006_oo00<00?ooool0iooo0P008Ooo00<00<iiool0I_oo00<00?ooool00_oo000Jool00`00
ooooo`3Xool2000Qool00`00cWWoo`1Uool00`00ooooo`02ool001[oo`03003ooooo0>[oo`03003o
oooo023oo`;>NFCoo`03003ooooo00;oo`006_oo00<00?ooool0jooo0P008Ooo00<00<iiool0H_oo
00<00?ooool00_oo000Jool00`00ooooo`3/ool2000Rool00liiooooo`1Pool00`00ooooo`02ool0
01[oo`800>ooo`03003ooooo023oo`03cWWooooo05goo`<000Coo`006_oo00<00?ooool0kooo0P00
8Ooo00?>NOooool0G_oo00<00?ooool00_oo000Jool00`00ooooo`3`ool00`00ooooo`0Pool00lii
ooooo`1Mool00`00ooooo`02ool001[oo`03003ooooo0?7oo`<0023oo`03cWWooooo05coo`03003o
oooo00;oo`006_oo00<00?ooool0looo00<00?ooool07ooo00?>NOooool0Fooo00<00?ooool00_oo
000Jool00`00ooooo`3dool00`00ooooo`0Oool00liiooooo`1Jool00`00ooooo`02ool001[oo`03
003ooooo0?Goo`80023oo`03cWT00?oo05Woo`03003ooooo00;oo`006_oo00<00?ooool0mooo00<0
0?ooool07_oo0/iiFOoo00<00?ooool00_oo000Jool2003iool00`00ooooo`0Oool00liiooooo`1D
ool30004ool001[oo`03003ooooo0?Woo`80023oo`03cWWooooo05Goo`03003ooooo00;oo`006_oo
00<00?ooool0n_oo0P008?oo00?>NOooool0E?oo00<00?ooool00_oo000Jool00`00ooooo`3lool0
0`00ooooo`0Nool00liiooooo`1Cool00`00ooooo`02ool001[oo`03003ooooo0?goo`8001ooo`03
cWWooooo05;oo`03003ooooo00;oo`001?oo10000ooo00<00?ooool00ooo0P001ooo00<00?ooool0
o_oo0P007ooo00?>NOooool0DOoo00<00?ooool00_oo0006ool00`00ooooo`07ool01000ooooo`00
1_oo00<00?ooool0oooo0Ooo00<00?ooool07Ooo00?>NOooool0D?oo00<00?ooool00_oo0006ool0
0`00ooooo`07ool01000ooooo`001_oo0`00oooo0_oo00<00?ooool07Ooo0/iiCOoo10001?oo0006
ool00`00ooooo`07ool30007ool00`00ooooo`3oool3ool00`00ooooo`0Nool00liiooooo`1=ool0
0`00ooooo`02ool000Koo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo0?ooo`Coo`8001oo
o`03cWWooooo04coo`03003ooooo00;oo`001?oo0`002Ooo00<00?ooool01ooo00<00?ooool0oooo
1_oo00<00?ooool07Ooo00?>NOooool0Booo00<00?ooool00_oo0006ool00`00ooooo`08ool30006
ool00`00ooooo`3oool7ool00`00ooooo`0Mool00liiooooo`1:ool00`00ooooo`02ool001[oo`03
003ooooo0?ooo`Soo`8001koo`;>ND[oo`03003ooooo00;oo`006_oo00<00?ooool0oooo2Ooo0P00
7ooo00?>NOooool0Aooo00<00?ooool00_oo000Jool00`00ooooo`3oool;ool00`00ooooo`0Mool0
0lii003oo`16ool00`00ooooo`02ool001[oo`800?ooo`goo`8001koo`03cWT00?oo04?oo`<000Co
o`006_oo00<00?ooool0oooo3Ooo0P007_oo0/iiAOoo00<00?ooool00_oo000Jool00`00ooooo`3o
ool?ool00`00ooooo`0Mool00liiooooo`12ool00`00ooooo`02ool001[oo`03003ooooo0?oooa3o
o`03003ooooo01goo`03cWWooooo047oo`03003ooooo00;oo`006_oo00<00?ooool0oooo4Ooo0P00
7_oo00?>NOooool0@?oo00<00?ooool00_oo000Jool00`00ooooo`3ooolCool00`00ooooo`0Lool0
0liiooooo`0oool00`00ooooo`02ool001[oo`03003ooooo0?oooaCoo`03003ooooo01coo`;>NCoo
o`03003ooooo00;oo`006_oo00<00?ooool0oooo5Ooo00<00?ooool07Ooo00?>NOooool0??oo00<0
0?ooool00_oo000Jool2003ooolGool2000Nool00liiooooo`0iool30004ool001[oo`03003ooooo
0?oooaOoo`03003ooooo01goo`03cWWooooo03[oo`03003ooooo00;oo`006_oo00<00?ooool0oooo
6?oo0P007_oo00?>NOooool0>Ooo00<00?ooool00_oo000Jool00`00ooooo`3ooolJool2000Mool0
0liiooooo`0hool00`00ooooo`02ool001[oo`03003ooooo0?oooa_oo`03003ooooo01coo`03cWT0
0?oo03Ooo`03003ooooo00;oo`006_oo00<00?ooool0oooo7?oo00<00?ooool07?oo0/ii=ooo00<0
0?ooool00_oo000Jool00`00ooooo`3ooolMool2000Nool00liiooooo`0dool00`00ooooo`02ool0
01[oo`03003ooooo0?oooakoo`8001koo`03cWWooooo03?oo`03003ooooo00;oo`006_oo0P00oooo
8Ooo00<00?ooool07?oo00?>NOooool0<?oo0`001?oo000Jool00`00ooooo`3ooolQool2000Mool0
0liiooooo`0aool00`00ooooo`02ool001[oo`03003ooooo0?ooob;oo`8001goo`03cWWooooo033o
o`03003ooooo00;oo`006_oo00<00?ooool0oooo9?oo0P007?oo00?>NOooool0;ooo00<00?ooool0
0_oo000Jool00`00ooooo`3ooolUool00`00ooooo`0Kool00liiooooo`0^ool00`00ooooo`02ool0
00Coo`@000?oo`03003ooooo00?oo`8000Ooo`03003ooooo0?ooobKoo`03003ooooo01_oo`03cWT0
0?oo02goo`03003ooooo00;oo`001_oo00<00?ooool01ooo00@00?ooool000Koo`03003ooooo0?oo
obOoo`8001coo`03cWT00?oo02coo`03003ooooo00;oo`001_oo00<00?ooool01ooo00@00?ooool0
00Koo`<00?ooobWoo`03003ooooo01[oo`;>NBWoo`@000Coo`001_oo00<00?ooool02?oo0P001ooo
00<00?ooool0oooo:_oo00<00?ooool06ooo00?>NOooool0:Ooo00<00?ooool00_oo0006ool00`00
ooooo`07ool01000ooooo`001_oo00<00?ooool0oooo:ooo0P007?oo00?>NOooool0:?oo00<00?oo
ool00_oo0004ool30009ool01000ooooo`001_oo00<00?ooool0oooo;Ooo00<00?ooool06_oo00?>
NOooool09ooo00<00?ooool00_oo0006ool00`00ooooo`08ool20007ool00`00ooooo`3oool^ool0
0`00ooooo`0Jool00liiooooo`0Vool00`00ooooo`02ool001[oo`03003ooooo0?ooobooo`8001_o
o`03cWWooooo02Goo`03003ooooo00;oo`006_oo00<00?ooool0oooo<?oo0P006ooo00?>NOooool0
9?oo00<00?ooool00_oo000Jool00`00ooooo`3ooolbool00`00ooooo`0Iool00lii003oo`0Sool0
0`00ooooo`02ool001[oo`800?ooocCoo`03003ooooo01Woo`;>NB7oo`<000Coo`006_oo00<00?oo
ool0oooo=?oo0P006ooo00?>NOooool08?oo00<00?ooool00_oo000Jool00`00ooooo`3ooolfool0
0`00ooooo`0Iool00liiooooo`0Oool00`00ooooo`02ool001[oo`03003ooooo0?ooocOoo`03003o
oooo01Woo`03cWWooooo01koo`03003ooooo00;oo`006_oo00<00?ooool0oooo>?oo00<00?ooool0
6Ooo00?>NOooool07Ooo00<00?ooool00_oo000Jool00`00ooooo`3oooliool2000Jool2cWTMool0
0`00ooooo`02ool001[oo`03003ooooo0?oooc[oo`03003ooooo01[oo`03cWWooooo01[oo`03003o
oooo00;oo`006_oo00<00?ooool0oooo>ooo0P006ooo00?>NOooool06Ooo00<00?ooool00_oo000J
ool2003ooolnool00`00ooooo`0Iool00liiooooo`0Fool30004ool001[oo`03003ooooo0?ooocko
o`03003ooooo01Woo`;>NASoo`03003ooooo00;oo`006_oo00<00?ooool0oooo?ooo00<00?ooool0
6Ooo00<00<iiool05_oo00<00?ooool00_oo000Jool00`00ooooo`3ooom0ool2000Kool00liioooo
o`0Dool00`00ooooo`02ool001[oo`03003ooooo0?oood7oo`03003ooooo01[oo`03cWWooooo01?o
o`03003ooooo00;oo`006_oo00<00?ooool0oooo@_oo0P006ooo00?>NOooool04_oo00<00?ooool0
0_oo000Jool00`00ooooo`3ooom4ool2000Jool2cWTBool00`00ooooo`02ool001[oo`800?ooodKo
o`03003ooooo01[oo`03cWWooooo00goo`<000Coo`006_oo00<00?ooool0ooooA_oo0`006_oo00?>
NOooool03_oo00<00?ooool00_oo000Jool00`00ooooo`3ooom8ool00`00ooooo`0Iool00liioooo
o`0=ool00`00ooooo`02ool001[oo`03003ooooo0?ooodWoo`03003ooooo01Woo`03cWWooooo00co
o`03003ooooo00;oo`006_oo00<00?ooool0ooooB_oo0P006_oo00?>NOooool02ooo00<00?ooool0
0_oo000Jool00`00ooooo`3ooom<ool00`00ooooo`0Hool2cWT;ool00`00ooooo`02ool0013oo`@0
00Koo`03003ooooo00Koo`03003ooooo05Koo`03003ooooo05Koo`03003ooooo05Koo`03003ooooo
03Soo`03003ooooo01Woo`03cWWoo`0000Soo`03003ooooo00;oo`004?oo00<00?ooool01ooo00<0
0?ooool01_oo00<00?ooool03ooo00<00?ooool03_oo00<00?ooool03ooo00<00?ooool03ooo00<0
0?ooool03ooo00<00?ooool03ooo00<00?ooool03_oo00<00?ooool03ooo00<00?ooool03ooo00<0
0?ooool03ooo00<00?ooool03ooo00<00?ooool03_oo00<00?ooool03ooo00<00?ooool03ooo00<0
0?ooool03ooo00<00?ooool03ooo00<00?ooool03_oo00<00?ooool03ooo00<00?ooool01?oo0P00
2Ooo00<00?ooool03_oo00?>N@00ool01ooo00<00?ooool00_oo000Aool00`00ooooo`06oooo001^
00000lii000000070004ool001;oo`03003ooooo0?oooh7oo`004ooo00<00?ooool0ooooP?oo000@
ool01000ooooo`00ooooP_oo000Aool2003ooon3ool00001\
\>"],
  ImageRangeCache->{{{0, 404}, {249.125, 0}} -> {-0.198517, 0.4112, \
0.00561964, 0.00649482}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<stablePi4.eps\>", %94]\)], "Input"],

Cell[BoxData[
    \("stablePi4.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%583, %464, %452, %426, PlotRange \[Rule] {0.5, 2}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.206011 0.412023 [
[.2619 .19351 -9 -9 ]
[.2619 .19351 9 0 ]
[.5 .19351 -3 -9 ]
[.5 .19351 3 0 ]
[.7381 .19351 -9 -9 ]
[.7381 .19351 9 0 ]
[.97619 .19351 -3 -9 ]
[.97619 .19351 3 0 ]
[.01131 .0412 -18 -4.5 ]
[.01131 .0412 0 4.5 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .28842 -18 -4.5 ]
[.01131 .28842 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .45322 -18 -4.5 ]
[.01131 .45322 0 4.5 ]
[.01131 .53563 -18 -4.5 ]
[.01131 .53563 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .20601 m
.2619 .21226 L
s
[(0.5)] .2619 .19351 0 1 Mshowa
.5 .20601 m
.5 .21226 L
s
[(1)] .5 .19351 0 1 Mshowa
.7381 .20601 m
.7381 .21226 L
s
[(1.5)] .7381 .19351 0 1 Mshowa
.97619 .20601 m
.97619 .21226 L
s
[(2)] .97619 .19351 0 1 Mshowa
.125 Mabswid
.07143 .20601 m
.07143 .20976 L
s
.11905 .20601 m
.11905 .20976 L
s
.16667 .20601 m
.16667 .20976 L
s
.21429 .20601 m
.21429 .20976 L
s
.30952 .20601 m
.30952 .20976 L
s
.35714 .20601 m
.35714 .20976 L
s
.40476 .20601 m
.40476 .20976 L
s
.45238 .20601 m
.45238 .20976 L
s
.54762 .20601 m
.54762 .20976 L
s
.59524 .20601 m
.59524 .20976 L
s
.64286 .20601 m
.64286 .20976 L
s
.69048 .20601 m
.69048 .20976 L
s
.78571 .20601 m
.78571 .20976 L
s
.83333 .20601 m
.83333 .20976 L
s
.88095 .20601 m
.88095 .20976 L
s
.92857 .20601 m
.92857 .20976 L
s
.25 Mabswid
0 .20601 m
1 .20601 L
s
.02381 .0412 m
.03006 .0412 L
s
[(0.6)] .01131 .0412 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.8)] .01131 .12361 1 0 Mshowa
.02381 .28842 m
.03006 .28842 L
s
[(1.2)] .01131 .28842 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(1.4)] .01131 .37082 1 0 Mshowa
.02381 .45322 m
.03006 .45322 L
s
[(1.6)] .01131 .45322 1 0 Mshowa
.02381 .53563 m
.03006 .53563 L
s
[(1.8)] .01131 .53563 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(2)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .41202 m
.02756 .41202 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.02381 .0206 m
.02756 .0206 L
s
.02381 0 m
.02756 0 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.8 g
.5 Mabswid
.2619 0 m
.30316 .03569 L
.34309 .07024 L
.3815 .10348 L
.42237 .13884 L
.46172 .17289 L
.49955 .20562 L
.53984 .24048 L
.57861 .27403 L
.61984 .3097 L
.65954 .34406 L
.69774 .3771 L
.73838 .41227 L
.77751 .44613 L
.81909 .48211 L
.85916 .51677 L
.89771 .55013 L
.93871 .58561 L
.97619 .61803 L
s
0 g
.02857 .10333 m
.07143 .10513 L
.1 .10801 L
.12381 .11147 L
.14762 .11598 L
.21429 .13482 L
.26667 .15677 L
.29048 .16875 L
.31429 .18179 L
.3381 .19573 L
.3619 .2104 L
.38571 .22568 L
.40952 .24146 L
.43333 .25768 L
.45714 .27426 L
.48095 .29115 L
.50476 .30832 L
.52857 .32573 L
.55238 .34335 L
.57619 .36115 L
.6 .37913 L
.62381 .39726 L
.64762 .41553 L
.67143 .43393 L
.69524 .45244 L
.71905 .47106 L
.74286 .48978 L
.76667 .50859 L
.79048 .52748 L
.81429 .54645 L
.8381 .5655 L
.8619 .58461 L
.88571 .60378 L
s
.88571 .60378 m
.90335 .61803 L
s
[ .02 .02 ] 0 setdash
.0481 .04938 m
.0719 .05217 L
.09571 .05664 L
.11952 .06262 L
.14333 .06985 L
.16714 .07809 L
.19095 .08706 L
.21476 .0965 L
.23857 .1062 L
.26238 .11606 L
.28619 .12605 L
.31 .13622 L
.33381 .14668 L
.35762 .15752 L
.38143 .16882 L
.40524 .18065 L
.42905 .19305 L
.45286 .20608 L
.47667 .21977 L
.50048 .23413 L
.52429 .2492 L
.5481 .26499 L
.5719 .28148 L
.59571 .29868 L
.61952 .31653 L
.64333 .335 L
.66714 .354 L
.69095 .37345 L
.71476 .39325 L
.73857 .41331 L
.76238 .43357 L
.78619 .45395 L
.81 .47442 L
.83381 .49494 L
.85762 .5155 L
.88143 .53607 L
.90524 .55666 L
.92905 .57725 L
.95286 .59785 L
s
[ .002 .01 ] 0 setdash
.02429 .04842 m
.07143 .05295 L
.09524 .05844 L
.11905 .06579 L
.14286 .07479 L
.16667 .08518 L
.19048 .09675 L
.21429 .10932 L
.2381 .12273 L
.2619 .13684 L
.28571 .15156 L
.30952 .16679 L
.33333 .18247 L
.35714 .19854 L
.38095 .21494 L
.40476 .23164 L
.42857 .24861 L
.45238 .26582 L
.47619 .28324 L
.5 .30085 L
.52381 .31863 L
.54762 .33658 L
.57143 .35467 L
.59524 .37289 L
.61905 .39123 L
.64286 .40969 L
.66667 .42825 L
.69048 .44691 L
.71429 .46566 L
.7381 .4845 L
.7619 .50341 L
.78571 .52239 L
.80952 .54145 L
.83333 .56057 L
.85714 .57975 L
.88095 .59898 L
s
.88095 .59898 m
.90447 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001Goo`8003ooo`03cWWooooo0<Ooo`005Ooo00<00?ooool0?ooo00?>NOooool0a_oo000Eool0
0`00ooooo`10ool2cWW6ool001Goo`03003ooooo04;oo`03cWWooooo0<?oo`005Ooo00<00?ooool0
@ooo00?>NOooool0`_oo000Eool20015ool00liiooooo`31ool001Goo`03003ooooo04Goo`03cWWo
oooo0<3oo`005Ooo00<00?ooool0A_oo0/ii`?oo000Eool00`00ooooo`18ool00liiooooo`2mool0
00;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo04Woo`03cWWooooo0;coo`0000Goo`00
ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1:ool00liiooooo`2kool00005ool00?ooool0
0008ool01000ooooo`001?oo0`00Booo00?>NOooool0^_oo00001Ooo003ooooo00002?oo0`001Ooo
00<00?ooool0C?oo00?>NOooool0^Ooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo0P0000?o
o`0000000_oo0`00A_oo00?>NOooool0^?oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<0
0?ooool01?oo00<00?ooool01@00@_oo0/ii^?oo0002ool2000:ool30004ool00`00ooooo`0=ool5
000nool00liiooooo`2eool001Goo`03003ooooo013oo`D003coo`03cWWooooo0;Coo`005Ooo0P00
5?oo0P001?oo00<00?ooool0=_oo00?>NOooool0/ooo000Eool00`00ooooo`0Gool00`00ooooo`02
000gool00liiooooo`2bool001Goo`03003ooooo01[oo`03003oo`00008003Goo`03cWWooooo0;7o
o`005Ooo00<00?ooool07Ooo00<00?ooool0=Ooo00?>NOooool0/?oo000Eool00`00ooooo`0Pool0
0`00ooooo`02000aool00liiooooo`2_ool001Goo`8002Coo`04003oo`00000`ool00liiooooo`2^
ool001Goo`03003ooooo02?oo`04003oooooool2000_ool00liiooooo`2]ool001Goo`03003ooooo
02Koo`03003ooooo033oo`;>NJgoo`005Ooo00<00?ooool0:Ooo00@00?ooooooo`8002coo`03cWWo
oooo0:[oo`005Ooo00<00?ooool0:Ooo00<00?ooool00ooo0`00:_oo00?>NOooool0ZOoo000Eool0
0`00ooooo`0/ool00`00ooooo`03ool00`00ooooo`0Xool00liiooooo`2Xool001Gooa40023oo`03
003ooooo02goo`03cWWooooo0:Ooo`005Ooo00<00?ooool03_oo1`006_oo00<00?ooool01Ooo0P00
9_oo00?>NOooool0Y_oo000Eool00`00ooooo`0Eool7000Eool20006ool2000Uool2cWVVool000;o
o`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo01coo`D001?oo`03003ooooo00Coo`8002Go
o`03cWWooooo0:?oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Qool4000@
ool00`00ooooo`0[ool00liiooooo`2Rool00005ool00?ooool00008ool01000ooooo`001?oo0`00
9Ooo0`003ooo00<00?ooool01ooo0P008Ooo00?>NOooool0XOoo00001Ooo003ooooo00002Ooo0P00
1Ooo00<00?ooool0:?oo10005ooo0P008?oo00?>NOooool0X?oo00001Ooo003ooooo00002?oo00@0
0?ooool000Coo`03003ooooo02coo`@000[oo`03003ooooo00Soo`8001ooo`;>NJ3oo`0000Goo`00
ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0`ool3000:ool00`00ooooo`0Xool00liioooo
o`2Mool000;oo`8000[oo`8000Goo`03003ooooo03?oo`8000Soo`03003ooooo00_oo`8001coo`03
cWWooooo09coo`005Ooo00<00?ooool0=Ooo0P002?oo0P003?oo0P006ooo00?>NOooool0Vooo000E
ool2000hool3000Eool2000Jool00liiooooo`2Jool001Goo`03003ooooo03[oo`8000Koo`8002So
o`03cWWooooo09Woo`005Ooo00<00?ooool0??oo0P001ooo00<00?ooool09Ooo00?>NOooool0V?oo
000Eool00`00ooooo`0nool20005ool00`00ooooo`0=ool2000Gool2cWVHool001Goo`03003ooooo
043oo`8000Koo`03003ooooo00coo`8001Ooo`03cWWooooo09Goo`005Ooo00<00?ooool0@_oo00<0
0?ooool00ooo00<00?ooool03_oo0P005_oo00?>NOooool0U?oo000Eool2000hool20004ool00`00
ooooo`02ool30005ool2000Vool00liiooooo`07ool4000gool40003ool00`00ooooo`02ool3000h
ool40006ool001Goo`03003ooooo03Koo`04003ooooo000;ool2000Hool00`00ooooo`0Aool00lii
ooooo`08ool00`00ooooo`0hool00`00ooooo`0:ool00`00ooooo`0eool00`00ooooo`07ool001Go
o`03003ooooo03Koo`04003ooooo000;ool01000ool000001?oo0P004Ooo0P004_oo00?>NOooool0
1ooo00<00?ooool0>?oo00<00?ooool02_oo00<00?ooool0=_oo00<00?ooool01_oo000Eool00`00
ooooo`0fool01000ooooo`002?oo0`001?oo0P005ooo0P004Ooo00?>NOooool01_oo00<00?ooool0
>?oo00<00?ooool01ooo0`00>_oo00<00?ooool01Ooo000Eool00`00ooooo`0fool01000ooooo`00
2?oo00<00?ooool01_oo0P000ooo00<00?ooool04Ooo00<00?ooool03ooo00?>NOooool01Ooo00<0
0?ooool0>?oo00<00?ooool01ooo00<00?ooool0>ooo00<00?ooool01?oo000Eool2000gool01000
ooooo`002?oo00<00?ooool02?oo0P001?oo00<00?ooool08Ooo00?>NOooool00_oo0`00>?oo0`00
2Ooo00<00?ooool0>?oo00@00?ooool000Koo`005Ooo00<00?ooool0=ooo0P002Ooo10002Ooo00D0
0?ooooooo`0001Goo`03003ooooo00coo`03cWWooooo00?oo`03003ooooo03Soo`03003ooooo00Oo
o`@003Soo`8000Ooo`005Ooo00<00?ooool0D?oo0P000ooo00<00?ooool04_oo0P003Ooo0/iiSOoo
000Eool00`00ooooo`1Bool01@00oooooooo00005Ooo0P003Ooo00?>NOooool0R_oo000Eool00`00
ooooo`1Cool20003ool00`00ooooo`0Cool00`00ooooo`0;ool00liiooooo`29ool001Goo`03003o
oooo05Goo`8002Koo`03cWWooooo08Soo`003_ooR00000?>N@000000Q`00000Eool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool2000;ool00`00ooooo`0:ool00`00ool00009ool00`00cWWoo`0:ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`05ool001Go
o`03003ooooo05[oo`800003ool0000001Koo`8000Woo`;>NHKoo`005Ooo00<00?ooool0G?oo00<0
0?ooool06?oo00<00?ooool02?oo00?>NOooool0Pooo000Eool00`00ooooo`1Mool200000ooo0000
000Qool00liiooooo`22ool001Goo`80063oo`05003oooooool0000Pool00liiooooo`21ool001Go
o`03003ooooo063oo`800003ool00?oo01Ooo`8000Ooo`03cWWooooo083oo`005Ooo00<00?ooool0
H_oo0P0000?oo`00ool05ooo0P001_oo0/iiP?oo000Eool00`00ooooo`1Tool3000Iool00`00oooo
o`05ool00liiooooo`1mool001Goo`03003ooooo06Koo`03003oo`0001Soo`03003ooooo00Goo`03
cWWooooo07coo`005Ooo00<00?ooool0Iooo00<00?oo00008?oo00?>NOooool0Nooo000Eool2001Y
ool01000ooooo`007ooo0/iiNooo000Eool00`00ooooo`1Yool2000Jool00`00ooooo`05ool00lii
ooooo`1hool001Goo`03003ooooo06_oo`04003oo`00000Gool20006ool00liiooooo`1gool001Go
o`03003ooooo06coo`03003ooooo01Woo`8000Goo`03cWWooooo07Koo`005Ooo00<00?ooool0KOoo
0P0000?oo`00ool06?oo00<00?ooool00ooo00?>NOooool0MOoo000Eool00`00ooooo`1_ool2000P
ool2cWUeool001Goo`8007;oo`<001ooo`03cWWooooo07;oo`005Ooo00<00?ooool0Looo00<00?oo
ool06?oo00<00?ooool00ooo00?>NOooool0LOoo000Eool00`00ooooo`1dool3000Hool20004ool0
0liiooooo`1`ool00003ool00000008000?oo`03003ooooo00;oo`@000Coo`03003ooooo07Goo`03
003ooooo01Woo`8000?oo`03cWWooooo06ooo`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<0
0?ooool0M_oo0`006_oo00D00?ooooooolii073oo`000ooo00<00?ooool02?oo00<00?ooool01?oo
0`00N?oo00<00?ooool07Ooo00?>NOooool0KOoo0003ool00`00ooooo`09ool00`00ooooo`03ool0
0`00ooooo`1iool00`00ool0000Mool2cWU]ool000?oo`03003ooooo00[oo`03003ooooo00;oo`03
003ooooo07[oo`8001_oo`05003oooooooo>N@1/ool00004ool000000009ool01000ooooo`001?oo
00<00?ooool0O?oo0`006Ooo0P000_oo00?>NOooool0JOoo0003ool00`00ooooo`08ool20005ool0
0`00ooooo`1mool00`00ooooo`0Jool01000oooooliiJ_oo000Eool00`00ooooo`1nool3000Jool0
1000oooooliiJOoo000Eool20021ool00`00ooooo`0Lool00liiooooo`1Vool001Goo`03003ooooo
087oo`<001coo`03cWWooooo06Goo`005Ooo00<00?ooool0Pooo00<00?ooool06ooo0/iiIOoo000E
ool00`00ooooo`24ool2000Jool01000oooooliiI?oo000Eool00`00ooooo`26ool00`00ool0000H
ool01000oooooliiHooo000Eool20028ool2000Iool200000ooocWWoo`1Qool001Goo`03003ooooo
08Woo`8001Woo`03003oolii067oo`005Ooo00<00?ooool0R_oo0P006ooo0/iiGooo000Eool00`00
ooooo`2;ool3000Kool00liiooooo`1Lool001Goo`03003ooooo08goo`03003ooooo01Soo`03003o
olii05goo`005Ooo00<00?ooool0S_oo0P006Ooo00<00?oocWT0G?oo000Eool2002@ool2000Iool2
00000liiooooo`1Iool001Goo`03003ooooo093oo`<001Woo`03003>NOoo05Woo`005Ooo00<00?oo
ool0T_oo00<00?ooool06?oo00<00<iiool0F?oo000Eool00`00ooooo`2Cool00`00ool0000Iool2
cWUGool00003ool00000008000?oo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo09Coo`80
01_oo`03cWWooooo05Coo`000ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0U_oo0P00
6Ooo00<00<iiool0E?oo0003ool00`00ooooo`06ool50004ool3002Gool2000Iool00`00cWWoo`1C
ool000?oo`03003ooooo00Koo`04003ooooo0005ool00`00ooooo`2Hool3000Hool00`00cWWoo`1B
ool000?oo`03003ooooo00Ooo`03003oo`0000Goo`03003ooooo09[oo`03003ooooo01Ooo`03003>
NOoo057oo`0000Coo`00000000[oo`8000Goo`03003ooooo09_oo`8001Soo`03003>NOoo053oo`00
0ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0W?oo0P006Ooo00?>NOooool0C_oo000E
ool2002Nool3000Hool00liiooooo`1=ool001Goo`03003ooooo09ooo`03003ooooo01Ooo`03003>
NOoo04coo`005Ooo00<00?ooool0X?oo00<00?oo00005ooo00<00<iiool0Booo000Eool00`00oooo
o`2Qool2000Hool00`00cWWoo`1:ool001Goo`03003ooooo0:?oo`8001Ooo`03003>NOoo04Woo`00
5Ooo00<00?ooool0Y?oo0P005ooo00<00<iiool0B?oo000Eool2002Vool3000Gool00liiooooo`16
ool001Goo`03003ooooo0:Ooo`03003ooooo01Koo`03cWWooooo04Goo`005Ooo00<00?ooool0Z?oo
0P005ooo00<00<iiool0A?oo000Eool00`00ooooo`2Yool2000Gool00`00cWWoo`13ool001Goo`03
003ooooo0:[oo`<001Koo`8004?oo`005Ooo0P00[Ooo00<00?ooool05_oo00<00?ooool0@?oo000E
ool00`00ooooo`2]ool2000Gool00`00ooooo`0oool001Goo`03003ooooo0:koo`8001Ooo`;>NCoo
o`005Ooo00<00?ooool0/?oo0P005ooo00?>NOooool0??oo00000ooo000000020003ool00`00oooo
o`03ool20005ool00`00ooooo`2aool00`00ooooo`0Fool00`00ooooo`0kool000?oo`03003ooooo
00Ooo`04003ooooo0004ool00`00ooooo`2bool3000Fool00`00ooooo`0jool000?oo`03003ooooo
00Ooo`04003ooooo0004ool3002dool00`00ooooo`0Eool00`00ooooo`0iool000?oo`03003ooooo
00Ooo`<000Goo`03003ooooo0;Goo`8001Koo`03003ooooo03Soo`000ooo00<00?ooool01ooo00<0
0?ooool01Ooo00<00?ooool0]_oo0P005_oo00<00?ooool0=ooo00001?oo000000002Ooo00<00?oo
ool01Ooo00<00?ooool0]ooo0`005Ooo0/ii=ooo0003ool00`00ooooo`08ool30004ool00`00oooo
o`2iool00`00ooooo`0Eool00liiooooo`0dool001Goo`03003ooooo0;[oo`8001Koo`03003ooooo
03?oo`005Ooo0P00_?oo0P005_oo00<00?ooool0<_oo000Eool00`00ooooo`2mool2000Eool00`00
ooooo`0aool001Goo`03003ooooo0;koo`03003ooooo01Coo`03003ooooo033oo`005Ooo00<00?oo
ool0_ooo0P005Ooo00<00?ooool0;ooo000Eool00`00ooooo`31ool00`00ooooo`0Cool2cWT_ool0
01Goo`800<?oo`03003ooooo01Coo`03cWWooooo02coo`005Ooo00<00?ooool0`ooo00<00?ooool0
5?oo00<00?ooool0:ooo000Eool00`00ooooo`34ool2000Eool00`00ooooo`0Zool001Goo`03003o
oooo0<Koo`03003ooooo01?oo`03003ooooo02Woo`005Ooo00<00?ooool0aooo00<00?ooool04ooo
00<00<iiool0:?oo000Eool00`00ooooo`38ool00`00ooooo`0Cool00`00cWWoo`0Wool001Goo`80
0<[oo`03003ooooo01Coo`03cWWooooo02Goo`005Ooo00<00?ooool0b_oo0P005Ooo00?>NOooool0
9?oo000Eool00`00ooooo`3<ool00`00ooooo`0Cool00lii003oo`0Sool00003ool00000008000?o
o`03003ooooo00?oo`8000Goo`03003ooooo0<goo`8001Goo`03003ooooo027oo`000ooo00<00?oo
ool01ooo00@00?ooool000Coo`03003ooooo0<koo`8001Coo`03003>NOoo027oo`000ooo00<00?oo
ool01ooo00@00?ooool000Coo`<00=3oo`03003ooooo01;oo`03003>NOoo023oo`000ooo00<00?oo
ool02?oo0P001Ooo00<00?ooool0dOoo00<00?ooool04_oo0P008?oo0003ool00`00ooooo`07ool0
1000ooooo`001?oo00<00?ooool0d_oo0P005?oo0/ii7_oo00001?oo000000002Ooo00@00?ooool0
00Coo`03003ooooo0=Coo`03003ooooo01?oo`03cWWooooo01_oo`000ooo00<00?ooool02?oo0P00
1Ooo00<00?ooool0eOoo00<00?ooool04ooo00<00?ooool06_oo000Eool00`00ooooo`3Fool00`00
ooooo`0Cool00`00ooooo`0Iool001Goo`800=Soo`8001Coo`03003ooooo01Soo`005Ooo00<00?oo
ool0f?oo00<00?ooool04ooo00<00?ooool05ooo000Eool00`00ooooo`3Iool2000Dool00`00oooo
o`0Fool001Goo`03003ooooo0=[oo`03003ooooo01?oo`;>NAKoo`005Ooo00<00?ooool0fooo0`00
5?oo00?>NOooool04ooo000Eool00`00ooooo`3Mool00`00ooooo`0Cool00liiooooo`0Bool001Go
o`800=ooo`8001?oo`03003>NOoo01;oo`005Ooo00<00?ooool0gooo0P004ooo00<00<iiool04Ooo
000Eool00`00ooooo`3Qool2000Bool2000Aool001Goo`03003ooooo0>;oo`03003ooooo01;oo`03
003ooooo00koo`005Ooo00<00?ooool0hooo0P004ooo00?>NOooool03Ooo000Eool2003Uool2000C
ool00liiooooo`0<ool001Goo`03003ooooo0>Koo`03003ooooo017oo`03cWWooooo00_oo`005Ooo
00<00?ooool0iooo00<00?ooool04Ooo0/ii2ooo000Eool00`00ooooo`3Xool2000Cool00liioooo
o`08ool000goo`@000Coo`03003ooooo0>Woo`8001?oo`03cWWooooo00Ooo`003Ooo00<00?ooool0
1Ooo00<00?ooool0jooo0P004_oo00?>NOooool01_oo000>ool00`00ooooo`04ool3003/ool00`00
ooooo`0Aool00liiooooo`05ool000ooo`03003ooooo0?ooo`ooo`004?oo00<00?ooool0oooo3_oo
000=ool01000ooooo`00oooo4?oo000>ool2003ooolAool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.165192, 0.480876, \
0.00771848, 0.00892053}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[x, {x, 0, 2}, PlotStyle \[Rule] {GrayLevel[ .8]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0147151 0.294302 [
[.2619 .00222 -9 -9 ]
[.2619 .00222 9 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -9 -9 ]
[.7381 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .16187 -18 -4.5 ]
[.01131 .16187 0 4.5 ]
[.01131 .30902 -6 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .45617 -18 -4.5 ]
[.01131 .45617 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(0.5)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(1.5)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(2)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .16187 m
.03006 .16187 L
s
[(0.5)] .01131 .16187 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1)] .01131 .30902 1 0 Mshowa
.02381 .45617 m
.03006 .45617 L
s
[(1.5)] .01131 .45617 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(2)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.8 g
.5 Mabswid
.02381 .01472 m
.06244 .03859 L
.10458 .06463 L
.14415 .08909 L
.18221 .11261 L
.22272 .13765 L
.26171 .16175 L
.30316 .18736 L
.34309 .21204 L
.3815 .23578 L
.42237 .26104 L
.46172 .28536 L
.49955 .30874 L
.53984 .33364 L
.57861 .3576 L
.61984 .38308 L
.65954 .40762 L
.69774 .43123 L
.73838 .45635 L
.77751 .48053 L
.81909 .50623 L
.85916 .53099 L
.89771 .55482 L
.93871 .58016 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool004ooo`8000Coo`03003ooooo00;oo`<003Ooo`@003Ooo`@000?oo`03003ooooo00;oo`<003So
o`@000Koo`00C_oo00@00?ooool000_oo`03003ooooo03Koo`03003ooooo03Soo`03003ooooo00[o
o`03003ooooo03Goo`03003ooooo00Ooo`00C_oo00@00?ooool000_oo`03003ooooo03Koo`03003o
oooo03Soo`03003ooooo00[oo`03003ooooo03Koo`03003ooooo00Koo`00C_oo00@00?ooool000So
o`<003Woo`03003ooooo03Soo`03003ooooo00Ooo`<003[oo`03003ooooo00Goo`00C_oo00@00?oo
ool000Soo`03003ooooo03Woo`03003ooooo03Soo`03003ooooo00Ooo`03003ooooo03_oo`03003o
oooo00Coo`00C_oo00@00?ooool000Soo`03003ooooo03Ooo`<003Soo`<000Woo`03003ooooo03So
o`04003ooooo0006ool001Goo`03003ooooo03Ooo`8000Woo`@003Soo`03003ooooo03Soo`03003o
oooo00Ooo`@003Soo`8000Ooo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`003ooo1P0000?>N@000000o`0020000Ooo000Eool00`00cWW>N@0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`05
ool001Goo`03003ooooo00;>NCcoo`03003ooooo03goo`03003ooooo03koo`03003ooooo03koo`03
003ooooo00Goo`005Ooo00<00?ooool00_oo00?>NOooool0oooo1?oo000Eool00`00ooooo`03ool2
cWWoool4ool001Goo`03003ooooo00Goo`;>NOooo`;oo`005Ooo00<00?ooool01ooo00?>NOooool0
o_oo000Eool00`00ooooo`08ool2cWWnool001Goo`8000_oo`03cWWooooo0?_oo`005Ooo00<00?oo
ool02ooo0/iinooo000Eool00`00ooooo`0=ool00liiooooo`3hool001Goo`03003ooooo00koo`;>
NOSoo`005Ooo00<00?ooool04?oo00?>NOooool0mOoo000Eool00`00ooooo`0Aool2cWWeool001Go
o`03003ooooo01?oo`03cWWooooo0?;oo`005Ooo00<00?ooool05?oo0/iil_oo000Eool2000Gool2
cWW`ool001Goo`03003ooooo01Soo`;>NNkoo`005Ooo00<00?ooool06_oo0/iik?oo000Eool00`00
ooooo`0Lool2cWWZool001Goo`03003ooooo01koo`03cWWooooo0>Ooo`005Ooo00<00?ooool07ooo
0/iiiooo000Eool00`00ooooo`0Qool00liiooooo`3Tool001Goo`03003ooooo02;oo`03cWWooooo
0>?oo`005Ooo0P009?oo0/iihooo000Eool00`00ooooo`0Uool00liiooooo`3Pool001Goo`03003o
oooo02Koo`;>NN3oo`005Ooo00<00?ooool0:?oo00?>NOooool0gOoo000Eool00`00ooooo`0Yool2
cWWMool001Goo`03003ooooo02_oo`;>NM_oo`005Ooo00<00?ooool0;Ooo0/iifOoo000Eool00`00
ooooo`0_ool2cWWGool001Goo`8003;oo`;>NMGoo`005Ooo00<00?ooool0<ooo00?>NOooool0d_oo
000Eool00`00ooooo`0dool2cWWBool001Goo`03003ooooo03Koo`03cWWooooo0<ooo`005Ooo00<0
0?ooool0=ooo0/iicooo000Eool00`00ooooo`0iool2cWW=ool000;oo`8000Coo`03003ooooo00;o
o`<000Goo`03003ooooo03_oo`03cWWooooo0<[oo`0000Goo`00ooooo`0000_oo`03003ooooo00;o
o`03003ooooo03coo`;>NL[oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`<003koo`03cWWo
oooo0<Ooo`0000Goo`00ooooo`0000Soo`<000Goo`03003ooooo03ooo`;>NLOoo`0000Goo`00oooo
o`0000Soo`03003ooooo00Goo`03003ooooo047oo`03cWWooooo0<Coo`0000Goo`00ooooo`0000So
o`03003ooooo00Goo`03003ooooo04;oo`;>NLCoo`000_oo0P002Ooo10001?oo00<00?ooool0A?oo
0/ii`_oo000Eool00`00ooooo`16ool00liiooooo`2oool001Goo`03003ooooo04Ooo`;>NKooo`00
5Ooo00<00?ooool0BOoo00?>NOooool0_?oo000Eool2001;ool2cWVlool001Goo`03003ooooo04co
o`03cWWooooo0;Woo`005Ooo00<00?ooool0COoo0/ii^Ooo000Eool00`00ooooo`1?ool2cWVgool0
01Goo`03003ooooo057oo`03cWWooooo0;Coo`005Ooo00<00?ooool0D_oo0/ii]?oo000Eool00`00
ooooo`1Dool00liiooooo`2aool001Goo`03003ooooo05Goo`;>NK7oo`005Ooo0P00F?oo0/ii[ooo
000Eool00`00ooooo`1Iool00liiooooo`2/ool001Goo`03003ooooo05[oo`;>NJcoo`005Ooo00<0
0?ooool0G?oo0/iiZ_oo000Eool00`00ooooo`1Nool00liiooooo`2Wool001Goo`03003ooooo05oo
o`;>NJOoo`005Ooo00<00?ooool0HOoo00?>NOooool0Y?oo000Eool00`00ooooo`1Rool2cWVTool0
01Goo`8006Goo`;>NJ;oo`005Ooo00<00?ooool0I_oo00?>NOooool0Wooo000Eool00`00ooooo`1W
ool2cWVOool001Goo`03003ooooo06Woo`03cWWooooo09coo`005Ooo00<00?ooool0J_oo0/iiW?oo
000Eool00`00ooooo`1/ool00liiooooo`2Iool001Goo`03003ooooo06goo`;>NIWoo`005Ooo00<0
0?ooool0Kooo0/iiUooo000Eool2001bool00liiooooo`2Dool001Goo`03003ooooo07;oo`;>NICo
o`005Ooo00<00?ooool0M?oo00?>NOooool0TOoo000Eool00`00ooooo`1eool2cWVAool001Goo`03
003ooooo07Ooo`;>NHooo`005Ooo00<00?ooool0NOoo00?>NOooool0S?oo000=ool40004ool00`00
ooooo`1jool2cWV<ool000ooo`03003ooooo00?oo`03003ooooo07coo`;>NH[oo`003ooo00<00?oo
ool00ooo0`00O_oo00?>NOooool0Qooo000?ool00`00ooooo`03ool00`00ooooo`1oool2cWV7ool0
00ooo`03003ooooo00?oo`03003ooooo087oo`03cWWooooo08Coo`003Ooo0`001Ooo00<00?ooool0
P_oo0/iiQ?oo000?ool00`00ooooo`03ool00`00ooooo`24ool2cWV2ool001Goo`03003ooooo08Ko
o`03cWWooooo07ooo`005Ooo00<00?ooool0Qooo0/iiOooo000Eool00`00ooooo`29ool00liioooo
o`1lool001Goo`8008_oo`;>NGcoo`005Ooo00<00?ooool0S?oo0/iiN_oo000Eool00`00ooooo`2>
ool2cWUhool001Goo`03003ooooo093oo`;>NGKoo`005Ooo00<00?ooool0T_oo0/iiM?oo000Eool0
0`00ooooo`2Dool00liiooooo`1aool001Goo`03003ooooo09Goo`;>NG7oo`005Ooo00<00?ooool0
Uooo00?>NOooool0K_oo000Eool2002Iool2cWU^ool001Goo`03003ooooo09[oo`;>NFcoo`005Ooo
00<00?ooool0W?oo00?>NOooool0JOoo000Eool00`00ooooo`2Mool2cWUYool001Goo`03003ooooo
09ooo`03cWWooooo06Koo`005Ooo00<00?ooool0X?oo0/iiI_oo000Eool00`00ooooo`2Rool00lii
ooooo`1Sool001Goo`03003ooooo0:?oo`;>NF?oo`005Ooo0P00Y_oo0/iiHOoo000Eool00`00oooo
o`2Wool00liiooooo`1Nool001Goo`03003ooooo0:Soo`;>NEkoo`005Ooo00<00?ooool0Z_oo00?>
NOooool0Fooo000Eool00`00ooooo`2[ool2cWUKool001Goo`03003ooooo0:goo`;>NEWoo`005Ooo
00<00?ooool0[ooo00?>NOooool0E_oo000Eool00`00ooooo`2`ool2cWUFool001Goo`800;?oo`;>
NECoo`005Ooo00<00?ooool0]?oo00?>NOooool0DOoo000Eool00`00ooooo`2eool2cWUAool001Go
o`03003ooooo0;Ooo`03cWWooooo04koo`005Ooo00<00?ooool0^?oo0/iiC_oo000Eool00`00oooo
o`2jool2cWU<ool00003ool00000008000?oo`03003ooooo00;oo`<000Goo`03003ooooo0;coo`03
cWWooooo04Woo`000ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0_Ooo0/iiBOoo0003
ool00`00ooooo`0:ool00`00ooooo`02ool3002oool00liiooooo`16ool000?oo`03003ooooo00Oo
o`<000Goo`03003ooooo0<3oo`;>NDKoo`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?oo
ool0`_oo00?>NOooool0@ooo00001?oo000000002Ooo00<00?ooool01Ooo00<00?ooool0`ooo0/ii
@ooo0003ool00`00ooooo`07ool40004ool00`00ooooo`35ool2cWU1ool001Goo`03003ooooo0<Oo
o`03cWWooooo03koo`005Ooo00<00?ooool0b?oo0/ii?_oo000Eool00`00ooooo`3:ool00liioooo
o`0kool001Goo`800<coo`;>NC_oo`005Ooo00<00?ooool0cOoo00?>NOooool0>?oo000Eool00`00
ooooo`3>ool2cWThool001Goo`03003ooooo0=3oo`;>NCKoo`005Ooo00<00?ooool0d_oo00?>NOoo
ool0<ooo000Eool00`00ooooo`3Cool2cWTcool001Goo`03003ooooo0=Goo`03cWWooooo033oo`00
5Ooo00<00?ooool0e_oo0/ii<?oo000Eool2003Iool2cWT^ool001Goo`03003ooooo0=[oo`;>NBco
o`005Ooo00<00?ooool0g?oo0/ii:_oo000Eool00`00ooooo`3Nool2cWTXool001Goo`03003ooooo
0>3oo`03cWWooooo02Goo`005Ooo00<00?ooool0hOoo0/ii9Ooo000Eool00`00ooooo`3Sool00lii
ooooo`0Rool001Goo`03003ooooo0>Coo`;>NB;oo`005Ooo0P00iooo0/ii8?oo000Eool00`00oooo
o`3Xool00liiooooo`0Mool001Goo`03003ooooo0>Woo`;>NAgoo`005Ooo00<00?ooool0jooo00?>
NOooool06_oo000Eool00`00ooooo`3/ool2cWTJool001Goo`03003ooooo0>koo`03cWWooooo01Oo
o`005Ooo00<00?ooool0kooo0/ii5ooo000Eool00`00ooooo`3aool2cWTEool001Goo`800?Coo`03
cWWooooo01;oo`005Ooo00<00?ooool0m?oo0/ii4_oo000Eool00`00ooooo`3fool00liiooooo`0?
ool001Goo`03003ooooo0?Ooo`;>N@ooo`005Ooo00<00?ooool0nOoo0/ii3Ooo000Eool00`00oooo
o`3kool00liiooooo`0:ool000goo`@000Coo`03003ooooo0?coo`;>N@[oo`003Ooo00<00?ooool0
1Ooo00<00?ooool0o_oo0/ii2?oo000>ool00`00ooooo`04ool3003oool1ool00liiooooo`05ool0
00ooo`03003ooooo00?oo`03003ooooo0?ooo`Woo`004?oo00<00?ooool00_oo00<00?ooool0oooo
2Ooo000=ool01000ooooo`001?oo00<00?ooool0oooo2Ooo000>ool20005ool00`00ooooo`3oool9
ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.165971, -0.15494, \
0.00772359, 0.012497}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<stablePi4.eps\>", %585]\)], "Input"],

Cell[BoxData[
    \("stablePi4.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omthree[IntegerPart[T*10^16]*10^\((\(-16\))\), 10^\((\(-5\))\), 
          10]}, {T, 0.01, 2, 1/20}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.6176340793990494`}, {0.060000000000000005`, 
        0.6219234245061466`}, {0.11000000000000001`, 
        0.6320828923254037`}, {0.16000000000000003`, 
        0.7556859403826365`}, {0.21000000000000002`, 
        0.7593089359375857`}, {0.26`, 0.7639633391005006`}, {0.31`, 
        0.7184803646230286`}, {0.36`, 0.7481075164815932`}, {0.41`, 
        0.7798494540348498`}, {0.45999999999999996`, 
        0.7939459728049775`}, {0.51`, 0.8484497520651897`}, {0.56`, 
        0.8847422848759972`}, {0.6100000000000001`, 
        0.9221198798392708`}, {0.6600000000000001`, 
        0.960439891314327`}, {0.7100000000000002`, 
        0.9995861492239321`}, {0.7600000000000002`, 
        1.039464287483336`}, {0.8100000000000003`, 
        1.079995433885675`}, {0.8600000000000003`, 
        1.1211134573907306`}, {0.9100000000000004`, 
        1.1627621797863483`}, {0.9600000000000004`, 
        1.204893376432847`}, {1.0100000000000005`, 
        1.2474652514879847`}, {1.0600000000000005`, 
        1.2904412658424804`}, {1.1100000000000005`, 
        1.3337892274125267`}, {1.1600000000000006`, 
        1.377480577667764`}, {1.2100000000000006`, 
        1.421489826086107`}, {1.2600000000000007`, 
        1.4657940970906858`}, {1.3100000000000007`, 
        1.510372763293748`}, {1.3600000000000008`, 
        1.5552071455763727`}, {1.4100000000000008`, 
        1.6002802654090997`}, {1.4600000000000009`, 
        1.6455766383881396`}, {1.510000000000001`, 
        1.691082100592294`}, {1.560000000000001`, 
        1.7367836613172793`}, {1.610000000000001`, 
        1.7826693772019135`}, {1.660000000000001`, 
        1.8287282438570855`}, {1.710000000000001`, 
        1.8749501019388253`}, {1.7600000000000011`, 
        1.9213255552401491`}, {1.8100000000000012`, 
        1.9678458988628087`}, {1.8600000000000012`, 
        2.0145030559064097`}, {1.9100000000000013`, 
        2.061289521405559`}, {1.9600000000000013`, 
        2.108198312475831`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.01`, 0.6176340793990494`}, {0.060000000000000005`, 
        0.6219234245061466`}, {0.11000000000000001`, 
        0.6320828923254037`}, {0.31`, 0.7184803646230286`}, {0.36`, 
        0.7481075164815932`}, {0.41`, 0.7798494540348498`}, {0.51`, 
        0.8484497520651897`}, {0.56`, 
        0.8847422848759972`}, {0.6100000000000001`, 
        0.9221198798392708`}, {0.6600000000000001`, 
        0.960439891314327`}, {0.7100000000000002`, 
        0.9995861492239321`}, {0.7600000000000002`, 
        1.039464287483336`}, {0.8100000000000003`, 
        1.079995433885675`}, {0.8600000000000003`, 
        1.1211134573907306`}, {0.9100000000000004`, 
        1.1627621797863483`}, {0.9600000000000004`, 
        1.204893376432847`}, {1.0100000000000005`, 
        1.2474652514879847`}, {1.0600000000000005`, 
        1.2904412658424804`}, {1.1100000000000005`, 
        1.3337892274125267`}, {1.1600000000000006`, 
        1.377480577667764`}, {1.2100000000000006`, 
        1.421489826086107`}, {1.2600000000000007`, 
        1.4657940970906858`}, {1.3100000000000007`, 
        1.510372763293748`}, {1.3600000000000008`, 
        1.5552071455763727`}, {1.4100000000000008`, 
        1.6002802654090997`}, {1.4600000000000009`, 
        1.6455766383881396`}, {1.510000000000001`, 
        1.691082100592294`}, {1.560000000000001`, 
        1.7367836613172793`}, {1.610000000000001`, 
        1.7826693772019135`}, {1.660000000000001`, 
        1.8287282438570855`}, {1.710000000000001`, 
        1.8749501019388253`}, {1.7600000000000011`, 
        1.9213255552401491`}, {1.8100000000000012`, 
        1.9678458988628087`}, {1.8600000000000012`, 
        2.0145030559064097`}, {1.9100000000000013`, 
        2.061289521405559`}, {1.9600000000000013`, 
        2.108198312475831`}}\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.6176340793990494`}, {0.060000000000000005`, 
        0.6219234245061466`}, {0.11000000000000001`, 
        0.6320828923254037`}, {0.31`, 0.7184803646230286`}, {0.36`, 
        0.7481075164815932`}, {0.41`, 0.7798494540348498`}, {0.51`, 
        0.8484497520651897`}, {0.56`, 
        0.8847422848759972`}, {0.6100000000000001`, 
        0.9221198798392708`}, {0.6600000000000001`, 
        0.960439891314327`}, {0.7100000000000002`, 
        0.9995861492239321`}, {0.7600000000000002`, 
        1.039464287483336`}, {0.8100000000000003`, 
        1.079995433885675`}, {0.8600000000000003`, 
        1.1211134573907306`}, {0.9100000000000004`, 
        1.1627621797863483`}, {0.9600000000000004`, 
        1.204893376432847`}, {1.0100000000000005`, 
        1.2474652514879847`}, {1.0600000000000005`, 
        1.2904412658424804`}, {1.1100000000000005`, 
        1.3337892274125267`}, {1.1600000000000006`, 
        1.377480577667764`}, {1.2100000000000006`, 
        1.421489826086107`}, {1.2600000000000007`, 
        1.4657940970906858`}, {1.3100000000000007`, 
        1.510372763293748`}, {1.3600000000000008`, 
        1.5552071455763727`}, {1.4100000000000008`, 
        1.6002802654090997`}, {1.4600000000000009`, 
        1.6455766383881396`}, {1.510000000000001`, 
        1.691082100592294`}, {1.560000000000001`, 
        1.7367836613172793`}, {1.610000000000001`, 
        1.7826693772019135`}, {1.660000000000001`, 
        1.8287282438570855`}, {1.710000000000001`, 
        1.8749501019388253`}, {1.7600000000000011`, 
        1.9213255552401491`}, {1.8100000000000012`, 
        1.9678458988628087`}, {1.8600000000000012`, 
        2.0145030559064097`}, {1.9100000000000013`, 
        2.061289521405559`}, {1.9600000000000013`, 
        2.108198312475831`}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \({0.01`, 0.6176340793990494`}, {0.060000000000000005`, 
      0.6219234245061466`}, {0.11000000000000001`, 
      0.6320828923254037`}, {0.31`, 0.7184803646230286`}, {0.36`, 
      0.7481075164815932`}, {0.41`, 0.7798494540348498`}, \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.001`, 0.7507332083329381`}, {0.051000000000000004`, 
        0.751234785897761`}, {0.101`, 
        0.7526983999163901`}, {0.15100000000000002`, 
        0.7551424823907917`}, {0.201`, 0.7585816171700839`}, {0.251`, 
        0.763051424667609`}, {0.301`, 0.7685846374439593`}, {0.351`, 
        0.7752374061515549`}, {0.351`, 0.7752362962013227`}, {0.4, 
        0.7829138917327014`}, {0.45099999999999996`, 
        0.7921646844892605`}, {0.51`, 0.8484497520651897`}, {0.56`, 
        0.8847422848759972`}, {0.6100000000000001`, 
        0.9221198798392708`}, {0.6600000000000001`, 
        0.960439891314327`}, {0.7100000000000002`, 
        0.9995861492239321`}, {0.7600000000000002`, 
        1.039464287483336`}, {0.8100000000000003`, 
        1.079995433885675`}, {0.8600000000000003`, 
        1.1211134573907306`}, {0.9100000000000004`, 
        1.1627621797863483`}, {0.9600000000000004`, 
        1.204893376432847`}, {1.0100000000000005`, 
        1.2474652514879847`}, {1.0600000000000005`, 
        1.2904412658424804`}, {1.1100000000000005`, 
        1.3337892274125267`}, {1.1600000000000006`, 
        1.377480577667764`}, {1.2100000000000006`, 
        1.421489826086107`}, {1.2600000000000007`, 
        1.4657940970906858`}, {1.3100000000000007`, 
        1.510372763293748`}, {1.3600000000000008`, 
        1.5552071455763727`}, {1.4100000000000008`, 
        1.6002802654090997`}, {1.4600000000000009`, 
        1.6455766383881396`}, {1.510000000000001`, 
        1.691082100592294`}, {1.560000000000001`, 
        1.7367836613172793`}, {1.610000000000001`, 
        1.7826693772019135`}, {1.660000000000001`, 
        1.8287282438570855`}, {1.710000000000001`, 
        1.8749501019388253`}, {1.7600000000000011`, 
        1.9213255552401491`}, {1.8100000000000012`, 
        1.9678458988628087`}, {1.8600000000000012`, 
        2.0145030559064097`}, {1.9100000000000013`, 
        2.061289521405559`}, {1.9600000000000013`, 
        2.108198312475831`}}\)], "Input"],

Cell[BoxData[
    \({{0.001`, 0.7507332083329381`}, {0.051000000000000004`, 
        0.751234785897761`}, {0.101`, 
        0.7526983999163901`}, {0.15100000000000002`, 
        0.7551424823907917`}, {0.201`, 0.7585816171700839`}, {0.251`, 
        0.763051424667609`}, {0.301`, 0.7685846374439593`}, {0.351`, 
        0.7752374061515549`}, {0.351`, 0.7752362962013227`}, {0.4`, 
        0.7829138917327014`}, {0.45099999999999996`, 
        0.7921646844892605`}, {0.51`, 0.8484497520651897`}, {0.56`, 
        0.8847422848759972`}, {0.6100000000000001`, 
        0.9221198798392708`}, {0.6600000000000001`, 
        0.960439891314327`}, {0.7100000000000002`, 
        0.9995861492239321`}, {0.7600000000000002`, 
        1.039464287483336`}, {0.8100000000000003`, 
        1.079995433885675`}, {0.8600000000000003`, 
        1.1211134573907306`}, {0.9100000000000004`, 
        1.1627621797863483`}, {0.9600000000000004`, 
        1.204893376432847`}, {1.0100000000000005`, 
        1.2474652514879847`}, {1.0600000000000005`, 
        1.2904412658424804`}, {1.1100000000000005`, 
        1.3337892274125267`}, {1.1600000000000006`, 
        1.377480577667764`}, {1.2100000000000006`, 
        1.421489826086107`}, {1.2600000000000007`, 
        1.4657940970906858`}, {1.3100000000000007`, 
        1.510372763293748`}, {1.3600000000000008`, 
        1.5552071455763727`}, {1.4100000000000008`, 
        1.6002802654090997`}, {1.4600000000000009`, 
        1.6455766383881396`}, {1.510000000000001`, 
        1.691082100592294`}, {1.560000000000001`, 
        1.7367836613172793`}, {1.610000000000001`, 
        1.7826693772019135`}, {1.660000000000001`, 
        1.8287282438570855`}, {1.710000000000001`, 
        1.8749501019388253`}, {1.7600000000000011`, 
        1.9213255552401491`}, {1.8100000000000012`, 
        1.9678458988628087`}, {1.8600000000000012`, 
        2.0145030559064097`}, {1.9100000000000013`, 
        2.061289521405559`}, {1.9600000000000013`, 
        2.108198312475831`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.485909 -0.310807 0.433605 [
[.26676 .1103 -9 -9 ]
[.26676 .1103 9 0 ]
[.50972 .1103 -3 -9 ]
[.50972 .1103 3 0 ]
[.75267 .1103 -9 -9 ]
[.75267 .1103 9 0 ]
[.99563 .1103 -3 -9 ]
[.99563 .1103 3 0 ]
[.01131 .03608 -18 -4.5 ]
[.01131 .03608 0 4.5 ]
[.01131 .20952 -18 -4.5 ]
[.01131 .20952 0 4.5 ]
[.01131 .29624 -18 -4.5 ]
[.01131 .29624 0 4.5 ]
[.01131 .38296 -18 -4.5 ]
[.01131 .38296 0 4.5 ]
[.01131 .46968 -18 -4.5 ]
[.01131 .46968 0 4.5 ]
[.01131 .5564 -6 -4.5 ]
[.01131 .5564 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26676 .1228 m
.26676 .12905 L
s
[(0.5)] .26676 .1103 0 1 Mshowa
.50972 .1228 m
.50972 .12905 L
s
[(1)] .50972 .1103 0 1 Mshowa
.75267 .1228 m
.75267 .12905 L
s
[(1.5)] .75267 .1103 0 1 Mshowa
.99563 .1228 m
.99563 .12905 L
s
[(2)] .99563 .1103 0 1 Mshowa
.125 Mabswid
.0724 .1228 m
.0724 .12655 L
s
.12099 .1228 m
.12099 .12655 L
s
.16958 .1228 m
.16958 .12655 L
s
.21817 .1228 m
.21817 .12655 L
s
.31535 .1228 m
.31535 .12655 L
s
.36395 .1228 m
.36395 .12655 L
s
.41254 .1228 m
.41254 .12655 L
s
.46113 .1228 m
.46113 .12655 L
s
.55831 .1228 m
.55831 .12655 L
s
.6069 .1228 m
.6069 .12655 L
s
.65549 .1228 m
.65549 .12655 L
s
.70408 .1228 m
.70408 .12655 L
s
.80126 .1228 m
.80126 .12655 L
s
.84985 .1228 m
.84985 .12655 L
s
.89845 .1228 m
.89845 .12655 L
s
.94704 .1228 m
.94704 .12655 L
s
.25 Mabswid
0 .1228 m
1 .1228 L
s
.02381 .03608 m
.03006 .03608 L
s
[(0.8)] .01131 .03608 1 0 Mshowa
.02381 .20952 m
.03006 .20952 L
s
[(1.2)] .01131 .20952 1 0 Mshowa
.02381 .29624 m
.03006 .29624 L
s
[(1.4)] .01131 .29624 1 0 Mshowa
.02381 .38296 m
.03006 .38296 L
s
[(1.6)] .01131 .38296 1 0 Mshowa
.02381 .46968 m
.03006 .46968 L
s
[(1.8)] .01131 .46968 1 0 Mshowa
.02381 .5564 m
.03006 .5564 L
s
[(2)] .01131 .5564 1 0 Mshowa
.125 Mabswid
.02381 .05776 m
.02756 .05776 L
s
.02381 .07944 m
.02756 .07944 L
s
.02381 .10112 m
.02756 .10112 L
s
.02381 .14448 m
.02756 .14448 L
s
.02381 .16616 m
.02756 .16616 L
s
.02381 .18784 m
.02756 .18784 L
s
.02381 .2312 m
.02756 .2312 L
s
.02381 .25288 m
.02756 .25288 L
s
.02381 .27456 m
.02756 .27456 L
s
.02381 .31792 m
.02756 .31792 L
s
.02381 .3396 m
.02756 .3396 L
s
.02381 .36128 m
.02756 .36128 L
s
.02381 .40464 m
.02756 .40464 L
s
.02381 .42632 m
.02756 .42632 L
s
.02381 .448 m
.02756 .448 L
s
.02381 .49136 m
.02756 .49136 L
s
.02381 .51304 m
.02756 .51304 L
s
.02381 .53472 m
.02756 .53472 L
s
.02381 .0144 m
.02756 .0144 L
s
.02381 .57808 m
.02756 .57808 L
s
.02381 .59976 m
.02756 .59976 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.0243 .01472 m
.04859 .01493 L
.07289 .01557 L
.09718 .01663 L
.12148 .01812 L
.14577 .02006 L
.17007 .02246 L
.19436 .02534 L
.19436 .02534 L
.21817 .02867 L
.24295 .03268 L
.27162 .05709 L
.29592 .07282 L
.32021 .08903 L
.34451 .10564 L
.3688 .12262 L
.3931 .13991 L
.4174 .15748 L
.44169 .17531 L
.46599 .19337 L
.49028 .21164 L
.51458 .2301 L
.53887 .24874 L
.56317 .26753 L
.58746 .28648 L
.61176 .30556 L
.63605 .32477 L
.66035 .3441 L
.68465 .36354 L
.70894 .38308 L
.73324 .40272 L
.75753 .42246 L
.78183 .44227 L
.80612 .46217 L
.83042 .48214 L
.85471 .50218 L
.87901 .52229 L
.9033 .54246 L
.9276 .56269 L
.9519 .58298 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo7P00kOoo000Eool00`00
ooooo`0Kool6003Wool001Goo`03003ooooo027oo`d00=[oo`000_oo0P001?oo00<00?ooool00ooo
0P001Ooo00<00?ooool0;_oo1`00dooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003o
oooo03Goo`@00<ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3000iool00`00ooooo`3<
ool00005ool00?ooool00009ool20005ool00`00ooooo`0jool00`00ooooo`3;ool00005ool00?oo
ool00008ool01000ooooo`001?oo00<00?ooool0>ooo00<00?ooool0b_oo00001Ooo003ooooo0000
2?oo00@00?ooool000Coo`03003ooooo03coo`800<[oo`000_oo0P002_oo0P001Ooo00<00?ooool0
?_oo00<00?ooool0aooo000Eool00`00ooooo`0oool00`00ooooo`36ool001Goo`80047oo`03003o
oooo0<Goo`005Ooo00<00?ooool0@Ooo0P00aOoo000Eool00`00ooooo`13ool00`00ooooo`32ool0
01Goo`03003ooooo04Coo`800<;oo`005Ooo00<00?ooool0A_oo00<00?ooool0_ooo000Eool00`00
ooooo`17ool2002oool001Goo`8004[oo`03003ooooo0;coo`005Ooo00<00?ooool0>?oo0P001?oo
00<00?ooool00_oo0`001?oo00<00?ooool0<Ooo1000>?oo10000ooo00<00?ooool00_oo0`00>Ooo
10000_oo000Eool00`00ooooo`0gool01000ooooo`002ooo00<00?ooool00_oo0P00<ooo00<00?oo
ool0>Ooo00<00?ooool02_oo00<00?ooool0=_oo00<00?ooool00ooo000Eool00`00ooooo`0gool0
1000ooooo`002ooo00<00?ooool01?oo00<00?ooool0<?oo00<00?ooool0>Ooo00<00?ooool02_oo
00<00?ooool0=ooo00<00?ooool00_oo000Eool00`00ooooo`0gool01000ooooo`002?oo0`002?oo
0P00<?oo00<00?ooool0>Ooo00<00?ooool01ooo0`00>ooo00<00?ooool00Ooo000Eool00`00oooo
o`0gool01000ooooo`002?oo00<00?ooool02_oo0P00;_oo00<00?ooool0>Ooo00<00?ooool01ooo
00<00?ooool0??oo0@000Ooo0Ooo000Eool2000hool01000ooooo`002?oo00<00?ooool03?oo0P00
:_oo0`00>Ooo0`002Ooo00<00?ooool0>Ooo00@00?ooool000;oo`005Ooo00<00?ooool0>?oo0P00
2Ooo10003Ooo00<00?ooool0:Ooo00<00?ooool0>Ooo00<00?ooool01ooo1000>Ooo0P000ooo000E
ool00`00ooooo`1Eool00`00ooooo`2`ool001Goo`03003ooooo05Koo`03003ooooo0:ooo`005Ooo
00<00?ooool0Eooo0P00[ooo000Eool00`00ooooo`1Iool00`00ooooo`2/ool000koool0014000;o
o`005Ooo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?ooool02_oo00<00?oo
ool02ooo00<00?ooool02_oo00<00?ooool02_oo00@00?oo000000Woo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003o
oooo00_oo`03003ooooo007oo`005Ooo00<00?ooool0GOoo0P00ZOoo000Eool00`00ooooo`1Oool2
002Wool001Goo`03003ooooo067oo`03003ooooo0:Coo`005Ooo00<00?ooool0H_oo00<00?ooool0
Xooo000Eool2001Tool00`00ooooo`2Rool001Goo`03003ooooo06Coo`800:;oo`005Ooo00<00?oo
ool0I_oo00<00?ooool0Wooo000Eool00`00ooooo`1Wool00`00ooooo`2Nool001Goo`03003ooooo
06Soo`8009koo`005Ooo0P00Jooo00<00?ooool0Vooo000Eool00`00ooooo`1[ool00`00ooooo`2J
ool001Goo`03003ooooo06coo`8009[oo`005Ooo00<00?ooool0K_oo00<00?ooool0Uooo000Eool0
0`00ooooo`1_ool00`00ooooo`2Fool001Goo`03003ooooo073oo`03003ooooo09Goo`005Ooo0P00
L_oo0P00UOoo000Eool00`00ooooo`1cool00`00ooooo`2Bool001Goo`03003ooooo07Coo`03003o
oooo097oo`005Ooo00<00?ooool0MOoo0P00TOoo00000ooo000000020003ool00`00ooooo`02ool4
0004ool00`00ooooo`1gool00`00ooooo`2>ool000?oo`03003ooooo00Ooo`03003ooooo00Goo`03
003ooooo07Soo`03003ooooo08goo`000ooo00<00?ooool02?oo00<00?ooool01?oo0`00NOoo0P00
SOoo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`1kool00`00ooooo`2:ool000?o
o`03003ooooo00[oo`03003ooooo00;oo`03003ooooo07coo`8008[oo`0000Coo`00000000Woo`04
003ooooo0004ool00`00ooooo`1nool00`00ooooo`27ool000?oo`03003ooooo00Soo`8000Goo`03
003ooooo07ooo`03003ooooo08Koo`005Ooo00<00?ooool0P?oo0P00Q_oo000Eool20023ool00`00
ooooo`23ool001Goo`03003ooooo08?oo`03003ooooo08;oo`005Ooo00<00?ooool0Q?oo00<00?oo
ool0POoo000Eool00`00ooooo`25ool20021ool001Goo`03003ooooo08Ooo`03003ooooo07koo`00
5Ooo00<00?ooool0R?oo00<00?ooool0OOoo000Eool2002:ool2001mool001Goo`03003ooooo08_o
o`03003ooooo07[oo`005Ooo00<00?ooool0S?oo00<00?ooool0NOoo000Eool00`00ooooo`2=ool2
001iool001Goo`03003ooooo08ooo`03003ooooo07Koo`005Ooo00<00?ooool0T?oo00<00?ooool0
MOoo000Eool2002Bool00`00ooooo`1dool001Goo`03003ooooo09;oo`8007Coo`005Ooo00<00?oo
ool0U?oo00<00?ooool0LOoo000Eool00`00ooooo`2Eool00`00ooooo`1`ool00003ool000000080
00?oo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo09Koo`80073oo`000ooo00<00?ooool0
2Ooo00<00?ooool00ooo00<00?ooool0V?oo00<00?ooool0KOoo0003ool00`00ooooo`06ool50004
ool3002Iool00`00ooooo`1/ool000?oo`03003ooooo00Koo`04003ooooo0005ool00`00ooooo`2J
ool2001/ool000?oo`03003ooooo00Ooo`03003oo`0000Goo`03003ooooo09coo`03003ooooo06Wo
o`0000Coo`00000000[oo`8000Goo`03003ooooo09goo`03003ooooo06Soo`000ooo00<00?ooool0
2Ooo00<00?ooool00ooo00<00?ooool0W_oo00<00?ooool0Iooo000Eool2002Pool2001Wool001Go
o`03003ooooo0:7oo`03003ooooo06Coo`005Ooo00<00?ooool0X_oo00<00?ooool0Hooo000Eool0
0`00ooooo`2Sool00`00ooooo`1Rool001Goo`03003ooooo0:Coo`03003ooooo067oo`005Ooo00<0
0?ooool0YOoo0P00HOoo000Eool2002Xool00`00ooooo`1Nool001Goo`03003ooooo0:Soo`03003o
oooo05goo`005Ooo00<00?ooool0ZOoo00<00?ooool0G?oo000Eool00`00ooooo`2Zool00`00oooo
o`1Kool001Goo`03003ooooo0:_oo`03003ooooo05[oo`005Ooo00<00?ooool0[?oo0P00F_oo000E
ool2002_ool00`00ooooo`1Gool001Goo`03003ooooo0:ooo`03003ooooo05Koo`005Ooo00<00?oo
ool0/?oo0P00E_oo000Eool00`00ooooo`2bool00`00ooooo`1Cool00003ool00000008000?oo`03
003ooooo00?oo`8000Goo`03003ooooo0;?oo`03003ooooo05;oo`000ooo00<00?ooool01ooo00@0
0?ooool000Coo`03003ooooo0;Coo`8005;oo`000ooo00<00?ooool01ooo00@00?ooool000Coo`<0
0;Koo`03003ooooo04ooo`000ooo00<00?ooool01ooo0`001Ooo00<00?ooool0]ooo0P00Cooo0003
ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`2iool00`00ooooo`1<ool00004ool00000
0009ool00`00ooooo`05ool00`00ooooo`2jool00`00ooooo`1;ool000?oo`03003ooooo00Soo`<0
00Coo`03003ooooo0;_oo`8004_oo`005Ooo00<00?ooool0_Ooo00<00?ooool0B?oo000Eool2002o
ool00`00ooooo`17ool001Goo`03003ooooo0;ooo`03003ooooo04Koo`005Ooo00<00?ooool0`?oo
00<00?ooool0AOoo000Eool00`00ooooo`31ool00`00ooooo`14ool001Goo`03003ooooo0<;oo`03
003ooooo04?oo`005Ooo00<00?ooool0`ooo00<00?ooool0@_oo000Eool20035ool20012ool001Go
o`03003ooooo0<Koo`03003ooooo03ooo`005Ooo00<00?ooool0aooo00<00?ooool0?_oo000Eool0
0`00ooooo`38ool2000nool001Goo`03003ooooo0<[oo`03003ooooo03_oo`005Ooo00<00?ooool0
booo00<00?ooool0>_oo000Eool2003=ool00`00ooooo`0iool001Goo`03003ooooo0<goo`8003Wo
o`005Ooo00<00?ooool0cooo00<00?ooool0=_oo00000ooo000000020003ool00`00ooooo`03ool2
0005ool00`00ooooo`3@ool00`00ooooo`0eool000?oo`03003ooooo00Ooo`04003ooooo0004ool0
0`00ooooo`3Aool00`00ooooo`0dool000?oo`03003ooooo00Ooo`04003ooooo0004ool3003Bool0
0`00ooooo`0cool000?oo`03003ooooo00Soo`8000Goo`03003ooooo0=?oo`8003?oo`000ooo00<0
0?ooool01ooo00@00?ooool000Coo`03003ooooo0=Goo`03003ooooo033oo`0000Coo`00000000Wo
o`04003ooooo0004ool00`00ooooo`3Fool00`00ooooo`0_ool000?oo`03003ooooo00Soo`8000Go
o`03003ooooo0=Ooo`03003ooooo02koo`005Ooo00<00?ooool0f?oo00<00?ooool0;Ooo000Eool2
003Jool00`00ooooo`0/ool001Goo`03003ooooo0=[oo`8002coo`005Ooo00<00?ooool0g?oo00<0
0?ooool0:Ooo000Eool00`00ooooo`3Mool00`00ooooo`0Xool001Goo`03003ooooo0=koo`03003o
oooo02Ooo`005Ooo00<00?ooool0gooo00<00?ooool09_oo000Eool2003Qool2000Vool001Goo`03
003ooooo0>;oo`03003ooooo02?oo`005Ooo00<00?ooool0hooo00<00?ooool08_oo000Eool00`00
ooooo`3Tool00`00ooooo`0Qool001Goo`03003ooooo0>Goo`80027oo`005Ooo00<00?ooool0iooo
00<00?ooool07_oo000Eool2003Yool00`00ooooo`0Mool001Goo`03003ooooo0>Woo`8001goo`00
5Ooo00<00?ooool0jooo00<00?ooool06_oo000Eool00`00ooooo`3/ool00`00ooooo`0Iool000go
o`@000Coo`03003ooooo0>goo`03003ooooo01Soo`003Ooo00<00?ooool01Ooo00<00?ooool0k_oo
00<00?ooool05ooo000>ool00`00ooooo`04ool3003_ool00`00ooooo`0Fool000ooo`03003ooooo
00?oo`03003ooooo0?3oo`03003ooooo01Goo`004?oo00<00?ooool00_oo00<00?ooool0lOoo00<0
0?ooool05?oo000=ool01000ooooo`001?oo00<00?ooool0l_oo0P005?oo000>ool20005ool00`00
ooooo`3dool00`00ooooo`0Aool001Goo`03003ooooo0?Goo`03003ooooo013oo`005Ooo0P00mooo
0P004?oo000Eool00`00ooooo`3hool00`00ooooo`0=ool001Goo`03003ooooo0?Woo`03003ooooo
00coo`005Ooo00<00?ooool0n_oo00<00?ooool02ooo000Eool00`00ooooo`3kool00`00ooooo`0:
ool001Goo`03003ooooo0?coo`03003ooooo00Woo`005Ooo0P00o_oo00<00?ooool02?oo000Eool0
0`00ooooo`3nool00`00ooooo`07ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo
2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo003ooolQool0
0?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.162815, 0.674449, \
0.0076156, 0.00853424}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%97, %99, %101, %91, PlotRange \[Rule] {0.5, 2}, 
      Frame \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.206011 0.412023 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.2619 -0.0125 -9 -9 ]
[.2619 -0.0125 9 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -9 -9 ]
[.7381 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .0412 -18 -4.5 ]
[-0.0125 .0412 0 4.5 ]
[-0.0125 .12361 -18 -4.5 ]
[-0.0125 .12361 0 4.5 ]
[-0.0125 .20601 -6 -4.5 ]
[-0.0125 .20601 0 4.5 ]
[-0.0125 .28842 -18 -4.5 ]
[-0.0125 .28842 0 4.5 ]
[-0.0125 .37082 -18 -4.5 ]
[-0.0125 .37082 0 4.5 ]
[-0.0125 .45322 -18 -4.5 ]
[-0.0125 .45322 0 4.5 ]
[-0.0125 .53563 -18 -4.5 ]
[-0.0125 .53563 0 4.5 ]
[-0.0125 .61803 -6 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.2619 0 m
.2619 .00625 L
s
[(0.5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(1)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(1.5)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(2)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0412 m
.00625 .0412 L
s
[(0.6)] -0.0125 .0412 1 0 Mshowa
0 .12361 m
.00625 .12361 L
s
[(0.8)] -0.0125 .12361 1 0 Mshowa
0 .20601 m
.00625 .20601 L
s
[(1)] -0.0125 .20601 1 0 Mshowa
0 .28842 m
.00625 .28842 L
s
[(1.2)] -0.0125 .28842 1 0 Mshowa
0 .37082 m
.00625 .37082 L
s
[(1.4)] -0.0125 .37082 1 0 Mshowa
0 .45322 m
.00625 .45322 L
s
[(1.6)] -0.0125 .45322 1 0 Mshowa
0 .53563 m
.00625 .53563 L
s
[(1.8)] -0.0125 .53563 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(2)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .10301 m
.00375 .10301 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .30902 m
.00375 .30902 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .39142 m
.00375 .39142 L
s
0 .41202 m
.00375 .41202 L
s
0 .43262 m
.00375 .43262 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .51503 m
.00375 .51503 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
0 .0206 m
.00375 .0206 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.2619 .61178 m
.2619 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.7381 .61178 m
.7381 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0412 m
1 .0412 L
s
.99375 .12361 m
1 .12361 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .28842 m
1 .28842 L
s
.99375 .37082 m
1 .37082 L
s
.99375 .45322 m
1 .45322 L
s
.99375 .53563 m
1 .53563 L
s
.125 Mabswid
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .10301 m
1 .10301 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .30902 m
1 .30902 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .41202 m
1 .41202 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .51503 m
1 .51503 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.99625 .0206 m
1 .0206 L
s
.99625 0 m
1 0 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .002 .01 ] 0 setdash
.02429 .04842 m
.07143 .0534 L
.09524 .05936 L
.11905 .06726 L
.14286 .0768 L
.16667 .0877 L
.19048 .09971 L
.21429 .11264 L
.2381 .12632 L
.2619 .14064 L
.28571 .15549 L
.30952 .17081 L
.33333 .18653 L
.35714 .20259 L
.38095 .21896 L
.40476 .23561 L
.42857 .25251 L
.45238 .26962 L
.47619 .28695 L
.5 .30445 L
.52381 .32213 L
.54762 .33996 L
.57143 .35793 L
.59524 .37604 L
.61905 .39427 L
.64286 .41261 L
.66667 .43107 L
.69048 .44962 L
.71429 .46826 L
.7381 .487 L
.7619 .50581 L
.78571 .5247 L
.80952 .54367 L
.83333 .5627 L
.85714 .58179 L
.88095 .60095 L
s
.88095 .60095 m
.90213 .61803 L
s
[ .02 .02 ] 0 setdash
.02857 .04847 m
.05238 .05024 L
.07619 .05442 L
.17143 .09002 L
.19524 .10223 L
.21905 .1153 L
.26238 .12469 L
.28619 .12961 L
.31 .1352 L
.33381 .14153 L
.35762 .14871 L
.38143 .15684 L
.40524 .16604 L
.42905 .17648 L
.45286 .18831 L
.47667 .20167 L
.50048 .21664 L
.52429 .23317 L
.5481 .25107 L
.5719 .27004 L
.59571 .28972 L
.61952 .30985 L
.64333 .33022 L
.66714 .35071 L
.69095 .37127 L
.71476 .39185 L
.73857 .41244 L
.76238 .43304 L
.78619 .45364 L
.81 .47424 L
.83381 .49484 L
.85762 .51544 L
.88143 .53604 L
.90524 .55664 L
.92905 .57724 L
.95286 .59784 L
s
[ ] 0 setdash
.02429 .10331 m
.0481 .10351 L
.0719 .10412 L
.09571 .10512 L
.11952 .10654 L
.14333 .10838 L
.16714 .11066 L
.19095 .1134 L
.19095 .1134 L
.21429 .11657 L
.23857 .12038 L
.26667 .14357 L
.29048 .15852 L
.31429 .17392 L
.3381 .18971 L
.3619 .20584 L
.38571 .22227 L
.40952 .23897 L
.43333 .25591 L
.45714 .27307 L
.48095 .29043 L
.50476 .30797 L
.52857 .32568 L
.55238 .34354 L
.57619 .36154 L
.6 .37967 L
.62381 .39793 L
.64762 .4163 L
.67143 .43477 L
.69524 .45334 L
.71905 .472 L
.74286 .49075 L
.76667 .50958 L
.79048 .52849 L
.81429 .54747 L
.8381 .56651 L
.8619 .58562 L
.88571 .60479 L
s
.88571 .60479 m
.90212 .61803 L
s
.8 g
.2619 0 m
.30316 .03569 L
.34309 .07024 L
.3815 .10348 L
.42237 .13884 L
.46172 .17289 L
.49955 .20562 L
.53984 .24048 L
.57861 .27403 L
.61984 .3097 L
.65954 .34406 L
.69774 .3771 L
.73838 .41227 L
.77751 .44613 L
.81909 .48211 L
.85916 .51677 L
.89771 .55013 L
.93871 .58561 L
.97619 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001goo`8003Koo`8000Coo`03003ooooo00;oo`<003Goo`@003?oo`@000?oo`03003ooooo00;o
o`<003Goo`@000Woo`007?oo00@00?ooool003Coo`04003ooooo000;ool00`00ooooo`0dool00`00
ooooo`0dool00`00ooooo`0:ool00`00ooooo`0bool00`00ooooo`0:ool001coo`04003ooooo000d
ool01000ooooo`002ooo00<00?ooool0=?oo00<00?ooool0=?oo00<00?ooool02_oo00<00?ooool0
<ooo00<00?ooool02Ooo000Lool01000ooooo`00=?oo00@00?ooool000Soo`<003Ooo`03003ooooo
03Coo`03003ooooo00Ooo`<003Ooo`03003ooooo00Soo`007?oo00@00?ooool003Coo`04003ooooo
0008ool00`00ooooo`0gool00`00ooooo`0dool00`00ooooo`07ool00`00ooooo`0hool00`00oooo
o`07ool001coo`04003ooooo000dool01000ooooo`002?oo00<00?ooool0=Ooo0`00=?oo0`002Ooo
00<00?ooool0=Ooo00@00?ooool000Woo`007Ooo0P00=_oo0P002Ooo1000=_oo00<00?ooool0=?oo
00<00?ooool01ooo1000=Ooo0P002_oo003ooolQool00?ooob7oo`00oooo8Ooo000Hoom400000lii
0000002m0004ool001Soo`03003ooooo00?oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo
00Woo`03003ooooo00[oo`03003ooooo00Woo`03003>NOoo00Woo`03003ooooo00[oo`03003ooooo
00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo
00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo
00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00?oo`03003ooooo00;oo`006?oo00<0
0?ooool0@ooo0/ii^ooo00<00?ooool00_oo000Hool00`00ooooo`15ool00liiooooo`2hool00`00
ooooo`02ool001Soo`03003ooooo04Koo`03cWWooooo0;Ooo`03003ooooo00;oo`006?oo0P00B?oo
00?>NOooool0]Ooo0P001?oo000Hool00`00ooooo`18ool00liiooooo`2eool00`00ooooo`02ool0
01Soo`03003ooooo04Woo`;>NKGoo`03003ooooo00;oo`001Ooo0P001?oo00<00?ooool00ooo0P00
1Ooo00<00?ooool0Booo00?>NOooool0/_oo00<00?ooool00_oo0004ool01000ooooo`002?oo00@0
0?ooool000Coo`03003ooooo04coo`03cWWooooo0;7oo`03003ooooo00;oo`001?oo00@00?ooool0
00Soo`04003ooooo0004ool3001=ool00liiooooo`2_ool20004ool000Coo`04003ooooo0008ool3
0005ool00`00ooooo`1>ool00liiooooo`2_ool00`00ooooo`02ool000Coo`04003ooooo0008ool0
0`00ooooo`05ool00`00ooooo`03ool40018ool00liiooooo`2^ool00`00ooooo`02ool000Coo`04
003ooooo0008ool00`00ooooo`05ool00`00ooooo`06ool50002ool00`00ooooo`10ool00liioooo
o`2]ool00`00ooooo`02ool000Goo`8000[oo`<000Coo`03003ooooo00ooo`D003goo`;>NJgoo`03
003ooooo00;oo`006?oo00<00?ooool04ooo0P0000?oo`00ool0>ooo00?>NOooool0Z_oo00<00?oo
ool00_oo000Hool2000Jool00`00ooooo`0hool00liiooooo`2Xool20004ool001Soo`03003ooooo
01Soo`8000;oo`03003ooooo03Koo`03cWWooooo0:Soo`03003ooooo00;oo`006?oo00<00?ooool0
6_oo0P000_oo00<00?ooool0=Ooo00?>NOooool0Yooo00<00?ooool00_oo000Hool00`00ooooo`0L
ool200000ooo003oo`0fool00liiooooo`2Vool00`00ooooo`02ool001Soo`03003ooooo027oo`<0
03Coo`03cWWooooo0:Goo`03003ooooo00;oo`006?oo0P009Ooo0P00<ooo00?>NOooool0Xooo0P00
1?oo000Hool00`00ooooo`0Vool2000bool00liiooooo`2Sool00`00ooooo`02ool001Soo`03003o
oooo02[oo`03003ooooo02koo`;>NJ?oo`03003ooooo00;oo`006?oo00<00?ooool0:_oo00<00?oo
0000<?oo00?>NOooool0X?oo00<00?ooool00_oo000Hool00`00ooooo`0]ool2000_ool00liioooo
o`2Oool00`00ooooo`02ool001Soo`03003ooooo02ooo`8002koo`03cWWooooo09koo`03003ooooo
00;oo`006?oo0P001?oo70004Ooo00<00?ooool0;Ooo00?>NOooool0W?oo0P001?oo000Hool00`00
ooooo`0Oool<0007ool2000]ool00liiooooo`2Lool00`00ooooo`02ool001Soo`03003ooooo02_o
o`L000?oo`03003ooooo02[oo`03cWWooooo09_oo`03003ooooo00;oo`001Ooo0P001?oo00<00?oo
ool00ooo0P001Ooo00<00?ooool0<_oo1P00:ooo00?>NOooool0V_oo00<00?ooool00_oo0004ool0
1000ooooo`002?oo00@00?ooool000Coo`03003ooooo03Soo`@002Soo`03cWWooooo09Woo`03003o
oooo00;oo`001?oo00@00?ooool000Soo`04003ooooo0004ool3000jool00`00ool00002ool4000R
ool2cWVHool20004ool000Coo`04003ooooo0009ool20005ool00`00ooooo`0kool00`00ool00005
ool2000Rool00liiooooo`2Fool00`00ooooo`02ool000Coo`04003ooooo0008ool01000ooooo`00
1?oo00<00?ooool0?Ooo0`002Ooo0P007Ooo00?>NOooool0UOoo00<00?ooool00_oo0004ool01000
ooooo`002?oo00@00?ooool000Coo`03003ooooo043oo`03003ooooo00Soo`@001[oo`03cWWooooo
09Coo`03003ooooo00;oo`001Ooo0P002_oo0P001Ooo00<00?ooool0@?oo0P004Ooo0P005Ooo00?>
NOooool0Tooo00<00?ooool00_oo000Hool20013ool2000Aool2000Dool00liiooooo`2Aool20004
ool001Soo`03003ooooo04?oo`8001;oo`8001?oo`;>NI;oo`03003ooooo00;oo`006?oo00<00?oo
ool0AOoo00<00?ooool09_oo00?>NOooool0Sooo00<00?ooool00_oo000Hool00`00ooooo`16ool2
000Eool4000>ool00liiooooo`2>ool00`00ooooo`02ool001Soo`03003ooooo04Soo`03003ooooo
01Koo`8000goo`03cWWooooo08goo`03003ooooo00;oo`006?oo00<00?ooool0BOoo0`009Ooo00?>
NOooool0S?oo00<00?ooool00_oo000Hool2001<ool00`00ooooo`0Iool00`00ooooo`08ool2cWV;
ool20004ool001Soo`03003ooooo04coo`8001[oo`8000[oo`03cWWooooo08Woo`03003ooooo00;o
o`006?oo00<00?ooool0C_oo00<00?ooool06Ooo0P002Ooo00?>NOooool0R?oo00<00?ooool00_oo
000Hool00`00ooooo`1?ool2000Uool00liiooooo`27ool00`00ooooo`02ool001Soo`03003ooooo
057oo`8002Coo`03cWWooooo08Koo`03003ooooo00;oo`006?oo0P00Dooo0`006ooo00<00?ooool0
1Ooo00?>NOooool0Q?oo0P001?oo000Hool00`00ooooo`1Dool2000Kool30005ool2cWV5ool00`00
ooooo`02ool001Soo`03003ooooo05Koo`8001coo`8000Goo`03cWWooooo08;oo`03003ooooo00;o
o`004?oo10001?oo00<00?ooool0Eooo0P008ooo00?>NOooool0POoo00<00?ooool00_oo000Bool0
0`00ooooo`03ool00`00ooooo`1Iool2000Rool00liiooooo`20ool00`00ooooo`02ool001;oo`03
003ooooo00?oo`<005[oo`8001goo`8000?oo`03cWWooooo07koo`8000Coo`004_oo00<00?ooool0
0ooo00<00?ooool0G?oo0P007Ooo0P000_oo00?>NOooool0O_oo00<00?ooool00_oo000Bool00`00
ooooo`03ool00`00ooooo`1Nool00`00ooooo`0Lool200000ooocWW>N@1nool00`00ooooo`02ool0
013oo`<000Goo`03003ooooo05ooo`80027oo`03cWWooooo07_oo`03003ooooo00;oo`004_oo00<0
0?ooool00ooo00<00?ooool0HOoo00<00?ooool07ooo00?>NOooool0N_oo00<00?ooool00_oo000H
ool00`00ooooo`1Qool2000Oool00`00ooo>N@1kool00`00ooooo`02ool001Soo`8006Coo`03003o
oooo01goo`800003cWWooooo07Ooo`8000Coo`006?oo00<00?ooool0I?oo0P007ooo00<00<iicWT0
N?oo00<00?ooool00_oo000Hool00`00ooooo`1Vool00`00ooooo`0Mool00`00ooo>N@1gool00`00
ooooo`02ool001Soo`03003ooooo06Ooo`03003ooooo01ooo`03cWWooooo07Coo`03003ooooo00;o
o`006?oo00<00?ooool0J?oo0P008?oo00?>NOooool0Looo00<00?ooool00_oo000Hool2001[ool0
0`00ooooo`0Mool00`00cWWoo`1bool20004ool001Soo`03003ooooo06_oo`8001koo`03003>NOoo
07;oo`03003ooooo00;oo`006?oo00<00?ooool0KOoo00<00?ooool07?oo00<00<iiool0LOoo00<0
0?ooool00_oo000Hool00`00ooooo`1^ool00`00ooooo`0Lool00`00cWW>N@1`ool00`00ooooo`02
ool001Soo`03003ooooo06ooo`03003ooooo01coo`03003oolii06ooo`03003ooooo00;oo`006?oo
0P00LOoo0P007ooo00?>NOooool0Jooo0P001?oo000Hool00`00ooooo`1bool2000Nool00liioooo
o`1[ool00`00ooooo`02ool001Soo`03003ooooo07?oo`03003ooooo01coo`03003>NOoo06_oo`03
003ooooo00;oo`006?oo00<00?ooool0M?oo0`007?oo00<00<iiool0J_oo00<00?ooool00_oo0004
ool40003ool00`00ooooo`02ool40004ool00`00ooooo`1fool2000Lool00`00cWW>N@1Yool00`00
ooooo`02ool000Koo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo07Soo`8001coo`03003>
NOoo06Ooo`03003ooooo00;oo`001_oo00<00?ooool02?oo00<00?ooool01?oo0`00N_oo00<00?oo
ool06ooo00?>NOooool0I?oo0P001?oo0006ool00`00ooooo`09ool00`00ooooo`03ool00`00oooo
o`1kool00`00ooooo`0Kool00liiooooo`1Tool00`00ooooo`02ool000Koo`03003ooooo00[oo`03
003ooooo00;oo`03003ooooo07coo`03003ooooo01_oo`03cWWooooo06?oo`03003ooooo00;oo`00
1?oo0`002Ooo00@00?ooool000Coo`03003ooooo07goo`8001coo`;>NF?oo`03003ooooo00;oo`00
1_oo00<00?ooool02?oo0P001Ooo00<00?ooool0Oooo00<00?ooool06_oo00<00<iiool0HOoo00<0
0?ooool00_oo000Hool20021ool00`00ooooo`0Kool00liiooooo`1Nool20004ool001Soo`03003o
oooo087oo`8001_oo`03003>NOoo05ooo`03003ooooo00;oo`006?oo00<00?ooool0Pooo00<00?oo
ool06_oo00?>NOooool0GOoo00<00?ooool00_oo000Hool00`00ooooo`24ool2000Kool00liioooo
o`1Lool00`00ooooo`02ool001Soo`03003ooooo08Goo`8001_oo`03cWWooooo05_oo`03003ooooo
00;oo`006?oo0P00R?oo0P006_oo0/iiF_oo0P001?oo000Hool00`00ooooo`28ool00`00ooooo`0J
ool00liiooooo`1Hool00`00ooooo`02ool001Soo`03003ooooo08Woo`8001[oo`03003>NOoo05So
o`03003ooooo00;oo`006?oo00<00?ooool0Rooo00<00?ooool06?oo00<00<iiool0Eooo00<00?oo
ool00_oo000Hool00`00ooooo`2<ool00`00ooooo`0Iool00liiooooo`1Eool00`00ooooo`02ool0
01Soo`03003ooooo08goo`03003ooooo01Woo`03cWWooooo05Coo`03003ooooo00;oo`006?oo0P00
Sooo0P006_oo00?>NOooool0D_oo0P001?oo000Hool00`00ooooo`2?ool2000Jool00liiooooo`1B
ool00`00ooooo`02ool001Soo`03003ooooo097oo`03003ooooo01Soo`03cWT00?oo057oo`03003o
oooo00;oo`001?oo10000ooo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0T_oo0P006Ooo
0/iiDOoo00<00?ooool00_oo0006ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2Cool2
000Jool00liiooooo`1>ool00`00ooooo`02ool000Koo`03003ooooo00Koo`D000Coo`<009Goo`80
01Woo`03cWWooooo04coo`8000Coo`001_oo00<00?ooool01_oo00@00?ooool000Goo`03003ooooo
09Koo`8001Woo`03cWWooooo04coo`03003ooooo00;oo`001_oo00<00?ooool01ooo00<00?oo0000
1Ooo00<00?ooool0V?oo00<00?ooool05ooo00?>NOooool0Booo00<00?ooool00_oo0004ool3000:
ool20005ool00`00ooooo`2Iool2000Hool00liiooooo`1:ool00`00ooooo`02ool000Koo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo09_oo`03003ooooo01Koo`03cWT00?oo04Woo`03003o
oooo00;oo`006?oo0P00WOoo00<00?ooool05_oo00?>N@00ool0Aooo0P001?oo000Hool00`00oooo
o`2Mool2000Gool00liiooooo`17ool00`00ooooo`02ool001Soo`03003ooooo09ooo`03003ooooo
01Goo`;>NDOoo`03003ooooo00;oo`006?oo00<00?ooool0X?oo00<00?ooool05_oo00?>N@00ool0
A?oo00<00?ooool00_oo000Hool00`00ooooo`2Qool00`00ooooo`0Fool00lii003oo`13ool00`00
ooooo`02ool001Soo`03003ooooo0:;oo`8001Ooo`03cWWooooo04;oo`03003ooooo00;oo`006?oo
0P00YOoo00<00?ooool05Ooo00?>NOooool0@?oo0P001?oo000Hool00`00ooooo`2Uool00`00oooo
o`0Eool00lii003oo`10ool00`00ooooo`02ool001Soo`03003ooooo0:Koo`8001Koo`;>ND3oo`03
003ooooo00;oo`006?oo00<00?ooool0Yooo0P005ooo00?>NOooool0?Ooo00<00?ooool00_oo000H
ool00`00ooooo`2Yool2000Fool00lii003oo`0lool00`00ooooo`02ool001Soo`800:_oo`8001Ko
o`03cWT00?oo03[oo`8000Coo`006?oo00<00?ooool0[?oo00<00?ooool05?oo0/ii>ooo00<00?oo
ool00_oo000Hool00`00ooooo`2]ool00`00ooooo`0Dool00`00cWWoo`0iool00`00ooooo`02ool0
00Coo`@000?oo`03003ooooo00?oo`8000Goo`03003ooooo0:koo`8001Goo`03003>NOoo03Soo`03
003ooooo00;oo`001_oo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo0:ooo`8001Koo`03
cWWooooo03Koo`03003ooooo00;oo`001_oo00<00?ooool01ooo00@00?ooool000Coo`<00;7oo`03
003ooooo01Coo`03cWWooooo03Coo`8000Coo`001_oo00<00?ooool01ooo0`001Ooo00<00?ooool0
/Ooo0P005_oo00?>NOooool0=?oo00<00?ooool00_oo0006ool00`00ooooo`07ool00`00ooooo`05
ool00`00ooooo`2cool2000Eool00liiooooo`0cool00`00ooooo`02ool000Coo`<000Woo`03003o
oooo00Goo`03003ooooo0;Coo`8001Goo`;>NC?oo`03003ooooo00;oo`001_oo00<00?ooool02?oo
0`001?oo00<00?ooool0]_oo00<00?ooool05?oo00?>NOooool0<?oo00<00?ooool00_oo000Hool0
0`00ooooo`2gool00`00ooooo`0Dool00liiooooo`0_ool00`00ooooo`02ool001Soo`800;Woo`80
01Goo`03cWWooooo02goo`8000Coo`006?oo00<00?ooool0^_oo00<00?ooool04ooo00?>NOooool0
;Ooo00<00?ooool00_oo000Hool00`00ooooo`2kool00`00ooooo`0Cool00liiooooo`0/ool00`00
ooooo`02ool001Soo`03003ooooo0;coo`8001Coo`03cWWooooo02_oo`03003ooooo00;oo`006?oo
00<00?ooool0_Ooo0P005?oo0/ii:ooo00<00?ooool00_oo000Hool20030ool00`00ooooo`0Cool0
0liiooooo`0Wool20004ool001Soo`03003ooooo0<3oo`03003ooooo01?oo`03cWWooooo02Ooo`03
003ooooo00;oo`006?oo00<00?ooool0`Ooo0P005?oo00?>NOooool09_oo00<00?ooool00_oo000H
ool00`00ooooo`32ool2000Dool00liiooooo`0Uool00`00ooooo`02ool001Soo`03003ooooo0<Co
o`03003ooooo01;oo`;>NBGoo`03003ooooo00;oo`006?oo0P00a_oo00<00?ooool04_oo00<00<ii
ool08_oo0P001?oo000Hool00`00ooooo`36ool00`00ooooo`0Cool00liiooooo`0Qool00`00oooo
o`02ool001Soo`03003ooooo0<Ooo`8001Coo`03cWWooooo023oo`03003ooooo00;oo`006?oo00<0
0?ooool0bOoo00<00?ooool04_oo0/ii8?oo00<00?ooool00_oo0004ool40003ool00`00ooooo`03
ool20005ool00`00ooooo`39ool2000Eool00liiooooo`0Mool00`00ooooo`02ool000Koo`03003o
oooo00Ooo`04003ooooo0004ool00`00ooooo`3;ool2000Dool00liiooooo`0Lool00`00ooooo`02
ool000Koo`03003ooooo00Ooo`04003ooooo0004ool3003<ool00`00ooooo`0Bool00`00cWWoo`0K
ool20004ool000Koo`03003ooooo00Soo`8000Goo`03003ooooo0<goo`8001?oo`03003>NLii01_o
o`03003ooooo00;oo`001_oo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo0<ooo`03003o
oooo017oo`800003cWWooooo01Soo`03003ooooo00;oo`001?oo0`002Ooo00@00?ooool000Coo`03
003ooooo0=3oo`8001?oo`03003>NOoo01Soo`03003ooooo00;oo`001_oo00<00?ooool02?oo0P00
1Ooo00<00?ooool0dOoo00<00?ooool04ooo00?>NOooool05_oo00<00?ooool00_oo000Hool2003C
ool2000Dool00liiooooo`0Dool20004ool001Soo`03003ooooo0=?oo`8001Coo`03cWWooooo01Co
o`03003ooooo00;oo`006?oo00<00?ooool0eOoo0P004ooo00?>NOooool04ooo00<00?ooool00_oo
000Hool00`00ooooo`3Fool00`00ooooo`0Bool00liiooooo`0Bool00`00ooooo`02ool001Soo`03
003ooooo0=Ooo`8001?oo`03cWWooooo017oo`03003ooooo00;oo`006?oo0P00fOoo00<00?ooool0
4_oo00?>NOooool03ooo0P001?oo000Hool00`00ooooo`3Iool3000Bool00liiooooo`0?ool00`00
ooooo`02ool001Soo`03003ooooo0=_oo`03003ooooo017oo`03cWWooooo00koo`03003ooooo00;o
o`006?oo00<00?ooool0g?oo0P004_oo00?>N@00ool03Ooo00<00?ooool00_oo000Hool00`00oooo
o`3Mool2000Bool00lii003oo`0<ool00`00ooooo`02ool001Soo`03003ooooo0=ooo`80017oo`;>
N@coo`03003ooooo00;oo`006?oo0P00hOoo00<00?ooool04Ooo00?>NOooool02?oo0P001?oo000H
ool00`00ooooo`3Qool2000Bool00liiooooo`08ool00`00ooooo`02ool001Soo`03003ooooo0>?o
o`03003ooooo013oo`03cWWooooo00Ooo`03003ooooo00;oo`004?oo10001?oo00<00?ooool00ooo
00<00?ooool0>ooo00<00?ooool0>ooo00<00?ooool0>_oo00<00?ooool09Ooo00<00?ooool04?oo
0/ii00?oo`00ool01?oo00<00?ooool00_oo000@ool00`00ooooo`05ool00`00ooooo`03ool00`00
ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00
ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00
ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00
ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool01000oooooooo0P001ooo00<00?ooool02?oo
00?>N@00ool01?oo00<00?ooool00_oo000Aool00`00ooooo`04ooom00000lii000000040004ool0
01;oo`03003ooooo0?ooo`coo`004ooo00<00?ooool0oooo2ooo000@ool01000ooooo`00oooo3Ooo
000Aool2003oool>ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.248132, 0.385382, \
0.00810174, 0.00936347}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<stable0.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("stable0.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%583, %819, %619, %824, PlotRange \[Rule] {0.5, 2}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.206011 0.412023 [
[.2619 .19351 -9 -9 ]
[.2619 .19351 9 0 ]
[.5 .19351 -3 -9 ]
[.5 .19351 3 0 ]
[.7381 .19351 -9 -9 ]
[.7381 .19351 9 0 ]
[.97619 .19351 -3 -9 ]
[.97619 .19351 3 0 ]
[.01131 .0412 -18 -4.5 ]
[.01131 .0412 0 4.5 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .28842 -18 -4.5 ]
[.01131 .28842 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .45322 -18 -4.5 ]
[.01131 .45322 0 4.5 ]
[.01131 .53563 -18 -4.5 ]
[.01131 .53563 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .20601 m
.2619 .21226 L
s
[(0.5)] .2619 .19351 0 1 Mshowa
.5 .20601 m
.5 .21226 L
s
[(1)] .5 .19351 0 1 Mshowa
.7381 .20601 m
.7381 .21226 L
s
[(1.5)] .7381 .19351 0 1 Mshowa
.97619 .20601 m
.97619 .21226 L
s
[(2)] .97619 .19351 0 1 Mshowa
.125 Mabswid
.07143 .20601 m
.07143 .20976 L
s
.11905 .20601 m
.11905 .20976 L
s
.16667 .20601 m
.16667 .20976 L
s
.21429 .20601 m
.21429 .20976 L
s
.30952 .20601 m
.30952 .20976 L
s
.35714 .20601 m
.35714 .20976 L
s
.40476 .20601 m
.40476 .20976 L
s
.45238 .20601 m
.45238 .20976 L
s
.54762 .20601 m
.54762 .20976 L
s
.59524 .20601 m
.59524 .20976 L
s
.64286 .20601 m
.64286 .20976 L
s
.69048 .20601 m
.69048 .20976 L
s
.78571 .20601 m
.78571 .20976 L
s
.83333 .20601 m
.83333 .20976 L
s
.88095 .20601 m
.88095 .20976 L
s
.92857 .20601 m
.92857 .20976 L
s
.25 Mabswid
0 .20601 m
1 .20601 L
s
.02381 .0412 m
.03006 .0412 L
s
[(0.6)] .01131 .0412 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.8)] .01131 .12361 1 0 Mshowa
.02381 .28842 m
.03006 .28842 L
s
[(1.2)] .01131 .28842 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(1.4)] .01131 .37082 1 0 Mshowa
.02381 .45322 m
.03006 .45322 L
s
[(1.6)] .01131 .45322 1 0 Mshowa
.02381 .53563 m
.03006 .53563 L
s
[(1.8)] .01131 .53563 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(2)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .41202 m
.02756 .41202 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.02381 .0206 m
.02756 .0206 L
s
.02381 0 m
.02756 0 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.8 g
.5 Mabswid
.2619 0 m
.30316 .03569 L
.34309 .07024 L
.3815 .10348 L
.42237 .13884 L
.46172 .17289 L
.49955 .20562 L
.53984 .24048 L
.57861 .27403 L
.61984 .3097 L
.65954 .34406 L
.69774 .3771 L
.73838 .41227 L
.77751 .44613 L
.81909 .48211 L
.85916 .51677 L
.89771 .55013 L
.93871 .58561 L
.97619 .61803 L
s
0 g
.02429 .10331 m
.0481 .10351 L
.0719 .10412 L
.09571 .10512 L
.11952 .10654 L
.14333 .10838 L
.16714 .11066 L
.19095 .1134 L
.19095 .1134 L
.21429 .11657 L
.23857 .12038 L
.26667 .14357 L
.29048 .15852 L
.31429 .17392 L
.3381 .18971 L
.3619 .20584 L
.38571 .22227 L
.40952 .23897 L
.43333 .25591 L
.45714 .27307 L
.48095 .29043 L
.50476 .30797 L
.52857 .32568 L
.55238 .34354 L
.57619 .36154 L
.6 .37967 L
.62381 .39793 L
.64762 .4163 L
.67143 .43477 L
.69524 .45334 L
.71905 .472 L
.74286 .49075 L
.76667 .50958 L
.79048 .52849 L
.81429 .54747 L
.8381 .56651 L
.8619 .58562 L
.88571 .60479 L
s
.88571 .60479 m
.90212 .61803 L
s
[ .002 .01 ] 0 setdash
.02429 .04842 m
.07143 .0534 L
.09524 .05936 L
.11905 .06726 L
.14286 .0768 L
.16667 .0877 L
.19048 .09971 L
.21429 .11264 L
.2381 .12632 L
.2619 .14064 L
.28571 .15549 L
.30952 .17081 L
.33333 .18653 L
.35714 .20259 L
.38095 .21896 L
.40476 .23561 L
.42857 .25251 L
.45238 .26962 L
.47619 .28695 L
.5 .30445 L
.52381 .32213 L
.54762 .33996 L
.57143 .35793 L
.59524 .37604 L
.61905 .39427 L
.64286 .41261 L
.66667 .43107 L
.69048 .44962 L
.71429 .46826 L
.7381 .487 L
.7619 .50581 L
.78571 .5247 L
.80952 .54367 L
.83333 .5627 L
.85714 .58179 L
.88095 .60095 L
s
.88095 .60095 m
.90213 .61803 L
s
[ .02 .02 ] 0 setdash
.02857 .04847 m
.05238 .05024 L
.07619 .05442 L
.17143 .09002 L
.19524 .10223 L
.21905 .1153 L
.26238 .12469 L
.28619 .12961 L
.31 .1352 L
.33381 .14153 L
.35762 .14871 L
.38143 .15684 L
.40524 .16604 L
.42905 .17648 L
.45286 .18831 L
.47667 .20167 L
.50048 .21664 L
.52429 .23317 L
.5481 .25107 L
.5719 .27004 L
.59571 .28972 L
.61952 .30985 L
.64333 .33022 L
.66714 .35071 L
.69095 .37127 L
.71476 .39185 L
.73857 .41244 L
.76238 .43304 L
.78619 .45364 L
.81 .47424 L
.83381 .49484 L
.85762 .51544 L
.88143 .53604 L
.90524 .55664 L
.92905 .57724 L
.95286 .59784 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001Goo`8003ooo`03cWWooooo0<Ooo`005Ooo00<00?ooool0?ooo00?>NOooool0a_oo000Eool0
0`00ooooo`10ool2cWW6ool001Goo`03003ooooo04;oo`03cWWooooo0<?oo`005Ooo00<00?ooool0
@ooo00?>NOooool0`_oo000Eool20015ool00liiooooo`31ool001Goo`03003ooooo04Goo`03cWWo
oooo0<3oo`005Ooo00<00?ooool0A_oo0/ii`?oo000Eool00`00ooooo`18ool00liiooooo`2mool0
00;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo04Woo`03cWWooooo0;coo`0000Goo`00
ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1:ool00liiooooo`2kool00005ool00?ooool0
0008ool01000ooooo`001?oo0`00Booo00?>NOooool0^_oo00001Ooo003ooooo00002?oo0`001Ooo
00<00?ooool0C?oo00?>NOooool0^Ooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo2000B?oo
00?>NOooool0^?oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool00ooo0P000_oo
00<00?oo00000P00@_oo0/ii^?oo0002ool2000:ool30004ool00`00ooooo`0:ool50011ool00lii
ooooo`2eool001Goo`03003ooooo00goo`800003ool0000003ooo`03cWWooooo0;Coo`005Ooo0P00
5?oo0P00?Ooo00?>NOooool0/ooo000Eool00`00ooooo`0Cool20002ool00`00ooooo`0iool00lii
ooooo`2bool001Goo`03003ooooo01Goo`8000?oo`03003ooooo03Ooo`03cWWooooo0;7oo`005Ooo
00<00?ooool05ooo0P001?oo00<00?ooool0=Ooo00?>NOooool0/?oo000Eool00`00ooooo`0Mool2
000gool00liiooooo`2_ool001Goo`80023oo`<003Goo`03cWWooooo0:koo`005Ooo00<00?ooool0
8_oo0P00=?oo00?>NOooool0[Ooo000Eool00`00ooooo`0Uool00`00ooooo`0aool2cWV]ool001Go
o`03003ooooo02Koo`03003ooooo03;oo`03cWWooooo0:[oo`005Ooo00<00?ooool0:?oo0P00<_oo
00?>NOooool0ZOoo000Eool00`00ooooo`0Yool3000aool00liiooooo`2Xool001GooaL001Ooo`80
037oo`03cWWooooo0:Ooo`005Ooo00<00?ooool05?oo3@003Ooo0P00;ooo00?>NOooool0Y_oo000E
ool00`00ooooo`0Qool60004ool00`00ooooo`0bool2cWVVool000;oo`8000Coo`03003ooooo00?o
o`8000Goo`03003ooooo02Ooo`d002koo`03cWWooooo0:?oo`0000Goo`00ooooo`0000Soo`04003o
oooo0004ool00`00ooooo`0dool4000[ool00liiooooo`2Rool00005ool00?ooool00008ool01000
ooooo`001?oo0`00>?oo00D00?ooooooo`0002Ooo`03cWWooooo0:7oo`0000Goo`00ooooo`0000Wo
o`8000Goo`03003ooooo03Ooo`03003oo`0000?oo`D002?oo`03cWWooooo0:3oo`0000Goo`00oooo
o`0000Soo`04003ooooo0004ool00`00ooooo`0jool00`00ooooo`09ool3000Mool2cWVPool00005
ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0>Ooo10003?oo0`007?oo00?>NOooool0
WOoo0002ool2000:ool20005ool00`00ooooo`0lool2000>ool00`00ooooo`0Jool00liiooooo`2L
ool001Goo`03003ooooo03koo`03003ooooo013oo`8001Ooo`03cWWooooo09_oo`005Ooo0P00@?oo
0P004_oo0P005_oo00?>NOooool0V_oo000Eool00`00ooooo`10ool2000Cool2000Eool00liioooo
o`2Iool001Goo`03003ooooo04;oo`03003ooooo01Koo`80013oo`03cWWooooo09Soo`005Ooo00<0
0?ooool0@ooo0`005ooo0P003ooo0/iiV?oo000Eool00`00ooooo`15ool00`00ooooo`0Gool2000?
ool00liiooooo`2Eool001Goo`03003ooooo04Koo`<002Soo`03cWWooooo09Coo`005Ooo0P00>?oo
0P001?oo00<00?ooool00_oo0`000ooo0P006ooo0P002ooo00?>NOooool01ooo1000=ooo10000ooo
00<00?ooool00_oo0`00>?oo10001_oo000Eool00`00ooooo`0fool01000ooooo`002ooo00<00?oo
ool00_oo0P006ooo0P002_oo00?>NOooool02?oo00<00?ooool0>?oo00<00?ooool02_oo00<00?oo
ool0=Ooo00<00?ooool01ooo000Eool00`00ooooo`0fool01000ooooo`002ooo00<00?ooool01?oo
00<00?ooool06_oo0P002Ooo00?>NOooool01ooo00<00?ooool0>?oo00<00?ooool02_oo00<00?oo
ool0=_oo00<00?ooool01_oo000Eool00`00ooooo`0fool01000ooooo`002?oo0`002?oo0P009_oo
00?>NOooool01_oo00<00?ooool0>?oo00<00?ooool01ooo0`00>_oo00<00?ooool01Ooo000Eool0
0`00ooooo`0fool01000ooooo`002?oo00<00?ooool02_oo0P009Ooo00?>NOooool01Ooo00<00?oo
ool0>?oo00<00?ooool01ooo00<00?ooool0>ooo00<00?ooool01?oo000Eool2000gool01000oooo
o`002?oo00<00?ooool02ooo0P007Ooo0P001_oo00?>NOooool00_oo0`00>?oo0`002Ooo00<00?oo
ool0>?oo00@00?ooool000Koo`005Ooo00<00?ooool0=ooo0P002Ooo10003?oo00<00?ooool07?oo
0P001Ooo00?>NOooool00ooo00<00?ooool0>?oo00<00?ooool01ooo1000>?oo0P001ooo000Eool0
0`00ooooo`1Cool2000Nool20004ool2cWV=ool001Goo`03003ooooo05Goo`03003ooooo02?oo`03
cWWooooo08[oo`005Ooo00<00?ooool0E_oo00<00?ooool08ooo00?>NOooool0ROoo000Eool00`00
ooooo`1Gool2000Pool01@00oooooooocWT0R_oo000>oon800000lii00000027000001Goo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00[oo`04003oo`000009ool00`00ooooo`0:ool00`00ooooo`06ool20000
0ooo003>N@0;ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool0
0`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool0
0`00ooooo`05ool001Goo`03003ooooo05coo`03003ooooo01koo`03003ooooo00;>NHKoo`005Ooo
00<00?ooool0GOoo0P008ooo00?>NOooool0Pooo000Eool00`00ooooo`1Oool2000Rool00liioooo
o`22ool001Goo`80067oo`80023oo`03003oolii08?oo`005Ooo00<00?ooool0H_oo00<00?ooool0
7_oo0P0000?>NOooool0P?oo000Eool00`00ooooo`1Sool00`00ooooo`0Oool00`00cWW>N@20ool0
01Goo`03003ooooo06Coo`80023oo`03003oolii07ooo`005Ooo00<00?ooool0I_oo00<00?ooool0
8?oo00?>NOooool0O?oo000Eool00`00ooooo`1Wool2000Qool00liiooooo`1kool001Goo`8006[o
o`03003ooooo01ooo`03003>NOoo07[oo`005Ooo00<00?ooool0J_oo0P008?oo00<00<iiool0NOoo
000Eool00`00ooooo`1/ool00`00ooooo`0Nool00`00cWWoo`1hool001Goo`03003ooooo06goo`80
01ooo`03003>NOoo07Ooo`005Ooo00<00?ooool0Kooo00<00?ooool07Ooo00<00<iiool0M_oo000E
ool00`00ooooo`1`ool00`00ooooo`0Nool2cWUeool001Goo`8007;oo`80023oo`03cWWooooo07;o
o`005Ooo00<00?ooool0L_oo0P007ooo00<00<iiool0L_oo000Eool00`00ooooo`1dool00`00oooo
o`0Mool00`00cWWoo`1aool00003ool00000008000?oo`03003ooooo00;oo`@000Coo`03003ooooo
07Goo`03003ooooo01goo`03003>NOoo073oo`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<0
0?ooool0M_oo0P007_oo00<00<iiool0Kooo0003ool00`00ooooo`08ool00`00ooooo`04ool3001h
ool00`00ooooo`0Lool00`00cWWoo`1^ool000?oo`03003ooooo00Woo`03003ooooo00?oo`03003o
oooo07Woo`03003ooooo01goo`;>NFgoo`000ooo00<00?ooool02_oo00<00?ooool00_oo00<00?oo
ool0N_oo0P007ooo00?>NOooool0J_oo00001?oo000000002Ooo00@00?ooool000Coo`03003ooooo
07coo`8001koo`03003ooooo06Woo`000ooo00<00?ooool02?oo0P001Ooo00<00?ooool0OOoo0`00
7Ooo00<00?ooool0J?oo000Eool00`00ooooo`20ool00`00ooooo`0Kool00`00ooooo`1Wool001Go
o`80087oo`8001goo`03003ooooo06Koo`005Ooo00<00?ooool0P_oo00<00?ooool06ooo00<00?oo
ool0IOoo000Eool00`00ooooo`23ool2000Lool2cWUUool001Goo`03003ooooo08Goo`03003ooooo
01_oo`03cWWooooo06;oo`005Ooo00<00?ooool0QOoo0P007Ooo00<00?ooool0HOoo000Eool20028
ool2000Kool00`00cWWoo`1Qool001Goo`03003ooooo08Soo`8001_oo`03003>NOoo063oo`005Ooo
00<00?ooool0R_oo00<00?ooool06Ooo0P0000?>NOooool0GOoo000Eool00`00ooooo`2:ool3000K
ool00`00cWWoo`1Mool001Goo`03003ooooo08goo`03003ooooo01[oo`03cWWooooo05_oo`005Ooo
00<00?ooool0S_oo00<00?ooool06_oo00?>NOooool0F_oo000Eool2002@ool2000Kool00`00oooo
o`1Iool001Goo`03003ooooo093oo`8001_oo`03003ooooo05Soo`005Ooo00<00?ooool0T_oo00<0
0?ooool06Ooo00<00?ooool0Eooo000Eool00`00ooooo`2Cool00`00ooooo`0Iool00`00cWWoo`1F
ool00003ool00000008000?oo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo09Coo`8001[o
o`03003>NOoo05Goo`000ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0U_oo00<00?oo
ool06Ooo00?>NOooool0Dooo0003ool00`00ooooo`06ool50004ool3002Gool00`00ooooo`0Iool0
0liiooooo`1Bool000?oo`03003ooooo00Koo`04003ooooo0005ool00`00ooooo`2Hool2000Jool0
0`00ooooo`1Aool000?oo`03003ooooo00Ooo`03003oo`0000Goo`03003ooooo09[oo`03003ooooo
01Soo`03003ooooo053oo`0000Coo`00000000[oo`8000Goo`03003ooooo09_oo`03003ooooo01So
o`03003ooooo04ooo`000ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0W?oo0P006Ooo
00<00?ooool0C_oo000Eool2002Nool2000Iool00`00ooooo`1=ool001Goo`03003ooooo09ooo`03
003ooooo01Ooo`;>NDgoo`005Ooo00<00?ooool0X?oo00<00?ooool06?oo00?>NOooool0B_oo000E
ool00`00ooooo`2Qool2000Iool00liiooooo`19ool001Goo`03003ooooo0:?oo`03003ooooo01Oo
o`03003ooooo04Soo`005Ooo00<00?ooool0Y?oo00<00?ooool05ooo00<00?ooool0Aooo000Eool2
002Vool2000Hool00`00ooooo`16ool001Goo`03003ooooo0:Ooo`03003ooooo01Koo`8004Koo`00
5Ooo00<00?ooool0Z?oo00<00?ooool05_oo0/iiAOoo000Eool00`00ooooo`2Yool2000Hool00lii
ooooo`12ool001Goo`03003ooooo0:[oo`8001Soo`03cWWooooo047oo`005Ooo0P00[Ooo00<00?oo
ool05_oo00<00?ooool0@?oo000Eool00`00ooooo`2]ool00`00ooooo`0Fool00`00ooooo`0oool0
01Goo`03003ooooo0:koo`8001Ooo`8003ooo`005Ooo00<00?ooool0/?oo00<00?ooool05_oo00<0
0?ooool0??oo00000ooo000000020003ool00`00ooooo`03ool20005ool00`00ooooo`2aool00`00
ooooo`0Fool00liiooooo`0kool000?oo`03003ooooo00Ooo`04003ooooo0004ool00`00ooooo`2b
ool2000Gool00liiooooo`0jool000?oo`03003ooooo00Ooo`04003ooooo0004ool3002cool2000G
ool00liiooooo`0iool000?oo`03003ooooo00Ooo`<000Goo`03003ooooo0;Goo`03003ooooo01Go
o`03003ooooo03Soo`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0]_oo00<00?oo
ool05Ooo00<00?ooool0=ooo00001?oo000000002Ooo00<00?ooool01Ooo00<00?ooool0]ooo0P00
5_oo0P00=ooo0003ool00`00ooooo`08ool30004ool00`00ooooo`2iool00`00ooooo`0Eool00`00
ooooo`0dool001Goo`03003ooooo0;[oo`03003ooooo01Goo`03cWWooooo03?oo`005Ooo0P00_?oo
0P005_oo00?>NOooool0<_oo000Eool00`00ooooo`2mool00`00ooooo`0Dool00liiooooo`0aool0
01Goo`03003ooooo0;koo`03003ooooo01Coo`03003ooooo033oo`005Ooo00<00?ooool0_ooo0P00
5Ooo00<00?ooool0;ooo000Eool00`00ooooo`30ool2000Eool2000_ool001Goo`800<?oo`03003o
oooo01Coo`03003ooooo02coo`005Ooo00<00?ooool0`ooo00<00?ooool04ooo00<00<iiool0;?oo
000Eool00`00ooooo`34ool2000Eool00liiooooo`0Zool001Goo`03003ooooo0<Goo`8001Goo`03
cWWooooo02Woo`005Ooo00<00?ooool0aooo00<00?ooool04ooo00?>N@00ool0:?oo000Eool00`00
ooooo`38ool00`00ooooo`0Dool00`00ooooo`0Vool001Goo`800<[oo`03003ooooo01Coo`03003o
oooo02Goo`005Ooo00<00?ooool0b_oo0P005Ooo00<00?ooool09?oo000Eool00`00ooooo`3;ool2
000Eool00`00cWWoo`0Sool00003ool00000008000?oo`03003ooooo00?oo`8000Goo`03003ooooo
0<goo`03003ooooo01Coo`03cWWooooo027oo`000ooo00<00?ooool01ooo00@00?ooool000Coo`03
003ooooo0<koo`03003ooooo01Coo`03cWWooooo023oo`000ooo00<00?ooool01ooo00@00?ooool0
00Coo`<00<ooo`8001Goo`03003ooooo01ooo`000ooo00<00?ooool02?oo0P001Ooo00<00?ooool0
d?oo0P005?oo00<00<iiool07ooo0003ool00`00ooooo`07ool01000ooooo`001?oo00<00?ooool0
d_oo00<00?ooool04_oo00<00<iicWT07_oo00001?oo000000002Ooo00@00?ooool000Coo`03003o
oooo0=?oo`03003ooooo01;oo`800003cWWooooo01_oo`000ooo00<00?ooool02?oo0P001Ooo00<0
0?ooool0e?oo0P005?oo00<00<iiool06ooo000Eool00`00ooooo`3Eool00`00ooooo`0Dool00lii
ooooo`0Iool001Goo`800=Ooo`8001Goo`03cWWooooo01Soo`005Ooo00<00?ooool0eooo0P005Ooo
00<00?ooool05ooo000Eool00`00ooooo`3Iool2000Dool00`00ooooo`0Fool001Goo`03003ooooo
0=[oo`03003ooooo01?oo`03003>NOoo01Goo`005Ooo00<00?ooool0fooo0P005?oo00<00<iiool0
5?oo000Eool00`00ooooo`3Mool00`00ooooo`0Bool00`00cWWoo`0Cool001Goo`800=ooo`8001Co
o`03cWWooooo017oo`005Ooo00<00?ooool0gooo0P005?oo00?>NOooool04?oo000Eool00`00oooo
o`3Qool00`00ooooo`0Bool00`00ooooo`0?ool001Goo`03003ooooo0>7oo`8001Coo`03003ooooo
00koo`005Ooo00<00?ooool0hooo00<00?ooool04_oo0P003_oo000Eool2003Uool2000Cool00lii
003oo`0<ool001Goo`03003ooooo0>Koo`03003ooooo017oo`03cWWooooo00_oo`005Ooo00<00?oo
ool0i_oo0P004ooo0/ii2ooo000Eool00`00ooooo`3Xool2000Cool00liiooooo`08ool000goo`@0
00Coo`03003ooooo0>Woo`8001?oo`03cWWooooo00Ooo`003Ooo00<00?ooool01Ooo00<00?ooool0
jooo00<00?ooool04Ooo00?>NOooool01_oo000>ool00`00ooooo`04ool3003/ool00`00ooooo`0A
ool00liiooooo`05ool000ooo`03003ooooo0?ooo`ooo`004?oo00<00?ooool0oooo3_oo000=ool0
1000ooooo`00oooo4?oo000>ool2003ooolAool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.165192, 0.480876, \
0.00771848, 0.00892053}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<stable0.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("stable0.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.6175683841893449/Sqrt[Sqrt[11]]*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(0.5873507270584292`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omthree[IntegerPart[T*10^16]*10^\((\(-16\))\), 
          Pi/2 \((1 - 10^\((\(-5\))\))\), 10]}, {T, 0.01, 2, 
        1/20}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.6175683841893449`}, {0.060000000000000005`, 
        0.6195902378594974`}, {0.11000000000000001`, 
        0.6244868595943324`}, {0.16000000000000003`, 
        0.7681224200592205`}, {0.21000000000000002`, 
        0.7804360603402923`}, {0.26`, 0.7957882677969733`}, {0.31`, 
        0.6718284334225318`}, {0.36`, 0.6902048598415703`}, {0.41`, 
        0.7109898489800215`}, {0.45999999999999996`, 
        0.7340638337708439`}, {0.51`, 0.9114795243320903`}, {0.56`, 
        0.9410118662917161`}, {0.6100000000000001`, 
        0.97224121672288`}, {0.6600000000000001`, 
        1.0050037972116912`}, {0.7100000000000002`, 
        1.0391508238488232`}, {0.7600000000000002`, 
        1.0745453809141872`}, {0.8100000000000003`, 
        1.1110671907326304`}, {0.8600000000000003`, 
        1.1486051105757356`}, {0.9100000000000004`, 
        1.1870602479542092`}, {0.9600000000000004`, 
        1.2263457179424437`}, {1.0100000000000005`, 
        1.2663825389721683`}, {1.0600000000000005`, 
        1.3071006187805911`}, {1.1100000000000005`, 
        1.3484374085848763`}, {1.1600000000000006`, 
        1.3903370682454976`}, {1.2100000000000006`, 
        1.4327497098747894`}, {1.2600000000000007`, 
        1.475630718292647`}, {1.3100000000000007`, 
        1.5189401446871431`}, {1.3600000000000008`, 
        1.5626416390314517`}, {1.4100000000000008`, 
        1.6067040491006312`}, {1.4600000000000009`, 
        1.6510979670384258`}, {1.510000000000001`, 
        1.6957973220153524`}, {1.560000000000001`, 
        1.7407785831815819`}, {1.610000000000001`, 
        1.7860204752793154`}, {1.660000000000001`, 
        1.8315037288176976`}, {1.710000000000001`, 
        1.87721086052069`}, {1.7600000000000011`, 
        1.9231259802473222`}, {1.8100000000000012`, 
        1.9692346210384344`}, {1.8600000000000012`, 
        2.015523589358042`}, {1.9100000000000013`, 
        2.0619808329687785`}, {1.9600000000000013`, 
        2.108595324210436`}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(omthree[k_, th_, a_] := 
      om /. FindRoot[
          testfunc[om, IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), th, 
              a] \[Equal] 0, {om, k + 5/8, k + 6/8}, 
          MaxIterations \[Rule] 25]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(omthree[0.001, Pi/2 \((1 - 10^\((\(-5\))\))\), 10]\)], "Input"],

Cell[BoxData[
    \(0.750733673711815`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.750733673711815`/Sqrt[Sqrt[11]]*Sqrt[3]\)], "Input"],

Cell[BoxData[
    \(0.7140002311819896`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.001, 0.750733673711815`}, {0.16000000000000003`, 
        0.7681224200592205`}, {0.21000000000000002`, 
        0.7804360603402923`}, {0.26`, 0.7957882677969733`}, {0.46, 
        0.8838194264327792`}, {0.51`, 0.9114795243320903`}, {0.56`, 
        0.9410118662917161`}, {0.6100000000000001`, 
        0.97224121672288`}, {0.6600000000000001`, 
        1.0050037972116912`}, {0.7100000000000002`, 
        1.0391508238488232`}, {0.7600000000000002`, 
        1.0745453809141872`}, {0.8100000000000003`, 
        1.1110671907326304`}, {0.8600000000000003`, 
        1.1486051105757356`}, {0.9100000000000004`, 
        1.1870602479542092`}, {0.9600000000000004`, 
        1.2263457179424437`}, {1.0100000000000005`, 
        1.2663825389721683`}, {1.0600000000000005`, 
        1.3071006187805911`}, {1.1100000000000005`, 
        1.3484374085848763`}, {1.1600000000000006`, 
        1.3903370682454976`}, {1.2100000000000006`, 
        1.4327497098747894`}, {1.2600000000000007`, 
        1.475630718292647`}, {1.3100000000000007`, 
        1.5189401446871431`}, {1.3600000000000008`, 
        1.5626416390314517`}, {1.4100000000000008`, 
        1.6067040491006312`}, {1.4600000000000009`, 
        1.6510979670384258`}, {1.510000000000001`, 
        1.6957973220153524`}, {1.560000000000001`, 
        1.7407785831815819`}, {1.610000000000001`, 
        1.7860204752793154`}, {1.660000000000001`, 
        1.8315037288176976`}, {1.710000000000001`, 
        1.87721086052069`}, {1.7600000000000011`, 
        1.9231259802473222`}, {1.8100000000000012`, 
        1.9692346210384344`}, {1.8600000000000012`, 
        2.015523589358042`}, {1.9100000000000013`, 
        2.0619808329687785`}, {1.9600000000000013`, 
        2.108595324210436`}}\)], "Input"],

Cell[BoxData[
    \({{0.001`, 0.750733673711815`}, {0.16000000000000003`, 
        0.7681224200592205`}, {0.21000000000000002`, 
        0.7804360603402923`}, {0.26`, 0.7957882677969733`}, {0.46`, 
        0.8838194264327792`}, {0.51`, 0.9114795243320903`}, {0.56`, 
        0.9410118662917161`}, {0.6100000000000001`, 
        0.97224121672288`}, {0.6600000000000001`, 
        1.0050037972116912`}, {0.7100000000000002`, 
        1.0391508238488232`}, {0.7600000000000002`, 
        1.0745453809141872`}, {0.8100000000000003`, 
        1.1110671907326304`}, {0.8600000000000003`, 
        1.1486051105757356`}, {0.9100000000000004`, 
        1.1870602479542092`}, {0.9600000000000004`, 
        1.2263457179424437`}, {1.0100000000000005`, 
        1.2663825389721683`}, {1.0600000000000005`, 
        1.3071006187805911`}, {1.1100000000000005`, 
        1.3484374085848763`}, {1.1600000000000006`, 
        1.3903370682454976`}, {1.2100000000000006`, 
        1.4327497098747894`}, {1.2600000000000007`, 
        1.475630718292647`}, {1.3100000000000007`, 
        1.5189401446871431`}, {1.3600000000000008`, 
        1.5626416390314517`}, {1.4100000000000008`, 
        1.6067040491006312`}, {1.4600000000000009`, 
        1.6510979670384258`}, {1.510000000000001`, 
        1.6957973220153524`}, {1.560000000000001`, 
        1.7407785831815819`}, {1.610000000000001`, 
        1.7860204752793154`}, {1.660000000000001`, 
        1.8315037288176976`}, {1.710000000000001`, 
        1.87721086052069`}, {1.7600000000000011`, 
        1.9231259802473222`}, {1.8100000000000012`, 
        1.9692346210384344`}, {1.8600000000000012`, 
        2.015523589358042`}, {1.9100000000000013`, 
        2.0619808329687785`}, {1.9600000000000013`, 
        2.108595324210436`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.485909 -0.310712 0.433478 [
[.26676 .11027 -9 -9 ]
[.26676 .11027 9 0 ]
[.50972 .11027 -3 -9 ]
[.50972 .11027 3 0 ]
[.75267 .11027 -9 -9 ]
[.75267 .11027 9 0 ]
[.99563 .11027 -3 -9 ]
[.99563 .11027 3 0 ]
[.01131 .03607 -18 -4.5 ]
[.01131 .03607 0 4.5 ]
[.01131 .20946 -18 -4.5 ]
[.01131 .20946 0 4.5 ]
[.01131 .29616 -18 -4.5 ]
[.01131 .29616 0 4.5 ]
[.01131 .38285 -18 -4.5 ]
[.01131 .38285 0 4.5 ]
[.01131 .46955 -18 -4.5 ]
[.01131 .46955 0 4.5 ]
[.01131 .55625 -6 -4.5 ]
[.01131 .55625 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26676 .12277 m
.26676 .12902 L
s
[(0.5)] .26676 .11027 0 1 Mshowa
.50972 .12277 m
.50972 .12902 L
s
[(1)] .50972 .11027 0 1 Mshowa
.75267 .12277 m
.75267 .12902 L
s
[(1.5)] .75267 .11027 0 1 Mshowa
.99563 .12277 m
.99563 .12902 L
s
[(2)] .99563 .11027 0 1 Mshowa
.125 Mabswid
.0724 .12277 m
.0724 .12652 L
s
.12099 .12277 m
.12099 .12652 L
s
.16958 .12277 m
.16958 .12652 L
s
.21817 .12277 m
.21817 .12652 L
s
.31535 .12277 m
.31535 .12652 L
s
.36395 .12277 m
.36395 .12652 L
s
.41254 .12277 m
.41254 .12652 L
s
.46113 .12277 m
.46113 .12652 L
s
.55831 .12277 m
.55831 .12652 L
s
.6069 .12277 m
.6069 .12652 L
s
.65549 .12277 m
.65549 .12652 L
s
.70408 .12277 m
.70408 .12652 L
s
.80126 .12277 m
.80126 .12652 L
s
.84985 .12277 m
.84985 .12652 L
s
.89845 .12277 m
.89845 .12652 L
s
.94704 .12277 m
.94704 .12652 L
s
.25 Mabswid
0 .12277 m
1 .12277 L
s
.02381 .03607 m
.03006 .03607 L
s
[(0.8)] .01131 .03607 1 0 Mshowa
.02381 .20946 m
.03006 .20946 L
s
[(1.2)] .01131 .20946 1 0 Mshowa
.02381 .29616 m
.03006 .29616 L
s
[(1.4)] .01131 .29616 1 0 Mshowa
.02381 .38285 m
.03006 .38285 L
s
[(1.6)] .01131 .38285 1 0 Mshowa
.02381 .46955 m
.03006 .46955 L
s
[(1.8)] .01131 .46955 1 0 Mshowa
.02381 .55625 m
.03006 .55625 L
s
[(2)] .01131 .55625 1 0 Mshowa
.125 Mabswid
.02381 .05774 m
.02756 .05774 L
s
.02381 .07942 m
.02756 .07942 L
s
.02381 .10109 m
.02756 .10109 L
s
.02381 .14444 m
.02756 .14444 L
s
.02381 .16611 m
.02756 .16611 L
s
.02381 .18779 m
.02756 .18779 L
s
.02381 .23114 m
.02756 .23114 L
s
.02381 .25281 m
.02756 .25281 L
s
.02381 .27448 m
.02756 .27448 L
s
.02381 .31783 m
.02756 .31783 L
s
.02381 .33951 m
.02756 .33951 L
s
.02381 .36118 m
.02756 .36118 L
s
.02381 .40453 m
.02756 .40453 L
s
.02381 .4262 m
.02756 .4262 L
s
.02381 .44788 m
.02756 .44788 L
s
.02381 .49122 m
.02756 .49122 L
s
.02381 .5129 m
.02756 .5129 L
s
.02381 .53457 m
.02756 .53457 L
s
.02381 .0144 m
.02756 .0144 L
s
.02381 .57792 m
.02756 .57792 L
s
.02381 .59959 m
.02756 .59959 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.0243 .01472 m
.10155 .02225 L
.12585 .02759 L
.15015 .03425 L
.24733 .0724 L
.27162 .08439 L
.29592 .0972 L
.32021 .11073 L
.34451 .12494 L
.3688 .13974 L
.3931 .15508 L
.4174 .17091 L
.44169 .18718 L
.46599 .20385 L
.49028 .22088 L
.51458 .23824 L
.53887 .25589 L
.56317 .27381 L
.58746 .29197 L
.61176 .31035 L
.63605 .32894 L
.66035 .34772 L
.68465 .36666 L
.70894 .38576 L
.73324 .405 L
.75753 .42438 L
.78183 .44388 L
.80612 .46349 L
.83042 .48321 L
.85471 .50302 L
.87901 .52292 L
.9033 .54291 L
.9276 .56297 L
.9519 .58311 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`800?ooo`[oo`005Ooo1P00oooo1_oo000Eool0
0`00ooooo`03ool:003kool001Goo`03003ooooo00goo`P00??oo`000_oo0P001?oo00<00?ooool0
0ooo0P001Ooo00<00?ooool05Ooo1@00k_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03
003ooooo01[oo`@00>[oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3000Nool3003Wool0
0005ool00?ooool00009ool20005ool00`00ooooo`0Qool2003Uool00005ool00?ooool00008ool0
1000ooooo`001?oo00<00?ooool08ooo0`00h_oo00001Ooo003ooooo00002?oo00@00?ooool000Co
o`03003ooooo02Koo`<00=ooo`000_oo0P002_oo0P001Ooo00<00?ooool0:Ooo0P00gOoo000Eool0
0`00ooooo`0[ool3003Jool001Goo`8002ooo`800=Soo`005Ooo00<00?ooool0<?oo0`00eOoo000E
ool00`00ooooo`0cool3003Bool001Goo`03003ooooo03Koo`800=3oo`005Ooo00<00?ooool0>?oo
0`00cOoo000Eool00`00ooooo`0kool2003;ool001Goo`8003koo`800<Woo`005Ooo00<00?ooool0
>?oo0P001?oo0`000_oo0`00>?oo1000>?oo10000ooo00<00?ooool00_oo0`00>Ooo10000_oo000E
ool00`00ooooo`0gool01000ooooo`001_oo0P000ooo00<00?ooool0=ooo00<00?ooool0>Ooo00<0
0?ooool02_oo00<00?ooool0=_oo00<00?ooool00ooo000Eool00`00ooooo`0gool01000ooooo`00
2?oo00@00?ooool003Woo`03003ooooo03Woo`03003ooooo00[oo`03003ooooo03Ooo`03003ooooo
00;oo`005Ooo00<00?ooool0=ooo00@00?ooool000Soo`<003[oo`03003ooooo03Woo`03003ooooo
00Ooo`<003_oo`03003ooooo007oo`005Ooo00<00?ooool0=ooo00@00?ooool000Soo`03003ooooo
008003Soo`03003ooooo03Woo`03003ooooo00Ooo`03003ooooo03coo`40007oo`7oo`005Ooo0P00
>?oo00@00?ooool000Soo`03003ooooo00;oo`8003Coo`<003Woo`<000Woo`03003ooooo03Woo`04
003ooooo0002ool001Goo`03003ooooo03Soo`8000Woo`@000?oo`8003Coo`03003ooooo03Woo`03
003ooooo00Ooo`@003Woo`8000?oo`005Ooo00<00?ooool0C?oo0P00^_oo000Eool00`00ooooo`1>
ool2002hool001Goo`03003ooooo053oo`800;Koo`005Ooo00<00?ooool0D_oo0P00]?oo000>oooo
000A0002ool001Goo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[o
o`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00Koo`8000;oo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00_oo`03003ooooo007oo`005Ooo00<00?ooool0Eooo00<00?ooool0[_oo
000Eool00`00ooooo`1Hool2002^ool001Goo`03003ooooo05[oo`03003ooooo0:_oo`005Ooo00<0
0?ooool0Fooo0P00Zooo000Eool2001Nool2002Yool001Goo`03003ooooo05ooo`800:Ooo`005Ooo
00<00?ooool0HOoo00<00?ooool0Y?oo000Eool00`00ooooo`1Rool00`00ooooo`2Sool001Goo`03
003ooooo06?oo`03003ooooo0:;oo`005Ooo0P00IOoo0P00X_oo000Eool00`00ooooo`1Vool00`00
ooooo`2Oool001Goo`03003ooooo06Ooo`03003ooooo09koo`005Ooo00<00?ooool0J?oo0P00W_oo
000Eool00`00ooooo`1Zool2002Lool001Goo`03003ooooo06coo`8009[oo`005Ooo0P00Kooo00<0
0?ooool0Uooo000Eool00`00ooooo`1_ool00`00ooooo`2Fool001Goo`03003ooooo073oo`03003o
oooo09Goo`005Ooo00<00?ooool0LOoo0P00UOoo00000ooo000000020003ool00`00ooooo`02ool4
0004ool00`00ooooo`1cool00`00ooooo`2Bool000?oo`03003ooooo00Ooo`03003ooooo00Goo`03
003ooooo07Coo`03003ooooo097oo`000ooo00<00?ooool02?oo00<00?ooool01?oo0`00MOoo0P00
TOoo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`1gool2002?ool000?oo`03003o
oooo00[oo`03003ooooo00;oo`03003ooooo07Woo`8008goo`0000Coo`00000000Woo`04003ooooo
0004ool00`00ooooo`1kool00`00ooooo`2:ool000?oo`03003ooooo00Soo`8000Goo`03003ooooo
07coo`8008[oo`005Ooo00<00?ooool0O_oo00<00?ooool0Qooo000Eool20020ool00`00ooooo`26
ool001Goo`03003ooooo083oo`8008Koo`005Ooo00<00?ooool0P_oo00<00?ooool0Pooo000Eool0
0`00ooooo`23ool00`00ooooo`22ool001Goo`03003ooooo08Coo`03003ooooo087oo`005Ooo00<0
0?ooool0QOoo0P00POoo000Eool20028ool00`00ooooo`1nool001Goo`03003ooooo08Soo`03003o
oooo07goo`005Ooo00<00?ooool0ROoo0P00OOoo000Eool00`00ooooo`2;ool00`00ooooo`1jool0
01Goo`03003ooooo08coo`03003ooooo07Woo`005Ooo00<00?ooool0SOoo0P00NOoo000Eool2002@
ool00`00ooooo`1fool001Goo`03003ooooo093oo`8007Koo`005Ooo00<00?ooool0T_oo00<00?oo
ool0Looo000Eool00`00ooooo`2Cool2001cool00003ool00000008000?oo`03003ooooo00Coo`03
003ooooo00?oo`03003ooooo09Goo`03003ooooo073oo`000ooo00<00?ooool02Ooo00<00?ooool0
0ooo00<00?ooool0U_oo0P00L?oo0003ool00`00ooooo`06ool50004ool3002Hool00`00ooooo`1]
ool000?oo`03003ooooo00Koo`04003ooooo0005ool00`00ooooo`2Iool00`00ooooo`1/ool000?o
o`03003ooooo00Ooo`03003oo`0000Goo`03003ooooo09[oo`8006coo`0000Coo`00000000[oo`80
00Goo`03003ooooo09coo`03003ooooo06Woo`000ooo00<00?ooool02Ooo00<00?ooool00ooo00<0
0?ooool0WOoo00<00?ooool0J?oo000Eool2002Oool00`00ooooo`1Wool001Goo`03003ooooo09oo
o`8006Ooo`005Ooo00<00?ooool0XOoo00<00?ooool0I?oo000Eool00`00ooooo`2Rool00`00oooo
o`1Sool001Goo`03003ooooo0:?oo`8006?oo`005Ooo00<00?ooool0YOoo00<00?ooool0H?oo000E
ool2002Wool00`00ooooo`1Oool001Goo`03003ooooo0:Ooo`8005ooo`005Ooo00<00?ooool0ZOoo
00<00?ooool0G?oo000Eool00`00ooooo`2Zool00`00ooooo`1Kool001Goo`03003ooooo0:_oo`03
003ooooo05[oo`005Ooo00<00?ooool0[?oo00<00?ooool0FOoo000Eool2002^ool00`00ooooo`1H
ool001Goo`03003ooooo0:koo`03003ooooo05Ooo`005Ooo00<00?ooool0[ooo00<00?ooool0E_oo
000Eool00`00ooooo`2`ool2001Fool00003ool00000008000?oo`03003ooooo00?oo`8000Goo`03
003ooooo0;;oo`03003ooooo05?oo`000ooo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo
0;?oo`03003ooooo05;oo`000ooo00<00?ooool01ooo00@00?ooool000Coo`<00;Coo`8005;oo`00
0ooo00<00?ooool01ooo0`001Ooo00<00?ooool0]_oo00<00?ooool0Cooo0003ool00`00ooooo`07
ool00`00ooooo`05ool00`00ooooo`2gool2001?ool00004ool000000009ool00`00ooooo`05ool0
0`00ooooo`2iool00`00ooooo`1<ool000?oo`03003ooooo00Soo`<000Coo`03003ooooo0;[oo`03
003ooooo04_oo`005Ooo00<00?ooool0^ooo0P00Booo000Eool2002nool00`00ooooo`18ool001Go
o`03003ooooo0;koo`03003ooooo04Ooo`005Ooo00<00?ooool0_ooo00<00?ooool0A_oo000Eool0
0`00ooooo`30ool20016ool001Goo`03003ooooo0<;oo`03003ooooo04?oo`005Ooo00<00?ooool0
`ooo00<00?ooool0@_oo000Eool20035ool20012ool001Goo`03003ooooo0<Koo`03003ooooo03oo
o`005Ooo00<00?ooool0aooo00<00?ooool0?_oo000Eool00`00ooooo`38ool2000nool001Goo`03
003ooooo0<[oo`03003ooooo03_oo`005Ooo00<00?ooool0booo00<00?ooool0>_oo000Eool2003=
ool00`00ooooo`0iool001Goo`03003ooooo0<goo`03003ooooo03Soo`005Ooo00<00?ooool0c_oo
00<00?ooool0=ooo00000ooo000000020003ool00`00ooooo`03ool20005ool00`00ooooo`3?ool0
0`00ooooo`0fool000?oo`03003ooooo00Ooo`04003ooooo0004ool00`00ooooo`3@ool00`00oooo
o`0eool000?oo`03003ooooo00Ooo`04003ooooo0004ool3003Aool2000eool000?oo`03003ooooo
00Soo`8000Goo`03003ooooo0=?oo`03003ooooo03;oo`000ooo00<00?ooool01ooo00@00?ooool0
00Coo`03003ooooo0=Coo`03003ooooo037oo`0000Coo`00000000Woo`04003ooooo0004ool00`00
ooooo`3Eool2000aool000?oo`03003ooooo00Soo`8000Goo`03003ooooo0=Ooo`03003ooooo02ko
o`005Ooo00<00?ooool0f?oo00<00?ooool0;Ooo000Eool2003Jool00`00ooooo`0/ool001Goo`03
003ooooo0=[oo`8002coo`005Ooo00<00?ooool0g?oo00<00?ooool0:Ooo000Eool00`00ooooo`3M
ool00`00ooooo`0Xool001Goo`03003ooooo0=koo`03003ooooo02Ooo`005Ooo00<00?ooool0gooo
00<00?ooool09_oo000Eool2003Qool2000Vool001Goo`03003ooooo0>;oo`03003ooooo02?oo`00
5Ooo00<00?ooool0hooo00<00?ooool08_oo000Eool00`00ooooo`3Tool00`00ooooo`0Qool001Go
o`03003ooooo0>Goo`80027oo`005Ooo00<00?ooool0iooo00<00?ooool07_oo000Eool2003Yool0
0`00ooooo`0Mool001Goo`03003ooooo0>Woo`8001goo`005Ooo00<00?ooool0jooo00<00?ooool0
6_oo000Eool00`00ooooo`3/ool00`00ooooo`0Iool000goo`@000Coo`03003ooooo0>goo`03003o
oooo01Soo`003Ooo00<00?ooool01Ooo00<00?ooool0k_oo00<00?ooool05ooo000>ool00`00oooo
o`04ool3003_ool00`00ooooo`0Fool000ooo`03003ooooo00?oo`03003ooooo0?3oo`03003ooooo
01Goo`004?oo00<00?ooool00_oo00<00?ooool0lOoo00<00?ooool05?oo000=ool01000ooooo`00
1?oo00<00?ooool0l_oo0P005?oo000>ool20005ool00`00ooooo`3dool00`00ooooo`0Aool001Go
o`03003ooooo0?Goo`03003ooooo013oo`005Ooo0P00mooo0P004?oo000Eool00`00ooooo`3hool0
0`00ooooo`0=ool001Goo`03003ooooo0?Woo`03003ooooo00coo`005Ooo00<00?ooool0n_oo00<0
0?ooool02ooo000Eool00`00ooooo`3kool00`00ooooo`0:ool001Goo`03003ooooo0?coo`03003o
oooo00Woo`005Ooo0P00o_oo00<00?ooool02?oo000Eool00`00ooooo`3nool00`00ooooo`07ool0
01Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.162815, 0.674427, \
0.0076156, 0.00853674}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%113, %111, %109, %91, PlotRange \[Rule] {0.5, 2}, 
      Frame \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.206011 0.412023 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.2619 -0.0125 -9 -9 ]
[.2619 -0.0125 9 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -9 -9 ]
[.7381 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .0412 -18 -4.5 ]
[-0.0125 .0412 0 4.5 ]
[-0.0125 .12361 -18 -4.5 ]
[-0.0125 .12361 0 4.5 ]
[-0.0125 .20601 -6 -4.5 ]
[-0.0125 .20601 0 4.5 ]
[-0.0125 .28842 -18 -4.5 ]
[-0.0125 .28842 0 4.5 ]
[-0.0125 .37082 -18 -4.5 ]
[-0.0125 .37082 0 4.5 ]
[-0.0125 .45322 -18 -4.5 ]
[-0.0125 .45322 0 4.5 ]
[-0.0125 .53563 -18 -4.5 ]
[-0.0125 .53563 0 4.5 ]
[-0.0125 .61803 -6 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.2619 0 m
.2619 .00625 L
s
[(0.5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(1)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(1.5)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(2)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0412 m
.00625 .0412 L
s
[(0.6)] -0.0125 .0412 1 0 Mshowa
0 .12361 m
.00625 .12361 L
s
[(0.8)] -0.0125 .12361 1 0 Mshowa
0 .20601 m
.00625 .20601 L
s
[(1)] -0.0125 .20601 1 0 Mshowa
0 .28842 m
.00625 .28842 L
s
[(1.2)] -0.0125 .28842 1 0 Mshowa
0 .37082 m
.00625 .37082 L
s
[(1.4)] -0.0125 .37082 1 0 Mshowa
0 .45322 m
.00625 .45322 L
s
[(1.6)] -0.0125 .45322 1 0 Mshowa
0 .53563 m
.00625 .53563 L
s
[(1.8)] -0.0125 .53563 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(2)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .10301 m
.00375 .10301 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .30902 m
.00375 .30902 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .39142 m
.00375 .39142 L
s
0 .41202 m
.00375 .41202 L
s
0 .43262 m
.00375 .43262 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .51503 m
.00375 .51503 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
0 .0206 m
.00375 .0206 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.2619 .61178 m
.2619 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.7381 .61178 m
.7381 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0412 m
1 .0412 L
s
.99375 .12361 m
1 .12361 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .28842 m
1 .28842 L
s
.99375 .37082 m
1 .37082 L
s
.99375 .45322 m
1 .45322 L
s
.99375 .53563 m
1 .53563 L
s
.125 Mabswid
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .10301 m
1 .10301 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .30902 m
1 .30902 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .41202 m
1 .41202 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .51503 m
1 .51503 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.99625 .0206 m
1 .0206 L
s
.99625 0 m
1 0 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02429 .10331 m
.1 .11047 L
.12381 .11555 L
.14762 .12187 L
.24286 .15814 L
.26667 .16954 L
.29048 .18171 L
.31429 .19457 L
.3381 .20807 L
.3619 .22214 L
.38571 .23673 L
.40952 .25177 L
.43333 .26724 L
.45714 .28308 L
.48095 .29927 L
.50476 .31577 L
.52857 .33254 L
.55238 .34958 L
.57619 .36684 L
.6 .38431 L
.62381 .40198 L
.64762 .41983 L
.67143 .43783 L
.69524 .45599 L
.71905 .47428 L
.74286 .4927 L
.76667 .51123 L
.79048 .52987 L
.81429 .54861 L
.8381 .56744 L
.8619 .58636 L
.88571 .60536 L
s
.88571 .60536 m
.90154 .61803 L
s
[ .02 .02 ] 0 setdash
.0481 .04899 m
.0719 .05084 L
.09571 .05382 L
.11952 .05795 L
.14333 .0632 L
.16714 .06954 L
.19095 .07693 L
.21476 .08532 L
.23857 .09466 L
.26238 .1049 L
.28619 .11599 L
.31 .12787 L
.33381 .14049 L
.35762 .1538 L
.38143 .16774 L
.40524 .18227 L
.42905 .19735 L
.45286 .21292 L
.47667 .22894 L
.50048 .24538 L
.52429 .26221 L
.5481 .27938 L
.5719 .29686 L
.59571 .31464 L
.61952 .33268 L
.64333 .35096 L
.66714 .36945 L
.69095 .38815 L
.71476 .40702 L
.73857 .42606 L
.76238 .44525 L
.78619 .46457 L
.81 .48402 L
.83381 .50358 L
.85762 .52324 L
.88143 .54299 L
.90524 .56282 L
.92905 .58274 L
.95286 .60272 L
s
[ .002 .01 ] 0 setdash
.02429 .04842 m
.07143 .0525 L
.09524 .05747 L
.11905 .06422 L
.14286 .07256 L
.16667 .0823 L
.19048 .09328 L
.21429 .1053 L
.2381 .11824 L
.2619 .13197 L
.28571 .14636 L
.30952 .16134 L
.33333 .17683 L
.35714 .19275 L
.38095 .20906 L
.40476 .22571 L
.42857 .24265 L
.45238 .25987 L
.47619 .27731 L
.5 .29497 L
.52381 .31282 L
.54762 .33084 L
.57143 .34902 L
.59524 .36734 L
.61905 .38578 L
.64286 .40435 L
.66667 .42302 L
.69048 .44179 L
.71429 .46065 L
.7381 .4796 L
.7619 .49863 L
.78571 .51773 L
.80952 .5369 L
.83333 .55613 L
.85714 .57542 L
.88095 .59476 L
.90476 .61416 L
s
.90476 .61416 m
.90951 .61803 L
s
.8 g
[ ] 0 setdash
.2619 0 m
.30316 .03569 L
.34309 .07024 L
.3815 .10348 L
.42237 .13884 L
.46172 .17289 L
.49955 .20562 L
.53984 .24048 L
.57861 .27403 L
.61984 .3097 L
.65954 .34406 L
.69774 .3771 L
.73838 .41227 L
.77751 .44613 L
.81909 .48211 L
.85916 .51677 L
.89771 .55013 L
.93871 .58561 L
.97619 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001goo`8003Koo`8000Coo`03003ooooo00;oo`<003Goo`@003?oo`@000?oo`03003ooooo00;o
o`<003Goo`@000Woo`007?oo00@00?ooool003Coo`04003ooooo000;ool00`00ooooo`0dool00`00
ooooo`0dool00`00ooooo`0:ool00`00ooooo`0bool00`00ooooo`0:ool001coo`04003ooooo000d
ool01000ooooo`002ooo00<00?ooool0=?oo00<00?ooool0=?oo00<00?ooool02_oo00<00?ooool0
<ooo00<00?ooool02Ooo000Lool01000ooooo`00=?oo00@00?ooool000Soo`<003Ooo`03003ooooo
03Coo`03003ooooo00Ooo`<003Ooo`03003ooooo00Soo`007?oo00@00?ooool003Coo`04003ooooo
0008ool00`00ooooo`0gool00`00ooooo`0dool00`00ooooo`07ool00`00ooooo`0hool00`00oooo
o`07ool001coo`04003ooooo000dool01000ooooo`002?oo00<00?ooool0=Ooo0`00=?oo0`002Ooo
00<00?ooool0=Ooo00@00?ooool000Woo`007Ooo0P00=_oo0P002Ooo1000=_oo00<00?ooool0=?oo
00<00?ooool01ooo1000=Ooo0P002_oo003ooolQool00?ooob7oo`00oooo8Ooo000Hoom400000lii
0000002m0004ool001Soo`03003ooooo00?oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo
00Woo`03003ooooo00[oo`03003ooooo00Woo`03003>NOoo00Woo`03003ooooo00[oo`03003ooooo
00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo
00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo
00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00?oo`03003ooooo00;oo`006?oo00<0
0?ooool0@ooo0/ii^ooo00<00?ooool00_oo000Hool00`00ooooo`15ool00liiooooo`2hool00`00
ooooo`02ool001Soo`03003ooooo04Koo`03cWWooooo0;Ooo`03003ooooo00;oo`006?oo0P00B?oo
00?>NOooool0]Ooo0P001?oo000Hool00`00ooooo`18ool00liiooooo`2eool00`00ooooo`02ool0
01Soo`03003ooooo04Woo`;>NKGoo`03003ooooo00;oo`001Ooo0P001?oo00<00?ooool00ooo0P00
1Ooo00<00?ooool0Booo00?>NOooool0/_oo00<00?ooool00_oo0004ool01000ooooo`002?oo00@0
0?ooool000Coo`03003ooooo04coo`03cWWooooo0;7oo`03003ooooo00;oo`001?oo00@00?ooool0
00Soo`04003ooooo0004ool3001=ool00liiooooo`2_ool20004ool000Coo`04003ooooo0008ool3
0005ool00`00ooooo`1>ool00liiooooo`2_ool00`00ooooo`02ool000Coo`04003ooooo0008ool0
0`00ooooo`05ool00`00ooooo`03ool20004ool30013ool00liiooooo`2^ool00`00ooooo`02ool0
00Coo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`06ool200001?oo00000?oo1@00?ooo
00?>NOooool0[Ooo00<00?ooool00_oo0005ool2000:ool30004ool00`00ooooo`0@ool00`00oooo
o`07000gool2cWV]ool00`00ooooo`02ool001Soo`03003ooooo01Koo`03003ooooo00Goo`8003?o
o`03cWWooooo0:[oo`03003ooooo00;oo`006?oo0P006_oo00@00?ooool000?oo`@0033oo`03cWWo
oooo0:Soo`8000Coo`006?oo00<00?ooool07?oo00@00?ooool003Goo`03cWWooooo0:Soo`03003o
oooo00;oo`006?oo00<00?ooool07ooo00<00?ooool01_oo0`00:ooo00?>NOooool0Yooo00<00?oo
ool00_oo000Hool00`00ooooo`0Rool00`00ooooo`06ool3000Yool00liiooooo`2Vool00`00oooo
o`02ool001Soo`03003ooooo02Coo`8003;oo`03cWWooooo0:Goo`03003ooooo00;oo`006?oo0P00
:?oo0P002Ooo0P009Ooo00?>NOooool0Xooo0P001?oo000Hool00`00ooooo`0Zool20008ool2000T
ool00liiooooo`2Sool00`00ooooo`02ool001Soo`03003ooooo02goo`03003ooooo00Koo`8002?o
o`;>NJ?oo`03003ooooo00;oo`006?oo00<00?ooool0;Ooo00<00?ooool0;Ooo00?>NOooool0X?oo
00<00?ooool00_oo000Hool00`00ooooo`0`ool00`00ooooo`09ool00`00ooooo`0Oool00liioooo
o`2Oool00`00ooooo`02ool001Soo`03003ooooo03?oo`03003ooooo00Ooo`80023oo`03cWWooooo
09koo`03003ooooo00;oo`006?oo0P001?oo2P00<_oo0P007ooo00?>NOooool0W?oo0P001?oo000H
ool00`00ooooo`0=ool<000Lool2000Zool00liiooooo`2Lool00`00ooooo`02ool001Soo`03003o
oooo01Woo`<001coo`8000_oo`03003ooooo01[oo`03cWWooooo09_oo`03003ooooo00;oo`001Ooo
0P001?oo00<00?ooool00ooo0P001Ooo00<00?ooool07?oo10009_oo0`006_oo00?>NOooool0V_oo
00<00?ooool00_oo0004ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo023oo`D001Ko
o`03003ooooo00_oo`8001Woo`03cWWooooo09Woo`03003ooooo00;oo`001?oo00@00?ooool000So
o`04003ooooo0004ool3000Uool2000Gool00`00ooooo`0Tool2cWVHool20004ool000Coo`04003o
oooo0009ool20005ool00`00ooooo`0Wool3000mool00liiooooo`2Fool00`00ooooo`02ool000Co
o`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0:_oo0P005?oo0P003?oo0P006?oo00?>
NOooool0UOoo00<00?ooool00_oo0004ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo
02coo`<001Coo`03003ooooo00[oo`8001Ooo`03cWWooooo09Coo`03003ooooo00;oo`001Ooo0P00
2_oo0P001Ooo00<00?ooool0;ooo0P004_oo00<00?ooool03?oo0P005_oo00?>NOooool0Tooo00<0
0?ooool00_oo000Hool2000bool3000Bool00`00ooooo`0Rool00liiooooo`2Aool20004ool001So
o`03003ooooo03Coo`80027oo`03003ooooo01;oo`;>NI;oo`03003ooooo00;oo`006?oo00<00?oo
ool0=_oo0`003ooo0P003_oo0P005?oo00?>NOooool0Sooo00<00?ooool00_oo000Hool00`00oooo
o`0iool2000@ool00`00ooooo`0<ool00`00ooooo`0Bool00liiooooo`2>ool00`00ooooo`02ool0
01Soo`03003ooooo03_oo`8001koo`8001?oo`03cWWooooo08goo`03003ooooo00;oo`006?oo00<0
0?ooool0?Ooo0P003_oo0P008_oo00?>NOooool0S?oo00<00?ooool00_oo000Hool20010ool2000a
ool2cWV;ool20004ool001Soo`03003ooooo047oo`8000goo`03003ooooo00goo`8001;oo`03cWWo
oooo08Woo`03003ooooo00;oo`006?oo00<00?ooool0@ooo0P003_oo00<00?ooool03?oo0P004Ooo
00?>NOooool0R?oo00<00?ooool00_oo000Hool00`00ooooo`15ool2000<ool00`00ooooo`0>ool2
000@ool00liiooooo`27ool00`00ooooo`02ool001Soo`03003ooooo04Ooo`8000coo`80023oo`03
cWWooooo08Koo`03003ooooo00;oo`006?oo0P00B_oo0P00;Ooo00?>NOooool0Q?oo0P001?oo000H
ool00`00ooooo`1;ool2000;ool00`00ooooo`0>ool00`00ooooo`0=ool2cWV5ool00`00ooooo`02
ool001Soo`03003ooooo04goo`8000_oo`03003ooooo00goo`8000ooo`03cWWooooo08;oo`03003o
oooo00;oo`004?oo10001?oo00<00?ooool0Cooo0P002_oo00<00?ooool03_oo00<00?ooool03Ooo
00?>NOooool0POoo00<00?ooool00_oo000Bool00`00ooooo`03ool00`00ooooo`1Aool2000:ool0
0`00ooooo`0=ool2000>ool00liiooooo`20ool00`00ooooo`02ool001;oo`03003ooooo00?oo`<0
05?oo`8002Woo`03cWWooooo07koo`8000Coo`004_oo00<00?ooool00ooo00<00?ooool0EOoo00<0
0?ooool02?oo00<00?ooool07?oo00?>NOooool0O_oo00<00?ooool00_oo000Bool00`00ooooo`03
ool00`00ooooo`1Fool2000Jool00`00ooooo`0;ool2cWUnool00`00ooooo`02ool0013oo`<000Go
o`03003ooooo05Soo`8000Soo`8000ooo`03003ooooo00coo`03cWWooooo07_oo`03003ooooo00;o
o`004_oo00<00?ooool00ooo00<00?ooool0F_oo0P002Ooo00<00?ooool03?oo0P003Ooo00?>NOoo
ool0N_oo00<00?ooool00_oo000Hool00`00ooooo`1Lool20007ool00`00ooooo`0>ool00`00oooo
o`0;ool00liiooooo`1iool00`00ooooo`02ool001Soo`8005ooo`03003ooooo00Koo`03003ooooo
01_oo`03cWWooooo07Ooo`8000Coo`006?oo00<00?ooool0Gooo0P001ooo00<00?ooool06ooo0/ii
N?oo00<00?ooool00_oo000Hool00`00ooooo`1Qool00`00ooooo`06ool00`00ooooo`0=ool2000<
ool00liiooooo`1eool00`00ooooo`02ool001Soo`03003ooooo06;oo`8001Soo`8000_oo`03cWWo
oooo07Coo`03003ooooo00;oo`006?oo00<00?ooool0I?oo00<00?ooool01Ooo0P004?oo0P002_oo
00?>NOooool0Looo00<00?ooool00_oo000Hool2001Vool2000Tool00liiooooo`1aool20004ool0
01Soo`03003ooooo06Ooo`03003ooooo00Goo`03003ooooo01[oo`03cWWooooo077oo`03003ooooo
00;oo`006?oo00<00?ooool0J?oo0P006Ooo00<00?ooool01ooo00?>NOooool0L?oo00<00?ooool0
0_oo000Hool00`00ooooo`1Zool00`00ooooo`04ool2000Aool00`00ooooo`07ool2cWU`ool00`00
ooooo`02ool001Soo`03003ooooo06_oo`8000Ooo`03003ooooo00koo`03003ooooo00Soo`03cWWo
oooo06goo`03003ooooo00;oo`006?oo0P00K_oo00<00?ooool01?oo00<00?ooool03ooo00<00?oo
ool02?oo00?>NOooool0Jooo0P001?oo000Hool00`00ooooo`1^ool20006ool00`00ooooo`0>ool0
0`00ooooo`08ool00liiooooo`1[ool00`00ooooo`02ool001Soo`03003ooooo073oo`03003ooooo
00Coo`03003ooooo01Woo`03cWWooooo06[oo`03003ooooo00;oo`006?oo00<00?ooool0LOoo0P00
1_oo00<00?ooool06?oo00?>NOooool0JOoo00<00?ooool00_oo0004ool40003ool00`00ooooo`02
ool40004ool00`00ooooo`1cool00`00ooooo`0Eool00`00ooooo`07ool2cWUYool00`00ooooo`02
ool000Koo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo07Coo`8000Goo`8000ooo`8000Wo
o`03cWWooooo06Koo`03003ooooo00;oo`001_oo00<00?ooool02?oo00<00?ooool01?oo0`00M_oo
0P005_oo0P002?oo00?>NOooool0I?oo0P001?oo0006ool00`00ooooo`09ool00`00ooooo`03ool0
0`00ooooo`1hool20004ool00`00ooooo`0?ool00`00ooooo`06ool00liiooooo`1Tool00`00oooo
o`02ool000Koo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo07[oo`03003ooooo01goo`03
cWWooooo06?oo`03003ooooo00;oo`001?oo0`002Ooo00@00?ooool000Coo`03003ooooo07_oo`03
003ooooo00;oo`8001Woo`;>NF?oo`03003ooooo00;oo`001_oo00<00?ooool02?oo0P001Ooo00<0
0?ooool0O?oo00<00?ooool05Ooo00<00?ooool01_oo00?>NOooool0H?oo00<00?ooool00_oo000H
ool2001nool20004ool00`00ooooo`0?ool20007ool00liiooooo`1Nool20004ool001Soo`03003o
oooo07ooo`03003ooooo00?oo`03003ooooo00ooo`03003ooooo00Goo`03cWWooooo05koo`03003o
oooo00;oo`006?oo00<00?ooool0P?oo00<00?ooool00ooo00<00?ooool03ooo00<00?ooool01Ooo
00?>NOooool0GOoo00<00?ooool00_oo000Hool00`00ooooo`21ool20005ool00`00ooooo`0Fool0
0liiooooo`1Lool00`00ooooo`02ool001Soo`03003ooooo08?oo`03003ooooo00;oo`03003ooooo
01Ooo`03cWWooooo05_oo`03003ooooo00;oo`006?oo0P00QOoo0P001?oo0P004?oo00<00?ooool0
1?oo0/iiF_oo0P001?oo000Hool00`00ooooo`26ool00`00ooooo`0Dool20006ool00liiooooo`1H
ool00`00ooooo`02ool001Soo`03003ooooo08Ooo`8000Coo`03003ooooo00ooo`03003ooooo00Co
o`03cWWooooo05Ooo`03003ooooo00;oo`006?oo00<00?ooool0ROoo00<00?ooool05?oo00<00?oo
ool01?oo00?>NOooool0E_oo00<00?ooool00_oo000Hool00`00ooooo`2:ool20003ool2000Gool0
0liiooooo`1Eool00`00ooooo`02ool001Soo`03003ooooo08coo`03003ooooo01[oo`03cWWooooo
05Coo`03003ooooo00;oo`006?oo0P00S_oo00<00?ooool00_oo00<00?ooool03ooo00<00?ooool0
0ooo00?>NOooool0D_oo0P001?oo000Hool00`00ooooo`2>ool00`00ooooo`0Dool00`00ooooo`03
ool00liiooooo`1Bool00`00ooooo`02ool001Soo`03003ooooo08ooo`8000?oo`80013oo`8000Co
o`03cWWooooo057oo`03003ooooo00;oo`001?oo10000ooo00<00?ooool01?oo00<00?ooool00ooo
00<00?ooool0TOoo00<00?ooool05?oo00<00?ooool00_oo0/iiDOoo00<00?ooool00_oo0006ool0
0`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2Bool00`00ooooo`02ool00`00ooooo`0Eool0
0liiooooo`1>ool00`00ooooo`02ool000Koo`03003ooooo00Koo`D000Coo`<009?oo`8000Coo`03
003ooooo01Coo`03cWWooooo04coo`8000Coo`001_oo00<00?ooool01_oo00@00?ooool000Goo`03
003ooooo09Goo`05003oooooool0000Gool00liiooooo`1<ool00`00ooooo`02ool000Koo`03003o
oooo00Ooo`03003oo`0000Goo`03003ooooo09Koo`8000Coo`03003ooooo00goo`03003ooooo00?o
o`03cWWooooo04_oo`03003ooooo00;oo`001?oo0`002_oo0P001Ooo00<00?ooool0V?oo00D00?oo
ooooo`00013oo`8000Coo`03cWWooooo04[oo`03003ooooo00;oo`001_oo00<00?ooool02Ooo00<0
0?ooool00ooo00<00?ooool0VOoo0P000ooo00<00?ooool03_oo0P000ooo00?>NOooool0BOoo00<0
0?ooool00_oo000Hool2002Lool01000ooooo`004Ooo00<00?ooool00_oo00?>NOooool0Aooo0P00
1?oo000Hool00`00ooooo`2Lool01@00oooooooo00005Ooo00?>NOooool0Aooo00<00?ooool00_oo
000Hool00`00ooooo`2Mool20002ool00`00ooooo`0Cool2cWU7ool00`00ooooo`02ool001Soo`03
003ooooo09ooo`05003oooooool0000Aool01@00oooooooocWT0A_oo00<00?ooool00_oo000Hool0
0`00ooooo`2Pool00`00ooooo`0Cool01@00oooooooocWT0AOoo00<00?ooool00_oo000Hool00`00
ooooo`2Qool01000oooooooo0P004?oo0P000_oo00?>NOooool0@_oo00<00?ooool00_oo000Hool2
002Sool2000Eool01000ooooolii@_oo0P001?oo000Hool00`00ooooo`2Tool01@00oooooooo0000
5?oo00?>NOooool0@?oo00<00?ooool00_oo000Hool00`00ooooo`2Uool00`00ooooo`0Fool2cWU0
ool00`00ooooo`02ool001Soo`03003ooooo0:Koo`8000;oo`80017oo`04003ooooocWToool00`00
ooooo`02ool001Soo`03003ooooo0:Soo`03003ooooo01?oo`04003ooooocWTnool00`00ooooo`02
ool001Soo`800:[oo`8000;oo`03003ooooo00ooo`04003ooooocWTlool20004ool001Soo`03003o
oooo0:_oo`03003ooooo01;oo`03003ooooo00;>NC_oo`03003ooooo00;oo`006?oo00<00?ooool0
[?oo00@00?ooool0017oo`05003oooooooo>N@0jool00`00ooooo`02ool000Coo`@000?oo`03003o
oooo00?oo`8000Goo`03003ooooo0:goo`03003ooooo01Koo`03cWWooooo03Ooo`03003ooooo00;o
o`001_oo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo0:koo`8000;oo`03003ooooo01;o
o`03cWWooooo03Koo`03003ooooo00;oo`001_oo00<00?ooool01ooo00@00?ooool000Coo`<00;3o
o`03003ooooo01;oo`04003ooooocWTfool20004ool000Koo`03003ooooo00Ooo`<000Goo`03003o
oooo0;7oo`04003ooooo000Aool01000ooooolii=_oo00<00?ooool00_oo0006ool00`00ooooo`07
ool00`00ooooo`05ool00`00ooooo`2bool00`00ooooo`0Bool200000ooocWWoo`0dool00`00oooo
o`02ool000Coo`<000Woo`03003ooooo00Goo`03003ooooo0;?oo`03003ooooo0080017oo`04003o
oliicWTcool00`00ooooo`02ool000Koo`03003ooooo00Soo`<000Coo`03003ooooo0;Coo`8000?o
o`03003ooooo017oo`03cWWooooo033oo`03003ooooo00;oo`006?oo00<00?ooool0]_oo00@00?oo
ool001Coo`03cWWooooo02ooo`03003ooooo00;oo`006?oo0P00^?oo00D00?ooooooo`0001?oo`03
cWWooooo02goo`8000Coo`006?oo00<00?ooool0^?oo0P000_oo00<00?ooool03_oo00@00?ooooo>
NBooo`03003ooooo00;oo`006?oo00<00?ooool0^_oo00D00?ooooooo`0000ooo`800003ooo>NOoo
02goo`03003ooooo00;oo`006?oo00<00?ooool0^ooo00@00?ooool0017oo`03003oolii02goo`03
003ooooo00;oo`006?oo00<00?ooool0_?oo0P000_oo00<00?ooool03_oo00@00?oocWW>NB_oo`03
003ooooo00;oo`006?oo0P00_ooo00@00?ooool001?oo`03cWWooooo02Ooo`8000Coo`006?oo00<0
0?ooool0_ooo0P000_oo00<00?ooool04?oo00?>NOooool09ooo00<00?ooool00_oo000Hool00`00
ooooo`31ool00`00ool0000Aool00`00ooo>N@0Xool00`00ooooo`02ool001Soo`03003ooooo0<;o
o`800003ool00?oo00ooo`03003oolii02Ooo`03003ooooo00;oo`006?oo00<00?ooool0a?oo00<0
0?ooool04?oo0P000/ii9Ooo00<00?ooool00_oo000Hool20036ool01000ool000004?oo00<00?oo
cWT08ooo0P001?oo000Hool00`00ooooo`36ool00`00ooooo`0Cool00liiooooo`0Qool00`00oooo
o`02ool001Soo`03003ooooo0<Ooo`800003ool00?oo017oo`03cWWooooo023oo`03003ooooo00;o
o`006?oo00<00?ooool0bOoo00<00?ooool04_oo0/ii8?oo00<00?ooool00_oo0004ool40003ool0
0`00ooooo`03ool20005ool00`00ooooo`3:ool01000ool000003ooo00@00?ooooo>NAooo`03003o
oooo00;oo`001_oo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo0<_oo`03003ooooo013o
o`04003ooooocWTNool00`00ooooo`02ool000Koo`03003ooooo00Ooo`04003ooooo0004ool3003<
ool01000ooooo`003ooo0P0000?ooliiool06ooo0P001?oo0006ool00`00ooooo`08ool20005ool0
0`00ooooo`3=ool2000Bool01000ooo>NLii6ooo00<00?ooool00_oo0006ool00`00ooooo`07ool0
1000ooooo`001?oo00<00?ooool0cooo00@00?oo000001;oo`03cWWooooo01Soo`03003ooooo00;o
o`001?oo0`002Ooo00@00?ooool000Coo`03003ooooo0=3oo`03003ooooo01?oo`03cWWooooo01Oo
o`03003ooooo00;oo`001_oo00<00?ooool02?oo0P001Ooo00<00?ooool0dOoo00@00?ooool0013o
o`03003oolii01Soo`03003ooooo00;oo`006?oo0P00dooo00<00?ooool04Ooo00<00?oocWT05_oo
0P001?oo000Hool00`00ooooo`3Cool200000ooo003oo`0?ool00`00ooo>N@0Fool00`00ooooo`02
ool001Soo`03003ooooo0=Goo`03003ooooo013oo`03003oolii01Goo`03003ooooo00;oo`006?oo
00<00?ooool0e_oo00@00?oo000000ooo`03003oolii01Coo`03003ooooo00;oo`006?oo00<00?oo
ool0eooo00<00?ooool04_oo00?>NOooool04Ooo00<00?ooool00_oo000Hool2003Iool01000oooo
o`004Ooo00?>NOooool03ooo0P001?oo000Hool00`00ooooo`3Iool2000Bool00`00cWWoo`0@ool0
0`00ooooo`02ool001Soo`03003ooooo0=_oo`04003oo`00000?ool00`00cWWoo`0?ool00`00oooo
o`02ool001Soo`03003ooooo0=coo`03003ooooo013oo`03003>NOoo00koo`03003ooooo00;oo`00
6?oo00<00?ooool0gOoo0P0000?oo`00ool03ooo00?>NOooool03?oo00<00?ooool00_oo000Hool0
0`00ooooo`3Oool00`00ooooo`0@ool2cWT<ool00`00ooooo`02ool001Soo`800>7oo`03003oo`00
017oo`03cWWooooo00Soo`8000Coo`006?oo00<00?ooool0hOoo0P004_oo00?>NOooool02?oo00<0
0?ooool00_oo000Hool00`00ooooo`3Sool3000@ool00liiooooo`07ool00`00ooooo`02ool0013o
o`@000Coo`03003ooooo00?oo`03003ooooo03_oo`03003ooooo03_oo`03003ooooo03[oo`03003o
oooo02Goo`03003ooooo013oo`;>N@03ool00?oo00Coo`03003ooooo00;oo`004?oo00<00?ooool0
1Ooo00<00?ooool00ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool0
2_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool0
2Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool0
2Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00@00?oooooo
o`@000Goo`03003ooooo00Soo`03cWT00?oo00Coo`03003ooooo00;oo`004Ooo00<00?ooool01?oo
o@0000?>N@00000010001?oo000Bool00`00ooooo`3oool<ool001?oo`03003ooooo0?ooo`_oo`00
4?oo00@00?ooool00?ooo`goo`004Ooo0P00oooo3_oo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.248132, 0.385382, \
0.00810174, 0.00936347}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<stablePi2.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("stablePi2.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%583, %597, %600, %614, PlotRange \[Rule] {0.5, 2}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -0.206011 0.412023 [
[.2619 .19351 -9 -9 ]
[.2619 .19351 9 0 ]
[.5 .19351 -3 -9 ]
[.5 .19351 3 0 ]
[.7381 .19351 -9 -9 ]
[.7381 .19351 9 0 ]
[.97619 .19351 -3 -9 ]
[.97619 .19351 3 0 ]
[.01131 .0412 -18 -4.5 ]
[.01131 .0412 0 4.5 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .28842 -18 -4.5 ]
[.01131 .28842 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .45322 -18 -4.5 ]
[.01131 .45322 0 4.5 ]
[.01131 .53563 -18 -4.5 ]
[.01131 .53563 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .20601 m
.2619 .21226 L
s
[(0.5)] .2619 .19351 0 1 Mshowa
.5 .20601 m
.5 .21226 L
s
[(1)] .5 .19351 0 1 Mshowa
.7381 .20601 m
.7381 .21226 L
s
[(1.5)] .7381 .19351 0 1 Mshowa
.97619 .20601 m
.97619 .21226 L
s
[(2)] .97619 .19351 0 1 Mshowa
.125 Mabswid
.07143 .20601 m
.07143 .20976 L
s
.11905 .20601 m
.11905 .20976 L
s
.16667 .20601 m
.16667 .20976 L
s
.21429 .20601 m
.21429 .20976 L
s
.30952 .20601 m
.30952 .20976 L
s
.35714 .20601 m
.35714 .20976 L
s
.40476 .20601 m
.40476 .20976 L
s
.45238 .20601 m
.45238 .20976 L
s
.54762 .20601 m
.54762 .20976 L
s
.59524 .20601 m
.59524 .20976 L
s
.64286 .20601 m
.64286 .20976 L
s
.69048 .20601 m
.69048 .20976 L
s
.78571 .20601 m
.78571 .20976 L
s
.83333 .20601 m
.83333 .20976 L
s
.88095 .20601 m
.88095 .20976 L
s
.92857 .20601 m
.92857 .20976 L
s
.25 Mabswid
0 .20601 m
1 .20601 L
s
.02381 .0412 m
.03006 .0412 L
s
[(0.6)] .01131 .0412 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.8)] .01131 .12361 1 0 Mshowa
.02381 .28842 m
.03006 .28842 L
s
[(1.2)] .01131 .28842 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(1.4)] .01131 .37082 1 0 Mshowa
.02381 .45322 m
.03006 .45322 L
s
[(1.6)] .01131 .45322 1 0 Mshowa
.02381 .53563 m
.03006 .53563 L
s
[(1.8)] .01131 .53563 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(2)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .41202 m
.02756 .41202 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.02381 .0206 m
.02756 .0206 L
s
.02381 0 m
.02756 0 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.8 g
.5 Mabswid
.2619 0 m
.30316 .03569 L
.34309 .07024 L
.3815 .10348 L
.42237 .13884 L
.46172 .17289 L
.49955 .20562 L
.53984 .24048 L
.57861 .27403 L
.61984 .3097 L
.65954 .34406 L
.69774 .3771 L
.73838 .41227 L
.77751 .44613 L
.81909 .48211 L
.85916 .51677 L
.89771 .55013 L
.93871 .58561 L
.97619 .61803 L
s
0 g
.02429 .10331 m
.1 .11047 L
.12381 .11555 L
.14762 .12187 L
.24286 .15814 L
.26667 .16954 L
.29048 .18171 L
.31429 .19457 L
.3381 .20807 L
.3619 .22214 L
.38571 .23673 L
.40952 .25177 L
.43333 .26724 L
.45714 .28308 L
.48095 .29927 L
.50476 .31577 L
.52857 .33254 L
.55238 .34958 L
.57619 .36684 L
.6 .38431 L
.62381 .40198 L
.64762 .41983 L
.67143 .43783 L
.69524 .45599 L
.71905 .47428 L
.74286 .4927 L
.76667 .51123 L
.79048 .52987 L
.81429 .54861 L
.8381 .56744 L
.8619 .58636 L
.88571 .60536 L
s
.88571 .60536 m
.90154 .61803 L
s
[ .02 .02 ] 0 setdash
.0481 .04899 m
.0719 .05084 L
.09571 .05382 L
.11952 .05795 L
.14333 .0632 L
.16714 .06954 L
.19095 .07693 L
.21476 .08532 L
.23857 .09466 L
.26238 .1049 L
.28619 .11599 L
.31 .12787 L
.33381 .14049 L
.35762 .1538 L
.38143 .16774 L
.40524 .18227 L
.42905 .19735 L
.45286 .21292 L
.47667 .22894 L
.50048 .24538 L
.52429 .26221 L
.5481 .27938 L
.5719 .29686 L
.59571 .31464 L
.61952 .33268 L
.64333 .35096 L
.66714 .36945 L
.69095 .38815 L
.71476 .40702 L
.73857 .42606 L
.76238 .44525 L
.78619 .46457 L
.81 .48402 L
.83381 .50358 L
.85762 .52324 L
.88143 .54299 L
.90524 .56282 L
.92905 .58274 L
.95286 .60272 L
s
[ .002 .01 ] 0 setdash
.02429 .04842 m
.07143 .0525 L
.09524 .05747 L
.11905 .06422 L
.14286 .07256 L
.16667 .0823 L
.19048 .09328 L
.21429 .1053 L
.2381 .11824 L
.2619 .13197 L
.28571 .14636 L
.30952 .16134 L
.33333 .17683 L
.35714 .19275 L
.38095 .20906 L
.40476 .22571 L
.42857 .24265 L
.45238 .25987 L
.47619 .27731 L
.5 .29497 L
.52381 .31282 L
.54762 .33084 L
.57143 .34902 L
.59524 .36734 L
.61905 .38578 L
.64286 .40435 L
.66667 .42302 L
.69048 .44179 L
.71429 .46065 L
.7381 .4796 L
.7619 .49863 L
.78571 .51773 L
.80952 .5369 L
.83333 .55613 L
.85714 .57542 L
.88095 .59476 L
.90476 .61416 L
s
.90476 .61416 m
.90951 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001Goo`8003ooo`03cWWooooo0<Ooo`005Ooo00<00?ooool0?ooo00?>NOooool0a_oo000Eool0
0`00ooooo`10ool2cWW6ool001Goo`03003ooooo04;oo`03cWWooooo0<?oo`005Ooo00<00?ooool0
@ooo00?>NOooool0`_oo000Eool20015ool00liiooooo`31ool001Goo`03003ooooo04Goo`03cWWo
oooo0<3oo`005Ooo00<00?ooool0A_oo0/ii`?oo000Eool00`00ooooo`18ool00liiooooo`2mool0
00;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo04Woo`03cWWooooo0;coo`0000Goo`00
ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1:ool00liiooooo`2kool00005ool00?ooool0
0008ool01000ooooo`001?oo0`00Booo00?>NOooool0^_oo00001Ooo003ooooo00002?oo0`001Ooo
00<00?ooool0C?oo00?>NOooool0^Ooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo0P0000?o
o`0000000_oo1P00@ooo00?>NOooool0^?oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<0
0?ooool01?oo00@00?ooool000;oo`8000;oo`@003coo`;>NKSoo`000_oo0P002_oo0`001?oo00<0
0?ooool03Ooo0P0000?oo`00ool00`00>ooo00?>NOooool0]Ooo000Eool00`00ooooo`0Aool01000
ooooo`001?oo0P00=_oo00?>NOooool0]?oo000Eool2000Hool01000oooooooo1000<ooo00?>NOoo
ool0/ooo000Eool00`00ooooo`0Jool00`00ooooo`0fool00liiooooo`2bool001Goo`03003ooooo
01goo`03003ooooo00Coo`<002goo`03cWWooooo0;7oo`005Ooo00<00?ooool07Ooo00<00?ooool0
1ooo0`00:ooo00?>NOooool0/?oo000Eool00`00ooooo`0Pool00`00ooooo`0cool00liiooooo`2_
ool001Goo`8002Coo`03003ooooo00Soo`8002Ooo`03cWWooooo0:koo`005Ooo00<00?ooool09_oo
00<00?ooool01ooo0P009_oo00?>NOooool0[Ooo000Eool00`00ooooo`0Yool00`00ooooo`06ool2
000Uool2cWV]ool001Goo`03003ooooo02Woo`03003ooooo02ooo`03cWWooooo0:[oo`005Ooo00<0
0?ooool0;?oo00<00?ooool02Ooo0P008_oo00?>NOooool0ZOoo000Eool00`00ooooo`0_ool00`00
ooooo`08ool2000Qool00liiooooo`2Xool001Goo`H002coo`03003ooooo00[oo`80023oo`03cWWo
oooo0:Ooo`005Ooo00<00?ooool00ooo2P009Ooo00<00?ooool0:_oo00?>NOooool0Y_oo000Eool0
0`00ooooo`0=ool8000Oool00`00ooooo`0;ool00`00ooooo`0Kool2cWVVool000;oo`8000Coo`03
003ooooo00?oo`8000Goo`03003ooooo01Goo`D001_oo`03003ooooo00_oo`8001goo`03cWWooooo
0:?oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Jool4000Iool2000<ool2
000Lool00liiooooo`2Rool00005ool00?ooool00008ool01000ooooo`001?oo0`007_oo0`009_oo
00<00?ooool06_oo00?>NOooool0XOoo00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool08Ooo
0P005ooo0P00:Ooo00?>NOooool0X?oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003o
oooo02?oo`<001Ooo`03003ooooo00coo`03003ooooo01Ooo`;>NJ3oo`0000Goo`00ooooo`0000So
o`04003ooooo0004ool00`00ooooo`0Vool3000Dool00`00ooooo`0=ool2000Iool00liiooooo`2M
ool000;oo`8000[oo`8000Goo`03003ooooo02Woo`8001Goo`03003ooooo00coo`8001Soo`03cWWo
oooo09coo`005Ooo00<00?ooool0:ooo0`008ooo00<00?ooool05_oo00?>NOooool0Vooo000Eool2
000_ool2000Cool00`00ooooo`0Uool00liiooooo`2Jool001Goo`03003ooooo033oo`<001;oo`03
003ooooo00goo`03003ooooo01Coo`03cWWooooo09Woo`005Ooo00<00?ooool0<ooo0`004?oo00<0
0?ooool03Ooo0P005Ooo00?>NOooool0V?oo000Eool00`00ooooo`0fool2000@ool2000>ool00`00
ooooo`0Cool2cWVHool001Goo`03003ooooo03Soo`8001ooo`8001Goo`03cWWooooo09Goo`005Ooo
00<00?ooool0>_oo0P003ooo00<00?ooool08ooo00?>NOooool0U?oo000Eool2000hool20003ool2
0004ool3000]ool00liiooooo`07ool4000gool40003ool00`00ooooo`02ool3000hool40006ool0
01Goo`03003ooooo03Koo`04003ooooo0004ool20005ool00`00ooooo`06ool00`00ooooo`0>ool0
0`00ooooo`0Aool00liiooooo`08ool00`00ooooo`0hool00`00ooooo`0:ool00`00ooooo`0eool0
0`00ooooo`07ool001Goo`03003ooooo03Koo`04003ooooo0006ool20003ool00`00ooooo`08ool0
0`00ooooo`0=ool00`00ooooo`0Aool00liiooooo`07ool00`00ooooo`0hool00`00ooooo`0:ool0
0`00ooooo`0fool00`00ooooo`06ool001Goo`03003ooooo03Koo`04003ooooo0008ool3000<ool0
0`00ooooo`0=ool2000Bool00liiooooo`06ool00`00ooooo`0hool00`00ooooo`07ool3000jool0
0`00ooooo`05ool001Goo`03003ooooo03Koo`04003ooooo0008ool01000ool000003Ooo00<00?oo
ool03Ooo00<00?ooool04?oo00?>NOooool01Ooo00<00?ooool0>?oo00<00?ooool01ooo00<00?oo
ool0>ooo00<00?ooool01?oo000Eool2000gool01000ooooo`002?oo00@00?ooooooo`8000coo`03
003ooooo023oo`03cWWooooo00;oo`<003Soo`<000Woo`03003ooooo03Soo`04003ooooo0006ool0
01Goo`03003ooooo03Ooo`8000Woo`@000;oo`8000coo`03003ooooo00ooo`03003ooooo00goo`03
cWWooooo00?oo`03003ooooo03Soo`03003ooooo00Ooo`@003Soo`8000Ooo`005Ooo00<00?ooool0
B_oo0P007Ooo0P003_oo0/iiSOoo000Eool00`00ooooo`1<ool00`00ooooo`09ool2000Aool00`00
ooooo`0=ool00liiooooo`2:ool001Goo`03003ooooo04goo`8001goo`8000koo`03cWWooooo08Wo
o`005Ooo00<00?ooool0Cooo0P002_oo0P008?oo00?>NOooool0R?oo000>oon800000lii00000027
000001Goo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00Goo`8000?oo`03003ooooo00?oo`03003ooooo00Co
o`03003ooooo00Woo`8000_oo`03003>NOoo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003o
oooo00[oo`03003ooooo00[oo`03003ooooo00Goo`005Ooo00<00?ooool0EOoo0P002_oo00<00?oo
ool03_oo00<00?ooool02ooo0/iiQ_oo000Eool00`00ooooo`1Gool20008ool00`00ooooo`0?ool2
000=ool00liiooooo`23ool001Goo`03003ooooo05Woo`03003ooooo00Ooo`03003ooooo00ooo`80
00coo`03cWWooooo08;oo`005Ooo0P00Fooo0P002?oo00<00?ooool07Ooo00?>NOooool0POoo000E
ool00`00ooooo`1Lool00`00ooooo`07ool00`00ooooo`0Lool00liiooooo`20ool001Goo`03003o
oooo05goo`8001_oo`03003ooooo00Woo`;>NH3oo`005Ooo00<00?ooool0Gooo00<00?ooool01ooo
00<00?ooool03ooo0P002ooo00?>NOooool0OOoo000Eool00`00ooooo`1Pool2000Kool2000:ool0
0liiooooo`1lool001Goo`03003ooooo06;oo`8000Ooo`8001;oo`03003ooooo00Soo`03cWWooooo
07_oo`005Ooo0P00IOoo0P006Ooo00<00?ooool02Ooo0/iiNooo000Eool00`00ooooo`1Vool00`00
ooooo`05ool00`00ooooo`0Lool00liiooooo`1hool001Goo`03003ooooo06Ooo`8002Goo`03cWWo
oooo07Ooo`005Ooo00<00?ooool0JOoo00<00?ooool01Ooo00<00?ooool03ooo00<00?ooool02Ooo
00?>NOooool0M_oo000Eool00`00ooooo`1Zool2000Hool2000:ool00liiooooo`1eool001Goo`03
003ooooo06coo`03003ooooo00Coo`80017oo`8000Woo`;>NGGoo`005Ooo0P00K_oo0P006Ooo00<0
0?ooool02?oo00?>NOooool0L_oo000Eool00`00ooooo`1_ool00`00ooooo`04ool00`00ooooo`0K
ool00liiooooo`1aool001Goo`03003ooooo073oo`03003ooooo02;oo`03cWWooooo073oo`0000?o
o`0000000P000ooo00<00?ooool00_oo10001?oo00<00?ooool0LOoo0P001Ooo0P004_oo00<00?oo
ool01ooo00?>NOooool0Kooo0003ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`1cool0
0`00ooooo`05ool00`00ooooo`0?ool00`00ooooo`07ool00liiooooo`1^ool000?oo`03003ooooo
00Soo`03003ooooo00Coo`<007Coo`8000Goo`03003ooooo013oo`8000Soo`03cWWooooo06goo`00
0ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0M_oo00<00?ooool01Ooo00<00?ooool0
3ooo00<00?ooool01_oo0/iiKOoo0003ool00`00ooooo`0:ool00`00ooooo`02ool00`00ooooo`1g
ool20005ool00`00ooooo`0Jool00liiooooo`1Zool00004ool000000009ool01000ooooo`001?oo
00<00?ooool0NOoo00<00?ooool01?oo00<00?ooool06Ooo00?>NOooool0JOoo0003ool00`00oooo
o`08ool20005ool00`00ooooo`1jool20005ool00`00ooooo`0Iool00liiooooo`1Xool001Goo`03
003ooooo07coo`03003ooooo00Coo`03003ooooo00ooo`03003ooooo00Koo`03cWWooooo06Ooo`00
5Ooo0P00O_oo00<00?ooool00ooo00<00?ooool04?oo0P001ooo00?>NOooool0I_oo000Eool00`00
ooooo`1nool20006ool00`00ooooo`0?ool20006ool00liiooooo`1Uool001Goo`03003ooooo083o
o`03003ooooo00?oo`03003ooooo013oo`03003ooooo00Goo`;>NFGoo`005Ooo00<00?ooool0POoo
0P001Ooo00<00?ooool06?oo00?>NOooool0H_oo000Eool00`00ooooo`23ool00`00ooooo`03ool0
0`00ooooo`0Hool00liiooooo`1Qool001Goo`8008Goo`8000Goo`03003ooooo00ooo`03003ooooo
00Goo`03cWWooooo063oo`005Ooo00<00?ooool0Q_oo00<00?ooool00_oo00<00?ooool04?oo0P00
1_oo00?>NOooool0Gooo000Eool00`00ooooo`27ool20004ool00`00ooooo`0@ool20005ool2cWUO
ool001Goo`03003ooooo08Woo`03003ooooo00;oo`03003ooooo017oo`03003ooooo00Coo`03cWWo
oooo05coo`005Ooo00<00?ooool0R_oo00<00?ooool00ooo00<00?ooool05ooo00?>NOooool0Fooo
000Eool00`00ooooo`2;ool20003ool00`00ooooo`0Hool00liiooooo`1Jool001Goo`8008koo`03
003ooooo00?oo`03003ooooo01Koo`03cWWooooo05Woo`005Ooo00<00?ooool0S_oo00<00?ooool0
0_oo00<00?ooool04?oo00<00?ooool01?oo00?>NOooool0F?oo000Eool00`00ooooo`2?ool00`00
ooooo`03ool2000@ool00`00ooooo`04ool00liiooooo`1Gool001Goo`03003ooooo093oo`8001Ko
o`8000Goo`;>NEOoo`0000?oo`0000000P000ooo00<00?ooool01?oo00<00?ooool00ooo00<00?oo
ool0T_oo00<00?ooool00ooo00<00?ooool03ooo00<00?ooool01?oo00?>NOooool0E?oo0003ool0
0`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2Cool00`00ooooo`0Lool00liiooooo`1Cool0
00?oo`03003ooooo00Koo`D000Coo`<009Coo`8000Coo`8001Ooo`03cWWooooo05;oo`000ooo00<0
0?ooool01_oo00@00?ooool000Goo`03003ooooo09Koo`8001Ooo`03003ooooo00;oo`03cWWooooo
057oo`000ooo00<00?ooool01ooo00<00?oo00001Ooo00<00?ooool0V?oo0P000ooo00<00?ooool0
4?oo00<00?ooool00_oo00?>NOooool0D?oo00001?oo000000002_oo0P001Ooo00<00?ooool0V_oo
00<00?ooool05?oo00<00?ooool00_oo00?>NOooool0Cooo0003ool00`00ooooo`09ool00`00oooo
o`03ool00`00ooooo`2Kool01000oooooooo0P004Ooo00<00?ooool00_oo00?>NOooool0C_oo000E
ool2002Mool00`00ooooo`0Dool00`00ooooo`02ool00liiooooo`1=ool001Goo`03003ooooo09go
o`8000?oo`03003ooooo01Coo`;>NDgoo`005Ooo00<00?ooool0Wooo00<00?ooool06Ooo00?>NOoo
ool0B_oo000Eool00`00ooooo`2Pool00`00ooooo`02ool00`00ooooo`0?ool00`00ooooo`02ool0
0liiooooo`19ool001Goo`03003ooooo0:7oo`8000;oo`03003ooooo013oo`8000?oo`03cWWooooo
04Soo`005Ooo00<00?ooool0Xooo00D00?ooooooo`0001;oo`05003oooooooo>N@19ool001Goo`80
0:Goo`05003oooooool0000Aool00`00ooooo`02ool00liiooooo`16ool001Goo`03003ooooo0:Go
o`8000?oo`03003ooooo00koo`03003ooooo00;oo`03cWWooooo04Goo`005Ooo00<00?ooool0Yooo
00@00?ooool001Koo`;>NDGoo`005Ooo00<00?ooool0Z?oo00D00?ooooooo`0001Koo`03cWWooooo
04;oo`005Ooo00<00?ooool0ZOoo00D00?ooooooo`0001;oo`05003oooooooo>N@13ool001Goo`80
0:_oo`8000?oo`03003ooooo00ooo`05003oooooooo>N@12ool001Goo`03003ooooo0:coo`04003o
oooo000Bool20002ool00liiooooo`0oool001Goo`03003ooooo0:goo`05003oooooool0000Bool0
0`00ooooo`02cWToool001Goo`03003ooooo0:koo`8000;oo`03003ooooo01Coo`03cWWooooo03co
o`0000?oo`0000000P000ooo00<00?ooool00ooo0P001Ooo00<00?ooool0/?oo00D00?ooooooo`00
01Goo`03cWWooooo03_oo`000ooo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo0;7oo`04
003ooooo000Fool00liiooooo`0jool000?oo`03003ooooo00Ooo`04003ooooo0004ool3002bool2
0003ool00`00ooooo`0>ool01@00oooooooocWT0>ooo0003ool00`00ooooo`07ool30005ool00`00
ooooo`2dool01000ooooo`004Ooo00D00?ooooooolii03[oo`000ooo00<00?ooool01ooo00<00?oo
ool01Ooo00<00?ooool0]Ooo00D00?ooooooo`00013oo`8000;oo`03cWWooooo03Ooo`0000Coo`00
000000Woo`03003ooooo00Goo`03003ooooo0;Koo`05003oooooool0000Aool00`00ooooo`02cWTg
ool000?oo`03003ooooo00Soo`<000Coo`03003ooooo0;Ooo`8000?oo`03003ooooo01;oo`03cWWo
oooo03Coo`005Ooo00<00?ooool0^Ooo00@00?ooool001Goo`03cWWooooo03?oo`005Ooo0P00^ooo
00D00?ooooooo`0001Coo`03cWWooooo03;oo`005Ooo00<00?ooool0^ooo0P000_oo00<00?ooool0
3ooo00@00?ooooo>NC?oo`005Ooo00<00?ooool0_Ooo00D00?ooooooo`00013oo`800003ooo>NOoo
037oo`005Ooo00<00?ooool0__oo00@00?ooool001;oo`03003oolii037oo`005Ooo00<00?ooool0
_ooo0P000_oo00<00?ooool03ooo00@00?oocWW>NBooo`005Ooo0P00`_oo00@00?ooool001Coo`03
cWWooooo02coo`005Ooo00<00?ooool0`_oo00<00?ooool05Ooo00?>NOooool0:ooo000Eool00`00
ooooo`33ool01000ooooo`005?oo00?>NOooool0:_oo000Eool00`00ooooo`34ool2000Cool01000
ooooolii:ooo000Eool00`00ooooo`36ool01000ool000004?oo00<00?ooool00/ii:Ooo000Eool0
0`00ooooo`37ool00`00ooooo`0Aool20002ool00liiooooo`0Vool001Goo`800<Woo`04003ooooo
000Aool01000ooooolii9ooo000Eool00`00ooooo`39ool00`00ooooo`0Eool00liiooooo`0Tool0
01Goo`03003ooooo0<[oo`800003ool0000001?oo`;>NBCoo`0000?oo`0000000P000ooo00<00?oo
ool00ooo0P001Ooo00<00?ooool0c?oo00<00?ooool04_oo00@00?ooooo>NB?oo`000ooo00<00?oo
ool01ooo00@00?ooool000Coo`03003ooooo0<goo`04003ooooo000Aool01000ooooolii8_oo0003
ool00`00ooooo`07ool01000ooooo`001?oo0`00c_oo0P004ooo0P0000?ooliiool08?oo0003ool0
0`00ooooo`08ool20005ool00`00ooooo`3@ool01000ooooo`004Ooo00<00?oocWT08?oo0003ool0
0`00ooooo`07ool01000ooooo`001?oo00<00?ooool0dOoo00<00?ooool04_oo00@00?oocWW>NAko
o`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3Bool200000ooo0000000Cool00lii
ooooo`0Kool000?oo`03003ooooo00Soo`8000Goo`03003ooooo0=Coo`03003ooooo01Coo`03cWWo
oooo01[oo`005Ooo00<00?ooool0eOoo00@00?ooool0017oo`03003oolii01_oo`005Ooo0P00eooo
00<00?ooool04_oo00<00?oocWT06_oo000Eool00`00ooooo`3Gool200000ooo0000000@ool00`00
ooo>N@0Iool001Goo`03003ooooo0=Woo`03003ooooo017oo`03003oolii01Soo`005Ooo00<00?oo
ool0f_oo00@00?ooool0013oo`04003ooliicWTFool001Goo`03003ooooo0=_oo`8001Goo`03cWWo
oooo01?oo`005Ooo00<00?ooool0gOoo00@00?oo000001;oo`03cWWooooo01;oo`005Ooo0P00gooo
00<00?ooool04Ooo00<00?oocWT04ooo000Eool00`00ooooo`3Oool200000ooo003oo`0?ool00`00
ooo>N@0Bool001Goo`03003ooooo0>7oo`03003ooooo013oo`03003oolii017oo`005Ooo00<00?oo
ool0h_oo00@00?oo000000ooo`03003oolii013oo`005Ooo00<00?ooool0hooo00<00?ooool04?oo
00<00?oocWT03ooo000Eool2003Uool200000ooo003oo`0@ool00liiooooo`0<ool001Goo`03003o
oooo0>Koo`8001;oo`03cWWooooo00_oo`005Ooo00<00?ooool0iooo00<00?oo00004Ooo0/ii2ooo
000Eool00`00ooooo`3Xool00`00ool0000Bool00liiooooo`08ool000goo`@000Coo`03003ooooo
0>Woo`800003ool00?oo013oo`03cWWooooo00Ooo`003Ooo00<00?ooool01Ooo00<00?ooool0jooo
0`004Ooo00?>NOooool01_oo000>ool00`00ooooo`04ool3003/ool00`00ooooo`0Aool00liioooo
o`05ool000ooo`03003ooooo0?ooo`ooo`004?oo00<00?ooool0oooo3_oo000=ool01000ooooo`00
oooo4?oo000>ool2003ooolAool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.165192, 0.480876, \
0.00771848, 0.00892053}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<stablePi2.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("stablePi2.eps"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \({0.31`, 0.693340005010564`}, {0.41`, 
      0.7384040207975904`}, {0.45999999999999996`, 
      0.762040846867269`}, \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.485909 -0.229545 0.395498 [
[.26676 .15345 -9 -9 ]
[.26676 .15345 9 0 ]
[.50972 .15345 -3 -9 ]
[.50972 .15345 3 0 ]
[.75267 .15345 -9 -9 ]
[.75267 .15345 9 0 ]
[.99563 .15345 -3 -9 ]
[.99563 .15345 3 0 ]
[.01131 .00775 -18 -4.5 ]
[.01131 .00775 0 4.5 ]
[.01131 .08685 -18 -4.5 ]
[.01131 .08685 0 4.5 ]
[.01131 .24505 -18 -4.5 ]
[.01131 .24505 0 4.5 ]
[.01131 .32415 -18 -4.5 ]
[.01131 .32415 0 4.5 ]
[.01131 .40325 -18 -4.5 ]
[.01131 .40325 0 4.5 ]
[.01131 .48235 -18 -4.5 ]
[.01131 .48235 0 4.5 ]
[.01131 .56145 -6 -4.5 ]
[.01131 .56145 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26676 .16595 m
.26676 .1722 L
s
[(0.5)] .26676 .15345 0 1 Mshowa
.50972 .16595 m
.50972 .1722 L
s
[(1)] .50972 .15345 0 1 Mshowa
.75267 .16595 m
.75267 .1722 L
s
[(1.5)] .75267 .15345 0 1 Mshowa
.99563 .16595 m
.99563 .1722 L
s
[(2)] .99563 .15345 0 1 Mshowa
.125 Mabswid
.0724 .16595 m
.0724 .1697 L
s
.12099 .16595 m
.12099 .1697 L
s
.16958 .16595 m
.16958 .1697 L
s
.21817 .16595 m
.21817 .1697 L
s
.31535 .16595 m
.31535 .1697 L
s
.36395 .16595 m
.36395 .1697 L
s
.41254 .16595 m
.41254 .1697 L
s
.46113 .16595 m
.46113 .1697 L
s
.55831 .16595 m
.55831 .1697 L
s
.6069 .16595 m
.6069 .1697 L
s
.65549 .16595 m
.65549 .1697 L
s
.70408 .16595 m
.70408 .1697 L
s
.80126 .16595 m
.80126 .1697 L
s
.84985 .16595 m
.84985 .1697 L
s
.89845 .16595 m
.89845 .1697 L
s
.94704 .16595 m
.94704 .1697 L
s
.25 Mabswid
0 .16595 m
1 .16595 L
s
.02381 .00775 m
.03006 .00775 L
s
[(0.6)] .01131 .00775 1 0 Mshowa
.02381 .08685 m
.03006 .08685 L
s
[(0.8)] .01131 .08685 1 0 Mshowa
.02381 .24505 m
.03006 .24505 L
s
[(1.2)] .01131 .24505 1 0 Mshowa
.02381 .32415 m
.03006 .32415 L
s
[(1.4)] .01131 .32415 1 0 Mshowa
.02381 .40325 m
.03006 .40325 L
s
[(1.6)] .01131 .40325 1 0 Mshowa
.02381 .48235 m
.03006 .48235 L
s
[(1.8)] .01131 .48235 1 0 Mshowa
.02381 .56145 m
.03006 .56145 L
s
[(2)] .01131 .56145 1 0 Mshowa
.125 Mabswid
.02381 .02753 m
.02756 .02753 L
s
.02381 .0473 m
.02756 .0473 L
s
.02381 .06708 m
.02756 .06708 L
s
.02381 .10663 m
.02756 .10663 L
s
.02381 .1264 m
.02756 .1264 L
s
.02381 .14618 m
.02756 .14618 L
s
.02381 .18573 m
.02756 .18573 L
s
.02381 .2055 m
.02756 .2055 L
s
.02381 .22528 m
.02756 .22528 L
s
.02381 .26483 m
.02756 .26483 L
s
.02381 .2846 m
.02756 .2846 L
s
.02381 .30438 m
.02756 .30438 L
s
.02381 .34393 m
.02756 .34393 L
s
.02381 .3637 m
.02756 .3637 L
s
.02381 .38348 m
.02756 .38348 L
s
.02381 .42303 m
.02756 .42303 L
s
.02381 .4428 m
.02756 .4428 L
s
.02381 .46258 m
.02756 .46258 L
s
.02381 .50213 m
.02756 .50213 L
s
.02381 .5219 m
.02756 .5219 L
s
.02381 .54168 m
.02756 .54168 L
s
.02381 .58123 m
.02756 .58123 L
s
.02381 .601 m
.02756 .601 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02867 .01472 m
.05296 .01596 L
.07726 .01894 L
.10155 .07188 L
.12585 .07521 L
.15015 .07953 L
.17444 .04467 L
.19874 .05337 L
.22303 .06249 L
.24733 .07184 L
.27162 .11869 L
.29592 .13019 L
.32021 .14271 L
.34451 .15608 L
.3688 .17016 L
.3931 .18483 L
.4174 .19998 L
.44169 .21555 L
.46599 .23147 L
.49028 .24768 L
.51458 .26416 L
.53887 .28087 L
.56317 .29778 L
.58746 .31487 L
.61176 .33213 L
.63605 .34953 L
.66035 .36707 L
.68465 .38473 L
.70894 .4025 L
.73324 .42037 L
.75753 .43834 L
.78183 .45639 L
.80612 .47453 L
.83042 .49274 L
.85471 .51102 L
.87901 .52937 L
.9033 .54777 L
.9276 .56624 L
.9519 .58475 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`000_oo0P001?oo00<00?ooool00ooo0P00
1Ooo00<00?ooool0oooo2Ooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo0?oo
o`Woo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3003oool9ool00005ool00?ooool00008
ool30005ool00`00ooooo`3oool9ool00005ool00?ooool00008ool00`00ooooo`05ool;003oool1
ool00005ool00?ooool00008ool00`00ooooo`05ool00`00ooooo`08ool4003lool000;oo`8000[o
o`<000Coo`03003ooooo00_oo`03003ooooo0?[oo`005Ooo0P003Ooo00<00?ooool0nOoo000Eool0
0`00ooooo`0<ool00`00ooooo`3iool001Goo`03003ooooo00goo`03003ooooo0?Soo`005Ooo00<0
0?ooool03Ooo00<00?ooool0n?oo000Eool00`00ooooo`0>ool00`00ooooo`3gool001Goo`03003o
oooo00koo`03003ooooo01Goo`800>3oo`005Ooo0P004?oo00<00?ooool04ooo00<00?ooool00`00
gOoo000Eool00`00ooooo`0?ool00`00ooooo`0Bool00`00ooooo`04ool3003Jool001Goo`03003o
oooo013oo`03003ooooo017oo`03003ooooo00Ooo`800=Soo`005Ooo00<00?ooool04?oo00<00?oo
ool04?oo00<00?ooool02_oo0P00e_oo000Eool00`00ooooo`0Aool00`00ooooo`0>ool00`00oooo
o`0=ool3003Cool001Goo`8001;oo`03003ooooo00goo`03003ooooo017oo`<00=3oo`005Ooo00<0
0?ooool04_oo0`003?oo00<00?ooool05?oo0P00c_oo000Eool00`00ooooo`0Eool70004ool00`00
ooooo`0Gool00`00ooooo`3;ool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo01co
o`@001[oo`03003ooooo0<_oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0k
ool00`00ooooo`3:ool00005ool00?ooool00008ool01000ooooo`001?oo0`00>ooo00<00?ooool0
b_oo00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool0??oo00<00?ooool0bOoo00001Ooo003o
oooo00002?oo00@00?ooool000Coo`03003ooooo03coo`03003ooooo0<Woo`0000Goo`00ooooo`00
00Soo`04003ooooo0004ool00`00ooooo`0mool00`00ooooo`38ool000;oo`8000[oo`8000Goo`03
003ooooo03goo`03003ooooo0<Soo`005Ooo00<00?ooool0?_oo00<00?ooool0aooo000Eool2000o
ool00`00ooooo`37ool001Goo`03003ooooo03ooo`03003ooooo0<Koo`005Ooo00<00?ooool0?ooo
00<00?ooool0a_oo000Eool00`00ooooo`10ool00`00ooooo`35ool001Goo`03003ooooo047oo`80
0<Goo`005Ooo0P00>Ooo0P001?oo00<00?ooool00_oo0`00>?oo1000>?oo10000ooo00<00?ooool0
0_oo0`00>Ooo10000_oo000Eool00`00ooooo`0gool01000ooooo`002_oo0`00>?oo00<00?ooool0
>Ooo00<00?ooool02_oo00<00?ooool0=_oo00<00?ooool00ooo000Eool00`00ooooo`0gool01000
ooooo`002ooo00@00?oo000003Koo`03003ooooo03Woo`03003ooooo00[oo`03003ooooo03Ooo`03
003ooooo00;oo`005Ooo00<00?ooool0=ooo00@00?ooool000Soo`<000Coo`8003Coo`03003ooooo
03Woo`03003ooooo00Ooo`<003_oo`03003ooooo007oo`005Ooo00<00?ooool0=ooo00@00?ooool0
00Soo`03003ooooo00Koo`8003;oo`03003ooooo03Woo`03003ooooo00Ooo`03003ooooo03coo`40
007oo`7oo`005Ooo0P00>?oo00@00?ooool000Soo`03003ooooo00Soo`8002koo`<003Woo`<000Wo
o`03003ooooo03Woo`04003ooooo0002ool001Goo`03003ooooo03Soo`8000Woo`@000Woo`8002ko
o`03003ooooo03Woo`03003ooooo00Ooo`@003Woo`8000?oo`005Ooo00<00?ooool0D_oo0P00]?oo
000Eool00`00ooooo`1Dool00`00ooooo`2aool001Goo`03003ooooo05Goo`800;7oo`005Ooo00<0
0?ooool0Eooo00<00?ooool0[_oo000>oooo000A0002ool001Goo`03003ooooo00[oo`03003ooooo
00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo
00[oo`8000_oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03
003ooooo00_oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03
003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo007oo`005Ooo00<00?ooool0
Fooo0P00Zooo000Eool00`00ooooo`1Mool2002Yool001Goo`03003ooooo05ooo`800:Ooo`005Ooo
0P00H_oo00<00?ooool0Y?oo000Eool00`00ooooo`1Rool2002Tool001Goo`03003ooooo06Coo`03
003ooooo0:7oo`005Ooo00<00?ooool0IOoo0P00XOoo000Eool00`00ooooo`1Wool00`00ooooo`2N
ool001Goo`8006Woo`8009koo`005Ooo00<00?ooool0J_oo0P00W?oo000Eool00`00ooooo`1/ool2
002Jool001Goo`03003ooooo06koo`03003ooooo09Ooo`005Ooo00<00?ooool0Kooo0P00Uooo000E
ool00`00ooooo`1aool00`00ooooo`2Dool001Goo`8007?oo`8009Coo`005Ooo00<00?ooool0M?oo
00<00?ooool0TOoo000Eool00`00ooooo`1eool2002Aool00003ool00000008000?oo`03003ooooo
00;oo`@000Coo`03003ooooo07Ooo`03003ooooo08koo`000ooo00<00?ooool01ooo00<00?ooool0
1Ooo00<00?ooool0N?oo00<00?ooool0SOoo0003ool00`00ooooo`08ool00`00ooooo`04ool3001i
ool2002=ool000?oo`03003ooooo00Woo`03003ooooo00?oo`03003ooooo07_oo`03003ooooo08[o
o`000ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0O?oo0P00R_oo00001?oo00000000
2Ooo00@00?ooool000Coo`03003ooooo07koo`8008Soo`000ooo00<00?ooool02?oo0P001Ooo00<0
0?ooool0P?oo0P00Q_oo000Eool20023ool00`00ooooo`23ool001Goo`03003ooooo08?oo`03003o
oooo08;oo`005Ooo00<00?ooool0Q?oo00<00?ooool0POoo000Eool00`00ooooo`25ool20021ool0
01Goo`03003ooooo08Ooo`03003ooooo07koo`005Ooo00<00?ooool0R?oo00<00?ooool0OOoo000E
ool2002:ool2001mool001Goo`03003ooooo08_oo`8007_oo`005Ooo00<00?ooool0SOoo0P00NOoo
000Eool00`00ooooo`2?ool00`00ooooo`1fool001Goo`03003ooooo093oo`03003ooooo07Goo`00
5Ooo0P00T_oo00<00?ooool0M?oo000Eool00`00ooooo`2Bool2001dool001Goo`03003ooooo09Co
o`03003ooooo077oo`0000?oo`0000000P000ooo00<00?ooool01?oo00<00?ooool00ooo00<00?oo
ool0UOoo00<00?ooool0L?oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2Fool2
001`ool000?oo`03003ooooo00Koo`D000Coo`<009Soo`8006koo`000ooo00<00?ooool01_oo00@0
0?ooool000Goo`03003ooooo09[oo`8006coo`000ooo00<00?ooool01ooo00<00?oo00001Ooo00<0
0?ooool0W?oo00<00?ooool0JOoo00001?oo000000002_oo0P001Ooo00<00?ooool0WOoo00<00?oo
ool0J?oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`2Nool00`00ooooo`1Wool0
01Goo`03003ooooo09ooo`8006Ooo`005Ooo0P00X_oo00<00?ooool0I?oo000Eool00`00ooooo`2R
ool00`00ooooo`1Sool001Goo`03003ooooo0:?oo`8006?oo`005Ooo00<00?ooool0YOoo00<00?oo
ool0H?oo000Eool00`00ooooo`2Vool00`00ooooo`1Oool001Goo`800:Soo`8005ooo`005Ooo00<0
0?ooool0ZOoo00<00?ooool0G?oo000Eool00`00ooooo`2Zool00`00ooooo`1Kool001Goo`03003o
oooo0:_oo`03003ooooo05[oo`005Ooo00<00?ooool0[?oo0P00F_oo000Eool2002_ool00`00oooo
o`1Gool001Goo`03003ooooo0:ooo`03003ooooo05Koo`005Ooo00<00?ooool0/?oo0P00E_oo000E
ool00`00ooooo`2bool2001Dool00003ool00000008000?oo`03003ooooo00?oo`8000Goo`03003o
oooo0;Coo`8005;oo`000ooo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo0;Koo`03003o
oooo04ooo`000ooo00<00?ooool01ooo00@00?ooool000Coo`<00;Ooo`8004ooo`000ooo00<00?oo
ool01ooo0`001Ooo00<00?ooool0^Ooo00<00?ooool0C?oo0003ool00`00ooooo`07ool00`00oooo
o`05ool00`00ooooo`2jool00`00ooooo`1;ool00004ool000000009ool00`00ooooo`05ool00`00
ooooo`2kool2001;ool000?oo`03003ooooo00Soo`<000Coo`03003ooooo0;goo`03003ooooo04So
o`005Ooo0P00_ooo00<00?ooool0Aooo000Eool00`00ooooo`2oool00`00ooooo`16ool001Goo`03
003ooooo0<3oo`8004Koo`005Ooo00<00?ooool0`_oo00<00?ooool0@ooo000Eool00`00ooooo`33
ool00`00ooooo`12ool001Goo`800<Goo`8004;oo`005Ooo00<00?ooool0a_oo00<00?ooool0?ooo
000Eool00`00ooooo`37ool00`00ooooo`0nool001Goo`03003ooooo0<Soo`8003koo`005Ooo00<0
0?ooool0b_oo00<00?ooool0>ooo000Eool00`00ooooo`3;ool00`00ooooo`0jool001Goo`800<go
o`03003ooooo03Woo`005Ooo00<00?ooool0cOoo0P00>Ooo000Eool00`00ooooo`3?ool00`00oooo
o`0fool00003ool00000008000?oo`03003ooooo00?oo`8000Goo`03003ooooo0=3oo`03003ooooo
03Goo`000ooo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo0=7oo`8003Goo`000ooo00<0
0?ooool01ooo00@00?ooool000Coo`<00=?oo`03003ooooo03;oo`000ooo00<00?ooool02?oo0P00
1Ooo00<00?ooool0e?oo00<00?ooool0<Ooo0003ool00`00ooooo`07ool01000ooooo`001?oo00<0
0?ooool0eOoo0P00<Ooo00001?oo000000002Ooo00@00?ooool000Coo`03003ooooo0=Ooo`03003o
oooo02koo`000ooo00<00?ooool02?oo0P001Ooo00<00?ooool0f?oo00<00?ooool0;Ooo000Eool2
003Jool00`00ooooo`0/ool001Goo`03003ooooo0=[oo`8002coo`005Ooo00<00?ooool0g?oo00<0
0?ooool0:Ooo000Eool00`00ooooo`3Mool00`00ooooo`0Xool001Goo`03003ooooo0=koo`8002So
o`005Ooo00<00?ooool0h?oo00<00?ooool09Ooo000Eool2003Rool00`00ooooo`0Tool001Goo`03
003ooooo0>;oo`8002Coo`005Ooo00<00?ooool0i?oo00<00?ooool08Ooo000Eool00`00ooooo`3U
ool2000Qool001Goo`03003ooooo0>Ooo`03003ooooo01koo`005Ooo0P00jOoo00<00?ooool07Ooo
000Eool00`00ooooo`3Yool2000Mool001Goo`03003ooooo0>_oo`03003ooooo01[oo`003Ooo1000
1?oo00<00?ooool0k?oo00<00?ooool06Ooo000=ool00`00ooooo`05ool00`00ooooo`3]ool00`00
ooooo`0Hool000koo`03003ooooo00Coo`<00>koo`8001Soo`003ooo00<00?ooool00ooo00<00?oo
ool0l?oo00<00?ooool05Ooo000@ool00`00ooooo`02ool00`00ooooo`3aool00`00ooooo`0Dool0
00goo`04003ooooo0004ool00`00ooooo`3bool2000Dool000koo`8000Goo`03003ooooo0?Coo`03
003ooooo017oo`005Ooo00<00?ooool0mOoo00<00?ooool04?oo000Eool2003gool2000@ool001Go
o`03003ooooo0?Soo`03003ooooo00goo`005Ooo00<00?ooool0nOoo00<00?ooool03?oo000Eool0
0`00ooooo`3jool00`00ooooo`0;ool001Goo`03003ooooo0?_oo`8000_oo`005Ooo0P00o_oo00<0
0?ooool02?oo000Eool00`00ooooo`3nool00`00ooooo`07ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.162815, 0.522711, \
0.0076156, 0.00935653}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, omtwo[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, 1]}, {T, 
        1/10, 3, 1/10}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frmp\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \({{1\/10, 0.5016565293358205`}, {1\/5, 0.5327686227052782`}, {3\/10, 
        0.5652348217326016`}, {2\/5, 0.6052401104840299`}, {1\/2, 
        0.6541316255521767`}, {3\/5, 0.7120297451854357`}, {7\/10, 
        0.7784888338627393`}, {4\/5, 0.8527480445989503`}, {9\/10, 
        0.9338285314254409`}, {1, 1.0205993869963188`}, {11\/10, 
        1.1118575199870153`}, {6\/5, 1.2064295413567065`}, {13\/10, 
        1.3032762985289976`}, {7\/5, 1.4015668334862876`}, {3\/2, 
        1.5007028558808457`}, {8\/5, 1.6002957503453141`}, {17\/10, 
        1.7001167578803014`}, {9\/5, 1.8000432615047237`}, {19\/10, 
        1.9000150493742`}, {2, 2.0000049143373415`}, {21\/10, 
        2.1000015046687253`}, {11\/5, 2.2000004297832034`}, {23\/10, 
        2.3000001172863285`}, {12\/5, 2.40000004129669`}, {5\/2, 
        2.5000000534941647`}, {13\/5, 2.6000000001`}, {27\/10, 
        2.7000000001`}, {14\/5, 2.8000000000683505`}, {29\/10, 
        2.900000000292116`}, {3, 2.9999999972293865`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omtwo[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, \(-9\)/10]}, {T, 
        0.1, 3, 0.1}]\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.6795565681034682`}, {0.2`, 
        0.7064116833164339`}, {0.30000000000000004`, 
        0.7471624401499433`}, {0.4`, 0.7976657289506576`}, {0.5`, 
        0.8540919806253656`}, {0.6000000000000001`, 
        0.9133959023788522`}, {0.7000000000000001`, 
        0.9739007424583586`}, {0.8`, 1.035592149788615`}, {0.9`, 
        1.0993667181939315`}, {1.`, 1.1660894479766952`}, {1.1`, 
        1.2362519233860554`}, {1.2000000000000002`, 
        1.310030674298375`}, {1.3`, 
        1.387400946363108`}, {1.4000000000000001`, 
        1.4682164107601618`}, {1.5`, 1.5522527349782564`}, {1.6`, 
        1.6392324062846066`}, {1.7000000000000002`, 
        1.72883975255289`}, {1.8`, 
        1.8207351169882042`}, {1.9000000000000001`, 
        1.914569102406069`}, {2.`, 2.0099978295007555`}, {2.1`, 
        2.1066981612133473`}, {2.2`, 
        2.204380255057502`}, {2.3000000000000003`, 
        2.302795866961035`}, {2.4000000000000004`, 
        2.401741948914626`}, {2.5`, 2.501059537815618`}, {2.6`, 
        2.600629257530151`}, {2.7`, 
        2.700364960455285`}, {2.8000000000000003`, 
        2.8002067470767247`}, {2.9000000000000004`, 
        2.900114411864448`}, {3.`, 3.0000618577280096`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<ximom2.dat\>", %33]\)], "Input"],

Cell[BoxData[
    \("ximom2.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%33, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 -0.157657 0.253653 [
[.18254 .0835 -9 -9 ]
[.18254 .0835 9 0 ]
[.34127 .0835 -3 -9 ]
[.34127 .0835 3 0 ]
[.5 .0835 -9 -9 ]
[.5 .0835 9 0 ]
[.65873 .0835 -3 -9 ]
[.65873 .0835 3 0 ]
[.81746 .0835 -9 -9 ]
[.81746 .0835 9 0 ]
[.97619 .0835 -3 -9 ]
[.97619 .0835 3 0 ]
[.01131 .22282 -18 -4.5 ]
[.01131 .22282 0 4.5 ]
[.01131 .34965 -6 -4.5 ]
[.01131 .34965 0 4.5 ]
[.01131 .47648 -18 -4.5 ]
[.01131 .47648 0 4.5 ]
[.01131 .6033 -6 -4.5 ]
[.01131 .6033 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .096 m
.18254 .10225 L
s
[(0.5)] .18254 .0835 0 1 Mshowa
.34127 .096 m
.34127 .10225 L
s
[(1)] .34127 .0835 0 1 Mshowa
.5 .096 m
.5 .10225 L
s
[(1.5)] .5 .0835 0 1 Mshowa
.65873 .096 m
.65873 .10225 L
s
[(2)] .65873 .0835 0 1 Mshowa
.81746 .096 m
.81746 .10225 L
s
[(2.5)] .81746 .0835 0 1 Mshowa
.97619 .096 m
.97619 .10225 L
s
[(3)] .97619 .0835 0 1 Mshowa
.125 Mabswid
.05556 .096 m
.05556 .09975 L
s
.0873 .096 m
.0873 .09975 L
s
.11905 .096 m
.11905 .09975 L
s
.15079 .096 m
.15079 .09975 L
s
.21429 .096 m
.21429 .09975 L
s
.24603 .096 m
.24603 .09975 L
s
.27778 .096 m
.27778 .09975 L
s
.30952 .096 m
.30952 .09975 L
s
.37302 .096 m
.37302 .09975 L
s
.40476 .096 m
.40476 .09975 L
s
.43651 .096 m
.43651 .09975 L
s
.46825 .096 m
.46825 .09975 L
s
.53175 .096 m
.53175 .09975 L
s
.56349 .096 m
.56349 .09975 L
s
.59524 .096 m
.59524 .09975 L
s
.62698 .096 m
.62698 .09975 L
s
.69048 .096 m
.69048 .09975 L
s
.72222 .096 m
.72222 .09975 L
s
.75397 .096 m
.75397 .09975 L
s
.78571 .096 m
.78571 .09975 L
s
.84921 .096 m
.84921 .09975 L
s
.88095 .096 m
.88095 .09975 L
s
.9127 .096 m
.9127 .09975 L
s
.94444 .096 m
.94444 .09975 L
s
.25 Mabswid
0 .096 m
1 .096 L
s
.02381 .22282 m
.03006 .22282 L
s
[(1.5)] .01131 .22282 1 0 Mshowa
.02381 .34965 m
.03006 .34965 L
s
[(2)] .01131 .34965 1 0 Mshowa
.02381 .47648 m
.03006 .47648 L
s
[(2.5)] .01131 .47648 1 0 Mshowa
.02381 .6033 m
.03006 .6033 L
s
[(3)] .01131 .6033 1 0 Mshowa
.125 Mabswid
.02381 .12136 m
.02756 .12136 L
s
.02381 .14673 m
.02756 .14673 L
s
.02381 .17209 m
.02756 .17209 L
s
.02381 .19746 m
.02756 .19746 L
s
.02381 .24819 m
.02756 .24819 L
s
.02381 .27355 m
.02756 .27355 L
s
.02381 .29892 m
.02756 .29892 L
s
.02381 .32428 m
.02756 .32428 L
s
.02381 .37502 m
.02756 .37502 L
s
.02381 .40038 m
.02756 .40038 L
s
.02381 .42575 m
.02756 .42575 L
s
.02381 .45111 m
.02756 .45111 L
s
.02381 .50184 m
.02756 .50184 L
s
.02381 .52721 m
.02756 .52721 L
s
.02381 .55257 m
.02756 .55257 L
s
.02381 .57794 m
.02756 .57794 L
s
.02381 .07063 m
.02756 .07063 L
s
.02381 .04527 m
.02756 .04527 L
s
.02381 .0199 m
.02756 .0199 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.0873 .02153 L
.11905 .03186 L
.15079 .04467 L
.18254 .05899 L
.21429 .07403 L
.24603 .08938 L
.27778 .10502 L
.30952 .1212 L
.34127 .13813 L
.37302 .15592 L
.40476 .17464 L
.43651 .19426 L
.46825 .21476 L
.5 .23608 L
.53175 .25814 L
.56349 .28087 L
.59524 .30418 L
.62698 .32798 L
.65873 .35219 L
.69048 .37671 L
.72222 .40149 L
.75397 .42646 L
.78571 .45155 L
.81746 .47675 L
.84921 .502 L
.88095 .5273 L
.9127 .55262 L
.94444 .57797 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`005@3oool00`000000oooo0?oo
o`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?oo
o`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000
003oool0oooo00H0oooo0P00003o0?ooo`40oooo000E0?ooo`8000002@3oool400000?`0oooo000E
0?ooo`030000003oool0oooo00`0oooo1@00003g0?ooo`005@3oool00`000000oooo0?ooo`0A0?oo
o`@00000l`3oool001D0oooo00<000000?ooo`3oool05@3oool300000?00oooo000E0?ooo`030000
003oool0oooo01P0oooo0P00003^0?ooo`005@3oool00`000000oooo0?ooo`0J0?ooo`800000k03o
ool001D0oooo00<000000?ooo`3oool0703oool200000>X0oooo000E0?ooo`8000007`3oool30000
0>L0oooo000E0?ooo`030000003oool0oooo0240oooo0P00003U0?ooo`005@3oool00`000000oooo
0?ooo`0S0?ooo`800000h`3oool001D0oooo00<000000?ooo`3oool08@3oool200000080oooo0`00
00040?ooo`<000008P3oool400000240oooo100000030?ooo`030000003oool0oooo0080oooo0`00
000S0?ooo`@000008@3oool4000000<0oooo00<000000?ooo`3oool00P3oool3000002<0oooo0P00
00070?ooo`005@3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo000000<0oooo0`00
00050?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool08P3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool08P3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo01l0oooo00@000000?ooo`3oool000001P3oool001D0oooo00<0
00000?ooo`3oool0803oool010000000oooo0?ooo`0000060?ooo`8000000`3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo00@0oooo000E0?ooo`030000003oool0oooo0200oooo00@0
00000?ooo`3oool00000203oool3000002@0oooo00<000000?ooo`3oool08P3oool00`000000oooo
0?ooo`070?ooo`<000009@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo
0`00000S0?ooo`8000001`3oool001D0oooo0P00000Q0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`00000000008`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`040?ooo`005@3oool00`000000oooo
0?ooo`0P0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0oooo0P00000O0?oo
o`<000008P3oool3000000T0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`00000Q
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool08P3oool010000000oooo0?oo
o`0000060?ooo`005@3oool00`000000oooo0?ooo`0Q0?ooo`8000002@3oool400000080oooo0P00
000O0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool01`3oool4000002<0oooo0P00
000S0?ooo`8000002@3oool400000280oooo0P0000070?ooo`005@3oool00`000000oooo0?ooo`0d
0?ooo`800000dP3oool001D0oooo00<000000?ooo`3oool0=P3oool200000=00oooo000E0?ooo`03
0000003oool0oooo03P0oooo0`00003=0?ooo`005@3oool00`000000oooo0?ooo`0k0?ooo`800000
b`3oool000h0ooooo`00000C000000005@3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo00<0oooo0P0000000`3oool000000?ooo`070?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo000E0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool05P3oool200000100oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo02P0oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`050?ooo`005@3oool00`000000oooo0?ooo`130?ooo`80
0000``3oool001D0oooo00<000000?ooo`3oool0A@3oool200000<40oooo000E0?ooo`030000003o
ool0oooo04L0oooo0P00002o0?ooo`005@3oool00`000000oooo0?ooo`190?ooo`800000_@3oool0
01D0oooo0P00001<0?ooo`030000003oool0oooo0;X0oooo000E0?ooo`030000003oool0oooo04`0
oooo0P00002j0?ooo`005@3oool00`000000oooo0?ooo`1>0?ooo`800000^03oool001D0oooo00<0
00000?ooo`3oool0D03oool200000;H0oooo000E0?ooo`030000003oool0oooo0580oooo0P00002d
0?ooo`005@3oool00`000000oooo0?ooo`1D0?ooo`800000/P3oool001D0oooo00<000000?ooo`3o
ool0EP3oool200000;00oooo000E0?ooo`800000F@3oool200000:h0oooo000E0?ooo`030000003o
ool0oooo05X0oooo0P00002/0?ooo`005@3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0
oooo0:T0oooo000E0?ooo`030000003oool0oooo05d0oooo0P00002Y0?ooo`005@3oool00`000000
oooo0?ooo`1O0?ooo`800000Y`3oool001D0oooo00<000000?ooo`3oool0H@3oool200000:D0oooo
000E0?ooo`030000003oool0oooo06<0oooo0P00002S0?ooo`005@3oool2000006H0oooo00<00000
0?ooo`3oool0X03oool001D0oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`2O0?oo
o`005@3oool00`000000oooo0?ooo`1W0?ooo`800000W`3oool001D0oooo00<000000?ooo`3oool0
J@3oool00`000000oooo0?ooo`2L0?ooo`005@3oool00`000000oooo0?ooo`1Z0?ooo`030000003o
ool0oooo09/0oooo000E0?ooo`030000003oool0oooo06/0oooo0P00002K0?ooo`005@3oool20000
06h0oooo00<000000?ooo`3oool0V03oool001D0oooo00<000000?ooo`3oool0KP3oool2000009P0
oooo000E0?ooo`030000003oool0oooo0700oooo0P00002F0?ooo`005@3oool00`000000oooo0?oo
o`1b0?ooo`800000U03oool001D0oooo00<000000?ooo`3oool0M03oool200000980oooo00000`3o
ool0000000000002000000<0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?oo
o`3oool0MP3oool00`000000oooo0?ooo`2?0?ooo`000`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`2>0?ooo`00
0`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo0`00001h0?ooo`800000
SP3oool000<0oooo00<000000?ooo`3oool01`3oool3000000D0oooo00<000000?ooo`3oool0NP3o
ool00`000000oooo0?ooo`2;0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`2:0?ooo`0000@0oooo0000
0000000000002@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo07`0oooo0P00002:
0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`@00000103oool00`000000oooo0?ooo`1n0?oo
o`030000003oool0oooo08L0oooo000E0?ooo`030000003oool0oooo07l0oooo0P0000270?ooo`00
5@3oool00`000000oooo0?ooo`210?ooo`030000003oool0oooo08@0oooo000E0?ooo`800000P`3o
ool2000008@0oooo000E0?ooo`030000003oool0oooo08@0oooo00<000000?ooo`3oool0P@3oool0
01D0oooo00<000000?ooo`3oool0Q@3oool200000840oooo000E0?ooo`030000003oool0oooo08L0
oooo00<000000?ooo`3oool0OP3oool001D0oooo00<000000?ooo`3oool0R03oool2000007h0oooo
000E0?ooo`030000003oool0oooo08X0oooo00<000000?ooo`3oool0N`3oool001D0oooo00<00000
0?ooo`3oool0R`3oool2000007/0oooo000E0?ooo`800000SP3oool00`000000oooo0?ooo`1h0?oo
o`005@3oool00`000000oooo0?ooo`2>0?ooo`800000N03oool001D0oooo00<000000?ooo`3oool0
T03oool00`000000oooo0?ooo`1e0?ooo`005@3oool00`000000oooo0?ooo`2A0?ooo`030000003o
ool0oooo07@0oooo000E0?ooo`030000003oool0oooo0980oooo0P00001d0?ooo`005@3oool00`00
0000oooo0?ooo`2D0?ooo`030000003oool0oooo0740oooo000E0?ooo`030000003oool0oooo09D0
oooo00<000000?ooo`3oool0L03oool001D0oooo0P00002G0?ooo`800000L03oool001D0oooo00<0
00000?ooo`3oool0V03oool00`000000oooo0?ooo`1]0?ooo`005@3oool00`000000oooo0?ooo`2I
0?ooo`030000003oool0oooo06`0oooo000E0?ooo`030000003oool0oooo09X0oooo0P00001/0?oo
o`005@3oool00`000000oooo0?ooo`2L0?ooo`030000003oool0oooo06T0oooo000E0?ooo`030000
003oool0oooo09d0oooo00<000000?ooo`3oool0J03oool001D0oooo00<000000?ooo`3oool0WP3o
ool2000006P0oooo000E0?ooo`800000X@3oool00`000000oooo0?ooo`1U0?ooo`005@3oool00`00
0000oooo0?ooo`2Q0?ooo`030000003oool0oooo06@0oooo000E0?ooo`030000003oool0oooo0:80
oooo00<000000?ooo`3oool0H`3oool001D0oooo00<000000?ooo`3oool0X`3oool2000006<0oooo
000E0?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool0H03oool000d0oooo10000004
0?ooo`030000003oool0oooo0:H0oooo00<000000?ooo`3oool0G`3oool000d0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`2W0?ooo`800000G`3oool000h0oooo00<000000?ooo`3o
ool0103oool300000:T0oooo00<000000?ooo`3oool0G03oool000l0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`2Z0?ooo`030000003oool0oooo05/0oooo000@0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0Z`3oool00`000000oooo0?ooo`1J0?ooo`003@3oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo0:`0oooo0P00001J0?ooo`003P3oool2
000000D0oooo00<000000?ooo`3oool0[P3oool00`000000oooo0?ooo`1G0?ooo`005@3oool00`00
0000oooo0?ooo`2_0?ooo`030000003oool0oooo05H0oooo000E0?ooo`030000003oool0oooo0;00
oooo0P00001F0?ooo`005@3oool200000;<0oooo00<000000?ooo`3oool0D`3oool001D0oooo00<0
00000?ooo`3oool0/`3oool00`000000oooo0?ooo`1B0?ooo`005@3oool00`000000oooo0?ooo`2d
0?ooo`800000DP3oool001D0oooo00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`1?0?oo
o`005@3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0oooo04h0oooo000E0?ooo`030000
003oool0oooo0;P0oooo0P00001>0?ooo`005@3oool00`000000oooo0?ooo`2j0?ooo`030000003o
ool0oooo04/0oooo000E0?ooo`800000_03oool00`000000oooo0?ooo`1:0?ooo`005@3oool00`00
0000oooo0?ooo`2l0?ooo`030000003oool0oooo04T0oooo000E0?ooo`030000003oool0oooo0;d0
oooo0P0000190?ooo`005@3oool00`000000oooo0?ooo`2o0?ooo`030000003oool0oooo04H0oooo
000E0?ooo`030000003oool0oooo0<00oooo00<000000?ooo`3oool0A@3oool001D0oooo00<00000
0?ooo`3oool0`@3oool2000004D0oooo000E0?ooo`030000003oool0oooo0<<0oooo00<000000?oo
o`3oool0@P3oool001D0oooo0P0000350?ooo`030000003oool0oooo0440oooo000E0?ooo`030000
003oool0oooo0<D0oooo00<000000?ooo`3oool0@03oool001D0oooo00<000000?ooo`3oool0aP3o
ool00`000000oooo0?ooo`0o0?ooo`005@3oool00`000000oooo0?ooo`370?ooo`030000003oool0
oooo03h0oooo000E0?ooo`030000003oool0oooo0<P0oooo0P00000n0?ooo`005@3oool00`000000
oooo0?ooo`3:0?ooo`030000003oool0oooo03/0oooo000E0?ooo`800000c03oool00`000000oooo
0?ooo`0j0?ooo`005@3oool00`000000oooo0?ooo`3<0?ooo`030000003oool0oooo03T0oooo000E
0?ooo`030000003oool0oooo0<d0oooo0P00000i0?ooo`005@3oool00`000000oooo0?ooo`3?0?oo
o`030000003oool0oooo03H0oooo000E0?ooo`030000003oool0oooo0=00oooo0P00000f0?ooo`00
00<0oooo0000000000000P0000030?ooo`030000003oool0oooo0080oooo0`0000050?ooo`030000
003oool0oooo0=80oooo00<000000?ooo`3oool0<`3oool000030?ooo`000000oooo00d0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`3C0?ooo`800000<`3oool00080oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`020?ooo`<00000e@3oool00`000000oooo0?ooo`0`
0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`<000001@3oool00`000000oooo0?ooo`3F0?oo
o`030000003oool0oooo02l0oooo00040?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`3G0?ooo`030000003oool0oooo02h0oooo00001@3oool00000
0?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3H0?ooo`03
0000003oool0oooo02d0oooo00020?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0
f@3oool2000002d0oooo000E0?ooo`030000003oool0oooo0=/0oooo00<000000?ooo`3oool0:P3o
ool001D0oooo00<000000?ooo`3oool0g03oool00`000000oooo0?ooo`0Y0?ooo`005@3oool20000
0=h0oooo00<000000?ooo`3oool0:03oool001D0oooo00<000000?ooo`3oool0gP3oool00`000000
oooo0?ooo`0W0?ooo`005@3oool00`000000oooo0?ooo`3O0?ooo`8000009`3oool001D0oooo00<0
00000?ooo`3oool0h@3oool00`000000oooo0?ooo`0T0?ooo`005@3oool00`000000oooo0?ooo`3R
0?ooo`030000003oool0oooo02<0oooo000E0?ooo`030000003oool0oooo0><0oooo0P00000S0?oo
o`005@3oool00`000000oooo0?ooo`3U0?ooo`030000003oool0oooo0200oooo000E0?ooo`800000
i`3oool00`000000oooo0?ooo`0O0?ooo`005@3oool00`000000oooo0?ooo`3W0?ooo`030000003o
ool0oooo01h0oooo000E0?ooo`030000003oool0oooo0>P0oooo0P00000N0?ooo`005@3oool00`00
0000oooo0?ooo`3Z0?ooo`030000003oool0oooo01/0oooo000E0?ooo`030000003oool0oooo0>/0
oooo00<000000?ooo`3oool06P3oool001D0oooo00<000000?ooo`3oool0k03oool2000001X0oooo
000E0?ooo`030000003oool0oooo0>h0oooo00<000000?ooo`3oool05`3oool001D0oooo0P00003`
0?ooo`030000003oool0oooo01H0oooo000E0?ooo`030000003oool0oooo0?00oooo00<000000?oo
o`3oool05@3oool001D0oooo00<000000?ooo`3oool0l@3oool00`000000oooo0?ooo`0D0?ooo`00
5@3oool00`000000oooo0?ooo`3b0?ooo`030000003oool0oooo01<0oooo000E0?ooo`030000003o
ool0oooo0?<0oooo0P00000C0?ooo`005@3oool00`000000oooo0?ooo`3e0?ooo`030000003oool0
oooo0100oooo000E0?ooo`030000003oool0oooo0?H0oooo00<000000?ooo`3oool03`3oool001D0
oooo0P00003h0?ooo`030000003oool0oooo00h0oooo000E0?ooo`030000003oool0oooo0?P0oooo
00<000000?ooo`3oool03@3oool001D0oooo00<000000?ooo`3oool0n@3oool2000000d0oooo000E
0?ooo`030000003oool0oooo0?/0oooo00<000000?ooo`3oool02P3oool001D0oooo00<000000?oo
o`3oool0o03oool00`000000oooo0?ooo`090?ooo`003P3oool2000000D0oooo00<000000?ooo`3o
ool0o@3oool2000000T0oooo000=0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool0o`3oool00`000000oooo0?ooo`060?ooo`00403oool00`000000oooo0?ooo`020?ooo`<00000
o`3oool10?ooo`030000003oool0oooo00D0oooo000>0?ooo`8000001@3oool00`000000oooo0?oo
o`3o0?ooo`T0oooo000@0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool9
0?ooo`003@3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0?l0oooo2@3oool0
00h0oooo0P0000050?ooo`030000003oool0oooo0?l0oooo2@3oool00?l0oooo8@3oool00?l0oooo
8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, 0.561448, \
0.0115777, 0.0144901}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, omtwo[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, 1]}, {T, 
        1/10, 1/2, 1/10}]\)], "Input"],

Cell[BoxData[
    \({{1\/10, 0.5092554845405696`}, {1\/5, 0.533065218894041`}, {3\/10, 
        0.5652348217326016`}, {2\/5, 0.6052401104840299`}, {1\/2, 
        0.6541316255521767`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[%236]\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.5092554845405696`}, {0.2`, 0.533065218894041`}, {0.3`, 
        0.5652348217326016`}, {0.4`, 0.6052401104840299`}, {0.5`, 
        0.6541316255521767`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, omthree[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, 1]}, {T, 
        1/10, 1/2, 1/10}]\)], "Input"],

Cell[BoxData[
    \({{1\/10, 0.542037875166733`}, {1\/5, 0.5673709763381093`}, {3\/10, 
        0.6117568756202727`}, {2\/5, 0.6708619162734202`}, {1\/2, 
        0.7396585440183191`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[%205]\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.5016565293358205`}, {0.2`, 0.5327686227052782`}, {0.3`, 
        0.5652348217326016`}, {0.4`, 0.6052401104840299`}, {0.5`, 
        0.6541316255521767`}, {0.6`, 0.7120297451854357`}, {0.7`, 
        0.7784888338627393`}, {0.8`, 0.8527480445989503`}, {0.9`, 
        0.9338285314254409`}, {1.`, 1.0205993869963188`}, {1.1`, 
        1.1118575199870153`}, {1.2`, 1.2064295413567065`}, {1.3`, 
        1.3032762985289976`}, {1.4`, 1.4015668334862876`}, {1.5`, 
        1.5007028558808457`}, {1.6`, 1.6002957503453141`}, {1.7`, 
        1.7001167578803014`}, {1.8`, 1.8000432615047237`}, {1.9`, 
        1.9000150493742`}, {2.`, 2.0000049143373415`}, {2.1`, 
        2.1000015046687253`}, {2.2`, 2.2000004297832034`}, {2.3`, 
        2.3000001172863285`}, {2.4`, 2.40000004129669`}, {2.5`, 
        2.5000000534941647`}, {2.6`, 2.6000000001`}, {2.7`, 
        2.7000000001`}, {2.8`, 2.8000000000683505`}, {2.9`, 
        2.900000000292116`}, {3.`, 2.9999999972293865`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(list1 = {{0.1`, 0.5092554845405696`}, {0.2`, 
          0.533065218894041`}, {0.3`, 0.5652348217326016`}, {0.4`, 
          0.6052401104840299`}, {0.5`, 0.6541316255521767`}, {0.6`, 
          0.7120297451854357`}, {0.7`, 0.7784888338627393`}, {0.8`, 
          0.8527480445989503`}, {0.9`, 0.9338285314254409`}, {1.`, 
          1.0205993869963188`}, {1.1`, 1.1118575199870153`}, {1.2`, 
          1.2064295413567065`}, {1.3`, 1.3032762985289976`}, {1.4`, 
          1.4015668334862876`}, {1.5`, 1.5007028558808457`}, {1.6`, 
          1.6002957503453141`}, {1.7`, 1.7001167578803014`}, {1.8`, 
          1.8000432615047237`}, {1.9`, 1.9000150493742`}, {2.`, 
          2.0000049143373415`}, {2.1`, 2.1000015046687253`}, {2.2`, 
          2.2000004297832034`}, {2.3`, 2.3000001172863285`}, {2.4`, 
          2.40000004129669`}, {2.5`, 2.5000000534941647`}, {2.6`, 
          2.6000000001`}, {2.7`, 2.7000000001`}, {2.8`, 
          2.8000000000683505`}, {2.9`, 2.900000000292116`}, {3.`, 
          2.9999999972293865`}}\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.5092554845405696`}, {0.2`, 0.533065218894041`}, {0.3`, 
        0.5652348217326016`}, {0.4`, 0.6052401104840299`}, {0.5`, 
        0.6541316255521767`}, {0.6`, 0.7120297451854357`}, {0.7`, 
        0.7784888338627393`}, {0.8`, 0.8527480445989503`}, {0.9`, 
        0.9338285314254409`}, {1.`, 1.0205993869963188`}, {1.1`, 
        1.1118575199870153`}, {1.2`, 1.2064295413567065`}, {1.3`, 
        1.3032762985289976`}, {1.4`, 1.4015668334862876`}, {1.5`, 
        1.5007028558808457`}, {1.6`, 1.6002957503453141`}, {1.7`, 
        1.7001167578803014`}, {1.8`, 1.8000432615047237`}, {1.9`, 
        1.9000150493742`}, {2.`, 2.0000049143373415`}, {2.1`, 
        2.1000015046687253`}, {2.2`, 2.2000004297832034`}, {2.3`, 
        2.3000001172863285`}, {2.4`, 2.40000004129669`}, {2.5`, 
        2.5000000534941647`}, {2.6`, 2.6000000001`}, {2.7`, 
        2.7000000001`}, {2.8`, 2.8000000000683505`}, {2.9`, 
        2.900000000292116`}, {3.`, 2.9999999972293865`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<bmode1.dat\>", N[%]]\)], "Input"],

Cell[BoxData[
    \("bmode1.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(l2 = Import["\<bmode2.dat\>"]\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.5092549498133029`}, {0.2`, 
        0.5330653658245935`}, {0.30000000000000004`, 
        0.6117568756202727`}, {0.4`, 0.6708619162734202`}, {0.5`, 
        0.7396585440183191`}, {0.6000000000000001`, 
        0.8150176272351078`}, {0.7000000000000001`, 
        0.8950271781537082`}, {0.8`, 0.9784590068124394`}, {0.9`, 
        1.0644875703240366`}, {1.`, 1.1525368713311395`}, {1.1`, 
        1.2421921738086275`}, {1.2000000000000002`, 
        1.3331467316240484`}, {1.3`, 
        1.4251683902542025`}, {1.4000000000000001`, 
        1.5180779322077156`}, {1.5`, 1.6117346109814372`}, {1.6`, 
        1.7060262265434696`}, {1.7000000000000002`, 
        1.8008624595972778`}, {1.8`, 
        1.896168550451231`}, {1.9000000000000001`, 
        1.9918837867908519`}, {2.`, 2.0879572097308268`}, {2.1`, 
        2.1843460025901886`}, {2.2`, 
        2.2810139170620563`}, {2.3000000000000003`, 
        2.3779300537807493`}, {2.4000000000000004`, 
        2.475067906784569`}, {2.5`, 2.572404607050119`}, {2.6`, 
        2.669920317987671`}, {2.7`, 
        2.7675977482015703`}, {2.8000000000000003`, 
        2.8654217556482293`}, {2.9000000000000004`, 
        2.9633790236877235`}, {3.`, 3.0614577941693057`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[%248]\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.542037875166733`}, {0.2`, 0.5673709763381093`}, {0.3`, 
        0.6117568756202727`}, {0.4`, 0.6708619162734202`}, {0.5`, 
        0.7396585440183191`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(l2 = {{0.1`, 0.542037875166733`}, {0.2`, 0.5673709763381093`}, {0.3`, 
          0.6117568756202727`}, {0.4`, 0.6708619162734202`}, {0.5`, 
          0.7396585440183191`}, {0.6000000000000001`, 
          0.8150176272351078`}, {0.7000000000000001`, 
          0.8950271781537082`}, {0.8`, 0.9784590068124394`}, {0.9`, 
          1.0644875703240366`}, {1.`, 1.1525368713311395`}, {1.1`, 
          1.2421921738086275`}, {1.2000000000000002`, 
          1.3331467316240484`}, {1.3`, 
          1.4251683902542025`}, {1.4000000000000001`, 
          1.5180779322077156`}, {1.5`, 1.6117346109814372`}, {1.6`, 
          1.7060262265434696`}, {1.7000000000000002`, 
          1.8008624595972778`}, {1.8`, 
          1.896168550451231`}, {1.9000000000000001`, 
          1.9918837867908519`}, {2.`, 2.0879572097308268`}, {2.1`, 
          2.1843460025901886`}, {2.2`, 
          2.2810139170620563`}, {2.3000000000000003`, 
          2.3779300537807493`}, {2.4000000000000004`, 
          2.475067906784569`}, {2.5`, 2.572404607050119`}, {2.6`, 
          2.669920317987671`}, {2.7`, 
          2.7675977482015703`}, {2.8000000000000003`, 
          2.8654217556482293`}, {2.9000000000000004`, 
          2.9633790236877235`}, {3.`, 3.0614577941693057`}}\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.542037875166733`}, {0.2`, 0.5673709763381093`}, {0.3`, 
        0.6117568756202727`}, {0.4`, 0.6708619162734202`}, {0.5`, 
        0.7396585440183191`}, {0.6000000000000001`, 
        0.8150176272351078`}, {0.7000000000000001`, 
        0.8950271781537082`}, {0.8`, 0.9784590068124394`}, {0.9`, 
        1.0644875703240366`}, {1.`, 1.1525368713311395`}, {1.1`, 
        1.2421921738086275`}, {1.2000000000000002`, 
        1.3331467316240484`}, {1.3`, 
        1.4251683902542025`}, {1.4000000000000001`, 
        1.5180779322077156`}, {1.5`, 1.6117346109814372`}, {1.6`, 
        1.7060262265434696`}, {1.7000000000000002`, 
        1.8008624595972778`}, {1.8`, 
        1.896168550451231`}, {1.9000000000000001`, 
        1.9918837867908519`}, {2.`, 2.0879572097308268`}, {2.1`, 
        2.1843460025901886`}, {2.2`, 
        2.2810139170620563`}, {2.3000000000000003`, 
        2.3779300537807493`}, {2.4000000000000004`, 
        2.475067906784569`}, {2.5`, 2.572404607050119`}, {2.6`, 
        2.669920317987671`}, {2.7`, 
        2.7675977482015703`}, {2.8000000000000003`, 
        2.8654217556482293`}, {2.9000000000000004`, 
        2.9633790236877235`}, {3.`, 3.0614577941693057`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<bmode2.dat\>", %]\)], "Input"],

Cell[BoxData[
    \("bmode2.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%250, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 -0.111919 0.233627 [
[.18254 .10921 -9 -9 ]
[.18254 .10921 9 0 ]
[.34127 .10921 -3 -9 ]
[.34127 .10921 3 0 ]
[.5 .10921 -9 -9 ]
[.5 .10921 9 0 ]
[.65873 .10921 -3 -9 ]
[.65873 .10921 3 0 ]
[.81746 .10921 -9 -9 ]
[.81746 .10921 9 0 ]
[.97619 .10921 -3 -9 ]
[.97619 .10921 3 0 ]
[.01131 .00489 -18 -4.5 ]
[.01131 .00489 0 4.5 ]
[.01131 .23852 -18 -4.5 ]
[.01131 .23852 0 4.5 ]
[.01131 .35533 -6 -4.5 ]
[.01131 .35533 0 4.5 ]
[.01131 .47215 -18 -4.5 ]
[.01131 .47215 0 4.5 ]
[.01131 .58896 -6 -4.5 ]
[.01131 .58896 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .12171 m
.18254 .12796 L
s
[(0.5)] .18254 .10921 0 1 Mshowa
.34127 .12171 m
.34127 .12796 L
s
[(1)] .34127 .10921 0 1 Mshowa
.5 .12171 m
.5 .12796 L
s
[(1.5)] .5 .10921 0 1 Mshowa
.65873 .12171 m
.65873 .12796 L
s
[(2)] .65873 .10921 0 1 Mshowa
.81746 .12171 m
.81746 .12796 L
s
[(2.5)] .81746 .10921 0 1 Mshowa
.97619 .12171 m
.97619 .12796 L
s
[(3)] .97619 .10921 0 1 Mshowa
.125 Mabswid
.05556 .12171 m
.05556 .12546 L
s
.0873 .12171 m
.0873 .12546 L
s
.11905 .12171 m
.11905 .12546 L
s
.15079 .12171 m
.15079 .12546 L
s
.21429 .12171 m
.21429 .12546 L
s
.24603 .12171 m
.24603 .12546 L
s
.27778 .12171 m
.27778 .12546 L
s
.30952 .12171 m
.30952 .12546 L
s
.37302 .12171 m
.37302 .12546 L
s
.40476 .12171 m
.40476 .12546 L
s
.43651 .12171 m
.43651 .12546 L
s
.46825 .12171 m
.46825 .12546 L
s
.53175 .12171 m
.53175 .12546 L
s
.56349 .12171 m
.56349 .12546 L
s
.59524 .12171 m
.59524 .12546 L
s
.62698 .12171 m
.62698 .12546 L
s
.69048 .12171 m
.69048 .12546 L
s
.72222 .12171 m
.72222 .12546 L
s
.75397 .12171 m
.75397 .12546 L
s
.78571 .12171 m
.78571 .12546 L
s
.84921 .12171 m
.84921 .12546 L
s
.88095 .12171 m
.88095 .12546 L
s
.9127 .12171 m
.9127 .12546 L
s
.94444 .12171 m
.94444 .12546 L
s
.25 Mabswid
0 .12171 m
1 .12171 L
s
.02381 .00489 m
.03006 .00489 L
s
[(0.5)] .01131 .00489 1 0 Mshowa
.02381 .23852 m
.03006 .23852 L
s
[(1.5)] .01131 .23852 1 0 Mshowa
.02381 .35533 m
.03006 .35533 L
s
[(2)] .01131 .35533 1 0 Mshowa
.02381 .47215 m
.03006 .47215 L
s
[(2.5)] .01131 .47215 1 0 Mshowa
.02381 .58896 m
.03006 .58896 L
s
[(3)] .01131 .58896 1 0 Mshowa
.125 Mabswid
.02381 .02826 m
.02756 .02826 L
s
.02381 .05162 m
.02756 .05162 L
s
.02381 .07498 m
.02756 .07498 L
s
.02381 .09834 m
.02756 .09834 L
s
.02381 .14507 m
.02756 .14507 L
s
.02381 .16843 m
.02756 .16843 L
s
.02381 .1918 m
.02756 .1918 L
s
.02381 .21516 m
.02756 .21516 L
s
.02381 .26188 m
.02756 .26188 L
s
.02381 .28525 m
.02756 .28525 L
s
.02381 .30861 m
.02756 .30861 L
s
.02381 .33197 m
.02756 .33197 L
s
.02381 .3787 m
.02756 .3787 L
s
.02381 .40206 m
.02756 .40206 L
s
.02381 .42542 m
.02756 .42542 L
s
.02381 .44878 m
.02756 .44878 L
s
.02381 .49551 m
.02756 .49551 L
s
.02381 .51887 m
.02756 .51887 L
s
.02381 .54224 m
.02756 .54224 L
s
.02381 .5656 m
.02756 .5656 L
s
.02381 .61232 m
.02756 .61232 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.0873 .02063 L
.11905 .031 L
.15079 .04481 L
.18254 .06088 L
.21429 .07849 L
.24603 .09718 L
.27778 .11667 L
.30952 .13677 L
.34127 .15734 L
.37302 .17829 L
.40476 .19954 L
.43651 .22104 L
.46825 .24274 L
.5 .26462 L
.53175 .28665 L
.56349 .30881 L
.59524 .33108 L
.62698 .35344 L
.65873 .37588 L
.69048 .3984 L
.72222 .42099 L
.75397 .44363 L
.78571 .46632 L
.81746 .48906 L
.84921 .51185 L
.88095 .53467 L
.9127 .55752 L
.94444 .58041 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`000P3oool2
000000@0oooo00<000000?ooo`3oool00P3oool300000?l0oooo4@3oool000050?ooo`000000oooo
0?ooo`0000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0oooo2@3oool0
00050?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`020?ooo`<00000o`3oool9
0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`000000oooo0?ooo`3o
0?ooo`T0oooo00001@3oool000000?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`3o0?ooo`T0oooo00001@3oool000000?ooo`3oool0000000P0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`@00000oP3oool00080oooo0P0000090?oo
o`@00000103oool00`000000oooo0?ooo`0:0?ooo`H00000n03oool001D0oooo00<000000?ooo`3o
ool0403oool300000?D0oooo000E0?ooo`800000503oool300000?80oooo000E0?ooo`030000003o
ool0oooo01H0oooo0P00003`0?ooo`005@3oool00`000000oooo0?ooo`0H0?ooo`800000kP3oool0
01D0oooo00<000000?ooo`3oool06P3oool200000>`0oooo000E0?ooo`030000003oool0oooo01`0
oooo0P00003Z0?ooo`005@3oool00`000000oooo0?ooo`0N0?ooo`<00000i`3oool001D0oooo00<0
00000?ooo`3oool08@3oool200000>D0oooo000E0?ooo`800000903oool200000><0oooo000E0?oo
o`030000003oool0oooo02D0oooo0P00003Q0?ooo`005@3oool00`000000oooo0?ooo`0W0?ooo`80
0000g`3oool001D0oooo00<000000?ooo`3oool0:@3oool200000=d0oooo000E0?ooo`030000003o
ool0oooo02/0oooo0P00003K0?ooo`005@3oool00`000000oooo0?ooo`0]0?ooo`800000f@3oool0
01D0oooo0P00000`0?ooo`800000e`3oool001D0oooo00<000000?ooo`3oool0<@3oool00`000000
oooo0?ooo`3D0?ooo`005@3oool00`000000oooo0?ooo`0Q0?ooo`800000103oool00`000000oooo
0?ooo`020?ooo`<000000`3oool2000001d0oooo1000000Q0?ooo`@000000`3oool00`000000oooo
0?ooo`020?ooo`<000008`3oool400000240oooo100000030?ooo`030000003oool0oooo0080oooo
0`00000S0?ooo`8000001`3oool001D0oooo00<000000?ooo`3oool0803oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool07`3oool010000000oooo
0?ooo`0000060?ooo`005@3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo000000/0
oooo00<000000?ooo`3oool00`3oool2000001`0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0103oool001D0oooo
00<000000?ooo`3oool0803oool010000000oooo0?ooo`0000080?ooo`<00000203oool2000001X0
oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`070?ooo`<000009@3oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo0`00000S0?ooo`8000001`3oool001D0oooo
0P00000Q0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0V0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`0U0?ooo`030000003oool0oooo00@0oooo000E0?ooo`030000003oool0oooo0200oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0;0?ooo`8000005@3oool300000280
oooo0`0000090?ooo`030000003oool0oooo02<0oooo00@000000?ooo`3oool000008@3oool01000
0000oooo0?ooo`0000080?ooo`030000003oool0oooo0280oooo00@000000?ooo`3oool000001P3o
ool001D0oooo00<000000?ooo`3oool08@3oool2000000T0oooo1000000<0?ooo`8000005@3oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo1000000S0?ooo`8000008`3oool2
000000T0oooo1000000R0?ooo`8000001`3oool001D0oooo00<000000?ooo`3oool0?P3oool20000
0<P0oooo000E0?ooo`030000003oool0oooo0400oooo0P0000360?ooo`005@3oool00`000000oooo
0?ooo`120?ooo`030000003oool0oooo0<<0oooo000E0?ooo`030000003oool0oooo04<0oooo0P00
00330?ooo`003P3ooooo000001<00000000E0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool010000000oooo0?ooo`3oool2000000<0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool0
01D0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`2m0?ooo`005@3oool00`000000
oooo0?ooo`190?ooo`800000_@3oool001D0oooo00<000000?ooo`3oool0B`3oool00`000000oooo
0?ooo`2j0?ooo`005@3oool00`000000oooo0?ooo`1<0?ooo`800000^P3oool001D0oooo0P00001?
0?ooo`030000003oool0oooo0;L0oooo000E0?ooo`030000003oool0oooo04l0oooo0P00002g0?oo
o`005@3oool00`000000oooo0?ooo`1A0?ooo`800000]@3oool001D0oooo00<000000?ooo`3oool0
D`3oool00`000000oooo0?ooo`2b0?ooo`005@3oool00`000000oooo0?ooo`1D0?ooo`800000/P3o
ool001D0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`2_0?ooo`005@3oool20000
05P0oooo0P00002_0?ooo`005@3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo0:`0
oooo000E0?ooo`030000003oool0oooo05X0oooo0P00002/0?ooo`005@3oool00`000000oooo0?oo
o`1L0?ooo`030000003oool0oooo0:T0oooo000E0?ooo`030000003oool0oooo05d0oooo0P00002Y
0?ooo`005@3oool00`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0:H0oooo000E0?ooo`03
0000003oool0oooo0600oooo0P00002V0?ooo`005@3oool2000006<0oooo00<000000?ooo`3oool0
X`3oool001D0oooo00<000000?ooo`3oool0H`3oool200000:<0oooo000E0?ooo`030000003oool0
oooo06D0oooo00<000000?ooo`3oool0X03oool001D0oooo00<000000?ooo`3oool0IP3oool00`00
0000oooo0?ooo`2O0?ooo`005@3oool00`000000oooo0?ooo`1W0?ooo`800000W`3oool001D0oooo
00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`2L0?ooo`005@3oool2000006/0oooo00<0
00000?ooo`3oool0V`3oool001D0oooo00<000000?ooo`3oool0J`3oool2000009/0oooo000E0?oo
o`030000003oool0oooo06d0oooo00<000000?ooo`3oool0V03oool001D0oooo00<000000?ooo`3o
ool0KP3oool2000009P0oooo00000`3oool0000000000002000000<0oooo00<000000?ooo`3oool0
0P3oool3000000D0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`2E0?ooo`000`3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0L@3o
ool2000009D0oooo00030?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool00P3oool3
000007<0oooo00<000000?ooo`3oool0TP3oool000<0oooo00<000000?ooo`3oool01`3oool30000
00D0oooo00<000000?ooo`3oool0M03oool200000980oooo00030?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo08l0
oooo0000103oool000000000000000090?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0M`3oool00`000000oooo0?ooo`2>0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`@00000
103oool00`000000oooo0?ooo`1h0?ooo`800000SP3oool001D0oooo00<000000?ooo`3oool0NP3o
ool00`000000oooo0?ooo`2;0?ooo`005@3oool00`000000oooo0?ooo`1k0?ooo`030000003oool0
oooo08X0oooo000E0?ooo`800000O@3oool2000008X0oooo000E0?ooo`030000003oool0oooo07h0
oooo00<000000?ooo`3oool0Q`3oool001D0oooo00<000000?ooo`3oool0O`3oool2000008L0oooo
000E0?ooo`030000003oool0oooo0840oooo00<000000?ooo`3oool0Q03oool001D0oooo00<00000
0?ooo`3oool0PP3oool2000008@0oooo000E0?ooo`030000003oool0oooo08@0oooo00<000000?oo
o`3oool0P@3oool001D0oooo0P0000260?ooo`800000P@3oool001D0oooo00<000000?ooo`3oool0
Q`3oool00`000000oooo0?ooo`1n0?ooo`005@3oool00`000000oooo0?ooo`280?ooo`800000OP3o
ool001D0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`1k0?ooo`005@3oool00`00
0000oooo0?ooo`2;0?ooo`800000N`3oool001D0oooo00<000000?ooo`3oool0S@3oool00`000000
oooo0?ooo`1h0?ooo`005@3oool00`000000oooo0?ooo`2>0?ooo`800000N03oool001D0oooo0P00
002A0?ooo`030000003oool0oooo07D0oooo000E0?ooo`030000003oool0oooo0940oooo00<00000
0?ooo`3oool0M03oool001D0oooo00<000000?ooo`3oool0TP3oool2000007@0oooo000E0?ooo`03
0000003oool0oooo09@0oooo00<000000?ooo`3oool0L@3oool001D0oooo00<000000?ooo`3oool0
U@3oool00`000000oooo0?ooo`1`0?ooo`005@3oool00`000000oooo0?ooo`2F0?ooo`800000L03o
ool001D0oooo0P00002I0?ooo`030000003oool0oooo06d0oooo000E0?ooo`030000003oool0oooo
09T0oooo0P00001]0?ooo`005@3oool00`000000oooo0?ooo`2K0?ooo`030000003oool0oooo06X0
oooo000E0?ooo`030000003oool0oooo09`0oooo0P00001Z0?ooo`003@3oool4000000@0oooo00<0
00000?ooo`3oool0WP3oool00`000000oooo0?ooo`1W0?ooo`003@3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo09l0oooo0P00001W0?ooo`003P3oool00`000000oooo0?ooo`040?oo
o`<00000X@3oool00`000000oooo0?ooo`1T0?ooo`003`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo0:80oooo0P00001T0?ooo`00403oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0:@0oooo00<000000?ooo`3oool0H@3oool000d0oooo00@000000?ooo`3oool00000
103oool00`000000oooo0?ooo`2U0?ooo`800000H@3oool000h0oooo0P0000050?ooo`030000003o
ool0oooo0:L0oooo00<000000?ooo`3oool0GP3oool001D0oooo00<000000?ooo`3oool0Z03oool2
000005h0oooo000E0?ooo`030000003oool0oooo0:X0oooo00<000000?ooo`3oool0F`3oool001D0
oooo0P00002/0?ooo`030000003oool0oooo05X0oooo000E0?ooo`030000003oool0oooo0:`0oooo
0P00001J0?ooo`005@3oool00`000000oooo0?ooo`2^0?ooo`030000003oool0oooo05L0oooo000E
0?ooo`030000003oool0oooo0:l0oooo00<000000?ooo`3oool0EP3oool001D0oooo00<000000?oo
o`3oool0/03oool2000005H0oooo000E0?ooo`030000003oool0oooo0;80oooo00<000000?ooo`3o
ool0D`3oool001D0oooo0P00002d0?ooo`800000D`3oool001D0oooo00<000000?ooo`3oool0]@3o
ool00`000000oooo0?ooo`1@0?ooo`005@3oool00`000000oooo0?ooo`2f0?ooo`800000D03oool0
01D0oooo00<000000?ooo`3oool0^03oool00`000000oooo0?ooo`1=0?ooo`005@3oool00`000000
oooo0?ooo`2i0?ooo`800000C@3oool001D0oooo00<000000?ooo`3oool0^`3oool00`000000oooo
0?ooo`1:0?ooo`005@3oool200000;d0oooo0P00001:0?ooo`005@3oool00`000000oooo0?ooo`2n
0?ooo`030000003oool0oooo04L0oooo000E0?ooo`030000003oool0oooo0;l0oooo0P0000170?oo
o`005@3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo04@0oooo000E0?ooo`030000
003oool0oooo0<80oooo0P0000140?ooo`005@3oool00`000000oooo0?ooo`340?ooo`030000003o
ool0oooo0440oooo000E0?ooo`030000003oool0oooo0<D0oooo00<000000?ooo`3oool0@03oool0
01D0oooo0P0000370?ooo`800000@03oool001D0oooo00<000000?ooo`3oool0b03oool00`000000
oooo0?ooo`0m0?ooo`005@3oool00`000000oooo0?ooo`390?ooo`030000003oool0oooo03`0oooo
000E0?ooo`030000003oool0oooo0<X0oooo0P00000l0?ooo`0000<0oooo0000000000000P000003
0?ooo`030000003oool0oooo0080oooo0`0000050?ooo`030000003oool0oooo0<`0oooo00<00000
0?ooo`3oool0>@3oool000030?ooo`000000oooo00d0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`3=0?ooo`030000003oool0oooo03P0oooo00020?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool00P3oool300000<h0oooo0P00000h0?ooo`000`3oool00`000000oooo
0?ooo`070?ooo`<000001@3oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo03D0oooo
00040?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`3A0?ooo`030000003oool0oooo03@0oooo00001@3oool000000?ooo`3oool0000000P0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`3B0?ooo`800000=03oool00080oooo0P000009
0?ooo`@00000103oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo0340oooo000E0?oo
o`030000003oool0oooo0=D0oooo00<000000?ooo`3oool0<03oool001D0oooo0P00003G0?ooo`03
0000003oool0oooo02l0oooo000E0?ooo`030000003oool0oooo0=L0oooo0P00000_0?ooo`005@3o
ool00`000000oooo0?ooo`3I0?ooo`030000003oool0oooo02`0oooo000E0?ooo`030000003oool0
oooo0=X0oooo00<000000?ooo`3oool0:`3oool001D0oooo00<000000?ooo`3oool0f`3oool20000
02/0oooo000E0?ooo`030000003oool0oooo0=d0oooo00<000000?ooo`3oool0:03oool001D0oooo
00<000000?ooo`3oool0gP3oool2000002P0oooo000E0?ooo`800000h@3oool00`000000oooo0?oo
o`0U0?ooo`005@3oool00`000000oooo0?ooo`3Q0?ooo`8000009@3oool001D0oooo00<000000?oo
o`3oool0h`3oool00`000000oooo0?ooo`0R0?ooo`005@3oool00`000000oooo0?ooo`3T0?ooo`80
00008P3oool001D0oooo00<000000?ooo`3oool0iP3oool00`000000oooo0?ooo`0O0?ooo`005@3o
ool00`000000oooo0?ooo`3W0?ooo`8000007`3oool001D0oooo0P00003Z0?ooo`030000003oool0
oooo01`0oooo000E0?ooo`030000003oool0oooo0>X0oooo0P00000L0?ooo`005@3oool00`000000
oooo0?ooo`3/0?ooo`030000003oool0oooo01T0oooo000E0?ooo`030000003oool0oooo0>d0oooo
0P00000I0?ooo`005@3oool00`000000oooo0?ooo`3_0?ooo`030000003oool0oooo01H0oooo000E
0?ooo`030000003oool0oooo0?00oooo00<000000?ooo`3oool05@3oool001D0oooo0P00003b0?oo
o`8000005@3oool001D0oooo00<000000?ooo`3oool0l`3oool00`000000oooo0?ooo`0B0?ooo`00
5@3oool00`000000oooo0?ooo`3d0?ooo`030000003oool0oooo0140oooo000E0?ooo`030000003o
ool0oooo0?D0oooo0P00000A0?ooo`005@3oool00`000000oooo0?ooo`3g0?ooo`030000003oool0
oooo00h0oooo000>0?ooo`8000001@3oool00`000000oooo0?ooo`3h0?ooo`030000003oool0oooo
00d0oooo000=0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0n@3oool20000
00d0oooo000@0?ooo`030000003oool0oooo0080oooo0`00003k0?ooo`030000003oool0oooo00X0
oooo000>0?ooo`8000001@3oool00`000000oooo0?ooo`3l0?ooo`030000003oool0oooo00T0oooo
000@0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool2000000T0oooo000=
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0o`3oool00`000000oooo0?oo
o`060?ooo`003P3oool2000000D0oooo00<000000?ooo`3oool0o`3oool10?ooo`030000003oool0
oooo00D0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo0P00003o0?ooo`X0
oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3o
ool90?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, 0.384996, \
0.0115777, 0.0157322}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%239, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 -0.103474 0.235598 [
[.18254 .11962 -9 -9 ]
[.18254 .11962 9 0 ]
[.34127 .11962 -3 -9 ]
[.34127 .11962 3 0 ]
[.5 .11962 -9 -9 ]
[.5 .11962 9 0 ]
[.65873 .11962 -3 -9 ]
[.65873 .11962 3 0 ]
[.81746 .11962 -9 -9 ]
[.81746 .11962 9 0 ]
[.97619 .11962 -3 -9 ]
[.97619 .11962 3 0 ]
[.01131 .01432 -18 -4.5 ]
[.01131 .01432 0 4.5 ]
[.01131 .24992 -18 -4.5 ]
[.01131 .24992 0 4.5 ]
[.01131 .36772 -6 -4.5 ]
[.01131 .36772 0 4.5 ]
[.01131 .48552 -18 -4.5 ]
[.01131 .48552 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .13212 m
.18254 .13837 L
s
[(0.5)] .18254 .11962 0 1 Mshowa
.34127 .13212 m
.34127 .13837 L
s
[(1)] .34127 .11962 0 1 Mshowa
.5 .13212 m
.5 .13837 L
s
[(1.5)] .5 .11962 0 1 Mshowa
.65873 .13212 m
.65873 .13837 L
s
[(2)] .65873 .11962 0 1 Mshowa
.81746 .13212 m
.81746 .13837 L
s
[(2.5)] .81746 .11962 0 1 Mshowa
.97619 .13212 m
.97619 .13837 L
s
[(3)] .97619 .11962 0 1 Mshowa
.125 Mabswid
.05556 .13212 m
.05556 .13587 L
s
.0873 .13212 m
.0873 .13587 L
s
.11905 .13212 m
.11905 .13587 L
s
.15079 .13212 m
.15079 .13587 L
s
.21429 .13212 m
.21429 .13587 L
s
.24603 .13212 m
.24603 .13587 L
s
.27778 .13212 m
.27778 .13587 L
s
.30952 .13212 m
.30952 .13587 L
s
.37302 .13212 m
.37302 .13587 L
s
.40476 .13212 m
.40476 .13587 L
s
.43651 .13212 m
.43651 .13587 L
s
.46825 .13212 m
.46825 .13587 L
s
.53175 .13212 m
.53175 .13587 L
s
.56349 .13212 m
.56349 .13587 L
s
.59524 .13212 m
.59524 .13587 L
s
.62698 .13212 m
.62698 .13587 L
s
.69048 .13212 m
.69048 .13587 L
s
.72222 .13212 m
.72222 .13587 L
s
.75397 .13212 m
.75397 .13587 L
s
.78571 .13212 m
.78571 .13587 L
s
.84921 .13212 m
.84921 .13587 L
s
.88095 .13212 m
.88095 .13587 L
s
.9127 .13212 m
.9127 .13587 L
s
.94444 .13212 m
.94444 .13587 L
s
.25 Mabswid
0 .13212 m
1 .13212 L
s
.02381 .01432 m
.03006 .01432 L
s
[(0.5)] .01131 .01432 1 0 Mshowa
.02381 .24992 m
.03006 .24992 L
s
[(1.5)] .01131 .24992 1 0 Mshowa
.02381 .36772 m
.03006 .36772 L
s
[(2)] .01131 .36772 1 0 Mshowa
.02381 .48552 m
.03006 .48552 L
s
[(2.5)] .01131 .48552 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03788 m
.02756 .03788 L
s
.02381 .06144 m
.02756 .06144 L
s
.02381 .085 m
.02756 .085 L
s
.02381 .10856 m
.02756 .10856 L
s
.02381 .15568 m
.02756 .15568 L
s
.02381 .17924 m
.02756 .17924 L
s
.02381 .2028 m
.02756 .2028 L
s
.02381 .22636 m
.02756 .22636 L
s
.02381 .27348 m
.02756 .27348 L
s
.02381 .29704 m
.02756 .29704 L
s
.02381 .3206 m
.02756 .3206 L
s
.02381 .34416 m
.02756 .34416 L
s
.02381 .39128 m
.02756 .39128 L
s
.02381 .41484 m
.02756 .41484 L
s
.02381 .4384 m
.02756 .4384 L
s
.02381 .46196 m
.02756 .46196 L
s
.02381 .50908 m
.02756 .50908 L
s
.02381 .53264 m
.02756 .53264 L
s
.02381 .5562 m
.02756 .5562 L
s
.02381 .57976 m
.02756 .57976 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.0873 .02205 L
.11905 .02969 L
.15079 .03912 L
.18254 .05064 L
.21429 .06428 L
.24603 .07994 L
.27778 .09743 L
.30952 .11653 L
.34127 .13698 L
.37302 .15848 L
.40476 .18076 L
.43651 .20357 L
.46825 .22673 L
.5 .25009 L
.53175 .27355 L
.56349 .29707 L
.59524 .32061 L
.62698 .34417 L
.65873 .36772 L
.69048 .39128 L
.72222 .41484 L
.75397 .4384 L
.78571 .46196 L
.81746 .48552 L
.84921 .50908 L
.88095 .53264 L
.9127 .5562 L
.94444 .57976 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`005@3oool00`000000oooo0?oo
o`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool00080oooo0P0000040?oo
o`030000003oool0oooo0080oooo0`0000050?ooo`030000003oool0oooo0?l0oooo2@3oool00005
0?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0?l0oooo2@3oool000050?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`020?oo
o`<000001P3oool200000?l0oooo0@3oool000050?ooo`000000oooo0?ooo`000000203oool30000
00D0oooo00<000000?ooo`3oool0203oool400000?`0oooo00001@3oool000000?ooo`3oool00000
00P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0<0?ooo`D00000m`3oool00005
0?ooo`000000oooo0?ooo`000000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
0140oooo1000003c0?ooo`000P3oool2000000T0oooo100000040?ooo`030000003oool0oooo01D0
oooo1000003_0?ooo`005@3oool00`000000oooo0?ooo`0I0?ooo`800000k@3oool001D0oooo0P00
000L0?ooo`<00000jP3oool001D0oooo00<000000?ooo`3oool07P3oool300000>L0oooo000E0?oo
o`030000003oool0oooo0240oooo0`00003T0?ooo`005@3oool00`000000oooo0?ooo`0T0?ooo`<0
0000h@3oool001D0oooo00<000000?ooo`3oool09`3oool300000=h0oooo000E0?ooo`030000003o
ool0oooo02X0oooo0`00003K0?ooo`005@3oool00`000000oooo0?ooo`0]0?ooo`<00000f03oool0
01D0oooo0P00000a0?ooo`800000eP3oool001D0oooo00<000000?ooo`3oool0<P3oool200000=@0
oooo000E0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0d@3oool001D0oooo00<0
00000?ooo`3oool0=@3oool200000=40oooo000E0?ooo`030000003oool0oooo03L0oooo0P00003?
0?ooo`005@3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo0<`0oooo000E0?ooo`80
0000>`3oool200000<`0oooo000E0?ooo`030000003oool0oooo03`0oooo0P00003:0?ooo`005@3o
ool00`000000oooo0?ooo`0n0?ooo`800000b03oool001D0oooo00<000000?ooo`3oool08@3oool2
000000@0oooo00<000000?ooo`3oool00P3oool300000140oooo0P00000?0?ooo`@000008@3oool4
000000<0oooo00<000000?ooo`3oool00P3oool3000002<0oooo1000000Q0?ooo`@000000`3oool0
0`000000oooo0?ooo`020?ooo`<000008`3oool2000000L0oooo000E0?ooo`030000003oool0oooo
0200oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo0200oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo01l0oooo00@000000?ooo`3oool000001P3oool001D0oooo00<000000?ooo`3oool0803oool0
10000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0140oooo0P00000>0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo00@0oooo000E0?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool00000203o
ool3000001H0oooo0P00000<0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool01`3o
ool3000002D0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`070?ooo`<000008`3o
ool2000000L0oooo000E0?ooo`8000008@3oool010000000oooo0?ooo`0000080?ooo`030000003o
ool0oooo01P0oooo0P00000:0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00@0oooo000E0?ooo`030000003oool0
oooo0200oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0J0?ooo`8000001P3o
ool300000280oooo0`0000090?ooo`030000003oool0oooo02<0oooo00@000000?ooo`3oool00000
8@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo0280oooo00@000000?ooo`3o
ool000001P3oool001D0oooo00<000000?ooo`3oool08@3oool2000000T0oooo1000000K0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`070?ooo`@0
00008`3oool2000002<0oooo0P0000090?ooo`@000008P3oool2000000L0oooo000E0?ooo`030000
003oool0oooo04`0oooo0P00002j0?ooo`005@3oool00`000000oooo0?ooo`1>0?ooo`030000003o
ool0oooo0;L0oooo000E0?ooo`030000003oool0oooo04l0oooo0P00002g0?ooo`003P3ooooo0000
01<00000000E0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo000E0?ooo`030000003oool0oooo02P0oooo
00<000000?ooo`3oool0:03oool3000002P0oooo00<000000?ooo`3oool0:@3oool00`000000oooo
0?ooo`0X0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool01@3oool001D0oooo00<0
00000?ooo`3oool0EP3oool00`000000oooo0?ooo`2_0?ooo`005@3oool00`000000oooo0?ooo`1G
0?ooo`800000[`3oool001D0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`2/0?oo
o`005@3oool2000005/0oooo0P00002/0?ooo`005@3oool00`000000oooo0?ooo`1L0?ooo`030000
003oool0oooo0:T0oooo000E0?ooo`030000003oool0oooo05d0oooo0P00002Y0?ooo`005@3oool0
0`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0:H0oooo000E0?ooo`030000003oool0oooo
0600oooo0P00002V0?ooo`005@3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo0:<0
oooo000E0?ooo`030000003oool0oooo06<0oooo0P00002S0?ooo`005@3oool2000006H0oooo00<0
00000?ooo`3oool0X03oool001D0oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`2O
0?ooo`005@3oool00`000000oooo0?ooo`1W0?ooo`800000W`3oool001D0oooo00<000000?ooo`3o
ool0J@3oool00`000000oooo0?ooo`2L0?ooo`005@3oool00`000000oooo0?ooo`1Z0?ooo`030000
003oool0oooo09/0oooo000E0?ooo`030000003oool0oooo06/0oooo0P00002K0?ooo`005@3oool2
000006h0oooo00<000000?ooo`3oool0V03oool001D0oooo00<000000?ooo`3oool0KP3oool00`00
0000oooo0?ooo`2G0?ooo`005@3oool00`000000oooo0?ooo`1_0?ooo`800000U`3oool001D0oooo
00<000000?ooo`3oool0L@3oool00`000000oooo0?ooo`2D0?ooo`005@3oool00`000000oooo0?oo
o`1b0?ooo`030000003oool0oooo09<0oooo000E0?ooo`030000003oool0oooo07<0oooo0P00002C
0?ooo`005@3oool00`000000oooo0?ooo`1e0?ooo`030000003oool0oooo0900oooo000E0?ooo`80
0000M`3oool00`000000oooo0?ooo`2?0?ooo`005@3oool00`000000oooo0?ooo`1g0?ooo`030000
003oool0oooo08h0oooo000E0?ooo`030000003oool0oooo07P0oooo0P00002>0?ooo`005@3oool0
0`000000oooo0?ooo`1j0?ooo`030000003oool0oooo08/0oooo00000`3oool00000000000020000
00<0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0N`3oool00`00
0000oooo0?ooo`2:0?ooo`000`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0O03oool2000008X0oooo00030?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool00P3oool3000007h0oooo00<000000?ooo`3oool0Q`3oool000<0oooo00<0
00000?ooo`3oool01`3oool3000000D0oooo00<000000?ooo`3oool0O`3oool2000008L0oooo0003
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`21
0?ooo`030000003oool0oooo08@0oooo0000103oool000000000000000090?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0PP3oool2000008@0oooo00030?ooo`030000003oool0oooo
00L0oooo100000040?ooo`030000003oool0oooo08@0oooo00<000000?ooo`3oool0P@3oool001D0
oooo00<000000?ooo`3oool0Q@3oool200000840oooo000E0?ooo`800000R03oool00`000000oooo
0?ooo`1n0?ooo`005@3oool00`000000oooo0?ooo`280?ooo`030000003oool0oooo07d0oooo000E
0?ooo`030000003oool0oooo08T0oooo0P00001m0?ooo`005@3oool00`000000oooo0?ooo`2;0?oo
o`030000003oool0oooo07X0oooo000E0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3o
ool0N@3oool001D0oooo00<000000?ooo`3oool0S@3oool2000007T0oooo000E0?ooo`030000003o
ool0oooo08l0oooo00<000000?ooo`3oool0MP3oool001D0oooo0P00002A0?ooo`030000003oool0
oooo07D0oooo000E0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0M03oool001D0
oooo00<000000?ooo`3oool0TP3oool2000007@0oooo000E0?ooo`030000003oool0oooo09@0oooo
00<000000?ooo`3oool0L@3oool001D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?oo
o`1`0?ooo`005@3oool00`000000oooo0?ooo`2F0?ooo`800000L03oool001D0oooo0P00002I0?oo
o`030000003oool0oooo06d0oooo000E0?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3o
ool0K03oool001D0oooo00<000000?ooo`3oool0VP3oool2000006`0oooo000E0?ooo`030000003o
ool0oooo09`0oooo00<000000?ooo`3oool0J@3oool001D0oooo00<000000?ooo`3oool0W@3oool0
0`000000oooo0?ooo`1X0?ooo`005@3oool00`000000oooo0?ooo`2N0?ooo`800000J03oool001D0
oooo00<000000?ooo`3oool0X03oool00`000000oooo0?ooo`1U0?ooo`005@3oool200000:80oooo
00<000000?ooo`3oool0I03oool001D0oooo00<000000?ooo`3oool0XP3oool2000006@0oooo000E
0?ooo`030000003oool0oooo0:@0oooo00<000000?ooo`3oool0H@3oool001D0oooo00<000000?oo
o`3oool0Y@3oool200000640oooo000=0?ooo`@00000103oool00`000000oooo0?ooo`2W0?ooo`03
0000003oool0oooo05h0oooo000=0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
Z03oool2000005h0oooo000>0?ooo`030000003oool0oooo00@0oooo0`00002Z0?ooo`030000003o
ool0oooo05/0oooo000?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0Z`3oool0
0`000000oooo0?ooo`1J0?ooo`00403oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0:`0oooo0P00001J0?ooo`003@3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
0:h0oooo00<000000?ooo`3oool0E`3oool000h0oooo0P0000050?ooo`030000003oool0oooo0:l0
oooo00<000000?ooo`3oool0EP3oool001D0oooo00<000000?ooo`3oool0/03oool2000005H0oooo
000E0?ooo`800000/`3oool00`000000oooo0?ooo`1C0?ooo`005@3oool00`000000oooo0?ooo`2c
0?ooo`030000003oool0oooo0580oooo000E0?ooo`030000003oool0oooo0;@0oooo0P00001B0?oo
o`005@3oool00`000000oooo0?ooo`2f0?ooo`030000003oool0oooo04l0oooo000E0?ooo`030000
003oool0oooo0;L0oooo00<000000?ooo`3oool0CP3oool001D0oooo00<000000?ooo`3oool0^03o
ool2000004h0oooo000E0?ooo`030000003oool0oooo0;X0oooo00<000000?ooo`3oool0B`3oool0
01D0oooo0P00002l0?ooo`030000003oool0oooo04X0oooo000E0?ooo`030000003oool0oooo0;`0
oooo0P00001:0?ooo`005@3oool00`000000oooo0?ooo`2n0?ooo`030000003oool0oooo04L0oooo
000E0?ooo`030000003oool0oooo0;l0oooo0P0000170?ooo`005@3oool00`000000oooo0?ooo`31
0?ooo`030000003oool0oooo04@0oooo000E0?ooo`030000003oool0oooo0<80oooo0P0000140?oo
o`005@3oool200000<D0oooo00<000000?ooo`3oool0@@3oool001D0oooo00<000000?ooo`3oool0
a@3oool00`000000oooo0?ooo`100?ooo`005@3oool00`000000oooo0?ooo`360?ooo`030000003o
ool0oooo03l0oooo000E0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0?P3oool0
01D0oooo00<000000?ooo`3oool0b03oool2000003h0oooo000E0?ooo`030000003oool0oooo0<X0
oooo00<000000?ooo`3oool0>`3oool001D0oooo00<000000?ooo`3oool0b`3oool00`000000oooo
0?ooo`0j0?ooo`005@3oool200000<d0oooo00<000000?ooo`3oool0>@3oool001D0oooo00<00000
0?ooo`3oool0c@3oool2000003T0oooo000E0?ooo`030000003oool0oooo0<l0oooo00<000000?oo
o`3oool0=P3oool001D0oooo00<000000?ooo`3oool0d03oool2000003H0oooo00000`3oool00000
00000002000000<0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0
dP3oool00`000000oooo0?ooo`0c0?ooo`0000<0oooo0000003oool03@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0=<0oooo0P00000c0?ooo`000P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0080oooo0`00003E0?ooo`030000003oool0oooo0300oooo00030?oo
o`030000003oool0oooo00L0oooo0`0000050?ooo`030000003oool0oooo0=H0oooo00<000000?oo
o`3oool0;`3oool000@0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo0=L0oooo00<000000?ooo`3oool0;P3oool000050?ooo`000000oooo0?ooo`00
0000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0=P0oooo00<000000?ooo`3o
ool0;@3oool00080oooo0P0000090?ooo`@00000103oool00`000000oooo0?ooo`3I0?ooo`800000
;@3oool001D0oooo00<000000?ooo`3oool0f`3oool00`000000oooo0?ooo`0Z0?ooo`005@3oool0
0`000000oooo0?ooo`3L0?ooo`030000003oool0oooo02T0oooo000E0?ooo`800000gP3oool00`00
0000oooo0?ooo`0X0?ooo`005@3oool00`000000oooo0?ooo`3N0?ooo`800000:03oool001D0oooo
00<000000?ooo`3oool0h03oool00`000000oooo0?ooo`0U0?ooo`005@3oool00`000000oooo0?oo
o`3Q0?ooo`8000009@3oool001D0oooo00<000000?ooo`3oool0h`3oool00`000000oooo0?ooo`0R
0?ooo`005@3oool00`000000oooo0?ooo`3T0?ooo`8000008P3oool001D0oooo0P00003W0?ooo`03
0000003oool0oooo01l0oooo000E0?ooo`030000003oool0oooo0>L0oooo0P00000O0?ooo`005@3o
ool00`000000oooo0?ooo`3Y0?ooo`030000003oool0oooo01`0oooo000E0?ooo`030000003oool0
oooo0>X0oooo0P00000L0?ooo`005@3oool00`000000oooo0?ooo`3/0?ooo`030000003oool0oooo
01T0oooo000E0?ooo`030000003oool0oooo0>d0oooo0P00000I0?ooo`005@3oool200000?00oooo
00<000000?ooo`3oool05P3oool001D0oooo00<000000?ooo`3oool0l03oool00`000000oooo0?oo
o`0E0?ooo`005@3oool00`000000oooo0?ooo`3a0?ooo`030000003oool0oooo01@0oooo000E0?oo
o`030000003oool0oooo0?80oooo00<000000?ooo`3oool04`3oool001D0oooo00<000000?ooo`3o
ool0l`3oool2000001<0oooo000E0?ooo`030000003oool0oooo0?D0oooo00<000000?ooo`3oool0
403oool001D0oooo00<000000?ooo`3oool0mP3oool00`000000oooo0?ooo`0?0?ooo`005@3oool2
00000?P0oooo00<000000?ooo`3oool03P3oool001D0oooo00<000000?ooo`3oool0n03oool20000
00h0oooo000E0?ooo`030000003oool0oooo0?X0oooo00<000000?ooo`3oool02`3oool001D0oooo
00<000000?ooo`3oool0n`3oool2000000/0oooo000>0?ooo`8000001@3oool00`000000oooo0?oo
o`3m0?ooo`030000003oool0oooo00P0oooo000=0?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0oP3oool2000000P0oooo000@0?ooo`030000003oool0oooo0080oooo0`00003o
0?ooo`40oooo00<000000?ooo`3oool01@3oool000h0oooo0P0000050?ooo`030000003oool0oooo
0?l0oooo2@3oool00100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooo`T0
oooo000=0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0o`3oool90?ooo`00
3P3oool2000000D0oooo00<000000?ooo`3oool0o`3oool90?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, 0.369826, \
0.0115777, 0.0156006}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %%]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147151 0.192263 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .11085 -18 -4.5 ]
[.01131 .11085 0 4.5 ]
[.01131 .20698 -6 -4.5 ]
[.01131 .20698 0 4.5 ]
[.01131 .30311 -18 -4.5 ]
[.01131 .30311 0 4.5 ]
[.01131 .39924 -6 -4.5 ]
[.01131 .39924 0 4.5 ]
[.01131 .49537 -18 -4.5 ]
[.01131 .49537 0 4.5 ]
[.01131 .5915 -6 -4.5 ]
[.01131 .5915 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11085 m
.03006 .11085 L
s
[(0.5)] .01131 .11085 1 0 Mshowa
.02381 .20698 m
.03006 .20698 L
s
[(1)] .01131 .20698 1 0 Mshowa
.02381 .30311 m
.03006 .30311 L
s
[(1.5)] .01131 .30311 1 0 Mshowa
.02381 .39924 m
.03006 .39924 L
s
[(2)] .01131 .39924 1 0 Mshowa
.02381 .49537 m
.03006 .49537 L
s
[(2.5)] .01131 .49537 1 0 Mshowa
.02381 .5915 m
.03006 .5915 L
s
[(3)] .01131 .5915 1 0 Mshowa
.125 Mabswid
.02381 .03394 m
.02756 .03394 L
s
.02381 .05317 m
.02756 .05317 L
s
.02381 .07239 m
.02756 .07239 L
s
.02381 .09162 m
.02756 .09162 L
s
.02381 .13007 m
.02756 .13007 L
s
.02381 .1493 m
.02756 .1493 L
s
.02381 .16853 m
.02756 .16853 L
s
.02381 .18775 m
.02756 .18775 L
s
.02381 .2262 m
.02756 .2262 L
s
.02381 .24543 m
.02756 .24543 L
s
.02381 .26466 m
.02756 .26466 L
s
.02381 .28388 m
.02756 .28388 L
s
.02381 .32234 m
.02756 .32234 L
s
.02381 .34156 m
.02756 .34156 L
s
.02381 .36079 m
.02756 .36079 L
s
.02381 .38001 m
.02756 .38001 L
s
.02381 .41847 m
.02756 .41847 L
s
.02381 .43769 m
.02756 .43769 L
s
.02381 .45692 m
.02756 .45692 L
s
.02381 .47615 m
.02756 .47615 L
s
.02381 .5146 m
.02756 .5146 L
s
.02381 .53382 m
.02756 .53382 L
s
.02381 .55305 m
.02756 .55305 L
s
.02381 .57228 m
.02756 .57228 L
s
.02381 .61073 m
.02756 .61073 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .11116 m
.0873 .11715 L
.11905 .12339 L
.15079 .13108 L
.18254 .14048 L
.21429 .15161 L
.24603 .16439 L
.27778 .17867 L
.30952 .19426 L
.34127 .21094 L
.37302 .22848 L
.40476 .24667 L
.43651 .26529 L
.46825 .28418 L
.5 .30324 L
.53175 .32239 L
.56349 .34158 L
.59524 .3608 L
.62698 .38002 L
.65873 .39924 L
.69048 .41847 L
.72222 .43769 L
.75397 .45692 L
.78571 .47615 L
.81746 .49537 L
.84921 .5146 L
.88095 .53382 L
.9127 .55305 L
.94444 .57228 L
.97619 .5915 L
s
.05556 .11893 m
.0873 .1238 L
.11905 .13233 L
.15079 .1437 L
.18254 .15692 L
.21429 .17141 L
.24603 .1868 L
.27778 .20284 L
.30952 .21938 L
.34127 .2363 L
.37302 .25354 L
.40476 .27103 L
.43651 .28872 L
.46825 .30658 L
.5 .32459 L
.53175 .34272 L
.56349 .36095 L
.59524 .37928 L
.62698 .39768 L
.65873 .41615 L
.69048 .43468 L
.72222 .45327 L
.75397 .4719 L
.78571 .49058 L
.81746 .50929 L
.84921 .52804 L
.88095 .54682 L
.9127 .56563 L
.94444 .58446 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00>@3oool2000000@0oooo00<000000?ooo`3oool00P3oool300000280
oooo1000000Q0?ooo`@000000`3oool00`000000oooo0?ooo`020?ooo`<000008`3oool400000240
oooo100000030?ooo`030000003oool0oooo0080oooo0`00000S0?ooo`8000001`3oool003P0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0O0?ooo`040000003oool0oooo0000
00H0oooo000h0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`040?ooo`00>03oool010000000oooo0?ooo`0000080?ooo`<00000903oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo0`00000U0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool01`3oool3000002<0oooo0P0000070?ooo`00>03oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0103oool003P0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0R0?ooo`<000008P3oool30000
00T0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`00000Q0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool08P3oool010000000oooo0?ooo`0000060?ooo`005@3o
ool00`000000oooo0?ooo`0Q0?ooo`8000002@3oool4000002<0oooo00<000000?ooo`3oool08P3o
ool00`000000oooo0?ooo`070?ooo`@000008`3oool2000002<0oooo0P0000090?ooo`@000008P3o
ool2000000L0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?oo
o`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000>0?ooool00000
4`00000001D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool001D0oooo00<000000?ooo`3oool0:03oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool01@3oool0
01D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0
oooo000E0?ooo`800000o`3oool:0?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E
0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?oo
o`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo
2@3oool001D0oooo0P00003o0?ooo`X0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool0
01D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0
oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo0P00003o0?ooo`X0oooo000E
0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?oo
o`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo
2@3oool001D0oooo0P00003o0?ooo`X0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool0
01D0oooo00<000000?ooo`3oool0o`3oool90?ooo`000P3oool2000000@0oooo00<000000?ooo`3o
ool00P3oool3000000D0oooo00<000000?ooo`3oool0o`3oool90?ooo`0000D0oooo0000003oool0
oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool90?ooo`00
00D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo0`0000060?ooo`80
0000o`3oool10?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`000000
oooo0?ooo`080?ooo`@00000o03oool000050?ooo`000000oooo0?ooo`000000203oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00`0oooo1@00003g0?ooo`0000D0oooo0000003oool0
oooo000000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool4000000L0
oooo1000003c0?ooo`000P3oool2000000T0oooo100000040?ooo`030000003oool0oooo00X0oooo
1`0000040?ooo`@00000k`3oool001D0oooo00<000000?ooo`3oool04@3oool4000000@0oooo1000
003[0?ooo`005@3oool2000001H0oooo100000040?ooo`D00000iP3oool001D0oooo00<000000?oo
o`3oool06@3oool2000000L0oooo1000003R0?ooo`005@3oool00`000000oooo0?ooo`0K0?ooo`<0
0000203oool400000=h0oooo000E0?ooo`030000003oool0oooo01h0oooo0`0000090?ooo`<00000
f`3oool001D0oooo00<000000?ooo`3oool08@3oool2000000X0oooo0`00003H0?ooo`005@3oool2
000002@0oooo0P00000;0?ooo`800000eP3oool001D0oooo00<000000?ooo`3oool09@3oool20000
00/0oooo0P00003D0?ooo`005@3oool00`000000oooo0?ooo`0W0?ooo`<000002P3oool200000=80
oooo000E0?ooo`030000003oool0oooo02X0oooo0P00000:0?ooo`800000d03oool001D0oooo00<0
00000?ooo`3oool0;03oool2000000X0oooo0`00003=0?ooo`005@3oool2000002l0oooo0P00000;
0?ooo`800000b`3oool001D0oooo00<000000?ooo`3oool0<03oool2000000/0oooo0P0000390?oo
o`005@3oool00`000000oooo0?ooo`0b0?ooo`8000002`3oool200000<L0oooo000E0?ooo`030000
003oool0oooo03@0oooo0P00000;0?ooo`<00000a03oool001D0oooo00<000000?ooo`3oool0=P3o
ool2000000`0oooo0P0000320?ooo`005@3oool2000003T0oooo0`00000;0?ooo`800000`03oool0
01D0oooo00<000000?ooo`3oool0>`3oool2000000/0oooo0P00002n0?ooo`005@3oool00`000000
oooo0?ooo`0m0?ooo`8000002`3oool200000;`0oooo000=0?ooo`@00000103oool00`000000oooo
0?ooo`0o0?ooo`8000002`3oool200000;X0oooo000?0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0@@3oool2000000/0oooo00<000000?ooo`3oool0]`3oool000l0oooo00<00000
0?ooo`3oool00`3oool3000004<0oooo0P00000:0?ooo`800000]`3oool000l0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`150?ooo`8000002P3oool200000;D0oooo000?0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0A`3oool2000000X0oooo0P00002c0?ooo`00
3@3oool3000000D0oooo00<000000?ooo`3oool0B@3oool2000000X0oooo0P00002a0?ooo`003`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool02@3o
ool200000:l0oooo000E0?ooo`030000003oool0oooo04`0oooo0P00000;0?ooo`800000[@3oool0
01D0oooo0P00001?0?ooo`8000002`3oool200000:/0oooo000E0?ooo`030000003oool0oooo0500
oooo0P00000;0?ooo`800000Z@3oool001D0oooo00<000000?ooo`3oool0DP3oool2000000/0oooo
0P00002W0?ooo`005@3oool00`000000oooo0?ooo`1D0?ooo`8000002`3oool200000:D0oooo000E
0?ooo`030000003oool0oooo05H0oooo0P00000;0?ooo`800000X`3oool001D0oooo0P00001I0?oo
o`8000002`3oool00`000000oooo0?ooo`2P0?ooo`005@3oool00`000000oooo0?ooo`1J0?ooo`80
00002P3oool200000:00oooo000E0?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`2M0?ooo`005@3oool00`000000oooo0?ooo`1M0?ooo`8000002P3o
ool2000009d0oooo000E0?ooo`030000003oool0oooo05l0oooo0P00000:0?ooo`800000V`3oool0
01D0oooo0P00001R0?ooo`8000002P3oool00`000000oooo0?ooo`2H0?ooo`005@3oool00`000000
oooo0?ooo`1S0?ooo`8000002@3oool2000009P0oooo000E0?ooo`030000003oool0oooo06D0oooo
00<000000?ooo`3oool0203oool2000009H0oooo000E0?ooo`030000003oool0oooo06H0oooo0P00
000:0?ooo`800000U03oool001D0oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`09
0?ooo`800000TP3oool001D0oooo0P00001Z0?ooo`8000002`3oool00`000000oooo0?ooo`2?0?oo
o`005@3oool00`000000oooo0?ooo`1[0?ooo`8000002P3oool00`000000oooo0?ooo`2>0?ooo`00
5@3oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo00P0oooo0P00002>0?ooo`005@3o
ool00`000000oooo0?ooo`1^0?ooo`8000002P3oool00`000000oooo0?ooo`2;0?ooo`0000<0oooo
0000000000000P0000030?ooo`030000003oool0oooo0080oooo0`0000050?ooo`030000003oool0
oooo0700oooo0P0000090?ooo`030000003oool0oooo08X0oooo00030?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1b0?ooo`800000203oool20000
08X0oooo00030?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool00P3oool3000007@0
oooo0P0000080?ooo`030000003oool0oooo08L0oooo00030?ooo`030000003oool0oooo00L0oooo
0`0000050?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool01P3oool2000008L0oooo
00030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`1g0?ooo`800000203oool2000008D0oooo0000103oool000000000000000090?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0N@3oool2000000P0oooo0P0000230?ooo`000`3oool0
0`000000oooo0?ooo`070?ooo`@00000103oool00`000000oooo0?ooo`1k0?ooo`800000203oool2
00000840oooo000E0?ooo`800000OP3oool2000000P0oooo00<000000?ooo`3oool0OP3oool001D0
oooo00<000000?ooo`3oool0O`3oool2000000L0oooo0P00001n0?ooo`005@3oool00`000000oooo
0?ooo`210?ooo`8000001`3oool2000007`0oooo000E0?ooo`030000003oool0oooo08<0oooo0P00
00070?ooo`800000NP3oool001D0oooo00<000000?ooo`3oool0Q@3oool2000000L0oooo0P00001h
0?ooo`005@3oool2000008P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`1e0?oo
o`005@3oool00`000000oooo0?ooo`280?ooo`8000001`3oool2000007D0oooo000E0?ooo`030000
003oool0oooo08X0oooo0P0000070?ooo`030000003oool0oooo0780oooo000E0?ooo`030000003o
ool0oooo08`0oooo0P0000060?ooo`800000LP3oool001D0oooo00<000000?ooo`3oool0SP3oool2
000000H0oooo0P00001`0?ooo`005@3oool200000940oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`1]0?ooo`005@3oool00`000000oooo0?ooo`2A0?ooo`8000001P3oool2000006d0
oooo000E0?ooo`030000003oool0oooo09<0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`1Z0?ooo`005@3oool00`000000oooo0?ooo`2D0?ooo`8000001P3oool2000006X0oooo000E
0?ooo`030000003oool0oooo09H0oooo0P0000060?ooo`030000003oool0oooo06L0oooo000E0?oo
o`030000003oool0oooo09P0oooo00<000000?ooo`3oool0103oool2000006L0oooo000E0?ooo`80
0000VP3oool2000000H0oooo00<000000?ooo`3oool0I03oool001D0oooo00<000000?ooo`3oool0
V`3oool2000000D0oooo0P00001T0?ooo`005@3oool00`000000oooo0?ooo`2M0?ooo`8000001@3o
ool200000680oooo000=0?ooo`@00000103oool00`000000oooo0?ooo`2O0?ooo`8000001@3oool2
00000600oooo000=0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0X@3oool00`00
0000oooo0?ooo`040?ooo`800000GP3oool000h0oooo00<000000?ooo`3oool0103oool300000:80
oooo0P0000060?ooo`030000003oool0oooo05/0oooo000?0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0Y03oool2000000D0oooo0P00001K0?ooo`00403oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0:H0oooo0P0000050?ooo`030000003oool0oooo05P0oooo000=
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0Z03oool2000000@0oooo0P00
001H0?ooo`003P3oool2000000D0oooo00<000000?ooo`3oool0ZP3oool00`000000oooo0?ooo`03
0?ooo`800000EP3oool001D0oooo0P00002/0?ooo`8000001@3oool00`000000oooo0?ooo`1C0?oo
o`005@3oool00`000000oooo0?ooo`2]0?ooo`030000003oool0oooo00<0oooo0P00001C0?ooo`00
5@3oool00`000000oooo0?ooo`2^0?ooo`8000001@3oool200000540oooo000E0?ooo`030000003o
ool0oooo0;00oooo0P0000050?ooo`800000C`3oool001D0oooo00<000000?ooo`3oool0/P3oool0
0`000000oooo0?ooo`040?ooo`800000C@3oool001D0oooo0P00002d0?ooo`8000001P3oool00`00
0000oooo0?ooo`1:0?ooo`005@3oool00`000000oooo0?ooo`2e0?ooo`8000001@3oool2000004X0
oooo000E0?ooo`030000003oool0oooo0;L0oooo0P0000050?ooo`800000B03oool001D0oooo00<0
00000?ooo`3oool0^@3oool2000000D0oooo0P0000160?ooo`005@3oool00`000000oooo0?ooo`2k
0?ooo`030000003oool0oooo00@0oooo0P0000140?ooo`005@3oool200000;d0oooo0P0000060?oo
o`030000003oool0oooo0440oooo000E0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`100?ooo`005@3oool00`000000oooo0?ooo`2o0?ooo`800000
1@3oool200000400oooo000E0?ooo`030000003oool0oooo0<40oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0m0?ooo`005@3oool00`000000oooo0?ooo`320?ooo`8000001@3oool0
0`000000oooo0?ooo`0l0?ooo`005@3oool00`000000oooo0?ooo`340?ooo`030000003oool0oooo
00<0oooo0P00000l0?ooo`005@3oool200000<H0oooo0P0000050?ooo`030000003oool0oooo03T0
oooo000E0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool00`3oool2000003T0oooo
000E0?ooo`030000003oool0oooo0<P0oooo0P0000050?ooo`800000=`3oool000030?ooo`000000
0000008000000`3oool00`000000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`3:
0?ooo`8000001@3oool2000003D0oooo00000`3oool000000?ooo`0=0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0c03oool00`000000oooo0?ooo`040?ooo`800000<`3oool00080
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`020?ooo`<00000c@3oool2000000H0
oooo00<000000?ooo`3oool0<03oool000<0oooo00<000000?ooo`3oool01`3oool3000000D0oooo
00<000000?ooo`3oool0c`3oool2000000D0oooo0P00000`0?ooo`00103oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0d@3oool2000000D0oooo00<0
00000?ooo`3oool0;@3oool000050?ooo`000000oooo0?ooo`000000203oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo0=<0oooo0P0000040?ooo`800000;@3oool00080oooo0P000009
0?ooo`@00000103oool00`000000oooo0?ooo`3E0?ooo`030000003oool0oooo00<0oooo0P00000[
0?ooo`005@3oool200000=L0oooo0P0000050?ooo`030000003oool0oooo02P0oooo000E0?ooo`03
0000003oool0oooo0=P0oooo00<000000?ooo`3oool00`3oool2000002P0oooo000E0?ooo`030000
003oool0oooo0=T0oooo0P0000050?ooo`8000009P3oool001D0oooo00<000000?ooo`3oool0f`3o
ool2000000D0oooo0P00000T0?ooo`005@3oool00`000000oooo0?ooo`3M0?ooo`030000003oool0
oooo00@0oooo0P00000R0?ooo`005@3oool200000=l0oooo0P0000060?ooo`030000003oool0oooo
01l0oooo000E0?ooo`030000003oool0oooo0>00oooo0P0000050?ooo`8000007`3oool001D0oooo
00<000000?ooo`3oool0hP3oool2000000D0oooo00<000000?ooo`3oool0703oool001D0oooo00<0
00000?ooo`3oool0i03oool2000000@0oooo0P00000L0?ooo`005@3oool00`000000oooo0?ooo`3V
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool06@3oool001D0oooo00<000000?oo
o`3oool0i`3oool2000000@0oooo0P00000I0?ooo`005@3oool200000>X0oooo0P0000040?ooo`03
0000003oool0oooo01H0oooo000E0?ooo`030000003oool0oooo0>/0oooo0P0000030?ooo`800000
5P3oool001D0oooo00<000000?ooo`3oool0k@3oool2000000<0oooo00<000000?ooo`3oool04`3o
ool001D0oooo00<000000?ooo`3oool0k`3oool010000000oooo0?ooo`3oool2000001<0oooo000E
0?ooo`030000003oool0oooo0?00oooo0P0000030?ooo`8000004@3oool001D0oooo0P00003c0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool001D0oooo00<000000?ooo`3o
ool0l`3oool2000000<0oooo0P00000>0?ooo`005@3oool00`000000oooo0?ooo`3e0?ooo`800000
0`3oool2000000`0oooo000>0?ooo`8000001@3oool00`000000oooo0?ooo`3g0?ooo`030000003o
ool0oooo0080oooo0P00000:0?ooo`003@3oool010000000oooo0?ooo`0000040?ooo`030000003o
ool0oooo0?P0oooo0P0000040?ooo`800000203oool00100oooo00<000000?ooo`3oool00P3oool3
00000?X0oooo0P0000040?ooo`030000003oool0oooo00D0oooo000>0?ooo`8000001@3oool00`00
0000oooo0?ooo`3l0?ooo`8000002P3oool00100oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`3n0?ooo`800000203oool000d0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`3o0?ooo`40oooo00<000000?ooo`3oool01@3oool000h0oooo0P0000050?ooo`030000
003oool0oooo0?l0oooo2@3oool001D0oooo0P00003o0?ooo`X0oooo000E0?ooo`030000003oool0
oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -0.240782, \
0.0115777, 0.0191169}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(omthree[k_, th_, a_] := 
      om /. FindRoot[
          testfunc[om, IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), th, 
              a] \[Equal] 0, {om, k + 3/10 + If[k < 3/10, 2/10, 0], 
            k + 35/100 + If[k < 3/10, 25/100, 0]}, 
          MaxIterations \[Rule] 25]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omthree[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, \(-9\)/10]}, {T, 
        0.1, 3, 0.1}]\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.6795569654121059`}, {0.2`, 
        0.7064116927168467`}, {0.30000000000000004`, 
        0.7471619046536044`}, {0.4`, 0.797664867012399`}, {0.5`, 
        0.8540907169602043`}, {0.6000000000000001`, 
        0.9133964021688734`}, {0.7000000000000001`, 
        1.0769331550062493`}, {0.8`, 1.14527857040528`}, {0.9`, 
        1.219864441023772`}, {1.`, 1.2989091642434294`}, {1.1`, 
        1.381162593664805`}, {1.2000000000000002`, 
        1.4658022590778426`}, {1.3`, 
        1.5522826442397792`}, {1.4000000000000001`, 
        1.6402294410937328`}, {1.5`, 1.7293756395347029`}, {1.6`, 
        1.8195237554963726`}, {1.7000000000000002`, 
        1.910523084312625`}, {1.8`, 
        2.0022555494515215`}, {1.9000000000000001`, 
        2.094626579087424`}, {2.`, 2.1875590185675278`}, {2.1`, 
        2.2809889388171705`}, {2.2`, 
        2.3748626678330993`}, {2.3000000000000003`, 
        2.4691346352423413`}, {2.4000000000000004`, 
        2.563765772288345`}, {2.5`, 2.6587223006736944`}, {2.6`, 
        2.7539747997154604`}, {2.7`, 
        2.849497476685298`}, {2.8000000000000003`, 
        2.945267588160741`}, {2.9000000000000004`, 
        3.0412649754366683`}, {3.`, 3.137471687355168`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<ximom3.dat\>", %]\)], "Input"],

Cell[BoxData[
    \("ximom3.dat"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(special\ treatment\ for\ low\ freq' s\)], "Input"],

Cell[BoxData[
    \(omthree[k_, th_, a_] := 
      om /. FindRoot[
          testfunc[om, IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), th, 
              a] \[Equal] 0, {om, k + 7/10, k + 9/10}, 
          MaxIterations \[Rule] 25]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      testfunc[IntegerPart[om*10^\((16)\)]*10^\((\(-16\))\), 1/10, 
        Pi/4, \(-9\)/10], {om, 0.1, 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.0149132 0.00591793 [
[.21429 .00241 -9 -9 ]
[.21429 .00241 9 0 ]
[.40476 .00241 -9 -9 ]
[.40476 .00241 9 0 ]
[.59524 .00241 -9 -9 ]
[.59524 .00241 9 0 ]
[.78571 .00241 -9 -9 ]
[.78571 .00241 9 0 ]
[.97619 .00241 -3 -9 ]
[.97619 .00241 3 0 ]
[.01131 .13327 -12 -4.5 ]
[.01131 .13327 0 4.5 ]
[.01131 .25163 -12 -4.5 ]
[.01131 .25163 0 4.5 ]
[.01131 .36999 -12 -4.5 ]
[.01131 .36999 0 4.5 ]
[.01131 .48835 -12 -4.5 ]
[.01131 .48835 0 4.5 ]
[.01131 .60671 -18 -4.5 ]
[.01131 .60671 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .01491 m
.21429 .02116 L
s
[(0.2)] .21429 .00241 0 1 Mshowa
.40476 .01491 m
.40476 .02116 L
s
[(0.4)] .40476 .00241 0 1 Mshowa
.59524 .01491 m
.59524 .02116 L
s
[(0.6)] .59524 .00241 0 1 Mshowa
.78571 .01491 m
.78571 .02116 L
s
[(0.8)] .78571 .00241 0 1 Mshowa
.97619 .01491 m
.97619 .02116 L
s
[(1)] .97619 .00241 0 1 Mshowa
.125 Mabswid
.07143 .01491 m
.07143 .01866 L
s
.11905 .01491 m
.11905 .01866 L
s
.16667 .01491 m
.16667 .01866 L
s
.2619 .01491 m
.2619 .01866 L
s
.30952 .01491 m
.30952 .01866 L
s
.35714 .01491 m
.35714 .01866 L
s
.45238 .01491 m
.45238 .01866 L
s
.5 .01491 m
.5 .01866 L
s
.54762 .01491 m
.54762 .01866 L
s
.64286 .01491 m
.64286 .01866 L
s
.69048 .01491 m
.69048 .01866 L
s
.7381 .01491 m
.7381 .01866 L
s
.83333 .01491 m
.83333 .01866 L
s
.88095 .01491 m
.88095 .01866 L
s
.92857 .01491 m
.92857 .01866 L
s
.25 Mabswid
0 .01491 m
1 .01491 L
s
.02381 .13327 m
.03006 .13327 L
s
[(20)] .01131 .13327 1 0 Mshowa
.02381 .25163 m
.03006 .25163 L
s
[(40)] .01131 .25163 1 0 Mshowa
.02381 .36999 m
.03006 .36999 L
s
[(60)] .01131 .36999 1 0 Mshowa
.02381 .48835 m
.03006 .48835 L
s
[(80)] .01131 .48835 1 0 Mshowa
.02381 .60671 m
.03006 .60671 L
s
[(100)] .01131 .60671 1 0 Mshowa
.125 Mabswid
.02381 .0445 m
.02756 .0445 L
s
.02381 .07409 m
.02756 .07409 L
s
.02381 .10368 m
.02756 .10368 L
s
.02381 .16286 m
.02756 .16286 L
s
.02381 .19245 m
.02756 .19245 L
s
.02381 .22204 m
.02756 .22204 L
s
.02381 .28122 m
.02756 .28122 L
s
.02381 .31081 m
.02756 .31081 L
s
.02381 .3404 m
.02756 .3404 L
s
.02381 .39958 m
.02756 .39958 L
s
.02381 .42917 m
.02756 .42917 L
s
.02381 .45876 m
.02756 .45876 L
s
.02381 .51794 m
.02756 .51794 L
s
.02381 .54753 m
.02756 .54753 L
s
.02381 .57712 m
.02756 .57712 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.12912 .61803 m
.12917 .61576 L
.13142 .5339 L
.13377 .46808 L
.13597 .41892 L
.14046 .34352 L
.14457 .29356 L
.14931 .25016 L
.15382 .21845 L
.15837 .193 L
.16322 .17112 L
.17195 .14122 L
.18233 .11598 L
.19174 .09923 L
.20076 .08682 L
.20906 .07766 L
.21875 .06896 L
.2278 .06232 L
.24474 .05264 L
.26334 .04486 L
.28166 .03913 L
.29836 .03507 L
.31698 .03147 L
.33422 .02879 L
.36871 .02477 L
.38551 .02327 L
.40405 .02188 L
.44024 .01978 L
.47506 .0183 L
.51208 .01714 L
.54774 .01633 L
.58204 .01575 L
.60117 .0155 L
.61855 .01531 L
.65369 .01502 L
.67095 .01491 L
.68967 .01483 L
.70756 .01477 L
.71658 .01475 L
.72167 .01474 L
.7265 .01473 L
.73087 .01473 L
.73564 .01472 L
.73774 .01472 L
.73998 .01472 L
.74207 .01472 L
.744 .01472 L
.74644 .01472 L
.74773 .01472 L
.74838 .01472 L
Mistroke
.74908 .01472 L
.75027 .01472 L
.75137 .01472 L
.75204 .01472 L
.75268 .01472 L
.75325 .01472 L
.75387 .01472 L
.75494 .01472 L
.75612 .01472 L
.75736 .01472 L
.75851 .01472 L
.76058 .01472 L
.7628 .01472 L
.76728 .01472 L
.77144 .01472 L
.78087 .01474 L
.79007 .01475 L
.79988 .01477 L
.81912 .01482 L
.83727 .01489 L
.87194 .01504 L
.90881 .01525 L
.94432 .0155 L
.97619 .01575 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00@P3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000002D0
oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool08`3oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000002H0oooo0P0000040?ooo`030000003oool0oooo00<0
oooo0P00000Z0?ooo`@000001`3oool00440oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`0U0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool08P3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000002@0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`00000[0?ooo`030000003oool0oooo00H0oooo00110?ooo`040000
003oool0oooo000000T0oooo00<000000?ooo`3oool0903oool010000000oooo0?ooo`0000070?oo
o`D000008`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002@0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`00000[0?ooo`030000003oool0oooo00H0
oooo00110?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool08`3oool010000000
oooo0?ooo`0000070?ooo`040000003oool0oooo000002@0oooo00@000000?ooo`3oool00000203o
ool3000002D0oooo00@000000?ooo`3oool000002@3oool2000002`0oooo00<000000?ooo`3oool0
1P3oool00440oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0R0?ooo`040000
003oool0oooo000000P0oooo00<000000?ooo`000000903oool010000000oooo0?ooo`0000080?oo
o`030000003oool0oooo02D0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
000[0?ooo`030000003oool0oooo00H0oooo00110?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool00000903oool010000000oooo0?ooo`0000090?ooo`800000903oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo02D0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`00000Y0?ooo`<00000203oool001H0oooo00<000000?ooo`3oool0:@3oool2
000000X0oooo0P00000V0?ooo`8000002`3oool00`000000oooo0?ooo`0S0?ooo`8000002P3oool3
000002D0oooo0P00000:0?ooo`800000;03oool00`000000oooo0?ooo`060?ooo`005P3oool00`00
0000oooo0?ooo`3o0?ooo`P0oooo000F0?ooo`030000003oool0oooo0?l0oooo203oool001H0oooo
00<000000?ooo`3oool0o`3oool80?ooo`003`3ooooo0000014000000@3oool001H0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo7@000004
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00H0oooo000F0?ooo`030000003oool0oooo0300oooo00<000000?oo
o`3oool0:P3oool<000002h0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0`0?oo
o`030000003oool0oooo00H0oooo000F0?ooo`030000003oool0oooo05H0oooo1`00002Z0?ooo`00
5P3oool00`000000oooo0?ooo`1:0?ooo``00000/@3oool001H0oooo00<000000?ooo`3oool0A@3o
ool500000;d0oooo000F0?ooo`030000003oool0oooo0440oooo100000320?ooo`005P3oool00`00
0000oooo0?ooo`0o0?ooo`800000aP3oool001H0oooo0P00000m0?ooo`<00000b03oool001H0oooo
00<000000?ooo`3oool0>P3oool200000</0oooo000F0?ooo`030000003oool0oooo03P0oooo0P00
003=0?ooo`005P3oool00`000000oooo0?ooo`0f0?ooo`800000c`3oool001H0oooo00<000000?oo
o`3oool0=@3oool00`000000oooo0?ooo`3?0?ooo`005P3oool00`000000oooo0?ooo`0d0?ooo`03
0000003oool0oooo0=00oooo000F0?ooo`030000003oool0oooo0380oooo0P00003C0?ooo`005P3o
ool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0=<0oooo000F0?ooo`800000<@3oool0
0`000000oooo0?ooo`3D0?ooo`005P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo
0=D0oooo000F0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0eP3oool001H0oooo
00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`3G0?ooo`005P3oool00`000000oooo0?oo
o`0/0?ooo`030000003oool0oooo0=P0oooo000F0?ooo`030000003oool0oooo02`0oooo00<00000
0?ooo`3oool0f03oool001H0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`3I0?oo
o`005P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0=X0oooo000F0?ooo`800000
:`3oool00`000000oooo0?ooo`3J0?ooo`005P3oool00`000000oooo0?ooo`0Y0?ooo`030000003o
ool0oooo0=/0oooo000F0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0f`3oool0
01H0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`3L0?ooo`005P3oool00`000000
oooo0?ooo`0X0?ooo`030000003oool0oooo0=`0oooo000F0?ooo`030000003oool0oooo02L0oooo
00<000000?ooo`3oool0g@3oool000L0oooo100000030?ooo`8000001P3oool00`000000oooo0?oo
o`0W0?ooo`030000003oool0oooo0=d0oooo00070?ooo`030000003oool0oooo00<0oooo00@00000
0?ooo`3oool000001@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0=h0oooo0008
0?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool000001@3oool2000002L0oooo00<0
00000?ooo`3oool0gP3oool000T0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`3O0?ooo`002P3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool09@3o
ool00`000000oooo0?ooo`3O0?ooo`001`3oool010000000oooo0?ooo`0000020?ooo`040000003o
ool0oooo000000D0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`3O0?ooo`00203o
ool2000000@0oooo0P0000060?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0h03o
ool001H0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`3P0?ooo`005P3oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo0>00oooo000F0?ooo`030000003oool0oooo02<0
oooo00<000000?ooo`3oool0h@3oool001H0oooo0P00000T0?ooo`030000003oool0oooo0>40oooo
000F0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0hP3oool001H0oooo00<00000
0?ooo`3oool08P3oool00`000000oooo0?ooo`3R0?ooo`005P3oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo0>80oooo000F0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3o
ool0hP3oool001H0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`3R0?ooo`005P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0><0oooo000F0?ooo`030000003oool0
oooo0240oooo00<000000?ooo`3oool0h`3oool001H0oooo0P00000R0?ooo`030000003oool0oooo
0><0oooo000F0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0h`3oool001H0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`3S0?ooo`005P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0><0oooo000F0?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool0i03oool001H0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3T0?oo
o`005P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0>@0oooo000F0?ooo`030000
003oool0oooo0200oooo00<000000?ooo`3oool0i03oool001H0oooo0P00000Q0?ooo`030000003o
ool0oooo0>@0oooo000F0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0i03oool0
01H0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3T0?ooo`005P3oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo0>@0oooo000F0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool0i@3oool001H0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`3U0?ooo`002@3oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo0>D0oooo00090?ooo`050000003oool0oooo0?ooo`0000000P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0i@3oool0
00H0oooo1@0000020?ooo`040000003oool0oooo000000D0oooo0P00000P0?ooo`030000003oool0
oooo0>D0oooo00060?ooo`040000003oool0oooo000000<0oooo00@000000?ooo`3oool000001@3o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0>D0oooo00070?ooo`030000003oool0
000000<0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo0>D0oooo00080?ooo`8000000`3oool010000000oooo0?ooo`0000050?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool0iP3oool000T0oooo00<000000?ooo`3oool00P3oool2
000000H0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`3V0?ooo`005P3oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo0>H0oooo000F0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool0iP3oool001H0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`3V0?ooo`005P3oool2000001l0oooo00<000000?ooo`3oool0iP3oool001H0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`3W0?ooo`005P3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo0>L0oooo000F0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3o
ool0i`3oool001H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`3W0?ooo`005P3o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0>L0oooo000F0?ooo`030000003oool0
oooo01d0oooo00<000000?ooo`3oool0i`3oool001H0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`3W0?ooo`005P3oool2000001h0oooo00<000000?ooo`3oool0i`3oool001H0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`3W0?ooo`005P3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo0>L0oooo000F0?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool0j03oool001H0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3X0?oo
o`005P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0>P0oooo000F0?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool0j03oool001H0oooo00<000000?ooo`3oool0703o
ool00`000000oooo0?ooo`3X0?ooo`005P3oool2000001d0oooo00<000000?ooo`3oool0j03oool0
01H0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3X0?ooo`005P3oool00`000000
oooo0?ooo`0L0?ooo`030000003oool0oooo0>P0oooo000F0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool0j03oool001H0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`3X0?ooo`005P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0>P0oooo00080?oo
o`800000103oool2000000H0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3X0?oo
o`001`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`3X0?ooo`001`3oool010000000oooo0?ooo`000002
0?ooo`040000003oool0oooo000000D0oooo0P00000M0?ooo`030000003oool0oooo0>P0oooo0007
0?ooo`<000000`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool0j03oool000L0oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`00
00050?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0j03oool000L0oooo00<00000
0?ooo`3oool00`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool0j03oool000P0oooo0`0000030?ooo`8000001P3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo0>T0oooo000F0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool0j@3oool001H0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3Y0?ooo`00
5P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0>T0oooo000F0?ooo`800000703o
ool00`000000oooo0?ooo`3Y0?ooo`005P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo0>T0oooo000F0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0j@3oool001H0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3Y0?ooo`005P3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo0>T0oooo000F0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool0j@3oool001H0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3Y
0?ooo`005P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0>T0oooo000F0?ooo`80
0000703oool00`000000oooo0?ooo`3Y0?ooo`005P3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo0>T0oooo000F0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0j@3o
ool001H0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3Y0?ooo`005P3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo0>T0oooo000F0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool0jP3oool001H0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`3Z0?ooo`005P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>X0oooo000F
0?ooo`8000006`3oool00`000000oooo0?ooo`3Z0?ooo`005P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo0>X0oooo000F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool0jP3oool001H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3Z0?ooo`005P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>X0oooo000F0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool0jP3oool000P0oooo0P0000040?ooo`8000001P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo0>X0oooo00070?ooo`040000003oool0oooo0000
0080oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo0>X0oooo00070?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3o
ool2000001/0oooo00<000000?ooo`3oool0jP3oool000P0oooo0P0000030?ooo`040000003oool0
oooo000000D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3Z0?ooo`001`3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`3Z0?ooo`001`3oool010000000oooo0?ooo`0000020?ooo`040000
003oool0oooo000000D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3Z0?ooo`00
203oool2000000@0oooo0P0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
jP3oool001H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3Z0?ooo`005P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>X0oooo000F0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool0jP3oool001H0oooo0P00000K0?ooo`030000003oool0oooo0>X0
oooo000F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0jP3oool001H0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`3Z0?ooo`005P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo0>X0oooo000F0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0jP3oool001H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3Z0?ooo`00
5P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>X0oooo000F0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0jP3oool001H0oooo0P00000K0?ooo`030000003oool0
oooo0>X0oooo000F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0jP3oool001H0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3Z0?ooo`005P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo0>X0oooo000F0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool0jP3oool001H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3Z
0?ooo`005P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>X0oooo000F0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool0jP3oool001H0oooo0P00000J0?ooo`030000
003oool0oooo0>/0oooo000F0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0j`3o
ool001H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`3[0?ooo`005P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo0>/0oooo000F0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool0j`3oool001H0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`3[0?ooo`0000<0oooo0000000000000P0000030?ooo`800000103oool2000000H0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`3[0?ooo`000`3oool01@000000oooo0?ooo`3o
ool000000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0j`3oool000<0oooo00D000000?ooo`3oool0oooo
000000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00`3oool2000001X0
oooo00<000000?ooo`3oool0j`3oool000<0oooo00D000000?ooo`3oool0oooo000000020?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo0>/0oooo00030?ooo`050000003oool0oooo0?ooo`0000000P3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`3[0?ooo`0000@0oooo00000000000000000`3oool010000000oooo0?ooo`00
00020?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`3[0?ooo`000`3oool00`000000oooo0?ooo`020?ooo`800000103oool200000?l0oooo4@3o
ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0861204, -7.7364, \
0.00388235, 0.624792}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, PlotRange \[Rule] {\(-0.1\), 0.1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.309017 3.09017 [
[.21429 .29652 -9 -9 ]
[.21429 .29652 9 0 ]
[.40476 .29652 -9 -9 ]
[.40476 .29652 9 0 ]
[.59524 .29652 -9 -9 ]
[.59524 .29652 9 0 ]
[.78571 .29652 -9 -9 ]
[.78571 .29652 9 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.01131 0 -24 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .07725 -36 -4.5 ]
[.01131 .07725 0 4.5 ]
[.01131 .15451 -30 -4.5 ]
[.01131 .15451 0 4.5 ]
[.01131 .23176 -36 -4.5 ]
[.01131 .23176 0 4.5 ]
[.01131 .38627 -30 -4.5 ]
[.01131 .38627 0 4.5 ]
[.01131 .46353 -24 -4.5 ]
[.01131 .46353 0 4.5 ]
[.01131 .54078 -30 -4.5 ]
[.01131 .54078 0 4.5 ]
[.01131 .61803 -18 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .30902 m
.21429 .31527 L
s
[(0.2)] .21429 .29652 0 1 Mshowa
.40476 .30902 m
.40476 .31527 L
s
[(0.4)] .40476 .29652 0 1 Mshowa
.59524 .30902 m
.59524 .31527 L
s
[(0.6)] .59524 .29652 0 1 Mshowa
.78571 .30902 m
.78571 .31527 L
s
[(0.8)] .78571 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(1)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.2619 .30902 m
.2619 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.5 .30902 m
.5 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.7381 .30902 m
.7381 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 0 m
.03006 0 L
s
[(-0.1)] .01131 0 1 0 Mshowa
.02381 .07725 m
.03006 .07725 L
s
[(-0.075)] .01131 .07725 1 0 Mshowa
.02381 .15451 m
.03006 .15451 L
s
[(-0.05)] .01131 .15451 1 0 Mshowa
.02381 .23176 m
.03006 .23176 L
s
[(-0.025)] .01131 .23176 1 0 Mshowa
.02381 .38627 m
.03006 .38627 L
s
[(0.025)] .01131 .38627 1 0 Mshowa
.02381 .46353 m
.03006 .46353 L
s
[(0.05)] .01131 .46353 1 0 Mshowa
.02381 .54078 m
.03006 .54078 L
s
[(0.075)] .01131 .54078 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.1)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .01545 m
.02756 .01545 L
s
.02381 .0309 m
.02756 .0309 L
s
.02381 .04635 m
.02756 .04635 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .10816 m
.02756 .10816 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .13906 m
.02756 .13906 L
s
.02381 .16996 m
.02756 .16996 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .20086 m
.02756 .20086 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26266 m
.02756 .26266 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .29357 m
.02756 .29357 L
s
.02381 .32447 m
.02756 .32447 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .35537 m
.02756 .35537 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .41717 m
.02756 .41717 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .44807 m
.02756 .44807 L
s
.02381 .47898 m
.02756 .47898 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .50988 m
.02756 .50988 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57168 m
.02756 .57168 L
s
.02381 .58713 m
.02756 .58713 L
s
.02381 .60258 m
.02756 .60258 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.60053 .61803 m
.60117 .61366 L
.61855 .51426 L
.65369 .36259 L
.67095 .30916 L
.68967 .26499 L
.70756 .23496 L
.71658 .224 L
.72167 .21899 L
.7265 .21499 L
.73087 .212 L
.73564 .2094 L
.73774 .20847 L
.73998 .20763 L
.74207 .20697 L
.744 .20647 L
.74644 .206 L
.74773 .20582 L
.74838 .20574 L
.74908 .20568 L
.75027 .2056 L
.75137 .20556 L
.75204 .20555 L
.75268 .20556 L
.75325 .20557 L
.75387 .20559 L
.75494 .20566 L
.75612 .20577 L
.75736 .20593 L
.75851 .20612 L
.76058 .20653 L
.7628 .20711 L
.76728 .20868 L
.77144 .2106 L
.78087 .21658 L
.79007 .22453 L
.79988 .23522 L
.81912 .26254 L
.83727 .29556 L
.87194 .37675 L
.90881 .48686 L
.94432 .61381 L
s
.94432 .61381 m
.94536 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00503oool2000000@0oooo00<000000?ooo`3oool0
0P3oool400000?d0oooo000C0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
o03oool001<0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`800000
m`3oool000d0oooo100000020?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`3f0?ooo`004`3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0mP3oool001<0oooo00@000000?ooo`3oool00000
203oool3000000D0oooo00<000000?ooo`3oool0mP3oool001@0oooo0P00000;0?ooo`030000003o
ool0oooo00<0oooo0P00003g0?ooo`009`3oool00`000000oooo0?ooo`3f0?ooo`009`3oool00`00
0000oooo0?ooo`3f0?ooo`009`3oool200000?L0oooo000W0?ooo`030000003oool0oooo0?H0oooo
000W0?ooo`030000003oool0oooo0?H0oooo000W0?ooo`030000003oool0oooo0?H0oooo000W0?oo
o`800000m`3oool002L0oooo00<000000?ooo`3oool0mP3oool002L0oooo00<000000?ooo`3oool0
mP3oool002L0oooo00<000000?ooo`3oool0mP3oool002L0oooo0P00003g0?ooo`009`3oool00`00
0000oooo0?ooo`3f0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0
oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0mP3oool000L0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3f0?ooo`001`3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`020?ooo`800000m`3oool000030?ooo`0000000000008000000P3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0oooo
0`0000050?ooo`030000003oool0oooo0?H0oooo00070?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000001@3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo
0?H0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0mP3o
ool000P0oooo0P00000:0?ooo`8000000`3oool400000080oooo100000040?ooo`800000m`3oool0
02L0oooo00<000000?ooo`3oool0mP3oool002L0oooo00<000000?ooo`3oool0mP3oool002L0oooo
00<000000?ooo`3oool0mP3oool002L0oooo0P00003g0?ooo`009`3oool00`000000oooo0?ooo`3f
0?ooo`009`3oool00`000000oooo0?ooo`3f0?ooo`009`3oool00`000000oooo0?ooo`3f0?ooo`00
9`3oool200000?L0oooo000W0?ooo`030000003oool0oooo0?H0oooo000W0?ooo`030000003oool0
oooo0?H0oooo000W0?ooo`030000003oool0oooo0?H0oooo000W0?ooo`800000m`3oool002L0oooo
00<000000?ooo`3oool0mP3oool000h0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P00
00030?ooo`<000001@3oool00`000000oooo0?ooo`3f0?ooo`003@3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`3f0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool00P3oool200000?L0oooo00070?ooo`@000000P3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`0000050?ooo`030000003oool0oooo
0?H0oooo000=0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0?H0oooo000=0?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo0?H0oooo000>0?ooo`8000002P3oool2000000<0oooo100000040?ooo`800000m`3oool0
02L0oooo00<000000?ooo`3oool0mP3oool002L0oooo00<000000?ooo`3oool0mP3oool002L0oooo
00<000000?ooo`3oool0mP3oool002L0oooo0P00003g0?ooo`009`3oool00`000000oooo0?ooo`3f
0?ooo`009`3oool00`000000oooo0?ooo`3f0?ooo`009`3oool00`000000oooo0?ooo`3f0?ooo`00
9`3oool200000?L0oooo000W0?ooo`030000003oool0oooo0;80oooo1`00000m0?ooo`009`3oool0
0`000000oooo0?ooo`2`0?ooo`<000001P3oool2000003/0oooo000W0?ooo`800000/03oool00`00
0000oooo0?ooo`090?ooo`800000>@3oool002L0oooo00<000000?ooo`3oool0[P3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo03H0oooo00080?ooo`800000103oool00`000000oooo
0?ooo`030?ooo`8000000`3oool400000080oooo0`0000050?ooo`030000003oool0oooo0:d0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0e0?ooo`001`3oool010000000oooo0?oo
o`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0:`0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0d0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`800000[03oool00`00
0000oooo0?ooo`0B0?ooo`800000=03oool000030?ooo`0000000000008000000P3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0oooo0`00
00050?ooo`030000003oool0oooo0:X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`0a0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo
00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`2Z0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool0<03oool000L0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0<03oool0
00P0oooo0P00000:0?ooo`800000103oool2000000<0oooo100000040?ooo`800000ZP3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo02l0oooo000W0?ooo`030000003oool0oooo0:P0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0^0?ooo`009`3oool00`000000oooo
0?ooo`2X0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0;@3oool002L0oooo00<0
00000?ooo`3oool0Y`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02d0oooo000W
0?ooo`800000Z03oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02`0oooo000W0?oo
o`030000003oool0oooo0:H0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0/0?oo
o`009`3oool00`000000oooo0?ooo`0V0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@0
00008@3oool2000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0Q0?ooo`80
0000103oool00`000000oooo0?ooo`030?ooo`8000004@3oool00`000000oooo0?ooo`0>0?ooo`80
0000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool00`000000oooo0?ooo`0Q0?ooo`@0
00001P3oool002L0oooo00<000000?ooo`3oool09@3oool010000000oooo0?ooo`0000080?ooo`03
0000003oool0oooo0240oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0P0?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000003`3oool00`000000oooo0?oo
o`0>0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo00D0oooo000W0?ooo`8000009P3oool010000000oooo
0?ooo`0000090?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool000001`3oool50000
0240oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000?0?ooo`030000003o
ool0oooo00h0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000030?ooo`03
0000003oool0oooo0280oooo00<000000?ooo`3oool01@3oool002L0oooo00<000000?ooo`3oool0
9@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo01l0oooo00@000000?ooo`3o
ool000001`3oool010000000oooo0?ooo`00000R0?ooo`040000003oool0oooo000000P0oooo0`00
000?0?ooo`030000003oool0oooo00l0oooo00@000000?ooo`3oool000002@3oool2000000@0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`050?ooo`009`3oool00`000000oooo0?oo
o`0U0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool07P3oool010000000oooo
0?ooo`0000080?ooo`030000003oool000000280oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00l0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool01@3o
ool002L0oooo00<000000?ooo`3oool09@3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000200oooo00@000000?ooo`3oool000002@3oool200000280oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0100oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo01l0oooo0`00
00070?ooo`009`3oool2000002L0oooo0P00000:0?ooo`8000008P3oool2000000/0oooo00<00000
0?ooo`3oool08@3oool2000000X0oooo0`00000=0?ooo`030000003oool0oooo0140oooo0P00000:
0?ooo`8000001P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo00D0oooo000W0?oo
o`030000003oool0oooo0:80oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0X0?oo
o`009`3oool00`000000oooo0?ooo`2R0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3o
ool0:03oool002L0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo02L0oooo000Q0?ooool00000000W0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo000000P0oooo00<0
00000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00D0oooo000W
0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0^
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool01@3oool002L0oooo00<000000?oo
o`3oool0X03oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo02D0oooo000W0?ooo`80
0000X@3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo02D0oooo000W0?ooo`030000
003oool0oooo0:00oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0T0?ooo`009`3o
ool00`000000oooo0?ooo`2O0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0903o
ool002L0oooo00<000000?ooo`3oool0W`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0
oooo02@0oooo000W0?ooo`800000X03oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
02<0oooo000W0?ooo`030000003oool0oooo09h0oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`0S0?ooo`009`3oool00`000000oooo0?ooo`2N0?ooo`030000003oool0oooo0300oooo
00<000000?ooo`3oool08P3oool002L0oooo00<000000?ooo`3oool0WP3oool00`000000oooo0?oo
o`0`0?ooo`030000003oool0oooo0280oooo000W0?ooo`800000W`3oool00`000000oooo0?ooo`0a
0?ooo`030000003oool0oooo0240oooo000W0?ooo`030000003oool0oooo09d0oooo00<000000?oo
o`3oool0<P3oool00`000000oooo0?ooo`0Q0?ooo`009`3oool00`000000oooo0?ooo`2M0?ooo`03
0000003oool0oooo03<0oooo00<000000?ooo`3oool0803oool002L0oooo00<000000?ooo`3oool0
W@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0200oooo000W0?ooo`800000WP3o
ool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01l0oooo000W0?ooo`030000003oool0
oooo09`0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0O0?ooo`00203oool20000
00@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo100000020?ooo`<000001@3oool00`00
0000oooo0?ooo`2L0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool07`3oool000L0
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2L0?ooo`030000003oool0
oooo03H0oooo00<000000?ooo`3oool07P3oool000L0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000030?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3o
ool2000009d0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`001`3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0
oooo0`0000050?ooo`030000003oool0oooo09/0oooo00<000000?ooo`3oool0=`3oool00`000000
oooo0?ooo`0N0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`2K0?ooo`030000003oool0
oooo03P0oooo00<000000?ooo`3oool07@3oool000L0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`2K0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0
7@3oool000P0oooo0P00000:0?ooo`800000103oool2000000<0oooo100000040?ooo`800000W03o
ool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo01d0oooo000W0?ooo`030000003oool0
oooo09X0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0L0?ooo`009`3oool00`00
0000oooo0?ooo`2J0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0703oool002L0
oooo0P00002K0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0703oool002L0oooo
00<000000?ooo`3oool0VP3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01/0oooo
000W0?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?oo
o`0K0?ooo`009`3oool00`000000oooo0?ooo`2I0?ooo`030000003oool0oooo03`0oooo00<00000
0?ooo`3oool06`3oool002L0oooo0P00002J0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool06`3oool002L0oooo00<000000?ooo`3oool0V@3oool00`000000oooo0?ooo`0m0?ooo`03
0000003oool0oooo01X0oooo000W0?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3oool0
?@3oool00`000000oooo0?ooo`0J0?ooo`009`3oool00`000000oooo0?ooo`2H0?ooo`030000003o
ool0oooo03h0oooo00<000000?ooo`3oool06P3oool002L0oooo0P00002I0?ooo`030000003oool0
oooo03l0oooo00<000000?ooo`3oool06@3oool002L0oooo00<000000?ooo`3oool0V03oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo01T0oooo000>0?ooo`800000103oool00`000000
oooo0?ooo`030?ooo`8000000`3oool3000000D0oooo00<000000?ooo`3oool0V03oool00`000000
oooo0?ooo`0o0?ooo`030000003oool0oooo01T0oooo000=0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
09L0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0H0?ooo`003@3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3oool2
000009P0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0H0?ooo`003@3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`0000050?ooo`030000003o
ool0oooo09L0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0H0?ooo`003@3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`2G0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0
5`3oool000d0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0UP3oool00`000000oooo0?ooo`130?ooo`03
0000003oool0oooo01L0oooo000>0?ooo`8000002P3oool2000000<0oooo100000040?ooo`800000
U`3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo01L0oooo000W0?ooo`030000003o
ool0oooo09H0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`0F0?ooo`009`3oool0
0`000000oooo0?ooo`2F0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool05P3oool0
02L0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo
01H0oooo000W0?ooo`800000UP3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo01D0
oooo000W0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0AP3oool00`000000oooo
0?ooo`0E0?ooo`009`3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo04H0oooo00<0
00000?ooo`3oool05@3oool002L0oooo00<000000?ooo`3oool0U03oool00`000000oooo0?ooo`17
0?ooo`030000003oool0oooo01D0oooo000W0?ooo`800000U@3oool00`000000oooo0?ooo`180?oo
o`030000003oool0oooo01@0oooo000W0?ooo`030000003oool0oooo09@0oooo00<000000?ooo`3o
ool0B03oool00`000000oooo0?ooo`0D0?ooo`009`3oool00`000000oooo0?ooo`2D0?ooo`030000
003oool0oooo04P0oooo00<000000?ooo`3oool0503oool002L0oooo00<000000?ooo`3oool0U03o
ool00`000000oooo0?ooo`190?ooo`030000003oool0oooo01<0oooo000W0?ooo`800000U03oool0
0`000000oooo0?ooo`1:0?ooo`030000003oool0oooo01<0oooo000W0?ooo`030000003oool0oooo
09<0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`0C0?ooo`00203oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0
oooo00<000000?ooo`3oool0T`3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo01<0
oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0T`3oool00`00
0000oooo0?ooo`1;0?ooo`030000003oool0oooo0180oooo00070?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0080oooo0P00002D0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool04P3oool0
00L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`040000003o
ool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`2B0?ooo`030000003oool0oooo04`0oooo
00<000000?ooo`3oool04P3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000050?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0TP3oool00`00
0000oooo0?ooo`1<0?ooo`030000003oool0oooo0180oooo00070?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0TP3oool00`000000oooo0?ooo`1=0?ooo`030000003o
ool0oooo0140oooo00080?ooo`8000002P3oool2000000<0oooo100000020?ooo`@00000103oool2
000009<0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0A0?ooo`009`3oool00`00
0000oooo0?ooo`2B0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool04@3oool002L0
oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo0100
oooo000W0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0C`3oool00`000000oooo
0?ooo`0@0?ooo`009`3oool200000980oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?oo
o`0@0?ooo`009`3oool00`000000oooo0?ooo`2A0?ooo`030000003oool0oooo04l0oooo00<00000
0?ooo`3oool0403oool002L0oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`1@0?oo
o`030000003oool0oooo00l0oooo000W0?ooo`030000003oool0oooo0900oooo00<000000?ooo`3o
ool0D@3oool00`000000oooo0?ooo`0?0?ooo`009`3oool200000940oooo00<000000?ooo`3oool0
D@3oool00`000000oooo0?ooo`0?0?ooo`009`3oool00`000000oooo0?ooo`2@0?ooo`030000003o
ool0oooo0580oooo00<000000?ooo`3oool03P3oool002L0oooo00<000000?ooo`3oool0T03oool0
0`000000oooo0?ooo`1B0?ooo`030000003oool0oooo00h0oooo000W0?ooo`030000003oool0oooo
0900oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0>0?ooo`009`3oool200000900
oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`0>0?ooo`00503oool2000000@0oooo
00<000000?ooo`3oool00P3oool4000000@0oooo00<000000?ooo`3oool0S`3oool00`000000oooo
0?ooo`1D0?ooo`030000003oool0oooo00d0oooo000C0?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo05@0oooo
00<000000?ooo`3oool03@3oool001<0oooo00@000000?ooo`3oool000002P3oool00`000000oooo
0?ooo`030?ooo`800000T03oool00`000000oooo0?ooo`1E0?ooo`030000003oool0oooo00`0oooo
000C0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0o03oool001<0oooo00@0
00000?ooo`3oool000002P3oool00`000000oooo0?ooo`3l0?ooo`004`3oool010000000oooo0?oo
o`0000080?ooo`<00000oP3oool001@0oooo0P00000;0?ooo`030000003oool0oooo0?`0oooo003o
0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o
0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.162025, -0.112772, \
0.004136, 0.0012747}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{T, 
        omthree[IntegerPart[T*10^16]*10^\((\(-16\))\), Pi/4, \(-9\)/10]}, {T, 
        0.1, 0.6, 0.1}]\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.8609554241572673`}, {0.2`, 
        0.8738684863721732`}, {0.30000000000000004`, 
        0.8955406531527417`}, {0.4`, 0.9262714049896931`}, {0.5`, 
        0.9665637532158607`}, {0.6000000000000001`, 
        1.0168747469242512`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(list = {{0.1`, 0.8609554241572673`}, {0.2`, 
          0.8738684863721732`}, {0.30000000000000004`, 
          0.8955406531527417`}, {0.4`, 0.9262714049896931`}, {0.5`, 
          0.9665637532158607`}, {0.6000000000000001`, 
          1.0168747469242512`}, {0.7000000000000001`, 
          1.0769331550062493`}, {0.8`, 1.14527857040528`}, {0.9`, 
          1.219864441023772`}, {1.`, 1.2989091642434294`}, {1.1`, 
          1.381162593664805`}, {1.2000000000000002`, 
          1.4658022590778426`}, {1.3`, 
          1.5522826442397792`}, {1.4000000000000001`, 
          1.6402294410937328`}, {1.5`, 1.7293756395347029`}, {1.6`, 
          1.8195237554963726`}, {1.7000000000000002`, 
          1.910523084312625`}, {1.8`, 
          2.0022555494515215`}, {1.9000000000000001`, 
          2.094626579087424`}, {2.`, 2.1875590185675278`}, {2.1`, 
          2.2809889388171705`}, {2.2`, 
          2.3748626678330993`}, {2.3000000000000003`, 
          2.4691346352423413`}, {2.4000000000000004`, 
          2.563765772288345`}, {2.5`, 2.6587223006736944`}, {2.6`, 
          2.7539747997154604`}, {2.7`, 
          2.849497476685298`}, {2.8000000000000003`, 
          2.945267588160741`}, {2.9000000000000004`, 
          3.0412649754366683`}, {3.`, 3.137471687355168`}}\)], "Input"],

Cell[BoxData[
    \({{0.1`, 0.8609554241572673`}, {0.2`, 
        0.8738684863721732`}, {0.30000000000000004`, 
        0.8955406531527417`}, {0.4`, 0.9262714049896931`}, {0.5`, 
        0.9665637532158607`}, {0.6000000000000001`, 
        1.0168747469242512`}, {0.7000000000000001`, 
        1.0769331550062493`}, {0.8`, 1.14527857040528`}, {0.9`, 
        1.219864441023772`}, {1.`, 1.2989091642434294`}, {1.1`, 
        1.381162593664805`}, {1.2000000000000002`, 
        1.4658022590778426`}, {1.3`, 
        1.5522826442397792`}, {1.4000000000000001`, 
        1.6402294410937328`}, {1.5`, 1.7293756395347029`}, {1.6`, 
        1.8195237554963726`}, {1.7000000000000002`, 
        1.910523084312625`}, {1.8`, 
        2.0022555494515215`}, {1.9000000000000001`, 
        2.094626579087424`}, {2.`, 2.1875590185675278`}, {2.1`, 
        2.2809889388171705`}, {2.2`, 
        2.3748626678330993`}, {2.3000000000000003`, 
        2.4691346352423413`}, {2.4000000000000004`, 
        2.563765772288345`}, {2.5`, 2.6587223006736944`}, {2.6`, 
        2.7539747997154604`}, {2.7`, 
        2.849497476685298`}, {2.8000000000000003`, 
        2.945267588160741`}, {2.9000000000000004`, 
        3.0412649754366683`}, {3.`, 3.137471687355168`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%34, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 -0.14802 0.239473 [
[.18254 .07895 -9 -9 ]
[.18254 .07895 9 0 ]
[.34127 .07895 -3 -9 ]
[.34127 .07895 3 0 ]
[.5 .07895 -9 -9 ]
[.5 .07895 9 0 ]
[.65873 .07895 -3 -9 ]
[.65873 .07895 3 0 ]
[.81746 .07895 -9 -9 ]
[.81746 .07895 9 0 ]
[.97619 .07895 -3 -9 ]
[.97619 .07895 3 0 ]
[.01131 .21119 -18 -4.5 ]
[.01131 .21119 0 4.5 ]
[.01131 .33093 -6 -4.5 ]
[.01131 .33093 0 4.5 ]
[.01131 .45066 -18 -4.5 ]
[.01131 .45066 0 4.5 ]
[.01131 .5704 -6 -4.5 ]
[.01131 .5704 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .09145 m
.18254 .0977 L
s
[(0.5)] .18254 .07895 0 1 Mshowa
.34127 .09145 m
.34127 .0977 L
s
[(1)] .34127 .07895 0 1 Mshowa
.5 .09145 m
.5 .0977 L
s
[(1.5)] .5 .07895 0 1 Mshowa
.65873 .09145 m
.65873 .0977 L
s
[(2)] .65873 .07895 0 1 Mshowa
.81746 .09145 m
.81746 .0977 L
s
[(2.5)] .81746 .07895 0 1 Mshowa
.97619 .09145 m
.97619 .0977 L
s
[(3)] .97619 .07895 0 1 Mshowa
.125 Mabswid
.05556 .09145 m
.05556 .0952 L
s
.0873 .09145 m
.0873 .0952 L
s
.11905 .09145 m
.11905 .0952 L
s
.15079 .09145 m
.15079 .0952 L
s
.21429 .09145 m
.21429 .0952 L
s
.24603 .09145 m
.24603 .0952 L
s
.27778 .09145 m
.27778 .0952 L
s
.30952 .09145 m
.30952 .0952 L
s
.37302 .09145 m
.37302 .0952 L
s
.40476 .09145 m
.40476 .0952 L
s
.43651 .09145 m
.43651 .0952 L
s
.46825 .09145 m
.46825 .0952 L
s
.53175 .09145 m
.53175 .0952 L
s
.56349 .09145 m
.56349 .0952 L
s
.59524 .09145 m
.59524 .0952 L
s
.62698 .09145 m
.62698 .0952 L
s
.69048 .09145 m
.69048 .0952 L
s
.72222 .09145 m
.72222 .0952 L
s
.75397 .09145 m
.75397 .0952 L
s
.78571 .09145 m
.78571 .0952 L
s
.84921 .09145 m
.84921 .0952 L
s
.88095 .09145 m
.88095 .0952 L
s
.9127 .09145 m
.9127 .0952 L
s
.94444 .09145 m
.94444 .0952 L
s
.25 Mabswid
0 .09145 m
1 .09145 L
s
.02381 .21119 m
.03006 .21119 L
s
[(1.5)] .01131 .21119 1 0 Mshowa
.02381 .33093 m
.03006 .33093 L
s
[(2)] .01131 .33093 1 0 Mshowa
.02381 .45066 m
.03006 .45066 L
s
[(2.5)] .01131 .45066 1 0 Mshowa
.02381 .5704 m
.03006 .5704 L
s
[(3)] .01131 .5704 1 0 Mshowa
.125 Mabswid
.02381 .1154 m
.02756 .1154 L
s
.02381 .13935 m
.02756 .13935 L
s
.02381 .16329 m
.02756 .16329 L
s
.02381 .18724 m
.02756 .18724 L
s
.02381 .23514 m
.02756 .23514 L
s
.02381 .25908 m
.02756 .25908 L
s
.02381 .28303 m
.02756 .28303 L
s
.02381 .30698 m
.02756 .30698 L
s
.02381 .35487 m
.02756 .35487 L
s
.02381 .37882 m
.02756 .37882 L
s
.02381 .40277 m
.02756 .40277 L
s
.02381 .42671 m
.02756 .42671 L
s
.02381 .47461 m
.02756 .47461 L
s
.02381 .49856 m
.02756 .49856 L
s
.02381 .5225 m
.02756 .5225 L
s
.02381 .54645 m
.02756 .54645 L
s
.02381 .06751 m
.02756 .06751 L
s
.02381 .04356 m
.02756 .04356 L
s
.02381 .01961 m
.02756 .01961 L
s
.02381 .59435 m
.02756 .59435 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.0873 .02115 L
.11905 .0309 L
.15079 .043 L
.18254 .05651 L
.21429 .07071 L
.24603 .10988 L
.27778 .12624 L
.30952 .1441 L
.34127 .16303 L
.37302 .18273 L
.40476 .203 L
.43651 .22371 L
.46825 .24477 L
.5 .26612 L
.53175 .28771 L
.56349 .3095 L
.59524 .33147 L
.62698 .35359 L
.65873 .37584 L
.69048 .39821 L
.72222 .42069 L
.75397 .44327 L
.78571 .46593 L
.81746 .48867 L
.84921 .51148 L
.88095 .53436 L
.9127 .55729 L
.94444 .58028 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`005@3oool00`000000oooo0?oo
o`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?oo
o`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000
003oool0oooo00H0oooo0P00003o0?ooo`40oooo000E0?ooo`8000002@3oool400000?`0oooo000E
0?ooo`030000003oool0oooo00`0oooo1@00003g0?ooo`005@3oool00`000000oooo0?ooo`0A0?oo
o`@00000l`3oool001D0oooo00<000000?ooo`3oool05@3oool300000?00oooo000E0?ooo`030000
003oool0oooo01P0oooo0P00003^0?ooo`005@3oool00`000000oooo0?ooo`0J0?ooo`800000k03o
ool001D0oooo00<000000?ooo`3oool0703oool200000>X0oooo000E0?ooo`8000007`3oool30000
0>L0oooo000E0?ooo`030000003oool0oooo0240oooo0`00003T0?ooo`005@3oool00`000000oooo
0?ooo`0Q0?ooo`80000000<0oooo0000000000000P0000040?ooo`<000008P3oool400000240oooo
100000030?ooo`030000003oool0oooo0080oooo0`00000S0?ooo`@000008@3oool4000000<0oooo
00<000000?ooo`3oool00P3oool3000002<0oooo0P0000070?ooo`005@3oool00`000000oooo0?oo
o`0P0?ooo`040000003oool0oooo000000<0oooo0`0000050?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01l0oooo
00@000000?ooo`3oool000001P3oool001D0oooo00<000000?ooo`3oool0803oool010000000oooo
0?ooo`0000060?ooo`8000000`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00@0
oooo000E0?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool00000203oool3000002@0
oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`070?ooo`<000009@3oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo0`00000S0?ooo`8000001`3oool001D0oooo
0P00000Q0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`00000000008`3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool09@3oool00`00
0000oooo0?ooo`040?ooo`005@3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool0oooo0P00000O0?ooo`<000008P3oool3000000T0oooo00<00000
0?ooo`3oool08`3oool010000000oooo0?ooo`00000Q0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool08P3oool010000000oooo0?ooo`0000060?ooo`005@3oool00`000000oooo
0?ooo`0Q0?ooo`8000002@3oool400000080oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo00L0oooo1000000S0?ooo`8000008`3oool2000000T0oooo
1000000R0?ooo`8000001`3oool001D0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?oo
o`3C0?ooo`005@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0=80oooo000E0?oo
o`030000003oool0oooo03@0oooo00<000000?ooo`3oool0d@3oool001D0oooo00<000000?ooo`3o
ool0=@3oool00`000000oooo0?ooo`3@0?ooo`003P3ooooo000001<00000000E0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00D0oooo000E0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:@3o
ool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool01@3o
ool001D0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`3=0?ooo`005@3oool00`00
0000oooo0?ooo`0h0?ooo`030000003oool0oooo0<d0oooo000E0?ooo`030000003oool0oooo03T0
oooo0P00003=0?ooo`005@3oool2000003`0oooo0P00003;0?ooo`005@3oool00`000000oooo0?oo
o`0m0?ooo`800000b@3oool001D0oooo00<000000?ooo`3oool0?`3oool200000<L0oooo000E0?oo
o`030000003oool0oooo0440oooo0P0000350?ooo`005@3oool00`000000oooo0?ooo`130?ooo`80
0000``3oool001D0oooo00<000000?ooo`3oool0A@3oool200000<40oooo000E0?ooo`030000003o
ool0oooo04L0oooo0P00002o0?ooo`005@3oool2000004X0oooo0P00002m0?ooo`005@3oool00`00
0000oooo0?ooo`1;0?ooo`030000003oool0oooo0;X0oooo000E0?ooo`030000003oool0oooo04`0
oooo0P00002j0?ooo`005@3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo0;L0oooo
000E0?ooo`030000003oool0oooo04l0oooo0P00002g0?ooo`005@3oool00`000000oooo0?ooo`1A
0?ooo`800000]@3oool001D0oooo0P00001D0?ooo`030000003oool0oooo0;80oooo000E0?ooo`03
0000003oool0oooo05@0oooo0P00002b0?ooo`005@3oool00`000000oooo0?ooo`1F0?ooo`030000
003oool0oooo0:l0oooo000E0?ooo`030000003oool0oooo05L0oooo0P00002_0?ooo`005@3oool0
0`000000oooo0?ooo`1I0?ooo`030000003oool0oooo0:`0oooo000E0?ooo`030000003oool0oooo
05X0oooo0P00002/0?ooo`005@3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo0:T0
oooo000E0?ooo`800000GP3oool200000:T0oooo000E0?ooo`030000003oool0oooo05l0oooo0P00
002W0?ooo`005@3oool00`000000oooo0?ooo`1Q0?ooo`800000Y@3oool001D0oooo00<000000?oo
o`3oool0H`3oool200000:<0oooo00000`3oool0000000000002000000<0oooo00<000000?ooo`3o
ool00P3oool3000000D0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`2P0?ooo`00
0`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
IP3oool00`000000oooo0?ooo`2O0?ooo`000`3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo0080oooo0`00001W0?ooo`800000W`3oool000<0oooo00<000000?ooo`3oool01`3oool3
000000D0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`2L0?ooo`000`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0JP3oool00`00
0000oooo0?ooo`2K0?ooo`0000@0oooo00000000000000002@3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo06/0oooo0P00002K0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`@0
0000103oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo09P0oooo000E0?ooo`030000
003oool0oooo06h0oooo0P00002H0?ooo`005@3oool00`000000oooo0?ooo`1`0?ooo`030000003o
ool0oooo09D0oooo000E0?ooo`800000LP3oool2000009D0oooo000E0?ooo`030000003oool0oooo
07<0oooo00<000000?ooo`3oool0TP3oool001D0oooo00<000000?ooo`3oool0M03oool200000980
oooo000E0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool0S`3oool001D0oooo00<0
00000?ooo`3oool0M`3oool2000008l0oooo000E0?ooo`030000003oool0oooo07T0oooo00<00000
0?ooo`3oool0S03oool001D0oooo0P00001k0?ooo`800000S03oool001D0oooo00<000000?ooo`3o
ool0O03oool2000008X0oooo000E0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3oool0
Q`3oool001D0oooo00<000000?ooo`3oool0O`3oool2000008L0oooo000E0?ooo`030000003oool0
oooo0840oooo00<000000?ooo`3oool0Q03oool001D0oooo00<000000?ooo`3oool0PP3oool20000
08@0oooo000E0?ooo`030000003oool0oooo08@0oooo00<000000?ooo`3oool0P@3oool001D0oooo
0P0000260?ooo`800000P@3oool001D0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo0?oo
o`1n0?ooo`005@3oool00`000000oooo0?ooo`280?ooo`800000OP3oool001D0oooo00<000000?oo
o`3oool0RP3oool00`000000oooo0?ooo`1k0?ooo`005@3oool00`000000oooo0?ooo`2;0?ooo`80
0000N`3oool001D0oooo00<000000?ooo`3oool0S@3oool00`000000oooo0?ooo`1h0?ooo`005@3o
ool2000008l0oooo0P00001h0?ooo`005@3oool00`000000oooo0?ooo`2@0?ooo`030000003oool0
oooo07D0oooo000E0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0M03oool001D0
oooo00<000000?ooo`3oool0TP3oool2000007@0oooo000E0?ooo`030000003oool0oooo09@0oooo
00<000000?ooo`3oool0L@3oool000d0oooo100000040?ooo`030000003oool0oooo09D0oooo00<0
00000?ooo`3oool0L03oool000d0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2F
0?ooo`800000L03oool000h0oooo00<000000?ooo`3oool0103oool3000009P0oooo00<000000?oo
o`3oool0K@3oool000l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2I0?ooo`80
0000K@3oool00100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2K0?ooo`030000
003oool0oooo06X0oooo000=0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0
W03oool2000006X0oooo000>0?ooo`8000001@3oool00`000000oooo0?ooo`2N0?ooo`030000003o
ool0oooo06L0oooo000E0?ooo`030000003oool0oooo09l0oooo0P00001W0?ooo`005@3oool00`00
0000oooo0?ooo`2Q0?ooo`030000003oool0oooo06@0oooo000E0?ooo`800000X`3oool2000006@0
oooo000E0?ooo`030000003oool0oooo0:@0oooo00<000000?ooo`3oool0H@3oool001D0oooo00<0
00000?ooo`3oool0Y@3oool200000640oooo000E0?ooo`030000003oool0oooo0:L0oooo00<00000
0?ooo`3oool0GP3oool001D0oooo00<000000?ooo`3oool0Z03oool2000005h0oooo000E0?ooo`03
0000003oool0oooo0:X0oooo00<000000?ooo`3oool0F`3oool001D0oooo0P00002/0?ooo`030000
003oool0oooo05X0oooo000E0?ooo`030000003oool0oooo0:`0oooo0P00001J0?ooo`005@3oool0
0`000000oooo0?ooo`2^0?ooo`030000003oool0oooo05L0oooo000E0?ooo`030000003oool0oooo
0:l0oooo00<000000?ooo`3oool0EP3oool001D0oooo00<000000?ooo`3oool0/03oool2000005H0
oooo000E0?ooo`030000003oool0oooo0;80oooo00<000000?ooo`3oool0D`3oool001D0oooo00<0
00000?ooo`3oool0/`3oool2000005<0oooo000E0?ooo`800000]P3oool00`000000oooo0?ooo`1@
0?ooo`005@3oool00`000000oooo0?ooo`2f0?ooo`800000D03oool001D0oooo00<000000?ooo`3o
ool0^03oool00`000000oooo0?ooo`1=0?ooo`005@3oool00`000000oooo0?ooo`2i0?ooo`800000
C@3oool001D0oooo00<000000?ooo`3oool0^`3oool00`000000oooo0?ooo`1:0?ooo`005@3oool0
0`000000oooo0?ooo`2l0?ooo`030000003oool0oooo04T0oooo000E0?ooo`800000_P3oool20000
04T0oooo000E0?ooo`030000003oool0oooo0;l0oooo00<000000?ooo`3oool0AP3oool001D0oooo
00<000000?ooo`3oool0`03oool00`000000oooo0?ooo`150?ooo`005@3oool00`000000oooo0?oo
o`310?ooo`800000A@3oool001D0oooo00<000000?ooo`3oool0``3oool00`000000oooo0?ooo`12
0?ooo`0000<0oooo0000000000000P0000030?ooo`030000003oool0oooo0080oooo0`0000050?oo
o`030000003oool0oooo0<@0oooo00<000000?ooo`3oool0@@3oool000030?ooo`000000oooo00d0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`350?ooo`030000003oool0oooo0400
oooo00020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool00P3oool300000<H0oooo
0P0000100?ooo`000`3oool00`000000oooo0?ooo`070?ooo`<000001@3oool00`000000oooo0?oo
o`380?ooo`030000003oool0oooo03d0oooo00040?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`390?ooo`030000003oool0oooo03`0oooo00001@3o
ool000000?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3:
0?ooo`800000?03oool00080oooo0P0000090?ooo`@00000103oool00`000000oooo0?ooo`3<0?oo
o`030000003oool0oooo03T0oooo000E0?ooo`030000003oool0oooo0<d0oooo0P00000i0?ooo`00
5@3oool200000=00oooo00<000000?ooo`3oool0=P3oool001D0oooo00<000000?ooo`3oool0d03o
ool2000003H0oooo000E0?ooo`030000003oool0oooo0=80oooo00<000000?ooo`3oool0<`3oool0
01D0oooo00<000000?ooo`3oool0d`3oool2000003<0oooo000E0?ooo`030000003oool0oooo0=D0
oooo00<000000?ooo`3oool0<03oool001D0oooo00<000000?ooo`3oool0eP3oool00`000000oooo
0?ooo`0_0?ooo`005@3oool00`000000oooo0?ooo`3G0?ooo`800000;`3oool001D0oooo0P00003J
0?ooo`030000003oool0oooo02`0oooo000E0?ooo`030000003oool0oooo0=X0oooo00<000000?oo
o`3oool0:`3oool001D0oooo00<000000?ooo`3oool0f`3oool2000002/0oooo000E0?ooo`030000
003oool0oooo0=d0oooo00<000000?ooo`3oool0:03oool001D0oooo00<000000?ooo`3oool0gP3o
ool2000002P0oooo000E0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool09@3oool0
01D0oooo0P00003R0?ooo`8000009@3oool001D0oooo00<000000?ooo`3oool0h`3oool00`000000
oooo0?ooo`0R0?ooo`005@3oool00`000000oooo0?ooo`3T0?ooo`8000008P3oool001D0oooo00<0
00000?ooo`3oool0iP3oool00`000000oooo0?ooo`0O0?ooo`005@3oool00`000000oooo0?ooo`3W
0?ooo`030000003oool0oooo01h0oooo000E0?ooo`030000003oool0oooo0>P0oooo0P00000N0?oo
o`005@3oool00`000000oooo0?ooo`3Z0?ooo`030000003oool0oooo01/0oooo000E0?ooo`800000
k03oool00`000000oooo0?ooo`0J0?ooo`005@3oool00`000000oooo0?ooo`3/0?ooo`8000006P3o
ool001D0oooo00<000000?ooo`3oool0kP3oool00`000000oooo0?ooo`0G0?ooo`005@3oool00`00
0000oooo0?ooo`3_0?ooo`030000003oool0oooo01H0oooo000>0?ooo`8000001@3oool00`000000
oooo0?ooo`3`0?ooo`030000003oool0oooo01D0oooo000=0?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool0l@3oool2000001D0oooo000@0?ooo`030000003oool0oooo0080oooo
0`00003c0?ooo`030000003oool0oooo0180oooo000>0?ooo`8000001@3oool00`000000oooo0?oo
o`3d0?ooo`030000003oool0oooo0140oooo000@0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0m@3oool200000140oooo000=0?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0m`3oool00`000000oooo0?ooo`0>0?ooo`003P3oool2000000D0oooo00<000000?oo
o`3oool0n03oool2000000h0oooo000E0?ooo`030000003oool0oooo0?X0oooo00<000000?ooo`3o
ool02`3oool001D0oooo00<000000?ooo`3oool0n`3oool2000000/0oooo000E0?ooo`800000oP3o
ool00`000000oooo0?ooo`080?ooo`005@3oool00`000000oooo0?ooo`3n0?ooo`800000203oool0
01D0oooo00<000000?ooo`3oool0o`3oool10?ooo`030000003oool0oooo00D0oooo000E0?ooo`03
0000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3o
ool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool0
0?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, 0.550673, \
0.0115777, 0.0153481}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %%]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 -0.14802 0.239473 [
[.18254 .07895 -9 -9 ]
[.18254 .07895 9 0 ]
[.34127 .07895 -3 -9 ]
[.34127 .07895 3 0 ]
[.5 .07895 -9 -9 ]
[.5 .07895 9 0 ]
[.65873 .07895 -3 -9 ]
[.65873 .07895 3 0 ]
[.81746 .07895 -9 -9 ]
[.81746 .07895 9 0 ]
[.97619 .07895 -3 -9 ]
[.97619 .07895 3 0 ]
[.01131 .21119 -18 -4.5 ]
[.01131 .21119 0 4.5 ]
[.01131 .33093 -6 -4.5 ]
[.01131 .33093 0 4.5 ]
[.01131 .45066 -18 -4.5 ]
[.01131 .45066 0 4.5 ]
[.01131 .5704 -6 -4.5 ]
[.01131 .5704 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .09145 m
.18254 .0977 L
s
[(0.5)] .18254 .07895 0 1 Mshowa
.34127 .09145 m
.34127 .0977 L
s
[(1)] .34127 .07895 0 1 Mshowa
.5 .09145 m
.5 .0977 L
s
[(1.5)] .5 .07895 0 1 Mshowa
.65873 .09145 m
.65873 .0977 L
s
[(2)] .65873 .07895 0 1 Mshowa
.81746 .09145 m
.81746 .0977 L
s
[(2.5)] .81746 .07895 0 1 Mshowa
.97619 .09145 m
.97619 .0977 L
s
[(3)] .97619 .07895 0 1 Mshowa
.125 Mabswid
.05556 .09145 m
.05556 .0952 L
s
.0873 .09145 m
.0873 .0952 L
s
.11905 .09145 m
.11905 .0952 L
s
.15079 .09145 m
.15079 .0952 L
s
.21429 .09145 m
.21429 .0952 L
s
.24603 .09145 m
.24603 .0952 L
s
.27778 .09145 m
.27778 .0952 L
s
.30952 .09145 m
.30952 .0952 L
s
.37302 .09145 m
.37302 .0952 L
s
.40476 .09145 m
.40476 .0952 L
s
.43651 .09145 m
.43651 .0952 L
s
.46825 .09145 m
.46825 .0952 L
s
.53175 .09145 m
.53175 .0952 L
s
.56349 .09145 m
.56349 .0952 L
s
.59524 .09145 m
.59524 .0952 L
s
.62698 .09145 m
.62698 .0952 L
s
.69048 .09145 m
.69048 .0952 L
s
.72222 .09145 m
.72222 .0952 L
s
.75397 .09145 m
.75397 .0952 L
s
.78571 .09145 m
.78571 .0952 L
s
.84921 .09145 m
.84921 .0952 L
s
.88095 .09145 m
.88095 .0952 L
s
.9127 .09145 m
.9127 .0952 L
s
.94444 .09145 m
.94444 .0952 L
s
.25 Mabswid
0 .09145 m
1 .09145 L
s
.02381 .21119 m
.03006 .21119 L
s
[(1.5)] .01131 .21119 1 0 Mshowa
.02381 .33093 m
.03006 .33093 L
s
[(2)] .01131 .33093 1 0 Mshowa
.02381 .45066 m
.03006 .45066 L
s
[(2.5)] .01131 .45066 1 0 Mshowa
.02381 .5704 m
.03006 .5704 L
s
[(3)] .01131 .5704 1 0 Mshowa
.125 Mabswid
.02381 .1154 m
.02756 .1154 L
s
.02381 .13935 m
.02756 .13935 L
s
.02381 .16329 m
.02756 .16329 L
s
.02381 .18724 m
.02756 .18724 L
s
.02381 .23514 m
.02756 .23514 L
s
.02381 .25908 m
.02756 .25908 L
s
.02381 .28303 m
.02756 .28303 L
s
.02381 .30698 m
.02756 .30698 L
s
.02381 .35487 m
.02756 .35487 L
s
.02381 .37882 m
.02756 .37882 L
s
.02381 .40277 m
.02756 .40277 L
s
.02381 .42671 m
.02756 .42671 L
s
.02381 .47461 m
.02756 .47461 L
s
.02381 .49856 m
.02756 .49856 L
s
.02381 .5225 m
.02756 .5225 L
s
.02381 .54645 m
.02756 .54645 L
s
.02381 .06751 m
.02756 .06751 L
s
.02381 .04356 m
.02756 .04356 L
s
.02381 .01961 m
.02756 .01961 L
s
.02381 .59435 m
.02756 .59435 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.0873 .02115 L
.11905 .0309 L
.15079 .043 L
.18254 .05651 L
.21429 .07071 L
.24603 .0852 L
.27778 .09998 L
.30952 .11525 L
.34127 .13123 L
.37302 .14803 L
.40476 .1657 L
.43651 .18422 L
.46825 .20358 L
.5 .2237 L
.53175 .24453 L
.56349 .26599 L
.59524 .288 L
.62698 .31047 L
.65873 .33332 L
.69048 .35648 L
.72222 .37987 L
.75397 .40344 L
.78571 .42713 L
.81746 .45092 L
.84921 .47476 L
.88095 .49864 L
.9127 .52255 L
.94444 .54648 L
.97619 .57041 L
s
.05556 .01472 m
.0873 .02115 L
.11905 .0309 L
.15079 .043 L
.18254 .05651 L
.21429 .07071 L
.24603 .10988 L
.27778 .12624 L
.30952 .1441 L
.34127 .16303 L
.37302 .18273 L
.40476 .203 L
.43651 .22371 L
.46825 .24477 L
.5 .26612 L
.53175 .28771 L
.56349 .3095 L
.59524 .33147 L
.62698 .35359 L
.65873 .37584 L
.69048 .39821 L
.72222 .42069 L
.75397 .44327 L
.78571 .46593 L
.81746 .48867 L
.84921 .51148 L
.88095 .53436 L
.9127 .55729 L
.94444 .58028 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`005@3oool00`000000oooo0?oo
o`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?oo
o`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000
003oool0oooo00H0oooo0P00003o0?ooo`40oooo000E0?ooo`8000002@3oool400000?`0oooo000E
0?ooo`030000003oool0oooo00`0oooo1@00003g0?ooo`005@3oool00`000000oooo0?ooo`0A0?oo
o`@00000l`3oool001D0oooo00<000000?ooo`3oool05@3oool300000?00oooo000E0?ooo`030000
003oool0oooo01P0oooo0P00003^0?ooo`005@3oool00`000000oooo0?ooo`0J0?ooo`800000k03o
ool001D0oooo00<000000?ooo`3oool0703oool200000>X0oooo000E0?ooo`8000007`3oool30000
0>L0oooo000E0?ooo`030000003oool0oooo0240oooo0`00003T0?ooo`005@3oool00`000000oooo
0?ooo`0Q0?ooo`80000000<0oooo0000000000000P0000040?ooo`<000008P3oool400000240oooo
100000030?ooo`030000003oool0oooo0080oooo0`00000S0?ooo`@000008@3oool4000000<0oooo
00<000000?ooo`3oool00P3oool3000002<0oooo0P0000070?ooo`005@3oool00`000000oooo0?oo
o`0P0?ooo`040000003oool0oooo000000<0oooo0`0000050?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01l0oooo
00@000000?ooo`3oool000001P3oool001D0oooo00<000000?ooo`3oool0803oool010000000oooo
0?ooo`0000060?ooo`8000000`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00@0
oooo000E0?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool00000203oool3000002@0
oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`070?ooo`<000009@3oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo0`00000S0?ooo`8000001`3oool001D0oooo
0P00000Q0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`00000000008`3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool09@3oool00`00
0000oooo0?ooo`040?ooo`005@3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool0oooo0P00000O0?ooo`<000008P3oool3000000T0oooo00<00000
0?ooo`3oool08`3oool010000000oooo0?ooo`00000Q0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool08P3oool010000000oooo0?ooo`0000060?ooo`005@3oool00`000000oooo
0?ooo`0Q0?ooo`8000002@3oool400000080oooo0P00000O0?ooo`030000003oool0oooo0280oooo
00<000000?ooo`3oool01`3oool4000002<0oooo0P00000S0?ooo`8000002@3oool400000280oooo
0P0000070?ooo`005@3oool00`000000oooo0?ooo`0b0?ooo`040000003oool0000000000=80oooo
000E0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool00P00003@0?ooo`005@3oool0
0`000000oooo0?ooo`0d0?ooo`040000003oool0oooo0?ooo`<00000c@3oool001D0oooo00<00000
0?ooo`3oool0=@3oool00`000000oooo0?ooo`030?ooo`800000b`3oool000h0ooooo`00000C0000
00005@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000D0oooo0P0000000`3o
ool000000?ooo`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo000E0?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`070?ooo`<000003`3oool00`000000
oooo0?ooo`0X0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`0X0?ooo`030000003oool0oooo00D0oooo000E0?ooo`030000003oool0oooo03P0oooo
00<000000?ooo`3oool02@3oool200000<80oooo000E0?ooo`030000003oool0oooo03P0oooo00<0
00000?ooo`3oool02`3oool200000<00oooo000E0?ooo`030000003oool0oooo03T0oooo0P00000=
0?ooo`800000_P3oool001D0oooo0P00000l0?ooo`8000003@3oool200000;`0oooo000E0?ooo`03
0000003oool0oooo03d0oooo0P00000=0?ooo`800000^P3oool001D0oooo00<000000?ooo`3oool0
?`3oool2000000d0oooo00<000000?ooo`3oool0]`3oool001D0oooo00<000000?ooo`3oool0@@3o
ool2000000`0oooo0P00002g0?ooo`005@3oool00`000000oooo0?ooo`130?ooo`800000303oool2
00000;D0oooo000E0?ooo`030000003oool0oooo04D0oooo0P00000<0?ooo`800000/`3oool001D0
oooo00<000000?ooo`3oool0A`3oool2000000`0oooo0P00002a0?ooo`005@3oool2000004X0oooo
0P00000<0?ooo`800000[`3oool001D0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?oo
o`0;0?ooo`800000[@3oool001D0oooo00<000000?ooo`3oool0C03oool2000000d0oooo0P00002[
0?ooo`005@3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo00`0oooo0P00002Y0?oo
o`005@3oool00`000000oooo0?ooo`1?0?ooo`8000003P3oool200000:L0oooo000E0?ooo`030000
003oool0oooo0540oooo0P00000>0?ooo`800000Y@3oool001D0oooo0P00001D0?ooo`030000003o
ool0oooo00d0oooo0P00002S0?ooo`005@3oool00`000000oooo0?ooo`1D0?ooo`8000003`3oool0
0`000000oooo0?ooo`2P0?ooo`005@3oool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo
00d0oooo0P00002P0?ooo`005@3oool00`000000oooo0?ooo`1G0?ooo`8000003`3oool00`000000
oooo0?ooo`2M0?ooo`005@3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo00d0oooo
0P00002M0?ooo`005@3oool00`000000oooo0?ooo`1J0?ooo`8000003`3oool2000009/0oooo000E
0?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`2H
0?ooo`005@3oool2000005h0oooo0P00000?0?ooo`800000V03oool001D0oooo00<000000?ooo`3o
ool0G`3oool2000000l0oooo0P00002F0?ooo`005@3oool00`000000oooo0?ooo`1Q0?ooo`800000
3`3oool2000009@0oooo000E0?ooo`030000003oool0oooo06<0oooo0P00000?0?ooo`800000TP3o
ool000030?ooo`0000000000008000000`3oool00`000000oooo0?ooo`020?ooo`<000001@3oool0
0`000000oooo0?ooo`1U0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0S`3oool0
00<0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
06H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`2>0?ooo`000`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo0`00001W0?ooo`8000003`3oool2000008h0
oooo00030?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`030000003oool0oooo06T0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`2;0?ooo`000`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo08X0oooo0000103oool000000000000000090?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0J`3oool2000000l0oooo0P00002:0?ooo`000`3oool0
0`000000oooo0?ooo`070?ooo`@00000103oool00`000000oooo0?ooo`1]0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0Q`3oool001D0oooo00<000000?ooo`3oool0KP3oool20000
00l0oooo0P0000270?ooo`005@3oool00`000000oooo0?ooo`1`0?ooo`030000003oool0oooo00h0
oooo0P0000250?ooo`005@3oool200000780oooo0P00000@0?ooo`800000P`3oool001D0oooo00<0
00000?ooo`3oool0L`3oool00`000000oooo0?ooo`0?0?ooo`800000P@3oool001D0oooo00<00000
0?ooo`3oool0M03oool200000140oooo00<000000?ooo`3oool0OP3oool001D0oooo00<000000?oo
o`3oool0MP3oool00`000000oooo0?ooo`0?0?ooo`800000OP3oool001D0oooo00<000000?ooo`3o
ool0M`3oool200000140oooo00<000000?ooo`3oool0N`3oool001D0oooo00<000000?ooo`3oool0
N@3oool00`000000oooo0?ooo`0?0?ooo`800000N`3oool001D0oooo0P00001k0?ooo`8000004@3o
ool00`000000oooo0?ooo`1h0?ooo`005@3oool00`000000oooo0?ooo`1l0?ooo`800000403oool2
000007P0oooo000E0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`1e0?ooo`005@3oool00`000000oooo0?ooo`1o0?ooo`800000403oool00`000000
oooo0?ooo`1d0?ooo`005@3oool00`000000oooo0?ooo`210?ooo`030000003oool0oooo00h0oooo
0P00001d0?ooo`005@3oool00`000000oooo0?ooo`220?ooo`800000403oool00`000000oooo0?oo
o`1a0?ooo`005@3oool00`000000oooo0?ooo`240?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0L03oool001D0oooo0P0000260?ooo`8000003`3oool200000700oooo000E0?ooo`03
0000003oool0oooo08L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1]0?ooo`00
5@3oool00`000000oooo0?ooo`280?ooo`8000003`3oool2000006d0oooo000E0?ooo`030000003o
ool0oooo08X0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1Z0?ooo`005@3oool0
0`000000oooo0?ooo`2;0?ooo`8000003`3oool2000006X0oooo000E0?ooo`030000003oool0oooo
08d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1W0?ooo`005@3oool2000008l0
oooo0P00000?0?ooo`800000I`3oool001D0oooo00<000000?ooo`3oool0T03oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo06@0oooo000E0?ooo`030000003oool0oooo0940oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`1S0?ooo`005@3oool00`000000oooo0?ooo`2B
0?ooo`8000003`3oool2000006<0oooo000E0?ooo`030000003oool0oooo09@0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`1P0?ooo`003@3oool4000000@0oooo00<000000?ooo`3o
ool0U@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05l0oooo000=0?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool0UP3oool2000000l0oooo0P00001O0?ooo`003P3o
ool00`000000oooo0?ooo`040?ooo`<00000V03oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo05`0oooo000?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0V@3oool2
000000l0oooo00<000000?ooo`3oool0F`3oool00100oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`2K0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0FP3oool000d0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`2L0?ooo`8000003P3oool20000
05X0oooo000>0?ooo`8000001@3oool00`000000oooo0?ooo`2N0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool0E`3oool001D0oooo00<000000?ooo`3oool0W`3oool2000000h0oooo
00<000000?ooo`3oool0EP3oool001D0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?oo
o`0<0?ooo`800000EP3oool001D0oooo0P00002S0?ooo`8000003P3oool00`000000oooo0?ooo`1C
0?ooo`005@3oool00`000000oooo0?ooo`2T0?ooo`030000003oool0oooo00`0oooo0P00001C0?oo
o`005@3oool00`000000oooo0?ooo`2U0?ooo`8000003P3oool00`000000oooo0?ooo`1@0?ooo`00
5@3oool00`000000oooo0?ooo`2W0?ooo`030000003oool0oooo00`0oooo0P00001@0?ooo`005@3o
ool00`000000oooo0?ooo`2X0?ooo`8000003P3oool00`000000oooo0?ooo`1=0?ooo`005@3oool0
0`000000oooo0?ooo`2Z0?ooo`030000003oool0oooo00`0oooo0P00001=0?ooo`005@3oool20000
0:`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`1:0?ooo`005@3oool00`000000
oooo0?ooo`2/0?ooo`8000003P3oool00`000000oooo0?ooo`190?ooo`005@3oool00`000000oooo
0?ooo`2^0?ooo`030000003oool0oooo00`0oooo0P0000190?ooo`005@3oool00`000000oooo0?oo
o`2_0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0AP3oool001D0oooo00<00000
0?ooo`3oool0/03oool2000000h0oooo00<000000?ooo`3oool0A@3oool001D0oooo00<000000?oo
o`3oool0/P3oool00`000000oooo0?ooo`0<0?ooo`800000A@3oool001D0oooo00<000000?ooo`3o
ool0/`3oool2000000h0oooo00<000000?ooo`3oool0@P3oool001D0oooo0P00002f0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0@@3oool001D0oooo00<000000?ooo`3oool0]P3o
ool2000000d0oooo00<000000?ooo`3oool0@03oool001D0oooo00<000000?ooo`3oool0^03oool0
0`000000oooo0?ooo`0;0?ooo`800000@03oool001D0oooo00<000000?ooo`3oool0^@3oool20000
00d0oooo00<000000?ooo`3oool0?@3oool001D0oooo00<000000?ooo`3oool0^`3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo03`0oooo000E0?ooo`030000003oool0oooo0;`0oooo
00<000000?ooo`3oool02`3oool2000003`0oooo000E0?ooo`800000_P3oool2000000d0oooo00<0
00000?ooo`3oool0>@3oool001D0oooo00<000000?ooo`3oool0_`3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo03P0oooo000E0?ooo`030000003oool0oooo0<00oooo00<000000?oo
o`3oool02`3oool2000003P0oooo000E0?ooo`030000003oool0oooo0<40oooo0P00000=0?ooo`03
0000003oool0oooo03D0oooo000E0?ooo`030000003oool0oooo0<<0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0d0?ooo`0000<0oooo0000000000000P0000030?ooo`030000003o
ool0oooo0080oooo0`0000050?ooo`030000003oool0oooo0<@0oooo00<000000?ooo`3oool02`3o
ool2000003@0oooo00000`3oool000000?ooo`0=0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0a@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0340oooo00020?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool00P3oool300000<H0oooo0P00000=0?oo
o`030000003oool0oooo0300oooo00030?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`03
0000003oool0oooo0<P0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0_0?ooo`00
103oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
b@3oool00`000000oooo0?ooo`0;0?ooo`800000;`3oool000050?ooo`000000oooo0?ooo`000000
203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0<X0oooo0P00000=0?ooo`030000
003oool0oooo02`0oooo00020?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0c03o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo02/0oooo000E0?ooo`030000003oool0
oooo0<d0oooo0P00000<0?ooo`800000:`3oool001D0oooo0P00003@0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0:03oool001D0oooo00<000000?ooo`3oool0d03oool2000000`0
oooo00<000000?ooo`3oool09`3oool001D0oooo00<000000?ooo`3oool0dP3oool00`000000oooo
0?ooo`0:0?ooo`8000009`3oool001D0oooo00<000000?ooo`3oool0d`3oool2000000`0oooo00<0
00000?ooo`3oool0903oool001D0oooo00<000000?ooo`3oool0e@3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo02<0oooo000E0?ooo`030000003oool0oooo0=H0oooo00<000000?oo
o`3oool02P3oool2000002<0oooo000E0?ooo`030000003oool0oooo0=L0oooo0P00000<0?ooo`03
0000003oool0oooo0200oooo000E0?ooo`800000fP3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo01l0oooo000E0?ooo`030000003oool0oooo0=X0oooo00<000000?ooo`3oool02P3o
ool2000001l0oooo000E0?ooo`030000003oool0oooo0=/0oooo0P00000<0?ooo`030000003oool0
oooo01`0oooo000E0?ooo`030000003oool0oooo0=d0oooo00<000000?ooo`3oool02P3oool20000
01`0oooo000E0?ooo`030000003oool0oooo0=h0oooo0P00000<0?ooo`030000003oool0oooo01T0
oooo000E0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool02P3oool2000001T0oooo
000E0?ooo`800000hP3oool2000000`0oooo00<000000?ooo`3oool05P3oool001D0oooo00<00000
0?ooo`3oool0h`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01D0oooo000E0?oo
o`030000003oool0oooo0>@0oooo0P00000;0?ooo`030000003oool0oooo01@0oooo000E0?ooo`03
0000003oool0oooo0>H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0C0?ooo`00
5@3oool00`000000oooo0?ooo`3W0?ooo`030000003oool0oooo00T0oooo0P00000C0?ooo`005@3o
ool00`000000oooo0?ooo`3X0?ooo`8000002`3oool00`000000oooo0?ooo`0@0?ooo`005@3oool0
0`000000oooo0?ooo`3Z0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03`3oool0
01D0oooo0P00003/0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03P3oool001D0
oooo00<000000?ooo`3oool0k03oool2000000X0oooo0P00000>0?ooo`005@3oool00`000000oooo
0?ooo`3^0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02`3oool001D0oooo00<0
00000?ooo`3oool0k`3oool00`000000oooo0?ooo`090?ooo`8000002`3oool000h0oooo0P000005
0?ooo`030000003oool0oooo0?00oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`08
0?ooo`003@3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0?40oooo0P00000;
0?ooo`800000203oool00100oooo00<000000?ooo`3oool00P3oool300000?<0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`050?ooo`003P3oool2000000D0oooo00<000000?ooo`3o
ool0m03oool00`000000oooo0?ooo`0A0?ooo`00403oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0?D0oooo0P00000A0?ooo`003@3oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo0?L0oooo00<000000?ooo`3oool03P3oool000h0oooo0P0000050?ooo`030000003o
ool0oooo0?P0oooo0P00000>0?ooo`005@3oool00`000000oooo0?ooo`3j0?ooo`030000003oool0
oooo00/0oooo000E0?ooo`030000003oool0oooo0?/0oooo0P00000;0?ooo`005@3oool200000?h0
oooo00<000000?ooo`3oool0203oool001D0oooo00<000000?ooo`3oool0oP3oool2000000P0oooo
000E0?ooo`030000003oool0oooo0?l0oooo0@3oool00`000000oooo0?ooo`050?ooo`005@3oool0
0`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0
oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo
003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, 0.550673, \
0.0115777, 0.0153481}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(omtwo[1/10, Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(0.5092554845405696`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      testfunc[IntegerPart[om*10^\((16)\)]*10^\((\(-16\))\), 2/10, Pi/4, 
        1], {om, 0.4, 0.7}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.24603 3.1746 0.0216213 5.27447 [
[.18254 .00912 -12 -9 ]
[.18254 .00912 12 0 ]
[.34127 .00912 -9 -9 ]
[.34127 .00912 9 0 ]
[.5 .00912 -12 -9 ]
[.5 .00912 12 0 ]
[.65873 .00912 -9 -9 ]
[.65873 .00912 9 0 ]
[.81746 .00912 -12 -9 ]
[.81746 .00912 12 0 ]
[.97619 .00912 -9 -9 ]
[.97619 .00912 9 0 ]
[.01131 .12711 -24 -4.5 ]
[.01131 .12711 0 4.5 ]
[.01131 .2326 -24 -4.5 ]
[.01131 .2326 0 4.5 ]
[.01131 .33809 -24 -4.5 ]
[.01131 .33809 0 4.5 ]
[.01131 .44358 -24 -4.5 ]
[.01131 .44358 0 4.5 ]
[.01131 .54907 -18 -4.5 ]
[.01131 .54907 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .02162 m
.18254 .02787 L
s
[(0.45)] .18254 .00912 0 1 Mshowa
.34127 .02162 m
.34127 .02787 L
s
[(0.5)] .34127 .00912 0 1 Mshowa
.5 .02162 m
.5 .02787 L
s
[(0.55)] .5 .00912 0 1 Mshowa
.65873 .02162 m
.65873 .02787 L
s
[(0.6)] .65873 .00912 0 1 Mshowa
.81746 .02162 m
.81746 .02787 L
s
[(0.65)] .81746 .00912 0 1 Mshowa
.97619 .02162 m
.97619 .02787 L
s
[(0.7)] .97619 .00912 0 1 Mshowa
.125 Mabswid
.05556 .02162 m
.05556 .02537 L
s
.0873 .02162 m
.0873 .02537 L
s
.11905 .02162 m
.11905 .02537 L
s
.15079 .02162 m
.15079 .02537 L
s
.21429 .02162 m
.21429 .02537 L
s
.24603 .02162 m
.24603 .02537 L
s
.27778 .02162 m
.27778 .02537 L
s
.30952 .02162 m
.30952 .02537 L
s
.37302 .02162 m
.37302 .02537 L
s
.40476 .02162 m
.40476 .02537 L
s
.43651 .02162 m
.43651 .02537 L
s
.46825 .02162 m
.46825 .02537 L
s
.53175 .02162 m
.53175 .02537 L
s
.56349 .02162 m
.56349 .02537 L
s
.59524 .02162 m
.59524 .02537 L
s
.62698 .02162 m
.62698 .02537 L
s
.69048 .02162 m
.69048 .02537 L
s
.72222 .02162 m
.72222 .02537 L
s
.75397 .02162 m
.75397 .02537 L
s
.78571 .02162 m
.78571 .02537 L
s
.84921 .02162 m
.84921 .02537 L
s
.88095 .02162 m
.88095 .02537 L
s
.9127 .02162 m
.9127 .02537 L
s
.94444 .02162 m
.94444 .02537 L
s
.25 Mabswid
0 .02162 m
1 .02162 L
s
.02381 .12711 m
.03006 .12711 L
s
[(0.02)] .01131 .12711 1 0 Mshowa
.02381 .2326 m
.03006 .2326 L
s
[(0.04)] .01131 .2326 1 0 Mshowa
.02381 .33809 m
.03006 .33809 L
s
[(0.06)] .01131 .33809 1 0 Mshowa
.02381 .44358 m
.03006 .44358 L
s
[(0.08)] .01131 .44358 1 0 Mshowa
.02381 .54907 m
.03006 .54907 L
s
[(0.1)] .01131 .54907 1 0 Mshowa
.125 Mabswid
.02381 .04799 m
.02756 .04799 L
s
.02381 .07437 m
.02756 .07437 L
s
.02381 .10074 m
.02756 .10074 L
s
.02381 .15348 m
.02756 .15348 L
s
.02381 .17986 m
.02756 .17986 L
s
.02381 .20623 m
.02756 .20623 L
s
.02381 .25897 m
.02756 .25897 L
s
.02381 .28534 m
.02756 .28534 L
s
.02381 .31172 m
.02756 .31172 L
s
.02381 .36446 m
.02756 .36446 L
s
.02381 .39083 m
.02756 .39083 L
s
.02381 .41721 m
.02756 .41721 L
s
.02381 .46995 m
.02756 .46995 L
s
.02381 .49632 m
.02756 .49632 L
s
.02381 .5227 m
.02756 .5227 L
s
.02381 .57544 m
.02756 .57544 L
s
.02381 .60181 m
.02756 .60181 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.08013 .61803 m
.10458 .53192 L
.14415 .41553 L
.18221 .3218 L
.22272 .23919 L
.26171 .17443 L
.30316 .11966 L
.34309 .0791 L
.3815 .05025 L
.40134 .03893 L
.42237 .02944 L
.4419 .02282 L
.45065 .02051 L
.4602 .01844 L
.46558 .01748 L
.47058 .01672 L
.47576 .01607 L
.48045 .01558 L
.48506 .01521 L
.48941 .01496 L
.49181 .01486 L
.49401 .01479 L
.49516 .01476 L
.49642 .01474 L
.49774 .01472 L
.49897 .01472 L
.50023 .01472 L
.50087 .01472 L
.50157 .01472 L
.50281 .01474 L
.50398 .01476 L
.50535 .01479 L
.5066 .01483 L
.50942 .01494 L
.51173 .01505 L
.51425 .01521 L
.51932 .0156 L
.52894 .01667 L
.53788 .01803 L
.55834 .02243 L
.57693 .02792 L
.61691 .04436 L
.65537 .06578 L
.69629 .0942 L
.7357 .1267 L
.77358 .16243 L
.81392 .20502 L
.85275 .2502 L
.89402 .30252 L
.93379 .35689 L
Mistroke
.97203 .41269 L
.97619 .41896 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00>`3oool2
000000@0oooo00<000000?ooo`3oool0103oool010000000oooo0?ooo`3oool3000001P0oooo0P00
00040?ooo`030000003oool0oooo0080oooo0`00000I0?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`<000000`3oool3000001T0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P00
000I0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool3000001T0oooo0P00
00040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool003X0oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01@0oooo00@0
00000?ooo`3oool000002`3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000/0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000005`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000000D0oooo00<000000?ooo`3oool05@3oool010000000oooo0?ooo`0000090?ooo`03
0000003oool0oooo0080oooo000j0?ooo`040000003oool0oooo000000L0oooo1@0000050?ooo`03
0000003oool0oooo01@0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0E0?oo
o`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0E
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000005`3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool05@3oool01000
0000oooo0?ooo`00000:0?ooo`030000003oool0oooo0040oooo000j0?ooo`040000003oool0oooo
000000L0oooo00@000000?ooo`3oool000000`3oool3000001L0oooo00@000000?ooo`3oool00000
203oool3000001P0oooo00@000000?ooo`3oool00000203oool3000000<0oooo0`00000H0?ooo`04
0000003oool0oooo000000P0oooo0`00000H0?ooo`040000003oool0oooo000000P0oooo0`000003
0?ooo`<00000603oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0040oooo000j
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`0000000`3oool00`000000oooo0?oo
o`0G0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0603oool010000000oooo
0?ooo`0000080?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0603oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool000002`3o
ool100000040oooo0@3oool001/0oooo00<000000?ooo`3oool0703oool010000000oooo0?ooo`00
00090?ooo`8000000`3oool00`000000oooo0?ooo`0G0?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool0603oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo
01P0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo01P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`006`3o
ool00`000000oooo0?ooo`0M0?ooo`8000002`3oool010000000oooo0?ooo`3oool4000001L0oooo
0P0000090?ooo`@00000603oool2000000T0oooo100000020?ooo`@00000603oool2000000X0oooo
0`00000H0?ooo`8000002P3oool300000080oooo1000000H0?ooo`8000002@3oool400000080oooo
000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool00`000000oooo0?ooo`1b0?oooa400000O`3oool001/0oooo00<000000?ooo`3o
ool0K03oool600000140oooo1000001k0?ooo`005@3ooooo000000T000000`3oool001/0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0103oool2000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo1@0000030?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo000K
0?ooo`030000003oool0oooo06D0oooo0`00000X0?ooo`800000L03oool001/0oooo00<000000?oo
o`3oool0HP3oool3000002d0oooo0P00001^0?ooo`006`3oool00`000000oooo0?ooo`1P0?ooo`80
0000<P3oool2000006`0oooo000K0?ooo`030000003oool0oooo05h0oooo0P00000f0?ooo`800000
JP3oool001/0oooo00<000000?ooo`3oool0G03oool2000003X0oooo0P00001X0?ooo`006`3oool2
000005`0oooo00<000000?ooo`3oool0?03oool2000006H0oooo000K0?ooo`030000003oool0oooo
05X0oooo00<000000?ooo`3oool0?`3oool2000006@0oooo000K0?ooo`030000003oool0oooo05P0
oooo0P0000140?ooo`800000HP3oool001/0oooo00<000000?ooo`3oool0E`3oool00`000000oooo
0?ooo`160?ooo`800000H03oool001/0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?oo
o`190?ooo`800000GP3oool001/0oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`1<
0?ooo`800000G03oool001/0oooo00<000000?ooo`3oool0D`3oool200000540oooo00<000000?oo
o`3oool0F@3oool001/0oooo0P00001C0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3o
ool0F03oool001/0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`1D0?ooo`800000
F03oool001/0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`1G0?ooo`030000003o
ool0oooo05D0oooo000K0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0F@3oool0
0`000000oooo0?ooo`1D0?ooo`006`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo
05/0oooo0P00001D0?ooo`006`3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo05h0
oooo00<000000?ooo`3oool0D@3oool001/0oooo00<000000?ooo`3oool0B`3oool200000680oooo
00<000000?ooo`3oool0D03oool001/0oooo0P00001;0?ooo`030000003oool0oooo06<0oooo0P00
001@0?ooo`006`3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo06H0oooo00<00000
0?ooo`3oool0C@3oool001/0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`1X0?oo
o`030000003oool0oooo04`0oooo000K0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3o
ool0JP3oool00`000000oooo0?ooo`1;0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool0
0`3oool2000000<0oooo100000040?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0
K03oool2000004/0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3o
ool0K`3oool00`000000oooo0?ooo`180?ooo`0000D0oooo0000003oool0oooo000000080?ooo`04
0000003oool0oooo000000<0oooo00<000000?ooo`3oool0103oool2000004H0oooo00<000000?oo
o`3oool0L03oool00`000000oooo0?ooo`170?ooo`0000D0oooo0000003oool0oooo000000080?oo
o`040000003oool0oooo000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`14
0?ooo`030000003oool0oooo0780oooo00<000000?ooo`3oool0AP3oool000050?ooo`000000oooo
0?ooo`000000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0@`3oool00`000000oooo0?ooo`1d0?ooo`030000003oool0oooo04D0oooo0000
1@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0MP3oool00`000000
oooo0?ooo`140?ooo`000P3oool2000000X0oooo0P0000040?ooo`8000001@3oool00`000000oooo
0?ooo`120?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool0@`3oool001/0oooo00<0
00000?ooo`3oool0@@3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo0480oooo000K
0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`11
0?ooo`006`3oool200000400oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`100?oo
o`006`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3o
ool0?`3oool001/0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`200?ooo`030000
003oool0oooo03h0oooo000K0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0PP3o
ool00`000000oooo0?ooo`0m0?ooo`006`3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0
oooo08@0oooo00<000000?ooo`3oool0?03oool001/0oooo00<000000?ooo`3oool0?03oool00`00
0000oooo0?ooo`250?ooo`030000003oool0oooo03/0oooo000K0?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool0Q`3oool00`000000oooo0?ooo`0j0?ooo`006`3oool2000003/0oooo
00<000000?ooo`3oool0R@3oool00`000000oooo0?ooo`0i0?ooo`006`3oool00`000000oooo0?oo
o`0j0?ooo`030000003oool0oooo08X0oooo00<000000?ooo`3oool0>03oool001/0oooo00<00000
0?ooo`3oool0>@3oool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo03L0oooo000K0?oo
o`030000003oool0oooo03T0oooo00<000000?ooo`3oool0S@3oool00`000000oooo0?ooo`0f0?oo
o`006`3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo08l0oooo00<000000?ooo`3o
ool0=@3oool001/0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`2A0?ooo`030000
003oool0oooo03@0oooo000K0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0TP3o
ool00`000000oooo0?ooo`0c0?ooo`006`3oool2000003L0oooo00<000000?ooo`3oool0U03oool0
0`000000oooo0?ooo`0b0?ooo`006`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo
09D0oooo00<000000?ooo`3oool0<@3oool001/0oooo00<000000?ooo`3oool0=@3oool00`000000
oooo0?ooo`2G0?ooo`030000003oool0oooo0300oooo000K0?ooo`030000003oool0oooo03D0oooo
00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`0`0?ooo`006`3oool00`000000oooo0?oo
o`0d0?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3oool0;`3oool00080oooo0P000004
0?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`2K0?ooo`030000003oool0oooo02h0oooo00001@3o
ool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0W03oool00`000000oooo0?oo
o`0]0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`050000003oool0oooo0000003oool0
1@0000040?ooo`800000<`3oool00`000000oooo0?ooo`2N0?ooo`030000003oool0oooo02`0oooo
00001@3oool000000?ooo`3oool0000000P0oooo00H000000?ooo`3oool000000?ooo`0000020?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`2O0?oo
o`030000003oool0oooo02/0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?oo
o`3oool000000P3oool00`000000oooo000000050?ooo`030000003oool0oooo0340oooo00<00000
0?ooo`3oool0X03oool00`000000oooo0?ooo`0[0?ooo`0000D0oooo0000003oool0oooo00000008
0?ooo`040000003oool0oooo000000<0oooo0P0000050?ooo`030000003oool0oooo0340oooo00<0
00000?ooo`3oool0X@3oool00`000000oooo0?ooo`0Z0?ooo`000P3oool2000000X0oooo0P000005
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`2S
0?ooo`030000003oool0oooo02T0oooo000K0?ooo`030000003oool0oooo0300oooo00<000000?oo
o`3oool0Y03oool00`000000oooo0?ooo`0X0?ooo`006`3oool00`000000oooo0?ooo`0_0?ooo`03
0000003oool0oooo0:H0oooo00<000000?ooo`3oool09`3oool001/0oooo0P00000`0?ooo`030000
003oool0oooo0:L0oooo00<000000?ooo`3oool09P3oool001/0oooo00<000000?ooo`3oool0;P3o
ool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo02H0oooo000K0?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`0U0?ooo`006`3oool00`00
0000oooo0?ooo`0]0?ooo`030000003oool0oooo0:/0oooo00<000000?ooo`3oool0903oool001/0
oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`2/0?ooo`030000003oool0oooo02<0
oooo000K0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0[@3oool00`000000oooo
0?ooo`0S0?ooo`006`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0:h0oooo00<0
00000?ooo`3oool08P3oool001/0oooo0P00000/0?ooo`030000003oool0oooo0;00oooo00<00000
0?ooo`3oool08@3oool001/0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`2a0?oo
o`030000003oool0oooo0200oooo000K0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3o
ool0/`3oool00`000000oooo0?ooo`0O0?ooo`006`3oool00`000000oooo0?ooo`0Z0?ooo`030000
003oool0oooo0;<0oooo00<000000?ooo`3oool07`3oool001/0oooo00<000000?ooo`3oool0:@3o
ool00`000000oooo0?ooo`2e0?ooo`030000003oool0oooo01h0oooo000K0?ooo`030000003oool0
oooo02T0oooo00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`0M0?ooo`006`3oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3oool0703oool001/0
oooo0P00000Y0?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3oool0703oool001/0oooo
00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`2j0?ooo`030000003oool0oooo01/0oooo
000K0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0^`3oool00`000000oooo0?oo
o`0J0?ooo`006`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0;d0oooo00<00000
0?ooo`3oool06@3oool001/0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`2m0?oo
o`030000003oool0oooo01T0oooo00020?ooo`800000103oool00`000000oooo0?ooo`030?ooo`80
0000103oool2000000D0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`2o0?ooo`03
0000003oool0oooo01P0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3o
ool000000P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02D0oooo00<00000
0?ooo`3oool0`03oool00`000000oooo0?ooo`0G0?ooo`0000D0oooo0000003oool0oooo00000008
0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool2000002D0oooo
00<000000?ooo`3oool0`@3oool00`000000oooo0?ooo`0G0?ooo`0000D0oooo0000003oool0oooo
000000080?ooo`040000003oool0oooo00000080oooo0`0000050?ooo`030000003oool0oooo02@0
oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0F0?ooo`0000D0oooo0000003oool0
oooo000000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo0<<0oooo00<000000?ooo`3oool05@3oool00005
0?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`350?ooo`030000003oool0
oooo01@0oooo00020?ooo`8000002P3oool2000000@0oooo0`0000040?ooo`030000003oool0oooo
02<0oooo00<000000?ooo`3oool0a@3oool00`000000oooo0?ooo`0D0?ooo`006`3oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool04`3oool001/0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`380?ooo`030000003oool0oooo0180oooo
000K0?ooo`8000008P3oool00`000000oooo0?ooo`390?ooo`030000003oool0oooo0180oooo000K
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0bP3oool00`000000oooo0?ooo`0A
0?ooo`006`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0</0oooo00<000000?oo
o`3oool0403oool001/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3<0?ooo`03
0000003oool0oooo0100oooo000K0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
c@3oool00`000000oooo0?ooo`0?0?ooo`006`3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo0<l0oooo00<000000?ooo`3oool03P3oool001/0oooo00<000000?ooo`3oool07`3oool0
0`000000oooo0?ooo`3?0?ooo`030000003oool0oooo00h0oooo000K0?ooo`800000803oool00`00
0000oooo0?ooo`3@0?ooo`030000003oool0oooo00d0oooo000K0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`0<0?ooo`006`3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo0=80oooo00<000000?ooo`3oool0303oool001/0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo00/0oooo000K
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0e@3oool00`000000oooo0?ooo`0:
0?ooo`006`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0=H0oooo00<000000?oo
o`3oool02P3oool001/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3G0?ooo`03
0000003oool0oooo00T0oooo000K0?ooo`8000007@3oool00`000000oooo0?ooo`3H0?ooo`800000
2@3oool001/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3T0?ooo`006`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0>@0oooo000K0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool0i03oool001/0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`3U0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo
0P0000050?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0i@3oool000050?ooo`00
0000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0000
00@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3U0?ooo`0000D0oooo0000003o
ool0oooo000000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103o
ool2000001X0oooo00<000000?ooo`3oool0iP3oool000050?ooo`000000oooo0?ooo`000000203o
ool010000000oooo0?ooo`0000030?ooo`8000001@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo0>H0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool0
00000P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0iP3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`000002
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`3W0?ooo`000P3oool2000000X0oooo0P0000040?ooo`8000001@3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo0>L0oooo000K0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0i`3oool001/0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`3X0?ooo`00
6`3oool2000001P0oooo00<000000?ooo`3oool0j03oool001/0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`3X0?ooo`006`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo0>P0oooo000K0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0j@3oool001/0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`3Y0?ooo`006`3oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo0>T0oooo000K0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool0jP3oool001/0oooo0P00000F0?ooo`030000003oool0oooo0>X0oooo000K0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool0jP3oool001/0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`3[0?ooo`006`3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo0>/0oooo000K0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0j`3o
ool001/0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3/0?ooo`006`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo0>`0oooo000K0?ooo`800000503oool00`000000
oooo0?ooo`3/0?ooo`006`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0>d0oooo
000K0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0k@3oool001/0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`3]0?ooo`006`3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0>h0oooo00080?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@0
0000103oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0>h0oooo00070?ooo`040000
003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo0>h0oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool00`3oool200000180oooo00<000000?ooo`3oool0kP3oool000L0oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0k`3oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0k`3oool000L0oooo00@000000?ooo`3oool0
0000203oool3000000D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3_0?ooo`00
203oool2000000/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0>l0oooo000K0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0l03o
ool001/0oooo0P00000@0?ooo`030000003oool0oooo0?00oooo000K0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0l03oool001/0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`3`0?ooo`006`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0?40oooo
000K0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0l@3oool001/0oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`3a0?ooo`006`3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo0?40oooo000K0?ooo`8000003P3oool00`000000oooo0?ooo`3b0?ooo`00
6`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?80oooo000K0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0lP3oool001/0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`3b0?ooo`006`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
0?<0oooo000K0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0l`3oool00?l0oooo
8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.367342, -0.0116057, \
0.00119662, 0.00072022}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, PlotRange \[Rule] {\(-0.01\), 0.01}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.24603 3.1746 0.309017 30.9017 [
[.18254 .29652 -12 -9 ]
[.18254 .29652 12 0 ]
[.34127 .29652 -9 -9 ]
[.34127 .29652 9 0 ]
[.5 .29652 -12 -9 ]
[.5 .29652 12 0 ]
[.65873 .29652 -9 -9 ]
[.65873 .29652 9 0 ]
[.81746 .29652 -12 -9 ]
[.81746 .29652 12 0 ]
[.97619 .29652 -9 -9 ]
[.97619 .29652 9 0 ]
[.01131 0 -30 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .07725 -42 -4.5 ]
[.01131 .07725 0 4.5 ]
[.01131 .15451 -36 -4.5 ]
[.01131 .15451 0 4.5 ]
[.01131 .23176 -42 -4.5 ]
[.01131 .23176 0 4.5 ]
[.01131 .38627 -36 -4.5 ]
[.01131 .38627 0 4.5 ]
[.01131 .46353 -30 -4.5 ]
[.01131 .46353 0 4.5 ]
[.01131 .54078 -36 -4.5 ]
[.01131 .54078 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .30902 m
.18254 .31527 L
s
[(0.45)] .18254 .29652 0 1 Mshowa
.34127 .30902 m
.34127 .31527 L
s
[(0.5)] .34127 .29652 0 1 Mshowa
.5 .30902 m
.5 .31527 L
s
[(0.55)] .5 .29652 0 1 Mshowa
.65873 .30902 m
.65873 .31527 L
s
[(0.6)] .65873 .29652 0 1 Mshowa
.81746 .30902 m
.81746 .31527 L
s
[(0.65)] .81746 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(0.7)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.05556 .30902 m
.05556 .31277 L
s
.0873 .30902 m
.0873 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.15079 .30902 m
.15079 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.24603 .30902 m
.24603 .31277 L
s
.27778 .30902 m
.27778 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.37302 .30902 m
.37302 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.43651 .30902 m
.43651 .31277 L
s
.46825 .30902 m
.46825 .31277 L
s
.53175 .30902 m
.53175 .31277 L
s
.56349 .30902 m
.56349 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.62698 .30902 m
.62698 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.72222 .30902 m
.72222 .31277 L
s
.75397 .30902 m
.75397 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.84921 .30902 m
.84921 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.9127 .30902 m
.9127 .31277 L
s
.94444 .30902 m
.94444 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 0 m
.03006 0 L
s
[(-0.01)] .01131 0 1 0 Mshowa
.02381 .07725 m
.03006 .07725 L
s
[(-0.0075)] .01131 .07725 1 0 Mshowa
.02381 .15451 m
.03006 .15451 L
s
[(-0.005)] .01131 .15451 1 0 Mshowa
.02381 .23176 m
.03006 .23176 L
s
[(-0.0025)] .01131 .23176 1 0 Mshowa
.02381 .38627 m
.03006 .38627 L
s
[(0.0025)] .01131 .38627 1 0 Mshowa
.02381 .46353 m
.03006 .46353 L
s
[(0.005)] .01131 .46353 1 0 Mshowa
.02381 .54078 m
.03006 .54078 L
s
[(0.0075)] .01131 .54078 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.01)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .01545 m
.02756 .01545 L
s
.02381 .0309 m
.02756 .0309 L
s
.02381 .04635 m
.02756 .04635 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .10816 m
.02756 .10816 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .13906 m
.02756 .13906 L
s
.02381 .16996 m
.02756 .16996 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .20086 m
.02756 .20086 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26266 m
.02756 .26266 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .29357 m
.02756 .29357 L
s
.02381 .32447 m
.02756 .32447 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .35537 m
.02756 .35537 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .41717 m
.02756 .41717 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .44807 m
.02756 .44807 L
s
.02381 .47898 m
.02756 .47898 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .50988 m
.02756 .50988 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57168 m
.02756 .57168 L
s
.02381 .58713 m
.02756 .58713 L
s
.02381 .60258 m
.02756 .60258 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.34939 .61803 m
.3815 .47673 L
.40134 .41045 L
.42237 .35482 L
.4419 .31601 L
.45065 .3025 L
.4602 .2904 L
.46558 .28478 L
.47058 .28032 L
.47576 .27647 L
.48045 .27365 L
.48506 .27148 L
.48941 .26999 L
.49181 .26939 L
.49401 .26899 L
.49516 .26883 L
.49642 .26869 L
.49774 .2686 L
.49897 .26856 L
.50023 .26856 L
.50087 .26857 L
.50157 .2686 L
.50281 .26869 L
.50398 .26881 L
.50535 .26899 L
.5066 .26921 L
.50942 .26985 L
.51173 .27053 L
.51425 .27143 L
.51932 .27377 L
.52894 .28003 L
.53788 .28799 L
.55834 .31374 L
.57693 .34594 L
.61691 .44225 L
.65537 .56775 L
s
.65537 .56775 m
.66773 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00503oool2
000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo1000003g0?ooo`004`3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0mP3o
ool001<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo00<0oooo0P00003a0?ooo`003@3oool400000080oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0l03oool001<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0l03oool001<0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000020?ooo`<000001@3oool00`000000oooo0?ooo`3`
0?ooo`00503oool2000000X0oooo0P0000050?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0
l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool0
02d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo
00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<00000
0?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3o
ool0l03oool002d0oooo00<000000?ooo`3oool0l03oool000P0oooo0P0000040?ooo`030000003o
ool0oooo00<0oooo0P0000040?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<000001@3o
ool00`000000oooo0?ooo`3`0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0l03oool000L0oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`800000l@3oool000030?ooo`0000000000
008000000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool00000103oool010000000oooo0?ooo`3oool3000000D0oooo00<000000?ooo`3o
ool0l03oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo000000D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?oo
o`3`0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0l03oool000P0oooo0P00000:0?ooo`800000103oool2000000<0
oooo100000020?ooo`@00000103oool00`000000oooo0?ooo`3`0?ooo`00;@3oool00`000000oooo
0?ooo`3`0?ooo`00;@3oool00`000000oooo0?ooo`3`0?ooo`00;@3oool00`000000oooo0?ooo`3`
0?ooo`00;@3oool00`000000oooo0?ooo`3`0?ooo`00;@3oool00`000000oooo0?ooo`3`0?ooo`00
;@3oool00`000000oooo0?ooo`3`0?ooo`00;@3oool00`000000oooo0?ooo`3`0?ooo`00;@3oool0
0`000000oooo0?ooo`3`0?ooo`00;@3oool00`000000oooo0?ooo`3`0?ooo`00;@3oool00`000000
oooo0?ooo`3`0?ooo`00;@3oool00`000000oooo0?ooo`3`0?ooo`00;@3oool00`000000oooo0?oo
o`3`0?ooo`003P3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P000003
0?ooo`<000001@3oool00`000000oooo0?ooo`3`0?ooo`003@3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0?00oooo000=0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo
0080oooo0P00003a0?ooo`001`3oool400000080oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo0`0000050?ooo`030000003o
ool0oooo0?00oooo000=0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0l03oool000d0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3`
0?ooo`003P3oool2000000X0oooo0P0000040?ooo`8000000`3oool4000000@0oooo00<000000?oo
o`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0
l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool0
02d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo
00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<00000
0?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<000000?ooo`3o
ool0l03oool002d0oooo00<000000?ooo`3oool0l03oool000P0oooo0P0000040?ooo`030000003o
ool0oooo00<0oooo0P0000040?ooo`8000000`3oool400000080oooo0`0000050?ooo`030000003o
ool0oooo0?00oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`3`0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo00000080oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo0080oooo0P00003a0?ooo`0000<0oooo0000000000000P0000020?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`3`0?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool000001@3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo0?00oooo0007
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`3`0?ooo`00203oool2000000X0oooo0P0000040?ooo`800000103oool2000000<0
oooo100000040?ooo`030000003oool0oooo0?00oooo000]0?ooo`030000003oool0oooo0?00oooo
000]0?ooo`030000003oool0oooo0?00oooo000]0?ooo`030000003oool0oooo0?00oooo000]0?oo
o`030000003oool0oooo0?00oooo000]0?ooo`030000003oool0oooo01T0oooo0P0000040?ooo`03
0000003oool0oooo00@0oooo00@000000?ooo`3oool0oooo0`00000F0?ooo`800000103oool00`00
0000oooo0?ooo`020?ooo`<000005P3oool2000000@0oooo200000030?ooo`<000005P3oool20000
00@0oooo00<000000?ooo`3oool00`3oool2000001H0oooo0P0000040?ooo`030000003oool0oooo
00<0oooo0P0000030?ooo`<000005P3oool2000000@0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`020?ooo`00;@3oool00`000000oooo0?ooo`0H0?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`040000003oool0oooo
000000/0oooo00<000000?ooo`3oool04P3oool010000000oooo0?ooo`0000020?ooo`8000001P3o
ool3000000@0oooo00<000000?ooo`3oool04P3oool010000000oooo0?ooo`0000080?ooo`040000
003oool0oooo000001@0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo0180oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?oo
o`020?ooo`00;@3oool00`000000oooo0?ooo`0H0?ooo`040000003oool0oooo000000L0oooo1@00
00050?ooo`030000003oool0oooo0180oooo00@000000?ooo`3oool000002`3oool00`000000oooo
0?ooo`0B0?ooo`060000003oool0oooo0000003oool000002@3oool00`000000oooo000000030?oo
o`030000003oool0oooo0180oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
000D0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`000000
oooo0?ooo`0B0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00@3oool002d0
oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo
000000<0oooo0`00000E0?ooo`040000003oool0oooo000000P0oooo0`00000E0?ooo`030000003o
ool0oooo008000001`3oool3000000<0oooo0`00000E0?ooo`040000003oool0oooo000000P0oooo
0`00000E0?ooo`040000003oool0oooo000000P0oooo0`0000030?ooo`<000005@3oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo0040oooo000]0?ooo`030000003oool0oooo01P0
oooo00@000000?ooo`3oool00000203oool00`000000oooo000000030?ooo`030000003oool0oooo
01D0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0E0?ooo`040000003oool0
0000000000P0oooo00<000000?ooo`3oool00`3oool2000001H0oooo00@000000?ooo`3oool00000
203oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000/0oooo0@0000010?oo
o`40oooo000]0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool000002@3oool20000
00<0oooo00<000000?ooo`3oool05@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0
oooo01D0oooo0P0000000`3oool000000?ooo`070?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`0000005@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo01D0oooo00@0
00000?ooo`3oool00000203oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01D0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`00;@3oool00`000000
oooo0?ooo`0I0?ooo`8000002`3oool010000000oooo0?ooo`3oool4000001D0oooo0P0000090?oo
o`@00000503oool3000000T0oooo100000020?ooo`@000005@3oool2000000X0oooo0`00000E0?oo
o`8000002P3oool300000080oooo1000000E0?ooo`8000002@3oool400000080oooo000]0?ooo`03
0000003oool0oooo06H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`1a0?ooo`00
;@3oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
L@3oool002d0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`0E0?ooo`030000003o
ool0oooo0700oooo000W0?ooooD00000103oool002d0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo000000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0000000H0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01`3oool002d0
oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00L0
oooo000]0?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`1]0?ooo`00;@3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool0K03oool002d0oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo06`0oooo000]0?ooo`030000003oool0oooo0640oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`1[0?ooo`00;@3oool00`000000oooo0?ooo`1P0?ooo`03
0000003oool0oooo0200oooo00<000000?ooo`3oool0JP3oool002d0oooo00<000000?ooo`3oool0
H03oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo06X0oooo000]0?ooo`030000003o
ool0oooo05l0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`1Y0?ooo`00;@3oool0
0`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0J@3oool0
02d0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
06P0oooo000]0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0903oool00`000000
oooo0?ooo`1X0?ooo`00;@3oool00`000000oooo0?ooo`1M0?ooo`030000003oool0oooo02H0oooo
00<000000?ooo`3oool0I`3oool002d0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?oo
o`0V0?ooo`030000003oool0oooo06L0oooo000]0?ooo`030000003oool0oooo05d0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`1W0?ooo`00;@3oool00`000000oooo0?ooo`1L0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0IP3oool000P0oooo0P0000040?ooo`03
0000003oool0oooo00<0oooo0P0000040?ooo`8000000`3oool400000080oooo0`0000050?ooo`03
0000003oool0oooo05`0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1V0?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0G03oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo06D0oooo00070?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?oo
o`0000030?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool2000005`0oooo
00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`1U0?ooo`001`3oool010000000oooo0?oo
o`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`3oool3000000D0oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`0[
0?ooo`030000003oool0oooo06@0oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`040000003oool0oooo000000L0
oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo06@0
oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0I03o
ool000P0oooo0P00000:0?ooo`800000103oool2000000@0oooo0P0000030?ooo`@00000103oool0
0`000000oooo0?ooo`1J0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0H`3oool0
02d0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
06<0oooo000]0?ooo`030000003oool0oooo05T0oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo02l0oooo
00<000000?ooo`3oool0HP3oool002d0oooo00<000000?ooo`3oool0F03oool00`000000oooo0?oo
o`0`0?ooo`030000003oool0oooo0680oooo000]0?ooo`030000003oool0oooo05P0oooo00<00000
0?ooo`3oool0<@3oool00`000000oooo0?ooo`1Q0?ooo`00;@3oool00`000000oooo0?ooo`1H0?oo
o`030000003oool0oooo0340oooo00<000000?ooo`3oool0H@3oool002d0oooo00<000000?ooo`3o
ool0E`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0600oooo000]0?ooo`030000
003oool0oooo05L0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`1P0?ooo`00;@3o
ool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0G`3o
ool002d0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0
oooo05l0oooo000]0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0=@3oool00`00
0000oooo0?ooo`1O0?ooo`00;@3oool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo03H0
oooo00<000000?ooo`3oool0GP3oool000h0oooo0P0000040?ooo`030000003oool0oooo00<0oooo
0P0000040?ooo`8000000`3oool3000000D0oooo00<000000?ooo`3oool0EP3oool00`000000oooo
0?ooo`0f0?ooo`030000003oool0oooo05h0oooo000=0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo
05h0oooo000=0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo0P00001F0?ooo`030000003o
ool0oooo03L0oooo00<000000?ooo`3oool0GP3oool000d0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo0`0000050?ooo`03
0000003oool0oooo05D0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`1M0?ooo`00
3@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo05@0oooo00<00000
0?ooo`3oool0>@3oool00`000000oooo0?ooo`1M0?ooo`003@3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool0>@3oool00`000000oooo
0?ooo`1M0?ooo`003P3oool2000000X0oooo0P0000040?ooo`8000000`3oool4000000@0oooo00<0
00000?ooo`3oool0E03oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo05`0oooo000]
0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`1L
0?ooo`00;@3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo03/0oooo00<000000?oo
o`3oool0G03oool002d0oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`0l0?ooo`03
0000003oool0oooo05/0oooo000]0?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool0
?03oool00`000000oooo0?ooo`1K0?ooo`00;@3oool00`000000oooo0?ooo`1C0?ooo`030000003o
ool0oooo03`0oooo00<000000?ooo`3oool0F`3oool002d0oooo00<000000?ooo`3oool0DP3oool0
0`000000oooo0?ooo`0m0?ooo`030000003oool0oooo05/0oooo000]0?ooo`030000003oool0oooo
0580oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`1J0?ooo`00;@3oool00`000000
oooo0?ooo`1B0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0FP3oool002d0oooo
00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo05X0oooo
000]0?ooo`030000003oool0oooo0540oooo00<000000?ooo`3oool0@03oool00`000000oooo0?oo
o`1I0?ooo`00;@3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo0400oooo00<00000
0?ooo`3oool0F@3oool002d0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`100?oo
o`030000003oool0oooo05T0oooo00080?ooo`800000103oool00`000000oooo0?ooo`030?ooo`80
0000103oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3o
ool0D@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo05P0oooo00070?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`000003
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1A
0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0F03oool000L0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`800000D@3oool00`000000oooo
0?ooo`120?ooo`030000003oool0oooo05P0oooo00070?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo
0?ooo`<000001@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo0480oooo00<00000
0?ooo`3oool0F03oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00020?ooo`040000003oool0oooo000000D0oooo00@000000?ooo`3oool000001`3oool00`000000
oooo0?ooo`1@0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0E`3oool000L0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo0500oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`1G0?ooo`00203oool2
000000X0oooo0P0000040?ooo`8000000`3oool400000080oooo100000040?ooo`030000003oool0
oooo04l0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`1G0?ooo`00;@3oool00`00
0000oooo0?ooo`1?0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0EP3oool002d0
oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo05H0
oooo000]0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0A@3oool00`000000oooo
0?ooo`1F0?ooo`00;@3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo04D0oooo00<0
00000?ooo`3oool0EP3oool002d0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`17
0?ooo`030000003oool0oooo05D0oooo000]0?ooo`030000003oool0oooo04h0oooo00<000000?oo
o`3oool0A`3oool00`000000oooo0?ooo`1E0?ooo`00;@3oool00`000000oooo0?ooo`1>0?ooo`03
0000003oool0oooo04L0oooo00<000000?ooo`3oool0E@3oool002d0oooo00<000000?ooo`3oool0
CP3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo05D0oooo000]0?ooo`030000003o
ool0oooo04d0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`1D0?ooo`00;@3oool0
0`000000oooo0?ooo`1=0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0E03oool0
02d0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo
05@0oooo000]0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0B@3oool00`000000
oooo0?ooo`1D0?ooo`00503oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo
100000040?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0B`3oool00`000000oooo
0?ooo`1C0?ooo`004`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo04/0
oooo00<000000?ooo`3oool0D`3oool001<0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo0P00001=0?ooo`030000003oool0oooo
04/0oooo00<000000?ooo`3oool0D`3oool001<0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo0?H0oooo000C0?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3f0?ooo`004`3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`00003h0?ooo`00503o
ool2000000X0oooo0P0000050?ooo`030000003oool0oooo0?H0oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.34197, -0.0117051, \
0.00128788, 0.000132307}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["damped mode II", "Subsection"],

Cell[CellGroupData[{

Cell["Prelims", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\(xi^3/Sqrt[xi^2 - kt^2]\)/\((1 + a\ xi^2)\)^2, {xi, kt, 
        1}]\)], "Input"],

Cell[BoxData[
    \(\((\(-\@a\)\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - 
          ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)] - 
          a\ ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)] - 
          2\ a\ kt\^2\ ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ \
kt\^2\)] - 
          2\ a\^2\ kt\^2\ ArcTanh[\(\@a\ \@\(1 - kt\^2\)\)\/\@\(\(-1\) - a\ \
kt\^2\)])\)/\((2\ a\^\(3/2\)\ \((1 + a)\)\ \@\(\(-1\) - a\ kt\^2\)\ \((1 + 
              a\ kt\^2)\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\(-\@a\)\ \@\(1 - kt\^2\)\ \@\(\(-1\) - a\ kt\^2\) - \((1 + a + 
                  2\ a\ kt\^2 + 
                  2\ a\^2\ kt\^2)\)\ ArcTanh[\(\@a\ \@\(1 - \
kt\^2\)\)\/\@\(\(-1\) - a\ kt\^2\)]\)\/\(2\ a\^\(3/2\)\ \((1 + 
                a)\)\ \((\(-1\) - a\ kt\^2)\)\^\(3/2\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% /. kt^2 \[Rule] \((1 - kz^2/k^2)\)\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\(-\@a\)\ \@\(kz\^2\/k\^2\)\ \@\(\(-1\) - a\ \((1 - kz\^2\/k\^2)\
\)\) - \((1 + a + 2\ a\ \((1 - kz\^2\/k\^2)\) + 
                  2\ a\^2\ \((1 - 
                        kz\^2\/k\^2)\))\)\ ArcTanh[\(\@a\ \@\(kz\^2\/k\^2\)\)\
\/\@\(\(-1\) - a\ \((1 - kz\^2\/k\^2)\)\)]\)\/\(2\ a\^\(3/2\)\ \((1 + 
                a)\)\ \((\(-1\) - a\ \((1 - \
kz\^2\/k\^2)\))\)\^\(3/2\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(three = Simplify[%, {k > 0, kz > 0}]\)], "Input"],

Cell[BoxData[
    \(\(\@a\ k\ kz\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\) + \((1 + \
a)\)\ \((\((1 + 2\ a)\)\ k\^2 - 2\ a\ kz\^2)\)\ ArcTanh[\(\@a\ \
kz\)\/\@\(\(-\((1 + a)\)\)\ k\^2 + a\ kz\^2\)]\)\/\(2\ a\^\(3/2\)\ \((1 + \
a)\)\ k\^2\ \((\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\))\)\^\(3/2\)\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(two = 
      Simplify[%44 /. kt^2 \[Rule] \((1 - kz^2/k^2)\), {k > 0, 
          kz > 0}]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\@a\ kz\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\) + \((1 + 
                  a)\)\ k\ ArcTanh[\(\@a\ kz\)\/\@\(\(-\((1 + a)\)\)\ k\^2 + \
a\ kz\^2\)]\)\/\(2\ a\^\(3/2\)\ \((1 + 
                a)\)\ k\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\)\)\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(one = %27\)], "Input"],

Cell[BoxData[
    \(\(\(-\@a\) + ArcTan[\@a] + a\ ArcTan[\@a]\)\/\(2\ a\^\(3/2\)\ \((1 + a)\
\)\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[two + three]\)], "Input"],

Cell[BoxData[
    \(\((\@a\ kz\ \((\(-\((2 + a)\)\)\ k\^2 + 
                a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\) + \
\((\(-\((2 + 5\ a + 3\ a\^2)\)\)\ k\^3 + 
                3\ a\ \((1 + 
                      a)\)\ k\ kz\^2)\)\ ArcTanh[\(\@a\ kz\)\/\@\(\(-\((1 + \
a)\)\)\ k\^2 + a\ kz\^2\)])\)/\((2\ a\^\(3/2\)\ \((1 + 
              a)\)\ k\ \((\((1 + a)\)\ k\^2 - 
              a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\))\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[\((1 + 2*kz^2/\((k^2 - kz^2)\))\)*one - 
        kz*k/\((1 - kz^2/k^2)\)*\((two + three)\)]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(2\ a\^\(3/2\)\ \((1 + a)\)\ \((k\^2 - kz\^2)\)\)\) \((\((k\^2 + 
                kz\^2)\)\ \((\(-\@a\) + \((1 + 
                      a)\)\ ArcTan[\@a])\) + \((k\^2\ kz\ \((\@a\ kz\ \((\((2 \
+ a)\)\ k\^2 - 
                          a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\) + \((1 + a)\)\ k\ \((\((2 + 3\ a)\)\ k\^2 - 
                          3\ a\ kz\^2)\)\ ArcTanh[\(\@a\ kz\)\/\@\(\(-\((1 + \
a)\)\)\ k\^2 + a\ kz\^2\)])\))\)/\((\((\((1 + a)\)\ k\^2 - 
                    a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\))\))\)\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(gamoz[md2_, kz_, k_, a_] := Evaluate[a*md2*Simplify[%114]]\)], "Input"],

Cell[BoxData[
    \(gamoz[md2_, kz_, k_, a_] := 
      Evaluate[a*
          md2*\(1\/\(2\ a\^\(3/2\)\ \((1 + a)\)\ \((k\^2 - 
                    kz\^2)\)\)\) \((\((k\^2 + 
                    kz\^2)\)\ \((\(-\@a\) + \((1 + 
                          a)\)\ ArcTan[\@a])\) + \((k\^2\ kz\ \((\@a\ kz\ \((\
\((2 + a)\)\ k\^2 - 
                              a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + \
kz\^2\/k\^2)\)\) + \((1 + a)\)\ k\ \((\((2 + 3\ a)\)\ k\^2 - 
                              3\ a\ kz\^2)\)\ ArcTanh[\(\@a\ \
kz\)\/\@\(\(-\((1 + a)\)\)\ k\^2 + a\ kz\^2\)])\))\)/\((\((\((1 + 
                              a)\)\ k\^2 - 
                        a\ kz\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\
\)\))\))\)]\)], "Input"],

Cell[BoxData[
    \(gamoz[md2_, kz_, k_, a_] := 
      md2*a*\((NIntegrate[
              xi^2*\((1 + 
                      2\ kz^2/\((k^2 - kz^2)\))\)/\((1 + a\ xi^2)\)^2, {xi, 
                0, 1}] - 
            NIntegrate[
              xi*\ kz*\ 
                k/\((k^2 - 
                      kz^2)\)\ \(\((2\ xi^2 - 1 + 
                        kz^2/k^2)\)/\((1 + a\ xi^2)\)^2\)/
                  Sqrt[xi^2 - 1 + kz^2/k^2], {xi, Sqrt[1 - kz^2/k^2], 1}, 
              WorkingPrecision \[Rule] 20, 
              MaxRecursion \[Rule] 20])\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(gamoz[1, 0.99, 1, 0.00001]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(xi\\ 0.99`\\ \
\(\((\(\(2\\ xi\^2\)\) - 1 + 0.99`\^2\/1\^2)\)\)\)\/\(\(\((1\^2 - 0.99`\^2)\)\
\)\\ \(\((\((\(\[LeftSkeleton] 1 \[RightSkeleton]\))\)\^2\\ \@\(xi\^2 - 1 + \
\(\[LeftSkeleton] 5 \[RightSkeleton]\)\^2\/1\^2\))\)\)\)\)) is less than \
WorkingPrecision (\!\(20\))."\)], "Message"],

Cell[BoxData[
    \(6.633412631593672`*^-8\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(gamoz[1, 0.99, 1, 0.00001]\)], "Input"],

Cell[BoxData[
    \(6.633409108117785`*^-8 + 0.`\ \[ImaginaryI]\)], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-\((alomz[1, 0.99, 1, a] + gamoz[1, 0.99, 1, a])\)\)/
        mMag[1, a], {a, \(-1\), 20}, 
      PlotStyle \[Rule] {RGBColor[1, 0, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 -0.375953 0.961805 [
[.29592 .57335 -3 -9 ]
[.29592 .57335 3 0 ]
[.52268 .57335 -6 -9 ]
[.52268 .57335 6 0 ]
[.74943 .57335 -6 -9 ]
[.74943 .57335 6 0 ]
[.97619 .57335 -6 -9 ]
[.97619 .57335 6 0 ]
[.05666 .00877 -18 -4.5 ]
[.05666 .00877 0 4.5 ]
[.05666 .10495 -18 -4.5 ]
[.05666 .10495 0 4.5 ]
[.05666 .20113 -18 -4.5 ]
[.05666 .20113 0 4.5 ]
[.05666 .29731 -18 -4.5 ]
[.05666 .29731 0 4.5 ]
[.05666 .39349 -18 -4.5 ]
[.05666 .39349 0 4.5 ]
[.05666 .48967 -18 -4.5 ]
[.05666 .48967 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .58585 m
.29592 .5921 L
s
[(5)] .29592 .57335 0 1 Mshowa
.52268 .58585 m
.52268 .5921 L
s
[(10)] .52268 .57335 0 1 Mshowa
.74943 .58585 m
.74943 .5921 L
s
[(15)] .74943 .57335 0 1 Mshowa
.97619 .58585 m
.97619 .5921 L
s
[(20)] .97619 .57335 0 1 Mshowa
.125 Mabswid
.11451 .58585 m
.11451 .5896 L
s
.15986 .58585 m
.15986 .5896 L
s
.20522 .58585 m
.20522 .5896 L
s
.25057 .58585 m
.25057 .5896 L
s
.34127 .58585 m
.34127 .5896 L
s
.38662 .58585 m
.38662 .5896 L
s
.43197 .58585 m
.43197 .5896 L
s
.47732 .58585 m
.47732 .5896 L
s
.56803 .58585 m
.56803 .5896 L
s
.61338 .58585 m
.61338 .5896 L
s
.65873 .58585 m
.65873 .5896 L
s
.70408 .58585 m
.70408 .5896 L
s
.79478 .58585 m
.79478 .5896 L
s
.84014 .58585 m
.84014 .5896 L
s
.88549 .58585 m
.88549 .5896 L
s
.93084 .58585 m
.93084 .5896 L
s
.02381 .58585 m
.02381 .5896 L
s
.25 Mabswid
0 .58585 m
1 .58585 L
s
.06916 .00877 m
.07541 .00877 L
s
[(0.4)] .05666 .00877 1 0 Mshowa
.06916 .10495 m
.07541 .10495 L
s
[(0.5)] .05666 .10495 1 0 Mshowa
.06916 .20113 m
.07541 .20113 L
s
[(0.6)] .05666 .20113 1 0 Mshowa
.06916 .29731 m
.07541 .29731 L
s
[(0.7)] .05666 .29731 1 0 Mshowa
.06916 .39349 m
.07541 .39349 L
s
[(0.8)] .05666 .39349 1 0 Mshowa
.06916 .48967 m
.07541 .48967 L
s
[(0.9)] .05666 .48967 1 0 Mshowa
.125 Mabswid
.06916 .02801 m
.07291 .02801 L
s
.06916 .04724 m
.07291 .04724 L
s
.06916 .06648 m
.07291 .06648 L
s
.06916 .08571 m
.07291 .08571 L
s
.06916 .12419 m
.07291 .12419 L
s
.06916 .14342 m
.07291 .14342 L
s
.06916 .16266 m
.07291 .16266 L
s
.06916 .18189 m
.07291 .18189 L
s
.06916 .22037 m
.07291 .22037 L
s
.06916 .2396 m
.07291 .2396 L
s
.06916 .25884 m
.07291 .25884 L
s
.06916 .27807 m
.07291 .27807 L
s
.06916 .31655 m
.07291 .31655 L
s
.06916 .33578 m
.07291 .33578 L
s
.06916 .35502 m
.07291 .35502 L
s
.06916 .37425 m
.07291 .37425 L
s
.06916 .41273 m
.07291 .41273 L
s
.06916 .43196 m
.07291 .43196 L
s
.06916 .4512 m
.07291 .4512 L
s
.06916 .47044 m
.07291 .47044 L
s
.06916 .50891 m
.07291 .50891 L
s
.06916 .52814 m
.07291 .52814 L
s
.06916 .54738 m
.07291 .54738 L
s
.06916 .56662 m
.07291 .56662 L
s
.06916 .60509 m
.07291 .60509 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.02381 .60332 m
.06244 .55425 L
.10458 .51393 L
.14415 .47868 L
.18221 .44661 L
.22272 .41419 L
.26171 .3845 L
.30316 .35442 L
.34309 .3268 L
.3815 .30139 L
.42237 .27553 L
.46172 .25171 L
.49955 .22974 L
.53984 .2073 L
.57861 .18659 L
.61984 .16547 L
.65954 .14594 L
.69774 .12788 L
.73838 .10941 L
.77751 .09231 L
.81909 .07484 L
.85916 .05865 L
.89771 .04364 L
.93871 .02826 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo203oool00080oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo203oool00080oooo
00@000000?ooo`3oool000001`3oool5000000@0oooo0`00003o0?ooo`P0oooo00020?ooo`040000
003oool0oooo000000L0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3o0?oo
o`P0oooo00020?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`0000001@3oool00`00
0000oooo0?ooo`3m0?ooo`80o`00203oool00080oooo00@000000?ooo`3oool000002@3oool20000
00D0oooo00<000000?ooo`3oool0nP3oool30?l000X0oooo00030?ooo`8000002`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?P0oooo0P3o000=0?ooo`005P3oool00`000000oooo
0?ooo`3e0?ooo`<0o`003`3oool001H0oooo0P00003c0?ooo`<0o`004P3oool001H0oooo00<00000
0?ooo`3oool0k`3oool30?l001D0oooo000F0?ooo`030000003oool0oooo0>d0oooo0P3o000H0?oo
o`005P3oool00`000000oooo0?ooo`3Z0?ooo`<0o`006P3oool001H0oooo00<000000?ooo`3oool0
i`3oool30?l001d0oooo000F0?ooo`800000i@3oool30?l00200oooo000F0?ooo`030000003oool0
oooo0>80oooo0P3o000S0?ooo`005P3oool00`000000oooo0?ooo`3O0?ooo`<0o`009@3oool001H0
oooo00<000000?ooo`3oool0g03oool30?l002P0oooo000F0?ooo`030000003oool0oooo0=X0oooo
0P3o000[0?ooo`005P3oool200000=P0oooo0`3o000]0?ooo`005P3oool00`000000oooo0?ooo`3E
0?ooo`80o`00<03oool001H0oooo00<000000?ooo`3oool0d`3oool20?l00380oooo000F0?ooo`03
0000003oool0oooo0=00oooo0`3o000d0?ooo`005P3oool00`000000oooo0?ooo`3>0?ooo`80o`00
=`3oool001H0oooo00<000000?ooo`3oool0c03oool20?l003T0oooo000F0?ooo`800000b`3oool2
0?l003/0oooo000F0?ooo`030000003oool0oooo0<P0oooo0P3o000m0?ooo`005P3oool00`000000
oooo0?ooo`350?ooo`<0o`00?`3oool000<0oooo0P0000040?ooo`030000003oool0oooo0080oooo
0`0000050?ooo`030000003oool0oooo0<<0oooo0P3o00120?ooo`000P3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0`03oool30?l004@0oooo
00020?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool300000;h0oooo
0P3o00170?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000oooo
0?ooo`2l0?ooo`80o`00B@3oool00080oooo00@000000?ooo`3oool00000203oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo0;T0oooo0`3o001;0?ooo`000P3oool010000000oooo0?oo
o`0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0]`3oool20?l004h0oooo
00030?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0]@3oool20?l00500oooo000F
0?ooo`030000003oool0oooo0;<0oooo0P3o001B0?ooo`005P3oool200000;80oooo0P3o001D0?oo
o`005P3oool00`000000oooo0?ooo`2^0?ooo`<0o`00EP3oool001H0oooo00<000000?ooo`3oool0
[03oool20?l005T0oooo000F0?ooo`030000003oool0oooo0:X0oooo0P3o001K0?ooo`005P3oool0
0`000000oooo0?ooo`2X0?ooo`80o`00G@3oool001H0oooo0P00002W0?ooo`80o`00G`3oool001H0
oooo00<000000?ooo`3oool0X`3oool30?l00640oooo000F0?ooo`030000003oool0oooo0:40oooo
0P3o001T0?ooo`005P3oool00`000000oooo0?ooo`2O0?ooo`80o`00IP3oool001H0oooo00<00000
0?ooo`3oool0W@3oool20?l006P0oooo000F0?ooo`030000003oool0oooo09/0oooo0P3o001Z0?oo
o`005P3oool2000009T0oooo0`3o001/0?ooo`005P3oool00`000000oooo0?ooo`2F0?ooo`80o`00
K`3oool001H0oooo00<000000?ooo`3oool0U03oool20?l00740oooo000F0?ooo`030000003oool0
oooo0980oooo0P3o001c0?ooo`005P3oool00`000000oooo0?ooo`2@0?ooo`80o`00M@3oool001H0
oooo0P00002?0?ooo`80o`00M`3oool001H0oooo00<000000?ooo`3oool0S@3oool00`3o0000oooo
0?ooo`1g0?ooo`005P3oool00`000000oooo0?ooo`2;0?ooo`80o`00NP3oool001H0oooo00<00000
0?ooo`3oool0R@3oool20?l007`0oooo00030?ooo`800000103oool00`000000oooo0?ooo`030?oo
o`8000001@3oool00`000000oooo0?ooo`270?ooo`80o`00OP3oool00080oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo08D0oooo0P3o0020
0?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo0`00
00230?ooo`80o`00PP3oool00080oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<0
00000?ooo`3oool0PP3oool00`3o0000oooo0?ooo`220?ooo`000P3oool010000000oooo0?ooo`00
00080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0P03oool20?l008D0oooo0002
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`1o0?ooo`030?l0003oool0oooo08D0oooo00030?ooo`8000002P3oool3000000@0oooo00<00000
0?ooo`3oool0O@3oool20?l008P0oooo000F0?ooo`800000O03oool20?l008X0oooo000F0?ooo`03
0000003oool0oooo07X0oooo00<0o`000?ooo`3oool0RP3oool001H0oooo00<000000?ooo`3oool0
N03oool20?l008d0oooo000F0?ooo`030000003oool0oooo07L0oooo00<0o`000?ooo`3oool0S@3o
ool001H0oooo00<000000?ooo`3oool0M@3oool20?l00900oooo000F0?ooo`800000M03oool20?l0
0980oooo000F0?ooo`030000003oool0oooo0740oooo0P3o002D0?ooo`005P3oool00`000000oooo
0?ooo`1_0?ooo`80o`00UP3oool001H0oooo00<000000?ooo`3oool0K@3oool20?l009P0oooo000F
0?ooo`030000003oool0oooo06`0oooo00<0o`000?ooo`3oool0V03oool001H0oooo00<000000?oo
o`3oool0JP3oool20?l009/0oooo000F0?ooo`800000JP3oool00`3o0000oooo0?ooo`2K0?ooo`00
5P3oool00`000000oooo0?ooo`1W0?ooo`80o`00WP3oool001H0oooo00<000000?ooo`3oool0I@3o
ool20?l00:00oooo000F0?ooo`030000003oool0oooo06@0oooo00<0o`000?ooo`3oool0X03oool0
01H0oooo00<000000?ooo`3oool0HP3oool20?l00:<0oooo000F0?ooo`800000HP3oool00`3o0000
oooo0?ooo`2S0?ooo`005P3oool00`000000oooo0?ooo`1O0?ooo`80o`00YP3oool001H0oooo00<0
00000?ooo`3oool0G@3oool20?l00:P0oooo000F0?ooo`030000003oool0oooo05/0oooo0P3o002Z
0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo05X0oooo00<0o`000?ooo`3oool0ZP3oool00080oooo00@000000?ooo`3o
ool000002@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo05P0oooo0P3o002]0?oo
o`000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo0`00001F0?oo
o`80o`00[`3oool00080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo05D0oooo00<0o`000?ooo`3oool0[`3oool00080oooo00@000000?ooo`3o
ool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo05<0oooo0P3o002b0?oo
o`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0DP3oool00`3o0000oooo0?ooo`2b0?ooo`000`3oool2000000T0oooo100000040?oo
o`030000003oool0oooo0500oooo0P3o002e0?ooo`005P3oool200000500oooo00<0o`000?ooo`3o
ool0]@3oool001H0oooo00<000000?ooo`3oool0CP3oool00`3o0000oooo0?ooo`2f0?ooo`005P3o
ool00`000000oooo0?ooo`1<0?ooo`80o`00^@3oool001H0oooo00<000000?ooo`3oool0B`3oool0
0`3o0000oooo0?ooo`2i0?ooo`005P3oool00`000000oooo0?ooo`190?ooo`80o`00_03oool001H0
oooo00<000000?ooo`3oool0B03oool00`3o0000oooo0?ooo`2l0?ooo`005P3oool2000004L0oooo
0P3o002o0?ooo`005P3oool00`000000oooo0?ooo`150?ooo`030?l0003oool0oooo0;l0oooo000F
0?ooo`030000003oool0oooo04<0oooo0P3o00320?ooo`005P3oool00`000000oooo0?ooo`120?oo
o`030?l0003oool0oooo0<80oooo000F0?ooo`030000003oool0oooo0400oooo0P3o00350?ooo`00
5P3oool200000400oooo00<0o`000?ooo`3oool0a@3oool001H0oooo00<000000?ooo`3oool0?@3o
ool20?l00<P0oooo000F0?ooo`030000003oool0oooo03`0oooo00<0o`000?ooo`3oool0b03oool0
01H0oooo00<000000?ooo`3oool0>`3oool00`3o0000oooo0?ooo`390?ooo`005P3oool00`000000
oooo0?ooo`0i0?ooo`80o`00c03oool001H0oooo00<000000?ooo`3oool0>03oool00`3o0000oooo
0?ooo`3<0?ooo`005P3oool2000003P0oooo00<0o`000?ooo`3oool0c@3oool001H0oooo00<00000
0?ooo`3oool0=@3oool20?l00=00oooo000F0?ooo`030000003oool0oooo03@0oooo00<0o`000?oo
o`3oool0d03oool000<0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`03
0000003oool0oooo03<0oooo00<0o`000?ooo`3oool0d@3oool00080oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0340oooo0P3o003D0?oo
o`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo0`00000`
0?ooo`030?l0003oool0oooo0=@0oooo00020?ooo`040000003oool0oooo000000T0oooo0P000005
0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool0e@3oool00080oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02h0oooo00<0
o`000?ooo`3oool0eP3oool00080oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo02d0oooo00<0o`000?ooo`3oool0e`3oool000<0oooo0P00
000:0?ooo`8000001@3oool00`000000oooo0?ooo`0[0?ooo`80o`00fP3oool001H0oooo0P00000[
0?ooo`030?l0003oool0oooo0=X0oooo000F0?ooo`030000003oool0oooo02T0oooo00<0o`000?oo
o`3oool0f`3oool001H0oooo00<000000?ooo`3oool09`3oool20?l00=h0oooo000F0?ooo`030000
003oool0oooo02H0oooo00<0o`000?ooo`3oool0gP3oool001H0oooo00<000000?ooo`3oool09@3o
ool00`3o0000oooo0?ooo`3O0?ooo`005P3oool00`000000oooo0?ooo`0S0?ooo`80o`00hP3oool0
01H0oooo0P00000S0?ooo`030?l0003oool0oooo0>80oooo000F0?ooo`030000003oool0oooo0240
oooo00<0o`000?ooo`3oool0h`3oool001H0oooo00<000000?ooo`3oool07`3oool20?l00>H0oooo
000F0?ooo`030000003oool0oooo01h0oooo00<0o`000?ooo`3oool0iP3oool001H0oooo00<00000
0?ooo`3oool07@3oool00`3o0000oooo0?ooo`3W0?ooo`005P3oool2000001d0oooo00<0o`000?oo
o`3oool0j03oool001H0oooo00<000000?ooo`3oool06P3oool20?l00>/0oooo000F0?ooo`030000
003oool0oooo01T0oooo00<0o`000?ooo`3oool0j`3oool001H0oooo00<000000?ooo`3oool0603o
ool00`3o0000oooo0?ooo`3/0?ooo`005P3oool00`000000oooo0?ooo`0G0?ooo`030?l0003oool0
oooo0>d0oooo000F0?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool0kP3oool001H0
oooo0P00000E0?ooo`80o`00l@3oool001H0oooo00<000000?ooo`3oool04`3oool00`3o0000oooo
0?ooo`3a0?ooo`005P3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo0?80oooo0003
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`0A
0?ooo`030?l0003oool0oooo0?<0oooo00020?ooo`040000003oool0oooo000000/0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`030?l0003oool0oooo0?@0oooo00020?oo
o`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool3000000l0oooo00<0o`00
0?ooo`3oool0m@3oool00080oooo00@000000?ooo`3oool000002@3oool3000000@0oooo00<00000
0?ooo`3oool03P3oool00`3o0000oooo0?ooo`3f0?ooo`000P3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0303oool20?l00?T0oooo0002
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo
0?ooo`0;0?ooo`030?l0003oool0oooo0?T0oooo00030?ooo`8000002P3oool2000000D0oooo00<0
00000?ooo`3oool02P3oool00`3o0000oooo0?ooo`3j0?ooo`005P3oool00`000000oooo0?ooo`09
0?ooo`030?l0003oool0oooo0?/0oooo000F0?ooo`8000002@3oool00`3o0000oooo0?ooo`3l0?oo
o`005P3oool00`000000oooo0?ooo`070?ooo`030?l0003oool0oooo0?d0oooo000F0?ooo`030000
003oool0oooo00H0oooo00<0o`000?ooo`3oool0oP3oool001H0oooo00<000000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`3o0?ooo`005P3oool00`000000oooo0?ooo`040?ooo`030?l0003oool0
oooo0?l0oooo0@3oool001H0oooo0P0000040?ooo`030?l0003oool0oooo0?l0oooo0P3oool001H0
oooo00@000000?ooo`3oool0oooo0P3o003o0?ooo`D0oooo000F0?ooo`040000003oool0oooo0?l0
0?l0oooo1`3oool001H0oooo00<000000?ooo`3o0000o`3oool80?ooo`005P3oool00`000000o`00
0?ooo`3o0?ooo`P0oooo000F0?ooo`030?l0003oool0oooo0?l0oooo203oool001D0oooo00<0o`00
000000000000?03oool3000003X0oooo100000030?ooo`800000>03oool400000080oooo0`00000g
0?ooo`@000000`3oool2000000D0oooo000D0?ooo`030?l0003oool000000400oooo00<000000?oo
o`3oool0>@3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0=`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool00`3oool0
10000000oooo0?ooo`0000040?ooo`004`3oool0103o0000oooo0?ooo`0000100?ooo`030000003o
ool0oooo03T0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo03L0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo0080oooo
00@000000?ooo`3oool00000103oool00180oooo00D0o`000?ooo`3oool0oooo0000000m0?ooo`<0
0000?03oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0=`3oool01000
0000oooo0?ooo`3oool3000003T0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003o
ool0oooo0080oooo000B0?ooo`050?l0003oool0oooo0?ooo`000000?@3oool00`000000oooo0?oo
o`0l0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`0g0?ooo`050000
003oool0oooo0?ooo`000000?03oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
0080oooo000A0?ooo`030?l0003oool0oooo0080oooo0P00000l0?ooo`030000003oool0oooo03X0
oooo0`0000030?ooo`040000003oool0oooo000003L0oooo0`0000030?ooo`030000003oool0oooo
03L0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`00403oool0
0`3o0000oooo0?ooo`030?ooo`030000003oool0oooo03/0oooo1000000k0?ooo`030000003oool0
oooo0080oooo0P00000j0?ooo`040000003oool0oooo0?ooo`@00000=`3oool2000000@0oooo0P00
00050?ooo`003`3oool00`3o0000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool0
00l0oooo00<0o`000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo000>0?ooo`03
0?l0003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool80?ooo`000P3oool;000000030?l0
0000000000000?l000003`0000020?ooo`002@3oool010000000oooo0?ooo`3o00090?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?ooo`002`3o
ool00`3o0000oooo0?ooo`080?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool01P3o
ool000/0oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?ooo`3o0?ooo`P0oooo000:0?oo
o`030?l0003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool80?ooo`002@3oool00`3o0000
oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo203oool001H0oooo0P00003o0?ooo`T0oooo
000F0?ooo`030000003oool0oooo0?l0oooo203oool001H0oooo00<000000?ooo`3oool0o`3oool8
0?ooo`005P3oool00`000000oooo0?ooo`3o0?ooo`P0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.75634, 0.383467, \
0.0779083, 0.00367357}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(compare\ to\ original\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %70]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.069161 0.0453515 -0.375953 0.961805 [
[.29592 .57335 -3 -9 ]
[.29592 .57335 3 0 ]
[.52268 .57335 -6 -9 ]
[.52268 .57335 6 0 ]
[.74943 .57335 -6 -9 ]
[.74943 .57335 6 0 ]
[.97619 .57335 -6 -9 ]
[.97619 .57335 6 0 ]
[.05666 .00877 -18 -4.5 ]
[.05666 .00877 0 4.5 ]
[.05666 .10495 -18 -4.5 ]
[.05666 .10495 0 4.5 ]
[.05666 .20113 -18 -4.5 ]
[.05666 .20113 0 4.5 ]
[.05666 .29731 -18 -4.5 ]
[.05666 .29731 0 4.5 ]
[.05666 .39349 -18 -4.5 ]
[.05666 .39349 0 4.5 ]
[.05666 .48967 -18 -4.5 ]
[.05666 .48967 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.29592 .58585 m
.29592 .5921 L
s
[(5)] .29592 .57335 0 1 Mshowa
.52268 .58585 m
.52268 .5921 L
s
[(10)] .52268 .57335 0 1 Mshowa
.74943 .58585 m
.74943 .5921 L
s
[(15)] .74943 .57335 0 1 Mshowa
.97619 .58585 m
.97619 .5921 L
s
[(20)] .97619 .57335 0 1 Mshowa
.125 Mabswid
.11451 .58585 m
.11451 .5896 L
s
.15986 .58585 m
.15986 .5896 L
s
.20522 .58585 m
.20522 .5896 L
s
.25057 .58585 m
.25057 .5896 L
s
.34127 .58585 m
.34127 .5896 L
s
.38662 .58585 m
.38662 .5896 L
s
.43197 .58585 m
.43197 .5896 L
s
.47732 .58585 m
.47732 .5896 L
s
.56803 .58585 m
.56803 .5896 L
s
.61338 .58585 m
.61338 .5896 L
s
.65873 .58585 m
.65873 .5896 L
s
.70408 .58585 m
.70408 .5896 L
s
.79478 .58585 m
.79478 .5896 L
s
.84014 .58585 m
.84014 .5896 L
s
.88549 .58585 m
.88549 .5896 L
s
.93084 .58585 m
.93084 .5896 L
s
.02381 .58585 m
.02381 .5896 L
s
.25 Mabswid
0 .58585 m
1 .58585 L
s
.06916 .00877 m
.07541 .00877 L
s
[(0.4)] .05666 .00877 1 0 Mshowa
.06916 .10495 m
.07541 .10495 L
s
[(0.5)] .05666 .10495 1 0 Mshowa
.06916 .20113 m
.07541 .20113 L
s
[(0.6)] .05666 .20113 1 0 Mshowa
.06916 .29731 m
.07541 .29731 L
s
[(0.7)] .05666 .29731 1 0 Mshowa
.06916 .39349 m
.07541 .39349 L
s
[(0.8)] .05666 .39349 1 0 Mshowa
.06916 .48967 m
.07541 .48967 L
s
[(0.9)] .05666 .48967 1 0 Mshowa
.125 Mabswid
.06916 .02801 m
.07291 .02801 L
s
.06916 .04724 m
.07291 .04724 L
s
.06916 .06648 m
.07291 .06648 L
s
.06916 .08571 m
.07291 .08571 L
s
.06916 .12419 m
.07291 .12419 L
s
.06916 .14342 m
.07291 .14342 L
s
.06916 .16266 m
.07291 .16266 L
s
.06916 .18189 m
.07291 .18189 L
s
.06916 .22037 m
.07291 .22037 L
s
.06916 .2396 m
.07291 .2396 L
s
.06916 .25884 m
.07291 .25884 L
s
.06916 .27807 m
.07291 .27807 L
s
.06916 .31655 m
.07291 .31655 L
s
.06916 .33578 m
.07291 .33578 L
s
.06916 .35502 m
.07291 .35502 L
s
.06916 .37425 m
.07291 .37425 L
s
.06916 .41273 m
.07291 .41273 L
s
.06916 .43196 m
.07291 .43196 L
s
.06916 .4512 m
.07291 .4512 L
s
.06916 .47044 m
.07291 .47044 L
s
.06916 .50891 m
.07291 .50891 L
s
.06916 .52814 m
.07291 .52814 L
s
.06916 .54738 m
.07291 .54738 L
s
.06916 .56662 m
.07291 .56662 L
s
.06916 .60509 m
.07291 .60509 L
s
.25 Mabswid
.06916 0 m
.06916 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.02381 .60332 m
.06244 .55425 L
.10458 .51393 L
.14415 .47868 L
.18221 .44661 L
.22272 .41419 L
.26171 .3845 L
.30316 .35442 L
.34309 .3268 L
.3815 .30139 L
.42237 .27553 L
.46172 .25171 L
.49955 .22974 L
.53984 .2073 L
.57861 .18659 L
.61984 .16547 L
.65954 .14594 L
.69774 .12788 L
.73838 .10941 L
.77751 .09231 L
.81909 .07484 L
.85916 .05865 L
.89771 .04364 L
.93871 .02826 L
.97619 .01472 L
s
0 g
.02381 .58518 m
.06244 .56895 L
.10458 .55532 L
.14415 .54316 L
.18221 .53189 L
.22272 .52028 L
.26171 .50945 L
.30316 .49827 L
.34309 .4878 L
.3815 .47801 L
.42237 .46786 L
.46172 .45834 L
.49955 .44942 L
.53984 .44015 L
.57861 .43145 L
.61984 .42242 L
.65954 .41394 L
.69774 .40596 L
.73838 .39767 L
.77751 .38987 L
.81909 .38176 L
.85916 .37414 L
.89771 .36695 L
.93871 .35948 L
.97619 .35279 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo203oool00080oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo203oool00080oooo
00@000000?ooo`3oool000001`3oool5000000@0oooo0`00003o0?ooo`P0oooo00020?ooo`040000
003oool0oooo000000L0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3o0?oo
o`P0oooo00020?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`0000001@3oool00`00
0000oooo0?ooo`3m0?ooo`80o`00203oool00080oooo00@000000?ooo`3oool000002@3oool20000
00D0oooo00<000000?ooo`3oool0nP3oool30?l000X0oooo00030?ooo`8000002`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0?P0oooo0P3o000=0?ooo`005P3oool00`000000oooo
0?ooo`3e0?ooo`<0o`003`3oool001H0oooo0P00003c0?ooo`<0o`004P3oool001H0oooo00<00000
0?ooo`3oool0k`3oool30?l001D0oooo000F0?ooo`030000003oool0oooo0>d0oooo0P3o000H0?oo
o`005P3oool00`000000oooo0?ooo`3Z0?ooo`<0o`006P3oool001H0oooo00<000000?ooo`3oool0
i`3oool30?l001d0oooo000F0?ooo`800000i@3oool30?l00200oooo000F0?ooo`030000003oool0
oooo0>80oooo0P3o000S0?ooo`005P3oool00`000000oooo0?ooo`3O0?ooo`<0o`009@3oool001H0
oooo00<000000?ooo`3oool0g03oool30?l002P0oooo000F0?ooo`030000003oool0oooo0=X0oooo
0P3o000[0?ooo`005P3oool200000=P0oooo0`3o000]0?ooo`005P3oool00`000000oooo0?ooo`3E
0?ooo`80o`00<03oool001H0oooo00<000000?ooo`3oool0d`3oool20?l00380oooo000F0?ooo`03
0000003oool0oooo0=00oooo0`3o000d0?ooo`005P3oool00`000000oooo0?ooo`3>0?ooo`80o`00
=`3oool001H0oooo00<000000?ooo`3oool0c03oool20?l003T0oooo000F0?ooo`800000b`3oool2
0?l003/0oooo000F0?ooo`030000003oool0oooo0<P0oooo0P3o000m0?ooo`005P3oool00`000000
oooo0?ooo`350?ooo`<0o`00?`3oool000<0oooo0P0000040?ooo`030000003oool0oooo0080oooo
0`0000050?ooo`030000003oool0oooo0<<0oooo0P3o00120?ooo`000P3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0`03oool30?l004@0oooo
00020?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool300000;h0oooo
0P3o00170?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000oooo
0?ooo`2l0?ooo`80o`00B@3oool00080oooo00@000000?ooo`3oool00000203oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo0;T0oooo0`3o001;0?ooo`000P3oool010000000oooo0?oo
o`0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0]`3oool20?l004h0oooo
00030?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0]@3oool20?l00500oooo000F
0?ooo`030000003oool0oooo0;<0oooo0P3o001B0?ooo`005P3oool200000;80oooo0P3o001D0?oo
o`005P3oool00`000000oooo0?ooo`2^0?ooo`<0o`00EP3oool001H0oooo00<000000?ooo`3oool0
[03oool20?l005T0oooo000F0?ooo`030000003oool0oooo0:X0oooo0P3o001K0?ooo`005P3oool0
0`000000oooo0?ooo`2X0?ooo`80o`00G@3oool001H0oooo0P00002W0?ooo`80o`00G`3oool001H0
oooo00<000000?ooo`3oool0X`3oool30?l00640oooo000F0?ooo`030000003oool0oooo0:40oooo
0P3o001T0?ooo`005P3oool00`000000oooo0?ooo`2O0?ooo`80o`00IP3oool001H0oooo00<00000
0?ooo`3oool0W@3oool20?l006P0oooo000F0?ooo`030000003oool0oooo09/0oooo0P3o001Z0?oo
o`005P3oool2000009T0oooo0`3o001/0?ooo`005P3oool00`000000oooo0?ooo`2F0?ooo`80o`00
K`3oool001H0oooo00<000000?ooo`3oool0U03oool20?l00740oooo000F0?ooo`030000003oool0
oooo0980oooo0P3o001c0?ooo`005P3oool00`000000oooo0?ooo`2@0?ooo`80o`00M@3oool001H0
oooo0P00002?0?ooo`80o`00M`3oool001H0oooo00<000000?ooo`3oool0S@3oool00`3o0000oooo
0?ooo`1g0?ooo`005P3oool00`000000oooo0?ooo`2;0?ooo`80o`00NP3oool001H0oooo00<00000
0?ooo`3oool0R@3oool20?l007`0oooo00030?ooo`800000103oool00`000000oooo0?ooo`030?oo
o`8000001@3oool00`000000oooo0?ooo`270?ooo`80o`00OP3oool00080oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo08D0oooo0P3o0020
0?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo0`00
00230?ooo`80o`00PP3oool00080oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<0
00000?ooo`3oool0PP3oool00`3o0000oooo0?ooo`220?ooo`000P3oool010000000oooo0?ooo`00
00080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0P03oool20?l008D0oooo0002
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`1o0?ooo`030?l0003oool0oooo08D0oooo00030?ooo`8000002P3oool3000000@0oooo00<00000
0?ooo`3oool0O@3oool20?l008P0oooo000F0?ooo`800000O03oool20?l008X0oooo000F0?ooo`03
0000003oool0oooo07X0oooo00<0o`000?ooo`3oool0RP3oool001H0oooo00<000000?ooo`3oool0
N03oool20?l008d0oooo000F0?ooo`030000003oool0oooo07L0oooo00<0o`000?ooo`3oool0S@3o
ool001H0oooo00<000000?ooo`3oool0M@3oool20?l00900oooo000F0?ooo`800000M03oool20?l0
0980oooo000F0?ooo`030000003oool0oooo0740oooo0P3o002D0?ooo`005P3oool00`000000oooo
0?ooo`1_0?ooo`80o`00UP3oool001H0oooo00<000000?ooo`3oool0K@3oool20?l009P0oooo000F
0?ooo`030000003oool0oooo06`0oooo00<0o`000?ooo`3oool0V03oool001H0oooo00<000000?oo
o`3oool0JP3oool20?l009/0oooo000F0?ooo`800000JP3oool00`3o0000oooo0?ooo`2K0?ooo`00
5P3oool00`000000oooo0?ooo`1W0?ooo`80o`00WP3oool001H0oooo00<000000?ooo`3oool0I@3o
ool20?l00:00oooo000F0?ooo`030000003oool0oooo06@0oooo00<0o`000?ooo`3oool0X03oool0
01H0oooo00<000000?ooo`3oool0HP3oool20?l00:<0oooo000F0?ooo`800000HP3oool00`3o0000
oooo0?ooo`2S0?ooo`005P3oool00`000000oooo0?ooo`1O0?ooo`80o`00YP3oool001H0oooo00<0
00000?ooo`3oool0G@3oool20?l00:P0oooo000F0?ooo`030000003oool0oooo05/0oooo0P3o002Z
0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo05X0oooo00<0o`000?ooo`3oool0ZP3oool00080oooo00@000000?ooo`3o
ool000002@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo05P0oooo0P3o002]0?oo
o`000P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo0`00001F0?oo
o`80o`00[`3oool00080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo05D0oooo00<0o`000?ooo`3oool0[`3oool00080oooo00@000000?ooo`3o
ool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo05<0oooo0P3o002b0?oo
o`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0DP3oool00`3o0000oooo0?ooo`2b0?ooo`000`3oool2000000T0oooo100000040?oo
o`030000003oool0oooo0500oooo0P3o002e0?ooo`005P3oool200000500oooo00<0o`000?ooo`3o
ool0]@3oool001H0oooo00<000000?ooo`3oool0CP3oool00`3o0000oooo0?ooo`2f0?ooo`005P3o
ool00`000000oooo0?ooo`1<0?ooo`80o`00^@3oool001H0oooo00<000000?ooo`3oool0B`3oool0
0`3o0000oooo0?ooo`2i0?ooo`005P3oool00`000000oooo0?ooo`190?ooo`80o`00_03oool001H0
oooo00<000000?ooo`3oool0B03oool00`3o0000oooo0?ooo`2l0?ooo`005P3oool2000004L0oooo
0P3o002o0?ooo`005P3oool00`000000oooo0?ooo`150?ooo`030?l0003oool0oooo0;l0oooo000F
0?ooo`030000003oool0oooo04<0oooo0P3o00320?ooo`005P3oool00`000000oooo0?ooo`120?oo
o`030?l0003oool0oooo0<80oooo000F0?ooo`030000003oool0oooo0400oooo0P3o002j0?ooo`<0
0000203oool001H0oooo0P0000100?ooo`030?l0003oool0oooo0;@0oooo1P00000;0?ooo`005P3o
ool00`000000oooo0?ooo`0m0?ooo`80o`00/P3oool500000140oooo000F0?ooo`030000003oool0
oooo03`0oooo00<0o`000?ooo`3oool0[03oool6000001H0oooo000F0?ooo`030000003oool0oooo
03/0oooo00<0o`000?ooo`3oool0Z03oool5000001`0oooo000F0?ooo`030000003oool0oooo03T0
oooo0P3o002U0?ooo`H000008@3oool001H0oooo00<000000?ooo`3oool0>03oool00`3o0000oooo
0?ooo`2P0?ooo`D000009`3oool001H0oooo0P00000h0?ooo`030?l0003oool0oooo09d0oooo1000
000/0?ooo`005P3oool00`000000oooo0?ooo`0e0?ooo`80o`00W03oool400000300oooo000F0?oo
o`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool0V03oool4000003@0oooo00030?ooo`80
0000103oool00`000000oooo0?ooo`030?ooo`8000001@3oool00`000000oooo0?ooo`0c0?ooo`03
0?l0003oool0oooo09<0oooo1P00000h0?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0<@3oool20?l00900oooo1P00000n0?oo
o`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo0`00000`
0?ooo`030?l0003oool0oooo08X0oooo1P0000140?ooo`000P3oool010000000oooo0?ooo`000009
0?ooo`8000001@3oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0oooo08H0oooo1@00001:
0?ooo`000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0;P3oool00`3o0000oooo0?ooo`210?ooo`H00000C`3oool00080oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02d0oooo
00<0o`000?ooo`3oool0OP3oool4000005D0oooo00030?ooo`8000002P3oool2000000D0oooo00<0
00000?ooo`3oool0:`3oool20?l007d0oooo1000001I0?ooo`005P3oool2000002/0oooo00<0o`00
0?ooo`3oool0N@3oool4000005d0oooo000F0?ooo`030000003oool0oooo02T0oooo00<0o`000?oo
o`3oool0MP3oool400000640oooo000F0?ooo`030000003oool0oooo02L0oooo0P3o001c0?ooo`H0
0000I@3oool001H0oooo00<000000?ooo`3oool09P3oool00`3o0000oooo0?ooo`1^0?ooo`D00000
J`3oool001H0oooo00<000000?ooo`3oool09@3oool00`3o0000oooo0?ooo`1[0?ooo`@00000L03o
ool001H0oooo00<000000?ooo`3oool08`3oool20?l006X0oooo1000001d0?ooo`005P3oool20000
02<0oooo00<0o`000?ooo`3oool0IP3oool4000007P0oooo000F0?ooo`030000003oool0oooo0240
oooo00<0o`000?ooo`3oool0H@3oool6000007`0oooo000F0?ooo`030000003oool0oooo01l0oooo
0P3o001P0?ooo`@00000PP3oool001H0oooo00<000000?ooo`3oool07P3oool00`3o0000oooo0?oo
o`1L0?ooo`@00000QP3oool001H0oooo00<000000?ooo`3oool07@3oool00`3o0000oooo0?ooo`1I
0?ooo`@00000RP3oool001H0oooo0P00000M0?ooo`030?l0003oool0oooo05H0oooo1000002>0?oo
o`005P3oool00`000000oooo0?ooo`0J0?ooo`80o`00D`3oool600000980oooo000F0?ooo`030000
003oool0oooo01T0oooo00<0o`000?ooo`3oool0C`3oool4000009P0oooo000F0?ooo`030000003o
ool0oooo01P0oooo00<0o`000?ooo`3oool0C03oool4000009`0oooo000F0?ooo`030000003oool0
oooo01L0oooo00<0o`000?ooo`3oool0B@3oool400000:00oooo000F0?ooo`030000003oool0oooo
01H0oooo00<0o`000?ooo`3oool0AP3oool400000:@0oooo000F0?ooo`8000005@3oool20?l004D0
oooo1000002X0?ooo`005P3oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo0440oooo
1000002/0?ooo`005P3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo03l0oooo0`00
002`0?ooo`000`3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<00000
0?ooo`3oool04@3oool00`3o0000oooo0?ooo`0l0?ooo`@00000/`3oool00080oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo00<0o`00
0?ooo`3oool0>P3oool300000;L0oooo00020?ooo`040000003oool0oooo000000/0oooo00<00000
0?ooo`3oool00P3oool3000000l0oooo00<0o`000?ooo`3oool0=`3oool400000;X0oooo00020?oo
o`040000003oool0oooo000000T0oooo0`0000040?ooo`030000003oool0oooo00h0oooo00<0o`00
0?ooo`3oool0=03oool400000;h0oooo00020?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0<0?ooo`80o`00<`3oool400000<80oooo0002
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo
0?ooo`0;0?ooo`030?l0003oool0oooo0300oooo0`0000360?ooo`000`3oool2000000X0oooo0P00
00050?ooo`030000003oool0oooo00X0oooo00<0o`000?ooo`3oool0;@3oool400000<T0oooo000F
0?ooo`030000003oool0oooo00T0oooo00<0o`000?ooo`3oool0:P3oool400000<d0oooo000F0?oo
o`8000002@3oool00`3o0000oooo0?ooo`0X0?ooo`<00000d@3oool001H0oooo00<000000?ooo`3o
ool01`3oool00`3o0000oooo0?ooo`0U0?ooo`@00000e03oool001H0oooo00<000000?ooo`3oool0
1P3oool00`3o0000oooo0?ooo`0R0?ooo`@00000f03oool001H0oooo00<000000?ooo`3oool01@3o
ool00`3o0000oooo0?ooo`0O0?ooo`@00000g03oool001H0oooo00<000000?ooo`3oool0103oool0
0`3o0000oooo0?ooo`0L0?ooo`@00000h03oool001H0oooo0P0000040?ooo`030?l0003oool0oooo
01T0oooo1000003T0?ooo`005P3oool010000000oooo0?ooo`3oool20?l001T0oooo0`00003X0?oo
o`005P3oool010000000oooo0?ooo`3o000G0?ooo`@00000j`3oool001H0oooo00<000000?ooo`3o
0000503oool400000>l0oooo000F0?ooo`030000003o0000oooo0140oooo0`00003c0?ooo`005P3o
ool00`3o0000oooo0?ooo`0>0?ooo`<00000mP3oool001D0oooo00<0o`000000000000003@3oool2
000002d0oooo0`00000j0?ooo`@000000`3oool2000003P0oooo100000020?ooo`<00000=`3oool4
000000<0oooo0P0000050?ooo`00503oool00`3o0000oooo0000000;0?ooo`<00000<P3oool00`00
0000oooo0?ooo`0i0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`0g
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`03
0?ooo`040000003oool0oooo000000@0oooo000C0?ooo`040?l0003oool0oooo000000P0oooo0`00
000e0?ooo`030000003oool0oooo03T0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000
003oool0oooo03L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0e0?ooo`030000
003oool0oooo0080oooo00@000000?ooo`3oool00000103oool00180oooo00D0o`000?ooo`3oool0
oooo000000050?ooo`<00000=@3oool3000003`0oooo00D000000?ooo`3oool0oooo000000020?oo
o`030000003oool0oooo03L0oooo00@000000?ooo`3oool0oooo0`00000i0?ooo`050000003oool0
oooo0?ooo`0000000P3oool00`000000oooo0?ooo`020?ooo`004P3oool01@3o0000oooo0?ooo`3o
ool000000080oooo0`00000h0?ooo`030000003oool0oooo03`0oooo00D000000?ooo`3oool0oooo
000000020?ooo`030000003oool0oooo03L0oooo00D000000?ooo`3oool0oooo0000000l0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00140oooo00<0o`000?ooo`3o
ool00P3oool3000003/0oooo00<000000?ooo`3oool0>P3oool3000000<0oooo00@000000?ooo`3o
ool00000=`3oool3000000<0oooo00<000000?ooo`3oool0=`3oool010000000oooo0?ooo`000002
0?ooo`040000003oool0oooo000000@0oooo000@0?ooo`030?l0003oool0oooo00@00000?@3oool4
000003/0oooo00<000000?ooo`3oool00P3oool2000003X0oooo00@000000?ooo`3oool0oooo1000
000g0?ooo`800000103oool2000000D0oooo000?0?ooo`040?l0000000000000000000<0oooo00<0
00000?ooo`3oool0o`3oool80?ooo`003P3oool2000000H0oooo00<000000?ooo`3oool0o`3oool8
0?ooo`002`3oool3000000030?l0003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool80?oo
o`000P3oool;000000030?l00000000000000?l000003`0000020?ooo`002@3oool010000000oooo
0?ooo`3o00090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`060?ooo`002`3oool00`3o0000oooo0?ooo`080?ooo`030000003oool0oooo03d0oooo
00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03d0oooo
00<000000?ooo`3oool01P3oool000/0oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?oo
o`3o0?ooo`P0oooo000:0?ooo`030?l0003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool8
0?ooo`002@3oool00`3o0000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo203oool001H0
oooo0P00003o0?ooo`T0oooo000F0?ooo`030000003oool0oooo0?l0oooo203oool001H0oooo00<0
00000?ooo`3oool0o`3oool80?ooo`005P3oool00`000000oooo0?ooo`3o0?ooo`P0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.75634, 0.383467, \
0.0779083, 0.00367357}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(lowers\ much\ further\)], "Input"],

Cell[BoxData[
    \(Does\ it\ get\ \(\(negative\)\(?\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-\((alomz[1, 0.9, 1, a] + gamoz[1, 0.9, 1, a])\)\), {a, \(-1\), 
        10}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.11039 0.0865801 0.531406 0.768442 [
[.28355 .51891 -3 -9 ]
[.28355 .51891 3 0 ]
[.45671 .51891 -3 -9 ]
[.45671 .51891 3 0 ]
[.62987 .51891 -3 -9 ]
[.62987 .51891 3 0 ]
[.80303 .51891 -3 -9 ]
[.80303 .51891 3 0 ]
[.97619 .51891 -6 -9 ]
[.97619 .51891 6 0 ]
[.09789 .07034 -24 -4.5 ]
[.09789 .07034 0 4.5 ]
[.09789 .22403 -24 -4.5 ]
[.09789 .22403 0 4.5 ]
[.09789 .37772 -24 -4.5 ]
[.09789 .37772 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.28355 .53141 m
.28355 .53766 L
s
[(2)] .28355 .51891 0 1 Mshowa
.45671 .53141 m
.45671 .53766 L
s
[(4)] .45671 .51891 0 1 Mshowa
.62987 .53141 m
.62987 .53766 L
s
[(6)] .62987 .51891 0 1 Mshowa
.80303 .53141 m
.80303 .53766 L
s
[(8)] .80303 .51891 0 1 Mshowa
.97619 .53141 m
.97619 .53766 L
s
[(10)] .97619 .51891 0 1 Mshowa
.125 Mabswid
.15368 .53141 m
.15368 .53516 L
s
.19697 .53141 m
.19697 .53516 L
s
.24026 .53141 m
.24026 .53516 L
s
.32684 .53141 m
.32684 .53516 L
s
.37013 .53141 m
.37013 .53516 L
s
.41342 .53141 m
.41342 .53516 L
s
.5 .53141 m
.5 .53516 L
s
.54329 .53141 m
.54329 .53516 L
s
.58658 .53141 m
.58658 .53516 L
s
.67316 .53141 m
.67316 .53516 L
s
.71645 .53141 m
.71645 .53516 L
s
.75974 .53141 m
.75974 .53516 L
s
.84632 .53141 m
.84632 .53516 L
s
.88961 .53141 m
.88961 .53516 L
s
.9329 .53141 m
.9329 .53516 L
s
.0671 .53141 m
.0671 .53516 L
s
.02381 .53141 m
.02381 .53516 L
s
.25 Mabswid
0 .53141 m
1 .53141 L
s
.11039 .07034 m
.11664 .07034 L
s
[(-0.6)] .09789 .07034 1 0 Mshowa
.11039 .22403 m
.11664 .22403 L
s
[(-0.4)] .09789 .22403 1 0 Mshowa
.11039 .37772 m
.11664 .37772 L
s
[(-0.2)] .09789 .37772 1 0 Mshowa
.125 Mabswid
.11039 .10876 m
.11414 .10876 L
s
.11039 .14719 m
.11414 .14719 L
s
.11039 .18561 m
.11414 .18561 L
s
.11039 .26245 m
.11414 .26245 L
s
.11039 .30087 m
.11414 .30087 L
s
.11039 .3393 m
.11414 .3393 L
s
.11039 .41614 m
.11414 .41614 L
s
.11039 .45456 m
.11414 .45456 L
s
.11039 .49298 m
.11414 .49298 L
s
.11039 .03192 m
.11414 .03192 L
s
.11039 .56983 m
.11414 .56983 L
s
.11039 .60825 m
.11414 .60825 L
s
.25 Mabswid
.11039 0 m
.11039 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.0327 0 m
.03279 .00277 L
.03527 .06473 L
.0379 .1173 L
.04262 .19067 L
.04749 .24865 L
.05205 .2924 L
.06244 .36781 L
.07305 .42288 L
.08274 .46099 L
.09394 .49497 L
.10458 .52004 L
.11417 .53805 L
.12439 .55353 L
.14277 .57399 L
.15306 .58225 L
.16418 .58918 L
.17469 .59414 L
.18435 .59756 L
.18905 .59889 L
.19409 .60009 L
.19844 .60095 L
.20318 .60171 L
.20873 .60241 L
.21128 .60265 L
.21399 .60287 L
.21642 .60303 L
.21771 .6031 L
.21911 .60316 L
.22038 .60321 L
.22155 .60325 L
.22267 .60327 L
.22384 .6033 L
.22508 .60331 L
.22577 .60332 L
.22641 .60332 L
.22759 .60332 L
.22884 .60331 L
.22996 .60329 L
.23101 .60328 L
.23215 .60325 L
.23339 .60321 L
.23612 .60311 L
.23872 .60298 L
.24358 .60267 L
.25237 .60186 L
.26195 .60068 L
.28166 .59744 L
.30265 .59304 L
.34428 .58245 L
Mistroke
.3844 .57093 L
.423 .55936 L
.46406 .547 L
.5036 .53532 L
.54162 .52443 L
.5821 .51332 L
.62106 .50313 L
.66247 .49286 L
.70237 .4835 L
.74075 .475 L
.78159 .46648 L
.82091 .45874 L
.85871 .45173 L
.89896 .44469 L
.9377 .4383 L
.97619 .43232 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003o2@3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0?h0oooo00090?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool0oP3oool000T0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`3n0?ooo`002@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0?h0oooo
00090?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0oP3oool000T0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`3n0?ooo`002@3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0?h0oooo00090?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0oP3oool000T0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3n0?ooo`002@3o
ool00`000000oooo0?ooo`0C0?ooo`<00000oP3oool000X0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`3n0?ooo`002P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
0?h0oooo000:0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0oP3oool000X0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3n0?ooo`002P3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo0?h0oooo000:0?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool0oP3oool000X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3n0?oo
o`002P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0?h0oooo000:0?ooo`040000
003oool00000000000@0oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3o
ool0oP3oool000X0oooo0P0000020?ooo`030000003oool0oooo00H0oooo00@000000?ooo`3oool0
0000103oool00`000000oooo0?ooo`3n0?ooo`002P3oool200000080oooo00<000000?ooo`3oool0
1P3oool010000000oooo0?ooo`0000040?ooo`<00000oP3oool000D0oooo100000000`3oool00000
000000020?ooo`030000003oool0oooo00H0oooo0`0000050?ooo`030000003oool0oooo0?h0oooo
000:0?ooo`8000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0oP3oool000X0oooo0P0000020?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`3n0?ooo`002P3oool010000000oooo00000000000:
0?ooo`<00000103oool00`000000oooo0?ooo`3n0?ooo`002P3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo0?h0oooo000;0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool0oP3oool000/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3n0?ooo`002`3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0?h0oooo000;0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool0oP3oool000/0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`3n0?ooo`002`3oool00`000000oooo0?ooo`0A0?ooo`<00000oP3oool000/0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3n0?ooo`002`3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo0?h0oooo000;0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool0oP3oool000/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3n0?oo
o`002`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0?h0oooo000;0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool0oP3oool000/0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`3n0?ooo`002`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0?h0oooo000;0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0oP3oool000/0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3n0?ooo`002`3oool00`000000oooo
0?ooo`0A0?ooo`<00000oP3oool000/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0?h0oooo000<0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0oP3oool000`0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0?h0oooo000<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0oP3o
ool000`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3n0?ooo`00303oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo0?h0oooo000<0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0oP3oool000`0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?ooo`<00000oP3oool000`0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0?h0oooo000<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0oP3oool000`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3n0?ooo`00303o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0?h0oooo000<0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0oP3oool000`0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`3n0?ooo`00303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0?h0
oooo000<0?ooo`800000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0oP3oool000/0oooo00@000000?ooo`00000000002P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0?h0oooo000;0?ooo`040000003oool00000000000L0oooo
1@0000040?ooo`<00000oP3oool000D0oooo100000020?ooo`040000003oool00000000000L0oooo
00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3n0?ooo`002`3oool010000000oooo
0000000000080?ooo`030000003oool0000000D0oooo00<000000?ooo`3oool0oP3oool000/0oooo
00@000000?ooo`00000000002@3oool2000000D0oooo00<000000?ooo`3oool0oP3oool000`0oooo
0P00000;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0oP3oool000d0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`3n0?ooo`003@3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo0?h0oooo000=0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0oP3oool000d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3n0?ooo`00
3@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0?h0oooo000=0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0oP3oool000h0oooo00<000000?ooo`3oool03P3oool3
00000?h0oooo000>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0oP3oool000h0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3n0?ooo`003P3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo0?h0oooo000>0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0oP3oool000h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3n
0?ooo`003`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?h0oooo000?0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool0oP3oool000l0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`3n0?ooo`003`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo0?h0oooo000?0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0oP3oool0
00l0oooo00<000000?ooo`3oool03@3oool300000?h0oooo000@0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0oP3oool00100oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`3n0?ooo`00403oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?h0oooo000@
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0oP3oool00100oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`3n0?ooo`00403oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo0?h0oooo000@0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
oP3oool00100oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3n0?ooo`004@3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?h0oooo000A0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0oP3oool00140oooo00<000000?ooo`3oool02`3oool300000?h0
oooo000A0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0oP3oool00140oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`3n0?ooo`004@3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0?h0oooo000A0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0oP3oool00180oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3n0?ooo`00
4P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?h0oooo000B0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0oP3oool00180oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`3n0?ooo`00303oool2000000@0oooo00<000000?ooo`3oool00P3oool40000
00@0oooo00<000000?ooo`3oool0oP3oool000/0oooo00@000000?ooo`3oool000000`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0oP3oool000/0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00@0oooo0`00003n0?ooo`001@3oool400000080oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0oP3oool000/0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0oP3oool000/0oooo00@000000?ooo`3oool00000103oool01@00
0000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3n
0?ooo`00303oool2000000D0oooo00<000000?ooo`3oool00P3oool2000000D0oooo00<000000?oo
o`3oool0oP3oool001<0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3n0?ooo`00
503oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?h0oooo000D0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0oP3oool001@0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`3n0?ooo`00503oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
0?h0oooo000D0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0oP3oool001D0oooo
00<000000?ooo`3oool01`3oool300000?h0oooo000E0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0oP3oool001D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3n
0?ooo`005@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?h0oooo000F0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0l`3oool3000000P0oooo000F0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0k@3oool6000000/0oooo000F0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool0j03oool500000140oooo000F0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool0hP3oool6000001H0oooo000G0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0g@3oool5000001`0oooo000G0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0e`3oool600000240oooo000G0?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0dP3oool5000002L0oooo000G0?ooo`030000003oool0oooo00D0oooo0`00
003<0?ooo`H00000;03oool001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`37
0?ooo`D00000<P3oool001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`310?oo
o`H00000=`3oool001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2l0?ooo`D0
0000?@3oool001T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2h0?ooo`@00000
@P3oool001T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2d0?ooo`@00000AP3o
ool001T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2`0?ooo`@00000BP3oool0
01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2Z0?ooo`H00000CP3oool001X0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2V0?ooo`@00000E03oool001X0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2R0?ooo`@00000F03oool001/0oooo00D0
00000?ooo`3oool0oooo0000002P0?ooo`@00000G03oool001/0oooo00@000000?ooo`3oool0oooo
0`00000]0?ooo`@00000<03oool00`000000oooo0?ooo`0]0?ooo`8000001`3oool4000002D0oooo
0P00000[0?ooo`@000000`3oool2000000D0oooo000K0?ooo`050000003oool0oooo0?ooo`000000
;`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo02`0oooo00@000000?ooo`3oool0
00000P3oool4000002P0oooo00@000000?ooo`3oool00000;03oool01@000000oooo0?ooo`3oool0
00000080oooo00<000000?ooo`3oool00P3oool001`0oooo00@000000?ooo`3oool00000<03oool0
0`000000oooo0?ooo`0]0?ooo`D00000;@3oool00`000000oooo00000003000002`0oooo00@00000
0?ooo`3oool00000;03oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0
0P3oool001`0oooo00@000000?ooo`3oool00000<@3oool00`000000oooo0?ooo`0/0?ooo`040000
003oool0oooo000002d0oooo1000000`0?ooo`800000;@3oool01@000000oooo0?ooo`3oool00000
0080oooo00<000000?ooo`3oool00P3oool001d0oooo00<000000?ooo`000000<P3oool00`000000
oooo0?ooo`0/0?ooo`030000003oool0000002T0oooo100000000`3oool000000?ooo`0`0?ooo`04
0000003oool0oooo000002`0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0
oooo0080oooo000M0?ooo`030000003oool0000002l0oooo00@000000?ooo`3oool00000;`3oool2
000002D0oooo100000050?ooo`030000003oool0oooo02l0oooo00@000000?ooo`3oool00000:P3o
ool3000000<0oooo00@000000?ooo`3oool00000103oool001h0oooo0P00000`0?ooo`800000<@3o
ool00`000000oooo0?ooo`0O0?ooo`@000002P3oool3000002l0oooo0P00000]0?ooo`030000003o
ool0oooo0080oooo0P0000050?ooo`007P3oool200000840oooo1000001k0?ooo`007P3oool20000
07d0oooo1000001o0?ooo`007`3oool00`000000oooo0?ooo`1h0?ooo`<00000P`3oool001l0oooo
00<000000?ooo`3oool0M03oool4000008H0oooo003o0000020000000@3oool000L0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`8000002`3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00T0oooo0`0000090?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool01P3oool001l0oooo00<000000?ooo`000000J@3oool400000940oooo000O0?ooo`03
0000003oool0000006D0oooo1000002E0?ooo`007`3oool010000000oooo0?ooo`00001Q0?ooo`<0
0000V@3oool001l0oooo00@000000?ooo`3oool00000GP3oool3000009`0oooo000O0?ooo`050000
003oool0oooo0?ooo`000000FP3oool3000009l0oooo000O0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0E03oool300000:80oooo000O0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0D03oool300000:D0oooo000O0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0B`3oool400000:P0oooo000O0?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool0AP3oool400000:`0oooo000O0?ooo`<000001P3oool00`000000oooo0?ooo`110?ooo`@0
0000/03oool001l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0l0?ooo`@00000
]03oool001l0oooo00<000000?ooo`3oool0203oool2000003P0oooo1000002h0?ooo`007`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0380oooo0`00002l0?ooo`007`3oool00`00
0000oooo0?ooo`0;0?ooo`800000;P3oool400000;l0oooo000O0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool09`3oool400000<<0oooo000O0?ooo`030000003oool0oooo00h0oooo
0P00000T0?ooo`<00000a`3oool001l0oooo00<000000?ooo`3oool0403oool2000001h0oooo1000
003:0?ooo`007`3oool00`000000oooo0?ooo`0B0?ooo`@000005P3oool400000<h0oooo000O0?oo
o`030000003oool0oooo01H0oooo5P00003B0?ooo`007`3oool00`000000oooo0?ooo`3n0?ooo`00
7`3oool300000?h0oooo000O0?ooo`030000003oool0oooo0?h0oooo000O0?ooo`030000003oool0
oooo0?h0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.28933, -0.691541, \
0.0403437, 0.00454551}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(\(YES!\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-\((alomz[1, 0.9, 1, a])\)\), {a, \(-1\), 1000}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0247609 0.00095143 0.272727 0.827694 [
[.21505 .26023 -9 -9 ]
[.21505 .26023 9 0 ]
[.40533 .26023 -9 -9 ]
[.40533 .26023 9 0 ]
[.59562 .26023 -9 -9 ]
[.59562 .26023 9 0 ]
[.7859 .26023 -9 -9 ]
[.7859 .26023 9 0 ]
[.97619 .26023 -12 -9 ]
[.97619 .26023 12 0 ]
[.01226 .02442 -24 -4.5 ]
[.01226 .02442 0 4.5 ]
[.01226 .10719 -24 -4.5 ]
[.01226 .10719 0 4.5 ]
[.01226 .18996 -24 -4.5 ]
[.01226 .18996 0 4.5 ]
[.01226 .3555 -18 -4.5 ]
[.01226 .3555 0 4.5 ]
[.01226 .43827 -18 -4.5 ]
[.01226 .43827 0 4.5 ]
[.01226 .52103 -18 -4.5 ]
[.01226 .52103 0 4.5 ]
[.01226 .6038 -18 -4.5 ]
[.01226 .6038 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21505 .27273 m
.21505 .27898 L
s
[(200)] .21505 .26023 0 1 Mshowa
.40533 .27273 m
.40533 .27898 L
s
[(400)] .40533 .26023 0 1 Mshowa
.59562 .27273 m
.59562 .27898 L
s
[(600)] .59562 .26023 0 1 Mshowa
.7859 .27273 m
.7859 .27898 L
s
[(800)] .7859 .26023 0 1 Mshowa
.97619 .27273 m
.97619 .27898 L
s
[(1000)] .97619 .26023 0 1 Mshowa
.125 Mabswid
.07233 .27273 m
.07233 .27648 L
s
.1199 .27273 m
.1199 .27648 L
s
.16748 .27273 m
.16748 .27648 L
s
.26262 .27273 m
.26262 .27648 L
s
.31019 .27273 m
.31019 .27648 L
s
.35776 .27273 m
.35776 .27648 L
s
.4529 .27273 m
.4529 .27648 L
s
.50048 .27273 m
.50048 .27648 L
s
.54805 .27273 m
.54805 .27648 L
s
.64319 .27273 m
.64319 .27648 L
s
.69076 .27273 m
.69076 .27648 L
s
.73833 .27273 m
.73833 .27648 L
s
.83348 .27273 m
.83348 .27648 L
s
.88105 .27273 m
.88105 .27648 L
s
.92862 .27273 m
.92862 .27648 L
s
.25 Mabswid
0 .27273 m
1 .27273 L
s
.02476 .02442 m
.03101 .02442 L
s
[(-0.3)] .01226 .02442 1 0 Mshowa
.02476 .10719 m
.03101 .10719 L
s
[(-0.2)] .01226 .10719 1 0 Mshowa
.02476 .18996 m
.03101 .18996 L
s
[(-0.1)] .01226 .18996 1 0 Mshowa
.02476 .3555 m
.03101 .3555 L
s
[(0.1)] .01226 .3555 1 0 Mshowa
.02476 .43827 m
.03101 .43827 L
s
[(0.2)] .01226 .43827 1 0 Mshowa
.02476 .52103 m
.03101 .52103 L
s
[(0.3)] .01226 .52103 1 0 Mshowa
.02476 .6038 m
.03101 .6038 L
s
[(0.4)] .01226 .6038 1 0 Mshowa
.125 Mabswid
.02476 .04097 m
.02851 .04097 L
s
.02476 .05753 m
.02851 .05753 L
s
.02476 .07408 m
.02851 .07408 L
s
.02476 .09063 m
.02851 .09063 L
s
.02476 .12374 m
.02851 .12374 L
s
.02476 .1403 m
.02851 .1403 L
s
.02476 .15685 m
.02851 .15685 L
s
.02476 .1734 m
.02851 .1734 L
s
.02476 .20651 m
.02851 .20651 L
s
.02476 .22307 m
.02851 .22307 L
s
.02476 .23962 m
.02851 .23962 L
s
.02476 .25617 m
.02851 .25617 L
s
.02476 .28928 m
.02851 .28928 L
s
.02476 .30583 m
.02851 .30583 L
s
.02476 .32239 m
.02851 .32239 L
s
.02476 .33894 m
.02851 .33894 L
s
.02476 .37205 m
.02851 .37205 L
s
.02476 .3886 m
.02851 .3886 L
s
.02476 .40516 m
.02851 .40516 L
s
.02476 .42171 m
.02851 .42171 L
s
.02476 .45482 m
.02851 .45482 L
s
.02476 .47137 m
.02851 .47137 L
s
.02476 .48793 m
.02851 .48793 L
s
.02476 .50448 m
.02851 .50448 L
s
.02476 .53759 m
.02851 .53759 L
s
.02476 .55414 m
.02851 .55414 L
s
.02476 .5707 m
.02851 .5707 L
s
.02476 .58725 m
.02851 .58725 L
s
.02476 .00786 m
.02851 .00786 L
s
.25 Mabswid
.02476 0 m
.02476 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02481 0 m
.02499 .31941 L
.02605 .44968 L
.02729 .52012 L
.02846 .55566 L
.02954 .57523 L
.03053 .5867 L
.03163 .59467 L
.03279 .59966 L
.03395 .60228 L
.0352 .60332 L
.03638 .60314 L
.03746 .60228 L
.03873 .60069 L
.04011 .59846 L
.04262 .59357 L
.06244 .55211 L
.08429 .51884 L
.10458 .49675 L
.1251 .47971 L
.14371 .46734 L
.18378 .44705 L
.22233 .43272 L
.26333 .42091 L
.30282 .41178 L
.34079 .40448 L
.38121 .39788 L
.42012 .39242 L
.46148 .38735 L
.50133 .38305 L
.53966 .37935 L
.58044 .37581 L
.61971 .37272 L
.65746 .37001 L
.69766 .36736 L
.73635 .365 L
.77749 .36269 L
.81711 .36062 L
.85522 .35876 L
.89578 .35691 L
.93483 .35524 L
.97619 .35358 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`00203oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?oo
o`3oool0o`3oool30?ooo`001`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo
0080oooo0P00003o0?ooo`@0oooo00000`3oool000000000000200000080oooo00@000000?ooo`3o
ool000002@3oool2000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`001`3oool010000000
oooo0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool30?oo
o`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0o`3oool30?ooo`00203oool2000000X0oooo0P0000050?ooo`800000o`3oool40?oo
o`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo
0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool200000?l0oooo103oool0
01/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0
oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K
0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?oo
o`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo
0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool0
00P0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo
0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool000002@3oool00`00
0000oooo0?ooo`040?ooo`800000o`3oool40?ooo`0000<0oooo0000000000000P0000020?ooo`04
0000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo00070?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`3o0?ooo`<0oooo00080?ooo`8000002P3oool20000
00D0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo
00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K
0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`03
0000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3o
ool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`800000o`3oool40?ooo`006`3oool00`00
0000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo
00<000000?ooo`3oool0o`3oool30?ooo`006`3oool200000?l0oooo103oool001/0oooo00<00000
0?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00080?ooo`80
0000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`3o0?ooo`<0
oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool00`3oool200000?l0oooo103oool000030?ooo`0000000000008000000P3oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
1`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0o`3oool30?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000
oooo0?ooo`3o0?ooo`<0oooo00080?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`800000
o`3oool40?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool200000?l0
oooo103oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo02H0oooo100000030?ooo`800000103oool2
000002D0oooo00<000000?ooo`3oool00P3oool2000000@0oooo0P00000S0?ooo`800000103oool2
000000@0oooo0P00000T0?ooo`800000103oool2000000@0oooo0P00000P0?ooo`@000000`3oool2
000000@0oooo0P0000040?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000903o
ool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`0O0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`00000R0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0
00000P3oool010000000oooo0?ooo`00000Q0?ooo`050000003oool0oooo0?ooo`0000000P3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo0@0000010?ooo`40oooo
000K0?ooo`800000:03oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool000008@3oool500000080oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`00000Q0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`00000R0?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool000000P3oool010000000oooo0?ooo`00000Q0?ooo`050000003oool0oooo0?ooo`000000
0P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo0@0000010?oo
o`40oooo000K0?ooo`030000003oool0oooo02P0oooo00D000000?ooo`3oool0oooo000000020?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool07`3oool010000000oooo0?ooo`00
00030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000008@3oool3000000<0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`00000S0?ooo`8000000`3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000240oooo00D000000?ooo`3oool0
oooo000000020?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool1
00000040oooo0@3oool001/0oooo00<000000?ooo`3oool0:@3oool010000000oooo0?ooo`000002
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0803oool00`000000oooo0000
00030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000008@3oool00`000000
oooo0?ooo`030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000008P3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0
00008@3oool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000020?ooo`4000000@3oool10?ooo`006`3oool00`000000oooo0?ooo`0V
0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`00000S0?ooo`8000000`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo
00000240oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`0000020?ooo`040000003o
ool0oooo00000280oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo000001l0oooo0`0000030?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`006`3oool2000002P0oooo
0P0000040?ooo`800000103oool2000002D0oooo00<000000?ooo`3oool00P3oool2000000@0oooo
0P00000S0?ooo`<000000`3oool2000000@0oooo0P00000T0?ooo`800000103oool2000000@0oooo
0P00000R0?ooo`030000003oool0oooo0080oooo0P0000040?ooo`800000103oool2000000<0oooo
000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000D0?ooool000001`0000060?ooo`00
6`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool001/0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0_0?ooo`030000003o
ool0oooo02h0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0_0?ooo`030000003o
ool0oooo00X0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?oo
o`@0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0
o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0oooo0?l0
oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?oo
o`3o0?ooo`<0oooo000K0?ooo`800000o`3oool40?ooo`006`3oool00`000000oooo0?ooo`3o0?oo
o`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0
o`3oool30?ooo`006`3oool200000?l0oooo103oool001/0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00080?ooo`800000103oool00`000000
oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`3o0?ooo`<0oooo00070?ooo`040000
003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3`0?ooo`H0
0000303oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`80
0000g03ooolE00000180oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`360?oooaD000009`3oool000L0oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0;40oooo5@00000l0?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000oooo0?ooo`2W0?ooo`X0
0000D@3oool000P0oooo0P00000;0?ooo`030000003oool0oooo00<0oooo0P00002D0?oooa@00000
F`3oool001/0oooo00<000000?ooo`3oool0R03oool;000006l0oooo000K0?ooo`030000003oool0
oooo07h0oooo2P00001j0?ooo`006`3oool00`000000oooo0?ooo`1d0?ooo`X00000Q03oool001/0
oooo0P00001Z0?ooo`/00000SP3oool001/0oooo00<000000?ooo`3oool0HP3oool7000009T0oooo
000K0?ooo`030000003oool0oooo05d0oooo1@00002P0?ooo`006`3oool00`000000oooo0?ooo`1E
0?ooo`P00000Y@3oool001/0oooo00<000000?ooo`3oool0C@3oool800000:d0oooo000K0?ooo`80
0000B@3oool500000;D0oooo000K0?ooo`030000003oool0oooo04@0oooo1000002j0?ooo`006`3o
ool00`000000oooo0?ooo`100?ooo`@00000_P3oool001/0oooo00<000000?ooo`3oool0?@3oool3
00000<80oooo000K0?ooo`800000>`3oool300000<D0oooo000K0?ooo`030000003oool0oooo03H0
oooo100000380?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000@0oooo
00<000000?ooo`3oool0<P3oool400000<`0oooo00070?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0_0?ooo`<00000d03oool000L0oooo00@0
00000?ooo`3oool000002@3oool00`000000oooo0?ooo`040?ooo`800000;P3oool200000=<0oooo
00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0Z0?ooo`<00000e@3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo02P0oooo0P00003H0?ooo`001`3oool010000000oooo0?oo
o`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool09@3oool300000=X0
oooo00080?ooo`8000002P3oool2000000D0oooo00<000000?ooo`3oool08`3oool200000=d0oooo
000K0?ooo`8000008P3oool200000=l0oooo000K0?ooo`030000003oool0oooo01l0oooo0P00003Q
0?ooo`006`3oool00`000000oooo0?ooo`0M0?ooo`800000h`3oool001/0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`3S0?ooo`006`3oool2000001/0oooo0P00003V0?ooo`006`3o
ool00`000000oooo0?ooo`0H0?ooo`800000j03oool001/0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`3X0?ooo`006`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
0>T0oooo000K0?ooo`8000005P3oool00`000000oooo0?ooo`3Z0?ooo`006`3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo0>/0oooo000K0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0k03oool001/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3]
0?ooo`006`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0>h0oooo000K0?ooo`80
0000403oool200000?40oooo000K0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
l@3oool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0lP3oool000L0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
l`3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?ooo`800000
303oool00`000000oooo0?ooo`3d0?ooo`001`3oool010000000oooo0?ooo`0000090?ooo`800000
1@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?@0oooo00070?ooo`040000003o
ool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo0?D0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
0000103oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?D0oooo00080?ooo`800000
2P3oool2000000D0oooo0P00000:0?ooo`030000003oool0oooo0?H0oooo000K0?ooo`8000002P3o
ool00`000000oooo0?ooo`3f0?ooo`006`3oool2000000T0oooo00<000000?ooo`3oool0m`3oool0
01/0oooo0P0000090?ooo`030000003oool0oooo0?L0oooo000K0?ooo`800000203oool00`000000
oooo0?ooo`3h0?ooo`006`3oool2000000L0oooo00<000000?ooo`3oool0n@3oool001/0oooo0P00
00070?ooo`030000003oool0oooo0?T0oooo000K0?ooo`8000001P3oool00`000000oooo0?ooo`3j
0?ooo`006`3oool2000000H0oooo00<000000?ooo`3oool0nP3oool001/0oooo0P0000050?ooo`03
0000003oool0oooo0?/0oooo000K0?ooo`8000001@3oool00`000000oooo0?ooo`3k0?ooo`006`3o
ool2000000@0oooo00<000000?ooo`3oool0o03oool001/0oooo0P0000040?ooo`030000003oool0
oooo0?`0oooo000K0?ooo`8000000`3oool00`000000oooo0?ooo`3m0?ooo`006`3oool2000000<0
oooo00<000000?ooo`3oool0o@3oool000P0oooo0P0000040?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool00`3oool01@000000oooo0000003oool000000?l0oooo0@3oool000L0oooo
00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`050000003oool000000?oo
o`000000o`3oool10?ooo`001`3oool010000000oooo0?ooo`0000070?ooo`D00000103oool40000
0?l0oooo0P3oool000L0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo000000050?ooo`030000003oool0oooo0?l0oooo0`3oool000L0oooo00@000000?oo
o`3oool000002@3oool2000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`00203oool20000
00/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo003o0?ooob40
oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40
oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-110.019, -0.364798, \
4.03645, 0.00463988}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[BoxData[
    \(\(never!\)\)], "Input"],

Cell[BoxData[
    \(\(\(=\)\(\(>\)\(\ \)\(The\ second\ mode\ has\ instabilities\ only\ for\ \
certain\ values\ of\ \(kx!\)\)\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(-\((alomz[1, Cos[Pi/4], 1, a] + 
            gamoz[1, Cos[Pi/4], 1, a])\)\), {a, 0, 200}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0047619 0.603319 3.0359 [
[.2619 .59082 -6 -9 ]
[.2619 .59082 6 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .14793 -30 -4.5 ]
[.01131 .14793 0 4.5 ]
[.01131 .29973 -24 -4.5 ]
[.01131 .29973 0 4.5 ]
[.01131 .45152 -30 -4.5 ]
[.01131 .45152 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(50)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(100)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(150)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(200)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .14793 m
.03006 .14793 L
s
[(-0.15)] .01131 .14793 1 0 Mshowa
.02381 .29973 m
.03006 .29973 L
s
[(-0.1)] .01131 .29973 1 0 Mshowa
.02381 .45152 m
.03006 .45152 L
s
[(-0.05)] .01131 .45152 1 0 Mshowa
.125 Mabswid
.02381 .0265 m
.02756 .0265 L
s
.02381 .05686 m
.02756 .05686 L
s
.02381 .08722 m
.02756 .08722 L
s
.02381 .11757 m
.02756 .11757 L
s
.02381 .17829 m
.02756 .17829 L
s
.02381 .20865 m
.02756 .20865 L
s
.02381 .23901 m
.02756 .23901 L
s
.02381 .26937 m
.02756 .26937 L
s
.02381 .33009 m
.02756 .33009 L
s
.02381 .36045 m
.02756 .36045 L
s
.02381 .39081 m
.02756 .39081 L
s
.02381 .42116 m
.02756 .42116 L
s
.02381 .48188 m
.02756 .48188 L
s
.02381 .51224 m
.02756 .51224 L
s
.02381 .5426 m
.02756 .5426 L
s
.02381 .57296 m
.02756 .57296 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.02846 .45889 L
.03279 .31709 L
.03527 .25604 L
.03754 .21129 L
.04018 .16942 L
.04262 .1386 L
.0475 .09343 L
.05008 .07619 L
.05281 .06158 L
.05739 .04349 L
.06 .03597 L
.06244 .03036 L
.06504 .02563 L
.06788 .0217 L
.07056 .01896 L
.07184 .01794 L
.07305 .01713 L
.07423 .01646 L
.07551 .01588 L
.07667 .01546 L
.07774 .01517 L
.0789 .01493 L
.08017 .01478 L
.08081 .01473 L
.0815 .01472 L
.08274 .01474 L
.08345 .0148 L
.0842 .01488 L
.08484 .01497 L
.08555 .01509 L
.08713 .01544 L
.08861 .01585 L
.09417 .01803 L
.09918 .02068 L
.10458 .02406 L
.14572 .05665 L
.18534 .08807 L
.22345 .11477 L
.26401 .13938 L
.30306 .15994 L
.34456 .17901 L
.38454 .19519 L
.423 .20907 L
.46392 .22232 L
.50333 .23385 L
.54121 .24397 L
.58155 .25383 L
.62038 .26256 L
.66165 .27113 L
Mistroke
.70141 .27878 L
.73966 .28563 L
.78036 .29245 L
.81954 .29859 L
.86117 .30472 L
.90129 .31028 L
.9399 .31533 L
.97619 .31983 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool00`000000oooo0?oo
o`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?oo
o`008@3oool00`000000oooo0?ooo`0;0?ooo`<00000kP3oool00240oooo00<000000?ooo`3oool0
2@3oool3000000040?ooo`000000000000000>`0oooo000Q0?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3Y0?ooo`008@3oool2000000P0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`3X0?ooo`008@3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00P0oooo0P00003X0?ooo`008@3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0i@3oool00240oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0>@0oooo000Q0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool03@3oool200000>@0oooo000Q0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3Q0?ooo`008@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0h03oool0
0240oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0B0?ooo`800000h03oool00240
oooo0P0000050?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0g@3oool00240oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0=`0oooo
000Q0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool05`3oool200000=`0oooo000Q
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`3I
0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool0f03oool00240oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo0=L0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
7@3oool200000=L0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`3D0?ooo`008@3oool2000000@0oooo00<000000?ooo`3oool0803oool0
0`000000oooo0?ooo`3C0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
0240oooo0P00003C0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02<0
oooo00<000000?ooo`3oool0d03oool00240oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0T0?ooo`030000003oool0oooo0<l0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool09@3oool200000<l0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`3<0?ooo`008@3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo02P0oooo0P00003<0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo02X0oooo00<000000?ooo`3oool0b@3oool00240oooo0P0000030?ooo`030000
003oool0oooo02`0oooo0P0000390?ooo`008@3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo02h0oooo0P0000370?ooo`008@3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0300oooo00<000000?ooo`3oool0a03oool00240oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0a0?ooo`800000a03oool00240oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0c0?ooo`800000`P3oool00240oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0e0?ooo`030000003oool0oooo0;l0oooo00080?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`@000000P3oool3000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0f0?ooo`800000_`3oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0h0?ooo`800000_@3oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0080oooo0P0000030?ooo`030000003oool0oooo03X0oooo00<0
00000?ooo`3oool0^P3oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`00
000:0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo03/0oooo0P00002j0?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`050000
003oool0oooo0?ooo`0000001`3oool01@000000oooo0?ooo`3oool000000400oooo0P00002h0?oo
o`001`3oool010000000oooo0?ooo`0000080?ooo`<000000`3oool00`000000oooo0?ooo`050?oo
o`050000003oool0oooo0?ooo`000000@P3oool200000;H0oooo00080?ooo`8000002`3oool01000
0000oooo0?ooo`3oool4000000@0oooo00D000000?ooo`3oool0oooo000000140?ooo`800000]03o
ool00240oooo00D000000?ooo`3oool0oooo000000160?ooo`800000/P3oool00240oooo00D00000
0?ooo`3oool0oooo000000180?ooo`800000/03oool00240oooo0P0000020?ooo`030000003oool0
oooo04P0oooo0P00002^0?ooo`008@3oool01@000000oooo0?ooo`3oool0000004`0oooo0`00002[
0?ooo`008@3oool01@000000oooo0?ooo`3oool0000004l0oooo0P00002Y0?ooo`008@3oool01@00
0000oooo0?ooo`3oool000000540oooo0`00002V0?ooo`008@3oool010000000oooo0?ooo`00001E
0?ooo`800000Y03oool00240oooo00@000000?ooo`3oool00000E`3oool300000:40oooo000Q0?oo
o`040000003oool0oooo000005X0oooo0`00002N0?ooo`008@3oool010000000oooo0?ooo`00001M
0?ooo`@00000VP3oool00240oooo0P0000000`3oool000000?ooo`1P0?ooo`<00000U`3oool00240
oooo00@000000?ooo`3oool00000I03oool3000009@0oooo000Q0?ooo`040000003oool0oooo0000
06L0oooo0P00002B0?ooo`008@3oool010000000oooo0?ooo`00001Y0?ooo`<00000S`3oool00240
oooo00@000000?ooo`3oool00000K03oool3000008`0oooo000Q0?ooo`040000003oool0oooo0000
06l0oooo0`0000290?ooo`008@3oool010000000oooo0?ooo`00001b0?ooo`@00000Q@3oool00240
oooo00@000000?ooo`3oool00000MP3oool400000840oooo000Q0?ooo`80000000<0oooo0000003o
ool0N@3oool5000007`0oooo000Q0?ooo`040000003oool0oooo000007l0oooo1000001h0?ooo`00
8@3oool010000000oooo0?ooo`0000230?ooo`<00000M@3oool00240oooo00@000000?ooo`3oool0
0000QP3oool400000740oooo000Q0?ooo`040000003oool0oooo000008X0oooo1000001]0?ooo`00
8@3oool010000000oooo0?ooo`00002>0?ooo`D00000J03oool00240oooo00@000000?ooo`3oool0
0000T`3oool5000006<0oooo000Q0?ooo`040000003oool0oooo000009P0oooo1P00001M0?ooo`00
8@3oool2000000030?ooo`000000oooo09d0oooo1@00001H0?ooo`008@3oool010000000oooo0?oo
o`00002S0?ooo`D00000D`3oool00240oooo00@000000?ooo`3oool00000Z03oool5000004h0oooo
000Q0?ooo`040000003oool0oooo00000:d0oooo1@0000190?ooo`008@3oool010000000oooo0?oo
o`00002b0?ooo`D00000A03oool00240oooo00<000000?ooo`000000^03oool5000003l0oooo000>
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`000000oooo0000002m
0?ooo`L00000>03oool000d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool000000<@0oooo2000000`0?ooo`003@3oool010000000oooo0?ooo`00000:
0?ooo`030000003oool0oooo00<0oooo0`00003<0?ooo`H00000:P3oool000L0oooo100000020?oo
o`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0000003B
0?ooo`L000008`3oool000d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool000000=T0oooo2000000K0?ooo`003@3oool010000000oooo0?ooo`000008
0?ooo`<000001@3oool00`000000oooo0000003Q0?ooo`D000005P3oool000h0oooo0P00000;0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`000000iP3oool7000000l0oooo000Q0?ooo`03
0000003oool000000>d0oooo1P0000090?ooo`008@3oool00`000000oooo0000003l0?ooo`008@3o
ool300000?`0oooo000Q0?ooo`030000003oool000000?`0oooo000Q0?ooo`030000003oool00000
0?`0oooo000Q0?ooo`030000003oool000000?`0oooo000Q0?ooo`030000003oool000000?`0oooo
000Q0?ooo`030000003oool000000?`0oooo000Q0?ooo`030000003oool000000?`0oooo000Q0?oo
o`030000003oool000000?`0oooo000Q0?ooo`<00000o03oool00240oooo00<000000?ooo`000000
o03oool00240oooo00<000000?ooo`000000o03oool00240oooo00<000000?ooo`000000o03oool0
0240oooo00<000000?ooo`000000o03oool00240oooo00<000000?ooo`000000o03oool00240oooo
00<000000?ooo`000000o03oool00240oooo00<000000?ooo`000000o03oool00240oooo0P00003m
0?ooo`008@3oool200000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3o
ool200000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0
oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo000Q0?oo
o`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo00080?ooo`800000103o
ool00`000000oooo0?ooo`030?ooo`8000000`3oool3000000D0oooo0P00003m0?ooo`001`3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0
0P3oool200000?d0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
00001@3oool00`000000oooo0?ooo`020?ooo`800000o@3oool000030?ooo`000000000000800000
0P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo0`0000050?oo
o`800000o@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo00D0oooo0P00003m0?ooo`001`3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool200000?d0oooo0008
0?ooo`8000002P3oool2000000<0oooo100000040?ooo`800000o@3oool00240oooo0P00003m0?oo
o`008@3oool200000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool2
00000?d0oooo000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo
000Q0?ooo`800000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo000Q0?ooo`80
0000o@3oool00240oooo0P00003m0?ooo`008@3oool200000?d0oooo000Q0?ooo`800000o@3oool0
0240oooo0P00003m0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo
0?ooo`3l0?ooo`008@3oool200000?d0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?oo
o`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?ooo`030000
003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo03D0oooo0`0000040?ooo`800000<@3o
ool4000000<0oooo0P0000040?ooo`800000;P3oool400000080oooo0`0000040?ooo`800000;`3o
ool4000000<0oooo0P0000040?ooo`8000000`3oool00240oooo00<000000?ooo`3oool0>03oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0300oooo00D000000?ooo`3oool0oooo
000000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0;@3oool00`000000
oooo0?ooo`040?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0;03oool00`00
0000oooo0?ooo`030?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3o
ool00240oooo00<000000?ooo`3oool0>03oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0300oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool000000P3oool00240oooo0P00000f0?ooo`<000000`3oool0
10000000oooo0?ooo`00000b0?ooo`050000003oool0oooo0?ooo`0000000P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo02d0oooo00@000000?ooo`3oool0oooo0`0000030?oo
o`040000003oool0oooo00000300oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003o
ool0oooo00000080oooo0@0000010?ooo`40oooo000Q0?ooo`030000003oool0oooo03D0oooo00<0
00000?ooo`3oool00`3oool010000000oooo0?ooo`00000b0?ooo`050000003oool0oooo0?ooo`00
00000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo02d0oooo00D000000?oo
o`3oool0oooo000000050?ooo`040000003oool0oooo00000340oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`4000000@3oool10?ooo`008@3oool00`000000oooo
0?ooo`0e0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool00000<03oool3000000<0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`00000]0?ooo`<000000`3oool0
0`000000oooo0?ooo`030?ooo`040000003oool0oooo000002h0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo000Q0?ooo`03
0000003oool0oooo03D0oooo100000030?ooo`800000<`3oool00`000000oooo0?ooo`020?ooo`80
0000103oool200000300oooo00@000000?ooo`3oool0oooo100000030?ooo`800000<03oool20000
00@0oooo0P0000040?ooo`8000000`3oool00240oooo00<000000?ooo`3oool0o03oool00240oooo
00<000000?ooo`3oool0o03oool00240oooo00<000000?ooo`3oool0o03oool00240oooo00<00000
0?ooo`3oool0o03oool001/0ooooo`000003000000<0oooo000Q0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`070?ooo`008@3oool00`000000oooo
0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?oo
o`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?oo
o`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-27.1372, -0.210319, \
0.817044, 0.00128156}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\(xi/\((1 + a\ xi^2)\)^2\)/Sqrt[xi^2 - kt], {xi, kt, 
        1}]\)], "Input"],

Cell[BoxData[
    \(\(\@a\ \@\(1 - kt\)\ \@\(\(-1\) - a\ kt\) - ArcTanh[\(\@a\ \@\(1 - kt\)\
\)\/\@\(\(-1\) - a\ kt\)] - a\ ArcTanh[\(\@a\ \@\(1 - kt\)\)\/\@\(\(-1\) - a\ \
kt\)]\)\/\(2\ \@a\ \((1 + a)\)\ \@\(\(-1\) - a\ kt\)\ \((1 + a\ kt)\)\) - \
\(\@a\ \@\(\(-1\) - a\ kt\)\ \@\(\(-kt\) + kt\^2\) - ArcTanh[\(\@a\ \@\(\(-kt\
\) + kt\^2\)\)\/\@\(\(-1\) - a\ kt\)] - a\ kt\^2\ ArcTanh[\(\@a\ \@\(\(-kt\) \
+ kt\^2\)\)\/\@\(\(-1\) - a\ kt\)]\)\/\(2\ \@a\ \@\(\(-1\) - a\ kt\)\ \((1 + \
a\ kt)\)\ \((1 + a\ kt\^2)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%277 /. kt \[Rule] 1 - kz^2/k^2, {kz > 0, k > 0}]\)], "Input"],

Cell[BoxData[
    \(\((\(\(\@a\ kz\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\)\)\/k - \
\((1 + a)\)\ ArcTanh[\(\@a\ kz\)\/\@\(\(-\((1 + a)\)\)\ k\^2 + a\ \
kz\^2\)]\)\/\(1 + a\) + \((\(-\@a\)\ k\^2\ \@\(\(-k\^2\)\ kz\^2 + kz\^4\)\ \@\
\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\) + \((\((1 + a)\)\ k\^4 - 
                      2\ a\ k\^2\ kz\^2 + 
                      a\ kz\^4)\)\ ArcTanh[\(\@a\ \@\(\(-k\^2\)\ kz\^2 + \
kz\^4\)\)\/\(k\^2\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\^2)\)\)\)])\)/\((\((1 \
+ a)\)\ k\^4 - 2\ a\ k\^2\ kz\^2 + a\ kz\^4)\))\)/\((2\ \@a\ \((1 + 
              a - \(a\ kz\^2\)\/k\^2)\)\ \@\(\(-1\) + a\ \((\(-1\) + kz\^2\/k\
\^2)\)\))\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[Integrate[1/\((1 + a\ xi^2)\)^2, {xi, 0, 1}]]\)], "Input"],

Cell[BoxData[
    \(\(\@a + \((1 + a)\)\ ArcTan[\@a]\)\/\(2\ \@a\ \((1 + a)\)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(deltamoz[md2_, kz_, k_, a_] := \(-I\)*\(md2/k^2\)/\((k^2 - kz^2)\)*
        a\ kz\ k*
        NIntegrate[\(xi^2/\((1 + a\ xi^2)\)^2\)/
            Sqrt[1 - kz^2/k^2 - xi^2], {xi, 0, Sqrt[1 - kz^2/k^2]}, 
          WorkingPrecision \[Rule] 20, MaxRecursion \[Rule] 20]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(deltamoz[1, 1/2, 1, 1]^2\)], "Input"],

Cell[BoxData[
    \(\(-0.0287743568544879123662192768`9.8689\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(klim2arg[k_, th_, 
        a_] := \((1 - Sqrt[1 + a] betamoz[1, k*Cos[th], k, a])\)*\((k^2 + 
              Sqrt[1 + a] alomz[1, k*Cos[th], k, a] + 
              Sqrt[1 + a] gamoz[1, k*Cos[th], k, a])\) + \((1 + a)\) 
          deltamoz[1, k*Cos[th], k, a]^2\ k^2 \((1 - Cos[th]^2)\)\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(klim2[th_, a_] := 
      k /. FindRoot[
          klim2arg[IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), th, 
              a] \[Equal] 0, {k, 0.1, 0.5}, 
          MaxIterations \[Rule] 20]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[Pi/16, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.5636769871008572`\)\(\[InvisibleSpace]\)\) - 
      1.496193039424535`*^-54\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[Pi/16, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.4739939947126368`\)\(\[InvisibleSpace]\)\) - 
      3.2107841974393127`*^-52\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      klim2[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 1], {th, 0.001, 
        Pi/4}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.21261 0.0147151 1.17721 [
[.26633 .00222 -9 -9 ]
[.26633 .00222 9 0 ]
[.50885 .00222 -9 -9 ]
[.50885 .00222 9 0 ]
[.75138 .00222 -9 -9 ]
[.75138 .00222 9 0 ]
[.9939 .00222 -9 -9 ]
[.9939 .00222 9 0 ]
[.01131 .13244 -18 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -18 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -18 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26633 .01472 m
.26633 .02097 L
s
[(0.2)] .26633 .00222 0 1 Mshowa
.50885 .01472 m
.50885 .02097 L
s
[(0.4)] .50885 .00222 0 1 Mshowa
.75138 .01472 m
.75138 .02097 L
s
[(0.6)] .75138 .00222 0 1 Mshowa
.9939 .01472 m
.9939 .02097 L
s
[(0.8)] .9939 .00222 0 1 Mshowa
.125 Mabswid
.08444 .01472 m
.08444 .01847 L
s
.14507 .01472 m
.14507 .01847 L
s
.2057 .01472 m
.2057 .01847 L
s
.32696 .01472 m
.32696 .01847 L
s
.38759 .01472 m
.38759 .01847 L
s
.44822 .01472 m
.44822 .01847 L
s
.56948 .01472 m
.56948 .01847 L
s
.63011 .01472 m
.63011 .01847 L
s
.69074 .01472 m
.69074 .01847 L
s
.81201 .01472 m
.81201 .01847 L
s
.87264 .01472 m
.87264 .01847 L
s
.93327 .01472 m
.93327 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(0.1)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.2)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(0.3)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(0.4)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(0.5)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03826 m
.02756 .03826 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .08535 m
.02756 .08535 L
s
.02381 .10889 m
.02756 .10889 L
s
.02381 .15598 m
.02756 .15598 L
s
.02381 .17952 m
.02756 .17952 L
s
.02381 .20307 m
.02756 .20307 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .2737 m
.02756 .2737 L
s
.02381 .29724 m
.02756 .29724 L
s
.02381 .32079 m
.02756 .32079 L
s
.02381 .34433 m
.02756 .34433 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .41497 m
.02756 .41497 L
s
.02381 .43851 m
.02756 .43851 L
s
.02381 .46205 m
.02756 .46205 L
s
.02381 .50914 m
.02756 .50914 L
s
.02381 .53269 m
.02756 .53269 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57977 m
.02756 .57977 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02502 .60332 m
.0262 .60332 L
.02726 .60331 L
.0285 .60331 L
.02966 .6033 L
.03174 .60328 L
.03399 .60326 L
.03909 .60319 L
.04381 .6031 L
.05342 .60284 L
.06361 .60245 L
.07395 .60194 L
.08363 .60135 L
.10569 .59964 L
.12591 .5976 L
.14441 .59536 L
.18407 .58931 L
.22221 .58194 L
.26281 .57248 L
.30189 .56182 L
.34342 .54891 L
.38343 .53495 L
.42193 .52018 L
.46289 .50306 L
.50232 .48523 L
.54025 .46687 L
.58062 .44599 L
.61948 .42459 L
.66079 .40036 L
.70058 .37546 L
.73886 .34989 L
.77959 .32065 L
.81881 .29004 L
.85651 .25761 L
.89666 .21839 L
.9353 .17298 L
.97619 .10496 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00CP3oool2000000@0oooo00<0
00000?ooo`3oool00P3oool400000340oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0<03oool2000000@0oooo00<000000?ooo`3oool00`3oool2000003<0oooo0P00
00040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`00C@3oool010000000oooo0?ooo`00
00080?ooo`030000003oool0oooo0340oooo00@000000?ooo`3oool000002P3oool00`000000oooo
0?ooo`0_0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000<@3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo001=0?ooo`040000003oool0
oooo000000T0oooo00<000000?ooo`3oool0<03oool010000000oooo0?ooo`0000070?ooo`D00000
<03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000340oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000020?ooo`00C@3oool010000000oooo0?ooo`00
000:0?ooo`030000003oool0oooo02l0oooo00@000000?ooo`3oool000001`3oool010000000oooo
0?ooo`00000a0?ooo`040000003oool0oooo000000P0oooo0`00000b0?ooo`040000003oool0oooo
000000T0oooo0P0000030?ooo`00C@3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0
oooo02h0oooo00@000000?ooo`3oool00000203oool00`000000oooo0000000a0?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`3oool0<P3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo00000080oooo001=0?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool00000<03oool010000000oooo0?ooo`0000090?ooo`800000<@3oool010000000oooo0?oo
o`0000080?ooo`030000003oool0oooo0380oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000020?ooo`005@3oool00`000000oooo0?ooo`0f0?ooo`8000002P3oool200000380
oooo0P00000;0?ooo`030000003oool0oooo0300oooo0P00000:0?ooo`<00000<P3oool2000000X0
oooo0P0000030?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003o
ool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`003`3ooooo0000
00X00000203oool001D0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`070?ooo`00
5@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0
?@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo00L0oooo000E0?ooo`030000003o
ool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`00
0000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`800000o`3oool:0?ooo`005@3oool00`000000oooo
0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<00000
0?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`03
0000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3o
ool200000?l0oooo2P3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`00
0000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo
00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E
0?ooo`800000o`3oool:0?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`03
0000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3o
ool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?T0oooo00<00000
0?ooo`3oool0303oool001D0oooo0P00003i0?ooo`030000003oool0oooo00d0oooo000E0?ooo`03
0000003oool0oooo0?P0oooo00<000000?ooo`3oool03@3oool001D0oooo00<000000?ooo`3oool0
m`3oool00`000000oooo0?ooo`0>0?ooo`005@3oool00`000000oooo0?ooo`3g0?ooo`030000003o
ool0oooo00h0oooo00020?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103oool0
0`000000oooo0?ooo`3f0?ooo`030000003oool0oooo00l0oooo00001@3oool000000?ooo`3oool0
000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3e0?ooo`030000003oool0
oooo0100oooo00001@3oool000000?ooo`3oool0000000X0oooo00<000000?ooo`3oool00`3oool2
00000?H0oooo00<000000?ooo`3oool0403oool000050?ooo`000000oooo0?ooo`0000002P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0?@0oooo00<000000?ooo`3oool04@3oool0
00050?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0?@0oooo00<000000?ooo`3oool04@3oool000050?ooo`000000oooo0?ooo`000000203oool3
000000D0oooo00<000000?ooo`3oool0l`3oool00`000000oooo0?ooo`0B0?ooo`000P3oool20000
00/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3b0?ooo`030000003oool0oooo
01<0oooo000E0?ooo`030000003oool0oooo0?80oooo00<000000?ooo`3oool04`3oool001D0oooo
0P00003b0?ooo`030000003oool0oooo01@0oooo000E0?ooo`030000003oool0oooo0?00oooo00<0
00000?ooo`3oool05@3oool001D0oooo00<000000?ooo`3oool0l03oool00`000000oooo0?ooo`0E
0?ooo`005@3oool00`000000oooo0?ooo`3_0?ooo`030000003oool0oooo01H0oooo000E0?ooo`03
0000003oool0oooo0>l0oooo00<000000?ooo`3oool05P3oool001D0oooo00<000000?ooo`3oool0
kP3oool00`000000oooo0?ooo`0G0?ooo`005@3oool00`000000oooo0?ooo`3]0?ooo`030000003o
ool0oooo01P0oooo000E0?ooo`800000k@3oool00`000000oooo0?ooo`0I0?ooo`005@3oool00`00
0000oooo0?ooo`3/0?ooo`030000003oool0oooo01T0oooo000E0?ooo`030000003oool0oooo0>/0
oooo00<000000?ooo`3oool06P3oool001D0oooo00<000000?ooo`3oool0jP3oool00`000000oooo
0?ooo`0K0?ooo`005@3oool00`000000oooo0?ooo`3Y0?ooo`030000003oool0oooo01`0oooo000E
0?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3oool07@3oool001D0oooo0P00003X0?oo
o`030000003oool0oooo01h0oooo000E0?ooo`030000003oool0oooo0>L0oooo00<000000?ooo`3o
ool07P3oool001D0oooo00<000000?ooo`3oool0iP3oool00`000000oooo0?ooo`0O0?ooo`005@3o
ool00`000000oooo0?ooo`3U0?ooo`030000003oool0oooo0200oooo000E0?ooo`030000003oool0
oooo0>@0oooo00<000000?ooo`3oool08@3oool001D0oooo00<000000?ooo`3oool0h`3oool00`00
0000oooo0?ooo`0R0?ooo`005@3oool200000><0oooo00<000000?ooo`3oool08`3oool001D0oooo
00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0T0?ooo`005@3oool00`000000oooo0?oo
o`3P0?ooo`030000003oool0oooo02D0oooo000E0?ooo`030000003oool0oooo0=h0oooo0P00000X
0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000@0oooo00<000000?oo
o`3oool0g@3oool00`000000oooo0?ooo`0X0?ooo`0000D0oooo0000003oool0oooo000000080?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0g03oool00`000000oooo0?ooo`0Y0?oo
o`0000D0oooo0000003oool0oooo000000090?ooo`030000003oool0oooo00@0oooo0P00003L0?oo
o`030000003oool0oooo02X0oooo00001@3oool000000?ooo`3oool0000000X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`3J0?ooo`030000003oool0oooo02/0oooo00001@3oool0
00000?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3I0?oo
o`030000003oool0oooo02`0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`3H0?ooo`030000003oool0oooo02d0oooo00020?oo
o`8000002P3oool2000000D0oooo00<000000?ooo`3oool0e`3oool00`000000oooo0?ooo`0^0?oo
o`005@3oool00`000000oooo0?ooo`3F0?ooo`030000003oool0oooo02l0oooo000E0?ooo`030000
003oool0oooo0=@0oooo0P00000b0?ooo`005@3oool200000=@0oooo00<000000?ooo`3oool0<P3o
ool001D0oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`0c0?ooo`005@3oool00`00
0000oooo0?ooo`3A0?ooo`030000003oool0oooo03@0oooo000E0?ooo`030000003oool0oooo0=00
oooo00<000000?ooo`3oool0=@3oool001D0oooo00<000000?ooo`3oool0c`3oool00`000000oooo
0?ooo`0f0?ooo`005@3oool00`000000oooo0?ooo`3>0?ooo`030000003oool0oooo03L0oooo000E
0?ooo`800000c@3oool2000003X0oooo000E0?ooo`030000003oool0oooo0</0oooo00<000000?oo
o`3oool0>P3oool001D0oooo00<000000?ooo`3oool0bP3oool00`000000oooo0?ooo`0k0?ooo`00
5@3oool00`000000oooo0?ooo`390?ooo`030000003oool0oooo03`0oooo000E0?ooo`030000003o
ool0oooo0<L0oooo0P00000o0?ooo`005@3oool00`000000oooo0?ooo`360?ooo`030000003oool0
oooo03l0oooo000E0?ooo`800000aP3oool00`000000oooo0?ooo`100?ooo`005@3oool00`000000
oooo0?ooo`330?ooo`800000@`3oool001D0oooo00<000000?ooo`3oool0`P3oool00`000000oooo
0?ooo`130?ooo`005@3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo04@0oooo000E
0?ooo`030000003oool0oooo0;l0oooo0P0000170?ooo`005@3oool00`000000oooo0?ooo`2n0?oo
o`030000003oool0oooo04L0oooo000E0?ooo`800000_P3oool00`000000oooo0?ooo`180?ooo`00
5@3oool00`000000oooo0?ooo`2k0?ooo`800000B`3oool001D0oooo00<000000?ooo`3oool0^P3o
ool00`000000oooo0?ooo`1;0?ooo`005@3oool00`000000oooo0?ooo`2h0?ooo`800000CP3oool0
01D0oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?ooo`1>0?ooo`000P3oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3oool0]@3oool200000540
oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool00`00
0000oooo0?ooo`2d0?ooo`030000003oool0oooo0540oooo00001@3oool000000?ooo`3oool00000
00/0oooo00<000000?ooo`3oool00P3oool200000;@0oooo00<000000?ooo`3oool0DP3oool00005
0?ooo`000000oooo0?ooo`0000002@3oool2000000D0oooo00<000000?ooo`3oool0/@3oool20000
05D0oooo00001@3oool000000?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`2`0?ooo`030000003oool0oooo05D0oooo00001@3oool000000?ooo`3oool00000
00P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`2^0?ooo`800000F03oool0
0080oooo0P00000:0?ooo`8000001@3oool00`000000oooo0?ooo`2/0?ooo`800000FP3oool001D0
oooo00<000000?ooo`3oool0ZP3oool2000005`0oooo000E0?ooo`800000Z@3oool2000005h0oooo
000E0?ooo`030000003oool0oooo0:H0oooo0P00001P0?ooo`005@3oool00`000000oooo0?ooo`2U
0?ooo`030000003oool0oooo0600oooo000E0?ooo`030000003oool0oooo0:<0oooo0P00001S0?oo
o`005@3oool00`000000oooo0?ooo`2R0?ooo`030000003oool0oooo06<0oooo000E0?ooo`030000
003oool0oooo0:00oooo0P00001V0?ooo`005@3oool2000009l0oooo0P00001X0?ooo`005@3oool0
0`000000oooo0?ooo`2M0?ooo`030000003oool0oooo06P0oooo000E0?ooo`030000003oool0oooo
09/0oooo0P00001[0?ooo`005@3oool00`000000oooo0?ooo`2I0?ooo`800000K@3oool001D0oooo
00<000000?ooo`3oool0U`3oool2000006l0oooo000E0?ooo`030000003oool0oooo09D0oooo0P00
001a0?ooo`005@3oool2000009@0oooo0P00001c0?ooo`005@3oool00`000000oooo0?ooo`2A0?oo
o`800000M@3oool001D0oooo00<000000?ooo`3oool0T03oool00`000000oooo0?ooo`1e0?ooo`00
5@3oool00`000000oooo0?ooo`2>0?ooo`800000N03oool001D0oooo00<000000?ooo`3oool0S03o
ool2000007X0oooo000E0?ooo`030000003oool0oooo08X0oooo0P00001l0?ooo`005@3oool20000
08T0oooo0P00001n0?ooo`005@3oool00`000000oooo0?ooo`260?ooo`800000P03oool001D0oooo
00<000000?ooo`3oool0Q03oool200000880oooo000E0?ooo`030000003oool0oooo0880oooo0P00
00240?ooo`005@3oool00`000000oooo0?ooo`200?ooo`800000QP3oool00080oooo0P0000040?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1n0?oo
o`800000R03oool000050?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo07`0oooo0P00002:0?ooo`0000D0oooo0000003oool0oooo000000070?oo
o`D00000103oool2000007/0oooo0P00002<0?ooo`0000D0oooo0000003oool0oooo000000070?oo
o`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0N03oool2000008h0oooo00001@3o
ool000000?ooo`3oool0000000P0oooo00<000000?ooo`0000001@3oool00`000000oooo0?ooo`1e
0?ooo`<00000T03oool000050?ooo`000000oooo0?ooo`0000002@3oool2000000D0oooo00<00000
0?ooo`3oool0L`3oool2000009<0oooo00020?ooo`8000002`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0700oooo0`00002E0?ooo`005@3oool00`000000oooo0?ooo`1^0?ooo`80
0000V03oool001D0oooo0P00001]0?ooo`800000VP3oool001D0oooo00<000000?ooo`3oool0JP3o
ool2000009`0oooo000E0?ooo`030000003oool0oooo06P0oooo0P00002N0?ooo`005@3oool00`00
0000oooo0?ooo`1U0?ooo`<00000X03oool001D0oooo00<000000?ooo`3oool0H`3oool200000:<0
oooo000E0?ooo`030000003oool0oooo0600oooo0`00002U0?ooo`005@3oool2000005l0oooo0P00
002X0?ooo`005@3oool00`000000oooo0?ooo`1K0?ooo`<00000ZP3oool001D0oooo00<000000?oo
o`3oool0E`3oool400000:d0oooo000E0?ooo`030000003oool0oooo05<0oooo1000002a0?ooo`00
5@3oool00`000000oooo0?ooo`1@0?ooo`<00000]@3oool001D0oooo00<000000?ooo`3oool0C@3o
ool300000;P0oooo000E0?ooo`800000C03oool200000;/0oooo000E0?ooo`030000003oool0oooo
04P0oooo0`00002m0?ooo`005@3oool00`000000oooo0?ooo`150?ooo`<00000`03oool001D0oooo
00<000000?ooo`3oool0@@3oool400000<<0oooo000E0?ooo`030000003oool0oooo03h0oooo0`00
00370?ooo`005@3oool00`000000oooo0?ooo`0j0?ooo`@00000bP3oool001D0oooo00<000000?oo
o`3oool0=03oool600000<h0oooo000E0?ooo`800000<03oool500000=@0oooo000E0?ooo`030000
003oool0oooo02X0oooo1@00003I0?ooo`005@3oool00`000000oooo0?ooo`0U0?ooo`D00000gP3o
ool001D0oooo00<000000?ooo`3oool0803oool500000><0oooo00020?ooo`800000103oool00`00
0000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`0E0?ooo`/00000j03oool00005
0?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00T0oooo3000003c0?ooo`0000D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo
0080oooo3000003o0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`00
0000oooo0?ooo`3o0?ooo`T0oooo00001@3oool000000?ooo`3oool0000000P0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo00001@3oool000000?ooo`3oool00000
00P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo00020?ooo`80
00002@3oool4000000@0oooo00<000000?ooo`3oool0o`3oool90?ooo`00o`3ooolQ0?ooo`00o`3o
oolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0664109, -0.0465441, \
0.00311661, 0.00321033}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(limiting\ angle\ for\ a = 1\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[
      klim2arg[10^\((\(-6\))\), IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 
        1], {th, Pi/4, 101/400*Pi}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \({th \[Rule] \(\(0.8020341412315661`\)\(\[InvisibleSpace]\)\) + 
          2.8166847230142103`*^-25\ \[ImaginaryI]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.8020341412315661`\[InvisibleSpace]/Pi\)], "Input"],

Cell[BoxData[
    \(0.2552953962109341`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%^\((\(-1\))\)\)], "Input"],

Cell[BoxData[
    \(3.9170310739711285`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[
      klim2arg[10^\((\(-6\))\), IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 
        10], {th, Pi/4, 101/400*Pi}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \({th \[Rule] \(\(0.8099197921304028`\)\(\[InvisibleSpace]\)\) + 
          1.1368769551758511`*^-25\ \[ImaginaryI]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(0.8099197921304028`/Pi\)], "Input"],

Cell[BoxData[
    \(0.25780547685102795`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(th /. 
      FindRoot[klim2arg[10^\((\(-6\))\), 
          IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 100], {th, Pi/4, 
          101/400*Pi}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(\(\(0.7981919329955551`\)\(\[InvisibleSpace]\)\) - 
      2.5627456051393104`*^-26\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{a, 
        th /. FindRoot[
            klim2arg[10^\((\(-6\))\), 
              IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 
              IntegerPart[a*10^\((16)\)]*10^\((\(-16\))\)], {th, Pi/4, 
              101/400*Pi}]}, {a, 0.1, 10.1, 1}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(15\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frmp\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.1`, \(\(0.7882868228638855`\)\(\[InvisibleSpace]\)\) - 
          1.1115504149409715`*^-25\ \[ImaginaryI]}, {1.1`, \
\(\(0.8028358885960747`\)\(\[InvisibleSpace]\)\) - 
          1.78903309336624`*^-26\ \[ImaginaryI]}, {2.1`, \
\(\(0.8076946591892528`\)\(\[InvisibleSpace]\)\) + 
          1.0758397157566463`*^-25\ \[ImaginaryI]}, {3.1`, \
\(\(0.8096895969588516`\)\(\[InvisibleSpace]\)\) + 
          2.0487453918064444`*^-25\ \[ImaginaryI]}, {4.1`, \
\(\(0.8105296520571235`\)\(\[InvisibleSpace]\)\) + 
          1.756716183260806`*^-25\ \[ImaginaryI]}, {5.1`, \
\(\(0.8108241818104519`\)\(\[InvisibleSpace]\)\) - 
          2.4632665629340815`*^-26\ \[ImaginaryI]}, {6.1`, \
\(\(0.8108363410019264`\)\(\[InvisibleSpace]\)\) - 
          8.426659316965144`*^-9\ \[ImaginaryI]}, {7.1`, \
\(\(0.8106944259768112`\)\(\[InvisibleSpace]\)\) + 
          0.26228219495944916`\ \[ImaginaryI]}, {8.1`, \
\(\(0.8104663942041868`\)\(\[InvisibleSpace]\)\) + 
          1.2085040698538525`*^-25\ \[ImaginaryI]}, {9.1`, \
\(\(0.8101904314563217`\)\(\[InvisibleSpace]\)\) + 
          1.8303031105806661`*^-25\ \[ImaginaryI]}, {10.1`, \
\(\(0.8098889273626387`\)\(\[InvisibleSpace]\)\) + 
          4.1116219233318796`*^-26\ \[ImaginaryI]}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[Re[%%], PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0142857 0.0952381 -20.5617 26.1027 [
[.20476 .30796 -3 -9 ]
[.20476 .30796 3 0 ]
[.39524 .30796 -3 -9 ]
[.39524 .30796 3 0 ]
[.58571 .30796 -3 -9 ]
[.58571 .30796 3 0 ]
[.77619 .30796 -3 -9 ]
[.77619 .30796 3 0 ]
[.96667 .30796 -6 -9 ]
[.96667 .30796 6 0 ]
[.00179 .05943 -24 -4.5 ]
[.00179 .05943 0 4.5 ]
[.00179 .18995 -30 -4.5 ]
[.00179 .18995 0 4.5 ]
[.00179 .45097 -30 -4.5 ]
[.00179 .45097 0 4.5 ]
[.00179 .58149 -24 -4.5 ]
[.00179 .58149 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.20476 .32046 m
.20476 .32671 L
s
[(2)] .20476 .30796 0 1 Mshowa
.39524 .32046 m
.39524 .32671 L
s
[(4)] .39524 .30796 0 1 Mshowa
.58571 .32046 m
.58571 .32671 L
s
[(6)] .58571 .30796 0 1 Mshowa
.77619 .32046 m
.77619 .32671 L
s
[(8)] .77619 .30796 0 1 Mshowa
.96667 .32046 m
.96667 .32671 L
s
[(10)] .96667 .30796 0 1 Mshowa
.125 Mabswid
.0619 .32046 m
.0619 .32421 L
s
.10952 .32046 m
.10952 .32421 L
s
.15714 .32046 m
.15714 .32421 L
s
.25238 .32046 m
.25238 .32421 L
s
.3 .32046 m
.3 .32421 L
s
.34762 .32046 m
.34762 .32421 L
s
.44286 .32046 m
.44286 .32421 L
s
.49048 .32046 m
.49048 .32421 L
s
.5381 .32046 m
.5381 .32421 L
s
.63333 .32046 m
.63333 .32421 L
s
.68095 .32046 m
.68095 .32421 L
s
.72857 .32046 m
.72857 .32421 L
s
.82381 .32046 m
.82381 .32421 L
s
.87143 .32046 m
.87143 .32421 L
s
.91905 .32046 m
.91905 .32421 L
s
.25 Mabswid
0 .32046 m
1 .32046 L
s
.01429 .05943 m
.02054 .05943 L
s
[(0.79)] .00179 .05943 1 0 Mshowa
.01429 .18995 m
.02054 .18995 L
s
[(0.795)] .00179 .18995 1 0 Mshowa
.01429 .45097 m
.02054 .45097 L
s
[(0.805)] .00179 .45097 1 0 Mshowa
.01429 .58149 m
.02054 .58149 L
s
[(0.81)] .00179 .58149 1 0 Mshowa
.125 Mabswid
.01429 .08554 m
.01804 .08554 L
s
.01429 .11164 m
.01804 .11164 L
s
.01429 .13774 m
.01804 .13774 L
s
.01429 .16384 m
.01804 .16384 L
s
.01429 .21605 m
.01804 .21605 L
s
.01429 .24215 m
.01804 .24215 L
s
.01429 .26826 m
.01804 .26826 L
s
.01429 .29436 m
.01804 .29436 L
s
.01429 .34656 m
.01804 .34656 L
s
.01429 .37267 m
.01804 .37267 L
s
.01429 .39877 m
.01804 .39877 L
s
.01429 .42487 m
.01804 .42487 L
s
.01429 .47708 m
.01804 .47708 L
s
.01429 .50318 m
.01804 .50318 L
s
.01429 .52928 m
.01804 .52928 L
s
.01429 .55539 m
.01804 .55539 L
s
.01429 .03333 m
.01804 .03333 L
s
.01429 .00723 m
.01804 .00723 L
s
.01429 .60759 m
.01804 .60759 L
s
.25 Mabswid
.01429 0 m
.01429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.11905 .39449 L
.21429 .52131 L
.30952 .57339 L
.40476 .59531 L
.5 .603 L
.59524 .60332 L
.69048 .59961 L
.78571 .59366 L
.88095 .58646 L
.97619 .57859 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3o
ool00`000000oooo0?ooo`3l0?ooo`008@3oool200000?d0oooo000Q0?ooo`030000003oool0oooo
0?`0oooo000Q0?ooo`030000003oool000000?`0oooo000Q0?ooo`030000003oool000000?`0oooo
000Q0?ooo`040000003oool0oooo00000?/0oooo000Q0?ooo`040000003oool0oooo00000?/0oooo
000Q0?ooo`040000003oool0oooo00000?/0oooo000Q0?ooo`80000000<0oooo0000003oool0nP3o
ool00240oooo00D000000?ooo`3oool0oooo0000003j0?ooo`008@3oool01@000000oooo0?ooo`3o
ool000000?X0oooo000Q0?ooo`050000003oool0oooo0?ooo`000000nP3oool00240oooo00D00000
0?ooo`3oool0oooo0000003j0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0?L0oooo00070?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0m`3oool0
00L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0080oooo0P0000030?ooo`030000003oool0oooo0?L0oooo00070?ooo`040000003oool0oooo
000000X0oooo00<000000?ooo`3oool00P3oool3000000@0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`3g0?ooo`001`3oool010000000oooo0?ooo`00000;0?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0?H0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0mP3oool000P0oooo0P0000090?ooo`@000000`3oool2000000D0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`3f0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0?H0oooo000Q0?ooo`8000001@3oool00`000000oooo0?ooo`3e0?ooo`008@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0?D0oooo000Q0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0m@3oool00240oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`3e0?ooo`008@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?@0oooo000Q
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0m03oool00240oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`3d0?ooo`008@3oool2000000H0oooo00<000000?ooo`3o
ool0m03oool00240oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3c0?ooo`008@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?<0oooo000Q0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool0l`3oool00240oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`3c0?ooo`008@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?80
oooo000Q0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0lP3oool00240oooo0P00
00080?ooo`030000003oool0oooo0?80oooo000Q0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool0lP3oool00240oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3a0?oo
o`008@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?40oooo000Q0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool0l@3oool00240oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`3a0?ooo`008@3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo0?00oooo000Q0?ooo`8000002P3oool00`000000oooo0?ooo`3`0?ooo`008@3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0?00oooo000Q0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0l03oool00240oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`3_0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`02
0?ooo`<000000`3oool3000000D0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3_
0?ooo`0000D0oooo0000003oool0oooo000000090?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool0k`3oool000050?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool2000000/0oooo00<00000
0?ooo`3oool0k`3oool000050?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`02
0?ooo`<000000P3oool3000000D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3^
0?ooo`0000D0oooo0000003oool0oooo0000000;0?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool000001`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0>h0oooo
00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`3^0?ooo`000P3oool2000000T0oooo100000030?ooo`8000000`3oool4000000@0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3^0?ooo`008@3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo0>d0oooo000Q0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool0k@3oool00240oooo0P00000=0?ooo`030000003oool0oooo0>d0oooo000Q0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0k03oool00240oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`3/0?ooo`008@3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo0>`0oooo000Q0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0k03o
ool00240oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3[0?ooo`008@3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo0>/0oooo000Q0?ooo`8000003`3oool00`000000
oooo0?ooo`3[0?ooo`008@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0>/0oooo
000Q0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0jP3oool00240oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`3Z0?ooo`008@3oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo0>X0oooo000Q0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool0jP3oool00240oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3Y0?ooo`008@3o
ool200000140oooo00<000000?ooo`3oool0j@3oool00240oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`3Y0?ooo`008@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0>T0oooo000Q0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0603oool4000002l0
oooo00<000000?ooo`3oool0;@3oool2000002l0oooo0P00000[0?ooo`@000000`3oool2000000H0
oooo000Q0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0`0?ooo`030000003oool0oooo02`0oooo00@000000?ooo`3oool00000;@3oool010000000
oooo0?ooo`00000/0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`03
0?ooo`008@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0;03oool5000002d0oooo00@000000?ooo`3oool00000;@3oool010000000oooo0?ooo`00
000/0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`030?ooo`008@3o
ool200000180oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0[0?ooo`040000003o
ool0oooo000002h0oooo0`00000_0?ooo`800000;@3oool01@000000oooo0?ooo`3oool000000080
oooo00<000000?ooo`3oool00`3oool00240oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`000000;P3oool00`000000oooo
0?ooo`0^0?ooo`040000003oool0oooo000002`0oooo00D000000?ooo`3oool0oooo000000020?oo
o`030000003oool0oooo00<0oooo000Q0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool05`3oool010000000oooo0?ooo`00000^0?ooo`800000;P3oool00`000000oooo0?ooo`0^0?oo
o`040000003oool0oooo000002X0oooo0`0000030?ooo`040000003oool0oooo000000D0oooo000Q
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0603oool200000300oooo00<00000
0?ooo`3oool0;@3oool3000002h0oooo0P00000]0?ooo`030000003oool0oooo0080oooo0P000006
0?ooo`008@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0>L0oooo000Q0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool0iP3oool00240oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`3V0?ooo`007@3ooooo000000@00000000Q0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0000000;0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`070?ooo`008@3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`0^0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`070?ooo`008@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0>D0
oooo000Q0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0i@3oool00240oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`3U0?ooo`008@3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo0>@0oooo000Q0?ooo`8000005P3oool00`000000oooo0?ooo`3T0?oo
o`008@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0>@0oooo000Q0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool0i03oool00240oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`3S0?ooo`008@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo0><0oooo000Q0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0h`3oool00240
oooo0P00000G0?ooo`030000003oool0oooo0><0oooo000Q0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool0hP3oool00240oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`3R0?ooo`008@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0>80oooo000Q0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool0hP3oool00240oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`3Q0?ooo`008@3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo0>40oooo000Q0?ooo`8000006P3oool00`000000oooo0?ooo`3P0?ooo`008@3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0>00oooo000Q0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool0g`3oool00240oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`3N0?ooo`008@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0=d0oooo
000Q0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0g@3oool00240oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`3L0?ooo`008@3oool2000001l0oooo00<000000?oo
o`3oool0f`3oool00240oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`3J0?ooo`00
8@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0=X0oooo000Q0?ooo`030000003o
ool0oooo0200oooo00<000000?ooo`3oool0f@3oool00240oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`3H0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00`3oool20000
00@0oooo0P0000030?ooo`<000001@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0=P0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool08P3o
ool00`000000oooo0?ooo`3G0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`80
0000903oool00`000000oooo0?ooo`3F0?ooo`0000D0oooo0000003oool0oooo000000090?ooo`80
00000`3oool010000000oooo0?ooo`0000020?ooo`<000001@3oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo0=D0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?oo
o`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`3E0?ooo`0000D0oooo0000003oool0oooo0000
00080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0e03oool00080oooo
0P00000:0?ooo`800000103oool2000000<0oooo100000040?ooo`030000003oool0oooo02H0oooo
00<000000?ooo`3oool0d`3oool00240oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`3B0?ooo`008@3oool2000002P0oooo00<000000?ooo`3oool0dP3oool00240oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`3A0?ooo`008@3oool00`000000oooo0?ooo`0Y0?ooo`03
0000003oool0oooo0=00oooo000Q0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0
d03oool00240oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3?0?ooo`008@3oool0
0`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0<h0oooo000Q0?ooo`030000003oool0oooo
02`0oooo00<000000?ooo`3oool0c@3oool00240oooo0P00000]0?ooo`030000003oool0oooo0<d0
oooo000Q0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0c03oool00240oooo00<0
00000?ooo`3oool0;P3oool00`000000oooo0?ooo`3;0?ooo`008@3oool00`000000oooo0?ooo`0_
0?ooo`030000003oool0oooo0<X0oooo000Q0?ooo`030000003oool0oooo02l0oooo00<000000?oo
o`3oool0bP3oool00240oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`390?ooo`00
8@3oool00`000000oooo0?ooo`0a0?ooo`800000b@3oool00240oooo0P00000d0?ooo`800000a`3o
ool00240oooo00<000000?ooo`3oool0=@3oool200000<D0oooo000Q0?ooo`030000003oool0oooo
03L0oooo0P0000330?ooo`008@3oool00`000000oooo0?ooo`0i0?ooo`800000`@3oool00240oooo
00<000000?ooo`3oool0>`3oool200000;l0oooo000Q0?ooo`030000003oool0oooo03d0oooo0P00
002m0?ooo`008@3oool200000400oooo0P00002k0?ooo`008@3oool00`000000oooo0?ooo`110?oo
o`800000^@3oool00240oooo00<000000?ooo`3oool0@`3oool200000;L0oooo000Q0?ooo`030000
003oool0oooo04D0oooo0P00002e0?ooo`008@3oool00`000000oooo0?ooo`170?ooo`800000/`3o
ool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`@00000103oool0
0`000000oooo0?ooo`190?ooo`800000/@3oool000L0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0B`3o
ool4000009l0oooo1`0000070?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000000@0oooo00<000000?ooo`3oool00`3oool200000500oooo1000002?0?ooo``00000
3P3oool000L0oooo00@000000?ooo`3oool000002@3oool2000000D0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`1C0?ooo`@00000O`3oool<000001X0oooo00070?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo05L0oooo1000001_0?ooo``000009P3oool000L0oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`0000020?ooo`<000001@3oool00`000000oooo0?ooo`1K0?oo
o`@00000GP3oool=00000380oooo00080?ooo`8000002P3oool2000000D0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`1O0?ooo`T00000B@3oool<000003l0oooo000Q0?ooo`030000
003oool0oooo06P0oooo3000000[0?oooa800000B`3oool00240oooo00<000000?ooo`3oool0M03o
ool[000005d0oooo000Q0?ooo`800000o@3oool00240oooo00<000000?ooo`3oool0o03oool00240
oooo00<000000?ooo`3oool0o03oool00240oooo00<000000?ooo`3oool0o03oool00?l0oooo8@3o
ool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3o
ool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.35476, 0.786397, \
0.0407833, 0.000148802}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      klim2arg[IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), Pi*101/400, 
        1], {k, 0.01, 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.534754 0.0366403 [
[.21429 .52225 -9 -9 ]
[.21429 .52225 9 0 ]
[.40476 .52225 -9 -9 ]
[.40476 .52225 9 0 ]
[.59524 .52225 -9 -9 ]
[.59524 .52225 9 0 ]
[.78571 .52225 -9 -9 ]
[.78571 .52225 9 0 ]
[.97619 .52225 -3 -9 ]
[.97619 .52225 3 0 ]
[.01131 .07675 -30 -4.5 ]
[.01131 .07675 0 4.5 ]
[.01131 .16835 -18 -4.5 ]
[.01131 .16835 0 4.5 ]
[.01131 .25995 -24 -4.5 ]
[.01131 .25995 0 4.5 ]
[.01131 .35155 -12 -4.5 ]
[.01131 .35155 0 4.5 ]
[.01131 .44315 -24 -4.5 ]
[.01131 .44315 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .53475 m
.21429 .541 L
s
[(0.2)] .21429 .52225 0 1 Mshowa
.40476 .53475 m
.40476 .541 L
s
[(0.4)] .40476 .52225 0 1 Mshowa
.59524 .53475 m
.59524 .541 L
s
[(0.6)] .59524 .52225 0 1 Mshowa
.78571 .53475 m
.78571 .541 L
s
[(0.8)] .78571 .52225 0 1 Mshowa
.97619 .53475 m
.97619 .541 L
s
[(1)] .97619 .52225 0 1 Mshowa
.125 Mabswid
.07143 .53475 m
.07143 .5385 L
s
.11905 .53475 m
.11905 .5385 L
s
.16667 .53475 m
.16667 .5385 L
s
.2619 .53475 m
.2619 .5385 L
s
.30952 .53475 m
.30952 .5385 L
s
.35714 .53475 m
.35714 .5385 L
s
.45238 .53475 m
.45238 .5385 L
s
.5 .53475 m
.5 .5385 L
s
.54762 .53475 m
.54762 .5385 L
s
.64286 .53475 m
.64286 .5385 L
s
.69048 .53475 m
.69048 .5385 L
s
.7381 .53475 m
.7381 .5385 L
s
.83333 .53475 m
.83333 .5385 L
s
.88095 .53475 m
.88095 .5385 L
s
.92857 .53475 m
.92857 .5385 L
s
.25 Mabswid
0 .53475 m
1 .53475 L
s
.02381 .07675 m
.03006 .07675 L
s
[(-12.5)] .01131 .07675 1 0 Mshowa
.02381 .16835 m
.03006 .16835 L
s
[(-10)] .01131 .16835 1 0 Mshowa
.02381 .25995 m
.03006 .25995 L
s
[(-7.5)] .01131 .25995 1 0 Mshowa
.02381 .35155 m
.03006 .35155 L
s
[(-5)] .01131 .35155 1 0 Mshowa
.02381 .44315 m
.03006 .44315 L
s
[(-2.5)] .01131 .44315 1 0 Mshowa
.125 Mabswid
.02381 .09507 m
.02756 .09507 L
s
.02381 .11339 m
.02756 .11339 L
s
.02381 .13171 m
.02756 .13171 L
s
.02381 .15003 m
.02756 .15003 L
s
.02381 .18667 m
.02756 .18667 L
s
.02381 .20499 m
.02756 .20499 L
s
.02381 .22331 m
.02756 .22331 L
s
.02381 .24163 m
.02756 .24163 L
s
.02381 .27827 m
.02756 .27827 L
s
.02381 .29659 m
.02756 .29659 L
s
.02381 .31491 m
.02756 .31491 L
s
.02381 .33323 m
.02756 .33323 L
s
.02381 .36987 m
.02756 .36987 L
s
.02381 .38819 m
.02756 .38819 L
s
.02381 .40651 m
.02756 .40651 L
s
.02381 .42483 m
.02756 .42483 L
s
.02381 .46147 m
.02756 .46147 L
s
.02381 .47979 m
.02756 .47979 L
s
.02381 .49811 m
.02756 .49811 L
s
.02381 .51643 m
.02756 .51643 L
s
.02381 .05843 m
.02756 .05843 L
s
.02381 .04011 m
.02756 .04011 L
s
.02381 .02179 m
.02756 .02179 L
s
.02381 .00347 m
.02756 .00347 L
s
.02381 .55307 m
.02756 .55307 L
s
.02381 .57139 m
.02756 .57139 L
s
.02381 .58971 m
.02756 .58971 L
s
.02381 .60803 m
.02756 .60803 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03645 0 m
.03678 .03127 L
.03793 .1153 L
.03999 .22271 L
.04115 .26721 L
.04222 .30105 L
.04447 .35575 L
.04694 .39851 L
.04953 .43066 L
.05195 .45309 L
.05448 .4711 L
.05689 .48452 L
.06141 .50313 L
.06393 .51089 L
.06627 .5169 L
.07158 .5274 L
.07703 .5351 L
.08217 .54047 L
.08678 .54423 L
.09177 .54746 L
.09722 .55028 L
.10298 .55266 L
.1133 .55585 L
.12268 .55796 L
.13274 .55967 L
.15089 .56185 L
.16096 .56275 L
.17192 .56356 L
.19184 .56473 L
.23221 .56646 L
.27108 .56777 L
.30846 .56894 L
.34826 .57017 L
.38656 .57141 L
.42729 .5728 L
.46651 .57424 L
.50424 .57571 L
.5444 .57739 L
.58305 .57912 L
.62413 .58109 L
.66372 .5831 L
.7018 .58515 L
.74231 .58746 L
.78132 .5898 L
.81883 .59217 L
.85876 .59481 L
.8972 .59747 L
.93806 .60044 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool010000000oooo0?ooo`00003k0?ooo`00
8@3oool2000000030?ooo`000000oooo0?X0oooo000Q0?ooo`040000003oool0oooo00000?/0oooo
000Q0?ooo`040000003oool0oooo00000?/0oooo000Q0?ooo`040000003oool0oooo00000?/0oooo
000Q0?ooo`80000000<0oooo0000003oool0nP3oool00240oooo00@000000?ooo`3oool00000n`3o
ool00240oooo00@000000?ooo`3oool00000n`3oool00240oooo00@000000?ooo`3oool00000n`3o
ool00240oooo00@000000?ooo`3oool00000n`3oool00240oooo0P0000000`3oool000000?ooo`3j
0?ooo`008@3oool010000000oooo0?ooo`00003k0?ooo`008@3oool010000000oooo0?ooo`00003k
0?ooo`008@3oool010000000oooo0?ooo`00003k0?ooo`008@3oool010000000oooo0?ooo`00003k
0?ooo`008@3oool2000000030?ooo`000000oooo0?X0oooo000Q0?ooo`040000003oool0oooo0000
0?/0oooo000Q0?ooo`040000003oool0oooo00000?/0oooo00070?ooo`@000000P3oool4000000<0
oooo00<000000?ooo`3oool00P3oool3000000D0oooo00@000000?ooo`3oool00000n`3oool000T0
oooo00D000000?ooo`3oool0oooo0000000>0?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool00000n`3oool000T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo0080oooo0P0000000`3oool000000?ooo`3j0?ooo`0000<0oooo00000000
00000P0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01`3oool3000000D0
oooo00@000000?ooo`3oool00000n`3oool000T0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00@000000?ooo`3oool00000n`3oool000L0
oooo0`0000030?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool01000
0000oooo0?ooo`00003k0?ooo`002@3oool00`000000oooo0?ooo`020?ooo`8000002@3oool40000
00@0oooo0P0000000`3oool000000?ooo`3j0?ooo`008@3oool010000000oooo0?ooo`00003k0?oo
o`008@3oool010000000oooo0?ooo`00003k0?ooo`008@3oool010000000oooo0?ooo`00003k0?oo
o`008@3oool010000000oooo0?ooo`00003k0?ooo`008@3oool2000000030?ooo`000000oooo0?X0
oooo000Q0?ooo`040000003oool0oooo00000?/0oooo000Q0?ooo`040000003oool0oooo00000?/0
oooo000Q0?ooo`040000003oool0oooo00000?/0oooo000Q0?ooo`040000003oool0oooo00000?/0
oooo000Q0?ooo`80000000<0oooo0000003oool0nP3oool00240oooo00@000000?ooo`3oool00000
n`3oool00240oooo00@000000?ooo`3oool00000n`3oool00240oooo00@000000?ooo`3oool00000
n`3oool00240oooo00@000000?ooo`3oool00000n`3oool00240oooo0P0000000`3oool000000?oo
o`3j0?ooo`008@3oool010000000oooo0?ooo`00003k0?ooo`004`3oool4000000<0oooo0P000005
0?ooo`040000003oool0oooo00000?/0oooo000E0?ooo`050000003oool0oooo0?ooo`0000000P3o
ool00`000000oooo0?ooo`020?ooo`040000003oool0oooo00000?/0oooo000E0?ooo`050000003o
ool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`020?ooo`80000000<0oooo0000003oool0
nP3oool000d0oooo100000040?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo
0?ooo`020?ooo`040000003oool0oooo00000?/0oooo000E0?ooo`050000003oool0oooo0?ooo`00
00000P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`000000nP3oool001<0
oooo0`0000030?ooo`040000003oool0oooo000000@0oooo00D000000?ooo`3oool0oooo0000003j
0?ooo`005@3oool00`000000oooo0?ooo`020?ooo`8000001@3oool01@000000oooo0?ooo`3oool0
00000?X0oooo000Q0?ooo`8000000P3oool00`000000oooo0?ooo`3h0?ooo`008@3oool01@000000
oooo0?ooo`3oool000000?X0oooo000Q0?ooo`050000003oool0oooo0?ooo`000000nP3oool00240
oooo00D000000?ooo`3oool0oooo0000003j0?ooo`008@3oool01@000000oooo0?ooo`3oool00000
0?X0oooo000Q0?ooo`8000000P3oool00`000000oooo0?ooo`3h0?ooo`008@3oool01@000000oooo
0?ooo`3oool000000?X0oooo000Q0?ooo`050000003oool0oooo0?ooo`000000nP3oool00240oooo
00D000000?ooo`3oool0oooo0000003j0?ooo`008@3oool01@000000oooo0?ooo`3oool000000?X0
oooo000Q0?ooo`8000000P3oool00`000000oooo0?ooo`3h0?ooo`008@3oool01@000000oooo0?oo
o`3oool000000?X0oooo000Q0?ooo`050000003oool0oooo0?ooo`000000nP3oool00240oooo00D0
00000?ooo`3oool0oooo0000003j0?ooo`008@3oool01@000000oooo0?ooo`3oool000000?X0oooo
000Q0?ooo`8000000P3oool00`000000oooo0?ooo`3h0?ooo`008@3oool01@000000oooo0?ooo`3o
ool000000?X0oooo000>0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool3
000000D0oooo00D000000?ooo`3oool0oooo0000003j0?ooo`003P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0oooo0000003j0?ooo`003`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo0P0000020?ooo`030000003oool0
oooo0?P0oooo00070?ooo`@00000103oool00`000000oooo0?ooo`070?ooo`<000001@3oool01@00
0000oooo0?ooo`3oool000000?X0oooo000@0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01@3oool01@000000oooo0?ooo`3oool000000?X0oooo000=0?ooo`040000003oool0oooo
000000P0oooo00<000000?ooo`3oool01@3oool01@000000oooo0?ooo`3oool000000?X0oooo000=
0?ooo`@00000203oool4000000@0oooo00D000000?ooo`3oool0oooo0000003j0?ooo`008@3oool2
00000080oooo00<000000?ooo`3oool0n03oool00240oooo00D000000?ooo`3oool0oooo0000003j
0?ooo`008@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?L0oooo000Q0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0m`3oool00240oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`3g0?ooo`008@3oool2000000<0oooo00<000000?ooo`3oool0m`3o
ool00240oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3g0?ooo`008@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0?L0oooo000Q0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0m`3oool00240oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`3g0?ooo`008@3oool2000000<0oooo00<000000?ooo`3oool0m`3oool00240oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`3g0?ooo`008@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0?L0oooo000Q0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0m`3oool00240oooo0P0000030?ooo`030000003oool0oooo0?L0oooo000Q0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0m`3oool00240oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`3g0?ooo`006@3oool3000000D0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`3g0?ooo`00703oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0m`3oool001`0oooo00<000000?ooo`3oool00P3oool2000000<0oooo
00<000000?ooo`3oool0m`3oool001<0oooo100000020?ooo`<000001@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0?L0oooo000I0?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`3g0?ooo`006@3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0m`3oool001T0oooo100000040?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0m`3oool00240oooo0P0000030?ooo`030000
003oool0oooo0?L0oooo000Q0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0m`3o
ool00240oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3f0?ooo`008@3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0?H0oooo000Q0?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0mP3oool00240oooo0P0000040?ooo`030000003oool0oooo0?H0oooo
000Q0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0mP3oool00240oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`3f0?ooo`008@3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0?H0oooo000Q0?ooo`800000103oool00`000000oooo0?ooo`3f0?ooo`00
8@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?H0oooo000Q0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0mP3oool00240oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`3f0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
0?H0oooo000Q0?ooo`800000103oool00`000000oooo0?ooo`3f0?ooo`008@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0?H0oooo000Q0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0mP3oool000d0oooo100000030?ooo`030000003oool0oooo0080oooo0`000005
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0mP3oool000d0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0mP3oool000h0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`020?ooo`80
00001@3oool00`000000oooo0?ooo`3e0?ooo`001`3oool4000000@0oooo00<000000?ooo`3oool0
1`3oool3000000D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3e0?ooo`00403o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`3e0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3e0?ooo`003P3oool2
000000T0oooo100000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0m@3oool0
0240oooo0P0000060?ooo`030000003oool0oooo0?@0oooo000Q0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0m03oool00240oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`3d0?ooo`008@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?@0oooo000Q
0?ooo`8000001P3oool00`000000oooo0?ooo`3d0?ooo`008@3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0?@0oooo000Q0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0m03oool00240oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3d0?ooo`008@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01h0oooo0P0000040?ooo`030000003o
ool0oooo0080oooo1000000S0?ooo`800000103oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0240oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P00000T0?ooo`800000103o
ool00`000000oooo0?ooo`030?ooo`800000:03oool4000000H0oooo000Q0?ooo`8000001`3oool0
0`000000oooo0?ooo`0M0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool08`3o
ool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`00000R0?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000:@3oool00`000000oooo0?ooo`050?ooo`008@3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo01`0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?oo
o`0R0?ooo`040000003oool0oooo000000L0oooo1@00000Q0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo000002T0oooo00<000000?ooo`3oool01@3oool00240oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0L0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool08@3o
ool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo00000280oooo00@000000?ooo`3o
ool00000203oool3000002<0oooo00@000000?ooo`3oool000002@3oool2000002X0oooo00<00000
0?ooo`3oool01@3oool00240oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0L0?oo
o`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0803oool010000000oooo0?ooo`00
00080?ooo`030000003oool000000280oooo00@000000?ooo`3oool00000203oool00`000000oooo
0?ooo`0S0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000:@3oool00`00
0000oooo0?ooo`050?ooo`008@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01`0
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000R0?ooo`040000003oool0
oooo000000T0oooo0P00000R0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0
8`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002L0oooo0`0000070?oo
o`008@3oool2000000T0oooo00<000000?ooo`3oool0703oool2000000X0oooo0P00000T0?ooo`80
00002`3oool00`000000oooo0?ooo`0Q0?ooo`8000002P3oool3000002<0oooo0P00000:0?ooo`80
0000:P3oool00`000000oooo0?ooo`050?ooo`008@3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo0?40oooo000Q0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0l03o
ool00240oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3`0?ooo`008@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo0>l0oooo000K0?ooool000001P0000000240oooo
00<000000?ooo`3oool02@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00D0
oooo000Q0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0k03oool00240oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`3[0?ooo`008@3oool200000100oooo1000003Y
0?ooo`008@3oool00`000000oooo0?ooo`0C0?ooo`800000i`3oool00240oooo00<000000?ooo`3o
ool05@3oool700000>00oooo000Q0?ooo`030000003oool0oooo01`0oooo4P00003>0?ooo`008@3o
ool00`000000oooo0?ooo`0^0?oooa@00000^P3oool00240oooo0P0000130?oooal00000V`3oool0
0240oooo00<000000?ooo`3oool0H@3ooolN000007d0oooo000Q0?ooo`030000003oool0oooo07l0
oooo5@00001X0?ooo`008@3oool00`000000oooo0?ooo`2D0?oooa@00000E03oool00240oooo00<0
00000?ooo`3oool0Z03ooolE000003l0oooo000Q0?ooo`800000_P3oool:000003D0oooo000Q0?oo
o`030000003oool0oooo0<L0oooo5000000Q0?ooo`008@3oool00`000000oooo0?ooo`3K0?ooo`X0
00005`3oool00240oooo00<000000?ooo`3oool0i@3oool:000000d0oooo000Q0?ooo`030000003o
ool0oooo0>l0oooo1P0000070?ooo`008@3oool200000?d0oooo000Q0?ooo`030000003oool0oooo
0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo003o0?ooob40oooo003o0?ooob40oooo003o
0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o
0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.134313, -15.4498, \
0.00403944, 0.104996}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      klim2[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\), 1], {th, Pi/4, 
        Pi/3}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[7/10, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.1935518913647979`\)\(\[InvisibleSpace]\)\) - 
      1.3008519672126527`*^-54\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[71/100, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.18343871058733655`\)\(\[InvisibleSpace]\)\) - 
      2.8864187027194525`*^-54\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      klim2arg[IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), 71/100, 1], {k, 
        0.01, 0.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 3.1746 0.601606 0.00268953 [
[.18254 .58911 -12 -9 ]
[.18254 .58911 12 0 ]
[.34127 .58911 -9 -9 ]
[.34127 .58911 9 0 ]
[.5 .58911 -12 -9 ]
[.5 .58911 12 0 ]
[.65873 .58911 -9 -9 ]
[.65873 .58911 9 0 ]
[.81746 .58911 -12 -9 ]
[.81746 .58911 12 0 ]
[.97619 .58911 -9 -9 ]
[.97619 .58911 9 0 ]
[.01131 .0637 -24 -4.5 ]
[.01131 .0637 0 4.5 ]
[.01131 .19818 -24 -4.5 ]
[.01131 .19818 0 4.5 ]
[.01131 .33265 -24 -4.5 ]
[.01131 .33265 0 4.5 ]
[.01131 .46713 -18 -4.5 ]
[.01131 .46713 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .60161 m
.18254 .60786 L
s
[(0.05)] .18254 .58911 0 1 Mshowa
.34127 .60161 m
.34127 .60786 L
s
[(0.1)] .34127 .58911 0 1 Mshowa
.5 .60161 m
.5 .60786 L
s
[(0.15)] .5 .58911 0 1 Mshowa
.65873 .60161 m
.65873 .60786 L
s
[(0.2)] .65873 .58911 0 1 Mshowa
.81746 .60161 m
.81746 .60786 L
s
[(0.25)] .81746 .58911 0 1 Mshowa
.97619 .60161 m
.97619 .60786 L
s
[(0.3)] .97619 .58911 0 1 Mshowa
.125 Mabswid
.05556 .60161 m
.05556 .60536 L
s
.0873 .60161 m
.0873 .60536 L
s
.11905 .60161 m
.11905 .60536 L
s
.15079 .60161 m
.15079 .60536 L
s
.21429 .60161 m
.21429 .60536 L
s
.24603 .60161 m
.24603 .60536 L
s
.27778 .60161 m
.27778 .60536 L
s
.30952 .60161 m
.30952 .60536 L
s
.37302 .60161 m
.37302 .60536 L
s
.40476 .60161 m
.40476 .60536 L
s
.43651 .60161 m
.43651 .60536 L
s
.46825 .60161 m
.46825 .60536 L
s
.53175 .60161 m
.53175 .60536 L
s
.56349 .60161 m
.56349 .60536 L
s
.59524 .60161 m
.59524 .60536 L
s
.62698 .60161 m
.62698 .60536 L
s
.69048 .60161 m
.69048 .60536 L
s
.72222 .60161 m
.72222 .60536 L
s
.75397 .60161 m
.75397 .60536 L
s
.78571 .60161 m
.78571 .60536 L
s
.84921 .60161 m
.84921 .60536 L
s
.88095 .60161 m
.88095 .60536 L
s
.9127 .60161 m
.9127 .60536 L
s
.94444 .60161 m
.94444 .60536 L
s
.25 Mabswid
0 .60161 m
1 .60161 L
s
.02381 .0637 m
.03006 .0637 L
s
[(-200)] .01131 .0637 1 0 Mshowa
.02381 .19818 m
.03006 .19818 L
s
[(-150)] .01131 .19818 1 0 Mshowa
.02381 .33265 m
.03006 .33265 L
s
[(-100)] .01131 .33265 1 0 Mshowa
.02381 .46713 m
.03006 .46713 L
s
[(-50)] .01131 .46713 1 0 Mshowa
.125 Mabswid
.02381 .0906 m
.02756 .0906 L
s
.02381 .11749 m
.02756 .11749 L
s
.02381 .14439 m
.02756 .14439 L
s
.02381 .17128 m
.02756 .17128 L
s
.02381 .22507 m
.02756 .22507 L
s
.02381 .25197 m
.02756 .25197 L
s
.02381 .27886 m
.02756 .27886 L
s
.02381 .30576 m
.02756 .30576 L
s
.02381 .35955 m
.02756 .35955 L
s
.02381 .38644 m
.02756 .38644 L
s
.02381 .41334 m
.02756 .41334 L
s
.02381 .44023 m
.02756 .44023 L
s
.02381 .49402 m
.02756 .49402 L
s
.02381 .52092 m
.02756 .52092 L
s
.02381 .54782 m
.02756 .54782 L
s
.02381 .57471 m
.02756 .57471 L
s
.02381 .0368 m
.02756 .0368 L
s
.02381 .00991 m
.02756 .00991 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.06138 0 m
.06423 .08632 L
.06921 .19359 L
.07374 .26465 L
.07846 .32079 L
.08359 .3673 L
.0929 .42681 L
.09779 .44947 L
.10318 .46974 L
.10843 .48589 L
.11408 .5002 L
.12418 .52004 L
.13363 .53388 L
.1434 .54487 L
.15406 .55415 L
.16425 .56113 L
.17348 .56626 L
.1925 .57429 L
.21277 .58033 L
.23138 .58439 L
.25151 .58771 L
.27078 .59016 L
.29114 .59219 L
.32785 .59488 L
.34671 .59592 L
.36692 .59686 L
.40691 .59828 L
.44542 .5993 L
.48248 .60004 L
.5219 .60067 L
.55986 .60114 L
.59635 .60152 L
.63521 .60185 L
.67261 .60211 L
.71238 .60235 L
.75068 .60255 L
.78752 .60271 L
.82673 .60287 L
.86447 .603 L
.90459 .60313 L
.94324 .60323 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?P0
oooo000K0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0n03oool001/0oooo0P00
00080?ooo`030000003oool0oooo0?P0oooo000K0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool0n03oool001/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3h0?oo
o`006`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?P0oooo000K0?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0n03oool001/0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`3h0?ooo`006`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo0?P0oooo000K0?ooo`800000203oool00`000000oooo0?ooo`3h0?ooo`006`3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo0?P0oooo000K0?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0n03oool001/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`3h0?ooo`006`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?P0oooo00070?oo
o`@000000`3oool2000000@0oooo0P0000050?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0n03oool000L0oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo000000@0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3h
0?ooo`00203oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo00000080oooo00@00000
0?ooo`3oool00000103oool2000000P0oooo00<000000?ooo`3oool0n03oool000030?ooo`000000
000000800000103oool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
n03oool000X0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3h0?ooo`00
1`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?P0oooo00080?oo
o`800000103oool2000000@0oooo0P0000050?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0n03oool001/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3h0?ooo`00
6`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?P0oooo000K0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0n03oool001/0oooo0P0000080?ooo`030000003oool0
oooo0?P0oooo000K0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0n03oool001/0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3h0?ooo`006`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo0?P0oooo000K0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0n03oool001/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3h
0?ooo`006`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?L0oooo000K0?ooo`80
00002@3oool00`000000oooo0?ooo`3g0?ooo`006`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo0?L0oooo000K0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0m`3o
ool001/0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3g0?ooo`006`3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo0?L0oooo000K0?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool0m`3oool001/0oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`3g0?ooo`006`3oool2000000T0oooo00<000000?ooo`3oool0m`3oool001/0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`3g0?ooo`006`3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0?L0oooo000K0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool0m`3oool001/0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3g0?ooo`006`3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?L0oooo000K0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0mP3oool001/0oooo0P00000:0?ooo`030000003oool0oooo
0?H0oooo000K0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0mP3oool001/0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3f0?ooo`006`3oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo0?H0oooo000K0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0mP3oool000L0oooo100000020?ooo`<00000103oool2000000D0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`3f0?ooo`002@3oool00`000000oooo0?ooo`040?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo0?H0oooo00090?ooo`030000003oool0oooo00@0oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`020?ooo`8000002P3oool00`000000oooo0?ooo`3f0?oo
o`0000<0oooo0000000000000P0000040?ooo`040000003oool0oooo0?ooo`<000000`3oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0mP3oool0
00T0oooo00D000000?ooo`3oool0oooo000000050?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool02@3oool00`000000oooo0?ooo`3f0?ooo`001`3oool3000000<0oooo00<00000
0?ooo`3oool00`3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0mP3oool000T0oooo00@000000?ooo`3oool0oooo100000030?ooo`8000001@3o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?H0oooo000K0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0mP3oool001/0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`3f0?ooo`006`3oool2000000X0oooo00<000000?ooo`3oool0mP3oool001/0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3f0?ooo`006`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0?D0oooo000K0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool0m@3oool001/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3e0?oo
o`006`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?D0oooo000K0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool0m@3oool001/0oooo0P00000;0?ooo`030000003o
ool0oooo0?D0oooo000K0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0m@3oool0
01/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3e0?ooo`006`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo0?D0oooo000K0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool0m@3oool001/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`3e0?ooo`006`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?D0oooo000K0?oo
o`8000002`3oool00`000000oooo0?ooo`3e0?ooo`006`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo0?D0oooo000K0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
m@3oool001/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3e0?ooo`006`3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?@0oooo000K0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0m03oool001/0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`3d0?ooo`006`3oool2000000`0oooo00<000000?ooo`3oool0m03oool001/0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`3d0?ooo`006`3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0?@0oooo000K0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0m03oool001/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3d0?ooo`00
1`3oool4000000<0oooo0P0000040?ooo`8000001@3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo0?@0oooo00090?ooo`050000003oool0oooo0?ooo`0000000P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`3d0?ooo`002@3oool01@000000oooo0?ooo`3oool000000080oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`020?ooo`800000303oool00`000000oooo0?ooo`3d0?oo
o`0000<0oooo0000000000000P0000040?ooo`050000003oool0oooo0?ooo`0000000P3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`3c0?ooo`002@3oool01@000000oooo0?ooo`3oool000000080oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool0l`3oool000L0oooo0`0000030?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool00000103oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?<0oooo
00090?ooo`030000003oool0oooo0080oooo0P0000040?ooo`8000001@3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo0?<0oooo000K0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool0l`3oool001/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3c0?oo
o`006`3oool2000000h0oooo00<000000?ooo`3oool0lP3oool001/0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`3b0?ooo`006`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo0?80oooo000K0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0lP3oool0
01/0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3b0?ooo`006`3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo0?80oooo000K0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0lP3oool001/0oooo0P00000?0?ooo`030000003oool0oooo0?40oooo000K
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0l@3oool001/0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`3a0?ooo`006`3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo0?40oooo000K0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
l@3oool001/0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3a0?ooo`006`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0?40oooo000K0?ooo`8000003`3oool00`00
0000oooo0?ooo`3a0?ooo`006`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0?00
oooo000K0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0l03oool001/0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`3`0?ooo`006`3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo0?00oooo000K0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0l03oool001/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3`0?ooo`00
6`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0?00oooo000K0?ooo`8000004@3o
ool00`000000oooo0?ooo`3_0?ooo`006`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0>l0oooo000K0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0k`3oool001/0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3_0?ooo`006`3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0>h0oooo000=0?ooo`<00000103oool2000000D0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`3^0?ooo`00403oool010000000oooo0?ooo`00
00020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`3]0?ooo`00403oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo0P00
000C0?ooo`030000003oool0oooo0>d0oooo00070?ooo`@000000P3oool3000000<0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0>d0oooo000=
0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo0>d0oooo000=0?ooo`030000003oool0oooo00<0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0>`0oooo000=
0?ooo`@000000`3oool2000000D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3/
0?ooo`006`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0>`0oooo000K0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool0j`3oool001/0oooo0P00000E0?ooo`030000
003oool0oooo0>/0oooo000K0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0jP3o
ool001/0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3Z0?ooo`006`3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo0>T0oooo000K0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool0j@3oool001/0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`3Y0?ooo`006`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0>P0oooo000K
0?ooo`800000603oool00`000000oooo0?ooo`3X0?ooo`006`3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo0>L0oooo000K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool0iP3oool001/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3U0?ooo`006`3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0>@0oooo000K0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool0i03oool001/0oooo00<000000?ooo`3oool0703oool00`00
0000oooo0?ooo`3S0?ooo`006`3oool2000001h0oooo00<000000?ooo`3oool0hP3oool001/0oooo
00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`3Q0?ooo`006`3oool00`000000oooo0?oo
o`0O0?ooo`800000h@3oool001/0oooo00<000000?ooo`3oool08@3oool200000=l0oooo000K0?oo
o`030000003oool0oooo01d0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?oo
o`<00000603oool2000000@0oooo00<000000?ooo`3oool00P3oool4000001P0oooo0P0000040?oo
o`030000003oool0oooo0080oooo100000020?ooo`<000006@3oool2000000@0oooo00<000000?oo
o`3oool00P3oool4000001P0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?oo
o`<000006@3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo000K0?ooo`03
0000003oool0oooo01`0oooo00@000000?ooo`3oool00000103oool200000080oooo00@000000?oo
o`3oool000001@3oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo000000X0oooo00<0
00000?ooo`3oool05P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool05@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo01P0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
01D0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`006`3oool0
0`000000oooo0?ooo`0L0?ooo`040000003oool0oooo000000H0oooo0`0000020?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo01H0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo01D0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`0G0?oo
o`040000003oool0oooo000000T0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0E
0?ooo`040000003oool0oooo000000/0oooo0@0000010?ooo`40oooo000K0?ooo`8000007@3oool0
10000000oooo0?ooo`0000080?ooo`@000000P3oool3000001L0oooo00@000000?ooo`3oool00000
2P3oool00`000000oooo0?ooo`0F0?ooo`040000003oool0oooo000000X0oooo00@000000?ooo`3o
ool0oooo0`00000H0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool05P3oool0
10000000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<00000603oool010000000oooo
0?ooo`0000090?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0703oool010000000oooo
0?ooo`0000080?ooo`030000003oool0oooo00D00000603oool010000000oooo0?ooo`00000:0?oo
o`030000003oool0oooo01H0oooo00@000000?ooo`3oool000002P3oool01@000000oooo0?ooo`3o
ool0000001X0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0E0?ooo`040000
003oool0oooo000000/0oooo00@000000?ooo`3oool000006P3oool010000000oooo0?ooo`00000;
0?ooo`4000000@3oool10?ooo`006`3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo00000000000F0?ooo`040000
003oool0oooo000000P0oooo0`00000H0?ooo`040000003oool0oooo000000P0oooo0`0000030?oo
o`030000003oool0oooo01P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
000G0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`0H0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
01/0oooo00<000000?ooo`3oool07@3oool2000000X0oooo0P0000030?ooo`h000003@3oool20000
00/0oooo00<000000?ooo`3oool05`3oool2000000/0oooo00@000000?ooo`3oool0oooo1000000H
0?ooo`8000002P3oool2000001T0oooo0P00000:0?ooo`8000000`3oool4000001P0oooo0P00000:
0?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0?03oool500000<40oooo000K0?ooo`03
0000003oool0oooo0440oooo3`00002b0?ooo`006`3oool00`000000oooo0?ooo`1@0?ooob400000
T@3oool001D0ooooo`000009000000<0oooo000K0?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`D000002@3oool001/0oooo00<000000?oo
o`3oool09`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool09`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool01`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo
0?ooo`3o0?ooo`<0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40
oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0326577, -233.745, \
0.00119662, 1.41243}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%%, PlotRange \[Rule] {\(-0.01\), 0.01}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 3.1746 0.309017 30.9017 [
[.18254 .29652 -12 -9 ]
[.18254 .29652 12 0 ]
[.34127 .29652 -9 -9 ]
[.34127 .29652 9 0 ]
[.5 .29652 -12 -9 ]
[.5 .29652 12 0 ]
[.65873 .29652 -9 -9 ]
[.65873 .29652 9 0 ]
[.81746 .29652 -12 -9 ]
[.81746 .29652 12 0 ]
[.97619 .29652 -9 -9 ]
[.97619 .29652 9 0 ]
[.01131 0 -30 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .07725 -42 -4.5 ]
[.01131 .07725 0 4.5 ]
[.01131 .15451 -36 -4.5 ]
[.01131 .15451 0 4.5 ]
[.01131 .23176 -42 -4.5 ]
[.01131 .23176 0 4.5 ]
[.01131 .38627 -36 -4.5 ]
[.01131 .38627 0 4.5 ]
[.01131 .46353 -30 -4.5 ]
[.01131 .46353 0 4.5 ]
[.01131 .54078 -36 -4.5 ]
[.01131 .54078 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .30902 m
.18254 .31527 L
s
[(0.05)] .18254 .29652 0 1 Mshowa
.34127 .30902 m
.34127 .31527 L
s
[(0.1)] .34127 .29652 0 1 Mshowa
.5 .30902 m
.5 .31527 L
s
[(0.15)] .5 .29652 0 1 Mshowa
.65873 .30902 m
.65873 .31527 L
s
[(0.2)] .65873 .29652 0 1 Mshowa
.81746 .30902 m
.81746 .31527 L
s
[(0.25)] .81746 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(0.3)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.05556 .30902 m
.05556 .31277 L
s
.0873 .30902 m
.0873 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.15079 .30902 m
.15079 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.24603 .30902 m
.24603 .31277 L
s
.27778 .30902 m
.27778 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.37302 .30902 m
.37302 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.43651 .30902 m
.43651 .31277 L
s
.46825 .30902 m
.46825 .31277 L
s
.53175 .30902 m
.53175 .31277 L
s
.56349 .30902 m
.56349 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.62698 .30902 m
.62698 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.72222 .30902 m
.72222 .31277 L
s
.75397 .30902 m
.75397 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.84921 .30902 m
.84921 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.9127 .30902 m
.9127 .31277 L
s
.94444 .30902 m
.94444 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 0 m
.03006 0 L
s
[(-0.01)] .01131 0 1 0 Mshowa
.02381 .07725 m
.03006 .07725 L
s
[(-0.0075)] .01131 .07725 1 0 Mshowa
.02381 .15451 m
.03006 .15451 L
s
[(-0.005)] .01131 .15451 1 0 Mshowa
.02381 .23176 m
.03006 .23176 L
s
[(-0.0025)] .01131 .23176 1 0 Mshowa
.02381 .38627 m
.03006 .38627 L
s
[(0.0025)] .01131 .38627 1 0 Mshowa
.02381 .46353 m
.03006 .46353 L
s
[(0.005)] .01131 .46353 1 0 Mshowa
.02381 .54078 m
.03006 .54078 L
s
[(0.0075)] .01131 .54078 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.01)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .01545 m
.02756 .01545 L
s
.02381 .0309 m
.02756 .0309 L
s
.02381 .04635 m
.02756 .04635 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .10816 m
.02756 .10816 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .13906 m
.02756 .13906 L
s
.02381 .16996 m
.02756 .16996 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .20086 m
.02756 .20086 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26266 m
.02756 .26266 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .29357 m
.02756 .29357 L
s
.02381 .32447 m
.02756 .32447 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .35537 m
.02756 .35537 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .41717 m
.02756 .41717 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .44807 m
.02756 .44807 L
s
.02381 .47898 m
.02756 .47898 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .50988 m
.02756 .50988 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57168 m
.02756 .57168 L
s
.02381 .58713 m
.02756 .58713 L
s
.02381 .60258 m
.02756 .60258 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.60369 0 m
.61002 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00503oool2
000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo1000003g0?ooo`004`3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0mP3o
ool001<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo00<0oooo0P00002;0?ooo`030000003oool0oooo06<0oooo000=0?ooo`@000000P3o
ool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo06<0oooo000C0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo08X0oooo00<000000?ooo`3oool0H`3oool001<0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000020?ooo`<000001@3oool00`000000oooo0?ooo`2:
0?ooo`030000003oool0oooo06<0oooo000D0?ooo`8000002P3oool2000000D0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo06<0oooo000]0?ooo`03
0000003oool0oooo08X0oooo00<000000?ooo`3oool0H`3oool002d0oooo00<000000?ooo`3oool0
RP3oool00`000000oooo0?ooo`1S0?ooo`00;@3oool00`000000oooo0?ooo`2:0?ooo`030000003o
ool0oooo06<0oooo000]0?ooo`030000003oool0oooo08X0oooo00<000000?ooo`3oool0H`3oool0
02d0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`1S0?ooo`00;@3oool00`000000
oooo0?ooo`2:0?ooo`030000003oool0oooo06<0oooo000]0?ooo`030000003oool0oooo08X0oooo
00<000000?ooo`3oool0H`3oool002d0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?oo
o`1S0?ooo`00;@3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo06<0oooo000]0?oo
o`030000003oool0oooo08X0oooo00<000000?ooo`3oool0H`3oool002d0oooo00<000000?ooo`3o
ool0RP3oool00`000000oooo0?ooo`1S0?ooo`00;@3oool00`000000oooo0?ooo`2:0?ooo`030000
003oool0oooo06<0oooo00080?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000103o
ool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0RP3o
ool00`000000oooo0?ooo`1S0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`1S0?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo0P00002;
0?ooo`030000003oool0oooo06<0oooo00000`3oool000000000000200000080oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo
00@000000?ooo`3oool0oooo0`0000050?ooo`030000003oool0oooo08X0oooo00<000000?ooo`3o
ool0H`3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo000000D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?oo
o`2:0?ooo`030000003oool0oooo06<0oooo00070?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo
06<0oooo00080?ooo`8000002P3oool2000000@0oooo0P0000030?ooo`@000000P3oool4000000@0
oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`1S0?ooo`00;@3oool00`000000oooo
0?ooo`2:0?ooo`030000003oool0oooo06<0oooo000]0?ooo`030000003oool0oooo08X0oooo00<0
00000?ooo`3oool0H`3oool002d0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`1S
0?ooo`00;@3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo06<0oooo000]0?ooo`03
0000003oool0oooo08X0oooo00<000000?ooo`3oool0H`3oool002d0oooo00<000000?ooo`3oool0
RP3oool00`000000oooo0?ooo`1S0?ooo`00;@3oool00`000000oooo0?ooo`2:0?ooo`030000003o
ool0oooo06<0oooo000]0?ooo`030000003oool0oooo08X0oooo00<000000?ooo`3oool0H`3oool0
02d0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`1S0?ooo`00;@3oool00`000000
oooo0?ooo`2:0?ooo`030000003oool0oooo06<0oooo000]0?ooo`030000003oool0oooo08X0oooo
00<000000?ooo`3oool0H`3oool002d0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?oo
o`1S0?ooo`003P3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P000003
0?ooo`<000001@3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo06<0oooo000=0?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?oo
o`0000050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0RP3oool00`000000oooo
0?ooo`1S0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080
oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`800000R`3oool00`00
0000oooo0?ooo`1S0?ooo`001`3oool400000080oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo0`0000050?ooo`030000003o
ool0oooo08/0oooo00<000000?ooo`3oool0HP3oool000d0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo000=0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R
0?ooo`003P3oool2000000X0oooo0P0000040?ooo`8000000`3oool4000000@0oooo00<000000?oo
o`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`03
0000003oool0oooo0680oooo000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0
HP3oool002d0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3oool0
0`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo000]0?ooo`030000003oool0oooo
08/0oooo00<000000?ooo`3oool0HP3oool002d0oooo00<000000?ooo`3oool0R`3oool00`000000
oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo
000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool002d0oooo00<00000
0?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?oo
o`030000003oool0oooo0680oooo000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3o
ool0HP3oool002d0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`00203o
ool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0oooo0P0000030?ooo`@000000P3o
ool3000000D0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`001`3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0R`3oool00`000000oooo0?ooo`1R0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo00000080oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo0080oooo0P00002<0?ooo`030000003oool0oooo0680oooo00000`3o
ool000000000000200000080oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0oooo0`0000050?ooo`03
0000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool000L0oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo00@00000
0?ooo`3oool000001`3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo0007
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo00080?ooo`8000002P3oool2000000@0
oooo0P0000040?ooo`8000000`3oool4000000@0oooo00<000000?ooo`3oool0R`3oool00`000000
oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo
000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool002d0oooo00<00000
0?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?oo
o`030000003oool0oooo0680oooo000]0?ooo`030000003oool0oooo01T0oooo0P0000040?ooo`03
0000003oool0oooo00<0oooo0P0000030?ooo`<000005P3oool2000000@0oooo00<000000?ooo`3o
ool00P3oool4000001D0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?ooo`<0
0000403oool00`000000oooo0?ooo`030?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@0
00005@3oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`00000F0?ooo`80
0000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool002d0oooo00<000000?ooo`3oool0
603oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?oo
o`3oool04P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo01<0oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo01D0oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0180
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`00;@3oool00`00
0000oooo0?ooo`0H0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3o
ool00`000000oooo0?ooo`0B0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
4`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000T0oooo00<000000?oo
o`3oool0503oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool04P3oool010000000oooo0?ooo`00000;0?ooo`4000000@3oool10?ooo`00;@3oool0
0`000000oooo0?ooo`0H0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool3000001D0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0C0?ooo`04
0000003oool0oooo000000X0oooo00@000000?ooo`3oool0oooo0`00000@0?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0C0?ooo`040000003o
ool0oooo000000X0oooo00@000000?ooo`3oool0oooo0`00000E0?ooo`040000003oool0oooo0000
00T0oooo0P0000030?ooo`00;@3oool00`000000oooo0?ooo`0H0?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0
oooo000000X0oooo00<000000?ooo`3oool04`3oool010000000oooo0?ooo`00000:0?ooo`050000
003oool0oooo0?ooo`0000004P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo0000
00/0oooo00<000000?ooo`3oool04P3oool010000000oooo0?ooo`00000;0?ooo`040000003oool0
oooo000001L0oooo00@000000?ooo`3oool000002`3oool100000040oooo0@3oool002d0oooo00<0
00000?ooo`3oool0603oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool05@3oool010000000oooo0?ooo`0000080?ooo`<000005@3oool01000
0000oooo0?ooo`0000080?ooo`<000000`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000D0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0E0?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool002d0oooo00<00000
0?ooo`3oool06@3oool2000000X0oooo0P0000030?ooo`@000005@3oool2000000/0oooo00<00000
0?ooo`3oool0503oool2000000/0oooo00@000000?ooo`3oool0oooo1000000?0?ooo`030000003o
ool0oooo00<0oooo0P00000:0?ooo`8000005P3oool2000000X0oooo0P0000030?ooo`@000005@3o
ool2000000X0oooo0P0000030?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0
oooo0680oooo000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool002d0
oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`009`3ooooe000000@0oooo
000]0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00L0oooo000]0?ooo`030000003oool0oooo02<0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02@0oooo00<0
00000?ooo`3oool01`3oool002d0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R
0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo000]0?ooo`03
0000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool002d0oooo00<000000?ooo`3oool0
R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000003o
ool0oooo0680oooo000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool0
02d0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3oool00`000000
oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo000]0?ooo`030000003oool0oooo08/0oooo
00<000000?ooo`3oool0HP3oool002d0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?oo
o`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo000]0?oo
o`030000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool002d0oooo00<000000?ooo`3o
ool0R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000
003oool0oooo0680oooo00080?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000103o
ool2000000<0oooo100000020?ooo`<000001@3oool00`000000oooo0?ooo`2;0?ooo`030000003o
ool0oooo0680oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo00070?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`000003
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool2000008`0oooo00<00000
0?ooo`3oool0HP3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00020?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0oooo0`0000050?ooo`03
0000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool000L0oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo00@00000
0?ooo`3oool000001`3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo0007
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo00080?ooo`8000002P3oool2000000@0
oooo0P0000040?ooo`8000000`3oool4000000@0oooo00<000000?ooo`3oool0R`3oool00`000000
oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo
000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool002d0oooo00<00000
0?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?oo
o`030000003oool0oooo0680oooo000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3o
ool0HP3oool002d0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`00;@3o
ool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680oooo000]0?ooo`030000003oool0
oooo08/0oooo00<000000?ooo`3oool0HP3oool002d0oooo00<000000?ooo`3oool0R`3oool00`00
0000oooo0?ooo`1R0?ooo`00;@3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0680
oooo000]0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0HP3oool002d0oooo00<0
00000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`003P3oool2000000@0oooo00<00000
0?ooo`3oool00`3oool2000000@0oooo0P0000030?ooo`<000001@3oool00`000000oooo0?ooo`2;
0?ooo`030000003oool0oooo0680oooo000=0?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`1R0?ooo`003@3oool010000000oooo0?oo
o`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`00
0000oooo0?ooo`020?ooo`800000S03oool00`000000oooo0?ooo`1R0?ooo`003@3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool000000P3o
ool3000000D0oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`1Q0?ooo`003@3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3o
ool0H@3oool000d0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2<
0?ooo`030000003oool0oooo0640oooo000>0?ooo`8000002P3oool2000000@0oooo0P0000030?oo
o`@00000103oool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo0640oooo000]0?ooo`03
0000003oool0oooo08`0oooo00<000000?ooo`3oool0H@3oool002d0oooo00<000000?ooo`3oool0
S03oool00`000000oooo0?ooo`1Q0?ooo`00;@3oool00`000000oooo0?ooo`2<0?ooo`030000003o
ool0oooo0640oooo000]0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool0H@3oool0
02d0oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`1Q0?ooo`00;@3oool00`000000
oooo0?ooo`2<0?ooo`030000003oool0oooo0640oooo000]0?ooo`030000003oool0oooo08`0oooo
00<000000?ooo`3oool0H@3oool002d0oooo00<000000?ooo`3oool0S03oool00`000000oooo0?oo
o`1Q0?ooo`00;@3oool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo0640oooo000]0?oo
o`030000003oool0oooo08`0oooo00<000000?ooo`3oool0H@3oool002d0oooo00<000000?ooo`3o
ool0S03oool00`000000oooo0?ooo`1Q0?ooo`00;@3oool00`000000oooo0?ooo`2<0?ooo`030000
003oool0oooo0640oooo00080?ooo`800000103oool00`000000oooo0?ooo`030?ooo`800000103o
ool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0S03o
ool00`000000oooo0?ooo`1Q0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`1Q0?ooo`00
1`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo0P00002=
0?ooo`030000003oool0oooo0640oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool000000P3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo0?ooo`<0
00001@3oool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo0640oooo00070?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`000005
0?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0S03oool00`000000oooo0?oo
o`1Q0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`1Q0?ooo`00203oool2000000X0
oooo0P0000040?ooo`8000000`3oool400000080oooo100000040?ooo`030000003oool0oooo08`0
oooo00<000000?ooo`3oool0H@3oool002d0oooo00<000000?ooo`3oool0S03oool00`000000oooo
0?ooo`1Q0?ooo`00;@3oool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo0640oooo000]
0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool0H@3oool002d0oooo00<000000?oo
o`3oool0S03oool00`000000oooo0?ooo`1Q0?ooo`00;@3oool00`000000oooo0?ooo`2<0?ooo`03
0000003oool0oooo0640oooo000]0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool0
H@3oool002d0oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`1Q0?ooo`00;@3oool0
0`000000oooo0?ooo`2<0?ooo`030000003oool0oooo0640oooo000]0?ooo`030000003oool0oooo
08`0oooo00<000000?ooo`3oool0H@3oool002d0oooo00<000000?ooo`3oool0S03oool00`000000
oooo0?ooo`1Q0?ooo`00;@3oool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo0640oooo
000]0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool0H@3oool001@0oooo0P000004
0?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`@00000103oool00`000000oooo0?ooo`2<
0?ooo`030000003oool0oooo0640oooo000C0?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08`0oooo00<0
00000?ooo`3oool0H@3oool001<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo00<0oooo0P00002=0?ooo`030000003oool0oooo0640oooo
000C0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`3f0?ooo`004`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00@0oooo00<000000?ooo`3oool0mP3oool001<0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000020?ooo`<00000n03oool001@0oooo0P00000:0?ooo`8000001@3oool00`00
0000oooo0?ooo`3f0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.05803, -0.0117051, \
0.00128788, 0.000132307}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[1/1000, 1]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(20\) iterations."\)], "Message"],

Cell[BoxData[
    \(\(\(0.49999993560194267`\)\(\[InvisibleSpace]\)\) - 
      2.29212669736516`*^-280\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(alphaspecial[1, 1/2, 1/2\ \ Cos[Pi/16], 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(-0.19489416060969234`\)."\)], "Message"],

Cell[BoxData[
    \(\(-0.2335208395910107263168497298`15.7824\) + 
      0``16.414\ \[ImaginaryI]\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[alomz[1, 1/2*Cos[Pi/16], 1/2, 1]]\)], "Input"],

Cell[BoxData[
    \(\(-0.2335209517648005`\)\)], "Output"]
}, Closed]],

Cell[BoxData[
    \(o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(betaspecial[1, 1/2, 1/2\ \ Cos[Pi/16], 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(21\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \
\!\(-0.19489431381225586`\)."\)], "Message"],

Cell[BoxData[
    \(\(-4.9186281673029626`4.1891*^-8\) + 
      0``11.4972\ \[ImaginaryI]\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[betamoz[1, 1/2*Cos[Pi/16], 1/2, 1]]\)], "Input"],

Cell[BoxData[
    \(\(-4.918829408519437`\) + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(gammaspecial[1, 1/2, 1/2\ \ Cos[Pi/16], 10^\((\(-4\))\), 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(31\) \
recursive bisections in \!\(xi\) near \!\(xi\) = \!\(0.19528647558763623`\)."\
\)], "Message"],

Cell[BoxData[
    \(0.0227337860595265556`4.2652 + 0``5.9085\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[gamoz[1, 1/2*Cos[Pi/16], 1/2, 1]]\)], "Input"],

Cell[BoxData[
    \(\(\(0.022732714797185416`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["full", "Subsubsection"],

Cell[BoxData[
    \(testfuncspec[om_, k_, th_, 
        a_] := \((om^2 - k^2 - 
              Sqrt[1 + a] alphaspecial[1, k, k*Cos[th], om, a] - 
              Sqrt[1 + a] gammaspecial[1, k, k*Cos[th], om, a])\)*\((1 - 
              Sqrt[1 + a] 
                betaspecial[1, k, k*Cos[th], om, a]/om^2)\) - \((1 + a)\) 
          deltaspecial[1, k, k*Cos[th], om, a]^2/om^2*\((1 - Cos[th]^2)\)*
          k^2\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(testfuncspec[I/10000, 50/100, Pi/16, 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(-0.153001084802520054673484389702343`17.7127\) + 
      0``20.9914\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Omegab[om_, k_, th_, a_] := 
      alphaspecial[1, k, k*Cos[th], om, a] + 
        betaspecial[1, k, k*Cos[th], om, a] + 
        gammaspecial[1, k, k*Cos[th], om, a] + k^2\)], "Input"],

Cell[BoxData[
    \(otherterm[om_, k_, th_, 
        a_] := \((alphaspecial[1, k, k*Cos[th], om, a] + 
              gammaspecial[1, k, k*Cos[th], om, a] + k^2)\) 
          betaspecial[1, k, k*Cos[th], om, a] - 
        k^2 \((1 - Cos[th]^2)\) 
          deltaspecial[1, k, k*Cos[th], om, a]^2\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Omegap[om_, k_, th_, a_] := 
      1/2*\((Omegab[om, k, th, a] + 
            Sqrt[Omegab[om, k, th, a]^2 - 
                4\ otherterm[om, k, th, a]])\)\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(Omegap\)\" is similar \
to existing symbol \"\!\(Omegab\)\"."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Omegap[I/10, 2/10, Pi/16, 1]\)], "Input"],

Cell[BoxData[
    \(0.165612182766709379027471603697146`17.3508 + 
      0``18.2006\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Omegam[om_, k_, th_, a_] := 
      1/2*\((Omegab[om, k, th, a] - 
            Sqrt[Omegab[om, k, th, a]^2 - 
                4\ otherterm[om, k, th, a]])\)\)], "Input"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(Omegam\)\" is similar \
to existing symbols \!\({Omegab, Omegap}\)."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Omegam[I/10, 2/10, Pi/16, 1]\)], "Input"],

Cell[BoxData[
    \(0.105660243026522269257473005055986`17.1556 + 
      0``18.2006\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Omegap[I*IntegerPart[om*10^\((16)\)]*10^\((\(-16\))\), 1/10, Pi/16, 
        1], {om, 0.01, 0.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 3.1746 0.0147151 2.1634 [
[.18254 .00222 -12 -9 ]
[.18254 .00222 12 0 ]
[.34127 .00222 -9 -9 ]
[.34127 .00222 9 0 ]
[.5 .00222 -12 -9 ]
[.5 .00222 12 0 ]
[.65873 .00222 -9 -9 ]
[.65873 .00222 9 0 ]
[.81746 .00222 -12 -9 ]
[.81746 .00222 12 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .12288 -24 -4.5 ]
[.01131 .12288 0 4.5 ]
[.01131 .23105 -18 -4.5 ]
[.01131 .23105 0 4.5 ]
[.01131 .33922 -24 -4.5 ]
[.01131 .33922 0 4.5 ]
[.01131 .44739 -18 -4.5 ]
[.01131 .44739 0 4.5 ]
[.01131 .55556 -24 -4.5 ]
[.01131 .55556 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.05)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(0.1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(0.15)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(0.2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(0.25)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(0.3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .12288 m
.03006 .12288 L
s
[(0.05)] .01131 .12288 1 0 Mshowa
.02381 .23105 m
.03006 .23105 L
s
[(0.1)] .01131 .23105 1 0 Mshowa
.02381 .33922 m
.03006 .33922 L
s
[(0.15)] .01131 .33922 1 0 Mshowa
.02381 .44739 m
.03006 .44739 L
s
[(0.2)] .01131 .44739 1 0 Mshowa
.02381 .55556 m
.03006 .55556 L
s
[(0.25)] .01131 .55556 1 0 Mshowa
.125 Mabswid
.02381 .03635 m
.02756 .03635 L
s
.02381 .05798 m
.02756 .05798 L
s
.02381 .07962 m
.02756 .07962 L
s
.02381 .10125 m
.02756 .10125 L
s
.02381 .14452 m
.02756 .14452 L
s
.02381 .16615 m
.02756 .16615 L
s
.02381 .18779 m
.02756 .18779 L
s
.02381 .20942 m
.02756 .20942 L
s
.02381 .25269 m
.02756 .25269 L
s
.02381 .27432 m
.02756 .27432 L
s
.02381 .29596 m
.02756 .29596 L
s
.02381 .31759 m
.02756 .31759 L
s
.02381 .36086 m
.02756 .36086 L
s
.02381 .38249 m
.02756 .38249 L
s
.02381 .40413 m
.02756 .40413 L
s
.02381 .42576 m
.02756 .42576 L
s
.02381 .46903 m
.02756 .46903 L
s
.02381 .49066 m
.02756 .49066 L
s
.02381 .5123 m
.02756 .5123 L
s
.02381 .53393 m
.02756 .53393 L
s
.02381 .5772 m
.02756 .5772 L
s
.02381 .59883 m
.02756 .59883 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .03801 m
.0929 .10534 L
.13363 .2082 L
.17189 .29488 L
.20867 .35738 L
.24783 .40653 L
.28553 .44188 L
.32559 .47052 L
.36419 .49202 L
.40132 .50869 L
.44083 .52333 L
.47887 .53518 L
.51544 .54499 L
.55438 .55406 L
.59186 .56168 L
.63171 .56877 L
.6701 .57475 L
.70702 .57982 L
.74631 .58458 L
.78413 .58863 L
.82433 .59244 L
.86306 .59569 L
.90033 .59848 L
.93996 .60114 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00>`3oool2000000@0oooo00<0
00000?ooo`3oool00`3oool2000000<0oooo0`00000H0?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`@00000603oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`00
000I0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000603oool2000000@0oooo00<0
00000?ooo`3oool00P3oool400000080oooo0`00000I0?ooo`800000103oool00`000000oooo0?oo
o`030?ooo`8000000`3oool003X0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000050?ooo`030000003oool0oooo01@0oooo00@000000?ooo`3oool000002P3oool00`000000
oooo0?ooo`0F0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0E0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0603oool0
10000000oooo0?ooo`0000080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool05@3o
ool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo000j0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`0D0?oo
o`040000003oool0oooo000000X0oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`00
000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool05@3oool010000000oooo0?oo
o`0000090?ooo`030000003oool0oooo01L0oooo00@000000?ooo`3oool000002@3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo01D0oooo00@000000?ooo`3oool000002`3oool10000
0040oooo0@3oool003X0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000002
0?ooo`<000005`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo01H0oooo00@0
00000?ooo`3oool000002P3oool010000000oooo0?ooo`3oool3000001P0oooo00@000000?ooo`3o
ool000002P3oool00`000000oooo0?ooo`0F0?ooo`040000003oool0oooo000000X0oooo00@00000
0?ooo`3oool0oooo0`00000H0?ooo`040000003oool0oooo000000T0oooo0P0000030?ooo`00>P3o
ool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool05`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo01H0oooo00@000000?oo
o`3oool000002P3oool01@000000oooo0?ooo`3oool0000001X0oooo00@000000?ooo`3oool00000
2`3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000/0oooo00@000000?ooo`3o
ool000006P3oool010000000oooo0?ooo`00000;0?ooo`4000000@3oool10?ooo`00>P3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool05`3o
ool010000000oooo0?ooo`0000080?ooo`<00000603oool010000000oooo0?ooo`0000080?ooo`<0
00000`3oool00`000000oooo0?ooo`0H0?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool000005`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool0603oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
0080oooo000k0?ooo`8000002P3oool2000000<0oooo1000000G0?ooo`8000002`3oool00`000000
oooo0?ooo`0G0?ooo`8000002`3oool010000000oooo0?ooo`3oool4000001P0oooo0P00000:0?oo
o`8000006@3oool2000000X0oooo0P0000030?ooo`@00000603oool2000000X0oooo0P0000030?oo
o`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo
0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o
0?ooo`<0oooo000E0?ooool000002@0000030?ooo`006`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`00
6`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3o
ool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool200000?l0oooo103oool001/0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3j0?ooo`006`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0?T0oooo000K0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0n@3oool001/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3h
0?ooo`006`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?P0oooo000K0?ooo`80
00002@3oool00`000000oooo0?ooo`3g0?ooo`006`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo0?L0oooo000K0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0mP3o
ool001/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3f0?ooo`006`3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo0?D0oooo000K0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool0m03oool001/0oooo0P00000<0?ooo`030000003oool0oooo0?@0oooo
000K0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0l`3oool001/0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`3c0?ooo`006`3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo0?80oooo000K0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool0lP3oool001/0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3a0?ooo`006`3o
ool2000000l0oooo00<000000?ooo`3oool0l@3oool001/0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`3`0?ooo`006`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
0?00oooo00020?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool3000000D0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3_0?ooo`0000D0oooo0000003oool0
oooo000000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo0>l0oooo00001@3oool000000?ooo`3oool00000
00P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`8000004P3oool0
0`000000oooo0?ooo`3^0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0
oooo00000080oooo0`0000050?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0kP3o
ool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3^0?ooo`0000D0
oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0>d0oooo00020?ooo`800000
2P3oool2000000<0oooo100000040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
k@3oool001/0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3/0?ooo`006`3oool2
000001@0oooo00<000000?ooo`3oool0k03oool001/0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`3/0?ooo`006`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0>/0
oooo000K0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0j`3oool001/0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`3Z0?ooo`006`3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo0>X0oooo000K0?ooo`8000005`3oool00`000000oooo0?ooo`3Y0?oo
o`006`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0>T0oooo000K0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool0j@3oool001/0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`3X0?ooo`006`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo0>P0oooo000K0?ooo`8000006@3oool00`000000oooo0?ooo`3W0?ooo`006`3oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo0>L0oooo000K0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool0i`3oool001/0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`3V0?ooo`006`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0>H0oooo000K0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0i@3oool001/0oooo0P00000K0?ooo`03
0000003oool0oooo0>D0oooo000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
i@3oool001/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3T0?ooo`006`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0>@0oooo00080?ooo`800000103oool00`00
0000oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
0><0oooo00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo0><0oooo00070?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool00`3oool2000001h0oooo00<000000?ooo`3oool0hP3oool000L0
oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool0hP3oool000L0oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0hP3oool000L0
oooo00@000000?ooo`3oool00000203oool3000000D0oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0?ooo`3Q0?ooo`00203oool2000000/0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo0>40oooo000K0?ooo`800000803oool00`000000oooo
0?ooo`3P0?ooo`006`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0>00oooo000K
0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0g`3oool001/0oooo00<000000?oo
o`3oool0803oool00`000000oooo0?ooo`3O0?ooo`006`3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0=h0oooo000K0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0
gP3oool001/0oooo0P00000R0?ooo`030000003oool0oooo0=h0oooo000K0?ooo`030000003oool0
oooo0280oooo00<000000?ooo`3oool0g@3oool001/0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`3M0?ooo`006`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0=`0
oooo000K0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0g03oool001/0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`3K0?ooo`006`3oool2000002D0oooo00<00000
0?ooo`3oool0f`3oool001/0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`3J0?oo
o`006`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0=X0oooo000K0?ooo`030000
003oool0oooo02H0oooo00<000000?ooo`3oool0f@3oool001/0oooo00<000000?ooo`3oool09P3o
ool00`000000oooo0?ooo`3I0?ooo`006`3oool2000002P0oooo00<000000?ooo`3oool0f03oool0
01/0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`3H0?ooo`006`3oool00`000000
oooo0?ooo`0X0?ooo`030000003oool0oooo0=L0oooo000K0?ooo`030000003oool0oooo02P0oooo
00<000000?ooo`3oool0e`3oool00080oooo0P0000040?ooo`030000003oool0oooo0080oooo1000
00020?ooo`<000001@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0=H0oooo0000
1@3oool000000?ooo`3oool0000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0e@3oool000050?ooo`000000
oooo0?ooo`0000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo0P00
000[0?ooo`030000003oool0oooo0=D0oooo00001@3oool000000?ooo`3oool0000000X0oooo00@0
00000?ooo`3oool0oooo0`0000050?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0
e03oool000050?ooo`000000oooo0?ooo`0000002P3oool01@000000oooo0?ooo`3oool0000000L0
oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`3D0?ooo`0000D0oooo0000003oool0
oooo000000080?ooo`<000000`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02`0
oooo00<000000?ooo`3oool0d`3oool00080oooo0P00000;0?ooo`040000003oool0oooo0?ooo`@0
0000103oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0=<0oooo000K0?ooo`030000
003oool0oooo02d0oooo00<000000?ooo`3oool0dP3oool001/0oooo0P00000_0?ooo`030000003o
ool0oooo0=40oooo000K0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0d03oool0
01/0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`3?0?ooo`006`3oool00`000000
oooo0?ooo`0`0?ooo`030000003oool0oooo0<l0oooo000K0?ooo`030000003oool0oooo0340oooo
00<000000?ooo`3oool0cP3oool001/0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?oo
o`3=0?ooo`006`3oool2000003@0oooo00<000000?ooo`3oool0c03oool001/0oooo00<000000?oo
o`3oool0=03oool00`000000oooo0?ooo`3;0?ooo`006`3oool00`000000oooo0?ooo`0e0?ooo`03
0000003oool0oooo0<X0oooo000K0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0
bP3oool001/0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`390?ooo`006`3oool2
000003P0oooo00<000000?ooo`3oool0b03oool001/0oooo00<000000?ooo`3oool0>03oool00`00
0000oooo0?ooo`370?ooo`006`3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo0<H0
oooo000K0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0a@3oool001/0oooo00<0
00000?ooo`3oool0>`3oool00`000000oooo0?ooo`340?ooo`006`3oool00`000000oooo0?ooo`0l
0?ooo`030000003oool0oooo0<<0oooo000K0?ooo`800000?P3oool200000<<0oooo000K0?ooo`03
0000003oool0oooo03l0oooo00<000000?ooo`3oool0`03oool001/0oooo00<000000?ooo`3oool0
@03oool00`000000oooo0?ooo`2o0?ooo`006`3oool00`000000oooo0?ooo`110?ooo`030000003o
ool0oooo0;h0oooo00080?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103oool0
0`000000oooo0?ooo`120?ooo`030000003oool0oooo0;d0oooo00070?ooo`040000003oool0oooo
000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`130?ooo`030000003oool0
oooo0;`0oooo00070?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool0103oool2
000004D0oooo0P00002l0?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`2i0?ooo`001`3oool01000
0000oooo0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0A`3oool0
0`000000oooo0?ooo`2h0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`2g0?ooo`00203oool2
000000X0oooo0P0000050?ooo`030000003oool0oooo04T0oooo0P00002g0?ooo`006`3oool20000
04`0oooo00<000000?ooo`3oool0]03oool001/0oooo00<000000?ooo`3oool0C03oool00`000000
oooo0?ooo`2c0?ooo`006`3oool00`000000oooo0?ooo`1=0?ooo`800000/`3oool001/0oooo00<0
00000?ooo`3oool0C`3oool200000;40oooo000K0?ooo`030000003oool0oooo0540oooo0P00002_
0?ooo`006`3oool00`000000oooo0?ooo`1C0?ooo`800000[@3oool001/0oooo0P00001F0?ooo`80
0000Z`3oool001/0oooo00<000000?ooo`3oool0E`3oool200000:T0oooo000K0?ooo`030000003o
ool0oooo05T0oooo0P00002W0?ooo`006`3oool00`000000oooo0?ooo`1K0?ooo`800000Y@3oool0
01/0oooo00<000000?ooo`3oool0G@3oool200000:<0oooo000K0?ooo`030000003oool0oooo05l0
oooo0`00002P0?ooo`006`3oool2000006<0oooo0`00002M0?ooo`006`3oool00`000000oooo0?oo
o`1U0?ooo`800000V`3oool001/0oooo00<000000?ooo`3oool0I`3oool3000009P0oooo000K0?oo
o`030000003oool0oooo06X0oooo0`00002E0?ooo`006`3oool00`000000oooo0?ooo`1]0?ooo`<0
0000TP3oool001/0oooo0P00001a0?ooo`@00000SP3oool001/0oooo00<000000?ooo`3oool0M03o
ool4000008X0oooo000K0?ooo`030000003oool0oooo07P0oooo100000260?ooo`006`3oool00`00
0000oooo0?ooo`1l0?ooo`@00000PP3oool00080oooo0P0000040?ooo`030000003oool0oooo0080
oooo100000020?ooo`<000001@3oool00`000000oooo0?ooo`200?ooo`<00000O`3oool000050?oo
o`000000oooo0?ooo`000000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0P`3oool4000007/0oooo00001@3oool000000?ooo`3oool0000000T0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`800000R03oool4000007L0
oooo00001@3oool000000?ooo`3oool0000000X0oooo00@000000?ooo`3oool0oooo0`0000050?oo
o`030000003oool0oooo08/0oooo1@00001b0?ooo`0000D0oooo0000003oool0oooo0000000;0?oo
o`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0T03oool5000006d0oooo00001@3o
ool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo09D0oooo1P00001W0?ooo`000P3oool2000000X0oooo0P000003
0?ooo`@00000103oool00`000000oooo0?ooo`2K0?ooo`L00000H03oool001/0oooo00<000000?oo
o`3oool0XP3oool8000005P0oooo000K0?ooo`800000Z`3oool5000005<0oooo000K0?ooo`030000
003oool0oooo0:l0oooo1`00001<0?ooo`006`3oool00`000000oooo0?ooo`2f0?ooo`X00000@P3o
ool001/0oooo00<000000?ooo`3oool0`03oool;000003L0oooo000K0?ooo`030000003oool0oooo
0</0oooo2P00000]0?ooo`006`3oool00`000000oooo0?ooo`3E0?oooa@000006@3oool001/0oooo
0P00003Z0?ooo`X000003`3oool001/0oooo00<000000?ooo`3oool0l`3oool6000000T0oooo000K
0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?oo
o`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo
0`3oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0326577, -0.0266977, \
0.00119662, 0.00175593}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Omegam[I*IntegerPart[om*10^\((16)\)]*10^\((\(-16\))\), 1/10, Pi/16, 
        1], {om, 0.01, 0.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 3.1746 0.164543 1.74858 [
[.18254 .15204 -12 -9 ]
[.18254 .15204 12 0 ]
[.34127 .15204 -9 -9 ]
[.34127 .15204 9 0 ]
[.5 .15204 -12 -9 ]
[.5 .15204 12 0 ]
[.65873 .15204 -9 -9 ]
[.65873 .15204 9 0 ]
[.81746 .15204 -12 -9 ]
[.81746 .15204 12 0 ]
[.97619 .15204 -9 -9 ]
[.97619 .15204 9 0 ]
[.01131 .07711 -30 -4.5 ]
[.01131 .07711 0 4.5 ]
[.01131 .25197 -24 -4.5 ]
[.01131 .25197 0 4.5 ]
[.01131 .3394 -18 -4.5 ]
[.01131 .3394 0 4.5 ]
[.01131 .42683 -24 -4.5 ]
[.01131 .42683 0 4.5 ]
[.01131 .51426 -18 -4.5 ]
[.01131 .51426 0 4.5 ]
[.01131 .60169 -24 -4.5 ]
[.01131 .60169 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .16454 m
.18254 .17079 L
s
[(0.05)] .18254 .15204 0 1 Mshowa
.34127 .16454 m
.34127 .17079 L
s
[(0.1)] .34127 .15204 0 1 Mshowa
.5 .16454 m
.5 .17079 L
s
[(0.15)] .5 .15204 0 1 Mshowa
.65873 .16454 m
.65873 .17079 L
s
[(0.2)] .65873 .15204 0 1 Mshowa
.81746 .16454 m
.81746 .17079 L
s
[(0.25)] .81746 .15204 0 1 Mshowa
.97619 .16454 m
.97619 .17079 L
s
[(0.3)] .97619 .15204 0 1 Mshowa
.125 Mabswid
.05556 .16454 m
.05556 .16829 L
s
.0873 .16454 m
.0873 .16829 L
s
.11905 .16454 m
.11905 .16829 L
s
.15079 .16454 m
.15079 .16829 L
s
.21429 .16454 m
.21429 .16829 L
s
.24603 .16454 m
.24603 .16829 L
s
.27778 .16454 m
.27778 .16829 L
s
.30952 .16454 m
.30952 .16829 L
s
.37302 .16454 m
.37302 .16829 L
s
.40476 .16454 m
.40476 .16829 L
s
.43651 .16454 m
.43651 .16829 L
s
.46825 .16454 m
.46825 .16829 L
s
.53175 .16454 m
.53175 .16829 L
s
.56349 .16454 m
.56349 .16829 L
s
.59524 .16454 m
.59524 .16829 L
s
.62698 .16454 m
.62698 .16829 L
s
.69048 .16454 m
.69048 .16829 L
s
.72222 .16454 m
.72222 .16829 L
s
.75397 .16454 m
.75397 .16829 L
s
.78571 .16454 m
.78571 .16829 L
s
.84921 .16454 m
.84921 .16829 L
s
.88095 .16454 m
.88095 .16829 L
s
.9127 .16454 m
.9127 .16829 L
s
.94444 .16454 m
.94444 .16829 L
s
.25 Mabswid
0 .16454 m
1 .16454 L
s
.02381 .07711 m
.03006 .07711 L
s
[(-0.05)] .01131 .07711 1 0 Mshowa
.02381 .25197 m
.03006 .25197 L
s
[(0.05)] .01131 .25197 1 0 Mshowa
.02381 .3394 m
.03006 .3394 L
s
[(0.1)] .01131 .3394 1 0 Mshowa
.02381 .42683 m
.03006 .42683 L
s
[(0.15)] .01131 .42683 1 0 Mshowa
.02381 .51426 m
.03006 .51426 L
s
[(0.2)] .01131 .51426 1 0 Mshowa
.02381 .60169 m
.03006 .60169 L
s
[(0.25)] .01131 .60169 1 0 Mshowa
.125 Mabswid
.02381 .0946 m
.02756 .0946 L
s
.02381 .11209 m
.02756 .11209 L
s
.02381 .12957 m
.02756 .12957 L
s
.02381 .14706 m
.02756 .14706 L
s
.02381 .18203 m
.02756 .18203 L
s
.02381 .19952 m
.02756 .19952 L
s
.02381 .217 m
.02756 .217 L
s
.02381 .23449 m
.02756 .23449 L
s
.02381 .26946 m
.02756 .26946 L
s
.02381 .28694 m
.02756 .28694 L
s
.02381 .30443 m
.02756 .30443 L
s
.02381 .32192 m
.02756 .32192 L
s
.02381 .35689 m
.02756 .35689 L
s
.02381 .37437 m
.02756 .37437 L
s
.02381 .39186 m
.02756 .39186 L
s
.02381 .40934 m
.02756 .40934 L
s
.02381 .44432 m
.02756 .44432 L
s
.02381 .4618 m
.02756 .4618 L
s
.02381 .47929 m
.02756 .47929 L
s
.02381 .49677 m
.02756 .49677 L
s
.02381 .53175 m
.02756 .53175 L
s
.02381 .54923 m
.02756 .54923 L
s
.02381 .56672 m
.02756 .56672 L
s
.02381 .5842 m
.02756 .5842 L
s
.02381 .05963 m
.02756 .05963 L
s
.02381 .04214 m
.02756 .04214 L
s
.02381 .02466 m
.02756 .02466 L
s
.02381 .00717 m
.02756 .00717 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.07374 .1001 L
.0929 .17073 L
.11234 .22409 L
.13363 .26751 L
.17279 .33009 L
.21049 .37976 L
.25055 .42361 L
.28915 .45818 L
.32628 .48535 L
.36579 .50877 L
.40383 .5269 L
.4404 .541 L
.47934 .55309 L
.51682 .56244 L
.55667 .57044 L
.59506 .57666 L
.63198 .58158 L
.67127 .58592 L
.70909 .58941 L
.74929 .59252 L
.78802 .59507 L
.82529 .59718 L
.86492 .59912 L
.90309 .60074 L
.9398 .60211 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3o
ool00`000000oooo0?ooo`3l0?ooo`008@3oool200000?d0oooo000Q0?ooo`030000003oool0oooo
0?`0oooo000Q0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0m03oool00240oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3d0?ooo`008@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo0?@0oooo000Q0?ooo`8000001`3oool00`000000oooo0?ooo`3c
0?ooo`008@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?<0oooo000Q0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0l`3oool00240oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`3c0?ooo`008@3oool2000000P0oooo00<000000?ooo`3oool0lP3o
ool00240oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3b0?ooo`008@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo0?80oooo000Q0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0lP3oool00240oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`3b0?ooo`008@3oool2000000T0oooo00<000000?ooo`3oool0l@3oool00240oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`3a0?ooo`00203oool2000000@0oooo00<000000?oo
o`3oool00`3oool2000000<0oooo0`0000050?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool0l@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`000005
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3a
0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<0
00000?ooo`3oool00P3oool2000000X0oooo00<000000?ooo`3oool0l03oool000030?ooo`000000
0000008000000P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo
0`0000050?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0l03oool000L0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3`0?ooo`001`3oool010000000oooo
0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo0?00oooo00080?ooo`8000002P3oool2000000<0
oooo100000040?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0k`3oool00240oooo
0P00000;0?ooo`030000003oool0oooo0>l0oooo000Q0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool0k`3oool00240oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3_
0?ooo`008@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0>l0oooo000Q0?ooo`80
0000303oool00`000000oooo0?ooo`3^0?ooo`008@3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo0>h0oooo000Q0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0kP3o
ool00240oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3^0?ooo`008@3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo0P0000040?ooo`030000003oool0oooo
00<0oooo0P0000030?ooo`<000005`3oool2000000@0oooo00<000000?ooo`3oool00P3oool40000
01L0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?ooo`<00000603oool20000
00@0oooo00<000000?ooo`3oool00P3oool4000001L0oooo0P0000040?ooo`030000003oool0oooo
0080oooo100000020?ooo`<00000603oool2000000@0oooo00<000000?ooo`3oool00`3oool20000
00<0oooo000Q0?ooo`8000003@3oool00`000000oooo0?ooo`0<0?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`0C0?ooo`040000003oool0
oooo000000X0oooo00<000000?ooo`3oool05@3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`0000080?ooo`03
0000003oool0oooo01L0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo01@0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00020?ooo`008@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo01<0oooo
00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo
000000T0oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`0000090?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`00000;0?ooo`400000
0@3oool10?ooo`008@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`<000005P3oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo01D0oooo00@000000?ooo`3oool000002P3oool0
10000000oooo0?ooo`3oool3000001L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo
0?ooo`0E0?ooo`040000003oool0oooo000000X0oooo00@000000?ooo`3oool0oooo0`00000G0?oo
o`040000003oool0oooo000000T0oooo0P0000030?ooo`008@3oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo00`0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00020?ooo`030000003oool0oooo01H0oooo00@000000?ooo`3oool000002P3oool00`000000oooo
0?ooo`0E0?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0oooo0000000I0?oo
o`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`00
000;0?ooo`040000003oool0oooo000001T0oooo00@000000?ooo`3oool000002`3oool100000040
oooo0@3oool00240oooo0P00000>0?ooo`030000003oool0oooo00/0oooo00@000000?ooo`3oool0
0000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo01H0oooo00@000000?oo
o`3oool00000203oool3000001L0oooo00@000000?ooo`3oool00000203oool3000000<0oooo00<0
00000?ooo`3oool05`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000001H0
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo01L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`008@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00`0oooo0P00000:0?ooo`8000000`3o
ool4000001H0oooo0P00000;0?ooo`030000003oool0oooo01H0oooo0P00000;0?ooo`040000003o
ool0oooo0?ooo`@000005`3oool2000000X0oooo0P00000H0?ooo`8000002P3oool2000000<0oooo
1000000G0?ooo`8000002P3oool2000000<0oooo000Q0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0k03oool00240oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3/
0?ooo`008@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0>/0oooo000K0?ooool0
00000`0000030?ooo`008@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo
0P0000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00L0oooo000Q0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0j`3oool00240oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3Z
0?ooo`008@3oool200000100oooo00<000000?ooo`3oool0jP3oool00240oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`3Z0?ooo`008@3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0>T0oooo000Q0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0j@3o
ool00240oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3Y0?ooo`008@3oool20000
0180oooo00<000000?ooo`3oool0j03oool00240oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`3X0?ooo`008@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0>L0oooo
000Q0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0i`3oool00240oooo0P00000C
0?ooo`030000003oool0oooo0>L0oooo000Q0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool0iP3oool00240oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3V0?ooo`00
8@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0>D0oooo000Q0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0i@3oool00240oooo0P00000F0?ooo`030000003oool0
oooo0>@0oooo000Q0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0i03oool000P0
oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`<000001@3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo0><0oooo00070?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0h`3oool000L0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo0P00000H0?ooo`030000003oool0
oooo0>80oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3o
ool3000000D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`3R0?ooo`001`3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0>40oooo00070?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0h@3oool000P0oooo0P00000:0?ooo`800000
0`3oool4000000@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`3P0?ooo`008@3o
ool2000001/0oooo00<000000?ooo`3oool0g`3oool00240oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`3O0?ooo`008@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
0=h0oooo000Q0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0gP3oool00240oooo
0P00000M0?ooo`030000003oool0oooo0=d0oooo000Q0?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool0g03oool00240oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`3L
0?ooo`008@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0=/0oooo000Q0?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool0fP3oool00240oooo0P00000P0?ooo`030000
003oool0oooo0=X0oooo000Q0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0f@3o
ool00240oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3I0?ooo`008@3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0=P0oooo000Q0?ooo`8000008`3oool00`000000
oooo0?ooo`3G0?ooo`008@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0=L0oooo
000Q0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0eP3oool000h0oooo0P000004
0?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo02@0oooo00<00000
0?ooo`3oool0e@3oool000d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0e03oool000d0oooo00@000000?oo
o`3oool000002P3oool00`000000oooo0?ooo`030?ooo`8000009P3oool00`000000oooo0?ooo`3D
0?ooo`003@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`3C0?ooo`003@3oool010000000oooo0?ooo`00000:
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`3B
0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`<000001@3oool00`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo0=40oooo000>0?ooo`8000002`3oool00`000000oooo0?ooo`030?oo
o`800000:@3oool00`000000oooo0?ooo`3A0?ooo`008@3oool00`000000oooo0?ooo`0Y0?ooo`03
0000003oool0oooo0=00oooo000Q0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0
c`3oool00240oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`3>0?ooo`008@3oool0
0`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0<h0oooo000Q0?ooo`800000;@3oool00`00
0000oooo0?ooo`3=0?ooo`008@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0<`0
oooo000Q0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0b`3oool00240oooo00<0
00000?ooo`3oool0;`3oool00`000000oooo0?ooo`3:0?ooo`008@3oool200000340oooo00<00000
0?ooo`3oool0b@3oool00240oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`380?oo
o`008@3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0<L0oooo000Q0?ooo`030000
003oool0oooo0380oooo00<000000?ooo`3oool0a`3oool00240oooo00<000000?ooo`3oool0<`3o
ool00`000000oooo0?ooo`360?ooo`008@3oool2000003D0oooo00<000000?ooo`3oool0a@3oool0
0240oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`340?ooo`00203oool2000000@0
oooo00<000000?ooo`3oool00P3oool400000080oooo0`0000050?ooo`030000003oool0oooo03H0
oooo00<000000?ooo`3oool0``3oool000L0oooo00@000000?ooo`3oool000002P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0=`3oool00`000000
oooo0?ooo`320?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool00P3oool2000003T0oooo00<000000?ooo`3oool0`@3oool000L0oooo
00@000000?ooo`3oool000002P3oool010000000oooo0?ooo`3oool3000000D0oooo00<000000?oo
o`3oool0>@3oool00`000000oooo0?ooo`300?ooo`001`3oool010000000oooo0?ooo`00000:0?oo
o`050000003oool0oooo0?ooo`0000001`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0
oooo0;l0oooo00070?ooo`040000003oool0oooo000000P0oooo0`0000030?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`2n0?ooo`00203oool20000
00/0oooo00@000000?ooo`3oool0oooo100000040?ooo`030000003oool0oooo03`0oooo0P00002n
0?ooo`008@3oool2000003l0oooo00<000000?ooo`3oool0^`3oool00240oooo00<000000?ooo`3o
ool0?`3oool00`000000oooo0?ooo`2j0?ooo`008@3oool00`000000oooo0?ooo`100?ooo`030000
003oool0oooo0;T0oooo000Q0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0^03o
ool00240oooo0P0000130?ooo`030000003oool0oooo0;L0oooo000Q0?ooo`030000003oool0oooo
04<0oooo0P00002g0?ooo`008@3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo0;@0
oooo000Q0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0/`3oool00240oooo00<0
00000?ooo`3oool0A`3oool200000;<0oooo000Q0?ooo`800000BP3oool00`000000oooo0?ooo`2`
0?ooo`008@3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo0:l0oooo000Q0?ooo`03
0000003oool0oooo04/0oooo0P00002_0?ooo`008@3oool00`000000oooo0?ooo`1=0?ooo`800000
[@3oool00240oooo0P00001@0?ooo`800000Z`3oool00240oooo00<000000?ooo`3oool0D@3oool2
00000:T0oooo000Q0?ooo`030000003oool0oooo05<0oooo0P00002W0?ooo`003P3oool2000000@0
oooo00<000000?ooo`3oool00P3oool4000000@0oooo00<000000?ooo`3oool0E@3oool00`000000
oooo0?ooo`2T0?ooo`003@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0EP3oool200000:@0oooo000=0?ooo`040000003oool0oooo000000T0
oooo00<000000?ooo`3oool0103oool2000005T0oooo0P00002R0?ooo`003@3oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0FP3oool200000:00
oooo000=0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`1L0?ooo`800000WP3oool000d0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo05h0oooo0P00002L0?ooo`003P3oool2000000X0
oooo0P0000050?ooo`800000H@3oool2000009X0oooo000Q0?ooo`030000003oool0oooo0680oooo
0`00002G0?ooo`008@3oool00`000000oooo0?ooo`1U0?ooo`800000U@3oool00240oooo00<00000
0?ooo`3oool0I`3oool300000980oooo000Q0?ooo`030000003oool0oooo06X0oooo0`00002?0?oo
o`008@3oool2000006h0oooo1000002;0?ooo`008@3oool00`000000oooo0?ooo`1a0?ooo`@00000
Q`3oool00240oooo00<000000?ooo`3oool0M@3oool4000008<0oooo000Q0?ooo`030000003oool0
oooo07T0oooo1@00001n0?ooo`008@3oool2000007l0oooo1P00001h0?ooo`008@3oool00`000000
oooo0?ooo`240?ooo`D00000L`3oool00240oooo00<000000?ooo`3oool0R@3oool5000006h0oooo
000Q0?ooo`030000003oool0oooo08h0oooo1`00001W0?ooo`008@3oool00`000000oooo0?ooo`2E
0?ooo`T00000GP3oool00240oooo0P00002O0?ooo`X00000E03oool00240oooo00<000000?ooo`3o
ool0Z03oool:000004X0oooo00080?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000
0P3oool3000000D0oooo00<000000?ooo`3oool0/P3ooolD000003H0oooo00070?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0<H0oooo5000000R0?ooo`001`3oool010000000oooo0?ooo`0000090?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool200000=/0oooo4`00000?0?ooo`001`3o
ool010000000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000
oooo0?ooo`3]0?ooo`H000002@3oool000L0oooo00@000000?ooo`3oool000002`3oool010000000
oooo0?ooo`0000070?ooo`030000003oool0oooo0?`0oooo00070?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo0?`0oooo00080?ooo`8000002P3oool2000000<0oooo100000040?ooo`030000003oool0oooo
0?`0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0407059, -0.113891, \
0.00122557, 0.00222505}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Omegab[I*IntegerPart[g*10^\((16)\)]*10^\((\(-16\))\), 1/10, Pi/16, 
        1], {g, 0.01, 0.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 3.1746 0.0884667 0.98441 [
[.18254 .07597 -12 -9 ]
[.18254 .07597 12 0 ]
[.34127 .07597 -9 -9 ]
[.34127 .07597 9 0 ]
[.5 .07597 -12 -9 ]
[.5 .07597 12 0 ]
[.65873 .07597 -9 -9 ]
[.65873 .07597 9 0 ]
[.81746 .07597 -12 -9 ]
[.81746 .07597 12 0 ]
[.97619 .07597 -9 -9 ]
[.97619 .07597 9 0 ]
[.01131 .18691 -18 -4.5 ]
[.01131 .18691 0 4.5 ]
[.01131 .28535 -18 -4.5 ]
[.01131 .28535 0 4.5 ]
[.01131 .38379 -18 -4.5 ]
[.01131 .38379 0 4.5 ]
[.01131 .48223 -18 -4.5 ]
[.01131 .48223 0 4.5 ]
[.01131 .58067 -18 -4.5 ]
[.01131 .58067 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .08847 m
.18254 .09472 L
s
[(0.05)] .18254 .07597 0 1 Mshowa
.34127 .08847 m
.34127 .09472 L
s
[(0.1)] .34127 .07597 0 1 Mshowa
.5 .08847 m
.5 .09472 L
s
[(0.15)] .5 .07597 0 1 Mshowa
.65873 .08847 m
.65873 .09472 L
s
[(0.2)] .65873 .07597 0 1 Mshowa
.81746 .08847 m
.81746 .09472 L
s
[(0.25)] .81746 .07597 0 1 Mshowa
.97619 .08847 m
.97619 .09472 L
s
[(0.3)] .97619 .07597 0 1 Mshowa
.125 Mabswid
.05556 .08847 m
.05556 .09222 L
s
.0873 .08847 m
.0873 .09222 L
s
.11905 .08847 m
.11905 .09222 L
s
.15079 .08847 m
.15079 .09222 L
s
.21429 .08847 m
.21429 .09222 L
s
.24603 .08847 m
.24603 .09222 L
s
.27778 .08847 m
.27778 .09222 L
s
.30952 .08847 m
.30952 .09222 L
s
.37302 .08847 m
.37302 .09222 L
s
.40476 .08847 m
.40476 .09222 L
s
.43651 .08847 m
.43651 .09222 L
s
.46825 .08847 m
.46825 .09222 L
s
.53175 .08847 m
.53175 .09222 L
s
.56349 .08847 m
.56349 .09222 L
s
.59524 .08847 m
.59524 .09222 L
s
.62698 .08847 m
.62698 .09222 L
s
.69048 .08847 m
.69048 .09222 L
s
.72222 .08847 m
.72222 .09222 L
s
.75397 .08847 m
.75397 .09222 L
s
.78571 .08847 m
.78571 .09222 L
s
.84921 .08847 m
.84921 .09222 L
s
.88095 .08847 m
.88095 .09222 L
s
.9127 .08847 m
.9127 .09222 L
s
.94444 .08847 m
.94444 .09222 L
s
.25 Mabswid
0 .08847 m
1 .08847 L
s
.02381 .18691 m
.03006 .18691 L
s
[(0.1)] .01131 .18691 1 0 Mshowa
.02381 .28535 m
.03006 .28535 L
s
[(0.2)] .01131 .28535 1 0 Mshowa
.02381 .38379 m
.03006 .38379 L
s
[(0.3)] .01131 .38379 1 0 Mshowa
.02381 .48223 m
.03006 .48223 L
s
[(0.4)] .01131 .48223 1 0 Mshowa
.02381 .58067 m
.03006 .58067 L
s
[(0.5)] .01131 .58067 1 0 Mshowa
.125 Mabswid
.02381 .10815 m
.02756 .10815 L
s
.02381 .12784 m
.02756 .12784 L
s
.02381 .14753 m
.02756 .14753 L
s
.02381 .16722 m
.02756 .16722 L
s
.02381 .2066 m
.02756 .2066 L
s
.02381 .22628 m
.02756 .22628 L
s
.02381 .24597 m
.02756 .24597 L
s
.02381 .26566 m
.02756 .26566 L
s
.02381 .30504 m
.02756 .30504 L
s
.02381 .32473 m
.02756 .32473 L
s
.02381 .34441 m
.02756 .34441 L
s
.02381 .3641 m
.02756 .3641 L
s
.02381 .40348 m
.02756 .40348 L
s
.02381 .42317 m
.02756 .42317 L
s
.02381 .44285 m
.02756 .44285 L
s
.02381 .46254 m
.02756 .46254 L
s
.02381 .50192 m
.02756 .50192 L
s
.02381 .52161 m
.02756 .52161 L
s
.02381 .5413 m
.02756 .5413 L
s
.02381 .56098 m
.02756 .56098 L
s
.02381 .06878 m
.02756 .06878 L
s
.02381 .04909 m
.02756 .04909 L
s
.02381 .0294 m
.02756 .0294 L
s
.02381 .00971 m
.02756 .00971 L
s
.02381 .60036 m
.02756 .60036 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.0929 .13319 L
.13363 .23448 L
.17189 .30841 L
.20867 .36432 L
.24783 .41108 L
.28553 .44649 L
.32559 .47622 L
.36419 .49896 L
.40132 .51664 L
.44083 .53192 L
.47887 .54396 L
.51544 .55359 L
.55438 .56216 L
.59186 .5691 L
.63171 .57534 L
.6701 .58046 L
.70702 .58469 L
.74631 .5886 L
.78413 .59186 L
.82433 .59488 L
.86306 .59743 L
.90033 .5996 L
.93996 .60165 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`005@3oool0
0`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0
oooo0P00003o0?ooo`X0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`40oooo000E0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0o`3oool10?ooo`005@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo0?l0oooo000E0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool0o`3oool001D0oooo0P0000070?ooo`030000003oool0oooo0?l0oooo000E0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0oP3oool001D0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`3n0?ooo`005@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
0?h0oooo000E0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0oP3oool001D0oooo
0P0000090?ooo`030000003oool0oooo01<0oooo0P0000040?ooo`030000003oool0oooo00<0oooo
0P0000030?ooo`<000006@3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000001T0oooo
0P0000040?ooo`030000003oool0oooo0080oooo100000020?ooo`<000006P3oool2000000@0oooo
00<000000?ooo`3oool00P3oool4000001T0oooo0P0000040?ooo`030000003oool0oooo0080oooo
100000020?ooo`<000006P3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo
000E0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04P3oool010000000oooo0?oo
o`0000080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool05@3oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo01L0oooo00@000000?ooo`3oool000002P3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo01H0oooo00@000000?ooo`3oool00000203o
ool00`000000oooo0?ooo`0I0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`0F0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000000P3oool001D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0B0?ooo`04
0000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`0E
0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool05`3oool010000000oooo0?oo
o`00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool05P3oool010000000oooo
0?ooo`0000090?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool000002@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo01H0oooo00@000000?ooo`3oool000002`3oool1
00000040oooo0@3oool001D0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0A0?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool3000001P0oooo00@0
00000?ooo`3oool000002P3oool00`000000oooo0?ooo`0G0?ooo`040000003oool0oooo000000X0
oooo00@000000?ooo`3oool0oooo0`00000I0?ooo`040000003oool0oooo000000X0oooo00<00000
0?ooo`3oool05`3oool010000000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<00000
6@3oool010000000oooo0?ooo`0000090?ooo`8000000`3oool001D0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`0A0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`0H0?ooo`040000003oool0oooo000000X0oooo00<00000
0?ooo`3oool05`3oool010000000oooo0?ooo`00000:0?ooo`050000003oool0oooo0?ooo`000000
6`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo01H0oooo00@000000?ooo`3o
ool000002`3oool010000000oooo0?ooo`00000K0?ooo`040000003oool0oooo000000/0oooo0@00
00010?ooo`40oooo000E0?ooo`8000002P3oool00`000000oooo0?ooo`0A0?ooo`040000003oool0
oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0H0?ooo`040000
003oool0oooo000000P0oooo0`00000I0?ooo`040000003oool0oooo000000P0oooo0`0000030?oo
o`030000003oool0oooo01T0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
000H0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`0I0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool0
01D0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0A0?ooo`8000002P3oool20000
00<0oooo1000000H0?ooo`8000002`3oool00`000000oooo0?ooo`0H0?ooo`8000002`3oool01000
0000oooo0?ooo`3oool4000001T0oooo0P00000:0?ooo`8000006P3oool2000000X0oooo0P000003
0?ooo`@000006@3oool2000000X0oooo0P0000030?ooo`005@3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo0?/0oooo000E0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool0n`3oool001D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3j0?ooo`005@3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?X0oooo000>0?ooool0000040000003
0?ooo`005@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`000000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`070?ooo`005@3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo0?T0oooo000E0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
n@3oool001D0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3i0?ooo`005@3oool2
000000h0oooo00<000000?ooo`3oool0n03oool001D0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`3h0?ooo`005@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?P0
oooo000E0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0m`3oool001D0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`3g0?ooo`005@3oool2000000l0oooo00<00000
0?ooo`3oool0m`3oool001D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3f0?oo
o`005@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0?H0oooo000E0?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool0mP3oool001D0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`3e0?ooo`005@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0?D0oooo000E0?ooo`8000004P3oool00`000000oooo0?ooo`3d0?ooo`005@3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo0?@0oooo000E0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool0m03oool001D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`3c0?ooo`005@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0?<0oooo000E0?oo
o`800000503oool00`000000oooo0?ooo`3b0?ooo`005@3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo0?80oooo000E0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
lP3oool00080oooo0P0000040?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0l@3oool000050?ooo`000000oooo0?ooo`0000002P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0l@3o
ool000050?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`<000005@3o
ool00`000000oooo0?ooo`3`0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3`0?ooo`0000D0oooo
0000003oool0oooo0000000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`3_0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0>l0oooo00020?ooo`8000002`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0k`3oool0
01D0oooo0P00000H0?ooo`030000003oool0oooo0>h0oooo000E0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool0kP3oool001D0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`3]0?ooo`005@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0>d0oooo000E
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0k@3oool001D0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`3/0?ooo`005@3oool2000001X0oooo00<000000?ooo`3o
ool0k03oool001D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3[0?ooo`005@3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>/0oooo000E0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool0jP3oool001D0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`3Z0?ooo`005@3oool2000001d0oooo00<000000?ooo`3oool0j@3oool001D0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3Y0?ooo`005@3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo0>P0oooo000E0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool0j03oool001D0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`3W0?oo
o`005@3oool2000001l0oooo00<000000?ooo`3oool0i`3oool001D0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`3V0?ooo`005@3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo0>H0oooo000E0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0i@3oool0
0080oooo0P0000040?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo
0240oooo00<000000?ooo`3oool0i03oool000050?ooo`000000oooo0?ooo`000000203oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0i03oool00005
0?ooo`000000oooo0?ooo`0000002@3oool00`000000oooo0?ooo`040?ooo`<000008P3oool00`00
0000oooo0?ooo`3S0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`3S0?ooo`0000D0oooo0000003o
ool0oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool08`3oool00`00
0000oooo0?ooo`3R0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`3R0?ooo`000P3oool20000
00X0oooo0P0000050?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0h@3oool001D0
oooo0P00000U0?ooo`030000003oool0oooo0>40oooo000E0?ooo`030000003oool0oooo02D0oooo
00<000000?ooo`3oool0h03oool001D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?oo
o`3O0?ooo`005@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0=l0oooo000E0?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool0gP3oool001D0oooo0P00000Y0?ooo`03
0000003oool0oooo0=d0oooo000E0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0
g@3oool001D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`3L0?ooo`005@3oool0
0`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0=/0oooo000E0?ooo`030000003oool0oooo
02X0oooo00<000000?ooo`3oool0f`3oool001D0oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`3J0?ooo`005@3oool2000002d0oooo00<000000?ooo`3oool0f@3oool001D0oooo00<0
00000?ooo`3oool0;03oool00`000000oooo0?ooo`3I0?ooo`005@3oool00`000000oooo0?ooo`0]
0?ooo`030000003oool0oooo0=P0oooo000E0?ooo`030000003oool0oooo02h0oooo00<000000?oo
o`3oool0e`3oool001D0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`3G0?ooo`00
5@3oool200000300oooo00<000000?ooo`3oool0eP3oool001D0oooo00<000000?ooo`3oool0<03o
ool00`000000oooo0?ooo`3E0?ooo`005@3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0
oooo0=@0oooo00020?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000001@3oool00`00
0000oooo0?ooo`0a0?ooo`030000003oool0oooo0=@0oooo00001@3oool000000?ooo`3oool00000
00P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0b0?ooo`030000003oool0
oooo0=<0oooo00001@3oool000000?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool3
000003<0oooo00<000000?ooo`3oool0dP3oool000050?ooo`000000oooo0?ooo`0000002@3oool2
000000D0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`3A0?ooo`0000D0oooo0000
003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0=03oool0
0`000000oooo0?ooo`3A0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`3@0?ooo`000P3oool2
000000X0oooo0P0000050?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0c`3oool0
01D0oooo0P00000h0?ooo`030000003oool0oooo0<h0oooo000E0?ooo`030000003oool0oooo03L0
oooo00<000000?ooo`3oool0cP3oool001D0oooo00<000000?ooo`3oool0>03oool00`000000oooo
0?ooo`3=0?ooo`005@3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo0<`0oooo000E
0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0b`3oool001D0oooo00<000000?oo
o`3oool0>`3oool00`000000oooo0?ooo`3:0?ooo`005@3oool2000003d0oooo00<000000?ooo`3o
ool0b@3oool001D0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`380?ooo`005@3o
ool00`000000oooo0?ooo`0n0?ooo`800000b03oool001D0oooo00<000000?ooo`3oool0@03oool0
0`000000oooo0?ooo`350?ooo`005@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo
0<@0oooo000E0?ooo`800000@`3oool00`000000oooo0?ooo`330?ooo`005@3oool00`000000oooo
0?ooo`130?ooo`030000003oool0oooo0<80oooo000E0?ooo`030000003oool0oooo04@0oooo0P00
00320?ooo`005@3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo0;l0oooo000E0?oo
o`030000003oool0oooo04L0oooo00<000000?ooo`3oool0_P3oool001D0oooo0P0000190?ooo`80
0000_P3oool001D0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`2k0?ooo`005@3o
ool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo0;X0oooo000E0?ooo`030000003oool0
oooo04`0oooo0P00002j0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0]`3oool00005
0?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
04l0oooo0P00002g0?ooo`0000D0oooo0000003oool0oooo000000070?ooo`D00000103oool30000
0540oooo0P00002e0?ooo`0000D0oooo0000003oool0oooo000000070?ooo`040000003oool0oooo
000000D0oooo00<000000?ooo`3oool0D`3oool200000;<0oooo00001@3oool000000?ooo`3oool0
000000P0oooo00<000000?ooo`0000001@3oool00`000000oooo0?ooo`1E0?ooo`800000/@3oool0
00050?ooo`000000oooo0?ooo`0000002@3oool2000000D0oooo00<000000?ooo`3oool0E`3oool2
00000:l0oooo00020?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
05T0oooo00<000000?ooo`3oool0[03oool001D0oooo0P00001K0?ooo`800000[03oool001D0oooo
00<000000?ooo`3oool0G03oool200000:X0oooo000E0?ooo`030000003oool0oooo05h0oooo0P00
002X0?ooo`005@3oool00`000000oooo0?ooo`1P0?ooo`800000YP3oool001D0oooo00<000000?oo
o`3oool0HP3oool300000:<0oooo000E0?ooo`800000IP3oool200000:40oooo000E0?ooo`030000
003oool0oooo06L0oooo0`00002N0?ooo`005@3oool00`000000oooo0?ooo`1Z0?ooo`800000W03o
ool001D0oooo00<000000?ooo`3oool0K03oool3000009T0oooo000E0?ooo`030000003oool0oooo
06l0oooo0`00002F0?ooo`005@3oool00`000000oooo0?ooo`1b0?ooo`@00000TP3oool001D0oooo
0P00001g0?ooo`<00000S`3oool001D0oooo00<000000?ooo`3oool0N@3oool3000008`0oooo000E
0?ooo`030000003oool0oooo07`0oooo100000280?ooo`005@3oool00`000000oooo0?ooo`200?oo
o`@00000Q03oool001D0oooo00<000000?ooo`3oool0Q03oool6000007h0oooo000E0?ooo`800000
R`3oool5000007T0oooo000E0?ooo`030000003oool0oooo08l0oooo1@00001d0?ooo`005@3oool0
0`000000oooo0?ooo`2D0?ooo`D00000K`3oool00080oooo0P0000040?ooo`030000003oool0oooo
0080oooo0`0000050?ooo`030000003oool0oooo09T0oooo1P00001Y0?ooo`0000D0oooo0000003o
ool0oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0W`3oool70000
0680oooo00001@3oool000000?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool30000
0:H0oooo2P00001H0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`00
0000oooo0?ooo`2`0?ooo`/00000C@3oool000050?ooo`000000oooo0?ooo`000000203oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0;/0oooo2P0000130?ooo`0000D0oooo0000003o
ool0oooo000000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0a@3oool;0000
03P0oooo00020?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0d03oool:000002h0
oooo000E0?ooo`030000003oool0oooo0=X0oooo5@00000I0?ooo`005@3oool200000?00oooo4000
00090?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?ooo`030000003oool0oooo
0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo
0?ooo`3o0?ooo`T0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40
oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0249809, -0.109476, \
0.001169, 0.00376989}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((1/100 + Omegap[I/10, 1/10, Pi/16, 1])\)\ \((1/100 + 
          Omegam[I/10, 1/10, Pi/16, 1])\)\)], "Input"],

Cell[BoxData[
    \(0.0447530084443611221203554635`16.404 + 
      0``17.7729\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(testfuncspec[I/10, 1/10, Pi/16, 1]*\((\(-1\)/100)\)\)], "Input"],

Cell[BoxData[
    \(0.044753008444361122120355451609655`17.7697 + 
      0``21.3492\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      testfuncspec[IntegerPart[om*10^16]*10^\((\(-16\))\)*I, 4/10, Pi/16, 
        1], {om, \(-0.5\), 0.5}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.952381 3.29605 2.74227 [
[.11905 .54128 -12 -9 ]
[.11905 .54128 12 0 ]
[.30952 .54128 -12 -9 ]
[.30952 .54128 12 0 ]
[.69048 .54128 -9 -9 ]
[.69048 .54128 9 0 ]
[.88095 .54128 -9 -9 ]
[.88095 .54128 9 0 ]
[.4875 .00533 -24 -4.5 ]
[.4875 .00533 0 4.5 ]
[.4875 .14244 -30 -4.5 ]
[.4875 .14244 0 4.5 ]
[.4875 .27955 -24 -4.5 ]
[.4875 .27955 0 4.5 ]
[.4875 .41667 -30 -4.5 ]
[.4875 .41667 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 .55378 m
.11905 .56003 L
s
[(-0.4)] .11905 .54128 0 1 Mshowa
.30952 .55378 m
.30952 .56003 L
s
[(-0.2)] .30952 .54128 0 1 Mshowa
.69048 .55378 m
.69048 .56003 L
s
[(0.2)] .69048 .54128 0 1 Mshowa
.88095 .55378 m
.88095 .56003 L
s
[(0.4)] .88095 .54128 0 1 Mshowa
.125 Mabswid
.16667 .55378 m
.16667 .55753 L
s
.21429 .55378 m
.21429 .55753 L
s
.2619 .55378 m
.2619 .55753 L
s
.35714 .55378 m
.35714 .55753 L
s
.40476 .55378 m
.40476 .55753 L
s
.45238 .55378 m
.45238 .55753 L
s
.54762 .55378 m
.54762 .55753 L
s
.59524 .55378 m
.59524 .55753 L
s
.64286 .55378 m
.64286 .55753 L
s
.7381 .55378 m
.7381 .55753 L
s
.78571 .55378 m
.78571 .55753 L
s
.83333 .55378 m
.83333 .55753 L
s
.07143 .55378 m
.07143 .55753 L
s
.02381 .55378 m
.02381 .55753 L
s
.92857 .55378 m
.92857 .55753 L
s
.97619 .55378 m
.97619 .55753 L
s
.25 Mabswid
0 .55378 m
1 .55378 L
s
.5 .00533 m
.50625 .00533 L
s
[(-1.2)] .4875 .00533 1 0 Mshowa
.5 .14244 m
.50625 .14244 L
s
[(-1.15)] .4875 .14244 1 0 Mshowa
.5 .27955 m
.50625 .27955 L
s
[(-1.1)] .4875 .27955 1 0 Mshowa
.5 .41667 m
.50625 .41667 L
s
[(-1.05)] .4875 .41667 1 0 Mshowa
.125 Mabswid
.5 .03275 m
.50375 .03275 L
s
.5 .06017 m
.50375 .06017 L
s
.5 .0876 m
.50375 .0876 L
s
.5 .11502 m
.50375 .11502 L
s
.5 .16986 m
.50375 .16986 L
s
.5 .19729 m
.50375 .19729 L
s
.5 .22471 m
.50375 .22471 L
s
.5 .25213 m
.50375 .25213 L
s
.5 .30698 m
.50375 .30698 L
s
.5 .3344 m
.50375 .3344 L
s
.5 .36182 m
.50375 .36182 L
s
.5 .38925 m
.50375 .38925 L
s
.5 .44409 m
.50375 .44409 L
s
.5 .47151 m
.50375 .47151 L
s
.5 .49894 m
.50375 .49894 L
s
.5 .52636 m
.50375 .52636 L
s
.5 .5812 m
.50375 .5812 L
s
.5 .60863 m
.50375 .60863 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .08909 m
.04262 .10854 L
.06244 .12435 L
.07286 .13068 L
.08255 .13535 L
.08802 .13746 L
.0932 .13911 L
.09916 .14058 L
.10196 .14111 L
.10458 .14152 L
.10699 .14182 L
.10919 .14203 L
.11045 .14213 L
.11164 .14219 L
.1129 .14225 L
.11361 .14227 L
.11428 .14228 L
.11492 .14229 L
.11562 .14229 L
.11688 .14228 L
.11812 .14225 L
.11925 .14221 L
.12048 .14214 L
.12183 .14205 L
.12455 .14179 L
.12696 .14148 L
.12916 .14114 L
.13409 .14015 L
.13866 .13897 L
.14299 .13761 L
.15265 .13378 L
.1629 .12853 L
.18132 .11622 L
.21907 .08169 L
.24005 .05955 L
.25928 .0399 L
.26904 .03105 L
.27812 .02402 L
.28309 .02083 L
.2876 .01843 L
.29261 .01639 L
.29393 .01597 L
.29536 .01559 L
.29672 .01528 L
.29796 .01506 L
.29917 .01489 L
.30032 .01478 L
.30153 .01472 L
.30285 .01472 L
.304 .01478 L
Mistroke
.30523 .0149 L
.30638 .01507 L
.30743 .01527 L
.31001 .01599 L
.31146 .01653 L
.31279 .01712 L
.31541 .01854 L
.31784 .0202 L
.32273 .02457 L
.32808 .03113 L
.3333 .03951 L
.33888 .05093 L
.34908 .07937 L
.35484 .10046 L
.36016 .12365 L
.36993 .1769 L
.3803 .25125 L
.38969 .33784 L
.39981 .45569 L
Mfstroke
.39981 .45569 m
.40986 .61803 L
s
.58944 .61803 m
.59782 .48765 L
.60733 .36972 L
.61631 .28021 L
.62657 .19975 L
.6359 .14335 L
.64594 .09734 L
.65157 .07722 L
.65684 .06158 L
.66629 .04026 L
.6713 .03201 L
.67669 .02519 L
.67939 .0225 L
.68233 .02008 L
.68492 .01835 L
.68768 .0169 L
.68909 .01632 L
.69041 .01585 L
.6917 .01548 L
.69289 .01521 L
.69402 .015 L
.69522 .01485 L
.69641 .01475 L
.69773 .01472 L
.69903 .01475 L
.70024 .01483 L
.70087 .0149 L
.70157 .01499 L
.70297 .01522 L
.70554 .01582 L
.70794 .01658 L
.71242 .01844 L
.71725 .02103 L
.72817 .02874 L
.73817 .03749 L
.77782 .07846 L
.81594 .1141 L
.83557 .12764 L
.84655 .13342 L
.85653 .13745 L
.86138 .13898 L
.86652 .14029 L
.8709 .14115 L
.87338 .14153 L
.87572 .14182 L
.87702 .14195 L
.87843 .14207 L
.87973 .14216 L
.88091 .14222 L
.88163 .14225 L
Mistroke
.88231 .14227 L
.88302 .14228 L
.8838 .14229 L
.88448 .14229 L
.88519 .14229 L
.88647 .14227 L
.88767 .14223 L
.88896 .14216 L
.89017 .14208 L
.89127 .14199 L
.89397 .14171 L
.89653 .14136 L
.90114 .14052 L
.90622 .13927 L
.91649 .13575 L
.92601 .13129 L
.93493 .12608 L
.95517 .11055 L
.97369 .09192 L
.97619 .08909 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00NP3oool4000000<0oooo00<000000?ooo`3oool00P3oool4000009H0
oooo001l0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`2>0?ooo`00O03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00D0oooo0`00
002>0?ooo`00M03oool4000000@0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo08h0oooo001l0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`2>0?ooo`00E03oool7000001l0oooo0`0000090?ooo`04
0000003oool0oooo000000D0oooo00<000000?ooo`3oool0<P3oool7000005D0oooo001B0?ooo`80
00001`3oool2000001l0oooo00<000000?ooo`3oool0203oool2000000H0oooo00<000000?ooo`3o
ool0<03oool2000000L0oooo0P00001C0?ooo`00D@3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo02l0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0500oooo001?0?ooo`8000003@3oool00`000000oooo0?ooo`0^0?ooo`030000003o
ool0oooo02l0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`1?0?ooo`00CP3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0;P3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo04h0oooo001=0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0;@3oool2000002h0oooo00<000000?ooo`3oool03P3oool20000
04h0oooo001<0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0;03oool00`000000
oooo0?ooo`0]0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0B`3oool004/0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo02`0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`1:0?ooo`00BP3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo04T0oooo00190?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0:`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0B03oool004P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0Z0?oo
o`030000003oool0oooo02/0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`170?oo
o`00A`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3o
ool0:P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo04H0oooo00160?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool0:@3oool2000002/0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`160?ooo`00A@3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo04D0oooo00140?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0A03oool0
04<0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`130?ooo`00@`3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo0480oooo00120?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool0@@3oool00440oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0W0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`100?ooo`00@03oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02L0oooo00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo03l0oooo000o0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool09`3oool2000002P0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0n0?ooo`002@3oool00`000000oooo0?ooo`0a0?ooo`80
00009P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool0903oool00`000000oooo0?ooo`0a0?ooo`8000002P3oool000X0oooo00<000000?ooo`3oool0
;`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0
9P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
2P3oool000/0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0X0?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`800000;P3o
ool00`000000oooo0?ooo`0;0?ooo`00303oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo02T0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo02T0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0;0?ooo`003@3oool00`00
0000oooo0?ooo`0Y0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:@3oool00`00
0000oooo0?ooo`0<0?ooo`003P3oool00`000000oooo0?ooo`0V0?ooo`800000;P3oool00`000000
oooo0?ooo`0U0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0;03oool00`000000
oooo0?ooo`0W0?ooo`030000003oool0oooo00d0oooo000?0?ooo`030000003oool0oooo02@0oooo
00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02D0oooo
00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00h0oooo
000@0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?oo
o`0U0?ooo`8000009P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0280oooo0P00
000A0?ooo`004@3oool200000240oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0;`3oool200000240oooo00<00000
0?ooo`3oool04@3oool001<0oooo00<000000?ooo`3oool0703oool2000003@0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0380oooo0P00000M0?oo
o`800000503oool001@0oooo0P00000K0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3o
ool0903oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0D0?ooo`005P3oool2000001L0oooo0P00000g0?ooo`030000
003oool0oooo02@0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0e0?ooo`800000
603oool00`000000oooo0?ooo`0E0?ooo`00603oool200000180oooo0`00000i0?ooo`030000003o
ool0oooo02@0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0g0?ooo`800000503o
ool2000001P0oooo000J0?ooo`8000003P3oool2000003d0oooo00<000000?ooo`3oool0203oool4
000000<0oooo00<000000?ooo`3oool00P3oool400000080oooo0`0000060?ooo`030000003oool0
oooo02<0oooo00<000000?ooo`3oool0>P3oool2000000l0oooo0`00000J0?ooo`00703oool50000
00@0oooo1@00000o0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool08`3oool00`00
0000oooo0?ooo`0l0?ooo`H00000103oool5000001d0oooo000Q0?ooo`D00000@`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`<000008`3oool00`000000oooo0?ooo`110?ooo`D000008P3oool006T0oooo
00<000000?ooo`3oool00P3oool4000000@0oooo00<000000?ooo`3oool02@3oool010000000oooo
0?ooo`3oool3000000H0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`1X0?ooo`00
J@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00D000000?ooo`3oool0
oooo000000080?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0J03oool006T0oooo
00<000000?ooo`3oool0203oool3000000T0oooo0`0000030?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`1X0?ooo`00J@3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool0oooo100000050?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool0J03oool006T0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo06T0oooo001Z0?ooo`030000003oool0
oooo0280oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`1Y0?ooo`00JP3oool00`00
0000oooo0?ooo`0R0?ooo`8000008`3oool00`000000oooo0?ooo`1Y0?ooo`00JP3oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0J@3oool006X0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo06T0oooo
001Z0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08P3oool00`000000oooo0?oo
o`1Y0?ooo`00JP3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0280oooo00<00000
0?ooo`3oool0J@3oool006X0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo06T0oooo001[0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`1Z0?ooo`00J`3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool0JP3oool006/0oooo00<000000?ooo`3oool08@3o
ool200000280oooo00<000000?ooo`3oool0JP3oool006/0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo06X0oooo001[0?ooo`030000003oool0oooo
0240oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`1Z0?ooo`00J`3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0JP3oool006/0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo06X0oooo
001/0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`1[0?ooo`00K03oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool0J`3oool006`0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo06/0oooo001/0?ooo`030000003oool0oooo0200oooo0P00000Q0?ooo`03
0000003oool0oooo06/0oooo001/0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`1[0?ooo`00K03oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo0200oooo00<000000?ooo`3oool0J`3oool006`0oooo00<000000?ooo`3oool0803oool0
0`000000oooo0?ooo`0P0?ooo`030000003oool0oooo06/0oooo001]0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`1[0?ooo`00K@3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0K03oool006d0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo06`0oooo
001]0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`1/0?ooo`00K@3oool00`000000oooo0?ooo`0O0?ooo`800000803oool00`000000oooo0?ooo`1/
0?ooo`00K@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<000000?oo
o`3oool0K03oool006d0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0oooo06`0oooo001]0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`1/0?ooo`00KP3oool00`000000oooo0?ooo`0N0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool0K@3oool006h0oooo00<000000?ooo`3oool02@3oool4
000000<0oooo00<000000?ooo`3oool00P3oool4000000D0oooo00<000000?ooo`3oool07P3oool0
0`000000oooo0?ooo`1]0?ooo`00KP3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
06d0oooo001^0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`040?ooo`<000007P3oool00`000000oooo0?ooo`1]0?ooo`00KP3oool00`000000oooo
0?ooo`030?ooo`@00000103oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`1]0?ooo`00KP3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo06d0oooo001^0?ooo`030000003oool0oooo00T0oooo0`000009
0?ooo`<000001P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo06d0oooo001^0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool0K@3oool006l0oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo06h0oooo001_0?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`1^0?ooo`00K`3o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0KP3o
ool006l0oooo00<000000?ooo`3oool07@3oool2000001h0oooo00<000000?ooo`3oool0KP3oool0
06l0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
06h0oooo001_0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`1^0?ooo`00K`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool0KP3oool006l0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo06h0oooo001`0?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`1^0?ooo`00L03oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool0K`3oool00700oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo06l0oooo001`0?ooo`030000
003oool0oooo01`0oooo0P00000M0?ooo`030000003oool0oooo06l0oooo001`0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`1_0?ooo`00L03oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0K`3oool0
0700oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
06l0oooo001`0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`1_0?ooo`00L03oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool0K`3oool00700oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo0700oooo001a0?ooo`030000003oool0oooo01/0oooo0P00000L
0?ooo`030000003oool0oooo0700oooo001a0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`1`0?ooo`00L@3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool0L03oool00740oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0700oooo001a0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`1`0?ooo`00L@3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0L03oool0
0740oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
0700oooo001a0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`1`0?ooo`00L@3oool00`000000oooo0?ooo`0K0?ooo`800000703oool00`000000oooo
0?ooo`1`0?ooo`00L@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool0L03oool00740oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo0700oooo001b0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`1`0?ooo`00LP3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool0L@3oool00780oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0740oooo001b0?ooo`030000003o
ool0000000<000000`3oool00`000000oooo0?ooo`030?ooo`8000000`3oool3000000H0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`1a0?ooo`00LP3oool01@000000oooo0?ooo`3o
ool0000000T0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool0L@3oool00780oooo00D000000?ooo`3oool0oooo
000000090?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00`3oool3000001X0
oooo00<000000?ooo`3oool0L@3oool006h0oooo1@0000030?ooo`030000003oool0oooo00L0oooo
00@000000?ooo`3oool000000P3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`1a0?ooo`00LP3oool01@000000oooo0?ooo`3oool0000000T0oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0L@3oool00780oooo00<000000?ooo`0000000P0000090?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
0740oooo001b0?ooo`050000003oool0oooo0?ooo`0000002P3oool2000000<0oooo100000050?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0L@3oool00780oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0740oooo001c0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1a0?ooo`00L`3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0L@3o
ool007<0oooo00<000000?ooo`3oool06@3oool2000001/0oooo00<000000?ooo`3oool0L@3oool0
07<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
0740oooo001c0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`1a0?ooo`00L`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool0L@3oool007<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo0780oooo001c0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`1b0?ooo`00L`3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0LP3oool007<0oooo00<000000?ooo`3o
ool06@3oool2000001X0oooo00<000000?ooo`3oool0LP3oool007<0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0780oooo001c0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1b0?ooo`00L`3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0LP3oool0
07@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
0780oooo001d0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`1b0?ooo`00M03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool0LP3oool007@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo0780oooo001d0?ooo`030000003oool0oooo01P0oooo0P00000J
0?ooo`030000003oool0oooo0780oooo001d0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`1b0?ooo`00M03oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0LP3oool007@0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0780oooo001d0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`1c0?ooo`00803oool2
000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0U0?ooo`800000103oool0
0`000000oooo0?ooo`020?ooo`@00000403oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool04@3oool2000000@0oooo00<000000?ooo`3oool00P3oool4
000002H0oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool07@3oool0
01l0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0T0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0403oool010000000oooo0?ooo`0000080?ooo`030000
003oool0oooo02H0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0M0?ooo`00
7`3oool010000000oooo0?ooo`0000070?ooo`D000009@3oool010000000oooo0?ooo`0000090?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo0100oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`0U
0?ooo`040000003oool0oooo000000L0oooo1@00000N0?ooo`006@3oool400000080oooo00@00000
0?ooo`3oool000001`3oool010000000oooo0?ooo`00000P0?ooo`@000000P3oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0603oool2000001T0
oooo00<000000?ooo`3oool0403oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo
02@0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00000O0?ooo`007`3oool0
10000000oooo0?ooo`0000080?ooo`030000003oool0000002H0oooo00@000000?ooo`3oool00000
2`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0@0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3o
ool08`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0000001l0oooo000O0?ooo`04
0000003oool0oooo000000T0oooo0P00000V0?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000403oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0403oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002D0
oooo00@000000?ooo`3oool000002@3oool2000001l0oooo000P0?ooo`8000002`3oool00`000000
oooo0?ooo`0U0?ooo`8000002P3oool200000180oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo0140oooo0P00000:0?ooo`8000009`3oool2000000/0
oooo00<000000?ooo`3oool07@3oool007D0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo07<0oooo001e0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`1c0?ooo`00M@3oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0L`3oool00080ooooo`00000L0000
00<0oooo00090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool2000000`0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0
oooo0P00000<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0203oool002@0oooo
00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0380oooo
00<000000?ooo`3oool08`3oool007D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo07@0oooo001e0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`1d0?ooo`00M@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool0M03oool007D0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo07@0oooo001e0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`1d0?ooo`00M@3o
ool00`000000oooo0?ooo`0G0?ooo`800000603oool00`000000oooo0?ooo`1d0?ooo`00M@3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0M03oool0
07D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
07@0oooo001f0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`1d0?ooo`00MP3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool0M03oool007H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo07D0oooo001f0?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`1e0?ooo`00MP3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool0M@3oool007H0oooo00<000000?ooo`3o
ool05P3oool2000001L0oooo00<000000?ooo`3oool0M@3oool007H0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo07D0oooo001f0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1e0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.535379, -1.20583, \
0.00373087, 0.00129572}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, PlotRange \[Rule] All]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.952381 0.414796 0.334355 [
[.11905 .4023 -12 -9 ]
[.11905 .4023 12 0 ]
[.30952 .4023 -12 -9 ]
[.30952 .4023 12 0 ]
[.69048 .4023 -9 -9 ]
[.69048 .4023 9 0 ]
[.88095 .4023 -9 -9 ]
[.88095 .4023 9 0 ]
[.4875 .08044 -12 -4.5 ]
[.4875 .08044 0 4.5 ]
[.4875 .16403 -30 -4.5 ]
[.4875 .16403 0 4.5 ]
[.4875 .24762 -24 -4.5 ]
[.4875 .24762 0 4.5 ]
[.4875 .33121 -30 -4.5 ]
[.4875 .33121 0 4.5 ]
[.4875 .49839 -24 -4.5 ]
[.4875 .49839 0 4.5 ]
[.4875 .58197 -18 -4.5 ]
[.4875 .58197 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 .4148 m
.11905 .42105 L
s
[(-0.4)] .11905 .4023 0 1 Mshowa
.30952 .4148 m
.30952 .42105 L
s
[(-0.2)] .30952 .4023 0 1 Mshowa
.69048 .4148 m
.69048 .42105 L
s
[(0.2)] .69048 .4023 0 1 Mshowa
.88095 .4148 m
.88095 .42105 L
s
[(0.4)] .88095 .4023 0 1 Mshowa
.125 Mabswid
.16667 .4148 m
.16667 .41855 L
s
.21429 .4148 m
.21429 .41855 L
s
.2619 .4148 m
.2619 .41855 L
s
.35714 .4148 m
.35714 .41855 L
s
.40476 .4148 m
.40476 .41855 L
s
.45238 .4148 m
.45238 .41855 L
s
.54762 .4148 m
.54762 .41855 L
s
.59524 .4148 m
.59524 .41855 L
s
.64286 .4148 m
.64286 .41855 L
s
.7381 .4148 m
.7381 .41855 L
s
.78571 .4148 m
.78571 .41855 L
s
.83333 .4148 m
.83333 .41855 L
s
.07143 .4148 m
.07143 .41855 L
s
.02381 .4148 m
.02381 .41855 L
s
.92857 .4148 m
.92857 .41855 L
s
.97619 .4148 m
.97619 .41855 L
s
.25 Mabswid
0 .4148 m
1 .4148 L
s
.5 .08044 m
.50625 .08044 L
s
[(-1)] .4875 .08044 1 0 Mshowa
.5 .16403 m
.50625 .16403 L
s
[(-0.75)] .4875 .16403 1 0 Mshowa
.5 .24762 m
.50625 .24762 L
s
[(-0.5)] .4875 .24762 1 0 Mshowa
.5 .33121 m
.50625 .33121 L
s
[(-0.25)] .4875 .33121 1 0 Mshowa
.5 .49839 m
.50625 .49839 L
s
[(0.25)] .4875 .49839 1 0 Mshowa
.5 .58197 m
.50625 .58197 L
s
[(0.5)] .4875 .58197 1 0 Mshowa
.125 Mabswid
.5 .01357 m
.50375 .01357 L
s
.5 .03029 m
.50375 .03029 L
s
.5 .04701 m
.50375 .04701 L
s
.5 .06372 m
.50375 .06372 L
s
.5 .09716 m
.50375 .09716 L
s
.5 .11388 m
.50375 .11388 L
s
.5 .13059 m
.50375 .13059 L
s
.5 .14731 m
.50375 .14731 L
s
.5 .18075 m
.50375 .18075 L
s
.5 .19747 m
.50375 .19747 L
s
.5 .21418 m
.50375 .21418 L
s
.5 .2309 m
.50375 .2309 L
s
.5 .26434 m
.50375 .26434 L
s
.5 .28105 m
.50375 .28105 L
s
.5 .29777 m
.50375 .29777 L
s
.5 .31449 m
.50375 .31449 L
s
.5 .34793 m
.50375 .34793 L
s
.5 .36464 m
.50375 .36464 L
s
.5 .38136 m
.50375 .38136 L
s
.5 .39808 m
.50375 .39808 L
s
.5 .43151 m
.50375 .43151 L
s
.5 .44823 m
.50375 .44823 L
s
.5 .46495 m
.50375 .46495 L
s
.5 .48167 m
.50375 .48167 L
s
.5 .5151 m
.50375 .5151 L
s
.5 .53182 m
.50375 .53182 L
s
.5 .54854 m
.50375 .54854 L
s
.5 .56526 m
.50375 .56526 L
s
.5 .59869 m
.50375 .59869 L
s
.5 .61541 m
.50375 .61541 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .02378 m
.04262 .02615 L
.06244 .02808 L
.07286 .02885 L
.08255 .02942 L
.08802 .02968 L
.0932 .02988 L
.09916 .03006 L
.10196 .03013 L
.10458 .03018 L
.10699 .03021 L
.10919 .03024 L
.11045 .03025 L
.11164 .03026 L
.1129 .03026 L
.11361 .03027 L
.11428 .03027 L
.11492 .03027 L
.11562 .03027 L
.11688 .03027 L
.11812 .03027 L
.11925 .03026 L
.12048 .03025 L
.12183 .03024 L
.12455 .03021 L
.12696 .03017 L
.12916 .03013 L
.13409 .03001 L
.13866 .02986 L
.14299 .0297 L
.15265 .02923 L
.1629 .02859 L
.18132 .02709 L
.21907 .02288 L
.24005 .02018 L
.25928 .01779 L
.26904 .01671 L
.27812 .01585 L
.28309 .01546 L
.2876 .01517 L
.29261 .01492 L
.29393 .01487 L
.29536 .01482 L
.29672 .01478 L
.29796 .01476 L
.29917 .01474 L
.30032 .01472 L
.30153 .01472 L
.30285 .01472 L
.304 .01472 L
Mistroke
.30523 .01474 L
.30638 .01476 L
.30743 .01478 L
.31001 .01487 L
.31146 .01494 L
.31279 .01501 L
.31541 .01518 L
.31784 .01538 L
.32273 .01592 L
.32808 .01672 L
.3333 .01774 L
.33888 .01913 L
.34908 .0226 L
.35484 .02517 L
.36016 .028 L
.36993 .03449 L
.3803 .04355 L
.38969 .05411 L
.39981 .06848 L
.4181 .10451 L
.4277 .12996 L
.43818 .16425 L
.44925 .20938 L
.45929 .25994 L
.47894 .39296 L
.48973 .49067 L
.49458 .54161 L
.49715 .5706 L
.49856 .58718 L
.49926 .59551 L
.4999 .60332 L
.50117 .5904 L
.50254 .57422 L
.50494 .54689 L
.50965 .49689 L
.51996 .40203 L
.53842 .27292 L
.54803 .22208 L
.55706 .18242 L
.56709 .14609 L
.57766 .11513 L
.58728 .09237 L
.59782 .07238 L
.60733 .058 L
.61631 .04709 L
.62657 .03728 L
.6359 .0304 L
.64594 .02479 L
.65157 .02234 L
.65684 .02043 L
Mistroke
.66629 .01783 L
.6713 .01682 L
.67669 .01599 L
.67939 .01566 L
.68233 .01537 L
.68492 .01516 L
.68768 .01498 L
.68909 .01491 L
.69041 .01485 L
.6917 .01481 L
.69289 .01477 L
.69402 .01475 L
.69522 .01473 L
.69641 .01472 L
.69773 .01472 L
.69903 .01472 L
.70024 .01473 L
.70087 .01474 L
.70157 .01475 L
.70297 .01478 L
.70554 .01485 L
.70794 .01494 L
.71242 .01517 L
.71725 .01549 L
.72817 .01643 L
.73817 .01749 L
.77782 .02249 L
.81594 .02683 L
.83557 .02848 L
.84655 .02919 L
.85653 .02968 L
.86138 .02987 L
.86652 .03003 L
.8709 .03013 L
.87338 .03018 L
.87572 .03021 L
.87702 .03023 L
.87843 .03024 L
.87973 .03025 L
.88091 .03026 L
.88163 .03026 L
.88231 .03027 L
.88302 .03027 L
.8838 .03027 L
.88448 .03027 L
.88519 .03027 L
.88647 .03027 L
.88767 .03026 L
.88896 .03025 L
.89017 .03024 L
Mistroke
.89127 .03023 L
.89397 .0302 L
.89653 .03016 L
.90114 .03005 L
.90622 .0299 L
.91649 .02947 L
.92601 .02893 L
.93493 .02829 L
.95517 .0264 L
.97369 .02413 L
.97619 .02378 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oS`3oool0
0`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000
oooo0?ooo`2>0?ooo`00S`3oool2000008l0oooo001<0?oooa<00000<03oool00`000000oooo0?oo
o`0^0?oooa800000CP3oool00480oooo2P00000C0?ooo`@00000;03oool00`000000oooo0?ooo`0Y
0?ooo`D000004P3oool7000004L0oooo00070?ooo`<00000<03oool800000240oooo0P00000Z0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0703oool;00000300oooo100000080?oo
o`002P3oool500000280oooo2@00000[0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool09P3oool2000002X0oooo2000000S0?ooo`D00000303oool000l0oooo8P00000e0?ooo`<00000
9P3oool2000002D0oooo0P00000d0?ooob<000004@3oool006T0oooo0P00000T0?ooo`030000003o
ool0oooo0280oooo0P00001Z0?ooo`00J`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0240oooo00<000000?ooo`3oool0JP3oool006`0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo06/0oooo001]0?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`1/0?ooo`00KP3oool00`000000oooo
0?ooo`0N0?ooo`8000007`3oool00`000000oooo0?ooo`1]0?ooo`00KP3oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0KP3oool006l0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo06h0oooo001`0?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`1_0?oo
o`00L03oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool0L03oool00740oooo00<000000?ooo`3oool06`3oool2000001`0oooo00<000000?ooo`3oool0
L03oool00780oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo0740oooo001c0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`1b0?ooo`00L`3oool00`000000oooo0?ooo`0@0?ooo`@000001@3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo0780oooo001d0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo07<0
oooo001d0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0103oool3000001P0oooo
00<000000?ooo`3oool0L`3oool007D0oooo00<000000?ooo`3oool0203oool4000000@0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo07@0oooo001f
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo07@0oooo001f0?ooo`030000003oool0oooo00d0oooo0`0000060?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool0M@3oool007L0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`040?ooo`8000005`3oool00`000000oooo0?ooo`1e0?ooo`00
M`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
MP3oool007P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0E0?ooo`030000003o
ool0oooo07H0oooo001h0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`1g0?ooo`00N03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool0M`3oool007T0oooo00<000000?ooo`3oool04`3oool2000001@0
oooo00<000000?ooo`3oool0N03oool007T0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo07P0oooo001i0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`1h0?ooo`00N@3oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0N@3oool007X0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo07T0oooo001j0?ooo`03
0000003oool0oooo0180oooo0P00000C0?ooo`030000003oool0oooo07T0oooo001j0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`1j0?ooo`00N`3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0NP3o
ool007/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo07/0oooo001k0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`1k0?ooo`00N`3oool00`000000oooo0?ooo`0A0?ooo`8000004@3oool00`000000
oooo0?ooo`1k0?ooo`00O03oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool0O03oool007`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo07`0oooo001e0?ooo`800000103oool2000000@0oooo00<00000
0?ooo`3oool00P3oool3000000H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`1l
0?ooo`00M03oool010000000oooo0?ooo`0000050?ooo`050000003oool0oooo0?ooo`0000001`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0O03o
ool007@0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool00`3oool3000000h0oooo00<000000?ooo`3oool0O@3o
ool006h0oooo100000020?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool00P3o
ool010000000oooo0?ooo`3oool3000000H0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`1m0?ooo`00M03oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo00<0oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo07d0
oooo001d0?ooo`040000003oool0oooo000000H0oooo00<000000?ooo`0000000P3oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0O@3oool007D0
oooo0P0000070?ooo`030000003oool0000000<000000P3oool4000000D0oooo0P00000>0?ooo`03
0000003oool0oooo07h0oooo001n0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`1n0?ooo`00OP3oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0OP3oool007l0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo07h0oooo001o0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`1o0?ooo`00O`3oool00`000000
oooo0?ooo`0=0?ooo`8000003@3oool00`000000oooo0?ooo`1o0?ooo`00O`3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0O`3oool00800oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo07l0oooo0020
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`1o
0?ooo`00P03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0O`3oool00800oooo00<000000?ooo`3oool0303oool2000000`0oooo00<000000?ooo`3o
ool0P03oool00800oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo0800oooo00210?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`200?ooo`00P@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool0P03oool00840oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo0800oooo00210?ooo`030000003oool0oooo00/0
oooo0P00000;0?ooo`030000003oool0oooo0840oooo00210?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`210?ooo`00N`3oool2000000@0oooo0P00
00030?ooo`<000001P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0840oooo001j
0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0P@3oool007X0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo0`00
000:0?ooo`030000003oool0oooo0840oooo001d0?ooo`@000000P3oool010000000oooo0?ooo`00
00040?ooo`040000003oool0oooo0?ooo`<000001P3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0880oooo001j0?ooo`040000003oool0oooo000000D0oooo00@000000?ooo`3oool0
0000203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0880oooo001j0?ooo`040000
003oool0oooo000000D0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo0880oooo001k0?ooo`8000001P3oool00`000000oooo0?ooo`04000000D0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`220?ooo`00P`3oool00`000000oooo
0?ooo`090?ooo`8000002P3oool00`000000oooo0?ooo`220?ooo`00P`3oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0P`3oool008<0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo08<0oooo00240?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`230?oo
o`00Q03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool0P`3oool008@0oooo00<000000?ooo`3oool0203oool2000000T0oooo00<000000?ooo`3oool0
P`3oool008@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo08<0oooo00240?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`240?ooo`00Q03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0Q03oool008D0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo08@0oooo00250?ooo`030000003oool0oooo00L0oooo
0P0000080?ooo`030000003oool0oooo08@0oooo00250?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`240?ooo`00Q@3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0Q03oool008D0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo08@0oooo00250?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`240?ooo`00
QP3oool00`000000oooo0?ooo`060?ooo`8000001`3oool00`000000oooo0?ooo`250?ooo`00QP3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0Q@3o
ool007D0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?ooo`<000001P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo08D0oooo001d0?ooo`040000003oool0oooo
000000P0oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`0000050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool0Q@3oool007@0oooo00@000000?ooo`3oool000002@3o
ool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000D0oooo0`0000060?ooo`030000
003oool0oooo08D0oooo001^0?ooo`@000000P3oool010000000oooo0?ooo`00000:0?ooo`040000
003oool0oooo0?ooo`<000001P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo08D0
oooo001d0?ooo`040000003oool0oooo000000/0oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo08D0oooo001d0?ooo`040000003oool0oooo0000
00P0oooo00@000000?ooo`3oool000000P3oool2000000L0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`260?ooo`00M@3oool2000000X0oooo0P0000030?ooo`@000001@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo08H0oooo00270?ooo`030000003oool0oooo00D0
oooo0P0000060?ooo`030000003oool0oooo08H0oooo00270?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`260?ooo`00Q`3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0QP3oool008L0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo08H0oooo00280?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`260?oo
o`00R03oool00`000000oooo0?ooo`040?ooo`8000001@3oool00`000000oooo0?ooo`270?ooo`00
R03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
Q`3oool008P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo08L0oooo00280?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`270?ooo`007P3oool2000000@0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0V0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000009@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool09P3oool20000
00@0oooo00<000000?ooo`3oool00P3oool4000002L0oooo0P0000040?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool06`3oool001d0oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`0U0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`030?ooo`8000001@3oool00`000000oooo0?ooo`0U0?ooo`040000003oool0
oooo000000P0oooo00<000000?ooo`3oool09`3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo01/0oooo000M0?ooo`040000003oool0oooo000000L0oooo1@00000V0?ooo`040000
003oool0oooo000000T0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool09@3oool010000000oooo0?ooo`0000090?oo
o`030000003oool0oooo02H0oooo00@000000?ooo`3oool000001`3oool5000001`0oooo000G0?oo
o`@000000P3oool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo00000240oooo1000
00020?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool09@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool09@3oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo02D0oooo00@000000?ooo`3oool000001`3oool01000
0000oooo0?ooo`00000M0?ooo`007@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0
000002L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0T0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0V0?ooo`040000003o
ool0oooo000000/0oooo00<000000?ooo`3oool0903oool010000000oooo0?ooo`0000080?ooo`03
0000003oool0000001d0oooo000M0?ooo`040000003oool0oooo000000T0oooo0P00000W0?ooo`04
0000003oool0oooo000000P0oooo00@000000?ooo`3oool000009P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool09P3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo000002H0oooo00@000000?ooo`3oool000002@3oool2000001d0
oooo000N0?ooo`8000002`3oool00`000000oooo0?ooo`0V0?ooo`8000002P3oool2000002L0oooo
00<000000?ooo`3oool00`3oool2000000@0oooo00<000000?ooo`3oool09`3oool2000000X0oooo
0P00000X0?ooo`8000002`3oool00`000000oooo0?ooo`0K0?ooo`00R@3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0R03oool008T0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08P0oooo002:0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`280?oo
o`00o`00000P00000040oooo00070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
1P3oool00280oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0_0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0_0?ooo`030000003o
ool0oooo03<0oooo00<000000?ooo`3oool08@3oool008X0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo08T0oooo002:0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`290?ooo`00RP3oool00`000000
oooo0?ooo`020?ooo`8000000`3oool00`000000oooo0?ooo`290?ooo`00RP3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0R@3oool008X0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo08T0oooo002:
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`29
0?ooo`00R`3oool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool0R@3oool0
08/0oooo00@000000?ooo`3oool0oooo0P0000030?ooo`030000003oool0oooo08T0oooo002;0?oo
o`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`2:0?ooo`00R`3oool01@00
0000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool0RP3oool008/0oooo00D000000?oo
o`3oool0oooo000000030?ooo`030000003oool0oooo08X0oooo002;0?ooo`050000003oool0oooo
0?ooo`0000000`3oool00`000000oooo0?ooo`2:0?ooo`00R`3oool010000000oooo0?ooo`3oool2
00000080oooo00<000000?ooo`3oool0RP3oool008/0oooo00D000000?ooo`3oool0oooo00000003
0?ooo`030000003oool0oooo08X0oooo002;0?ooo`050000003oool0oooo0?ooo`0000000`3oool0
0`000000oooo0?ooo`2:0?ooo`00S03oool010000000oooo0?ooo`0000030?ooo`030000003oool0
oooo08X0oooo002<0?ooo`030000003oool0oooo008000000P3oool00`000000oooo0?ooo`2:0?oo
o`00S03oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo08/0oooo002<0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool0R`3oool007D0oooo0P0000040?ooo`03
0000003oool0oooo0080oooo100000020?ooo`<000000`3oool010000000oooo0?ooo`0000020?oo
o`030000003oool0oooo08/0oooo001d0?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`3oool01P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000008d0oooo001d
0?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool01@3oool010000000oooo0?oo
o`0000020?ooo`@00000S@3oool007@0oooo00@000000?ooo`3oool000002P3oool010000000oooo
0?ooo`3oool3000000<0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`2;0?oo
o`00M03oool010000000oooo0?ooo`00000;0?ooo`040000003oool0oooo000000D0oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`2;0?ooo`00M03oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00`3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo08/0oooo001e0?ooo`8000002P3oool2000000<0oooo
100000020?ooo`060000003oool0oooo0000003oool00000SP3oool008`0oooo00<000000?ooo`3o
ool00`00002>0?ooo`00S@3oool01@000000oooo0000003oool0000008h0oooo002=0?ooo`050000
003oool000000?ooo`000000SP3oool008d0oooo00D000000?ooo`000000oooo0000002>0?ooo`00
S@3oool01@000000oooo0000003oool0000008h0oooo002=0?ooo`030000003oool0000000800000
SP3oool008d0oooo00@000000?ooo`0000000000S`3oool008d0oooo00@000000?ooo`0000000000
S`3oool008d0oooo00@000000?ooo`0000000000S`3oool008d0oooo00@000000?ooo`0000000000
S`3oool008d0oooo00@000000?ooo`0000000000S`3oool008d0oooo00@000000?ooo`0000000000
S`3oool008d0oooo00@000000?ooo`0000000000S`3oool008h0oooo0`00002?0?ooo`00SP3oool3
000008l0oooo002>0?ooo`<00000S`3oool008h0oooo0`00002?0?ooo`00N`3oool2000000@0oooo
00<000000?ooo`3oool00P3oool3000000D0oooo0`00002?0?ooo`00NP3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0080oooo0`00002?0?ooo`00NP3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo00<0oooo0`00002>0?ooo`00NP3oool010000000oooo0?oo
o`0000080?ooo`<000001P3oool00`000000oooo0?ooo`2>0?ooo`00NP3oool010000000oooo0?oo
o`0000080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0SP3oool007X0oooo00@0
00000?ooo`3oool00000203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo08h0oooo
001k0?ooo`8000002@3oool4000000D0oooo00<000000?ooo`3oool0SP3oool008l0oooo0P00002?
0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00
S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool00`000000oooo0?ooo`2>0?ooo`00S`3oool2
000008l0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.526302, -1.24059, \
0.00366761, 0.0104469}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(omdamp2[k_, th_, a_] := 
      om /. FindRoot[
          testfuncspec[IntegerPart[om*10^16]*10^\((\(-16\))\)*I, 
              IntegerPart[k*10^\((16)\)]*10^\((\(-16\))\), th, a] \[Equal] 
            0, {om, 10^\((\(-4\))\), 10^\((\(-3\))\)}, 
          MaxIterations \[Rule] 25]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[Pi/4, \(-9\)/10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.4269833947516716`\)\(\[InvisibleSpace]\)\) - 
      7.72228790963765`*^-55\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?klim2\)\)], "Input"],

Cell["Global`klim2", "Print",
  CellTags->"Info3274767031-9777815"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(klim2[th_, a_] := 
                    k /. \[InvisibleSpace]FindRoot[
                        klim2arg[IntegerPart[k\ 10\^16]\/10\^16, th, a] == 
                          0, {k, 0.1`, 0.5`}, MaxIterations \[Rule] 20]\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "klim2"],
      Editable->False]], "Print",
  CellTags->"Info3274767031-9777815"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[Pi/8, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.1033675516522878`\)\(\[InvisibleSpace]\)\) - 
      1.548208116360902`*^-54\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, omdamp2[k, Pi/8, 10]}, {k, 0.001, 1.1, 0.05}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(25\) iterations."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.001`, \(-0.45042016019678177`\)}, {0.051000000000000004`, 
        0.02494192916694007`}, {0.101`, 0.04841416921539886`}, {0.151`, 
        0.0700911793864591`}, {0.201`, 0.08937608523712091`}, {0.251`, 
        0.1059044858190274`}, {0.301`, 
        0.11950676557122594`}, {0.35100000000000003`, 
        0.13015249089833647`}, {0.401`, 0.13789874396420324`}, {0.451`, 
        0.14285068824661265`}, {0.501`, 0.14513496379322416`}, {0.551`, 
        0.14488299262479104`}, {0.601`, 0.14222123591286565`}, {0.651`, 
        0.13726578874556472`}, {0.7010000000000001`, 
        0.13011932479638252`}, {0.751`, 0.12086971919840503`}, {0.801`, 
        0.10958931565734842`}, {0.8510000000000001`, 
        0.09633404221530083`}, {0.901`, 
        0.08114298725829854`}, {0.9510000000000001`, 
        0.06403737558203748`}, {1.0010000000000001`, 
        0.04501877021394251`}, {1.0510000000000002`, 
        0.024066918362112697`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0, 0}, {0.051000000000000004`, 0.02494192916694007`}, {0.101`, 
        0.04841416921539886`}, {0.151`, 0.0700911793864591`}, {0.201`, 
        0.08937608523712091`}, {0.251`, 0.1059044858190274`}, {0.301`, 
        0.11950676557122594`}, {0.35100000000000003`, 
        0.13015249089833647`}, {0.401`, 0.13789874396420324`}, {0.451`, 
        0.14285068824661265`}, {0.501`, 0.14513496379322416`}, {0.551`, 
        0.14488299262479104`}, {0.601`, 0.14222123591286565`}, {0.651`, 
        0.13726578874556472`}, {0.7010000000000001`, 
        0.13011932479638252`}, {0.751`, 0.12086971919840503`}, {0.801`, 
        0.10958931565734842`}, {0.8510000000000001`, 
        0.09633404221530083`}, {0.901`, 
        0.08114298725829854`}, {0.9510000000000001`, 
        0.06403737558203748`}, {1.0010000000000001`, 
        0.04501877021394251`}, {1.0510000000000002`, 
        0.024066918362112697`}, {1.1033675516522878`\[InvisibleSpace], 
        0}}\)], "Input"],

Cell[BoxData[
    \({{0, 0}, {0.051000000000000004`, 0.02494192916694007`}, {0.101`, 
        0.04841416921539886`}, {0.151`, 0.0700911793864591`}, {0.201`, 
        0.08937608523712091`}, {0.251`, 0.1059044858190274`}, {0.301`, 
        0.11950676557122594`}, {0.35100000000000003`, 
        0.13015249089833647`}, {0.401`, 0.13789874396420324`}, {0.451`, 
        0.14285068824661265`}, {0.501`, 0.14513496379322416`}, {0.551`, 
        0.14488299262479104`}, {0.601`, 0.14222123591286565`}, {0.651`, 
        0.13726578874556472`}, {0.7010000000000001`, 
        0.13011932479638252`}, {0.751`, 0.12086971919840503`}, {0.801`, 
        0.10958931565734842`}, {0.8510000000000001`, 
        0.09633404221530083`}, {0.901`, 
        0.08114298725829854`}, {0.9510000000000001`, 
        0.06403737558203748`}, {1.0010000000000001`, 
        0.04501877021394251`}, {1.0510000000000002`, 
        0.024066918362112697`}, {1.1033675516522878`, 0}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.863158 0.0147151 4.05556 [
[.19644 .00222 -9 -9 ]
[.19644 .00222 9 0 ]
[.36907 .00222 -9 -9 ]
[.36907 .00222 9 0 ]
[.5417 .00222 -9 -9 ]
[.5417 .00222 9 0 ]
[.71434 .00222 -9 -9 ]
[.71434 .00222 9 0 ]
[.88697 .00222 -3 -9 ]
[.88697 .00222 3 0 ]
[.01131 .09583 -24 -4.5 ]
[.01131 .09583 0 4.5 ]
[.01131 .17694 -24 -4.5 ]
[.01131 .17694 0 4.5 ]
[.01131 .25805 -24 -4.5 ]
[.01131 .25805 0 4.5 ]
[.01131 .33916 -24 -4.5 ]
[.01131 .33916 0 4.5 ]
[.01131 .42027 -18 -4.5 ]
[.01131 .42027 0 4.5 ]
[.01131 .50138 -24 -4.5 ]
[.01131 .50138 0 4.5 ]
[.01131 .58249 -24 -4.5 ]
[.01131 .58249 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.19644 .01472 m
.19644 .02097 L
s
[(0.2)] .19644 .00222 0 1 Mshowa
.36907 .01472 m
.36907 .02097 L
s
[(0.4)] .36907 .00222 0 1 Mshowa
.5417 .01472 m
.5417 .02097 L
s
[(0.6)] .5417 .00222 0 1 Mshowa
.71434 .01472 m
.71434 .02097 L
s
[(0.8)] .71434 .00222 0 1 Mshowa
.88697 .01472 m
.88697 .02097 L
s
[(1)] .88697 .00222 0 1 Mshowa
.125 Mabswid
.06697 .01472 m
.06697 .01847 L
s
.11013 .01472 m
.11013 .01847 L
s
.15328 .01472 m
.15328 .01847 L
s
.2396 .01472 m
.2396 .01847 L
s
.28276 .01472 m
.28276 .01847 L
s
.32591 .01472 m
.32591 .01847 L
s
.41223 .01472 m
.41223 .01847 L
s
.45539 .01472 m
.45539 .01847 L
s
.49855 .01472 m
.49855 .01847 L
s
.58486 .01472 m
.58486 .01847 L
s
.62802 .01472 m
.62802 .01847 L
s
.67118 .01472 m
.67118 .01847 L
s
.75749 .01472 m
.75749 .01847 L
s
.80065 .01472 m
.80065 .01847 L
s
.84381 .01472 m
.84381 .01847 L
s
.93013 .01472 m
.93013 .01847 L
s
.97328 .01472 m
.97328 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .09583 m
.03006 .09583 L
s
[(0.02)] .01131 .09583 1 0 Mshowa
.02381 .17694 m
.03006 .17694 L
s
[(0.04)] .01131 .17694 1 0 Mshowa
.02381 .25805 m
.03006 .25805 L
s
[(0.06)] .01131 .25805 1 0 Mshowa
.02381 .33916 m
.03006 .33916 L
s
[(0.08)] .01131 .33916 1 0 Mshowa
.02381 .42027 m
.03006 .42027 L
s
[(0.1)] .01131 .42027 1 0 Mshowa
.02381 .50138 m
.03006 .50138 L
s
[(0.12)] .01131 .50138 1 0 Mshowa
.02381 .58249 m
.03006 .58249 L
s
[(0.14)] .01131 .58249 1 0 Mshowa
.125 Mabswid
.02381 .03499 m
.02756 .03499 L
s
.02381 .05527 m
.02756 .05527 L
s
.02381 .07555 m
.02756 .07555 L
s
.02381 .1161 m
.02756 .1161 L
s
.02381 .13638 m
.02756 .13638 L
s
.02381 .15666 m
.02756 .15666 L
s
.02381 .19722 m
.02756 .19722 L
s
.02381 .21749 m
.02756 .21749 L
s
.02381 .23777 m
.02756 .23777 L
s
.02381 .27833 m
.02756 .27833 L
s
.02381 .2986 m
.02756 .2986 L
s
.02381 .31888 m
.02756 .31888 L
s
.02381 .35944 m
.02756 .35944 L
s
.02381 .37972 m
.02756 .37972 L
s
.02381 .39999 m
.02756 .39999 L
s
.02381 .44055 m
.02756 .44055 L
s
.02381 .46083 m
.02756 .46083 L
s
.02381 .4811 m
.02756 .4811 L
s
.02381 .52166 m
.02756 .52166 L
s
.02381 .54194 m
.02756 .54194 L
s
.02381 .56222 m
.02756 .56222 L
s
.02381 .60277 m
.02756 .60277 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06783 .11587 L
.11099 .21106 L
.15415 .29897 L
.1973 .37719 L
.24046 .44422 L
.28362 .49938 L
.32678 .54256 L
.36994 .57397 L
.41309 .59405 L
.45625 .60332 L
.49941 .6023 L
.54257 .5915 L
.58573 .5714 L
.62888 .54242 L
.67204 .50491 L
.7152 .45916 L
.75836 .4054 L
.80152 .3438 L
.84467 .27442 L
.88783 .19729 L
.93099 .11232 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo0012ool20004ool00`00ooooo`02ool4000Oool20004ool00`00oooo
o`04ool00`00ooooo`0Nool20004ool00`00ooooo`03ool2000Pool20004ool00`00ooooo`03ool2
000Tool4000Nool0047oo`04003ooooo0008ool00`00ooooo`0Oool01000ooooo`002_oo00<00?oo
ool07Ooo00@00?ooool000Soo`04003ooooo000Nool01000ooooo`002?oo00@00?ooool002Goo`03
003ooooo01goo`00@Ooo00@00?ooool000Woo`03003ooooo01koo`04003ooooo0007ool5000Nool0
1000ooooo`002?oo00@00?ooool001koo`04003ooooo0008ool01000ooooo`009Ooo00<00?ooool0
7Ooo0011ool01000ooooo`002_oo00<00?ooool07Ooo00@00?ooool000Ooo`04003ooooo000Oool0
1000ooooo`002?oo0`007ooo00@00?ooool000Woo`8002Koo`03003ooooo01goo`00@Ooo00@00?oo
ool000_oo`03003ooooo01coo`04003ooooo0008ool00`00ool0000Oool01000ooooo`002?oo00<0
0?ooool07ooo00@00?ooool000Soo`04003ooooo000Uool00`00ooooo`0Mool0047oo`04003ooooo
0008ool01000ooooo`007_oo00@00?ooool000Woo`8001ooo`04003ooooo0008ool00`00ooooo`0O
ool01000ooooo`002?oo00@00?ooool002?oo`<001ooo`006ooo00<00?ooool09?oo0P002_oo0P00
8?oo0P002ooo00<00?ooool07_oo0P002_oo0`007ooo0P002_oo0P009_oo00<00?ooool07Ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000D
oooo000=000001_oo`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00Soo`03003ooooo00Wo
o`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00So
o`03003ooooo00Woo`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00Soo`03003ooooo00Wo
o`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Wo
o`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00Soo`8000Ooo`006ooo0P00;?oo00<00?oo
ool0:ooo00<00?ooool0:ooo00<00?ooool0:ooo00<00?ooool0:_oo00<00?ooool05?oo00<00?oo
ool01_oo000Kool2003jool00`00ooooo`06ool001_oo`03003oo`000?Soo`03003ooooo00Ooo`00
6ooo0`00n?oo00<00?ooool01ooo000Kool01000ooooo`00m_oo00<00?ooool02?oo000Kool01000
ooooo`00m_oo00<00?ooool02?oo000Kool01@00oooooooo0000m?oo00<00?ooool02Ooo000Kool0
1@00oooooooo0000m?oo00<00?ooool02Ooo000Kool01@00oooooooo0000looo00<00?ooool02_oo
000Kool20003ool00`00ooooo`3`ool00`00ooooo`0:ool001_oo`03003ooooo00;oo`03003ooooo
0>ooo`03003ooooo00_oo`006ooo00<00?ooool00ooo00<00?ooool0k_oo00<00?ooool02ooo000K
ool00`00ooooo`03ool00`00ooooo`3^ool00`00ooooo`0;ool001_oo`03003ooooo00Coo`03003o
oooo0>coo`03003ooooo00coo`006ooo0P001Ooo00<00?ooool0k?oo00<00?ooool03?oo000Kool0
0`00ooooo`05ool00`00ooooo`3Zool00`00ooooo`0=ool001_oo`03003ooooo00Goo`03003ooooo
0>[oo`03003ooooo00goo`006ooo00<00?ooool01Ooo00<00?ooool0jOoo00<00?ooool03_oo0002
ool20004ool00`00ooooo`03ool20003ool40004ool00`00ooooo`06ool00`00ooooo`3Xool00`00
ooooo`0>ool00005ool00?ooool00008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0
1_oo00<00?ooool0iooo00<00?ooool03ooo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03
003ooooo00Coo`8000Soo`03003ooooo0>Koo`03003ooooo00ooo`0000Goo`00ooooo`0000Soo`04
003ooooo0004ool00`00ooooo`03ool00`00ooooo`07ool00`00ooooo`3Uool00`00ooooo`0@ool0
0005ool00?ooool00008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool02?oo00<00?oo
ool0i?oo00<00?ooool04?oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0004
ool00`00ooooo`08ool00`00ooooo`3Sool00`00ooooo`0Aool000;oo`8000[oo`8000Coo`8000Go
o`03003ooooo00Woo`03003ooooo0>;oo`03003ooooo017oo`006ooo0P002_oo00<00?ooool0h_oo
00<00?ooool04Ooo000Kool00`00ooooo`09ool00`00ooooo`3Qool00`00ooooo`0Bool001_oo`03
003ooooo00[oo`03003ooooo0>3oo`03003ooooo01;oo`006ooo00<00?ooool02_oo00<00?ooool0
gooo00<00?ooool04ooo000Kool00`00ooooo`0;ool00`00ooooo`3Nool00`00ooooo`0Cool001_o
o`8000coo`03003ooooo0=goo`03003ooooo01Coo`006ooo00<00?ooool03?oo00<00?ooool0g?oo
00<00?ooool05?oo000Kool00`00ooooo`0<ool00`00ooooo`3Kool00`00ooooo`0Eool001_oo`03
003ooooo00goo`03003ooooo0=[oo`03003ooooo01Goo`006ooo00<00?ooool03Ooo00<00?ooool0
fOoo00<00?ooool05_oo000Kool00`00ooooo`0=ool00`00ooooo`3Iool00`00ooooo`0Fool001_o
o`8000ooo`03003ooooo0=Ooo`03003ooooo01Ooo`006ooo00<00?ooool03_oo00<00?ooool0eooo
00<00?ooool05ooo000Kool00`00ooooo`0?ool00`00ooooo`3Eool00`00ooooo`0Hool000;oo`80
00Coo`03003ooooo00?oo`8000Goo`03003ooooo00?oo`03003ooooo00ooo`03003ooooo0=Goo`03
003ooooo01Soo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`03ool00`00oooo
o`0@ool00`00ooooo`3Cool00`00ooooo`0Iool00005ool00?ooool00008ool01@00ooooo`00ool0
1@001?oo0P004Ooo00<00?ooool0dooo00<00?ooool06Ooo00001Ooo003ooooo00002?oo00H00?oo
ool00?oo0002ool00`00ooooo`03ool00`00ooooo`0@ool00`00ooooo`3Bool00`00ooooo`0Jool0
0005ool00?ooool00008ool01000ooooo`000_oo00<00?oo00001Ooo00<00?ooool04Ooo00<00?oo
ool0dOoo00<00?ooool06_oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`8000Goo`03003o
oooo017oo`03003ooooo0=3oo`03003ooooo01_oo`000_oo0P002_oo0P001Ooo00<00?ooool00ooo
00<00?ooool04_oo00<00?ooool0cooo00<00?ooool06ooo000Kool2000Cool00`00ooooo`3>ool0
0`00ooooo`0Lool001_oo`03003ooooo01?oo`03003ooooo0<goo`03003ooooo01coo`006ooo00<0
0?ooool04ooo00<00?ooool0c?oo00<00?ooool07Ooo000Kool00`00ooooo`0Dool00`00ooooo`3;
ool00`00ooooo`0Mool001_oo`03003ooooo01Coo`03003ooooo0<[oo`03003ooooo01koo`006ooo
00<00?ooool05?oo00<00?ooool0b_oo00<00?ooool07_oo000Kool2000Fool00`00ooooo`38ool0
0`00ooooo`0Oool001_oo`03003ooooo01Goo`03003ooooo0<Ooo`03003ooooo023oo`006ooo00<0
0?ooool05_oo00<00?ooool0a_oo00<00?ooool08?oo000Kool00`00ooooo`0Fool00`00ooooo`35
ool00`00ooooo`0Qool001_oo`03003ooooo01Ooo`03003ooooo0<Coo`03003ooooo027oo`006ooo
0P006?oo00<00?ooool0`ooo00<00?ooool08_oo000Kool00`00ooooo`0Hool00`00ooooo`31ool0
0`00ooooo`0Sool001_oo`03003ooooo01Soo`03003ooooo0<7oo`03003ooooo02?oo`006ooo00<0
0?ooool06Ooo00<00?ooool0_ooo00<00?ooool09?oo0002ool20004ool00`00ooooo`03ool20004
ool20005ool00`00ooooo`0Iool00`00ooooo`2oool00`00ooooo`0Tool00005ool00?ooool00008
ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo01Woo`03003ooooo0;koo`03003ooooo
02Goo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001?oo0P006ooo00<00?oo
ool0_?oo00<00?ooool09_oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`<000Goo`03003o
oooo01[oo`03003ooooo0;coo`03003ooooo02Koo`0000Goo`00ooooo`0000Soo`04003ooooo0002
ool00`00ooooo`05ool00`00ooooo`0Kool00`00ooooo`2jool00`00ooooo`0Wool00005ool00?oo
ool00008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool06ooo00<00?ooool0^_oo00<0
0?ooool09ooo0002ool2000:ool20004ool30004ool00`00ooooo`0Lool00`00ooooo`2hool00`00
ooooo`0Xool001_oo`8001goo`03003ooooo0;Ooo`03003ooooo02Woo`006ooo00<00?ooool07Ooo
00<00?ooool0]_oo00<00?ooool0:Ooo000Kool00`00ooooo`0Mool00`00ooooo`2eool00`00oooo
o`0Zool001_oo`03003ooooo01koo`03003ooooo0;Coo`03003ooooo02[oo`006ooo00<00?ooool0
7_oo00<00?ooool0/ooo00<00?ooool0:ooo000Kool2000Pool00`00ooooo`2aool00`00ooooo`0/
ool001_oo`03003ooooo01ooo`03003ooooo0;7oo`03003ooooo02coo`006ooo00<00?ooool08?oo
00<00?ooool0[ooo00<00?ooool0;Ooo000Kool00`00ooooo`0Pool00`00ooooo`2_ool00`00oooo
o`0]ool001_oo`03003ooooo027oo`03003ooooo0:goo`03003ooooo02koo`006ooo00<00?ooool0
8Ooo00<00?ooool0[?oo00<00?ooool0;ooo000Kool2000Sool00`00ooooo`2[ool00`00ooooo`0_
ool001_oo`03003ooooo02?oo`03003ooooo0:Woo`03003ooooo033oo`006ooo00<00?ooool08ooo
00<00?ooool0Z?oo00<00?ooool0<Ooo0002ool20004ool00`00ooooo`03ool20004ool20005ool0
0`00ooooo`0Tool00`00ooooo`2Wool00`00ooooo`0aool00005ool00?ooool00008ool01000oooo
o`000_oo00@00?ooool000Coo`03003ooooo02Coo`03003ooooo0:Koo`03003ooooo03;oo`0000Go
o`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001?oo0P009_oo00<00?ooool0YOoo00<0
0?ooool0<_oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`8000Goo`03003ooooo02Koo`03
003ooooo0:?oo`03003ooooo03?oo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000oooo
o`001?oo00<00?ooool09_oo00<00?ooool0X_oo00<00?ooool0=?oo00001Ooo003ooooo00002?oo
00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`0Wool00`00ooooo`2Qool00`00ooooo`0d
ool000;oo`8000[oo`8000Coo`8000Goo`03003ooooo02Ooo`03003ooooo0:3oo`03003ooooo03Go
o`006ooo00<00?ooool0:?oo00<00?ooool0W_oo00<00?ooool0=_oo000Kool2000Zool00`00oooo
o`2Lool00`00ooooo`0gool001_oo`03003ooooo02Woo`03003ooooo09coo`03003ooooo03Ooo`00
6ooo00<00?ooool0:_oo00<00?ooool0V_oo00<00?ooool0>?oo000Kool00`00ooooo`0Zool00`00
ooooo`2Iool00`00ooooo`0iool001_oo`03003ooooo02_oo`03003ooooo09Soo`03003ooooo03Wo
o`006ooo0P00;Ooo00<00?ooool0U_oo00<00?ooool0>_oo000Kool00`00ooooo`0/ool00`00oooo
o`2Eool00`00ooooo`0kool001_oo`03003ooooo02goo`03003ooooo09Coo`03003ooooo03_oo`00
6ooo00<00?ooool0;Ooo00<00?ooool0Tooo00<00?ooool0??oo000Kool00`00ooooo`0^ool00`00
ooooo`2Aool00`00ooooo`0mool001_oo`80033oo`03003ooooo08ooo`03003ooooo03koo`006ooo
00<00?ooool0;ooo00<00?ooool0Sooo00<00?ooool0?_oo000Kool00`00ooooo`0`ool00`00oooo
o`2=ool00`00ooooo`0oool001_oo`03003ooooo033oo`03003ooooo08coo`03003ooooo043oo`00
2?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool0<Ooo00<00?ooool0R_oo00<00?ooool0
@Ooo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool0<_oo00<00?ooool0ROoo00<0
0?ooool0@Ooo0007ool01000ooooo`002_oo00<00?ooool00ooo0P00<ooo00<00?ooool0R?oo00<0
0?ooool0@_oo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool0<ooo00<00?ooool0
Q_oo00<00?ooool0@ooo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool0=?oo00<0
0?ooool0Q?oo00<00?ooool0A?oo0007ool01000ooooo`002?oo0`001Ooo00<00?ooool0=?oo00<0
0?ooool0Q?oo00<00?ooool0A?oo0008ool2000;ool00`00ooooo`03ool00`00ooooo`0eool00`00
ooooo`22ool00`00ooooo`15ool001_oo`8003Koo`03003ooooo087oo`03003ooooo04Koo`006ooo
00<00?ooool0=_oo00<00?ooool0Oooo00<00?ooool0Aooo000Kool00`00ooooo`0gool00`00oooo
o`1mool00`00ooooo`18ool001_oo`03003ooooo03Soo`03003ooooo07coo`03003ooooo04Soo`00
6ooo00<00?ooool0>?oo00<00?ooool0Nooo00<00?ooool0BOoo000Kool00`00ooooo`0iool00`00
ooooo`1iool00`00ooooo`1:ool001_oo`8003_oo`03003ooooo07Ooo`03003ooooo04_oo`006ooo
00<00?ooool0>ooo00<00?ooool0MOoo00<00?ooool0C?oo000Kool00`00ooooo`0lool00`00oooo
o`1cool00`00ooooo`1=ool001_oo`03003ooooo03coo`03003ooooo07;oo`03003ooooo04koo`00
6ooo00<00?ooool0?Ooo00<00?ooool0L?oo00<00?ooool0Cooo000Kool2000oool00`00ooooo`1^
ool00`00ooooo`1@ool001_oo`03003ooooo03ooo`03003ooooo06coo`03003ooooo057oo`006ooo
00<00?ooool0@?oo00<00?ooool0J_oo00<00?ooool0D_oo0002ool20004ool00`00ooooo`02ool4
0002ool40004ool00`00ooooo`10ool00`00ooooo`1Yool00`00ooooo`1Cool00005ool00?ooool0
000:ool01@00oooooooo00001ooo00<00?ooool0@Ooo00<00?ooool0Iooo00<00?ooool0E?oo0000
1Ooo003ooooo00002_oo00<00?ooool00_oo00<00?ooool01?oo0P00@ooo00<00?ooool0IOoo00<0
0?ooool0EOoo00001Ooo003ooooo00002_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool0
@ooo00<00?ooool0Hooo00<00?ooool0E_oo00001Ooo003ooooo00002_oo00<00?ooool01?oo00<0
0?ooool00_oo00<00?ooool0A?oo00<00?ooool0HOoo00<00?ooool0Eooo00001Ooo003ooooo0000
2?oo0`000ooo00@00?ooool000Coo`03003ooooo04Goo`03003ooooo05ooo`03003ooooo05Soo`00
0_oo0P002ooo00<00?ooool00_oo0P001Ooo00<00?ooool0A_oo00<00?ooool0GOoo00<00?ooool0
FOoo000Kool00`00ooooo`17ool00`00ooooo`1Kool00`00ooooo`1Jool001_oo`8004Woo`03003o
oooo05Soo`8005goo`006ooo00<00?ooool0BOoo00<00?ooool0E_oo00<00?ooool0GOoo000Kool0
0`00ooooo`1:ool00`00ooooo`1Dool00`00ooooo`1Nool001_oo`03003ooooo04_oo`03003ooooo
05;oo`03003ooooo05ooo`006ooo00<00?ooool0C?oo00<00?ooool0D?oo00<00?ooool0H?oo000K
ool2001>ool00`00ooooo`1>ool00`00ooooo`1Qool001_oo`03003ooooo04koo`03003ooooo04_o
o`8006Coo`006ooo00<00?ooool0Cooo0P00B_oo00<00?ooool0I?oo000Kool00`00ooooo`1Aool0
0`00ooooo`15ool2001Wool001_oo`03003ooooo05;oo`03003ooooo04?oo`03003ooooo06Ooo`00
6ooo00<00?ooool0Dooo0P00@Ooo0P00J_oo000Kool2001Fool00`00ooooo`0mool00`00ooooo`1Z
ool001_oo`03003ooooo05Koo`03003ooooo03[oo`8006goo`006ooo00<00?ooool0Eooo0P00>?oo
0P00Kooo0002ool20004ool00`00ooooo`02ool40004ool00`00ooooo`03ool00`00ooooo`1Iool2
000dool2001aool00005ool00?ooool0000:ool00`00ooooo`03ool00`00ooooo`03ool00`00oooo
o`1Kool2000`ool2001cool00005ool00?ooool0000:ool00`00ooooo`050004ool2001Nool2000/
ool2001eool00005ool00?ooool0000:ool01000ooooo`000_oo00<00?ooool00ooo00<00?ooool0
Gooo0P00:?oo0P00Mooo00001Ooo003ooooo00002_oo00L00?ooooooo`00ool00005ool00`00oooo
o`1Qool2000Rool4001iool00005ool00?ooool00008ool30004ool20005ool00`00ooooo`1Sool3
000Kool4001mool000;oo`8000_oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo06Koo`@0
01?oo`@0087oo`006ooo0P00Jooo10002?oo1`00QOoo000Kool00`00ooooo`1^ool8002<ool001_o
o`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool00?oo
ob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.119014, -0.013337, \
0.0043553, 0.000926953}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%119, %117, Frame \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.761296 0.0147151 3.54937 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.17607 -0.0125 -9 -9 ]
[.17607 -0.0125 9 0 ]
[.32833 -0.0125 -9 -9 ]
[.32833 -0.0125 9 0 ]
[.48059 -0.0125 -9 -9 ]
[.48059 -0.0125 9 0 ]
[.63285 -0.0125 -9 -9 ]
[.63285 -0.0125 9 0 ]
[.78511 -0.0125 -3 -9 ]
[.78511 -0.0125 3 0 ]
[.93736 -0.0125 -9 -9 ]
[.93736 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .10345 -30 -4.5 ]
[-0.0125 .10345 0 4.5 ]
[-0.0125 .19218 -24 -4.5 ]
[-0.0125 .19218 0 4.5 ]
[-0.0125 .28092 -30 -4.5 ]
[-0.0125 .28092 0 4.5 ]
[-0.0125 .36965 -18 -4.5 ]
[-0.0125 .36965 0 4.5 ]
[-0.0125 .45839 -30 -4.5 ]
[-0.0125 .45839 0 4.5 ]
[-0.0125 .54712 -24 -4.5 ]
[-0.0125 .54712 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.17607 0 m
.17607 .00625 L
s
[(0.2)] .17607 -0.0125 0 1 Mshowa
.32833 0 m
.32833 .00625 L
s
[(0.4)] .32833 -0.0125 0 1 Mshowa
.48059 0 m
.48059 .00625 L
s
[(0.6)] .48059 -0.0125 0 1 Mshowa
.63285 0 m
.63285 .00625 L
s
[(0.8)] .63285 -0.0125 0 1 Mshowa
.78511 0 m
.78511 .00625 L
s
[(1)] .78511 -0.0125 0 1 Mshowa
.93736 0 m
.93736 .00625 L
s
[(1.2)] .93736 -0.0125 0 1 Mshowa
.125 Mabswid
.06187 0 m
.06187 .00375 L
s
.09994 0 m
.09994 .00375 L
s
.138 0 m
.138 .00375 L
s
.21413 0 m
.21413 .00375 L
s
.2522 0 m
.2522 .00375 L
s
.29026 0 m
.29026 .00375 L
s
.36639 0 m
.36639 .00375 L
s
.40446 0 m
.40446 .00375 L
s
.44252 0 m
.44252 .00375 L
s
.51865 0 m
.51865 .00375 L
s
.55672 0 m
.55672 .00375 L
s
.59478 0 m
.59478 .00375 L
s
.67091 0 m
.67091 .00375 L
s
.70898 0 m
.70898 .00375 L
s
.74704 0 m
.74704 .00375 L
s
.82317 0 m
.82317 .00375 L
s
.86123 0 m
.86123 .00375 L
s
.8993 0 m
.8993 .00375 L
s
.97543 0 m
.97543 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .10345 m
.00625 .10345 L
s
[(0.025)] -0.0125 .10345 1 0 Mshowa
0 .19218 m
.00625 .19218 L
s
[(0.05)] -0.0125 .19218 1 0 Mshowa
0 .28092 m
.00625 .28092 L
s
[(0.075)] -0.0125 .28092 1 0 Mshowa
0 .36965 m
.00625 .36965 L
s
[(0.1)] -0.0125 .36965 1 0 Mshowa
0 .45839 m
.00625 .45839 L
s
[(0.125)] -0.0125 .45839 1 0 Mshowa
0 .54712 m
.00625 .54712 L
s
[(0.15)] -0.0125 .54712 1 0 Mshowa
.125 Mabswid
0 .03246 m
.00375 .03246 L
s
0 .05021 m
.00375 .05021 L
s
0 .06796 m
.00375 .06796 L
s
0 .0857 m
.00375 .0857 L
s
0 .1212 m
.00375 .1212 L
s
0 .13894 m
.00375 .13894 L
s
0 .15669 m
.00375 .15669 L
s
0 .17444 m
.00375 .17444 L
s
0 .20993 m
.00375 .20993 L
s
0 .22768 m
.00375 .22768 L
s
0 .24542 m
.00375 .24542 L
s
0 .26317 m
.00375 .26317 L
s
0 .29866 m
.00375 .29866 L
s
0 .31641 m
.00375 .31641 L
s
0 .33416 m
.00375 .33416 L
s
0 .35191 m
.00375 .35191 L
s
0 .3874 m
.00375 .3874 L
s
0 .40515 m
.00375 .40515 L
s
0 .42289 m
.00375 .42289 L
s
0 .44064 m
.00375 .44064 L
s
0 .47613 m
.00375 .47613 L
s
0 .49388 m
.00375 .49388 L
s
0 .51163 m
.00375 .51163 L
s
0 .52937 m
.00375 .52937 L
s
0 .56487 m
.00375 .56487 L
s
0 .58261 m
.00375 .58261 L
s
0 .60036 m
.00375 .60036 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.17607 .61178 m
.17607 .61803 L
s
.32833 .61178 m
.32833 .61803 L
s
.48059 .61178 m
.48059 .61803 L
s
.63285 .61178 m
.63285 .61803 L
s
.78511 .61178 m
.78511 .61803 L
s
.93736 .61178 m
.93736 .61803 L
s
.125 Mabswid
.06187 .61428 m
.06187 .61803 L
s
.09994 .61428 m
.09994 .61803 L
s
.138 .61428 m
.138 .61803 L
s
.21413 .61428 m
.21413 .61803 L
s
.2522 .61428 m
.2522 .61803 L
s
.29026 .61428 m
.29026 .61803 L
s
.36639 .61428 m
.36639 .61803 L
s
.40446 .61428 m
.40446 .61803 L
s
.44252 .61428 m
.44252 .61803 L
s
.51865 .61428 m
.51865 .61803 L
s
.55672 .61428 m
.55672 .61803 L
s
.59478 .61428 m
.59478 .61803 L
s
.67091 .61428 m
.67091 .61803 L
s
.70898 .61428 m
.70898 .61803 L
s
.74704 .61428 m
.74704 .61803 L
s
.82317 .61428 m
.82317 .61803 L
s
.86123 .61428 m
.86123 .61803 L
s
.8993 .61428 m
.8993 .61803 L
s
.97543 .61428 m
.97543 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .10345 m
1 .10345 L
s
.99375 .19218 m
1 .19218 L
s
.99375 .28092 m
1 .28092 L
s
.99375 .36965 m
1 .36965 L
s
.99375 .45839 m
1 .45839 L
s
.99375 .54712 m
1 .54712 L
s
.125 Mabswid
.99625 .03246 m
1 .03246 L
s
.99625 .05021 m
1 .05021 L
s
.99625 .06796 m
1 .06796 L
s
.99625 .0857 m
1 .0857 L
s
.99625 .1212 m
1 .1212 L
s
.99625 .13894 m
1 .13894 L
s
.99625 .15669 m
1 .15669 L
s
.99625 .17444 m
1 .17444 L
s
.99625 .20993 m
1 .20993 L
s
.99625 .22768 m
1 .22768 L
s
.99625 .24542 m
1 .24542 L
s
.99625 .26317 m
1 .26317 L
s
.99625 .29866 m
1 .29866 L
s
.99625 .31641 m
1 .31641 L
s
.99625 .33416 m
1 .33416 L
s
.99625 .35191 m
1 .35191 L
s
.99625 .3874 m
1 .3874 L
s
.99625 .40515 m
1 .40515 L
s
.99625 .42289 m
1 .42289 L
s
.99625 .44064 m
1 .44064 L
s
.99625 .47613 m
1 .47613 L
s
.99625 .49388 m
1 .49388 L
s
.99625 .51163 m
1 .51163 L
s
.99625 .52937 m
1 .52937 L
s
.99625 .56487 m
1 .56487 L
s
.99625 .58261 m
1 .58261 L
s
.99625 .60036 m
1 .60036 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06264 .10324 L
.1007 .18656 L
.13877 .26349 L
.17683 .33194 L
.21489 .39061 L
.25296 .43889 L
.29102 .47667 L
.32909 .50417 L
.36715 .52175 L
.40522 .52985 L
.44328 .52896 L
.48135 .51951 L
.51941 .50192 L
.55748 .47656 L
.59554 .44373 L
.63361 .40369 L
.67167 .35664 L
.70974 .30272 L
.7478 .24201 L
.78587 .1745 L
.82393 .10014 L
.8638 .01472 L
s
[ .002 .01 ] 0 setdash
.02457 .01472 m
.1007 .20123 L
.13877 .28507 L
.17683 .36013 L
.21489 .42517 L
.25296 .4797 L
.29102 .52373 L
.32909 .55759 L
.36715 .58177 L
.40522 .59682 L
.44328 .60332 L
.48135 .60182 L
.51941 .59283 L
.55748 .57682 L
.59554 .55423 L
.63361 .52543 L
.67167 .49077 L
.70974 .45056 L
.7478 .40505 L
.78587 .35449 L
.82393 .29908 L
.862 .239 L
.90006 .17439 L
.93813 .10539 L
.97619 .03209 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`009_oo0P007_oo0P001?oo00<00?ooool0
0_oo10006?oo0P001?oo00<00?ooool01?oo00<00?ooool05ooo0P001?oo00<00?ooool00ooo0P00
6?oo0P001?oo00<00?ooool00ooo0P007_oo10007?oo10000ooo00<00?ooool00_oo10002_oo000U
ool01000ooooo`007?oo00@00?ooool000Soo`03003ooooo01Soo`04003ooooo000:ool00`00oooo
o`0Fool01000ooooo`002?oo00@00?ooool001Koo`04003ooooo0008ool01000ooooo`007ooo00<0
0?ooool07Ooo00<00?ooool01ooo00<00?ooool02ooo000Uool01000ooooo`007?oo00@00?ooool0
00Woo`03003ooooo01Ooo`04003ooooo0007ool5000Gool01000ooooo`002?oo00@00?ooool001Ko
o`04003ooooo0008ool01000ooooo`007ooo00<00?ooool07Ooo00<00?ooool02?oo00<00?ooool0
2_oo000Uool01000ooooo`007?oo00@00?ooool000[oo`03003ooooo01Koo`04003ooooo0007ool0
1000ooooo`006?oo00@00?ooool000Soo`<001Ooo`04003ooooo0009ool2000Pool00`00ooooo`0M
ool00`00ooooo`09ool00`00ooooo`09ool002Goo`04003ooooo000Lool01000ooooo`002ooo00<0
0?ooool05Ooo00@00?ooool000Soo`03003oo`0001Soo`04003ooooo0008ool00`00ooooo`0Gool0
1000ooooo`002?oo00@00?ooool001ooo`03003ooooo01goo`03003ooooo00[oo`03003ooooo00So
o`009Ooo00@00?ooool001coo`04003ooooo0008ool01000ooooo`005ooo00@00?ooool000Woo`80
01Soo`04003ooooo0008ool00`00ooooo`0Gool01000ooooo`002?oo00@00?ooool001goo`<001go
o`<000Woo`04003ooooo000:ool002Koo`8001koo`8000[oo`8001Woo`8000_oo`03003ooooo01Oo
o`8000[oo`<001Ooo`8000[oo`80023oo`03003ooooo01goo`03003ooooo00Soo`8000_oo`00oooo
8Ooo003ooolQool00?ooob7oo`008OoooP000Ooo000Jool20005ool00`00ooooo`03ool00`00oooo
o`06ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00oooo
o`07ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`06ool00`00oooo
o`07ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`07ool00`00oooo
o`06ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00oooo
o`07ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00oooo
o`03ool10001ool001Woo`04003ooooo0004ool00`00ooooo`3jool10001ool001Woo`04003ooooo
0004ool20004ool00`00ooooo`3Bool00`00ooooo`0Nool20001ool001Woo`04003ooooo0004ool0
0`00ooooo`03ool00`00ooooo`3Bool00`00ooooo`0Oool10001ool001Woo`04003ooooo0004ool0
0`00ooooo`04ool00`00ooooo`3@ool00`00ooooo`0Pool10001ool001Woo`04003ooooo0004ool0
0`00ooooo`04ool00`00ooooo`3@ool00`00ooooo`0Pool10001ool001[oo`8000Goo`03003ooooo
00Coo`800=3oo`03003ooooo027oo`40007oo`008Ooo0P001_oo00<00?ooool0c_oo00<00?ooool0
8?oo0P000Ooo000Qool00`00ooooo`05ool2003>ool00`00ooooo`0Lool00`00ooooo`03ool10001
ool0027oo`03003ooooo00Goo`800<koo`03003ooooo01_oo`03003ooooo00Coo`40007oo`008Ooo
00<00?ooool01_oo00<00?ooool0booo00<00?ooool08ooo0@000Ooo000Qool20007ool2003<ool0
0`00ooooo`0Kool00`00ooooo`04ool20001ool0027oo`03003ooooo00Ooo`03003ooooo0<Woo`03
003ooooo02Coo`40007oo`008Ooo00<00?ooool02?oo00<00?ooool0b?oo00<00?ooool09?oo0@00
0Ooo000Qool00`00ooooo`07ool2003:ool00`00ooooo`0Kool00`00ooooo`06ool10001ool0027o
o`03003ooooo00Woo`03003ooooo0<Koo`03003ooooo02Goo`40007oo`008Ooo0P002_oo00<00?oo
ool0a_oo00<00?ooool06_oo00<00?ooool01ooo0P000Ooo000Qool00`00ooooo`09ool20036ool0
0`00ooooo`0Kool00`00ooooo`08ool10001ool0027oo`03003ooooo00[oo`03003ooooo0<Coo`03
003ooooo02Koo`40007oo`008Ooo00<00?ooool02_oo00<00?ooool0`ooo00<00?ooool06ooo00<0
0?ooool02Ooo0@000Ooo000Qool2000;ool20034ool00`00ooooo`0Vool20001ool0027oo`03003o
oooo00_oo`03003ooooo0<7oo`03003ooooo02Soo`40007oo`008Ooo00<00?ooool02ooo0P00`_oo
00<00?ooool06_oo0P003?oo0@000Ooo0002ool20004ool00`00ooooo`03ool20003ool40002ool3
0005ool00`00ooooo`0;ool20031ool00`00ooooo`0Yool10001ool00005ool00?ooool00008ool0
1000ooooo`000_oo00<00?ooool01_oo00<00?ooool00_oo00<00?ooool03Ooo00<00?ooool0__oo
00<00?ooool06_oo00<00?ooool03?oo0@000Ooo00001Ooo003ooooo00002?oo00@00?ooool000?o
o`03003ooooo00Goo`03003ooooo00;oo`8000goo`800;ooo`03003ooooo01_oo`03003ooooo00_o
o`80007oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool01000oooooooo0`001Ooo00<00?oo
ool03?oo0P00_ooo00<00?ooool0:_oo0@000Ooo00001Ooo003ooooo00002?oo00@00?ooool000Go
o`04003ooooo0007ool00`00ooooo`0>ool00`00ooooo`2kool00`00ooooo`0Kool00`00ooooo`0=
ool10001ool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo
00Goo`03003ooooo00goo`800;goo`03003ooooo01[oo`03003ooooo00koo`40007oo`000_oo0P00
2_oo0P001?oo0P000ooo10001?oo0P004?oo00<00?ooool0^Ooo00<00?ooool0:ooo0P000Ooo000Q
ool00`00ooooo`0?ool00`00ooooo`2iool00`00ooooo`0Jool00`00ooooo`0?ool10001ool0027o
o`03003ooooo00koo`<00;Woo`03003ooooo02goo`40007oo`008Ooo00<00?ooool04?oo00<00?oo
ool0]ooo00<00?ooool06_oo00<00?ooool04?oo0@000Ooo000Qool00`00ooooo`0@ool00`00oooo
o`2fool00`00ooooo`0Kool00`00ooooo`0@ool10001ool0027oo`80017oo`800;Ooo`03003ooooo
02goo`80007oo`008Ooo00<00?ooool04Ooo00<00?ooool0]?oo00<00?ooool06_oo00<00?ooool0
4_oo0@000Ooo000Qool00`00ooooo`0Aool2002eool00`00ooooo`0Jool00`00ooooo`0Bool10001
ool0027oo`03003ooooo017oo`800;Coo`03003ooooo033oo`40007oo`008Ooo0P005?oo00<00?oo
ool0/Ooo00<00?ooool06_oo00<00?ooool04_oo0P000Ooo000Qool00`00ooooo`0Bool2002bool0
0`00ooooo`0aool10001ool0027oo`03003ooooo01;oo`800;;oo`03003ooooo01Woo`03003ooooo
01Goo`40007oo`008Ooo00<00?ooool05?oo00<00?ooool0[_oo00<00?ooool06_oo00<00?ooool0
5Ooo0@000Ooo000Qool00`00ooooo`0Cool2002`ool00`00ooooo`0bool10001ool0027oo`8001Ko
o`03003ooooo0:coo`03003ooooo01[oo`03003ooooo01Goo`80007oo`008Ooo00<00?ooool05Ooo
00<00?ooool0Zooo00<00?ooool06ooo00<00?ooool05_oo0@000Ooo0008ool20004ool00`00oooo
o`03ool20003ool30005ool00`00ooooo`0Dool00`00ool0002/ool00`00ooooo`0dool10001ool0
00Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool05_oo00<00?oo
ool0ZOoo00<00?ooool06_oo00<00?ooool06?oo0@000Ooo0007ool01000ooooo`002?oo00@00?oo
ool000Goo`03003ooooo00;oo`8001Ooo`03003ooooo0:Woo`03003ooooo03Coo`80007oo`001ooo
00@00?ooool000Soo`04003ooooo0002ool30005ool00`00ooooo`0Eool00`00ool0002Yool00`00
ooooo`0Jool00`00ooooo`0Iool10001ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00<0
0?ooool01Ooo00<00?ooool05ooo00<00?ooool0Y_oo00<00?ooool06ooo00<00?ooool06Ooo0@00
0Ooo0007ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo01Koo`03
003oo`000:Ooo`03003ooooo03Ooo`40007oo`002?oo0P002_oo0P000ooo10001?oo00<00?ooool0
5ooo0P00Y_oo00<00?ooool06_oo00<00?ooool06ooo0@000Ooo000Qool2000Jool00`00ooooo`2S
ool00`00ooooo`0gool20001ool0027oo`03003ooooo01Soo`800:Coo`03003ooooo03Woo`40007o
o`008Ooo00<00?ooool06?oo00<00?oo0000Xooo00<00?ooool06Ooo0P007_oo0@000Ooo000Qool0
0`00ooooo`0Jool00`00ooooo`2Pool00`00ooooo`0jool10001ool0027oo`8001[oo`03003oo`00
0:3oo`03003ooooo01[oo`03003ooooo01goo`80007oo`008Ooo00<00?ooool06ooo00<00?ooool0
W_oo00<00?ooool06_oo00<00?ooool07_oo0@000Ooo000Qool00`00ooooo`0Lool00`00ooooo`2L
ool00`00ooooo`0lool10001ool0027oo`03003ooooo01[oo`03003oo`0009koo`03003ooooo01Wo
o`80027oo`40007oo`008Ooo00<00?ooool07Ooo00<00?ooool0V_oo00<00?ooool0?Ooo0@000Ooo
000Qool2000Mool2002Kool00`00ooooo`0Jool00`00ooooo`0Pool20001ool0027oo`03003ooooo
01coo`03003oo`0009[oo`03003ooooo01[oo`03003ooooo027oo`40007oo`008Ooo00<00?ooool0
7_oo00<00?ooool0Uooo00<00?ooool0?ooo0@000Ooo000Qool00`00ooooo`0Mool00`00ool0002H
ool00`00ooooo`0Iool00`00ooooo`0Sool10001ool0027oo`8001koo`03003oo`0009Ooo`03003o
oooo03ooo`80007oo`008Ooo00<00?ooool08?oo00<00?ooool0U?oo00<00?ooool0@?oo0@000Ooo
000Qool00`00ooooo`0Nool01000ooooo`00U?oo00<00?ooool06Ooo0P009_oo0@000Ooo0002ool2
0004ool00`00ooooo`03ool20004ool00`00ooooo`02ool30005ool00`00ooooo`0Qool00`00oooo
o`2Bool00`00ooooo`11ool10001ool00005ool00?ooool00008ool01000ooooo`000ooo00<00?oo
ool01Ooo00<00?ooool00_oo00<00?ooool08_oo00<00?ooool0T?oo00<00?ooool06Ooo00<00?oo
ool09_oo0@000Ooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo00Coo`03003o
oooo00;oo`80023oo`800003ool00?oo093oo`03003ooooo01[oo`03003ooooo02Goo`80007oo`00
00Goo`00ooooo`0000Soo`04003ooooo0004ool01000oooooooo0`001Ooo00<00?ooool08ooo00<0
0?ooool0S_oo00<00?ooool0@ooo0@000Ooo00001Ooo003ooooo00002?oo00@00?ooool000Goo`04
003ooooo0007ool00`00ooooo`0Tool00`00ooooo`2<ool00`00ooooo`0Iool2000Yool10001ool0
0005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Goo`03003o
oooo027oo`04003ooooo002>ool00`00ooooo`14ool10001ool000;oo`8000[oo`8000?oo`@000;o
o`@000Coo`8002Koo`03003ooooo08[oo`03003ooooo01Woo`03003ooooo02Soo`80007oo`008Ooo
00<00?ooool08_oo00@00?ooool008coo`03003ooooo01Woo`03003ooooo02Woo`40007oo`008Ooo
00<00?ooool08_oo00D00?ooooooo`0008[oo`03003ooooo04Koo`40007oo`008Ooo00<00?ooool0
9_oo00<00?ooool0Qooo00<00?ooool06Ooo00<00?ooool0:ooo0@000Ooo000Qool00`00ooooo`0S
ool20002ool00`00ooooo`25ool00`00ooooo`18ool10001ool0027oo`8002Woo`03003ooooo08Co
o`03003ooooo01Soo`03003ooooo02coo`80007oo`008Ooo00<00?ooool0:?oo00<00?ooool0Pooo
00<00?ooool06Ooo00<00?ooool0;Ooo0@000Ooo000Qool00`00ooooo`0Uool01@00oooooooo0000
Pooo00<00?ooool0B_oo0@000Ooo000Qool00`00ooooo`0Yool00`00ooooo`20ool00`00ooooo`0I
ool2000`ool10001ool0027oo`8002Ooo`05003oooooool00021ool00`00ooooo`1:ool20001ool0
027oo`03003ooooo02Koo`03003ooooo00;oo`03003ooooo07goo`03003ooooo01Woo`03003ooooo
033oo`40007oo`008Ooo00<00?ooool0:ooo00<00?ooool0O?oo00<00?ooool06_oo00<00?ooool0
<?oo0@000Ooo000Qool00`00ooooo`0Wool00`00ooooo`02ool00`00ooooo`1jool00`00ooooo`1>
ool10001ool0027oo`03003ooooo02Soo`05003oooooool0001lool00`00ooooo`0Iool00`00oooo
o`0bool10001ool0027oo`8002koo`03003ooooo07Soo`03003ooooo04koo`80007oo`008Ooo00<0
0?ooool0:Ooo00<00?ooool00_oo00<00?ooool0M_oo00<00?ooool06_oo00<00?ooool0<ooo0@00
0Ooo000>ool20004ool00`00ooooo`02ool40004ool00`00ooooo`0^ool00`00ooooo`1eool00`00
ooooo`0Jool00`00ooooo`0dool10001ool000goo`04003ooooo000:ool00`00ooooo`03ool00`00
ooooo`0_ool00`00ooooo`1cool00`00ooooo`1Bool10001ool000goo`04003ooooo000:ool00`00
ooooo`03ool2000[ool20003ool00`00ooooo`1cool00`00ooooo`0Iool2000fool20001ool000go
o`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`0`ool00`00ooooo`1aool00`00ooooo`1C
ool10001ool000goo`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`0/ool00`00ooooo`02
ool00`00ooooo`1_ool00`00ooooo`0Jool00`00ooooo`0gool10001ool000goo`04003ooooo0008
ool30005ool00`00ooooo`0/ool00`00ooooo`02ool00`00ooooo`1^ool00`00ooooo`0Kool00`00
ooooo`0gool10001ool000koo`8000_oo`03003ooooo00?oo`03003ooooo03;oo`03003ooooo06co
o`03003ooooo05Koo`40007oo`008Ooo0P00;_oo0P000ooo00<00?ooool0Jooo00<00?ooool06ooo
00<00?ooool0>?oo0P000Ooo000Qool00`00ooooo`0cool00`00ooooo`1Zool00`00ooooo`1Gool1
0001ool0027oo`03003ooooo02ooo`03003ooooo00;oo`03003ooooo06Soo`03003ooooo01[oo`03
003ooooo03_oo`40007oo`008Ooo00<00?ooool0;ooo00<00?ooool00ooo00<00?ooool0I_oo00<0
0?ooool0FOoo0@000Ooo000Qool00`00ooooo`0eool00`00ooooo`1Uool00`00ooooo`1Jool10001
ool0027oo`80037oo`8000Coo`03003ooooo06?oo`03003ooooo01_oo`03003ooooo03coo`80007o
o`008Ooo00<00?ooool0=ooo00<00?ooool0HOoo00<00?ooool0G?oo0@000Ooo000Qool00`00oooo
o`0hool00`00ooooo`1Oool00`00ooooo`0Kool00`00ooooo`0oool10001ool0027oo`03003ooooo
03;oo`03003ooooo00Coo`03003ooooo05goo`03003ooooo05koo`40007oo`008Ooo0P00>ooo00<0
0?ooool0G?oo00<00?ooool06_oo00<00?ooool0@?oo0P000Ooo000Qool00`00ooooo`0dool00`00
ooooo`03ool00`00ooooo`1Kool00`00ooooo`0Kool00`00ooooo`11ool10001ool0027oo`03003o
oooo03Coo`03003ooooo00Coo`03003ooooo05Woo`03003ooooo063oo`40007oo`008Ooo00<00?oo
ool0??oo00<00?ooool0Eooo00<00?ooool06ooo0P00A?oo0@000Ooo000Qool00`00ooooo`0eool2
0006ool00`00ooooo`1Eool00`00ooooo`1Rool10001ool0027oo`8003ooo`03003ooooo05?oo`03
003ooooo01_oo`8004Goo`80007oo`008Ooo00<00?ooool0=ooo00<00?ooool01Ooo00<00?ooool0
DOoo00<00?ooool0I?oo0@000Ooo0002ool20004ool00`00ooooo`02ool40002ool40002ool30005
ool00`00ooooo`0gool00`00ooooo`06ool00`00ooooo`1?ool00`00ooooo`0Lool00`00ooooo`16
ool10001ool00005ool00?ooool0000:ool01@00oooooooo00002?oo00<00?ooool00_oo00<00?oo
ool0@Ooo00<00?ooool0COoo00<00?ooool07?oo00<00?ooool0Aooo0@000Ooo00001Ooo003ooooo
00002_oo00<00?ooool00_oo00<00?ooool01Ooo00<00?ooool00_oo0P00>_oo00<00?ooool01_oo
00<00?ooool0B_oo0P007_oo00<00?ooool0Aooo0P000Ooo00001Ooo003ooooo00002_oo00<00?oo
ool00ooo00@00?ooooooo`<000Goo`03003ooooo04?oo`03003ooooo04Soo`03003ooooo01goo`03
003ooooo04Woo`40007oo`0000Goo`00ooooo`0000[oo`03003ooooo00Coo`04003ooooo0007ool0
0`00ooooo`0kool00`00ooooo`06ool00`00ooooo`16ool00`00ooooo`1Zool10001ool00005ool0
0?ooool00008ool30003ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0>ooo00<00?oo
ool01ooo00<00?ooool0A?oo00<00?ooool07Ooo00<00?ooool0Booo0@000Ooo0002ool2000;ool0
0`00ooooo`02ool20003ool40004ool00`00ooooo`16ool00`00ooooo`12ool00`00ooooo`1/ool1
0001ool0027oo`8003goo`8000Woo`03003ooooo043oo`03003ooooo01goo`03003ooooo04coo`80
007oo`008Ooo00<00?ooool0B?oo00<00?ooool0?Ooo0P00L?oo0@000Ooo000Qool00`00ooooo`0n
ool20009ool2000kool2000Pool00`00ooooo`1?ool10001ool0027oo`03003ooooo04_oo`03003o
oooo03Ooo`03003ooooo07;oo`40007oo`008Ooo0P00@Ooo00<00?ooool02Ooo00<00?ooool0=?oo
0P008Ooo00<00?ooool0D?oo0P000Ooo000Qool00`00ooooo`11ool00`00ooooo`09ool2000bool2
001gool10001ool0027oo`03003ooooo04ooo`03003ooooo02koo`03003ooooo027oo`03003ooooo
05?oo`40007oo`008Ooo00<00?ooool0@_oo0P003?oo0P00;?oo0P008ooo00<00?ooool0E?oo0@00
0Ooo000Qool00`00ooooo`1Bool2000Xool2000Tool00`00ooooo`1Eool10001ool0027oo`8004Go
o`8000koo`<002?oo`8002Goo`03003ooooo05Goo`80007oo`008Ooo00<00?ooool0Eooo0P007ooo
0P009Ooo00<00?ooool0F?oo0@000Ooo000Qool00`00ooooo`16ool00`00ooooo`0@ool4000Hool3
000Wool00`00ooooo`1Hool10001ool0027oo`03003ooooo04Ooo`03003ooooo01?oo`@000ooo`D0
02Soo`03003ooooo05[oo`40007oo`008Ooo0P00BOoo00<00?ooool05_oo3`00;Ooo00<00?ooool0
FOoo0P000Ooo000Qool00`00ooooo`19ool00`00ooooo`1>ool2001Nool10001ool0027oo`03003o
oooo04_oo`03003ooooo0:coo`40007oo`002?oo0P001?oo00<00?ooool00_oo10000_oo0`001Ooo
00<00?ooool0Booo00<00?ooool0B_oo00<00?ooool0Gooo0@000Ooo0007ool01000ooooo`002_oo
00<00?ooool01?oo00<00?ooool00_oo00<00?ooool0COoo00<00?ooool0Z_oo0@000Ooo0007ool0
1000ooooo`002_oo00<00?ooool01?oo00<00?ooool00_oo0P00C_oo00<00?ooool0AOoo0P00H_oo
0P000Ooo0007ool01000ooooo`002_oo00@00?ooooooo`<000Goo`03003ooooo04ooo`03003ooooo
0:Soo`40007oo`001ooo00@00?ooool000[oo`05003oooooool00007ool00`00ooooo`1@ool00`00
ooooo`10ool00`00ooooo`1Tool10001ool000Ooo`04003ooooo0008ool30003ool00`00ooooo`05
ool00`00ooooo`1Bool00`00ooooo`0kool2001Xool10001ool000Soo`8000_oo`04003oooooool4
0004ool2003jool20001ool0027oo`03003ooooo05Coo`8003Soo`03003ooooo06Woo`40007oo`00
8Ooo00<00?ooool0Eooo00<00?ooool0<Ooo00<00?ooool0K?oo0@000Ooo000Qool00`00ooooo`1G
ool00`00ooooo`0aool00`00ooooo`1/ool10001ool0027oo`03003ooooo05Woo`8002goo`80073o
o`40007oo`008Ooo0P00GOoo00<00?ooool09_oo0P00L_oo0P000Ooo000Qool00`00ooooo`1Mool0
0`00ool0000Sool00`00ooooo`1dool10001ool0027oo`03003ooooo063oo`03003ooooo01goo`03
003ooooo07Ooo`40007oo`008Ooo00<00?ooool0H_oo00@00?ooool001Ooo`03003ooooo07[oo`40
007oo`008Ooo0P00JOoo00@00?ooool000_oo`04003ooooo001nool20001ool0027oo`03003ooooo
06_oo`04003ooooo0002ool01000ooooo`00QOoo0@000Ooo000Qool00`00ooooo`3jool10001ool0
027oo`03003ooooo00?oo`03003ooooo02?oo`03003ooooo02Coo`03003ooooo02Coo`03003ooooo
02?oo`03003ooooo02Coo`03003ooooo02?oo`03003ooooo00goo`40007oo`008Ooo00<00?ooool0
0ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool0
1ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool0
1_oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool0
1ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool0
1ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool0
1ooo00<00?ooool00ooo0@000Ooo000Qooon0001ool00?ooob7oo`00oooo8Ooo003ooolQool00001
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.203027, -0.0220884, \
0.00517755, 0.00111052}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<ustablenew1.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("ustablenew1.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%330, %320]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.761296 0.0147151 3.54937 [
[.17607 .00222 -9 -9 ]
[.17607 .00222 9 0 ]
[.32833 .00222 -9 -9 ]
[.32833 .00222 9 0 ]
[.48059 .00222 -9 -9 ]
[.48059 .00222 9 0 ]
[.63285 .00222 -9 -9 ]
[.63285 .00222 9 0 ]
[.78511 .00222 -3 -9 ]
[.78511 .00222 3 0 ]
[.93736 .00222 -9 -9 ]
[.93736 .00222 9 0 ]
[.01131 .10345 -30 -4.5 ]
[.01131 .10345 0 4.5 ]
[.01131 .19218 -24 -4.5 ]
[.01131 .19218 0 4.5 ]
[.01131 .28092 -30 -4.5 ]
[.01131 .28092 0 4.5 ]
[.01131 .36965 -18 -4.5 ]
[.01131 .36965 0 4.5 ]
[.01131 .45839 -30 -4.5 ]
[.01131 .45839 0 4.5 ]
[.01131 .54712 -24 -4.5 ]
[.01131 .54712 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17607 .01472 m
.17607 .02097 L
s
[(0.2)] .17607 .00222 0 1 Mshowa
.32833 .01472 m
.32833 .02097 L
s
[(0.4)] .32833 .00222 0 1 Mshowa
.48059 .01472 m
.48059 .02097 L
s
[(0.6)] .48059 .00222 0 1 Mshowa
.63285 .01472 m
.63285 .02097 L
s
[(0.8)] .63285 .00222 0 1 Mshowa
.78511 .01472 m
.78511 .02097 L
s
[(1)] .78511 .00222 0 1 Mshowa
.93736 .01472 m
.93736 .02097 L
s
[(1.2)] .93736 .00222 0 1 Mshowa
.125 Mabswid
.06187 .01472 m
.06187 .01847 L
s
.09994 .01472 m
.09994 .01847 L
s
.138 .01472 m
.138 .01847 L
s
.21413 .01472 m
.21413 .01847 L
s
.2522 .01472 m
.2522 .01847 L
s
.29026 .01472 m
.29026 .01847 L
s
.36639 .01472 m
.36639 .01847 L
s
.40446 .01472 m
.40446 .01847 L
s
.44252 .01472 m
.44252 .01847 L
s
.51865 .01472 m
.51865 .01847 L
s
.55672 .01472 m
.55672 .01847 L
s
.59478 .01472 m
.59478 .01847 L
s
.67091 .01472 m
.67091 .01847 L
s
.70898 .01472 m
.70898 .01847 L
s
.74704 .01472 m
.74704 .01847 L
s
.82317 .01472 m
.82317 .01847 L
s
.86123 .01472 m
.86123 .01847 L
s
.8993 .01472 m
.8993 .01847 L
s
.97543 .01472 m
.97543 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .10345 m
.03006 .10345 L
s
[(0.025)] .01131 .10345 1 0 Mshowa
.02381 .19218 m
.03006 .19218 L
s
[(0.05)] .01131 .19218 1 0 Mshowa
.02381 .28092 m
.03006 .28092 L
s
[(0.075)] .01131 .28092 1 0 Mshowa
.02381 .36965 m
.03006 .36965 L
s
[(0.1)] .01131 .36965 1 0 Mshowa
.02381 .45839 m
.03006 .45839 L
s
[(0.125)] .01131 .45839 1 0 Mshowa
.02381 .54712 m
.03006 .54712 L
s
[(0.15)] .01131 .54712 1 0 Mshowa
.125 Mabswid
.02381 .03246 m
.02756 .03246 L
s
.02381 .05021 m
.02756 .05021 L
s
.02381 .06796 m
.02756 .06796 L
s
.02381 .0857 m
.02756 .0857 L
s
.02381 .1212 m
.02756 .1212 L
s
.02381 .13894 m
.02756 .13894 L
s
.02381 .15669 m
.02756 .15669 L
s
.02381 .17444 m
.02756 .17444 L
s
.02381 .20993 m
.02756 .20993 L
s
.02381 .22768 m
.02756 .22768 L
s
.02381 .24542 m
.02756 .24542 L
s
.02381 .26317 m
.02756 .26317 L
s
.02381 .29866 m
.02756 .29866 L
s
.02381 .31641 m
.02756 .31641 L
s
.02381 .33416 m
.02756 .33416 L
s
.02381 .35191 m
.02756 .35191 L
s
.02381 .3874 m
.02756 .3874 L
s
.02381 .40515 m
.02756 .40515 L
s
.02381 .42289 m
.02756 .42289 L
s
.02381 .44064 m
.02756 .44064 L
s
.02381 .47613 m
.02756 .47613 L
s
.02381 .49388 m
.02756 .49388 L
s
.02381 .51163 m
.02756 .51163 L
s
.02381 .52937 m
.02756 .52937 L
s
.02381 .56487 m
.02756 .56487 L
s
.02381 .58261 m
.02756 .58261 L
s
.02381 .60036 m
.02756 .60036 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .002 .01 ] 0 setdash
.02457 .01472 m
.1007 .20123 L
.13877 .28507 L
.17683 .36013 L
.21489 .42517 L
.25296 .4797 L
.29102 .52373 L
.32909 .55759 L
.36715 .58177 L
.40522 .59682 L
.44328 .60332 L
.48135 .60182 L
.51941 .59283 L
.55748 .57682 L
.59554 .55423 L
.63361 .52543 L
.67167 .49077 L
.70974 .45056 L
.7478 .40505 L
.78587 .35449 L
.82393 .29908 L
.862 .239 L
.90006 .17439 L
.93813 .10539 L
.97619 .03209 L
s
[ ] 0 setdash
.02381 .01472 m
.06264 .10324 L
.1007 .18656 L
.13877 .26349 L
.17683 .33194 L
.21489 .39061 L
.25296 .43889 L
.29102 .47667 L
.32909 .50417 L
.36715 .52175 L
.40522 .52985 L
.44328 .52896 L
.48135 .51951 L
.51941 .50192 L
.55748 .47656 L
.59554 .44373 L
.63361 .40369 L
.67167 .35664 L
.70974 .30272 L
.7478 .24201 L
.78587 .1745 L
.82393 .10014 L
.8638 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool0047oo`8000Coo`03003ooooo00;oo`@001Woo`8000Co
o`03003ooooo00Coo`03003ooooo01Ooo`8000Coo`03003ooooo00?oo`8001[oo`8000Coo`03003o
oooo00?oo`8001ooo`@001goo`@000?oo`03003ooooo00;oo`@000[oo`00@?oo00@00?ooool000So
o`03003ooooo01Woo`04003ooooo000:ool00`00ooooo`0Fool01000ooooo`002?oo00@00?ooool0
01Soo`04003ooooo0008ool01000ooooo`008?oo00<00?ooool07_oo00<00?ooool01ooo00<00?oo
ool02ooo0010ool01000ooooo`002Ooo00<00?ooool06?oo00@00?ooool000Ooo`D001Ooo`04003o
oooo0008ool01000ooooo`006?oo00@00?ooool000Soo`04003ooooo000Pool00`00ooooo`0Nool0
0`00ooooo`08ool00`00ooooo`0:ool0043oo`04003ooooo000:ool00`00ooooo`0Gool01000oooo
o`001ooo00@00?ooool001Soo`04003ooooo0008ool3000Iool01000ooooo`002Ooo0P008Ooo00<0
0?ooool07_oo00<00?ooool02Ooo00<00?ooool02Ooo0010ool01000ooooo`002ooo00<00?ooool0
5_oo00@00?ooool000Soo`03003oo`0001Soo`04003ooooo0008ool00`00ooooo`0Iool01000oooo
o`002?oo00@00?ooool0023oo`03003ooooo01koo`03003ooooo00[oo`03003ooooo00Soo`00@?oo
00@00?ooool000Soo`04003ooooo000Hool01000ooooo`002Ooo0P006?oo00@00?ooool000Soo`03
003ooooo01Woo`04003ooooo0008ool01000ooooo`007_oo0`007_oo0`002Ooo00@00?ooool000[o
o`00@Ooo0P002_oo0P006_oo0P002ooo00<00?ooool05ooo0P002_oo0`006Ooo0P002_oo0P008Ooo
00<00?ooool07_oo00<00?ooool02?oo0P002ooo000Qool00`00ooooo`3lool0027oo`03003ooooo
0?coo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool001_oool000H000008Ooo00<00?oo
ool01ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool01ooo00<00?ooool01_oo0P002?oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01Ooo
000Qool2003Gool00`00ooooo`0Sool0027oo`800=Ooo`03003ooooo02?oo`008Ooo0`00eOoo00<0
0?ooool09?oo000Qool00`00ool0003Eool00`00ooooo`0Tool0027oo`04003oo`00003Cool00`00
ooooo`0Lool20007ool0027oo`04003ooooo003Cool00`00ooooo`0Uool0027oo`04003ooooo003B
ool00`00ooooo`0Lool00`00ooooo`07ool0027oo`8000;oo`03003ooooo0<ooo`03003ooooo01co
o`03003ooooo00Ooo`008Ooo00D00?ooooooo`000=3oo`03003ooooo02Ooo`008Ooo00<00?ooool0
0_oo00<00?ooool0cOoo00<00?ooool07?oo00<00?ooool02?oo000Qool00`00ooooo`02ool00`00
ooooo`3=ool00`00ooooo`0Kool00`00ooooo`09ool0027oo`03003ooooo00?oo`03003ooooo0<_o
o`03003ooooo02Soo`008Ooo0P001?oo00<00?ooool0booo00<00?ooool06ooo00<00?ooool02_oo
000Qool00`00ooooo`03ool2003;ool00`00ooooo`0Yool0027oo`03003ooooo00Coo`03003ooooo
0<Woo`03003ooooo02Woo`008Ooo00<00?ooool01?oo00<00?ooool0b?oo00<00?ooool07?oo00<0
0?ooool02ooo000Qool20005ool20039ool00`00ooooo`0Zool0027oo`03003ooooo00Goo`03003o
oooo0<Koo`03003ooooo02_oo`008Ooo00<00?ooool01Ooo0P00aooo00<00?ooool06ooo00<00?oo
ool03Ooo0002ool20004ool00`00ooooo`03ool20003ool40002ool30005ool00`00ooooo`06ool0
0`00ooooo`34ool00`00ooooo`0/ool00005ool00?ooool00008ool01000ooooo`000_oo00<00?oo
ool01_oo00<00?ooool00_oo00<00?ooool01ooo00<00?ooool0`ooo00<00?ooool06ooo00<00?oo
ool03_oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo00Goo`03003ooooo00;o
o`8000Ooo`800<Coo`03003ooooo01coo`03003ooooo00koo`0000Goo`00ooooo`0000Soo`04003o
oooo0004ool01000oooooooo0`001Ooo00<00?ooool01ooo00<00?ooool0`_oo00<00?ooool0;Ooo
00001Ooo003ooooo00002?oo00@00?ooool000Goo`04003ooooo0007ool00`00ooooo`08ool00`00
ooooo`30ool00`00ooooo`0Kool00`00ooooo`0@ool00005ool00?ooool00008ool01000ooooo`00
0_oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo00Soo`03003ooooo0<3oo`03003ooooo
02koo`000_oo0P002_oo0P001?oo0P000ooo10001?oo0P002_oo00<00?ooool0__oo00<00?ooool0
;ooo000Qool00`00ooooo`09ool00`00ooooo`2nool00`00ooooo`0Kool00`00ooooo`0Aool0027o
o`03003ooooo00Woo`800;koo`03003ooooo033oo`008Ooo00<00?ooool02_oo00<00?ooool0_?oo
00<00?ooool06_oo00<00?ooool04ooo000Qool00`00ooooo`0;ool00`00ooooo`2jool00`00oooo
o`0Kool00`00ooooo`0Cool0027oo`8000_oo`800;coo`03003ooooo037oo`008Ooo00<00?ooool0
3?oo00<00?ooool0^?oo00<00?ooool06ooo00<00?ooool05?oo000Qool00`00ooooo`0;ool2002j
ool00`00ooooo`0Kool00`00ooooo`0Dool0027oo`03003ooooo00_oo`03003oo`000;Soo`03003o
oooo03?oo`008Ooo00<00?ooool03Ooo00<00?ooool0]_oo00<00?ooool06_oo00<00?ooool05_oo
000Qool2000=ool00`00ool0002fool00`00ooooo`0dool0027oo`03003ooooo00coo`03003oo`00
0;Koo`03003ooooo03Coo`008Ooo00<00?ooool03ooo00<00?ooool0/_oo00<00?ooool06ooo00<0
0?ooool05ooo000Qool00`00ooooo`0=ool00`00ool0002dool00`00ooooo`0eool0027oo`8000oo
o`03003oo`000;;oo`03003ooooo01[oo`03003ooooo01Woo`008Ooo00<00?ooool04?oo00<00?oo
ool0[ooo00<00?ooool06ooo00<00?ooool06Ooo000Qool00`00ooooo`0?ool00`00ool0002`ool0
0`00ooooo`0gool000Soo`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo00ooo`03
003oo`000:ooo`03003ooooo01_oo`03003ooooo01[oo`001ooo00@00?ooool000Soo`04003ooooo
0005ool00`00ooooo`02ool00`00ooooo`0Aool00`00ooooo`2]ool00`00ooooo`0Kool00`00oooo
o`0Jool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool00_oo0P004Ooo00<00?oo
0000[Ooo00<00?ooool0>Ooo0007ool01000ooooo`002?oo00@00?ooool000;oo`<000Goo`03003o
oooo01;oo`03003ooooo0:_oo`03003ooooo01[oo`03003ooooo01coo`001ooo00@00?ooool000So
o`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`0Cool00`00ooooo`2Yool00`00ooooo`0j
ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool04Ooo00<0
0?oo0000Zooo00<00?ooool06_oo00<00?ooool07Ooo0008ool2000:ool20003ool40004ool2000E
ool00`00ooooo`2Wool00`00ooooo`0Jool00`00ooooo`0Nool0027oo`03003ooooo01Coo`03003o
oooo0:Koo`03003ooooo03coo`008Ooo00<00?ooool04_oo00@00?ooool00:Ooo`03003ooooo01[o
o`03003ooooo01ooo`008Ooo00<00?ooool05Ooo00<00?ooool0Y?oo00<00?ooool06ooo00<00?oo
ool07ooo000Qool00`00ooooo`0Fool00`00ooooo`2Sool00`00ooooo`0mool0027oo`8001Goo`03
003oo`000:Coo`03003ooooo01[oo`8002;oo`008Ooo00<00?ooool05ooo00<00?ooool0XOoo00<0
0?ooool0?_oo000Qool00`00ooooo`0Eool00`00ool0002Rool00`00ooooo`0oool0027oo`03003o
oooo01Goo`04003ooooo002Qool00`00ooooo`0Jool00`00ooooo`0Rool0027oo`03003ooooo01So
o`03003ooooo09koo`03003ooooo043oo`008Ooo0P005ooo00@00?ooool009koo`03003ooooo01[o
o`03003ooooo02Coo`008Ooo00<00?ooool05_oo00@00?ooool009koo`03003ooooo01[oo`03003o
oooo02Coo`008Ooo00<00?ooool06_oo00<00?ooool0V_oo00<00?ooool0@_oo000Qool00`00oooo
o`0Gool01000ooooo`00W?oo00<00?ooool06Ooo0P009ooo000Qool2000Iool01000ooooo`00V_oo
00<00?ooool0@ooo000Qool00`00ooooo`0Lool00`00ooooo`2Fool00`00ooooo`0Jool00`00oooo
o`0Wool0027oo`03003ooooo01Woo`04003ooooo002Hool00`00ooooo`0Jool00`00ooooo`0Wool0
00;oo`8000Coo`03003ooooo00?oo`8000Coo`03003ooooo00;oo`<000Goo`03003ooooo01goo`03
003ooooo09Coo`03003ooooo04Goo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool00`00oooo
o`05ool00`00ooooo`02ool00`00ooooo`0Mool00`00ooooo`2Cool00`00ooooo`0Jool00`00oooo
o`0Yool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool01?oo00<00?ooool00_oo
0P006ooo00D00?ooooooo`0009Coo`03003ooooo01[oo`03003ooooo02Woo`0000Goo`00ooooo`00
00Soo`04003ooooo0004ool01000oooooooo0`001Ooo00<00?ooool07_oo00<00?ooool0TOoo00<0
0?ooool0Aooo00001Ooo003ooooo00002?oo00@00?ooool000Goo`04003ooooo0007ool00`00oooo
o`0Oool00`00ooooo`2@ool00`00ooooo`0Iool2000/ool00005ool00?ooool00008ool01000oooo
o`000_oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo01_oo`8000;oo`03003ooooo08oo
o`03003ooooo04Soo`000_oo0P002_oo0P000ooo10000_oo10001?oo00<00?ooool08?oo00<00?oo
ool0SOoo00<00?ooool06_oo00<00?ooool0;?oo000Qool2000Nool01@00oooooooo0000S_oo00<0
0?ooool06_oo00<00?ooool0;?oo000Qool00`00ooooo`0Mool01@00oooooooo0000SOoo00<00?oo
ool0B_oo000Qool00`00ooooo`0Rool00`00ooooo`29ool00`00ooooo`0Jool00`00ooooo`0^ool0
027oo`03003ooooo01koo`05003oooooool0002;ool00`00ooooo`1;ool0027oo`8001ooo`03003o
oooo00;oo`03003ooooo08Ooo`03003ooooo04coo`008Ooo00<00?ooool08ooo00<00?ooool0Q_oo
00<00?ooool06_oo00<00?ooool0<?oo000Qool00`00ooooo`0Pool01@00oooooooo0000Q_oo00<0
0?ooool0C_oo000Qool00`00ooooo`0Tool00`00ooooo`24ool00`00ooooo`0Jool00`00ooooo`0a
ool0027oo`03003ooooo02Goo`03003ooooo08;oo`03003ooooo01[oo`03003ooooo03;oo`008Ooo
0P008_oo00<00?ooool00_oo00<00?ooool0P?oo00<00?ooool0D?oo000Qool00`00ooooo`0Vool0
0`00ooooo`20ool00`00ooooo`0Jool00`00ooooo`0cool0027oo`03003ooooo02;oo`03003ooooo
00;oo`03003ooooo07koo`03003ooooo057oo`008Ooo00<00?ooool08ooo00D00?ooooooo`0007oo
o`03003ooooo01[oo`03003ooooo03Goo`008Ooo0P00:Ooo00<00?ooool0Nooo00<00?ooool06ooo
00<00?ooool0=Ooo000Qool00`00ooooo`0Tool00`00ooooo`02ool00`00ooooo`1jool00`00oooo
o`1Cool0027oo`03003ooooo02Coo`03003ooooo00;oo`03003ooooo07Woo`03003ooooo01[oo`03
003ooooo03Ooo`003_oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool0:_oo00<00?ooool0
Mooo00<00?ooool0EOoo000=ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool09Ooo0P00
0ooo00<00?ooool0M_oo00<00?ooool06_oo00<00?ooool0>Ooo000=ool01000ooooo`002_oo00<0
0?ooool00ooo0P00;?oo00<00?ooool0MOoo00<00?ooool06_oo00<00?ooool0>Ooo000=ool01000
ooooo`002_oo00<00?ooool00ooo00<00?ooool0;?oo00<00?ooool0Looo00<00?ooool0Eooo000=
ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool09ooo00<00?ooool00_oo00<00?ooool0
L_oo00<00?ooool06_oo0P00??oo000=ool01000ooooo`002?oo0`001Ooo00<00?ooool0;Ooo00<0
0?ooool0L?oo00<00?ooool0FOoo000>ool2000;ool00`00ooooo`03ool00`00ooooo`0Xool00`00
ooooo`02ool00`00ooooo`1_ool00`00ooooo`0Kool00`00ooooo`0lool0027oo`8002[oo`03003o
oooo00;oo`03003ooooo06goo`03003ooooo01_oo`03003ooooo03goo`008Ooo00<00?ooool0;ooo
00<00?ooool0K?oo00<00?ooool0Fooo000Qool00`00ooooo`0Zool00`00ooooo`03ool00`00oooo
o`1Zool00`00ooooo`0Jool20010ool0027oo`03003ooooo033oo`03003ooooo06Woo`03003ooooo
05goo`008Ooo0P00<_oo00<00?ooool0Iooo00<00?ooool06ooo00<00?ooool0@?oo000Qool00`00
ooooo`0/ool00`00ooooo`03ool00`00ooooo`1Uool00`00ooooo`0Lool00`00ooooo`10ool0027o
o`03003ooooo03?oo`03003ooooo06?oo`03003ooooo063oo`008Ooo00<00?ooool0;Ooo00<00?oo
ool00ooo00<00?ooool0H_oo00<00?ooool07?oo00<00?ooool0@_oo000Qool00`00ooooo`0]ool0
0`00ooooo`04ool00`00ooooo`1Pool00`00ooooo`1Rool0027oo`8003Koo`03003ooooo05koo`03
003ooooo01coo`03003ooooo04Coo`008Ooo00<00?ooool0;ooo00<00?ooool01?oo00<00?ooool0
G?oo00<00?ooool0I?oo000Qool00`00ooooo`0_ool00`00ooooo`04ool00`00ooooo`1Kool00`00
ooooo`0Lool00`00ooooo`16ool0027oo`03003ooooo03Ooo`03003ooooo05Woo`03003ooooo01go
o`03003ooooo04Koo`008Ooo0P00<_oo00<00?ooool01?oo00<00?ooool0Eooo00<00?ooool0Iooo
000Qool00`00ooooo`0iool00`00ooooo`1Eool00`00ooooo`0Mool00`00ooooo`18ool0027oo`03
003ooooo03;oo`03003ooooo00Goo`03003ooooo05?oo`03003ooooo06Woo`000_oo0P001?oo00<0
0?ooool00_oo10000_oo10000_oo0`001Ooo00<00?ooool0<ooo00<00?ooool01Ooo00<00?ooool0
DOoo00<00?ooool07Ooo00<00?ooool0B_oo00001Ooo003ooooo00002_oo00D00?ooooooo`0000So
o`03003ooooo00;oo`03003ooooo03coo`03003ooooo04ooo`03003ooooo01koo`03003ooooo04[o
o`0000Goo`00ooooo`0000[oo`03003ooooo00;oo`03003ooooo00Goo`03003ooooo00;oo`8003Go
o`03003ooooo00Koo`03003ooooo04coo`8001ooo`03003ooooo04coo`0000Goo`00ooooo`0000[o
o`03003ooooo00?oo`04003oooooool30005ool00`00ooooo`0nool00`00ooooo`1:ool00`00oooo
o`0Nool00`00ooooo`1=ool00005ool00?ooool0000:ool00`00ooooo`04ool01000ooooo`001ooo
00<00?ooool0=_oo00<00?ooool01_oo00<00?ooool0B?oo00<00?ooool0Kooo00001Ooo003ooooo
00002?oo0`000ooo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo03Koo`03003ooooo00Oo
o`03003ooooo04Koo`03003ooooo01koo`80053oo`000_oo0P002ooo00<00?ooool00_oo0P000ooo
10001?oo00<00?ooool0@Ooo00<00?ooool0A?oo00<00?ooool0LOoo000Qool2000iool00`00oooo
o`07ool00`00ooooo`12ool00`00ooooo`0Nool00`00ooooo`1Aool0027oo`03003ooooo04?oo`80
043oo`8007Goo`008Ooo00<00?ooool0>_oo00<00?ooool02?oo00<00?ooool0??oo00<00?ooool0
7ooo00<00?ooool0Dooo000Qool00`00ooooo`0jool00`00ooooo`09ool00`00ooooo`0iool2000R
ool00`00ooooo`1Cool0027oo`8004Soo`8003Soo`03003ooooo023oo`03003ooooo05Goo`008Ooo
00<00?ooool0??oo00<00?ooool02_oo00<00?ooool0<ooo0P008ooo00<00?ooool0EOoo000Qool0
0`00ooooo`1:ool2000aool2001mool0027oo`03003ooooo03koo`03003ooooo00_oo`03003ooooo
02coo`8002Coo`8005Woo`008Ooo00<00?ooool0COoo0P00:_oo0P00POoo000Qool20011ool00`00
ooooo`0<ool2000Vool2000Vool00`00ooooo`1Jool0027oo`03003ooooo043oo`03003ooooo00ko
o`8002;oo`8008Goo`008Ooo00<00?ooool0Dooo0P007Ooo0`00:?oo00<00?ooool0G?oo000Qool0
0`00ooooo`12ool00`00ooooo`0@ool4000Dool5002:ool0027oo`03003ooooo05Woo`D000Ooo`P0
02goo`80063oo`008Ooo0P00AOoo0P006?oo1`00Uooo000Qool00`00ooooo`2Hool00`00ooooo`1Q
ool000Soo`8000Coo`03003ooooo00;oo`@000;oo`<000Goo`03003ooooo04Ooo`03003ooooo0;;o
o`001ooo00@00?ooool000[oo`03003ooooo00Coo`03003ooooo00;oo`03003ooooo09Goo`8006Go
o`001ooo00@00?ooool000[oo`03003ooooo00Coo`03003ooooo00;oo`8004[oo`03003ooooo04Oo
o`03003ooooo06Koo`001ooo00@00?ooool000[oo`04003oooooool30005ool00`00ooooo`2Cool0
0`00ooooo`1Vool000Ooo`04003ooooo000:ool01@00oooooooo00001ooo00<00?ooool0Booo0P00
A?oo00<00?ooool0J?oo0007ool01000ooooo`002?oo0`000ooo00<00?ooool01Ooo00<00?ooool0
T?oo00<00?ooool0JOoo0008ool2000;ool01000oooooooo10001?oo00<00?ooool0C_oo00<00?oo
ool0?Ooo00<00?ooool0Jooo000Qool2002<ool00`00ooooo`1^ool0027oo`03003ooooo053oo`80
03Woo`03003ooooo06koo`008Ooo00<00?ooool0Dooo00<00?ooool0<_oo0P00L_oo000Qool00`00
ooooo`1Cool00`00ooooo`0`ool00`00ooooo`1cool0027oo`8005Ooo`03003ooooo02goo`03003o
oooo07?oo`008Ooo00<00?ooool0F?oo0P00:Ooo00<00?ooool0M_oo000Qool00`00ooooo`1Kool2
000Sool00`00ooooo`1iool0027oo`03003ooooo05koo`03003ooooo01coo`03003ooooo07coo`00
8Ooo00<00?ooool0Gooo00@00?oo000001Ooo`03003ooooo07ooo`008Ooo0P00IOoo0P0000?oo`00
00002?oo00@00?ooool000;oo`03003ooooo08;oo`008Ooo00<00?ooool0J_oo0P000_oo00<00?oo
ool0Rooo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`008Ooo00<00?ooool0o?oo000Q
ool00`00ooooo`3lool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.168025, -0.0175379, \
0.00505333, 0.00108388}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<ustablenew1.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("ustablenew1.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[Pi/2, \(-9\)/10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.1305151037327643`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, omdamp2[k, Pi/2, \(-9\)/10]}, {k, 0.001, 1.1, 
        0.05}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.001`, 0.0007045916232833904`}, {0.051000000000000004`, 
        0.0352926090737061`}, {0.101`, 0.06656894062719095`}, {0.151`, 
        0.09269218599181668`}, {0.201`, 0.11324713791579334`}, {0.251`, 
        0.12858455354430834`}, {0.301`, 
        0.13930624045484807`}, {0.35100000000000003`, 
        0.14602935063652533`}, {0.401`, 0.14930747505110348`}, {0.451`, 
        0.14961702460280774`}, {0.501`, 0.1473635008152616`}, {0.551`, 
        0.14289250310436927`}, {0.601`, 0.1365004699511695`}, {0.651`, 
        0.12844275162052268`}, {0.7010000000000001`, 
        0.11894071020408435`}, {0.751`, 0.10818682664763356`}, {0.801`, 
        0.09634865559209468`}, {0.8510000000000001`, 
        0.08357232424112758`}, {0.901`, 
        0.06998523575766243`}, {0.9510000000000001`, 
        0.055698304448914424`}, {1.0010000000000001`, 
        0.04080822339998842`}, {1.0510000000000002`, 
        0.0253988342590307`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.001`, 0.0007045916232833904`}, {0.051000000000000004`, 
        0.0352926090737061`}, {0.101`, 0.06656894062719095`}, {0.151`, 
        0.09269218599181668`}, {0.201`, 0.11324713791579334`}, {0.251`, 
        0.12858455354430834`}, {0.301`, 
        0.13930624045484807`}, {0.35100000000000003`, 
        0.14602935063652533`}, {0.401`, 0.14930747505110348`}, {0.451`, 
        0.14961702460280774`}, {0.501`, 0.1473635008152616`}, {0.551`, 
        0.14289250310436927`}, {0.601`, 0.1365004699511695`}, {0.651`, 
        0.12844275162052268`}, {0.7010000000000001`, 
        0.11894071020408435`}, {0.751`, 0.10818682664763356`}, {0.801`, 
        0.09634865559209468`}, {0.8510000000000001`, 
        0.08357232424112758`}, {0.901`, 
        0.06998523575766243`}, {0.9510000000000001`, 
        0.055698304448914424`}, {1.0010000000000001`, 
        0.04080822339998842`}, {1.0510000000000002`, 
        0.0253988342590307`}, {1.1305151037327643`\[InvisibleSpace], 
        0}}\)], "Input"],

Cell[BoxData[
    \({{0.001`, 0.0007045916232833904`}, {0.051000000000000004`, 
        0.0352926090737061`}, {0.101`, 0.06656894062719095`}, {0.151`, 
        0.09269218599181668`}, {0.201`, 0.11324713791579334`}, {0.251`, 
        0.12858455354430834`}, {0.301`, 
        0.13930624045484807`}, {0.35100000000000003`, 
        0.14602935063652533`}, {0.401`, 0.14930747505110348`}, {0.451`, 
        0.14961702460280774`}, {0.501`, 0.1473635008152616`}, {0.551`, 
        0.14289250310436927`}, {0.601`, 0.1365004699511695`}, {0.651`, 
        0.12844275162052268`}, {0.7010000000000001`, 
        0.11894071020408435`}, {0.751`, 0.10818682664763356`}, {0.801`, 
        0.09634865559209468`}, {0.8510000000000001`, 
        0.08357232424112758`}, {0.901`, 
        0.06998523575766243`}, {0.9510000000000001`, 
        0.055698304448914424`}, {1.0010000000000001`, 
        0.04080822339998842`}, {1.0510000000000002`, 
        0.0253988342590307`}, {1.1305151037327643`, 0}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True, Frame \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.842431 0.0147151 3.93407 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.1923 -0.0125 -9 -9 ]
[.1923 -0.0125 9 0 ]
[.36078 -0.0125 -9 -9 ]
[.36078 -0.0125 9 0 ]
[.52927 -0.0125 -9 -9 ]
[.52927 -0.0125 9 0 ]
[.69775 -0.0125 -9 -9 ]
[.69775 -0.0125 9 0 ]
[.86624 -0.0125 -3 -9 ]
[.86624 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01472 -6 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .0934 -24 -4.5 ]
[-0.0125 .0934 0 4.5 ]
[-0.0125 .17208 -24 -4.5 ]
[-0.0125 .17208 0 4.5 ]
[-0.0125 .25076 -24 -4.5 ]
[-0.0125 .25076 0 4.5 ]
[-0.0125 .32944 -24 -4.5 ]
[-0.0125 .32944 0 4.5 ]
[-0.0125 .40812 -18 -4.5 ]
[-0.0125 .40812 0 4.5 ]
[-0.0125 .4868 -24 -4.5 ]
[-0.0125 .4868 0 4.5 ]
[-0.0125 .56548 -24 -4.5 ]
[-0.0125 .56548 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.1923 0 m
.1923 .00625 L
s
[(0.2)] .1923 -0.0125 0 1 Mshowa
.36078 0 m
.36078 .00625 L
s
[(0.4)] .36078 -0.0125 0 1 Mshowa
.52927 0 m
.52927 .00625 L
s
[(0.6)] .52927 -0.0125 0 1 Mshowa
.69775 0 m
.69775 .00625 L
s
[(0.8)] .69775 -0.0125 0 1 Mshowa
.86624 0 m
.86624 .00625 L
s
[(1)] .86624 -0.0125 0 1 Mshowa
.125 Mabswid
.06593 0 m
.06593 .00375 L
s
.10805 0 m
.10805 .00375 L
s
.15017 0 m
.15017 .00375 L
s
.23442 0 m
.23442 .00375 L
s
.27654 0 m
.27654 .00375 L
s
.31866 0 m
.31866 .00375 L
s
.4029 0 m
.4029 .00375 L
s
.44503 0 m
.44503 .00375 L
s
.48715 0 m
.48715 .00375 L
s
.57139 0 m
.57139 .00375 L
s
.61351 0 m
.61351 .00375 L
s
.65563 0 m
.65563 .00375 L
s
.73988 0 m
.73988 .00375 L
s
.782 0 m
.782 .00375 L
s
.82412 0 m
.82412 .00375 L
s
.90836 0 m
.90836 .00375 L
s
.95048 0 m
.95048 .00375 L
s
.99261 0 m
.99261 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .01472 m
.00625 .01472 L
s
[(0)] -0.0125 .01472 1 0 Mshowa
0 .0934 m
.00625 .0934 L
s
[(0.02)] -0.0125 .0934 1 0 Mshowa
0 .17208 m
.00625 .17208 L
s
[(0.04)] -0.0125 .17208 1 0 Mshowa
0 .25076 m
.00625 .25076 L
s
[(0.06)] -0.0125 .25076 1 0 Mshowa
0 .32944 m
.00625 .32944 L
s
[(0.08)] -0.0125 .32944 1 0 Mshowa
0 .40812 m
.00625 .40812 L
s
[(0.1)] -0.0125 .40812 1 0 Mshowa
0 .4868 m
.00625 .4868 L
s
[(0.12)] -0.0125 .4868 1 0 Mshowa
0 .56548 m
.00625 .56548 L
s
[(0.14)] -0.0125 .56548 1 0 Mshowa
.125 Mabswid
0 .03439 m
.00375 .03439 L
s
0 .05406 m
.00375 .05406 L
s
0 .07373 m
.00375 .07373 L
s
0 .11307 m
.00375 .11307 L
s
0 .13274 m
.00375 .13274 L
s
0 .15241 m
.00375 .15241 L
s
0 .19175 m
.00375 .19175 L
s
0 .21142 m
.00375 .21142 L
s
0 .23109 m
.00375 .23109 L
s
0 .27043 m
.00375 .27043 L
s
0 .2901 m
.00375 .2901 L
s
0 .30977 m
.00375 .30977 L
s
0 .34911 m
.00375 .34911 L
s
0 .36878 m
.00375 .36878 L
s
0 .38845 m
.00375 .38845 L
s
0 .42779 m
.00375 .42779 L
s
0 .44746 m
.00375 .44746 L
s
0 .46713 m
.00375 .46713 L
s
0 .50647 m
.00375 .50647 L
s
0 .52614 m
.00375 .52614 L
s
0 .54581 m
.00375 .54581 L
s
0 .58516 m
.00375 .58516 L
s
0 .60483 m
.00375 .60483 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.1923 .61178 m
.1923 .61803 L
s
.36078 .61178 m
.36078 .61803 L
s
.52927 .61178 m
.52927 .61803 L
s
.69775 .61178 m
.69775 .61803 L
s
.86624 .61178 m
.86624 .61803 L
s
.125 Mabswid
.06593 .61428 m
.06593 .61803 L
s
.10805 .61428 m
.10805 .61803 L
s
.15017 .61428 m
.15017 .61803 L
s
.23442 .61428 m
.23442 .61803 L
s
.27654 .61428 m
.27654 .61803 L
s
.31866 .61428 m
.31866 .61803 L
s
.4029 .61428 m
.4029 .61803 L
s
.44503 .61428 m
.44503 .61803 L
s
.48715 .61428 m
.48715 .61803 L
s
.57139 .61428 m
.57139 .61803 L
s
.61351 .61428 m
.61351 .61803 L
s
.65563 .61428 m
.65563 .61803 L
s
.73988 .61428 m
.73988 .61803 L
s
.782 .61428 m
.782 .61803 L
s
.82412 .61428 m
.82412 .61803 L
s
.90836 .61428 m
.90836 .61803 L
s
.95048 .61428 m
.95048 .61803 L
s
.99261 .61428 m
.99261 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .0934 m
1 .0934 L
s
.99375 .17208 m
1 .17208 L
s
.99375 .25076 m
1 .25076 L
s
.99375 .32944 m
1 .32944 L
s
.99375 .40812 m
1 .40812 L
s
.99375 .4868 m
1 .4868 L
s
.99375 .56548 m
1 .56548 L
s
.125 Mabswid
.99625 .03439 m
1 .03439 L
s
.99625 .05406 m
1 .05406 L
s
.99625 .07373 m
1 .07373 L
s
.99625 .11307 m
1 .11307 L
s
.99625 .13274 m
1 .13274 L
s
.99625 .15241 m
1 .15241 L
s
.99625 .19175 m
1 .19175 L
s
.99625 .21142 m
1 .21142 L
s
.99625 .23109 m
1 .23109 L
s
.99625 .27043 m
1 .27043 L
s
.99625 .2901 m
1 .2901 L
s
.99625 .30977 m
1 .30977 L
s
.99625 .34911 m
1 .34911 L
s
.99625 .36878 m
1 .36878 L
s
.99625 .38845 m
1 .38845 L
s
.99625 .42779 m
1 .42779 L
s
.99625 .44746 m
1 .44746 L
s
.99625 .46713 m
1 .46713 L
s
.99625 .50647 m
1 .50647 L
s
.99625 .52614 m
1 .52614 L
s
.99625 .54581 m
1 .54581 L
s
.99625 .58516 m
1 .58516 L
s
.99625 .60483 m
1 .60483 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02465 .01749 m
.06677 .15356 L
.1089 .2766 L
.15102 .37937 L
.19314 .46024 L
.23526 .52058 L
.27738 .56276 L
.3195 .5892 L
.36162 .6021 L
.40375 .60332 L
.44587 .59445 L
.48799 .57686 L
.53011 .55172 L
.57223 .52002 L
.61435 .48264 L
.65648 .44033 L
.6986 .39376 L
.74072 .34349 L
.78284 .29004 L
.82496 .23384 L
.86708 .17526 L
.9092 .11464 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Pool2000Tool20004ool00`00ooooo`02ool4000Lool20004ool0
0`00ooooo`04ool00`00ooooo`0Lool20004ool00`00ooooo`03ool2000Nool20004ool00`00oooo
o`03ool2000Sool4000Rool001ooo`04003ooooo000Rool01000ooooo`002?oo00<00?ooool07?oo
00@00?ooool000[oo`03003ooooo01_oo`04003ooooo0008ool01000ooooo`007?oo00@00?ooool0
00Soo`04003ooooo000Tool00`00ooooo`0Qool001ooo`04003ooooo000Rool01000ooooo`002Ooo
00<00?ooool06ooo00@00?ooool000Ooo`D001coo`04003ooooo0008ool01000ooooo`007?oo00@0
0?ooool000Soo`04003ooooo000Tool00`00ooooo`0Qool001ooo`04003ooooo000Rool01000oooo
o`002_oo00<00?ooool06_oo00@00?ooool000Ooo`04003ooooo000Mool01000ooooo`002?oo0`00
7Ooo00@00?ooool000Woo`8002Goo`03003ooooo027oo`007ooo00@00?ooool002;oo`04003ooooo
000;ool00`00ooooo`0Iool01000ooooo`002?oo00<00?oo00007Ooo00@00?ooool000Soo`03003o
oooo01goo`04003ooooo0008ool01000ooooo`009?oo00<00?ooool08Ooo000Oool01000ooooo`00
8_oo00@00?ooool000Soo`04003ooooo000Kool01000ooooo`002Ooo0P007Ooo00@00?ooool000So
o`03003ooooo01goo`04003ooooo0008ool01000ooooo`008_oo0`008ooo000Pool2000Tool2000:
ool2000Mool2000;ool00`00ooooo`0Lool2000:ool3000Mool2000:ool2000Uool00`00ooooo`0Q
ool00?ooob7oo`00oooo8Ooo003ooolQool001_oool000D0007oo`006ooo00<00?ooool00ooo00<0
0?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<0
0?ooool02?oo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<0
0?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<0
0?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<0
0?ooool02?oo00<00?ooool02?oo00<00?ooool01ooo00<00?oo00000Ooo000Dool20005ool00`00
ooooo`03ool00`00ooooo`0Yool00`00ooooo`0Xool00`00ooooo`0Yool00`00ooooo`0Yool00`00
ooooo`0Yool00`00ooooo`0Oool10001ool001?oo`04003ooooo0004ool00`00ooooo`3oool1ool1
0001ool001?oo`04003ooooo0004ool20004ool00`00ooooo`3dool00`00ooooo`02ool20001ool0
01?oo`04003ooooo0004ool00`00ooooo`03ool00`00ooooo`3cool00`00ooooo`04ool10001ool0
01?oo`04003ooooo0004ool00`00ooooo`04ool00`00ooooo`3bool00`00ooooo`04ool10001ool0
01?oo`04003ooooo0004ool00`00ooooo`04ool00`00ooooo`3aool00`00ooooo`05ool10001ool0
01Coo`8000Goo`03003ooooo00Coo`03003ooooo0?3oo`03003ooooo00Koo`40007oo`006ooo0P00
1_oo00<00?ooool0kooo00<00?ooool01Ooo0P000Ooo000Kool00`00ooooo`05ool00`00ooooo`3^
ool00`00ooooo`07ool10001ool001_oo`03003ooooo00Goo`03003ooooo0>goo`03003ooooo00So
o`40007oo`006ooo00<00?ooool01Ooo00<00?ooool0kOoo00<00?ooool02?oo0@000Ooo000Kool0
0`00ooooo`06ool00`00ooooo`3[ool00`00ooooo`09ool10001ool001_oo`8000Ooo`03003ooooo
0>[oo`03003ooooo00Woo`80007oo`006ooo00<00?ooool01_oo00<00?ooool0j_oo00<00?ooool0
2_oo0@000Ooo000Kool00`00ooooo`07ool00`00ooooo`3Xool00`00ooooo`0;ool10001ool001_o
o`03003ooooo00Ooo`03003ooooo0>Soo`03003ooooo00_oo`40007oo`006ooo00<00?ooool01ooo
00<00?ooool0iooo00<00?ooool03?oo0@000Ooo000Kool20009ool00`00ooooo`3Uool00`00oooo
o`0<ool20001ool001_oo`03003ooooo00Soo`03003ooooo0>Goo`03003ooooo00goo`40007oo`00
6ooo00<00?ooool02?oo00<00?ooool0i?oo00<00?ooool03_oo0@000Ooo0002ool20004ool00`00
ooooo`03ool20003ool40004ool00`00ooooo`08ool00`00ooooo`3Sool00`00ooooo`0?ool10001
ool00005ool00?ooool00008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool02Ooo00<0
0?ooool0h_oo00<00?ooool03ooo0@000Ooo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03
003ooooo00Coo`8000[oo`03003ooooo0>7oo`03003ooooo00ooo`80007oo`0000Goo`00ooooo`00
00Soo`04003ooooo0004ool00`00ooooo`03ool00`00ooooo`09ool00`00ooooo`3Pool00`00oooo
o`0Aool10001ool00005ool00?ooool00008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?oo
ool02_oo00<00?ooool0gooo00<00?ooool04Ooo0@000Ooo00001Ooo003ooooo00002?oo00@00?oo
ool000;oo`04003ooooo0004ool00`00ooooo`0:ool00`00ooooo`3Nool00`00ooooo`0Bool10001
ool000;oo`8000[oo`8000Coo`8000Goo`03003ooooo00[oo`03003ooooo0=goo`03003ooooo01?o
o`40007oo`006ooo0P003?oo00<00?ooool0g?oo00<00?ooool04_oo0P000Ooo000Kool00`00oooo
o`0;ool00`00ooooo`3Kool00`00ooooo`0Dool10001ool001_oo`03003ooooo00_oo`03003ooooo
0=[oo`03003ooooo01Goo`40007oo`006ooo00<00?ooool03?oo00<00?ooool0fOoo00<00?ooool0
5Ooo0@000Ooo000Kool00`00ooooo`0<ool00`00ooooo`3Hool00`00ooooo`0Fool10001ool001_o
o`8000goo`03003ooooo0=Ooo`03003ooooo01Koo`80007oo`006ooo00<00?ooool03?oo00<00?oo
ool0e_oo00<00?ooool06?oo0@000Ooo000Kool00`00ooooo`0=ool00`00ooooo`3Eool00`00oooo
o`0Hool10001ool001_oo`03003ooooo00goo`03003ooooo0=Coo`03003ooooo01Woo`40007oo`00
6ooo00<00?ooool03Ooo00<00?ooool0dooo00<00?ooool06_oo0@000Ooo000Kool2000?ool00`00
ooooo`3Aool00`00ooooo`0Jool20001ool001_oo`03003ooooo00koo`03003ooooo0=7oo`03003o
oooo01_oo`40007oo`006ooo00<00?ooool03_oo00<00?ooool0d?oo00<00?ooool07?oo0@000Ooo
000Kool00`00ooooo`0?ool00`00ooooo`3>ool00`00ooooo`0Mool10001ool000;oo`8000Coo`03
003ooooo00?oo`8000Goo`03003ooooo00?oo`03003ooooo00ooo`03003ooooo0<goo`03003ooooo
01koo`40007oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`03ool00`00oooo
o`0?ool00`00ooooo`3=ool00`00ooooo`0Nool10001ool00005ool00?ooool00008ool01@00oooo
o`00ool01@001?oo0P004Ooo00<00?ooool0booo00<00?ooool07_oo0P000Ooo00001Ooo003ooooo
00002?oo00H00?ooool00?oo0002ool00`00ooooo`03ool00`00ooooo`0@ool00`00ooooo`3:ool0
0`00ooooo`0Pool10001ool00005ool00?ooool00008ool01000ooooo`000_oo00<00?oo00001Ooo
00<00?ooool04?oo00<00?ooool0b_oo00<00?ooool08?oo0@000Ooo00001Ooo003ooooo00002?oo
00@00?ooool000?oo`8000Goo`03003ooooo017oo`03003ooooo0<Soo`03003ooooo027oo`40007o
o`000_oo0P002_oo0P001Ooo00<00?ooool00ooo00<00?ooool04Ooo00<00?ooool0aooo00<00?oo
ool08_oo0@000Ooo000Kool2000Bool00`00ooooo`37ool00`00ooooo`0Qool20001ool001_oo`03
003ooooo01;oo`03003ooooo0<Goo`03003ooooo02?oo`40007oo`006ooo00<00?ooool04_oo00<0
0?ooool0a?oo00<00?ooool09?oo0@000Ooo000Kool00`00ooooo`0Bool00`00ooooo`34ool00`00
ooooo`0Tool10001ool001_oo`03003ooooo01?oo`03003ooooo0<;oo`03003ooooo02Goo`40007o
o`006ooo0P005?oo00<00?ooool0`Ooo00<00?ooool09Ooo0P000Ooo000Kool00`00ooooo`0Cool0
0`00ooooo`30ool00`00ooooo`0Wool10001ool001_oo`03003ooooo01Coo`03003ooooo0;ooo`03
003ooooo02Ooo`40007oo`006ooo00<00?ooool05?oo00<00?ooool0__oo00<00?ooool0:?oo0@00
0Ooo000Kool00`00ooooo`0Eool00`00ooooo`2lool00`00ooooo`0Yool10001ool001_oo`8001Ko
o`03003ooooo0;coo`03003ooooo02Soo`80007oo`006ooo00<00?ooool05Ooo00<00?ooool0^ooo
00<00?ooool0:_oo0@000Ooo000Kool00`00ooooo`0Fool00`00ooooo`2iool00`00ooooo`0[ool1
0001ool000;oo`8000Coo`03003ooooo00?oo`8000Coo`8000Goo`03003ooooo01Koo`03003ooooo
0;Soo`03003ooooo02coo`40007oo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000oooo
o`001?oo00<00?ooool05_oo00<00?ooool0^?oo00<00?ooool0;?oo0@000Ooo00001Ooo003ooooo
00002?oo00@00?ooool000;oo`04003ooooo0004ool2000Hool00`00ooooo`2fool00`00ooooo`0/
ool20001ool00005ool00?ooool00008ool01000ooooo`000_oo0`001Ooo00<00?ooool05ooo00<0
0?ooool0]Ooo00<00?ooool0;_oo0@000Ooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`03
003ooooo00Goo`03003ooooo01Ooo`03003ooooo0;Coo`03003ooooo02ooo`40007oo`0000Goo`00
ooooo`0000Soo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`0Hool00`00ooooo`2cool0
0`00ooooo`0_ool10001ool000;oo`8000[oo`8000Coo`<000Coo`03003ooooo01Soo`03003ooooo
0;;oo`03003ooooo033oo`40007oo`006ooo0P006Ooo00<00?ooool0/Ooo00<00?ooool0<?oo0P00
0Ooo000Kool00`00ooooo`0Iool00`00ooooo`2_ool00`00ooooo`0bool10001ool001_oo`03003o
oooo01Woo`03003ooooo0:koo`03003ooooo03?oo`40007oo`006ooo00<00?ooool06Ooo00<00?oo
ool0[_oo00<00?ooool0<ooo0@000Ooo000Kool00`00ooooo`0Jool00`00ooooo`2/ool00`00oooo
o`0dool10001ool001_oo`8001_oo`03003ooooo0:_oo`03003ooooo03Coo`80007oo`006ooo00<0
0?ooool06ooo00<00?ooool0ZOoo00<00?ooool0=_oo0@000Ooo000Kool00`00ooooo`0Kool00`00
ooooo`2Xool00`00ooooo`0gool10001ool001_oo`03003ooooo01coo`03003ooooo0:Ooo`03003o
oooo03Ooo`40007oo`006ooo00<00?ooool07?oo00<00?ooool0Y_oo00<00?ooool0>?oo0@000Ooo
000Kool2000Mool00`00ooooo`2Uool00`00ooooo`0hool20001ool001_oo`03003ooooo01goo`03
003ooooo0:?oo`03003ooooo03[oo`40007oo`006ooo00<00?ooool07Ooo00<00?ooool0Xooo00<0
0?ooool0>_oo0@000Ooo0002ool20004ool00`00ooooo`03ool20004ool20005ool00`00ooooo`0N
ool00`00ooooo`2Qool00`00ooooo`0kool10001ool00005ool00?ooool00008ool01000ooooo`00
0_oo00@00?ooool000Coo`03003ooooo01koo`03003ooooo0:3oo`03003ooooo03coo`40007oo`00
00Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001?oo0P007ooo00<00?ooool0Wooo
00<00?ooool0??oo0P000Ooo00001Ooo003ooooo00002?oo00@00?ooool000?oo`8000Goo`03003o
oooo01ooo`03003ooooo09goo`03003ooooo03koo`40007oo`0000Goo`00ooooo`0000Soo`04003o
oooo0002ool01000ooooo`001?oo00<00?ooool07ooo00<00?ooool0WOoo00<00?ooool0?_oo0@00
0Ooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`0Pool0
0`00ooooo`2Kool00`00ooooo`0oool10001ool000;oo`8000[oo`8000Coo`8000Goo`03003ooooo
023oo`03003ooooo09[oo`03003ooooo043oo`40007oo`006ooo00<00?ooool08Ooo00<00?ooool0
V?oo00<00?ooool0@Ooo0@000Ooo000Kool2000Rool00`00ooooo`2Gool00`00ooooo`11ool20001
ool001_oo`03003ooooo027oo`03003ooooo09Koo`03003ooooo04?oo`40007oo`006ooo00<00?oo
ool08_oo00<00?ooool0UOoo00<00?ooool0@ooo0@000Ooo000Kool00`00ooooo`0Rool00`00oooo
o`2Dool00`00ooooo`14ool10001ool001_oo`03003ooooo02?oo`03003ooooo09;oo`03003ooooo
04Goo`40007oo`006ooo0P009?oo00<00?ooool0TOoo00<00?ooool0AOoo0P000Ooo000Kool00`00
ooooo`0Tool00`00ooooo`2?ool00`00ooooo`17ool10001ool001_oo`03003ooooo02Coo`03003o
oooo08koo`03003ooooo04Soo`40007oo`006ooo00<00?ooool09Ooo00<00?ooool0SOoo00<00?oo
ool0B?oo0@000Ooo000Kool00`00ooooo`0Uool00`00ooooo`2<ool00`00ooooo`19ool10001ool0
01_oo`8002Ooo`03003ooooo08[oo`03003ooooo04Woo`80007oo`006ooo00<00?ooool09_oo00<0
0?ooool0ROoo00<00?ooool0Booo0@000Ooo000Kool00`00ooooo`0Wool00`00ooooo`27ool00`00
ooooo`1<ool10001ool000Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo02Ooo`03003o
oooo08Koo`03003ooooo04goo`40007oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003o
oooo02Soo`03003ooooo08Coo`03003ooooo04koo`40007oo`001ooo00@00?ooool000[oo`03003o
oooo00?oo`8002Woo`03003ooooo08?oo`03003ooooo04koo`80007oo`001ooo00@00?ooool000[o
o`03003ooooo00?oo`03003ooooo02Woo`03003ooooo087oo`03003ooooo053oo`40007oo`001ooo
00@00?ooool000[oo`03003ooooo00?oo`03003ooooo02Woo`03003ooooo087oo`03003ooooo053o
o`40007oo`001ooo00@00?ooool000Soo`<000Goo`03003ooooo02[oo`03003ooooo07ooo`03003o
oooo057oo`40007oo`002?oo0P002ooo00<00?ooool00ooo00<00?ooool0:_oo00<00?ooool0O_oo
00<00?ooool0D_oo0@000Ooo000Kool2000/ool00`00ooooo`1lool00`00ooooo`1Bool20001ool0
01_oo`03003ooooo02_oo`03003ooooo07_oo`03003ooooo05Coo`40007oo`006ooo00<00?ooool0
;?oo00<00?ooool0NOoo00<00?ooool0EOoo0@000Ooo000Kool00`00ooooo`0/ool00`00ooooo`1h
ool00`00ooooo`1Fool10001ool001_oo`03003ooooo02goo`03003ooooo07Koo`03003ooooo05Oo
o`40007oo`006ooo0P00;_oo00<00?ooool0MOoo00<00?ooool0Eooo0P000Ooo000Kool00`00oooo
o`0^ool00`00ooooo`1cool00`00ooooo`1Iool10001ool001_oo`03003ooooo02koo`03003ooooo
07;oo`03003ooooo05[oo`40007oo`006ooo00<00?ooool0;ooo00<00?ooool0L?oo00<00?ooool0
Fooo0@000Ooo000Kool00`00ooooo`0`ool00`00ooooo`1^ool00`00ooooo`1Lool10001ool001_o
o`80037oo`03003ooooo06goo`03003ooooo05coo`80007oo`006ooo00<00?ooool0<Ooo00<00?oo
ool0Jooo00<00?ooool0G_oo0@000Ooo000Kool00`00ooooo`0bool00`00ooooo`1Yool00`00oooo
o`1Oool10001ool000;oo`8000Coo`03003ooooo00;oo`@000;oo`@000Coo`03003ooooo03;oo`03
003ooooo06Soo`03003ooooo063oo`40007oo`0000Goo`00ooooo`0000[oo`05003oooooool00007
ool00`00ooooo`0cool00`00ooooo`1Vool00`00ooooo`1Qool10001ool00005ool00?ooool0000:
ool00`00ooooo`02ool00`00ooooo`04ool2000eool00`00ooooo`1Tool00`00ooooo`1Qool20001
ool00005ool00?ooool0000:ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`0dool00`00
ooooo`1Sool00`00ooooo`1Sool10001ool00005ool00?ooool0000:ool00`00ooooo`04ool00`00
ooooo`02ool00`00ooooo`0eool00`00ooooo`1Qool00`00ooooo`1Tool10001ool00005ool00?oo
ool00008ool30003ool01000ooooo`001?oo00<00?ooool0=_oo00<00?ooool0Gooo00<00?ooool0
IOoo0@000Ooo0002ool2000;ool00`00ooooo`02ool20005ool00`00ooooo`0gool00`00ooooo`1L
ool2001Xool10001ool001_oo`8003Soo`03003ooooo05_oo`03003ooooo06Ooo`80007oo`006ooo
00<00?ooool0>?oo00<00?ooool0FOoo00<00?ooool0JOoo0@000Ooo000Kool00`00ooooo`0iool0
0`00ooooo`1Gool00`00ooooo`1Zool10001ool001_oo`03003ooooo03Woo`03003ooooo05Koo`03
003ooooo06_oo`40007oo`006ooo00<00?ooool0>_oo00<00?ooool0E?oo00<00?ooool0K?oo0@00
0Ooo000Kool00`00ooooo`0kool00`00ooooo`1Aool2001_ool10001ool001_oo`8003goo`03003o
oooo04ooo`03003ooooo06koo`80007oo`006ooo00<00?ooool0?Ooo00<00?ooool0COoo00<00?oo
ool0L?oo0@000Ooo000Kool00`00ooooo`0nool00`00ooooo`1:ool2001cool10001ool001_oo`03
003ooooo03ooo`03003ooooo04Soo`03003ooooo07?oo`40007oo`006ooo00<00?ooool0@?oo00<0
0?ooool0A_oo00<00?ooool0M?oo0@000Ooo000Kool20012ool00`00ooooo`13ool2001fool20001
ool001_oo`03003ooooo04;oo`03003ooooo047oo`03003ooooo07Ooo`40007oo`006ooo00<00?oo
ool0@ooo00<00?ooool0?_oo0P00N_oo0@000Ooo0002ool20004ool00`00ooooo`02ool40004ool0
0`00ooooo`03ool00`00ooooo`14ool00`00ooooo`0lool00`00ooooo`1jool10001ool00005ool0
0?ooool0000:ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`15ool00`00ooooo`0iool2
001mool10001ool00005ool00?ooool0000:ool00`00ooooo`050004ool20017ool2000gool2001n
ool20001ool00005ool00?ooool0000:ool01000ooooo`000_oo00<00?ooool00ooo00<00?ooool0
B?oo00<00?ooool0<ooo00<00?ooool0Oooo0@000Ooo00001Ooo003ooooo00002_oo00L00?oooooo
o`00ool00005ool00`00ooooo`19ool2000aool20022ool10001ool00005ool00?ooool00008ool3
0004ool20005ool00`00ooooo`1;ool2000]ool20024ool10001ool000;oo`8000_oo`03003ooooo
00?oo`03003ooooo00?oo`03003ooooo04goo`03003ooooo02Soo`8008Koo`40007oo`006ooo0P00
Cooo0P009Ooo0`00Qooo0P000Ooo000Kool00`00ooooo`1@ool2000Qool2002;ool10001ool001_o
o`03003ooooo05;oo`@001[oo`<008goo`40007oo`006ooo00<00?ooool0E_oo10004_oo1000T?oo
0@000Ooo000Kool00`00ooooo`1Jool70007ool4002Dool10001ool001_oo`8006;oo`L009Ooo`80
007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo000Kool00`00ooooo`03ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool0
0`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`07ool00`00ool00001ool001_oool000D0007oo`00oooo8Ooo003o
oolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.152837, -0.0178024, \
0.0045721, 0.000979057}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<ustablenew2.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("ustablenew2.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, omdamp2[k, Pi/4, \(-9\)/10]}, {k, 0.01, 0.4269, 
        0.1}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.01`, 0.002031839151388947`}, {0.11`, 
        0.020687192858023132`}, {0.21000000000000002`, 
        0.03143687369281081`}, {0.31`, 
        0.028378588570490883`}, {0.41000000000000003`, 
        0.006160908378086199`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<ximmodadamp.dat\>", %45]\)], "Input"],

Cell[BoxData[
    \("ximmodadamp.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[Pi/4, 1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(xi\/\(\@\(xi\^2 - \(\((1 - \
\(\[LeftSkeleton] 19 \[RightSkeleton]\)\^2\/0.5`\^2)\)\)\)\\ \((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)\)) is less than WorkingPrecision (\!\(20\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(xi\/\(\@\(xi\^2 - \(\((1 - \
\(\[LeftSkeleton] 19 \[RightSkeleton]\)\^2\/0.5`\^2)\)\)\)\\ \((1 + \(\(1\\ \
xi\^2\)\))\)\^2\)\)) is less than WorkingPrecision (\!\(20\))."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"precw" \(\(:\)\(\ \)\) 
      "The precision of the argument function (\!\(\(xi\\ 0.3535533905932738` \
\(\(\[LeftSkeleton] 1 \[RightSkeleton]\)\) \(\(\[LeftSkeleton] 4 \
\[RightSkeleton]\)\)\\ \(\((\(\(2\\ xi\^2\)\) - 1 + \(\[LeftSkeleton] 19 \
\[RightSkeleton]\)\^2\/0.5`\^2)\)\)\)\/\(\(\((0.5`\^2 - \(\[LeftSkeleton] 19 \
\[RightSkeleton]\)\^2)\)\)\\ \(\((\((1 + \(\(1\\ xi\^2\)\))\)\^2\\ \@\(xi\^2 \
- 1 + \(\[LeftSkeleton] 1 \[RightSkeleton]\)\/\(\[LeftSkeleton] 1 \
\[RightSkeleton]\)\))\)\)\)\)) is less than WorkingPrecision (\!\(20\))."\)], \
"Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"precw\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(\(\(0.07665615402447888`\)\(\[InvisibleSpace]\)\) - 
      1.025140514101287`*^-8\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, omdamp2[k, Pi/16, 1]}, {k, 0.01, 0.474, 0.01}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.01`, 0.0022628582026016364`}, {0.02`, 
        0.00451609339314644`}, {0.03`, 0.006750146625071374`}, {0.04`, 
        0.008955585742296405`}, {0.05`, 0.01112316542383685`}, {0.06`, 
        0.013243883381409291`}, {0.07`, 0.015309031683177944`}, {0.08`, 
        0.01731024236349265`}, {0.09`, 0.019239526698239997`}, {0.1`, 
        0.021089307752791834`}, {0.11`, 0.02285244603355796`}, {0.12`, 
        0.024522258284094916`}, {0.13`, 0.026092529653613392`}, {0.14`, 
        0.02755751962311826`}, {0.15`, 0.028911962198557815`}, {0.16`, 
        0.030151060970063072`}, {0.17`, 0.031270479692650566`}, {0.18`, 
        0.032266329069397214`}, {0.19`, 0.033135150417065024`}, {0.2`, 
        0.03387388673735166`}, {0.21`, 0.03447990620692733`}, {0.22`, 
        0.03495090756172427`}, {0.23`, 0.03528495122338688`}, {0.24`, 
        0.035480421118708645`}, {0.25`, 0.03553600168313129`}, {0.26`, 
        0.03545065522605731`}, {0.27`, 0.03522359990713933`}, {0.28`, 
        0.034854288518269674`}, {0.29`, 0.034342388217555087`}, {0.3`, 
        0.03368776131923211`}, {0.31`, 0.03289044720708918`}, {0.32`, 
        0.03195062210879003`}, {0.33`, 0.030868688300655745`}, {0.34`, 
        0.029645078871071744`}, {0.35000000000000003`, 
        0.02828038628772072`}, {0.36`, 0.026775306066352273`}, {0.37`, 
        0.02513062575992676`}, {0.38`, 0.0233472148777841`}, {0.39`, 
        0.021426015674488687`}, {0.4`, 
        0.019368034755514164`}, {0.41000000000000003`, 
        0.01717431864945086`}, {0.42`, 0.014846026360624445`}, {0.43`, 
        0.012384276968323189`}, {0.44`, 0.009790270951692144`}, {0.45`, 
        0.007065193130858717`}, {0.46`, 
        0.004210432990624037`}, {0.47000000000000003`, 
        0.0012271392213732072`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.02634 0.0147151 16.5636 [
[.22644 .00222 -9 -9 ]
[.22644 .00222 9 0 ]
[.42908 .00222 -9 -9 ]
[.42908 .00222 9 0 ]
[.63171 .00222 -9 -9 ]
[.63171 .00222 9 0 ]
[.83435 .00222 -9 -9 ]
[.83435 .00222 9 0 ]
[.01131 .09753 -30 -4.5 ]
[.01131 .09753 0 4.5 ]
[.01131 .18035 -24 -4.5 ]
[.01131 .18035 0 4.5 ]
[.01131 .26317 -30 -4.5 ]
[.01131 .26317 0 4.5 ]
[.01131 .34599 -24 -4.5 ]
[.01131 .34599 0 4.5 ]
[.01131 .4288 -30 -4.5 ]
[.01131 .4288 0 4.5 ]
[.01131 .51162 -24 -4.5 ]
[.01131 .51162 0 4.5 ]
[.01131 .59444 -30 -4.5 ]
[.01131 .59444 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.22644 .01472 m
.22644 .02097 L
s
[(0.1)] .22644 .00222 0 1 Mshowa
.42908 .01472 m
.42908 .02097 L
s
[(0.2)] .42908 .00222 0 1 Mshowa
.63171 .01472 m
.63171 .02097 L
s
[(0.3)] .63171 .00222 0 1 Mshowa
.83435 .01472 m
.83435 .02097 L
s
[(0.4)] .83435 .00222 0 1 Mshowa
.125 Mabswid
.06434 .01472 m
.06434 .01847 L
s
.10486 .01472 m
.10486 .01847 L
s
.14539 .01472 m
.14539 .01847 L
s
.18592 .01472 m
.18592 .01847 L
s
.26697 .01472 m
.26697 .01847 L
s
.3075 .01472 m
.3075 .01847 L
s
.34802 .01472 m
.34802 .01847 L
s
.38855 .01472 m
.38855 .01847 L
s
.4696 .01472 m
.4696 .01847 L
s
.51013 .01472 m
.51013 .01847 L
s
.55066 .01472 m
.55066 .01847 L
s
.59119 .01472 m
.59119 .01847 L
s
.67224 .01472 m
.67224 .01847 L
s
.71277 .01472 m
.71277 .01847 L
s
.75329 .01472 m
.75329 .01847 L
s
.79382 .01472 m
.79382 .01847 L
s
.87487 .01472 m
.87487 .01847 L
s
.9154 .01472 m
.9154 .01847 L
s
.95593 .01472 m
.95593 .01847 L
s
.99645 .01472 m
.99645 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .09753 m
.03006 .09753 L
s
[(0.005)] .01131 .09753 1 0 Mshowa
.02381 .18035 m
.03006 .18035 L
s
[(0.01)] .01131 .18035 1 0 Mshowa
.02381 .26317 m
.03006 .26317 L
s
[(0.015)] .01131 .26317 1 0 Mshowa
.02381 .34599 m
.03006 .34599 L
s
[(0.02)] .01131 .34599 1 0 Mshowa
.02381 .4288 m
.03006 .4288 L
s
[(0.025)] .01131 .4288 1 0 Mshowa
.02381 .51162 m
.03006 .51162 L
s
[(0.03)] .01131 .51162 1 0 Mshowa
.02381 .59444 m
.03006 .59444 L
s
[(0.035)] .01131 .59444 1 0 Mshowa
.125 Mabswid
.02381 .03128 m
.02756 .03128 L
s
.02381 .04784 m
.02756 .04784 L
s
.02381 .06441 m
.02756 .06441 L
s
.02381 .08097 m
.02756 .08097 L
s
.02381 .1141 m
.02756 .1141 L
s
.02381 .13066 m
.02756 .13066 L
s
.02381 .14722 m
.02756 .14722 L
s
.02381 .16379 m
.02756 .16379 L
s
.02381 .19691 m
.02756 .19691 L
s
.02381 .21348 m
.02756 .21348 L
s
.02381 .23004 m
.02756 .23004 L
s
.02381 .24661 m
.02756 .24661 L
s
.02381 .27973 m
.02756 .27973 L
s
.02381 .2963 m
.02756 .2963 L
s
.02381 .31286 m
.02756 .31286 L
s
.02381 .32942 m
.02756 .32942 L
s
.02381 .36255 m
.02756 .36255 L
s
.02381 .37911 m
.02756 .37911 L
s
.02381 .39568 m
.02756 .39568 L
s
.02381 .41224 m
.02756 .41224 L
s
.02381 .44537 m
.02756 .44537 L
s
.02381 .46193 m
.02756 .46193 L
s
.02381 .4785 m
.02756 .4785 L
s
.02381 .49506 m
.02756 .49506 L
s
.02381 .52819 m
.02756 .52819 L
s
.02381 .54475 m
.02756 .54475 L
s
.02381 .56131 m
.02756 .56131 L
s
.02381 .57788 m
.02756 .57788 L
s
.02381 .611 m
.02756 .611 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.04407 .0522 m
.06434 .08952 L
.0846 .12652 L
.10486 .16305 L
.12513 .19895 L
.14539 .23408 L
.16565 .26829 L
.18592 .30143 L
.20618 .33339 L
.22644 .36403 L
.24671 .39323 L
.26697 .42089 L
.28723 .4469 L
.3075 .47117 L
.32776 .4936 L
.34802 .51412 L
.36829 .53267 L
.38855 .54916 L
.40881 .56355 L
.42908 .57579 L
.44934 .58583 L
.4696 .59363 L
.48987 .59916 L
.51013 .6024 L
.5304 .60332 L
.55066 .60191 L
.57092 .59814 L
.59119 .59203 L
.61145 .58355 L
.63171 .57271 L
.65198 .5595 L
.67224 .54393 L
.6925 .52601 L
.71277 .50574 L
.73303 .48314 L
.75329 .45821 L
.77356 .43097 L
.79382 .40143 L
.81408 .36961 L
.83435 .33552 L
.85461 .29918 L
.87487 .26062 L
.89514 .21984 L
.9154 .17688 L
.93566 .13174 L
.95593 .08445 L
.97619 .03504 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00CP3oool2
000000@0oooo00<000000?ooo`3oool00P3oool4000002H0oooo0P0000040?ooo`030000003oool0
oooo0080oooo1000000V0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000009P3oool2
000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0T0?ooo`00C@3oool01000
0000oooo0?ooo`00000:0?ooo`030000003oool0oooo02@0oooo00@000000?ooo`3oool00000203o
ool00`000000oooo0?ooo`0V0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
0000903oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo02@0oooo001=0?ooo`04
0000003oool0oooo000000X0oooo00<000000?ooo`3oool0903oool010000000oooo0?ooo`000009
0?ooo`030000003oool0oooo02D0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?oo
o`0R0?ooo`040000003oool0oooo000000L0oooo1@00000U0?ooo`00C@3oool010000000oooo0?oo
o`00000:0?ooo`030000003oool0oooo02@0oooo00@000000?ooo`3oool000002P3oool00`000000
oooo0?ooo`0T0?ooo`040000003oool0oooo000000T0oooo0P00000U0?ooo`040000003oool0oooo
000000L0oooo00@000000?ooo`3oool000009P3oool004d0oooo00@000000?ooo`3oool000002P3o
ool00`000000oooo0?ooo`0T0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0
8`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0280oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0000000V0?ooo`00C@3oool010000000oooo0?ooo`0000080?oo
o`<000009P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002D0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`00000T0?ooo`040000003oool0oooo0000
00T0oooo0P00000V0?ooo`00CP3oool2000000/0oooo00<000000?ooo`3oool09@3oool2000000X0
oooo0P00000W0?ooo`8000002P3oool2000002H0oooo0P00000;0?ooo`030000003oool0oooo02@0
oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q
0?ooo`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000K0?ooool0
00001P0000000240oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo00P0oooo0@0000010?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool0
0`000000oooo0?ooo`3l0?ooo`008@3oool200000?d0oooo000Q0?ooo`030000003oool0oooo0?@0
oooo00<000000?ooo`3oool01@3oool00240oooo00<000000?ooo`3oool0m03oool00`000000oooo
0?ooo`050?ooo`008@3oool00`000000oooo0?ooo`3c0?ooo`030000003oool0oooo00H0oooo000Q
0?ooo`800000m03oool00`000000oooo0?ooo`060?ooo`008@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0>d0oooo00<000000?ooo`3oool01`3oool00240oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`3]0?ooo`030000003oool0oooo00L0oooo000Q0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0k03oool00`000000oooo0?ooo`070?ooo`008@3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0>/0oooo00<000000?ooo`3oool0203o
ool00240oooo0P0000050?ooo`030000003oool0oooo0>X0oooo00<000000?ooo`3oool0203oool0
0240oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3Z0?ooo`030000003oool0oooo
00P0oooo000Q0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0j03oool00`000000
oooo0?ooo`090?ooo`008@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0>P0oooo
00<000000?ooo`3oool02@3oool00240oooo0P0000070?ooo`030000003oool0oooo0>H0oooo00<0
00000?ooo`3oool02P3oool00240oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3V
0?ooo`030000003oool0oooo00X0oooo00020?ooo`800000103oool00`000000oooo0?ooo`030?oo
o`800000103oool2000000<0oooo0`0000050?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0i@3oool00`000000oooo0?ooo`0:0?ooo`0000D0oooo0000003oool0oooo000000080?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0h`3oool00`000000oooo0?oo
o`0;0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo00000080oooo
00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`8000002@3oool00`000000
oooo0?ooo`3S0?ooo`030000003oool0oooo00/0oooo00001@3oool000000?ooo`3oool0000000P0
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`<000001@3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0>40oooo00<000000?ooo`3oool0303oool0
00050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000020?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0>40oooo00<000000?ooo`3oool0303oool000050?ooo`000000oooo0?ooo`000000
203oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0>00oooo00<000000?oo
o`3oool0303oool00080oooo0P00000:0?ooo`800000103oool2000000<0oooo100000040?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool0gP3oool00`000000oooo0?ooo`0=0?ooo`00
8@3oool2000000`0oooo00<000000?ooo`3oool0gP3oool00`000000oooo0?ooo`0=0?ooo`008@3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0=`0oooo00<000000?ooo`3oool03P3o
ool00240oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3L0?ooo`030000003oool0
oooo00h0oooo000Q0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0fP3oool00`00
0000oooo0?ooo`0?0?ooo`008@3oool2000000l0oooo00<000000?ooo`3oool0f@3oool00`000000
oooo0?ooo`0?0?ooo`008@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0=T0oooo
00<000000?ooo`3oool03`3oool00240oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`3G0?ooo`030000003oool0oooo0100oooo000Q0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool0e`3oool00`000000oooo0?ooo`0@0?ooo`008@3oool200000140oooo00<000000?oo
o`3oool0e@3oool00`000000oooo0?ooo`0A0?ooo`008@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0=D0oooo00<000000?ooo`3oool04@3oool00240oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo0140oooo000Q0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0d`3oool00`000000oooo0?ooo`0B0?ooo`008@3oool2
000001<0oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`0B0?ooo`008@3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo0=40oooo00<000000?ooo`3oool04`3oool00240
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo01<0
oooo00080?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool4000000@0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3?0?ooo`030000003oool0oooo01@0oooo
00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0cP3oool00`000000
oooo0?ooo`0D0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
00@0oooo00<000000?ooo`3oool00`3oool2000001D0oooo00<000000?ooo`3oool0c@3oool00`00
0000oooo0?ooo`0E0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0
oooo0<`0oooo00<000000?ooo`3oool05@3oool000L0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`3;0?ooo`030000003oool0oooo01H0oooo00070?ooo`040000003oool0
oooo000000P0oooo00@000000?ooo`3oool000000P3oool3000000D0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`3:0?ooo`030000003oool0oooo01H0oooo00080?ooo`8000002P3o
ool2000000D0oooo00<000000?ooo`3oool00`3oool2000001L0oooo00<000000?ooo`3oool0b@3o
ool00`000000oooo0?ooo`0G0?ooo`008@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo0<P0oooo00<000000?ooo`3oool05`3oool00240oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`360?ooo`030000003oool0oooo01P0oooo000Q0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0aP3oool00`000000oooo0?ooo`0H0?ooo`008@3oool2000001X0oooo
00<000000?ooo`3oool0a03oool00`000000oooo0?ooo`0I0?ooo`008@3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo0<@0oooo00<000000?ooo`3oool06@3oool00240oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`320?ooo`030000003oool0oooo01X0oooo000Q0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0J0?oo
o`008@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0<00oooo00<000000?ooo`3o
ool06`3oool00240oooo0P00000L0?ooo`030000003oool0oooo0<00oooo00<000000?ooo`3oool0
6`3oool00240oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`2n0?ooo`030000003o
ool0oooo01`0oooo000Q0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0_@3oool0
0`000000oooo0?ooo`0L0?ooo`008@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
0;`0oooo00<000000?ooo`3oool07@3oool00240oooo0P00000O0?ooo`030000003oool0oooo0;/0
oooo00<000000?ooo`3oool07@3oool00240oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`2i0?ooo`030000003oool0oooo01h0oooo00020?ooo`800000103oool00`000000oooo0?oo
o`030?ooo`8000000`3oool400000080oooo0`0000050?ooo`030000003oool0oooo01l0oooo00<0
00000?ooo`3oool0^@3oool00`000000oooo0?ooo`0N0?ooo`0000D0oooo0000003oool0oooo0000
00080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0]`3oool00`000000oooo
0?ooo`0O0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`8000008P3oool00`000000
oooo0?ooo`2f0?ooo`030000003oool0oooo01l0oooo00001@3oool000000?ooo`3oool0000000P0
oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`3oool3000000D0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`2f0?ooo`030000003oool0oooo01l0oooo00001@3o
ool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool01@000000oooo0?oo
o`3oool0000000L0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`2d0?ooo`030000
003oool0oooo0200oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool0
00000P3oool3000000<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0S0?ooo`03
0000003oool0oooo0;<0oooo00<000000?ooo`3oool0803oool00080oooo0P00000:0?ooo`800000
1@3oool010000000oooo0?ooo`3oool4000000@0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`2b0?ooo`030000003oool0oooo0240oooo000Q0?ooo`8000009@3oool00`000000oooo
0?ooo`2a0?ooo`030000003oool0oooo0240oooo000Q0?ooo`030000003oool0oooo02@0oooo00<0
00000?ooo`3oool0/03oool00`000000oooo0?ooo`0R0?ooo`008@3oool00`000000oooo0?ooo`0U
0?ooo`030000003oool0oooo0:l0oooo00<000000?ooo`3oool08P3oool00240oooo00<000000?oo
o`3oool09P3oool00`000000oooo0?ooo`2]0?ooo`030000003oool0oooo02<0oooo000Q0?ooo`80
00009`3oool00`000000oooo0?ooo`2]0?ooo`030000003oool0oooo02<0oooo000Q0?ooo`030000
003oool0oooo02L0oooo00<000000?ooo`3oool0Z`3oool00`000000oooo0?ooo`0T0?ooo`008@3o
ool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0:T0oooo00<000000?ooo`3oool09@3o
ool00240oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`2Y0?ooo`030000003oool0
oooo02D0oooo000Q0?ooo`800000:P3oool00`000000oooo0?ooo`2W0?ooo`030000003oool0oooo
02H0oooo000Q0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0YP3oool00`000000
oooo0?ooo`0W0?ooo`008@3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0:D0oooo
00<000000?ooo`3oool09`3oool00240oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?oo
o`2T0?ooo`030000003oool0oooo02P0oooo000Q0?ooo`030000003oool0oooo02/0oooo00<00000
0?ooo`3oool0XP3oool00`000000oooo0?ooo`0Y0?ooo`008@3oool2000002`0oooo00<000000?oo
o`3oool0XP3oool00`000000oooo0?ooo`0Y0?ooo`008@3oool00`000000oooo0?ooo`0/0?ooo`03
0000003oool0oooo0:00oooo00<000000?ooo`3oool0:P3oool000P0oooo0P0000040?ooo`030000
003oool0oooo00<0oooo0P0000030?ooo`@00000103oool00`000000oooo0?ooo`0]0?ooo`030000
003oool0oooo09h0oooo00<000000?ooo`3oool0:`3oool000L0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0;@3oool00`000000oooo0?ooo`2N0?ooo`030000003oool0oooo02/0oooo00070?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`040?oo
o`800000;`3oool00`000000oooo0?ooo`2L0?ooo`030000003oool0oooo02`0oooo00070?ooo`04
0000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0W03oool00`000000oooo0?ooo`0/
0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000D0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo09X0oooo00<0
00000?ooo`3oool0;@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000020?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0<03oool00`000000
oooo0?ooo`2I0?ooo`030000003oool0oooo02d0oooo00080?ooo`8000002P3oool2000000@0oooo
0P0000050?ooo`800000<@3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo02h0oooo
000Q0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?oo
o`0^0?ooo`008@3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo09D0oooo00<00000
0?ooo`3oool0;`3oool00240oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`2C0?oo
o`030000003oool0oooo0300oooo000Q0?ooo`800000=@3oool00`000000oooo0?ooo`2B0?ooo`03
0000003oool0oooo0300oooo000Q0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0
T@3oool00`000000oooo0?ooo`0a0?ooo`008@3oool00`000000oooo0?ooo`0e0?ooo`030000003o
ool0oooo0900oooo00<000000?ooo`3oool0<@3oool00240oooo00<000000?ooo`3oool0=P3oool0
0`000000oooo0?ooo`2>0?ooo`030000003oool0oooo0380oooo000Q0?ooo`030000003oool0oooo
03L0oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`0c0?ooo`008@3oool2000003T0
oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?ooo`0c0?ooo`008@3oool00`000000oooo
0?ooo`0h0?ooo`030000003oool0oooo08X0oooo00<000000?ooo`3oool0=03oool00240oooo00<0
00000?ooo`3oool0>@3oool00`000000oooo0?ooo`280?ooo`030000003oool0oooo03D0oooo000Q
0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo0?ooo`0e
0?ooo`008@3oool2000003`0oooo00<000000?ooo`3oool0Q@3oool00`000000oooo0?ooo`0f0?oo
o`008@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo08D0oooo00<000000?ooo`3o
ool0=P3oool00080oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`@00000
0P3oool3000000D0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`230?ooo`030000
003oool0oooo03L0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0?@3oool00`000000oooo0?ooo`210?ooo`030000003oool0oooo03P0oooo00001@3oool00000
0?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0080oooo0P00000n0?ooo`030000003oool0oooo0840oooo00<000000?oo
o`3oool0>03oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`000004
0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo0?ooo`0n0?ooo`030000003o
ool0oooo07l0oooo00<000000?ooo`3oool0>@3oool000050?ooo`000000oooo0?ooo`000000203o
ool010000000oooo0?ooo`0000050?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3o
ool0?`3oool00`000000oooo0?ooo`1m0?ooo`030000003oool0oooo03X0oooo00001@3oool00000
0?ooo`3oool0000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`1k
0?ooo`030000003oool0oooo03/0oooo00020?ooo`8000002P3oool2000000@0oooo0P0000030?oo
o`@00000103oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo07X0oooo00<000000?oo
o`3oool0?03oool00240oooo0P0000120?ooo`030000003oool0oooo07T0oooo00<000000?ooo`3o
ool0?03oool00240oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1g0?ooo`030000
003oool0oooo03d0oooo000Q0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0M@3o
ool00`000000oooo0?ooo`0n0?ooo`008@3oool00`000000oooo0?ooo`130?ooo`030000003oool0
oooo07@0oooo00<000000?ooo`3oool0?`3oool00240oooo0P0000150?ooo`030000003oool0oooo
0780oooo00<000000?ooo`3oool0@03oool00240oooo00<000000?ooo`3oool0A@3oool00`000000
oooo0?ooo`1a0?ooo`030000003oool0oooo0400oooo000Q0?ooo`030000003oool0oooo04H0oooo
00<000000?ooo`3oool0K`3oool00`000000oooo0?ooo`110?ooo`008@3oool00`000000oooo0?oo
o`170?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0@P3oool00240oooo0P000019
0?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0@`3oool00240oooo00<000000?oo
o`3oool0B@3oool00`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo04<0oooo000Q0?ooo`03
0000003oool0oooo04X0oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`140?ooo`00
8@3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0
A@3oool00240oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`1T0?ooo`030000003o
ool0oooo04H0oooo000Q0?ooo`800000CP3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0
oooo04L0oooo000Q0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0H03oool00`00
0000oooo0?ooo`180?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000@0
oooo0P0000050?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0G`3oool00`000000
oooo0?ooo`190?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
0080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`1?0?ooo`030000003oool0
oooo05d0oooo00<000000?ooo`3oool0BP3oool000L0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo0P00001A0?ooo`030000003o
ool0oooo05/0oooo00<000000?ooo`3oool0B`3oool000L0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`0000030?ooo`8000001@3oool00`000000oooo0?ooo`1A0?ooo`030000
003oool0oooo05X0oooo00<000000?ooo`3oool0B`3oool000L0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0DP3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo04`0oooo00070?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo05<0oooo0P00001G0?ooo`030000003oool0oooo04d0oooo00080?oo
o`8000002P3oool2000000@0oooo0P0000050?ooo`800000EP3oool00`000000oooo0?ooo`1C0?oo
o`030000003oool0oooo04h0oooo000Q0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3o
ool0D03oool200000540oooo000Q0?ooo`030000003oool0oooo05L0oooo00<000000?ooo`3oool0
CP3oool00`000000oooo0?ooo`1A0?ooo`008@3oool00`000000oooo0?ooo`1H0?ooo`030000003o
ool0oooo04/0oooo0P00001D0?ooo`008@3oool00`000000oooo0?ooo`1I0?ooo`800000BP3oool0
0`000000oooo0?ooo`1D0?ooo`008@3oool2000005`0oooo00<000000?ooo`3oool0AP3oool00`00
0000oooo0?ooo`1E0?ooo`008@3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo04<0
oooo0P00001H0?ooo`008@3oool00`000000oooo0?ooo`1M0?ooo`800000@P3oool00`000000oooo
0?ooo`1H0?ooo`008@3oool00`000000oooo0?ooo`1O0?ooo`800000?`3oool00`000000oooo0?oo
o`1I0?ooo`008@3oool200000680oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`1J
0?ooo`008@3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo03P0oooo0P00001M0?oo
o`008@3oool00`000000oooo0?ooo`1S0?ooo`800000=`3oool00`000000oooo0?ooo`1M0?ooo`00
8@3oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
GP3oool00240oooo0P00001W0?ooo`800000<@3oool200000640oooo000Q0?ooo`030000003oool0
oooo06P0oooo0P00000]0?ooo`800000H`3oool00080oooo0P0000040?ooo`030000003oool0oooo
00<0oooo0P0000040?ooo`8000000`3oool3000000D0oooo00<000000?ooo`3oool0JP3oool30000
02P0oooo0P00001U0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo06d0oooo0`00000R0?ooo`<00000I`3oool000050?ooo`000000oooo0?ooo`000000203o
ool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
0P3oool200000740oooo0`00000K0?ooo`@00000JP3oool000050?ooo`000000oooo0?ooo`000000
203oool010000000oooo0?ooo`0000030?ooo`8000000`3oool3000000D0oooo00<000000?ooo`3o
ool0L`3oool2000001@0oooo1@00001^0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`04
0000003oool0oooo000000D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`1e
0?ooo`@000002P3oool6000007<0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@00000
0?ooo`3oool000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0N@3oool:000007T0oooo00020?ooo`8000002P3oool2000000@0oooo0P00
00030?ooo`@00000103oool00`000000oooo0?ooo`3l0?ooo`008@3oool200000?d0oooo000Q0?oo
o`030000003oool0oooo0?`0oooo000Q0?ooo`030000003oool0oooo0?`0oooo003o0?ooob40oooo
003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0631271, -0.00375815, \
0.00189854, 0.000232261}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<instbmode.dat\>", %367]\)], "Input"],

Cell[BoxData[
    \("instbmode.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(omdamp2[4/10, Pi/16, 1]\)], "Input"],

Cell[BoxData[
    \(0.019368035627205685`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(better!\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      testfuncspec[IntegerPart[om*10^16]*10^\((\(-16\))\)*I, 3/10, Pi/16, 
        1], {om, 0.001, 0.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.90476 1.29646 0.938924 [
[.21429 .34503 -9 -9 ]
[.21429 .34503 9 0 ]
[.40476 .34503 -9 -9 ]
[.40476 .34503 9 0 ]
[.59524 .34503 -9 -9 ]
[.59524 .34503 9 0 ]
[.78571 .34503 -9 -9 ]
[.78571 .34503 9 0 ]
[.97619 .34503 -9 -9 ]
[.97619 .34503 9 0 ]
[.01131 .07586 -24 -4.5 ]
[.01131 .07586 0 4.5 ]
[.01131 .16975 -24 -4.5 ]
[.01131 .16975 0 4.5 ]
[.01131 .26364 -24 -4.5 ]
[.01131 .26364 0 4.5 ]
[.01131 .45143 -24 -4.5 ]
[.01131 .45143 0 4.5 ]
[.01131 .54532 -24 -4.5 ]
[.01131 .54532 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .35753 m
.21429 .36378 L
s
[(0.1)] .21429 .34503 0 1 Mshowa
.40476 .35753 m
.40476 .36378 L
s
[(0.2)] .40476 .34503 0 1 Mshowa
.59524 .35753 m
.59524 .36378 L
s
[(0.3)] .59524 .34503 0 1 Mshowa
.78571 .35753 m
.78571 .36378 L
s
[(0.4)] .78571 .34503 0 1 Mshowa
.97619 .35753 m
.97619 .36378 L
s
[(0.5)] .97619 .34503 0 1 Mshowa
.125 Mabswid
.0619 .35753 m
.0619 .36128 L
s
.1 .35753 m
.1 .36128 L
s
.1381 .35753 m
.1381 .36128 L
s
.17619 .35753 m
.17619 .36128 L
s
.25238 .35753 m
.25238 .36128 L
s
.29048 .35753 m
.29048 .36128 L
s
.32857 .35753 m
.32857 .36128 L
s
.36667 .35753 m
.36667 .36128 L
s
.44286 .35753 m
.44286 .36128 L
s
.48095 .35753 m
.48095 .36128 L
s
.51905 .35753 m
.51905 .36128 L
s
.55714 .35753 m
.55714 .36128 L
s
.63333 .35753 m
.63333 .36128 L
s
.67143 .35753 m
.67143 .36128 L
s
.70952 .35753 m
.70952 .36128 L
s
.74762 .35753 m
.74762 .36128 L
s
.82381 .35753 m
.82381 .36128 L
s
.8619 .35753 m
.8619 .36128 L
s
.9 .35753 m
.9 .36128 L
s
.9381 .35753 m
.9381 .36128 L
s
.25 Mabswid
0 .35753 m
1 .35753 L
s
.02381 .07586 m
.03006 .07586 L
s
[(-1.3)] .01131 .07586 1 0 Mshowa
.02381 .16975 m
.03006 .16975 L
s
[(-1.2)] .01131 .16975 1 0 Mshowa
.02381 .26364 m
.03006 .26364 L
s
[(-1.1)] .01131 .26364 1 0 Mshowa
.02381 .45143 m
.03006 .45143 L
s
[(-0.9)] .01131 .45143 1 0 Mshowa
.02381 .54532 m
.03006 .54532 L
s
[(-0.8)] .01131 .54532 1 0 Mshowa
.125 Mabswid
.02381 .09463 m
.02756 .09463 L
s
.02381 .11341 m
.02756 .11341 L
s
.02381 .13219 m
.02756 .13219 L
s
.02381 .15097 m
.02756 .15097 L
s
.02381 .18853 m
.02756 .18853 L
s
.02381 .2073 m
.02756 .2073 L
s
.02381 .22608 m
.02756 .22608 L
s
.02381 .24486 m
.02756 .24486 L
s
.02381 .28242 m
.02756 .28242 L
s
.02381 .3012 m
.02756 .3012 L
s
.02381 .31998 m
.02756 .31998 L
s
.02381 .33875 m
.02756 .33875 L
s
.02381 .37631 m
.02756 .37631 L
s
.02381 .39509 m
.02756 .39509 L
s
.02381 .41387 m
.02756 .41387 L
s
.02381 .43265 m
.02756 .43265 L
s
.02381 .4702 m
.02756 .4702 L
s
.02381 .48898 m
.02756 .48898 L
s
.02381 .50776 m
.02756 .50776 L
s
.02381 .52654 m
.02756 .52654 L
s
.02381 .05708 m
.02756 .05708 L
s
.02381 .0383 m
.02756 .0383 L
s
.02381 .01952 m
.02756 .01952 L
s
.02381 .00074 m
.02756 .00074 L
s
.02381 .5641 m
.02756 .5641 L
s
.02381 .58287 m
.02756 .58287 L
s
.02381 .60165 m
.02756 .60165 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.137 .61803 m
.14675 .51995 L
.16801 .36011 L
.18811 .24806 L
.20805 .16626 L
.22645 .11121 L
.24514 .07125 L
.25577 .05441 L
.26572 .0419 L
.27608 .03184 L
.28593 .02473 L
.29477 .02014 L
.29977 .01822 L
.30442 .01684 L
.30948 .01574 L
.31234 .01531 L
.3137 .01514 L
.31496 .01501 L
.31617 .01491 L
.31727 .01484 L
.31849 .01478 L
.31979 .01474 L
.32105 .01472 L
.32176 .01472 L
.32242 .01472 L
.32362 .01474 L
.3249 .01479 L
.32604 .01484 L
.32709 .01491 L
.32945 .01511 L
.33153 .01534 L
.33373 .01564 L
.33838 .01644 L
.34347 .01758 L
.35372 .02061 L
.36304 .02412 L
.38395 .03408 L
.42385 .05842 L
.46224 .08497 L
.50308 .11363 L
.5424 .13993 L
.58021 .16307 L
.62047 .18488 L
.65922 .20287 L
.70041 .2187 L
.74009 .23078 L
.77826 .23956 L
.79793 .24304 L
.81888 .24599 L
.82917 .24717 L
Mistroke
.84029 .24823 L
.8499 .24899 L
.86043 .24964 L
.86541 .24989 L
.87075 .25011 L
.87536 .25026 L
.88037 .25039 L
.88331 .25045 L
.88602 .25049 L
.88862 .25052 L
.88993 .25053 L
.89064 .25053 L
.89138 .25054 L
.89268 .25054 L
.89387 .25054 L
.89515 .25054 L
.89588 .25054 L
.89655 .25054 L
.89778 .25054 L
.8989 .25053 L
.90019 .25052 L
.9014 .25051 L
.9038 .25048 L
.90604 .25044 L
.91113 .25033 L
.91609 .25019 L
.92142 .25 L
.93105 .24954 L
.93984 .249 L
.95725 .24762 L
.97619 .24564 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool200000?l0oooo103oool001/0oooo00<000000?ooo`3oool0
o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0A@3oool>00000:l0oooo000K0?ooo`80
0000A03oool2000000h0oooo0P00002]0?ooo`006`3oool00`000000oooo0?ooo`120?ooo`030000
003oool0oooo0100oooo0P00002[0?ooo`006`3oool00`000000oooo0?ooo`100?ooo`8000005@3o
ool200000:T0oooo000K0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool05`3oool2
00000:L0oooo000K0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool06P3oool20000
0:D0oooo000K0?ooo`800000?P3oool00`000000oooo0?ooo`0M0?ooo`800000X`3oool001/0oooo
00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0P0?ooo`800000X@3oool001/0oooo00<0
00000?ooo`3oool0>`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo09h0oooo000K
0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0903oool2000009h0oooo000K0?oo
o`030000003oool0oooo03X0oooo00<000000?ooo`3oool09`3oool2000009`0oooo000K0?ooo`80
0000>P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo09T0oooo000K0?ooo`030000
003oool0oooo03T0oooo00<000000?ooo`3oool0:`3oool2000009T0oooo000K0?ooo`030000003o
ool0oooo03P0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`2F0?ooo`001`3oool4
000000<0oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3oool0=`3oool0
0`000000oooo0?ooo`0`0?ooo`030000003oool0oooo09D0oooo00090?ooo`030000003oool0oooo
00L0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0g0?ooo`030000003oool0
oooo0340oooo0P00002E0?ooo`002@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
0080oooo0P00000g0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0TP3oool00003
0?ooo`000000000000800000103oool00`000000oooo0?ooo`080?ooo`8000001@3oool00`000000
oooo0?ooo`0f0?ooo`030000003oool0oooo03D0oooo0P00002B0?ooo`002@3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0=@3oool00`000000oooo
0?ooo`0h0?ooo`030000003oool0oooo08l0oooo00070?ooo`<000002@3oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0>@3oool2000008l0oooo
00090?ooo`030000003oool0oooo00P0oooo0P0000050?ooo`030000003oool0oooo03@0oooo00<0
00000?ooo`3oool0?03oool00`000000oooo0?ooo`2<0?ooo`006`3oool2000003D0oooo00<00000
0?ooo`3oool0?@3oool00`000000oooo0?ooo`2;0?ooo`006`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo03l0oooo0P00002;0?ooo`006`3oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo0440oooo00<000000?ooo`3oool0R03oool001/0oooo00<000000?ooo`3oool0
<P3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo08L0oooo000K0?ooo`030000003o
ool0oooo0380oooo00<000000?ooo`3oool0A03oool2000008L0oooo000K0?ooo`800000<`3oool0
0`000000oooo0?ooo`160?ooo`030000003oool0oooo08@0oooo000K0?ooo`030000003oool0oooo
0340oooo00<000000?ooo`3oool0B03oool2000008@0oooo000K0?ooo`030000003oool0oooo0340
oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`210?ooo`006`3oool00`000000oooo
0?ooo`0a0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0P03oool001/0oooo0P00
000a0?ooo`030000003oool0oooo04d0oooo0P0000200?ooo`006`3oool00`000000oooo0?ooo`0`
0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0O@3oool001/0oooo00<000000?oo
o`3oool0<03oool00`000000oooo0?ooo`1@0?ooo`800000O@3oool001/0oooo00<000000?ooo`3o
ool0;`3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo07X0oooo000K0?ooo`030000
003oool0oooo02l0oooo00<000000?ooo`3oool0E03oool2000007X0oooo000K0?ooo`800000;`3o
ool00`000000oooo0?ooo`1G0?ooo`800000N03oool001/0oooo00<000000?ooo`3oool0;P3oool0
0`000000oooo0?ooo`1I0?ooo`030000003oool0oooo07D0oooo000K0?ooo`030000003oool0oooo
02h0oooo00<000000?ooo`3oool0FP3oool2000007D0oooo00070?ooo`@000000`3oool00`000000
oooo0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo05d0
oooo0P00001c0?ooo`002@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`1O0?ooo`800000L@3oool000T0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`040?ooo`800000;P3oool00`000000oooo0?oo
o`1Q0?ooo`800000K`3oool000030?ooo`000000000000800000103oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`1S
0?ooo`800000K@3oool000T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo02`0oooo00<000000?ooo`3oool0IP3oool2000006/0oooo00070?ooo`<0
00002@3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo02`0oooo00<000000?oo
o`3oool0J03oool2000006T0oooo00090?ooo`030000003oool0oooo00P0oooo0P0000050?ooo`03
0000003oool0oooo02`0oooo00<000000?ooo`3oool0JP3oool2000006L0oooo000K0?ooo`800000
;@3oool00`000000oooo0?ooo`1/0?ooo`800000I@3oool001/0oooo00<000000?ooo`3oool0:`3o
ool00`000000oooo0?ooo`1_0?ooo`800000H`3oool001/0oooo00<000000?ooo`3oool0:`3oool0
0`000000oooo0?ooo`1a0?ooo`800000H@3oool001/0oooo00<000000?ooo`3oool0:`3oool00`00
0000oooo0?ooo`1c0?ooo`800000G`3oool001/0oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`1e0?ooo`<00000G03oool001/0oooo0P00000/0?ooo`030000003oool0oooo07P0oooo
0`00001I0?ooo`006`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo07`0oooo0P00
001G0?ooo`006`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo07h0oooo0`00001D
0?ooo`006`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0840oooo0`00001A0?oo
o`006`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo08@0oooo0`00001>0?ooo`00
6`3oool2000002X0oooo00<000000?ooo`3oool0R03oool4000004X0oooo000K0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0S03oool3000004L0oooo000K0?ooo`030000003oool0
oooo02T0oooo00<000000?ooo`3oool0S`3oool3000004@0oooo000K0?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool0TP3oool400000400oooo000K0?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0U`3oool4000003`0oooo000K0?ooo`800000:@3oool00`000000oooo
0?ooo`2K0?ooo`X000009P3oool3000000T0oooo000K0?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool0Y@3oool;000000h0oooo3@00000<0?ooo`006`3oool00`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo0;00oooo3P00000I0?ooo`001`3oool4000000<0oooo00<000000?oo
o`3oool00P3oool4000000@0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`3G0?oo
o`002@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`3H0?ooo`002@3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo00<0oooo0P00000X0?ooo`030000003oool0oooo0=P0oooo00000`3oool000000000
0002000000@0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo02L0oooo00<000000?ooo`3oool0f03oool000T0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0f03oool0
00L0oooo0`0000090?ooo`<000001@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo
0=P0oooo00090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0W0?ooo`030000003oool0oooo0=P0oooo000K0?ooo`8000009`3oool00`000000oooo
0?ooo`3I0?ooo`006`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0=T0oooo000K
0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0f@3oool001/0oooo00<000000?oo
o`3oool09P3oool00`000000oooo0?ooo`3I0?ooo`006`3oool00`000000oooo0?ooo`0V0?ooo`03
0000003oool0oooo0=T0oooo000K0?ooo`8000009`3oool00`000000oooo0?ooo`3I0?ooo`006`3o
ool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0=X0oooo000K0?ooo`030000003oool0
oooo02D0oooo00<000000?ooo`3oool0fP3oool001/0oooo00<000000?ooo`3oool09@3oool00`00
0000oooo0?ooo`3J0?ooo`006`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0=X0
oooo000K0?ooo`8000009P3oool00`000000oooo0?ooo`02000000@0oooo00<000000?ooo`3oool0
0P3oool4000002<0oooo0P0000040?ooo`030000003oool0oooo0080oooo1000000S0?ooo`800000
103oool00`000000oooo0?ooo`030?ooo`800000903oool2000000@0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0S0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<00000
0`3oool001/0oooo00<000000?ooo`3oool09@3oool00`000000oooo000000020?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool08@3oool010000000oooo0?ooo`0000080?ooo`030000
003oool0oooo02<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000R0?oo
o`040000003oool0oooo000000X0oooo00<000000?ooo`3oool08P3oool010000000oooo0?ooo`00
000;0?ooo`4000000@3oool10?ooo`006`3oool00`000000oooo0?ooo`0T0?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0Q0?ooo`040000003o
ool0oooo000000T0oooo00<000000?ooo`3oool08P3oool010000000oooo0?ooo`00000;0?ooo`03
0000003oool0oooo0200oooo00@000000?ooo`3oool000001`3oool5000002<0oooo00@000000?oo
o`3oool000002`3oool100000040oooo0@3oool001/0oooo00<000000?ooo`3oool0903oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool08@3oool0
10000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool00000
2@3oool2000002<0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00000T0?oo
o`040000003oool0oooo000000P0oooo0`0000030?ooo`006`3oool00`000000oooo0?ooo`0T0?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0Q
0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0803oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool00000203oool00`000000
oooo0000000T0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool00`3oool001/0
oooo0P00000U0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3oool30000
02<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000R0?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`0000090?oo
o`800000903oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00<0oooo000K0?oo
o`030000003oool0oooo02@0oooo00@000000?ooo`3oool0oooo0P00000;0?ooo`030000003oool0
oooo0280oooo0P00000:0?ooo`800000903oool2000000X0oooo0P00000T0?ooo`8000002`3oool0
0`000000oooo0?ooo`0S0?ooo`8000002@3oool400000080oooo000K0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0f`3oool001/0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`3L0?ooo`006`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0=`0oooo
000E0?ooool000002@0000030?ooo`006`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0
000000T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00L0oooo000K0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0g03oool001/0
oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`3L0?ooo`006`3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo0=d0oooo000K0?ooo`8000008`3oool00`000000oooo0?oo
o`3M0?ooo`006`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0=d0oooo000K0?oo
o`030000003oool0oooo0280oooo00<000000?ooo`3oool0g@3oool001/0oooo00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`3M0?ooo`006`3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo0=d0oooo000K0?ooo`8000008`3oool00`000000oooo0?ooo`3M0?ooo`006`3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0=h0oooo000K0?ooo`030000003oool0oooo
0240oooo00<000000?ooo`3oool0gP3oool001/0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`3N0?ooo`006`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0=h0oooo
000K0?ooo`8000008P3oool00`000000oooo0?ooo`3N0?ooo`006`3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0=h0oooo000K0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool0gP3oool001/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3O0?ooo`00
6`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0=l0oooo000K0?ooo`8000008@3o
ool00`000000oooo0?ooo`3O0?ooo`006`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0
oooo0=l0oooo000K0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0g`3oool000P0
oooo0P0000040?ooo`030000003oool0oooo0080oooo0`0000050?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool0g`3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0g`3oool000L0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?ooo`8000008@3oool00`000000
oooo0?ooo`3O0?ooo`0000<0oooo0000000000000P0000020?ooo`040000003oool0oooo000000T0
oooo0`0000040?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0h03oool000L0oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0h03oool000L0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0h03oool0
00P0oooo0P00000:0?ooo`8000001@3oool200000200oooo00<000000?ooo`3oool0h03oool001/0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`3P0?ooo`006`3oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo0>00oooo000K0?ooo`030000003oool0oooo01l0oooo00<0
00000?ooo`3oool0h03oool001/0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`3Q
0?ooo`006`3oool2000001l0oooo00<000000?ooo`3oool0h@3oool001/0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`3Q0?ooo`006`3oool00`000000oooo0?ooo`0N0?ooo`030000
003oool0oooo0>40oooo000K0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0h@3o
ool001/0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`3Q0?ooo`006`3oool20000
01l0oooo00<000000?ooo`3oool0h@3oool001/0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`3R0?ooo`006`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0>80oooo
000K0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0hP3oool001/0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`3R0?ooo`006`3oool2000001h0oooo00<000000?oo
o`3oool0hP3oool001/0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`3R0?ooo`00
6`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0>80oooo00080?ooo`800000103o
ool00`000000oooo0?ooo`030?ooo`8000001@3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo0>80oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
103oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0>80oooo00070?ooo`040000003o
ool0oooo000000P0oooo00@000000?ooo`3oool00000103oool2000001h0oooo00<000000?ooo`3o
ool0hP3oool000030?ooo`0000000000008000000P3oool010000000oooo0?ooo`0000090?ooo`80
00001@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0><0oooo00070?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo0><0oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool00000103oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0><0oooo00080?oo
o`8000002P3oool2000000D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3S0?oo
o`006`3oool2000001d0oooo00<000000?ooo`3oool0h`3oool001/0oooo00<000000?ooo`3oool0
703oool00`000000oooo0?ooo`3S0?ooo`006`3oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo0><0oooo000K0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0h`3oool0
01/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3S0?ooo`006`3oool2000001d0
oooo00<000000?ooo`3oool0h`3oool001/0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`3S0?ooo`006`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0><0oooo000K
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0i03oool001/0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`3T0?ooo`006`3oool2000001`0oooo00<000000?ooo`3o
ool0i03oool001/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3T0?ooo`006`3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0>@0oooo000K0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool0i03oool001/0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`3T0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0544294, -1.40961, \
0.00199436, 0.00404588}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, PlotRange \[Rule] All]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0219009 1.90858 0.260037 0.179707 [
[.21276 .24754 -9 -9 ]
[.21276 .24754 9 0 ]
[.40362 .24754 -9 -9 ]
[.40362 .24754 9 0 ]
[.59447 .24754 -9 -9 ]
[.59447 .24754 9 0 ]
[.78533 .24754 -9 -9 ]
[.78533 .24754 9 0 ]
[.97619 .24754 -9 -9 ]
[.97619 .24754 9 0 ]
[.0094 .08033 -12 -4.5 ]
[.0094 .08033 0 4.5 ]
[.0094 .17018 -24 -4.5 ]
[.0094 .17018 0 4.5 ]
[.0094 .34989 -18 -4.5 ]
[.0094 .34989 0 4.5 ]
[.0094 .43974 -6 -4.5 ]
[.0094 .43974 0 4.5 ]
[.0094 .5296 -18 -4.5 ]
[.0094 .5296 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21276 .26004 m
.21276 .26629 L
s
[(0.1)] .21276 .24754 0 1 Mshowa
.40362 .26004 m
.40362 .26629 L
s
[(0.2)] .40362 .24754 0 1 Mshowa
.59447 .26004 m
.59447 .26629 L
s
[(0.3)] .59447 .24754 0 1 Mshowa
.78533 .26004 m
.78533 .26629 L
s
[(0.4)] .78533 .24754 0 1 Mshowa
.97619 .26004 m
.97619 .26629 L
s
[(0.5)] .97619 .24754 0 1 Mshowa
.125 Mabswid
.06007 .26004 m
.06007 .26379 L
s
.09824 .26004 m
.09824 .26379 L
s
.13642 .26004 m
.13642 .26379 L
s
.17459 .26004 m
.17459 .26379 L
s
.25093 .26004 m
.25093 .26379 L
s
.2891 .26004 m
.2891 .26379 L
s
.32727 .26004 m
.32727 .26379 L
s
.36545 .26004 m
.36545 .26379 L
s
.44179 .26004 m
.44179 .26379 L
s
.47996 .26004 m
.47996 .26379 L
s
.51813 .26004 m
.51813 .26379 L
s
.5563 .26004 m
.5563 .26379 L
s
.63265 .26004 m
.63265 .26379 L
s
.67082 .26004 m
.67082 .26379 L
s
.70899 .26004 m
.70899 .26379 L
s
.74716 .26004 m
.74716 .26379 L
s
.8235 .26004 m
.8235 .26379 L
s
.86168 .26004 m
.86168 .26379 L
s
.89985 .26004 m
.89985 .26379 L
s
.93802 .26004 m
.93802 .26379 L
s
.25 Mabswid
0 .26004 m
1 .26004 L
s
.0219 .08033 m
.02815 .08033 L
s
[(-1)] .0094 .08033 1 0 Mshowa
.0219 .17018 m
.02815 .17018 L
s
[(-0.5)] .0094 .17018 1 0 Mshowa
.0219 .34989 m
.02815 .34989 L
s
[(0.5)] .0094 .34989 1 0 Mshowa
.0219 .43974 m
.02815 .43974 L
s
[(1)] .0094 .43974 1 0 Mshowa
.0219 .5296 m
.02815 .5296 L
s
[(1.5)] .0094 .5296 1 0 Mshowa
.125 Mabswid
.0219 .0983 m
.02565 .0983 L
s
.0219 .11627 m
.02565 .11627 L
s
.0219 .13424 m
.02565 .13424 L
s
.0219 .15221 m
.02565 .15221 L
s
.0219 .18815 m
.02565 .18815 L
s
.0219 .20612 m
.02565 .20612 L
s
.0219 .2241 m
.02565 .2241 L
s
.0219 .24207 m
.02565 .24207 L
s
.0219 .27801 m
.02565 .27801 L
s
.0219 .29598 m
.02565 .29598 L
s
.0219 .31395 m
.02565 .31395 L
s
.0219 .33192 m
.02565 .33192 L
s
.0219 .36786 m
.02565 .36786 L
s
.0219 .38583 m
.02565 .38583 L
s
.0219 .4038 m
.02565 .4038 L
s
.0219 .42177 m
.02565 .42177 L
s
.0219 .45771 m
.02565 .45771 L
s
.0219 .47569 m
.02565 .47569 L
s
.0219 .49366 m
.02565 .49366 L
s
.0219 .51163 m
.02565 .51163 L
s
.0219 .54757 m
.02565 .54757 L
s
.0219 .56554 m
.02565 .56554 L
s
.0219 .58351 m
.02565 .58351 L
s
.0219 .60148 m
.02565 .60148 L
s
.0219 .06236 m
.02565 .06236 L
s
.0219 .04439 m
.02565 .04439 L
s
.0219 .02642 m
.02565 .02642 L
s
.0219 .00845 m
.02565 .00845 L
s
.25 Mabswid
.0219 0 m
.0219 .61803 L
s
.5 Mabswid
.02381 .60332 m
.04262 .47035 L
.06244 .36029 L
.08255 .27352 L
.10458 .20086 L
.12415 .15163 L
.14509 .11142 L
.16639 .08082 L
.18653 .05938 L
.20651 .04372 L
.22495 .03318 L
.24367 .02554 L
.25432 .02231 L
.2643 .01992 L
.27468 .01799 L
.28455 .01663 L
.2934 .01575 L
.29842 .01539 L
.30307 .01512 L
.30815 .01491 L
.31101 .01483 L
.31237 .0148 L
.31364 .01477 L
.31485 .01475 L
.31595 .01474 L
.31717 .01473 L
.31848 .01472 L
.31974 .01472 L
.32045 .01472 L
.32111 .01472 L
.32231 .01472 L
.32359 .01473 L
.32474 .01474 L
.32579 .01475 L
.32815 .01479 L
.33024 .01484 L
.33244 .01489 L
.3371 .01505 L
.3422 .01526 L
.35248 .01584 L
.36181 .01651 L
.38276 .01842 L
.42274 .02308 L
.46121 .02816 L
.50213 .03365 L
.54153 .03868 L
.57942 .04311 L
.61976 .04728 L
.65858 .05073 L
.69986 .05376 L
Mistroke
.73962 .05607 L
.77787 .05775 L
.79758 .05842 L
.81856 .05898 L
.82887 .05921 L
.84001 .05941 L
.84964 .05955 L
.8602 .05968 L
.86519 .05973 L
.87054 .05977 L
.87516 .0598 L
.88017 .05982 L
.88312 .05983 L
.88584 .05984 L
.88845 .05985 L
.88976 .05985 L
.89047 .05985 L
.89121 .05985 L
.89251 .05985 L
.8937 .05985 L
.89499 .05985 L
.89572 .05985 L
.89639 .05985 L
.89762 .05985 L
.89874 .05985 L
.90004 .05985 L
.90125 .05985 L
.90365 .05984 L
.9059 .05983 L
.911 .05981 L
.91597 .05978 L
.92131 .05975 L
.93096 .05966 L
.93977 .05956 L
.95721 .05929 L
.97619 .05891 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0@@3ooolH00000:T0oooo000K0?ooo`03
0000003oool0oooo03/0oooo1P00000H0?ooo`P00000X@3oool001/0oooo00<000000?ooo`3oool0
>@3oool2000002H0oooo2P00002G0?ooo`006`3oool2000003L0oooo0`00000b0?ooo`P00000S`3o
ool001/0oooo00<000000?ooo`3oool0=03oool2000003d0oooo1P0000290?ooo`006`3oool00`00
0000oooo0?ooo`0a0?ooo`<00000A@3oool700000880oooo000K0?ooo`030000003oool0oooo02l0
oooo0P00001?0?ooo`X00000N03oool001/0oooo00<000000?ooo`3oool0;@3oool2000005/0oooo
2`00001]0?ooo`006`3oool2000002d0oooo00<000000?ooo`3oool0IP3oool:000006<0oooo000K
0?ooo`030000003oool0oooo02X0oooo0P00001c0?ooo`/00000F03oool001/0oooo00<000000?oo
o`3oool0:@3oool00`000000oooo0?ooo`1n0?ooo`X00000CP3oool001/0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`290?ooob@000005@3oool<000000T0oooo000K0?ooo`800000
:03oool00`000000oooo0?ooo`2^0?oooaD000005@3oool001/0oooo00<000000?ooo`3oool09P3o
ool00`000000oooo0?ooo`3I0?ooo`006`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo0=T0oooo000C0?ooo`@00000103oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo
0=X0oooo000E0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0903oool00`000000
oooo0?ooo`3K0?ooo`005@3oool00`000000oooo0?ooo`030?ooo`800000903oool00`000000oooo
0?ooo`3L0?ooo`003@3oool4000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0R0?ooo`030000003oool0oooo0=d0oooo000E0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool08P3oool00`000000oooo0?ooo`3M0?ooo`004`3oool3000000D0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`3N0?ooo`005@3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo0200oooo00<000000?ooo`3oool0g`3oool001/0oooo0P00000P0?ooo`030000
003oool0oooo0>00oooo000K0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0h03o
ool001/0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`3Q0?ooo`006`3oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo0>80oooo000K0?ooo`8000007P3oool00`000000
oooo0?ooo`3R0?ooo`006`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0><0oooo
000K0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0h`3oool001/0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`3T0?ooo`006`3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo0>@0oooo000K0?ooo`8000006`3oool00`000000oooo0?ooo`3U0?ooo`00
6`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0>D0oooo000K0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool0iP3oool001/0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`3V0?ooo`006`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
0>L0oooo000K0?ooo`8000006@3oool00`000000oooo0?ooo`3W0?ooo`006`3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo0>L0oooo000K0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0j03oool000P0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`000005
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0j03oool000L0oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool0j@3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`02
0?ooo`8000005`3oool00`000000oooo0?ooo`3Y0?ooo`0000<0oooo0000000000000P0000020?oo
o`040000003oool0oooo000000P0oooo0`0000050?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool0j@3oool000L0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0jP3oool000L0oooo00@000000?oo
o`3oool00000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0jP3oool000P0oooo0P0000090?ooo`@00000103oool2000001H0oooo00<000000?oo
o`3oool0jP3oool001/0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`3[0?ooo`00
6`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0>/0oooo000K0?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool0k03oool001/0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`3/0?ooo`006`3oool2000001@0oooo00<000000?ooo`3oool0k03oool001/0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3]0?ooo`006`3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo0>d0oooo000K0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool0k@3oool001/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0D
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000008`3oool2000000@0oooo00<00000
0?ooo`3oool00P3oool4000002<0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P00000T
0?ooo`800000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02<0oooo0P000004
0?ooo`030000003oool0oooo0080oooo0`0000030?ooo`006`3oool200000180oooo00<000000?oo
o`3oool04`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0240oooo00@00000
0?ooo`3oool00000203oool00`000000oooo0?ooo`0S0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo
0280oooo00@000000?ooo`3oool000002`3oool100000040oooo0@3oool001/0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0C0?ooo`040000003oool0oooo000000X0oooo00<00000
0?ooo`3oool08@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo0280oooo00@0
00000?ooo`3oool000002`3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo000000L0
oooo1@00000S0?ooo`040000003oool0oooo000000/0oooo0@0000010?ooo`40oooo000K0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`00000:0?oo
o`030000003oool0oooo0240oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0Q
0?ooo`040000003oool0oooo000000T0oooo0P00000S0?ooo`040000003oool0oooo000000L0oooo
00@000000?ooo`3oool00000903oool010000000oooo0?ooo`0000080?ooo`<000000`3oool001/0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo0000
00X0oooo00<000000?ooo`3oool08@3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0
oooo0200oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0P0?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`000000903oool010000000oooo0?ooo`0000080?ooo`03
0000003oool0oooo00<0oooo000K0?ooo`8000004@3oool00`000000oooo0?ooo`0D0?ooo`040000
003oool0oooo000000P0oooo0`00000S0?ooo`040000003oool0oooo000000P0oooo00@000000?oo
o`3oool000008P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000280oooo
00@000000?ooo`3oool000002@3oool2000002@0oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`030?ooo`006`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01D0
oooo0P00000;0?ooo`030000003oool0oooo0280oooo0P00000:0?ooo`800000903oool2000000X0
oooo0P00000T0?ooo`8000002`3oool00`000000oooo0?ooo`0S0?ooo`8000002@3oool400000080
oooo000K0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0l03oool001/0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`3`0?ooo`006`3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo0?00oooo000E0?ooool000002@0000030?ooo`006`3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo00@0oooo00@000000?ooo`3oool000002@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo000K0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo02l0
oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo00L0
oooo000K0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0lP3oool001/0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`3b0?ooo`006`3oool2000000h0oooo00<00000
0?ooo`3oool0lP3oool001/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3c0?oo
o`006`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?<0oooo000K0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0l`3oool001/0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`3c0?ooo`006`3oool2000000`0oooo00<000000?ooo`3oool0m03oool0
01/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3d0?ooo`006`3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo0?@0oooo000K0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0m03oool001/0oooo0P00000;0?ooo`030000003oool0oooo0?D0oooo000K
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0m@3oool001/0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`3e0?ooo`006`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo0?D0oooo000K0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
mP3oool001/0oooo0P00000:0?ooo`030000003oool0oooo0?H0oooo000K0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0mP3oool001/0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`3f0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3g0?ooo`001`3oool010000000oooo
0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`3g0?ooo`001`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0080
oooo0P0000090?ooo`030000003oool0oooo0?L0oooo00070?ooo`040000003oool0oooo000000P0
oooo0`0000050?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0m`3oool000L0oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0n03oool000L0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0n03oool000P0oooo
0P0000090?ooo`@00000103oool2000000P0oooo00<000000?ooo`3oool0n03oool001/0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`3h0?ooo`006`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo0?T0oooo000K0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0n@3oool001/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3i0?ooo`00
6`3oool2000000L0oooo00<000000?ooo`3oool0n@3oool001/0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`3i0?ooo`006`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo0?T0oooo000K0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0nP3oool001/0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3j0?ooo`006`3oool2000000H0oooo
00<000000?ooo`3oool0nP3oool001/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`3j0?ooo`006`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?X0oooo000K0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0nP3oool001/0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`3k0?ooo`006`3oool2000000D0oooo00<000000?ooo`3oool0
n`3oool001/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3k0?ooo`004`3oool4
000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3k0?ooo`005@3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0n`3oool001D0
oooo00<000000?ooo`3oool00`3oool2000000@0oooo00<000000?ooo`3oool0o03oool001D0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?`0oooo
000E0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`3l0?ooo`004`3oool3000000D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3l
0?ooo`005@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0o03oool001/0oooo0P0000040?ooo`030000003oool0oooo0?`0oooo000K0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool001/0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`3m0?ooo`006`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0?d0oooo000K0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o@3oool001/0
oooo0P0000030?ooo`030000003oool0oooo0?d0oooo000K0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0o@3oool001/0oooo00D000000?ooo`3oool0oooo0000003o0?ooo`40oooo
000K0?ooo`050000003oool0oooo0?ooo`000000o`3oool10?ooo`006`3oool01@000000oooo0?oo
o`3oool000000?l0oooo0@3oool001/0oooo0P0000020?ooo`030000003oool0oooo0?h0oooo000K
0?ooo`050000003oool0oooo0?ooo`000000o`3oool10?ooo`006`3oool01@000000oooo0?ooo`3o
ool000000?l0oooo0@3oool001/0oooo00D000000?ooo`3oool0oooo0000003o0?ooo`40oooo000K
0?ooo`80000000<0oooo0000003oool0o`3oool10?ooo`006`3oool010000000oooo0?ooo`00003o
0?ooo`80oooo000K0?ooo`040000003oool0oooo00000?l0oooo0P3oool000L0oooo100000030?oo
o`030000003oool0oooo0080oooo0`0000050?ooo`040000003oool0oooo00000?l0oooo0P3oool0
00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
00000?l0oooo0P3oool000T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?oo
o`80000000<0oooo0000003oool0o`3oool10?ooo`002@3oool00`000000oooo0?ooo`070?ooo`<0
00001@3oool010000000oooo0?ooo`00003o0?ooo`80oooo00090?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0000003o0?ooo`<0oooo00070?ooo`<00000
2@3oool00`000000oooo0?ooo`050?ooo`030000003oool000000?l0oooo0`3oool000T0oooo00<0
00000?ooo`3oool01`3oool4000000@0oooo00<000000?ooo`000000o`3oool30?ooo`006`3oool3
00000?l0oooo0`3oool001/0oooo00<000000?ooo`000000o`3oool30?ooo`006`3oool00`000000
oooo0000003o0?ooo`<0oooo000K0?ooo`030000003oool000000?l0oooo0`3oool001/0oooo0P00
003o0?ooo`@0oooo000K0?ooo`800000o`3oool40?ooo`006`3oool200000?l0oooo103oool001/0
oooo0P00003o0?ooo`@0oooo000K0?ooo`800000o`3oool40?ooo`006`3oool200000?l0oooo103o
ool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0
oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo
000K0?ooo`800000o`3oool40?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?oo
o`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
6`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0544158, -1.60128, \
0.00199431, 0.0211806}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(omdamp2[3/10, Pi/16, 1]\)], "Input"],

Cell[BoxData[
    \(0.03368776133206933`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      testfuncspec[IntegerPart[om*10^16]*10^\((\(-16\))\)*I, 1/10, Pi/16, 
        1], {om, 0.001, 0.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.90476 0.302964 0.0481058 [
[.21429 .29046 -9 -9 ]
[.21429 .29046 9 0 ]
[.40476 .29046 -9 -9 ]
[.40476 .29046 9 0 ]
[.59524 .29046 -9 -9 ]
[.59524 .29046 9 0 ]
[.78571 .29046 -9 -9 ]
[.78571 .29046 9 0 ]
[.97619 .29046 -9 -9 ]
[.97619 .29046 9 0 ]
[.01131 .01433 -12 -4.5 ]
[.01131 .01433 0 4.5 ]
[.01131 .11054 -12 -4.5 ]
[.01131 .11054 0 4.5 ]
[.01131 .20675 -12 -4.5 ]
[.01131 .20675 0 4.5 ]
[.01131 .39918 -6 -4.5 ]
[.01131 .39918 0 4.5 ]
[.01131 .49539 -6 -4.5 ]
[.01131 .49539 0 4.5 ]
[.01131 .5916 -6 -4.5 ]
[.01131 .5916 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .30296 m
.21429 .30921 L
s
[(0.1)] .21429 .29046 0 1 Mshowa
.40476 .30296 m
.40476 .30921 L
s
[(0.2)] .40476 .29046 0 1 Mshowa
.59524 .30296 m
.59524 .30921 L
s
[(0.3)] .59524 .29046 0 1 Mshowa
.78571 .30296 m
.78571 .30921 L
s
[(0.4)] .78571 .29046 0 1 Mshowa
.97619 .30296 m
.97619 .30921 L
s
[(0.5)] .97619 .29046 0 1 Mshowa
.125 Mabswid
.0619 .30296 m
.0619 .30671 L
s
.1 .30296 m
.1 .30671 L
s
.1381 .30296 m
.1381 .30671 L
s
.17619 .30296 m
.17619 .30671 L
s
.25238 .30296 m
.25238 .30671 L
s
.29048 .30296 m
.29048 .30671 L
s
.32857 .30296 m
.32857 .30671 L
s
.36667 .30296 m
.36667 .30671 L
s
.44286 .30296 m
.44286 .30671 L
s
.48095 .30296 m
.48095 .30671 L
s
.51905 .30296 m
.51905 .30671 L
s
.55714 .30296 m
.55714 .30671 L
s
.63333 .30296 m
.63333 .30671 L
s
.67143 .30296 m
.67143 .30671 L
s
.70952 .30296 m
.70952 .30671 L
s
.74762 .30296 m
.74762 .30671 L
s
.82381 .30296 m
.82381 .30671 L
s
.8619 .30296 m
.8619 .30671 L
s
.9 .30296 m
.9 .30671 L
s
.9381 .30296 m
.9381 .30671 L
s
.25 Mabswid
0 .30296 m
1 .30296 L
s
.02381 .01433 m
.03006 .01433 L
s
[(-6)] .01131 .01433 1 0 Mshowa
.02381 .11054 m
.03006 .11054 L
s
[(-4)] .01131 .11054 1 0 Mshowa
.02381 .20675 m
.03006 .20675 L
s
[(-2)] .01131 .20675 1 0 Mshowa
.02381 .39918 m
.03006 .39918 L
s
[(2)] .01131 .39918 1 0 Mshowa
.02381 .49539 m
.03006 .49539 L
s
[(4)] .01131 .49539 1 0 Mshowa
.02381 .5916 m
.03006 .5916 L
s
[(6)] .01131 .5916 1 0 Mshowa
.125 Mabswid
.02381 .03838 m
.02756 .03838 L
s
.02381 .06244 m
.02756 .06244 L
s
.02381 .08649 m
.02756 .08649 L
s
.02381 .13459 m
.02756 .13459 L
s
.02381 .15865 m
.02756 .15865 L
s
.02381 .1827 m
.02756 .1827 L
s
.02381 .23081 m
.02756 .23081 L
s
.02381 .25486 m
.02756 .25486 L
s
.02381 .27891 m
.02756 .27891 L
s
.02381 .32702 m
.02756 .32702 L
s
.02381 .35107 m
.02756 .35107 L
s
.02381 .37512 m
.02756 .37512 L
s
.02381 .42323 m
.02756 .42323 L
s
.02381 .44728 m
.02756 .44728 L
s
.02381 .47133 m
.02756 .47133 L
s
.02381 .51944 m
.02756 .51944 L
s
.02381 .54349 m
.02756 .54349 L
s
.02381 .56755 m
.02756 .56755 L
s
.02381 .61565 m
.02756 .61565 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.04766 .61803 m
.04943 .57135 L
.05473 .4561 L
.06427 .29896 L
.0697 .2325 L
.07559 .17506 L
.08078 .13489 L
.08629 .10114 L
.09125 .07722 L
.09671 .05677 L
.10167 .04264 L
.10632 .03263 L
.11128 .02486 L
.11408 .02159 L
.11668 .01921 L
.11918 .01745 L
.12145 .01626 L
.12271 .01576 L
.12406 .01533 L
.12535 .01503 L
.12653 .01485 L
.1276 .01475 L
.12875 .01472 L
.12983 .01474 L
.13082 .01482 L
.13197 .01497 L
.13323 .01521 L
.13552 .01581 L
.13791 .01666 L
.14045 .01779 L
.14502 .02033 L
.15437 .02716 L
.16445 .0362 L
.18269 .05476 L
.22375 .09697 L
.2633 .1313 L
.30133 .15735 L
.34181 .17875 L
.38077 .19458 L
.42219 .20754 L
.46209 .21722 L
.50048 .22456 L
.54132 .23075 L
.58064 .23548 L
.61845 .23913 L
.65871 .24225 L
.69746 .24464 L
.73865 .24664 L
.77833 .24814 L
.8165 .24925 L
Mistroke
.83782 .24975 L
.85712 .25012 L
.89622 .25071 L
.91547 .25092 L
.92539 .251 L
.93626 .25108 L
.94564 .25114 L
.95569 .25119 L
.96124 .25121 L
.9663 .25123 L
.97619 .25126 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000P0oooo0P0000050?ooo`030000003oool0
oooo0?l0oooo3`3oool000L0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3o
0?ooo`l0oooo00070?ooo`040000003oool0oooo000000@0oooo0`00000H0?ooo`D00000l@3oool0
00030?ooo`0000000000008000000P3oool3000000D0oooo00<000000?ooo`3oool05`3oool20000
00<0oooo0P00003`0?ooo`001`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3]0?ooo`001`3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool01`3oool200000>d0oooo
00080?ooo`<00000103oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0jP3oool000l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0;
0?ooo`800000jP3oool000l0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo0>L0oooo000?0?ooo`800000503oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo0>H0oooo000?0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`3U0?ooo`003`3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool0i03oool000l0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0><0oooo000?0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3R0?ooo`003`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0h@3oool000l0oooo
0P00000B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0h03oool000l0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0=l0oooo000?
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3N
0?ooo`003`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool0g@3oool000l0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0=d0oooo000?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
703oool00`000000oooo0?ooo`3L0?ooo`003`3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool0f`3oool000l0oooo0P00000@0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool0fP3oool000l0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo0=T0oooo000?0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`3H0?ooo`003`3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0e`3oool000l0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0=H0oooo0009
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0U
0?ooo`800000eP3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool0d`3oool000H0oooo1@0000040?ooo`<0
00003P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0=80oooo00000`3oool00000
00000002000000050?ooo`000000oooo0?ooo`0000001@3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo02T0oooo00<000000?ooo`3oool0d@3oool000L0oooo00<000000?ooo`000000
1@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0
d03oool000P0oooo0P0000050?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0;03o
ool200000=00oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0<d0oooo000?0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`3<0?ooo`003`3oool2000000h0
oooo00<000000?ooo`3oool0<03oool200000<`0oooo000?0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`390?ooo`003`3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0b03oool000l0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0e0?ooo`800000b03oool000l0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0<D0oooo000?0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0>03oool200000<D0oooo000?0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`320?ooo`003`3o
ool2000000d0oooo00<000000?ooo`3oool0>`3oool200000<80oooo000?0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0?@3oool200000<00oooo000?0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0@03oool200000;h0oooo000?0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool0@P3oool200000;`0oooo000?0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0A03oool200000;X0oooo000?0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0AP3oool200000;P0oooo000?0?ooo`800000303oool00`000000oooo0?ooo`18
0?ooo`800000]P3oool000l0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`1:0?oo
o`800000]03oool000l0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`1<0?ooo`80
0000/P3oool000l0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`1>0?ooo`800000
/03oool000l0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`1@0?ooo`<00000[@3o
ool000L0oooo100000040?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0D`3oool4
00000:T0oooo00070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`1G0?ooo`@00000Y@3oool000P0oooo00<000000?ooo`3oool0103oool3000000X0
oooo00<000000?ooo`3oool0G03oool400000:40oooo00000`3oool0000000000002000000@0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0600oooo
1000002M0?ooo`002P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool0I03oool4000009T0oooo00070?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`1X0?ooo`@00000U@3oool000P0oooo0P000005
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0K03oool500000900oooo000?0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0L@3oool5000008/0oooo000?0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0MP3oool6000008D0oooo000?0?ooo`800000
2`3oool00`000000oooo0?ooo`1l0?ooo`P00000O@3oool000l0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`240?ooo`/00000LP3oool000l0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`2?0?ooo`/00000I`3oool000l0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`2J0?ooo`/00000G03oool000l0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`2U0?oooaH00000AP3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`2l0?ooocd000002@3oool000l0oooo0P00000:0?ooo`030000003oool0oooo0?l0oooo0`3o
ool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3oool30?ooo`003`3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo01h0oooo0P0000040?ooo`030000003oool0oooo0080
oooo1000000V0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@000009@3oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000002L0oooo0P0000040?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool09@3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000<0
oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool07@3oool010000000oooo
0?ooo`00000:0?ooo`030000003oool0oooo02@0oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`0U0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000009@3o
ool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo02@0oooo00@000000?ooo`3oool0
00002`3oool100000040oooo0@3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`0M0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0903oool010000000
oooo0?ooo`0000090?ooo`030000003oool0oooo02@0oooo00@000000?ooo`3oool000002`3oool0
0`000000oooo0?ooo`0S0?ooo`040000003oool0oooo000000L0oooo1@00000U0?ooo`040000003o
ool0oooo000000/0oooo0@0000010?ooo`40oooo000?0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool07@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo02@0oooo
00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0S0?ooo`040000003oool0oooo0000
00T0oooo0P00000V0?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool000009P3o
ool010000000oooo0?ooo`0000080?ooo`<000000`3oool000l0oooo0P00000:0?ooo`030000003o
ool0oooo01d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0T0?ooo`040000
003oool0oooo000000/0oooo00<000000?ooo`3oool08P3oool010000000oooo0?ooo`00000;0?oo
o`030000003oool0oooo02<0oooo00@000000?ooo`3oool00000203oool00`000000oooo0000000V
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool00`3oool000l0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0M0?ooo`040000003oool0oooo000000P0oooo0`00
000V0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000903oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000002D0oooo00@000000?ooo`3oool000002@3o
ool2000002H0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`030?ooo`003`3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo01l0oooo0P00000;0?ooo`030000003o
ool0oooo02D0oooo0P00000:0?ooo`8000009P3oool2000000X0oooo0P00000W0?ooo`8000002`3o
ool00`000000oooo0?ooo`0U0?ooo`8000002@3oool400000080oooo000?0?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool0o`3oool40?ooo`003`3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0?l0oooo103oool000l0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`3o0?ooo`@0oooo000?0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
o`3oool40?ooo`00203ooooo000001H000000`3oool000l0oooo00<000000?ooo`3oool01`3oool2
000000T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00L0oooo000?0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0o`3oool40?oo
o`003`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?l0oooo103oool000l0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3o0?ooo`@0oooo000?0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0o`3oool40?ooo`003`3oool2000000T0oooo00<00000
0?ooo`3oool0o`3oool40?ooo`003`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
0?l0oooo1@3oool000l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3o0?ooo`D0
oooo000?0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0o`3oool50?ooo`003`3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo1@3oool000l0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`3o0?ooo`D0oooo000?0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0o`3oool50?ooo`003`3oool2000000P0oooo00<000000?ooo`3o
ool0o`3oool50?ooo`003`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo
1@3oool000l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3o0?ooo`D0oooo000?
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0o`3oool50?ooo`003`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo1@3oool000l0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`3o0?ooo`D0oooo000?0?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0o`3oool50?ooo`003`3oool2000000P0oooo00<000000?ooo`3oool0o`3o
ool50?ooo`003`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo1P3oool0
00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooo`H0oooo000?0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3oool60?ooo`001`3oool4000000@0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooo`H0oooo00070?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooo`H0oooo0008
0?ooo`030000003oool0oooo00@0oooo0`0000060?ooo`030000003oool0oooo0?l0oooo1P3oool0
00T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
0?l0oooo1P3oool000X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo0?l0oooo1P3oool000L0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo1P3oool000P0oooo0P0000050?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0o`3oool60?ooo`003`3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo0?l0oooo1P3oool000l0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`3o0?ooo`H0oooo000?0?ooo`8000001`3oool00`000000oooo0?ooo`3o0?oo
o`H0oooo000?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3oool60?ooo`00
3`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?l0oooo1`3oool000l0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo000?0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0o`3oool70?ooo`003`3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0?l0oooo1`3oool000l0oooo0P0000060?ooo`030000003oool0oooo0?l0
oooo1`3oool000l0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo
000?0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool70?ooo`003`3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0?l0oooo1`3oool000l0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo000?0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0o`3oool70?ooo`003`3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo0?l0oooo1`3oool000l0oooo0P0000060?ooo`030000003oool0oooo0?l0oooo1`3o
ool000l0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo000?0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool70?ooo`003`3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo0?l0oooo1`3oool000l0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00090?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00090?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00060?oo
o`D00000103oool3000000D0oooo00<000000?ooo`3oool0o`3oool70?ooo`001P3oool010000000
oooo0?ooo`0000050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool70?oo
o`001`3oool00`000000oooo000000050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0o`3oool70?ooo`00203oool2000000D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`3o0?ooo`L0oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`3o0?ooo`L0oooo000?0?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0o`3oool70?ooo`003`3oool2000000D0oooo00<000000?ooo`3oool0o`3oool80?oo
o`003`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool000l0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo000?0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0o`3oool80?ooo`003`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0?l0oooo203oool000l0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`3o0?ooo`P0oooo000?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0o`3oool80?ooo`003`3oool2000000D0oooo00<000000?ooo`3oool0o`3oool80?ooo`003`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool000l0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo000?0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0o`3oool80?ooo`003`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?l0oooo203oool000l0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`3o0?ooo`P0oooo000?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3o
ool80?ooo`003`3oool2000000D0oooo00<000000?ooo`3oool0o`3oool80?ooo`003`3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool000l0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo000?0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0o`3oool80?ooo`00203oool2000000D0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`3o0?ooo`P0oooo00070?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo00070?ooo`040000003oool0
oooo000000@0oooo0`0000040?ooo`030000003oool0oooo0?l0oooo203oool000L0oooo0`000005
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool80?ooo`001`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool80?oo
o`001`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0o`3oool80?ooo`00203oool3000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`3o0?ooo`P0oooo000?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3o
ool80?ooo`003`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool0
00l0oooo0P0000050?ooo`030000003oool0oooo0?l0oooo203oool00?l0oooo8@3oool00?l0oooo
8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0294174, -6.56589, \
0.00190439, 0.0754047}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, PlotRange \[Rule] All]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0219009 1.90858 0.130208 0.0192746 [
[.21276 .11771 -9 -9 ]
[.21276 .11771 9 0 ]
[.40362 .11771 -9 -9 ]
[.40362 .11771 9 0 ]
[.59447 .11771 -9 -9 ]
[.59447 .11771 9 0 ]
[.78533 .11771 -9 -9 ]
[.78533 .11771 9 0 ]
[.97619 .11771 -9 -9 ]
[.97619 .11771 9 0 ]
[.0094 .03384 -12 -4.5 ]
[.0094 .03384 0 4.5 ]
[.0094 .22658 -6 -4.5 ]
[.0094 .22658 0 4.5 ]
[.0094 .32295 -12 -4.5 ]
[.0094 .32295 0 4.5 ]
[.0094 .41933 -12 -4.5 ]
[.0094 .41933 0 4.5 ]
[.0094 .5157 -12 -4.5 ]
[.0094 .5157 0 4.5 ]
[.0094 .61207 -12 -4.5 ]
[.0094 .61207 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21276 .13021 m
.21276 .13646 L
s
[(0.1)] .21276 .11771 0 1 Mshowa
.40362 .13021 m
.40362 .13646 L
s
[(0.2)] .40362 .11771 0 1 Mshowa
.59447 .13021 m
.59447 .13646 L
s
[(0.3)] .59447 .11771 0 1 Mshowa
.78533 .13021 m
.78533 .13646 L
s
[(0.4)] .78533 .11771 0 1 Mshowa
.97619 .13021 m
.97619 .13646 L
s
[(0.5)] .97619 .11771 0 1 Mshowa
.125 Mabswid
.06007 .13021 m
.06007 .13396 L
s
.09824 .13021 m
.09824 .13396 L
s
.13642 .13021 m
.13642 .13396 L
s
.17459 .13021 m
.17459 .13396 L
s
.25093 .13021 m
.25093 .13396 L
s
.2891 .13021 m
.2891 .13396 L
s
.32727 .13021 m
.32727 .13396 L
s
.36545 .13021 m
.36545 .13396 L
s
.44179 .13021 m
.44179 .13396 L
s
.47996 .13021 m
.47996 .13396 L
s
.51813 .13021 m
.51813 .13396 L
s
.5563 .13021 m
.5563 .13396 L
s
.63265 .13021 m
.63265 .13396 L
s
.67082 .13021 m
.67082 .13396 L
s
.70899 .13021 m
.70899 .13396 L
s
.74716 .13021 m
.74716 .13396 L
s
.8235 .13021 m
.8235 .13396 L
s
.86168 .13021 m
.86168 .13396 L
s
.89985 .13021 m
.89985 .13396 L
s
.93802 .13021 m
.93802 .13396 L
s
.25 Mabswid
0 .13021 m
1 .13021 L
s
.0219 .03384 m
.02815 .03384 L
s
[(-5)] .0094 .03384 1 0 Mshowa
.0219 .22658 m
.02815 .22658 L
s
[(5)] .0094 .22658 1 0 Mshowa
.0219 .32295 m
.02815 .32295 L
s
[(10)] .0094 .32295 1 0 Mshowa
.0219 .41933 m
.02815 .41933 L
s
[(15)] .0094 .41933 1 0 Mshowa
.0219 .5157 m
.02815 .5157 L
s
[(20)] .0094 .5157 1 0 Mshowa
.0219 .61207 m
.02815 .61207 L
s
[(25)] .0094 .61207 1 0 Mshowa
.125 Mabswid
.0219 .05311 m
.02565 .05311 L
s
.0219 .07238 m
.02565 .07238 L
s
.0219 .09166 m
.02565 .09166 L
s
.0219 .11093 m
.02565 .11093 L
s
.0219 .14948 m
.02565 .14948 L
s
.0219 .16876 m
.02565 .16876 L
s
.0219 .18803 m
.02565 .18803 L
s
.0219 .20731 m
.02565 .20731 L
s
.0219 .24586 m
.02565 .24586 L
s
.0219 .26513 m
.02565 .26513 L
s
.0219 .2844 m
.02565 .2844 L
s
.0219 .30368 m
.02565 .30368 L
s
.0219 .34223 m
.02565 .34223 L
s
.0219 .3615 m
.02565 .3615 L
s
.0219 .38078 m
.02565 .38078 L
s
.0219 .40005 m
.02565 .40005 L
s
.0219 .4386 m
.02565 .4386 L
s
.0219 .45788 m
.02565 .45788 L
s
.0219 .47715 m
.02565 .47715 L
s
.0219 .49642 m
.02565 .49642 L
s
.0219 .53497 m
.02565 .53497 L
s
.0219 .55425 m
.02565 .55425 L
s
.0219 .57352 m
.02565 .57352 L
s
.0219 .5928 m
.02565 .5928 L
s
.0219 .01456 m
.02565 .01456 L
s
.25 Mabswid
.0219 0 m
.0219 .61803 L
s
.5 Mabswid
.02381 .60332 m
.03279 .42694 L
.04262 .29005 L
.04758 .23774 L
.05288 .19157 L
.06244 .1286 L
.06788 .10198 L
.07379 .07896 L
.07899 .06287 L
.08451 .04934 L
.08947 .03976 L
.09494 .03156 L
.09992 .0259 L
.10458 .02189 L
.10954 .01878 L
.11235 .01747 L
.11496 .01652 L
.11746 .01581 L
.11974 .01533 L
.121 .01513 L
.12236 .01496 L
.12364 .01484 L
.12483 .01477 L
.1259 .01473 L
.12705 .01472 L
.12813 .01473 L
.12912 .01476 L
.13028 .01482 L
.13154 .01491 L
.13383 .01515 L
.13623 .01549 L
.13878 .01595 L
.14335 .01697 L
.15272 .0197 L
.16282 .02332 L
.1811 .03076 L
.22224 .04767 L
.26187 .06143 L
.29997 .07186 L
.34054 .08044 L
.37958 .08678 L
.42108 .09198 L
.46106 .09585 L
.49953 .0988 L
.54045 .10127 L
.57985 .10317 L
.61774 .10463 L
.65808 .10588 L
.6969 .10684 L
.73818 .10764 L
Mistroke
.77794 .10824 L
.81618 .10869 L
.83754 .10888 L
.85688 .10904 L
.89606 .10927 L
.91535 .10935 L
.92529 .10939 L
.93618 .10942 L
.94558 .10944 L
.95565 .10946 L
.96121 .10947 L
.96628 .10948 L
.97619 .10949 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000
003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`8000005`3oool:00000>h0oooo000?0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool0203oool300000>/0oooo000?0?ooo`030000
003oool0oooo01<0oooo0P00000>0?ooo`<00000j03oool000L0oooo0`0000050?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool04@3oool200000>H0oooo000:0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0D0?ooo`<00000h`3oool0
00X0oooo00<000000?ooo`3oool00P3oool300000140oooo00<000000?ooo`3oool05`3oool20000
0>40oooo00000`3oool000000000000200000080oooo0`0000050?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool06P3oool200000=l0oooo00070?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0L0?ooo`800000g@3oool000L0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01l0oooo0P00
003K0?ooo`001`3oool4000000@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0R
0?ooo`<00000f03oool000l0oooo0P00000?0?ooo`030000003oool0oooo02D0oooo1000003D0?oo
o`003`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo02T0oooo1000003@0?ooo`00
3`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02h0oooo0`00003=0?ooo`003`3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0340oooo0`00003:0?ooo`003`3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo03@0oooo100000360?ooo`003`3oool20000
00d0oooo00<000000?ooo`3oool0>@3oool300000<<0oooo000?0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0?03oool400000;l0oooo000?0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0@@3oool400000;/0oooo000?0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0A@3oool700000;@0oooo000?0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0C03oool800000:`0oooo000?0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0E03oool600000:H0oooo000?0?ooo`8000002`3oool00`000000oooo0?ooo`0M0?ooo`80
0000103oool00`000000oooo0?ooo`020?ooo`@000009P3oool2000000@0oooo00<000000?ooo`3o
ool02@00000R0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000009`3oool2000000@0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0U0?ooo`800000103oool00`000000
oooo0?ooo`020?ooo`<000000`3oool000l0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0L0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0903oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo00@0oooo2P00000G0?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool000009@3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo02@0oooo00@000000?ooo`3oool000002`3oool100000040oooo0@3oool000l0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0oooo000000X0
oooo00<000000?ooo`3oool0903oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo
00d0oooo5P0000001@3oool000000?ooo`3oool0000000/0oooo00<000000?ooo`3oool08`3oool0
10000000oooo0?ooo`0000070?ooo`D000009@3oool010000000oooo0?ooo`00000;0?ooo`400000
0@3oool10?ooo`003`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01`0oooo00@0
00000?ooo`3oool000002P3oool00`000000oooo0?ooo`0T0?ooo`040000003oool0oooo000000X0
oooo00<000000?ooo`3oool08P3ooolF00000200oooo00@000000?ooo`3oool000001`3oool01000
0000oooo0?ooo`00000V0?ooo`040000003oool0oooo000000P0oooo0`0000030?ooo`003`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01`0oooo00@000000?ooo`3oool000002P3o
ool00`000000oooo0?ooo`0T0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0
8P3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo00<0oooo<`00000R0?ooo`04
0000003oool0oooo000000P0oooo00<000000?ooo`3oool00`3oool000l0oooo0P00000:0?ooo`03
0000003oool0oooo01d0oooo00@000000?ooo`3oool00000203oool3000002H0oooo00@000000?oo
o`3oool00000203oool010000000oooo0?ooo`00000T0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000009@3oool010000000oooo0?ooo`0000090?ooo`800000103oool[0000
00<0oooo00<000000?ooo`3oool00`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`0N0?ooo`8000002`3oool00`000000oooo0?ooo`0U0?ooo`8000002P3oool2000002H0
oooo0P00000:0?ooo`8000009`3oool2000000/0oooo00<000000?ooo`3oool09@3oool2000000T0
oooo100000020?ooo`003`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooo
0`3oool000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000?
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0o`3oool40?ooo`002@3ooooo0000
01D000000`3oool000l0oooo00<000000?ooo`3oool01`3oool2000000T0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo000?0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0b0?ooo`030000
003oool0oooo0340oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0b0?ooo`030000
003oool0oooo00L0oooo000?0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0o`3o
ool50?ooo`003`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo1@3oool0
00l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3o0?ooo`D0oooo000?0?ooo`80
0000203oool00`000000oooo0?ooo`3o0?ooo`D0oooo000?0?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0o`3oool50?ooo`003`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo0?l0oooo1@3oool000l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o
0?ooo`H0oooo000?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3oool60?oo
o`003`3oool2000000L0oooo00<000000?ooo`3oool0o`3oool60?ooo`003`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0?l0oooo1P3oool000l0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`3o0?ooo`H0oooo000?0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0o`3oool60?ooo`003`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
0?l0oooo1`3oool000l0oooo0P0000060?ooo`030000003oool0oooo0?l0oooo1`3oool000l0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo000?0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0o`3oool70?ooo`003`3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo0?l0oooo1`3oool000l0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`3o0?ooo`L0oooo000?0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0o`3oool70?ooo`003`3oool2000000H0oooo00<000000?ooo`3oool0o`3oool70?ooo`003`3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?l0oooo1`3oool000l0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo00070?ooo`<000001@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool000X0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool000X0oooo
00<000000?ooo`3oool00P3oool3000000@0oooo00<000000?ooo`3oool0o`3oool80?ooo`001`3o
ool3000000D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo0007
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o
0?ooo`P0oooo00070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`3o0?ooo`P0oooo00070?ooo`@00000103oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?l0oooo203oool000l0oooo0P0000050?ooo`030000003oool0oooo0?l0oooo
203oool000l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo000?
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool80?ooo`003`3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo203oool000l0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo000?0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0o`3oool80?ooo`003`3oool2000000@0oooo00<000000?ooo`3oool0o`3o
ool90?ooo`003`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo2@3oool0
00l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000?0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool90?ooo`003`3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0?l0oooo2@3oool000l0oooo0P0000040?ooo`030000003o
ool0oooo0?l0oooo2@3oool000l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o
0?ooo`T0oooo000?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool90?oo
o`003`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo2@3oool000l0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000?0?ooo`800000103o
ool00`000000oooo0?ooo`3o0?ooo`T0oooo000?0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0o`3oool90?ooo`003`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
0?l0oooo2@3oool000030?ooo`0000000000008000000`3oool2000000D0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00030?ooo`050000003oool0oooo0?ooo`00
00000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0o`3oool:0?ooo`000`3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3o
ool00P3oool300000080oooo00<000000?ooo`3oool0o`3oool:0?ooo`000`3oool01@000000oooo
0?ooo`3oool000000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0?l0oooo2P3oool000<0oooo00D000000?ooo`3oool0oooo000000020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooo`X0
oooo0000103oool000000000000000030?ooo`040000003oool0oooo000000@0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo00030?ooo`030000003oool0oooo0080
oooo0P0000050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool:0?ooo`00
3`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0oooo2P3oool000l0oooo0P00
00030?ooo`030000003oool0oooo0?l0oooo2P3oool000l0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`3o0?ooo`X0oooo000?0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0o`3oool:0?ooo`003`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0
oooo2P3oool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooo`X0oooo
000?0?ooo`8000000P3oool00`000000oooo0?ooo`3o0?ooo`/0oooo000?0?ooo`050000003oool0
oooo0?ooo`000000o`3oool=0?ooo`003`3oool01@000000oooo0?ooo`3oool000000?l0oooo3@3o
ool000l0oooo00D000000?ooo`3oool0oooo0000003o0?ooo`d0oooo000?0?ooo`050000003oool0
oooo0?ooo`000000o`3oool=0?ooo`003`3oool200000080oooo00<000000?ooo`3oool0o`3oool;
0?ooo`003`3oool01@000000oooo0?ooo`3oool000000?l0oooo3@3oool000l0oooo00D000000?oo
o`3oool0oooo0000003o0?ooo`d0oooo000?0?ooo`050000003oool0oooo0?ooo`000000o`3oool=
0?ooo`003`3oool01@000000oooo0?ooo`3oool000000?l0oooo3@3oool000l0oooo00D000000?oo
o`3oool0oooo0000003o0?ooo`d0oooo000?0?ooo`8000000P3oool00`000000oooo0?ooo`3o0?oo
o`/0oooo000?0?ooo`040000003oool0oooo00000?l0oooo3P3oool000l0oooo00@000000?ooo`3o
ool00000o`3oool>0?ooo`0000<0oooo0000000000000P0000020?ooo`<000001@3oool010000000
oooo0?ooo`00003o0?ooo`h0oooo00030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool00P3oool010000000oooo0?ooo`00003o0?ooo`h0oooo00030?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool00P3oool400000?l0oooo3P3oool000<0oooo00@000000?ooo`3oool0
oooo0`0000050?ooo`040000003oool0oooo00000?l0oooo3P3oool000<0oooo00D000000?ooo`3o
ool0oooo000000070?ooo`040000003oool0oooo00000?l0oooo3P3oool000040?ooo`0000000000
000000<0oooo00<000000?ooo`3oool01@3oool010000000oooo0?ooo`00003o0?ooo`h0oooo0003
0?ooo`040000003oool0oooo0?ooo`@00000103oool010000000oooo0?ooo`00003o0?ooo`h0oooo
000?0?ooo`80000000<0oooo0000003oool0o`3oool=0?ooo`003`3oool010000000oooo0?ooo`00
003o0?ooo`h0oooo000?0?ooo`040000003oool0oooo00000?l0oooo3P3oool000l0oooo00@00000
0?ooo`3oool00000o`3oool>0?ooo`003`3oool010000000oooo0?ooo`00003o0?ooo`h0oooo000?
0?ooo`040000003oool0oooo00000?l0oooo3P3oool000l0oooo0`00003o0?ooo`l0oooo000?0?oo
o`030000003oool000000?l0oooo3`3oool000l0oooo00<000000?ooo`000000o`3oool?0?ooo`00
3`3oool00`000000oooo0000003o0?ooo`l0oooo000?0?ooo`030000003oool000000?l0oooo3`3o
ool000l0oooo0`00003o0?ooo`l0oooo000?0?ooo`030000003oool000000?l0oooo3`3oool000l0
oooo00<000000?ooo`000000o`3oool?0?ooo`003`3oool00`000000oooo0000003o0?ooo`l0oooo
000?0?ooo`030000003oool000000?l0oooo3`3oool000l0oooo0`00003o0?ooo`l0oooo000?0?oo
o`030000003oool000000?l0oooo3`3oool000l0oooo00<000000?ooo`000000o`3oool?0?ooo`00
00<0oooo0000000000000P0000030?ooo`8000001@3oool00`000000oooo0000003o0?ooo`l0oooo
00000`3oool000000?ooo`040?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`000000
o`3oool?0?ooo`000P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000@0oooo
0`00003o0?ooo`l0oooo00030?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo
0?ooo`020?ooo`800000o`3oool@0?ooo`00103oool010000000oooo0?ooo`0000020?ooo`030000
003oool0oooo0080oooo0P00003o0?oooa00oooo00001@3oool000000?ooo`3oool000000080oooo
00@000000?ooo`3oool00000103oool200000?l0oooo403oool00080oooo0P0000040?ooo`800000
1@3oool200000?l0oooo403oool000l0oooo0P00003o0?oooa00oooo000?0?ooo`800000o`3oool@
0?ooo`003`3oool200000?l0oooo403oool000l0oooo0P00003o0?oooa00oooo000?0?ooo`800000
o`3oool@0?ooo`003`3oool200000?l0oooo403oool000l0oooo0P00003o0?oooa00oooo000?0?oo
o`800000o`3oool@0?ooo`003`3oool200000?l0oooo403oool000l0oooo0P00003o0?oooa00oooo
000?0?ooo`800000o`3oool@0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?oo
o`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000
003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3oool@0?ooo`003`3oool00`000000
oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000030?ooo`00
00000000008000000P3oool3000000D0oooo00<000000?ooo`3oool0o`3oool?0?ooo`0000<0oooo
0000003oool01`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0oooo3`3oool0
0080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`<00000o`3oool?0?oo
o`000`3oool010000000oooo0?ooo`3oool3000000D0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`00103oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo0?l0oooo3`3oool00005
0?ooo`000000oooo0?ooo`0000000P3oool00`000000oooo0?ooo`3o0?oooaL0oooo00020?ooo`80
00000`3oool400000?l0oooo5P3oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0294045, -7.13594, \
0.00190434, 0.188569}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      testfuncspec[IntegerPart[om*10^16]*10^\((\(-16\))\)*I, 1/10, Pi/16, 
        1], {om, 0.5, 10}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 0.603319 0.00585407 [
[.21429 .59082 -3 -9 ]
[.21429 .59082 3 0 ]
[.40476 .59082 -3 -9 ]
[.40476 .59082 3 0 ]
[.59524 .59082 -3 -9 ]
[.59524 .59082 3 0 ]
[.78571 .59082 -3 -9 ]
[.78571 .59082 3 0 ]
[.97619 .59082 -6 -9 ]
[.97619 .59082 6 0 ]
[.01131 .01791 -24 -4.5 ]
[.01131 .01791 0 4.5 ]
[.01131 .13499 -18 -4.5 ]
[.01131 .13499 0 4.5 ]
[.01131 .25207 -18 -4.5 ]
[.01131 .25207 0 4.5 ]
[.01131 .36916 -18 -4.5 ]
[.01131 .36916 0 4.5 ]
[.01131 .48624 -18 -4.5 ]
[.01131 .48624 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60332 m
.21429 .60957 L
s
[(2)] .21429 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(4)] .40476 .59082 0 1 Mshowa
.59524 .60332 m
.59524 .60957 L
s
[(6)] .59524 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(8)] .78571 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(10)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.2619 .60332 m
.2619 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.7381 .60332 m
.7381 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .01791 m
.03006 .01791 L
s
[(-100)] .01131 .01791 1 0 Mshowa
.02381 .13499 m
.03006 .13499 L
s
[(-80)] .01131 .13499 1 0 Mshowa
.02381 .25207 m
.03006 .25207 L
s
[(-60)] .01131 .25207 1 0 Mshowa
.02381 .36916 m
.03006 .36916 L
s
[(-40)] .01131 .36916 1 0 Mshowa
.02381 .48624 m
.03006 .48624 L
s
[(-20)] .01131 .48624 1 0 Mshowa
.125 Mabswid
.02381 .04718 m
.02756 .04718 L
s
.02381 .07645 m
.02756 .07645 L
s
.02381 .10572 m
.02756 .10572 L
s
.02381 .16426 m
.02756 .16426 L
s
.02381 .19353 m
.02756 .19353 L
s
.02381 .2228 m
.02756 .2228 L
s
.02381 .28135 m
.02756 .28135 L
s
.02381 .31062 m
.02756 .31062 L
s
.02381 .33989 m
.02756 .33989 L
s
.02381 .39843 m
.02756 .39843 L
s
.02381 .4277 m
.02756 .4277 L
s
.02381 .45697 m
.02756 .45697 L
s
.02381 .51551 m
.02756 .51551 L
s
.02381 .54478 m
.02756 .54478 L
s
.02381 .57405 m
.02756 .57405 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .59703 m
.07255 .59703 L
.07356 .59703 L
.07473 .59702 L
.07584 .597 L
.07687 .59698 L
.07782 .59696 L
.07996 .5969 L
.08449 .59673 L
.0893 .59648 L
.09826 .59587 L
.10813 .595 L
.1288 .59267 L
.14816 .5899 L
.18499 .58321 L
.22416 .57412 L
.26188 .56348 L
.29816 .5515 L
.33678 .53687 L
.37395 .52097 L
.41345 .50211 L
.45152 .48204 L
.48814 .46096 L
.52709 .43663 L
.56461 .41136 L
.60068 .38534 L
.63908 .35579 L
.67604 .32555 L
.71534 .29148 L
.75319 .25676 L
.7896 .22162 L
.82835 .18236 L
.86565 .14272 L
.90151 .10292 L
.93971 .05871 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0
oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`001`3oool4000000<0oooo0P00
00040?ooo`8000001@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00090?ooo`050000003oool0
oooo0?ooo`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0nP3oool00`000000oooo0?ooo`050?ooo`002@3oool01@000000oooo0?oo
o`3oool000000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`80
0000nP3oool00`000000oooo0?ooo`060?ooo`0000<0oooo0000000000000P0000040?ooo`050000
003oool0oooo0?ooo`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0n03oool00`000000oooo0?ooo`070?ooo`002@3oool01@000000
oooo0?ooo`3oool000000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0?L0oooo00<000000?ooo`3oool0203oool000L0oooo0`0000030?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`3f0?ooo`030000003oool0oooo00T0oooo00090?ooo`030000003oool0oooo0080oooo0P000004
0?ooo`8000001@3oool00`000000oooo0?ooo`3e0?ooo`030000003oool0oooo00X0oooo000K0?oo
o`030000003oool0oooo0?D0oooo00<000000?ooo`3oool02P3oool001/0oooo00<000000?ooo`3o
ool0m03oool00`000000oooo0?ooo`0;0?ooo`006`3oool200000?@0oooo00<000000?ooo`3oool0
303oool001/0oooo00<000000?ooo`3oool0lP3oool00`000000oooo0?ooo`0=0?ooo`006`3oool0
0`000000oooo0?ooo`3a0?ooo`030000003oool0oooo00h0oooo000K0?ooo`030000003oool0oooo
0?00oooo00<000000?ooo`3oool03`3oool001/0oooo00<000000?ooo`3oool0k`3oool00`000000
oooo0?ooo`0@0?ooo`006`3oool00`000000oooo0?ooo`3^0?ooo`030000003oool0oooo0140oooo
000K0?ooo`030000003oool0oooo0>h0oooo00<000000?ooo`3oool04@3oool001/0oooo00<00000
0?ooo`3oool0k@3oool00`000000oooo0?ooo`0B0?ooo`006`3oool200000>d0oooo00<000000?oo
o`3oool04`3oool001/0oooo00<000000?ooo`3oool0j`3oool00`000000oooo0?ooo`0D0?ooo`00
6`3oool00`000000oooo0?ooo`3Z0?ooo`030000003oool0oooo01D0oooo000K0?ooo`030000003o
ool0oooo0>T0oooo00<000000?ooo`3oool05P3oool001/0oooo00<000000?ooo`3oool0j@3oool0
0`000000oooo0?ooo`0F0?ooo`006`3oool00`000000oooo0?ooo`3X0?ooo`030000003oool0oooo
01L0oooo000K0?ooo`030000003oool0oooo0>L0oooo00<000000?ooo`3oool0603oool001/0oooo
00<000000?ooo`3oool0iP3oool00`000000oooo0?ooo`0I0?ooo`006`3oool200000>H0oooo00<0
00000?ooo`3oool06P3oool001/0oooo00<000000?ooo`3oool0i03oool00`000000oooo0?ooo`0K
0?ooo`006`3oool00`000000oooo0?ooo`3T0?ooo`030000003oool0oooo01/0oooo000K0?ooo`03
0000003oool0oooo0><0oooo00<000000?ooo`3oool0703oool001/0oooo00<000000?ooo`3oool0
hP3oool00`000000oooo0?ooo`0M0?ooo`006`3oool00`000000oooo0?ooo`3Q0?ooo`030000003o
ool0oooo01h0oooo000>0?ooo`800000103oool2000000D0oooo00<000000?ooo`3oool0h03oool0
0`000000oooo0?ooo`0O0?ooo`003@3oool010000000oooo0?ooo`0000020?ooo`040000003oool0
oooo000000@0oooo00<000000?ooo`3oool0g`3oool00`000000oooo0?ooo`0P0?ooo`003@3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo0P00003P0?ooo`030000
003oool0oooo0200oooo00070?ooo`@000000`3oool2000000<0oooo00@000000?ooo`3oool00000
103oool00`000000oooo0?ooo`3N0?ooo`030000003oool0oooo0240oooo000=0?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3M0?ooo`03
0000003oool0oooo0280oooo000=0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3o
ool00000103oool00`000000oooo0?ooo`3L0?ooo`030000003oool0oooo02<0oooo000>0?ooo`80
0000103oool2000000D0oooo00<000000?ooo`3oool0f`3oool00`000000oooo0?ooo`0T0?ooo`00
6`3oool00`000000oooo0?ooo`3J0?ooo`030000003oool0oooo02D0oooo000K0?ooo`030000003o
ool0oooo0=T0oooo00<000000?ooo`3oool09P3oool001/0oooo00<000000?ooo`3oool0f03oool0
0`000000oooo0?ooo`0W0?ooo`006`3oool200000=P0oooo00<000000?ooo`3oool0:03oool001/0
oooo00<000000?ooo`3oool0eP3oool00`000000oooo0?ooo`0Y0?ooo`006`3oool00`000000oooo
0?ooo`3E0?ooo`030000003oool0oooo02X0oooo000K0?ooo`030000003oool0oooo0=@0oooo00<0
00000?ooo`3oool0:`3oool001/0oooo00<000000?ooo`3oool0d`3oool00`000000oooo0?ooo`0/
0?ooo`006`3oool00`000000oooo0?ooo`3B0?ooo`030000003oool0oooo02d0oooo000K0?ooo`03
0000003oool0oooo0=40oooo00<000000?ooo`3oool0;P3oool001/0oooo0P00003A0?ooo`030000
003oool0oooo02l0oooo000K0?ooo`030000003oool0oooo0<l0oooo00<000000?ooo`3oool0<03o
ool001/0oooo00<000000?ooo`3oool0cP3oool00`000000oooo0?ooo`0a0?ooo`006`3oool00`00
0000oooo0?ooo`3>0?ooo`030000003oool0oooo0340oooo000K0?ooo`030000003oool0oooo0<d0
oooo00<000000?ooo`3oool0<P3oool001/0oooo00<000000?ooo`3oool0c03oool00`000000oooo
0?ooo`0c0?ooo`006`3oool00`000000oooo0?ooo`3;0?ooo`030000003oool0oooo03@0oooo000K
0?ooo`030000003oool0oooo0<X0oooo00<000000?ooo`3oool0=@3oool001/0oooo0P00003:0?oo
o`030000003oool0oooo03H0oooo000K0?ooo`030000003oool0oooo0<P0oooo00<000000?ooo`3o
ool0=`3oool001/0oooo00<000000?ooo`3oool0a`3oool00`000000oooo0?ooo`0h0?ooo`006`3o
ool00`000000oooo0?ooo`360?ooo`030000003oool0oooo03T0oooo000K0?ooo`030000003oool0
oooo0<@0oooo0P00000l0?ooo`006`3oool00`000000oooo0?ooo`330?ooo`030000003oool0oooo
03`0oooo000>0?ooo`800000103oool2000000D0oooo00<000000?ooo`3oool0`P3oool00`000000
oooo0?ooo`0m0?ooo`003@3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0000
00@0oooo00<000000?ooo`3oool0`@3oool00`000000oooo0?ooo`0n0?ooo`003@3oool010000000
oooo0?ooo`0000020?ooo`040000003oool0oooo000000@0oooo0P0000310?ooo`030000003oool0
oooo03l0oooo00070?ooo`@000000P3oool3000000<0oooo00@000000?ooo`3oool00000103oool0
0`000000oooo0?ooo`2o0?ooo`030000003oool0oooo0400oooo000=0?ooo`030000003oool0oooo
00<0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`2n0?ooo`030000003oool0
oooo0440oooo000=0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool00000103oool0
0`000000oooo0?ooo`2m0?ooo`030000003oool0oooo0480oooo000>0?ooo`<000000`3oool20000
00D0oooo00<000000?ooo`3oool0_03oool00`000000oooo0?ooo`130?ooo`006`3oool00`000000
oooo0?ooo`2j0?ooo`800000AP3oool001/0oooo00<000000?ooo`3oool0^@3oool00`000000oooo
0?ooo`160?ooo`006`3oool00`000000oooo0?ooo`2h0?ooo`030000003oool0oooo04L0oooo000K
0?ooo`800000^03oool00`000000oooo0?ooo`180?ooo`006`3oool00`000000oooo0?ooo`2f0?oo
o`030000003oool0oooo04T0oooo000K0?ooo`030000003oool0oooo0;D0oooo00<000000?ooo`3o
ool0BP3oool001/0oooo00<000000?ooo`3oool0]03oool00`000000oooo0?ooo`1;0?ooo`006`3o
ool00`000000oooo0?ooo`2b0?ooo`800000CP3oool001/0oooo00<000000?ooo`3oool0/@3oool0
0`000000oooo0?ooo`1>0?ooo`006`3oool00`000000oooo0?ooo`2`0?ooo`030000003oool0oooo
04l0oooo000K0?ooo`800000/03oool00`000000oooo0?ooo`1@0?ooo`006`3oool00`000000oooo
0?ooo`2^0?ooo`030000003oool0oooo0540oooo000K0?ooo`030000003oool0oooo0:`0oooo0P00
001D0?ooo`006`3oool00`000000oooo0?ooo`2[0?ooo`030000003oool0oooo05@0oooo000K0?oo
o`030000003oool0oooo0:X0oooo00<000000?ooo`3oool0E@3oool001/0oooo00<000000?ooo`3o
ool0Z@3oool00`000000oooo0?ooo`1F0?ooo`006`3oool00`000000oooo0?ooo`2W0?ooo`800000
F@3oool001/0oooo00<000000?ooo`3oool0YP3oool00`000000oooo0?ooo`1I0?ooo`006`3oool2
00000:H0oooo00<000000?ooo`3oool0FP3oool001/0oooo00<000000?ooo`3oool0Y03oool00`00
0000oooo0?ooo`1K0?ooo`006`3oool00`000000oooo0?ooo`2R0?ooo`800000GP3oool001/0oooo
00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`1N0?ooo`006`3oool00`000000oooo0?oo
o`2P0?ooo`030000003oool0oooo05l0oooo000K0?ooo`030000003oool0oooo09l0oooo00<00000
0?ooo`3oool0H03oool000l0oooo00<000000?ooo`3oool00P3oool2000000D0oooo00<000000?oo
o`3oool0W@3oool2000006<0oooo000?0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo09`0oooo00<000000?ooo`3oool0H`3oool000`0
oooo1@0000020?ooo`040000003oool0oooo000000@0oooo0P00002L0?ooo`030000003oool0oooo
06@0oooo00070?ooo`@0000000D0oooo0000003oool0oooo000000030?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool0VP3oool00`000000oooo0?ooo`1U0?ooo`003@3oool00`00
0000oooo000000030?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0V03oool2
000006P0oooo000>0?ooo`8000000`3oool010000000oooo0?ooo`0000040?ooo`030000003oool0
oooo09L0oooo00<000000?ooo`3oool0J03oool000l0oooo00<000000?ooo`3oool00P3oool20000
00D0oooo00<000000?ooo`3oool0UP3oool00`000000oooo0?ooo`1Y0?ooo`006`3oool00`000000
oooo0?ooo`2E0?ooo`030000003oool0oooo06X0oooo000K0?ooo`030000003oool0oooo09<0oooo
0P00001]0?ooo`006`3oool00`000000oooo0?ooo`2B0?ooo`030000003oool0oooo06d0oooo000K
0?ooo`800000TP3oool00`000000oooo0?ooo`1^0?ooo`006`3oool00`000000oooo0?ooo`2?0?oo
o`800000L@3oool001/0oooo00<000000?ooo`3oool0SP3oool00`000000oooo0?ooo`1a0?ooo`00
6`3oool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo0780oooo000K0?ooo`030000003o
ool0oooo08/0oooo0P00001e0?ooo`006`3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0
oooo07D0oooo000K0?ooo`030000003oool0oooo08P0oooo0P00001h0?ooo`006`3oool00`000000
oooo0?ooo`270?ooo`030000003oool0oooo07P0oooo000K0?ooo`800000Q`3oool00`000000oooo
0?ooo`1i0?ooo`006`3oool00`000000oooo0?ooo`240?ooo`800000O03oool001/0oooo00<00000
0?ooo`3oool0P`3oool00`000000oooo0?ooo`1l0?ooo`006`3oool00`000000oooo0?ooo`210?oo
o`800000O`3oool001/0oooo00<000000?ooo`3oool0O`3oool200000840oooo000K0?ooo`030000
003oool0oooo07d0oooo0P0000230?ooo`006`3oool00`000000oooo0?ooo`1k0?ooo`800000Q@3o
ool001/0oooo0P00001j0?ooo`800000Q`3oool001/0oooo00<000000?ooo`3oool0N03oool00`00
0000oooo0?ooo`270?ooo`006`3oool00`000000oooo0?ooo`1f0?ooo`800000RP3oool001/0oooo
00<000000?ooo`3oool0M@3oool00`000000oooo0?ooo`2:0?ooo`006`3oool00`000000oooo0?oo
o`1c0?ooo`800000S@3oool001/0oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`2=
0?ooo`003@3oool4000000<0oooo0P0000050?ooo`030000003oool0oooo0700oooo0P00002@0?oo
o`003@3oool00`000000oooo0?ooo`030?ooo`040000003oool0oooo000000@0oooo00<000000?oo
o`3oool0KP3oool200000980oooo000>0?ooo`030000003oool0oooo0080oooo00@000000?ooo`3o
ool00000103oool2000006d0oooo0P00002D0?ooo`001`3oool4000000@0oooo00D000000?ooo`3o
ool0oooo000000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0JP3oool20000
09H0oooo000@0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`1X0?ooo`800000V03oool000d0oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo06H0oooo0P00002J0?ooo`003P3oool20000
00@0oooo0P0000050?ooo`030000003oool0oooo06<0oooo0`00002L0?ooo`006`3oool00`000000
oooo0?ooo`1Q0?ooo`800000W`3oool001/0oooo00<000000?ooo`3oool0G`3oool200000:40oooo
000K0?ooo`030000003oool0oooo05d0oooo0P00002S0?ooo`006`3oool2000005`0oooo0P00002U
0?ooo`006`3oool00`000000oooo0?ooo`1I0?ooo`800000Y`3oool001/0oooo00<000000?ooo`3o
ool0E`3oool200000:T0oooo000K0?ooo`030000003oool0oooo05D0oooo0P00002[0?ooo`006`3o
ool00`000000oooo0?ooo`1C0?ooo`800000[@3oool001/0oooo00<000000?ooo`3oool0D@3oool2
00000:l0oooo000K0?ooo`030000003oool0oooo04l0oooo0P00002a0?ooo`006`3oool00`000000
oooo0?ooo`1=0?ooo`800000/`3oool001/0oooo0P00001;0?ooo`<00000]@3oool001/0oooo00<0
00000?ooo`3oool0B03oool200000;P0oooo000K0?ooo`030000003oool0oooo04D0oooo0`00002j
0?ooo`006`3oool00`000000oooo0?ooo`110?ooo`@00000_@3oool001/0oooo00<000000?ooo`3o
ool0?P3oool300000<40oooo000K0?ooo`030000003oool0oooo02d0oooo1000000:0?ooo`<00000
903oool00`000000oooo0?ooo`0_0?ooo`800000<03oool2000002d0oooo100000030?ooo`800000
103oool001/0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`070?ooo`@000009`3o
ool00`000000oooo0?ooo`0^0?ooo`040000003oool0oooo000002h0oooo00@000000?ooo`3oool0
0000;P3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00@3oool001/0
oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`030?ooo`<00000:03oool5000002l0
oooo00@000000?ooo`3oool00000;P3oool010000000oooo0?ooo`00000^0?ooo`050000003oool0
oooo0?ooo`0000000P3oool00`000000oooo0?ooo`010?ooo`006`3oool200000300oooo1@00000[
0?ooo`040000003oool0oooo00000300oooo0`00000`0?ooo`800000;`3oool01@000000oooo0?oo
o`3oool000000080oooo00<000000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3oool0:`3o
ool6000002l0oooo00<000000?ooo`000000<03oool00`000000oooo0?ooo`0_0?ooo`040000003o
ool0oooo000002h0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo0040
oooo000K0?ooo`030000003oool0oooo02H0oooo1@0000020?ooo`040000003oool0oooo00000300
oooo0P00000`0?ooo`030000003oool0oooo02l0oooo00@000000?ooo`3oool00000;03oool30000
00<0oooo00@000000?ooo`3oool000000`3oool001/0oooo00<000000?ooo`3oool08@3oool50000
00P0oooo0P00000b0?ooo`030000003oool0oooo02l0oooo0`00000_0?ooo`800000;`3oool00`00
0000oooo0?ooo`020?ooo`800000103oool001/0oooo00<000000?ooo`3oool05P3oool;00000>40
oooo000K0?ooo`030000003oool0oooo00h0oooo2000003/0?ooo`006`3oool00`000000oooo0?oo
o`090?ooo`D00000m03oool001@0ooooo`00000=000000006`3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool01@3oool001/0oooo00<000000?ooo`3o
ool0;`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3o
ool0;`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo00D0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.07864, -107.085, \
0.0394728, 0.642171}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, PlotRange \[Rule] {\(-5\), 5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 0.309017 0.0618034 [
[.21429 .29652 -3 -9 ]
[.21429 .29652 3 0 ]
[.40476 .29652 -3 -9 ]
[.40476 .29652 3 0 ]
[.59524 .29652 -3 -9 ]
[.59524 .29652 3 0 ]
[.78571 .29652 -3 -9 ]
[.78571 .29652 3 0 ]
[.97619 .29652 -6 -9 ]
[.97619 .29652 6 0 ]
[.01131 .0618 -12 -4.5 ]
[.01131 .0618 0 4.5 ]
[.01131 .18541 -12 -4.5 ]
[.01131 .18541 0 4.5 ]
[.01131 .43262 -6 -4.5 ]
[.01131 .43262 0 4.5 ]
[.01131 .55623 -6 -4.5 ]
[.01131 .55623 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .30902 m
.21429 .31527 L
s
[(2)] .21429 .29652 0 1 Mshowa
.40476 .30902 m
.40476 .31527 L
s
[(4)] .40476 .29652 0 1 Mshowa
.59524 .30902 m
.59524 .31527 L
s
[(6)] .59524 .29652 0 1 Mshowa
.78571 .30902 m
.78571 .31527 L
s
[(8)] .78571 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(10)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.2619 .30902 m
.2619 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.5 .30902 m
.5 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.7381 .30902 m
.7381 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 .0618 m
.03006 .0618 L
s
[(-4)] .01131 .0618 1 0 Mshowa
.02381 .18541 m
.03006 .18541 L
s
[(-2)] .01131 .18541 1 0 Mshowa
.02381 .43262 m
.03006 .43262 L
s
[(2)] .01131 .43262 1 0 Mshowa
.02381 .55623 m
.03006 .55623 L
s
[(4)] .01131 .55623 1 0 Mshowa
.125 Mabswid
.02381 .09271 m
.02756 .09271 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .0309 m
.02756 .0309 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .24259 m
.07255 .24261 L
.07356 .24258 L
.07473 .24248 L
.07584 .24232 L
.07687 .24213 L
.07782 .24192 L
.07996 .2413 L
.08449 .23946 L
.0893 .23682 L
.09826 .23035 L
.10813 .22124 L
.1288 .19656 L
.14816 .16735 L
.18499 .09673 L
.22416 .0008 L
s
.22416 .0008 m
.22443 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo0=H0
oooo000?0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0eP3oool000l0oooo00<0
00000?ooo`3oool0=03oool00`000000oooo0?ooo`3G0?ooo`003`3oool00`000000oooo0?ooo`0d
0?ooo`030000003oool0oooo0=L0oooo000?0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool0f03oool000l0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`3H0?ooo`00
3`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0=P0oooo000?0?ooo`030000003o
ool0oooo0380oooo00<000000?ooo`3oool0f@3oool000l0oooo00<000000?ooo`3oool0<P3oool0
0`000000oooo0?ooo`3I0?ooo`003`3oool200000380oooo00<000000?ooo`3oool0fP3oool000l0
oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`3J0?ooo`003`3oool00`000000oooo
0?ooo`0a0?ooo`030000003oool0oooo0=X0oooo000?0?ooo`030000003oool0oooo0300oooo00<0
00000?ooo`3oool0f`3oool000l0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`3K
0?ooo`003`3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0=`0oooo00090?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`3L0?ooo`00
2@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0
g@3oool000H0oooo1@0000040?ooo`<00000;P3oool00`000000oooo0?ooo`3M0?ooo`0000<0oooo
0000000000000P0000001@3oool000000?ooo`3oool0000000D0oooo00<000000?ooo`3oool0;P3o
ool00`000000oooo0?ooo`3M0?ooo`001`3oool00`000000oooo000000050?ooo`030000003oool0
oooo02d0oooo00<000000?ooo`3oool0gP3oool000P0oooo0P0000050?ooo`030000003oool0oooo
02d0oooo00<000000?ooo`3oool0gP3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0/0?ooo`030000003oool0oooo0=l0oooo000?0?ooo`030000003oool0oooo02`0oooo
00<000000?ooo`3oool0g`3oool000l0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?oo
o`3O0?ooo`003`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0>00oooo000?0?oo
o`030000003oool0oooo02/0oooo00<000000?ooo`3oool0h03oool000l0oooo0P00000[0?ooo`03
0000003oool0oooo0>40oooo000?0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0
h@3oool000l0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3Q0?ooo`003`3oool0
0`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0>80oooo000?0?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool0hP3oool000l0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`3S0?ooo`003`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0><0oooo
000?0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0i03oool000l0oooo0P00000X
0?ooo`030000003oool0oooo0>@0oooo000?0?ooo`030000003oool0oooo02H0oooo00<000000?oo
o`3oool0i@3oool000l0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`3U0?ooo`00
3`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0>H0oooo000?0?ooo`030000003o
ool0oooo02D0oooo00<000000?ooo`3oool0iP3oool000l0oooo00<000000?ooo`3oool0903oool0
0`000000oooo0?ooo`3W0?ooo`003`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
0>L0oooo000?0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0j03oool000l0oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`3X0?ooo`003`3oool2000002<0oooo00<0
00000?ooo`3oool0j@3oool000l0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`3Y
0?ooo`003`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0>X0oooo000?0?ooo`03
0000003oool0oooo0240oooo00<000000?ooo`3oool0jP3oool000l0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`3[0?ooo`003`3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo0>`0oooo000?0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0k03oool0
00L0oooo100000040?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0k@3oool000L0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0>h0
oooo00080?ooo`030000003oool0oooo00@0oooo0`00000L0?ooo`030000003oool0oooo0>l0oooo
00000`3oool0000000000002000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo0>l0oooo000:0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`3`0?ooo`001`3oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0l@3oool000P0oooo0P0000050?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0lP3oool000l0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`3c0?ooo`003`3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo0?@0oooo000?0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0m03o
ool000l0oooo0P00000G0?ooo`030000003oool0oooo0?D0oooo000?0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0mP3oool000l0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`3g0?ooo`003`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0?P0oooo
000?0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0n@3oool000l0oooo00<00000
0?ooo`3oool0403oool200000?`0oooo000?0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0o03oool000l0oooo00<000000?ooo`3oool02`3oool400000?l0oooo000?0?ooo`030000
003oool0oooo00X0oooo0P00003o0?ooo`<0oooo000?0?ooo`800000o`3oool@0?ooo`003`3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0
oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool00`000000oooo0?ooo`0`0?ooo`@00000<`3oool00`000000oooo0?ooo`0a0?oo
o`800000<`3oool2000002l0oooo100000030?ooo`800000103oool000l0oooo0P00000a0?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool0<03oool010000000oooo0?ooo`00000a0?oo
o`040000003oool0oooo00000300oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003o
ool0oooo0040oooo000?0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0<03oool5
00000340oooo00@000000?ooo`3oool00000<@3oool010000000oooo0?ooo`00000`0?ooo`050000
003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`010?ooo`003`3oool00`000000oooo
0?ooo`0b0?ooo`030000003oool0oooo02l0oooo00@000000?ooo`3oool00000<P3oool3000003<0
oooo0P00000a0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`010?oo
o`003`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`00
0000<P3oool00`000000oooo0?ooo`0b0?ooo`040000003oool0oooo00000300oooo00D000000?oo
o`3oool0oooo000000020?ooo`030000003oool0oooo0040oooo000?0?ooo`030000003oool0oooo
0300oooo00@000000?ooo`3oool00000<P3oool200000380oooo00<000000?ooo`3oool0<P3oool0
10000000oooo0?ooo`00000^0?ooo`<000000`3oool010000000oooo0?ooo`0000030?ooo`003`3o
ool00`000000oooo0?ooo`0a0?ooo`800000=03oool00`000000oooo0?ooo`0a0?ooo`<00000<P3o
ool200000340oooo00<000000?ooo`3oool00P3oool2000000@0oooo000?0?ooo`030000003oool0
oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000
oooo0?ooo`3o0?ooo`l0oooo00080?ooool000006@00000000l0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00D0oooo000?0?ooo`030000003oool0
oooo0380oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0
oooo0380oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`050?ooo`003`3oool00`00
0000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo
00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?
0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3o
ool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool0
00l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0
oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3o
ool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo
3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o
0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3o
ool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3o
ool@0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo
0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo
0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000L0oooo10000004
0?ooo`030000003oool0oooo0?l0oooo3`3oool000L0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`3o0?ooo`l0oooo00080?ooo`030000003oool0oooo00@0oooo0`00003o0?ooo`l0
oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool?0?ooo`002P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0oooo3`3oool000L0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`3o0?ooo`l0oooo00080?ooo`8000001@3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0
oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo0P00003o0?oooa00oooo000?0?oo
o`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`00
3`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3o
ool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?oo
o`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0
o`3oool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?oo
o`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?oo
o`3oool0o`3oool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0
o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0
oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000
oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000T0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo00090?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0o`3oool?0?ooo`001P3oool5000000@0oooo0`00003o0?oo
o`l0oooo00060?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`001`3oool00`000000oooo000000050?ooo`030000003oool0oooo0?l0oooo3`3oool000P0oooo
0P0000050?ooo`030000003oool0oooo0?l0oooo3`3oool000T0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool0
00l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0
oooo000?0?ooo`800000o`3oool@0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?
0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo
3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o
0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3o
ool0o`3oool?0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.584301, -5.1465, \
0.0377504, 0.0581727}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(tja ... \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      testfuncspec[IntegerPart[om*10^16]*10^\((\(-16\))\)*I, 1/2, Pi/16, 
        1], {om, 0.001, 5}, PlotRange \[Rule] {\(-5\), 5}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"slwcon\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 0.309017 0.0618034 [
[.21429 .29652 -3 -9 ]
[.21429 .29652 3 0 ]
[.40476 .29652 -3 -9 ]
[.40476 .29652 3 0 ]
[.59524 .29652 -3 -9 ]
[.59524 .29652 3 0 ]
[.78571 .29652 -3 -9 ]
[.78571 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.01131 .0618 -12 -4.5 ]
[.01131 .0618 0 4.5 ]
[.01131 .18541 -12 -4.5 ]
[.01131 .18541 0 4.5 ]
[.01131 .43262 -6 -4.5 ]
[.01131 .43262 0 4.5 ]
[.01131 .55623 -6 -4.5 ]
[.01131 .55623 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .30902 m
.21429 .31527 L
s
[(1)] .21429 .29652 0 1 Mshowa
.40476 .30902 m
.40476 .31527 L
s
[(2)] .40476 .29652 0 1 Mshowa
.59524 .30902 m
.59524 .31527 L
s
[(3)] .59524 .29652 0 1 Mshowa
.78571 .30902 m
.78571 .31527 L
s
[(4)] .78571 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(5)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.0619 .30902 m
.0619 .31277 L
s
.1 .30902 m
.1 .31277 L
s
.1381 .30902 m
.1381 .31277 L
s
.17619 .30902 m
.17619 .31277 L
s
.25238 .30902 m
.25238 .31277 L
s
.29048 .30902 m
.29048 .31277 L
s
.32857 .30902 m
.32857 .31277 L
s
.36667 .30902 m
.36667 .31277 L
s
.44286 .30902 m
.44286 .31277 L
s
.48095 .30902 m
.48095 .31277 L
s
.51905 .30902 m
.51905 .31277 L
s
.55714 .30902 m
.55714 .31277 L
s
.63333 .30902 m
.63333 .31277 L
s
.67143 .30902 m
.67143 .31277 L
s
.70952 .30902 m
.70952 .31277 L
s
.74762 .30902 m
.74762 .31277 L
s
.82381 .30902 m
.82381 .31277 L
s
.8619 .30902 m
.8619 .31277 L
s
.9 .30902 m
.9 .31277 L
s
.9381 .30902 m
.9381 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 .0618 m
.03006 .0618 L
s
[(-4)] .01131 .0618 1 0 Mshowa
.02381 .18541 m
.03006 .18541 L
s
[(-2)] .01131 .18541 1 0 Mshowa
.02381 .43262 m
.03006 .43262 L
s
[(2)] .01131 .43262 1 0 Mshowa
.02381 .55623 m
.03006 .55623 L
s
[(4)] .01131 .55623 1 0 Mshowa
.125 Mabswid
.02381 .09271 m
.02756 .09271 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .0309 m
.02756 .0309 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.024 .29864 m
.02624 .28763 L
.02865 .27782 L
.03297 .26426 L
.03562 .25803 L
.03812 .25326 L
.0428 .24662 L
.0452 .24413 L
.04775 .24202 L
.04994 .24055 L
.05231 .23928 L
.05484 .2382 L
.05722 .23742 L
.06004 .23672 L
.06263 .23625 L
.06522 .23591 L
.06664 .23577 L
.06798 .23565 L
.06923 .23556 L
.07059 .23548 L
.07295 .23538 L
.07431 .23533 L
.07581 .23529 L
.07882 .23522 L
.08422 .23512 L
.08686 .23507 L
.08935 .235 L
.09222 .2349 L
.09484 .23478 L
.09959 .2345 L
.10475 .23408 L
.1096 .23356 L
.11415 .23296 L
.12452 .23118 L
.13463 .2289 L
.14553 .22585 L
.16556 .21872 L
.18666 .20924 L
.22476 .18743 L
.26531 .15804 L
.30435 .12407 L
.34584 .08201 L
.38581 .03579 L
s
.38581 .03579 m
.41351 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo0:80
oooo000?0?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool0X`3oool000l0oooo00<0
00000?ooo`3oool0J03oool00`000000oooo0?ooo`2S0?ooo`003`3oool00`000000oooo0?ooo`1W
0?ooo`030000003oool0oooo0:@0oooo000?0?ooo`030000003oool0oooo06H0oooo00<000000?oo
o`3oool0Y@3oool000l0oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`2U0?ooo`00
3`3oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo0:H0oooo000?0?ooo`030000003o
ool0oooo06@0oooo00<000000?ooo`3oool0Y`3oool000l0oooo00<000000?ooo`3oool0H`3oool0
0`000000oooo0?ooo`2X0?ooo`003`3oool2000006@0oooo00<000000?ooo`3oool0Z03oool000l0
oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`2Y0?ooo`003`3oool00`000000oooo
0?ooo`1Q0?ooo`030000003oool0oooo0:X0oooo000?0?ooo`030000003oool0oooo0600oooo00<0
00000?ooo`3oool0Z`3oool000l0oooo00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`2/
0?ooo`003`3oool00`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0:`0oooo00090?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`2]0?ooo`00
2@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool0
[P3oool000H0oooo1@0000040?ooo`<00000G03oool00`000000oooo0?ooo`2_0?ooo`0000<0oooo
0000000000000P0000001@3oool000000?ooo`3oool0000000D0oooo00<000000?ooo`3oool0F`3o
ool00`000000oooo0?ooo`2`0?ooo`001`3oool00`000000oooo000000050?ooo`030000003oool0
oooo05X0oooo00<000000?ooo`3oool0/@3oool000P0oooo0P0000050?ooo`030000003oool0oooo
05X0oooo00<000000?ooo`3oool0/@3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`1I0?ooo`030000003oool0oooo0;80oooo000?0?ooo`030000003oool0oooo05P0oooo
00<000000?ooo`3oool0/`3oool000l0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?oo
o`2d0?ooo`003`3oool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo0;D0oooo000?0?oo
o`030000003oool0oooo05D0oooo00<000000?ooo`3oool0]P3oool000l0oooo0P00001E0?ooo`03
0000003oool0oooo0;L0oooo000?0?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool0
^03oool000l0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`2i0?ooo`003`3oool0
0`000000oooo0?ooo`1A0?ooo`030000003oool0oooo0;X0oooo000?0?ooo`030000003oool0oooo
0500oooo00<000000?ooo`3oool0^`3oool000l0oooo00<000000?ooo`3oool0C`3oool00`000000
oooo0?ooo`2l0?ooo`003`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo0;d0oooo
000?0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0_P3oool000l0oooo0P00001=
0?ooo`030000003oool0oooo0;l0oooo000?0?ooo`030000003oool0oooo04/0oooo00<000000?oo
o`3oool0`03oool000l0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`310?ooo`00
3`3oool00`000000oooo0?ooo`180?ooo`800000a03oool000l0oooo00<000000?ooo`3oool0A`3o
ool00`000000oooo0?ooo`340?ooo`003`3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo0<D0oooo000?0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0aP3oool000l0
oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`370?ooo`003`3oool00`000000oooo
0?ooo`120?ooo`800000bP3oool000l0oooo0P0000120?ooo`030000003oool0oooo0<X0oooo000?
0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0b`3oool000l0oooo00<000000?oo
o`3oool0?P3oool200000<h0oooo000?0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3o
ool0cP3oool000l0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`3?0?ooo`003`3o
ool00`000000oooo0?ooo`0j0?ooo`800000dP3oool000l0oooo00<000000?ooo`3oool0>@3oool0
0`000000oooo0?ooo`3B0?ooo`001`3oool4000000@0oooo00<000000?ooo`3oool0>03oool00`00
0000oooo0?ooo`3C0?ooo`001`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03H0
oooo0P00003F0?ooo`00203oool00`000000oooo0?ooo`040?ooo`<00000=@3oool00`000000oooo
0?ooo`3F0?ooo`0000<0oooo0000000000000P0000040?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0<`3oool200000=T0oooo000:0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0<@3oool200000=/0oooo00070?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool0;`3oool200000=d0oooo00080?ooo`8000001@3oool00`000000oooo0?ooo`0]0?oo
o`800000g`3oool000l0oooo00<000000?ooo`3oool0:`3oool200000>40oooo000?0?ooo`030000
003oool0oooo02T0oooo0P00003S0?ooo`003`3oool00`000000oooo0?ooo`0W0?ooo`800000i@3o
ool000l0oooo0P00000V0?ooo`800000i`3oool000l0oooo00<000000?ooo`3oool08`3oool20000
0>T0oooo000?0?ooo`030000003oool0oooo0240oooo0P00003[0?ooo`003`3oool00`000000oooo
0?ooo`0N0?ooo`<00000k@3oool000l0oooo00<000000?ooo`3oool0603oool600000?00oooo000?
0?ooo`030000003oool0oooo00d0oooo2`00003f0?ooo`003`3oool00`000000oooo0?ooo`060?oo
o`L00000o`3oool20?ooo`003`3oool00`000000oooo0?ooo`040?ooo`800000o`3oool90?ooo`00
3`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo2@3oool000l0oooo0P00
00030?ooo`030000003oool0oooo0?l0oooo2P3oool000l0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`3o0?ooo`X0oooo000?0?ooo`050000003oool0oooo0?ooo`000000o`3oool=
0?ooo`003`3oool010000000oooo0?ooo`00003o0?ooo`h0oooo000?0?ooo`040000003oool0oooo
00000?l0oooo3P3oool000l0oooo00@000000?ooo`3oool00000o`3oool>0?ooo`003`3oool00`00
0000oooo0000003o0?ooo`l0oooo000?0?ooo`030000003oool000000300oooo1000000a0?ooo`@0
0000<P3oool2000003@0oooo00<000000?ooo`3oool0<03oool3000000L0oooo000?0?ooo`800000
<`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0380oooo00@000000?ooo`3oool0
0000<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00@0oooo000?0?ooo`800000
<`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0
;P3oool5000003@0oooo00<000000?ooo`3oool0103oool000l0oooo0P00000c0?ooo`030000003o
ool0oooo0380oooo00<000000?ooo`3oool0<@3oool200000340oooo00@000000?ooo`3oool00000
<P3oool3000000L0oooo000?0?ooo`800000<`3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo0380oooo00<000000?ooo`3oool0;`3oool00`000000oooo0000000b0?ooo`030000003o
ool0oooo00L0oooo000?0?ooo`800000<@3oool300000380oooo00@000000?ooo`3oool00000<@3o
ool010000000oooo0?ooo`00000b0?ooo`800000<P3oool00`000000oooo0?ooo`070?ooo`003`3o
ool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0340oooo0P00000c0?ooo`800000=03o
ool00`000000oooo0?ooo`0`0?ooo`@000001P3oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000P0ooooo`00000I000000003`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00D0oooo000?0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0<P3oool0
0`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0<P3oool0
0`000000oooo0?ooo`050?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`03
0000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3o
ool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool0
00l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo
00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?
0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo
3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool200000?l0oooo403oool0
00l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0
oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3o
ool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo
0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo
0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3oool@0?ooo`003`3oool00`000000oooo0?ooo`3o
0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3o
ool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003o
ool0oooo0?l0oooo3`3oool000L0oooo100000040?ooo`030000003oool0oooo0?l0oooo3`3oool0
00L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`l0oooo00080?ooo`03
0000003oool0oooo00@0oooo0`00003o0?ooo`l0oooo00090?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0o`3oool?0?ooo`002P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0?l0oooo3`3oool000L0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`3o0?ooo`l0oooo00080?ooo`8000001@3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?oo
o`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`00
3`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3o
ool000l0oooo0P00003o0?oooa00oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0
oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool200000?l0oooo403o
ool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?oo
o`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0
o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0
oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool200000?l0
oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?oo
o`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?oo
o`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000
003oool0oooo0?l0oooo3`3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`3o0?ooo`l0oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`001P3oool5000000@0oooo0`00003o0?ooo`l0oooo00060?ooo`040000003oool0oooo0000
00D0oooo00<000000?ooo`3oool0o`3oool?0?ooo`001`3oool00`000000oooo000000050?ooo`03
0000003oool0oooo0?l0oooo3`3oool000P0oooo0P0000050?ooo`030000003oool0oooo0?l0oooo
3`3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?
0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3oool@0?ooo`003`3o
ool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool0
00l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0
oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3o
ool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo
0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`00o`3ooolQ0?ooo`00o`3o
oolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.292151, -5.1465, \
0.0188752, 0.0581727}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["masses", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(alomz[1, Cos[0.01], 1, 10]\)], "Input"],

Cell[BoxData[
    \(\(-1.1487308621264625`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[alomz[1, Cos[th], 1, 10], {th, 0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.603319 0.512395 [
[.2619 .59082 -9 -9 ]
[.2619 .59082 9 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .09092 -12 -4.5 ]
[.01131 .09092 0 4.5 ]
[.01131 .1934 -24 -4.5 ]
[.01131 .1934 0 4.5 ]
[.01131 .29588 -24 -4.5 ]
[.01131 .29588 0 4.5 ]
[.01131 .39836 -24 -4.5 ]
[.01131 .39836 0 4.5 ]
[.01131 .50084 -24 -4.5 ]
[.01131 .50084 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(0.2)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(0.4)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.6)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(0.8)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.08333 .60332 m
.08333 .60707 L
s
.14286 .60332 m
.14286 .60707 L
s
.20238 .60332 m
.20238 .60707 L
s
.32143 .60332 m
.32143 .60707 L
s
.38095 .60332 m
.38095 .60707 L
s
.44048 .60332 m
.44048 .60707 L
s
.55952 .60332 m
.55952 .60707 L
s
.61905 .60332 m
.61905 .60707 L
s
.67857 .60332 m
.67857 .60707 L
s
.79762 .60332 m
.79762 .60707 L
s
.85714 .60332 m
.85714 .60707 L
s
.91667 .60332 m
.91667 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .09092 m
.03006 .09092 L
s
[(-1)] .01131 .09092 1 0 Mshowa
.02381 .1934 m
.03006 .1934 L
s
[(-0.8)] .01131 .1934 1 0 Mshowa
.02381 .29588 m
.03006 .29588 L
s
[(-0.6)] .01131 .29588 1 0 Mshowa
.02381 .39836 m
.03006 .39836 L
s
[(-0.4)] .01131 .39836 1 0 Mshowa
.02381 .50084 m
.03006 .50084 L
s
[(-0.2)] .01131 .50084 1 0 Mshowa
.125 Mabswid
.02381 .11654 m
.02756 .11654 L
s
.02381 .14216 m
.02756 .14216 L
s
.02381 .16778 m
.02756 .16778 L
s
.02381 .21902 m
.02756 .21902 L
s
.02381 .24464 m
.02756 .24464 L
s
.02381 .27026 m
.02756 .27026 L
s
.02381 .3215 m
.02756 .3215 L
s
.02381 .34712 m
.02756 .34712 L
s
.02381 .37274 m
.02756 .37274 L
s
.02381 .42398 m
.02756 .42398 L
s
.02381 .4496 m
.02756 .4496 L
s
.02381 .47522 m
.02756 .47522 L
s
.02381 .52646 m
.02756 .52646 L
s
.02381 .55208 m
.02756 .55208 L
s
.02381 .5777 m
.02756 .5777 L
s
.02381 .0653 m
.02756 .0653 L
s
.02381 .03968 m
.02756 .03968 L
s
.02381 .01406 m
.02756 .01406 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .01472 m
.0403 .01518 L
.04458 .01575 L
.05429 .01752 L
.06375 .01987 L
.07387 .02305 L
.09565 .03214 L
.11547 .04285 L
.15491 .06998 L
.19284 .10152 L
.2332 .13851 L
.27205 .17544 L
.30942 .21072 L
.3492 .24694 L
.38749 .27982 L
.4282 .31225 L
.46741 .34089 L
.50512 .36604 L
.54526 .39032 L
.5839 .41145 L
.62496 .43167 L
.66453 .44919 L
.7026 .46439 L
.74309 .47898 L
.78208 .49164 L
.81958 .50266 L
.8595 .51328 L
.89792 .52253 L
.93877 .53143 L
.97619 .53883 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool2
00000?l0oooo103oool001/0oooo00<000000?ooo`3oool01P00003l0?ooo`006`3oool00`000000
oooo0?ooo`060?ooo`<00000n@3oool001/0oooo00<000000?ooo`3oool02@3oool300000?H0oooo
000K0?ooo`030000003oool0oooo00`0oooo0`00003c0?ooo`006`3oool00`000000oooo0?ooo`0?
0?ooo`800000l@3oool001/0oooo00<000000?ooo`3oool04@3oool200000>l0oooo000K0?ooo`80
0000503oool200000>d0oooo000K0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
jP3oool001/0oooo00<000000?ooo`3oool05P3oool200000>X0oooo000K0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0i`3oool001/0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`3V0?ooo`006`3oool00`000000oooo0?ooo`0J0?ooo`800000iP3oool001/0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3S0?ooo`006`3oool2000001h0oooo0P00
003S0?ooo`006`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0>00oooo000K0?oo
o`030000003oool0oooo0200oooo00<000000?ooo`3oool0g`3oool001/0oooo00<000000?ooo`3o
ool08@3oool200000=l0oooo000K0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
g03oool001<0oooo100000040?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0f`3o
ool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0
oooo0=X0oooo000E0?ooo`030000003oool0oooo00<0oooo0P00000W0?ooo`800000fP3oool000d0
oooo100000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`3G0?ooo`005@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02T0oooo
00<000000?ooo`3oool0eP3oool001<0oooo0`0000050?ooo`030000003oool0oooo02X0oooo00<0
00000?ooo`3oool0e@3oool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0[
0?ooo`030000003oool0oooo0=@0oooo000K0?ooo`030000003oool0oooo02`0oooo00<000000?oo
o`3oool0d`3oool001/0oooo0P00000^0?ooo`030000003oool0oooo0=80oooo000K0?ooo`030000
003oool0oooo02h0oooo0P00003B0?ooo`006`3oool00`000000oooo0?ooo`0`0?ooo`030000003o
ool0oooo0<l0oooo000K0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0cP3oool0
01/0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`3=0?ooo`006`3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo0<`0oooo000K0?ooo`030000003oool0oooo03@0oooo
00<000000?ooo`3oool0b`3oool001/0oooo0P00000f0?ooo`030000003oool0oooo0<X0oooo000K
0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0b@3oool001/0oooo00<000000?oo
o`3oool0=`3oool00`000000oooo0?ooo`380?ooo`006`3oool00`000000oooo0?ooo`0h0?ooo`03
0000003oool0oooo0<L0oooo000K0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0
aP3oool001/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`350?ooo`006`3oool0
0`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0<@0oooo000K0?ooo`800000?@3oool00`00
0000oooo0?ooo`330?ooo`006`3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0<80
oooo000K0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0`@3oool001/0oooo00<0
00000?ooo`3oool0?`3oool00`000000oooo0?ooo`300?ooo`006`3oool00`000000oooo0?ooo`10
0?ooo`030000003oool0oooo0;l0oooo00080?ooo`800000103oool00`000000oooo0?ooo`030?oo
o`8000001@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0;h0oooo00070?ooo`04
0000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`12
0?ooo`030000003oool0oooo0;d0oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool2000004@0oooo0P00002m0?ooo`0000<0oooo0000000000000P000002
0?ooo`040000003oool0oooo000000T0oooo0P0000050?ooo`030000003oool0oooo04D0oooo00<0
00000?ooo`3oool0^P3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0^@3oool000L0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo04L0
oooo00<000000?ooo`3oool0^03oool000P0oooo0P00000:0?ooo`8000001@3oool00`000000oooo
0?ooo`180?ooo`030000003oool0oooo0;L0oooo000K0?ooo`030000003oool0oooo04T0oooo00<0
00000?ooo`3oool0]P3oool001/0oooo0P00001;0?ooo`030000003oool0oooo0;D0oooo000K0?oo
o`030000003oool0oooo04/0oooo00<000000?ooo`3oool0]03oool001/0oooo00<000000?ooo`3o
ool0C03oool00`000000oooo0?ooo`2c0?ooo`006`3oool00`000000oooo0?ooo`1=0?ooo`030000
003oool0oooo0;80oooo000K0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0/@3o
ool001/0oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`2`0?ooo`006`3oool00`00
0000oooo0?ooo`1@0?ooo`030000003oool0oooo0:l0oooo000K0?ooo`800000DP3oool00`000000
oooo0?ooo`2^0?ooo`006`3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo0:d0oooo
000K0?ooo`030000003oool0oooo05<0oooo0P00002]0?ooo`006`3oool00`000000oooo0?ooo`1E
0?ooo`030000003oool0oooo0:X0oooo000K0?ooo`030000003oool0oooo05H0oooo00<000000?oo
o`3oool0Z@3oool001/0oooo00<000000?ooo`3oool0E`3oool200000:T0oooo000K0?ooo`030000
003oool0oooo05T0oooo00<000000?ooo`3oool0YP3oool001/0oooo0P00001K0?ooo`030000003o
ool0oooo0:D0oooo000K0?ooo`030000003oool0oooo05/0oooo0P00002U0?ooo`006`3oool00`00
0000oooo0?ooo`1M0?ooo`030000003oool0oooo0:80oooo000K0?ooo`030000003oool0oooo05h0
oooo00<000000?ooo`3oool0X@3oool001/0oooo00<000000?ooo`3oool0G`3oool00`000000oooo
0?ooo`2P0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<0
00000?ooo`3oool0H03oool00`000000oooo0?ooo`2O0?ooo`001`3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0H@3oool00`000000oooo
0?ooo`2N0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0
oooo0P00001S0?ooo`800000WP3oool000030?ooo`0000000000008000000P3oool010000000oooo
0?ooo`0000080?ooo`<000001@3oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo09/0
oooo00070?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`1U0?ooo`030000003oool0oooo09X0oooo00070?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`1V0?ooo`030000003oool0oooo09T0
oooo00080?ooo`8000002P3oool3000000@0oooo00<000000?ooo`3oool0I`3oool00`000000oooo
0?ooo`2H0?ooo`006`3oool00`000000oooo0?ooo`1X0?ooo`800000V03oool001/0oooo0P00001[
0?ooo`030000003oool0oooo09D0oooo000K0?ooo`030000003oool0oooo06/0oooo0P00002E0?oo
o`006`3oool00`000000oooo0?ooo`1]0?ooo`800000T`3oool001/0oooo00<000000?ooo`3oool0
K`3oool00`000000oooo0?ooo`2@0?ooo`006`3oool00`000000oooo0?ooo`1`0?ooo`800000T03o
ool001/0oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`2=0?ooo`006`3oool00`00
0000oooo0?ooo`1c0?ooo`030000003oool0oooo08`0oooo000K0?ooo`800000M@3oool2000008`0
oooo000K0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool0R@3oool001/0oooo00<0
00000?ooo`3oool0M`3oool00`000000oooo0?ooo`280?ooo`006`3oool00`000000oooo0?ooo`1h
0?ooo`800000R03oool001/0oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`250?oo
o`006`3oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo08@0oooo000K0?ooo`030000
003oool0oooo07`0oooo0P0000240?ooo`006`3oool2000007l0oooo0P0000220?ooo`006`3oool0
0`000000oooo0?ooo`200?ooo`800000P03oool001/0oooo00<000000?ooo`3oool0PP3oool20000
07h0oooo000K0?ooo`030000003oool0oooo08@0oooo0P00001l0?ooo`006`3oool00`000000oooo
0?ooo`260?ooo`030000003oool0oooo07T0oooo00080?ooo`800000103oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0Q`3oool2000007T0oooo0007
0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`290?ooo`800000M`3oool000L0oooo00@000000?ooo`3oool000001`3oool5000000@0oooo0P00
002<0?ooo`030000003oool0oooo07@0oooo00000`3oool000000000000200000080oooo00@00000
0?ooo`3oool000001`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo08`0oooo
0P00001d0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0000000D0oooo
00<000000?ooo`3oool0SP3oool200000780oooo00070?ooo`040000003oool0oooo000000T0oooo
0P0000050?ooo`030000003oool0oooo0900oooo0P00001`0?ooo`00203oool2000000/0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`2B0?ooo`800000KP3oool001/0oooo00<00000
0?ooo`3oool0U03oool2000006`0oooo000K0?ooo`800000U`3oool2000006X0oooo000K0?ooo`03
0000003oool0oooo09P0oooo0P00001X0?ooo`006`3oool00`000000oooo0?ooo`2J0?ooo`800000
IP3oool001/0oooo00<000000?ooo`3oool0W03oool2000006@0oooo000K0?ooo`030000003oool0
oooo09h0oooo0P00001R0?ooo`006`3oool00`000000oooo0?ooo`2P0?ooo`800000H03oool001/0
oooo00<000000?ooo`3oool0XP3oool2000005h0oooo000K0?ooo`800000Y@3oool3000005/0oooo
000K0?ooo`030000003oool0oooo0:L0oooo0P00001I0?ooo`006`3oool00`000000oooo0?ooo`2Y
0?ooo`<00000EP3oool001/0oooo00<000000?ooo`3oool0[03oool2000005@0oooo000K0?ooo`03
0000003oool0oooo0:h0oooo0`00001A0?ooo`006`3oool00`000000oooo0?ooo`2a0?ooo`<00000
CP3oool001/0oooo00<000000?ooo`3oool0]03oool2000004`0oooo000K0?ooo`800000]`3oool3
000004T0oooo000K0?ooo`030000003oool0oooo0;T0oooo0`0000160?ooo`006`3oool00`000000
oooo0?ooo`2l0?ooo`<00000@`3oool001/0oooo00<000000?ooo`3oool0_`3oool4000003l0oooo
000K0?ooo`030000003oool0oooo0<<0oooo0`00000l0?ooo`00203oool2000000@0oooo00<00000
0?ooo`3oool00P3oool4000000@0oooo00<000000?ooo`3oool0aP3oool3000003T0oooo00070?oo
o`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`39
0?ooo`@00000=@3oool000L0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`04
0?ooo`800000cP3oool300000380oooo00000`3oool000000000000200000080oooo00@000000?oo
o`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0=00oooo1000000^
0?ooo`001`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0e03oool4000002X0oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`3H0?ooo`@000009P3oool000P0oooo0P00000:
0?ooo`8000001@3oool00`000000oooo0?ooo`3L0?ooo`D000008@3oool001/0oooo00<000000?oo
o`3oool0h@3oool4000001d0oooo000K0?ooo`800000iP3oool4000001T0oooo000K0?ooo`030000
003oool0oooo0>T0oooo1000000E0?ooo`006`3oool00`000000oooo0?ooo`3]0?ooo`@000004@3o
ool001/0oooo00<000000?ooo`3oool0l@3oool5000000`0oooo000K0?ooo`030000003oool0oooo
0?H0oooo0`0000090?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo0P00003o0?ooo`@0oooo000K0?ooo`030000003oool0
oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool00`000000
oooo0?ooo`0d0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000<03oool2000000@0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0_0?ooo`800000103oool00`000000
oooo0?ooo`030?ooo`800000<@3oool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0
oooo000K0?ooo`030000003oool0oooo03<0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`0`0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0;P3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000002l0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`0000020?ooo`006`3oool00`000000oooo0?ooo`0c0?ooo`040000
003oool0oooo000000T0oooo00<000000?ooo`3oool0;`3oool010000000oooo0?ooo`0000070?oo
o`D00000;`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002l0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`006`3oool00`000000oooo
0?ooo`0c0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0;P3oool010000000
oooo0?ooo`0000070?ooo`040000003oool0oooo00000300oooo00@000000?ooo`3oool00000203o
ool300000300oooo00@000000?ooo`3oool000002@3oool2000000<0oooo000K0?ooo`800000=03o
ool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo02d0oooo00@000000?ooo`3oool0
0000203oool00`000000oooo0000000`0?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`3oool0<03oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo000K
0?ooo`030000003oool0oooo03<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`00000_0?ooo`040000003oool0oooo000000T0oooo0P00000`0?ooo`040000003oool0oooo0000
00P0oooo00<000000?ooo`3oool0<03oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo000K0?ooo`030000003oool0oooo03@0oooo0P00000:0?ooo`800000<@3oool2
000000/0oooo00<000000?ooo`3oool0;`3oool2000000X0oooo0`00000`0?ooo`8000002P3oool2
000000<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3o
ool0o`3oool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003o
ool0oooo0?l0oooo0`3oool001D0ooooo`000009000000<0oooo000K0?ooo`030000003oool0oooo
03/0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo
03`0oooo00<000000?ooo`3oool01`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`00
6`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?oo
ob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0870867, -1.23026, \
0.00319096, 0.00741377}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{th, alomz[3, Cos[th], 1, 10]}, {th, 0.01, Pi/2, 
        0.01}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, \(-3.4461925863793876`\)}, {0.02`, \(-3.437311859788748`\)}, \
{0.03`, \(-3.4226118812507638`\)}, {0.04`, \(-3.402241660964136`\)}, {0.05`, \
\(-3.37640428463936`\)}, {0.06`, \(-3.3453517187095407`\)}, {0.07`, \
\(-3.3093785776764624`\)}, {0.08`, \(-3.2688151389852687`\)}, {0.09`, \
\(-3.2240199065509634`\)}, {0.1`, \(-3.175372019889019`\)}, {0.11`, \
\(-3.123263784135687`\)}, {0.12`, \(-3.0680935608037267`\)}, {0.13`, \
\(-3.0102592143405995`\)}, {0.14`, \(-2.950152260010975`\)}, {0.15`, \
\(-2.888152808521608`\)}, {0.16`, \(-2.824625355637589`\)}, {0.17`, \
\(-2.7599154233830188`\)}, {0.18`, \(-2.694347024890442`\)}, {0.19`, \
\(-2.6282208982820143`\)}, {0.2`, \(-2.5618134361026783`\)}, {0.21`, \
\(-2.4953762251928913`\)}, {0.22`, \(-2.429136106528485`\)}, {0.23`, \
\(-2.36329566431993`\)}, {0.24`, \(-2.298034057356002`\)}, {0.25`, \
\(-2.2335081120589164`\)}, {0.26`, \(-2.1698536049776767`\)}, {0.27`, \
\(-2.107186671637062`\)}, {0.28`, \(-2.045605288109985`\)}, {0.29`, \
\(-1.9851907808899782`\)}, {0.3`, \(-1.9260093292600622`\)}, {0.31`, \
\(-1.8681134321648527`\)}, {0.32`, \(-1.8115433184779137`\)}, {0.33`, \
\(-1.7563282854800157`\)}, {0.34`, \(-1.7024879553450272`\)}, \
{0.35000000000000003`, \(-1.6500334435277826`\)}, {0.36`, \
\(-1.5989684362433207`\)}, {0.37`, \(-1.5492901768135807`\)}, {0.38`, \
\(-1.500990362632084`\)}, {0.39`, \(-1.4540559559545085`\)}, {0.4`, \
\(-1.4084699127508253`\)}, {0.41000000000000003`, \(-1.3642118345331802`\)}, \
{0.42`, \(-1.3212585484726658`\)}, {0.43`, \(-1.2795846212985356`\)}, {0.44`, \
\(-1.2391628124862157`\)}, {0.45`, \(-1.199964472128534`\)}, {0.46`, \
\(-1.161959888682649`\)}, {0.47000000000000003`, \(-1.1251185915215711`\)}, \
{0.48`, \(-1.0894096129159636`\)}, {0.49`, \(-1.0548017137468353`\)}, {0.5`, \
\(-1.0212635769157847`\)}, {0.51`, \(-0.9887639720868672`\)}, {0.52`, \
\(-0.9572718950699831`\)}, {0.53`, \(-0.9267566848451405`\)}, {0.54`, \
\(-0.8971881209333267`\)}, {0.55`, \(-0.8685365035452021`\)}, {0.56`, \
\(-0.84077271868425`\)}, {0.5700000000000001`, \(-0.8138682901468968`\)}, \
{0.58`, \(-0.7877954201480094`\)}, {0.59`, \(-0.7625270201054785`\)}, {0.6`, \
\(-0.7380367329414151`\)}, {0.61`, \(-0.7142989480986202`\)}, {0.62`, \
\(-0.6912888103283497`\)}, {0.63`, \(-0.668982223177652`\)}, {0.64`, \
\(-0.6473558479905224`\)}, {0.65`, \(-0.6263870991355814`\)}, {0.66`, \
\(-0.606054136082753`\)}, {0.67`, \(-0.5863358528714574`\)}, {0.68`, \
\(-0.5672118654420413`\)}, {0.6900000000000001`, \(-0.5486624972396968`\)}, \
{0.7000000000000001`, \(-0.5306687634449964`\)}, {0.71`, \
\(-0.5132123541366733`\)}, {0.72`, \(-0.4962756166496535`\)}, {0.73`, \
\(-0.4798415373539328`\)}, {0.74`, \(-0.46389372304712945`\)}, {0.75`, \
\(-0.4484163821248554`\)}, {0.76`, \(-0.43339430566802045`\)}, {0.77`, \
\(-0.41881284856432816`\)}, {0.78`, \(-0.40465791076220414`\)}, {0.79`, \
\(-0.3909159187388646`\)}, {0.8`, \(-0.3775738072498834`\)}, {0.81`, \
\(-0.3646190014151913`\)}, {0.8200000000000001`, \(-0.35203939918570076`\)}, \
{0.8300000000000001`, \(-0.3398233542254897`\)}, {0.84`, \
\(-0.32795965923651266`\)}, {0.85`, \(-0.31643752974597383`\)}, {0.86`, \
\(-0.3052465883706599`\)}, {0.87`, \(-0.29437684956754473`\)}, {0.88`, \
\(-0.2838187048757644`\)}, {0.89`, \(-0.2735629086514852`\)}, {0.9`, \
\(-0.26360056429419804`\)}, {0.91`, \(-0.2539231109604702`\)}, {0.92`, \
\(-0.24452231075911135`\)}, {0.93`, \(-0.235390236420023`\)}, \
{0.9400000000000001`, \(-0.2265192594276121`\)}, {0.9500000000000001`, \
\(-0.21790203860856378`\)}, {0.96`, \(-0.20953150916289642`\)}, {0.97`, \
\(-0.2014008721265793`\)}, {0.98`, \(-0.19350358425350656`\)}, {0.99`, \
\(-0.18583334830429313`\)}, {1.`, \(-0.17838410372916014`\)}, {1.01`, \
\(-0.1711500177320788`\)}, {1.02`, \(-0.1641254767033433`\)}, {1.03`, \
\(-0.15730507800780993`\)}, {1.04`, \(-0.15068362211618366`\)}, {1.05`, \
\(-0.14425610506691555`\)}, {1.06`, \(-0.13801771124650908`\)}, {1.07`, \
\(-0.1319638064762978`\)}, {1.08`, \(-0.12608993139405045`\)}, {1.09`, \
\(-0.12039179511907354`\)}, {1.1`, \(-0.11486526918981116`\)}, {1.11`, \
\(-0.10950638176328481`\)}, {1.12`, \(-0.10431131206606373`\)}, \
{1.1300000000000001`, \(-0.09927638508681187`\)}, {1.1400000000000001`, \
\(-0.09439806650081198`\)}, {1.1500000000000001`, \(-0.08967295781722365`\)}, \
{1.16`, \(-0.08509779174018364`\)}, {1.17`, \(-0.0806694277352069`\)}, \
{1.18`, \(-0.07638484779269065`\)}, {1.19`, \(-0.07224115238065819`\)}, \
{1.2`, \(-0.06823555657921367`\)}, {1.21`, \(-0.06436538638949783`\)}, \
{1.22`, \(-0.06062807521025114`\)}, {1.23`, \(-0.05702116047539479`\)}, \
{1.24`, \(-0.05354228044633831`\)}, {1.25`, \(-0.05018917115300851`\)}, \
{1.26`, \(-0.04695966347787525`\)}, {1.27`, \(-0.043851680377517355`\)}, \
{1.28`, \(-0.04086323423653422`\)}, {1.29`, \(-0.03799242434886009`\)}, \
{1.3`, \(-0.03523743452178111`\)}, {1.31`, \(-0.03259653079819067`\)}, \
{1.32`, \(-0.0300680592928452`\)}, {1.33`, \(-0.02765044413860076`\)}, \
{1.34`, \(-0.025342185538822604`\)}, {1.35`, \(-0.02314185792236269`\)}, \
{1.36`, \(-0.02104810819769728`\)}, {1.37`, \(-0.019059654103005818`\)}, \
{1.3800000000000001`, \(-0.017175282649155806`\)}, {1.3900000000000001`, \
\(-0.015393848652734748`\)}, {1.4000000000000001`, \
\(-0.013714273356441702`\)}, {1.41`, \(-0.012135543134315795`\)}, {1.42`, \
\(-0.010656708279439117`\)}, {1.43`, \(-0.009276881871906853`\)}, {1.44`, \
\(-0.00799523872500669`\)}, {1.45`, \(-0.006811014407696433`\)}, {1.46`, \
\(-0.005723504341609748`\)}, {1.47`, \(-0.004732062970958055`\)}, {1.48`, \
\(-0.003836103003830726`\)}, {1.49`, \(-0.0030350947235268867`\)}, {1.5`, \
\(-0.0023285653686799536`\)}, {1.51`, \(-0.001716098581061479`\)}, {1.52`, \
\(-0.001197333920073547`\)}, {1.53`, \(-0.0007719664430596968`\)}, {1.54`, \
\(-0.0004397463506830135`\)}, {1.55`, \(-0.0002004786967370549`\)}, {1.56`, \
\(-0.00005402316187094679`\)}, {1.57`, \(-2.9389082443590493`*^-7\)}}\)], \
"Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[alomz[1, Cos[th], 1, \(-9\)/10], {th, 0.01, Pi/2}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606305 0.0147151 0.891189 [
[.17539 .00222 -12 -9 ]
[.17539 .00222 12 0 ]
[.32696 .00222 -9 -9 ]
[.32696 .00222 9 0 ]
[.47854 .00222 -12 -9 ]
[.47854 .00222 12 0 ]
[.63011 .00222 -3 -9 ]
[.63011 .00222 3 0 ]
[.78169 .00222 -12 -9 ]
[.78169 .00222 12 0 ]
[.93327 .00222 -9 -9 ]
[.93327 .00222 9 0 ]
[.01131 .10383 -18 -4.5 ]
[.01131 .10383 0 4.5 ]
[.01131 .19295 -18 -4.5 ]
[.01131 .19295 0 4.5 ]
[.01131 .28207 -18 -4.5 ]
[.01131 .28207 0 4.5 ]
[.01131 .37119 -18 -4.5 ]
[.01131 .37119 0 4.5 ]
[.01131 .46031 -18 -4.5 ]
[.01131 .46031 0 4.5 ]
[.01131 .54943 -18 -4.5 ]
[.01131 .54943 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .01472 m
.17539 .02097 L
s
[(0.25)] .17539 .00222 0 1 Mshowa
.32696 .01472 m
.32696 .02097 L
s
[(0.5)] .32696 .00222 0 1 Mshowa
.47854 .01472 m
.47854 .02097 L
s
[(0.75)] .47854 .00222 0 1 Mshowa
.63011 .01472 m
.63011 .02097 L
s
[(1)] .63011 .00222 0 1 Mshowa
.78169 .01472 m
.78169 .02097 L
s
[(1.25)] .78169 .00222 0 1 Mshowa
.93327 .01472 m
.93327 .02097 L
s
[(1.5)] .93327 .00222 0 1 Mshowa
.125 Mabswid
.05412 .01472 m
.05412 .01847 L
s
.08444 .01472 m
.08444 .01847 L
s
.11476 .01472 m
.11476 .01847 L
s
.14507 .01472 m
.14507 .01847 L
s
.2057 .01472 m
.2057 .01847 L
s
.23602 .01472 m
.23602 .01847 L
s
.26633 .01472 m
.26633 .01847 L
s
.29665 .01472 m
.29665 .01847 L
s
.35728 .01472 m
.35728 .01847 L
s
.38759 .01472 m
.38759 .01847 L
s
.41791 .01472 m
.41791 .01847 L
s
.44822 .01472 m
.44822 .01847 L
s
.50885 .01472 m
.50885 .01847 L
s
.53917 .01472 m
.53917 .01847 L
s
.56948 .01472 m
.56948 .01847 L
s
.5998 .01472 m
.5998 .01847 L
s
.66043 .01472 m
.66043 .01847 L
s
.69074 .01472 m
.69074 .01847 L
s
.72106 .01472 m
.72106 .01847 L
s
.75138 .01472 m
.75138 .01847 L
s
.81201 .01472 m
.81201 .01847 L
s
.84232 .01472 m
.84232 .01847 L
s
.87264 .01472 m
.87264 .01847 L
s
.90295 .01472 m
.90295 .01847 L
s
.96358 .01472 m
.96358 .01847 L
s
.9939 .01472 m
.9939 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .10383 m
.03006 .10383 L
s
[(0.1)] .01131 .10383 1 0 Mshowa
.02381 .19295 m
.03006 .19295 L
s
[(0.2)] .01131 .19295 1 0 Mshowa
.02381 .28207 m
.03006 .28207 L
s
[(0.3)] .01131 .28207 1 0 Mshowa
.02381 .37119 m
.03006 .37119 L
s
[(0.4)] .01131 .37119 1 0 Mshowa
.02381 .46031 m
.03006 .46031 L
s
[(0.5)] .01131 .46031 1 0 Mshowa
.02381 .54943 m
.03006 .54943 L
s
[(0.6)] .01131 .54943 1 0 Mshowa
.125 Mabswid
.02381 .03254 m
.02756 .03254 L
s
.02381 .05036 m
.02756 .05036 L
s
.02381 .06819 m
.02756 .06819 L
s
.02381 .08601 m
.02756 .08601 L
s
.02381 .12166 m
.02756 .12166 L
s
.02381 .13948 m
.02756 .13948 L
s
.02381 .15731 m
.02756 .15731 L
s
.02381 .17513 m
.02756 .17513 L
s
.02381 .21078 m
.02756 .21078 L
s
.02381 .2286 m
.02756 .2286 L
s
.02381 .24642 m
.02756 .24642 L
s
.02381 .26425 m
.02756 .26425 L
s
.02381 .2999 m
.02756 .2999 L
s
.02381 .31772 m
.02756 .31772 L
s
.02381 .33554 m
.02756 .33554 L
s
.02381 .35337 m
.02756 .35337 L
s
.02381 .38901 m
.02756 .38901 L
s
.02381 .40684 m
.02756 .40684 L
s
.02381 .42466 m
.02756 .42466 L
s
.02381 .44249 m
.02756 .44249 L
s
.02381 .47813 m
.02756 .47813 L
s
.02381 .49596 m
.02756 .49596 L
s
.02381 .51378 m
.02756 .51378 L
s
.02381 .5316 m
.02756 .5316 L
s
.02381 .56725 m
.02756 .56725 L
s
.02381 .58508 m
.02756 .58508 L
s
.02381 .6029 m
.02756 .6029 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02987 .60332 m
.0321 .60331 L
.03449 .60329 L
.03879 .60325 L
.04391 .60318 L
.04856 .60311 L
.05869 .60289 L
.06826 .60261 L
.08834 .60181 L
.11013 .60059 L
.12984 .59919 L
.15074 .59737 L
.18833 .59321 L
.22836 .58748 L
.26689 .58059 L
.30785 .57167 L
.3473 .56134 L
.38525 .54962 L
.42564 .53498 L
.46452 .51848 L
.50584 .49798 L
.54565 .47486 L
.58395 .44902 L
.6247 .41704 L
.66393 .38119 L
.70166 .34147 L
.74183 .29304 L
.78049 .24049 L
.82159 .17954 L
.86118 .1195 L
.89927 .06707 L
.91853 .04543 L
.92876 .03589 L
.93979 .02739 L
.94958 .02156 L
.95387 .01955 L
.95856 .01774 L
.96301 .01641 L
.96715 .01551 L
.96944 .01516 L
.97152 .01493 L
.97267 .01484 L
.97392 .01477 L
.9751 .01473 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00=03oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080
oooo0`00000H0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<000006@3oool2000000@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`<000007@3oool4000001`0
oooo100000030?ooo`030000003oool0oooo0080oooo100000020?ooo`<000005`3oool4000000<0
oooo00<000000?ooo`3oool00P3oool3000000d0oooo000c0?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo0000
00/0oooo00<000000?ooo`3oool05@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02P3oool003<0oooo00@000000?ooo`3oool000002@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo00@000000?ooo`3oool000002`3o
ool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`00<`3oool010000000oooo0?ooo`00000:0?ooo`040000
003oool0oooo0?ooo`<000005`3oool010000000oooo0?ooo`0000080?ooo`<00000603oool01000
0000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000007`3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool0oooo0`00000I0?ooo`030000
003oool0oooo00L0oooo0`00000=0?ooo`00<`3oool010000000oooo0?ooo`00000;0?ooo`040000
003oool0oooo000001T0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0H0?oo
o`040000003oool0oooo000000/0oooo00@000000?ooo`3oool000008@3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00X0oooo00@000000?ooo`3oool000006`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00d0oooo000c0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0G0?ooo`040000003oool0oooo0000
00P0oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool07@3oool3000001d0oooo0`0000090?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool05`3oool3000000T0oooo00<000000?ooo`3oool0
3@3oool001D0oooo00<000000?ooo`3oool0703oool2000000X0oooo0P0000030?ooo`@000005`3o
ool2000000T0oooo1000000H0?ooo`8000002@3oool400000080oooo1000000N0?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool0203oool2000000<0oooo1000000H0?ooo`030000003o
ool0oooo00L0oooo1000000<0?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?oo
o`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`00
3P3ooooo000001<00000000E0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo100000070?ooo`4000000@3o
ool10?ooo`005@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo000E0?oo
o`030000003oool0oooo0?L0oooo0P00000?0?ooo`005@3oool00`000000oooo0?ooo`3f0?ooo`03
0000003oool0oooo00l0oooo000E0?ooo`800000m@3oool200000180oooo000E0?ooo`030000003o
ool0oooo0?<0oooo00<000000?ooo`3oool04P3oool001D0oooo00<000000?ooo`3oool0lP3oool0
0`000000oooo0?ooo`0C0?ooo`005@3oool00`000000oooo0?ooo`3a0?ooo`030000003oool0oooo
01@0oooo000E0?ooo`030000003oool0oooo0?00oooo00<000000?ooo`3oool05@3oool001D0oooo
0P00003`0?ooo`030000003oool0oooo01H0oooo000E0?ooo`030000003oool0oooo0>h0oooo00<0
00000?ooo`3oool05`3oool001D0oooo00<000000?ooo`3oool0k@3oool00`000000oooo0?ooo`0H
0?ooo`005@3oool00`000000oooo0?ooo`3/0?ooo`030000003oool0oooo01T0oooo000E0?ooo`03
0000003oool0oooo0>/0oooo00<000000?ooo`3oool06P3oool001D0oooo0P00003[0?ooo`030000
003oool0oooo01/0oooo000E0?ooo`030000003oool0oooo0>X0oooo00<000000?ooo`3oool06`3o
ool001D0oooo00<000000?ooo`3oool0j@3oool00`000000oooo0?ooo`0L0?ooo`005@3oool00`00
0000oooo0?ooo`3X0?ooo`030000003oool0oooo01d0oooo000E0?ooo`030000003oool0oooo0>P0
oooo00<000000?ooo`3oool07@3oool001D0oooo0P00003X0?ooo`030000003oool0oooo01h0oooo
000E0?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool07`3oool00080oooo0P000004
0?ooo`030000003oool0oooo0080oooo100000040?ooo`030000003oool0oooo0>H0oooo00<00000
0?ooo`3oool07`3oool000050?ooo`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0>D0oooo00<000000?ooo`3oool0803oool000050?ooo`000000oooo
0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`<00000i03oool00`000000oooo0?ooo`0Q
0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0i03oool00`000000oooo0?ooo`0Q0?ooo`0000D0oooo0000003oool0oooo0000000:
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0h`3oool00`000000oooo0?ooo`0R
0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`000000oooo0?ooo`3R
0?ooo`030000003oool0oooo02<0oooo00020?ooo`8000002`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0>80oooo00<000000?ooo`3oool08`3oool001D0oooo0P00003R0?ooo`03
0000003oool0oooo02@0oooo000E0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
9@3oool001D0oooo00<000000?ooo`3oool0h03oool00`000000oooo0?ooo`0U0?ooo`005@3oool0
0`000000oooo0?ooo`3O0?ooo`030000003oool0oooo02H0oooo000E0?ooo`030000003oool0oooo
0=h0oooo00<000000?ooo`3oool09`3oool001D0oooo0P00003O0?ooo`030000003oool0oooo02L0
oooo000E0?ooo`030000003oool0oooo0=d0oooo00<000000?ooo`3oool0:03oool001D0oooo00<0
00000?ooo`3oool0g03oool00`000000oooo0?ooo`0Y0?ooo`005@3oool00`000000oooo0?ooo`3L
0?ooo`030000003oool0oooo02T0oooo000E0?ooo`030000003oool0oooo0=/0oooo00<000000?oo
o`3oool0:P3oool001D0oooo0P00003K0?ooo`030000003oool0oooo02/0oooo000E0?ooo`030000
003oool0oooo0=T0oooo00<000000?ooo`3oool0;03oool001D0oooo00<000000?ooo`3oool0f@3o
ool00`000000oooo0?ooo`0/0?ooo`005@3oool00`000000oooo0?ooo`3H0?ooo`030000003oool0
oooo02d0oooo000E0?ooo`030000003oool0oooo0=L0oooo00<000000?ooo`3oool0;P3oool001D0
oooo0P00003H0?ooo`030000003oool0oooo02h0oooo000E0?ooo`030000003oool0oooo0=H0oooo
00<000000?ooo`3oool0;`3oool001D0oooo00<000000?ooo`3oool0e@3oool00`000000oooo0?oo
o`0`0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000@0oooo00<00000
0?ooo`3oool0e@3oool00`000000oooo0?ooo`0`0?ooo`0000D0oooo0000003oool0oooo00000008
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0e03oool00`000000oooo0?ooo`0a
0?ooo`0000D0oooo0000003oool0oooo000000090?ooo`030000003oool0oooo00@0oooo0`00003C
0?ooo`030000003oool0oooo0380oooo00001@3oool000000?ooo`3oool0000000X0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`3C0?ooo`030000003oool0oooo0380oooo00001@3o
ool000000?ooo`3oool0000000/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3B
0?ooo`030000003oool0oooo03<0oooo00001@3oool000000?ooo`3oool0000000P0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`3A0?ooo`030000003oool0oooo03@0oooo0002
0?ooo`8000002P3oool2000000D0oooo00<000000?ooo`3oool0d@3oool00`000000oooo0?ooo`0d
0?ooo`005@3oool200000=40oooo00<000000?ooo`3oool0=@3oool001D0oooo00<000000?ooo`3o
ool0d03oool00`000000oooo0?ooo`0e0?ooo`005@3oool00`000000oooo0?ooo`3?0?ooo`030000
003oool0oooo03H0oooo000E0?ooo`030000003oool0oooo0<h0oooo00<000000?ooo`3oool0=`3o
ool001D0oooo0P00003?0?ooo`030000003oool0oooo03L0oooo000E0?ooo`030000003oool0oooo
0<d0oooo00<000000?ooo`3oool0>03oool001D0oooo00<000000?ooo`3oool0c03oool00`000000
oooo0?ooo`0i0?ooo`005@3oool00`000000oooo0?ooo`3<0?ooo`030000003oool0oooo03T0oooo
000E0?ooo`030000003oool0oooo0</0oooo00<000000?ooo`3oool0>P3oool001D0oooo0P00003;
0?ooo`030000003oool0oooo03/0oooo000E0?ooo`030000003oool0oooo0<T0oooo00<000000?oo
o`3oool0?03oool001D0oooo00<000000?ooo`3oool0b@3oool00`000000oooo0?ooo`0l0?ooo`00
5@3oool00`000000oooo0?ooo`380?ooo`030000003oool0oooo03d0oooo000E0?ooo`030000003o
ool0oooo0<L0oooo00<000000?ooo`3oool0?P3oool001D0oooo0P0000370?ooo`030000003oool0
oooo03l0oooo000E0?ooo`030000003oool0oooo0<H0oooo00<000000?ooo`3oool0?`3oool001D0
oooo00<000000?ooo`3oool0a@3oool00`000000oooo0?ooo`100?ooo`000P3oool2000000@0oooo
00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3oool0a03oool00`000000oooo
0?ooo`110?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000003oool0oooo000000@0
oooo00<000000?ooo`3oool0``3oool00`000000oooo0?ooo`120?ooo`0000D0oooo0000003oool0
oooo0000000;0?ooo`030000003oool0oooo0080oooo0`0000320?ooo`030000003oool0oooo04<0
oooo00001@3oool000000?ooo`3oool0000000T0oooo0P0000050?ooo`030000003oool0oooo0<80
oooo00<000000?ooo`3oool0@`3oool000050?ooo`000000oooo0?ooo`0000002`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0<40oooo00<000000?ooo`3oool0A03oool000050?oo
o`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo
0<00oooo00<000000?ooo`3oool0A@3oool00080oooo0P00000:0?ooo`8000001@3oool00`000000
oooo0?ooo`2o0?ooo`030000003oool0oooo04H0oooo000E0?ooo`800000_`3oool00`000000oooo
0?ooo`170?ooo`005@3oool00`000000oooo0?ooo`2m0?ooo`030000003oool0oooo04P0oooo000E
0?ooo`030000003oool0oooo0;d0oooo00<000000?ooo`3oool0B03oool001D0oooo00<000000?oo
o`3oool0_03oool00`000000oooo0?ooo`190?ooo`005@3oool00`000000oooo0?ooo`2k0?ooo`03
0000003oool0oooo04X0oooo000E0?ooo`800000^`3oool00`000000oooo0?ooo`1;0?ooo`005@3o
ool00`000000oooo0?ooo`2i0?ooo`030000003oool0oooo04`0oooo000E0?ooo`030000003oool0
oooo0;P0oooo00<000000?ooo`3oool0C@3oool001D0oooo00<000000?ooo`3oool0^03oool00`00
0000oooo0?ooo`1=0?ooo`005@3oool200000;P0oooo00<000000?ooo`3oool0CP3oool001D0oooo
00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`1?0?ooo`005@3oool00`000000oooo0?oo
o`2e0?ooo`030000003oool0oooo0500oooo000E0?ooo`030000003oool0oooo0;@0oooo00<00000
0?ooo`3oool0D@3oool001D0oooo00<000000?ooo`3oool0/`3oool00`000000oooo0?ooo`1B0?oo
o`005@3oool200000;<0oooo00<000000?ooo`3oool0D`3oool001D0oooo00<000000?ooo`3oool0
/@3oool00`000000oooo0?ooo`1D0?ooo`005@3oool00`000000oooo0?ooo`2`0?ooo`030000003o
ool0oooo05D0oooo00020?ooo`800000103oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0/03oool00`000000oooo0?ooo`1E0?ooo`0000D0oooo0000
003oool0oooo0000000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0[`3oool0
0`000000oooo0?ooo`1F0?ooo`0000D0oooo0000003oool0oooo000000070?ooo`D00000103oool3
00000:h0oooo00<000000?ooo`3oool0E`3oool000050?ooo`000000oooo0?ooo`0000001`3oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo0:d0oooo00<000000?ooo`3oool0F03o
ool000050?ooo`000000oooo0?ooo`000000203oool00`000000oooo000000050?ooo`030000003o
ool0oooo0:`0oooo00<000000?ooo`3oool0F@3oool000050?ooo`000000oooo0?ooo`0000002@3o
ool2000000D0oooo00<000000?ooo`3oool0Z`3oool00`000000oooo0?ooo`1J0?ooo`000P3oool2
000000/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2Z0?ooo`030000003oool0
oooo05/0oooo000E0?ooo`800000ZP3oool00`000000oooo0?ooo`1L0?ooo`005@3oool00`000000
oooo0?ooo`2X0?ooo`030000003oool0oooo05d0oooo000E0?ooo`030000003oool0oooo0:L0oooo
00<000000?ooo`3oool0GP3oool001D0oooo00<000000?ooo`3oool0Y@3oool200000640oooo000E
0?ooo`030000003oool0oooo0:@0oooo00<000000?ooo`3oool0H@3oool001D0oooo0P00002T0?oo
o`030000003oool0oooo0680oooo000E0?ooo`030000003oool0oooo0:80oooo00<000000?ooo`3o
ool0H`3oool001D0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`1T0?ooo`005@3o
ool00`000000oooo0?ooo`2P0?ooo`030000003oool0oooo06D0oooo000E0?ooo`030000003oool0
oooo09h0oooo0P00001X0?ooo`005@3oool2000009h0oooo00<000000?ooo`3oool0J03oool001D0
oooo00<000000?ooo`3oool0W03oool00`000000oooo0?ooo`1Y0?ooo`005@3oool00`000000oooo
0?ooo`2J0?ooo`800000K03oool001D0oooo00<000000?ooo`3oool0V@3oool00`000000oooo0?oo
o`1/0?ooo`005@3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo06d0oooo000E0?oo
o`800000U`3oool200000700oooo000E0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3o
ool0L03oool00080oooo0P0000040?ooo`030000003oool0oooo0080oooo0`0000050?ooo`030000
003oool0oooo09<0oooo0P00001c0?ooo`0000D0oooo0000003oool0oooo0000000;0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0TP3oool00`000000oooo0?ooo`1c0?ooo`0000D0
oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo0`00002@0?ooo`800000
MP3oool000050?ooo`000000oooo0?ooo`000000203oool3000000D0oooo00<000000?ooo`3oool0
S`3oool00`000000oooo0?ooo`1f0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool0SP3oool00`000000oooo0?ooo`1g0?ooo`0000D0
oooo0000003oool0oooo000000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
S03oool2000007X0oooo00020?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0R`3o
ool00`000000oooo0?ooo`1j0?ooo`005@3oool2000008X0oooo0P00001m0?ooo`005@3oool00`00
0000oooo0?ooo`280?ooo`030000003oool0oooo07d0oooo000E0?ooo`030000003oool0oooo08H0
oooo0P0000200?ooo`005@3oool00`000000oooo0?ooo`240?ooo`800000PP3oool001D0oooo00<0
00000?ooo`3oool0P`3oool00`000000oooo0?ooo`220?ooo`005@3oool200000880oooo0P000025
0?ooo`005@3oool00`000000oooo0?ooo`1o0?ooo`800000Q`3oool001D0oooo00<000000?ooo`3o
ool0O@3oool2000008T0oooo000E0?ooo`030000003oool0oooo07/0oooo0P00002;0?ooo`005@3o
ool00`000000oooo0?ooo`1i0?ooo`800000S@3oool001D0oooo0P00001h0?ooo`800000S`3oool0
01D0oooo00<000000?ooo`3oool0M03oool300000940oooo000E0?ooo`030000003oool0oooo0780
oooo0P00002D0?ooo`005@3oool00`000000oooo0?ooo`1`0?ooo`800000UP3oool001D0oooo00<0
00000?ooo`3oool0KP3oool2000009P0oooo000E0?ooo`800000K@3oool2000009X0oooo000E0?oo
o`030000003oool0oooo06T0oooo0`00002L0?ooo`005@3oool00`000000oooo0?ooo`1V0?ooo`<0
0000W`3oool00080oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`030000
003oool0oooo06@0oooo0P00002R0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`040000
003oool0oooo000000@0oooo00<000000?ooo`3oool0H@3oool300000:@0oooo00001@3oool00000
0?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool3000005h0oooo0`00002W0?oo
o`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`000000oooo0?ooo`1J0?oo
o`@00000ZP3oool000050?ooo`000000oooo0?ooo`000000203oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo05L0oooo0`00002^0?ooo`0000D0oooo0000003oool0oooo000000080?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0E03oool300000;40oooo00020?ooo`80
00002P3oool3000000@0oooo0P00001A0?ooo`@00000]03oool001D0oooo00<000000?ooo`3oool0
C03oool400000;P0oooo000E0?ooo`030000003oool0oooo04P0oooo1000002l0?ooo`005@3oool0
0`000000oooo0?ooo`120?ooo`H00000`03oool001D0oooo00<000000?ooo`3oool0?@3oool50000
0<H0oooo000E0?ooo`800000>@3oool500000</0oooo000E0?ooo`030000003oool0oooo03<0oooo
1@00003@0?ooo`005@3oool00`000000oooo0?ooo`0]0?ooo`H00000e@3oool001D0oooo00<00000
0?ooo`3oool09@3oool800000=/0oooo000E0?ooo`030000003oool0oooo01L0oooo3P00003S0?oo
o`005@3ooolJ00000?40oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<0
00000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?oo
o`030000003oool0oooo0?l0oooo2@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.129741, -0.0519457, \
0.00606207, 0.00412423}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<malpha.dat\>", %130]\)], "Input"],

Cell[BoxData[
    \("malpha.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[alomz[3, Cos[th], 1, 10], {th, 0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.603319 0.170798 [
[.2619 .59082 -9 -9 ]
[.2619 .59082 9 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .00552 -24 -4.5 ]
[.01131 .00552 0 4.5 ]
[.01131 .09092 -12 -4.5 ]
[.01131 .09092 0 4.5 ]
[.01131 .17632 -24 -4.5 ]
[.01131 .17632 0 4.5 ]
[.01131 .26172 -12 -4.5 ]
[.01131 .26172 0 4.5 ]
[.01131 .34712 -24 -4.5 ]
[.01131 .34712 0 4.5 ]
[.01131 .43252 -12 -4.5 ]
[.01131 .43252 0 4.5 ]
[.01131 .51792 -24 -4.5 ]
[.01131 .51792 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(0.2)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(0.4)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.6)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(0.8)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.08333 .60332 m
.08333 .60707 L
s
.14286 .60332 m
.14286 .60707 L
s
.20238 .60332 m
.20238 .60707 L
s
.32143 .60332 m
.32143 .60707 L
s
.38095 .60332 m
.38095 .60707 L
s
.44048 .60332 m
.44048 .60707 L
s
.55952 .60332 m
.55952 .60707 L
s
.61905 .60332 m
.61905 .60707 L
s
.67857 .60332 m
.67857 .60707 L
s
.79762 .60332 m
.79762 .60707 L
s
.85714 .60332 m
.85714 .60707 L
s
.91667 .60332 m
.91667 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .00552 m
.03006 .00552 L
s
[(-3.5)] .01131 .00552 1 0 Mshowa
.02381 .09092 m
.03006 .09092 L
s
[(-3)] .01131 .09092 1 0 Mshowa
.02381 .17632 m
.03006 .17632 L
s
[(-2.5)] .01131 .17632 1 0 Mshowa
.02381 .26172 m
.03006 .26172 L
s
[(-2)] .01131 .26172 1 0 Mshowa
.02381 .34712 m
.03006 .34712 L
s
[(-1.5)] .01131 .34712 1 0 Mshowa
.02381 .43252 m
.03006 .43252 L
s
[(-1)] .01131 .43252 1 0 Mshowa
.02381 .51792 m
.03006 .51792 L
s
[(-0.5)] .01131 .51792 1 0 Mshowa
.125 Mabswid
.02381 .0226 m
.02756 .0226 L
s
.02381 .03968 m
.02756 .03968 L
s
.02381 .05676 m
.02756 .05676 L
s
.02381 .07384 m
.02756 .07384 L
s
.02381 .108 m
.02756 .108 L
s
.02381 .12508 m
.02756 .12508 L
s
.02381 .14216 m
.02756 .14216 L
s
.02381 .15924 m
.02756 .15924 L
s
.02381 .1934 m
.02756 .1934 L
s
.02381 .21048 m
.02756 .21048 L
s
.02381 .22756 m
.02756 .22756 L
s
.02381 .24464 m
.02756 .24464 L
s
.02381 .2788 m
.02756 .2788 L
s
.02381 .29588 m
.02756 .29588 L
s
.02381 .31296 m
.02756 .31296 L
s
.02381 .33004 m
.02756 .33004 L
s
.02381 .3642 m
.02756 .3642 L
s
.02381 .38128 m
.02756 .38128 L
s
.02381 .39836 m
.02756 .39836 L
s
.02381 .41544 m
.02756 .41544 L
s
.02381 .4496 m
.02756 .4496 L
s
.02381 .46668 m
.02756 .46668 L
s
.02381 .48376 m
.02756 .48376 L
s
.02381 .50084 m
.02756 .50084 L
s
.02381 .535 m
.02756 .535 L
s
.02381 .55208 m
.02756 .55208 L
s
.02381 .56916 m
.02756 .56916 L
s
.02381 .58624 m
.02756 .58624 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .01472 m
.0403 .01518 L
.04458 .01575 L
.05429 .01752 L
.06375 .01987 L
.07387 .02305 L
.09565 .03214 L
.11547 .04285 L
.15491 .06998 L
.19284 .10152 L
.2332 .13851 L
.27205 .17544 L
.30942 .21072 L
.3492 .24694 L
.38749 .27982 L
.4282 .31225 L
.46741 .34089 L
.50512 .36604 L
.54526 .39032 L
.5839 .41145 L
.62496 .43167 L
.66453 .44919 L
.7026 .46439 L
.74309 .47898 L
.78208 .49164 L
.81958 .50266 L
.8595 .51328 L
.89792 .52253 L
.93877 .53143 L
.97619 .53883 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool0
0P3oool3000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`001`3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool30?ooo`002P3o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0080oooo0P00003o0?ooo`@0oooo0000
0`3oool0000000000002000000<0oooo0P0000090?ooo`<000001@3oool00`000000oooo0?ooo`3o
0?ooo`<0oooo000:0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0400000?h0oooo00070?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`040?ooo`D00000n@3oool000P0oooo0P0000090?ooo`@0
0000103oool2000000X0oooo0P00003g0?ooo`006`3oool00`000000oooo0?ooo`0;0?ooo`800000
m@3oool001/0oooo00<000000?ooo`3oool03@3oool200000?<0oooo000K0?ooo`030000003oool0
oooo00l0oooo0`00003`0?ooo`006`3oool00`000000oooo0?ooo`0B0?ooo`800000kP3oool001/0
oooo0P00000E0?ooo`800000k03oool001/0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`3Y0?ooo`006`3oool00`000000oooo0?ooo`0G0?ooo`800000j@3oool001/0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`3V0?ooo`006`3oool2000001/0oooo00<000000?oo
o`3oool0i@3oool001/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3T0?ooo`00
6`3oool00`000000oooo0?ooo`0L0?ooo`800000i03oool001/0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`3Q0?ooo`006`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0
oooo0>00oooo000K0?ooo`8000008@3oool00`000000oooo0?ooo`3O0?ooo`006`3oool00`000000
oooo0?ooo`0Q0?ooo`800000g`3oool001@0oooo0P0000050?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool0g03oool001<0oooo00@000000?ooo`3oool00000103oool00`000000oooo
0?ooo`0T0?ooo`030000003oool0oooo0=/0oooo000F0?ooo`030000003oool0oooo0080oooo0P00
000V0?ooo`030000003oool0oooo0=X0oooo000=0?ooo`@000000`3oool2000000D0oooo00<00000
0?ooo`3oool09P3oool200000=X0oooo000F0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`3G0?ooo`004`3oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo02T0oooo00<000000?ooo`3oool0eP3oool001@0oooo0P0000050?ooo`03
0000003oool0oooo02X0oooo00<000000?ooo`3oool0e@3oool001/0oooo0P00000/0?ooo`030000
003oool0oooo0=@0oooo000K0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0d`3o
ool001/0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`3B0?ooo`006`3oool00`00
0000oooo0?ooo`0^0?ooo`800000dP3oool001/0oooo0P00000a0?ooo`030000003oool0oooo0<l0
oooo000K0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0cP3oool001/0oooo00<0
00000?ooo`3oool0<P3oool00`000000oooo0?ooo`3=0?ooo`006`3oool00`000000oooo0?ooo`0c
0?ooo`030000003oool0oooo0<`0oooo000K0?ooo`030000003oool0oooo03@0oooo00<000000?oo
o`3oool0b`3oool001/0oooo0P00000f0?ooo`030000003oool0oooo0<X0oooo000K0?ooo`030000
003oool0oooo03H0oooo00<000000?ooo`3oool0b@3oool001/0oooo00<000000?ooo`3oool0=`3o
ool00`000000oooo0?ooo`380?ooo`006`3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0
oooo0<L0oooo000K0?ooo`800000>P3oool200000<L0oooo000K0?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool0a03oool001/0oooo00<000000?ooo`3oool0?03oool00`000000oooo
0?ooo`330?ooo`001`3oool4000000<0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<0
00000?ooo`3oool0?@3oool00`000000oooo0?ooo`320?ooo`001`3oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`31
0?ooo`00203oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0080oooo0P0000100?oo
o`030000003oool0oooo0<00oooo00000`3oool0000000000002000000@0oooo00<000000?ooo`3o
ool01`3oool3000000D0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`2o0?ooo`00
2P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
@@3oool00`000000oooo0?ooo`2n0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`2m0?ooo`00203o
ool2000000T0oooo100000040?ooo`800000A03oool00`000000oooo0?ooo`2l0?ooo`006`3oool0
0`000000oooo0?ooo`140?ooo`030000003oool0oooo0;/0oooo000K0?ooo`030000003oool0oooo
04D0oooo00<000000?ooo`3oool0^P3oool001/0oooo00<000000?ooo`3oool0AP3oool00`000000
oooo0?ooo`2i0?ooo`006`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0;P0oooo
000K0?ooo`800000B@3oool00`000000oooo0?ooo`2g0?ooo`006`3oool00`000000oooo0?ooo`19
0?ooo`030000003oool0oooo0;H0oooo000K0?ooo`030000003oool0oooo04X0oooo00<000000?oo
o`3oool0]@3oool001/0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`2d0?ooo`00
6`3oool2000004d0oooo00<000000?ooo`3oool0/`3oool001/0oooo00<000000?ooo`3oool0C@3o
ool200000;<0oooo000K0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0/03oool0
01/0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`2_0?ooo`006`3oool00`000000
oooo0?ooo`1A0?ooo`030000003oool0oooo0:h0oooo000K0?ooo`800000D`3oool00`000000oooo
0?ooo`2]0?ooo`006`3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo0:`0oooo000C
0?ooo`@00000103oool00`000000oooo0?ooo`1D0?ooo`800000[03oool001<0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo0:T0oooo000D0?ooo`03
0000003oool0oooo00@0oooo0P00001H0?ooo`030000003oool0oooo0:P0oooo000=0?ooo`@00000
103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool0
Y`3oool001H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1I0?ooo`030000003o
ool0oooo0:H0oooo000C0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0FP3o
ool200000:H0oooo000D0?ooo`8000001@3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0
oooo0:<0oooo000K0?ooo`800000GP3oool00`000000oooo0?ooo`2R0?ooo`006`3oool00`000000
oooo0?ooo`1N0?ooo`030000003oool0oooo0:40oooo000K0?ooo`030000003oool0oooo05l0oooo
0P00002Q0?ooo`006`3oool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo09h0oooo000K
0?ooo`800000H`3oool00`000000oooo0?ooo`2M0?ooo`006`3oool00`000000oooo0?ooo`1S0?oo
o`030000003oool0oooo09`0oooo000K0?ooo`030000003oool0oooo06@0oooo0P00002L0?ooo`00
6`3oool00`000000oooo0?ooo`1V0?ooo`030000003oool0oooo09T0oooo000K0?ooo`030000003o
ool0oooo06L0oooo00<000000?ooo`3oool0V03oool001/0oooo0P00001Y0?ooo`800000V03oool0
01/0oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?ooo`2E0?ooo`006`3oool00`000000
oooo0?ooo`1[0?ooo`030000003oool0oooo09@0oooo000K0?ooo`030000003oool0oooo06`0oooo
0P00002D0?ooo`006`3oool2000006l0oooo00<000000?ooo`3oool0T@3oool001/0oooo00<00000
0?ooo`3oool0K`3oool00`000000oooo0?ooo`2@0?ooo`006`3oool00`000000oooo0?ooo`1`0?oo
o`800000T03oool000L0oooo100000030?ooo`030000003oool0oooo0080oooo0`0000050?ooo`03
0000003oool0oooo0780oooo00<000000?ooo`3oool0S@3oool000T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo07<0oooo00<000000?ooo`3oool0
S03oool000T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`800000M@3o
ool2000008`0oooo00000`3oool0000000000002000000@0oooo00<000000?ooo`3oool01`3oool3
000000D0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`290?ooo`002@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0M`3oool00`00
0000oooo0?ooo`280?ooo`001`3oool3000000T0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`1h0?ooo`800000R03oool000T0oooo00<000000?ooo`3oool01`3oool4000000@0oooo
0P00001k0?ooo`030000003oool0oooo08D0oooo000K0?ooo`030000003oool0oooo07/0oooo00<0
00000?ooo`3oool0Q03oool001/0oooo00<000000?ooo`3oool0O03oool2000008@0oooo000K0?oo
o`030000003oool0oooo07h0oooo0P0000220?ooo`006`3oool00`000000oooo0?ooo`200?ooo`80
0000P03oool001/0oooo0P0000230?ooo`800000OP3oool001/0oooo00<000000?ooo`3oool0Q03o
ool2000007`0oooo000K0?ooo`030000003oool0oooo08H0oooo00<000000?ooo`3oool0N@3oool0
01/0oooo00<000000?ooo`3oool0Q`3oool2000007T0oooo000K0?ooo`800000RP3oool2000007L0
oooo000K0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0M03oool001/0oooo00<0
00000?ooo`3oool0S03oool2000007@0oooo000K0?ooo`030000003oool0oooo08h0oooo0P00001b
0?ooo`006`3oool00`000000oooo0?ooo`2@0?ooo`800000L03oool001/0oooo0P00002C0?ooo`80
0000KP3oool001/0oooo00<000000?ooo`3oool0U03oool2000006`0oooo000C0?ooo`@00000103o
ool00`000000oooo0?ooo`2F0?ooo`800000JP3oool001D0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`2H0?ooo`800000J03oool001D0oooo00<000000?ooo`3oool00`3oool20000
09/0oooo0`00001U0?ooo`003@3oool4000000@0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`2M0?ooo`800000H`3oool001D0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`2O0?ooo`<00000H03oool001<0oooo0`0000050?ooo`030000003oool0oooo0:80oooo0P00
001N0?ooo`005@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0:@0oooo0`00001K
0?ooo`006`3oool200000:P0oooo0P00001I0?ooo`006`3oool00`000000oooo0?ooo`2Y0?ooo`<0
0000EP3oool001/0oooo00<000000?ooo`3oool0[03oool2000005@0oooo000K0?ooo`030000003o
ool0oooo0:h0oooo0`00001A0?ooo`006`3oool200000;80oooo0`00001>0?ooo`006`3oool00`00
0000oooo0?ooo`2d0?ooo`800000C03oool001/0oooo00<000000?ooo`3oool0]P3oool3000004T0
oooo000K0?ooo`030000003oool0oooo0;T0oooo0`0000160?ooo`006`3oool00`000000oooo0?oo
o`2l0?ooo`800000A03oool001/0oooo0P00002o0?ooo`<00000@@3oool001/0oooo00<000000?oo
o`3oool0`@3oool2000003l0oooo000K0?ooo`030000003oool0oooo0<<0oooo0`00000l0?ooo`00
6`3oool00`000000oooo0?ooo`360?ooo`<00000>@3oool001/0oooo0P00003:0?ooo`@00000=@3o
ool001/0oooo00<000000?ooo`3oool0c@3oool400000340oooo000K0?ooo`030000003oool0oooo
0=40oooo1P00000[0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0
oooo00<000000?ooo`3oool0e`3oool4000002L0oooo00070?ooo`040000003oool0oooo000000/0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3K0?ooo`<00000903oool000L0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?ooo`800000g`3oool400000200
oooo00000`3oool000000000000200000080oooo00@000000?ooo`3oool00000203oool3000000D0
oooo00<000000?ooo`3oool0hP3oool4000001`0oooo00070?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3V0?ooo`H000005P3oool000L0oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0>`0
oooo1@00000A0?ooo`00203oool2000000T0oooo100000040?ooo`800000lP3oool5000000`0oooo
000K0?ooo`030000003oool0oooo0?H0oooo0`0000090?ooo`006`3oool00`000000oooo0?ooo`3o
0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3o
ool0o`3oool30?ooo`006`3oool200000?l0oooo103oool001/0oooo00<000000?ooo`3oool0o`3o
ool30?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo
03@0oooo0P0000040?ooo`030000003oool0oooo0080oooo1000000`0?ooo`800000103oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo02l0oooo0P0000040?ooo`030000003oool0oooo
00<0oooo0P00000a0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`8000000`3oool001/0
oooo0P00000d0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0<03oool01000
0000oooo0?ooo`00000:0?ooo`030000003oool0oooo02h0oooo00@000000?ooo`3oool00000203o
ool010000000oooo0?ooo`00000_0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000000P3oool001/0oooo00<000000?ooo`3oool0<`3oool010000000oooo0?ooo`0000090?oo
o`030000003oool0oooo02l0oooo00@000000?ooo`3oool000001`3oool5000002l0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`00000_0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000000P3oool001/0oooo00<000000?ooo`3oool0<`3oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo02h0oooo00@000000?ooo`3oool000001`3oool0
10000000oooo0?ooo`00000`0?ooo`040000003oool0oooo000000P0oooo0`00000`0?ooo`040000
003oool0oooo000000T0oooo0P0000030?ooo`006`3oool00`000000oooo0?ooo`0c0?ooo`040000
003oool0oooo000000/0oooo00<000000?ooo`3oool0;@3oool010000000oooo0?ooo`0000080?oo
o`030000003oool000000300oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0`
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool001/0oooo00<0
00000?ooo`3oool0<`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002l0
oooo00@000000?ooo`3oool000002@3oool200000300oooo00@000000?ooo`3oool00000203oool0
0`000000oooo0?ooo`0`0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
0P3oool001/0oooo0P00000e0?ooo`8000002P3oool200000340oooo0P00000;0?ooo`030000003o
ool0oooo02l0oooo0P00000:0?ooo`<00000<03oool2000000X0oooo0P0000030?ooo`006`3oool0
0`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0
oooo00<000000?ooo`3oool0o`3oool30?ooo`005@3ooooo000000T000000`3oool001/0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`070?ooo`006`3oool00`000000oooo0?ooo`0k
0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0l
0?ooo`030000003oool0oooo00L0oooo000K0?ooo`030000003oool0oooo0?l0oooo0`3oool001/0
oooo00<000000?ooo`3oool0o`3oool30?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0870867, -3.72466, \
0.00319096, 0.0222414}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[betamoz[1, Cos[Pi/3], 1, 10]]\)], "Input"],

Cell[BoxData[
    \(\(-0.27461444263519214`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[betamoz[1, 1/2*Cos[Pi/3], 1/2, 10]]/4\)], "Input"],

Cell[BoxData[
    \(\(-0.27461444263519214`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[gamoz[1, Cos[Pi/3], 1, 10]]\)], "Input"],

Cell[BoxData[
    \(\(\(0.22262029986260426`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[gamoz[1, 1/2*Cos[Pi/3], 1/2, 10]]\)], "Input"],

Cell[BoxData[
    \(\(\(0.22262029986260426`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[deltamoz[1, Cos[Pi/3], 1, 10]]^2\)], "Input"],

Cell[BoxData[
    \(\(-0.02511094138278383`\) + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[deltamoz[1, 1/2*Cos[Pi/3], 1/2, 10]]^2/16\)], "Input"],

Cell[BoxData[
    \(\(-0.02511094138278383`\) + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(klim2arg[k_, th_, 
        a_] := \((1 - betamoz[1, k*Cos[th], k, a])\)*\((k^2 + 
              alomz[1, k*Cos[th], k, a] + gamoz[1, k*Cos[th], k, a])\) + 
        deltamoz[1, k*Cos[th], k, a]^2\ k^2 \((1 - Cos[th]^2)\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2arg[1, 4/10, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.4342657490544408`\)\(\[InvisibleSpace]\)\) + 
      3.6661618212280335`*^-55\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2[4/10, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.5960579387338052`\)\(\[InvisibleSpace]\)\) - 
      6.917033802323945`*^-55\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(klim2arg[6/10, 4/10, 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.02075423453295966`\)\(\[InvisibleSpace]\)\) + 
      6.542123220885845`*^-55\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(k^4 + k^2  M + theother\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(1 + Mtot[1, 1, Cos[4/10], 10] + 
      theotherm[1, 1, Cos[4/10], 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.4342657490544406`\)\(\[InvisibleSpace]\)\) + 
      3.6661618212280335`*^-55\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((\((6/10)\)^4 + \((6/10)\)^2*Mtot[1, 6/10, 6/10*Cos[4/10], 10] + 
          theotherm[1, 6/10, 6/10*Cos[4/10], 10])\)/\((6/10)\)^2\)], "Input"],

Cell[BoxData[
    \(\(\(0.020754234532959284`\)\(\[InvisibleSpace]\)\) + 
      6.542123220885638`*^-55\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(o . k . \)], "Input"],

Cell[BoxData[
    \(Mtot[md2_, k_, kz_, a_] := 
      alomz[md2, kz, k, a] - betamoz[md2, kz, k, a]*k^2 + 
        gamoz[md2, kz, k, a]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(theotherm[md2_, k_, kz_, 
        a_] := \(-betamoz[md2, kz, k, a]\)*
          k^2*\((alomz[md2, kz, k, a] + gamoz[md2, kz, k, a])\) - \((k^2 - 
              kz^2)\)*\((Im[deltamoz[md2, kz, k, a]]^2)\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Mtot[1, 1, Cos[4/10], 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.8693659759727717`\)\(\[InvisibleSpace]\)\) + 
      2.048433533921`0*^-55\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Mtot[1, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 10]^2 - 
        4*theotherm[1, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 
            10], {th, 0, Pi/2}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606305 0.0147151 0.016177 [
[.17539 .00222 -12 -9 ]
[.17539 .00222 12 0 ]
[.32696 .00222 -9 -9 ]
[.32696 .00222 9 0 ]
[.47854 .00222 -12 -9 ]
[.47854 .00222 12 0 ]
[.63011 .00222 -3 -9 ]
[.63011 .00222 3 0 ]
[.78169 .00222 -12 -9 ]
[.78169 .00222 12 0 ]
[.93327 .00222 -9 -9 ]
[.93327 .00222 9 0 ]
[.01131 .0956 -6 -4.5 ]
[.01131 .0956 0 4.5 ]
[.01131 .17648 -12 -4.5 ]
[.01131 .17648 0 4.5 ]
[.01131 .25737 -12 -4.5 ]
[.01131 .25737 0 4.5 ]
[.01131 .33825 -12 -4.5 ]
[.01131 .33825 0 4.5 ]
[.01131 .41914 -12 -4.5 ]
[.01131 .41914 0 4.5 ]
[.01131 .50002 -12 -4.5 ]
[.01131 .50002 0 4.5 ]
[.01131 .58091 -12 -4.5 ]
[.01131 .58091 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .01472 m
.17539 .02097 L
s
[(0.25)] .17539 .00222 0 1 Mshowa
.32696 .01472 m
.32696 .02097 L
s
[(0.5)] .32696 .00222 0 1 Mshowa
.47854 .01472 m
.47854 .02097 L
s
[(0.75)] .47854 .00222 0 1 Mshowa
.63011 .01472 m
.63011 .02097 L
s
[(1)] .63011 .00222 0 1 Mshowa
.78169 .01472 m
.78169 .02097 L
s
[(1.25)] .78169 .00222 0 1 Mshowa
.93327 .01472 m
.93327 .02097 L
s
[(1.5)] .93327 .00222 0 1 Mshowa
.125 Mabswid
.05412 .01472 m
.05412 .01847 L
s
.08444 .01472 m
.08444 .01847 L
s
.11476 .01472 m
.11476 .01847 L
s
.14507 .01472 m
.14507 .01847 L
s
.2057 .01472 m
.2057 .01847 L
s
.23602 .01472 m
.23602 .01847 L
s
.26633 .01472 m
.26633 .01847 L
s
.29665 .01472 m
.29665 .01847 L
s
.35728 .01472 m
.35728 .01847 L
s
.38759 .01472 m
.38759 .01847 L
s
.41791 .01472 m
.41791 .01847 L
s
.44822 .01472 m
.44822 .01847 L
s
.50885 .01472 m
.50885 .01847 L
s
.53917 .01472 m
.53917 .01847 L
s
.56948 .01472 m
.56948 .01847 L
s
.5998 .01472 m
.5998 .01847 L
s
.66043 .01472 m
.66043 .01847 L
s
.69074 .01472 m
.69074 .01847 L
s
.72106 .01472 m
.72106 .01847 L
s
.75138 .01472 m
.75138 .01847 L
s
.81201 .01472 m
.81201 .01847 L
s
.84232 .01472 m
.84232 .01847 L
s
.87264 .01472 m
.87264 .01847 L
s
.90295 .01472 m
.90295 .01847 L
s
.96358 .01472 m
.96358 .01847 L
s
.9939 .01472 m
.9939 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .0956 m
.03006 .0956 L
s
[(5)] .01131 .0956 1 0 Mshowa
.02381 .17648 m
.03006 .17648 L
s
[(10)] .01131 .17648 1 0 Mshowa
.02381 .25737 m
.03006 .25737 L
s
[(15)] .01131 .25737 1 0 Mshowa
.02381 .33825 m
.03006 .33825 L
s
[(20)] .01131 .33825 1 0 Mshowa
.02381 .41914 m
.03006 .41914 L
s
[(25)] .01131 .41914 1 0 Mshowa
.02381 .50002 m
.03006 .50002 L
s
[(30)] .01131 .50002 1 0 Mshowa
.02381 .58091 m
.03006 .58091 L
s
[(35)] .01131 .58091 1 0 Mshowa
.125 Mabswid
.02381 .03089 m
.02756 .03089 L
s
.02381 .04707 m
.02756 .04707 L
s
.02381 .06325 m
.02756 .06325 L
s
.02381 .07942 m
.02756 .07942 L
s
.02381 .11178 m
.02756 .11178 L
s
.02381 .12795 m
.02756 .12795 L
s
.02381 .14413 m
.02756 .14413 L
s
.02381 .16031 m
.02756 .16031 L
s
.02381 .19266 m
.02756 .19266 L
s
.02381 .20884 m
.02756 .20884 L
s
.02381 .22502 m
.02756 .22502 L
s
.02381 .24119 m
.02756 .24119 L
s
.02381 .27355 m
.02756 .27355 L
s
.02381 .28972 m
.02756 .28972 L
s
.02381 .3059 m
.02756 .3059 L
s
.02381 .32208 m
.02756 .32208 L
s
.02381 .35443 m
.02756 .35443 L
s
.02381 .37061 m
.02756 .37061 L
s
.02381 .38679 m
.02756 .38679 L
s
.02381 .40296 m
.02756 .40296 L
s
.02381 .43532 m
.02756 .43532 L
s
.02381 .45149 m
.02756 .45149 L
s
.02381 .46767 m
.02756 .46767 L
s
.02381 .48385 m
.02756 .48385 L
s
.02381 .5162 m
.02756 .5162 L
s
.02381 .53238 m
.02756 .53238 L
s
.02381 .54856 m
.02756 .54856 L
s
.02381 .56473 m
.02756 .56473 L
s
.02381 .59709 m
.02756 .59709 L
s
.02381 .61326 m
.02756 .61326 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02499 .61803 m
.02499 .25436 L
.02605 .25432 L
.02729 .25424 L
.02846 .25412 L
.03053 .25384 L
.03279 .25343 L
.03527 .25283 L
.0379 .25204 L
.04262 .25024 L
.05205 .24519 L
.06244 .23762 L
.0842 .21643 L
.10458 .1926 L
.14335 .14608 L
.18458 .1047 L
.20517 .08842 L
.22428 .0758 L
.26248 .05665 L
.283 .04905 L
.30161 .04341 L
.32247 .03826 L
.34167 .03439 L
.38022 .02856 L
.39905 .02643 L
.41971 .02449 L
.44086 .02287 L
.46014 .02164 L
.49904 .01972 L
.53889 .01833 L
.57722 .01737 L
.618 .01663 L
.65727 .01611 L
.69502 .01574 L
.73522 .01545 L
.77391 .01525 L
.79356 .01516 L
.81505 .01509 L
.85467 .01498 L
.87428 .01494 L
.89523 .0149 L
.90548 .01489 L
.91658 .01488 L
.92618 .01487 L
.93673 .01486 L
.94663 .01486 L
.9559 .01485 L
.96101 .01485 L
.96568 .01485 L
.96826 .01485 L
Mistroke
.96962 .01485 L
.97107 .01485 L
.97179 .01485 L
.97247 .01485 L
.97373 .01485 L
.97491 .01485 L
.97619 .01485 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00<@3oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080
oooo0`00000H0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<000006@3oool2000000@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`<000007@3oool4000001`0
oooo100000030?ooo`030000003oool0oooo0080oooo100000020?ooo`<00000603oool4000000<0
oooo00<000000?ooo`3oool00P3oool3000000l0oooo000`0?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo0000
00/0oooo00<000000?ooo`3oool05@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool0303oool00300oooo00@000000?ooo`3oool000002@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo00@000000?ooo`3oool000002`3o
ool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0<0?ooo`00<03oool010000000oooo0?ooo`00000:0?ooo`040000
003oool0oooo0?ooo`<000005`3oool010000000oooo0?ooo`0000080?ooo`<00000603oool01000
0000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000007`3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool0oooo0`00000J0?ooo`030000
003oool0oooo00L0oooo0`00000?0?ooo`00<03oool010000000oooo0?ooo`00000;0?ooo`040000
003oool0oooo000001T0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0H0?oo
o`040000003oool0oooo000000/0oooo00@000000?ooo`3oool000008@3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00X0oooo00@000000?ooo`3oool00000703oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00l0oooo000`0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0G0?ooo`040000003oool0oooo0000
00P0oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool07@3oool3000001d0oooo0`0000090?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool0603oool3000000T0oooo00<000000?ooo`3oool0
3`3oool00180oooo00<000000?ooo`3oool0703oool2000000X0oooo0P0000030?ooo`@000005`3o
ool2000000T0oooo1000000H0?ooo`8000002@3oool400000080oooo1000000N0?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool0203oool2000000<0oooo1000000I0?ooo`030000003o
ool0oooo00L0oooo1000000>0?ooo`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?oo
o`030000003oool0oooo0?l0oooo303oool00180oooo00<000000?ooo`3oool0o`3oool<0?ooo`00
2`3ooooo000001<000000`3oool00180oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00H0oooo8P0000070?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo000B
0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0J
0?ooo`h00000:03oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02L0oooo00<00000
0?ooo`3oool04`3oool00180oooo00<000000?ooo`3oool0H@3oool;000009l0oooo000B0?ooo`80
0000FP3oool800000:X0oooo000B0?ooo`030000003oool0oooo0540oooo2000002b0?ooo`004P3o
ool00`000000oooo0?ooo`1=0?ooo`@00000^P3oool00180oooo00<000000?ooo`3oool0BP3oool3
00000;h0oooo000B0?ooo`030000003oool0oooo04H0oooo100000310?ooo`004P3oool2000004<0
oooo100000350?ooo`004P3oool00`000000oooo0?ooo`100?ooo`800000b@3oool00180oooo00<0
00000?ooo`3oool0?P3oool200000</0oooo000B0?ooo`030000003oool0oooo03`0oooo0P00003=
0?ooo`004P3oool2000003/0oooo0P00003?0?ooo`004P3oool00`000000oooo0?ooo`0h0?ooo`80
0000d@3oool00180oooo00<000000?ooo`3oool0=P3oool200000=<0oooo000B0?ooo`030000003o
ool0oooo03@0oooo0P00003E0?ooo`004P3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo0=D0oooo000B0?ooo`800000<P3oool200000=P0oooo000B0?ooo`030000003oool0oooo02l0
oooo0P00003J0?ooo`002@3oool3000000H0oooo00<000000?ooo`3oool0;P3oool00`000000oooo
0?ooo`3J0?ooo`00303oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02d0oooo00<0
00000?ooo`3oool0f`3oool000`0oooo00<000000?ooo`3oool00`3oool2000002`0oooo0P00003N
0?ooo`002@3oool3000000H0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3N0?oo
o`002@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3o
ool0g`3oool000T0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo0>00oooo00090?ooo`@000001@3oool2000002P0oooo00<000000?ooo`3oool0h@3o
ool00180oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`3R0?ooo`004P3oool00`00
0000oooo0?ooo`0U0?ooo`030000003oool0oooo0><0oooo000B0?ooo`030000003oool0oooo02@0
oooo00<000000?ooo`3oool0i03oool00180oooo00<000000?ooo`3oool08`3oool00`000000oooo
0?ooo`3U0?ooo`004P3oool2000002<0oooo00<000000?ooo`3oool0iP3oool00180oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`3W0?ooo`004P3oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo0>P0oooo000B0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3o
ool0j@3oool00180oooo0P00000O0?ooo`030000003oool0oooo0>X0oooo000B0?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool0j`3oool00180oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`3/0?ooo`004P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
0>d0oooo000B0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0k@3oool00180oooo
0P00000K0?ooo`030000003oool0oooo0>h0oooo000B0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0k`3oool000<0oooo100000030?ooo`8000001P3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo0?00oooo00050?ooo`050000003oool0oooo0?ooo`0000000P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0l@3oool0
00D0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo0P00000G
0?ooo`030000003oool0oooo0?80oooo00050?ooo`050000003oool0oooo0?ooo`0000000P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0lP3oool0
00D0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`3c0?ooo`000`3oool3000000<0oooo00@000000?oo
o`3oool000001@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0?@0oooo00050?oo
o`030000003oool0oooo0080oooo0P0000060?ooo`800000503oool00`000000oooo0?ooo`3e0?oo
o`004P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0?H0oooo000B0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool0m`3oool00180oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`3h0?ooo`004P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0?P0oooo000B0?ooo`800000403oool00`000000oooo0?ooo`3i0?ooo`004P3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo0?X0oooo000B0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0n`3oool00180oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`3l0?ooo`004P3oool2000000`0oooo00<000000?ooo`3oool0o@3oool00180oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`3n0?ooo`004P3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo0?l0oooo000B0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
o`3oool10?ooo`004P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo0P3o
ool00180oooo0P0000060?ooo`800000o`3oool50?ooo`004P3oool00`000000oooo0?ooo`030?oo
o`800000o`3oool70?ooo`000`3oool400000080oooo0`0000060?ooo`040000003oool0oooo0?oo
o`800000o`3oool90?ooo`001@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0
oooo1000003o0?ooo`/0oooo00050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
0`3oool200000?l0oooo3@3oool000D0oooo00@000000?ooo`3oool0oooo0`0000060?ooo`030000
003oool0oooo0?l0oooo303oool000D0oooo00D000000?ooo`3oool0oooo000000080?ooo`030000
003oool0oooo0?l0oooo303oool000<0oooo0`0000030?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0o`3oool<0?ooo`001@3oool010000000oooo0?ooo`3oool4000000D0oooo00<0
00000?ooo`3oool0o`3oool<0?ooo`004P3oool200000?l0oooo3@3oool00180oooo00<000000?oo
o`3oool0o`3oool<0?ooo`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`030000
003oool0oooo0?l0oooo303oool00180oooo0P00003o0?ooo`d0oooo000B0?ooo`030000003oool0
oooo0?l0oooo303oool00180oooo00<000000?ooo`3oool0o`3oool<0?ooo`004P3oool00`000000
oooo0?ooo`3o0?ooo``0oooo000B0?ooo`800000o`3oool=0?ooo`004P3oool00`000000oooo0?oo
o`3o0?ooo``0oooo000B0?ooo`030000003oool0oooo0?l0oooo303oool00180oooo00<000000?oo
o`3oool0o`3oool<0?ooo`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`800000
o`3oool=0?ooo`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo00030?ooo`@000000`3oool2
000000H0oooo00<000000?ooo`3oool0o`3oool<0?ooo`000`3oool00`000000oooo0?ooo`030?oo
o`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0o`3oool<0?ooo`00103oool00`00
0000oooo0?ooo`020?ooo`040000003oool0oooo000000D0oooo0P00003o0?ooo`d0oooo00050?oo
o`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0?l0oooo303oool000H0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0?l0oooo303oool000<0oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo0?l0oooo303oool000@0oooo0P000004
0?ooo`8000001P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`800000o`3oool=0?oo
o`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`030000003oool0oooo0?l0oooo
303oool00180oooo00<000000?ooo`3oool0o`3oool<0?ooo`004P3oool200000?l0oooo3@3oool0
0180oooo00<000000?ooo`3oool0o`3oool<0?ooo`004P3oool00`000000oooo0?ooo`3o0?ooo``0
oooo000B0?ooo`030000003oool0oooo0?l0oooo303oool00180oooo0P00003o0?ooo`d0oooo000B
0?ooo`030000003oool0oooo0?l0oooo303oool00180oooo00<000000?ooo`3oool0o`3oool<0?oo
o`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`030000003oool0oooo0?l0oooo
303oool00180oooo0P00003o0?ooo`d0oooo000B0?ooo`030000003oool0oooo0?l0oooo303oool0
00<0oooo100000020?ooo`<000001P3oool00`000000oooo0?ooo`3o0?ooo``0oooo00030?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo``0
oooo00040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00`3oool200000?l0oooo
3@3oool000D0oooo00@000000?ooo`3oool0oooo0`0000060?ooo`030000003oool0oooo0?l0oooo
303oool000H0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`3o0?ooo``0oooo
00030?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`3o0?ooo``0oooo00040?ooo`8000000`3oool4000000D0oooo00<000000?ooo`3oool0o`3o
ool<0?ooo`004P3oool200000?l0oooo3@3oool00180oooo00<000000?ooo`3oool0o`3oool<0?oo
o`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`030000003oool0oooo0?l0oooo
303oool00180oooo0P00003o0?ooo`d0oooo000B0?ooo`030000003oool0oooo0?l0oooo303oool0
0180oooo00<000000?ooo`3oool0o`3oool<0?ooo`004P3oool00`000000oooo0?ooo`3o0?ooo``0
oooo000B0?ooo`800000o`3oool=0?ooo`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B
0?ooo`030000003oool0oooo0?l0oooo303oool00180oooo00<000000?ooo`3oool0o`3oool<0?oo
o`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`800000o`3oool=0?ooo`004P3o
ool00`000000oooo0?ooo`3o0?ooo``0oooo00040?ooo`800000103oool2000000H0oooo00<00000
0?ooo`3oool0o`3oool<0?ooo`000`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0
oooo000000D0oooo00<000000?ooo`3oool0o`3oool<0?ooo`001P3oool010000000oooo0?ooo`00
00020?ooo`030000003oool0oooo00<0oooo0P00003o0?ooo`d0oooo00040?ooo`8000000`3oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo0?l0oooo303oool000H0oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo303o
ool000<0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`0000050?ooo`030000
003oool0oooo0?l0oooo303oool000@0oooo0P0000040?ooo`8000001P3oool00`000000oooo0?oo
o`3o0?ooo``0oooo000B0?ooo`800000o`3oool=0?ooo`004P3oool00`000000oooo0?ooo`3o0?oo
o``0oooo000B0?ooo`030000003oool0oooo0?l0oooo303oool00180oooo00<000000?ooo`3oool0
o`3oool<0?ooo`004P3oool200000?l0oooo3@3oool00180oooo00<000000?ooo`3oool0o`3oool<
0?ooo`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`030000003oool0oooo0?l0
oooo303oool00180oooo00<000000?ooo`3oool0o`3oool<0?ooo`004P3oool200000?l0oooo3@3o
ool00180oooo00<000000?ooo`3oool0o`3oool<0?ooo`004P3oool00`000000oooo0?ooo`3o0?oo
o``0oooo000B0?ooo`030000003oool0oooo0?l0oooo303oool00180oooo0P00003o0?ooo`d0oooo
000B0?ooo`030000003oool0oooo0?l0oooo303oool000@0oooo0P0000030?ooo`<000001P3oool0
0`000000oooo0?ooo`3o0?ooo``0oooo00030?ooo`040000003oool0oooo000000D0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo``0oooo00060?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool00`3oool200000?l0oooo3@3oool000@0oooo0P0000030?ooo`<0
00001P3oool00`000000oooo0?ooo`3o0?ooo``0oooo00060?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool0o`3oool<0?ooo`000`3oool010000000oooo0?ooo`0000020?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3oool<0?ooo`00103oool2000000<0oooo
100000050?ooo`030000003oool0oooo0?l0oooo303oool00180oooo0P00003o0?ooo`d0oooo000B
0?ooo`030000003oool0oooo0?l0oooo303oool00180oooo00<000000?ooo`3oool0o`3oool<0?oo
o`004P3oool00`000000oooo0?ooo`3o0?ooo``0oooo000B0?ooo`800000o`3oool=0?ooo`004P3o
ool00`000000oooo0?ooo`3o0?ooo``0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.109452, -2.81267, \
0.00604801, 0.226676}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      theotherm[1, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 
        10], {th, 0, Pi/2}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606305 0.257954 0.0783821 [
[.17539 .24545 -12 -9 ]
[.17539 .24545 12 0 ]
[.32696 .24545 -9 -9 ]
[.32696 .24545 9 0 ]
[.47854 .24545 -12 -9 ]
[.47854 .24545 12 0 ]
[.63011 .24545 -3 -9 ]
[.63011 .24545 3 0 ]
[.78169 .24545 -12 -9 ]
[.78169 .24545 12 0 ]
[.93327 .24545 -9 -9 ]
[.93327 .24545 9 0 ]
[.01131 .02281 -12 -4.5 ]
[.01131 .02281 0 4.5 ]
[.01131 .10119 -12 -4.5 ]
[.01131 .10119 0 4.5 ]
[.01131 .17957 -12 -4.5 ]
[.01131 .17957 0 4.5 ]
[.01131 .33634 -6 -4.5 ]
[.01131 .33634 0 4.5 ]
[.01131 .41472 -6 -4.5 ]
[.01131 .41472 0 4.5 ]
[.01131 .4931 -6 -4.5 ]
[.01131 .4931 0 4.5 ]
[.01131 .57148 -6 -4.5 ]
[.01131 .57148 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .25795 m
.17539 .2642 L
s
[(0.25)] .17539 .24545 0 1 Mshowa
.32696 .25795 m
.32696 .2642 L
s
[(0.5)] .32696 .24545 0 1 Mshowa
.47854 .25795 m
.47854 .2642 L
s
[(0.75)] .47854 .24545 0 1 Mshowa
.63011 .25795 m
.63011 .2642 L
s
[(1)] .63011 .24545 0 1 Mshowa
.78169 .25795 m
.78169 .2642 L
s
[(1.25)] .78169 .24545 0 1 Mshowa
.93327 .25795 m
.93327 .2642 L
s
[(1.5)] .93327 .24545 0 1 Mshowa
.125 Mabswid
.05412 .25795 m
.05412 .2617 L
s
.08444 .25795 m
.08444 .2617 L
s
.11476 .25795 m
.11476 .2617 L
s
.14507 .25795 m
.14507 .2617 L
s
.2057 .25795 m
.2057 .2617 L
s
.23602 .25795 m
.23602 .2617 L
s
.26633 .25795 m
.26633 .2617 L
s
.29665 .25795 m
.29665 .2617 L
s
.35728 .25795 m
.35728 .2617 L
s
.38759 .25795 m
.38759 .2617 L
s
.41791 .25795 m
.41791 .2617 L
s
.44822 .25795 m
.44822 .2617 L
s
.50885 .25795 m
.50885 .2617 L
s
.53917 .25795 m
.53917 .2617 L
s
.56948 .25795 m
.56948 .2617 L
s
.5998 .25795 m
.5998 .2617 L
s
.66043 .25795 m
.66043 .2617 L
s
.69074 .25795 m
.69074 .2617 L
s
.72106 .25795 m
.72106 .2617 L
s
.75138 .25795 m
.75138 .2617 L
s
.81201 .25795 m
.81201 .2617 L
s
.84232 .25795 m
.84232 .2617 L
s
.87264 .25795 m
.87264 .2617 L
s
.90295 .25795 m
.90295 .2617 L
s
.96358 .25795 m
.96358 .2617 L
s
.9939 .25795 m
.9939 .2617 L
s
.25 Mabswid
0 .25795 m
1 .25795 L
s
.02381 .02281 m
.03006 .02281 L
s
[(-3)] .01131 .02281 1 0 Mshowa
.02381 .10119 m
.03006 .10119 L
s
[(-2)] .01131 .10119 1 0 Mshowa
.02381 .17957 m
.03006 .17957 L
s
[(-1)] .01131 .17957 1 0 Mshowa
.02381 .33634 m
.03006 .33634 L
s
[(1)] .01131 .33634 1 0 Mshowa
.02381 .41472 m
.03006 .41472 L
s
[(2)] .01131 .41472 1 0 Mshowa
.02381 .4931 m
.03006 .4931 L
s
[(3)] .01131 .4931 1 0 Mshowa
.02381 .57148 m
.03006 .57148 L
s
[(4)] .01131 .57148 1 0 Mshowa
.125 Mabswid
.02381 .03848 m
.02756 .03848 L
s
.02381 .05416 m
.02756 .05416 L
s
.02381 .06984 m
.02756 .06984 L
s
.02381 .08551 m
.02756 .08551 L
s
.02381 .11687 m
.02756 .11687 L
s
.02381 .13254 m
.02756 .13254 L
s
.02381 .14822 m
.02756 .14822 L
s
.02381 .16389 m
.02756 .16389 L
s
.02381 .19525 m
.02756 .19525 L
s
.02381 .21092 m
.02756 .21092 L
s
.02381 .2266 m
.02756 .2266 L
s
.02381 .24228 m
.02756 .24228 L
s
.02381 .27363 m
.02756 .27363 L
s
.02381 .28931 m
.02756 .28931 L
s
.02381 .30498 m
.02756 .30498 L
s
.02381 .32066 m
.02756 .32066 L
s
.02381 .35201 m
.02756 .35201 L
s
.02381 .36769 m
.02756 .36769 L
s
.02381 .38336 m
.02756 .38336 L
s
.02381 .39904 m
.02756 .39904 L
s
.02381 .43039 m
.02756 .43039 L
s
.02381 .44607 m
.02756 .44607 L
s
.02381 .46175 m
.02756 .46175 L
s
.02381 .47742 m
.02756 .47742 L
s
.02381 .50878 m
.02756 .50878 L
s
.02381 .52445 m
.02756 .52445 L
s
.02381 .54013 m
.02756 .54013 L
s
.02381 .55581 m
.02756 .55581 L
s
.02381 .00713 m
.02756 .00713 L
s
.02381 .58716 m
.02756 .58716 L
s
.02381 .60283 m
.02756 .60283 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02499 .61803 m
.02499 .01472 L
.02605 .01476 L
.02729 .01485 L
.02846 .01497 L
.03053 .01527 L
.03279 .01572 L
.03527 .01636 L
.0379 .01721 L
.04262 .01914 L
.05205 .02455 L
.06244 .03266 L
.0842 .05532 L
.10458 .08071 L
.14335 .12994 L
.18458 .17319 L
.20517 .19001 L
.22428 .20294 L
.26248 .2223 L
.283 .22986 L
.30161 .2354 L
.32247 .24042 L
.34167 .24413 L
.38022 .24961 L
.39905 .25157 L
.41971 .25332 L
.44086 .25476 L
.46014 .25583 L
.49904 .25745 L
.51813 .25805 L
.53889 .25858 L
.55828 .25899 L
.57967 .25936 L
.61894 .25987 L
.63829 .26006 L
.65914 .26022 L
.67879 .26036 L
.70028 .26047 L
.73991 .26064 L
.75952 .2607 L
.78047 .26075 L
.80038 .26079 L
.82196 .26083 L
.841 .26085 L
.86195 .26088 L
.88182 .26089 L
.90287 .26091 L
.92231 .26091 L
.93323 .26092 L
.94321 .26092 L
Mistroke
.95139 .26092 L
.95899 .26092 L
.96336 .26092 L
.96732 .26092 L
.96952 .26092 L
.97067 .26092 L
.97189 .26092 L
.97296 .26092 L
.97413 .26092 L
.9752 .26092 L
.97619 .26092 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000
003oool0oooo0?l0oooo3`3oool000l0oooo0P00003o0?oooa00oooo000?0?ooo`030000003oool0
oooo0?l0oooo3`3oool000P0oooo0P0000050?ooo`<00000o`3oool?0?ooo`001`3oool010000000
oooo0?ooo`0000040?ooo`030000003oool0oooo00<00000o`3oool<0?ooo`002P3oool00`000000
oooo0?ooo`020?ooo`<000000`3oool200000?l0oooo2P3oool000030?ooo`000000000000800000
0`3oool2000000D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`L0oooo
000:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01P3oool200000?l0oooo1`3o
ool000L0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo0?l0oooo103oool000P0oooo0P0000050?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0o`3oool30?ooo`003`3oool2000000/0oooo00<000000?ooo`3oool0o`3oool20?oo
o`003`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0?l0oooo0@3oool000l0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3o0?ooo`003`3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo0?h0oooo000?0?ooo`8000003`3oool00`000000oooo0?ooo`3m
0?ooo`003`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0?`0oooo000?0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool0o03oool000l0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`3k0?ooo`003`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo0?/0oooo000?0?ooo`8000004P3oool00`000000oooo0?ooo`3j0?ooo`003`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo0?T0oooo000?0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool0n@3oool000l0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`3h0?ooo`003`3oool2000001D0oooo00<000000?ooo`3oool0m`3oool000l0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`3f0?ooo`001`3oool4000000@0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`3e0?ooo`001`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool0m@3oool000P0oooo00<000000?ooo`3oool0
103oool3000001L0oooo00<000000?ooo`3oool0m03oool000030?ooo`000000000000800000103o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0l`3o
ool000X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo0?80oooo00070?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`3a0?ooo`00203oool2000000D0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`3`0?ooo`003`3oool2000001`0oooo00<000000?ooo`3oool0l03oool000l0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3_0?ooo`003`3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo0>h0oooo000?0?ooo`030000003oool0oooo01h0oooo00<00000
0?ooo`3oool0k@3oool000l0oooo0P00000P0?ooo`030000003oool0oooo0>`0oooo000?0?ooo`03
0000003oool0oooo0200oooo00<000000?ooo`3oool0j`3oool000l0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`3Z0?ooo`003`3oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo0>T0oooo000?0?ooo`800000903oool00`000000oooo0?ooo`3X0?ooo`003`3oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo0>L0oooo000?0?ooo`030000003oool0oooo02D0
oooo00<000000?ooo`3oool0iP3oool000l0oooo00<000000?ooo`3oool09P3oool00`000000oooo
0?ooo`3U0?ooo`003`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0>@0oooo000?
0?ooo`800000:@3oool00`000000oooo0?ooo`3S0?ooo`003`3oool00`000000oooo0?ooo`0Y0?oo
o`030000003oool0oooo0>80oooo00070?ooo`@00000103oool00`000000oooo0?ooo`0Z0?ooo`03
0000003oool0oooo0>40oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
:`3oool00`000000oooo0?ooo`3P0?ooo`002@3oool00`000000oooo0?ooo`030?ooo`<00000;03o
ool00`000000oooo0?ooo`3O0?ooo`0000<0oooo0000000000000P0000040?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`3N0?ooo`002@3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0g@3oool000L0
oooo0`0000050?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0g03oool000T0oooo
00<000000?ooo`3oool00`3oool200000340oooo0P00003L0?ooo`003`3oool00`000000oooo0?oo
o`0b0?ooo`030000003oool0oooo0=T0oooo000?0?ooo`030000003oool0oooo03<0oooo0P00003I
0?ooo`003`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo0=H0oooo000?0?ooo`03
0000003oool0oooo03H0oooo0P00003F0?ooo`003`3oool2000003T0oooo0P00003D0?ooo`003`3o
ool00`000000oooo0?ooo`0j0?ooo`800000dP3oool000l0oooo00<000000?ooo`3oool07@3oool2
000000@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`00000;0?ooo`800000303oool2
000000@0oooo00<000000?ooo`3oool00P3oool3000001T0oooo0P0000040?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool00P3oool3000001l0oooo1000000M0?ooo`@000000`3oool0
0`000000oooo0?ooo`020?ooo`@000000P3oool3000001P0oooo100000030?ooo`030000003oool0
oooo0080oooo0`00000=0?ooo`003`3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0oooo
000000P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0:0?ooo`<00000203oool0
10000000oooo0?ooo`00000;0?ooo`030000003oool0oooo01D0oooo00@000000?ooo`3oool00000
2@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo000?0?ooo`8000007@3o
ool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
3@3oool3000000D0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0E0?ooo`04
0000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?oo
o`003`3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0oooo000000X0oooo00@000000?oo
o`3oool0oooo0`00000C0?ooo`<000000P3oool010000000oooo0?ooo`0000080?ooo`<00000603o
ool010000000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000008@3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool0oooo0`00000J0?oo
o`030000003oool0oooo00L0oooo0`00000=0?ooo`003`3oool00`000000oooo0?ooo`0L0?ooo`04
0000003oool0oooo000000/0oooo00@000000?ooo`3oool00000603oool300000080oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0H0?ooo`040000003oool0oooo000000/0oooo00@0
00000?ooo`3oool000008`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo00X0oooo
00@000000?ooo`3oool00000703oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00d0
oooo000?0?ooo`030000003oool0oooo01`0oooo00@000000?ooo`3oool00000203oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo01P0oooo100000080?ooo`030000003oool0oooo
01P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo01l0oooo0`00000N0?ooo`<000002@3oool010000000oooo0?ooo`0000020?ooo`030000
003oool0oooo01P0oooo0`0000090?ooo`030000003oool0oooo00d0oooo000?0?ooo`8000007P3o
ool2000000X0oooo0P0000030?ooo`@00000603oool2000000030?ooo`000000000000@000000P3o
ool4000001P0oooo0P0000090?ooo`@000000P3oool400000200oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`080?ooo`8000000`3oool4000001T0oooo00<000000?ooo`3oool01`3o
ool4000000`0oooo000?0?ooo`030000003oool0oooo05<0oooo1@00002f0?ooo`003`3oool00`00
0000oooo0?ooo`1H0?ooo`H00000/03oool000l0oooo00<000000?ooo`3oool0GP3oool:00000:H0
oooo000?0?ooo`030000003oool0oooo06P0oooo5000002B0?ooo`00203ooooo000001T00000000?
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo00D0ooooD@0000040?ooo`4000000@3oool10?ooo`003`3oool00`00
0000oooo0?ooo`0W0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool09`3oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool09`3oool00`00
0000oooo0?ooo`0A0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000
o`3oool@0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0
oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000
oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3oool@0?ooo`003`3oool00`000000oooo0?oo
o`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?oo
o`3oool0o`3oool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0
o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0
oooo0?l0oooo3`3oool000l0oooo0P00003o0?oooa00oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`001`3oool4000000@0oooo00<0
00000?ooo`3oool0o`3oool?0?ooo`002@3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0?l0oooo3`3oool000T0oooo00<000000?ooo`3oool00`3oool300000?l0oooo3`3oool000T0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo00090?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool?0?ooo`001`3oool3000000D0oooo00<0
00000?ooo`3oool0o`3oool?0?ooo`002@3oool00`000000oooo0?ooo`030?ooo`800000o`3oool@
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool200000?l0oooo403o
ool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?oo
o`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0
o`3oool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000l0oooo0P00003o0?oooa00oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3o
ool000L0oooo100000040?ooo`030000003oool0oooo0?l0oooo3`3oool000L0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`l0oooo00080?ooo`030000003oool0oooo00@0
oooo0`00003o0?ooo`l0oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
o`3oool?0?ooo`002P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0oooo3`3o
ool000L0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3o0?ooo`l0oooo0008
0?ooo`8000001@3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3oool@0?oo
o`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo
3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool200000?l0oooo403oool0
00l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0
oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3o
ool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo
3`3oool000l0oooo0P00003o0?oooa00oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool0
00P0oooo0P0000050?ooo`030000003oool0oooo0?l0oooo3`3oool000L0oooo00@000000?ooo`3o
ool00000103oool00`000000oooo0?ooo`3o0?ooo`l0oooo000:0?ooo`030000003oool0oooo0080
oooo0`00003o0?ooo`l0oooo00080?ooo`8000001@3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
000:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool?0?ooo`001`3oool0
10000000oooo0?ooo`0000040?ooo`030000003oool0oooo0?l0oooo3`3oool000P0oooo0P000005
0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo0P00003o0?oooa00oooo000?0?ooo`03
0000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3o
ool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3oool@0?ooo`003`3oool00`00
0000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo
00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<00000
0?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`03
0000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3o
ool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`002@3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0?l0oooo3`3oool000T0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo00060?ooo`D00000103oool300000?l0oooo
3`3oool000H0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
00070?ooo`030000003oool0000000D0oooo00<000000?ooo`3oool0o`3oool?0?ooo`00203oool2
000000D0oooo00<000000?ooo`3oool0o`3oool?0?ooo`002@3oool00`000000oooo0?ooo`030?oo
o`800000o`3oool@0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000
003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3oool@0?ooo`003`3oool00`000000
oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<0
00000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo003o0?oo
ob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0917817, -3.4065, \
0.00592981, 0.0458685}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mp[md2_, k_, kz_, a_] := 
      1/2*\((Mtot[md2, k, kz, a] + 
            Sqrt[Mtot[md2, k, kz, a]^2 - 
                4*theotherm[md2, k, kz, a]])\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(mm[md2_, k_, kz_, a_] := 
      1/2*\((Mtot[md2, k, kz, a] - 
            Sqrt[Mtot[md2, k, kz, a]^2 - 
                4*theotherm[md2, k, kz, a]])\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(mm[1, 1, Cos[9/10], 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.0348031395917614`\)\(\[InvisibleSpace]\)\) + 
      7.136260431322027`*^-27\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Mtot[1, 1, Cos[3/10], 10]\)], "Input"],

Cell[BoxData[
    \(\(\(1.0418791148092312`\)\(\[InvisibleSpace]\)\) + 
      5.603492937798`0*^-55\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Mtot[1, 1, Cos[9/10], 10]^2\)], "Input"],

Cell[BoxData[
    \(\(\(0.23789309730589012`\)\(\[InvisibleSpace]\)\) + 
      1.0864766633990857`*^-26\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(4*theotherm[1, 1, Cos[9/10], 10]\)], "Input"],

Cell[BoxData[
    \(\(\(0.06305489723909805`\)\(\[InvisibleSpace]\)\) + 
      1.3486248028582082`*^-26\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%% - %\)], "Input"],

Cell[BoxData[
    \(\(\(0.17483820006679207`\)\(\[InvisibleSpace]\)\) - 
      2.6214813945912247`*^-27\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      mp[1, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 10], {th, 
        0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 -0.129267 0.271623 [
[.2619 .12986 -9 -9 ]
[.2619 .12986 9 0 ]
[.5 .12986 -9 -9 ]
[.5 .12986 9 0 ]
[.7381 .12986 -9 -9 ]
[.7381 .12986 9 0 ]
[.97619 .12986 -9 -9 ]
[.97619 .12986 9 0 ]
[.01131 .00654 -18 -4.5 ]
[.01131 .00654 0 4.5 ]
[.01131 .27817 -18 -4.5 ]
[.01131 .27817 0 4.5 ]
[.01131 .41398 -6 -4.5 ]
[.01131 .41398 0 4.5 ]
[.01131 .54979 -18 -4.5 ]
[.01131 .54979 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .14236 m
.2619 .14861 L
s
[(0.2)] .2619 .12986 0 1 Mshowa
.5 .14236 m
.5 .14861 L
s
[(0.4)] .5 .12986 0 1 Mshowa
.7381 .14236 m
.7381 .14861 L
s
[(0.6)] .7381 .12986 0 1 Mshowa
.97619 .14236 m
.97619 .14861 L
s
[(0.8)] .97619 .12986 0 1 Mshowa
.125 Mabswid
.08333 .14236 m
.08333 .14611 L
s
.14286 .14236 m
.14286 .14611 L
s
.20238 .14236 m
.20238 .14611 L
s
.32143 .14236 m
.32143 .14611 L
s
.38095 .14236 m
.38095 .14611 L
s
.44048 .14236 m
.44048 .14611 L
s
.55952 .14236 m
.55952 .14611 L
s
.61905 .14236 m
.61905 .14611 L
s
.67857 .14236 m
.67857 .14611 L
s
.79762 .14236 m
.79762 .14611 L
s
.85714 .14236 m
.85714 .14611 L
s
.91667 .14236 m
.91667 .14611 L
s
.25 Mabswid
0 .14236 m
1 .14236 L
s
.02381 .00654 m
.03006 .00654 L
s
[(0.5)] .01131 .00654 1 0 Mshowa
.02381 .27817 m
.03006 .27817 L
s
[(1.5)] .01131 .27817 1 0 Mshowa
.02381 .41398 m
.03006 .41398 L
s
[(2)] .01131 .41398 1 0 Mshowa
.02381 .54979 m
.03006 .54979 L
s
[(2.5)] .01131 .54979 1 0 Mshowa
.125 Mabswid
.02381 .03371 m
.02756 .03371 L
s
.02381 .06087 m
.02756 .06087 L
s
.02381 .08803 m
.02756 .08803 L
s
.02381 .11519 m
.02756 .11519 L
s
.02381 .16952 m
.02756 .16952 L
s
.02381 .19668 m
.02756 .19668 L
s
.02381 .22384 m
.02756 .22384 L
s
.02381 .251 m
.02756 .251 L
s
.02381 .30533 m
.02756 .30533 L
s
.02381 .33249 m
.02756 .33249 L
s
.02381 .35965 m
.02756 .35965 L
s
.02381 .38682 m
.02756 .38682 L
s
.02381 .44114 m
.02756 .44114 L
s
.02381 .4683 m
.02756 .4683 L
s
.02381 .49546 m
.02756 .49546 L
s
.02381 .52263 m
.02756 .52263 L
s
.02381 .57695 m
.02756 .57695 L
s
.02381 .60411 m
.02756 .60411 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .60332 m
.0403 .60276 L
.04458 .60208 L
.05429 .59995 L
.06375 .59713 L
.07387 .59331 L
.09565 .58243 L
.11547 .56962 L
.15491 .53736 L
.19284 .50012 L
.2332 .45681 L
.27205 .41399 L
.30942 .37347 L
.3492 .33228 L
.38749 .29525 L
.4282 .25906 L
.46741 .22738 L
.50512 .19979 L
.54526 .17333 L
.5839 .15046 L
.62496 .1287 L
.66453 .10995 L
.7026 .09373 L
.74309 .07822 L
.78208 .06479 L
.81958 .05311 L
.8595 .04186 L
.89792 .03205 L
.93877 .0226 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?oo
o`3oool0o`3oool90?ooo`0000D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0o`3oool90?ooo`0000D0oooo0000003oool0oooo0000000;0?oo
o`030000003oool0oooo0080oooo0`00003o0?ooo`T0oooo00001@3oool000000?ooo`3oool00000
00P0oooo0`0000050?ooo`030000003oool0oooo0?l0oooo2@3oool000050?ooo`000000oooo0?oo
o`000000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?`0oooo0`0000090?oo
o`0000D0oooo0000003oool0oooo000000080?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool0mP3oool6000000`0oooo00020?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3o
ool0lP3oool400000180oooo000E0?ooo`030000003oool0oooo0>h0oooo1000000F0?ooo`005@3o
ool00`000000oooo0?ooo`3Z0?ooo`@000006P3oool001D0oooo0P00003W0?ooo`@000007P3oool0
01D0oooo00<000000?ooo`3oool0h@3oool500000280oooo000E0?ooo`030000003oool0oooo0=d0
oooo1000000W0?ooo`005@3oool00`000000oooo0?ooo`3I0?ooo`@00000:`3oool001D0oooo00<0
00000?ooo`3oool0e@3oool4000002l0oooo000E0?ooo`030000003oool0oooo0=80oooo0`00000c
0?ooo`005@3oool00`000000oooo0?ooo`3@0?ooo`800000=P3oool001D0oooo00<000000?ooo`3o
ool0c@3oool3000003P0oooo000E0?ooo`800000c03oool2000003/0oooo000E0?ooo`030000003o
ool0oooo0<P0oooo0`00000m0?ooo`005@3oool00`000000oooo0?ooo`340?ooo`@00000@03oool0
01D0oooo00<000000?ooo`3oool0`@3oool3000004@0oooo000E0?ooo`030000003oool0oooo0;h0
oooo0`0000170?ooo`005@3oool00`000000oooo0?ooo`2k0?ooo`<00000BP3oool001D0oooo00<0
00000?ooo`3oool0^@3oool2000004d0oooo000E0?ooo`800000]`3oool3000004l0oooo000E0?oo
o`030000003oool0oooo0;<0oooo0`00001B0?ooo`005@3oool00`000000oooo0?ooo`2a0?ooo`80
0000E@3oool001D0oooo00<000000?ooo`3oool0[`3oool2000005L0oooo000E0?ooo`030000003o
ool0oooo03H0oooo0P0000040?ooo`030000003oool0oooo0080oooo1000000a0?ooo`800000103o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02L0oooo0P0000070?ooo`800000103o
ool00`000000oooo0?ooo`030?ooo`800000<`3oool2000000@0oooo00<000000?ooo`3oool00`3o
ool2000000<0oooo000E0?ooo`030000003oool0oooo03D0oooo00@000000?ooo`3oool00000203o
ool00`000000oooo0?ooo`0a0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
9@3oool2000000P0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000a0?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000000P3oool001D0oooo00<00000
0?ooo`3oool0=@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo0300oooo00@0
00000?ooo`3oool000001`3oool5000002@0oooo0P00000:0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool00000<@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo000E0?ooo`800000=P3oool010000000oooo0?ooo`00000:0?ooo`030000003o
ool0oooo02l0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00000S0?ooo`80
0000303oool010000000oooo0?ooo`0000080?ooo`<00000<P3oool010000000oooo0?ooo`000009
0?ooo`8000000`3oool001D0oooo00<000000?ooo`3oool0=@3oool010000000oooo0?ooo`00000;
0?ooo`030000003oool0oooo02h0oooo00@000000?ooo`3oool00000203oool00`000000oooo0000
000Q0?ooo`8000003P3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo0380oooo
00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`005@3oool00`000000
oooo0?ooo`0e0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000<03oool0
10000000oooo0?ooo`0000090?ooo`8000007`3oool200000100oooo00@000000?ooo`3oool00000
203oool00`000000oooo0?ooo`0b0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3o
ool000000P3oool001D0oooo00<000000?ooo`3oool0=P3oool2000000X0oooo0P00000b0?ooo`80
00002`3oool00`000000oooo0?ooo`0K0?ooo`8000004`3oool2000000X0oooo0`00000b0?ooo`80
00002P3oool2000000<0oooo000E0?ooo`030000003oool0oooo09l0oooo0P00001W0?ooo`005@3o
ool00`000000oooo0?ooo`2M0?ooo`800000J@3oool001D0oooo00<000000?ooo`3oool0V`3oool2
000006/0oooo000>0?ooool00000400000030?ooo`005@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`070?ooo`80
0000103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`070?ooo`005@3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`050?ooo`800000
1P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`070?ooo`005@3oool00`000000oooo0?ooo`2D0?ooo`030000003o
ool0oooo0740oooo000E0?ooo`030000003oool0oooo0980oooo0P00001d0?ooo`005@3oool00`00
0000oooo0?ooo`2@0?ooo`800000MP3oool001D0oooo00<000000?ooo`3oool0SP3oool2000007P0
oooo000E0?ooo`030000003oool0oooo08`0oooo0P00001j0?ooo`005@3oool2000008/0oooo0P00
001l0?ooo`005@3oool00`000000oooo0?ooo`290?ooo`030000003oool0oooo07`0oooo000E0?oo
o`030000003oool0oooo08L0oooo0P00001o0?ooo`005@3oool00`000000oooo0?ooo`260?ooo`03
0000003oool0oooo07l0oooo000E0?ooo`030000003oool0oooo08@0oooo0P0000220?ooo`005@3o
ool00`000000oooo0?ooo`230?ooo`030000003oool0oooo0880oooo000E0?ooo`030000003oool0
oooo0880oooo00<000000?ooo`3oool0P`3oool001D0oooo0P0000210?ooo`800000QP3oool001D0
oooo00<000000?ooo`3oool0O`3oool00`000000oooo0?ooo`260?ooo`005@3oool00`000000oooo
0?ooo`1m0?ooo`800000R@3oool001D0oooo00<000000?ooo`3oool0O03oool00`000000oooo0?oo
o`290?ooo`005@3oool00`000000oooo0?ooo`1j0?ooo`800000S03oool001D0oooo00<000000?oo
o`3oool0N03oool2000008h0oooo000E0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3o
ool0SP3oool001D0oooo0P00001f0?ooo`800000T@3oool001D0oooo00<000000?ooo`3oool0M03o
ool00`000000oooo0?ooo`2A0?ooo`005@3oool00`000000oooo0?ooo`1c0?ooo`030000003oool0
oooo0980oooo000E0?ooo`030000003oool0oooo0780oooo00<000000?ooo`3oool0T`3oool001D0
oooo00<000000?ooo`3oool0L@3oool00`000000oooo0?ooo`2D0?ooo`005@3oool00`000000oooo
0?ooo`1_0?ooo`800000U`3oool001D0oooo00<000000?ooo`3oool0KP3oool00`000000oooo0?oo
o`2G0?ooo`005@3oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo09P0oooo000E0?oo
o`800000K@3oool00`000000oooo0?ooo`2I0?ooo`005@3oool00`000000oooo0?ooo`1[0?ooo`03
0000003oool0oooo09X0oooo000E0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool0
V`3oool001D0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`2L0?ooo`005@3oool0
0`000000oooo0?ooo`1X0?ooo`030000003oool0oooo09d0oooo00000`3oool00000000000020000
00<0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?ooo`3oool0I`3oool00`00
0000oooo0?ooo`2N0?ooo`000`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`2O0?ooo`000`3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo0080oooo0`00001T0?ooo`800000XP3oool000<0oooo00<0
00000?ooo`3oool01`3oool3000000D0oooo00<000000?ooo`3oool0H`3oool00`000000oooo0?oo
o`2R0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0HP3oool00`000000oooo0?ooo`2S0?ooo`0000@0oooo00000000000000002@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0Y03oool0
00<0oooo00<000000?ooo`3oool01`3oool4000000@0oooo00<000000?ooo`3oool0H03oool00`00
0000oooo0?ooo`2U0?ooo`005@3oool00`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0:H0
oooo000E0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0Y`3oool001D0oooo0P00
001N0?ooo`030000003oool0oooo0:P0oooo000E0?ooo`030000003oool0oooo05`0oooo00<00000
0?ooo`3oool0Z@3oool001D0oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`2Z0?oo
o`005@3oool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo0:/0oooo000E0?ooo`030000
003oool0oooo05T0oooo00<000000?ooo`3oool0[03oool001D0oooo00<000000?ooo`3oool0F03o
ool00`000000oooo0?ooo`2]0?ooo`005@3oool00`000000oooo0?ooo`1G0?ooo`030000003oool0
oooo0:h0oooo000E0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0[`3oool001D0
oooo0P00001F0?ooo`030000003oool0oooo0;00oooo000E0?ooo`030000003oool0oooo05@0oooo
00<000000?ooo`3oool0/@3oool001D0oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?oo
o`2b0?ooo`005@3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo0;<0oooo000E0?oo
o`030000003oool0oooo0540oooo00<000000?ooo`3oool0]03oool001D0oooo00<000000?ooo`3o
ool0D03oool00`000000oooo0?ooo`2e0?ooo`005@3oool00`000000oooo0?ooo`1?0?ooo`030000
003oool0oooo0;H0oooo000E0?ooo`800000C`3oool00`000000oooo0?ooo`2g0?ooo`005@3oool0
0`000000oooo0?ooo`1=0?ooo`030000003oool0oooo0;P0oooo000E0?ooo`030000003oool0oooo
04`0oooo00<000000?ooo`3oool0^@3oool001D0oooo00<000000?ooo`3oool0B`3oool00`000000
oooo0?ooo`2j0?ooo`005@3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo0;/0oooo
000E0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0_03oool001D0oooo00<00000
0?ooo`3oool0B03oool00`000000oooo0?ooo`2m0?ooo`005@3oool2000004P0oooo00<000000?oo
o`3oool0_P3oool001D0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`2o0?ooo`00
5@3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo0<00oooo000E0?ooo`030000003o
ool0oooo04D0oooo00<000000?ooo`3oool0`03oool001D0oooo00<000000?ooo`3oool0A03oool0
0`000000oooo0?ooo`310?ooo`005@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo
0<80oooo000=0?ooo`@00000103oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0<<0
oooo000=0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0@@3oool00`000000oooo
0?ooo`340?ooo`003P3oool00`000000oooo0?ooo`040?ooo`<00000@03oool00`000000oooo0?oo
o`350?ooo`003`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo03l0oooo00<00000
0?ooo`3oool0aP3oool00100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0n0?oo
o`030000003oool0oooo0<L0oooo000=0?ooo`040000003oool0oooo000000@0oooo00<000000?oo
o`3oool0?@3oool00`000000oooo0?ooo`380?ooo`003P3oool2000000D0oooo00<000000?ooo`3o
ool0?03oool00`000000oooo0?ooo`390?ooo`005@3oool00`000000oooo0?ooo`0k0?ooo`030000
003oool0oooo0<X0oooo000E0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0b`3o
ool001D0oooo0P00000j0?ooo`030000003oool0oooo0<`0oooo000E0?ooo`030000003oool0oooo
03P0oooo00<000000?ooo`3oool0c@3oool001D0oooo00<000000?ooo`3oool0=`3oool00`000000
oooo0?ooo`3>0?ooo`005@3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0<l0oooo
000E0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0d03oool001D0oooo00<00000
0?ooo`3oool0=03oool00`000000oooo0?ooo`3A0?ooo`005@3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo0=80oooo000E0?ooo`800000<`3oool00`000000oooo0?ooo`3C0?ooo`00
5@3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0=@0oooo000E0?ooo`030000003o
ool0oooo0300oooo00<000000?ooo`3oool0e@3oool001D0oooo00<000000?ooo`3oool0<03oool0
0`000000oooo0?ooo`3E0?ooo`005@3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo
0=H0oooo000E0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0e`3oool001D0oooo
00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`3H0?ooo`005@3oool00`000000oooo0?oo
o`0/0?ooo`030000003oool0oooo0=T0oooo000E0?ooo`800000;03oool00`000000oooo0?ooo`3J
0?ooo`005@3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0=/0oooo000E0?ooo`03
0000003oool0oooo02T0oooo00<000000?ooo`3oool0g03oool001D0oooo00<000000?ooo`3oool0
:03oool00`000000oooo0?ooo`3M0?ooo`005@3oool00`000000oooo0?ooo`0W0?ooo`030000003o
ool0oooo0=h0oooo000E0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0g`3oool0
01D0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`3P0?ooo`005@3oool2000002D0
oooo00<000000?ooo`3oool0h@3oool001D0oooo00<000000?ooo`3oool08`3oool00`000000oooo
0?ooo`3R0?ooo`005@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0><0oooo000E
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0i03oool001D0oooo00<000000?oo
o`3oool0803oool00`000000oooo0?ooo`3U0?ooo`0000<0oooo0000000000000P0000030?ooo`03
0000003oool0oooo0080oooo0`0000050?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3o
ool0iP3oool000030?ooo`000000oooo00d0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo0>L0oooo00020?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool00P3oool3000001d0oooo00<000000?ooo`3oool0j03oool000<0oooo00<00000
0?ooo`3oool01`3oool3000000D0oooo00<000000?ooo`3oool06`3oool200000>/0oooo00040?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo0>/0oooo00001@3oool000000?ooo`3oool0000000P0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0>`0oooo00020?ooo`80
00002@3oool4000000@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`3]0?ooo`00
5@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0>h0oooo000E0?ooo`030000003o
ool0oooo01D0oooo0P00003a0?ooo`005@3oool00`000000oooo0?ooo`0C0?ooo`800000l`3oool0
01D0oooo0P00000B0?ooo`800000m@3oool001D0oooo00<000000?ooo`3oool03`3oool200000?L0
oooo000E0?ooo`030000003oool0oooo00d0oooo0P00003i0?ooo`005@3oool00`000000oooo0?oo
o`0;0?ooo`800000n`3oool001D0oooo00<000000?ooo`3oool02@3oool200000?d0oooo000E0?oo
o`030000003oool0oooo00L0oooo0P00003o0?ooo`005@3oool00`000000oooo0?ooo`020?ooo`D0
0000o`3oool20?ooo`005@3oool2000000030?ooo`00000000000?l0oooo1`3oool001D0oooo00<0
00000?ooo`3oool0o`3oool90?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`T0oooo000E0?oo
o`030000003oool0oooo0?l0oooo2@3oool001D0oooo00<000000?ooo`3oool0o`3oool90?ooo`00
o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0666155, 0.38614, \
0.00311733, 0.0136627}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      mp[3, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 10], {th, 
        0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 -0.129267 0.0905409 [
[.2619 .03931 -9 -9 ]
[.2619 .03931 9 0 ]
[.5 .03931 -9 -9 ]
[.5 .03931 9 0 ]
[.7381 .03931 -9 -9 ]
[.7381 .03931 9 0 ]
[.97619 .03931 -9 -9 ]
[.97619 .03931 9 0 ]
[.01131 .14236 -6 -4.5 ]
[.01131 .14236 0 4.5 ]
[.01131 .2329 -6 -4.5 ]
[.01131 .2329 0 4.5 ]
[.01131 .32344 -6 -4.5 ]
[.01131 .32344 0 4.5 ]
[.01131 .41398 -6 -4.5 ]
[.01131 .41398 0 4.5 ]
[.01131 .50452 -6 -4.5 ]
[.01131 .50452 0 4.5 ]
[.01131 .59506 -6 -4.5 ]
[.01131 .59506 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .05181 m
.2619 .05806 L
s
[(0.2)] .2619 .03931 0 1 Mshowa
.5 .05181 m
.5 .05806 L
s
[(0.4)] .5 .03931 0 1 Mshowa
.7381 .05181 m
.7381 .05806 L
s
[(0.6)] .7381 .03931 0 1 Mshowa
.97619 .05181 m
.97619 .05806 L
s
[(0.8)] .97619 .03931 0 1 Mshowa
.125 Mabswid
.08333 .05181 m
.08333 .05556 L
s
.14286 .05181 m
.14286 .05556 L
s
.20238 .05181 m
.20238 .05556 L
s
.32143 .05181 m
.32143 .05556 L
s
.38095 .05181 m
.38095 .05556 L
s
.44048 .05181 m
.44048 .05556 L
s
.55952 .05181 m
.55952 .05556 L
s
.61905 .05181 m
.61905 .05556 L
s
.67857 .05181 m
.67857 .05556 L
s
.79762 .05181 m
.79762 .05556 L
s
.85714 .05181 m
.85714 .05556 L
s
.91667 .05181 m
.91667 .05556 L
s
.25 Mabswid
0 .05181 m
1 .05181 L
s
.02381 .14236 m
.03006 .14236 L
s
[(3)] .01131 .14236 1 0 Mshowa
.02381 .2329 m
.03006 .2329 L
s
[(4)] .01131 .2329 1 0 Mshowa
.02381 .32344 m
.03006 .32344 L
s
[(5)] .01131 .32344 1 0 Mshowa
.02381 .41398 m
.03006 .41398 L
s
[(6)] .01131 .41398 1 0 Mshowa
.02381 .50452 m
.03006 .50452 L
s
[(7)] .01131 .50452 1 0 Mshowa
.02381 .59506 m
.03006 .59506 L
s
[(8)] .01131 .59506 1 0 Mshowa
.125 Mabswid
.02381 .06992 m
.02756 .06992 L
s
.02381 .08803 m
.02756 .08803 L
s
.02381 .10614 m
.02756 .10614 L
s
.02381 .12425 m
.02756 .12425 L
s
.02381 .16046 m
.02756 .16046 L
s
.02381 .17857 m
.02756 .17857 L
s
.02381 .19668 m
.02756 .19668 L
s
.02381 .21479 m
.02756 .21479 L
s
.02381 .251 m
.02756 .251 L
s
.02381 .26911 m
.02756 .26911 L
s
.02381 .28722 m
.02756 .28722 L
s
.02381 .30533 m
.02756 .30533 L
s
.02381 .34155 m
.02756 .34155 L
s
.02381 .35965 m
.02756 .35965 L
s
.02381 .37776 m
.02756 .37776 L
s
.02381 .39587 m
.02756 .39587 L
s
.02381 .43209 m
.02756 .43209 L
s
.02381 .45019 m
.02756 .45019 L
s
.02381 .4683 m
.02756 .4683 L
s
.02381 .48641 m
.02756 .48641 L
s
.02381 .52263 m
.02756 .52263 L
s
.02381 .54074 m
.02756 .54074 L
s
.02381 .55884 m
.02756 .55884 L
s
.02381 .57695 m
.02756 .57695 L
s
.02381 .03371 m
.02756 .03371 L
s
.02381 .0156 m
.02756 .0156 L
s
.02381 .61317 m
.02756 .61317 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .60332 m
.0403 .60276 L
.04458 .60208 L
.05429 .59995 L
.06375 .59713 L
.07387 .59331 L
.09565 .58243 L
.11547 .56962 L
.15491 .53736 L
.19284 .50012 L
.2332 .45681 L
.27205 .41399 L
.30942 .37347 L
.3492 .33228 L
.38749 .29525 L
.4282 .25906 L
.46741 .22738 L
.50512 .19979 L
.54526 .17333 L
.5839 .15046 L
.62496 .1287 L
.66453 .10995 L
.7026 .09373 L
.74309 .07822 L
.78208 .06479 L
.81958 .05311 L
.8595 .04186 L
.89792 .03205 L
.93877 .0226 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`002@3oool00`000000oooo0?ooo`3o0?oooaD0oooo00090?ooo`030000003oool0oooo0?l0
oooo5@3oool000T0oooo00<000000?ooo`3oool0o`3ooolE0?ooo`002@3oool00`000000oooo0?oo
o`3o0?oooaD0oooo00090?ooo`800000>P3oool2000000@0oooo00<000000?ooo`3oool00P3oool4
000003@0oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0<`3oool2
000000@0oooo00<000000?ooo`3oool00`3oool2000003H0oooo0P0000030?ooo`<00000103oool2
000000<0oooo00090?ooo`030000003oool0oooo03P0oooo00@000000?ooo`3oool00000203oool0
0`000000oooo0?ooo`0d0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0<P3o
ool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000003@0oooo1P0000060?ooo`04
0000003oool0oooo00000080oooo00090?ooo`030000003oool0oooo03P0oooo00@000000?ooo`3o
ool000002@3oool00`000000oooo0?ooo`0c0?ooo`040000003oool0oooo000000L0oooo1@00000c
0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000;`3oool600000080oooo
00<000000?ooo`3oool01P3oool010000000oooo0?ooo`0000020?ooo`002@3oool00`000000oooo
0?ooo`0h0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0<P3oool010000000
oooo0?ooo`0000070?ooo`040000003oool0oooo000003@0oooo00@000000?ooo`3oool00000203o
ool3000002`0oooo100000050?ooo`040000003oool0oooo000000T0oooo0P0000030?ooo`002@3o
ool00`000000oooo0?ooo`0h0?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool0
<@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0000003@0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0?ooo`0X0?ooo`@000002@3oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo00000080oooo00090?ooo`800000>@3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo000003<0oooo00@000000?ooo`3oool000002@3oool2000003@0
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0U0?ooo`<000003@3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00090?ooo`030000003oool0
oooo03T0oooo0P00000:0?ooo`800000=@3oool2000000/0oooo00<000000?ooo`3oool0<`3oool2
000000X0oooo0`00000P0?ooo`@000004@3oool2000000X0oooo0P0000030?ooo`002@3oool00`00
0000oooo0?ooo`3[0?ooo`<000009P3oool000T0oooo00<000000?ooo`3oool0i`3oool4000002T0
oooo00090?ooo`030000003oool0oooo0><0oooo1000000]0?ooo`002@3oool00`000000oooo0?oo
o`3O0?ooo`@00000<@3oool00080ooooo`00000L000000<0oooo00090?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00h0oooo1@00000<0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`070?ooo`002@3oool00`000000oooo0?ooo`3E0?ooo`<00000?03oool000T0oooo
00<000000?ooo`3oool0dP3oool3000003l0oooo00090?ooo`030000003oool0oooo0<l0oooo0`00
00120?ooo`002@3oool200000<h0oooo0P0000150?ooo`002@3oool00`000000oooo0?ooo`3:0?oo
o`<00000A`3oool000T0oooo00<000000?ooo`3oool0a`3oool3000004X0oooo00090?ooo`030000
003oool0oooo0<@0oooo0`00001=0?ooo`002@3oool00`000000oooo0?ooo`310?ooo`<00000D03o
ool000T0oooo0P00002o0?ooo`<00000D`3oool000T0oooo00<000000?ooo`3oool0^`3oool30000
05H0oooo00090?ooo`030000003oool0oooo0;T0oooo0P00001I0?ooo`002@3oool00`000000oooo
0?ooo`2g0?ooo`800000F`3oool000T0oooo00<000000?ooo`3oool0]@3oool2000005d0oooo0009
0?ooo`800000]03oool2000005l0oooo00090?ooo`030000003oool0oooo0;40oooo0P00001Q0?oo
o`002@3oool00`000000oooo0?ooo`2_0?ooo`800000H`3oool000T0oooo00<000000?ooo`3oool0
[03oool3000006D0oooo00090?ooo`030000003oool0oooo0:X0oooo0P00001X0?ooo`002@3oool2
00000:T0oooo0P00001Z0?ooo`002@3oool00`000000oooo0?ooo`2V0?ooo`800000K03oool000T0
oooo00<000000?ooo`3oool0Y03oool2000006h0oooo00020?ooo`8000001@3oool00`000000oooo
0?ooo`2R0?ooo`800000L03oool000050?ooo`000000oooo0?ooo`000000103oool00`000000oooo
0?ooo`2P0?ooo`800000LP3oool000@0oooo00<000000?ooo`3oool00P3oool3000009h0oooo0P00
001d0?ooo`000P3oool2000000D0oooo00<000000?ooo`3oool0W03oool2000007H0oooo00040?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0V`3oool00`000000oooo0?ooo`1f0?oo
o`0000D0oooo0000003oool0oooo000000040?ooo`030000003oool0oooo09T0oooo0P00001i0?oo
o`000P3oool2000000D0oooo00<000000?ooo`3oool0V03oool00`000000oooo0?ooo`1i0?ooo`00
2@3oool2000009L0oooo0P00001l0?ooo`002@3oool00`000000oooo0?ooo`2D0?ooo`800000OP3o
ool000T0oooo00<000000?ooo`3oool0T`3oool00`000000oooo0?ooo`1n0?ooo`002@3oool00`00
0000oooo0?ooo`2A0?ooo`800000P@3oool000T0oooo00<000000?ooo`3oool0T03oool00`000000
oooo0?ooo`210?ooo`002@3oool2000008l0oooo0P0000240?ooo`002@3oool00`000000oooo0?oo
o`2=0?ooo`030000003oool0oooo08@0oooo00090?ooo`030000003oool0oooo08/0oooo0P000027
0?ooo`002@3oool00`000000oooo0?ooo`290?ooo`800000R@3oool000T0oooo00<000000?ooo`3o
ool0R03oool00`000000oooo0?ooo`290?ooo`002@3oool2000008L0oooo0P00002<0?ooo`002@3o
ool00`000000oooo0?ooo`250?ooo`030000003oool0oooo08`0oooo00090?ooo`030000003oool0
oooo08<0oooo0P00002?0?ooo`002@3oool00`000000oooo0?ooo`220?ooo`030000003oool0oooo
08l0oooo00090?ooo`030000003oool0oooo0840oooo00<000000?ooo`3oool0T03oool000T0oooo
0P0000200?ooo`800000T`3oool000T0oooo00<000000?ooo`3oool0OP3oool00`000000oooo0?oo
o`2C0?ooo`002@3oool00`000000oooo0?ooo`1m0?ooo`030000003oool0oooo09@0oooo00090?oo
o`030000003oool0oooo07/0oooo0P00002G0?ooo`000`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo07X0oooo00<000000?ooo`3oool0U`3oool000<0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo09P0oooo0005000000@0oooo0`00
001g0?ooo`800000V`3oool000040000003oool0oooo000000D0oooo00<000000?ooo`3oool0MP3o
ool00`000000oooo0?ooo`2K0?ooo`0000@0oooo0000003oool000001@3oool00`000000oooo0?oo
o`1e0?ooo`030000003oool0oooo09`0oooo00020?ooo`8000001@3oool00`000000oooo0?ooo`1d
0?ooo`030000003oool0oooo09d0oooo00030?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0L`3oool00`000000oooo0?ooo`2N0?ooo`002@3oool200000780oooo0P00002Q0?ooo`00
2@3oool00`000000oooo0?ooo`1`0?ooo`030000003oool0oooo0:40oooo00090?ooo`030000003o
ool0oooo06l0oooo00<000000?ooo`3oool0XP3oool000T0oooo00<000000?ooo`3oool0KP3oool0
0`000000oooo0?ooo`2S0?ooo`002@3oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo
0:@0oooo00090?ooo`800000K@3oool00`000000oooo0?ooo`2U0?ooo`002@3oool00`000000oooo
0?ooo`1[0?ooo`030000003oool0oooo0:H0oooo00090?ooo`030000003oool0oooo06T0oooo0P00
002Y0?ooo`002@3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo0:T0oooo00090?oo
o`030000003oool0oooo06L0oooo00<000000?ooo`3oool0ZP3oool000T0oooo0P00001W0?ooo`03
0000003oool0oooo0:/0oooo00090?ooo`030000003oool0oooo06D0oooo00<000000?ooo`3oool0
[03oool000T0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`2]0?ooo`002@3oool0
0`000000oooo0?ooo`1S0?ooo`030000003oool0oooo0:h0oooo00090?ooo`030000003oool0oooo
0680oooo00<000000?ooo`3oool0[`3oool000T0oooo0P00001R0?ooo`030000003oool0oooo0;00
oooo00090?ooo`030000003oool0oooo0600oooo00<000000?ooo`3oool0/@3oool000T0oooo00<0
00000?ooo`3oool0G`3oool00`000000oooo0?ooo`2b0?ooo`0000@0oooo00000000000000001@3o
ool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo0;<0oooo00040?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`2d0?ooo`00103oool00`00
0000oooo0?ooo`020?ooo`<00000G03oool00`000000oooo0?ooo`2e0?ooo`0000@0oooo00000000
000000001@3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo0;H0oooo00000`3oool0
00000?ooo`060?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool0]`3oool000030?oo
o`000000oooo00H0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`2h0?ooo`0000<0
oooo0000000000000P0000040?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool0^@3o
ool000T0oooo0P00001H0?ooo`030000003oool0oooo0;X0oooo00090?ooo`030000003oool0oooo
05H0oooo00<000000?ooo`3oool0^`3oool000T0oooo00<000000?ooo`3oool0E@3oool00`000000
oooo0?ooo`2l0?ooo`002@3oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo0;d0oooo
00090?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool0_P3oool000T0oooo0P00001C
0?ooo`030000003oool0oooo0;l0oooo00090?ooo`030000003oool0oooo0540oooo00<000000?oo
o`3oool0`03oool000T0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`310?ooo`00
2@3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo0<80oooo00090?ooo`030000003o
ool0oooo04h0oooo00<000000?ooo`3oool0``3oool000T0oooo0P00001>0?ooo`030000003oool0
oooo0<@0oooo00090?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0a@3oool000T0
oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`360?ooo`002@3oool00`000000oooo
0?ooo`1:0?ooo`030000003oool0oooo0<L0oooo00090?ooo`030000003oool0oooo04T0oooo00<0
00000?ooo`3oool0b03oool000T0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`38
0?ooo`002@3oool2000004T0oooo00<000000?ooo`3oool0b@3oool000T0oooo00<000000?ooo`3o
ool0A`3oool00`000000oooo0?ooo`3:0?ooo`002@3oool00`000000oooo0?ooo`160?ooo`030000
003oool0oooo0</0oooo00020?ooo`8000001@3oool00`000000oooo0?ooo`150?ooo`030000003o
ool0oooo0<`0oooo00001@3oool000000?ooo`3oool0000000@0oooo00<000000?ooo`3oool0A03o
ool00`000000oooo0?ooo`3=0?ooo`0000D0oooo0000003oool0oooo000000040?ooo`<00000@`3o
ool00`000000oooo0?ooo`3>0?ooo`0000@0oooo00000000000000001@3oool00`000000oooo0?oo
o`120?ooo`030000003oool0oooo0<l0oooo00000`3oool000000?ooo`060?ooo`030000003oool0
oooo0440oooo00<000000?ooo`3oool0d03oool000030?ooo`000000oooo00H0oooo00<000000?oo
o`3oool0@03oool00`000000oooo0?ooo`3A0?ooo`000P3oool3000000@0oooo00<000000?ooo`3o
ool0?`3oool00`000000oooo0?ooo`3B0?ooo`002@3oool2000003l0oooo00<000000?ooo`3oool0
d`3oool000T0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`3C0?ooo`002@3oool0
0`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0=@0oooo00090?ooo`030000003oool0oooo
03`0oooo00<000000?ooo`3oool0e@3oool000T0oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`3F0?ooo`002@3oool2000003/0oooo00<000000?ooo`3oool0e`3oool000T0oooo00<0
00000?ooo`3oool0>@3oool00`000000oooo0?ooo`3H0?ooo`002@3oool00`000000oooo0?ooo`0h
0?ooo`030000003oool0oooo0=T0oooo00090?ooo`030000003oool0oooo03L0oooo00<000000?oo
o`3oool0fP3oool000T0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`3K0?ooo`00
2@3oool2000003H0oooo00<000000?ooo`3oool0g03oool000T0oooo00<000000?ooo`3oool0=03o
ool00`000000oooo0?ooo`3M0?ooo`002@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo0=h0oooo00090?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0gP3oool000T0
oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`3O0?ooo`002@3oool200000380oooo
00<000000?ooo`3oool0h03oool000T0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?oo
o`3Q0?ooo`002@3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0>80oooo00020?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`3S0?oo
o`000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3o
ool0i03oool000<0oooo00<000000?ooo`3oool00`3oool3000002`0oooo00<000000?ooo`3oool0
i@3oool000<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0[0?ooo`030000003o
ool0oooo0>H0oooo00040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`3W0?ooo`0000D0oooo0000003oool0oooo000000040?ooo`030000003oool0
oooo02T0oooo00<000000?ooo`3oool0j03oool000030?ooo`000000000000800000103oool00`00
0000oooo0?ooo`0W0?ooo`800000j`3oool000T0oooo0P00000W0?ooo`030000003oool0oooo0>/0
oooo00090?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0k03oool000T0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`3]0?ooo`002@3oool00`000000oooo0?ooo`0S
0?ooo`030000003oool0oooo0>h0oooo00090?ooo`030000003oool0oooo0280oooo00<000000?oo
o`3oool0k`3oool000T0oooo0P00000R0?ooo`030000003oool0oooo0?00oooo00090?ooo`030000
003oool0oooo01l0oooo0P00003c0?ooo`002@3oool00`000000oooo0?ooo`0N0?ooo`030000003o
ool0oooo0?<0oooo00090?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0m03oool0
00T0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3e0?ooo`002@3oool2000001`0
oooo00<000000?ooo`3oool0mP3oool000T0oooo00<000000?ooo`3oool06@3oool200000?T0oooo
00090?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0n@3oool000T0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`3j0?ooo`002@3oool00`000000oooo0?ooo`0E0?oo
o`800000o@3oool000T0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`3m0?ooo`00
2@3oool2000001<0oooo0P00003o0?ooo`40oooo00090?ooo`030000003oool0oooo0100oooo0P00
003o0?ooo`<0oooo00090?ooo`030000003oool0oooo00h0oooo0P00003o0?ooo`D0oooo00020?oo
o`8000001@3oool00`000000oooo0?ooo`0<0?ooo`800000o`3oool70?ooo`0000D0oooo0000003o
ool0oooo000000040?ooo`030000003oool0oooo00X0oooo0P00003o0?ooo`T0oooo00001@3oool0
00000?ooo`3oool0000000@0oooo0`0000070?ooo`<00000o`3oool;0?ooo`000P3oool2000000D0
oooo00<000000?ooo`3oool01@3oool200000?l0oooo3P3oool000050?ooo`000000oooo0?ooo`00
0000103oool00`000000oooo0?ooo`0500000?l0oooo403oool000050?ooo`000000oooo0?ooo`00
0000103oool00`000000oooo0?ooo`3o0?oooaD0oooo00020?ooo`8000001@3oool00`000000oooo
0?ooo`3o0?oooaD0oooo00090?ooo`800000o`3ooolF0?ooo`002@3oool00`000000oooo0?ooo`3o
0?oooaD0oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0283824, 1.37451, \
0.0029798, 0.03918}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      mm[1, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 10], {th, 
        0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.603319 0.512513 [
[.2619 .59082 -9 -9 ]
[.2619 .59082 9 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .09081 -12 -4.5 ]
[.01131 .09081 0 4.5 ]
[.01131 .19331 -24 -4.5 ]
[.01131 .19331 0 4.5 ]
[.01131 .29581 -24 -4.5 ]
[.01131 .29581 0 4.5 ]
[.01131 .39831 -24 -4.5 ]
[.01131 .39831 0 4.5 ]
[.01131 .50082 -24 -4.5 ]
[.01131 .50082 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(0.2)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(0.4)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.6)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(0.8)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.08333 .60332 m
.08333 .60707 L
s
.14286 .60332 m
.14286 .60707 L
s
.20238 .60332 m
.20238 .60707 L
s
.32143 .60332 m
.32143 .60707 L
s
.38095 .60332 m
.38095 .60707 L
s
.44048 .60332 m
.44048 .60707 L
s
.55952 .60332 m
.55952 .60707 L
s
.61905 .60332 m
.61905 .60707 L
s
.67857 .60332 m
.67857 .60707 L
s
.79762 .60332 m
.79762 .60707 L
s
.85714 .60332 m
.85714 .60707 L
s
.91667 .60332 m
.91667 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .09081 m
.03006 .09081 L
s
[(-1)] .01131 .09081 1 0 Mshowa
.02381 .19331 m
.03006 .19331 L
s
[(-0.8)] .01131 .19331 1 0 Mshowa
.02381 .29581 m
.03006 .29581 L
s
[(-0.6)] .01131 .29581 1 0 Mshowa
.02381 .39831 m
.03006 .39831 L
s
[(-0.4)] .01131 .39831 1 0 Mshowa
.02381 .50082 m
.03006 .50082 L
s
[(-0.2)] .01131 .50082 1 0 Mshowa
.125 Mabswid
.02381 .11643 m
.02756 .11643 L
s
.02381 .14206 m
.02756 .14206 L
s
.02381 .16768 m
.02756 .16768 L
s
.02381 .21893 m
.02756 .21893 L
s
.02381 .24456 m
.02756 .24456 L
s
.02381 .27019 m
.02756 .27019 L
s
.02381 .32144 m
.02756 .32144 L
s
.02381 .34706 m
.02756 .34706 L
s
.02381 .37269 m
.02756 .37269 L
s
.02381 .42394 m
.02756 .42394 L
s
.02381 .44957 m
.02756 .44957 L
s
.02381 .47519 m
.02756 .47519 L
s
.02381 .52644 m
.02756 .52644 L
s
.02381 .55207 m
.02756 .55207 L
s
.02381 .57769 m
.02756 .57769 L
s
.02381 .06518 m
.02756 .06518 L
s
.02381 .03955 m
.02756 .03955 L
s
.02381 .01393 m
.02756 .01393 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .01472 m
.0403 .01531 L
.04458 .01602 L
.05429 .01826 L
.06375 .02124 L
.07387 .02526 L
.09565 .03671 L
.11547 .05015 L
.15491 .0839 L
.19284 .12262 L
.2332 .16733 L
.27205 .2112 L
.30942 .25235 L
.3492 .29385 L
.38749 .33084 L
.4282 .36669 L
.46741 .39781 L
.50512 .42472 L
.54526 .45035 L
.5839 .47236 L
.62496 .4932 L
.66453 .51107 L
.7026 .52646 L
.74309 .54113 L
.78208 .55381 L
.81958 .56482 L
.8595 .57543 L
.89792 .58468 L
.93877 .59362 L
.97619 .60109 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`030000
003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`006`3oool2
00000?l0oooo103oool001/0oooo00<000000?ooo`3oool01P00003l0?ooo`006`3oool00`000000
oooo0?ooo`060?ooo`<00000n@3oool001/0oooo00<000000?ooo`3oool02@3oool200000?L0oooo
000K0?ooo`030000003oool0oooo00/0oooo0P00003e0?ooo`006`3oool00`000000oooo0?ooo`0=
0?ooo`800000l`3oool001/0oooo00<000000?ooo`3oool03`3oool200000?40oooo000K0?ooo`80
00004P3oool00`000000oooo0?ooo`3^0?ooo`006`3oool00`000000oooo0?ooo`0B0?ooo`800000
kP3oool001/0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`3[0?ooo`006`3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0>X0oooo000K0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0j@3oool001/0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`3X0?ooo`006`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0>L0oooo
000K0?ooo`8000006P3oool00`000000oooo0?ooo`3V0?ooo`006`3oool00`000000oooo0?ooo`0J
0?ooo`800000iP3oool001/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3S0?oo
o`006`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0>80oooo000K0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool0h@3oool001<0oooo100000040?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool0h03oool001D0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0=l0oooo000E0?ooo`030000003oool0oooo
00<0oooo0P00000R0?ooo`030000003oool0oooo0=h0oooo000=0?ooo`@00000103oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0g@3oool001D0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0=`0oooo
000C0?ooo`<000001@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0=/0oooo000E
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`3J
0?ooo`006`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0=T0oooo000K0?ooo`80
0000:03oool00`000000oooo0?ooo`3H0?ooo`006`3oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo0=L0oooo000K0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0eP3o
ool001/0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3E0?ooo`006`3oool00`00
0000oooo0?ooo`0[0?ooo`030000003oool0oooo0=@0oooo000K0?ooo`030000003oool0oooo02`0
oooo00<000000?ooo`3oool0d`3oool001/0oooo00<000000?ooo`3oool0;@3oool00`000000oooo
0?ooo`3B0?ooo`006`3oool2000002l0oooo00<000000?ooo`3oool0d@3oool001/0oooo00<00000
0?ooo`3oool0;P3oool00`000000oooo0?ooo`3A0?ooo`006`3oool00`000000oooo0?ooo`0_0?oo
o`030000003oool0oooo0=00oooo000K0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3o
ool0c`3oool001/0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`3>0?ooo`006`3o
ool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0<d0oooo000K0?ooo`030000003oool0
oooo03<0oooo00<000000?ooo`3oool0c03oool001/0oooo0P00000e0?ooo`030000003oool0oooo
0</0oooo000K0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0bP3oool001/0oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`390?ooo`006`3oool00`000000oooo0?oo
o`0g0?ooo`030000003oool0oooo0<P0oooo000K0?ooo`030000003oool0oooo03P0oooo00<00000
0?ooo`3oool0a`3oool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000050?oo
o`030000003oool0oooo03T0oooo00<000000?ooo`3oool0aP3oool000L0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo03T0oooo00<00000
0?ooo`3oool0aP3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00040?ooo`800000>`3oool00`000000oooo0?ooo`350?ooo`0000<0oooo0000000000000P000002
0?ooo`040000003oool0oooo000000T0oooo0P0000050?ooo`030000003oool0oooo03/0oooo00<0
00000?ooo`3oool0a03oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0``3oool000L0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo03d0
oooo00<000000?ooo`3oool0`P3oool000P0oooo0P00000:0?ooo`8000001@3oool00`000000oooo
0?ooo`0n0?ooo`030000003oool0oooo0<40oooo000K0?ooo`030000003oool0oooo03l0oooo00<0
00000?ooo`3oool0`03oool001/0oooo0P0000110?ooo`030000003oool0oooo0;l0oooo000K0?oo
o`030000003oool0oooo0440oooo00<000000?ooo`3oool0_P3oool001/0oooo00<000000?ooo`3o
ool0@P3oool00`000000oooo0?ooo`2m0?ooo`006`3oool00`000000oooo0?ooo`130?ooo`030000
003oool0oooo0;`0oooo000K0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0_03o
ool001/0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`2k0?ooo`006`3oool00`00
0000oooo0?ooo`150?ooo`030000003oool0oooo0;X0oooo000K0?ooo`800000A`3oool00`000000
oooo0?ooo`2i0?ooo`006`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0;P0oooo
000K0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0]`3oool001/0oooo00<00000
0?ooo`3oool0B@3oool00`000000oooo0?ooo`2f0?ooo`006`3oool00`000000oooo0?ooo`1:0?oo
o`030000003oool0oooo0;D0oooo000K0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3o
ool0]03oool001/0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`2c0?ooo`006`3o
ool2000004h0oooo00<000000?ooo`3oool0/P3oool001/0oooo00<000000?ooo`3oool0C@3oool0
0`000000oooo0?ooo`2b0?ooo`006`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo
0;40oooo000K0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0/03oool001/0oooo
00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`2_0?ooo`00203oool2000000@0oooo00<0
00000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?oo
o`2^0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo
00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`2]0?ooo`001`3oool010000000oooo0?oo
o`0000080?ooo`040000003oool0oooo000000@0oooo0P00001D0?ooo`030000003oool0oooo0:`0
oooo00000`3oool000000000000200000080oooo00@000000?ooo`3oool00000203oool3000000D0
oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`2[0?ooo`001`3oool010000000oooo
0?ooo`0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0E@3oool00`000000
oooo0?ooo`2Z0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`2Y0?ooo`00203oool2000000X0oooo
0`0000040?ooo`030000003oool0oooo05L0oooo0P00002Y0?ooo`006`3oool00`000000oooo0?oo
o`1I0?ooo`030000003oool0oooo0:H0oooo000K0?ooo`800000F`3oool00`000000oooo0?ooo`2U
0?ooo`006`3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo0:@0oooo000K0?ooo`03
0000003oool0oooo05`0oooo00<000000?ooo`3oool0X`3oool001/0oooo00<000000?ooo`3oool0
G@3oool00`000000oooo0?ooo`2R0?ooo`006`3oool00`000000oooo0?ooo`1N0?ooo`030000003o
ool0oooo0:40oooo000K0?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3oool0X03oool0
01/0oooo00<000000?ooo`3oool0H03oool00`000000oooo0?ooo`2O0?ooo`006`3oool200000680
oooo00<000000?ooo`3oool0WP3oool001/0oooo00<000000?ooo`3oool0HP3oool2000009h0oooo
000K0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool0V`3oool001/0oooo00<00000
0?ooo`3oool0I@3oool00`000000oooo0?ooo`2J0?ooo`006`3oool00`000000oooo0?ooo`1V0?oo
o`030000003oool0oooo09T0oooo000K0?ooo`030000003oool0oooo06L0oooo00<000000?ooo`3o
ool0V03oool001/0oooo00<000000?ooo`3oool0J03oool2000009P0oooo000K0?ooo`800000J`3o
ool00`000000oooo0?ooo`2E0?ooo`006`3oool00`000000oooo0?ooo`1[0?ooo`030000003oool0
oooo09@0oooo000K0?ooo`030000003oool0oooo06`0oooo0P00002D0?ooo`006`3oool00`000000
oooo0?ooo`1^0?ooo`030000003oool0oooo0940oooo000K0?ooo`030000003oool0oooo06l0oooo
00<000000?ooo`3oool0T03oool000P0oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`1`0?ooo`800000T03oool000L0oooo00@00000
0?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0780oooo00<0
00000?ooo`3oool0S@3oool000L0oooo00@000000?ooo`3oool000001`3oool5000000@0oooo0P00
001d0?ooo`030000003oool0oooo08`0oooo00000`3oool000000000000200000080oooo00@00000
0?ooo`3oool000001`3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo07@0oooo
0P00002<0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0000000D0oooo
00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`290?ooo`001`3oool010000000oooo0?oo
o`0000090?ooo`8000001@3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo08P0oooo
00080?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo07P0oooo0P00
00280?ooo`006`3oool00`000000oooo0?ooo`1j0?ooo`030000003oool0oooo08D0oooo000K0?oo
o`800000O03oool00`000000oooo0?ooo`240?ooo`006`3oool00`000000oooo0?ooo`1l0?ooo`80
0000Q03oool001/0oooo00<000000?ooo`3oool0OP3oool00`000000oooo0?ooo`210?ooo`006`3o
ool00`000000oooo0?ooo`1o0?ooo`800000P@3oool001/0oooo00<000000?ooo`3oool0P@3oool2
000007l0oooo000K0?ooo`030000003oool0oooo08<0oooo00<000000?ooo`3oool0O03oool001/0
oooo00<000000?ooo`3oool0Q03oool2000007`0oooo000K0?ooo`800000Q`3oool00`000000oooo
0?ooo`1i0?ooo`006`3oool00`000000oooo0?ooo`270?ooo`800000N@3oool001/0oooo00<00000
0?ooo`3oool0R@3oool2000007L0oooo000K0?ooo`030000003oool0oooo08/0oooo00<000000?oo
o`3oool0M03oool001/0oooo00<000000?ooo`3oool0S03oool2000007@0oooo000K0?ooo`030000
003oool0oooo08h0oooo0P00001b0?ooo`006`3oool00`000000oooo0?ooo`2@0?ooo`030000003o
ool0oooo06l0oooo000K0?ooo`800000TP3oool2000006l0oooo000K0?ooo`030000003oool0oooo
09<0oooo0P00001]0?ooo`006`3oool00`000000oooo0?ooo`2E0?ooo`800000J`3oool001/0oooo
00<000000?ooo`3oool0U`3oool2000006T0oooo000K0?ooo`030000003oool0oooo09T0oooo0P00
001W0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool4000000@0oooo00<00000
0?ooo`3oool0V`3oool2000006D0oooo00070?ooo`040000003oool0oooo000000P0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`2M0?ooo`800000H`3oool000L0oooo00@000000?oo
o`3oool000002@3oool00`000000oooo0?ooo`040?ooo`800000X03oool300000600oooo00000`3o
ool000000000000200000080oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0:80oooo0P00001N0?ooo`001`3oool010000000oooo0?ooo`00000;
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0Y03oool3000005/0oooo00070?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?oo
o`2W0?ooo`800000F@3oool000P0oooo0P00000:0?ooo`8000001@3oool00`000000oooo0?ooo`2Y
0?ooo`<00000EP3oool001/0oooo00<000000?ooo`3oool0[03oool2000005@0oooo000K0?ooo`80
0000[`3oool300000540oooo000K0?ooo`030000003oool0oooo0;40oooo0`00001>0?ooo`006`3o
ool00`000000oooo0?ooo`2d0?ooo`800000C03oool001/0oooo00<000000?ooo`3oool0]P3oool3
000004T0oooo000K0?ooo`030000003oool0oooo0;T0oooo0`0000160?ooo`006`3oool00`000000
oooo0?ooo`2l0?ooo`<00000@`3oool001/0oooo00<000000?ooo`3oool0_`3oool4000003l0oooo
000K0?ooo`800000a03oool3000003`0oooo000K0?ooo`030000003oool0oooo0<H0oooo0`00000i
0?ooo`006`3oool00`000000oooo0?ooo`390?ooo`@00000=@3oool001/0oooo00<000000?ooo`3o
ool0=03oool2000000@0oooo00<000000?ooo`3oool00P3oool400000300oooo0P0000040?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0;`3oool2000000@0oooo00<000000?ooo`3o
ool00`3oool2000000d0oooo0`00000Q0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`80
00000`3oool001/0oooo00<000000?ooo`3oool0<`3oool010000000oooo0?ooo`0000080?ooo`03
0000003oool0oooo0300oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0^0?oo
o`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000003`3oool4000001`0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`006`3oool00`000000oooo
0?ooo`0c0?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool0;`3oool010000000
oooo0?ooo`0000070?ooo`D00000;`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo000001<0oooo1000000H0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
00000P3oool001/0oooo00<000000?ooo`3oool0<`3oool010000000oooo0?ooo`00000:0?ooo`03
0000003oool0oooo02h0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00000`
0?ooo`040000003oool0oooo000000P0oooo0`00000H0?ooo`<000005@3oool010000000oooo0?oo
o`0000090?ooo`8000000`3oool001/0oooo0P00000d0?ooo`040000003oool0oooo000000/0oooo
00<000000?ooo`3oool0;@3oool010000000oooo0?ooo`0000080?ooo`030000003oool000000300
oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0K0?ooo`<000004P3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo000K0?ooo`030000003oool0
oooo03<0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000_0?ooo`040000
003oool0oooo000000T0oooo0P00000`0?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`3oool07P3oool4000000h0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
00020?ooo`006`3oool00`000000oooo0?ooo`0d0?ooo`8000002P3oool200000340oooo0P00000;
0?ooo`030000003oool0oooo02l0oooo0P00000:0?ooo`<000008@3oool4000000/0oooo0P00000:
0?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0iP3oool6000001H0oooo000K0?ooo`03
0000003oool0oooo0>`0oooo1@00000A0?ooo`006`3oool00`000000oooo0?ooo`3a0?ooo`D00000
303oool001/0oooo00<000000?ooo`3oool0mP3oool3000000T0oooo000E0?ooool000002@000003
0?ooo`006`3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool0?03oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00L0oooo000K0?ooo`03
0000003oool0oooo0?l0oooo0`3oool001/0oooo00<000000?ooo`3oool0o`3oool30?ooo`00o`3o
oolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3o
oolQ0?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0870867, -1.22997, \
0.00319096, 0.00741206}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      mm[3, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 10], {th, 
        0.01, 0.8}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.603319 0.170838 [
[.2619 .59082 -9 -9 ]
[.2619 .59082 9 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.7381 .59082 -9 -9 ]
[.7381 .59082 9 0 ]
[.97619 .59082 -9 -9 ]
[.97619 .59082 9 0 ]
[.01131 .00539 -24 -4.5 ]
[.01131 .00539 0 4.5 ]
[.01131 .09081 -12 -4.5 ]
[.01131 .09081 0 4.5 ]
[.01131 .17622 -24 -4.5 ]
[.01131 .17622 0 4.5 ]
[.01131 .26164 -12 -4.5 ]
[.01131 .26164 0 4.5 ]
[.01131 .34706 -24 -4.5 ]
[.01131 .34706 0 4.5 ]
[.01131 .43248 -12 -4.5 ]
[.01131 .43248 0 4.5 ]
[.01131 .5179 -24 -4.5 ]
[.01131 .5179 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .60332 m
.2619 .60957 L
s
[(0.2)] .2619 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(0.4)] .5 .59082 0 1 Mshowa
.7381 .60332 m
.7381 .60957 L
s
[(0.6)] .7381 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(0.8)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.08333 .60332 m
.08333 .60707 L
s
.14286 .60332 m
.14286 .60707 L
s
.20238 .60332 m
.20238 .60707 L
s
.32143 .60332 m
.32143 .60707 L
s
.38095 .60332 m
.38095 .60707 L
s
.44048 .60332 m
.44048 .60707 L
s
.55952 .60332 m
.55952 .60707 L
s
.61905 .60332 m
.61905 .60707 L
s
.67857 .60332 m
.67857 .60707 L
s
.79762 .60332 m
.79762 .60707 L
s
.85714 .60332 m
.85714 .60707 L
s
.91667 .60332 m
.91667 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .00539 m
.03006 .00539 L
s
[(-3.5)] .01131 .00539 1 0 Mshowa
.02381 .09081 m
.03006 .09081 L
s
[(-3)] .01131 .09081 1 0 Mshowa
.02381 .17622 m
.03006 .17622 L
s
[(-2.5)] .01131 .17622 1 0 Mshowa
.02381 .26164 m
.03006 .26164 L
s
[(-2)] .01131 .26164 1 0 Mshowa
.02381 .34706 m
.03006 .34706 L
s
[(-1.5)] .01131 .34706 1 0 Mshowa
.02381 .43248 m
.03006 .43248 L
s
[(-1)] .01131 .43248 1 0 Mshowa
.02381 .5179 m
.03006 .5179 L
s
[(-0.5)] .01131 .5179 1 0 Mshowa
.125 Mabswid
.02381 .02247 m
.02756 .02247 L
s
.02381 .03955 m
.02756 .03955 L
s
.02381 .05664 m
.02756 .05664 L
s
.02381 .07372 m
.02756 .07372 L
s
.02381 .10789 m
.02756 .10789 L
s
.02381 .12497 m
.02756 .12497 L
s
.02381 .14206 m
.02756 .14206 L
s
.02381 .15914 m
.02756 .15914 L
s
.02381 .19331 m
.02756 .19331 L
s
.02381 .21039 m
.02756 .21039 L
s
.02381 .22748 m
.02756 .22748 L
s
.02381 .24456 m
.02756 .24456 L
s
.02381 .27873 m
.02756 .27873 L
s
.02381 .29581 m
.02756 .29581 L
s
.02381 .3129 m
.02756 .3129 L
s
.02381 .32998 m
.02756 .32998 L
s
.02381 .36415 m
.02756 .36415 L
s
.02381 .38123 m
.02756 .38123 L
s
.02381 .39831 m
.02756 .39831 L
s
.02381 .4154 m
.02756 .4154 L
s
.02381 .44957 m
.02756 .44957 L
s
.02381 .46665 m
.02756 .46665 L
s
.02381 .48373 m
.02756 .48373 L
s
.02381 .50082 m
.02756 .50082 L
s
.02381 .53498 m
.02756 .53498 L
s
.02381 .55207 m
.02756 .55207 L
s
.02381 .56915 m
.02756 .56915 L
s
.02381 .58624 m
.02756 .58624 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .01472 m
.0403 .01531 L
.04458 .01602 L
.05429 .01826 L
.06375 .02124 L
.07387 .02526 L
.09565 .03671 L
.11547 .05015 L
.15491 .0839 L
.19284 .12262 L
.2332 .16733 L
.27205 .2112 L
.30942 .25235 L
.3492 .29385 L
.38749 .33084 L
.4282 .36669 L
.46741 .39781 L
.50512 .42472 L
.54526 .45035 L
.5839 .47236 L
.62496 .4932 L
.66453 .51107 L
.7026 .52646 L
.74309 .54113 L
.78208 .55381 L
.81958 .56482 L
.8595 .57543 L
.89792 .58468 L
.93877 .59362 L
.97619 .60109 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool0
0P3oool3000000D0oooo00<000000?ooo`3oool0o`3oool30?ooo`001`3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3oool30?ooo`002P3o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0080oooo0P00003o0?ooo`@0oooo0000
0`3oool0000000000002000000<0oooo0P0000090?ooo`<000001@3oool00`000000oooo0?ooo`3o
0?ooo`<0oooo000:0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0400000?h0oooo00070?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`040?ooo`800000o03oool000P0oooo0P0000090?ooo`@0
0000103oool2000000L0oooo0`00003i0?ooo`006`3oool00`000000oooo0?ooo`090?ooo`800000
m`3oool001/0oooo00<000000?ooo`3oool02`3oool200000?D0oooo000K0?ooo`030000003oool0
oooo00d0oooo0P00003c0?ooo`006`3oool00`000000oooo0?ooo`0?0?ooo`800000l@3oool001/0
oooo0P00000B0?ooo`800000k`3oool001/0oooo00<000000?ooo`3oool04`3oool200000>d0oooo
000K0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0jP3oool001/0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`3Y0?ooo`006`3oool2000001P0oooo00<000000?oo
o`3oool0j03oool001/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`3W0?ooo`00
6`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0>H0oooo000K0?ooo`030000003o
ool0oooo01X0oooo0P00003V0?ooo`006`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo0><0oooo000K0?ooo`8000007P3oool00`000000oooo0?ooo`3R0?ooo`006`3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo0>40oooo000D0?ooo`8000001@3oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo0>00oooo000C0?ooo`040000003oool0oooo000000@0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3O0?ooo`005P3oool00`000000oooo0?oo
o`020?ooo`8000008P3oool00`000000oooo0?ooo`3N0?ooo`003@3oool4000000<0oooo0P000005
0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0g@3oool001H0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0=`0oooo000C0?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`3K0?oo
o`00503oool2000000D0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`3K0?ooo`00
6`3oool2000002H0oooo00<000000?ooo`3oool0fP3oool001/0oooo00<000000?ooo`3oool09P3o
ool00`000000oooo0?ooo`3I0?ooo`006`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0
oooo0=P0oooo000K0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0e`3oool001/0
oooo0P00000Z0?ooo`030000003oool0oooo0=H0oooo000K0?ooo`030000003oool0oooo02X0oooo
00<000000?ooo`3oool0e@3oool001/0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?oo
o`3D0?ooo`006`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0=<0oooo000K0?oo
o`030000003oool0oooo02d0oooo00<000000?ooo`3oool0dP3oool001/0oooo0P00000_0?ooo`03
0000003oool0oooo0=40oooo000K0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
d03oool001/0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`3?0?ooo`006`3oool0
0`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0<h0oooo000K0?ooo`800000<`3oool00`00
0000oooo0?ooo`3=0?ooo`006`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0<`0
oooo000K0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0b`3oool000L0oooo1000
00030?ooo`030000003oool0oooo0080oooo0`0000050?ooo`030000003oool0oooo03D0oooo00<0
00000?ooo`3oool0bP3oool000L0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0b@3oool000P0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0?ooo`020?ooo`800000=`3oool00`000000oooo0?ooo`390?oo
o`0000<0oooo0000000000000P0000040?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`03
0000003oool0oooo03L0oooo00<000000?ooo`3oool0b03oool000X0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0
a`3oool000L0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo03T0oooo00<000000?ooo`3oool0aP3oool000P0oooo0P0000090?ooo`@00000103o
ool2000003/0oooo00<000000?ooo`3oool0a@3oool001/0oooo00<000000?ooo`3oool0>`3oool0
0`000000oooo0?ooo`340?ooo`006`3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo
0<@0oooo000K0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0``3oool001/0oooo
00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`320?ooo`006`3oool2000003l0oooo00<0
00000?ooo`3oool0`@3oool001/0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`30
0?ooo`006`3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo0;l0oooo000K0?ooo`03
0000003oool0oooo0440oooo00<000000?ooo`3oool0_P3oool001/0oooo0P0000130?ooo`030000
003oool0oooo0;d0oooo000K0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0_03o
ool001/0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`2l0?ooo`006`3oool00`00
0000oooo0?ooo`140?ooo`030000003oool0oooo0;/0oooo000K0?ooo`030000003oool0oooo04D0
oooo00<000000?ooo`3oool0^P3oool001/0oooo0P0000170?ooo`030000003oool0oooo0;T0oooo
000K0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0^03oool001<0oooo10000004
0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0]`3oool001<0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo0;H0oooo000D0?ooo`03
0000003oool0oooo00@0oooo0P00001;0?ooo`030000003oool0oooo0;D0oooo000=0?ooo`@00000
103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0
]03oool001H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1<0?ooo`030000003o
ool0oooo0;<0oooo000C0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0C@3o
ool00`000000oooo0?ooo`2b0?ooo`00503oool2000000D0oooo00<000000?ooo`3oool0C@3oool0
0`000000oooo0?ooo`2b0?ooo`006`3oool2000004l0oooo00<000000?ooo`3oool0/@3oool001/0
oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`2`0?ooo`006`3oool00`000000oooo
0?ooo`1@0?ooo`030000003oool0oooo0:l0oooo000K0?ooo`030000003oool0oooo0540oooo00<0
00000?ooo`3oool0[P3oool001/0oooo0P00001C0?ooo`030000003oool0oooo0:d0oooo000K0?oo
o`030000003oool0oooo05<0oooo00<000000?ooo`3oool0[03oool001/0oooo00<000000?ooo`3o
ool0E03oool200000:`0oooo000K0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0
Z@3oool001/0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`2X0?ooo`006`3oool2
000005T0oooo00<000000?ooo`3oool0Y`3oool001/0oooo00<000000?ooo`3oool0F@3oool00`00
0000oooo0?ooo`2V0?ooo`006`3oool00`000000oooo0?ooo`1J0?ooo`800000YP3oool001/0oooo
00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`2S0?ooo`006`3oool2000005h0oooo00<0
00000?ooo`3oool0XP3oool001/0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`2Q
0?ooo`006`3oool00`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0:00oooo00070?ooo`@0
00000`3oool00`000000oooo0?ooo`020?ooo`<000001@3oool00`000000oooo0?ooo`1P0?ooo`03
0000003oool0oooo09l0oooo00090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo09h0oooo00090?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool00P3oool2000006<0oooo00<000000?ooo`3oool0W@3o
ool000030?ooo`000000000000800000103oool00`000000oooo0?ooo`070?ooo`<000001@3oool0
0`000000oooo0?ooo`1S0?ooo`030000003oool0oooo09`0oooo00090?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo
09/0oooo00070?ooo`<000002@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo06D0
oooo00<000000?ooo`3oool0VP3oool000T0oooo00<000000?ooo`3oool01`3oool4000000@0oooo
0P00001W0?ooo`030000003oool0oooo09T0oooo000K0?ooo`030000003oool0oooo06L0oooo00<0
00000?ooo`3oool0V03oool001/0oooo00<000000?ooo`3oool0J03oool2000009P0oooo000K0?oo
o`030000003oool0oooo06X0oooo00<000000?ooo`3oool0U@3oool001/0oooo00<000000?ooo`3o
ool0J`3oool00`000000oooo0?ooo`2D0?ooo`006`3oool2000006d0oooo0P00002D0?ooo`006`3o
ool00`000000oooo0?ooo`1^0?ooo`030000003oool0oooo0940oooo000K0?ooo`030000003oool0
oooo06l0oooo00<000000?ooo`3oool0T03oool001/0oooo00<000000?ooo`3oool0L03oool20000
0900oooo000K0?ooo`800000L`3oool00`000000oooo0?ooo`2=0?ooo`006`3oool00`000000oooo
0?ooo`1c0?ooo`030000003oool0oooo08`0oooo000K0?ooo`030000003oool0oooo07@0oooo0P00
002<0?ooo`006`3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo08T0oooo000K0?oo
o`030000003oool0oooo07L0oooo00<000000?ooo`3oool0R03oool001/0oooo0P00001i0?ooo`80
0000R03oool001/0oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`250?ooo`004`3o
ool4000000@0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`240?ooo`005@3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo07`0oooo0P0000240?ooo`005@3oool00`00
0000oooo0?ooo`030?ooo`800000O`3oool00`000000oooo0?ooo`210?ooo`003@3oool4000000@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1o0?ooo`800000P@3oool001D0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`210?ooo`800000O`3oool001<0oooo0`00
00050?ooo`030000003oool0oooo08<0oooo00<000000?ooo`3oool0O03oool001D0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`240?ooo`800000O03oool001/0oooo0P0000270?oo
o`030000003oool0oooo07T0oooo000K0?ooo`030000003oool0oooo08L0oooo0P00001i0?ooo`00
6`3oool00`000000oooo0?ooo`290?ooo`800000M`3oool001/0oooo00<000000?ooo`3oool0R`3o
ool00`000000oooo0?ooo`1d0?ooo`006`3oool2000008d0oooo0P00001d0?ooo`006`3oool00`00
0000oooo0?ooo`2>0?ooo`800000LP3oool001/0oooo00<000000?ooo`3oool0T03oool200000700
oooo000K0?ooo`030000003oool0oooo0980oooo0P00001^0?ooo`006`3oool00`000000oooo0?oo
o`2D0?ooo`800000K03oool001/0oooo0P00002G0?ooo`800000JP3oool001/0oooo00<000000?oo
o`3oool0V03oool2000006P0oooo000K0?ooo`030000003oool0oooo09X0oooo0P00001V0?ooo`00
6`3oool00`000000oooo0?ooo`2L0?ooo`800000I03oool001/0oooo0P00002O0?ooo`800000HP3o
ool001/0oooo00<000000?ooo`3oool0X03oool200000600oooo000K0?ooo`030000003oool0oooo
0:80oooo0P00001N0?ooo`00203oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0
oooo00<000000?ooo`3oool0Y03oool3000005/0oooo00070?ooo`040000003oool0oooo000000/0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2W0?ooo`800000F@3oool000L0oooo
00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?ooo`800000ZP3oool3000005H0
oooo00000`3oool000000000000200000080oooo00@000000?ooo`3oool00000203oool3000000D0
oooo00<000000?ooo`3oool0[03oool2000005@0oooo00070?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2^0?ooo`<00000D@3oool000L0oooo
00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0;40
oooo0`00001>0?ooo`00203oool2000000T0oooo100000040?ooo`800000]@3oool2000004`0oooo
000K0?ooo`030000003oool0oooo0;H0oooo0`0000190?ooo`006`3oool00`000000oooo0?ooo`2i
0?ooo`<00000AP3oool001/0oooo00<000000?ooo`3oool0_03oool3000004<0oooo000K0?ooo`03
0000003oool0oooo0;l0oooo1000000o0?ooo`006`3oool200000<@0oooo0`00000l0?ooo`006`3o
ool00`000000oooo0?ooo`360?ooo`<00000>@3oool001/0oooo00<000000?ooo`3oool0b@3oool4
000003D0oooo000K0?ooo`030000003oool0oooo03@0oooo0P0000040?ooo`030000003oool0oooo
0080oooo1000000`0?ooo`800000103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
02l0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P00000=0?ooo`<000008@3oool20000
00@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo000K0?ooo`800000=03oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo0300oooo00@000000?ooo`3oool000002P3oool0
0`000000oooo0?ooo`0^0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000
3`3oool4000001`0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?oo
o`006`3oool00`000000oooo0?ooo`0c0?ooo`040000003oool0oooo000000T0oooo00<000000?oo
o`3oool0;`3oool010000000oooo0?ooo`0000070?ooo`D00000;`3oool010000000oooo0?ooo`00
00080?ooo`040000003oool0oooo000001<0oooo1000000H0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000000P3oool001/0oooo00<000000?ooo`3oool0<`3oool010000000
oooo0?ooo`00000:0?ooo`030000003oool0oooo02h0oooo00@000000?ooo`3oool000001`3oool0
10000000oooo0?ooo`00000`0?ooo`040000003oool0oooo000000P0oooo0`00000H0?ooo`@00000
503oool010000000oooo0?ooo`0000090?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0
<`3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo02d0oooo00@000000?ooo`3o
ool00000203oool00`000000oooo0000000`0?ooo`040000003oool0oooo000000P0oooo00<00000
0?ooo`3oool0703oool5000000l0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000020?ooo`006`3oool00`000000oooo0?ooo`0c0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool00000;`3oool010000000oooo0?ooo`0000090?ooo`800000<03oool01000
0000oooo0?ooo`0000080?ooo`030000003oool0oooo0240oooo1@00000:0?ooo`040000003oool0
oooo000000P0oooo00@000000?ooo`3oool000000P3oool001/0oooo0P00000e0?ooo`8000002P3o
ool200000340oooo0P00000;0?ooo`030000003oool0oooo02l0oooo0P00000:0?ooo`<000009@3o
ool6000000D0oooo0P00000:0?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0k03oool5
00000140oooo000K0?ooo`030000003oool0oooo0?40oooo1@00000<0?ooo`006`3oool00`000000
oooo0?ooo`3f0?ooo`<000002@3oool001D0ooooo`000009000000<0oooo000K0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool01`3oool001/0oooo00<000000?ooo`3oool0>`3oool0
0`000000oooo0?ooo`0l0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0?03oool0
0`000000oooo0?ooo`070?ooo`006`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000K0?ooo`03
0000003oool0oooo0?l0oooo0`3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool0
0?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0870867, -3.72417, \
0.00319096, 0.0222361}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%121, %122, %23]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.190529 0.153051 [
[.2619 .17803 -9 -9 ]
[.2619 .17803 9 0 ]
[.5 .17803 -9 -9 ]
[.5 .17803 9 0 ]
[.7381 .17803 -9 -9 ]
[.7381 .17803 9 0 ]
[.97619 .17803 -9 -9 ]
[.97619 .17803 9 0 ]
[.01131 .03748 -12 -4.5 ]
[.01131 .03748 0 4.5 ]
[.01131 .34358 -6 -4.5 ]
[.01131 .34358 0 4.5 ]
[.01131 .49663 -6 -4.5 ]
[.01131 .49663 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .19053 m
.2619 .19678 L
s
[(0.2)] .2619 .17803 0 1 Mshowa
.5 .19053 m
.5 .19678 L
s
[(0.4)] .5 .17803 0 1 Mshowa
.7381 .19053 m
.7381 .19678 L
s
[(0.6)] .7381 .17803 0 1 Mshowa
.97619 .19053 m
.97619 .19678 L
s
[(0.8)] .97619 .17803 0 1 Mshowa
.125 Mabswid
.08333 .19053 m
.08333 .19428 L
s
.14286 .19053 m
.14286 .19428 L
s
.20238 .19053 m
.20238 .19428 L
s
.32143 .19053 m
.32143 .19428 L
s
.38095 .19053 m
.38095 .19428 L
s
.44048 .19053 m
.44048 .19428 L
s
.55952 .19053 m
.55952 .19428 L
s
.61905 .19053 m
.61905 .19428 L
s
.67857 .19053 m
.67857 .19428 L
s
.79762 .19053 m
.79762 .19428 L
s
.85714 .19053 m
.85714 .19428 L
s
.91667 .19053 m
.91667 .19428 L
s
.25 Mabswid
0 .19053 m
1 .19053 L
s
.02381 .03748 m
.03006 .03748 L
s
[(-1)] .01131 .03748 1 0 Mshowa
.02381 .34358 m
.03006 .34358 L
s
[(1)] .01131 .34358 1 0 Mshowa
.02381 .49663 m
.03006 .49663 L
s
[(2)] .01131 .49663 1 0 Mshowa
.125 Mabswid
.02381 .06809 m
.02756 .06809 L
s
.02381 .0987 m
.02756 .0987 L
s
.02381 .12931 m
.02756 .12931 L
s
.02381 .15992 m
.02756 .15992 L
s
.02381 .22114 m
.02756 .22114 L
s
.02381 .25175 m
.02756 .25175 L
s
.02381 .28236 m
.02756 .28236 L
s
.02381 .31297 m
.02756 .31297 L
s
.02381 .37419 m
.02756 .37419 L
s
.02381 .4048 m
.02756 .4048 L
s
.02381 .43541 m
.02756 .43541 L
s
.02381 .46602 m
.02756 .46602 L
s
.02381 .00687 m
.02756 .00687 L
s
.02381 .52724 m
.02756 .52724 L
s
.02381 .55785 m
.02756 .55785 L
s
.02381 .58846 m
.02756 .58846 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .60332 m
.0403 .603 L
.04458 .60262 L
.05429 .60142 L
.06375 .59983 L
.07387 .59768 L
.09565 .59155 L
.11547 .58433 L
.15491 .56615 L
.19284 .54517 L
.2332 .52077 L
.27205 .49664 L
.30942 .47381 L
.3492 .4506 L
.38749 .42973 L
.4282 .40934 L
.46741 .39149 L
.50512 .37594 L
.54526 .36103 L
.5839 .34815 L
.62496 .33589 L
.66453 .32532 L
.7026 .31618 L
.74309 .30744 L
.78208 .29987 L
.81958 .29329 L
.8595 .28695 L
.89792 .28143 L
.93877 .2761 L
.97619 .27166 L
s
.03571 .01476 m
.0403 .01493 L
.04458 .01515 L
.05429 .01581 L
.06375 .0167 L
.07387 .0179 L
.09565 .02132 L
.11547 .02534 L
.15491 .03542 L
.19284 .04698 L
.2332 .06033 L
.27205 .07343 L
.30942 .08572 L
.3492 .09811 L
.38749 .10916 L
.4282 .11986 L
.46741 .12916 L
.50512 .1372 L
.54526 .14485 L
.5839 .15142 L
.62496 .15764 L
.66453 .16298 L
.7026 .16758 L
.74309 .17196 L
.78208 .17574 L
.81958 .17903 L
.8595 .1822 L
.89792 .18496 L
.93877 .18763 L
.97619 .18986 L
s
.03571 .01472 m
.0403 .01485 L
.04458 .01502 L
.05429 .01555 L
.06375 .01625 L
.07387 .0172 L
.09565 .01992 L
.11547 .02312 L
.15491 .03122 L
.19284 .04064 L
.2332 .05169 L
.27205 .06272 L
.30942 .07326 L
.3492 .08408 L
.38749 .0939 L
.4282 .10359 L
.46741 .11214 L
.50512 .11965 L
.54526 .12691 L
.5839 .13322 L
.62496 .13926 L
.66453 .14449 L
.7026 .14903 L
.74309 .15339 L
.78208 .15717 L
.81958 .16046 L
.8595 .16363 L
.89792 .1664 L
.93877 .16906 L
.97619 .17127 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo0P00003o0?oooa00oooo000?0?oo
o`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool02P00003o0?ooo`D0
oooo000?0?ooo`030000003oool0oooo00X0oooo2P00003j0?ooo`003`3oool00`000000oooo0?oo
o`0>0?ooo`/00000m@3oool000l0oooo00<000000?ooo`3oool0503oool4000000030?ooo`000000
000000@00000k`3oool000L0oooo100000040?ooo`030000003oool0oooo01P0oooo100000030?oo
o`@00000j`3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0L0?ooo`@0
00000`3oool400000>L0oooo00090?ooo`030000003oool0oooo00<0oooo0`00000P0?ooo`<00000
103oool400000><0oooo00000`3oool0000000000002000000@0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0S0?ooo`@00000103oool300000>00oooo00090?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool09`3oool4000000<0oooo1000003L0?ooo`001`3oool30000
00D0oooo00<000000?ooo`3oool0:`3oool3000000@0oooo1000003H0?ooo`002@3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo02h0oooo100000040?ooo`<00000e@3oool000l0oooo
00<000000?ooo`3oool0<P3oool4000000<0oooo0`00003B0?ooo`003`3oool00`000000oooo0?oo
o`0f0?ooo`<000000`3oool400000<h0oooo000?0?ooo`030000003oool0oooo03T0oooo0P000005
0?ooo`<00000b`3oool000l0oooo00<000000?ooo`3oool0>`3oool3000000D0oooo100000370?oo
o`003`3oool2000003l0oooo0P0000070?ooo`@00000``3oool000l0oooo00<000000?ooo`3oool0
@03oool3000000P0oooo0`0000300?ooo`003`3oool00`000000oooo0?ooo`130?ooo`@000001`3o
ool400000;`0oooo000?0?ooo`030000003oool0oooo04L0oooo100000070?ooo`@00000^03oool0
00l0oooo00<000000?ooo`3oool0B`3oool3000000P0oooo0`00002e0?ooo`003`3oool00`000000
oooo0?ooo`1>0?ooo`<00000203oool300000;80oooo000?0?ooo`030000003oool0oooo0540oooo
0P0000090?ooo`@00000[P3oool000l0oooo00<000000?ooo`3oool0D`3oool3000000X0oooo1000
002Z0?ooo`003`3oool2000005L0oooo0`00000;0?ooo`H00000Y03oool000l0oooo00<000000?oo
o`3oool0F@3oool3000000h0oooo1000002P0?ooo`003`3oool00`000000oooo0?ooo`1L0?ooo`@0
00003P3oool4000009`0oooo000?0?ooo`030000003oool0oooo0600oooo0`00000?0?ooo`@00000
V03oool000l0oooo00<000000?ooo`3oool0H`3oool4000000l0oooo1000002D0?ooo`003`3oool0
0`000000oooo0?ooo`1W0?ooo`@000003`3oool6000008h0oooo000?0?ooo`030000003oool0oooo
06/0oooo1000000A0?ooo`D00000R@3oool000l0oooo00<000000?ooo`3oool0K`3oool600000100
oooo1P0000230?ooo`003`3oool2000007H0oooo1000000B0?ooo`D00000OP3oool000l0oooo00<0
00000?ooo`3oool0N@3oool4000001<0oooo1@00001i0?ooo`003`3oool00`000000oooo0?ooo`1m
0?ooo`@00000503oool800000740oooo000?0?ooo`030000003oool0oooo0840oooo1000000H0?oo
o`T00000J03oool000l0oooo00<000000?ooo`3oool0Q@3oool6000001/0oooo1P00001R0?ooo`00
3`3oool00`000000oooo0?ooo`2;0?ooo`D00000703oool7000005/0oooo000?0?ooo`030000003o
ool0oooo0900oooo1@00000N0?ooo`/00000D03oool000l0oooo00<000000?ooo`3oool0>03oool2
000000@0oooo00<000000?ooo`3oool00P3oool400000380oooo0P0000040?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0303oool8000001h0oooo0P0000000`3oool0000000000009
000003@0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`003`3oool00`00
0000oooo0?ooo`0g0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0<P3oool0
10000000oooo0?ooo`00000:0?ooo`030000003oool0oooo01@0oooo2@00000D0?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`3oool02P00000Y0?ooo`040000003oool0oooo000000P0
oooo00@000000?ooo`3oool000000P3oool000l0oooo0P00000h0?ooo`040000003oool0oooo0000
00T0oooo00<000000?ooo`3oool0<@3oool010000000oooo0?ooo`0000070?ooo`D000007P3oool6
000000h0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000090?ooo`/00000
7P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo000?0?ooo`03
0000003oool0oooo03L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`0`0?oo
o`040000003oool0oooo000000L0oooo00@000000?ooo`3oool000009@3oool7000000L0oooo00@0
00000?ooo`3oool00000203oool3000001D0oooo2`00000C0?ooo`040000003oool0oooo000000T0
oooo0P0000030?ooo`003`3oool00`000000oooo0?ooo`0g0?ooo`040000003oool0oooo000000/0
oooo00<000000?ooo`3oool0;`3oool010000000oooo0?ooo`0000080?ooo`030000003oool00000
02`0oooo2`0000080?ooo`030000003oool0oooo0200oooo5`0000080?ooo`040000003oool0oooo
00000080oooo000?0?ooo`030000003oool0oooo03L0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`00000a0?ooo`040000003oool0oooo000000T0oooo0P00000c0?ooo`030000
003oool0oooo00`00000<`3oool00`000000oooo0?ooo`06000000<0oooo00@000000?ooo`3oool0
00000P3oool000l0oooo00<000000?ooo`3oool0>03oool2000000X0oooo0P00000c0?ooo`800000
2`3oool00`000000oooo0?ooo`0b0?ooo`8000002P3oool<000002X0oooo0P00000:0?ooo`800000
0`3oool000l0oooo00<000000?ooo`3oool0d`3oool;00000300oooo000?0?ooo`030000003oool0
oooo0=h0oooo2`00000U0?ooo`003`3oool00`000000oooo0?ooo`3Y0?ooo`/000006P3oool000P0
ooooo`00000F000000<0oooo000?0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
1`3oool000l0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0n0?ooo`030000003o
ool0oooo03l0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`070?ooo`003`3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0
oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`00
3`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3o
ool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?oo
o`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0
o`3oool?0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?oo
o`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo1P0000090?ooo`003`3oool00`00
0000oooo0?ooo`3d0?ooo`/000003`3oool000l0oooo00<000000?ooo`3oool0k03oool8000001X0
oooo000?0?ooo`800000i`3oool600000280oooo000?0?ooo`030000003oool0oooo0>40oooo1@00
000X0?ooo`003`3oool00`000000oooo0?ooo`3K0?ooo`H00000;@3oool000l0oooo00<000000?oo
o`3oool0eP3oool5000003<0oooo000?0?ooo`030000003oool0oooo0=40oooo1@00000h0?ooo`00
3`3oool00`000000oooo0?ooo`3<0?ooo`D00000?@3oool000l0oooo00<000000?ooo`3oool0aP3o
ool600000480oooo000?0?ooo`030000003oool0oooo0<40oooo1@0000180?ooo`003`3oool20000
0;`0oooo1P00001=0?ooo`003`3oool00`000000oooo0?ooo`2g0?ooo`@00000D`3oool000l0oooo
00<000000?ooo`3oool0/`3oool4000005L0oooo000?0?ooo`030000003oool0oooo0;00oooo0`00
001K0?ooo`003`3oool00`000000oooo0?ooo`2/0?ooo`@00000GP3oool000l0oooo00<000000?oo
o`3oool0YP3oool600000680oooo000?0?ooo`030000003oool0oooo0:80oooo1000001X0?ooo`00
1`3oool4000000@0oooo00<000000?ooo`3oool0W`3oool3000006`0oooo00090?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0W03oool3000006l0oooo00090?ooo`030000003oool0
oooo00<0oooo0`00002I0?ooo`<00000LP3oool000T0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`2F0?ooo`<00000M@3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`2B0?ooo`@00000N03oool000L0oooo0`0000050?ooo`030000003oool0oooo08l0oooo
0`00001l0?ooo`002@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08`0oooo0`00
001o0?ooo`003`3oool00`000000oooo0?ooo`290?ooo`<00000PP3oool000l0oooo00<000000?oo
o`3oool0Q`3oool2000008D0oooo000?0?ooo`030000003oool0oooo08@0oooo0`0000270?ooo`00
3`3oool200000880oooo0`00002:0?ooo`003`3oool00`000000oooo0?ooo`1o0?ooo`800000S@3o
ool000l0oooo00<000000?ooo`3oool0O@3oool2000008l0oooo000?0?ooo`030000003oool0oooo
07/0oooo0P00002A0?ooo`003`3oool00`000000oooo0?ooo`1i0?ooo`800000T`3oool000l0oooo
00<000000?ooo`3oool0MP3oool3000009D0oooo000?0?ooo`030000003oool0oooo07@0oooo0P00
002H0?ooo`003`3oool00`000000oooo0?ooo`1b0?ooo`800000VP3oool000l0oooo0P00001a0?oo
o`800000W03oool000l0oooo00<000000?ooo`3oool0KP3oool2000009h0oooo000?0?ooo`030000
003oool0oooo06/0oooo0`00002P0?ooo`003`3oool00`000000oooo0?ooo`1Y0?ooo`800000X`3o
ool000l0oooo00<000000?ooo`3oool0I`3oool200000:D0oooo000?0?ooo`030000003oool0oooo
06D0oooo0P00002W0?ooo`003`3oool00`000000oooo0?ooo`1S0?ooo`800000Z@3oool000l0oooo
00<000000?ooo`3oool0H@3oool200000:/0oooo000?0?ooo`030000003oool0oooo05l0oooo0P00
002]0?ooo`003`3oool2000005h0oooo0P00002_0?ooo`003`3oool00`000000oooo0?ooo`1L0?oo
o`030000003oool0oooo0:l0oooo000?0?ooo`030000003oool0oooo05X0oooo0P00002b0?ooo`00
3`3oool00`000000oooo0?ooo`1H0?ooo`800000]03oool000l0oooo00<000000?ooo`3oool0E`3o
ool00`000000oooo0?ooo`2d0?ooo`003`3oool00`000000oooo0?ooo`1E0?ooo`800000]`3oool0
00l0oooo00<000000?ooo`3oool0D`3oool200000;T0oooo000?0?ooo`030000003oool0oooo0540
oooo0P00002k0?ooo`003`3oool200000500oooo0P00002m0?ooo`003`3oool00`000000oooo0?oo
o`1=0?ooo`800000_`3oool000l0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`2o
0?ooo`003`3oool00`000000oooo0?ooo`1:0?ooo`800000`P3oool000l0oooo00<000000?ooo`3o
ool0B@3oool00`000000oooo0?ooo`320?ooo`003`3oool00`000000oooo0?ooo`170?ooo`800000
a@3oool000l0oooo00<000000?ooo`3oool0A@3oool200000<L0oooo00070?ooo`@00000103oool0
0`000000oooo0?ooo`140?ooo`030000003oool0oooo0<L0oooo00070?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0@P3oool200000<X0oooo00080?ooo`030000003oool0oooo00@0
oooo0`0000110?ooo`030000003oool0oooo0<X0oooo00090?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0?`3oool200000<d0oooo000:0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0?@3oool200000<l0oooo00070?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0?03oool00`000000oooo0?ooo`3?0?ooo`00203oool2000000D0oooo00<00000
0?ooo`3oool0>P3oool200000=80oooo000?0?ooo`030000003oool0oooo03P0oooo0P00003D0?oo
o`003`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0=@0oooo000?0?ooo`030000
003oool0oooo03D0oooo0P00003G0?ooo`003`3oool2000003D0oooo00<000000?ooo`3oool0e`3o
ool000l0oooo00<000000?ooo`3oool0<P3oool200000=X0oooo000?0?ooo`030000003oool0oooo
0300oooo0P00003L0?ooo`003`3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0=`0
oooo000?0?ooo`030000003oool0oooo02d0oooo0P00003O0?ooo`003`3oool00`000000oooo0?oo
o`0/0?ooo`030000003oool0oooo0=l0oooo000?0?ooo`030000003oool0oooo02X0oooo0P00003R
0?ooo`003`3oool00`000000oooo0?ooo`0X0?ooo`800000i03oool000l0oooo00<000000?ooo`3o
ool09P3oool200000>H0oooo000?0?ooo`8000009@3oool200000>P0oooo000?0?ooo`030000003o
ool0oooo0280oooo0P00003Z0?ooo`003`3oool00`000000oooo0?ooo`0P0?ooo`800000k03oool0
00l0oooo00<000000?ooo`3oool07P3oool200000>h0oooo000?0?ooo`030000003oool0oooo01`0
oooo0P00003`0?ooo`003`3oool00`000000oooo0?ooo`0J0?ooo`800000lP3oool000l0oooo00<0
00000?ooo`3oool0603oool200000?@0oooo000?0?ooo`030000003oool0oooo01D0oooo0`00003f
0?ooo`003`3oool2000001@0oooo0P00003i0?ooo`003`3oool00`000000oooo0?ooo`0@0?ooo`<0
0000n`3oool000l0oooo00<000000?ooo`3oool03@3oool300000?h0oooo000?0?ooo`030000003o
ool0oooo00L0oooo1P00003o0?ooo`80oooo000?0?ooo`030000003oool0oooo00L00000o`3oool8
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?oo
o`3o0?ooo`l0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0470677, -1.3226, \
0.00304701, 0.0237006}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(for\ md = Sqrt[3]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%124, %125, %126]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.190529 0.051017 [
[.2619 .17803 -9 -9 ]
[.2619 .17803 9 0 ]
[.5 .17803 -9 -9 ]
[.5 .17803 9 0 ]
[.7381 .17803 -9 -9 ]
[.7381 .17803 9 0 ]
[.97619 .17803 -9 -9 ]
[.97619 .17803 9 0 ]
[.01131 .0885 -12 -4.5 ]
[.01131 .0885 0 4.5 ]
[.01131 .29256 -6 -4.5 ]
[.01131 .29256 0 4.5 ]
[.01131 .3946 -6 -4.5 ]
[.01131 .3946 0 4.5 ]
[.01131 .49663 -6 -4.5 ]
[.01131 .49663 0 4.5 ]
[.01131 .59867 -6 -4.5 ]
[.01131 .59867 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .19053 m
.2619 .19678 L
s
[(0.2)] .2619 .17803 0 1 Mshowa
.5 .19053 m
.5 .19678 L
s
[(0.4)] .5 .17803 0 1 Mshowa
.7381 .19053 m
.7381 .19678 L
s
[(0.6)] .7381 .17803 0 1 Mshowa
.97619 .19053 m
.97619 .19678 L
s
[(0.8)] .97619 .17803 0 1 Mshowa
.125 Mabswid
.08333 .19053 m
.08333 .19428 L
s
.14286 .19053 m
.14286 .19428 L
s
.20238 .19053 m
.20238 .19428 L
s
.32143 .19053 m
.32143 .19428 L
s
.38095 .19053 m
.38095 .19428 L
s
.44048 .19053 m
.44048 .19428 L
s
.55952 .19053 m
.55952 .19428 L
s
.61905 .19053 m
.61905 .19428 L
s
.67857 .19053 m
.67857 .19428 L
s
.79762 .19053 m
.79762 .19428 L
s
.85714 .19053 m
.85714 .19428 L
s
.91667 .19053 m
.91667 .19428 L
s
.25 Mabswid
0 .19053 m
1 .19053 L
s
.02381 .0885 m
.03006 .0885 L
s
[(-2)] .01131 .0885 1 0 Mshowa
.02381 .29256 m
.03006 .29256 L
s
[(2)] .01131 .29256 1 0 Mshowa
.02381 .3946 m
.03006 .3946 L
s
[(4)] .01131 .3946 1 0 Mshowa
.02381 .49663 m
.03006 .49663 L
s
[(6)] .01131 .49663 1 0 Mshowa
.02381 .59867 m
.03006 .59867 L
s
[(8)] .01131 .59867 1 0 Mshowa
.125 Mabswid
.02381 .114 m
.02756 .114 L
s
.02381 .13951 m
.02756 .13951 L
s
.02381 .16502 m
.02756 .16502 L
s
.02381 .21604 m
.02756 .21604 L
s
.02381 .24155 m
.02756 .24155 L
s
.02381 .26705 m
.02756 .26705 L
s
.02381 .31807 m
.02756 .31807 L
s
.02381 .34358 m
.02756 .34358 L
s
.02381 .36909 m
.02756 .36909 L
s
.02381 .42011 m
.02756 .42011 L
s
.02381 .44561 m
.02756 .44561 L
s
.02381 .47112 m
.02756 .47112 L
s
.02381 .52214 m
.02756 .52214 L
s
.02381 .54765 m
.02756 .54765 L
s
.02381 .57316 m
.02756 .57316 L
s
.02381 .06299 m
.02756 .06299 L
s
.02381 .03748 m
.02756 .03748 L
s
.02381 .01197 m
.02756 .01197 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03571 .01472 m
.0403 .01485 L
.04458 .01502 L
.05429 .01555 L
.06375 .01625 L
.07387 .0172 L
.09565 .01992 L
.11547 .02312 L
.15491 .03122 L
.19284 .04064 L
.2332 .05169 L
.27205 .06272 L
.30942 .07326 L
.3492 .08408 L
.38749 .0939 L
.4282 .10359 L
.46741 .11214 L
.50512 .11965 L
.54526 .12691 L
.5839 .13322 L
.62496 .13926 L
.66453 .14449 L
.7026 .14903 L
.74309 .15339 L
.78208 .15717 L
.81958 .16046 L
.8595 .16363 L
.89792 .1664 L
.93877 .16906 L
.97619 .17127 L
s
.03571 .60332 m
.0403 .603 L
.04458 .60262 L
.05429 .60142 L
.06375 .59983 L
.07387 .59768 L
.09565 .59155 L
.11547 .58433 L
.15491 .56615 L
.19284 .54517 L
.2332 .52077 L
.27205 .49664 L
.30942 .47381 L
.3492 .4506 L
.38749 .42973 L
.4282 .40934 L
.46741 .39149 L
.50512 .37594 L
.54526 .36103 L
.5839 .34815 L
.62496 .33589 L
.66453 .32532 L
.7026 .31618 L
.74309 .30744 L
.78208 .29987 L
.81958 .29329 L
.8595 .28695 L
.89792 .28143 L
.93877 .2761 L
.97619 .27166 L
s
.03571 .01476 m
.0403 .01493 L
.04458 .01515 L
.05429 .01581 L
.06375 .0167 L
.07387 .0179 L
.09565 .02132 L
.11547 .02534 L
.15491 .03542 L
.19284 .04698 L
.2332 .06033 L
.27205 .07343 L
.30942 .08572 L
.3492 .09811 L
.38749 .10916 L
.4282 .11986 L
.46741 .12916 L
.50512 .1372 L
.54526 .14485 L
.5839 .15142 L
.62496 .15764 L
.66453 .16298 L
.7026 .16758 L
.74309 .17196 L
.78208 .17574 L
.81958 .17903 L
.8595 .1822 L
.89792 .18496 L
.93877 .18763 L
.97619 .18986 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo
000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool02P00003o0?ooo`D0
oooo000?0?ooo`030000003oool0oooo00X0oooo2P00003j0?ooo`003`3oool00`000000oooo0?oo
o`0>0?ooo`/00000m@3oool000l0oooo00<000000?ooo`3oool0503oool4000000030?ooo`000000
000000@00000k`3oool000l0oooo00<000000?ooo`3oool0603oool4000000<0oooo1000003[0?oo
o`003`3oool00`000000oooo0?ooo`0L0?ooo`@000000`3oool400000>L0oooo000?0?ooo`800000
8@3oool3000000@0oooo1000003S0?ooo`003`3oool00`000000oooo0?ooo`0S0?ooo`@00000103o
ool300000>00oooo000?0?ooo`030000003oool0oooo02L0oooo100000030?ooo`@00000g03oool0
00l0oooo00<000000?ooo`3oool0:`3oool3000000@0oooo1000003H0?ooo`003`3oool00`000000
oooo0?ooo`0^0?ooo`@00000103oool300000=D0oooo000?0?ooo`030000003oool0oooo0380oooo
100000030?ooo`<00000dP3oool000l0oooo00<000000?ooo`3oool0=P3oool3000000<0oooo1000
003>0?ooo`003`3oool2000003X0oooo0P0000050?ooo`<00000b`3oool000l0oooo00<000000?oo
o`3oool0>`3oool3000000D0oooo100000370?ooo`003`3oool00`000000oooo0?ooo`0n0?ooo`80
00001`3oool400000<<0oooo000?0?ooo`030000003oool0oooo0400oooo0`0000080?ooo`<00000
`03oool000l0oooo00<000000?ooo`3oool0@`3oool4000000L0oooo1000002l0?ooo`001`3oool4
000000@0oooo00<000000?ooo`3oool0A`3oool4000000L0oooo1000002h0?ooo`001`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo04/0oooo0`0000080?ooo`<00000]@3oool000P0
oooo00<000000?ooo`3oool0103oool3000004h0oooo0`0000080?ooo`<00000/P3oool000030?oo
o`000000000000800000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0540oooo
0P0000090?ooo`@00000[P3oool000X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`1C0?ooo`<000002P3oool400000:X0oooo00070?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0EP3oool3000000/0oooo1P00002T0?ooo`00203oool2000000D0oooo00<00000
0?ooo`3oool0F@3oool3000000h0oooo1000002P0?ooo`003`3oool00`000000oooo0?ooo`1L0?oo
o`@000003P3oool4000009`0oooo000?0?ooo`030000003oool0oooo0600oooo0`00000?0?ooo`@0
0000V03oool000l0oooo0P00001T0?ooo`@000003`3oool4000009@0oooo000?0?ooo`030000003o
ool0oooo06L0oooo1000000?0?ooo`H00000SP3oool000l0oooo00<000000?ooo`3oool0J`3oool4
00000140oooo1@0000290?ooo`003`3oool00`000000oooo0?ooo`1_0?ooo`H00000403oool60000
08<0oooo000?0?ooo`030000003oool0oooo07D0oooo1000000B0?ooo`D00000OP3oool000l0oooo
00<000000?ooo`3oool0N@3oool4000001<0oooo1@00001i0?ooo`003`3oool00`000000oooo0?oo
o`1m0?ooo`@00000503oool800000740oooo000?0?ooo`800000PP3oool4000001P0oooo2@00001X
0?ooo`003`3oool00`000000oooo0?ooo`250?ooo`H000006`3oool600000680oooo000?0?ooo`03
0000003oool0oooo08/0oooo1@00000L0?ooo`L00000F`3oool000l0oooo00<000000?ooo`3oool0
T03oool5000001h0oooo2`00001@0?ooo`003`3oool00`000000oooo0?ooo`0h0?ooo`800000103o
ool00`000000oooo0?ooo`020?ooo`@00000<P3oool2000000@0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0<0?ooo`P000007P3oool2000000030?ooo`000000000000T00000=03o
ool2000000@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo000?0?ooo`030000003oool0
oooo03L0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0b0?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0503oool9000001@0oooo00@000000?ooo`3oool0
0000203oool00`000000oooo0?ooo`0:000002T0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000020?ooo`003`3oool00`000000oooo0?ooo`0g0?ooo`040000003oool0oooo
000000T0oooo00<000000?ooo`3oool0<@3oool010000000oooo0?ooo`0000070?ooo`D000007P3o
ool6000000h0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000090?ooo`/0
00007P3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo000?0?oo
o`800000>03oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo0300oooo00@00000
0?ooo`3oool000001`3oool010000000oooo0?ooo`00000U0?ooo`L000001`3oool010000000oooo
0?ooo`0000080?ooo`<000005@3oool;000001<0oooo00@000000?ooo`3oool000002@3oool20000
00<0oooo000?0?ooo`030000003oool0oooo03L0oooo00@000000?ooo`3oool000002`3oool00`00
0000oooo0?ooo`0_0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`000000;03oool;
000000P0oooo00<000000?ooo`3oool0803ooolG000000P0oooo00@000000?ooo`3oool000000P3o
ool000l0oooo00<000000?ooo`3oool0=`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo00000340oooo00@000000?ooo`3oool000002@3oool2000003<0oooo00<000000?ooo`3o
ool03000000c0?ooo`030000003oool0oooo00H000000`3oool010000000oooo0?ooo`0000020?oo
o`003`3oool00`000000oooo0?ooo`0h0?ooo`8000002P3oool2000003<0oooo0P00000;0?ooo`03
0000003oool0oooo0380oooo0P00000:0?ooo``00000:P3oool2000000X0oooo0P0000030?ooo`00
3`3oool00`000000oooo0?ooo`3C0?ooo`/00000<03oool000l0oooo00<000000?ooo`3oool0gP3o
ool;000002D0oooo000?0?ooo`030000003oool0oooo0>T0oooo2`00000J0?ooo`00203ooooo0000
01H000000`3oool000l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`070?ooo`00
3`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0
?`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00L0oooo000?0?ooo`030000003o
ool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`00
0000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo
0P00003o0?oooa00oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<00000
0?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`03
0000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3o
ool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`800000o`3oool@0?ooo`003`3oool00`00
0000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo
00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?
0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?oo
o`003`3oool200000?l0oooo403oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3o
ool00`000000oooo0?ooo`3o0?ooo`H000002@3oool000l0oooo00<000000?ooo`3oool0m03oool;
000000l0oooo000?0?ooo`030000003oool0oooo0>`0oooo2000000J0?ooo`001`3oool4000000@0
oooo00<000000?ooo`3oool0iP3oool600000280oooo00070?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0h@3oool5000002P0oooo00080?ooo`030000003oool0oooo00@0oooo0`00
003K0?ooo`H00000;@3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3F
0?ooo`D00000<`3oool000X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3A0?oo
o`D00000>03oool000L0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3<0?oo
o`D00000?@3oool000P0oooo0P0000050?ooo`030000003oool0oooo0<H0oooo1P0000120?ooo`00
3`3oool00`000000oooo0?ooo`310?ooo`D00000B03oool000l0oooo00<000000?ooo`3oool0^`3o
ool6000004d0oooo000?0?ooo`030000003oool0oooo0;L0oooo1000001C0?ooo`003`3oool20000
0;@0oooo1000001G0?ooo`003`3oool00`000000oooo0?ooo`2`0?ooo`<00000F`3oool000l0oooo
00<000000?ooo`3oool0[03oool4000005h0oooo000?0?ooo`030000003oool0oooo0:H0oooo1P00
001R0?ooo`003`3oool00`000000oooo0?ooo`2R0?ooo`@00000J03oool000l0oooo00<000000?oo
o`3oool0W`3oool3000006`0oooo000?0?ooo`030000003oool0oooo09`0oooo0`00001_0?ooo`00
3`3oool2000009X0oooo0`00001b0?ooo`003`3oool00`000000oooo0?ooo`2F0?ooo`<00000M@3o
ool000l0oooo00<000000?ooo`3oool0TP3oool4000007P0oooo000?0?ooo`030000003oool0oooo
08l0oooo0`00001l0?ooo`003`3oool00`000000oooo0?ooo`2<0?ooo`<00000O`3oool000l0oooo
00<000000?ooo`3oool0R@3oool300000880oooo000?0?ooo`030000003oool0oooo08L0oooo0P00
00250?ooo`003`3oool2000008D0oooo0`0000270?ooo`003`3oool00`000000oooo0?ooo`210?oo
o`<00000RP3oool000l0oooo00<000000?ooo`3oool0O`3oool2000008d0oooo000?0?ooo`030000
003oool0oooo07d0oooo0P00002?0?ooo`003`3oool00`000000oooo0?ooo`1k0?ooo`800000T@3o
ool000T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1i0?ooo`800000T`3oool0
00T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1f0?ooo`<00000U@3oool000H0
oooo1@0000040?ooo`<00000M03oool2000009P0oooo00060?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool0LP3oool2000009X0oooo00070?ooo`030000003oool0000000D0oooo
00<000000?ooo`3oool0L03oool2000009`0oooo00080?ooo`8000001@3oool00`000000oooo0?oo
o`1^0?ooo`800000WP3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1[
0?ooo`<00000X03oool000l0oooo00<000000?ooo`3oool0J@3oool200000:<0oooo000?0?ooo`03
0000003oool0oooo06L0oooo0P00002U0?ooo`003`3oool2000006H0oooo0P00002W0?ooo`003`3o
ool00`000000oooo0?ooo`1S0?ooo`800000Z@3oool000l0oooo00<000000?ooo`3oool0H@3oool2
00000:/0oooo000?0?ooo`030000003oool0oooo05l0oooo0P00002]0?ooo`003`3oool00`000000
oooo0?ooo`1M0?ooo`800000[`3oool000l0oooo00<000000?ooo`3oool0G03oool00`000000oooo
0?ooo`2_0?ooo`003`3oool00`000000oooo0?ooo`1J0?ooo`800000/P3oool000l0oooo0P00001I
0?ooo`800000]03oool000l0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`2d0?oo
o`003`3oool00`000000oooo0?ooo`1E0?ooo`800000]`3oool000l0oooo00<000000?ooo`3oool0
D`3oool200000;T0oooo000?0?ooo`030000003oool0oooo0540oooo0P00002k0?ooo`003`3oool0
0`000000oooo0?ooo`1?0?ooo`800000_@3oool000l0oooo00<000000?ooo`3oool0C@3oool20000
0;l0oooo000?0?ooo`800000C@3oool00`000000oooo0?ooo`2o0?ooo`003`3oool00`000000oooo
0?ooo`1:0?ooo`800000`P3oool000l0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?oo
o`320?ooo`003`3oool00`000000oooo0?ooo`170?ooo`800000a@3oool000l0oooo00<000000?oo
o`3oool0A@3oool200000<L0oooo00080?ooo`8000001@3oool00`000000oooo0?ooo`140?ooo`03
0000003oool0oooo0<L0oooo00070?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool0@P3oool200000<X0oooo00070?ooo`040000003oool0oooo000000@0oooo0`0000110?ooo`03
0000003oool0oooo0<X0oooo00070?ooo`<000001@3oool00`000000oooo0?ooo`0o0?ooo`800000
c@3oool000L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0m0?ooo`800000c`3o
ool000L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0
oooo0<l0oooo00080?ooo`<00000103oool00`000000oooo0?ooo`0j0?ooo`800000dP3oool000l0
oooo00<000000?ooo`3oool0>03oool200000=@0oooo000?0?ooo`030000003oool0oooo03L0oooo
00<000000?ooo`3oool0e03oool000l0oooo0P00000f0?ooo`800000e`3oool000l0oooo00<00000
0?ooo`3oool0=03oool00`000000oooo0?ooo`3G0?ooo`003`3oool00`000000oooo0?ooo`0b0?oo
o`800000fP3oool000l0oooo00<000000?ooo`3oool0<03oool200000=`0oooo000?0?ooo`030000
003oool0oooo02l0oooo00<000000?ooo`3oool0g03oool000l0oooo00<000000?ooo`3oool0;@3o
ool200000=l0oooo000?0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0g`3oool0
00l0oooo0P00000[0?ooo`800000hP3oool000l0oooo00<000000?ooo`3oool0:03oool200000>@0
oooo000?0?ooo`030000003oool0oooo02H0oooo0P00003V0?ooo`003`3oool00`000000oooo0?oo
o`0T0?ooo`800000j03oool000l0oooo00<000000?ooo`3oool08P3oool200000>X0oooo000?0?oo
o`030000003oool0oooo0200oooo0P00003/0?ooo`003`3oool00`000000oooo0?ooo`0N0?ooo`80
0000kP3oool000l0oooo0P00000M0?ooo`800000l03oool000l0oooo00<000000?ooo`3oool06P3o
ool200000?80oooo000?0?ooo`030000003oool0oooo01P0oooo0P00003d0?ooo`003`3oool00`00
0000oooo0?ooo`0E0?ooo`<00000mP3oool000l0oooo00<000000?ooo`3oool04`3oool200000?T0
oooo00080?ooo`8000001@3oool00`000000oooo0?ooo`0@0?ooo`<00000n`3oool000L0oooo00@0
00000?ooo`3oool00000103oool00`000000oooo0?ooo`0=0?ooo`<00000oP3oool000L0oooo00@0
00000?ooo`3oool00000103oool3000000L0oooo1P00003o0?ooo`80oooo00080?ooo`8000001@3o
ool00`000000oooo0?ooo`0700000?l0oooo203oool000L0oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`3o0?ooo`l0oooo00070?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool0o`3oool?0?ooo`00203oool2000000D0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo003o0?ooob40oooo003o0?ooob40oooo
003o0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0470677, -3.96781, \
0.00304701, 0.0711019}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{th, 
        mp[3, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 
          10]}, {th, 0.01, Pi/2, 0.01}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, \(\(8.091221002427346`\)\(\[InvisibleSpace]\)\) + 
          6.475273068794123`*^-51\ \[ImaginaryI]}, {0.02`, \
\(\(8.071096003432718`\)\(\[InvisibleSpace]\)\) + 
          3.231450740184634`*^-50\ \[ImaginaryI]}, {0.03`, \
\(\(8.037799715239919`\)\(\[InvisibleSpace]\)\) + 
          1.632401498659133`*^-54\ \[ImaginaryI]}, {0.04`, \
\(\(7.991692954492406`\)\(\[InvisibleSpace]\)\) + 
          9.430457148698461`*^-54\ \[ImaginaryI]}, {0.05`, \
\(\(7.933266762515794`\)\(\[InvisibleSpace]\)\) + 
          6.297026406794687`*^-54\ \[ImaginaryI]}, {0.06`, \
\(\(7.863129191001624`\)\(\[InvisibleSpace]\)\) + 
          1.3929893753343292`*^-53\ \[ImaginaryI]}, {0.07`, \
\(\(7.781989496396674`\)\(\[InvisibleSpace]\)\) + 
          8.080819738748364`*^-56\ \[ImaginaryI]}, {0.08`, \
\(\(7.690640495505058`\)\(\[InvisibleSpace]\)\) + 
          6.791770876673504`*^-56\ \[ImaginaryI]}, {0.09`, \
\(\(7.589939871653866`\)\(\[InvisibleSpace]\)\) + 
          1.4136757324154027`*^-55\ \[ImaginaryI]}, {0.1`, \
\(\(7.480791203891997`\)\(\[InvisibleSpace]\)\) + 
          9.87695162204226`*^-56\ \[ImaginaryI]}, {0.11`, \
\(\(7.364125428198873`\)\(\[InvisibleSpace]\)\) + 
          1.37737608971869`*^-55\ \[ImaginaryI]}, {0.12`, \
\(\(7.2408833402032124`\)\(\[InvisibleSpace]\)\) + 
          1.5730126467282594`*^-55\ \[ImaginaryI]}, {0.13`, \
\(\(7.111999625859454`\)\(\[InvisibleSpace]\)\) + 
          4.1470045666174774`*^-55\ \[ImaginaryI]}, {0.14`, \
\(\(6.978388772544864`\)\(\[InvisibleSpace]\)\) + 
          1.763638523735274`*^-55\ \[ImaginaryI]}, {0.15`, \
\(\(6.840933079521645`\)\(\[InvisibleSpace]\)\) + 
          1.7435044893859112`*^-55\ \[ImaginaryI]}, {0.16`, \
\(\(6.700472862942005`\)\(\[InvisibleSpace]\)\) + 
          2.613303685309506`*^-55\ \[ImaginaryI]}, {0.17`, \
\(\(6.557798843183876`\)\(\[InvisibleSpace]\)\) + 
          4.644304758674091`*^-55\ \[ImaginaryI]}, {0.18`, \
\(\(6.413646615206744`\)\(\[InvisibleSpace]\)\) + 
          4.608154169317233`*^-55\ \[ImaginaryI]}, {0.19`, \
\(\(6.268693037264747`\)\(\[InvisibleSpace]\)\) + 
          
          2.601547319266274`*^-55\ \[ImaginaryI]}, {0.2`, \
\(\(6.123554329150081`\)\(\[InvisibleSpace]\)\) + 
          4.4699602914925456`*^-55\ \[ImaginaryI]}, {0.21`, \
\(\(5.978785646161084`\)\(\[InvisibleSpace]\)\) + 
          4.435232540279423`*^-55\ \[ImaginaryI]}, {0.22`, \
\(\(5.834881886283829`\)\(\[InvisibleSpace]\)\) + 
          3.0953951349653444`*^-55\ \[ImaginaryI]}, {0.23`, \
\(\(5.692279492324774`\)\(\[InvisibleSpace]\)\) + 
          3.4844493706866854`*^-55\ \[ImaginaryI]}, {0.24`, \
\(\(5.551359024585173`\)\(\[InvisibleSpace]\)\) + 
          2.4643543186541654`*^-55\ \[ImaginaryI]}, {0.25`, \
\(\(5.412448300024698`\)\(\[InvisibleSpace]\)\) + 
          2.8430783976268476`*^-55\ \[ImaginaryI]}, {0.26`, \
\(\(5.275825918035514`\)\(\[InvisibleSpace]\)\) + 
          3.129991404932024`*^-55\ \[ImaginaryI]}, {0.27`, \
\(\(5.141725018750279`\)\(\[InvisibleSpace]\)\) + 
          5.0526835393687636`*^-55\ \[ImaginaryI]}, {0.28`, \
\(\(5.01033714555548`\)\(\[InvisibleSpace]\)\) + 
          3.295432303370583`*^-55\ \[ImaginaryI]}, {0.29`, \
\(\(4.881816107966436`\)\(\[InvisibleSpace]\)\) + 
          3.362767159700888`*^-55\ \[ImaginaryI]}, {0.3`, \
\(\(4.756281763447327`\)\(\[InvisibleSpace]\)\) + 
          3.504232733552221`*^-55\ \[ImaginaryI]}, {0.31`, \
\(\(4.633823656671998`\)\(\[InvisibleSpace]\)\) + 
          2.430242811045436`*^-55\ \[ImaginaryI]}, {0.32`, \
\(\(4.514504471925127`\)\(\[InvisibleSpace]\)\) + 
          2.753384748840639`*^-55\ \[ImaginaryI]}, {0.33`, \
\(\(4.3983632688423615`\)\(\[InvisibleSpace]\)\) + 
          6.950295407947603`*^-55\ \[ImaginaryI]}, {0.34`, \
\(\(4.2854184836152545`\)\(\[InvisibleSpace]\)\) + 
          2.312704995531232`*^-55\ \[ImaginaryI]}, {0.35000000000000003`, \
\(\(4.17567068736426`\)\(\[InvisibleSpace]\)\) + 
          1.9765939999983102`*^-55\ \[ImaginaryI]}, {0.36`, \
\(\(4.06910510087409`\)\(\[InvisibleSpace]\)\) + 
          3.9917599590679515`*^-55\ \[ImaginaryI]}, {0.37`, \
\(\(3.9656938705748734`\)\(\[InvisibleSpace]\)\) + 
          3.720883521850131`*^-55\ \[ImaginaryI]}, {0.38`, \
\(\(3.8653981148171948`\)\(\[InvisibleSpace]\)\) + 
          2.0431321246636244`*^-55\ \[ImaginaryI]}, {0.39`, \
\(\(3.7681697523936393`\)\(\[InvisibleSpace]\)\) + 
          3.2025650001325104`*^-55\ \[ImaginaryI]}, {0.4`, \
\(\(3.6739531271412744`\)\(\[InvisibleSpace]\)\) + 
          1.6916278518476895`*^-55\ \[ImaginaryI]}, {0.41000000000000003`, \(\
\(3.5826864435299566`\)\(\[InvisibleSpace]\)\) + 
          1.795724718214071`*^-55\ \[ImaginaryI]}, {0.42`, \
\(\(3.4943030285805747`\)\(\[InvisibleSpace]\)\) + 
          3.348270321954415`*^-55\ \[ImaginaryI]}, {0.43`, \
\(\(3.4087324354192132`\)\(\[InvisibleSpace]\)\) + 
          2.484796255659155`*^-55\ \[ImaginaryI]}, {0.44`, \
\(\(3.325901403383241`\)\(\[InvisibleSpace]\)\) + 
          2.3602951622719787`*^-55\ \[ImaginaryI]}, {0.45`, \
\(\(3.245734688957353`\)\(\[InvisibleSpace]\)\) + 
          2.105232055581516`*^-55\ \[ImaginaryI]}, {0.46`, \
\(\(3.1681557810131427`\)\(\[InvisibleSpace]\)\) + 
          4.441965344580958`*^-55\ \[ImaginaryI]}, {0.47000000000000003`, \
\(\(3.0930875129200737`\)\(\[InvisibleSpace]\)\) + 
          3.475547363973203`*^-55\ \[ImaginaryI]}, {0.48`, \
\(\(3.0204525831383124`\)\(\[InvisibleSpace]\)\) + 
          1.2775742599056662`*^-55\ \[ImaginaryI]}, {0.49`, \
\(\(2.9501739949324564`\)\(\[InvisibleSpace]\)\) + 
          3.1929447431728996`*^-55\ \[ImaginaryI]}, {0.5`, \
\(\(2.8821754248873175`\)\(\[InvisibleSpace]\)\) + 
          1.962339884420036`*^-55\ \[ImaginaryI]}, {0.51`, \
\(\(2.816381528982346`\)\(\[InvisibleSpace]\)\) + 
          1.1382868048494616`*^-55\ \[ImaginaryI]}, {0.52`, \
\(\(2.7527181941032284`\)\(\[InvisibleSpace]\)\) + 
          1.466005766252087`*^-55\ \[ImaginaryI]}, {0.53`, \
\(\(2.691112742046101`\)\(\[InvisibleSpace]\)\) + 
          4.05802568358309`*^-55\ \[ImaginaryI]}, {0.54`, \
\(\(2.6314940923063936`\)\(\[InvisibleSpace]\)\) + 
          9.37883736633449`*^-56\ \[ImaginaryI]}, {0.55`, \
\(\(2.5737928892424016`\)\(\[InvisibleSpace]\)\) + 
          2.0349845063962026`*^-55\ \[ImaginaryI]}, {0.56`, \
\(\(2.5179415985629516`\)\(\[InvisibleSpace]\)\) + 
          
          1.358934719555151`*^-55\ \[ImaginaryI]}, {0.5700000000000001`, \
\(\(2.463874577507386`\)\(\[InvisibleSpace]\)\) + 
          7.574024745484273`*^-56\ \[ImaginaryI]}, {0.58`, \
\(\(2.4115281225618563`\)\(\[InvisibleSpace]\)\) + 
          8.624759455997248`*^-56\ \[ImaginaryI]}, {0.59`, \
\(\(2.3608404980850555`\)\(\[InvisibleSpace]\)\) + 
          3.027312410624375`*^-55\ \[ImaginaryI]}, {0.6`, \
\(\(2.3117519487955693`\)\(\[InvisibleSpace]\)\) + 
          1.2655073123039506`*^-55\ \[ImaginaryI]}, {0.61`, \
\(\(2.2642046986978555`\)\(\[InvisibleSpace]\)\) + 
          8.720903715779199`*^-56\ \[ImaginaryI]}, {0.62`, \
\(\(2.2181429386907143`\)\(\[InvisibleSpace]\)\) + 
          6.920749569359762`*^-56\ \[ImaginaryI]}, {0.63`, \
\(\(2.173512804807058`\)\(\[InvisibleSpace]\)\) + 
          5.602414745247729`*^-56\ \[ImaginaryI]}, {0.64`, \
\(\(2.130262348773254`\)\(\[InvisibleSpace]\)\) + 
          8.898160282707148`*^-56\ \[ImaginaryI]}, {0.65`, \
\(\(2.0883415023467045`\)\(\[InvisibleSpace]\)\) + 
          4.294540650797308`*^-56\ \[ImaginaryI]}, {0.66`, \
\(\(2.0477020366885643`\)\(\[InvisibleSpace]\)\) + 
          5.744866844414846`*^-56\ \[ImaginaryI]}, {0.67`, \
\(\(2.0082975178514517`\)\(\[InvisibleSpace]\)\) + 
          5.033873772535636`*^-56\ \[ImaginaryI]}, {0.68`, \
\(\(1.970083259307085`\)\(\[InvisibleSpace]\)\) + 
          4.837089292427959`*^-56\ \[ImaginaryI]}, {0.6900000000000001`, \
\(\(1.9330162723033508`\)\(\[InvisibleSpace]\)\) + 
          3.5964773764730596`*^-56\ \[ImaginaryI]}, {0.7000000000000001`, \
\(\(1.8970552147222717`\)\(\[InvisibleSpace]\)\) + 
          5.718702992852534`*^-56\ \[ImaginaryI]}, {0.71`, \
\(\(1.8621603390075387`\)\(\[InvisibleSpace]\)\) + 
          6.313260431891049`*^-56\ \[ImaginaryI]}, {0.72`, \
\(\(1.828293439641005`\)\(\[InvisibleSpace]\)\) + 
          3.16800952909811`*^-56\ \[ImaginaryI]}, {0.73`, \
\(\(1.7954178005700814`\)\(\[InvisibleSpace]\)\) + 
          1.4237272537474888`*^-55\ \[ImaginaryI]}, {0.74`, \
\(\(1.7634981429209753`\)\(\[InvisibleSpace]\)\) + 
          3.235804500034764`*^-56\ \[ImaginaryI]}, {0.75`, \
\(\(1.732500573274791`\)\(\[InvisibleSpace]\)\) + 
          5.888335748277298`*^-56\ \[ImaginaryI]}, {0.76`, \
\(\(1.7023925327336675`\)\(\[InvisibleSpace]\)\) + 
          2.4410634414824137`*^-56\ \[ImaginaryI]}, {0.77`, \
\(\(1.6731427469611846`\)\(\[InvisibleSpace]\)\) + 
          2.5277314095265354`*^-56\ \[ImaginaryI]}, {0.78`, \
\(\(1.6447211773444934`\)\(\[InvisibleSpace]\)\) + 
          3.3422019322708904`*^-56\ \[ImaginaryI]}, {0.79`, \
\(\(1.6170989733941623`\)\(\[InvisibleSpace]\)\) + 
          1.498698883507114`*^-25\ \[ImaginaryI]}, {0.8`, \
\(\(1.5902484264708907`\)\(\[InvisibleSpace]\)\) + 
          5.174393498197768`*^-56\ \[ImaginaryI]}, {0.81`, \
\(\(1.5641429249054575`\)\(\[InvisibleSpace]\)\) + 
          2.4576143337884916`*^-56\ \[ImaginaryI]}, {0.8200000000000001`, \
\(\(1.538756910558988`\)\(\[InvisibleSpace]\)\) + 
          2.6433993675952485`*^-26\ \[ImaginaryI]}, {0.8300000000000001`, \
\(\(1.5140658368543825`\)\(\[InvisibleSpace]\)\) + 
          2.4237206924784536`*^-56\ \[ImaginaryI]}, {0.84`, \
\(\(1.4900461282961752`\)\(\[InvisibleSpace]\)\) + 
          2.0346161448629362`*^-56\ \[ImaginaryI]}, {0.85`, \
\(\(1.466675141484766`\)\(\[InvisibleSpace]\)\) + 
          2.21706867318766`*^-26\ \[ImaginaryI]}, {0.86`, \
\(\(1.4439311276216884`\)\(\[InvisibleSpace]\)\) + 
          3.784450336135506`*^-26\ \[ImaginaryI]}, {0.87`, \
\(\(1.4217931964949355`\)\(\[InvisibleSpace]\)\) + 
          1.614498430585681`*^-26\ \[ImaginaryI]}, {0.88`, \
\(\(1.4002412819272856`\)\(\[InvisibleSpace]\)\) + 
          3.743695433002142`*^-56\ \[ImaginaryI]}, {0.89`, \
\(\(1.3792561086657125`\)\(\[InvisibleSpace]\)\) + 
          1.4182939646153877`*^-26\ \[ImaginaryI]}, {0.9`, \
\(\(1.358819160686247`\)\(\[InvisibleSpace]\)\) + 
          1.2004624197953211`*^-26\ \[ImaginaryI]}, {0.91`, \
\(\(1.338912650885889`\)\(\[InvisibleSpace]\)\) + 
          1.9743256342645634`*^-26\ \[ImaginaryI]}, {0.92`, \
\(\(1.319519492131211`\)\(\[InvisibleSpace]\)\) + 
          2.48856309935872`*^-26\ \[ImaginaryI]}, {0.93`, \
\(\(1.300623269632086`\)\(\[InvisibleSpace]\)\) + 
          
          1.0747072156480738`*^-26\ \[ImaginaryI]}, {0.9400000000000001`, \
\(\(1.282208214608335`\)\(\[InvisibleSpace]\)\) + 
          9.467214932238999`*^-27\ \[ImaginaryI]}, {0.9500000000000001`, \
\(\(1.264259179217043`\)\(\[InvisibleSpace]\)\) + 
          1.2009600833670123`*^-26\ \[ImaginaryI]}, {0.96`, \
\(\(1.2467616127086674`\)\(\[InvisibleSpace]\)\) + 
          7.835233601563979`*^-27\ \[ImaginaryI]}, {0.97`, \
\(\(1.229701538780879`\)\(\[InvisibleSpace]\)\) + 
          8.572557772034024`*^-57\ \[ImaginaryI]}, {0.98`, \
\(\(1.2130655341002434`\)\(\[InvisibleSpace]\)\) + 
          1.33847885251387`*^-26\ \[ImaginaryI]}, {0.99`, \
\(\(1.1968407079633376`\)\(\[InvisibleSpace]\)\) + 
          1.885523207818406`*^-26\ \[ImaginaryI]}, {1.`, \
\(\(1.1810146830706887`\)\(\[InvisibleSpace]\)\) + 
          1.1214998239241902`*^-26\ \[ImaginaryI]}, {1.01`, \
\(\(1.165575577388958`\)\(\[InvisibleSpace]\)\) + 
          4.936473370130903`*^-27\ \[ImaginaryI]}, {1.02`, \
\(\(1.1505119870790996`\)\(\[InvisibleSpace]\)\) + 
          1.6357244764212546`*^-26\ \[ImaginaryI]}, {1.03`, \
\(\(1.1358129704707314`\)\(\[InvisibleSpace]\)\) + 
          3.9519879469297196`*^-27\ \[ImaginaryI]}, {1.04`, \
\(\(1.121468033065725`\)\(\[InvisibleSpace]\)\) + 
          9.921084557065406`*^-57\ \[ImaginaryI]}, {1.05`, \
\(\(1.1074671135569565`\)\(\[InvisibleSpace]\)\) + 
          5.483728637782915`*^-57\ \[ImaginaryI]}, {1.06`, \
\(\(1.0938005708513496`\)\(\[InvisibleSpace]\)\) + 
          6.068603617895443`*^-27\ \[ImaginaryI]}, {1.07`, \
\(\(1.0804591720897196`\)\(\[InvisibleSpace]\)\) + 
          1.0910233521751927`*^-56\ \[ImaginaryI]}, {1.08`, \
\(\(1.0674340816595083`\)\(\[InvisibleSpace]\)\) + 
          5.413845081358546`*^-57\ \[ImaginaryI]}, {1.09`, \
\(\(1.0547168512003222`\)\(\[InvisibleSpace]\)\) + 
          3.671976824159691`*^-57\ \[ImaginaryI]}, {1.1`, \
\(\(1.042299410606176`\)\(\[InvisibleSpace]\)\) + 
          4.853386886878006`*^-27\ \[ImaginaryI]}, {1.11`, \
\(\(1.030174060032579`\)\(\[InvisibleSpace]\)\) + 
          2.2208268590108363`*^-27\ \[ImaginaryI]}, {1.12`, \
\(\(1.0183334629210137`\)\(\[InvisibleSpace]\)\) + 
          2.3247896043459624`*^-27\ \[ImaginaryI]}, {1.1300000000000001`, \
\(\(1.0067706400579737`\)\(\[InvisibleSpace]\)\) + 
          2.009669044808278`*^-27\ \[ImaginaryI]}, {1.1400000000000001`, \
\(\(0.9954789646904798`\)\(\[InvisibleSpace]\)\) + 
          2.8196173359666256`*^-57\ \[ImaginaryI]}, {1.1500000000000001`, \
\(\(0.9844521587249113`\)\(\[InvisibleSpace]\)\) + 
          4.740448408174007`*^-27\ \[ImaginaryI]}, {1.16`, \
\(\(0.9736842900409419`\)\(\[InvisibleSpace]\)\) + 
          1.2813907818351667`*^-27\ \[ImaginaryI]}, {1.17`, \
\(\(0.9631697709573401`\)\(\[InvisibleSpace]\)\) + 
          2.0330163235414245`*^-57\ \[ImaginaryI]}, {1.18`, \
\(\(0.9529033578912232`\)\(\[InvisibleSpace]\)\) + 
          2.5339467772741013`*^-27\ \[ImaginaryI]}, {1.19`, \
\(\(0.9428801522569177`\)\(\[InvisibleSpace]\)\) + 
          9.916104969996723`*^-28\ \[ImaginaryI]}, {1.2`, \
\(\(0.9330956026546517`\)\(\[InvisibleSpace]\)\) + 
          1.6425891146178716`*^-27\ \[ImaginaryI]}, {1.21`, \
\(\(0.9235455084026052`\)\(\[InvisibleSpace]\)\) + 
          6.2294793001683115`*^-28\ \[ImaginaryI]}, {1.22`, \
\(\(0.9142260244680087`\)\(\[InvisibleSpace]\)\) + 
          5.387274262365993`*^-28\ \[ImaginaryI]}, {1.23`, \
\(\(0.905133667853535`\)\(\[InvisibleSpace]\)\) + 
          9.607000695823986`*^-28\ \[ImaginaryI]}, {1.24`, \
\(\(0.8962653254935184`\)\(\[InvisibleSpace]\)\) + 
          1.0344039116253253`*^-27\ \[ImaginaryI]}, {1.25`, \
\(\(0.887618263709825`\)\(\[InvisibleSpace]\)\) + 
          5.690752868962891`*^-28\ \[ImaginaryI]}, {1.26`, \
\(\(0.8791901392684275`\)\(\[InvisibleSpace]\)\) + 
          1.1939972746731836`*^-27\ \[ImaginaryI]}, {1.27`, \
\(\(0.8709790120637122`\)\(\[InvisibleSpace]\)\) + 
          2.9688973115369817`*^-28\ \[ImaginaryI]}, {1.28`, \
\(\(0.8629833594366902`\)\(\[InvisibleSpace]\)\) + 
          4.043115087609675`*^-28\ \[ImaginaryI]}, {1.29`, \
\(\(0.855202092103744`\)\(\[InvisibleSpace]\)\) + 
          7.011256391817474`*^-58\ \[ImaginaryI]}, {1.3`, \
\(\(0.8476345716320607`\)\(\[InvisibleSpace]\)\) + 
          
          1.7712818097107688`*^-28\ \[ImaginaryI]}, {1.31`, \
\(\(0.8402806293438689`\)\(\[InvisibleSpace]\)\) + 
          2.1750851180086484`*^-28\ \[ImaginaryI]}, {1.32`, \
\(\(0.8331405864609304`\)\(\[InvisibleSpace]\)\) + 
          4.825278958167898`*^-28\ \[ImaginaryI]}, {1.33`, \
\(\(0.8262152752101338`\)\(\[InvisibleSpace]\)\) + 
          2.7024096554880443`*^-28\ \[ImaginaryI]}, {1.34`, \
\(\(0.819506060496876`\)\(\[InvisibleSpace]\)\) + 
          1.2977858635590687`*^-28\ \[ImaginaryI]}, {1.35`, \
\(\(0.8130148616116194`\)\(\[InvisibleSpace]\)\) + 
          9.442693784128935`*^-29\ \[ImaginaryI]}, {1.36`, \
\(\(0.8067441732634351`\)\(\[InvisibleSpace]\)\) + 
          7.169451704147751`*^-29\ \[ImaginaryI]}, {1.37`, \
\(\(0.8006970850302494`\)\(\[InvisibleSpace]\)\) + 
          1.1993212748781487`*^-28\ \[ImaginaryI]}, {1.3800000000000001`, \
\(\(0.7948772980785541`\)\(\[InvisibleSpace]\)\) + 
          4.5707893103914555`*^-29\ \[ImaginaryI]}, {1.3900000000000001`, \
\(\(0.7892891377380733`\)\(\[InvisibleSpace]\)\) + 
          4.3713800605194487`*^-29\ \[ImaginaryI]}, {1.4000000000000001`, \
\(\(0.7839375602264702`\)\(\[InvisibleSpace]\)\) + 
          3.2443161571154615`*^-29\ \[ImaginaryI]}, {1.41`, \
\(\(0.7788281515188544`\)\(\[InvisibleSpace]\)\) + 
          1.7390044102533378`*^-29\ \[ImaginaryI]}, {1.42`, \
\(\(0.7739671160684208`\)\(\[InvisibleSpace]\)\) + 
          1.621582805276203`*^-29\ \[ImaginaryI]}, {1.43`, \
\(\(0.7693612528397301`\)\(\[InvisibleSpace]\)\) + 
          2.0375699729410055`*^-29\ \[ImaginaryI]}, {1.44`, \
\(\(0.7650179159585258`\)\(\[InvisibleSpace]\)\) + 
          7.719699594242932`*^-30\ \[ImaginaryI]}, {1.45`, \
\(\(0.7609449572667146`\)\(\[InvisibleSpace]\)\) + 
          5.134077700234205`*^-59\ \[ImaginaryI]}, {1.46`, \
\(\(0.757150648263057`\)\(\[InvisibleSpace]\)\) + 
          3.428040381301858`*^-30\ \[ImaginaryI]}, {1.47`, \
\(\(0.7536435793783486`\)\(\[InvisibleSpace]\)\) + 
          3.6338314571791515`*^-30\ \[ImaginaryI]}, {1.48`, \
\(\(0.7504325353419685`\)\(\[InvisibleSpace]\)\) + 
          
          1.3146819484299599`*^-30\ \[ImaginaryI]}, {1.49`, \
\(\(0.7475263465885927`\)\(\[InvisibleSpace]\)\) + 
          9.206661916406242`*^-31\ \[ImaginaryI]}, {1.5`, \
\(\(0.7449337182352012`\)\(\[InvisibleSpace]\)\) + 
          1.0005543558694035`*^-30\ \[ImaginaryI]}, {1.51`, \
\(\(0.742663040077542`\)\(\[InvisibleSpace]\)\) + 
          2.499595014686579`*^-31\ \[ImaginaryI]}, {1.52`, \
\(\(0.7407221831860371`\)\(\[InvisibleSpace]\)\) + 
          1.333351577954149`*^-31\ \[ImaginaryI]}, {1.53`, \
\(\(0.7391182908180548`\)\(\[InvisibleSpace]\)\) + 
          5.303497220357455`*^-32\ \[ImaginaryI]}, {1.54`, \
\(\(0.7378575732312873`\)\(\[InvisibleSpace]\)\) + 
          8.363239506124483`*^-33\ \[ImaginaryI]}, {1.55`, \
\(\(0.7369451172693737`\)\(\[InvisibleSpace]\)\) + 
          1.1722145063887156`*^-33\ \[ImaginaryI]}, {1.56`, \
\(\(0.7363847220000694`\)\(\[InvisibleSpace]\)\) + 
          4.965186003356222`*^-35\ \[ImaginaryI]}, {1.57`, \
\(\(0.7361787710046104`\)\(\[InvisibleSpace]\)\) + 
          1.1933106206055553`*^-40\ \[ImaginaryI]}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{th, 
        N[mp[1, 1, 
            Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], \(-9\)/
              10]]}, {th, 0.01, Pi/2, 0.01}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, \(\(0.6612697181696835`\)\(\[InvisibleSpace]\)\) + 
          1.2688179537457422`*^-76\ \[ImaginaryI]}, {0.02`, \
\(\(0.6635221897121905`\)\(\[InvisibleSpace]\)\) + 
          9.653891927932688`*^-74\ \[ImaginaryI]}, {0.03`, \
\(\(0.6669958807986851`\)\(\[InvisibleSpace]\)\) + 
          4.998925969835249`*^-74\ \[ImaginaryI]}, {0.04`, \
\(\(0.6714082805146675`\)\(\[InvisibleSpace]\)\) + 
          1.584914571939874`*^-73\ \[ImaginaryI]}, {0.05`, \
\(\(0.6765153104654936`\)\(\[InvisibleSpace]\)\) + 
          3.893205020005363`*^-49\ \[ImaginaryI]}, {0.06`, \
\(\(0.6821330591939189`\)\(\[InvisibleSpace]\)\) + 
          5.9333147291083305`*^-49\ \[ImaginaryI]}, {0.07`, \
\(\(0.6881314208106776`\)\(\[InvisibleSpace]\)\) + 
          2.53391363507727`*^-51\ \[ImaginaryI]}, {0.08`, \
\(\(0.6944205282178326`\)\(\[InvisibleSpace]\)\) + 
          1.6498492127902014`*^-51\ \[ImaginaryI]}, {0.09`, \
\(\(0.7009385670541164`\)\(\[InvisibleSpace]\)\) + 
          2.770625098803671`*^-51\ \[ImaginaryI]}, {0.1`, \
\(\(0.707642825165457`\)\(\[InvisibleSpace]\)\) + 
          1.614562185834603`*^-51\ \[ImaginaryI]}, {0.11`, \
\(\(0.7145035808681792`\)\(\[InvisibleSpace]\)\) + 
          1.930837676479766`*^-51\ \[ImaginaryI]}, {0.12`, \
\(\(0.7215000302969813`\)\(\[InvisibleSpace]\)\) + 
          1.9360642704970737`*^-51\ \[ImaginaryI]}, {0.13`, \
\(\(0.7286175814853002`\)\(\[InvisibleSpace]\)\) + 
          4.573129091318545`*^-51\ \[ImaginaryI]}, {0.14`, \
\(\(0.73584604260966`\)\(\[InvisibleSpace]\)\) + 
          1.773461762889889`*^-51\ \[ImaginaryI]}, {0.15`, \
\(\(0.7431783937667252`\)\(\[InvisibleSpace]\)\) + 
          1.6235422988468163`*^-51\ \[ImaginaryI]}, {0.16`, \
\(\(0.750609942099234`\)\(\[InvisibleSpace]\)\) + 
          2.2843615630887647`*^-51\ \[ImaginaryI]}, {0.17`, \
\(\(0.7581377317462316`\)\(\[InvisibleSpace]\)\) + 
          3.857179997691281`*^-51\ \[ImaginaryI]}, {0.18`, \
\(\(0.7657601256059107`\)\(\[InvisibleSpace]\)\) + 
          3.675535585074366`*^-51\ \[ImaginaryI]}, {0.19`, \
\(\(0.7734765046415514`\)\(\[InvisibleSpace]\)\) + 
          
          2.0120933665677277`*^-51\ \[ImaginaryI]}, {0.2`, \
\(\(0.781287048847871`\)\(\[InvisibleSpace]\)\) + 
          3.381370252659992`*^-51\ \[ImaginaryI]}, {0.21`, \
\(\(0.7891925757484873`\)\(\[InvisibleSpace]\)\) + 
          3.307136151655115`*^-51\ \[ImaginaryI]}, {0.22`, \
\(\(0.7971944200194911`\)\(\[InvisibleSpace]\)\) + 
          2.2910809837260837`*^-51\ \[ImaginaryI]}, {0.23`, \
\(\(0.8052943429057129`\)\(\[InvisibleSpace]\)\) + 
          1.0871904669994959`*^-55\ \[ImaginaryI]}, {0.24`, \
\(\(0.8134944634988363`\)\(\[InvisibleSpace]\)\) + 
          7.724951380560019`*^-56\ \[ImaginaryI]}, {0.25`, \
\(\(0.8217972062523367`\)\(\[InvisibleSpace]\)\) + 
          9.000210966391374`*^-56\ \[ImaginaryI]}, {0.26`, \
\(\(0.8302052606992294`\)\(\[InvisibleSpace]\)\) + 
          1.005350878959606`*^-55\ \[ImaginaryI]}, {0.27`, \
\(\(0.8387215504457673`\)\(\[InvisibleSpace]\)\) + 
          1.653701678522782`*^-55\ \[ImaginaryI]}, {0.28`, \
\(\(0.8473492092871513`\)\(\[InvisibleSpace]\)\) + 
          1.1032833828555258`*^-55\ \[ImaginaryI]}, {0.29`, \
\(\(0.856091562854715`\)\(\[InvisibleSpace]\)\) + 
          1.1556755220987876`*^-55\ \[ImaginaryI]}, {0.3`, \
\(\(0.8649521146032771`\)\(\[InvisibleSpace]\)\) + 
          1.2401717969484895`*^-55\ \[ImaginaryI]}, {0.31`, \
\(\(0.8739345352335082`\)\(\[InvisibleSpace]\)\) + 
          8.88275061431064`*^-56\ \[ImaginaryI]}, {0.32`, \
\(\(0.8830426548694473`\)\(\[InvisibleSpace]\)\) + 
          1.042121346134616`*^-55\ \[ImaginaryI]}, {0.33`, \
\(\(0.8922804574612455`\)\(\[InvisibleSpace]\)\) + 
          2.7305439663184227`*^-55\ \[ImaginaryI]}, {0.34`, \
\(\(0.9016520770102678`\)\(\[InvisibleSpace]\)\) + 
          9.451645780200951`*^-56\ \[ImaginaryI]}, {0.35000000000000003`, \
\(\(0.9111617952981154`\)\(\[InvisibleSpace]\)\) + 
          8.419905585271853`*^-56\ \[ImaginaryI]}, {0.36`, \
\(\(0.9208140408770469`\)\(\[InvisibleSpace]\)\) + 
          1.775580225647131`*^-55\ \[ImaginaryI]}, {0.37`, \
\(\(0.9306133891251053`\)\(\[InvisibleSpace]\)\) + 
          1.7310957131092356`*^-55\ \[ImaginaryI]}, {0.38`, \
\(\(0.9405645632162706`\)\(\[InvisibleSpace]\)\) + 
          9.956791216608641`*^-56\ \[ImaginaryI]}, {0.39`, \
\(\(0.9506724358837455`\)\(\[InvisibleSpace]\)\) + 
          1.6370400943233484`*^-55\ \[ImaginaryI]}, {0.4`, \
\(\(0.9609420318833564`\)\(\[InvisibleSpace]\)\) + 
          9.081174495008818`*^-56\ \[ImaginaryI]}, {0.41000000000000003`, \
\(\(0.9713785310801661`\)\(\[InvisibleSpace]\)\) + 
          1.0135424655877845`*^-55\ \[ImaginaryI]}, {0.42`, \
\(\(0.9819872721008756`\)\(\[InvisibleSpace]\)\) + 
          1.988991747062795`*^-55\ \[ImaginaryI]}, {0.43`, \
\(\(0.9927737565044881`\)\(\[InvisibleSpace]\)\) + 
          1.554958886098146`*^-55\ \[ImaginaryI]}, {0.44`, \
\(\(1.0037436534359738`\)\(\[InvisibleSpace]\)\) + 
          1.557324796537343`*^-55\ \[ImaginaryI]}, {0.45`, \
\(\(1.0149028047347548`\)\(\[InvisibleSpace]\)\) + 
          1.4656616520780677`*^-55\ \[ImaginaryI]}, {0.46`, \
\(\(1.026257230477381`\)\(\[InvisibleSpace]\)\) + 
          3.265396474306424`*^-55\ \[ImaginaryI]}, {0.47000000000000003`, \
\(\(1.0378131349390771`\)\(\[InvisibleSpace]\)\) + 
          2.6995406157006872`*^-55\ \[ImaginaryI]}, {0.48`, \
\(\(1.0495769129639068`\)\(\[InvisibleSpace]\)\) + 
          1.0490903025873442`*^-55\ \[ImaginaryI]}, {0.49`, \
\(\(1.0615551567365262`\)\(\[InvisibleSpace]\)\) + 
          2.773383319832461`*^-55\ \[ImaginaryI]}, {0.5`, \
\(\(1.0737546629525956`\)\(\[InvisibleSpace]\)\) + 
          1.803831018603373`*^-55\ \[ImaginaryI]}, {0.51`, \
\(\(1.0861824403872151`\)\(\[InvisibleSpace]\)\) + 
          1.1078203272414901`*^-55\ \[ImaginaryI]}, {0.52`, \
\(\(1.09884571786294`\)\(\[InvisibleSpace]\)\) + 
          1.511212300775265`*^-55\ \[ImaginaryI]}, {0.53`, \
\(\(1.1117519526215456`\)\(\[InvisibleSpace]\)\) + 
          4.432388326393793`*^-55\ \[ImaginaryI]}, {0.54`, \
\(\(1.1249088391050268`\)\(\[InvisibleSpace]\)\) + 
          1.0858068806337841`*^-55\ \[ImaginaryI]}, {0.55`, \
\(\(1.138324318152584`\)\(\[InvisibleSpace]\)\) + 
          2.497930137837236`*^-55\ \[ImaginaryI]}, {0.56`, \
\(\(1.152006586622284`\)\(\[InvisibleSpace]\)\) + 
          
          1.76911992342286`*^-55\ \[ImaginaryI]}, {0.5700000000000001`, \
\(\(1.1659641074464218`\)\(\[InvisibleSpace]\)\) + 
          1.0460171195646134`*^-55\ \[ImaginaryI]}, {0.58`, \
\(\(1.1802056201308493`\)\(\[InvisibleSpace]\)\) + 
          1.2639135513978851`*^-55\ \[ImaginaryI]}, {0.59`, \
\(\(1.1947401517094152`\)\(\[InvisibleSpace]\)\) + 
          4.708502338768335`*^-55\ \[ImaginaryI]}, {0.6`, \
\(\(1.209577028164715`\)\(\[InvisibleSpace]\)\) + 
          2.089464765810413`*^-55\ \[ImaginaryI]}, {0.61`, \
\(\(1.2247258863271755`\)\(\[InvisibleSpace]\)\) + 
          1.5288328198292667`*^-55\ \[ImaginaryI]}, {0.62`, \
\(\(1.2401966862646516`\)\(\[InvisibleSpace]\)\) + 
          1.288419635592521`*^-55\ \[ImaginaryI]}, {0.63`, \
\(\(1.255999724174505`\)\(\[InvisibleSpace]\)\) + 
          1.1077906805741263`*^-55\ \[ImaginaryI]}, {0.64`, \
\(\(1.2721456457903222`\)\(\[InvisibleSpace]\)\) + 
          1.8690850240973988`*^-55\ \[ImaginaryI]}, {0.65`, \
\(\(1.2886454603148918`\)\(\[InvisibleSpace]\)\) + 
          9.58420721548035`*^-56\ \[ImaginaryI]}, {0.66`, \
\(\(1.3055105548904602`\)\(\[InvisibleSpace]\)\) + 
          1.3623571741173677`*^-55\ \[ImaginaryI]}, {0.67`, \
\(\(1.3227527096165854`\)\(\[InvisibleSpace]\)\) + 
          1.2686530540786887`*^-55\ \[ImaginaryI]}, {0.68`, \
\(\(1.3403841131246945`\)\(\[InvisibleSpace]\)\) + 
          1.2957156284816788`*^-55\ \[ImaginaryI]}, {0.6900000000000001`, \
\(\(1.3584173787169114`\)\(\[InvisibleSpace]\)\) + 
          1.024097691955211`*^-55\ \[ImaginaryI]}, {0.7000000000000001`, \
\(\(1.3768655610749938`\)\(\[InvisibleSpace]\)\) + 
          1.7312178944053404`*^-55\ \[ImaginaryI]}, {0.71`, \
\(\(1.395742173542922`\)\(\[InvisibleSpace]\)\) + 
          2.03211500548686`*^-55\ \[ImaginaryI]}, {0.72`, \
\(\(1.4150612059838323`\)\(\[InvisibleSpace]\)\) + 
          1.0843526894616329`*^-55\ \[ImaginaryI]}, {0.73`, \
\(\(1.4348371432087834`\)\(\[InvisibleSpace]\)\) + 
          5.182611215361321`*^-55\ \[ImaginaryI]}, {0.74`, \
\(\(1.4550849839706979`\)\(\[InvisibleSpace]\)\) + 
          1.2528222817278893`*^-55\ \[ImaginaryI]}, {0.75`, \
\(\(1.4758202605123178`\)\(\[InvisibleSpace]\)\) + 
          2.4251148545968788`*^-55\ \[ImaginaryI]}, {0.76`, \
\(\(1.4970590586512489`\)\(\[InvisibleSpace]\)\) + 
          1.0695431821984521`*^-55\ \[ImaginaryI]}, {0.77`, \
\(\(1.5188180383787981`\)\(\[InvisibleSpace]\)\) + 
          1.1783561854126478`*^-55\ \[ImaginaryI]}, {0.78`, \
\(\(1.5411144549416007`\)\(\[InvisibleSpace]\)\) + 
          1.657874784582517`*^-55\ \[ImaginaryI]}, {0.79`, \
\(\(1.5639661803662934`\)\(\[InvisibleSpace]\)\) + 
          7.911411736241153`*^-25\ \[ImaginaryI]}, {0.8`, \
\(\(1.5873917253769787`\)\(\[InvisibleSpace]\)\) + 
          2.907147882604584`*^-55\ \[ImaginaryI]}, {0.81`, \
\(\(1.6114102616435722`\)\(\[InvisibleSpace]\)\) + 
          1.4697293923183044`*^-55\ \[ImaginaryI]}, {0.8200000000000001`, \
\(\(1.6360416442851322`\)\(\[InvisibleSpace]\)\) + 
          1.6828708728013086`*^-25\ \[ImaginaryI]}, {0.8300000000000001`, \
\(\(1.6613064345365687`\)\(\[InvisibleSpace]\)\) + 
          1.642796866291241`*^-55\ \[ImaginaryI]}, {0.84`, \
\(\(1.6872259224687194`\)\(\[InvisibleSpace]\)\) + 
          1.4684073561145493`*^-55\ \[ImaginaryI]}, {0.85`, \
\(\(1.7138221496309964`\)\(\[InvisibleSpace]\)\) + 
          1.7039468885714706`*^-25\ \[ImaginaryI]}, {0.86`, \
\(\(1.7411179314616048`\)\(\[InvisibleSpace]\)\) + 
          3.0977275789747775`*^-25\ \[ImaginaryI]}, {0.87`, \
\(\(1.7691368792829167`\)\(\[InvisibleSpace]\)\) + 
          1.4076438057636176`*^-25\ \[ImaginaryI]}, {0.88`, \
\(\(1.797903421668113`\)\(\[InvisibleSpace]\)\) + 
          3.4771398435314605`*^-55\ \[ImaginaryI]}, {0.89`, \
\(\(1.8274428249293782`\)\(\[InvisibleSpace]\)\) + 
          1.4034813739867415`*^-25\ \[ImaginaryI]}, {0.9`, \
\(\(1.857781212436947`\)\(\[InvisibleSpace]\)\) + 
          1.2657862647177017`*^-25\ \[ImaginaryI]}, {0.91`, \
\(\(1.8889455824318646`\)\(\[InvisibleSpace]\)\) + 
          2.218477277102306`*^-25\ \[ImaginaryI]}, {0.92`, \
\(\(1.9209638239424567`\)\(\[InvisibleSpace]\)\) + 
          2.9803168268128792`*^-25\ \[ImaginaryI]}, {0.93`, \
\(\(1.953864730354456`\)\(\[InvisibleSpace]\)\) + 
          
          1.3719389327389477`*^-25\ \[ImaginaryI]}, {0.9400000000000001`, \
\(\(1.9876780101169151`\)\(\[InvisibleSpace]\)\) + 
          1.2884004581470114`*^-25\ \[ImaginaryI]}, {0.9500000000000001`, \
\(\(2.0224342939891082`\)\(\[InvisibleSpace]\)\) + 
          1.7425856904393846`*^-25\ \[ImaginaryI]}, {0.96`, \
\(\(2.058165138146924`\)\(\[InvisibleSpace]\)\) + 
          1.2122910852546655`*^-25\ \[ImaginaryI]}, {0.97`, \
\(\(2.0949030223695106`\)\(\[InvisibleSpace]\)\) + 
          1.4145116708787469`*^-55\ \[ImaginaryI]}, {0.98`, \
\(\(2.1326813424169284`\)\(\[InvisibleSpace]\)\) + 
          2.3555893835092597`*^-25\ \[ImaginaryI]}, {0.99`, \
\(\(2.171534395586246`\)\(\[InvisibleSpace]\)\) + 
          3.5396594772280178`*^-25\ \[ImaginaryI]}, {1.`, \
\(\(2.2114973582952246`\)\(\[InvisibleSpace]\)\) + 
          2.246044446078918`*^-25\ \[ImaginaryI]}, {1.01`, \
\(\(2.2526062543885117`\)\(\[InvisibleSpace]\)\) + 
          1.0548035023023075`*^-25\ \[ImaginaryI]}, {1.02`, \
\(\(2.2948979126893057`\)\(\[InvisibleSpace]\)\) + 
          3.7294461030811686`*^-25\ \[ImaginaryI]}, {1.03`, \
\(\(2.3384099121286894`\)\(\[InvisibleSpace]\)\) + 
          9.615464559935055`*^-26\ \[ImaginaryI]}, {1.04`, \
\(\(2.3831805125738255`\)\(\[InvisibleSpace]\)\) + 
          2.5761559282422894`*^-55\ \[ImaginaryI]}, {1.05`, \
\(\(2.4292485692436365`\)\(\[InvisibleSpace]\)\) + 
          1.519782461791481`*^-55\ \[ImaginaryI]}, {1.06`, \
\(\(2.476653428345826`\)\(\[InvisibleSpace]\)\) + 
          1.7952155013597956`*^-25\ \[ImaginaryI]}, {1.07`, \
\(\(2.525434801291081`\)\(\[InvisibleSpace]\)\) + 
          3.4451583645327825`*^-55\ \[ImaginaryI]}, {1.08`, \
\(\(2.5756326145392077`\)\(\[InvisibleSpace]\)\) + 
          1.8249363562317806`*^-55\ \[ImaginaryI]}, {1.09`, \
\(\(2.627286831808044`\)\(\[InvisibleSpace]\)\) + 
          1.3213617855091315`*^-55\ \[ImaginaryI]}, {1.1`, \
\(\(2.680437245030754`\)\(\[InvisibleSpace]\)\) + 
          1.8644585706062232`*^-25\ \[ImaginaryI]}, {1.11`, \
\(\(2.735123230083124`\)\(\[InvisibleSpace]\)\) + 
          9.107658555988823`*^-26\ \[ImaginaryI]}, {1.12`, \
\(\(2.791383462923665`\)\(\[InvisibleSpace]\)\) + 
          1.0177691774650347`*^-25\ \[ImaginaryI]}, {1.1300000000000001`, \
\(\(2.8492555914018527`\)\(\[InvisibleSpace]\)\) + 
          9.391681745803753`*^-26\ \[ImaginaryI]}, {1.1400000000000001`, \
\(\(2.90877585760213`\)\(\[InvisibleSpace]\)\) + 
          1.4064713962998586`*^-55\ \[ImaginaryI]}, {1.1500000000000001`, \
\(\(2.969978665214749`\)\(\[InvisibleSpace]\)\) + 
          2.5237061614819527`*^-25\ \[ImaginaryI]}, {1.16`, \
\(\(3.0328960860744876`\)\(\[InvisibleSpace]\)\) + 
          7.279861121343433`*^-26\ \[ImaginaryI]}, {1.17`, \
\(\(3.0975572997044805`\)\(\[InvisibleSpace]\)\) + 
          1.232348326537308`*^-55\ \[ImaginaryI]}, {1.18`, \
\(\(3.163987959469797`\)\(\[InvisibleSpace]\)\) + 
          1.6385295714846617`*^-25\ \[ImaginaryI]}, {1.19`, \
\(\(3.232209478815749`\)\(\[InvisibleSpace]\)\) + 
          6.838451422027976`*^-26\ \[ImaginaryI]}, {1.2`, \
\(\(3.3022382310776948`\)\(\[InvisibleSpace]\)\) + 
          1.2077642395292046`*^-25\ \[ImaginaryI]}, {1.21`, \
\(\(3.3740846565496847`\)\(\[InvisibleSpace]\)\) + 
          4.881998714741139`*^-26\ \[ImaginaryI]}, {1.22`, \
\(\(3.447752270944835`\)\(\[InvisibleSpace]\)\) + 
          4.4982279191277854`*^-26\ \[ImaginaryI]}, {1.23`, \
\(\(3.5232365701371364`\)\(\[InvisibleSpace]\)\) + 
          8.542721804237195`*^-26\ \[ImaginaryI]}, {1.24`, \
\(\(3.6005238272189795`\)\(\[InvisibleSpace]\)\) + 
          9.79080125691732`*^-26\ \[ImaginaryI]}, {1.25`, \
\(\(3.679589779527463`\)\(\[InvisibleSpace]\)\) + 
          5.730242731636754`*^-26\ \[ImaginaryI]}, {1.26`, \
\(\(3.7603982054810823`\)\(\[InvisibleSpace]\)\) + 
          1.2782253233131249`*^-25\ \[ImaginaryI]}, {1.27`, \
\(\(3.842899393928671`\)\(\[InvisibleSpace]\)\) + 
          3.376708652209254`*^-26\ \[ImaginaryI]}, {1.28`, \
\(\(3.927028512349617`\)\(\[InvisibleSpace]\)\) + 
          4.8816688261869787`*^-26\ \[ImaginaryI]}, {1.29`, \
\(\(4.012703884760326`\)\(\[InvisibleSpace]\)\) + 
          8.97894265998763`*^-56\ \[ImaginaryI]}, {1.3`, \
\(\(4.099825195667011`\)\(\[InvisibleSpace]\)\) + 
          
          2.4036951301150674`*^-26\ \[ImaginaryI]}, {1.31`, \
\(\(4.188271642927435`\)\(\[InvisibleSpace]\)\) + 
          3.124464218248075`*^-26\ \[ImaginaryI]}, {1.32`, \
\(\(4.277900069973698`\)\(\[InvisibleSpace]\)\) + 
          7.32879693267822`*^-26\ \[ImaginaryI]}, {1.33`, \
\(\(4.368543116479621`\)\(\[InvisibleSpace]\)\) + 
          4.334433620982777`*^-26\ \[ImaginaryI]}, {1.34`, \
\(\(4.460007436126269`\)\(\[InvisibleSpace]\)\) + 
          2.195180469332534`*^-26\ \[ImaginaryI]}, {1.35`, \
\(\(4.552072040422616`\)\(\[InvisibleSpace]\)\) + 
          1.6819701215233357`*^-26\ \[ImaginaryI]}, {1.36`, \
\(\(4.644486838243482`\)\(\[InvisibleSpace]\)\) + 
          1.3427215167815913`*^-26\ \[ImaginaryI]}, {1.37`, \
\(\(4.7369714513690795`\)\(\[InvisibleSpace]\)\) + 
          2.3577002052974638`*^-26\ \[ImaginaryI]}, {1.3800000000000001`, \
\(\(4.829214396195071`\)\(\[InvisibleSpace]\)\) + 
          9.415118340943591`*^-27\ \[ImaginaryI]}, {1.3900000000000001`, \
\(\(4.920872730091874`\)\(\[InvisibleSpace]\)\) + 
          9.41709534523444`*^-27\ \[ImaginaryI]}, {1.4000000000000001`, \
\(\(5.01157226661479`\)\(\[InvisibleSpace]\)\) + 
          7.294957400279759`*^-27\ \[ImaginaryI]}, {1.41`, \
\(\(5.100908465740986`\)\(\[InvisibleSpace]\)\) + 
          4.072826136960984`*^-27\ \[ImaginaryI]}, {1.42`, \
\(\(5.188448102281637`\)\(\[InvisibleSpace]\)\) + 
          3.9471532361007466`*^-27\ \[ImaginaryI]}, {1.43`, \
\(\(5.273731806331421`\)\(\[InvisibleSpace]\)\) + 
          5.143089757571928`*^-27\ \[ImaginaryI]}, {1.44`, \
\(\(5.356277552940208`\)\(\[InvisibleSpace]\)\) + 
          2.01587064165525`*^-27\ \[ImaginaryI]}, {1.45`, \
\(\(5.435585153274126`\)\(\[InvisibleSpace]\)\) + 
          1.3836645505046988`*^-56\ \[ImaginaryI]}, {1.46`, \
\(\(5.51114176599439`\)\(\[InvisibleSpace]\)\) + 
          9.511572031711317`*^-28\ \[ImaginaryI]}, {1.47`, \
\(\(5.582428405700534`\)\(\[InvisibleSpace]\)\) + 
          1.0354348290322024`*^-27\ \[ImaginaryI]}, {1.48`, \
\(\(5.648927376168515`\)\(\[InvisibleSpace]\)\) + 
          3.837346966201597`*^-28\ \[ImaginaryI]}, {1.49`, \
\(\(5.710130501820911`\)\(\[InvisibleSpace]\)\) + 
          2.745731943339226`*^-28\ \[ImaginaryI]}, {1.5`, \
\(\(5.765547974429987`\)\(\[InvisibleSpace]\)\) + 
          3.0411139203523254`*^-28\ \[ImaginaryI]}, {1.51`, \
\(\(5.814717577389308`\)\(\[InvisibleSpace]\)\) + 
          7.723063699480253`*^-29\ \[ImaginaryI]}, {1.52`, \
\(\(5.85721400155187`\)\(\[InvisibleSpace]\)\) + 
          4.1772409536222826`*^-29\ \[ImaginaryI]}, {1.53`, \
\(\(5.892657929428923`\)\(\[InvisibleSpace]\)\) + 
          1.6804895657572408`*^-29\ \[ImaginaryI]}, {1.54`, \
\(\(5.920724543021734`\)\(\[InvisibleSpace]\)\) + 
          2.6735533463977454`*^-30\ \[ImaginaryI]}, {1.55`, \
\(\(5.941151108430077`\)\(\[InvisibleSpace]\)\) + 
          3.771222223835993`*^-31\ \[ImaginaryI]}, {1.56`, \
\(\(5.95374330995195`\)\(\[InvisibleSpace]\)\) + 
          1.6036069538185793`*^-32\ \[ImaginaryI]}, {1.57`, \
\(\(5.958380048106877`\)\(\[InvisibleSpace]\)\) + 
          3.859530398885807`*^-38\ \[ImaginaryI]}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<mplus.dat\>", Re[%27]]\)], "Input"],

Cell[BoxData[
    \("mplus.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{th, 
        mm[3, 1, Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], 
          10]}, {th, 0.01, Pi/2, 0.01}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, \(-3.4454004015937363`\) + 
          1.5559362116108074`*^-47\ \[ImaginaryI]}, {0.02`, \
\(-3.434156128466813`\) + 
          1.9440033788688253`*^-47\ \[ImaginaryI]}, {0.03`, \
\(-3.415559818709949`\) + 
          4.3750640568663504`*^-52\ \[ImaginaryI]}, {0.04`, \
\(-3.389823278352077`\) + 
          1.4264752795234522`*^-51\ \[ImaginaryI]}, {0.05`, \
\(-3.3572344626844948`\) + 
          6.122158663054981`*^-52\ \[ImaginaryI]}, {0.06`, \
\(-3.3181494571122485`\) + 
          9.453914633444604`*^-52\ \[ImaginaryI]}, {0.07`, \
\(-3.27298290354766`\) + 
          4.0539200773172723`*^-54\ \[ImaginaryI]}, {0.08`, \
\(-3.2221973413441116`\) + 
          2.626950415482364`*^-54\ \[ImaginaryI]}, {0.09`, \
\(-3.16629194993076`\) + 
          4.354321453302808`*^-54\ \[ImaginaryI]}, {0.1`, \
\(-3.105791168831562`\) + 
          2.485700886012539`*^-54\ \[ImaginaryI]}, {0.11`, \
\(-3.041233628893604`\) + 
          2.8921147136571186`*^-54\ \[ImaginaryI]}, {0.12`, \
\(-2.9731617642299115`\) + 
          2.804017987550972`*^-54\ \[ImaginaryI]}, {0.13`, \
\(-2.902112396349555`\) + 
          6.368682869968115`*^-54\ \[ImaginaryI]}, {0.14`, \
\(-2.828608497365216`\) + 
          2.3629875718771024`*^-54\ \[ImaginaryI]}, {0.15`, \
\(-2.753152255831611`\) + 
          2.0604262960948472`*^-54\ \[ImaginaryI]}, {0.16`, \
\(-2.676219492136026`\) + 
          2.7502016336779744`*^-54\ \[ImaginaryI]}, {0.17`, \
\(-2.598255404632318`\) + 
          4.389469681168788`*^-54\ \[ImaginaryI]}, {0.18`, \
\(-2.519671575147972`\) + 
          3.941025705716255`*^-54\ \[ImaginaryI]}, {0.19`, \
\(-2.4408441238145824`\) + 
          2.0269388086870696`*^-54\ \[ImaginaryI]}, {0.2`, \
\(-2.3621128778793343`\) + 
          3.192175841151382`*^-54\ \[ImaginaryI]}, {0.21`, \
\(-2.2837814058001764`\) + 
          2.919267373467457`*^-54\ \[ImaginaryI]}, {0.22`, \
\(-2.2061177645873977`\) + 
          1.8872685876049316`*^-54\ \[ImaginaryI]}, {0.23`, \
\(-2.1293558128369185`\) + 
          1.9770187497082044`*^-54\ \[ImaginaryI]}, {0.24`, \
\(-2.05369695206131`\) + 
          1.3066929783861984`*^-54\ \[ImaginaryI]}, {0.25`, \
\(-1.9793121727980119`\) + 
          
          1.4143042587715576`*^-54\ \[ImaginaryI]}, {0.26`, \
\(-1.9063442978900957`\) + 
          1.466026793904875`*^-54\ \[ImaginaryI]}, {0.27`, \
\(-1.8349103319454947`\) + 
          2.2356918593360523`*^-54\ \[ImaginaryI]}, {0.28`, \
\(-1.7651038422778214`\) + 
          1.3817732592654842`*^-54\ \[ImaginaryI]}, {0.29`, \
\(-1.6969973119087909`\) + 
          1.3400056553079229`*^-54\ \[ImaginaryI]}, {0.3`, \
\(-1.6306444190196343`\) + 
          1.3306246079842964`*^-54\ \[ImaginaryI]}, {0.31`, \
\(-1.5660822093395317`\) + 
          8.815697289066926`*^-55\ \[ImaginaryI]}, {0.32`, \
\(-1.5033331382796278`\) + 
          9.564034083408733`*^-55\ \[ImaginaryI]}, {0.33`, \
\(-1.4424069681947604`\) + 
          2.3168867661002205`*^-54\ \[ImaginaryI]}, {0.34`, \
\(-1.3833025131026708`\) + 
          7.413989362574293`*^-55\ \[ImaginaryI]}, {0.35000000000000003`, \
\(-1.3260092286689318`\) + 
          6.105619769507976`*^-55\ \[ImaginaryI]}, {0.36`, \
\(-1.2705086494632392`\) + 
          1.190308420458299`*^-54\ \[ImaginaryI]}, {0.37`, \
\(-1.2167756785937704`\) + 
          1.0729560650474136`*^-54\ \[ImaginaryI]}, {0.38`, \
\(-1.1647797370176054`\) + 
          5.706768652850869`*^-55\ \[ImaginaryI]}, {0.39`, \
\(-1.1144857812716298`\) + 
          8.678191736963533`*^-55\ \[ImaginaryI]}, {0.4`, \
\(-1.0658551992229597`\) + 
          4.453672749913856`*^-55\ \[ImaginaryI]}, {0.41000000000000003`, \
\(-1.018846593829295`\) + 
          4.599894896326636`*^-55\ \[ImaginaryI]}, {0.42`, \
\(-0.9734164649404438`\) + 
          8.356144412392316`*^-55\ \[ImaginaryI]}, {0.43`, \
\(-0.9295197989532202`\) + 
          6.049344889562066`*^-55\ \[ImaginaryI]}, {0.44`, \
\(-0.8871105757295736`\) + 
          5.612354473522607`*^-55\ \[ImaginaryI]}, {0.45`, \
\(-0.8461422016613742`\) + 
          4.894913815305918`*^-55\ \[ImaginaryI]}, {0.46`, \
\(-0.806567877163056`\) + 
          1.0110421769535131`*^-54\ \[ImaginaryI]}, {0.47000000000000003`, \
\(-0.7683409062310687`\) + 
          7.752258484649867`*^-55\ \[ImaginaryI]}, {0.48`, \
\(-0.7314149550549216`\) + 
          2.795403866634722`*^-55\ \[ImaginaryI]}, {0.49`, \
\(-0.6957442660182003`\) + 
          6.860053019338794`*^-55\ \[ImaginaryI]}, {0.5`, \
\(-0.6612838328041153`\) + 
          4.143766491290332`*^-55\ \[ImaginaryI]}, {0.51`, \
\(-0.627989541728065`\) + 
          2.3645535653974894`*^-55\ \[ImaginaryI]}, {0.52`, \
\(-0.5958182838658368`\) + 
          2.99837625736119`*^-55\ \[ImaginaryI]}, {0.53`, \
\(-0.5647280420335559`\) + 
          8.178636718548682`*^-55\ \[ImaginaryI]}, {0.54`, \
\(-0.5346779562056658`\) + 
          1.8641483078900277`*^-55\ \[ImaginaryI]}, {0.55`, \
\(-0.5056283705298041`\) + 
          3.9920442382112205`*^-55\ \[ImaginaryI]}, {0.56`, \
\(-0.47754086471130885`\) + 
          2.6330653295411867`*^-55\ \[ImaginaryI]}, {0.5700000000000001`, \
\(-0.4503782721930998`\) + 
          1.4505538805815312`*^-55\ \[ImaginaryI]}, {0.58`, \
\(-0.42410468724646144`\) + 
          1.6338173501285504`*^-55\ \[ImaginaryI]}, {0.59`, \
\(-0.39868546281219364`\) + 
          5.676216737866314`*^-55\ \[ImaginaryI]}, {0.6`, \
\(-0.3740872006867739`\) + 
          2.3501665249582194`*^-55\ \[ImaginaryI]}, {0.61`, \
\(-0.35027773543185914`\) + 
          1.6051162451983163`*^-55\ \[ImaginaryI]}, {0.62`, \
\(-0.32722611319498984`\) + 
          1.263223498844365`*^-55\ \[ImaginaryI]}, {0.63`, \
\(-0.30490256646203373`\) + 
          1.0147246006320681`*^-55\ \[ImaginaryI]}, {0.64`, \
\(-0.28327848561536095`\) + 
          1.6002066701308237`*^-55\ \[ImaginaryI]}, {0.65`, \
\(-0.26232638804375585`\) + 
          7.672664765009951`*^-56\ \[ImaginaryI]}, {0.66`, \
\(-0.2420198854384461`\) + 
          1.020253218867975`*^-55\ \[ImaginaryI]}, {0.67`, \
\(-0.22233364981263115`\) + 
          8.891360034749119`*^-56\ \[ImaginaryI]}, {0.68`, \
\(-0.20324337869768205`\) + 
          8.502020015721424`*^-56\ \[ImaginaryI]}, {0.6900000000000001`, \
\(-0.1847257598962947`\) + 
          6.293859571852259`*^-56\ \[ImaginaryI]}, {0.7000000000000001`, \
\(-0.16675843610991792`\) + 
          9.969288068284856`*^-56\ \[ImaginaryI]}, {0.71`, \
\(-0.1493199697035098`\) + 
          1.0969033600563856`*^-55\ \[ImaginaryI]}, {0.72`, \
\(-0.13238980782399712`\) + 
          5.488664449874044`*^-56\ \[ImaginaryI]}, {0.73`, \
\(-0.11594824804879689`\) + 
          2.4608585724257566`*^-55\ \[ImaginaryI]}, {0.74`, \
\(-0.09997640470649438`\) + 
          5.582552886282969`*^-56\ \[ImaginaryI]}, {0.75`, \
\(-0.08445617598258548`\) + 
          1.014476895870186`*^-55\ \[ImaginaryI]}, {0.76`, \
\(-0.0693702118982985`\) + 
          4.20178224860989`*^-56\ \[ImaginaryI]}, {0.77`, \
\(-0.054701883229510084`\) + 
          4.34904572986958`*^-56\ \[ImaginaryI]}, {0.78`, \
\(-0.040435251414944084`\) + 
          5.7505068933251805`*^-56\ \[ImaginaryI]}, {0.79`, \
\(-0.02655503948795368`\) + 
          2.5798737032694973`*^-25\ \[ImaginaryI]}, {0.8`, \
\(-0.013046604053664312`\) + 
          8.915650937641154`*^-56\ \[ImaginaryI]}, {0.81`, \
\(\(0.00010409167707547962`\)\(\[InvisibleSpace]\)\) + 
          4.240477519932719`*^-56\ \[ImaginaryI]}, {0.8200000000000001`, \
\(\(0.012910503794054118`\)\(\[InvisibleSpace]\)\) + 
          4.569496878419352`*^-26\ \[ImaginaryI]}, {0.8300000000000001`, \
\(\(0.025385532538446443`\)\(\[InvisibleSpace]\)\) + 
          4.199420985675011`*^-56\ \[ImaginaryI]}, {0.84`, \
\(\(0.03754154610767624`\)\(\[InvisibleSpace]\)\) + 
          3.534980767394969`*^-56\ \[ImaginaryI]}, {0.85`, \
\(\(0.049390403428545504`\)\(\[InvisibleSpace]\)\) + 
          3.864363976949652`*^-26\ \[ImaginaryI]}, {0.86`, \
\(\(0.06094347591630844`\)\(\[InvisibleSpace]\)\) + 
          6.620533324633005`*^-26\ \[ImaginaryI]}, {0.87`, \
\(\(0.07221166823994507`\)\(\[InvisibleSpace]\)\) + 
          2.8360686592008684`*^-26\ \[ImaginaryI]}, {0.88`, \
\(\(0.08320543811581338`\)\(\[InvisibleSpace]\)\) + 
          6.606435834096958`*^-56\ \[ImaginaryI]}, {0.89`, \
\(\(0.09393481515310675`\)\(\[InvisibleSpace]\)\) + 
          2.5154779469747596`*^-26\ \[ImaginaryI]}, {0.9`, \
\(\(0.10440941877528431`\)\(\[InvisibleSpace]\)\) + 
          2.140878129399729`*^-26\ \[ImaginaryI]}, {0.91`, \
\(\(0.11463847524191384`\)\(\[InvisibleSpace]\)\) + 
          3.542050896547958`*^-26\ \[ImaginaryI]}, {0.92`, \
\(\(0.12463083379523199`\)\(\[InvisibleSpace]\)\) + 
          4.4934831782493446`*^-26\ \[ImaginaryI]}, {0.93`, \
\(\(0.13439498195526833`\)\(\[InvisibleSpace]\)\) + 
          1.954031421982999`*^-26\ \[ImaginaryI]}, {0.9400000000000001`, \
\(\(0.14393905998660772`\)\(\[InvisibleSpace]\)\) + 
          1.7341314752960714`*^-26\ \[ImaginaryI]}, {0.9500000000000001`, \
\(\(0.1532708745588508`\)\(\[InvisibleSpace]\)\) + 
          2.2172857693875378`*^-26\ \[ImaginaryI]}, {0.96`, \
\(\(0.16239791162155814`\)\(\[InvisibleSpace]\)\) + 
          1.458804499401138`*^-26\ \[ImaginaryI]}, {0.97`, \
\(\(0.17132734851302833`\)\(\[InvisibleSpace]\)\) + 
          1.6103883168058981`*^-56\ \[ImaginaryI]}, {0.98`, \
\(\(0.18006606532059088`\)\(\[InvisibleSpace]\)\) + 
          2.5382504265277918`*^-26\ \[ImaginaryI]}, {0.99`, \
\(\(0.18862065550828266`\)\(\[InvisibleSpace]\)\) + 
          3.611524154781505`*^-26\ \[ImaginaryI]}, {1.`, \
\(\(0.19699743582579815`\)\(\[InvisibleSpace]\)\) + 
          2.1708618471091466`*^-26\ \[ImaginaryI]}, {1.01`, \
\(\(0.20520245551044675`\)\(\[InvisibleSpace]\)\) + 
          9.662044404709253`*^-27\ \[ImaginaryI]}, {1.02`, \
\(\(0.21324150479156934`\)\(\[InvisibleSpace]\)\) + 
          3.239167577839719`*^-26\ \[ImaginaryI]}, {1.03`, \
\(\(0.2211201227044035`\)\(\[InvisibleSpace]\)\) + 
          7.922626413998436`*^-27\ \[ImaginaryI]}, {1.04`, \
\(\(0.22884360421778316`\)\(\[InvisibleSpace]\)\) + 
          2.0147002922043648`*^-56\ \[ImaginaryI]}, {1.05`, \
\(\(0.2364170066772831`\)\(\[InvisibleSpace]\)\) + 
          1.1287592004736276`*^-56\ \[ImaginaryI]}, {1.06`, \
\(\(0.24384515556248276`\)\(\[InvisibleSpace]\)\) + 
          1.2669928182729704`*^-26\ \[ImaginaryI]}, {1.07`, \
\(\(0.2511326495539016`\)\(\[InvisibleSpace]\)\) + 
          2.311936788201682`*^-56\ \[ImaginaryI]}, {1.08`, \
\(\(0.258283864901865`\)\(\[InvisibleSpace]\)\) + 
          1.1652313907769293`*^-56\ \[ImaginaryI]}, {1.09`, \
\(\(0.26530295908607365`\)\(\[InvisibleSpace]\)\) + 
          8.033245644047528`*^-57\ \[ImaginaryI]}, {1.1`, \
\(\(0.2721938737509568`\)\(\[InvisibleSpace]\)\) + 
          1.0800766893055122`*^-26\ \[ImaginaryI]}, {1.11`, \
\(\(0.2789603368980274`\)\(\[InvisibleSpace]\)\) + 
          
          5.03142950596083`*^-27\ \[ImaginaryI]}, {1.12`, \
\(\(0.2856058643123712`\)\(\[InvisibleSpace]\)\) + 
          5.366507207216671`*^-27\ \[ImaginaryI]}, {1.1300000000000001`, \
\(\(0.29213376019615894`\)\(\[InvisibleSpace]\)\) + 
          4.730920377337481`*^-27\ \[ImaginaryI]}, {1.1400000000000001`, \
\(\(0.2985471169776526`\)\(\[InvisibleSpace]\)\) + 
          6.775241564670035`*^-57\ \[ImaginaryI]}, {1.1500000000000001`, \
\(\(0.30484881425963234`\)\(\[InvisibleSpace]\)\) + 
          1.1638272421650616`*^-26\ \[ImaginaryI]}, {1.16`, \
\(\(0.3110415168665614`\)\(\[InvisibleSpace]\)\) + 
          3.2175732895993515`*^-27\ \[ImaginaryI]}, {1.17`, \
\(\(0.3171276719451861`\)\(\[InvisibleSpace]\)\) + 
          5.226766191105728`*^-57\ \[ImaginaryI]}, {1.18`, \
\(\(0.3231095050687869`\)\(\[InvisibleSpace]\)\) + 
          6.677735071724821`*^-27\ \[ImaginaryI]}, {1.19`, \
\(\(0.32898901529105934`\)\(\[InvisibleSpace]\)\) + 
          2.6818557906626263`*^-27\ \[ImaginaryI]}, {1.2`, \
\(\(0.3347679690918804`\)\(\[InvisibleSpace]\)\) + 
          4.564998871599414`*^-27\ \[ImaginaryI]}, {1.21`, \
\(\(0.3404478931542107`\)\(\[InvisibleSpace]\)\) + 
          1.781436870122091`*^-27\ \[ImaginaryI]}, {1.22`, \
\(\(0.34603006590956037`\)\(\[InvisibleSpace]\)\) + 
          1.5875313595973`*^-27\ \[ImaginaryI]}, {1.23`, \
\(\(0.35151550778917773`\)\(\[InvisibleSpace]\)\) + 
          2.9217628039256885`*^-27\ \[ImaginaryI]}, {1.24`, \
\(\(0.3569049701201374`\)\(\[InvisibleSpace]\)\) + 
          3.252119684494185`*^-27\ \[ImaginaryI]}, {1.25`, \
\(\(0.3621989226105008`\)\(\[InvisibleSpace]\)\) + 
          1.8528116778534245`*^-27\ \[ImaginaryI]}, {1.26`, \
\(\(0.3673975393767649`\)\(\[InvisibleSpace]\)\) + 
          4.033405729853602`*^-27\ \[ImaginaryI]}, {1.27`, \
\(\(0.372500683481122`\)\(\[InvisibleSpace]\)\) + 
          1.0426880689675438`*^-27\ \[ImaginaryI]}, {1.28`, \
\(\(0.3775078899671604`\)\(\[InvisibleSpace]\)\) + 
          1.4795106521414747`*^-27\ \[ImaginaryI]}, {1.29`, \
\(\(0.3824183474124325`\)\(\[InvisibleSpace]\)\) + 
          
          2.6795926947602795`*^-57\ \[ImaginaryI]}, {1.3`, \
\(\(0.3872308780570367`\)\(\[InvisibleSpace]\)\) + 
          7.088338003914353`*^-28\ \[ImaginaryI]}, {1.31`, \
\(\(0.3919439166216376`\)\(\[InvisibleSpace]\)\) + 
          9.139538543036213`*^-28\ \[ImaginaryI]}, {1.32`, \
\(\(0.39655548799923046`\)\(\[InvisibleSpace]\)\) + 
          2.135379190644072`*^-27\ \[ImaginaryI]}, {1.33`, \
\(\(0.4010631840957829`\)\(\[InvisibleSpace]\)\) + 
          1.2636900588033182`*^-27\ \[ImaginaryI]}, {1.34`, \
\(\(0.40546414020926225`\)\(\[InvisibleSpace]\)\) + 
          6.435695763513958`*^-28\ \[ImaginaryI]}, {1.35`, \
\(\(0.4097550114780587`\)\(\[InvisibleSpace]\)\) + 
          4.985507658399646`*^-28\ \[ImaginaryI]}, {1.36`, \
\(\(0.41393195010157746`\)\(\[InvisibleSpace]\)\) + 
          4.047722152405105`*^-28\ \[ImaginaryI]}, {1.37`, \
\(\(0.4179905842400716`\)\(\[InvisibleSpace]\)\) + 
          7.275427760736814`*^-28\ \[ImaginaryI]}, {1.3800000000000001`, \
\(\(0.42192599973791656`\)\(\[InvisibleSpace]\)\) + 
          2.9951956527555246`*^-28\ \[ImaginaryI]}, {1.3900000000000001`, \
\(\(0.4257327260819761`\)\(\[InvisibleSpace]\)\) + 
          3.112698053689339`*^-28\ \[ImaginaryI]}, {1.4000000000000001`, \
\(\(0.42940472829728615`\)\(\[InvisibleSpace]\)\) + 
          2.526984068502325`*^-28\ \[ImaginaryI]}, {1.41`, \
\(\(0.43293540678277603`\)\(\[InvisibleSpace]\)\) + 
          1.4926996695313782`*^-28\ \[ImaginaryI]}, {1.42`, \
\(\(0.4363176073783329`\)\(\[InvisibleSpace]\)\) + 
          1.5468725083835687`*^-28\ \[ImaginaryI]}, {1.43`, \
\(\(0.4395436441994912`\)\(\[InvisibleSpace]\)\) + 
          2.180841590908278`*^-28\ \[ImaginaryI]}, {1.44`, \
\(\(0.4426053379337953`\)\(\[InvisibleSpace]\)\) + 
          9.372746798069281`*^-29\ \[ImaginaryI]}, {1.45`, \
\(\(0.4454940723082963`\)\(\[InvisibleSpace]\)\) + 
          7.161200028258382`*^-58\ \[ImaginaryI]}, {1.46`, \
\(\(0.44820087124586694`\)\(\[InvisibleSpace]\)\) + 
          5.575129664617186`*^-29\ \[ImaginaryI]}, {1.47`, \
\(\(0.4507164987599236`\)\(\[InvisibleSpace]\)\) + 
          
          7.012405406553729`*^-29\ \[ImaginaryI]}, {1.48`, \
\(\(0.4530315828291762`\)\(\[InvisibleSpace]\)\) + 
          3.074387867124862`*^-29\ \[ImaginaryI]}, {1.49`, \
\(\(0.4551367633022484`\)\(\[InvisibleSpace]\)\) + 
          2.6771671685476616`*^-29\ \[ImaginaryI]}, {1.5`, \
\(\(0.4570228623007928`\)\(\[InvisibleSpace]\)\) + 
          3.7368380251665524`*^-29\ \[ImaginaryI]}, {1.51`, \
\(\(0.4586810736708334`\)\(\[InvisibleSpace]\)\) + 
          1.2503179501427047`*^-29\ \[ImaginaryI]}, {1.52`, \
\(\(0.4601031659013503`\)\(\[InvisibleSpace]\)\) + 
          9.452373043929387`*^-30\ \[ImaginaryI]}, {1.53`, \
\(\(0.4612816907923186`\)\(\[InvisibleSpace]\)\) + 
          5.7770964338476756`*^-30\ \[ImaginaryI]}, {1.54`, \
\(\(0.46221018828670957`\)\(\[InvisibleSpace]\)\) + 
          1.5874571992048143`*^-30\ \[ImaginaryI]}, {1.55`, \
\(\(0.4628833765946772`\)\(\[InvisibleSpace]\)\) + 
          4.854334263582496`*^-31\ \[ImaginaryI]}, {1.56`, \
\(\(0.4632973163291152`\)\(\[InvisibleSpace]\)\) + 
          7.605103856764867`*^-32\ \[ImaginaryI]}, {1.57`, \
\(\(0.46344953805352773`\)\(\[InvisibleSpace]\)\) + 
          3.3557264716148393`*^-35\ \[ImaginaryI]}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{th, 
        N[mm[1, 1, 
            Cos[IntegerPart[th*10^\((16)\)]*10^\((\(-16\))\)], \(-9\)/
              10]]}, {th, 0.01, Pi/2, 0.01}]\)], "Input"],

Cell[BoxData[
    \({{0.01`, \(\(0.5950710418892246`\)\(\[InvisibleSpace]\)\) + 
          1.4808236714393717`*^-78\ \[ImaginaryI]}, {0.02`, \
\(\(0.5928637531762605`\)\(\[InvisibleSpace]\)\) + 
          4.211379750662285`*^-75\ \[ImaginaryI]}, {0.03`, \
\(\(0.5894653826115575`\)\(\[InvisibleSpace]\)\) + 
          4.438797491676149`*^-75\ \[ImaginaryI]}, {0.04`, \
\(\(0.5851584729542938`\)\(\[InvisibleSpace]\)\) + 
          2.2182031010103737`*^-74\ \[ImaginaryI]}, {0.05`, \
\(\(0.5801871412393965`\)\(\[InvisibleSpace]\)\) + 
          7.470731972164404`*^-50\ \[ImaginaryI]}, {0.06`, \
\(\(0.5747353534873488`\)\(\[InvisibleSpace]\)\) + 
          1.433387541991859`*^-49\ \[ImaginaryI]}, {0.07`, \
\(\(0.5689332796538015`\)\(\[InvisibleSpace]\)\) + 
          7.287678350877541`*^-52\ \[ImaginaryI]}, {0.08`, \
\(\(0.5628708631382189`\)\(\[InvisibleSpace]\)\) + 
          5.435945299580921`*^-52\ \[ImaginaryI]}, {0.09`, \
\(\(0.5566100066892117`\)\(\[InvisibleSpace]\)\) + 
          1.017465743110201`*^-51\ \[ImaginaryI]}, {0.1`, \
\(\(0.5501935225812111`\)\(\[InvisibleSpace]\)\) + 
          6.476365396189226`*^-52\ \[ImaginaryI]}, {0.11`, \
\(\(0.5436512447820989`\)\(\[InvisibleSpace]\)\) + 
          8.331599228803843`*^-52\ \[ImaginaryI]}, {0.12`, \
\(\(0.537004101518724`\)\(\[InvisibleSpace]\)\) + 
          8.88161905331646`*^-52\ \[ImaginaryI]}, {0.13`, \
\(\(0.530266821397456`\)\(\[InvisibleSpace]\)\) + 
          2.2097946585312003`*^-51\ \[ImaginaryI]}, {0.14`, \
\(\(0.5234497452068247`\)\(\[InvisibleSpace]\)\) + 
          8.960044817292995`*^-52\ \[ImaginaryI]}, {0.15`, \
\(\(0.51656005426808`\)\(\[InvisibleSpace]\)\) + 
          8.525000641866882`*^-52\ \[ImaginaryI]}, {0.16`, \
\(\(0.5096026154518953`\)\(\[InvisibleSpace]\)\) + 
          1.2405028743202606`*^-51\ \[ImaginaryI]}, {0.17`, \
\(\(0.5025805713564141`\)\(\[InvisibleSpace]\)\) + 
          2.1573901005476408`*^-51\ \[ImaginaryI]}, {0.18`, \
\(\(0.49549575869770035`\)\(\[InvisibleSpace]\)\) + 
          2.1101667232551255`*^-51\ \[ImaginaryI]}, {0.19`, \
\(\(0.48834900914965995`\)\(\[InvisibleSpace]\)\) + 
          
          1.1822887405721586`*^-51\ \[ImaginaryI]}, {0.2`, \
\(\(0.48114036855752096`\)\(\[InvisibleSpace]\)\) + 
          2.0285044377680545`*^-51\ \[ImaginaryI]}, {0.21`, \
\(\(0.47386925861810225`\)\(\[InvisibleSpace]\)\) + 
          2.021270153631394`*^-51\ \[ImaginaryI]}, {0.22`, \
\(\(0.46653459744565123`\)\(\[InvisibleSpace]\)\) + 
          1.4239974955434783`*^-51\ \[ImaginaryI]}, {0.23`, \
\(\(0.4591348903568897`\)\(\[InvisibleSpace]\)\) + 
          6.860939183031375`*^-56\ \[ImaginaryI]}, {0.24`, \
\(\(0.4516682988074837`\)\(\[InvisibleSpace]\)\) + 
          4.942923889112959`*^-56\ \[ImaginaryI]}, {0.25`, \
\(\(0.44413269310148934`\)\(\[InvisibleSpace]\)\) + 
          5.832128370627155`*^-56\ \[ImaginaryI]}, {0.26`, \
\(\(0.4365256929095285`\)\(\[InvisibleSpace]\)\) + 
          6.590496424900337`*^-56\ \[ImaginaryI]}, {0.27`, \
\(\(0.42884469852651363`\)\(\[InvisibleSpace]\)\) + 
          1.095666621129865`*^-55\ \[ImaginaryI]}, {0.28`, \
\(\(0.42108691501585205`\)\(\[InvisibleSpace]\)\) + 
          7.381952444260851`*^-56\ \[ImaginaryI]}, {0.29`, \
\(\(0.41324937083575985`\)\(\[InvisibleSpace]\)\) + 
          7.803083792342158`*^-56\ \[ImaginaryI]}, {0.3`, \
\(\(0.4053289321414394`\)\(\[InvisibleSpace]\)\) + 
          8.444572600673662`*^-56\ \[ImaginaryI]}, {0.31`, \
\(\(0.39732231366062315`\)\(\[InvisibleSpace]\)\) + 
          6.096206287308498`*^-56\ \[ImaginaryI]}, {0.32`, \
\(\(0.3892260868319245`\)\(\[InvisibleSpace]\)\) + 
          7.20486172487464`*^-56\ \[ImaginaryI]}, {0.33`, \
\(\(0.38103668572905686`\)\(\[InvisibleSpace]\)\) + 
          1.900882554052977`*^-55\ \[ImaginaryI]}, {0.34`, \
\(\(0.37275041117921015`\)\(\[InvisibleSpace]\)\) + 
          6.622743829669213`*^-56\ \[ImaginaryI]}, {0.35000000000000003`, \
\(\(0.36436343338908317`\)\(\[InvisibleSpace]\)\) + 
          5.936186974072431`*^-56\ \[ImaginaryI]}, {0.36`, \
\(\(0.35587179332779`\)\(\[InvisibleSpace]\)\) + 
          1.2591390007780676`*^-55\ \[ImaginaryI]}, {0.37`, \
\(\(0.3472714030578089`\)\(\[InvisibleSpace]\)\) + 
          1.2344307441914254`*^-55\ \[ImaginaryI]}, {0.38`, \
\(\(0.3385580451681417`\)\(\[InvisibleSpace]\)\) + 
          7.137910858972615`*^-56\ \[ImaginaryI]}, {0.39`, \
\(\(0.329727371428854`\)\(\[InvisibleSpace]\)\) + 
          1.1795711795470067`*^-55\ \[ImaginaryI]}, {0.4`, \
\(\(0.32077490076357423`\)\(\[InvisibleSpace]\)\) + 
          6.575656646542248`*^-56\ \[ImaginaryI]}, {0.41000000000000003`, \
\(\(0.3116960166138911`\)\(\[InvisibleSpace]\)\) + 
          7.373973774348745`*^-56\ \[ImaginaryI]}, {0.42`, \
\(\(0.3024859637562953`\)\(\[InvisibleSpace]\)\) + 
          1.4537697978701744`*^-55\ \[ImaginaryI]}, {0.43`, \
\(\(0.29313984461754533`\)\(\[InvisibleSpace]\)\) + 
          1.1416540946972504`*^-55\ \[ImaginaryI]}, {0.44`, \
\(\(0.2836526151254382`\)\(\[InvisibleSpace]\)\) + 
          1.1484352419111932`*^-55\ \[ImaginaryI]}, {0.45`, \
\(\(0.2740190801227171`\)\(\[InvisibleSpace]\)\) + 
          1.0855244433831236`*^-55\ \[ImaginaryI]}, {0.46`, \
\(\(0.2642338883654615`\)\(\[InvisibleSpace]\)\) + 
          2.4288175359364554`*^-55\ \[ImaginaryI]}, {0.47000000000000003`, \(\
\(0.2542915271214262`\)\(\[InvisibleSpace]\)\) + 
          2.0164320968520674`*^-55\ \[ImaginaryI]}, {0.48`, \
\(\(0.24418631637938998`\)\(\[InvisibleSpace]\)\) + 
          7.869185606861768`*^-56\ \[ImaginaryI]}, {0.49`, \
\(\(0.23391240267610958`\)\(\[InvisibleSpace]\)\) + 
          2.0890304731698518`*^-55\ \[ImaginaryI]}, {0.5`, \
\(\(0.22346375254480938`\)\(\[InvisibleSpace]\)\) + 
          1.3644244624312082`*^-55\ \[ImaginaryI]}, {0.51`, \
\(\(0.21283414558593355`\)\(\[InvisibleSpace]\)\) + 
          8.414887111159023`*^-56\ \[ImaginaryI]}, {0.52`, \
\(\(0.2020171671584125`\)\(\[InvisibleSpace]\)\) + 
          1.1527676620727342`*^-55\ \[ImaginaryI]}, {0.53`, \
\(\(0.1910062006880438`\)\(\[InvisibleSpace]\)\) + 
          3.3955389249142524`*^-55\ \[ImaginaryI]}, {0.54`, \
\(\(0.17979441958756992`\)\(\[InvisibleSpace]\)\) + 
          8.354114227525795`*^-56\ \[ImaginaryI]}, {0.55`, \
\(\(0.16837477878132057`\)\(\[InvisibleSpace]\)\) + 
          1.9303327340981007`*^-55\ \[ImaginaryI]}, {0.56`, \
\(\(0.15674000582648878`\)\(\[InvisibleSpace]\)\) + 
          
          1.373235697834521`*^-55\ \[ImaginaryI]}, {0.5700000000000001`, \
\(\(0.14488259162146366`\)\(\[InvisibleSpace]\)\) + 
          8.156407968108026`*^-56\ \[ImaginaryI]}, {0.58`, \
\(\(0.13279478069088413`\)\(\[InvisibleSpace]\)\) + 
          9.901265448551158`*^-56\ \[ImaginaryI]}, {0.59`, \
\(\(0.12046856103645553`\)\(\[InvisibleSpace]\)\) + 
          3.706077446113197`*^-55\ \[ImaginaryI]}, {0.6`, \
\(\(0.10789565354146402`\)\(\[InvisibleSpace]\)\) + 
          1.6526258498763487`*^-55\ \[ImaginaryI]}, {0.61`, \
\(\(0.09506750091677107`\)\(\[InvisibleSpace]\)\) + 
          1.2152366712510301`*^-55\ \[ImaginaryI]}, {0.62`, \
\(\(0.0819752561756224`\)\(\[InvisibleSpace]\)\) + 
          1.0293836832003294`*^-55\ \[ImaginaryI]}, {0.63`, \
\(\(0.0686097706241906`\)\(\[InvisibleSpace]\)\) + 
          8.897305055934002`*^-56\ \[ImaginaryI]}, {0.64`, \
\(\(0.054961581354979194`\)\(\[InvisibleSpace]\)\) + 
          1.5093029912708916`*^-55\ \[ImaginaryI]}, {0.65`, \
\(\(0.04102089823018895`\)\(\[InvisibleSpace]\)\) + 
          7.782514459547506`*^-56\ \[ImaginaryI]}, {0.66`, \
\(\(0.02677759034246574`\)\(\[InvisibleSpace]\)\) + 
          1.1126148024070238`*^-55\ \[ImaginaryI]}, {0.67`, \
\(\(0.012221171941111275`\)\(\[InvisibleSpace]\)\) + 
          1.0422323358294118`*^-55\ \[ImaginaryI]}, {0.68`, \
\(-0.0026592121874138286`\) + 
          1.070978826613976`*^-55\ \[ImaginaryI]}, {0.6900000000000001`, \
\(-0.01787480189482593`\) + 
          8.518198745629264`*^-56\ \[ImaginaryI]}, {0.7000000000000001`, \
\(-0.033437237410800735`\) + 
          1.449382455053462`*^-55\ \[ImaginaryI]}, {0.71`, \
\(-0.04935857565402446`\) + 
          1.7127665503686255`*^-55\ \[ImaginaryI]}, {0.72`, \
\(-0.0656513071191489`\) + 
          9.203158369921705`*^-56\ \[ImaginaryI]}, {0.73`, \
\(-0.08232837334145149`\) + 
          4.430302355293033`*^-55\ \[ImaginaryI]}, {0.74`, \
\(-0.09940318493783828`\) + 
          1.0789436339694521`*^-55\ \[ImaginaryI]}, {0.75`, \
\(-0.11688964021868231`\) + 
          2.1046339116686476`*^-55\ \[ImaginaryI]}, {0.76`, \
\(-0.13480214436046734`\) + 
          
          9.356044726186035`*^-56\ \[ImaginaryI]}, {0.77`, \
\(-0.15315562912351188`\) + 
          1.0392964367758994`*^-55\ \[ImaginaryI]}, {0.78`, \
\(-0.17196557309259275`\) + 
          1.4747118072954063`*^-55\ \[ImaginaryI]}, {0.79`, \
\(-0.19124802241051264`\) + 
          7.099541669068118`*^-25\ \[ImaginaryI]}, {0.8`, \
\(-0.2110196119658968`\) + 
          2.6326758932947826`*^-55\ \[ImaginaryI]}, {0.81`, \
\(-0.2312975869858771`\) + 
          1.3435675766039304`*^-55\ \[ImaginaryI]}, {0.8200000000000001`, \
\(-0.2520998249724422`\) + 
          1.5534872935038866`*^-25\ \[ImaginaryI]}, {0.8300000000000001`, \
\(-0.2734448579071166`\) + 
          1.5318759977985075`*^-55\ \[ImaginaryI]}, {0.84`, \
\(-0.2953518946326431`\) + 
          1.3836380320088577`*^-55\ \[ImaginaryI]}, {0.85`, \
\(-0.31784084330158624`\) + 
          1.6230317608878358`*^-25\ \[ImaginaryI]}, {0.86`, \
\(-0.34093233376054144`\) + 
          2.983827403401659`*^-25\ \[ImaginaryI]}, {0.87`, \
\(-0.3646477397140395`\) + 
          1.371680917605953`*^-25\ \[ImaginaryI]}, {0.88`, \
\(-0.38900920048407905`\) + 
          3.4291677038357524`*^-55\ \[ImaginaryI]}, {0.89`, \
\(-0.4140396421490329`\) + 
          1.4014001084848973`*^-25\ \[ImaginaryI]}, {0.9`, \
\(-0.43976279780895733`\) + 
          1.2802477064082055`*^-25\ \[ImaginaryI]}, {0.91`, \
\(-0.46620322668233205`\) + 
          2.2738548536499485`*^-25\ \[ImaginaryI]}, {0.92`, \
\(-0.4933863316914582`\) + 
          3.0970439442937984`*^-25\ \[ImaginaryI]}, {0.93`, \
\(-0.5213383751395189`\) + 
          1.4461298773962564`*^-25\ \[ImaginaryI]}, {0.9400000000000001`, \
\(-0.5500864920205069`\) + 
          1.3782533325045322`*^-25\ \[ImaginaryI]}, {0.9500000000000001`, \
\(-0.5796587004334295`\) + 
          1.8927971419140149`*^-25\ \[ImaginaryI]}, {0.96`, \
\(-0.6100839084930723`\) + 
          1.3377748933951543`*^-25\ \[ImaginaryI]}, {0.97`, \
\(-0.6413919170402971`\) + 
          1.586689968305412`*^-55\ \[ImaginaryI]}, {0.98`, \
\(-0.6736134173542536`\) + 
          2.687490390898897`*^-25\ \[ImaginaryI]}, {0.99`, \
\(-0.7067799829556226`\) + 
          4.10991272600844`*^-25\ \[ImaginaryI]}, {1.`, \
\(-0.7409240544631257`\) + 
          2.6557386112907944`*^-25\ \[ImaginaryI]}, {1.01`, \
\(-0.7760789163233315`\) + 
          1.2709167153602051`*^-25\ \[ImaginaryI]}, {1.02`, \
\(-0.8122786640752828`\) + 
          4.582075471241493`*^-25\ \[ImaginaryI]}, {1.03`, \
\(-0.8495581606350857`\) + 
          1.2054969366631873`*^-25\ \[ImaginaryI]}, {1.04`, \
\(-0.8879529798900265`\) + 
          3.298100246346719`*^-55\ \[ImaginaryI]}, {1.05`, \
\(-0.9274993356761169`\) + 
          1.9883827079423663`*^-55\ \[ImaginaryI]}, {1.06`, \
\(-0.968233993975691`\) + 
          2.402186602908549`*^-25\ \[ImaginaryI]}, {1.07`, \
\(-1.010194165912636`\) + 
          4.718785897469054`*^-55\ \[ImaginaryI]}, {1.08`, \
\(-1.0534173788412393`\) + 
          2.560793114811217`*^-55\ \[ImaginaryI]}, {1.09`, \
\(-1.0979413225211103`\) + 
          1.901276604031216`*^-55\ \[ImaginaryI]}, {1.1`, \
\(-1.143803667046074`\) + 
          2.7534775573304357`*^-25\ \[ImaginaryI]}, {1.11`, \
\(-1.1910418488515435`\) + 
          1.3818715213324338`*^-25\ \[ImaginaryI]}, {1.12`, \
\(-1.2396928207662463`\) + 
          1.5881401587392336`*^-25\ \[ImaginaryI]}, {1.1300000000000001`, \
\(-1.2897927617055207`\) + 
          1.508790269688291`*^-25\ \[ImaginaryI]}, {1.1400000000000001`, \
\(-1.3413767412323825`\) + 
          2.328906347352905`*^-55\ \[ImaginaryI]}, {1.1500000000000001`, \
\(-1.3944783338493738`\) + 
          4.312299832985769`*^-25\ \[ImaginaryI]}, {1.16`, \
\(-1.449129177542979`\) + 
          1.2852309408963475`*^-25\ \[ImaginaryI]}, {1.17`, \
\(-1.505358470800608`\) + 
          2.2508451113258565`*^-55\ \[ImaginaryI]}, {1.18`, \
\(-1.563192402081397`\) + 
          3.1003866404330308`*^-25\ \[ImaginaryI]}, {1.19`, \
\(-1.622653505574707`\) + 
          1.3424422498029168`*^-25\ \[ImaginaryI]}, {1.2`, \
\(-1.6837599370603957`\) + 
          2.4635249758911694`*^-25\ \[ImaginaryI]}, {1.21`, \
\(-1.7465246638359737`\) + 
          1.0363504143956598`*^-25\ \[ImaginaryI]}, {1.22`, \
\(-1.810954563050012`\) + 
          9.954541451453964`*^-26\ \[ImaginaryI]}, {1.23`, \
\(-1.8770494234401633`\) + 
          1.9743599510111604`*^-25\ \[ImaginaryI]}, {1.24`, \
\(-1.9448008464891966`\) + 
          2.367687638835428`*^-25\ \[ImaginaryI]}, {1.25`, \
\(-2.01419104446445`\) + 
          1.4528862779076107`*^-25\ \[ImaginaryI]}, {1.26`, \
\(-2.0851915347846104`\) + 
          3.4052566948668044`*^-25\ \[ImaginaryI]}, {1.27`, \
\(-2.157761732758777`\) + 
          9.473522761336659`*^-26\ \[ImaginaryI]}, {1.28`, \
\(-2.2318474480658814`\) + 
          1.4458372360968199`*^-25\ \[ImaginaryI]}, {1.29`, \
\(-2.3073792944844573`\) + 
          2.8147502220242046`*^-55\ \[ImaginaryI]}, {1.3`, \
\(-2.3842710274308083`\) + 
          7.997757785895579`*^-26\ \[ImaginaryI]}, {1.31`, \
\(-2.462417829884058`\) + 
          1.106720954533432`*^-25\ \[ImaginaryI]}, {1.32`, \
\(-2.5416945743017294`\) + 
          2.7724731295551675`*^-25\ \[ImaginaryI]}, {1.33`, \
\(-2.6219540961401426`\) + 
          1.7573078892101516`*^-25\ \[ImaginaryI]}, {1.34`, \
\(-2.7030255234991913`\) + 
          9.574127423676508`*^-26\ \[ImaginaryI]}, {1.35`, \
\(-2.7847127170257586`\) + 
          7.923790904643048`*^-26\ \[ImaginaryI]}, {1.36`, \
\(-2.8667928842298482`\) + 
          6.863048365674471`*^-26\ \[ImaginaryI]}, {1.37`, \
\(-2.9490154423472004`\) + 
          1.3138545597317273`*^-25\ \[ImaginaryI]}, {1.3800000000000001`, \
\(-3.0311012132149204`\) + 
          5.75083171570529`*^-26\ \[ImaginaryI]}, {1.3900000000000001`, \
\(-3.1127420415361464`\) + 
          6.342001996175887`*^-26\ \[ImaginaryI]}, {1.4000000000000001`, \
\(-3.1936009334524087`\) + 
          5.452236118361963`*^-26\ \[ImaginaryI]}, {1.41`, \
\(-3.273312814433961`\) + 
          3.402962377075474`*^-26\ \[ImaginaryI]}, {1.42`, \
\(-3.3514860029642795`\) + 
          3.717170202533547`*^-26\ \[ImaginaryI]}, {1.43`, \
\(-3.4277044881545446`\) + 
          5.50994692463005`*^-26\ \[ImaginaryI]}, {1.44`, \
\(-3.501531084207846`\) + 
          2.4830220208628196`*^-26\ \[ImaginaryI]}, {1.45`, \
\(-3.5725115117526736`\) + 
          1.9835808748978722`*^-55\ \[ImaginaryI]}, {1.46`, \
\(-3.6401794251063055`\) + 
          1.609783352816441`*^-26\ \[ImaginaryI]}, {1.47`, \
\(-3.704062365750329`\) + 
          2.1041043536752177`*^-26\ \[ImaginaryI]}, {1.48`, \
\(-3.763688576716775`\) + 
          9.555027677863305`*^-27\ \[ImaginaryI]}, {1.49`, \
\(-3.8185945620966706`\) + 
          8.589297571349305`*^-27\ \[ImaginaryI]}, {1.5`, \
\(-3.8683332233111667`\) + 
          1.2333517898450897`*^-26\ \[ImaginaryI]}, {1.51`, \
\(-3.9124823527762214`\) + 
          4.2301591705251056`*^-27\ \[ImaginaryI]}, {1.52`, \
\(-3.9506532203973963`\) + 
          3.266258932010012`*^-27\ \[ImaginaryI]}, {1.53`, \
\(-3.982498953446669`\) + 
          2.031353025307902`*^-27\ \[ImaginaryI]}, {1.54`, \
\(-4.007722390057932`\) + 
          5.658669648754873`*^-28\ \[ImaginaryI]}, {1.55`, \
\(-4.026083084314326`\) + 
          1.7475604777183565`*^-28\ \[ImaginaryI]}, {1.56`, \
\(-4.037403158861892`\) + 
          2.7544807393676006`*^-29\ \[ImaginaryI]}, {1.57`, \
\(-4.041571739604743`\) + 
          1.2181138727187403`*^-32\ \[ImaginaryI]}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<mminus.dat\>", Re[%28]]\)], "Input"],

Cell[BoxData[
    \("mminus.dat"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%134, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606612 0.533532 0.150582 [
[.17546 .52103 -12 -9 ]
[.17546 .52103 12 0 ]
[.32712 .52103 -9 -9 ]
[.32712 .52103 9 0 ]
[.47877 .52103 -12 -9 ]
[.47877 .52103 12 0 ]
[.63042 .52103 -3 -9 ]
[.63042 .52103 3 0 ]
[.78207 .52103 -12 -9 ]
[.78207 .52103 12 0 ]
[.93373 .52103 -9 -9 ]
[.93373 .52103 9 0 ]
[.01131 .08178 -12 -4.5 ]
[.01131 .08178 0 4.5 ]
[.01131 .23237 -12 -4.5 ]
[.01131 .23237 0 4.5 ]
[.01131 .38295 -12 -4.5 ]
[.01131 .38295 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17546 .53353 m
.17546 .53978 L
s
[(0.25)] .17546 .52103 0 1 Mshowa
.32712 .53353 m
.32712 .53978 L
s
[(0.5)] .32712 .52103 0 1 Mshowa
.47877 .53353 m
.47877 .53978 L
s
[(0.75)] .47877 .52103 0 1 Mshowa
.63042 .53353 m
.63042 .53978 L
s
[(1)] .63042 .52103 0 1 Mshowa
.78207 .53353 m
.78207 .53978 L
s
[(1.25)] .78207 .52103 0 1 Mshowa
.93373 .53353 m
.93373 .53978 L
s
[(1.5)] .93373 .52103 0 1 Mshowa
.125 Mabswid
.05414 .53353 m
.05414 .53728 L
s
.08447 .53353 m
.08447 .53728 L
s
.1148 .53353 m
.1148 .53728 L
s
.14513 .53353 m
.14513 .53728 L
s
.20579 .53353 m
.20579 .53728 L
s
.23612 .53353 m
.23612 .53728 L
s
.26645 .53353 m
.26645 .53728 L
s
.29678 .53353 m
.29678 .53728 L
s
.35745 .53353 m
.35745 .53728 L
s
.38778 .53353 m
.38778 .53728 L
s
.41811 .53353 m
.41811 .53728 L
s
.44844 .53353 m
.44844 .53728 L
s
.5091 .53353 m
.5091 .53728 L
s
.53943 .53353 m
.53943 .53728 L
s
.56976 .53353 m
.56976 .53728 L
s
.60009 .53353 m
.60009 .53728 L
s
.66075 .53353 m
.66075 .53728 L
s
.69108 .53353 m
.69108 .53728 L
s
.72141 .53353 m
.72141 .53728 L
s
.75174 .53353 m
.75174 .53728 L
s
.81241 .53353 m
.81241 .53728 L
s
.84274 .53353 m
.84274 .53728 L
s
.87307 .53353 m
.87307 .53728 L
s
.9034 .53353 m
.9034 .53728 L
s
.96406 .53353 m
.96406 .53728 L
s
.99439 .53353 m
.99439 .53728 L
s
.25 Mabswid
0 .53353 m
1 .53353 L
s
.02381 .08178 m
.03006 .08178 L
s
[(-3)] .01131 .08178 1 0 Mshowa
.02381 .23237 m
.03006 .23237 L
s
[(-2)] .01131 .23237 1 0 Mshowa
.02381 .38295 m
.03006 .38295 L
s
[(-1)] .01131 .38295 1 0 Mshowa
.125 Mabswid
.02381 .1119 m
.02756 .1119 L
s
.02381 .14202 m
.02756 .14202 L
s
.02381 .17213 m
.02756 .17213 L
s
.02381 .20225 m
.02756 .20225 L
s
.02381 .26248 m
.02756 .26248 L
s
.02381 .2926 m
.02756 .2926 L
s
.02381 .32272 m
.02756 .32272 L
s
.02381 .35283 m
.02756 .35283 L
s
.02381 .41307 m
.02756 .41307 L
s
.02381 .44318 m
.02756 .44318 L
s
.02381 .4733 m
.02756 .4733 L
s
.02381 .50342 m
.02756 .50342 L
s
.02381 .05167 m
.02756 .05167 L
s
.02381 .02155 m
.02756 .02155 L
s
.02381 .56365 m
.02756 .56365 L
s
.02381 .59376 m
.02756 .59376 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02988 .01472 m
.03594 .01641 L
.04201 .01921 L
.04807 .02308 L
.05414 .02799 L
.06021 .03388 L
.06627 .04068 L
.07234 .04833 L
.0784 .05674 L
.08447 .06585 L
.09054 .07558 L
.0966 .08583 L
.10267 .09652 L
.10874 .10759 L
.1148 .11896 L
.12087 .13054 L
.12693 .14228 L
.133 .15411 L
.13907 .16598 L
.14513 .17784 L
.1512 .18963 L
.15726 .20133 L
.16333 .21289 L
.1694 .22428 L
.17546 .23548 L
.18153 .24647 L
.18759 .25723 L
.19366 .26774 L
.19973 .27799 L
.20579 .28799 L
.21186 .29771 L
.21793 .30716 L
.22399 .31633 L
.23006 .32523 L
.23612 .33386 L
.24219 .34222 L
.24826 .35031 L
.25432 .35814 L
.26039 .36571 L
.26645 .37303 L
.27252 .38011 L
.27859 .38695 L
.28465 .39356 L
.29072 .39995 L
.29678 .40612 L
.30285 .41208 L
.30892 .41783 L
.31498 .42339 L
.32105 .42876 L
.32712 .43395 L
Mistroke
.33318 .43897 L
.33925 .44381 L
.34531 .44849 L
.35138 .45302 L
.35745 .45739 L
.36351 .46162 L
.36958 .46571 L
.37564 .46967 L
.38171 .4735 L
.38778 .4772 L
.39384 .48079 L
.39991 .48426 L
.40598 .48762 L
.41204 .49087 L
.41811 .49403 L
.42417 .49709 L
.43024 .50005 L
.43631 .50293 L
.44237 .50572 L
.44844 .50842 L
.4545 .51105 L
.46057 .5136 L
.46664 .51607 L
.4727 .51848 L
.47877 .52081 L
.48483 .52309 L
.4909 .52529 L
.49697 .52744 L
.50303 .52953 L
.5091 .53157 L
.51517 .53355 L
.52123 .53548 L
.5273 .53735 L
.53336 .53918 L
.53943 .54097 L
.5455 .54271 L
.55156 .54441 L
.55763 .54606 L
.56369 .54768 L
.56976 .54925 L
.57583 .55079 L
.58189 .5523 L
.58796 .55377 L
.59402 .55521 L
.60009 .55661 L
.60616 .55799 L
.61222 .55933 L
.61829 .56065 L
.62436 .56193 L
.63042 .5632 L
Mistroke
.63649 .56443 L
.64255 .56564 L
.64862 .56683 L
.65469 .56799 L
.66075 .56913 L
.66682 .57025 L
.67288 .57135 L
.67895 .57242 L
.68502 .57348 L
.69108 .57452 L
.69715 .57554 L
.70322 .57654 L
.70928 .57752 L
.71535 .57849 L
.72141 .57944 L
.72748 .58037 L
.73355 .58129 L
.73961 .58219 L
.74568 .58307 L
.75174 .58394 L
.75781 .5848 L
.76388 .58564 L
.76994 .58646 L
.77601 .58728 L
.78207 .58807 L
.78814 .58886 L
.79421 .58962 L
.80027 .59038 L
.80634 .59112 L
.81241 .59184 L
.81847 .59255 L
.82454 .59325 L
.8306 .59392 L
.83667 .59459 L
.84274 .59523 L
.8488 .59586 L
.85487 .59647 L
.86093 .59707 L
.867 .59764 L
.87307 .59819 L
.87913 .59872 L
.8852 .59923 L
.89126 .59972 L
.89733 .60018 L
.9034 .60062 L
.90946 .60102 L
.91553 .6014 L
.9216 .60175 L
.92766 .60207 L
.93373 .60235 L
Mistroke
.93979 .6026 L
.94586 .60281 L
.95193 .60299 L
.95799 .60313 L
.96406 .60323 L
.97012 .6033 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000
003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool000000?l0oooo3`3oool000l0
oooo00<000000?ooo`3oool00`00003o0?ooo``0oooo000?0?ooo`800000103oool200000?l0oooo
2P3oool000l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooo`P0oooo000?
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool70?ooo`003`3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo1P3oool000l0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`3o0?ooo`D0oooo000?0?ooo`030000003oool0oooo00P0oooo
0P00003o0?ooo`D0oooo000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3o
ool30?ooo`003`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooo0P3oool0
00l0oooo0P00000;0?ooo`030000003oool0oooo0?l0oooo0P3oool000l0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`3o0?ooo`40oooo000?0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0o`3oool000l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`3n0?ooo`003`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0?d0oooo000?0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool0o03oool000l0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`3l0?ooo`00203oool2000000D0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`3k0?ooo`001`3oool010000000oooo0?ooo`0000040?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0n`3oool000X0oooo00<000000?ooo`3oool00P3o
ool300000140oooo00<000000?ooo`3oool0nP3oool000030?ooo`0000000000008000000`3oool2
000000D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3j0?ooo`002P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0n@3oool000L0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
0?T0oooo00080?ooo`8000001@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0?P0
oooo000?0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0m`3oool000l0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`3g0?ooo`003`3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo0?H0oooo000?0?ooo`8000005P3oool00`000000oooo0?ooo`3f0?oo
o`003`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0?D0oooo000?0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool0m@3oool000l0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`3d0?ooo`003`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo0?@0oooo000?0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0l`3oool000l0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`3c0?ooo`003`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo0?80oooo000?0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0lP3oool000l0oooo0P00000K0?ooo`030000003oool0oooo0?40oooo000?0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0l@3oool000l0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`3`0?ooo`003`3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo0?00oooo000?0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0k`3o
ool000l0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`3_0?ooo`003`3oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo0>h0oooo000?0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool0kP3oool000l0oooo0P00000O0?ooo`030000003oool0oooo0>d0oooo
000?0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0k@3oool000l0oooo00<00000
0?ooo`3oool07`3oool00`000000oooo0?ooo`3/0?ooo`003`3oool00`000000oooo0?ooo`0O0?oo
o`030000003oool0oooo0>`0oooo000?0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3o
ool0j`3oool000l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3[0?ooo`003`3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0>X0oooo000?0?ooo`030000003oool0
oooo0240oooo00<000000?ooo`3oool0jP3oool000l0oooo0P00000S0?ooo`030000003oool0oooo
0>T0oooo000?0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0j03oool000l0oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`3X0?ooo`003`3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo0>L0oooo000?0?ooo`030000003oool0oooo02@0oooo00<00000
0?ooo`3oool0i`3oool000l0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`3V0?oo
o`003`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0>H0oooo00070?ooo`@00000
103oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0>H0oooo00070?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`3U0?ooo`00203oool0
0`000000oooo0?ooo`040?ooo`<000009P3oool00`000000oooo0?ooo`3U0?ooo`0000<0oooo0000
000000000P0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool09`3oool00`00
0000oooo0?ooo`3T0?ooo`002P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0h`3oool000L0oooo00@000000?ooo`3oool00000103oool00`000000
oooo0?ooo`0X0?ooo`030000003oool0oooo0><0oooo00080?ooo`8000001@3oool00`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo0>80oooo000?0?ooo`030000003oool0oooo02X0oooo00<0
00000?ooo`3oool0h@3oool000l0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3Q
0?ooo`003`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0>00oooo000?0?ooo`80
0000;03oool00`000000oooo0?ooo`3P0?ooo`003`3oool00`000000oooo0?ooo`0/0?ooo`030000
003oool0oooo0=l0oooo000?0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0g`3o
ool000l0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`3N0?ooo`003`3oool00`00
0000oooo0?ooo`0^0?ooo`030000003oool0oooo0=d0oooo000?0?ooo`030000003oool0oooo02l0
oooo00<000000?ooo`3oool0g03oool000l0oooo00<000000?ooo`3oool0;`3oool00`000000oooo
0?ooo`3L0?ooo`003`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0=/0oooo000?
0?ooo`800000<@3oool00`000000oooo0?ooo`3K0?ooo`003`3oool00`000000oooo0?ooo`0a0?oo
o`030000003oool0oooo0=X0oooo000?0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3o
ool0fP3oool000l0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`3I0?ooo`003`3o
ool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0=T0oooo000?0?ooo`030000003oool0
oooo03<0oooo00<000000?ooo`3oool0f03oool000l0oooo00<000000?ooo`3oool0=03oool00`00
0000oooo0?ooo`3G0?ooo`003`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo0=H0
oooo000?0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0eP3oool000l0oooo0P00
000g0?ooo`030000003oool0oooo0=D0oooo000?0?ooo`030000003oool0oooo03H0oooo00<00000
0?ooo`3oool0e@3oool000l0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`3D0?oo
o`003`3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo0=<0oooo000?0?ooo`030000
003oool0oooo03T0oooo00<000000?ooo`3oool0dP3oool000l0oooo00<000000?ooo`3oool0>P3o
ool00`000000oooo0?ooo`3A0?ooo`003`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0
oooo0=40oooo000?0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0d03oool000l0
oooo0P00000l0?ooo`030000003oool0oooo0=00oooo000?0?ooo`030000003oool0oooo03`0oooo
00<000000?ooo`3oool0c`3oool000l0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?oo
o`3>0?ooo`003`3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo0<d0oooo000?0?oo
o`030000003oool0oooo03l0oooo00<000000?ooo`3oool0c03oool000l0oooo00<000000?ooo`3o
ool0?`3oool00`000000oooo0?ooo`3<0?ooo`003`3oool00`000000oooo0?ooo`100?ooo`030000
003oool0oooo0</0oooo00070?ooo`@00000103oool00`000000oooo0?ooo`110?ooo`030000003o
ool0oooo0<X0oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0@P3oool0
0`000000oooo0?ooo`390?ooo`002@3oool00`000000oooo0?ooo`030?ooo`<00000@`3oool00`00
0000oooo0?ooo`380?ooo`0000<0oooo0000000000000P0000040?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`370?ooo`002@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0aP3oool000L0oooo0`00
00050?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0a@3oool000T0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0<@0oooo000?0?oo
o`030000003oool0oooo04P0oooo00<000000?ooo`3oool0``3oool000l0oooo00<000000?ooo`3o
ool0B@3oool00`000000oooo0?ooo`320?ooo`003`3oool00`000000oooo0?ooo`1:0?ooo`030000
003oool0oooo0<40oooo000?0?ooo`800000C03oool00`000000oooo0?ooo`300?ooo`003`3oool0
0`000000oooo0?ooo`1<0?ooo`030000003oool0oooo0;l0oooo000?0?ooo`030000003oool0oooo
04d0oooo00<000000?ooo`3oool0_P3oool000l0oooo00<000000?ooo`3oool0CP3oool00`000000
oooo0?ooo`2m0?ooo`003`3oool00`000000oooo0?ooo`1?0?ooo`800000_@3oool000l0oooo00<0
00000?ooo`3oool0D03oool00`000000oooo0?ooo`2k0?ooo`003`3oool00`000000oooo0?ooo`1A
0?ooo`030000003oool0oooo0;X0oooo000?0?ooo`030000003oool0oooo0580oooo0P00002j0?oo
o`003`3oool2000005D0oooo00<000000?ooo`3oool0]`3oool000l0oooo00<000000?ooo`3oool0
E@3oool00`000000oooo0?ooo`2f0?ooo`003`3oool00`000000oooo0?ooo`1F0?ooo`030000003o
ool0oooo0;D0oooo000?0?ooo`030000003oool0oooo05L0oooo0P00002e0?ooo`003`3oool00`00
0000oooo0?ooo`1I0?ooo`800000/`3oool000l0oooo00<000000?ooo`3oool0F`3oool00`000000
oooo0?ooo`2`0?ooo`003`3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo0:l0oooo
000?0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool0[P3oool000l0oooo00<00000
0?ooo`3oool0GP3oool200000:h0oooo000?0?ooo`800000H@3oool200000:`0oooo000?0?ooo`03
0000003oool0oooo0680oooo00<000000?ooo`3oool0Z@3oool000l0oooo00<000000?ooo`3oool0
H`3oool200000:T0oooo000?0?ooo`030000003oool0oooo06D0oooo0P00002W0?ooo`003`3oool0
0`000000oooo0?ooo`1W0?ooo`030000003oool0oooo0:@0oooo000?0?ooo`030000003oool0oooo
06P0oooo1000002R0?ooo`003`3oool00`000000oooo0?ooo`0M0?ooo`800000103oool00`000000
oooo0?ooo`020?ooo`@000000P3oool3000001T0oooo0P0000040?ooo`030000003oool0oooo0080
oooo0`00000D0?ooo`030000003oool0oooo00<0oooo0P0000040?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool00P3oool3000001h0oooo1000000M0?ooo`@000000`3oool00`000000
oooo0?ooo`020?ooo`@000000P3oool3000001P0oooo100000030?ooo`030000003oool0oooo0080
oooo0`00000=0?ooo`003`3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo0000
00/0oooo00<000000?ooo`3oool04P3oool200000080oooo00@000000?ooo`3oool000002@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool07P3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo000?0?ooo`8000007@3oool01000
0000oooo0?ooo`0000090?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool05@3oool0
10000000oooo0?ooo`00000;0?ooo`030000003oool0oooo01@0oooo0`0000020?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`003`3oool0
0`000000oooo0?ooo`0L0?ooo`040000003oool0oooo000000X0oooo00@000000?ooo`3oool0oooo
0`00000H0?ooo`040000003oool0oooo000000P0oooo0`00000I0?ooo`@000002P3oool010000000
oooo0?ooo`3oool300000200oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`090?oo
o`040000003oool0oooo0?ooo`<000006P3oool00`000000oooo0?ooo`070?ooo`<000003@3oool0
00l0oooo00<000000?ooo`3oool0703oool010000000oooo0?ooo`00000;0?ooo`040000003oool0
oooo000001X0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo008000002P3oool010000000oooo0?ooo`00000R0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool02P3oool010000000oooo0?ooo`00000L0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool03@3oool000l0oooo00<000000?ooo`3oool0703oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0603oool0
10000000oooo0?ooo`0000080?ooo`030000003oool0oooo01T0oooo00H000000?ooo`3oool00000
0?ooo`0000060?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool07P3oool30000
01h0oooo0`0000090?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0603oool3
000000T0oooo00<000000?ooo`3oool03@3oool000l0oooo00<000000?ooo`3oool07@3oool20000
00X0oooo0P0000030?ooo`@00000603oool2000000T0oooo1000000I0?ooo`8000000`3oool40000
0080oooo100000020?ooo`@000007`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
00P0oooo0P0000030?ooo`@000006@3oool00`000000oooo0?ooo`070?ooo`@00000303oool000l0
oooo00<000000?ooo`3oool0N`3oool200000940oooo000?0?ooo`030000003oool0oooo07d0oooo
0`00002>0?ooo`003`3oool00`000000oooo0?ooo`200?ooo`<00000R`3oool000P0ooooo`00000I
000000003`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`040000003oool0oooo000000@0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo0@000001
0?ooo`40oooo000?0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00@000000?ooo`3oool0oooo1@000008
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool100000040oooo0@3oool0
00l0oooo00<000000?ooo`3oool0S03oool3000007l0oooo000?0?ooo`030000003oool0oooo08l0
oooo1@00001j0?ooo`003`3oool00`000000oooo0?ooo`2D0?ooo`@00000MP3oool000l0oooo00<0
00000?ooo`3oool0V03oool3000007<0oooo000?0?ooo`030000003oool0oooo09/0oooo1@00001^
0?ooo`003`3oool00`000000oooo0?ooo`2P0?ooo`D00000J@3oool000l0oooo0P00002V0?ooo`D0
0000I03oool000l0oooo00<000000?ooo`3oool0ZP3oool5000005l0oooo000?0?ooo`030000003o
ool0oooo0:l0oooo1@00001J0?ooo`003`3oool00`000000oooo0?ooo`2d0?ooo`L00000D`3oool0
00l0oooo00<000000?ooo`3oool0^`3oool5000004h0oooo000?0?ooo`030000003oool0oooo0<00
oooo200000160?ooo`003`3oool00`000000oooo0?ooo`380?ooo`L00000?`3oool000l0oooo00<0
00000?ooo`3oool0c`3oool9000003H0oooo000?0?ooo`800000f@3oool8000002h0oooo000?0?oo
o`030000003oool0oooo0>00oooo2P00000T0?ooo`003`3oool00`000000oooo0?ooo`3Z0?ooo`h0
00005P3oool000l0oooo00<000000?ooo`3oool0n03oool?000000L0oooo000?0?ooo`030000003o
ool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`00
0000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool00?l0oooo
8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0917353, -3.60326, \
0.00592681, 0.0238758}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%132, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606612 0.0147151 0.072746 [
[.17546 .00222 -12 -9 ]
[.17546 .00222 12 0 ]
[.32712 .00222 -9 -9 ]
[.32712 .00222 9 0 ]
[.47877 .00222 -12 -9 ]
[.47877 .00222 12 0 ]
[.63042 .00222 -3 -9 ]
[.63042 .00222 3 0 ]
[.78207 .00222 -12 -9 ]
[.78207 .00222 12 0 ]
[.93373 .00222 -9 -9 ]
[.93373 .00222 9 0 ]
[.01131 .16021 -6 -4.5 ]
[.01131 .16021 0 4.5 ]
[.01131 .3057 -6 -4.5 ]
[.01131 .3057 0 4.5 ]
[.01131 .45119 -6 -4.5 ]
[.01131 .45119 0 4.5 ]
[.01131 .59668 -6 -4.5 ]
[.01131 .59668 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17546 .01472 m
.17546 .02097 L
s
[(0.25)] .17546 .00222 0 1 Mshowa
.32712 .01472 m
.32712 .02097 L
s
[(0.5)] .32712 .00222 0 1 Mshowa
.47877 .01472 m
.47877 .02097 L
s
[(0.75)] .47877 .00222 0 1 Mshowa
.63042 .01472 m
.63042 .02097 L
s
[(1)] .63042 .00222 0 1 Mshowa
.78207 .01472 m
.78207 .02097 L
s
[(1.25)] .78207 .00222 0 1 Mshowa
.93373 .01472 m
.93373 .02097 L
s
[(1.5)] .93373 .00222 0 1 Mshowa
.125 Mabswid
.05414 .01472 m
.05414 .01847 L
s
.08447 .01472 m
.08447 .01847 L
s
.1148 .01472 m
.1148 .01847 L
s
.14513 .01472 m
.14513 .01847 L
s
.20579 .01472 m
.20579 .01847 L
s
.23612 .01472 m
.23612 .01847 L
s
.26645 .01472 m
.26645 .01847 L
s
.29678 .01472 m
.29678 .01847 L
s
.35745 .01472 m
.35745 .01847 L
s
.38778 .01472 m
.38778 .01847 L
s
.41811 .01472 m
.41811 .01847 L
s
.44844 .01472 m
.44844 .01847 L
s
.5091 .01472 m
.5091 .01847 L
s
.53943 .01472 m
.53943 .01847 L
s
.56976 .01472 m
.56976 .01847 L
s
.60009 .01472 m
.60009 .01847 L
s
.66075 .01472 m
.66075 .01847 L
s
.69108 .01472 m
.69108 .01847 L
s
.72141 .01472 m
.72141 .01847 L
s
.75174 .01472 m
.75174 .01847 L
s
.81241 .01472 m
.81241 .01847 L
s
.84274 .01472 m
.84274 .01847 L
s
.87307 .01472 m
.87307 .01847 L
s
.9034 .01472 m
.9034 .01847 L
s
.96406 .01472 m
.96406 .01847 L
s
.99439 .01472 m
.99439 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .16021 m
.03006 .16021 L
s
[(2)] .01131 .16021 1 0 Mshowa
.02381 .3057 m
.03006 .3057 L
s
[(4)] .01131 .3057 1 0 Mshowa
.02381 .45119 m
.03006 .45119 L
s
[(6)] .01131 .45119 1 0 Mshowa
.02381 .59668 m
.03006 .59668 L
s
[(8)] .01131 .59668 1 0 Mshowa
.125 Mabswid
.02381 .05109 m
.02756 .05109 L
s
.02381 .08746 m
.02756 .08746 L
s
.02381 .12383 m
.02756 .12383 L
s
.02381 .19658 m
.02756 .19658 L
s
.02381 .23295 m
.02756 .23295 L
s
.02381 .26933 m
.02756 .26933 L
s
.02381 .34207 m
.02756 .34207 L
s
.02381 .37844 m
.02756 .37844 L
s
.02381 .41482 m
.02756 .41482 L
s
.02381 .48756 m
.02756 .48756 L
s
.02381 .52394 m
.02756 .52394 L
s
.02381 .56031 m
.02756 .56031 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02988 .60332 m
.03594 .60185 L
.04201 .59943 L
.04807 .59608 L
.05414 .59183 L
.06021 .58673 L
.06627 .58082 L
.07234 .57418 L
.0784 .56685 L
.08447 .55891 L
.09054 .55043 L
.0966 .54146 L
.10267 .53208 L
.10874 .52236 L
.1148 .51237 L
.12087 .50215 L
.12693 .49177 L
.133 .48128 L
.13907 .47074 L
.14513 .46018 L
.1512 .44965 L
.15726 .43918 L
.16333 .42881 L
.1694 .41855 L
.17546 .40845 L
.18153 .39851 L
.18759 .38875 L
.19366 .3792 L
.19973 .36985 L
.20579 .36072 L
.21186 .35181 L
.21793 .34313 L
.22399 .33468 L
.23006 .32646 L
.23612 .31848 L
.24219 .31073 L
.24826 .3032 L
.25432 .29591 L
.26039 .28883 L
.26645 .28198 L
.27252 .27534 L
.27859 .26891 L
.28465 .26269 L
.29072 .25666 L
.29678 .25083 L
.30285 .24519 L
.30892 .23972 L
.31498 .23444 L
.32105 .22933 L
.32712 .22438 L
Mistroke
.33318 .2196 L
.33925 .21496 L
.34531 .21048 L
.35138 .20615 L
.35745 .20195 L
.36351 .19789 L
.36958 .19395 L
.37564 .19014 L
.38171 .18646 L
.38778 .18289 L
.39384 .17943 L
.39991 .17608 L
.40598 .17283 L
.41204 .16968 L
.41811 .16663 L
.42417 .16368 L
.43024 .16081 L
.43631 .15803 L
.44237 .15533 L
.44844 .15272 L
.4545 .15018 L
.46057 .14772 L
.46664 .14532 L
.4727 .143 L
.47877 .14075 L
.48483 .13856 L
.4909 .13643 L
.49697 .13436 L
.50303 .13235 L
.5091 .1304 L
.51517 .1285 L
.52123 .12665 L
.5273 .12486 L
.53336 .12311 L
.53943 .12141 L
.5455 .11976 L
.55156 .11814 L
.55763 .11658 L
.56369 .11505 L
.56976 .11356 L
.57583 .11212 L
.58189 .1107 L
.58796 .10933 L
.59402 .10799 L
.60009 .10668 L
.60616 .10541 L
.61222 .10417 L
.61829 .10296 L
.62436 .10178 L
.63042 .10063 L
Mistroke
.63649 .09951 L
.64255 .09841 L
.64862 .09734 L
.65469 .0963 L
.66075 .09528 L
.66682 .09428 L
.67288 .09331 L
.67895 .09237 L
.68502 .09144 L
.69108 .09054 L
.69715 .08966 L
.70322 .08879 L
.70928 .08795 L
.71535 .08713 L
.72141 .08633 L
.72748 .08555 L
.73355 .08478 L
.73961 .08403 L
.74568 .08331 L
.75174 .08259 L
.75781 .0819 L
.76388 .08122 L
.76994 .08056 L
.77601 .07991 L
.78207 .07929 L
.78814 .07867 L
.79421 .07808 L
.80027 .07749 L
.80634 .07693 L
.81241 .07638 L
.81847 .07584 L
.82454 .07532 L
.8306 .07482 L
.83667 .07433 L
.84274 .07386 L
.8488 .0734 L
.85487 .07296 L
.86093 .07254 L
.867 .07213 L
.87307 .07174 L
.87913 .07137 L
.8852 .07102 L
.89126 .07068 L
.89733 .07037 L
.9034 .07007 L
.90946 .06979 L
.91553 .06954 L
.9216 .06931 L
.92766 .06909 L
.93373 .06891 L
Mistroke
.93979 .06874 L
.94586 .0686 L
.95193 .06848 L
.95799 .06839 L
.96406 .06832 L
.97012 .06828 L
.97619 .06827 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00;P3oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080
oooo0`00000H0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<000006@3oool2000000@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`<000007@3oool4000001`0
oooo100000030?ooo`030000003oool0oooo0080oooo100000020?ooo`<00000603oool4000000<0
oooo00<000000?ooo`3oool00P3oool300000180oooo000]0?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo0000
00/0oooo00<000000?ooo`3oool05@3oool010000000oooo0?ooo`0000090?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool03`3oool002d0oooo00@000000?ooo`3oool000002@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo00@000000?ooo`3oool000002`3o
ool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0?0?ooo`00;@3oool010000000oooo0?ooo`00000:0?ooo`040000
003oool0oooo0?ooo`<000005`3oool010000000oooo0?ooo`0000080?ooo`<00000603oool01000
0000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000007`3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool0oooo0`00000J0?ooo`030000
003oool0oooo00L0oooo0`00000B0?ooo`00;@3oool010000000oooo0?ooo`00000;0?ooo`040000
003oool0oooo000001T0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0H0?oo
o`040000003oool0oooo000000/0oooo00@000000?ooo`3oool000008@3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00X0oooo00@000000?ooo`3oool00000703oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo0180oooo000]0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0G0?ooo`040000003oool0oooo0000
00P0oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool07@3oool3000001d0oooo0`0000090?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool0603oool3000000T0oooo00<000000?ooo`3oool0
4P3oool000l0oooo00<000000?ooo`3oool0703oool2000000X0oooo0P0000030?ooo`@000005`3o
ool2000000T0oooo1000000H0?ooo`8000002@3oool400000080oooo1000000N0?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool0203oool2000000<0oooo1000000I0?ooo`030000003o
ool0oooo00L0oooo1000000A0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?oo
o`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`00
203ooooo000001<000001P3oool000l0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00H0oooo000?0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo01H0oooo000?0?oo
o`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`00
3`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3o
ool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?oo
o`l0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo0P00003o0?oooa00oooo
000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?
0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000003oool0oooo0?l0
oooo3`3oool000l0oooo00<000000?ooo`3oool0i@3ooolL000000d0oooo000?0?ooo`030000003o
ool0oooo0=H0oooo3`00000Y0?ooo`003`3oool00`000000oooo0?ooo`3<0?ooo`X00000>03oool0
00l0oooo00<000000?ooo`3oool0`P3oool:00000480oooo000?0?ooo`030000003oool0oooo0;X0
oooo2000001<0?ooo`003`3oool200000;D0oooo1P00001D0?ooo`003`3oool00`000000oooo0?oo
o`2]0?ooo`L00000FP3oool000l0oooo00<000000?ooo`3oool0Z03oool500000640oooo000?0?oo
o`030000003oool0oooo0:40oooo1`00001V0?ooo`003`3oool00`000000oooo0?ooo`2L0?ooo`D0
0000K@3oool000l0oooo00<000000?ooo`3oool0U`3oool500000780oooo000?0?ooo`030000003o
ool0oooo09@0oooo0`00001g0?ooo`003`3oool00`000000oooo0?ooo`2?0?ooo`D00000NP3oool0
00l0oooo00<000000?ooo`3oool0S03oool3000007l0oooo000?0?ooo`030000003oool0oooo08T0
oooo0`0000220?ooo`003`3oool2000008D0oooo1@0000250?ooo`003`3oool00`000000oooo0?oo
o`220?ooo`800000RP3oool000l0oooo00<000000?ooo`3oool0O`3oool3000008`0oooo000?0?oo
o`030000003oool0oooo07/0oooo1000002?0?ooo`003`3oool00`000000oooo0?ooo`1h0?ooo`<0
0000T`3oool000l0oooo00<000000?ooo`3oool0MP3oool2000009H0oooo000?0?ooo`030000003o
ool0oooo07<0oooo0`00002H0?ooo`003`3oool00`000000oooo0?ooo`1a0?ooo`800000V`3oool0
00H0oooo100000050?ooo`030000003oool0oooo06h0oooo0`00002M0?ooo`001P3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo06`0oooo0P00002P0?ooo`001`3oool00`000000oooo
0?ooo`050?ooo`800000K03oool00`000000oooo0?ooo`2P0?ooo`00203oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo06L0oooo1000002S0?ooo`002@3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0Y@3oool000H0oooo00@000000?oo
o`3oool000001@3oool00`000000oooo0?ooo`1T0?ooo`800000Z03oool000L0oooo0P0000060?oo
o`030000003oool0oooo06<0oooo00<000000?ooo`3oool0Z03oool000l0oooo00<000000?ooo`3o
ool0H@3oool200000:/0oooo000?0?ooo`030000003oool0oooo05l0oooo0P00002]0?ooo`003`3o
ool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo0:d0oooo000?0?ooo`030000003oool0
oooo05`0oooo0P00002`0?ooo`003`3oool00`000000oooo0?ooo`1J0?ooo`800000/P3oool000l0
oooo0P00001J0?ooo`030000003oool0oooo0;80oooo000?0?ooo`030000003oool0oooo05L0oooo
0P00002e0?ooo`003`3oool00`000000oooo0?ooo`1E0?ooo`800000]`3oool000l0oooo00<00000
0?ooo`3oool0E03oool00`000000oooo0?ooo`2g0?ooo`003`3oool00`000000oooo0?ooo`1D0?oo
o`030000003oool0oooo0;L0oooo000?0?ooo`030000003oool0oooo0580oooo0P00002j0?ooo`00
3`3oool00`000000oooo0?ooo`1@0?ooo`800000_03oool000l0oooo00<000000?ooo`3oool0C`3o
ool00`000000oooo0?ooo`2l0?ooo`003`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0
oooo0;d0oooo000?0?ooo`800000CP3oool00`000000oooo0?ooo`2n0?ooo`003`3oool00`000000
oooo0?ooo`1;0?ooo`800000`@3oool000l0oooo00<000000?ooo`3oool0BP3oool00`000000oooo
0?ooo`310?ooo`003`3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo0<40oooo000?
0?ooo`030000003oool0oooo04P0oooo0P0000340?ooo`003`3oool00`000000oooo0?ooo`170?oo
o`030000003oool0oooo0<@0oooo000?0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3o
ool0a@3oool000l0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`360?ooo`003`3o
ool00`000000oooo0?ooo`150?ooo`030000003oool0oooo0<H0oooo000?0?ooo`030000003oool0
oooo04<0oooo0P0000390?ooo`003`3oool2000004<0oooo00<000000?ooo`3oool0b@3oool000l0
oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`3:0?ooo`003`3oool00`000000oooo
0?ooo`100?ooo`030000003oool0oooo0</0oooo000?0?ooo`030000003oool0oooo0400oooo00<0
00000?ooo`3oool0b`3oool000l0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`3<
0?ooo`003`3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo0<d0oooo000?0?ooo`03
0000003oool0oooo03d0oooo00<000000?ooo`3oool0cP3oool000l0oooo00<000000?ooo`3oool0
?03oool00`000000oooo0?ooo`3?0?ooo`00203oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo03/0oooo00<000000?ooo`3oool0d03oool000P0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0=00oooo00050?ooo`D000001@3oool20000
03/0oooo00<000000?ooo`3oool0d@3oool000D0oooo00@000000?ooo`3oool000001P3oool00`00
0000oooo0?ooo`0i0?ooo`030000003oool0oooo0=80oooo00060?ooo`030000003oool0000000H0
oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`3C0?ooo`001`3oool2000000H0oooo
00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`3D0?ooo`00203oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0e@3oool000l0oooo00<00000
0?ooo`3oool0=P3oool00`000000oooo0?ooo`3E0?ooo`003`3oool00`000000oooo0?ooo`0e0?oo
o`030000003oool0oooo0=H0oooo000?0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3o
ool0e`3oool000l0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`3H0?ooo`003`3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0=P0oooo000?0?ooo`800000<`3oool0
0`000000oooo0?ooo`3I0?ooo`003`3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo
0=T0oooo000?0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0fP3oool000l0oooo
00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`3K0?ooo`003`3oool00`000000oooo0?oo
o`0_0?ooo`030000003oool0oooo0=`0oooo000?0?ooo`030000003oool0oooo02h0oooo00<00000
0?ooo`3oool0g@3oool000l0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`3M0?oo
o`003`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0=h0oooo000?0?ooo`030000
003oool0oooo02d0oooo00<000000?ooo`3oool0gP3oool000l0oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`3O0?ooo`003`3oool2000002`0oooo00<000000?ooo`3oool0h03oool0
00l0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`3Q0?ooo`003`3oool00`000000
oooo0?ooo`0Z0?ooo`030000003oool0oooo0>40oooo000?0?ooo`030000003oool0oooo02T0oooo
00<000000?ooo`3oool0hP3oool000l0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?oo
o`3R0?ooo`003`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0><0oooo000?0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0h`3oool000l0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`3T0?ooo`003`3oool00`000000oooo0?ooo`0V0?ooo`030000
003oool0oooo0>D0oooo000?0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0iP3o
ool000l0oooo0P00000V0?ooo`030000003oool0oooo0>H0oooo000?0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0i`3oool000l0oooo00<000000?ooo`3oool0903oool00`000000
oooo0?ooo`3W0?ooo`003`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0>P0oooo
000?0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0j03oool000l0oooo00<00000
0?ooo`3oool08P3oool00`000000oooo0?ooo`3Y0?ooo`003`3oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo0>T0oooo000?0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3o
ool0jP3oool000L0oooo0P0000060?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
j`3oool000H0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`0P0?ooo`030000
003oool0oooo0>/0oooo00060?ooo`040000003oool0oooo000000D0oooo0P00000P0?ooo`030000
003oool0oooo0>`0oooo00060?ooo`<000001P3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo0>`0oooo00060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool07P3oool0
0`000000oooo0?ooo`3]0?ooo`001P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool0kP3oool000L0oooo0`0000050?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool0kP3oool000l0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`3_0?ooo`003`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0?00oooo000?
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0l03oool000l0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`3a0?ooo`003`3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo0?40oooo000?0?ooo`8000006P3oool00`000000oooo0?ooo`3b0?ooo`003`3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0?80oooo000?0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0l`3oool000l0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`3c0?ooo`003`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0?@0
oooo000?0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0m@3oool000l0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`3e0?ooo`003`3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo0?H0oooo000?0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool0mP3oool000l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`3g0?ooo`00
3`3oool2000001@0oooo00<000000?ooo`3oool0n03oool000l0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`3h0?ooo`003`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo0?T0oooo000?0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0nP3oool000l0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3j0?ooo`003`3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0?/0oooo000?0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool0n`3oool000l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3l
0?ooo`003`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0?d0oooo000?0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool0oP3oool000l0oooo0P00000=0?ooo`030000
003oool0oooo0?l0oooo000?0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0o`3o
ool000l0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3o0?ooo`40oooo000?0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0o`3oool10?ooo`003`3oool00`000000
oooo0?ooo`090?ooo`800000o`3oool40?ooo`003`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo0?l0oooo103oool000l0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`3o0?ooo`@0oooo000?0?ooo`030000003oool0oooo00H0oooo0P00003o0?ooo`L0oooo00070?oo
o`8000001P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?l0oooo1`3oool000H0
oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0?l0oooo203oool000H0oooo00@000000?ooo`3oool000001@3oool200000080oooo0`00003o0?oo
o`/0oooo00070?ooo`8000001P3oool400000?l0oooo3P3oool000H0oooo00@000000?ooo`3oool0
00001@3oool00`000000oooo0?ooo`3o0?ooo`l0oooo00060?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool0o`3oool?0?ooo`001`3oool2000000H0oooo00<000000?ooo`3oool0
o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0912621, -0.625472, \
0.00604495, 0.0504074}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(for\ negative\ \(\(xi\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[Re[%27], PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606612 0.0147151 0.0987859 [
[.17546 .00222 -12 -9 ]
[.17546 .00222 12 0 ]
[.32712 .00222 -9 -9 ]
[.32712 .00222 9 0 ]
[.47877 .00222 -12 -9 ]
[.47877 .00222 12 0 ]
[.63042 .00222 -3 -9 ]
[.63042 .00222 3 0 ]
[.78207 .00222 -12 -9 ]
[.78207 .00222 12 0 ]
[.93373 .00222 -9 -9 ]
[.93373 .00222 9 0 ]
[.01131 .1135 -6 -4.5 ]
[.01131 .1135 0 4.5 ]
[.01131 .21229 -6 -4.5 ]
[.01131 .21229 0 4.5 ]
[.01131 .31107 -6 -4.5 ]
[.01131 .31107 0 4.5 ]
[.01131 .40986 -6 -4.5 ]
[.01131 .40986 0 4.5 ]
[.01131 .50864 -6 -4.5 ]
[.01131 .50864 0 4.5 ]
[.01131 .60743 -6 -4.5 ]
[.01131 .60743 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17546 .01472 m
.17546 .02097 L
s
[(0.25)] .17546 .00222 0 1 Mshowa
.32712 .01472 m
.32712 .02097 L
s
[(0.5)] .32712 .00222 0 1 Mshowa
.47877 .01472 m
.47877 .02097 L
s
[(0.75)] .47877 .00222 0 1 Mshowa
.63042 .01472 m
.63042 .02097 L
s
[(1)] .63042 .00222 0 1 Mshowa
.78207 .01472 m
.78207 .02097 L
s
[(1.25)] .78207 .00222 0 1 Mshowa
.93373 .01472 m
.93373 .02097 L
s
[(1.5)] .93373 .00222 0 1 Mshowa
.125 Mabswid
.05414 .01472 m
.05414 .01847 L
s
.08447 .01472 m
.08447 .01847 L
s
.1148 .01472 m
.1148 .01847 L
s
.14513 .01472 m
.14513 .01847 L
s
.20579 .01472 m
.20579 .01847 L
s
.23612 .01472 m
.23612 .01847 L
s
.26645 .01472 m
.26645 .01847 L
s
.29678 .01472 m
.29678 .01847 L
s
.35745 .01472 m
.35745 .01847 L
s
.38778 .01472 m
.38778 .01847 L
s
.41811 .01472 m
.41811 .01847 L
s
.44844 .01472 m
.44844 .01847 L
s
.5091 .01472 m
.5091 .01847 L
s
.53943 .01472 m
.53943 .01847 L
s
.56976 .01472 m
.56976 .01847 L
s
.60009 .01472 m
.60009 .01847 L
s
.66075 .01472 m
.66075 .01847 L
s
.69108 .01472 m
.69108 .01847 L
s
.72141 .01472 m
.72141 .01847 L
s
.75174 .01472 m
.75174 .01847 L
s
.81241 .01472 m
.81241 .01847 L
s
.84274 .01472 m
.84274 .01847 L
s
.87307 .01472 m
.87307 .01847 L
s
.9034 .01472 m
.9034 .01847 L
s
.96406 .01472 m
.96406 .01847 L
s
.99439 .01472 m
.99439 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .1135 m
.03006 .1135 L
s
[(1)] .01131 .1135 1 0 Mshowa
.02381 .21229 m
.03006 .21229 L
s
[(2)] .01131 .21229 1 0 Mshowa
.02381 .31107 m
.03006 .31107 L
s
[(3)] .01131 .31107 1 0 Mshowa
.02381 .40986 m
.03006 .40986 L
s
[(4)] .01131 .40986 1 0 Mshowa
.02381 .50864 m
.03006 .50864 L
s
[(5)] .01131 .50864 1 0 Mshowa
.02381 .60743 m
.03006 .60743 L
s
[(6)] .01131 .60743 1 0 Mshowa
.125 Mabswid
.02381 .03447 m
.02756 .03447 L
s
.02381 .05423 m
.02756 .05423 L
s
.02381 .07399 m
.02756 .07399 L
s
.02381 .09374 m
.02756 .09374 L
s
.02381 .13326 m
.02756 .13326 L
s
.02381 .15302 m
.02756 .15302 L
s
.02381 .17277 m
.02756 .17277 L
s
.02381 .19253 m
.02756 .19253 L
s
.02381 .23204 m
.02756 .23204 L
s
.02381 .2518 m
.02756 .2518 L
s
.02381 .27156 m
.02756 .27156 L
s
.02381 .29132 m
.02756 .29132 L
s
.02381 .33083 m
.02756 .33083 L
s
.02381 .35059 m
.02756 .35059 L
s
.02381 .37034 m
.02756 .37034 L
s
.02381 .3901 m
.02756 .3901 L
s
.02381 .42962 m
.02756 .42962 L
s
.02381 .44937 m
.02756 .44937 L
s
.02381 .46913 m
.02756 .46913 L
s
.02381 .48889 m
.02756 .48889 L
s
.02381 .5284 m
.02756 .5284 L
s
.02381 .54816 m
.02756 .54816 L
s
.02381 .56792 m
.02756 .56792 L
s
.02381 .58767 m
.02756 .58767 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02988 .08004 m
.03594 .08026 L
.04201 .0806 L
.04807 .08104 L
.05414 .08155 L
.06021 .0821 L
.06627 .08269 L
.07234 .08331 L
.0784 .08396 L
.08447 .08462 L
.09054 .0853 L
.0966 .08599 L
.10267 .08669 L
.10874 .08741 L
.1148 .08813 L
.12087 .08886 L
.12693 .08961 L
.133 .09036 L
.13907 .09112 L
.14513 .0919 L
.1512 .09268 L
.15726 .09347 L
.16333 .09427 L
.1694 .09508 L
.17546 .0959 L
.18153 .09673 L
.18759 .09757 L
.19366 .09842 L
.19973 .09928 L
.20579 .10016 L
.21186 .10105 L
.21793 .10195 L
.22399 .10286 L
.23006 .10379 L
.23612 .10473 L
.24219 .10568 L
.24826 .10665 L
.25432 .10763 L
.26039 .10863 L
.26645 .10964 L
.27252 .11067 L
.27859 .11172 L
.28465 .11279 L
.29072 .11387 L
.29678 .11497 L
.30285 .11609 L
.30892 .11724 L
.31498 .1184 L
.32105 .11958 L
.32712 .12079 L
Mistroke
.33318 .12201 L
.33925 .12327 L
.34531 .12454 L
.35138 .12584 L
.35745 .12717 L
.36351 .12852 L
.36958 .1299 L
.37564 .1313 L
.38171 .13274 L
.38778 .1342 L
.39384 .1357 L
.39991 .13723 L
.40598 .13879 L
.41204 .14039 L
.41811 .14202 L
.42417 .14368 L
.43024 .14538 L
.43631 .14713 L
.44237 .14891 L
.44844 .15073 L
.4545 .15259 L
.46057 .1545 L
.46664 .15646 L
.4727 .15846 L
.47877 .16051 L
.48483 .1626 L
.4909 .16475 L
.49697 .16696 L
.50303 .16921 L
.5091 .17153 L
.51517 .1739 L
.52123 .17633 L
.5273 .17883 L
.53336 .18139 L
.53943 .18402 L
.5455 .18671 L
.55156 .18948 L
.55763 .19232 L
.56369 .19524 L
.56976 .19824 L
.57583 .20132 L
.58189 .20448 L
.58796 .20773 L
.59402 .21107 L
.60009 .2145 L
.60616 .21803 L
.61222 .22166 L
.61829 .22539 L
.62436 .22923 L
.63042 .23318 L
Mistroke
.63649 .23724 L
.64255 .24142 L
.64862 .24572 L
.65469 .25014 L
.66075 .25469 L
.66682 .25937 L
.67288 .26419 L
.67895 .26915 L
.68502 .27425 L
.69108 .2795 L
.69715 .28491 L
.70322 .29046 L
.70928 .29618 L
.71535 .30206 L
.72141 .30811 L
.72748 .31432 L
.73355 .32071 L
.73961 .32727 L
.74568 .33401 L
.75174 .34093 L
.75781 .34803 L
.76388 .3553 L
.76994 .36276 L
.77601 .3704 L
.78207 .37821 L
.78814 .38619 L
.79421 .39434 L
.80027 .40265 L
.80634 .41111 L
.81241 .41972 L
.81847 .42846 L
.82454 .43731 L
.8306 .44627 L
.83667 .4553 L
.84274 .4644 L
.8488 .47352 L
.85487 .48266 L
.86093 .49177 L
.867 .50083 L
.87307 .50979 L
.87913 .51861 L
.8852 .52726 L
.89126 .53569 L
.89733 .54384 L
.9034 .55167 L
.90946 .55914 L
.91553 .56618 L
.9216 .57275 L
.92766 .5788 L
.93373 .58427 L
Mistroke
.93979 .58913 L
.94586 .59333 L
.95193 .59683 L
.95799 .5996 L
.96406 .60162 L
.97012 .60286 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00;`3oool2000000@0oooo00<000000?ooo`3oool00P3oool400000080
oooo0`00000H0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`<00000603oool2000000@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`<000007@3oool4000001`0
oooo100000030?ooo`030000003oool0oooo0080oooo100000020?ooo`<000005`3oool4000000<0
oooo00<000000?ooo`3oool00P3oool3000001<0oooo000^0?ooo`040000003oool0oooo000000P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo0000
00/0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`0000090?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool0403oool002h0oooo00@000000?ooo`3oool000002@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo00@000000?ooo`3oool000002`3o
ool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0@0?ooo`00;P3oool010000000oooo0?ooo`00000:0?ooo`040000
003oool0oooo0?ooo`<000005`3oool010000000oooo0?ooo`0000080?ooo`<000005`3oool01000
0000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000007`3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool0oooo0`00000I0?ooo`030000
003oool0oooo00L0oooo0`00000C0?ooo`00;P3oool010000000oooo0?ooo`00000;0?ooo`040000
003oool0oooo000001T0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`0G0?oo
o`040000003oool0oooo000000/0oooo00@000000?ooo`3oool000008@3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00X0oooo00@000000?ooo`3oool000006`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo01<0oooo000^0?ooo`040000003oool0oooo000000P0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0G0?ooo`040000003oool0oooo0000
00P0oooo00<000000?ooo`3oool05`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool07@3oool3000001d0oooo0`0000090?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool05`3oool3000000T0oooo00<000000?ooo`3oool0
4`3oool00100oooo00<000000?ooo`3oool0703oool2000000X0oooo0P0000030?ooo`@000005`3o
ool2000000T0oooo1000000G0?ooo`8000002@3oool400000080oooo1000000N0?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool0203oool2000000<0oooo1000000H0?ooo`030000003o
ool0oooo00L0oooo1000000B0?ooo`00403oool00`000000oooo0?ooo`3o0?ooo`h0oooo000@0?oo
o`030000003oool0oooo0?l0oooo3P3oool00100oooo00<000000?ooo`3oool0o`3oool>0?ooo`00
2@3ooooo000001000000203oool00100oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00L0oooo000@0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo01L0oooo000@0?oo
o`030000003oool0oooo0?l0oooo3P3oool00100oooo00<000000?ooo`3oool0o`3oool>0?ooo`00
403oool200000?l0oooo3`3oool00100oooo00<000000?ooo`3oool0o`3oool>0?ooo`00403oool0
0`000000oooo0?ooo`3o0?ooo`h0oooo000@0?ooo`030000003oool0oooo0?l0oooo3P3oool00100
oooo00<000000?ooo`3oool0o`3oool>0?ooo`00403oool00`000000oooo0?ooo`3o0?ooo`h0oooo
000@0?ooo`800000o`3oool?0?ooo`00403oool00`000000oooo0?ooo`3o0?ooo`h0oooo000@0?oo
o`030000003oool0oooo0?l0oooo3P3oool00100oooo00<000000?ooo`3oool0o`3oool>0?ooo`00
403oool00`000000oooo0?ooo`3o0?ooo`h0oooo000@0?ooo`800000o`3oool?0?ooo`00403oool0
0`000000oooo0?ooo`3o0?ooo`h0oooo000@0?ooo`030000003oool0000000/00000o`3oool30?oo
o`00403oool00`000000oooo0?ooo`0;0?ooo`P00000nP3oool00100oooo00<000000?ooo`3oool0
4`3oool800000?80oooo000@0?ooo`800000703oool900000>T0oooo000@0?ooo`030000003oool0
oooo02@0oooo1P00003S0?ooo`00403oool00`000000oooo0?ooo`0Z0?ooo`P00000f`3oool00100
oooo00<000000?ooo`3oool0<P3oool500000=H0oooo00080?ooo`@00000103oool00`000000oooo
0?ooo`0g0?ooo`L00000c`3oool000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0n0?ooo`H00000b@3oool000X0oooo00<000000?ooo`3oool00`3oool3000004@0oooo1@000034
0?ooo`002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo04T0oooo1@00002o0?oo
o`002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo04h0oooo1@00002j0?ooo`00
203oool3000000D0oooo00<000000?ooo`3oool0D`3oool500000;D0oooo000:0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0F03oool500000;00oooo000@0?ooo`800000GP3oool3
00000:d0oooo000@0?ooo`030000003oool0oooo0600oooo1@00002X0?ooo`00403oool00`000000
oooo0?ooo`1U0?ooo`<00000Y@3oool00100oooo00<000000?ooo`3oool0J03oool400000:40oooo
000@0?ooo`030000003oool0oooo06`0oooo0`00002N0?ooo`00403oool200000700oooo0`00002K
0?ooo`00403oool00`000000oooo0?ooo`1b0?ooo`<00000V03oool00100oooo00<000000?ooo`3o
ool0M@3oool4000009@0oooo000@0?ooo`030000003oool0oooo07T0oooo0`00002A0?ooo`00403o
ool00`000000oooo0?ooo`1l0?ooo`800000S`3oool00100oooo00<000000?ooo`3oool0OP3oool3
000008`0oooo000@0?ooo`800000PP3oool2000008X0oooo000@0?ooo`030000003oool0oooo08<0
oooo0`0000270?ooo`00403oool00`000000oooo0?ooo`260?ooo`030000003oool0oooo08@0oooo
000@0?ooo`030000003oool0oooo08L0oooo100000220?ooo`00403oool00`000000oooo0?ooo`2;
0?ooo`030000003oool0oooo07l0oooo000@0?ooo`800000S@3oool2000007l0oooo000@0?ooo`03
0000003oool0oooo08h0oooo0`00001l0?ooo`00403oool00`000000oooo0?ooo`2A0?ooo`800000
NP3oool000P0oooo100000040?ooo`030000003oool0oooo09<0oooo0P00001h0?ooo`00203oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0M@3oool0
00T0oooo00<000000?ooo`3oool0103oool3000009H0oooo0P00001e0?ooo`002P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo09P0oooo0P00001c0?ooo`002`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo09X0oooo00<000000?ooo`3oool0L03oool000P0oooo00@0
00000?ooo`3oool00000103oool00`000000oooo0?ooo`2K0?ooo`800000L03oool000T0oooo0P00
00050?ooo`030000003oool0oooo09d0oooo00<000000?ooo`3oool0K@3oool00100oooo00<00000
0?ooo`3oool0WP3oool2000006d0oooo000@0?ooo`800000X@3oool2000006/0oooo000@0?ooo`03
0000003oool0oooo0:80oooo00<000000?ooo`3oool0J03oool00100oooo00<000000?ooo`3oool0
X`3oool2000006P0oooo000@0?ooo`030000003oool0oooo0:D0oooo0P00001V0?ooo`00403oool0
0`000000oooo0?ooo`2V0?ooo`030000003oool0oooo06@0oooo000@0?ooo`800000Z03oool00`00
0000oooo0?ooo`1S0?ooo`00403oool00`000000oooo0?ooo`2X0?ooo`800000H`3oool00100oooo
00<000000?ooo`3oool0ZP3oool200000640oooo000@0?ooo`030000003oool0oooo0:`0oooo00<0
00000?ooo`3oool0GP3oool00100oooo00<000000?ooo`3oool0[@3oool00`000000oooo0?ooo`1M
0?ooo`00403oool200000:l0oooo00<000000?ooo`3oool0G03oool00100oooo00<000000?ooo`3o
ool0[`3oool00`000000oooo0?ooo`1K0?ooo`00403oool00`000000oooo0?ooo`2`0?ooo`800000
F`3oool00100oooo00<000000?ooo`3oool0/P3oool00`000000oooo0?ooo`1H0?ooo`00403oool0
0`000000oooo0?ooo`2c0?ooo`030000003oool0oooo05L0oooo000@0?ooo`030000003oool0oooo
0;@0oooo00<000000?ooo`3oool0EP3oool00100oooo0P00002f0?ooo`030000003oool0oooo05D0
oooo000@0?ooo`030000003oool0oooo0;H0oooo00<000000?ooo`3oool0E03oool00100oooo00<0
00000?ooo`3oool0]`3oool00`000000oooo0?ooo`1C0?ooo`002@3oool2000000D0oooo00<00000
0?ooo`3oool0^03oool00`000000oooo0?ooo`1B0?ooo`00203oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo0;T0oooo00<000000?ooo`3oool0D@3oool000/0oooo00<000000?oo
o`3oool00P3oool300000;X0oooo00<000000?ooo`3oool0D03oool000T0oooo0P0000050?ooo`03
0000003oool0oooo0;/0oooo00<000000?ooo`3oool0C`3oool000/0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`2l0?ooo`030000003oool0oooo04h0oooo00080?ooo`040000003o
ool0oooo000000@0oooo00<000000?ooo`3oool0_@3oool00`000000oooo0?ooo`1=0?ooo`002@3o
ool2000000D0oooo00<000000?ooo`3oool0_P3oool00`000000oooo0?ooo`1<0?ooo`00403oool2
00000<00oooo00<000000?ooo`3oool0B`3oool00100oooo00<000000?ooo`3oool0`03oool00`00
0000oooo0?ooo`1:0?ooo`00403oool00`000000oooo0?ooo`300?ooo`030000003oool0oooo04X0
oooo000@0?ooo`030000003oool0oooo0<40oooo00<000000?ooo`3oool0B@3oool00100oooo00<0
00000?ooo`3oool0`P3oool00`000000oooo0?ooo`180?ooo`00403oool00`000000oooo0?ooo`33
0?ooo`030000003oool0oooo04L0oooo000@0?ooo`800000a@3oool00`000000oooo0?ooo`160?oo
o`00403oool00`000000oooo0?ooo`350?ooo`030000003oool0oooo04D0oooo000@0?ooo`030000
003oool0oooo0<D0oooo00<000000?ooo`3oool0A@3oool00100oooo00<000000?ooo`3oool0aP3o
ool00`000000oooo0?ooo`140?ooo`00403oool00`000000oooo0?ooo`370?ooo`030000003oool0
oooo04<0oooo000@0?ooo`800000b@3oool00`000000oooo0?ooo`120?ooo`00403oool00`000000
oooo0?ooo`390?ooo`030000003oool0oooo0440oooo000@0?ooo`030000003oool0oooo0<X0oooo
00<000000?ooo`3oool0@03oool00100oooo00<000000?ooo`3oool0bP3oool00`000000oooo0?oo
o`100?ooo`00403oool00`000000oooo0?ooo`3;0?ooo`030000003oool0oooo03l0oooo000@0?oo
o`800000c@3oool00`000000oooo0?ooo`0n0?ooo`00403oool00`000000oooo0?ooo`3=0?ooo`03
0000003oool0oooo03d0oooo000@0?ooo`030000003oool0oooo0<h0oooo00<000000?ooo`3oool0
?03oool00100oooo00<000000?ooo`3oool0cP3oool00`000000oooo0?ooo`0l0?ooo`002P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0<l0oooo00<000000?ooo`3oool0>`3oool0
00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3?0?ooo`030000003oool0oooo
03/0oooo00070?ooo`D00000103oool300000=00oooo00<000000?ooo`3oool0>P3oool000L0oooo
00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`3A0?ooo`030000003oool0oooo03T0
oooo00080?ooo`030000003oool0000000D0oooo00<000000?ooo`3oool0dP3oool00`000000oooo
0?ooo`0h0?ooo`002@3oool2000000D0oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?oo
o`0h0?ooo`002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0=<0oooo00<00000
0?ooo`3oool0=`3oool00100oooo0P00003D0?ooo`030000003oool0oooo03L0oooo000@0?ooo`03
0000003oool0oooo0=@0oooo00<000000?ooo`3oool0=P3oool00100oooo00<000000?ooo`3oool0
e@3oool00`000000oooo0?ooo`0e0?ooo`00403oool00`000000oooo0?ooo`3F0?ooo`030000003o
ool0oooo03@0oooo000@0?ooo`030000003oool0oooo0=L0oooo00<000000?ooo`3oool0<`3oool0
0100oooo0P00003H0?ooo`030000003oool0oooo03<0oooo000@0?ooo`030000003oool0oooo0=P0
oooo00<000000?ooo`3oool0<P3oool00100oooo00<000000?ooo`3oool0f03oool00`000000oooo
0?ooo`0b0?ooo`00403oool00`000000oooo0?ooo`3I0?ooo`030000003oool0oooo0340oooo000@
0?ooo`030000003oool0oooo0=X0oooo00<000000?ooo`3oool0<03oool00100oooo00<000000?oo
o`3oool0f`3oool00`000000oooo0?ooo`0_0?ooo`00403oool200000=`0oooo00<000000?ooo`3o
ool0;`3oool00100oooo00<000000?ooo`3oool0g03oool00`000000oooo0?ooo`0^0?ooo`00403o
ool00`000000oooo0?ooo`3L0?ooo`030000003oool0oooo02h0oooo000@0?ooo`030000003oool0
oooo0=d0oooo00<000000?ooo`3oool0;@3oool00100oooo00<000000?ooo`3oool0gP3oool00`00
0000oooo0?ooo`0/0?ooo`00403oool200000=l0oooo00<000000?ooo`3oool0;03oool00100oooo
00<000000?ooo`3oool0g`3oool00`000000oooo0?ooo`0[0?ooo`00403oool00`000000oooo0?oo
o`3P0?ooo`030000003oool0oooo02X0oooo00080?ooo`<000001@3oool00`000000oooo0?ooo`3Q
0?ooo`030000003oool0oooo02T0oooo000;0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0h@3oool00`000000oooo0?ooo`0Y0?ooo`002`3oool00`000000oooo0?ooo`020?ooo`<0
0000hP3oool00`000000oooo0?ooo`0X0?ooo`00203oool3000000D0oooo00<000000?ooo`3oool0
hP3oool00`000000oooo0?ooo`0X0?ooo`00203oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo0><0oooo00<000000?ooo`3oool09`3oool000P0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`3T0?ooo`030000003oool0oooo02H0oooo00080?ooo`@00000103oool00`00
0000oooo0?ooo`3T0?ooo`030000003oool0oooo02H0oooo000@0?ooo`030000003oool0oooo0>D0
oooo00<000000?ooo`3oool09@3oool00100oooo0P00003W0?ooo`030000003oool0oooo02@0oooo
000@0?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool0903oool00100oooo00<00000
0?ooo`3oool0i`3oool00`000000oooo0?ooo`0S0?ooo`00403oool00`000000oooo0?ooo`3X0?oo
o`030000003oool0oooo0280oooo000@0?ooo`030000003oool0oooo0>T0oooo00<000000?ooo`3o
ool08@3oool00100oooo0P00003Z0?ooo`030000003oool0oooo0240oooo000@0?ooo`030000003o
ool0oooo0>X0oooo00<000000?ooo`3oool0803oool00100oooo00<000000?ooo`3oool0j`3oool0
0`000000oooo0?ooo`0O0?ooo`00403oool00`000000oooo0?ooo`3/0?ooo`030000003oool0oooo
01h0oooo000@0?ooo`030000003oool0oooo0>d0oooo00<000000?ooo`3oool07@3oool00100oooo
0P00003_0?ooo`030000003oool0oooo01`0oooo000@0?ooo`030000003oool0oooo0>h0oooo00<0
00000?ooo`3oool0703oool00100oooo00<000000?ooo`3oool0k`3oool00`000000oooo0?ooo`0K
0?ooo`00403oool00`000000oooo0?ooo`3`0?ooo`8000006`3oool00100oooo00<000000?ooo`3o
ool0lP3oool00`000000oooo0?ooo`0H0?ooo`00403oool00`000000oooo0?ooo`3c0?ooo`030000
003oool0oooo01L0oooo000@0?ooo`800000m@3oool00`000000oooo0?ooo`0F0?ooo`00403oool0
0`000000oooo0?ooo`3e0?ooo`8000005P3oool00100oooo00<000000?ooo`3oool0m`3oool20000
01@0oooo00090?ooo`8000001@3oool00`000000oooo0?ooo`3i0?ooo`<000004@3oool000P0oooo
00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3l0?ooo`<000003P3oool000P0oooo
00@000000?ooo`3oool00000103oool300000?l0oooo3P3oool000P0oooo0`0000050?ooo`030000
003oool0oooo0?l0oooo3P3oool000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`3o0?ooo`h0oooo00080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool>
0?ooo`002@3oool300000?l0oooo5@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.099982, -0.463887, \
0.00610447, 0.0374855}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(Import["\<../../paper4/mminus.dat\>"];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606612 0.527777 0.126946 [
[.17546 .51528 -12 -9 ]
[.17546 .51528 12 0 ]
[.32712 .51528 -9 -9 ]
[.32712 .51528 9 0 ]
[.47877 .51528 -12 -9 ]
[.47877 .51528 12 0 ]
[.63042 .51528 -3 -9 ]
[.63042 .51528 3 0 ]
[.78207 .51528 -12 -9 ]
[.78207 .51528 12 0 ]
[.93373 .51528 -9 -9 ]
[.93373 .51528 9 0 ]
[.01131 .01999 -12 -4.5 ]
[.01131 .01999 0 4.5 ]
[.01131 .14694 -12 -4.5 ]
[.01131 .14694 0 4.5 ]
[.01131 .27388 -12 -4.5 ]
[.01131 .27388 0 4.5 ]
[.01131 .40083 -12 -4.5 ]
[.01131 .40083 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17546 .52778 m
.17546 .53403 L
s
[(0.25)] .17546 .51528 0 1 Mshowa
.32712 .52778 m
.32712 .53403 L
s
[(0.5)] .32712 .51528 0 1 Mshowa
.47877 .52778 m
.47877 .53403 L
s
[(0.75)] .47877 .51528 0 1 Mshowa
.63042 .52778 m
.63042 .53403 L
s
[(1)] .63042 .51528 0 1 Mshowa
.78207 .52778 m
.78207 .53403 L
s
[(1.25)] .78207 .51528 0 1 Mshowa
.93373 .52778 m
.93373 .53403 L
s
[(1.5)] .93373 .51528 0 1 Mshowa
.125 Mabswid
.05414 .52778 m
.05414 .53153 L
s
.08447 .52778 m
.08447 .53153 L
s
.1148 .52778 m
.1148 .53153 L
s
.14513 .52778 m
.14513 .53153 L
s
.20579 .52778 m
.20579 .53153 L
s
.23612 .52778 m
.23612 .53153 L
s
.26645 .52778 m
.26645 .53153 L
s
.29678 .52778 m
.29678 .53153 L
s
.35745 .52778 m
.35745 .53153 L
s
.38778 .52778 m
.38778 .53153 L
s
.41811 .52778 m
.41811 .53153 L
s
.44844 .52778 m
.44844 .53153 L
s
.5091 .52778 m
.5091 .53153 L
s
.53943 .52778 m
.53943 .53153 L
s
.56976 .52778 m
.56976 .53153 L
s
.60009 .52778 m
.60009 .53153 L
s
.66075 .52778 m
.66075 .53153 L
s
.69108 .52778 m
.69108 .53153 L
s
.72141 .52778 m
.72141 .53153 L
s
.75174 .52778 m
.75174 .53153 L
s
.81241 .52778 m
.81241 .53153 L
s
.84274 .52778 m
.84274 .53153 L
s
.87307 .52778 m
.87307 .53153 L
s
.9034 .52778 m
.9034 .53153 L
s
.96406 .52778 m
.96406 .53153 L
s
.99439 .52778 m
.99439 .53153 L
s
.25 Mabswid
0 .52778 m
1 .52778 L
s
.02381 .01999 m
.03006 .01999 L
s
[(-4)] .01131 .01999 1 0 Mshowa
.02381 .14694 m
.03006 .14694 L
s
[(-3)] .01131 .14694 1 0 Mshowa
.02381 .27388 m
.03006 .27388 L
s
[(-2)] .01131 .27388 1 0 Mshowa
.02381 .40083 m
.03006 .40083 L
s
[(-1)] .01131 .40083 1 0 Mshowa
.125 Mabswid
.02381 .04538 m
.02756 .04538 L
s
.02381 .07077 m
.02756 .07077 L
s
.02381 .09616 m
.02756 .09616 L
s
.02381 .12155 m
.02756 .12155 L
s
.02381 .17233 m
.02756 .17233 L
s
.02381 .19772 m
.02756 .19772 L
s
.02381 .22311 m
.02756 .22311 L
s
.02381 .2485 m
.02756 .2485 L
s
.02381 .29927 m
.02756 .29927 L
s
.02381 .32466 m
.02756 .32466 L
s
.02381 .35005 m
.02756 .35005 L
s
.02381 .37544 m
.02756 .37544 L
s
.02381 .42622 m
.02756 .42622 L
s
.02381 .45161 m
.02756 .45161 L
s
.02381 .477 m
.02756 .477 L
s
.02381 .50239 m
.02756 .50239 L
s
.02381 .55317 m
.02756 .55317 L
s
.02381 .57856 m
.02756 .57856 L
s
.02381 .60394 m
.02756 .60394 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02988 .60332 m
.03594 .60304 L
.04201 .60261 L
.04807 .60206 L
.05414 .60143 L
.06021 .60074 L
.06627 .6 L
.07234 .59923 L
.0784 .59844 L
.08447 .59762 L
.09054 .59679 L
.0966 .59595 L
.10267 .59509 L
.10874 .59423 L
.1148 .59335 L
.12087 .59247 L
.12693 .59158 L
.133 .59068 L
.13907 .58977 L
.14513 .58886 L
.1512 .58793 L
.15726 .587 L
.16333 .58606 L
.1694 .58511 L
.17546 .58416 L
.18153 .58319 L
.18759 .58222 L
.19366 .58123 L
.19973 .58024 L
.20579 .57923 L
.21186 .57822 L
.21793 .57719 L
.22399 .57615 L
.23006 .5751 L
.23612 .57403 L
.24219 .57295 L
.24826 .57186 L
.25432 .57076 L
.26039 .56963 L
.26645 .5685 L
.27252 .56735 L
.27859 .56618 L
.28465 .56499 L
.29072 .56379 L
.29678 .56256 L
.30285 .56132 L
.30892 .56006 L
.31498 .55878 L
.32105 .55747 L
.32712 .55614 L
Mistroke
.33318 .5548 L
.33925 .55342 L
.34531 .55202 L
.35138 .5506 L
.35745 .54915 L
.36351 .54767 L
.36958 .54617 L
.37564 .54463 L
.38171 .54307 L
.38778 .54147 L
.39384 .53985 L
.39991 .53818 L
.40598 .53649 L
.41204 .53475 L
.41811 .53298 L
.42417 .53118 L
.43024 .52933 L
.43631 .52744 L
.44237 .52551 L
.44844 .52353 L
.4545 .52151 L
.46057 .51944 L
.46664 .51733 L
.4727 .51516 L
.47877 .51294 L
.48483 .51066 L
.4909 .50833 L
.49697 .50595 L
.50303 .5035 L
.5091 .50099 L
.51517 .49841 L
.52123 .49577 L
.5273 .49306 L
.53336 .49028 L
.53943 .48743 L
.5455 .4845 L
.55156 .48149 L
.55763 .47839 L
.56369 .47522 L
.56976 .47195 L
.57583 .46859 L
.58189 .46514 L
.58796 .4616 L
.59402 .45795 L
.60009 .45419 L
.60616 .45033 L
.61222 .44635 L
.61829 .44226 L
.62436 .43805 L
.63042 .43372 L
Mistroke
.63649 .42926 L
.64255 .42466 L
.64862 .41993 L
.65469 .41505 L
.66075 .41003 L
.66682 .40486 L
.67288 .39954 L
.67895 .39405 L
.68502 .3884 L
.69108 .38258 L
.69715 .37658 L
.70322 .3704 L
.70928 .36404 L
.71535 .35749 L
.72141 .35075 L
.72748 .34382 L
.73355 .33668 L
.73961 .32934 L
.74568 .32179 L
.75174 .31403 L
.75781 .30606 L
.76388 .29788 L
.76994 .28949 L
.77601 .28089 L
.78207 .27208 L
.78814 .26307 L
.79421 .25386 L
.80027 .24445 L
.80634 .23486 L
.81241 .2251 L
.81847 .21518 L
.82454 .20512 L
.8306 .19493 L
.83667 .18464 L
.84274 .17427 L
.8488 .16385 L
.85487 .15341 L
.86093 .14299 L
.867 .13263 L
.87307 .12236 L
.87913 .11224 L
.8852 .10232 L
.89126 .09264 L
.89733 .08327 L
.9034 .07426 L
.90946 .06567 L
.91553 .05756 L
.9216 .04999 L
.92766 .04302 L
.93373 .03671 L
Mistroke
.93979 .0311 L
.94586 .02626 L
.95193 .02221 L
.95799 .01901 L
.96406 .01668 L
.97012 .01524 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?ooo`030000
003oool0oooo0?l0oooo3`3oool000l0oooo00<000000?ooo`3oool0o`3oool?0?ooo`003`3oool0
0`000000oooo0?ooo`3o0?ooo`l0oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0o`3oool50?ooo`<000001`3oool000T0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`3o0?ooo`40oooo1000000:0?ooo`001P3oool5000000@0oooo0`00003n0?ooo`800000
3P3oool000030?ooo`00000000000080000000D0oooo0000003oool0oooo000000050?ooo`030000
003oool0oooo0?d0oooo00<000000?ooo`3oool03P3oool000L0oooo00<000000?ooo`0000001@3o
ool00`000000oooo0?ooo`3m0?ooo`030000003oool0oooo00h0oooo00080?ooo`8000001@3oool0
0`000000oooo0?ooo`3k0?ooo`8000004@3oool000T0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`3j0?ooo`030000003oool0oooo0140oooo000?0?ooo`030000003oool0oooo0?T0
oooo00<000000?ooo`3oool04P3oool000l0oooo00<000000?ooo`3oool0n03oool00`000000oooo
0?ooo`0C0?ooo`003`3oool200000?T0oooo00<000000?ooo`3oool04`3oool000l0oooo00<00000
0?ooo`3oool0m`3oool00`000000oooo0?ooo`0D0?ooo`003`3oool00`000000oooo0?ooo`3f0?oo
o`030000003oool0oooo01D0oooo000?0?ooo`030000003oool0oooo0?D0oooo00<000000?ooo`3o
ool05P3oool000l0oooo00<000000?ooo`3oool0m03oool00`000000oooo0?ooo`0G0?ooo`003`3o
ool00`000000oooo0?ooo`3c0?ooo`030000003oool0oooo01P0oooo000?0?ooo`030000003oool0
oooo0?<0oooo00<000000?ooo`3oool0603oool000l0oooo0P00003c0?ooo`030000003oool0oooo
01T0oooo000?0?ooo`030000003oool0oooo0?80oooo00<000000?ooo`3oool06@3oool000l0oooo
00<000000?ooo`3oool0l@3oool00`000000oooo0?ooo`0J0?ooo`003`3oool00`000000oooo0?oo
o`3`0?ooo`030000003oool0oooo01/0oooo000?0?ooo`030000003oool0oooo0>l0oooo00<00000
0?ooo`3oool0703oool000l0oooo00<000000?ooo`3oool0k`3oool00`000000oooo0?ooo`0L0?oo
o`003`3oool00`000000oooo0?ooo`3^0?ooo`030000003oool0oooo01d0oooo000?0?ooo`800000
k`3oool00`000000oooo0?ooo`0M0?ooo`003`3oool00`000000oooo0?ooo`3]0?ooo`030000003o
ool0oooo01h0oooo000?0?ooo`030000003oool0oooo0>d0oooo00<000000?ooo`3oool07P3oool0
00l0oooo00<000000?ooo`3oool0k03oool00`000000oooo0?ooo`0O0?ooo`003`3oool00`000000
oooo0?ooo`3[0?ooo`030000003oool0oooo0200oooo000?0?ooo`030000003oool0oooo0>X0oooo
00<000000?ooo`3oool08@3oool000l0oooo00<000000?ooo`3oool0jP3oool00`000000oooo0?oo
o`0Q0?ooo`003`3oool200000>X0oooo00<000000?ooo`3oool08P3oool000l0oooo00<000000?oo
o`3oool0j@3oool00`000000oooo0?ooo`0R0?ooo`003`3oool00`000000oooo0?ooo`3X0?ooo`03
0000003oool0oooo02<0oooo000?0?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3oool0
8`3oool000l0oooo00<000000?ooo`3oool0i`3oool00`000000oooo0?ooo`0T0?ooo`00203oool2
000000D0oooo00<000000?ooo`3oool0i`3oool00`000000oooo0?ooo`0T0?ooo`001`3oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool09@3oool0
00X0oooo00<000000?ooo`3oool00P3oool300000>D0oooo00<000000?ooo`3oool09P3oool00003
0?ooo`0000000000008000000`3oool2000000D0oooo00<000000?ooo`3oool0i@3oool00`000000
oooo0?ooo`0V0?ooo`002P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0>@0oooo
00<000000?ooo`3oool09`3oool000L0oooo00@000000?ooo`3oool00000103oool00`000000oooo
0?ooo`3S0?ooo`030000003oool0oooo02P0oooo00080?ooo`8000001@3oool00`000000oooo0?oo
o`3S0?ooo`030000003oool0oooo02P0oooo000?0?ooo`030000003oool0oooo0>80oooo00<00000
0?ooo`3oool0:@3oool000l0oooo00<000000?ooo`3oool0hP3oool00`000000oooo0?ooo`0Y0?oo
o`003`3oool200000>80oooo00<000000?ooo`3oool0:P3oool000l0oooo00<000000?ooo`3oool0
h@3oool00`000000oooo0?ooo`0Z0?ooo`003`3oool00`000000oooo0?ooo`3P0?ooo`030000003o
ool0oooo02/0oooo000?0?ooo`030000003oool0oooo0=l0oooo00<000000?ooo`3oool0;03oool0
00l0oooo00<000000?ooo`3oool0gP3oool00`000000oooo0?ooo`0]0?ooo`003`3oool00`000000
oooo0?ooo`3N0?ooo`030000003oool0oooo02d0oooo000?0?ooo`030000003oool0oooo0=d0oooo
00<000000?ooo`3oool0;P3oool000l0oooo0P00003N0?ooo`030000003oool0oooo02h0oooo000?
0?ooo`030000003oool0oooo0=`0oooo00<000000?ooo`3oool0;`3oool000l0oooo00<000000?oo
o`3oool0g03oool00`000000oooo0?ooo`0_0?ooo`003`3oool00`000000oooo0?ooo`3K0?ooo`03
0000003oool0oooo0300oooo000?0?ooo`030000003oool0oooo0=/0oooo00<000000?ooo`3oool0
<03oool000l0oooo00<000000?ooo`3oool0fP3oool00`000000oooo0?ooo`0a0?ooo`003`3oool0
0`000000oooo0?ooo`3I0?ooo`030000003oool0oooo0380oooo000?0?ooo`800000fP3oool00`00
0000oooo0?ooo`0b0?ooo`003`3oool00`000000oooo0?ooo`3H0?ooo`030000003oool0oooo03<0
oooo000?0?ooo`030000003oool0oooo0=P0oooo00<000000?ooo`3oool0<`3oool000l0oooo00<0
00000?ooo`3oool0e`3oool00`000000oooo0?ooo`0d0?ooo`003`3oool00`000000oooo0?ooo`3F
0?ooo`030000003oool0oooo03D0oooo000?0?ooo`030000003oool0oooo0=H0oooo00<000000?oo
o`3oool0=@3oool000l0oooo00<000000?ooo`3oool0e@3oool00`000000oooo0?ooo`0f0?ooo`00
3`3oool200000=D0oooo00<000000?ooo`3oool0=`3oool000l0oooo00<000000?ooo`3oool0e03o
ool00`000000oooo0?ooo`0g0?ooo`003`3oool00`000000oooo0?ooo`3C0?ooo`030000003oool0
oooo03P0oooo000?0?ooo`030000003oool0oooo0=<0oooo00<000000?ooo`3oool0>03oool000l0
oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`0i0?ooo`001`3oool4000000@0oooo
00<000000?ooo`3oool0d@3oool00`000000oooo0?ooo`0j0?ooo`001`3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo0=40oooo00<000000?ooo`3oool0>P3oool000P0oooo00<00000
0?ooo`3oool0103oool300000=00oooo00<000000?ooo`3oool0>`3oool000030?ooo`0000000000
00800000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0<l0oooo00<000000?oo
o`3oool0?03oool000X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3>0?ooo`03
0000003oool0oooo03d0oooo00070?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3o
ool0cP3oool00`000000oooo0?ooo`0m0?ooo`00203oool2000000D0oooo00<000000?ooo`3oool0
c@3oool00`000000oooo0?ooo`0n0?ooo`003`3oool00`000000oooo0?ooo`3=0?ooo`030000003o
ool0oooo03h0oooo000?0?ooo`030000003oool0oooo0<`0oooo00<000000?ooo`3oool0?`3oool0
00l0oooo0P00003<0?ooo`030000003oool0oooo0400oooo000?0?ooo`030000003oool0oooo0<X0
oooo00<000000?ooo`3oool0@@3oool000l0oooo00<000000?ooo`3oool0b@3oool00`000000oooo
0?ooo`120?ooo`003`3oool00`000000oooo0?ooo`380?ooo`030000003oool0oooo04<0oooo000?
0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0A03oool000l0oooo00<000000?oo
o`3oool0a`3oool00`000000oooo0?ooo`140?ooo`003`3oool00`000000oooo0?ooo`360?ooo`03
0000003oool0oooo04D0oooo000?0?ooo`800000a`3oool00`000000oooo0?ooo`150?ooo`003`3o
ool00`000000oooo0?ooo`350?ooo`030000003oool0oooo04H0oooo000?0?ooo`030000003oool0
oooo0<@0oooo00<000000?ooo`3oool0A`3oool000l0oooo00<000000?ooo`3oool0``3oool00`00
0000oooo0?ooo`180?ooo`003`3oool00`000000oooo0?ooo`320?ooo`030000003oool0oooo04T0
oooo000?0?ooo`030000003oool0oooo0<80oooo00<000000?ooo`3oool0B@3oool000l0oooo00<0
00000?ooo`3oool0`@3oool00`000000oooo0?ooo`1:0?ooo`003`3oool200000<40oooo00<00000
0?ooo`3oool0B`3oool000l0oooo00<000000?ooo`3oool0_`3oool00`000000oooo0?ooo`1<0?oo
o`003`3oool00`000000oooo0?ooo`2n0?ooo`030000003oool0oooo04d0oooo000?0?ooo`030000
003oool0oooo0;d0oooo00<000000?ooo`3oool0CP3oool000l0oooo00<000000?ooo`3oool0_03o
ool00`000000oooo0?ooo`1?0?ooo`003`3oool00`000000oooo0?ooo`2k0?ooo`030000003oool0
oooo0500oooo000?0?ooo`030000003oool0oooo0;X0oooo00<000000?ooo`3oool0D@3oool000l0
oooo00<000000?ooo`3oool0^@3oool00`000000oooo0?ooo`1B0?ooo`003`3oool200000;T0oooo
00<000000?ooo`3oool0D`3oool000l0oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?oo
o`1D0?ooo`003`3oool00`000000oooo0?ooo`2f0?ooo`030000003oool0oooo05D0oooo000?0?oo
o`030000003oool0oooo0;D0oooo00<000000?ooo`3oool0EP3oool000l0oooo00<000000?ooo`3o
ool0]03oool00`000000oooo0?ooo`1G0?ooo`001`3oool4000000@0oooo00<000000?ooo`3oool0
/`3oool00`000000oooo0?ooo`1H0?ooo`002@3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0;80oooo00<000000?ooo`3oool0F@3oool000T0oooo00<000000?ooo`3oool00`3oool3
00000;40oooo00<000000?ooo`3oool0FP3oool000030?ooo`000000000000800000103oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0:l0oooo0P00001M0?ooo`002@3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0:h0oooo00<000000?ooo`3oool0G@3oool000L0oooo
0`0000050?ooo`030000003oool0oooo0:d0oooo00<000000?ooo`3oool0GP3oool000T0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`2/0?ooo`030000003oool0oooo05l0oooo000?
0?ooo`030000003oool0oooo0:X0oooo0P00001R0?ooo`003`3oool00`000000oooo0?ooo`2X0?oo
o`800000I03oool000l0oooo0P00002X0?ooo`030000003oool0oooo06@0oooo000?0?ooo`030000
003oool0oooo0:L0oooo00<000000?ooo`3oool0I03oool000l0oooo00<000000?ooo`3oool0Y@3o
ool2000006L0oooo000?0?ooo`030000003oool0oooo0:<0oooo0P00001Y0?ooo`003`3oool00`00
0000oooo0?ooo`2R0?ooo`030000003oool0oooo06T0oooo000?0?ooo`030000003oool0oooo0:00
oooo0P00001/0?ooo`003`3oool00`000000oooo0?ooo`2N0?ooo`800000KP3oool000l0oooo0P00
002N0?ooo`030000003oool0oooo06h0oooo000?0?ooo`030000003oool0oooo09/0oooo0P00001a
0?ooo`003`3oool00`000000oooo0?ooo`2I0?ooo`800000L`3oool000l0oooo00<000000?ooo`3o
ool0V03oool00`000000oooo0?ooo`1c0?ooo`003`3oool00`000000oooo0?ooo`2F0?ooo`800000
MP3oool000l0oooo00<000000?ooo`3oool0U03oool2000007P0oooo000?0?ooo`030000003oool0
oooo0980oooo0P00001j0?ooo`003`3oool200000980oooo00<000000?ooo`3oool0NP3oool000l0
oooo00<000000?ooo`3oool0S`3oool2000007d0oooo000?0?ooo`030000003oool0oooo08d0oooo
0P00001o0?ooo`003`3oool00`000000oooo0?ooo`0M0?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`@000000P3oool3000001T0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`00
000J0?ooo`800000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0080oooo0`00
00040?ooo`<000005`3oool4000001d0oooo100000030?ooo`030000003oool0oooo0080oooo1000
00020?ooo`<00000603oool4000000<0oooo00<000000?ooo`3oool00P3oool3000000d0oooo000?
0?ooo`030000003oool0oooo01`0oooo00@000000?ooo`3oool00000203oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo01D0oooo00@000000?ooo`3oool000002`3oool00`000000oooo
0?ooo`0F0?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool01@3oool010000000
oooo00000000000L0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0:0?ooo`003`3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0oooo0000
00T0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo
000000/0oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`00000:0?ooo`030000003o
ool0oooo00@0oooo0P00000N0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0:0?ooo`003`3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0
oooo000000X0oooo00@000000?ooo`3oool0oooo0`00000H0?ooo`040000003oool0oooo000000P0
oooo0`00000I0?ooo`040000003oool0oooo000000X0oooo00@000000?ooo`3oool0oooo1000000O
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool02@3oool010000000oooo0?ooo`3o
ool3000001X0oooo00<000000?ooo`3oool01`3oool3000000d0oooo000?0?ooo`8000007@3oool0
10000000oooo0?ooo`00000;0?ooo`040000003oool0oooo000001X0oooo00@000000?ooo`3oool0
0000203oool00`000000oooo0?ooo`0I0?ooo`040000003oool0oooo000000/0oooo00@000000?oo
o`00000000008P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo00X0oooo00@00000
0?ooo`3oool00000703oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00d0oooo000?
0?ooo`030000003oool0oooo01`0oooo00@000000?ooo`3oool00000203oool010000000oooo0?oo
o`0000020?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`0I0?ooo`040000003oool0oooo000000P0oooo1@0000000`3oool000000?ooo`0O0?oo
o`<000007P3oool3000000T0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0H
0?ooo`<000002@3oool00`000000oooo0?ooo`0=0?ooo`003`3oool00`000000oooo0?ooo`0M0?oo
o`8000002P3oool2000000<0oooo1000000H0?ooo`8000002@3oool4000001T0oooo0P0000090?oo
o`@000000P3oool4000001l0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`080?oo
o`8000000`3oool4000001T0oooo00<000000?ooo`3oool01`3oool4000000`0oooo000?0?ooo`03
0000003oool0oooo07T0oooo1000002A0?ooo`003`3oool00`000000oooo0?ooo`1g0?ooo`800000
U@3oool000l0oooo00<000000?ooo`3oool0M03oool3000009L0oooo000?0?ooo`030000003oool0
oooo0740oooo0`00002J0?ooo`00203ooooo000001T00000000?0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0
oooo0`0000060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`050?ooo`4000000@3oool10?ooo`003`3oool00`000000oooo0?ooo`0W0?ooo`030000
003oool0oooo02L0oooo00<000000?ooo`3oool04`3oool300000180oooo00<000000?ooo`3oool0
9`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0
4@3oool000l0oooo00<000000?ooo`3oool0H`3oool400000:L0oooo000?0?ooo`030000003oool0
oooo0600oooo0`00002[0?ooo`003`3oool00`000000oooo0?ooo`1K0?ooo`D00000[P3oool000l0
oooo00<000000?ooo`3oool0E`3oool400000;<0oooo000?0?ooo`800000D`3oool500000;L0oooo
000?0?ooo`030000003oool0oooo04d0oooo1@00002l0?ooo`003`3oool00`000000oooo0?ooo`1:
0?ooo`<00000`@3oool000l0oooo00<000000?ooo`3oool0A@3oool500000<@0oooo000?0?ooo`03
0000003oool0oooo0400oooo1@0000390?ooo`003`3oool00`000000oooo0?ooo`0i0?ooo`L00000
cP3oool000l0oooo00<000000?ooo`3oool0=03oool500000=D0oooo000?0?ooo`800000;P3oool7
00000=X0oooo000?0?ooo`030000003oool0oooo02P0oooo1@00003Q0?ooo`003`3oool00`000000
oooo0?ooo`0Q0?ooo`L00000iP3oool000l0oooo00<000000?ooo`3oool06`3oool600000>d0oooo
000?0?ooo`030000003oool0oooo01@0oooo1`00003c0?ooo`003`3oool00`000000oooo0?ooo`0=
0?ooo`L00000nP3oool000l0oooo00<000000?ooo`3oool01@3oool800000?l0oooo0P3oool000l0
oooo2000003o0?ooo`X0oooo000?0?ooo`030000003oool0oooo0?l0oooo3`3oool000l0oooo00<0
00000?ooo`3oool0o`3oool?0?ooo`003`3oool00`000000oooo0?ooo`3o0?ooo`l0oooo000?0?oo
o`030000003oool0oooo0?l0oooo3`3oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0917353, -4.22882, \
0.00592681, 0.0283213}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[Pi/4]\)], "Input"],

Cell[BoxData[
    \(0.7853981633974483`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{alomz[1, Cos[th], 1, 10]*Sqrt[11], 
        mm[1, 1, Cos[IntegerPart[th*10^16]*10^\((\(-16\))\)], 10]*Sqrt[11], 
        mp[1, 1, Cos[IntegerPart[th*10^16]*10^\((\(-16\))\)], 10]*
          Sqrt[11]}, {th, 0.001, Pi/2}, 
      PlotStyle \[Rule] {{Dashing[{0.002, 0.01}]}, {Dashing[{0.02, 
                0.02}]}, {}}, Frame \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606305 0.190536 0.0461079 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.17539 -0.0125 -12 -9 ]
[.17539 -0.0125 12 0 ]
[.32696 -0.0125 -9 -9 ]
[.32696 -0.0125 9 0 ]
[.47854 -0.0125 -12 -9 ]
[.47854 -0.0125 12 0 ]
[.63011 -0.0125 -3 -9 ]
[.63011 -0.0125 3 0 ]
[.78169 -0.0125 -12 -9 ]
[.78169 -0.0125 12 0 ]
[.93327 -0.0125 -9 -9 ]
[.93327 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .0061 -12 -4.5 ]
[-0.0125 .0061 0 4.5 ]
[-0.0125 .09832 -12 -4.5 ]
[-0.0125 .09832 0 4.5 ]
[-0.0125 .19054 -6 -4.5 ]
[-0.0125 .19054 0 4.5 ]
[-0.0125 .28275 -6 -4.5 ]
[-0.0125 .28275 0 4.5 ]
[-0.0125 .37497 -6 -4.5 ]
[-0.0125 .37497 0 4.5 ]
[-0.0125 .46718 -6 -4.5 ]
[-0.0125 .46718 0 4.5 ]
[-0.0125 .5594 -6 -4.5 ]
[-0.0125 .5594 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.17539 0 m
.17539 .00625 L
s
[(0.25)] .17539 -0.0125 0 1 Mshowa
.32696 0 m
.32696 .00625 L
s
[(0.5)] .32696 -0.0125 0 1 Mshowa
.47854 0 m
.47854 .00625 L
s
[(0.75)] .47854 -0.0125 0 1 Mshowa
.63011 0 m
.63011 .00625 L
s
[(1)] .63011 -0.0125 0 1 Mshowa
.78169 0 m
.78169 .00625 L
s
[(1.25)] .78169 -0.0125 0 1 Mshowa
.93327 0 m
.93327 .00625 L
s
[(1.5)] .93327 -0.0125 0 1 Mshowa
.125 Mabswid
.05412 0 m
.05412 .00375 L
s
.08444 0 m
.08444 .00375 L
s
.11476 0 m
.11476 .00375 L
s
.14507 0 m
.14507 .00375 L
s
.2057 0 m
.2057 .00375 L
s
.23602 0 m
.23602 .00375 L
s
.26633 0 m
.26633 .00375 L
s
.29665 0 m
.29665 .00375 L
s
.35728 0 m
.35728 .00375 L
s
.38759 0 m
.38759 .00375 L
s
.41791 0 m
.41791 .00375 L
s
.44822 0 m
.44822 .00375 L
s
.50885 0 m
.50885 .00375 L
s
.53917 0 m
.53917 .00375 L
s
.56948 0 m
.56948 .00375 L
s
.5998 0 m
.5998 .00375 L
s
.66043 0 m
.66043 .00375 L
s
.69074 0 m
.69074 .00375 L
s
.72106 0 m
.72106 .00375 L
s
.75138 0 m
.75138 .00375 L
s
.81201 0 m
.81201 .00375 L
s
.84232 0 m
.84232 .00375 L
s
.87264 0 m
.87264 .00375 L
s
.90295 0 m
.90295 .00375 L
s
.96358 0 m
.96358 .00375 L
s
.9939 0 m
.9939 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0061 m
.00625 .0061 L
s
[(-4)] -0.0125 .0061 1 0 Mshowa
0 .09832 m
.00625 .09832 L
s
[(-2)] -0.0125 .09832 1 0 Mshowa
0 .19054 m
.00625 .19054 L
s
[(0)] -0.0125 .19054 1 0 Mshowa
0 .28275 m
.00625 .28275 L
s
[(2)] -0.0125 .28275 1 0 Mshowa
0 .37497 m
.00625 .37497 L
s
[(4)] -0.0125 .37497 1 0 Mshowa
0 .46718 m
.00625 .46718 L
s
[(6)] -0.0125 .46718 1 0 Mshowa
0 .5594 m
.00625 .5594 L
s
[(8)] -0.0125 .5594 1 0 Mshowa
.125 Mabswid
0 .02916 m
.00375 .02916 L
s
0 .05221 m
.00375 .05221 L
s
0 .07527 m
.00375 .07527 L
s
0 .12137 m
.00375 .12137 L
s
0 .14443 m
.00375 .14443 L
s
0 .16748 m
.00375 .16748 L
s
0 .21359 m
.00375 .21359 L
s
0 .23664 m
.00375 .23664 L
s
0 .2597 m
.00375 .2597 L
s
0 .30581 m
.00375 .30581 L
s
0 .32886 m
.00375 .32886 L
s
0 .35191 m
.00375 .35191 L
s
0 .39802 m
.00375 .39802 L
s
0 .42108 m
.00375 .42108 L
s
0 .44413 m
.00375 .44413 L
s
0 .49024 m
.00375 .49024 L
s
0 .51329 m
.00375 .51329 L
s
0 .53634 m
.00375 .53634 L
s
0 .58245 m
.00375 .58245 L
s
0 .60551 m
.00375 .60551 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.17539 .61178 m
.17539 .61803 L
s
.32696 .61178 m
.32696 .61803 L
s
.47854 .61178 m
.47854 .61803 L
s
.63011 .61178 m
.63011 .61803 L
s
.78169 .61178 m
.78169 .61803 L
s
.93327 .61178 m
.93327 .61803 L
s
.125 Mabswid
.05412 .61428 m
.05412 .61803 L
s
.08444 .61428 m
.08444 .61803 L
s
.11476 .61428 m
.11476 .61803 L
s
.14507 .61428 m
.14507 .61803 L
s
.2057 .61428 m
.2057 .61803 L
s
.23602 .61428 m
.23602 .61803 L
s
.26633 .61428 m
.26633 .61803 L
s
.29665 .61428 m
.29665 .61803 L
s
.35728 .61428 m
.35728 .61803 L
s
.38759 .61428 m
.38759 .61803 L
s
.41791 .61428 m
.41791 .61803 L
s
.44822 .61428 m
.44822 .61803 L
s
.50885 .61428 m
.50885 .61803 L
s
.53917 .61428 m
.53917 .61803 L
s
.56948 .61428 m
.56948 .61803 L
s
.5998 .61428 m
.5998 .61803 L
s
.66043 .61428 m
.66043 .61803 L
s
.69074 .61428 m
.69074 .61803 L
s
.72106 .61428 m
.72106 .61803 L
s
.75138 .61428 m
.75138 .61803 L
s
.81201 .61428 m
.81201 .61803 L
s
.84232 .61428 m
.84232 .61803 L
s
.87264 .61428 m
.87264 .61803 L
s
.90295 .61428 m
.90295 .61803 L
s
.96358 .61428 m
.96358 .61803 L
s
.9939 .61428 m
.9939 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0061 m
1 .0061 L
s
.99375 .09832 m
1 .09832 L
s
.99375 .19054 m
1 .19054 L
s
.99375 .28275 m
1 .28275 L
s
.99375 .37497 m
1 .37497 L
s
.99375 .46718 m
1 .46718 L
s
.99375 .5594 m
1 .5594 L
s
.125 Mabswid
.99625 .02916 m
1 .02916 L
s
.99625 .05221 m
1 .05221 L
s
.99625 .07527 m
1 .07527 L
s
.99625 .12137 m
1 .12137 L
s
.99625 .14443 m
1 .14443 L
s
.99625 .16748 m
1 .16748 L
s
.99625 .21359 m
1 .21359 L
s
.99625 .23664 m
1 .23664 L
s
.99625 .2597 m
1 .2597 L
s
.99625 .30581 m
1 .30581 L
s
.99625 .32886 m
1 .32886 L
s
.99625 .35191 m
1 .35191 L
s
.99625 .39802 m
1 .39802 L
s
.99625 .42108 m
1 .42108 L
s
.99625 .44413 m
1 .44413 L
s
.99625 .49024 m
1 .49024 L
s
.99625 .51329 m
1 .51329 L
s
.99625 .53634 m
1 .53634 L
s
.99625 .58245 m
1 .58245 L
s
.99625 .60551 m
1 .60551 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .19054 m
1 .19054 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .002 .01 ] 0 setdash
.02442 .01472 m
.02559 .01473 L
.02666 .01475 L
.02789 .01479 L
.02906 .01483 L
.03113 .01494 L
.03339 .0151 L
.03587 .01532 L
.0385 .0156 L
.04321 .01626 L
.05264 .01808 L
.06303 .02084 L
.08477 .02881 L
.10513 .03834 L
.14388 .05929 L
.18508 .08185 L
.22476 .10141 L
.26293 .11745 L
.30355 .13157 L
.34266 .14266 L
.38421 .15222 L
.42425 .15969 L
.46278 .16558 L
.50376 .17072 L
.54322 .17479 L
.58117 .17804 L
.62156 .1809 L
.66045 .18318 L
.70178 .18517 L
.7416 .18673 L
.77991 .18793 L
.82066 .18893 L
.8409 .18933 L
.85991 .18965 L
.87828 .18991 L
.89763 .19014 L
.9178 .19032 L
.9363 .19043 L
.94583 .19048 L
.95108 .1905 L
.95602 .19051 L
.96064 .19052 L
.96307 .19052 L
.96564 .19053 L
.96781 .19053 L
.96895 .19053 L
.97019 .19053 L
.97132 .19053 L
.97235 .19054 L
.97339 .19054 L
Mistroke
.97438 .19054 L
.97532 .19054 L
.97619 .19054 L
Mfstroke
[ .02 .02 ] 0 setdash
.02442 .01472 m
.02559 .01473 L
.02666 .01476 L
.02789 .0148 L
.02906 .01486 L
.03113 .015 L
.03339 .0152 L
.03587 .01547 L
.0385 .01584 L
.04321 .01666 L
.05264 .01897 L
.06303 .02244 L
.08477 .03239 L
.10513 .04414 L
.14388 .06934 L
.18508 .09555 L
.22476 .11745 L
.26293 .13483 L
.30355 .1497 L
.34266 .1611 L
.38421 .17077 L
.42425 .17825 L
.46278 .18413 L
.50376 .18929 L
.54322 .19345 L
.58117 .19685 L
.62156 .19997 L
.66045 .20259 L
.70178 .20503 L
.7416 .20712 L
.77991 .20892 L
.82066 .21062 L
.85991 .21202 L
.89763 .21312 L
.91697 .21355 L
.92692 .21373 L
.93781 .2139 L
.9475 .21401 L
.95651 .21409 L
.96149 .21412 L
.96602 .21414 L
.96853 .21415 L
.96985 .21415 L
.97125 .21416 L
.97248 .21416 L
.97317 .21416 L
.97382 .21416 L
.97496 .21416 L
.97619 .21416 L
s
[ ] 0 setdash
.02442 .60332 m
.02559 .60329 L
.02666 .60325 L
.02789 .60317 L
.02906 .60306 L
.03113 .60282 L
.03339 .60247 L
.03587 .60197 L
.0385 .60132 L
.04321 .59984 L
.05264 .59571 L
.06303 .58948 L
.08477 .57155 L
.10513 .55028 L
.14388 .50413 L
.18508 .45536 L
.22476 .4139 L
.26293 .38049 L
.30355 .35149 L
.34266 .32899 L
.38421 .30976 L
.42425 .29482 L
.46278 .28305 L
.50376 .27275 L
.54322 .26452 L
.58117 .25787 L
.62156 .25188 L
.66045 .24699 L
.70178 .24255 L
.7416 .23891 L
.77991 .23591 L
.82066 .23321 L
.85991 .23108 L
.89763 .22951 L
.91697 .2289 L
.92692 .22865 L
.93781 .22842 L
.9475 .22826 L
.95651 .22816 L
.96149 .22812 L
.96602 .22809 L
.96853 .22808 L
.96985 .22807 L
.97125 .22807 L
.97248 .22807 L
.97317 .22806 L
.97382 .22806 L
.97496 .22806 L
.97619 .22806 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001Ooo`8001goo`8000Coo`03003ooooo00;oo`@000;oo`<001Soo`8000Coo`03003ooooo00;o
o`<001Ooo`8000Coo`03003ooooo00?oo`03003ooooo00;oo`<001coo`@001coo`@000?oo`03003o
oooo00;oo`@000;oo`<001Koo`@000?oo`03003ooooo00;oo`<000ooo`005_oo00@00?ooool001_o
o`04003ooooo0008ool00`00ooooo`06ool00`00ooooo`0Dool01000ooooo`002ooo00<00?ooool0
4ooo00@00?ooool000Woo`03003ooooo00Goo`03003ooooo01_oo`03003ooooo01goo`03003ooooo
00Ooo`03003ooooo00Koo`03003ooooo01Goo`03003ooooo00[oo`03003ooooo00coo`005_oo00@0
0?ooool001_oo`04003ooooo0009ool00`00ooooo`05ool00`00ooooo`0Dool01000ooooo`002ooo
00<00?ooool04ooo00@00?ooool000[oo`03003ooooo00Coo`03003ooooo01_oo`03003ooooo01go
o`03003ooooo00Soo`03003ooooo00Goo`03003ooooo01Goo`03003ooooo00[oo`03003ooooo00co
o`005_oo00@00?ooool001_oo`04003ooooo000:ool01000oooooooo0`005ooo00@00?ooool000So
o`<001Koo`04003ooooo000:ool01000oooooooo0`007_oo00<00?ooool07Ooo00<00?ooool02Ooo
00@00?ooooooo`<001Soo`03003ooooo00Ooo`<000ooo`005_oo00@00?ooool001_oo`04003ooooo
000;ool01000ooooo`006Ooo00@00?ooool000Soo`03003ooooo01Koo`04003ooooo000;ool01000
ooooo`008?oo00<00?ooool07Ooo00<00?ooool02_oo00@00?ooool001[oo`03003ooooo00Ooo`03
003ooooo00ooo`005_oo00@00?ooool001_oo`04003ooooo0008ool01000ooooo`000_oo00<00?oo
ool05ooo00@00?ooool000Soo`03003ooooo01Koo`04003ooooo0008ool01000ooooo`000_oo00<0
0?ooool07?oo0`007Ooo0`002Ooo00@00?ooool000;oo`03003ooooo01Koo`<000Woo`03003ooooo
00ooo`005ooo0P007Ooo0P002_oo0P000ooo10005ooo0P002Ooo10005_oo0P002Ooo10000_oo1000
7Ooo00<00?ooool07Ooo00<00?ooool02?oo0P000ooo10005ooo00<00?ooool01ooo10003_oo003o
oolQool00?ooob7oo`00oooo8Ooo000;ool00`00ooooo`03oooo000=0003ool000_oo`03003ooooo
00?oo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003oo`0000?oo`002?oo1@00
1?oo0`001?oo00<00?ooool09Ooo00<00?ooool09_oo00<00?ooool09Ooo00<00?ooool09Ooo00<0
0?ooool09_oo00<00?ooool09Ooo00<00?ooool03Ooo0`000ooo0003ool400001Ooo003ooooo0000
1Ooo00<00?ooool0oooo2Ooo00<00?ooool00Ooo0009ool00`00ool00005ool00`00ooooo`04ool6
003nool00`00ooooo`01ool000[oo`8000Goo`03003ooooo00coo`03003ooooo0?Woo`03003ooooo
007oo`002ooo00<00?ooool00ooo00<00?ooool03_oo0P00n?oo00<00?ooool00Ooo000Aool00`00
ooooo`0@ool3003eool00`00ooooo`01ool0017oo`8001?oo`800003ool00?oo0?3oo`8000?oo`00
4Ooo00<00?ooool06?oo00<00?ooool0kOoo00<00?ooool00Ooo000Aool00`00ooooo`0Hool00`00
ooooo`3]ool00`00ooooo`01ool0017oo`03003ooooo01Soo`04003oo`00003/ool00`00ooooo`01
ool0017oo`03003ooooo01Woo`04003oooooool2003Yool00`00ooooo`01ool0017oo`03003ooooo
01[oo`800>coo`03003ooooo007oo`004Ooo0P007Ooo00@00?ooooooo`800>Goo`8000?oo`004Ooo
00<00?ooool08ooo00<00?ooool0h_oo00<00?ooool00Ooo000Aool00`00ooooo`0Qool00`00oooo
o`3Tool00`00ooooo`01ool0017oo`03003ooooo02;oo`8000;oo`03003ooooo0=ooo`03003ooooo
007oo`004Ooo00<00?ooool09?oo00D00?ooooooo`000=ooo`03003ooooo007oo`004Ooo00<00?oo
ool09Ooo00D00?ooooooo`000=koo`03003ooooo007oo`004Ooo0P00;?oo0P00f_oo0P000ooo000A
ool00`00ooooo`0^ool00`00ooooo`3Gool00`00ooooo`01ool0017oo`03003ooooo02[oo`03003o
oooo00;oo`03003ooooo0=Koo`03003ooooo007oo`004Ooo00<00?ooool0:ooo00<00?ooool00ooo
00<00?ooool0e?oo00<00?ooool00Ooo0009ool40004ool00`00ooooo`0/ool20006ool00`00oooo
o`3Aool00`00ooooo`01ool000Woo`03003ooooo00Goo`03003ooooo02koo`03003ooooo0=Ooo`03
003ooooo007oo`002_oo00<00?ooool01?oo0`00=ooo00<00?ooool0c?oo0`000ooo0003ool40004
ool00`00ooooo`03ool00`00ooooo`0jool00`00ooooo`3;ool00`00ooooo`01ool000coo`03003o
oooo00;oo`03003ooooo03?oo`8000Ooo`800<[oo`03003ooooo007oo`002Ooo00@00?ooool000Co
o`03003ooooo03Goo`800=7oo`03003ooooo007oo`002_oo0P001Ooo00<00?ooool0=ooo0P001_oo
0P00aooo00<00?ooool00Ooo000Aool00`00ooooo`12ool20034ool00`00ooooo`01ool0017oo`80
04Koo`800<3oo`8000?oo`004Ooo00<00?ooool0??oo0P002_oo0P00__oo00<00?ooool00Ooo000A
ool00`00ooooo`0nool2000;ool2002kool00`00ooooo`01ool0017oo`03003ooooo043oo`8000co
o`800;Soo`03003ooooo007oo`004Ooo00<00?ooool0D_oo00<00?ooool0/ooo00<00?ooool00Ooo
000Aool00`00ooooo`16ool2000=ool00`00ooooo`2`ool00`00ooooo`01ool0017oo`8004Woo`80
00koo`04003ooooo002[ool20003ool0017oo`03003ooooo04[oo`8000ooo`04003ooooo002Yool0
0`00ooooo`01ool0017oo`03003ooooo053oo`8000ooo`800:Goo`03003ooooo007oo`004Ooo00<0
0?ooool0D_oo0P004?oo0P0000?oo`000000Wooo00<00?ooool00Ooo000Aool00`00ooooo`1Dool2
000Dool20002ool00`00ooooo`2Gool00`00ooooo`01ool0017oo`03003ooooo05[oo`<001Coo`04
003ooooo002Cool00`00ooooo`01ool0017oo`8005koo`@001Koo`800003ool0000008_oo`8000?o
o`004Ooo00<00?ooool0OOoo0P000_oo00@00?ooool008?oo`03003ooooo007oo`004Ooo00<00?oo
ool0IOoo0`007ooo00<00?ooool00P0000?oo`000000NOoo00<00?ooool00Ooo000Aool00`00oooo
o`1Xool3000Uool20002ool01000ooooo`000_oo00<00?ooool0Jooo00<00?ooool00Ooo000Aool0
0`00ooooo`1_ool7000Wool200000ooo00000002ool01000ooooo`00H?oo00<00?ooool00Ooo000:
ool20005ool00`00ooooo`1jool3000]ool00`00ooooo`0200001?oo00000?oo0P000_oo00@00?oo
ool000;oo`8004Woo`03003ooooo007oo`002Ooo00@00?ooool000Coo`03003ooooo07goo`<000Co
o`<003Woo`800003ool0000000;oo`04003ooooo0002ool200001?oo00000?oo0P000_oo00@00?oo
ool000;oo`800004ool00000ool20002ool00`00ooooo`0Nool00`00ooooo`01ool000Woo`04003o
oooo0004oooo000=0003ool000Woo`04003ooooo0004ool00`00ooooo`2?ool60005ool3001[ool0
0`00ooooo`01ool000Woo`04003ooooo0004ool00`00ooooo`2Mool30004ool3001Qool00`00oooo
o`01ool000Woo`04003ooooo0004ool00`00ooooo`2Wool40004ool6001Cool00`00ooooo`01ool0
00[oo`8000Goo`03003ooooo0;[oo`H000Coo`H003koo`03003ooooo007oo`004Ooo00<00?ooool0
cooo1P001?oo1`001?oo0`008Ooo00<00?ooool00Ooo000Aool2003Xool30005ool60004ool70007
ool20003ool0017oo`03003ooooo0?ooo`Woo`03003ooooo007oo`004Ooo00<00?ooool0oooo2Ooo
00<00?ooool00Ooo000Aool00`00ooooo`3oool9ool00`00ooooo`01ool0017oo`03003ooooo0=go
obD000Koo`03003ooooo007oo`004Ooo00<00?ooool0dooo2P00:ooo00<00?ooool00Ooo000Aool2
0039ool;000dool20003ool0017oo`03003ooooo0;Kooa80043oo`03003ooooo007oo`004Ooo00<0
0?ooool0/?oo1P00D_oo00<00?ooool00Ooo000Aool00`00ooooo`2Xool8001Hool00`00ooooo`01
ool0017oo`03003ooooo0:7oo`L0063oo`03003ooooo007oo`004Ooo00<00?ooool0Vooo1P00Iooo
00<00?ooool00Ooo000Aool2002Dool8001/ool20003ool0017oo`03003ooooo08goo`H007Goo`03
003ooooo007oo`004Ooo00<00?ooool0ROoo1000Nooo00<00?ooool00Ooo000Aool00`00ooooo`26
ool3001oool00`00ooooo`01ool000Woo`@000Coo`03003ooooo08;oo`@008;oo`03003ooooo007o
o`002Ooo00<00?ooool01Ooo00<00?ooool0O?oo1P00Q_oo00<00?ooool00Ooo000:ool00`00oooo
o`04ool3001hool4002:ool30003ool000_oo`03003ooooo00?oo`03003ooooo07Coo`@0093oo`03
003ooooo007oo`003?oo00<00?ooool00_oo00<00?ooool0L?oo1000U?oo00<00?ooool00Ooo0009
ool01000ooooo`001?oo00<00?ooool0KOoo0`00V?oo00<00?ooool00Ooo000:ool20005ool00`00
ooooo`1[ool2002Kool00`00ooooo`01ool0017oo`03003ooooo06Soo`<009goo`03003ooooo007o
o`004Ooo0P00Iooo0P00Wooo0P000ooo000Aool00`00ooooo`1Tool2002Rool00`00ooooo`01ool0
017oo`03003ooooo06;oo`800:Coo`03003ooooo007oo`004Ooo00<00?ooool0H?oo0P00Y_oo00<0
0?ooool00Ooo000Aool00`00ooooo`1Nool2002Xool00`00ooooo`01ool0017oo`03003ooooo05co
o`800:[oo`03003ooooo007oo`004Ooo0P00Fooo0P00Zooo0P000ooo000Aool00`00ooooo`1Iool0
0`00ooooo`2/ool00`00ooooo`01ool0017oo`03003ooooo05Ooo`800:ooo`03003ooooo007oo`00
4Ooo00<00?ooool0EOoo0P00/Ooo00<00?ooool00Ooo000Aool00`00ooooo`1Cool2002cool00`00
ooooo`01ool0017oo`03003ooooo057oo`800;Goo`03003ooooo007oo`004Ooo00<00?ooool0Cooo
0P00]ooo00<00?ooool00Ooo000Aool2001?ool00`00ooooo`2fool20003ool0017oo`03003ooooo
04coo`800;[oo`03003ooooo007oo`004Ooo00<00?ooool0Booo00<00?ooool0^_oo00<00?ooool0
0Ooo000Aool00`00ooooo`19ool2002mool00`00ooooo`01ool000_oo`03003ooooo00?oo`03003o
oooo04Ooo`800;ooo`03003ooooo007oo`002ooo00<00?ooool00ooo00<00?ooool0A_oo00<00?oo
ool0_ooo00<00?ooool00Ooo0008ool50004ool30014ool20030ool30003ool000Soo`04003ooooo
0005ool00`00ooooo`13ool00`00ooooo`32ool00`00ooooo`01ool000Woo`03003oo`0000Goo`03
003ooooo04;oo`03003ooooo0<?oo`03003ooooo007oo`002_oo0P001Ooo00<00?ooool0@Ooo00<0
0?ooool0a?oo00<00?ooool00Ooo000;ool00`00ooooo`03ool00`00ooooo`10ool00`00ooooo`35
ool00`00ooooo`01ool0017oo`03003ooooo03koo`800<Soo`03003ooooo007oo`004Ooo0P00?_oo
00<00?ooool0aooo0P000ooo000Aool00`00ooooo`0lool00`00ooooo`39ool00`00ooooo`01ool0
017oo`03003ooooo03_oo`03003ooooo0<[oo`03003ooooo007oo`004Ooo00<00?ooool0>_oo00<0
0?ooool0booo00<00?ooool00Ooo000Aool00`00ooooo`0iool00`00ooooo`3<ool00`00ooooo`01
ool0017oo`03003ooooo03Soo`03003ooooo0<goo`03003ooooo007oo`004Ooo0P00>?oo00<00?oo
ool0cOoo0P000ooo000Aool00`00ooooo`0fool00`00ooooo`3?ool00`00ooooo`01ool0017oo`03
003ooooo03Goo`03003ooooo0=3oo`03003ooooo007oo`004Ooo00<00?ooool0=?oo00<00?ooool0
dOoo00<00?ooool00Ooo000Aool00`00ooooo`0dool00`00ooooo`3Aool00`00ooooo`01ool0017o
o`03003ooooo03?oo`03003ooooo0=;oo`03003ooooo007oo`004Ooo0P00<ooo00<00?ooool0d_oo
0P000ooo000Aool00`00ooooo`0aool00`00ooooo`3Dool00`00ooooo`01ool0017oo`03003ooooo
033oo`03003ooooo0=Goo`03003ooooo007oo`004Ooo00<00?ooool0;ooo00<00?ooool0e_oo00<0
0?ooool00Ooo000:ool20005ool00`00ooooo`0^ool00`00ooooo`3Gool00`00ooooo`01ool000Wo
o`04003ooooo0004ool00`00ooooo`0]ool00`00ooooo`3Hool00`00ooooo`01ool000Woo`04003o
oooo0004ool3000/ool00`00ooooo`3Gool30003ool000Woo`<000Goo`03003ooooo02coo`03003o
oooo0=Woo`03003ooooo007oo`002Ooo00<00?ooool01Ooo00<00?ooool0:ooo00<00?ooool0f_oo
00<00?ooool00Ooo0009ool00`00ooooo`05ool00`00ooooo`0Zool00`00ooooo`3Kool00`00oooo
o`01ool000[oo`<000Coo`03003ooooo02Woo`03003ooooo0=coo`03003ooooo007oo`004Ooo00<0
0?ooool0:?oo00<00?ooool0gOoo00<00?ooool00Ooo000Aool2000Xool00`00ooooo`3Mool20003
ool0017oo`03003ooooo02Ooo`03003ooooo0=koo`03003ooooo007oo`004Ooo00<00?ooool09_oo
00<00?ooool0gooo00<00?ooool00Ooo000Aool00`00ooooo`0Uool00`00ooooo`3Pool00`00oooo
o`01ool0017oo`03003ooooo02Coo`03003ooooo0>7oo`03003ooooo007oo`004Ooo00<00?ooool0
8ooo00<00?ooool0h_oo00<00?ooool00Ooo000Aool00`00ooooo`0Rool00`00ooooo`3Sool00`00
ooooo`01ool0017oo`8002;oo`03003ooooo0>?oo`8000?oo`004Ooo00<00?ooool08?oo00<00?oo
ool0iOoo00<00?ooool00Ooo000Aool00`00ooooo`0Oool00`00ooooo`3Vool00`00ooooo`01ool0
017oo`03003ooooo01ooo`03003ooooo0>Koo`03003ooooo007oo`004Ooo00<00?ooool07_oo00<0
0?ooool0iooo00<00?ooool00Ooo000Aool00`00ooooo`0Mool00`00ooooo`3Xool00`00ooooo`01
ool0017oo`8001goo`03003ooooo0>Soo`8000?oo`004Ooo00<00?ooool06ooo00<00?ooool0j_oo
00<00?ooool00Ooo000Aool00`00ooooo`0Jool00`00ooooo`3[ool00`00ooooo`01ool0017oo`03
003ooooo01Woo`03003ooooo0>coo`03003ooooo007oo`002_oo0P001Ooo00<00?ooool06?oo00<0
0?ooool0kOoo00<00?ooool00Ooo0009ool01000ooooo`001?oo00<00?ooool05ooo00<00?ooool0
k_oo00<00?ooool00Ooo0009ool01000ooooo`001?oo0`005ooo00<00?ooool0k?oo0`000ooo000:
ool20005ool00`00ooooo`0Fool00`00ooooo`3_ool00`00ooooo`01ool000Woo`04003ooooo0004
ool00`00ooooo`0Eool00`00ooooo`3`ool00`00ooooo`01ool000Woo`04003ooooo0004ool00`00
ooooo`0Dool00`00ooooo`3aool00`00ooooo`01ool000[oo`8000Goo`03003ooooo01?oo`03003o
oooo0?;oo`03003ooooo007oo`004Ooo00<00?ooool04Ooo0P00mOoo00<00?ooool00Ooo000Aool2
000Aool00`00ooooo`3dool20003ool0017oo`03003ooooo00ooo`03003ooooo0?Koo`03003ooooo
007oo`004Ooo00<00?ooool03_oo00<00?ooool0mooo00<00?ooool00Ooo000Aool00`00ooooo`0<
ool2003jool00`00ooooo`01ool0017oo`03003ooooo00[oo`800?coo`03003ooooo007oo`004Ooo
00<00?ooool01_oo1000o_oo00<00?ooool00Ooo000Aool20005ool2003oool2ool20003ool0017o
o`03003ooooo0?ooo`Woo`03003ooooo007oo`004Ooo00<00?ooool01?oo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?oo00000ooo000Aoooo000=0003ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.150314, -5.13644, \
0.00619257, 0.0814305}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<massesnew.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("massesnew.eps"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \("\<massesnew.eps\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{alomz[1, Cos[th], 1, \(-0.9\)]*Sqrt[0.1], 
        mm[1, 1, Cos[IntegerPart[th*10^16]*10^\((\(-16\))\)], \(-9\)/10]*
          Sqrt[0.1], 
        mp[1, 1, Cos[IntegerPart[th*10^16]*10^\((\(-16\))\)], \(-9\)/10]*
          Sqrt[0.1]}, {th, 0.001, Pi/2}, 
      PlotStyle \[Rule] {{Dashing[{0.002, 0.01}]}, {Dashing[{0.02, 
                0.02}]}, {}}, Frame \[Rule] True, 
      PlotRange \[Rule] All]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"ploss\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.606305 0.304067 0.226398 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.17539 -0.0125 -12 -9 ]
[.17539 -0.0125 12 0 ]
[.32696 -0.0125 -9 -9 ]
[.32696 -0.0125 9 0 ]
[.47854 -0.0125 -12 -9 ]
[.47854 -0.0125 12 0 ]
[.63011 -0.0125 -3 -9 ]
[.63011 -0.0125 3 0 ]
[.78169 -0.0125 -12 -9 ]
[.78169 -0.0125 12 0 ]
[.93327 -0.0125 -9 -9 ]
[.93327 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .07767 -12 -4.5 ]
[-0.0125 .07767 0 4.5 ]
[-0.0125 .19087 -24 -4.5 ]
[-0.0125 .19087 0 4.5 ]
[-0.0125 .30407 -6 -4.5 ]
[-0.0125 .30407 0 4.5 ]
[-0.0125 .41727 -18 -4.5 ]
[-0.0125 .41727 0 4.5 ]
[-0.0125 .53047 -6 -4.5 ]
[-0.0125 .53047 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.17539 0 m
.17539 .00625 L
s
[(0.25)] .17539 -0.0125 0 1 Mshowa
.32696 0 m
.32696 .00625 L
s
[(0.5)] .32696 -0.0125 0 1 Mshowa
.47854 0 m
.47854 .00625 L
s
[(0.75)] .47854 -0.0125 0 1 Mshowa
.63011 0 m
.63011 .00625 L
s
[(1)] .63011 -0.0125 0 1 Mshowa
.78169 0 m
.78169 .00625 L
s
[(1.25)] .78169 -0.0125 0 1 Mshowa
.93327 0 m
.93327 .00625 L
s
[(1.5)] .93327 -0.0125 0 1 Mshowa
.125 Mabswid
.05412 0 m
.05412 .00375 L
s
.08444 0 m
.08444 .00375 L
s
.11476 0 m
.11476 .00375 L
s
.14507 0 m
.14507 .00375 L
s
.2057 0 m
.2057 .00375 L
s
.23602 0 m
.23602 .00375 L
s
.26633 0 m
.26633 .00375 L
s
.29665 0 m
.29665 .00375 L
s
.35728 0 m
.35728 .00375 L
s
.38759 0 m
.38759 .00375 L
s
.41791 0 m
.41791 .00375 L
s
.44822 0 m
.44822 .00375 L
s
.50885 0 m
.50885 .00375 L
s
.53917 0 m
.53917 .00375 L
s
.56948 0 m
.56948 .00375 L
s
.5998 0 m
.5998 .00375 L
s
.66043 0 m
.66043 .00375 L
s
.69074 0 m
.69074 .00375 L
s
.72106 0 m
.72106 .00375 L
s
.75138 0 m
.75138 .00375 L
s
.81201 0 m
.81201 .00375 L
s
.84232 0 m
.84232 .00375 L
s
.87264 0 m
.87264 .00375 L
s
.90295 0 m
.90295 .00375 L
s
.96358 0 m
.96358 .00375 L
s
.9939 0 m
.9939 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .07767 m
.00625 .07767 L
s
[(-1)] -0.0125 .07767 1 0 Mshowa
0 .19087 m
.00625 .19087 L
s
[(-0.5)] -0.0125 .19087 1 0 Mshowa
0 .30407 m
.00625 .30407 L
s
[(0)] -0.0125 .30407 1 0 Mshowa
0 .41727 m
.00625 .41727 L
s
[(0.5)] -0.0125 .41727 1 0 Mshowa
0 .53047 m
.00625 .53047 L
s
[(1)] -0.0125 .53047 1 0 Mshowa
.125 Mabswid
0 .10031 m
.00375 .10031 L
s
0 .12295 m
.00375 .12295 L
s
0 .14559 m
.00375 .14559 L
s
0 .16823 m
.00375 .16823 L
s
0 .21351 m
.00375 .21351 L
s
0 .23615 m
.00375 .23615 L
s
0 .25879 m
.00375 .25879 L
s
0 .28143 m
.00375 .28143 L
s
0 .32671 m
.00375 .32671 L
s
0 .34935 m
.00375 .34935 L
s
0 .37199 m
.00375 .37199 L
s
0 .39463 m
.00375 .39463 L
s
0 .43991 m
.00375 .43991 L
s
0 .46255 m
.00375 .46255 L
s
0 .48519 m
.00375 .48519 L
s
0 .50783 m
.00375 .50783 L
s
0 .05503 m
.00375 .05503 L
s
0 .03239 m
.00375 .03239 L
s
0 .00975 m
.00375 .00975 L
s
0 .55311 m
.00375 .55311 L
s
0 .57574 m
.00375 .57574 L
s
0 .59838 m
.00375 .59838 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.17539 .61178 m
.17539 .61803 L
s
.32696 .61178 m
.32696 .61803 L
s
.47854 .61178 m
.47854 .61803 L
s
.63011 .61178 m
.63011 .61803 L
s
.78169 .61178 m
.78169 .61803 L
s
.93327 .61178 m
.93327 .61803 L
s
.125 Mabswid
.05412 .61428 m
.05412 .61803 L
s
.08444 .61428 m
.08444 .61803 L
s
.11476 .61428 m
.11476 .61803 L
s
.14507 .61428 m
.14507 .61803 L
s
.2057 .61428 m
.2057 .61803 L
s
.23602 .61428 m
.23602 .61803 L
s
.26633 .61428 m
.26633 .61803 L
s
.29665 .61428 m
.29665 .61803 L
s
.35728 .61428 m
.35728 .61803 L
s
.38759 .61428 m
.38759 .61803 L
s
.41791 .61428 m
.41791 .61803 L
s
.44822 .61428 m
.44822 .61803 L
s
.50885 .61428 m
.50885 .61803 L
s
.53917 .61428 m
.53917 .61803 L
s
.56948 .61428 m
.56948 .61803 L
s
.5998 .61428 m
.5998 .61803 L
s
.66043 .61428 m
.66043 .61803 L
s
.69074 .61428 m
.69074 .61803 L
s
.72106 .61428 m
.72106 .61803 L
s
.75138 .61428 m
.75138 .61803 L
s
.81201 .61428 m
.81201 .61803 L
s
.84232 .61428 m
.84232 .61803 L
s
.87264 .61428 m
.87264 .61803 L
s
.90295 .61428 m
.90295 .61803 L
s
.96358 .61428 m
.96358 .61803 L
s
.9939 .61428 m
.9939 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .07767 m
1 .07767 L
s
.99375 .19087 m
1 .19087 L
s
.99375 .30407 m
1 .30407 L
s
.99375 .41727 m
1 .41727 L
s
.99375 .53047 m
1 .53047 L
s
.125 Mabswid
.99625 .10031 m
1 .10031 L
s
.99625 .12295 m
1 .12295 L
s
.99625 .14559 m
1 .14559 L
s
.99625 .16823 m
1 .16823 L
s
.99625 .21351 m
1 .21351 L
s
.99625 .23615 m
1 .23615 L
s
.99625 .25879 m
1 .25879 L
s
.99625 .28143 m
1 .28143 L
s
.99625 .32671 m
1 .32671 L
s
.99625 .34935 m
1 .34935 L
s
.99625 .37199 m
1 .37199 L
s
.99625 .39463 m
1 .39463 L
s
.99625 .43991 m
1 .43991 L
s
.99625 .46255 m
1 .46255 L
s
.99625 .48519 m
1 .48519 L
s
.99625 .50783 m
1 .50783 L
s
.99625 .05503 m
1 .05503 L
s
.99625 .03239 m
1 .03239 L
s
.99625 .00975 m
1 .00975 L
s
.99625 .55311 m
1 .55311 L
s
.99625 .57574 m
1 .57574 L
s
.99625 .59838 m
1 .59838 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .30407 m
1 .30407 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .002 .01 ] 0 setdash
.02442 .35135 m
.02559 .35135 L
.02666 .35135 L
.02789 .35135 L
.02906 .35135 L
.03113 .35135 L
.03339 .35135 L
.03587 .35135 L
.0385 .35135 L
.04321 .35134 L
.05264 .35133 L
.06303 .35131 L
.0735 .35128 L
.08477 .35124 L
.10513 .35116 L
.12626 .35104 L
.14633 .35091 L
.1845 .35058 L
.22512 .35012 L
.26423 .34957 L
.30578 .34885 L
.34582 .34801 L
.38435 .34706 L
.42533 .34587 L
.46479 .34453 L
.50274 .34302 L
.54313 .34116 L
.58202 .33907 L
.62335 .33648 L
.66317 .33357 L
.70148 .33033 L
.74223 .32638 L
.78148 .32209 L
.8192 .3176 L
.85938 .3127 L
.89804 .30839 L
.91768 .3066 L
.92806 .30582 L
.93916 .30512 L
.94921 .30463 L
.95834 .30432 L
.96275 .30421 L
.96507 .30416 L
.96752 .30413 L
.96967 .3041 L
.97089 .30409 L
.97203 .30408 L
.97308 .30407 L
.97404 .30407 L
.97514 .30407 L
Mistroke
.97619 .30407 L
Mfstroke
[ .02 .02 ] 0 setdash
.02442 .34672 m
.02559 .34672 L
.02666 .34671 L
.02789 .3467 L
.02906 .34668 L
.03113 .34665 L
.03339 .34659 L
.0385 .34642 L
.04321 .34621 L
.06303 .34502 L
.10513 .34183 L
.14446 .33857 L
.18227 .33524 L
.22253 .33148 L
.26128 .32757 L
.30248 .32301 L
.34216 .31813 L
.38033 .31287 L
.42095 .30649 L
.46006 .29942 L
.50161 .29065 L
.54165 .28064 L
.58018 .26921 L
.62116 .25461 L
.66062 .23757 L
.69856 .2178 L
.73896 .19235 L
.77785 .16303 L
.81918 .12674 L
.859 .08852 L
.89731 .05302 L
.91668 .03772 L
.92697 .03082 L
.93806 .02458 L
.94832 .02007 L
.95281 .0185 L
.95773 .01709 L
.96239 .01604 L
.96672 .01534 L
.96912 .01507 L
.9713 .01488 L
.97251 .01481 L
.97382 .01475 L
.97505 .01472 L
.97619 .01472 L
s
[ ] 0 setdash
.02442 .35138 m
.02559 .35136 L
.02666 .35137 L
.02789 .35138 L
.02906 .3514 L
.03113 .35143 L
.03339 .35149 L
.0385 .35167 L
.04321 .35188 L
.06303 .3531 L
.10513 .35644 L
.14446 .35995 L
.18227 .36359 L
.22253 .3678 L
.26128 .37225 L
.30248 .37751 L
.34216 .3832 L
.38033 .38939 L
.42095 .39693 L
.46006 .40531 L
.50161 .41572 L
.54165 .42756 L
.58018 .44105 L
.62116 .4582 L
.66062 .47809 L
.69856 .50104 L
.73896 .53036 L
.77785 .5639 L
.81918 .60509 L
s
.81918 .60509 m
.83115 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Pool2000Lool20004ool00`00ooooo`02ool40002ool3000Gool2
0004ool00`00ooooo`02ool3000Fool20004ool00`00ooooo`03ool00`00ooooo`02ool3000Kool4
000Kool40003ool00`00ooooo`02ool40002ool3000Eool40003ool00`00ooooo`02ool3000<ool0
01ooo`04003ooooo000Jool01000ooooo`002?oo00<00?ooool01_oo00<00?ooool04ooo00@00?oo
ool000_oo`03003ooooo01;oo`04003ooooo0009ool00`00ooooo`05ool00`00ooooo`0Jool00`00
ooooo`0Lool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`0Dool00`00ooooo`0:ool00`00
ooooo`09ool001ooo`04003ooooo000Jool01000ooooo`002Ooo00<00?ooool01Ooo00<00?ooool0
4ooo00@00?ooool000_oo`03003ooooo01;oo`04003ooooo000:ool00`00ooooo`04ool00`00oooo
o`0Jool00`00ooooo`0Lool00`00ooooo`08ool00`00ooooo`05ool00`00ooooo`0Dool00`00oooo
o`0:ool00`00ooooo`09ool001ooo`04003ooooo000Jool01000ooooo`002_oo00@00?ooooooo`<0
01Koo`04003ooooo0008ool3000Eool01000ooooo`002_oo00@00?ooooooo`<001goo`03003ooooo
01coo`03003ooooo00Woo`04003oooooool3000Gool00`00ooooo`07ool3000<ool001ooo`04003o
oooo000Jool01000ooooo`002ooo00@00?ooool001Soo`04003ooooo0008ool00`00ooooo`0Eool0
1000ooooo`002ooo00@00?ooool001ooo`03003ooooo01coo`03003ooooo00[oo`04003ooooo000I
ool00`00ooooo`07ool00`00ooooo`0<ool001ooo`04003ooooo000Jool01000ooooo`002?oo00@0
0?ooool000;oo`03003ooooo01Koo`04003ooooo0008ool00`00ooooo`0Eool01000ooooo`002?oo
00@00?ooool000;oo`03003ooooo01_oo`<001coo`<000Woo`04003ooooo0002ool00`00ooooo`0E
ool30009ool00`00ooooo`0<ool0023oo`8001coo`8000[oo`8000?oo`@001Koo`8000Woo`@001Go
o`8000Woo`@000;oo`@001coo`03003ooooo01coo`03003ooooo00Soo`8000?oo`@001Koo`03003o
oooo00Ooo`@000_oo`00oooo8Ooo003ooolQool00?ooob7oo`006oooo`001@000Ooo000Kool00`00
ooooo`03ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool20001ool001_oo`8000Coo`03003ooooo02Co
o`03003ooooo02Goo`03003ooooo02Coo`03003ooooo02Coo`03003ooooo02Goo`03003ooooo02Co
o`03003ooooo00goo`80007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo000Kool00`00ooooo`3e
ool00`00ooooo`02ool00`00ooooo`03ool10001ool001_oo`03003ooooo0?;oo`<000_oo`40007o
o`006ooo00<00?ooool0l?oo0P003_oo0@000Ooo000Kool00`00ooooo`3oool1ool10001ool001_o
o`800?ooo`7oo`80007oo`006ooo00<00?ooool0k?oo00<00?ooool04Ooo0@000Ooo000Kool00`00
ooooo`3[ool00`00ooooo`0Bool10001ool001_oo`03003ooooo0>Woo`8001Goo`40007oo`006ooo
00<00?ooool0j?oo00<00?ooool05Ooo0@000Ooo000Kool00`00ooooo`3oool1ool10001ool001_o
o`800?ooo`7oo`80007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo000Kool00`00ooooo`3Tool0
0`00ooooo`0Iool10001ool001_oo`03003ooooo0>;oo`8001coo`40007oo`004ooo10001?oo00<0
0?ooool0hOoo00<00?ooool07?oo0@000Ooo000Eool00`00ooooo`03ool00`00ooooo`3Pool00`00
ooooo`0Mool10001ool001Goo`03003ooooo00?oo`800?ooo`7oo`80007oo`003Ooo10001?oo00<0
0?ooool00ooo00<00?ooool0oooo0Ooo0@000Ooo000Eool00`00ooooo`03ool00`00ooooo`3oool1
ool10001ool001?oo`<000Goo`03003ooooo0=coo`03003ooooo027oo`40007oo`005Ooo00<00?oo
ool00ooo00<00?ooool0f_oo0P009?oo0@000Ooo000Kool00`00ooooo`3Iool00`00ooooo`0Tool1
0001ool001_oo`800=Woo`03003ooooo02Coo`80007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo
000Kool00`00ooooo`3oool1ool10001ool001_oo`03003ooooo0?ooo`7oo`40007oo`006ooo00<0
0?ooool0eOoo00<00?ooool0:?oo0@000Ooo000Kool00`00ooooo`3Cool2000[ool10001ool001_o
o`800=?oo`03003ooooo02[oo`80007oo`006ooo00<00?ooool0dOoo00<00?ooool0;?oo0@000Ooo
000Kool00`00ooooo`3oool1ool10001ool001_oo`03003ooooo0?ooo`7oo`40007oo`006ooo00<0
0?ooool0oooo0Ooo0@000Ooo000Kool00`00ooooo`3=ool00`00ooooo`0`ool10001ool001_oo`80
0<coo`8003;oo`80007oo`006ooo00<00?ooool0b_oo00<00?ooool0<ooo0@000Ooo000Kool00`00
ooooo`39ool00`00ooooo`0dool10001ool001_oo`03003ooooo0?ooo`7oo`40007oo`006ooo00<0
0?ooool0oooo0Ooo0@000Ooo000Kool00`00ooooo`3oool1ool10001ool001_oo`800<Koo`03003o
oooo03Ooo`80007oo`006ooo00<00?ooool0`ooo0P00>ooo0@000Ooo000Kool00`00ooooo`32ool0
0`00ooooo`0kool10001ool000Soo`8000Coo`03003ooooo00;oo`<000Goo`03003ooooo0<7oo`03
003ooooo03coo`40007oo`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo0?ooo`7o
o`40007oo`001ooo00@00?ooool000_oo`03003ooooo00;oo`800?ooo`7oo`80007oo`0000?oo`00
00000P000_oo00@00?ooool000Soo`<000Goo`03003ooooo0;goo`03003ooooo043oo`40007oo`00
1ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo0;_oo`8004?oo`40007oo`001ooo00@0
0?ooool000Soo`03003ooooo00Goo`03003ooooo0;Woo`8004Goo`40007oo`002?oo0P002Ooo1000
1?oo00<00?ooool0^?oo00<00?ooool0AOoo0@000Ooo000Kool00`00ooooo`3oool1ool10001ool0
01_oo`800;Coo`8004[oo`80007oo`006ooo00<00?ooool0/_oo00<00?ooool0Booo0@000Ooo000K
ool00`00ooooo`2`ool2001>ool10001ool001_oo`03003ooooo0:ooo`03003ooooo04koo`40007o
o`006ooo00<00?ooool0oooo0Ooo0@000Ooo000Kool00`00ooooo`2[ool00`00ooooo`1Bool10001
ool001_oo`800:[oo`8005Coo`80007oo`006ooo00<00?ooool0Yooo0P00Eooo0@000Ooo000Kool0
0`00ooooo`2Vool00`00ooooo`1Gool10001ool001_oo`03003ooooo0?ooo`7oo`40007oo`006ooo
00<00?ooool0X?oo0P00G_oo0@000Ooo000Kool00`00ooooo`2Nool2001Pool10001ool001_oo`80
09koo`03003ooooo05ooo`80007oo`006ooo00<00?ooool0Uooo0P00Iooo0@000Ooo000Kool00`00
ooooo`2Eool2001Yool10001ool001_oo`03003ooooo09?oo`8006_oo`40007oo`006ooo00<00?oo
ool0oooo0Ooo0@000Ooo000Kool00`00ooooo`2<ool3001aool10001ool001_oo`8008[oo`<007?o
o`80007oo`006ooo00<00?ooool0P_oo0`00Nooo0@000Ooo000Kool00`00ooooo`1oool3001nool1
0001ool001_oo`03003ooooo0?ooo`7oo`40007oo`005?oo0P001Ooo00<00?ooool0N?oo0`00QOoo
0@000Ooo000Cool01000ooooo`001?oo00<00?ooool0MOoo0`00R?oo0@000Ooo000Cool01000oooo
o`001?ooo`001@000Ooo000Cool01000ooooo`001?oo00<00?ooool0Hooo0`00O?oo00@00?ooool0
00;oo`03003ooooo008001?oo`40007oo`004ooo00@00?ooool000Coo`03003ooooo063oo`<007Ko
o`04003ooooo0002ool00`00ooooo`0Nool10001ool001?oo`04003ooooo0004ool00`00ooooo`1I
ool3001fool200001Ooo00000?oo00009ooo0@000Ooo000Dool20005ool00`00ooooo`1>ool40004
ool3001`ool200001?oo00000?oo0P00;ooo0@000Ooo000Kool00`00ooooo`14ool30004ool3001b
ool01000ooooo`000_oo00<00?ooool0=ooo0@000Ooo000Kool2000kool30004ool3001bool20002
ool01000ooooo`00@Ooo0P000Ooo000Kool00`00ooooo`0_ool40004ool3001`ool200001?oo0000
0?oo0P0000?oo`000000Booo0@000Ooo000Kool00`00ooooo`0Uool30004ool3001_ool01000oooo
o`000_oo00@00?ooool005Soo`40007oo`006ooo00<00?ooool06ooo0`001?oo0`00I_oo00<00?oo
ool00P0000Coo`00003oo`800004ool00000ool2001Tool10001ool001_oo`03003ooooo00goo`L0
00Coo`<005Ooo`800004ool00000ool200001?oo00000?oo0P000_oo00@00?ooool000;oo`03003o
oooo07Goo`40007oo`006ooo00<00?ooool00ooo1P00A?oo00@00?ooool000;oo`03003ooooo0080
0004ool00000ool200001?oo00000?oo0P000_oo00@00?ooool000;oo`04003ooooo002@ool10001
ool001_oo`8000Coo`X00004ool00000ool20002ool01000ooooo`000_oo00@00?ooool000;oo`80
0004ool00000ool200001?oo00000?oo0P000_oo00@00?ooool000;oo`04003ooooo0002ool20000
1?oo00000?oo0P0000?oo`0000000_oo00<00?ooool0/_oo0P000Ooo000Kool00`00ooooo`0;oolB
003Sool10001ool001_oo`03003ooooo01goo`X00=Woo`40007oo`006ooo00<00?ooool09ooo2`00
c_oo0@000Ooo000Kool00`00ooooo`0bool80036ool10001ool001_oo`03003ooooo03[oo`D00<7o
o`40007oo`006ooo0P00@?oo1`00^Ooo0P000Ooo000Kool00`00ooooo`16ool;002_ool10001ool0
01_oo`03003ooooo057oo`P00:Ooo`40007oo`006ooo00<00?ooool0FOoo1000Xooo0@000Ooo000K
ool00`00ooooo`1Mool5002Nool10001ool001_oo`8006?oo`H009Ooo`80007oo`006ooo00<00?oo
ool0J?oo1@00Tooo0@000Ooo000Kool00`00ooooo`1]ool5002>ool10001ool001_oo`03003ooooo
07;oo`@008[oo`40007oo`002?oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0M_oo1000
Q_oo0@000Ooo0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0N_oo1000P_oo0@00
0Ooo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00Oooo0`00O_oo0P000Ooo0007ool01000
ooooo`002?oo0`001Ooo00<00?ooool0POoo0`00O?oo0@000Ooo0007ool01000ooooo`002?oo00<0
0?ooool01Ooo00<00?ooool0Q?oo1000N?oo0@000Ooo0007ool01000ooooo`002?oo00<00?ooool0
1Ooo00<00?ooool0R?oo0`00MOoo0@000Ooo0008ool20009ool40004ool00`00ooooo`2;ool3001b
ool10001ool001_oo`03003ooooo08koo`@006koo`40007oo`006ooo0P00Tooo0`00J_oo0P000Ooo
000Kool00`00ooooo`2Eool2001Yool10001ool001_oo`03003ooooo09Ooo`8006Ooo`40007oo`00
6ooo00<00?ooool0VOoo0P00IOoo0@000Ooo000Kool00`00ooooo`2Kool2001Sool10001ool001_o
o`03003ooooo09goo`80067oo`40007oo`006ooo0P00X?oo0P00G_oo0P000Ooo000Kool00`00oooo
o`2Qool2001Mool10001ool001_oo`03003ooooo0:?oo`8005_oo`40007oo`006ooo00<00?ooool0
YOoo0P00FOoo0@000Ooo000Kool00`00ooooo`2Wool2001Gool10001ool001_oo`03003ooooo0:Wo
o`8005Goo`40007oo`006ooo0P00[?oo0P00D_oo0P000Ooo000Kool00`00ooooo`2]ool00`00oooo
o`1@ool10001ool001_oo`03003ooooo0:koo`80053oo`40007oo`006ooo00<00?ooool0/?oo0P00
C_oo0@000Ooo000Kool00`00ooooo`2bool00`00ooooo`1;ool10001ool001_oo`03003ooooo0;?o
o`8004_oo`40007oo`006ooo0P00]_oo00<00?ooool0Aooo0P000Ooo000Kool00`00ooooo`2fool0
0`00ooooo`17ool10001ool001_oo`03003ooooo0;Ooo`8004Ooo`40007oo`006ooo00<00?ooool0
^Ooo00<00?ooool0A?oo0@000Ooo000Cool40004ool00`00ooooo`2jool00`00ooooo`13ool10001
ool001Goo`03003ooooo00?oo`03003ooooo0;_oo`8004?oo`40007oo`005Ooo00<00?ooool00ooo
0P00__oo00<00?ooool0?ooo0P000Ooo000Eool00`00ooooo`03ool00`00ooooo`2nool00`00oooo
o`0oool10001ool001Goo`03003ooooo00?oo`03003ooooo0;ooo`8003ooo`40007oo`004ooo0`00
1Ooo00<00?ooool0`Ooo00<00?ooool0??oo0@000Ooo000Eool00`00ooooo`03ool00`00ooooo`32
ool00`00ooooo`0kool10001ool001_oo`03003ooooo0<?oo`03003ooooo03[oo`40007oo`006ooo
0P00aOoo0P00>Ooo0P000Ooo000Kool00`00ooooo`36ool00`00ooooo`0gool10001ool001_oo`03
003ooooo0<Ooo`03003ooooo03Koo`40007oo`006ooo00<00?ooool0b?oo00<00?ooool0=Ooo0@00
0Ooo000Kool00`00ooooo`39ool00`00ooooo`0dool10001ool001_oo`800<_oo`03003ooooo03;o
o`80007oo`006ooo00<00?ooool0booo00<00?ooool0<_oo0@000Ooo000Kool00`00ooooo`3<ool0
0`00ooooo`0aool10001ool001_oo`03003ooooo0<coo`03003ooooo037oo`40007oo`006ooo00<0
0?ooool0cOoo00<00?ooool0<?oo0@000Ooo000Kool00`00ooooo`3>ool00`00ooooo`0_ool10001
ool001_oo`800=3oo`03003ooooo02goo`80007oo`006ooo00<00?ooool0d?oo00<00?ooool0;Ooo
0@000Ooo000Kool00`00ooooo`3Aool00`00ooooo`0/ool10001ool001_oo`03003ooooo0=;oo`03
003ooooo02_oo`40007oo`006ooo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool00P000_oo
00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo
0P000Ooo000Koooo00050001ool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.21236, -1.58742, \
0.00635271, 0.0170129}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, PlotRange \[Rule] All]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0232028 0.606691 0.252605 0.186133 [
[.0232 -0.0125 -3 -9 ]
[.0232 -0.0125 3 0 ]
[.17488 -0.0125 -12 -9 ]
[.17488 -0.0125 12 0 ]
[.32655 -0.0125 -9 -9 ]
[.32655 -0.0125 9 0 ]
[.47822 -0.0125 -12 -9 ]
[.47822 -0.0125 12 0 ]
[.62989 -0.0125 -3 -9 ]
[.62989 -0.0125 3 0 ]
[.78157 -0.0125 -12 -9 ]
[.78157 -0.0125 12 0 ]
[.93324 -0.0125 -9 -9 ]
[.93324 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .06647 -12 -4.5 ]
[-0.0125 .06647 0 4.5 ]
[-0.0125 .15954 -24 -4.5 ]
[-0.0125 .15954 0 4.5 ]
[-0.0125 .2526 -6 -4.5 ]
[-0.0125 .2526 0 4.5 ]
[-0.0125 .34567 -18 -4.5 ]
[-0.0125 .34567 0 4.5 ]
[-0.0125 .43874 -6 -4.5 ]
[-0.0125 .43874 0 4.5 ]
[-0.0125 .5318 -18 -4.5 ]
[-0.0125 .5318 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.0232 0 m
.0232 .00625 L
s
[(0)] .0232 -0.0125 0 1 Mshowa
.17488 0 m
.17488 .00625 L
s
[(0.25)] .17488 -0.0125 0 1 Mshowa
.32655 0 m
.32655 .00625 L
s
[(0.5)] .32655 -0.0125 0 1 Mshowa
.47822 0 m
.47822 .00625 L
s
[(0.75)] .47822 -0.0125 0 1 Mshowa
.62989 0 m
.62989 .00625 L
s
[(1)] .62989 -0.0125 0 1 Mshowa
.78157 0 m
.78157 .00625 L
s
[(1.25)] .78157 -0.0125 0 1 Mshowa
.93324 0 m
.93324 .00625 L
s
[(1.5)] .93324 -0.0125 0 1 Mshowa
.125 Mabswid
.05354 0 m
.05354 .00375 L
s
.08387 0 m
.08387 .00375 L
s
.11421 0 m
.11421 .00375 L
s
.14454 0 m
.14454 .00375 L
s
.20521 0 m
.20521 .00375 L
s
.23554 0 m
.23554 .00375 L
s
.26588 0 m
.26588 .00375 L
s
.29621 0 m
.29621 .00375 L
s
.35688 0 m
.35688 .00375 L
s
.38722 0 m
.38722 .00375 L
s
.41755 0 m
.41755 .00375 L
s
.44789 0 m
.44789 .00375 L
s
.50856 0 m
.50856 .00375 L
s
.53889 0 m
.53889 .00375 L
s
.56922 0 m
.56922 .00375 L
s
.59956 0 m
.59956 .00375 L
s
.66023 0 m
.66023 .00375 L
s
.69056 0 m
.69056 .00375 L
s
.7209 0 m
.7209 .00375 L
s
.75123 0 m
.75123 .00375 L
s
.8119 0 m
.8119 .00375 L
s
.84224 0 m
.84224 .00375 L
s
.87257 0 m
.87257 .00375 L
s
.9029 0 m
.9029 .00375 L
s
.96357 0 m
.96357 .00375 L
s
.99391 0 m
.99391 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .06647 m
.00625 .06647 L
s
[(-1)] -0.0125 .06647 1 0 Mshowa
0 .15954 m
.00625 .15954 L
s
[(-0.5)] -0.0125 .15954 1 0 Mshowa
0 .2526 m
.00625 .2526 L
s
[(0)] -0.0125 .2526 1 0 Mshowa
0 .34567 m
.00625 .34567 L
s
[(0.5)] -0.0125 .34567 1 0 Mshowa
0 .43874 m
.00625 .43874 L
s
[(1)] -0.0125 .43874 1 0 Mshowa
0 .5318 m
.00625 .5318 L
s
[(1.5)] -0.0125 .5318 1 0 Mshowa
.125 Mabswid
0 .08509 m
.00375 .08509 L
s
0 .1037 m
.00375 .1037 L
s
0 .12231 m
.00375 .12231 L
s
0 .14093 m
.00375 .14093 L
s
0 .17815 m
.00375 .17815 L
s
0 .19677 m
.00375 .19677 L
s
0 .21538 m
.00375 .21538 L
s
0 .23399 m
.00375 .23399 L
s
0 .27122 m
.00375 .27122 L
s
0 .28983 m
.00375 .28983 L
s
0 .30844 m
.00375 .30844 L
s
0 .32706 m
.00375 .32706 L
s
0 .36428 m
.00375 .36428 L
s
0 .3829 m
.00375 .3829 L
s
0 .40151 m
.00375 .40151 L
s
0 .42012 m
.00375 .42012 L
s
0 .45735 m
.00375 .45735 L
s
0 .47596 m
.00375 .47596 L
s
0 .49458 m
.00375 .49458 L
s
0 .51319 m
.00375 .51319 L
s
0 .04786 m
.00375 .04786 L
s
0 .02925 m
.00375 .02925 L
s
0 .01063 m
.00375 .01063 L
s
0 .55042 m
.00375 .55042 L
s
0 .56903 m
.00375 .56903 L
s
0 .58764 m
.00375 .58764 L
s
0 .60626 m
.00375 .60626 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.0232 .61178 m
.0232 .61803 L
s
.17488 .61178 m
.17488 .61803 L
s
.32655 .61178 m
.32655 .61803 L
s
.47822 .61178 m
.47822 .61803 L
s
.62989 .61178 m
.62989 .61803 L
s
.78157 .61178 m
.78157 .61803 L
s
.93324 .61178 m
.93324 .61803 L
s
.125 Mabswid
.05354 .61428 m
.05354 .61803 L
s
.08387 .61428 m
.08387 .61803 L
s
.11421 .61428 m
.11421 .61803 L
s
.14454 .61428 m
.14454 .61803 L
s
.20521 .61428 m
.20521 .61803 L
s
.23554 .61428 m
.23554 .61803 L
s
.26588 .61428 m
.26588 .61803 L
s
.29621 .61428 m
.29621 .61803 L
s
.35688 .61428 m
.35688 .61803 L
s
.38722 .61428 m
.38722 .61803 L
s
.41755 .61428 m
.41755 .61803 L
s
.44789 .61428 m
.44789 .61803 L
s
.50856 .61428 m
.50856 .61803 L
s
.53889 .61428 m
.53889 .61803 L
s
.56922 .61428 m
.56922 .61803 L
s
.59956 .61428 m
.59956 .61803 L
s
.66023 .61428 m
.66023 .61803 L
s
.69056 .61428 m
.69056 .61803 L
s
.7209 .61428 m
.7209 .61803 L
s
.75123 .61428 m
.75123 .61803 L
s
.8119 .61428 m
.8119 .61803 L
s
.84224 .61428 m
.84224 .61803 L
s
.87257 .61428 m
.87257 .61803 L
s
.9029 .61428 m
.9029 .61803 L
s
.96357 .61428 m
.96357 .61803 L
s
.99391 .61428 m
.99391 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .06647 m
1 .06647 L
s
.99375 .15954 m
1 .15954 L
s
.99375 .2526 m
1 .2526 L
s
.99375 .34567 m
1 .34567 L
s
.99375 .43874 m
1 .43874 L
s
.99375 .5318 m
1 .5318 L
s
.125 Mabswid
.99625 .08509 m
1 .08509 L
s
.99625 .1037 m
1 .1037 L
s
.99625 .12231 m
1 .12231 L
s
.99625 .14093 m
1 .14093 L
s
.99625 .17815 m
1 .17815 L
s
.99625 .19677 m
1 .19677 L
s
.99625 .21538 m
1 .21538 L
s
.99625 .23399 m
1 .23399 L
s
.99625 .27122 m
1 .27122 L
s
.99625 .28983 m
1 .28983 L
s
.99625 .30844 m
1 .30844 L
s
.99625 .32706 m
1 .32706 L
s
.99625 .36428 m
1 .36428 L
s
.99625 .3829 m
1 .3829 L
s
.99625 .40151 m
1 .40151 L
s
.99625 .42012 m
1 .42012 L
s
.99625 .45735 m
1 .45735 L
s
.99625 .47596 m
1 .47596 L
s
.99625 .49458 m
1 .49458 L
s
.99625 .51319 m
1 .51319 L
s
.99625 .04786 m
1 .04786 L
s
.99625 .02925 m
1 .02925 L
s
.99625 .01063 m
1 .01063 L
s
.99625 .55042 m
1 .55042 L
s
.99625 .56903 m
1 .56903 L
s
.99625 .58764 m
1 .58764 L
s
.99625 .60626 m
1 .60626 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .2526 m
1 .2526 L
s
.5 Mabswid
[ .002 .01 ] 0 setdash
.02381 .29148 m
.02499 .29148 L
.02605 .29148 L
.02729 .29148 L
.02846 .29148 L
.03053 .29148 L
.03279 .29148 L
.03527 .29148 L
.0379 .29148 L
.04262 .29147 L
.05205 .29146 L
.06244 .29144 L
.07293 .29142 L
.0842 .29139 L
.10458 .29132 L
.12571 .29123 L
.1458 .29111 L
.184 .29084 L
.22464 .29047 L
.26377 .29001 L
.30535 .28942 L
.34542 .28874 L
.38397 .28795 L
.42497 .28698 L
.46446 .28587 L
.50243 .28463 L
.54286 .2831 L
.58177 .28138 L
.62313 .27925 L
.66297 .27686 L
.7013 .2742 L
.74208 .27095 L
.78135 .26742 L
.8191 .26373 L
.85931 .2597 L
.89799 .25616 L
.91764 .25469 L
.92803 .25404 L
.93913 .25347 L
.94919 .25307 L
.95833 .25281 L
.96274 .25272 L
.96506 .25268 L
.96752 .25265 L
.96967 .25263 L
.97089 .25262 L
.97203 .25262 L
.97308 .25261 L
.97404 .25261 L
.97514 .25261 L
Mistroke
.97619 .2526 L
Mfstroke
[ .02 .02 ] 0 setdash
.02381 .28768 m
.02499 .28767 L
.02605 .28767 L
.02729 .28766 L
.02846 .28764 L
.03053 .28761 L
.03279 .28757 L
.0379 .28742 L
.04262 .28725 L
.06244 .28628 L
.10458 .28365 L
.14393 .28097 L
.18177 .27824 L
.22205 .27514 L
.26083 .27193 L
.30205 .26818 L
.34176 .26417 L
.37995 .25984 L
.4206 .2546 L
.45973 .24879 L
.50131 .24157 L
.54137 .23334 L
.57993 .22395 L
.62093 .21194 L
.66042 .19794 L
.69839 .18168 L
.73881 .16076 L
.77772 .13665 L
.81908 .10681 L
.85893 .07539 L
.89726 .04621 L
.91664 .03363 L
.92694 .02796 L
.93804 .02283 L
.9483 .01911 L
.9528 .01783 L
.95772 .01666 L
.96238 .01581 L
.96672 .01523 L
.96912 .015 L
.9713 .01485 L
.9725 .01479 L
.97381 .01475 L
.97505 .01472 L
.97619 .01472 L
s
[ ] 0 setdash
.02381 .29151 m
.02499 .29149 L
.02605 .29149 L
.02729 .2915 L
.02846 .29152 L
.03053 .29155 L
.03279 .29159 L
.0379 .29174 L
.04262 .29191 L
.06244 .29292 L
.10458 .29567 L
.14393 .29855 L
.18177 .30154 L
.22205 .305 L
.26083 .30866 L
.30205 .31299 L
.34176 .31767 L
.37995 .32276 L
.4206 .32895 L
.45973 .33584 L
.50131 .3444 L
.54137 .35413 L
.57993 .36522 L
.62093 .37932 L
.66042 .39568 L
.69839 .41455 L
.73881 .43865 L
.77772 .46623 L
.81908 .50009 L
.85893 .53551 L
.89726 .56821 L
.91664 .58225 L
.92694 .58857 L
.93804 .59429 L
.9483 .59842 L
.9528 .59986 L
.95772 .60115 L
.96238 .6021 L
.96672 .60275 L
.96912 .603 L
.9713 .60317 L
.9725 .60323 L
.97381 .60328 L
.97505 .60331 L
.97619 .60332 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Pool2000Lool20004ool00`00ooooo`02ool40002ool3000Gool2
0004ool00`00ooooo`02ool3000Fool20004ool00`00ooooo`03ool00`00ooooo`02ool3000Kool4
000Kool40003ool00`00ooooo`02ool40002ool3000Eool40003ool00`00ooooo`02ool3000<ool0
01ooo`04003ooooo000Jool01000ooooo`002?oo00<00?ooool01_oo00<00?ooool04ooo00@00?oo
ool000_oo`03003ooooo01;oo`04003ooooo0009ool00`00ooooo`05ool00`00ooooo`0Jool00`00
ooooo`0Lool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`0Dool00`00ooooo`0:ool00`00
ooooo`09ool001ooo`04003ooooo000Jool01000ooooo`002Ooo00<00?ooool01Ooo00<00?ooool0
4ooo00@00?ooool000_oo`03003ooooo01;oo`04003ooooo000:ool00`00ooooo`04ool00`00oooo
o`0Jool00`00ooooo`0Lool00`00ooooo`08ool00`00ooooo`05ool00`00ooooo`0Dool00`00oooo
o`0:ool00`00ooooo`09ool001ooo`04003ooooo000Jool01000ooooo`002_oo00@00?ooooooo`<0
01Koo`04003ooooo0008ool3000Eool01000ooooo`002_oo00@00?ooooooo`<001goo`03003ooooo
01coo`03003ooooo00Woo`04003oooooool3000Gool00`00ooooo`07ool3000<ool001ooo`04003o
oooo000Jool01000ooooo`002ooo00@00?ooool001Soo`04003ooooo0008ool00`00ooooo`0Eool0
1000ooooo`002ooo00@00?ooool001ooo`03003ooooo01coo`03003ooooo00[oo`04003ooooo000I
ool00`00ooooo`07ool00`00ooooo`0<ool001ooo`04003ooooo000Jool01000ooooo`002?oo00@0
0?ooool000;oo`03003ooooo01Koo`04003ooooo0008ool00`00ooooo`0Eool01000ooooo`002?oo
00@00?ooool000;oo`03003ooooo01_oo`<001coo`<000Woo`04003ooooo0002ool00`00ooooo`0E
ool30009ool00`00ooooo`0<ool0023oo`8001coo`8000[oo`8000?oo`@001Koo`8000Woo`@001Go
o`8000Woo`@000;oo`@001coo`03003ooooo01coo`03003ooooo00Soo`8000?oo`@001Koo`03003o
oooo00Ooo`@000_oo`00oooo8Ooo003ooolQool00?ooob7oo`006oooo`001@000Ooo000Kool00`00
ooooo`03ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool20001ool001_oo`03003ooooo00?oo`03003o
oooo02Coo`03003ooooo02Goo`03003ooooo02Coo`03003ooooo02Coo`03003ooooo02Goo`03003o
oooo02Coo`03003ooooo00koo`40007oo`006ooo0P00oooo0Ooo0P000Ooo000Kool00`00ooooo`3f
ool50005ool10001ool001_oo`03003ooooo0?7oo`03003ooooo00coo`40007oo`006ooo00<00?oo
ool0kooo0P003ooo0@000Ooo000Kool00`00ooooo`3]ool2000Aool10001ool001_oo`800>koo`03
003ooooo00ooo`80007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo000Kool00`00ooooo`3Xool0
0`00ooooo`0Eool10001ool001_oo`03003ooooo0>Ooo`03003ooooo01Koo`40007oo`006ooo0P00
i_oo0P006?oo0P000Ooo000Kool00`00ooooo`3Tool00`00ooooo`0Iool10001ool001_oo`03003o
oooo0?ooo`7oo`40007oo`004ooo10001?oo00<00?ooool0oooo0Ooo0@000Ooo000Eool00`00oooo
o`03ool00`00ooooo`3Pool00`00ooooo`0Mool10001ool001Goo`03003ooooo00?oo`800=ooo`80
01ooo`80007oo`003Ooo10001?oo00<00?ooool00ooo00<00?ooool0gOoo00<00?ooool08?oo0@00
0Ooo000Eool00`00ooooo`03ool00`00ooooo`3Lool00`00ooooo`0Qool10001ool001?oo`<000Go
o`03003ooooo0?ooo`7oo`40007oo`005Ooo00<00?ooool00ooo00<00?ooool0oooo0Ooo0@000Ooo
000Kool2003oool1ool20001ool001_oo`03003ooooo0=Soo`03003ooooo02Goo`40007oo`006ooo
00<00?ooool0e_oo0P00:?oo0@000Ooo000Kool00`00ooooo`3Eool00`00ooooo`0Xool10001ool0
01_oo`03003ooooo0=Coo`03003ooooo02Woo`40007oo`006ooo0P00oooo0Ooo0P000Ooo000Kool0
0`00ooooo`3oool1ool10001ool001_oo`03003ooooo0<ooo`03003ooooo02koo`40007oo`006ooo
00<00?ooool0cOoo0P00<Ooo0@000Ooo000Kool00`00ooooo`3<ool00`00ooooo`0aool10001ool0
01_oo`800<coo`03003ooooo037oo`80007oo`006ooo00<00?ooool0oooo0Ooo0@000Ooo000Kool0
0`00ooooo`3oool1ool10001ool001_oo`03003ooooo0<Ooo`03003ooooo03Koo`40007oo`006ooo
00<00?ooool0aOoo0P00>Ooo0@000Ooo000Kool20035ool00`00ooooo`0hool20001ool001_oo`03
003ooooo0<?oo`03003ooooo03[oo`40007oo`002?oo0P001?oo00<00?ooool00_oo0`001Ooo00<0
0?ooool0oooo0Ooo0@000Ooo0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0oooo
0Ooo0@000Ooo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00_ooo00<00?ooool0?_oo0P00
0Ooo00000ooo000000020002ool01000ooooo`002?oo0`001Ooo00<00?ooool0_?oo0P00@_oo0@00
0Ooo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0^_oo0P00A?oo0@000Ooo0007
ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0oooo0Ooo0@000Ooo0008ool20009ool4
0004ool00`00ooooo`3oool1ool10001ool001_oo`800;Goo`8004Woo`80007oo`006ooo00<00?oo
ool0/Ooo0`00C?oo0@000Ooo000Kool00`00ooooo`2`ool00`00ooooo`1=ool10001ool001_oo`03
003ooooo0?ooo`7oo`40007oo`006ooo00<00?ooool0Z_oo0`00Dooo0@000Ooo000Kool2002Yool2
001Eool20001ool001_oo`03003ooooo0:Ooo`03003ooooo05Koo`40007oo`006ooo00<00?ooool0
oooo0Ooo0@000Ooo000Kool00`00ooooo`2Pool3001Mool10001ool001_oo`03003ooooo09goo`<0
063oo`40007oo`006ooo0P00oooo0Ooo0P000Ooo000Kool00`00ooooo`2Fool3001Wool10001ool0
01_oo`03003ooooo09?oo`<006[oo`40007oo`006ooo00<00?ooool0SOoo0P00LOoo0@000Ooo000K
ool00`00ooooo`2;ool2001cool10001ool001_oo`8008[oo`8007Coo`80007oo`006ooo00<00?oo
ool0P_oo0`00Nooo0@000Ooo000Kool00`00ooooo`1oool3001nool10001ool001Coo`8000Goo`03
003ooooo07Goo`H008Goo`40007oo`004ooo00@00?ooool000Coo`03003ooooo06goo`<008Soo`03
003ooooo00Goo`40007oo`004ooo00@00?ooool000Coool000D0007oo`004ooo00@00?ooool000Co
o`03003ooooo063oo`H007Goo`8000;oo`04003ooooo0002ool00`00ooooo`0Hool10001ool001?o
o`04003ooooo0004ool00`00ooooo`1Iool3001fool200001?oo00000?oo0P009_oo0@000Ooo000C
ool01000ooooo`001?oo00<00?ooool0C_oo10001?oo0`00KOoo00@00?ooool000;oo`800003ool0
000002ooo`40007oo`005?oo0P001Ooo0P00AOoo0`001?oo0`00K_oo0P000_oo00@00?ooool000;o
o`03003ooooo03Koo`80007oo`006ooo00<00?ooool0=ooo1P001?oo0`00JOoo00<00?ooool00P00
00Coo`00003oo`800003ool0000004Goo`40007oo`006ooo00<00?ooool0;?oo1`00J_oo0P000_oo
00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`1@ool10001ool001_oo`03003ooooo01_o
o`<000Coo`H006?oo`04003ooooo0002ool00`00ooooo`0200001?oo00000?oo0P00I?oo0@000Ooo
000Kool00`00ooooo`0=ool60005ool3001=ool20002ool01000ooooo`000_oo00@00?ooool000;o
o`800004ool00000ool200001?oo00000?oo0P000_oo00<00?ooool0MOoo0@000Ooo000Kool20004
ool6000Oool01@00ool00000ool00P0000?oo`0000000_oo00@00?ooool000;oo`04003ooooo0002
ool00`00ooooo`0200001?oo00000?oo0P0000?oo`0000000_oo00@00?ooool000;oo`04003ooooo
0002ool200001?oo00000?oo0P0000?oo`000000V?oo0P000Ooo000Kool00`00ooooo`03ool@0000
1?oo00000?oo0P000_oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`3Fool10001ool0
01_oo`03003ooooo01?ooa@00=Woo`40007oo`006ooo00<00?ooool09ooo2P00cooo0@000Ooo000K
ool00`00ooooo`0aool:0035ool10001ool001_oo`8003coo`/00;Woo`80007oo`006ooo00<00?oo
ool0A_oo2P00/?oo0@000Ooo000Kool00`00ooooo`1@ool8002Xool10001ool001_oo`03003ooooo
05Soo`D00:?oo`40007oo`006ooo00<00?ooool0GOoo2000Vooo0@000Ooo000Kool2001Vool8002B
ool20001ool001_oo`03003ooooo06goo`D008koo`40007oo`006ooo00<00?ooool0L_oo1@00ROoo
0@000Ooo0008ool20004ool00`00ooooo`02ool30005ool00`00ooooo`1gool60023ool10001ool0
00Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`1mool4001oool10001ool000Ooo`04
003ooooo000;ool00`00ooooo`02ool20022ool3001kool20001ool000Ooo`04003ooooo0008ool3
0005ool00`00ooooo`24ool4001hool10001ool000Ooo`04003ooooo0008ool00`00ooooo`05ool0
0`00ooooo`28ool3001eool10001ool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00oooo
o`2;ool3001bool10001ool000Soo`8000Woo`@000Coo`03003ooooo08koo`@006koo`40007oo`00
6ooo0P00Tooo0`00J_oo0P000Ooo000Kool00`00ooooo`2Eool4001Wool10001ool001_oo`03003o
oooo09Woo`@006?oo`40007oo`006ooo00<00?ooool0WOoo0P00HOoo0@000Ooo000Kool2002Pool2
001Nool20001ool001_oo`03003ooooo0:7oo`8005goo`40007oo`006ooo00<00?ooool0Xooo0P00
Fooo0@000Ooo000Kool00`00ooooo`2Uool2001Iool10001ool001_oo`03003ooooo0:Ooo`8005Oo
o`40007oo`006ooo0P00Z_oo0P00E?oo0P000Ooo000Kool00`00ooooo`2[ool2001Cool10001ool0
01_oo`03003ooooo0:goo`80057oo`40007oo`006ooo00<00?ooool0[ooo0P00Cooo0@000Ooo000K
ool00`00ooooo`2aool2001=ool10001ool001_oo`800;Coo`8004[oo`80007oo`006ooo00<00?oo
ool0]Ooo0P00BOoo0@000Ooo000Kool00`00ooooo`2gool20017ool10001ool001?oo`@000Coo`03
003ooooo0;Woo`8004Goo`40007oo`005Ooo00<00?ooool00ooo00<00?ooool0^ooo0P00@ooo0@00
0Ooo000Eool00`00ooooo`03ool2002nool00`00ooooo`0oool20001ool001Goo`03003ooooo00?o
o`03003ooooo0;koo`80043oo`40007oo`005Ooo00<00?ooool00ooo00<00?ooool0`?oo00<00?oo
ool0?Ooo0@000Ooo000Cool30005ool00`00ooooo`31ool00`00ooooo`0lool10001ool001Goo`03
003ooooo00?oo`03003ooooo0<;oo`8003coo`40007oo`006ooo0P00aOoo00<00?ooool0>?oo0P00
0Ooo000Kool00`00ooooo`35ool2000iool10001ool001_oo`03003ooooo0<Ooo`03003ooooo03Ko
o`40007oo`006ooo00<00?ooool0b?oo00<00?ooool0=Ooo0@000Ooo000Kool2003:ool00`00oooo
o`0cool20001ool001_oo`03003ooooo0<[oo`03003ooooo03?oo`40007oo`006ooo00<00?ooool0
booo00<00?ooool0<_oo0@000Ooo000Kool00`00ooooo`3<ool2000bool10001ool001_oo`03003o
oooo0<koo`03003ooooo02ooo`40007oo`006ooo0P00d?oo00<00?ooool0;Ooo0P000Ooo000Kool0
0`00ooooo`3@ool00`00ooooo`0]ool10001ool001_oo`03003ooooo0=7oo`03003ooooo02coo`40
007oo`006ooo00<00?ooool0d_oo00<00?ooool0:ooo0@000Ooo000Kool00`00ooooo`3Cool2000[
ool10001ool001_oo`800=Koo`03003ooooo02Ooo`80007oo`006ooo00<00?ooool0e_oo00<00?oo
ool09ooo0@000Ooo000Kool00`00ooooo`3Gool00`00ooooo`0Vool10001ool000Ooo`@000?oo`03
003ooooo00;oo`<000Goo`03003ooooo0=Soo`03003ooooo02Goo`40007oo`002Ooo00<00?ooool0
2_oo00<00?ooool00_oo00<00?ooool0fOoo0P009Ooo0@000Ooo0009ool00`00ooooo`0:ool00`00
ooooo`02ool2003Lool00`00ooooo`0Qool20001ool000Woo`03003ooooo00Ooo`<000Goo`03003o
oooo0=coo`03003ooooo027oo`40007oo`002Ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?oo
ool0gOoo00<00?ooool08?oo0@000Ooo0007ool30009ool00`00ooooo`05ool00`00ooooo`3Nool2
000Pool10001ool000Woo`03003ooooo00Ooo`@000Coo`03003ooooo0>3oo`03003ooooo01goo`40
007oo`006ooo0P00h_oo00<00?ooool06ooo0P000Ooo000Kool00`00ooooo`3Rool00`00ooooo`0K
ool10001ool001_oo`03003ooooo0>?oo`8001_oo`40007oo`006ooo00<00?ooool0iOoo00<00?oo
ool06?oo0@000Ooo000Kool00`00ooooo`3Vool00`00ooooo`0Gool10001ool001_oo`800>Soo`03
003ooooo01Goo`80007oo`006ooo00<00?ooool0j?oo0P005_oo0@000Ooo000Kool00`00ooooo`3Z
ool00`00ooooo`0Cool10001ool001_oo`03003ooooo0>_oo`03003ooooo01;oo`40007oo`006ooo
0P00kOoo00<00?ooool04?oo0P000Ooo000Kool00`00ooooo`3]ool2000Aool10001ool001_oo`03
003ooooo0>ooo`<000koo`40007oo`006ooo00<00?ooool0l_oo0P003?oo0@000Ooo000Kool00`00
ooooo`3dool50007ool10001ool001_oo`800?[oo`8000Coo`80007oo`006ooo00<00?ooool0oooo
0Ooo0@000Ooo000Kool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`04
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool20001ool001_o
ool000D0007oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.211225, -1.65433, \
0.00634867, 0.0206932}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<masses2new.eps\>", %]\)], "Input"],

Cell[BoxData[
    \("masses2new.eps"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["static limit, small a", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ma = \(-xi\)/6 \((1 + Cos[t])\)\)], "Input"],

Cell[BoxData[
    \(\(-\(1\/6\)\)\ xi\ \((1 + Cos[t])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(mb = \((1 + xi/6 \((3\ Cos[t] - 1)\))\)\)], "Input"],

Cell[BoxData[
    \(1 + 1\/6\ xi\ \((\(-1\) + 3\ Cos[t])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify\ [mc = xi/3*\((1 - 1/2 \((1 + Cos[t])\))\)]\)], "Input"],

Cell[BoxData[
    \(\(-\(1\/6\)\)\ xi\ \((\(-1\) + Cos[t])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[\((ma + mb + mc)\)^2 - 4\ mb\ \((ma + mc)\)]\)], "Input"],

Cell[BoxData[
    \(1\/36\ \((\(-6\) + xi - 5\ xi\ Cos[t])\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[\((ma + mb + mc)\)]\)], "Input"],

Cell[BoxData[
    \(1\/6\ \((6 - xi + xi\ Cos[t])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%133 + 1/6*\((\(-6\) + xi - 5\ xi\ Cos[t])\)]\)], "Input"],

Cell[BoxData[
    \(\(-\(2\/3\)\)\ xi\ Cos[t]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%133 - 1/6*\((\(-6\) + xi - 5\ xi\ Cos[t])\)]\)], "Input"],

Cell[BoxData[
    \(2 - xi\/3 + xi\ Cos[t]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["kx=0 prop modes", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?WSalphakx0korr\)\)], "Input"],

Cell["Global`WSalphakx0korr", "Print",
  CellTags->"Info3280046340-8820928"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(WSalphakx0korr[eta_, 
                      a_] = \(-\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\
\[ImaginaryI]\ \@\(1 + a\)\ \((\((\(-1\) - eta\^2 - 
                                    a\ \((\(-1\) + \((6 + 
                                        a)\)\ eta\^2 + \((\(-1\) + 
                                        a)\)\ eta\^4)\))\)\ Log[
                                  1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                                    a\ \((\(-1\) + \((6 + 
                                        a)\)\ eta\^2 + \((\(-1\) + 
                                        a)\)\ eta\^4)\))\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \
\@a\ \((\(-1\) + eta\^2)\)\ \((2 + 2\ a\ eta\^2 + 
                                    2\ \((1 + 
                                        a)\)\ eta\ \((2\ \[ImaginaryI]\ \[Pi]\
\ If[Im[eta] < 0 && Re[eta] < 1, 1, 0] + Log[\(-1\) + eta] - 
                                        Log[1 + eta])\))\))\))\)\)\)\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "WSalphakx0korr"],
      Editable->False]], "Print",
  CellTags->"Info3280046340-8820928"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSalphakx0korr[1.01, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.5401678969358953`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSalphakx0korr[0.99, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(0.5705583412172746`\)\(\[InvisibleSpace]\)\) - 
      0.011162176831109621`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[1.01, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(1.5420735311056473`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WSbetakx0korr[0.99, 1]\)], "Input"],

Cell[BoxData[
    \(\(\(1.4970386196908454`\)\(\[InvisibleSpace]\)\) - 
      1.0995024635347261`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(kx0korroma[k_, a_] := 
      om /. FindRoot[\(-om^2\) + k^2 + alphakx0korr[om/k, a] \[Equal] 0, {om, 
            k + 10^\((\(-5\))\), 3/2*k}, MaxIterations \[Rule] 15]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{kx0korroma[k, 10], k}, {k, 1/100, 3}, 
      PlotStyle \[Rule] {{}, Dashing[{0.05, 0.05}]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147151 0.189672 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .10955 -18 -4.5 ]
[.01131 .10955 0 4.5 ]
[.01131 .20439 -6 -4.5 ]
[.01131 .20439 0 4.5 ]
[.01131 .29922 -18 -4.5 ]
[.01131 .29922 0 4.5 ]
[.01131 .39406 -6 -4.5 ]
[.01131 .39406 0 4.5 ]
[.01131 .4889 -18 -4.5 ]
[.01131 .4889 0 4.5 ]
[.01131 .58373 -6 -4.5 ]
[.01131 .58373 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .10955 m
.03006 .10955 L
s
[(0.5)] .01131 .10955 1 0 Mshowa
.02381 .20439 m
.03006 .20439 L
s
[(1)] .01131 .20439 1 0 Mshowa
.02381 .29922 m
.03006 .29922 L
s
[(1.5)] .01131 .29922 1 0 Mshowa
.02381 .39406 m
.03006 .39406 L
s
[(2)] .01131 .39406 1 0 Mshowa
.02381 .4889 m
.03006 .4889 L
s
[(2.5)] .01131 .4889 1 0 Mshowa
.02381 .58373 m
.03006 .58373 L
s
[(3)] .01131 .58373 1 0 Mshowa
.125 Mabswid
.02381 .03368 m
.02756 .03368 L
s
.02381 .05265 m
.02756 .05265 L
s
.02381 .07162 m
.02756 .07162 L
s
.02381 .09058 m
.02756 .09058 L
s
.02381 .12852 m
.02756 .12852 L
s
.02381 .14749 m
.02756 .14749 L
s
.02381 .16645 m
.02756 .16645 L
s
.02381 .18542 m
.02756 .18542 L
s
.02381 .22335 m
.02756 .22335 L
s
.02381 .24232 m
.02756 .24232 L
s
.02381 .26129 m
.02756 .26129 L
s
.02381 .28026 m
.02756 .28026 L
s
.02381 .31819 m
.02756 .31819 L
s
.02381 .33716 m
.02756 .33716 L
s
.02381 .35612 m
.02756 .35612 L
s
.02381 .37509 m
.02756 .37509 L
s
.02381 .41303 m
.02756 .41303 L
s
.02381 .43199 m
.02756 .43199 L
s
.02381 .45096 m
.02756 .45096 L
s
.02381 .46993 m
.02756 .46993 L
s
.02381 .50786 m
.02756 .50786 L
s
.02381 .52683 m
.02756 .52683 L
s
.02381 .5458 m
.02756 .5458 L
s
.02381 .56476 m
.02756 .56476 L
s
.02381 .6027 m
.02756 .6027 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02698 .13186 m
.02816 .13188 L
.02922 .13191 L
.03162 .13198 L
.03383 .13207 L
.03593 .13218 L
.04063 .13249 L
.04573 .13295 L
.056 .1342 L
.06549 .13574 L
.08588 .14015 L
.10749 .14621 L
.14519 .15926 L
.18535 .17549 L
.22399 .19259 L
.26508 .21187 L
.30465 .23124 L
.34272 .25044 L
.38323 .27136 L
.42223 .2919 L
.46367 .31406 L
.5036 .3357 L
.54203 .35674 L
.58289 .37933 L
.62225 .40126 L
.66009 .42248 L
.70038 .44521 L
.73916 .46719 L
.78039 .49067 L
.8201 .51337 L
.8583 .53529 L
.89895 .55869 L
.93808 .58127 L
.97571 .60304 L
.97619 .60332 L
s
[ .05 .05 ] 0 setdash
.02698 .01661 m
.06549 .03962 L
.10749 .06471 L
.14693 .08827 L
.18485 .11093 L
.22523 .13506 L
.26409 .15828 L
.3054 .18296 L
.3452 .20673 L
.38348 .22961 L
.42421 .25394 L
.46343 .27737 L
.50114 .2999 L
.54129 .32389 L
.57994 .34698 L
.62102 .37153 L
.6606 .39518 L
.69867 .41792 L
.73918 .44212 L
.77817 .46542 L
.81962 .49018 L
.85955 .51404 L
.89797 .537 L
.93884 .56142 L
.97619 .58373 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool003Woo`8000Coo`03003ooooo00;oo`<002;oo`@0027oo`@000?oo`03003ooooo00;oo`<002?o
o`@0027oo`@000?oo`03003ooooo00;oo`<002?oo`8000Ooo`00>?oo00@00?ooool000_oo`03003o
oooo027oo`03003ooooo02;oo`03003ooooo00[oo`03003ooooo023oo`03003ooooo02;oo`03003o
oooo00coo`03003ooooo01ooo`04003ooooo0006ool003Soo`04003ooooo000;ool00`00ooooo`0Q
ool00`00ooooo`0Rool00`00ooooo`0:ool00`00ooooo`0Qool00`00ooooo`0Rool00`00ooooo`0;
ool00`00ooooo`0Rool00`00ooooo`04ool003Soo`04003ooooo0008ool3000Tool00`00ooooo`0R
ool00`00ooooo`07ool3000Uool00`00ooooo`0Rool00`00ooooo`07ool3000Sool20007ool003So
o`04003ooooo0008ool00`00ooooo`0Tool00`00ooooo`0Rool00`00ooooo`07ool00`00ooooo`0V
ool00`00ooooo`0Rool00`00ooooo`06ool00`00ooooo`0Uool00`00ooooo`04ool003Soo`04003o
oooo0008ool00`00ooooo`0Rool3000Rool30009ool00`00ooooo`0Sool01000ooooo`008Ooo00@0
0?ooool000Soo`03003ooooo02;oo`04003ooooo0006ool001Goo`03003ooooo027oo`8000Woo`@0
02?oo`03003ooooo02;oo`03003ooooo00Ooo`@002?oo`8002?oo`8000Woo`@002;oo`8000Ooo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
3_ooo`004`00000Eool20007ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00
ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00
ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00
ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00
ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00
ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00
ooooo`06ool00`00ooooo`05ool001Goo`04003oo`00000Wool00`00ooooo`0Xool00`00ooooo`0X
ool00`00ooooo`0Yool00`00ooooo`0Xool00`00ooooo`0Xool00`00ooooo`05ool001Goo`04003o
ooooool2003oool6ool001Goo`03003ooooo00?oo`03003ooooo0?ooo`?oo`005Ooo0P001Ooo0P00
oooo0ooo000Eool00`00ooooo`06ool2003oool1ool001Goo`03003ooooo00Soo`03003ooooo0?go
o`005Ooo00<00?ooool02Ooo00<00?ooool0o?oo000Eool00`00ooooo`3oool9ool001Goo`800?oo
o`[oo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?oo
o`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`0Eool00`00ooooo`3`ool001Goo`80
01Ooo`800?3oo`005Ooo00<00?ooool06?oo0P00k_oo000Eool00`00ooooo`0Jool2003/ool001Go
o`03003ooooo01coo`800>[oo`005Ooo00<00?ooool07_oo00<00?ooool0iooo000Eool2000Pool2
003Wool001Goo`03003ooooo027oo`03003ooooo0>Coo`005Ooo00<00?ooool0oooo2Ooo0002ool2
0004ool00`00ooooo`02ool30005ool00`00ooooo`3oool9ool00005ool00?ooool0000;ool00`00
ooooo`02ool00`00ooooo`3oool9ool00005ool00?ooool0000;ool00`00ooooo`02ool3003oool9
ool00005ool00?ooool00008ool30005ool00`00ooooo`3oool9ool00005ool00?ooool00008ool0
0`00ooooo`05ool00`00ooooo`3oool9ool00005ool00?ooool00008ool00`00ooooo`05ool00`00
ooooo`0/ool00`00ooooo`3Iool000;oo`8000Woo`@000Coo`03003ooooo02goo`800=Woo`005Ooo
0P00<?oo0P00eooo000Eool8000/ool2003Eool001Goo`03003ooooo00Goo`H002Soo`800=?oo`00
5Ooo00<00?ooool02ooo1@009Ooo00<00?ooool0d?oo000Eool00`00ooooo`0@ool3000Sool2003@
ool001Goo`8001Coo`<002;oo`03003ooooo0<goo`005Ooo00<00?ooool05_oo0`00kooo000Eool0
0`00ooooo`0Iool4003[ool001Goo`03003ooooo01goo`<00>Soo`005Ooo00<00?ooool08?oo0P00
i_oo000Eool2000Sool2003Tool001Goo`03003ooooo02Coo`800>;oo`005Ooo00<00?ooool09_oo
0P007?oo00<00?ooool0`Ooo000Eool00`00ooooo`0Xool2000Kool20031ool001Goo`03003ooooo
02[oo`8001_oo`800;ooo`005Ooo00<00?ooool0;?oo0P006ooo0P00_Ooo000Eool2000_ool2000K
ool00`00ooooo`2jool001Goo`03003ooooo033oo`8001[oo`800;[oo`005Ooo00<00?ooool0<_oo
0`006Ooo00<00?ooool0]ooo000=ool40004ool00`00ooooo`0eool2000Hool00`00ooooo`2fool0
00ooo`03003ooooo00?oo`03003ooooo03Ooo`800<ooo`003ooo00<00?ooool00ooo0`00>Ooo0`00
c?oo000?ool00`00ooooo`03ool00`00ooooo`0lool2003:ool000ooo`03003ooooo00?oo`03003o
oooo03koo`800<Soo`003Ooo0`001Ooo00<00?ooool0@?oo0P00a_oo000?ool00`00ooooo`03ool0
0`00ooooo`12ool20034ool001Goo`8004Goo`8001Goo`03003ooooo0:[oo`005Ooo00<00?ooool0
A_oo0P005?oo0P00Z_oo000Eool00`00ooooo`18ool3000Cool2002Xool001Goo`03003ooooo04_o
o`8001?oo`800:Koo`005Ooo00<00?ooool0COoo0P004ooo0P00Y?oo000Eool2001@ool2000Cool2
002Rool001Goo`03003ooooo057oo`8001?oo`800:3oo`005Ooo00<00?ooool0Dooo0P00/ooo000E
ool00`00ooooo`1Eool2002aool001Goo`03003ooooo05Ooo`800:ooo`005Ooo0P00F_oo0P00[Ooo
000Eool00`00ooooo`1Kool2002[ool001Goo`03003ooooo05goo`800:Woo`005Ooo00<00?ooool0
Gooo00<00?ooool04Ooo00<00?ooool0T_oo000Eool00`00ooooo`1Pool2000Bool00`00ooooo`2A
ool001Goo`8006?oo`80017oo`03003ooooo093oo`005Ooo00<00?ooool0I?oo00<00?ooool03ooo
0P00T?oo000Eool00`00ooooo`1Uool2000Aool2002>ool001Goo`03003ooooo06Ooo`80017oo`80
08coo`0000?oo`0000000P000ooo00<00?ooool00_oo0`001Ooo00<00?ooool0JOoo00<00?ooool0
4?oo0P00R_oo0003ool00`00ooooo`0:ool00`00ooooo`02ool00`00ooooo`1Zool2000Bool00`00
ooooo`27ool000?oo`03003ooooo00[oo`03003ooooo00;oo`<006coo`8009[oo`000ooo00<00?oo
ool01ooo0`001Ooo00<00?ooool0K_oo0P00V?oo0003ool00`00ooooo`07ool00`00ooooo`05ool0
0`00ooooo`1`ool2002Fool00004ool000000009ool00`00ooooo`05ool00`00ooooo`1bool2002D
ool000?oo`03003ooooo00Ooo`@000Coo`03003ooooo07Coo`8009;oo`005Ooo0P00Mooo0P00T?oo
000Eool00`00ooooo`1hool2000@ool00`00ooooo`1kool001Goo`03003ooooo07[oo`8000ooo`80
07_oo`005Ooo00<00?ooool0O?oo0P003ooo00<00?ooool0N?oo000Eool00`00ooooo`1nool2000>
ool00`00ooooo`1gool001Goo`80087oo`8000goo`8007Ooo`005Ooo00<00?ooool0P_oo0P003Ooo
00<00?ooool0M?oo000Eool00`00ooooo`24ool2000<ool2001dool001Goo`03003ooooo08Koo`80
00coo`<0077oo`005Ooo00<00?ooool0R?oo0P00O_oo000Eool2002;ool00`00ooooo`1kool001Go
o`03003ooooo08_oo`8007_oo`005Ooo00<00?ooool0SOoo0P00NOoo000Eool00`00ooooo`2?ool2
001gool001Goo`03003ooooo097oo`8007Goo`005Ooo0P00U?oo0P003?oo00<00?ooool0I?oo000E
ool00`00ooooo`2Eool00`00ooooo`0:ool2001Tool001Goo`03003ooooo09Koo`8000coo`03003o
oooo067oo`003Ooo10001?oo00<00?ooool0V?oo0P002ooo0P00HOoo000=ool00`00ooooo`05ool0
0`00ooooo`2Jool2000;ool00`00ooooo`1Nool000koo`03003ooooo00Coo`<009coo`8000[oo`80
05koo`003ooo00<00?ooool00ooo00<00?ooool0W_oo0P002_oo0P00G?oo000@ool00`00ooooo`02
ool00`00ooooo`2Pool00`00ooooo`09ool2001Jool000goo`04003ooooo0004ool00`00ooooo`2Q
ool2001Uool000koo`8000Goo`03003ooooo0:?oo`8006?oo`005Ooo00<00?ooool0YOoo00<00?oo
ool0H?oo000Eool2002Wool2001Pool001Goo`03003ooooo0:Soo`8005koo`005Ooo00<00?ooool0
Z_oo00<00?ooool0Fooo000Eool00`00ooooo`2[ool2000<ool00`00ooooo`1<ool001Goo`03003o
oooo0:goo`8000_oo`8004coo`005Ooo0P00/?oo0P002ooo0P00B_oo000Eool00`00ooooo`2aool2
000;ool20018ool001Goo`03003ooooo0;?oo`8000_oo`03003ooooo04Goo`005Ooo00<00?ooool0
]Ooo00<00?ooool02Ooo00<00?ooool0A?oo000Eool00`00ooooo`2fool2000:ool20014ool001Go
o`800;Woo`8000[oo`03003ooooo047oo`005Ooo00<00?ooool0^_oo0P00C?oo000Eool00`00oooo
o`2lool2001:ool001Goo`03003ooooo0;koo`8004Soo`005Ooo00<00?ooool0`?oo00<00?ooool0
AOoo000Eool20032ool20015ool001Goo`03003ooooo0<?oo`03003ooooo04;oo`005Ooo00<00?oo
ool0a?oo0P002_oo00<00?ooool0=Ooo00000ooo000000020003ool00`00ooooo`02ool30005ool0
0`00ooooo`36ool20009ool00`00ooooo`0dool00003ool00?oo00goo`03003ooooo00;oo`03003o
oooo0<Soo`03003ooooo00Ooo`8003Coo`000_oo00<00?ooool02ooo00<00?ooool00_oo0`00bOoo
0P002Ooo00<00?ooool0<Ooo0003ool00`00ooooo`07ool30005ool00`00ooooo`3;ool00`00oooo
o`07ool2000aool000Coo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo0<coo`8000Woo`80
02ooo`0000Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo0<koo`8000Woo`8002goo`00
0_oo0P002Ooo10001?oo00<00?ooool0d?oo0P002Ooo0P00:ooo000Eool2003Cool2000dool001Go
o`03003ooooo0=Coo`8003;oo`005Ooo00<00?ooool0e_oo00<00?ooool0;ooo000Eool00`00oooo
o`3Gool2000_ool001Goo`03003ooooo0=Woo`8002goo`005Ooo0P00g?oo00<00?ooool0:_oo000E
ool00`00ooooo`3Lool20009ool00`00ooooo`0Nool001Goo`03003ooooo0=koo`8000Soo`8001ko
o`005Ooo00<00?ooool0h?oo00<00?ooool01ooo0P007?oo000Eool00`00ooooo`3Qool20009ool2
000Jool001Goo`03003ooooo0>?oo`8000Woo`03003ooooo01Ooo`005Ooo0P00i_oo0P002?oo0P00
5ooo000Eool00`00ooooo`3Wool20008ool2000Eool001Goo`03003ooooo0>Woo`8000Soo`03003o
oooo01;oo`005Ooo00<00?ooool0jooo00<00?ooool06_oo000Eool00`00ooooo`3/ool2000Jool0
01Goo`800>ooo`8001Soo`005Ooo00<00?ooool0l?oo0P005_oo000Eool00`00ooooo`3bool2000D
ool000koo`8000Goo`03003ooooo0?Coo`8001;oo`003Ooo00@00?ooool000Coo`03003ooooo0?Ko
o`03003ooooo00Koo`03003ooooo00Koo`004?oo00<00?ooool00_oo0`00mooo0P001ooo00<00?oo
ool01Ooo000>ool20005ool00`00ooooo`3iool2000=ool0013oo`03003ooooo00;oo`03003ooooo
0?_oo`03003ooooo00[oo`003Ooo00@00?ooool000Coo`03003ooooo0?coo`8000[oo`003_oo0P00
1Ooo00<00?ooool0o_oo0P002?oo000Eool2003oool2ool00`00ooooo`05ool001Goo`03003ooooo
0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo
0?ooo`Woo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -0.244071, \
0.0115777, 0.019378}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(looks\ nice . \ Below\ Lightcone\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[\((\(-om^2\) + k^2 + alphakx0korr[om/k, 10])\)] /. k \[Rule] 2, {om, 
        0.01, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.312155 0.0658038 [
[.18254 .29966 -9 -9 ]
[.18254 .29966 9 0 ]
[.34127 .29966 -3 -9 ]
[.34127 .29966 3 0 ]
[.5 .29966 -9 -9 ]
[.5 .29966 9 0 ]
[.65873 .29966 -3 -9 ]
[.65873 .29966 3 0 ]
[.81746 .29966 -9 -9 ]
[.81746 .29966 9 0 ]
[.97619 .29966 -3 -9 ]
[.97619 .29966 3 0 ]
[.01131 .04894 -12 -4.5 ]
[.01131 .04894 0 4.5 ]
[.01131 .18055 -12 -4.5 ]
[.01131 .18055 0 4.5 ]
[.01131 .44376 -6 -4.5 ]
[.01131 .44376 0 4.5 ]
[.01131 .57537 -6 -4.5 ]
[.01131 .57537 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .31216 m
.18254 .31841 L
s
[(0.5)] .18254 .29966 0 1 Mshowa
.34127 .31216 m
.34127 .31841 L
s
[(1)] .34127 .29966 0 1 Mshowa
.5 .31216 m
.5 .31841 L
s
[(1.5)] .5 .29966 0 1 Mshowa
.65873 .31216 m
.65873 .31841 L
s
[(2)] .65873 .29966 0 1 Mshowa
.81746 .31216 m
.81746 .31841 L
s
[(2.5)] .81746 .29966 0 1 Mshowa
.97619 .31216 m
.97619 .31841 L
s
[(3)] .97619 .29966 0 1 Mshowa
.125 Mabswid
.05556 .31216 m
.05556 .31591 L
s
.0873 .31216 m
.0873 .31591 L
s
.11905 .31216 m
.11905 .31591 L
s
.15079 .31216 m
.15079 .31591 L
s
.21429 .31216 m
.21429 .31591 L
s
.24603 .31216 m
.24603 .31591 L
s
.27778 .31216 m
.27778 .31591 L
s
.30952 .31216 m
.30952 .31591 L
s
.37302 .31216 m
.37302 .31591 L
s
.40476 .31216 m
.40476 .31591 L
s
.43651 .31216 m
.43651 .31591 L
s
.46825 .31216 m
.46825 .31591 L
s
.53175 .31216 m
.53175 .31591 L
s
.56349 .31216 m
.56349 .31591 L
s
.59524 .31216 m
.59524 .31591 L
s
.62698 .31216 m
.62698 .31591 L
s
.69048 .31216 m
.69048 .31591 L
s
.72222 .31216 m
.72222 .31591 L
s
.75397 .31216 m
.75397 .31591 L
s
.78571 .31216 m
.78571 .31591 L
s
.84921 .31216 m
.84921 .31591 L
s
.88095 .31216 m
.88095 .31591 L
s
.9127 .31216 m
.9127 .31591 L
s
.94444 .31216 m
.94444 .31591 L
s
.25 Mabswid
0 .31216 m
1 .31216 L
s
.02381 .04894 m
.03006 .04894 L
s
[(-4)] .01131 .04894 1 0 Mshowa
.02381 .18055 m
.03006 .18055 L
s
[(-2)] .01131 .18055 1 0 Mshowa
.02381 .44376 m
.03006 .44376 L
s
[(2)] .01131 .44376 1 0 Mshowa
.02381 .57537 m
.03006 .57537 L
s
[(4)] .01131 .57537 1 0 Mshowa
.125 Mabswid
.02381 .08184 m
.02756 .08184 L
s
.02381 .11474 m
.02756 .11474 L
s
.02381 .14765 m
.02756 .14765 L
s
.02381 .21345 m
.02756 .21345 L
s
.02381 .24635 m
.02756 .24635 L
s
.02381 .27925 m
.02756 .27925 L
s
.02381 .34506 m
.02756 .34506 L
s
.02381 .37796 m
.02756 .37796 L
s
.02381 .41086 m
.02756 .41086 L
s
.02381 .47666 m
.02756 .47666 L
s
.02381 .50957 m
.02756 .50957 L
s
.02381 .54247 m
.02756 .54247 L
s
.02381 .01604 m
.02756 .01604 L
s
.02381 .60827 m
.02756 .60827 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02698 .32467 m
.02816 .32487 L
.02922 .3251 L
.03162 .3258 L
.03383 .32668 L
.03593 .3277 L
.04063 .33067 L
.04573 .33493 L
.056 .34652 L
.06549 .36037 L
.10749 .44225 L
.12773 .48336 L
.1467 .51769 L
.16604 .54674 L
.18441 .56826 L
.19406 .57723 L
.20459 .5853 L
.2141 .59113 L
.22306 .59546 L
.228 .5974 L
.23334 .59916 L
.23778 .60037 L
.24266 .60144 L
.24796 .60231 L
.25085 .60267 L
.25355 .60293 L
.25485 .60303 L
.25608 .60312 L
.2572 .60318 L
.25838 .60323 L
.25967 .60327 L
.26087 .6033 L
.26215 .60332 L
.26288 .60332 L
.26357 .60332 L
.26474 .6033 L
.26599 .60328 L
.26717 .60324 L
.26825 .6032 L
.27088 .60306 L
.27337 .60287 L
.27832 .60237 L
.28368 .60163 L
.29328 .59983 L
.30212 .59769 L
.32083 .59184 L
.3414 .58368 L
.3816 .56383 L
.4203 .5412 L
.46144 .51438 L
Mistroke
.50106 .48636 L
.53918 .45761 L
.57974 .42514 L
.61879 .39201 L
.66029 .35427 L
.70028 .31666 L
.73875 .27958 L
.77967 .23858 L
.81908 .19736 L
.85697 .15603 L
.89731 .11016 L
.93615 .06413 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo3ooo000?ool00`00ooooo`3oool?
ool000ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo1ooo00<00?ooool01Ooo000?ool2
003oool7ool00`00ooooo`06ool000ooo`03003ooooo0?ooo`Goo`03003ooooo00Ooo`003ooo00<0
0?ooool0oooo1Ooo00<00?ooool01ooo000?ool00`00ooooo`3oool4ool00`00ooooo`08ool000oo
o`03003ooooo0?ooo`?oo`03003ooooo00Woo`003ooo00<00?ooool0oooo0_oo00<00?ooool02_oo
000?ool00`00ooooo`3oool2ool00`00ooooo`0:ool000Woo`03003ooooo00?oo`03003ooooo0?oo
o`7oo`03003ooooo00_oo`002Ooo00<00?ooool00ooo00<00?ooool0oooo00<00?ooool03?oo0006
ool50004ool3003nool00`00ooooo`0=ool00003ool0000000800005ool00?ooool00005ool00`00
ooooo`3mool00`00ooooo`0>ool000Ooo`03003oo`0000Goo`03003ooooo0?goo`03003ooooo00ko
o`002?oo0P001Ooo00<00?ooool0o?oo00<00?ooool03ooo0009ool00`00ooooo`03ool00`00oooo
o`3kool00`00ooooo`0@ool000ooo`03003ooooo0?[oo`03003ooooo017oo`003ooo00<00?ooool0
nOoo00<00?ooool04_oo000?ool00`00ooooo`3hool00`00ooooo`0Cool000ooo`03003ooooo0?So
o`03003ooooo01?oo`003ooo0P00n?oo00<00?ooool05?oo000?ool00`00ooooo`3fool00`00oooo
o`0Eool000ooo`03003ooooo0?Goo`03003ooooo01Koo`003ooo00<00?ooool0m?oo00<00?ooool0
5ooo000?ool00`00ooooo`3cool00`00ooooo`0Hool000ooo`03003ooooo0??oo`03003ooooo01So
o`003ooo00<00?ooool0l_oo00<00?ooool06Ooo000?ool00`00ooooo`3aool00`00ooooo`0Jool0
00ooo`03003ooooo0?3oo`03003ooooo01_oo`003ooo0P00l?oo00<00?ooool07?oo000?ool00`00
ooooo`3^ool00`00ooooo`0Mool000ooo`03003ooooo0>goo`03003ooooo01koo`003ooo00<00?oo
ool0k?oo00<00?ooool07ooo000?ool00`00ooooo`3[ool00`00ooooo`0Pool000ooo`03003ooooo
0>_oo`03003ooooo023oo`003ooo00<00?ooool0j_oo00<00?ooool08Ooo000?ool00`00ooooo`3Y
ool00`00ooooo`0Rool000ooo`03003ooooo0>Soo`03003ooooo02?oo`003ooo0P00j?oo00<00?oo
ool09?oo000?ool00`00ooooo`3Vool00`00ooooo`0Uool000ooo`03003ooooo0>Goo`03003ooooo
02Koo`003ooo00<00?ooool0i?oo00<00?ooool09ooo000?ool00`00ooooo`3Sool00`00ooooo`0X
ool000ooo`03003ooooo0>;oo`03003ooooo02Woo`003ooo00<00?ooool0hOoo00<00?ooool0:_oo
0007ool40004ool00`00ooooo`3Pool00`00ooooo`0[ool000Ooo`03003ooooo00Goo`03003ooooo
0>3oo`03003ooooo02_oo`002?oo00<00?ooool01?oo0`00gooo00<00?ooool0;?oo00000ooo0000
00020004ool00`00ooooo`03ool00`00ooooo`3Nool00`00ooooo`0]ool000[oo`03003ooooo00;o
o`03003ooooo0=goo`03003ooooo02koo`001ooo00@00?ooool000Coo`03003ooooo0=coo`03003o
oooo02ooo`002?oo0P001Ooo00<00?ooool0fooo00<00?ooool0<?oo000?ool00`00ooooo`3Jool0
0`00ooooo`0aool000ooo`03003ooooo0=Woo`03003ooooo03;oo`003ooo00<00?ooool0f?oo00<0
0?ooool0<ooo000?ool00`00ooooo`3Gool00`00ooooo`0dool000ooo`800=Ooo`03003ooooo03Go
o`003ooo00<00?ooool0eOoo00<00?ooool0=_oo000?ool00`00ooooo`3Dool00`00ooooo`0gool0
00ooo`03003ooooo0=?oo`03003ooooo03Soo`003ooo00<00?ooool0d_oo00<00?ooool0>Ooo000?
ool00`00ooooo`3Aool00`00ooooo`0jool000ooo`03003ooooo0=3oo`03003ooooo03_oo`003ooo
00<00?ooool0cooo00<00?ooool0??oo000?ool00`00ooooo`3>ool00`00ooooo`0mool000ooo`03
003ooooo0<goo`03003ooooo03koo`003ooo0P00cOoo00<00?ooool0?ooo000?ool00`00ooooo`3;
ool00`00ooooo`10ool000ooo`03003ooooo0<[oo`03003ooooo047oo`003ooo00<00?ooool0b_oo
00<00?ooool0@Ooo000?ool00`00ooooo`39ool00`00ooooo`12ool000ooo`03003ooooo0<Soo`03
003ooooo04?oo`003ooo00<00?ooool0aooo00<00?ooool0A?oo000?ool00`00ooooo`0Rool20004
ool00`00ooooo`02ool3000Sool4000Rool40003ool00`00ooooo`02ool3000Tool4000Fool00`00
ooooo`09ool40003ool00`00ooooo`02ool3000Tool20007ool000ooo`03003ooooo027oo`04003o
oooo000;ool00`00ooooo`0Rool00`00ooooo`0Sool00`00ooooo`0:ool00`00ooooo`0Qool00`00
ooooo`0Fool00`00ooooo`0:ool00`00ooooo`0<ool00`00ooooo`0Pool01000ooooo`001_oo000?
ool2000Rool01000ooooo`002ooo00<00?ooool08_oo00<00?ooool08ooo00<00?ooool02_oo00<0
0?ooool08_oo00<00?ooool05?oo00<00?ooool03?oo00<00?ooool02ooo00<00?ooool08ooo00<0
0?ooool01?oo000?ool00`00ooooo`0Qool01000ooooo`002?oo0`009Ooo00<00?ooool08ooo00<0
0?ooool01ooo0`009_oo00<00?ooool04_oo00<00?ooool03_oo00<00?ooool01ooo0`009?oo0P00
1ooo000?ool00`00ooooo`0Qool01000ooooo`002?oo00<00?ooool09Ooo00<00?ooool08ooo00<0
0?ooool01ooo00<00?ooool09ooo00<00?ooool04?oo00<00?ooool04?oo00<00?ooool01_oo00<0
0?ooool09_oo00<00?ooool01?oo000?ool00`00ooooo`0Qool01000ooooo`002?oo00<00?ooool0
8ooo0`008ooo0`002Ooo00<00?ooool09?oo00@00?ooool0017oo`03003ooooo00koo`04003ooooo
0008ool00`00ooooo`0Sool01000ooooo`001_oo000?ool00`00ooooo`0Rool20009ool4000Tool0
0`00ooooo`0Sool00`00ooooo`07ool4000Tool2000Aool00`00ooooo`0@ool20009ool4000Sool2
0007ool000ooo`03003ooooo0;koo`8004koo`003ooo00<00?ooool0_Ooo00<00?ooool0C_oo000?
ool00`00ooooo`2lool00`00ooooo`1?ool000ooo`03003ooooo0;_oo`03003ooooo053oo`002?oo
o`006@00000?ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool01000ooooo`001Ooo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo000?ool00`00ooooo`0Yool00`00ooooo`0Yool00`00oooo
o`0Yool00`00ooooo`0Zool00`00ooooo`07ool00`00ooooo`0Oool00`00ooooo`0Yool00`00oooo
o`05ool000ooo`800;Soo`03003ooooo05Coo`003ooo1000]Ooo00<00?ooool0EOoo000?ool01000
oooooooo0P00/_oo00<00?ooool0E_oo000?ool00`00ooooo`03ool00`00ooooo`2^ool00`00oooo
o`1Gool000ooo`03003ooooo00Coo`03003ooooo0:coo`03003ooooo05Soo`003ooo00<00?ooool0
1Ooo00<00?ooool0Z_oo00<00?ooool0FOoo000?ool20007ool00`00ooooo`2Xool00`00ooooo`1J
ool000ooo`03003ooooo00Ooo`03003ooooo0:Koo`03003ooooo05_oo`003ooo00<00?ooool01ooo
00<00?ooool0YOoo00<00?ooool0G?oo000?ool00`00ooooo`08ool00`00ooooo`2Sool00`00oooo
o`1Mool000ooo`03003ooooo00Woo`03003ooooo0:7oo`03003ooooo05koo`003ooo00<00?ooool0
2Ooo00<00?ooool0Wooo0P00HOoo000?ool00`00ooooo`0:ool00`00ooooo`2Mool00`00ooooo`1Q
ool000ooo`03003ooooo00[oo`03003ooooo09coo`03003ooooo06;oo`003ooo00<00?ooool02ooo
00<00?ooool0V_oo00<00?ooool0Hooo000?ool2000<ool00`00ooooo`2Iool00`00ooooo`1Tool0
00ooo`03003ooooo00coo`03003ooooo09Koo`8006Ooo`003ooo00<00?ooool03?oo00<00?ooool0
UOoo00<00?ooool0Iooo000?ool00`00ooooo`0=ool00`00ooooo`2Cool00`00ooooo`1Xool000oo
o`03003ooooo00goo`03003ooooo09;oo`03003ooooo06Woo`003ooo00<00?ooool03_oo00<00?oo
ool0T?oo00<00?ooool0J_oo000?ool00`00ooooo`0>ool00`00ooooo`2?ool00`00ooooo`1[ool0
00ooo`03003ooooo00ooo`03003ooooo08goo`03003ooooo06coo`003ooo00<00?ooool03ooo00<0
0?ooool0Rooo0P00Kooo000?ool2000Aool00`00ooooo`29ool00`00ooooo`1_ool000ooo`03003o
oooo013oo`03003ooooo08Soo`03003ooooo073oo`003ooo00<00?ooool04Ooo00<00?ooool0Q_oo
00<00?ooool0LOoo000?ool00`00ooooo`0Aool00`00ooooo`25ool00`00ooooo`1bool000ooo`03
003ooooo01;oo`03003ooooo08?oo`03003ooooo07?oo`003ooo00<00?ooool04_oo00<00?ooool0
POoo0P00M_oo000?ool00`00ooooo`0Cool00`00ooooo`1oool00`00ooooo`1fool000ooo`03003o
oooo01?oo`03003ooooo07koo`03003ooooo07Ooo`001ooo10001?oo00<00?ooool05?oo00<00?oo
ool0Nooo0P00N_oo0007ool00`00ooooo`05ool00`00ooooo`0Dool00`00ooooo`1jool00`00oooo
o`1jool000Soo`03003ooooo00Coo`<001Coo`03003ooooo07Woo`03003ooooo07_oo`002Ooo00<0
0?ooool00ooo00<00?ooool05Ooo00<00?ooool0M_oo0P00O_oo000:ool00`00ooooo`02ool00`00
ooooo`0Eool00`00ooooo`1eool00`00ooooo`1nool000Ooo`04003ooooo0004ool00`00ooooo`0F
ool00`00ooooo`1cool00`00ooooo`1oool000Soo`8000Goo`03003ooooo01Koo`03003ooooo07;o
o`03003ooooo083oo`003ooo00<00?ooool05ooo00<00?ooool0Kooo0P00Pooo000?ool00`00oooo
o`0Gool00`00ooooo`1^ool00`00ooooo`23ool000ooo`03003ooooo01Soo`03003ooooo06coo`03
003ooooo08Coo`003ooo00<00?ooool06?oo00<00?ooool0Jooo00<00?ooool0QOoo000?ool2000J
ool00`00ooooo`1Xool20028ool000ooo`03003ooooo01Woo`03003ooooo06Ooo`03003ooooo08So
o`003ooo00<00?ooool06_oo00<00?ooool0IOoo00<00?ooool0ROoo000?ool00`00ooooo`0Kool0
0`00ooooo`1Rool2002<ool000ooo`03003ooooo01_oo`03003ooooo067oo`03003ooooo08coo`00
3ooo00<00?ooool07?oo00<00?ooool0Gooo00<00?ooool0SOoo000?ool00`00ooooo`0Lool00`00
ooooo`1Mool2002@ool000ooo`03003ooooo01goo`03003ooooo05_oo`03003ooooo093oo`003ooo
00<00?ooool07Ooo00<00?ooool0F_oo00<00?ooool0TOoo000?ool2000Oool00`00ooooo`1Gool2
002Dool000ooo`03003ooooo01koo`03003ooooo05Koo`03003ooooo09Coo`003ooo00<00?ooool0
7ooo00<00?ooool0Dooo0P00Uooo000?ool00`00ooooo`0Pool00`00ooooo`1Aool00`00ooooo`2G
ool000ooo`03003ooooo023oo`03003ooooo04ooo`8009[oo`003ooo00<00?ooool08Ooo00<00?oo
ool0COoo00<00?ooool0V_oo000?ool00`00ooooo`0Rool00`00ooooo`1:ool2002Mool000ooo`03
003ooooo02?oo`03003ooooo04Soo`03003ooooo09goo`003ooo00<00?ooool08ooo00<00?ooool0
A_oo0P00X?oo000?ool2000Uool00`00ooooo`14ool00`00ooooo`2Pool000ooo`03003ooooo02Go
o`03003ooooo047oo`800:?oo`003ooo00<00?ooool09_oo00<00?ooool0?_oo0P00YOoo000?ool0
0`00ooooo`0Wool00`00ooooo`0kool2002Wool000ooo`03003ooooo02Ooo`03003ooooo03Woo`80
0:Woo`003ooo00<00?ooool0:?oo00<00?ooool0=_oo0P00Zooo000?ool00`00ooooo`0Yool00`00
ooooo`0cool2002]ool000Woo`03003ooooo00?oo`03003ooooo02[oo`03003ooooo033oo`800:oo
o`002Ooo00<00?ooool00ooo00<00?ooool0:ooo00<00?ooool0;Ooo0P00/Ooo0006ool50004ool3
000[ool00`00ooooo`0[ool2002cool000Koo`04003ooooo0005ool00`00ooooo`0/ool00`00oooo
o`0Xool2002eool000Ooo`03003oo`0000Goo`03003ooooo02goo`8002Koo`800;Ooo`002?oo0P00
1Ooo00<00?ooool0;ooo0P008_oo0P00^Ooo0009ool00`00ooooo`03ool00`00ooooo`0aool2000N
ool2002kool000ooo`03003ooooo03?oo`03003ooooo01Ooo`@00;goo`003ooo00<00?ooool0=?oo
0`004_oo1000`Ooo000?ool00`00ooooo`0gool4000:ool40035ool000ooo`03003ooooo03_oo`X0
0<Woo`003ooo0P00oooo4?oo000?ool00`00ooooo`3oool?ool000ooo`03003ooooo0?ooo`ooo`00
3ooo00<00?ooool0oooo3ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.17529, -4.88132, \
0.0113251, 0.0546362}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(Re[\((\(-om^2\) + k^2 + WSalphakx0korr[om/k, 10])\)] /. 
          om \[Rule] Om - I\ 0.1\) /. k \[Rule] 2, {Om, 0.01, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.29792 0.0627876 [
[.18254 .28542 -9 -9 ]
[.18254 .28542 9 0 ]
[.34127 .28542 -3 -9 ]
[.34127 .28542 3 0 ]
[.5 .28542 -9 -9 ]
[.5 .28542 9 0 ]
[.65873 .28542 -3 -9 ]
[.65873 .28542 3 0 ]
[.81746 .28542 -9 -9 ]
[.81746 .28542 9 0 ]
[.97619 .28542 -3 -9 ]
[.97619 .28542 3 0 ]
[.01131 .04677 -12 -4.5 ]
[.01131 .04677 0 4.5 ]
[.01131 .17235 -12 -4.5 ]
[.01131 .17235 0 4.5 ]
[.01131 .4235 -6 -4.5 ]
[.01131 .4235 0 4.5 ]
[.01131 .54907 -6 -4.5 ]
[.01131 .54907 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .29792 m
.18254 .30417 L
s
[(0.5)] .18254 .28542 0 1 Mshowa
.34127 .29792 m
.34127 .30417 L
s
[(1)] .34127 .28542 0 1 Mshowa
.5 .29792 m
.5 .30417 L
s
[(1.5)] .5 .28542 0 1 Mshowa
.65873 .29792 m
.65873 .30417 L
s
[(2)] .65873 .28542 0 1 Mshowa
.81746 .29792 m
.81746 .30417 L
s
[(2.5)] .81746 .28542 0 1 Mshowa
.97619 .29792 m
.97619 .30417 L
s
[(3)] .97619 .28542 0 1 Mshowa
.125 Mabswid
.05556 .29792 m
.05556 .30167 L
s
.0873 .29792 m
.0873 .30167 L
s
.11905 .29792 m
.11905 .30167 L
s
.15079 .29792 m
.15079 .30167 L
s
.21429 .29792 m
.21429 .30167 L
s
.24603 .29792 m
.24603 .30167 L
s
.27778 .29792 m
.27778 .30167 L
s
.30952 .29792 m
.30952 .30167 L
s
.37302 .29792 m
.37302 .30167 L
s
.40476 .29792 m
.40476 .30167 L
s
.43651 .29792 m
.43651 .30167 L
s
.46825 .29792 m
.46825 .30167 L
s
.53175 .29792 m
.53175 .30167 L
s
.56349 .29792 m
.56349 .30167 L
s
.59524 .29792 m
.59524 .30167 L
s
.62698 .29792 m
.62698 .30167 L
s
.69048 .29792 m
.69048 .30167 L
s
.72222 .29792 m
.72222 .30167 L
s
.75397 .29792 m
.75397 .30167 L
s
.78571 .29792 m
.78571 .30167 L
s
.84921 .29792 m
.84921 .30167 L
s
.88095 .29792 m
.88095 .30167 L
s
.9127 .29792 m
.9127 .30167 L
s
.94444 .29792 m
.94444 .30167 L
s
.25 Mabswid
0 .29792 m
1 .29792 L
s
.02381 .04677 m
.03006 .04677 L
s
[(-4)] .01131 .04677 1 0 Mshowa
.02381 .17235 m
.03006 .17235 L
s
[(-2)] .01131 .17235 1 0 Mshowa
.02381 .4235 m
.03006 .4235 L
s
[(2)] .01131 .4235 1 0 Mshowa
.02381 .54907 m
.03006 .54907 L
s
[(4)] .01131 .54907 1 0 Mshowa
.125 Mabswid
.02381 .07816 m
.02756 .07816 L
s
.02381 .10956 m
.02756 .10956 L
s
.02381 .14095 m
.02756 .14095 L
s
.02381 .20374 m
.02756 .20374 L
s
.02381 .23513 m
.02756 .23513 L
s
.02381 .26653 m
.02756 .26653 L
s
.02381 .32931 m
.02756 .32931 L
s
.02381 .36071 m
.02756 .36071 L
s
.02381 .3921 m
.02756 .3921 L
s
.02381 .45489 m
.02756 .45489 L
s
.02381 .48628 m
.02756 .48628 L
s
.02381 .51768 m
.02756 .51768 L
s
.02381 .01538 m
.02756 .01538 L
s
.02381 .58046 m
.02756 .58046 L
s
.02381 .61186 m
.02756 .61186 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02698 .19283 m
.02816 .19321 L
.02922 .19366 L
.03162 .19503 L
.03383 .19673 L
.03593 .19871 L
.04063 .20446 L
.04573 .21264 L
.056 .23466 L
.06549 .26045 L
.10749 .40118 L
.12773 .46396 L
.1467 .51198 L
.16487 .54709 L
.18441 .57379 L
.19448 .58362 L
.20548 .59171 L
.2105 .5946 L
.21586 .59715 L
.22055 .59898 L
.2255 .60053 L
.22805 .60118 L
.23084 .60179 L
.23341 .60225 L
.23575 .6026 L
.23852 .60292 L
.24004 .60305 L
.24149 .60315 L
.24282 .60322 L
.24405 .60327 L
.24469 .60329 L
.2454 .60331 L
.24615 .60332 L
.24684 .60332 L
.24802 .60331 L
.2493 .60329 L
.24995 .60327 L
.25064 .60325 L
.25189 .60319 L
.25431 .60304 L
.25652 .60285 L
.26188 .60221 L
.26694 .60137 L
.27663 .59923 L
.28689 .59628 L
.30528 .58953 L
.34304 .57154 L
.38325 .54852 L
.42194 .52403 L
.46308 .4962 L
Mistroke
.50271 .4679 L
.54083 .43935 L
.58139 .40749 L
.62044 .37529 L
.66194 .33645 L
.70192 .3011 L
.7404 .26578 L
.78131 .22663 L
.82072 .18726 L
.85862 .14777 L
.89896 .10394 L
.93779 .05996 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo3ooo000?ool00`00ooooo`3oool?
ool000ooo`03003ooooo0?ooo`ooo`003ooo0P00oooo2?oo00<00?ooool01Ooo000?ool00`00oooo
o`3oool6ool00`00ooooo`06ool000ooo`03003ooooo0?ooo`Goo`03003ooooo00Ooo`003ooo00<0
0?ooool0oooo1?oo00<00?ooool02?oo000?ool00`00ooooo`3oool4ool00`00ooooo`08ool000oo
o`03003ooooo0?ooo`?oo`03003ooooo00Woo`003ooo00<00?ooool0oooo0_oo00<00?ooool02_oo
0009ool00`00ooooo`03ool00`00ooooo`3oool1ool00`00ooooo`0;ool000Woo`03003ooooo00?o
o`03003ooooo0?ooo`03003ooooo00coo`001_oo1@001?oo0`00o_oo00<00?ooool03Ooo00000ooo
0000000200001Ooo003ooooo00001Ooo00<00?ooool0o_oo00<00?ooool03Ooo0007ool00`00ool0
0005ool00`00ooooo`3mool00`00ooooo`0>ool000Soo`8000Goo`03003ooooo0?coo`03003ooooo
00ooo`002Ooo00<00?ooool00ooo00<00?ooool0nooo00<00?ooool04?oo000?ool00`00ooooo`3j
ool00`00ooooo`0Aool000ooo`03003ooooo0?Woo`03003ooooo01;oo`003ooo00<00?ooool0n?oo
00<00?ooool04ooo000?ool00`00ooooo`3gool00`00ooooo`0Dool000ooo`800?Ooo`03003ooooo
01Goo`003ooo00<00?ooool0m_oo00<00?ooool05Ooo000?ool00`00ooooo`3eool00`00ooooo`0F
ool000ooo`03003ooooo0?Coo`03003ooooo01Ooo`003ooo00<00?ooool0looo00<00?ooool06?oo
000?ool00`00ooooo`3bool00`00ooooo`0Iool000ooo`03003ooooo0?7oo`03003ooooo01[oo`00
3ooo00<00?ooool0l?oo00<00?ooool06ooo000?ool00`00ooooo`3_ool00`00ooooo`0Lool000oo
o`800>ooo`03003ooooo01goo`003ooo00<00?ooool0kOoo00<00?ooool07_oo000?ool00`00oooo
o`3/ool00`00ooooo`0Oool000ooo`03003ooooo0>_oo`03003ooooo023oo`003ooo00<00?ooool0
jooo00<00?ooool08?oo000?ool00`00ooooo`3Zool00`00ooooo`0Qool000ooo`03003ooooo0>Wo
o`03003ooooo02;oo`003ooo00<00?ooool0j?oo00<00?ooool08ooo000?ool2003Xool00`00oooo
o`0Tool000ooo`03003ooooo0>Koo`03003ooooo02Goo`003ooo00<00?ooool0iOoo00<00?ooool0
9_oo000?ool00`00ooooo`3Tool00`00ooooo`0Wool000ooo`03003ooooo0>?oo`03003ooooo02So
o`003ooo00<00?ooool0h_oo00<00?ooool0:Ooo000?ool00`00ooooo`3Qool00`00ooooo`0Zool0
00Ooo`@000Coo`03003ooooo0>3oo`03003ooooo02_oo`001ooo00<00?ooool01Ooo00<00?ooool0
h?oo00<00?ooool0:ooo0008ool00`00ooooo`04ool3003Oool00`00ooooo`0/ool00003ool00000
008000Coo`03003ooooo00?oo`03003ooooo0=koo`03003ooooo02goo`002_oo00<00?ooool00_oo
00<00?ooool0gOoo00<00?ooool0;_oo0007ool01000ooooo`001?oo00<00?ooool0g?oo00<00?oo
ool0;ooo0008ool20005ool00`00ooooo`3Kool00`00ooooo`0`ool000ooo`03003ooooo0=[oo`03
003ooooo037oo`003ooo0`00fOoo00<00?ooool0<_oo000?ool01000ooooo`00eooo00<00?ooool0
<ooo000?ool01@00oooooooo0000eOoo00<00?ooool0=?oo000?ool20003ool00`00ooooo`3Aool0
0`00ooooo`0eool000ooo`03003ooooo00;oo`03003ooooo0=3oo`03003ooooo03Koo`003ooo00<0
0?ooool00ooo00<00?ooool0c_oo00<00?ooool0=ooo000?ool00`00ooooo`03ool00`00ooooo`3=
ool00`00ooooo`0hool000ooo`03003ooooo00Coo`03003ooooo0<_oo`03003ooooo03Woo`003ooo
00<00?ooool01?oo00<00?ooool0b_oo00<00?ooool0>_oo000?ool00`00ooooo`05ool00`00oooo
o`38ool00`00ooooo`0kool000ooo`03003ooooo00Goo`03003ooooo0<Ooo`03003ooooo03coo`00
3ooo0P001ooo00<00?ooool0aOoo00<00?ooool0?Ooo000?ool00`00ooooo`06ool00`00ooooo`34
ool00`00ooooo`0nool000ooo`03003ooooo00Ooo`03003ooooo0<;oo`03003ooooo03ooo`003ooo
00<00?ooool01ooo00<00?ooool0`?oo0P00@_oo000?ool00`00ooooo`07ool00`00ooooo`2oool0
0`00ooooo`12ool000ooo`03003ooooo00Soo`03003ooooo0;goo`03003ooooo04?oo`003ooo00<0
0?ooool02?oo00<00?ooool0_?oo00<00?ooool0A?oo000?ool00`00ooooo`09ool00`00ooooo`0F
ool20004ool00`00ooooo`02ool3000Sool4000Rool40003ool00`00ooooo`02ool3000Tool4000F
ool00`00ooooo`09ool40003ool00`00ooooo`02ool3000Tool20007ool000ooo`03003ooooo00Wo
o`03003ooooo01Goo`04003ooooo000;ool00`00ooooo`0Rool00`00ooooo`0Sool00`00ooooo`0:
ool00`00ooooo`0Qool00`00ooooo`0Fool00`00ooooo`0:ool00`00ooooo`0<ool00`00ooooo`0P
ool01000ooooo`001_oo000?ool2000:ool00`00ooooo`0Eool01000ooooo`002ooo00<00?ooool0
8_oo00<00?ooool08ooo00<00?ooool02_oo00<00?ooool08_oo00<00?ooool05?oo00<00?ooool0
3?oo00<00?ooool02ooo00<00?ooool08ooo00<00?ooool01?oo000?ool00`00ooooo`0:ool00`00
ooooo`0Dool01000ooooo`002?oo0`009Ooo00<00?ooool08ooo00<00?ooool01ooo0`009_oo00<0
0?ooool04_oo00<00?ooool03_oo00<00?ooool01ooo0`009?oo0P001ooo000?ool00`00ooooo`0:
ool00`00ooooo`0Dool01000ooooo`002?oo00<00?ooool09Ooo00<00?ooool08ooo00<00?ooool0
1ooo00<00?ooool09ooo00<00?ooool04?oo00<00?ooool04?oo00<00?ooool01_oo00<00?ooool0
9_oo00<00?ooool01?oo000?ool00`00ooooo`0:ool00`00ooooo`0Dool01000ooooo`002?oo00<0
0?ooool08ooo0`008ooo0`002Ooo00<00?ooool09?oo00@00?ooool0017oo`03003ooooo00koo`04
003ooooo0008ool00`00ooooo`0Sool01000ooooo`001_oo000?ool00`00ooooo`0:ool00`00oooo
o`0Eool20009ool4000Tool00`00ooooo`0Sool00`00ooooo`07ool4000Tool2000Aool00`00oooo
o`0@ool20009ool4000Sool20007ool000ooo`03003ooooo00_oo`03003ooooo0;7oo`03003ooooo
04coo`003ooo00<00?ooool02ooo00<00?ooool0/?oo00<00?ooool0COoo000?ool00`00ooooo`0;
ool00`00ooooo`2_ool00`00ooooo`1>ool000ooo`03003ooooo00coo`03003ooooo0:goo`03003o
oooo04ooo`002?ooo`006@00000?ool00`00ooooo`06ool00`00ooooo`03ool01000ooooo`001ooo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00D00?ooooooo`0000Coo`03003ooooo
00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`003ooo00<00?ooool03?oo00<0
0?ooool06_oo00<00?ooool0:Ooo00<00?ooool0:Ooo00<00?ooool0:_oo00<00?ooool02?oo00<0
0?ooool07_oo00<00?ooool0:Ooo00<00?ooool01Ooo000?ool00`00ooooo`0=ool00`00ooooo`2W
ool2001Eool000ooo`03003ooooo00goo`03003ooooo0:Koo`03003ooooo05Goo`003ooo00<00?oo
ool03Ooo00<00?ooool0YOoo00<00?ooool0E_oo000?ool00`00ooooo`0>ool00`00ooooo`2Sool0
0`00ooooo`1Gool000ooo`03003ooooo00koo`03003ooooo0:;oo`03003ooooo05Soo`003ooo00<0
0?ooool03_oo00<00?ooool0X?oo0P00Fooo000?ool2000?ool00`00ooooo`2Oool00`00ooooo`1K
ool000ooo`03003ooooo00ooo`03003ooooo09goo`03003ooooo05coo`003ooo00<00?ooool03ooo
00<00?ooool0W?oo00<00?ooool0GOoo000?ool00`00ooooo`0?ool00`00ooooo`2Kool00`00oooo
o`1Nool000ooo`03003ooooo00ooo`03003ooooo09[oo`03003ooooo05ooo`003ooo00<00?ooool0
4?oo00<00?ooool0V?oo00<00?ooool0H?oo000?ool00`00ooooo`0@ool00`00ooooo`2Gool00`00
ooooo`1Qool000ooo`03003ooooo013oo`03003ooooo09Goo`8006Coo`003ooo0P004_oo00<00?oo
ool0Tooo00<00?ooool0I?oo000?ool00`00ooooo`0Aool00`00ooooo`2Bool00`00ooooo`1Uool0
00ooo`03003ooooo017oo`03003ooooo097oo`03003ooooo06Koo`003ooo00<00?ooool04Ooo00<0
0?ooool0T?oo00<00?ooool0Iooo000?ool00`00ooooo`0Bool00`00ooooo`2>ool00`00ooooo`1X
ool000ooo`03003ooooo01;oo`03003ooooo08goo`03003ooooo06Woo`003ooo00<00?ooool04_oo
00<00?ooool0Rooo0P00K?oo000?ool00`00ooooo`0Cool00`00ooooo`29ool00`00ooooo`1/ool0
00ooo`03003ooooo01?oo`03003ooooo08Soo`03003ooooo06goo`003ooo0P005?oo00<00?ooool0
Qooo00<00?ooool0K_oo000?ool00`00ooooo`0Cool00`00ooooo`26ool00`00ooooo`1_ool000oo
o`03003ooooo01Coo`03003ooooo08?oo`8007;oo`003ooo00<00?ooool05?oo00<00?ooool0P_oo
00<00?ooool0L_oo000?ool00`00ooooo`0Dool00`00ooooo`21ool00`00ooooo`1cool000ooo`03
003ooooo01Goo`03003ooooo07ooo`03003ooooo07Coo`003ooo00<00?ooool05Ooo00<00?ooool0
OOoo0P00Mooo0007ool40004ool00`00ooooo`0Eool00`00ooooo`1lool00`00ooooo`1gool000Oo
o`03003ooooo00Goo`03003ooooo01Koo`03003ooooo07[oo`03003ooooo07Soo`002?oo00<00?oo
ool01?oo0`005_oo00<00?ooool0NOoo00<00?ooool0NOoo0009ool00`00ooooo`03ool00`00oooo
o`0Fool00`00ooooo`1hool00`00ooooo`1jool000[oo`03003ooooo00;oo`03003ooooo01Ooo`03
003ooooo07Goo`8007goo`001ooo00@00?ooool000Coo`03003ooooo01Ooo`03003ooooo07Coo`03
003ooooo07goo`002?oo0P001Ooo00<00?ooool06?oo00<00?ooool0L_oo00<00?ooool0O_oo000?
ool00`00ooooo`0Hool00`00ooooo`1`ool20021ool000ooo`03003ooooo01Soo`03003ooooo06oo
o`03003ooooo087oo`003ooo00<00?ooool06Ooo00<00?ooool0KOoo00<00?ooool0P_oo000?ool0
0`00ooooo`0Iool00`00ooooo`1[ool20025ool000ooo`8001[oo`03003ooooo06[oo`03003ooooo
08Goo`003ooo00<00?ooool06_oo00<00?ooool0J?oo00<00?ooool0Q_oo000?ool00`00ooooo`0J
ool00`00ooooo`1Vool20029ool000ooo`03003ooooo01[oo`03003ooooo06Goo`03003ooooo08Wo
o`003ooo00<00?ooool06ooo00<00?ooool0H_oo0P00S?oo000?ool00`00ooooo`0Kool00`00oooo
o`1Qool00`00ooooo`2<ool000ooo`03003ooooo01coo`03003ooooo05ooo`03003ooooo08goo`00
3ooo00<00?ooool07?oo00<00?ooool0GOoo0P00T?oo000?ool2000Mool00`00ooooo`1Lool00`00
ooooo`2@ool000ooo`03003ooooo01goo`03003ooooo05[oo`03003ooooo097oo`003ooo00<00?oo
ool07Ooo00<00?ooool0F?oo0P00U?oo000?ool00`00ooooo`0Mool00`00ooooo`1Gool00`00oooo
o`2Dool000ooo`03003ooooo01koo`03003ooooo05Coo`8009Ooo`003ooo00<00?ooool07_oo00<0
0?ooool0Dooo00<00?ooool0Uooo000?ool00`00ooooo`0Oool00`00ooooo`1Aool00`00ooooo`2H
ool000ooo`03003ooooo01ooo`03003ooooo04ooo`8009_oo`003ooo00<00?ooool07ooo00<00?oo
ool0C_oo00<00?ooool0Vooo000?ool2000Qool00`00ooooo`1<ool00`00ooooo`2Lool000ooo`03
003ooooo023oo`03003ooooo04[oo`8009ooo`003ooo00<00?ooool08Ooo00<00?ooool0B?oo00<0
0?ooool0Wooo000?ool00`00ooooo`0Qool00`00ooooo`16ool2002Rool000ooo`03003ooooo02;o
o`03003ooooo04Coo`03003ooooo0:;oo`003ooo00<00?ooool08_oo00<00?ooool0@_oo0P00YOoo
000?ool00`00ooooo`0Sool00`00ooooo`0oool2002Wool000Woo`03003ooooo00?oo`03003ooooo
02?oo`03003ooooo03koo`03003ooooo0:Ooo`002Ooo00<00?ooool00ooo00<00?ooool09?oo00<0
0?ooool0>ooo0P00Z_oo0006ool50004ool3000Uool00`00ooooo`0iool00`00ooooo`2Zool000Ko
o`04003ooooo0005ool00`00ooooo`0Uool00`00ooooo`0gool2002]ool000Ooo`03003oo`0000Go
o`03003ooooo02Koo`03003ooooo03Coo`800:ooo`002?oo0P001Ooo00<00?ooool09ooo00<00?oo
ool0<Ooo0P00/Ooo0009ool00`00ooooo`03ool00`00ooooo`0Xool00`00ooooo`0^ool2002cool0
00ooo`03003ooooo02Soo`03003ooooo02coo`800;Goo`003ooo00<00?ooool0:Ooo00<00?ooool0
:Ooo0P00]ooo000?ool00`00ooooo`0Zool00`00ooooo`0Vool2002iool000ooo`03003ooooo02_o
o`03003ooooo02?oo`800;_oo`003ooo0P00;Ooo00<00?ooool08?oo0P00_Ooo000?ool00`00oooo
o`0]ool00`00ooooo`0Mool2002oool000ooo`03003ooooo02koo`8001[oo`<00<7oo`003ooo00<0
0?ooool0<?oo0P005_oo0P00a?oo000?ool00`00ooooo`0bool00`00ooooo`0@ool30036ool000oo
o`03003ooooo03?oo`D000Soo`D00<Woo`003ooo00<00?ooool0=ooo2@00c_oo000?ool00`00oooo
o`3oool?ool000ooo`800?oooa3oo`003ooo00<00?ooool0oooo3ooo000?ool00`00ooooo`3oool?
ool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.17529, -4.88909, \
0.0113251, 0.0572609}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\(Im[\((\(-om^2\) + k^2 + WSalphakx0korr[om/k, 10])\)] /. 
          om \[Rule] Om - I\ 0.1\) /. k \[Rule] 2, {Om, 0.01, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.526492 0.1265 [
[.18254 .51399 -9 -9 ]
[.18254 .51399 9 0 ]
[.34127 .51399 -3 -9 ]
[.34127 .51399 3 0 ]
[.5 .51399 -9 -9 ]
[.5 .51399 9 0 ]
[.65873 .51399 -3 -9 ]
[.65873 .51399 3 0 ]
[.81746 .51399 -9 -9 ]
[.81746 .51399 9 0 ]
[.97619 .51399 -3 -9 ]
[.97619 .51399 3 0 ]
[.01131 .02049 -12 -4.5 ]
[.01131 .02049 0 4.5 ]
[.01131 .14699 -12 -4.5 ]
[.01131 .14699 0 4.5 ]
[.01131 .27349 -12 -4.5 ]
[.01131 .27349 0 4.5 ]
[.01131 .39999 -12 -4.5 ]
[.01131 .39999 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .52649 m
.18254 .53274 L
s
[(0.5)] .18254 .51399 0 1 Mshowa
.34127 .52649 m
.34127 .53274 L
s
[(1)] .34127 .51399 0 1 Mshowa
.5 .52649 m
.5 .53274 L
s
[(1.5)] .5 .51399 0 1 Mshowa
.65873 .52649 m
.65873 .53274 L
s
[(2)] .65873 .51399 0 1 Mshowa
.81746 .52649 m
.81746 .53274 L
s
[(2.5)] .81746 .51399 0 1 Mshowa
.97619 .52649 m
.97619 .53274 L
s
[(3)] .97619 .51399 0 1 Mshowa
.125 Mabswid
.05556 .52649 m
.05556 .53024 L
s
.0873 .52649 m
.0873 .53024 L
s
.11905 .52649 m
.11905 .53024 L
s
.15079 .52649 m
.15079 .53024 L
s
.21429 .52649 m
.21429 .53024 L
s
.24603 .52649 m
.24603 .53024 L
s
.27778 .52649 m
.27778 .53024 L
s
.30952 .52649 m
.30952 .53024 L
s
.37302 .52649 m
.37302 .53024 L
s
.40476 .52649 m
.40476 .53024 L
s
.43651 .52649 m
.43651 .53024 L
s
.46825 .52649 m
.46825 .53024 L
s
.53175 .52649 m
.53175 .53024 L
s
.56349 .52649 m
.56349 .53024 L
s
.59524 .52649 m
.59524 .53024 L
s
.62698 .52649 m
.62698 .53024 L
s
.69048 .52649 m
.69048 .53024 L
s
.72222 .52649 m
.72222 .53024 L
s
.75397 .52649 m
.75397 .53024 L
s
.78571 .52649 m
.78571 .53024 L
s
.84921 .52649 m
.84921 .53024 L
s
.88095 .52649 m
.88095 .53024 L
s
.9127 .52649 m
.9127 .53024 L
s
.94444 .52649 m
.94444 .53024 L
s
.25 Mabswid
0 .52649 m
1 .52649 L
s
.02381 .02049 m
.03006 .02049 L
s
[(-4)] .01131 .02049 1 0 Mshowa
.02381 .14699 m
.03006 .14699 L
s
[(-3)] .01131 .14699 1 0 Mshowa
.02381 .27349 m
.03006 .27349 L
s
[(-2)] .01131 .27349 1 0 Mshowa
.02381 .39999 m
.03006 .39999 L
s
[(-1)] .01131 .39999 1 0 Mshowa
.125 Mabswid
.02381 .04579 m
.02756 .04579 L
s
.02381 .07109 m
.02756 .07109 L
s
.02381 .09639 m
.02756 .09639 L
s
.02381 .12169 m
.02756 .12169 L
s
.02381 .17229 m
.02756 .17229 L
s
.02381 .19759 m
.02756 .19759 L
s
.02381 .22289 m
.02756 .22289 L
s
.02381 .24819 m
.02756 .24819 L
s
.02381 .29879 m
.02756 .29879 L
s
.02381 .32409 m
.02756 .32409 L
s
.02381 .34939 m
.02756 .34939 L
s
.02381 .37469 m
.02756 .37469 L
s
.02381 .42529 m
.02756 .42529 L
s
.02381 .45059 m
.02756 .45059 L
s
.02381 .47589 m
.02756 .47589 L
s
.02381 .50119 m
.02756 .50119 L
s
.02381 .55179 m
.02756 .55179 L
s
.02381 .57709 m
.02756 .57709 L
s
.02381 .60239 m
.02756 .60239 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02698 .49592 m
.04573 .3225 L
.06549 .17167 L
.07587 .11192 L
.08553 .06958 L
.09109 .05104 L
.09614 .03771 L
.09894 .03173 L
.10197 .02632 L
.10463 .02248 L
.10749 .01925 L
.10879 .01808 L
.11002 .01713 L
.11123 .01637 L
.11234 .01579 L
.1134 .01536 L
.11452 .01503 L
.11576 .0148 L
.11689 .01472 L
.11754 .01472 L
.11823 .01477 L
.11946 .01497 L
.12014 .01513 L
.12078 .01533 L
.12223 .01589 L
.12466 .01723 L
.12726 .01918 L
.13289 .02508 L
.13822 .03255 L
.14822 .0506 L
.18642 .14594 L
.22706 .25216 L
.2662 .33672 L
.30382 .39899 L
.34389 .44822 L
.38245 .4831 L
.42345 .5106 L
.46295 .5305 L
.50093 .54533 L
.54135 .55783 L
.58027 .5676 L
.62163 .57636 L
.6317 .57833 L
.64236 .58039 L
.65146 .58214 L
.65396 .58262 L
.65533 .58289 L
.65664 .58314 L
.65785 .58337 L
.65853 .5835 L
Mistroke
.65917 .58291 L
.66028 .58292 L
.66148 .58293 L
.66646 .58301 L
.67184 .58313 L
.6816 .58343 L
.69981 .58419 L
.74073 .58646 L
.78014 .589 L
.81804 .59161 L
.85838 .5945 L
.89721 .59736 L
.93849 .60046 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo3ooo000?ool00`00ooooo`3oool?
ool000ooo`03003ooooo0?ooo`ooo`002Ooo00<00?ooool00ooo00<00?ooool05_oo0`00mOoo0009
ool00`00ooooo`03ool00`00ooooo`0Dool20003ool2003cool000Koo`D000Coo`<001?oo`03003o
oooo00Coo`03003ooooo0?7oo`0000?oo`0000000P0000Goo`00ooooo`0000Goo`03003ooooo01?o
o`03003ooooo00Goo`03003ooooo0?3oo`001ooo00<00?oo00001Ooo00<00?ooool04ooo00<00?oo
ool01_oo00<00?ooool0kooo0008ool20005ool00`00ooooo`0Bool00`00ooooo`08ool00`00oooo
o`3^ool000Woo`03003ooooo00?oo`03003ooooo01;oo`03003ooooo00Woo`03003ooooo0>goo`00
3ooo00<00?ooool04Ooo00<00?ooool02_oo00<00?ooool0kOoo000?ool00`00ooooo`0Aool00`00
ooooo`0;ool00`00ooooo`3/ool000ooo`80017oo`03003ooooo00coo`03003ooooo0>coo`003ooo
00<00?ooool04?oo00<00?ooool03Ooo00<00?ooool0jooo000?ool00`00ooooo`0@ool00`00oooo
o`0=ool00`00ooooo`3[ool000ooo`03003ooooo00ooo`03003ooooo00ooo`03003ooooo0>[oo`00
3ooo00<00?ooool03ooo00<00?ooool03ooo00<00?ooool0j_oo000?ool00`00ooooo`0>ool00`00
ooooo`0@ool00`00ooooo`3Zool000ooo`03003ooooo00koo`03003ooooo017oo`03003ooooo0>Wo
o`003ooo0P003ooo00<00?ooool04Ooo00<00?ooool0jOoo000?ool00`00ooooo`0>ool00`00oooo
o`0Bool00`00ooooo`3Xool000ooo`03003ooooo00goo`03003ooooo01?oo`03003ooooo0>Soo`00
3ooo00<00?ooool03Ooo00<00?ooool04ooo00<00?ooool0j?oo000?ool00`00ooooo`0=ool00`00
ooooo`0Dool00`00ooooo`3Wool000ooo`03003ooooo00goo`03003ooooo01Coo`03003ooooo0>Oo
o`003ooo00<00?ooool03?oo00<00?ooool05Ooo00<00?ooool0iooo000?ool2000=ool00`00oooo
o`0Fool00`00ooooo`3Vool000ooo`03003ooooo00coo`03003ooooo01Koo`03003ooooo0>Koo`00
3ooo00<00?ooool03?oo00<00?ooool05ooo00<00?ooool0iOoo000?ool00`00ooooo`0;ool00`00
ooooo`0Hool00`00ooooo`3Uool000ooo`03003ooooo00_oo`03003ooooo01Soo`03003ooooo0>Go
o`003ooo00<00?ooool02ooo00<00?ooool06Ooo00<00?ooool0i?oo000?ool00`00ooooo`0;ool0
0`00ooooo`0Iool00`00ooooo`3Tool000ooo`8000coo`03003ooooo01Woo`03003ooooo0>Coo`00
3ooo00<00?ooool02ooo00<00?ooool06_oo00<00?ooool0hooo000?ool00`00ooooo`0:ool00`00
ooooo`0Kool00`00ooooo`3Sool000ooo`03003ooooo00[oo`03003ooooo01coo`03003ooooo0>;o
o`003ooo00<00?ooool02_oo00<00?ooool07?oo00<00?ooool0h_oo0008ool20005ool00`00oooo
o`0:ool00`00ooooo`0Lool00`00ooooo`3Rool000Ooo`04003ooooo0004ool00`00ooooo`0:ool0
0`00ooooo`0Mool00`00ooooo`3Qool000[oo`03003ooooo00;oo`<000[oo`03003ooooo01goo`03
003ooooo0>7oo`0000?oo`0000000P000ooo0P001Ooo00<00?ooool02_oo00<00?ooool07Ooo00<0
0?ooool0hOoo000:ool00`00ooooo`02ool00`00ooooo`0:ool00`00ooooo`0Nool00`00ooooo`3P
ool000Ooo`04003ooooo0004ool00`00ooooo`09ool00`00ooooo`0Oool00`00ooooo`3Pool000So
o`8000Goo`03003ooooo00Woo`03003ooooo023oo`03003ooooo0=ooo`003ooo00<00?ooool02Ooo
00<00?ooool08?oo00<00?ooool0gooo000?ool00`00ooooo`09ool00`00ooooo`0Pool00`00oooo
o`3Oool000ooo`8000[oo`03003ooooo027oo`03003ooooo0=koo`003ooo00<00?ooool02Ooo00<0
0?ooool08Ooo00<00?ooool0g_oo000?ool00`00ooooo`09ool00`00ooooo`0Rool00`00ooooo`3M
ool000ooo`03003ooooo00Woo`03003ooooo02;oo`03003ooooo0=goo`003ooo00<00?ooool02?oo
00<00?ooool09?oo00<00?ooool0g?oo000?ool00`00ooooo`08ool00`00ooooo`0Tool00`00oooo
o`3Lool000ooo`03003ooooo00Soo`03003ooooo02Coo`03003ooooo0=coo`003ooo0P002Ooo00<0
0?ooool09Ooo00<00?ooool0fooo000?ool00`00ooooo`08ool00`00ooooo`0Uool00`00ooooo`3K
ool000ooo`03003ooooo00Soo`03003ooooo02Koo`03003ooooo0=[oo`003ooo00<00?ooool02?oo
00<00?ooool09_oo00<00?ooool0f_oo000?ool00`00ooooo`07ool00`00ooooo`0Wool00`00oooo
o`3Jool000ooo`03003ooooo00Ooo`03003ooooo02Soo`03003ooooo0=Woo`003ooo00<00?ooool0
1ooo00<00?ooool0:?oo00<00?ooool0fOoo000?ool20008ool00`00ooooo`0Yool00`00ooooo`3H
ool000ooo`03003ooooo00Ooo`03003ooooo02Woo`03003ooooo0=Soo`003ooo00<00?ooool01ooo
00<00?ooool0:_oo00<00?ooool0eooo000?ool00`00ooooo`07ool00`00ooooo`0Zool00`00oooo
o`3Gool000ooo`03003ooooo00Koo`03003ooooo02_oo`03003ooooo0=Ooo`003ooo00<00?ooool0
1_oo00<00?ooool0;?oo00<00?ooool0e_oo000?ool00`00ooooo`06ool00`00ooooo`0/ool00`00
ooooo`3Fool000ooo`8000Ooo`03003ooooo02goo`03003ooooo0=Goo`003ooo00<00?ooool01_oo
00<00?ooool0;Ooo00<00?ooool0eOoo000?ool00`00ooooo`06ool00`00ooooo`0]ool00`00oooo
o`3Eool000ooo`03003ooooo00Koo`03003ooooo02koo`03003ooooo0=Coo`003ooo00<00?ooool0
1Ooo00<00?ooool0;ooo00<00?ooool0e?oo0007ool40004ool00`00ooooo`05ool00`00ooooo`0`
ool00`00ooooo`3Cool000Ooo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo033oo`03003o
oooo0=?oo`002?oo00<00?ooool01?oo0`001Ooo00<00?ooool0<?oo00<00?ooool0dooo00000ooo
000000020004ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`0aool00`00ooooo`3Bool0
00[oo`03003ooooo00;oo`03003ooooo00Goo`03003ooooo037oo`03003ooooo0=;oo`001ooo00@0
0?ooool000Coo`03003ooooo00Goo`03003ooooo03;oo`03003ooooo0=7oo`002?oo0P001Ooo00<0
0?ooool01?oo00<00?ooool0<ooo00<00?ooool0dOoo000?ool00`00ooooo`04ool00`00ooooo`0d
ool00`00ooooo`3@ool000ooo`03003ooooo00Coo`03003ooooo03Coo`03003ooooo0=3oo`003ooo
0P001Ooo00<00?ooool0=?oo00<00?ooool0d?oo000?ool00`00ooooo`04ool00`00ooooo`0eool0
0`00ooooo`3?ool000ooo`03003ooooo00Coo`03003ooooo03Goo`03003ooooo0<ooo`003ooo00<0
0?ooool01?oo00<00?ooool0=_oo00<00?ooool0c_oo000?ool00`00ooooo`03ool00`00ooooo`0g
ool00`00ooooo`3>ool000ooo`03003ooooo00?oo`03003ooooo03Ooo`03003ooooo0<koo`003ooo
00<00?ooool00ooo00<00?ooool0>?oo00<00?ooool0cOoo000?ool20004ool00`00ooooo`0hool0
0`00ooooo`3=ool000ooo`03003ooooo00?oo`03003ooooo03Woo`03003ooooo0<coo`003ooo00<0
0?ooool00ooo00<00?ooool0>Ooo00<00?ooool0c?oo000?ool00`00ooooo`03ool00`00ooooo`0j
ool00`00ooooo`3;ool000ooo`03003ooooo00?oo`03003ooooo03[oo`03003ooooo0<_oo`003ooo
00<00?ooool00_oo00<00?ooool0??oo00<00?ooool0b_oo000?ool00`00ooooo`02ool00`00oooo
o`0lool00`00ooooo`3:ool000ooo`8000?oo`03003ooooo03goo`03003ooooo0<Woo`003ooo00<0
0?ooool00_oo00<00?ooool0?_oo00<00?ooool0b?oo000?ool00`00ooooo`02ool00`00ooooo`0n
ool00`00ooooo`38ool000ooo`03003ooooo00;oo`03003ooooo03ooo`03003ooooo0<Ooo`003ooo
00<00?ooool00_oo00<00?ooool0@?oo00<00?ooool0a_oo000?ool00`00ooooo`02ool00`00oooo
o`10ool00`00ooooo`36ool000ooo`03003ooooo00;oo`03003ooooo047oo`03003ooooo0<Goo`00
3ooo0P000ooo00<00?ooool0@Ooo00<00?ooool0aOoo000?ool01@00oooooooo0000AOoo00<00?oo
ool0a?oo000?ool01@00oooooooo0000A_oo00<00?ooool0`ooo000?ool01@00oooooooo0000A_oo
00<00?ooool0`ooo000?ool01@00oooooooo0000Aooo00<00?ooool0`_oo0007ool40004ool01@00
oooooooo0000B?oo00<00?ooool0`Ooo0009ool00`00ooooo`03ool01@00oooooooo0000B?oo00<0
0?ooool0`Ooo0009ool00`00ooooo`03ool300000ooo003oo`18ool00`00ooooo`30ool00003ool0
0000008000Coo`03003ooooo00?oo`05003oooooool0001:ool00`00ooooo`2oool000Woo`03003o
oooo00?oo`05003oooooool0001;ool00`00ooooo`2nool000Ooo`<000Goo`05003oooooool0001;
ool00`00ooooo`2nool000Woo`03003ooooo00?oo`04003ooooo001=ool00`00ooooo`2mool000oo
o`04003ooooo001>ool00`00ooooo`2lool000ooo`04003ooooo001?ool00`00ooooo`2kool000oo
o`800003ool00?oo04koo`03003ooooo0;_oo`003ooo00@00?ooool0053oo`03003ooooo0;[oo`00
3ooo00@00?ooool0057oo`03003ooooo0;Woo`003ooo00@00?ooool005;oo`03003ooooo0;Soo`00
3ooo00@00?ooool005?oo`03003ooooo0;Ooo`003ooo00@00?ooool005?oo`03003ooooo0;Ooo`00
3ooo00<00?oo0000EOoo00<00?ooool0]_oo000?ool3001Fool00`00ooooo`2eool000ooo`03003o
o`0005Ooo`03003ooooo0;Coo`003ooo00<00?oo0000F?oo0P00]?oo000?ool00`00ool0001Jool0
0`00ooooo`2aool000ooo`03003oo`0005_oo`03003ooooo0;3oo`003ooo00<00?oo0000G?oo00<0
0?ooool0[ooo000?ool00`00ool0001Mool00`00ooooo`2^ool000ooo`<005koo`800:koo`003ooo
00<00?oo0000H?oo00<00?ooool0Zooo000?ool2001Rool00`00ooooo`2Zool000ooo`8006?oo`80
0:[oo`003ooo0P008ooo0P001?oo00<00?ooool00_oo0`008ooo10003Ooo00<00?ooool04_oo1000
0ooo00<00?ooool00_oo0`009?oo10008_oo10000ooo00<00?ooool00_oo0`009?oo0P001ooo000?
ool2000Rool01000ooooo`002ooo00<00?ooool08_oo00<00?ooool03Ooo00<00?ooool04ooo00<0
0?ooool02_oo00<00?ooool08Ooo00<00?ooool08ooo00<00?ooool03?oo00<00?ooool08?oo00@0
0?ooool000Koo`003ooo0P008_oo00@00?ooool000_oo`03003ooooo02;oo`03003ooooo00koo`80
01?oo`03003ooooo00[oo`03003ooooo02;oo`03003ooooo02?oo`03003ooooo00_oo`03003ooooo
02?oo`03003ooooo00Coo`003ooo0P008_oo00@00?ooool000Soo`<002Goo`03003ooooo013oo`03
003ooooo013oo`03003ooooo00Ooo`<002Koo`03003ooooo02?oo`03003ooooo00Ooo`<002Coo`80
00Ooo`003ooo00<00?ooool08Ooo00@00?ooool000Soo`03003ooooo02Goo`03003ooooo017oo`03
003ooooo00ooo`03003ooooo00Ooo`03003ooooo02Ooo`03003ooooo02?oo`03003ooooo00Koo`03
003ooooo02Koo`03003ooooo00Coo`003ooo00<00?ooool08Ooo00@00?ooool000Soo`03003ooooo
02?oo`<001Coo`8000goo`<000Woo`03003ooooo02Coo`04003ooooo000Rool01000ooooo`002?oo
00<00?ooool08ooo00@00?ooool000Koo`003ooo00<00?ooool08_oo0P002Ooo10009?oo00<00?oo
ool05?oo00<00?ooool03?oo00<00?ooool01ooo10009?oo0P009?oo0P002Ooo10008ooo0P001ooo
000?ool00`00ooooo`1]ool2002Oool000ooo`03003ooooo06ooo`8009goo`003ooo00<00?ooool0
LOoo0P00Vooo000?ool00`00ooooo`1cool2002Iool000Soool001T000003ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01?oo0`001ooo00<00?ooool01_oo00<00?oo
ool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?oo
ool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?oo
ool01Ooo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo000?ool0
0`00ooooo`1iool3002Bool000ooo`03003ooooo07coo`80093oo`003ooo00<00?ooool0O_oo0`00
SOoo000?ool00`00ooooo`21ool3002:ool000ooo`03003ooooo08Coo`@008Koo`003ooo0P00ROoo
1000P_oo000?ool00`00ooooo`2<ool3001oool000ooo`03003ooooo08ooo`@007_oo`003ooo00<0
0?ooool0Tooo1000Mooo000?ool00`00ooooo`2Gool4001cool000ooo`03003ooooo09_oo`H006go
o`003ooo00<00?ooool0XOoo1000JOoo000?ool2002Vool3001Vool000ooo`03003ooooo0:Soo`l0
05Ooo`003ooo00<00?ooool0]ooo4`00A?oo000?ool00`00ooooo`3:ool;000iool000ooo`03003o
oooo0=GooaH002?oo`003ooo00<00?ooool0jooo2`006?oo000?ool00`00ooooo`3fool;000=ool0
00ooo`800?ooo`?oo`H000Ooo`003ooo00<00?ooool0oooo3ooo000?ool00`00ooooo`3oool?ool0
00ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo3ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.17529, -4.23357, \
0.0113251, 0.0284211}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[BoxData[
    \(doesn' 
      t\ look\ like\ a\ second\ mode . \ \(next : \ 
          wII\ and\ wIII\)\)], "Input"],

Cell[BoxData[
    \(kx0corrtestfunc[om_, k_, 
        a_] := \((om^2 - k^2 - WSalphakx0korr[om/k, a])\)*\((1 - 
            WSbetakx0korr[om/k, a]/om^2)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[0.5, 2, 10]\)], "Input"],

Cell[BoxData[
    \(\(-8.446181857545245`\) - 1.949559689593272`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[kx0corrtestfunc[om, 2, 75]], {om, 1.9, 2.1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-9.02381 4.76191 0.806171 0.891948 [
[.02381 -0.00908 -9 -9 ]
[.02381 -0.00908 9 0 ]
[.2619 -0.00908 -12 -9 ]
[.2619 -0.00908 12 0 ]
[.7381 -0.00908 -12 -9 ]
[.7381 -0.00908 12 0 ]
[.97619 -0.00908 -9 -9 ]
[.97619 -0.00908 9 0 ]
[.4875 .09261 -24 -4.5 ]
[.4875 .09261 0 4.5 ]
[.4875 .18181 -24 -4.5 ]
[.4875 .18181 0 4.5 ]
[.4875 .271 -24 -4.5 ]
[.4875 .271 0 4.5 ]
[.4875 .3602 -24 -4.5 ]
[.4875 .3602 0 4.5 ]
[.4875 .44939 -24 -4.5 ]
[.4875 .44939 0 4.5 ]
[.4875 .53859 -24 -4.5 ]
[.4875 .53859 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .00342 m
.02381 .00967 L
s
[(1.9)] .02381 -0.00908 0 1 Mshowa
.2619 .00342 m
.2619 .00967 L
s
[(1.95)] .2619 -0.00908 0 1 Mshowa
.7381 .00342 m
.7381 .00967 L
s
[(2.05)] .7381 -0.00908 0 1 Mshowa
.97619 .00342 m
.97619 .00967 L
s
[(2.1)] .97619 -0.00908 0 1 Mshowa
.125 Mabswid
.07143 .00342 m
.07143 .00717 L
s
.11905 .00342 m
.11905 .00717 L
s
.16667 .00342 m
.16667 .00717 L
s
.21429 .00342 m
.21429 .00717 L
s
.30952 .00342 m
.30952 .00717 L
s
.35714 .00342 m
.35714 .00717 L
s
.40476 .00342 m
.40476 .00717 L
s
.45238 .00342 m
.45238 .00717 L
s
.54762 .00342 m
.54762 .00717 L
s
.59524 .00342 m
.59524 .00717 L
s
.64286 .00342 m
.64286 .00717 L
s
.69048 .00342 m
.69048 .00717 L
s
.78571 .00342 m
.78571 .00717 L
s
.83333 .00342 m
.83333 .00717 L
s
.88095 .00342 m
.88095 .00717 L
s
.92857 .00342 m
.92857 .00717 L
s
.25 Mabswid
0 .00342 m
1 .00342 L
s
.5 .09261 m
.50625 .09261 L
s
[(-0.8)] .4875 .09261 1 0 Mshowa
.5 .18181 m
.50625 .18181 L
s
[(-0.7)] .4875 .18181 1 0 Mshowa
.5 .271 m
.50625 .271 L
s
[(-0.6)] .4875 .271 1 0 Mshowa
.5 .3602 m
.50625 .3602 L
s
[(-0.5)] .4875 .3602 1 0 Mshowa
.5 .44939 m
.50625 .44939 L
s
[(-0.4)] .4875 .44939 1 0 Mshowa
.5 .53859 m
.50625 .53859 L
s
[(-0.3)] .4875 .53859 1 0 Mshowa
.125 Mabswid
.5 .02126 m
.50375 .02126 L
s
.5 .0391 m
.50375 .0391 L
s
.5 .05693 m
.50375 .05693 L
s
.5 .07477 m
.50375 .07477 L
s
.5 .11045 m
.50375 .11045 L
s
.5 .12829 m
.50375 .12829 L
s
.5 .14613 m
.50375 .14613 L
s
.5 .16397 m
.50375 .16397 L
s
.5 .19965 m
.50375 .19965 L
s
.5 .21749 m
.50375 .21749 L
s
.5 .23532 m
.50375 .23532 L
s
.5 .25316 m
.50375 .25316 L
s
.5 .28884 m
.50375 .28884 L
s
.5 .30668 m
.50375 .30668 L
s
.5 .32452 m
.50375 .32452 L
s
.5 .34236 m
.50375 .34236 L
s
.5 .37804 m
.50375 .37804 L
s
.5 .39587 m
.50375 .39587 L
s
.5 .41371 m
.50375 .41371 L
s
.5 .43155 m
.50375 .43155 L
s
.5 .46723 m
.50375 .46723 L
s
.5 .48507 m
.50375 .48507 L
s
.5 .50291 m
.50375 .50291 L
s
.5 .52075 m
.50375 .52075 L
s
.5 .55643 m
.50375 .55643 L
s
.5 .57426 m
.50375 .57426 L
s
.5 .5921 m
.50375 .5921 L
s
.5 .60994 m
.50375 .60994 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .03747 L
.10458 .06256 L
.14415 .08638 L
.18221 .10956 L
.22272 .13454 L
.26171 .15892 L
.30316 .18526 L
.34309 .21116 L
.3815 .23676 L
.42237 .26523 L
.44268 .28025 L
.46172 .29543 L
.47994 .31235 L
.48519 .3183 L
.49018 .32509 L
.49235 .3287 L
.49466 .33341 L
.49599 .33687 L
.4972 .34092 L
.49841 .34692 L
.49955 .35942 L
.50071 .35571 L
.50196 .34694 L
.50313 .34321 L
.50421 .34107 L
.50546 .33935 L
.50617 .33862 L
.50683 .33806 L
.50803 .33724 L
.50931 .33661 L
.51053 .33617 L
.51184 .33585 L
.51308 .33565 L
.51422 .33554 L
.51548 .33549 L
.51619 .3355 L
.51685 .33552 L
.51756 .33555 L
.5182 .3356 L
.51963 .33574 L
.52078 .3359 L
.522 .33609 L
.52419 .33652 L
.52913 .33775 L
.5381 .34067 L
.57599 .35748 L
.61633 .37872 L
.65515 .40063 L
.69643 .42491 L
Mistroke
.73619 .44899 L
.77444 .47265 L
.81513 .49827 L
.85432 .52333 L
.89595 .55033 L
.93607 .57667 L
.97467 .6023 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000Ooo`@000?oo`03003ooooo00;oo`<002koo`@000?oo`03003ooooo00;oo`<000?oo`<006_o
o`@000?oo`03003ooooo00?oo`8000?oo`<002koo`@000?oo`03003ooooo00;oo`@000Woo`002Ooo
00<00?ooool02_oo00<00?ooool0;Ooo00<00?ooool02_oo00<00?ooool00ooo00<00?ooool0J?oo
00<00?ooool02Ooo00@00?ooool000Goo`03003ooooo02_oo`03003ooooo00_oo`03003ooooo00So
o`002Ooo00<00?ooool02_oo00<00?ooool0;Ooo00<00?ooool02_oo00<00?ooool00ooo00<00?oo
ool0JOoo00<00?ooool02?oo00@00?ooool000Goo`03003ooooo02coo`03003ooooo00[oo`03003o
oooo00Soo`002Ooo00<00?ooool02?oo0`00;ooo00<00?ooool02?oo0`000_oo0`00KOoo00<00?oo
ool01ooo00@00?ooool000;oo`<0033oo`03003ooooo00Woo`03003ooooo00Soo`002Ooo00<00?oo
ool01ooo00@00?ooool002ooo`03003ooooo00Ooo`04003ooooo0002ool00`00ooooo`1^ool00`00
ooooo`06ool01000ooooo`000_oo00<00?ooool0<Ooo00<00?ooool02?oo00<00?ooool02?oo0007
ool30009ool01000ooooo`00;Ooo0`002Ooo00@00?ooool000;oo`03003ooooo06_oo`04003ooooo
0008ool01000ooooo`000_oo00<00?ooool0;_oo00@00?ooool000Soo`<000[oo`002Ooo00<00?oo
ool02?oo0P00<?oo00<00?ooool02?oo0P000ooo1000Jooo0P002_oo0P000ooo1000;_oo0P002ooo
00<00?ooool02?oo003ooolQool00?ooob7oo`00Sooo00<00?ooool0S_oo0009oooo000>000:ool0
00ooo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo
00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo
00[oo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo
00[oo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo
00[oo`03003ooooo00koo`003ooo00<00?ooool0?Ooo00<00?ooool0?Ooo00<00?ooool0?Ooo00<0
0?ooool0?Ooo00<00?ooool03_oo000?ool00`00ooooo`1mool00`00ooooo`2>ool0013oo`8007go
o`03003ooooo08koo`004_oo0P00Nooo0P00Sooo000Dool2001iool00`00ooooo`2>ool001Koo`80
07Ooo`03003ooooo08koo`006?oo0P00MOoo00<00?ooool0S_oo000Jool00`00ooooo`1bool2002?
ool001_oo`8007;oo`03003ooooo08koo`007Ooo00<00?ooool0Kooo00<00?ooool0S_oo000Nool2
001_ool00`00ooooo`2>ool0023oo`8006goo`03003ooooo08koo`008_oo00<00?ooool0J_oo0P00
Sooo000Sool2001Zool00`00ooooo`2>ool002Goo`03003ooooo06Ooo`03003ooooo08koo`009_oo
0P00Iooo00<00?ooool0S_oo000Xool2001Uool00`00ooooo`2>ool002[oo`8006?oo`8008ooo`00
;?oo0P00HOoo00<00?ooool0S_oo000^ool2001Oool00`00ooooo`2>ool0033oo`03003ooooo04Wo
o`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo08koo`00<Ooo0P00B?oo00@00?ooool000So
o`04003ooooo0004ool00`00ooooo`2>ool003?oo`8004Koo`04003ooooo0008ool01000ooooo`00
1?oo0`00S_oo000eool00`00ooooo`0mool40002ool01000ooooo`002Ooo0P001Ooo00<00?ooool0
S_oo000fool20013ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo08koo`00>?oo0P00
@Ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`2>ool003[oo`03003ooooo03ooo`80
00[oo`8000Goo`03003ooooo08koo`00>ooo0P00D_oo0P00Sooo000mool00`00ooooo`1?ool00`00
ooooo`2>ool003koo`8004ooo`03003ooooo08koo`00@?oo0P00COoo00<00?ooool0S_oo0012ool0
0`00ooooo`1:ool2002?ool004?oo`8004[oo`03003ooooo08koo`00AOoo00<00?ooool0Aooo00<0
0?ooool0S_oo0016ool20017ool00`00ooooo`2>ool004Soo`03003ooooo04Coo`03003ooooo08ko
o`00BOoo00<00?ooool0@ooo0P00Sooo001:ool20013ool00`00ooooo`2>ool004coo`03003ooooo
043oo`03003ooooo08koo`00COoo0P00@?oo00<00?ooool0S_oo001?ool00`00ooooo`0mool00`00
ooooo`2>ool0053oo`8003goo`8008ooo`00D_oo00<00?ooool0>_oo00<00?ooool0S_oo001Cool2
000jool00`00ooooo`2>ool005Goo`8002Goo`8000Coo`03003ooooo00?oo`03003ooooo00Coo`03
003ooooo08koo`00Eooo00<00?ooool08Ooo00@00?ooool000Woo`03003ooooo00Coo`03003ooooo
08koo`00F?oo0P008Ooo00@00?ooool000[oo`03003ooooo00?oo`<008koo`00F_oo00<00?ooool0
6?oo10000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo08koo`00Fooo0P007_oo00@0
0?ooool000_oo`03003ooooo00;oo`03003ooooo08koo`00GOoo00<00?ooool06ooo00@00?ooool0
00Soo`04003ooooo0004ool00`00ooooo`2>ool005koo`8001coo`8000Woo`@000Coo`8008ooo`00
H?oo0P00;Ooo00<00?ooool0S_oo001Rool00`00ooooo`0Zool00`00ooooo`2>ool006?oo`8002[o
o`03003ooooo08koo`00IOoo00<00?ooool09ooo00<00?ooool0S_oo001Vool2000Wool2002?ool0
06Soo`8002Goo`03003ooooo08koo`00J_oo00<00?ooool08_oo00<00?ooool0S_oo001[ool2000R
ool00`00ooooo`2>ool006goo`80023oo`03003ooooo08koo`00Kooo00<00?ooool07Ooo0P00Sooo
001`ool2000Mool00`00ooooo`2>ool007;oo`03003ooooo01[oo`03003ooooo08koo`00Looo00<0
0?ooool06Ooo00<00?ooool0S_oo001dool2000Iool00`00ooooo`2>ool007Koo`03003ooooo01Ko
o`8008ooo`00Mooo00<00?ooool05Ooo00<00?ooool0S_oo001hool2000Eool00`00ooooo`2>ool0
07[oo`04003oo`000004ool00`00ooooo`03ool20005ool00`00ooooo`2>ool007_oo`800003ool0
0?oo00Ooo`04003ooooo0004ool00`00ooooo`2>ool007_oo`04003oo`000008ool01000ooooo`00
1?oo0`00S_oo001eool40002ool00`00ooooo`020007ool30005ool00`00ooooo`2>ool007_oo`06
003ooooo003oo`001_oo00<00?ooool01Ooo00<00?ooool0S_oo001kool01000ooooo`000_oo00<0
0?ooool00ooo00<00?ooool01Ooo00<00?ooool0S_oo001lool20004ool20004ool30004ool2002?
ool008Coo`03003ooooo00Soo`03003ooooo08koo`00QOoo00<00?ooool01ooo00<00?ooool0S_oo
0026ool00`00ooooo`06ool00`00ooooo`2>ool008Ooo`03003ooooo00Goo`03003ooooo08koo`00
R?oo00<00?ooool01?oo0P00Sooo0029ool00`00ooooo`03ool00`00ooooo`2>ool008[oo`03003o
oooo00;oo`03003ooooo08koo`00Rooo00D00?ooooooo`00093oo`00Rooo00D00?ooooooo`00093o
o`00S?oo00<00?ooool00P00Sooo002=ool00`00ool0002@ool008koo`80093oo`00S_oo0P0000?o
o`0000001@00R?oo002>ool40006ool20026ool008koo`<000Woo`8008Coo`00Sooo0P002ooo0P00
P_oo001lool20004ool00`00ooooo`02ool30005ool2000=ool20020ool007_oo`04003ooooo000;
ool00`00ooooo`02ool00`00ooooo`0>ool2001nool007_oo`04003ooooo000;ool00`00ooooo`02
ool3000@ool3001kool007Goo`@000;oo`04003ooooo0008ool30005ool00`00ooooo`0Cool2001i
ool007_oo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0Eool2001gool007_oo`04003o
oooo0008ool00`00ooooo`05ool00`00ooooo`0Gool2001eool007coo`8000Woo`@000Coo`03003o
oooo01Woo`8007?oo`00Sooo0P007?oo0P00LOoo002?ool00`00ooooo`0Mool2001_ool008ooo`03
003ooooo01ooo`8006goo`00Sooo00<00?ooool08Ooo0P00Jooo002?ool00`00ooooo`0Sool2001Y
ool008ooo`8002Koo`8006Ooo`00Sooo00<00?ooool09ooo00<00?ooool0I?oo002?ool00`00oooo
o`0Xool2001Tool008ooo`03003ooooo02[oo`03003ooooo067oo`00Sooo00<00?ooool0:ooo0P00
HOoo002?ool2000^ool2001Oool008ooo`03003ooooo02ooo`03003ooooo05coo`00Sooo00<00?oo
ool0<?oo0P00G?oo002?ool00`00ooooo`0bool00`00ooooo`1Iool008ooo`03003ooooo03?oo`80
05Woo`00Sooo0P00=_oo0P00Eooo002?ool00`00ooooo`0gool00`00ooooo`1Dool007coo`8000Co
o`03003ooooo00Coo`03003ooooo00?oo`03003ooooo03Soo`8005Coo`00Nooo00@00?ooool000[o
o`03003ooooo00?oo`03003ooooo03[oo`8005;oo`00Nooo00@00?ooool000Ooo`D000Coo`<003co
o`03003ooooo04ooo`00MOoo10000_oo00@00?ooool000Ooo`04003ooooo0005ool00`00ooooo`0m
ool2001?ool007_oo`04003ooooo0008ool00`00ool00005ool00`00ooooo`0oool00`00ooooo`1<
ool007_oo`04003ooooo0009ool20005ool00`00ooooo`10ool00`00ooooo`1;ool007coo`8000_o
o`03003ooooo00?oo`03003ooooo047oo`8004_oo`00Sooo0P00A?oo00<00?ooool0B?oo002?ool0
0`00ooooo`14ool20018ool008ooo`03003ooooo04Koo`03003ooooo04Goo`00Sooo00<00?ooool0
Aooo0P00AOoo002?ool00`00ooooo`19ool20013ool008ooo`8004coo`80047oo`00Sooo00<00?oo
ool0COoo0P00?ooo002?ool00`00ooooo`1?ool2000mool008ooo`03003ooooo057oo`03003ooooo
03[oo`00Sooo00<00?ooool0D_oo0P00>_oo002?ool2001Eool00`00ooooo`0gool008ooo`03003o
oooo05Goo`8003Ooo`00Sooo00<00?ooool0Eooo0P00=Ooo002?ool00`00ooooo`1Iool00`00oooo
o`0bool008ooo`8005_oo`8003;oo`00Sooo00<00?ooool0G?oo00<00?ooool0;ooo002?ool00`00
ooooo`1Mool2000_ool007coo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo05ooo`03003o
oooo02coo`00Nooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`1Pool2000/ool007_o
o`04003ooooo000;ool00`00ooooo`02ool3001Rool2000Zool007Goo`@000;oo`04003ooooo0009
ool20005ool00`00ooooo`1Tool00`00ooooo`0Wool007_oo`04003ooooo000;ool00`00ooooo`02
ool00`00ooooo`1Uool2000Wool007_oo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0
Iooo00<00?ooool09?oo001lool2000:ool20005ool00`00ooooo`1Xool2000Tool008ooo`8006_o
o`03003ooooo027oo`00Sooo00<00?ooool0Jooo0P008Ooo002?ool00`00ooooo`1]ool2000Oool0
08ooo`03003ooooo06ooo`03003ooooo01coo`00Sooo00<00?ooool0L?oo0P007?oo002?ool2001c
ool00`00ooooo`0Iool008ooo`03003ooooo07?oo`8001Woo`00Sooo00<00?ooool0MOoo00<00?oo
ool05_oo002?ool00`00ooooo`1fool00`00ooooo`0Eool008ooo`03003ooooo07Ooo`8001Goo`00
Sooo0P00N_oo00<00?ooool04_oo002?ool00`00ooooo`1jool2000Bool008ooo`03003ooooo07co
o`03003ooooo00ooo`00Sooo00<00?ooool0OOoo00<00?ooool03_oo002?ool2002?ool008ooo`03
003ooooo08koo`00Sooo00<00?ooool0S_oo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {1.88747, -0.951695, \
0.000784186, 0.00418659}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[kx0corrtestfunc[om, 2, 75]], {om, 0.5, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.380952 0.208124 0.0962375 [
[.02381 .19562 -9 -9 ]
[.02381 .19562 9 0 ]
[.40476 .19562 -9 -9 ]
[.40476 .19562 9 0 ]
[.59524 .19562 -3 -9 ]
[.59524 .19562 3 0 ]
[.78571 .19562 -9 -9 ]
[.78571 .19562 9 0 ]
[.97619 .19562 -3 -9 ]
[.97619 .19562 3 0 ]
[.20179 .01565 -12 -4.5 ]
[.20179 .01565 0 4.5 ]
[.20179 .11189 -12 -4.5 ]
[.20179 .11189 0 4.5 ]
[.20179 .30436 -6 -4.5 ]
[.20179 .30436 0 4.5 ]
[.20179 .4006 -6 -4.5 ]
[.20179 .4006 0 4.5 ]
[.20179 .49684 -6 -4.5 ]
[.20179 .49684 0 4.5 ]
[.20179 .59307 -6 -4.5 ]
[.20179 .59307 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .20812 m
.02381 .21437 L
s
[(0.5)] .02381 .19562 0 1 Mshowa
.40476 .20812 m
.40476 .21437 L
s
[(1.5)] .40476 .19562 0 1 Mshowa
.59524 .20812 m
.59524 .21437 L
s
[(2)] .59524 .19562 0 1 Mshowa
.78571 .20812 m
.78571 .21437 L
s
[(2.5)] .78571 .19562 0 1 Mshowa
.97619 .20812 m
.97619 .21437 L
s
[(3)] .97619 .19562 0 1 Mshowa
.125 Mabswid
.0619 .20812 m
.0619 .21187 L
s
.1 .20812 m
.1 .21187 L
s
.1381 .20812 m
.1381 .21187 L
s
.17619 .20812 m
.17619 .21187 L
s
.25238 .20812 m
.25238 .21187 L
s
.29048 .20812 m
.29048 .21187 L
s
.32857 .20812 m
.32857 .21187 L
s
.36667 .20812 m
.36667 .21187 L
s
.44286 .20812 m
.44286 .21187 L
s
.48095 .20812 m
.48095 .21187 L
s
.51905 .20812 m
.51905 .21187 L
s
.55714 .20812 m
.55714 .21187 L
s
.63333 .20812 m
.63333 .21187 L
s
.67143 .20812 m
.67143 .21187 L
s
.70952 .20812 m
.70952 .21187 L
s
.74762 .20812 m
.74762 .21187 L
s
.82381 .20812 m
.82381 .21187 L
s
.8619 .20812 m
.8619 .21187 L
s
.9 .20812 m
.9 .21187 L
s
.9381 .20812 m
.9381 .21187 L
s
.25 Mabswid
0 .20812 m
1 .20812 L
s
.21429 .01565 m
.22054 .01565 L
s
[(-2)] .20179 .01565 1 0 Mshowa
.21429 .11189 m
.22054 .11189 L
s
[(-1)] .20179 .11189 1 0 Mshowa
.21429 .30436 m
.22054 .30436 L
s
[(1)] .20179 .30436 1 0 Mshowa
.21429 .4006 m
.22054 .4006 L
s
[(2)] .20179 .4006 1 0 Mshowa
.21429 .49684 m
.22054 .49684 L
s
[(3)] .20179 .49684 1 0 Mshowa
.21429 .59307 m
.22054 .59307 L
s
[(4)] .20179 .59307 1 0 Mshowa
.125 Mabswid
.21429 .0349 m
.21804 .0349 L
s
.21429 .05414 m
.21804 .05414 L
s
.21429 .07339 m
.21804 .07339 L
s
.21429 .09264 m
.21804 .09264 L
s
.21429 .13113 m
.21804 .13113 L
s
.21429 .15038 m
.21804 .15038 L
s
.21429 .16963 m
.21804 .16963 L
s
.21429 .18888 m
.21804 .18888 L
s
.21429 .22737 m
.21804 .22737 L
s
.21429 .24662 m
.21804 .24662 L
s
.21429 .26587 m
.21804 .26587 L
s
.21429 .28511 m
.21804 .28511 L
s
.21429 .32361 m
.21804 .32361 L
s
.21429 .34286 m
.21804 .34286 L
s
.21429 .3621 m
.21804 .3621 L
s
.21429 .38135 m
.21804 .38135 L
s
.21429 .41985 m
.21804 .41985 L
s
.21429 .43909 m
.21804 .43909 L
s
.21429 .45834 m
.21804 .45834 L
s
.21429 .47759 m
.21804 .47759 L
s
.21429 .51608 m
.21804 .51608 L
s
.21429 .53533 m
.21804 .53533 L
s
.21429 .55458 m
.21804 .55458 L
s
.21429 .57383 m
.21804 .57383 L
s
.21429 .61232 m
.21804 .61232 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02846 .06447 L
.03279 .10221 L
.03793 .13791 L
.04262 .16309 L
.04754 .18329 L
.0522 .19743 L
.05711 .20799 L
.05988 .21226 L
.06244 .21525 L
.06487 .21733 L
.06623 .2182 L
.06747 .21882 L
.06862 .21926 L
.06988 .21958 L
.07107 .21976 L
.07218 .21981 L
.07285 .21979 L
.07358 .21973 L
.0749 .21951 L
.07618 .21917 L
.07737 .21875 L
.08006 .21746 L
.0829 .21563 L
.08824 .21108 L
.09323 .20578 L
.10458 .19112 L
.14349 .1324 L
.1633 .10487 L
.18485 .07921 L
.20379 .06063 L
.22469 .0443 L
.24447 .03259 L
.25543 .02752 L
.26548 .02371 L
.27472 .02087 L
.28482 .01845 L
.29498 .01669 L
.30052 .01601 L
.30321 .01575 L
.30568 .01554 L
.30808 .01537 L
.31073 .01523 L
.31339 .01513 L
.31484 .01509 L
.31549 .01507 L
.3162 .01506 L
.31749 .01504 L
.31871 .01503 L
.31989 .01504 L
Mistroke
.32099 .01504 L
.3223 .01506 L
.32348 .01508 L
.32485 .01512 L
.32614 .01516 L
.3284 .01526 L
.33079 .01539 L
.33507 .0157 L
.33968 .01613 L
.34473 .01672 L
.35496 .01828 L
.36427 .0201 L
.3852 .02552 L
.40432 .03194 L
.4251 .04036 L
.46593 .06087 L
.50525 .085 L
.54305 .11184 L
.58331 .14448 L
.62205 .17628 L
.66324 .21433 L
.70292 .25461 L
.74108 .29614 L
.78169 .34307 L
.82079 .39078 L
.86234 .44408 L
.90237 .4979 L
.94089 .55192 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O?ooo00<00?ooool0
g_oo000oool00`00ooooo`3Nool003ooo`03003ooooo0=koo`00=_oo10001Ooo00<00?ooool0g_oo
0009ool00`00ooooo`0Zool00`00ooooo`06ool00`00ooooo`0Gool8002oool000Woo`03003ooooo
02_oo`03003ooooo00Goo`<0013oo`L000Soo`L00;Soo`002Ooo00<00?ooool09?oo10001?oo00<0
0?ooool01?oo00<00?ooool03Ooo0`005_oo1000]?oo0009ool00`00ooooo`0]ool00`00ooooo`03
ool00`00ooooo`0:ool3000Mool5002_ool000Woo`03003ooooo02[oo`04003ooooo0005ool00`00
ooooo`07ool3000Uool3002/ool000Woo`03003ooooo02_oo`8000Koo`03003ooooo00Goo`8002_o
o`800:[oo`002Ooo00<00?ooool0<ooo0P001Ooo00<00?ooool0;Ooo0P00Z?oo0009ool00`00oooo
o`0cool00`00ooooo`02ool2000bool2002Vool000[oo`03003ooooo03;oo`05003oooooool0000f
ool3002Sool000[oo`03003ooooo03;oo`04003ooooo000jool2002Qool000[oo`03003ooooo03;o
o`<003goo`8009ooo`002_oo00<00?ooool0<_oo0P00@?oo0P00WOoo000:ool00`00ooooo`0`ool3
0013ool2002Kool000[oo`03003ooooo02ooo`04003ooooo0015ool2002Iool000[oo`03003ooooo
02koo`05003oooooool00017ool2002Gool000[oo`03003ooooo02goo`03003ooooo00;oo`03003o
oooo04Ooo`8009Goo`002_oo00<00?ooool0;?oo00<00?ooool00ooo00<00?ooool0BOoo00<00?oo
ool0T_oo000:ool00`00ooooo`0[ool00`00ooooo`04ool2001;ool2002Bool000[oo`03003ooooo
02[oo`03003ooooo00Goo`03003ooooo04coo`80093oo`002_oo00<00?ooool0:Ooo00<00?ooool0
1_oo00<00?ooool0C_oo0P00S_oo000;ool00`00ooooo`0Wool00`00ooooo`07ool00`00ooooo`1@
ool00`00ooooo`2;ool000_oo`03003ooooo02Koo`03003ooooo00Soo`03003ooooo057oo`03003o
oooo08[oo`002ooo00<00?ooool09Ooo00<00?ooool02Ooo0P00Dooo0P00R_oo000;ool00`00oooo
o`0Uool00`00ooooo`09ool00`00ooooo`1Dool00`00ooooo`27ool000_oo`03003ooooo02Coo`03
003ooooo00[oo`03003ooooo05Goo`03003ooooo08Koo`002ooo00<00?ooool08ooo00<00?ooool0
2ooo00<00?ooool0E_oo00<00?ooool0QOoo000;ool00`00ooooo`0Rool00`00ooooo`03ool40005
ool00`00ooooo`1Gool20025ool000_oo`03003ooooo027oo`03003ooooo00Koo`03003ooooo00Co
o`03003ooooo05Woo`03003ooooo08;oo`003?oo00<00?ooool08?oo00<00?ooool01_oo00<00?oo
ool01?oo0`00F_oo00<00?ooool0POoo000<ool00`00ooooo`0Oool00`00ool000030004ool00`00
ooooo`04ool00`00ooooo`1Kool00`00ooooo`20ool000coo`03003ooooo01ooo`03003ooooo00Oo
o`03003ooooo00Coo`03003ooooo05coo`80083oo`003?oo00<00?ooool07_oo00<00?ooool01_oo
0`001_oo00<00?ooool0G_oo00<00?ooool0OOoo000<ool00`00ooooo`0Mool00`00ooooo`09ool0
0`00ooooo`04ool00`00ooooo`1Oool00`00ooooo`1lool000goo`03003ooooo01coo`03003ooooo
013oo`80067oo`8007coo`003Ooo00<00?ooool06ooo00<00?ooool04Ooo00<00?ooool0H_oo00<0
0?ooool0NOoo000=ool00`00ooooo`0Jool00`00ooooo`0Bool00`00ooooo`1Sool00`00ooooo`1h
ool000goo`03003ooooo01[oo`03003ooooo01;oo`03003ooooo06Coo`8007Soo`003Ooo00<00?oo
ool06Ooo00<00?ooool04ooo00<00?ooool0I_oo00<00?ooool0MOoo000=ool00`00ooooo`0Hool0
0`00ooooo`0Dool00`00ooooo`1Wool00`00ooooo`1dool000koo`03003ooooo01Ooo`03003ooooo
01Coo`8006Woo`8007Coo`003_oo00<00?ooool05_oo00<00?ooool05Ooo00<00?ooool0J_oo00<0
0?ooool0LOoo000>ool00`00ooooo`0Eool00`00ooooo`0Fool00`00ooooo`1[ool00`00ooooo`1`
ool000koo`03003ooooo01Goo`03003ooooo01Koo`03003ooooo06coo`03003ooooo06ooo`003_oo
00<00?ooool05?oo00<00?ooool05ooo00<00?ooool0KOoo00<00?ooool0K_oo000>ool00`00oooo
o`0Cool00`00ooooo`0Hool2001_ool2001^ool000;oo`8000Coo`03003ooooo00;oo`<001?oo`03
003ooooo01Woo`03003ooooo02_oo`@000?oo`03003ooooo00;oo`<002goo`@000Goo`03003ooooo
02Coo`@000?oo`03003ooooo00;oo`<002ooo`8000Ooo`0000Goo`00ooooo`0000[oo`8001;oo`03
003ooooo01Woo`03003ooooo02goo`03003ooooo00[oo`03003ooooo02[oo`03003ooooo00Ooo`03
003ooooo02?oo`03003ooooo00coo`03003ooooo02_oo`04003ooooo0006ool00005ool00?ooool0
000:ool2000Aool00`00ooooo`0Jool00`00ooooo`0]ool00`00ooooo`0:ool00`00ooooo`0[ool0
0`00ooooo`07ool00`00ooooo`0Sool00`00ooooo`0;ool00`00ooooo`0^ool00`00ooooo`04ool0
0005ool00?ooool00008ool3000Aool00`00ooooo`0Kool00`00ooooo`0]ool00`00ooooo`07ool3
000_ool00`00ooooo`07ool00`00ooooo`0Sool00`00ooooo`07ool3000_ool20007ool00005ool0
0?ooool00008ool00`00ool0000Aool00`00ooooo`0Kool00`00ooooo`0]ool00`00ooooo`07ool0
0`00ooooo`0`ool00`00ooooo`07ool00`00ooooo`0Sool00`00ooooo`06ool00`00ooooo`0aool0
0`00ooooo`04ool00005ool00?ooool00008ool01000ooooo`003ooo00<00?ooool07?oo0P00;?oo
0`002Ooo00<00?ooool0;Ooo00@00?ooool000[oo`03003ooooo01ooo`04003ooooo0008ool00`00
ooooo`0^ool01000ooooo`001_oo0002ool20009ool4000>ool00`00ooooo`0Mool00`00ooooo`0]
ool00`00ooooo`07ool4000]ool2000<ool00`00ooooo`0Oool20009ool4000^ool20007ool0017o
o`03003ooooo00[oo`03003ooooo01koo`03003ooooo07Ooo`8006Goo`004Ooo00<00?ooool02_oo
00<00?ooool07_oo00<00?ooool0NOoo00<00?ooool0H_oo000Bool00`00ooooo`08ool00`00oooo
o`0Oool00`00ooooo`1jool00`00ooooo`1Qool000;oool001l000002Ooo00<00?ooool01ooo00<0
0?ooool01?oo0P000_oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool0
2?oo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool0
2?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool0
2?oo00<00?ooool01Ooo00@00?ooool000[oo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo
00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo
00Goo`002Ooo00<00?ooool02?oo00<00?ooool00_oo00<00?ooool08ooo00<00?ooool0<ooo00<0
0?ooool0<ooo00<00?ooool04Ooo00<00?ooool07ooo00<00?ooool0=?oo00<00?ooool01Ooo000D
ool200001?oo000000009Ooo00<00?ooool0O_oo00<00?ooool0GOoo000Fool2000Wool00`00oooo
o`1oool00`00ooooo`1Lool003ooo`03003ooooo083oo`03003ooooo05_oo`00?ooo0P00P_oo00<0
0?ooool0F_oo000oool00`00ooooo`22ool00`00ooooo`1Iool003ooo`03003ooooo08;oo`03003o
oooo05Woo`00?ooo00<00?ooool0Pooo00<00?ooool0F?oo000oool00`00ooooo`24ool00`00oooo
o`1Gool003ooo`8008Koo`03003ooooo05Koo`00?ooo00<00?ooool0Q_oo00<00?ooool0EOoo000o
ool00`00ooooo`27ool00`00ooooo`1Dool003ooo`03003ooooo08Soo`03003ooooo05?oo`00?ooo
00<00?ooool0ROoo00<00?ooool0D_oo000oool00`00ooooo`2:ool00`00ooooo`1Aool003ooo`80
08coo`03003ooooo053oo`00?ooo00<00?ooool0S?oo00<00?ooool0Cooo000oool00`00ooooo`2=
ool00`00ooooo`1>ool003ooo`03003ooooo08koo`03003ooooo04goo`00?ooo00<00?ooool0Sooo
00<00?ooool0C?oo000oool2002Aool00`00ooooo`1;ool003ooo`03003ooooo097oo`03003ooooo
04[oo`00?ooo00<00?ooool0T_oo00<00?ooool0BOoo000oool00`00ooooo`2Cool00`00ooooo`18
ool003Koo`@000Goo`03003ooooo09Coo`03003ooooo04Ooo`00>?oo00<00?ooool01?oo00<00?oo
ool0UOoo00<00?ooool0A_oo000hool00`00ooooo`04ool3002Eool00`00ooooo`16ool003Soo`03
003ooooo00Coo`03003ooooo09Koo`03003ooooo04Goo`00>?oo00<00?ooool01?oo00<00?ooool0
Uooo00<00?ooool0A?oo000fool30006ool00`00ooooo`2Hool00`00ooooo`13ool003Soo`03003o
oooo00Coo`03003ooooo09Soo`03003ooooo04?oo`00?ooo0P00V_oo00<00?ooool0@_oo000oool0
0`00ooooo`2Jool00`00ooooo`11ool003ooo`03003ooooo09_oo`03003ooooo043oo`00?ooo00<0
0?ooool0W?oo00<00?ooool0?ooo000oool00`00ooooo`2Lool00`00ooooo`0oool003ooo`03003o
oooo09goo`03003ooooo03koo`00?ooo0P00Wooo00<00?ooool0?Ooo000oool00`00ooooo`2Oool0
0`00ooooo`0lool003ooo`03003ooooo0:3oo`03003ooooo03_oo`00?ooo00<00?ooool0XOoo00<0
0?ooool0>_oo000oool00`00ooooo`2Qool00`00ooooo`0jool003ooo`800:?oo`03003ooooo03Wo
o`00?ooo00<00?ooool0Xooo00<00?ooool0>?oo000oool00`00ooooo`2Tool00`00ooooo`0gool0
03ooo`03003ooooo0:Goo`03003ooooo03Koo`00?ooo00<00?ooool0Y_oo00<00?ooool0=Ooo000o
ool00`00ooooo`2Vool00`00ooooo`0eool003ooo`800:Soo`03003ooooo03Coo`00?ooo00<00?oo
ool0Z?oo00<00?ooool0<ooo000oool00`00ooooo`2Yool00`00ooooo`0bool003Koo`@000Goo`03
003ooooo0:[oo`03003ooooo037oo`00=_oo00<00?ooool01_oo00<00?ooool0Z_oo00<00?ooool0
<Ooo000gool00`00ooooo`05ool3002[ool00`00ooooo`0`ool003Soo`03003ooooo00Coo`03003o
oooo0:coo`03003ooooo02ooo`00>Ooo00<00?ooool00ooo00<00?ooool0[Ooo00<00?ooool0;_oo
000fool01000ooooo`001Ooo00<00?ooool0[Ooo00<00?ooool0;_oo000gool20006ool00`00oooo
o`2^ool00`00ooooo`0]ool003ooo`03003ooooo0:ooo`03003ooooo02coo`00?ooo0P00/Ooo00<0
0?ooool0:ooo000oool00`00ooooo`2`ool00`00ooooo`0[ool003ooo`03003ooooo0;7oo`03003o
oooo02[oo`00?ooo00<00?ooool0/_oo00<00?ooool0:Ooo000oool00`00ooooo`2cool00`00oooo
o`0Xool003ooo`800;Coo`03003ooooo02Soo`00?ooo00<00?ooool0]?oo00<00?ooool09ooo000o
ool00`00ooooo`2eool00`00ooooo`0Vool003ooo`03003ooooo0;Koo`03003ooooo02Goo`00?ooo
00<00?ooool0]ooo00<00?ooool09?oo000oool00`00ooooo`2gool00`00ooooo`0Tool003ooo`80
0;Woo`03003ooooo02?oo`00?ooo00<00?ooool0^Ooo00<00?ooool08_oo000oool00`00ooooo`2j
ool00`00ooooo`0Qool003ooo`03003ooooo0;_oo`03003ooooo023oo`00?ooo00<00?ooool0^ooo
00<00?ooool08?oo000oool2002mool00`00ooooo`0Oool003ooo`03003ooooo0;goo`03003ooooo
01koo`00?ooo00<00?ooool0__oo00<00?ooool07Ooo000oool00`00ooooo`2oool00`00ooooo`0L
ool003Ooo`8000Koo`03003ooooo0;ooo`03003ooooo01coo`00=_oo00@00?ooool000Goo`03003o
oooo0<3oo`03003ooooo01_oo`00>Ooo00<00?ooool00ooo0`00`Ooo00<00?ooool06_oo000gool2
0006ool00`00ooooo`32ool00`00ooooo`0Iool003Woo`03003ooooo00?oo`03003ooooo0<;oo`03
003ooooo01Woo`00=_oo00@00?ooool000Goo`03003ooooo0<?oo`03003ooooo01Soo`00=ooo0P00
1_oo00<00?ooool0a?oo00<00?ooool05ooo000oool20036ool00`00ooooo`0Fool003ooo`03003o
oooo0<Goo`03003ooooo01Koo`00?ooo00<00?ooool0a_oo00<00?ooool05Ooo000oool00`00oooo
o`37ool00`00ooooo`0Dool003ooo`03003ooooo0<Soo`03003ooooo01?oo`00?ooo00<00?ooool0
b?oo00<00?ooool04ooo000oool2003:ool00`00ooooo`0Bool003ooo`03003ooooo0<[oo`03003o
oooo017oo`00?ooo00<00?ooool0booo00<00?ooool04?oo000oool00`00ooooo`3;ool00`00oooo
o`0@ool003ooo`03003ooooo0<coo`03003ooooo00ooo`00?ooo0P00c_oo00<00?ooool03_oo000o
ool00`00ooooo`3=ool00`00ooooo`0>ool003ooo`03003ooooo0<koo`03003ooooo00goo`00?ooo
00<00?ooool0cooo00<00?ooool03?oo000oool00`00ooooo`3?ool00`00ooooo`0<ool003ooo`80
0=7oo`03003ooooo00_oo`00?ooo00<00?ooool0dOoo00<00?ooool02_oo000oool00`00ooooo`3A
ool00`00ooooo`0:ool003ooo`03003ooooo0=;oo`03003ooooo00Woo`00>?oo00<00?ooool01?oo
00<00?ooool0dooo00<00?ooool02?oo000hool00`00ooooo`04ool00`00ooooo`3Cool00`00oooo
o`08ool003Goo`D000Goo`<00=Coo`03003ooooo00Ooo`00=Ooo00@00?ooool000Koo`03003ooooo
0=Goo`03003ooooo00Koo`00=_oo00<00?oo00001_oo00<00?ooool0eOoo00<00?ooool01_oo000g
ool20006ool00`00ooooo`3Fool00`00ooooo`05ool003Soo`03003ooooo00Coo`03003ooooo0=ko
o`00?ooo0P00gooo000oool00`00ooooo`3Nool003ooo`03003ooooo0=koo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.41703, -2.18048, \
0.00921772, 0.036488}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Im[kx0corrtestfunc[om, 2, 10^4]], {om, 0.5, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.380952 0.603319 0.341656 [
[.02381 .59082 -9 -9 ]
[.02381 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.59524 .59082 -3 -9 ]
[.59524 .59082 3 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.20179 .00542 -30 -4.5 ]
[.20179 .00542 0 4.5 ]
[.20179 .09083 -24 -4.5 ]
[.20179 .09083 0 4.5 ]
[.20179 .17625 -30 -4.5 ]
[.20179 .17625 0 4.5 ]
[.20179 .26166 -12 -4.5 ]
[.20179 .26166 0 4.5 ]
[.20179 .34708 -30 -4.5 ]
[.20179 .34708 0 4.5 ]
[.20179 .43249 -24 -4.5 ]
[.20179 .43249 0 4.5 ]
[.20179 .5179 -30 -4.5 ]
[.20179 .5179 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60332 m
.02381 .60957 L
s
[(0.5)] .02381 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(1.5)] .40476 .59082 0 1 Mshowa
.59524 .60332 m
.59524 .60957 L
s
[(2)] .59524 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(2.5)] .78571 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(3)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.0619 .60332 m
.0619 .60707 L
s
.1 .60332 m
.1 .60707 L
s
.1381 .60332 m
.1381 .60707 L
s
.17619 .60332 m
.17619 .60707 L
s
.25238 .60332 m
.25238 .60707 L
s
.29048 .60332 m
.29048 .60707 L
s
.32857 .60332 m
.32857 .60707 L
s
.36667 .60332 m
.36667 .60707 L
s
.44286 .60332 m
.44286 .60707 L
s
.48095 .60332 m
.48095 .60707 L
s
.51905 .60332 m
.51905 .60707 L
s
.55714 .60332 m
.55714 .60707 L
s
.63333 .60332 m
.63333 .60707 L
s
.67143 .60332 m
.67143 .60707 L
s
.70952 .60332 m
.70952 .60707 L
s
.74762 .60332 m
.74762 .60707 L
s
.82381 .60332 m
.82381 .60707 L
s
.8619 .60332 m
.8619 .60707 L
s
.9 .60332 m
.9 .60707 L
s
.9381 .60332 m
.9381 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.21429 .00542 m
.22054 .00542 L
s
[(-1.75)] .20179 .00542 1 0 Mshowa
.21429 .09083 m
.22054 .09083 L
s
[(-1.5)] .20179 .09083 1 0 Mshowa
.21429 .17625 m
.22054 .17625 L
s
[(-1.25)] .20179 .17625 1 0 Mshowa
.21429 .26166 m
.22054 .26166 L
s
[(-1)] .20179 .26166 1 0 Mshowa
.21429 .34708 m
.22054 .34708 L
s
[(-0.75)] .20179 .34708 1 0 Mshowa
.21429 .43249 m
.22054 .43249 L
s
[(-0.5)] .20179 .43249 1 0 Mshowa
.21429 .5179 m
.22054 .5179 L
s
[(-0.25)] .20179 .5179 1 0 Mshowa
.125 Mabswid
.21429 .0225 m
.21804 .0225 L
s
.21429 .03959 m
.21804 .03959 L
s
.21429 .05667 m
.21804 .05667 L
s
.21429 .07375 m
.21804 .07375 L
s
.21429 .10792 m
.21804 .10792 L
s
.21429 .125 m
.21804 .125 L
s
.21429 .14208 m
.21804 .14208 L
s
.21429 .15917 m
.21804 .15917 L
s
.21429 .19333 m
.21804 .19333 L
s
.21429 .21041 m
.21804 .21041 L
s
.21429 .2275 m
.21804 .2275 L
s
.21429 .24458 m
.21804 .24458 L
s
.21429 .27875 m
.21804 .27875 L
s
.21429 .29583 m
.21804 .29583 L
s
.21429 .31291 m
.21804 .31291 L
s
.21429 .32999 m
.21804 .32999 L
s
.21429 .36416 m
.21804 .36416 L
s
.21429 .38124 m
.21804 .38124 L
s
.21429 .39833 m
.21804 .39833 L
s
.21429 .41541 m
.21804 .41541 L
s
.21429 .44957 m
.21804 .44957 L
s
.21429 .46666 m
.21804 .46666 L
s
.21429 .48374 m
.21804 .48374 L
s
.21429 .50082 m
.21804 .50082 L
s
.21429 .53499 m
.21804 .53499 L
s
.21429 .55207 m
.21804 .55207 L
s
.21429 .56915 m
.21804 .56915 L
s
.21429 .58624 m
.21804 .58624 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05512 0 m
.06244 .08739 L
.0731 .18609 L
.0828 .2564 L
.09413 .32096 L
.10458 .36788 L
.12373 .43135 L
.13381 .4563 L
.14451 .47807 L
.16404 .5085 L
.18234 .52915 L
.20259 .54597 L
.22112 .55746 L
.2401 .56647 L
.26083 .574 L
.2805 .57952 L
.30147 .58411 L
.32145 .58756 L
.34305 .59052 L
.38312 .59452 L
.40306 .59599 L
.42412 .59727 L
.46209 .59905 L
.50252 .6004 L
.54143 .60135 L
.56124 .60173 L
.57169 .60191 L
.58279 .60209 L
.58752 .60216 L
.59001 .6022 L
.59263 .60223 L
.59402 .60225 L
.59481 .60227 L
.59555 .60332 L
.59684 .60332 L
.59823 .60332 L
.60348 .60332 L
.62263 .60332 L
.66413 .60332 L
.70411 .60332 L
.74258 .60332 L
.7835 .60332 L
.8229 .60332 L
.86079 .60332 L
.90113 .60332 L
.93995 .60332 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool002Goo`@000?oo`03003ooooo00?oo`03003ooooo00;oo`<00>Koo`004ooo00<00?ooool04Ooo
00<00?ooool02?oo00<00?ooool01Ooo00<00?ooool00ooo00<00?ooool0gOoo000Cool00`00oooo
o`0Aool00`00ooooo`09ool00`00ooooo`04ool00`00ooooo`03ool2003Nool001?oo`03003ooooo
00Woo`@000Coo`03003ooooo00Woo`04003oooooool30006ool00`00ooooo`3Mool001?oo`03003o
oooo017oo`03003ooooo00[oo`04003ooooo0008ool00`00ooooo`3Mool001?oo`03003ooooo00oo
o`<000Woo`04003ooooo0002ool00`00ooooo`06ool00`00ooooo`3Mool001?oo`03003ooooo017o
o`03003ooooo00Ooo`@000;oo`@000Goo`03003ooooo0=goo`004ooo00<00?ooool0:_oo0P00g_oo
000Dool00`00ooooo`0Yool00`00ooooo`3Mool001Coo`03003ooooo02Woo`03003ooooo0=goo`00
5?oo00<00?ooool0:Ooo00<00?ooool0gOoo000Dool00`00ooooo`0Yool00`00ooooo`3Mool001Co
o`03003ooooo02Woo`800=koo`005?oo00<00?ooool0:Ooo00<00?ooool0gOoo000Dool00`00oooo
o`0Yool00`00ooooo`3Mool001Coo`03003ooooo02Woo`03003ooooo0=goo`005?oo00<00?ooool0
:Ooo0P00g_oo000Dool00`00ooooo`0Yool00`00ooooo`3Mool001Coo`03003ooooo02Woo`03003o
oooo0=goo`005?oo00<00?ooool0:Ooo00<00?ooool0gOoo000Eool00`00ooooo`0Xool00`00oooo
o`3Mool001Goo`03003ooooo02Soo`800=koo`005Ooo00<00?ooool0:?oo00<00?ooool0gOoo000E
ool00`00ooooo`0Xool00`00ooooo`3Mool001Goo`03003ooooo01?oo`@000?oo`03003ooooo00;o
o`<000Koo`03003ooooo0=goo`005Ooo00<00?ooool05Ooo00<00?ooool02_oo00<00?ooool00ooo
00<00?ooool0gOoo000Eool00`00ooooo`0Eool00`00ooooo`0:ool00`00ooooo`03ool2003Nool0
01Goo`03003ooooo00goo`@000Coo`03003ooooo00Ooo`<000Koo`03003ooooo0=goo`005Ooo00<0
0?ooool05Ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool0gOoo000Eool00`00ooooo`0C
ool30009ool00`00ooooo`06ool00`00ooooo`3Mool001Koo`03003ooooo01Coo`03003ooooo00Oo
o`@000Goo`03003ooooo0=goo`005_oo00<00?ooool09ooo0P00g_oo000Fool00`00ooooo`0Wool0
0`00ooooo`3Mool001Koo`03003ooooo02Ooo`03003ooooo0=goo`005_oo00<00?ooool09ooo00<0
0?ooool0gOoo000Fool00`00ooooo`0Wool00`00ooooo`3Mool001Koo`03003ooooo02Ooo`800=ko
o`005_oo00<00?ooool09ooo00<00?ooool0gOoo000Fool00`00ooooo`0Wool00`00ooooo`3Mool0
01Koo`03003ooooo02Ooo`03003ooooo0=goo`005ooo00<00?ooool09_oo00<00?ooool0gOoo000G
ool00`00ooooo`0Vool2003Nool001Ooo`03003ooooo02Koo`03003ooooo0=goo`005ooo00<00?oo
ool09_oo00<00?ooool0gOoo000Gool00`00ooooo`0Vool00`00ooooo`3Mool001Ooo`03003ooooo
02Koo`800=koo`005ooo00<00?ooool09_oo00<00?ooool0gOoo000Gool00`00ooooo`0Vool00`00
ooooo`3Mool001Ooo`03003ooooo00_oo`@000?oo`03003ooooo00;oo`@000;oo`<000Koo`03003o
oooo0=goo`006?oo00<00?ooool03?oo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool00ooo
00<00?ooool0gOoo000Hool00`00ooooo`0<ool00`00ooooo`08ool00`00ooooo`05ool00`00oooo
o`03ool2003Nool001Soo`03003ooooo00Coo`@000Coo`03003ooooo00Woo`04003oooooool30006
ool00`00ooooo`3Mool001Soo`03003ooooo00coo`03003ooooo00[oo`04003ooooo0008ool00`00
ooooo`3Mool001Soo`03003ooooo00[oo`<000Woo`04003ooooo0002ool00`00ooooo`06ool00`00
ooooo`3Mool001Soo`03003ooooo00coo`03003ooooo00Soo`8000?oo`@000Goo`03003ooooo0=go
o`006?oo00<00?ooool09Ooo0P00g_oo000Hool00`00ooooo`0Uool00`00ooooo`3Mool001Woo`03
003ooooo02Coo`03003ooooo0=goo`006Ooo00<00?ooool09?oo00<00?ooool0gOoo000Iool00`00
ooooo`0Tool00`00ooooo`3Mool001Woo`03003ooooo02Coo`800=koo`006Ooo00<00?ooool09?oo
00<00?ooool0gOoo000Iool00`00ooooo`0Tool00`00ooooo`3Mool001Woo`03003ooooo02Coo`03
003ooooo0=goo`006_oo00<00?ooool08ooo00<00?ooool0gOoo000Jool00`00ooooo`0Sool2003N
ool001[oo`03003ooooo02?oo`03003ooooo0=goo`006_oo00<00?ooool08ooo00<00?ooool0gOoo
000Jool00`00ooooo`0Sool00`00ooooo`3Mool001[oo`03003ooooo02?oo`800=koo`006ooo00<0
0?ooool08_oo00<00?ooool0gOoo000Kool00`00ooooo`0Rool00`00ooooo`3Mool001_oo`03003o
oooo01Woo`@000Goo`03003ooooo0=goo`006ooo00<00?ooool06ooo00<00?ooool01?oo00<00?oo
ool0gOoo000Kool00`00ooooo`0Kool00`00ooooo`04ool2003Nool001_oo`03003ooooo01?oo`@0
00Coo`03003ooooo00Coo`03003ooooo0=goo`006ooo00<00?ooool06ooo00<00?ooool01?oo00<0
0?ooool0gOoo000Lool00`00ooooo`0Hool30006ool00`00ooooo`3Mool001coo`03003ooooo01[o
o`03003ooooo00Coo`03003ooooo0=goo`007?oo00<00?ooool08Ooo0P00g_oo000Lool00`00oooo
o`0Qool00`00ooooo`3Mool001coo`03003ooooo027oo`03003ooooo0=goo`007?oo00<00?ooool0
8Ooo00<00?ooool0gOoo000Mool00`00ooooo`0Pool00`00ooooo`3Mool001goo`03003ooooo023o
o`800=koo`007Ooo00<00?ooool08?oo00<00?ooool0gOoo000Mool00`00ooooo`0Pool00`00oooo
o`3Mool001goo`03003ooooo023oo`03003ooooo0=goo`007Ooo00<00?ooool08?oo00<00?ooool0
gOoo000Nool00`00ooooo`0Oool2003Nool001koo`03003ooooo01ooo`03003ooooo0=goo`007_oo
00<00?ooool07ooo00<00?ooool0gOoo000Nool00`00ooooo`0Oool00`00ooooo`3Mool001koo`03
003ooooo01ooo`800=koo`007ooo00<00?ooool07_oo00<00?ooool0gOoo000Oool00`00ooooo`0N
ool00`00ooooo`3Mool001ooo`03003ooooo00Coo`8000Coo`03003ooooo00?oo`03003ooooo00;o
o`<000Koo`03003ooooo0=goo`007ooo00<00?ooool00ooo00@00?ooool000Woo`03003ooooo00Go
o`03003ooooo00?oo`03003ooooo0=goo`008?oo00<00?ooool00_oo00@00?ooool000[oo`03003o
oooo00Coo`03003ooooo00?oo`800=koo`007ooo10000_oo00@00?ooool000[oo`04003oooooool3
0006ool00`00ooooo`3Mool0023oo`03003ooooo00;oo`04003ooooo000;ool01000ooooo`002?oo
00<00?ooool0gOoo000Pool00`00ooooo`02ool01000ooooo`002?oo00@00?ooool000;oo`03003o
oooo00Koo`03003ooooo0=goo`008Ooo00<00?ooool00_oo0P002Ooo10000_oo10001Ooo00<00?oo
ool0gOoo000Qool00`00ooooo`0Lool2003Nool0027oo`03003ooooo01coo`03003ooooo0=goo`00
8Ooo00<00?ooool07?oo00<00?ooool0gOoo000Rool00`00ooooo`0Kool00`00ooooo`3Mool002;o
o`03003ooooo01_oo`03003ooooo0=goo`008_oo00<00?ooool06ooo0P00g_oo000Rool00`00oooo
o`0Kool00`00ooooo`3Mool002?oo`03003ooooo01[oo`03003ooooo0=goo`008ooo00<00?ooool0
6_oo00<00?ooool0gOoo000Sool00`00ooooo`0Jool00`00ooooo`3Mool002?oo`03003ooooo01[o
o`800=koo`009?oo00<00?ooool06Ooo00<00?ooool0gOoo000Tool00`00ooooo`0Iool00`00oooo
o`3Mool002Coo`03003ooooo01Woo`03003ooooo0=goo`009Ooo00<00?ooool06?oo0P00g_oo000U
ool00`00ooooo`0Hool00`00ooooo`3Mool002Goo`03003ooooo01Soo`03003ooooo0=goo`009Ooo
00<00?ooool01?oo0P001?oo00<00?ooool00_oo0`001_oo00<00?ooool0gOoo000Vool00`00oooo
o`02ool01000ooooo`002ooo00<00?ooool00ooo00<00?ooool0gOoo000Vool00`00ooooo`02ool0
1000ooooo`002ooo00<00?ooool00ooo0P00g_oo000Uool40002ool01000ooooo`002?oo0`001_oo
00<00?ooool0gOoo000Wool01@00oooooooo00000_oo00<00?ooool01_oo00<00?ooool01_oo00<0
0?ooool0gOoo000Wool01@00oooooooo00000_oo00<00?ooool01_oo00<00?ooool01_oo00<00?oo
ool0gOoo000Xool01000oooooooo0P002Ooo10001Ooo00<00?ooool0gOoo000Xool00`00ooooo`0E
ool2003Nool002Woo`03003ooooo01Coo`03003ooooo0=goo`00:Ooo00<00?ooool05?oo00<00?oo
ool0gOoo000Yool00`00ooooo`0Dool00`00ooooo`3Mool002[oo`03003ooooo01?oo`03003ooooo
0=goo`00:_oo00<00?ooool04ooo0P00g_oo000[ool00`00ooooo`0Bool00`00ooooo`3Mool002_o
o`03003ooooo01;oo`03003ooooo0=goo`00;?oo00<00?ooool04Ooo00<00?ooool0gOoo000]ool0
0`00ooooo`0@ool00`00ooooo`3Mool002goo`03003ooooo013oo`800=koo`00;_oo00<00?ooool0
3ooo00<00?ooool0gOoo000_ool00`00ooooo`0>ool00`00ooooo`3Mool002ooo`03003ooooo00ko
o`03003ooooo0=goo`00<?oo00<00?ooool03Ooo00<00?ooool0gOoo000aool00`00ooooo`0<ool2
003Nool0037oo`03003ooooo00coo`03003ooooo0=goo`009_oo0P001?oo00<00?ooool00_oo1000
0_oo0`001_oo00<00?ooool0gOoo000Uool01000ooooo`002?oo00<00?oo00001_oo00<00?ooool0
0ooo00<00?ooool0gOoo000Uool01000ooooo`002Ooo00<00?oo00001Ooo00<00?ooool00ooo0P00
g_oo000Oool40002ool01000ooooo`002_oo00@00?oo003oo`<000Koo`03003ooooo0=goo`009Ooo
00@00?ooool000_oo`04003oo`000008ool00`00ooooo`3Mool002Goo`04003ooooo0008ool01000
ooooo`000_oo00<00?ooool01_oo00<00?ooool0gOoo000Vool2000:ool20003ool40005ool00`00
ooooo`3Mool003Woo`03003ooooo00Coo`800=koo`00>_oo00<00?ooool00ooo00<00?ooool0gOoo
000kool00`00ooooo`02ool00`00ooooo`3Mool003coo`05003oooooool0003Oool003goo`800003
ool00?oo0=koo`00?ooo0`00g_oo0010ool3003Mool0043oo`03003ooooo00800=_oo`00@?oo00<0
0?ooool00_oo0P00fOoo0003ool20004ool00`00ooooo`02ool3000_ool00`00ooooo`04ool2000T
ool40003ool00`00ooooo`02ool3000]ool4000/ool40003ool00`00ooooo`02ool3000]ool20009
ool000;oo`04003ooooo000;ool00`00ooooo`0/ool20007ool3000Sool00`00ooooo`0:ool00`00
ooooo`0Zool00`00ooooo`0]ool00`00ooooo`0<ool00`00ooooo`0Yool01000ooooo`002?oo0002
ool01000ooooo`002ooo00<00?ooool0;?oo00<00?ooool02Ooo0`008?oo00<00?ooool02_oo00<0
0?ooool0:ooo00<00?ooool0;Ooo00<00?ooool02ooo00<00?ooool0;?oo00<00?ooool01_oo0002
ool01000ooooo`002?oo0`00;ooo00<00?ooool03?oo0P007_oo00<00?ooool01ooo0`00;ooo00<0
0?ooool0;Ooo00<00?ooool01ooo0`00;Ooo0P002Ooo0002ool01000ooooo`002?oo00<00?ooool0
;ooo00<00?ooool03_oo10006_oo00<00?ooool01ooo00<00?ooool0<?oo00<00?ooool0;Ooo00<0
0?ooool01_oo00<00?ooool0;ooo00<00?ooool01_oo0002ool01000ooooo`002?oo00<00?ooool0
;ooo00<00?ooool04_oo1P004_oo0`002Ooo00<00?ooool0;Ooo00@00?ooool002coo`04003ooooo
0008ool00`00ooooo`0/ool01000ooooo`002?oo0003ool20009ool4000^ool2000Iool6000>ool0
0`00ooooo`07ool4000]ool2000^ool20009ool4000/ool20009ool0043oo`03003ooooo01koo`T0
0;Koo`00@?oo00<00?ooool09ooo3P00Z?oo0010ool00`00ooooo`0eoolC002Eool000?oool001`0
00;oo`002_oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool01ooo00<00?ooool02?oo00<0
0?ooool02?oo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<0
0?ooool01ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool01ooo00<0
0?ooool02?oo00<00?ooool02?oo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool02?oo00<0
0?ooool02?oo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool01ooo00<0
0?ooool02?oo00<00?ooool01ooo000:ool00`00ooooo`0cool00`00ooooo`0bool00`00ooooo`0c
ool00`00ooooo`0cool00`00ooooo`0bool00`00ooooo`07ool0043oo`03003ooooo0=goo`00@?oo
00<00?ooool0gOoo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.401034, -1.7968, \
0.00932581, 0.0103984}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[kx0corrtestfunc[om, 2, 10^4]], {om, 0.5, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.380952 0.109795 0.0488087 [
[.02381 .0973 -9 -9 ]
[.02381 .0973 9 0 ]
[.40476 .0973 -9 -9 ]
[.40476 .0973 9 0 ]
[.59524 .0973 -3 -9 ]
[.59524 .0973 3 0 ]
[.78571 .0973 -9 -9 ]
[.78571 .0973 9 0 ]
[.97619 .0973 -3 -9 ]
[.97619 .0973 3 0 ]
[.20179 .01218 -12 -4.5 ]
[.20179 .01218 0 4.5 ]
[.20179 .20741 -6 -4.5 ]
[.20179 .20741 0 4.5 ]
[.20179 .30503 -6 -4.5 ]
[.20179 .30503 0 4.5 ]
[.20179 .40265 -6 -4.5 ]
[.20179 .40265 0 4.5 ]
[.20179 .50026 -6 -4.5 ]
[.20179 .50026 0 4.5 ]
[.20179 .59788 -12 -4.5 ]
[.20179 .59788 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .1098 m
.02381 .11605 L
s
[(0.5)] .02381 .0973 0 1 Mshowa
.40476 .1098 m
.40476 .11605 L
s
[(1.5)] .40476 .0973 0 1 Mshowa
.59524 .1098 m
.59524 .11605 L
s
[(2)] .59524 .0973 0 1 Mshowa
.78571 .1098 m
.78571 .11605 L
s
[(2.5)] .78571 .0973 0 1 Mshowa
.97619 .1098 m
.97619 .11605 L
s
[(3)] .97619 .0973 0 1 Mshowa
.125 Mabswid
.0619 .1098 m
.0619 .11355 L
s
.1 .1098 m
.1 .11355 L
s
.1381 .1098 m
.1381 .11355 L
s
.17619 .1098 m
.17619 .11355 L
s
.25238 .1098 m
.25238 .11355 L
s
.29048 .1098 m
.29048 .11355 L
s
.32857 .1098 m
.32857 .11355 L
s
.36667 .1098 m
.36667 .11355 L
s
.44286 .1098 m
.44286 .11355 L
s
.48095 .1098 m
.48095 .11355 L
s
.51905 .1098 m
.51905 .11355 L
s
.55714 .1098 m
.55714 .11355 L
s
.63333 .1098 m
.63333 .11355 L
s
.67143 .1098 m
.67143 .11355 L
s
.70952 .1098 m
.70952 .11355 L
s
.74762 .1098 m
.74762 .11355 L
s
.82381 .1098 m
.82381 .11355 L
s
.8619 .1098 m
.8619 .11355 L
s
.9 .1098 m
.9 .11355 L
s
.9381 .1098 m
.9381 .11355 L
s
.25 Mabswid
0 .1098 m
1 .1098 L
s
.21429 .01218 m
.22054 .01218 L
s
[(-2)] .20179 .01218 1 0 Mshowa
.21429 .20741 m
.22054 .20741 L
s
[(2)] .20179 .20741 1 0 Mshowa
.21429 .30503 m
.22054 .30503 L
s
[(4)] .20179 .30503 1 0 Mshowa
.21429 .40265 m
.22054 .40265 L
s
[(6)] .20179 .40265 1 0 Mshowa
.21429 .50026 m
.22054 .50026 L
s
[(8)] .20179 .50026 1 0 Mshowa
.21429 .59788 m
.22054 .59788 L
s
[(10)] .20179 .59788 1 0 Mshowa
.125 Mabswid
.21429 .03658 m
.21804 .03658 L
s
.21429 .06099 m
.21804 .06099 L
s
.21429 .08539 m
.21804 .08539 L
s
.21429 .1342 m
.21804 .1342 L
s
.21429 .1586 m
.21804 .1586 L
s
.21429 .18301 m
.21804 .18301 L
s
.21429 .23182 m
.21804 .23182 L
s
.21429 .25622 m
.21804 .25622 L
s
.21429 .28063 m
.21804 .28063 L
s
.21429 .32943 m
.21804 .32943 L
s
.21429 .35384 m
.21804 .35384 L
s
.21429 .37824 m
.21804 .37824 L
s
.21429 .42705 m
.21804 .42705 L
s
.21429 .45146 m
.21804 .45146 L
s
.21429 .47586 m
.21804 .47586 L
s
.21429 .52467 m
.21804 .52467 L
s
.21429 .54907 m
.21804 .54907 L
s
.21429 .57348 m
.21804 .57348 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05963 .61803 m
.06244 .58337 L
.07298 .4887 L
.08426 .40661 L
.10458 .29525 L
.1148 .25244 L
.12589 .21348 L
.14603 .15814 L
.16586 .11802 L
.18689 .08661 L
.20643 .06493 L
.22473 .04957 L
.24497 .0369 L
.2635 .02843 L
.28248 .02225 L
.29323 .01969 L
.30321 .01785 L
.30828 .01709 L
.31377 .0164 L
.31862 .01589 L
.32373 .01546 L
.32835 .01516 L
.3308 .01504 L
.33342 .01492 L
.33583 .01484 L
.33803 .01479 L
.33909 .01476 L
.34024 .01475 L
.34133 .01473 L
.34234 .01472 L
.34351 .01472 L
.34475 .01472 L
.34581 .01472 L
.34697 .01472 L
.34822 .01474 L
.34956 .01476 L
.35082 .01478 L
.35196 .0148 L
.35419 .01486 L
.35661 .01494 L
.361 .01514 L
.36621 .01544 L
.37172 .01584 L
.3816 .01675 L
.40118 .01927 L
.41949 .02239 L
.45831 .03112 L
.49959 .04308 L
.53935 .05683 L
.5776 .07186 L
Mistroke
.6183 .0895 L
.65748 .10806 L
.69911 .12938 L
.73923 .15137 L
.77783 .17382 L
.81889 .19901 L
.85843 .22453 L
.89645 .2502 L
.93693 .27872 L
.97589 .30732 L
.97619 .30755 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo000oool0
0`00ooooo`3Nool003Koo`@000Goo`03003ooooo0=koo`00=_oo00<00?ooool01_oo00<00?ooool0
g_oo000gool00`00ooooo`05ool3003Nool0033oo`@000Coo`03003ooooo00Coo`03003ooooo01Oo
oaP00:ooo`00>Ooo00<00?ooool00ooo00<00?ooool04_oo1@006?oo1P00ZOoo000fool01000oooo
o`001Ooo00<00?ooool03ooo0`008ooo1@00Y?oo000gool20006ool00`00ooooo`0=ool2000[ool6
002Nool003ooo`03003ooooo00[oo`<003?oo`@009[oo`00?ooo00<00?ooool02?oo0P00>_oo0`00
Uooo000oool20007ool2000oool3002Dool003ooo`03003ooooo00Coo`8004Coo`<0097oo`00?ooo
00<00?ooool00ooo00<00?ooool0Aooo0`00S_oo000oool01000oooooooo0P00COoo0`00Rooo000o
ool01000ooooo`00D_oo0P00ROoo000oool00`00ool0001Eool30026ool003ooo`8005Woo`<008?o
o`00?_oo0`00G?oo0`00P?oo000mool00`00ool0001Pool2001nool003coo`04003ooooo001Rool3
001kool000;oo`8000Coo`03003ooooo00;oo`<002_oo`05003oooooool0000]ool40003ool00`00
ooooo`02ool3000Yool300000ooo00000002000/ool40003ool00`00ooooo`02ool3000_ool20007
ool00005ool00?ooool0000;ool00`00ooooo`0Wool00`00ooooo`02ool00`00ooooo`0]ool00`00
ooooo`0:ool00`00ooooo`0Yool2000_ool00`00ooooo`0<ool00`00ooooo`0[ool01000ooooo`00
1_oo00001Ooo003ooooo00002ooo00<00?ooool09_oo00<00?ooool00ooo00<00?ooool0;Ooo00<0
0?ooool02_oo00<00?ooool0:ooo0P00;_oo00<00?ooool02ooo00<00?ooool0;_oo00<00?ooool0
1?oo00001Ooo003ooooo00002?oo0`00:?oo00<00?ooool01?oo00<00?ooool0;Ooo00<00?ooool0
1ooo0`00;ooo1000;?oo00<00?ooool01ooo0`00;ooo0P001ooo00001Ooo003ooooo00002?oo00<0
0?ooool09ooo00<00?ooool01Ooo0P00;_oo00<00?ooool01ooo00<00?ooool0<?oo00<00?ooool0
0P00:ooo00<00?ooool01_oo00<00?ooool0<Ooo00<00?ooool01?oo00001Ooo003ooooo00002?oo
00<00?ooool09_oo00<00?ooool01_oo00<00?ooool0:ooo0`002Ooo00<00?ooool0;Ooo00@00?oo
ool000Coo`<002Goo`04003ooooo0008ool00`00ooooo`0^ool01000ooooo`001_oo0002ool20009
ool4000Uool00`00ooooo`06ool00`00ooooo`0]ool00`00ooooo`07ool4000]ool20008ool2000T
ool20009ool4000^ool20007ool003Goo`03003ooooo00Ooo`03003ooooo07Coo`8006Soo`00=?oo
00<00?ooool02?oo00<00?ooool0M_oo0P00I_oo000dool00`00ooooo`08ool00`00ooooo`1hool2
001Tool003?oo`03003ooooo00Woo`03003ooooo07[oo`8006;oo`000_ooo`007P000Ooo0009ool0
0`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`06ool00`00ool0000:ool0
0`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`07ool2000:ool00`00ooooo`08ool00`00ooooo`07ool00`00oooo
o`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00oooo
o`05ool000Woo`03003ooooo02Goo`03003ooooo00_oo`03003ooooo03?oo`03003ooooo03?oo`03
003ooooo01Coo`8001goo`03003ooooo03Coo`03003ooooo00Goo`00<Ooo00<00?ooool02ooo00<0
0?ooool0P_oo0P00F_oo000`ool00`00ooooo`0<ool00`00ooooo`24ool2001Hool0033oo`03003o
oooo00coo`03003ooooo08Koo`8005Koo`00;ooo00<00?ooool03Ooo00<00?ooool0R?oo0P00E?oo
000_ool00`00ooooo`0=ool2002;ool00`00ooooo`1Aool002koo`03003ooooo00koo`03003ooooo
08_oo`80057oo`00;_oo00<00?ooool03_oo00<00?ooool0SOoo0P00Cooo000]ool00`00ooooo`0?
ool00`00ooooo`2?ool00`00ooooo`1<ool002goo`03003ooooo00ooo`03003ooooo093oo`8004co
o`00;?oo00<00?ooool04?oo00<00?ooool0T_oo00<00?ooool0BOoo000/ool00`00ooooo`0@ool0
0`00ooooo`2Cool20019ool002_oo`03003ooooo017oo`8009Koo`8004Ooo`00:ooo00<00?ooool0
4Ooo00<00?ooool0Uooo0P00AOoo000Zool00`00ooooo`0Bool00`00ooooo`2Iool20013ool002[o
o`03003ooooo01;oo`03003ooooo09_oo`80047oo`00:_oo00<00?ooool04_oo00<00?ooool0WOoo
00<00?ooool0?_oo000Yool00`00ooooo`0Cool00`00ooooo`2Nool2000nool002Woo`03003ooooo
01?oo`03003ooooo0:3oo`8003coo`00:Ooo00<00?ooool04ooo0P00Xooo00<00?ooool0>Ooo000X
ool00`00ooooo`0Dool00`00ooooo`2Sool2000iool002Soo`03003ooooo01Coo`03003ooooo0:Go
o`8003Ooo`00:?oo00<00?ooool05?oo00<00?ooool0Yooo0P00=Ooo000Wool00`00ooooo`0Eool0
0`00ooooo`2Yool00`00ooooo`0bool002Ooo`03003ooooo00coo`@000Goo`03003ooooo0:[oo`80
03;oo`009ooo00<00?ooool03?oo00<00?ooool01_oo00<00?ooool0[?oo00<00?ooool0;ooo000V
ool00`00ooooo`0>ool00`00ooooo`05ool3002]ool00`00ooooo`0^ool002Koo`03003ooooo00oo
o`03003ooooo00Coo`03003ooooo0:koo`8002koo`009_oo00<00?ooool04?oo00<00?ooool00ooo
00<00?ooool0/?oo00<00?ooool0:ooo000Uool00`00ooooo`0>ool01000ooooo`001Ooo00<00?oo
ool0/Ooo00<00?ooool0:_oo000Uool00`00ooooo`0?ool20006ool00`00ooooo`2bool2000Zool0
02Goo`03003ooooo01Ooo`03003ooooo0;Coo`03003ooooo02Ooo`009?oo00<00?ooool06?oo00<0
0?ooool0]Ooo0P009ooo000Tool00`00ooooo`0Hool2002hool00`00ooooo`0Tool002?oo`03003o
oooo01Woo`03003ooooo0;Soo`8002Coo`008ooo00<00?ooool06Ooo00<00?ooool0^_oo0P008_oo
000Sool00`00ooooo`0Iool00`00ooooo`2lool00`00ooooo`0Oool002;oo`03003ooooo01[oo`03
003ooooo0;goo`8001ooo`008_oo00<00?ooool06_oo00<00?ooool0_ooo00<00?ooool07?oo000R
ool00`00ooooo`0Jool00`00ooooo`30ool2000Lool002;oo`03003ooooo01[oo`800<?oo`03003o
oooo01Woo`008_oo00<00?ooool06_oo00<00?ooool0`ooo00<00?ooool06?oo000Qool00`00oooo
o`0Kool00`00ooooo`34ool2000Hool0027oo`03003ooooo01_oo`03003ooooo0<Koo`03003ooooo
01Goo`008Ooo00<00?ooool06ooo00<00?ooool0aooo00<00?ooool05?oo000Qool00`00ooooo`0K
ool00`00ooooo`38ool2000Dool0027oo`03003ooooo01_oo`800<_oo`03003ooooo017oo`008Ooo
00<00?ooool06ooo00<00?ooool0booo0P004Ooo000Pool00`00ooooo`0Lool00`00ooooo`3=ool0
0`00ooooo`0>ool0023oo`03003ooooo01coo`03003ooooo0<koo`8000koo`008?oo00<00?ooool0
7?oo00<00?ooool0d?oo00<00?ooool02ooo000Pool00`00ooooo`0Eool00`00ooooo`04ool00`00
ooooo`3Aool2000;ool0023oo`03003ooooo01Goo`03003ooooo00Coo`03003ooooo0=?oo`03003o
oooo00Soo`008?oo00<00?ooool04_oo1@001Ooo0`00e?oo0P002?oo000Oool00`00ooooo`0Cool0
1000ooooo`001_oo00<00?ooool0e_oo00<00?ooool01Ooo000Oool00`00ooooo`0Dool00`00ool0
0006ool00`00ooooo`3Nool001ooo`03003ooooo01Goo`8000Koo`03003ooooo0=koo`007ooo00<0
0?ooool05_oo00<00?ooool01?oo00<00?ooool0g_oo000Oool00`00ooooo`0Mool00`00ooooo`3N
ool001koo`03003ooooo01koo`03003ooooo0=koo`007_oo00<00?ooool07_oo0P00gooo000Nool0
0`00ooooo`0Nool00`00ooooo`3Nool001koo`03003ooooo01koo`03003ooooo0=koo`007_oo00<0
0?ooool07_oo00<00?ooool0g_oo000Mool00`00ooooo`0Oool00`00ooooo`3Nool001goo`03003o
oooo01ooo`03003ooooo0=koo`007Ooo00<00?ooool07ooo00<00?ooool0g_oo000Mool00`00oooo
o`0Oool2003Oool001goo`03003ooooo01ooo`03003ooooo0=koo`007Ooo00<00?ooool07ooo00<0
0?ooool0g_oo000Lool00`00ooooo`0Pool00`00ooooo`3Nool001coo`03003ooooo023oo`03003o
oooo0=koo`007?oo00<00?ooool08?oo00<00?ooool0g_oo000Lool00`00ooooo`0Pool00`00oooo
o`3Nool001coo`03003ooooo023oo`800=ooo`006ooo00<00?ooool08Ooo00<00?ooool0g_oo000K
ool00`00ooooo`0Qool00`00ooooo`3Nool001_oo`03003ooooo027oo`03003ooooo0=koo`006ooo
00<00?ooool08Ooo00<00?ooool0g_oo000Kool00`00ooooo`0Iool20006ool00`00ooooo`3Nool0
01[oo`03003ooooo01Woo`04003ooooo0005ool00`00ooooo`3Nool001[oo`03003ooooo01Woo`04
003ooooo0005ool3003Nool001[oo`03003ooooo01Woo`<000Koo`03003ooooo0=koo`006_oo00<0
0?ooool06Ooo00<00?ooool01_oo00<00?ooool0g_oo000Jool00`00ooooo`0Iool00`00ooooo`06
ool00`00ooooo`3Nool001[oo`03003ooooo01[oo`<000Goo`03003ooooo0=koo`006_oo00<00?oo
ool08_oo00<00?ooool0g_oo000Iool00`00ooooo`0Sool00`00ooooo`3Nool001Woo`03003ooooo
02?oo`800=ooo`006Ooo00<00?ooool08ooo00<00?ooool0g_oo000Iool00`00ooooo`0Sool00`00
ooooo`3Nool001Woo`03003ooooo02?oo`03003ooooo0=koo`006Ooo00<00?ooool08ooo00<00?oo
ool0g_oo000Iool00`00ooooo`0Sool00`00ooooo`3Nool001Woo`03003ooooo02?oo`03003ooooo
0=koo`006?oo00<00?ooool09?oo0P00gooo000Hool00`00ooooo`0Tool00`00ooooo`3Nool001So
o`03003ooooo02Coo`03003ooooo0=koo`006?oo00<00?ooool09?oo00<00?ooool0g_oo000Hool0
0`00ooooo`0Tool00`00ooooo`3Nool001Soo`03003ooooo02Coo`03003ooooo0=koo`006?oo00<0
0?ooool09?oo00<00?ooool0g_oo000Hool00`00ooooo`0Tool2003Oool001Ooo`03003ooooo02Go
o`03003ooooo0=koo`005ooo00<00?ooool09Ooo00<00?ooool0g_oo000Gool00`00ooooo`0Uool0
0`00ooooo`3Nool001Ooo`03003ooooo02Goo`03003ooooo0=koo`005ooo00<00?ooool07Ooo0P00
1_oo00<00?ooool0g_oo000Gool00`00ooooo`0Lool01000ooooo`001Ooo00<00?ooool0g_oo000G
ool00`00ooooo`0Lool01000ooooo`001Ooo0`00g_oo000Gool00`00ooooo`0Mool20006ool00`00
ooooo`3Nool001Koo`03003ooooo01goo`04003ooooo0005ool00`00ooooo`3Nool001Koo`03003o
oooo01goo`04003ooooo0005ool00`00ooooo`3Nool001Koo`03003ooooo01koo`8000Koo`03003o
oooo0=koo`005_oo00<00?ooool09_oo00<00?ooool0g_oo000Fool00`00ooooo`0Vool00`00oooo
o`3Nool001Koo`03003ooooo02Koo`800=ooo`005_oo00<00?ooool09_oo00<00?ooool0g_oo000F
ool00`00ooooo`0Vool00`00ooooo`3Nool001Koo`03003ooooo02Koo`03003ooooo0=koo`005Ooo
00<00?ooool09ooo00<00?ooool0g_oo000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03
003ooooo02Ooo`03003ooooo0=koo`005Ooo00<00?ooool09ooo0P00gooo000Eool00`00ooooo`0W
ool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03003ooooo0=koo`005Ooo00<00?ooool09ooo
00<00?ooool0g_oo000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03
003ooooo0=koo`005?oo00<00?ooool0:?oo00<00?ooool0g_oo000Dool00`00ooooo`0Xool2003O
ool001Coo`03003ooooo02Soo`03003ooooo0=koo`005?oo00<00?ooool0:?oo00<00?ooool0g_oo
000Dool00`00ooooo`0Xool00`00ooooo`3Nool001Coo`03003ooooo02Soo`03003ooooo0=koo`00
5?oo00<00?ooool06Ooo10000ooo0P001_oo00<00?ooool0g_oo000Dool00`00ooooo`0Kool01@00
oooooooo00000_oo00<00?ooool00ooo00<00?ooool0g_oo000Dool00`00ooooo`0Kool01@00oooo
oooo00000_oo00<00?ooool00ooo0`00g_oo000Cool00`00ooooo`0Lool01@00oooooooo00000_oo
00<00?ooool00ooo00<00?ooool0g_oo000Cool00`00ooooo`0Lool01@00oooooooo00000_oo00<0
0?ooool00ooo00<00?ooool0g_oo000Cool00`00ooooo`0Jool30003ool01000ooooo`001Ooo00<0
0?ooool0g_oo000Cool00`00ooooo`0Lool00`00ooooo`02ool20006ool00`00ooooo`3Nool001?o
o`03003ooooo02Woo`03003ooooo0=koo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.416264, -2.32394, \
0.0092229, 0.0719848}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[LXtestfunc[om, 2, 0.0001]], {om, 0.5, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.380952 0.110052 0.0488076 [
[.02381 .09755 -9 -9 ]
[.02381 .09755 9 0 ]
[.40476 .09755 -9 -9 ]
[.40476 .09755 9 0 ]
[.59524 .09755 -3 -9 ]
[.59524 .09755 3 0 ]
[.78571 .09755 -9 -9 ]
[.78571 .09755 9 0 ]
[.97619 .09755 -3 -9 ]
[.97619 .09755 3 0 ]
[.20179 .01244 -12 -4.5 ]
[.20179 .01244 0 4.5 ]
[.20179 .20767 -6 -4.5 ]
[.20179 .20767 0 4.5 ]
[.20179 .30528 -6 -4.5 ]
[.20179 .30528 0 4.5 ]
[.20179 .4029 -6 -4.5 ]
[.20179 .4029 0 4.5 ]
[.20179 .50051 -6 -4.5 ]
[.20179 .50051 0 4.5 ]
[.20179 .59813 -12 -4.5 ]
[.20179 .59813 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .11005 m
.02381 .1163 L
s
[(0.5)] .02381 .09755 0 1 Mshowa
.40476 .11005 m
.40476 .1163 L
s
[(1.5)] .40476 .09755 0 1 Mshowa
.59524 .11005 m
.59524 .1163 L
s
[(2)] .59524 .09755 0 1 Mshowa
.78571 .11005 m
.78571 .1163 L
s
[(2.5)] .78571 .09755 0 1 Mshowa
.97619 .11005 m
.97619 .1163 L
s
[(3)] .97619 .09755 0 1 Mshowa
.125 Mabswid
.0619 .11005 m
.0619 .1138 L
s
.1 .11005 m
.1 .1138 L
s
.1381 .11005 m
.1381 .1138 L
s
.17619 .11005 m
.17619 .1138 L
s
.25238 .11005 m
.25238 .1138 L
s
.29048 .11005 m
.29048 .1138 L
s
.32857 .11005 m
.32857 .1138 L
s
.36667 .11005 m
.36667 .1138 L
s
.44286 .11005 m
.44286 .1138 L
s
.48095 .11005 m
.48095 .1138 L
s
.51905 .11005 m
.51905 .1138 L
s
.55714 .11005 m
.55714 .1138 L
s
.63333 .11005 m
.63333 .1138 L
s
.67143 .11005 m
.67143 .1138 L
s
.70952 .11005 m
.70952 .1138 L
s
.74762 .11005 m
.74762 .1138 L
s
.82381 .11005 m
.82381 .1138 L
s
.8619 .11005 m
.8619 .1138 L
s
.9 .11005 m
.9 .1138 L
s
.9381 .11005 m
.9381 .1138 L
s
.25 Mabswid
0 .11005 m
1 .11005 L
s
.21429 .01244 m
.22054 .01244 L
s
[(-2)] .20179 .01244 1 0 Mshowa
.21429 .20767 m
.22054 .20767 L
s
[(2)] .20179 .20767 1 0 Mshowa
.21429 .30528 m
.22054 .30528 L
s
[(4)] .20179 .30528 1 0 Mshowa
.21429 .4029 m
.22054 .4029 L
s
[(6)] .20179 .4029 1 0 Mshowa
.21429 .50051 m
.22054 .50051 L
s
[(8)] .20179 .50051 1 0 Mshowa
.21429 .59813 m
.22054 .59813 L
s
[(10)] .20179 .59813 1 0 Mshowa
.125 Mabswid
.21429 .03684 m
.21804 .03684 L
s
.21429 .06124 m
.21804 .06124 L
s
.21429 .08565 m
.21804 .08565 L
s
.21429 .13446 m
.21804 .13446 L
s
.21429 .15886 m
.21804 .15886 L
s
.21429 .18326 m
.21804 .18326 L
s
.21429 .23207 m
.21804 .23207 L
s
.21429 .25647 m
.21804 .25647 L
s
.21429 .28088 m
.21804 .28088 L
s
.21429 .32969 m
.21804 .32969 L
s
.21429 .35409 m
.21804 .35409 L
s
.21429 .37849 m
.21804 .37849 L
s
.21429 .4273 m
.21804 .4273 L
s
.21429 .45171 m
.21804 .45171 L
s
.21429 .47611 m
.21804 .47611 L
s
.21429 .52492 m
.21804 .52492 L
s
.21429 .54932 m
.21804 .54932 L
s
.21429 .57372 m
.21804 .57372 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.06011 .61803 m
.06244 .5888 L
.07298 .49287 L
.08426 .4098 L
.10458 .29726 L
.1148 .25405 L
.12589 .21476 L
.14603 .15899 L
.16586 .11861 L
.18689 .087 L
.20643 .06521 L
.22473 .04977 L
.24497 .03703 L
.2635 .02853 L
.28248 .02231 L
.29323 .01974 L
.30321 .01788 L
.30828 .01712 L
.31377 .01642 L
.31862 .01591 L
.32373 .01548 L
.32835 .01517 L
.3308 .01504 L
.33342 .01493 L
.33583 .01485 L
.33803 .01479 L
.33909 .01477 L
.34024 .01475 L
.34133 .01473 L
.34234 .01472 L
.34351 .01472 L
.34475 .01472 L
.34581 .01472 L
.34697 .01472 L
.34822 .01473 L
.34892 .01474 L
.34956 .01475 L
.35196 .0148 L
.35419 .01486 L
.35661 .01494 L
.361 .01513 L
.36621 .01542 L
.37172 .01582 L
.3816 .01673 L
.40118 .01924 L
.41949 .02236 L
.45831 .03108 L
.49959 .04303 L
.53935 .05677 L
.5776 .07178 L
Mistroke
.6183 .08949 L
.65748 .1081 L
.69911 .12943 L
.73923 .15142 L
.77783 .17386 L
.81889 .19904 L
.85843 .22454 L
.89645 .2502 L
.93693 .27871 L
.97589 .3073 L
.97619 .30752 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo000oool0
0`00ooooo`3Nool003Koo`@000Goo`03003ooooo0=koo`00=_oo00<00?ooool01_oo00<00?ooool0
g_oo000gool00`00ooooo`05ool3003Nool0033oo`@000Coo`03003ooooo00Coo`03003ooooo01Oo
oaP00:ooo`00>Ooo00<00?ooool00ooo00<00?ooool04_oo1@006?oo1P00ZOoo000fool01000oooo
o`001Ooo00<00?ooool03ooo0`008ooo1@00Y?oo000gool20006ool00`00ooooo`0=ool2000[ool6
002Nool003ooo`03003ooooo00[oo`<003?oo`@009[oo`00?ooo00<00?ooool01ooo0`00>_oo0`00
Uooo000oool20006ool20010ool3002Dool003ooo`03003ooooo00Coo`03003ooooo04?oo`<0097o
o`00?ooo00<00?ooool00_oo0P00BOoo0`00S_oo000oool01@00oooooooo0000C_oo0`00Rooo000o
ool01000ooooo`00D_oo0P00ROoo000oool00`00ool0001Eool30026ool003ooo`8005Woo`<008?o
o`00?_oo0`00G?oo0`00P?oo000mool00`00ool0001Pool2001nool003coo`04003ooooo001Rool3
001kool000;oo`8000Coo`03003ooooo00;oo`<002_oo`05003oooooool0000]ool40003ool00`00
ooooo`02ool3000Yool300000ooo00000002000/ool40003ool00`00ooooo`02ool3000_ool20007
ool00005ool00?ooool0000;ool00`00ooooo`0Wool00`00ooooo`02ool00`00ooooo`0]ool00`00
ooooo`0:ool00`00ooooo`0Yool2000_ool00`00ooooo`0<ool00`00ooooo`0[ool01000ooooo`00
1_oo00001Ooo003ooooo00002ooo00<00?ooool09_oo00<00?ooool00ooo00<00?ooool0;Ooo00<0
0?ooool02_oo00<00?ooool0:ooo0P00;_oo00<00?ooool02ooo00<00?ooool0;_oo00<00?ooool0
1?oo00001Ooo003ooooo00002?oo0`00:?oo00<00?ooool01?oo00<00?ooool0;Ooo00<00?ooool0
1ooo0`00;ooo1000;?oo00<00?ooool01ooo0`00;ooo0P001ooo00001Ooo003ooooo00002?oo00<0
0?ooool09ooo00<00?ooool01Ooo0P00;_oo00<00?ooool01ooo00<00?ooool0<?oo00<00?ooool0
0P00:ooo00<00?ooool01_oo00<00?ooool0<Ooo00<00?ooool01?oo00001Ooo003ooooo00002?oo
00<00?ooool09_oo00<00?ooool01_oo00<00?ooool0:ooo0`002Ooo00<00?ooool0;Ooo00@00?oo
ool000Coo`<002Goo`04003ooooo0008ool00`00ooooo`0^ool01000ooooo`001_oo0002ool20009
ool4000Uool00`00ooooo`06ool00`00ooooo`0]ool00`00ooooo`07ool4000]ool20008ool2000T
ool20009ool4000^ool20007ool003Goo`03003ooooo00Ooo`03003ooooo07Coo`8006Soo`00=?oo
00<00?ooool02?oo00<00?ooool0M_oo0P00I_oo000dool00`00ooooo`08ool00`00ooooo`1hool2
001Tool003?oo`03003ooooo00Woo`03003ooooo07[oo`8006;oo`000_ooo`007P000Ooo0009ool0
0`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`06ool00`00ool0000:ool0
0`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`07ool2000:ool00`00ooooo`08ool00`00ooooo`07ool00`00oooo
o`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00oooo
o`05ool000Woo`03003ooooo02Goo`03003ooooo00_oo`03003ooooo03?oo`03003ooooo03?oo`03
003ooooo01Coo`8001goo`03003ooooo03Coo`03003ooooo00Goo`00<Ooo00<00?ooool02ooo00<0
0?ooool0P_oo0P00F_oo000`ool00`00ooooo`0<ool00`00ooooo`24ool2001Hool0033oo`03003o
oooo00coo`03003ooooo08Koo`8005Koo`00;ooo00<00?ooool03Ooo00<00?ooool0R?oo0P00E?oo
000_ool00`00ooooo`0=ool2002;ool00`00ooooo`1Aool002koo`03003ooooo00koo`03003ooooo
08_oo`80057oo`00;_oo00<00?ooool03_oo00<00?ooool0SOoo0P00Cooo000]ool00`00ooooo`0?
ool00`00ooooo`2?ool00`00ooooo`1<ool002goo`03003ooooo00ooo`03003ooooo093oo`8004co
o`00;?oo00<00?ooool04?oo00<00?ooool0T_oo00<00?ooool0BOoo000/ool00`00ooooo`0@ool0
0`00ooooo`2Cool20019ool002_oo`03003ooooo017oo`8009Koo`8004Ooo`00:ooo00<00?ooool0
4Ooo00<00?ooool0Uooo0P00AOoo000Zool00`00ooooo`0Bool00`00ooooo`2Iool20013ool002[o
o`03003ooooo01;oo`03003ooooo09_oo`80047oo`00:_oo00<00?ooool04_oo00<00?ooool0WOoo
00<00?ooool0?_oo000Yool00`00ooooo`0Cool00`00ooooo`2Nool2000nool002Woo`03003ooooo
01?oo`03003ooooo0:3oo`8003coo`00:Ooo00<00?ooool04ooo0P00Xooo00<00?ooool0>Ooo000Y
ool00`00ooooo`0Cool00`00ooooo`2Sool2000iool002Soo`03003ooooo01Coo`03003ooooo0:Go
o`8003Ooo`00:?oo00<00?ooool05?oo00<00?ooool0Yooo0P00=Ooo000Xool00`00ooooo`0Dool0
0`00ooooo`2Yool00`00ooooo`0bool002Ooo`03003ooooo00coo`@000Goo`03003ooooo0:[oo`80
03;oo`009ooo00<00?ooool03?oo00<00?ooool01_oo00<00?ooool0[?oo00<00?ooool0;ooo000W
ool00`00ooooo`0=ool00`00ooooo`05ool3002]ool2000_ool002Koo`03003ooooo00ooo`03003o
oooo00Coo`03003ooooo0:ooo`8002goo`009_oo00<00?ooool04?oo00<00?ooool00ooo00<00?oo
ool0/Ooo00<00?ooool0:_oo000Vool00`00ooooo`0=ool01000ooooo`001Ooo00<00?ooool0/_oo
0P00:_oo000Uool00`00ooooo`0?ool20006ool00`00ooooo`2dool00`00ooooo`0Wool002Goo`03
003ooooo01Ooo`03003ooooo0;Goo`8002Ooo`009Ooo00<00?ooool05ooo00<00?ooool0]ooo00<0
0?ooool09?oo000Tool00`00ooooo`0Hool2002iool00`00ooooo`0Sool002Coo`03003ooooo01So
o`03003ooooo0;Woo`8002?oo`008ooo00<00?ooool06Ooo00<00?ooool0^ooo00<00?ooool08?oo
000Sool00`00ooooo`0Iool00`00ooooo`2lool00`00ooooo`0Oool002?oo`03003ooooo01Woo`03
003ooooo0;goo`8001ooo`008_oo00<00?ooool06_oo00<00?ooool0_ooo00<00?ooool07?oo000R
ool00`00ooooo`0Jool00`00ooooo`30ool2000Lool002;oo`03003ooooo01[oo`800<?oo`03003o
oooo01Woo`008_oo00<00?ooool06_oo00<00?ooool0`ooo0P006Ooo000Rool00`00ooooo`0Jool0
0`00ooooo`35ool00`00ooooo`0Fool0027oo`03003ooooo01_oo`03003ooooo0<Koo`8001Koo`00
8Ooo00<00?ooool06ooo00<00?ooool0b?oo00<00?ooool04ooo000Qool00`00ooooo`0Kool00`00
ooooo`39ool2000Cool0027oo`03003ooooo01_oo`800<coo`03003ooooo013oo`008Ooo00<00?oo
ool06ooo00<00?ooool0c?oo0P004?oo000Qool00`00ooooo`0Kool00`00ooooo`3>ool00`00oooo
o`0=ool0023oo`03003ooooo01coo`03003ooooo0<ooo`03003ooooo00coo`008?oo00<00?ooool0
7?oo00<00?ooool0d?oo0P003?oo000Pool00`00ooooo`0Eool00`00ooooo`04ool00`00ooooo`3B
ool00`00ooooo`09ool0023oo`03003ooooo01Goo`03003ooooo00Coo`03003ooooo0=?oo`03003o
oooo00Soo`008?oo00<00?ooool04_oo1@001Ooo0`00e?oo0P002?oo000Oool00`00ooooo`0Cool0
1000ooooo`001_oo00<00?ooool0e_oo00<00?ooool01Ooo000Oool00`00ooooo`0Dool00`00ool0
0006ool00`00ooooo`3Nool001ooo`03003ooooo01Goo`8000Koo`03003ooooo0=koo`007ooo00<0
0?ooool05_oo00<00?ooool01?oo00<00?ooool0g_oo000Oool00`00ooooo`0Mool00`00ooooo`3N
ool001ooo`03003ooooo01goo`03003ooooo0=koo`007_oo00<00?ooool07_oo0P00gooo000Nool0
0`00ooooo`0Nool00`00ooooo`3Nool001koo`03003ooooo01koo`03003ooooo0=koo`007_oo00<0
0?ooool07_oo00<00?ooool0g_oo000Nool00`00ooooo`0Nool00`00ooooo`3Nool001goo`03003o
oooo01ooo`03003ooooo0=koo`007Ooo00<00?ooool07ooo00<00?ooool0g_oo000Mool00`00oooo
o`0Oool2003Oool001goo`03003ooooo01ooo`03003ooooo0=koo`007Ooo00<00?ooool07ooo00<0
0?ooool0g_oo000Lool00`00ooooo`0Pool00`00ooooo`3Nool001coo`03003ooooo023oo`03003o
oooo0=koo`007?oo00<00?ooool08?oo00<00?ooool0g_oo000Lool00`00ooooo`0Pool00`00oooo
o`3Nool001coo`03003ooooo023oo`800=ooo`007?oo00<00?ooool08?oo00<00?ooool0g_oo000K
ool00`00ooooo`0Qool00`00ooooo`3Nool001_oo`03003ooooo027oo`03003ooooo0=koo`006ooo
00<00?ooool08Ooo00<00?ooool0g_oo000Kool00`00ooooo`0Iool20006ool00`00ooooo`3Nool0
01_oo`03003ooooo01Soo`04003ooooo0005ool00`00ooooo`3Nool001[oo`03003ooooo01Woo`04
003ooooo0005ool3003Nool001[oo`03003ooooo01Woo`<000Koo`03003ooooo0=koo`006_oo00<0
0?ooool06Ooo00<00?ooool01_oo00<00?ooool0g_oo000Jool00`00ooooo`0Iool00`00ooooo`06
ool00`00ooooo`3Nool001[oo`03003ooooo01[oo`<000Goo`03003ooooo0=koo`006_oo00<00?oo
ool08_oo00<00?ooool0g_oo000Jool00`00ooooo`0Rool00`00ooooo`3Nool001Woo`03003ooooo
02?oo`800=ooo`006Ooo00<00?ooool08ooo00<00?ooool0g_oo000Iool00`00ooooo`0Sool00`00
ooooo`3Nool001Woo`03003ooooo02?oo`03003ooooo0=koo`006Ooo00<00?ooool08ooo00<00?oo
ool0g_oo000Iool00`00ooooo`0Sool00`00ooooo`3Nool001Woo`03003ooooo02?oo`03003ooooo
0=koo`006Ooo00<00?ooool08ooo0P00gooo000Hool00`00ooooo`0Tool00`00ooooo`3Nool001So
o`03003ooooo02Coo`03003ooooo0=koo`006?oo00<00?ooool09?oo00<00?ooool0g_oo000Hool0
0`00ooooo`0Tool00`00ooooo`3Nool001Soo`03003ooooo02Coo`03003ooooo0=koo`006?oo00<0
0?ooool09?oo00<00?ooool0g_oo000Hool00`00ooooo`0Tool2003Oool001Soo`03003ooooo02Co
o`03003ooooo0=koo`005ooo00<00?ooool09Ooo00<00?ooool0g_oo000Gool00`00ooooo`0Uool0
0`00ooooo`3Nool001Ooo`03003ooooo02Goo`03003ooooo0=koo`005ooo00<00?ooool07Ooo0P00
1_oo00<00?ooool0g_oo000Gool00`00ooooo`0Lool01000ooooo`001Ooo00<00?ooool0g_oo000G
ool00`00ooooo`0Lool01000ooooo`001Ooo0`00g_oo000Gool00`00ooooo`0Mool20006ool00`00
ooooo`3Nool001Ooo`03003ooooo01coo`04003ooooo0005ool00`00ooooo`3Nool001Koo`03003o
oooo01goo`04003ooooo0005ool00`00ooooo`3Nool001Koo`03003ooooo01koo`8000Koo`03003o
oooo0=koo`005_oo00<00?ooool09_oo00<00?ooool0g_oo000Fool00`00ooooo`0Vool00`00oooo
o`3Nool001Koo`03003ooooo02Koo`800=ooo`005_oo00<00?ooool09_oo00<00?ooool0g_oo000F
ool00`00ooooo`0Vool00`00ooooo`3Nool001Koo`03003ooooo02Koo`03003ooooo0=koo`005_oo
00<00?ooool09_oo00<00?ooool0g_oo000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03
003ooooo02Ooo`03003ooooo0=koo`005Ooo00<00?ooool09ooo0P00gooo000Eool00`00ooooo`0W
ool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03003ooooo0=koo`005Ooo00<00?ooool09ooo
00<00?ooool0g_oo000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03
003ooooo0=koo`005Ooo00<00?ooool09ooo00<00?ooool0g_oo000Dool00`00ooooo`0Xool2003O
ool001Coo`03003ooooo02Soo`03003ooooo0=koo`005?oo00<00?ooool0:?oo00<00?ooool0g_oo
000Dool00`00ooooo`0Xool00`00ooooo`3Nool001Coo`03003ooooo02Soo`03003ooooo0=koo`00
5?oo00<00?ooool06Ooo10000ooo0P001_oo00<00?ooool0g_oo000Dool00`00ooooo`0Kool01@00
oooooooo00000_oo00<00?ooool00ooo00<00?ooool0g_oo000Dool00`00ooooo`0Kool01@00oooo
oooo00000_oo00<00?ooool00ooo0`00g_oo000Dool00`00ooooo`0Kool01@00oooooooo00000_oo
00<00?ooool00ooo00<00?ooool0g_oo000Cool00`00ooooo`0Lool01@00oooooooo00000_oo00<0
0?ooool00ooo00<00?ooool0g_oo000Cool00`00ooooo`0Jool30003ool01000ooooo`001Ooo00<0
0?ooool0g_oo000Cool00`00ooooo`0Lool00`00ooooo`02ool20006ool00`00ooooo`3Nool001?o
o`03003ooooo02Woo`03003ooooo0=koo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.41685, -2.3238, \
0.00921894, 0.0719555}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %231]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.380952 0.282652 0.0317228 [
[.02381 .27015 -9 -9 ]
[.02381 .27015 9 0 ]
[.40476 .27015 -9 -9 ]
[.40476 .27015 9 0 ]
[.59524 .27015 -3 -9 ]
[.59524 .27015 3 0 ]
[.78571 .27015 -9 -9 ]
[.78571 .27015 9 0 ]
[.97619 .27015 -3 -9 ]
[.97619 .27015 3 0 ]
[.20179 .04473 -24 -4.5 ]
[.20179 .04473 0 4.5 ]
[.20179 .12404 -12 -4.5 ]
[.20179 .12404 0 4.5 ]
[.20179 .20334 -24 -4.5 ]
[.20179 .20334 0 4.5 ]
[.20179 .36196 -18 -4.5 ]
[.20179 .36196 0 4.5 ]
[.20179 .44127 -6 -4.5 ]
[.20179 .44127 0 4.5 ]
[.20179 .52057 -18 -4.5 ]
[.20179 .52057 0 4.5 ]
[.20179 .59988 -12 -4.5 ]
[.20179 .59988 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .28265 m
.02381 .2889 L
s
[(0.5)] .02381 .27015 0 1 Mshowa
.40476 .28265 m
.40476 .2889 L
s
[(1.5)] .40476 .27015 0 1 Mshowa
.59524 .28265 m
.59524 .2889 L
s
[(2)] .59524 .27015 0 1 Mshowa
.78571 .28265 m
.78571 .2889 L
s
[(2.5)] .78571 .27015 0 1 Mshowa
.97619 .28265 m
.97619 .2889 L
s
[(3)] .97619 .27015 0 1 Mshowa
.125 Mabswid
.0619 .28265 m
.0619 .2864 L
s
.1 .28265 m
.1 .2864 L
s
.1381 .28265 m
.1381 .2864 L
s
.17619 .28265 m
.17619 .2864 L
s
.25238 .28265 m
.25238 .2864 L
s
.29048 .28265 m
.29048 .2864 L
s
.32857 .28265 m
.32857 .2864 L
s
.36667 .28265 m
.36667 .2864 L
s
.44286 .28265 m
.44286 .2864 L
s
.48095 .28265 m
.48095 .2864 L
s
.51905 .28265 m
.51905 .2864 L
s
.55714 .28265 m
.55714 .2864 L
s
.63333 .28265 m
.63333 .2864 L
s
.67143 .28265 m
.67143 .2864 L
s
.70952 .28265 m
.70952 .2864 L
s
.74762 .28265 m
.74762 .2864 L
s
.82381 .28265 m
.82381 .2864 L
s
.8619 .28265 m
.8619 .2864 L
s
.9 .28265 m
.9 .2864 L
s
.9381 .28265 m
.9381 .2864 L
s
.25 Mabswid
0 .28265 m
1 .28265 L
s
.21429 .04473 m
.22054 .04473 L
s
[(-7.5)] .20179 .04473 1 0 Mshowa
.21429 .12404 m
.22054 .12404 L
s
[(-5)] .20179 .12404 1 0 Mshowa
.21429 .20334 m
.22054 .20334 L
s
[(-2.5)] .20179 .20334 1 0 Mshowa
.21429 .36196 m
.22054 .36196 L
s
[(2.5)] .20179 .36196 1 0 Mshowa
.21429 .44127 m
.22054 .44127 L
s
[(5)] .20179 .44127 1 0 Mshowa
.21429 .52057 m
.22054 .52057 L
s
[(7.5)] .20179 .52057 1 0 Mshowa
.21429 .59988 m
.22054 .59988 L
s
[(10)] .20179 .59988 1 0 Mshowa
.125 Mabswid
.21429 .06059 m
.21804 .06059 L
s
.21429 .07645 m
.21804 .07645 L
s
.21429 .09231 m
.21804 .09231 L
s
.21429 .10818 m
.21804 .10818 L
s
.21429 .1399 m
.21804 .1399 L
s
.21429 .15576 m
.21804 .15576 L
s
.21429 .17162 m
.21804 .17162 L
s
.21429 .18748 m
.21804 .18748 L
s
.21429 .21921 m
.21804 .21921 L
s
.21429 .23507 m
.21804 .23507 L
s
.21429 .25093 m
.21804 .25093 L
s
.21429 .26679 m
.21804 .26679 L
s
.21429 .29851 m
.21804 .29851 L
s
.21429 .31437 m
.21804 .31437 L
s
.21429 .33024 m
.21804 .33024 L
s
.21429 .3461 m
.21804 .3461 L
s
.21429 .37782 m
.21804 .37782 L
s
.21429 .39368 m
.21804 .39368 L
s
.21429 .40954 m
.21804 .40954 L
s
.21429 .4254 m
.21804 .4254 L
s
.21429 .45713 m
.21804 .45713 L
s
.21429 .47299 m
.21804 .47299 L
s
.21429 .48885 m
.21804 .48885 L
s
.21429 .50471 m
.21804 .50471 L
s
.21429 .53643 m
.21804 .53643 L
s
.21429 .5523 m
.21804 .5523 L
s
.21429 .56816 m
.21804 .56816 L
s
.21429 .58402 m
.21804 .58402 L
s
.21429 .02887 m
.21804 .02887 L
s
.21429 .01301 m
.21804 .01301 L
s
.21429 .61574 m
.21804 .61574 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05947 .61803 m
.06244 .59382 L
.07298 .53147 L
.08426 .47747 L
.10458 .40433 L
.1148 .37625 L
.12589 .35071 L
.14603 .31446 L
.16586 .28821 L
.18689 .26767 L
.20643 .25351 L
.22473 .24347 L
.24497 .23519 L
.2635 .22966 L
.28248 .22562 L
.29323 .22395 L
.30321 .22274 L
.30828 .22225 L
.31377 .22179 L
.31862 .22146 L
.32373 .22118 L
.32835 .22098 L
.3308 .2209 L
.33342 .22083 L
.33583 .22077 L
.33803 .22074 L
.33909 .22072 L
.34024 .22071 L
.34133 .2207 L
.34234 .22069 L
.34351 .22069 L
.34475 .22069 L
.34581 .22069 L
.34697 .22069 L
.34822 .2207 L
.34892 .22071 L
.34956 .22071 L
.35196 .22074 L
.35419 .22078 L
.35661 .22083 L
.361 .22095 L
.36621 .22115 L
.37172 .22141 L
.3816 .222 L
.40118 .22363 L
.41949 .22565 L
.45831 .23132 L
.49959 .23909 L
.53935 .24802 L
.5776 .25778 L
Mistroke
.6183 .26929 L
.65748 .28138 L
.69911 .29525 L
.73923 .30954 L
.77783 .32412 L
.81889 .34049 L
.85843 .35707 L
.89645 .37374 L
.93693 .39227 L
.97589 .41085 L
.97619 .411 L
Mfstroke
.02381 .01472 m
.06244 .06234 L
.10458 .10873 L
.12507 .1268 L
.14415 .14098 L
.18221 .16273 L
.20262 .17158 L
.2212 .17836 L
.26113 .19002 L
.29954 .19898 L
.34041 .20741 L
.37976 .21527 L
.4176 .22303 L
.45788 .2318 L
.49666 .24091 L
.53788 .25143 L
.57759 .26267 L
.61578 .27267 L
.65643 .28441 L
.69556 .2973 L
.73714 .31215 L
.77721 .32743 L
.81576 .34296 L
.85676 .36031 L
.89625 .37781 L
.93422 .39535 L
.97464 .41477 L
.97619 .41553 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O?ooo00<00?ooool0
g_oo000oool00`00ooooo`3Nool003ooo`03003ooooo0=koo`00?ooo00<00?ooool0g_oo0009ool0
0`00ooooo`0cool2003Oool000[oo`03003ooooo03;oo`03003ooooo0=koo`002ooo00<00?ooool0
<Ooo00<00?ooool0g_oo000;ool00`00ooooo`0aool00`00ooooo`3Nool000coo`03003ooooo033o
o`800=ooo`003Ooo00<00?ooool0;ooo00<00?ooool0g_oo000>ool00`00ooooo`0^ool00`00oooo
o`3Nool000koo`03003ooooo01[oo`03003ooooo00?oo`03003ooooo00;oo`<000Koo`03003ooooo
0=koo`003ooo00<00?ooool06Ooo00<00?ooool02ooo00<00?ooool00ooo00<00?ooool0g_oo000@
ool00`00ooooo`0Iool00`00ooooo`0:ool00`00ooooo`03ool3003Nool0017oo`03003ooooo013o
o`@000Coo`03003ooooo00Ooo`<000Koo`03003ooooo0=koo`004_oo00<00?ooool06?oo00<00?oo
ool01_oo00<00?ooool01_oo00<00?ooool0g_oo000Bool00`00ooooo`0Eool01000ooooo`002?oo
00<00?ooool01_oo00<00?ooool0g_oo000Cool00`00ooooo`0Dool40008ool40005ool2003Oool0
01Coo`03003ooooo02Soo`03003ooooo0=koo`005Ooo00<00?ooool09ooo00<00?ooool0g_oo000F
ool00`00ooooo`0Vool00`00ooooo`3Nool001Ooo`03003ooooo02Goo`03003ooooo0=koo`006?oo
00<00?ooool09?oo0P00gooo000Iool00`00ooooo`0Sool00`00ooooo`3Nool001[oo`03003ooooo
02;oo`03003ooooo0=koo`006_oo00<00?ooool08_oo00<00?ooool0g_oo000Kool00`00ooooo`0Q
ool2003Oool001coo`03003ooooo023oo`03003ooooo0=koo`007Ooo00<00?ooool07ooo00<00?oo
ool0g_oo000Nool00`00ooooo`0Nool00`00ooooo`3Nool001ooo`03003ooooo01goo`03003ooooo
0=koo`008?oo00<00?ooool07?oo0P00gooo000Qool00`00ooooo`0Kool00`00ooooo`3Nool002;o
o`03003ooooo017oo`<000Koo`03003ooooo0=koo`008ooo00<00?ooool04ooo00<00?ooool00ooo
00<00?ooool0g_oo000Tool00`00ooooo`0Bool00`00ooooo`03ool3003Nool002Goo`03003ooooo
00Soo`@000;oo`<000Koo`03003ooooo0=koo`009_oo0P003_oo00<00?ooool01_oo00<00?ooool0
g_oo000Xool00`00ooooo`0;ool00`00ooooo`06ool00`00ooooo`3Nool002Woo`8000_oo`@000Go
o`03003ooooo0=koo`00:ooo00<00?ooool04Ooo0P00gooo000/ool2000Aool00`00ooooo`3Nool0
02koo`8000ooo`03003ooooo0=koo`00<?oo0P003Ooo00<00?ooool0g_oo000bool2000;ool2003O
ool003Coo`8000Woo`03003ooooo0=koo`00=_oo00<00?ooool01_oo00<00?ooool0g_oo000gool2
0006ool00`00ooooo`3Nool003Woo`8000Coo`03003ooooo0=koo`00>ooo0P000_oo0P00gooo000m
ool3003Pool003ooo`@00=goo`00?ooo00@00?ooooooo`@00=Woo`00?ooo0P001_oo1000eOoo000o
ool00`00ooooo`09ool3003Bool003ooo`03003ooooo00coo`@00<koo`00:_oo10000ooo00<00?oo
ool00_oo0`001_oo00<00?ooool04?oo1000b_oo000Zool00`00ooooo`0<ool00`00ooooo`03ool0
0`00ooooo`0Dool40036ool002_oo`03003ooooo00_oo`03003ooooo00?oo`<001Soo`H00<3oo`00
9?oo10001?oo00<00?ooool01ooo0`001_oo00<00?ooool07_oo1P00^_oo000]ool00`00ooooo`06
ool00`00ooooo`06ool00`00ooooo`0Tool6002dool002[oo`04003ooooo0008ool00`00ooooo`06
ool00`00ooooo`0Zool4002`ool002_oo`8000Woo`@000Goo`8002ooo`@00:coo`00?ooo00<00?oo
ool05Ooo6P000ooo1000Z?oo000oool00`00ooooo`0>ool7000Jool;002Tool003ooo`03003ooooo
00[oo`@002coo`H009koo`00?ooo00<00?ooool01ooo0`00=_oo1@00VOoo000oool20005ool3000m
ool7002Cool003ooo`03003ooooo00;oo`8004Coo`P008koo`00?ooo00<00?ooool00P00B_oo0`00
00?oo`0000000P00R_oo0002ool20004ool00`00ooooo`02ool3000_ool3000[ool40003ool00`00
ooooo`02ool3000Eool400000ooo00000002000?ool4000/ool40003ool00`00ooooo`02ool3000_
ool20007ool00005ool00?ooool0000;ool00`00ooooo`0[ool3000^ool00`00ooooo`0:ool00`00
ooooo`0Fool400000ooo00000002000;ool00`00ooooo`0]ool00`00ooooo`0<ool00`00ooooo`0[
ool01000ooooo`001_oo00001Ooo003ooooo00002ooo00<00?ooool0:_oo00<00?oo0000;ooo00<0
0?ooool02_oo00<00?ooool06_oo100000?oo`00000010001_oo00<00?ooool0;Ooo00<00?ooool0
2ooo00<00?ooool0;_oo00<00?ooool01?oo00001Ooo003ooooo00002?oo0`00:ooo0P000_oo00<0
0?ooool0;Ooo00<00?ooool01ooo0`008Ooo10000ooo10000ooo00<00?ooool0;Ooo00<00?ooool0
1ooo0`00;ooo0P001ooo00001Ooo003ooooo00002?oo00<00?ooool0:_oo00<00?ooool00_oo00<0
0?ooool0;Ooo00<00?ooool01ooo00<00?ooool09Ooo10000ooo0`0000?oo`00ool0;_oo00<00?oo
ool01_oo00<00?ooool0<Ooo00<00?ooool01?oo00001Ooo003ooooo00002?oo00<00?ooool0:?oo
0P001Ooo00<00?ooool0:ooo0`002Ooo00<00?ooool0:Ooo0`0000?oo`00ool00`00:ooo00@00?oo
ool000Soo`03003ooooo02koo`04003ooooo0006ool000;oo`8000Woo`@002Koo`03003ooooo00Go
o`8002koo`03003ooooo00Ooo`@002_oo`@000;oo`<002Woo`8000Woo`@002koo`8000Ooo`00=_oo
00<00?ooool01_oo00<00?ooool0J_oo100000Coo`00000006coo`00=Ooo00<00?ooool01ooo00<0
0?ooool0K_oo0`0000?oo`0000000P00J?oo000dool00`00ooooo`08ool00`00ooooo`1aool40000
0ooo00000002001Tool003?oo`03003ooooo00Woo`03003ooooo07Goo`@00004ool00000001Qool0
00;oool001l000002Ooo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool0
1Ooo00@00?ooool000[oo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo
00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo
00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Koo`H000Ooo`03003ooooo00Soo`03
003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03
003ooooo00Soo`03003ooooo00Goo`00<?oo00<00?ooool03?oo00<00?ooool0P?oo0P0000Coo`00
000005Soo`00<?oo00<00?ooool03?oo00<00?ooool0P_oo0`0000Coo`00000005Goo`00;ooo00<0
0?ooool03Ooo0P00Q_oo0`0000Coo`00000005;oo`00;_oo00<00?ooool03_oo00<00?ooool0R?oo
0`0000?oo`000000D?oo000]ool00`00ooooo`0?ool00`00ooooo`2;ool6001=ool002goo`03003o
oooo00ooo`03003ooooo08koo`H004[oo`00;?oo00<00?ooool04?oo00<00?ooool0TOoo1P00Aooo
000[ool00`00ooooo`0Aool2002Eool50015ool002_oo`03003ooooo017oo`03003ooooo09Ooo`D0
04;oo`00:_oo00<00?ooool04_oo00<00?ooool0VOoo1P00?ooo000Zool00`00ooooo`0Bool00`00
ooooo`2Lool5000mool002Woo`03003ooooo01?oo`800:3oo`@003_oo`00:Ooo00<00?ooool04ooo
00<00?ooool0XOoo1@00>?oo000Xool00`00ooooo`0Dool00`00ooooo`2Sool200000ooo0000000f
ool002Soo`03003ooooo01Coo`03003ooooo0:Goo`800004ool00000000cool002Ooo`03003ooooo
01Goo`03003ooooo0:Ooo`<00003ool00000037oo`009ooo00<00?ooool05Ooo0P00Zooo0P0000?o
o`000000;ooo000Vool00`00ooooo`0Fool00`00ooooo`2/ool200000ooo0000000]ool002Goo`03
003ooooo00;oo`@000?oo`03003ooooo00;oo`<000Koo`03003ooooo0:koo`D002_oo`009Ooo00<0
0?ooool00_oo00<00?ooool03?oo00<00?ooool00ooo00<00?ooool0/Ooo1@00:?oo000Tool00`00
ooooo`04ool00`00ooooo`0;ool00`00ooooo`03ool3002cool6000Uool002Coo`03003ooooo00Go
o`03003ooooo00Ooo`<000Koo`03003ooooo0;Koo`800003ool0000002?oo`008ooo00<00?ooool0
1ooo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool0^?oo0P0000Coo`000000023oo`008ooo
00<00?ooool01?oo00@00?ooool000Soo`03003ooooo00Koo`03003ooooo0;[oo`8000;oo`8001ko
o`008_oo00<00?ooool01_oo0P002Ooo10001Ooo00<00?ooool0_?oo0P000_oo0P007?oo000Rool0
0`00ooooo`0Jool2002oool300000ooo0000000Jool002;oo`03003ooooo01[oo`03003ooooo0<7o
o`800003ool0000001Soo`008Ooo00<00?ooool06ooo00<00?ooool0`ooo0P0000?oo`0000005_oo
000Qool00`00ooooo`0Kool00`00ooooo`35ool200000ooo0000000Dool0027oo`03003ooooo01_o
o`800<Soo`800004ool00000000Aool0027oo`03003ooooo01_oo`03003ooooo0<Woo`<00003ool0
000000ooo`008?oo00<00?ooool07?oo00<00?ooool0c?oo0P0000?oo`0000003Ooo000Pool00`00
ooooo`0Lool00`00ooooo`3>ool200000ooo0000000;ool0023oo`03003ooooo01coo`03003ooooo
0=3oo`800003ool0000000Woo`007ooo00<00?ooool07Ooo0P00dooo0P0000?oo`0000001ooo000O
ool00`00ooooo`0Mool00`00ooooo`3Dool30007ool001ooo`03003ooooo01goo`03003ooooo0=ko
o`007ooo00<00?ooool07Ooo00<00?ooool0g_oo000Nool00`00ooooo`0Nool2003Oool001koo`03
003ooooo01koo`03003ooooo0=koo`007_oo00<00?ooool07_oo00<00?ooool0g_oo000Mool00`00
ooooo`0Fool30006ool00`00ooooo`3Nool001goo`03003ooooo01Woo`03003ooooo00?oo`03003o
oooo0=koo`007Ooo00<00?ooool06Ooo00<00?ooool00ooo0`00g_oo000Mool00`00ooooo`0Fool3
0006ool00`00ooooo`3Nool001coo`03003ooooo01Ooo`03003ooooo00Koo`03003ooooo0=koo`00
7?oo00<00?ooool05ooo00<00?ooool01_oo00<00?ooool0g_oo000Lool00`00ooooo`0Gool40005
ool2003Oool001_oo`03003ooooo027oo`03003ooooo0=koo`006ooo00<00?ooool08Ooo00<00?oo
ool0g_oo000Kool00`00ooooo`0Qool00`00ooooo`3Nool001_oo`03003ooooo027oo`03003ooooo
0=koo`006_oo00<00?ooool08_oo0P00gooo000Jool00`00ooooo`0Rool00`00ooooo`3Nool001[o
o`03003ooooo02;oo`03003ooooo0=koo`006_oo00<00?ooool08_oo00<00?ooool0g_oo000Iool0
0`00ooooo`0Sool2003Oool001Woo`03003ooooo02?oo`03003ooooo0=koo`006Ooo00<00?ooool0
8ooo00<00?ooool0g_oo000Iool00`00ooooo`0Sool00`00ooooo`3Nool001Woo`03003ooooo02?o
o`03003ooooo0=koo`006?oo00<00?ooool09?oo0P00gooo000Hool00`00ooooo`0Tool00`00oooo
o`3Nool001Soo`03003ooooo013oo`03003ooooo00?oo`03003ooooo00;oo`<000Koo`03003ooooo
0=koo`006?oo00<00?ooool04?oo00<00?ooool02ooo00<00?ooool00ooo00<00?ooool0g_oo000H
ool00`00ooooo`0Aool00`00ooooo`0:ool00`00ooooo`03ool3003Nool001Ooo`03003ooooo01;o
o`03003ooooo00Ooo`<000Koo`03003ooooo0=koo`005ooo00<00?ooool04ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool0g_oo000Gool00`00ooooo`0@ool01000ooooo`002?oo00<00?oo
ool01_oo00<00?ooool0g_oo000Gool00`00ooooo`0@ool40008ool40005ool00`00ooooo`3Nool0
01Ooo`03003ooooo02Goo`800=ooo`005ooo00<00?ooool09Ooo00<00?ooool0g_oo000Fool00`00
ooooo`0Vool00`00ooooo`3Nool001Koo`03003ooooo02Koo`03003ooooo0=koo`005_oo00<00?oo
ool09_oo0P00gooo000Fool00`00ooooo`0Vool00`00ooooo`3Nool001Koo`03003ooooo02Koo`03
003ooooo0=koo`005_oo00<00?ooool09_oo00<00?ooool0g_oo000Eool00`00ooooo`0Wool00`00
ooooo`3Nool001Goo`03003ooooo02Ooo`800=ooo`005Ooo00<00?ooool09ooo00<00?ooool0g_oo
000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03003ooooo0=koo`00
5Ooo00<00?ooool09ooo0P00gooo000Dool00`00ooooo`0Xool00`00ooooo`3Nool001Coo`03003o
oooo02Soo`03003ooooo0=koo`005?oo00<00?ooool06Ooo10000ooo0P001_oo00<00?ooool0g_oo
000Dool00`00ooooo`0Kool01@00oooooooo00000_oo00<00?ooool00ooo00<00?ooool0g_oo000D
ool00`00ooooo`0Kool01@00oooooooo00000_oo00<00?ooool00ooo0`00g_oo000Dool00`00oooo
o`0Kool01@00oooooooo00000_oo00<00?ooool00ooo00<00?ooool0g_oo000Cool00`00ooooo`0L
ool01@00oooooooo00000_oo00<00?ooool00ooo00<00?ooool0g_oo000Cool00`00ooooo`0Jool3
0003ool01000ooooo`001Ooo00<00?ooool0g_oo000Cool00`00ooooo`0Lool00`00ooooo`02ool2
0006ool2003Oool001?oo`03003ooooo02Woo`03003ooooo0=koo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.41703, -8.96188, \
0.00921772, 0.110694}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(thLXtestfunc[om_, k_, 
        th_] := \((om^2 - k^2 - LXpintnt[om/k, th])\)*\((1 - 
            LXbeta[om/k, th]/om^2)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[thLXtestfunc[om, 2, 0.0001]], {om, 0.5, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.380952 0.110052 0.0488076 [
[.02381 .09755 -9 -9 ]
[.02381 .09755 9 0 ]
[.40476 .09755 -9 -9 ]
[.40476 .09755 9 0 ]
[.59524 .09755 -3 -9 ]
[.59524 .09755 3 0 ]
[.78571 .09755 -9 -9 ]
[.78571 .09755 9 0 ]
[.97619 .09755 -3 -9 ]
[.97619 .09755 3 0 ]
[.20179 .01244 -12 -4.5 ]
[.20179 .01244 0 4.5 ]
[.20179 .20767 -6 -4.5 ]
[.20179 .20767 0 4.5 ]
[.20179 .30528 -6 -4.5 ]
[.20179 .30528 0 4.5 ]
[.20179 .4029 -6 -4.5 ]
[.20179 .4029 0 4.5 ]
[.20179 .50051 -6 -4.5 ]
[.20179 .50051 0 4.5 ]
[.20179 .59813 -12 -4.5 ]
[.20179 .59813 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .11005 m
.02381 .1163 L
s
[(0.5)] .02381 .09755 0 1 Mshowa
.40476 .11005 m
.40476 .1163 L
s
[(1.5)] .40476 .09755 0 1 Mshowa
.59524 .11005 m
.59524 .1163 L
s
[(2)] .59524 .09755 0 1 Mshowa
.78571 .11005 m
.78571 .1163 L
s
[(2.5)] .78571 .09755 0 1 Mshowa
.97619 .11005 m
.97619 .1163 L
s
[(3)] .97619 .09755 0 1 Mshowa
.125 Mabswid
.0619 .11005 m
.0619 .1138 L
s
.1 .11005 m
.1 .1138 L
s
.1381 .11005 m
.1381 .1138 L
s
.17619 .11005 m
.17619 .1138 L
s
.25238 .11005 m
.25238 .1138 L
s
.29048 .11005 m
.29048 .1138 L
s
.32857 .11005 m
.32857 .1138 L
s
.36667 .11005 m
.36667 .1138 L
s
.44286 .11005 m
.44286 .1138 L
s
.48095 .11005 m
.48095 .1138 L
s
.51905 .11005 m
.51905 .1138 L
s
.55714 .11005 m
.55714 .1138 L
s
.63333 .11005 m
.63333 .1138 L
s
.67143 .11005 m
.67143 .1138 L
s
.70952 .11005 m
.70952 .1138 L
s
.74762 .11005 m
.74762 .1138 L
s
.82381 .11005 m
.82381 .1138 L
s
.8619 .11005 m
.8619 .1138 L
s
.9 .11005 m
.9 .1138 L
s
.9381 .11005 m
.9381 .1138 L
s
.25 Mabswid
0 .11005 m
1 .11005 L
s
.21429 .01244 m
.22054 .01244 L
s
[(-2)] .20179 .01244 1 0 Mshowa
.21429 .20767 m
.22054 .20767 L
s
[(2)] .20179 .20767 1 0 Mshowa
.21429 .30528 m
.22054 .30528 L
s
[(4)] .20179 .30528 1 0 Mshowa
.21429 .4029 m
.22054 .4029 L
s
[(6)] .20179 .4029 1 0 Mshowa
.21429 .50051 m
.22054 .50051 L
s
[(8)] .20179 .50051 1 0 Mshowa
.21429 .59813 m
.22054 .59813 L
s
[(10)] .20179 .59813 1 0 Mshowa
.125 Mabswid
.21429 .03684 m
.21804 .03684 L
s
.21429 .06124 m
.21804 .06124 L
s
.21429 .08565 m
.21804 .08565 L
s
.21429 .13446 m
.21804 .13446 L
s
.21429 .15886 m
.21804 .15886 L
s
.21429 .18326 m
.21804 .18326 L
s
.21429 .23207 m
.21804 .23207 L
s
.21429 .25647 m
.21804 .25647 L
s
.21429 .28088 m
.21804 .28088 L
s
.21429 .32969 m
.21804 .32969 L
s
.21429 .35409 m
.21804 .35409 L
s
.21429 .37849 m
.21804 .37849 L
s
.21429 .4273 m
.21804 .4273 L
s
.21429 .45171 m
.21804 .45171 L
s
.21429 .47611 m
.21804 .47611 L
s
.21429 .52492 m
.21804 .52492 L
s
.21429 .54932 m
.21804 .54932 L
s
.21429 .57372 m
.21804 .57372 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.06011 .61803 m
.06244 .5888 L
.07298 .49287 L
.08426 .4098 L
.10458 .29726 L
.1148 .25405 L
.12589 .21476 L
.14603 .15899 L
.16586 .11861 L
.18689 .087 L
.20643 .06521 L
.22473 .04977 L
.24497 .03703 L
.2635 .02853 L
.28248 .02231 L
.29323 .01974 L
.30321 .01788 L
.30828 .01712 L
.31377 .01642 L
.31862 .01591 L
.32373 .01548 L
.32835 .01517 L
.3308 .01504 L
.33342 .01493 L
.33583 .01485 L
.33803 .01479 L
.33909 .01477 L
.34024 .01475 L
.34133 .01473 L
.34234 .01472 L
.34351 .01472 L
.34475 .01472 L
.34581 .01472 L
.34697 .01472 L
.34822 .01473 L
.34892 .01474 L
.34956 .01475 L
.35196 .0148 L
.35419 .01486 L
.35661 .01494 L
.361 .01513 L
.36621 .01542 L
.37172 .01582 L
.3816 .01673 L
.40118 .01924 L
.41949 .02236 L
.45831 .03108 L
.49959 .04303 L
.53935 .05677 L
.5776 .07178 L
Mistroke
.6183 .08949 L
.65748 .1081 L
.69911 .12943 L
.73923 .15142 L
.77783 .17386 L
.81889 .19904 L
.85843 .22454 L
.89645 .2502 L
.93693 .27871 L
.97589 .3073 L
.97619 .30752 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo000oool0
0`00ooooo`3Nool003Koo`@000Goo`03003ooooo0=koo`00=_oo00<00?ooool01_oo00<00?ooool0
g_oo000gool00`00ooooo`05ool3003Nool0033oo`@000Coo`03003ooooo00Coo`03003ooooo01Oo
oaP00:ooo`00>Ooo00<00?ooool00ooo00<00?ooool04_oo1@006?oo1P00ZOoo000fool01000oooo
o`001Ooo00<00?ooool03ooo0`008ooo1@00Y?oo000gool20006ool00`00ooooo`0=ool2000[ool6
002Nool003ooo`03003ooooo00[oo`<003?oo`@009[oo`00?ooo00<00?ooool01ooo0`00>_oo0`00
Uooo000oool20006ool20010ool3002Dool003ooo`03003ooooo00Coo`03003ooooo04?oo`<0097o
o`00?ooo00<00?ooool00_oo0P00BOoo0`00S_oo000oool01@00oooooooo0000C_oo0`00Rooo000o
ool01000ooooo`00D_oo0P00ROoo000oool00`00ool0001Eool30026ool003ooo`8005Woo`<008?o
o`00?_oo0`00G?oo0`00P?oo000mool00`00ool0001Pool2001nool003coo`04003ooooo001Rool3
001kool000;oo`8000Coo`03003ooooo00;oo`<002_oo`05003oooooool0000]ool40003ool00`00
ooooo`02ool3000Yool300000ooo00000002000/ool40003ool00`00ooooo`02ool3000_ool20007
ool00005ool00?ooool0000;ool00`00ooooo`0Wool00`00ooooo`02ool00`00ooooo`0]ool00`00
ooooo`0:ool00`00ooooo`0Yool2000_ool00`00ooooo`0<ool00`00ooooo`0[ool01000ooooo`00
1_oo00001Ooo003ooooo00002ooo00<00?ooool09_oo00<00?ooool00ooo00<00?ooool0;Ooo00<0
0?ooool02_oo00<00?ooool0:ooo0P00;_oo00<00?ooool02ooo00<00?ooool0;_oo00<00?ooool0
1?oo00001Ooo003ooooo00002?oo0`00:?oo00<00?ooool01?oo00<00?ooool0;Ooo00<00?ooool0
1ooo0`00;ooo1000;?oo00<00?ooool01ooo0`00;ooo0P001ooo00001Ooo003ooooo00002?oo00<0
0?ooool09ooo00<00?ooool01Ooo0P00;_oo00<00?ooool01ooo00<00?ooool0<?oo00<00?ooool0
0P00:ooo00<00?ooool01_oo00<00?ooool0<Ooo00<00?ooool01?oo00001Ooo003ooooo00002?oo
00<00?ooool09_oo00<00?ooool01_oo00<00?ooool0:ooo0`002Ooo00<00?ooool0;Ooo00@00?oo
ool000Coo`<002Goo`04003ooooo0008ool00`00ooooo`0^ool01000ooooo`001_oo0002ool20009
ool4000Uool00`00ooooo`06ool00`00ooooo`0]ool00`00ooooo`07ool4000]ool20008ool2000T
ool20009ool4000^ool20007ool003Goo`03003ooooo00Ooo`03003ooooo07Coo`8006Soo`00=?oo
00<00?ooool02?oo00<00?ooool0M_oo0P00I_oo000dool00`00ooooo`08ool00`00ooooo`1hool2
001Tool003?oo`03003ooooo00Woo`03003ooooo07[oo`8006;oo`000_ooo`007P000Ooo0009ool0
0`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`06ool00`00ool0000:ool0
0`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`07ool2000:ool00`00ooooo`08ool00`00ooooo`07ool00`00oooo
o`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00oooo
o`05ool000Woo`03003ooooo02Goo`03003ooooo00_oo`03003ooooo03?oo`03003ooooo03?oo`03
003ooooo01Coo`8001goo`03003ooooo03Coo`03003ooooo00Goo`00<Ooo00<00?ooool02ooo00<0
0?ooool0P_oo0P00F_oo000`ool00`00ooooo`0<ool00`00ooooo`24ool2001Hool0033oo`03003o
oooo00coo`03003ooooo08Koo`8005Koo`00;ooo00<00?ooool03Ooo00<00?ooool0R?oo0P00E?oo
000_ool00`00ooooo`0=ool2002;ool00`00ooooo`1Aool002koo`03003ooooo00koo`03003ooooo
08_oo`80057oo`00;_oo00<00?ooool03_oo00<00?ooool0SOoo0P00Cooo000]ool00`00ooooo`0?
ool00`00ooooo`2?ool00`00ooooo`1<ool002goo`03003ooooo00ooo`03003ooooo093oo`8004co
o`00;?oo00<00?ooool04?oo00<00?ooool0T_oo00<00?ooool0BOoo000/ool00`00ooooo`0@ool0
0`00ooooo`2Cool20019ool002_oo`03003ooooo017oo`8009Koo`8004Ooo`00:ooo00<00?ooool0
4Ooo00<00?ooool0Uooo0P00AOoo000Zool00`00ooooo`0Bool00`00ooooo`2Iool20013ool002[o
o`03003ooooo01;oo`03003ooooo09_oo`80047oo`00:_oo00<00?ooool04_oo00<00?ooool0WOoo
00<00?ooool0?_oo000Yool00`00ooooo`0Cool00`00ooooo`2Nool2000nool002Woo`03003ooooo
01?oo`03003ooooo0:3oo`8003coo`00:Ooo00<00?ooool04ooo0P00Xooo00<00?ooool0>Ooo000Y
ool00`00ooooo`0Cool00`00ooooo`2Sool2000iool002Soo`03003ooooo01Coo`03003ooooo0:Go
o`8003Ooo`00:?oo00<00?ooool05?oo00<00?ooool0Yooo0P00=Ooo000Xool00`00ooooo`0Dool0
0`00ooooo`2Yool00`00ooooo`0bool002Ooo`03003ooooo00coo`@000Goo`03003ooooo0:[oo`80
03;oo`009ooo00<00?ooool03?oo00<00?ooool01_oo00<00?ooool0[?oo00<00?ooool0;ooo000W
ool00`00ooooo`0=ool00`00ooooo`05ool3002]ool2000_ool002Koo`03003ooooo00ooo`03003o
oooo00Coo`03003ooooo0:ooo`8002goo`009_oo00<00?ooool04?oo00<00?ooool00ooo00<00?oo
ool0/Ooo00<00?ooool0:_oo000Vool00`00ooooo`0=ool01000ooooo`001Ooo00<00?ooool0/_oo
0P00:_oo000Uool00`00ooooo`0?ool20006ool00`00ooooo`2dool00`00ooooo`0Wool002Goo`03
003ooooo01Ooo`03003ooooo0;Goo`8002Ooo`009Ooo00<00?ooool05ooo00<00?ooool0]ooo00<0
0?ooool09?oo000Tool00`00ooooo`0Hool2002iool00`00ooooo`0Sool002Coo`03003ooooo01So
o`03003ooooo0;Woo`8002?oo`008ooo00<00?ooool06Ooo00<00?ooool0^ooo00<00?ooool08?oo
000Sool00`00ooooo`0Iool00`00ooooo`2lool00`00ooooo`0Oool002?oo`03003ooooo01Woo`03
003ooooo0;goo`8001ooo`008_oo00<00?ooool06_oo00<00?ooool0_ooo00<00?ooool07?oo000R
ool00`00ooooo`0Jool00`00ooooo`30ool2000Lool002;oo`03003ooooo01[oo`800<?oo`03003o
oooo01Woo`008_oo00<00?ooool06_oo00<00?ooool0`ooo0P006Ooo000Rool00`00ooooo`0Jool0
0`00ooooo`35ool00`00ooooo`0Fool0027oo`03003ooooo01_oo`03003ooooo0<Koo`8001Koo`00
8Ooo00<00?ooool06ooo00<00?ooool0b?oo00<00?ooool04ooo000Qool00`00ooooo`0Kool00`00
ooooo`39ool2000Cool0027oo`03003ooooo01_oo`800<coo`03003ooooo013oo`008Ooo00<00?oo
ool06ooo00<00?ooool0c?oo0P004?oo000Qool00`00ooooo`0Kool00`00ooooo`3>ool00`00oooo
o`0=ool0023oo`03003ooooo01coo`03003ooooo0<ooo`03003ooooo00coo`008?oo00<00?ooool0
7?oo00<00?ooool0d?oo0P003?oo000Pool00`00ooooo`0Eool00`00ooooo`04ool00`00ooooo`3B
ool00`00ooooo`09ool0023oo`03003ooooo01Goo`03003ooooo00Coo`03003ooooo0=?oo`03003o
oooo00Soo`008?oo00<00?ooool04_oo1@001Ooo0`00e?oo0P002?oo000Oool00`00ooooo`0Cool0
1000ooooo`001_oo00<00?ooool0e_oo00<00?ooool01Ooo000Oool00`00ooooo`0Dool00`00ool0
0006ool00`00ooooo`3Nool001ooo`03003ooooo01Goo`8000Koo`03003ooooo0=koo`007ooo00<0
0?ooool05_oo00<00?ooool01?oo00<00?ooool0g_oo000Oool00`00ooooo`0Mool00`00ooooo`3N
ool001ooo`03003ooooo01goo`03003ooooo0=koo`007_oo00<00?ooool07_oo0P00gooo000Nool0
0`00ooooo`0Nool00`00ooooo`3Nool001koo`03003ooooo01koo`03003ooooo0=koo`007_oo00<0
0?ooool07_oo00<00?ooool0g_oo000Nool00`00ooooo`0Nool00`00ooooo`3Nool001goo`03003o
oooo01ooo`03003ooooo0=koo`007Ooo00<00?ooool07ooo00<00?ooool0g_oo000Mool00`00oooo
o`0Oool2003Oool001goo`03003ooooo01ooo`03003ooooo0=koo`007Ooo00<00?ooool07ooo00<0
0?ooool0g_oo000Lool00`00ooooo`0Pool00`00ooooo`3Nool001coo`03003ooooo023oo`03003o
oooo0=koo`007?oo00<00?ooool08?oo00<00?ooool0g_oo000Lool00`00ooooo`0Pool00`00oooo
o`3Nool001coo`03003ooooo023oo`800=ooo`007?oo00<00?ooool08?oo00<00?ooool0g_oo000K
ool00`00ooooo`0Qool00`00ooooo`3Nool001_oo`03003ooooo027oo`03003ooooo0=koo`006ooo
00<00?ooool08Ooo00<00?ooool0g_oo000Kool00`00ooooo`0Iool20006ool00`00ooooo`3Nool0
01_oo`03003ooooo01Soo`04003ooooo0005ool00`00ooooo`3Nool001[oo`03003ooooo01Woo`04
003ooooo0005ool3003Nool001[oo`03003ooooo01Woo`<000Koo`03003ooooo0=koo`006_oo00<0
0?ooool06Ooo00<00?ooool01_oo00<00?ooool0g_oo000Jool00`00ooooo`0Iool00`00ooooo`06
ool00`00ooooo`3Nool001[oo`03003ooooo01[oo`<000Goo`03003ooooo0=koo`006_oo00<00?oo
ool08_oo00<00?ooool0g_oo000Jool00`00ooooo`0Rool00`00ooooo`3Nool001Woo`03003ooooo
02?oo`800=ooo`006Ooo00<00?ooool08ooo00<00?ooool0g_oo000Iool00`00ooooo`0Sool00`00
ooooo`3Nool001Woo`03003ooooo02?oo`03003ooooo0=koo`006Ooo00<00?ooool08ooo00<00?oo
ool0g_oo000Iool00`00ooooo`0Sool00`00ooooo`3Nool001Woo`03003ooooo02?oo`03003ooooo
0=koo`006Ooo00<00?ooool08ooo0P00gooo000Hool00`00ooooo`0Tool00`00ooooo`3Nool001So
o`03003ooooo02Coo`03003ooooo0=koo`006?oo00<00?ooool09?oo00<00?ooool0g_oo000Hool0
0`00ooooo`0Tool00`00ooooo`3Nool001Soo`03003ooooo02Coo`03003ooooo0=koo`006?oo00<0
0?ooool09?oo00<00?ooool0g_oo000Hool00`00ooooo`0Tool2003Oool001Soo`03003ooooo02Co
o`03003ooooo0=koo`005ooo00<00?ooool09Ooo00<00?ooool0g_oo000Gool00`00ooooo`0Uool0
0`00ooooo`3Nool001Ooo`03003ooooo02Goo`03003ooooo0=koo`005ooo00<00?ooool07Ooo0P00
1_oo00<00?ooool0g_oo000Gool00`00ooooo`0Lool01000ooooo`001Ooo00<00?ooool0g_oo000G
ool00`00ooooo`0Lool01000ooooo`001Ooo0`00g_oo000Gool00`00ooooo`0Mool20006ool00`00
ooooo`3Nool001Ooo`03003ooooo01coo`04003ooooo0005ool00`00ooooo`3Nool001Koo`03003o
oooo01goo`04003ooooo0005ool00`00ooooo`3Nool001Koo`03003ooooo01koo`8000Koo`03003o
oooo0=koo`005_oo00<00?ooool09_oo00<00?ooool0g_oo000Fool00`00ooooo`0Vool00`00oooo
o`3Nool001Koo`03003ooooo02Koo`800=ooo`005_oo00<00?ooool09_oo00<00?ooool0g_oo000F
ool00`00ooooo`0Vool00`00ooooo`3Nool001Koo`03003ooooo02Koo`03003ooooo0=koo`005_oo
00<00?ooool09_oo00<00?ooool0g_oo000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03
003ooooo02Ooo`03003ooooo0=koo`005Ooo00<00?ooool09ooo0P00gooo000Eool00`00ooooo`0W
ool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03003ooooo0=koo`005Ooo00<00?ooool09ooo
00<00?ooool0g_oo000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03
003ooooo0=koo`005Ooo00<00?ooool09ooo00<00?ooool0g_oo000Dool00`00ooooo`0Xool2003O
ool001Coo`03003ooooo02Soo`03003ooooo0=koo`005?oo00<00?ooool0:?oo00<00?ooool0g_oo
000Dool00`00ooooo`0Xool00`00ooooo`3Nool001Coo`03003ooooo02Soo`03003ooooo0=koo`00
5?oo00<00?ooool06Ooo10000ooo0P001_oo00<00?ooool0g_oo000Dool00`00ooooo`0Kool01@00
oooooooo00000_oo00<00?ooool00ooo00<00?ooool0g_oo000Dool00`00ooooo`0Kool01@00oooo
oooo00000_oo00<00?ooool00ooo0`00g_oo000Dool00`00ooooo`0Kool01@00oooooooo00000_oo
00<00?ooool00ooo00<00?ooool0g_oo000Cool00`00ooooo`0Lool01@00oooooooo00000_oo00<0
0?ooool00ooo00<00?ooool0g_oo000Cool00`00ooooo`0Jool30003ool01000ooooo`001Ooo00<0
0?ooool0g_oo000Cool00`00ooooo`0Lool00`00ooooo`02ool20006ool00`00ooooo`3Nool001?o
o`03003ooooo02Woo`03003ooooo0=koo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.41685, -2.3238, \
0.00921894, 0.0719555}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(looking\ for\ \(\(solutions\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Re[kx0corrtestfunc[0.5 + I\ 0.1, 2, 10^4]]\)], "Input"],

Cell[BoxData[
    \(11.84666000901881`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, 2, 10^4]] \[Equal] 0, 
        Im[kx0corrtestfunc[Om + I\ mGamma, 2, 10^4]] \[Equal] 0}, {Om, 0.5, 
        0.6}, {mGamma, \(-0.1\), \(-0.2\)}, 
      MaxIterations \[Rule] 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8867137271967592`, 
      mGamma \[Rule] \(-0.020002676073059722`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Im[kx0corrtestfunc[0.8857 + I\ mGamma, 2, 10^4]], {mGamma, \(-1\), 
        1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 0.310471 0.126135 [
[.02381 .29797 -6 -9 ]
[.02381 .29797 6 0 ]
[.2619 .29797 -12 -9 ]
[.2619 .29797 12 0 ]
[.7381 .29797 -9 -9 ]
[.7381 .29797 9 0 ]
[.97619 .29797 -3 -9 ]
[.97619 .29797 3 0 ]
[.4875 .0582 -12 -4.5 ]
[.4875 .0582 0 4.5 ]
[.4875 .18434 -12 -4.5 ]
[.4875 .18434 0 4.5 ]
[.4875 .43661 -6 -4.5 ]
[.4875 .43661 0 4.5 ]
[.4875 .56274 -6 -4.5 ]
[.4875 .56274 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .31047 m
.02381 .31672 L
s
[(-1)] .02381 .29797 0 1 Mshowa
.2619 .31047 m
.2619 .31672 L
s
[(-0.5)] .2619 .29797 0 1 Mshowa
.7381 .31047 m
.7381 .31672 L
s
[(0.5)] .7381 .29797 0 1 Mshowa
.97619 .31047 m
.97619 .31672 L
s
[(1)] .97619 .29797 0 1 Mshowa
.125 Mabswid
.07143 .31047 m
.07143 .31422 L
s
.11905 .31047 m
.11905 .31422 L
s
.16667 .31047 m
.16667 .31422 L
s
.21429 .31047 m
.21429 .31422 L
s
.30952 .31047 m
.30952 .31422 L
s
.35714 .31047 m
.35714 .31422 L
s
.40476 .31047 m
.40476 .31422 L
s
.45238 .31047 m
.45238 .31422 L
s
.54762 .31047 m
.54762 .31422 L
s
.59524 .31047 m
.59524 .31422 L
s
.64286 .31047 m
.64286 .31422 L
s
.69048 .31047 m
.69048 .31422 L
s
.78571 .31047 m
.78571 .31422 L
s
.83333 .31047 m
.83333 .31422 L
s
.88095 .31047 m
.88095 .31422 L
s
.92857 .31047 m
.92857 .31422 L
s
.25 Mabswid
0 .31047 m
1 .31047 L
s
.5 .0582 m
.50625 .0582 L
s
[(-2)] .4875 .0582 1 0 Mshowa
.5 .18434 m
.50625 .18434 L
s
[(-1)] .4875 .18434 1 0 Mshowa
.5 .43661 m
.50625 .43661 L
s
[(1)] .4875 .43661 1 0 Mshowa
.5 .56274 m
.50625 .56274 L
s
[(2)] .4875 .56274 1 0 Mshowa
.125 Mabswid
.5 .08343 m
.50375 .08343 L
s
.5 .10865 m
.50375 .10865 L
s
.5 .13388 m
.50375 .13388 L
s
.5 .15911 m
.50375 .15911 L
s
.5 .20956 m
.50375 .20956 L
s
.5 .23479 m
.50375 .23479 L
s
.5 .26002 m
.50375 .26002 L
s
.5 .28524 m
.50375 .28524 L
s
.5 .3357 m
.50375 .3357 L
s
.5 .36092 m
.50375 .36092 L
s
.5 .38615 m
.50375 .38615 L
s
.5 .41138 m
.50375 .41138 L
s
.5 .46183 m
.50375 .46183 L
s
.5 .48706 m
.50375 .48706 L
s
.5 .51229 m
.50375 .51229 L
s
.5 .53751 m
.50375 .53751 L
s
.5 .03297 m
.50375 .03297 L
s
.5 .00775 m
.50375 .00775 L
s
.5 .58797 m
.50375 .58797 L
s
.5 .61319 m
.50375 .61319 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .21235 m
.06244 .25268 L
.10458 .30261 L
.14415 .35575 L
.18221 .4122 L
.22272 .47557 L
.26171 .53421 L
.28158 .56029 L
.29205 .57227 L
.30316 .58326 L
.31304 .5913 L
.31828 .59483 L
.32392 .59799 L
.32909 .60027 L
.33381 .60183 L
.33609 .60238 L
.33729 .60263 L
.33857 .60285 L
.3398 .60302 L
.34091 .60314 L
.34203 .60323 L
.34309 .60329 L
.34425 .60332 L
.34549 .60331 L
.34667 .60326 L
.34774 .60319 L
.349 .60306 L
.35036 .60288 L
.35285 .60241 L
.35545 .60173 L
.35821 .60081 L
.36316 .59859 L
.36769 .59594 L
.37194 .59289 L
.38164 .58387 L
.39147 .57174 L
.40182 .55573 L
.42032 .51902 L
.45843 .41542 L
.49899 .28189 L
.53804 .15908 L
.55795 .10731 L
.56843 .08423 L
.57954 .06328 L
.58944 .04775 L
.60034 .03417 L
.60511 .02938 L
.61025 .02501 L
.61474 .02183 L
.61952 .01912 L
Mistroke
.62228 .01785 L
.62477 .01689 L
.62731 .0161 L
.62965 .01553 L
.63095 .01528 L
.63238 .01506 L
.63382 .01489 L
.63461 .01482 L
.63534 .01477 L
.63603 .01474 L
.63669 .01472 L
.63792 .01472 L
.63861 .01473 L
.63925 .01475 L
.63995 .01479 L
.64069 .01484 L
.64193 .01497 L
.64307 .01511 L
.64563 .01556 L
.64788 .01608 L
.65025 .01676 L
.65529 .01865 L
.66079 .02135 L
.6711 .02809 L
.68044 .03592 L
.70141 .05836 L
.74144 .11362 L
.77996 .17304 L
.82093 .23515 L
.86039 .29014 L
.89833 .33744 L
.93872 .38201 L
.97619 .41875 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00OSooo00<00?ooool0
S_oo002?ool00`00ooooo`2>ool008ooo`8008ooo`00Sooo00<00?ooool0S_oo002?ool00`00oooo
o`0Qool8001Uool008ooo`03003ooooo01ooo`8000Soo`03003ooooo06;oo`00Sooo00<00?ooool0
7_oo00<00?ooool02Ooo0P00H_oo002?ool00`00ooooo`0Lool2000>ool2001Pool008ooo`03003o
oooo01_oo`03003ooooo013oo`03003ooooo05goo`00Sooo0P006ooo00<00?ooool04_oo0P00GOoo
002?ool00`00ooooo`0Iool00`00ooooo`0Eool00`00ooooo`1Jool008ooo`03003ooooo01Woo`03
003ooooo01Koo`03003ooooo05Woo`00Sooo00<00?ooool06?oo00<00?ooool06?oo00<00?ooool0
F?oo002?ool00`00ooooo`0Gool00`00ooooo`0Jool00`00ooooo`1Gool008Koo`@000Goo`03003o
oooo01Koo`03003ooooo01coo`03003ooooo05Koo`00Q_oo00<00?ooool01_oo00<00?ooool05_oo
00<00?ooool07Ooo00<00?ooool0EOoo0027ool00`00ooooo`05ool3000Eool00`00ooooo`0Oool0
0`00ooooo`1Dool0083oo`@000Coo`03003ooooo00Coo`03003ooooo01Goo`03003ooooo023oo`03
003ooooo05?oo`00ROoo00<00?ooool00ooo00<00?ooool05?oo00<00?ooool08Ooo00<00?ooool0
Dooo0026ool01000ooooo`001Ooo00<00?ooool05?oo00<00?ooool08_oo00<00?ooool0D_oo0027
ool20006ool00`00ooooo`0Cool00`00ooooo`0Tool00`00ooooo`1Aool008ooo`03003ooooo01?o
o`03003ooooo02Coo`03003ooooo057oo`00Sooo00<00?ooool04_oo00<00?ooool09_oo00<00?oo
ool0D?oo002?ool2000Cool00`00ooooo`0Wool00`00ooooo`1?ool008ooo`03003ooooo017oo`03
003ooooo02Soo`03003ooooo04ooo`00Sooo00<00?ooool04Ooo00<00?ooool0:Ooo00<00?ooool0
C_oo002?ool00`00ooooo`0@ool00`00ooooo`0[ool00`00ooooo`1=ool008ooo`03003ooooo013o
o`03003ooooo02coo`03003ooooo04coo`00Sooo00<00?ooool03ooo00<00?ooool0;Ooo00<00?oo
ool0C?oo002?ool00`00ooooo`0?ool00`00ooooo`0^ool00`00ooooo`1;ool008ooo`03003ooooo
00koo`03003ooooo033oo`03003ooooo04[oo`00Sooo0P003ooo00<00?ooool0<?oo00<00?ooool0
B_oo002?ool00`00ooooo`0=ool00`00ooooo`0bool00`00ooooo`19ool008ooo`03003ooooo00go
o`03003ooooo03?oo`03003ooooo04Soo`00Sooo00<00?ooool03?oo00<00?ooool0=?oo00<00?oo
ool0B?oo002?ool00`00ooooo`0<ool00`00ooooo`0eool00`00ooooo`17ool008ooo`03003ooooo
00coo`03003ooooo03Koo`03003ooooo04Koo`00Sooo00<00?ooool02ooo00<00?ooool0=ooo00<0
0?ooool0A_oo002?ool2000<ool00`00ooooo`0hool00`00ooooo`15ool008ooo`03003ooooo00[o
o`03003ooooo03[oo`03003ooooo04Coo`00Sooo00<00?ooool02_oo00<00?ooool0>_oo00<00?oo
ool0A?oo002?ool00`00ooooo`0:ool00`00ooooo`0kool00`00ooooo`13ool008ooo`03003ooooo
00Woo`03003ooooo03coo`03003ooooo04?oo`00Sooo00<00?ooool02Ooo00<00?ooool0?Ooo00<0
0?ooool0@_oo002?ool00`00ooooo`08ool00`00ooooo`0oool00`00ooooo`11ool008ooo`8000Wo
o`03003ooooo03ooo`03003ooooo047oo`00Sooo00<00?ooool02?oo00<00?ooool0@?oo00<00?oo
ool0@?oo002?ool00`00ooooo`07ool00`00ooooo`12ool00`00ooooo`0oool008ooo`03003ooooo
00Ooo`03003ooooo04;oo`03003ooooo03ooo`00Sooo00<00?ooool01ooo00<00?ooool0@ooo00<0
0?ooool0?_oo0026ool40005ool00`00ooooo`06ool00`00ooooo`15ool00`00ooooo`0mool008So
o`03003ooooo00Coo`03003ooooo00Koo`03003ooooo04Goo`03003ooooo03goo`00R?oo00<00?oo
ool01?oo0`001_oo00<00?ooool0A_oo00<00?ooool0??oo0020ool40004ool00`00ooooo`04ool0
0`00ooooo`05ool00`00ooooo`18ool00`00ooooo`0kool008Soo`03003ooooo00Coo`03003ooooo
00Goo`03003ooooo04Soo`03003ooooo03_oo`00Q_oo0`001_oo00<00?ooool01Ooo00<00?ooool0
BOoo00<00?ooool0>_oo0028ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`1:ool00`00
ooooo`0iool008ooo`03003ooooo00Coo`03003ooooo04_oo`03003ooooo03Woo`00Sooo00<00?oo
ool01?oo00<00?ooool0C?oo00<00?ooool0>?oo002?ool00`00ooooo`04ool00`00ooooo`1=ool0
0`00ooooo`0gool000Ooo`03003ooooo08Goo`8000Coo`03003ooooo04koo`03003ooooo03Ooo`00
2?oo00<00?ooool0Q?oo00<00?ooool00ooo00<00?ooool0Cooo00<00?ooool0=_oo0009ool00`00
ooooo`23ool00`00ooooo`03ool00`00ooooo`1@ool00`00ooooo`0eool000[oo`03003ooooo08;o
o`03003ooooo00;oo`03003ooooo057oo`03003ooooo03Goo`002ooo00<00?ooool0POoo00<00?oo
ool00_oo00<00?ooool0D_oo00<00?ooool0=?oo000<ool00`00ooooo`20ool00`00ooooo`02ool0
0`00ooooo`1Cool00`00ooooo`0cool000coo`03003ooooo083oo`05003oooooool0001Fool00`00
ooooo`0cool000goo`03003ooooo07ooo`8000;oo`03003ooooo05Goo`03003ooooo03;oo`003_oo
00<00?ooool0O_oo00D00?ooooooo`0005Soo`03003ooooo037oo`003ooo00<00?ooool0OOoo00@0
0?ooool005Woo`03003ooooo037oo`004?oo00<00?ooool0O?oo00@00?ooool005[oo`03003ooooo
033oo`004Ooo00<00?ooool0Nooo00@00?ooool005_oo`03003ooooo02ooo`004_oo00<00?ooool0
N_oo00@00?ooool005_oo`03003ooooo02ooo`004ooo00<00?ooool0NOoo00<00?oo0000GOoo00<0
0?ooool0;_oo000Dool00`00ooooo`1hool3001Nool00`00ooooo`0]ool001Goo`03003ooooo07Oo
o`03003oo`0005koo`03003ooooo02goo`005Ooo00<00?ooool0Mooo0P00H?oo00<00?ooool0;?oo
000Fool00`00ooooo`1fool2001Qool00`00ooooo`0[ool000Soo`@000_oo`03003ooooo02goo`80
00Coo`03003ooooo00;oo`<003[oo`8003_oo`8000Coo`03003ooooo00;oo`<001Woo`03003ooooo
01ooo`@000Ooo`002_oo00<00?ooool02ooo00<00?ooool0:ooo00@00?ooool000_oo`03003ooooo
03Ooo`03003ooooo03Woo`04003ooooo000;ool00`00ooooo`0Fool00`00ooooo`0Qool00`00oooo
o`06ool000[oo`03003ooooo00coo`03003ooooo02[oo`04003ooooo000;ool00`00ooooo`0gool0
0`00ooooo`0iool01000ooooo`002ooo00<00?ooool05ooo00<00?ooool08?oo00<00?ooool01_oo
0002ool40004ool00`00ooooo`0=ool00`00ooooo`0Sool40002ool01000ooooo`002?oo0`00>_oo
0P00>_oo00@00?ooool000Soo`<001_oo`03003ooooo01ooo`03003ooooo00Koo`002_oo00<00?oo
ool03_oo00<00?ooool0:?oo00@00?ooool000Soo`03003ooooo03Woo`8003_oo`04003ooooo0008
ool00`00ooooo`0Kool00`00ooooo`0Oool00`00ooooo`06ool000Soo`<0013oo`03003ooooo02So
o`04003ooooo0008ool00`00ooooo`0iool2000kool01000ooooo`002?oo00<00?ooool07?oo00<0
0?ooool07?oo0`002?oo000:ool00`00ooooo`0?ool00`00ooooo`0Xool20009ool4000hool2000l
ool20009ool4000Lool00`00ooooo`0Mool00`00ooooo`06ool001goo`03003ooooo06goo`03003o
o`0006Soo`03003ooooo02Goo`007_oo00<00?ooool0K?oo00<00?oo0000JOoo00<00?ooool09?oo
000Oool00`00ooooo`1[ool00`00ool0001Yool00`00ooooo`0Tool00?l00200007oo`001ooo00<0
0?ooool02_oo00<00?ooool02Ooo00<00?oo00003Ooo00<00?ooool02_oo00<00?ooool02ooo00<0
0?ooool02_oo00<00?ooool02ooo00<00?ooool02ooo00<00?ooool02_oo00<00?ooool02?oo00@0
0?ooool000goo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00_oo`03003ooooo00[oo`03
003ooooo00_oo`03003ooooo00[oo`03003ooooo00[oo`8000goo`03003ooooo00[oo`03003ooooo
00Koo`001ooo00<00?ooool02_oo00<00?ooool02_oo0P003Ooo00<00?ooool02_oo00<00?ooool0
2ooo00<00?ooool02_oo00<00?ooool02ooo00<00?ooool02ooo00<00?ooool02_oo00<00?ooool0
2?oo00@00?ooool000goo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00_oo`03003ooooo
00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00_oo`03003ooooo
00[oo`03003ooooo00Koo`008_oo00<00?ooool0Iooo00@00?ooool006goo`03003ooooo023oo`00
8_oo00<00?ooool0I_oo00D00?ooooooo`0006koo`03003ooooo01ooo`008ooo00<00?ooool0IOoo
00D00?ooooooo`0006koo`03003ooooo01ooo`009?oo00<00?ooool0I?oo00D00?ooooooo`0006oo
o`03003ooooo01koo`009Ooo00<00?ooool0H_oo00<00?ooool00_oo00<00?ooool0K_oo00<00?oo
ool07Ooo000Uool00`00ooooo`1Rool00`00ooooo`02ool2001`ool00`00ooooo`0Lool002Koo`03
003ooooo067oo`03003ooooo00;oo`03003ooooo073oo`03003ooooo01_oo`009ooo00<00?ooool0
Gooo00<00?ooool00ooo00<00?ooool0LOoo00<00?ooool06_oo000Xool00`00ooooo`1Nool00`00
ooooo`03ool00`00ooooo`1bool00`00ooooo`0Iool002Soo`03003ooooo05koo`03003ooooo00?o
o`03003ooooo07?oo`03003ooooo01Soo`00:Ooo00<00?ooool0G?oo00<00?ooool01?oo00<00?oo
ool0M?oo00<00?ooool05ooo000Zool00`00ooooo`1Kool00`00ooooo`04ool00`00ooooo`1eool0
0`00ooooo`0Fool002[oo`03003ooooo05_oo`03003ooooo00Coo`8007Koo`03003ooooo01Koo`00
:ooo00<00?ooool0FOoo00<00?ooool01Ooo00<00?ooool0M_oo00<00?ooool05Ooo000/ool00`00
ooooo`1Hool00`00ooooo`05ool00`00ooooo`1gool00`00ooooo`0Dool002coo`03003ooooo05So
o`03003ooooo00Goo`03003ooooo07Soo`03003ooooo01?oo`00;Ooo00<00?ooool0E_oo00<00?oo
ool01_oo00<00?ooool0NOoo00<00?ooool04_oo000^ool00`00ooooo`1Eool00`00ooooo`06ool0
0`00ooooo`1jool00`00ooooo`0Aool002koo`03003ooooo05Goo`03003ooooo00Koo`03003ooooo
07_oo`03003ooooo013oo`00;ooo00<00?ooool0E?oo00<00?ooool01_oo0P00OOoo00<00?ooool0
3ooo000_ool00`00ooooo`1Cool00`00ooooo`07ool00`00ooooo`1mool00`00ooooo`0>ool0033o
o`03003ooooo05;oo`03003ooooo00Ooo`03003ooooo07koo`03003ooooo00goo`00<Ooo00<00?oo
ool0DOoo00<00?ooool01ooo00<00?ooool0Oooo00<00?ooool03?oo000aool00`00ooooo`1@ool0
0`00ooooo`08ool00`00ooooo`20ool00`00ooooo`0;ool003;oo`03003ooooo04ooo`03003ooooo
00Soo`03003ooooo087oo`03003ooooo00[oo`00<ooo00<00?ooool0C_oo00<00?ooool02?oo00<0
0?ooool0P_oo00<00?ooool02Ooo000cool00`00ooooo`1=ool00`00ooooo`09ool20024ool00`00
ooooo`08ool003Coo`03003ooooo04coo`03003ooooo00Woo`03003ooooo08Coo`03003ooooo00Oo
o`00=Ooo00<00?ooool0Booo00<00?ooool02Ooo00<00?ooool0QOoo00<00?ooool01_oo000eool0
0`00ooooo`1:ool00`00ooooo`0:ool00`00ooooo`2>ool003Koo`03003ooooo04Woo`03003ooooo
00[oo`03003ooooo08koo`00=ooo00<00?ooool0B?oo00<00?ooool02_oo00<00?ooool0S_oo000g
ool00`00ooooo`17ool00`00ooooo`02ool40005ool00`00ooooo`2>ool003Soo`03003ooooo04Ko
o`03003ooooo00Coo`03003ooooo00Coo`03003ooooo08koo`00>Ooo00<00?ooool0A?oo00<00?oo
ool01Ooo00<00?ooool01?oo0`00S_oo000iool00`00ooooo`14ool00`00ooooo`05ool00`00oooo
o`04ool00`00ooooo`2>ool003[oo`03003ooooo04?oo`03003ooooo00Goo`03003ooooo00Coo`03
003ooooo08koo`00>ooo00<00?ooool0@Ooo00<00?ooool01?oo0`001_oo00<00?ooool0S_oo000k
ool00`00ooooo`11ool00`00ooooo`06ool00`00ooooo`04ool00`00ooooo`2>ool003coo`03003o
oooo043oo`03003ooooo00goo`03003ooooo08koo`00?Ooo00<00?ooool0?_oo00<00?ooool03_oo
00<00?ooool0S_oo000mool00`00ooooo`0nool00`00ooooo`0>ool2002?ool003koo`03003ooooo
03goo`03003ooooo00koo`03003ooooo08koo`00?ooo00<00?ooool0>ooo00<00?ooool03ooo00<0
0?ooool0S_oo000oool00`00ooooo`0kool00`00ooooo`0?ool00`00ooooo`2>ool0043oo`03003o
oooo03Woo`03003ooooo013oo`03003ooooo08koo`00@Ooo00<00?ooool0>?oo00<00?ooool04?oo
00<00?ooool0S_oo0011ool00`00ooooo`0hool00`00ooooo`0@ool00`00ooooo`2>ool004;oo`03
003ooooo03Koo`03003ooooo017oo`8008ooo`00@ooo00<00?ooool0=Ooo00<00?ooool04Ooo00<0
0?ooool0S_oo0013ool00`00ooooo`0eool00`00ooooo`0Aool00`00ooooo`2>ool004Coo`03003o
oooo03?oo`03003ooooo01;oo`03003ooooo08koo`00AOoo00<00?ooool0<_oo00<00?ooool04_oo
00<00?ooool0S_oo0015ool00`00ooooo`0aool00`00ooooo`0Cool00`00ooooo`2>ool004Koo`03
003ooooo033oo`03003ooooo01?oo`03003ooooo08koo`00A_oo00<00?ooool0<?oo00<00?ooool0
4ooo0P00Sooo0017ool00`00ooooo`0^ool00`00ooooo`0Dool00`00ooooo`2>ool004Soo`03003o
oooo02goo`03003ooooo01Coo`03003ooooo08koo`00B?oo00<00?ooool0;Ooo00<00?ooool05?oo
00<00?ooool0S_oo0019ool00`00ooooo`0[ool00`00ooooo`0Eool00`00ooooo`2>ool004[oo`03
003ooooo02[oo`03003ooooo01Goo`03003ooooo08koo`00B_oo00<00?ooool0:Ooo00<00?ooool0
5_oo00<00?ooool0S_oo001;ool00`00ooooo`0Xool00`00ooooo`0Fool2002?ool004coo`03003o
oooo02Koo`03003ooooo01Ooo`03003ooooo08koo`00C?oo00<00?ooool09_oo00<00?ooool05ooo
00<00?ooool0S_oo001=ool00`00ooooo`0Tool00`00ooooo`0Hool00`00ooooo`2>ool004koo`03
003ooooo02?oo`03003ooooo01Soo`03003ooooo08koo`00Cooo00<00?ooool08Ooo00<00?ooool0
6Ooo00<00?ooool0S_oo001?ool00`00ooooo`0Qool00`00ooooo`0@ool40005ool00`00ooooo`2>
ool0053oo`03003ooooo01ooo`03003ooooo017oo`03003ooooo00Koo`03003ooooo08koo`00DOoo
00<00?ooool07_oo00<00?ooool04_oo00<00?ooool01Ooo0`00S_oo001Bool00`00ooooo`0Lool0
0`00ooooo`0Dool00`00ooooo`04ool00`00ooooo`2>ool005?oo`03003ooooo01[oo`03003ooooo
01Koo`03003ooooo00?oo`03003ooooo08koo`00E?oo00<00?ooool06?oo00<00?ooool05?oo00@0
0?ooool000Goo`03003ooooo08koo`00EOoo00<00?ooool05ooo00<00?ooool05Ooo0P001_oo00<0
0?ooool0S_oo001Fool00`00ooooo`0Eool00`00ooooo`0Nool00`00ooooo`2>ool005Ooo`03003o
oooo01?oo`03003ooooo01ooo`03003ooooo08koo`00F?oo00<00?ooool04?oo0P008_oo0P00Sooo
001Iool00`00ooooo`0>ool00`00ooooo`0Rool00`00ooooo`2>ool005[oo`8000goo`03003ooooo
02?oo`03003ooooo08koo`00G?oo0`002?oo0P009_oo00<00?ooool0S_oo001Oool8000Xool00`00
ooooo`2>ool008ooo`03003ooooo08koo`00Sooo00<00?ooool0S_oo002?ool2002?ool008ooo`03
003ooooo08koo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.0526, -2.46144, \
0.00733521, 0.0276923}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[kx0corrtestfunc[0.8857 + I\ mGamma, 2, 10^4]], {mGamma, \(-1\), 
        1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 0.602035 0.099473 [
[.02381 .58954 -6 -9 ]
[.02381 .58954 6 0 ]
[.2619 .58954 -12 -9 ]
[.2619 .58954 12 0 ]
[.7381 .58954 -9 -9 ]
[.7381 .58954 9 0 ]
[.97619 .58954 -3 -9 ]
[.97619 .58954 3 0 ]
[.4875 .0052 -12 -4.5 ]
[.4875 .0052 0 4.5 ]
[.4875 .10467 -12 -4.5 ]
[.4875 .10467 0 4.5 ]
[.4875 .20414 -12 -4.5 ]
[.4875 .20414 0 4.5 ]
[.4875 .30362 -12 -4.5 ]
[.4875 .30362 0 4.5 ]
[.4875 .40309 -12 -4.5 ]
[.4875 .40309 0 4.5 ]
[.4875 .50256 -12 -4.5 ]
[.4875 .50256 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60204 m
.02381 .60829 L
s
[(-1)] .02381 .58954 0 1 Mshowa
.2619 .60204 m
.2619 .60829 L
s
[(-0.5)] .2619 .58954 0 1 Mshowa
.7381 .60204 m
.7381 .60829 L
s
[(0.5)] .7381 .58954 0 1 Mshowa
.97619 .60204 m
.97619 .60829 L
s
[(1)] .97619 .58954 0 1 Mshowa
.125 Mabswid
.07143 .60204 m
.07143 .60579 L
s
.11905 .60204 m
.11905 .60579 L
s
.16667 .60204 m
.16667 .60579 L
s
.21429 .60204 m
.21429 .60579 L
s
.30952 .60204 m
.30952 .60579 L
s
.35714 .60204 m
.35714 .60579 L
s
.40476 .60204 m
.40476 .60579 L
s
.45238 .60204 m
.45238 .60579 L
s
.54762 .60204 m
.54762 .60579 L
s
.59524 .60204 m
.59524 .60579 L
s
.64286 .60204 m
.64286 .60579 L
s
.69048 .60204 m
.69048 .60579 L
s
.78571 .60204 m
.78571 .60579 L
s
.83333 .60204 m
.83333 .60579 L
s
.88095 .60204 m
.88095 .60579 L
s
.92857 .60204 m
.92857 .60579 L
s
.25 Mabswid
0 .60204 m
1 .60204 L
s
.5 .0052 m
.50625 .0052 L
s
[(-6)] .4875 .0052 1 0 Mshowa
.5 .10467 m
.50625 .10467 L
s
[(-5)] .4875 .10467 1 0 Mshowa
.5 .20414 m
.50625 .20414 L
s
[(-4)] .4875 .20414 1 0 Mshowa
.5 .30362 m
.50625 .30362 L
s
[(-3)] .4875 .30362 1 0 Mshowa
.5 .40309 m
.50625 .40309 L
s
[(-2)] .4875 .40309 1 0 Mshowa
.5 .50256 m
.50625 .50256 L
s
[(-1)] .4875 .50256 1 0 Mshowa
.125 Mabswid
.5 .02509 m
.50375 .02509 L
s
.5 .04499 m
.50375 .04499 L
s
.5 .06488 m
.50375 .06488 L
s
.5 .08478 m
.50375 .08478 L
s
.5 .12456 m
.50375 .12456 L
s
.5 .14446 m
.50375 .14446 L
s
.5 .16435 m
.50375 .16435 L
s
.5 .18425 m
.50375 .18425 L
s
.5 .22404 m
.50375 .22404 L
s
.5 .24393 m
.50375 .24393 L
s
.5 .26383 m
.50375 .26383 L
s
.5 .28372 m
.50375 .28372 L
s
.5 .32351 m
.50375 .32351 L
s
.5 .34341 m
.50375 .34341 L
s
.5 .3633 m
.50375 .3633 L
s
.5 .38319 m
.50375 .38319 L
s
.5 .42298 m
.50375 .42298 L
s
.5 .44288 m
.50375 .44288 L
s
.5 .46277 m
.50375 .46277 L
s
.5 .48267 m
.50375 .48267 L
s
.5 .52246 m
.50375 .52246 L
s
.5 .54235 m
.50375 .54235 L
s
.5 .56225 m
.50375 .56225 L
s
.5 .58214 m
.50375 .58214 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .03027 L
.10458 .04896 L
.14415 .07044 L
.18221 .09765 L
.22272 .13767 L
.26171 .19093 L
.30316 .26608 L
.34309 .35462 L
.3815 .44634 L
.40095 .49067 L
.42237 .53423 L
.44268 .56765 L
.45178 .57946 L
.46172 .58981 L
.4671 .59424 L
.4721 .59759 L
.47687 .60008 L
.47932 .60108 L
.48196 .60196 L
.48323 .60231 L
.48457 .60262 L
.48581 .60285 L
.48697 .60303 L
.48805 .60316 L
.48921 .60325 L
.4905 .60331 L
.49168 .60332 L
.4928 .60328 L
.49385 .60322 L
.49499 .60311 L
.4962 .60294 L
.49824 .60256 L
.50049 .60199 L
.50284 .60123 L
.5054 .60021 L
.51058 .59752 L
.51984 .59072 L
.53064 .57968 L
.54069 .56662 L
.57974 .4954 L
.62123 .39989 L
.66122 .30792 L
.69968 .23139 L
.7406 .16773 L
.78 .12289 L
.81789 .09172 L
.85823 .06732 L
.89706 .04896 L
.93833 .03217 L
Mistroke
.97619 .01758 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool008Ooo`8009Ooo`00Q_oo00@00?ooool000Goo`03003ooooo08koo`00Q_oo00@00?ooool000Go
o`<008koo`00P?oo10000_oo0`001_oo00<00?ooool0S_oo0026ool00`00ooooo`06ool00`00oooo
o`2>ool000Woo`8007_oo`03003ooooo00Koo`03003ooooo08koo`002ooo0`00NOoo0`001Ooo00<0
0?ooool0P_oo0P002_oo000>ool2001oool00`00ooooo`1oool3000<ool0013oo`<007coo`8007ko
o`8000ooo`004ooo0`00NOoo00<00?ooool0N_oo0`004Ooo000Fool2001gool00`00ooooo`1gool3
000Dool001Soo`8007Goo`03003ooooo07Coo`<001Ooo`006_oo0`00L_oo00<00?ooool0L_oo0P00
6_oo000Mool2001`ool2001`ool3000Lool001ooo`8006koo`03003ooooo06goo`8001ooo`008Ooo
0P00K?oo00<00?ooool0Jooo0P008Ooo000Sool2001Zool00`00ooooo`1Yool2000Sool002Goo`80
06Soo`03003ooooo06Ooo`8002Goo`009ooo0P00I_oo00<00?ooool0IOoo0P009ooo000Yool2001T
ool2001Tool2000Yool002_oo`03003ooooo067oo`03003ooooo067oo`8002_oo`00;?oo0P00HOoo
00<00?ooool0Gooo0P00;Ooo000^ool00`00ooooo`1Nool00`00ooooo`1Mool2000_ool002ooo`03
003ooooo05goo`03003ooooo05_oo`80037oo`00<?oo0P00GOoo0P00F_oo0P00<ooo000bool00`00
ooooo`1Jool00`00ooooo`1Gool2000eool003?oo`03003ooooo05Woo`03003ooooo05Koo`03003o
oooo03Goo`00=?oo0P00FOoo00<00?ooool0EOoo00<00?ooool0=_oo000fool00`00ooooo`1=ool3
0006ool00`00ooooo`1Dool00`00ooooo`0gool003Ooo`03003ooooo04ooo`03003ooooo00?oo`03
003ooooo05?oo`03003ooooo03Soo`00>?oo00<00?ooool0C_oo00<00?ooool00ooo0`00DOoo0P00
>ooo000iool00`00ooooo`14ool40002ool30006ool00`00ooooo`1@ool00`00ooooo`0kool003[o
o`03003ooooo04Woo`03003ooooo00Koo`03003ooooo04ooo`03003ooooo03coo`00>ooo00<00?oo
ool0B?oo00<00?ooool01_oo00<00?ooool0C_oo00<00?ooool0?Ooo000lool00`00ooooo`17ool4
0005ool00`00ooooo`1=ool00`00ooooo`0nool003goo`03003ooooo04ooo`03003ooooo04coo`03
003ooooo03ooo`00?_oo00<00?ooool0C_oo0P00C?oo00<00?ooool0@?oo000oool00`00ooooo`1=
ool00`00ooooo`1:ool00`00ooooo`11ool0043oo`03003ooooo04coo`03003ooooo04Woo`03003o
oooo04;oo`00@Ooo00<00?ooool0Booo00<00?ooool0BOoo00<00?ooool0@_oo0012ool00`00oooo
o`1:ool00`00ooooo`18ool00`00ooooo`13ool004;oo`03003ooooo04[oo`8004Soo`03003ooooo
04Coo`00@ooo00<00?ooool0BOoo00<00?ooool0A_oo00<00?ooool0AOoo0014ool00`00ooooo`18
ool00`00ooooo`15ool00`00ooooo`16ool004Goo`03003ooooo04Ooo`03003ooooo04Coo`03003o
oooo04Ooo`00AOoo00<00?ooool0Aooo00<00?ooool0A?oo00<00?ooool0Aooo0016ool00`00oooo
o`16ool00`00ooooo`13ool00`00ooooo`18ool004Ooo`03003ooooo04Goo`8004?oo`03003ooooo
04Woo`00B?oo00<00?ooool0A?oo00<00?ooool0@Ooo00<00?ooool0B_oo0018ool00`00ooooo`14
ool00`00ooooo`10ool00`00ooooo`1;ool004Woo`03003ooooo04?oo`03003ooooo043oo`03003o
oooo04_oo`00B_oo00<00?ooool0@_oo00<00?ooool0?ooo00<00?ooool0C?oo001;ool00`00oooo
o`11ool2000oool00`00ooooo`1=ool004_oo`03003ooooo047oo`03003ooooo03koo`03003ooooo
04goo`00C?oo00<00?ooool0@?oo00<00?ooool0?Ooo00<00?ooool0C_oo001=ool00`00ooooo`0o
ool00`00ooooo`0lool00`00ooooo`1?ool004goo`03003ooooo03Soo`03003ooooo00Coo`03003o
oooo03coo`03003ooooo04ooo`00C_oo00<00?ooool0=ooo00<00?ooool01?oo00<00?ooool0>ooo
00<00?ooool0D?oo001>ool00`00ooooo`0dool50005ool3000jool00`00ooooo`1Aool004ooo`03
003ooooo02koo`@00005ool00?ooool00006ool00`00ooooo`0jool00`00ooooo`1Aool004ooo`03
003ooooo03Coo`03003oo`0000Koo`03003ooooo03Woo`03003ooooo05;oo`00D?oo00<00?ooool0
=?oo0P001_oo00<00?ooool0>?oo00<00?ooool0Dooo001Aool00`00ooooo`0dool00`00ooooo`04
ool00`00ooooo`0hool00`00ooooo`1Cool0057oo`03003ooooo03_oo`03003ooooo03Ooo`03003o
oooo05Coo`00D_oo00<00?ooool0>_oo0P00=ooo00<00?ooool0EOoo001Bool00`00ooooo`0jool0
0`00ooooo`0fool00`00ooooo`1Eool005?oo`03003ooooo03Woo`03003ooooo03Goo`03003ooooo
05Koo`00Dooo00<00?ooool0>Ooo00<00?ooool0=?oo00<00?ooool0Eooo001Dool00`00ooooo`0h
ool00`00ooooo`0dool00`00ooooo`1Gool005Goo`03003ooooo03Ooo`8003Coo`03003ooooo05So
o`00EOoo00<00?ooool0=ooo00<00?ooool0<ooo00<00?ooool0F?oo001Fool00`00ooooo`0fool0
0`00ooooo`0bool00`00ooooo`1Iool005Koo`03003ooooo03Koo`03003ooooo03;oo`03003ooooo
05Woo`00Eooo00<00?ooool0=Ooo00<00?ooool0<Ooo00<00?ooool0F_oo001Gool00`00ooooo`0e
ool00`00ooooo`0aool00`00ooooo`1Jool005Soo`03003ooooo03Coo`80037oo`03003ooooo05_o
o`00F?oo00<00?ooool0=?oo00<00?ooool0<?oo00<00?ooool0Fooo001Iool00`00ooooo`0cool0
0`00ooooo`0_ool00`00ooooo`1Lool005Woo`03003ooooo03?oo`03003ooooo02ooo`03003ooooo
05coo`00F_oo00<00?ooool0<_oo00<00?ooool0;_oo00<00?ooool0GOoo001Jool00`00ooooo`0b
ool2000_ool00`00ooooo`1Mool005_oo`03003ooooo037oo`03003ooooo02goo`03003ooooo05ko
o`00Fooo00<00?ooool0<Ooo00<00?ooool0;Ooo00<00?ooool0G_oo001Lool00`00ooooo`0`ool0
0`00ooooo`0/ool00`00ooooo`1Oool005coo`03003ooooo02Soo`8000Koo`03003ooooo02coo`03
003ooooo05ooo`00G?oo00<00?ooool09ooo00@00?ooool000Goo`03003ooooo02_oo`03003ooooo
063oo`00GOoo00<00?ooool0:Ooo00<00?ooool00ooo0`00:ooo00<00?ooool0H?oo001Mool00`00
ooooo`0Pool40003ool20006ool00`00ooooo`0Zool00`00ooooo`1Qool005koo`03003ooooo02So
o`03003ooooo00?oo`03003ooooo02[oo`03003ooooo067oo`00G_oo00<00?ooool09Ooo00@00?oo
ool000Goo`03003ooooo02Woo`03003ooooo06;oo`00Gooo00<00?ooool09Ooo0P001_oo00<00?oo
ool0:Ooo00<00?ooool0H_oo001Oool00`00ooooo`0]ool00`00ooooo`0Xool00`00ooooo`1Sool0
05ooo`03003ooooo02goo`8002Woo`03003ooooo06?oo`00H?oo00<00?ooool0;?oo00<00?ooool0
9ooo00<00?ooool0I?oo001Pool00`00ooooo`0/ool00`00ooooo`0Wool00`00ooooo`1Tool0067o
o`03003ooooo02_oo`03003ooooo02Ooo`03003ooooo06Coo`00HOoo00<00?ooool0:ooo00<00?oo
ool09_oo00<00?ooool0IOoo001Rool00`00ooooo`0Zool2000Wool00`00ooooo`1Uool006;oo`03
003ooooo02[oo`03003ooooo02Goo`03003ooooo06Koo`00Hooo00<00?ooool0:Ooo00<00?ooool0
9Ooo00<00?ooool0I_oo001Sool00`00ooooo`0Yool00`00ooooo`0Uool00`00ooooo`1Vool006?o
o`03003ooooo02Woo`03003ooooo02Coo`03003ooooo06Ooo`00I?oo00<00?ooool0:?oo00<00?oo
ool09?oo00<00?ooool0Iooo001Tool00`00ooooo`0Xool2000Tool00`00ooooo`1Xool006Goo`03
003ooooo02Ooo`03003ooooo02?oo`03003ooooo06Soo`00IOoo00<00?ooool09ooo00<00?ooool0
8_oo00<00?ooool0JOoo001Vool00`00ooooo`0Vool00`00ooooo`0Rool00`00ooooo`1Yool006Ko
o`03003ooooo02Koo`03003ooooo02;oo`03003ooooo06Woo`00I_oo00<00?ooool09_oo0P008_oo
00<00?ooool0J_oo001Wool00`00ooooo`0Uool00`00ooooo`0Qool00`00ooooo`1Zool006Ooo`03
003ooooo02Goo`03003ooooo023oo`03003ooooo06_oo`00J?oo00<00?ooool09?oo00<00?ooool0
8?oo00<00?ooool0Jooo001Xool00`00ooooo`0Kool40005ool00`00ooooo`0Oool00`00ooooo`1/
ool006Soo`03003ooooo01_oo`03003ooooo00Koo`03003ooooo01ooo`03003ooooo06coo`00JOoo
00<00?ooool06ooo00<00?ooool01Ooo0`007ooo00<00?ooool0K?oo001Yool00`00ooooo`0Dool4
0004ool00`00ooooo`04ool00`00ooooo`0Nool00`00ooooo`1]ool006[oo`03003ooooo01coo`03
003ooooo00?oo`03003ooooo01koo`03003ooooo06goo`00J_oo00<00?ooool06Ooo00@00?ooool0
00Goo`03003ooooo01goo`03003ooooo06koo`00Jooo00<00?ooool06Ooo0P001_oo00<00?ooool0
7Ooo00<00?ooool0K_oo001[ool00`00ooooo`0Qool00`00ooooo`0Lool00`00ooooo`1_ool006_o
o`03003ooooo027oo`8001goo`03003ooooo06ooo`00K?oo00<00?ooool08?oo00<00?ooool06ooo
00<00?ooool0L?oo001/ool00`00ooooo`0Pool00`00ooooo`0Kool00`00ooooo`1`ool006goo`03
003ooooo01ooo`03003ooooo01_oo`03003ooooo073oo`00KOoo00<00?ooool07ooo00<00?ooool0
6_oo00<00?ooool0LOoo001^ool00`00ooooo`0Nool2000Kool00`00ooooo`1aool006koo`03003o
oooo01koo`03003ooooo01Woo`03003ooooo07;oo`00K_oo00<00?ooool07_oo00<00?ooool06Ooo
00<00?ooool0L_oo001_ool00`00ooooo`0Mool00`00ooooo`0Hool00`00ooooo`1cool006ooo`03
003ooooo01goo`03003ooooo01Soo`03003ooooo07?oo`00L?oo00<00?ooool07?oo00<00?ooool0
5ooo00<00?ooool0M?oo001`ool00`00ooooo`0Lool2000Hool00`00ooooo`1dool0073oo`03003o
oooo01coo`03003ooooo01Ooo`03003ooooo07Coo`00LOoo00<00?ooool06ooo00<00?ooool05_oo
00<00?ooool0MOoo001aool00`00ooooo`0Kool00`00ooooo`0Fool00`00ooooo`1eool0077oo`03
003ooooo01_oo`03003ooooo01Goo`03003ooooo07Koo`00L_oo00<00?ooool06_oo0P005_oo00<0
0?ooool0M_oo001bool00`00ooooo`0Jool00`00ooooo`0Dool00`00ooooo`1gool007?oo`03003o
oooo01Woo`03003ooooo01Coo`03003ooooo07Ooo`00Looo00<00?ooool06Ooo00<00?ooool04ooo
00<00?ooool0N?oo001cool00`00ooooo`0@ool40005ool00`00ooooo`0Cool00`00ooooo`1hool0
07Coo`03003ooooo017oo`03003ooooo00Coo`03003ooooo01;oo`03003ooooo07Woo`00M?oo00<0
0?ooool04Ooo00<00?ooool01?oo0`004_oo00<00?ooool0NOoo001eool00`00ooooo`08ool40004
ool00`00ooooo`04ool00`00ooooo`0Aool00`00ooooo`1jool007Goo`03003ooooo013oo`03003o
oooo00Coo`03003ooooo017oo`03003ooooo07[oo`00M_oo00<00?ooool03Ooo0`001_oo00<00?oo
ool04?oo00<00?ooool0Nooo001fool00`00ooooo`0?ool00`00ooooo`04ool00`00ooooo`0@ool0
0`00ooooo`1kool007Ooo`03003ooooo01Goo`03003ooooo00ooo`03003ooooo07coo`00Mooo00<0
0?ooool05Ooo0P004?oo00<00?ooool0O?oo001hool00`00ooooo`0Dool00`00ooooo`0>ool00`00
ooooo`1mool007Soo`03003ooooo01Coo`03003ooooo00koo`03003ooooo07goo`00NOoo00<00?oo
ool04ooo00<00?ooool03Ooo00<00?ooool0O_oo001jool00`00ooooo`0Bool00`00ooooo`0<ool0
0`00ooooo`1oool007[oo`03003ooooo01;oo`8000goo`03003ooooo07ooo`00Nooo00<00?ooool0
4Ooo00<00?ooool02ooo00<00?ooool0P?oo001lool00`00ooooo`0@ool00`00ooooo`0;ool00`00
ooooo`20ool007coo`03003ooooo013oo`03003ooooo00[oo`03003ooooo087oo`00OOoo00<00?oo
ool03ooo00<00?ooool02_oo00<00?ooool0POoo001nool00`00ooooo`0>ool00`00ooooo`09ool0
0`00ooooo`22ool000[oo`@003[oo`8000Coo`03003ooooo00;oo`<002Soo`03003ooooo00koo`80
00[oo`03003ooooo02goo`8000Coo`03003ooooo00;oo`<003[oo`@000Woo`003?oo00<00?ooool0
>?oo00@00?ooool000_oo`03003ooooo02Koo`03003ooooo00goo`03003ooooo00Soo`03003ooooo
02goo`04003ooooo000;ool00`00ooooo`0iool00`00ooooo`08ool000coo`03003ooooo03Soo`04
003ooooo000;ool00`00ooooo`0Vool00`00ooooo`0=ool00`00ooooo`08ool00`00ooooo`0]ool0
1000ooooo`002ooo00<00?ooool0>Ooo00<00?ooool02?oo0004ool40004ool00`00ooooo`0bool4
0002ool01000ooooo`002?oo0`00:_oo00<00?ooool03?oo00<00?ooool01ooo00<00?ooool0;_oo
00@00?ooool000Soo`<003coo`03003ooooo00Soo`003?oo00<00?ooool0>?oo00@00?ooool000So
o`03003ooooo02[oo`03003ooooo00coo`03003ooooo00Ooo`03003ooooo02koo`04003ooooo0008
ool00`00ooooo`0lool00`00ooooo`08ool000[oo`<003[oo`04003ooooo0008ool00`00ooooo`0[
ool00`00ooooo`0;ool20007ool00`00ooooo`0_ool01000ooooo`002?oo00<00?ooool0>_oo0`00
2_oo000<ool00`00ooooo`0iool20009ool4000[ool00`00ooooo`0:ool00`00ooooo`05ool00`00
ooooo`0aool20009ool4000kool00`00ooooo`08ool008?oo`03003ooooo00Woo`03003ooooo00Co
o`03003ooooo08Ooo`00Q?oo00<00?ooool02?oo00<00?ooool00ooo00<00?ooool0R?oo0025ool2
0008ool01000oooooooo0P00Rooo0027ool20006ool01000ool00000SOoo0002oooo000L0003ool0
00Woo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo
00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo
00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo
00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo
00[oo`03003ooooo00Soo`002Ooo00<00?ooool0@?oo00<00?ooool0@?oo00<00?ooool0@?oo00<0
0?ooool0@?oo00<00?ooool02?oo002?ool00`00ooooo`2>ool008ooo`03003ooooo08koo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.07098, -6.16078, \
0.00746331, 0.0357279}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.7, 
          0.8}, {mGamma, \(-0.3\), \(-0.4\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(mGamma\)\" is similar \
to existing symbol \"\!\(Gamma\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(realsol[k_, a_] := 
      FindRoot[kx0corrtestfunc[Om, k, a] \[Equal] 0, {Om, 0.6, 0.7}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[BoxData[
    \(realsol2[k_, a_] := 
      FindRoot[kx0corrtestfunc[Om, k, a] \[Equal] 0, {Om, 0.8, 0.9}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell["xi=1", "Subsubsection"],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, k*0.99, 
          k*0.999}, {mGamma, \(-k\)*0.93, \(-k\)*0.98}, 
        MaxIterations \[Rule] 200]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[2, 1]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.5380077540253834`, 
      mGamma \[Rule] \(-2.026372015840169`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.9, 1]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.542140645262367`, 
      mGamma \[Rule] \(-1.9310538340591938`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.6, 1]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.5562098414957524`, 
      mGamma \[Rule] \(-1.652925223506468`\)}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 1.49, 
          1.5}, {mGamma, \(-2.5\), \(-3\)}, 
        MaxIterations \[Rule] 200]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[3, 1]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.5130368034469688`, 
      mGamma \[Rule] \(-3.007215575193522`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, Om /. strangesol[k, 3]}, {k, 1.158, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-3.20092 2.78474 -1.87372 1.65136 [
[.00153 .09541 -12 -9 ]
[.00153 .09541 12 0 ]
[.28001 .09541 -12 -9 ]
[.28001 .09541 12 0 ]
[.41924 .09541 -9 -9 ]
[.41924 .09541 9 0 ]
[.55848 .09541 -12 -9 ]
[.55848 .09541 12 0 ]
[.69772 .09541 -9 -9 ]
[.69772 .09541 9 0 ]
[.83695 .09541 -12 -9 ]
[.83695 .09541 12 0 ]
[.97619 .09541 -9 -9 ]
[.97619 .09541 9 0 ]
[.12827 .02534 -24 -4.5 ]
[.12827 .02534 0 4.5 ]
[.12827 .19048 -24 -4.5 ]
[.12827 .19048 0 4.5 ]
[.12827 .27305 -18 -4.5 ]
[.12827 .27305 0 4.5 ]
[.12827 .35561 -24 -4.5 ]
[.12827 .35561 0 4.5 ]
[.12827 .43818 -18 -4.5 ]
[.12827 .43818 0 4.5 ]
[.12827 .52075 -24 -4.5 ]
[.12827 .52075 0 4.5 ]
[.12827 .60332 -18 -4.5 ]
[.12827 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.00153 .10791 m
.00153 .11416 L
s
[(1.15)] .00153 .09541 0 1 Mshowa
.28001 .10791 m
.28001 .11416 L
s
[(1.25)] .28001 .09541 0 1 Mshowa
.41924 .10791 m
.41924 .11416 L
s
[(1.3)] .41924 .09541 0 1 Mshowa
.55848 .10791 m
.55848 .11416 L
s
[(1.35)] .55848 .09541 0 1 Mshowa
.69772 .10791 m
.69772 .11416 L
s
[(1.4)] .69772 .09541 0 1 Mshowa
.83695 .10791 m
.83695 .11416 L
s
[(1.45)] .83695 .09541 0 1 Mshowa
.97619 .10791 m
.97619 .11416 L
s
[(1.5)] .97619 .09541 0 1 Mshowa
.125 Mabswid
.02938 .10791 m
.02938 .11166 L
s
.05723 .10791 m
.05723 .11166 L
s
.08507 .10791 m
.08507 .11166 L
s
.11292 .10791 m
.11292 .11166 L
s
.16862 .10791 m
.16862 .11166 L
s
.19646 .10791 m
.19646 .11166 L
s
.22431 .10791 m
.22431 .11166 L
s
.25216 .10791 m
.25216 .11166 L
s
.30785 .10791 m
.30785 .11166 L
s
.3357 .10791 m
.3357 .11166 L
s
.36355 .10791 m
.36355 .11166 L
s
.3914 .10791 m
.3914 .11166 L
s
.44709 .10791 m
.44709 .11166 L
s
.47494 .10791 m
.47494 .11166 L
s
.50278 .10791 m
.50278 .11166 L
s
.53063 .10791 m
.53063 .11166 L
s
.58633 .10791 m
.58633 .11166 L
s
.61417 .10791 m
.61417 .11166 L
s
.64202 .10791 m
.64202 .11166 L
s
.66987 .10791 m
.66987 .11166 L
s
.72556 .10791 m
.72556 .11166 L
s
.75341 .10791 m
.75341 .11166 L
s
.78126 .10791 m
.78126 .11166 L
s
.80911 .10791 m
.80911 .11166 L
s
.8648 .10791 m
.8648 .11166 L
s
.89265 .10791 m
.89265 .11166 L
s
.9205 .10791 m
.9205 .11166 L
s
.94834 .10791 m
.94834 .11166 L
s
.25 Mabswid
0 .10791 m
1 .10791 L
s
.14077 .02534 m
.14702 .02534 L
s
[(1.15)] .12827 .02534 1 0 Mshowa
.14077 .19048 m
.14702 .19048 L
s
[(1.25)] .12827 .19048 1 0 Mshowa
.14077 .27305 m
.14702 .27305 L
s
[(1.3)] .12827 .27305 1 0 Mshowa
.14077 .35561 m
.14702 .35561 L
s
[(1.35)] .12827 .35561 1 0 Mshowa
.14077 .43818 m
.14702 .43818 L
s
[(1.4)] .12827 .43818 1 0 Mshowa
.14077 .52075 m
.14702 .52075 L
s
[(1.45)] .12827 .52075 1 0 Mshowa
.14077 .60332 m
.14702 .60332 L
s
[(1.5)] .12827 .60332 1 0 Mshowa
.125 Mabswid
.14077 .04186 m
.14452 .04186 L
s
.14077 .05837 m
.14452 .05837 L
s
.14077 .07488 m
.14452 .07488 L
s
.14077 .0914 m
.14452 .0914 L
s
.14077 .12442 m
.14452 .12442 L
s
.14077 .14094 m
.14452 .14094 L
s
.14077 .15745 m
.14452 .15745 L
s
.14077 .17397 m
.14452 .17397 L
s
.14077 .20699 m
.14452 .20699 L
s
.14077 .22351 m
.14452 .22351 L
s
.14077 .24002 m
.14452 .24002 L
s
.14077 .25653 m
.14452 .25653 L
s
.14077 .28956 m
.14452 .28956 L
s
.14077 .30607 m
.14452 .30607 L
s
.14077 .32259 m
.14452 .32259 L
s
.14077 .3391 m
.14452 .3391 L
s
.14077 .37213 m
.14452 .37213 L
s
.14077 .38864 m
.14452 .38864 L
s
.14077 .40516 m
.14452 .40516 L
s
.14077 .42167 m
.14452 .42167 L
s
.14077 .4547 m
.14452 .4547 L
s
.14077 .47121 m
.14452 .47121 L
s
.14077 .48772 m
.14452 .48772 L
s
.14077 .50424 m
.14452 .50424 L
s
.14077 .53726 m
.14452 .53726 L
s
.14077 .55378 m
.14452 .55378 L
s
.14077 .57029 m
.14452 .57029 L
s
.14077 .58681 m
.14452 .58681 L
s
.14077 .00883 m
.14452 .00883 L
s
.25 Mabswid
.14077 0 m
.14077 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .03855 m
.06244 .06146 L
.10458 .08645 L
.14415 .10992 L
.18221 .13248 L
.22272 .15651 L
.26171 .17963 L
.30316 .20421 L
.34309 .22789 L
.3815 .25066 L
.42237 .2749 L
.46172 .29823 L
.49955 .32067 L
.53984 .34456 L
.57861 .36755 L
.61984 .392 L
.65954 .41555 L
.69774 .43819 L
.73838 .4623 L
.77751 .4855 L
.81909 .51016 L
.85916 .53392 L
.89771 .55678 L
.93871 .5811 L
.97619 .60332 L
s
.02381 .03827 m
.06244 .03785 L
.10458 .03729 L
.14415 .03668 L
.18221 .03602 L
.22272 .03524 L
.26171 .03443 L
.30316 .03351 L
.34309 .03257 L
.3815 .03162 L
.42237 .03057 L
.46172 .02953 L
.49955 .0285 L
.53984 .02737 L
.57861 .02628 L
.61984 .02509 L
.65954 .02394 L
.69774 .02283 L
.73838 .02164 L
.77751 .02049 L
.81909 .01927 L
.85916 .0181 L
.89771 .01698 L
.93871 .01579 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00<_oo00<00?ooool0jooo000bool00`00ooooo`3[ool003;oo`03003ooooo0>_o
o`00<_oo0P00k?oo000bool00`00ooooo`3Lool60009ool001Ooo`@000?oo`03003ooooo00;oo`@0
00;oo`<000Koo`03003ooooo0;coob0000ooo`006Ooo00<00?ooool02Ooo00<00?ooool01?oo00<0
0?ooool00ooo00<00?ooool0Vooo8@00;ooo000Iool00`00ooooo`09ool00`00ooooo`04ool00`00
ooooo`03ool2001koolQ001@ool001Woo`03003ooooo00Woo`04003oooooool30006ool00`00oooo
o`1IoolQ001aool001Woo`03003ooooo00Woo`05003oooooool00008ool00`00ooooo`0_oolZ002B
ool001Ooo`<000Woo`<000?oo`04003ooooooolg002lool001;ooa/000Goo`03003ooooo0>_oo`00
4ooo0P007Ooo0P00k?oo000Eool2000Kool00`00ooooo`3[ool001Ooo`03003ooooo01Soo`03003o
oooo0>_oo`006?oo0P006?oo00<00?ooool0jooo000Jool2000Fool2003/ool001coo`03003ooooo
01?oo`03003ooooo0>_oo`007Ooo0P004ooo00<00?ooool0jooo00000ooo000000020003ool00`00
ooooo`02ool40002ool30009ool2000Aool00`00ooooo`0Hool40003ool00`00ooooo`02ool40002
ool3000Dool40003ool00`00ooooo`03ool2000Dool40003ool00`00ooooo`03ool20003ool3000D
ool40003ool00`00ooooo`04ool00`00ooooo`0Bool40003ool00`00ooooo`04ool01000oooooooo
0`005?oo10000ooo00<00?ooool00_oo0`000ooo0003ool00`00ooooo`09ool00`00ooooo`04ool0
0`00ooooo`08ool00`00ooooo`0>ool00`00ooooo`0Jool00`00ooooo`07ool00`00ooooo`06ool0
0`00ooooo`0Cool00`00ooooo`07ool01000ooooo`005Ooo00<00?ooool01ooo00@00?ooool000Go
o`03003ooooo01?oo`03003ooooo00Woo`03003ooooo01Coo`03003ooooo00Woo`03003ooooo00Co
o`03003ooooo01?oo`03003ooooo00[oo`40007oo`7oo`000ooo00<00?ooool02Ooo00<00?ooool0
1?oo00<00?ooool02Ooo0P003_oo0P006ooo00<00?ooool02?oo00<00?ooool01Ooo00<00?ooool0
4ooo00<00?ooool02_oo00<00?ooool04ooo00<00?ooool02_oo00<00?ooool00ooo00<00?ooool0
4ooo00<00?ooool01_oo1@005Ooo00<00?ooool01_oo1@001Ooo00<00?ooool04ooo00<00?ooool0
2_oo0@000Ooo0Ooo0003ool00`00ooooo`09ool01000oooooooo0`003_oo0P003?oo00<00?ooool0
6_oo00<00?ooool02Ooo00@00?ooooooo`<001Koo`03003ooooo00Soo`8001Koo`03003ooooo00So
o`8000?oo`<001Koo`03003ooooo00Koo`04003ooooo000Fool00`00ooooo`06ool01000ooooo`00
0ooo0`005_oo00<00?ooool01ooo0`000ooo0003ool00`00ooooo`09ool01@00oooooooo00004_oo
0P002_oo00<00?ooool06_oo00<00?ooool02_oo00@00?ooool001Soo`03003ooooo00[oo`03003o
oooo01?oo`03003ooooo00[oo`04003ooooo000Hool00`00ooooo`07ool00`00ool0000Fool00`00
ooooo`07ool00`00ool00003ool00`00ooooo`0Fool00`00ooooo`07ool00`00ooooo`03ool00004
ool000000009ool30003ool00`00ooooo`0Bool00`00ooooo`07ool00`00ooooo`0Hool30009ool0
1000ooooo`000_oo00<00?ooool05?oo0`002Ooo00@00?ooool001?oo`<000Woo`04003ooooo0002
ool00`00ooooo`0Dool3000:ool2000Dool3000:ool20003ool00`00ooooo`0Dool30009ool00`00
ooooo`03ool000?oo`03003ooooo00Woo`04003oooooool4000Bool20007ool2000Kool00`00oooo
o`08ool20003ool4000Eool00`00ooooo`08ool2000Fool00`00ooooo`08ool20003ool4000Eool0
0`00ooooo`09ool00`00ooooo`0Dool00`00ooooo`09ool01000oooooooo10005Ooo00<00?ooool0
1ooo10000_oo000[ool20005ool00`00ooooo`3[ool002goo`03003ooooo00;oo`03003ooooo0>_o
o`00;_oo0P000_oo00<00?ooool0jooo000`ool3003]ool000_oool001<000?oo`003?oo00<00?oo
ool01?oo00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo0`001?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01ooo
000<ool00`00ooooo`0Sool01000ooooo`008_oo00<00?ooool08ooo00<00?ooool08ooo00<00?oo
ool08ooo00<00?ooool08ooo00<00?ooool08ooo00<00?ooool01ooo000bool01000oooooooo0P00
j?oo000bool20004ool2003Vool003;oo`03003ooooo00Goo`03003ooooo0>?oo`00<_oo00<00?oo
ool01_oo0P00hooo000bool00`00ooooo`08ool00`00ooooo`3Pool003;oo`03003ooooo00Woo`80
0>3oo`00<_oo0P003?oo0P00g_oo000bool00`00ooooo`0=ool2003Lool003;oo`03003ooooo00oo
o`800=[oo`00<_oo00<00?ooool04Ooo0P00f?oo000bool2000Dool00`00ooooo`3Eool003;oo`03
003ooooo01Coo`800=Goo`00<_oo00<00?ooool05_oo0P00dooo000bool00`00ooooo`0Hool2003A
ool003;oo`03003ooooo01[oo`800<ooo`00<_oo0P007Ooo0P00cOoo000bool00`00ooooo`0Nool0
0`00ooooo`3:ool001Ooo`@000?oo`03003ooooo00;oo`@000;oo`<000Koo`03003ooooo01ooo`80
0<[oo`006Ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool00ooo00<00?ooool08Ooo00<0
0?ooool0aooo000Iool00`00ooooo`08ool00`00ooooo`05ool00`00ooooo`03ool2000Sool20037
ool001Woo`03003ooooo00Woo`04003oooooool30006ool00`00ooooo`0Tool20035ool001Woo`03
003ooooo00[oo`04003ooooo0008ool00`00ooooo`0Vool00`00ooooo`32ool001Ooo`<000Woo`04
003ooooo0002ool00`00ooooo`06ool00`00ooooo`0Wool20032ool001Woo`03003ooooo00Soo`80
00?oo`@000Goo`03003ooooo02Woo`03003ooooo0;ooo`00<_oo0P00:ooo0P00_ooo000bool00`00
ooooo`0/ool00`00ooooo`2lool003;oo`03003ooooo02goo`800;coo`00<_oo00<00?ooool0;ooo
0P00^_oo000bool2000bool00`00ooooo`2gool003;oo`03003ooooo03;oo`800;Ooo`00<_oo00<0
0?ooool0=?oo00<00?ooool0]?oo000bool00`00ooooo`0eool2002dool003;oo`03003ooooo03Oo
o`800;;oo`00<_oo0P00>_oo00<00?ooool0[ooo000bool00`00ooooo`0jool2002_ool003;oo`03
003ooooo03coo`800:goo`00<_oo00<00?ooool0?_oo00<00?ooool0Z_oo000bool20010ool2002Z
ool003;oo`03003ooooo047oo`800:Soo`00<_oo00<00?ooool0@ooo0P00Y_oo000Mool40003ool0
0`00ooooo`03ool20006ool00`00ooooo`15ool2002Tool001ooo`03003ooooo00Ooo`04003ooooo
0005ool00`00ooooo`17ool2002Rool001ooo`03003ooooo00[oo`03003ooooo00?oo`8004[oo`03
003ooooo09ooo`007ooo00<00?ooool02?oo0P001_oo00<00?ooool0B_oo0P00Wooo000Oool00`00
ooooo`0:ool00`00ooooo`03ool00`00ooooo`1<ool00`00ooooo`2Lool001goo`<000Woo`04003o
oooo0005ool00`00ooooo`1=ool2002Lool001ooo`03003ooooo00Soo`8000Koo`80053oo`8009[o
o`00<_oo00<00?ooool0DOoo00<00?ooool0Uooo000bool00`00ooooo`1Bool2002Gool003;oo`03
003ooooo05Coo`03003ooooo09Coo`00<_oo00<00?ooool0EOoo0P00U?oo000bool2001Hool2002B
ool003;oo`03003ooooo05Woo`03003ooooo08ooo`00<_oo00<00?ooool0F_oo0P00Sooo000bool0
0`00ooooo`1Lool2002=ool003;oo`8005ooo`03003ooooo08[oo`00<_oo00<00?ooool0Gooo0P00
R_oo000bool00`00ooooo`1Qool20028ool003;oo`03003ooooo06?oo`8008Koo`00<_oo00<00?oo
ool0IOoo0P00Q?oo000bool2001Xool20022ool003;oo`03003ooooo06Woo`03003ooooo07ooo`00
<_oo00<00?ooool0J_oo0P00Oooo000Gool40003ool00`00ooooo`03ool20003ool30006ool00`00
ooooo`1/ool00`00ooooo`1lool001Woo`03003ooooo00Ooo`04003ooooo0005ool00`00ooooo`03
ool00`00ooooo`1]ool2001lool001Woo`03003ooooo00[oo`03003ooooo00?oo`03003ooooo00?o
o`80073oo`8007[oo`006Ooo00<00?ooool02?oo0P000ooo0`001_oo00<00?ooool0LOoo00<00?oo
ool0Mooo000Iool00`00ooooo`0:ool01000ooooo`002?oo00<00?ooool0L_oo0P00Mooo000Gool3
0009ool01000ooooo`000_oo00<00?ooool01_oo00<00?ooool0M?oo00<00?ooool0M?oo000Iool0
0`00ooooo`08ool20003ool40005ool2001fool2001dool003;oo`03003ooooo07Ooo`8007;oo`00
<_oo00<00?ooool0NOoo0P00L?oo000bool00`00ooooo`1kool2001^ool003;oo`03003ooooo07go
o`8006coo`00<_oo0P00P?oo00<00?ooool0JOoo000bool00`00ooooo`20ool2001Yool003;oo`03
003ooooo08;oo`03003ooooo06Koo`00<_oo00<00?ooool0Pooo0P00I_oo000bool20026ool2001T
ool003;oo`03003ooooo08Ooo`03003ooooo067oo`00<_oo00<00?ooool0R?oo0P00HOoo000bool0
0`00ooooo`2:ool00`00ooooo`1Nool003;oo`03003ooooo08_oo`8005koo`00<_oo0P00S_oo0P00
G?oo000bool00`00ooooo`2?ool2001Jool001goo`@000?oo`03003ooooo00Coo`03003ooooo00Co
o`03003ooooo097oo`8005Soo`007ooo00<00?ooool02Ooo00<00?ooool01?oo00<00?ooool0Tooo
0P00E_oo000Oool00`00ooooo`06ool50005ool2002Fool00`00ooooo`1Cool001ooo`03003ooooo
00Koo`04003ooooo0006ool00`00ooooo`2Fool2001Cool001ooo`03003ooooo00Ooo`03003oo`00
00Koo`03003ooooo09Soo`03003ooooo053oo`007Ooo0`002_oo0P001_oo00<00?ooool0VOoo0P00
D?oo000Oool00`00ooooo`09ool00`00ooooo`04ool00`00ooooo`2Kool2001>ool003;oo`8009ko
o`03003ooooo04_oo`00<_oo00<00?ooool0W_oo0P00Booo000bool00`00ooooo`2Pool00`00oooo
o`18ool003;oo`03003ooooo0:7oo`8004Soo`00<_oo0P00Y?oo0P00A_oo000bool00`00ooooo`2U
ool00`00ooooo`13ool003;oo`03003ooooo0:Koo`8004?oo`00<_oo00<00?ooool0Z?oo0P00@Ooo
000bool00`00ooooo`2Zool00`00ooooo`0nool003;oo`800:coo`8003koo`00<_oo00<00?ooool0
[Ooo0P00??oo000bool00`00ooooo`2_ool00`00ooooo`0iool003;oo`03003ooooo0;3oo`8003Wo
o`00<_oo0P00/ooo0P00=ooo000bool00`00ooooo`2dool2000eool003;oo`03003ooooo0;Koo`03
003ooooo03;oo`005ooo10000ooo00<00?ooool01?oo00@00?ooooooo`<000Koo`03003ooooo0;Oo
o`8003;oo`006Ooo00<00?ooool02Ooo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0^Ooo
0P00<?oo000Iool00`00ooooo`06ool50005ool00`00ooooo`03ool2002lool2000^ool001Woo`03
003ooooo00Koo`04003ooooo0003ool30006ool00`00ooooo`2mool2000/ool001Woo`03003ooooo
00Ooo`03003oo`0000?oo`03003ooooo00Koo`03003ooooo0;ooo`8002[oo`005ooo0`002_oo0P00
0ooo00<00?ooool01_oo00<00?ooool0`Ooo00<00?ooool09ooo000Iool00`00ooooo`09ool01000
oooooooo10001Ooo0P00`ooo0P009ooo000bool00`00ooooo`34ool2000Uool003;oo`03003ooooo
0<Koo`03003ooooo02;oo`00<_oo00<00?ooool0aooo0P008_oo000bool00`00ooooo`39ool2000P
ool003;oo`800<coo`03003ooooo01goo`00<_oo00<00?ooool0c?oo0P007Ooo000bool00`00oooo
o`3>ool00`00ooooo`0Jool003;oo`03003ooooo0<ooo`8001[oo`00<_oo0P00d_oo0P006?oo000b
ool00`00ooooo`3Cool00`00ooooo`0Eool003;oo`03003ooooo0=Coo`8001Goo`00<_oo00<00?oo
ool0e_oo00<00?ooool04_oo000bool00`00ooooo`3Gool2000Bool003;oo`800=[oo`80013oo`00
<_oo00<00?ooool0fooo0P003_oo000Mool40003ool00`00ooooo`02ool30006ool00`00ooooo`3M
ool2000<ool001ooo`03003ooooo00[oo`03003ooooo00?oo`03003ooooo0=ooo`8000[oo`007ooo
00<00?ooool02_oo00<00?ooool00ooo0P00h_oo00<00?ooool01ooo000Oool00`00ooooo`07ool3
0006ool00`00ooooo`3[ool001ooo`03003ooooo00Ooo`03003ooooo00Koo`03003ooooo0>_oo`00
7Ooo0`002Ooo00<00?ooool01_oo00<00?ooool0jooo000Oool00`00ooooo`07ool40005ool00`00
ooooo`3[ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {1.13421, 1.12602, 0.0013158, \
0.00221888}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 1, 
          1.1}, {mGamma, \(-0.7\), \(-0.9\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.1, 3]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \({Om \[Rule] 1.1035468759640183`, 
      mGamma \[Rule] \(-0.6825439118939567`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[1.1705547647282446 - I\ 0.6941242019418665, 1.1, 
      3]\)], "Input"],

Cell[BoxData[
    \(\(-1.1333043603593906`\) - 
      1.4215316459103413`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[kx0corrtestfunc[Om - I\ 0.72, 1.1, 3]], {Om, 0.8, 1.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.5 1.90476 0.276409 0.193495 [
[.02381 .26391 -9 -9 ]
[.02381 .26391 9 0 ]
[.21429 .26391 -9 -9 ]
[.21429 .26391 9 0 ]
[.59524 .26391 -9 -9 ]
[.59524 .26391 9 0 ]
[.78571 .26391 -9 -9 ]
[.78571 .26391 9 0 ]
[.97619 .26391 -9 -9 ]
[.97619 .26391 9 0 ]
[.39226 .08291 -12 -4.5 ]
[.39226 .08291 0 4.5 ]
[.39226 .17966 -24 -4.5 ]
[.39226 .17966 0 4.5 ]
[.39226 .37316 -18 -4.5 ]
[.39226 .37316 0 4.5 ]
[.39226 .4699 -6 -4.5 ]
[.39226 .4699 0 4.5 ]
[.39226 .56665 -18 -4.5 ]
[.39226 .56665 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .27641 m
.02381 .28266 L
s
[(0.8)] .02381 .26391 0 1 Mshowa
.21429 .27641 m
.21429 .28266 L
s
[(0.9)] .21429 .26391 0 1 Mshowa
.59524 .27641 m
.59524 .28266 L
s
[(1.1)] .59524 .26391 0 1 Mshowa
.78571 .27641 m
.78571 .28266 L
s
[(1.2)] .78571 .26391 0 1 Mshowa
.97619 .27641 m
.97619 .28266 L
s
[(1.3)] .97619 .26391 0 1 Mshowa
.125 Mabswid
.0619 .27641 m
.0619 .28016 L
s
.1 .27641 m
.1 .28016 L
s
.1381 .27641 m
.1381 .28016 L
s
.17619 .27641 m
.17619 .28016 L
s
.25238 .27641 m
.25238 .28016 L
s
.29048 .27641 m
.29048 .28016 L
s
.32857 .27641 m
.32857 .28016 L
s
.36667 .27641 m
.36667 .28016 L
s
.44286 .27641 m
.44286 .28016 L
s
.48095 .27641 m
.48095 .28016 L
s
.51905 .27641 m
.51905 .28016 L
s
.55714 .27641 m
.55714 .28016 L
s
.63333 .27641 m
.63333 .28016 L
s
.67143 .27641 m
.67143 .28016 L
s
.70952 .27641 m
.70952 .28016 L
s
.74762 .27641 m
.74762 .28016 L
s
.82381 .27641 m
.82381 .28016 L
s
.8619 .27641 m
.8619 .28016 L
s
.9 .27641 m
.9 .28016 L
s
.9381 .27641 m
.9381 .28016 L
s
.25 Mabswid
0 .27641 m
1 .27641 L
s
.40476 .08291 m
.41101 .08291 L
s
[(-1)] .39226 .08291 1 0 Mshowa
.40476 .17966 m
.41101 .17966 L
s
[(-0.5)] .39226 .17966 1 0 Mshowa
.40476 .37316 m
.41101 .37316 L
s
[(0.5)] .39226 .37316 1 0 Mshowa
.40476 .4699 m
.41101 .4699 L
s
[(1)] .39226 .4699 1 0 Mshowa
.40476 .56665 m
.41101 .56665 L
s
[(1.5)] .39226 .56665 1 0 Mshowa
.125 Mabswid
.40476 .10226 m
.40851 .10226 L
s
.40476 .12161 m
.40851 .12161 L
s
.40476 .14096 m
.40851 .14096 L
s
.40476 .16031 m
.40851 .16031 L
s
.40476 .19901 m
.40851 .19901 L
s
.40476 .21836 m
.40851 .21836 L
s
.40476 .23771 m
.40851 .23771 L
s
.40476 .25706 m
.40851 .25706 L
s
.40476 .29576 m
.40851 .29576 L
s
.40476 .31511 m
.40851 .31511 L
s
.40476 .33446 m
.40851 .33446 L
s
.40476 .35381 m
.40851 .35381 L
s
.40476 .39251 m
.40851 .39251 L
s
.40476 .41186 m
.40851 .41186 L
s
.40476 .43121 m
.40851 .43121 L
s
.40476 .45055 m
.40851 .45055 L
s
.40476 .48925 m
.40851 .48925 L
s
.40476 .5086 m
.40851 .5086 L
s
.40476 .52795 m
.40851 .52795 L
s
.40476 .5473 m
.40851 .5473 L
s
.40476 .06356 m
.40851 .06356 L
s
.40476 .04421 m
.40851 .04421 L
s
.40476 .02486 m
.40851 .02486 L
s
.40476 .00552 m
.40851 .00552 L
s
.40476 .586 m
.40851 .586 L
s
.40476 .60535 m
.40851 .60535 L
s
.25 Mabswid
.40476 0 m
.40476 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .55858 L
.10458 .51547 L
.14415 .47976 L
.18221 .44928 L
.22272 .42055 L
.26171 .39614 L
.30316 .37331 L
.34309 .35404 L
.3815 .33778 L
.42237 .32271 L
.46172 .31015 L
.49955 .29973 L
.53984 .29026 L
.55838 .28643 L
.57861 .2826 L
.58392 .28166 L
.58957 .28068 L
.59232 .28021 L
.59367 .27998 L
.59491 .27978 L
.59604 .01472 L
.59723 .01502 L
.59975 .01565 L
.61984 .02071 L
.65781 .03036 L
.69823 .04074 L
.73714 .05084 L
.7785 .06168 L
.81835 .07223 L
.85668 .08249 L
.89746 .09351 L
.93672 .10425 L
.97448 .11467 L
.97619 .11514 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{44, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo001dool0
0`00ooooo`2Yool007Coo`800:[oo`00M?oo00<00?ooool0ZOoo001dool00`00ooooo`2Yool007Co
o`03003ooooo03?oo`<007?oo`00M?oo00<00?ooool0<ooo00<00?ooool00`00L?oo001dool00`00
ooooo`0cool00`00ooooo`03ool4001/ool007Coo`8003Coo`03003ooooo00Ooo`H006Koo`00M?oo
00<00?ooool0<ooo00<00?ooool03Ooo1000H_oo001dool00`00ooooo`0cool00`00ooooo`0Aool4
001Nool007Coo`03003ooooo03?oo`03003ooooo01Goo`@005[oo`00M?oo00<00?ooool0<ooo00<0
0?ooool06Ooo0`00Eooo001dool2000dool00`00ooooo`0Lool4001Cool007Coo`03003ooooo03?o
o`03003ooooo023oo`@004ooo`00M?oo00<00?ooool0<ooo00<00?ooool09?oo0`00C?oo001dool0
0`00ooooo`0cool00`00ooooo`0Wool40018ool007Coo`03003ooooo03?oo`03003ooooo02_oo`@0
04Coo`00M?oo00<00?ooool0<ooo00<00?ooool0;ooo1000@?oo001dool2000dool00`00ooooo`0c
ool4000lool007Coo`03003ooooo03?oo`03003ooooo03Ooo`@003Soo`00M?oo00<00?ooool0<ooo
00<00?ooool0>ooo0`00=Ooo001/ool40004ool00`00ooooo`0cool00`00ooooo`0nool4000aool0
06koo`03003ooooo00?oo`03003ooooo03?oo`03003ooooo04;oo`@002goo`00K_oo00<00?ooool0
0ooo0`00<ooo00<00?ooool0A_oo0`00:_oo001Vool40004ool00`00ooooo`03ool00`00ooooo`0c
ool00`00ooooo`19ool4000Vool006koo`03003ooooo00?oo`03003ooooo03?oo`03003ooooo04go
o`@002;oo`00K?oo0`001Ooo00<00?ooool0<ooo00<00?ooool0DOoo0`007ooo001^ool00`00oooo
o`03ool00`00ooooo`0cool00`00ooooo`1Dool4000Kool007Coo`03003ooooo03?oo`03003ooooo
05Soo`@001Ooo`00M?oo0P00=?oo00<00?ooool0G?oo0`005?oo001dool00`00ooooo`0cool00`00
ooooo`1Oool4000@ool007Coo`03003ooooo03?oo`03003ooooo06?oo`@000coo`00M?oo00<00?oo
ool0<ooo00<00?ooool0Iooo0`002Ooo001dool00`00ooooo`0cool00`00ooooo`1cool007Coo`80
03Coo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001dool00`00ooooo`0c
ool00`00ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo
00<00?ooool0Looo001dool00`00ooooo`0cool00`00ooooo`1cool007Coo`8003Coo`03003ooooo
07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001dool00`00ooooo`0cool00`00ooooo`1c
ool007Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo
001dool2000dool00`00ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<0
0?ooool0<ooo00<00?ooool0Looo001Qool20004ool00`00ooooo`02ool30005ool00`00ooooo`0c
ool00`00ooooo`1cool0063oo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0cool00`00
ooooo`1cool0063oo`04003ooooo000;ool00`00ooooo`02ool3000cool00`00ooooo`1cool005[o
o`@000;oo`04003ooooo0008ool30005ool00`00ooooo`0cool00`00ooooo`1cool0063oo`04003o
oooo0008ool00`00ooooo`05ool00`00ooooo`0cool00`00ooooo`1cool0063oo`04003ooooo0008
ool00`00ooooo`05ool00`00ooooo`0cool00`00ooooo`1cool0067oo`8000Woo`@000Coo`03003o
oooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001dool2000dool0
0`00ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<0
0?ooool0Looo001dool00`00ooooo`0cool00`00ooooo`1cool007Coo`03003ooooo03?oo`03003o
oooo07?oo`00M?oo0P00=?oo00<00?ooool0Looo001dool00`00ooooo`0cool00`00ooooo`1cool0
07Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001d
ool00`00ooooo`0cool00`00ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`000_oo
0P001?oo00<00?ooool00ooo0P009ooo0P001?oo00<00?ooool00_oo0`00;ooo0P00;?oo10000ooo
0P000ooo10009_oo10000ooo00<00?ooool00_oo10009_oo10000ooo00<00?ooool00ooo0P000ooo
00001Ooo003ooooo00002?oo00@00?ooool002Goo`04003ooooo000;ool00`00ooooo`0/ool00`00
ooooo`0]ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`0Wool00`00ooooo`07ool00`00
ooooo`0Yool00`00ooooo`07ool01000ooooo`000_oo00001Ooo003ooooo00002?oo00@00?ooool0
02Goo`04003ooooo000;ool00`00ooooo`0/ool00`00ooooo`0]ool00`00ooooo`03ool00`00oooo
o`03ool00`00ooooo`0Wool00`00ooooo`08ool00`00ooooo`0Xool00`00ooooo`0:ool10001ool1
ool00005ool00?ooool00009ool2000Vool01000ooooo`002Ooo0`00;_oo00<00?ooool0;Ooo00<0
0?ooool00ooo00<00?ooool00ooo00<00?ooool09ooo00<00?ooool02Ooo00<00?ooool09ooo00<0
0?ooool02?oo0P000ooo00001Ooo003ooooo00002?oo00@00?ooool002Goo`04003ooooo0008ool0
1000ooooo`00;_oo00<00?ooool0;Ooo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool09ooo
00<00?ooool02_oo00<00?ooool09_oo00<00?ooool02_oo0@000Ooo0Ooo00001Ooo003ooooo0000
2?oo00@00?ooool002Goo`04003ooooo0008ool01000ooooo`00;_oo0P00;?oo0`001Ooo00@00?oo
ooooo`<002Ooo`<000Woo`04003ooooo000Vool30009ool01000ooooo`000_oo0002ool2000:ool2
000Wool2000:ool2000_ool00`00ooooo`0]ool00`00ooooo`03ool00`00ooooo`03ool00`00oooo
o`0Wool00`00ooooo`08ool2000Yool00`00ooooo`08ool20003ool007Coo`03003ooooo03?oo`03
003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001dool00`00ooooo`0cool00`00
ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`000_ooo`0070000ooo0009ool00`00
ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`04ool5000:
ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07
ool000Woo`03003ooooo03;oo`03003ooooo03?oo`03003ooooo02_oo`@000Coo`03003ooooo03?o
o`03003ooooo03?oo`03003ooooo00Ooo`00M?oo00<00?ooool09_oo1@00O_oo001dool00`00oooo
o`0Qool50023ool007Coo`8001coo`H008Soo`00M?oo00<00?ooool05ooo1000S_oo001dool00`00
ooooo`0Cool4002Bool007Coo`03003ooooo00ooo`@009Koo`00M?oo00<00?ooool03?oo0`00V_oo
001dool00`00ooooo`09ool3002Mool007Coo`8000Soo`800:3oo`00M?oo00<00?ooool01?oo0`00
X_oo001dool01000oooooooo0`00YOoo001dool4002Xool007?oo`800:_oo`00L?oo0`0000?oo`00
0000Z_oo001]ool30004ool00`00ooooo`2Yool006_oo`8000Ooo`03003ooooo0:Woo`00JOoo0P00
2Ooo00<00?ooool0ZOoo001Wool2000;ool00`00ooooo`2Yool006Goo`8000goo`03003ooooo0:Wo
o`00H_oo0`003ooo0P00Z_oo001Pool2000Bool00`00ooooo`2Yool005goo`<001Coo`03003ooooo
0:Woo`00Fooo0P001?oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0ZOoo001Iool20005
ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0ZOoo001Gool20007ool01000ooooo`00
2ooo00<00?ooool00_oo0`00ZOoo001Eool20009ool01000ooooo`002?oo0`001Ooo00<00?ooool0
ZOoo001Dool00`00ooooo`09ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0ZOoo001B
ool2000<ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0ZOoo001@ool2000?ool20009
ool40004ool00`00ooooo`2Yool004ooo`03003ooooo02;oo`03003ooooo0:Woo`00COoo0P009Ooo
0P00Z_oo001<ool00`00ooooo`0Uool00`00ooooo`2Yool004[oo`8002Soo`03003ooooo0:Woo`00
BOoo00<00?ooool0:?oo00<00?ooool0ZOoo0017ool2000[ool00`00ooooo`2Yool004Goo`8002go
o`800:[oo`00A?oo00<00?ooool0;Ooo00<00?ooool0ZOoo0012ool2000`ool00`00ooooo`2Yool0
047oo`03003ooooo033oo`03003ooooo0:Woo`00?ooo0P00<ooo00<00?ooool0ZOoo000nool00`00
ooooo`0cool00`00ooooo`2Yool003coo`8003Koo`800:[oo`00>ooo00<00?ooool0=_oo00<00?oo
ool0ZOoo000iool2000iool00`00ooooo`2Yool003Soo`03003ooooo03Woo`03003ooooo0:Woo`00
=_oo0P00??oo00<00?ooool0ZOoo000eool00`00ooooo`0lool2002Zool003?oo`8003ooo`03003o
oooo0:Woo`00<_oo00<00?ooool0?ooo00<00?ooool0ZOoo000aool00`00ooooo`0hool40004ool0
0`00ooooo`2Yool002ooo`8003goo`03003ooooo00?oo`03003ooooo0:Woo`00;_oo00<00?ooool0
?Ooo00<00?ooool00ooo0`00ZOoo000]ool00`00ooooo`0nool00`00ooooo`03ool00`00ooooo`2Y
ool002_oo`80047oo`03003ooooo00?oo`03003ooooo0:Woo`00:_oo00<00?ooool0?ooo0`001Ooo
00<00?ooool0ZOoo000Yool00`00ooooo`12ool00`00ooooo`03ool00`00ooooo`2Yool002Soo`03
003ooooo04Woo`03003ooooo0:Woo`009ooo00<00?ooool0B_oo0P00Z_oo000Vool00`00ooooo`1;
ool00`00ooooo`2Yool002Coo`8004koo`03003ooooo0:Woo`008ooo00<00?ooool0C_oo00<00?oo
ool0ZOoo000Rool00`00ooooo`1?ool00`00ooooo`2Yool0027oo`03003ooooo053oo`800:[oo`00
8?oo00<00?ooool0DOoo00<00?ooool0ZOoo000Oool00`00ooooo`1Bool00`00ooooo`2Yool001ko
o`03003ooooo05?oo`03003ooooo0:Woo`007Ooo00<00?ooool0E?oo00<00?ooool0ZOoo000Lool0
0`00ooooo`1Eool00`00ooooo`2Yool001_oo`03003ooooo05Koo`800:[oo`006_oo00<00?ooool0
Eooo00<00?ooool0ZOoo000Iool00`00ooooo`1Hool00`00ooooo`2Yool001Woo`03003ooooo05So
o`03003ooooo0:Woo`006?oo00<00?ooool0FOoo00<00?ooool0ZOoo000Gool00`00ooooo`1Jool2
002Zool001Koo`03003ooooo05_oo`03003ooooo0:Woo`005Ooo00<00?ooool0G?oo00<00?ooool0
ZOoo000Dool00`00ooooo`1Mool00`00ooooo`2Yool001?oo`03003ooooo04[oo`@000?oo`03003o
oooo00;oo`<000Goo`03003ooooo0:Woo`004_oo00<00?ooool0COoo00<00?ooool02_oo00<00?oo
ool00_oo00<00?ooool0ZOoo000Aool00`00ooooo`1>ool00`00ooooo`0:ool00`00ooooo`02ool3
002Yool0017oo`03003ooooo04koo`03003ooooo00Ooo`<000Goo`03003ooooo0:Woo`004?oo00<0
0?ooool0Cooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0ZOoo000?ool00`00ooooo`1>
ool30009ool00`00ooooo`05ool00`00ooooo`2Yool000koo`03003ooooo057oo`03003ooooo00Oo
o`@000Coo`03003ooooo0:Woo`003Ooo00<00?ooool0I?oo0P00Z_oo000<ool00`00ooooo`1Uool0
0`00ooooo`2Yool000coo`03003ooooo06Goo`03003ooooo0:Woo`002ooo00<00?ooool0I_oo00<0
0?ooool0ZOoo000:ool00`00ooooo`1Wool00`00ooooo`2Yool000Woo`03003ooooo06Soo`03003o
oooo0:Woo`00M?oo0P00Z_oo001dool00`00ooooo`2Yool007Coo`03003ooooo0:Woo`00M?oo00<0
0?ooool0ZOoo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.78327, -1.45025, \
0.00185875, 0.0182975}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Im[kx0corrtestfunc[Om - I\ 0.71, 1.15, 3]], {Om, 0.8, 1.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.5 1.90476 0.233817 0.131453 [
[.02381 .22132 -9 -9 ]
[.02381 .22132 9 0 ]
[.21429 .22132 -9 -9 ]
[.21429 .22132 9 0 ]
[.59524 .22132 -9 -9 ]
[.59524 .22132 9 0 ]
[.78571 .22132 -9 -9 ]
[.78571 .22132 9 0 ]
[.97619 .22132 -9 -9 ]
[.97619 .22132 9 0 ]
[.39226 .10236 -12 -4.5 ]
[.39226 .10236 0 4.5 ]
[.39226 .36527 -6 -4.5 ]
[.39226 .36527 0 4.5 ]
[.39226 .49672 -6 -4.5 ]
[.39226 .49672 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .23382 m
.02381 .24007 L
s
[(0.8)] .02381 .22132 0 1 Mshowa
.21429 .23382 m
.21429 .24007 L
s
[(0.9)] .21429 .22132 0 1 Mshowa
.59524 .23382 m
.59524 .24007 L
s
[(1.1)] .59524 .22132 0 1 Mshowa
.78571 .23382 m
.78571 .24007 L
s
[(1.2)] .78571 .22132 0 1 Mshowa
.97619 .23382 m
.97619 .24007 L
s
[(1.3)] .97619 .22132 0 1 Mshowa
.125 Mabswid
.0619 .23382 m
.0619 .23757 L
s
.1 .23382 m
.1 .23757 L
s
.1381 .23382 m
.1381 .23757 L
s
.17619 .23382 m
.17619 .23757 L
s
.25238 .23382 m
.25238 .23757 L
s
.29048 .23382 m
.29048 .23757 L
s
.32857 .23382 m
.32857 .23757 L
s
.36667 .23382 m
.36667 .23757 L
s
.44286 .23382 m
.44286 .23757 L
s
.48095 .23382 m
.48095 .23757 L
s
.51905 .23382 m
.51905 .23757 L
s
.55714 .23382 m
.55714 .23757 L
s
.63333 .23382 m
.63333 .23757 L
s
.67143 .23382 m
.67143 .23757 L
s
.70952 .23382 m
.70952 .23757 L
s
.74762 .23382 m
.74762 .23757 L
s
.82381 .23382 m
.82381 .23757 L
s
.8619 .23382 m
.8619 .23757 L
s
.9 .23382 m
.9 .23757 L
s
.9381 .23382 m
.9381 .23757 L
s
.25 Mabswid
0 .23382 m
1 .23382 L
s
.40476 .10236 m
.41101 .10236 L
s
[(-1)] .39226 .10236 1 0 Mshowa
.40476 .36527 m
.41101 .36527 L
s
[(1)] .39226 .36527 1 0 Mshowa
.40476 .49672 m
.41101 .49672 L
s
[(2)] .39226 .49672 1 0 Mshowa
.125 Mabswid
.40476 .12865 m
.40851 .12865 L
s
.40476 .15495 m
.40851 .15495 L
s
.40476 .18124 m
.40851 .18124 L
s
.40476 .20753 m
.40851 .20753 L
s
.40476 .26011 m
.40851 .26011 L
s
.40476 .2864 m
.40851 .2864 L
s
.40476 .31269 m
.40851 .31269 L
s
.40476 .33898 m
.40851 .33898 L
s
.40476 .39156 m
.40851 .39156 L
s
.40476 .41785 m
.40851 .41785 L
s
.40476 .44414 m
.40851 .44414 L
s
.40476 .47043 m
.40851 .47043 L
s
.40476 .07607 m
.40851 .07607 L
s
.40476 .04978 m
.40851 .04978 L
s
.40476 .02349 m
.40851 .02349 L
s
.40476 .52301 m
.40851 .52301 L
s
.40476 .54931 m
.40851 .54931 L
s
.40476 .5756 m
.40851 .5756 L
s
.40476 .60189 m
.40851 .60189 L
s
.25 Mabswid
.40476 0 m
.40476 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .56532 L
.10458 .52787 L
.14415 .49597 L
.18221 .46791 L
.22272 .44049 L
.26171 .4162 L
.30316 .39238 L
.34309 .37115 L
.3815 .35213 L
.42237 .33325 L
.46172 .31625 L
.49955 .30087 L
.53984 .28545 L
.57861 .27143 L
.61984 .25734 L
.64043 .25058 L
.65954 .24447 L
.67803 .2387 L
.68335 .23706 L
.68596 .23626 L
.68711 .23591 L
.68837 .23553 L
.6895 .23519 L
.69055 .04728 L
.69167 .04715 L
.69287 .04701 L
.69774 .04646 L
.7391 .04177 L
.77894 .03724 L
.81727 .03287 L
.85806 .02821 L
.89732 .02373 L
.93507 .01941 L
.97528 .01482 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo001dool0
0`00ooooo`2Yool007Coo`03003ooooo0:Woo`00M?oo00<00?ooool0ZOoo001dool00`00ooooo`2Y
ool007Coo`03003ooooo09Woo`L000Woo`00M?oo00<00?ooool0S_oo2`004?oo001dool20027ool8
000Kool007Coo`03003ooooo083oo`H002?oo`00M?oo00<00?ooool0N?oo2000:Ooo001dool00`00
ooooo`1]ool;000aool007Coo`03003ooooo06Goo`P003coo`00M?oo00<00?ooool0Gooo1P00A?oo
001dool00`00ooooo`1Fool9001:ool007Coo`03003ooooo04koo`P005?oo`00M?oo0P00Cooo00<0
0?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003o
oooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00oooo
o`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0
F?oo001dool2001?ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo
00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03
003ooooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo001/ool40004
ool00`00ooooo`1>ool00`00ooooo`1Hool006koo`03003ooooo00?oo`03003ooooo04koo`03003o
oooo05Soo`00K_oo00<00?ooool00ooo0`00C_oo00<00?ooool0F?oo001Vool40004ool00`00oooo
o`03ool00`00ooooo`1>ool00`00ooooo`1Hool006koo`03003ooooo00?oo`03003ooooo04koo`03
003ooooo05Soo`00K?oo0`001Ooo00<00?ooool0C_oo00<00?ooool0F?oo001^ool00`00ooooo`03
ool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo
00<00?ooool0C_oo00<00?ooool0F?oo001dool2001?ool00`00ooooo`1Hool007Coo`03003ooooo
04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00ooooo`1>
ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo
00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03
003ooooo05Soo`00M?oo0P00Cooo00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1H
ool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo
001dool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00
M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool2001?ool00`00ooooo`1Hool007Coo`03003o
oooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00oooo
o`1>ool00`00ooooo`1Hool000;oo`8000Coo`03003ooooo00?oo`8002Ooo`8000Coo`03003ooooo
00;oo`<002ooo`03003ooooo02_oo`@000?oo`03003ooooo00;oo`@001?oo`03003ooooo013oo`@0
00?oo`03003ooooo00;oo`@002Koo`@000?oo`03003ooooo00?oo`8000?oo`0000Goo`00ooooo`00
00Soo`04003ooooo000Uool01000ooooo`002ooo00<00?ooool0;?oo00<00?ooool0;Ooo00<00?oo
ool02Ooo00<00?ooool04_oo00<00?ooool04_oo00<00?ooool01ooo00<00?ooool0:Ooo00<00?oo
ool01ooo00@00?ooool000;oo`0000Goo`00ooooo`0000Soo`04003ooooo000Uool01000ooooo`00
2ooo00<00?ooool0;?oo00<00?ooool0;Ooo00<00?ooool02Ooo00<00?ooool04_oo00<00?ooool0
4_oo00<00?ooool02?oo00<00?ooool0:?oo00<00?ooool02_oo0@000Ooo0Ooo00001Ooo003ooooo
00002Ooo0P009_oo00@00?ooool000Woo`<002koo`8002koo`03003ooooo00Woo`03003ooooo01;o
o`03003ooooo01;oo`03003ooooo00Woo`03003ooooo02Ooo`03003ooooo00Soo`8000?oo`0000Go
o`00ooooo`0000Soo`04003ooooo000Uool01000ooooo`002?oo00@00?ooool002koo`03003ooooo
02goo`03003ooooo00Woo`03003ooooo01;oo`03003ooooo01;oo`03003ooooo00[oo`03003ooooo
02Koo`03003ooooo00[oo`40007oo`7oo`0000Goo`00ooooo`0000Soo`04003ooooo000Uool01000
ooooo`002?oo00@00?ooool002koo`03003ooooo02_oo`<000Woo`<001Coo`03003ooooo013oo`<0
00Woo`04003ooooo000Vool30009ool01000ooooo`000_oo0002ool2000:ool2000Wool2000:ool2
000_ool00`00ooooo`0]ool00`00ooooo`09ool00`00ooooo`0Bool00`00ooooo`0Bool00`00oooo
o`08ool2000Yool00`00ooooo`08ool20003ool007Coo`03003ooooo04koo`03003ooooo05Soo`00
M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1Hool007Co
o`03003ooooo04koo`03003ooooo05Soo`000_ooo`0070000ooo0009ool00`00ooooo`07ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00
ooooo`07ool50006ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool000Woo`03003o
oooo03;oo`03003ooooo03?oo`03003ooooo03?oo`03003ooooo017oo`80023oo`03003ooooo03?o
o`03003ooooo00Ooo`00M?oo00<00?ooool0A?oo0`00H_oo001dool00`00ooooo`12ool2001Uool0
07Coo`03003ooooo03goo`D006Ooo`00M?oo00<00?ooool0>Ooo1000K?oo001dool2000gool3001`
ool007Coo`03003ooooo03?oo`<007?oo`00M?oo00<00?ooool0<?oo0`00M_oo001dool00`00oooo
o`0]ool3001iool007Coo`03003ooooo02[oo`<007coo`00M?oo00<00?ooool0:?oo0P00Oooo001d
ool00`00ooooo`0Uool30021ool007Coo`03003ooooo02;oo`<008Coo`00M?oo0P008Ooo0P00Qooo
001dool00`00ooooo`0Nool20029ool007Coo`03003ooooo01coo`8008_oo`00M?oo00<00?ooool0
6_oo0P00SOoo001dool00`00ooooo`0Gool3002?ool007Coo`03003ooooo01Coo`<009;oo`00M?oo
00<00?ooool04_oo0P00UOoo001dool2000@ool3002Gool007Coo`03003ooooo00coo`<009[oo`00
M?oo00<00?ooool02_oo0P00WOoo001dool00`00ooooo`08ool2002Oool007Coo`03003ooooo00Ko
o`800:7oo`00M?oo00<00?ooool01?oo0P00Xooo001dool01000oooooooo0`00YOoo001dool4002X
ool007Coo`800:[oo`00L_oo0`00Zooo001`ool20002ool00`00ooooo`2Yool006koo`8000Coo`03
003ooooo0:Woo`00K?oo0P001_oo00<00?ooool0ZOoo001Zool60004ool00`00ooooo`2Yool006So
o`8000Coo`03003ooooo00?oo`03003ooooo0:Woo`00I_oo0P001_oo00<00?ooool00ooo0`00ZOoo
001Tool20008ool00`00ooooo`03ool00`00ooooo`2Yool006;oo`8000[oo`03003ooooo00?oo`03
003ooooo0:Woo`00H?oo0P002_oo0`001Ooo00<00?ooool0ZOoo001Nool2000>ool00`00ooooo`03
ool00`00ooooo`2Yool005coo`8001Koo`03003ooooo0:Woo`00F_oo0P006?oo00<00?ooool0ZOoo
001Hool2000Jool2002Zool005Ooo`03003ooooo01[oo`03003ooooo0:Woo`00EOoo0P007Ooo00<0
0?ooool0ZOoo001Cool2000Oool00`00ooooo`2Yool0057oo`80027oo`03003ooooo0:Woo`00Cooo
0P008ooo00<00?ooool0ZOoo001=ool2000Uool00`00ooooo`2Yool004coo`03003ooooo02Goo`03
003ooooo0:Woo`00B_oo0P00:?oo0P00Z_oo0019ool00`00ooooo`0Xool00`00ooooo`2Yool004Oo
o`8002_oo`03003ooooo0:Woo`00AOoo0P00;Ooo00<00?ooool0ZOoo0014ool00`00ooooo`0]ool0
0`00ooooo`2Yool004;oo`80033oo`03003ooooo0:Woo`00@Ooo00<00?ooool0<?oo00<00?ooool0
ZOoo000oool2000cool2002Zool003koo`03003ooooo03?oo`03003ooooo0:Woo`00??oo0P00=_oo
00<00?ooool0ZOoo000kool00`00ooooo`0fool00`00ooooo`2Yool003Woo`8003Woo`03003ooooo
0:Woo`00>?oo00<00?ooool0>Ooo00<00?ooool0ZOoo000fool2000lool00`00ooooo`2Yool003Go
o`03003ooooo03coo`03003ooooo0:Woo`00<ooo0P00?ooo0P00Z_oo000bool00`00ooooo`0oool0
0`00ooooo`2Yool0037oo`03003ooooo043oo`03003ooooo0:Woo`00;ooo0P00@ooo00<00?ooool0
ZOoo000^ool00`00ooooo`13ool00`00ooooo`2Yool002goo`03003ooooo03coo`@000Coo`03003o
oooo0:Woo`00:ooo0P00?ooo00<00?ooool01Ooo00<00?ooool0ZOoo000Zool00`00ooooo`10ool0
0`00ooooo`04ool3002Yool002Woo`03003ooooo04;oo`03003ooooo00?oo`03003ooooo0:Woo`00
9ooo0P00A_oo00<00?ooool00_oo00<00?ooool0ZOoo000Vool00`00ooooo`13ool01000ooooo`00
1?oo00<00?ooool0ZOoo000Uool00`00ooooo`15ool20005ool00`00ooooo`2Yool002Coo`03003o
oooo04goo`03003ooooo0:Woo`008ooo00<00?ooool0C_oo00<00?ooool0ZOoo000Qool2001Aool2
002Zool0023oo`03003ooooo057oo`03003ooooo0:Woo`007ooo00<00?ooool0D_oo00<00?ooool0
ZOoo000Nool00`00ooooo`1Cool00`00ooooo`2Yool001coo`8005Koo`03003ooooo0:Woo`006ooo
00<00?ooool0E_oo00<00?ooool0ZOoo000Jool00`00ooooo`1Gool00`00ooooo`2Yool001Woo`03
003ooooo05Soo`03003ooooo0:Woo`006?oo00<00?ooool0FOoo0P00Z_oo000Fool2001Lool00`00
ooooo`2Yool001Goo`03003ooooo05coo`03003ooooo0:Woo`005?oo00<00?ooool0GOoo00<00?oo
ool0ZOoo000Cool00`00ooooo`1Nool00`00ooooo`2Yool001;oo`03003ooooo05ooo`03003ooooo
0:Woo`004Ooo00<00?ooool0H?oo00<00?ooool0ZOoo000@ool00`00ooooo`1Qool2002Zool000oo
o`03003ooooo06;oo`03003ooooo0:Woo`003_oo00<00?ooool0Hooo00<00?ooool0ZOoo000>ool0
0`00ooooo`1Sool00`00ooooo`2Yool000goo`03003ooooo06Coo`03003ooooo0:Woo`003?oo00<0
0?ooool0IOoo00<00?ooool0ZOoo000;ool00`00ooooo`1Vool00`00ooooo`2Yool000[oo`03003o
oooo06Ooo`03003ooooo0:Woo`002Ooo00<00?ooool0J?oo0P00Z_oo001dool00`00ooooo`2Yool0
07Coo`03003ooooo0:Woo`00M?oo00<00?ooool0ZOoo001dool00`00ooooo`2Yool00?ooob7oo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.78327, -1.81072, \
0.00185875, 0.0269334}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.74, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] \(\(0.8760984351000806`\)\(\[InvisibleSpace]\)\) + 
          1.0274376357597258`*^-7\ \[ImaginaryI]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol[k, 10]], {k, 0.01, 1}, 
      PlotStyle \[Rule] {RGBColor[1, 0, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 -0.877637 1.44479 [
[.21429 .55465 -9 -9 ]
[.21429 .55465 9 0 ]
[.40476 .55465 -9 -9 ]
[.40476 .55465 9 0 ]
[.59524 .55465 -9 -9 ]
[.59524 .55465 9 0 ]
[.78571 .55465 -9 -9 ]
[.78571 .55465 9 0 ]
[.97619 .55465 -3 -9 ]
[.97619 .55465 3 0 ]
[.01131 .13372 -18 -4.5 ]
[.01131 .13372 0 4.5 ]
[.01131 .2782 -18 -4.5 ]
[.01131 .2782 0 4.5 ]
[.01131 .42267 -18 -4.5 ]
[.01131 .42267 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .56715 m
.21429 .5734 L
s
[(0.2)] .21429 .55465 0 1 Mshowa
.40476 .56715 m
.40476 .5734 L
s
[(0.4)] .40476 .55465 0 1 Mshowa
.59524 .56715 m
.59524 .5734 L
s
[(0.6)] .59524 .55465 0 1 Mshowa
.78571 .56715 m
.78571 .5734 L
s
[(0.8)] .78571 .55465 0 1 Mshowa
.97619 .56715 m
.97619 .5734 L
s
[(1)] .97619 .55465 0 1 Mshowa
.125 Mabswid
.07143 .56715 m
.07143 .5709 L
s
.11905 .56715 m
.11905 .5709 L
s
.16667 .56715 m
.16667 .5709 L
s
.2619 .56715 m
.2619 .5709 L
s
.30952 .56715 m
.30952 .5709 L
s
.35714 .56715 m
.35714 .5709 L
s
.45238 .56715 m
.45238 .5709 L
s
.5 .56715 m
.5 .5709 L
s
.54762 .56715 m
.54762 .5709 L
s
.64286 .56715 m
.64286 .5709 L
s
.69048 .56715 m
.69048 .5709 L
s
.7381 .56715 m
.7381 .5709 L
s
.83333 .56715 m
.83333 .5709 L
s
.88095 .56715 m
.88095 .5709 L
s
.92857 .56715 m
.92857 .5709 L
s
.25 Mabswid
0 .56715 m
1 .56715 L
s
.02381 .13372 m
.03006 .13372 L
s
[(0.7)] .01131 .13372 1 0 Mshowa
.02381 .2782 m
.03006 .2782 L
s
[(0.8)] .01131 .2782 1 0 Mshowa
.02381 .42267 m
.03006 .42267 L
s
[(0.9)] .01131 .42267 1 0 Mshowa
.125 Mabswid
.02381 .16261 m
.02756 .16261 L
s
.02381 .19151 m
.02756 .19151 L
s
.02381 .2204 m
.02756 .2204 L
s
.02381 .2493 m
.02756 .2493 L
s
.02381 .30709 m
.02756 .30709 L
s
.02381 .33599 m
.02756 .33599 L
s
.02381 .36488 m
.02756 .36488 L
s
.02381 .39378 m
.02756 .39378 L
s
.02381 .45157 m
.02756 .45157 L
s
.02381 .48047 m
.02756 .48047 L
s
.02381 .50936 m
.02756 .50936 L
s
.02381 .53826 m
.02756 .53826 L
s
.02381 .10482 m
.02756 .10482 L
s
.02381 .07592 m
.02756 .07592 L
s
.02381 .04703 m
.02756 .04703 L
s
.02381 .01813 m
.02756 .01813 L
s
.02381 .59605 m
.02756 .59605 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03333 .01472 m
.03793 .01493 L
.04222 .0152 L
.04692 .01559 L
.05195 .01609 L
.06223 .01743 L
.07158 .019 L
.09303 .02386 L
.1133 .03 L
.15133 .04537 L
.19178 .06673 L
.23074 .09158 L
.2682 .11887 L
.30808 .15107 L
.34647 .18466 L
.38728 .22275 L
.40766 .24254 L
.41759 .25239 L
.41987 .25458 L
.42228 .25692 L
.42327 .25769 L
.42433 .25833 L
.4255 .25885 L
.42659 .25923 L
.43112 .2605 L
.43601 .26186 L
.44489 .26441 L
.4644 .27021 L
.50535 .28348 L
.5448 .29778 L
.58274 .31305 L
.62312 .33106 L
.66199 .3503 L
.69936 .37073 L
.73916 .39479 L
.77746 .42045 L
.81819 .45073 L
.85742 .48313 L
.89515 .51757 L
.9353 .55801 L
.97396 .60074 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003oooP000Kh0?ooo`?o
o`005Ooo0P001ooo1oP0nooo000Eool00`00ooooo`0=ool5n03fool001Goo`03003ooooo01;oo`;h
0?Coo`005Ooo00<00?ooool05?oo0oP0lOoo000Eool00`00ooooo`0Gool3n03^ool001Goo`03003o
oooo01[oo`;h0>coo`005Ooo00<00?ooool07?oo0oP0jOoo000Eool00`00ooooo`0Oool2n03Wool0
01Goo`8002;oo`;h0>Goo`005Ooo00<00?ooool08ooo0_P0hooo000Eool00`00ooooo`0Uool2n03Q
ool001Goo`03003ooooo02Ooo`;h0=ooo`005Ooo00<00?ooool0:Ooo0_P0gOoo000Eool00`00oooo
o`0[ool00oP0ooooo`3Jool001Goo`03003ooooo02coo`;h0=[oo`005Ooo00<00?ooool0;_oo00?h
0?ooool0eooo000Eool2000`ool00oP0ooooo`3Fool001Goo`03003ooooo033oo`;h0=Koo`005Ooo
00<00?ooool0<_oo00?h0?ooool0dooo000Eool00`00ooooo`0cool2n03Cool001Goo`03003ooooo
03Goo`03n03ooooo0=3oo`005Ooo00<00?ooool0=_oo0_P0d?oo000Eool00`00ooooo`0hool00oP0
ooooo`3=ool001Goo`8003[oo`03n03ooooo0<coo`005Ooo00<00?ooool0>_oo0_P0c?oo000Eool0
0`00ooooo`0lool00oP0ooooo`39ool001Goo`03003ooooo03goo`;h0<Woo`005Ooo00<00?ooool0
?ooo00?h0?ooool0a_oo000Eool00`00ooooo`10ool00oP0ooooo`35ool000;oo`8000Coo`03003o
oooo00?oo`03003ooooo00Coo`03003ooooo047oo`;h0<Goo`0000Goo`00ooooo`0000Woo`03003o
oooo00Coo`03003ooooo04?oo`03n03ooooo0<;oo`0000Goo`00ooooo`0000[oo`03003ooooo00?o
o`<004Coo`03n03ooooo0<7oo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo04Go
o`03n03ooooo0<3oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003ooooo04Koo`03n03o
oooo0;ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`17ool2n02oool000;o
o`8000Woo`@000Coo`03003ooooo04Woo`03n03ooooo0;coo`005Ooo00<00?ooool0B_oo00?h0?oo
ool0^ooo000Eool00`00ooooo`1;ool00oP0ooooo`2jool001Goo`03003ooooo04coo`;h0;[oo`00
5Ooo0P00Cooo00?h0?ooool0]ooo000Eool00`00ooooo`1?ool00oP0ooooo`2fool001Goo`03003o
oooo053oo`03n03ooooo0;Goo`005Ooo00<00?ooool0DOoo00?h0?ooool0]?oo000Eool00`00oooo
o`1Bool2n02dool001Goo`03003ooooo05Coo`03n03ooooo0;7oo`005Ooo00<00?ooool0EOoo00?h
0?ooool0/?oo000Eool00`00ooooo`1Fool00oP0ooooo`2_ool001Goo`8005Soo`03n03ooooo0:ko
o`005Ooo00<00?ooool0F?oo00?h0?ooool0[Ooo000Eool00`00ooooo`1Iool00oP0ooooo`2/ool0
01Goo`03003ooooo05[oo`03n03ooooo0:_oo`005Ooo00<00?ooool0Fooo00?h0?ooool0Z_oo000E
ool00`00ooooo`1Lool00oP0ooooo`2Yool001Goo`03003ooooo05goo`03n03ooooo0:Soo`005Ooo
00<00?ooool0G_oo00?h0?ooool0Yooo000Eool2001Pool00oP0ooooo`2Vool001Goo`03003ooooo
063oo`03n03ooooo0:Goo`005Ooo00<00?ooool0HOoo00?h0?ooool0Y?oo000Eool00`00ooooo`1R
ool00oP0ooooo`2Sool001Goo`03003ooooo06?oo`03n03ooooo0:;oo`005Ooo00<00?ooool0I?oo
00?h0?ooool0XOoo000Eool00`00ooooo`1Uool00oP0ooooo`2Pool001Goo`03003ooooo06Koo`03
n03ooooo09ooo`005Ooo0P00J?oo00?h0?ooool0W_oo000Eool00`00ooooo`1Xool2n02Nool001Go
o`03003ooooo06Woo`;h09goo`005Ooo00<00?ooool0Jooo0oP0V_oo000Eool00`00ooooo`1^ool4
n02Fool001Goo`03003ooooo07;oo`Gh097oo`000_oo0P001?oo00<00?ooool00ooo0P001Ooo00<0
0?ooool0Mooo0oP0S_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo07[oo`;h
08coo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3001lool3n029ool00005ool00?ooool0
0009ool20005ool00`00ooooo`1oool3n026ool00005ool00?ooool00008ool01000ooooo`001?oo
00<00?ooool0P_oo0oP0Pooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo08Go
o`;h087oo`000_oo0P002_oo0P001Ooo00<00?ooool0Qooo0oP0O_oo000Eool00`00ooooo`2:ool3
n01kool001Goo`03003ooooo08goo`;h07Woo`005Ooo00<00?ooool0Sooo0oP0M_oo000Eool2002C
ool2n01dool001Goo`03003ooooo09Coo`?h077oo`005Ooo00<00?ooool0Uooo0_P0Kooo000Eool0
0`00ooooo`2Iool2n01]ool001Goo`03003ooooo09_oo`;h06_oo`005Ooo00<00?ooool0WOoo0_P0
JOoo000Eool00`00ooooo`2Oool3n01Vool001Goo`800:?oo`;h06Coo`005Ooo00<00?ooool0Y?oo
0_P0H_oo000Eool00`00ooooo`2Vool2n01Pool001Goo`03003ooooo0:Soo`;h05koo`005Ooo00<0
0?ooool0Z_oo0_P0G?oo000Eool00`00ooooo`2/ool2n01Jool001Goo`03003ooooo0:koo`;h05So
o`005Ooo00<00?ooool0/?oo00?h0?ooool0EOoo000Eool2002bool2n01Eool001Goo`03003ooooo
0;?oo`;h05?oo`005Ooo00<00?ooool0]Ooo00?h0?ooool0D?oo000Eool00`00ooooo`2fool2n01@
ool001Goo`03003ooooo0;Soo`;h04koo`005Ooo00<00?ooool0^_oo0_P0C?oo000Eool00`00oooo
o`2lool2n01:ool001Goo`03003ooooo0;koo`;h04Soo`005Ooo0P00`Ooo00?h0?ooool0AOoo000E
ool00`00ooooo`31ool2n015ool001Goo`03003ooooo0<?oo`03n03ooooo04;oo`005Ooo00<00?oo
ool0a?oo00?h0?ooool0@Ooo000Eool00`00ooooo`35ool2n011ool001Goo`03003ooooo0<Ooo`03
n03ooooo03koo`000_oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0b?oo0_P0?_oo0000
1Ooo003ooooo00002ooo00<00?ooool00_oo00<00?ooool0b_oo00?h0?ooool0>ooo00001Ooo003o
oooo00002ooo00<00?ooool00_oo0`00booo00?h0?ooool0>_oo00001Ooo003ooooo00002Ooo0`00
1?oo00<00?ooool0c?oo0_P0>_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo
0<koo`03n03ooooo03Ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3?ool0
0oP0ooooo`0fool000;oo`8000[oo`8000Goo`03003ooooo0=3oo`03n03ooooo03Goo`005Ooo00<0
0?ooool0dOoo00?h0?ooool0=?oo000Eool00`00ooooo`3Bool2n00dool001Goo`03003ooooo0=Co
o`03n03ooooo037oo`005Ooo0P00e_oo00?h0?ooool0<?oo000Eool00`00ooooo`3Fool2n00`ool0
01Goo`03003ooooo0=Soo`03n03ooooo02goo`005Ooo00<00?ooool0fOoo00?h0?ooool0;?oo000E
ool00`00ooooo`3Jool2n00/ool001Goo`03003ooooo0=coo`03n03ooooo02Woo`005Ooo00<00?oo
ool0gOoo00?h0?ooool0:?oo000Eool00`00ooooo`3Nool2n00Xool001Goo`800>7oo`03n03ooooo
02Goo`005Ooo00<00?ooool0hOoo00?h0?ooool09?oo000Eool00`00ooooo`3Rool00oP0ooooo`0S
ool001Goo`03003ooooo0>?oo`03n03ooooo02;oo`005Ooo00<00?ooool0i?oo00?h0?ooool08Ooo
000Eool00`00ooooo`3Uool00oP0ooooo`0Pool001Goo`03003ooooo0>Koo`03n03ooooo01ooo`00
5Ooo00<00?ooool0iooo00?h0?ooool07_oo000Eool2003Yool00oP0ooooo`0Mool001Goo`03003o
oooo0>Woo`03n03ooooo01coo`005Ooo00<00?ooool0j_oo00?h0?ooool06ooo000Eool00`00oooo
o`3[ool00oP0ooooo`0Jool001Goo`03003ooooo0>coo`03n03ooooo01Woo`005Ooo00<00?ooool0
:_oo0P001?oo00<00?ooool00_oo10009Ooo0P001?oo00<00?ooool01?oo00<00?ooool08ooo0P00
1?oo00<00?ooool00ooo0P009_oo0P001?oo00<00?ooool00ooo0P006_oo00?h0?ooool03_oo1000
1_oo000Eool00`00ooooo`0Yool01000ooooo`002?oo00<00?ooool09Ooo00@00?ooool000[oo`03
003ooooo02;oo`04003ooooo0008ool01000ooooo`009?oo00@00?ooool000Soo`04003ooooo000J
ool00oP0ooooo`0?ool00`00ooooo`05ool001Goo`8002[oo`04003ooooo0009ool00`00ooooo`0T
ool01000ooooo`001ooo1@008ooo00@00?ooool000Soo`04003ooooo000Tool01000ooooo`002?oo
00@00?ooool001_oo`03n03ooooo00koo`03003ooooo00Goo`005Ooo00<00?ooool0:Ooo00@00?oo
ool000[oo`03003ooooo02?oo`04003ooooo0007ool01000ooooo`009?oo00@00?ooool000Soo`<0
02Goo`04003ooooo0009ool2000Mool00oP0ooooo`0=ool00`00ooooo`05ool001Goo`03003ooooo
02Woo`04003ooooo000;ool00`00ooooo`0Rool01000ooooo`002?oo00<00?oo00009?oo00@00?oo
ool000Soo`03003ooooo02Goo`04003ooooo0008ool01000ooooo`007Ooo00?h0?ooool03?oo00<0
0?ooool01Ooo000Eool00`00ooooo`0Yool01000ooooo`002?oo00@00?ooool002Coo`04003ooooo
0009ool2000Tool01000ooooo`002?oo00<00?ooool09Ooo00@00?ooool000Soo`04003ooooo000N
ool00oP0ooooo`09ool30007ool001Goo`03003ooooo02[oo`8000[oo`8002Koo`8000_oo`03003o
oooo02?oo`8000[oo`<002Goo`8000[oo`80023oo`03n03ooooo00[oo`03003ooooo00Goo`005Ooo
00<00?ooool0m?oo00?h0?ooool04Ooo000Eool00`00ooooo`3eool00oP0ooooo`0@ool001Goo`03
003ooooo0?Koo`03n03ooooo00ooo`003_ooo`000P0000?h000000003P00000Eool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`02ool00oP0ooooo`05
ool00`00ooooo`05ool001Goo`03003ooooo037oo`03003ooooo037oo`03003ooooo033oo`03003o
oooo037oo`03003ooooo02[oo`03n03ooooo00Coo`03003ooooo00Goo`005Ooo00<00?ooool0n_oo
00?h0?ooool02ooo000Eool00`00ooooo`3jool00oP0ooooo`0;ool001Goo`03003ooooo0?_oo`03
n03ooooo00[oo`005Ooo00<00?ooool0o?oo00?h0?ooool02Ooo000Eool00`00ooooo`3mool00oP0
ooooo`08ool001Goo`800?ooo`03n03ooooo00Ooo`005Ooo00<00?ooool0oooo00?h0?ooool01_oo
000Eool00`00ooooo`3oool1ool00oP0ooooo`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0825959, 0.596272, \
0.00385924, 0.00254394}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%1066, %1067]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.0147151 0.392403 [
[.15079 .00222 -9 -9 ]
[.15079 .00222 9 0 ]
[.27778 .00222 -9 -9 ]
[.27778 .00222 9 0 ]
[.40476 .00222 -9 -9 ]
[.40476 .00222 9 0 ]
[.53175 .00222 -9 -9 ]
[.53175 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.78571 .00222 -9 -9 ]
[.78571 .00222 9 0 ]
[.9127 .00222 -9 -9 ]
[.9127 .00222 9 0 ]
[.01131 .0932 -18 -4.5 ]
[.01131 .0932 0 4.5 ]
[.01131 .17168 -18 -4.5 ]
[.01131 .17168 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .32864 -18 -4.5 ]
[.01131 .32864 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .56408 -18 -4.5 ]
[.01131 .56408 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .01472 m
.15079 .02097 L
s
[(0.2)] .15079 .00222 0 1 Mshowa
.27778 .01472 m
.27778 .02097 L
s
[(0.4)] .27778 .00222 0 1 Mshowa
.40476 .01472 m
.40476 .02097 L
s
[(0.6)] .40476 .00222 0 1 Mshowa
.53175 .01472 m
.53175 .02097 L
s
[(0.8)] .53175 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(1)] .65873 .00222 0 1 Mshowa
.78571 .01472 m
.78571 .02097 L
s
[(1.2)] .78571 .00222 0 1 Mshowa
.9127 .01472 m
.9127 .02097 L
s
[(1.4)] .9127 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.18254 .01472 m
.18254 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.34127 .01472 m
.34127 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.5 .01472 m
.5 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.81746 .01472 m
.81746 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.97619 .01472 m
.97619 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .0932 m
.03006 .0932 L
s
[(0.2)] .01131 .0932 1 0 Mshowa
.02381 .17168 m
.03006 .17168 L
s
[(0.4)] .01131 .17168 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.6)] .01131 .25016 1 0 Mshowa
.02381 .32864 m
.03006 .32864 L
s
[(0.8)] .01131 .32864 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(1)] .01131 .40712 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(1.2)] .01131 .4856 1 0 Mshowa
.02381 .56408 m
.03006 .56408 L
s
[(1.4)] .01131 .56408 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .11282 m
.02756 .11282 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .21092 m
.02756 .21092 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .50522 m
.02756 .50522 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .5837 m
.02756 .5837 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03016 .25708 m
.03323 .25713 L
.03608 .25721 L
.03922 .25731 L
.04257 .25745 L
.04942 .25781 L
.05566 .25824 L
.06995 .25956 L
.08347 .26123 L
.10882 .2654 L
.13579 .2712 L
.16176 .27795 L
.18674 .28537 L
.21332 .29411 L
.23891 .30323 L
.26612 .31358 L
.27971 .31895 L
.28633 .32163 L
.28785 .32222 L
.28946 .32286 L
.29012 .32307 L
.29083 .32324 L
.2916 .32338 L
.29233 .32349 L
.29535 .32383 L
.29861 .3242 L
.30453 .32489 L
.31754 .32647 L
.34484 .33007 L
.37113 .33396 L
.39643 .3381 L
.42335 .343 L
.44926 .34822 L
.47418 .35377 L
.50071 .36031 L
.52625 .36727 L
.5534 .3755 L
.57955 .3843 L
.6047 .39365 L
.63147 .40463 L
.65724 .41624 L
.65873 .41694 L
s
0 g
.03016 .01864 m
.06854 .04236 L
.11039 .06823 L
.1497 .09252 L
.1875 .11588 L
.22774 .14075 L
.26647 .16469 L
.30764 .19013 L
.34731 .21465 L
.38546 .23823 L
.42606 .26332 L
.46515 .28748 L
.50273 .3107 L
.54275 .33544 L
.58126 .35924 L
.62221 .38455 L
.66166 .40893 L
.69959 .43237 L
.73997 .45733 L
.77884 .48135 L
.82014 .50688 L
.85994 .53147 L
.89823 .55514 L
.93896 .58031 L
.97619 .60332 L
s
.03016 .30939 m
.03133 .30943 L
.03239 .30947 L
.03362 .30932 L
.03477 .30933 L
.03585 .30962 L
.03684 .30967 L
.03908 .30981 L
.04372 .31016 L
.04884 .31064 L
.0584 .3118 L
.06854 .31335 L
.11039 .32202 L
.15041 .33176 L
.18892 .34119 L
.22987 .35077 L
.26932 .35939 L
.30725 .36708 L
.34763 .3746 L
.3865 .38121 L
.4278 .38753 L
.4676 .39292 L
.5059 .3974 L
.54663 .40134 L
.56691 .40295 L
.58585 .40422 L
.60522 .40528 L
.62357 .40606 L
.63325 .40638 L
.64379 .40665 L
.64843 .40675 L
.65336 .40684 L
.65767 .4069 L
.66222 .40695 L
.66454 .40698 L
.66707 .407 L
.66941 .40701 L
.67153 .40702 L
.67273 .40703 L
.67404 .40703 L
.67476 .40703 L
.67542 .40704 L
.67672 .40704 L
.67784 .40704 L
.67907 .40704 L
.68036 .40704 L
.68156 .40704 L
.68268 .40704 L
.68391 .40703 L
.68613 .40703 L
Mistroke
.6883 .40702 L
.69031 .407 L
.69487 .40697 L
.69972 .40692 L
.70996 .40678 L
.72103 .40657 L
.74107 .40608 L
.78186 .40477 L
.82113 .40335 L
.8589 .402 L
.89911 .40067 L
.93781 .39952 L
.97619 .39851 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool0033oo`8000Coo`03003ooooo00;oo`@001Coo`8000Coo`03003ooooo00Coo`03003ooooo01?o
o`8000Coo`03003ooooo00?oo`8001Coo`8000Coo`03003ooooo00?oo`8001[oo`@001Soo`@000?o
o`03003ooooo00;oo`@001?oo`@000?oo`03003ooooo00Coo`03003ooooo013oo`00;ooo00@00?oo
ool000Soo`03003ooooo01Coo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`002?oo00@0
0?ooool001;oo`04003ooooo0008ool01000ooooo`006ooo00<00?ooool06Ooo00<00?ooool01ooo
00<00?ooool05_oo00<00?ooool02Ooo00<00?ooool04?oo000_ool01000ooooo`002Ooo00<00?oo
ool04ooo00@00?ooool000Ooo`D001?oo`04003ooooo0008ool01000ooooo`004_oo00@00?ooool0
00Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`08ool00`00ooooo`0Eool00`00oooo
o`06ool5000Aool002ooo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`001ooo00@00?oo
ool001Coo`04003ooooo0008ool3000Cool01000ooooo`002Ooo0P007?oo00<00?ooool06Ooo00<0
0?ooool02Ooo00<00?ooool05?oo00<00?ooool01_oo00@00?ooool001;oo`00;ooo00@00?ooool0
00_oo`03003ooooo017oo`04003ooooo0008ool00`00ool0000Dool01000ooooo`002?oo00<00?oo
ool04ooo00@00?ooool000Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`0:ool00`00
ooooo`0Cool00`00ooooo`07ool00`00ool0000Bool002ooo`04003ooooo0008ool01000ooooo`00
4ooo00@00?ooool000Woo`8001Coo`04003ooooo0008ool00`00ooooo`0Cool01000ooooo`002?oo
00@00?ooool001Woo`<001Woo`<000Woo`04003ooooo000Cool3000:ool2000Bool001Goo`03003o
oooo01Soo`8000[oo`8001Goo`8000_oo`03003ooooo01?oo`8000[oo`<001?oo`8000[oo`8001co
o`03003ooooo01Woo`03003ooooo00Soo`8001Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo
00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`003_oo
o`004`00000Eool00`00ool00006ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool001Goo`03003ooooo008001goo`03003ooooo023oo`03003o
oooo023oo`03003ooooo01ooo`03003ooooo023oo`03003ooooo01ooo`03003ooooo023oo`03003o
oooo01Koo`005Ooo00<00?ooool00_oo00<00?ooool0oooo1?oo000Eool00`00ooooo`03ool00`00
ooooo`3oool3ool001Goo`8000Goo`800?ooo`?oo`005Ooo00<00?ooool01_oo00<00?ooool0oooo
000Eool00`00ooooo`07ool2003oool001Goo`03003ooooo00Woo`03003ooooo0?coo`005Ooo00<0
0?ooool02_oo0P00o?oo000Eool00`00ooooo`0<ool00`00ooooo`3iool001Goo`8000koo`800?Wo
o`005Ooo00<00?ooool03ooo0P00mooo000Eool00`00ooooo`0Aool00`00ooooo`3dool001Goo`03
003ooooo01;oo`800?Coo`005Ooo00<00?ooool05?oo00<00?ooool0lOoo000Eool2000Fool2003a
ool001Goo`03003ooooo01Ooo`800>ooo`005Ooo00<00?ooool06Ooo0P00kOoo000Eool00`00oooo
o`0Kool2003[ool000;oo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo01goo`800>Woo`00
00Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo01ooo`03003ooooo0>Koo`0000Goo`00
ooooo`0000Woo`03003ooooo00Coo`<0023oo`800>Koo`0000Goo`00ooooo`0000[oo`03003ooooo
00?oo`03003ooooo02;oo`03003ooooo0>?oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03
003ooooo02?oo`03003ooooo0>;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`0Tool2003Rool000;oo`8000[oo`8000Goo`03003ooooo02Koo`03003ooooo0=ooo`005Ooo0P00
:?oo0P00gooo000Eool00`00ooooo`0Yool00`00ooooo`3Lool001Goo`03003ooooo02[oo`800=co
o`005Ooo00<00?ooool0;?oo0P00f_oo000Eool00`00ooooo`0^ool2003Hool001Goo`80037oo`80
0=Koo`005Ooo00<00?ooool0<_oo0P00e?oo000Eool00`00ooooo`0dool00`00ooooo`3Aool001Go
o`03003ooooo03Goo`800=7oo`005Ooo00<00?ooool0=ooo00<00?ooool0c_oo000Eool00`00oooo
o`0hool2003>ool001Goo`8003_oo`800<coo`005Ooo00<00?ooool0??oo00<00?ooool0bOoo000E
ool00`00ooooo`0mool20039ool000;oo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003o
oooo03ooo`03003ooooo0<Koo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo043o
o`800<Koo`0000Goo`00ooooo`0000Ooo`D000Coo`<004;oo`03003ooooo0<?oo`0000Goo`00oooo
o`0000Ooo`04003ooooo0005ool00`00ooooo`13ool20033ool00005ool00?ooool00008ool00`00
ool00005ool00`00ooooo`15ool20031ool00005ool00?ooool00009ool20005ool00`00ooooo`17
ool00`00ooooo`2nool000;oo`8000_oo`03003ooooo00?oo`03003ooooo04Soo`800;koo`005Ooo
0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool2002kool001Goo`03003ooooo04goo`03
003ooooo0;Soo`005Ooo00<00?ooool0C_oo0P00^?oo000Eool00`00ooooo`1@ool2002fool001Go
o`03003ooooo05;oo`03003ooooo0;?oo`005Ooo0P00E?oo0P00/ooo000Eool00`00ooooo`1Eool0
0`00ooooo`2`ool001Goo`03003ooooo05Koo`800;3oo`005Ooo00<00?ooool0F?oo0P00[_oo000E
ool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`8005coo`800:_oo`005Ooo00<00?ooool0GOoo
0P00ZOoo000Eool00`00ooooo`1Oool00`00ooooo`2Vool000;oo`8000Coo`03003ooooo00?oo`80
00Goo`03003ooooo063oo`800:Koo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`1Rool00`00ooooo`2Sool00005ool00?ooool00008ool01000ooooo`001?oo0`00Hooo0P00Xooo
00001Ooo003ooooo00002?oo0`001Ooo00<00?ooool0IOoo0P00XOoo00001Ooo003ooooo00002?oo
00<00?ooool01Ooo00<00?oon0002?P0Gooo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<0
0?ooool01Ooo00<00?ooool02?oo2OP0Eooo0P00W_oo0002ool2000:ool30004ool00`00ooooo`0A
ool5n01Dool00`00ooooo`2Kool001Goo`03003ooooo01Koo`Ch057oo`8009_oo`005Ooo0P006ooo
0oP0D?oo0P00VOoo000Eool00`00ooooo`0Mool4n01>ool2002Gool001Goo`03003ooooo027oo`?h
04goo`8009Goo`005Ooo00<00?ooool09?oo1?P0Booo0P00Tooo000Eool00`00ooooo`0Xool3n01:
ool00`00ooooo`2@ool001Goo`8002coo`Ch04Ooo`80093oo`005Ooo00<00?ooool0;ooo0oP0A_oo
00<00?ooool0SOoo000Eool00`00ooooo`0bool2n015ool00`00ooooo`2<ool001Goo`03003ooooo
03Coo`;h04Coo`8008coo`005Ooo00<00?ooool0=_oo0oP0@ooo00<00?ooool0ROoo000Eool5000g
ool4n010ool20029ool001Goo`03003ooooo00;oo`T003;oo`?h03ooo`03003ooooo08Koo`005Ooo
00<00?ooool02ooo1000<Ooo0_P0?_oo0P00Q_oo000Eool00`00ooooo`0?ool4000_ool2n00nool2
0024ool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo01?oo`@002goo`Gh03_oo`80
08;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Gool6000/ool8n00eool2
0020ool00005ool00?ooool00008ool01000ooooo`001?oo0`007Ooo1000<?oo1oP0<?oo0P00O_oo
00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool08Ooo1000<ooo1oP0:ooo00<00?ooool0Nooo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo02Goo`@003Koo`Oh02Goo`8007_o
o`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Yool3000jool6n00Qool00`00
ooooo`1hool000;oo`8000[oo`8000Goo`03003ooooo02coo`@003coo`Ch01koo`8007Soo`005Ooo
0P00<Ooo1000??oo1?P07?oo0P00M_oo000Eool00`00ooooo`0dool4000lool5n00Iool00`00oooo
o`1cool001Goo`03003ooooo03Soo`H003_oo`Ch01Koo`8007?oo`005Ooo00<00?ooool0?_oo1@00
>_oo1?P05?oo00<00?ooool0L?oo000Eool00`00ooooo`13ool5000iool4n00Aool2001`ool001Go
o`8004Woo`D003Soo`?h013oo`03003ooooo06goo`005Ooo00<00?ooool0COoo1P00=Ooo1?P03Ooo
0P00KOoo000Eool00`00ooooo`1Cool5000dool3n00<ool2001[ool001Goo`03003ooooo05Soo`H0
037oo`;h00coo`03003ooooo06Soo`005Ooo00<00?ooool0G_oo1@00;_oo0_P02ooo0P00J?oo000E
ool00`00ooooo`1Sool6000Zool3n00:ool00`00ooooo`1Uool001Goo`8006[oo`P002Goo`Ch00Oo
o`8006Goo`005Ooo00<00?ooool0LOoo2P007ooo0oP01_oo0P00Hooo000Eool00`00ooooo`1kool;
000Gool2n006ool00`00ooooo`1Pool000goo`@000Coo`03003ooooo08Koo`P0017oo`;h00Goo`80
03kooa/000Ooo`003ooo00<00?ooool00ooo00<00?ooool0S_oo40000ooo0oP01?oo0P007?oo8000
8_oo000?ool00`00ooooo`03ool3002NoolX0012ool000ooo`03003ooooo00?oo`03003ooooo0:Ko
o`;h00?oo`8005_oo`003ooo00<00?ooool00ooo00<00?ooool0Z?oo0_P00ooo00<00?ooool0F?oo
000=ool30005ool00`00ooooo`2Zool01?P0oooooooo0P00F?oo000?ool00`00ooooo`03ool00`00
ooooo`2`ool2001Fool001Goo`800;?oo`03003ooooo05?oo`005Ooo00<00?ooool0/ooo0P00Dooo
000Eool00`00ooooo`2eool00`00ooooo`1@ool001Goo`03003ooooo0;Koo`80053oo`005Ooo00<0
0?ooool0^?oo00<00?ooool0COoo000Eool00`00ooooo`2iool2001=ool001Goo`800;coo`8004_o
o`005Ooo00<00?ooool0_Ooo00<00?ooool0B?oo000Eool00`00ooooo`2nool20018ool001Goo`03
003ooooo0<3oo`03003ooooo04Goo`005Ooo00<00?ooool0`Ooo0P00AOoo000Eool20034ool20013
ool001Goo`03003ooooo0<Goo`03003ooooo043oo`005Ooo00<00?ooool0a_oo0P00@?oo00000ooo
000000020003ool00`00ooooo`02ool40004ool00`00ooooo`38ool2000nool000?oo`03003ooooo
00Ooo`03003ooooo00Goo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?ooool02?oo00<0
0?ooool01?oo0`00booo0P00>ooo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3=
ool2000iool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo0<ooo`03003ooooo03Ko
o`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3@ool2000fool000?oo`03003ooooo
00Soo`8000Goo`03003ooooo0=;oo`8003Coo`005Ooo00<00?ooool0e?oo0P00<_oo000Eool2003G
ool00`00ooooo`0_ool001Goo`03003ooooo0=Ooo`8002ooo`005Ooo00<00?ooool0fOoo00<00?oo
ool0;?oo000Eool00`00ooooo`3Jool00`00ooooo`0[ool001Goo`03003ooooo0=_oo`8002_oo`00
5Ooo0P00g_oo00<00?ooool0:?oo000Eool00`00ooooo`3Nool2000Xool001Goo`03003ooooo0>3o
o`03003ooooo02Goo`005Ooo00<00?ooool0hOoo0P009Ooo000Eool00`00ooooo`3Sool2000Sool0
01Goo`800>Koo`80027oo`005Ooo00<00?ooool0iooo0P007ooo000Eool00`00ooooo`3Yool2000M
ool001Goo`03003ooooo0>_oo`03003ooooo01[oo`0000?oo`0000000P000ooo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool0k?oo0P006_oo0003ool00`00ooooo`09ool00`00ooooo`03ool0
0`00ooooo`3^ool00`00ooooo`0Gool000?oo`03003ooooo00Koo`D000Coo`<00>ooo`8001Ooo`00
0ooo00<00?ooool01_oo00@00?ooool000Goo`03003ooooo0?7oo`8001Goo`000ooo00<00?ooool0
1ooo00<00?oo00001Ooo00<00?ooool0looo00<00?ooool04_oo00001?oo000000002_oo0P001Ooo
00<00?ooool0m?oo0P004_oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3fool0
0`00ooooo`0?ool001Goo`800?Soo`8000ooo`005Ooo00<00?ooool0nOoo0P003Ooo000Eool00`00
ooooo`3kool00`00ooooo`0:ool001Goo`03003ooooo0?coo`8000[oo`005Ooo00<00?ooool0o_oo
0P002?oo000Eool2003oool2ool00`00ooooo`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, -0.117974, \
0.00578885, 0.00936656}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol2[k, 10]], {k, 0.01, 0.4}, 
      PlotStyle \[Rule] {RGBColor[0, 1, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -13.7251 18.3014 [
[.2619 -0.01155 -9 -9 ]
[.2619 -0.01155 9 0 ]
[.5 -0.01155 -9 -9 ]
[.5 -0.01155 9 0 ]
[.7381 -0.01155 -9 -9 ]
[.7381 -0.01155 9 0 ]
[.97619 -0.01155 -9 -9 ]
[.97619 -0.01155 9 0 ]
[.01131 .09246 -30 -4.5 ]
[.01131 .09246 0 4.5 ]
[.01131 .18396 -24 -4.5 ]
[.01131 .18396 0 4.5 ]
[.01131 .27547 -30 -4.5 ]
[.01131 .27547 0 4.5 ]
[.01131 .36698 -24 -4.5 ]
[.01131 .36698 0 4.5 ]
[.01131 .45848 -30 -4.5 ]
[.01131 .45848 0 4.5 ]
[.01131 .54999 -24 -4.5 ]
[.01131 .54999 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .00095 m
.2619 .0072 L
s
[(0.1)] .2619 -0.01155 0 1 Mshowa
.5 .00095 m
.5 .0072 L
s
[(0.2)] .5 -0.01155 0 1 Mshowa
.7381 .00095 m
.7381 .0072 L
s
[(0.3)] .7381 -0.01155 0 1 Mshowa
.97619 .00095 m
.97619 .0072 L
s
[(0.4)] .97619 -0.01155 0 1 Mshowa
.125 Mabswid
.07143 .00095 m
.07143 .0047 L
s
.11905 .00095 m
.11905 .0047 L
s
.16667 .00095 m
.16667 .0047 L
s
.21429 .00095 m
.21429 .0047 L
s
.30952 .00095 m
.30952 .0047 L
s
.35714 .00095 m
.35714 .0047 L
s
.40476 .00095 m
.40476 .0047 L
s
.45238 .00095 m
.45238 .0047 L
s
.54762 .00095 m
.54762 .0047 L
s
.59524 .00095 m
.59524 .0047 L
s
.64286 .00095 m
.64286 .0047 L
s
.69048 .00095 m
.69048 .0047 L
s
.78571 .00095 m
.78571 .0047 L
s
.83333 .00095 m
.83333 .0047 L
s
.88095 .00095 m
.88095 .0047 L
s
.92857 .00095 m
.92857 .0047 L
s
.25 Mabswid
0 .00095 m
1 .00095 L
s
.02381 .09246 m
.03006 .09246 L
s
[(0.755)] .01131 .09246 1 0 Mshowa
.02381 .18396 m
.03006 .18396 L
s
[(0.76)] .01131 .18396 1 0 Mshowa
.02381 .27547 m
.03006 .27547 L
s
[(0.765)] .01131 .27547 1 0 Mshowa
.02381 .36698 m
.03006 .36698 L
s
[(0.77)] .01131 .36698 1 0 Mshowa
.02381 .45848 m
.03006 .45848 L
s
[(0.775)] .01131 .45848 1 0 Mshowa
.02381 .54999 m
.03006 .54999 L
s
[(0.78)] .01131 .54999 1 0 Mshowa
.125 Mabswid
.02381 .01925 m
.02756 .01925 L
s
.02381 .03755 m
.02756 .03755 L
s
.02381 .05585 m
.02756 .05585 L
s
.02381 .07415 m
.02756 .07415 L
s
.02381 .11076 m
.02756 .11076 L
s
.02381 .12906 m
.02756 .12906 L
s
.02381 .14736 m
.02756 .14736 L
s
.02381 .16566 m
.02756 .16566 L
s
.02381 .20226 m
.02756 .20226 L
s
.02381 .22057 m
.02756 .22057 L
s
.02381 .23887 m
.02756 .23887 L
s
.02381 .25717 m
.02756 .25717 L
s
.02381 .29377 m
.02756 .29377 L
s
.02381 .31207 m
.02756 .31207 L
s
.02381 .33037 m
.02756 .33037 L
s
.02381 .34868 m
.02756 .34868 L
s
.02381 .38528 m
.02756 .38528 L
s
.02381 .40358 m
.02756 .40358 L
s
.02381 .42188 m
.02756 .42188 L
s
.02381 .44018 m
.02756 .44018 L
s
.02381 .47678 m
.02756 .47678 L
s
.02381 .49509 m
.02756 .49509 L
s
.02381 .51339 m
.02756 .51339 L
s
.02381 .53169 m
.02756 .53169 L
s
.02381 .56829 m
.02756 .56829 L
s
.02381 .58659 m
.02756 .58659 L
s
.02381 .60489 m
.02756 .60489 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.04762 .01472 m
.05637 .01502 L
.06596 .01547 L
.08529 .01671 L
.10656 .01861 L
.12637 .02089 L
.16439 .02663 L
.2048 .03471 L
.24373 .04443 L
.28118 .05558 L
.32103 .06939 L
.35939 .08459 L
.39628 .10099 L
.43556 .12039 L
.47337 .14095 L
.51356 .16487 L
.55228 .18995 L
.58951 .21597 L
.62914 .24574 L
.6673 .27645 L
.70784 .31131 L
.7469 .34713 L
.78449 .38367 L
.82447 .42484 L
.86297 .46678 L
.89999 .50906 L
.9394 .55657 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool005Ooo`8000Coo`03003ooooo00;oo`@002koo`8000Co
o`03003ooooo00;oo`@002koo`8000Coo`03003ooooo00?oo`80033oo`8000Coo`03003ooooo00Co
o`03003ooooo007oo`00E_oo00@00?ooool000[oo`03003ooooo02coo`04003ooooo0008ool00`00
ooooo`0^ool01000ooooo`002?oo00@00?ooool002koo`04003ooooo000:ool00`00ooooo`01ool0
05Koo`04003ooooo000:ool00`00ooooo`0/ool01000ooooo`002Ooo00<00?ooool0;Ooo00@00?oo
ool000_oo`03003ooooo02coo`04003ooooo0007ool50002ool005Koo`04003ooooo000:ool00`00
ooooo`0/ool01000ooooo`002_oo00<00?ooool0;?oo00@00?ooool000Woo`8002ooo`04003ooooo
0007ool01000ooooo`000ooo001Fool01000ooooo`002_oo00<00?ooool0;?oo00@00?ooool000_o
o`03003ooooo02_oo`04003ooooo000;ool00`00ooooo`0/ool01000ooooo`002?oo00<00?oo0000
0ooo001Fool01000ooooo`002?oo0`00;_oo00@00?ooool000Soo`04003ooooo000]ool01000oooo
o`002?oo00@00?ooool002koo`04003ooooo0009ool20003ool005Ooo`8000_oo`03003ooooo02go
o`8000[oo`8002ooo`8000[oo`80033oo`8000_oo`03003ooooo007oo`00oooo8Ooo003ooolQool0
0?ooob7oo`006oooo`000`000ooo000Qool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`07ool0027oo`03003ooooo0?coo`008Ooo
00<00?ooool00ooo10OPmOoo000Qool00`00ooooo`07ool>1n3Wool0027oo`8001Koo`L7h>3oo`00
8Ooo00<00?ooool07?oo1`OPfOoo000Qool00`00ooooo`0Sool31n3Fool0027oo`03003ooooo02Ko
o`@7h=;oo`008Ooo0P00:ooo10OPc_oo000Qool00`00ooooo`0^ool51n39ool0027oo`03003ooooo
03?oo`@7h<Goo`008Ooo00<00?ooool0=ooo0`OP`_oo000Qool00`00ooooo`0jool41n2nool0027o
o`8003ooo`<7h;_oo`008Ooo00<00?ooool0@Ooo0`OP^?oo000Qool00`00ooooo`14ool41n2dool0
027oo`03003ooooo04Soo`<7h;7oo`008Ooo00<00?ooool0Booo0POP[ooo000Qool2001>ool31n2/
ool0027oo`03003ooooo053oo`87h:[oo`000_oo0P001?oo00<00?ooool00ooo00<00?ooool00_oo
0`000ooo0`001Ooo00<00?ooool0D_oo0POPZ?oo00001Ooo003ooooo00002Ooo00<00?ooool01Ooo
00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0E?oo0POPY_oo00001Ooo003ooooo00002_oo
00<00?ooool01?oo00<00?ooool00ooo00<00?ooool00_oo0P00Eooo0POPY?oo00001Ooo003ooooo
00002_oo00@00?ooooooo`<000?oo`<000Goo`03003ooooo05Soo`87h:;oo`0000Goo`00ooooo`00
00_oo`04003ooooo0005ool00`00ooooo`05ool00`00ooooo`1Jool21n2Pool00005ool00?ooool0
0008ool01000ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0G?oo0POPW_oo
0002ool20009ool40002ool40002ool40004ool00`00ooooo`1Nool21n2Lool0027oo`80067oo`87
h9[oo`008Ooo00<00?ooool0H_oo0POPV?oo000Qool00`00ooooo`1Tool21n2Fool0027oo`03003o
oooo06Koo`87h9Coo`008Ooo00<00?ooool0J?oo0POPT_oo000Qool2001[ool21n2@ool0027oo`03
003ooooo06coo`87h8koo`008Ooo00<00?ooool0K_oo0POPS?oo000Qool00`00ooooo`1`ool00`OP
ooooo`29ool0027oo`8007;oo`87h8Woo`008Ooo00<00?ooool0Looo0POPQooo000Qool00`00oooo
o`1eool21n25ool0027oo`03003ooooo07Ooo`87h8?oo`008Ooo00<00?ooool0NOoo0POPPOoo000Q
ool2001lool00`OPooooo`1nool0027oo`03003ooooo07coo`87h7koo`008Ooo00<00?ooool0O_oo
0POPO?oo0008ool20004ool00`00ooooo`03ool00`00ooooo`03ool20005ool00`00ooooo`20ool0
0`OPooooo`1iool000Ooo`04003ooooo0009ool00`00ooooo`02ool01000ooooo`001?oo00<00?oo
ool0POoo0POPNOoo0007ool01000ooooo`002_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`80
08Coo`87h7Ooo`001ooo00@00?ooool000[oo`04003oooooool30005ool00`00ooooo`25ool00`OP
ooooo`1dool000Ooo`04003ooooo000;ool01000ooooo`001ooo00<00?ooool0Q_oo00<7h?ooool0
Looo0007ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo08Ooo`87
h7?oo`002?oo0P002Ooo10000ooo0`001?oo00<00?ooool0ROoo00<7h?ooool0L?oo000Qool2002;
ool00`OPooooo`1_ool0027oo`03003ooooo08_oo`87h6ooo`008Ooo00<00?ooool0SOoo00<7h?oo
ool0K?oo000Qool00`00ooooo`2>ool00`OPooooo`1[ool0027oo`80093oo`031n3ooooo06[oo`00
8Ooo00<00?ooool0T?oo0POPJ_oo000Qool00`00ooooo`2Bool00`OPooooo`1Wool0027oo`03003o
oooo09?oo`031n3ooooo06Koo`008Ooo00<00?ooool0U?oo00<7h?ooool0IOoo000Qool2002Fool2
1n1Uool0027oo`03003ooooo09Ooo`031n3ooooo06;oo`008Ooo00<00?ooool0V?oo00<7h?ooool0
HOoo000Qool00`00ooooo`2Iool00`OPooooo`1Pool0027oo`03003ooooo09[oo`87h63oo`008Ooo
0P00WOoo00<7h?ooool0GOoo000Qool00`00ooooo`2Mool00`OPooooo`1Lool000;oo`8000Coo`03
003ooooo00?oo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo09koo`031n3ooooo05_oo`00
00Goo`00ooooo`0000Woo`03003ooooo00;oo`04003ooooo0005ool00`00ooooo`02ool00`00oooo
o`2Oool21n1Kool00005ool00?ooool0000:ool01@00oooooooo00000_oo00<00?ooool00ooo00<0
0?ooool00_oo0P00X_oo00<7h?ooool0F?oo00001Ooo003ooooo00002_oo00@00?ooooooo`<000?o
o`<000Goo`03003ooooo0:;oo`031n3ooooo05Ooo`0000Goo`00ooooo`0000_oo`04003ooooo0005
ool00`00ooooo`05ool00`00ooooo`2Sool00`OPooooo`1Fool00005ool00?ooool00008ool01000
ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0Y?oo0POPE_oo0002ool20009
ool40003ool30002ool40004ool00`00ooooo`2Vool00`OPooooo`1Cool0027oo`800:Soo`031n3o
oooo05;oo`008Ooo00<00?ooool0Z?oo00<7h?ooool0DOoo000Qool00`00ooooo`2Yool00`OPoooo
o`1@ool0027oo`03003ooooo0:[oo`87h53oo`008Ooo00<00?ooool0[?oo00<7h?ooool0COoo000Q
ool2002^ool00`OPooooo`1<ool0027oo`03003ooooo0:koo`031n3ooooo04_oo`008Ooo00<00?oo
ool0[ooo00<7h?ooool0B_oo000Qool00`00ooooo`2`ool00`OPooooo`19ool0027oo`03003ooooo
0;7oo`031n3ooooo04Soo`008Ooo0P00/ooo0POPB?oo000Qool00`00ooooo`2dool00`OPooooo`15
ool0027oo`03003ooooo0;Goo`031n3ooooo04Coo`008Ooo00<00?ooool0]_oo00<7h?ooool0@ooo
000Qool2002hool00`OPooooo`12ool0027oo`03003ooooo0;Soo`031n3ooooo047oo`008Ooo00<0
0?ooool0^Ooo00<7h?ooool0@?oo0008ool20004ool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`04ool00`00ooooo`2jool00`OPooooo`0oool000Ooo`04003ooooo0009ool00`00ooooo`03
ool00`00ooooo`04ool00`00ooooo`2kool00`OPooooo`0nool000Ooo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`03ool2002mool00`OPooooo`0mool000Ooo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`2mool00`OPooooo`0lool000Ooo`04003ooooo000;
ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`2nool00`OPooooo`0kool000Ooo`04003o
oooo0008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo0;ooo`031n3ooooo03[oo`00
2?oo0P002Ooo10000_oo10001?oo00<00?ooool0`?oo00<7h?ooool0>Ooo000Qool20032ool00`OP
ooooo`0hool0027oo`03003ooooo0<;oo`031n3ooooo03Ooo`008Ooo00<00?ooool0`ooo00<7h?oo
ool0=_oo000Qool00`00ooooo`34ool00`OPooooo`0eool0027oo`800<Koo`031n3ooooo03Coo`00
8Ooo00<00?ooool0a_oo00<7h?ooool0<ooo000Qool00`00ooooo`37ool00`OPooooo`0bool0027o
o`03003ooooo0<Soo`031n3ooooo037oo`008Ooo00<00?ooool0bOoo00<7h?ooool0<?oo000Qool2
003;ool00`OPooooo`0_ool0027oo`03003ooooo0<_oo`031n3ooooo02koo`008Ooo00<00?ooool0
c?oo00<7h?ooool0;Ooo000Qool00`00ooooo`3=ool00`OPooooo`0/ool0027oo`03003ooooo0<go
o`031n3ooooo02coo`008Ooo0P00cooo00<7h?ooool0:ooo000Qool00`00ooooo`3?ool00`OPoooo
o`0Zool000;oo`8000Coo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00;oo`<000Goo`03
003ooooo0=3oo`031n3ooooo02Woo`0000Goo`00ooooo`0000Woo`03003ooooo00?oo`03003ooooo
00Goo`03003ooooo00;oo`03003ooooo0=7oo`031n3ooooo02Soo`0000Goo`00ooooo`0000[oo`03
003ooooo00?oo`03003ooooo00Coo`03003ooooo00;oo`800=?oo`031n3ooooo02Ooo`0000Goo`00
ooooo`0000[oo`03003ooooo00?oo`04003oooooool30005ool00`00ooooo`3Bool00`OPooooo`0W
ool00005ool00?ooool0000;ool00`00ooooo`03ool01000ooooo`001ooo00<00?ooool0dooo00<7
h?ooool09_oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool00`00oooo
o`05ool00`00ooooo`3Dool00`OPooooo`0Uool000;oo`8000Woo`@000;oo`@000;oo`@000Coo`03
003ooooo0=Goo`031n3ooooo02Coo`008Ooo0P00eooo00<7h?ooool08ooo000Qool00`00ooooo`3G
ool00`OPooooo`0Rool0027oo`03003ooooo0=Soo`031n3ooooo027oo`008Ooo00<00?ooool0fOoo
00<7h?ooool08?oo000Qool00`00ooooo`3Jool00`OPooooo`0Oool0027oo`800=coo`031n3ooooo
01koo`008Ooo00<00?ooool0g?oo00<7h?ooool07Ooo000Qool00`00ooooo`3Mool00`OPooooo`0L
ool0027oo`03003ooooo0=koo`031n3ooooo01_oo`008Ooo00<00?ooool0gooo00<7h?ooool06_oo
000Qool2003Qool00`OPooooo`0Iool0027oo`03003ooooo0>3oo`031n3ooooo01Woo`008Ooo00<0
0?ooool0hOoo00<7h?ooool06?oo000Qool00`00ooooo`3Rool00`OPooooo`0Gool0027oo`800>Co
o`031n3ooooo01Koo`008Ooo00<00?ooool0hooo00<7h?ooool05_oo000Qool00`00ooooo`3Tool0
0`OPooooo`0Eool000Soo`8000Coo`03003ooooo00?oo`03003ooooo00?oo`8000Goo`03003ooooo
0>Goo`031n3ooooo01Coo`001ooo00@00?ooool000Woo`03003ooooo00;oo`04003ooooo0004ool0
0`00ooooo`3Vool00`OPooooo`0Cool000Ooo`04003ooooo000:ool01@00oooooooo00000_oo00<0
0?ooool00_oo0P00iooo00<7h?ooool04ooo0007ool01000ooooo`002_oo00<00?ooool00_oo0P00
1Ooo00<00?ooool0iooo00<7h?ooool04_oo0007ool01000ooooo`002ooo00@00?ooool000;oo`03
003ooooo00;oo`03003ooooo0>Soo`031n3ooooo017oo`001ooo00@00?ooool000Soo`04003ooooo
0002ool01000ooooo`001?oo00<00?ooool0jOoo00<7h?ooool04?oo0008ool20009ool40003ool2
0005ool00`00ooooo`3Zool00`OPooooo`0?ool0027oo`800>_oo`031n3ooooo00ooo`008Ooo00<0
0?ooool0jooo00<7h?ooool03_oo000Qool00`00ooooo`3/ool00`OPooooo`0=ool0027oo`03003o
oooo0>goo`031n3ooooo00coo`008Ooo0P00kooo00<7h?ooool02ooo000Qool00`00ooooo`3_ool0
0`OPooooo`0:ool0027oo`03003ooooo0>ooo`031n3ooooo00[oo`008Ooo00<00?ooool0l?oo00<7
h?ooool02Ooo000Qool00`00ooooo`3aool00`OPooooo`08ool0027oo`800??oo`031n3ooooo00Oo
o`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`00oooo8Ooo
003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0542745, 0.746753, \
0.00163409, 0.000212589}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %1070]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -0.183123 0.524294 [
[.15079 .32867 -9 -9 ]
[.15079 .32867 9 0 ]
[.27778 .32867 -9 -9 ]
[.27778 .32867 9 0 ]
[.40476 .32867 -9 -9 ]
[.40476 .32867 9 0 ]
[.53175 .32867 -9 -9 ]
[.53175 .32867 9 0 ]
[.65873 .32867 -3 -9 ]
[.65873 .32867 3 0 ]
[.78571 .32867 -9 -9 ]
[.78571 .32867 9 0 ]
[.9127 .32867 -9 -9 ]
[.9127 .32867 9 0 ]
[.01131 .0266 -18 -4.5 ]
[.01131 .0266 0 4.5 ]
[.01131 .13145 -18 -4.5 ]
[.01131 .13145 0 4.5 ]
[.01131 .23631 -18 -4.5 ]
[.01131 .23631 0 4.5 ]
[.01131 .44603 -18 -4.5 ]
[.01131 .44603 0 4.5 ]
[.01131 .55089 -18 -4.5 ]
[.01131 .55089 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .34117 m
.15079 .34742 L
s
[(0.2)] .15079 .32867 0 1 Mshowa
.27778 .34117 m
.27778 .34742 L
s
[(0.4)] .27778 .32867 0 1 Mshowa
.40476 .34117 m
.40476 .34742 L
s
[(0.6)] .40476 .32867 0 1 Mshowa
.53175 .34117 m
.53175 .34742 L
s
[(0.8)] .53175 .32867 0 1 Mshowa
.65873 .34117 m
.65873 .34742 L
s
[(1)] .65873 .32867 0 1 Mshowa
.78571 .34117 m
.78571 .34742 L
s
[(1.2)] .78571 .32867 0 1 Mshowa
.9127 .34117 m
.9127 .34742 L
s
[(1.4)] .9127 .32867 0 1 Mshowa
.125 Mabswid
.05556 .34117 m
.05556 .34492 L
s
.0873 .34117 m
.0873 .34492 L
s
.11905 .34117 m
.11905 .34492 L
s
.18254 .34117 m
.18254 .34492 L
s
.21429 .34117 m
.21429 .34492 L
s
.24603 .34117 m
.24603 .34492 L
s
.30952 .34117 m
.30952 .34492 L
s
.34127 .34117 m
.34127 .34492 L
s
.37302 .34117 m
.37302 .34492 L
s
.43651 .34117 m
.43651 .34492 L
s
.46825 .34117 m
.46825 .34492 L
s
.5 .34117 m
.5 .34492 L
s
.56349 .34117 m
.56349 .34492 L
s
.59524 .34117 m
.59524 .34492 L
s
.62698 .34117 m
.62698 .34492 L
s
.69048 .34117 m
.69048 .34492 L
s
.72222 .34117 m
.72222 .34492 L
s
.75397 .34117 m
.75397 .34492 L
s
.81746 .34117 m
.81746 .34492 L
s
.84921 .34117 m
.84921 .34492 L
s
.88095 .34117 m
.88095 .34492 L
s
.94444 .34117 m
.94444 .34492 L
s
.97619 .34117 m
.97619 .34492 L
s
.25 Mabswid
0 .34117 m
1 .34117 L
s
.02381 .0266 m
.03006 .0266 L
s
[(0.4)] .01131 .0266 1 0 Mshowa
.02381 .13145 m
.03006 .13145 L
s
[(0.6)] .01131 .13145 1 0 Mshowa
.02381 .23631 m
.03006 .23631 L
s
[(0.8)] .01131 .23631 1 0 Mshowa
.02381 .44603 m
.03006 .44603 L
s
[(1.2)] .01131 .44603 1 0 Mshowa
.02381 .55089 m
.03006 .55089 L
s
[(1.4)] .01131 .55089 1 0 Mshowa
.125 Mabswid
.02381 .05281 m
.02756 .05281 L
s
.02381 .07902 m
.02756 .07902 L
s
.02381 .10524 m
.02756 .10524 L
s
.02381 .15767 m
.02756 .15767 L
s
.02381 .18388 m
.02756 .18388 L
s
.02381 .2101 m
.02756 .2101 L
s
.02381 .26253 m
.02756 .26253 L
s
.02381 .28874 m
.02756 .28874 L
s
.02381 .31496 m
.02756 .31496 L
s
.02381 .36739 m
.02756 .36739 L
s
.02381 .3936 m
.02756 .3936 L
s
.02381 .41982 m
.02756 .41982 L
s
.02381 .47225 m
.02756 .47225 L
s
.02381 .49846 m
.02756 .49846 L
s
.02381 .52467 m
.02756 .52467 L
s
.02381 .00038 m
.02756 .00038 L
s
.02381 .5771 m
.02756 .5771 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.03016 .21049 m
.03249 .2105 L
.03505 .21051 L
.0402 .21055 L
.04588 .2106 L
.05116 .21067 L
.0613 .21083 L
.07207 .21107 L
.08245 .21134 L
.09244 .21166 L
.10307 .21206 L
.1133 .21249 L
.12314 .21296 L
.13361 .21352 L
.14369 .21411 L
.15441 .21479 L
.16473 .21551 L
.17466 .21626 L
.18523 .21711 L
.19541 .21799 L
.20622 .21899 L
.21663 .22002 L
.22666 .22106 L
.23732 .22224 L
.24759 .22344 L
.25746 .22465 L
.26797 .22602 L
.27778 .22735 L
s
1 0 0 r
.03016 .1407 m
.03323 .14078 L
.03608 .14088 L
.03922 .14102 L
.04257 .1412 L
.04942 .14169 L
.05566 .14226 L
.06995 .14402 L
.08347 .14625 L
.10882 .15182 L
.13579 .15958 L
.16176 .16859 L
.18674 .1785 L
.21332 .19018 L
.23891 .20237 L
.26612 .21619 L
.27971 .22338 L
.28633 .22695 L
.28785 .22774 L
.28946 .22859 L
.29012 .22887 L
.29083 .22911 L
.2916 .22929 L
.29233 .22943 L
.29535 .22989 L
.29861 .23038 L
.30453 .23131 L
.31754 .23341 L
.34484 .23823 L
.37113 .24342 L
.39643 .24896 L
.42335 .2555 L
.44926 .26248 L
.47418 .26989 L
.50071 .27862 L
.52625 .28793 L
.5534 .29892 L
.57955 .31068 L
.6047 .32318 L
.63147 .33785 L
.65724 .35336 L
.65873 .3543 L
s
0 g
.24557 0 m
.26647 .01726 L
.30764 .05126 L
.34731 .08401 L
.38546 .11552 L
.42606 .14904 L
.46515 .18132 L
.50273 .21235 L
.54275 .2454 L
.58126 .2772 L
.62221 .31102 L
.66166 .34359 L
.69959 .37492 L
.73997 .40825 L
.77884 .44035 L
.82014 .47446 L
.85994 .50732 L
.89823 .53895 L
.93896 .57258 L
.97619 .60332 L
s
.03016 .2106 m
.03133 .21065 L
.03239 .2107 L
.03362 .21051 L
.03477 .21051 L
.03585 .21091 L
.03684 .21098 L
.03908 .21116 L
.04372 .21163 L
.04884 .21227 L
.0584 .21382 L
.06854 .21589 L
.11039 .22747 L
.15041 .24049 L
.18892 .25309 L
.22987 .26588 L
.26932 .2774 L
.30725 .28767 L
.34763 .29773 L
.3865 .30656 L
.4278 .31501 L
.4676 .3222 L
.5059 .32819 L
.54663 .33345 L
.56691 .3356 L
.58585 .3373 L
.60522 .33872 L
.62357 .33976 L
.63325 .34018 L
.64379 .34055 L
.64843 .34068 L
.65336 .3408 L
.65767 .34088 L
.66222 .34095 L
.66454 .34098 L
.66707 .34101 L
.66941 .34103 L
.67153 .34105 L
.67273 .34105 L
.67404 .34106 L
.67476 .34106 L
.67542 .34106 L
.67672 .34107 L
.67784 .34107 L
.67907 .34107 L
.68036 .34107 L
.68156 .34107 L
.68268 .34106 L
.68391 .34106 L
.68613 .34105 L
Mistroke
.6883 .34104 L
.69031 .34102 L
.69487 .34097 L
.69972 .34091 L
.70996 .34072 L
.72103 .34044 L
.74107 .33979 L
.78186 .33804 L
.82113 .33614 L
.8589 .33433 L
.89911 .33256 L
.93781 .33102 L
.97619 .32968 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool2000jool00`00ooooo`3<ool001Goo`03003ooooo03[oo`03
003ooooo0<_oo`005Ooo00<00?ooool0>ooo00<00?ooool0b_oo000Eool00`00ooooo`0lool2003:
ool001Goo`03003ooooo03koo`03003ooooo0<Ooo`000_oo0P001?oo00<00?ooool01?oo00<00?oo
ool00ooo00<00?ooool0?ooo00<00?ooool0a_oo00001Ooo003ooooo00002_oo00<00?ooool00ooo
00<00?ooool0@?oo00<00?ooool0aOoo00001Ooo003ooooo00001ooo1@001?oo0`00@Ooo0P00aOoo
00001Ooo003ooooo00001ooo00@00?ooool000Goo`03003ooooo04?oo`03003ooooo0<;oo`0000Go
o`00ooooo`0000Soo`03003oo`0000Goo`03003ooooo04Coo`03003ooooo0<7oo`0000Goo`00oooo
o`0000Woo`8000Goo`03003ooooo04Goo`03003ooooo0<3oo`000_oo0P002ooo00<00?ooool00ooo
00<00?ooool0A_oo00<00?ooool0_ooo000Eool00`00ooooo`17ool2002oool001Goo`03003ooooo
04Woo`03003ooooo0;coo`005Ooo0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool00`00
ooooo`2jool001Goo`03003ooooo04coo`800;[oo`005Ooo00<00?ooool0C_oo00<00?ooool0]ooo
000Eool00`00ooooo`1?ool00`00ooooo`2fool001Goo`03003ooooo053oo`03003ooooo0;Goo`00
5Ooo00<00?ooool0DOoo00<00?ooool0]?oo000Eool2001Cool2002dool001Goo`03003ooooo05Co
o`03003ooooo0;7oo`005Ooo00<00?ooool0EOoo00<00?ooool0/?oo000Eool00`00ooooo`1Fool0
0`00ooooo`2_ool001Goo`03003ooooo05Ooo`800:ooo`005Ooo00<00?ooool0FOoo00<00?ooool0
[?oo000Eool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`03003ooooo05_oo`03003ooooo0:[o
o`005Ooo0P00GOoo0P00Z_oo000Eool00`00ooooo`1Nool00`00ooooo`2Wool001Goo`03003ooooo
05ooo`03003ooooo0:Koo`005Ooo00<00?ooool0H?oo00<00?ooool0YOoo000Eool00`00ooooo`1Q
ool00`00ooooo`2Tool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo06;oo`03003o
oooo0:?oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1Sool00`00ooooo`2R
ool00005ool00?ooool00008ool01000ooooo`001?oo0`00I?oo0P00X_oo00001Ooo003ooooo0000
2?oo0`001Ooo00<00?ooool0I_oo00<00?ooool0Wooo00001Ooo003ooooo00002?oo00<00?ooool0
1Ooo00<00?oon0000oP0I?oo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo
00<00?ooool00ooo2?P0GOoo00<00?ooool0WOoo0002ool2000:ool30004ool00`00ooooo`0;ool6
n01Hool00`00ooooo`2Lool001Goo`03003ooooo017oo`Gh05Coo`03003ooooo09_oo`005Ooo00<0
0?ooool05_oo1?P0DOoo0P00Vooo000Eool2000Kool3n01@ool00`00ooooo`2Hool001Goo`03003o
oooo01goo`;h04ooo`03003ooooo09Ooo`005Ooo00<00?ooool07ooo0_P0C_oo0P00Uooo000Eool0
0`00ooooo`0Qool3n01=ool00`00ooooo`2Dool001Goo`03003ooooo02Coo`;h04coo`03003ooooo
09?oo`005Ooo00<00?ooool09_oo0_P0Booo0P00Tooo000Eool00`00ooooo`0Xool3n01:ool00`00
ooooo`2@ool001Goo`8002coo`;h04Woo`03003ooooo08ooo`005Ooo00<00?ooool0;Ooo0_P0B?oo
00<00?ooool0S_oo000Eool00`00ooooo`0_ool3n016ool00`00ooooo`2=ool001Goo`03003ooooo
03;oo`;h04Goo`03003ooooo08coo`005Ooo00<00?ooool0=?oo0_P0A?oo0P00S?oo000Eool00`00
ooooo`0fool2n014ool00`00ooooo`29ool001Goo`03003ooooo03Soo`;h04?oo`03003ooooo08So
o`005Ooo1@003@OP:ooo0_P0@_oo00<00?ooool0Qooo000Eool00`00ooooo`02ool60007oolC1n0J
ool2n011ool00`00ooooo`26ool001Goo`03003ooooo00Soo`<001Ooo``7h13oo`;h043oo`03003o
oooo08Goo`005Ooo00<00?ooool02ooo10007ooo2`OP1ooo0_P0?ooo0P00QOoo000Eool00`00oooo
o`0?ool4000Vool81n000ooon03h000oool00`00ooooo`22ool000;oo`8000Coo`03003ooooo00?o
o`8000Goo`03003ooooo01?oo`<002_oo`87h004oooh0?P0n00mool00`00ooooo`21ool00005ool0
0?ooool00008ool01000ooooo`001?oo00<00?ooool05_oo1000;Ooo2?P0=_oo00<00?ooool0P?oo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`<001[oo`@0037oo`Ch03?oo`03003ooooo07oo
o`0000Goo`00ooooo`0000Woo`8000Goo`03003ooooo01koo`<003;oo`Gh02ooo`8007ooo`0000Go
o`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Qool3000dool5n00/ool00`00ooooo`1l
ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool09?oo0P00=ooo1?P0:Ooo00<0
0?ooool0Nooo0002ool2000:ool20005ool00`00ooooo`0Vool3000hool5n00Uool2001kool001Go
o`03003ooooo02Woo`<003[oo`Kh027oo`03003ooooo07Soo`005Ooo00<00?ooool0;?oo1000??oo
1?P07_oo00<00?ooool0Mooo000Eool2000aool4000lool3n00Lool2001gool001Goo`03003ooooo
03Coo`<003coo`?h01_oo`03003ooooo07Coo`005Ooo00<00?ooool0=ooo1000>ooo0oP06Ooo00<0
0?ooool0Looo000Eool00`00ooooo`0kool4000jool2n00Hool2001cool001Goo`03003ooooo03oo
o`<003Woo`?h01Ooo`03003ooooo073oo`005Ooo00<00?ooool0@_oo0`00>Ooo0oP05Ooo00<00?oo
ool0Kooo000Eool00`00ooooo`15ool4000hool4n00Bool00`00ooooo`1^ool001Goo`03003ooooo
04Woo`<003Woo`?h013oo`03003ooooo06goo`005Ooo0P00COoo1000>?oo0_P03ooo00<00?ooool0
K?oo000Eool00`00ooooo`1@ool4000fool2n00>ool2001/ool001Goo`03003ooooo05Coo`@003Co
o`?h00goo`03003ooooo06Woo`005Ooo00<00?ooool06?oo0P001?oo00<00?ooool00_oo10005?oo
0P001?oo00<00?ooool01?oo00<00?ooool03Ooo20001?oo00<00?ooool00ooo0P005?oo0P001?oo
00<00?ooool00ooo0P0000?oooP0n0003?oo00<00?ooool02?oo10006?oo10000ooo00<00?ooool0
0_oo10004ooo10000ooo00<00?ooool01?oo00<00?ooool04?oo000Eool00`00ooooo`0Gool01000
ooooo`002?oo00<00?ooool05?oo00@00?ooool000[oo`03003ooooo01;oo`D000Ooo`04003ooooo
000Bool01000ooooo`002?oo00@00?ooool000;oo`;h00_oo`03003ooooo00Woo`03003ooooo01Wo
o`03003ooooo00Ooo`03003ooooo01Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo00<00?oo
ool05ooo00@00?ooool000Woo`03003ooooo01?oo`04003ooooo0007ool5000Cool01@00ooooo`00
ool010000ooo00@00?ooool001;oo`04003ooooo0008ool01000ooooo`001?oo0_P02_oo00<00?oo
ool02?oo00<00?ooool06Ooo00<00?ooool02?oo00<00?ooool05Ooo00<00?ooool01_oo1@004Ooo
000Eool00`00ooooo`0Gool01000ooooo`002_oo00<00?ooool04_oo00@00?ooool000Ooo`04003o
oooo000Dool01000ooooo`001Ooo1P004ooo00@00?ooool000Woo`8000Ooo`;h00Woo`03003ooooo
00Ooo`03003ooooo01Woo`03003ooooo00Woo`03003ooooo01Coo`03003ooooo00Koo`04003ooooo
000Bool001Goo`8001Soo`04003ooooo000;ool00`00ooooo`0Aool01000ooooo`002?oo00<00?oo
00005?oo00@00?ooool000Soo`D0017oo`04003ooooo0008ool01000ooooo`002?oo0_P02?oo00<0
0?ooool01_oo00<00?ooool06Ooo00<00?ooool02_oo00<00?ooool04ooo00<00?ooool01ooo00<0
0?oo00004_oo000Eool00`00ooooo`0Gool01000ooooo`002?oo00@00?ooool001?oo`04003ooooo
0009ool2000Dool01000ooooo`002?oo00<00?ooool00_oo1P002ooo00@00?ooool000Soo`04003o
oooo000:ool2n007ool20004ool3000Iool30009ool01000ooooo`004ooo0`002_oo0P004_oo000E
ool00`00ooooo`0Hool2000:ool2000Eool2000;ool00`00ooooo`0Cool2000:ool30007ool70005
ool2000:ool2000=ool00oP0ooooo`06ool00`00ooooo`03ool00`00ooooo`0Iool00`00ooooo`08
ool2000Fool00`00ooooo`09ool00`00ooooo`0@ool001Goo`03003ooooo07_oo`P001Woo`;h00Oo
o`03003ooooo063oo`005Ooo00<00?ooool0Pooo1P005Ooo0_P01_oo00<00?ooool0?Ooo6`001ooo
000Eool00`00ooooo`29ool;000<ool2n005ool2000XoolE000Rool001Goo`03003ooooo09Coo`h0
0003n03ooooo00Coo`03003ooooo01;ooa<003Ooo`003_ooo`004`00000Eool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool01000oooooooo0_P00ooo0P001_oo00<0
0?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo000E
ool00`00ooooo`2Wool2n003ool2001Jool001Goo`03003ooooo0:Woo`;h00?oo`03003ooooo05Oo
o`005Ooo00<00?ooool0[ooo00<00?ooool0E_oo000Eool00`00ooooo`2`ool00`00ooooo`1Eool0
01Goo`03003ooooo0;7oo`03003ooooo05Coo`005Ooo0P00/ooo0P00E?oo000Eool00`00ooooo`2d
ool00`00ooooo`1Aool001Goo`03003ooooo0;Goo`03003ooooo053oo`005Ooo00<00?ooool0]_oo
00<00?ooool0Cooo000Eool00`00ooooo`2gool2001?ool001Goo`03003ooooo0;Woo`03003ooooo
04coo`005Ooo00<00?ooool0^_oo00<00?ooool0Booo000Eool2002lool00`00ooooo`1:ool001Go
o`03003ooooo0;coo`03003ooooo04Woo`005Ooo00<00?ooool0_Ooo0P00BOoo000Eool00`00oooo
o`2oool00`00ooooo`16ool001Goo`03003ooooo0<3oo`03003ooooo04Goo`005Ooo00<00?ooool0
`Ooo00<00?ooool0A?oo000Eool00`00ooooo`32ool00`00ooooo`13ool001Goo`800<Coo`03003o
oooo04;oo`005Ooo00<00?ooool0a?oo00<00?ooool0@Ooo000Eool00`00ooooo`35ool20011ool0
01Goo`03003ooooo0<Ooo`03003ooooo03koo`005Ooo00<00?ooool0b?oo00<00?ooool0?Ooo0000
0ooo000000020003ool00`00ooooo`02ool40004ool00`00ooooo`39ool00`00ooooo`0lool000?o
o`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?oo
ool02?oo00<00?ooool01?oo0`00booo00<00?ooool0>_oo0003ool00`00ooooo`09ool00`00oooo
o`03ool00`00ooooo`3<ool2000jool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo
0<koo`03003ooooo03Ooo`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3?ool00`00
ooooo`0fool000?oo`03003ooooo00Soo`8000Goo`03003ooooo0=3oo`8003Koo`005Ooo00<00?oo
ool0d_oo00<00?ooool0<ooo000Eool00`00ooooo`3Cool00`00ooooo`0bool001Goo`800=Goo`80
03;oo`005Ooo00<00?ooool0e_oo00<00?ooool0;ooo000Eool00`00ooooo`3Gool00`00ooooo`0^
ool001Goo`03003ooooo0=Soo`03003ooooo02goo`005Ooo00<00?ooool0fOoo00<00?ooool0;?oo
000Eool00`00ooooo`3Jool00`00ooooo`0[ool001Goo`03003ooooo0=_oo`8002_oo`005Ooo00<0
0?ooool0gOoo00<00?ooool0:?oo000Eool2003Oool00`00ooooo`0Wool001Goo`03003ooooo0=oo
o`03003ooooo02Koo`005Ooo00<00?ooool0h?oo00<00?ooool09Ooo000Eool00`00ooooo`3Qool0
0`00ooooo`0Tool001Goo`03003ooooo0>;oo`8002Coo`005Ooo00<00?ooool0i?oo00<00?ooool0
8Ooo000Eool00`00ooooo`3Uool00`00ooooo`0Pool001Goo`800>Ooo`03003ooooo01ooo`005Ooo
00<00?ooool0iooo00<00?ooool07_oo000Eool00`00ooooo`3Xool2000Nool001Goo`03003ooooo
0>[oo`03003ooooo01_oo`005Ooo00<00?ooool0jooo00<00?ooool06_oo00000ooo000000020003
ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`3/ool00`00ooooo`0Iool000?oo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo0>goo`8001Woo`000ooo00<00?ooool01_oo1@001?oo
0`00kooo00<00?ooool05_oo0003ool00`00ooooo`06ool01000ooooo`001Ooo00<00?ooool0l?oo
00<00?ooool05Ooo0003ool00`00ooooo`07ool00`00ool00005ool00`00ooooo`3aool00`00oooo
o`0Dool00004ool00000000:ool20005ool00`00ooooo`3bool00`00ooooo`0Cool000?oo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo0??oo`8001?oo`005Ooo00<00?ooool0mOoo00<00?oo
ool04?oo000Eool00`00ooooo`3fool00`00ooooo`0?ool001Goo`800?Soo`03003ooooo00koo`00
5Ooo00<00?ooool0n?oo0P003_oo000Eool00`00ooooo`3jool00`00ooooo`0;ool001Goo`03003o
oooo0?_oo`03003ooooo00[oo`005Ooo00<00?ooool0o?oo00<00?ooool02Ooo000Eool00`00oooo
o`3mool20009ool001Goo`03003ooooo0?ooo`03003ooooo00Koo`005Ooo0P00oooo0_oo00<00?oo
ool01Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, 0.318475, \
0.00578885, 0.00701031}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{mGamma /. strangesol[k, 10]}, {k, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.603319 1.21817 [
[.15079 .59082 -9 -9 ]
[.15079 .59082 9 0 ]
[.27778 .59082 -9 -9 ]
[.27778 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.53175 .59082 -9 -9 ]
[.53175 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.9127 .59082 -9 -9 ]
[.9127 .59082 9 0 ]
[.01131 .11605 -24 -4.5 ]
[.01131 .11605 0 4.5 ]
[.01131 .23787 -24 -4.5 ]
[.01131 .23787 0 4.5 ]
[.01131 .35969 -24 -4.5 ]
[.01131 .35969 0 4.5 ]
[.01131 .4815 -24 -4.5 ]
[.01131 .4815 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .60332 m
.15079 .60957 L
s
[(0.2)] .15079 .59082 0 1 Mshowa
.27778 .60332 m
.27778 .60957 L
s
[(0.4)] .27778 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(0.6)] .40476 .59082 0 1 Mshowa
.53175 .60332 m
.53175 .60957 L
s
[(0.8)] .53175 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(1)] .65873 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(1.2)] .78571 .59082 0 1 Mshowa
.9127 .60332 m
.9127 .60957 L
s
[(1.4)] .9127 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.18254 .60332 m
.18254 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.34127 .60332 m
.34127 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.81746 .60332 m
.81746 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.97619 .60332 m
.97619 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .11605 m
.03006 .11605 L
s
[(-0.4)] .01131 .11605 1 0 Mshowa
.02381 .23787 m
.03006 .23787 L
s
[(-0.3)] .01131 .23787 1 0 Mshowa
.02381 .35969 m
.03006 .35969 L
s
[(-0.2)] .01131 .35969 1 0 Mshowa
.02381 .4815 m
.03006 .4815 L
s
[(-0.1)] .01131 .4815 1 0 Mshowa
.125 Mabswid
.02381 .0186 m
.02756 .0186 L
s
.02381 .04296 m
.02756 .04296 L
s
.02381 .06733 m
.02756 .06733 L
s
.02381 .09169 m
.02756 .09169 L
s
.02381 .14042 m
.02756 .14042 L
s
.02381 .16478 m
.02756 .16478 L
s
.02381 .18914 m
.02756 .18914 L
s
.02381 .21351 m
.02756 .21351 L
s
.02381 .26223 m
.02756 .26223 L
s
.02381 .2866 m
.02756 .2866 L
s
.02381 .31096 m
.02756 .31096 L
s
.02381 .33532 m
.02756 .33532 L
s
.02381 .38405 m
.02756 .38405 L
s
.02381 .40841 m
.02756 .40841 L
s
.02381 .43278 m
.02756 .43278 L
s
.02381 .45714 m
.02756 .45714 L
s
.02381 .50587 m
.02756 .50587 L
s
.02381 .53023 m
.02756 .53023 L
s
.02381 .55459 m
.02756 .55459 L
s
.02381 .57896 m
.02756 .57896 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03016 .59094 m
.06854 .51944 L
.11039 .45381 L
.1497 .4043 L
.1875 .36523 L
.22774 .33046 L
.26647 .302 L
.30764 .27595 L
.34731 .25415 L
.38546 .23576 L
.42606 .21858 L
.46515 .20409 L
.50273 .19183 L
.54275 .18033 L
.58126 .17043 L
.62221 .1606 L
.66166 .15098 L
.69959 .14071 L
.73997 .12785 L
.77884 .11321 L
.82014 .09539 L
.85994 .07642 L
.89823 .05691 L
.93896 .0352 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3iool20007ool001_oo`800?Soo`8000Woo`006ooo00<00?ooool0mOoo0P002ooo000K
ool00`00ooooo`3cool2000=ool001_oo`03003ooooo0?7oo`8000ooo`006ooo00<00?ooool0l?oo
00<00?ooool03ooo000Kool00`00ooooo`3^ool2000Bool001_oo`800>goo`8001Coo`006ooo00<0
0?ooool0j_oo0P005_oo000Kool00`00ooooo`3Xool2000Hool001_oo`03003ooooo0>Koo`8001[o
o`006ooo00<00?ooool0i?oo0P007?oo000Kool00`00ooooo`3Rool2000Nool001_oo`03003ooooo
0>3oo`80023oo`006ooo0P00gooo0P008_oo000Kool00`00ooooo`3Lool2000Tool001_oo`03003o
oooo0=[oo`8002Koo`006ooo00<00?ooool0f?oo0P00:?oo000Kool00`00ooooo`3Fool2000Zool0
01_oo`03003ooooo0=Coo`8002coo`006ooo0P00dooo0P00;_oo000Kool00`00ooooo`3@ool2000`
ool001_oo`03003ooooo0<koo`8003;oo`006ooo00<00?ooool0c?oo0P00=?oo000Kool00`00oooo
o`3:ool2000fool000Soo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo0<Soo`80
03Soo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo0<Goo`<003[oo`001ooo00@0
0?ooool000Ooo`D000Coo`800<?oo`<003goo`0000?oo`0000000P000_oo00@00?ooool000Ooo`04
003ooooo0005ool00`00ooooo`30ool20010ool000Ooo`04003ooooo0008ool00`00ool00005ool0
0`00ooooo`2mool30012ool000Ooo`04003ooooo0009ool20005ool00`00ooooo`2jool30015ool0
00Soo`8000_oo`03003ooooo00?oo`03003ooooo0;Koo`@004Soo`006ooo00<00?ooool0/ooo0`00
C?oo000Kool2002aool3001?ool001_oo`03003ooooo0:coo`@005;oo`006ooo00<00?ooool0ZOoo
0`00E_oo000Kool00`00ooooo`2Vool3001Iool001_oo`03003ooooo0:;oo`@005coo`006ooo00<0
0?ooool0W_oo1000H?oo000Kool00`00ooooo`2Jool4001Tool001_oo`8009Goo`H006Soo`006ooo
00<00?ooool0T?oo1000K_oo000Kool00`00ooooo`2<ool4001bool001_oo`03003ooooo08Woo`<0
07Koo`006ooo00<00?ooool0Q_oo0`00NOoo000Kool00`00ooooo`22ool4001lool001_oo`8007oo
o`@0083oo`006ooo00<00?ooool0Nooo0`00Q?oo000Kool00`00ooooo`1gool40027ool001_oo`03
003ooooo07Coo`<008_oo`006ooo00<00?ooool0LOoo0`00S_oo000Kool00`00ooooo`1_ool2002A
ool001_oo`03003ooooo06coo`<009?oo`006ooo0P00Jooo0P00U_oo000Kool00`00ooooo`1Wool3
002Hool001_oo`03003ooooo06Goo`8009_oo`006ooo00<00?ooool0Hooo0P00WOoo0008ool20004
ool00`00ooooo`03ool20005ool00`00ooooo`1Qool2002Oool000Ooo`04003ooooo0008ool01000
ooooo`001?oo00<00?ooool0Gooo0P00XOoo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00
GOoo0`00Xooo00000ooo000000020002ool01000ooooo`002Ooo0P001Ooo00<00?ooool0F_oo0P00
Y_oo0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0Eooo0`00Z?oo0007ool01000
ooooo`002?oo00@00?ooool000Coo`03003ooooo05Goo`800:_oo`002?oo0P002_oo0P001Ooo00<0
0?ooool0Dooo0P00[Ooo000Kool00`00ooooo`1Aool2002_ool001_oo`03003ooooo04ooo`800;7o
o`006ooo0P00C_oo0P00/ooo000Kool00`00ooooo`1;ool2002eool001_oo`03003ooooo04Woo`80
0;Ooo`006ooo00<00?ooool0B?oo00<00?ooool0]ooo000Kool00`00ooooo`16ool2002jool001_o
o`03003ooooo04Goo`03003ooooo0;[oo`006ooo0P00A?oo0P00_Ooo000Kool00`00ooooo`11ool2
002oool001_oo`03003ooooo043oo`03003ooooo0;ooo`006ooo00<00?ooool0?_oo0P00`_oo000K
ool00`00ooooo`0mool00`00ooooo`32ool001_oo`03003ooooo03coo`03003ooooo0<?oo`006ooo
00<00?ooool0>_oo0P00a_oo000Kool2000jool00`00ooooo`36ool001_oo`03003ooooo03Soo`03
003ooooo0<Ooo`006ooo00<00?ooool0=ooo00<00?ooool0b?oo000Kool00`00ooooo`0eool2003;
ool001_oo`03003ooooo03Coo`03003ooooo0<_oo`006ooo00<00?ooool0<ooo00<00?ooool0c?oo
000Kool2000cool00`00ooooo`3=ool001_oo`03003ooooo033oo`800=3oo`006ooo00<00?ooool0
;ooo00<00?ooool0d?oo000Kool00`00ooooo`0^ool00`00ooooo`3Aool001_oo`03003ooooo02go
o`03003ooooo0=;oo`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool0;?oo00<00?oo
ool0dooo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0:_oo0P00e_oo0007ool0
1000ooooo`002Ooo00<00?ooool01?oo0P00:_oo00<00?ooool0e_oo00000ooo000000020002ool0
1000ooooo`002_oo00<00?ooool00ooo00<00?ooool0:?oo00<00?ooool0eooo0007ool01000oooo
o`002ooo00<00?ooool00_oo00<00?ooool09ooo00<00?ooool0f?oo0007ool01000ooooo`002?oo
00@00?ooool000Coo`03003ooooo02Koo`03003ooooo0=Woo`002?oo0P002_oo0P001Ooo00<00?oo
ool09Ooo00<00?ooool0f_oo000Kool00`00ooooo`0Tool00`00ooooo`3Kool001_oo`8002Coo`03
003ooooo0=coo`006ooo00<00?ooool08_oo00<00?ooool0gOoo000Kool00`00ooooo`0Qool00`00
ooooo`3Nool001_oo`03003ooooo023oo`03003ooooo0=ooo`006ooo00<00?ooool07ooo00<00?oo
ool0h?oo000Kool00`00ooooo`0Nool00`00ooooo`3Qool001_oo`8001koo`03003ooooo0>;oo`00
6ooo00<00?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Lool00`00ooooo`3Sool001_o
o`03003ooooo01_oo`03003ooooo0>Coo`006ooo00<00?ooool06_oo00<00?ooool0iOoo000Kool0
0`00ooooo`0Jool00`00ooooo`3Uool001_oo`03003ooooo01Woo`03003ooooo0>Koo`006ooo0P00
6Ooo00<00?ooool0iooo000Kool00`00ooooo`0Hool00`00ooooo`3Wool001_oo`03003ooooo01Oo
o`03003ooooo0>Soo`006ooo00<00?ooool05_oo00<00?ooool0jOoo000Kool00`00ooooo`0Eool0
0`00ooooo`3Zool001_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo0P005Ooo00<00?ooool0
jooo000Kool00`00ooooo`0Cool00`00ooooo`3/ool001_oo`03003ooooo01?oo`03003ooooo0>co
o`006ooo00<00?ooool04_oo00<00?ooool0kOoo000Kool00`00ooooo`0Aool00`00ooooo`3^ool0
00Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo017oo`03003ooooo0>koo`001ooo00@0
0?ooool000[oo`03003ooooo00?oo`03003ooooo013oo`03003ooooo0>ooo`001ooo00@00?ooool0
00[oo`03003ooooo00?oo`80013oo`03003ooooo0?3oo`0000?oo`0000000P000_oo00@00?ooool0
00[oo`03003ooooo00?oo`03003ooooo00ooo`03003ooooo0?3oo`001ooo00@00?ooool000[oo`03
003ooooo00?oo`03003ooooo00koo`03003ooooo0?7oo`001ooo00@00?ooool000Soo`<000Goo`03
003ooooo00koo`03003ooooo0?7oo`002?oo0P002ooo00<00?ooool00ooo00<00?ooool03Ooo00<0
0?ooool0l_oo000Kool00`00ooooo`0<ool00`00ooooo`3cool001_oo`8000goo`03003ooooo0??o
o`006ooo00<00?ooool02ooo00<00?ooool0m?oo000Kool00`00ooooo`0:ool00`00ooooo`3eool0
01_oo`03003ooooo00[oo`03003ooooo0?Goo`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000K
ool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo
0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo
00Koo`03003ooooo0?Woo`006ooo00<00?ooool01_oo00<00?ooool0nOoo000Kool00`00ooooo`05
ool00`00ooooo`3jool001_oo`03003ooooo00Goo`03003ooooo0?[oo`006ooo0P001Ooo00<00?oo
ool0nooo000Kool00`00ooooo`04ool00`00ooooo`3kool001_oo`03003ooooo00?oo`03003ooooo
0?coo`006ooo00<00?ooool00ooo00<00?ooool04_oo0P001?oo00<00?ooool00_oo10004_oo0P00
1?oo00<00?ooool01?oo00<00?ooool04_oo0P001?oo00<00?ooool00ooo0P005?oo0P001?oo00<0
0?ooool00ooo0P006Ooo10005ooo10000ooo00<00?ooool00_oo10004_oo10000ooo00<00?ooool0
1?oo00<00?ooool04?oo000Kool00`00ooooo`02ool00`00ooooo`0Bool01000ooooo`002?oo00<0
0?ooool04_oo00@00?ooool000[oo`03003ooooo017oo`04003ooooo0008ool01000ooooo`004_oo
00@00?ooool000Soo`04003ooooo000Jool00`00ooooo`0Hool00`00ooooo`07ool00`00ooooo`0E
ool00`00ooooo`09ool00`00ooooo`0@ool001_oo`03003ooooo00;oo`03003ooooo01;oo`04003o
oooo0009ool00`00ooooo`0Aool01000ooooo`001ooo1@004_oo00@00?ooool000Soo`04003ooooo
000Bool01000ooooo`002?oo00@00?ooool001[oo`03003ooooo01Soo`03003ooooo00Soo`03003o
oooo01Coo`03003ooooo00Koo`D0017oo`006ooo00D00?ooooooo`0001Goo`04003ooooo000:ool0
0`00ooooo`0@ool01000ooooo`001ooo00@00?ooool001?oo`04003ooooo0008ool3000Cool01000
ooooo`002Ooo0P006ooo00<00?ooool06?oo00<00?ooool02Ooo00<00?ooool04ooo00<00?ooool0
1_oo00@00?ooool001;oo`006ooo0P000_oo00<00?ooool04ooo00@00?ooool000_oo`03003ooooo
00ooo`04003ooooo0008ool00`00ool0000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?oo
ool000Soo`04003ooooo000Jool00`00ooooo`0Hool00`00ooooo`0:ool00`00ooooo`0Bool00`00
ooooo`07ool00`00ool0000Bool001_oo`04003ooooo000Fool01000ooooo`002?oo00@00?ooool0
017oo`04003ooooo0009ool2000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool000So
o`04003ooooo000Hool3000Hool30009ool01000ooooo`004_oo0`002_oo0P004_oo000Kool01000
ooooo`005ooo0P002_oo0P004ooo0P002ooo00<00?ooool04_oo0P002_oo0`004ooo0P002_oo0P00
6ooo00<00?ooool06?oo00<00?ooool02?oo0P005Ooo00<00?ooool02Ooo00<00?ooool04?oo000K
ool00`00ool0003oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000D
oooo000=000001_oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Goo`006ooo00<00?ooool07ooo00<00?ooool07_oo00<00?oo
ool07ooo00<00?ooool07ooo00<00?ooool07ooo00<00?ooool07_oo00<00?ooool07ooo00<00?oo
ool05_oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, -0.514613, \
0.00592092, 0.00308603}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9994820046162144`, 
      mGamma \[Rule] \(-0.3707184166384133`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.001, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.7507348023467588`, 
      mGamma \[Rule] \(-0.0010167970670121491`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.001, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.7507331914045487`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[0.7507331914045487`, 0.001, 10]\)], "Input"],

Cell[BoxData[
    \(2.0805037335362946`*^-9 + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[
      0.7507352528464437476 + I \((\(-0.001016796662459896262083\))\), 0.001, 
      10]\)], "Input"],

Cell[BoxData[
    \(9.931444982616892`*^-22 + 
      7.905253850325371`*^-20\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["xi=3", "Subsubsection"],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, k*0.9, 
          k*0.92}, {mGamma, \(-0.6\), \(-0.75\)}, 
        MaxIterations \[Rule] 150]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.5, 3]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.143564371623242`, 
      mGamma \[Rule] \(-0.8920118251514229`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.2, 3]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.1568984358901682`, 
      mGamma \[Rule] \(-0.7494633962660274`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.18, 3]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.157398960815601`, 
      mGamma \[Rule] \(-0.74096356721166`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.1579, 3]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.1578279943863556`, 
      mGamma \[Rule] \(-0.7317313460362467`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, Om /. strangesol[k, 3]}, {k, 1.158, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-3.20092 2.78474 -1.87372 1.65136 [
[.00153 .09541 -12 -9 ]
[.00153 .09541 12 0 ]
[.28001 .09541 -12 -9 ]
[.28001 .09541 12 0 ]
[.41924 .09541 -9 -9 ]
[.41924 .09541 9 0 ]
[.55848 .09541 -12 -9 ]
[.55848 .09541 12 0 ]
[.69772 .09541 -9 -9 ]
[.69772 .09541 9 0 ]
[.83695 .09541 -12 -9 ]
[.83695 .09541 12 0 ]
[.97619 .09541 -9 -9 ]
[.97619 .09541 9 0 ]
[.12827 .02534 -24 -4.5 ]
[.12827 .02534 0 4.5 ]
[.12827 .19048 -24 -4.5 ]
[.12827 .19048 0 4.5 ]
[.12827 .27305 -18 -4.5 ]
[.12827 .27305 0 4.5 ]
[.12827 .35561 -24 -4.5 ]
[.12827 .35561 0 4.5 ]
[.12827 .43818 -18 -4.5 ]
[.12827 .43818 0 4.5 ]
[.12827 .52075 -24 -4.5 ]
[.12827 .52075 0 4.5 ]
[.12827 .60332 -18 -4.5 ]
[.12827 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.00153 .10791 m
.00153 .11416 L
s
[(1.15)] .00153 .09541 0 1 Mshowa
.28001 .10791 m
.28001 .11416 L
s
[(1.25)] .28001 .09541 0 1 Mshowa
.41924 .10791 m
.41924 .11416 L
s
[(1.3)] .41924 .09541 0 1 Mshowa
.55848 .10791 m
.55848 .11416 L
s
[(1.35)] .55848 .09541 0 1 Mshowa
.69772 .10791 m
.69772 .11416 L
s
[(1.4)] .69772 .09541 0 1 Mshowa
.83695 .10791 m
.83695 .11416 L
s
[(1.45)] .83695 .09541 0 1 Mshowa
.97619 .10791 m
.97619 .11416 L
s
[(1.5)] .97619 .09541 0 1 Mshowa
.125 Mabswid
.02938 .10791 m
.02938 .11166 L
s
.05723 .10791 m
.05723 .11166 L
s
.08507 .10791 m
.08507 .11166 L
s
.11292 .10791 m
.11292 .11166 L
s
.16862 .10791 m
.16862 .11166 L
s
.19646 .10791 m
.19646 .11166 L
s
.22431 .10791 m
.22431 .11166 L
s
.25216 .10791 m
.25216 .11166 L
s
.30785 .10791 m
.30785 .11166 L
s
.3357 .10791 m
.3357 .11166 L
s
.36355 .10791 m
.36355 .11166 L
s
.3914 .10791 m
.3914 .11166 L
s
.44709 .10791 m
.44709 .11166 L
s
.47494 .10791 m
.47494 .11166 L
s
.50278 .10791 m
.50278 .11166 L
s
.53063 .10791 m
.53063 .11166 L
s
.58633 .10791 m
.58633 .11166 L
s
.61417 .10791 m
.61417 .11166 L
s
.64202 .10791 m
.64202 .11166 L
s
.66987 .10791 m
.66987 .11166 L
s
.72556 .10791 m
.72556 .11166 L
s
.75341 .10791 m
.75341 .11166 L
s
.78126 .10791 m
.78126 .11166 L
s
.80911 .10791 m
.80911 .11166 L
s
.8648 .10791 m
.8648 .11166 L
s
.89265 .10791 m
.89265 .11166 L
s
.9205 .10791 m
.9205 .11166 L
s
.94834 .10791 m
.94834 .11166 L
s
.25 Mabswid
0 .10791 m
1 .10791 L
s
.14077 .02534 m
.14702 .02534 L
s
[(1.15)] .12827 .02534 1 0 Mshowa
.14077 .19048 m
.14702 .19048 L
s
[(1.25)] .12827 .19048 1 0 Mshowa
.14077 .27305 m
.14702 .27305 L
s
[(1.3)] .12827 .27305 1 0 Mshowa
.14077 .35561 m
.14702 .35561 L
s
[(1.35)] .12827 .35561 1 0 Mshowa
.14077 .43818 m
.14702 .43818 L
s
[(1.4)] .12827 .43818 1 0 Mshowa
.14077 .52075 m
.14702 .52075 L
s
[(1.45)] .12827 .52075 1 0 Mshowa
.14077 .60332 m
.14702 .60332 L
s
[(1.5)] .12827 .60332 1 0 Mshowa
.125 Mabswid
.14077 .04186 m
.14452 .04186 L
s
.14077 .05837 m
.14452 .05837 L
s
.14077 .07488 m
.14452 .07488 L
s
.14077 .0914 m
.14452 .0914 L
s
.14077 .12442 m
.14452 .12442 L
s
.14077 .14094 m
.14452 .14094 L
s
.14077 .15745 m
.14452 .15745 L
s
.14077 .17397 m
.14452 .17397 L
s
.14077 .20699 m
.14452 .20699 L
s
.14077 .22351 m
.14452 .22351 L
s
.14077 .24002 m
.14452 .24002 L
s
.14077 .25653 m
.14452 .25653 L
s
.14077 .28956 m
.14452 .28956 L
s
.14077 .30607 m
.14452 .30607 L
s
.14077 .32259 m
.14452 .32259 L
s
.14077 .3391 m
.14452 .3391 L
s
.14077 .37213 m
.14452 .37213 L
s
.14077 .38864 m
.14452 .38864 L
s
.14077 .40516 m
.14452 .40516 L
s
.14077 .42167 m
.14452 .42167 L
s
.14077 .4547 m
.14452 .4547 L
s
.14077 .47121 m
.14452 .47121 L
s
.14077 .48772 m
.14452 .48772 L
s
.14077 .50424 m
.14452 .50424 L
s
.14077 .53726 m
.14452 .53726 L
s
.14077 .55378 m
.14452 .55378 L
s
.14077 .57029 m
.14452 .57029 L
s
.14077 .58681 m
.14452 .58681 L
s
.14077 .00883 m
.14452 .00883 L
s
.25 Mabswid
.14077 0 m
.14077 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .03855 m
.06244 .06146 L
.10458 .08645 L
.14415 .10992 L
.18221 .13248 L
.22272 .15651 L
.26171 .17963 L
.30316 .20421 L
.34309 .22789 L
.3815 .25066 L
.42237 .2749 L
.46172 .29823 L
.49955 .32067 L
.53984 .34456 L
.57861 .36755 L
.61984 .392 L
.65954 .41555 L
.69774 .43819 L
.73838 .4623 L
.77751 .4855 L
.81909 .51016 L
.85916 .53392 L
.89771 .55678 L
.93871 .5811 L
.97619 .60332 L
s
.02381 .03827 m
.06244 .03785 L
.10458 .03729 L
.14415 .03668 L
.18221 .03602 L
.22272 .03524 L
.26171 .03443 L
.30316 .03351 L
.34309 .03257 L
.3815 .03162 L
.42237 .03057 L
.46172 .02953 L
.49955 .0285 L
.53984 .02737 L
.57861 .02628 L
.61984 .02509 L
.65954 .02394 L
.69774 .02283 L
.73838 .02164 L
.77751 .02049 L
.81909 .01927 L
.85916 .0181 L
.89771 .01698 L
.93871 .01579 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00<_oo00<00?ooool0jooo000bool00`00ooooo`3[ool003;oo`03003ooooo0>_o
o`00<_oo0P00k?oo000bool00`00ooooo`3Lool60009ool001Ooo`@000?oo`03003ooooo00;oo`@0
00;oo`<000Koo`03003ooooo0;coob0000ooo`006Ooo00<00?ooool02Ooo00<00?ooool01?oo00<0
0?ooool00ooo00<00?ooool0Vooo8@00;ooo000Iool00`00ooooo`09ool00`00ooooo`04ool00`00
ooooo`03ool2001koolQ001@ool001Woo`03003ooooo00Woo`04003oooooool30006ool00`00oooo
o`1IoolQ001aool001Woo`03003ooooo00Woo`05003oooooool00008ool00`00ooooo`0_oolZ002B
ool001Ooo`<000Woo`<000?oo`04003ooooooolg002lool001;ooa/000Goo`03003ooooo0>_oo`00
4ooo0P007Ooo0P00k?oo000Eool2000Kool00`00ooooo`3[ool001Ooo`03003ooooo01Soo`03003o
oooo0>_oo`006?oo0P006?oo00<00?ooool0jooo000Jool2000Fool2003/ool001coo`03003ooooo
01?oo`03003ooooo0>_oo`007Ooo0P004ooo00<00?ooool0jooo00000ooo000000020003ool00`00
ooooo`02ool40002ool30009ool2000Aool00`00ooooo`0Hool40003ool00`00ooooo`02ool40002
ool3000Dool40003ool00`00ooooo`03ool2000Dool40003ool00`00ooooo`03ool20003ool3000D
ool40003ool00`00ooooo`04ool00`00ooooo`0Bool40003ool00`00ooooo`04ool01000oooooooo
0`005?oo10000ooo00<00?ooool00_oo0`000ooo0003ool00`00ooooo`09ool00`00ooooo`04ool0
0`00ooooo`08ool00`00ooooo`0>ool00`00ooooo`0Jool00`00ooooo`07ool00`00ooooo`06ool0
0`00ooooo`0Cool00`00ooooo`07ool01000ooooo`005Ooo00<00?ooool01ooo00@00?ooool000Go
o`03003ooooo01?oo`03003ooooo00Woo`03003ooooo01Coo`03003ooooo00Woo`03003ooooo00Co
o`03003ooooo01?oo`03003ooooo00[oo`40007oo`7oo`000ooo00<00?ooool02Ooo00<00?ooool0
1?oo00<00?ooool02Ooo0P003_oo0P006ooo00<00?ooool02?oo00<00?ooool01Ooo00<00?ooool0
4ooo00<00?ooool02_oo00<00?ooool04ooo00<00?ooool02_oo00<00?ooool00ooo00<00?ooool0
4ooo00<00?ooool01_oo1@005Ooo00<00?ooool01_oo1@001Ooo00<00?ooool04ooo00<00?ooool0
2_oo0@000Ooo0Ooo0003ool00`00ooooo`09ool01000oooooooo0`003_oo0P003?oo00<00?ooool0
6_oo00<00?ooool02Ooo00@00?ooooooo`<001Koo`03003ooooo00Soo`8001Koo`03003ooooo00So
o`8000?oo`<001Koo`03003ooooo00Koo`04003ooooo000Fool00`00ooooo`06ool01000ooooo`00
0ooo0`005_oo00<00?ooool01ooo0`000ooo0003ool00`00ooooo`09ool01@00oooooooo00004_oo
0P002_oo00<00?ooool06_oo00<00?ooool02_oo00@00?ooool001Soo`03003ooooo00[oo`03003o
oooo01?oo`03003ooooo00[oo`04003ooooo000Hool00`00ooooo`07ool00`00ool0000Fool00`00
ooooo`07ool00`00ool00003ool00`00ooooo`0Fool00`00ooooo`07ool00`00ooooo`03ool00004
ool000000009ool30003ool00`00ooooo`0Bool00`00ooooo`07ool00`00ooooo`0Hool30009ool0
1000ooooo`000_oo00<00?ooool05?oo0`002Ooo00@00?ooool001?oo`<000Woo`04003ooooo0002
ool00`00ooooo`0Dool3000:ool2000Dool3000:ool20003ool00`00ooooo`0Dool30009ool00`00
ooooo`03ool000?oo`03003ooooo00Woo`04003oooooool4000Bool20007ool2000Kool00`00oooo
o`08ool20003ool4000Eool00`00ooooo`08ool2000Fool00`00ooooo`08ool20003ool4000Eool0
0`00ooooo`09ool00`00ooooo`0Dool00`00ooooo`09ool01000oooooooo10005Ooo00<00?ooool0
1ooo10000_oo000[ool20005ool00`00ooooo`3[ool002goo`03003ooooo00;oo`03003ooooo0>_o
o`00;_oo0P000_oo00<00?ooool0jooo000`ool3003]ool000_oool001<000?oo`003?oo00<00?oo
ool01?oo00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo0`001?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01?oo
00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01ooo
000<ool00`00ooooo`0Sool01000ooooo`008_oo00<00?ooool08ooo00<00?ooool08ooo00<00?oo
ool08ooo00<00?ooool08ooo00<00?ooool08ooo00<00?ooool01ooo000bool01000oooooooo0P00
j?oo000bool20004ool2003Vool003;oo`03003ooooo00Goo`03003ooooo0>?oo`00<_oo00<00?oo
ool01_oo0P00hooo000bool00`00ooooo`08ool00`00ooooo`3Pool003;oo`03003ooooo00Woo`80
0>3oo`00<_oo0P003?oo0P00g_oo000bool00`00ooooo`0=ool2003Lool003;oo`03003ooooo00oo
o`800=[oo`00<_oo00<00?ooool04Ooo0P00f?oo000bool2000Dool00`00ooooo`3Eool003;oo`03
003ooooo01Coo`800=Goo`00<_oo00<00?ooool05_oo0P00dooo000bool00`00ooooo`0Hool2003A
ool003;oo`03003ooooo01[oo`800<ooo`00<_oo0P007Ooo0P00cOoo000bool00`00ooooo`0Nool0
0`00ooooo`3:ool001Ooo`@000?oo`03003ooooo00;oo`@000;oo`<000Koo`03003ooooo01ooo`80
0<[oo`006Ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool00ooo00<00?ooool08Ooo00<0
0?ooool0aooo000Iool00`00ooooo`08ool00`00ooooo`05ool00`00ooooo`03ool2000Sool20037
ool001Woo`03003ooooo00Woo`04003oooooool30006ool00`00ooooo`0Tool20035ool001Woo`03
003ooooo00[oo`04003ooooo0008ool00`00ooooo`0Vool00`00ooooo`32ool001Ooo`<000Woo`04
003ooooo0002ool00`00ooooo`06ool00`00ooooo`0Wool20032ool001Woo`03003ooooo00Soo`80
00?oo`@000Goo`03003ooooo02Woo`03003ooooo0;ooo`00<_oo0P00:ooo0P00_ooo000bool00`00
ooooo`0/ool00`00ooooo`2lool003;oo`03003ooooo02goo`800;coo`00<_oo00<00?ooool0;ooo
0P00^_oo000bool2000bool00`00ooooo`2gool003;oo`03003ooooo03;oo`800;Ooo`00<_oo00<0
0?ooool0=?oo00<00?ooool0]?oo000bool00`00ooooo`0eool2002dool003;oo`03003ooooo03Oo
o`800;;oo`00<_oo0P00>_oo00<00?ooool0[ooo000bool00`00ooooo`0jool2002_ool003;oo`03
003ooooo03coo`800:goo`00<_oo00<00?ooool0?_oo00<00?ooool0Z_oo000bool20010ool2002Z
ool003;oo`03003ooooo047oo`800:Soo`00<_oo00<00?ooool0@ooo0P00Y_oo000Mool40003ool0
0`00ooooo`03ool20006ool00`00ooooo`15ool2002Tool001ooo`03003ooooo00Ooo`04003ooooo
0005ool00`00ooooo`17ool2002Rool001ooo`03003ooooo00[oo`03003ooooo00?oo`8004[oo`03
003ooooo09ooo`007ooo00<00?ooool02?oo0P001_oo00<00?ooool0B_oo0P00Wooo000Oool00`00
ooooo`0:ool00`00ooooo`03ool00`00ooooo`1<ool00`00ooooo`2Lool001goo`<000Woo`04003o
oooo0005ool00`00ooooo`1=ool2002Lool001ooo`03003ooooo00Soo`8000Koo`80053oo`8009[o
o`00<_oo00<00?ooool0DOoo00<00?ooool0Uooo000bool00`00ooooo`1Bool2002Gool003;oo`03
003ooooo05Coo`03003ooooo09Coo`00<_oo00<00?ooool0EOoo0P00U?oo000bool2001Hool2002B
ool003;oo`03003ooooo05Woo`03003ooooo08ooo`00<_oo00<00?ooool0F_oo0P00Sooo000bool0
0`00ooooo`1Lool2002=ool003;oo`8005ooo`03003ooooo08[oo`00<_oo00<00?ooool0Gooo0P00
R_oo000bool00`00ooooo`1Qool20028ool003;oo`03003ooooo06?oo`8008Koo`00<_oo00<00?oo
ool0IOoo0P00Q?oo000bool2001Xool20022ool003;oo`03003ooooo06Woo`03003ooooo07ooo`00
<_oo00<00?ooool0J_oo0P00Oooo000Gool40003ool00`00ooooo`03ool20003ool30006ool00`00
ooooo`1/ool00`00ooooo`1lool001Woo`03003ooooo00Ooo`04003ooooo0005ool00`00ooooo`03
ool00`00ooooo`1]ool2001lool001Woo`03003ooooo00[oo`03003ooooo00?oo`03003ooooo00?o
o`80073oo`8007[oo`006Ooo00<00?ooool02?oo0P000ooo0`001_oo00<00?ooool0LOoo00<00?oo
ool0Mooo000Iool00`00ooooo`0:ool01000ooooo`002?oo00<00?ooool0L_oo0P00Mooo000Gool3
0009ool01000ooooo`000_oo00<00?ooool01_oo00<00?ooool0M?oo00<00?ooool0M?oo000Iool0
0`00ooooo`08ool20003ool40005ool2001fool2001dool003;oo`03003ooooo07Ooo`8007;oo`00
<_oo00<00?ooool0NOoo0P00L?oo000bool00`00ooooo`1kool2001^ool003;oo`03003ooooo07go
o`8006coo`00<_oo0P00P?oo00<00?ooool0JOoo000bool00`00ooooo`20ool2001Yool003;oo`03
003ooooo08;oo`03003ooooo06Koo`00<_oo00<00?ooool0Pooo0P00I_oo000bool20026ool2001T
ool003;oo`03003ooooo08Ooo`03003ooooo067oo`00<_oo00<00?ooool0R?oo0P00HOoo000bool0
0`00ooooo`2:ool00`00ooooo`1Nool003;oo`03003ooooo08_oo`8005koo`00<_oo0P00S_oo0P00
G?oo000bool00`00ooooo`2?ool2001Jool001goo`@000?oo`03003ooooo00Coo`03003ooooo00Co
o`03003ooooo097oo`8005Soo`007ooo00<00?ooool02Ooo00<00?ooool01?oo00<00?ooool0Tooo
0P00E_oo000Oool00`00ooooo`06ool50005ool2002Fool00`00ooooo`1Cool001ooo`03003ooooo
00Koo`04003ooooo0006ool00`00ooooo`2Fool2001Cool001ooo`03003ooooo00Ooo`03003oo`00
00Koo`03003ooooo09Soo`03003ooooo053oo`007Ooo0`002_oo0P001_oo00<00?ooool0VOoo0P00
D?oo000Oool00`00ooooo`09ool00`00ooooo`04ool00`00ooooo`2Kool2001>ool003;oo`8009ko
o`03003ooooo04_oo`00<_oo00<00?ooool0W_oo0P00Booo000bool00`00ooooo`2Pool00`00oooo
o`18ool003;oo`03003ooooo0:7oo`8004Soo`00<_oo0P00Y?oo0P00A_oo000bool00`00ooooo`2U
ool00`00ooooo`13ool003;oo`03003ooooo0:Koo`8004?oo`00<_oo00<00?ooool0Z?oo0P00@Ooo
000bool00`00ooooo`2Zool00`00ooooo`0nool003;oo`800:coo`8003koo`00<_oo00<00?ooool0
[Ooo0P00??oo000bool00`00ooooo`2_ool00`00ooooo`0iool003;oo`03003ooooo0;3oo`8003Wo
o`00<_oo0P00/ooo0P00=ooo000bool00`00ooooo`2dool2000eool003;oo`03003ooooo0;Koo`03
003ooooo03;oo`005ooo10000ooo00<00?ooool01?oo00@00?ooooooo`<000Koo`03003ooooo0;Oo
o`8003;oo`006Ooo00<00?ooool02Ooo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0^Ooo
0P00<?oo000Iool00`00ooooo`06ool50005ool00`00ooooo`03ool2002lool2000^ool001Woo`03
003ooooo00Koo`04003ooooo0003ool30006ool00`00ooooo`2mool2000/ool001Woo`03003ooooo
00Ooo`03003oo`0000?oo`03003ooooo00Koo`03003ooooo0;ooo`8002[oo`005ooo0`002_oo0P00
0ooo00<00?ooool01_oo00<00?ooool0`Ooo00<00?ooool09ooo000Iool00`00ooooo`09ool01000
oooooooo10001Ooo0P00`ooo0P009ooo000bool00`00ooooo`34ool2000Uool003;oo`03003ooooo
0<Koo`03003ooooo02;oo`00<_oo00<00?ooool0aooo0P008_oo000bool00`00ooooo`39ool2000P
ool003;oo`800<coo`03003ooooo01goo`00<_oo00<00?ooool0c?oo0P007Ooo000bool00`00oooo
o`3>ool00`00ooooo`0Jool003;oo`03003ooooo0<ooo`8001[oo`00<_oo0P00d_oo0P006?oo000b
ool00`00ooooo`3Cool00`00ooooo`0Eool003;oo`03003ooooo0=Coo`8001Goo`00<_oo00<00?oo
ool0e_oo00<00?ooool04_oo000bool00`00ooooo`3Gool2000Bool003;oo`800=[oo`80013oo`00
<_oo00<00?ooool0fooo0P003_oo000Mool40003ool00`00ooooo`02ool30006ool00`00ooooo`3M
ool2000<ool001ooo`03003ooooo00[oo`03003ooooo00?oo`03003ooooo0=ooo`8000[oo`007ooo
00<00?ooool02_oo00<00?ooool00ooo0P00h_oo00<00?ooool01ooo000Oool00`00ooooo`07ool3
0006ool00`00ooooo`3[ool001ooo`03003ooooo00Ooo`03003ooooo00Koo`03003ooooo0>_oo`00
7Ooo0`002Ooo00<00?ooool01_oo00<00?ooool0jooo000Oool00`00ooooo`07ool40005ool00`00
ooooo`3[ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {1.13421, 1.12602, 0.0013158, \
0.00221888}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 1, 
          1.1}, {mGamma, \(-0.7\), \(-0.9\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.1, 3]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \({Om \[Rule] 1.1035468759640183`, 
      mGamma \[Rule] \(-0.6825439118939567`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[1.1705547647282446 - I\ 0.6941242019418665, 1.1, 
      3]\)], "Input"],

Cell[BoxData[
    \(\(-1.1333043603593906`\) - 
      1.4215316459103413`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[kx0corrtestfunc[Om - I\ 0.72, 1.1, 3]], {Om, 0.8, 1.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.5 1.90476 0.276409 0.193495 [
[.02381 .26391 -9 -9 ]
[.02381 .26391 9 0 ]
[.21429 .26391 -9 -9 ]
[.21429 .26391 9 0 ]
[.59524 .26391 -9 -9 ]
[.59524 .26391 9 0 ]
[.78571 .26391 -9 -9 ]
[.78571 .26391 9 0 ]
[.97619 .26391 -9 -9 ]
[.97619 .26391 9 0 ]
[.39226 .08291 -12 -4.5 ]
[.39226 .08291 0 4.5 ]
[.39226 .17966 -24 -4.5 ]
[.39226 .17966 0 4.5 ]
[.39226 .37316 -18 -4.5 ]
[.39226 .37316 0 4.5 ]
[.39226 .4699 -6 -4.5 ]
[.39226 .4699 0 4.5 ]
[.39226 .56665 -18 -4.5 ]
[.39226 .56665 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .27641 m
.02381 .28266 L
s
[(0.8)] .02381 .26391 0 1 Mshowa
.21429 .27641 m
.21429 .28266 L
s
[(0.9)] .21429 .26391 0 1 Mshowa
.59524 .27641 m
.59524 .28266 L
s
[(1.1)] .59524 .26391 0 1 Mshowa
.78571 .27641 m
.78571 .28266 L
s
[(1.2)] .78571 .26391 0 1 Mshowa
.97619 .27641 m
.97619 .28266 L
s
[(1.3)] .97619 .26391 0 1 Mshowa
.125 Mabswid
.0619 .27641 m
.0619 .28016 L
s
.1 .27641 m
.1 .28016 L
s
.1381 .27641 m
.1381 .28016 L
s
.17619 .27641 m
.17619 .28016 L
s
.25238 .27641 m
.25238 .28016 L
s
.29048 .27641 m
.29048 .28016 L
s
.32857 .27641 m
.32857 .28016 L
s
.36667 .27641 m
.36667 .28016 L
s
.44286 .27641 m
.44286 .28016 L
s
.48095 .27641 m
.48095 .28016 L
s
.51905 .27641 m
.51905 .28016 L
s
.55714 .27641 m
.55714 .28016 L
s
.63333 .27641 m
.63333 .28016 L
s
.67143 .27641 m
.67143 .28016 L
s
.70952 .27641 m
.70952 .28016 L
s
.74762 .27641 m
.74762 .28016 L
s
.82381 .27641 m
.82381 .28016 L
s
.8619 .27641 m
.8619 .28016 L
s
.9 .27641 m
.9 .28016 L
s
.9381 .27641 m
.9381 .28016 L
s
.25 Mabswid
0 .27641 m
1 .27641 L
s
.40476 .08291 m
.41101 .08291 L
s
[(-1)] .39226 .08291 1 0 Mshowa
.40476 .17966 m
.41101 .17966 L
s
[(-0.5)] .39226 .17966 1 0 Mshowa
.40476 .37316 m
.41101 .37316 L
s
[(0.5)] .39226 .37316 1 0 Mshowa
.40476 .4699 m
.41101 .4699 L
s
[(1)] .39226 .4699 1 0 Mshowa
.40476 .56665 m
.41101 .56665 L
s
[(1.5)] .39226 .56665 1 0 Mshowa
.125 Mabswid
.40476 .10226 m
.40851 .10226 L
s
.40476 .12161 m
.40851 .12161 L
s
.40476 .14096 m
.40851 .14096 L
s
.40476 .16031 m
.40851 .16031 L
s
.40476 .19901 m
.40851 .19901 L
s
.40476 .21836 m
.40851 .21836 L
s
.40476 .23771 m
.40851 .23771 L
s
.40476 .25706 m
.40851 .25706 L
s
.40476 .29576 m
.40851 .29576 L
s
.40476 .31511 m
.40851 .31511 L
s
.40476 .33446 m
.40851 .33446 L
s
.40476 .35381 m
.40851 .35381 L
s
.40476 .39251 m
.40851 .39251 L
s
.40476 .41186 m
.40851 .41186 L
s
.40476 .43121 m
.40851 .43121 L
s
.40476 .45055 m
.40851 .45055 L
s
.40476 .48925 m
.40851 .48925 L
s
.40476 .5086 m
.40851 .5086 L
s
.40476 .52795 m
.40851 .52795 L
s
.40476 .5473 m
.40851 .5473 L
s
.40476 .06356 m
.40851 .06356 L
s
.40476 .04421 m
.40851 .04421 L
s
.40476 .02486 m
.40851 .02486 L
s
.40476 .00552 m
.40851 .00552 L
s
.40476 .586 m
.40851 .586 L
s
.40476 .60535 m
.40851 .60535 L
s
.25 Mabswid
.40476 0 m
.40476 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .55858 L
.10458 .51547 L
.14415 .47976 L
.18221 .44928 L
.22272 .42055 L
.26171 .39614 L
.30316 .37331 L
.34309 .35404 L
.3815 .33778 L
.42237 .32271 L
.46172 .31015 L
.49955 .29973 L
.53984 .29026 L
.55838 .28643 L
.57861 .2826 L
.58392 .28166 L
.58957 .28068 L
.59232 .28021 L
.59367 .27998 L
.59491 .27978 L
.59604 .01472 L
.59723 .01502 L
.59975 .01565 L
.61984 .02071 L
.65781 .03036 L
.69823 .04074 L
.73714 .05084 L
.7785 .06168 L
.81835 .07223 L
.85668 .08249 L
.89746 .09351 L
.93672 .10425 L
.97448 .11467 L
.97619 .11514 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{44, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo001dool0
0`00ooooo`2Yool007Coo`800:[oo`00M?oo00<00?ooool0ZOoo001dool00`00ooooo`2Yool007Co
o`03003ooooo03?oo`<007?oo`00M?oo00<00?ooool0<ooo00<00?ooool00`00L?oo001dool00`00
ooooo`0cool00`00ooooo`03ool4001/ool007Coo`8003Coo`03003ooooo00Ooo`H006Koo`00M?oo
00<00?ooool0<ooo00<00?ooool03Ooo1000H_oo001dool00`00ooooo`0cool00`00ooooo`0Aool4
001Nool007Coo`03003ooooo03?oo`03003ooooo01Goo`@005[oo`00M?oo00<00?ooool0<ooo00<0
0?ooool06Ooo0`00Eooo001dool2000dool00`00ooooo`0Lool4001Cool007Coo`03003ooooo03?o
o`03003ooooo023oo`@004ooo`00M?oo00<00?ooool0<ooo00<00?ooool09?oo0`00C?oo001dool0
0`00ooooo`0cool00`00ooooo`0Wool40018ool007Coo`03003ooooo03?oo`03003ooooo02_oo`@0
04Coo`00M?oo00<00?ooool0<ooo00<00?ooool0;ooo1000@?oo001dool2000dool00`00ooooo`0c
ool4000lool007Coo`03003ooooo03?oo`03003ooooo03Ooo`@003Soo`00M?oo00<00?ooool0<ooo
00<00?ooool0>ooo0`00=Ooo001/ool40004ool00`00ooooo`0cool00`00ooooo`0nool4000aool0
06koo`03003ooooo00?oo`03003ooooo03?oo`03003ooooo04;oo`@002goo`00K_oo00<00?ooool0
0ooo0`00<ooo00<00?ooool0A_oo0`00:_oo001Vool40004ool00`00ooooo`03ool00`00ooooo`0c
ool00`00ooooo`19ool4000Vool006koo`03003ooooo00?oo`03003ooooo03?oo`03003ooooo04go
o`@002;oo`00K?oo0`001Ooo00<00?ooool0<ooo00<00?ooool0DOoo0`007ooo001^ool00`00oooo
o`03ool00`00ooooo`0cool00`00ooooo`1Dool4000Kool007Coo`03003ooooo03?oo`03003ooooo
05Soo`@001Ooo`00M?oo0P00=?oo00<00?ooool0G?oo0`005?oo001dool00`00ooooo`0cool00`00
ooooo`1Oool4000@ool007Coo`03003ooooo03?oo`03003ooooo06?oo`@000coo`00M?oo00<00?oo
ool0<ooo00<00?ooool0Iooo0`002Ooo001dool00`00ooooo`0cool00`00ooooo`1cool007Coo`80
03Coo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001dool00`00ooooo`0c
ool00`00ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo
00<00?ooool0Looo001dool00`00ooooo`0cool00`00ooooo`1cool007Coo`8003Coo`03003ooooo
07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001dool00`00ooooo`0cool00`00ooooo`1c
ool007Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo
001dool2000dool00`00ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<0
0?ooool0<ooo00<00?ooool0Looo001Qool20004ool00`00ooooo`02ool30005ool00`00ooooo`0c
ool00`00ooooo`1cool0063oo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0cool00`00
ooooo`1cool0063oo`04003ooooo000;ool00`00ooooo`02ool3000cool00`00ooooo`1cool005[o
o`@000;oo`04003ooooo0008ool30005ool00`00ooooo`0cool00`00ooooo`1cool0063oo`04003o
oooo0008ool00`00ooooo`05ool00`00ooooo`0cool00`00ooooo`1cool0063oo`04003ooooo0008
ool00`00ooooo`05ool00`00ooooo`0cool00`00ooooo`1cool0067oo`8000Woo`@000Coo`03003o
oooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001dool2000dool0
0`00ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<0
0?ooool0Looo001dool00`00ooooo`0cool00`00ooooo`1cool007Coo`03003ooooo03?oo`03003o
oooo07?oo`00M?oo0P00=?oo00<00?ooool0Looo001dool00`00ooooo`0cool00`00ooooo`1cool0
07Coo`03003ooooo03?oo`03003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001d
ool00`00ooooo`0cool00`00ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`000_oo
0P001?oo00<00?ooool00ooo0P009ooo0P001?oo00<00?ooool00_oo0`00;ooo0P00;?oo10000ooo
0P000ooo10009_oo10000ooo00<00?ooool00_oo10009_oo10000ooo00<00?ooool00ooo0P000ooo
00001Ooo003ooooo00002?oo00@00?ooool002Goo`04003ooooo000;ool00`00ooooo`0/ool00`00
ooooo`0]ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`0Wool00`00ooooo`07ool00`00
ooooo`0Yool00`00ooooo`07ool01000ooooo`000_oo00001Ooo003ooooo00002?oo00@00?ooool0
02Goo`04003ooooo000;ool00`00ooooo`0/ool00`00ooooo`0]ool00`00ooooo`03ool00`00oooo
o`03ool00`00ooooo`0Wool00`00ooooo`08ool00`00ooooo`0Xool00`00ooooo`0:ool10001ool1
ool00005ool00?ooool00009ool2000Vool01000ooooo`002Ooo0`00;_oo00<00?ooool0;Ooo00<0
0?ooool00ooo00<00?ooool00ooo00<00?ooool09ooo00<00?ooool02Ooo00<00?ooool09ooo00<0
0?ooool02?oo0P000ooo00001Ooo003ooooo00002?oo00@00?ooool002Goo`04003ooooo0008ool0
1000ooooo`00;_oo00<00?ooool0;Ooo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool09ooo
00<00?ooool02_oo00<00?ooool09_oo00<00?ooool02_oo0@000Ooo0Ooo00001Ooo003ooooo0000
2?oo00@00?ooool002Goo`04003ooooo0008ool01000ooooo`00;_oo0P00;?oo0`001Ooo00@00?oo
ooooo`<002Ooo`<000Woo`04003ooooo000Vool30009ool01000ooooo`000_oo0002ool2000:ool2
000Wool2000:ool2000_ool00`00ooooo`0]ool00`00ooooo`03ool00`00ooooo`03ool00`00oooo
o`0Wool00`00ooooo`08ool2000Yool00`00ooooo`08ool20003ool007Coo`03003ooooo03?oo`03
003ooooo07?oo`00M?oo00<00?ooool0<ooo00<00?ooool0Looo001dool00`00ooooo`0cool00`00
ooooo`1cool007Coo`03003ooooo03?oo`03003ooooo07?oo`000_ooo`0070000ooo0009ool00`00
ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`04ool5000:
ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07
ool000Woo`03003ooooo03;oo`03003ooooo03?oo`03003ooooo02_oo`@000Coo`03003ooooo03?o
o`03003ooooo03?oo`03003ooooo00Ooo`00M?oo00<00?ooool09_oo1@00O_oo001dool00`00oooo
o`0Qool50023ool007Coo`8001coo`H008Soo`00M?oo00<00?ooool05ooo1000S_oo001dool00`00
ooooo`0Cool4002Bool007Coo`03003ooooo00ooo`@009Koo`00M?oo00<00?ooool03?oo0`00V_oo
001dool00`00ooooo`09ool3002Mool007Coo`8000Soo`800:3oo`00M?oo00<00?ooool01?oo0`00
X_oo001dool01000oooooooo0`00YOoo001dool4002Xool007?oo`800:_oo`00L?oo0`0000?oo`00
0000Z_oo001]ool30004ool00`00ooooo`2Yool006_oo`8000Ooo`03003ooooo0:Woo`00JOoo0P00
2Ooo00<00?ooool0ZOoo001Wool2000;ool00`00ooooo`2Yool006Goo`8000goo`03003ooooo0:Wo
o`00H_oo0`003ooo0P00Z_oo001Pool2000Bool00`00ooooo`2Yool005goo`<001Coo`03003ooooo
0:Woo`00Fooo0P001?oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0ZOoo001Iool20005
ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0ZOoo001Gool20007ool01000ooooo`00
2ooo00<00?ooool00_oo0`00ZOoo001Eool20009ool01000ooooo`002?oo0`001Ooo00<00?ooool0
ZOoo001Dool00`00ooooo`09ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0ZOoo001B
ool2000<ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0ZOoo001@ool2000?ool20009
ool40004ool00`00ooooo`2Yool004ooo`03003ooooo02;oo`03003ooooo0:Woo`00COoo0P009Ooo
0P00Z_oo001<ool00`00ooooo`0Uool00`00ooooo`2Yool004[oo`8002Soo`03003ooooo0:Woo`00
BOoo00<00?ooool0:?oo00<00?ooool0ZOoo0017ool2000[ool00`00ooooo`2Yool004Goo`8002go
o`800:[oo`00A?oo00<00?ooool0;Ooo00<00?ooool0ZOoo0012ool2000`ool00`00ooooo`2Yool0
047oo`03003ooooo033oo`03003ooooo0:Woo`00?ooo0P00<ooo00<00?ooool0ZOoo000nool00`00
ooooo`0cool00`00ooooo`2Yool003coo`8003Koo`800:[oo`00>ooo00<00?ooool0=_oo00<00?oo
ool0ZOoo000iool2000iool00`00ooooo`2Yool003Soo`03003ooooo03Woo`03003ooooo0:Woo`00
=_oo0P00??oo00<00?ooool0ZOoo000eool00`00ooooo`0lool2002Zool003?oo`8003ooo`03003o
oooo0:Woo`00<_oo00<00?ooool0?ooo00<00?ooool0ZOoo000aool00`00ooooo`0hool40004ool0
0`00ooooo`2Yool002ooo`8003goo`03003ooooo00?oo`03003ooooo0:Woo`00;_oo00<00?ooool0
?Ooo00<00?ooool00ooo0`00ZOoo000]ool00`00ooooo`0nool00`00ooooo`03ool00`00ooooo`2Y
ool002_oo`80047oo`03003ooooo00?oo`03003ooooo0:Woo`00:_oo00<00?ooool0?ooo0`001Ooo
00<00?ooool0ZOoo000Yool00`00ooooo`12ool00`00ooooo`03ool00`00ooooo`2Yool002Soo`03
003ooooo04Woo`03003ooooo0:Woo`009ooo00<00?ooool0B_oo0P00Z_oo000Vool00`00ooooo`1;
ool00`00ooooo`2Yool002Coo`8004koo`03003ooooo0:Woo`008ooo00<00?ooool0C_oo00<00?oo
ool0ZOoo000Rool00`00ooooo`1?ool00`00ooooo`2Yool0027oo`03003ooooo053oo`800:[oo`00
8?oo00<00?ooool0DOoo00<00?ooool0ZOoo000Oool00`00ooooo`1Bool00`00ooooo`2Yool001ko
o`03003ooooo05?oo`03003ooooo0:Woo`007Ooo00<00?ooool0E?oo00<00?ooool0ZOoo000Lool0
0`00ooooo`1Eool00`00ooooo`2Yool001_oo`03003ooooo05Koo`800:[oo`006_oo00<00?ooool0
Eooo00<00?ooool0ZOoo000Iool00`00ooooo`1Hool00`00ooooo`2Yool001Woo`03003ooooo05So
o`03003ooooo0:Woo`006?oo00<00?ooool0FOoo00<00?ooool0ZOoo000Gool00`00ooooo`1Jool2
002Zool001Koo`03003ooooo05_oo`03003ooooo0:Woo`005Ooo00<00?ooool0G?oo00<00?ooool0
ZOoo000Dool00`00ooooo`1Mool00`00ooooo`2Yool001?oo`03003ooooo04[oo`@000?oo`03003o
oooo00;oo`<000Goo`03003ooooo0:Woo`004_oo00<00?ooool0COoo00<00?ooool02_oo00<00?oo
ool00_oo00<00?ooool0ZOoo000Aool00`00ooooo`1>ool00`00ooooo`0:ool00`00ooooo`02ool3
002Yool0017oo`03003ooooo04koo`03003ooooo00Ooo`<000Goo`03003ooooo0:Woo`004?oo00<0
0?ooool0Cooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0ZOoo000?ool00`00ooooo`1>
ool30009ool00`00ooooo`05ool00`00ooooo`2Yool000koo`03003ooooo057oo`03003ooooo00Oo
o`@000Coo`03003ooooo0:Woo`003Ooo00<00?ooool0I?oo0P00Z_oo000<ool00`00ooooo`1Uool0
0`00ooooo`2Yool000coo`03003ooooo06Goo`03003ooooo0:Woo`002ooo00<00?ooool0I_oo00<0
0?ooool0ZOoo000:ool00`00ooooo`1Wool00`00ooooo`2Yool000Woo`03003ooooo06Soo`03003o
oooo0:Woo`00M?oo0P00Z_oo001dool00`00ooooo`2Yool007Coo`03003ooooo0:Woo`00M?oo00<0
0?ooool0ZOoo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.78327, -1.45025, \
0.00185875, 0.0182975}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Im[kx0corrtestfunc[Om - I\ 0.71, 1.15, 3]], {Om, 0.8, 1.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.5 1.90476 0.233817 0.131453 [
[.02381 .22132 -9 -9 ]
[.02381 .22132 9 0 ]
[.21429 .22132 -9 -9 ]
[.21429 .22132 9 0 ]
[.59524 .22132 -9 -9 ]
[.59524 .22132 9 0 ]
[.78571 .22132 -9 -9 ]
[.78571 .22132 9 0 ]
[.97619 .22132 -9 -9 ]
[.97619 .22132 9 0 ]
[.39226 .10236 -12 -4.5 ]
[.39226 .10236 0 4.5 ]
[.39226 .36527 -6 -4.5 ]
[.39226 .36527 0 4.5 ]
[.39226 .49672 -6 -4.5 ]
[.39226 .49672 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .23382 m
.02381 .24007 L
s
[(0.8)] .02381 .22132 0 1 Mshowa
.21429 .23382 m
.21429 .24007 L
s
[(0.9)] .21429 .22132 0 1 Mshowa
.59524 .23382 m
.59524 .24007 L
s
[(1.1)] .59524 .22132 0 1 Mshowa
.78571 .23382 m
.78571 .24007 L
s
[(1.2)] .78571 .22132 0 1 Mshowa
.97619 .23382 m
.97619 .24007 L
s
[(1.3)] .97619 .22132 0 1 Mshowa
.125 Mabswid
.0619 .23382 m
.0619 .23757 L
s
.1 .23382 m
.1 .23757 L
s
.1381 .23382 m
.1381 .23757 L
s
.17619 .23382 m
.17619 .23757 L
s
.25238 .23382 m
.25238 .23757 L
s
.29048 .23382 m
.29048 .23757 L
s
.32857 .23382 m
.32857 .23757 L
s
.36667 .23382 m
.36667 .23757 L
s
.44286 .23382 m
.44286 .23757 L
s
.48095 .23382 m
.48095 .23757 L
s
.51905 .23382 m
.51905 .23757 L
s
.55714 .23382 m
.55714 .23757 L
s
.63333 .23382 m
.63333 .23757 L
s
.67143 .23382 m
.67143 .23757 L
s
.70952 .23382 m
.70952 .23757 L
s
.74762 .23382 m
.74762 .23757 L
s
.82381 .23382 m
.82381 .23757 L
s
.8619 .23382 m
.8619 .23757 L
s
.9 .23382 m
.9 .23757 L
s
.9381 .23382 m
.9381 .23757 L
s
.25 Mabswid
0 .23382 m
1 .23382 L
s
.40476 .10236 m
.41101 .10236 L
s
[(-1)] .39226 .10236 1 0 Mshowa
.40476 .36527 m
.41101 .36527 L
s
[(1)] .39226 .36527 1 0 Mshowa
.40476 .49672 m
.41101 .49672 L
s
[(2)] .39226 .49672 1 0 Mshowa
.125 Mabswid
.40476 .12865 m
.40851 .12865 L
s
.40476 .15495 m
.40851 .15495 L
s
.40476 .18124 m
.40851 .18124 L
s
.40476 .20753 m
.40851 .20753 L
s
.40476 .26011 m
.40851 .26011 L
s
.40476 .2864 m
.40851 .2864 L
s
.40476 .31269 m
.40851 .31269 L
s
.40476 .33898 m
.40851 .33898 L
s
.40476 .39156 m
.40851 .39156 L
s
.40476 .41785 m
.40851 .41785 L
s
.40476 .44414 m
.40851 .44414 L
s
.40476 .47043 m
.40851 .47043 L
s
.40476 .07607 m
.40851 .07607 L
s
.40476 .04978 m
.40851 .04978 L
s
.40476 .02349 m
.40851 .02349 L
s
.40476 .52301 m
.40851 .52301 L
s
.40476 .54931 m
.40851 .54931 L
s
.40476 .5756 m
.40851 .5756 L
s
.40476 .60189 m
.40851 .60189 L
s
.25 Mabswid
.40476 0 m
.40476 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .56532 L
.10458 .52787 L
.14415 .49597 L
.18221 .46791 L
.22272 .44049 L
.26171 .4162 L
.30316 .39238 L
.34309 .37115 L
.3815 .35213 L
.42237 .33325 L
.46172 .31625 L
.49955 .30087 L
.53984 .28545 L
.57861 .27143 L
.61984 .25734 L
.64043 .25058 L
.65954 .24447 L
.67803 .2387 L
.68335 .23706 L
.68596 .23626 L
.68711 .23591 L
.68837 .23553 L
.6895 .23519 L
.69055 .04728 L
.69167 .04715 L
.69287 .04701 L
.69774 .04646 L
.7391 .04177 L
.77894 .03724 L
.81727 .03287 L
.85806 .02821 L
.89732 .02373 L
.93507 .01941 L
.97528 .01482 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo001dool0
0`00ooooo`2Yool007Coo`03003ooooo0:Woo`00M?oo00<00?ooool0ZOoo001dool00`00ooooo`2Y
ool007Coo`03003ooooo09Woo`L000Woo`00M?oo00<00?ooool0S_oo2`004?oo001dool20027ool8
000Kool007Coo`03003ooooo083oo`H002?oo`00M?oo00<00?ooool0N?oo2000:Ooo001dool00`00
ooooo`1]ool;000aool007Coo`03003ooooo06Goo`P003coo`00M?oo00<00?ooool0Gooo1P00A?oo
001dool00`00ooooo`1Fool9001:ool007Coo`03003ooooo04koo`P005?oo`00M?oo0P00Cooo00<0
0?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003o
oooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00oooo
o`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0
F?oo001dool2001?ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo
00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03
003ooooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo001/ool40004
ool00`00ooooo`1>ool00`00ooooo`1Hool006koo`03003ooooo00?oo`03003ooooo04koo`03003o
oooo05Soo`00K_oo00<00?ooool00ooo0`00C_oo00<00?ooool0F?oo001Vool40004ool00`00oooo
o`03ool00`00ooooo`1>ool00`00ooooo`1Hool006koo`03003ooooo00?oo`03003ooooo04koo`03
003ooooo05Soo`00K?oo0`001Ooo00<00?ooool0C_oo00<00?ooool0F?oo001^ool00`00ooooo`03
ool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo
00<00?ooool0C_oo00<00?ooool0F?oo001dool2001?ool00`00ooooo`1Hool007Coo`03003ooooo
04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00ooooo`1>
ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo
00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03
003ooooo05Soo`00M?oo0P00Cooo00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1H
ool007Coo`03003ooooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo
001dool00`00ooooo`1>ool00`00ooooo`1Hool007Coo`03003ooooo04koo`03003ooooo05Soo`00
M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool2001?ool00`00ooooo`1Hool007Coo`03003o
oooo04koo`03003ooooo05Soo`00M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00oooo
o`1>ool00`00ooooo`1Hool000;oo`8000Coo`03003ooooo00?oo`8002Ooo`8000Coo`03003ooooo
00;oo`<002ooo`03003ooooo02_oo`@000?oo`03003ooooo00;oo`@001?oo`03003ooooo013oo`@0
00?oo`03003ooooo00;oo`@002Koo`@000?oo`03003ooooo00?oo`8000?oo`0000Goo`00ooooo`00
00Soo`04003ooooo000Uool01000ooooo`002ooo00<00?ooool0;?oo00<00?ooool0;Ooo00<00?oo
ool02Ooo00<00?ooool04_oo00<00?ooool04_oo00<00?ooool01ooo00<00?ooool0:Ooo00<00?oo
ool01ooo00@00?ooool000;oo`0000Goo`00ooooo`0000Soo`04003ooooo000Uool01000ooooo`00
2ooo00<00?ooool0;?oo00<00?ooool0;Ooo00<00?ooool02Ooo00<00?ooool04_oo00<00?ooool0
4_oo00<00?ooool02?oo00<00?ooool0:?oo00<00?ooool02_oo0@000Ooo0Ooo00001Ooo003ooooo
00002Ooo0P009_oo00@00?ooool000Woo`<002koo`8002koo`03003ooooo00Woo`03003ooooo01;o
o`03003ooooo01;oo`03003ooooo00Woo`03003ooooo02Ooo`03003ooooo00Soo`8000?oo`0000Go
o`00ooooo`0000Soo`04003ooooo000Uool01000ooooo`002?oo00@00?ooool002koo`03003ooooo
02goo`03003ooooo00Woo`03003ooooo01;oo`03003ooooo01;oo`03003ooooo00[oo`03003ooooo
02Koo`03003ooooo00[oo`40007oo`7oo`0000Goo`00ooooo`0000Soo`04003ooooo000Uool01000
ooooo`002?oo00@00?ooool002koo`03003ooooo02_oo`<000Woo`<001Coo`03003ooooo013oo`<0
00Woo`04003ooooo000Vool30009ool01000ooooo`000_oo0002ool2000:ool2000Wool2000:ool2
000_ool00`00ooooo`0]ool00`00ooooo`09ool00`00ooooo`0Bool00`00ooooo`0Bool00`00oooo
o`08ool2000Yool00`00ooooo`08ool20003ool007Coo`03003ooooo04koo`03003ooooo05Soo`00
M?oo00<00?ooool0C_oo00<00?ooool0F?oo001dool00`00ooooo`1>ool00`00ooooo`1Hool007Co
o`03003ooooo04koo`03003ooooo05Soo`000_ooo`0070000ooo0009ool00`00ooooo`07ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00
ooooo`07ool50006ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07ool000Woo`03003o
oooo03;oo`03003ooooo03?oo`03003ooooo03?oo`03003ooooo017oo`80023oo`03003ooooo03?o
o`03003ooooo00Ooo`00M?oo00<00?ooool0A?oo0`00H_oo001dool00`00ooooo`12ool2001Uool0
07Coo`03003ooooo03goo`D006Ooo`00M?oo00<00?ooool0>Ooo1000K?oo001dool2000gool3001`
ool007Coo`03003ooooo03?oo`<007?oo`00M?oo00<00?ooool0<?oo0`00M_oo001dool00`00oooo
o`0]ool3001iool007Coo`03003ooooo02[oo`<007coo`00M?oo00<00?ooool0:?oo0P00Oooo001d
ool00`00ooooo`0Uool30021ool007Coo`03003ooooo02;oo`<008Coo`00M?oo0P008Ooo0P00Qooo
001dool00`00ooooo`0Nool20029ool007Coo`03003ooooo01coo`8008_oo`00M?oo00<00?ooool0
6_oo0P00SOoo001dool00`00ooooo`0Gool3002?ool007Coo`03003ooooo01Coo`<009;oo`00M?oo
00<00?ooool04_oo0P00UOoo001dool2000@ool3002Gool007Coo`03003ooooo00coo`<009[oo`00
M?oo00<00?ooool02_oo0P00WOoo001dool00`00ooooo`08ool2002Oool007Coo`03003ooooo00Ko
o`800:7oo`00M?oo00<00?ooool01?oo0P00Xooo001dool01000oooooooo0`00YOoo001dool4002X
ool007Coo`800:[oo`00L_oo0`00Zooo001`ool20002ool00`00ooooo`2Yool006koo`8000Coo`03
003ooooo0:Woo`00K?oo0P001_oo00<00?ooool0ZOoo001Zool60004ool00`00ooooo`2Yool006So
o`8000Coo`03003ooooo00?oo`03003ooooo0:Woo`00I_oo0P001_oo00<00?ooool00ooo0`00ZOoo
001Tool20008ool00`00ooooo`03ool00`00ooooo`2Yool006;oo`8000[oo`03003ooooo00?oo`03
003ooooo0:Woo`00H?oo0P002_oo0`001Ooo00<00?ooool0ZOoo001Nool2000>ool00`00ooooo`03
ool00`00ooooo`2Yool005coo`8001Koo`03003ooooo0:Woo`00F_oo0P006?oo00<00?ooool0ZOoo
001Hool2000Jool2002Zool005Ooo`03003ooooo01[oo`03003ooooo0:Woo`00EOoo0P007Ooo00<0
0?ooool0ZOoo001Cool2000Oool00`00ooooo`2Yool0057oo`80027oo`03003ooooo0:Woo`00Cooo
0P008ooo00<00?ooool0ZOoo001=ool2000Uool00`00ooooo`2Yool004coo`03003ooooo02Goo`03
003ooooo0:Woo`00B_oo0P00:?oo0P00Z_oo0019ool00`00ooooo`0Xool00`00ooooo`2Yool004Oo
o`8002_oo`03003ooooo0:Woo`00AOoo0P00;Ooo00<00?ooool0ZOoo0014ool00`00ooooo`0]ool0
0`00ooooo`2Yool004;oo`80033oo`03003ooooo0:Woo`00@Ooo00<00?ooool0<?oo00<00?ooool0
ZOoo000oool2000cool2002Zool003koo`03003ooooo03?oo`03003ooooo0:Woo`00??oo0P00=_oo
00<00?ooool0ZOoo000kool00`00ooooo`0fool00`00ooooo`2Yool003Woo`8003Woo`03003ooooo
0:Woo`00>?oo00<00?ooool0>Ooo00<00?ooool0ZOoo000fool2000lool00`00ooooo`2Yool003Go
o`03003ooooo03coo`03003ooooo0:Woo`00<ooo0P00?ooo0P00Z_oo000bool00`00ooooo`0oool0
0`00ooooo`2Yool0037oo`03003ooooo043oo`03003ooooo0:Woo`00;ooo0P00@ooo00<00?ooool0
ZOoo000^ool00`00ooooo`13ool00`00ooooo`2Yool002goo`03003ooooo03coo`@000Coo`03003o
oooo0:Woo`00:ooo0P00?ooo00<00?ooool01Ooo00<00?ooool0ZOoo000Zool00`00ooooo`10ool0
0`00ooooo`04ool3002Yool002Woo`03003ooooo04;oo`03003ooooo00?oo`03003ooooo0:Woo`00
9ooo0P00A_oo00<00?ooool00_oo00<00?ooool0ZOoo000Vool00`00ooooo`13ool01000ooooo`00
1?oo00<00?ooool0ZOoo000Uool00`00ooooo`15ool20005ool00`00ooooo`2Yool002Coo`03003o
oooo04goo`03003ooooo0:Woo`008ooo00<00?ooool0C_oo00<00?ooool0ZOoo000Qool2001Aool2
002Zool0023oo`03003ooooo057oo`03003ooooo0:Woo`007ooo00<00?ooool0D_oo00<00?ooool0
ZOoo000Nool00`00ooooo`1Cool00`00ooooo`2Yool001coo`8005Koo`03003ooooo0:Woo`006ooo
00<00?ooool0E_oo00<00?ooool0ZOoo000Jool00`00ooooo`1Gool00`00ooooo`2Yool001Woo`03
003ooooo05Soo`03003ooooo0:Woo`006?oo00<00?ooool0FOoo0P00Z_oo000Fool2001Lool00`00
ooooo`2Yool001Goo`03003ooooo05coo`03003ooooo0:Woo`005?oo00<00?ooool0GOoo00<00?oo
ool0ZOoo000Cool00`00ooooo`1Nool00`00ooooo`2Yool001;oo`03003ooooo05ooo`03003ooooo
0:Woo`004Ooo00<00?ooool0H?oo00<00?ooool0ZOoo000@ool00`00ooooo`1Qool2002Zool000oo
o`03003ooooo06;oo`03003ooooo0:Woo`003_oo00<00?ooool0Hooo00<00?ooool0ZOoo000>ool0
0`00ooooo`1Sool00`00ooooo`2Yool000goo`03003ooooo06Coo`03003ooooo0:Woo`003?oo00<0
0?ooool0IOoo00<00?ooool0ZOoo000;ool00`00ooooo`1Vool00`00ooooo`2Yool000[oo`03003o
oooo06Ooo`03003ooooo0:Woo`002Ooo00<00?ooool0J?oo0P00Z_oo001dool00`00ooooo`2Yool0
07Coo`03003ooooo0:Woo`00M?oo00<00?ooool0ZOoo001dool00`00ooooo`2Yool00?ooob7oo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.78327, -1.81072, \
0.00185875, 0.0269334}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.74, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] \(\(0.8760984351000806`\)\(\[InvisibleSpace]\)\) + 
          1.0274376357597258`*^-7\ \[ImaginaryI]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol[k, 10]], {k, 0.01, 1}, 
      PlotStyle \[Rule] {RGBColor[1, 0, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 -0.877637 1.44479 [
[.21429 .55465 -9 -9 ]
[.21429 .55465 9 0 ]
[.40476 .55465 -9 -9 ]
[.40476 .55465 9 0 ]
[.59524 .55465 -9 -9 ]
[.59524 .55465 9 0 ]
[.78571 .55465 -9 -9 ]
[.78571 .55465 9 0 ]
[.97619 .55465 -3 -9 ]
[.97619 .55465 3 0 ]
[.01131 .13372 -18 -4.5 ]
[.01131 .13372 0 4.5 ]
[.01131 .2782 -18 -4.5 ]
[.01131 .2782 0 4.5 ]
[.01131 .42267 -18 -4.5 ]
[.01131 .42267 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .56715 m
.21429 .5734 L
s
[(0.2)] .21429 .55465 0 1 Mshowa
.40476 .56715 m
.40476 .5734 L
s
[(0.4)] .40476 .55465 0 1 Mshowa
.59524 .56715 m
.59524 .5734 L
s
[(0.6)] .59524 .55465 0 1 Mshowa
.78571 .56715 m
.78571 .5734 L
s
[(0.8)] .78571 .55465 0 1 Mshowa
.97619 .56715 m
.97619 .5734 L
s
[(1)] .97619 .55465 0 1 Mshowa
.125 Mabswid
.07143 .56715 m
.07143 .5709 L
s
.11905 .56715 m
.11905 .5709 L
s
.16667 .56715 m
.16667 .5709 L
s
.2619 .56715 m
.2619 .5709 L
s
.30952 .56715 m
.30952 .5709 L
s
.35714 .56715 m
.35714 .5709 L
s
.45238 .56715 m
.45238 .5709 L
s
.5 .56715 m
.5 .5709 L
s
.54762 .56715 m
.54762 .5709 L
s
.64286 .56715 m
.64286 .5709 L
s
.69048 .56715 m
.69048 .5709 L
s
.7381 .56715 m
.7381 .5709 L
s
.83333 .56715 m
.83333 .5709 L
s
.88095 .56715 m
.88095 .5709 L
s
.92857 .56715 m
.92857 .5709 L
s
.25 Mabswid
0 .56715 m
1 .56715 L
s
.02381 .13372 m
.03006 .13372 L
s
[(0.7)] .01131 .13372 1 0 Mshowa
.02381 .2782 m
.03006 .2782 L
s
[(0.8)] .01131 .2782 1 0 Mshowa
.02381 .42267 m
.03006 .42267 L
s
[(0.9)] .01131 .42267 1 0 Mshowa
.125 Mabswid
.02381 .16261 m
.02756 .16261 L
s
.02381 .19151 m
.02756 .19151 L
s
.02381 .2204 m
.02756 .2204 L
s
.02381 .2493 m
.02756 .2493 L
s
.02381 .30709 m
.02756 .30709 L
s
.02381 .33599 m
.02756 .33599 L
s
.02381 .36488 m
.02756 .36488 L
s
.02381 .39378 m
.02756 .39378 L
s
.02381 .45157 m
.02756 .45157 L
s
.02381 .48047 m
.02756 .48047 L
s
.02381 .50936 m
.02756 .50936 L
s
.02381 .53826 m
.02756 .53826 L
s
.02381 .10482 m
.02756 .10482 L
s
.02381 .07592 m
.02756 .07592 L
s
.02381 .04703 m
.02756 .04703 L
s
.02381 .01813 m
.02756 .01813 L
s
.02381 .59605 m
.02756 .59605 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03333 .01472 m
.03793 .01493 L
.04222 .0152 L
.04692 .01559 L
.05195 .01609 L
.06223 .01743 L
.07158 .019 L
.09303 .02386 L
.1133 .03 L
.15133 .04537 L
.19178 .06673 L
.23074 .09158 L
.2682 .11887 L
.30808 .15107 L
.34647 .18466 L
.38728 .22275 L
.40766 .24254 L
.41759 .25239 L
.41987 .25458 L
.42228 .25692 L
.42327 .25769 L
.42433 .25833 L
.4255 .25885 L
.42659 .25923 L
.43112 .2605 L
.43601 .26186 L
.44489 .26441 L
.4644 .27021 L
.50535 .28348 L
.5448 .29778 L
.58274 .31305 L
.62312 .33106 L
.66199 .3503 L
.69936 .37073 L
.73916 .39479 L
.77746 .42045 L
.81819 .45073 L
.85742 .48313 L
.89515 .51757 L
.9353 .55801 L
.97396 .60074 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003oooP000Kh0?ooo`?o
o`005Ooo0P001ooo1oP0nooo000Eool00`00ooooo`0=ool5n03fool001Goo`03003ooooo01;oo`;h
0?Coo`005Ooo00<00?ooool05?oo0oP0lOoo000Eool00`00ooooo`0Gool3n03^ool001Goo`03003o
oooo01[oo`;h0>coo`005Ooo00<00?ooool07?oo0oP0jOoo000Eool00`00ooooo`0Oool2n03Wool0
01Goo`8002;oo`;h0>Goo`005Ooo00<00?ooool08ooo0_P0hooo000Eool00`00ooooo`0Uool2n03Q
ool001Goo`03003ooooo02Ooo`;h0=ooo`005Ooo00<00?ooool0:Ooo0_P0gOoo000Eool00`00oooo
o`0[ool00oP0ooooo`3Jool001Goo`03003ooooo02coo`;h0=[oo`005Ooo00<00?ooool0;_oo00?h
0?ooool0eooo000Eool2000`ool00oP0ooooo`3Fool001Goo`03003ooooo033oo`;h0=Koo`005Ooo
00<00?ooool0<_oo00?h0?ooool0dooo000Eool00`00ooooo`0cool2n03Cool001Goo`03003ooooo
03Goo`03n03ooooo0=3oo`005Ooo00<00?ooool0=_oo0_P0d?oo000Eool00`00ooooo`0hool00oP0
ooooo`3=ool001Goo`8003[oo`03n03ooooo0<coo`005Ooo00<00?ooool0>_oo0_P0c?oo000Eool0
0`00ooooo`0lool00oP0ooooo`39ool001Goo`03003ooooo03goo`;h0<Woo`005Ooo00<00?ooool0
?ooo00?h0?ooool0a_oo000Eool00`00ooooo`10ool00oP0ooooo`35ool000;oo`8000Coo`03003o
oooo00?oo`03003ooooo00Coo`03003ooooo047oo`;h0<Goo`0000Goo`00ooooo`0000Woo`03003o
oooo00Coo`03003ooooo04?oo`03n03ooooo0<;oo`0000Goo`00ooooo`0000[oo`03003ooooo00?o
o`<004Coo`03n03ooooo0<7oo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo04Go
o`03n03ooooo0<3oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003ooooo04Koo`03n03o
oooo0;ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`17ool2n02oool000;o
o`8000Woo`@000Coo`03003ooooo04Woo`03n03ooooo0;coo`005Ooo00<00?ooool0B_oo00?h0?oo
ool0^ooo000Eool00`00ooooo`1;ool00oP0ooooo`2jool001Goo`03003ooooo04coo`;h0;[oo`00
5Ooo0P00Cooo00?h0?ooool0]ooo000Eool00`00ooooo`1?ool00oP0ooooo`2fool001Goo`03003o
oooo053oo`03n03ooooo0;Goo`005Ooo00<00?ooool0DOoo00?h0?ooool0]?oo000Eool00`00oooo
o`1Bool2n02dool001Goo`03003ooooo05Coo`03n03ooooo0;7oo`005Ooo00<00?ooool0EOoo00?h
0?ooool0/?oo000Eool00`00ooooo`1Fool00oP0ooooo`2_ool001Goo`8005Soo`03n03ooooo0:ko
o`005Ooo00<00?ooool0F?oo00?h0?ooool0[Ooo000Eool00`00ooooo`1Iool00oP0ooooo`2/ool0
01Goo`03003ooooo05[oo`03n03ooooo0:_oo`005Ooo00<00?ooool0Fooo00?h0?ooool0Z_oo000E
ool00`00ooooo`1Lool00oP0ooooo`2Yool001Goo`03003ooooo05goo`03n03ooooo0:Soo`005Ooo
00<00?ooool0G_oo00?h0?ooool0Yooo000Eool2001Pool00oP0ooooo`2Vool001Goo`03003ooooo
063oo`03n03ooooo0:Goo`005Ooo00<00?ooool0HOoo00?h0?ooool0Y?oo000Eool00`00ooooo`1R
ool00oP0ooooo`2Sool001Goo`03003ooooo06?oo`03n03ooooo0:;oo`005Ooo00<00?ooool0I?oo
00?h0?ooool0XOoo000Eool00`00ooooo`1Uool00oP0ooooo`2Pool001Goo`03003ooooo06Koo`03
n03ooooo09ooo`005Ooo0P00J?oo00?h0?ooool0W_oo000Eool00`00ooooo`1Xool2n02Nool001Go
o`03003ooooo06Woo`;h09goo`005Ooo00<00?ooool0Jooo0oP0V_oo000Eool00`00ooooo`1^ool4
n02Fool001Goo`03003ooooo07;oo`Gh097oo`000_oo0P001?oo00<00?ooool00ooo0P001Ooo00<0
0?ooool0Mooo0oP0S_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo07[oo`;h
08coo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3001lool3n029ool00005ool00?ooool0
0009ool20005ool00`00ooooo`1oool3n026ool00005ool00?ooool00008ool01000ooooo`001?oo
00<00?ooool0P_oo0oP0Pooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo08Go
o`;h087oo`000_oo0P002_oo0P001Ooo00<00?ooool0Qooo0oP0O_oo000Eool00`00ooooo`2:ool3
n01kool001Goo`03003ooooo08goo`;h07Woo`005Ooo00<00?ooool0Sooo0oP0M_oo000Eool2002C
ool2n01dool001Goo`03003ooooo09Coo`?h077oo`005Ooo00<00?ooool0Uooo0_P0Kooo000Eool0
0`00ooooo`2Iool2n01]ool001Goo`03003ooooo09_oo`;h06_oo`005Ooo00<00?ooool0WOoo0_P0
JOoo000Eool00`00ooooo`2Oool3n01Vool001Goo`800:?oo`;h06Coo`005Ooo00<00?ooool0Y?oo
0_P0H_oo000Eool00`00ooooo`2Vool2n01Pool001Goo`03003ooooo0:Soo`;h05koo`005Ooo00<0
0?ooool0Z_oo0_P0G?oo000Eool00`00ooooo`2/ool2n01Jool001Goo`03003ooooo0:koo`;h05So
o`005Ooo00<00?ooool0/?oo00?h0?ooool0EOoo000Eool2002bool2n01Eool001Goo`03003ooooo
0;?oo`;h05?oo`005Ooo00<00?ooool0]Ooo00?h0?ooool0D?oo000Eool00`00ooooo`2fool2n01@
ool001Goo`03003ooooo0;Soo`;h04koo`005Ooo00<00?ooool0^_oo0_P0C?oo000Eool00`00oooo
o`2lool2n01:ool001Goo`03003ooooo0;koo`;h04Soo`005Ooo0P00`Ooo00?h0?ooool0AOoo000E
ool00`00ooooo`31ool2n015ool001Goo`03003ooooo0<?oo`03n03ooooo04;oo`005Ooo00<00?oo
ool0a?oo00?h0?ooool0@Ooo000Eool00`00ooooo`35ool2n011ool001Goo`03003ooooo0<Ooo`03
n03ooooo03koo`000_oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0b?oo0_P0?_oo0000
1Ooo003ooooo00002ooo00<00?ooool00_oo00<00?ooool0b_oo00?h0?ooool0>ooo00001Ooo003o
oooo00002ooo00<00?ooool00_oo0`00booo00?h0?ooool0>_oo00001Ooo003ooooo00002Ooo0`00
1?oo00<00?ooool0c?oo0_P0>_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo
0<koo`03n03ooooo03Ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3?ool0
0oP0ooooo`0fool000;oo`8000[oo`8000Goo`03003ooooo0=3oo`03n03ooooo03Goo`005Ooo00<0
0?ooool0dOoo00?h0?ooool0=?oo000Eool00`00ooooo`3Bool2n00dool001Goo`03003ooooo0=Co
o`03n03ooooo037oo`005Ooo0P00e_oo00?h0?ooool0<?oo000Eool00`00ooooo`3Fool2n00`ool0
01Goo`03003ooooo0=Soo`03n03ooooo02goo`005Ooo00<00?ooool0fOoo00?h0?ooool0;?oo000E
ool00`00ooooo`3Jool2n00/ool001Goo`03003ooooo0=coo`03n03ooooo02Woo`005Ooo00<00?oo
ool0gOoo00?h0?ooool0:?oo000Eool00`00ooooo`3Nool2n00Xool001Goo`800>7oo`03n03ooooo
02Goo`005Ooo00<00?ooool0hOoo00?h0?ooool09?oo000Eool00`00ooooo`3Rool00oP0ooooo`0S
ool001Goo`03003ooooo0>?oo`03n03ooooo02;oo`005Ooo00<00?ooool0i?oo00?h0?ooool08Ooo
000Eool00`00ooooo`3Uool00oP0ooooo`0Pool001Goo`03003ooooo0>Koo`03n03ooooo01ooo`00
5Ooo00<00?ooool0iooo00?h0?ooool07_oo000Eool2003Yool00oP0ooooo`0Mool001Goo`03003o
oooo0>Woo`03n03ooooo01coo`005Ooo00<00?ooool0j_oo00?h0?ooool06ooo000Eool00`00oooo
o`3[ool00oP0ooooo`0Jool001Goo`03003ooooo0>coo`03n03ooooo01Woo`005Ooo00<00?ooool0
:_oo0P001?oo00<00?ooool00_oo10009Ooo0P001?oo00<00?ooool01?oo00<00?ooool08ooo0P00
1?oo00<00?ooool00ooo0P009_oo0P001?oo00<00?ooool00ooo0P006_oo00?h0?ooool03_oo1000
1_oo000Eool00`00ooooo`0Yool01000ooooo`002?oo00<00?ooool09Ooo00@00?ooool000[oo`03
003ooooo02;oo`04003ooooo0008ool01000ooooo`009?oo00@00?ooool000Soo`04003ooooo000J
ool00oP0ooooo`0?ool00`00ooooo`05ool001Goo`8002[oo`04003ooooo0009ool00`00ooooo`0T
ool01000ooooo`001ooo1@008ooo00@00?ooool000Soo`04003ooooo000Tool01000ooooo`002?oo
00@00?ooool001_oo`03n03ooooo00koo`03003ooooo00Goo`005Ooo00<00?ooool0:Ooo00@00?oo
ool000[oo`03003ooooo02?oo`04003ooooo0007ool01000ooooo`009?oo00@00?ooool000Soo`<0
02Goo`04003ooooo0009ool2000Mool00oP0ooooo`0=ool00`00ooooo`05ool001Goo`03003ooooo
02Woo`04003ooooo000;ool00`00ooooo`0Rool01000ooooo`002?oo00<00?oo00009?oo00@00?oo
ool000Soo`03003ooooo02Goo`04003ooooo0008ool01000ooooo`007Ooo00?h0?ooool03?oo00<0
0?ooool01Ooo000Eool00`00ooooo`0Yool01000ooooo`002?oo00@00?ooool002Coo`04003ooooo
0009ool2000Tool01000ooooo`002?oo00<00?ooool09Ooo00@00?ooool000Soo`04003ooooo000N
ool00oP0ooooo`09ool30007ool001Goo`03003ooooo02[oo`8000[oo`8002Koo`8000_oo`03003o
oooo02?oo`8000[oo`<002Goo`8000[oo`80023oo`03n03ooooo00[oo`03003ooooo00Goo`005Ooo
00<00?ooool0m?oo00?h0?ooool04Ooo000Eool00`00ooooo`3eool00oP0ooooo`0@ool001Goo`03
003ooooo0?Koo`03n03ooooo00ooo`003_ooo`000P0000?h000000003P00000Eool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`02ool00oP0ooooo`05
ool00`00ooooo`05ool001Goo`03003ooooo037oo`03003ooooo037oo`03003ooooo033oo`03003o
oooo037oo`03003ooooo02[oo`03n03ooooo00Coo`03003ooooo00Goo`005Ooo00<00?ooool0n_oo
00?h0?ooool02ooo000Eool00`00ooooo`3jool00oP0ooooo`0;ool001Goo`03003ooooo0?_oo`03
n03ooooo00[oo`005Ooo00<00?ooool0o?oo00?h0?ooool02Ooo000Eool00`00ooooo`3mool00oP0
ooooo`08ool001Goo`800?ooo`03n03ooooo00Ooo`005Ooo00<00?ooool0oooo00?h0?ooool01_oo
000Eool00`00ooooo`3oool1ool00oP0ooooo`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0825959, 0.596272, \
0.00385924, 0.00254394}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%1066, %1067]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.0147151 0.392403 [
[.15079 .00222 -9 -9 ]
[.15079 .00222 9 0 ]
[.27778 .00222 -9 -9 ]
[.27778 .00222 9 0 ]
[.40476 .00222 -9 -9 ]
[.40476 .00222 9 0 ]
[.53175 .00222 -9 -9 ]
[.53175 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.78571 .00222 -9 -9 ]
[.78571 .00222 9 0 ]
[.9127 .00222 -9 -9 ]
[.9127 .00222 9 0 ]
[.01131 .0932 -18 -4.5 ]
[.01131 .0932 0 4.5 ]
[.01131 .17168 -18 -4.5 ]
[.01131 .17168 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .32864 -18 -4.5 ]
[.01131 .32864 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .56408 -18 -4.5 ]
[.01131 .56408 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .01472 m
.15079 .02097 L
s
[(0.2)] .15079 .00222 0 1 Mshowa
.27778 .01472 m
.27778 .02097 L
s
[(0.4)] .27778 .00222 0 1 Mshowa
.40476 .01472 m
.40476 .02097 L
s
[(0.6)] .40476 .00222 0 1 Mshowa
.53175 .01472 m
.53175 .02097 L
s
[(0.8)] .53175 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(1)] .65873 .00222 0 1 Mshowa
.78571 .01472 m
.78571 .02097 L
s
[(1.2)] .78571 .00222 0 1 Mshowa
.9127 .01472 m
.9127 .02097 L
s
[(1.4)] .9127 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.18254 .01472 m
.18254 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.34127 .01472 m
.34127 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.5 .01472 m
.5 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.81746 .01472 m
.81746 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.97619 .01472 m
.97619 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .0932 m
.03006 .0932 L
s
[(0.2)] .01131 .0932 1 0 Mshowa
.02381 .17168 m
.03006 .17168 L
s
[(0.4)] .01131 .17168 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.6)] .01131 .25016 1 0 Mshowa
.02381 .32864 m
.03006 .32864 L
s
[(0.8)] .01131 .32864 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(1)] .01131 .40712 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(1.2)] .01131 .4856 1 0 Mshowa
.02381 .56408 m
.03006 .56408 L
s
[(1.4)] .01131 .56408 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .11282 m
.02756 .11282 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .21092 m
.02756 .21092 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .50522 m
.02756 .50522 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .5837 m
.02756 .5837 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03016 .25708 m
.03323 .25713 L
.03608 .25721 L
.03922 .25731 L
.04257 .25745 L
.04942 .25781 L
.05566 .25824 L
.06995 .25956 L
.08347 .26123 L
.10882 .2654 L
.13579 .2712 L
.16176 .27795 L
.18674 .28537 L
.21332 .29411 L
.23891 .30323 L
.26612 .31358 L
.27971 .31895 L
.28633 .32163 L
.28785 .32222 L
.28946 .32286 L
.29012 .32307 L
.29083 .32324 L
.2916 .32338 L
.29233 .32349 L
.29535 .32383 L
.29861 .3242 L
.30453 .32489 L
.31754 .32647 L
.34484 .33007 L
.37113 .33396 L
.39643 .3381 L
.42335 .343 L
.44926 .34822 L
.47418 .35377 L
.50071 .36031 L
.52625 .36727 L
.5534 .3755 L
.57955 .3843 L
.6047 .39365 L
.63147 .40463 L
.65724 .41624 L
.65873 .41694 L
s
0 g
.03016 .01864 m
.06854 .04236 L
.11039 .06823 L
.1497 .09252 L
.1875 .11588 L
.22774 .14075 L
.26647 .16469 L
.30764 .19013 L
.34731 .21465 L
.38546 .23823 L
.42606 .26332 L
.46515 .28748 L
.50273 .3107 L
.54275 .33544 L
.58126 .35924 L
.62221 .38455 L
.66166 .40893 L
.69959 .43237 L
.73997 .45733 L
.77884 .48135 L
.82014 .50688 L
.85994 .53147 L
.89823 .55514 L
.93896 .58031 L
.97619 .60332 L
s
.03016 .30939 m
.03133 .30943 L
.03239 .30947 L
.03362 .30932 L
.03477 .30933 L
.03585 .30962 L
.03684 .30967 L
.03908 .30981 L
.04372 .31016 L
.04884 .31064 L
.0584 .3118 L
.06854 .31335 L
.11039 .32202 L
.15041 .33176 L
.18892 .34119 L
.22987 .35077 L
.26932 .35939 L
.30725 .36708 L
.34763 .3746 L
.3865 .38121 L
.4278 .38753 L
.4676 .39292 L
.5059 .3974 L
.54663 .40134 L
.56691 .40295 L
.58585 .40422 L
.60522 .40528 L
.62357 .40606 L
.63325 .40638 L
.64379 .40665 L
.64843 .40675 L
.65336 .40684 L
.65767 .4069 L
.66222 .40695 L
.66454 .40698 L
.66707 .407 L
.66941 .40701 L
.67153 .40702 L
.67273 .40703 L
.67404 .40703 L
.67476 .40703 L
.67542 .40704 L
.67672 .40704 L
.67784 .40704 L
.67907 .40704 L
.68036 .40704 L
.68156 .40704 L
.68268 .40704 L
.68391 .40703 L
.68613 .40703 L
Mistroke
.6883 .40702 L
.69031 .407 L
.69487 .40697 L
.69972 .40692 L
.70996 .40678 L
.72103 .40657 L
.74107 .40608 L
.78186 .40477 L
.82113 .40335 L
.8589 .402 L
.89911 .40067 L
.93781 .39952 L
.97619 .39851 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool0033oo`8000Coo`03003ooooo00;oo`@001Coo`8000Coo`03003ooooo00Coo`03003ooooo01?o
o`8000Coo`03003ooooo00?oo`8001Coo`8000Coo`03003ooooo00?oo`8001[oo`@001Soo`@000?o
o`03003ooooo00;oo`@001?oo`@000?oo`03003ooooo00Coo`03003ooooo013oo`00;ooo00@00?oo
ool000Soo`03003ooooo01Coo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`002?oo00@0
0?ooool001;oo`04003ooooo0008ool01000ooooo`006ooo00<00?ooool06Ooo00<00?ooool01ooo
00<00?ooool05_oo00<00?ooool02Ooo00<00?ooool04?oo000_ool01000ooooo`002Ooo00<00?oo
ool04ooo00@00?ooool000Ooo`D001?oo`04003ooooo0008ool01000ooooo`004_oo00@00?ooool0
00Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`08ool00`00ooooo`0Eool00`00oooo
o`06ool5000Aool002ooo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`001ooo00@00?oo
ool001Coo`04003ooooo0008ool3000Cool01000ooooo`002Ooo0P007?oo00<00?ooool06Ooo00<0
0?ooool02Ooo00<00?ooool05?oo00<00?ooool01_oo00@00?ooool001;oo`00;ooo00@00?ooool0
00_oo`03003ooooo017oo`04003ooooo0008ool00`00ool0000Dool01000ooooo`002?oo00<00?oo
ool04ooo00@00?ooool000Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`0:ool00`00
ooooo`0Cool00`00ooooo`07ool00`00ool0000Bool002ooo`04003ooooo0008ool01000ooooo`00
4ooo00@00?ooool000Woo`8001Coo`04003ooooo0008ool00`00ooooo`0Cool01000ooooo`002?oo
00@00?ooool001Woo`<001Woo`<000Woo`04003ooooo000Cool3000:ool2000Bool001Goo`03003o
oooo01Soo`8000[oo`8001Goo`8000_oo`03003ooooo01?oo`8000[oo`<001?oo`8000[oo`8001co
o`03003ooooo01Woo`03003ooooo00Soo`8001Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo
00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`003_oo
o`004`00000Eool00`00ool00006ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool001Goo`03003ooooo008001goo`03003ooooo023oo`03003o
oooo023oo`03003ooooo01ooo`03003ooooo023oo`03003ooooo01ooo`03003ooooo023oo`03003o
oooo01Koo`005Ooo00<00?ooool00_oo00<00?ooool0oooo1?oo000Eool00`00ooooo`03ool00`00
ooooo`3oool3ool001Goo`8000Goo`800?ooo`?oo`005Ooo00<00?ooool01_oo00<00?ooool0oooo
000Eool00`00ooooo`07ool2003oool001Goo`03003ooooo00Woo`03003ooooo0?coo`005Ooo00<0
0?ooool02_oo0P00o?oo000Eool00`00ooooo`0<ool00`00ooooo`3iool001Goo`8000koo`800?Wo
o`005Ooo00<00?ooool03ooo0P00mooo000Eool00`00ooooo`0Aool00`00ooooo`3dool001Goo`03
003ooooo01;oo`800?Coo`005Ooo00<00?ooool05?oo00<00?ooool0lOoo000Eool2000Fool2003a
ool001Goo`03003ooooo01Ooo`800>ooo`005Ooo00<00?ooool06Ooo0P00kOoo000Eool00`00oooo
o`0Kool2003[ool000;oo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo01goo`800>Woo`00
00Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo01ooo`03003ooooo0>Koo`0000Goo`00
ooooo`0000Woo`03003ooooo00Coo`<0023oo`800>Koo`0000Goo`00ooooo`0000[oo`03003ooooo
00?oo`03003ooooo02;oo`03003ooooo0>?oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03
003ooooo02?oo`03003ooooo0>;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`0Tool2003Rool000;oo`8000[oo`8000Goo`03003ooooo02Koo`03003ooooo0=ooo`005Ooo0P00
:?oo0P00gooo000Eool00`00ooooo`0Yool00`00ooooo`3Lool001Goo`03003ooooo02[oo`800=co
o`005Ooo00<00?ooool0;?oo0P00f_oo000Eool00`00ooooo`0^ool2003Hool001Goo`80037oo`80
0=Koo`005Ooo00<00?ooool0<_oo0P00e?oo000Eool00`00ooooo`0dool00`00ooooo`3Aool001Go
o`03003ooooo03Goo`800=7oo`005Ooo00<00?ooool0=ooo00<00?ooool0c_oo000Eool00`00oooo
o`0hool2003>ool001Goo`8003_oo`800<coo`005Ooo00<00?ooool0??oo00<00?ooool0bOoo000E
ool00`00ooooo`0mool20039ool000;oo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003o
oooo03ooo`03003ooooo0<Koo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo043o
o`800<Koo`0000Goo`00ooooo`0000Ooo`D000Coo`<004;oo`03003ooooo0<?oo`0000Goo`00oooo
o`0000Ooo`04003ooooo0005ool00`00ooooo`13ool20033ool00005ool00?ooool00008ool00`00
ool00005ool00`00ooooo`15ool20031ool00005ool00?ooool00009ool20005ool00`00ooooo`17
ool00`00ooooo`2nool000;oo`8000_oo`03003ooooo00?oo`03003ooooo04Soo`800;koo`005Ooo
0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool2002kool001Goo`03003ooooo04goo`03
003ooooo0;Soo`005Ooo00<00?ooool0C_oo0P00^?oo000Eool00`00ooooo`1@ool2002fool001Go
o`03003ooooo05;oo`03003ooooo0;?oo`005Ooo0P00E?oo0P00/ooo000Eool00`00ooooo`1Eool0
0`00ooooo`2`ool001Goo`03003ooooo05Koo`800;3oo`005Ooo00<00?ooool0F?oo0P00[_oo000E
ool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`8005coo`800:_oo`005Ooo00<00?ooool0GOoo
0P00ZOoo000Eool00`00ooooo`1Oool00`00ooooo`2Vool000;oo`8000Coo`03003ooooo00?oo`80
00Goo`03003ooooo063oo`800:Koo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`1Rool00`00ooooo`2Sool00005ool00?ooool00008ool01000ooooo`001?oo0`00Hooo0P00Xooo
00001Ooo003ooooo00002?oo0`001Ooo00<00?ooool0IOoo0P00XOoo00001Ooo003ooooo00002?oo
00<00?ooool01Ooo00<00?oon0002?P0Gooo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<0
0?ooool01Ooo00<00?ooool02?oo2OP0Eooo0P00W_oo0002ool2000:ool30004ool00`00ooooo`0A
ool5n01Dool00`00ooooo`2Kool001Goo`03003ooooo01Koo`Ch057oo`8009_oo`005Ooo0P006ooo
0oP0D?oo0P00VOoo000Eool00`00ooooo`0Mool4n01>ool2002Gool001Goo`03003ooooo027oo`?h
04goo`8009Goo`005Ooo00<00?ooool09?oo1?P0Booo0P00Tooo000Eool00`00ooooo`0Xool3n01:
ool00`00ooooo`2@ool001Goo`8002coo`Ch04Ooo`80093oo`005Ooo00<00?ooool0;ooo0oP0A_oo
00<00?ooool0SOoo000Eool00`00ooooo`0bool2n015ool00`00ooooo`2<ool001Goo`03003ooooo
03Coo`;h04Coo`8008coo`005Ooo00<00?ooool0=_oo0oP0@ooo00<00?ooool0ROoo000Eool5000g
ool4n010ool20029ool001Goo`03003ooooo00;oo`T003;oo`?h03ooo`03003ooooo08Koo`005Ooo
00<00?ooool02ooo1000<Ooo0_P0?_oo0P00Q_oo000Eool00`00ooooo`0?ool4000_ool2n00nool2
0024ool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo01?oo`@002goo`Gh03_oo`80
08;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Gool6000/ool8n00eool2
0020ool00005ool00?ooool00008ool01000ooooo`001?oo0`007Ooo1000<?oo1oP0<?oo0P00O_oo
00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool08Ooo1000<ooo1oP0:ooo00<00?ooool0Nooo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo02Goo`@003Koo`Oh02Goo`8007_o
o`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Yool3000jool6n00Qool00`00
ooooo`1hool000;oo`8000[oo`8000Goo`03003ooooo02coo`@003coo`Ch01koo`8007Soo`005Ooo
0P00<Ooo1000??oo1?P07?oo0P00M_oo000Eool00`00ooooo`0dool4000lool5n00Iool00`00oooo
o`1cool001Goo`03003ooooo03Soo`H003_oo`Ch01Koo`8007?oo`005Ooo00<00?ooool0?_oo1@00
>_oo1?P05?oo00<00?ooool0L?oo000Eool00`00ooooo`13ool5000iool4n00Aool2001`ool001Go
o`8004Woo`D003Soo`?h013oo`03003ooooo06goo`005Ooo00<00?ooool0COoo1P00=Ooo1?P03Ooo
0P00KOoo000Eool00`00ooooo`1Cool5000dool3n00<ool2001[ool001Goo`03003ooooo05Soo`H0
037oo`;h00coo`03003ooooo06Soo`005Ooo00<00?ooool0G_oo1@00;_oo0_P02ooo0P00J?oo000E
ool00`00ooooo`1Sool6000Zool3n00:ool00`00ooooo`1Uool001Goo`8006[oo`P002Goo`Ch00Oo
o`8006Goo`005Ooo00<00?ooool0LOoo2P007ooo0oP01_oo0P00Hooo000Eool00`00ooooo`1kool;
000Gool2n006ool00`00ooooo`1Pool000goo`@000Coo`03003ooooo08Koo`P0017oo`;h00Goo`80
03kooa/000Ooo`003ooo00<00?ooool00ooo00<00?ooool0S_oo40000ooo0oP01?oo0P007?oo8000
8_oo000?ool00`00ooooo`03ool3002NoolX0012ool000ooo`03003ooooo00?oo`03003ooooo0:Ko
o`;h00?oo`8005_oo`003ooo00<00?ooool00ooo00<00?ooool0Z?oo0_P00ooo00<00?ooool0F?oo
000=ool30005ool00`00ooooo`2Zool01?P0oooooooo0P00F?oo000?ool00`00ooooo`03ool00`00
ooooo`2`ool2001Fool001Goo`800;?oo`03003ooooo05?oo`005Ooo00<00?ooool0/ooo0P00Dooo
000Eool00`00ooooo`2eool00`00ooooo`1@ool001Goo`03003ooooo0;Koo`80053oo`005Ooo00<0
0?ooool0^?oo00<00?ooool0COoo000Eool00`00ooooo`2iool2001=ool001Goo`800;coo`8004_o
o`005Ooo00<00?ooool0_Ooo00<00?ooool0B?oo000Eool00`00ooooo`2nool20018ool001Goo`03
003ooooo0<3oo`03003ooooo04Goo`005Ooo00<00?ooool0`Ooo0P00AOoo000Eool20034ool20013
ool001Goo`03003ooooo0<Goo`03003ooooo043oo`005Ooo00<00?ooool0a_oo0P00@?oo00000ooo
000000020003ool00`00ooooo`02ool40004ool00`00ooooo`38ool2000nool000?oo`03003ooooo
00Ooo`03003ooooo00Goo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?ooool02?oo00<0
0?ooool01?oo0`00booo0P00>ooo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3=
ool2000iool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo0<ooo`03003ooooo03Ko
o`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3@ool2000fool000?oo`03003ooooo
00Soo`8000Goo`03003ooooo0=;oo`8003Coo`005Ooo00<00?ooool0e?oo0P00<_oo000Eool2003G
ool00`00ooooo`0_ool001Goo`03003ooooo0=Ooo`8002ooo`005Ooo00<00?ooool0fOoo00<00?oo
ool0;?oo000Eool00`00ooooo`3Jool00`00ooooo`0[ool001Goo`03003ooooo0=_oo`8002_oo`00
5Ooo0P00g_oo00<00?ooool0:?oo000Eool00`00ooooo`3Nool2000Xool001Goo`03003ooooo0>3o
o`03003ooooo02Goo`005Ooo00<00?ooool0hOoo0P009Ooo000Eool00`00ooooo`3Sool2000Sool0
01Goo`800>Koo`80027oo`005Ooo00<00?ooool0iooo0P007ooo000Eool00`00ooooo`3Yool2000M
ool001Goo`03003ooooo0>_oo`03003ooooo01[oo`0000?oo`0000000P000ooo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool0k?oo0P006_oo0003ool00`00ooooo`09ool00`00ooooo`03ool0
0`00ooooo`3^ool00`00ooooo`0Gool000?oo`03003ooooo00Koo`D000Coo`<00>ooo`8001Ooo`00
0ooo00<00?ooool01_oo00@00?ooool000Goo`03003ooooo0?7oo`8001Goo`000ooo00<00?ooool0
1ooo00<00?oo00001Ooo00<00?ooool0looo00<00?ooool04_oo00001?oo000000002_oo0P001Ooo
00<00?ooool0m?oo0P004_oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3fool0
0`00ooooo`0?ool001Goo`800?Soo`8000ooo`005Ooo00<00?ooool0nOoo0P003Ooo000Eool00`00
ooooo`3kool00`00ooooo`0:ool001Goo`03003ooooo0?coo`8000[oo`005Ooo00<00?ooool0o_oo
0P002?oo000Eool2003oool2ool00`00ooooo`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, -0.117974, \
0.00578885, 0.00936656}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol2[k, 10]], {k, 0.01, 0.4}, 
      PlotStyle \[Rule] {RGBColor[0, 1, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -13.7251 18.3014 [
[.2619 -0.01155 -9 -9 ]
[.2619 -0.01155 9 0 ]
[.5 -0.01155 -9 -9 ]
[.5 -0.01155 9 0 ]
[.7381 -0.01155 -9 -9 ]
[.7381 -0.01155 9 0 ]
[.97619 -0.01155 -9 -9 ]
[.97619 -0.01155 9 0 ]
[.01131 .09246 -30 -4.5 ]
[.01131 .09246 0 4.5 ]
[.01131 .18396 -24 -4.5 ]
[.01131 .18396 0 4.5 ]
[.01131 .27547 -30 -4.5 ]
[.01131 .27547 0 4.5 ]
[.01131 .36698 -24 -4.5 ]
[.01131 .36698 0 4.5 ]
[.01131 .45848 -30 -4.5 ]
[.01131 .45848 0 4.5 ]
[.01131 .54999 -24 -4.5 ]
[.01131 .54999 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .00095 m
.2619 .0072 L
s
[(0.1)] .2619 -0.01155 0 1 Mshowa
.5 .00095 m
.5 .0072 L
s
[(0.2)] .5 -0.01155 0 1 Mshowa
.7381 .00095 m
.7381 .0072 L
s
[(0.3)] .7381 -0.01155 0 1 Mshowa
.97619 .00095 m
.97619 .0072 L
s
[(0.4)] .97619 -0.01155 0 1 Mshowa
.125 Mabswid
.07143 .00095 m
.07143 .0047 L
s
.11905 .00095 m
.11905 .0047 L
s
.16667 .00095 m
.16667 .0047 L
s
.21429 .00095 m
.21429 .0047 L
s
.30952 .00095 m
.30952 .0047 L
s
.35714 .00095 m
.35714 .0047 L
s
.40476 .00095 m
.40476 .0047 L
s
.45238 .00095 m
.45238 .0047 L
s
.54762 .00095 m
.54762 .0047 L
s
.59524 .00095 m
.59524 .0047 L
s
.64286 .00095 m
.64286 .0047 L
s
.69048 .00095 m
.69048 .0047 L
s
.78571 .00095 m
.78571 .0047 L
s
.83333 .00095 m
.83333 .0047 L
s
.88095 .00095 m
.88095 .0047 L
s
.92857 .00095 m
.92857 .0047 L
s
.25 Mabswid
0 .00095 m
1 .00095 L
s
.02381 .09246 m
.03006 .09246 L
s
[(0.755)] .01131 .09246 1 0 Mshowa
.02381 .18396 m
.03006 .18396 L
s
[(0.76)] .01131 .18396 1 0 Mshowa
.02381 .27547 m
.03006 .27547 L
s
[(0.765)] .01131 .27547 1 0 Mshowa
.02381 .36698 m
.03006 .36698 L
s
[(0.77)] .01131 .36698 1 0 Mshowa
.02381 .45848 m
.03006 .45848 L
s
[(0.775)] .01131 .45848 1 0 Mshowa
.02381 .54999 m
.03006 .54999 L
s
[(0.78)] .01131 .54999 1 0 Mshowa
.125 Mabswid
.02381 .01925 m
.02756 .01925 L
s
.02381 .03755 m
.02756 .03755 L
s
.02381 .05585 m
.02756 .05585 L
s
.02381 .07415 m
.02756 .07415 L
s
.02381 .11076 m
.02756 .11076 L
s
.02381 .12906 m
.02756 .12906 L
s
.02381 .14736 m
.02756 .14736 L
s
.02381 .16566 m
.02756 .16566 L
s
.02381 .20226 m
.02756 .20226 L
s
.02381 .22057 m
.02756 .22057 L
s
.02381 .23887 m
.02756 .23887 L
s
.02381 .25717 m
.02756 .25717 L
s
.02381 .29377 m
.02756 .29377 L
s
.02381 .31207 m
.02756 .31207 L
s
.02381 .33037 m
.02756 .33037 L
s
.02381 .34868 m
.02756 .34868 L
s
.02381 .38528 m
.02756 .38528 L
s
.02381 .40358 m
.02756 .40358 L
s
.02381 .42188 m
.02756 .42188 L
s
.02381 .44018 m
.02756 .44018 L
s
.02381 .47678 m
.02756 .47678 L
s
.02381 .49509 m
.02756 .49509 L
s
.02381 .51339 m
.02756 .51339 L
s
.02381 .53169 m
.02756 .53169 L
s
.02381 .56829 m
.02756 .56829 L
s
.02381 .58659 m
.02756 .58659 L
s
.02381 .60489 m
.02756 .60489 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.04762 .01472 m
.05637 .01502 L
.06596 .01547 L
.08529 .01671 L
.10656 .01861 L
.12637 .02089 L
.16439 .02663 L
.2048 .03471 L
.24373 .04443 L
.28118 .05558 L
.32103 .06939 L
.35939 .08459 L
.39628 .10099 L
.43556 .12039 L
.47337 .14095 L
.51356 .16487 L
.55228 .18995 L
.58951 .21597 L
.62914 .24574 L
.6673 .27645 L
.70784 .31131 L
.7469 .34713 L
.78449 .38367 L
.82447 .42484 L
.86297 .46678 L
.89999 .50906 L
.9394 .55657 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool005Ooo`8000Coo`03003ooooo00;oo`@002koo`8000Co
o`03003ooooo00;oo`@002koo`8000Coo`03003ooooo00?oo`80033oo`8000Coo`03003ooooo00Co
o`03003ooooo007oo`00E_oo00@00?ooool000[oo`03003ooooo02coo`04003ooooo0008ool00`00
ooooo`0^ool01000ooooo`002?oo00@00?ooool002koo`04003ooooo000:ool00`00ooooo`01ool0
05Koo`04003ooooo000:ool00`00ooooo`0/ool01000ooooo`002Ooo00<00?ooool0;Ooo00@00?oo
ool000_oo`03003ooooo02coo`04003ooooo0007ool50002ool005Koo`04003ooooo000:ool00`00
ooooo`0/ool01000ooooo`002_oo00<00?ooool0;?oo00@00?ooool000Woo`8002ooo`04003ooooo
0007ool01000ooooo`000ooo001Fool01000ooooo`002_oo00<00?ooool0;?oo00@00?ooool000_o
o`03003ooooo02_oo`04003ooooo000;ool00`00ooooo`0/ool01000ooooo`002?oo00<00?oo0000
0ooo001Fool01000ooooo`002?oo0`00;_oo00@00?ooool000Soo`04003ooooo000]ool01000oooo
o`002?oo00@00?ooool002koo`04003ooooo0009ool20003ool005Ooo`8000_oo`03003ooooo02go
o`8000[oo`8002ooo`8000[oo`80033oo`8000_oo`03003ooooo007oo`00oooo8Ooo003ooolQool0
0?ooob7oo`006oooo`000`000ooo000Qool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`07ool0027oo`03003ooooo0?coo`008Ooo
00<00?ooool00ooo10OPmOoo000Qool00`00ooooo`07ool>1n3Wool0027oo`8001Koo`L7h>3oo`00
8Ooo00<00?ooool07?oo1`OPfOoo000Qool00`00ooooo`0Sool31n3Fool0027oo`03003ooooo02Ko
o`@7h=;oo`008Ooo0P00:ooo10OPc_oo000Qool00`00ooooo`0^ool51n39ool0027oo`03003ooooo
03?oo`@7h<Goo`008Ooo00<00?ooool0=ooo0`OP`_oo000Qool00`00ooooo`0jool41n2nool0027o
o`8003ooo`<7h;_oo`008Ooo00<00?ooool0@Ooo0`OP^?oo000Qool00`00ooooo`14ool41n2dool0
027oo`03003ooooo04Soo`<7h;7oo`008Ooo00<00?ooool0Booo0POP[ooo000Qool2001>ool31n2/
ool0027oo`03003ooooo053oo`87h:[oo`000_oo0P001?oo00<00?ooool00ooo00<00?ooool00_oo
0`000ooo0`001Ooo00<00?ooool0D_oo0POPZ?oo00001Ooo003ooooo00002Ooo00<00?ooool01Ooo
00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0E?oo0POPY_oo00001Ooo003ooooo00002_oo
00<00?ooool01?oo00<00?ooool00ooo00<00?ooool00_oo0P00Eooo0POPY?oo00001Ooo003ooooo
00002_oo00@00?ooooooo`<000?oo`<000Goo`03003ooooo05Soo`87h:;oo`0000Goo`00ooooo`00
00_oo`04003ooooo0005ool00`00ooooo`05ool00`00ooooo`1Jool21n2Pool00005ool00?ooool0
0008ool01000ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0G?oo0POPW_oo
0002ool20009ool40002ool40002ool40004ool00`00ooooo`1Nool21n2Lool0027oo`80067oo`87
h9[oo`008Ooo00<00?ooool0H_oo0POPV?oo000Qool00`00ooooo`1Tool21n2Fool0027oo`03003o
oooo06Koo`87h9Coo`008Ooo00<00?ooool0J?oo0POPT_oo000Qool2001[ool21n2@ool0027oo`03
003ooooo06coo`87h8koo`008Ooo00<00?ooool0K_oo0POPS?oo000Qool00`00ooooo`1`ool00`OP
ooooo`29ool0027oo`8007;oo`87h8Woo`008Ooo00<00?ooool0Looo0POPQooo000Qool00`00oooo
o`1eool21n25ool0027oo`03003ooooo07Ooo`87h8?oo`008Ooo00<00?ooool0NOoo0POPPOoo000Q
ool2001lool00`OPooooo`1nool0027oo`03003ooooo07coo`87h7koo`008Ooo00<00?ooool0O_oo
0POPO?oo0008ool20004ool00`00ooooo`03ool00`00ooooo`03ool20005ool00`00ooooo`20ool0
0`OPooooo`1iool000Ooo`04003ooooo0009ool00`00ooooo`02ool01000ooooo`001?oo00<00?oo
ool0POoo0POPNOoo0007ool01000ooooo`002_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`80
08Coo`87h7Ooo`001ooo00@00?ooool000[oo`04003oooooool30005ool00`00ooooo`25ool00`OP
ooooo`1dool000Ooo`04003ooooo000;ool01000ooooo`001ooo00<00?ooool0Q_oo00<7h?ooool0
Looo0007ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo08Ooo`87
h7?oo`002?oo0P002Ooo10000ooo0`001?oo00<00?ooool0ROoo00<7h?ooool0L?oo000Qool2002;
ool00`OPooooo`1_ool0027oo`03003ooooo08_oo`87h6ooo`008Ooo00<00?ooool0SOoo00<7h?oo
ool0K?oo000Qool00`00ooooo`2>ool00`OPooooo`1[ool0027oo`80093oo`031n3ooooo06[oo`00
8Ooo00<00?ooool0T?oo0POPJ_oo000Qool00`00ooooo`2Bool00`OPooooo`1Wool0027oo`03003o
oooo09?oo`031n3ooooo06Koo`008Ooo00<00?ooool0U?oo00<7h?ooool0IOoo000Qool2002Fool2
1n1Uool0027oo`03003ooooo09Ooo`031n3ooooo06;oo`008Ooo00<00?ooool0V?oo00<7h?ooool0
HOoo000Qool00`00ooooo`2Iool00`OPooooo`1Pool0027oo`03003ooooo09[oo`87h63oo`008Ooo
0P00WOoo00<7h?ooool0GOoo000Qool00`00ooooo`2Mool00`OPooooo`1Lool000;oo`8000Coo`03
003ooooo00?oo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo09koo`031n3ooooo05_oo`00
00Goo`00ooooo`0000Woo`03003ooooo00;oo`04003ooooo0005ool00`00ooooo`02ool00`00oooo
o`2Oool21n1Kool00005ool00?ooool0000:ool01@00oooooooo00000_oo00<00?ooool00ooo00<0
0?ooool00_oo0P00X_oo00<7h?ooool0F?oo00001Ooo003ooooo00002_oo00@00?ooooooo`<000?o
o`<000Goo`03003ooooo0:;oo`031n3ooooo05Ooo`0000Goo`00ooooo`0000_oo`04003ooooo0005
ool00`00ooooo`05ool00`00ooooo`2Sool00`OPooooo`1Fool00005ool00?ooool00008ool01000
ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0Y?oo0POPE_oo0002ool20009
ool40003ool30002ool40004ool00`00ooooo`2Vool00`OPooooo`1Cool0027oo`800:Soo`031n3o
oooo05;oo`008Ooo00<00?ooool0Z?oo00<7h?ooool0DOoo000Qool00`00ooooo`2Yool00`OPoooo
o`1@ool0027oo`03003ooooo0:[oo`87h53oo`008Ooo00<00?ooool0[?oo00<7h?ooool0COoo000Q
ool2002^ool00`OPooooo`1<ool0027oo`03003ooooo0:koo`031n3ooooo04_oo`008Ooo00<00?oo
ool0[ooo00<7h?ooool0B_oo000Qool00`00ooooo`2`ool00`OPooooo`19ool0027oo`03003ooooo
0;7oo`031n3ooooo04Soo`008Ooo0P00/ooo0POPB?oo000Qool00`00ooooo`2dool00`OPooooo`15
ool0027oo`03003ooooo0;Goo`031n3ooooo04Coo`008Ooo00<00?ooool0]_oo00<7h?ooool0@ooo
000Qool2002hool00`OPooooo`12ool0027oo`03003ooooo0;Soo`031n3ooooo047oo`008Ooo00<0
0?ooool0^Ooo00<7h?ooool0@?oo0008ool20004ool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`04ool00`00ooooo`2jool00`OPooooo`0oool000Ooo`04003ooooo0009ool00`00ooooo`03
ool00`00ooooo`04ool00`00ooooo`2kool00`OPooooo`0nool000Ooo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`03ool2002mool00`OPooooo`0mool000Ooo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`2mool00`OPooooo`0lool000Ooo`04003ooooo000;
ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`2nool00`OPooooo`0kool000Ooo`04003o
oooo0008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo0;ooo`031n3ooooo03[oo`00
2?oo0P002Ooo10000_oo10001?oo00<00?ooool0`?oo00<7h?ooool0>Ooo000Qool20032ool00`OP
ooooo`0hool0027oo`03003ooooo0<;oo`031n3ooooo03Ooo`008Ooo00<00?ooool0`ooo00<7h?oo
ool0=_oo000Qool00`00ooooo`34ool00`OPooooo`0eool0027oo`800<Koo`031n3ooooo03Coo`00
8Ooo00<00?ooool0a_oo00<7h?ooool0<ooo000Qool00`00ooooo`37ool00`OPooooo`0bool0027o
o`03003ooooo0<Soo`031n3ooooo037oo`008Ooo00<00?ooool0bOoo00<7h?ooool0<?oo000Qool2
003;ool00`OPooooo`0_ool0027oo`03003ooooo0<_oo`031n3ooooo02koo`008Ooo00<00?ooool0
c?oo00<7h?ooool0;Ooo000Qool00`00ooooo`3=ool00`OPooooo`0/ool0027oo`03003ooooo0<go
o`031n3ooooo02coo`008Ooo0P00cooo00<7h?ooool0:ooo000Qool00`00ooooo`3?ool00`OPoooo
o`0Zool000;oo`8000Coo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00;oo`<000Goo`03
003ooooo0=3oo`031n3ooooo02Woo`0000Goo`00ooooo`0000Woo`03003ooooo00?oo`03003ooooo
00Goo`03003ooooo00;oo`03003ooooo0=7oo`031n3ooooo02Soo`0000Goo`00ooooo`0000[oo`03
003ooooo00?oo`03003ooooo00Coo`03003ooooo00;oo`800=?oo`031n3ooooo02Ooo`0000Goo`00
ooooo`0000[oo`03003ooooo00?oo`04003oooooool30005ool00`00ooooo`3Bool00`OPooooo`0W
ool00005ool00?ooool0000;ool00`00ooooo`03ool01000ooooo`001ooo00<00?ooool0dooo00<7
h?ooool09_oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool00`00oooo
o`05ool00`00ooooo`3Dool00`OPooooo`0Uool000;oo`8000Woo`@000;oo`@000;oo`@000Coo`03
003ooooo0=Goo`031n3ooooo02Coo`008Ooo0P00eooo00<7h?ooool08ooo000Qool00`00ooooo`3G
ool00`OPooooo`0Rool0027oo`03003ooooo0=Soo`031n3ooooo027oo`008Ooo00<00?ooool0fOoo
00<7h?ooool08?oo000Qool00`00ooooo`3Jool00`OPooooo`0Oool0027oo`800=coo`031n3ooooo
01koo`008Ooo00<00?ooool0g?oo00<7h?ooool07Ooo000Qool00`00ooooo`3Mool00`OPooooo`0L
ool0027oo`03003ooooo0=koo`031n3ooooo01_oo`008Ooo00<00?ooool0gooo00<7h?ooool06_oo
000Qool2003Qool00`OPooooo`0Iool0027oo`03003ooooo0>3oo`031n3ooooo01Woo`008Ooo00<0
0?ooool0hOoo00<7h?ooool06?oo000Qool00`00ooooo`3Rool00`OPooooo`0Gool0027oo`800>Co
o`031n3ooooo01Koo`008Ooo00<00?ooool0hooo00<7h?ooool05_oo000Qool00`00ooooo`3Tool0
0`OPooooo`0Eool000Soo`8000Coo`03003ooooo00?oo`03003ooooo00?oo`8000Goo`03003ooooo
0>Goo`031n3ooooo01Coo`001ooo00@00?ooool000Woo`03003ooooo00;oo`04003ooooo0004ool0
0`00ooooo`3Vool00`OPooooo`0Cool000Ooo`04003ooooo000:ool01@00oooooooo00000_oo00<0
0?ooool00_oo0P00iooo00<7h?ooool04ooo0007ool01000ooooo`002_oo00<00?ooool00_oo0P00
1Ooo00<00?ooool0iooo00<7h?ooool04_oo0007ool01000ooooo`002ooo00@00?ooool000;oo`03
003ooooo00;oo`03003ooooo0>Soo`031n3ooooo017oo`001ooo00@00?ooool000Soo`04003ooooo
0002ool01000ooooo`001?oo00<00?ooool0jOoo00<7h?ooool04?oo0008ool20009ool40003ool2
0005ool00`00ooooo`3Zool00`OPooooo`0?ool0027oo`800>_oo`031n3ooooo00ooo`008Ooo00<0
0?ooool0jooo00<7h?ooool03_oo000Qool00`00ooooo`3/ool00`OPooooo`0=ool0027oo`03003o
oooo0>goo`031n3ooooo00coo`008Ooo0P00kooo00<7h?ooool02ooo000Qool00`00ooooo`3_ool0
0`OPooooo`0:ool0027oo`03003ooooo0>ooo`031n3ooooo00[oo`008Ooo00<00?ooool0l?oo00<7
h?ooool02Ooo000Qool00`00ooooo`3aool00`OPooooo`08ool0027oo`800??oo`031n3ooooo00Oo
o`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`00oooo8Ooo
003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0542745, 0.746753, \
0.00163409, 0.000212589}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %1070]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -0.183123 0.524294 [
[.15079 .32867 -9 -9 ]
[.15079 .32867 9 0 ]
[.27778 .32867 -9 -9 ]
[.27778 .32867 9 0 ]
[.40476 .32867 -9 -9 ]
[.40476 .32867 9 0 ]
[.53175 .32867 -9 -9 ]
[.53175 .32867 9 0 ]
[.65873 .32867 -3 -9 ]
[.65873 .32867 3 0 ]
[.78571 .32867 -9 -9 ]
[.78571 .32867 9 0 ]
[.9127 .32867 -9 -9 ]
[.9127 .32867 9 0 ]
[.01131 .0266 -18 -4.5 ]
[.01131 .0266 0 4.5 ]
[.01131 .13145 -18 -4.5 ]
[.01131 .13145 0 4.5 ]
[.01131 .23631 -18 -4.5 ]
[.01131 .23631 0 4.5 ]
[.01131 .44603 -18 -4.5 ]
[.01131 .44603 0 4.5 ]
[.01131 .55089 -18 -4.5 ]
[.01131 .55089 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .34117 m
.15079 .34742 L
s
[(0.2)] .15079 .32867 0 1 Mshowa
.27778 .34117 m
.27778 .34742 L
s
[(0.4)] .27778 .32867 0 1 Mshowa
.40476 .34117 m
.40476 .34742 L
s
[(0.6)] .40476 .32867 0 1 Mshowa
.53175 .34117 m
.53175 .34742 L
s
[(0.8)] .53175 .32867 0 1 Mshowa
.65873 .34117 m
.65873 .34742 L
s
[(1)] .65873 .32867 0 1 Mshowa
.78571 .34117 m
.78571 .34742 L
s
[(1.2)] .78571 .32867 0 1 Mshowa
.9127 .34117 m
.9127 .34742 L
s
[(1.4)] .9127 .32867 0 1 Mshowa
.125 Mabswid
.05556 .34117 m
.05556 .34492 L
s
.0873 .34117 m
.0873 .34492 L
s
.11905 .34117 m
.11905 .34492 L
s
.18254 .34117 m
.18254 .34492 L
s
.21429 .34117 m
.21429 .34492 L
s
.24603 .34117 m
.24603 .34492 L
s
.30952 .34117 m
.30952 .34492 L
s
.34127 .34117 m
.34127 .34492 L
s
.37302 .34117 m
.37302 .34492 L
s
.43651 .34117 m
.43651 .34492 L
s
.46825 .34117 m
.46825 .34492 L
s
.5 .34117 m
.5 .34492 L
s
.56349 .34117 m
.56349 .34492 L
s
.59524 .34117 m
.59524 .34492 L
s
.62698 .34117 m
.62698 .34492 L
s
.69048 .34117 m
.69048 .34492 L
s
.72222 .34117 m
.72222 .34492 L
s
.75397 .34117 m
.75397 .34492 L
s
.81746 .34117 m
.81746 .34492 L
s
.84921 .34117 m
.84921 .34492 L
s
.88095 .34117 m
.88095 .34492 L
s
.94444 .34117 m
.94444 .34492 L
s
.97619 .34117 m
.97619 .34492 L
s
.25 Mabswid
0 .34117 m
1 .34117 L
s
.02381 .0266 m
.03006 .0266 L
s
[(0.4)] .01131 .0266 1 0 Mshowa
.02381 .13145 m
.03006 .13145 L
s
[(0.6)] .01131 .13145 1 0 Mshowa
.02381 .23631 m
.03006 .23631 L
s
[(0.8)] .01131 .23631 1 0 Mshowa
.02381 .44603 m
.03006 .44603 L
s
[(1.2)] .01131 .44603 1 0 Mshowa
.02381 .55089 m
.03006 .55089 L
s
[(1.4)] .01131 .55089 1 0 Mshowa
.125 Mabswid
.02381 .05281 m
.02756 .05281 L
s
.02381 .07902 m
.02756 .07902 L
s
.02381 .10524 m
.02756 .10524 L
s
.02381 .15767 m
.02756 .15767 L
s
.02381 .18388 m
.02756 .18388 L
s
.02381 .2101 m
.02756 .2101 L
s
.02381 .26253 m
.02756 .26253 L
s
.02381 .28874 m
.02756 .28874 L
s
.02381 .31496 m
.02756 .31496 L
s
.02381 .36739 m
.02756 .36739 L
s
.02381 .3936 m
.02756 .3936 L
s
.02381 .41982 m
.02756 .41982 L
s
.02381 .47225 m
.02756 .47225 L
s
.02381 .49846 m
.02756 .49846 L
s
.02381 .52467 m
.02756 .52467 L
s
.02381 .00038 m
.02756 .00038 L
s
.02381 .5771 m
.02756 .5771 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.03016 .21049 m
.03249 .2105 L
.03505 .21051 L
.0402 .21055 L
.04588 .2106 L
.05116 .21067 L
.0613 .21083 L
.07207 .21107 L
.08245 .21134 L
.09244 .21166 L
.10307 .21206 L
.1133 .21249 L
.12314 .21296 L
.13361 .21352 L
.14369 .21411 L
.15441 .21479 L
.16473 .21551 L
.17466 .21626 L
.18523 .21711 L
.19541 .21799 L
.20622 .21899 L
.21663 .22002 L
.22666 .22106 L
.23732 .22224 L
.24759 .22344 L
.25746 .22465 L
.26797 .22602 L
.27778 .22735 L
s
1 0 0 r
.03016 .1407 m
.03323 .14078 L
.03608 .14088 L
.03922 .14102 L
.04257 .1412 L
.04942 .14169 L
.05566 .14226 L
.06995 .14402 L
.08347 .14625 L
.10882 .15182 L
.13579 .15958 L
.16176 .16859 L
.18674 .1785 L
.21332 .19018 L
.23891 .20237 L
.26612 .21619 L
.27971 .22338 L
.28633 .22695 L
.28785 .22774 L
.28946 .22859 L
.29012 .22887 L
.29083 .22911 L
.2916 .22929 L
.29233 .22943 L
.29535 .22989 L
.29861 .23038 L
.30453 .23131 L
.31754 .23341 L
.34484 .23823 L
.37113 .24342 L
.39643 .24896 L
.42335 .2555 L
.44926 .26248 L
.47418 .26989 L
.50071 .27862 L
.52625 .28793 L
.5534 .29892 L
.57955 .31068 L
.6047 .32318 L
.63147 .33785 L
.65724 .35336 L
.65873 .3543 L
s
0 g
.24557 0 m
.26647 .01726 L
.30764 .05126 L
.34731 .08401 L
.38546 .11552 L
.42606 .14904 L
.46515 .18132 L
.50273 .21235 L
.54275 .2454 L
.58126 .2772 L
.62221 .31102 L
.66166 .34359 L
.69959 .37492 L
.73997 .40825 L
.77884 .44035 L
.82014 .47446 L
.85994 .50732 L
.89823 .53895 L
.93896 .57258 L
.97619 .60332 L
s
.03016 .2106 m
.03133 .21065 L
.03239 .2107 L
.03362 .21051 L
.03477 .21051 L
.03585 .21091 L
.03684 .21098 L
.03908 .21116 L
.04372 .21163 L
.04884 .21227 L
.0584 .21382 L
.06854 .21589 L
.11039 .22747 L
.15041 .24049 L
.18892 .25309 L
.22987 .26588 L
.26932 .2774 L
.30725 .28767 L
.34763 .29773 L
.3865 .30656 L
.4278 .31501 L
.4676 .3222 L
.5059 .32819 L
.54663 .33345 L
.56691 .3356 L
.58585 .3373 L
.60522 .33872 L
.62357 .33976 L
.63325 .34018 L
.64379 .34055 L
.64843 .34068 L
.65336 .3408 L
.65767 .34088 L
.66222 .34095 L
.66454 .34098 L
.66707 .34101 L
.66941 .34103 L
.67153 .34105 L
.67273 .34105 L
.67404 .34106 L
.67476 .34106 L
.67542 .34106 L
.67672 .34107 L
.67784 .34107 L
.67907 .34107 L
.68036 .34107 L
.68156 .34107 L
.68268 .34106 L
.68391 .34106 L
.68613 .34105 L
Mistroke
.6883 .34104 L
.69031 .34102 L
.69487 .34097 L
.69972 .34091 L
.70996 .34072 L
.72103 .34044 L
.74107 .33979 L
.78186 .33804 L
.82113 .33614 L
.8589 .33433 L
.89911 .33256 L
.93781 .33102 L
.97619 .32968 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool2000jool00`00ooooo`3<ool001Goo`03003ooooo03[oo`03
003ooooo0<_oo`005Ooo00<00?ooool0>ooo00<00?ooool0b_oo000Eool00`00ooooo`0lool2003:
ool001Goo`03003ooooo03koo`03003ooooo0<Ooo`000_oo0P001?oo00<00?ooool01?oo00<00?oo
ool00ooo00<00?ooool0?ooo00<00?ooool0a_oo00001Ooo003ooooo00002_oo00<00?ooool00ooo
00<00?ooool0@?oo00<00?ooool0aOoo00001Ooo003ooooo00001ooo1@001?oo0`00@Ooo0P00aOoo
00001Ooo003ooooo00001ooo00@00?ooool000Goo`03003ooooo04?oo`03003ooooo0<;oo`0000Go
o`00ooooo`0000Soo`03003oo`0000Goo`03003ooooo04Coo`03003ooooo0<7oo`0000Goo`00oooo
o`0000Woo`8000Goo`03003ooooo04Goo`03003ooooo0<3oo`000_oo0P002ooo00<00?ooool00ooo
00<00?ooool0A_oo00<00?ooool0_ooo000Eool00`00ooooo`17ool2002oool001Goo`03003ooooo
04Woo`03003ooooo0;coo`005Ooo0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool00`00
ooooo`2jool001Goo`03003ooooo04coo`800;[oo`005Ooo00<00?ooool0C_oo00<00?ooool0]ooo
000Eool00`00ooooo`1?ool00`00ooooo`2fool001Goo`03003ooooo053oo`03003ooooo0;Goo`00
5Ooo00<00?ooool0DOoo00<00?ooool0]?oo000Eool2001Cool2002dool001Goo`03003ooooo05Co
o`03003ooooo0;7oo`005Ooo00<00?ooool0EOoo00<00?ooool0/?oo000Eool00`00ooooo`1Fool0
0`00ooooo`2_ool001Goo`03003ooooo05Ooo`800:ooo`005Ooo00<00?ooool0FOoo00<00?ooool0
[?oo000Eool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`03003ooooo05_oo`03003ooooo0:[o
o`005Ooo0P00GOoo0P00Z_oo000Eool00`00ooooo`1Nool00`00ooooo`2Wool001Goo`03003ooooo
05ooo`03003ooooo0:Koo`005Ooo00<00?ooool0H?oo00<00?ooool0YOoo000Eool00`00ooooo`1Q
ool00`00ooooo`2Tool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo06;oo`03003o
oooo0:?oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1Sool00`00ooooo`2R
ool00005ool00?ooool00008ool01000ooooo`001?oo0`00I?oo0P00X_oo00001Ooo003ooooo0000
2?oo0`001Ooo00<00?ooool0I_oo00<00?ooool0Wooo00001Ooo003ooooo00002?oo00<00?ooool0
1Ooo00<00?oon0000oP0I?oo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo
00<00?ooool00ooo2?P0GOoo00<00?ooool0WOoo0002ool2000:ool30004ool00`00ooooo`0;ool6
n01Hool00`00ooooo`2Lool001Goo`03003ooooo017oo`Gh05Coo`03003ooooo09_oo`005Ooo00<0
0?ooool05_oo1?P0DOoo0P00Vooo000Eool2000Kool3n01@ool00`00ooooo`2Hool001Goo`03003o
oooo01goo`;h04ooo`03003ooooo09Ooo`005Ooo00<00?ooool07ooo0_P0C_oo0P00Uooo000Eool0
0`00ooooo`0Qool3n01=ool00`00ooooo`2Dool001Goo`03003ooooo02Coo`;h04coo`03003ooooo
09?oo`005Ooo00<00?ooool09_oo0_P0Booo0P00Tooo000Eool00`00ooooo`0Xool3n01:ool00`00
ooooo`2@ool001Goo`8002coo`;h04Woo`03003ooooo08ooo`005Ooo00<00?ooool0;Ooo0_P0B?oo
00<00?ooool0S_oo000Eool00`00ooooo`0_ool3n016ool00`00ooooo`2=ool001Goo`03003ooooo
03;oo`;h04Goo`03003ooooo08coo`005Ooo00<00?ooool0=?oo0_P0A?oo0P00S?oo000Eool00`00
ooooo`0fool2n014ool00`00ooooo`29ool001Goo`03003ooooo03Soo`;h04?oo`03003ooooo08So
o`005Ooo1@003@OP:ooo0_P0@_oo00<00?ooool0Qooo000Eool00`00ooooo`02ool60007oolC1n0J
ool2n011ool00`00ooooo`26ool001Goo`03003ooooo00Soo`<001Ooo``7h13oo`;h043oo`03003o
oooo08Goo`005Ooo00<00?ooool02ooo10007ooo2`OP1ooo0_P0?ooo0P00QOoo000Eool00`00oooo
o`0?ool4000Vool81n000ooon03h000oool00`00ooooo`22ool000;oo`8000Coo`03003ooooo00?o
o`8000Goo`03003ooooo01?oo`<002_oo`87h004oooh0?P0n00mool00`00ooooo`21ool00005ool0
0?ooool00008ool01000ooooo`001?oo00<00?ooool05_oo1000;Ooo2?P0=_oo00<00?ooool0P?oo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`<001[oo`@0037oo`Ch03?oo`03003ooooo07oo
o`0000Goo`00ooooo`0000Woo`8000Goo`03003ooooo01koo`<003;oo`Gh02ooo`8007ooo`0000Go
o`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Qool3000dool5n00/ool00`00ooooo`1l
ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool09?oo0P00=ooo1?P0:Ooo00<0
0?ooool0Nooo0002ool2000:ool20005ool00`00ooooo`0Vool3000hool5n00Uool2001kool001Go
o`03003ooooo02Woo`<003[oo`Kh027oo`03003ooooo07Soo`005Ooo00<00?ooool0;?oo1000??oo
1?P07_oo00<00?ooool0Mooo000Eool2000aool4000lool3n00Lool2001gool001Goo`03003ooooo
03Coo`<003coo`?h01_oo`03003ooooo07Coo`005Ooo00<00?ooool0=ooo1000>ooo0oP06Ooo00<0
0?ooool0Looo000Eool00`00ooooo`0kool4000jool2n00Hool2001cool001Goo`03003ooooo03oo
o`<003Woo`?h01Ooo`03003ooooo073oo`005Ooo00<00?ooool0@_oo0`00>Ooo0oP05Ooo00<00?oo
ool0Kooo000Eool00`00ooooo`15ool4000hool4n00Bool00`00ooooo`1^ool001Goo`03003ooooo
04Woo`<003Woo`?h013oo`03003ooooo06goo`005Ooo0P00COoo1000>?oo0_P03ooo00<00?ooool0
K?oo000Eool00`00ooooo`1@ool4000fool2n00>ool2001/ool001Goo`03003ooooo05Coo`@003Co
o`?h00goo`03003ooooo06Woo`005Ooo00<00?ooool06?oo0P001?oo00<00?ooool00_oo10005?oo
0P001?oo00<00?ooool01?oo00<00?ooool03Ooo20001?oo00<00?ooool00ooo0P005?oo0P001?oo
00<00?ooool00ooo0P0000?oooP0n0003?oo00<00?ooool02?oo10006?oo10000ooo00<00?ooool0
0_oo10004ooo10000ooo00<00?ooool01?oo00<00?ooool04?oo000Eool00`00ooooo`0Gool01000
ooooo`002?oo00<00?ooool05?oo00@00?ooool000[oo`03003ooooo01;oo`D000Ooo`04003ooooo
000Bool01000ooooo`002?oo00@00?ooool000;oo`;h00_oo`03003ooooo00Woo`03003ooooo01Wo
o`03003ooooo00Ooo`03003ooooo01Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo00<00?oo
ool05ooo00@00?ooool000Woo`03003ooooo01?oo`04003ooooo0007ool5000Cool01@00ooooo`00
ool010000ooo00@00?ooool001;oo`04003ooooo0008ool01000ooooo`001?oo0_P02_oo00<00?oo
ool02?oo00<00?ooool06Ooo00<00?ooool02?oo00<00?ooool05Ooo00<00?ooool01_oo1@004Ooo
000Eool00`00ooooo`0Gool01000ooooo`002_oo00<00?ooool04_oo00@00?ooool000Ooo`04003o
oooo000Dool01000ooooo`001Ooo1P004ooo00@00?ooool000Woo`8000Ooo`;h00Woo`03003ooooo
00Ooo`03003ooooo01Woo`03003ooooo00Woo`03003ooooo01Coo`03003ooooo00Koo`04003ooooo
000Bool001Goo`8001Soo`04003ooooo000;ool00`00ooooo`0Aool01000ooooo`002?oo00<00?oo
00005?oo00@00?ooool000Soo`D0017oo`04003ooooo0008ool01000ooooo`002?oo0_P02?oo00<0
0?ooool01_oo00<00?ooool06Ooo00<00?ooool02_oo00<00?ooool04ooo00<00?ooool01ooo00<0
0?oo00004_oo000Eool00`00ooooo`0Gool01000ooooo`002?oo00@00?ooool001?oo`04003ooooo
0009ool2000Dool01000ooooo`002?oo00<00?ooool00_oo1P002ooo00@00?ooool000Soo`04003o
oooo000:ool2n007ool20004ool3000Iool30009ool01000ooooo`004ooo0`002_oo0P004_oo000E
ool00`00ooooo`0Hool2000:ool2000Eool2000;ool00`00ooooo`0Cool2000:ool30007ool70005
ool2000:ool2000=ool00oP0ooooo`06ool00`00ooooo`03ool00`00ooooo`0Iool00`00ooooo`08
ool2000Fool00`00ooooo`09ool00`00ooooo`0@ool001Goo`03003ooooo07_oo`P001Woo`;h00Oo
o`03003ooooo063oo`005Ooo00<00?ooool0Pooo1P005Ooo0_P01_oo00<00?ooool0?Ooo6`001ooo
000Eool00`00ooooo`29ool;000<ool2n005ool2000XoolE000Rool001Goo`03003ooooo09Coo`h0
0003n03ooooo00Coo`03003ooooo01;ooa<003Ooo`003_ooo`004`00000Eool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool01000oooooooo0_P00ooo0P001_oo00<0
0?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo000E
ool00`00ooooo`2Wool2n003ool2001Jool001Goo`03003ooooo0:Woo`;h00?oo`03003ooooo05Oo
o`005Ooo00<00?ooool0[ooo00<00?ooool0E_oo000Eool00`00ooooo`2`ool00`00ooooo`1Eool0
01Goo`03003ooooo0;7oo`03003ooooo05Coo`005Ooo0P00/ooo0P00E?oo000Eool00`00ooooo`2d
ool00`00ooooo`1Aool001Goo`03003ooooo0;Goo`03003ooooo053oo`005Ooo00<00?ooool0]_oo
00<00?ooool0Cooo000Eool00`00ooooo`2gool2001?ool001Goo`03003ooooo0;Woo`03003ooooo
04coo`005Ooo00<00?ooool0^_oo00<00?ooool0Booo000Eool2002lool00`00ooooo`1:ool001Go
o`03003ooooo0;coo`03003ooooo04Woo`005Ooo00<00?ooool0_Ooo0P00BOoo000Eool00`00oooo
o`2oool00`00ooooo`16ool001Goo`03003ooooo0<3oo`03003ooooo04Goo`005Ooo00<00?ooool0
`Ooo00<00?ooool0A?oo000Eool00`00ooooo`32ool00`00ooooo`13ool001Goo`800<Coo`03003o
oooo04;oo`005Ooo00<00?ooool0a?oo00<00?ooool0@Ooo000Eool00`00ooooo`35ool20011ool0
01Goo`03003ooooo0<Ooo`03003ooooo03koo`005Ooo00<00?ooool0b?oo00<00?ooool0?Ooo0000
0ooo000000020003ool00`00ooooo`02ool40004ool00`00ooooo`39ool00`00ooooo`0lool000?o
o`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?oo
ool02?oo00<00?ooool01?oo0`00booo00<00?ooool0>_oo0003ool00`00ooooo`09ool00`00oooo
o`03ool00`00ooooo`3<ool2000jool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo
0<koo`03003ooooo03Ooo`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3?ool00`00
ooooo`0fool000?oo`03003ooooo00Soo`8000Goo`03003ooooo0=3oo`8003Koo`005Ooo00<00?oo
ool0d_oo00<00?ooool0<ooo000Eool00`00ooooo`3Cool00`00ooooo`0bool001Goo`800=Goo`80
03;oo`005Ooo00<00?ooool0e_oo00<00?ooool0;ooo000Eool00`00ooooo`3Gool00`00ooooo`0^
ool001Goo`03003ooooo0=Soo`03003ooooo02goo`005Ooo00<00?ooool0fOoo00<00?ooool0;?oo
000Eool00`00ooooo`3Jool00`00ooooo`0[ool001Goo`03003ooooo0=_oo`8002_oo`005Ooo00<0
0?ooool0gOoo00<00?ooool0:?oo000Eool2003Oool00`00ooooo`0Wool001Goo`03003ooooo0=oo
o`03003ooooo02Koo`005Ooo00<00?ooool0h?oo00<00?ooool09Ooo000Eool00`00ooooo`3Qool0
0`00ooooo`0Tool001Goo`03003ooooo0>;oo`8002Coo`005Ooo00<00?ooool0i?oo00<00?ooool0
8Ooo000Eool00`00ooooo`3Uool00`00ooooo`0Pool001Goo`800>Ooo`03003ooooo01ooo`005Ooo
00<00?ooool0iooo00<00?ooool07_oo000Eool00`00ooooo`3Xool2000Nool001Goo`03003ooooo
0>[oo`03003ooooo01_oo`005Ooo00<00?ooool0jooo00<00?ooool06_oo00000ooo000000020003
ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`3/ool00`00ooooo`0Iool000?oo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo0>goo`8001Woo`000ooo00<00?ooool01_oo1@001?oo
0`00kooo00<00?ooool05_oo0003ool00`00ooooo`06ool01000ooooo`001Ooo00<00?ooool0l?oo
00<00?ooool05Ooo0003ool00`00ooooo`07ool00`00ool00005ool00`00ooooo`3aool00`00oooo
o`0Dool00004ool00000000:ool20005ool00`00ooooo`3bool00`00ooooo`0Cool000?oo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo0??oo`8001?oo`005Ooo00<00?ooool0mOoo00<00?oo
ool04?oo000Eool00`00ooooo`3fool00`00ooooo`0?ool001Goo`800?Soo`03003ooooo00koo`00
5Ooo00<00?ooool0n?oo0P003_oo000Eool00`00ooooo`3jool00`00ooooo`0;ool001Goo`03003o
oooo0?_oo`03003ooooo00[oo`005Ooo00<00?ooool0o?oo00<00?ooool02Ooo000Eool00`00oooo
o`3mool20009ool001Goo`03003ooooo0?ooo`03003ooooo00Koo`005Ooo0P00oooo0_oo00<00?oo
ool01Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, 0.318475, \
0.00578885, 0.00701031}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{mGamma /. strangesol[k, 10]}, {k, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.603319 1.21817 [
[.15079 .59082 -9 -9 ]
[.15079 .59082 9 0 ]
[.27778 .59082 -9 -9 ]
[.27778 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.53175 .59082 -9 -9 ]
[.53175 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.9127 .59082 -9 -9 ]
[.9127 .59082 9 0 ]
[.01131 .11605 -24 -4.5 ]
[.01131 .11605 0 4.5 ]
[.01131 .23787 -24 -4.5 ]
[.01131 .23787 0 4.5 ]
[.01131 .35969 -24 -4.5 ]
[.01131 .35969 0 4.5 ]
[.01131 .4815 -24 -4.5 ]
[.01131 .4815 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .60332 m
.15079 .60957 L
s
[(0.2)] .15079 .59082 0 1 Mshowa
.27778 .60332 m
.27778 .60957 L
s
[(0.4)] .27778 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(0.6)] .40476 .59082 0 1 Mshowa
.53175 .60332 m
.53175 .60957 L
s
[(0.8)] .53175 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(1)] .65873 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(1.2)] .78571 .59082 0 1 Mshowa
.9127 .60332 m
.9127 .60957 L
s
[(1.4)] .9127 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.18254 .60332 m
.18254 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.34127 .60332 m
.34127 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.81746 .60332 m
.81746 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.97619 .60332 m
.97619 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .11605 m
.03006 .11605 L
s
[(-0.4)] .01131 .11605 1 0 Mshowa
.02381 .23787 m
.03006 .23787 L
s
[(-0.3)] .01131 .23787 1 0 Mshowa
.02381 .35969 m
.03006 .35969 L
s
[(-0.2)] .01131 .35969 1 0 Mshowa
.02381 .4815 m
.03006 .4815 L
s
[(-0.1)] .01131 .4815 1 0 Mshowa
.125 Mabswid
.02381 .0186 m
.02756 .0186 L
s
.02381 .04296 m
.02756 .04296 L
s
.02381 .06733 m
.02756 .06733 L
s
.02381 .09169 m
.02756 .09169 L
s
.02381 .14042 m
.02756 .14042 L
s
.02381 .16478 m
.02756 .16478 L
s
.02381 .18914 m
.02756 .18914 L
s
.02381 .21351 m
.02756 .21351 L
s
.02381 .26223 m
.02756 .26223 L
s
.02381 .2866 m
.02756 .2866 L
s
.02381 .31096 m
.02756 .31096 L
s
.02381 .33532 m
.02756 .33532 L
s
.02381 .38405 m
.02756 .38405 L
s
.02381 .40841 m
.02756 .40841 L
s
.02381 .43278 m
.02756 .43278 L
s
.02381 .45714 m
.02756 .45714 L
s
.02381 .50587 m
.02756 .50587 L
s
.02381 .53023 m
.02756 .53023 L
s
.02381 .55459 m
.02756 .55459 L
s
.02381 .57896 m
.02756 .57896 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03016 .59094 m
.06854 .51944 L
.11039 .45381 L
.1497 .4043 L
.1875 .36523 L
.22774 .33046 L
.26647 .302 L
.30764 .27595 L
.34731 .25415 L
.38546 .23576 L
.42606 .21858 L
.46515 .20409 L
.50273 .19183 L
.54275 .18033 L
.58126 .17043 L
.62221 .1606 L
.66166 .15098 L
.69959 .14071 L
.73997 .12785 L
.77884 .11321 L
.82014 .09539 L
.85994 .07642 L
.89823 .05691 L
.93896 .0352 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3iool20007ool001_oo`800?Soo`8000Woo`006ooo00<00?ooool0mOoo0P002ooo000K
ool00`00ooooo`3cool2000=ool001_oo`03003ooooo0?7oo`8000ooo`006ooo00<00?ooool0l?oo
00<00?ooool03ooo000Kool00`00ooooo`3^ool2000Bool001_oo`800>goo`8001Coo`006ooo00<0
0?ooool0j_oo0P005_oo000Kool00`00ooooo`3Xool2000Hool001_oo`03003ooooo0>Koo`8001[o
o`006ooo00<00?ooool0i?oo0P007?oo000Kool00`00ooooo`3Rool2000Nool001_oo`03003ooooo
0>3oo`80023oo`006ooo0P00gooo0P008_oo000Kool00`00ooooo`3Lool2000Tool001_oo`03003o
oooo0=[oo`8002Koo`006ooo00<00?ooool0f?oo0P00:?oo000Kool00`00ooooo`3Fool2000Zool0
01_oo`03003ooooo0=Coo`8002coo`006ooo0P00dooo0P00;_oo000Kool00`00ooooo`3@ool2000`
ool001_oo`03003ooooo0<koo`8003;oo`006ooo00<00?ooool0c?oo0P00=?oo000Kool00`00oooo
o`3:ool2000fool000Soo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo0<Soo`80
03Soo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo0<Goo`<003[oo`001ooo00@0
0?ooool000Ooo`D000Coo`800<?oo`<003goo`0000?oo`0000000P000_oo00@00?ooool000Ooo`04
003ooooo0005ool00`00ooooo`30ool20010ool000Ooo`04003ooooo0008ool00`00ool00005ool0
0`00ooooo`2mool30012ool000Ooo`04003ooooo0009ool20005ool00`00ooooo`2jool30015ool0
00Soo`8000_oo`03003ooooo00?oo`03003ooooo0;Koo`@004Soo`006ooo00<00?ooool0/ooo0`00
C?oo000Kool2002aool3001?ool001_oo`03003ooooo0:coo`@005;oo`006ooo00<00?ooool0ZOoo
0`00E_oo000Kool00`00ooooo`2Vool3001Iool001_oo`03003ooooo0:;oo`@005coo`006ooo00<0
0?ooool0W_oo1000H?oo000Kool00`00ooooo`2Jool4001Tool001_oo`8009Goo`H006Soo`006ooo
00<00?ooool0T?oo1000K_oo000Kool00`00ooooo`2<ool4001bool001_oo`03003ooooo08Woo`<0
07Koo`006ooo00<00?ooool0Q_oo0`00NOoo000Kool00`00ooooo`22ool4001lool001_oo`8007oo
o`@0083oo`006ooo00<00?ooool0Nooo0`00Q?oo000Kool00`00ooooo`1gool40027ool001_oo`03
003ooooo07Coo`<008_oo`006ooo00<00?ooool0LOoo0`00S_oo000Kool00`00ooooo`1_ool2002A
ool001_oo`03003ooooo06coo`<009?oo`006ooo0P00Jooo0P00U_oo000Kool00`00ooooo`1Wool3
002Hool001_oo`03003ooooo06Goo`8009_oo`006ooo00<00?ooool0Hooo0P00WOoo0008ool20004
ool00`00ooooo`03ool20005ool00`00ooooo`1Qool2002Oool000Ooo`04003ooooo0008ool01000
ooooo`001?oo00<00?ooool0Gooo0P00XOoo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00
GOoo0`00Xooo00000ooo000000020002ool01000ooooo`002Ooo0P001Ooo00<00?ooool0F_oo0P00
Y_oo0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0Eooo0`00Z?oo0007ool01000
ooooo`002?oo00@00?ooool000Coo`03003ooooo05Goo`800:_oo`002?oo0P002_oo0P001Ooo00<0
0?ooool0Dooo0P00[Ooo000Kool00`00ooooo`1Aool2002_ool001_oo`03003ooooo04ooo`800;7o
o`006ooo0P00C_oo0P00/ooo000Kool00`00ooooo`1;ool2002eool001_oo`03003ooooo04Woo`80
0;Ooo`006ooo00<00?ooool0B?oo00<00?ooool0]ooo000Kool00`00ooooo`16ool2002jool001_o
o`03003ooooo04Goo`03003ooooo0;[oo`006ooo0P00A?oo0P00_Ooo000Kool00`00ooooo`11ool2
002oool001_oo`03003ooooo043oo`03003ooooo0;ooo`006ooo00<00?ooool0?_oo0P00`_oo000K
ool00`00ooooo`0mool00`00ooooo`32ool001_oo`03003ooooo03coo`03003ooooo0<?oo`006ooo
00<00?ooool0>_oo0P00a_oo000Kool2000jool00`00ooooo`36ool001_oo`03003ooooo03Soo`03
003ooooo0<Ooo`006ooo00<00?ooool0=ooo00<00?ooool0b?oo000Kool00`00ooooo`0eool2003;
ool001_oo`03003ooooo03Coo`03003ooooo0<_oo`006ooo00<00?ooool0<ooo00<00?ooool0c?oo
000Kool2000cool00`00ooooo`3=ool001_oo`03003ooooo033oo`800=3oo`006ooo00<00?ooool0
;ooo00<00?ooool0d?oo000Kool00`00ooooo`0^ool00`00ooooo`3Aool001_oo`03003ooooo02go
o`03003ooooo0=;oo`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool0;?oo00<00?oo
ool0dooo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0:_oo0P00e_oo0007ool0
1000ooooo`002Ooo00<00?ooool01?oo0P00:_oo00<00?ooool0e_oo00000ooo000000020002ool0
1000ooooo`002_oo00<00?ooool00ooo00<00?ooool0:?oo00<00?ooool0eooo0007ool01000oooo
o`002ooo00<00?ooool00_oo00<00?ooool09ooo00<00?ooool0f?oo0007ool01000ooooo`002?oo
00@00?ooool000Coo`03003ooooo02Koo`03003ooooo0=Woo`002?oo0P002_oo0P001Ooo00<00?oo
ool09Ooo00<00?ooool0f_oo000Kool00`00ooooo`0Tool00`00ooooo`3Kool001_oo`8002Coo`03
003ooooo0=coo`006ooo00<00?ooool08_oo00<00?ooool0gOoo000Kool00`00ooooo`0Qool00`00
ooooo`3Nool001_oo`03003ooooo023oo`03003ooooo0=ooo`006ooo00<00?ooool07ooo00<00?oo
ool0h?oo000Kool00`00ooooo`0Nool00`00ooooo`3Qool001_oo`8001koo`03003ooooo0>;oo`00
6ooo00<00?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Lool00`00ooooo`3Sool001_o
o`03003ooooo01_oo`03003ooooo0>Coo`006ooo00<00?ooool06_oo00<00?ooool0iOoo000Kool0
0`00ooooo`0Jool00`00ooooo`3Uool001_oo`03003ooooo01Woo`03003ooooo0>Koo`006ooo0P00
6Ooo00<00?ooool0iooo000Kool00`00ooooo`0Hool00`00ooooo`3Wool001_oo`03003ooooo01Oo
o`03003ooooo0>Soo`006ooo00<00?ooool05_oo00<00?ooool0jOoo000Kool00`00ooooo`0Eool0
0`00ooooo`3Zool001_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo0P005Ooo00<00?ooool0
jooo000Kool00`00ooooo`0Cool00`00ooooo`3/ool001_oo`03003ooooo01?oo`03003ooooo0>co
o`006ooo00<00?ooool04_oo00<00?ooool0kOoo000Kool00`00ooooo`0Aool00`00ooooo`3^ool0
00Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo017oo`03003ooooo0>koo`001ooo00@0
0?ooool000[oo`03003ooooo00?oo`03003ooooo013oo`03003ooooo0>ooo`001ooo00@00?ooool0
00[oo`03003ooooo00?oo`80013oo`03003ooooo0?3oo`0000?oo`0000000P000_oo00@00?ooool0
00[oo`03003ooooo00?oo`03003ooooo00ooo`03003ooooo0?3oo`001ooo00@00?ooool000[oo`03
003ooooo00?oo`03003ooooo00koo`03003ooooo0?7oo`001ooo00@00?ooool000Soo`<000Goo`03
003ooooo00koo`03003ooooo0?7oo`002?oo0P002ooo00<00?ooool00ooo00<00?ooool03Ooo00<0
0?ooool0l_oo000Kool00`00ooooo`0<ool00`00ooooo`3cool001_oo`8000goo`03003ooooo0??o
o`006ooo00<00?ooool02ooo00<00?ooool0m?oo000Kool00`00ooooo`0:ool00`00ooooo`3eool0
01_oo`03003ooooo00[oo`03003ooooo0?Goo`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000K
ool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo
0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo
00Koo`03003ooooo0?Woo`006ooo00<00?ooool01_oo00<00?ooool0nOoo000Kool00`00ooooo`05
ool00`00ooooo`3jool001_oo`03003ooooo00Goo`03003ooooo0?[oo`006ooo0P001Ooo00<00?oo
ool0nooo000Kool00`00ooooo`04ool00`00ooooo`3kool001_oo`03003ooooo00?oo`03003ooooo
0?coo`006ooo00<00?ooool00ooo00<00?ooool04_oo0P001?oo00<00?ooool00_oo10004_oo0P00
1?oo00<00?ooool01?oo00<00?ooool04_oo0P001?oo00<00?ooool00ooo0P005?oo0P001?oo00<0
0?ooool00ooo0P006Ooo10005ooo10000ooo00<00?ooool00_oo10004_oo10000ooo00<00?ooool0
1?oo00<00?ooool04?oo000Kool00`00ooooo`02ool00`00ooooo`0Bool01000ooooo`002?oo00<0
0?ooool04_oo00@00?ooool000[oo`03003ooooo017oo`04003ooooo0008ool01000ooooo`004_oo
00@00?ooool000Soo`04003ooooo000Jool00`00ooooo`0Hool00`00ooooo`07ool00`00ooooo`0E
ool00`00ooooo`09ool00`00ooooo`0@ool001_oo`03003ooooo00;oo`03003ooooo01;oo`04003o
oooo0009ool00`00ooooo`0Aool01000ooooo`001ooo1@004_oo00@00?ooool000Soo`04003ooooo
000Bool01000ooooo`002?oo00@00?ooool001[oo`03003ooooo01Soo`03003ooooo00Soo`03003o
oooo01Coo`03003ooooo00Koo`D0017oo`006ooo00D00?ooooooo`0001Goo`04003ooooo000:ool0
0`00ooooo`0@ool01000ooooo`001ooo00@00?ooool001?oo`04003ooooo0008ool3000Cool01000
ooooo`002Ooo0P006ooo00<00?ooool06?oo00<00?ooool02Ooo00<00?ooool04ooo00<00?ooool0
1_oo00@00?ooool001;oo`006ooo0P000_oo00<00?ooool04ooo00@00?ooool000_oo`03003ooooo
00ooo`04003ooooo0008ool00`00ool0000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?oo
ool000Soo`04003ooooo000Jool00`00ooooo`0Hool00`00ooooo`0:ool00`00ooooo`0Bool00`00
ooooo`07ool00`00ool0000Bool001_oo`04003ooooo000Fool01000ooooo`002?oo00@00?ooool0
017oo`04003ooooo0009ool2000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool000So
o`04003ooooo000Hool3000Hool30009ool01000ooooo`004_oo0`002_oo0P004_oo000Kool01000
ooooo`005ooo0P002_oo0P004ooo0P002ooo00<00?ooool04_oo0P002_oo0`004ooo0P002_oo0P00
6ooo00<00?ooool06?oo00<00?ooool02?oo0P005Ooo00<00?ooool02Ooo00<00?ooool04?oo000K
ool00`00ool0003oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000D
oooo000=000001_oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Goo`006ooo00<00?ooool07ooo00<00?ooool07_oo00<00?oo
ool07ooo00<00?ooool07ooo00<00?ooool07ooo00<00?ooool07_oo00<00?ooool07ooo00<00?oo
ool05_oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, -0.514613, \
0.00592092, 0.00308603}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9994820046162144`, 
      mGamma \[Rule] \(-0.3707184166384133`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.001, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.7507348023467588`, 
      mGamma \[Rule] \(-0.0010167970670121491`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.001, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.7507331914045487`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[0.7507331914045487`, 0.001, 10]\)], "Input"],

Cell[BoxData[
    \(2.0805037335362946`*^-9 + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[
      0.7507352528464437476 + I \((\(-0.001016796662459896262083\))\), 0.001, 
      10]\)], "Input"],

Cell[BoxData[
    \(9.931444982616892`*^-22 + 
      7.905253850325371`*^-20\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["xi=5", "Subsubsection"],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.8, 
          0.9}, {mGamma, \(-0.6\), \(-0.7\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, Om /. strangesol[k, 5]}, {k, 1.07, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.34607 2.21484 -1.36567 1.31266 [
[.31174 .06576 -9 -9 ]
[.31174 .06576 9 0 ]
[.53322 .06576 -9 -9 ]
[.53322 .06576 9 0 ]
[.75471 .06576 -9 -9 ]
[.75471 .06576 9 0 ]
[.97619 .06576 -9 -9 ]
[.97619 .06576 9 0 ]
[.07775 .20952 -18 -4.5 ]
[.07775 .20952 0 4.5 ]
[.07775 .34079 -18 -4.5 ]
[.07775 .34079 0 4.5 ]
[.07775 .47205 -18 -4.5 ]
[.07775 .47205 0 4.5 ]
[.07775 .60332 -18 -4.5 ]
[.07775 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.31174 .07826 m
.31174 .08451 L
s
[(1.2)] .31174 .06576 0 1 Mshowa
.53322 .07826 m
.53322 .08451 L
s
[(1.3)] .53322 .06576 0 1 Mshowa
.75471 .07826 m
.75471 .08451 L
s
[(1.4)] .75471 .06576 0 1 Mshowa
.97619 .07826 m
.97619 .08451 L
s
[(1.5)] .97619 .06576 0 1 Mshowa
.125 Mabswid
.13455 .07826 m
.13455 .08201 L
s
.17885 .07826 m
.17885 .08201 L
s
.22315 .07826 m
.22315 .08201 L
s
.26744 .07826 m
.26744 .08201 L
s
.35604 .07826 m
.35604 .08201 L
s
.40033 .07826 m
.40033 .08201 L
s
.44463 .07826 m
.44463 .08201 L
s
.48893 .07826 m
.48893 .08201 L
s
.57752 .07826 m
.57752 .08201 L
s
.62182 .07826 m
.62182 .08201 L
s
.66611 .07826 m
.66611 .08201 L
s
.71041 .07826 m
.71041 .08201 L
s
.799 .07826 m
.799 .08201 L
s
.8433 .07826 m
.8433 .08201 L
s
.8876 .07826 m
.8876 .08201 L
s
.93189 .07826 m
.93189 .08201 L
s
.04596 .07826 m
.04596 .08201 L
s
.00166 .07826 m
.00166 .08201 L
s
.25 Mabswid
0 .07826 m
1 .07826 L
s
.09025 .20952 m
.0965 .20952 L
s
[(1.2)] .07775 .20952 1 0 Mshowa
.09025 .34079 m
.0965 .34079 L
s
[(1.3)] .07775 .34079 1 0 Mshowa
.09025 .47205 m
.0965 .47205 L
s
[(1.4)] .07775 .47205 1 0 Mshowa
.09025 .60332 m
.0965 .60332 L
s
[(1.5)] .07775 .60332 1 0 Mshowa
.125 Mabswid
.09025 .10451 m
.094 .10451 L
s
.09025 .13076 m
.094 .13076 L
s
.09025 .15701 m
.094 .15701 L
s
.09025 .18327 m
.094 .18327 L
s
.09025 .23577 m
.094 .23577 L
s
.09025 .26203 m
.094 .26203 L
s
.09025 .28828 m
.094 .28828 L
s
.09025 .31453 m
.094 .31453 L
s
.09025 .36704 m
.094 .36704 L
s
.09025 .39329 m
.094 .39329 L
s
.09025 .41955 m
.094 .41955 L
s
.09025 .4458 m
.094 .4458 L
s
.09025 .49831 m
.094 .49831 L
s
.09025 .52456 m
.094 .52456 L
s
.09025 .55081 m
.094 .55081 L
s
.09025 .57707 m
.094 .57707 L
s
.09025 .052 m
.094 .052 L
s
.09025 .02575 m
.094 .02575 L
s
.25 Mabswid
.09025 0 m
.09025 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .03888 m
.06244 .06177 L
.10458 .08675 L
.14415 .1102 L
.18221 .13275 L
.22272 .15676 L
.26171 .17987 L
.30316 .20443 L
.34309 .2281 L
.3815 .25087 L
.42237 .27509 L
.46172 .29841 L
.49955 .32083 L
.53984 .34471 L
.57861 .36769 L
.61984 .39212 L
.65954 .41565 L
.69774 .43829 L
.73838 .46238 L
.77751 .48557 L
.81909 .51021 L
.85916 .53396 L
.89771 .55681 L
.93871 .58111 L
.97619 .60332 L
s
.02381 .03825 m
.02499 .03825 L
.02605 .03825 L
.02729 .03825 L
.02846 .03825 L
.02954 .03825 L
.03053 .03825 L
.03163 .03825 L
.03279 .03825 L
.03395 .03825 L
.0352 .03825 L
.03746 .03825 L
.04016 .03825 L
.04262 .03824 L
.04748 .03823 L
.05278 .03822 L
.06244 .03819 L
.07302 .03814 L
.08429 .03807 L
.10458 .03792 L
.12599 .03772 L
.14616 .03749 L
.18471 .03694 L
.22572 .03622 L
.2652 .03541 L
.30318 .03454 L
.3436 .03353 L
.38251 .03249 L
.42387 .03133 L
.46371 .03016 L
.50204 .02901 L
.54283 .02775 L
.58209 .02653 L
.61984 .02535 L
.66005 .02409 L
.69873 .02289 L
.73987 .02162 L
.7795 .02041 L
.81761 .01926 L
.85817 .01806 L
.89721 .01693 L
.93871 .01576 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O6Ooo00<00?ooool0
oooo1Ooo000Iool00`00ooooo`3oool5ool001Woo`03003ooooo0?ooo`Goo`006Ooo00<00?ooool0
oooo1Ooo000Iool00`00ooooo`3YoolB0009ool001Woo`03003ooooo0<Ooob8001_oo`006Ooo00<0
0?ooool0YOoo8P00?Ooo000Iool20025oolQ001Oool001Woo`03003ooooo06;oob80083oo`006Ooo
00<00?ooool0@?oo8P00X_oo000Iool00`00ooooo`0Cool]0034ool000KoobT00?7oo`001ooo0P00
4?oo00<00?ooool0=?oo10000ooo00<00?ooool00_oo1000;ooo10000ooo00<00?ooool00ooo0P00
<?oo10000ooo00<00?ooool01?oo00<00?ooool0;ooo10000ooo00<00?ooool00_oo0`000ooo0009
ool2000>ool00`00ooooo`0fool00`00ooooo`07ool00`00ooooo`0bool00`00ooooo`07ool01000
ooooo`00<Ooo00<00?ooool02Ooo00<00?ooool0<Ooo00<00?ooool02_oo0@000Ooo0Ooo000;ool2
000<ool00`00ooooo`0fool00`00ooooo`08ool00`00ooooo`0aool00`00ooooo`0:ool00`00oooo
o`0_ool00`00ooooo`06ool5000bool00`00ooooo`0:ool10001ool1ool000goo`8000[oo`8003Oo
o`03003ooooo00Woo`03003ooooo033oo`03003ooooo00Soo`8003;oo`03003ooooo00Koo`04003o
oooo000cool00`00ooooo`07ool30003ool000ooo`8000Soo`03003ooooo03Koo`03003ooooo00[o
o`03003ooooo02ooo`03003ooooo00[oo`03003ooooo02ooo`03003ooooo00Ooo`03003oo`0003?o
o`03003ooooo00Ooo`03003ooooo00?oo`004Ooo00<00?ooool01Ooo00<00?ooool0=?oo0`002Ooo
00@00?ooool002ooo`<000Woo`04003ooooo000_ool3000:ool2000aool30009ool00`00ooooo`03
ool001;oo`8000Goo`03003ooooo03Koo`03003ooooo00Soo`8003;oo`03003ooooo00Soo`8003;o
o`03003ooooo00Woo`03003ooooo037oo`03003ooooo00Ooo`@000;oo`005?oo00<00?ooool00_oo
00<00?ooool0oooo1Ooo000Eool20002ool00`00ooooo`3oool5ool001Ooo`03003oo`000?ooo`Oo
o`00o`007P000ooo00000`00ooooo`0:ool00`00ooooo`09ool2000;ool00`00ooooo`09ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`07ool001Woo`04003oo`00000k
ool00`00ooooo`0lool00`00ooooo`0lool00`00ooooo`0mool00`00ooooo`07ool001Woo`05003o
ooooool0003oool3ool001Woo`03003ooooo00;oo`800?ooo`7oo`006Ooo00<00?ooool01?oo0P00
o_oo000Iool00`00ooooo`06ool2003lool001Woo`03003ooooo00Soo`800?[oo`006Ooo0P002ooo
0P00n?oo000Iool00`00ooooo`0<ool2003fool001Woo`03003ooooo00koo`800?Coo`006Ooo00<0
0?ooool04?oo00<00?ooool0lOoo000Iool00`00ooooo`0Aool00`00ooooo`3`ool001Woo`03003o
oooo01;oo`800?3oo`006Ooo00<00?ooool05?oo00<00?ooool0kOoo000Iool2000Fool2003]ool0
01Woo`03003ooooo01Ooo`03003ooooo0>[oo`006Ooo00<00?ooool06?oo0P00j_oo000Iool00`00
ooooo`0Jool2003Xool001Woo`03003ooooo01coo`03003ooooo0>Goo`006Ooo00<00?ooool07Ooo
0P00iOoo000Iool00`00ooooo`0Oool2003Sool001Woo`03003ooooo027oo`800>7oo`006Ooo0P00
9?oo00<00?ooool0g_oo000Iool00`00ooooo`0Tool2003Nool001Woo`03003ooooo02Koo`800=co
o`006Ooo00<00?ooool0:?oo0P00f_oo000Iool00`00ooooo`0Zool2003Hool001Woo`03003ooooo
02coo`800=Koo`006Ooo00<00?ooool0;_oo00<00?ooool0dooo000Iool2000`ool2003Cool001Wo
o`03003ooooo037oo`800=7oo`006Ooo00<00?ooool0<ooo00<00?ooool0c_oo000Iool00`00oooo
o`0dool2003>ool001Woo`03003ooooo03Koo`800<coo`006Ooo00<00?ooool0>?oo0P00b_oo0005
ool40003ool00`00ooooo`02ool40004ool00`00ooooo`0jool00`00ooooo`37ool000Ooo`03003o
oooo00Ooo`03003ooooo00Goo`03003ooooo03_oo`800<Ooo`001ooo00<00?ooool02?oo00<00?oo
ool01?oo0`00?Ooo00<00?ooool0a?oo0007ool00`00ooooo`09ool00`00ooooo`03ool00`00oooo
o`0nool20034ool000Ooo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo043oo`800<;oo`00
1Ooo0`002Ooo00@00?ooool000Coo`03003ooooo04;oo`03003ooooo0;ooo`001ooo00<00?ooool0
2?oo0P001Ooo00<00?ooool0@ooo0P00_ooo000Iool00`00ooooo`15ool00`00ooooo`2lool001Wo
o`03003ooooo04Koo`800;coo`006Ooo0P00BOoo0P00^_oo000Iool00`00ooooo`1:ool2002hool0
01Woo`03003ooooo04coo`800;Koo`006Ooo00<00?ooool0C_oo0P00]?oo000Iool00`00ooooo`1@
ool00`00ooooo`2aool001Woo`03003ooooo057oo`800;7oo`006Ooo00<00?ooool0Dooo0P00[ooo
000Iool2001Fool00`00ooooo`2/ool001Woo`03003ooooo05Koo`800:coo`006Ooo00<00?ooool0
F?oo0P00Z_oo000Iool00`00ooooo`1Jool2002Xool001Woo`03003ooooo05coo`03003ooooo0:Go
o`006Ooo00<00?ooool0GOoo0P00YOoo000Iool00`00ooooo`1Oool00`00ooooo`2Rool001Woo`03
003ooooo063oo`800:;oo`006Ooo0P00Hooo0P00X?oo000Iool00`00ooooo`1Tool00`00ooooo`2M
ool001Woo`03003ooooo06Goo`8009goo`006Ooo00<00?ooool0Iooo00<00?ooool0V_oo000Iool0
0`00ooooo`1Xool2002Jool001Woo`03003ooooo06[oo`8009Soo`006Ooo00<00?ooool0K?oo0P00
U_oo000Iool2001_ool2002Dool001Woo`03003ooooo073oo`8009;oo`006Ooo00<00?ooool0L_oo
00<00?ooool0Sooo000Iool00`00ooooo`1cool2002?ool001Woo`03003ooooo07Goo`03003ooooo
08coo`006Ooo00<00?ooool0M_oo0P00S?oo0005ool40003ool00`00ooooo`03ool20005ool00`00
ooooo`1hool2002:ool000Ooo`03003ooooo00Ooo`04003ooooo0004ool00`00ooooo`1jool00`00
ooooo`27ool000Ooo`03003ooooo00[oo`03003ooooo00;oo`<007_oo`8008Ooo`001ooo00<00?oo
ool02?oo0P001Ooo00<00?ooool0OOoo00<00?ooool0Q?oo0007ool00`00ooooo`0:ool00`00oooo
o`02ool00`00ooooo`1nool20024ool000Goo`<000Woo`04003ooooo0004ool00`00ooooo`20ool0
0`00ooooo`21ool000Ooo`03003ooooo00Soo`8000Goo`03003ooooo087oo`80087oo`006Ooo00<0
0?ooool0Pooo0P00Oooo000Iool00`00ooooo`25ool00`00ooooo`1lool001Woo`8008Ooo`8007co
o`006Ooo00<00?ooool0R?oo00<00?ooool0NOoo000Iool00`00ooooo`29ool2001iool001Woo`03
003ooooo08_oo`8007Ooo`006Ooo00<00?ooool0SOoo00<00?ooool0M?oo000Iool00`00ooooo`2>
ool2001dool001Woo`03003ooooo093oo`8007;oo`006Ooo00<00?ooool0T_oo0P00L?oo000Iool2
002Eool00`00ooooo`1]ool001Woo`03003ooooo09Goo`8006goo`006Ooo00<00?ooool0Uooo0P00
Jooo000Iool00`00ooooo`2Iool2001Yool001Woo`03003ooooo09_oo`8006Ooo`006Ooo00<00?oo
ool0WOoo0P00IOoo000Iool00`00ooooo`2Oool00`00ooooo`1Rool001Woo`800:7oo`8006;oo`00
6Ooo00<00?ooool0X_oo00<00?ooool0Gooo000Iool00`00ooooo`2Sool2001Oool001Woo`03003o
oooo0:Goo`8005goo`006Ooo00<00?ooool0Yooo00<00?ooool0F_oo000Iool00`00ooooo`2Xool2
001Jool001Woo`03003ooooo0:[oo`03003ooooo05Ooo`006Ooo00<00?ooool0Zooo0P00Eooo000I
ool2002^ool2001Eool001Woo`03003ooooo0:ooo`03003ooooo05;oo`006Ooo00<00?ooool0/?oo
0P00D_oo000Iool00`00ooooo`2bool2001@ool001Woo`03003ooooo0;Coo`8004koo`001Ooo1000
0ooo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0]_oo00<00?ooool0Booo0007ool00`00
ooooo`09ool00`00ooooo`03ool00`00ooooo`2gool2001;ool000Ooo`03003ooooo00Koo`D000Co
o`<00;Woo`8004Woo`001ooo00<00?ooool01_oo00@00?ooool000Goo`03003ooooo0;_oo`8004Oo
o`001ooo00<00?ooool01ooo00<00?oo00001Ooo00<00?ooool0_Ooo0P00AOoo0005ool3000:ool2
0005ool00`00ooooo`2oool20013ool000Ooo`03003ooooo00Woo`03003ooooo00?oo`03003ooooo
0<7oo`03003ooooo043oo`006Ooo00<00?ooool0`_oo0P00@?oo000Iool00`00ooooo`34ool2000n
ool001Woo`03003ooooo0<Koo`03003ooooo03_oo`006Ooo0P00b?oo0P00>ooo000Iool00`00oooo
o`39ool2000iool001Woo`03003ooooo0<_oo`8003Ooo`006Ooo00<00?ooool0cOoo00<00?ooool0
=?oo000Iool00`00ooooo`3>ool2000dool001Woo`03003ooooo0=3oo`03003ooooo037oo`006Ooo
00<00?ooool0dOoo0P00<Ooo000Iool2003Dool2000_ool001Woo`03003ooooo0=Goo`03003ooooo
02coo`006Ooo00<00?ooool0e_oo0P00;?oo000Iool00`00ooooo`3Hool00`00ooooo`0Yool001Wo
o`03003ooooo0=Woo`8002Woo`006Ooo00<00?ooool0fooo0P009ooo000Iool00`00ooooo`3Mool2
000Uool001Woo`03003ooooo0=ooo`8002?oo`006Ooo0P00h_oo0P008Ooo000Iool00`00ooooo`3S
ool00`00ooooo`0Nool001Woo`03003ooooo0>Coo`8001koo`006Ooo00<00?ooool0i_oo0P007?oo
000Iool00`00ooooo`3Xool00`00ooooo`0Iool001Woo`03003ooooo0>Woo`8001Woo`006Ooo00<0
0?ooool0jooo0P005ooo000Iool2003^ool2000Eool001Woo`03003ooooo0>ooo`03003ooooo01;o
o`006Ooo00<00?ooool0l?oo0P004_oo000Iool00`00ooooo`3bool00`00ooooo`0?ool001Woo`03
003ooooo0??oo`8000ooo`006Ooo00<00?ooool0mOoo0P003Ooo0005ool40003ool00`00ooooo`02
ool30005ool00`00ooooo`3gool00`00ooooo`0:ool000Ooo`03003ooooo00[oo`03003ooooo00;o
o`03003ooooo0?Soo`8000[oo`001ooo00<00?ooool02_oo00<00?ooool00_oo0`00n_oo00<00?oo
ool01ooo0007ool00`00ooooo`07ool30005ool00`00ooooo`3oool5ool000Ooo`03003ooooo00Oo
o`03003ooooo00Goo`03003ooooo0?ooo`Goo`001Ooo0`002Ooo00<00?ooool01Ooo00<00?ooool0
oooo1Ooo0007ool00`00ooooo`07ool40004ool00`00ooooo`3oool5ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {1.05925, 1.03955, \
0.00158545, 0.00267511}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.7, 
          0.8}, {mGamma, \(-0.3\), \(-0.4\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.5, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8413359292417021`, 
      mGamma \[Rule] 1.66830551348445`*^-7}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.74, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] \(\(0.8760984351000806`\)\(\[InvisibleSpace]\)\) + 
          1.0274376357597258`*^-7\ \[ImaginaryI]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol[k, 10]], {k, 0.01, 1}, 
      PlotStyle \[Rule] {RGBColor[1, 0, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 -0.877637 1.44479 [
[.21429 .55465 -9 -9 ]
[.21429 .55465 9 0 ]
[.40476 .55465 -9 -9 ]
[.40476 .55465 9 0 ]
[.59524 .55465 -9 -9 ]
[.59524 .55465 9 0 ]
[.78571 .55465 -9 -9 ]
[.78571 .55465 9 0 ]
[.97619 .55465 -3 -9 ]
[.97619 .55465 3 0 ]
[.01131 .13372 -18 -4.5 ]
[.01131 .13372 0 4.5 ]
[.01131 .2782 -18 -4.5 ]
[.01131 .2782 0 4.5 ]
[.01131 .42267 -18 -4.5 ]
[.01131 .42267 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .56715 m
.21429 .5734 L
s
[(0.2)] .21429 .55465 0 1 Mshowa
.40476 .56715 m
.40476 .5734 L
s
[(0.4)] .40476 .55465 0 1 Mshowa
.59524 .56715 m
.59524 .5734 L
s
[(0.6)] .59524 .55465 0 1 Mshowa
.78571 .56715 m
.78571 .5734 L
s
[(0.8)] .78571 .55465 0 1 Mshowa
.97619 .56715 m
.97619 .5734 L
s
[(1)] .97619 .55465 0 1 Mshowa
.125 Mabswid
.07143 .56715 m
.07143 .5709 L
s
.11905 .56715 m
.11905 .5709 L
s
.16667 .56715 m
.16667 .5709 L
s
.2619 .56715 m
.2619 .5709 L
s
.30952 .56715 m
.30952 .5709 L
s
.35714 .56715 m
.35714 .5709 L
s
.45238 .56715 m
.45238 .5709 L
s
.5 .56715 m
.5 .5709 L
s
.54762 .56715 m
.54762 .5709 L
s
.64286 .56715 m
.64286 .5709 L
s
.69048 .56715 m
.69048 .5709 L
s
.7381 .56715 m
.7381 .5709 L
s
.83333 .56715 m
.83333 .5709 L
s
.88095 .56715 m
.88095 .5709 L
s
.92857 .56715 m
.92857 .5709 L
s
.25 Mabswid
0 .56715 m
1 .56715 L
s
.02381 .13372 m
.03006 .13372 L
s
[(0.7)] .01131 .13372 1 0 Mshowa
.02381 .2782 m
.03006 .2782 L
s
[(0.8)] .01131 .2782 1 0 Mshowa
.02381 .42267 m
.03006 .42267 L
s
[(0.9)] .01131 .42267 1 0 Mshowa
.125 Mabswid
.02381 .16261 m
.02756 .16261 L
s
.02381 .19151 m
.02756 .19151 L
s
.02381 .2204 m
.02756 .2204 L
s
.02381 .2493 m
.02756 .2493 L
s
.02381 .30709 m
.02756 .30709 L
s
.02381 .33599 m
.02756 .33599 L
s
.02381 .36488 m
.02756 .36488 L
s
.02381 .39378 m
.02756 .39378 L
s
.02381 .45157 m
.02756 .45157 L
s
.02381 .48047 m
.02756 .48047 L
s
.02381 .50936 m
.02756 .50936 L
s
.02381 .53826 m
.02756 .53826 L
s
.02381 .10482 m
.02756 .10482 L
s
.02381 .07592 m
.02756 .07592 L
s
.02381 .04703 m
.02756 .04703 L
s
.02381 .01813 m
.02756 .01813 L
s
.02381 .59605 m
.02756 .59605 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03333 .01472 m
.03793 .01493 L
.04222 .0152 L
.04692 .01559 L
.05195 .01609 L
.06223 .01743 L
.07158 .019 L
.09303 .02386 L
.1133 .03 L
.15133 .04537 L
.19178 .06673 L
.23074 .09158 L
.2682 .11887 L
.30808 .15107 L
.34647 .18466 L
.38728 .22275 L
.40766 .24254 L
.41759 .25239 L
.41987 .25458 L
.42228 .25692 L
.42327 .25769 L
.42433 .25833 L
.4255 .25885 L
.42659 .25923 L
.43112 .2605 L
.43601 .26186 L
.44489 .26441 L
.4644 .27021 L
.50535 .28348 L
.5448 .29778 L
.58274 .31305 L
.62312 .33106 L
.66199 .3503 L
.69936 .37073 L
.73916 .39479 L
.77746 .42045 L
.81819 .45073 L
.85742 .48313 L
.89515 .51757 L
.9353 .55801 L
.97396 .60074 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003oooP000Kh0?ooo`?o
o`005Ooo0P001ooo1oP0nooo000Eool00`00ooooo`0=ool5n03fool001Goo`03003ooooo01;oo`;h
0?Coo`005Ooo00<00?ooool05?oo0oP0lOoo000Eool00`00ooooo`0Gool3n03^ool001Goo`03003o
oooo01[oo`;h0>coo`005Ooo00<00?ooool07?oo0oP0jOoo000Eool00`00ooooo`0Oool2n03Wool0
01Goo`8002;oo`;h0>Goo`005Ooo00<00?ooool08ooo0_P0hooo000Eool00`00ooooo`0Uool2n03Q
ool001Goo`03003ooooo02Ooo`;h0=ooo`005Ooo00<00?ooool0:Ooo0_P0gOoo000Eool00`00oooo
o`0[ool00oP0ooooo`3Jool001Goo`03003ooooo02coo`;h0=[oo`005Ooo00<00?ooool0;_oo00?h
0?ooool0eooo000Eool2000`ool00oP0ooooo`3Fool001Goo`03003ooooo033oo`;h0=Koo`005Ooo
00<00?ooool0<_oo00?h0?ooool0dooo000Eool00`00ooooo`0cool2n03Cool001Goo`03003ooooo
03Goo`03n03ooooo0=3oo`005Ooo00<00?ooool0=_oo0_P0d?oo000Eool00`00ooooo`0hool00oP0
ooooo`3=ool001Goo`8003[oo`03n03ooooo0<coo`005Ooo00<00?ooool0>_oo0_P0c?oo000Eool0
0`00ooooo`0lool00oP0ooooo`39ool001Goo`03003ooooo03goo`;h0<Woo`005Ooo00<00?ooool0
?ooo00?h0?ooool0a_oo000Eool00`00ooooo`10ool00oP0ooooo`35ool000;oo`8000Coo`03003o
oooo00?oo`03003ooooo00Coo`03003ooooo047oo`;h0<Goo`0000Goo`00ooooo`0000Woo`03003o
oooo00Coo`03003ooooo04?oo`03n03ooooo0<;oo`0000Goo`00ooooo`0000[oo`03003ooooo00?o
o`<004Coo`03n03ooooo0<7oo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo04Go
o`03n03ooooo0<3oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003ooooo04Koo`03n03o
oooo0;ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`17ool2n02oool000;o
o`8000Woo`@000Coo`03003ooooo04Woo`03n03ooooo0;coo`005Ooo00<00?ooool0B_oo00?h0?oo
ool0^ooo000Eool00`00ooooo`1;ool00oP0ooooo`2jool001Goo`03003ooooo04coo`;h0;[oo`00
5Ooo0P00Cooo00?h0?ooool0]ooo000Eool00`00ooooo`1?ool00oP0ooooo`2fool001Goo`03003o
oooo053oo`03n03ooooo0;Goo`005Ooo00<00?ooool0DOoo00?h0?ooool0]?oo000Eool00`00oooo
o`1Bool2n02dool001Goo`03003ooooo05Coo`03n03ooooo0;7oo`005Ooo00<00?ooool0EOoo00?h
0?ooool0/?oo000Eool00`00ooooo`1Fool00oP0ooooo`2_ool001Goo`8005Soo`03n03ooooo0:ko
o`005Ooo00<00?ooool0F?oo00?h0?ooool0[Ooo000Eool00`00ooooo`1Iool00oP0ooooo`2/ool0
01Goo`03003ooooo05[oo`03n03ooooo0:_oo`005Ooo00<00?ooool0Fooo00?h0?ooool0Z_oo000E
ool00`00ooooo`1Lool00oP0ooooo`2Yool001Goo`03003ooooo05goo`03n03ooooo0:Soo`005Ooo
00<00?ooool0G_oo00?h0?ooool0Yooo000Eool2001Pool00oP0ooooo`2Vool001Goo`03003ooooo
063oo`03n03ooooo0:Goo`005Ooo00<00?ooool0HOoo00?h0?ooool0Y?oo000Eool00`00ooooo`1R
ool00oP0ooooo`2Sool001Goo`03003ooooo06?oo`03n03ooooo0:;oo`005Ooo00<00?ooool0I?oo
00?h0?ooool0XOoo000Eool00`00ooooo`1Uool00oP0ooooo`2Pool001Goo`03003ooooo06Koo`03
n03ooooo09ooo`005Ooo0P00J?oo00?h0?ooool0W_oo000Eool00`00ooooo`1Xool2n02Nool001Go
o`03003ooooo06Woo`;h09goo`005Ooo00<00?ooool0Jooo0oP0V_oo000Eool00`00ooooo`1^ool4
n02Fool001Goo`03003ooooo07;oo`Gh097oo`000_oo0P001?oo00<00?ooool00ooo0P001Ooo00<0
0?ooool0Mooo0oP0S_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo07[oo`;h
08coo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3001lool3n029ool00005ool00?ooool0
0009ool20005ool00`00ooooo`1oool3n026ool00005ool00?ooool00008ool01000ooooo`001?oo
00<00?ooool0P_oo0oP0Pooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo08Go
o`;h087oo`000_oo0P002_oo0P001Ooo00<00?ooool0Qooo0oP0O_oo000Eool00`00ooooo`2:ool3
n01kool001Goo`03003ooooo08goo`;h07Woo`005Ooo00<00?ooool0Sooo0oP0M_oo000Eool2002C
ool2n01dool001Goo`03003ooooo09Coo`?h077oo`005Ooo00<00?ooool0Uooo0_P0Kooo000Eool0
0`00ooooo`2Iool2n01]ool001Goo`03003ooooo09_oo`;h06_oo`005Ooo00<00?ooool0WOoo0_P0
JOoo000Eool00`00ooooo`2Oool3n01Vool001Goo`800:?oo`;h06Coo`005Ooo00<00?ooool0Y?oo
0_P0H_oo000Eool00`00ooooo`2Vool2n01Pool001Goo`03003ooooo0:Soo`;h05koo`005Ooo00<0
0?ooool0Z_oo0_P0G?oo000Eool00`00ooooo`2/ool2n01Jool001Goo`03003ooooo0:koo`;h05So
o`005Ooo00<00?ooool0/?oo00?h0?ooool0EOoo000Eool2002bool2n01Eool001Goo`03003ooooo
0;?oo`;h05?oo`005Ooo00<00?ooool0]Ooo00?h0?ooool0D?oo000Eool00`00ooooo`2fool2n01@
ool001Goo`03003ooooo0;Soo`;h04koo`005Ooo00<00?ooool0^_oo0_P0C?oo000Eool00`00oooo
o`2lool2n01:ool001Goo`03003ooooo0;koo`;h04Soo`005Ooo0P00`Ooo00?h0?ooool0AOoo000E
ool00`00ooooo`31ool2n015ool001Goo`03003ooooo0<?oo`03n03ooooo04;oo`005Ooo00<00?oo
ool0a?oo00?h0?ooool0@Ooo000Eool00`00ooooo`35ool2n011ool001Goo`03003ooooo0<Ooo`03
n03ooooo03koo`000_oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0b?oo0_P0?_oo0000
1Ooo003ooooo00002ooo00<00?ooool00_oo00<00?ooool0b_oo00?h0?ooool0>ooo00001Ooo003o
oooo00002ooo00<00?ooool00_oo0`00booo00?h0?ooool0>_oo00001Ooo003ooooo00002Ooo0`00
1?oo00<00?ooool0c?oo0_P0>_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo
0<koo`03n03ooooo03Ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3?ool0
0oP0ooooo`0fool000;oo`8000[oo`8000Goo`03003ooooo0=3oo`03n03ooooo03Goo`005Ooo00<0
0?ooool0dOoo00?h0?ooool0=?oo000Eool00`00ooooo`3Bool2n00dool001Goo`03003ooooo0=Co
o`03n03ooooo037oo`005Ooo0P00e_oo00?h0?ooool0<?oo000Eool00`00ooooo`3Fool2n00`ool0
01Goo`03003ooooo0=Soo`03n03ooooo02goo`005Ooo00<00?ooool0fOoo00?h0?ooool0;?oo000E
ool00`00ooooo`3Jool2n00/ool001Goo`03003ooooo0=coo`03n03ooooo02Woo`005Ooo00<00?oo
ool0gOoo00?h0?ooool0:?oo000Eool00`00ooooo`3Nool2n00Xool001Goo`800>7oo`03n03ooooo
02Goo`005Ooo00<00?ooool0hOoo00?h0?ooool09?oo000Eool00`00ooooo`3Rool00oP0ooooo`0S
ool001Goo`03003ooooo0>?oo`03n03ooooo02;oo`005Ooo00<00?ooool0i?oo00?h0?ooool08Ooo
000Eool00`00ooooo`3Uool00oP0ooooo`0Pool001Goo`03003ooooo0>Koo`03n03ooooo01ooo`00
5Ooo00<00?ooool0iooo00?h0?ooool07_oo000Eool2003Yool00oP0ooooo`0Mool001Goo`03003o
oooo0>Woo`03n03ooooo01coo`005Ooo00<00?ooool0j_oo00?h0?ooool06ooo000Eool00`00oooo
o`3[ool00oP0ooooo`0Jool001Goo`03003ooooo0>coo`03n03ooooo01Woo`005Ooo00<00?ooool0
:_oo0P001?oo00<00?ooool00_oo10009Ooo0P001?oo00<00?ooool01?oo00<00?ooool08ooo0P00
1?oo00<00?ooool00ooo0P009_oo0P001?oo00<00?ooool00ooo0P006_oo00?h0?ooool03_oo1000
1_oo000Eool00`00ooooo`0Yool01000ooooo`002?oo00<00?ooool09Ooo00@00?ooool000[oo`03
003ooooo02;oo`04003ooooo0008ool01000ooooo`009?oo00@00?ooool000Soo`04003ooooo000J
ool00oP0ooooo`0?ool00`00ooooo`05ool001Goo`8002[oo`04003ooooo0009ool00`00ooooo`0T
ool01000ooooo`001ooo1@008ooo00@00?ooool000Soo`04003ooooo000Tool01000ooooo`002?oo
00@00?ooool001_oo`03n03ooooo00koo`03003ooooo00Goo`005Ooo00<00?ooool0:Ooo00@00?oo
ool000[oo`03003ooooo02?oo`04003ooooo0007ool01000ooooo`009?oo00@00?ooool000Soo`<0
02Goo`04003ooooo0009ool2000Mool00oP0ooooo`0=ool00`00ooooo`05ool001Goo`03003ooooo
02Woo`04003ooooo000;ool00`00ooooo`0Rool01000ooooo`002?oo00<00?oo00009?oo00@00?oo
ool000Soo`03003ooooo02Goo`04003ooooo0008ool01000ooooo`007Ooo00?h0?ooool03?oo00<0
0?ooool01Ooo000Eool00`00ooooo`0Yool01000ooooo`002?oo00@00?ooool002Coo`04003ooooo
0009ool2000Tool01000ooooo`002?oo00<00?ooool09Ooo00@00?ooool000Soo`04003ooooo000N
ool00oP0ooooo`09ool30007ool001Goo`03003ooooo02[oo`8000[oo`8002Koo`8000_oo`03003o
oooo02?oo`8000[oo`<002Goo`8000[oo`80023oo`03n03ooooo00[oo`03003ooooo00Goo`005Ooo
00<00?ooool0m?oo00?h0?ooool04Ooo000Eool00`00ooooo`3eool00oP0ooooo`0@ool001Goo`03
003ooooo0?Koo`03n03ooooo00ooo`003_ooo`000P0000?h000000003P00000Eool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`02ool00oP0ooooo`05
ool00`00ooooo`05ool001Goo`03003ooooo037oo`03003ooooo037oo`03003ooooo033oo`03003o
oooo037oo`03003ooooo02[oo`03n03ooooo00Coo`03003ooooo00Goo`005Ooo00<00?ooool0n_oo
00?h0?ooool02ooo000Eool00`00ooooo`3jool00oP0ooooo`0;ool001Goo`03003ooooo0?_oo`03
n03ooooo00[oo`005Ooo00<00?ooool0o?oo00?h0?ooool02Ooo000Eool00`00ooooo`3mool00oP0
ooooo`08ool001Goo`800?ooo`03n03ooooo00Ooo`005Ooo00<00?ooool0oooo00?h0?ooool01_oo
000Eool00`00ooooo`3oool1ool00oP0ooooo`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0825959, 0.596272, \
0.00385924, 0.00254394}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%1066, %1067]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.0147151 0.392403 [
[.15079 .00222 -9 -9 ]
[.15079 .00222 9 0 ]
[.27778 .00222 -9 -9 ]
[.27778 .00222 9 0 ]
[.40476 .00222 -9 -9 ]
[.40476 .00222 9 0 ]
[.53175 .00222 -9 -9 ]
[.53175 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.78571 .00222 -9 -9 ]
[.78571 .00222 9 0 ]
[.9127 .00222 -9 -9 ]
[.9127 .00222 9 0 ]
[.01131 .0932 -18 -4.5 ]
[.01131 .0932 0 4.5 ]
[.01131 .17168 -18 -4.5 ]
[.01131 .17168 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .32864 -18 -4.5 ]
[.01131 .32864 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .56408 -18 -4.5 ]
[.01131 .56408 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .01472 m
.15079 .02097 L
s
[(0.2)] .15079 .00222 0 1 Mshowa
.27778 .01472 m
.27778 .02097 L
s
[(0.4)] .27778 .00222 0 1 Mshowa
.40476 .01472 m
.40476 .02097 L
s
[(0.6)] .40476 .00222 0 1 Mshowa
.53175 .01472 m
.53175 .02097 L
s
[(0.8)] .53175 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(1)] .65873 .00222 0 1 Mshowa
.78571 .01472 m
.78571 .02097 L
s
[(1.2)] .78571 .00222 0 1 Mshowa
.9127 .01472 m
.9127 .02097 L
s
[(1.4)] .9127 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.18254 .01472 m
.18254 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.34127 .01472 m
.34127 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.5 .01472 m
.5 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.81746 .01472 m
.81746 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.97619 .01472 m
.97619 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .0932 m
.03006 .0932 L
s
[(0.2)] .01131 .0932 1 0 Mshowa
.02381 .17168 m
.03006 .17168 L
s
[(0.4)] .01131 .17168 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.6)] .01131 .25016 1 0 Mshowa
.02381 .32864 m
.03006 .32864 L
s
[(0.8)] .01131 .32864 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(1)] .01131 .40712 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(1.2)] .01131 .4856 1 0 Mshowa
.02381 .56408 m
.03006 .56408 L
s
[(1.4)] .01131 .56408 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .11282 m
.02756 .11282 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .21092 m
.02756 .21092 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .50522 m
.02756 .50522 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .5837 m
.02756 .5837 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03016 .25708 m
.03323 .25713 L
.03608 .25721 L
.03922 .25731 L
.04257 .25745 L
.04942 .25781 L
.05566 .25824 L
.06995 .25956 L
.08347 .26123 L
.10882 .2654 L
.13579 .2712 L
.16176 .27795 L
.18674 .28537 L
.21332 .29411 L
.23891 .30323 L
.26612 .31358 L
.27971 .31895 L
.28633 .32163 L
.28785 .32222 L
.28946 .32286 L
.29012 .32307 L
.29083 .32324 L
.2916 .32338 L
.29233 .32349 L
.29535 .32383 L
.29861 .3242 L
.30453 .32489 L
.31754 .32647 L
.34484 .33007 L
.37113 .33396 L
.39643 .3381 L
.42335 .343 L
.44926 .34822 L
.47418 .35377 L
.50071 .36031 L
.52625 .36727 L
.5534 .3755 L
.57955 .3843 L
.6047 .39365 L
.63147 .40463 L
.65724 .41624 L
.65873 .41694 L
s
0 g
.03016 .01864 m
.06854 .04236 L
.11039 .06823 L
.1497 .09252 L
.1875 .11588 L
.22774 .14075 L
.26647 .16469 L
.30764 .19013 L
.34731 .21465 L
.38546 .23823 L
.42606 .26332 L
.46515 .28748 L
.50273 .3107 L
.54275 .33544 L
.58126 .35924 L
.62221 .38455 L
.66166 .40893 L
.69959 .43237 L
.73997 .45733 L
.77884 .48135 L
.82014 .50688 L
.85994 .53147 L
.89823 .55514 L
.93896 .58031 L
.97619 .60332 L
s
.03016 .30939 m
.03133 .30943 L
.03239 .30947 L
.03362 .30932 L
.03477 .30933 L
.03585 .30962 L
.03684 .30967 L
.03908 .30981 L
.04372 .31016 L
.04884 .31064 L
.0584 .3118 L
.06854 .31335 L
.11039 .32202 L
.15041 .33176 L
.18892 .34119 L
.22987 .35077 L
.26932 .35939 L
.30725 .36708 L
.34763 .3746 L
.3865 .38121 L
.4278 .38753 L
.4676 .39292 L
.5059 .3974 L
.54663 .40134 L
.56691 .40295 L
.58585 .40422 L
.60522 .40528 L
.62357 .40606 L
.63325 .40638 L
.64379 .40665 L
.64843 .40675 L
.65336 .40684 L
.65767 .4069 L
.66222 .40695 L
.66454 .40698 L
.66707 .407 L
.66941 .40701 L
.67153 .40702 L
.67273 .40703 L
.67404 .40703 L
.67476 .40703 L
.67542 .40704 L
.67672 .40704 L
.67784 .40704 L
.67907 .40704 L
.68036 .40704 L
.68156 .40704 L
.68268 .40704 L
.68391 .40703 L
.68613 .40703 L
Mistroke
.6883 .40702 L
.69031 .407 L
.69487 .40697 L
.69972 .40692 L
.70996 .40678 L
.72103 .40657 L
.74107 .40608 L
.78186 .40477 L
.82113 .40335 L
.8589 .402 L
.89911 .40067 L
.93781 .39952 L
.97619 .39851 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool0033oo`8000Coo`03003ooooo00;oo`@001Coo`8000Coo`03003ooooo00Coo`03003ooooo01?o
o`8000Coo`03003ooooo00?oo`8001Coo`8000Coo`03003ooooo00?oo`8001[oo`@001Soo`@000?o
o`03003ooooo00;oo`@001?oo`@000?oo`03003ooooo00Coo`03003ooooo013oo`00;ooo00@00?oo
ool000Soo`03003ooooo01Coo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`002?oo00@0
0?ooool001;oo`04003ooooo0008ool01000ooooo`006ooo00<00?ooool06Ooo00<00?ooool01ooo
00<00?ooool05_oo00<00?ooool02Ooo00<00?ooool04?oo000_ool01000ooooo`002Ooo00<00?oo
ool04ooo00@00?ooool000Ooo`D001?oo`04003ooooo0008ool01000ooooo`004_oo00@00?ooool0
00Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`08ool00`00ooooo`0Eool00`00oooo
o`06ool5000Aool002ooo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`001ooo00@00?oo
ool001Coo`04003ooooo0008ool3000Cool01000ooooo`002Ooo0P007?oo00<00?ooool06Ooo00<0
0?ooool02Ooo00<00?ooool05?oo00<00?ooool01_oo00@00?ooool001;oo`00;ooo00@00?ooool0
00_oo`03003ooooo017oo`04003ooooo0008ool00`00ool0000Dool01000ooooo`002?oo00<00?oo
ool04ooo00@00?ooool000Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`0:ool00`00
ooooo`0Cool00`00ooooo`07ool00`00ool0000Bool002ooo`04003ooooo0008ool01000ooooo`00
4ooo00@00?ooool000Woo`8001Coo`04003ooooo0008ool00`00ooooo`0Cool01000ooooo`002?oo
00@00?ooool001Woo`<001Woo`<000Woo`04003ooooo000Cool3000:ool2000Bool001Goo`03003o
oooo01Soo`8000[oo`8001Goo`8000_oo`03003ooooo01?oo`8000[oo`<001?oo`8000[oo`8001co
o`03003ooooo01Woo`03003ooooo00Soo`8001Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo
00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`003_oo
o`004`00000Eool00`00ool00006ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool001Goo`03003ooooo008001goo`03003ooooo023oo`03003o
oooo023oo`03003ooooo01ooo`03003ooooo023oo`03003ooooo01ooo`03003ooooo023oo`03003o
oooo01Koo`005Ooo00<00?ooool00_oo00<00?ooool0oooo1?oo000Eool00`00ooooo`03ool00`00
ooooo`3oool3ool001Goo`8000Goo`800?ooo`?oo`005Ooo00<00?ooool01_oo00<00?ooool0oooo
000Eool00`00ooooo`07ool2003oool001Goo`03003ooooo00Woo`03003ooooo0?coo`005Ooo00<0
0?ooool02_oo0P00o?oo000Eool00`00ooooo`0<ool00`00ooooo`3iool001Goo`8000koo`800?Wo
o`005Ooo00<00?ooool03ooo0P00mooo000Eool00`00ooooo`0Aool00`00ooooo`3dool001Goo`03
003ooooo01;oo`800?Coo`005Ooo00<00?ooool05?oo00<00?ooool0lOoo000Eool2000Fool2003a
ool001Goo`03003ooooo01Ooo`800>ooo`005Ooo00<00?ooool06Ooo0P00kOoo000Eool00`00oooo
o`0Kool2003[ool000;oo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo01goo`800>Woo`00
00Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo01ooo`03003ooooo0>Koo`0000Goo`00
ooooo`0000Woo`03003ooooo00Coo`<0023oo`800>Koo`0000Goo`00ooooo`0000[oo`03003ooooo
00?oo`03003ooooo02;oo`03003ooooo0>?oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03
003ooooo02?oo`03003ooooo0>;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`0Tool2003Rool000;oo`8000[oo`8000Goo`03003ooooo02Koo`03003ooooo0=ooo`005Ooo0P00
:?oo0P00gooo000Eool00`00ooooo`0Yool00`00ooooo`3Lool001Goo`03003ooooo02[oo`800=co
o`005Ooo00<00?ooool0;?oo0P00f_oo000Eool00`00ooooo`0^ool2003Hool001Goo`80037oo`80
0=Koo`005Ooo00<00?ooool0<_oo0P00e?oo000Eool00`00ooooo`0dool00`00ooooo`3Aool001Go
o`03003ooooo03Goo`800=7oo`005Ooo00<00?ooool0=ooo00<00?ooool0c_oo000Eool00`00oooo
o`0hool2003>ool001Goo`8003_oo`800<coo`005Ooo00<00?ooool0??oo00<00?ooool0bOoo000E
ool00`00ooooo`0mool20039ool000;oo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003o
oooo03ooo`03003ooooo0<Koo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo043o
o`800<Koo`0000Goo`00ooooo`0000Ooo`D000Coo`<004;oo`03003ooooo0<?oo`0000Goo`00oooo
o`0000Ooo`04003ooooo0005ool00`00ooooo`13ool20033ool00005ool00?ooool00008ool00`00
ool00005ool00`00ooooo`15ool20031ool00005ool00?ooool00009ool20005ool00`00ooooo`17
ool00`00ooooo`2nool000;oo`8000_oo`03003ooooo00?oo`03003ooooo04Soo`800;koo`005Ooo
0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool2002kool001Goo`03003ooooo04goo`03
003ooooo0;Soo`005Ooo00<00?ooool0C_oo0P00^?oo000Eool00`00ooooo`1@ool2002fool001Go
o`03003ooooo05;oo`03003ooooo0;?oo`005Ooo0P00E?oo0P00/ooo000Eool00`00ooooo`1Eool0
0`00ooooo`2`ool001Goo`03003ooooo05Koo`800;3oo`005Ooo00<00?ooool0F?oo0P00[_oo000E
ool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`8005coo`800:_oo`005Ooo00<00?ooool0GOoo
0P00ZOoo000Eool00`00ooooo`1Oool00`00ooooo`2Vool000;oo`8000Coo`03003ooooo00?oo`80
00Goo`03003ooooo063oo`800:Koo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`1Rool00`00ooooo`2Sool00005ool00?ooool00008ool01000ooooo`001?oo0`00Hooo0P00Xooo
00001Ooo003ooooo00002?oo0`001Ooo00<00?ooool0IOoo0P00XOoo00001Ooo003ooooo00002?oo
00<00?ooool01Ooo00<00?oon0002?P0Gooo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<0
0?ooool01Ooo00<00?ooool02?oo2OP0Eooo0P00W_oo0002ool2000:ool30004ool00`00ooooo`0A
ool5n01Dool00`00ooooo`2Kool001Goo`03003ooooo01Koo`Ch057oo`8009_oo`005Ooo0P006ooo
0oP0D?oo0P00VOoo000Eool00`00ooooo`0Mool4n01>ool2002Gool001Goo`03003ooooo027oo`?h
04goo`8009Goo`005Ooo00<00?ooool09?oo1?P0Booo0P00Tooo000Eool00`00ooooo`0Xool3n01:
ool00`00ooooo`2@ool001Goo`8002coo`Ch04Ooo`80093oo`005Ooo00<00?ooool0;ooo0oP0A_oo
00<00?ooool0SOoo000Eool00`00ooooo`0bool2n015ool00`00ooooo`2<ool001Goo`03003ooooo
03Coo`;h04Coo`8008coo`005Ooo00<00?ooool0=_oo0oP0@ooo00<00?ooool0ROoo000Eool5000g
ool4n010ool20029ool001Goo`03003ooooo00;oo`T003;oo`?h03ooo`03003ooooo08Koo`005Ooo
00<00?ooool02ooo1000<Ooo0_P0?_oo0P00Q_oo000Eool00`00ooooo`0?ool4000_ool2n00nool2
0024ool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo01?oo`@002goo`Gh03_oo`80
08;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Gool6000/ool8n00eool2
0020ool00005ool00?ooool00008ool01000ooooo`001?oo0`007Ooo1000<?oo1oP0<?oo0P00O_oo
00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool08Ooo1000<ooo1oP0:ooo00<00?ooool0Nooo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo02Goo`@003Koo`Oh02Goo`8007_o
o`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Yool3000jool6n00Qool00`00
ooooo`1hool000;oo`8000[oo`8000Goo`03003ooooo02coo`@003coo`Ch01koo`8007Soo`005Ooo
0P00<Ooo1000??oo1?P07?oo0P00M_oo000Eool00`00ooooo`0dool4000lool5n00Iool00`00oooo
o`1cool001Goo`03003ooooo03Soo`H003_oo`Ch01Koo`8007?oo`005Ooo00<00?ooool0?_oo1@00
>_oo1?P05?oo00<00?ooool0L?oo000Eool00`00ooooo`13ool5000iool4n00Aool2001`ool001Go
o`8004Woo`D003Soo`?h013oo`03003ooooo06goo`005Ooo00<00?ooool0COoo1P00=Ooo1?P03Ooo
0P00KOoo000Eool00`00ooooo`1Cool5000dool3n00<ool2001[ool001Goo`03003ooooo05Soo`H0
037oo`;h00coo`03003ooooo06Soo`005Ooo00<00?ooool0G_oo1@00;_oo0_P02ooo0P00J?oo000E
ool00`00ooooo`1Sool6000Zool3n00:ool00`00ooooo`1Uool001Goo`8006[oo`P002Goo`Ch00Oo
o`8006Goo`005Ooo00<00?ooool0LOoo2P007ooo0oP01_oo0P00Hooo000Eool00`00ooooo`1kool;
000Gool2n006ool00`00ooooo`1Pool000goo`@000Coo`03003ooooo08Koo`P0017oo`;h00Goo`80
03kooa/000Ooo`003ooo00<00?ooool00ooo00<00?ooool0S_oo40000ooo0oP01?oo0P007?oo8000
8_oo000?ool00`00ooooo`03ool3002NoolX0012ool000ooo`03003ooooo00?oo`03003ooooo0:Ko
o`;h00?oo`8005_oo`003ooo00<00?ooool00ooo00<00?ooool0Z?oo0_P00ooo00<00?ooool0F?oo
000=ool30005ool00`00ooooo`2Zool01?P0oooooooo0P00F?oo000?ool00`00ooooo`03ool00`00
ooooo`2`ool2001Fool001Goo`800;?oo`03003ooooo05?oo`005Ooo00<00?ooool0/ooo0P00Dooo
000Eool00`00ooooo`2eool00`00ooooo`1@ool001Goo`03003ooooo0;Koo`80053oo`005Ooo00<0
0?ooool0^?oo00<00?ooool0COoo000Eool00`00ooooo`2iool2001=ool001Goo`800;coo`8004_o
o`005Ooo00<00?ooool0_Ooo00<00?ooool0B?oo000Eool00`00ooooo`2nool20018ool001Goo`03
003ooooo0<3oo`03003ooooo04Goo`005Ooo00<00?ooool0`Ooo0P00AOoo000Eool20034ool20013
ool001Goo`03003ooooo0<Goo`03003ooooo043oo`005Ooo00<00?ooool0a_oo0P00@?oo00000ooo
000000020003ool00`00ooooo`02ool40004ool00`00ooooo`38ool2000nool000?oo`03003ooooo
00Ooo`03003ooooo00Goo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?ooool02?oo00<0
0?ooool01?oo0`00booo0P00>ooo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3=
ool2000iool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo0<ooo`03003ooooo03Ko
o`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3@ool2000fool000?oo`03003ooooo
00Soo`8000Goo`03003ooooo0=;oo`8003Coo`005Ooo00<00?ooool0e?oo0P00<_oo000Eool2003G
ool00`00ooooo`0_ool001Goo`03003ooooo0=Ooo`8002ooo`005Ooo00<00?ooool0fOoo00<00?oo
ool0;?oo000Eool00`00ooooo`3Jool00`00ooooo`0[ool001Goo`03003ooooo0=_oo`8002_oo`00
5Ooo0P00g_oo00<00?ooool0:?oo000Eool00`00ooooo`3Nool2000Xool001Goo`03003ooooo0>3o
o`03003ooooo02Goo`005Ooo00<00?ooool0hOoo0P009Ooo000Eool00`00ooooo`3Sool2000Sool0
01Goo`800>Koo`80027oo`005Ooo00<00?ooool0iooo0P007ooo000Eool00`00ooooo`3Yool2000M
ool001Goo`03003ooooo0>_oo`03003ooooo01[oo`0000?oo`0000000P000ooo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool0k?oo0P006_oo0003ool00`00ooooo`09ool00`00ooooo`03ool0
0`00ooooo`3^ool00`00ooooo`0Gool000?oo`03003ooooo00Koo`D000Coo`<00>ooo`8001Ooo`00
0ooo00<00?ooool01_oo00@00?ooool000Goo`03003ooooo0?7oo`8001Goo`000ooo00<00?ooool0
1ooo00<00?oo00001Ooo00<00?ooool0looo00<00?ooool04_oo00001?oo000000002_oo0P001Ooo
00<00?ooool0m?oo0P004_oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3fool0
0`00ooooo`0?ool001Goo`800?Soo`8000ooo`005Ooo00<00?ooool0nOoo0P003Ooo000Eool00`00
ooooo`3kool00`00ooooo`0:ool001Goo`03003ooooo0?coo`8000[oo`005Ooo00<00?ooool0o_oo
0P002?oo000Eool2003oool2ool00`00ooooo`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, -0.117974, \
0.00578885, 0.00936656}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol2[k, 10]], {k, 0.01, 0.4}, 
      PlotStyle \[Rule] {RGBColor[0, 1, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -13.7251 18.3014 [
[.2619 -0.01155 -9 -9 ]
[.2619 -0.01155 9 0 ]
[.5 -0.01155 -9 -9 ]
[.5 -0.01155 9 0 ]
[.7381 -0.01155 -9 -9 ]
[.7381 -0.01155 9 0 ]
[.97619 -0.01155 -9 -9 ]
[.97619 -0.01155 9 0 ]
[.01131 .09246 -30 -4.5 ]
[.01131 .09246 0 4.5 ]
[.01131 .18396 -24 -4.5 ]
[.01131 .18396 0 4.5 ]
[.01131 .27547 -30 -4.5 ]
[.01131 .27547 0 4.5 ]
[.01131 .36698 -24 -4.5 ]
[.01131 .36698 0 4.5 ]
[.01131 .45848 -30 -4.5 ]
[.01131 .45848 0 4.5 ]
[.01131 .54999 -24 -4.5 ]
[.01131 .54999 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .00095 m
.2619 .0072 L
s
[(0.1)] .2619 -0.01155 0 1 Mshowa
.5 .00095 m
.5 .0072 L
s
[(0.2)] .5 -0.01155 0 1 Mshowa
.7381 .00095 m
.7381 .0072 L
s
[(0.3)] .7381 -0.01155 0 1 Mshowa
.97619 .00095 m
.97619 .0072 L
s
[(0.4)] .97619 -0.01155 0 1 Mshowa
.125 Mabswid
.07143 .00095 m
.07143 .0047 L
s
.11905 .00095 m
.11905 .0047 L
s
.16667 .00095 m
.16667 .0047 L
s
.21429 .00095 m
.21429 .0047 L
s
.30952 .00095 m
.30952 .0047 L
s
.35714 .00095 m
.35714 .0047 L
s
.40476 .00095 m
.40476 .0047 L
s
.45238 .00095 m
.45238 .0047 L
s
.54762 .00095 m
.54762 .0047 L
s
.59524 .00095 m
.59524 .0047 L
s
.64286 .00095 m
.64286 .0047 L
s
.69048 .00095 m
.69048 .0047 L
s
.78571 .00095 m
.78571 .0047 L
s
.83333 .00095 m
.83333 .0047 L
s
.88095 .00095 m
.88095 .0047 L
s
.92857 .00095 m
.92857 .0047 L
s
.25 Mabswid
0 .00095 m
1 .00095 L
s
.02381 .09246 m
.03006 .09246 L
s
[(0.755)] .01131 .09246 1 0 Mshowa
.02381 .18396 m
.03006 .18396 L
s
[(0.76)] .01131 .18396 1 0 Mshowa
.02381 .27547 m
.03006 .27547 L
s
[(0.765)] .01131 .27547 1 0 Mshowa
.02381 .36698 m
.03006 .36698 L
s
[(0.77)] .01131 .36698 1 0 Mshowa
.02381 .45848 m
.03006 .45848 L
s
[(0.775)] .01131 .45848 1 0 Mshowa
.02381 .54999 m
.03006 .54999 L
s
[(0.78)] .01131 .54999 1 0 Mshowa
.125 Mabswid
.02381 .01925 m
.02756 .01925 L
s
.02381 .03755 m
.02756 .03755 L
s
.02381 .05585 m
.02756 .05585 L
s
.02381 .07415 m
.02756 .07415 L
s
.02381 .11076 m
.02756 .11076 L
s
.02381 .12906 m
.02756 .12906 L
s
.02381 .14736 m
.02756 .14736 L
s
.02381 .16566 m
.02756 .16566 L
s
.02381 .20226 m
.02756 .20226 L
s
.02381 .22057 m
.02756 .22057 L
s
.02381 .23887 m
.02756 .23887 L
s
.02381 .25717 m
.02756 .25717 L
s
.02381 .29377 m
.02756 .29377 L
s
.02381 .31207 m
.02756 .31207 L
s
.02381 .33037 m
.02756 .33037 L
s
.02381 .34868 m
.02756 .34868 L
s
.02381 .38528 m
.02756 .38528 L
s
.02381 .40358 m
.02756 .40358 L
s
.02381 .42188 m
.02756 .42188 L
s
.02381 .44018 m
.02756 .44018 L
s
.02381 .47678 m
.02756 .47678 L
s
.02381 .49509 m
.02756 .49509 L
s
.02381 .51339 m
.02756 .51339 L
s
.02381 .53169 m
.02756 .53169 L
s
.02381 .56829 m
.02756 .56829 L
s
.02381 .58659 m
.02756 .58659 L
s
.02381 .60489 m
.02756 .60489 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.04762 .01472 m
.05637 .01502 L
.06596 .01547 L
.08529 .01671 L
.10656 .01861 L
.12637 .02089 L
.16439 .02663 L
.2048 .03471 L
.24373 .04443 L
.28118 .05558 L
.32103 .06939 L
.35939 .08459 L
.39628 .10099 L
.43556 .12039 L
.47337 .14095 L
.51356 .16487 L
.55228 .18995 L
.58951 .21597 L
.62914 .24574 L
.6673 .27645 L
.70784 .31131 L
.7469 .34713 L
.78449 .38367 L
.82447 .42484 L
.86297 .46678 L
.89999 .50906 L
.9394 .55657 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool005Ooo`8000Coo`03003ooooo00;oo`@002koo`8000Co
o`03003ooooo00;oo`@002koo`8000Coo`03003ooooo00?oo`80033oo`8000Coo`03003ooooo00Co
o`03003ooooo007oo`00E_oo00@00?ooool000[oo`03003ooooo02coo`04003ooooo0008ool00`00
ooooo`0^ool01000ooooo`002?oo00@00?ooool002koo`04003ooooo000:ool00`00ooooo`01ool0
05Koo`04003ooooo000:ool00`00ooooo`0/ool01000ooooo`002Ooo00<00?ooool0;Ooo00@00?oo
ool000_oo`03003ooooo02coo`04003ooooo0007ool50002ool005Koo`04003ooooo000:ool00`00
ooooo`0/ool01000ooooo`002_oo00<00?ooool0;?oo00@00?ooool000Woo`8002ooo`04003ooooo
0007ool01000ooooo`000ooo001Fool01000ooooo`002_oo00<00?ooool0;?oo00@00?ooool000_o
o`03003ooooo02_oo`04003ooooo000;ool00`00ooooo`0/ool01000ooooo`002?oo00<00?oo0000
0ooo001Fool01000ooooo`002?oo0`00;_oo00@00?ooool000Soo`04003ooooo000]ool01000oooo
o`002?oo00@00?ooool002koo`04003ooooo0009ool20003ool005Ooo`8000_oo`03003ooooo02go
o`8000[oo`8002ooo`8000[oo`80033oo`8000_oo`03003ooooo007oo`00oooo8Ooo003ooolQool0
0?ooob7oo`006oooo`000`000ooo000Qool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`07ool0027oo`03003ooooo0?coo`008Ooo
00<00?ooool00ooo10OPmOoo000Qool00`00ooooo`07ool>1n3Wool0027oo`8001Koo`L7h>3oo`00
8Ooo00<00?ooool07?oo1`OPfOoo000Qool00`00ooooo`0Sool31n3Fool0027oo`03003ooooo02Ko
o`@7h=;oo`008Ooo0P00:ooo10OPc_oo000Qool00`00ooooo`0^ool51n39ool0027oo`03003ooooo
03?oo`@7h<Goo`008Ooo00<00?ooool0=ooo0`OP`_oo000Qool00`00ooooo`0jool41n2nool0027o
o`8003ooo`<7h;_oo`008Ooo00<00?ooool0@Ooo0`OP^?oo000Qool00`00ooooo`14ool41n2dool0
027oo`03003ooooo04Soo`<7h;7oo`008Ooo00<00?ooool0Booo0POP[ooo000Qool2001>ool31n2/
ool0027oo`03003ooooo053oo`87h:[oo`000_oo0P001?oo00<00?ooool00ooo00<00?ooool00_oo
0`000ooo0`001Ooo00<00?ooool0D_oo0POPZ?oo00001Ooo003ooooo00002Ooo00<00?ooool01Ooo
00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0E?oo0POPY_oo00001Ooo003ooooo00002_oo
00<00?ooool01?oo00<00?ooool00ooo00<00?ooool00_oo0P00Eooo0POPY?oo00001Ooo003ooooo
00002_oo00@00?ooooooo`<000?oo`<000Goo`03003ooooo05Soo`87h:;oo`0000Goo`00ooooo`00
00_oo`04003ooooo0005ool00`00ooooo`05ool00`00ooooo`1Jool21n2Pool00005ool00?ooool0
0008ool01000ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0G?oo0POPW_oo
0002ool20009ool40002ool40002ool40004ool00`00ooooo`1Nool21n2Lool0027oo`80067oo`87
h9[oo`008Ooo00<00?ooool0H_oo0POPV?oo000Qool00`00ooooo`1Tool21n2Fool0027oo`03003o
oooo06Koo`87h9Coo`008Ooo00<00?ooool0J?oo0POPT_oo000Qool2001[ool21n2@ool0027oo`03
003ooooo06coo`87h8koo`008Ooo00<00?ooool0K_oo0POPS?oo000Qool00`00ooooo`1`ool00`OP
ooooo`29ool0027oo`8007;oo`87h8Woo`008Ooo00<00?ooool0Looo0POPQooo000Qool00`00oooo
o`1eool21n25ool0027oo`03003ooooo07Ooo`87h8?oo`008Ooo00<00?ooool0NOoo0POPPOoo000Q
ool2001lool00`OPooooo`1nool0027oo`03003ooooo07coo`87h7koo`008Ooo00<00?ooool0O_oo
0POPO?oo0008ool20004ool00`00ooooo`03ool00`00ooooo`03ool20005ool00`00ooooo`20ool0
0`OPooooo`1iool000Ooo`04003ooooo0009ool00`00ooooo`02ool01000ooooo`001?oo00<00?oo
ool0POoo0POPNOoo0007ool01000ooooo`002_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`80
08Coo`87h7Ooo`001ooo00@00?ooool000[oo`04003oooooool30005ool00`00ooooo`25ool00`OP
ooooo`1dool000Ooo`04003ooooo000;ool01000ooooo`001ooo00<00?ooool0Q_oo00<7h?ooool0
Looo0007ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo08Ooo`87
h7?oo`002?oo0P002Ooo10000ooo0`001?oo00<00?ooool0ROoo00<7h?ooool0L?oo000Qool2002;
ool00`OPooooo`1_ool0027oo`03003ooooo08_oo`87h6ooo`008Ooo00<00?ooool0SOoo00<7h?oo
ool0K?oo000Qool00`00ooooo`2>ool00`OPooooo`1[ool0027oo`80093oo`031n3ooooo06[oo`00
8Ooo00<00?ooool0T?oo0POPJ_oo000Qool00`00ooooo`2Bool00`OPooooo`1Wool0027oo`03003o
oooo09?oo`031n3ooooo06Koo`008Ooo00<00?ooool0U?oo00<7h?ooool0IOoo000Qool2002Fool2
1n1Uool0027oo`03003ooooo09Ooo`031n3ooooo06;oo`008Ooo00<00?ooool0V?oo00<7h?ooool0
HOoo000Qool00`00ooooo`2Iool00`OPooooo`1Pool0027oo`03003ooooo09[oo`87h63oo`008Ooo
0P00WOoo00<7h?ooool0GOoo000Qool00`00ooooo`2Mool00`OPooooo`1Lool000;oo`8000Coo`03
003ooooo00?oo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo09koo`031n3ooooo05_oo`00
00Goo`00ooooo`0000Woo`03003ooooo00;oo`04003ooooo0005ool00`00ooooo`02ool00`00oooo
o`2Oool21n1Kool00005ool00?ooool0000:ool01@00oooooooo00000_oo00<00?ooool00ooo00<0
0?ooool00_oo0P00X_oo00<7h?ooool0F?oo00001Ooo003ooooo00002_oo00@00?ooooooo`<000?o
o`<000Goo`03003ooooo0:;oo`031n3ooooo05Ooo`0000Goo`00ooooo`0000_oo`04003ooooo0005
ool00`00ooooo`05ool00`00ooooo`2Sool00`OPooooo`1Fool00005ool00?ooool00008ool01000
ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0Y?oo0POPE_oo0002ool20009
ool40003ool30002ool40004ool00`00ooooo`2Vool00`OPooooo`1Cool0027oo`800:Soo`031n3o
oooo05;oo`008Ooo00<00?ooool0Z?oo00<7h?ooool0DOoo000Qool00`00ooooo`2Yool00`OPoooo
o`1@ool0027oo`03003ooooo0:[oo`87h53oo`008Ooo00<00?ooool0[?oo00<7h?ooool0COoo000Q
ool2002^ool00`OPooooo`1<ool0027oo`03003ooooo0:koo`031n3ooooo04_oo`008Ooo00<00?oo
ool0[ooo00<7h?ooool0B_oo000Qool00`00ooooo`2`ool00`OPooooo`19ool0027oo`03003ooooo
0;7oo`031n3ooooo04Soo`008Ooo0P00/ooo0POPB?oo000Qool00`00ooooo`2dool00`OPooooo`15
ool0027oo`03003ooooo0;Goo`031n3ooooo04Coo`008Ooo00<00?ooool0]_oo00<7h?ooool0@ooo
000Qool2002hool00`OPooooo`12ool0027oo`03003ooooo0;Soo`031n3ooooo047oo`008Ooo00<0
0?ooool0^Ooo00<7h?ooool0@?oo0008ool20004ool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`04ool00`00ooooo`2jool00`OPooooo`0oool000Ooo`04003ooooo0009ool00`00ooooo`03
ool00`00ooooo`04ool00`00ooooo`2kool00`OPooooo`0nool000Ooo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`03ool2002mool00`OPooooo`0mool000Ooo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`2mool00`OPooooo`0lool000Ooo`04003ooooo000;
ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`2nool00`OPooooo`0kool000Ooo`04003o
oooo0008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo0;ooo`031n3ooooo03[oo`00
2?oo0P002Ooo10000_oo10001?oo00<00?ooool0`?oo00<7h?ooool0>Ooo000Qool20032ool00`OP
ooooo`0hool0027oo`03003ooooo0<;oo`031n3ooooo03Ooo`008Ooo00<00?ooool0`ooo00<7h?oo
ool0=_oo000Qool00`00ooooo`34ool00`OPooooo`0eool0027oo`800<Koo`031n3ooooo03Coo`00
8Ooo00<00?ooool0a_oo00<7h?ooool0<ooo000Qool00`00ooooo`37ool00`OPooooo`0bool0027o
o`03003ooooo0<Soo`031n3ooooo037oo`008Ooo00<00?ooool0bOoo00<7h?ooool0<?oo000Qool2
003;ool00`OPooooo`0_ool0027oo`03003ooooo0<_oo`031n3ooooo02koo`008Ooo00<00?ooool0
c?oo00<7h?ooool0;Ooo000Qool00`00ooooo`3=ool00`OPooooo`0/ool0027oo`03003ooooo0<go
o`031n3ooooo02coo`008Ooo0P00cooo00<7h?ooool0:ooo000Qool00`00ooooo`3?ool00`OPoooo
o`0Zool000;oo`8000Coo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00;oo`<000Goo`03
003ooooo0=3oo`031n3ooooo02Woo`0000Goo`00ooooo`0000Woo`03003ooooo00?oo`03003ooooo
00Goo`03003ooooo00;oo`03003ooooo0=7oo`031n3ooooo02Soo`0000Goo`00ooooo`0000[oo`03
003ooooo00?oo`03003ooooo00Coo`03003ooooo00;oo`800=?oo`031n3ooooo02Ooo`0000Goo`00
ooooo`0000[oo`03003ooooo00?oo`04003oooooool30005ool00`00ooooo`3Bool00`OPooooo`0W
ool00005ool00?ooool0000;ool00`00ooooo`03ool01000ooooo`001ooo00<00?ooool0dooo00<7
h?ooool09_oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool00`00oooo
o`05ool00`00ooooo`3Dool00`OPooooo`0Uool000;oo`8000Woo`@000;oo`@000;oo`@000Coo`03
003ooooo0=Goo`031n3ooooo02Coo`008Ooo0P00eooo00<7h?ooool08ooo000Qool00`00ooooo`3G
ool00`OPooooo`0Rool0027oo`03003ooooo0=Soo`031n3ooooo027oo`008Ooo00<00?ooool0fOoo
00<7h?ooool08?oo000Qool00`00ooooo`3Jool00`OPooooo`0Oool0027oo`800=coo`031n3ooooo
01koo`008Ooo00<00?ooool0g?oo00<7h?ooool07Ooo000Qool00`00ooooo`3Mool00`OPooooo`0L
ool0027oo`03003ooooo0=koo`031n3ooooo01_oo`008Ooo00<00?ooool0gooo00<7h?ooool06_oo
000Qool2003Qool00`OPooooo`0Iool0027oo`03003ooooo0>3oo`031n3ooooo01Woo`008Ooo00<0
0?ooool0hOoo00<7h?ooool06?oo000Qool00`00ooooo`3Rool00`OPooooo`0Gool0027oo`800>Co
o`031n3ooooo01Koo`008Ooo00<00?ooool0hooo00<7h?ooool05_oo000Qool00`00ooooo`3Tool0
0`OPooooo`0Eool000Soo`8000Coo`03003ooooo00?oo`03003ooooo00?oo`8000Goo`03003ooooo
0>Goo`031n3ooooo01Coo`001ooo00@00?ooool000Woo`03003ooooo00;oo`04003ooooo0004ool0
0`00ooooo`3Vool00`OPooooo`0Cool000Ooo`04003ooooo000:ool01@00oooooooo00000_oo00<0
0?ooool00_oo0P00iooo00<7h?ooool04ooo0007ool01000ooooo`002_oo00<00?ooool00_oo0P00
1Ooo00<00?ooool0iooo00<7h?ooool04_oo0007ool01000ooooo`002ooo00@00?ooool000;oo`03
003ooooo00;oo`03003ooooo0>Soo`031n3ooooo017oo`001ooo00@00?ooool000Soo`04003ooooo
0002ool01000ooooo`001?oo00<00?ooool0jOoo00<7h?ooool04?oo0008ool20009ool40003ool2
0005ool00`00ooooo`3Zool00`OPooooo`0?ool0027oo`800>_oo`031n3ooooo00ooo`008Ooo00<0
0?ooool0jooo00<7h?ooool03_oo000Qool00`00ooooo`3/ool00`OPooooo`0=ool0027oo`03003o
oooo0>goo`031n3ooooo00coo`008Ooo0P00kooo00<7h?ooool02ooo000Qool00`00ooooo`3_ool0
0`OPooooo`0:ool0027oo`03003ooooo0>ooo`031n3ooooo00[oo`008Ooo00<00?ooool0l?oo00<7
h?ooool02Ooo000Qool00`00ooooo`3aool00`OPooooo`08ool0027oo`800??oo`031n3ooooo00Oo
o`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`00oooo8Ooo
003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0542745, 0.746753, \
0.00163409, 0.000212589}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %1070]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -0.183123 0.524294 [
[.15079 .32867 -9 -9 ]
[.15079 .32867 9 0 ]
[.27778 .32867 -9 -9 ]
[.27778 .32867 9 0 ]
[.40476 .32867 -9 -9 ]
[.40476 .32867 9 0 ]
[.53175 .32867 -9 -9 ]
[.53175 .32867 9 0 ]
[.65873 .32867 -3 -9 ]
[.65873 .32867 3 0 ]
[.78571 .32867 -9 -9 ]
[.78571 .32867 9 0 ]
[.9127 .32867 -9 -9 ]
[.9127 .32867 9 0 ]
[.01131 .0266 -18 -4.5 ]
[.01131 .0266 0 4.5 ]
[.01131 .13145 -18 -4.5 ]
[.01131 .13145 0 4.5 ]
[.01131 .23631 -18 -4.5 ]
[.01131 .23631 0 4.5 ]
[.01131 .44603 -18 -4.5 ]
[.01131 .44603 0 4.5 ]
[.01131 .55089 -18 -4.5 ]
[.01131 .55089 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .34117 m
.15079 .34742 L
s
[(0.2)] .15079 .32867 0 1 Mshowa
.27778 .34117 m
.27778 .34742 L
s
[(0.4)] .27778 .32867 0 1 Mshowa
.40476 .34117 m
.40476 .34742 L
s
[(0.6)] .40476 .32867 0 1 Mshowa
.53175 .34117 m
.53175 .34742 L
s
[(0.8)] .53175 .32867 0 1 Mshowa
.65873 .34117 m
.65873 .34742 L
s
[(1)] .65873 .32867 0 1 Mshowa
.78571 .34117 m
.78571 .34742 L
s
[(1.2)] .78571 .32867 0 1 Mshowa
.9127 .34117 m
.9127 .34742 L
s
[(1.4)] .9127 .32867 0 1 Mshowa
.125 Mabswid
.05556 .34117 m
.05556 .34492 L
s
.0873 .34117 m
.0873 .34492 L
s
.11905 .34117 m
.11905 .34492 L
s
.18254 .34117 m
.18254 .34492 L
s
.21429 .34117 m
.21429 .34492 L
s
.24603 .34117 m
.24603 .34492 L
s
.30952 .34117 m
.30952 .34492 L
s
.34127 .34117 m
.34127 .34492 L
s
.37302 .34117 m
.37302 .34492 L
s
.43651 .34117 m
.43651 .34492 L
s
.46825 .34117 m
.46825 .34492 L
s
.5 .34117 m
.5 .34492 L
s
.56349 .34117 m
.56349 .34492 L
s
.59524 .34117 m
.59524 .34492 L
s
.62698 .34117 m
.62698 .34492 L
s
.69048 .34117 m
.69048 .34492 L
s
.72222 .34117 m
.72222 .34492 L
s
.75397 .34117 m
.75397 .34492 L
s
.81746 .34117 m
.81746 .34492 L
s
.84921 .34117 m
.84921 .34492 L
s
.88095 .34117 m
.88095 .34492 L
s
.94444 .34117 m
.94444 .34492 L
s
.97619 .34117 m
.97619 .34492 L
s
.25 Mabswid
0 .34117 m
1 .34117 L
s
.02381 .0266 m
.03006 .0266 L
s
[(0.4)] .01131 .0266 1 0 Mshowa
.02381 .13145 m
.03006 .13145 L
s
[(0.6)] .01131 .13145 1 0 Mshowa
.02381 .23631 m
.03006 .23631 L
s
[(0.8)] .01131 .23631 1 0 Mshowa
.02381 .44603 m
.03006 .44603 L
s
[(1.2)] .01131 .44603 1 0 Mshowa
.02381 .55089 m
.03006 .55089 L
s
[(1.4)] .01131 .55089 1 0 Mshowa
.125 Mabswid
.02381 .05281 m
.02756 .05281 L
s
.02381 .07902 m
.02756 .07902 L
s
.02381 .10524 m
.02756 .10524 L
s
.02381 .15767 m
.02756 .15767 L
s
.02381 .18388 m
.02756 .18388 L
s
.02381 .2101 m
.02756 .2101 L
s
.02381 .26253 m
.02756 .26253 L
s
.02381 .28874 m
.02756 .28874 L
s
.02381 .31496 m
.02756 .31496 L
s
.02381 .36739 m
.02756 .36739 L
s
.02381 .3936 m
.02756 .3936 L
s
.02381 .41982 m
.02756 .41982 L
s
.02381 .47225 m
.02756 .47225 L
s
.02381 .49846 m
.02756 .49846 L
s
.02381 .52467 m
.02756 .52467 L
s
.02381 .00038 m
.02756 .00038 L
s
.02381 .5771 m
.02756 .5771 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.03016 .21049 m
.03249 .2105 L
.03505 .21051 L
.0402 .21055 L
.04588 .2106 L
.05116 .21067 L
.0613 .21083 L
.07207 .21107 L
.08245 .21134 L
.09244 .21166 L
.10307 .21206 L
.1133 .21249 L
.12314 .21296 L
.13361 .21352 L
.14369 .21411 L
.15441 .21479 L
.16473 .21551 L
.17466 .21626 L
.18523 .21711 L
.19541 .21799 L
.20622 .21899 L
.21663 .22002 L
.22666 .22106 L
.23732 .22224 L
.24759 .22344 L
.25746 .22465 L
.26797 .22602 L
.27778 .22735 L
s
1 0 0 r
.03016 .1407 m
.03323 .14078 L
.03608 .14088 L
.03922 .14102 L
.04257 .1412 L
.04942 .14169 L
.05566 .14226 L
.06995 .14402 L
.08347 .14625 L
.10882 .15182 L
.13579 .15958 L
.16176 .16859 L
.18674 .1785 L
.21332 .19018 L
.23891 .20237 L
.26612 .21619 L
.27971 .22338 L
.28633 .22695 L
.28785 .22774 L
.28946 .22859 L
.29012 .22887 L
.29083 .22911 L
.2916 .22929 L
.29233 .22943 L
.29535 .22989 L
.29861 .23038 L
.30453 .23131 L
.31754 .23341 L
.34484 .23823 L
.37113 .24342 L
.39643 .24896 L
.42335 .2555 L
.44926 .26248 L
.47418 .26989 L
.50071 .27862 L
.52625 .28793 L
.5534 .29892 L
.57955 .31068 L
.6047 .32318 L
.63147 .33785 L
.65724 .35336 L
.65873 .3543 L
s
0 g
.24557 0 m
.26647 .01726 L
.30764 .05126 L
.34731 .08401 L
.38546 .11552 L
.42606 .14904 L
.46515 .18132 L
.50273 .21235 L
.54275 .2454 L
.58126 .2772 L
.62221 .31102 L
.66166 .34359 L
.69959 .37492 L
.73997 .40825 L
.77884 .44035 L
.82014 .47446 L
.85994 .50732 L
.89823 .53895 L
.93896 .57258 L
.97619 .60332 L
s
.03016 .2106 m
.03133 .21065 L
.03239 .2107 L
.03362 .21051 L
.03477 .21051 L
.03585 .21091 L
.03684 .21098 L
.03908 .21116 L
.04372 .21163 L
.04884 .21227 L
.0584 .21382 L
.06854 .21589 L
.11039 .22747 L
.15041 .24049 L
.18892 .25309 L
.22987 .26588 L
.26932 .2774 L
.30725 .28767 L
.34763 .29773 L
.3865 .30656 L
.4278 .31501 L
.4676 .3222 L
.5059 .32819 L
.54663 .33345 L
.56691 .3356 L
.58585 .3373 L
.60522 .33872 L
.62357 .33976 L
.63325 .34018 L
.64379 .34055 L
.64843 .34068 L
.65336 .3408 L
.65767 .34088 L
.66222 .34095 L
.66454 .34098 L
.66707 .34101 L
.66941 .34103 L
.67153 .34105 L
.67273 .34105 L
.67404 .34106 L
.67476 .34106 L
.67542 .34106 L
.67672 .34107 L
.67784 .34107 L
.67907 .34107 L
.68036 .34107 L
.68156 .34107 L
.68268 .34106 L
.68391 .34106 L
.68613 .34105 L
Mistroke
.6883 .34104 L
.69031 .34102 L
.69487 .34097 L
.69972 .34091 L
.70996 .34072 L
.72103 .34044 L
.74107 .33979 L
.78186 .33804 L
.82113 .33614 L
.8589 .33433 L
.89911 .33256 L
.93781 .33102 L
.97619 .32968 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool2000jool00`00ooooo`3<ool001Goo`03003ooooo03[oo`03
003ooooo0<_oo`005Ooo00<00?ooool0>ooo00<00?ooool0b_oo000Eool00`00ooooo`0lool2003:
ool001Goo`03003ooooo03koo`03003ooooo0<Ooo`000_oo0P001?oo00<00?ooool01?oo00<00?oo
ool00ooo00<00?ooool0?ooo00<00?ooool0a_oo00001Ooo003ooooo00002_oo00<00?ooool00ooo
00<00?ooool0@?oo00<00?ooool0aOoo00001Ooo003ooooo00001ooo1@001?oo0`00@Ooo0P00aOoo
00001Ooo003ooooo00001ooo00@00?ooool000Goo`03003ooooo04?oo`03003ooooo0<;oo`0000Go
o`00ooooo`0000Soo`03003oo`0000Goo`03003ooooo04Coo`03003ooooo0<7oo`0000Goo`00oooo
o`0000Woo`8000Goo`03003ooooo04Goo`03003ooooo0<3oo`000_oo0P002ooo00<00?ooool00ooo
00<00?ooool0A_oo00<00?ooool0_ooo000Eool00`00ooooo`17ool2002oool001Goo`03003ooooo
04Woo`03003ooooo0;coo`005Ooo0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool00`00
ooooo`2jool001Goo`03003ooooo04coo`800;[oo`005Ooo00<00?ooool0C_oo00<00?ooool0]ooo
000Eool00`00ooooo`1?ool00`00ooooo`2fool001Goo`03003ooooo053oo`03003ooooo0;Goo`00
5Ooo00<00?ooool0DOoo00<00?ooool0]?oo000Eool2001Cool2002dool001Goo`03003ooooo05Co
o`03003ooooo0;7oo`005Ooo00<00?ooool0EOoo00<00?ooool0/?oo000Eool00`00ooooo`1Fool0
0`00ooooo`2_ool001Goo`03003ooooo05Ooo`800:ooo`005Ooo00<00?ooool0FOoo00<00?ooool0
[?oo000Eool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`03003ooooo05_oo`03003ooooo0:[o
o`005Ooo0P00GOoo0P00Z_oo000Eool00`00ooooo`1Nool00`00ooooo`2Wool001Goo`03003ooooo
05ooo`03003ooooo0:Koo`005Ooo00<00?ooool0H?oo00<00?ooool0YOoo000Eool00`00ooooo`1Q
ool00`00ooooo`2Tool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo06;oo`03003o
oooo0:?oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1Sool00`00ooooo`2R
ool00005ool00?ooool00008ool01000ooooo`001?oo0`00I?oo0P00X_oo00001Ooo003ooooo0000
2?oo0`001Ooo00<00?ooool0I_oo00<00?ooool0Wooo00001Ooo003ooooo00002?oo00<00?ooool0
1Ooo00<00?oon0000oP0I?oo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo
00<00?ooool00ooo2?P0GOoo00<00?ooool0WOoo0002ool2000:ool30004ool00`00ooooo`0;ool6
n01Hool00`00ooooo`2Lool001Goo`03003ooooo017oo`Gh05Coo`03003ooooo09_oo`005Ooo00<0
0?ooool05_oo1?P0DOoo0P00Vooo000Eool2000Kool3n01@ool00`00ooooo`2Hool001Goo`03003o
oooo01goo`;h04ooo`03003ooooo09Ooo`005Ooo00<00?ooool07ooo0_P0C_oo0P00Uooo000Eool0
0`00ooooo`0Qool3n01=ool00`00ooooo`2Dool001Goo`03003ooooo02Coo`;h04coo`03003ooooo
09?oo`005Ooo00<00?ooool09_oo0_P0Booo0P00Tooo000Eool00`00ooooo`0Xool3n01:ool00`00
ooooo`2@ool001Goo`8002coo`;h04Woo`03003ooooo08ooo`005Ooo00<00?ooool0;Ooo0_P0B?oo
00<00?ooool0S_oo000Eool00`00ooooo`0_ool3n016ool00`00ooooo`2=ool001Goo`03003ooooo
03;oo`;h04Goo`03003ooooo08coo`005Ooo00<00?ooool0=?oo0_P0A?oo0P00S?oo000Eool00`00
ooooo`0fool2n014ool00`00ooooo`29ool001Goo`03003ooooo03Soo`;h04?oo`03003ooooo08So
o`005Ooo1@003@OP:ooo0_P0@_oo00<00?ooool0Qooo000Eool00`00ooooo`02ool60007oolC1n0J
ool2n011ool00`00ooooo`26ool001Goo`03003ooooo00Soo`<001Ooo``7h13oo`;h043oo`03003o
oooo08Goo`005Ooo00<00?ooool02ooo10007ooo2`OP1ooo0_P0?ooo0P00QOoo000Eool00`00oooo
o`0?ool4000Vool81n000ooon03h000oool00`00ooooo`22ool000;oo`8000Coo`03003ooooo00?o
o`8000Goo`03003ooooo01?oo`<002_oo`87h004oooh0?P0n00mool00`00ooooo`21ool00005ool0
0?ooool00008ool01000ooooo`001?oo00<00?ooool05_oo1000;Ooo2?P0=_oo00<00?ooool0P?oo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`<001[oo`@0037oo`Ch03?oo`03003ooooo07oo
o`0000Goo`00ooooo`0000Woo`8000Goo`03003ooooo01koo`<003;oo`Gh02ooo`8007ooo`0000Go
o`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Qool3000dool5n00/ool00`00ooooo`1l
ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool09?oo0P00=ooo1?P0:Ooo00<0
0?ooool0Nooo0002ool2000:ool20005ool00`00ooooo`0Vool3000hool5n00Uool2001kool001Go
o`03003ooooo02Woo`<003[oo`Kh027oo`03003ooooo07Soo`005Ooo00<00?ooool0;?oo1000??oo
1?P07_oo00<00?ooool0Mooo000Eool2000aool4000lool3n00Lool2001gool001Goo`03003ooooo
03Coo`<003coo`?h01_oo`03003ooooo07Coo`005Ooo00<00?ooool0=ooo1000>ooo0oP06Ooo00<0
0?ooool0Looo000Eool00`00ooooo`0kool4000jool2n00Hool2001cool001Goo`03003ooooo03oo
o`<003Woo`?h01Ooo`03003ooooo073oo`005Ooo00<00?ooool0@_oo0`00>Ooo0oP05Ooo00<00?oo
ool0Kooo000Eool00`00ooooo`15ool4000hool4n00Bool00`00ooooo`1^ool001Goo`03003ooooo
04Woo`<003Woo`?h013oo`03003ooooo06goo`005Ooo0P00COoo1000>?oo0_P03ooo00<00?ooool0
K?oo000Eool00`00ooooo`1@ool4000fool2n00>ool2001/ool001Goo`03003ooooo05Coo`@003Co
o`?h00goo`03003ooooo06Woo`005Ooo00<00?ooool06?oo0P001?oo00<00?ooool00_oo10005?oo
0P001?oo00<00?ooool01?oo00<00?ooool03Ooo20001?oo00<00?ooool00ooo0P005?oo0P001?oo
00<00?ooool00ooo0P0000?oooP0n0003?oo00<00?ooool02?oo10006?oo10000ooo00<00?ooool0
0_oo10004ooo10000ooo00<00?ooool01?oo00<00?ooool04?oo000Eool00`00ooooo`0Gool01000
ooooo`002?oo00<00?ooool05?oo00@00?ooool000[oo`03003ooooo01;oo`D000Ooo`04003ooooo
000Bool01000ooooo`002?oo00@00?ooool000;oo`;h00_oo`03003ooooo00Woo`03003ooooo01Wo
o`03003ooooo00Ooo`03003ooooo01Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo00<00?oo
ool05ooo00@00?ooool000Woo`03003ooooo01?oo`04003ooooo0007ool5000Cool01@00ooooo`00
ool010000ooo00@00?ooool001;oo`04003ooooo0008ool01000ooooo`001?oo0_P02_oo00<00?oo
ool02?oo00<00?ooool06Ooo00<00?ooool02?oo00<00?ooool05Ooo00<00?ooool01_oo1@004Ooo
000Eool00`00ooooo`0Gool01000ooooo`002_oo00<00?ooool04_oo00@00?ooool000Ooo`04003o
oooo000Dool01000ooooo`001Ooo1P004ooo00@00?ooool000Woo`8000Ooo`;h00Woo`03003ooooo
00Ooo`03003ooooo01Woo`03003ooooo00Woo`03003ooooo01Coo`03003ooooo00Koo`04003ooooo
000Bool001Goo`8001Soo`04003ooooo000;ool00`00ooooo`0Aool01000ooooo`002?oo00<00?oo
00005?oo00@00?ooool000Soo`D0017oo`04003ooooo0008ool01000ooooo`002?oo0_P02?oo00<0
0?ooool01_oo00<00?ooool06Ooo00<00?ooool02_oo00<00?ooool04ooo00<00?ooool01ooo00<0
0?oo00004_oo000Eool00`00ooooo`0Gool01000ooooo`002?oo00@00?ooool001?oo`04003ooooo
0009ool2000Dool01000ooooo`002?oo00<00?ooool00_oo1P002ooo00@00?ooool000Soo`04003o
oooo000:ool2n007ool20004ool3000Iool30009ool01000ooooo`004ooo0`002_oo0P004_oo000E
ool00`00ooooo`0Hool2000:ool2000Eool2000;ool00`00ooooo`0Cool2000:ool30007ool70005
ool2000:ool2000=ool00oP0ooooo`06ool00`00ooooo`03ool00`00ooooo`0Iool00`00ooooo`08
ool2000Fool00`00ooooo`09ool00`00ooooo`0@ool001Goo`03003ooooo07_oo`P001Woo`;h00Oo
o`03003ooooo063oo`005Ooo00<00?ooool0Pooo1P005Ooo0_P01_oo00<00?ooool0?Ooo6`001ooo
000Eool00`00ooooo`29ool;000<ool2n005ool2000XoolE000Rool001Goo`03003ooooo09Coo`h0
0003n03ooooo00Coo`03003ooooo01;ooa<003Ooo`003_ooo`004`00000Eool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool01000oooooooo0_P00ooo0P001_oo00<0
0?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo000E
ool00`00ooooo`2Wool2n003ool2001Jool001Goo`03003ooooo0:Woo`;h00?oo`03003ooooo05Oo
o`005Ooo00<00?ooool0[ooo00<00?ooool0E_oo000Eool00`00ooooo`2`ool00`00ooooo`1Eool0
01Goo`03003ooooo0;7oo`03003ooooo05Coo`005Ooo0P00/ooo0P00E?oo000Eool00`00ooooo`2d
ool00`00ooooo`1Aool001Goo`03003ooooo0;Goo`03003ooooo053oo`005Ooo00<00?ooool0]_oo
00<00?ooool0Cooo000Eool00`00ooooo`2gool2001?ool001Goo`03003ooooo0;Woo`03003ooooo
04coo`005Ooo00<00?ooool0^_oo00<00?ooool0Booo000Eool2002lool00`00ooooo`1:ool001Go
o`03003ooooo0;coo`03003ooooo04Woo`005Ooo00<00?ooool0_Ooo0P00BOoo000Eool00`00oooo
o`2oool00`00ooooo`16ool001Goo`03003ooooo0<3oo`03003ooooo04Goo`005Ooo00<00?ooool0
`Ooo00<00?ooool0A?oo000Eool00`00ooooo`32ool00`00ooooo`13ool001Goo`800<Coo`03003o
oooo04;oo`005Ooo00<00?ooool0a?oo00<00?ooool0@Ooo000Eool00`00ooooo`35ool20011ool0
01Goo`03003ooooo0<Ooo`03003ooooo03koo`005Ooo00<00?ooool0b?oo00<00?ooool0?Ooo0000
0ooo000000020003ool00`00ooooo`02ool40004ool00`00ooooo`39ool00`00ooooo`0lool000?o
o`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?oo
ool02?oo00<00?ooool01?oo0`00booo00<00?ooool0>_oo0003ool00`00ooooo`09ool00`00oooo
o`03ool00`00ooooo`3<ool2000jool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo
0<koo`03003ooooo03Ooo`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3?ool00`00
ooooo`0fool000?oo`03003ooooo00Soo`8000Goo`03003ooooo0=3oo`8003Koo`005Ooo00<00?oo
ool0d_oo00<00?ooool0<ooo000Eool00`00ooooo`3Cool00`00ooooo`0bool001Goo`800=Goo`80
03;oo`005Ooo00<00?ooool0e_oo00<00?ooool0;ooo000Eool00`00ooooo`3Gool00`00ooooo`0^
ool001Goo`03003ooooo0=Soo`03003ooooo02goo`005Ooo00<00?ooool0fOoo00<00?ooool0;?oo
000Eool00`00ooooo`3Jool00`00ooooo`0[ool001Goo`03003ooooo0=_oo`8002_oo`005Ooo00<0
0?ooool0gOoo00<00?ooool0:?oo000Eool2003Oool00`00ooooo`0Wool001Goo`03003ooooo0=oo
o`03003ooooo02Koo`005Ooo00<00?ooool0h?oo00<00?ooool09Ooo000Eool00`00ooooo`3Qool0
0`00ooooo`0Tool001Goo`03003ooooo0>;oo`8002Coo`005Ooo00<00?ooool0i?oo00<00?ooool0
8Ooo000Eool00`00ooooo`3Uool00`00ooooo`0Pool001Goo`800>Ooo`03003ooooo01ooo`005Ooo
00<00?ooool0iooo00<00?ooool07_oo000Eool00`00ooooo`3Xool2000Nool001Goo`03003ooooo
0>[oo`03003ooooo01_oo`005Ooo00<00?ooool0jooo00<00?ooool06_oo00000ooo000000020003
ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`3/ool00`00ooooo`0Iool000?oo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo0>goo`8001Woo`000ooo00<00?ooool01_oo1@001?oo
0`00kooo00<00?ooool05_oo0003ool00`00ooooo`06ool01000ooooo`001Ooo00<00?ooool0l?oo
00<00?ooool05Ooo0003ool00`00ooooo`07ool00`00ool00005ool00`00ooooo`3aool00`00oooo
o`0Dool00004ool00000000:ool20005ool00`00ooooo`3bool00`00ooooo`0Cool000?oo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo0??oo`8001?oo`005Ooo00<00?ooool0mOoo00<00?oo
ool04?oo000Eool00`00ooooo`3fool00`00ooooo`0?ool001Goo`800?Soo`03003ooooo00koo`00
5Ooo00<00?ooool0n?oo0P003_oo000Eool00`00ooooo`3jool00`00ooooo`0;ool001Goo`03003o
oooo0?_oo`03003ooooo00[oo`005Ooo00<00?ooool0o?oo00<00?ooool02Ooo000Eool00`00oooo
o`3mool20009ool001Goo`03003ooooo0?ooo`03003ooooo00Koo`005Ooo0P00oooo0_oo00<00?oo
ool01Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, 0.318475, \
0.00578885, 0.00701031}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{mGamma /. strangesol[k, 10]}, {k, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.603319 1.21817 [
[.15079 .59082 -9 -9 ]
[.15079 .59082 9 0 ]
[.27778 .59082 -9 -9 ]
[.27778 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.53175 .59082 -9 -9 ]
[.53175 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.9127 .59082 -9 -9 ]
[.9127 .59082 9 0 ]
[.01131 .11605 -24 -4.5 ]
[.01131 .11605 0 4.5 ]
[.01131 .23787 -24 -4.5 ]
[.01131 .23787 0 4.5 ]
[.01131 .35969 -24 -4.5 ]
[.01131 .35969 0 4.5 ]
[.01131 .4815 -24 -4.5 ]
[.01131 .4815 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .60332 m
.15079 .60957 L
s
[(0.2)] .15079 .59082 0 1 Mshowa
.27778 .60332 m
.27778 .60957 L
s
[(0.4)] .27778 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(0.6)] .40476 .59082 0 1 Mshowa
.53175 .60332 m
.53175 .60957 L
s
[(0.8)] .53175 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(1)] .65873 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(1.2)] .78571 .59082 0 1 Mshowa
.9127 .60332 m
.9127 .60957 L
s
[(1.4)] .9127 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.18254 .60332 m
.18254 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.34127 .60332 m
.34127 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.81746 .60332 m
.81746 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.97619 .60332 m
.97619 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .11605 m
.03006 .11605 L
s
[(-0.4)] .01131 .11605 1 0 Mshowa
.02381 .23787 m
.03006 .23787 L
s
[(-0.3)] .01131 .23787 1 0 Mshowa
.02381 .35969 m
.03006 .35969 L
s
[(-0.2)] .01131 .35969 1 0 Mshowa
.02381 .4815 m
.03006 .4815 L
s
[(-0.1)] .01131 .4815 1 0 Mshowa
.125 Mabswid
.02381 .0186 m
.02756 .0186 L
s
.02381 .04296 m
.02756 .04296 L
s
.02381 .06733 m
.02756 .06733 L
s
.02381 .09169 m
.02756 .09169 L
s
.02381 .14042 m
.02756 .14042 L
s
.02381 .16478 m
.02756 .16478 L
s
.02381 .18914 m
.02756 .18914 L
s
.02381 .21351 m
.02756 .21351 L
s
.02381 .26223 m
.02756 .26223 L
s
.02381 .2866 m
.02756 .2866 L
s
.02381 .31096 m
.02756 .31096 L
s
.02381 .33532 m
.02756 .33532 L
s
.02381 .38405 m
.02756 .38405 L
s
.02381 .40841 m
.02756 .40841 L
s
.02381 .43278 m
.02756 .43278 L
s
.02381 .45714 m
.02756 .45714 L
s
.02381 .50587 m
.02756 .50587 L
s
.02381 .53023 m
.02756 .53023 L
s
.02381 .55459 m
.02756 .55459 L
s
.02381 .57896 m
.02756 .57896 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03016 .59094 m
.06854 .51944 L
.11039 .45381 L
.1497 .4043 L
.1875 .36523 L
.22774 .33046 L
.26647 .302 L
.30764 .27595 L
.34731 .25415 L
.38546 .23576 L
.42606 .21858 L
.46515 .20409 L
.50273 .19183 L
.54275 .18033 L
.58126 .17043 L
.62221 .1606 L
.66166 .15098 L
.69959 .14071 L
.73997 .12785 L
.77884 .11321 L
.82014 .09539 L
.85994 .07642 L
.89823 .05691 L
.93896 .0352 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3iool20007ool001_oo`800?Soo`8000Woo`006ooo00<00?ooool0mOoo0P002ooo000K
ool00`00ooooo`3cool2000=ool001_oo`03003ooooo0?7oo`8000ooo`006ooo00<00?ooool0l?oo
00<00?ooool03ooo000Kool00`00ooooo`3^ool2000Bool001_oo`800>goo`8001Coo`006ooo00<0
0?ooool0j_oo0P005_oo000Kool00`00ooooo`3Xool2000Hool001_oo`03003ooooo0>Koo`8001[o
o`006ooo00<00?ooool0i?oo0P007?oo000Kool00`00ooooo`3Rool2000Nool001_oo`03003ooooo
0>3oo`80023oo`006ooo0P00gooo0P008_oo000Kool00`00ooooo`3Lool2000Tool001_oo`03003o
oooo0=[oo`8002Koo`006ooo00<00?ooool0f?oo0P00:?oo000Kool00`00ooooo`3Fool2000Zool0
01_oo`03003ooooo0=Coo`8002coo`006ooo0P00dooo0P00;_oo000Kool00`00ooooo`3@ool2000`
ool001_oo`03003ooooo0<koo`8003;oo`006ooo00<00?ooool0c?oo0P00=?oo000Kool00`00oooo
o`3:ool2000fool000Soo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo0<Soo`80
03Soo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo0<Goo`<003[oo`001ooo00@0
0?ooool000Ooo`D000Coo`800<?oo`<003goo`0000?oo`0000000P000_oo00@00?ooool000Ooo`04
003ooooo0005ool00`00ooooo`30ool20010ool000Ooo`04003ooooo0008ool00`00ool00005ool0
0`00ooooo`2mool30012ool000Ooo`04003ooooo0009ool20005ool00`00ooooo`2jool30015ool0
00Soo`8000_oo`03003ooooo00?oo`03003ooooo0;Koo`@004Soo`006ooo00<00?ooool0/ooo0`00
C?oo000Kool2002aool3001?ool001_oo`03003ooooo0:coo`@005;oo`006ooo00<00?ooool0ZOoo
0`00E_oo000Kool00`00ooooo`2Vool3001Iool001_oo`03003ooooo0:;oo`@005coo`006ooo00<0
0?ooool0W_oo1000H?oo000Kool00`00ooooo`2Jool4001Tool001_oo`8009Goo`H006Soo`006ooo
00<00?ooool0T?oo1000K_oo000Kool00`00ooooo`2<ool4001bool001_oo`03003ooooo08Woo`<0
07Koo`006ooo00<00?ooool0Q_oo0`00NOoo000Kool00`00ooooo`22ool4001lool001_oo`8007oo
o`@0083oo`006ooo00<00?ooool0Nooo0`00Q?oo000Kool00`00ooooo`1gool40027ool001_oo`03
003ooooo07Coo`<008_oo`006ooo00<00?ooool0LOoo0`00S_oo000Kool00`00ooooo`1_ool2002A
ool001_oo`03003ooooo06coo`<009?oo`006ooo0P00Jooo0P00U_oo000Kool00`00ooooo`1Wool3
002Hool001_oo`03003ooooo06Goo`8009_oo`006ooo00<00?ooool0Hooo0P00WOoo0008ool20004
ool00`00ooooo`03ool20005ool00`00ooooo`1Qool2002Oool000Ooo`04003ooooo0008ool01000
ooooo`001?oo00<00?ooool0Gooo0P00XOoo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00
GOoo0`00Xooo00000ooo000000020002ool01000ooooo`002Ooo0P001Ooo00<00?ooool0F_oo0P00
Y_oo0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0Eooo0`00Z?oo0007ool01000
ooooo`002?oo00@00?ooool000Coo`03003ooooo05Goo`800:_oo`002?oo0P002_oo0P001Ooo00<0
0?ooool0Dooo0P00[Ooo000Kool00`00ooooo`1Aool2002_ool001_oo`03003ooooo04ooo`800;7o
o`006ooo0P00C_oo0P00/ooo000Kool00`00ooooo`1;ool2002eool001_oo`03003ooooo04Woo`80
0;Ooo`006ooo00<00?ooool0B?oo00<00?ooool0]ooo000Kool00`00ooooo`16ool2002jool001_o
o`03003ooooo04Goo`03003ooooo0;[oo`006ooo0P00A?oo0P00_Ooo000Kool00`00ooooo`11ool2
002oool001_oo`03003ooooo043oo`03003ooooo0;ooo`006ooo00<00?ooool0?_oo0P00`_oo000K
ool00`00ooooo`0mool00`00ooooo`32ool001_oo`03003ooooo03coo`03003ooooo0<?oo`006ooo
00<00?ooool0>_oo0P00a_oo000Kool2000jool00`00ooooo`36ool001_oo`03003ooooo03Soo`03
003ooooo0<Ooo`006ooo00<00?ooool0=ooo00<00?ooool0b?oo000Kool00`00ooooo`0eool2003;
ool001_oo`03003ooooo03Coo`03003ooooo0<_oo`006ooo00<00?ooool0<ooo00<00?ooool0c?oo
000Kool2000cool00`00ooooo`3=ool001_oo`03003ooooo033oo`800=3oo`006ooo00<00?ooool0
;ooo00<00?ooool0d?oo000Kool00`00ooooo`0^ool00`00ooooo`3Aool001_oo`03003ooooo02go
o`03003ooooo0=;oo`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool0;?oo00<00?oo
ool0dooo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0:_oo0P00e_oo0007ool0
1000ooooo`002Ooo00<00?ooool01?oo0P00:_oo00<00?ooool0e_oo00000ooo000000020002ool0
1000ooooo`002_oo00<00?ooool00ooo00<00?ooool0:?oo00<00?ooool0eooo0007ool01000oooo
o`002ooo00<00?ooool00_oo00<00?ooool09ooo00<00?ooool0f?oo0007ool01000ooooo`002?oo
00@00?ooool000Coo`03003ooooo02Koo`03003ooooo0=Woo`002?oo0P002_oo0P001Ooo00<00?oo
ool09Ooo00<00?ooool0f_oo000Kool00`00ooooo`0Tool00`00ooooo`3Kool001_oo`8002Coo`03
003ooooo0=coo`006ooo00<00?ooool08_oo00<00?ooool0gOoo000Kool00`00ooooo`0Qool00`00
ooooo`3Nool001_oo`03003ooooo023oo`03003ooooo0=ooo`006ooo00<00?ooool07ooo00<00?oo
ool0h?oo000Kool00`00ooooo`0Nool00`00ooooo`3Qool001_oo`8001koo`03003ooooo0>;oo`00
6ooo00<00?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Lool00`00ooooo`3Sool001_o
o`03003ooooo01_oo`03003ooooo0>Coo`006ooo00<00?ooool06_oo00<00?ooool0iOoo000Kool0
0`00ooooo`0Jool00`00ooooo`3Uool001_oo`03003ooooo01Woo`03003ooooo0>Koo`006ooo0P00
6Ooo00<00?ooool0iooo000Kool00`00ooooo`0Hool00`00ooooo`3Wool001_oo`03003ooooo01Oo
o`03003ooooo0>Soo`006ooo00<00?ooool05_oo00<00?ooool0jOoo000Kool00`00ooooo`0Eool0
0`00ooooo`3Zool001_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo0P005Ooo00<00?ooool0
jooo000Kool00`00ooooo`0Cool00`00ooooo`3/ool001_oo`03003ooooo01?oo`03003ooooo0>co
o`006ooo00<00?ooool04_oo00<00?ooool0kOoo000Kool00`00ooooo`0Aool00`00ooooo`3^ool0
00Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo017oo`03003ooooo0>koo`001ooo00@0
0?ooool000[oo`03003ooooo00?oo`03003ooooo013oo`03003ooooo0>ooo`001ooo00@00?ooool0
00[oo`03003ooooo00?oo`80013oo`03003ooooo0?3oo`0000?oo`0000000P000_oo00@00?ooool0
00[oo`03003ooooo00?oo`03003ooooo00ooo`03003ooooo0?3oo`001ooo00@00?ooool000[oo`03
003ooooo00?oo`03003ooooo00koo`03003ooooo0?7oo`001ooo00@00?ooool000Soo`<000Goo`03
003ooooo00koo`03003ooooo0?7oo`002?oo0P002ooo00<00?ooool00ooo00<00?ooool03Ooo00<0
0?ooool0l_oo000Kool00`00ooooo`0<ool00`00ooooo`3cool001_oo`8000goo`03003ooooo0??o
o`006ooo00<00?ooool02ooo00<00?ooool0m?oo000Kool00`00ooooo`0:ool00`00ooooo`3eool0
01_oo`03003ooooo00[oo`03003ooooo0?Goo`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000K
ool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo
0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo
00Koo`03003ooooo0?Woo`006ooo00<00?ooool01_oo00<00?ooool0nOoo000Kool00`00ooooo`05
ool00`00ooooo`3jool001_oo`03003ooooo00Goo`03003ooooo0?[oo`006ooo0P001Ooo00<00?oo
ool0nooo000Kool00`00ooooo`04ool00`00ooooo`3kool001_oo`03003ooooo00?oo`03003ooooo
0?coo`006ooo00<00?ooool00ooo00<00?ooool04_oo0P001?oo00<00?ooool00_oo10004_oo0P00
1?oo00<00?ooool01?oo00<00?ooool04_oo0P001?oo00<00?ooool00ooo0P005?oo0P001?oo00<0
0?ooool00ooo0P006Ooo10005ooo10000ooo00<00?ooool00_oo10004_oo10000ooo00<00?ooool0
1?oo00<00?ooool04?oo000Kool00`00ooooo`02ool00`00ooooo`0Bool01000ooooo`002?oo00<0
0?ooool04_oo00@00?ooool000[oo`03003ooooo017oo`04003ooooo0008ool01000ooooo`004_oo
00@00?ooool000Soo`04003ooooo000Jool00`00ooooo`0Hool00`00ooooo`07ool00`00ooooo`0E
ool00`00ooooo`09ool00`00ooooo`0@ool001_oo`03003ooooo00;oo`03003ooooo01;oo`04003o
oooo0009ool00`00ooooo`0Aool01000ooooo`001ooo1@004_oo00@00?ooool000Soo`04003ooooo
000Bool01000ooooo`002?oo00@00?ooool001[oo`03003ooooo01Soo`03003ooooo00Soo`03003o
oooo01Coo`03003ooooo00Koo`D0017oo`006ooo00D00?ooooooo`0001Goo`04003ooooo000:ool0
0`00ooooo`0@ool01000ooooo`001ooo00@00?ooool001?oo`04003ooooo0008ool3000Cool01000
ooooo`002Ooo0P006ooo00<00?ooool06?oo00<00?ooool02Ooo00<00?ooool04ooo00<00?ooool0
1_oo00@00?ooool001;oo`006ooo0P000_oo00<00?ooool04ooo00@00?ooool000_oo`03003ooooo
00ooo`04003ooooo0008ool00`00ool0000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?oo
ool000Soo`04003ooooo000Jool00`00ooooo`0Hool00`00ooooo`0:ool00`00ooooo`0Bool00`00
ooooo`07ool00`00ool0000Bool001_oo`04003ooooo000Fool01000ooooo`002?oo00@00?ooool0
017oo`04003ooooo0009ool2000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool000So
o`04003ooooo000Hool3000Hool30009ool01000ooooo`004_oo0`002_oo0P004_oo000Kool01000
ooooo`005ooo0P002_oo0P004ooo0P002ooo00<00?ooool04_oo0P002_oo0`004ooo0P002_oo0P00
6ooo00<00?ooool06?oo00<00?ooool02?oo0P005Ooo00<00?ooool02Ooo00<00?ooool04?oo000K
ool00`00ool0003oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000D
oooo000=000001_oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Goo`006ooo00<00?ooool07ooo00<00?ooool07_oo00<00?oo
ool07ooo00<00?ooool07ooo00<00?ooool07ooo00<00?ooool07_oo00<00?ooool07ooo00<00?oo
ool05_oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, -0.514613, \
0.00592092, 0.00308603}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9994820046162144`, 
      mGamma \[Rule] \(-0.3707184166384133`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.001, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.7507348023467588`, 
      mGamma \[Rule] \(-0.0010167970670121491`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.001, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.7507331914045487`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[0.7507331914045487`, 0.001, 10]\)], "Input"],

Cell[BoxData[
    \(2.0805037335362946`*^-9 + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[
      0.7507352528464437476 + I \((\(-0.001016796662459896262083\))\), 0.001, 
      10]\)], "Input"],

Cell[BoxData[
    \(9.931444982616892`*^-22 + 
      7.905253850325371`*^-20\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["xi=10", "Subsubsection"],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.7, 
          0.8}, {mGamma, \(-0.3\), \(-0.4\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.01, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9996526951384059`, 
      mGamma \[Rule] \(-0.37204105854191877`\)}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.8, 
          0.9}, {mGamma, \(-0.6\), \(-0.7\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[3, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9575911566412707`, 
      mGamma \[Rule] \(-0.949362118919148`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.99, 10]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \({Om \[Rule] 1.0036572331873266`, 
      mGamma \[Rule] \(-0.36960490117285555`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[1.0036572331873266 - I\ 0.36960490117285555`, 0.99, 
      10]\)], "Input"],

Cell[BoxData[
    \(\(-0.83233351413502`\) - 0.3093315925468989`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol2[0.99, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] \(\(1.0174701121329586`\)\(\[InvisibleSpace]\)\) - 
          1.05916612036627`*^-10\ \[ImaginaryI]}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(realsol2[k_, a_] := 
      FindRoot[kx0corrtestfunc[Om, k, a] \[Equal] 0, {Om, 1.4, 1.5}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol2[0.99, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 1.2303860356428027`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[1.0174701121329586`\[InvisibleSpace], 0.99, 
      10]\)], "Input"],

Cell[BoxData[
    \(2.0011741425513902`*^-10 + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, Om /. strangesol[k, 10]}, {k, 0.01, 1.5}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -0.240587 0.562604 [
[.15079 .30952 -9 -9 ]
[.15079 .30952 9 0 ]
[.27778 .30952 -9 -9 ]
[.27778 .30952 9 0 ]
[.40476 .30952 -9 -9 ]
[.40476 .30952 9 0 ]
[.53175 .30952 -9 -9 ]
[.53175 .30952 9 0 ]
[.65873 .30952 -3 -9 ]
[.65873 .30952 3 0 ]
[.78571 .30952 -9 -9 ]
[.78571 .30952 9 0 ]
[.9127 .30952 -9 -9 ]
[.9127 .30952 9 0 ]
[.01131 .09698 -18 -4.5 ]
[.01131 .09698 0 4.5 ]
[.01131 .2095 -18 -4.5 ]
[.01131 .2095 0 4.5 ]
[.01131 .43454 -18 -4.5 ]
[.01131 .43454 0 4.5 ]
[.01131 .54706 -18 -4.5 ]
[.01131 .54706 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .32202 m
.15079 .32827 L
s
[(0.2)] .15079 .30952 0 1 Mshowa
.27778 .32202 m
.27778 .32827 L
s
[(0.4)] .27778 .30952 0 1 Mshowa
.40476 .32202 m
.40476 .32827 L
s
[(0.6)] .40476 .30952 0 1 Mshowa
.53175 .32202 m
.53175 .32827 L
s
[(0.8)] .53175 .30952 0 1 Mshowa
.65873 .32202 m
.65873 .32827 L
s
[(1)] .65873 .30952 0 1 Mshowa
.78571 .32202 m
.78571 .32827 L
s
[(1.2)] .78571 .30952 0 1 Mshowa
.9127 .32202 m
.9127 .32827 L
s
[(1.4)] .9127 .30952 0 1 Mshowa
.125 Mabswid
.05556 .32202 m
.05556 .32577 L
s
.0873 .32202 m
.0873 .32577 L
s
.11905 .32202 m
.11905 .32577 L
s
.18254 .32202 m
.18254 .32577 L
s
.21429 .32202 m
.21429 .32577 L
s
.24603 .32202 m
.24603 .32577 L
s
.30952 .32202 m
.30952 .32577 L
s
.34127 .32202 m
.34127 .32577 L
s
.37302 .32202 m
.37302 .32577 L
s
.43651 .32202 m
.43651 .32577 L
s
.46825 .32202 m
.46825 .32577 L
s
.5 .32202 m
.5 .32577 L
s
.56349 .32202 m
.56349 .32577 L
s
.59524 .32202 m
.59524 .32577 L
s
.62698 .32202 m
.62698 .32577 L
s
.69048 .32202 m
.69048 .32577 L
s
.72222 .32202 m
.72222 .32577 L
s
.75397 .32202 m
.75397 .32577 L
s
.81746 .32202 m
.81746 .32577 L
s
.84921 .32202 m
.84921 .32577 L
s
.88095 .32202 m
.88095 .32577 L
s
.94444 .32202 m
.94444 .32577 L
s
.97619 .32202 m
.97619 .32577 L
s
.25 Mabswid
0 .32202 m
1 .32202 L
s
.02381 .09698 m
.03006 .09698 L
s
[(0.6)] .01131 .09698 1 0 Mshowa
.02381 .2095 m
.03006 .2095 L
s
[(0.8)] .01131 .2095 1 0 Mshowa
.02381 .43454 m
.03006 .43454 L
s
[(1.2)] .01131 .43454 1 0 Mshowa
.02381 .54706 m
.03006 .54706 L
s
[(1.4)] .01131 .54706 1 0 Mshowa
.125 Mabswid
.02381 .12511 m
.02756 .12511 L
s
.02381 .15324 m
.02756 .15324 L
s
.02381 .18137 m
.02756 .18137 L
s
.02381 .23763 m
.02756 .23763 L
s
.02381 .26576 m
.02756 .26576 L
s
.02381 .29389 m
.02756 .29389 L
s
.02381 .35015 m
.02756 .35015 L
s
.02381 .37828 m
.02756 .37828 L
s
.02381 .40641 m
.02756 .40641 L
s
.02381 .46267 m
.02756 .46267 L
s
.02381 .4908 m
.02756 .4908 L
s
.02381 .51893 m
.02756 .51893 L
s
.02381 .06884 m
.02756 .06884 L
s
.02381 .04071 m
.02756 .04071 L
s
.02381 .01258 m
.02756 .01258 L
s
.02381 .57519 m
.02756 .57519 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.29532 0 m
.30764 .01092 L
.34731 .04606 L
.38546 .07988 L
.42606 .11585 L
.46515 .15048 L
.50273 .18378 L
.54275 .21925 L
.58126 .25337 L
.62221 .28966 L
.66166 .32461 L
.69959 .35823 L
.73997 .394 L
.77884 .42844 L
.82014 .46504 L
.85994 .50031 L
.89823 .53424 L
.93896 .57033 L
.97619 .60332 L
s
.03016 .18179 m
.03239 .1818 L
.03477 .18181 L
.03908 .18184 L
.04419 .18189 L
.04884 .18195 L
.05897 .18211 L
.06854 .18232 L
.08861 .18291 L
.11039 .1838 L
.1301 .18483 L
.15098 .18616 L
.18857 .18919 L
.22859 .19335 L
.2671 .19832 L
.27714 .19978 L
.28244 .20058 L
.28535 .20102 L
.28678 .20125 L
.28809 .20145 L
.28936 .20169 L
.29 .20167 L
.2907 .20204 L
.29352 .20365 L
.29844 .20657 L
.30805 .21232 L
.3482 .23712 L
.38685 .26208 L
.42794 .28957 L
.46752 .31683 L
.50559 .34368 L
.51011 .3469 L
.51238 .34852 L
.51366 .34944 L
.51486 .26031 L
.51554 .26058 L
.51618 .26083 L
.51738 .26131 L
.52007 .26238 L
.52485 .26432 L
.52618 .26486 L
.5269 .26516 L
.52759 .26544 L
.52879 .3603 L
.52942 .36076 L
.53011 .36125 L
.53581 .36537 L
.5461 .37283 L
.55656 .38044 L
.56171 .3842 L
Mistroke
.56421 .38603 L
.56648 .38769 L
.56755 .38848 L
.56871 .38932 L
.5698 .24322 L
.5708 .24411 L
.57304 .24608 L
.57416 .24709 L
.57539 .26489 L
.57666 .28042 L
.57782 .25806 L
.57846 .25096 L
.57916 .26015 L
.58042 .25289 L
.5816 .25766 L
.58286 .2548 L
.58393 .25576 L
.58511 .31911 L
.58632 .26558 L
.58745 .25886 L
.58869 .25995 L
.58938 .26246 L
.59001 .40501 L
.59128 .40595 L
.59266 .40697 L
.59396 .40793 L
.59516 .27827 L
.59626 .26977 L
.59729 .26758 L
.59842 .26867 L
.59963 .27002 L
.60436 .27384 L
.62509 .29222 L
.63462 .30087 L
.63943 .30492 L
.64008 .30565 L
.64075 .30738 L
.64149 .30675 L
.64217 .30735 L
.6434 .30847 L
.6441 .30959 L
.64475 .32597 L
.64544 .31026 L
.6462 .3257 L
.64691 .3256 L
.64756 .32554 L
.6488 .31322 L
.64943 .31414 L
.65012 .32563 L
.65087 .31505 L
Mistroke
.65155 .3198 L
.65234 .31659 L
.65308 .31701 L
.65377 .31807 L
.65441 .31819 L
.65511 .32362 L
.65586 .31949 L
.65656 .32236 L
.6572 .32066 L
.6579 .32191 L
.65864 .32172 L
.65989 .32175 L
.66126 .32177 L
.66615 .32183 L
.6688 .32186 L
.67131 .32188 L
.67357 .32189 L
.67485 .3219 L
.67602 .3219 L
.67728 .3219 L
.67846 .32191 L
.67976 .32191 L
.68049 .3219 L
.68116 .3219 L
.68242 .3219 L
.68314 .3219 L
.68381 .3219 L
.68661 .32188 L
.68909 .32186 L
.69144 .32184 L
.69584 .32179 L
.70057 .32172 L
.7057 .32162 L
.71511 .3214 L
.72545 .3211 L
.74636 .32032 L
.76569 .31945 L
.77126 .31918 L
.77654 .31892 L
.78128 .31868 L
.78258 .31862 L
.78397 .31855 L
.78527 .31848 L
.78645 .55476 L
.78765 .31836 L
.78877 .3183 L
.79132 .31817 L
.79644 .31791 L
.80562 .31743 L
.82632 .31634 L
Mistroke
.86504 .31438 L
.9062 .31246 L
.94585 .31081 L
.97619 .30968 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`17ool00`00ooooo`2nool001Goo`03003ooooo
04Soo`03003ooooo0;goo`005Ooo00<00?ooool0BOoo00<00?ooool0_?oo000Eool2001;ool00`00
ooooo`2kool001Goo`03003ooooo04_oo`03003ooooo0;[oo`005Ooo00<00?ooool0C?oo0P00^_oo
000Eool00`00ooooo`1>ool00`00ooooo`2gool001Goo`03003ooooo04ooo`03003ooooo0;Koo`00
5Ooo00<00?ooool0D?oo00<00?ooool0]Ooo000Eool00`00ooooo`1Aool00`00ooooo`2dool001Go
o`03003ooooo05;oo`800;Coo`005Ooo0P00EOoo00<00?ooool0/Ooo000Eool00`00ooooo`1Eool0
0`00ooooo`2`ool001Goo`03003ooooo05Koo`03003ooooo0:ooo`005Ooo00<00?ooool0Eooo00<0
0?ooool0[_oo000Eool00`00ooooo`1Hool00`00ooooo`2]ool001Goo`03003ooooo05Woo`03003o
oooo0:coo`005Ooo00<00?ooool0F_oo00<00?ooool0Zooo000Eool00`00ooooo`1Kool00`00oooo
o`2Zool001Goo`8005goo`03003ooooo0:Woo`005Ooo00<00?ooool0GOoo00<00?ooool0Z?oo000E
ool00`00ooooo`1Nool00`00ooooo`2Wool001Goo`03003ooooo05ooo`03003ooooo0:Koo`005Ooo
00<00?ooool0H?oo00<00?ooool0YOoo0002ool20004ool00`00ooooo`03ool20005ool00`00oooo
o`1Qool2002Uool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0Hooo00<00?oo
ool0X_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`<006Coo`03003ooooo0:7oo`0000Go
o`00ooooo`0000Soo`<000Goo`03003ooooo06Goo`03003ooooo0:3oo`0000Goo`00ooooo`0000So
o`03003ooooo00Goo`03003ooooo06Koo`03003ooooo09ooo`0000Goo`00ooooo`0000Soo`03003o
oooo00Goo`03003ooooo06Ooo`8009ooo`000_oo0P002_oo0`001?oo00<00?ooool0JOoo00<00?oo
ool0W?oo000Eool00`00ooooo`1Zool00`00ooooo`2Kool001Goo`03003ooooo06_oo`03003ooooo
09[oo`005Ooo00<00?ooool0K?oo00<00?ooool0VOoo000Eool2001^ool00`00ooooo`2Hool001Go
o`03003ooooo06koo`03003ooooo09Ooo`005Ooo00<00?ooool0Kooo0P00Uooo000Eool00`00oooo
o`1aool00`00ooooo`2Dool001Goo`03003ooooo07;oo`03003ooooo09?oo`005Ooo00<00?ooool0
Looo00<00?ooool0T_oo000Eool00`00ooooo`1dool00`00ooooo`2Aool001Goo`03003ooooo07Go
o`03003ooooo093oo`005Ooo0P00Mooo00<00?ooool0Sooo000Eool00`00ooooo`1gool00`00oooo
o`2>ool001Goo`03003ooooo07Soo`03003ooooo08goo`005Ooo00<00?ooool0NOoo00<00?ooool0
S?oo000Eool00`00ooooo`1jool2002<ool001Goo`03003ooooo07coo`03003ooooo08Woo`005Ooo
00<00?ooool0OOoo00<00?ooool0R?oo000Eool8001iool00`00ooooo`27ool001Goo`03003ooooo
00GooaP006;oo`03003ooooo08Koo`005Ooo00<00?ooool07Ooo4000Dooo00<00?ooool0QOoo000E
ool00`00ooooo`0]ool6001>ool00`00ooooo`24ool001Goo`03003ooooo03?oo`L004Soo`03003o
oooo08?oo`005Ooo00<00?ooool0>_oo2@00@?oo00<00?ooool0P_oo0002ool20004ool00`00oooo
o`03ool20005ool00`00ooooo`13ool4000mool20022ool00005ool00?ooool00008ool01000oooo
o`001?oo00<00?ooool0Aooo0P00?Ooo00<00?ooool0Oooo00001Ooo003ooooo00002?oo00@00?oo
ool000Coo`<004Woo`03003ooooo03_oo`03003ooooo07koo`0000Goo`00ooooo`0000Woo`8000Go
o`03003ooooo04[oo`03003ooooo03_oo`03003ooooo07goo`0000Goo`00ooooo`0000Soo`04003o
oooo0004ool00`00ooooo`1;ool2000lool00`00ooooo`1lool00005ool00?ooool00008ool01000
ooooo`001?oo00<00?ooool0COoo0P00>ooo00<00?ooool0Nooo0002ool2000:ool20005ool00`00
ooooo`1?ool2000jool00`00ooooo`1jool001Goo`03003ooooo057oo`8003Woo`8007[oo`005Ooo
00<00?ooool0Dooo0P00>Ooo00<00?ooool0Mooo000Eool00`00ooooo`1Eool00`00ooooo`0gool0
0`00ooooo`1fool001Goo`8005Ooo`8003Soo`03003ooooo07Goo`005Ooo00<00?ooool0F?oo00<0
0?ooool0=_oo0P00MOoo000Eool00`00ooooo`1Iool2000fool3001dool001Goo`03003ooooo05_o
o`8003Coo`04003oo`00001cool001Goo`03003ooooo05goo`03003ooooo037oo`03003oo`000080
07;oo`005Ooo00<00?ooool0G_oo0P00<Ooo00<00?oo00000`00LOoo000Eool00`00ooooo`1Pool0
0`00ooooo`0Pool2000<ool00`00ool00004001`ool001Goo`8006;oo`80023oo`04003oo`00000:
ool00`00ool00005001_ool001Goo`03003ooooo06?oo`03003ooooo01goo`04003ooooo000:ool0
0`00ool0000400000ooo0000001]ool001Goo`03003ooooo06Coo`03003ooooo01coo`04003ooooo
000:ool01000ool00?oo0`0000Coo`00000006coo`005Ooo00<00?ooool0IOoo0P007?oo00@00?oo
ool000[oo`04003oo`00ool40003ool00`00ooooo`1Yool001Goo`03003ooooo06Ooo`03003ooooo
01Woo`04003ooooo000:ool01000ool00?oo10001?oo00<00?ooool0J?oo000Eool00`00ooooo`0H
ool20004ool00`00ooooo`02ool4000Dool20004ool00`00ooooo`04ool00`00ooooo`0Cool20004
ool01P00oooooooo003oo`8001Coo`800005ool00?ooool00005ool20003ool01000oooooooo1000
1Ooo00<00?ooool01ooo10006?oo10000ooo00<00?ooool00_oo10004ooo10000ooo00<00?ooool0
1?oo00<00?ooool04?oo000Eool00`00ooooo`0Gool01000ooooo`002?oo00<00?ooool05?oo00@0
0?ooool000[oo`03003ooooo01;oo`04003ooooo0008ool200000ooo003oo`0Aool00`00ooooo`02
0002ool00`00ooooo`02ool01000ooooo`000_oo00@00?ooooooo`@000Koo`03003ooooo00Soo`03
003ooooo01Woo`03003ooooo00Ooo`03003ooooo01Koo`03003ooooo00Woo`03003ooooo013oo`00
5Ooo00<00?ooool05ooo00@00?ooool000Woo`03003ooooo01?oo`04003ooooo0007ool5000Cool0
1000ooooo`002?oo00@00?oo000001;oo`03003ooooo008000;oo`03003ooooo00;oo`04003ooooo
0002ool00`00ooooo`02ool30007ool00`00ooooo`07ool00`00ooooo`0Iool00`00ooooo`08ool0
0`00ooooo`0Eool00`00ooooo`06ool5000Aool001Goo`8001Soo`04003ooooo000:ool00`00oooo
o`0Bool01000ooooo`001ooo00@00?ooool001Coo`04003ooooo0008ool5000Aool00`00ooooo`02
0002ool00`00ooooo`03ool20003ool00`00ooooo`02ool30008ool00`00ooooo`06ool00`00oooo
o`0Iool00`00ooooo`09ool00`00ooooo`0Dool00`00ooooo`06ool01000ooooo`004_oo000Eool0
0`00ooooo`0Gool01000ooooo`002ooo00<00?ooool04Ooo00@00?ooool000Soo`03003oo`0001Co
o`04003ooooo0008ool00`00ooooo`02ool00`00ooooo`0>ool00`00ooooo`020002ool00`00oooo
o`02ool01000ooooo`000_oo00<00?ooool00_oo0`002Ooo00<00?ooool01Ooo00<00?ooool06Ooo
00<00?ooool02_oo00<00?ooool04ooo00<00?ooool01ooo00<00?oo00004_oo000Eool00`00oooo
o`0Gool01000ooooo`002?oo00@00?ooool001?oo`04003ooooo0009ool2000Dool01000ooooo`00
2?oo00<00?ooool00ooo0P003_oo00<00?ooool00P000_oo00<00?ooool00_oo00@00?ooool000;o
o`03003ooooo00;oo`<000[oo`03003ooooo00;oo`<001Woo`<000Woo`04003ooooo000Cool3000:
ool2000Bool001Goo`03003ooooo01Soo`8000[oo`8001Goo`8000_oo`03003ooooo01?oo`8000[o
o`<000Coo`8000goo`800005ool00?ooool00005ool20003ool00`00ooooo`02ool3000;ool20004
ool00`00ooooo`0Iool00`00ooooo`08ool2000Fool00`00ooooo`09ool00`00ooooo`0@ool001Go
o`03003ooooo07?oo`03003ooooo00coo`05003oooooool0000:ool00`00ooooo`02ool3000<ool2
001Eool50007ool001Goo`03003ooooo07Coo`8000coo`05003oooooool0000:ool00`00ooooo`02
ool3000=ool2000eoolO000<ool001Goo`03003ooooo07Koo`03003ooooo00Woo`05003oooooool0
000:ool00`00ooooo`02ool3000=ool3000Vool>000[ool001Goo`03003ooooo07Ooo`8000Woo`05
003oooooool0000:ool00`00ooooo`02ool3000=ool5000:oolJ000iool000koool001<000005Ooo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00D00?ooooooo`00
00?oo`05003oooooool00003ool20008ool20005ool20008ool00`00ooooo`02ool20002ool20006
ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool20007ool00`00ooooo`05ool00`00
ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool001Go
o`03003ooooo07_oo`8000Goo`05003oooooool0000:ool00`00ooooo`03ool2000Cool2000Nool2
000jool001Goo`03003ooooo07goo`03003ooooo00;oo`05003oooooool0000:ool00`00ooooo`03
ool2000Eool00`00ooooo`0Kool2000jool001Goo`03003ooooo07koo`8000;oo`05003oooooool0
000:ool00`00ooooo`03ool2000Fool00`00ooooo`0Jool2000jool001Goo`03003ooooo083oo`03
003oo`0000?oo`03003ooooo00Soo`03003ooooo00?oo`8001Ooo`03003ooooo01Woo`8003[oo`00
5Ooo00<00?ooool0POoo0P000ooo00<00?ooool02?oo00<00?ooool00ooo0P006?oo00<00?ooool0
6?oo0P00>_oo000Eool20023ool01@00oooooooo00002_oo00<00?ooool00ooo0P006Ooo0P006?oo
0P00>_oo000Eool00`00ooooo`26ool00`00ooooo`08ool00`00ooooo`03ool2000Kool00`00oooo
o`0Eool2000jool001Goo`03003ooooo08Koo`03003ooooo00Soo`03003ooooo00?oo`8001coo`03
003ooooo01Coo`8003[oo`005Ooo00<00?ooool0Q_oo0P002Ooo00<00?ooool00ooo0P007Ooo00<0
0?ooool04ooo0P00>_oo000Eool00`00ooooo`28ool00`00ooooo`06ool00`00ooooo`03ool2000N
ool00`00ooooo`0Bool2000jool001Goo`03003ooooo08Woo`8000Koo`03003ooooo00?oo`8001oo
o`03003ooooo017oo`8003[oo`005Ooo00<00?ooool0Rooo00<00?ooool00ooo00<00?ooool00ooo
0P008?oo00<00?ooool04?oo0P00>_oo000Eool00`00ooooo`2<ool20003ool00`00ooooo`03ool2
000Qool2000@ool2000jool001Goo`8008ooo`04003ooooo0005ool2000Sool00`00ooooo`0=ool2
000jool001Goo`03003ooooo08ooo`03003oo`0000Goo`8002Coo`03003ooooo00coo`8003[oo`00
5Ooo00<00?ooool0T?oo0P001Ooo0P009Ooo00<00?ooool02ooo0P00>_oo000Eool00`00ooooo`2A
ool00`00ooooo`03ool2000Vool00`00ooooo`0:ool2000jool001Goo`03003ooooo09Ooo`8002Oo
o`03003ooooo00Woo`8003[oo`005Ooo00<00?ooool0Uooo0P00:?oo00<00?ooool02?oo0P00>_oo
000Eool00`00ooooo`2Gool2000Yool00`00ooooo`07ool2000jool001Goo`03003ooooo09Ooo`80
02[oo`03003ooooo00Koo`8003[oo`005Ooo0P00VOoo00<00?ooool0:Ooo00<00?ooool01Ooo0P00
>_oo000Eool00`00ooooo`35ool00`00ooooo`04ool2000jool001Goo`03003ooooo0<Koo`03003o
oooo00?oo`8003[oo`005Ooo00<00?ooool0aooo00<00?ooool00_oo0P00>_oo000Eool00`00oooo
o`38ool01000oooooooo0P00>_oo00000ooo000000020003ool00`00ooooo`02ool40004ool00`00
ooooo`39ool00`00ooooo`02000jool000?oo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo
0<[oo`04003oo`00000jool000?oo`03003ooooo00Soo`03003ooooo00Coo`<00<_oo`<003[oo`00
0ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0c?oo0P00>_oo0003ool00`00ooooo`0:
ool00`00ooooo`02ool00`00ooooo`3<ool00`00ool0000iool00004ool000000009ool01000oooo
o`001?oo00<00?ooool0c?oo00@00?ooool003Soo`000ooo00<00?ooool02?oo0P001Ooo00<00?oo
ool0c?oo00@00?ooooooo`8003Koo`005Ooo00<00?ooool0c?oo00<00?ooool00ooo00<00?ooool0
<ooo000Eool00`00ooooo`3<ool00`00ooooo`04ool00`00ooooo`0bool001Goo`03003ooooo0<co
o`03003ooooo00Goo`8003;oo`005Ooo0P00cOoo00<00?ooool01ooo00<00?ooool0;ooo000Eool0
0`00ooooo`3<ool00`00ooooo`08ool00`00ooooo`0^ool001Goo`03003ooooo0<coo`03003ooooo
00Woo`03003ooooo02goo`005Ooo00<00?ooool0c?oo00<00?ooool02_oo00<00?ooool0;?oo000E
ool00`00ooooo`3<ool00`00ooooo`0;ool00`00ooooo`0[ool001Goo`03003ooooo0<coo`03003o
oooo00coo`03003ooooo02[oo`005Ooo00<00?ooool0c?oo00<00?ooool03Ooo00<00?ooool0:Ooo
000Eool2003=ool00`00ooooo`0>ool00`00ooooo`0Xool001Goo`03003ooooo0<coo`03003ooooo
00ooo`03003ooooo02Ooo`005Ooo00<00?ooool0c?oo00<00?ooool04?oo00<00?ooool09_oo000E
ool00`00ooooo`3<ool00`00ooooo`0Aool00`00ooooo`0Uool001Goo`03003ooooo0<coo`03003o
oooo01;oo`03003ooooo02Coo`005Ooo00<00?ooool0c?oo00<00?ooool04ooo0P009?oo000Eool0
0`00ooooo`3<ool00`00ooooo`0Eool00`00ooooo`0Qool001Goo`03003ooooo0<coo`03003ooooo
01Koo`03003ooooo023oo`005Ooo0P00cOoo00<00?ooool05ooo00<00?ooool07ooo000Eool00`00
ooooo`3<ool00`00ooooo`0Hool00`00ooooo`0Nool001Goo`03003ooooo0<coo`03003ooooo01Wo
o`8001koo`005Ooo00<00?ooool0c?oo00<00?ooool06ooo00<00?ooool06ooo000Eool00`00oooo
o`3<ool00`00ooooo`0Lool00`00ooooo`0Jool001Goo`03003ooooo0<coo`03003ooooo01goo`03
003ooooo01Woo`0000?oo`0000000P000ooo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0
c?oo00<00?ooool07_oo00<00?ooool06?oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00
ooooo`3<ool00`00ooooo`0Oool00`00ooooo`0Gool000?oo`03003ooooo00Koo`D000Coo`<00<co
o`03003ooooo023oo`03003ooooo01Koo`000ooo00<00?ooool01_oo00@00?ooool000Goo`03003o
oooo0<coo`03003ooooo027oo`8001Koo`000ooo00<00?ooool01ooo00<00?oo00001Ooo00<00?oo
ool0c?oo00<00?ooool08ooo00<00?ooool04ooo00001?oo000000002_oo0P001Ooo00<00?ooool0
looo00<00?ooool04_oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3dool00`00
ooooo`0Aool001Goo`03003ooooo0?Goo`03003ooooo013oo`005Ooo00<00?ooool0m_oo00<00?oo
ool03ooo000Eool2003hool00`00ooooo`0>ool001Goo`03003ooooo0?Soo`03003ooooo00goo`00
5Ooo00<00?ooool0nOoo00<00?ooool03?oo000Eool00`00ooooo`3jool00`00ooooo`0;ool001Go
o`03003ooooo0?_oo`8000_oo`005Ooo00<00?ooool0oOoo00<00?ooool02?oo000Eool00`00oooo
o`3nool00`00ooooo`07ool001Goo`03003ooooo0?ooo`03003ooooo00Koo`005Ooo0P00oooo0_oo
00<00?ooool01Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool00?ooob7oo`00oooo8Ooo003ooolQool00?oo
ob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, 0.398928, \
0.00578885, 0.00653295}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, Om /. strangesol[k, 10]}, {k, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -0.183537 0.524571 [
[.15079 .32853 -9 -9 ]
[.15079 .32853 9 0 ]
[.27778 .32853 -9 -9 ]
[.27778 .32853 9 0 ]
[.40476 .32853 -9 -9 ]
[.40476 .32853 9 0 ]
[.53175 .32853 -9 -9 ]
[.53175 .32853 9 0 ]
[.65873 .32853 -3 -9 ]
[.65873 .32853 3 0 ]
[.78571 .32853 -9 -9 ]
[.78571 .32853 9 0 ]
[.9127 .32853 -9 -9 ]
[.9127 .32853 9 0 ]
[.01131 .02629 -18 -4.5 ]
[.01131 .02629 0 4.5 ]
[.01131 .13121 -18 -4.5 ]
[.01131 .13121 0 4.5 ]
[.01131 .23612 -18 -4.5 ]
[.01131 .23612 0 4.5 ]
[.01131 .44595 -18 -4.5 ]
[.01131 .44595 0 4.5 ]
[.01131 .55086 -18 -4.5 ]
[.01131 .55086 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .34103 m
.15079 .34728 L
s
[(0.2)] .15079 .32853 0 1 Mshowa
.27778 .34103 m
.27778 .34728 L
s
[(0.4)] .27778 .32853 0 1 Mshowa
.40476 .34103 m
.40476 .34728 L
s
[(0.6)] .40476 .32853 0 1 Mshowa
.53175 .34103 m
.53175 .34728 L
s
[(0.8)] .53175 .32853 0 1 Mshowa
.65873 .34103 m
.65873 .34728 L
s
[(1)] .65873 .32853 0 1 Mshowa
.78571 .34103 m
.78571 .34728 L
s
[(1.2)] .78571 .32853 0 1 Mshowa
.9127 .34103 m
.9127 .34728 L
s
[(1.4)] .9127 .32853 0 1 Mshowa
.125 Mabswid
.05556 .34103 m
.05556 .34478 L
s
.0873 .34103 m
.0873 .34478 L
s
.11905 .34103 m
.11905 .34478 L
s
.18254 .34103 m
.18254 .34478 L
s
.21429 .34103 m
.21429 .34478 L
s
.24603 .34103 m
.24603 .34478 L
s
.30952 .34103 m
.30952 .34478 L
s
.34127 .34103 m
.34127 .34478 L
s
.37302 .34103 m
.37302 .34478 L
s
.43651 .34103 m
.43651 .34478 L
s
.46825 .34103 m
.46825 .34478 L
s
.5 .34103 m
.5 .34478 L
s
.56349 .34103 m
.56349 .34478 L
s
.59524 .34103 m
.59524 .34478 L
s
.62698 .34103 m
.62698 .34478 L
s
.69048 .34103 m
.69048 .34478 L
s
.72222 .34103 m
.72222 .34478 L
s
.75397 .34103 m
.75397 .34478 L
s
.81746 .34103 m
.81746 .34478 L
s
.84921 .34103 m
.84921 .34478 L
s
.88095 .34103 m
.88095 .34478 L
s
.94444 .34103 m
.94444 .34478 L
s
.97619 .34103 m
.97619 .34478 L
s
.25 Mabswid
0 .34103 m
1 .34103 L
s
.02381 .02629 m
.03006 .02629 L
s
[(0.4)] .01131 .02629 1 0 Mshowa
.02381 .13121 m
.03006 .13121 L
s
[(0.6)] .01131 .13121 1 0 Mshowa
.02381 .23612 m
.03006 .23612 L
s
[(0.8)] .01131 .23612 1 0 Mshowa
.02381 .44595 m
.03006 .44595 L
s
[(1.2)] .01131 .44595 1 0 Mshowa
.02381 .55086 m
.03006 .55086 L
s
[(1.4)] .01131 .55086 1 0 Mshowa
.125 Mabswid
.02381 .05252 m
.02756 .05252 L
s
.02381 .07875 m
.02756 .07875 L
s
.02381 .10498 m
.02756 .10498 L
s
.02381 .15743 m
.02756 .15743 L
s
.02381 .18366 m
.02756 .18366 L
s
.02381 .20989 m
.02756 .20989 L
s
.02381 .26235 m
.02756 .26235 L
s
.02381 .28858 m
.02756 .28858 L
s
.02381 .31481 m
.02756 .31481 L
s
.02381 .36726 m
.02756 .36726 L
s
.02381 .39349 m
.02756 .39349 L
s
.02381 .41972 m
.02756 .41972 L
s
.02381 .47218 m
.02756 .47218 L
s
.02381 .4984 m
.02756 .4984 L
s
.02381 .52463 m
.02756 .52463 L
s
.02381 6e-05 m
.02756 6e-05 L
s
.02381 .57709 m
.02756 .57709 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.24596 0 m
.26647 .01695 L
.30764 .05097 L
.34731 .08374 L
.38546 .11526 L
.42606 .1488 L
.46515 .18109 L
.50273 .21215 L
.54275 .24521 L
.58126 .27703 L
.62221 .31086 L
.66166 .34345 L
.69959 .37479 L
.73997 .40815 L
.77884 .44027 L
.82014 .47439 L
.85994 .50727 L
.89823 .53891 L
.93896 .57256 L
.97619 .60332 L
s
.03016 .21039 m
.03133 .21044 L
.03239 .21049 L
.03362 .2103 L
.03477 .21031 L
.03585 .2107 L
.03684 .21077 L
.03908 .21095 L
.04372 .21142 L
.04884 .21207 L
.0584 .21361 L
.06854 .21568 L
.11039 .22728 L
.15041 .2403 L
.18892 .2529 L
.22987 .2657 L
.26932 .27723 L
.30725 .28751 L
.34763 .29757 L
.3865 .3064 L
.4278 .31485 L
.4676 .32206 L
.5059 .32805 L
.54663 .33331 L
.56691 .33546 L
.58585 .33716 L
.60522 .33858 L
.62357 .33962 L
.63325 .34005 L
.64379 .34041 L
.64843 .34054 L
.65336 .34066 L
.65767 .34074 L
.66222 .34082 L
.66454 .34085 L
.66707 .34087 L
.66941 .34089 L
.67153 .34091 L
.67273 .34091 L
.67404 .34092 L
.67476 .34092 L
.67542 .34092 L
.67672 .34093 L
.67784 .34093 L
.67907 .34093 L
.68036 .34093 L
.68156 .34093 L
.68268 .34092 L
.68391 .34092 L
.68613 .34091 L
Mistroke
.6883 .3409 L
.69031 .34088 L
.69487 .34083 L
.69972 .34077 L
.70996 .34058 L
.72103 .3403 L
.74107 .33965 L
.78186 .3379 L
.82113 .336 L
.8589 .33419 L
.89911 .33241 L
.93781 .33088 L
.97619 .32953 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool2000jool00`00ooooo`3<ool001Goo`03003ooooo03[oo`03
003ooooo0<_oo`005Ooo00<00?ooool0>ooo00<00?ooool0b_oo000Eool00`00ooooo`0lool2003:
ool001Goo`03003ooooo03koo`03003ooooo0<Ooo`000_oo0P001?oo00<00?ooool01?oo00<00?oo
ool00ooo00<00?ooool0?ooo00<00?ooool0a_oo00001Ooo003ooooo00002_oo00<00?ooool00ooo
00<00?ooool0@?oo00<00?ooool0aOoo00001Ooo003ooooo00001ooo1@001?oo0`00@Ooo0P00aOoo
00001Ooo003ooooo00001ooo00@00?ooool000Goo`03003ooooo04?oo`03003ooooo0<;oo`0000Go
o`00ooooo`0000Soo`03003oo`0000Goo`03003ooooo04Coo`03003ooooo0<7oo`0000Goo`00oooo
o`0000Woo`8000Goo`03003ooooo04Goo`03003ooooo0<3oo`000_oo0P002ooo00<00?ooool00ooo
00<00?ooool0A_oo00<00?ooool0_ooo000Eool00`00ooooo`17ool2002oool001Goo`03003ooooo
04Woo`03003ooooo0;coo`005Ooo0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool00`00
ooooo`2jool001Goo`03003ooooo04coo`800;[oo`005Ooo00<00?ooool0C_oo00<00?ooool0]ooo
000Eool00`00ooooo`1?ool00`00ooooo`2fool001Goo`03003ooooo053oo`03003ooooo0;Goo`00
5Ooo00<00?ooool0DOoo00<00?ooool0]?oo000Eool2001Cool2002dool001Goo`03003ooooo05Co
o`03003ooooo0;7oo`005Ooo00<00?ooool0EOoo00<00?ooool0/?oo000Eool00`00ooooo`1Fool0
0`00ooooo`2_ool001Goo`03003ooooo05Ooo`800:ooo`005Ooo00<00?ooool0FOoo00<00?ooool0
[?oo000Eool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`03003ooooo05_oo`03003ooooo0:[o
o`005Ooo0P00GOoo0P00Z_oo000Eool00`00ooooo`1Nool00`00ooooo`2Wool001Goo`03003ooooo
05ooo`03003ooooo0:Koo`005Ooo00<00?ooool0H?oo00<00?ooool0YOoo000Eool00`00ooooo`1Q
ool2002Uool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo06?oo`03003ooooo0:;o
o`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1Tool00`00ooooo`2Qool00005
ool00?ooool00008ool01000ooooo`001?oo0`00IOoo00<00?ooool0X?oo00001Ooo003ooooo0000
2?oo0`001Ooo00<00?ooool0I_oo00<00?ooool0Wooo00001Ooo003ooooo00002?oo00<00?ooool0
1Ooo00<00?ooool0Iooo0P00Wooo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool0
JOoo00<00?ooool0W?oo0002ool2000:ool30004ool00`00ooooo`1Zool00`00ooooo`2Kool001Go
o`03003ooooo06_oo`03003ooooo09[oo`005Ooo00<00?ooool0K?oo0P00V_oo000Eool2001_ool0
0`00ooooo`2Gool001Goo`03003ooooo06ooo`03003ooooo09Koo`005Ooo00<00?ooool0L?oo00<0
0?ooool0UOoo000Eool00`00ooooo`1aool00`00ooooo`2Dool001Goo`03003ooooo07;oo`8009Co
o`005Ooo00<00?ooool0M?oo00<00?ooool0TOoo000Eool00`00ooooo`1eool00`00ooooo`2@ool0
01Goo`8007Ooo`03003ooooo08ooo`005Ooo00<00?ooool0Mooo00<00?ooool0S_oo000Eool00`00
ooooo`1hool00`00ooooo`2=ool001Goo`03003ooooo07Woo`03003ooooo08coo`005Ooo00<00?oo
ool0N_oo0P00S?oo000Eool00`00ooooo`1lool00`00ooooo`29ool001Goo`03003ooooo07goo`03
003ooooo08Soo`005Ooo1P00Nooo00<00?ooool0Qooo000Eool00`00ooooo`03ool5001gool00`00
ooooo`26ool001Goo`03003ooooo00Soo`<007Goo`03003ooooo08Goo`005Ooo00<00?ooool02ooo
1000L_oo0P00QOoo000Eool00`00ooooo`0?ool4001`ool00`00ooooo`22ool000;oo`8000Coo`03
003ooooo00?oo`8000Goo`03003ooooo01?oo`<006koo`03003ooooo087oo`0000Goo`00ooooo`00
00Soo`04003ooooo0004ool00`00ooooo`0Fool4001[ool00`00ooooo`20ool00005ool00?ooool0
0008ool01000ooooo`001?oo0`006_oo1000J?oo00<00?ooool0Oooo00001Ooo003ooooo00002Ooo
0P001Ooo00<00?ooool07_oo0`00I_oo0P00Oooo00001Ooo003ooooo00002?oo00@00?ooool000Co
o`03003ooooo027oo`<006Goo`03003ooooo07coo`0000Goo`00ooooo`0000Soo`04003ooooo0004
ool00`00ooooo`0Tool2001Tool00`00ooooo`1kool000;oo`8000[oo`8000Goo`03003ooooo02Ko
o`<006;oo`8007_oo`005Ooo00<00?ooool0:Ooo0`00HOoo00<00?ooool0N?oo000Eool00`00oooo
o`0/ool4001Nool00`00ooooo`1gool001Goo`80037oo`@005_oo`8007Ooo`005Ooo00<00?ooool0
=?oo0`00F_oo00<00?ooool0M?oo000Eool00`00ooooo`0gool4001Gool00`00ooooo`1cool001Go
o`03003ooooo03_oo`@005Coo`8007?oo`005Ooo00<00?ooool0?ooo0`00Dooo00<00?ooool0L?oo
000Eool00`00ooooo`12ool3001Aool00`00ooooo`1_ool001Goo`03003ooooo04Goo`@004koo`03
003ooooo06koo`005Ooo0P00B_oo0`00C?oo00<00?ooool0KOoo000Eool00`00ooooo`1<ool40019
ool00`00ooooo`1/ool001Goo`03003ooooo053oo`@004Koo`8006coo`005Ooo00<00?ooool0E?oo
1000A?oo00<00?ooool0JOoo000Eool00`00ooooo`0Hool20004ool00`00ooooo`02ool4000Dool2
0004ool00`00ooooo`04ool00`00ooooo`0=ool80004ool00`00ooooo`03ool2000Dool20004ool0
0`00ooooo`03ool2000?ool00`00ooooo`08ool4000Hool40003ool00`00ooooo`02ool4000Cool4
0003ool00`00ooooo`04ool00`00ooooo`0@ool001Goo`03003ooooo01Ooo`04003ooooo0008ool0
0`00ooooo`0Dool01000ooooo`002_oo00<00?ooool04_oo1@001ooo00@00?ooool001;oo`04003o
oooo0008ool01000ooooo`003ooo00<00?ooool02Ooo00<00?ooool06Ooo00<00?ooool01ooo00<0
0?ooool05_oo00<00?ooool02Ooo00<00?ooool04?oo000Eool00`00ooooo`0Gool01000ooooo`00
2Ooo00<00?ooool04ooo00@00?ooool000Ooo`D001?oo`05003ooooo003oo`040003ool01000oooo
o`004_oo00@00?ooool000Soo`04003ooooo000@ool00`00ooooo`08ool00`00ooooo`0Iool00`00
ooooo`08ool00`00ooooo`0Eool00`00ooooo`06ool5000Aool001Goo`03003ooooo01Ooo`04003o
oooo000:ool00`00ooooo`0Bool01000ooooo`001ooo00@00?ooool001Coo`04003ooooo0005ool6
000Cool01000ooooo`002Ooo0P004_oo00<00?ooool01ooo00<00?ooool06Ooo00<00?ooool02Ooo
00<00?ooool05?oo00<00?ooool01_oo00@00?ooool001;oo`005Ooo0P006?oo00@00?ooool000_o
o`03003ooooo017oo`04003ooooo0008ool00`00ool0000Dool01000ooooo`002?oo1@004Ooo00@0
0?ooool000Soo`04003ooooo000Bool00`00ooooo`06ool00`00ooooo`0Iool00`00ooooo`0:ool0
0`00ooooo`0Cool00`00ooooo`07ool00`00ool0000Bool001Goo`03003ooooo01Ooo`04003ooooo
0008ool01000ooooo`004ooo00@00?ooool000Woo`8001Coo`04003ooooo0008ool00`00ooooo`02
ool6000;ool01000ooooo`002?oo00@00?ooool001?oo`8000Coo`<001Woo`<000Woo`04003ooooo
000Cool3000:ool2000Bool001Goo`03003ooooo01Soo`8000[oo`8001Goo`8000_oo`03003ooooo
01?oo`8000[oo`<000Ooo`L000Goo`8000[oo`8001Koo`03003ooooo00?oo`03003ooooo01Woo`03
003ooooo00Soo`8001Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo00<00?ooool0Nooo2000
8_oo00<00?ooool0H?oo000Eool00`00ooooo`23ool6000Mool00`00ooooo`0moolK0007ool001Go
o`03003ooooo08Woo`/001?oo`8002SooaD002;oo`005Ooo00<00?ooool0U?oo4@001?oo00<00?oo
ool04_oo4`00=ooo000>oooo000C000001Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo
00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo
00Goo`03003ooooo00Koo`03003ooooo00Koo`8000Koo`03003ooooo00Koo`03003ooooo00Koo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`005Ooo00<00?ooool0[?oo0P00F_oo000E
ool00`00ooooo`2^ool00`00ooooo`1Gool001Goo`03003ooooo0:ooo`03003ooooo05Koo`005Ooo
00<00?ooool0/?oo00<00?ooool0EOoo000Eool00`00ooooo`2aool00`00ooooo`1Dool001Goo`80
0;?oo`8005Coo`005Ooo00<00?ooool0]?oo00<00?ooool0DOoo000Eool00`00ooooo`2eool00`00
ooooo`1@ool001Goo`03003ooooo0;Koo`03003ooooo04ooo`005Ooo00<00?ooool0]ooo0P00Cooo
000Eool00`00ooooo`2iool00`00ooooo`1<ool001Goo`03003ooooo0;[oo`03003ooooo04_oo`00
5Ooo0P00_?oo00<00?ooool0B_oo000Eool00`00ooooo`2lool00`00ooooo`19ool001Goo`03003o
oooo0;goo`8004Woo`005Ooo00<00?ooool0_ooo00<00?ooool0A_oo000Eool00`00ooooo`30ool0
0`00ooooo`15ool001Goo`03003ooooo0<7oo`03003ooooo04Coo`005Ooo00<00?ooool0`_oo00<0
0?ooool0@ooo000Eool20034ool00`00ooooo`12ool001Goo`03003ooooo0<Coo`03003ooooo047o
o`005Ooo00<00?ooool0aOoo0P00@Ooo000Eool00`00ooooo`37ool00`00ooooo`0nool001Goo`03
003ooooo0<Soo`03003ooooo03goo`0000?oo`0000000P000ooo00<00?ooool00_oo10001?oo00<0
0?ooool0bOoo00<00?ooool0??oo0003ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`3:
ool00`00ooooo`0kool000?oo`03003ooooo00Soo`03003ooooo00Coo`<00<_oo`03003ooooo03[o
o`000ooo00<00?ooool02Ooo00<00?ooool00ooo00<00?ooool0c?oo0P00>_oo0003ool00`00oooo
o`0:ool00`00ooooo`02ool00`00ooooo`3>ool00`00ooooo`0gool00004ool000000009ool01000
ooooo`001?oo00<00?ooool0cooo00<00?ooool0=_oo0003ool00`00ooooo`08ool20005ool00`00
ooooo`3@ool2000fool001Goo`03003ooooo0=;oo`03003ooooo03?oo`005Ooo00<00?ooool0dooo
00<00?ooool0<_oo000Eool2003Eool2000bool001Goo`03003ooooo0=Koo`03003ooooo02ooo`00
5Ooo00<00?ooool0eooo00<00?ooool0;_oo000Eool00`00ooooo`3Hool00`00ooooo`0]ool001Go
o`03003ooooo0=Woo`03003ooooo02coo`005Ooo00<00?ooool0f_oo00<00?ooool0:ooo000Eool0
0`00ooooo`3Kool2000[ool001Goo`03003ooooo0=goo`03003ooooo02Soo`005Ooo0P00gooo00<0
0?ooool09ooo000Eool00`00ooooo`3Oool00`00ooooo`0Vool001Goo`03003ooooo0>3oo`03003o
oooo02Goo`005Ooo00<00?ooool0hOoo00<00?ooool09?oo000Eool00`00ooooo`3Rool2000Tool0
01Goo`03003ooooo0>Coo`03003ooooo027oo`005Ooo00<00?ooool0iOoo00<00?ooool08?oo000E
ool2003Wool00`00ooooo`0Oool001Goo`03003ooooo0>Ooo`03003ooooo01koo`005Ooo00<00?oo
ool0j?oo0P007_oo000Eool00`00ooooo`3Zool00`00ooooo`0Kool001Goo`03003ooooo0>_oo`03
003ooooo01[oo`0000?oo`0000000P000ooo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool0
k?oo00<00?ooool06Ooo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3]ool2000I
ool000?oo`03003ooooo00Koo`D000Coo`<00>ooo`03003ooooo01Koo`000ooo00<00?ooool01_oo
00@00?ooool000Goo`03003ooooo0?3oo`03003ooooo01Goo`000ooo00<00?ooool01ooo00<00?oo
00001Ooo00<00?ooool0lOoo00<00?ooool05?oo00001?oo000000002_oo0P001Ooo00<00?ooool0
l_oo00<00?ooool04ooo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3cool2000C
ool001Goo`03003ooooo0?Goo`03003ooooo013oo`005Ooo00<00?ooool0m_oo00<00?ooool03ooo
000Eool2003hool00`00ooooo`0>ool001Goo`03003ooooo0?Soo`8000koo`005Ooo00<00?ooool0
n_oo00<00?ooool02ooo000Eool00`00ooooo`3kool00`00ooooo`0:ool001Goo`03003ooooo0?co
o`03003ooooo00Woo`005Ooo00<00?ooool0oOoo0P002Ooo000Eool00`00ooooo`3oool00`00oooo
o`06ool001Goo`800?ooo`;oo`03003ooooo00Goo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo003ooolQool0
0?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, 0.319096, \
0.00578885, 0.00700661}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.74, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] \(\(0.8760984351000806`\)\(\[InvisibleSpace]\)\) + 
          1.0274376357597258`*^-7\ \[ImaginaryI]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol[k, 10]], {k, 0.01, 1}, 
      PlotStyle \[Rule] {RGBColor[1, 0, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 -0.877637 1.44479 [
[.21429 .55465 -9 -9 ]
[.21429 .55465 9 0 ]
[.40476 .55465 -9 -9 ]
[.40476 .55465 9 0 ]
[.59524 .55465 -9 -9 ]
[.59524 .55465 9 0 ]
[.78571 .55465 -9 -9 ]
[.78571 .55465 9 0 ]
[.97619 .55465 -3 -9 ]
[.97619 .55465 3 0 ]
[.01131 .13372 -18 -4.5 ]
[.01131 .13372 0 4.5 ]
[.01131 .2782 -18 -4.5 ]
[.01131 .2782 0 4.5 ]
[.01131 .42267 -18 -4.5 ]
[.01131 .42267 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .56715 m
.21429 .5734 L
s
[(0.2)] .21429 .55465 0 1 Mshowa
.40476 .56715 m
.40476 .5734 L
s
[(0.4)] .40476 .55465 0 1 Mshowa
.59524 .56715 m
.59524 .5734 L
s
[(0.6)] .59524 .55465 0 1 Mshowa
.78571 .56715 m
.78571 .5734 L
s
[(0.8)] .78571 .55465 0 1 Mshowa
.97619 .56715 m
.97619 .5734 L
s
[(1)] .97619 .55465 0 1 Mshowa
.125 Mabswid
.07143 .56715 m
.07143 .5709 L
s
.11905 .56715 m
.11905 .5709 L
s
.16667 .56715 m
.16667 .5709 L
s
.2619 .56715 m
.2619 .5709 L
s
.30952 .56715 m
.30952 .5709 L
s
.35714 .56715 m
.35714 .5709 L
s
.45238 .56715 m
.45238 .5709 L
s
.5 .56715 m
.5 .5709 L
s
.54762 .56715 m
.54762 .5709 L
s
.64286 .56715 m
.64286 .5709 L
s
.69048 .56715 m
.69048 .5709 L
s
.7381 .56715 m
.7381 .5709 L
s
.83333 .56715 m
.83333 .5709 L
s
.88095 .56715 m
.88095 .5709 L
s
.92857 .56715 m
.92857 .5709 L
s
.25 Mabswid
0 .56715 m
1 .56715 L
s
.02381 .13372 m
.03006 .13372 L
s
[(0.7)] .01131 .13372 1 0 Mshowa
.02381 .2782 m
.03006 .2782 L
s
[(0.8)] .01131 .2782 1 0 Mshowa
.02381 .42267 m
.03006 .42267 L
s
[(0.9)] .01131 .42267 1 0 Mshowa
.125 Mabswid
.02381 .16261 m
.02756 .16261 L
s
.02381 .19151 m
.02756 .19151 L
s
.02381 .2204 m
.02756 .2204 L
s
.02381 .2493 m
.02756 .2493 L
s
.02381 .30709 m
.02756 .30709 L
s
.02381 .33599 m
.02756 .33599 L
s
.02381 .36488 m
.02756 .36488 L
s
.02381 .39378 m
.02756 .39378 L
s
.02381 .45157 m
.02756 .45157 L
s
.02381 .48047 m
.02756 .48047 L
s
.02381 .50936 m
.02756 .50936 L
s
.02381 .53826 m
.02756 .53826 L
s
.02381 .10482 m
.02756 .10482 L
s
.02381 .07592 m
.02756 .07592 L
s
.02381 .04703 m
.02756 .04703 L
s
.02381 .01813 m
.02756 .01813 L
s
.02381 .59605 m
.02756 .59605 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03333 .01472 m
.03793 .01493 L
.04222 .0152 L
.04692 .01559 L
.05195 .01609 L
.06223 .01743 L
.07158 .019 L
.09303 .02386 L
.1133 .03 L
.15133 .04537 L
.19178 .06673 L
.23074 .09158 L
.2682 .11887 L
.30808 .15107 L
.34647 .18466 L
.38728 .22275 L
.40766 .24254 L
.41759 .25239 L
.41987 .25458 L
.42228 .25692 L
.42327 .25769 L
.42433 .25833 L
.4255 .25885 L
.42659 .25923 L
.43112 .2605 L
.43601 .26186 L
.44489 .26441 L
.4644 .27021 L
.50535 .28348 L
.5448 .29778 L
.58274 .31305 L
.62312 .33106 L
.66199 .3503 L
.69936 .37073 L
.73916 .39479 L
.77746 .42045 L
.81819 .45073 L
.85742 .48313 L
.89515 .51757 L
.9353 .55801 L
.97396 .60074 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003oooP000Kh0?ooo`?o
o`005Ooo0P001ooo1oP0nooo000Eool00`00ooooo`0=ool5n03fool001Goo`03003ooooo01;oo`;h
0?Coo`005Ooo00<00?ooool05?oo0oP0lOoo000Eool00`00ooooo`0Gool3n03^ool001Goo`03003o
oooo01[oo`;h0>coo`005Ooo00<00?ooool07?oo0oP0jOoo000Eool00`00ooooo`0Oool2n03Wool0
01Goo`8002;oo`;h0>Goo`005Ooo00<00?ooool08ooo0_P0hooo000Eool00`00ooooo`0Uool2n03Q
ool001Goo`03003ooooo02Ooo`;h0=ooo`005Ooo00<00?ooool0:Ooo0_P0gOoo000Eool00`00oooo
o`0[ool00oP0ooooo`3Jool001Goo`03003ooooo02coo`;h0=[oo`005Ooo00<00?ooool0;_oo00?h
0?ooool0eooo000Eool2000`ool00oP0ooooo`3Fool001Goo`03003ooooo033oo`;h0=Koo`005Ooo
00<00?ooool0<_oo00?h0?ooool0dooo000Eool00`00ooooo`0cool2n03Cool001Goo`03003ooooo
03Goo`03n03ooooo0=3oo`005Ooo00<00?ooool0=_oo0_P0d?oo000Eool00`00ooooo`0hool00oP0
ooooo`3=ool001Goo`8003[oo`03n03ooooo0<coo`005Ooo00<00?ooool0>_oo0_P0c?oo000Eool0
0`00ooooo`0lool00oP0ooooo`39ool001Goo`03003ooooo03goo`;h0<Woo`005Ooo00<00?ooool0
?ooo00?h0?ooool0a_oo000Eool00`00ooooo`10ool00oP0ooooo`35ool000;oo`8000Coo`03003o
oooo00?oo`03003ooooo00Coo`03003ooooo047oo`;h0<Goo`0000Goo`00ooooo`0000Woo`03003o
oooo00Coo`03003ooooo04?oo`03n03ooooo0<;oo`0000Goo`00ooooo`0000[oo`03003ooooo00?o
o`<004Coo`03n03ooooo0<7oo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo04Go
o`03n03ooooo0<3oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003ooooo04Koo`03n03o
oooo0;ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`17ool2n02oool000;o
o`8000Woo`@000Coo`03003ooooo04Woo`03n03ooooo0;coo`005Ooo00<00?ooool0B_oo00?h0?oo
ool0^ooo000Eool00`00ooooo`1;ool00oP0ooooo`2jool001Goo`03003ooooo04coo`;h0;[oo`00
5Ooo0P00Cooo00?h0?ooool0]ooo000Eool00`00ooooo`1?ool00oP0ooooo`2fool001Goo`03003o
oooo053oo`03n03ooooo0;Goo`005Ooo00<00?ooool0DOoo00?h0?ooool0]?oo000Eool00`00oooo
o`1Bool2n02dool001Goo`03003ooooo05Coo`03n03ooooo0;7oo`005Ooo00<00?ooool0EOoo00?h
0?ooool0/?oo000Eool00`00ooooo`1Fool00oP0ooooo`2_ool001Goo`8005Soo`03n03ooooo0:ko
o`005Ooo00<00?ooool0F?oo00?h0?ooool0[Ooo000Eool00`00ooooo`1Iool00oP0ooooo`2/ool0
01Goo`03003ooooo05[oo`03n03ooooo0:_oo`005Ooo00<00?ooool0Fooo00?h0?ooool0Z_oo000E
ool00`00ooooo`1Lool00oP0ooooo`2Yool001Goo`03003ooooo05goo`03n03ooooo0:Soo`005Ooo
00<00?ooool0G_oo00?h0?ooool0Yooo000Eool2001Pool00oP0ooooo`2Vool001Goo`03003ooooo
063oo`03n03ooooo0:Goo`005Ooo00<00?ooool0HOoo00?h0?ooool0Y?oo000Eool00`00ooooo`1R
ool00oP0ooooo`2Sool001Goo`03003ooooo06?oo`03n03ooooo0:;oo`005Ooo00<00?ooool0I?oo
00?h0?ooool0XOoo000Eool00`00ooooo`1Uool00oP0ooooo`2Pool001Goo`03003ooooo06Koo`03
n03ooooo09ooo`005Ooo0P00J?oo00?h0?ooool0W_oo000Eool00`00ooooo`1Xool2n02Nool001Go
o`03003ooooo06Woo`;h09goo`005Ooo00<00?ooool0Jooo0oP0V_oo000Eool00`00ooooo`1^ool4
n02Fool001Goo`03003ooooo07;oo`Gh097oo`000_oo0P001?oo00<00?ooool00ooo0P001Ooo00<0
0?ooool0Mooo0oP0S_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo07[oo`;h
08coo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3001lool3n029ool00005ool00?ooool0
0009ool20005ool00`00ooooo`1oool3n026ool00005ool00?ooool00008ool01000ooooo`001?oo
00<00?ooool0P_oo0oP0Pooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo08Go
o`;h087oo`000_oo0P002_oo0P001Ooo00<00?ooool0Qooo0oP0O_oo000Eool00`00ooooo`2:ool3
n01kool001Goo`03003ooooo08goo`;h07Woo`005Ooo00<00?ooool0Sooo0oP0M_oo000Eool2002C
ool2n01dool001Goo`03003ooooo09Coo`?h077oo`005Ooo00<00?ooool0Uooo0_P0Kooo000Eool0
0`00ooooo`2Iool2n01]ool001Goo`03003ooooo09_oo`;h06_oo`005Ooo00<00?ooool0WOoo0_P0
JOoo000Eool00`00ooooo`2Oool3n01Vool001Goo`800:?oo`;h06Coo`005Ooo00<00?ooool0Y?oo
0_P0H_oo000Eool00`00ooooo`2Vool2n01Pool001Goo`03003ooooo0:Soo`;h05koo`005Ooo00<0
0?ooool0Z_oo0_P0G?oo000Eool00`00ooooo`2/ool2n01Jool001Goo`03003ooooo0:koo`;h05So
o`005Ooo00<00?ooool0/?oo00?h0?ooool0EOoo000Eool2002bool2n01Eool001Goo`03003ooooo
0;?oo`;h05?oo`005Ooo00<00?ooool0]Ooo00?h0?ooool0D?oo000Eool00`00ooooo`2fool2n01@
ool001Goo`03003ooooo0;Soo`;h04koo`005Ooo00<00?ooool0^_oo0_P0C?oo000Eool00`00oooo
o`2lool2n01:ool001Goo`03003ooooo0;koo`;h04Soo`005Ooo0P00`Ooo00?h0?ooool0AOoo000E
ool00`00ooooo`31ool2n015ool001Goo`03003ooooo0<?oo`03n03ooooo04;oo`005Ooo00<00?oo
ool0a?oo00?h0?ooool0@Ooo000Eool00`00ooooo`35ool2n011ool001Goo`03003ooooo0<Ooo`03
n03ooooo03koo`000_oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0b?oo0_P0?_oo0000
1Ooo003ooooo00002ooo00<00?ooool00_oo00<00?ooool0b_oo00?h0?ooool0>ooo00001Ooo003o
oooo00002ooo00<00?ooool00_oo0`00booo00?h0?ooool0>_oo00001Ooo003ooooo00002Ooo0`00
1?oo00<00?ooool0c?oo0_P0>_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo
0<koo`03n03ooooo03Ooo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`3?ool0
0oP0ooooo`0fool000;oo`8000[oo`8000Goo`03003ooooo0=3oo`03n03ooooo03Goo`005Ooo00<0
0?ooool0dOoo00?h0?ooool0=?oo000Eool00`00ooooo`3Bool2n00dool001Goo`03003ooooo0=Co
o`03n03ooooo037oo`005Ooo0P00e_oo00?h0?ooool0<?oo000Eool00`00ooooo`3Fool2n00`ool0
01Goo`03003ooooo0=Soo`03n03ooooo02goo`005Ooo00<00?ooool0fOoo00?h0?ooool0;?oo000E
ool00`00ooooo`3Jool2n00/ool001Goo`03003ooooo0=coo`03n03ooooo02Woo`005Ooo00<00?oo
ool0gOoo00?h0?ooool0:?oo000Eool00`00ooooo`3Nool2n00Xool001Goo`800>7oo`03n03ooooo
02Goo`005Ooo00<00?ooool0hOoo00?h0?ooool09?oo000Eool00`00ooooo`3Rool00oP0ooooo`0S
ool001Goo`03003ooooo0>?oo`03n03ooooo02;oo`005Ooo00<00?ooool0i?oo00?h0?ooool08Ooo
000Eool00`00ooooo`3Uool00oP0ooooo`0Pool001Goo`03003ooooo0>Koo`03n03ooooo01ooo`00
5Ooo00<00?ooool0iooo00?h0?ooool07_oo000Eool2003Yool00oP0ooooo`0Mool001Goo`03003o
oooo0>Woo`03n03ooooo01coo`005Ooo00<00?ooool0j_oo00?h0?ooool06ooo000Eool00`00oooo
o`3[ool00oP0ooooo`0Jool001Goo`03003ooooo0>coo`03n03ooooo01Woo`005Ooo00<00?ooool0
:_oo0P001?oo00<00?ooool00_oo10009Ooo0P001?oo00<00?ooool01?oo00<00?ooool08ooo0P00
1?oo00<00?ooool00ooo0P009_oo0P001?oo00<00?ooool00ooo0P006_oo00?h0?ooool03_oo1000
1_oo000Eool00`00ooooo`0Yool01000ooooo`002?oo00<00?ooool09Ooo00@00?ooool000[oo`03
003ooooo02;oo`04003ooooo0008ool01000ooooo`009?oo00@00?ooool000Soo`04003ooooo000J
ool00oP0ooooo`0?ool00`00ooooo`05ool001Goo`8002[oo`04003ooooo0009ool00`00ooooo`0T
ool01000ooooo`001ooo1@008ooo00@00?ooool000Soo`04003ooooo000Tool01000ooooo`002?oo
00@00?ooool001_oo`03n03ooooo00koo`03003ooooo00Goo`005Ooo00<00?ooool0:Ooo00@00?oo
ool000[oo`03003ooooo02?oo`04003ooooo0007ool01000ooooo`009?oo00@00?ooool000Soo`<0
02Goo`04003ooooo0009ool2000Mool00oP0ooooo`0=ool00`00ooooo`05ool001Goo`03003ooooo
02Woo`04003ooooo000;ool00`00ooooo`0Rool01000ooooo`002?oo00<00?oo00009?oo00@00?oo
ool000Soo`03003ooooo02Goo`04003ooooo0008ool01000ooooo`007Ooo00?h0?ooool03?oo00<0
0?ooool01Ooo000Eool00`00ooooo`0Yool01000ooooo`002?oo00@00?ooool002Coo`04003ooooo
0009ool2000Tool01000ooooo`002?oo00<00?ooool09Ooo00@00?ooool000Soo`04003ooooo000N
ool00oP0ooooo`09ool30007ool001Goo`03003ooooo02[oo`8000[oo`8002Koo`8000_oo`03003o
oooo02?oo`8000[oo`<002Goo`8000[oo`80023oo`03n03ooooo00[oo`03003ooooo00Goo`005Ooo
00<00?ooool0m?oo00?h0?ooool04Ooo000Eool00`00ooooo`3eool00oP0ooooo`0@ool001Goo`03
003ooooo0?Koo`03n03ooooo00ooo`003_ooo`000P0000?h000000003P00000Eool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`02ool00oP0ooooo`05
ool00`00ooooo`05ool001Goo`03003ooooo037oo`03003ooooo037oo`03003ooooo033oo`03003o
oooo037oo`03003ooooo02[oo`03n03ooooo00Coo`03003ooooo00Goo`005Ooo00<00?ooool0n_oo
00?h0?ooool02ooo000Eool00`00ooooo`3jool00oP0ooooo`0;ool001Goo`03003ooooo0?_oo`03
n03ooooo00[oo`005Ooo00<00?ooool0o?oo00?h0?ooool02Ooo000Eool00`00ooooo`3mool00oP0
ooooo`08ool001Goo`800?ooo`03n03ooooo00Ooo`005Ooo00<00?ooool0oooo00?h0?ooool01_oo
000Eool00`00ooooo`3oool1ool00oP0ooooo`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0825959, 0.596272, \
0.00385924, 0.00254394}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%1066, %1067]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.0147151 0.392403 [
[.15079 .00222 -9 -9 ]
[.15079 .00222 9 0 ]
[.27778 .00222 -9 -9 ]
[.27778 .00222 9 0 ]
[.40476 .00222 -9 -9 ]
[.40476 .00222 9 0 ]
[.53175 .00222 -9 -9 ]
[.53175 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.78571 .00222 -9 -9 ]
[.78571 .00222 9 0 ]
[.9127 .00222 -9 -9 ]
[.9127 .00222 9 0 ]
[.01131 .0932 -18 -4.5 ]
[.01131 .0932 0 4.5 ]
[.01131 .17168 -18 -4.5 ]
[.01131 .17168 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .32864 -18 -4.5 ]
[.01131 .32864 0 4.5 ]
[.01131 .40712 -6 -4.5 ]
[.01131 .40712 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .56408 -18 -4.5 ]
[.01131 .56408 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .01472 m
.15079 .02097 L
s
[(0.2)] .15079 .00222 0 1 Mshowa
.27778 .01472 m
.27778 .02097 L
s
[(0.4)] .27778 .00222 0 1 Mshowa
.40476 .01472 m
.40476 .02097 L
s
[(0.6)] .40476 .00222 0 1 Mshowa
.53175 .01472 m
.53175 .02097 L
s
[(0.8)] .53175 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(1)] .65873 .00222 0 1 Mshowa
.78571 .01472 m
.78571 .02097 L
s
[(1.2)] .78571 .00222 0 1 Mshowa
.9127 .01472 m
.9127 .02097 L
s
[(1.4)] .9127 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.18254 .01472 m
.18254 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.34127 .01472 m
.34127 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.5 .01472 m
.5 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.81746 .01472 m
.81746 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.97619 .01472 m
.97619 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .0932 m
.03006 .0932 L
s
[(0.2)] .01131 .0932 1 0 Mshowa
.02381 .17168 m
.03006 .17168 L
s
[(0.4)] .01131 .17168 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.6)] .01131 .25016 1 0 Mshowa
.02381 .32864 m
.03006 .32864 L
s
[(0.8)] .01131 .32864 1 0 Mshowa
.02381 .40712 m
.03006 .40712 L
s
[(1)] .01131 .40712 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(1.2)] .01131 .4856 1 0 Mshowa
.02381 .56408 m
.03006 .56408 L
s
[(1.4)] .01131 .56408 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .05396 m
.02756 .05396 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .11282 m
.02756 .11282 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .15206 m
.02756 .15206 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .21092 m
.02756 .21092 L
s
.02381 .23054 m
.02756 .23054 L
s
.02381 .26978 m
.02756 .26978 L
s
.02381 .2894 m
.02756 .2894 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34826 m
.02756 .34826 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .3875 m
.02756 .3875 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .46598 m
.02756 .46598 L
s
.02381 .50522 m
.02756 .50522 L
s
.02381 .52484 m
.02756 .52484 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .5837 m
.02756 .5837 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03016 .25708 m
.03323 .25713 L
.03608 .25721 L
.03922 .25731 L
.04257 .25745 L
.04942 .25781 L
.05566 .25824 L
.06995 .25956 L
.08347 .26123 L
.10882 .2654 L
.13579 .2712 L
.16176 .27795 L
.18674 .28537 L
.21332 .29411 L
.23891 .30323 L
.26612 .31358 L
.27971 .31895 L
.28633 .32163 L
.28785 .32222 L
.28946 .32286 L
.29012 .32307 L
.29083 .32324 L
.2916 .32338 L
.29233 .32349 L
.29535 .32383 L
.29861 .3242 L
.30453 .32489 L
.31754 .32647 L
.34484 .33007 L
.37113 .33396 L
.39643 .3381 L
.42335 .343 L
.44926 .34822 L
.47418 .35377 L
.50071 .36031 L
.52625 .36727 L
.5534 .3755 L
.57955 .3843 L
.6047 .39365 L
.63147 .40463 L
.65724 .41624 L
.65873 .41694 L
s
0 g
.03016 .01864 m
.06854 .04236 L
.11039 .06823 L
.1497 .09252 L
.1875 .11588 L
.22774 .14075 L
.26647 .16469 L
.30764 .19013 L
.34731 .21465 L
.38546 .23823 L
.42606 .26332 L
.46515 .28748 L
.50273 .3107 L
.54275 .33544 L
.58126 .35924 L
.62221 .38455 L
.66166 .40893 L
.69959 .43237 L
.73997 .45733 L
.77884 .48135 L
.82014 .50688 L
.85994 .53147 L
.89823 .55514 L
.93896 .58031 L
.97619 .60332 L
s
.03016 .30939 m
.03133 .30943 L
.03239 .30947 L
.03362 .30932 L
.03477 .30933 L
.03585 .30962 L
.03684 .30967 L
.03908 .30981 L
.04372 .31016 L
.04884 .31064 L
.0584 .3118 L
.06854 .31335 L
.11039 .32202 L
.15041 .33176 L
.18892 .34119 L
.22987 .35077 L
.26932 .35939 L
.30725 .36708 L
.34763 .3746 L
.3865 .38121 L
.4278 .38753 L
.4676 .39292 L
.5059 .3974 L
.54663 .40134 L
.56691 .40295 L
.58585 .40422 L
.60522 .40528 L
.62357 .40606 L
.63325 .40638 L
.64379 .40665 L
.64843 .40675 L
.65336 .40684 L
.65767 .4069 L
.66222 .40695 L
.66454 .40698 L
.66707 .407 L
.66941 .40701 L
.67153 .40702 L
.67273 .40703 L
.67404 .40703 L
.67476 .40703 L
.67542 .40704 L
.67672 .40704 L
.67784 .40704 L
.67907 .40704 L
.68036 .40704 L
.68156 .40704 L
.68268 .40704 L
.68391 .40703 L
.68613 .40703 L
Mistroke
.6883 .40702 L
.69031 .407 L
.69487 .40697 L
.69972 .40692 L
.70996 .40678 L
.72103 .40657 L
.74107 .40608 L
.78186 .40477 L
.82113 .40335 L
.8589 .402 L
.89911 .40067 L
.93781 .39952 L
.97619 .39851 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool0033oo`8000Coo`03003ooooo00;oo`@001Coo`8000Coo`03003ooooo00Coo`03003ooooo01?o
o`8000Coo`03003ooooo00?oo`8001Coo`8000Coo`03003ooooo00?oo`8001[oo`@001Soo`@000?o
o`03003ooooo00;oo`@001?oo`@000?oo`03003ooooo00Coo`03003ooooo013oo`00;ooo00@00?oo
ool000Soo`03003ooooo01Coo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`002?oo00@0
0?ooool001;oo`04003ooooo0008ool01000ooooo`006ooo00<00?ooool06Ooo00<00?ooool01ooo
00<00?ooool05_oo00<00?ooool02Ooo00<00?ooool04?oo000_ool01000ooooo`002Ooo00<00?oo
ool04ooo00@00?ooool000Ooo`D001?oo`04003ooooo0008ool01000ooooo`004_oo00@00?ooool0
00Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`08ool00`00ooooo`0Eool00`00oooo
o`06ool5000Aool002ooo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`001ooo00@00?oo
ool001Coo`04003ooooo0008ool3000Cool01000ooooo`002Ooo0P007?oo00<00?ooool06Ooo00<0
0?ooool02Ooo00<00?ooool05?oo00<00?ooool01_oo00@00?ooool001;oo`00;ooo00@00?ooool0
00_oo`03003ooooo017oo`04003ooooo0008ool00`00ool0000Dool01000ooooo`002?oo00<00?oo
ool04ooo00@00?ooool000Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`0:ool00`00
ooooo`0Cool00`00ooooo`07ool00`00ool0000Bool002ooo`04003ooooo0008ool01000ooooo`00
4ooo00@00?ooool000Woo`8001Coo`04003ooooo0008ool00`00ooooo`0Cool01000ooooo`002?oo
00@00?ooool001Woo`<001Woo`<000Woo`04003ooooo000Cool3000:ool2000Bool001Goo`03003o
oooo01Soo`8000[oo`8001Goo`8000_oo`03003ooooo01?oo`8000[oo`<001?oo`8000[oo`8001co
o`03003ooooo01Woo`03003ooooo00Soo`8001Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo
00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`003_oo
o`004`00000Eool00`00ool00006ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`05ool001Goo`03003ooooo008001goo`03003ooooo023oo`03003o
oooo023oo`03003ooooo01ooo`03003ooooo023oo`03003ooooo01ooo`03003ooooo023oo`03003o
oooo01Koo`005Ooo00<00?ooool00_oo00<00?ooool0oooo1?oo000Eool00`00ooooo`03ool00`00
ooooo`3oool3ool001Goo`8000Goo`800?ooo`?oo`005Ooo00<00?ooool01_oo00<00?ooool0oooo
000Eool00`00ooooo`07ool2003oool001Goo`03003ooooo00Woo`03003ooooo0?coo`005Ooo00<0
0?ooool02_oo0P00o?oo000Eool00`00ooooo`0<ool00`00ooooo`3iool001Goo`8000koo`800?Wo
o`005Ooo00<00?ooool03ooo0P00mooo000Eool00`00ooooo`0Aool00`00ooooo`3dool001Goo`03
003ooooo01;oo`800?Coo`005Ooo00<00?ooool05?oo00<00?ooool0lOoo000Eool2000Fool2003a
ool001Goo`03003ooooo01Ooo`800>ooo`005Ooo00<00?ooool06Ooo0P00kOoo000Eool00`00oooo
o`0Kool2003[ool000;oo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo01goo`800>Woo`00
00Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo01ooo`03003ooooo0>Koo`0000Goo`00
ooooo`0000Woo`03003ooooo00Coo`<0023oo`800>Koo`0000Goo`00ooooo`0000[oo`03003ooooo
00?oo`03003ooooo02;oo`03003ooooo0>?oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03
003ooooo02?oo`03003ooooo0>;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`0Tool2003Rool000;oo`8000[oo`8000Goo`03003ooooo02Koo`03003ooooo0=ooo`005Ooo0P00
:?oo0P00gooo000Eool00`00ooooo`0Yool00`00ooooo`3Lool001Goo`03003ooooo02[oo`800=co
o`005Ooo00<00?ooool0;?oo0P00f_oo000Eool00`00ooooo`0^ool2003Hool001Goo`80037oo`80
0=Koo`005Ooo00<00?ooool0<_oo0P00e?oo000Eool00`00ooooo`0dool00`00ooooo`3Aool001Go
o`03003ooooo03Goo`800=7oo`005Ooo00<00?ooool0=ooo00<00?ooool0c_oo000Eool00`00oooo
o`0hool2003>ool001Goo`8003_oo`800<coo`005Ooo00<00?ooool0??oo00<00?ooool0bOoo000E
ool00`00ooooo`0mool20039ool000;oo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003o
oooo03ooo`03003ooooo0<Koo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo043o
o`800<Koo`0000Goo`00ooooo`0000Ooo`D000Coo`<004;oo`03003ooooo0<?oo`0000Goo`00oooo
o`0000Ooo`04003ooooo0005ool00`00ooooo`13ool20033ool00005ool00?ooool00008ool00`00
ool00005ool00`00ooooo`15ool20031ool00005ool00?ooool00009ool20005ool00`00ooooo`17
ool00`00ooooo`2nool000;oo`8000_oo`03003ooooo00?oo`03003ooooo04Soo`800;koo`005Ooo
0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool2002kool001Goo`03003ooooo04goo`03
003ooooo0;Soo`005Ooo00<00?ooool0C_oo0P00^?oo000Eool00`00ooooo`1@ool2002fool001Go
o`03003ooooo05;oo`03003ooooo0;?oo`005Ooo0P00E?oo0P00/ooo000Eool00`00ooooo`1Eool0
0`00ooooo`2`ool001Goo`03003ooooo05Koo`800;3oo`005Ooo00<00?ooool0F?oo0P00[_oo000E
ool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`8005coo`800:_oo`005Ooo00<00?ooool0GOoo
0P00ZOoo000Eool00`00ooooo`1Oool00`00ooooo`2Vool000;oo`8000Coo`03003ooooo00?oo`80
00Goo`03003ooooo063oo`800:Koo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`1Rool00`00ooooo`2Sool00005ool00?ooool00008ool01000ooooo`001?oo0`00Hooo0P00Xooo
00001Ooo003ooooo00002?oo0`001Ooo00<00?ooool0IOoo0P00XOoo00001Ooo003ooooo00002?oo
00<00?ooool01Ooo00<00?oon0002?P0Gooo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<0
0?ooool01Ooo00<00?ooool02?oo2OP0Eooo0P00W_oo0002ool2000:ool30004ool00`00ooooo`0A
ool5n01Dool00`00ooooo`2Kool001Goo`03003ooooo01Koo`Ch057oo`8009_oo`005Ooo0P006ooo
0oP0D?oo0P00VOoo000Eool00`00ooooo`0Mool4n01>ool2002Gool001Goo`03003ooooo027oo`?h
04goo`8009Goo`005Ooo00<00?ooool09?oo1?P0Booo0P00Tooo000Eool00`00ooooo`0Xool3n01:
ool00`00ooooo`2@ool001Goo`8002coo`Ch04Ooo`80093oo`005Ooo00<00?ooool0;ooo0oP0A_oo
00<00?ooool0SOoo000Eool00`00ooooo`0bool2n015ool00`00ooooo`2<ool001Goo`03003ooooo
03Coo`;h04Coo`8008coo`005Ooo00<00?ooool0=_oo0oP0@ooo00<00?ooool0ROoo000Eool5000g
ool4n010ool20029ool001Goo`03003ooooo00;oo`T003;oo`?h03ooo`03003ooooo08Koo`005Ooo
00<00?ooool02ooo1000<Ooo0_P0?_oo0P00Q_oo000Eool00`00ooooo`0?ool4000_ool2n00nool2
0024ool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo01?oo`@002goo`Gh03_oo`80
08;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Gool6000/ool8n00eool2
0020ool00005ool00?ooool00008ool01000ooooo`001?oo0`007Ooo1000<?oo1oP0<?oo0P00O_oo
00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool08Ooo1000<ooo1oP0:ooo00<00?ooool0Nooo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo02Goo`@003Koo`Oh02Goo`8007_o
o`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Yool3000jool6n00Qool00`00
ooooo`1hool000;oo`8000[oo`8000Goo`03003ooooo02coo`@003coo`Ch01koo`8007Soo`005Ooo
0P00<Ooo1000??oo1?P07?oo0P00M_oo000Eool00`00ooooo`0dool4000lool5n00Iool00`00oooo
o`1cool001Goo`03003ooooo03Soo`H003_oo`Ch01Koo`8007?oo`005Ooo00<00?ooool0?_oo1@00
>_oo1?P05?oo00<00?ooool0L?oo000Eool00`00ooooo`13ool5000iool4n00Aool2001`ool001Go
o`8004Woo`D003Soo`?h013oo`03003ooooo06goo`005Ooo00<00?ooool0COoo1P00=Ooo1?P03Ooo
0P00KOoo000Eool00`00ooooo`1Cool5000dool3n00<ool2001[ool001Goo`03003ooooo05Soo`H0
037oo`;h00coo`03003ooooo06Soo`005Ooo00<00?ooool0G_oo1@00;_oo0_P02ooo0P00J?oo000E
ool00`00ooooo`1Sool6000Zool3n00:ool00`00ooooo`1Uool001Goo`8006[oo`P002Goo`Ch00Oo
o`8006Goo`005Ooo00<00?ooool0LOoo2P007ooo0oP01_oo0P00Hooo000Eool00`00ooooo`1kool;
000Gool2n006ool00`00ooooo`1Pool000goo`@000Coo`03003ooooo08Koo`P0017oo`;h00Goo`80
03kooa/000Ooo`003ooo00<00?ooool00ooo00<00?ooool0S_oo40000ooo0oP01?oo0P007?oo8000
8_oo000?ool00`00ooooo`03ool3002NoolX0012ool000ooo`03003ooooo00?oo`03003ooooo0:Ko
o`;h00?oo`8005_oo`003ooo00<00?ooool00ooo00<00?ooool0Z?oo0_P00ooo00<00?ooool0F?oo
000=ool30005ool00`00ooooo`2Zool01?P0oooooooo0P00F?oo000?ool00`00ooooo`03ool00`00
ooooo`2`ool2001Fool001Goo`800;?oo`03003ooooo05?oo`005Ooo00<00?ooool0/ooo0P00Dooo
000Eool00`00ooooo`2eool00`00ooooo`1@ool001Goo`03003ooooo0;Koo`80053oo`005Ooo00<0
0?ooool0^?oo00<00?ooool0COoo000Eool00`00ooooo`2iool2001=ool001Goo`800;coo`8004_o
o`005Ooo00<00?ooool0_Ooo00<00?ooool0B?oo000Eool00`00ooooo`2nool20018ool001Goo`03
003ooooo0<3oo`03003ooooo04Goo`005Ooo00<00?ooool0`Ooo0P00AOoo000Eool20034ool20013
ool001Goo`03003ooooo0<Goo`03003ooooo043oo`005Ooo00<00?ooool0a_oo0P00@?oo00000ooo
000000020003ool00`00ooooo`02ool40004ool00`00ooooo`38ool2000nool000?oo`03003ooooo
00Ooo`03003ooooo00Goo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?ooool02?oo00<0
0?ooool01?oo0`00booo0P00>ooo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3=
ool2000iool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo0<ooo`03003ooooo03Ko
o`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3@ool2000fool000?oo`03003ooooo
00Soo`8000Goo`03003ooooo0=;oo`8003Coo`005Ooo00<00?ooool0e?oo0P00<_oo000Eool2003G
ool00`00ooooo`0_ool001Goo`03003ooooo0=Ooo`8002ooo`005Ooo00<00?ooool0fOoo00<00?oo
ool0;?oo000Eool00`00ooooo`3Jool00`00ooooo`0[ool001Goo`03003ooooo0=_oo`8002_oo`00
5Ooo0P00g_oo00<00?ooool0:?oo000Eool00`00ooooo`3Nool2000Xool001Goo`03003ooooo0>3o
o`03003ooooo02Goo`005Ooo00<00?ooool0hOoo0P009Ooo000Eool00`00ooooo`3Sool2000Sool0
01Goo`800>Koo`80027oo`005Ooo00<00?ooool0iooo0P007ooo000Eool00`00ooooo`3Yool2000M
ool001Goo`03003ooooo0>_oo`03003ooooo01[oo`0000?oo`0000000P000ooo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool0k?oo0P006_oo0003ool00`00ooooo`09ool00`00ooooo`03ool0
0`00ooooo`3^ool00`00ooooo`0Gool000?oo`03003ooooo00Koo`D000Coo`<00>ooo`8001Ooo`00
0ooo00<00?ooool01_oo00@00?ooool000Goo`03003ooooo0?7oo`8001Goo`000ooo00<00?ooool0
1ooo00<00?oo00001Ooo00<00?ooool0looo00<00?ooool04_oo00001?oo000000002_oo0P001Ooo
00<00?ooool0m?oo0P004_oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3fool0
0`00ooooo`0?ool001Goo`800?Soo`8000ooo`005Ooo00<00?ooool0nOoo0P003Ooo000Eool00`00
ooooo`3kool00`00ooooo`0:ool001Goo`03003ooooo0?coo`8000[oo`005Ooo00<00?ooool0o_oo
0P002?oo000Eool2003oool2ool00`00ooooo`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, -0.117974, \
0.00578885, 0.00936656}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol2[k, 10]], {k, 0.01, 0.4}, 
      PlotStyle \[Rule] {RGBColor[0, 1, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 2.38095 -13.7251 18.3014 [
[.2619 -0.01155 -9 -9 ]
[.2619 -0.01155 9 0 ]
[.5 -0.01155 -9 -9 ]
[.5 -0.01155 9 0 ]
[.7381 -0.01155 -9 -9 ]
[.7381 -0.01155 9 0 ]
[.97619 -0.01155 -9 -9 ]
[.97619 -0.01155 9 0 ]
[.01131 .09246 -30 -4.5 ]
[.01131 .09246 0 4.5 ]
[.01131 .18396 -24 -4.5 ]
[.01131 .18396 0 4.5 ]
[.01131 .27547 -30 -4.5 ]
[.01131 .27547 0 4.5 ]
[.01131 .36698 -24 -4.5 ]
[.01131 .36698 0 4.5 ]
[.01131 .45848 -30 -4.5 ]
[.01131 .45848 0 4.5 ]
[.01131 .54999 -24 -4.5 ]
[.01131 .54999 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .00095 m
.2619 .0072 L
s
[(0.1)] .2619 -0.01155 0 1 Mshowa
.5 .00095 m
.5 .0072 L
s
[(0.2)] .5 -0.01155 0 1 Mshowa
.7381 .00095 m
.7381 .0072 L
s
[(0.3)] .7381 -0.01155 0 1 Mshowa
.97619 .00095 m
.97619 .0072 L
s
[(0.4)] .97619 -0.01155 0 1 Mshowa
.125 Mabswid
.07143 .00095 m
.07143 .0047 L
s
.11905 .00095 m
.11905 .0047 L
s
.16667 .00095 m
.16667 .0047 L
s
.21429 .00095 m
.21429 .0047 L
s
.30952 .00095 m
.30952 .0047 L
s
.35714 .00095 m
.35714 .0047 L
s
.40476 .00095 m
.40476 .0047 L
s
.45238 .00095 m
.45238 .0047 L
s
.54762 .00095 m
.54762 .0047 L
s
.59524 .00095 m
.59524 .0047 L
s
.64286 .00095 m
.64286 .0047 L
s
.69048 .00095 m
.69048 .0047 L
s
.78571 .00095 m
.78571 .0047 L
s
.83333 .00095 m
.83333 .0047 L
s
.88095 .00095 m
.88095 .0047 L
s
.92857 .00095 m
.92857 .0047 L
s
.25 Mabswid
0 .00095 m
1 .00095 L
s
.02381 .09246 m
.03006 .09246 L
s
[(0.755)] .01131 .09246 1 0 Mshowa
.02381 .18396 m
.03006 .18396 L
s
[(0.76)] .01131 .18396 1 0 Mshowa
.02381 .27547 m
.03006 .27547 L
s
[(0.765)] .01131 .27547 1 0 Mshowa
.02381 .36698 m
.03006 .36698 L
s
[(0.77)] .01131 .36698 1 0 Mshowa
.02381 .45848 m
.03006 .45848 L
s
[(0.775)] .01131 .45848 1 0 Mshowa
.02381 .54999 m
.03006 .54999 L
s
[(0.78)] .01131 .54999 1 0 Mshowa
.125 Mabswid
.02381 .01925 m
.02756 .01925 L
s
.02381 .03755 m
.02756 .03755 L
s
.02381 .05585 m
.02756 .05585 L
s
.02381 .07415 m
.02756 .07415 L
s
.02381 .11076 m
.02756 .11076 L
s
.02381 .12906 m
.02756 .12906 L
s
.02381 .14736 m
.02756 .14736 L
s
.02381 .16566 m
.02756 .16566 L
s
.02381 .20226 m
.02756 .20226 L
s
.02381 .22057 m
.02756 .22057 L
s
.02381 .23887 m
.02756 .23887 L
s
.02381 .25717 m
.02756 .25717 L
s
.02381 .29377 m
.02756 .29377 L
s
.02381 .31207 m
.02756 .31207 L
s
.02381 .33037 m
.02756 .33037 L
s
.02381 .34868 m
.02756 .34868 L
s
.02381 .38528 m
.02756 .38528 L
s
.02381 .40358 m
.02756 .40358 L
s
.02381 .42188 m
.02756 .42188 L
s
.02381 .44018 m
.02756 .44018 L
s
.02381 .47678 m
.02756 .47678 L
s
.02381 .49509 m
.02756 .49509 L
s
.02381 .51339 m
.02756 .51339 L
s
.02381 .53169 m
.02756 .53169 L
s
.02381 .56829 m
.02756 .56829 L
s
.02381 .58659 m
.02756 .58659 L
s
.02381 .60489 m
.02756 .60489 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.04762 .01472 m
.05637 .01502 L
.06596 .01547 L
.08529 .01671 L
.10656 .01861 L
.12637 .02089 L
.16439 .02663 L
.2048 .03471 L
.24373 .04443 L
.28118 .05558 L
.32103 .06939 L
.35939 .08459 L
.39628 .10099 L
.43556 .12039 L
.47337 .14095 L
.51356 .16487 L
.55228 .18995 L
.58951 .21597 L
.62914 .24574 L
.6673 .27645 L
.70784 .31131 L
.7469 .34713 L
.78449 .38367 L
.82447 .42484 L
.86297 .46678 L
.89999 .50906 L
.9394 .55657 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool005Ooo`8000Coo`03003ooooo00;oo`@002koo`8000Co
o`03003ooooo00;oo`@002koo`8000Coo`03003ooooo00?oo`80033oo`8000Coo`03003ooooo00Co
o`03003ooooo007oo`00E_oo00@00?ooool000[oo`03003ooooo02coo`04003ooooo0008ool00`00
ooooo`0^ool01000ooooo`002?oo00@00?ooool002koo`04003ooooo000:ool00`00ooooo`01ool0
05Koo`04003ooooo000:ool00`00ooooo`0/ool01000ooooo`002Ooo00<00?ooool0;Ooo00@00?oo
ool000_oo`03003ooooo02coo`04003ooooo0007ool50002ool005Koo`04003ooooo000:ool00`00
ooooo`0/ool01000ooooo`002_oo00<00?ooool0;?oo00@00?ooool000Woo`8002ooo`04003ooooo
0007ool01000ooooo`000ooo001Fool01000ooooo`002_oo00<00?ooool0;?oo00@00?ooool000_o
o`03003ooooo02_oo`04003ooooo000;ool00`00ooooo`0/ool01000ooooo`002?oo00<00?oo0000
0ooo001Fool01000ooooo`002?oo0`00;_oo00@00?ooool000Soo`04003ooooo000]ool01000oooo
o`002?oo00@00?ooool002koo`04003ooooo0009ool20003ool005Ooo`8000_oo`03003ooooo02go
o`8000[oo`8002ooo`8000[oo`80033oo`8000_oo`03003ooooo007oo`00oooo8Ooo003ooolQool0
0?ooob7oo`006oooo`000`000ooo000Qool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`09
ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`07ool0027oo`03003ooooo0?coo`008Ooo
00<00?ooool00ooo10OPmOoo000Qool00`00ooooo`07ool>1n3Wool0027oo`8001Koo`L7h>3oo`00
8Ooo00<00?ooool07?oo1`OPfOoo000Qool00`00ooooo`0Sool31n3Fool0027oo`03003ooooo02Ko
o`@7h=;oo`008Ooo0P00:ooo10OPc_oo000Qool00`00ooooo`0^ool51n39ool0027oo`03003ooooo
03?oo`@7h<Goo`008Ooo00<00?ooool0=ooo0`OP`_oo000Qool00`00ooooo`0jool41n2nool0027o
o`8003ooo`<7h;_oo`008Ooo00<00?ooool0@Ooo0`OP^?oo000Qool00`00ooooo`14ool41n2dool0
027oo`03003ooooo04Soo`<7h;7oo`008Ooo00<00?ooool0Booo0POP[ooo000Qool2001>ool31n2/
ool0027oo`03003ooooo053oo`87h:[oo`000_oo0P001?oo00<00?ooool00ooo00<00?ooool00_oo
0`000ooo0`001Ooo00<00?ooool0D_oo0POPZ?oo00001Ooo003ooooo00002Ooo00<00?ooool01Ooo
00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0E?oo0POPY_oo00001Ooo003ooooo00002_oo
00<00?ooool01?oo00<00?ooool00ooo00<00?ooool00_oo0P00Eooo0POPY?oo00001Ooo003ooooo
00002_oo00@00?ooooooo`<000?oo`<000Goo`03003ooooo05Soo`87h:;oo`0000Goo`00ooooo`00
00_oo`04003ooooo0005ool00`00ooooo`05ool00`00ooooo`1Jool21n2Pool00005ool00?ooool0
0008ool01000ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0G?oo0POPW_oo
0002ool20009ool40002ool40002ool40004ool00`00ooooo`1Nool21n2Lool0027oo`80067oo`87
h9[oo`008Ooo00<00?ooool0H_oo0POPV?oo000Qool00`00ooooo`1Tool21n2Fool0027oo`03003o
oooo06Koo`87h9Coo`008Ooo00<00?ooool0J?oo0POPT_oo000Qool2001[ool21n2@ool0027oo`03
003ooooo06coo`87h8koo`008Ooo00<00?ooool0K_oo0POPS?oo000Qool00`00ooooo`1`ool00`OP
ooooo`29ool0027oo`8007;oo`87h8Woo`008Ooo00<00?ooool0Looo0POPQooo000Qool00`00oooo
o`1eool21n25ool0027oo`03003ooooo07Ooo`87h8?oo`008Ooo00<00?ooool0NOoo0POPPOoo000Q
ool2001lool00`OPooooo`1nool0027oo`03003ooooo07coo`87h7koo`008Ooo00<00?ooool0O_oo
0POPO?oo0008ool20004ool00`00ooooo`03ool00`00ooooo`03ool20005ool00`00ooooo`20ool0
0`OPooooo`1iool000Ooo`04003ooooo0009ool00`00ooooo`02ool01000ooooo`001?oo00<00?oo
ool0POoo0POPNOoo0007ool01000ooooo`002_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`80
08Coo`87h7Ooo`001ooo00@00?ooool000[oo`04003oooooool30005ool00`00ooooo`25ool00`OP
ooooo`1dool000Ooo`04003ooooo000;ool01000ooooo`001ooo00<00?ooool0Q_oo00<7h?ooool0
Looo0007ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo08Ooo`87
h7?oo`002?oo0P002Ooo10000ooo0`001?oo00<00?ooool0ROoo00<7h?ooool0L?oo000Qool2002;
ool00`OPooooo`1_ool0027oo`03003ooooo08_oo`87h6ooo`008Ooo00<00?ooool0SOoo00<7h?oo
ool0K?oo000Qool00`00ooooo`2>ool00`OPooooo`1[ool0027oo`80093oo`031n3ooooo06[oo`00
8Ooo00<00?ooool0T?oo0POPJ_oo000Qool00`00ooooo`2Bool00`OPooooo`1Wool0027oo`03003o
oooo09?oo`031n3ooooo06Koo`008Ooo00<00?ooool0U?oo00<7h?ooool0IOoo000Qool2002Fool2
1n1Uool0027oo`03003ooooo09Ooo`031n3ooooo06;oo`008Ooo00<00?ooool0V?oo00<7h?ooool0
HOoo000Qool00`00ooooo`2Iool00`OPooooo`1Pool0027oo`03003ooooo09[oo`87h63oo`008Ooo
0P00WOoo00<7h?ooool0GOoo000Qool00`00ooooo`2Mool00`OPooooo`1Lool000;oo`8000Coo`03
003ooooo00?oo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo09koo`031n3ooooo05_oo`00
00Goo`00ooooo`0000Woo`03003ooooo00;oo`04003ooooo0005ool00`00ooooo`02ool00`00oooo
o`2Oool21n1Kool00005ool00?ooool0000:ool01@00oooooooo00000_oo00<00?ooool00ooo00<0
0?ooool00_oo0P00X_oo00<7h?ooool0F?oo00001Ooo003ooooo00002_oo00@00?ooooooo`<000?o
o`<000Goo`03003ooooo0:;oo`031n3ooooo05Ooo`0000Goo`00ooooo`0000_oo`04003ooooo0005
ool00`00ooooo`05ool00`00ooooo`2Sool00`OPooooo`1Fool00005ool00?ooool00008ool01000
ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0Y?oo0POPE_oo0002ool20009
ool40003ool30002ool40004ool00`00ooooo`2Vool00`OPooooo`1Cool0027oo`800:Soo`031n3o
oooo05;oo`008Ooo00<00?ooool0Z?oo00<7h?ooool0DOoo000Qool00`00ooooo`2Yool00`OPoooo
o`1@ool0027oo`03003ooooo0:[oo`87h53oo`008Ooo00<00?ooool0[?oo00<7h?ooool0COoo000Q
ool2002^ool00`OPooooo`1<ool0027oo`03003ooooo0:koo`031n3ooooo04_oo`008Ooo00<00?oo
ool0[ooo00<7h?ooool0B_oo000Qool00`00ooooo`2`ool00`OPooooo`19ool0027oo`03003ooooo
0;7oo`031n3ooooo04Soo`008Ooo0P00/ooo0POPB?oo000Qool00`00ooooo`2dool00`OPooooo`15
ool0027oo`03003ooooo0;Goo`031n3ooooo04Coo`008Ooo00<00?ooool0]_oo00<7h?ooool0@ooo
000Qool2002hool00`OPooooo`12ool0027oo`03003ooooo0;Soo`031n3ooooo047oo`008Ooo00<0
0?ooool0^Ooo00<7h?ooool0@?oo0008ool20004ool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`04ool00`00ooooo`2jool00`OPooooo`0oool000Ooo`04003ooooo0009ool00`00ooooo`03
ool00`00ooooo`04ool00`00ooooo`2kool00`OPooooo`0nool000Ooo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`03ool2002mool00`OPooooo`0mool000Ooo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`2mool00`OPooooo`0lool000Ooo`04003ooooo000;
ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`2nool00`OPooooo`0kool000Ooo`04003o
oooo0008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo0;ooo`031n3ooooo03[oo`00
2?oo0P002Ooo10000_oo10001?oo00<00?ooool0`?oo00<7h?ooool0>Ooo000Qool20032ool00`OP
ooooo`0hool0027oo`03003ooooo0<;oo`031n3ooooo03Ooo`008Ooo00<00?ooool0`ooo00<7h?oo
ool0=_oo000Qool00`00ooooo`34ool00`OPooooo`0eool0027oo`800<Koo`031n3ooooo03Coo`00
8Ooo00<00?ooool0a_oo00<7h?ooool0<ooo000Qool00`00ooooo`37ool00`OPooooo`0bool0027o
o`03003ooooo0<Soo`031n3ooooo037oo`008Ooo00<00?ooool0bOoo00<7h?ooool0<?oo000Qool2
003;ool00`OPooooo`0_ool0027oo`03003ooooo0<_oo`031n3ooooo02koo`008Ooo00<00?ooool0
c?oo00<7h?ooool0;Ooo000Qool00`00ooooo`3=ool00`OPooooo`0/ool0027oo`03003ooooo0<go
o`031n3ooooo02coo`008Ooo0P00cooo00<7h?ooool0:ooo000Qool00`00ooooo`3?ool00`OPoooo
o`0Zool000;oo`8000Coo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00;oo`<000Goo`03
003ooooo0=3oo`031n3ooooo02Woo`0000Goo`00ooooo`0000Woo`03003ooooo00?oo`03003ooooo
00Goo`03003ooooo00;oo`03003ooooo0=7oo`031n3ooooo02Soo`0000Goo`00ooooo`0000[oo`03
003ooooo00?oo`03003ooooo00Coo`03003ooooo00;oo`800=?oo`031n3ooooo02Ooo`0000Goo`00
ooooo`0000[oo`03003ooooo00?oo`04003oooooool30005ool00`00ooooo`3Bool00`OPooooo`0W
ool00005ool00?ooool0000;ool00`00ooooo`03ool01000ooooo`001ooo00<00?ooool0dooo00<7
h?ooool09_oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool00`00oooo
o`05ool00`00ooooo`3Dool00`OPooooo`0Uool000;oo`8000Woo`@000;oo`@000;oo`@000Coo`03
003ooooo0=Goo`031n3ooooo02Coo`008Ooo0P00eooo00<7h?ooool08ooo000Qool00`00ooooo`3G
ool00`OPooooo`0Rool0027oo`03003ooooo0=Soo`031n3ooooo027oo`008Ooo00<00?ooool0fOoo
00<7h?ooool08?oo000Qool00`00ooooo`3Jool00`OPooooo`0Oool0027oo`800=coo`031n3ooooo
01koo`008Ooo00<00?ooool0g?oo00<7h?ooool07Ooo000Qool00`00ooooo`3Mool00`OPooooo`0L
ool0027oo`03003ooooo0=koo`031n3ooooo01_oo`008Ooo00<00?ooool0gooo00<7h?ooool06_oo
000Qool2003Qool00`OPooooo`0Iool0027oo`03003ooooo0>3oo`031n3ooooo01Woo`008Ooo00<0
0?ooool0hOoo00<7h?ooool06?oo000Qool00`00ooooo`3Rool00`OPooooo`0Gool0027oo`800>Co
o`031n3ooooo01Koo`008Ooo00<00?ooool0hooo00<7h?ooool05_oo000Qool00`00ooooo`3Tool0
0`OPooooo`0Eool000Soo`8000Coo`03003ooooo00?oo`03003ooooo00?oo`8000Goo`03003ooooo
0>Goo`031n3ooooo01Coo`001ooo00@00?ooool000Woo`03003ooooo00;oo`04003ooooo0004ool0
0`00ooooo`3Vool00`OPooooo`0Cool000Ooo`04003ooooo000:ool01@00oooooooo00000_oo00<0
0?ooool00_oo0P00iooo00<7h?ooool04ooo0007ool01000ooooo`002_oo00<00?ooool00_oo0P00
1Ooo00<00?ooool0iooo00<7h?ooool04_oo0007ool01000ooooo`002ooo00@00?ooool000;oo`03
003ooooo00;oo`03003ooooo0>Soo`031n3ooooo017oo`001ooo00@00?ooool000Soo`04003ooooo
0002ool01000ooooo`001?oo00<00?ooool0jOoo00<7h?ooool04?oo0008ool20009ool40003ool2
0005ool00`00ooooo`3Zool00`OPooooo`0?ool0027oo`800>_oo`031n3ooooo00ooo`008Ooo00<0
0?ooool0jooo00<7h?ooool03_oo000Qool00`00ooooo`3/ool00`OPooooo`0=ool0027oo`03003o
oooo0>goo`031n3ooooo00coo`008Ooo0P00kooo00<7h?ooool02ooo000Qool00`00ooooo`3_ool0
0`OPooooo`0:ool0027oo`03003ooooo0>ooo`031n3ooooo00[oo`008Ooo00<00?ooool0l?oo00<7
h?ooool02Ooo000Qool00`00ooooo`3aool00`OPooooo`08ool0027oo`800??oo`031n3ooooo00Oo
o`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`00oooo8Ooo
003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0542745, 0.746753, \
0.00163409, 0.000212589}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %1070]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -0.183123 0.524294 [
[.15079 .32867 -9 -9 ]
[.15079 .32867 9 0 ]
[.27778 .32867 -9 -9 ]
[.27778 .32867 9 0 ]
[.40476 .32867 -9 -9 ]
[.40476 .32867 9 0 ]
[.53175 .32867 -9 -9 ]
[.53175 .32867 9 0 ]
[.65873 .32867 -3 -9 ]
[.65873 .32867 3 0 ]
[.78571 .32867 -9 -9 ]
[.78571 .32867 9 0 ]
[.9127 .32867 -9 -9 ]
[.9127 .32867 9 0 ]
[.01131 .0266 -18 -4.5 ]
[.01131 .0266 0 4.5 ]
[.01131 .13145 -18 -4.5 ]
[.01131 .13145 0 4.5 ]
[.01131 .23631 -18 -4.5 ]
[.01131 .23631 0 4.5 ]
[.01131 .44603 -18 -4.5 ]
[.01131 .44603 0 4.5 ]
[.01131 .55089 -18 -4.5 ]
[.01131 .55089 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .34117 m
.15079 .34742 L
s
[(0.2)] .15079 .32867 0 1 Mshowa
.27778 .34117 m
.27778 .34742 L
s
[(0.4)] .27778 .32867 0 1 Mshowa
.40476 .34117 m
.40476 .34742 L
s
[(0.6)] .40476 .32867 0 1 Mshowa
.53175 .34117 m
.53175 .34742 L
s
[(0.8)] .53175 .32867 0 1 Mshowa
.65873 .34117 m
.65873 .34742 L
s
[(1)] .65873 .32867 0 1 Mshowa
.78571 .34117 m
.78571 .34742 L
s
[(1.2)] .78571 .32867 0 1 Mshowa
.9127 .34117 m
.9127 .34742 L
s
[(1.4)] .9127 .32867 0 1 Mshowa
.125 Mabswid
.05556 .34117 m
.05556 .34492 L
s
.0873 .34117 m
.0873 .34492 L
s
.11905 .34117 m
.11905 .34492 L
s
.18254 .34117 m
.18254 .34492 L
s
.21429 .34117 m
.21429 .34492 L
s
.24603 .34117 m
.24603 .34492 L
s
.30952 .34117 m
.30952 .34492 L
s
.34127 .34117 m
.34127 .34492 L
s
.37302 .34117 m
.37302 .34492 L
s
.43651 .34117 m
.43651 .34492 L
s
.46825 .34117 m
.46825 .34492 L
s
.5 .34117 m
.5 .34492 L
s
.56349 .34117 m
.56349 .34492 L
s
.59524 .34117 m
.59524 .34492 L
s
.62698 .34117 m
.62698 .34492 L
s
.69048 .34117 m
.69048 .34492 L
s
.72222 .34117 m
.72222 .34492 L
s
.75397 .34117 m
.75397 .34492 L
s
.81746 .34117 m
.81746 .34492 L
s
.84921 .34117 m
.84921 .34492 L
s
.88095 .34117 m
.88095 .34492 L
s
.94444 .34117 m
.94444 .34492 L
s
.97619 .34117 m
.97619 .34492 L
s
.25 Mabswid
0 .34117 m
1 .34117 L
s
.02381 .0266 m
.03006 .0266 L
s
[(0.4)] .01131 .0266 1 0 Mshowa
.02381 .13145 m
.03006 .13145 L
s
[(0.6)] .01131 .13145 1 0 Mshowa
.02381 .23631 m
.03006 .23631 L
s
[(0.8)] .01131 .23631 1 0 Mshowa
.02381 .44603 m
.03006 .44603 L
s
[(1.2)] .01131 .44603 1 0 Mshowa
.02381 .55089 m
.03006 .55089 L
s
[(1.4)] .01131 .55089 1 0 Mshowa
.125 Mabswid
.02381 .05281 m
.02756 .05281 L
s
.02381 .07902 m
.02756 .07902 L
s
.02381 .10524 m
.02756 .10524 L
s
.02381 .15767 m
.02756 .15767 L
s
.02381 .18388 m
.02756 .18388 L
s
.02381 .2101 m
.02756 .2101 L
s
.02381 .26253 m
.02756 .26253 L
s
.02381 .28874 m
.02756 .28874 L
s
.02381 .31496 m
.02756 .31496 L
s
.02381 .36739 m
.02756 .36739 L
s
.02381 .3936 m
.02756 .3936 L
s
.02381 .41982 m
.02756 .41982 L
s
.02381 .47225 m
.02756 .47225 L
s
.02381 .49846 m
.02756 .49846 L
s
.02381 .52467 m
.02756 .52467 L
s
.02381 .00038 m
.02756 .00038 L
s
.02381 .5771 m
.02756 .5771 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.03016 .21049 m
.03249 .2105 L
.03505 .21051 L
.0402 .21055 L
.04588 .2106 L
.05116 .21067 L
.0613 .21083 L
.07207 .21107 L
.08245 .21134 L
.09244 .21166 L
.10307 .21206 L
.1133 .21249 L
.12314 .21296 L
.13361 .21352 L
.14369 .21411 L
.15441 .21479 L
.16473 .21551 L
.17466 .21626 L
.18523 .21711 L
.19541 .21799 L
.20622 .21899 L
.21663 .22002 L
.22666 .22106 L
.23732 .22224 L
.24759 .22344 L
.25746 .22465 L
.26797 .22602 L
.27778 .22735 L
s
1 0 0 r
.03016 .1407 m
.03323 .14078 L
.03608 .14088 L
.03922 .14102 L
.04257 .1412 L
.04942 .14169 L
.05566 .14226 L
.06995 .14402 L
.08347 .14625 L
.10882 .15182 L
.13579 .15958 L
.16176 .16859 L
.18674 .1785 L
.21332 .19018 L
.23891 .20237 L
.26612 .21619 L
.27971 .22338 L
.28633 .22695 L
.28785 .22774 L
.28946 .22859 L
.29012 .22887 L
.29083 .22911 L
.2916 .22929 L
.29233 .22943 L
.29535 .22989 L
.29861 .23038 L
.30453 .23131 L
.31754 .23341 L
.34484 .23823 L
.37113 .24342 L
.39643 .24896 L
.42335 .2555 L
.44926 .26248 L
.47418 .26989 L
.50071 .27862 L
.52625 .28793 L
.5534 .29892 L
.57955 .31068 L
.6047 .32318 L
.63147 .33785 L
.65724 .35336 L
.65873 .3543 L
s
0 g
.24557 0 m
.26647 .01726 L
.30764 .05126 L
.34731 .08401 L
.38546 .11552 L
.42606 .14904 L
.46515 .18132 L
.50273 .21235 L
.54275 .2454 L
.58126 .2772 L
.62221 .31102 L
.66166 .34359 L
.69959 .37492 L
.73997 .40825 L
.77884 .44035 L
.82014 .47446 L
.85994 .50732 L
.89823 .53895 L
.93896 .57258 L
.97619 .60332 L
s
.03016 .2106 m
.03133 .21065 L
.03239 .2107 L
.03362 .21051 L
.03477 .21051 L
.03585 .21091 L
.03684 .21098 L
.03908 .21116 L
.04372 .21163 L
.04884 .21227 L
.0584 .21382 L
.06854 .21589 L
.11039 .22747 L
.15041 .24049 L
.18892 .25309 L
.22987 .26588 L
.26932 .2774 L
.30725 .28767 L
.34763 .29773 L
.3865 .30656 L
.4278 .31501 L
.4676 .3222 L
.5059 .32819 L
.54663 .33345 L
.56691 .3356 L
.58585 .3373 L
.60522 .33872 L
.62357 .33976 L
.63325 .34018 L
.64379 .34055 L
.64843 .34068 L
.65336 .3408 L
.65767 .34088 L
.66222 .34095 L
.66454 .34098 L
.66707 .34101 L
.66941 .34103 L
.67153 .34105 L
.67273 .34105 L
.67404 .34106 L
.67476 .34106 L
.67542 .34106 L
.67672 .34107 L
.67784 .34107 L
.67907 .34107 L
.68036 .34107 L
.68156 .34107 L
.68268 .34106 L
.68391 .34106 L
.68613 .34105 L
Mistroke
.6883 .34104 L
.69031 .34102 L
.69487 .34097 L
.69972 .34091 L
.70996 .34072 L
.72103 .34044 L
.74107 .33979 L
.78186 .33804 L
.82113 .33614 L
.8589 .33433 L
.89911 .33256 L
.93781 .33102 L
.97619 .32968 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool2000jool00`00ooooo`3<ool001Goo`03003ooooo03[oo`03
003ooooo0<_oo`005Ooo00<00?ooool0>ooo00<00?ooool0b_oo000Eool00`00ooooo`0lool2003:
ool001Goo`03003ooooo03koo`03003ooooo0<Ooo`000_oo0P001?oo00<00?ooool01?oo00<00?oo
ool00ooo00<00?ooool0?ooo00<00?ooool0a_oo00001Ooo003ooooo00002_oo00<00?ooool00ooo
00<00?ooool0@?oo00<00?ooool0aOoo00001Ooo003ooooo00001ooo1@001?oo0`00@Ooo0P00aOoo
00001Ooo003ooooo00001ooo00@00?ooool000Goo`03003ooooo04?oo`03003ooooo0<;oo`0000Go
o`00ooooo`0000Soo`03003oo`0000Goo`03003ooooo04Coo`03003ooooo0<7oo`0000Goo`00oooo
o`0000Woo`8000Goo`03003ooooo04Goo`03003ooooo0<3oo`000_oo0P002ooo00<00?ooool00ooo
00<00?ooool0A_oo00<00?ooool0_ooo000Eool00`00ooooo`17ool2002oool001Goo`03003ooooo
04Woo`03003ooooo0;coo`005Ooo0P00Booo00<00?ooool0^ooo000Eool00`00ooooo`1;ool00`00
ooooo`2jool001Goo`03003ooooo04coo`800;[oo`005Ooo00<00?ooool0C_oo00<00?ooool0]ooo
000Eool00`00ooooo`1?ool00`00ooooo`2fool001Goo`03003ooooo053oo`03003ooooo0;Goo`00
5Ooo00<00?ooool0DOoo00<00?ooool0]?oo000Eool2001Cool2002dool001Goo`03003ooooo05Co
o`03003ooooo0;7oo`005Ooo00<00?ooool0EOoo00<00?ooool0/?oo000Eool00`00ooooo`1Fool0
0`00ooooo`2_ool001Goo`03003ooooo05Ooo`800:ooo`005Ooo00<00?ooool0FOoo00<00?ooool0
[?oo000Eool00`00ooooo`1Jool00`00ooooo`2[ool001Goo`03003ooooo05_oo`03003ooooo0:[o
o`005Ooo0P00GOoo0P00Z_oo000Eool00`00ooooo`1Nool00`00ooooo`2Wool001Goo`03003ooooo
05ooo`03003ooooo0:Koo`005Ooo00<00?ooool0H?oo00<00?ooool0YOoo000Eool00`00ooooo`1Q
ool00`00ooooo`2Tool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo06;oo`03003o
oooo0:?oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1Sool00`00ooooo`2R
ool00005ool00?ooool00008ool01000ooooo`001?oo0`00I?oo0P00X_oo00001Ooo003ooooo0000
2?oo0`001Ooo00<00?ooool0I_oo00<00?ooool0Wooo00001Ooo003ooooo00002?oo00<00?ooool0
1Ooo00<00?oon0000oP0I?oo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo
00<00?ooool00ooo2?P0GOoo00<00?ooool0WOoo0002ool2000:ool30004ool00`00ooooo`0;ool6
n01Hool00`00ooooo`2Lool001Goo`03003ooooo017oo`Gh05Coo`03003ooooo09_oo`005Ooo00<0
0?ooool05_oo1?P0DOoo0P00Vooo000Eool2000Kool3n01@ool00`00ooooo`2Hool001Goo`03003o
oooo01goo`;h04ooo`03003ooooo09Ooo`005Ooo00<00?ooool07ooo0_P0C_oo0P00Uooo000Eool0
0`00ooooo`0Qool3n01=ool00`00ooooo`2Dool001Goo`03003ooooo02Coo`;h04coo`03003ooooo
09?oo`005Ooo00<00?ooool09_oo0_P0Booo0P00Tooo000Eool00`00ooooo`0Xool3n01:ool00`00
ooooo`2@ool001Goo`8002coo`;h04Woo`03003ooooo08ooo`005Ooo00<00?ooool0;Ooo0_P0B?oo
00<00?ooool0S_oo000Eool00`00ooooo`0_ool3n016ool00`00ooooo`2=ool001Goo`03003ooooo
03;oo`;h04Goo`03003ooooo08coo`005Ooo00<00?ooool0=?oo0_P0A?oo0P00S?oo000Eool00`00
ooooo`0fool2n014ool00`00ooooo`29ool001Goo`03003ooooo03Soo`;h04?oo`03003ooooo08So
o`005Ooo1@003@OP:ooo0_P0@_oo00<00?ooool0Qooo000Eool00`00ooooo`02ool60007oolC1n0J
ool2n011ool00`00ooooo`26ool001Goo`03003ooooo00Soo`<001Ooo``7h13oo`;h043oo`03003o
oooo08Goo`005Ooo00<00?ooool02ooo10007ooo2`OP1ooo0_P0?ooo0P00QOoo000Eool00`00oooo
o`0?ool4000Vool81n000ooon03h000oool00`00ooooo`22ool000;oo`8000Coo`03003ooooo00?o
o`8000Goo`03003ooooo01?oo`<002_oo`87h004oooh0?P0n00mool00`00ooooo`21ool00005ool0
0?ooool00008ool01000ooooo`001?oo00<00?ooool05_oo1000;Ooo2?P0=_oo00<00?ooool0P?oo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`<001[oo`@0037oo`Ch03?oo`03003ooooo07oo
o`0000Goo`00ooooo`0000Woo`8000Goo`03003ooooo01koo`<003;oo`Gh02ooo`8007ooo`0000Go
o`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0Qool3000dool5n00/ool00`00ooooo`1l
ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool09?oo0P00=ooo1?P0:Ooo00<0
0?ooool0Nooo0002ool2000:ool20005ool00`00ooooo`0Vool3000hool5n00Uool2001kool001Go
o`03003ooooo02Woo`<003[oo`Kh027oo`03003ooooo07Soo`005Ooo00<00?ooool0;?oo1000??oo
1?P07_oo00<00?ooool0Mooo000Eool2000aool4000lool3n00Lool2001gool001Goo`03003ooooo
03Coo`<003coo`?h01_oo`03003ooooo07Coo`005Ooo00<00?ooool0=ooo1000>ooo0oP06Ooo00<0
0?ooool0Looo000Eool00`00ooooo`0kool4000jool2n00Hool2001cool001Goo`03003ooooo03oo
o`<003Woo`?h01Ooo`03003ooooo073oo`005Ooo00<00?ooool0@_oo0`00>Ooo0oP05Ooo00<00?oo
ool0Kooo000Eool00`00ooooo`15ool4000hool4n00Bool00`00ooooo`1^ool001Goo`03003ooooo
04Woo`<003Woo`?h013oo`03003ooooo06goo`005Ooo0P00COoo1000>?oo0_P03ooo00<00?ooool0
K?oo000Eool00`00ooooo`1@ool4000fool2n00>ool2001/ool001Goo`03003ooooo05Coo`@003Co
o`?h00goo`03003ooooo06Woo`005Ooo00<00?ooool06?oo0P001?oo00<00?ooool00_oo10005?oo
0P001?oo00<00?ooool01?oo00<00?ooool03Ooo20001?oo00<00?ooool00ooo0P005?oo0P001?oo
00<00?ooool00ooo0P0000?oooP0n0003?oo00<00?ooool02?oo10006?oo10000ooo00<00?ooool0
0_oo10004ooo10000ooo00<00?ooool01?oo00<00?ooool04?oo000Eool00`00ooooo`0Gool01000
ooooo`002?oo00<00?ooool05?oo00@00?ooool000[oo`03003ooooo01;oo`D000Ooo`04003ooooo
000Bool01000ooooo`002?oo00@00?ooool000;oo`;h00_oo`03003ooooo00Woo`03003ooooo01Wo
o`03003ooooo00Ooo`03003ooooo01Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo00<00?oo
ool05ooo00@00?ooool000Woo`03003ooooo01?oo`04003ooooo0007ool5000Cool01@00ooooo`00
ool010000ooo00@00?ooool001;oo`04003ooooo0008ool01000ooooo`001?oo0_P02_oo00<00?oo
ool02?oo00<00?ooool06Ooo00<00?ooool02?oo00<00?ooool05Ooo00<00?ooool01_oo1@004Ooo
000Eool00`00ooooo`0Gool01000ooooo`002_oo00<00?ooool04_oo00@00?ooool000Ooo`04003o
oooo000Dool01000ooooo`001Ooo1P004ooo00@00?ooool000Woo`8000Ooo`;h00Woo`03003ooooo
00Ooo`03003ooooo01Woo`03003ooooo00Woo`03003ooooo01Coo`03003ooooo00Koo`04003ooooo
000Bool001Goo`8001Soo`04003ooooo000;ool00`00ooooo`0Aool01000ooooo`002?oo00<00?oo
00005?oo00@00?ooool000Soo`D0017oo`04003ooooo0008ool01000ooooo`002?oo0_P02?oo00<0
0?ooool01_oo00<00?ooool06Ooo00<00?ooool02_oo00<00?ooool04ooo00<00?ooool01ooo00<0
0?oo00004_oo000Eool00`00ooooo`0Gool01000ooooo`002?oo00@00?ooool001?oo`04003ooooo
0009ool2000Dool01000ooooo`002?oo00<00?ooool00_oo1P002ooo00@00?ooool000Soo`04003o
oooo000:ool2n007ool20004ool3000Iool30009ool01000ooooo`004ooo0`002_oo0P004_oo000E
ool00`00ooooo`0Hool2000:ool2000Eool2000;ool00`00ooooo`0Cool2000:ool30007ool70005
ool2000:ool2000=ool00oP0ooooo`06ool00`00ooooo`03ool00`00ooooo`0Iool00`00ooooo`08
ool2000Fool00`00ooooo`09ool00`00ooooo`0@ool001Goo`03003ooooo07_oo`P001Woo`;h00Oo
o`03003ooooo063oo`005Ooo00<00?ooool0Pooo1P005Ooo0_P01_oo00<00?ooool0?Ooo6`001ooo
000Eool00`00ooooo`29ool;000<ool2n005ool2000XoolE000Rool001Goo`03003ooooo09Coo`h0
0003n03ooooo00Coo`03003ooooo01;ooa<003Ooo`003_ooo`004`00000Eool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool01000oooooooo0_P00ooo0P001_oo00<0
0?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo000E
ool00`00ooooo`2Wool2n003ool2001Jool001Goo`03003ooooo0:Woo`;h00?oo`03003ooooo05Oo
o`005Ooo00<00?ooool0[ooo00<00?ooool0E_oo000Eool00`00ooooo`2`ool00`00ooooo`1Eool0
01Goo`03003ooooo0;7oo`03003ooooo05Coo`005Ooo0P00/ooo0P00E?oo000Eool00`00ooooo`2d
ool00`00ooooo`1Aool001Goo`03003ooooo0;Goo`03003ooooo053oo`005Ooo00<00?ooool0]_oo
00<00?ooool0Cooo000Eool00`00ooooo`2gool2001?ool001Goo`03003ooooo0;Woo`03003ooooo
04coo`005Ooo00<00?ooool0^_oo00<00?ooool0Booo000Eool2002lool00`00ooooo`1:ool001Go
o`03003ooooo0;coo`03003ooooo04Woo`005Ooo00<00?ooool0_Ooo0P00BOoo000Eool00`00oooo
o`2oool00`00ooooo`16ool001Goo`03003ooooo0<3oo`03003ooooo04Goo`005Ooo00<00?ooool0
`Ooo00<00?ooool0A?oo000Eool00`00ooooo`32ool00`00ooooo`13ool001Goo`800<Coo`03003o
oooo04;oo`005Ooo00<00?ooool0a?oo00<00?ooool0@Ooo000Eool00`00ooooo`35ool20011ool0
01Goo`03003ooooo0<Ooo`03003ooooo03koo`005Ooo00<00?ooool0b?oo00<00?ooool0?Ooo0000
0ooo000000020003ool00`00ooooo`02ool40004ool00`00ooooo`39ool00`00ooooo`0lool000?o
o`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?oo
ool02?oo00<00?ooool01?oo0`00booo00<00?ooool0>_oo0003ool00`00ooooo`09ool00`00oooo
o`03ool00`00ooooo`3<ool2000jool000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo
0<koo`03003ooooo03Ooo`0000Coo`00000000Woo`04003ooooo0004ool00`00ooooo`3?ool00`00
ooooo`0fool000?oo`03003ooooo00Soo`8000Goo`03003ooooo0=3oo`8003Koo`005Ooo00<00?oo
ool0d_oo00<00?ooool0<ooo000Eool00`00ooooo`3Cool00`00ooooo`0bool001Goo`800=Goo`80
03;oo`005Ooo00<00?ooool0e_oo00<00?ooool0;ooo000Eool00`00ooooo`3Gool00`00ooooo`0^
ool001Goo`03003ooooo0=Soo`03003ooooo02goo`005Ooo00<00?ooool0fOoo00<00?ooool0;?oo
000Eool00`00ooooo`3Jool00`00ooooo`0[ool001Goo`03003ooooo0=_oo`8002_oo`005Ooo00<0
0?ooool0gOoo00<00?ooool0:?oo000Eool2003Oool00`00ooooo`0Wool001Goo`03003ooooo0=oo
o`03003ooooo02Koo`005Ooo00<00?ooool0h?oo00<00?ooool09Ooo000Eool00`00ooooo`3Qool0
0`00ooooo`0Tool001Goo`03003ooooo0>;oo`8002Coo`005Ooo00<00?ooool0i?oo00<00?ooool0
8Ooo000Eool00`00ooooo`3Uool00`00ooooo`0Pool001Goo`800>Ooo`03003ooooo01ooo`005Ooo
00<00?ooool0iooo00<00?ooool07_oo000Eool00`00ooooo`3Xool2000Nool001Goo`03003ooooo
0>[oo`03003ooooo01_oo`005Ooo00<00?ooool0jooo00<00?ooool06_oo00000ooo000000020003
ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`3/ool00`00ooooo`0Iool000?oo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo0>goo`8001Woo`000ooo00<00?ooool01_oo1@001?oo
0`00kooo00<00?ooool05_oo0003ool00`00ooooo`06ool01000ooooo`001Ooo00<00?ooool0l?oo
00<00?ooool05Ooo0003ool00`00ooooo`07ool00`00ool00005ool00`00ooooo`3aool00`00oooo
o`0Dool00004ool00000000:ool20005ool00`00ooooo`3bool00`00ooooo`0Cool000?oo`03003o
oooo00Woo`03003ooooo00?oo`03003ooooo0??oo`8001?oo`005Ooo00<00?ooool0mOoo00<00?oo
ool04?oo000Eool00`00ooooo`3fool00`00ooooo`0?ool001Goo`800?Soo`03003ooooo00koo`00
5Ooo00<00?ooool0n?oo0P003_oo000Eool00`00ooooo`3jool00`00ooooo`0;ool001Goo`03003o
oooo0?_oo`03003ooooo00[oo`005Ooo00<00?ooool0o?oo00<00?ooool02Ooo000Eool00`00oooo
o`3mool20009ool001Goo`03003ooooo0?ooo`03003ooooo00Koo`005Ooo0P00oooo0_oo00<00?oo
ool01Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0
oooo2Ooo000Eool00`00ooooo`3oool9ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, 0.318475, \
0.00578885, 0.00701031}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{mGamma /. strangesol[k, 10]}, {k, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.603319 1.21817 [
[.15079 .59082 -9 -9 ]
[.15079 .59082 9 0 ]
[.27778 .59082 -9 -9 ]
[.27778 .59082 9 0 ]
[.40476 .59082 -9 -9 ]
[.40476 .59082 9 0 ]
[.53175 .59082 -9 -9 ]
[.53175 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.78571 .59082 -9 -9 ]
[.78571 .59082 9 0 ]
[.9127 .59082 -9 -9 ]
[.9127 .59082 9 0 ]
[.01131 .11605 -24 -4.5 ]
[.01131 .11605 0 4.5 ]
[.01131 .23787 -24 -4.5 ]
[.01131 .23787 0 4.5 ]
[.01131 .35969 -24 -4.5 ]
[.01131 .35969 0 4.5 ]
[.01131 .4815 -24 -4.5 ]
[.01131 .4815 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .60332 m
.15079 .60957 L
s
[(0.2)] .15079 .59082 0 1 Mshowa
.27778 .60332 m
.27778 .60957 L
s
[(0.4)] .27778 .59082 0 1 Mshowa
.40476 .60332 m
.40476 .60957 L
s
[(0.6)] .40476 .59082 0 1 Mshowa
.53175 .60332 m
.53175 .60957 L
s
[(0.8)] .53175 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(1)] .65873 .59082 0 1 Mshowa
.78571 .60332 m
.78571 .60957 L
s
[(1.2)] .78571 .59082 0 1 Mshowa
.9127 .60332 m
.9127 .60957 L
s
[(1.4)] .9127 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.18254 .60332 m
.18254 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.34127 .60332 m
.34127 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.81746 .60332 m
.81746 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.97619 .60332 m
.97619 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .11605 m
.03006 .11605 L
s
[(-0.4)] .01131 .11605 1 0 Mshowa
.02381 .23787 m
.03006 .23787 L
s
[(-0.3)] .01131 .23787 1 0 Mshowa
.02381 .35969 m
.03006 .35969 L
s
[(-0.2)] .01131 .35969 1 0 Mshowa
.02381 .4815 m
.03006 .4815 L
s
[(-0.1)] .01131 .4815 1 0 Mshowa
.125 Mabswid
.02381 .0186 m
.02756 .0186 L
s
.02381 .04296 m
.02756 .04296 L
s
.02381 .06733 m
.02756 .06733 L
s
.02381 .09169 m
.02756 .09169 L
s
.02381 .14042 m
.02756 .14042 L
s
.02381 .16478 m
.02756 .16478 L
s
.02381 .18914 m
.02756 .18914 L
s
.02381 .21351 m
.02756 .21351 L
s
.02381 .26223 m
.02756 .26223 L
s
.02381 .2866 m
.02756 .2866 L
s
.02381 .31096 m
.02756 .31096 L
s
.02381 .33532 m
.02756 .33532 L
s
.02381 .38405 m
.02756 .38405 L
s
.02381 .40841 m
.02756 .40841 L
s
.02381 .43278 m
.02756 .43278 L
s
.02381 .45714 m
.02756 .45714 L
s
.02381 .50587 m
.02756 .50587 L
s
.02381 .53023 m
.02756 .53023 L
s
.02381 .55459 m
.02756 .55459 L
s
.02381 .57896 m
.02756 .57896 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03016 .59094 m
.06854 .51944 L
.11039 .45381 L
.1497 .4043 L
.1875 .36523 L
.22774 .33046 L
.26647 .302 L
.30764 .27595 L
.34731 .25415 L
.38546 .23576 L
.42606 .21858 L
.46515 .20409 L
.50273 .19183 L
.54275 .18033 L
.58126 .17043 L
.62221 .1606 L
.66166 .15098 L
.69959 .14071 L
.73997 .12785 L
.77884 .11321 L
.82014 .09539 L
.85994 .07642 L
.89823 .05691 L
.93896 .0352 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3iool20007ool001_oo`800?Soo`8000Woo`006ooo00<00?ooool0mOoo0P002ooo000K
ool00`00ooooo`3cool2000=ool001_oo`03003ooooo0?7oo`8000ooo`006ooo00<00?ooool0l?oo
00<00?ooool03ooo000Kool00`00ooooo`3^ool2000Bool001_oo`800>goo`8001Coo`006ooo00<0
0?ooool0j_oo0P005_oo000Kool00`00ooooo`3Xool2000Hool001_oo`03003ooooo0>Koo`8001[o
o`006ooo00<00?ooool0i?oo0P007?oo000Kool00`00ooooo`3Rool2000Nool001_oo`03003ooooo
0>3oo`80023oo`006ooo0P00gooo0P008_oo000Kool00`00ooooo`3Lool2000Tool001_oo`03003o
oooo0=[oo`8002Koo`006ooo00<00?ooool0f?oo0P00:?oo000Kool00`00ooooo`3Fool2000Zool0
01_oo`03003ooooo0=Coo`8002coo`006ooo0P00dooo0P00;_oo000Kool00`00ooooo`3@ool2000`
ool001_oo`03003ooooo0<koo`8003;oo`006ooo00<00?ooool0c?oo0P00=?oo000Kool00`00oooo
o`3:ool2000fool000Soo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo0<Soo`80
03Soo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo0<Goo`<003[oo`001ooo00@0
0?ooool000Ooo`D000Coo`800<?oo`<003goo`0000?oo`0000000P000_oo00@00?ooool000Ooo`04
003ooooo0005ool00`00ooooo`30ool20010ool000Ooo`04003ooooo0008ool00`00ool00005ool0
0`00ooooo`2mool30012ool000Ooo`04003ooooo0009ool20005ool00`00ooooo`2jool30015ool0
00Soo`8000_oo`03003ooooo00?oo`03003ooooo0;Koo`@004Soo`006ooo00<00?ooool0/ooo0`00
C?oo000Kool2002aool3001?ool001_oo`03003ooooo0:coo`@005;oo`006ooo00<00?ooool0ZOoo
0`00E_oo000Kool00`00ooooo`2Vool3001Iool001_oo`03003ooooo0:;oo`@005coo`006ooo00<0
0?ooool0W_oo1000H?oo000Kool00`00ooooo`2Jool4001Tool001_oo`8009Goo`H006Soo`006ooo
00<00?ooool0T?oo1000K_oo000Kool00`00ooooo`2<ool4001bool001_oo`03003ooooo08Woo`<0
07Koo`006ooo00<00?ooool0Q_oo0`00NOoo000Kool00`00ooooo`22ool4001lool001_oo`8007oo
o`@0083oo`006ooo00<00?ooool0Nooo0`00Q?oo000Kool00`00ooooo`1gool40027ool001_oo`03
003ooooo07Coo`<008_oo`006ooo00<00?ooool0LOoo0`00S_oo000Kool00`00ooooo`1_ool2002A
ool001_oo`03003ooooo06coo`<009?oo`006ooo0P00Jooo0P00U_oo000Kool00`00ooooo`1Wool3
002Hool001_oo`03003ooooo06Goo`8009_oo`006ooo00<00?ooool0Hooo0P00WOoo0008ool20004
ool00`00ooooo`03ool20005ool00`00ooooo`1Qool2002Oool000Ooo`04003ooooo0008ool01000
ooooo`001?oo00<00?ooool0Gooo0P00XOoo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00
GOoo0`00Xooo00000ooo000000020002ool01000ooooo`002Ooo0P001Ooo00<00?ooool0F_oo0P00
Y_oo0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0Eooo0`00Z?oo0007ool01000
ooooo`002?oo00@00?ooool000Coo`03003ooooo05Goo`800:_oo`002?oo0P002_oo0P001Ooo00<0
0?ooool0Dooo0P00[Ooo000Kool00`00ooooo`1Aool2002_ool001_oo`03003ooooo04ooo`800;7o
o`006ooo0P00C_oo0P00/ooo000Kool00`00ooooo`1;ool2002eool001_oo`03003ooooo04Woo`80
0;Ooo`006ooo00<00?ooool0B?oo00<00?ooool0]ooo000Kool00`00ooooo`16ool2002jool001_o
o`03003ooooo04Goo`03003ooooo0;[oo`006ooo0P00A?oo0P00_Ooo000Kool00`00ooooo`11ool2
002oool001_oo`03003ooooo043oo`03003ooooo0;ooo`006ooo00<00?ooool0?_oo0P00`_oo000K
ool00`00ooooo`0mool00`00ooooo`32ool001_oo`03003ooooo03coo`03003ooooo0<?oo`006ooo
00<00?ooool0>_oo0P00a_oo000Kool2000jool00`00ooooo`36ool001_oo`03003ooooo03Soo`03
003ooooo0<Ooo`006ooo00<00?ooool0=ooo00<00?ooool0b?oo000Kool00`00ooooo`0eool2003;
ool001_oo`03003ooooo03Coo`03003ooooo0<_oo`006ooo00<00?ooool0<ooo00<00?ooool0c?oo
000Kool2000cool00`00ooooo`3=ool001_oo`03003ooooo033oo`800=3oo`006ooo00<00?ooool0
;ooo00<00?ooool0d?oo000Kool00`00ooooo`0^ool00`00ooooo`3Aool001_oo`03003ooooo02go
o`03003ooooo0=;oo`002?oo0P001?oo00<00?ooool00_oo10001?oo00<00?ooool0;?oo00<00?oo
ool0dooo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0:_oo0P00e_oo0007ool0
1000ooooo`002Ooo00<00?ooool01?oo0P00:_oo00<00?ooool0e_oo00000ooo000000020002ool0
1000ooooo`002_oo00<00?ooool00ooo00<00?ooool0:?oo00<00?ooool0eooo0007ool01000oooo
o`002ooo00<00?ooool00_oo00<00?ooool09ooo00<00?ooool0f?oo0007ool01000ooooo`002?oo
00@00?ooool000Coo`03003ooooo02Koo`03003ooooo0=Woo`002?oo0P002_oo0P001Ooo00<00?oo
ool09Ooo00<00?ooool0f_oo000Kool00`00ooooo`0Tool00`00ooooo`3Kool001_oo`8002Coo`03
003ooooo0=coo`006ooo00<00?ooool08_oo00<00?ooool0gOoo000Kool00`00ooooo`0Qool00`00
ooooo`3Nool001_oo`03003ooooo023oo`03003ooooo0=ooo`006ooo00<00?ooool07ooo00<00?oo
ool0h?oo000Kool00`00ooooo`0Nool00`00ooooo`3Qool001_oo`8001koo`03003ooooo0>;oo`00
6ooo00<00?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Lool00`00ooooo`3Sool001_o
o`03003ooooo01_oo`03003ooooo0>Coo`006ooo00<00?ooool06_oo00<00?ooool0iOoo000Kool0
0`00ooooo`0Jool00`00ooooo`3Uool001_oo`03003ooooo01Woo`03003ooooo0>Koo`006ooo0P00
6Ooo00<00?ooool0iooo000Kool00`00ooooo`0Hool00`00ooooo`3Wool001_oo`03003ooooo01Oo
o`03003ooooo0>Soo`006ooo00<00?ooool05_oo00<00?ooool0jOoo000Kool00`00ooooo`0Eool0
0`00ooooo`3Zool001_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo0P005Ooo00<00?ooool0
jooo000Kool00`00ooooo`0Cool00`00ooooo`3/ool001_oo`03003ooooo01?oo`03003ooooo0>co
o`006ooo00<00?ooool04_oo00<00?ooool0kOoo000Kool00`00ooooo`0Aool00`00ooooo`3^ool0
00Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo017oo`03003ooooo0>koo`001ooo00@0
0?ooool000[oo`03003ooooo00?oo`03003ooooo013oo`03003ooooo0>ooo`001ooo00@00?ooool0
00[oo`03003ooooo00?oo`80013oo`03003ooooo0?3oo`0000?oo`0000000P000_oo00@00?ooool0
00[oo`03003ooooo00?oo`03003ooooo00ooo`03003ooooo0?3oo`001ooo00@00?ooool000[oo`03
003ooooo00?oo`03003ooooo00koo`03003ooooo0?7oo`001ooo00@00?ooool000Soo`<000Goo`03
003ooooo00koo`03003ooooo0?7oo`002?oo0P002ooo00<00?ooool00ooo00<00?ooool03Ooo00<0
0?ooool0l_oo000Kool00`00ooooo`0<ool00`00ooooo`3cool001_oo`8000goo`03003ooooo0??o
o`006ooo00<00?ooool02ooo00<00?ooool0m?oo000Kool00`00ooooo`0:ool00`00ooooo`3eool0
01_oo`03003ooooo00[oo`03003ooooo0?Goo`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000K
ool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo
0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo
00Koo`03003ooooo0?Woo`006ooo00<00?ooool01_oo00<00?ooool0nOoo000Kool00`00ooooo`05
ool00`00ooooo`3jool001_oo`03003ooooo00Goo`03003ooooo0?[oo`006ooo0P001Ooo00<00?oo
ool0nooo000Kool00`00ooooo`04ool00`00ooooo`3kool001_oo`03003ooooo00?oo`03003ooooo
0?coo`006ooo00<00?ooool00ooo00<00?ooool04_oo0P001?oo00<00?ooool00_oo10004_oo0P00
1?oo00<00?ooool01?oo00<00?ooool04_oo0P001?oo00<00?ooool00ooo0P005?oo0P001?oo00<0
0?ooool00ooo0P006Ooo10005ooo10000ooo00<00?ooool00_oo10004_oo10000ooo00<00?ooool0
1?oo00<00?ooool04?oo000Kool00`00ooooo`02ool00`00ooooo`0Bool01000ooooo`002?oo00<0
0?ooool04_oo00@00?ooool000[oo`03003ooooo017oo`04003ooooo0008ool01000ooooo`004_oo
00@00?ooool000Soo`04003ooooo000Jool00`00ooooo`0Hool00`00ooooo`07ool00`00ooooo`0E
ool00`00ooooo`09ool00`00ooooo`0@ool001_oo`03003ooooo00;oo`03003ooooo01;oo`04003o
oooo0009ool00`00ooooo`0Aool01000ooooo`001ooo1@004_oo00@00?ooool000Soo`04003ooooo
000Bool01000ooooo`002?oo00@00?ooool001[oo`03003ooooo01Soo`03003ooooo00Soo`03003o
oooo01Coo`03003ooooo00Koo`D0017oo`006ooo00D00?ooooooo`0001Goo`04003ooooo000:ool0
0`00ooooo`0@ool01000ooooo`001ooo00@00?ooool001?oo`04003ooooo0008ool3000Cool01000
ooooo`002Ooo0P006ooo00<00?ooool06?oo00<00?ooool02Ooo00<00?ooool04ooo00<00?ooool0
1_oo00@00?ooool001;oo`006ooo0P000_oo00<00?ooool04ooo00@00?ooool000_oo`03003ooooo
00ooo`04003ooooo0008ool00`00ool0000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?oo
ool000Soo`04003ooooo000Jool00`00ooooo`0Hool00`00ooooo`0:ool00`00ooooo`0Bool00`00
ooooo`07ool00`00ool0000Bool001_oo`04003ooooo000Fool01000ooooo`002?oo00@00?ooool0
017oo`04003ooooo0009ool2000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool000So
o`04003ooooo000Hool3000Hool30009ool01000ooooo`004_oo0`002_oo0P004_oo000Kool01000
ooooo`005ooo0P002_oo0P004ooo0P002ooo00<00?ooool04_oo0P002_oo0`004ooo0P002_oo0P00
6ooo00<00?ooool06?oo00<00?ooool02?oo0P005Ooo00<00?ooool02Ooo00<00?ooool04?oo000K
ool00`00ool0003oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000D
oooo000=000001_oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Goo`006ooo00<00?ooool07ooo00<00?ooool07_oo00<00?oo
ool07ooo00<00?ooool07ooo00<00?ooool07ooo00<00?ooool07_oo00<00?ooool07ooo00<00?oo
ool05_oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, -0.514613, \
0.00592092, 0.00308603}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9994820046162144`, 
      mGamma \[Rule] \(-0.3707184166384133`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.001, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.7507348023467588`, 
      mGamma \[Rule] \(-0.0010167970670121491`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.001, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.7507331914045487`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[0.7507331914045487`, 0.001, 10]\)], "Input"],

Cell[BoxData[
    \(2.0805037335362946`*^-9 + 0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[
      0.7507352528464437476 + I \((\(-0.001016796662459896262083\))\), 0.001, 
      10]\)], "Input"],

Cell[BoxData[
    \(9.931444982616892`*^-22 + 
      7.905253850325371`*^-20\ \[ImaginaryI]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["xi=100", "Subsubsection"],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.75, 
          0.8}, {mGamma, \(-0.1\), \(-0.2\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, Om /. strangesol[k, 100]}, {k, 0.01, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -1.12792 1.63373 [
[.15079 .49331 -9 -9 ]
[.15079 .49331 9 0 ]
[.27778 .49331 -9 -9 ]
[.27778 .49331 9 0 ]
[.40476 .49331 -9 -9 ]
[.40476 .49331 9 0 ]
[.53175 .49331 -9 -9 ]
[.53175 .49331 9 0 ]
[.65873 .49331 -3 -9 ]
[.65873 .49331 3 0 ]
[.78571 .49331 -9 -9 ]
[.78571 .49331 9 0 ]
[.9127 .49331 -9 -9 ]
[.9127 .49331 9 0 ]
[.01131 .01569 -18 -4.5 ]
[.01131 .01569 0 4.5 ]
[.01131 .09738 -24 -4.5 ]
[.01131 .09738 0 4.5 ]
[.01131 .17907 -18 -4.5 ]
[.01131 .17907 0 4.5 ]
[.01131 .26075 -24 -4.5 ]
[.01131 .26075 0 4.5 ]
[.01131 .34244 -18 -4.5 ]
[.01131 .34244 0 4.5 ]
[.01131 .42413 -24 -4.5 ]
[.01131 .42413 0 4.5 ]
[.01131 .5875 -24 -4.5 ]
[.01131 .5875 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .50581 m
.15079 .51206 L
s
[(0.2)] .15079 .49331 0 1 Mshowa
.27778 .50581 m
.27778 .51206 L
s
[(0.4)] .27778 .49331 0 1 Mshowa
.40476 .50581 m
.40476 .51206 L
s
[(0.6)] .40476 .49331 0 1 Mshowa
.53175 .50581 m
.53175 .51206 L
s
[(0.8)] .53175 .49331 0 1 Mshowa
.65873 .50581 m
.65873 .51206 L
s
[(1)] .65873 .49331 0 1 Mshowa
.78571 .50581 m
.78571 .51206 L
s
[(1.2)] .78571 .49331 0 1 Mshowa
.9127 .50581 m
.9127 .51206 L
s
[(1.4)] .9127 .49331 0 1 Mshowa
.125 Mabswid
.05556 .50581 m
.05556 .50956 L
s
.0873 .50581 m
.0873 .50956 L
s
.11905 .50581 m
.11905 .50956 L
s
.18254 .50581 m
.18254 .50956 L
s
.21429 .50581 m
.21429 .50956 L
s
.24603 .50581 m
.24603 .50956 L
s
.30952 .50581 m
.30952 .50956 L
s
.34127 .50581 m
.34127 .50956 L
s
.37302 .50581 m
.37302 .50956 L
s
.43651 .50581 m
.43651 .50956 L
s
.46825 .50581 m
.46825 .50956 L
s
.5 .50581 m
.5 .50956 L
s
.56349 .50581 m
.56349 .50956 L
s
.59524 .50581 m
.59524 .50956 L
s
.62698 .50581 m
.62698 .50956 L
s
.69048 .50581 m
.69048 .50956 L
s
.72222 .50581 m
.72222 .50956 L
s
.75397 .50581 m
.75397 .50956 L
s
.81746 .50581 m
.81746 .50956 L
s
.84921 .50581 m
.84921 .50956 L
s
.88095 .50581 m
.88095 .50956 L
s
.94444 .50581 m
.94444 .50956 L
s
.97619 .50581 m
.97619 .50956 L
s
.25 Mabswid
0 .50581 m
1 .50581 L
s
.02381 .01569 m
.03006 .01569 L
s
[(0.7)] .01131 .01569 1 0 Mshowa
.02381 .09738 m
.03006 .09738 L
s
[(0.75)] .01131 .09738 1 0 Mshowa
.02381 .17907 m
.03006 .17907 L
s
[(0.8)] .01131 .17907 1 0 Mshowa
.02381 .26075 m
.03006 .26075 L
s
[(0.85)] .01131 .26075 1 0 Mshowa
.02381 .34244 m
.03006 .34244 L
s
[(0.9)] .01131 .34244 1 0 Mshowa
.02381 .42413 m
.03006 .42413 L
s
[(0.95)] .01131 .42413 1 0 Mshowa
.02381 .5875 m
.03006 .5875 L
s
[(1.05)] .01131 .5875 1 0 Mshowa
.125 Mabswid
.02381 .03203 m
.02756 .03203 L
s
.02381 .04837 m
.02756 .04837 L
s
.02381 .0647 m
.02756 .0647 L
s
.02381 .08104 m
.02756 .08104 L
s
.02381 .11372 m
.02756 .11372 L
s
.02381 .13005 m
.02756 .13005 L
s
.02381 .14639 m
.02756 .14639 L
s
.02381 .16273 m
.02756 .16273 L
s
.02381 .1954 m
.02756 .1954 L
s
.02381 .21174 m
.02756 .21174 L
s
.02381 .22808 m
.02756 .22808 L
s
.02381 .24442 m
.02756 .24442 L
s
.02381 .27709 m
.02756 .27709 L
s
.02381 .29343 m
.02756 .29343 L
s
.02381 .30976 m
.02756 .30976 L
s
.02381 .3261 m
.02756 .3261 L
s
.02381 .35878 m
.02756 .35878 L
s
.02381 .37511 m
.02756 .37511 L
s
.02381 .39145 m
.02756 .39145 L
s
.02381 .40779 m
.02756 .40779 L
s
.02381 .44046 m
.02756 .44046 L
s
.02381 .4568 m
.02756 .4568 L
s
.02381 .47314 m
.02756 .47314 L
s
.02381 .48947 m
.02756 .48947 L
s
.02381 .52215 m
.02756 .52215 L
s
.02381 .53849 m
.02756 .53849 L
s
.02381 .55482 m
.02756 .55482 L
s
.02381 .57116 m
.02756 .57116 L
s
.02381 .60384 m
.02756 .60384 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.46216 0 m
.46515 .0077 L
.50273 .1044 L
.54275 .20738 L
.58126 .30648 L
.62221 .41184 L
.66166 .51334 L
.69959 .61096 L
s
.69959 .61096 m
.70234 .61803 L
s
.03016 .23544 m
.03239 .23546 L
.03477 .23549 L
.03908 .23557 L
.04419 .23569 L
.04884 .23583 L
.05897 .23623 L
.06854 .23673 L
.08861 .23816 L
.11039 .24029 L
.15098 .24575 L
.19007 .25271 L
.22766 .26081 L
.26768 .27077 L
.30619 .28148 L
.34714 .29393 L
.38658 .30682 L
.42452 .31994 L
.46489 .33454 L
.50376 .34896 L
.54507 .364 L
.5658 .37093 L
.58487 .37644 L
.5945 .37874 L
.60349 .38048 L
.60839 .38123 L
.61287 .38179 L
.61525 .38203 L
.61784 .38224 L
.61926 .38234 L
.62057 .38242 L
.62183 .38248 L
.62316 .38253 L
.62431 .38257 L
.62556 .3826 L
.6267 .38261 L
.62776 .38261 L
.62902 .38261 L
.63016 .38259 L
.63079 .38258 L
.63148 .38256 L
.63272 .38252 L
.63499 .38242 L
.63739 .38227 L
.6417 .3819 L
.64674 .38131 L
.65145 .38063 L
.66217 .37872 L
.70073 .37047 L
.74173 .36305 L
Mistroke
.78123 .35775 L
.81922 .35389 L
.85964 .35071 L
.89857 .34829 L
.93992 .34621 L
.97619 .3447 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0LOoo00<00?ooool0
S_oo000Kool00`00ooooo`1aool00`00ooooo`2>ool000Soo`8000Coo`03003ooooo00?oo`03003o
oooo00Coo`03003ooooo07;oo`03003ooooo08goo`001ooo00@00?ooool000Woo`03003ooooo00Co
o`03003ooooo07;oo`03003ooooo08goo`001ooo00@00?ooool000[oo`03003ooooo00?oo`8007Co
o`03003ooooo08coo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo07?oo`03003o
oooo08coo`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo07Coo`03003ooooo08_o
o`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`1dool00`00ooooo`2;ool000So
o`8000Woo`@000Coo`03003ooooo07Coo`03003ooooo08_oo`006ooo0P00M_oo00<00?ooool0R_oo
000Kool00`00ooooo`1eool00`00ooooo`2:ool001_oo`03003ooooo07Goo`03003ooooo08[oo`00
6ooo00<00?ooool0M_oo00<00?ooool0ROoo000Kool2001gool00`00ooooo`29ool001_oo`03003o
oooo07Ooo`03003ooooo08Soo`006ooo00<00?ooool0Mooo00<00?ooool0R?oo000Kool00`00oooo
o`1gool00`00ooooo`28ool001_oo`8007Woo`03003ooooo08Ooo`006ooo00<00?ooool0N?oo00<0
0?ooool0Qooo000Kool00`00ooooo`1iool00`00ooooo`26ool001_oo`03003ooooo07Woo`03003o
oooo08Koo`006ooo00<00?ooool0NOoo00<00?ooool0Q_oo000Kool2001kool00`00ooooo`25ool0
01_oo`03003ooooo07[oo`03003ooooo08Goo`000_oo0P001?oo00<00?ooool00ooo00<00?ooool0
0_oo0`001Ooo00<00?ooool0N_oo00<00?ooool0QOoo00001Ooo003ooooo00002Ooo00<00?ooool0
1Ooo00<00?ooool00_oo00<00?ooool0Nooo00<00?ooool0Q?oo00001Ooo003ooooo00002_oo00<0
0?ooool01?oo00<00?ooool00_oo0P00O?oo00<00?ooool0Q?oo00001Ooo003ooooo00002_oo00@0
0?ooooooo`<000Goo`03003ooooo07coo`03003ooooo08?oo`0000Goo`00ooooo`0000_oo`04003o
oooo0007ool00`00ooooo`1lool00`00ooooo`23ool00005ool00?ooool00008ool01000ooooo`00
0_oo00<00?ooool01Ooo00<00?ooool0O?oo00<00?ooool0Pooo0002ool20009ool40002ool40004
ool2001nool00`00ooooo`22ool001_oo`03003ooooo07goo`03003ooooo08;oo`006ooo00<00?oo
ool0O_oo00<00?ooool0POoo000Kool00`00ooooo`1nool00`00ooooo`21ool001_oo`03003ooooo
07koo`03003ooooo087oo`006ooo0P00P?oo00<00?ooool0P?oo000Kool00`00ooooo`1oool00`00
ooooo`20ool001_oo`03003ooooo083oo`03003ooooo07ooo`006ooo00<00?ooool0P?oo00<00?oo
ool0Oooo000Kool20021ool00`00ooooo`1oool001_oo`03003ooooo087oo`03003ooooo07koo`00
6ooo00<00?ooool0POoo00<00?ooool0O_oo000Kool00`00ooooo`22ool00`00ooooo`1mool001_o
o`8008?oo`03003ooooo07goo`006ooo00<00?ooool0Pooo00<00?ooool0O?oo000Kool00`00oooo
o`23ool00`00ooooo`1lool000Soo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo08?oo`03
003ooooo07coo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`24ool00`00oooo
o`1kool000Ooo`04003ooooo0008ool01000ooooo`001?oo0P00QOoo00<00?ooool0Nooo0007ool0
1000ooooo`002Ooo0P001Ooo00<00?ooool0QOoo00<00?ooool0N_oo0007ool01000ooooo`002?oo
00@00?ooool000Coo`03003ooooo08Goo`03003ooooo07[oo`001ooo00@00?ooool000Soo`04003o
oooo0004ool00`00ooooo`25ool00`00ooooo`1jool000Soo`8000[oo`8000Goo`8008Ooo`03003o
oooo07Woo`006ooo00<00?ooool0Q_oo00<00?ooool0NOoo000Kool00`00ooooo`27ool00`00oooo
o`1hool001_oo`03003ooooo08Ooo`03003ooooo07Soo`006ooo0P00R?oo00<00?ooool0N?oo000K
ool00`00ooooo`28ool00`00ooooo`1gool001_oo`03003ooooo08Soo`03003ooooo07Ooo`006ooo
00<00?ooool0R?oo00<00?ooool0Mooo000Kool00`00ooooo`29ool00`00ooooo`1fool001_oo`80
08[oo`03003ooooo07Koo`006ooo00<00?ooool0R_oo00<00?ooool0MOoo000Kool00`00ool0000A
001iool00`00ooooo`1eool001_oo`03003ooooo017oo`T0073oo`03003ooooo07Goo`006ooo0P00
6ooo2000JOoo00<00?ooool0M?oo000Kool00`00ooooo`0Rool5001Tool00`00ooooo`1dool000;o
o`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo02Ooo`D005ooo`03003ooooo07Co
o`0000Goo`00ooooo`0000Soo`04003ooooo0005ool00`00ooooo`02ool00`00ooooo`0/ool5001K
ool00`00ooooo`1cool00005ool00?ooool00008ool01000ooooo`001Ooo00<00?ooool00_oo0P00
<_oo1000Eooo00<00?ooool0Looo00001Ooo003ooooo00002Ooo0P000ooo0`001Ooo00<00?ooool0
=Ooo1000E?oo00<00?ooool0L_oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`03003ooooo
00Goo`03003ooooo03Woo`@0053oo`03003ooooo07;oo`0000Goo`00ooooo`0000Soo`04003ooooo
0002ool00`00ooooo`05ool00`00ooooo`0mool3001=ool00`00ooooo`1bool000;oo`8000[oo`80
00?oo`@000Coo`03003ooooo043oo`<004_oo`03003ooooo077oo`006ooo0P00A?oo1000Aooo00<0
0?ooool0LOoo000Kool00`00ooooo`17ool30014ool00`00ooooo`1aool001_oo`03003ooooo04[o
o`@0047oo`03003ooooo073oo`006ooo00<00?ooool0C_oo1000?Ooo00<00?ooool0L?oo000Kool2
001Cool3000kool00`00ooooo`1_ool001_oo`03003ooooo05Goo`8003Woo`03003ooooo06ooo`00
6ooo00<00?ooool0Eooo0`00=_oo00<00?ooool0Kooo000Kool00`00ooooo`1Jool2000eool00`00
ooooo`1^ool001_oo`8005goo`<003;oo`03003ooooo06koo`006ooo00<00?ooool0Gooo0`00;ooo
00<00?ooool0K_oo000Kool00`00ooooo`1Rool4000/ool00`00ooooo`1]ool001_oo`03003ooooo
06Koo`<002Woo`03003ooooo06goo`006ooo00<00?ooool0JOoo0`009ooo00<00?ooool0K?oo000K
ool2001]ool2000Uool00`00ooooo`1/ool001_oo`03003ooooo06koo`<002;oo`03003ooooo06co
o`002?oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0LOoo0`008?oo00<00?ooool0Jooo
0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0M?oo0P007_oo00<00?ooool0Jooo
0007ool01000ooooo`002ooo00<00?ooool00_oo0P00Mooo0`007?oo00<00?ooool0J_oo0007ool0
1000ooooo`002Ooo0`001?oo00<00?ooool0NOoo0P006_oo00<00?ooool0E?oo3`001ooo0007ool0
1000ooooo`002?oo00@00?ooool000Coo`03003ooooo07_oo`<001Ooo`03003ooooo03kooaH001Ko
o`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`1nool3000Eool00`00ooooo`0b
ool;000/ool000Soo`8000[oo`8000Goo`8008;oo`8001?oo`03003ooooo02_oo`L003Ooo`006ooo
00<00?ooool0Pooo0`004?oo00<00?ooool09_oo1@00?_oo000Kool00`00ooooo`26ool3000>ool0
0`00ooooo`0Pool50013ool001_oo`03003ooooo08Woo`<000_oo`03003ooooo01[oo`H004Soo`00
6ooo00<00?ooool0S?oo10002?oo00<00?ooool05?oo1@00C_oo000Kool2002Aool40004ool00`00
ooooo`0?ool5001Cool001_oo`03003ooooo09Coo`D000Woo`P005Soo`006ooo00<00?ooool0VOoo
2@00H?oo000Kool00`00ooooo`2Iool00`00ooooo`1Vool001_oo`8009_oo`03003ooooo06Goo`00
6ooo00<00?ooool0V_oo00<00?ooool0IOoo000Kool00`00ooooo`2Jool00`00ooooo`1Uool001_o
o`03003ooooo09_oo`03003ooooo06Coo`006ooo0P00W?oo00<00?ooool0I?oo000Kool00`00oooo
o`2Lool00`00ooooo`1Sool001_oo`03003ooooo09coo`03003ooooo06?oo`000_oo0P001?oo00<0
0?ooool00_oo0`000ooo0`001Ooo00<00?ooool0W?oo00<00?ooool0Hooo00001Ooo003ooooo0000
2ooo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0WOoo00<00?ooool0H_oo00001Ooo003o
oooo00002ooo00<00?ooool00ooo00<00?ooool00_oo0P00W_oo00<00?ooool0H_oo00001Ooo003o
oooo00002Ooo0`000_oo0`001Ooo00<00?ooool0W_oo00<00?ooool0HOoo00001Ooo003ooooo0000
2?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo09koo`03003ooooo067oo`0000Goo`00
ooooo`0000Soo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`2Nool00`00ooooo`1Qool0
00;oo`8000[oo`8000?oo`@000Coo`800:3oo`03003ooooo063oo`006ooo00<00?ooool0Wooo00<0
0?ooool0H?oo000Kool00`00ooooo`2Pool00`00ooooo`1Oool001_oo`03003ooooo0:3oo`03003o
oooo05ooo`006ooo00<00?ooool0X?oo00<00?ooool0Gooo000Kool2002Rool00`00ooooo`1Nool0
01_oo`03003ooooo0:7oo`03003ooooo05koo`006ooo00<00?ooool06?oo0P001?oo00<00?ooool0
0_oo10004_oo0P001?oo00<00?ooool01?oo00<00?ooool04_oo0P001?oo00<00?ooool00ooo0P00
5?oo0P001?oo00<00?ooool00ooo0P005ooo00<00?oo00000`005ooo10000ooo00<00?ooool00_oo
10004_oo10000ooo00<00?ooool01?oo00<00?ooool04?oo000Kool00`00ooooo`0Gool01000oooo
o`002?oo00<00?ooool04_oo00@00?ooool000[oo`03003ooooo017oo`04003ooooo0008ool01000
ooooo`004_oo00@00?ooool000Soo`04003ooooo000Fool01@00oooooooo00006_oo00<00?ooool0
1ooo00<00?ooool05Ooo00<00?ooool02Ooo00<00?ooool04?oo000Kool2000Hool01000ooooo`00
2Ooo00<00?ooool04Ooo00@00?ooool000Ooo`D001;oo`04003ooooo0008ool01000ooooo`004_oo
00@00?ooool000Soo`04003ooooo000Gool01000ooooo`006_oo00<00?ooool02?oo00<00?ooool0
5?oo00<00?ooool01_oo1@004Ooo000Kool00`00ooooo`0Gool01000ooooo`002_oo00<00?ooool0
4?oo00@00?ooool000Ooo`04003ooooo000Cool01000ooooo`002?oo0`004ooo00@00?ooool000Wo
o`8001Soo`04003ooooo000Jool00`00ooooo`09ool00`00ooooo`0Cool00`00ooooo`06ool01000
ooooo`004_oo000Kool00`00ooooo`0Gool01000ooooo`002ooo00<00?ooool03ooo00@00?ooool0
00Soo`03003oo`0001?oo`04003ooooo0008ool00`00ooooo`0Cool01000ooooo`002?oo00@00?oo
ool001Ooo`04003ooooo000Jool00`00ooooo`0:ool00`00ooooo`0Bool00`00ooooo`07ool00`00
ool0000Bool001_oo`03003ooooo01Ooo`04003ooooo0008ool01000ooooo`004Ooo00@00?ooool0
00Woo`8001?oo`04003ooooo0008ool00`00ooooo`0Cool01000ooooo`002?oo00@00?ooool001So
o`<001Soo`<000Woo`04003ooooo000Bool3000:ool2000Bool001_oo`8001Woo`8000[oo`8001?o
o`8000_oo`03003ooooo01;oo`8000[oo`<001?oo`8000[oo`8001Woo`03003oo`0001[oo`03003o
oooo00Soo`8001Goo`03003ooooo00Woo`03003ooooo013oo`006ooo00<00?ooool0YOoo00<00?oo
ool0F_oo000Kool00`00ooooo`2Uool00`00ooooo`1Jool001_oo`03003ooooo0:Goo`03003ooooo
05[oo`006ooo00<00?ooool0Y_oo00<00?ooool0FOoo000Doooo000=000001_oo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`8000Koo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`006ooo
00<00?ooool0Yooo00<00?ooool0F?oo000Kool00`00ooooo`2Wool00`00ooooo`1Hool001_oo`80
0:Woo`03003ooooo05Ooo`006ooo00<00?ooool0Z?oo00<00?ooool0Eooo000Kool00`00ooooo`2Y
ool00`00ooooo`1Fool001_oo`03003ooooo0:Woo`03003ooooo05Koo`006ooo0P00Z_oo00<00?oo
ool0E_oo000Kool00`00ooooo`2Zool00`00ooooo`1Eool001_oo`03003ooooo0:[oo`03003ooooo
05Goo`006ooo00<00?ooool0Z_oo00<00?ooool0EOoo000Kool00`00ooooo`2[ool00`00ooooo`1D
ool001_oo`800:coo`03003ooooo05Coo`006ooo00<00?ooool0[?oo00<00?ooool0Dooo000Kool0
0`00ooooo`2/ool00`00ooooo`1Cool001_oo`03003ooooo0:coo`03003ooooo05?oo`006ooo0P00
[_oo00<00?ooool0D_oo000Kool00`00ooooo`2]ool00`00ooooo`1Bool00003ool00000008000?o
o`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo0:koo`03003ooooo057oo`000ooo00<00?oo
ool01ooo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo0:koo`03003ooooo057oo`000ooo
00<00?ooool01ooo00@00?ooool000Goo`03003ooooo00;oo`800:ooo`03003ooooo057oo`000ooo
00<00?ooool01ooo00@00?ooool000;oo`<000Goo`03003ooooo0:ooo`03003ooooo053oo`000ooo
00<00?ooool01ooo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo0:ooo`03003ooooo053o
o`0000Coo`00000000Woo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`2_ool00`00oooo
o`1@ool000?oo`03003ooooo00Soo`8000?oo`@000Coo`03003ooooo0;3oo`03003ooooo04ooo`00
6ooo0P00/Ooo00<00?ooool0Cooo000Kool00`00ooooo`2aool00`00ooooo`1>ool001_oo`03003o
oooo0;7oo`03003ooooo04koo`006ooo00<00?ooool0/Ooo00<00?ooool0C_oo003ooolQool00?oo
ob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, 0.675595, \
0.00592092, 0.00230106}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.6, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8819965443323613`, 
      mGamma \[Rule] \(-0.11184214154858815`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.74, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] \(\(0.88459747102775`\)\(\[InvisibleSpace]\)\) + 
          3.456264716480783`*^-7\ \[ImaginaryI]}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(realsol[k_, a_] := 
      FindRoot[Re[kx0corrtestfunc[Om, k, a]] \[Equal] 0, {Om, 0.5, 0.6}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[1, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9144929470110464`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol[k, 100]], {k, 0.01, 0.95}, 
      PlotStyle \[Rule] {RGBColor[1, 0, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.00251 -1.07972 1.74976 [
[.22431 .13261 -9 -9 ]
[.22431 .13261 9 0 ]
[.42481 .13261 -9 -9 ]
[.42481 .13261 9 0 ]
[.62531 .13261 -9 -9 ]
[.62531 .13261 9 0 ]
[.82581 .13261 -9 -9 ]
[.82581 .13261 9 0 ]
[.01131 .05762 -24 -4.5 ]
[.01131 .05762 0 4.5 ]
[.01131 .2326 -24 -4.5 ]
[.01131 .2326 0 4.5 ]
[.01131 .32009 -18 -4.5 ]
[.01131 .32009 0 4.5 ]
[.01131 .40757 -24 -4.5 ]
[.01131 .40757 0 4.5 ]
[.01131 .49506 -18 -4.5 ]
[.01131 .49506 0 4.5 ]
[.01131 .58255 -24 -4.5 ]
[.01131 .58255 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.22431 .14511 m
.22431 .15136 L
s
[(0.2)] .22431 .13261 0 1 Mshowa
.42481 .14511 m
.42481 .15136 L
s
[(0.4)] .42481 .13261 0 1 Mshowa
.62531 .14511 m
.62531 .15136 L
s
[(0.6)] .62531 .13261 0 1 Mshowa
.82581 .14511 m
.82581 .15136 L
s
[(0.8)] .82581 .13261 0 1 Mshowa
.125 Mabswid
.07393 .14511 m
.07393 .14886 L
s
.12406 .14511 m
.12406 .14886 L
s
.17419 .14511 m
.17419 .14886 L
s
.27444 .14511 m
.27444 .14886 L
s
.32456 .14511 m
.32456 .14886 L
s
.37469 .14511 m
.37469 .14886 L
s
.47494 .14511 m
.47494 .14886 L
s
.52506 .14511 m
.52506 .14886 L
s
.57519 .14511 m
.57519 .14886 L
s
.67544 .14511 m
.67544 .14886 L
s
.72556 .14511 m
.72556 .14886 L
s
.77569 .14511 m
.77569 .14886 L
s
.87594 .14511 m
.87594 .14886 L
s
.92607 .14511 m
.92607 .14886 L
s
.97619 .14511 m
.97619 .14886 L
s
.25 Mabswid
0 .14511 m
1 .14511 L
s
.02381 .05762 m
.03006 .05762 L
s
[(0.65)] .01131 .05762 1 0 Mshowa
.02381 .2326 m
.03006 .2326 L
s
[(0.75)] .01131 .2326 1 0 Mshowa
.02381 .32009 m
.03006 .32009 L
s
[(0.8)] .01131 .32009 1 0 Mshowa
.02381 .40757 m
.03006 .40757 L
s
[(0.85)] .01131 .40757 1 0 Mshowa
.02381 .49506 m
.03006 .49506 L
s
[(0.9)] .01131 .49506 1 0 Mshowa
.02381 .58255 m
.03006 .58255 L
s
[(0.95)] .01131 .58255 1 0 Mshowa
.125 Mabswid
.02381 .07512 m
.02756 .07512 L
s
.02381 .09262 m
.02756 .09262 L
s
.02381 .11011 m
.02756 .11011 L
s
.02381 .12761 m
.02756 .12761 L
s
.02381 .16261 m
.02756 .16261 L
s
.02381 .1801 m
.02756 .1801 L
s
.02381 .1976 m
.02756 .1976 L
s
.02381 .2151 m
.02756 .2151 L
s
.02381 .2501 m
.02756 .2501 L
s
.02381 .26759 m
.02756 .26759 L
s
.02381 .28509 m
.02756 .28509 L
s
.02381 .30259 m
.02756 .30259 L
s
.02381 .33758 m
.02756 .33758 L
s
.02381 .35508 m
.02756 .35508 L
s
.02381 .37258 m
.02756 .37258 L
s
.02381 .39008 m
.02756 .39008 L
s
.02381 .42507 m
.02756 .42507 L
s
.02381 .44257 m
.02756 .44257 L
s
.02381 .46007 m
.02756 .46007 L
s
.02381 .47756 m
.02756 .47756 L
s
.02381 .51256 m
.02756 .51256 L
s
.02381 .53006 m
.02756 .53006 L
s
.02381 .54755 m
.02756 .54755 L
s
.02381 .56505 m
.02756 .56505 L
s
.02381 .04012 m
.02756 .04012 L
s
.02381 .02263 m
.02756 .02263 L
s
.02381 .00513 m
.02756 .00513 L
s
.02381 .60005 m
.02756 .60005 L
s
.02381 .61754 m
.02756 .61754 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03383 .01472 m
.03843 .015 L
.04272 .01535 L
.05244 .0165 L
.06192 .01807 L
.07206 .02022 L
.09389 .02652 L
.11375 .03413 L
.15326 .05425 L
.19127 .07917 L
.23171 .11077 L
.27065 .14534 L
.30808 .18175 L
.34795 .22335 L
.38631 .26569 L
.4271 .31278 L
.46639 .35985 L
.48621 .38415 L
.49484 .39484 L
.50418 .40641 L
.50672 .40949 L
.508 .41063 L
.50873 .41107 L
.5094 .41134 L
.51062 .41147 L
.51172 .41174 L
.51421 .41217 L
.52365 .41357 L
.5444 .41675 L
.58383 .4234 L
.62175 .4306 L
.66211 .43929 L
.70096 .44885 L
.73831 .45938 L
.77809 .47241 L
.81637 .48728 L
.85708 .50651 L
.89629 .52988 L
.93399 .55907 L
.97413 .60081 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool2
003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooh
0006n03lool001_oo`03003ooooo00Koo`Kh0?Koo`006ooo0P003Ooo0_P0m?oo000Kool00`00oooo
o`0>ool3n03aool001_oo`03003ooooo017oo`?h0>koo`006ooo00<00?ooool05?oo0_P0k?oo000K
ool00`00ooooo`0Fool2n03Zool001_oo`8001Woo`;h0>Soo`006ooo00<00?ooool06_oo0_P0i_oo
0002ool20004ool00`00ooooo`03ool20003ool30005ool00`00ooooo`0Lool2n03Tool00005ool0
0?ooool00008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool07_oo0_P0h_oo00001Ooo
003ooooo00002?oo00@00?ooool000Goo`03003ooooo00;oo`80027oo`;h0>3oo`0000Goo`00oooo
o`0000Soo`<000?oo`<000Goo`03003ooooo02;oo`03n03ooooo0=goo`0000Goo`00ooooo`0000So
o`03003ooooo00?oo`03003ooooo00Goo`03003ooooo02?oo`03n03ooooo0=coo`0000Goo`00oooo
o`0000Soo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo02Coo`;h0=coo`000_oo0P002_oo
0`000_oo10001?oo00<00?ooool09_oo00?h0?ooool0fOoo000Kool2000Xool2n03Iool001_oo`03
003ooooo02Woo`03n03ooooo0=Koo`006ooo00<00?ooool0:_oo0_P0e_oo000Kool00`00ooooo`0/
ool00oP0ooooo`3Cool001_oo`8002koo`03n03ooooo0=;oo`006ooo00<00?ooool0;_oo0_P0d_oo
000Kool00`00ooooo`0`ool00oP0ooooo`3?ool001_oo`03003ooooo037oo`03n03ooooo0<koo`00
6ooo00<00?ooool0:ooo0P001?oo00<00?P0n0000_oo10009_oo0P001?oo00<00?ooool01?oo00<0
0?ooool09_oo0P001?oo00<00?ooool00ooo0P009ooo0P001?oo00<00?ooool00ooo0P00:Ooo000K
ool2000[ool01000ooooo`001_oo00?h0?oo0000:?oo00@00?ooool000[oo`03003ooooo02Goo`04
003ooooo0008ool01000ooooo`009Ooo00@00?ooool000Soo`04003ooooo000Xool001_oo`03003o
oooo02[oo`04003ooooo0007ool00oP0ool0000Wool01000ooooo`001ooo1@009_oo00@00?ooool0
00Soo`04003ooooo000Uool01000ooooo`002?oo00@00?ooool002Soo`006ooo00<00?ooool0:_oo
00@00?ooool000Soo`03n03oo`0002Koo`04003ooooo0007ool01000ooooo`009ooo00@00?ooool0
00Soo`<002Koo`04003ooooo0009ool2000Yool001_oo`03003ooooo02[oo`04003ooooo0009ool0
0oP0ool0000Uool01000ooooo`002?oo00<00?oo00009ooo00@00?ooool000Soo`03003ooooo02Ko
o`04003ooooo0008ool01000ooooo`00:?oo000Kool00`00ooooo`0Zool01000ooooo`002?oo00@0
0?oon00002Goo`04003ooooo0009ool2000Wool01000ooooo`002?oo00<00?ooool09_oo00@00?oo
ool000Soo`04003ooooo000Xool001_oo`8002coo`8000[oo`8000;h02Goo`8000_oo`03003ooooo
02Koo`8000[oo`<002Koo`8000[oo`8002Woo`006ooo00<00?ooool0>ooo00?h0?ooool0a?oo000K
ool00`00ooooo`0lool00oP0ooooo`33ool001_oo`03003ooooo03goo`03n03ooooo0<;oo`005?oo
B00000?h00000000`@00000Kool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0;ool00`00
ooooo`0:ool00`00ooooo`0:ool00oP0ooooo`0;ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`0;ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0;ool00`00ooooo`0:ool00`00
ooooo`0:ool00`00ooooo`0;ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0;ool00`00
ooooo`0:ool00`00ooooo`05ool001_oo`03003ooooo03;oo`03003ooooo00_oo`03n03ooooo02Co
o`03003ooooo03?oo`03003ooooo03;oo`03003ooooo02goo`006ooo00<00?ooool0@Ooo00?h0?oo
ool0__oo000Kool00`00ooooo`12ool00oP0ooooo`2mool001_oo`8004Coo`03n03ooooo0;coo`00
6ooo00<00?ooool0A?oo00?h0?ooool0^ooo000Kool00`00ooooo`15ool00oP0ooooo`2jool001_o
o`03003ooooo04Koo`03n03ooooo0;Woo`006ooo00<00?ooool0Aooo00?h0?ooool0^?oo000Kool2
0019ool00oP0ooooo`2gool001_oo`03003ooooo04Woo`03n03ooooo0;Koo`006ooo00<00?ooool0
B_oo00?h0?ooool0]Ooo000Kool00`00ooooo`1;ool00oP0ooooo`2dool001_oo`8004goo`03n03o
oooo0;?oo`006ooo00<00?ooool0COoo00?h0?ooool0/_oo000Kool00`00ooooo`1>ool00oP0oooo
o`2aool001_oo`03003ooooo04ooo`03n03ooooo0;3oo`006ooo00<00?ooool0D?oo00?h0?ooool0
[ooo000Kool2001Bool00oP0ooooo`2^ool001_oo`03003ooooo05;oo`03n03ooooo0:goo`006ooo
00<00?ooool0Dooo00?h0?ooool0[?oo0002ool20004ool00`00ooooo`03ool00`00ooooo`02ool3
0005ool00`00ooooo`1Dool00oP0ooooo`2[ool00005ool00?ooool00009ool00`00ooooo`05ool0
0`00ooooo`02ool00`00ooooo`1Eool00oP0ooooo`2Zool00005ool00?ooool0000:ool00`00oooo
o`04ool00`00ooooo`02ool2001Fool00oP0ooooo`2Zool00005ool00?ooool0000:ool01000oooo
oooo0`001Ooo00<00?ooool0E_oo00?h0?ooool0ZOoo00001Ooo003ooooo00002ooo00@00?ooool0
00Ooo`03003ooooo05Ooo`03n03ooooo0:Soo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool0
0`00ooooo`05ool00`00ooooo`1Hool00oP0ooooo`2Wool000;oo`8000Woo`@000;oo`@000Coo`80
05[oo`03n03ooooo0:Koo`006ooo00<00?ooool0F_oo00?h0?ooool0YOoo000Kool00`00ooooo`1J
ool00oP0ooooo`2Uool001_oo`03003ooooo05_oo`03n03ooooo0:Coo`006ooo00<00?ooool0G?oo
00?h0?ooool0Xooo000Kool2001Nool00oP0ooooo`2Rool001_oo`03003ooooo05koo`03n03ooooo
0:7oo`006ooo00<00?ooool0Gooo00?h0?ooool0X?oo000Kool00`00ooooo`1Pool00oP0ooooo`2O
ool001_oo`03003ooooo067oo`03n03ooooo09koo`006ooo0P00Hooo00?h0?ooool0WOoo000Kool0
0`00ooooo`1Rool00oP0ooooo`2Mool001_oo`03003ooooo06?oo`03n03ooooo09coo`006ooo00<0
0?ooool0I?oo00?h0?ooool0Vooo000Kool2001Vool00oP0ooooo`2Jool001_oo`03003ooooo06Ko
o`03n03ooooo09Woo`006ooo00<00?ooool0Iooo00?h0?ooool0V?oo0008ool20004ool00`00oooo
o`03ool20005ool00`00ooooo`1Xool00oP0ooooo`2Gool000Ooo`04003ooooo0008ool01000oooo
o`001?oo00<00?ooool0JOoo00?h0?ooool0U_oo0007ool01000ooooo`002?oo00@00?ooool000Co
o`8006_oo`03n03ooooo09Goo`001ooo00@00?ooool000Woo`8000Goo`03003ooooo06_oo`03n03o
oooo09Coo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`1[ool00oP0ooooo`2D
ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0K?oo00?h0?ooool0Tooo0008
ool2000:ool20005ool00`00ooooo`1]ool00oP0ooooo`2Bool001_oo`8006ooo`03n03ooooo097o
o`006ooo00<00?ooool0Kooo00?h0?ooool0T?oo000Kool00`00ooooo`1`ool00oP0ooooo`2?ool0
01_oo`03003ooooo073oo`03n03ooooo08ooo`006ooo0P00L_oo00?h0?ooool0S_oo000Kool00`00
ooooo`1bool00oP0ooooo`2=ool001_oo`03003ooooo07?oo`03n03ooooo08coo`006ooo00<00?oo
ool0M?oo00?h0?ooool0Rooo000Kool00`00ooooo`1eool00oP0ooooo`2:ool001_oo`8007Koo`03
n03ooooo08[oo`006ooo00<00?ooool0M_oo00?h0?ooool0ROoo000Kool00`00ooooo`1gool00oP0
ooooo`28ool001_oo`03003ooooo07Soo`03n03ooooo08Ooo`006ooo00<00?ooool0NOoo00?h0?oo
ool0Q_oo000Kool2001jool00oP0ooooo`26ool001_oo`03003ooooo07[oo`03n03ooooo08Goo`00
0_oo0P001?oo00<00?ooool00ooo0P000ooo0`001Ooo00<00?ooool0Nooo00?h0?ooool0Q?oo0000
1Ooo003ooooo00002?oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo07coo`03n03ooooo
08?oo`0000Goo`00ooooo`0000Soo`04003ooooo0005ool00`00ooooo`02ool2001nool00oP0oooo
o`22ool00005ool00?ooool00009ool20003ool30005ool00`00ooooo`1mool4n021ool00005ool0
0?ooool00008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0POoo1?P0OOoo00001Ooo
003ooooo00002?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo08Goo`Sh07Goo`000_oo
0P002_oo0P000ooo10001?oo00<00?ooool0SOoo2?P0KOoo000Kool2002Fool5n01Xool001_oo`03
003ooooo09[oo`Ch06Coo`006ooo00<00?ooool0W_oo1?P0H?oo000Kool00`00ooooo`2Rool4n01L
ool001_oo`03003ooooo0:Koo`Ch05Soo`006ooo0P00Zooo1OP0Dooo000Kool00`00ooooo`2_ool4
n01?ool001_oo`03003ooooo0;?oo`?h04coo`006ooo00<00?ooool0]_oo1?P0B?oo000Kool2002k
ool3n015ool001_oo`03003ooooo0;goo`?h04;oo`006ooo00<00?ooool0`?oo1?P0?_oo000Kool0
0`00ooooo`34ool3n00kool001_oo`03003ooooo0<Ooo`?h03Soo`006ooo0P00booo0_P0=_oo000K
ool00`00ooooo`3<ool3n00cool001_oo`03003ooooo0<ooo`;h037oo`002?oo0P001?oo00<00?oo
ool00_oo0`001Ooo00<00?ooool0dOoo0_P0;ooo0007ool01000ooooo`002ooo00<00?ooool00_oo
00<00?ooool0dooo0_P0;Ooo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00e_oo00?h0?oo
ool0:_oo0007ool01000ooooo`002Ooo0`001?oo00<00?ooool0e_oo0_P0:_oo0007ool01000oooo
o`002?oo00@00?ooool000Coo`03003ooooo0=Soo`;h02Soo`001ooo00@00?ooool000Soo`04003o
oooo0004ool00`00ooooo`3Jool00oP0ooooo`0Uool000Soo`8000[oo`8000Goo`800=coo`;h02Go
o`006ooo00<00?ooool0gOoo0_P08ooo000Kool00`00ooooo`3Oool2n00Qool001_oo`03003ooooo
0>7oo`;h01ooo`006ooo00<00?ooool0hooo0_P07Ooo000Kool2003Vool00oP0ooooo`0Jool001_o
o`03003ooooo0>Koo`03n03ooooo01Woo`006ooo00<00?ooool0iooo0_P06Ooo000Kool00`00oooo
o`3Yool00oP0ooooo`0Fool001_oo`800>_oo`03n03ooooo01Goo`006ooo00<00?ooool0jooo00?h
0?ooool05?oo000Kool00`00ooooo`3/ool2n00Dool001_oo`03003ooooo0>koo`03n03ooooo017o
o`006ooo00<00?ooool0kooo00?h0?ooool04?oo000Kool2003aool00oP0ooooo`0?ool001_oo`03
003ooooo0?7oo`03n03ooooo00koo`006ooo00<00?ooool0l_oo00?h0?ooool03Ooo0002ool20004
ool00`00ooooo`02ool30003ool30005ool00`00ooooo`3cool00oP0ooooo`0<ool00005ool00?oo
ool0000;ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`3dool00oP0ooooo`0;ool00005
ool00?ooool0000;ool00`00ooooo`03ool00`00ooooo`02ool2003fool00oP0ooooo`0:ool00005
ool00?ooool00009ool30002ool30005ool00`00ooooo`3fool00oP0ooooo`09ool00005ool00?oo
ool00008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0mooo00?h0?ooool02?oo0000
1Ooo003ooooo00002?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo0?Soo`03n03ooooo
00Ooo`000_oo0P002_oo0P000ooo10001?oo0P00n_oo00?h0?ooool01_oo000Kool00`00ooooo`3j
ool00oP0ooooo`05ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`800?ooo`Coo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.102471, 0.603599, \
0.0037499, 0.00214847}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%1121, %1109]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -0.141201 0.496346 [
[.15079 .34265 -9 -9 ]
[.15079 .34265 9 0 ]
[.27778 .34265 -9 -9 ]
[.27778 .34265 9 0 ]
[.40476 .34265 -9 -9 ]
[.40476 .34265 9 0 ]
[.53175 .34265 -9 -9 ]
[.53175 .34265 9 0 ]
[.65873 .34265 -3 -9 ]
[.65873 .34265 3 0 ]
[.78571 .34265 -9 -9 ]
[.78571 .34265 9 0 ]
[.9127 .34265 -9 -9 ]
[.9127 .34265 9 0 ]
[.01131 .05734 -18 -4.5 ]
[.01131 .05734 0 4.5 ]
[.01131 .15661 -18 -4.5 ]
[.01131 .15661 0 4.5 ]
[.01131 .25588 -18 -4.5 ]
[.01131 .25588 0 4.5 ]
[.01131 .45442 -18 -4.5 ]
[.01131 .45442 0 4.5 ]
[.01131 .55368 -18 -4.5 ]
[.01131 .55368 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .35515 m
.15079 .3614 L
s
[(0.2)] .15079 .34265 0 1 Mshowa
.27778 .35515 m
.27778 .3614 L
s
[(0.4)] .27778 .34265 0 1 Mshowa
.40476 .35515 m
.40476 .3614 L
s
[(0.6)] .40476 .34265 0 1 Mshowa
.53175 .35515 m
.53175 .3614 L
s
[(0.8)] .53175 .34265 0 1 Mshowa
.65873 .35515 m
.65873 .3614 L
s
[(1)] .65873 .34265 0 1 Mshowa
.78571 .35515 m
.78571 .3614 L
s
[(1.2)] .78571 .34265 0 1 Mshowa
.9127 .35515 m
.9127 .3614 L
s
[(1.4)] .9127 .34265 0 1 Mshowa
.125 Mabswid
.05556 .35515 m
.05556 .3589 L
s
.0873 .35515 m
.0873 .3589 L
s
.11905 .35515 m
.11905 .3589 L
s
.18254 .35515 m
.18254 .3589 L
s
.21429 .35515 m
.21429 .3589 L
s
.24603 .35515 m
.24603 .3589 L
s
.30952 .35515 m
.30952 .3589 L
s
.34127 .35515 m
.34127 .3589 L
s
.37302 .35515 m
.37302 .3589 L
s
.43651 .35515 m
.43651 .3589 L
s
.46825 .35515 m
.46825 .3589 L
s
.5 .35515 m
.5 .3589 L
s
.56349 .35515 m
.56349 .3589 L
s
.59524 .35515 m
.59524 .3589 L
s
.62698 .35515 m
.62698 .3589 L
s
.69048 .35515 m
.69048 .3589 L
s
.72222 .35515 m
.72222 .3589 L
s
.75397 .35515 m
.75397 .3589 L
s
.81746 .35515 m
.81746 .3589 L
s
.84921 .35515 m
.84921 .3589 L
s
.88095 .35515 m
.88095 .3589 L
s
.94444 .35515 m
.94444 .3589 L
s
.97619 .35515 m
.97619 .3589 L
s
.25 Mabswid
0 .35515 m
1 .35515 L
s
.02381 .05734 m
.03006 .05734 L
s
[(0.4)] .01131 .05734 1 0 Mshowa
.02381 .15661 m
.03006 .15661 L
s
[(0.6)] .01131 .15661 1 0 Mshowa
.02381 .25588 m
.03006 .25588 L
s
[(0.8)] .01131 .25588 1 0 Mshowa
.02381 .45442 m
.03006 .45442 L
s
[(1.2)] .01131 .45442 1 0 Mshowa
.02381 .55368 m
.03006 .55368 L
s
[(1.4)] .01131 .55368 1 0 Mshowa
.125 Mabswid
.02381 .08216 m
.02756 .08216 L
s
.02381 .10697 m
.02756 .10697 L
s
.02381 .13179 m
.02756 .13179 L
s
.02381 .18142 m
.02756 .18142 L
s
.02381 .20624 m
.02756 .20624 L
s
.02381 .23106 m
.02756 .23106 L
s
.02381 .28069 m
.02756 .28069 L
s
.02381 .30551 m
.02756 .30551 L
s
.02381 .33033 m
.02756 .33033 L
s
.02381 .37996 m
.02756 .37996 L
s
.02381 .40478 m
.02756 .40478 L
s
.02381 .4296 m
.02756 .4296 L
s
.02381 .47923 m
.02756 .47923 L
s
.02381 .50405 m
.02756 .50405 L
s
.02381 .52887 m
.02756 .52887 L
s
.02381 .03252 m
.02756 .03252 L
s
.02381 .0077 m
.02756 .0077 L
s
.02381 .5785 m
.02756 .5785 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.03016 .16925 m
.03307 .16933 L
.03578 .16943 L
.04194 .16976 L
.04795 .1702 L
.05437 .17082 L
.0682 .1726 L
.08077 .17476 L
.1058 .18047 L
.12987 .18754 L
.15548 .1965 L
.18014 .20631 L
.20385 .21664 L
.2291 .22844 L
.25339 .24045 L
.27923 .2538 L
.30411 .26716 L
.31666 .27405 L
.32213 .27708 L
.32805 .28036 L
.32965 .28124 L
.33046 .28156 L
.33092 .28168 L
.33135 .28176 L
.33212 .2818 L
.33282 .28188 L
.33439 .282 L
.34037 .28239 L
.35352 .2833 L
.37849 .28518 L
.40251 .28723 L
.42806 .28969 L
.45267 .2924 L
.47633 .29539 L
.50152 .29909 L
.52577 .3033 L
.55155 .30876 L
.57638 .31539 L
.60026 .32367 L
.62568 .33551 L
.62698 .33622 L
s
0 g
.20443 0 m
.22774 .01822 L
.26647 .0485 L
.30764 .08068 L
.34731 .11169 L
.38546 .14152 L
.42606 .17326 L
.46515 .20381 L
.50273 .23319 L
.54275 .26448 L
.58126 .29459 L
.62221 .3266 L
.66166 .35743 L
.69959 .38709 L
.73997 .41865 L
.77884 .44904 L
.82014 .48133 L
.85994 .51244 L
.89823 .54238 L
.93896 .57422 L
.97619 .60332 L
s
.03016 .273 m
.03239 .27301 L
.03477 .27302 L
.03908 .27304 L
.04419 .27308 L
.04884 .27312 L
.05897 .27324 L
.06854 .2734 L
.08861 .27383 L
.11039 .27448 L
.15098 .27614 L
.19007 .27825 L
.22766 .28071 L
.26768 .28374 L
.30619 .28699 L
.34714 .29077 L
.38658 .29469 L
.42452 .29868 L
.46489 .30311 L
.50376 .30749 L
.54507 .31206 L
.5658 .31417 L
.58487 .31584 L
.5945 .31654 L
.60349 .31707 L
.60839 .3173 L
.61287 .31747 L
.61525 .31754 L
.61784 .3176 L
.61926 .31763 L
.62057 .31766 L
.62183 .31768 L
.62316 .31769 L
.62431 .3177 L
.62556 .31771 L
.6267 .31772 L
.62776 .31772 L
.62902 .31772 L
.63016 .31771 L
.63079 .31771 L
.63148 .3177 L
.63272 .31769 L
.63499 .31766 L
.63739 .31761 L
.6417 .3175 L
.64674 .31732 L
.65145 .31711 L
.66217 .31653 L
.70073 .31403 L
.74173 .31177 L
Mistroke
.78123 .31016 L
.81922 .30899 L
.85964 .30803 L
.89857 .30729 L
.93992 .30666 L
.97619 .3062 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`0^ool00`00ooooo`3Gool001Goo`03003ooooo
02ooo`03003ooooo0=Koo`005Ooo0P00<Ooo00<00?ooool0eOoo000Eool00`00ooooo`0aool2003E
ool001Goo`03003ooooo03?oo`03003ooooo0=;oo`005Ooo00<00?ooool0=?oo00<00?ooool0dOoo
000Eool00`00ooooo`0eool2003Aool001Goo`03003ooooo03Ooo`03003ooooo0<koo`005Ooo00<0
0?ooool0>?oo00<00?ooool0cOoo000Eool2000jool2003=ool001Goo`03003ooooo03_oo`03003o
oooo0<[oo`005Ooo00<00?ooool0??oo00<00?ooool0bOoo000Eool00`00ooooo`0mool20039ool0
01Goo`03003ooooo03ooo`03003ooooo0<Koo`000_oo0P001?oo00<00?ooool01?oo00<00?ooool0
0ooo00<00?ooool0@?oo00<00?ooool0aOoo00001Ooo003ooooo00002_oo00<00?ooool00ooo00<0
0?ooool0@Ooo0P00aOoo00001Ooo003ooooo00001ooo1@001?oo0`00@ooo00<00?ooool0`_oo0000
1Ooo003ooooo00001ooo00@00?ooool000Goo`03003ooooo04Coo`03003ooooo0<7oo`0000Goo`00
ooooo`0000Soo`03003oo`0000Goo`03003ooooo04Goo`03003ooooo0<3oo`0000Goo`00ooooo`00
00Woo`8000Goo`03003ooooo04Koo`03003ooooo0;ooo`000_oo0P002ooo00<00?ooool00ooo00<0
0?ooool0Aooo0P00_ooo000Eool00`00ooooo`19ool00`00ooooo`2lool001Goo`8004_oo`03003o
oooo0;_oo`005Ooo00<00?ooool0Booo0P00^ooo000Eool00`00ooooo`1=ool00`00ooooo`2hool0
01Goo`03003ooooo04koo`03003ooooo0;Ooo`005Ooo00<00?ooool0Cooo0P00]ooo000Eool00`00
ooooo`1Aool00`00ooooo`2dool001Goo`03003ooooo05;oo`03003ooooo0;?oo`005Ooo0P00E?oo
0P00/ooo000Eool00`00ooooo`1Eool00`00ooooo`2`ool001Goo`03003ooooo05Koo`03003ooooo
0:ooo`005Ooo00<00?ooool0Eooo0P00[ooo000Eool00`00ooooo`1Iool00`00ooooo`2/ool001Go
o`03003ooooo05[oo`03003ooooo0:_oo`005Ooo00<00?ooool0Fooo00<00?ooool0Z_oo000Eool2
001Mool2002Zool001Goo`03003ooooo05koo`03003ooooo0:Ooo`005Ooo00<00?ooool0Gooo00<0
0?ooool0Y_oo000Eool00`00ooooo`1Pool00`00ooooo`2Uool001Goo`03003ooooo067oo`800:Go
o`000_oo0P001?oo00<00?ooool00ooo0P001Ooo00<00?ooool0Hooo00<00?ooool0X_oo00001Ooo
003ooooo00002?oo00@00?ooool000Coo`03003ooooo06Coo`03003ooooo0:7oo`0000Goo`00oooo
o`0000Soo`04003ooooo0004ool3001Uool00`00ooooo`2Pool00005ool00?ooool00008ool30005
ool00`00ooooo`1Vool00`00ooooo`2Oool00005ool00?ooool00008ool00`00ooooo`05ool00`00
ooooo`1Wool2002Oool00005ool00?ooool00008ool00`00ooooo`05ool00`00oooh0007n01Rool0
0`00ooooo`2Lool000;oo`8000[oo`<000Coo`03003ooooo00Ooo`Ch05ooo`03003ooooo09_oo`00
5Ooo00<00?ooool02ooo1OP0Fooo0P00Vooo000Eool2000Aool5n01Hool00`00ooooo`2Hool001Go
o`03003ooooo01Goo`Ch05Goo`03003ooooo09Ooo`005Ooo00<00?ooool06Ooo0oP0Dooo0P00Uooo
000Eool00`00ooooo`0Lool4n01Aool00`00ooooo`2Dool001Goo`03003ooooo023oo`;h053oo`03
003ooooo09?oo`005Ooo00<00?ooool08_oo0_P0Cooo0P00Tooo000Eool00`00ooooo`0Tool2n01?
ool00`00ooooo`2@ool001Goo`8002Ooo`?h04goo`03003ooooo08ooo`005Ooo00<00?ooool0:Ooo
0_P0C?oo0P00Sooo000Eool00`00ooooo`0[ool2n01<ool00`00ooooo`2<ool001Goo`03003ooooo
02goo`?h04[oo`03003ooooo08_oo`005Ooo00<00?ooool0<?oo0_P0BOoo00<00?ooool0R_oo000E
ool00`00ooooo`0bool2n018ool2002:ool001Goo`03003ooooo03Coo`;h04Soo`03003ooooo08Oo
o`005Ooo0P00=ooo0_P0Aooo00<00?ooool0Q_oo000Eool00`00ooooo`0hool2n016ool00`00oooo
o`25ool001Goo`03003ooooo03[oo`;h04Goo`8008Goo`005Ooo00<00?ooool0??oo0_P0AOoo00<0
0?ooool0P_oo000Eool00`00ooooo`0nool2n014ool00`00ooooo`21ool000;oo`8000Coo`03003o
oooo00?oo`8000Goo`03003ooooo043oo`;h04?oo`03003ooooo083oo`0000Goo`00ooooo`0000So
o`04003ooooo0004ool00`00ooooo`12ool00oP0ooooo`11ool00`00ooooo`1oool00005ool00?oo
ool00008ool01000ooooo`001?oo0`00@ooo0_P0@_oo0P00Oooo00001Ooo003ooooo00002Ooo0P00
1Ooo00<00?ooool0AOoo0_P0@_oo00<00?ooool0O?oo00001Ooo003ooooo00002?oo00@00?ooool0
00Coo`03003ooooo04Ooo`;h047oo`03003ooooo07_oo`0000Goo`00ooooo`0000Soo`04003ooooo
0004ool00`00ooooo`19ool00oP0ooooo`0oool2001kool000;oo`8000[oo`8000Goo`03003oo`00
018003Soo`;h047oo`03003ooooo07Soo`005Ooo00<00?ooool04_oo4`009ooo0oP0?ooo00<00?oo
ool0Mooo000Eool2000VoolE000Eool2n00nool2001gool001Goo`03003ooooo03[oo`/000_oo`[h
03Ooo`03003ooooo07Coo`005Ooo00<00?ooool0AOoo2`002_oo3_P0:_oo00<00?ooool0Looo000E
ool00`00ooooo`1@ool;000=ool7n00Tool2001cool001Goo`03003ooooo05_oo`X000[oo`gh01Wo
o`03003ooooo073oo`005Ooo00<00?ooool0IOoo2`003?oo1_P05?oo00<00?ooool0Kooo000Eool0
0`00ooooo`1`ool8000:ool5n00@ool2001_ool001Goo`8007Woo`D000[oo`Ch00koo`03003ooooo
05Gooa0000Ooo`005Ooo00<00?ooool0OOoo20001_oo0oP03?oo00<00?ooool0:Ooo:`005ooo000E
ool00`00ooooo`25ool>0008ool00`00ooooo`0BoolF0012ool001Goo`03003ooooo01Soo`8000Co
o`03003ooooo00;oo`@001Coo`8000Coo`03003ooooo00Coo`03003ooooo01?oo`8000Coo`03003o
oooo00?oo`8001Coo`8000Coo`03003ooooo00?oo`8000Coo`03n000000001/001Coo`@000?oo`03
003ooooo00;oo`@001?oo`@000?oo`03003ooooo00Coo`03003ooooo013oo`005Ooo00<00?ooool0
5ooo00@00?ooool000Soo`03003ooooo01Coo`04003ooooo000:ool00`00ooooo`0Bool01000oooo
o`002?oo00@00?ooool001;oo`04003ooooo0008ool01000ooooo`001_oo1?P01?oo0P002ooo00<0
0?ooool06Ooo00<00?ooool01ooo00<00?ooool05_oo00<00?ooool02Ooo00<00?ooool04?oo000E
ool00`00ooooo`0Gool01000ooooo`002Ooo00<00?ooool04ooo00@00?ooool000Ooo`D001?oo`04
003ooooo0008ool01000ooooo`004_oo00@00?ooool000Soo`04003ooooo000:ool3n003ool00`00
ooooo`08ool00`00ooooo`0Iool00`00ooooo`08ool00`00ooooo`0Eool00`00ooooo`06ool5000A
ool001Goo`03003ooooo01Ooo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`001ooo00@0
0?ooool001Coo`04003ooooo0008ool3000Cool01000ooooo`002Ooo0P003_oo0_P00_oo00<00?oo
ool01ooo00<00?ooool06Ooo00<00?ooool02Ooo00<00?ooool05?oo00<00?ooool01_oo00@00?oo
ool001;oo`005Ooo0P006?oo00@00?ooool000_oo`03003ooooo017oo`04003ooooo0008ool00`00
ool0000Dool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool000Soo`04003ooooo000?ool2
n0000ooo003oo`07ool00`00ooooo`0Iool00`00ooooo`0:ool00`00ooooo`0Cool00`00ooooo`07
ool00`00ool0000Bool001Goo`03003ooooo01Ooo`04003ooooo0008ool01000ooooo`004ooo00@0
0?ooool000Woo`8001Coo`04003ooooo0008ool00`00ooooo`0Cool01000ooooo`002?oo00@00?oo
ool0017oo`;h008000Coo`<001Woo`<000Woo`04003ooooo000Cool3000:ool2000Bool001Goo`03
003ooooo01Soo`8000[oo`8001Goo`8000_oo`03003ooooo01?oo`8000[oo`<001?oo`8000[oo`80
01Koo`03003ooooo00?oo`03003ooooo01Woo`03003ooooo00Soo`8001Koo`03003ooooo00Woo`03
003ooooo013oo`005Ooo00<00?ooool0YOoo00<00?ooool0H?oo000Eool00`00ooooo`2Vool00`00
ooooo`1Oool001Goo`03003ooooo0:Ooo`8005ooo`005Ooo00<00?ooool0ZOoo00<00?ooool0G?oo
000>oooo000C000001Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo
00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo
00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Koo`<000Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Goo`005Ooo00<00?ooool0[Ooo00<00?ooool0F?oo000Eool00`00
ooooo`2^ool00`00ooooo`1Gool001Goo`03003ooooo0:ooo`8005Ooo`005Ooo00<00?ooool0/Ooo
00<00?ooool0E?oo000Eool2002cool00`00ooooo`1Cool001Goo`03003ooooo0;?oo`8005?oo`00
5Ooo00<00?ooool0]Ooo00<00?ooool0D?oo000Eool00`00ooooo`2fool00`00ooooo`1?ool001Go
o`03003ooooo0;Ooo`8004ooo`005Ooo00<00?ooool0^Ooo00<00?ooool0C?oo000Eool00`00oooo
o`2jool00`00ooooo`1;ool001Goo`800;coo`03003ooooo04[oo`005Ooo00<00?ooool0_?oo00<0
0?ooool0BOoo000Eool00`00ooooo`2mool20019ool001Goo`03003ooooo0;ooo`03003ooooo04Ko
o`005Ooo00<00?ooool0`?oo00<00?ooool0AOoo000Eool00`00ooooo`31ool00`00ooooo`14ool0
01Goo`03003ooooo0<;oo`8004Coo`005Ooo0P00aOoo00<00?ooool0@Ooo000Eool00`00ooooo`35
ool00`00ooooo`10ool001Goo`03003ooooo0<Koo`03003ooooo03ooo`005Ooo00<00?ooool0aooo
0P00?ooo000Eool00`00ooooo`39ool00`00ooooo`0lool00003ool00000008000?oo`03003ooooo
00;oo`@000Coo`03003ooooo0<[oo`03003ooooo03_oo`000ooo00<00?ooool01ooo00<00?ooool0
1Ooo00<00?ooool0booo00<00?ooool0>_oo0003ool00`00ooooo`08ool00`00ooooo`04ool3003<
ool2000jool000?oo`03003ooooo00Woo`03003ooooo00?oo`03003ooooo0<koo`03003ooooo03Oo
o`000ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0cooo00<00?ooool0=_oo00001?oo
000000002Ooo00@00?ooool000Coo`03003ooooo0=3oo`8003Koo`000ooo00<00?ooool02?oo0P00
1Ooo00<00?ooool0d_oo00<00?ooool0<ooo000Eool00`00ooooo`3Cool00`00ooooo`0bool001Go
o`800=Goo`8003;oo`005Ooo00<00?ooool0e_oo00<00?ooool0;ooo000Eool00`00ooooo`3Gool0
0`00ooooo`0^ool001Goo`03003ooooo0=Soo`8002koo`005Ooo00<00?ooool0f_oo00<00?ooool0
:ooo000Eool00`00ooooo`3Kool00`00ooooo`0Zool001Goo`03003ooooo0=coo`03003ooooo02Wo
o`005Ooo0P00g_oo0P00:Ooo000Eool00`00ooooo`3Oool00`00ooooo`0Vool001Goo`03003ooooo
0>3oo`03003ooooo02Goo`005Ooo00<00?ooool0hOoo00<00?ooool09?oo000Eool00`00ooooo`3R
ool2000Tool001Goo`03003ooooo0>Coo`03003ooooo027oo`005Ooo00<00?ooool0iOoo00<00?oo
ool08?oo000Eool2003Wool00`00ooooo`0Oool001Goo`03003ooooo0>Ooo`03003ooooo01koo`00
5Ooo00<00?ooool0j?oo0P007_oo000Eool00`00ooooo`3Zool00`00ooooo`0Kool001Goo`03003o
oooo0>_oo`03003ooooo01[oo`0000?oo`0000000P000ooo00<00?ooool01?oo00<00?ooool00ooo
00<00?ooool0k?oo0P006_oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3^ool0
0`00ooooo`0Gool000?oo`03003ooooo00Koo`D000Coo`<00>ooo`03003ooooo01Koo`000ooo00<0
0?ooool01_oo00@00?ooool000Goo`03003ooooo0?3oo`8001Koo`000ooo00<00?ooool01ooo00<0
0?oo00001Ooo00<00?ooool0l_oo00<00?ooool04ooo00001?oo000000002_oo0P001Ooo00<00?oo
ool0looo00<00?ooool04_oo0003ool00`00ooooo`09ool00`00ooooo`03ool00`00ooooo`3dool2
000Bool001Goo`03003ooooo0?Koo`03003ooooo00ooo`005Ooo0P00n?oo00<00?ooool03_oo000E
ool00`00ooooo`3hool2000>ool001Goo`03003ooooo0?[oo`03003ooooo00_oo`005Ooo00<00?oo
ool0nooo00<00?ooool02_oo000Eool00`00ooooo`3lool00`00ooooo`09ool001Goo`03003ooooo
0?goo`8000Woo`005Ooo00<00?ooool0oooo00<00?ooool01_oo000Eool2003oool2ool00`00oooo
o`05ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3o
ool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, 0.251946, \
0.00578885, 0.00740504}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(realsol2[k_, a_] := 
      FindRoot[kx0corrtestfunc[Om, k, a] \[Equal] 0, {Om, 0.85, 0.9}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol2[0.42, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8474825678107697`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol2[k, 100]], {k, 0.01, 0.5}, 
      PlotStyle \[Rule] {RGBColor[0, 1, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.90476 -25.7951 30.9285 [
[.21429 .17238 -9 -9 ]
[.21429 .17238 9 0 ]
[.40476 .17238 -9 -9 ]
[.40476 .17238 9 0 ]
[.59524 .17238 -9 -9 ]
[.59524 .17238 9 0 ]
[.78571 .17238 -9 -9 ]
[.78571 .17238 9 0 ]
[.97619 .17238 -9 -9 ]
[.97619 .17238 9 0 ]
[.01131 .03024 -30 -4.5 ]
[.01131 .03024 0 4.5 ]
[.01131 .10756 -36 -4.5 ]
[.01131 .10756 0 4.5 ]
[.01131 .26221 -36 -4.5 ]
[.01131 .26221 0 4.5 ]
[.01131 .33953 -30 -4.5 ]
[.01131 .33953 0 4.5 ]
[.01131 .41685 -36 -4.5 ]
[.01131 .41685 0 4.5 ]
[.01131 .49417 -24 -4.5 ]
[.01131 .49417 0 4.5 ]
[.01131 .57149 -36 -4.5 ]
[.01131 .57149 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .18488 m
.21429 .19113 L
s
[(0.1)] .21429 .17238 0 1 Mshowa
.40476 .18488 m
.40476 .19113 L
s
[(0.2)] .40476 .17238 0 1 Mshowa
.59524 .18488 m
.59524 .19113 L
s
[(0.3)] .59524 .17238 0 1 Mshowa
.78571 .18488 m
.78571 .19113 L
s
[(0.4)] .78571 .17238 0 1 Mshowa
.97619 .18488 m
.97619 .19113 L
s
[(0.5)] .97619 .17238 0 1 Mshowa
.125 Mabswid
.0619 .18488 m
.0619 .18863 L
s
.1 .18488 m
.1 .18863 L
s
.1381 .18488 m
.1381 .18863 L
s
.17619 .18488 m
.17619 .18863 L
s
.25238 .18488 m
.25238 .18863 L
s
.29048 .18488 m
.29048 .18863 L
s
.32857 .18488 m
.32857 .18863 L
s
.36667 .18488 m
.36667 .18863 L
s
.44286 .18488 m
.44286 .18863 L
s
.48095 .18488 m
.48095 .18863 L
s
.51905 .18488 m
.51905 .18863 L
s
.55714 .18488 m
.55714 .18863 L
s
.63333 .18488 m
.63333 .18863 L
s
.67143 .18488 m
.67143 .18863 L
s
.70952 .18488 m
.70952 .18863 L
s
.74762 .18488 m
.74762 .18863 L
s
.82381 .18488 m
.82381 .18863 L
s
.8619 .18488 m
.8619 .18863 L
s
.9 .18488 m
.9 .18863 L
s
.9381 .18488 m
.9381 .18863 L
s
.25 Mabswid
0 .18488 m
1 .18488 L
s
.02381 .03024 m
.03006 .03024 L
s
[(0.835)] .01131 .03024 1 0 Mshowa
.02381 .10756 m
.03006 .10756 L
s
[(0.8375)] .01131 .10756 1 0 Mshowa
.02381 .26221 m
.03006 .26221 L
s
[(0.8425)] .01131 .26221 1 0 Mshowa
.02381 .33953 m
.03006 .33953 L
s
[(0.845)] .01131 .33953 1 0 Mshowa
.02381 .41685 m
.03006 .41685 L
s
[(0.8475)] .01131 .41685 1 0 Mshowa
.02381 .49417 m
.03006 .49417 L
s
[(0.85)] .01131 .49417 1 0 Mshowa
.02381 .57149 m
.03006 .57149 L
s
[(0.8525)] .01131 .57149 1 0 Mshowa
.125 Mabswid
.02381 .04571 m
.02756 .04571 L
s
.02381 .06117 m
.02756 .06117 L
s
.02381 .07663 m
.02756 .07663 L
s
.02381 .0921 m
.02756 .0921 L
s
.02381 .12303 m
.02756 .12303 L
s
.02381 .13849 m
.02756 .13849 L
s
.02381 .15396 m
.02756 .15396 L
s
.02381 .16942 m
.02756 .16942 L
s
.02381 .20035 m
.02756 .20035 L
s
.02381 .21581 m
.02756 .21581 L
s
.02381 .23128 m
.02756 .23128 L
s
.02381 .24674 m
.02756 .24674 L
s
.02381 .27767 m
.02756 .27767 L
s
.02381 .29313 m
.02756 .29313 L
s
.02381 .3086 m
.02756 .3086 L
s
.02381 .32406 m
.02756 .32406 L
s
.02381 .35499 m
.02756 .35499 L
s
.02381 .37046 m
.02756 .37046 L
s
.02381 .38592 m
.02756 .38592 L
s
.02381 .40138 m
.02756 .40138 L
s
.02381 .43231 m
.02756 .43231 L
s
.02381 .44778 m
.02756 .44778 L
s
.02381 .46324 m
.02756 .46324 L
s
.02381 .47871 m
.02756 .47871 L
s
.02381 .50963 m
.02756 .50963 L
s
.02381 .5251 m
.02756 .5251 L
s
.02381 .54056 m
.02756 .54056 L
s
.02381 .55603 m
.02756 .55603 L
s
.02381 .01478 m
.02756 .01478 L
s
.02381 .58696 m
.02756 .58696 L
s
.02381 .60242 m
.02756 .60242 L
s
.02381 .61788 m
.02756 .61788 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.04286 .01472 m
.05165 .01496 L
.06129 .01532 L
.07135 .01582 L
.08072 .01639 L
.10067 .01795 L
.12201 .02014 L
.16263 .02578 L
.20176 .03305 L
.23941 .04178 L
.27945 .05292 L
.31802 .0655 L
.3551 .07933 L
.39458 .09594 L
.43258 .1138 L
.47298 .13486 L
.51189 .15718 L
.54932 .1806 L
.58915 .20767 L
.6275 .23589 L
.66825 .26827 L
.70751 .30188 L
.74529 .33654 L
.78548 .37609 L
.82417 .4167 L
.86139 .4583 L
.88211 .48273 L
.901 .50571 L
.91038 .51705 L
.92058 .52996 L
.92526 .53573 L
.93022 .54188 L
.93446 .54703 L
.93562 .54979 L
.93687 .55036 L
.93805 .49423 L
.93913 .50343 L
.94023 .51555 L
.94141 .52957 L
.94253 .53866 L
.94357 .54514 L
.94477 .5561 L
.94605 .56026 L
.94712 .56276 L
.9483 .56499 L
.95043 .56837 L
.95271 .57164 L
.95683 .576 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool002Ooo`03003ooooo0?Koo`009ooo00<00?ooool0m_oo000Wool00`00oooo
o`3fool002Ooo`8000;oo`T7h>coo`009ooo00<00?ooool02_oo2@OPhooo0008ool20004ool00`00
ooooo`03ool20004ool20003ool30005ool00`00ooooo`0Cool71n3Lool000Ooo`04003ooooo0008
ool01000ooooo`000_oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo01[oo`P7h=Coo`00
1ooo00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`03ool00`00ooooo`02ool2000Sool5
1n3?ool000Ooo`04003ooooo0009ool20004ool20003ool30005ool00`00ooooo`0Wool51n3:ool0
00Ooo`04003ooooo0008ool01000ooooo`001Ooo00@00?ooool000Ooo`03003ooooo02coo`D7h<Go
o`001ooo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool01Ooo00<0
0?ooool0<Ooo10OP`Ooo0008ool2000:ool20004ool20003ool40004ool2000fool31n2nool002Oo
o`03003ooooo03Soo`@7h;[oo`009ooo00<00?ooool0??oo0`OP]ooo000Wool00`00ooooo`0oool3
1n2dool002Ooo`8004?oo`<7h;7oo`009ooo00<00?ooool0AOoo0`OP[_oo000Wool00`00ooooo`18
ool31n2[ool002Ooo`03003ooooo04_oo`@7h:Ooo`009ooo0P00D?oo0POPYOoo000Wool00`00oooo
o`1Aool21n2Sool002Ooo`03003ooooo05?oo`87h:7oo`009ooo00<00?ooool0EOoo0POPWooo000W
ool2001Hool21n2Mool002Ooo`03003ooooo05Woo`<7h9[oo`000_oo0P001?oo00<00?ooool00ooo
0P001?oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0G?oo0POPV?oo00001Ooo003ooooo
00002?oo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`05ool00`00ooooo`02ool00`00
ooooo`1Nool21n2Fool00005ool00?ooool00008ool01000ooooo`001Ooo00<00?ooool00_oo00<0
0?ooool01?oo00<00?ooool00_oo0P00HOoo0POPU?oo00001Ooo003ooooo00002Ooo0P001?oo0P00
1Ooo00@00?ooooooo`<000Goo`03003ooooo06;oo`87h9;oo`0000Goo`00ooooo`0000Soo`04003o
oooo0005ool00`00ooooo`03ool01000ooooo`001ooo00<00?ooool0I?oo0POPT?oo00001Ooo003o
oooo00002?oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool01Ooo
0P00Iooo0POPS_oo0002ool2000:ool20004ool20003ool40002ool40004ool00`00ooooo`1Xool2
1n2<ool002Ooo`03003ooooo06[oo`87h8[oo`009ooo00<00?ooool0K?oo0POPR?oo000Wool2001_
ool21n26ool002Ooo`03003ooooo073oo`87h8Coo`009ooo00<00?ooool09Ooo0P001?oo00<00?oo
ool00_oo10008Ooo0P001?oo00<00?ooool00_oo10003_oo00<7h?ooool04?oo0P001?oo00<00?oo
ool00ooo0P008_oo0P001?oo00<00?ooool01?oo00<00?ooool08?oo0P001?oo00<00?ooool00_oo
0`000ooo000Wool00`00ooooo`0Tool01000ooooo`002_oo00<00?ooool07ooo00@00?ooool000So
o`03003ooooo013oo`87h0ooo`04003ooooo0008ool01000ooooo`008?oo00@00?ooool000[oo`03
003ooooo01ooo`04003ooooo000;ool10001ool1ool002Ooo`8002Goo`04003ooooo000:ool00`00
ooooo`0Oool01000ooooo`002Ooo00<00?ooool04Ooo0POP3Ooo00@00?ooool000_oo`03003ooooo
01koo`04003ooooo0007ool5000Pool01000ooooo`002ooo0@000Ooo0Ooo000Wool00`00ooooo`0T
ool01000ooooo`002_oo00<00?ooool07ooo00@00?ooool000[oo`03003ooooo01;oo`031n3ooooo
00[oo`04003ooooo0009ool2000Qool01000ooooo`001ooo00@00?ooool0027oo`04003ooooo0008
ool30003ool002Ooo`03003ooooo02Coo`04003ooooo000:ool00`00ooooo`0Oool01000ooooo`00
2ooo00<00?ooool04_oo0POP2_oo00@00?ooool000_oo`03003ooooo01koo`04003ooooo0008ool0
0`00ool0000Qool01000ooooo`002?oo00<00?ooool00ooo000Wool00`00ooooo`0Tool01000oooo
o`002?oo0`008Ooo00@00?ooool000Soo`04003ooooo000Fool00`OPooooo`07ool01000ooooo`00
2?oo00@00?ooool0023oo`04003ooooo0009ool2000Qool01000ooooo`002?oo00<00?ooool00ooo
000Wool2000Vool2000;ool00`00ooooo`0Pool2000:ool2000Hool21n08ool2000:ool2000Rool2
000;ool00`00ooooo`0Pool20009ool40002ool002Ooo`03003ooooo07goo`031n3ooooo07Koo`00
9ooo00<00?ooool0O_oo0POPM_oo000Wool00`00ooooo`20ool00`OPooooo`1cool0027oohX00087
h6l000Coo`009ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo
00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo
00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01_oo
00<000OPool01ooo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01_oo
00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool01_oo00<00?ooool01ooo
00<00?ooool01ooo00<00?ooool01ooo000Wool00`00ooooo`24ool00`OPooooo`1_ool002Ooo`03
003ooooo08Goo`87h6ooo`009ooo0P00R?oo00<7h?ooool0K?oo000Wool00`00ooooo`28ool21n1/
ool002Ooo`03003ooooo08[oo`031n3ooooo06Woo`009ooo00<00?ooool0Rooo0POPJOoo000Wool2
002>ool00`OPooooo`1Vool002Ooo`03003ooooo08koo`031n3ooooo06Goo`009ooo00<00?ooool0
Sooo0POPIOoo000Wool00`00ooooo`2Aool00`OPooooo`1Rool002Ooo`8009?oo`87h6;oo`009ooo
00<00?ooool0U?oo00<7h?ooool0Gooo000Wool00`00ooooo`2Eool00`OPooooo`1Nool002Ooo`80
09Ooo`87h5koo`009ooo00<00?ooool0V?oo00<7h?ooool0Fooo0002ool20004ool00`00ooooo`03
ool20005ool01000oooooooo10000_oo0`001Ooo00<00?ooool0VOoo00<7h?ooool0F_oo00001Ooo
003ooooo00002?oo00@00?ooool000Coo`05003oooooool00008ool00`00ooooo`02ool00`00oooo
o`2Jool00`OPooooo`1Iool00005ool00?ooool00008ool01@00ooooo`00ool01@000ooo00<00?oo
ool01Ooo00<00?ooool00_oo0P00W?oo0POPFOoo00001Ooo003ooooo00002Ooo0P000_oo00@00?oo
ool000Goo`04003oooooool30005ool00`00ooooo`2Mool00`OPooooo`1Fool00005ool00?ooool0
0008ool01000ooooo`000_oo00<00?oo00001_oo00@00?ooool000Ooo`03003ooooo09koo`031n3o
oooo05Goo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool20003ool01000ooooo`000_oo00<0
0?ooool01Ooo00<00?ooool0Wooo00<7h?ooool0E?oo0002ool2000:ool20005ool00`00ooooo`02
ool20003ool40004ool2002Qool21n1Dool002Ooo`03003ooooo0:;oo`031n3ooooo057oo`009ooo
00<00?ooool0Xooo00<7h?ooool0D?oo000Wool00`00ooooo`2Tool00`OPooooo`1?ool002Ooo`80
0:Koo`87h4ooo`009ooo00<00?ooool0Yooo00<7h?ooool0C?oo000Wool00`00ooooo`2Xool00`OP
ooooo`1;ool002Ooo`03003ooooo0:Woo`031n3ooooo04[oo`009ooo0P00Zooo00<7h?ooool0BOoo
000Wool00`00ooooo`2[ool00`OPooooo`18ool002Ooo`03003ooooo0:coo`031n3ooooo04Ooo`00
9ooo00<00?ooool0[Ooo0POPAooo000Wool2002`ool00`OPooooo`14ool002Ooo`03003ooooo0;3o
o`031n3ooooo04?oo`002?oo0P001?oo00<00?ooool00ooo0P001Ooo00@00?ooooooo`<000Goo`03
003ooooo0;7oo`031n3ooooo04;oo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00oooo
o`04ool00`00ooooo`02ool00`00ooooo`2bool00`OPooooo`11ool000Ooo`04003ooooo0008ool0
1@00ooooo`00ool01@001Ooo00<00?ooool00_oo0P00]?oo00<7h?ooool0@?oo0007ool01000oooo
o`002Ooo0P000_oo00@00?ooool000?oo`<000Goo`03003ooooo0;Coo`031n3ooooo03ooo`001ooo
00@00?ooool000Soo`04003ooooo0002ool00`00ool00003ool00`00ooooo`05ool00`00ooooo`2e
ool00`OPooooo`0nool000Ooo`04003ooooo0008ool01000ooooo`000ooo0P000ooo00<00?ooool0
1Ooo00<00?ooool0]_oo00<7h?ooool0?Ooo0008ool2000:ool20005ool01000oooooooo10001?oo
0P00^?oo00<7h?ooool0??oo000Wool00`00ooooo`2hool00`OPooooo`0kool002Ooo`03003ooooo
0;Woo`031n3ooooo03[oo`009ooo0P00^ooo00<7h?ooool0>Ooo000Wool00`00ooooo`2kool00`OP
ooooo`0hool002Ooo`03003ooooo0;coo`031n3ooooo03Ooo`009ooo00<00?ooool0_Ooo00<7h?oo
ool0=_oo000Wool2002oool00`OPooooo`0eool002Ooo`03003ooooo0;ooo`031n3ooooo03Coo`00
9ooo00<00?ooool0`?oo00<7h?ooool0<ooo000Wool00`00ooooo`30ool00`OPooooo`0cool002Oo
o`800<;oo`031n3ooooo03;oo`009ooo00<00?ooool0`_oo00<7h?ooool0<Ooo0002ool20004ool0
0`00ooooo`03ool20005ool00`00ooooo`02ool00`00ooooo`02ool30005ool00`00ooooo`33ool0
0`OPooooo`0`ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool00_oo00<00?oo
ool01Ooo00<00?ooool00_oo00<00?ooool0a?oo00<7h?ooool0;ooo00001Ooo003ooooo00002?oo
00D00?ooool00?oo00D000Coo`03003ooooo00Coo`03003ooooo00;oo`800<Koo`031n3ooooo02ko
o`0000Goo`00ooooo`0000Woo`8000;oo`04003ooooo0005ool01000oooooooo0`001Ooo00<00?oo
ool0a_oo00<7h?ooool0;Ooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`03003oo`0000Ko
o`04003ooooo0007ool00`00ooooo`37ool00`OPooooo`0/ool00005ool00?ooool00008ool01000
ooooo`000ooo0P000ooo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo0<Soo`031n3ooooo
02_oo`000_oo0P002_oo0P001Ooo00@00?ooooooo`@000;oo`@000Coo`800<[oo`031n3ooooo02[o
o`009ooo00<00?ooool0b_oo00<7h?ooool0:Ooo000Wool00`00ooooo`3;ool00`OPooooo`0Xool0
02Ooo`03003ooooo0<coo`031n3ooooo02Ooo`009ooo0P00c_oo00<7h?ooool09_oo000Wool00`00
ooooo`3>ool00`OPooooo`0Uool002Ooo`03003ooooo0<ooo`031n3ooooo02Coo`009ooo00<00?oo
ool0d?oo00<7h?ooool08ooo000Wool2003Aool00`OPooooo`0Sool002Ooo`03003ooooo0=7oo`03
1n3ooooo02;oo`009ooo00<00?ooool0d_oo00<7h?ooool08Ooo000Wool00`00ooooo`3Cool00`OP
ooooo`0Pool002Ooo`800=Coo`031n3ooooo023oo`003_oo0P001?oo00<00?ooool00ooo0P000ooo
0`001Ooo00<00?ooool0e?oo00<7h?ooool07ooo000=ool01000ooooo`002?oo00@00?ooool000Go
o`03003ooooo00;oo`03003ooooo0=Goo`031n3ooooo01koo`003Ooo00@00?ooool000Soo`04003o
oooo0005ool00`00ooooo`02ool2003Gool00`OPooooo`09ool00`OPooooo`0Aool000goo`04003o
oooo0009ool20003ool30005ool00`00ooooo`3Gool00`OPooooo`08ool00`OPooooo`0Aool000go
o`04003ooooo0008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0f?oo00<7h?ooool0
1ooo00<7h?ooool04Ooo000=ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo00Goo`03
003ooooo0=Woo`031n3ooooo00Koo`031n3ooooo017oo`003_oo0P002_oo0P000ooo10001?oo0P00
fooo00<7h?ooool01Ooo00<7h?ooool04Ooo000Wool00`00ooooo`3Jool00`OPooooo`05ool00`OP
ooooo`0Aool002Ooo`03003ooooo0=_oo`031n3ooooo00Coo`031n3ooooo017oo`009ooo00<00?oo
ool0g?oo00<7h?ooool00ooo00<7h?ooool04Ooo000Wool2003Nool00`OPooooo`02ool21n0Bool0
02Ooo`03003ooooo0=koo`041n3oooooool21n0Bool002Ooo`03003ooooo0=koo`041n3oooooool2
1n0Bool002Ooo`03003ooooo0=ooo`031n3ooooo0087h1;oo`009ooo0P00hOoo00@7h?oo1n07h1;o
o`009ooo00<00?ooool0h?oo00@7h?oo1n07h1;oo`009ooo00<00?ooool0hOoo0`OP4_oo000Wool0
0`00ooooo`3Rool00`OPool7h00Aool002Ooo`800>Goo`031n3ooooo00ooo`009ooo00<00?ooool0
i?oo00<7h?ooool03ooo0002ool20004ool00`00ooooo`03ool20003ool30003ool40002ool30005
ool00`00ooooo`3Tool21n0@ool00005ool00?ooool00008ool01000ooooo`001Ooo00@00?ooool0
00Soo`03003ooooo00;oo`03003ooooo0>Goo`031n3ooooo00koo`0000Goo`00ooooo`0000Soo`04
003ooooo0005ool01@00oooooooo00001ooo00<00?ooool00_oo0P00iooo00<7h?ooool03Ooo0000
1Ooo003ooooo00002Ooo0P000ooo0`001Ooo00@00?ooooooo`<000Goo`03003ooooo0>Ooo`031n3o
oooo00coo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool00`00ooooo`06ool01000ooooo`00
1ooo00<00?ooool0j?oo00<7h?ooool02ooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`03
003ooooo00?oo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`3Xool00`OPooooo`0;ool0
00;oo`8000[oo`8000?oo`@000?oo`8000?oo`@000Coo`800>[oo`031n3ooooo00[oo`009ooo00<0
0?ooool0j_oo00<7h?ooool02Ooo000Wool00`00ooooo`3[ool00`OPooooo`08ool002Ooo`03003o
oooo0>_oo`031n3ooooo00Soo`009ooo0P00kOoo00<7h?ooool01ooo000Wool00`00ooooo`3fool0
02Ooo`03003ooooo0?Koo`009ooo0P00mooo000Wool00`00ooooo`3fool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?oo
ob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0819219, 0.83261, \
0.00209325, 0.000128915}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %1122]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -0.889587 1.37647 [
[.15079 .47438 -9 -9 ]
[.15079 .47438 9 0 ]
[.27778 .47438 -9 -9 ]
[.27778 .47438 9 0 ]
[.40476 .47438 -9 -9 ]
[.40476 .47438 9 0 ]
[.53175 .47438 -9 -9 ]
[.53175 .47438 9 0 ]
[.65873 .47438 -3 -9 ]
[.65873 .47438 3 0 ]
[.78571 .47438 -9 -9 ]
[.78571 .47438 9 0 ]
[.9127 .47438 -9 -9 ]
[.9127 .47438 9 0 ]
[.01131 .07394 -18 -4.5 ]
[.01131 .07394 0 4.5 ]
[.01131 .21159 -18 -4.5 ]
[.01131 .21159 0 4.5 ]
[.01131 .34924 -18 -4.5 ]
[.01131 .34924 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .48688 m
.15079 .49313 L
s
[(0.2)] .15079 .47438 0 1 Mshowa
.27778 .48688 m
.27778 .49313 L
s
[(0.4)] .27778 .47438 0 1 Mshowa
.40476 .48688 m
.40476 .49313 L
s
[(0.6)] .40476 .47438 0 1 Mshowa
.53175 .48688 m
.53175 .49313 L
s
[(0.8)] .53175 .47438 0 1 Mshowa
.65873 .48688 m
.65873 .49313 L
s
[(1)] .65873 .47438 0 1 Mshowa
.78571 .48688 m
.78571 .49313 L
s
[(1.2)] .78571 .47438 0 1 Mshowa
.9127 .48688 m
.9127 .49313 L
s
[(1.4)] .9127 .47438 0 1 Mshowa
.125 Mabswid
.05556 .48688 m
.05556 .49063 L
s
.0873 .48688 m
.0873 .49063 L
s
.11905 .48688 m
.11905 .49063 L
s
.18254 .48688 m
.18254 .49063 L
s
.21429 .48688 m
.21429 .49063 L
s
.24603 .48688 m
.24603 .49063 L
s
.30952 .48688 m
.30952 .49063 L
s
.34127 .48688 m
.34127 .49063 L
s
.37302 .48688 m
.37302 .49063 L
s
.43651 .48688 m
.43651 .49063 L
s
.46825 .48688 m
.46825 .49063 L
s
.5 .48688 m
.5 .49063 L
s
.56349 .48688 m
.56349 .49063 L
s
.59524 .48688 m
.59524 .49063 L
s
.62698 .48688 m
.62698 .49063 L
s
.69048 .48688 m
.69048 .49063 L
s
.72222 .48688 m
.72222 .49063 L
s
.75397 .48688 m
.75397 .49063 L
s
.81746 .48688 m
.81746 .49063 L
s
.84921 .48688 m
.84921 .49063 L
s
.88095 .48688 m
.88095 .49063 L
s
.94444 .48688 m
.94444 .49063 L
s
.97619 .48688 m
.97619 .49063 L
s
.25 Mabswid
0 .48688 m
1 .48688 L
s
.02381 .07394 m
.03006 .07394 L
s
[(0.7)] .01131 .07394 1 0 Mshowa
.02381 .21159 m
.03006 .21159 L
s
[(0.8)] .01131 .21159 1 0 Mshowa
.02381 .34924 m
.03006 .34924 L
s
[(0.9)] .01131 .34924 1 0 Mshowa
.125 Mabswid
.02381 .10147 m
.02756 .10147 L
s
.02381 .129 m
.02756 .129 L
s
.02381 .15653 m
.02756 .15653 L
s
.02381 .18406 m
.02756 .18406 L
s
.02381 .23912 m
.02756 .23912 L
s
.02381 .26665 m
.02756 .26665 L
s
.02381 .29418 m
.02756 .29418 L
s
.02381 .32171 m
.02756 .32171 L
s
.02381 .37677 m
.02756 .37677 L
s
.02381 .4043 m
.02756 .4043 L
s
.02381 .43182 m
.02756 .43182 L
s
.02381 .45935 m
.02756 .45935 L
s
.02381 .51441 m
.02756 .51441 L
s
.02381 .54194 m
.02756 .54194 L
s
.02381 .56947 m
.02756 .56947 L
s
.02381 .597 m
.02756 .597 L
s
.02381 .04641 m
.02756 .04641 L
s
.02381 .01888 m
.02756 .01888 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.03016 .25907 m
.03309 .25909 L
.0363 .2591 L
.03966 .25912 L
.04278 .25915 L
.04943 .25922 L
.05654 .25932 L
.07008 .25957 L
.08313 .25989 L
.09567 .26028 L
.10902 .26078 L
.12188 .26134 L
.13424 .26195 L
.1474 .26269 L
.16006 .26348 L
.17353 .26442 L
.1865 .26542 L
.19898 .26646 L
.21226 .26766 L
.22504 .26892 L
.23862 .27036 L
.25171 .27186 L
.2643 .2734 L
.2777 .27516 L
.2906 .27697 L
.303 .27882 L
.30991 .2799 L
.31621 .28093 L
.31933 .28143 L
.32273 .28201 L
.32429 .28226 L
.32595 .28254 L
.32736 .28277 L
.32775 .28289 L
.32816 .28291 L
.32856 .28042 L
.32892 .28083 L
.32928 .28136 L
.32968 .28199 L
.33005 .28239 L
.33039 .28268 L
.3308 .28317 L
.33122 .28335 L
.33158 .28347 L
.33197 .28356 L
.33268 .28372 L
.33344 .28386 L
.33482 .28405 L
.34127 .28527 L
s
1 0 0 r
.10189 0 m
.1058 .00247 L
.12987 .02207 L
.15548 .04693 L
.18014 .07412 L
.20385 .10276 L
.2291 .13549 L
.25339 .1688 L
.27923 .20584 L
.30411 .24287 L
.31666 .26199 L
.32213 .2704 L
.32805 .2795 L
.32965 .28192 L
.33046 .28282 L
.33092 .28316 L
.33135 .28337 L
.33212 .28348 L
.33282 .28369 L
.33439 .28403 L
.34037 .28513 L
.35352 .28763 L
.37849 .29286 L
.40251 .29853 L
.42806 .30536 L
.45267 .31288 L
.47633 .32117 L
.50152 .33142 L
.52577 .34311 L
.55155 .35825 L
.57638 .37662 L
.60026 .39959 L
.62568 .43243 L
.62698 .4344 L
s
0 g
.43415 0 m
.46515 .06721 L
.50273 .14868 L
.54275 .23544 L
.58126 .31894 L
.62221 .40771 L
.66166 .49322 L
.69959 .57547 L
s
.69959 .57547 m
.71923 .61803 L
s
.03016 .25909 m
.03239 .25911 L
.03477 .25913 L
.03908 .25919 L
.04419 .2593 L
.04884 .25941 L
.05897 .25975 L
.06854 .26017 L
.08861 .26138 L
.11039 .26317 L
.15098 .26777 L
.19007 .27364 L
.22766 .28046 L
.26768 .28885 L
.30619 .29788 L
.34714 .30836 L
.38658 .31923 L
.42452 .33028 L
.46489 .34258 L
.50376 .35473 L
.54507 .3674 L
.5658 .37324 L
.58487 .37789 L
.5945 .37982 L
.60349 .38129 L
.60839 .38192 L
.61287 .38239 L
.61525 .38259 L
.61784 .38277 L
.61926 .38286 L
.62057 .38292 L
.62183 .38297 L
.62316 .38302 L
.62431 .38305 L
.62556 .38307 L
.6267 .38308 L
.62776 .38309 L
.62902 .38308 L
.63016 .38307 L
.63079 .38306 L
.63148 .38304 L
.63272 .38301 L
.63499 .38292 L
.63739 .38279 L
.6417 .38248 L
.64674 .38199 L
.65145 .38142 L
.66217 .3798 L
.70073 .37285 L
.74173 .3666 L
Mistroke
.78123 .36213 L
.81922 .35889 L
.85964 .35621 L
.89857 .35417 L
.93992 .35241 L
.97619 .35114 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`0Bool00oP0ooooo`1Hool00`00ooooo`2Hool0
01Goo`03003ooooo01?oo`03n03ooooo05Ooo`03003ooooo09Soo`005Ooo00<00?ooool05?oo0_P0
F?oo00<00?ooool0Uooo000Eool00`00ooooo`0Fool00oP0ooooo`1Eool00`00ooooo`2Gool001Go
o`03003ooooo01Ooo`03n03ooooo05Goo`03003ooooo09Koo`005Ooo0P006Ooo0_P0EOoo00<00?oo
ool0U_oo000Eool00`00ooooo`0Jool00oP0ooooo`1Cool00`00ooooo`2Eool001Goo`03003ooooo
01_oo`03n03ooooo05;oo`03003ooooo09Goo`005Ooo00<00?ooool07?oo00?h0?ooool0D_oo00<0
0?ooool0U?oo000Eool00`00ooooo`0Mool00oP0ooooo`1Aool00`00ooooo`2Dool001Goo`03003o
oooo01koo`03n03ooooo053oo`03003ooooo09Coo`005Ooo00<00?ooool07ooo00?h0?ooool0D?oo
00<00?ooool0Tooo000Eool00`00ooooo`0Pool00oP0ooooo`1?ool00`00ooooo`2Cool001Goo`80
02;oo`03n03ooooo04ooo`03003ooooo09;oo`005Ooo00<00?ooool08_oo00?h0?ooool0C_oo00<0
0?ooool0T_oo000Eool00`00ooooo`0Sool00oP0ooooo`1>ool00`00ooooo`2Aool001Goo`03003o
oooo02Coo`03n03ooooo04goo`03003ooooo097oo`005Ooo00<00?ooool09?oo00?h0?ooool0C_oo
00<00?ooool0T?oo0002ool20004ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`0Uool0
0oP0ooooo`1=ool00`00ooooo`2@ool00005ool00?ooool00009ool00`00ooooo`04ool00`00oooo
o`0Vool00oP0ooooo`1<ool00`00ooooo`2@ool00005ool00?ooool0000:ool00`00ooooo`03ool3
000Wool00oP0ooooo`1<ool00`00ooooo`2?ool00005ool00?ooool0000:ool00`00ooooo`03ool0
0`00ooooo`0Xool00oP0ooooo`1;ool00`00ooooo`2?ool00005ool00?ooool0000;ool00`00oooo
o`02ool00`00ooooo`0Yool00oP0ooooo`1;ool00`00ooooo`2>ool00005ool00?ooool00008ool0
1000ooooo`001?oo00<00?ooool0:_oo00?h0?ooool0B_oo00<00?ooool0S_oo0002ool20009ool4
0004ool00`00ooooo`0Zool00oP0ooooo`1;ool00`00ooooo`2=ool001Goo`03003ooooo02_oo`03
n03ooooo04[oo`03003ooooo08goo`005Ooo00<00?ooool0;?oo00?h0?ooool0B_oo00<00?ooool0
S?oo000Eool00`00ooooo`0]ool00oP0ooooo`19ool00`00ooooo`2<ool001Goo`8002ooo`03n03o
oooo04Woo`03003ooooo08_oo`005Ooo00<00?ooool0;ooo00?h0?ooool0B?oo00<00?ooool0Rooo
000Eool00`00ooooo`0`ool00oP0ooooo`17ool00`00ooooo`2;ool001Goo`03003ooooo033oo`03
n03ooooo04Soo`03003ooooo08[oo`005Ooo00<00?ooool0<Ooo00?h0?ooool0Aooo00<00?ooool0
R_oo000Eool00`00ooooo`0bool00oP0ooooo`17ool00`00ooooo`29ool001Goo`03003ooooo03?o
o`03n03ooooo04Koo`03003ooooo08Woo`005Ooo0P00=?oo00?h0?ooool0Aooo00<00?ooool0R?oo
000Eool00`00ooooo`0dool00oP0ooooo`16ool00`00ooooo`28ool001Goo`03003ooooo03Goo`03
n03ooooo04Koo`03003ooooo08Ooo`005Ooo00<00?ooool0=_oo00?h0?ooool0AOoo00<00?ooool0
Qooo000Eool00`00ooooo`0fool00oP0ooooo`16ool00`00ooooo`26ool001Goo`03003ooooo03Oo
o`03n03ooooo04Goo`03003ooooo08Koo`005Ooo00<00?ooool0>?oo00?h0?ooool0A?oo00<00?oo
ool0Q_oo000Eool00`00ooooo`0hool00oP0ooooo`15ool00`00ooooo`25ool001Goo`8003[oo`03
n03ooooo04Coo`03003ooooo08Goo`005Ooo00<00?ooool0>_oo00?h0?ooool0A?oo00<00?ooool0
Q?oo000Eool00`00ooooo`0jool00oP0ooooo`14ool00`00ooooo`24ool001Goo`03003ooooo03_o
o`03n03ooooo04Coo`03003ooooo08?oo`005Ooo00<00?ooool0??oo00?h0?ooool0@ooo00<00?oo
ool0Pooo000Eool00`00ooooo`0lool00oP0ooooo`14ool00`00ooooo`22ool001Goo`03003ooooo
03goo`03n03ooooo04?oo`03003ooooo08;oo`005Ooo0P00?ooo00?h0?ooool0@ooo00<00?ooool0
POoo000Eool00`00ooooo`0nool00oP0ooooo`13ool00`00ooooo`21ool001Goo`03003ooooo03oo
o`03n03ooooo04;oo`03003ooooo087oo`005Ooo00<00?ooool0@?oo00?h0?ooool0@_oo00<00?oo
ool0P?oo000Eool00`00ooooo`11ool00oP0ooooo`11ool00`00ooooo`20ool001Goo`03003ooooo
047oo`03n03ooooo04;oo`03003ooooo07ooo`000_oo0P001?oo00<00?ooool00ooo0P001Ooo00<0
0?ooool0@_oo00?h0?ooool0@Ooo00<00?ooool0Oooo00001Ooo003ooooo00002?oo00@00?ooool0
00Coo`03003ooooo04?oo`03n03ooooo047oo`03003ooooo07koo`0000Goo`00ooooo`0000Soo`04
003ooooo0004ool30013ool00oP0ooooo`11ool00`00ooooo`1nool00005ool00?ooool00009ool2
0005ool00`00ooooo`14ool00oP0ooooo`11ool00`00ooooo`1mool00005ool00?ooool00008ool0
1000ooooo`001?oo00<00?ooool0AOoo00?h0?ooool0@?oo00<00?ooool0OOoo00001Ooo003ooooo
00002?oo00@00?ooool000Coo`03003ooooo04Goo`03n03ooooo047oo`03003ooooo07coo`000_oo
0P002_oo0P001Ooo00<00?ooool0A_oo00?h0?ooool0@?oo00<00?ooool0O?oo000Eool00`00oooo
o`17ool00oP0ooooo`10ool00`00ooooo`1kool001Goo`03003ooooo04Soo`03n03ooooo03ooo`03
003ooooo07_oo`005Ooo0P00BOoo00?h0?ooool0?ooo00<00?ooool0Nooo000Eool00`00ooooo`19
ool00oP0ooooo`0oool00`00ooooo`1jool001Goo`03003ooooo04[oo`03n03ooooo03koo`03003o
oooo07[oo`005Ooo00<00?ooool0Booo00?h0?ooool0?_oo00<00?ooool0NOoo000Eool00`00oooo
o`1;ool00oP0ooooo`0nool00`00ooooo`1iool001Goo`03003oo`00008000<7h4Ooo`03n03ooooo
03koo`03003ooooo07Soo`005Ooo00<00?ooool00_oo40003`OP;?oo00?h0?ooool0?Ooo00<00?oo
ool0N?oo000Eool00`00ooooo`0Bool80007ool>1n0Nool00oP0ooooo`0nool00`00ooooo`1gool0
01Goo`8001_oo`/000[oo`L7h1Soo`03n03ooooo03goo`03003ooooo07Ooo`005Ooo00<00?ooool0
9Ooo20002Ooo2POP3_oo00?h0?ooool0?_oo00<00?ooool0M_oo000Eool00`00ooooo`0]ool5000>
ool71n08ool00oP0ooooo`0mool00`00ooooo`1fool001Goo`03003ooooo03;oo`@0017oo`H7h0?o
o`03n03ooooo03goo`03003ooooo07Goo`005Ooo00<00?ooool0=_oo10004ooo0`OP0_P000<7h?oo
ool0>ooo00<00?ooool0MOoo000Eool00`00ooooo`0jool4000Dool7n00hool00`00ooooo`1dool0
01Goo`03003ooooo03koo`@001Ooo`?h03Goo`03003ooooo07Coo`005Ooo0P00@ooo1P005?oo1OP0
<Ooo00<00?ooool0Looo000Eool00`00ooooo`18ool4000Eool5n00/ool00`00ooooo`1cool001Go
o`03003ooooo04coo`@001Koo`Ch02Woo`03003ooooo07;oo`005Ooo00<00?ooool0D?oo10005_oo
0oP09_oo00<00?ooool0L_oo000Eool00`00ooooo`1Dool3000Fool4n00Sool00`00ooooo`1aool0
01Goo`03003ooooo05Ooo`@001Koo`?h023oo`03003ooooo077oo`005Ooo00<00?ooool0Fooo1000
5Ooo0oP07_oo00<00?ooool0L?oo000Eool2001Pool3000Eool3n00Kool00`00ooooo`1`ool001Go
o`03003ooooo06;oo`<001Goo`;h01Woo`03003ooooo073oo`005Ooo00<00?ooool0IOoo10004ooo
0_P06?oo00<00?ooool0Kooo000Eool00`00ooooo`1Yool3000Bool2n00Fool00`00ooooo`1_ool0
01Goo`03003ooooo06coo`@0013oo`;h01Goo`03003ooooo06koo`005Ooo00<00?ooool0L?oo1000
3_oo0_P04ooo00<00?ooool0K_oo0002ool20004ool00`00ooooo`02ool30005ool00`00ooooo`1d
ool3000=ool2n00Bool00`00ooooo`1]ool00005ool00?ooool0000;ool00`00ooooo`02ool00`00
ooooo`1gool3000<ool2n00@ool00`00ooooo`1]ool00005ool00?ooool0000;ool00`00ooooo`02
ool3001jool4000:ool2n00?ool00`00ooooo`1Oool60007ool00005ool00?ooool00009ool30004
ool00`00ooooo`1nool30009ool2n00=ool00`00ooooo`1:oolE000=ool00005ool00?ooool00008
ool01000ooooo`001?oo00<00?ooool0POoo0`002?oo00?h0?ooool02ooo00<00?ooool0?_oo2`00
8_oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo08Coo`<000Koo`;h00_oo`03
003ooooo02cooa8002goo`000_oo0P002_oo0P001Ooo00<00?ooool0Qooo0`001Ooo00?h0?ooool0
2?oo00<00?ooool09_oo1P00?ooo000Eool00`00ooooo`2:ool30003ool00oP0ooooo`08ool00`00
ooooo`0Mool80015ool001Goo`03003ooooo08goo`8000;oo`;h00Soo`03003ooooo01Goo`P004go
o`005Ooo0P00T?oo100000?h0?ooool01_oo00<00?ooool03ooo1@00EOoo000Eool00`00ooooo`2C
ool70002ool00`00ooooo`0;ool4001Jool001Goo`03003ooooo09Goo`03n03ooooo00;ooa0005ko
o`005Ooo00<00?ooool0U_oo00?h0?ooool01?oo00<00?ooool0J?oo000Eool00`00ooooo`2Gool0
0oP0ooooo`04ool00`00ooooo`1Wool001Goo`03003ooooo09Soo`03n03ooooo00?oo`03003ooooo
06Ooo`005Ooo00<00?ooool0VOoo00?h0?ooool00ooo00<00?ooool0I_oo000Eool00`00ooooo`2J
ool00oP0ooooo`02ool00`00ooooo`1Vool001Goo`8009coo`03n03ooooo00;oo`03003ooooo06Go
o`005Ooo00<00?ooool0W?oo00Gh0?ooooooo`0006Ooo`005Ooo00<00?ooool0W?oo00Gh0?oooooo
o`0006Ooo`005Ooo00<00?ooool0WOoo00Gh0?ooooooo`0006Koo`005Ooo00<00?ooool0W_oo00Ch
0?ooool006Koo`005Ooo00<00?ooool0Wooo00Ch0?ooool006Goo`005Ooo00<00?ooool0Wooo00Ch
0?ooool006Goo`005Ooo0P00XOoo00Ch0?ooool006Coo`005Ooo00<00?ooool0XOoo00?h0?oo0000
I?oo000Eool00`00ooooo`2Sool00`00ooooo`1Rool001Goo`03003ooooo0:Coo`03003ooooo067o
o`005Ooo00<00?ooool0Y?oo00<00?ooool0HOoo000Eool00`00ooooo`0Hool20004ool00`00oooo
o`02ool4000Dool20004ool00`00ooooo`04ool00`00ooooo`0Cool20004ool00`00ooooo`03ool2
000Dool20004ool00`00ooooo`03ool2000Gool00`00ooooo`04000Hool40003ool00`00ooooo`02
ool4000Cool40003ool00`00ooooo`04ool00`00ooooo`0@ool001Goo`03003ooooo01Ooo`04003o
oooo0008ool00`00ooooo`0Dool01000ooooo`002_oo00<00?ooool04_oo00@00?ooool000Soo`04
003ooooo000Bool01000ooooo`002?oo00@00?ooool001Koo`03003ooooo00;oo`03003ooooo01Wo
o`03003ooooo00Ooo`03003ooooo01Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo00<00?oo
ool05ooo00@00?ooool000Woo`03003ooooo01?oo`04003ooooo0007ool5000Cool01000ooooo`00
2?oo00@00?ooool001;oo`04003ooooo0008ool01000ooooo`005ooo00D00?ooooooo`0001_oo`03
003ooooo00Soo`03003ooooo01Goo`03003ooooo00Koo`D0017oo`005Ooo0P006?oo00@00?ooool0
00[oo`03003ooooo01;oo`04003ooooo0007ool01000ooooo`005?oo00@00?ooool000Soo`<001?o
o`04003ooooo0009ool2000Hool01@00oooooooo00006ooo00<00?ooool02Ooo00<00?ooool05?oo
00<00?ooool01_oo00@00?ooool001;oo`005Ooo00<00?ooool05ooo00@00?ooool000_oo`03003o
oooo017oo`04003ooooo0008ool00`00ool0000Dool01000ooooo`002?oo00<00?ooool04ooo00@0
0?ooool000Soo`04003ooooo000Hool01000ooooo`006ooo00<00?ooool02_oo00<00?ooool04ooo
00<00?ooool01ooo00<00?oo00004_oo000Eool00`00ooooo`0Gool01000ooooo`002?oo00@00?oo
ool001?oo`04003ooooo0009ool2000Dool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool0
00Soo`04003ooooo000Hool4000Iool30009ool01000ooooo`004ooo0`002_oo0P004_oo000Eool0
0`00ooooo`0Hool2000:ool2000Eool2000;ool00`00ooooo`0Cool2000:ool3000Cool2000:ool2
000Iool01000ooooo`006ooo00<00?ooool02?oo0P005_oo00<00?ooool02Ooo00<00?ooool04?oo
000Eool00`00ooooo`2Xool00`00ooooo`1Mool001Goo`03003ooooo0:Soo`03003ooooo05goo`00
5Ooo00<00?ooool0ZOoo00<00?ooool0G?oo000>oooo000C000001Goo`03003ooooo00Koo`03003o
oooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003o
oooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003o
oooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003o
oooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`005Ooo
00<00?ooool07ooo00<00?ooool08?oo00<00?ooool08?oo00<00?ooool07ooo00<00?ooool08?oo
00<00?ooool07ooo00<00?ooool08?oo00<00?ooool05_oo000Eool00`00ooooo`2Zool00`00oooo
o`1Kool001Goo`03003ooooo0:_oo`03003ooooo05[oo`005Ooo00<00?ooool0Zooo00<00?ooool0
F_oo000Eool00`00ooooo`2/ool00`00ooooo`1Iool001Goo`03003ooooo0:coo`03003ooooo05Wo
o`005Ooo0P00[_oo00<00?ooool0F?oo000Eool00`00ooooo`2]ool00`00ooooo`1Hool001Goo`03
003ooooo0:koo`03003ooooo05Ooo`005Ooo00<00?ooool0[_oo00<00?ooool0Eooo000Eool00`00
ooooo`2_ool00`00ooooo`1Fool001Goo`03003ooooo0:ooo`03003ooooo05Koo`005Ooo00<00?oo
ool0/?oo00<00?ooool0EOoo000Eool2002aool00`00ooooo`1Eool001Goo`03003ooooo0;7oo`03
003ooooo05Coo`005Ooo00<00?ooool0/Ooo00<00?ooool0E?oo000Eool00`00ooooo`2bool00`00
ooooo`1Cool001Goo`03003ooooo0;;oo`03003ooooo05?oo`005Ooo00<00?ooool0/ooo00<00?oo
ool0D_oo000Eool00`00ooooo`2cool00`00ooooo`1Bool001Goo`03003ooooo0;Coo`03003ooooo
057oo`005Ooo0P00]Ooo00<00?ooool0DOoo000Eool00`00ooooo`2eool00`00ooooo`1@ool001Go
o`03003ooooo0;Goo`03003ooooo053oo`005Ooo00<00?ooool0]Ooo00<00?ooool0D?oo000Eool0
0`00ooooo`2fool00`00ooooo`1?ool001Goo`03003ooooo0;Koo`03003ooooo04ooo`005Ooo00<0
0?ooool0]ooo00<00?ooool0C_oo000Eool2002hool00`00ooooo`1>ool001Goo`03003ooooo0;So
o`03003ooooo04goo`005Ooo00<00?ooool0^?oo00<00?ooool0COoo000Eool00`00ooooo`2iool0
0`00ooooo`1<ool001Goo`03003ooooo0;Woo`03003ooooo04coo`005Ooo00<00?ooool0^_oo00<0
0?ooool0Booo000Eool00`00ooooo`2jool00`00ooooo`1;ool00?ooob7oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123894, 0.634549, \
0.00578885, 0.00267021}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{mGamma /. strangesol[k, 100]}, {k, 0.01, 1.5}, 
      PlotRange \[Rule] All]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0174177 0.639182 0.612385 3.96102 [
[.14525 .59989 -9 -9 ]
[.14525 .59989 9 0 ]
[.27309 .59989 -9 -9 ]
[.27309 .59989 9 0 ]
[.40093 .59989 -9 -9 ]
[.40093 .59989 9 0 ]
[.52876 .59989 -9 -9 ]
[.52876 .59989 9 0 ]
[.6566 .59989 -3 -9 ]
[.6566 .59989 3 0 ]
[.78444 .59989 -9 -9 ]
[.78444 .59989 9 0 ]
[.91227 .59989 -9 -9 ]
[.91227 .59989 9 0 ]
[.00492 .05784 -30 -4.5 ]
[.00492 .05784 0 4.5 ]
[.00492 .13706 -30 -4.5 ]
[.00492 .13706 0 4.5 ]
[.00492 .21628 -24 -4.5 ]
[.00492 .21628 0 4.5 ]
[.00492 .2955 -30 -4.5 ]
[.00492 .2955 0 4.5 ]
[.00492 .37472 -30 -4.5 ]
[.00492 .37472 0 4.5 ]
[.00492 .45394 -30 -4.5 ]
[.00492 .45394 0 4.5 ]
[.00492 .53316 -30 -4.5 ]
[.00492 .53316 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61864 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.14525 .61239 m
.14525 .61864 L
s
[(0.2)] .14525 .59989 0 1 Mshowa
.27309 .61239 m
.27309 .61864 L
s
[(0.4)] .27309 .59989 0 1 Mshowa
.40093 .61239 m
.40093 .61864 L
s
[(0.6)] .40093 .59989 0 1 Mshowa
.52876 .61239 m
.52876 .61864 L
s
[(0.8)] .52876 .59989 0 1 Mshowa
.6566 .61239 m
.6566 .61864 L
s
[(1)] .6566 .59989 0 1 Mshowa
.78444 .61239 m
.78444 .61864 L
s
[(1.2)] .78444 .59989 0 1 Mshowa
.91227 .61239 m
.91227 .61864 L
s
[(1.4)] .91227 .59989 0 1 Mshowa
.125 Mabswid
.04938 .61239 m
.04938 .61614 L
s
.08134 .61239 m
.08134 .61614 L
s
.11329 .61239 m
.11329 .61614 L
s
.17721 .61239 m
.17721 .61614 L
s
.20917 .61239 m
.20917 .61614 L
s
.24113 .61239 m
.24113 .61614 L
s
.30505 .61239 m
.30505 .61614 L
s
.33701 .61239 m
.33701 .61614 L
s
.36897 .61239 m
.36897 .61614 L
s
.43289 .61239 m
.43289 .61614 L
s
.46484 .61239 m
.46484 .61614 L
s
.4968 .61239 m
.4968 .61614 L
s
.56072 .61239 m
.56072 .61614 L
s
.59268 .61239 m
.59268 .61614 L
s
.62464 .61239 m
.62464 .61614 L
s
.68856 .61239 m
.68856 .61614 L
s
.72052 .61239 m
.72052 .61614 L
s
.75248 .61239 m
.75248 .61614 L
s
.8164 .61239 m
.8164 .61614 L
s
.84835 .61239 m
.84835 .61614 L
s
.88031 .61239 m
.88031 .61614 L
s
.94423 .61239 m
.94423 .61614 L
s
.97619 .61239 m
.97619 .61614 L
s
.25 Mabswid
0 .61239 m
1 .61239 L
s
.01742 .05784 m
.02367 .05784 L
s
[(-0.14)] .00492 .05784 1 0 Mshowa
.01742 .13706 m
.02367 .13706 L
s
[(-0.12)] .00492 .13706 1 0 Mshowa
.01742 .21628 m
.02367 .21628 L
s
[(-0.1)] .00492 .21628 1 0 Mshowa
.01742 .2955 m
.02367 .2955 L
s
[(-0.08)] .00492 .2955 1 0 Mshowa
.01742 .37472 m
.02367 .37472 L
s
[(-0.06)] .00492 .37472 1 0 Mshowa
.01742 .45394 m
.02367 .45394 L
s
[(-0.04)] .00492 .45394 1 0 Mshowa
.01742 .53316 m
.02367 .53316 L
s
[(-0.02)] .00492 .53316 1 0 Mshowa
.125 Mabswid
.01742 .07765 m
.02117 .07765 L
s
.01742 .09745 m
.02117 .09745 L
s
.01742 .11726 m
.02117 .11726 L
s
.01742 .15687 m
.02117 .15687 L
s
.01742 .17667 m
.02117 .17667 L
s
.01742 .19648 m
.02117 .19648 L
s
.01742 .23609 m
.02117 .23609 L
s
.01742 .25589 m
.02117 .25589 L
s
.01742 .2757 m
.02117 .2757 L
s
.01742 .31531 m
.02117 .31531 L
s
.01742 .33511 m
.02117 .33511 L
s
.01742 .35492 m
.02117 .35492 L
s
.01742 .39453 m
.02117 .39453 L
s
.01742 .41433 m
.02117 .41433 L
s
.01742 .43414 m
.02117 .43414 L
s
.01742 .47375 m
.02117 .47375 L
s
.01742 .49355 m
.02117 .49355 L
s
.01742 .51336 m
.02117 .51336 L
s
.01742 .55297 m
.02117 .55297 L
s
.01742 .57278 m
.02117 .57278 L
s
.01742 .59258 m
.02117 .59258 L
s
.01742 .03804 m
.02117 .03804 L
s
.01742 .01823 m
.02117 .01823 L
s
.25 Mabswid
.01742 0 m
.01742 .61803 L
s
.5 Mabswid
.02381 .60332 m
.06244 .54871 L
.10458 .4902 L
.14415 .43704 L
.18221 .38816 L
.22272 .33906 L
.26171 .29507 L
.30316 .2522 L
.34309 .21505 L
.3815 .18356 L
.42237 .15522 L
.44268 .14338 L
.46172 .13383 L
.48111 .12582 L
.49955 .12001 L
.50923 .11774 L
.51435 .11678 L
.5198 .11595 L
.52484 .11535 L
.52941 .11497 L
.53161 .11484 L
.53277 .11478 L
.534 .11474 L
.53507 .11471 L
.53625 .11468 L
.53732 .11467 L
.53832 .11466 L
.53958 .11467 L
.54022 .11468 L
.54092 .11469 L
.54216 .11472 L
.54333 .11476 L
.5447 .11482 L
.54595 .11489 L
.54877 .11509 L
.55108 .1153 L
.5536 .11559 L
.55868 .11633 L
.56332 .1172 L
.56829 .11834 L
.57723 .12096 L
.58747 .12484 L
.59858 .13003 L
.61873 .14136 L
.63821 .15237 L
.64355 .15493 L
.64929 .15732 L
.65466 .15917 L
.65965 .16053 L
.6647 .16156 L
Mistroke
.66737 .16195 L
.66874 .16212 L
.67021 .16227 L
.67162 .16239 L
.67292 .16247 L
.67416 .16253 L
.67547 .16258 L
.67665 .1626 L
.67775 .1626 L
.67895 .16259 L
.68024 .16256 L
.6815 .16251 L
.68266 .16245 L
.6853 .16225 L
.68762 .16202 L
.69009 .16171 L
.69905 .16014 L
.70845 .15781 L
.71856 .15467 L
.73685 .14776 L
.77805 .1285 L
.81772 .10739 L
.85588 .08588 L
.8965 .06225 L
.93559 .03907 L
.97619 .01472 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`03003o
oooo0?coo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`00
8Ooo00<00?ooool0m?oo00<00?ooool01Ooo000Qool2003cool20008ool0027oo`03003ooooo0?3o
o`8000[oo`008Ooo00<00?ooool0kooo00<00?ooool02_oo000Qool00`00ooooo`3]ool2000=ool0
027oo`03003ooooo0>_oo`8000ooo`008Ooo0P00jooo00<00?ooool03ooo000Qool00`00ooooo`3X
ool2000Bool0027oo`03003ooooo0>Koo`8001Coo`002?oo0P001?oo00<00?ooool00_oo10001?oo
00<00?ooool00ooo00<00?ooool0iOoo00<00?ooool05?oo0007ool01000ooooo`002_oo00<00?oo
ool00ooo00<00?ooool00ooo00<00?ooool0hooo0P005ooo0007ool01000ooooo`002_oo00<00?oo
ool01@001?oo0P00h_oo0P006Ooo00000ooo000000020002ool01000ooooo`002_oo00@00?ooool0
00;oo`03003ooooo00?oo`03003ooooo0>3oo`03003ooooo01Woo`001ooo00@00?ooool000[oo`07
003oooooool00?oo00001Ooo00<00?ooool0g_oo0P007?oo0007ool01000ooooo`002?oo0`001?oo
0P001Ooo00<00?ooool0g?oo0P007_oo0008ool2000;ool00`00ooooo`03ool00`00ooooo`03ool0
0`00ooooo`3Jool2000Pool0027oo`800=Woo`8002;oo`008Ooo00<00?ooool0e_oo0P009?oo000Q
ool00`00ooooo`3Eool00`00ooooo`0Tool0027oo`03003ooooo0=?oo`8002Ooo`008Ooo00<00?oo
ool0dOoo0P00:Ooo000Qool2003Aool00`00ooooo`0Yool0027oo`03003ooooo0<koo`8002coo`00
8Ooo00<00?ooool0c?oo0P00;_oo000Qool00`00ooooo`3:ool2000`ool0027oo`03003ooooo0<So
o`8003;oo`008Ooo0P00O_oo3P00>ooo0P00=?oo000Qool00`00ooooo`1hool5000>ool4000eool2
000fool0027oo`03003ooooo07Koo`8001Ooo`80037oo`8003Soo`008Ooo00<00?ooool0Looo0`00
6ooo0P00;Ooo0P00>_oo0008ool20004ool00`00ooooo`02ool40002ool40004ool00`00ooooo`1a
ool2000Pool00`00ooooo`0Xool2000lool000Ooo`04003ooooo000:ool01@00oooooooo00001ooo
00<00?ooool0K_oo0`008ooo0P009_oo0P00?_oo0007ool01000ooooo`002_oo00<00?ooool00_oo
00<00?ooool01?oo0P00KOoo0P00:?oo0P008_oo0P00@?oo00000ooo000000020002ool01000oooo
o`002_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool0J_oo0P00;?oo0P007_oo0P00@_oo
0007ool01000ooooo`002_oo00<00?ooool01?oo00<00?ooool00_oo00<00?ooool0J?oo0P00<?oo
0P006_oo0P00A?oo0007ool01000ooooo`002?oo0`000ooo00@00?ooool000Coo`03003ooooo06Ko
o`8003Coo`8001Koo`8004Koo`002?oo0P002ooo00<00?ooool00_oo0P001Ooo00<00?ooool0IOoo
00<00?ooool0=_oo00<00?ooool04?oo0`00B?oo000Qool2001Uool00`00ooooo`0hool3000<ool3
001;ool0027oo`03003ooooo06;oo`8003koo``004koo`008Ooo00<00?ooool0HOoo00<00?ooool0
V?oo000Qool00`00ooooo`1Pool00`00ooooo`2Iool0027oo`03003ooooo05ooo`03003ooooo09[o
o`008Ooo0P00G_oo0P00WOoo000Qool00`00ooooo`1Lool00`00ooooo`2Mool0027oo`03003ooooo
05_oo`03003ooooo09koo`008Ooo00<00?ooool0F_oo00<00?ooool0Wooo000Qool00`00ooooo`1H
ool2002Rool0027oo`8005Soo`03003ooooo0:;oo`008Ooo00<00?ooool0E_oo00<00?ooool0Xooo
000Qool00`00ooooo`1Eool00`00ooooo`2Tool000koo`8000Coo`03003ooooo00;oo`@000Coo`03
003ooooo05?oo`800:Ooo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo05;oo`03
003ooooo0:Ooo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`8005;oo`03003ooooo0:Soo`00
1ooo10000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo053oo`03003ooooo0:Woo`00
3Ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo04ooo`03003ooooo0:[oo`003Ooo00@0
0?ooool000Soo`<000Goo`03003ooooo04koo`03003ooooo0:_oo`003_oo0P002ooo00<00?ooool0
0ooo00<00?ooool0C?oo0P00[_oo000Qool2001<ool00`00ooooo`2^ool0027oo`03003ooooo04[o
o`03003ooooo0:ooo`008Ooo00<00?ooool0BOoo00<00?ooool0/?oo000Qool00`00ooooo`18ool0
0`00ooooo`2aool0027oo`03003ooooo04Ooo`03003ooooo0;;oo`008Ooo0P00Aooo00<00?ooool0
/ooo000Qool00`00ooooo`15ool00`00ooooo`2dool0027oo`03003ooooo04Coo`03003ooooo0;Go
o`008Ooo00<00?ooool0@ooo00<00?ooool0]_oo000Qool00`00ooooo`12ool00`00ooooo`2gool0
027oo`8004;oo`03003ooooo0;Soo`008Ooo00<00?ooool0@?oo00<00?ooool0^Ooo000Qool00`00
ooooo`0oool00`00ooooo`2jool000Soo`8000Coo`03003ooooo00?oo`8000Coo`8000Goo`03003o
oooo03koo`03003ooooo0;_oo`001ooo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`00
1?oo00<00?ooool0?Ooo00<00?ooool0_?oo0007ool01000ooooo`002?oo00@00?ooool000;oo`04
003ooooo0004ool2000mool00`00ooooo`2mool00003ool00000008000;oo`04003ooooo0008ool0
1000ooooo`000ooo0P001Ooo00<00?ooool0>ooo00<00?ooool0__oo0007ool01000ooooo`002?oo
00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`0jool00`00ooooo`2oool000Ooo`04003o
oooo0008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo03[oo`03003ooooo0;ooo`00
2?oo0P002_oo0P001?oo0P001Ooo00<00?ooool0>Ooo00<00?ooool0`?oo000Qool00`00ooooo`0h
ool00`00ooooo`31ool0027oo`8003Soo`03003ooooo0<;oo`008Ooo00<00?ooool0=_oo00<00?oo
ool0`ooo000Qool00`00ooooo`0eool00`00ooooo`34ool0027oo`03003ooooo03Goo`03003ooooo
0<Coo`008Ooo00<00?ooool0=?oo00<00?ooool0aOoo000Qool2000dool00`00ooooo`36ool0027o
o`03003ooooo03;oo`03003ooooo0<Ooo`008Ooo00<00?ooool0<Ooo00<00?ooool0b?oo000Qool0
0`00ooooo`0`ool00`00ooooo`39ool0027oo`03003ooooo02ooo`03003ooooo0<[oo`008Ooo0P00
;ooo00<00?ooool0booo000Qool00`00ooooo`0]ool00`00ooooo`3<ool0027oo`03003ooooo02go
o`03003ooooo0<coo`002?oo0P001?oo00<00?ooool00ooo0P001?oo0P001Ooo00<00?ooool0;?oo
00<00?ooool0cOoo0007ool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0004ool00`00
ooooo`0[ool00`00ooooo`3>ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool0
00Coo`8002_oo`03003ooooo0<ooo`0000?oo`0000000P000_oo00@00?ooool000Soo`04003ooooo
0002ool30005ool00`00ooooo`0Yool00`00ooooo`3@ool000Ooo`04003ooooo0008ool01000oooo
o`000_oo00<00?ooool01Ooo00<00?ooool0:?oo00<00?ooool0dOoo0007ool01000ooooo`002?oo
00@00?ooool000;oo`03003ooooo00Goo`03003ooooo02Ooo`03003ooooo0=;oo`002?oo0P002_oo
0P001?oo0`001?oo00<00?ooool09_oo00<00?ooool0dooo000Qool2000Vool00`00ooooo`3Dool0
027oo`03003ooooo02Goo`03003ooooo0=Coo`008Ooo00<00?ooool09?oo00<00?ooool0eOoo000Q
ool00`00ooooo`0Sool00`00ooooo`3Fool0027oo`03003ooooo02;oo`03003ooooo0=Ooo`008Ooo
0P008ooo00<00?ooool0eooo000Qool00`00ooooo`0Qool00`00ooooo`3Hool0027oo`03003ooooo
023oo`03003ooooo0=Woo`008Ooo00<00?ooool07ooo00<00?ooool0f_oo000Qool00`00ooooo`0O
ool00`00ooooo`3Jool0027oo`8001ooo`03003ooooo0=_oo`008Ooo00<00?ooool07Ooo00<00?oo
ool0g?oo000Qool00`00ooooo`0Lool00`00ooooo`3Mool000Soo`8000Coo`03003ooooo00?oo`80
00Goo`03003ooooo00?oo`03003ooooo01coo`03003ooooo0=goo`001ooo00@00?ooool000Soo`04
003ooooo0004ool00`00ooooo`03ool00`00ooooo`0Kool00`00ooooo`3Nool000Ooo`04003ooooo
0008ool01@00ooooo`00ool01@001?oo0P006ooo00<00?ooool0gooo00000ooo000000020002ool0
1000ooooo`002?oo00H00?ooool00?oo0002ool00`00ooooo`03ool00`00ooooo`0Iool00`00oooo
o`3Pool000Ooo`04003ooooo0008ool01000ooooo`000_oo00<00?oo00001Ooo00<00?ooool06Ooo
00<00?ooool0h?oo0007ool01000ooooo`002?oo00@00?ooool000?oo`8000Goo`03003ooooo01So
o`03003ooooo0>7oo`002?oo0P002_oo0P001Ooo00<00?ooool00ooo00<00?ooool05ooo00<00?oo
ool0h_oo000Qool2000Hool00`00ooooo`3Rool0027oo`03003ooooo01Koo`03003ooooo0>?oo`00
8Ooo00<00?ooool05Ooo00<00?ooool0i?oo000Qool00`00ooooo`0Dool00`00ooooo`3Uool0027o
o`03003ooooo01Coo`03003ooooo0>Goo`008Ooo00<00?ooool04ooo00<00?ooool0i_oo000Qool2
000Cool00`00ooooo`3Wool0027oo`03003ooooo01;oo`03003ooooo0>Ooo`008Ooo00<00?ooool0
4Ooo00<00?ooool0j?oo000Qool00`00ooooo`0@ool00`00ooooo`3Yool0027oo`03003ooooo00oo
o`03003ooooo0>[oo`008Ooo0P004?oo00<00?ooool0j_oo000Qool00`00ooooo`0>ool00`00oooo
o`3[ool0027oo`03003ooooo00goo`03003ooooo0>coo`002?oo0P001?oo00<00?ooool00ooo0P00
0ooo10001?oo00<00?ooool03?oo00<00?ooool0kOoo0007ool01000ooooo`002?oo00@00?ooool0
00;oo`03003ooooo00Goo`03003ooooo00coo`03003ooooo0>goo`001ooo00@00?ooool000Soo`04
003ooooo0003ool00`00ooooo`04ool2000<ool00`00ooooo`3^ool00003ool00000008000;oo`04
003ooooo0008ool01000ooooo`001?oo00<00?ooool00ooo00<00?ooool02_oo00<00?ooool0kooo
0007ool01000ooooo`002?oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo00Woo`03003o
oooo0?3oo`001ooo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool0
2Ooo00<00?ooool0l?oo0008ool2000:ool20004ool20005ool00`00ooooo`08ool00`00ooooo`3a
ool0027oo`8000Soo`03003ooooo0?;oo`008Ooo00<00?ooool01ooo00<00?ooool0l_oo000Qool0
0`00ooooo`06ool00`00ooooo`3cool0027oo`03003ooooo00Goo`03003ooooo0?Coo`008Ooo00<0
0?ooool01?oo00<00?ooool0mOoo000Qool20005ool00`00ooooo`0@ool20004ool00`00ooooo`02
ool4000Bool20004ool00`00ooooo`04ool00`00ooooo`0Aool20004ool00`00ooooo`03ool2000C
ool20004ool00`00ooooo`03ool2000Hool4000Gool40003ool00`00ooooo`02ool4000Aool40003
ool00`00ooooo`04ool00`00ooooo`0?ool0027oo`03003ooooo00?oo`03003ooooo013oo`04003o
oooo0008ool00`00ooooo`0Bool01000ooooo`002_oo00<00?ooool04?oo00@00?ooool000Soo`04
003ooooo000Aool01000ooooo`002?oo00@00?ooool001Woo`03003ooooo01Soo`03003ooooo00Oo
o`03003ooooo01Coo`03003ooooo00Woo`03003ooooo00ooo`008Ooo00<00?ooool00_oo00<00?oo
ool04Ooo00@00?ooool000Woo`03003ooooo017oo`04003ooooo0007ool5000Aool01000ooooo`00
2?oo00@00?ooool0017oo`04003ooooo0008ool01000ooooo`006Ooo00<00?ooool06?oo00<00?oo
ool02?oo00<00?ooool04ooo00<00?ooool01_oo1@004?oo000Qool00`00ooooo`02ool00`00oooo
o`0Aool01000ooooo`002_oo00<00?ooool04?oo00@00?ooool000Ooo`04003ooooo000Bool01000
ooooo`002?oo0`004_oo00@00?ooool000Woo`8001[oo`03003ooooo01Soo`03003ooooo00Woo`03
003ooooo01;oo`03003ooooo00Koo`04003ooooo000Aool0027oo`05003oooooool0000Dool01000
ooooo`002ooo00<00?ooool03ooo00@00?ooool000Soo`03003oo`0001;oo`04003ooooo0008ool0
0`00ooooo`0Bool01000ooooo`002?oo00@00?ooool001Woo`03003ooooo01Soo`03003ooooo00[o
o`03003ooooo017oo`03003ooooo00Ooo`03003oo`00017oo`008Ooo0P0000?oo`00ool05?oo00@0
0?ooool000Soo`04003ooooo000Aool01000ooooo`002Ooo0P004_oo00@00?ooool000Soo`03003o
oooo01;oo`04003ooooo0008ool01000ooooo`005ooo0`006?oo0`002Ooo00@00?ooool0017oo`<0
00[oo`80017oo`008Ooo00<00?oo00005ooo0P002_oo0P004ooo0P002ooo00<00?ooool04Ooo0P00
2_oo0`004_oo0P002_oo0P006_oo00<00?ooool06?oo00<00?ooool02?oo0P005?oo00<00?ooool0
2Ooo00<00?ooool03ooo000Qool00`00ool0003lool0027oo`800?goo`008Ooo00<00?ooool0o?oo
000Loooo00050000027oo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`008Ooo00<00?ooool07_oo00<00?ooool07_oo00<0
0?ooool07_oo00<00?ooool07_oo00<00?ooool07_oo00<00?ooool07_oo00<00?ooool07_oo00<0
0?ooool05Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003o
oolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.20129, -0.162991, \
0.00605766, 0.000977512}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9226110483737413`, 
      mGamma \[Rule] \(-0.11427496829226737`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.001, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8344913387822289`, 
      mGamma \[Rule] 2.3105048691169752`*^-7}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.001, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.6253371629361478`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol2[0.01, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8344979855070702`}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["xi=10^4", "Subsubsection"],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.85, 
          0.88}, {mGamma, \(-0.07\), \(-0.12\)}, 
        MaxIterations \[Rule] 120]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, Om /. strangesol[k, 10^4]}, {k, 0.01, 1.5}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -9.57197 11.126 [
[.15079 .42895 -9 -9 ]
[.15079 .42895 9 0 ]
[.27778 .42895 -9 -9 ]
[.27778 .42895 9 0 ]
[.40476 .42895 -9 -9 ]
[.40476 .42895 9 0 ]
[.53175 .42895 -9 -9 ]
[.53175 .42895 9 0 ]
[.65873 .42895 -3 -9 ]
[.65873 .42895 3 0 ]
[.78571 .42895 -9 -9 ]
[.78571 .42895 9 0 ]
[.9127 .42895 -9 -9 ]
[.9127 .42895 9 0 ]
[.01131 .10767 -24 -4.5 ]
[.01131 .10767 0 4.5 ]
[.01131 .21893 -24 -4.5 ]
[.01131 .21893 0 4.5 ]
[.01131 .33019 -24 -4.5 ]
[.01131 .33019 0 4.5 ]
[.01131 .55271 -24 -4.5 ]
[.01131 .55271 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .44145 m
.15079 .4477 L
s
[(0.2)] .15079 .42895 0 1 Mshowa
.27778 .44145 m
.27778 .4477 L
s
[(0.4)] .27778 .42895 0 1 Mshowa
.40476 .44145 m
.40476 .4477 L
s
[(0.6)] .40476 .42895 0 1 Mshowa
.53175 .44145 m
.53175 .4477 L
s
[(0.8)] .53175 .42895 0 1 Mshowa
.65873 .44145 m
.65873 .4477 L
s
[(1)] .65873 .42895 0 1 Mshowa
.78571 .44145 m
.78571 .4477 L
s
[(1.2)] .78571 .42895 0 1 Mshowa
.9127 .44145 m
.9127 .4477 L
s
[(1.4)] .9127 .42895 0 1 Mshowa
.125 Mabswid
.05556 .44145 m
.05556 .4452 L
s
.0873 .44145 m
.0873 .4452 L
s
.11905 .44145 m
.11905 .4452 L
s
.18254 .44145 m
.18254 .4452 L
s
.21429 .44145 m
.21429 .4452 L
s
.24603 .44145 m
.24603 .4452 L
s
.30952 .44145 m
.30952 .4452 L
s
.34127 .44145 m
.34127 .4452 L
s
.37302 .44145 m
.37302 .4452 L
s
.43651 .44145 m
.43651 .4452 L
s
.46825 .44145 m
.46825 .4452 L
s
.5 .44145 m
.5 .4452 L
s
.56349 .44145 m
.56349 .4452 L
s
.59524 .44145 m
.59524 .4452 L
s
.62698 .44145 m
.62698 .4452 L
s
.69048 .44145 m
.69048 .4452 L
s
.72222 .44145 m
.72222 .4452 L
s
.75397 .44145 m
.75397 .4452 L
s
.81746 .44145 m
.81746 .4452 L
s
.84921 .44145 m
.84921 .4452 L
s
.88095 .44145 m
.88095 .4452 L
s
.94444 .44145 m
.94444 .4452 L
s
.97619 .44145 m
.97619 .4452 L
s
.25 Mabswid
0 .44145 m
1 .44145 L
s
.02381 .10767 m
.03006 .10767 L
s
[(0.87)] .01131 .10767 1 0 Mshowa
.02381 .21893 m
.03006 .21893 L
s
[(0.88)] .01131 .21893 1 0 Mshowa
.02381 .33019 m
.03006 .33019 L
s
[(0.89)] .01131 .33019 1 0 Mshowa
.02381 .55271 m
.03006 .55271 L
s
[(0.91)] .01131 .55271 1 0 Mshowa
.125 Mabswid
.02381 .01866 m
.02756 .01866 L
s
.02381 .04091 m
.02756 .04091 L
s
.02381 .06316 m
.02756 .06316 L
s
.02381 .08541 m
.02756 .08541 L
s
.02381 .12992 m
.02756 .12992 L
s
.02381 .15217 m
.02756 .15217 L
s
.02381 .17442 m
.02756 .17442 L
s
.02381 .19667 m
.02756 .19667 L
s
.02381 .24118 m
.02756 .24118 L
s
.02381 .26343 m
.02756 .26343 L
s
.02381 .28568 m
.02756 .28568 L
s
.02381 .30793 m
.02756 .30793 L
s
.02381 .35244 m
.02756 .35244 L
s
.02381 .37469 m
.02756 .37469 L
s
.02381 .39694 m
.02756 .39694 L
s
.02381 .41919 m
.02756 .41919 L
s
.02381 .4637 m
.02756 .4637 L
s
.02381 .48595 m
.02756 .48595 L
s
.02381 .5082 m
.02756 .5082 L
s
.02381 .53045 m
.02756 .53045 L
s
.02381 .57496 m
.02756 .57496 L
s
.02381 .59721 m
.02756 .59721 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.57005 0 m
.58126 .19653 L
s
.58126 .19653 m
.60532 .61803 L
s
.03016 .22598 m
.03239 .22599 L
.03477 .226 L
.03908 .22602 L
.04419 .22606 L
.04884 .22611 L
.05897 .22624 L
.06854 .22641 L
.08861 .22689 L
.11039 .22761 L
.1301 .22845 L
.15098 .22953 L
.18857 .23201 L
.22859 .23544 L
.2671 .23959 L
.28668 .24205 L
.30805 .24503 L
.3105 .24539 L
.31315 .24579 L
.31436 .24597 L
.31565 .24616 L
.31675 .24632 L
.31795 .2449 L
.31867 .24663 L
.31933 .24674 L
.32081 .24697 L
.32147 .24708 L
.32218 .24718 L
.32348 .2457 L
.32475 .24754 L
.32539 .2477 L
.32609 .24777 L
.32734 .24797 L
.3285 .24823 L
.33757 .24972 L
.34006 .25017 L
.34271 .2506 L
.344 .25081 L
.34522 .251 L
.34631 .24033 L
.34749 .25287 L
.34867 .26816 L
.34996 .38829 L
.35061 .25202 L
.35131 .37347 L
.35257 .24182 L
.35383 .27433 L
.35502 .25281 L
.35609 .251 L
.35726 .25112 L
Mistroke
.35796 .25129 L
.35862 .25141 L
.35984 .25373 L
.36053 .25184 L
.36117 .25189 L
.36186 .44949 L
.36259 .35363 L
.36382 .2545 L
Mfstroke
.36382 .2545 m
.36405 .61803 L
s
.36595 .61803 m
.3662 .25278 L
.3669 .38068 L
.36754 .39041 L
.36868 .25323 L
.36989 .40378 L
.37093 .56383 L
.37207 .43499 L
.37329 .42634 L
.37398 .25429 L
.37461 .25665 L
.37577 .27309 L
.37699 .25715 L
.37806 .25505 L
.3792 .2576 L
.38028 .25782 L
.38126 .25805 L
.38241 .25828 L
.38366 .25853 L
s
.38366 .25853 m
.38452 .61803 L
s
.38512 .61803 m
.38592 .25663 L
.38717 .2804 L
.38836 .25957 L
.38942 .25735 L
.39058 .26007 L
.39184 .26035 L
.39247 .26049 L
s
.39247 .26049 m
.39289 .61803 L
s
.39389 .61803 m
.39441 .43901 L
.39558 .43918 L
.39668 .26148 L
.39784 .43825 L
.39883 .26197 L
.39992 .4407 L
.40109 .46349 L
.40234 .2628 L
.40352 .26306 L
.4046 .26332 L
.42512 .26856 L
.4648 .28075 L
.50298 .29629 L
.5227 .30671 L
.54359 .32086 L
.55295 .32881 L
.56301 .33909 L
.58119 .36495 L
.58606 .37358 L
.58873 .37808 L
.59004 .37999 L
.59124 .38144 L
.59194 .38209 L
.59267 .38259 L
.59341 .38287 L
.59422 .38287 L
.5949 .38261 L
.59562 .38207 L
.59693 .38045 L
.59834 .37796 L
.59969 .37524 L
.6022 .36999 L
.60684 .36158 L
.61176 .35465 L
.61717 .34875 L
.62216 .34436 L
.63148 .33797 L
.64152 .33277 L
.65968 .32592 L
.6796 .32062 L
.70057 .31649 L
.73844 .31119 L
.75794 .30915 L
.77875 .30732 L
.81999 .30445 L
.85973 .30235 L
.89796 .30075 L
.93862 .29938 L
.97619 .29832 L
Mistroke
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0S_oo00<00?ooool0
LOoo000Kool00`00ooooo`2>ool00`00ooooo`1aool001_oo`03003ooooo08koo`03003ooooo077o
o`006ooo00<00?ooool0S_oo00<00?ooool0LOoo000Kool00`00ooooo`2>ool00`00ooooo`1aool0
01_oo`8008ooo`03003ooooo077oo`006ooo00<00?ooool0S_oo00<00?ooool0LOoo000Kool00`00
ooooo`2>ool00`00ooooo`1aool001_oo`03003ooooo08koo`03003ooooo077oo`006ooo00<00?oo
ool0Sooo00<00?ooool0L?oo000Kool00`00ooooo`2?ool00`00ooooo`1`ool001_oo`80093oo`03
003ooooo073oo`006ooo00<00?ooool0Sooo00<00?ooool0L?oo000Kool00`00ooooo`2?ool00`00
ooooo`1`ool001_oo`03003ooooo08ooo`03003ooooo073oo`006ooo00<00?ooool0Sooo00<00?oo
ool0L?oo000Kool00`00ooooo`2?ool00`00ooooo`1`ool001_oo`80093oo`03003ooooo073oo`00
6ooo00<00?ooool0Sooo00<00?ooool0L?oo000Kool00`00ooooo`2?ool00`00ooooo`1`ool001_o
o`03003ooooo08ooo`03003ooooo073oo`006ooo00<00?ooool0Sooo00<00?ooool0L?oo000Kool0
0`00ooooo`2?ool00`00ooooo`1`ool001_oo`80093oo`03003ooooo073oo`006ooo00<00?ooool0
Sooo00<00?ooool0L?oo000Kool00`00ooooo`2?ool00`00ooooo`1`ool000;oo`8000Coo`03003o
oooo00?oo`8000Coo`03003ooooo00Coo`03003ooooo08ooo`03003ooooo073oo`0000Goo`00oooo
o`0000Soo`04003ooooo0003ool00`00ooooo`04ool00`00ooooo`2@ool00`00ooooo`1_ool00005
ool00?ooool00008ool01000ooooo`001?oo00<00?ooool00ooo0P00TOoo00<00?ooool0Kooo0000
1Ooo003ooooo00002Ooo0P001Ooo00<00?ooool00ooo00<00?ooool0T?oo00<00?ooool0Kooo0000
1Ooo003ooooo00002?oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo093oo`03003ooooo
06ooo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool0T?oo
00<00?ooool0Kooo0002ool2000:ool20003ool40004ool00`00ooooo`2@ool00`00ooooo`1_ool0
01_oo`03003ooooo093oo`03003ooooo06ooo`006ooo0P00TOoo00<00?ooool0Kooo000Kool00`00
ooooo`2@ool00`00ooooo`1_ool001_oo`03003ooooo093oo`03003ooooo06ooo`006ooo00<00?oo
ool0T?oo00<00?ooool0Kooo000Kool00`00ooooo`2@ool00`00ooooo`1_ool001_oo`03003ooooo
093oo`03003ooooo06ooo`006ooo0P00TOoo00<00?ooool0Kooo000Kool00`00ooooo`2@ool00`00
ooooo`1_ool001_oo`03003ooooo093oo`03003ooooo06ooo`006ooo00<00?ooool0T?oo00<00?oo
ool0Kooo000Kool00`00ooooo`2Aool00`00ooooo`1^ool001_oo`03003ooooo097oo`03003ooooo
06koo`006ooo0P00T_oo00<00?ooool0K_oo000Kool00`00ooooo`2Aool00`00ooooo`1^ool001_o
o`03003ooooo097oo`03003ooooo06koo`006ooo00<00?ooool0TOoo00<00?ooool0K_oo000Kool0
0`00ooooo`2Aool00`00ooooo`1^ool001_oo`03003ooooo097oo`03003ooooo06koo`006ooo0P00
T_oo00<00?ooool0K_oo000Kool00`00ooooo`2Aool00`00ooooo`1^ool001_oo`03003ooooo097o
o`03003ooooo06koo`006ooo00<00?ooool0TOoo00<00?ooool0K_oo0002ool20004ool00`00oooo
o`03ool20004ool20005ool00`00ooooo`2Aool00`00ooooo`1^ool00005ool00?ooool00008ool0
1000ooooo`000_oo00@00?ooool000Coo`03003ooooo097oo`03003ooooo06koo`0000Goo`00oooo
o`0000Soo`04003ooooo0002ool01000ooooo`001?oo0P00T_oo00<00?ooool0K_oo00001Ooo003o
oooo00002Ooo0P001?oo0P001Ooo00<00?ooool0TOoo00<00?ooool0K_oo00001Ooo003ooooo0000
2?oo00@00?ooool000;oo`04003ooooo0004ool00`00ool0000G001jool00`00ooooo`1^ool00005
ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo01Ooo`d006koo`03
003ooooo06goo`000_oo0P002_oo0P001?oo0P001Ooo00<00?ooool09?oo4P00G?oo00<00?ooool0
KOoo000Kool00`00ooooo`0fool6001Fool00`00ooooo`1]ool001_oo`8003goo`X000goo`8003go
o`03003ooooo06goo`006ooo00<00?ooool0A_oo1P0000?oo`00ool01?oo0P00?Ooo00<00?ooool0
KOoo000Kool00`00ooooo`1<ool9000mool00`00ooooo`1]ool001_oo`03003ooooo05;oo`T003Oo
o`03003ooooo06goo`006ooo00<00?ooool0Dooo0`0000?oo`0000001P00<ooo00<00?ooool0KOoo
000Kool00`00ooooo`1Cool300000ooo00000008000aool00`00ooooo`1]ool001_oo`8005Coo`<0
0003ool0000000<00003ool0000000H002goo`03003ooooo06goo`006ooo00<00?ooool0Dooo0`00
00?oo`0000000P000_oo1P000_oo1@00:?oo00<00?ooool0KOoo000Kool00`00ooooo`1Cool30000
0ooo000000020002ool00`00ool000030007ool2000Vool00`00ooooo`1]ool001_oo`03003ooooo
05?oo`<00003ool00000008000;oo`03003oo`0000<000Woo`<002?oo`03003ooooo06goo`006ooo
00<00?ooool0Dooo0P000_oo10000_oo00<00?oo00000`003?oo0P008Ooo00<00?ooool0KOoo000K
ool00`00ooooo`1Cool20002ool40002ool00`00ool00003000>ool3000Nool00`00ooooo`1]ool0
01_oo`8005Coo`8000;oo`@000;oo`03003oo`0000<0017oo`8001coo`03003ooooo06goo`006ooo
00<00?ooool0Dooo0P000_oo10000_oo00<00?oo00000`004ooo0`006_oo00<00?ooool0K?oo000K
ool00`00ooooo`1Cool20002ool40002ool00`00ool00003000Fool2000Hool00`00ooooo`1/ool0
01_oo`03003ooooo05?oo`8000;oo`@000;oo`03003oo`0000<001Soo`<001Goo`03003ooooo05Go
oa0000Ooo`006ooo00<00?ooool0Dooo0P000_oo10000_oo00<00?oo00000`006ooo0`004_oo00<0
0?ooool0@?oo5@005ooo000Kool00`00ooooo`1Cool20002ool40002ool00`00ool00003000Nool2
000@ool00`00ooooo`0fool:000/ool001_oo`8005Coo`8000;oo`@000;oo`03003oo`0000<0023o
o`03003ooooo00goo`03003ooooo02Soo`h003Koo`006ooo00<00?ooool0Dooo0P000_oo10000_oo
00<00?oo00000`008Ooo0P003Ooo00<00?ooool08?oo2000A?oo000Kool00`00ooooo`1Cool20002
ool40002ool00`00ool00003000Sool00`00ooooo`0:ool00`00ooooo`0Hool8001<ool001_oo`03
003ooooo05?oo`8000;oo`@000;oo`03003oo`0000<002Coo`03003ooooo00Woo`03003ooooo01Co
o`@005Coo`000_oo0P001?oo00<00?ooool00ooo0P000ooo0`001Ooo00<00?ooool0Dooo0P000_oo
10000_oo00<00?oo00000`009Ooo0P002Ooo00<00?ooool04_oo0P00F?oo00001Ooo003ooooo0000
2?oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo05?oo`8000;oo`@000;oo`03003oo`00
00<002Ooo`03003ooooo00Koo`03003ooooo00koo`@005[oo`0000Goo`00ooooo`0000Soo`04003o
oooo0005ool00`00ooooo`02ool2001Eool00`00ooooo`040002ool00`00ool00003000Xool00`00
ooooo`05ool00`00ooooo`0;ool3001Nool00005ool00?ooool00009ool20004ool30004ool00`00
ooooo`1Dool00`00ooooo`040002ool00`00ool00003000Xool00`00ooooo`05ool00`00ooooo`09
ool2001Qool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo
05Coo`03003ooooo00@000;oo`03003oo`0000<002Woo`03003ooooo00Coo`03003ooooo00Ooo`80
06?oo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool0E?oo
00<00?ooool010000_oo00<00?oo00000`00:_oo00<00?ooool00ooo00<00?ooool01_oo00<00?oo
ool0Hooo0002ool2000:ool20004ool20005ool00`00ooooo`1Dool00`00ooooo`040002ool00`00
ool00003000Zool00`00ooooo`03ool00`00ooooo`06ool00`00ooooo`1Sool001_oo`03003ooooo
05Coo`03003ooooo00@000;oo`03003oo`0000<002_oo`03003ooooo00?oo`03003ooooo00?oo`80
06Koo`006ooo0P00EOoo00<00?ooool010000_oo00<00?oo00000`00;?oo00<00?ooool00_oo00<0
0?ooool00_oo00<00?ooool0I_oo000Kool00`00ooooo`1Dool00`00ooooo`040002ool00`00ool0
0003000]ool01@00oooooooo00001?oo00<00?ooool0I_oo000Kool00`00ooooo`1Dool00`00oooo
o`040002ool00`00ool00003000]ool01@00oooooooo00000ooo00<00?ooool0Iooo000Kool00`00
ooooo`1Dool00`00ooooo`040002ool00`00ool00002000_ool01000ooooo`000ooo00<00?ooool0
Iooo000Kool00`00ooooo`1Dool00`00ooooo`040002ool00`00ool00002000_ool01000ooooo`00
0_oo00<00?ooool0J?oo000Kool2001Eool00`00ooooo`040002ool00`00ool00002000`ool00`00
ool00002ool00`00ooooo`1Xool001_oo`03003ooooo05Coo`03003ooooo00@000;oo`03003oo`00
0080037oo`800003ool00?oo06[oo`006ooo00<00?ooool0E?oo00<00?ooool010000_oo00<00?oo
00000P00<Ooo0`00K?oo000Kool00`00ooooo`1Dool00`00ooooo`040002ool00`00ool00002000b
ool2001/ool001_oo`03003ooooo05Coo`03003ooooo00@000;oo`03003oo`00008003;oo`03003o
oooo06_oo`006ooo00<00?ooool0Eooo10000_oo00<00?oo00000P00<_oo00<00?ooool0Jooo000K
ool2001Hool40002ool00`00ool00002000bool00`00ooooo`1[ool001_oo`03003ooooo05Ooo`@0
00;oo`03003oo`00008003;oo`03003ooooo06_oo`006ooo00<00?ooool06?oo0P001?oo00<00?oo
ool00_oo10004_oo0P001?oo00<00?ooool01?oo00<00?ooool03_oo1`0000Coo`00000000Goo`80
01Coo`8000Coo`03003ooooo00?oo`8000Woo`03003ooooo00goo`@001Ooo`@000?oo`03003ooooo
00;oo`@001;oo`@000?oo`03003ooooo00Coo`03003ooooo013oo`006ooo00<00?ooool05ooo00@0
0?ooool000Soo`03003ooooo01;oo`04003ooooo000:ool00`00ooooo`0>ool40002ool00`00ool0
00020004ool01000ooooo`004_oo00@00?ooool000Soo`04003ooooo0008ool00`00ooooo`0?ool0
0`00ooooo`0Hool00`00ooooo`07ool00`00ooooo`0Eool00`00ooooo`09ool00`00ooooo`0@ool0
01_oo`03003ooooo01Ooo`04003ooooo0009ool00`00ooooo`0Aool01000ooooo`001ooo1@003ooo
10000_oo00<00?oo00000P001?oo00@00?ooool001;oo`04003ooooo0008ool01000ooooo`002Ooo
00<00?ooool03_oo00<00?ooool06?oo00<00?ooool02?oo00<00?ooool05?oo00<00?ooool01_oo
1@004Ooo000Kool00`00ooooo`0Gool01000ooooo`002_oo00<00?ooool04?oo00@00?ooool000Oo
o`04003ooooo000@ool40002ool00`00ool000020004ool3000Cool01000ooooo`002Ooo0P002_oo
00<00?ooool03_oo00<00?ooool06?oo00<00?ooool02Ooo00<00?ooool04ooo00<00?ooool01_oo
00@00?ooool001;oo`006ooo0P006?oo00@00?ooool000_oo`03003ooooo00ooo`04003ooooo0008
ool00`00ool0000@ool40002ool00`00ool000020004ool00`00ooooo`0Cool01000ooooo`002?oo
00@00?ooool000Woo`03003ooooo00koo`03003ooooo01Soo`03003ooooo00[oo`03003ooooo01;o
o`03003ooooo00Ooo`03003oo`0001;oo`006ooo00<00?ooool05ooo00@00?ooool000Soo`04003o
oooo000Aool01000ooooo`002Ooo0P004?oo10000_oo00<00?oo00000P001?oo00<00?ooool04ooo
00@00?ooool000Soo`04003ooooo0009ool00`00ooooo`0<ool3000Hool30009ool01000ooooo`00
4_oo0`002_oo0P004_oo000Kool00`00ooooo`0Hool2000:ool2000Cool2000;ool00`00ooooo`0>
ool700001?oo000000001Ooo0`004ooo0P002_oo0P002_oo00<00?ooool03_oo00<00?ooool06?oo
00<00?ooool02?oo0P005Ooo00<00?ooool02Ooo00<00?ooool04?oo000Kool00`00ooooo`1Gool4
0002ool00`00ool00002000cool00`00ooooo`1Zool001_oo`03003ooooo05Ooo`<000?oo`03003o
o`00008003?oo`03003ooooo06[oo`006ooo00<00?ooool0Eooo0`000ooo00<00?oo00000P00<ooo
00<00?ooool0J_oo000Doooo000=000001_oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo
00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo
00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00?oo`@000;oo`04003oo`00ool20008
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool001_oo`03003ooooo01ooo`03003o
oooo01koo`03003ooooo01Coo`<000?oo`04003oo`00ool2000Qool00`00ooooo`0>ool00`00oooo
o`0>ool00`00ooooo`0Nool00`00ooooo`0Oool00`00ooooo`0Fool001_oo`03003ooooo05Ooo`<0
00?oo`05003oo`00ool0000cool00`00ooooo`1Zool001_oo`03003ooooo05Ooo`<000?oo`05003o
o`00ool0000cool00`00ooooo`1Zool001_oo`03003ooooo05Ooo`<000?oo`05003oo`00ool0000c
ool00`00ooooo`1Zool001_oo`8005Soo`<000?oo`05003oo`00ool0000cool00`00ooooo`1Zool0
01_oo`03003ooooo05Ooo`<000?oo`03003oo`0003Goo`03003ooooo06[oo`006ooo00<00?ooool0
Eooo0`000ooo00<00?oo0000=_oo00<00?ooool0JOoo000Kool00`00ooooo`1Gool30003ool00`00
ool0000fool00`00ooooo`1Yool001_oo`03003ooooo05Ooo`<000?oo`03003oo`0003Koo`03003o
oooo06Woo`006ooo00<00?ooool0Eooo0`000ooo00<00?oo0000=_oo00<00?ooool0JOoo000Kool2
001Hool30003ool00`00ool0000fool00`00ooooo`1Yool001_oo`03003ooooo05Ooo`<000?oo`03
003oo`0003Koo`03003ooooo06Woo`006ooo00<00?ooool0Eooo0`000ooo00<00?oo0000=_oo00<0
0?ooool0JOoo000Kool00`00ooooo`1Gool30003ool00`00ool0000fool00`00ooooo`1Yool001_o
o`03003ooooo05Ooo`<000?oo`03003oo`0003Koo`03003ooooo06Woo`006ooo00<00?ooool0Eooo
0`000ooo00<00?oo0000=_oo00<00?ooool0JOoo000Kool2001Hool30003ool00`00ool0000fool0
0`00ooooo`1Yool001_oo`03003ooooo05Ooo`<000?oo`03003oo`0003Koo`03003ooooo06Woo`00
6ooo00<00?ooool0Eooo0`000ooo00<00?oo0000=_oo00<00?ooool0JOoo000Kool00`00ooooo`1G
ool30003ool00`00ool0000fool00`00ooooo`1Yool001_oo`03003ooooo05Ooo`<000?oo`03003o
o`0003Koo`03003ooooo06Woo`006ooo00<00?ooool0Eooo0`000ooo00<00?oo0000=_oo00<00?oo
ool0JOoo000Kool2001Hool30003ool00`00ool0000gool00`00ooooo`1Xool001_oo`03003ooooo
05Ooo`<000?oo`03003oo`0003Ooo`03003ooooo06Soo`006ooo00<00?ooool0Eooo0`000ooo00<0
0?oo0000=ooo00<00?ooool0J?oo000Kool00`00ooooo`1Gool30003ool00`00ool0000gool00`00
ooooo`1Xool000;oo`8000Coo`03003ooooo00;oo`<000?oo`@000Coo`03003ooooo05Ooo`<000?o
o`03003oo`0003Ooo`03003ooooo06Soo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003o
oooo00?oo`03003ooooo05Ooo`<000?oo`03003oo`0003Ooo`03003ooooo06Soo`0000Goo`00oooo
o`0000_oo`03003ooooo00;oo`03003ooooo00?oo`8005Soo`<000?oo`03003oo`0003Ooo`03003o
oooo06Soo`0000Goo`00ooooo`0000Woo`<000Coo`03003ooooo00?oo`03003ooooo05Ooo`<000?o
o`03003oo`0003Ooo`03003ooooo06Soo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00
ooooo`03ool00`00ooooo`1Gool30003ool00`00ool0000gool00`00ooooo`1Xool00005ool00?oo
ool00008ool01000ooooo`000_oo0`001Ooo00<00?ooool0Eooo0`000ooo00<00?oo0000=ooo00<0
0?ooool0J?oo0002ool2000:ool20005ool00`00ooooo`03ool00`00ooooo`1Gool20004ool00`00
ool0000gool00`00ooooo`1Xool001_oo`03003ooooo05Ooo`8000Coo`03003oo`0003Ooo`03003o
oooo06Soo`006ooo0P00F?oo0P001?oo00<00?oo0000=ooo00<00?ooool0J?oo000Kool00`00oooo
o`1Gool20004ool00`00ool0000gool00`00ooooo`1Xool001_oo`03003ooooo05Ooo`8000Coo`03
003oo`0003Ooo`03003ooooo06Soo`006ooo00<00?ooool0Eooo0P001?oo00<00?oo0000=ooo00<0
0?ooool0J?oo000Kool00`00ooooo`1Gool20004ool00`00ool0000hool00`00ooooo`1Wool001_o
o`03003ooooo05Ooo`8000Coo`03003oo`0003Soo`03003ooooo06Ooo`006ooo0P00F?oo0P001?oo
00<00?oo0000>?oo00<00?ooool0Iooo000Kool00`00ooooo`1Gool20004ool00`00ool0000hool0
0`00ooooo`1Wool001_oo`03003ooooo05Ooo`8000Coo`03003oo`0003Soo`03003ooooo06Ooo`00
6ooo00<00?ooool0Eooo0P001?oo00<00?oo0000>?oo00<00?ooool0Iooo000Kool00`00ooooo`1G
ool20004ool00`00ool0000hool00`00ooooo`1Wool001_oo`03003ooooo05Ooo`8000Coo`03003o
o`0003Soo`03003ooooo06Ooo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0
0?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161797, 0.858206, \
0.00592092, 0.000337886}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[k], Om /. strangesol[k, 10^4]}, {k, 0.01, 1.5, 
        0.01}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.01`, 0.8806341584381355`}, {0.02`, 0.880636516377243`}, {0.03`, 
        0.8806404436970839`}, {0.04`, 0.8806459479859553`}, {0.05`, 
        0.8806530416606214`}, {0.06`, 0.8806617513921337`}, {0.07`, 
        0.8806721289199834`}, {0.08`, 0.8806842648940764`}, {0.09`, 
        0.8806982997866014`}, {0.1`, 0.8807122135387484`}, {0.11`, 
        0.88072886796599`}, {0.12`, 0.8807471577076371`}, {0.13`, 
        0.8807670769899504`}, {0.14`, 0.8807886292584165`}, {0.15`, 
        0.8808118077889169`}, {0.16`, 0.8808366077900293`}, {0.17`, 
        0.8808630147486081`}, {0.18`, 0.8808910965255993`}, {0.19`, 
        0.8809208536627164`}, {0.2`, 0.8809522959033994`}, {0.21`, 
        0.8809847114767474`}, {0.22`, 0.8810205495161759`}, {0.23`, 
        0.881056796050523`}, {0.24`, 0.8810946308571438`}, {0.25`, 
        0.8811360372215906`}, {0.26`, 0.8811781725567692`}, {0.27`, 
        0.8812224564479246`}, {0.28`, 0.8812676318142505`}, {0.29`, 
        0.8813157429127656`}, {0.3`, 0.8813654101916851`}, {0.31`, 
        0.881416944354512`}, {0.32`, 0.8814705879910725`}, {0.33`, 
        0.8815261988575193`}, {0.34`, 
        0.8815838028067957`}, {0.35000000000000003`, 
        0.8816433235192964`}, {0.36`, 0.8817054366831196`}, {0.37`, 
        0.8817694706629516`}, {0.38`, 0.8818357341681339`}, {0.39`, 
        0.8819042249905517`}, {0.4`, 
        0.8819749854140265`}, {0.41000000000000003`, 
        0.8819301155278085`}, {0.42`, 0.8821237695935136`}, {0.43`, 
        0.882203793002103`}, {0.44`, 0.8822817645964927`}, {0.45`, 
        0.8823656138366212`}, {0.46`, 
        0.8824509335811959`}, {0.47000000000000003`, 
        0.8825407025979198`}, {0.48`, 0.8826343888896682`}, {0.49`, 
        0.8827243892844563`}, {0.5`, 0.8828239264954276`}, {0.51`, 
        0.8830619733684731`}, {0.52`, 0.8830351021681585`}, {0.53`, 
        0.8890117080862644`}, {0.54`, 0.9325890723292227`}, {0.55`, 
        0.8833623920384304`}, {0.56`, 
        0.883271088170484`}, {0.5700000000000001`, 
        0.8836014098015493`}, {0.58`, 0.8837277561694317`}, {0.59`, 
        0.8838585675216777`}, {0.6`, 0.8839938455265696`}, {0.61`, 
        0.88413394974562`}, {0.62`, 0.8840258230164297`}, {0.63`, 
        0.8844297372067496`}, {0.64`, 0.8846078429317532`}, {0.65`, 
        0.8847482486488593`}, {0.66`, 0.8849170277536739`}, {0.67`, 
        0.8850927387593782`}, {0.68`, 
        0.8852768812797177`}, {0.6900000000000001`, 
        0.8854680237343963`}, {0.7000000000000001`, 
        0.8856667609877751`}, {0.71`, 0.8858759285509044`}, {0.72`, 
        0.8860956568478756`}, {0.73`, 0.8863264069182158`}, {0.74`, 
        0.886569574418449`}, {0.75`, 0.8868270625774893`}, {0.76`, 
        0.8871000605361989`}, {0.77`, 0.8873907112130838`}, {0.78`, 
        0.8877015555727047`}, {0.79`, 0.8880356432967923`}, {0.8`, 
        0.888396876368498`}, {0.81`, 
        0.888789904577055`}, {0.8200000000000001`, 
        0.8892220344548822`}, {0.8300000000000001`, 
        0.8897009940611749`}, {0.84`, 0.8902383798255504`}, {0.85`, 
        0.8908497143043895`}, {0.86`, 0.8915556395454393`}, {0.87`, 
        0.8923806332325089`}, {0.88`, 0.8933368823298763`}, {0.89`, 
        0.8943270442424447`}, {0.9`, 0.894692356269355`}, {0.91`, 
        0.8936908473109122`}, {0.92`, 0.8926683136860369`}, {0.93`, 
        0.8919351237576246`}, {0.9400000000000001`, 
        0.891386860220209`}, {0.9500000000000001`, 
        0.890955779991944`}, {0.96`, 0.8906040343594888`}, {0.97`, 
        0.8903091215834303`}, {0.98`, 0.890056738953177`}, {0.99`, 
        0.8898372796038082`}, {1.`, 0.8896439988379299`}, {1.01`, 
        0.8894719890698176`}, {1.02`, 0.8893175732803424`}, {1.03`, 
        0.8891779284941188`}, {1.04`, 0.8890509696103457`}, {1.05`, 
        0.8889346657809305`}, {1.06`, 0.8888277284660863`}, {1.07`, 
        0.888728985420536`}, {1.08`, 0.8886374635400355`}, {1.09`, 
        0.8885523466825501`}, {1.1`, 0.8884729440429879`}, {1.11`, 
        0.8883986660716652`}, {1.12`, 
        0.8883290058810112`}, {1.1300000000000001`, 
        0.8882635247089891`}, {1.1400000000000001`, 
        0.8882018404244568`}, {1.1500000000000001`, 
        0.8881436183435025`}, {1.16`, 0.8880885638224716`}, {1.17`, 
        0.8880364162319044`}, {1.18`, 0.8879869440146061`}, {1.19`, 
        0.8879399406027553`}, {1.2`, 0.8878952210215609`}, {1.21`, 
        0.887852619045985`}, {1.22`, 0.8878119848063174`}, {1.23`, 
        0.8877731827605423`}, {1.24`, 0.887736089968385`}, {1.25`, 
        0.8877005946149908`}, {1.26`, 0.8876665947423396`}, {1.27`, 
        0.8876339971544646`}, {1.28`, 0.8876027164688153`}, {1.29`, 
        0.8875726742910943`}, {1.3`, 0.8875437984948849`}, {1.31`, 
        0.8875160225905897`}, {1.32`, 0.8874892851708036`}, {1.33`, 
        0.8874635294213384`}, {1.34`, 0.8874387026888602`}, {1.35`, 
        0.8874147560975`}, {1.36`, 0.8873916442079793`}, {1.37`, 
        0.8873693247137594`}, {1.3800000000000001`, 
        0.8873477581695222`}, {1.3900000000000001`, 
        0.8873269077479786`}, {1.4000000000000001`, 
        0.8873067390215552`}, {1.41`, 0.8872872197659943`}, {1.42`, 
        0.8872683197832989`}, {1.43`, 0.8872500107418041`}, {1.44`, 
        0.8872322660314391`}, {1.45`, 0.887215060632499`}, {1.46`, 
        0.8871983709964517`}, {1.47`, 0.8871821749374975`}, {1.48`, 
        0.8871664515337444`}, {1.49`, 0.8871511810370044`}, {1.5`, 
        0.8871363447903378`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{0.01`, 0.8806341584381355`}, {0.02`, 0.880636516377243`}, {0.03`, 
        0.8806404436970839`}, {0.04`, 0.8806459479859553`}, {0.05`, 
        0.8806530416606214`}, {0.06`, 0.8806617513921337`}, {0.07`, 
        0.8806721289199834`}, {0.08`, 0.8806842648940764`}, {0.09`, 
        0.8806982997866014`}, {0.1`, 0.8807122135387484`}, {0.11`, 
        0.88072886796599`}, {0.12`, 0.8807471577076371`}, {0.13`, 
        0.8807670769899504`}, {0.14`, 0.8807886292584165`}, {0.15`, 
        0.8808118077889169`}, {0.16`, 0.8808366077900293`}, {0.17`, 
        0.8808630147486081`}, {0.18`, 0.8808910965255993`}, {0.19`, 
        0.8809208536627164`}, {0.2`, 0.8809522959033994`}, {0.21`, 
        0.8809847114767474`}, {0.22`, 0.8810205495161759`}, {0.23`, 
        0.881056796050523`}, {0.24`, 0.8810946308571438`}, {0.25`, 
        0.8811360372215906`}, {0.26`, 0.8811781725567692`}, {0.27`, 
        0.8812224564479246`}, {0.28`, 0.8812676318142505`}, {0.29`, 
        0.8813157429127656`}, {0.3`, 0.8813654101916851`}, {0.31`, 
        0.881416944354512`}, {0.32`, 0.8814705879910725`}, {0.33`, 
        0.8815261988575193`}, {0.34`, 
        0.8815838028067957`}, {0.35000000000000003`, 
        0.8816433235192964`}, {0.36`, 0.8817054366831196`}, {0.37`, 
        0.8817694706629516`}, {0.38`, 0.8818357341681339`}, {0.39`, 
        0.8819042249905517`}, {0.4`, 
        0.8819749854140265`}, {0.41000000000000003`, 
        0.8819301155278085`}, {0.42`, 0.8821237695935136`}, {0.43`, 
        0.882203793002103`}, {0.44`, 0.8822817645964927`}, {0.45`, 
        0.8823656138366212`}, {0.46`, 
        0.8824509335811959`}, {0.47000000000000003`, 
        0.8825407025979198`}, {0.48`, 0.8826343888896682`}, {0.49`, 
        0.8827243892844563`}, {0.5`, 0.8828239264954276`}, {0.51`, 
        0.8830619733684731`}, {0.52`, 0.8830351021681585`}, {0.55`, 
        0.8833623920384304`}, {0.56`, 
        0.883271088170484`}, {0.5700000000000001`, 
        0.8836014098015493`}, {0.58`, 0.8837277561694317`}, {0.59`, 
        0.8838585675216777`}, {0.6`, 0.8839938455265696`}, {0.61`, 
        0.88413394974562`}, {0.62`, 0.8840258230164297`}, {0.63`, 
        0.8844297372067496`}, {0.64`, 0.8846078429317532`}, {0.65`, 
        0.8847482486488593`}, {0.66`, 0.8849170277536739`}, {0.67`, 
        0.8850927387593782`}, {0.68`, 
        0.8852768812797177`}, {0.6900000000000001`, 
        0.8854680237343963`}, {0.7000000000000001`, 
        0.8856667609877751`}, {0.71`, 0.8858759285509044`}, {0.72`, 
        0.8860956568478756`}, {0.73`, 0.8863264069182158`}, {0.74`, 
        0.886569574418449`}, {0.75`, 0.8868270625774893`}, {0.76`, 
        0.8871000605361989`}, {0.77`, 0.8873907112130838`}, {0.78`, 
        0.8877015555727047`}, {0.79`, 0.8880356432967923`}, {0.8`, 
        0.888396876368498`}, {0.81`, 
        0.888789904577055`}, {0.8200000000000001`, 
        0.8892220344548822`}, {0.8300000000000001`, 
        0.8897009940611749`}, {0.84`, 0.8902383798255504`}, {0.85`, 
        0.8908497143043895`}, {0.86`, 0.8915556395454393`}, {0.87`, 
        0.8923806332325089`}, {0.88`, 0.8933368823298763`}, {0.89`, 
        0.8943270442424447`}, {0.9`, 0.894692356269355`}, {0.91`, 
        0.8936908473109122`}, {0.92`, 0.8926683136860369`}, {0.93`, 
        0.8919351237576246`}, {0.9400000000000001`, 
        0.891386860220209`}, {0.9500000000000001`, 
        0.890955779991944`}, {0.96`, 0.8906040343594888`}, {0.97`, 
        0.8903091215834303`}, {0.98`, 0.890056738953177`}, {0.99`, 
        0.8898372796038082`}, {1.`, 0.8896439988379299`}, {1.01`, 
        0.8894719890698176`}, {1.02`, 0.8893175732803424`}, {1.03`, 
        0.8891779284941188`}, {1.04`, 0.8890509696103457`}, {1.05`, 
        0.8889346657809305`}, {1.06`, 0.8888277284660863`}, {1.07`, 
        0.888728985420536`}, {1.08`, 0.8886374635400355`}, {1.09`, 
        0.8885523466825501`}, {1.1`, 0.8884729440429879`}, {1.11`, 
        0.8883986660716652`}, {1.12`, 
        0.8883290058810112`}, {1.1300000000000001`, 
        0.8882635247089891`}, {1.1400000000000001`, 
        0.8882018404244568`}, {1.1500000000000001`, 
        0.8881436183435025`}, {1.16`, 0.8880885638224716`}, {1.17`, 
        0.8880364162319044`}, {1.18`, 0.8879869440146061`}, {1.19`, 
        0.8879399406027553`}, {1.2`, 0.8878952210215609`}, {1.21`, 
        0.887852619045985`}, {1.22`, 0.8878119848063174`}, {1.23`, 
        0.8877731827605423`}, {1.24`, 0.887736089968385`}, {1.25`, 
        0.8877005946149908`}, {1.26`, 0.8876665947423396`}, {1.27`, 
        0.8876339971544646`}, {1.28`, 0.8876027164688153`}, {1.29`, 
        0.8875726742910943`}, {1.3`, 0.8875437984948849`}, {1.31`, 
        0.8875160225905897`}, {1.32`, 0.8874892851708036`}, {1.33`, 
        0.8874635294213384`}, {1.34`, 0.8874387026888602`}, {1.35`, 
        0.8874147560975`}, {1.36`, 0.8873916442079793`}, {1.37`, 
        0.8873693247137594`}, {1.3800000000000001`, 
        0.8873477581695222`}, {1.3900000000000001`, 
        0.8873269077479786`}, {1.4000000000000001`, 
        0.8873067390215552`}, {1.41`, 0.8872872197659943`}, {1.42`, 
        0.8872683197832989`}, {1.43`, 0.8872500107418041`}, {1.44`, 
        0.8872322660314391`}, {1.45`, 0.887215060632499`}, {1.46`, 
        0.8871983709964517`}, {1.47`, 0.8871821749374975`}, {1.48`, 
        0.8871664515337444`}, {1.49`, 0.8871511810370044`}, {1.5`, 
        0.8871363447903378`}}\)], "Input"],

Cell[BoxData[
    \({{0.01`, 0.8806341584381355`}, {0.02`, 0.880636516377243`}, {0.03`, 
        0.8806404436970839`}, {0.04`, 0.8806459479859553`}, {0.05`, 
        0.8806530416606214`}, {0.06`, 0.8806617513921337`}, {0.07`, 
        0.8806721289199834`}, {0.08`, 0.8806842648940764`}, {0.09`, 
        0.8806982997866014`}, {0.1`, 0.8807122135387484`}, {0.11`, 
        0.88072886796599`}, {0.12`, 0.8807471577076371`}, {0.13`, 
        0.8807670769899504`}, {0.14`, 0.8807886292584165`}, {0.15`, 
        0.8808118077889169`}, {0.16`, 0.8808366077900293`}, {0.17`, 
        0.8808630147486081`}, {0.18`, 0.8808910965255993`}, {0.19`, 
        0.8809208536627164`}, {0.2`, 0.8809522959033994`}, {0.21`, 
        0.8809847114767474`}, {0.22`, 0.8810205495161759`}, {0.23`, 
        0.881056796050523`}, {0.24`, 0.8810946308571438`}, {0.25`, 
        0.8811360372215906`}, {0.26`, 0.8811781725567692`}, {0.27`, 
        0.8812224564479246`}, {0.28`, 0.8812676318142505`}, {0.29`, 
        0.8813157429127656`}, {0.3`, 0.8813654101916851`}, {0.31`, 
        0.881416944354512`}, {0.32`, 0.8814705879910725`}, {0.33`, 
        0.8815261988575193`}, {0.34`, 
        0.8815838028067957`}, {0.35000000000000003`, 
        0.8816433235192964`}, {0.36`, 0.8817054366831196`}, {0.37`, 
        0.8817694706629516`}, {0.38`, 0.8818357341681339`}, {0.39`, 
        0.8819042249905517`}, {0.4`, 
        0.8819749854140265`}, {0.41000000000000003`, 
        0.8819301155278085`}, {0.42`, 0.8821237695935136`}, {0.43`, 
        0.882203793002103`}, {0.44`, 0.8822817645964927`}, {0.45`, 
        0.8823656138366212`}, {0.46`, 
        0.8824509335811959`}, {0.47000000000000003`, 
        0.8825407025979198`}, {0.48`, 0.8826343888896682`}, {0.49`, 
        0.8827243892844563`}, {0.5`, 0.8828239264954276`}, {0.51`, 
        0.8830619733684731`}, {0.52`, 0.8830351021681585`}, {0.55`, 
        0.8833623920384304`}, {0.56`, 
        0.883271088170484`}, {0.5700000000000001`, 
        0.8836014098015493`}, {0.58`, 0.8837277561694317`}, {0.59`, 
        0.8838585675216777`}, {0.6`, 0.8839938455265696`}, {0.61`, 
        0.88413394974562`}, {0.62`, 0.8840258230164297`}, {0.63`, 
        0.8844297372067496`}, {0.64`, 0.8846078429317532`}, {0.65`, 
        0.8847482486488593`}, {0.66`, 0.8849170277536739`}, {0.67`, 
        0.8850927387593782`}, {0.68`, 
        0.8852768812797177`}, {0.6900000000000001`, 
        0.8854680237343963`}, {0.7000000000000001`, 
        0.8856667609877751`}, {0.71`, 0.8858759285509044`}, {0.72`, 
        0.8860956568478756`}, {0.73`, 0.8863264069182158`}, {0.74`, 
        0.886569574418449`}, {0.75`, 0.8868270625774893`}, {0.76`, 
        0.8871000605361989`}, {0.77`, 0.8873907112130838`}, {0.78`, 
        0.8877015555727047`}, {0.79`, 0.8880356432967923`}, {0.8`, 
        0.888396876368498`}, {0.81`, 
        0.888789904577055`}, {0.8200000000000001`, 
        0.8892220344548822`}, {0.8300000000000001`, 
        0.8897009940611749`}, {0.84`, 0.8902383798255504`}, {0.85`, 
        0.8908497143043895`}, {0.86`, 0.8915556395454393`}, {0.87`, 
        0.8923806332325089`}, {0.88`, 0.8933368823298763`}, {0.89`, 
        0.8943270442424447`}, {0.9`, 0.894692356269355`}, {0.91`, 
        0.8936908473109122`}, {0.92`, 0.8926683136860369`}, {0.93`, 
        0.8919351237576246`}, {0.9400000000000001`, 
        0.891386860220209`}, {0.9500000000000001`, 
        0.890955779991944`}, {0.96`, 0.8906040343594888`}, {0.97`, 
        0.8903091215834303`}, {0.98`, 0.890056738953177`}, {0.99`, 
        0.8898372796038082`}, {1.`, 0.8896439988379299`}, {1.01`, 
        0.8894719890698176`}, {1.02`, 0.8893175732803424`}, {1.03`, 
        0.8891779284941188`}, {1.04`, 0.8890509696103457`}, {1.05`, 
        0.8889346657809305`}, {1.06`, 0.8888277284660863`}, {1.07`, 
        0.888728985420536`}, {1.08`, 0.8886374635400355`}, {1.09`, 
        0.8885523466825501`}, {1.1`, 0.8884729440429879`}, {1.11`, 
        0.8883986660716652`}, {1.12`, 
        0.8883290058810112`}, {1.1300000000000001`, 
        0.8882635247089891`}, {1.1400000000000001`, 
        0.8882018404244568`}, {1.1500000000000001`, 
        0.8881436183435025`}, {1.16`, 0.8880885638224716`}, {1.17`, 
        0.8880364162319044`}, {1.18`, 0.8879869440146061`}, {1.19`, 
        0.8879399406027553`}, {1.2`, 0.8878952210215609`}, {1.21`, 
        0.887852619045985`}, {1.22`, 0.8878119848063174`}, {1.23`, 
        0.8877731827605423`}, {1.24`, 0.887736089968385`}, {1.25`, 
        0.8877005946149908`}, {1.26`, 0.8876665947423396`}, {1.27`, 
        0.8876339971544646`}, {1.28`, 0.8876027164688153`}, {1.29`, 
        0.8875726742910943`}, {1.3`, 0.8875437984948849`}, {1.31`, 
        0.8875160225905897`}, {1.32`, 0.8874892851708036`}, {1.33`, 
        0.8874635294213384`}, {1.34`, 0.8874387026888602`}, {1.35`, 
        0.8874147560975`}, {1.36`, 0.8873916442079793`}, {1.37`, 
        0.8873693247137594`}, {1.3800000000000001`, 
        0.8873477581695222`}, {1.3900000000000001`, 
        0.8873269077479786`}, {1.4000000000000001`, 
        0.8873067390215552`}, {1.41`, 0.8872872197659943`}, {1.42`, 
        0.8872683197832989`}, {1.43`, 0.8872500107418041`}, {1.44`, 
        0.8872322660314391`}, {1.45`, 0.887215060632499`}, {1.46`, 
        0.8871983709964517`}, {1.47`, 0.8871821749374975`}, {1.48`, 
        0.8871664515337444`}, {1.49`, 0.8871511810370044`}, {1.5`, 
        0.8871363447903378`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -36.8566 41.8691 [
[.15079 .39435 -9 -9 ]
[.15079 .39435 9 0 ]
[.27778 .39435 -9 -9 ]
[.27778 .39435 9 0 ]
[.40476 .39435 -9 -9 ]
[.40476 .39435 9 0 ]
[.53175 .39435 -9 -9 ]
[.53175 .39435 9 0 ]
[.65873 .39435 -3 -9 ]
[.65873 .39435 3 0 ]
[.78571 .39435 -9 -9 ]
[.78571 .39435 9 0 ]
[.9127 .39435 -9 -9 ]
[.9127 .39435 9 0 ]
[.01131 .0719 -30 -4.5 ]
[.01131 .0719 0 4.5 ]
[.01131 .15564 -30 -4.5 ]
[.01131 .15564 0 4.5 ]
[.01131 .23938 -30 -4.5 ]
[.01131 .23938 0 4.5 ]
[.01131 .32312 -30 -4.5 ]
[.01131 .32312 0 4.5 ]
[.01131 .49059 -30 -4.5 ]
[.01131 .49059 0 4.5 ]
[.01131 .57433 -30 -4.5 ]
[.01131 .57433 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .40685 m
.15079 .4131 L
s
[(0.2)] .15079 .39435 0 1 Mshowa
.27778 .40685 m
.27778 .4131 L
s
[(0.4)] .27778 .39435 0 1 Mshowa
.40476 .40685 m
.40476 .4131 L
s
[(0.6)] .40476 .39435 0 1 Mshowa
.53175 .40685 m
.53175 .4131 L
s
[(0.8)] .53175 .39435 0 1 Mshowa
.65873 .40685 m
.65873 .4131 L
s
[(1)] .65873 .39435 0 1 Mshowa
.78571 .40685 m
.78571 .4131 L
s
[(1.2)] .78571 .39435 0 1 Mshowa
.9127 .40685 m
.9127 .4131 L
s
[(1.4)] .9127 .39435 0 1 Mshowa
.125 Mabswid
.05556 .40685 m
.05556 .4106 L
s
.0873 .40685 m
.0873 .4106 L
s
.11905 .40685 m
.11905 .4106 L
s
.18254 .40685 m
.18254 .4106 L
s
.21429 .40685 m
.21429 .4106 L
s
.24603 .40685 m
.24603 .4106 L
s
.30952 .40685 m
.30952 .4106 L
s
.34127 .40685 m
.34127 .4106 L
s
.37302 .40685 m
.37302 .4106 L
s
.43651 .40685 m
.43651 .4106 L
s
.46825 .40685 m
.46825 .4106 L
s
.5 .40685 m
.5 .4106 L
s
.56349 .40685 m
.56349 .4106 L
s
.59524 .40685 m
.59524 .4106 L
s
.62698 .40685 m
.62698 .4106 L
s
.69048 .40685 m
.69048 .4106 L
s
.72222 .40685 m
.72222 .4106 L
s
.75397 .40685 m
.75397 .4106 L
s
.81746 .40685 m
.81746 .4106 L
s
.84921 .40685 m
.84921 .4106 L
s
.88095 .40685 m
.88095 .4106 L
s
.94444 .40685 m
.94444 .4106 L
s
.97619 .40685 m
.97619 .4106 L
s
.25 Mabswid
0 .40685 m
1 .40685 L
s
.02381 .0719 m
.03006 .0719 L
s
[(0.882)] .01131 .0719 1 0 Mshowa
.02381 .15564 m
.03006 .15564 L
s
[(0.884)] .01131 .15564 1 0 Mshowa
.02381 .23938 m
.03006 .23938 L
s
[(0.886)] .01131 .23938 1 0 Mshowa
.02381 .32312 m
.03006 .32312 L
s
[(0.888)] .01131 .32312 1 0 Mshowa
.02381 .49059 m
.03006 .49059 L
s
[(0.892)] .01131 .49059 1 0 Mshowa
.02381 .57433 m
.03006 .57433 L
s
[(0.894)] .01131 .57433 1 0 Mshowa
.125 Mabswid
.02381 .09284 m
.02756 .09284 L
s
.02381 .11377 m
.02756 .11377 L
s
.02381 .13471 m
.02756 .13471 L
s
.02381 .17657 m
.02756 .17657 L
s
.02381 .19751 m
.02756 .19751 L
s
.02381 .21844 m
.02756 .21844 L
s
.02381 .26031 m
.02756 .26031 L
s
.02381 .28125 m
.02756 .28125 L
s
.02381 .30218 m
.02756 .30218 L
s
.02381 .34405 m
.02756 .34405 L
s
.02381 .36499 m
.02756 .36499 L
s
.02381 .38592 m
.02756 .38592 L
s
.02381 .42779 m
.02756 .42779 L
s
.02381 .44872 m
.02756 .44872 L
s
.02381 .46966 m
.02756 .46966 L
s
.02381 .51153 m
.02756 .51153 L
s
.02381 .53246 m
.02756 .53246 L
s
.02381 .5534 m
.02756 .5534 L
s
.02381 .05097 m
.02756 .05097 L
s
.02381 .03003 m
.02756 .03003 L
s
.02381 .0091 m
.02756 .0091 L
s
.02381 .59527 m
.02756 .59527 L
s
.02381 .6162 m
.02756 .6162 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03016 .01472 m
.03651 .01481 L
.04286 .01498 L
.04921 .01521 L
.05556 .01551 L
.0619 .01587 L
.06825 .0163 L
.0746 .01681 L
.08095 .0174 L
.0873 .01798 L
.09365 .01868 L
.1 .01945 L
.10635 .02028 L
.1127 .02118 L
.11905 .02215 L
.1254 .02319 L
.13175 .0243 L
.1381 .02547 L
.14444 .02672 L
.15079 .02804 L
.15714 .02939 L
.16349 .03089 L
.16984 .03241 L
.17619 .03399 L
.18254 .03573 L
.18889 .03749 L
.19524 .03935 L
.20159 .04124 L
.20794 .04325 L
.21429 .04533 L
.22063 .04749 L
.22698 .04974 L
.23333 .05206 L
.23968 .05448 L
.24603 .05697 L
.25238 .05957 L
.25873 .06225 L
.26508 .06502 L
.27143 .06789 L
.27778 .07085 L
.28413 .06898 L
.29048 .07708 L
.29683 .08043 L
.30317 .0837 L
.30952 .08721 L
.31587 .09078 L
.32222 .09454 L
.32857 .09846 L
.33492 .10223 L
.34127 .1064 L
Mistroke
.34762 .11637 L
.35397 .11524 L
.37302 .12894 L
.37937 .12512 L
.38571 .13895 L
.39206 .14424 L
.39841 .14972 L
.40476 .15538 L
.41111 .16125 L
.41746 .15672 L
.42381 .17363 L
.43016 .18109 L
.43651 .18697 L
.44286 .19403 L
.44921 .20139 L
.45556 .2091 L
.4619 .2171 L
.46825 .22543 L
.4746 .23418 L
.48095 .24338 L
.4873 .25304 L
.49365 .26323 L
.5 .27401 L
.50635 .28544 L
.5127 .29761 L
.51905 .31062 L
.5254 .32461 L
.53175 .33973 L
.5381 .35619 L
.54444 .37428 L
.55079 .39434 L
.55714 .41684 L
.56349 .44243 L
.56984 .47199 L
.57619 .50653 L
.58254 .54657 L
.58889 .58802 L
.59524 .60332 L
.60159 .56139 L
.60794 .51857 L
.61429 .48788 L
.62063 .46492 L
.62698 .44687 L
.63333 .43214 L
.63968 .4198 L
.64603 .40923 L
.65238 .40004 L
.65873 .39195 L
.66508 .38475 L
.67143 .37828 L
Mistroke
.67778 .37243 L
.68413 .36712 L
.69048 .36225 L
.69683 .35777 L
.70317 .35364 L
.70952 .34981 L
.71587 .34624 L
.72222 .34292 L
.72857 .33981 L
.73492 .33689 L
.74127 .33415 L
.74762 .33157 L
.75397 .32913 L
.76032 .32682 L
.76667 .32464 L
.77302 .32257 L
.77937 .3206 L
.78571 .31873 L
.79206 .31695 L
.79841 .31524 L
.80476 .31362 L
.81111 .31207 L
.81746 .31058 L
.82381 .30916 L
.83016 .30779 L
.83651 .30648 L
.84286 .30522 L
.84921 .30402 L
.85556 .30285 L
.8619 .30173 L
.86825 .30065 L
.8746 .29962 L
.88095 .29861 L
.8873 .29764 L
.89365 .29671 L
.9 .29581 L
.90635 .29493 L
.9127 .29409 L
.91905 .29327 L
.9254 .29248 L
.93175 .29171 L
.9381 .29097 L
.94444 .29025 L
.95079 .28955 L
.95714 .28887 L
.96349 .28822 L
.96984 .28758 L
.97619 .28696 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`03003o
oooo0?coo`008Ooo00<00?ooool0o?oo000Qool2003mool0027oo`03003ooooo0?coo`008Ooo4@00
k_oo000Qool00`00ooooo`0>ool9003Uool0027oo`03003ooooo01Ooo`@00>7oo`008Ooo00<00?oo
ool06ooo1@00g?oo000Qool2000Qool4003Hool0027oo`03003ooooo02Coo`D00=?oo`008Ooo00<0
0?ooool0:Ooo0`00d?oo000Qool00`00ooooo`0/ool3003=ool0027oo`03003ooooo02ooo`800<_o
o`008Ooo0P00<_oo0`00b?oo000Qool00`00ooooo`0dool20036ool0027oo`03003ooooo03Koo`<0
0<?oo`000_oo0P001?oo00<00?ooool00ooo0P001?oo0P000ooo10001?oo00<00?ooool0>Ooo0P00
`Ooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`05ool0
0`00ooooo`0kool3002nool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000?o
o`03003ooooo00Coo`8003ooo`<00;_oo`0000Goo`00ooooo`0000Woo`8000Coo`8000Goo`03003o
oooo00?oo`03003ooooo047oo`03003ooooo0;Soo`0000Goo`00ooooo`0000Soo`04003ooooo0002
ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool0@_oo00<00?ooool0]ooo00001Ooo003o
oooo00002?oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool0@ooo
0`00]_oo0002ool2000:ool20004ool20004ool20005ool00`00ooooo`16ool2002dool0027oo`03
003ooooo04Soo`800;;oo`008Ooo0P00Booo00<00?ooool0[ooo000Qool00`00ooooo`1;ool2002_
ool0027oo`03003ooooo04goo`800:goo`008Ooo00<00?ooool0Cooo00<00?ooool0Z_oo000Qool0
0`00ooooo`1@ool00`00ooooo`2Yool0027oo`80057oo`03003ooooo0:Woo`008Ooo00<00?ooool0
DOoo0`00Z?oo000Qool00`00ooooo`1Dool2002Vool0027oo`03003ooooo05Koo`800003ool00?oo
0:7oo`008Ooo00<00?ooool0F?oo0P00X_oo000Qool00`00ooooo`1Jool00`00ooooo`2Oool0027o
o`8005_oo`03003ooooo09ooo`008Ooo00<00?ooool0Fooo00<00?ooool0W_oo000Qool00`00oooo
o`1Lool00`00ooooo`2Mool000;oo`8000Coo`03003ooooo00?oo`8000Coo`8000Goo`03003ooooo
00?oo`03003ooooo05goo`03003ooooo09coo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool0
1000ooooo`001?oo00<00?ooool00ooo00<00?ooool0G_oo00<00?ooool0Vooo00001Ooo003ooooo
00002?oo00@00?ooool000;oo`05003ooooo003oo`050004ool2001Pool20002ool00`00ooooo`2F
ool00005ool00?ooool00009ool20004ool20002ool01000ooooo`001Ooo00<00?ooool0H?oo00@0
0?oo000009Soo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`000_oo00<00?oo
00001Ooo00<00?ooool0HOoo00@00?ooool009Ooo`0000Goo`00ooooo`0000Soo`04003ooooo0002
ool01000ooooo`000ooo0P001Ooo00<00?ooool0I?oo00<00?ooool0UOoo0002ool2000:ool20004
ool20005ool00`00ooooo`03ool00`00ooooo`1Uool00`00ooooo`2Dool0027oo`03003ooooo06Go
o`03003ooooo09Coo`008Ooo0P00I_oo00<00?ooool0U?oo000Qool00`00ooooo`1Vool00`00oooo
o`2Cool0027oo`03003ooooo06Ooo`8009?oo`008Ooo00<00?ooool0JOoo00<00?ooool0T?oo000Q
ool00`00ooooo`1Zool00`00ooooo`2?ool0027oo`8006_oo`03003ooooo08ooo`008Ooo00<00?oo
ool0Jooo00<00?ooool0S_oo000Qool00`00ooooo`1/ool00`00ooooo`2=ool0027oo`03003ooooo
06goo`03003ooooo08coo`008Ooo00<00?ooool0K_oo00<00?ooool0Rooo000Qool2001`ool00`00
ooooo`2:ool0027oo`03003ooooo06ooo`03003ooooo08[oo`008Ooo00<00?ooool0L?oo00<00?oo
ool0ROoo000Qool00`00ooooo`1aool00`00ooooo`28ool000;oo`8000Coo`03003ooooo00?oo`80
00Coo`8000Coo`8000Goo`03003ooooo077oo`03003ooooo08Soo`0000Goo`00ooooo`0000Soo`04
003ooooo0002ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo07;oo`03003ooooo08Oo
o`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000Coo`80
07Coo`03003ooooo08Koo`0000Goo`00ooooo`0000Woo`8000Coo`8000?oo`<000Goo`03003ooooo
07Coo`03003ooooo08Goo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`000_oo
00<00?ooool01Ooo00<00?ooool0M?oo00<00?ooool0QOoo00001Ooo003ooooo00002?oo00@00?oo
ool000;oo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`1eool00`00ooooo`24ool000;o
o`8000[oo`8000Coo`8000Coo`<000Coo`03003ooooo07Koo`03003ooooo08?oo`008Ooo0P00Mooo
00<00?ooool0Pooo000Qool00`00ooooo`1gool00`00ooooo`22ool0027oo`03003ooooo07Soo`03
003ooooo087oo`008Ooo00<00?ooool0N?oo00<00?ooool0POoo000Qool00`00ooooo`1iool00`00
ooooo`20ool0027oo`03003ooooo07Woo`03003ooooo083oo`008Ooo0P00Nooo00<00?ooool0Oooo
000Qool00`00ooooo`1jool00`00ooooo`1eool30007ool0027oo`03003ooooo07_oo`03003ooooo
06coo`P000[oo`008Ooo00<00?ooool0Nooo00<00?ooool0I?oo20004_oo000Qool00`00ooooo`1l
ool00`00ooooo`1Lool7000Jool0027oo`8007goo`03003ooooo05Goo`L0027oo`008Ooo00<00?oo
ool0OOoo00<00?ooool0Cooo1@00:?oo000Qool00`00ooooo`1mool00`00ooooo`1:ool5000]ool0
027oo`03003ooooo07goo`03003ooooo04Ooo`<003;oo`000_oo0P001?oo00<00?ooool00ooo0P00
1?oo0P001?oo0P001Ooo00<00?ooool0O_oo00<00?ooool0@ooo0`00=Ooo00001Ooo003ooooo0000
2?oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool0O_oo00<00?oo
ool0@?oo0`00>?oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`001?oo0P00P?oo00<00?ooool0>ooo1000>ooo00001Ooo003ooooo00002Ooo0P001?oo0P00
1?oo0P001Ooo00<00?ooool0Oooo00<00?ooool0>?oo0`00?ooo00001Ooo003ooooo00002?oo00@0
0?ooool000;oo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool0P?oo00<00?ooool0=Ooo
0P00@_oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
1?oo00<00?ooool0P?oo00<00?ooool0<_oo0`00A?oo0002ool2000:ool20004ool20004ool20005
ool00`00ooooo`21ool00`00ooooo`0_ool20017ool0027oo`8008;oo`03003ooooo02koo`03003o
oooo04Ooo`008Ooo00<00?ooool0POoo00<00?ooool0;?oo0P00B_oo000Qool00`00ooooo`22ool0
0`00ooooo`0Yool2001<ool0027oo`03003ooooo08;oo`03003ooooo02Soo`03003ooooo04coo`00
8Ooo00<00?ooool0P_oo00<00?ooool09_oo0P00Cooo000Qool00`00ooooo`23ool00`00ooooo`0S
ool2001Aool0027oo`8001Soo`8000Coo`03003ooooo00;oo`@001;oo`8000Coo`03003ooooo00Co
o`03003ooooo017oo`8000Coo`03003ooooo00?oo`8001?oo`8000Coo`04003ooooo0002ool2000H
ool40004ool00`00ooooo`0@ool40003ool00`00ooooo`02ool4000Aool40003ool00`00ooooo`04
ool00`00ooooo`0?ool0027oo`03003ooooo01Koo`04003ooooo0008ool00`00ooooo`0Bool01000
ooooo`002_oo00<00?ooool04?oo00@00?ooool000Soo`04003ooooo000Aool01000ooooo`001ooo
0P000_oo00<00?ooool05ooo00<00?ooool00ooo00<00?ooool04_oo00<00?ooool01ooo00<00?oo
ool05?oo00<00?ooool02Ooo00<00?ooool03ooo000Qool00`00ooooo`0Fool01000ooooo`002Ooo
00<00?ooool04Ooo00@00?ooool000Ooo`D0017oo`04003ooooo0008ool01000ooooo`004Ooo00@0
0?ooool000Ooo`8000;oo`03003ooooo01Ooo`04003oooooool2000Eool00`00ooooo`08ool00`00
ooooo`0Cool00`00ooooo`06ool5000@ool0027oo`03003ooooo01Koo`04003ooooo000:ool00`00
ooooo`0@ool01000ooooo`001ooo00@00?ooool001;oo`04003ooooo0008ool3000Bool01000oooo
o`001ooo00@00?oo000001[oo`04003ooooo000Gool00`00ooooo`09ool00`00ooooo`0Bool00`00
ooooo`06ool01000ooooo`004Ooo000Qool00`00ooooo`0Fool01000ooooo`002ooo00<00?ooool0
3ooo00@00?ooool000Soo`03003oo`0001;oo`04003ooooo0008ool00`00ooooo`0Bool01000oooo
o`002?oo00@00?ooool001Woo`03003oo`0001Soo`03003ooooo00[oo`03003ooooo017oo`03003o
oooo00Ooo`03003oo`00017oo`008Ooo0P005ooo00@00?ooool000Soo`04003ooooo000Aool01000
ooooo`002Ooo0P004_oo00@00?ooool000Soo`03003ooooo01;oo`04003ooooo0008ool01000oooo
o`005ooo10005ooo0`002Ooo00@00?ooool0017oo`<000[oo`80017oo`008Ooo00<00?ooool05ooo
0P002_oo0P004ooo0P002ooo00<00?ooool04Ooo0P002_oo0`004_oo0P002_oo0P006_oo0P006Ooo
00<00?ooool02?oo0P005?oo00<00?ooool02Ooo00<00?ooool03ooo000Qool00`00ooooo`26ool0
0`00ooooo`0Iool00`00ooooo`1Gool0027oo`03003ooooo08Koo`03003ooooo01Soo`03003ooooo
05Soo`008Ooo00<00?ooool0Q_oo00<00?ooool05ooo00<00?ooool0FOoo000Koooo00060000027o
o`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Go
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00?oo`03003oo`0000Ooo`03003ooooo00Koo`05003oooooool0
0003ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00oooo
o`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00oooo
o`05ool00`00ooooo`05ool0027oo`03003ooooo01koo`03003ooooo01koo`03003ooooo01koo`03
003ooooo01koo`03003ooooo00?oo`03003ooooo01Coo`05003oooooool0000Pool00`00ooooo`0N
ool00`00ooooo`0Eool0027oo`03003ooooo08Ooo`03003ooooo01?oo`03003ooooo05coo`008Ooo
00<00?ooool0Qooo00<00?ooool04ooo00<00?ooool0G?oo000Qool00`00ooooo`28ool00`00oooo
o`0Aool00`00ooooo`1Mool0027oo`8008Woo`03003ooooo017oo`03003ooooo05goo`008Ooo00<0
0?ooool0R?oo00<00?ooool04?oo00<00?ooool0G_oo000Qool00`00ooooo`28ool00`00ooooo`0@
ool00`00ooooo`1Nool0027oo`03003ooooo08Woo`03003ooooo00ooo`03003ooooo05koo`008Ooo
00<00?ooool0ROoo00<00?ooool03_oo00<00?ooool0Gooo000Qool2002:ool00`00ooooo`0>ool0
0`00ooooo`1Oool0027oo`03003ooooo08[oo`03003ooooo00goo`03003ooooo05ooo`008Ooo00<0
0?ooool0R_oo00<00?ooool03?oo00<00?ooool0H?oo000Qool00`00ooooo`2:ool00`00ooooo`0<
ool00`00ooooo`1Pool0027oo`03003ooooo08[oo`03003ooooo00_oo`03003ooooo067oo`008Ooo
00<00?ooool0Rooo00<00?ooool02_oo00<00?ooool0HOoo000Qool2002<ool00`00ooooo`0:ool0
0`00ooooo`1Qool0027oo`03003ooooo08_oo`03003ooooo00Woo`03003ooooo06;oo`008Ooo00<0
0?ooool0Rooo00<00?ooool02Ooo00<00?ooool0H_oo0002ool20004ool00`00ooooo`03ool20003
ool30003ool40004ool00`00ooooo`2;ool00`00ooooo`09ool00`00ooooo`1Rool00005ool00?oo
ool00008ool01000ooooo`001Ooo00@00?ooool000Ooo`03003ooooo08_oo`03003ooooo00Soo`03
003ooooo06?oo`0000Goo`00ooooo`0000Soo`04003ooooo0005ool01@00oooooooo00001_oo0P00
SOoo00<00?ooool01ooo00<00?ooool0Hooo00001Ooo003ooooo00002Ooo0P001?oo0`001?oo00<0
0?ooool00ooo00<00?ooool0S?oo00<00?ooool01ooo00<00?ooool0Hooo00001Ooo003ooooo0000
2?oo00@00?ooool000;oo`04003ooooo0005ool00`00ooooo`02ool00`00ooooo`2<ool00`00oooo
o`07ool00`00ooooo`1Sool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000;o
o`04003ooooo0004ool00`00ooooo`2<ool00`00ooooo`07ool00`00ooooo`1Sool000;oo`8000[o
o`8000Coo`8000Coo`8000Goo`03003ooooo08coo`03003ooooo00Ooo`03003ooooo06?oo`008Ooo
00<00?ooool0S?oo00<00?ooool01_oo00<00?ooool0I?oo000Qool2002=ool00`00ooooo`06ool0
0`00ooooo`1Tool0027oo`03003ooooo08goo`03003ooooo00Goo`03003ooooo06Coo`008Ooo00<0
0?ooool0SOoo00<00?ooool01Ooo00<00?ooool0I?oo000Qool00`00ooooo`2=ool00`00ooooo`05
ool00`00ooooo`1Tool0027oo`03003ooooo08goo`03003ooooo00Goo`03003ooooo06Coo`008Ooo
0P00S_oo00<00?ooool01?oo00<00?ooool0IOoo000Qool00`00ooooo`2=ool00`00ooooo`04ool0
0`00ooooo`1Uool0027oo`03003ooooo08koo`03003ooooo00?oo`03003ooooo06Goo`008Ooo00<0
0?ooool0S_oo00<00?ooool00ooo00<00?ooool0IOoo000Qool00`00ooooo`2>ool00`00ooooo`03
ool00`00ooooo`1Uool0027oo`03003ooooo08koo`03003ooooo00?oo`03003ooooo06Goo`008Ooo
0P00Sooo00<00?ooool00_oo00<00?ooool0I_oo000Qool00`00ooooo`2?ool01@00oooooooo0000
J?oo000Qool00`00ooooo`2?ool01@00oooooooo0000J?oo0002ool20004ool00`00ooooo`03ool2
0003ool30005ool00`00ooooo`03ool00`00ooooo`2?ool01@00oooooooo0000J?oo00001Ooo003o
oooo00002?oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo08ooo`05
003oooooool0001Xool00005ool00?ooool00008ool01000ooooo`001Ooo00<00?oo000010001?oo
0P00T?oo00@00?ooool006Woo`0000Goo`00ooooo`0000Woo`8000Coo`<00005ool00?ooool00005
ool00`00ooooo`2?ool01000ooooo`00JOoo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04
003ooooo0002ool00`00ool00005ool00`00ooooo`2@ool00`00ool0001Yool00005ool00?ooool0
0008ool01000ooooo`000_oo00@00?ooool000?oo`8000Goo`03003ooooo093oo`03003oo`0006Wo
o`000_oo0P002_oo0P001?oo0P001Ooo00<00?ooool00ooo00<00?ooool0T?oo00<00?oo0000JOoo
000Qool2002Aool00`00ool0001Yool0027oo`03003ooooo093oo`8006[oo`008Ooo00<00?ooool0
TOoo00<00?ooool0J?oo000Qool00`00ooooo`2Aool00`00ooooo`1Xool0027oo`03003ooooo0?co
o`008Ooo00<00?ooool0o?oo000Qool2003mool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.201469, 0.879533, \
0.00605915, 9.18835*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.5, 10^4]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8828239264954276`, 
      mGamma \[Rule] \(-0.01001877302967988`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[0.74, 10^4]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8862330963770715`}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(realsol[k_, a_] := 
      FindRoot[Re[kx0corrtestfunc[Om, k, a]] \[Equal] 0, {Om, 0.5, 0.6}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol[1, 10^4]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8894592146897627`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, Re[Om /. realsol[k, 10^4]]}, {k, 0.01, 0.95}, 
      PlotStyle \[Rule] {{}, RGBColor[1, 0, 0]}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.00251 0.0147151 0.619583 [
[.22431 .00222 -9 -9 ]
[.22431 .00222 9 0 ]
[.42481 .00222 -9 -9 ]
[.42481 .00222 9 0 ]
[.62531 .00222 -9 -9 ]
[.62531 .00222 9 0 ]
[.82581 .00222 -9 -9 ]
[.82581 .00222 9 0 ]
[.01131 .13863 -18 -4.5 ]
[.01131 .13863 0 4.5 ]
[.01131 .26255 -18 -4.5 ]
[.01131 .26255 0 4.5 ]
[.01131 .38646 -18 -4.5 ]
[.01131 .38646 0 4.5 ]
[.01131 .51038 -18 -4.5 ]
[.01131 .51038 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.22431 .01472 m
.22431 .02097 L
s
[(0.2)] .22431 .00222 0 1 Mshowa
.42481 .01472 m
.42481 .02097 L
s
[(0.4)] .42481 .00222 0 1 Mshowa
.62531 .01472 m
.62531 .02097 L
s
[(0.6)] .62531 .00222 0 1 Mshowa
.82581 .01472 m
.82581 .02097 L
s
[(0.8)] .82581 .00222 0 1 Mshowa
.125 Mabswid
.07393 .01472 m
.07393 .01847 L
s
.12406 .01472 m
.12406 .01847 L
s
.17419 .01472 m
.17419 .01847 L
s
.27444 .01472 m
.27444 .01847 L
s
.32456 .01472 m
.32456 .01847 L
s
.37469 .01472 m
.37469 .01847 L
s
.47494 .01472 m
.47494 .01847 L
s
.52506 .01472 m
.52506 .01847 L
s
.57519 .01472 m
.57519 .01847 L
s
.67544 .01472 m
.67544 .01847 L
s
.72556 .01472 m
.72556 .01847 L
s
.77569 .01472 m
.77569 .01847 L
s
.87594 .01472 m
.87594 .01847 L
s
.92607 .01472 m
.92607 .01847 L
s
.97619 .01472 m
.97619 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13863 m
.03006 .13863 L
s
[(0.2)] .01131 .13863 1 0 Mshowa
.02381 .26255 m
.03006 .26255 L
s
[(0.4)] .01131 .26255 1 0 Mshowa
.02381 .38646 m
.03006 .38646 L
s
[(0.6)] .01131 .38646 1 0 Mshowa
.02381 .51038 m
.03006 .51038 L
s
[(0.8)] .01131 .51038 1 0 Mshowa
.125 Mabswid
.02381 .04569 m
.02756 .04569 L
s
.02381 .07667 m
.02756 .07667 L
s
.02381 .10765 m
.02756 .10765 L
s
.02381 .16961 m
.02756 .16961 L
s
.02381 .20059 m
.02756 .20059 L
s
.02381 .23157 m
.02756 .23157 L
s
.02381 .29353 m
.02756 .29353 L
s
.02381 .32451 m
.02756 .32451 L
s
.02381 .35549 m
.02756 .35549 L
s
.02381 .41744 m
.02756 .41744 L
s
.02381 .44842 m
.02756 .44842 L
s
.02381 .4794 m
.02756 .4794 L
s
.02381 .54136 m
.02756 .54136 L
s
.02381 .57234 m
.02756 .57234 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03383 .02091 m
.07206 .04454 L
.11375 .0703 L
.15291 .0945 L
.19057 .11778 L
.23065 .14255 L
.26923 .16639 L
.31024 .19174 L
.34975 .21616 L
.38776 .23965 L
.4282 .26464 L
.46713 .2887 L
.50457 .31184 L
.54443 .33648 L
.5828 .36019 L
.62359 .3854 L
.66288 .40968 L
.70067 .43304 L
.74089 .45789 L
.7796 .48182 L
.82075 .50725 L
.86039 .53175 L
.89854 .55533 L
.93911 .5804 L
.97619 .60332 L
s
1 0 0 r
.03383 .40307 m
.03843 .40318 L
.04272 .40332 L
.05244 .40377 L
.06192 .40438 L
.07206 .40521 L
.09389 .40766 L
.11375 .41061 L
.15326 .41835 L
.19127 .42785 L
.23171 .43979 L
.27065 .45274 L
.30808 .46626 L
.34795 .4816 L
.38631 .4971 L
.4271 .51425 L
.46639 .53131 L
.50418 .54814 L
.51504 .55304 L
.52528 .55768 L
.52974 .55971 L
.53092 .56022 L
.53218 .56081 L
.53338 .56125 L
.53446 .5615 L
.53573 .56162 L
.53693 .56162 L
.53823 .56165 L
.53896 .56167 L
.53964 .56167 L
.54074 .56168 L
.54195 .56168 L
.54321 .56169 L
.5444 .5617 L
.58312 .56196 L
.6214 .56224 L
.66211 .56259 L
.70131 .56299 L
.73902 .56344 L
.77915 .56402 L
.81779 .56475 L
.83891 .56526 L
.85885 .56587 L
.86847 .56622 L
.87865 .56666 L
.88819 .56717 L
.89691 .56776 L
.90625 .56861 L
.91139 .56926 L
.91628 .57009 L
Mistroke
.91846 .57056 L
.92078 .57115 L
.92278 .57177 L
.92495 .57258 L
.9256 .57284 L
.92628 .57314 L
.9275 .57234 L
.92865 .5703 L
.92991 .5698 L
.93103 .56949 L
.93223 .56924 L
.9344 .56888 L
.93694 .56856 L
.93964 .56829 L
.94449 .56791 L
.94971 .5676 L
.95545 .56732 L
.96574 .56694 L
.97538 .56667 L
.97619 .56665 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool004Coo`8000Coo`03003ooooo00;oo`@002Soo`8000Coo`03003ooooo00Coo`03003ooooo02Oo
o`8000Coo`03003ooooo00?oo`8002Soo`8000Coo`03003ooooo00?oo`8002[oo`00@ooo00@00?oo
ool000Soo`03003ooooo02Soo`04003ooooo000:ool00`00ooooo`0Vool01000ooooo`002?oo00@0
0?ooool002Koo`04003ooooo0008ool01000ooooo`00:Ooo0013ool01000ooooo`002Ooo00<00?oo
ool09ooo00@00?ooool000Ooo`D002Ooo`04003ooooo0008ool01000ooooo`009_oo00@00?ooool0
00Soo`04003ooooo000Yool004?oo`04003ooooo000:ool00`00ooooo`0Vool01000ooooo`001ooo
00@00?ooool002Soo`04003ooooo0008ool3000Wool01000ooooo`002Ooo0P00:_oo0013ool01000
ooooo`002ooo00<00?ooool09Ooo00@00?ooool000Soo`03003oo`0002Soo`04003ooooo0008ool0
0`00ooooo`0Wool01000ooooo`002?oo00@00?ooool002Woo`00@ooo00@00?ooool000Soo`04003o
oooo000Wool01000ooooo`002Ooo0P00:?oo00@00?ooool000Soo`03003ooooo02Ooo`04003ooooo
0008ool01000ooooo`00:Ooo000Eool00`00ooooo`0/ool2000:ool2000Yool2000;ool00`00oooo
o`0Wool2000:ool3000Wool2000:ool2000Zool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?oo
ool0oooo2Ooo000Eool00`00ooooo`3oool9ool000koool001<000005Ooo00<00?ooool02ooo00<0
0?ooool02_oo00<00?ooool02ooo00<00?ooool02_oo00<00?ooool02ooo00<00?ooool02ooo00<0
0?ooool02_oo00<00?ooool02ooo00<00?ooool02ooo00<00?ooool02_oo00<00?ooool02ooo00<0
0?ooool02ooo00<00?ooool02_oo00<00?ooool02ooo00<00?ooool02_oo00<00?ooool02ooo00<0
0?ooool02ooo00<00?ooool02_oo00<00?ooool02ooo00<00?ooool01Ooo000Eool01000ooooo`00
<_oo00<00?ooool0=?oo00<00?ooool0=?oo00<00?ooool0<ooo00<00?ooool0;_oo000Eool01000
oooooooo0P00oooo1_oo000Eool00`00ooooo`03ool2003oool4ool001Goo`03003ooooo00Goo`03
003ooooo0?ooo`7oo`005Ooo00<00?ooool01_oo0P00oooo0Ooo000Eool00`00ooooo`08ool2003n
ool001Goo`03003ooooo00[oo`03003ooooo0?_oo`005Ooo0P003?oo0P00nooo000Eool00`00oooo
o`0=ool00`00ooooo`3hool001Goo`03003ooooo00koo`800?Soo`005Ooo00<00?ooool04?oo0P00
m_oo000Eool00`00ooooo`0Bool00`00ooooo`3cool001Goo`03003ooooo01?oo`800??oo`005Ooo
00<00?ooool05Ooo00<00?ooool0l?oo000Eool00`00ooooo`0Fool2003`ool001Goo`8001Woo`03
003ooooo0>goo`005Ooo00<00?ooool06Ooo0P00kOoo000Eool00`00ooooo`0Kool2003[ool001Go
o`03003ooooo01goo`03003ooooo0>Soo`005Ooo00<00?ooool07_oo0P00j?oo000Eool00`00oooo
o`0Pool00`00ooooo`3Uool001Goo`03003ooooo027oo`800>Goo`005Ooo00<00?ooool08ooo0P00
hooo000Eool2000Vool00`00ooooo`3Pool001Goo`03003ooooo02Koo`800>3oo`005Ooo00<00?oo
ool0:?oo0P00g_oo000Eool00`00ooooo`0Zool00`00ooooo`3Kool001Goo`03003ooooo02_oo`80
0=_oo`005Ooo00<00?ooool0;Ooo00<00?ooool0f?oo000Eool00`00ooooo`0^ool2003Hool000;o
o`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo033oo`800=Koo`0000Goo`00ooooo`0000So
o`03003ooooo00Goo`03003ooooo03;oo`03003ooooo0=?oo`0000Goo`00ooooo`0000Woo`03003o
oooo00Coo`<003?oo`800=?oo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo03Go
o`03003ooooo0=3oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003ooooo03Koo`800=3o
o`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0hool2003>ool000;oo`8000[o
o`8000Goo`03003ooooo03[oo`800<coo`005Ooo00<00?ooool0??oo0P00b_oo000Eool00`00oooo
o`0nool20038ool001Goo`03003ooooo043oo`03003ooooo0<Goo`005Ooo0P00@_oo0P00aOoo000E
ool00`00ooooo`13ool00`00ooooo`32ool001Goo`03003ooooo04Coo`800<;oo`005Ooo00<00?oo
ool0A_oo0P00`?oo000Eool00`00ooooo`18ool00`00ooooo`2mool001Goo`03003ooooo04Woo`80
0;goo`005Ooo00<00?ooool0Booo00<00?ooool0^_oo000Eool00`00ooooo`1<ool2002jool001Go
o`03003ooooo04koo`03003ooooo0;Ooo`005Ooo0P00D?oo0P00]ooo000Eool00`00ooooo`1Aool2
002eool001Goo`03003ooooo05?oo`03003ooooo0;;oo`005Ooo00<00?ooool0E?oo0P00/_oo000E
ool00`00ooooo`1Fool00`00ooooo`2_ool001Goo`03003ooooo05Ooo`800:ooo`005Ooo00<00?oo
ool0FOoo0P00[Ooo000Eool00`00ooooo`1Kool00`00ooooo`2Zool001Goo`8005goo`800:[oo`00
5Ooo00<00?ooool0G_oo0P00Z?oo000Eool00`00ooooo`1Pool00`00ooooo`2Uool001Goo`03003o
oooo067oo`800:Goo`005Ooo00<00?ooool0Hooo00<00?ooool0X_oo000Eool00`00ooooo`1Tool2
002Rool001Goo`03003ooooo06Koo`800:3oo`000_oo0P001?oo00<00?ooool01?oo00<00?ooool0
0ooo00<00?ooool0J?oo00<00?ooool0WOoo00001Ooo003ooooo00002_oo00<00?ooool00ooo00<0
0?ooool0JOoo0P00WOoo00001Ooo003ooooo00001ooo1@001?oo0`00Jooo00<00?ooool0V_oo0000
1Ooo003ooooo00001ooo00@00?ooool000Goo`03003ooooo06coo`8009[oo`0000Goo`00ooooo`00
00Soo`03003oo`0000Goo`03003ooooo06koo`03003ooooo09Ooo`0000Goo`00ooooo`0000Woo`80
00Goo`03003ooooo06ooo`8009Ooo`000_oo0P002ooo00<00?ooool00ooo00<00?ooool0LOoo0P00
UOoo000Eool00`00ooooo`1cool00`00ooooo`2Bool001Goo`03003ooooo07Coo`8009;oo`005Ooo
00<00?ooool0M_oo00<00?ooool0Sooo000Eool2001hool2002?ool001Goo`03003ooooo07Woo`80
08goo`005Ooo00<00?ooool0Nooo00<00?ooool0R_oo000Eool00`00ooooo`1lool2002:ool001Go
o`03003ooooo07koo`8008Soo`005Ooo00<00?ooool0P?oo00<00?ooool0QOoo000Eool00`00oooo
o`21ool20025ool001Goo`03003ooooo08?oo`03003ooooo08;oo`005Ooo0P00QOoo0P00P_oo000E
ool00`00ooooo`26ool20020ool001Goo`03003ooooo08Soo`03003ooooo07goo`005Ooo00<00?oo
ool0ROoo0P00OOoo000Eool00`00ooooo`2;ool00`00ooooo`1jool001Goo`03003ooooo08coo`80
07[oo`005Ooo00<00?ooool0S_oo0P00N?oo000Eool00`00ooooo`2@ool00`00ooooo`1eool001Go
o`03003ooooo097oo`8007Goo`005Ooo0P00U?oo0P00Looo000Eool00`00ooooo`2Eool00`00oooo
o`1`ool001Goo`03003ooooo09Koo`80073oo`005Ooo00<00?ooool0V?oo00<00?ooool0KOoo000E
ool00`00ooooo`2Iool2001]ool001Goo`03003ooooo09_oo`8006_oo`000_oo0P001?oo00<00?oo
ool00ooo0P001Ooo00<00?ooool0WOoo00<00?ooool0J?oo00001Ooo003ooooo00002?oo00@00?oo
ool000Coo`03003ooooo09koo`8006Soo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool3002P
ool00`00ooooo`1Uool00005ool00?ooool00008ool30005ool00`00ooooo`2Qool2001Uool00005
ool00?ooool00008ool00`00ooooo`05ool00`00ooooo`2Sool00`00ooooo`1Rool00005ool00?oo
ool00008ool00`00ooooo`05ool00`00ooooo`2Tool2001Rool000;oo`8000[oo`<000Coo`03003o
oooo0:Koo`80063oo`005Ooo00<00?ooool03OP0Vooo00<00?ooool0GOoo000Eool00`00ooooo`0=
ool6n02Fool2001Mool001Goo`03003ooooo01?oo`Gh09?oo`03003ooooo05[oo`005Ooo00<00?oo
ool06?oo1_P0S_oo0P00F_oo000Eool2000Oool4n02<ool2001Hool001Goo`03003ooooo02;oo`?h
08_oo`03003ooooo05Goo`005Ooo00<00?ooool09Ooo1?P0R?oo0P00EOoo000Eool00`00ooooo`0Y
ool3n027ool2001Cool001Goo`03003ooooo02coo`Ch08Goo`03003ooooo053oo`005Ooo00<00?oo
ool0<?oo1?P0P_oo0P00D?oo000Eool00`00ooooo`0dool3n021ool00`00ooooo`1=ool001Goo`03
003ooooo03Ooo`Ch07koo`8004goo`005Ooo0P00??oo1?P0O?oo0P00Booo000Eool00`00ooooo`0o
ool3n01kool00`00ooooo`18ool001Goo`03003ooooo04;oo`;h07[oo`8004Soo`005Ooo00<00?oo
ool0A?oo0oP0NOoo00<00?ooool0AOoo000Eool00`00ooooo`17ool2n01hool20015ool001Goo`03
003ooooo04Woo`?h07Ooo`8004?oo`005Ooo00<00?ooool0C?oo0oP0M_oo0P00@Ooo000Eool00`00
ooooo`1?ool2n01fool2000oool001Goo`03003ooooo057oo`?h07Goo`8003goo`005Ooo0P00EOoo
0oP0M?oo00<00?ooool0>_oo000Eool00`00ooooo`1Gool3n01bool2000jool001Goo`03003ooooo
05[oo`;h07;oo`03003ooooo03Ooo`005Ooo00<00?ooool0G?oo0oP0L?oo0P00=ooo000Eool00`00
ooooo`1Oool3n01_ool2000eool001Goo`03003ooooo06;oo`;h06ooo`03003ooooo03;oo`000_oo
0P001?oo00<00?ooool00ooo0P001Ooo00<00?ooool0I?oo0_P0K_oo0P00<_oo00001Ooo003ooooo
00002?oo00@00?ooool000Coo`03003ooooo06Koo`;h06koo`03003ooooo02ooo`0000Goo`00oooo
o`0000Soo`04003ooooo0004ool3001Xool2n01]ool2000_ool00005ool00?ooool00009ool20005
ool00`00ooooo`1Zool2n01]ool00`00ooooo`0/ool00005ool00?ooool00008ool01000ooooo`00
1?oo00<00?ooool0K?oo0_P0K?oo0P00;?oo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03
003ooooo06koo`;h06coo`8002[oo`000_oo0P002_oo0P001Ooo00<00?ooool0L?oo0_P0K?oo00<0
0?ooool09ooo000Eool00`00ooooo`1bool2n01[ool2000Wool001Goo`03003ooooo07Coo`?h06[o
o`03003ooooo02Coo`005Ooo00<00?ooool0Mooo0oP0J?oo0P009?oo000Eool2001kool2n01Xool2
000Rool001Goo`03003ooooo07coo`?h06Ooo`03003ooooo01ooo`005Ooo00<00?ooool0Oooo0_P0
I_oo0P007ooo000Eool00`00ooooo`21ool2n01Vool2000Mool001Goo`03003ooooo08?oo`03n03o
oooo06Goo`03003ooooo01[oo`005Ooo00<00?ooool0Q?oo1?P0I?oo0P006_oo000Eool00`00oooo
o`27oolnn00Yool00`00ooooo`0Gool001Goo`03003ooooo0<GoobCh00Koo`8000_oo`Gh00Ooo`00
5Ooo00<00?ooool0jOoo2?P00P002OP03?oo000Eool2003bool3n00Dool001Goo`03003ooooo0?Co
o`8001;oo`005Ooo00<00?ooool0m_oo00<00?ooool03ooo000Eool00`00ooooo`3gool2000?ool0
01Goo`03003ooooo0?Woo`8000goo`005Ooo00<00?ooool0nooo00<00?ooool02_oo000Eool00`00
ooooo`3lool2000:ool001Goo`03003ooooo0?koo`8000Soo`005Ooo0P00oooo0_oo00<00?ooool0
1Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo
2Ooo000Eool00`00ooooo`3oool9ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0784658, -0.0747171, \
0.00366626, 0.00593216}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%1183, %1174, PlotRange \[Rule] {0.6, 0.9}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -1.23607 2.06011 [
[.15079 -0.0125 -9 -9 ]
[.15079 -0.0125 9 0 ]
[.27778 -0.0125 -9 -9 ]
[.27778 -0.0125 9 0 ]
[.40476 -0.0125 -9 -9 ]
[.40476 -0.0125 9 0 ]
[.53175 -0.0125 -9 -9 ]
[.53175 -0.0125 9 0 ]
[.65873 -0.0125 -3 -9 ]
[.65873 -0.0125 3 0 ]
[.78571 -0.0125 -9 -9 ]
[.78571 -0.0125 9 0 ]
[.9127 -0.0125 -9 -9 ]
[.9127 -0.0125 9 0 ]
[.01131 .10301 -24 -4.5 ]
[.01131 .10301 0 4.5 ]
[.01131 .20601 -18 -4.5 ]
[.01131 .20601 0 4.5 ]
[.01131 .30902 -24 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .41202 -18 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .51503 -24 -4.5 ]
[.01131 .51503 0 4.5 ]
[.01131 .61803 -18 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 0 m
.15079 .00625 L
s
[(0.2)] .15079 -0.0125 0 1 Mshowa
.27778 0 m
.27778 .00625 L
s
[(0.4)] .27778 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(0.6)] .40476 -0.0125 0 1 Mshowa
.53175 0 m
.53175 .00625 L
s
[(0.8)] .53175 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(1)] .65873 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(1.2)] .78571 -0.0125 0 1 Mshowa
.9127 0 m
.9127 .00625 L
s
[(1.4)] .9127 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.18254 0 m
.18254 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.34127 0 m
.34127 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.5 0 m
.5 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.81746 0 m
.81746 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .10301 m
.03006 .10301 L
s
[(0.65)] .01131 .10301 1 0 Mshowa
.02381 .20601 m
.03006 .20601 L
s
[(0.7)] .01131 .20601 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(0.75)] .01131 .30902 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(0.8)] .01131 .41202 1 0 Mshowa
.02381 .51503 m
.03006 .51503 L
s
[(0.85)] .01131 .51503 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.9)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.40476 0 m
.42855 .07719 L
.45249 .15485 L
.47796 .2375 L
.50248 .31706 L
.52854 .40161 L
.55365 .48308 L
.5778 .56147 L
s
.5778 .56147 m
.59524 .61803 L
s
1 0 0 r
.03016 .05521 m
.03307 .05558 L
.03578 .05604 L
.04194 .05753 L
.04795 .05955 L
.05437 .06234 L
.0682 .07047 L
.08077 .08028 L
.1058 .10602 L
.12987 .13762 L
.15548 .17732 L
.18014 .22035 L
.20385 .26531 L
.2291 .31631 L
.25339 .36788 L
.27923 .4249 L
.30411 .48162 L
.32805 .53756 L
.33492 .55387 L
.3414 .5693 L
.34423 .57605 L
.34498 .57774 L
.34578 .5797 L
.34653 .58115 L
.34722 .582 L
.34803 .5824 L
.34879 .58238 L
.34961 .58251 L
.35007 .58255 L
.3505 .58257 L
.3512 .5826 L
.35196 .58259 L
.35276 .58263 L
.35352 .58267 L
.37804 .58352 L
.40228 .58446 L
.42806 .58562 L
.45289 .58693 L
.47678 .58843 L
.50219 .59038 L
.52666 .59281 L
.54004 .5945 L
.55267 .59651 L
.55876 .59769 L
.56521 .59916 L
.57125 .60085 L
.57677 .6028 L
.58269 .60565 L
.58595 .6078 L
.58904 .61054 L
Mistroke
.59042 .6121 L
.59189 .61409 L
.59316 .61616 L
Mfstroke
.59316 .61616 m
.59413 .61803 L
s
.59615 .61803 m
.59615 .61802 L
.59688 .61126 L
.59767 .60958 L
.59838 .60856 L
.59914 .60771 L
.60052 .60652 L
.60213 .60545 L
.60384 .60455 L
.60691 .6033 L
.61021 .60226 L
.61385 .60135 L
.62036 .60009 L
.62647 .59918 L
.62698 .59912 L
s
0 g
.03016 .57814 m
.03651 .57814 L
.04286 .57815 L
.04921 .57816 L
.05556 .57818 L
.0619 .5782 L
.06825 .57822 L
.0746 .57824 L
.08095 .57827 L
.0873 .5783 L
.09365 .57833 L
.1 .57837 L
.10635 .57841 L
.1127 .57846 L
.11905 .5785 L
.1254 .57856 L
.13175 .57861 L
.1381 .57867 L
.14444 .57873 L
.15079 .57879 L
.15714 .57886 L
.16349 .57893 L
.16984 .57901 L
.17619 .57909 L
.18254 .57917 L
.18889 .57926 L
.19524 .57935 L
.20159 .57944 L
.20794 .57954 L
.21429 .57964 L
.22063 .57975 L
.22698 .57986 L
.23333 .57998 L
.23968 .58009 L
.24603 .58022 L
.25238 .58035 L
.25873 .58048 L
.26508 .58061 L
.27143 .58075 L
.27778 .5809 L
.28413 .58081 L
.29048 .58121 L
.29683 .58137 L
.30317 .58153 L
.30952 .58171 L
.31587 .58188 L
.32222 .58207 L
.32857 .58226 L
.33492 .58244 L
.34127 .58265 L
Mistroke
.34762 .58314 L
.35397 .58308 L
.37302 .58376 L
.37937 .58357 L
.38571 .58425 L
.39206 .58451 L
.39841 .58478 L
.40476 .58506 L
.41111 .58535 L
.41746 .58513 L
.42381 .58596 L
.43016 .58632 L
.43651 .58661 L
.44286 .58696 L
.44921 .58732 L
.45556 .5877 L
.4619 .5881 L
.46825 .58851 L
.4746 .58894 L
.48095 .58939 L
.4873 .58986 L
.49365 .59037 L
.5 .5909 L
.50635 .59146 L
.5127 .59206 L
.51905 .5927 L
.5254 .59339 L
.53175 .59413 L
.5381 .59494 L
.54444 .59583 L
.55079 .59682 L
.55714 .59792 L
.56349 .59918 L
.56984 .60064 L
.57619 .60234 L
.58254 .60431 L
.58889 .60635 L
.59524 .6071 L
.60159 .60504 L
.60794 .60293 L
.61429 .60142 L
.62063 .60029 L
.62698 .5994 L
.63333 .59868 L
.63968 .59807 L
.64603 .59755 L
.65238 .5971 L
.65873 .5967 L
.66508 .59635 L
.67143 .59603 L
Mistroke
.67778 .59574 L
.68413 .59548 L
.69048 .59524 L
.69683 .59502 L
.70317 .59481 L
.70952 .59463 L
.71587 .59445 L
.72222 .59429 L
.72857 .59413 L
.73492 .59399 L
.74127 .59386 L
.74762 .59373 L
.75397 .59361 L
.76032 .5935 L
.76667 .59339 L
.77302 .59329 L
.77937 .59319 L
.78571 .5931 L
.79206 .59301 L
.79841 .59293 L
.80476 .59285 L
.81111 .59277 L
.81746 .5927 L
.82381 .59263 L
.83016 .59256 L
.83651 .59249 L
.84286 .59243 L
.84921 .59237 L
.85556 .59232 L
.8619 .59226 L
.86825 .59221 L
.8746 .59216 L
.88095 .59211 L
.8873 .59206 L
.89365 .59201 L
.9 .59197 L
.90635 .59193 L
.9127 .59188 L
.91905 .59184 L
.9254 .59181 L
.93175 .59177 L
.9381 .59173 L
.94444 .5917 L
.95079 .59166 L
.95714 .59163 L
.96349 .5916 L
.96984 .59156 L
.97619 .59153 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool003Soo`8000Coo`03003ooooo00;oo`@001;oo`8000Coo`03003ooooo00Coo`03003ooooo017o
o`8000Coo`03003ooooo00?oo`8001?oo`8000Coo`03003ooooo00?oo`8001Soo`@001Ooo`@000?o
o`03003ooooo00;oo`@0017oo`@000?oo`03003ooooo00Coo`03003ooooo01;oo`00=ooo00@00?oo
ool000Soo`03003ooooo01;oo`04003ooooo000:ool00`00ooooo`0@ool01000ooooo`002?oo00@0
0?ooool0017oo`04003ooooo0008ool01000ooooo`006Ooo00<00?ooool06?oo00<00?ooool01ooo
00<00?ooool05?oo00<00?ooool02Ooo00<00?ooool04_oo000gool01000ooooo`002Ooo00<00?oo
ool04Ooo00@00?ooool000Ooo`D0017oo`04003ooooo0008ool01000ooooo`004Ooo00@00?ooool0
00Soo`04003ooooo000Iool00`00ooooo`0Hool00`00ooooo`08ool00`00ooooo`0Cool00`00oooo
o`06ool5000Cool003Ooo`04003ooooo000:ool00`00ooooo`0@ool01000ooooo`001ooo00@00?oo
ool001;oo`04003ooooo0008ool3000Bool01000ooooo`002Ooo0P006_oo00<00?ooool06?oo00<0
0?ooool02Ooo00<00?ooool04_oo00<00?ooool01_oo00@00?ooool001Coo`00=ooo00@00?ooool0
00_oo`03003ooooo00ooo`04003ooooo0008ool00`00ool0000Bool01000ooooo`002?oo00<00?oo
ool04_oo00@00?ooool000Soo`04003ooooo000Iool00`00ooooo`0Hool00`00ooooo`0:ool00`00
ooooo`0Aool00`00ooooo`07ool00`00ool0000Dool003Ooo`04003ooooo0008ool01000ooooo`00
4Ooo00@00?ooool000Woo`8001;oo`04003ooooo0008ool00`00ooooo`0Bool01000ooooo`002?oo
00@00?ooool001Ooo`<001Soo`<000Woo`04003ooooo000Aool3000:ool2000Dool003Soo`8000[o
o`8001?oo`8000_oo`03003ooooo017oo`8000[oo`<001;oo`8000[oo`8001[oo`03003ooooo01So
o`03003ooooo00Soo`8001Coo`03003ooooo00Woo`03003ooooo01;oo`00oooo8Ooo003ooolQool0
0?ooob7oo`006?ooo`001@001?oo000Nool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`08ool001koo`03003ooooo067oo`03003o
oooo09_oo`007_oo00<00?ooool0HOoo00<00?ooool0Vooo000Nool00`00ooooo`1Qool00`00oooo
o`2Kool001koo`8006;oo`03003ooooo09_oo`007_oo00<00?ooool0H_oo00<00?ooool0V_oo000N
ool00`00ooooo`1Rool00`00ooooo`2Jool001koo`03003ooooo06;oo`03003ooooo09[oo`007_oo
00<00?ooool0Hooo00<00?ooool0VOoo000Nool2001Tool00`00ooooo`2Iool001koo`03003ooooo
06?oo`03003ooooo09Woo`007_oo00<00?ooool0I?oo00<00?ooool0V?oo000Nool00`00ooooo`1T
ool00`00ooooo`2Hool001koo`04003oooP0n01Sool00`00ooooo`2Hool001koo`04003oooooool3
n01Pool00`00ooooo`2Hool001koo`8000Goo`;h05ooo`03003ooooo09Ooo`007_oo00<00?ooool0
1_oo0_P0GOoo00<00?ooool0Uooo000Nool00`00ooooo`08ool2n01Kool00`00ooooo`2Gool001ko
o`03003ooooo00[oo`03n03ooooo05Woo`03003ooooo09Koo`007_oo00<00?ooool02ooo00?h0?oo
ool0F?oo00<00?ooool0U_oo000Nool2000=ool00oP0ooooo`1Gool00`00ooooo`2Fool001koo`03
003ooooo00goo`03n03ooooo05Ooo`03003ooooo09Goo`007_oo00<00?ooool03_oo00?h0?ooool0
E_oo00<00?ooool0UOoo000Nool00`00ooooo`0?ool00oP0ooooo`1Eool00`00ooooo`2Eool000Go
o`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo013oo`03n03ooooo05Coo`03003o
oooo09Goo`001?oo00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`02ool00`00ooooo`0A
ool00oP0ooooo`1Dool00`00ooooo`2Dool000Coo`04003ooooo0008ool01000ooooo`001Ooo00<0
0?ooool00_oo0`004_oo00?h0?ooool0Dooo00<00?ooool0U?oo0004ool01000ooooo`002?oo0`00
0ooo0`001Ooo00<00?ooool04ooo00?h0?ooool0D_oo00<00?ooool0U?oo0004ool01000ooooo`00
2?oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool05?oo00?h0?ooool0D_oo00<00?ooool0
Tooo0004ool01000ooooo`002?oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool05Ooo00?h
0?ooool0DOoo00<00?ooool0Tooo0005ool2000:ool30002ool40004ool00`00ooooo`0Eool00oP0
ooooo`1Aool00`00ooooo`2Cool001koo`8001Ooo`03n03ooooo057oo`03003ooooo09;oo`007_oo
00<00?ooool05ooo00?h0?ooool0D?oo00<00?ooool0T_oo000Nool00`00ooooo`0Hool00oP0oooo
o`1?ool00`00ooooo`2Bool001koo`03003ooooo01Woo`03n03ooooo04koo`03003ooooo09;oo`00
7_oo00<00?ooool06_oo00?h0?ooool0C_oo00<00?ooool0TOoo000Nool2000Kool00oP0ooooo`1>
ool00`00ooooo`2Aool001koo`03003ooooo01_oo`03n03ooooo04goo`03003ooooo097oo`007_oo
00<00?ooool06ooo00?h0?ooool0C_oo00<00?ooool0T?oo000Nool00`00ooooo`0Lool00oP0oooo
o`1=ool00`00ooooo`2@ool001koo`03003ooooo01coo`03n03ooooo04goo`03003ooooo093oo`00
7_oo00<00?ooool07Ooo00?h0?ooool0COoo00<00?ooool0Sooo000Nool2000Nool00oP0ooooo`1=
ool00`00ooooo`2?ool001koo`03003ooooo01koo`03n03ooooo04coo`03003ooooo08ooo`007_oo
00<00?ooool07_oo00?h0?ooool0COoo00<00?ooool0S_oo000Nool00`00ooooo`0Oool00oP0oooo
o`1<ool00`00ooooo`2>ool001koo`03003ooooo023oo`03n03ooooo04_oo`03003ooooo08koo`00
7_oo0P008Ooo00?h0?ooool0C?oo00<00?ooool0SOoo000Nool00`00ooooo`0Qool00oP0ooooo`1;
ool00`00ooooo`2=ool001koo`03003ooooo02;oo`03n03ooooo04[oo`03003ooooo08goo`002ooo
0P001?oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool08_oo00?h0?ooool0Booo00<00?oo
ool0S?oo000:ool01000ooooo`002Ooo00<00?ooool01?oo00<00?ooool08ooo00?h0?ooool0B_oo
00<00?ooool0S?oo000:ool01000ooooo`002_oo00<00?ooool00ooo0`008ooo00?h0?ooool0B_oo
00<00?ooool0S?oo000:ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool09?oo00?h0?oo
ool0B_oo00<00?ooool0Rooo000:ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool09Ooo
00?h0?ooool0BOoo00<00?ooool0Rooo000:ool01000ooooo`002?oo00@00?ooool000Coo`03003o
oooo02Goo`03n03ooooo04Woo`03003ooooo08_oo`002ooo0P002Ooo10001?oo00<00?ooool09_oo
00?h0?ooool0BOoo00<00?ooool0R_oo000Nool00`00ooooo`0Vool00oP0ooooo`19ool00`00oooo
o`2:ool001koo`8002Soo`03n03ooooo04Soo`03003ooooo08[oo`007_oo00<00?ooool09ooo00?h
0?ooool0BOoo00<00?ooool0ROoo000Nool00`00ooooo`0Xool00oP0ooooo`18ool00`00ooooo`29
ool001koo`03003ooooo02Soo`03n03ooooo04Soo`03003ooooo08Woo`007_oo00<00?ooool0:Ooo
00?h0?ooool0B?oo00<00?ooool0R?oo000Nool2000Zool00oP0ooooo`18ool00`00ooooo`28ool0
01koo`03003ooooo02[oo`03n03ooooo04Ooo`03003ooooo08Soo`007_oo00<00?ooool0:_oo00?h
0?ooool0Aooo00<00?ooool0R?oo000Nool00`00ooooo`0[ool00oP0ooooo`17ool00`00ooooo`27
ool001koo`03003ooooo02_oo`03n03ooooo04Ooo`03003ooooo08Ooo`007_oo0P00;Ooo00?h0?oo
ool0A_oo00<00?ooool0Qooo000Nool00`00ooooo`0/ool00oP0ooooo`17ool00`00ooooo`26ool0
01koo`03003ooooo02goo`03n03ooooo04Koo`03003ooooo08Koo`007_oo00<00?ooool0;Ooo00?h
0?ooool0A_oo00<00?ooool0Q_oo000Nool00`00ooooo`0^ool00oP0ooooo`15ool00`00ooooo`26
ool001koo`03003ooooo02koo`03n03ooooo04Koo`03003ooooo08Goo`007_oo0P00<?oo00?h0?oo
ool0AOoo00<00?ooool0QOoo000Nool00`00ooooo`0_ool00oP0ooooo`15ool00`00ooooo`25ool0
01koo`03003ooooo033oo`03n03ooooo04Goo`03003ooooo08Coo`001Ooo0P001?oo00<00?ooool0
0ooo00<00?ooool00_oo0`001Ooo00<00?ooool0<?oo00?h0?ooool0AOoo00<00?ooool0Q?oo0004
ool01000ooooo`002Ooo00<00?ooool01Ooo00<00?ooool00_oo00<00?ooool0<Ooo00?h0?ooool0
A?oo00<00?ooool0Q?oo0004ool01000ooooo`002_oo00<00?ooool01?oo00<00?ooool00_oo0`00
<Ooo00?h0?ooool0A?oo00<00?ooool0Q?oo0004ool01000ooooo`002_oo00@00?ooooooo`<000Go
o`03003ooooo03;oo`03n03ooooo04Coo`03003ooooo08?oo`001?oo00@00?ooool000_oo`04003o
oooo0007ool00`00ooooo`0bool00oP0ooooo`14ool00`00ooooo`23ool000Coo`04003ooooo0008
ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0<ooo00?h0?ooool0@ooo00<00?ooool0
Pooo0005ool20009ool40002ool40004ool00`00ooooo`0cool00oP0ooooo`14ool00`00ooooo`22
ool001koo`8003Goo`03n03ooooo04?oo`03003ooooo08;oo`007_oo00<00?ooool0=?oo00?h0?oo
ool0@ooo00<00?ooool0P_oo000Nool00`00ooooo`0eool00oP0ooooo`13ool00`00ooooo`21ool0
01koo`03003ooooo03Goo`03n03ooooo04?oo`03003ooooo087oo`007_oo00<00?ooool0=_oo00?h
0?ooool0@_oo00<00?ooool0POoo000Nool00`00ooooo`0fool00oP0ooooo`13ool00`00ooooo`20
ool001koo`8003Soo`03n03ooooo04;oo`03003ooooo083oo`007_oo00<00?ooool0=ooo00?h0?oo
ool0@_oo00<00?ooool0P?oo000Nool00`00ooooo`0hool00oP0ooooo`11ool00`00ooooo`20ool0
01koo`03003ooooo03Soo`03n03ooooo04;oo`03003ooooo07ooo`007_oo00<00?ooool0>Ooo00?h
0?ooool0@Ooo00<00?ooool0Oooo000Nool2000jool00oP0ooooo`11ool00`00ooooo`1oool001ko
o`03003ooooo03[oo`03n03ooooo047oo`03003ooooo07koo`007_oo00<00?ooool0>_oo00?h0?oo
ool0@Ooo00<00?ooool0O_oo000Nool00`00ooooo`0kool00oP0ooooo`10ool00`00ooooo`1nool0
01koo`03003ooooo03_oo`03n03ooooo047oo`03003ooooo07goo`007_oo0P00??oo00?h0?ooool0
@Ooo00<00?ooool0OOoo000Nool00`00ooooo`0lool00oP0ooooo`10ool00`00ooooo`1mool001ko
o`03003ooooo03coo`03n03ooooo047oo`03003ooooo07coo`007_oo00<00?ooool0?Ooo00?h0?oo
ool0@?oo00<00?ooool0O?oo000;ool20004ool00`00ooooo`03ool20005ool00`00ooooo`0mool0
0oP0ooooo`10ool00`00ooooo`1lool000[oo`04003ooooo0008ool01000ooooo`001?oo00<00?oo
ool0?Ooo00?h0?ooool0@Ooo00<00?ooool0Nooo000:ool01000ooooo`002?oo00@00?ooool000Co
o`<003koo`03n03ooooo043oo`03003ooooo07_oo`002_oo00@00?ooool000Woo`8000Goo`03003o
oooo03koo`03n03ooooo043oo`03003ooooo07_oo`002_oo00@00?ooool000Soo`04003ooooo0004
ool00`00ooooo`0oool00oP0ooooo`0oool00`00ooooo`1kool000[oo`04003ooooo0008ool01000
ooooo`001?oo00<00?ooool0?ooo00?h0?ooool0@?oo00<00?ooool0N_oo000;ool2000:ool20005
ool00`00ooooo`0oool00oP0ooooo`10ool00`00ooooo`1jool001koo`80047oo`03n03ooooo03oo
o`03003ooooo07[oo`007_oo00<00?ooool0@?oo00?h0?ooool0@?oo00<00?ooool0NOoo000Nool0
0`00ooooo`11ool00oP0ooooo`0oool00`00ooooo`1iool001koo`03003ooooo047oo`03n03ooooo
03ooo`03003ooooo07Woo`007_oo00<00?ooool0@_oo00?h0?ooool0?_oo00<00?ooool0NOoo000N
ool20013ool00oP0ooooo`0oool00`00ooooo`1hool001koo`03003ooooo04?oo`03n03ooooo03ko
o`03003ooooo07Soo`007_oo00<00?ooool0@ooo00?h0?ooool0?_oo00<00?ooool0N?oo000Nool0
0`00ooooo`14ool00oP0ooooo`0nool00`00ooooo`1gool001koo`03003ooooo04Coo`03n03ooooo
03koo`03003ooooo07Ooo`007_oo00<00?ooool0AOoo00?h0?ooool0?Ooo00<00?ooool0Mooo000N
ool20016ool00oP0ooooo`0mool00`00ooooo`1gool001koo`03003ooooo04Koo`03n03ooooo03go
o`03003ooooo07Koo`007_oo00<00?ooool0A_oo00?h0?ooool0?Ooo00<00?ooool0M_oo000Nool0
0`00ooooo`16ool00oP0ooooo`0mool00`00ooooo`1fool001koo`03003ooooo04Ooo`03n03ooooo
03goo`03003ooooo07Goo`007_oo0P00B?oo00?h0?ooool0?Ooo00<00?ooool0MOoo000Nool00`00
ooooo`18ool00oP0ooooo`0lool00`00ooooo`1eool001koo`03003ooooo04Soo`03n03ooooo03go
o`03003ooooo07Coo`001Ooo0P001?oo00<00?ooool00ooo0P000ooo0`001Ooo00<00?ooool0BOoo
00?h0?ooool0??oo00<00?ooool0M?oo0004ool01000ooooo`002?oo00@00?ooool000Goo`03003o
oooo00;oo`03003ooooo04Woo`03n03ooooo03coo`03003ooooo07Coo`001?oo00@00?ooool000So
o`04003ooooo0005ool00`00ooooo`02ool30019ool00oP0ooooo`0mool00`00ooooo`1cool000Co
o`04003ooooo0009ool20003ool30005ool00`00ooooo`1:ool00oP0ooooo`0lool00`00ooooo`1c
ool000Coo`04003ooooo0008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0B_oo00?h
0?ooool0??oo00<00?ooool0Looo0004ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo
00Goo`03003ooooo04_oo`03n03ooooo03coo`03003ooooo07;oo`001Ooo0P002_oo0P000ooo1000
1?oo00<00?ooool0Booo00?h0?ooool0??oo00<00?ooool0L_oo000Nool00`00ooooo`1<ool00oP0
ooooo`0lool00`00ooooo`1aool001koo`8004goo`03n03ooooo03coo`03003ooooo077oo`007_oo
00<00?ooool0C?oo00?h0?ooool0??oo00<00?ooool0LOoo000Nool00`00ooooo`1=ool00oP0oooo
o`0lool00`00ooooo`1`ool001koo`03003ooooo04goo`03n03ooooo03coo`03003ooooo073oo`00
7_oo00<00?ooool0C_oo00?h0?ooool0>ooo00<00?ooool0L?oo000Nool2001?ool00oP0ooooo`0l
ool00`00ooooo`1_ool001koo`03003ooooo04koo`03n03ooooo03coo`03003ooooo06ooo`007_oo
00<00?ooool0Cooo00?h0?ooool0>ooo00<00?ooool0Kooo000Nool00`00ooooo`1?ool00oP0oooo
o`0lool00`00ooooo`1^ool001koo`03003ooooo04ooo`03n03ooooo03coo`03003ooooo06koo`00
7_oo0P00DOoo00?h0?ooool0>ooo00<00?ooool0K_oo000Nool00`00ool00015000;ool2n00lool0
0`00ooooo`1^ool001koo`03003ooooo04Gooa`00003n000000002_oo`03003ooooo06goo`007_oo
00<00?ooool0HOoo00<00?oon0003`000oP06Ooo00<00?ooool0KOoo000Nool00`00ooooo`1cool;
0003n00>ool00`00ooooo`0cool`000:ool001koo`03003ooooo07koo`T000Soo`03003ooooo00oo
ob@003[oo`007_oo0P00R?oo1@001?oo00<00?ooool01?oo2P00G_oo000Nool00`00ooooo`2:ool2
n00300000ooo003oo`02ool3001Xool001koo`03003ooooo08goo`;h00D006_oo`002ooo0P001?oo
00<00?ooool00_oo0`001Ooo00<00?ooool0T?oo00?h0?oon000K?oo000:ool01000ooooo`002ooo
00<00?ooool00_oo00<00?ooool0T?oo00?h0000n000K?oo000:ool01000ooooo`002ooo00<00?oo
ool00_oo0`00TOoo0_P0K?oo000:ool01000ooooo`002Ooo0`00oooo1ooo000:ool01000ooooo`00
2?oo00@00?ooool00?ooo`Ooo`002_oo00@00?ooool000Soo`04003ooooo003oool7ool000_oo`80
00[oo`800?ooo`Soo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.185957, 0.577078, \
0.00607629, 0.0018727}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(realsol2[k_, a_] := 
      FindRoot[kx0corrtestfunc[Om, k, a] \[Equal] 0, {Om, 0.9, 0.95}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(realsol2[0.42, 100]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8474825678107697`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Om /. realsol2[k, 10^4]], {k, 0.01, 0.5}, 
      PlotStyle \[Rule] {RGBColor[0, 1, 0]}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.90476 -256.298 291.054 [
[.21429 .10871 -9 -9 ]
[.21429 .10871 9 0 ]
[.40476 .10871 -9 -9 ]
[.40476 .10871 9 0 ]
[.59524 .10871 -9 -9 ]
[.59524 .10871 9 0 ]
[.78571 .10871 -9 -9 ]
[.78571 .10871 9 0 ]
[.97619 .10871 -9 -9 ]
[.97619 .10871 9 0 ]
[.01131 .26674 -36 -4.5 ]
[.01131 .26674 0 4.5 ]
[.01131 .41227 -30 -4.5 ]
[.01131 .41227 0 4.5 ]
[.01131 .5578 -36 -4.5 ]
[.01131 .5578 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .12121 m
.21429 .12746 L
s
[(0.1)] .21429 .10871 0 1 Mshowa
.40476 .12121 m
.40476 .12746 L
s
[(0.2)] .40476 .10871 0 1 Mshowa
.59524 .12121 m
.59524 .12746 L
s
[(0.3)] .59524 .10871 0 1 Mshowa
.78571 .12121 m
.78571 .12746 L
s
[(0.4)] .78571 .10871 0 1 Mshowa
.97619 .12121 m
.97619 .12746 L
s
[(0.5)] .97619 .10871 0 1 Mshowa
.125 Mabswid
.0619 .12121 m
.0619 .12496 L
s
.1 .12121 m
.1 .12496 L
s
.1381 .12121 m
.1381 .12496 L
s
.17619 .12121 m
.17619 .12496 L
s
.25238 .12121 m
.25238 .12496 L
s
.29048 .12121 m
.29048 .12496 L
s
.32857 .12121 m
.32857 .12496 L
s
.36667 .12121 m
.36667 .12496 L
s
.44286 .12121 m
.44286 .12496 L
s
.48095 .12121 m
.48095 .12496 L
s
.51905 .12121 m
.51905 .12496 L
s
.55714 .12121 m
.55714 .12496 L
s
.63333 .12121 m
.63333 .12496 L
s
.67143 .12121 m
.67143 .12496 L
s
.70952 .12121 m
.70952 .12496 L
s
.74762 .12121 m
.74762 .12496 L
s
.82381 .12121 m
.82381 .12496 L
s
.8619 .12121 m
.8619 .12496 L
s
.9 .12121 m
.9 .12496 L
s
.9381 .12121 m
.9381 .12496 L
s
.25 Mabswid
0 .12121 m
1 .12121 L
s
.02381 .26674 m
.03006 .26674 L
s
[(0.8815)] .01131 .26674 1 0 Mshowa
.02381 .41227 m
.03006 .41227 L
s
[(0.882)] .01131 .41227 1 0 Mshowa
.02381 .5578 m
.03006 .5578 L
s
[(0.8825)] .01131 .5578 1 0 Mshowa
.125 Mabswid
.02381 .15032 m
.02756 .15032 L
s
.02381 .17943 m
.02756 .17943 L
s
.02381 .20853 m
.02756 .20853 L
s
.02381 .23764 m
.02756 .23764 L
s
.02381 .29585 m
.02756 .29585 L
s
.02381 .32495 m
.02756 .32495 L
s
.02381 .35406 m
.02756 .35406 L
s
.02381 .38316 m
.02756 .38316 L
s
.02381 .44137 m
.02756 .44137 L
s
.02381 .47048 m
.02756 .47048 L
s
.02381 .49958 m
.02756 .49958 L
s
.02381 .52869 m
.02756 .52869 L
s
.02381 .09211 m
.02756 .09211 L
s
.02381 .063 m
.02756 .063 L
s
.02381 .0339 m
.02756 .0339 L
s
.02381 .00479 m
.02756 .00479 L
s
.02381 .5869 m
.02756 .5869 L
s
.02381 .61601 m
.02756 .61601 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.04286 .01472 m
.05165 .01495 L
.06129 .01531 L
.07135 .0158 L
.08072 .01637 L
.10067 .0179 L
.12201 .02005 L
.16263 .02559 L
.20176 .03274 L
.23941 .04132 L
.27945 .05229 L
.31802 .06467 L
.3551 .07829 L
.39458 .09466 L
.43258 .11228 L
.47298 .13307 L
.51189 .15514 L
.54932 .17831 L
.58915 .20514 L
.6275 .23314 L
.66825 .26533 L
.70751 .29882 L
.74529 .33345 L
.78548 .37306 L
.82417 .41426 L
.86139 .4554 L
.901 .50312 L
.93913 .55423 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool002Ooo`03003ooooo0?Koo`009ooo0P00mooo000Wool00`00ooooo`3fool0
02Ooo`04003oooooool91n3/ool002Ooo`03003ooooo00[oo`T7h>?oo`009ooo00<00?ooool04ooo
1`OPg?oo000Wool00`00ooooo`0Jool81n3Dool002Ooo`03003ooooo02;oo`D7h<ooo`009ooo0P00
:?oo1@OPb_oo000Wool00`00ooooo`0/ool51n35ool002Ooo`03003ooooo037oo`@7h<7oo`009ooo
00<00?ooool0=Ooo0`OP__oo000Wool00`00ooooo`0hool41n2jool002Ooo`03003ooooo03coo`<7
h;Ooo`009ooo00<00?ooool0?ooo0`OP]?oo000Wool20013ool31n2aool002Ooo`03003ooooo04Go
o`<7h:koo`009ooo00<00?ooool0B?oo0`OPZooo000Wool00`00ooooo`1;ool41n2Wool002Ooo`03
003ooooo04ooo`<7h:Coo`009ooo00<00?ooool09Ooo0P001?oo00<00?ooool00_oo10007_oo0POP
00?oo`0000001?oo00<00?ooool00_oo10008Ooo0P001?oo00<00?ooool00ooo0P008_oo0P001?oo
00<00?ooool01?oo00<00?ooool08?oo0P001?oo00<00?ooool00_oo0`000ooo000Wool00`00oooo
o`0Tool01000ooooo`002_oo00<00?ooool07ooo0`OP00<00?ooool01_oo00<00?ooool08Ooo00@0
0?ooool000Soo`04003ooooo000Pool01000ooooo`002_oo00<00?ooool07ooo00@00?ooool000_o
o`40007oo`7oo`009ooo00<00?ooool09?oo00@00?ooool000[oo`03003ooooo01ooo`03003ooooo
0087h0Soo`03003ooooo023oo`04003ooooo000;ool00`00ooooo`0Nool01000ooooo`001ooo1@00
8?oo00@00?ooool000_oo`40007oo`7oo`009ooo0P009Ooo00@00?ooool000[oo`03003ooooo01oo
o`05003ooooo003oo`021n07ool00`00ooooo`0Oool01000ooooo`002Ooo0P008Ooo00@00?ooool0
00Ooo`04003ooooo000Qool01000ooooo`002?oo0`000ooo000Wool00`00ooooo`0Tool01000oooo
o`002_oo00<00?ooool07ooo00@00?ooool000?oo`87h0Koo`03003ooooo01koo`04003ooooo000;
ool00`00ooooo`0Nool01000ooooo`002?oo00<00?oo00008Ooo00@00?ooool000Soo`03003ooooo
00?oo`009ooo00<00?ooool09?oo00@00?ooool000Soo`<0027oo`04003ooooo0005ool21n001Ooo
003ooooo00008?oo00@00?ooool000Soo`04003ooooo000Pool01000ooooo`002Ooo0P008Ooo00@0
0?ooool000Soo`03003ooooo00?oo`009ooo00<00?ooool09Ooo0P002ooo00<00?ooool08?oo0P00
2?oo0POP0P008_oo0P002_oo0P008_oo0P002ooo00<00?ooool08?oo0P002Ooo10000_oo000Wool0
0`00ooooo`1Qool21n2Cool002Ooo`03003ooooo06?oo`031n3ooooo093oo`009ooo00<00?ooool0
I?oo0POPT?oo000Qoom_00021n2:0004ool002Ooo`03003ooooo00Koo`03003ooooo00Ooo`03003o
oooo00Koo`03003ooooo00Ooo`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00Ooo`03003o
oooo00Koo`03003ooooo00Ooo`03003ooooo00Koo`03003ooooo00Ooo`04003oo`OP1n05ool00`00
ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`06ool00`00
ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00
ooooo`06ool00`00ooooo`07ool00`00ooooo`07ool00`00ooooo`07ool002Ooo`03003ooooo06[o
o`87h8[oo`009ooo00<00?ooool0K?oo0POPR?oo000Wool00`00ooooo`1^ool21n26ool002Ooo`03
003ooooo073oo`87h8Coo`009ooo00<00?ooool0L_oo0POPP_oo000Wool2001eool21n20ool002Oo
o`03003ooooo07Koo`87h7koo`009ooo00<00?ooool0N?oo0POPO?oo000Wool00`00ooooo`1jool0
0`OPooooo`1iool002Ooo`03003ooooo07_oo`87h7Woo`009ooo00<00?ooool0OOoo00<7h?ooool0
M_oo000Wool00`00ooooo`1nool21n1fool002Ooo`03003ooooo083oo`031n3ooooo07?oo`009ooo
0P00P_oo0POPLooo000Wool00`00ooooo`23ool00`OPooooo`1`ool002Ooo`03003ooooo08Coo`03
1n3ooooo06ooo`009ooo00<00?ooool0QOoo0POPKooo000Wool00`00ooooo`27ool00`OPooooo`1/
ool002Ooo`03003ooooo08Soo`87h6coo`009ooo00<00?ooool0R_oo00<7h?ooool0JOoo000Wool2
002<ool21n1Yool002Ooo`03003ooooo08goo`031n3ooooo06Koo`009ooo00<00?ooool0S_oo00<7
h?ooool0IOoo000Wool00`00ooooo`2?ool21n1Uool002Ooo`03003ooooo097oo`031n3ooooo06;o
o`009ooo00<00?ooool0T_oo0POPH_oo000Wool00`00ooooo`2Dool00`OPooooo`1Oool002Ooo`80
09Koo`031n3ooooo05koo`009ooo00<00?ooool0U_oo0POPG_oo000Wool00`00ooooo`2Hool00`OP
ooooo`1Kool002Ooo`03003ooooo09Woo`031n3ooooo05[oo`009ooo00<00?ooool0V_oo00<7h?oo
ool0FOoo0002ool20004ool00`00ooooo`03ool20004ool20003ool40002ool30005ool00`00oooo
o`2Kool21n1Iool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000Coo`03003o
oooo00Coo`03003ooooo00;oo`03003ooooo09goo`031n3ooooo05Koo`0000Goo`00ooooo`0000So
o`04003ooooo0002ool01000ooooo`001?oo00<00?ooool01?oo00<00?ooool00_oo0P00Wooo00<7
h?ooool0EOoo00001Ooo003ooooo00002Ooo0P001?oo0P001Ooo00@00?ooooooo`<000Goo`03003o
oooo09ooo`031n3ooooo05Coo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`00
1?oo00D00?ooooooo`0000Ooo`03003ooooo0:3oo`87h5Coo`0000Goo`00ooooo`0000Soo`04003o
oooo0002ool01000ooooo`000_oo0`000ooo00<00?ooool01Ooo00<00?ooool0X_oo00<7h?ooool0
DOoo0002ool2000:ool20004ool20005ool01000oooooooo10001?oo00<00?ooool0Xooo00<7h?oo
ool0D?oo000Wool00`00ooooo`2Tool00`OPooooo`1?ool002Ooo`03003ooooo0:Goo`87h4ooo`00
9ooo00<00?ooool0Yooo00<7h?ooool0C?oo000Wool2002Yool00`OPooooo`1;ool002Ooo`03003o
oooo0:Woo`031n3ooooo04[oo`009ooo00<00?ooool0Z_oo00<7h?ooool0BOoo000Wool00`00oooo
o`2[ool00`OPooooo`18ool002Ooo`03003ooooo0:coo`031n3ooooo04Ooo`009ooo00<00?ooool0
[Ooo0POPAooo000Wool00`00ooooo`2_ool00`OPooooo`14ool002Ooo`800;7oo`031n3ooooo04?o
o`009ooo00<00?ooool0/Ooo00<7h?ooool0@_oo000Wool00`00ooooo`2bool00`OPooooo`11ool0
02Ooo`03003ooooo0;?oo`031n3ooooo043oo`009ooo00<00?ooool0]?oo00<7h?ooool0?ooo000W
ool00`00ooooo`2eool00`OPooooo`0nool002Ooo`03003ooooo0;Koo`031n3ooooo03goo`009ooo
0P00^?oo00<7h?ooool0??oo000Wool00`00ooooo`2hool00`OPooooo`0kool002Ooo`03003ooooo
0;Woo`031n3ooooo03[oo`009ooo00<00?ooool0^_oo00<7h?ooool0>Ooo000Wool00`00ooooo`2k
ool00`OPooooo`0hool002Ooo`03003ooooo0;coo`031n3ooooo03Ooo`009ooo00<00?ooool0_Ooo
00<7h?ooool0=_oo000Wool00`00ooooo`2nool00`OPooooo`0eool002Ooo`800<3oo`031n3ooooo
03Coo`009ooo00<00?ooool0`?oo00<7h?ooool0<ooo000Wool00`00ooooo`30ool00`OPooooo`0c
ool002Ooo`03003ooooo0<7oo`031n3ooooo03;oo`009ooo00<00?ooool0`_oo00<7h?ooool0<Ooo
0008ool20004ool00`00ooooo`03ool20004ool20003ool40004ool00`00ooooo`33ool00`OPoooo
o`0`ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Go
o`03003ooooo0<Coo`031n3ooooo02ooo`001ooo00@00?ooool000Soo`04003ooooo0002ool01000
ooooo`000ooo00<00?ooool01?oo0P00a_oo00<7h?ooool0;_oo0007ool01000ooooo`002Ooo0P00
1?oo0P001Ooo00<00?ooool00ooo00<00?ooool0a_oo00<7h?ooool0;Ooo0007ool01000ooooo`00
2?oo00@00?ooool000;oo`04003ooooo0005ool00`00ooooo`02ool00`00ooooo`37ool00`OPoooo
o`0/ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0004
ool00`00ooooo`38ool00`OPooooo`0[ool000Soo`8000[oo`8000Coo`8000Coo`8000Goo`03003o
oooo0<Woo`031n3ooooo02[oo`009ooo00<00?ooool0b_oo00<7h?ooool0:Ooo000Wool00`00oooo
o`3:ool00`OPooooo`0Yool002Ooo`800<coo`031n3ooooo02Soo`009ooo00<00?ooool0c?oo00<7
h?ooool09ooo000Wool00`00ooooo`3=ool00`OPooooo`0Vool002Ooo`03003ooooo0<koo`031n3o
oooo02Goo`009ooo00<00?ooool0cooo00<7h?ooool09?oo000Wool00`00ooooo`3@ool00`OPoooo
o`0Sool002Ooo`03003ooooo0=7oo`031n3ooooo02;oo`009ooo00<00?ooool0dOoo00<7h?ooool0
8_oo000Wool2003Cool00`OPooooo`0Qool002Ooo`03003ooooo0=?oo`031n3ooooo023oo`009ooo
00<00?ooool0e?oo00<7h?ooool07ooo000Wool00`00ooooo`3Eool00`OPooooo`0Nool002Ooo`03
003ooooo0=Koo`031n3ooooo01goo`009ooo00<00?ooool0e_oo00<7h?ooool07Ooo000Wool00`00
ooooo`3Gool00`OPooooo`0Lool002Ooo`800=Woo`031n3ooooo01_oo`009ooo00<00?ooool0fOoo
00<7h?ooool06_oo000Wool00`00ooooo`3Jool00`OPooooo`0Iool002Ooo`03003ooooo0=[oo`03
1n3ooooo01Woo`009ooo00<00?ooool0fooo00<7h?ooool06?oo000Wool00`00ooooo`3Lool00`OP
ooooo`0Gool002Ooo`03003ooooo0=coo`031n3ooooo01Ooo`009ooo0P00g_oo00<7h?ooool05_oo
000Wool00`00ooooo`3Nool00`OPooooo`0Eool002Ooo`03003ooooo0=ooo`031n3ooooo01Coo`00
9ooo00<00?ooool0gooo00<7h?ooool05?oo000Wool00`00ooooo`3Pool00`OPooooo`0Cool000;o
o`8000Coo`03003ooooo00?oo`8000Coo`8000?oo`@000;oo`<000Goo`03003ooooo0>7oo`031n3o
oooo01;oo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool0
1_oo00<00?ooool00_oo00<00?ooool0hOoo00<7h?ooool04_oo00001Ooo003ooooo00002?oo00@0
0?ooool000;oo`04003ooooo0003ool00`00ooooo`05ool00`00ooooo`02ool2003Sool00`OPoooo
o`0Aool00005ool00?ooool00009ool20004ool20005ool01000oooooooo0`001Ooo00<00?ooool0
hooo00<7h?ooool04?oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0005ool0
1000ooooo`001ooo00<00?ooool0i?oo00<7h?ooool03ooo00001Ooo003ooooo00002?oo00@00?oo
ool000;oo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0i?oo00<7
h?ooool03ooo0002ool2000:ool20004ool20004ool20003ool40004ool00`00ooooo`3Uool00`OP
ooooo`0>ool002Ooo`03003ooooo0>Koo`031n3ooooo00goo`009ooo00<00?ooool0iooo00<7h?oo
ool03?oo000Wool00`00ooooo`3Xool00`OPooooo`0;ool002Ooo`800>[oo`031n3ooooo00[oo`00
9ooo00<00?ooool0jOoo00<7h?ooool02_oo000Wool00`00ooooo`3Zool00`OPooooo`09ool002Oo
o`03003ooooo0>_oo`031n3ooooo00Soo`009ooo00<00?ooool0k?oo00<7h?ooool01ooo000Wool0
0`00ooooo`3fool002Ooo`03003ooooo0?Koo`009ooo0P00mooo000Wool00`00ooooo`3fool00?oo
ob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0819219, 0.880436, \
0.00209325, 1.3699*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %1189]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -14.2995 16.5051 [
[.15079 .54256 -9 -9 ]
[.15079 .54256 9 0 ]
[.27778 .54256 -9 -9 ]
[.27778 .54256 9 0 ]
[.40476 .54256 -9 -9 ]
[.40476 .54256 9 0 ]
[.53175 .54256 -9 -9 ]
[.53175 .54256 9 0 ]
[.65873 .54256 -3 -9 ]
[.65873 .54256 3 0 ]
[.78571 .54256 -9 -9 ]
[.78571 .54256 9 0 ]
[.9127 .54256 -9 -9 ]
[.9127 .54256 9 0 ]
[.01131 .05991 -24 -4.5 ]
[.01131 .05991 0 4.5 ]
[.01131 .14244 -30 -4.5 ]
[.01131 .14244 0 4.5 ]
[.01131 .22496 -24 -4.5 ]
[.01131 .22496 0 4.5 ]
[.01131 .30749 -30 -4.5 ]
[.01131 .30749 0 4.5 ]
[.01131 .39001 -24 -4.5 ]
[.01131 .39001 0 4.5 ]
[.01131 .47254 -30 -4.5 ]
[.01131 .47254 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .55506 m
.15079 .56131 L
s
[(0.2)] .15079 .54256 0 1 Mshowa
.27778 .55506 m
.27778 .56131 L
s
[(0.4)] .27778 .54256 0 1 Mshowa
.40476 .55506 m
.40476 .56131 L
s
[(0.6)] .40476 .54256 0 1 Mshowa
.53175 .55506 m
.53175 .56131 L
s
[(0.8)] .53175 .54256 0 1 Mshowa
.65873 .55506 m
.65873 .56131 L
s
[(1)] .65873 .54256 0 1 Mshowa
.78571 .55506 m
.78571 .56131 L
s
[(1.2)] .78571 .54256 0 1 Mshowa
.9127 .55506 m
.9127 .56131 L
s
[(1.4)] .9127 .54256 0 1 Mshowa
.125 Mabswid
.05556 .55506 m
.05556 .55881 L
s
.0873 .55506 m
.0873 .55881 L
s
.11905 .55506 m
.11905 .55881 L
s
.18254 .55506 m
.18254 .55881 L
s
.21429 .55506 m
.21429 .55881 L
s
.24603 .55506 m
.24603 .55881 L
s
.30952 .55506 m
.30952 .55881 L
s
.34127 .55506 m
.34127 .55881 L
s
.37302 .55506 m
.37302 .55881 L
s
.43651 .55506 m
.43651 .55881 L
s
.46825 .55506 m
.46825 .55881 L
s
.5 .55506 m
.5 .55881 L
s
.56349 .55506 m
.56349 .55881 L
s
.59524 .55506 m
.59524 .55881 L
s
.62698 .55506 m
.62698 .55881 L
s
.69048 .55506 m
.69048 .55881 L
s
.72222 .55506 m
.72222 .55881 L
s
.75397 .55506 m
.75397 .55881 L
s
.81746 .55506 m
.81746 .55881 L
s
.84921 .55506 m
.84921 .55881 L
s
.88095 .55506 m
.88095 .55881 L
s
.94444 .55506 m
.94444 .55881 L
s
.97619 .55506 m
.97619 .55881 L
s
.25 Mabswid
0 .55506 m
1 .55506 L
s
.02381 .05991 m
.03006 .05991 L
s
[(0.87)] .01131 .05991 1 0 Mshowa
.02381 .14244 m
.03006 .14244 L
s
[(0.875)] .01131 .14244 1 0 Mshowa
.02381 .22496 m
.03006 .22496 L
s
[(0.88)] .01131 .22496 1 0 Mshowa
.02381 .30749 m
.03006 .30749 L
s
[(0.885)] .01131 .30749 1 0 Mshowa
.02381 .39001 m
.03006 .39001 L
s
[(0.89)] .01131 .39001 1 0 Mshowa
.02381 .47254 m
.03006 .47254 L
s
[(0.895)] .01131 .47254 1 0 Mshowa
.125 Mabswid
.02381 .07642 m
.02756 .07642 L
s
.02381 .09292 m
.02756 .09292 L
s
.02381 .10943 m
.02756 .10943 L
s
.02381 .12593 m
.02756 .12593 L
s
.02381 .15894 m
.02756 .15894 L
s
.02381 .17545 m
.02756 .17545 L
s
.02381 .19195 m
.02756 .19195 L
s
.02381 .20846 m
.02756 .20846 L
s
.02381 .24147 m
.02756 .24147 L
s
.02381 .25797 m
.02756 .25797 L
s
.02381 .27448 m
.02756 .27448 L
s
.02381 .29098 m
.02756 .29098 L
s
.02381 .32399 m
.02756 .32399 L
s
.02381 .3405 m
.02756 .3405 L
s
.02381 .357 m
.02756 .357 L
s
.02381 .37351 m
.02756 .37351 L
s
.02381 .40652 m
.02756 .40652 L
s
.02381 .42302 m
.02756 .42302 L
s
.02381 .43953 m
.02756 .43953 L
s
.02381 .45603 m
.02756 .45603 L
s
.02381 .48904 m
.02756 .48904 L
s
.02381 .50555 m
.02756 .50555 L
s
.02381 .52205 m
.02756 .52205 L
s
.02381 .53856 m
.02756 .53856 L
s
.02381 .04341 m
.02756 .04341 L
s
.02381 .0269 m
.02756 .0269 L
s
.02381 .0104 m
.02756 .0104 L
s
.02381 .57157 m
.02756 .57157 L
s
.02381 .58807 m
.02756 .58807 L
s
.02381 .60458 m
.02756 .60458 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 1 0 r
.5 Mabswid
.03016 .23543 m
.03309 .23544 L
.0363 .23546 L
.03966 .23549 L
.04278 .23552 L
.04943 .23561 L
.05654 .23573 L
.07008 .23604 L
.08313 .23645 L
.09567 .23694 L
.10902 .23756 L
.12188 .23826 L
.13424 .23903 L
.1474 .23996 L
.16006 .24096 L
.17353 .24214 L
.1865 .24339 L
.19898 .24471 L
.21226 .24623 L
.22504 .24781 L
.23862 .24964 L
.25171 .25154 L
.2643 .2535 L
.2777 .25575 L
.2906 .25809 L
.303 .26042 L
.31621 .26312 L
.32892 .26602 L
.34127 .26881 L
s
0 g
.57389 0 m
.5778 .10185 L
s
.5778 .10185 m
.59766 .61803 L
s
1 0 0 r
.33277 0 m
.33492 .04101 L
.3414 .1646 L
.34423 .21866 L
.34498 .23223 L
.34578 .24791 L
.34653 .25959 L
.34722 .26637 L
.34803 .26961 L
.34879 .26939 L
.34961 .27043 L
.35007 .27074 L
.3505 .27094 L
.3512 .27119 L
.35196 .27106 L
.35276 .27145 L
.35352 .27173 L
.37804 .27853 L
.40228 .2861 L
.42806 .29541 L
.45289 .3059 L
.47678 .31788 L
.50219 .3335 L
.52666 .35294 L
.54004 .36654 L
.55267 .38259 L
.55876 .39204 L
.56521 .40387 L
.57125 .41741 L
.57677 .43305 L
.58269 .45582 L
.58595 .47307 L
.58904 .49504 L
.59042 .50755 L
.59189 .52348 L
.59316 .54001 L
.59453 .56135 L
.59494 .5685 L
.59537 .57643 L
.59615 .55497 L
.59688 .50078 L
.59767 .48737 L
.59838 .4792 L
.59914 .47236 L
.60052 .46282 L
.60213 .45427 L
.60384 .44706 L
.60691 .43701 L
.61021 .4287 L
.61385 .42139 L
Mistroke
.62036 .41132 L
.62647 .40405 L
.62698 .40351 L
Mfstroke
0 g
.03016 .23543 m
.03651 .23547 L
.04286 .23553 L
.04921 .23562 L
.05556 .23574 L
.0619 .23588 L
.06825 .23606 L
.0746 .23626 L
.08095 .23649 L
.0873 .23672 L
.09365 .23699 L
.1 .23729 L
.10635 .23762 L
.1127 .23798 L
.11905 .23836 L
.1254 .23877 L
.13175 .23921 L
.1381 .23967 L
.14444 .24016 L
.15079 .24068 L
.15714 .24122 L
.16349 .24181 L
.16984 .2424 L
.17619 .24303 L
.18254 .24371 L
.18889 .24441 L
.19524 .24514 L
.20159 .24588 L
.20794 .24668 L
.21429 .2475 L
.22063 .24835 L
.22698 .24923 L
.23333 .25015 L
.23968 .2511 L
.24603 .25209 L
.25238 .25311 L
.25873 .25417 L
.26508 .25526 L
.27143 .25639 L
.27778 .25756 L
.28413 .25682 L
.29048 .26002 L
.29683 .26134 L
.30317 .26262 L
.30952 .26401 L
.31587 .26542 L
.32222 .2669 L
.32857 .26844 L
.33492 .26993 L
.34127 .27157 L
Mistroke
.34762 .2755 L
.35397 .27506 L
.37302 .28046 L
.37937 .27895 L
.38571 .2844 L
.39206 .28649 L
.39841 .28865 L
.40476 .29088 L
.41111 .29319 L
.41746 .29141 L
.42381 .29808 L
.43016 .30102 L
.43651 .30333 L
.44286 .30612 L
.44921 .30902 L
.45556 .31206 L
.4619 .31521 L
.46825 .31849 L
.4746 .32195 L
.48095 .32557 L
.4873 .32938 L
.49365 .33339 L
.5 .33764 L
.50635 .34215 L
.5127 .34695 L
.51905 .35208 L
.5254 .35759 L
.53175 .36355 L
.5381 .37004 L
.54444 .37717 L
.55079 .38508 L
.55714 .39395 L
.56349 .40404 L
.56984 .41569 L
.57619 .42931 L
.58254 .44509 L
.58889 .46143 L
.59524 .46746 L
.60159 .45093 L
.60794 .43405 L
.61429 .42195 L
.62063 .4129 L
.62698 .40579 L
.63333 .39998 L
.63968 .39512 L
.64603 .39095 L
.65238 .38733 L
.65873 .38414 L
.66508 .3813 L
.67143 .37875 L
Mistroke
.67778 .37644 L
.68413 .37435 L
.69048 .37243 L
.69683 .37066 L
.70317 .36904 L
.70952 .36752 L
.71587 .36612 L
.72222 .36481 L
.72857 .36358 L
.73492 .36243 L
.74127 .36135 L
.74762 .36033 L
.75397 .35937 L
.76032 .35846 L
.76667 .3576 L
.77302 .35679 L
.77937 .35601 L
.78571 .35527 L
.79206 .35457 L
.79841 .3539 L
.80476 .35326 L
.81111 .35265 L
.81746 .35206 L
.82381 .3515 L
.83016 .35096 L
.83651 .35045 L
.84286 .34995 L
.84921 .34947 L
.85556 .34901 L
.8619 .34857 L
.86825 .34815 L
.8746 .34774 L
.88095 .34734 L
.8873 .34696 L
.89365 .34659 L
.9 .34624 L
.90635 .34589 L
.9127 .34556 L
.91905 .34524 L
.9254 .34493 L
.93175 .34462 L
.9381 .34433 L
.94444 .34405 L
.95079 .34377 L
.95714 .3435 L
.96349 .34325 L
.96984 .34299 L
.97619 .34275 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`03003o
oooo04goo`03n03ooooo03coo`03003ooooo06goo`008Ooo00<00?ooool0COoo00?h0?ooool0??oo
00<00?ooool0KOoo000Qool2001>ool00oP0ooooo`0lool00`00ooooo`1]ool0027oo`03003ooooo
04goo`03n03ooooo03coo`03003ooooo06goo`008Ooo00<00?ooool0COoo00?h0?ooool0??oo00<0
0?ooool0KOoo000Qool00`00ooooo`1=ool00oP0ooooo`0lool00`00ooooo`1]ool0027oo`03003o
oooo04koo`03n03ooooo03_oo`03003ooooo06goo`008Ooo0P00Cooo00?h0?ooool0>ooo00<00?oo
ool0KOoo000Qool00`00ooooo`1>ool00oP0ooooo`0kool00`00ooooo`1]ool0027oo`03003ooooo
04koo`03n03ooooo03_oo`03003ooooo06goo`008Ooo00<00?ooool0C_oo00?h0?ooool0>ooo00<0
0?ooool0KOoo000Qool2001?ool00oP0ooooo`0kool00`00ooooo`1]ool0027oo`03003ooooo04ko
o`03n03ooooo03_oo`03003ooooo06goo`002?oo0P001?oo00<00?ooool00ooo0P001?oo00<00?oo
ool01?oo00<00?ooool0C_oo00?h0?ooool0>ooo00<00?ooool0KOoo0007ool01000ooooo`002?oo
00@00?ooool000?oo`03003ooooo00Coo`03003ooooo04koo`03n03ooooo03coo`03003ooooo06co
o`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`03ool2001?ool00oP0ooooo`0l
ool00`00ooooo`1/ool000Ooo`04003ooooo0009ool20005ool00`00ooooo`03ool00`00ooooo`1>
ool00oP0ooooo`0lool00`00ooooo`1/ool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<0
0?ooool00_oo00<00?ooool0C_oo00?h0?ooool0??oo00<00?ooool0K?oo0007ool01000ooooo`00
2?oo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`1>ool00oP0ooooo`0lool00`00oooo
o`1/ool000Soo`8000[oo`8000?oo`@000Coo`03003ooooo04koo`03n03ooooo03coo`03003ooooo
06coo`008Ooo0P00Cooo00?h0?ooool0??oo00<00?ooool0K?oo000Qool00`00ooooo`1>ool00oP0
ooooo`0lool00`00ooooo`1/ool0027oo`03003ooooo04koo`03n03ooooo03coo`03003ooooo06co
o`008Ooo00<00?ooool0C_oo00?h0?ooool0??oo00<00?ooool0K?oo000Qool2001?ool00oP0oooo
o`0lool00`00ooooo`1/ool0027oo`03003ooooo04koo`03n03ooooo03coo`03003ooooo06coo`00
8Ooo00<00?ooool0C_oo00?h0?ooool0??oo00<00?ooool0K?oo000Qool00`00ooooo`1?ool00oP0
ooooo`0kool00`00ooooo`1/ool0027oo`80053oo`03n03ooooo03_oo`03003ooooo06coo`008Ooo
00<00?ooool0Cooo00?h0?ooool0>ooo00<00?ooool0K?oo000Qool00`00ooooo`1?ool00oP0oooo
o`0kool00`00ooooo`1/ool0027oo`03003ooooo04ooo`03n03ooooo03_oo`03003ooooo06coo`00
8Ooo0P00D?oo00?h0?ooool0>ooo00<00?ooool0K?oo000Qool00`00ooooo`1?ool00oP0ooooo`0k
ool00`00ooooo`1/ool0027oo`03003ooooo04ooo`03n03ooooo03_oo`03003ooooo06coo`000_oo
0P001?oo00<00?ooool00ooo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0Cooo00?h0?oo
ool0>ooo00<00?ooool0K?oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo00Go
o`03003ooooo00;oo`03003ooooo04ooo`03n03ooooo03_oo`03003ooooo06coo`0000Goo`00oooo
o`0000Soo`04003ooooo0004ool00`00ooooo`04ool00`00ooooo`02ool2001@ool00oP0ooooo`0k
ool00`00ooooo`1/ool00005ool00?ooool00009ool20005ool01000oooooooo0`001Ooo00<00?oo
ool0Cooo00?h0?ooool0>ooo00<00?ooool0K?oo00001Ooo003ooooo00002?oo00@00?ooool000Go
o`04003ooooo0007ool00`00ooooo`1?ool00oP0ooooo`0kool00`00ooooo`1/ool00005ool00?oo
ool00008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo04ooo`03
n03ooooo03coo`03003ooooo06_oo`000_oo0P002_oo0P000ooo10000_oo10001?oo0P00D?oo00?h
0?ooool0??oo00<00?ooool0Jooo000Qool00`00ooooo`1?ool00oP0ooooo`0lool00`00ooooo`1[
ool0027oo`03003ooooo04ooo`03n03ooooo03coo`03003ooooo06_oo`008Ooo00<00?ooool0Cooo
00?h0?ooool0??oo00<00?ooool0Jooo000Qool2001@ool00oP0ooooo`0lool00`00ooooo`1[ool0
027oo`03003ooooo04ooo`03n03ooooo03coo`03003ooooo06_oo`008Ooo00<00?ooool0Cooo00?h
0?ooool0??oo00<00?ooool0Jooo000Qool00`00ooooo`1?ool00oP0ooooo`0lool00`00ooooo`1[
ool0027oo`03003ooooo04ooo`03n03ooooo03coo`03003ooooo06_oo`008Ooo0P00DOoo00?h0?oo
ool0>ooo00<00?ooool0Jooo000Qool00`00ooooo`1@ool00oP0ooooo`0kool00`00ooooo`1[ool0
027oo`03003ooooo053oo`03n03ooooo03_oo`03003ooooo06_oo`008Ooo00<00?ooool0D?oo00?h
0?ooool0>ooo00<00?ooool0Jooo000Qool2001Aool00oP0ooooo`0kool00`00ooooo`1[ool0027o
o`03003ooooo053oo`03n03ooooo03_oo`03003ooooo06_oo`002?oo0P001?oo00<00?ooool00ooo
0P001?oo0P001Ooo00<00?ooool0D?oo00?h0?ooool0>ooo00<00?ooool0Jooo0007ool01000oooo
o`002?oo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`1@ool00oP0ooooo`0kool00`00
ooooo`1[ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000Coo`80057oo`03
n03ooooo03_oo`03003ooooo06_oo`001ooo00@00?ooool000Woo`8000Coo`8000Goo`03003ooooo
053oo`03n03ooooo03_oo`03003ooooo06_oo`001ooo00@00?ooool000Soo`04003ooooo0002ool0
1000ooooo`001?oo00<00?ooool0D?oo00?h0?ooool0>ooo00<00?ooool0Jooo0007ool01000oooo
o`002?oo00@00?ooool000;oo`04003ooooo0004oolF00021n0kool00oP0ooooo`0kool00`00oooo
o`1[ool000Soo`8000[oo`8000Coo`8000Goo`8001Coo`d00087h2koo`03n03ooooo03_oo`03003o
oooo06_oo`008Ooo00<00?ooool08?oo2P000POP9?oo00?h0?ooool0>ooo00<00?ooool0Jooo000Q
ool00`00ooooo`0Zool9000Mool00oP0ooooo`0kool00`00ooooo`1[ool0027oo`03003ooooo03?o
o`H000031n3ooooo01Coo`03n03ooooo03_oo`03003ooooo06_oo`008Ooo00<00?ooool0>Ooo1@00
0POP00<00?ooool03_oo00?h0?ooool0>_oo00<00?ooool0Jooo000Qool2000oool200000ooo0000
00041n0:ool00oP0ooooo`0kool00`00ooooo`1Zool0027oo`03003ooooo04?oo`D000<7h0Koo`03
n03ooooo03_oo`03003ooooo06[oo`008Ooo00<00?ooool0B?oo1@0000D7h?ooooooooP003goo`03
003ooooo06[oo`008Ooo00<00?ooool0COoo0`0000CoooP0n03h03_oo`03003ooooo06[oo`008Ooo
0P00DOoo1@000oP0=ooo00<00?ooool0J_oo000Qool00`00ooooo`1Eool200001?oon0000?P0=?oo
00<00?ooool0J_oo000Qool00`00ooooo`1Gool200000ooo003h0002n00aool00`00ooooo`1Zool0
027oo`03003ooooo05_oo`800004oooh0?P0n00^ool00`00ooooo`1Zool0027oo`8005koo`@00004
n0000000n00Zool00`00ooooo`1Zool0027oo`03003ooooo067oo`04003ooooo0002n00Xool00`00
ooooo`1Zool0027oo`03003ooooo06Goo`04003oooP0n00Vool00`00ooooo`1Zool000;oo`8000Co
o`03003ooooo00?oo`8000Coo`8000?oo`<000Goo`03003ooooo06Koo`03003ooooo00;h02Coo`03
003ooooo06[oo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001Ooo00<00?oo
ool00_oo00<00?ooool0Iooo10000_P08_oo00<00?ooool0J_oo00001Ooo003ooooo00002?oo00@0
0?ooool000;oo`04003ooooo0005ool00`00ooooo`02ool2001/ool01000oooh0?P08?oo00<00?oo
ool0J_oo00001Ooo003ooooo00002Ooo0P001?oo0P000ooo0`001Ooo00<00?ooool0K?oo0P0000?o
ooP0n0007_oo00<00?ooool0J_oo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo
0002ool00`00ooooo`05ool00`00ooooo`1^ool30002n00Lool00`00ooooo`1Zool00005ool00?oo
ool00008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo077oo`80
00;h01[oo`03003ooooo06[oo`000_oo0P002_oo0P001?oo0P000ooo10001?oo0P00M?oo0P000_P0
6?oo00<00?ooool0J_oo000Qool00`00ooooo`1eool01000oooh0?P05_oo00<00?ooool0J_oo000Q
ool00`00ooooo`1fool200000ooon03oo`0Dool00`00ooooo`1Zool0027oo`03003ooooo07Soo`80
0003n03ooooo01;oo`03003ooooo06[oo`008Ooo0P00N_oo00<00?oon0004ooo00<00?ooool0J_oo
000Qool00`00ooooo`1jool01000oooh0?P04Ooo00<00?ooool0Dooo40001ooo000Qool00`00oooo
o`1kool200000ooon03oo`0?ool00`00ooooo`12oolA000Gool0027oo`03003ooooo07goo`03003o
ooP000ooo`03003ooooo03Ooo`/002Soo`008Ooo00<00?ooool0O_oo00<00?oon0003_oo00<00?oo
ool0;Ooo2P00<ooo000Qool20020ool00`00oooh000=ool00`00ooooo`0Vool7000mool0027oo`03
003ooooo083oo`800003n03ooooo00_oo`03003ooooo023oo`D004Coo`008Ooo00<00?ooool0POoo
00<00?oon0003?oo00<00?ooool06ooo1@00BOoo000Qool00`00ooooo`22ool00`00oooh000;ool0
0`00ooooo`0Hool3001>ool0027oo`8008Coo`03003h0?oo00[oo`03003ooooo01Goo`<0057oo`00
8Ooo00<00?ooool0Q?oo00<00?P0ool02Ooo00<00?ooool04Ooo1000E?oo0008ool20004ool00`00
ooooo`03ool20003ool30005ool00`00ooooo`25ool00`00n03oo`08ool00`00ooooo`0@ool00`00
ooooo`1Fool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool0
Q_oo00<00?P0ool01ooo00<00?ooool03?oo1000FOoo0007ool01000ooooo`002?oo00@00?ooool0
00Goo`03003ooooo00;oo`8008Ooo`03003h0?oo00Ooo`03003ooooo00_oo`03003ooooo05_oo`00
1ooo00@00?ooool000Woo`8000Coo`<000Coo`03003ooooo08Ooo`03003h0?oo00Koo`03003ooooo
00[oo`03003ooooo05coo`001ooo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`001?oo
00<00?ooool0R?oo00<00?ooool01Ooo00<00?ooool02Ooo00<00?ooool0GOoo0007ool01000oooo
o`002?oo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`28ool00`00n03oo`05ool00`00
ooooo`08ool00`00ooooo`1Nool000Soo`8000[oo`8000Coo`8000Goo`8008[oo`03003ooooo00Co
o`03003ooooo00Koo`03n0000?oo063oo`008Ooo00<00?ooool0R_oo00<00?ooool00ooo00<00?oo
ool01_oo00<00?ooool0H?oo000Qool00`00ooooo`2:ool00`00ooooo`03ool00`00ooooo`05ool0
0`00ooooo`1Qool0027oo`03003ooooo08_oo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo
06;oo`008Ooo00<00?ooool0Rooo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool0Hooo000Q
ool2002<ool00oP0003oo`02ool00`00ooooo`03ool00`00ooooo`1Sool0027oo`03003ooooo08co
o`05003oooooool00004ool00oP0003oo`1Tool0027oo`03003ooooo08coo`05003oooooool00004
ool00`00ooooo`1Tool0027oo`03003ooooo08coo`05n0000?ooool00003ool00oP0003oo`1Uool0
027oo`8008koo`04003ooooo0003ool00oP0003oo`1Uool0027oo`03003ooooo08goo`04n0000?oo
0003ool00`00ooooo`1Uool0027oo`03003ooooo08goo`04n0000?oo0003ool00`00ooooo`1Uool0
027oo`03003ooooo08koo`03n000000000;oo`03003h0?oo06Koo`008Ooo0P00Sooo00?h00000000
0_oo00<00?ooool0I_oo000Qool00`00ooooo`2>ool01_P0ool00?oo003h06Soo`008Ooo00<00?oo
ool0S_oo00?h0?oo00000P0000?h0?ooool0I_oo0002ool20004ool00`00ooooo`03ool20003ool3
0003ool30005ool00`00ooooo`2?ool01OP0ool00?oon000J?oo00001Ooo003ooooo00002?oo00@0
0?ooool000Goo`03003ooooo00?oo`03003ooooo00;oo`03003ooooo08ooo`04n03oo`00n01Yool0
0005ool00?ooool00008ool01000ooooo`001Ooo00<00?ooool00ooo00<00?ooool00_oo0P00T?oo
00Ch0?oo003h06Woo`0000Goo`00ooooo`0000Woo`8000Coo`<000;oo`<000Goo`03003ooooo08oo
o`04n03oo`00n01Yool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000;oo`03
003ooooo00Goo`03003ooooo08ooo`04n03oo`00n01Yool00005ool00?ooool00008ool01000oooo
o`000_oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo093oo`03n0000?P006Woo`000_oo
0P002_oo0P001?oo0P000ooo10001?oo0P00TOoo00?h0000n000JOoo000Qool00`00ooooo`2@ool0
0oP0003h001Yool0027oo`03003ooooo093oo`03n0000?P006Woo`008Ooo00<00?ooool0T?oo00?h
0000n000JOoo000Qool2002Aool00oP0003h001Yool0027oo`03003ooooo093oo`03n0000?P006Wo
o`008Ooo00<00?ooool0T?oo00?h0000n000JOoo000Qool00`00ooooo`0Gool20004ool00`00oooo
o`02ool4000Bool20004ool00`00ooooo`04ool00`00ooooo`0Aool20004ool00`00ooooo`03ool2
000Cool20004ool00`00ooooo`03ool20008ool00oP0003h000=ool4000Gool40003ool00`00oooo
o`02ool4000Aool40003ool00`00ooooo`04ool00`00ooooo`0?ool0027oo`8001Ooo`04003ooooo
0008ool00`00ooooo`0Bool01000ooooo`002_oo00<00?ooool04?oo00@00?ooool000Soo`04003o
oooo000Aool01000ooooo`002?oo00@00?ooool000Ooo`03n0000?P000ooo`03003ooooo01Soo`03
003ooooo00Ooo`03003ooooo01Coo`03003ooooo00Woo`03003ooooo00ooo`008Ooo00<00?ooool0
5_oo00@00?ooool000Woo`03003ooooo017oo`04003ooooo0007ool5000Aool01000ooooo`002?oo
00@00?ooool0017oo`04003ooooo0008ool01000ooooo`001ooo00?h0000n0003ooo00<00?ooool0
6?oo00<00?ooool02?oo00<00?ooool04ooo00<00?ooool01_oo1@004?oo000Qool00`00ooooo`0F
ool01000ooooo`002_oo00<00?ooool04?oo00@00?ooool000Ooo`04003ooooo000Bool01000oooo
o`002?oo0`004_oo00@00?ooool000Woo`8000Soo`03n0000?P000ooo`03003ooooo01Soo`03003o
oooo00Woo`03003ooooo01;oo`03003ooooo00Koo`04003ooooo000Aool0027oo`03003ooooo01Ko
o`04003ooooo000;ool00`00ooooo`0?ool01000ooooo`002?oo00<00?oo00004_oo00@00?ooool0
00Soo`03003ooooo01;oo`04003ooooo0008ool01000ooooo`001ooo00?h0000n0003ooo00<00?oo
ool06?oo00<00?ooool02_oo00<00?ooool04Ooo00<00?ooool01ooo00<00?oo00004Ooo000Qool0
0`00ooooo`0Fool01000ooooo`002?oo00@00?ooool0017oo`04003ooooo0009ool2000Bool01000
ooooo`002?oo00<00?ooool04_oo00@00?ooool000Soo`04003ooooo0008ool2n00=ool3000Hool3
0009ool01000ooooo`004Ooo0`002_oo0P004Ooo000Qool2000Hool2000:ool2000Cool2000;ool0
0`00ooooo`0Aool2000:ool3000Bool2000:ool20009ool2n00?ool00`00ooooo`0Hool00`00oooo
o`08ool2000Dool00`00ooooo`09ool00`00ooooo`0?ool0027oo`03003ooooo097oo`;h06Woo`00
8Ooo00<00?ooool0TOoo0_P0JOoo000Qool00`00ooooo`2Aool2n01Yool001_ooiX000;h06T00000
8Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool0
1Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool0
1Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool0
1_oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo0_P01ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<0
0?ooool01Ooo000Qool00`00ooooo`0Nool00`00ooooo`0Nool00`00ooooo`0Nool00`00ooooo`0N
ool00`00ooooo`0=ool2n00?ool00`00ooooo`0Nool00`00ooooo`0Nool00`00ooooo`0Eool0027o
o`03003ooooo097oo`;h06Woo`008Ooo0P00T_oo00?h0000ool0J?oo000Qool00`00ooooo`2Aool0
0oP0003oo`1Xool0027oo`03003ooooo097oo`03n0000?oo06Soo`008Ooo00<00?ooool0T_oo00<0
0?ooool0Iooo000Qool00`00ooooo`2Bool00`00ooooo`1Wool0027oo`8009?oo`03003ooooo06Oo
o`008Ooo00<00?ooool0T_oo00<00?ooool0Iooo000Qool00`00ooooo`2Bool00`00ooooo`1Wool0
027oo`03003ooooo09;oo`03003ooooo06Ooo`008Ooo0P00Tooo00<00?ooool0Iooo000Qool00`00
ooooo`2Bool00`00ooooo`1Wool0027oo`03003ooooo09;oo`03003ooooo06Ooo`008Ooo00<00?oo
ool0T_oo00<00?ooool0Iooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.201469, 0.86447, \
0.00605915, 0.000233084}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[k], mGamma /. strangesol[k, 10^4]}, {k, 0.01, 1.5, 
        0.01}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(120\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{0.01`, \(-0.00020255205352157177`\)}, {0.02`, \
\(-0.00040515282305279566`\)}, {0.03`, \(-0.0006077187099958084`\)}, {0.04`, \
\(-0.0008102715318867183`\)}, {0.05`, \(-0.0010128063792692835`\)}, {0.06`, \
\(-0.0012153178465499498`\)}, {0.07`, \(-0.0014177997671485536`\)}, {0.08`, \
\(-0.0016202448367483124`\)}, {0.09`, \(-0.0018226441788508144`\)}, {0.1`, \
\(-0.002025075697407818`\)}, {0.11`, \(-0.00222741419929459`\)}, {0.12`, \
\(-0.002429707103618965`\)}, {0.13`, \(-0.002631944927247204`\)}, {0.14`, \
\(-0.00283412627200009`\)}, {0.15`, \(-0.0030362480692298476`\)}, {0.16`, \
\(-0.00323830694476008`\)}, {0.17`, \(-0.0034402860649563848`\)}, {0.18`, \
\(-0.003642208457765416`\)}, {0.19`, \(-0.003844051060655462`\)}, {0.2`, \
\(-0.004045805469342372`\)}, {0.21`, \(-0.0042474133121606795`\)}, {0.22`, \
\(-0.004449034652256983`\)}, {0.23`, \(-0.004650427139100105`\)}, {0.24`, \
\(-0.004851584083002242`\)}, {0.25`, \(-0.005053034007555`\)}, {0.26`, \
\(-0.005254143418568381`\)}, {0.27`, \(-0.005455213583140289`\)}, {0.28`, \
\(-0.005655578482643048`\)}, {0.29`, \(-0.005856682616181318`\)}, {0.3`, \
\(-0.0060572744564295665`\)}, {0.31`, \(-0.006257601321147433`\)}, {0.32`, \
\(-0.006457920856089998`\)}, {0.33`, \(-0.006657988244528324`\)}, {0.34`, \
\(-0.006857777523789394`\)}, {0.35000000000000003`, \
\(-0.007057359843749808`\)}, {0.36`, \(-0.007257205372104027`\)}, {0.37`, \
\(-0.0074564775427995444`\)}, {0.38`, \(-0.0076556281170799215`\)}, {0.39`, \
\(-0.007854494547887522`\)}, {0.4`, \(-0.0080535904443963`\)}, \
{0.41000000000000003`, 
        3.404218891400317`*^-7}, {0.42`, \(-0.008449716429201453`\)}, {0.43`, \
\(-0.0086467976594371`\)}, {0.44`, \(-0.008844158877881463`\)}, {0.45`, \
\(-0.009042541239405599`\)}, {0.46`, \(-0.009239314946541856`\)}, \
{0.47000000000000003`, \(-0.009437295188565007`\)}, {0.48`, \
\(-0.009628948374164878`\)}, {0.49`, \(-0.009830953345921616`\)}, {0.5`, \
\(-0.01001877302967988`\)}, {0.51`, 
        5.266476313954072`*^-6}, {0.52`, \(-0.010416724752222844`\)}, {0.53`, \
\(-0.00813183960864896`\)}, {0.54`, \(-0.4105237970220052`\)}, {0.55`, \
\(-0.010996858886354608`\)}, {0.56`, 
        1.9553195270358133`*^-6}, {0.5700000000000001`, \
\(-0.011380475924762368`\)}, {0.58`, \(-0.011571531743165815`\)}, {0.59`, \
\(-0.0117626075964188`\)}, {0.6`, \(-0.011952712011078344`\)}, {0.61`, \
\(-0.012142037650705501`\)}, {0.62`, 
        6.367152908746407`*^-10}, {0.63`, \(-0.012518379628401433`\)}, \
{0.64`, \(-0.003963474043225118`\)}, {0.65`, \(-0.012890205811521527`\)}, \
{0.66`, \(-0.013074479425264343`\)}, {0.67`, \(-0.013257445030557055`\)}, \
{0.68`, \(-0.0134392884735839`\)}, {0.6900000000000001`, \
\(-0.01361897214891989`\)}, {0.7000000000000001`, \
\(-0.013796924335800112`\)}, {0.71`, \(-0.013972712826410954`\)}, {0.72`, \
\(-0.014146693781551718`\)}, {0.73`, \(-0.01431708923229897`\)}, {0.74`, \
\(-0.014484547466805386`\)}, {0.75`, \(-0.014648488564418484`\)}, {0.76`, \
\(-0.014807976104283947`\)}, {0.77`, \(-0.014962120918805976`\)}, {0.78`, \
\(-0.015109777771287155`\)}, {0.79`, \(-0.015249366156605501`\)}, {0.8`, \
\(-0.015378724616237027`\)}, {0.81`, \(-0.015494887379103922`\)}, \
{0.8200000000000001`, \(-0.015593161042373793`\)}, {0.8300000000000001`, \
\(-0.015667004847574318`\)}, {0.84`, \(-0.01570569135376953`\)}, {0.85`, \
\(-0.01569131491730606`\)}, {0.86`, \(-0.015591927800712583`\)}, {0.87`, \
\(-0.015346162712489678`\)}, {0.88`, \(-0.014827349462421043`\)}, {0.89`, \
\(-0.013771327458082173`\)}, {0.9`, \(-0.011902009344287265`\)}, {0.91`, \
\(-0.01043703542364441`\)}, {0.92`, \(-0.009960476608155876`\)}, {0.93`, \
\(-0.009812400552054186`\)}, {0.9400000000000001`, \
\(-0.009778631565907057`\)}, {0.9500000000000001`, \
\(-0.009795924283805964`\)}, {0.96`, \(-0.009840131369900269`\)}, {0.97`, \
\(-0.009900174642374315`\)}, {0.98`, \(-0.009970286562825496`\)}, {0.99`, \
\(-0.010047178134419578`\)}, {1.`, \(-0.01012884082158027`\)}, {1.01`, \
\(-0.010213980853135326`\)}, {1.02`, \(-0.010301728506386696`\)}, {1.03`, \
\(-0.010391478314764597`\)}, {1.04`, \(-0.010483014788410667`\)}, {1.05`, \
\(-0.010575550415991489`\)}, {1.06`, \(-0.010669101278809616`\)}, {1.07`, \
\(-0.010763483575614154`\)}, {1.08`, \(-0.010858554505523524`\)}, {1.09`, \
\(-0.010954201538382984`\)}, {1.1`, \(-0.011050334872267881`\)}, {1.11`, \
\(-0.011146882028397356`\)}, {1.12`, \(-0.01124378391053936`\)}, \
{1.1300000000000001`, \(-0.011340991887305267`\)}, {1.1400000000000001`, \
\(-0.011438465601368328`\)}, {1.1500000000000001`, \
\(-0.011536171303450707`\)}, {1.16`, \(-0.011634080570580857`\)}, {1.17`, \
\(-0.011732169309447578`\)}, {1.18`, \(-0.011830416973809502`\)}, {1.19`, \
\(-0.011928805944413185`\)}, {1.2`, \(-0.012027321033545596`\)}, {1.21`, \
\(-0.01212594908608067`\)}, {1.22`, \(-0.01222467865589679`\)}, {1.23`, \
\(-0.01232349974164417`\)}, {1.24`, \(-0.012422403569609983`\)}, {1.25`, \
\(-0.012521382414224952`\)}, {1.26`, \(-0.01262042944885614`\)}, {1.27`, \
\(-0.012719538621124458`\)}, {1.28`, \(-0.012818704548197835`\)}, {1.29`, \
\(-0.012917922428447992`\)}, {1.3`, \(-0.013017187966582478`\)}, {1.31`, \
\(-0.013116497309930142`\)}, {1.32`, \(-0.013215846994001162`\)}, {1.33`, \
\(-0.013315233895796436`\)}, {1.34`, \(-0.013414655193617106`\)}, {1.35`, \
\(-0.013514108332351782`\)}, {1.36`, \(-0.013613590993394773`\)}, {1.37`, \
\(-0.013713101068497428`\)}, {1.3800000000000001`, \
\(-0.013812636636969024`\)}, {1.3900000000000001`, \
\(-0.013912195945741829`\)}, {1.4000000000000001`, \
\(-0.014011777391892058`\)}, {1.41`, \(-0.01411137950727422`\)}, {1.42`, \
\(-0.014211000944979043`\)}, {1.43`, \(-0.014310640467369226`\)}, {1.44`, \
\(-0.01441029693548498`\)}, {1.45`, \(-0.014509969299641367`\)}, {1.46`, \
\(-0.014609656591065084`\)}, {1.47`, \(-0.014709357914440252`\)}, {1.48`, \
\(-0.014809072441251257`\)}, {1.49`, \(-0.014908799403826327`\)}, {1.5`, \
\(-0.015008538089997067`\)}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \({{0.01`, \(-0.00020255205352157177`\)}, {0.02`, \
\(-0.00040515282305279566`\)}, {0.03`, \(-0.0006077187099958084`\)}, {0.04`, \
\(-0.0008102715318867183`\)}, {0.05`, \(-0.0010128063792692835`\)}, {0.06`, \
\(-0.0012153178465499498`\)}, {0.07`, \(-0.0014177997671485536`\)}, {0.08`, \
\(-0.0016202448367483124`\)}, {0.09`, \(-0.0018226441788508144`\)}, {0.1`, \
\(-0.002025075697407818`\)}, {0.11`, \(-0.00222741419929459`\)}, {0.12`, \
\(-0.002429707103618965`\)}, {0.13`, \(-0.002631944927247204`\)}, {0.14`, \
\(-0.00283412627200009`\)}, {0.15`, \(-0.0030362480692298476`\)}, {0.16`, \
\(-0.00323830694476008`\)}, {0.17`, \(-0.0034402860649563848`\)}, {0.18`, \
\(-0.003642208457765416`\)}, {0.19`, \(-0.003844051060655462`\)}, {0.2`, \
\(-0.004045805469342372`\)}, {0.21`, \(-0.0042474133121606795`\)}, {0.22`, \
\(-0.004449034652256983`\)}, {0.23`, \(-0.004650427139100105`\)}, {0.24`, \
\(-0.004851584083002242`\)}, {0.25`, \(-0.005053034007555`\)}, {0.26`, \
\(-0.005254143418568381`\)}, {0.27`, \(-0.005455213583140289`\)}, {0.28`, \
\(-0.005655578482643048`\)}, {0.29`, \(-0.005856682616181318`\)}, {0.3`, \
\(-0.0060572744564295665`\)}, {0.31`, \(-0.006257601321147433`\)}, {0.32`, \
\(-0.006457920856089998`\)}, {0.33`, \(-0.006657988244528324`\)}, {0.34`, \
\(-0.006857777523789394`\)}, {0.35000000000000003`, \
\(-0.007057359843749808`\)}, {0.36`, \(-0.007257205372104027`\)}, {0.37`, \
\(-0.0074564775427995444`\)}, {0.38`, \(-0.0076556281170799215`\)}, {0.39`, \
\(-0.007854494547887522`\)}, {0.4`, \(-0.0080535904443963`\)}, \
{0.41000000000000003`, 
        3.404218891400317`*^-7}, {0.42`, \(-0.008449716429201453`\)}, {0.43`, \
\(-0.0086467976594371`\)}, {0.44`, \(-0.008844158877881463`\)}, {0.45`, \
\(-0.009042541239405599`\)}, {0.46`, \(-0.009239314946541856`\)}, \
{0.47000000000000003`, \(-0.009437295188565007`\)}, {0.48`, \
\(-0.009628948374164878`\)}, {0.49`, \(-0.009830953345921616`\)}, {0.5`, \
\(-0.01001877302967988`\)}, {0.51`, 
        5.266476313954072`*^-6}, {0.52`, \(-0.010416724752222844`\)}, {0.55`, \
\(-0.010996858886354608`\)}, {0.56`, 
        1.9553195270358133`*^-6}, {0.5700000000000001`, \
\(-0.011380475924762368`\)}, {0.58`, \(-0.011571531743165815`\)}, {0.59`, \
\(-0.0117626075964188`\)}, {0.6`, \(-0.011952712011078344`\)}, {0.61`, \
\(-0.012142037650705501`\)}, {0.62`, 
        6.367152908746407`*^-10}, {0.63`, \(-0.012518379628401433`\)}, \
{0.64`, \(-0.003963474043225118`\)}, {0.65`, \(-0.012890205811521527`\)}, \
{0.66`, \(-0.013074479425264343`\)}, {0.67`, \(-0.013257445030557055`\)}, \
{0.68`, \(-0.0134392884735839`\)}, {0.6900000000000001`, \
\(-0.01361897214891989`\)}, {0.7000000000000001`, \
\(-0.013796924335800112`\)}, {0.71`, \(-0.013972712826410954`\)}, {0.72`, \
\(-0.014146693781551718`\)}, {0.73`, \(-0.01431708923229897`\)}, {0.74`, \
\(-0.014484547466805386`\)}, {0.75`, \(-0.014648488564418484`\)}, {0.76`, \
\(-0.014807976104283947`\)}, {0.77`, \(-0.014962120918805976`\)}, {0.78`, \
\(-0.015109777771287155`\)}, {0.79`, \(-0.015249366156605501`\)}, {0.8`, \
\(-0.015378724616237027`\)}, {0.81`, \(-0.015494887379103922`\)}, \
{0.8200000000000001`, \(-0.015593161042373793`\)}, {0.8300000000000001`, \
\(-0.015667004847574318`\)}, {0.84`, \(-0.01570569135376953`\)}, {0.85`, \
\(-0.01569131491730606`\)}, {0.86`, \(-0.015591927800712583`\)}, {0.87`, \
\(-0.015346162712489678`\)}, {0.88`, \(-0.014827349462421043`\)}, {0.89`, \
\(-0.013771327458082173`\)}, {0.9`, \(-0.011902009344287265`\)}, {0.91`, \
\(-0.01043703542364441`\)}, {0.92`, \(-0.009960476608155876`\)}, {0.93`, \
\(-0.009812400552054186`\)}, {0.9400000000000001`, \
\(-0.009778631565907057`\)}, {0.9500000000000001`, \
\(-0.009795924283805964`\)}, {0.96`, \(-0.009840131369900269`\)}, {0.97`, \
\(-0.009900174642374315`\)}, {0.98`, \(-0.009970286562825496`\)}, {0.99`, \
\(-0.010047178134419578`\)}, {1.`, \(-0.01012884082158027`\)}, {1.01`, \
\(-0.010213980853135326`\)}, {1.02`, \(-0.010301728506386696`\)}, {1.03`, \
\(-0.010391478314764597`\)}, {1.04`, \(-0.010483014788410667`\)}, {1.05`, \
\(-0.010575550415991489`\)}, {1.06`, \(-0.010669101278809616`\)}, {1.07`, \
\(-0.010763483575614154`\)}, {1.08`, \(-0.010858554505523524`\)}, {1.09`, \
\(-0.010954201538382984`\)}, {1.1`, \(-0.011050334872267881`\)}, {1.11`, \
\(-0.011146882028397356`\)}, {1.12`, \(-0.01124378391053936`\)}, \
{1.1300000000000001`, \(-0.011340991887305267`\)}, {1.1400000000000001`, \
\(-0.011438465601368328`\)}, {1.1500000000000001`, \
\(-0.011536171303450707`\)}, {1.16`, \(-0.011634080570580857`\)}, {1.17`, \
\(-0.011732169309447578`\)}, {1.18`, \(-0.011830416973809502`\)}, {1.19`, \
\(-0.011928805944413185`\)}, {1.2`, \(-0.012027321033545596`\)}, {1.21`, \
\(-0.01212594908608067`\)}, {1.22`, \(-0.01222467865589679`\)}, {1.23`, \
\(-0.01232349974164417`\)}, {1.24`, \(-0.012422403569609983`\)}, {1.25`, \
\(-0.012521382414224952`\)}, {1.26`, \(-0.01262042944885614`\)}, {1.27`, \
\(-0.012719538621124458`\)}, {1.28`, \(-0.012818704548197835`\)}, {1.29`, \
\(-0.012917922428447992`\)}, {1.3`, \(-0.013017187966582478`\)}, {1.31`, \
\(-0.013116497309930142`\)}, {1.32`, \(-0.013215846994001162`\)}, {1.33`, \
\(-0.013315233895796436`\)}, {1.34`, \(-0.013414655193617106`\)}, {1.35`, \
\(-0.013514108332351782`\)}, {1.36`, \(-0.013613590993394773`\)}, {1.37`, \
\(-0.013713101068497428`\)}, {1.3800000000000001`, \
\(-0.013812636636969024`\)}, {1.3900000000000001`, \
\(-0.013912195945741829`\)}, {1.4000000000000001`, \
\(-0.014011777391892058`\)}, {1.41`, \(-0.01411137950727422`\)}, {1.42`, \
\(-0.014211000944979043`\)}, {1.43`, \(-0.014310640467369226`\)}, {1.44`, \
\(-0.01441029693548498`\)}, {1.45`, \(-0.014509969299641367`\)}, {1.46`, \
\(-0.014609656591065084`\)}, {1.47`, \(-0.014709357914440252`\)}, {1.48`, \
\(-0.014809072441251257`\)}, {1.49`, \(-0.014908799403826327`\)}, {1.5`, \
\(-0.015008538089997067`\)}}\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 0.603211 20.51 [
[.15079 .59071 -9 -9 ]
[.15079 .59071 9 0 ]
[.27778 .59071 -9 -9 ]
[.27778 .59071 9 0 ]
[.40476 .59071 -9 -9 ]
[.40476 .59071 9 0 ]
[.53175 .59071 -9 -9 ]
[.53175 .59071 9 0 ]
[.65873 .59071 -3 -9 ]
[.65873 .59071 3 0 ]
[.78571 .59071 -9 -9 ]
[.78571 .59071 9 0 ]
[.9127 .59071 -9 -9 ]
[.9127 .59071 9 0 ]
[.01131 .09046 -36 -4.5 ]
[.01131 .09046 0 4.5 ]
[.01131 .19301 -30 -4.5 ]
[.01131 .19301 0 4.5 ]
[.01131 .29556 -36 -4.5 ]
[.01131 .29556 0 4.5 ]
[.01131 .39811 -30 -4.5 ]
[.01131 .39811 0 4.5 ]
[.01131 .50066 -36 -4.5 ]
[.01131 .50066 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .60321 m
.15079 .60946 L
s
[(0.2)] .15079 .59071 0 1 Mshowa
.27778 .60321 m
.27778 .60946 L
s
[(0.4)] .27778 .59071 0 1 Mshowa
.40476 .60321 m
.40476 .60946 L
s
[(0.6)] .40476 .59071 0 1 Mshowa
.53175 .60321 m
.53175 .60946 L
s
[(0.8)] .53175 .59071 0 1 Mshowa
.65873 .60321 m
.65873 .60946 L
s
[(1)] .65873 .59071 0 1 Mshowa
.78571 .60321 m
.78571 .60946 L
s
[(1.2)] .78571 .59071 0 1 Mshowa
.9127 .60321 m
.9127 .60946 L
s
[(1.4)] .9127 .59071 0 1 Mshowa
.125 Mabswid
.05556 .60321 m
.05556 .60696 L
s
.0873 .60321 m
.0873 .60696 L
s
.11905 .60321 m
.11905 .60696 L
s
.18254 .60321 m
.18254 .60696 L
s
.21429 .60321 m
.21429 .60696 L
s
.24603 .60321 m
.24603 .60696 L
s
.30952 .60321 m
.30952 .60696 L
s
.34127 .60321 m
.34127 .60696 L
s
.37302 .60321 m
.37302 .60696 L
s
.43651 .60321 m
.43651 .60696 L
s
.46825 .60321 m
.46825 .60696 L
s
.5 .60321 m
.5 .60696 L
s
.56349 .60321 m
.56349 .60696 L
s
.59524 .60321 m
.59524 .60696 L
s
.62698 .60321 m
.62698 .60696 L
s
.69048 .60321 m
.69048 .60696 L
s
.72222 .60321 m
.72222 .60696 L
s
.75397 .60321 m
.75397 .60696 L
s
.81746 .60321 m
.81746 .60696 L
s
.84921 .60321 m
.84921 .60696 L
s
.88095 .60321 m
.88095 .60696 L
s
.94444 .60321 m
.94444 .60696 L
s
.97619 .60321 m
.97619 .60696 L
s
.25 Mabswid
0 .60321 m
1 .60321 L
s
.02381 .09046 m
.03006 .09046 L
s
[(-0.025)] .01131 .09046 1 0 Mshowa
.02381 .19301 m
.03006 .19301 L
s
[(-0.02)] .01131 .19301 1 0 Mshowa
.02381 .29556 m
.03006 .29556 L
s
[(-0.015)] .01131 .29556 1 0 Mshowa
.02381 .39811 m
.03006 .39811 L
s
[(-0.01)] .01131 .39811 1 0 Mshowa
.02381 .50066 m
.03006 .50066 L
s
[(-0.005)] .01131 .50066 1 0 Mshowa
.125 Mabswid
.02381 .11097 m
.02756 .11097 L
s
.02381 .13148 m
.02756 .13148 L
s
.02381 .15199 m
.02756 .15199 L
s
.02381 .1725 m
.02756 .1725 L
s
.02381 .21352 m
.02756 .21352 L
s
.02381 .23403 m
.02756 .23403 L
s
.02381 .25454 m
.02756 .25454 L
s
.02381 .27505 m
.02756 .27505 L
s
.02381 .31607 m
.02756 .31607 L
s
.02381 .33658 m
.02756 .33658 L
s
.02381 .35709 m
.02756 .35709 L
s
.02381 .3776 m
.02756 .3776 L
s
.02381 .41862 m
.02756 .41862 L
s
.02381 .43913 m
.02756 .43913 L
s
.02381 .45964 m
.02756 .45964 L
s
.02381 .48015 m
.02756 .48015 L
s
.02381 .52117 m
.02756 .52117 L
s
.02381 .54168 m
.02756 .54168 L
s
.02381 .56219 m
.02756 .56219 L
s
.02381 .5827 m
.02756 .5827 L
s
.02381 .06995 m
.02756 .06995 L
s
.02381 .04944 m
.02756 .04944 L
s
.02381 .02893 m
.02756 .02893 L
s
.02381 .00842 m
.02756 .00842 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.03016 .59906 m
.03651 .5949 L
.04286 .59075 L
.04921 .58659 L
.05556 .58244 L
.0619 .57828 L
.06825 .57413 L
.0746 .56998 L
.08095 .56583 L
.0873 .56168 L
.09365 .55753 L
.1 .55338 L
.10635 .54923 L
.1127 .54508 L
.11905 .54094 L
.1254 .53679 L
.13175 .53265 L
.1381 .52851 L
.14444 .52437 L
.15079 .52023 L
.15714 .5161 L
.16349 .51196 L
.16984 .50783 L
.17619 .50371 L
.18254 .49957 L
.18889 .49545 L
.19524 .49132 L
.20159 .48722 L
.20794 .48309 L
.21429 .47898 L
.22063 .47487 L
.22698 .47076 L
.23333 .46666 L
.23968 .46256 L
.24603 .45846 L
.25238 .45437 L
.25873 .45028 L
.26508 .44619 L
.27143 .44212 L
.27778 .43803 L
.28413 .60322 L
.29048 .42991 L
.29683 .42587 L
.30317 .42182 L
.30952 .41775 L
.31587 .41371 L
.32222 .40965 L
.32857 .40572 L
.33492 .40158 L
.34127 .39773 L
Mistroke
.34762 .60332 L
.35397 .38956 L
.36032 .43643 L
Mfstroke
.36032 .43643 m
.36065 0 L
s
.37272 0 m
.37302 .37767 L
.37937 .60325 L
.38571 .3698 L
.39206 .36588 L
.39841 .36196 L
.40476 .35806 L
.41111 .35418 L
.41746 .60321 L
.42381 .34646 L
.43016 .52192 L
.43651 .33883 L
.44286 .33505 L
.44921 .3313 L
.45556 .32757 L
.4619 .32389 L
.46825 .32024 L
.4746 .31663 L
.48095 .31306 L
.4873 .30957 L
.49365 .30613 L
.5 .30277 L
.50635 .2995 L
.5127 .29634 L
.51905 .29331 L
.5254 .29045 L
.53175 .28779 L
.5381 .28541 L
.54444 .2834 L
.55079 .28188 L
.55714 .28109 L
.56349 .28138 L
.56984 .28342 L
.57619 .28846 L
.58254 .2991 L
.58889 .32076 L
.59524 .3591 L
.60159 .38915 L
.60794 .39892 L
.61429 .40196 L
.62063 .40265 L
.62698 .4023 L
.63333 .40139 L
.63968 .40016 L
.64603 .39872 L
.65238 .39714 L
.65873 .39547 L
.66508 .39372 L
.67143 .39192 L
.67778 .39008 L
Mistroke
.68413 .3882 L
.69048 .38631 L
.69683 .38439 L
.70317 .38245 L
.70952 .3805 L
.71587 .37854 L
.72222 .37657 L
.72857 .37459 L
.73492 .3726 L
.74127 .37061 L
.74762 .36861 L
.75397 .3666 L
.76032 .3646 L
.76667 .36258 L
.77302 .36057 L
.77937 .35855 L
.78571 .35653 L
.79206 .35451 L
.79841 .35248 L
.80476 .35046 L
.81111 .34843 L
.81746 .3464 L
.82381 .34437 L
.83016 .34233 L
.83651 .3403 L
.84286 .33826 L
.84921 .33623 L
.85556 .33419 L
.8619 .33215 L
.86825 .33012 L
.8746 .32808 L
.88095 .32604 L
.8873 .324 L
.89365 .32196 L
.9 .31991 L
.90635 .31787 L
.9127 .31583 L
.91905 .31379 L
.9254 .31174 L
.93175 .3097 L
.9381 .30766 L
.94444 .30561 L
.95079 .30357 L
.95714 .30152 L
.96349 .29948 L
.96984 .29743 L
.97619 .29539 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000oooo
o`00X?oo000Wool2001Cool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo
000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo
000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool2001Cool01000ooooo`00X?oo000Wool0
0`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool0
0`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool2
001Cool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00oooo
o`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00oooo
o`1Bool01000ooooo`00X?oo000Wool2001Cool01000ooooo`00X?oo000Wool00`00ooooo`1Bool0
1000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool0
1000ooooo`00X?oo0008ool20004ool00`00ooooo`03ool20003ool40002ool30005ool00`00oooo
o`1Bool01000ooooo`00X?oo0007ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo00Ko
o`03003ooooo00;oo`03003ooooo05;oo`04003ooooo002Pool000Ooo`04003ooooo0008ool01000
ooooo`000ooo00<00?ooool01Ooo00<00?ooool00_oo0P00Dooo00@00?ooool00:3oo`0000?oo`00
00000P000_oo00@00?ooool000Soo`04003ooooo0004ool01000oooooooo0`001Ooo00<00?ooool0
D_oo00@00?ooool00:3oo`001ooo00@00?ooool000Soo`04003ooooo0005ool01000ooooo`001ooo
00<00?ooool0D_oo00@00?ooool00:3oo`001ooo00@00?ooool000Soo`04003ooooo0002ool01000
ooooo`000_oo00<00?ooool01Ooo00<00?ooool0D_oo00@00?ooool00:3oo`002?oo0P002_oo0P00
1?oo0P000ooo10001?oo00<00?ooool0D_oo00@00?ooool00:3oo`009ooo0P00Dooo00@00?ooool0
0:3oo`009ooo00<00?ooool0D_oo00@00?ooool00:3oo`009ooo00<00?ooool0D_oo00@00?ooool0
0:3oo`009ooo00<00?ooool0D_oo00@00?ooool00:3oo`009ooo00<00?ooool0D_oo00@00?ooool0
0:3oo`009ooo0P00Dooo00@00?ooool00:3oo`009ooo00<00?ooool0D_oo00@00?ooool00:3oo`00
9ooo00<00?ooool0D_oo00@00?ooool00:3oo`009ooo00<00?ooool0D_oo00@00?ooool00:3oo`00
9ooo00<00?ooool0D_oo00@00?ooool00:3oo`009ooo0P00Dooo00@00?ooool00:3oo`009ooo00<0
0?ooool0D_oo00@00?ooool00:3oo`009ooo00<00?ooool0D_oo00@00?ooool00:3oo`009ooo00<0
0?ooool0D_oo00@00?ooool00:3oo`009ooo00<00?ooool0D_oo00@00?ooool00:3oo`009ooo0P00
Dooo00@00?ooool00:3oo`009ooo00<00?ooool0D_oo00@00?ooool00:3oo`009ooo00<00?ooool0
D_oo00@00?ooool00:3oo`009ooo00<00?ooool0D_oo00@00?ooool00:3oo`003_oo0P001?oo00<0
0?ooool00ooo0P000ooo10001?oo00<00?ooool0D_oo00@00?ooool00:3oo`003Ooo00@00?ooool0
00Soo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`1Bool01000ooooo`00X?oo000=ool0
1000ooooo`002?oo00@00?ooool000?oo`03003ooooo00Coo`8005?oo`04003ooooo002Pool000Oo
o`@000;oo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool00ooo00<00?ooool0D_oo00@0
0?ooool00:3oo`003Ooo00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`02ool00`00oooo
o`1Bool01000ooooo`00X?oo000=ool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0004
ool00`00ooooo`1Bool01000ooooo`00X?oo000>ool2000:ool20004ool20005ool00`00ooooo`1B
ool01000ooooo`00X?oo000Wool2001Cool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000
ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000
ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool2001Cool01000ooooo`00
X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00
X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00
X?oo000Wool2001Cool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000W
ool00`00ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00X?oo000W
ool00`00ooooo`1Bool01000ooooo`00X?oo000Wool2001Cool01000ooooo`00X?oo000Wool00`00
ooooo`1Bool01000ooooo`00X?oo000Wool00`00ooooo`1Bool01000ooooo`00;?oo1@00Kooo000W
ool00`00ooooo`1Bool01000ooooo`00:Ooo0`001Ooo00<00?ooool0K?oo0008ool20004ool00`00
ooooo`03ool20003ool40002ool30005ool00`00ooooo`1Bool01000ooooo`009Ooo10002Ooo0P00
K?oo0007ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo00Coo`03003ooooo00;oo`03
003ooooo05;oo`04003ooooo000Tool00`00ooooo`0=ool00`00ooooo`1Yool000Ooo`04003ooooo
0008ool01000ooooo`001?oo00<00?ooool01?oo00<00?ooool00_oo0P00Dooo00@00?ooool002;o
o`80013oo`03003ooooo05ooo`<000Ooo`0000?oo`0000000P000_oo00@00?ooool000Soo`04003o
oooo0004ool01000oooooooo0`001Ooo00<00?ooool0D_oo00@00?ooool001ooo`<001?oo`03003o
oooo05_oo`<000[oo`001ooo00@00?ooool000Soo`04003ooooo0004ool01@00oooooooo00001ooo
00<00?ooool0D_oo00@00?ooool001goo`8001Koo`03003ooooo05Soo`<000goo`001ooo00@00?oo
ool000Soo`04003ooooo0002ool30003ool00`00ooooo`05ool00`00ooooo`1Bool01000ooooo`00
7?oo00<00?ooool05_oo00<00?ooool0EOoo0`004?oo0008ool2000:ool20005ool01000oooooooo
10001?oo00<00?ooool0D_oo00@00?ooool001[oo`8001[oo`03003ooooo053oo`@001?oo`009ooo
0P00Dooo00@00?ooool001Woo`03003ooooo01[oo`03003ooooo04goo`<001Ooo`009ooo00<00?oo
ool0D_oo00@00?ooool001Ooo`8001goo`03003ooooo04[oo`<001[oo`009ooo00<00?ooool0D_oo
00@00?ooool001Goo`8001ooo`03003ooooo04Ooo`<001goo`009ooo00<00?ooool0D_oo00@00?oo
ool001Coo`03003ooooo01ooo`03003ooooo04?oo`@0023oo`009ooo00<00?ooool0D_oo00@00?oo
ool001;oo`8002?oo`03003ooooo03ooo`<002Coo`009ooo0P00Dooo00@00?ooool0013oo`8002Go
o`03003ooooo03coo`<002Ooo`009ooo00<00?ooool0D_oo00@00?ooool000ooo`03003ooooo02Go
o`03003ooooo03Woo`<002[oo`009ooo00<00?ooool0D_oo00@00?ooool000ooo`03003ooooo02Go
o`03003ooooo03Ooo`8002goo`009ooo00<00?ooool0D_oo00@00?ooool000coo`04003ooooo000W
ool00`00ooooo`0dool3000_ool002Ooo`03003ooooo05;oo`04003ooooo0008ool20002ool01000
ooooo`00:?oo00<00?ooool0<?oo0`00<_oo000Wool2001Cool01000ooooo`001ooo00<00?oo0000
0_oo00@00?ooool002Soo`03003ooooo02coo`@003Goo`009ooo00<00?ooool0D_oo00@00?ooool0
00Goo`8000;oo`04003ooooo0002ool00`00ooooo`0Vool00`00ooooo`0Yool3000iool002Ooo`03
003ooooo05;oo`04003ooooo0004ool00`00ooooo`02ool01000ooooo`000_oo00<00?ooool09_oo
00<00?ooool09_oo0`00??oo000Wool00`00ooooo`1Bool01000ooooo`000ooo00<00?ooool00ooo
00@00?ooool000;oo`03003ooooo02Koo`03003ooooo02?oo`<003ooo`009ooo00<00?ooool0D_oo
00@00?ooool000?oo`03003ooooo00?oo`04003ooooo0002ool00`00ooooo`0Vool00`00ooooo`0P
ool30012ool002Ooo`8005?oo`04003ooooo0003ool00`00ooooo`03ool01000ooooo`000_oo00<0
0?ooool09ooo00<00?ooool06ooo1000AOoo000Wool00`00ooooo`1Bool01000ooooo`000ooo00<0
0?ooool00ooo00@00?ooool000;oo`03003ooooo02Ooo`03003ooooo01Soo`<004Woo`009ooo00<0
0?ooool0D_oo00@00?ooool000?oo`03003ooooo00?oo`04003ooooo0002ool00`00ooooo`0Wool0
0`00ooooo`0Eool3001<ool002Ooo`03003ooooo053oo`03003oo`0000;oo`05003oooooool00005
ool01000ooooo`000_oo00<00?ooool09ooo00<00?ooool04_oo0`00Cooo000>ool20004ool00`00
ooooo`03ool20003ool40004ool00`00ooooo`1@ool00`00ool00002ool01@00oooooooo00001Ooo
00@00?ooool000;oo`03003ooooo02Soo`03003ooooo00goo`@005;oo`003Ooo00@00?ooool000So
o`04003ooooo0004ool00`00ooooo`03ool00`00ooooo`1@ool00`00ool00002ool01@00oooooooo
00001Ooo00<00?ooool00P0000?oo`00ool0:Ooo00<00?ooool02Ooo1000E_oo000=ool01000oooo
o`002?oo00@00?ooool000Coo`03003ooooo00?oo`8004koo`06003ooooo003oo`000_oo00D00?oo
ooooo`0000Goo`03003ooooo00800003ool00?oo02[oo`03003ooooo00?oo`D005[oo`001ooo1000
0_oo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`03ool00`00ooooo`1;ool30002ool3
0002ool01@00oooooooo00001Ooo00<00?ooool00P0000?oo`00ool0:ooo1@00Gooo000=ool01000
ooooo`002?oo00@00?ooool000Coo`03003ooooo00?oo`03003ooooo04Woo`8000;oo`03003ooooo
00<000;oo`05003oooooool00005ool00`00ooooo`0200000ooo003oo`2?ool000goo`04003ooooo
0008ool01000ooooo`000_oo0`001Ooo00<00?ooool0B?oo00<00?ooool00_oo00<00?ooool00`00
0_oo00D00?ooooooo`0000Goo`03003ooooo00800003ool00?oo08ooo`003_oo0P002_oo0P001Ooo
00<00?ooool00ooo00<00?ooool0A_oo0P001Ooo00<00?ooool00`000_oo00D00?ooooooo`0000Go
o`03003ooooo00800003ool00?oo08ooo`009ooo0P00A_oo00<00?ooool01Ooo00<00?ooool00`00
0_oo00D00?ooooooo`0000Koo`06003oo`00003oo`00T?oo000Wool00`00ooooo`13ool20008ool0
0`00ooooo`030002ool01@00oooooooo00001_oo00H00?oo00000?oo002@ool002Ooo`03003ooooo
047oo`8000[oo`06003ooooo003oo`000_oo00D00?ooooooo`0000Koo`06003oo`00003oo`00T?oo
000Wool00`00ooooo`10ool00`00ooooo`0:ool01P00ooooo`00ool000;oo`04003ooooo0007ool0
1P00ool00000ool0093oo`009ooo00<00?ooool0@?oo00<00?ooool02_oo00H00?ooool00?oo0003
ool00`00ool00007ool00`00ool00002002Aool002Ooo`8003koo`04003ooooo000<ool01000oooo
o`001Ooo00<00?oo00001ooo00<00?oo00000P00TOoo000Wool00`00ooooo`0kool30002ool00`00
ooooo`0:ool01000ooooo`001Ooo00<00?oo00001ooo00<00?oo00000P00TOoo000Wool00`00oooo
o`0iool20002ool01000ooooo`003?oo00@00?ooool000Goo`03003oo`0000Ooo`03003oo`000080
097oo`009ooo00<00?ooool0>?oo00<00?ooool00_oo00@00?ooool000coo`04003ooooo0005ool0
0`00ool00007ool00`00ool00002002Aool002Ooo`03003ooooo03Koo`8000Goo`04003ooooo000=
ool00`00ool00005ool00`00ool00007ool00`00ool00002002Aool002Ooo`8003Goo`8000Ooo`04
003ooooo000=ool00`00ool00005ool00`00ool00007ool00`00ool00002002Aool002Ooo`03003o
oooo03?oo`03003ooooo00Ooo`04003ooooo000=ool00`00ool00005ool00`00ool00007ool00`00
ool00002002Aool002Ooo`03003ooooo037oo`8000[oo`04003ooooo000=ool00`00ool00005ool0
0`00ool00007ool00`00ool00002002Aool002Ooo`03003ooooo033oo`03003ooooo00[oo`04003o
oooo000=ool00`00ool00005ool00`00ool00007ool00`00ool00002002Aool002Ooo`03003ooooo
02koo`8000goo`04003ooooo000=ool00`00ool00005ool00`00ool00007ool00`00ool00002002A
ool002Ooo`8002goo`8000ooo`04003ooooo000=ool00`00ool00005ool00`00ool00007ool20000
0ooo0000002Aool002Ooo`03003ooooo02_oo`03003ooooo013oo`03003oo`0000goo`03003oo`00
00Goo`03003oo`0000Ooo`8000;oo`03003ooooo08ooo`009ooo00<00?ooool0:Ooo0P004ooo00<0
0?oo00003Ooo00<00?oo00001Ooo00<00?oo00001ooo0P000_oo00<00?ooool0Sooo000Wool00`00
ooooo`0Xool00`00ooooo`0Cool00`00ool0000=ool00`00ool00005ool00`00ool00007ool20002
ool00`00ooooo`2?ool000Soo`8000Coo`03003ooooo00?oo`8000Coo`8000?oo`<000Goo`03003o
oooo02Koo`8001Koo`03003oo`0000goo`03003oo`0000Goo`03003oo`0000Ooo`8000;oo`03003o
oooo08ooo`001ooo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`001Ooo00<00?ooool0
0_oo00<00?ooool09?oo0P006?oo00<00?oo00003Ooo0P001_oo00<00?oo00001ooo0P000_oo00<0
0?ooool0Sooo0007ool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0005ool00`00oooo
o`02ool2000Tool00`00ooooo`0Hool00`00ool0000=ool20006ool00`00ool00007ool20002ool0
0`00ooooo`2?ool00003ool00000008000;oo`04003ooooo0008ool01000ooooo`000_oo00@00?oo
ool000;oo`<000Goo`03003ooooo02?oo`03003ooooo01Soo`03003oo`0000goo`8000Koo`03003o
o`0000Ooo`8000;oo`03003ooooo08ooo`001ooo00@00?ooool000Soo`04003ooooo0002ool01000
ooooo`000_oo00<00?ooool01Ooo00<00?ooool08Ooo0P006ooo00<00?oo00003Ooo0P001_oo00<0
0?oo00001ooo0P000_oo00<00?ooool0Sooo0007ool01000ooooo`002?oo00@00?ooool000;oo`04
003ooooo0002ool00`00ooooo`05ool00`00ooooo`0Pool00`00ooooo`0Kool00`00ool0000=ool2
0006ool00`00ool00007ool20002ool00`00ooooo`2?ool000Soo`8000[oo`8000Coo`8000?oo`@0
00Coo`03003ooooo01koo`8001koo`03003oo`0000goo`8000Koo`03003oo`0000Ooo`8000;oo`03
003ooooo08ooo`009ooo0P007Ooo0P008?oo0P003_oo0P001_oo00<00?oo00001ooo0P000_oo00<0
0?ooool0Sooo000Wool00`00ooooo`0Kool00`00ooooo`0Pool2000>ool20006ool00`00ool00007
ool2002Dool002Ooo`03003ooooo01Woo`8002?oo`8000koo`8000Koo`03003oo`0000Ooo`8009Co
o`009ooo00<00?ooool05ooo0P009Ooo0P003_oo0P001_oo00<00?oo00001ooo0P00U?oo000Wool0
0`00ooooo`0Fool00`00ooooo`0Uool2000>ool20006ool00`00ool00007ool2002Dool002Ooo`80
01Goo`8002Soo`8000koo`8000Koo`03003oo`0000Ooo`8009Coo`009ooo00<00?ooool04ooo00<0
0?ooool0:?oo0P003_oo0P001_oo00<00?oo00002?oo00<00?ooool0T_oo000Wool00`00ooooo`0A
ool2000[ool2000>ool20007ool00`00ooooo`07ool00`00ooooo`2Bool002Ooo`03003ooooo00oo
o`8002goo`8000koo`8000Ooo`03003ooooo00Ooo`03003ooooo09;oo`009ooo00<00?ooool03_oo
00<00?ooool0;Ooo0P003ooo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool0T_oo000Wool2
000=ool2000`ool2000?ool00`00ooooo`05ool00`00ooooo`07ool00`00ooooo`2Bool002Ooo`03
003ooooo00_oo`03003ooooo00Soo`8000Coo`03003ooooo00;oo`@0017oo`8000Coo`04003ooooo
0003ool00`00ooooo`09ool00`00ooooo`04ool20004ool00`00ooooo`02ool3000Cool20004ool0
0`00ooooo`03ool2000Gool4000Fool40003ool00`00ooooo`02ool4000@ool40003ool00`00oooo
o`04ool00`00ooooo`0?ool002Ooo`03003ooooo00Woo`8000[oo`04003ooooo0008ool00`00oooo
o`0Aool01000ooooo`001_oo00D00?ooooooo`0000_oo`03003ooooo00?oo`04003oo`000008ool0
1000ooooo`004Ooo00@00?ooool000Soo`04003ooooo000Hool00`00ooooo`0Gool00`00ooooo`07
ool00`00ooooo`0Cool00`00ooooo`09ool00`00ooooo`0?ool002Ooo`03003ooooo00Ooo`8000co
o`04003ooooo0009ool00`00ooooo`0@ool01000ooooo`001_oo1P002_oo00<00?ooool00ooo00@0
0?oo000000Soo`04003ooooo000Aool01000ooooo`002?oo00@00?ooool001Soo`03003ooooo01Oo
o`03003ooooo00Soo`03003ooooo01;oo`03003ooooo00Koo`D0013oo`009ooo00<00?ooool01_oo
00<00?ooool03?oo00@00?ooool000[oo`03003ooooo00ooo`04003ooooo0006ool20002ool00`00
ooooo`09ool00`00ooooo`03ool01000ool000002?oo0`004_oo00@00?ooool000Woo`8001Woo`03
003ooooo01Ooo`03003ooooo00Woo`03003ooooo017oo`03003ooooo00Koo`04003ooooo000Aool0
02Ooo`03003ooooo00Coo`8000ooo`04003ooooo000;ool00`00ooooo`0>ool01000ooooo`001_oo
00D00?oo003oo`0000_oo`03003ooooo00?oo`04003oo`000008ool00`00ooooo`0Bool01000oooo
o`002?oo00@00?ooool001Soo`03003ooooo01Ooo`03003ooooo00[oo`03003ooooo013oo`03003o
oooo00Ooo`03003oo`00017oo`009ooo0P001?oo00<00?ooool03ooo00@00?ooool000Soo`04003o
oooo000@ool01000ooooo`001_oo00<00?ooool00P002ooo00<00?ooool00ooo00@00?oo000000So
o`03003ooooo01;oo`04003ooooo0008ool01000ooooo`005_oo0`005ooo0`002Ooo00@00?ooool0
013oo`<000[oo`80017oo`009ooo00<00?ooool00ooo00<00?ooool04?oo0P002_oo0P004_oo0P00
1ooo00D00?ooooooo`0000_oo`03003ooooo00Coo`8000Woo`@001;oo`8000[oo`8001Woo`03003o
oooo01Ooo`03003ooooo00Soo`8001?oo`03003ooooo00Woo`03003ooooo00ooo`009ooo00@00?oo
ooooo`8003coo`03003ooooo00goo`03003ooooo00Goo`03003ooooo00Ooo`03003ooooo09;oo`00
9ooo00@00?oo000003koo`03003ooooo00goo`03003ooooo00Goo`03003ooooo00Ooo`03003ooooo
09;oo`009ooo0P00@?oo00<00?ooool03Ooo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool0
T_oo000Qoooo000002Ooo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo
00Goo`03003ooooo00Goo`03003ooooo00Goo`009ooo00<00?ooool0m_oo000Wool00`00ooooo`3f
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.243038, -0.0313349, \
0.00620399, 0.000192055}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["LX", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?LXtestfunc\)\)], "Input"],

Cell["Global`LXtestfunc", "Print",
  CellTags->"Info3280045044-9066215"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(LXtestfunc[om_, k_, 
                      th_] := \((om\^2 - k\^2 - 
                            LXpintnt[om\/k, th])\)\ \((1 - 
                            LXbeta[om\/k, th]\/om\^2)\) - \(LXdelta[om\/k, \
th]\^2\ \((1 - Cos[th]\^2)\)\)\/om\^2\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "LXtestfunc"],
      Editable->False]], "Print",
  CellTags->"Info3280045044-9066215"]
}, Open  ]],

Cell[BoxData[
    \(LXsol[k_] := 
      Om /. FindRoot[
          LXtestfunc[Om, k, 10^\((\(-4\))\)] \[Equal] 0, {Om, 0.6, 0.7}, 
          MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(LXsol[0.2]\)], "Input"],

Cell[BoxData[
    \(0.6829200103057246`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{k, LXsol[k]}, {k, 0.01, 1.5}, PlotRange \[Rule] All]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0174177 0.639182 0.0107647 0.395036 [
[.14525 -0.00174 -9 -9 ]
[.14525 -0.00174 9 0 ]
[.27309 -0.00174 -9 -9 ]
[.27309 -0.00174 9 0 ]
[.40093 -0.00174 -9 -9 ]
[.40093 -0.00174 9 0 ]
[.52876 -0.00174 -9 -9 ]
[.52876 -0.00174 9 0 ]
[.6566 -0.00174 -3 -9 ]
[.6566 -0.00174 3 0 ]
[.78444 -0.00174 -9 -9 ]
[.78444 -0.00174 9 0 ]
[.91227 -0.00174 -9 -9 ]
[.91227 -0.00174 9 0 ]
[.00492 .08977 -18 -4.5 ]
[.00492 .08977 0 4.5 ]
[.00492 .16878 -18 -4.5 ]
[.00492 .16878 0 4.5 ]
[.00492 .24779 -18 -4.5 ]
[.00492 .24779 0 4.5 ]
[.00492 .32679 -18 -4.5 ]
[.00492 .32679 0 4.5 ]
[.00492 .4058 -6 -4.5 ]
[.00492 .4058 0 4.5 ]
[.00492 .48481 -18 -4.5 ]
[.00492 .48481 0 4.5 ]
[.00492 .56382 -18 -4.5 ]
[.00492 .56382 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.14525 .01076 m
.14525 .01701 L
s
[(0.2)] .14525 -0.00174 0 1 Mshowa
.27309 .01076 m
.27309 .01701 L
s
[(0.4)] .27309 -0.00174 0 1 Mshowa
.40093 .01076 m
.40093 .01701 L
s
[(0.6)] .40093 -0.00174 0 1 Mshowa
.52876 .01076 m
.52876 .01701 L
s
[(0.8)] .52876 -0.00174 0 1 Mshowa
.6566 .01076 m
.6566 .01701 L
s
[(1)] .6566 -0.00174 0 1 Mshowa
.78444 .01076 m
.78444 .01701 L
s
[(1.2)] .78444 -0.00174 0 1 Mshowa
.91227 .01076 m
.91227 .01701 L
s
[(1.4)] .91227 -0.00174 0 1 Mshowa
.125 Mabswid
.04938 .01076 m
.04938 .01451 L
s
.08134 .01076 m
.08134 .01451 L
s
.11329 .01076 m
.11329 .01451 L
s
.17721 .01076 m
.17721 .01451 L
s
.20917 .01076 m
.20917 .01451 L
s
.24113 .01076 m
.24113 .01451 L
s
.30505 .01076 m
.30505 .01451 L
s
.33701 .01076 m
.33701 .01451 L
s
.36897 .01076 m
.36897 .01451 L
s
.43289 .01076 m
.43289 .01451 L
s
.46484 .01076 m
.46484 .01451 L
s
.4968 .01076 m
.4968 .01451 L
s
.56072 .01076 m
.56072 .01451 L
s
.59268 .01076 m
.59268 .01451 L
s
.62464 .01076 m
.62464 .01451 L
s
.68856 .01076 m
.68856 .01451 L
s
.72052 .01076 m
.72052 .01451 L
s
.75248 .01076 m
.75248 .01451 L
s
.8164 .01076 m
.8164 .01451 L
s
.84835 .01076 m
.84835 .01451 L
s
.88031 .01076 m
.88031 .01451 L
s
.94423 .01076 m
.94423 .01451 L
s
.97619 .01076 m
.97619 .01451 L
s
.25 Mabswid
0 .01076 m
1 .01076 L
s
.01742 .08977 m
.02367 .08977 L
s
[(0.2)] .00492 .08977 1 0 Mshowa
.01742 .16878 m
.02367 .16878 L
s
[(0.4)] .00492 .16878 1 0 Mshowa
.01742 .24779 m
.02367 .24779 L
s
[(0.6)] .00492 .24779 1 0 Mshowa
.01742 .32679 m
.02367 .32679 L
s
[(0.8)] .00492 .32679 1 0 Mshowa
.01742 .4058 m
.02367 .4058 L
s
[(1)] .00492 .4058 1 0 Mshowa
.01742 .48481 m
.02367 .48481 L
s
[(1.2)] .00492 .48481 1 0 Mshowa
.01742 .56382 m
.02367 .56382 L
s
[(1.4)] .00492 .56382 1 0 Mshowa
.125 Mabswid
.01742 .03052 m
.02117 .03052 L
s
.01742 .05027 m
.02117 .05027 L
s
.01742 .07002 m
.02117 .07002 L
s
.01742 .10952 m
.02117 .10952 L
s
.01742 .12928 m
.02117 .12928 L
s
.01742 .14903 m
.02117 .14903 L
s
.01742 .18853 m
.02117 .18853 L
s
.01742 .20828 m
.02117 .20828 L
s
.01742 .22803 m
.02117 .22803 L
s
.01742 .26754 m
.02117 .26754 L
s
.01742 .28729 m
.02117 .28729 L
s
.01742 .30704 m
.02117 .30704 L
s
.01742 .34655 m
.02117 .34655 L
s
.01742 .3663 m
.02117 .3663 L
s
.01742 .38605 m
.02117 .38605 L
s
.01742 .42555 m
.02117 .42555 L
s
.01742 .4453 m
.02117 .4453 L
s
.01742 .46506 m
.02117 .46506 L
s
.01742 .50456 m
.02117 .50456 L
s
.01742 .52431 m
.02117 .52431 L
s
.01742 .54406 m
.02117 .54406 L
s
.01742 .58357 m
.02117 .58357 L
s
.01742 .60332 m
.02117 .60332 L
s
.25 Mabswid
.01742 0 m
.01742 .61803 L
s
.5 Mabswid
.02381 .01472 m
.06244 .03859 L
.10458 .06463 L
.14415 .08909 L
.18221 .11261 L
.22272 .13765 L
.26171 .16175 L
.30316 .18736 L
.34309 .21204 L
.3815 .23578 L
.42237 .26104 L
.46172 .28536 L
.49955 .30874 L
.53984 .33364 L
.57861 .3576 L
.61984 .38308 L
.65954 .40762 L
.69774 .43123 L
.73838 .45635 L
.77751 .48053 L
.81909 .50623 L
.85916 .53099 L
.89771 .55482 L
.93871 .58016 L
.97619 .60332 L
s
.02381 .25838 m
.02605 .25844 L
.02846 .2585 L
.03072 .25859 L
.03279 .25868 L
.04262 .25929 L
.0475 .2597 L
.05281 .26023 L
.06244 .26139 L
.08435 .26496 L
.10458 .26931 L
.14393 .28013 L
.18177 .29281 L
.22205 .30803 L
.26083 .3239 L
.30205 .34173 L
.32264 .35093 L
.33267 .35547 L
.3374 .35762 L
.34176 .35961 L
.34291 .36012 L
.34398 .36059 L
.3451 .36082 L
.34633 .36084 L
.34758 .36083 L
.34872 .36085 L
.35003 .36085 L
.35128 .36086 L
.35236 .36085 L
.35351 .36085 L
.35449 .36086 L
.35557 .36086 L
.35673 .36086 L
.35798 .36086 L
.35916 .36085 L
.36024 .36085 L
.36139 .36085 L
.36262 .36086 L
.36378 .36086 L
.36483 .36086 L
.36607 .36086 L
.36676 .36086 L
.36741 .36086 L
.36859 .36086 L
.36984 .36086 L
.37104 .36086 L
.37233 .36086 L
.37355 .36086 L
.37466 .36085 L
.37596 .36085 L
Mistroke
.37737 .36086 L
.37871 .36086 L
.37995 .36086 L
.38124 .36086 L
.38261 .36086 L
.38377 .36086 L
.38504 .36086 L
.38642 .36086 L
.38789 .36086 L
.38928 .36086 L
.39055 .36086 L
.39182 .36086 L
.393 .36086 L
.39429 .36086 L
.39567 .36086 L
.39686 .36086 L
.39797 .36086 L
.3992 .36086 L
.40051 .36086 L
.40169 .36086 L
.40278 .36086 L
.40397 .36086 L
.40525 .36086 L
.4059 .36086 L
.4066 .36086 L
.40786 .36086 L
.4091 .36086 L
.41023 .36086 L
.41247 .36086 L
.41456 .36086 L
.41674 .36086 L
.41913 .36086 L
.42173 .36086 L
.42413 .36086 L
.42676 .36086 L
.42957 .36086 L
.43199 .36086 L
.43427 .36086 L
.43677 .36086 L
.43946 .36086 L
.44171 .36086 L
.44417 .36086 L
.44674 .36086 L
.44913 .36086 L
.45342 .36086 L
.45582 .36086 L
.45801 .36086 L
.46285 .36086 L
.46529 .36086 L
.46797 .36086 L
Mistroke
.47026 .36086 L
.47272 .36086 L
.47717 .36086 L
.47983 .36086 L
.48274 .36086 L
.4853 .36086 L
.48803 .36086 L
.49318 .36086 L
.49546 .36086 L
.49667 .36086 L
.49796 .36086 L
.49867 .36086 L
.49931 .36086 L
.50057 .36086 L
.50332 .36086 L
.50825 .36086 L
.51397 .36086 L
.51939 .36086 L
.52905 .36086 L
.53451 .36086 L
.53956 .36086 L
.5442 .36086 L
.5492 .36086 L
.55824 .36086 L
.56348 .36086 L
.56899 .36086 L
.57885 .36086 L
.5884 .36086 L
.5986 .36086 L
.60823 .36086 L
.61698 .36086 L
.62697 .36086 L
.63618 .36086 L
.64628 .36086 L
.65698 .36086 L
.66642 .36086 L
.67657 .36086 L
.69488 .36086 L
.71608 .36086 L
.73618 .36086 L
.77445 .36086 L
.81516 .36086 L
.85437 .36086 L
.89602 .36086 L
.93616 .36086 L
.97478 .36086 L
.97619 .36086 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool0033oo`8000Coo`03003ooooo00;oo`@001Coo`8000Coo`03003ooooo00Coo`03003ooooo01?o
o`8000Coo`03003ooooo00?oo`8001Coo`8000Coo`03003ooooo00?oo`8001[oo`@001Soo`@000?o
o`03003ooooo00;oo`@001?oo`@000?oo`03003ooooo00Coo`03003ooooo013oo`00;ooo00@00?oo
ool000Soo`03003ooooo01Coo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`002?oo00@0
0?ooool001;oo`04003ooooo0008ool01000ooooo`006ooo00<00?ooool06Ooo00<00?ooool01ooo
00<00?ooool05_oo00<00?ooool02Ooo00<00?ooool04?oo000_ool01000ooooo`002Ooo00<00?oo
ool04ooo00@00?ooool000Ooo`D001?oo`04003ooooo0008ool01000ooooo`004_oo00@00?ooool0
00Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`08ool00`00ooooo`0Eool00`00oooo
o`06ool5000Aool002ooo`04003ooooo000:ool00`00ooooo`0Bool01000ooooo`001ooo00@00?oo
ool001Coo`04003ooooo0008ool3000Cool01000ooooo`002Ooo0P007?oo00<00?ooool06Ooo00<0
0?ooool02Ooo00<00?ooool05?oo00<00?ooool01_oo00@00?ooool001;oo`00;ooo00@00?ooool0
00_oo`03003ooooo017oo`04003ooooo0008ool00`00ool0000Dool01000ooooo`002?oo00<00?oo
ool04ooo00@00?ooool000Soo`04003ooooo000Kool00`00ooooo`0Iool00`00ooooo`0:ool00`00
ooooo`0Cool00`00ooooo`07ool00`00ool0000Bool002ooo`04003ooooo0008ool01000ooooo`00
4ooo00@00?ooool000Woo`8001Coo`04003ooooo0008ool00`00ooooo`0Cool01000ooooo`002?oo
00@00?ooool001Woo`<001Woo`<000Woo`04003ooooo000Cool3000:ool2000Bool0033oo`8000[o
o`8001Goo`8000_oo`03003ooooo01?oo`8000[oo`<001?oo`8000[oo`8001coo`03003ooooo01Wo
o`03003ooooo00Soo`8001Koo`03003ooooo00Woo`03003ooooo013oo`005Ooo00<00?ooool0oooo
2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`004?ooo`004@00000Eool0
0`00ool00006ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool001Goo`03003ooooo008001goo`03003ooooo023oo`03003ooooo023oo`03003o
oooo01ooo`03003ooooo023oo`03003ooooo01ooo`03003ooooo023oo`03003ooooo01Koo`005Ooo
00<00?ooool00_oo00<00?ooool0oooo1?oo000Eool00`00ooooo`03ool00`00ooooo`3oool3ool0
01Goo`8000Goo`800?ooo`?oo`005Ooo00<00?ooool01_oo00<00?ooool0oooo000Eool00`00oooo
o`07ool2003oool001Goo`03003ooooo00Woo`03003ooooo0?coo`005Ooo00<00?ooool02_oo0P00
o?oo000Eool00`00ooooo`0<ool00`00ooooo`3iool001Goo`8000koo`800?Woo`005Ooo00<00?oo
ool03ooo0P00mooo000Eool00`00ooooo`0Aool00`00ooooo`3dool001Goo`03003ooooo01;oo`80
0?Coo`005Ooo00<00?ooool05?oo00<00?ooool0lOoo000Eool2000Fool2003aool001Goo`03003o
oooo01Ooo`800>ooo`005Ooo00<00?ooool06Ooo0P00kOoo0002ool20004ool00`00ooooo`02ool4
0004ool00`00ooooo`0Kool2003[ool00005ool00?ooool00008ool00`00ooooo`05ool00`00oooo
o`0Mool2003Yool00005ool00?ooool00009ool00`00ooooo`04ool3000Oool00`00ooooo`3Vool0
0005ool00?ooool0000:ool00`00ooooo`03ool00`00ooooo`0Pool2003Vool00005ool00?ooool0
000;ool00`00ooooo`02ool00`00ooooo`0Rool00`00ooooo`3Sool00005ool00?ooool00008ool0
1000ooooo`001?oo00<00?ooool08ooo00<00?ooool0h_oo0002ool2000:ool20005ool00`00oooo
o`0Tool2003Rool001Goo`03003ooooo02Koo`03003ooooo0=ooo`005Ooo0P00:?oo0P00gooo000E
ool00`00ooooo`0Yool00`00ooooo`3Lool001Goo`03003ooooo02[oo`800=coo`005Ooo00<00?oo
ool0;?oo0P00f_oo000Eool00`00ooooo`0^ool2003Hool001Goo`80037oo`800=Koo`005Ooo00<0
0?ooool0<_oo0P00e?oo000Eool00`00ooooo`0dool00`00ooooo`3Aool001Goo`03003ooooo03Go
o`800=7oo`005Ooo00<00?ooool0=ooo00<00?ooool0c_oo000Eool00`00ooooo`0hool2003>ool0
01Goo`8003_oo`800<coo`005Ooo00<00?ooool0??oo00<00?ooool0bOoo000Eool00`00ooooo`0m
ool20039ool000;oo`8000Coo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo03ooo`03003o
oooo0<Koo`0000Goo`00ooooo`0000[oo`03003ooooo00?oo`03003ooooo043oo`800<Koo`0000Go
o`00ooooo`0000Ooo`D000Coo`<004;oo`03003ooooo0<?oo`0000Goo`00ooooo`0000Ooo`04003o
oooo0005ool00`00ooooo`13ool20033ool00005ool00?ooool00008ool00`00ool00005ool00`00
ooooo`15ool20031ool00005ool00?ooool00009ool20005ool00`00ooooo`17ool00`00ooooo`2n
ool000;oo`8000_oo`03003ooooo00?oo`03003ooooo04Soo`800;koo`005Ooo0P00Booo00<00?oo
ool0^ooo000Eool00`00ooooo`1;ool2002kool001Goo`03003ooooo04goo`03003ooooo0;Soo`00
5Ooo00<00?ooool0C_oo0P00^?oo000Eool00`00ooooo`1@ool2002fool001Goo`03003ooooo05;o
o`03003ooooo0;?oo`005Ooo0P00E?oo0P00/ooo000Eool00`00ooooo`1Eool00`00ooooo`2`ool0
01Goo`03003ooooo05Koo`800;3oo`005Ooo00<00?ooool0F?oo0P00[_oo000Eool00`00ooooo`1J
ool00`00ooooo`2[ool001Goo`8005coo`800:_oo`005Ooo00<00?ooool0GOoo0P00ZOoo000Eool0
0`00ooooo`1Oool00`00ooooo`2Vool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo
063oo`800:Koo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`1Rool00`00oooo
o`2Sool00005ool00?ooool00008ool01000ooooo`001?oo0`00Hooo0P00Xooo00001Ooo003ooooo
00002?oo0`001Ooo00<00?ooool0IOoo0P00XOoo00001Ooo003ooooo00002?oo00<00?ooool01Ooo
00<00?ooool0Iooo00<00?ooool0W_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?oo
00001P00H_oo0P00W_oo0002ool2000:ool30004ool00`00ooooo`06ool6001Nool00`00ooooo`2K
ool001Goo`03003ooooo00coo`H005Woo`8009_oo`005Ooo0P004ooo1000Eooo0P00VOoo000Eool0
0`00ooooo`0Fool4001Eool2002Gool001Goo`03003ooooo01[oo`@005?oo`8009Goo`005Ooo00<0
0?ooool07_oo0`00D_oo0P00Tooo000Eool00`00ooooo`0Qool3001Aool00`00ooooo`2@ool001Go
o`8002Goo`@004koo`80093oo`005Ooo00<00?ooool0:?oo0`00COoo00<00?ooool0SOoo000Eool0
0`00ooooo`0[ool3001;ool00`00ooooo`2<ool001Goo`03003ooooo02koo`8004[oo`8008coo`00
5Ooo00<00?ooool0<?oo0`00BOoo00<00?ooool0ROoo000Eool2000dool30017ool20029ool001Go
o`03003ooooo03Koo`8004Ooo`03003ooooo08Koo`005Ooo00<00?ooool0>?oo0P00A_oo0P00Q_oo
000Eool00`00ooooo`0jool20016ool20024ool000;oo`8000Coo`03003ooooo00?oo`8000Goo`03
003ooooo03coo`8004Koo`8008;oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00oooo
o`0nool30015ool20020ool00005ool00?ooool00008ool01000ooooo`001?oo0`00@Ooo0P00AOoo
0P00O_oo00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool0@ooo0P00AOoo00<00?ooool0Nooo
00001Ooo003ooooo00002?oo00@00?ooool000Coo`03003ooooo04Goo`8004Coo`8007_oo`0000Go
o`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`17ool20014ool00`00ooooo`1hool000;o
o`8000[oo`8000Goo`03003ooooo04Woo`<004;oo`8007Soo`005Ooo0P00COoo0P00@_oo0P00M_oo
000Eool00`00ooooo`1>ool30011ool00`00ooooo`1cool001Goo`03003ooooo057oo`80043oo`80
07?oo`005Ooo00<00?ooool0Dooo0`00?ooo00<00?ooool0L?oo000Eool00`00ooooo`1Eoon/0007
ool001Goo`8009Woo`03003ooooo06goo`005Ooo00<00?ooool0VOoo0P00KOoo000Eool00`00oooo
o`2Kool2001[ool001Goo`03003ooooo09goo`03003ooooo06Soo`005Ooo00<00?ooool0W_oo0P00
J?oo000Eool00`00ooooo`2Pool00`00ooooo`1Uool001Goo`800:;oo`8006Goo`005Ooo00<00?oo
ool0Xooo0P00Hooo000Eool00`00ooooo`2Uool00`00ooooo`1Pool000goo`@000Coo`03003ooooo
0:Koo`80063oo`003ooo00<00?ooool00ooo00<00?ooool0Z?oo0P00G_oo000?ool00`00ooooo`03
ool3002Zool00`00ooooo`1Kool000ooo`03003ooooo00?oo`03003ooooo0:_oo`8005_oo`003ooo
00<00?ooool00ooo00<00?ooool0[Ooo00<00?ooool0F?oo000=ool30005ool00`00ooooo`2^ool2
001Hool000ooo`03003ooooo00?oo`03003ooooo0;3oo`8005Koo`005Ooo0P00/ooo00<00?ooool0
Dooo000Eool00`00ooooo`2cool2001Cool001Goo`03003ooooo0;Goo`03003ooooo053oo`005Ooo
00<00?ooool0]_oo0P00D?oo000Eool00`00ooooo`2hool00`00ooooo`1=ool001Goo`03003ooooo
0;Woo`8004goo`005Ooo0P00_?oo0P00Booo000Eool00`00ooooo`2mool00`00ooooo`18ool001Go
o`03003ooooo0;koo`8004Soo`005Ooo00<00?ooool0`?oo00<00?ooool0AOoo000Eool00`00oooo
o`31ool20015ool001Goo`800<Coo`8004?oo`005Ooo00<00?ooool0aOoo00<00?ooool0@?oo000E
ool00`00ooooo`36ool20010ool00003ool00000008000?oo`03003ooooo00;oo`@000Coo`03003o
oooo0<Soo`8003koo`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0b_oo00<00?oo
ool0>ooo0003ool00`00ooooo`08ool00`00ooooo`04ool3003;ool2000kool000?oo`03003ooooo
00Woo`03003ooooo00?oo`03003ooooo0<goo`8003Woo`000ooo00<00?ooool02_oo00<00?ooool0
0_oo00<00?ooool0cooo00<00?ooool0=_oo00001?oo000000002Ooo00@00?ooool000Coo`03003o
oooo0=3oo`8003Koo`000ooo00<00?ooool02?oo0P001Ooo00<00?ooool0d_oo0P00=?oo000Eool0
0`00ooooo`3Dool2000bool001Goo`800=Ooo`03003ooooo02ooo`005Ooo00<00?ooool0eooo0P00
;ooo000Eool00`00ooooo`3Iool00`00ooooo`0/ool001Goo`03003ooooo0=[oo`03003ooooo02_o
o`005Ooo00<00?ooool0fooo0P00:ooo000Eool2003Nool00`00ooooo`0Xool001Goo`03003ooooo
0=koo`8002Soo`005Ooo00<00?ooool0h?oo00<00?ooool09Ooo000Eool00`00ooooo`3Qool2000U
ool001Goo`03003ooooo0>?oo`8002?oo`005Ooo0P00i_oo0P008Ooo000Eool00`00ooooo`3Wool2
000Oool001Goo`03003ooooo0>Woo`8001goo`005Ooo00<00?ooool0jooo00<00?ooool06_oo0000
0ooo000000020003ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`3/ool2000Jool000?o
o`03003ooooo00Woo`03003ooooo00?oo`03003ooooo0>koo`03003ooooo01Ooo`000ooo00<00?oo
ool01_oo1@001?oo0`00kooo0P005ooo0003ool00`00ooooo`06ool01000ooooo`001Ooo00<00?oo
ool0lOoo0P005Ooo0003ool00`00ooooo`07ool00`00ool00005ool00`00ooooo`3cool00`00oooo
o`0Bool00004ool00000000:ool20005ool00`00ooooo`3dool2000Bool000?oo`03003ooooo00Wo
o`03003ooooo00?oo`03003ooooo0?Koo`03003ooooo00ooo`005Ooo0P00n?oo0P003ooo000Eool0
0`00ooooo`3iool2000=ool001Goo`03003ooooo0?_oo`03003ooooo00[oo`005Ooo00<00?ooool0
o?oo0P002_oo000Eool00`00ooooo`3nool20008ool001Goo`800?ooo`;oo`03003ooooo00Goo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.123734, -0.11779, \
0.00578743, 0.00936426}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{LXsol[k]}, {k, 0.01, 1.5}, PlotRange \[Rule] All, 
      PlotStyle \[Rule] {RGBColor[0, 0, 1]}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the accuracy \
\!\(1.0000000000000004`*^-6\)."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0174177 0.639182 -1.4076 2.26908 [
[.14525 .16825 -9 -9 ]
[.14525 .16825 9 0 ]
[.27309 .16825 -9 -9 ]
[.27309 .16825 9 0 ]
[.40093 .16825 -9 -9 ]
[.40093 .16825 9 0 ]
[.52876 .16825 -9 -9 ]
[.52876 .16825 9 0 ]
[.6566 .16825 -3 -9 ]
[.6566 .16825 3 0 ]
[.78444 .16825 -9 -9 ]
[.78444 .16825 9 0 ]
[.91227 .16825 -9 -9 ]
[.91227 .16825 9 0 ]
[.00492 .0673 -24 -4.5 ]
[.00492 .0673 0 4.5 ]
[.00492 .29421 -24 -4.5 ]
[.00492 .29421 0 4.5 ]
[.00492 .40766 -18 -4.5 ]
[.00492 .40766 0 4.5 ]
[.00492 .52112 -24 -4.5 ]
[.00492 .52112 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.14525 .18075 m
.14525 .187 L
s
[(0.2)] .14525 .16825 0 1 Mshowa
.27309 .18075 m
.27309 .187 L
s
[(0.4)] .27309 .16825 0 1 Mshowa
.40093 .18075 m
.40093 .187 L
s
[(0.6)] .40093 .16825 0 1 Mshowa
.52876 .18075 m
.52876 .187 L
s
[(0.8)] .52876 .16825 0 1 Mshowa
.6566 .18075 m
.6566 .187 L
s
[(1)] .6566 .16825 0 1 Mshowa
.78444 .18075 m
.78444 .187 L
s
[(1.2)] .78444 .16825 0 1 Mshowa
.91227 .18075 m
.91227 .187 L
s
[(1.4)] .91227 .16825 0 1 Mshowa
.125 Mabswid
.04938 .18075 m
.04938 .1845 L
s
.08134 .18075 m
.08134 .1845 L
s
.11329 .18075 m
.11329 .1845 L
s
.17721 .18075 m
.17721 .1845 L
s
.20917 .18075 m
.20917 .1845 L
s
.24113 .18075 m
.24113 .1845 L
s
.30505 .18075 m
.30505 .1845 L
s
.33701 .18075 m
.33701 .1845 L
s
.36897 .18075 m
.36897 .1845 L
s
.43289 .18075 m
.43289 .1845 L
s
.46484 .18075 m
.46484 .1845 L
s
.4968 .18075 m
.4968 .1845 L
s
.56072 .18075 m
.56072 .1845 L
s
.59268 .18075 m
.59268 .1845 L
s
.62464 .18075 m
.62464 .1845 L
s
.68856 .18075 m
.68856 .1845 L
s
.72052 .18075 m
.72052 .1845 L
s
.75248 .18075 m
.75248 .1845 L
s
.8164 .18075 m
.8164 .1845 L
s
.84835 .18075 m
.84835 .1845 L
s
.88031 .18075 m
.88031 .1845 L
s
.94423 .18075 m
.94423 .1845 L
s
.97619 .18075 m
.97619 .1845 L
s
.25 Mabswid
0 .18075 m
1 .18075 L
s
.01742 .0673 m
.02367 .0673 L
s
[(0.65)] .00492 .0673 1 0 Mshowa
.01742 .29421 m
.02367 .29421 L
s
[(0.75)] .00492 .29421 1 0 Mshowa
.01742 .40766 m
.02367 .40766 L
s
[(0.8)] .00492 .40766 1 0 Mshowa
.01742 .52112 m
.02367 .52112 L
s
[(0.85)] .00492 .52112 1 0 Mshowa
.125 Mabswid
.01742 .08999 m
.02117 .08999 L
s
.01742 .11268 m
.02117 .11268 L
s
.01742 .13537 m
.02117 .13537 L
s
.01742 .15806 m
.02117 .15806 L
s
.01742 .20345 m
.02117 .20345 L
s
.01742 .22614 m
.02117 .22614 L
s
.01742 .24883 m
.02117 .24883 L
s
.01742 .27152 m
.02117 .27152 L
s
.01742 .3169 m
.02117 .3169 L
s
.01742 .33959 m
.02117 .33959 L
s
.01742 .36228 m
.02117 .36228 L
s
.01742 .38497 m
.02117 .38497 L
s
.01742 .43035 m
.02117 .43035 L
s
.01742 .45304 m
.02117 .45304 L
s
.01742 .47574 m
.02117 .47574 L
s
.01742 .49843 m
.02117 .49843 L
s
.01742 .04461 m
.02117 .04461 L
s
.01742 .02192 m
.02117 .02192 L
s
.01742 .54381 m
.02117 .54381 L
s
.01742 .5665 m
.02117 .5665 L
s
.01742 .58919 m
.02117 .58919 L
s
.01742 .61188 m
.02117 .61188 L
s
.25 Mabswid
.01742 0 m
.01742 .61803 L
s
0 0 1 r
.5 Mabswid
.02381 .01472 m
.02605 .01503 L
.02846 .01541 L
.03072 .0159 L
.03279 .01643 L
.04262 .01993 L
.0475 .02229 L
.05281 .02531 L
.06244 .03197 L
.08435 .0525 L
.10458 .07746 L
.14393 .13964 L
.18177 .21244 L
.22205 .29991 L
.26083 .39106 L
.30205 .49347 L
.32264 .5463 L
.33267 .57237 L
.3374 .58472 L
.34176 .59618 L
.34291 .59909 L
.34398 .60177 L
.3451 .60313 L
.34633 .60324 L
.34758 .60319 L
.34872 .60328 L
.35003 .60331 L
.35128 .60332 L
.35236 .60329 L
.35351 .6033 L
.35449 .60331 L
.35557 .60332 L
.35673 .60332 L
.35798 .60332 L
.35916 .60329 L
.36024 .6033 L
.36139 .60331 L
.36262 .60331 L
.36378 .60331 L
.36483 .60332 L
.36607 .60332 L
.36676 .60332 L
.36741 .60332 L
.36859 .60332 L
.36984 .60332 L
.37104 .60332 L
.37233 .60332 L
.37355 .60332 L
.37466 .60331 L
.37596 .60331 L
Mistroke
.37737 .60331 L
.37871 .60331 L
.37995 .60332 L
.38124 .60332 L
.38261 .60332 L
.38377 .60332 L
.38504 .60332 L
.38642 .60332 L
.38789 .60332 L
.38928 .60332 L
.39055 .60332 L
.39182 .60332 L
.393 .60332 L
.39429 .60332 L
.39567 .60332 L
.39686 .60332 L
.39797 .60332 L
.3992 .60332 L
.40051 .60332 L
.40169 .60332 L
.40278 .60332 L
.40397 .60332 L
.40525 .60332 L
.4059 .60332 L
.4066 .60332 L
.40786 .60331 L
.4091 .60331 L
.41023 .60331 L
.41247 .60332 L
.41456 .60332 L
.41674 .60332 L
.41913 .60332 L
.42173 .60332 L
.42413 .60332 L
.42676 .60332 L
.42957 .60332 L
.43199 .60332 L
.43427 .60332 L
.43677 .60332 L
.43946 .60332 L
.44171 .60332 L
.44417 .60332 L
.44674 .60332 L
.44913 .60332 L
.45342 .60332 L
.45582 .60332 L
.45801 .60332 L
.46285 .60332 L
.46529 .60332 L
.46797 .60332 L
Mistroke
.47026 .60332 L
.47272 .60332 L
.47717 .60332 L
.47983 .60332 L
.48274 .60332 L
.4853 .60332 L
.48803 .60332 L
.49318 .60332 L
.49546 .60332 L
.49667 .60332 L
.49796 .60332 L
.49867 .60332 L
.49931 .60332 L
.50057 .60332 L
.50332 .60332 L
.50825 .60332 L
.51397 .60332 L
.51939 .60332 L
.52905 .60332 L
.53451 .60332 L
.53956 .60332 L
.5442 .60332 L
.5492 .60332 L
.55824 .60332 L
.56348 .60332 L
.56899 .60332 L
.57885 .60332 L
.5884 .60332 L
.5986 .60332 L
.60823 .60332 L
.61698 .60332 L
.62697 .60332 L
.63618 .60332 L
.64628 .60332 L
.65698 .60332 L
.66642 .60332 L
.67657 .60332 L
.69488 .60332 L
.71608 .60332 L
.73618 .60332 L
.77445 .60332 L
.81516 .60332 L
.85437 .60332 L
.89602 .60332 L
.93616 .60332 L
.97478 .60332 L
.97619 .60332 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
100001l07`0Ooooo0_oo000Kool00`00ooooo`0201ooool1ool001_oo`8000?oo`@07ocoo`006ooo
00<00?ooool01_oo00<07oooool0nOoo000Kool00`00ooooo`07ool201oiool001_oo`03003ooooo
00Woo`0301oooooo0?Koo`006ooo00<00?ooool02_oo00<07oooool0mOoo000Kool00`00ooooo`0;
ool00`0Oooooo`3dool001_oo`8000goo`0301oooooo0??oo`006ooo00<00?ooool03Ooo00<07ooo
ool0l_oo000Kool00`00ooooo`0>ool00`0Oooooo`3aool001_oo`03003ooooo00ooo`0301oooooo
0?3oo`000_oo0P001?oo00<00?ooool00ooo0P000ooo0`001Ooo00<00?ooool04?oo00<07oooool0
kooo00001Ooo003ooooo00002?oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo017oo`03
01oooooo0>koo`0000Goo`00ooooo`0000Soo`04003ooooo0005ool00`00ooooo`02ool2000Bool0
0`0Oooooo`3^ool00005ool00?ooool00008ool30003ool30005ool00`00ooooo`0Bool00`0Ooooo
o`3]ool00005ool00?ooool00008ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`0Cool0
0`0Oooooo`3/ool00005ool00?ooool00008ool00`00ooooo`03ool00`00ooooo`05ool00`00oooo
o`0Dool00`0Oooooo`3[ool000;oo`8000[oo`<000;oo`@000Coo`03003ooooo01Goo`0301oooooo
0>[oo`006ooo00<00?ooool05Ooo00<07oooool0j_oo000Kool2000Gool00`0Oooooo`3Yool001_o
o`03003ooooo01Koo`0301oooooo0>Woo`006ooo00<00?ooool05ooo00<07oooool0j?oo000Kool0
0`00ooooo`0Hool00`0Oooooo`3Wool001_oo`03003ooooo01Soo`0301oooooo0>Ooo`006ooo00<0
0?ooool06Ooo00<07oooool0i_oo000Kool2000Kool00`0Oooooo`3Uool001_oo`03003ooooo01[o
o`0301oooooo0>Goo`006ooo00<00?ooool06ooo00<07oooool0i?oo000Kool00`00ooooo`0Kool0
0`0Oooooo`3Tool001_oo`03003ooooo01coo`0301oooooo0>?oo`006ooo00<00?ooool07Ooo00<0
7oooool0h_oo000Kool2000Nool00`0Oooooo`3Rool001_oo`03003ooooo01koo`0301oooooo0>7o
o`006ooo00<00?ooool06?oo0P001?oo00<07oooool00_oo10004_oo0P001?oo00<00?ooool01?oo
00<00?ooool04_oo0P001?oo00<00?ooool00ooo0P005?oo0P001?oo00<00?ooool00ooo0P006Ooo
10006?oo10000ooo00<00?ooool00_oo10004Ooo10000ooo00<00?ooool01?oo00<00?ooool04?oo
000Kool00`00ooooo`0Gool01000ooooo`001?oo00D07oooooooo`0001Coo`04003ooooo000:ool0
0`00ooooo`0Aool01000ooooo`002?oo00@00?ooool001;oo`04003ooooo0008ool01000ooooo`00
6_oo00<00?ooool06Ooo00<00?ooool01ooo00<00?ooool05?oo00<00?ooool02Ooo00<00?ooool0
4?oo000Kool00`00ooooo`0Gool01000ooooo`001?oo00<07oooool00_oo00<00?ooool04Ooo00@0
0?ooool000Ooo`D001;oo`04003ooooo0008ool01000ooooo`004_oo00@00?ooool000Soo`04003o
oooo000Jool00`00ooooo`0Iool00`00ooooo`08ool00`00ooooo`0Cool00`00ooooo`06ool5000A
ool001_oo`03003ooooo01Ooo`04003ooooo0005ool00`0Oooooo`02ool00`00ooooo`0@ool01000
ooooo`001ooo00@00?ooool001?oo`04003ooooo0008ool3000Cool01000ooooo`002Ooo0P006ooo
00<00?ooool06Ooo00<00?ooool02Ooo00<00?ooool04_oo00<00?ooool01_oo00@00?ooool001;o
o`006ooo0P006?oo00@00?ooool000Goo`0301oooooo00?oo`03003ooooo00ooo`04003ooooo0008
ool00`00ool0000Cool01000ooooo`002?oo00<00?ooool04ooo00@00?ooool000Soo`04003ooooo
000Jool00`00ooooo`0Iool00`00ooooo`0:ool00`00ooooo`0Aool00`00ooooo`07ool00`00ool0
000Bool001_oo`03003ooooo01Ooo`04003ooooo0006ool00`0Oool00002ool00`00ooooo`0?ool0
1000ooooo`002Ooo0P004ooo00@00?ooool000Soo`03003ooooo01?oo`04003ooooo0008ool01000
ooooo`006?oo0`006Ooo0`002Ooo00@00?ooool0017oo`<000[oo`8001;oo`006ooo00<00?ooool0
6?oo0P001ooo00<07oooool00P004ooo0P002ooo00<00?ooool04_oo0P002_oo0`004ooo0P002_oo
0P006ooo00<00?ooool06Ooo00<00?ooool02?oo0P005?oo00<00?ooool02Ooo00<00?ooool04?oo
000Kool00`00ooooo`0Rool00`0Oooooo`3Mool001_oo`03003ooooo02;oo`0301oooooo0=goo`00
6ooo00<00?ooool08ooo00<07oooool0g?oo000Fool[00000`0O0000003L000001_oo`03003ooooo
00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00;oo`0401oooooo
0007ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00oooo
o`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00oooo
o`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00oooo
o`05ool001_oo`03003ooooo01ooo`03003ooooo00;oo`0301oooooo01Woo`03003ooooo01ooo`03
003ooooo01ooo`03003ooooo01ooo`03003ooooo01ooo`03003ooooo01koo`03003ooooo01Koo`00
6ooo00<00?ooool09Ooo00<07oooool0f_oo000Kool00`00ooooo`0Uool00`0Oooooo`3Jool001_o
o`03003ooooo02Koo`0301oooooo0=Woo`006ooo0P009ooo00<07oooool0fOoo000Kool00`00oooo
o`0Wool00`0Oooooo`3Hool001_oo`03003ooooo02Ooo`0301oooooo0=Soo`006ooo00<00?ooool0
:?oo00<07oooool0eooo000Kool00`00ooooo`0Xool00`0Oooooo`3Gool001_oo`03003ooooo02Wo
o`0301oooooo0=Koo`006ooo0P00:_oo00<07oooool0e_oo000Kool00`00ooooo`0Zool00`0Ooooo
o`3Eool001_oo`03003ooooo02[oo`0301oooooo0=Goo`006ooo00<00?ooool0:ooo00<07oooool0
e?oo000Kool00`00ooooo`0[ool00`0Oooooo`3Dool001_oo`03003ooooo02coo`0301oooooo0=?o
o`006ooo0P00;Ooo00<07oooool0dooo000Kool00`00ooooo`0]ool00`0Oooooo`3Bool001_oo`03
003ooooo02goo`0301oooooo0=;oo`006ooo00<00?ooool0;_oo00<07oooool0dOoo000Kool00`00
ooooo`0^ool00`0Oooooo`3Aool001_oo`03003ooooo02ooo`0301oooooo0=3oo`006ooo0P00<?oo
00<07oooool0d?oo000Kool00`00ooooo`0`ool00`0Oooooo`3?ool001_oo`03003ooooo033oo`03
01oooooo0<ooo`006ooo00<00?ooool0<Ooo00<07oooool0c_oo0002ool20004ool00`00ooooo`03
ool00`00ooooo`02ool30005ool00`00ooooo`0aool00`0Oooooo`3>ool00005ool00?ooool00009
ool00`00ooooo`05ool00`00ooooo`02ool00`00ooooo`0bool00`0Oooooo`3=ool00005ool00?oo
ool0000:ool00`00ooooo`04ool00`00ooooo`02ool2000cool00`0Oooooo`3=ool00005ool00?oo
ool0000:ool01000oooooooo0`001Ooo00<00?ooool0<ooo00<07oooool0c?oo00001Ooo003ooooo
00002ooo00@00?ooool000Ooo`03003ooooo03?oo`0301oooooo0<coo`0000Goo`00ooooo`0000So
o`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`0cool00`0Oooooo`3<ool000;oo`8000Wo
o`@000;oo`@000Coo`03003ooooo03Coo`0301oooooo0<_oo`006ooo00<00?ooool0=?oo00<07ooo
ool0booo000Kool2000fool00`0Oooooo`3:ool001_oo`03003ooooo03Goo`0301oooooo0<[oo`00
6ooo00<00?ooool0=Ooo00<07oooool0b_oo000Kool00`00ooooo`0fool00`0Oooooo`39ool001_o
o`03003ooooo03Koo`0301oooooo0<Woo`006ooo00<00?ooool0=ooo00<07oooool0b?oo000Kool2
000hool00`0Oooooo`38ool001_oo`03003ooooo03Soo`0301oooooo0<Ooo`006ooo00<00?ooool0
>?oo00<07oooool0aooo000Kool00`00ooooo`0hool00`0Oooooo`37ool001_oo`03003ooooo03Wo
o`0301oooooo0<Koo`006ooo00<00?ooool0>Ooo00<07oooool0a_oo000Kool2000kool00`0Ooooo
o`35ool001_oo`03003ooooo03[oo`0301oooooo0<Goo`006ooo00<00?ooool0>_oo00<07oooool0
aOoo000Kool00`00ooooo`0kool00`0Oooooo`34ool001_oo`03003ooooo03_oo`0301oooooo0<Co
o`006ooo00<00?ooool0??oo00<07oooool0`ooo000Kool2000mool00`0Oooooo`33ool001_oo`03
003ooooo03goo`0301oooooo0<;oo`006ooo00<00?ooool0?Ooo00<07oooool0`_oo000Kool00`00
ooooo`0mool00`0Oooooo`32ool000Soo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo03ko
o`0301oooooo0<7oo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`0nool00`0O
ooooo`31ool000Ooo`04003ooooo0008ool01000ooooo`001?oo0P00@?oo00<07oooool0`?oo0007
ool01000ooooo`002Ooo0P001Ooo00<00?ooool0?ooo00<07oooool0`?oo0007ool01000ooooo`00
2?oo00@00?ooool000Coo`03003ooooo03ooo`0301oooooo0<3oo`001ooo00@00?ooool000Soo`04
003ooooo0004ool00`00ooooo`10ool00`0Oooooo`2oool000Soo`8000[oo`8000Goo`03003ooooo
043oo`0301oooooo0;ooo`006ooo00<00?ooool0@Ooo00<07oooool0__oo000Kool20012ool00`0O
ooooo`2nool001_oo`03003ooooo047oo`0301oooooo0;koo`006ooo00<00?ooool0@_oo00<07ooo
ool0_Ooo000Kool00`00ooooo`12ool00`0Oooooo`2mool001_oo`03003ooooo04?oo`0301oooooo
0;coo`006ooo00<00?ooool0@ooo00<07oooool0_?oo000Kool20015ool00`0Oooooo`2kool001_o
o`03003ooooo04Coo`0301oooooo0;_oo`006ooo00<00?ooool0A?oo00<07oooool0^ooo000Kool0
0`00ooooo`15ool00`0Oooooo`2jool001_oo`03003ooooo04Goo`0301oooooo0;[oo`006ooo00<0
0?ooool0A_oo00<07oooool0^Ooo000Kool20017ool00`0Oooooo`2iool001_oo`03003ooooo04Ko
o`0301oooooo0;Woo`006ooo00<00?ooool0Aooo00<07oooool0^?oo000Kool00`00ooooo`17ool0
0`0Oooooo`2hool001_oo`03003ooooo04Soo`0301oooooo0;Ooo`006ooo00<00?ooool0B?oo00<0
7oooool0]ooo000Kool20019ool00`0Oooooo`2gool001_oo`03003ooooo04Woo`0301oooooo0;Ko
o`006ooo00<00?ooool0BOoo00<07oooool0]_oo000Kool00`00ooooo`19ool00`0Oooooo`2fool0
00;oo`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo04[oo`0301oooooo0;Goo`00
00Goo`00ooooo`0000Soo`04003ooooo0005ool00`00ooooo`02ool00`00ooooo`1:ool00`0Ooooo
o`2eool00005ool00?ooool00008ool01000ooooo`001Ooo00<00?ooool00_oo0P00Booo00<07ooo
ool0]Ooo00001Ooo003ooooo00002Ooo0P000ooo0`001Ooo00<00?ooool0Booo00<07oooool0]?oo
00001Ooo003ooooo00002?oo00@00?ooool000;oo`03003ooooo00Goo`03003ooooo04_oo`0301oo
oooo0;Coo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`1<
ool00`0Oooooo`2cool000;oo`8000[oo`8000?oo`@000Coo`03003ooooo04coo`0301oooooo0;?o
o`006ooo00<00?ooool0C?oo00<07oooool0/ooo000Kool2001>ool00`0Oooooo`2bool001_oo`03
003ooooo04goo`0301oooooo0;;oo`006ooo00<00?ooool0COoo00<07oooool0/_oo000Kool00`00
ooooo`1>ool00`0Oooooo`2aool001_oo`03003ooooo04koo`0301oooooo0;7oo`006ooo00<00?oo
ool0Cooo00<07oooool0/?oo000Kool2001@ool00`0Oooooo`2`ool001_oo`03003ooooo053oo`03
01oooooo0:ooo`006ooo00<00?ooool0D?oo00<07oooool0[ooo000Kool00`00ooooo`1@ool00`0O
ooooo`2_ool001_oo`03003ooooo057oo`0301oooooo0:koo`006ooo00<00?ooool0DOoo00<07ooo
ool0[_oo000Kool2001Cool00`0Oooooo`2]ool001_oo`03003ooooo05;oo`0301oooooo0:goo`00
6ooo00<00?ooool0Dooo00<07oooool0[?oo000Kool00`00ooooo`1Cool00`0Oooooo`2/ool001_o
o`03003ooooo05?oojP07`Ooo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03
003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo003ooolQool00?ooob7oo`00oooo8Ooo003o
oolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.161627, 0.609006, \
0.00591946, 0.00166747}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %1193, PlotRange \[Rule] {0.6, 0.9}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.634921 -1.23607 2.06011 [
[.15079 -0.0125 -9 -9 ]
[.15079 -0.0125 9 0 ]
[.27778 -0.0125 -9 -9 ]
[.27778 -0.0125 9 0 ]
[.40476 -0.0125 -9 -9 ]
[.40476 -0.0125 9 0 ]
[.53175 -0.0125 -9 -9 ]
[.53175 -0.0125 9 0 ]
[.65873 -0.0125 -3 -9 ]
[.65873 -0.0125 3 0 ]
[.78571 -0.0125 -9 -9 ]
[.78571 -0.0125 9 0 ]
[.9127 -0.0125 -9 -9 ]
[.9127 -0.0125 9 0 ]
[.01131 .10301 -24 -4.5 ]
[.01131 .10301 0 4.5 ]
[.01131 .20601 -18 -4.5 ]
[.01131 .20601 0 4.5 ]
[.01131 .30902 -24 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .41202 -18 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .51503 -24 -4.5 ]
[.01131 .51503 0 4.5 ]
[.01131 .61803 -18 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 0 m
.15079 .00625 L
s
[(0.2)] .15079 -0.0125 0 1 Mshowa
.27778 0 m
.27778 .00625 L
s
[(0.4)] .27778 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(0.6)] .40476 -0.0125 0 1 Mshowa
.53175 0 m
.53175 .00625 L
s
[(0.8)] .53175 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(1)] .65873 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(1.2)] .78571 -0.0125 0 1 Mshowa
.9127 0 m
.9127 .00625 L
s
[(1.4)] .9127 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.18254 0 m
.18254 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.34127 0 m
.34127 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.5 0 m
.5 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.81746 0 m
.81746 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .10301 m
.03006 .10301 L
s
[(0.65)] .01131 .10301 1 0 Mshowa
.02381 .20601 m
.03006 .20601 L
s
[(0.7)] .01131 .20601 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(0.75)] .01131 .30902 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(0.8)] .01131 .41202 1 0 Mshowa
.02381 .51503 m
.03006 .51503 L
s
[(0.85)] .01131 .51503 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.9)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.5 Mabswid
.03016 .05526 m
.03239 .05555 L
.03477 .05589 L
.03703 .05634 L
.03908 .05682 L
.04884 .05999 L
.0537 .06214 L
.05897 .06488 L
.06854 .07093 L
.0903 .08957 L
.11039 .11223 L
.14948 .16869 L
.18706 .23478 L
.22708 .31419 L
.26559 .39695 L
.30654 .48992 L
.327 .53789 L
.33696 .56156 L
.34166 .57277 L
.34599 .58318 L
.34713 .58582 L
.3482 .58826 L
.34931 .58948 L
.35053 .58958 L
.35177 .58955 L
.3529 .58963 L
.35421 .58965 L
.35544 .58966 L
.35652 .58963 L
.35766 .58965 L
.35863 .58965 L
.35971 .58966 L
.36086 .58966 L
.3621 .58966 L
.36328 .58964 L
.36435 .58964 L
.36549 .58965 L
.36671 .58965 L
.36786 .58966 L
.3689 .58966 L
.37013 .58966 L
.37083 .58966 L
.37147 .58966 L
.37264 .58966 L
.37389 .58966 L
.37508 .58966 L
.37636 .58966 L
.37756 .58966 L
.37867 .58965 L
.37996 .58965 L
Mistroke
.38136 .58965 L
.38269 .58966 L
.38393 .58966 L
.38521 .58966 L
.38656 .58966 L
.38772 .58966 L
.38898 .58966 L
.39035 .58966 L
.39181 .58966 L
.39319 .58966 L
.39445 .58966 L
.39572 .58966 L
.39689 .58966 L
.39817 .58966 L
.39954 .58966 L
.40072 .58966 L
.40183 .58966 L
.40304 .58966 L
.40435 .58966 L
.40552 .58966 L
.4066 .58966 L
.40779 .58966 L
.40906 .58966 L
.4097 .58966 L
.41039 .58966 L
.41165 .58965 L
.41288 .58966 L
.41401 .58966 L
.41623 .58966 L
.4183 .58966 L
.42047 .58966 L
.42285 .58966 L
.42542 .58966 L
.42781 .58966 L
.43043 .58966 L
.43321 .58966 L
.43562 .58966 L
.43788 .58966 L
.44036 .58966 L
.44304 .58966 L
.44527 .58966 L
.44772 .58966 L
.45027 .58966 L
.45265 .58966 L
.45691 .58966 L
.45929 .58966 L
.46146 .58966 L
.46627 .58966 L
.46869 .58966 L
.47136 .58966 L
Mistroke
.47364 .58966 L
.47608 .58966 L
.4805 .58966 L
.48314 .58966 L
.48603 .58966 L
.48857 .58966 L
.49128 .58966 L
.4964 .58966 L
.49866 .58966 L
.49986 .58966 L
.50115 .58966 L
.50185 .58966 L
.50249 .58966 L
.50374 .58966 L
.50647 .58966 L
.51137 .58966 L
.51705 .58966 L
.52243 .58966 L
.53203 .58966 L
.53746 .58966 L
.54247 .58966 L
.54708 .58966 L
.55204 .58966 L
.56103 .58966 L
.56623 .58966 L
.57171 .58966 L
.5815 .58966 L
.59099 .58966 L
.60112 .58966 L
.61069 .58966 L
.61937 .58966 L
.6293 .58966 L
.63844 .58966 L
.64848 .58966 L
.65911 .58966 L
.66849 .58966 L
.67856 .58966 L
.69676 .58966 L
.71781 .58966 L
.73778 .58966 L
.77579 .58966 L
.81624 .58966 L
.85518 .58966 L
.89655 .58966 L
.93643 .58966 L
.97479 .58966 L
.97619 .58966 L
Mfstroke
0 1 0 r
.03016 .57814 m
.03309 .57814 L
.0363 .57814 L
.03966 .57815 L
.04278 .57815 L
.04943 .57816 L
.05654 .57818 L
.07008 .57821 L
.08313 .57827 L
.09567 .57833 L
.10902 .5784 L
.12188 .57849 L
.13424 .57859 L
.1474 .5787 L
.16006 .57883 L
.17353 .57898 L
.1865 .57913 L
.19898 .5793 L
.21226 .57949 L
.22504 .57968 L
.23862 .57991 L
.25171 .58015 L
.2643 .58039 L
.2777 .58067 L
.2906 .58097 L
.303 .58126 L
.31621 .58159 L
.32892 .58196 L
.34127 .5823 L
s
0 g
.40476 0 m
.42855 .07719 L
.45249 .15485 L
.47796 .2375 L
.50248 .31706 L
.52854 .40161 L
.55365 .48308 L
.5778 .56147 L
s
.5778 .56147 m
.59524 .61803 L
s
1 0 0 r
.03016 .05521 m
.03307 .05558 L
.03578 .05604 L
.04194 .05753 L
.04795 .05955 L
.05437 .06234 L
.0682 .07047 L
.08077 .08028 L
.1058 .10602 L
.12987 .13762 L
.15548 .17732 L
.18014 .22035 L
.20385 .26531 L
.2291 .31631 L
.25339 .36788 L
.27923 .4249 L
.30411 .48162 L
.32805 .53756 L
.33492 .55387 L
.3414 .5693 L
.34423 .57605 L
.34498 .57774 L
.34578 .5797 L
.34653 .58115 L
.34722 .582 L
.34803 .5824 L
.34879 .58238 L
.34961 .58251 L
.35007 .58255 L
.3505 .58257 L
.3512 .5826 L
.35196 .58259 L
.35276 .58263 L
.35352 .58267 L
.37804 .58352 L
.40228 .58446 L
.42806 .58562 L
.45289 .58693 L
.47678 .58843 L
.50219 .59038 L
.52666 .59281 L
.54004 .5945 L
.55267 .59651 L
.55876 .59769 L
.56521 .59916 L
.57125 .60085 L
.57677 .6028 L
.58269 .60565 L
.58595 .6078 L
.58904 .61054 L
Mistroke
.59042 .6121 L
.59189 .61409 L
.59316 .61616 L
Mfstroke
.59316 .61616 m
.59413 .61803 L
s
.59615 .61803 m
.59615 .61802 L
.59688 .61126 L
.59767 .60958 L
.59838 .60856 L
.59914 .60771 L
.60052 .60652 L
.60213 .60545 L
.60384 .60455 L
.60691 .6033 L
.61021 .60226 L
.61385 .60135 L
.62036 .60009 L
.62647 .59918 L
.62698 .59912 L
s
0 g
.03016 .57814 m
.03651 .57814 L
.04286 .57815 L
.04921 .57816 L
.05556 .57818 L
.0619 .5782 L
.06825 .57822 L
.0746 .57824 L
.08095 .57827 L
.0873 .5783 L
.09365 .57833 L
.1 .57837 L
.10635 .57841 L
.1127 .57846 L
.11905 .5785 L
.1254 .57856 L
.13175 .57861 L
.1381 .57867 L
.14444 .57873 L
.15079 .57879 L
.15714 .57886 L
.16349 .57893 L
.16984 .57901 L
.17619 .57909 L
.18254 .57917 L
.18889 .57926 L
.19524 .57935 L
.20159 .57944 L
.20794 .57954 L
.21429 .57964 L
.22063 .57975 L
.22698 .57986 L
.23333 .57998 L
.23968 .58009 L
.24603 .58022 L
.25238 .58035 L
.25873 .58048 L
.26508 .58061 L
.27143 .58075 L
.27778 .5809 L
.28413 .58081 L
.29048 .58121 L
.29683 .58137 L
.30317 .58153 L
.30952 .58171 L
.31587 .58188 L
.32222 .58207 L
.32857 .58226 L
.33492 .58244 L
.34127 .58265 L
Mistroke
.34762 .58314 L
.35397 .58308 L
.37302 .58376 L
.37937 .58357 L
.38571 .58425 L
.39206 .58451 L
.39841 .58478 L
.40476 .58506 L
.41111 .58535 L
.41746 .58513 L
.42381 .58596 L
.43016 .58632 L
.43651 .58661 L
.44286 .58696 L
.44921 .58732 L
.45556 .5877 L
.4619 .5881 L
.46825 .58851 L
.4746 .58894 L
.48095 .58939 L
.4873 .58986 L
.49365 .59037 L
.5 .5909 L
.50635 .59146 L
.5127 .59206 L
.51905 .5927 L
.5254 .59339 L
.53175 .59413 L
.5381 .59494 L
.54444 .59583 L
.55079 .59682 L
.55714 .59792 L
.56349 .59918 L
.56984 .60064 L
.57619 .60234 L
.58254 .60431 L
.58889 .60635 L
.59524 .6071 L
.60159 .60504 L
.60794 .60293 L
.61429 .60142 L
.62063 .60029 L
.62698 .5994 L
.63333 .59868 L
.63968 .59807 L
.64603 .59755 L
.65238 .5971 L
.65873 .5967 L
.66508 .59635 L
.67143 .59603 L
Mistroke
.67778 .59574 L
.68413 .59548 L
.69048 .59524 L
.69683 .59502 L
.70317 .59481 L
.70952 .59463 L
.71587 .59445 L
.72222 .59429 L
.72857 .59413 L
.73492 .59399 L
.74127 .59386 L
.74762 .59373 L
.75397 .59361 L
.76032 .5935 L
.76667 .59339 L
.77302 .59329 L
.77937 .59319 L
.78571 .5931 L
.79206 .59301 L
.79841 .59293 L
.80476 .59285 L
.81111 .59277 L
.81746 .5927 L
.82381 .59263 L
.83016 .59256 L
.83651 .59249 L
.84286 .59243 L
.84921 .59237 L
.85556 .59232 L
.8619 .59226 L
.86825 .59221 L
.8746 .59216 L
.88095 .59211 L
.8873 .59206 L
.89365 .59201 L
.9 .59197 L
.90635 .59193 L
.9127 .59188 L
.91905 .59184 L
.9254 .59181 L
.93175 .59177 L
.9381 .59173 L
.94444 .5917 L
.95079 .59166 L
.95714 .59163 L
.96349 .5916 L
.96984 .59156 L
.97619 .59153 L
Mfstroke
0 1 0 r
.03016 .57814 m
.03309 .57814 L
.0363 .57814 L
.03966 .57815 L
.04278 .57815 L
.04943 .57816 L
.05654 .57818 L
.07008 .57821 L
.08313 .57827 L
.09567 .57833 L
.10902 .5784 L
.12188 .57849 L
.13424 .57859 L
.1474 .5787 L
.16006 .57883 L
.17353 .57898 L
.1865 .57913 L
.19898 .5793 L
.21226 .57949 L
.22504 .57968 L
.23862 .57991 L
.25171 .58015 L
.2643 .58039 L
.2777 .58067 L
.2906 .58097 L
.303 .58126 L
.31621 .58159 L
.32892 .58196 L
.34127 .5823 L
s
0 g
.40476 0 m
.42855 .07719 L
.45249 .15485 L
.47796 .2375 L
.50248 .31706 L
.52854 .40161 L
.55365 .48308 L
.5778 .56147 L
s
.5778 .56147 m
.59524 .61803 L
s
1 0 0 r
.03016 .05521 m
.03307 .05558 L
.03578 .05604 L
.04194 .05753 L
.04795 .05955 L
.05437 .06234 L
.0682 .07047 L
.08077 .08028 L
.1058 .10602 L
.12987 .13762 L
.15548 .17732 L
.18014 .22035 L
.20385 .26531 L
.2291 .31631 L
.25339 .36788 L
.27923 .4249 L
.30411 .48162 L
.32805 .53756 L
.33492 .55387 L
.3414 .5693 L
.34423 .57605 L
.34498 .57774 L
.34578 .5797 L
.34653 .58115 L
.34722 .582 L
.34803 .5824 L
.34879 .58238 L
.34961 .58251 L
.35007 .58255 L
.3505 .58257 L
.3512 .5826 L
.35196 .58259 L
.35276 .58263 L
.35352 .58267 L
.37804 .58352 L
.40228 .58446 L
.42806 .58562 L
.45289 .58693 L
.47678 .58843 L
.50219 .59038 L
.52666 .59281 L
.54004 .5945 L
.55267 .59651 L
.55876 .59769 L
.56521 .59916 L
.57125 .60085 L
.57677 .6028 L
.58269 .60565 L
.58595 .6078 L
.58904 .61054 L
Mistroke
.59042 .6121 L
.59189 .61409 L
.59316 .61616 L
Mfstroke
.59316 .61616 m
.59413 .61803 L
s
.59615 .61803 m
.59615 .61802 L
.59688 .61126 L
.59767 .60958 L
.59838 .60856 L
.59914 .60771 L
.60052 .60652 L
.60213 .60545 L
.60384 .60455 L
.60691 .6033 L
.61021 .60226 L
.61385 .60135 L
.62036 .60009 L
.62647 .59918 L
.62698 .59912 L
s
0 g
.03016 .57814 m
.03651 .57814 L
.04286 .57815 L
.04921 .57816 L
.05556 .57818 L
.0619 .5782 L
.06825 .57822 L
.0746 .57824 L
.08095 .57827 L
.0873 .5783 L
.09365 .57833 L
.1 .57837 L
.10635 .57841 L
.1127 .57846 L
.11905 .5785 L
.1254 .57856 L
.13175 .57861 L
.1381 .57867 L
.14444 .57873 L
.15079 .57879 L
.15714 .57886 L
.16349 .57893 L
.16984 .57901 L
.17619 .57909 L
.18254 .57917 L
.18889 .57926 L
.19524 .57935 L
.20159 .57944 L
.20794 .57954 L
.21429 .57964 L
.22063 .57975 L
.22698 .57986 L
.23333 .57998 L
.23968 .58009 L
.24603 .58022 L
.25238 .58035 L
.25873 .58048 L
.26508 .58061 L
.27143 .58075 L
.27778 .5809 L
.28413 .58081 L
.29048 .58121 L
.29683 .58137 L
.30317 .58153 L
.30952 .58171 L
.31587 .58188 L
.32222 .58207 L
.32857 .58226 L
.33492 .58244 L
.34127 .58265 L
Mistroke
.34762 .58314 L
.35397 .58308 L
.37302 .58376 L
.37937 .58357 L
.38571 .58425 L
.39206 .58451 L
.39841 .58478 L
.40476 .58506 L
.41111 .58535 L
.41746 .58513 L
.42381 .58596 L
.43016 .58632 L
.43651 .58661 L
.44286 .58696 L
.44921 .58732 L
.45556 .5877 L
.4619 .5881 L
.46825 .58851 L
.4746 .58894 L
.48095 .58939 L
.4873 .58986 L
.49365 .59037 L
.5 .5909 L
.50635 .59146 L
.5127 .59206 L
.51905 .5927 L
.5254 .59339 L
.53175 .59413 L
.5381 .59494 L
.54444 .59583 L
.55079 .59682 L
.55714 .59792 L
.56349 .59918 L
.56984 .60064 L
.57619 .60234 L
.58254 .60431 L
.58889 .60635 L
.59524 .6071 L
.60159 .60504 L
.60794 .60293 L
.61429 .60142 L
.62063 .60029 L
.62698 .5994 L
.63333 .59868 L
.63968 .59807 L
.64603 .59755 L
.65238 .5971 L
.65873 .5967 L
.66508 .59635 L
.67143 .59603 L
Mistroke
.67778 .59574 L
.68413 .59548 L
.69048 .59524 L
.69683 .59502 L
.70317 .59481 L
.70952 .59463 L
.71587 .59445 L
.72222 .59429 L
.72857 .59413 L
.73492 .59399 L
.74127 .59386 L
.74762 .59373 L
.75397 .59361 L
.76032 .5935 L
.76667 .59339 L
.77302 .59329 L
.77937 .59319 L
.78571 .5931 L
.79206 .59301 L
.79841 .59293 L
.80476 .59285 L
.81111 .59277 L
.81746 .5927 L
.82381 .59263 L
.83016 .59256 L
.83651 .59249 L
.84286 .59243 L
.84921 .59237 L
.85556 .59232 L
.8619 .59226 L
.86825 .59221 L
.8746 .59216 L
.88095 .59211 L
.8873 .59206 L
.89365 .59201 L
.9 .59197 L
.90635 .59193 L
.9127 .59188 L
.91905 .59184 L
.9254 .59181 L
.93175 .59177 L
.9381 .59173 L
.94444 .5917 L
.95079 .59166 L
.95714 .59163 L
.96349 .5916 L
.96984 .59156 L
.97619 .59153 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool003Soo`8000Coo`03003ooooo00;oo`@001;oo`8000Coo`03003ooooo00Coo`03003ooooo017o
o`8000Coo`03003ooooo00?oo`8001?oo`8000Coo`03003ooooo00?oo`8001Soo`@001Ooo`@000?o
o`03003ooooo00;oo`@0017oo`@000?oo`03003ooooo00Coo`03003ooooo01;oo`00=ooo00@00?oo
ool000Soo`03003ooooo01;oo`04003ooooo000:ool00`00ooooo`0@ool01000ooooo`002?oo00@0
0?ooool0017oo`04003ooooo0008ool01000ooooo`006Ooo00<00?ooool06?oo00<00?ooool01ooo
00<00?ooool05?oo00<00?ooool02Ooo00<00?ooool04_oo000gool01000ooooo`002Ooo00<00?oo
ool04Ooo00@00?ooool000Ooo`D0017oo`04003ooooo0008ool01000ooooo`004Ooo00@00?ooool0
00Soo`04003ooooo000Iool00`00ooooo`0Hool00`00ooooo`08ool00`00ooooo`0Cool00`00oooo
o`06ool5000Cool003Ooo`04003ooooo000:ool00`00ooooo`0@ool01000ooooo`001ooo00@00?oo
ool001;oo`04003ooooo0008ool3000Bool01000ooooo`002Ooo0P006_oo00<00?ooool06?oo00<0
0?ooool02Ooo00<00?ooool04_oo00<00?ooool01_oo00@00?ooool001Coo`00=ooo00@00?ooool0
00_oo`03003ooooo00ooo`04003ooooo0008ool00`00ool0000Bool01000ooooo`002?oo00<00?oo
ool04_oo00@00?ooool000Soo`04003ooooo000Iool00`00ooooo`0Hool00`00ooooo`0:ool00`00
ooooo`0Aool00`00ooooo`07ool00`00ool0000Dool003Ooo`04003ooooo0008ool01000ooooo`00
4Ooo00@00?ooool000Woo`8001;oo`04003ooooo0008ool00`00ooooo`0Bool01000ooooo`002?oo
00@00?ooool001Ooo`<001Soo`<000Woo`04003ooooo000Aool3000:ool2000Dool003Soo`8000[o
o`8001?oo`8000_oo`03003ooooo017oo`8000[oo`<001;oo`8000[oo`8001[oo`03003ooooo01So
o`03003ooooo00Soo`8001Coo`03003ooooo00Woo`03003ooooo01;oo`00oooo8Ooo003ooolQool0
0?ooob7oo`006?ooo`001@001?oo000Nool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`08ool001koo`03003ooooo067oo`03003o
oooo09_oo`007_oo00<00?ooool0HOoo00<00?ooool0Vooo000Nool00`00ooooo`1Qool00`00oooo
o`2Kool001koo`8006;oo`03003ooooo09_oo`007_oo00<00?ooool0H_oo00<00?ooool0V_oo000N
ool00`00ooooo`1Rool00`00ooooo`2Jool001koo`03003ooooo06;oo`03003ooooo09[oo`007_oo
00<00?ooool0Hooo00<00?ooool0VOoo000Nool2001Tool00`00ooooo`2Iool001koo`03003ooooo
06?oo`03003ooooo09Woo`007_oo00<00?ooool0I?oo00<00?ooool0V?oo000Nool00`00ooooo`1T
ool00`00ooooo`2Hool001koo`05003oooP0n0007`1Rool00`00ooooo`2Hool001koo`04003ooooo
ool3n0000`0Oooooo`1Mool00`00ooooo`2Hool001koo`8000Goo`;h05ooo`03003ooooo09Ooo`00
7_oo00<00?ooool01_oo0_P0GOoo00<00?ooool0Uooo000Nool00`00ooooo`08ool2n01Kool00`00
ooooo`2Gool001koo`03003ooooo00[oo`03n03ooooo05Woo`03003ooooo09Koo`007_oo00<00?oo
ool02ooo00?h0?ooool0F?oo00<00?ooool0U_oo000Nool2000=ool00oP0ooooo`1Gool00`00oooo
o`2Fool001koo`03003ooooo00goo`03n03ooooo05Ooo`03003ooooo09Goo`007_oo00<00?ooool0
3_oo00?h0?ooool0E_oo00<00?ooool0UOoo000Nool00`00ooooo`0?ool00oP0ooooo`1Eool00`00
ooooo`2Eool000Goo`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo013oo`03n03o
oooo05Coo`03003ooooo09Goo`001?oo00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`02
ool00`00ooooo`0Aool00oP0ooooo`1Dool00`00ooooo`2Dool000Coo`04003ooooo0008ool01000
ooooo`001Ooo00<00?ooool00_oo0`004_oo00?h0?ooool0Dooo00<00?ooool0U?oo0004ool01000
ooooo`002?oo0`000ooo0`001Ooo00<00?ooool04ooo00?h0?ooool0D_oo00<00?ooool0U?oo0004
ool01000ooooo`002?oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool05?oo00?h0?ooool0
D_oo00<00?ooool0Tooo0004ool01000ooooo`002?oo00<00?ooool00ooo00<00?ooool01Ooo00<0
0?ooool05Ooo00?h0?ooool0DOoo00<00?ooool0Tooo0005ool2000:ool30002ool40004ool00`00
ooooo`0Eool00oP0ooooo`1Aool00`00ooooo`2Cool001koo`8001Ooo`03n03ooooo057oo`03003o
oooo09;oo`007_oo00<00?ooool05ooo00?h0?ooool0D?oo00<00?ooool0T_oo000Nool00`00oooo
o`0Gool00`0On03oo`1@ool00`00ooooo`2Bool001koo`03003ooooo01Soo`0301oh0?oo04ooo`03
003ooooo09;oo`007_oo00<00?ooool06Ooo00<07oP0ool0Cooo00<00?ooool0TOoo000Nool2000J
ool00`0On03oo`1?ool00`00ooooo`2Aool001koo`03003ooooo01[oo`0301oh0?oo04koo`03003o
oooo097oo`007_oo00<00?ooool06ooo00?h0?ooool0C_oo00<00?ooool0T?oo000Nool00`00oooo
o`0Kool00`0On03oo`1>ool00`00ooooo`2@ool001koo`03003ooooo01coo`03n03ooooo04goo`03
003ooooo093oo`007_oo00<00?ooool07Ooo00?h0?ooool0COoo00<00?ooool0Sooo000Nool2000N
ool00oP0ooooo`1=ool00`00ooooo`2?ool001koo`03003ooooo01koo`03n03ooooo04coo`03003o
oooo08ooo`007_oo00<00?ooool07_oo00?h000Oool0COoo00<00?ooool0S_oo000Nool00`00oooo
o`0Oool00oP0ooooo`1<ool00`00ooooo`2>ool001koo`03003ooooo023oo`03n03ooooo04_oo`03
003ooooo08koo`007_oo0P008Ooo00?h0?ooool0C?oo00<00?ooool0SOoo000Nool00`00ooooo`0Q
ool00oP0ooooo`1;ool00`00ooooo`2=ool001koo`03003ooooo027oo`0301oh0?oo04_oo`03003o
oooo08goo`002ooo0P001?oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool08_oo00?h0?oo
ool0Booo00<00?ooool0S?oo000:ool01000ooooo`002Ooo00<00?ooool01?oo00<00?ooool08_oo
00<07oP0ool0Booo00<00?ooool0S?oo000:ool01000ooooo`002_oo00<00?ooool00ooo0`008ooo
00?h0?ooool0B_oo00<00?ooool0S?oo000:ool01000ooooo`002_oo00<00?ooool00ooo00<00?oo
ool08ooo00<07oP0ool0Booo00<00?ooool0Rooo000:ool01000ooooo`002ooo00<00?ooool00_oo
00<00?ooool09?oo00<07oP0ool0B_oo00<00?ooool0Rooo000:ool01000ooooo`002?oo00@00?oo
ool000Coo`03003ooooo02Coo`0301oh0?oo04[oo`03003ooooo08_oo`002ooo0P002Ooo10001?oo
00<00?ooool09Ooo00<07oP0ool0B_oo00<00?ooool0R_oo000Nool00`00ooooo`0Uool00`0On03o
o`1:ool00`00ooooo`2:ool001koo`8002Ooo`0301oh0?oo04Woo`03003ooooo08[oo`007_oo00<0
0?ooool09_oo00<07oP0ool0B_oo00<00?ooool0ROoo000Nool00`00ooooo`0Wool00`0On03oo`19
ool00`00ooooo`29ool001koo`03003ooooo02Soo`03n03ooooo04Soo`03003ooooo08Woo`007_oo
00<00?ooool0:?oo00<07oP0ool0BOoo00<00?ooool0R?oo000Nool2000Zool00oP0ooooo`18ool0
0`00ooooo`28ool001koo`03003ooooo02Woo`0301oh0?oo04Soo`03003ooooo08Soo`007_oo00<0
0?ooool0:_oo00?h0?ooool0Aooo00<00?ooool0R?oo000Nool00`00ooooo`0Zool00`0On03oo`18
ool00`00ooooo`27ool001koo`03003ooooo02_oo`03n03ooooo04Ooo`03003ooooo08Ooo`007_oo
0P00;?oo00<07oP0ool0Aooo00<00?ooool0Qooo000Nool00`00ooooo`0/ool00oP0ooooo`17ool0
0`00ooooo`26ool001koo`03003ooooo02coo`0301oh0?oo04Ooo`03003ooooo08Koo`007_oo00<0
0?ooool0;Ooo00?h0?ooool0A_oo00<00?ooool0Q_oo000Nool00`00ooooo`0^ool00oP0ooooo`15
ool00`00ooooo`26ool001koo`03003ooooo02koo`03n03ooooo04Koo`03003ooooo08Goo`007_oo
0P00<?oo00?h0?ooool0AOoo00<00?ooool0QOoo000Nool00`00ooooo`0_ool00oP0ooooo`15ool0
0`00ooooo`25ool001koo`03003ooooo033oo`03n03ooooo04Goo`03003ooooo08Coo`001Ooo0P00
1?oo00<00?ooool00ooo00<00?ooool00_oo0`001Ooo00<00?ooool0<?oo00?h0?ooool0AOoo00<0
0?ooool0Q?oo0004ool01000ooooo`002Ooo00<00?ooool01Ooo00<00?ooool00_oo00<00?ooool0
<Ooo00?h0?ooool0A?oo00<00?ooool0Q?oo0004ool01000ooooo`002_oo00<00?ooool01?oo00<0
0?ooool00_oo0`00<Ooo00?h0?ooool0A?oo00<00?ooool0Q?oo0004ool01000ooooo`002_oo00@0
0?ooooooo`<000Goo`03003ooooo03;oo`03n03ooooo04Coo`03003ooooo08?oo`001?oo00@00?oo
ool000_oo`04003ooooo0007ool00`00ooooo`0bool00oP0ooooo`14ool00`00ooooo`23ool000Co
o`04003ooooo0008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0<ooo00?h0?ooool0
@ooo00<00?ooool0Pooo0005ool20009ool40002ool40004ool00`00ooooo`0cool00oP0ooooo`14
ool00`00ooooo`22ool001koo`8003Goo`03n03ooooo04?oo`03003ooooo08;oo`007_oo00<00?oo
ool0=?oo00?h0?ooool0@ooo00<00?ooool0P_oo000Nool00`00ooooo`0eool00oP0ooooo`13ool0
0`00ooooo`21ool001koo`03003ooooo03Goo`03n03ooooo04?oo`03003ooooo087oo`007_oo00<0
0?ooool0=_oo00?h0?ooool0@_oo00<00?ooool0POoo000Nool00`00ooooo`0fool00oP0ooooo`13
ool00`00ooooo`20ool001koo`8003Soo`03n03ooooo04;oo`03003ooooo083oo`007_oo00<00?oo
ool0=ooo00?h0?ooool0@_oo00<00?ooool0P?oo000Nool00`00ooooo`0gool00`0On03oo`12ool0
0`00ooooo`20ool001koo`03003ooooo03Soo`03n03ooooo04;oo`03003ooooo07ooo`007_oo00<0
0?ooool0>?oo00<07oP0ool0@_oo00<00?ooool0Oooo000Nool2000jool00oP0ooooo`11ool00`00
ooooo`1oool001koo`03003ooooo03Woo`0301oh0?oo04;oo`03003ooooo07koo`007_oo00<00?oo
ool0>_oo00?h0?ooool0@Ooo00<00?ooool0O_oo000Nool00`00ooooo`0jool00`0On03oo`11ool0
0`00ooooo`1nool001koo`03003ooooo03_oo`03n03ooooo047oo`03003ooooo07goo`007_oo0P00
??oo00?h0?ooool0@Ooo00<00?ooool0OOoo000Nool00`00ooooo`0lool00oP0ooooo`10ool00`00
ooooo`1mool001koo`03003ooooo03coo`03n03ooooo047oo`03003ooooo07coo`007_oo00<00?oo
ool0??oo00<07oP0ool0@Ooo00<00?ooool0O?oo000;ool20004ool00`00ooooo`03ool20005ool0
0`00ooooo`0mool00oP0ooooo`10ool00`00ooooo`1lool000[oo`04003ooooo0008ool01000oooo
o`001?oo00<00?ooool0?Ooo00?h0?ooool0@Ooo00<00?ooool0Nooo000:ool01000ooooo`002?oo
00@00?ooool000Coo`<003koo`03n03ooooo043oo`03003ooooo07_oo`002_oo00@00?ooool000Wo
o`8000Goo`03003ooooo03koo`03n03ooooo043oo`03003ooooo07_oo`002_oo00@00?ooool000So
o`04003ooooo0004ool00`00ooooo`0nool00`0On03oo`10ool00`00ooooo`1kool000[oo`04003o
oooo0008ool01000ooooo`001?oo00<00?ooool0?ooo00?h0?ooool0@?oo00<00?ooool0N_oo000;
ool2000:ool20005ool00`00ooooo`0oool00oP0ooooo`10ool00`00ooooo`1jool001koo`80047o
o`03n03ooooo03ooo`03003ooooo07[oo`007_oo00<00?ooool0@?oo00?h0?ooool0@?oo00<00?oo
ool0NOoo000Nool00`00ooooo`11ool00oP0ooooo`0oool00`00ooooo`1iool001koo`03003ooooo
047oo`03n03ooooo03ooo`03003ooooo07Woo`007_oo00<00?ooool0@Ooo00<07oP0ool0?ooo00<0
0?ooool0NOoo000Nool20013ool00oP0ooooo`0oool00`00ooooo`1hool001koo`03003ooooo04;o
o`0301oh0?oo03ooo`03003ooooo07Soo`007_oo00<00?ooool0@ooo00?h0?ooool0?_oo00<00?oo
ool0N?oo000Nool00`00ooooo`13ool00`0On03oo`0oool00`00ooooo`1gool001koo`03003ooooo
04?oo`0301oh0?oo03ooo`03003ooooo07Ooo`007_oo00<00?ooool0A?oo00<07oP0ool0?_oo00<0
0?ooool0Mooo000Nool20015ool00`0On03oo`0nool00`00ooooo`1gool001koo`03003ooooo04Go
o`0301oh0?oo03koo`03003ooooo07Koo`007_oo00<00?ooool0AOoo00<07oP0ool0?_oo00<00?oo
ool0M_oo000Nool00`00ooooo`16ool00oP0ooooo`0mool00`00ooooo`1fool001koo`03003ooooo
04Koo`0301oh0?oo03koo`03003ooooo07Goo`007_oo0P00Aooo00<07oP0ool0?_oo00<00?ooool0
MOoo000Nool00`00ooooo`17ool00`0On03oo`0mool00`00ooooo`1eool001koo`03003ooooo04Oo
o`0301oh0?oo03koo`03003ooooo07Coo`001Ooo0P001?oo00<00?ooool00ooo0P000ooo0`001Ooo
00<00?ooool0B?oo00<07oP0ool0?Ooo00<00?ooool0M?oo0004ool01000ooooo`002?oo00@00?oo
ool000Goo`03003ooooo00;oo`03003ooooo04Soo`0301oh0?oo03goo`03003ooooo07Coo`001?oo
00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`02ool30019ool00oP0ooooo`0mool00`00
ooooo`1cool000Coo`04003ooooo0009ool20003ool30005ool00`00ooooo`19ool00`0On03oo`0m
ool00`00ooooo`1cool000Coo`04003ooooo0008ool01000ooooo`000_oo00<00?ooool01Ooo00<0
0?ooool0B_oo00?h0?ooool0??oo00<00?ooool0Looo0004ool01000ooooo`002?oo00@00?ooool0
00;oo`03003ooooo00Goo`03003ooooo04[oo`0301oh0?oo03goo`03003ooooo07;oo`001Ooo0P00
2_oo0P000ooo10001?oo00<00?ooool0Booo00?h0?ooool0??oo00<00?ooool0L_oo000Nool00`00
ooooo`1;ool00`0On03oo`0mool00`00ooooo`1aool001koo`8004goo`03n03ooooo03coo`03003o
oooo077oo`007_oo00<00?ooool0C?oo00?h0?ooool0??oo00<00?ooool0LOoo000Nool00`00oooo
o`1=ool00oP0ooooo`0lool00`00ooooo`1`ool001koo`03003ooooo04goo`03n03ooooo03coo`03
003ooooo073oo`007_oo00<00?ooool0COoo00<07oP0ool0??oo00<00?ooool0L?oo000Nool2001?
ool00oP0ooooo`0lool00`00ooooo`1_ool001koo`03003ooooo04koo`03n03ooooo03coo`03003o
oooo06ooo`007_oo00<00?ooool0C_oo00<07oP0ool0??oo00<00?ooool0Kooo000Nool00`00oooo
o`1?ool00oP0ooooo`0lool00`00ooooo`1^ool001koo`03003ooooo04ooo`03n03ooooo03coo`03
003ooooo06koo`007_oo0P00DOoo00?h0?ooool0>ooo00<00?ooool0K_oo000Nool00`00ool00015
00021n09ool2n00lool00`00ooooo`1^ool001koo`03003ooooo04Gooa`00003n000000002_oo`03
003ooooo06goo`007_oo00<00?ooool0DOoo00<07oooool03Ooo00<00?oon0003`000oP06Ooo00<0
0?ooool0KOoo000Nool00`00ooooo`1AoolR01l;0003n00>01l00`0001l07`0c01l`000:ool001ko
o`03003ooooo07koo`T000Soo`03003ooooo00ooob@003[oo`007_oo0P00R?oo1@001?oo00<00?oo
ool01?oo2P00G_oo000Nool00`00ooooo`2:ool2n00300000ooo003oo`02ool3001Xool001koo`03
003ooooo08goo`;h00D006_oo`002ooo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0T?oo
00?h0?oon000K?oo000:ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0T?oo00?h0000
n000K?oo000:ool01000ooooo`002ooo00<00?ooool00_oo0`00TOoo0_P0K?oo000:ool01000oooo
o`002Ooo0`00oooo1ooo000:ool01000ooooo`002?oo00@00?ooool00?ooo`Ooo`002_oo00@00?oo
ool000Soo`04003ooooo003oool7ool000_oo`8000[oo`800?ooo`Soo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.185957, 0.577078, \
0.00607629, 0.0018727}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["approximate solutions (old version)", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?kx0corrtestfunc\)\)], "Input"],

Cell["Global`kx0corrtestfunc", "Print",
  CellTags->"Info3279609948-4320224"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(kx0corrtestfunc[om_, k_, 
                      a_] := \((om\^2 - k\^2 - 
                          alphakx0korr[om\/k, a])\)\ \((1 - 
                          betakx0korr[om\/k, a]\/om\^2)\)\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "kx0corrtestfunc"],
      Editable->False]], "Print",
  CellTags->"Info3279609948-4320224"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Series[
      kx0corrtestfunc[Om + I\ mGamma, k, a], {mGamma, 0, 2}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(\((\(-k\^2\) + 
              Om\^2 + \(\(1\/\(8\ \@a\ \((1 + \(a\ Om\^2\)\/k\^2)\)\^2\)\)\((\
\[ImaginaryI]\ \@\(1 + a\)\ \((\((\(-1\) - Om\^2\/k\^2 + 
                            a\ \((1 - \(\((6 + a)\)\ Om\^2\)\/k\^2 - \
\(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                          1 - \[ImaginaryI]\ \@a] + \((1 + Om\^2\/k\^2 + 
                            a\ \((\(-1\) + \(\((6 + a)\)\ Om\^2\)\/k\^2 + \
\(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                          1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \
\((\(-1\) + 
                            Om\^2\/k\^2)\)\ \((2 + \(2\ a\ Om\^2\)\/k\^2 + \
\(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + Om\/k] - Log[1 + \
Om\/k])\)\)\/k)\))\))\)\))\)\ \((1 - \(\(1\/\(4\ \@a\ k\^2\ \((1 + \(a\ Om\^2\
\)\/k\^2)\)\^2\)\)\((\@\(1 + a\)\ \((\[ImaginaryI]\ \((1 + 
                            a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ Log[
                          1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                            a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ Log[
                          1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + \
\(a\ Om\^2\)\/k\^2)\) + \(2\ \((1 + a)\)\ Om\ \((\(-Log[\(-1\) + Om\/k]\) + \
Log[1 + Om\/k])\)\)\/k)\))\))\)\))\)\), 
        "+", \(\((\((2\ \[ImaginaryI]\ Om + \(\(1\/\(8\ \@a\)\)\((\
\[ImaginaryI]\ \@\(1 + a\)\ \((\(\(1\/\((1 + \(a\ Om\^2\)\/k\^2)\)\^2\)\((\((\
\(-\(\(2\ \[ImaginaryI]\ Om\)\/k\^2\)\) + 
                                      a\ \((\(-\(\(2\ \[ImaginaryI]\ \((6 + 
                                        a)\)\ Om\)\/k\^2\)\) - \(4\ \
\[ImaginaryI]\ \((\(-1\) + a)\)\ Om\^3\)\/k\^4)\))\)\ Log[
                                    1 - \[ImaginaryI]\ \@a] + \((\(2\ \
\[ImaginaryI]\ Om\)\/k\^2 + 
                                      a\ \((\(2\ \[ImaginaryI]\ \((6 + a)\)\ \
Om\)\/k\^2 + \(4\ \[ImaginaryI]\ \((\(-1\) + a)\)\ Om\^3\)\/k\^4)\))\)\ Log[
                                    1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \
\@a\ \((\((\(-1\) + 
                                        Om\^2\/k\^2)\)\ \((\(4\ \[ImaginaryI]\
\ a\ Om\)\/k\^2 + \(2\ \((1 + a)\)\ \((Om\ \((\[ImaginaryI]\/\(k\ \((\(-1\) + \
Om\/k)\)\) - \[ImaginaryI]\/\(k\ \((1 + Om\/k)\)\))\) + \[ImaginaryI]\ \
\((Log[\(-1\) + Om\/k] - Log[1 + Om\/k])\))\)\)\/k)\) + \(2\ \[ImaginaryI]\ \
Om\ \((2 + \(2\ a\ Om\^2\)\/k\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + \
Om\/k] - Log[1 + Om\/k])\)\)\/k)\)\)\/k\^2)\))\)\) - \(\(1\/\(k\^2\ \((1 + \
\(a\ Om\^2\)\/k\^2)\)\^3\)\)\((4\ \[ImaginaryI]\ a\ Om\ \((\((\(-1\) - 
                                        Om\^2\/k\^2 + 
                                        a\ \((1 - \(\((6 + a)\)\ \
Om\^2\)\/k\^2 - \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                        1 - \[ImaginaryI]\ \@a] + \((1 + 
                                        Om\^2\/k\^2 + 
                                        a\ \((\(-1\) + \(\((6 + a)\)\ Om\^2\)\
\/k\^2 + \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                        1 + \[ImaginaryI]\ \@a] + \
\[ImaginaryI]\ \@a\ \((\(-1\) + 
                                        Om\^2\/k\^2)\)\ \((2 + \(2\ a\ \
Om\^2\)\/k\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + Om\/k] - Log[1 + \
Om\/k])\)\)\/k)\))\))\)\))\))\)\))\)\ \((1 - \(\(1\/\(4\ \@a\ k\^2\ \((1 + \
\(a\ Om\^2\)\/k\^2)\)\^2\)\)\((\@\(1 + a\)\ \((\[ImaginaryI]\ \((1 + 
                                  a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                                  a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + \(a\ Om\^2\)\/k\^2)\) + \
\(2\ \((1 + a)\)\ Om\ \((\(-Log[\(-1\) + Om\/k]\) + Log[1 + \
Om\/k])\)\)\/k)\))\))\)\))\) + \(\(1\/\(4\ \@a\ k\^2\)\)\((\@\(1 + a\)\ \
\((\(-k\^2\) + 
                      Om\^2 + \(\(1\/\(8\ \@a\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^2\)\)\((\[ImaginaryI]\ \@\(1 + a\)\ \((\((\(-1\) - 
                                    Om\^2\/k\^2 + 
                                    a\ \((1 - \(\((6 + a)\)\ Om\^2\)\/k\^2 - \
\(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                  1 - \[ImaginaryI]\ \@a] + \((1 + 
                                    Om\^2\/k\^2 + 
                                    a\ \((\(-1\) + \(\((6 + a)\)\ \
Om\^2\)\/k\^2 + \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \
\@a\ \((\(-1\) + 
                                    Om\^2\/k\^2)\)\ \((2 + \(2\ a\ Om\^2\)\/k\
\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + Om\/k] - Log[1 + \
Om\/k])\)\)\/k)\))\))\)\))\)\ \((\(\(1\/\(k\^2\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^3\)\)\((4\ \[ImaginaryI]\ a\ Om\ \((\[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + \(a\ Om\^2\)\/k\^2)\) + \
\(2\ \((1 + a)\)\ Om\ \((\(-Log[\(-1\) + Om\/k]\) + Log[1 + \
Om\/k])\)\)\/k)\))\))\)\) - \(\(1\/\((1 + \(a\ \
Om\^2\)\/k\^2)\)\^2\)\((\(-\(\(2\ a\ \((1 + a)\)\ Om\ Log[
                                    1 - \[ImaginaryI]\ \@a]\)\/k\^2\)\) + \(2\
\ a\ \((1 + a)\)\ Om\ Log[1 + \[ImaginaryI]\ \@a]\)\/k\^2 + \@a\ \((\(-\(\(4\ \
\[ImaginaryI]\ a\ Om\)\/k\^2\)\) + \(2\ \((1 + a)\)\ \((Om\ \((\(-\(\
\[ImaginaryI]\/\(k\ \((\(-1\) + Om\/k)\)\)\)\) + \[ImaginaryI]\/\(k\ \((1 + \
Om\/k)\)\))\) + \[ImaginaryI]\ \((\(-Log[\(-1\) + Om\/k]\) + Log[1 + \
Om\/k])\))\)\)\/k)\))\)\))\))\)\))\)\ mGamma\), 
        "+", \(\((\((\(-1\) + \(\(1\/\(8\ \@a\)\)\((\[ImaginaryI]\ \@\(1 + \
a\)\ \((\(\(1\/\((1 + \(a\ Om\^2\)\/k\^2)\)\^2\)\((\((1\/k\^2 + 
                                      a\ \((\(-\(\(\(-6\) - 
                                        a\)\/k\^2\)\) + \(6\ \((\(-1\) + a)\)\
\ Om\^2\)\/k\^4)\))\)\ Log[1 - \[ImaginaryI]\ \@a] + \((\(-\(1\/k\^2\)\) + 
                                      a\ \((\(\(-6\) - a\)\/k\^2 - \(6\ \
\((\(-1\) + a)\)\ Om\^2\)\/k\^4)\))\)\ Log[
                                    1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \
\@a\ \((\((\(-1\) + 
                                        Om\^2\/k\^2)\)\ \((\(-\(\(2\ \
a\)\/k\^2\)\) + \(2\ \((1 + a)\)\ \((Om\ \((1\/\(2\ k\^2\ \((\(-1\) + \
Om\/k)\)\^2\) - 1\/\(2\ k\^2\ \((1 + Om\/k)\)\^2\))\) + \[ImaginaryI]\ \((\
\[ImaginaryI]\/\(k\ \((\(-1\) + Om\/k)\)\) - \[ImaginaryI]\/\(k\ \((1 + \
Om\/k)\)\))\))\)\)\/k)\) + \(2\ \[ImaginaryI]\ Om\ \((\(4\ \[ImaginaryI]\ a\ \
Om\)\/k\^2 + \(2\ \((1 + a)\)\ \((Om\ \((\[ImaginaryI]\/\(k\ \((\(-1\) + \
Om\/k)\)\) - \[ImaginaryI]\/\(k\ \((1 + Om\/k)\)\))\) + \[ImaginaryI]\ \
\((Log[\(-1\) + Om\/k] - Log[1 + Om\/k])\))\)\)\/k)\)\)\/k\^2 - \(2 + \(2\ a\ \
Om\^2\)\/k\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + Om\/k] - Log[1 + \
Om\/k])\)\)\/k\)\/k\^2)\))\)\) - \(\(1\/\(k\^2\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^3\)\)\((4\ \[ImaginaryI]\ a\ Om\ \((\((\(-\(\(2\ \
\[ImaginaryI]\ Om\)\/k\^2\)\) + a\ \((\(-\(\(2\ \[ImaginaryI]\ \((6 + 
                                        a)\)\ Om\)\/k\^2\)\) - \(4\ \
\[ImaginaryI]\ \((\(-1\) + a)\)\ Om\^3\)\/k\^4)\))\)\ Log[
                                        1 - \[ImaginaryI]\ \@a] + \((\(2\ \
\[ImaginaryI]\ Om\)\/k\^2 + 
                                        a\ \((\(2\ \[ImaginaryI]\ \((6 + a)\)\
\ Om\)\/k\^2 + \(4\ \[ImaginaryI]\ \((\(-1\) + a)\)\ Om\^3\)\/k\^4)\))\)\ Log[
                                        1 + \[ImaginaryI]\ \@a] + \
\[ImaginaryI]\ \@a\ \((\((\(-1\) + 
                                        Om\^2\/k\^2)\)\ \((\(4\ \[ImaginaryI]\
\ a\ Om\)\/k\^2 + \(2\ \((1 + a)\)\ \((Om\ \((\[ImaginaryI]\/\(k\ \((\(-1\) + \
Om\/k)\)\) - \[ImaginaryI]\/\(k\ \((1 + Om\/k)\)\))\) + \[ImaginaryI]\ \
\((Log[\(-1\) + Om\/k] - Log[1 + Om\/k])\))\)\)\/k)\) + \(2\ \[ImaginaryI]\ \
Om\ \((2 + \(2\ a\ Om\^2\)\/k\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + \
Om\/k] - Log[1 + Om\/k])\)\)\/k)\)\)\/k\^2)\))\))\)\) + \(\(1\/\(2\ \((1 + \
\(a\ Om\^2\)\/k\^2)\)\^2\)\)\((\((\(-\(\(24\ a\^2\ Om\^2\)\/\(k\^4\ \((1 + \
\(a\ Om\^2\)\/k\^2)\)\^2\)\)\) + \(4\ a\)\/\(k\^2\ \((1 + \(a\ Om\^2\)\/k\^2)\
\)\))\)\ \((\((\(-1\) - Om\^2\/k\^2 + 
                                        a\ \((1 - \(\((6 + a)\)\ \
Om\^2\)\/k\^2 - \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                        1 - \[ImaginaryI]\ \@a] + \((1 + 
                                        Om\^2\/k\^2 + 
                                        a\ \((\(-1\) + \(\((6 + a)\)\ Om\^2\)\
\/k\^2 + \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                        1 + \[ImaginaryI]\ \@a] + \
\[ImaginaryI]\ \@a\ \((\(-1\) + 
                                        Om\^2\/k\^2)\)\ \((2 + \(2\ a\ \
Om\^2\)\/k\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + Om\/k] - Log[1 + \
Om\/k])\)\)\/k)\))\))\)\))\))\)\))\)\ \((1 - \(\(1\/\(4\ \@a\ k\^2\ \((1 + \
\(a\ Om\^2\)\/k\^2)\)\^2\)\)\((\@\(1 + a\)\ \((\[ImaginaryI]\ \((1 + 
                                  a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                                  a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + \(a\ Om\^2\)\/k\^2)\) + \
\(2\ \((1 + a)\)\ Om\ \((\(-Log[\(-1\) + Om\/k]\) + Log[1 + \
Om\/k])\)\)\/k)\))\))\)\))\) + \(\(1\/\(4\ \@a\ k\^2\)\)\((\@\(1 + a\)\ \
\((\(-k\^2\) + 
                      Om\^2 + \(\(1\/\(8\ \@a\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^2\)\)\((\[ImaginaryI]\ \@\(1 + a\)\ \((\((\(-1\) - 
                                    Om\^2\/k\^2 + 
                                    a\ \((1 - \(\((6 + a)\)\ Om\^2\)\/k\^2 - \
\(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                  1 - \[ImaginaryI]\ \@a] + \((1 + 
                                    Om\^2\/k\^2 + 
                                    a\ \((\(-1\) + \(\((6 + a)\)\ \
Om\^2\)\/k\^2 + \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \
\@a\ \((\(-1\) + 
                                    Om\^2\/k\^2)\)\ \((2 + \(2\ a\ Om\^2\)\/k\
\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + Om\/k] - Log[1 + \
Om\/k])\)\)\/k)\))\))\)\))\)\ \((\(-\(\(1\/\((1 + \(a\ \
Om\^2\)\/k\^2)\)\^2\)\((\@a\ \((\(2\ a\)\/k\^2 + \(2\ \((1 + a)\)\ \((Om\ \((\
\(-\(1\/\(2\ k\^2\ \((\(-1\) + Om\/k)\)\^2\)\)\) + 1\/\(2\ k\^2\ \((1 + \
Om\/k)\)\^2\))\) + \[ImaginaryI]\ \((\(-\(\[ImaginaryI]\/\(k\ \((\(-1\) + \
Om\/k)\)\)\)\) + \[ImaginaryI]\/\(k\ \((1 + Om\/k)\)\))\))\)\)\/k)\) - \(\
\[ImaginaryI]\ a\ \((1 + a)\)\ Log[1 - \[ImaginaryI]\ \@a]\)\/k\^2 + \(\
\[ImaginaryI]\ a\ \((1 + a)\)\ Log[1 + \[ImaginaryI]\ \@a]\)\/k\^2)\)\)\) - \
\(\(1\/\(2\ \((1 + \(a\ Om\^2\)\/k\^2)\)\^2\)\)\((\((\(-\(\(24\ a\^2\ Om\^2\)\
\/\(k\^4\ \((1 + \(a\ Om\^2\)\/k\^2)\)\^2\)\)\) + \(4\ a\)\/\(k\^2\ \((1 + \
\(a\ Om\^2\)\/k\^2)\)\))\)\ \((\[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + \(a\ Om\^2\)\/k\^2)\) + \
\(2\ \((1 + a)\)\ Om\ \((\(-Log[\(-1\) + Om\/k]\) + Log[1 + \
Om\/k])\)\)\/k)\))\))\)\) + \(\(1\/\(k\^2\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^3\)\)\((4\ \[ImaginaryI]\ a\ Om\ \((\(-\(\(2\ a\ \((1 + 
                                        a)\)\ Om\ Log[
                                        1 - \[ImaginaryI]\ \@a]\)\/k\^2\)\) + \
\(2\ a\ \((1 + a)\)\ Om\ Log[1 + \[ImaginaryI]\ \@a]\)\/k\^2 + \@a\ \
\((\(-\(\(4\ \[ImaginaryI]\ a\ Om\)\/k\^2\)\) + \(2\ \((1 + a)\)\ \((Om\ \
\((\(-\(\[ImaginaryI]\/\(k\ \((\(-1\) + Om\/k)\)\)\)\) + \[ImaginaryI]\/\(k\ \
\((1 + Om\/k)\)\))\) + \[ImaginaryI]\ \((\(-Log[\(-1\) + Om\/k]\) + Log[1 + \
Om\/k])\))\)\)\/k)\))\))\)\))\))\)\) + \(\(1\/\(4\ \@a\ k\^2\)\)\((\@\(1 + \
a\)\ \((2\ \[ImaginaryI]\ Om + \(\(1\/\(8\ \@a\)\)\((\[ImaginaryI]\ \@\(1 + a\
\)\ \((\(\(1\/\((1 + \(a\ Om\^2\)\/k\^2)\)\^2\)\((\((\(-\(\(2\ \[ImaginaryI]\ \
Om\)\/k\^2\)\) + a\ \((\(-\(\(2\ \[ImaginaryI]\ \((6 + 
                                        a)\)\ Om\)\/k\^2\)\) - \(4\ \
\[ImaginaryI]\ \((\(-1\) + a)\)\ Om\^3\)\/k\^4)\))\)\ Log[
                                      1 - \[ImaginaryI]\ \@a] + \((\(2\ \
\[ImaginaryI]\ Om\)\/k\^2 + 
                                        a\ \((\(2\ \[ImaginaryI]\ \((6 + a)\)\
\ Om\)\/k\^2 + \(4\ \[ImaginaryI]\ \((\(-1\) + a)\)\ Om\^3\)\/k\^4)\))\)\ Log[
                                      1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\
\ \@a\ \((\((\(-1\) + 
                                        Om\^2\/k\^2)\)\ \((\(4\ \[ImaginaryI]\
\ a\ Om\)\/k\^2 + \(2\ \((1 + a)\)\ \((Om\ \((\[ImaginaryI]\/\(k\ \((\(-1\) + \
Om\/k)\)\) - \[ImaginaryI]\/\(k\ \((1 + Om\/k)\)\))\) + \[ImaginaryI]\ \
\((Log[\(-1\) + Om\/k] - Log[1 + Om\/k])\))\)\)\/k)\) + \(2\ \[ImaginaryI]\ \
Om\ \((2 + \(2\ a\ Om\^2\)\/k\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + \
Om\/k] - Log[1 + Om\/k])\)\)\/k)\)\)\/k\^2)\))\)\) - \(\(1\/\(k\^2\ \((1 + \
\(a\ Om\^2\)\/k\^2)\)\^3\)\)\((4\ \[ImaginaryI]\ a\ Om\ \((\((\(-1\) - 
                                        Om\^2\/k\^2 + 
                                        a\ \((1 - \(\((6 + a)\)\ \
Om\^2\)\/k\^2 - \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                        1 - \[ImaginaryI]\ \@a] + \((1 + 
                                        Om\^2\/k\^2 + 
                                        a\ \((\(-1\) + \(\((6 + a)\)\ Om\^2\)\
\/k\^2 + \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                        1 + \[ImaginaryI]\ \@a] + \
\[ImaginaryI]\ \@a\ \((\(-1\) + 
                                        Om\^2\/k\^2)\)\ \((2 + \(2\ a\ \
Om\^2\)\/k\^2 + \(2\ \((1 + a)\)\ Om\ \((Log[\(-1\) + Om\/k] - Log[1 + \
Om\/k])\)\)\/k)\))\))\)\))\))\)\))\)\ \((\(\(1\/\(k\^2\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^3\)\)\((4\ \[ImaginaryI]\ a\ Om\ \((\[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + \(a\ Om\^2\)\/k\^2)\) + \
\(2\ \((1 + a)\)\ Om\ \((\(-Log[\(-1\) + Om\/k]\) + Log[1 + \
Om\/k])\)\)\/k)\))\))\)\) - \(\(1\/\((1 + \(a\ \
Om\^2\)\/k\^2)\)\^2\)\((\(-\(\(2\ a\ \((1 + a)\)\ Om\ Log[
                                    1 - \[ImaginaryI]\ \@a]\)\/k\^2\)\) + \(2\
\ a\ \((1 + a)\)\ Om\ Log[1 + \[ImaginaryI]\ \@a]\)\/k\^2 + \@a\ \((\(-\(\(4\ \
\[ImaginaryI]\ a\ Om\)\/k\^2\)\) + \(2\ \((1 + a)\)\ \((Om\ \((\(-\(\
\[ImaginaryI]\/\(k\ \((\(-1\) + Om\/k)\)\)\)\) + \[ImaginaryI]\/\(k\ \((1 + \
Om\/k)\)\))\) + \[ImaginaryI]\ \((\(-Log[\(-1\) + Om\/k]\) + Log[1 + \
Om\/k])\))\)\)\/k)\))\)\))\))\)\))\)\ mGamma\^2\), "+", 
        InterpretationBox[\(O[mGamma]\^3\),
          SeriesData[ mGamma, 0, {}, 0, 3, 1]]}],
      SeriesData[ mGamma, 0, {
        Times[ 
          Plus[ 
            Times[ -1, 
              Power[ k, 2]], 
            Power[ Om, 2], 
            Times[ 
              Complex[ 0, 
                Rational[ 1, 8]], 
              Power[ a, 
                Rational[ -1, 2]], 
              Power[ 
                Plus[ 1, a], 
                Rational[ 1, 2]], 
              Power[ 
                Plus[ 1, 
                  Times[ a, 
                    Power[ k, -2], 
                    Power[ Om, 2]]], -2], 
              Plus[ 
                Times[ 
                  Plus[ -1, 
                    Times[ -1, 
                      Power[ k, -2], 
                      Power[ Om, 2]], 
                    Times[ a, 
                      Plus[ 1, 
                        Times[ -1, 
                          Plus[ 6, a], 
                          Power[ k, -2], 
                          Power[ Om, 2]], 
                        Times[ -1, 
                          Plus[ -1, a], 
                          Power[ k, -4], 
                          Power[ Om, 4]]]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, -1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Plus[ 1, 
                    Times[ 
                      Power[ k, -2], 
                      Power[ Om, 2]], 
                    Times[ a, 
                      Plus[ -1, 
                        Times[ 
                          Plus[ 6, a], 
                          Power[ k, -2], 
                          Power[ Om, 2]], 
                        Times[ 
                          Plus[ -1, a], 
                          Power[ k, -4], 
                          Power[ Om, 4]]]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Complex[ 0, 1], 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ -1, 
                    Times[ 
                      Power[ k, -2], 
                      Power[ Om, 2]]], 
                  Plus[ 2, 
                    Times[ 2, a, 
                      Power[ k, -2], 
                      Power[ Om, 2]], 
                    Times[ 2, 
                      Plus[ 1, a], 
                      Power[ k, -1], Om, 
                      Plus[ 
                        Log[ 
                          Plus[ -1, 
                            Times[ 
                              Power[ k, -1], Om]]], 
                        Times[ -1, 
                          Log[ 
                            Plus[ 1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]]]], 
          Plus[ 1, 
            Times[ 
              Rational[ -1, 4], 
              Power[ a, 
                Rational[ -1, 2]], 
              Power[ 
                Plus[ 1, a], 
                Rational[ 1, 2]], 
              Power[ k, -2], 
              Power[ 
                Plus[ 1, 
                  Times[ a, 
                    Power[ k, -2], 
                    Power[ Om, 2]]], -2], 
              Plus[ 
                Times[ 
                  Complex[ 0, 1], 
                  Plus[ 1, a], 
                  Plus[ -1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, -1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Complex[ 0, -1], 
                  Plus[ 1, a], 
                  Plus[ -1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ 
                    Times[ -2, 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]]], 
                    Times[ 2, 
                      Plus[ 1, a], 
                      Power[ k, -1], Om, 
                      Plus[ 
                        Times[ -1, 
                          Log[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -1], Om]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Power[ k, -1], Om]]]]]]]]]]], 
        Plus[ 
          Times[ 
            Plus[ 
              Times[ 
                Complex[ 0, 2], Om], 
              Times[ 
                Complex[ 0, 
                  Rational[ 1, 8]], 
                Power[ a, 
                  Rational[ -1, 2]], 
                Power[ 
                  Plus[ 1, a], 
                  Rational[ 1, 2]], 
                Plus[ 
                  Times[ 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -2], 
                    Plus[ 
                      Times[ 
                        Plus[ 
                          Times[ 
                            Complex[ 0, -2], 
                            Power[ k, -2], Om], 
                          Times[ a, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, -2], 
                                Plus[ 6, a], 
                                Power[ k, -2], Om], 
                              Times[ 
                                Complex[ 0, -4], 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 3]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, -1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Plus[ 
                          Times[ 
                            Complex[ 0, 2], 
                            Power[ k, -2], Om], 
                          Times[ a, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, 2], 
                                Plus[ 6, a], 
                                Power[ k, -2], Om], 
                              Times[ 
                                Complex[ 0, 4], 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 3]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, 1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]], 
                        Plus[ 
                          Times[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -2], 
                                Power[ Om, 2]]], 
                            Plus[ 
                              Times[ 
                                Complex[ 0, 4], a, 
                                Power[ k, -2], Om], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], 
                                Plus[ 
                                  Times[ Om, 
                                    Plus[ 
                                      Times[ 
                                        Complex[ 0, 1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]], 
                                      Times[ 
                                        Complex[ 0, -1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]]]], 
                                  Times[ 
                                    Complex[ 0, 1], 
                                    Plus[ 
                                      Log[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]]], 
                                      Times[ -1, 
                                        Log[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]]]]]]]]]], 
                          Times[ 
                            Complex[ 0, 2], 
                            Power[ k, -2], Om, 
                            Plus[ 2, 
                              Times[ 2, a, 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], Om, 
                                Plus[ 
                                  Log[ 
                                    Plus[ -1, 
                                      Times[ 
                                        Power[ k, -1], Om]]], 
                                  Times[ -1, 
                                    Log[ 
                                      Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]]]]]]]]]]]], 
                  Times[ 
                    Complex[ 0, -4], a, 
                    Power[ k, -2], Om, 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -3], 
                    Plus[ 
                      Times[ 
                        Plus[ -1, 
                          Times[ -1, 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ a, 
                            Plus[ 1, 
                              Times[ -1, 
                                Plus[ 6, a], 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ -1, 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 4]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, -1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Plus[ 1, 
                          Times[ 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ a, 
                            Plus[ -1, 
                              Times[ 
                                Plus[ 6, a], 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 4]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, 1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]], 
                        Plus[ -1, 
                          Times[ 
                            Power[ k, -2], 
                            Power[ Om, 2]]], 
                        Plus[ 2, 
                          Times[ 2, a, 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ 2, 
                            Plus[ 1, a], 
                            Power[ k, -1], Om, 
                            Plus[ 
                              Log[ 
                                Plus[ -1, 
                                  Times[ 
                                    Power[ k, -1], Om]]], 
                              Times[ -1, 
                                Log[ 
                                  Plus[ 1, 
                                    Times[ 
                                      Power[ k, -1], Om]]]]]]]]]]]]], 
            Plus[ 1, 
              Times[ 
                Rational[ -1, 4], 
                Power[ a, 
                  Rational[ -1, 2]], 
                Power[ 
                  Plus[ 1, a], 
                  Rational[ 1, 2]], 
                Power[ k, -2], 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -2], 
                Plus[ 
                  Times[ 
                    Complex[ 0, 1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Complex[ 0, -1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ -2, 
                        Plus[ 1, 
                          Times[ a, 
                            Power[ k, -2], 
                            Power[ Om, 2]]]], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], Om, 
                        Plus[ 
                          Times[ -1, 
                            Log[ 
                              Plus[ -1, 
                                Times[ 
                                  Power[ k, -1], Om]]]], 
                          Log[ 
                            Plus[ 1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]]]], 
          Times[ 
            Rational[ 1, 4], 
            Power[ a, 
              Rational[ -1, 2]], 
            Power[ 
              Plus[ 1, a], 
              Rational[ 1, 2]], 
            Power[ k, -2], 
            Plus[ 
              Times[ -1, 
                Power[ k, 2]], 
              Power[ Om, 2], 
              Times[ 
                Complex[ 0, 
                  Rational[ 1, 8]], 
                Power[ a, 
                  Rational[ -1, 2]], 
                Power[ 
                  Plus[ 1, a], 
                  Rational[ 1, 2]], 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -2], 
                Plus[ 
                  Times[ 
                    Plus[ -1, 
                      Times[ -1, 
                        Power[ k, -2], 
                        Power[ Om, 2]], 
                      Times[ a, 
                        Plus[ 1, 
                          Times[ -1, 
                            Plus[ 6, a], 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ -1, 
                            Plus[ -1, a], 
                            Power[ k, -4], 
                            Power[ Om, 4]]]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Plus[ 1, 
                      Times[ 
                        Power[ k, -2], 
                        Power[ Om, 2]], 
                      Times[ a, 
                        Plus[ -1, 
                          Times[ 
                            Plus[ 6, a], 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ 
                            Plus[ -1, a], 
                            Power[ k, -4], 
                            Power[ Om, 4]]]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Complex[ 0, 1], 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Plus[ 2, 
                      Times[ 2, a, 
                        Power[ k, -2], 
                        Power[ Om, 2]], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], Om, 
                        Plus[ 
                          Log[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -1], Om]]], 
                          Times[ -1, 
                            Log[ 
                              Plus[ 1, 
                                Times[ 
                                  Power[ k, -1], Om]]]]]]]]]]], 
            Plus[ 
              Times[ 
                Complex[ 0, 4], a, 
                Power[ k, -2], Om, 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -3], 
                Plus[ 
                  Times[ 
                    Complex[ 0, 1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Complex[ 0, -1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ -2, 
                        Plus[ 1, 
                          Times[ a, 
                            Power[ k, -2], 
                            Power[ Om, 2]]]], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], Om, 
                        Plus[ 
                          Times[ -1, 
                            Log[ 
                              Plus[ -1, 
                                Times[ 
                                  Power[ k, -1], Om]]]], 
                          Log[ 
                            Plus[ 1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]], 
              Times[ -1, 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -2], 
                Plus[ 
                  Times[ -2, a, 
                    Plus[ 1, a], 
                    Power[ k, -2], Om, 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 2, a, 
                    Plus[ 1, a], 
                    Power[ k, -2], Om, 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ 
                        Complex[ 0, -4], a, 
                        Power[ k, -2], Om], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], 
                        Plus[ 
                          Times[ Om, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, -1], 
                                Power[ k, -1], 
                                Power[ 
                                  Plus[ -1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -1]], 
                              Times[ 
                                Complex[ 0, 1], 
                                Power[ k, -1], 
                                Power[ 
                                  Plus[ 1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -1]]]], 
                          Times[ 
                            Complex[ 0, 1], 
                            Plus[ 
                              Times[ -1, 
                                Log[ 
                                  Plus[ -1, 
                                    Times[ 
                                      Power[ k, -1], Om]]]], 
                              Log[ 
                                Plus[ 1, 
                                  Times[ 
                                    Power[ k, -1], Om]]]]]]]]]]]]]], 
        Plus[ 
          Times[ 
            Plus[ -1, 
              Times[ 
                Complex[ 0, 
                  Rational[ 1, 8]], 
                Power[ a, 
                  Rational[ -1, 2]], 
                Power[ 
                  Plus[ 1, a], 
                  Rational[ 1, 2]], 
                Plus[ 
                  Times[ 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -2], 
                    Plus[ 
                      Times[ 
                        Plus[ 
                          Power[ k, -2], 
                          Times[ a, 
                            Plus[ 
                              Times[ -1, 
                                Plus[ -6, 
                                  Times[ -1, a]], 
                                Power[ k, -2]], 
                              Times[ 6, 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 2]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, -1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Plus[ 
                          Times[ -1, 
                            Power[ k, -2]], 
                          Times[ a, 
                            Plus[ 
                              Times[ 
                                Plus[ -6, 
                                  Times[ -1, a]], 
                                Power[ k, -2]], 
                              Times[ -6, 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 2]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, 1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]], 
                        Plus[ 
                          Times[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -2], 
                                Power[ Om, 2]]], 
                            Plus[ 
                              Times[ -2, a, 
                                Power[ k, -2]], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], 
                                Plus[ 
                                  Times[ Om, 
                                    Plus[ 
                                      Times[ 
                                        Rational[ 1, 2], 
                                        Power[ k, -2], 
                                        Power[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -2]], 
                                      Times[ 
                                        Rational[ -1, 2], 
                                        Power[ k, -2], 
                                        Power[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -2]]]], 
                                  Times[ 
                                    Complex[ 0, 1], 
                                    Plus[ 
                                      Times[ 
                                        Complex[ 0, 1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]], 
                                      Times[ 
                                        Complex[ 0, -1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]]]]]]]], 
                          Times[ 
                            Complex[ 0, 2], 
                            Power[ k, -2], Om, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, 4], a, 
                                Power[ k, -2], Om], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], 
                                Plus[ 
                                  Times[ Om, 
                                    Plus[ 
                                      Times[ 
                                        Complex[ 0, 1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]], 
                                      Times[ 
                                        Complex[ 0, -1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]]]], 
                                  Times[ 
                                    Complex[ 0, 1], 
                                    Plus[ 
                                      Log[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]]], 
                                      Times[ -1, 
                                        Log[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]]]]]]]]]], 
                          Times[ -1, 
                            Power[ k, -2], 
                            Plus[ 2, 
                              Times[ 2, a, 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], Om, 
                                Plus[ 
                                  Log[ 
                                    Plus[ -1, 
                                      Times[ 
                                        Power[ k, -1], Om]]], 
                                  Times[ -1, 
                                    Log[ 
                                      Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]]]]]]]]]]]], 
                  Times[ 
                    Complex[ 0, -4], a, 
                    Power[ k, -2], Om, 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -3], 
                    Plus[ 
                      Times[ 
                        Plus[ 
                          Times[ 
                            Complex[ 0, -2], 
                            Power[ k, -2], Om], 
                          Times[ a, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, -2], 
                                Plus[ 6, a], 
                                Power[ k, -2], Om], 
                              Times[ 
                                Complex[ 0, -4], 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 3]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, -1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Plus[ 
                          Times[ 
                            Complex[ 0, 2], 
                            Power[ k, -2], Om], 
                          Times[ a, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, 2], 
                                Plus[ 6, a], 
                                Power[ k, -2], Om], 
                              Times[ 
                                Complex[ 0, 4], 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 3]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, 1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]], 
                        Plus[ 
                          Times[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -2], 
                                Power[ Om, 2]]], 
                            Plus[ 
                              Times[ 
                                Complex[ 0, 4], a, 
                                Power[ k, -2], Om], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], 
                                Plus[ 
                                  Times[ Om, 
                                    Plus[ 
                                      Times[ 
                                        Complex[ 0, 1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]], 
                                      Times[ 
                                        Complex[ 0, -1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]]]], 
                                  Times[ 
                                    Complex[ 0, 1], 
                                    Plus[ 
                                      Log[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]]], 
                                      Times[ -1, 
                                        Log[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]]]]]]]]]], 
                          Times[ 
                            Complex[ 0, 2], 
                            Power[ k, -2], Om, 
                            Plus[ 2, 
                              Times[ 2, a, 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], Om, 
                                Plus[ 
                                  Log[ 
                                    Plus[ -1, 
                                      Times[ 
                                        Power[ k, -1], Om]]], 
                                  Times[ -1, 
                                    Log[ 
                                      Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]]]]]]]]]]]], 
                  Times[ 
                    Rational[ 1, 2], 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -2], 
                    Plus[ 
                      Times[ -24, 
                        Power[ a, 2], 
                        Power[ k, -4], 
                        Power[ Om, 2], 
                        Power[ 
                          Plus[ 1, 
                            Times[ a, 
                              Power[ k, -2], 
                              Power[ Om, 2]]], -2]], 
                      Times[ 4, a, 
                        Power[ k, -2], 
                        Power[ 
                          Plus[ 1, 
                            Times[ a, 
                              Power[ k, -2], 
                              Power[ Om, 2]]], -1]]], 
                    Plus[ 
                      Times[ 
                        Plus[ -1, 
                          Times[ -1, 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ a, 
                            Plus[ 1, 
                              Times[ -1, 
                                Plus[ 6, a], 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ -1, 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 4]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, -1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Plus[ 1, 
                          Times[ 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ a, 
                            Plus[ -1, 
                              Times[ 
                                Plus[ 6, a], 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 4]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, 1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]], 
                        Plus[ -1, 
                          Times[ 
                            Power[ k, -2], 
                            Power[ Om, 2]]], 
                        Plus[ 2, 
                          Times[ 2, a, 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ 2, 
                            Plus[ 1, a], 
                            Power[ k, -1], Om, 
                            Plus[ 
                              Log[ 
                                Plus[ -1, 
                                  Times[ 
                                    Power[ k, -1], Om]]], 
                              Times[ -1, 
                                Log[ 
                                  Plus[ 1, 
                                    Times[ 
                                      Power[ k, -1], Om]]]]]]]]]]]]], 
            Plus[ 1, 
              Times[ 
                Rational[ -1, 4], 
                Power[ a, 
                  Rational[ -1, 2]], 
                Power[ 
                  Plus[ 1, a], 
                  Rational[ 1, 2]], 
                Power[ k, -2], 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -2], 
                Plus[ 
                  Times[ 
                    Complex[ 0, 1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Complex[ 0, -1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ -2, 
                        Plus[ 1, 
                          Times[ a, 
                            Power[ k, -2], 
                            Power[ Om, 2]]]], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], Om, 
                        Plus[ 
                          Times[ -1, 
                            Log[ 
                              Plus[ -1, 
                                Times[ 
                                  Power[ k, -1], Om]]]], 
                          Log[ 
                            Plus[ 1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]]]], 
          Times[ 
            Rational[ 1, 4], 
            Power[ a, 
              Rational[ -1, 2]], 
            Power[ 
              Plus[ 1, a], 
              Rational[ 1, 2]], 
            Power[ k, -2], 
            Plus[ 
              Times[ -1, 
                Power[ k, 2]], 
              Power[ Om, 2], 
              Times[ 
                Complex[ 0, 
                  Rational[ 1, 8]], 
                Power[ a, 
                  Rational[ -1, 2]], 
                Power[ 
                  Plus[ 1, a], 
                  Rational[ 1, 2]], 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -2], 
                Plus[ 
                  Times[ 
                    Plus[ -1, 
                      Times[ -1, 
                        Power[ k, -2], 
                        Power[ Om, 2]], 
                      Times[ a, 
                        Plus[ 1, 
                          Times[ -1, 
                            Plus[ 6, a], 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ -1, 
                            Plus[ -1, a], 
                            Power[ k, -4], 
                            Power[ Om, 4]]]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Plus[ 1, 
                      Times[ 
                        Power[ k, -2], 
                        Power[ Om, 2]], 
                      Times[ a, 
                        Plus[ -1, 
                          Times[ 
                            Plus[ 6, a], 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ 
                            Plus[ -1, a], 
                            Power[ k, -4], 
                            Power[ Om, 4]]]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Complex[ 0, 1], 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Plus[ 2, 
                      Times[ 2, a, 
                        Power[ k, -2], 
                        Power[ Om, 2]], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], Om, 
                        Plus[ 
                          Log[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -1], Om]]], 
                          Times[ -1, 
                            Log[ 
                              Plus[ 1, 
                                Times[ 
                                  Power[ k, -1], Om]]]]]]]]]]], 
            Plus[ 
              Times[ -1, 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -2], 
                Plus[ 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ 2, a, 
                        Power[ k, -2]], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], 
                        Plus[ 
                          Times[ Om, 
                            Plus[ 
                              Times[ 
                                Rational[ -1, 2], 
                                Power[ k, -2], 
                                Power[ 
                                  Plus[ -1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -2]], 
                              Times[ 
                                Rational[ 1, 2], 
                                Power[ k, -2], 
                                Power[ 
                                  Plus[ 1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -2]]]], 
                          Times[ 
                            Complex[ 0, 1], 
                            Plus[ 
                              Times[ 
                                Complex[ 0, -1], 
                                Power[ k, -1], 
                                Power[ 
                                  Plus[ -1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -1]], 
                              Times[ 
                                Complex[ 0, 1], 
                                Power[ k, -1], 
                                Power[ 
                                  Plus[ 1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -1]]]]]]]], 
                  Times[ 
                    Complex[ 0, -1], a, 
                    Plus[ 1, a], 
                    Power[ k, -2], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Complex[ 0, 1], a, 
                    Plus[ 1, a], 
                    Power[ k, -2], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]]]], 
              Times[ 
                Rational[ -1, 2], 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -2], 
                Plus[ 
                  Times[ -24, 
                    Power[ a, 2], 
                    Power[ k, -4], 
                    Power[ Om, 2], 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -2]], 
                  Times[ 4, a, 
                    Power[ k, -2], 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -1]]], 
                Plus[ 
                  Times[ 
                    Complex[ 0, 1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Complex[ 0, -1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ -2, 
                        Plus[ 1, 
                          Times[ a, 
                            Power[ k, -2], 
                            Power[ Om, 2]]]], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], Om, 
                        Plus[ 
                          Times[ -1, 
                            Log[ 
                              Plus[ -1, 
                                Times[ 
                                  Power[ k, -1], Om]]]], 
                          Log[ 
                            Plus[ 1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]], 
              Times[ 
                Complex[ 0, 4], a, 
                Power[ k, -2], Om, 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -3], 
                Plus[ 
                  Times[ -2, a, 
                    Plus[ 1, a], 
                    Power[ k, -2], Om, 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 2, a, 
                    Plus[ 1, a], 
                    Power[ k, -2], Om, 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ 
                        Complex[ 0, -4], a, 
                        Power[ k, -2], Om], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], 
                        Plus[ 
                          Times[ Om, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, -1], 
                                Power[ k, -1], 
                                Power[ 
                                  Plus[ -1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -1]], 
                              Times[ 
                                Complex[ 0, 1], 
                                Power[ k, -1], 
                                Power[ 
                                  Plus[ 1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -1]]]], 
                          Times[ 
                            Complex[ 0, 1], 
                            Plus[ 
                              Times[ -1, 
                                Log[ 
                                  Plus[ -1, 
                                    Times[ 
                                      Power[ k, -1], Om]]]], 
                              Log[ 
                                Plus[ 1, 
                                  Times[ 
                                    Power[ k, -1], Om]]]]]]]]]]]]], 
          Times[ 
            Rational[ 1, 4], 
            Power[ a, 
              Rational[ -1, 2]], 
            Power[ 
              Plus[ 1, a], 
              Rational[ 1, 2]], 
            Power[ k, -2], 
            Plus[ 
              Times[ 
                Complex[ 0, 2], Om], 
              Times[ 
                Complex[ 0, 
                  Rational[ 1, 8]], 
                Power[ a, 
                  Rational[ -1, 2]], 
                Power[ 
                  Plus[ 1, a], 
                  Rational[ 1, 2]], 
                Plus[ 
                  Times[ 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -2], 
                    Plus[ 
                      Times[ 
                        Plus[ 
                          Times[ 
                            Complex[ 0, -2], 
                            Power[ k, -2], Om], 
                          Times[ a, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, -2], 
                                Plus[ 6, a], 
                                Power[ k, -2], Om], 
                              Times[ 
                                Complex[ 0, -4], 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 3]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, -1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Plus[ 
                          Times[ 
                            Complex[ 0, 2], 
                            Power[ k, -2], Om], 
                          Times[ a, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, 2], 
                                Plus[ 6, a], 
                                Power[ k, -2], Om], 
                              Times[ 
                                Complex[ 0, 4], 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 3]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, 1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]], 
                        Plus[ 
                          Times[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -2], 
                                Power[ Om, 2]]], 
                            Plus[ 
                              Times[ 
                                Complex[ 0, 4], a, 
                                Power[ k, -2], Om], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], 
                                Plus[ 
                                  Times[ Om, 
                                    Plus[ 
                                      Times[ 
                                        Complex[ 0, 1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]], 
                                      Times[ 
                                        Complex[ 0, -1], 
                                        Power[ k, -1], 
                                        Power[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]], -1]]]], 
                                  Times[ 
                                    Complex[ 0, 1], 
                                    Plus[ 
                                      Log[ 
                                        Plus[ -1, 
                                        Times[ 
                                        Power[ k, -1], Om]]], 
                                      Times[ -1, 
                                        Log[ 
                                        Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]]]]]]]]]], 
                          Times[ 
                            Complex[ 0, 2], 
                            Power[ k, -2], Om, 
                            Plus[ 2, 
                              Times[ 2, a, 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ 2, 
                                Plus[ 1, a], 
                                Power[ k, -1], Om, 
                                Plus[ 
                                  Log[ 
                                    Plus[ -1, 
                                      Times[ 
                                        Power[ k, -1], Om]]], 
                                  Times[ -1, 
                                    Log[ 
                                      Plus[ 1, 
                                        Times[ 
                                        Power[ k, -1], Om]]]]]]]]]]]], 
                  Times[ 
                    Complex[ 0, -4], a, 
                    Power[ k, -2], Om, 
                    Power[ 
                      Plus[ 1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], -3], 
                    Plus[ 
                      Times[ 
                        Plus[ -1, 
                          Times[ -1, 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ a, 
                            Plus[ 1, 
                              Times[ -1, 
                                Plus[ 6, a], 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ -1, 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 4]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, -1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Plus[ 1, 
                          Times[ 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ a, 
                            Plus[ -1, 
                              Times[ 
                                Plus[ 6, a], 
                                Power[ k, -2], 
                                Power[ Om, 2]], 
                              Times[ 
                                Plus[ -1, a], 
                                Power[ k, -4], 
                                Power[ Om, 4]]]]], 
                        Log[ 
                          Plus[ 1, 
                            Times[ 
                              Complex[ 0, 1], 
                              Power[ a, 
                                Rational[ 1, 2]]]]]], 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]], 
                        Plus[ -1, 
                          Times[ 
                            Power[ k, -2], 
                            Power[ Om, 2]]], 
                        Plus[ 2, 
                          Times[ 2, a, 
                            Power[ k, -2], 
                            Power[ Om, 2]], 
                          Times[ 2, 
                            Plus[ 1, a], 
                            Power[ k, -1], Om, 
                            Plus[ 
                              Log[ 
                                Plus[ -1, 
                                  Times[ 
                                    Power[ k, -1], Om]]], 
                              Times[ -1, 
                                Log[ 
                                  Plus[ 1, 
                                    Times[ 
                                      Power[ k, -1], Om]]]]]]]]]]]]], 
            Plus[ 
              Times[ 
                Complex[ 0, 4], a, 
                Power[ k, -2], Om, 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -3], 
                Plus[ 
                  Times[ 
                    Complex[ 0, 1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Complex[ 0, -1], 
                    Plus[ 1, a], 
                    Plus[ -1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ -2, 
                        Plus[ 1, 
                          Times[ a, 
                            Power[ k, -2], 
                            Power[ Om, 2]]]], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], Om, 
                        Plus[ 
                          Times[ -1, 
                            Log[ 
                              Plus[ -1, 
                                Times[ 
                                  Power[ k, -1], Om]]]], 
                          Log[ 
                            Plus[ 1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]], 
              Times[ -1, 
                Power[ 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], -2], 
                Plus[ 
                  Times[ -2, a, 
                    Plus[ 1, a], 
                    Power[ k, -2], Om, 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, -1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 2, a, 
                    Plus[ 1, a], 
                    Power[ k, -2], Om, 
                    Log[ 
                      Plus[ 1, 
                        Times[ 
                          Complex[ 0, 1], 
                          Power[ a, 
                            Rational[ 1, 2]]]]]], 
                  Times[ 
                    Power[ a, 
                      Rational[ 1, 2]], 
                    Plus[ 
                      Times[ 
                        Complex[ 0, -4], a, 
                        Power[ k, -2], Om], 
                      Times[ 2, 
                        Plus[ 1, a], 
                        Power[ k, -1], 
                        Plus[ 
                          Times[ Om, 
                            Plus[ 
                              Times[ 
                                Complex[ 0, -1], 
                                Power[ k, -1], 
                                Power[ 
                                  Plus[ -1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -1]], 
                              Times[ 
                                Complex[ 0, 1], 
                                Power[ k, -1], 
                                Power[ 
                                  Plus[ 1, 
                                    Times[ 
                                      Power[ k, -1], Om]], -1]]]], 
                          Times[ 
                            Complex[ 0, 1], 
                            Plus[ 
                              Times[ -1, 
                                Log[ 
                                  Plus[ -1, 
                                    Times[ 
                                      Power[ k, -1], Om]]]], 
                              Log[ 
                                Plus[ 1, 
                                  Times[ 
                                    Power[ k, -1], Om]]]]]]]]]]]]]]}, 0, 3, 
        1]]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(\((1 - \(\(1\/\(4\ \@\(a\/\(1 + a\)\)\ \((k\^2 + a\ \
Om\^2)\)\^2\)\)\((\[ImaginaryI]\ \((\((1 + a)\)\ \((\(-k\^2\) + 
                            a\ Om\^2)\)\ Log[
                          1 - \[ImaginaryI]\ \@a] + \((1 + a)\)\ \((k\^2 - 
                            a\ Om\^2)\)\ Log[1 + \[ImaginaryI]\ \@a] + 
                      2\ \[ImaginaryI]\ \@a\ \((k\^2 + 
                            a\ Om\^2 - \((1 + 
                                  a)\)\ k\ Om\ Log[\(k + Om\)\/k] + \((1 + 
                                  a)\)\ k\ Om\ Log[\(-1\) + 
                                  Om\/k])\))\))\)\))\)\ \((\(-k\^2\) + 
              Om\^2 + \(\(1\/\(8\ \@\(a\/\(1 + a\)\)\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^2\)\)\((\[ImaginaryI]\ \((\(\((\((\(-1\) + a)\)\ k\^4 - \
\((1 + 6\ a + a\^2)\)\ k\^2\ Om\^2 - \((\(-1\) + a)\)\ a\ Om\^4)\)\ Log[1 - \
\[ImaginaryI]\ \@a]\)\/k\^4 + \((1 + Om\^2\/k\^2 + 
                            a\ \((\(-1\) + \(\((6 + a)\)\ Om\^2\)\/k\^2 + \
\(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                          1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \
\((\(-1\) + 
                            Om\^2\/k\^2)\)\ \((2 + \(2\ a\ Om\^2\)\/k\^2 + \
\(2\ \((1 + a)\)\ Om\ \((\(-Log[\(k + Om\)\/k]\) + Log[\(-1\) + \
Om\/k])\)\)\/k)\))\))\)\))\)\), 
        "+", \(1\/16\ \((\(\(1\/\(a\ \((k\^2 + a\ Om\^2)\)\^5\)\)\((\((\(-\
\[ImaginaryI]\)\ \((1 + a)\)\^\(3/2\)\ \((\(-k\^2\) + a\ Om\^2)\)\ Log[
                          1 - \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \((1 + \
a)\)\^\(3/2\)\ \((\(-k\^2\) + a\ Om\^2)\)\ Log[1 + \[ImaginaryI]\ \@a] + 
                      2\ \@a\ \((\((k\^2 + a\ Om\^2)\)\ \((\@\(1 + a\) + 
                                  2\ k\^2 + 
                                  2\ a\ Om\^2)\) - \((1 + a)\)\^\(3/2\)\ k\ \
Om\ Log[\(k + Om\)\/k] + \((1 + a)\)\^\(3/2\)\ k\ Om\ Log[\(-1\) + 
                                  Om\/k])\))\)\ \((\((1 + a)\)\^\(3/2\)\ k\^2\
\ Om\ \((\((1 + 3\ a)\)\ k\^2 - a\ \((3 + a)\)\ Om\^2)\)\ Log[
                          1 - \[ImaginaryI]\ \@a] + \((1 + a)\)\^\(3/2\)\ \
k\^2\ Om\ \((\(-\((1 + 3\ a)\)\)\ k\^2 + a\ \((3 + a)\)\ Om\^2)\)\ Log[
                          1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \((4\ \
Om\ \((k\^2 + a\ Om\^2)\)\ \((2\ k\^4 + 2\ a\^2\ Om\^4 - 
                                  k\^2\ \((\@\(1 + a\) + 
                                        a\ \((\@\(1 + a\) - 
                                        4\ Om\^2)\))\))\) - \((1 + \
a)\)\^\(3/2\)\ k\ \((k\^4 - 3\ \((1 + a)\)\ k\^2\ Om\^2 + 
                                  a\ Om\^4)\)\ Log[\(k + Om\)\/k] + \((1 + a)\
\)\^\(3/2\)\ k\ \((k\^4 - 3\ \((1 + a)\)\ k\^2\ Om\^2 + 
                                  a\ Om\^4)\)\ Log[\(-1\) + 
                                  Om\/k])\))\))\)\) + \(\(1\/\(\@\(a\/\(1 + a\
\)\)\ k\^2\ \((1 + \(a\ Om\^2\)\/k\^2)\)\^3\)\)\((4\ \((\(\(1\/k\^2\)\((4\ \
\[ImaginaryI]\ a\ Om\ \((\[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + \(a\ Om\^2\)\/k\^2)\)\ \
Log[1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \((1 + \(a\ Om\^2\)\/k\^2)\) + \
\(2\ \((1 + a)\)\ Om\ \((Log[\(k + Om\)\/k] - Log[\(-1\) + Om\/k])\)\)\/k)\))\
\))\)\) - \((1 + \(a\ Om\^2\)\/k\^2)\)\ \((\(-\(\(2\ a\ \((1 + a)\)\ Om\ Log[
                                      1 - \[ImaginaryI]\ \@a]\)\/k\^2\)\) + \
\(2\ a\ \((1 + a)\)\ Om\ Log[1 + \[ImaginaryI]\ \@a]\)\/k\^2 + \(2\ \
\[ImaginaryI]\ \@a\ \((2\ Om\ \((k\^2 + a\ Om\^2)\) + \((1 + a)\)\ k\ \((k\^2 \
- Om\^2)\)\ Log[\(k + Om\)\/k] - \((1 + a)\)\ k\ \((k\^2 - Om\^2)\)\ Log[\(-1\
\) + Om\/k])\)\)\/\(k\^2\ \((k - Om)\)\ \((k + Om)\)\))\))\)\ \((\(-k\^2\) + 
                      Om\^2 + \(\(1\/\(8\ \@\(a\/\(1 + a\)\)\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^2\)\)\((\[ImaginaryI]\ \((\(\((\((\(-1\) + a)\)\ k\^4 - \
\((1 + 6\ a + a\^2)\)\ k\^2\ Om\^2 - \((\(-1\) + a)\)\ a\ Om\^4)\)\ Log[1 - \
\[ImaginaryI]\ \@a]\)\/k\^4 + \((1 + Om\^2\/k\^2 + 
                                    a\ \((\(-1\) + \(\((6 + a)\)\ \
Om\^2\)\/k\^2 + \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \
\@a\ \((\(-1\) + 
                                    Om\^2\/k\^2)\)\ \((2 + \(2\ a\ Om\^2\)\/k\
\^2 + \(2\ \((1 + a)\)\ Om\ \((\(-Log[\(k + Om\)\/k]\) + Log[\(-1\) + Om\/k])\
\)\)\/k)\))\))\)\))\))\)\))\)\ mGamma\), 
        "+", \(1\/4\ \((\(-\(\((\[ImaginaryI]\ \((\(-2\)\ k\^4\ Om - 
                          4\ a\ k\^4\ Om - 4\ a\^2\ k\^2\ Om\^3 + 
                          2\ a\^2\ Om\^5 - \[ImaginaryI]\ \@a\ \((1 + 
                                a)\)\ Om\ \((\(-k\^2\) + 
                                Om\^2)\)\ \((\(-3\)\ k\^2 + a\ Om\^2)\)\ Log[
                              1 - \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \
\((1 + a)\)\ Om\ \((\(-k\^2\) + Om\^2)\)\ \((\(-3\)\ k\^2 + a\ Om\^2)\)\ Log[
                              1 + \[ImaginaryI]\ \@a] - 
                          k\^5\ Log[\(k + Om\)\/k] - 
                          a\ k\^5\ Log[\(k + Om\)\/k] + 
                          k\^3\ Om\^2\ Log[\(k + Om\)\/k] + 
                          4\ a\ k\^3\ Om\^2\ Log[\(k + Om\)\/k] + 
                          3\ a\^2\ k\^3\ Om\^2\ Log[\(k + Om\)\/k] - 
                          3\ a\ k\ Om\^4\ Log[\(k + Om\)\/k] - 
                          3\ a\^2\ k\ Om\^4\ Log[\(k + Om\)\/k] + 
                          k\^5\ Log[\(-1\) + Om\/k] + 
                          a\ k\^5\ Log[\(-1\) + Om\/k] - 
                          k\^3\ Om\^2\ Log[\(-1\) + Om\/k] - 
                          4\ a\ k\^3\ Om\^2\ Log[\(-1\) + Om\/k] - 
                          3\ a\^2\ k\^3\ Om\^2\ Log[\(-1\) + Om\/k] + 
                          3\ a\ k\ Om\^4\ Log[\(-1\) + Om\/k] + 
                          3\ a\^2\ k\ Om\^4\ Log[\(-1\) + 
                                Om\/k])\)\ \((\((1 + a)\)\^\(3/2\)\ k\^2\ Om\ \
\((\(-\((1 + 3\ a)\)\)\ k\^2 + a\ \((3 + a)\)\ Om\^2)\)\ Log[
                              1 - \[ImaginaryI]\ \@a] + \((1 + a)\)\^\(3/2\)\ \
k\^2\ Om\ \((\((1 + 3\ a)\)\ k\^2 - a\ \((3 + a)\)\ Om\^2)\)\ Log[
                              1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \@a\ \
\((4\ Om\ \((k\^2 + a\ Om\^2)\)\ \((\(-2\)\ k\^4 - 2\ a\^2\ Om\^4 + 
                                      k\^2\ \((\@\(1 + a\) + 
                                        a\ \((\@\(1 + a\) - 
                                        4\ Om\^2)\))\))\) + \((1 + \
a)\)\^\(3/2\)\ k\ \((k\^4 - 3\ \((1 + a)\)\ k\^2\ Om\^2 + 
                                      a\ Om\^4)\)\ Log[\(k + Om\)\/k] - \((1 \
+ a)\)\^\(3/2\)\ k\ \((k\^4 - 3\ \((1 + a)\)\ k\^2\ Om\^2 + 
                                      a\ Om\^4)\)\ Log[\(-1\) + 
                                      Om\/k])\))\))\)/\((2\ \@\(a\/\(1 + \
a\)\)\ \((k - Om)\)\ \((k + Om)\)\ \((k\^2 + a\ Om\^2)\)\^6)\)\)\) + 
              4\ \((1 - \(\(1\/\(4\ \@\(a\/\(1 + a\)\)\ \((k\^2 + a\ Om\^2)\)\
\^2\)\)\((\[ImaginaryI]\ \((\((1 + a)\)\ \((\(-k\^2\) + a\ Om\^2)\)\ Log[
                                1 - \[ImaginaryI]\ \@a] + \((1 + 
                                  a)\)\ \((k\^2 - a\ Om\^2)\)\ Log[
                                1 + \[ImaginaryI]\ \@a] + 
                            2\ \[ImaginaryI]\ \@a\ \((k\^2 + 
                                  a\ Om\^2 - \((1 + 
                                        a)\)\ k\ Om\ Log[\(k + Om\)\/k] + \
\((1 + a)\)\ k\ Om\ Log[\(-1\) + 
                                        Om\/k])\))\))\)\))\)\ \((\(-1\) + \((\
\((1 + a)\)\^\(3/2\)\ k\ \((\[ImaginaryI]\ k\ \((k\^2 - 
                                    Om\^2)\)\ \((\((1 + 3\ a)\)\ k\^4 - 
                                    2\ a\ \((7 + 9\ a)\)\ k\^2\ Om\^2 + 
                                    3\ a\^2\ \((3 + a)\)\ Om\^4)\)\ Log[
                                  1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ k\ \
\((k\^2 - Om\^2)\)\ \((\((1 + 3\ a)\)\ k\^4 - 
                                    2\ a\ \((7 + 9\ a)\)\ k\^2\ Om\^2 + 
                                    3\ a\^2\ \((3 + a)\)\ Om\^4)\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + 
                              2\ \@a\ \((3\ k\^7 - 5\ k\^5\ Om\^2 - 
                                    6\ a\ k\^5\ Om\^2 + 2\ a\ k\^3\ Om\^4 - 
                                    9\ a\^2\ k\^3\ Om\^4 + 
                                    7\ a\^2\ k\ Om\^6 + 
                                    Om\ \((\(-k\^2\) + 
                                        Om\^2)\)\ \((\((3 + 6\ a)\)\ k\^4 - 
                                        2\ a\ \((4 + 3\ a)\)\ k\^2\ Om\^2 + 
                                        a\^2\ Om\^4)\)\ Log[\(k + Om\)\/k] + 
                                    Om\ \((k\^2 - 
                                        Om\^2)\)\ \((\((3 + 6\ a)\)\ k\^4 - 
                                        2\ a\ \((4 + 3\ a)\)\ k\^2\ Om\^2 + 
                                        a\^2\ Om\^4)\)\ Log[\(-1\) + 
                                        Om\/k])\))\))\)/\((8\ \@a\ \((k - 
                              Om)\)\ \((k + 
                              Om)\)\ \((k\^2 + a\ Om\^2)\)\^4)\))\) + \(\(1\/\
\(\@\(a\/\(1 + a\)\)\ k\^2\ \((1 + \(a\ Om\^2\)\/k\^2)\)\^3\)\)\((2\ \
\[ImaginaryI]\ \((\(\(1\/\(2\ k\^4\ \((k - Om)\)\^2\ \((k + \
Om)\)\^2\)\)\((\((k\^2 + 
                              a\ Om\^2)\)\ \((\(-2\)\ \[ImaginaryI]\ \@a\ \((\
\((2 + a)\)\ k\^4 + 2\ a\ k\^2\ Om\^2 - a\ Om\^4)\) + 
                              a\ \((1 + a)\)\ \((k\^2 - Om\^2)\)\^2\ Log[
                                  1 - \[ImaginaryI]\ \@a] - 
                              a\ \((1 + a)\)\ \((k\^2 - Om\^2)\)\^2\ Log[
                                  1 + \[ImaginaryI]\ \@a])\))\)\) + \
\(\(1\/\(k\^6 + a\ k\^4\ Om\^2\)\)\((a\ \((\(-k\^2\) + 
                              5\ a\ Om\^2)\)\ \((\((1 + a)\)\ \((\(-k\^2\) + 
                                    a\ Om\^2)\)\ Log[
                                  1 - \[ImaginaryI]\ \@a] + \((1 + 
                                    a)\)\ \((k\^2 - a\ Om\^2)\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + 
                              2\ \[ImaginaryI]\ \@a\ \((k\^2 + 
                                    a\ Om\^2 - \((1 + 
                                        a)\)\ k\ Om\ Log[\(k + Om\)\/k] + \
\((1 + a)\)\ k\ Om\ Log[\(-1\) + 
                                        Om\/k])\))\))\)\) + \(\(1\/k\^2\)\((2\
\ a\ Om\ \((\(-\(\(2\ a\ \((1 + a)\)\ Om\ Log[
                                        1 - \[ImaginaryI]\ \@a]\)\/k\^2\)\) + \
\(2\ a\ \((1 + a)\)\ Om\ Log[1 + \[ImaginaryI]\ \@a]\)\/k\^2 + \(2\ \
\[ImaginaryI]\ \@a\ \((2\ Om\ \((k\^2 + a\ Om\^2)\) + \((1 + a)\)\ k\ \((k\^2 \
- Om\^2)\)\ Log[\(k + Om\)\/k] - \((1 + a)\)\ k\ \((k\^2 - Om\^2)\)\ Log[\(-1\
\) + Om\/k])\)\)\/\(k\^2\ \((k - Om)\)\ \((k + Om)\)\))\))\)\))\)\ \((\(-k\^2\
\) + Om\^2 + \(\(1\/\(8\ \@\(a\/\(1 + a\)\)\ \((1 + \(a\ \
Om\^2\)\/k\^2)\)\^2\)\)\((\[ImaginaryI]\ \((\(\((\((\(-1\) + a)\)\ k\^4 - \
\((1 + 6\ a + a\^2)\)\ k\^2\ Om\^2 - \((\(-1\) + a)\)\ a\ Om\^4)\)\ Log[1 - \
\[ImaginaryI]\ \@a]\)\/k\^4 + \((1 + Om\^2\/k\^2 + 
                                    a\ \((\(-1\) + \(\((6 + a)\)\ \
Om\^2\)\/k\^2 + \(\((\(-1\) + a)\)\ Om\^4\)\/k\^4)\))\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \
\@a\ \((\(-1\) + 
                                    Om\^2\/k\^2)\)\ \((2 + \(2\ a\ Om\^2\)\/k\
\^2 + \(2\ \((1 + a)\)\ Om\ \((\(-Log[\(k + Om\)\/k]\) + Log[\(-1\) + Om\/k])\
\)\)\/k)\))\))\)\))\))\)\))\)\ mGamma\^2\), "+", 
        InterpretationBox[\(O[mGamma]\^3\),
          SeriesData[ mGamma, 0, {}, 0, 3, 1]]}],
      SeriesData[ mGamma, 0, {
        Times[ 
          Plus[ 1, 
            Times[ 
              Complex[ 0, 
                Rational[ -1, 4]], 
              Power[ 
                Times[ a, 
                  Power[ 
                    Plus[ 1, a], -1]], 
                Rational[ -1, 2]], 
              Power[ 
                Plus[ 
                  Power[ k, 2], 
                  Times[ a, 
                    Power[ Om, 2]]], -2], 
              Plus[ 
                Times[ 
                  Plus[ 1, a], 
                  Plus[ 
                    Times[ -1, 
                      Power[ k, 2]], 
                    Times[ a, 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, -1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Plus[ 1, a], 
                  Plus[ 
                    Power[ k, 2], 
                    Times[ -1, a, 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Complex[ 0, 2], 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ 
                    Power[ k, 2], 
                    Times[ a, 
                      Power[ Om, 2]], 
                    Times[ -1, 
                      Plus[ 1, a], k, Om, 
                      Log[ 
                        Times[ 
                          Power[ k, -1], 
                          Plus[ k, Om]]]], 
                    Times[ 
                      Plus[ 1, a], k, Om, 
                      Log[ 
                        Plus[ -1, 
                          Times[ 
                            Power[ k, -1], Om]]]]]]]]], 
          Plus[ 
            Times[ -1, 
              Power[ k, 2]], 
            Power[ Om, 2], 
            Times[ 
              Complex[ 0, 
                Rational[ 1, 8]], 
              Power[ 
                Times[ a, 
                  Power[ 
                    Plus[ 1, a], -1]], 
                Rational[ -1, 2]], 
              Power[ 
                Plus[ 1, 
                  Times[ a, 
                    Power[ k, -2], 
                    Power[ Om, 2]]], -2], 
              Plus[ 
                Times[ 
                  Power[ k, -4], 
                  Plus[ 
                    Times[ 
                      Plus[ -1, a], 
                      Power[ k, 4]], 
                    Times[ -1, 
                      Plus[ 1, 
                        Times[ 6, a], 
                        Power[ a, 2]], 
                      Power[ k, 2], 
                      Power[ Om, 2]], 
                    Times[ -1, 
                      Plus[ -1, a], a, 
                      Power[ Om, 4]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, -1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Plus[ 1, 
                    Times[ 
                      Power[ k, -2], 
                      Power[ Om, 2]], 
                    Times[ a, 
                      Plus[ -1, 
                        Times[ 
                          Plus[ 6, a], 
                          Power[ k, -2], 
                          Power[ Om, 2]], 
                        Times[ 
                          Plus[ -1, a], 
                          Power[ k, -4], 
                          Power[ Om, 4]]]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Complex[ 0, 1], 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ -1, 
                    Times[ 
                      Power[ k, -2], 
                      Power[ Om, 2]]], 
                  Plus[ 2, 
                    Times[ 2, a, 
                      Power[ k, -2], 
                      Power[ Om, 2]], 
                    Times[ 2, 
                      Plus[ 1, a], 
                      Power[ k, -1], Om, 
                      Plus[ 
                        Times[ -1, 
                          Log[ 
                            Times[ 
                              Power[ k, -1], 
                              Plus[ k, Om]]]], 
                        Log[ 
                          Plus[ -1, 
                            Times[ 
                              Power[ k, -1], Om]]]]]]]]]]], 
        Times[ 
          Rational[ 1, 16], 
          Plus[ 
            Times[ 
              Power[ a, -1], 
              Power[ 
                Plus[ 
                  Power[ k, 2], 
                  Times[ a, 
                    Power[ Om, 2]]], -5], 
              Plus[ 
                Times[ 
                  Complex[ 0, -1], 
                  Power[ 
                    Plus[ 1, a], 
                    Rational[ 3, 2]], 
                  Plus[ 
                    Times[ -1, 
                      Power[ k, 2]], 
                    Times[ a, 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, -1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Complex[ 0, 1], 
                  Power[ 
                    Plus[ 1, a], 
                    Rational[ 3, 2]], 
                  Plus[ 
                    Times[ -1, 
                      Power[ k, 2]], 
                    Times[ a, 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 2, 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ 
                    Times[ 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ a, 
                          Power[ Om, 2]]], 
                      Plus[ 
                        Power[ 
                          Plus[ 1, a], 
                          Rational[ 1, 2]], 
                        Times[ 2, 
                          Power[ k, 2]], 
                        Times[ 2, a, 
                          Power[ Om, 2]]]], 
                    Times[ -1, 
                      Power[ 
                        Plus[ 1, a], 
                        Rational[ 3, 2]], k, Om, 
                      Log[ 
                        Times[ 
                          Power[ k, -1], 
                          Plus[ k, Om]]]], 
                    Times[ 
                      Power[ 
                        Plus[ 1, a], 
                        Rational[ 3, 2]], k, Om, 
                      Log[ 
                        Plus[ -1, 
                          Times[ 
                            Power[ k, -1], Om]]]]]]], 
              Plus[ 
                Times[ 
                  Power[ 
                    Plus[ 1, a], 
                    Rational[ 3, 2]], 
                  Power[ k, 2], Om, 
                  Plus[ 
                    Times[ 
                      Plus[ 1, 
                        Times[ 3, a]], 
                      Power[ k, 2]], 
                    Times[ -1, a, 
                      Plus[ 3, a], 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, -1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Power[ 
                    Plus[ 1, a], 
                    Rational[ 3, 2]], 
                  Power[ k, 2], Om, 
                  Plus[ 
                    Times[ -1, 
                      Plus[ 1, 
                        Times[ 3, a]], 
                      Power[ k, 2]], 
                    Times[ a, 
                      Plus[ 3, a], 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Complex[ 0, 1], 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ 
                    Times[ 4, Om, 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ a, 
                          Power[ Om, 2]]], 
                      Plus[ 
                        Times[ 2, 
                          Power[ k, 4]], 
                        Times[ 2, 
                          Power[ a, 2], 
                          Power[ Om, 4]], 
                        Times[ -1, 
                          Power[ k, 2], 
                          Plus[ 
                            Power[ 
                              Plus[ 1, a], 
                              Rational[ 1, 2]], 
                            Times[ a, 
                              Plus[ 
                                Power[ 
                                  Plus[ 1, a], 
                                  Rational[ 1, 2]], 
                                Times[ -4, 
                                  Power[ Om, 2]]]]]]]], 
                    Times[ -1, 
                      Power[ 
                        Plus[ 1, a], 
                        Rational[ 3, 2]], k, 
                      Plus[ 
                        Power[ k, 4], 
                        Times[ -3, 
                          Plus[ 1, a], 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ a, 
                          Power[ Om, 4]]], 
                      Log[ 
                        Times[ 
                          Power[ k, -1], 
                          Plus[ k, Om]]]], 
                    Times[ 
                      Power[ 
                        Plus[ 1, a], 
                        Rational[ 3, 2]], k, 
                      Plus[ 
                        Power[ k, 4], 
                        Times[ -3, 
                          Plus[ 1, a], 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ a, 
                          Power[ Om, 4]]], 
                      Log[ 
                        Plus[ -1, 
                          Times[ 
                            Power[ k, -1], Om]]]]]]]], 
            Times[ 4, 
              Power[ 
                Times[ a, 
                  Power[ 
                    Plus[ 1, a], -1]], 
                Rational[ -1, 2]], 
              Power[ k, -2], 
              Power[ 
                Plus[ 1, 
                  Times[ a, 
                    Power[ k, -2], 
                    Power[ Om, 2]]], -3], 
              Plus[ 
                Times[ 
                  Complex[ 0, 4], a, 
                  Power[ k, -2], Om, 
                  Plus[ 
                    Times[ 
                      Complex[ 0, 1], 
                      Plus[ 1, a], 
                      Plus[ -1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Complex[ 0, -1], 
                      Plus[ 1, a], 
                      Plus[ -1, 
                        Times[ a, 
                          Power[ k, -2], 
                          Power[ Om, 2]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Plus[ 
                        Times[ -2, 
                          Plus[ 1, 
                            Times[ a, 
                              Power[ k, -2], 
                              Power[ Om, 2]]]], 
                        Times[ 2, 
                          Plus[ 1, a], 
                          Power[ k, -1], Om, 
                          Plus[ 
                            Log[ 
                              Times[ 
                                Power[ k, -1], 
                                Plus[ k, Om]]], 
                            Times[ -1, 
                              Log[ 
                                Plus[ -1, 
                                  Times[ 
                                    Power[ k, -1], Om]]]]]]]]]], 
                Times[ -1, 
                  Plus[ 1, 
                    Times[ a, 
                      Power[ k, -2], 
                      Power[ Om, 2]]], 
                  Plus[ 
                    Times[ -2, a, 
                      Plus[ 1, a], 
                      Power[ k, -2], Om, 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 2, a, 
                      Plus[ 1, a], 
                      Power[ k, -2], Om, 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Complex[ 0, 2], 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Power[ k, -2], 
                      Power[ 
                        Plus[ k, 
                          Times[ -1, Om]], -1], 
                      Power[ 
                        Plus[ k, Om], -1], 
                      Plus[ 
                        Times[ 2, Om, 
                          Plus[ 
                            Power[ k, 2], 
                            Times[ a, 
                              Power[ Om, 2]]]], 
                        Times[ 
                          Plus[ 1, a], k, 
                          Plus[ 
                            Power[ k, 2], 
                            Times[ -1, 
                              Power[ Om, 2]]], 
                          Log[ 
                            Times[ 
                              Power[ k, -1], 
                              Plus[ k, Om]]]], 
                        Times[ -1, 
                          Plus[ 1, a], k, 
                          Plus[ 
                            Power[ k, 2], 
                            Times[ -1, 
                              Power[ Om, 2]]], 
                          Log[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]], 
              Plus[ 
                Times[ -1, 
                  Power[ k, 2]], 
                Power[ Om, 2], 
                Times[ 
                  Complex[ 0, 
                    Rational[ 1, 8]], 
                  Power[ 
                    Times[ a, 
                      Power[ 
                        Plus[ 1, a], -1]], 
                    Rational[ -1, 2]], 
                  Power[ 
                    Plus[ 1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], -2], 
                  Plus[ 
                    Times[ 
                      Power[ k, -4], 
                      Plus[ 
                        Times[ 
                          Plus[ -1, a], 
                          Power[ k, 4]], 
                        Times[ -1, 
                          Plus[ 1, 
                            Times[ 6, a], 
                            Power[ a, 2]], 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ -1, 
                          Plus[ -1, a], a, 
                          Power[ Om, 4]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Plus[ 1, 
                        Times[ 
                          Power[ k, -2], 
                          Power[ Om, 2]], 
                        Times[ a, 
                          Plus[ -1, 
                            Times[ 
                              Plus[ 6, a], 
                              Power[ k, -2], 
                              Power[ Om, 2]], 
                            Times[ 
                              Plus[ -1, a], 
                              Power[ k, -4], 
                              Power[ Om, 4]]]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Complex[ 0, 1], 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Plus[ -1, 
                        Times[ 
                          Power[ k, -2], 
                          Power[ Om, 2]]], 
                      Plus[ 2, 
                        Times[ 2, a, 
                          Power[ k, -2], 
                          Power[ Om, 2]], 
                        Times[ 2, 
                          Plus[ 1, a], 
                          Power[ k, -1], Om, 
                          Plus[ 
                            Times[ -1, 
                              Log[ 
                                Times[ 
                                  Power[ k, -1], 
                                  Plus[ k, Om]]]], 
                            Log[ 
                              Plus[ -1, 
                                Times[ 
                                  Power[ k, -1], Om]]]]]]]]]]]]], 
        Times[ 
          Rational[ 1, 4], 
          Plus[ 
            Times[ 
              Complex[ 0, 
                Rational[ -1, 2]], 
              Power[ 
                Times[ a, 
                  Power[ 
                    Plus[ 1, a], -1]], 
                Rational[ -1, 2]], 
              Power[ 
                Plus[ k, 
                  Times[ -1, Om]], -1], 
              Power[ 
                Plus[ k, Om], -1], 
              Power[ 
                Plus[ 
                  Power[ k, 2], 
                  Times[ a, 
                    Power[ Om, 2]]], -6], 
              Plus[ 
                Times[ -2, 
                  Power[ k, 4], Om], 
                Times[ -4, a, 
                  Power[ k, 4], Om], 
                Times[ -4, 
                  Power[ a, 2], 
                  Power[ k, 2], 
                  Power[ Om, 3]], 
                Times[ 2, 
                  Power[ a, 2], 
                  Power[ Om, 5]], 
                Times[ 
                  Complex[ 0, -1], 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ 1, a], Om, 
                  Plus[ 
                    Times[ -1, 
                      Power[ k, 2]], 
                    Power[ Om, 2]], 
                  Plus[ 
                    Times[ -3, 
                      Power[ k, 2]], 
                    Times[ a, 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, -1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Complex[ 0, 1], 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ 1, a], Om, 
                  Plus[ 
                    Times[ -1, 
                      Power[ k, 2]], 
                    Power[ Om, 2]], 
                  Plus[ 
                    Times[ -3, 
                      Power[ k, 2]], 
                    Times[ a, 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ -1, 
                  Power[ k, 5], 
                  Log[ 
                    Times[ 
                      Power[ k, -1], 
                      Plus[ k, Om]]]], 
                Times[ -1, a, 
                  Power[ k, 5], 
                  Log[ 
                    Times[ 
                      Power[ k, -1], 
                      Plus[ k, Om]]]], 
                Times[ 
                  Power[ k, 3], 
                  Power[ Om, 2], 
                  Log[ 
                    Times[ 
                      Power[ k, -1], 
                      Plus[ k, Om]]]], 
                Times[ 4, a, 
                  Power[ k, 3], 
                  Power[ Om, 2], 
                  Log[ 
                    Times[ 
                      Power[ k, -1], 
                      Plus[ k, Om]]]], 
                Times[ 3, 
                  Power[ a, 2], 
                  Power[ k, 3], 
                  Power[ Om, 2], 
                  Log[ 
                    Times[ 
                      Power[ k, -1], 
                      Plus[ k, Om]]]], 
                Times[ -3, a, k, 
                  Power[ Om, 4], 
                  Log[ 
                    Times[ 
                      Power[ k, -1], 
                      Plus[ k, Om]]]], 
                Times[ -3, 
                  Power[ a, 2], k, 
                  Power[ Om, 4], 
                  Log[ 
                    Times[ 
                      Power[ k, -1], 
                      Plus[ k, Om]]]], 
                Times[ 
                  Power[ k, 5], 
                  Log[ 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -1], Om]]]], 
                Times[ a, 
                  Power[ k, 5], 
                  Log[ 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -1], Om]]]], 
                Times[ -1, 
                  Power[ k, 3], 
                  Power[ Om, 2], 
                  Log[ 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -1], Om]]]], 
                Times[ -4, a, 
                  Power[ k, 3], 
                  Power[ Om, 2], 
                  Log[ 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -1], Om]]]], 
                Times[ -3, 
                  Power[ a, 2], 
                  Power[ k, 3], 
                  Power[ Om, 2], 
                  Log[ 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -1], Om]]]], 
                Times[ 3, a, k, 
                  Power[ Om, 4], 
                  Log[ 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -1], Om]]]], 
                Times[ 3, 
                  Power[ a, 2], k, 
                  Power[ Om, 4], 
                  Log[ 
                    Plus[ -1, 
                      Times[ 
                        Power[ k, -1], Om]]]]], 
              Plus[ 
                Times[ 
                  Power[ 
                    Plus[ 1, a], 
                    Rational[ 3, 2]], 
                  Power[ k, 2], Om, 
                  Plus[ 
                    Times[ -1, 
                      Plus[ 1, 
                        Times[ 3, a]], 
                      Power[ k, 2]], 
                    Times[ a, 
                      Plus[ 3, a], 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, -1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Power[ 
                    Plus[ 1, a], 
                    Rational[ 3, 2]], 
                  Power[ k, 2], Om, 
                  Plus[ 
                    Times[ 
                      Plus[ 1, 
                        Times[ 3, a]], 
                      Power[ k, 2]], 
                    Times[ -1, a, 
                      Plus[ 3, a], 
                      Power[ Om, 2]]], 
                  Log[ 
                    Plus[ 1, 
                      Times[ 
                        Complex[ 0, 1], 
                        Power[ a, 
                          Rational[ 1, 2]]]]]], 
                Times[ 
                  Complex[ 0, 1], 
                  Power[ a, 
                    Rational[ 1, 2]], 
                  Plus[ 
                    Times[ 4, Om, 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ a, 
                          Power[ Om, 2]]], 
                      Plus[ 
                        Times[ -2, 
                          Power[ k, 4]], 
                        Times[ -2, 
                          Power[ a, 2], 
                          Power[ Om, 4]], 
                        Times[ 
                          Power[ k, 2], 
                          Plus[ 
                            Power[ 
                              Plus[ 1, a], 
                              Rational[ 1, 2]], 
                            Times[ a, 
                              Plus[ 
                                Power[ 
                                  Plus[ 1, a], 
                                  Rational[ 1, 2]], 
                                Times[ -4, 
                                  Power[ Om, 2]]]]]]]], 
                    Times[ 
                      Power[ 
                        Plus[ 1, a], 
                        Rational[ 3, 2]], k, 
                      Plus[ 
                        Power[ k, 4], 
                        Times[ -3, 
                          Plus[ 1, a], 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ a, 
                          Power[ Om, 4]]], 
                      Log[ 
                        Times[ 
                          Power[ k, -1], 
                          Plus[ k, Om]]]], 
                    Times[ -1, 
                      Power[ 
                        Plus[ 1, a], 
                        Rational[ 3, 2]], k, 
                      Plus[ 
                        Power[ k, 4], 
                        Times[ -3, 
                          Plus[ 1, a], 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ a, 
                          Power[ Om, 4]]], 
                      Log[ 
                        Plus[ -1, 
                          Times[ 
                            Power[ k, -1], Om]]]]]]]], 
            Times[ 4, 
              Plus[ 1, 
                Times[ 
                  Complex[ 0, 
                    Rational[ -1, 4]], 
                  Power[ 
                    Times[ a, 
                      Power[ 
                        Plus[ 1, a], -1]], 
                    Rational[ -1, 2]], 
                  Power[ 
                    Plus[ 
                      Power[ k, 2], 
                      Times[ a, 
                        Power[ Om, 2]]], -2], 
                  Plus[ 
                    Times[ 
                      Plus[ 1, a], 
                      Plus[ 
                        Times[ -1, 
                          Power[ k, 2]], 
                        Times[ a, 
                          Power[ Om, 2]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Plus[ 1, a], 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ -1, a, 
                          Power[ Om, 2]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Complex[ 0, 2], 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ a, 
                          Power[ Om, 2]], 
                        Times[ -1, 
                          Plus[ 1, a], k, Om, 
                          Log[ 
                            Times[ 
                              Power[ k, -1], 
                              Plus[ k, Om]]]], 
                        Times[ 
                          Plus[ 1, a], k, Om, 
                          Log[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]], 
              Plus[ -1, 
                Times[ 
                  Rational[ 1, 8], 
                  Power[ a, 
                    Rational[ -1, 2]], 
                  Power[ 
                    Plus[ 1, a], 
                    Rational[ 3, 2]], k, 
                  Power[ 
                    Plus[ k, 
                      Times[ -1, Om]], -1], 
                  Power[ 
                    Plus[ k, Om], -1], 
                  Power[ 
                    Plus[ 
                      Power[ k, 2], 
                      Times[ a, 
                        Power[ Om, 2]]], -4], 
                  Plus[ 
                    Times[ 
                      Complex[ 0, 1], k, 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ -1, 
                          Power[ Om, 2]]], 
                      Plus[ 
                        Times[ 
                          Plus[ 1, 
                            Times[ 3, a]], 
                          Power[ k, 4]], 
                        Times[ -2, a, 
                          Plus[ 7, 
                            Times[ 9, a]], 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ 3, 
                          Power[ a, 2], 
                          Plus[ 3, a], 
                          Power[ Om, 4]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Complex[ 0, -1], k, 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ -1, 
                          Power[ Om, 2]]], 
                      Plus[ 
                        Times[ 
                          Plus[ 1, 
                            Times[ 3, a]], 
                          Power[ k, 4]], 
                        Times[ -2, a, 
                          Plus[ 7, 
                            Times[ 9, a]], 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ 3, 
                          Power[ a, 2], 
                          Plus[ 3, a], 
                          Power[ Om, 4]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 2, 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Plus[ 
                        Times[ 3, 
                          Power[ k, 7]], 
                        Times[ -5, 
                          Power[ k, 5], 
                          Power[ Om, 2]], 
                        Times[ -6, a, 
                          Power[ k, 5], 
                          Power[ Om, 2]], 
                        Times[ 2, a, 
                          Power[ k, 3], 
                          Power[ Om, 4]], 
                        Times[ -9, 
                          Power[ a, 2], 
                          Power[ k, 3], 
                          Power[ Om, 4]], 
                        Times[ 7, 
                          Power[ a, 2], k, 
                          Power[ Om, 6]], 
                        Times[ Om, 
                          Plus[ 
                            Times[ -1, 
                              Power[ k, 2]], 
                            Power[ Om, 2]], 
                          Plus[ 
                            Times[ 
                              Plus[ 3, 
                                Times[ 6, a]], 
                              Power[ k, 4]], 
                            Times[ -2, a, 
                              Plus[ 4, 
                                Times[ 3, a]], 
                              Power[ k, 2], 
                              Power[ Om, 2]], 
                            Times[ 
                              Power[ a, 2], 
                              Power[ Om, 4]]], 
                          Log[ 
                            Times[ 
                              Power[ k, -1], 
                              Plus[ k, Om]]]], 
                        Times[ Om, 
                          Plus[ 
                            Power[ k, 2], 
                            Times[ -1, 
                              Power[ Om, 2]]], 
                          Plus[ 
                            Times[ 
                              Plus[ 3, 
                                Times[ 6, a]], 
                              Power[ k, 4]], 
                            Times[ -2, a, 
                              Plus[ 4, 
                                Times[ 3, a]], 
                              Power[ k, 2], 
                              Power[ Om, 2]], 
                            Times[ 
                              Power[ a, 2], 
                              Power[ Om, 4]]], 
                          Log[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]]], 
            Times[ 
              Complex[ 0, 2], 
              Power[ 
                Times[ a, 
                  Power[ 
                    Plus[ 1, a], -1]], 
                Rational[ -1, 2]], 
              Power[ k, -2], 
              Power[ 
                Plus[ 1, 
                  Times[ a, 
                    Power[ k, -2], 
                    Power[ Om, 2]]], -3], 
              Plus[ 
                Times[ 
                  Rational[ 1, 2], 
                  Power[ k, -4], 
                  Power[ 
                    Plus[ k, 
                      Times[ -1, Om]], -2], 
                  Power[ 
                    Plus[ k, Om], -2], 
                  Plus[ 
                    Power[ k, 2], 
                    Times[ a, 
                      Power[ Om, 2]]], 
                  Plus[ 
                    Times[ 
                      Complex[ 0, -2], 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Plus[ 
                        Times[ 
                          Plus[ 2, a], 
                          Power[ k, 4]], 
                        Times[ 2, a, 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ -1, a, 
                          Power[ Om, 4]]]], 
                    Times[ a, 
                      Plus[ 1, a], 
                      Power[ 
                        Plus[ 
                          Power[ k, 2], 
                          Times[ -1, 
                            Power[ Om, 2]]], 2], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ -1, a, 
                      Plus[ 1, a], 
                      Power[ 
                        Plus[ 
                          Power[ k, 2], 
                          Times[ -1, 
                            Power[ Om, 2]]], 2], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]]]], 
                Times[ a, 
                  Plus[ 
                    Times[ -1, 
                      Power[ k, 2]], 
                    Times[ 5, a, 
                      Power[ Om, 2]]], 
                  Power[ 
                    Plus[ 
                      Power[ k, 6], 
                      Times[ a, 
                        Power[ k, 4], 
                        Power[ Om, 2]]], -1], 
                  Plus[ 
                    Times[ 
                      Plus[ 1, a], 
                      Plus[ 
                        Times[ -1, 
                          Power[ k, 2]], 
                        Times[ a, 
                          Power[ Om, 2]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Plus[ 1, a], 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ -1, a, 
                          Power[ Om, 2]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Complex[ 0, 2], 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Plus[ 
                        Power[ k, 2], 
                        Times[ a, 
                          Power[ Om, 2]], 
                        Times[ -1, 
                          Plus[ 1, a], k, Om, 
                          Log[ 
                            Times[ 
                              Power[ k, -1], 
                              Plus[ k, Om]]]], 
                        Times[ 
                          Plus[ 1, a], k, Om, 
                          Log[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]], 
                Times[ 2, a, 
                  Power[ k, -2], Om, 
                  Plus[ 
                    Times[ -2, a, 
                      Plus[ 1, a], 
                      Power[ k, -2], Om, 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 2, a, 
                      Plus[ 1, a], 
                      Power[ k, -2], Om, 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Complex[ 0, 2], 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Power[ k, -2], 
                      Power[ 
                        Plus[ k, 
                          Times[ -1, Om]], -1], 
                      Power[ 
                        Plus[ k, Om], -1], 
                      Plus[ 
                        Times[ 2, Om, 
                          Plus[ 
                            Power[ k, 2], 
                            Times[ a, 
                              Power[ Om, 2]]]], 
                        Times[ 
                          Plus[ 1, a], k, 
                          Plus[ 
                            Power[ k, 2], 
                            Times[ -1, 
                              Power[ Om, 2]]], 
                          Log[ 
                            Times[ 
                              Power[ k, -1], 
                              Plus[ k, Om]]]], 
                        Times[ -1, 
                          Plus[ 1, a], k, 
                          Plus[ 
                            Power[ k, 2], 
                            Times[ -1, 
                              Power[ Om, 2]]], 
                          Log[ 
                            Plus[ -1, 
                              Times[ 
                                Power[ k, -1], Om]]]]]]]]], 
              Plus[ 
                Times[ -1, 
                  Power[ k, 2]], 
                Power[ Om, 2], 
                Times[ 
                  Complex[ 0, 
                    Rational[ 1, 8]], 
                  Power[ 
                    Times[ a, 
                      Power[ 
                        Plus[ 1, a], -1]], 
                    Rational[ -1, 2]], 
                  Power[ 
                    Plus[ 1, 
                      Times[ a, 
                        Power[ k, -2], 
                        Power[ Om, 2]]], -2], 
                  Plus[ 
                    Times[ 
                      Power[ k, -4], 
                      Plus[ 
                        Times[ 
                          Plus[ -1, a], 
                          Power[ k, 4]], 
                        Times[ -1, 
                          Plus[ 1, 
                            Times[ 6, a], 
                            Power[ a, 2]], 
                          Power[ k, 2], 
                          Power[ Om, 2]], 
                        Times[ -1, 
                          Plus[ -1, a], a, 
                          Power[ Om, 4]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, -1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Plus[ 1, 
                        Times[ 
                          Power[ k, -2], 
                          Power[ Om, 2]], 
                        Times[ a, 
                          Plus[ -1, 
                            Times[ 
                              Plus[ 6, a], 
                              Power[ k, -2], 
                              Power[ Om, 2]], 
                            Times[ 
                              Plus[ -1, a], 
                              Power[ k, -4], 
                              Power[ Om, 4]]]]], 
                      Log[ 
                        Plus[ 1, 
                          Times[ 
                            Complex[ 0, 1], 
                            Power[ a, 
                              Rational[ 1, 2]]]]]], 
                    Times[ 
                      Complex[ 0, 1], 
                      Power[ a, 
                        Rational[ 1, 2]], 
                      Plus[ -1, 
                        Times[ 
                          Power[ k, -2], 
                          Power[ Om, 2]]], 
                      Plus[ 2, 
                        Times[ 2, a, 
                          Power[ k, -2], 
                          Power[ Om, 2]], 
                        Times[ 2, 
                          Plus[ 1, a], 
                          Power[ k, -1], Om, 
                          Plus[ 
                            Times[ -1, 
                              Log[ 
                                Times[ 
                                  Power[ k, -1], 
                                  Plus[ k, Om]]]], 
                            Log[ 
                              Plus[ -1, 
                                Times[ 
                                  Power[ k, -1], Om]]]]]]]]]]]]]}, 0, 3, 
        1]]], "Output"]
}, Closed]],

Cell[BoxData[
    \(\(LOkx0corrtestfunc[Om_, k_, a_] = 
        Normal[%853[\([3, 1]\)]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(NLOkx0corrtestfunc[Om_, k_, a_] = 
        Normal[%853[\([3, 2]\)]];\)\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(NLOkx0corrtestfunc\)\" \
is similar to existing symbol \"\!\(LOkx0corrtestfunc\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(\(NNLOkx0corrtestfunc[Om_, k_, a_] = 
        Normal[%956[\([3, 3]\)]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[Re[LOkx0corrtestfunc[Om, 2, 10^5]] \[Equal] 0, {Om, 0.5, 0.6}, 
      MaxIterations \[Rule] 160]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8862728316093783`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Im[LOkx0corrtestfunc[0.8862728316093783`, 2, 10^5]]\)], "Input"],

Cell[BoxData[
    \(\(-0.07999201168203071`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[
      Im[LOkx0corrtestfunc[0.8862728316093783`, 2, 10^5] + 
            NLOkx0corrtestfunc[0.8862728316093783`, 2, 
                10^5]\ mGamma] \[Equal] 0, {mGamma, \(-0.5\), \(-0.6\)}, 
      MaxIterations \[Rule] 160]\)], "Input"],

Cell[BoxData[
    \({mGamma \[Rule] \(-0.006327232667774241`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[
      Im[LOkx0corrtestfunc[0.8862728316093783` - I\ 10^\((\(-10\))\), 2, 
              10^5] + NLOkx0corrtestfunc[
                0.8862728316093783` - I\ 10^\((\(-10\))\), 2, 
                10^5]\ mGamma] \[Equal] 0, {mGamma, \(-0.5\), \(-0.6\)}, 
      MaxIterations \[Rule] 160]\)], "Input"],

Cell[BoxData[
    \({mGamma \[Rule] 0.006327232767904589`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[
      Im[LOkx0corrtestfunc[0.8862728316093783` + I\ 10^\((\(-10\))\), 2, 
              10^5] + NLOkx0corrtestfunc[
                0.8862728316093783` + I\ 10^\((\(-10\))\), 2, 
                10^5]\ mGamma] \[Equal] 0, {mGamma, \(-0.5\), \(-0.6\)}, 
      MaxIterations \[Rule] 160]\)], "Input"],

Cell[BoxData[
    \({mGamma \[Rule] \(-0.006327232767904645`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Im[LOkx0corrtestfunc[0.8862728316093783`, 2, 10^5] + 
        NLOkx0corrtestfunc[0.8862728316093783`, 2, 
            10^5] \((\(-0.006327232667774241\))\)]\)], "Input"],

Cell[BoxData[
    \(\(-1.8457457784393227`*^-15\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Im[LOkx0corrtestfunc[0.8862728316093783` - I\ 10^\((\(-10\))\), 2, 
          10^5] + NLOkx0corrtestfunc[
            0.8862728316093783` - I\ 10^\((\(-10\))\), 2, 
            10^5] \((0.006327232667774241)\)]\)], "Input"],

Cell[BoxData[
    \(1.2658978865154324`*^-9\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Im[LOkx0corrtestfunc[0.8862728316093783` + I\ 10^\((\(-10\))\), 2, 
          10^5] + NLOkx0corrtestfunc[
            0.8862728316093783` + I\ 10^\((\(-10\))\), 2, 
            10^5] \((0.006327232667774241)\)]\)], "Input"],

Cell[BoxData[
    \(\(-0.1599840246266647`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[
      Re[LOkx0corrtestfunc[Om, 2, 10^5] + 
            NLOkx0corrtestfunc[Om, 2, 
                10^5]\ \((\(-0.006327232667774241\))\)] \[Equal] 0, {Om, 0.5, 
        0.6}, MaxIterations \[Rule] 160]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8864789578385567`}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(aha, have\ to\ write\ a\ module ... \)], "Input"],

Cell[BoxData[
    \(\(approxkx0sol[k_, a_, eps_] := 
        Module[{Om, mGamma, tester, resarr, cnt}, 
          Om = w /. FindRoot[
                Re[LOkx0corrtestfunc[w, k, a]] \[Equal] 0, {w, 0.5, 0.6}, 
                MaxIterations \[Rule] 100]; \[IndentingNewLine]tester = 
            1; \[IndentingNewLine]mGamma = \(-0.1\); \
\[IndentingNewLine]resarr = {}; \[IndentingNewLine]cnt = 
            0; \[IndentingNewLine]While[\((Abs[tester/mGamma] > 
                  eps)\) && \((cnt < 100)\), 
            tester = \((g /. 
                    FindRoot[
                      Im[LOkx0corrtestfunc[Om + I\ Sign[mGamma]\ eps, k, a] + 
                            NLOkx0corrtestfunc[Om + I\ Sign[mGamma]\ eps, k, 
                                a]\ g] \[Equal] 0, {g, \(-0.1\), \(-0.2\)}, 
                      MaxIterations \[Rule] 100])\) - 
                mGamma; \[IndentingNewLine]mGamma = 
              mGamma + tester; \[IndentingNewLine]Om = 
              w /. FindRoot[
                  Re[LOkx0corrtestfunc[w + I\ Sign[mGamma]\ eps, k, a] + 
                        NLOkx0corrtestfunc[w + I\ Sign[mGamma]\ eps, k, 
                            a]\ mGamma] \[Equal] 0, {w, 0.5, 0.6}, 
                  MaxIterations \[Rule] 100]; \[IndentingNewLine]cnt = 
              cnt + 1; \[IndentingNewLine]If[cnt \[Equal] 100, 
              Print["\<Counter overflow: aborting\>"]]; \
\[IndentingNewLine]Print["\<Quality:   LO\>", 
              LOkx0corrtestfunc[Om + I\ Sign[mGamma]\ eps, k, a]]; 
            Print["\<Quality:  NLO\>", 
              mGamma\ NLOkx0corrtestfunc[Om + I\ Sign[mGamma]\ eps, k, a]]; 
            Print["\<Quality: NNLO\>", 
              mGamma^2\ NNLOkx0corrtestfunc[Om\  + I\ Sign[mGamma]\ eps, k, 
                  a]]; \[IndentingNewLine]Print["\<Om=\>", Om, "\<Gamma=\>", 
              mGamma]]; \ 
          Print["\<Qualitycheck: 0=?\>", 
            kx0corrtestfunc[Om + I\ mGamma, k, a]]; \[IndentingNewLine]{Om, 
            mGamma}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(approxkx0sol[4, 10^5, 10^\((\(-9\))\)]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.0425346898526981`\)\(\[InvisibleSpace]\)\) - 
          0.6771955790230522`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.0425346898526981, -0.67719557902305216]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253468741416944`\) - 
          0.6772538891228658`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042534687414169443, -0.67725388912286577]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.02173103253402368`\) + 
          0.0016895927091042852`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.02173103253402368, 0.0016895927091042852]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.8850788690341932`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.012669462500568773`\),
      SequenceForm[ 
      "Om=", 0.88507886903419319, "Gamma=", 0.012669462500568773],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100934539132`\)\(\[InvisibleSpace]\)\) + 
          0.6771953478701183`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009345391321, 0.67719534787011826]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100690682417`\) + 
          0.6771953428506621`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531006906824168, 0.67719534285066207]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.02172728145428407`\) - 
          0.0016893009047014582`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.02172728145428407, -0.0016893009047014582]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.8850789378858576`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837168716059`\)\),
      SequenceForm[ 
      "Om=", 0.88507893788585756, "Gamma=", -0.01266837168716059],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966221895`\)\(\[InvisibleSpace]\)\) - 
          0.6771953478900161`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662218948, -0.67719534789001612]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223650046`\) - 
          0.6771953478904478`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223650046, -0.67719534789044777]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777171`\) + 
          0.0016893009298195101`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777170999, 0.0016893009298195101]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.8850789378799306`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.012668371781060131`\),
      SequenceForm[ 
      "Om=", 0.88507893787993064, "Gamma=", 0.012668371781060131],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362233`\) + 
          0.6771953478900156`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223622332, 0.67719534789001556]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777143264`\) - 
          0.001689300929817338`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777143264, -0.0016893009298173381]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.012668371781052054`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781052054],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.04253100722362191`\) + 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621908, 0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) - 
          0.0016893009298173049`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, -0.0016893009298173049]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.01266837178105193`\)\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", -0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220342`\)\(\[InvisibleSpace]\)\) - 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203419, -0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:  \
NLO"\[InvisibleSpace]\(\(-0.042531007223621846`\) - 
          0.6771953478900089`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:  NLO", 
        Complex[ -0.042531007223621846, -0.6771953478900089]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality: \
NNLO"\[InvisibleSpace]\(\(-0.021727281777142837`\) + 
          0.0016893009298173008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality: NNLO", 
        Complex[ -0.021727281777142837, 0.0016893009298173008]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.885078937879931`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]0.01266837178105193`\),
      SequenceForm[ 
      "Om=", 0.88507893787993097, "Gamma=", 0.012668371781051929],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Quality:   \
LO"\[InvisibleSpace]\(\(\(0.04253100966220334`\)\(\[InvisibleSpace]\)\) + 
          0.677195347890008`\ \[ImaginaryI]\)\),
      SequenceForm[ "Quality:   LO", 
        Complex[ 0.042531009662203342, 0.67719534789000801]],
      Editable->False]], "Print"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Im[LOkx0corrtestfunc[0.8866688345280878`, 4, 10^5] + 
        NLOkx0corrtestfunc[0.8866688345280878`, 4, 
            10^5] \((\(-0.012670552796748544`\))\)]\)], "Input"],

Cell[BoxData[
    \(9.992007221626409`*^-16\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[
      0.8866688345280878` + I \((\(-0.012670552796748544`\))\)\ , 4, 
      10^5]\)], "Input"],

Cell[BoxData[
    \(\(-0.10568956733429184`\) + 
      1.3415392550501397`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[0.890099910720201 - I\ 0.010154191465976209`, 1, 
      10^4]\)], "Input"],

Cell[BoxData[
    \(\(-0.004172150362678984`\) + 
      0.048560805595648314`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[
      0.8873255655750196`\[InvisibleSpace] - \ I\ 0.00317539434976552`\ , 1, 
      10^5]\)], "Input"],

Cell[BoxData[
    \(\(-0.0004181814253599773`\) + 
      0.015679092192604275`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(approxkx0sol[1, 10^3, 10^\((\(-10\))\)]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.9029298324011136`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.03336565265041583`\)\),
      SequenceForm[ 
      "Om=", 0.90292983240111357, "Gamma=", -0.033365652650415828],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.9030921997929043`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.03415939246649452`\)\),
      SequenceForm[ 
      "Om=", 0.90309219979290434, "Gamma=", -0.03415939246649452],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.90309628829191`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.03417944031265489`\)\),
      SequenceForm[ 
      "Om=", 0.90309628829191002, "Gamma=", -0.034179440312654893],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.9030963914124293`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.034179945964509684`\)\),
      SequenceForm[ 
      "Om=", 0.90309639141242926, "Gamma=", -0.034179945964509684],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.9030963940134531`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.034179958718641865`\)\),
      SequenceForm[ 
      "Om=", 0.9030963940134531, "Gamma=", -0.034179958718641865],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.9030963940790593`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.034179959040341545`\)\),
      SequenceForm[ 
      "Om=", 0.90309639407905928, "Gamma=", -0.034179959040341545],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.9030963940807141`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.034179959048455846`\)\),
      SequenceForm[ 
      "Om=", 0.90309639408071407, "Gamma=", -0.034179959048455846],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Om="\[InvisibleSpace]0.9030963940807558`\
\[InvisibleSpace]"Gamma="\[InvisibleSpace]\(-0.034179959048660544`\)\),
      SequenceForm[ 
      "Om=", 0.90309639408075582, "Gamma=", -0.034179959048660544],
      Editable->False]], "Print"]
}, Open  ]],

Cell[BoxData[
    \(0.8864789578385567`\)], "Input"],

Cell[BoxData[
    \(\(\(0.886478977960205`\)\(\[InvisibleSpace]\)\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["other1", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[Re[kx0corrtestfunc[Om, 2, 10^4]] \[Equal] 0, {Om, 0.5, 0.6}, 
      MaxIterations \[Rule] 160]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8857765122338269`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[{Im[
            kx0corrtestfunc[0.8857765122338269 + I\ mGamma, 2, 
              10^4]] \[Equal] 0}, {mGamma, \(-0.1\), \(-0.2\)}, 
      MaxIterations \[Rule] 160]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(160\) iterations."\)], "Message"],

Cell[BoxData[
    \({mGamma \[Rule] \(-1.5580177690164673`*^-18\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?kx0corrtestfunc\)\)], "Input"],

Cell["Global`kx0corrtestfunc", "Print",
  CellTags->"Info3279609199-6771777"],

Cell[BoxData[
    RowBox[{
      InterpretationBox[GridBox[{
            {GridBox[{
                  {\(kx0corrtestfunc[om_, k_, 
                        a_] := \((om\^2 - k\^2 - 
                            alphakx0korr[om\/k, a])\)\ \((1 - 
                            betakx0korr[om\/k, a]\/om\^2)\)\)}
                  },
                GridBaseline->{Baseline, {1, 1}},
                ColumnWidths->0.999,
                ColumnAlignments->{Left}]}
            },
          GridBaseline->{Baseline, {1, 1}},
          ColumnAlignments->{Left}],
        Definition[ "kx0corrtestfunc"],
        Editable->False], \(FindRoot[
        Re[kx0corrtestfunc[Om, 2, 10^4]] \[Equal] 0, {Om, 0.5, 0.6}, 
        MaxIterations \[Rule] 160]\)}]], "Print",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  CellTags->"Info3279609199-6771777"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[Re[betakx0korr[Om/2, 10^4]] \[Equal] Om^2, {Om, 0.5, 0.6}, 
      MaxIterations \[Rule] 160]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8860363030334242`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Re[betakx0korr[0.8860363030334242`/2, 10^4] - 
        0.8860363030334242`^2]\)], "Input"],

Cell[BoxData[
    \(1.1782169684337873`*^-8\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Re[betakx0korr[0.8857765122338269`/2, 10^4] - 
        0.8857765122338269`^2]\)], "Input"],

Cell[BoxData[
    \(0.0004591115064902507`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(kx0corrtestfunc[om_, k_, 
        a_] := \((1 - betakx0korr[om\/k, a]\/om\^2)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[0.8857765122338269`, 2, 10^4]\)], "Input"],

Cell[BoxData[
    \(\(-0.0005851535676204556`\) + 
      0.04516477254689391`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(real\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kx0corrtestfunc[0.8857765122338269`, 2, 10^4]\)], "Input"],

Cell[BoxData[
    \(2.7761288485832415`*^-9 - 
      0.2531674654627558`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[
      Re[thLXtestfunc[om, 2, 10^\((\(-4\))\)]] \[Equal] 0, {om, 0.5, 
        0.8}]\)], "Input"],

Cell[BoxData[
    \({om \[Rule] 0.8862269568958404`}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.7, 
          0.8}, {mGamma, \(-0.1\), \(-0.2\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.3, 10^4]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8875437871875099`, 
      mGamma \[Rule] \(-0.013017178550815788`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1.3, 10^4]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8875438877217313`, 
      mGamma \[Rule] \(-0.0130170715500317`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Om /. strangesol[k, 10^4], {k, 0.1, 2}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindRoot :: \"frsec\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -11.3233 13.1499 [
[.2619 .49906 -9 -9 ]
[.2619 .49906 9 0 ]
[.5 .49906 -3 -9 ]
[.5 .49906 3 0 ]
[.7381 .49906 -9 -9 ]
[.7381 .49906 9 0 ]
[.97619 .49906 -3 -9 ]
[.97619 .49906 3 0 ]
[.01131 .11706 -24 -4.5 ]
[.01131 .11706 0 4.5 ]
[.01131 .24856 -24 -4.5 ]
[.01131 .24856 0 4.5 ]
[.01131 .38006 -24 -4.5 ]
[.01131 .38006 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .51156 m
.2619 .51781 L
s
[(0.5)] .2619 .49906 0 1 Mshowa
.5 .51156 m
.5 .51781 L
s
[(1)] .5 .49906 0 1 Mshowa
.7381 .51156 m
.7381 .51781 L
s
[(1.5)] .7381 .49906 0 1 Mshowa
.97619 .51156 m
.97619 .51781 L
s
[(2)] .97619 .49906 0 1 Mshowa
.125 Mabswid
.07143 .51156 m
.07143 .51531 L
s
.11905 .51156 m
.11905 .51531 L
s
.16667 .51156 m
.16667 .51531 L
s
.21429 .51156 m
.21429 .51531 L
s
.30952 .51156 m
.30952 .51531 L
s
.35714 .51156 m
.35714 .51531 L
s
.40476 .51156 m
.40476 .51531 L
s
.45238 .51156 m
.45238 .51531 L
s
.54762 .51156 m
.54762 .51531 L
s
.59524 .51156 m
.59524 .51531 L
s
.64286 .51156 m
.64286 .51531 L
s
.69048 .51156 m
.69048 .51531 L
s
.78571 .51156 m
.78571 .51531 L
s
.83333 .51156 m
.83333 .51531 L
s
.88095 .51156 m
.88095 .51531 L
s
.92857 .51156 m
.92857 .51531 L
s
.25 Mabswid
0 .51156 m
1 .51156 L
s
.02381 .11706 m
.03006 .11706 L
s
[(0.87)] .01131 .11706 1 0 Mshowa
.02381 .24856 m
.03006 .24856 L
s
[(0.88)] .01131 .24856 1 0 Mshowa
.02381 .38006 m
.03006 .38006 L
s
[(0.89)] .01131 .38006 1 0 Mshowa
.125 Mabswid
.02381 .14336 m
.02756 .14336 L
s
.02381 .16966 m
.02756 .16966 L
s
.02381 .19596 m
.02756 .19596 L
s
.02381 .22226 m
.02756 .22226 L
s
.02381 .27486 m
.02756 .27486 L
s
.02381 .30116 m
.02756 .30116 L
s
.02381 .32746 m
.02756 .32746 L
s
.02381 .35376 m
.02756 .35376 L
s
.02381 .40636 m
.02756 .40636 L
s
.02381 .43266 m
.02756 .43266 L
s
.02381 .45896 m
.02756 .45896 L
s
.02381 .48526 m
.02756 .48526 L
s
.02381 .09076 m
.02756 .09076 L
s
.02381 .06446 m
.02756 .06446 L
s
.02381 .03816 m
.02756 .03816 L
s
.02381 .01186 m
.02756 .01186 L
s
.02381 .53786 m
.02756 .53786 L
s
.02381 .56416 m
.02756 .56416 L
s
.02381 .59046 m
.02756 .59046 L
s
.02381 .61676 m
.02756 .61676 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .25793 m
.0893 .25886 L
.10813 .26017 L
.12724 .26185 L
.14816 .26412 L
.15313 .26473 L
.15852 .26542 L
.16099 .26574 L
.16361 .26609 L
.16585 .2664 L
.16646 .26649 L
.16712 .26659 L
.16831 .26591 L
.16953 .26692 L
.17066 .26708 L
.17322 .26746 L
.17788 .26816 L
s
.17788 .26816 m
.17812 0 L
s
.17971 0 m
.17993 .26749 L
.18096 .26863 L
s
.18096 .26863 m
.18129 0 L
s
.1829 0 m
.18324 .26897 L
.18446 .26919 L
.18664 .26954 L
s
.18664 .26954 m
.18703 0 L
s
.18755 0 m
.18794 .22495 L
.18918 .26996 L
.19038 .27016 L
.19149 .55906 L
.19254 .27052 L
.1937 .49466 L
.19491 .50998 L
.19605 .53135 L
.19709 .53998 L
.19819 .45136 L
.19912 .43701 L
s
.19912 .43701 m
.19992 0 L
s
.20161 0 m
.20244 .27229 L
.20356 .12824 L
s
.20356 .12824 m
.20455 0 L
s
.20505 0 m
.20577 .00463 L
.20637 .00856 L
.20703 .45781 L
.2082 .47308 L
.20929 .18119 L
.20995 .18597 L
.21058 .50044 L
.21176 .51134 L
.21237 .51617 L
.21302 .52061 L
.21374 .05486 L
.21441 .05854 L
.21561 .06482 L
.21629 .54112 L
.21691 .3529 L
.21806 .27529 L
.21928 .27553 L
.22037 .27577 L
.22138 .23631 L
.22249 .23555 L
.22369 .23398 L
.22485 .1247 L
.22551 .27524 L
.22613 .27698 L
.22738 .46534 L
.22808 .27741 L
.22872 .27755 L
.23338 .27859 L
.23456 .27886 L
.23521 .279 L
.23583 .27915 L
.23656 .35174 L
.23722 .34545 L
.23784 .27075 L
.23851 .31852 L
.23916 .27992 L
.23987 .27824 L
.24062 .26041 L
.24132 .27852 L
.24256 .28074 L
.24326 .28091 L
.2439 .26012 L
.24509 .27938 L
.24568 .35632 L
.24634 .28165 L
.24751 .27996 L
.2486 .32443 L
.24923 .28037 L
.24989 .34302 L
Mistroke
.25107 .215 L
Mfstroke
.25107 .215 m
.25126 0 L
s
.25228 0 m
.25233 .04277 L
.25305 .28334 L
.25372 .2814 L
.25492 .28172 L
.2556 .28401 L
.25623 .28414 L
.25738 .28233 L
.2586 .28313 L
.25962 .28508 L
.26072 .28538 L
.26192 .28564 L
.26303 .28597 L
.26543 .28672 L
.2677 .28738 L
.26976 .2879 L
.27198 .2885 L
.27264 .2888 L
.27333 .28892 L
.27458 .28686 L
.27575 .48219 L
.27641 .49594 L
.27703 .50849 L
.27938 .55713 L
.28053 .58115 L
.2816 .60332 L
.28262 .29179 L
.28371 .29212 L
.28599 .29286 L
.29 .29417 L
.29899 .29723 L
.33487 .31171 L
.35481 .32177 L
.37309 .33285 L
.39214 .34715 L
.40986 .36453 L
.41856 .37545 L
.42785 .38994 L
.43816 .41152 L
.44262 .42327 L
.44515 .43039 L
.44752 .43672 L
.44866 .4393 L
.44974 .44119 L
.45099 .44232 L
.45213 .44202 L
.45334 .44016 L
.45447 .43726 L
.45701 .42902 L
.46152 .41607 L
Mistroke
.46576 .40712 L
.47078 .39917 L
.47533 .39356 L
.48552 .38427 L
.49483 .37815 L
.50465 .37317 L
.52221 .36655 L
.54092 .36147 L
.55835 .35786 L
.59539 .35236 L
.61562 .35016 L
.63475 .34842 L
.67123 .34577 L
.71004 .34364 L
.74741 .34204 L
.78334 .34079 L
.82161 .33971 L
.85843 .33884 L
.89758 .33807 L
.93529 .33743 L
.97156 .33691 L
.97619 .33685 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool09_oo0`000ooo0P00
2ooo0P00aooo000Kool00`00ooooo`0Vool30003ool2000;ool20037ool001_oo`03003ooooo02Ko
o`<000?oo`8000_oo`800<Ooo`006ooo0P009ooo0`000ooo0P002ooo0P00aooo000Kool00`00oooo
o`0Vool30003ool2000;ool20037ool001_oo`03003ooooo02Koo`<000?oo`8000_oo`800<Ooo`00
6ooo00<00?ooool09_oo0`000ooo0P002ooo0P00aooo000Kool00`00ooooo`0Vool30003ool2000;
ool20037ool001_oo`03003ooooo02Koo`<000?oo`8000_oo`800<Ooo`006ooo00<00?ooool09_oo
0`000ooo0P002ooo0P00aooo000Kool2000Wool30003ool2000;ool20037ool001_oo`03003ooooo
02Koo`<000?oo`8000_oo`800<Ooo`006ooo00<00?ooool09_oo0`000ooo0P002ooo0P00aooo000K
ool00`00ooooo`0Vool30003ool2000;ool20037ool001_oo`03003ooooo02Koo`<000?oo`800003
ool00?oo00Soo`800<Ooo`006ooo00<00?ooool09_oo0`000ooo0P0000?oo`0000002?oo0P00aooo
000Kool00`00ooooo`0Vool30003ool200000ooo00000008ool20037ool001_oo`8002Ooo`<000?o
o`800003ool0000000Soo`800<Ooo`006ooo00<00?ooool09_oo0`000ooo0P0000?oo`0000002?oo
0P00aooo000Kool00`00ooooo`0Vool30003ool200000ooo00000008ool20037ool001_oo`03003o
oooo02Koo`<000?oo`800003ool0000000Soo`800<Ooo`006ooo00<00?ooool09_oo0`000ooo0P00
00?oo`0000002?oo0P00aooo000Kool00`00ooooo`0Vool30003ool200000ooo00000008ool20037
ool001_oo`03003ooooo02Koo`<000?oo`800003ool0000000Soo`800<Ooo`006ooo0P009ooo0`00
0ooo0P0000?oo`0000002?oo0P00aooo000Kool00`00ooooo`0Vool30003ool200000ooo00000008
ool20037ool001_oo`03003ooooo02Koo`<000?oo`800003ool0000000Soo`800<Ooo`006ooo00<0
0?ooool09_oo0`000ooo0P0000?oo`0000002?oo0P00aooo000Kool00`00ooooo`0Vool30003ool2
00000ooo00000008ool20037ool000;oo`8000Coo`03003ooooo00?oo`8000Coo`03003ooooo00Co
o`03003ooooo02Koo`<000?oo`800003ool0000000Soo`800<Ooo`0000Goo`00ooooo`0000Soo`04
003ooooo0003ool00`00ooooo`04ool00`00ooooo`0Vool30003ool200000ooo00000008ool20037
ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool00ooo0P009ooo0`000ooo0P00
00?oo`0000002?oo0P00aooo00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool00ooo00<00?oo
ool09_oo0`000ooo0P0000?oo`0000002?oo0P00aooo00001Ooo003ooooo00002?oo00@00?ooool0
00Goo`03003ooooo00;oo`03003ooooo02Koo`<000?oo`800005ool00000ool00006ool20037ool0
0005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo02Koo`<000?o
o`800005ool00000ool00006ool20037ool000;oo`8000[oo`8000?oo`@000Coo`03003ooooo02Ko
o`<000?oo`800005ool00000ool00006ool20037ool001_oo`03003ooooo02Koo`<000?oo`800005
ool00000ool00006ool20037ool001_oo`03003ooooo02Koo`<000?oo`800005ool00000ool00006
ool20037ool001_oo`8002Ooo`<000?oo`800005ool00000ool00006ool20037ool001_oo`03003o
oooo02Koo`<000?oo`800005ool00000ool00006ool20037ool001_oo`03003ooooo02Koo`<000?o
o`800005ool00000ool00006ool20037ool001_oo`03003ooooo02Koo`<000?oo`800005ool00000
ool00006ool20037ool001_oo`03003ooooo02Koo`<000?oo`800005ool00000ool00006ool20037
ool001_oo`03003ooooo02Koo`<000?oo`800005ool00000ool00006ool20037ool001_oo`03003o
oooo02Koo`<000?oo`800005ool00000ool00006ool20037ool001_oo`8002Ooo`<000?oo`800005
ool00000ool00006ool20037ool001_oo`03003ooooo02Koo`<000?oo`800005ool00000ool00006
ool20037ool001_oo`03003ooooo02Koo`<000?oo`800005ool00000ool00006ool20037ool001_o
o`03003ooooo02Koo`<000?oo`D00003ool00?oo00Goo`800<Ooo`006ooo00<00?ooool09_oo0`00
0ooo1@0000?oo`00ool01Ooo0P00aooo000Kool00`00ooooo`0Vool30003ool500000ooo003oo`05
ool20037ool001_oo`03003ooooo02Koo`<000?oo`D00003ool00?oo00Goo`800<Ooo`006ooo0P00
9ooo0`000ooo1@0000?oo`00ool01Ooo0P00aooo000Kool00`00ooooo`0Vool30003ool500000ooo
003oo`05ool20037ool001_oo`03003ooooo02Koo`<000?oo`D00003ool00?oo00Goo`800<Ooo`00
6ooo00<00?ooool09_oo0`000ooo1@0000?oo`00ool01Ooo0P00aooo000Kool00`00ooooo`0Vool3
0003ool500000ooo003oo`05ool20037ool001_oo`03003ooooo02Koo`<000?oo`D00003ool00?oo
00Goo`800<Ooo`006ooo00<00?ooool09_oo0`000ooo1@0000?oo`00ool01Ooo0P00aooo000Kool2
000Wool30002ool600000ooo003oo`05ool20037ool001_oo`03003ooooo02Koo`<000;oo`H00003
ool00?oo00Goo`800<Ooo`006ooo00<00?ooool09_oo0`000_oo1P0000?oo`00ool01Ooo0P00aooo
000Kool00`00ooooo`0Vool30002ool600000ooo003oo`05ool20037ool001_oo`03003ooooo02Ko
o`<000;oo`800003ool0000000<000Koo`800<Ooo`000_oo0P001?oo00<00?ooool00ooo0P001?oo
0P001Ooo00<00?ooool09_oo0`000_oo0P0000?oo`0000000`001_oo0P00aooo00001Ooo003ooooo
00002?oo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`0Vool30002ool200000ooo0000
00030006ool20037ool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000Coo`80
02Ooo`<000;oo`800003ool0000000<000Koo`800<Ooo`0000Goo`00ooooo`0000Woo`8000Coo`80
00Goo`03003ooooo02Koo`@00004ool00000ool50006ool20037ool00005ool00?ooool00008ool0
1000ooooo`000_oo00@00?ooool000Coo`03003ooooo00Woo`<001[oo`@00004ool00000ool50006
ool20037ool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo
00coo`l000_oo`@00004ool00000ool50004ool01000ool00000aooo0002ool2000:ool20004ool2
0005ool00`00ooooo`0Kool70004ool400001?oo00000?oo1@001?oo00@00?oo00000<Ooo`006ooo
00<00?ooool08Ooo2@0000Coo`00003oo`D000Coo`04003oo`000037ool001_oo`03003ooooo02So
o`D00003ool0000000<000?oo`D00<Ooo`006ooo0P00:_oo0`000_oo1@000ooo1@00aooo000Kool0
0`00ooooo`0Yool30002ool30002ool80037ool001_oo`03003ooooo02Woo`<000;oo`<000;oo`03
003oo`0000H00<Koo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00D00?oo00000?oo008000;o
o`<00003ool00?oo0<3oo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00D00?oo00000?oo0080
00Goo`800003ool00?oo0;koo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00D00?oo00000?oo
008000Koo`03003oo`0000800;goo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00D00?oo0000
0?oo008000Koo`03003oo`0000;oo`<00;[oo`006ooo0P00:_oo0`000_oo0`000_oo00D00?oo0000
0?oo008000Koo`03003oo`0000Goo`800;Soo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00D0
0?oo00000?oo008000Koo`03003oo`0000Ooo`<00;Goo`006ooo00<00?ooool0:Ooo0`000_oo0`00
0_oo00D00?oo00000?oo008000Koo`03003oo`0000[oo`800;?oo`006ooo00<00?ooool0:Ooo0`00
0_oo0`000_oo00D00?oo00000?oo008000Koo`03003oo`0000coo`<00;3oo`006ooo00<00?ooool0
:Ooo0`000_oo0`000_oo00D00?oo00000?oo008000Koo`03003oo`0000ooo`800:koo`006ooo00<0
0?ooool0:Ooo0`000_oo0`000_oo00D00?oo00000?oo008000Koo`03003oo`00017oo`800:coo`00
6ooo00<00?ooool0:Ooo0`000_oo0`000_oo00D00?oo00000?oo008000Koo`03003oo`0001?oo`80
0:[oo`006ooo0P00:_oo0`000_oo0`000_oo00D00?oo00000?oo008000Koo`03003oo`0001Goo`80
0:Soo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00D00?oo00000?oo008000Koo`03003oo`00
01Ooo`03003ooooo0:Goo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00D00?oo00000?oo0080
00Koo`03003oo`0001Soo`03003ooooo09Koo`L000Ooo`006ooo00<00?ooool0:Ooo0`000_oo0`00
0_oo00D00?oo00000?oo008000Koo`03003oo`0001Woo`8006Cooc8000koo`006ooo00<00?ooool0
:Ooo0`000_oo0`000_oo00D00?oo00000?oo008000Koo`03003oo`0001_oo`03003ooooo04gooa@0
043oo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00H00?oo00000?oo0007ool00`00ool0000L
ool00`00ooooo`12ool:001Dool001_oo`03003ooooo02Woo`<000;oo`<000;oo`03003oo`0000;o
o`03003ooooo00Goo`03003oo`0001goo`03003ooooo03Goo``005koo`006ooo0P00:_oo0`000_oo
0`000_oo00<00?ooool00_oo00<00?ooool01Ooo00<00?oo00007_oo00<00?ooool0;?oo2000J_oo
000Kool00`00ooooo`0Yool30002ool30002ool00`00ooooo`02ool00`00ooooo`05ool00`00ool0
000Oool00`00ooooo`0Tool7001bool001_oo`03003ooooo02Woo`<000;oo`<000;oo`03003ooooo
00[oo`03003oo`00023oo`03003ooooo01koo`D007Woo`006ooo00<00?ooool0:Ooo0`000_oo0`00
0_oo00<00?ooool02_oo00<00?oo00008Ooo00<00?ooool06Ooo1000O_oo000Kool00`00ooooo`0Y
ool30002ool30002ool00`00ooooo`0:ool00`00ool0000Rool00`00ooooo`0Fool20022ool000;o
o`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo02Woo`<000;oo`<000;oo`03003o
oooo00[oo`03003oo`0002;oo`03003ooooo01Coo`8008Coo`0000Goo`00ooooo`0000Soo`04003o
oooo0005ool00`00ooooo`02ool00`00ooooo`0Yool30002ool30002ool00`00ooooo`0:ool00`00
ool0000Sool00`00ooooo`0Aool20026ool00005ool00?ooool00008ool01000ooooo`001Ooo00<0
0?ooool00_oo0P00:_oo0`000_oo0`000_oo00<00?ooool02_oo00<00?oo00008ooo00<00?ooool0
4?oo00<00?ooool0Q_oo00001Ooo003ooooo00002Ooo0P001?oo0`001?oo00<00?ooool0:Ooo0`00
0_oo0`000_oo00<00?ooool02_oo00<00?oo00009?oo00<00?ooool03_oo00<00?ooool0Qooo0000
1Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`0Yool30002ool3
0002ool00`00ooooo`0:ool00`00ool0000Tool00`00ooooo`0=ool00`00ooooo`28ool00005ool0
0?ooool00008ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo02Woo`<000;oo`<000;o
o`03003ooooo00[oo`03003oo`0002Goo`03003ooooo00_oo`03003ooooo08Woo`000_oo0P002_oo
0P001?oo0P001Ooo00<00?ooool0:Ooo0`000_oo0`000_oo00<00?ooool02_oo00<00?oo00009_oo
00<00?ooool02Ooo00<00?ooool0R_oo000Kool00`00ooooo`0Yool30002ool30002ool00`00oooo
o`0:ool00`00ool0000Vool00`00ooooo`08ool00`00ooooo`2;ool001_oo`03003ooooo02Woo`<0
00;oo`<000;oo`03003ooooo00[oo`03003oo`0002Ooo`03003ooooo00Koo`03003ooooo08coo`00
6ooo0P00:_oo0`000_oo0`000_oo00<00?ooool02_oo00<00?oo00009ooo00<00?ooool01Ooo00<0
0?ooool0SOoo000Kool00`00ooooo`0Yool30002ool30002ool00`00ooooo`0:ool00`00ool0000X
ool00`00ooooo`04ool00`00ooooo`2=ool001_oo`03003ooooo02Woo`<000;oo`<000;oo`03003o
oooo00[oo`03003oo`0002Soo`03003ooooo00?oo`03003ooooo08koo`006ooo00<00?ooool0:Ooo
0`000_oo0`000_oo00<00?ooool02_oo00<00?oo0000:Ooo00<00?ooool00_oo00<00?ooool0S_oo
000Kool00`00ooooo`0Yool30002ool30002ool00`00ooooo`0:ool00`00ool0000Yool00`00oooo
o`02ool00`00ooooo`2>ool001_oo`03003ooooo02Woo`<000;oo`<000;oo`03003ooooo00[oo`03
003oo`0002Woo`05003oooooool0002Aool001_oo`03003ooooo02Woo`<000;oo`<000;oo`03003o
oooo00[oo`03003oo`0002[oo`04003ooooo002Aool001_oo`8002[oo`<000;oo`<000;oo`03003o
oooo00[oo`03003oo`0002[oo`04003ooooo002Aool001_oo`03003ooooo02Woo`<000;oo`<000;o
o`03003ooooo00[oo`03003oo`0002_oo`8009;oo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo
00<00?ooool02_oo00<00?oo0000:ooo0P00T_oo000Kool00`00ooooo`0Yool30002ool30002ool0
0`00ooooo`0:ool00`00ool0002oool001_oo`03003ooooo02Woo`<000;oo`<000;oo`03003ooooo
00[oo`800<3oo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00<00?ooool02_oo0P00`?oo000K
ool00`00ooooo`0Yool30002ool30002ool00`00ooooo`0:ool20030ool001_oo`8002[oo`<000;o
o`<000;oo`03003ooooo00[oo`800<3oo`006ooo00<00?ooool0:Ooo0`000_oo0`000_oo00<00?oo
ool02_oo0P00`?oo000Kool00`00ooooo`0Yool30002ool30002ool00`00ooooo`0:ool20030ool0
01_oo`03003ooooo02Woo`<000;oo`<000ooo`800<3oo`006ooo00<00?ooool0:Ooo0`000_oo0`00
1?oo0P001?oo00<00?ooool00_oo0`00=_oo1000=_oo10000ooo00<00?ooool00_oo0`00=_oo1000
1_oo000Kool00`00ooooo`0Yool30003ool20003ool01000ooooo`002?oo0P0000?oo`00ool0=_oo
00<00?ooool0=ooo00<00?ooool02_oo00<00?ooool0<ooo00<00?ooool01ooo000Kool00`00oooo
o`0Yool30003ool20003ool01000ooooo`002?oo0P0000?oo`00ool0=_oo00<00?ooool0=ooo00<0
0?ooool02_oo00<00?ooool0=?oo00<00?ooool01_oo000Kool2000Zool30003ool20003ool01000
ooooo`002?oo0`00>?oo00<00?ooool0=ooo00<00?ooool01ooo0`00>?oo00<00?ooool01Ooo000K
ool00`00ooooo`0Yool30003ool20003ool01000ooooo`002?oo0P00>Ooo00<00?ooool0=ooo00<0
0?ooool01ooo00<00?ooool0>Ooo00<00?ooool01?oo000Kool00`00ooooo`0Yool30003ool20003
ool01000ooooo`002?oo0P00=ooo0`00=ooo0`002Ooo00<00?ooool0=_oo00@00?ooool000Koo`00
6ooo00<00?ooool0:Ooo0`000ooo0P001?oo0P002Ooo1000=ooo00<00?ooool0=ooo00<00?ooool0
1ooo1000=_oo0P001ooo000Kool00`00ooooo`0Yool30003ool2000?ool20030ool001_oo`03003o
oooo02Woo`<000?oo`8000ooo`800<3oo`006ooo00<00?ooool0:Ooo0`000ooo0P003ooo0P00`?oo
000Doooo000=000001_oo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo
00?oo`<000?oo`8000_oo`04003oooooool20007ool00`00ooooo`09ool00`00ooooo`0:ool00`00
ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`09ool00`00ooooo`0:ool00`00ooooo`05ool001_oo`03003ooooo02Woo`<000?oo`8000_o
o`04003oooooool2000iool00`00ooooo`0mool00`00ooooo`0lool00`00ooooo`05ool001_oo`03
003ooooo02Woo`03003oo`0000Coo`03003ooooo00goo`800<3oo`006ooo00<00?ooool0:Ooo00<0
0?oo00001?oo00<00?ooool03Ooo0P00`?oo000Kool00`00ooooo`0Yool00`00ool00004ool00`00
ooooo`0=ool20030ool001_oo`03003ooooo02Woo`03003oo`0000Coo`03003ooooo00koo`03003o
oooo0;koo`006ooo0P00:_oo00<00?oo00001?oo00<00?ooool03_oo00<00?ooool0__oo000Kool0
0`00ooooo`0Yool00`00ooooo`04ool00`00ooooo`0>ool00`00ooooo`2nool001_oo`03003ooooo
02Woo`03003ooooo01Goo`03003ooooo0;koo`006ooo00<00?ooool0:Ooo00<00?ooool05Ooo00<0
0?ooool0__oo000Kool00`00ooooo`0Yool00`00ooooo`0Eool00`00ooooo`2nool001_oo`03003o
oooo02Woo`03003ooooo01Goo`03003ooooo0;koo`006ooo00<00?ooool0:Ooo00<00?ooool05Ooo
00<00?ooool0__oo000Kool20012ool00`00ooooo`2nool001_oo`03003ooooo047oo`03003ooooo
0;koo`006ooo00<00?ooool0@Ooo00<00?ooool0__oo000Kool00`00ooooo`11ool00`00ooooo`2n
ool001_oo`03003ooooo047oo`03003ooooo0;koo`006ooo00<00?ooool0@Ooo00<00?ooool0__oo
000Kool00`00ooooo`11ool00`00ooooo`2nool001_oo`8004;oo`03003ooooo0;koo`006ooo00<0
0?ooool0@Ooo00<00?ooool0__oo000Kool00`00ooooo`11ool00`00ooooo`2nool001_oo`03003o
oooo047oo`03003ooooo0;koo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool0
01_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo003ooolQool00?ooob7oo`00oooo8Ooo003o
oolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.215729, 0.859302, \
0.00789457, 0.000285882}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[k, {k, 0, 2}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0147151 0.294302 [
[.2619 .00222 -9 -9 ]
[.2619 .00222 9 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -9 -9 ]
[.7381 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .16187 -18 -4.5 ]
[.01131 .16187 0 4.5 ]
[.01131 .30902 -6 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .45617 -18 -4.5 ]
[.01131 .45617 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(0.5)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(1.5)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(2)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .16187 m
.03006 .16187 L
s
[(0.5)] .01131 .16187 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1)] .01131 .30902 1 0 Mshowa
.02381 .45617 m
.03006 .45617 L
s
[(1.5)] .01131 .45617 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(2)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .03859 L
.10458 .06463 L
.14415 .08909 L
.18221 .11261 L
.22272 .13765 L
.26171 .16175 L
.30316 .18736 L
.34309 .21204 L
.3815 .23578 L
.42237 .26104 L
.46172 .28536 L
.49955 .30874 L
.53984 .33364 L
.57861 .3576 L
.61984 .38308 L
.65954 .40762 L
.69774 .43123 L
.73838 .45635 L
.77751 .48053 L
.81909 .50623 L
.85916 .53099 L
.89771 .55482 L
.93871 .58016 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool004ooo`8000Coo`03003ooooo00;oo`<003Ooo`@003Ooo`@000?oo`03003ooooo00;oo`<003So
o`@000Koo`00C_oo00@00?ooool000_oo`03003ooooo03Koo`03003ooooo03Soo`03003ooooo00[o
o`03003ooooo03Goo`03003ooooo00Ooo`00C_oo00@00?ooool000_oo`03003ooooo03Koo`03003o
oooo03Soo`03003ooooo00[oo`03003ooooo03Koo`03003ooooo00Koo`00C_oo00@00?ooool000So
o`<003Woo`03003ooooo03Soo`03003ooooo00Ooo`<003[oo`03003ooooo00Goo`00C_oo00@00?oo
ool000Soo`03003ooooo03Woo`03003ooooo03Soo`03003ooooo00Ooo`03003ooooo03_oo`03003o
oooo00Coo`00C_oo00@00?ooool000Soo`03003ooooo03Ooo`<003Soo`<000Woo`03003ooooo03So
o`04003ooooo0006ool001Goo`03003ooooo03Ooo`8000Woo`@003Soo`03003ooooo03Soo`03003o
oooo00Ooo`@003Soo`8000Ooo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool0
01Goo`03003ooooo0?ooo`Woo`003oooo`004@000Ooo000Eool3000:ool00`00ooooo`0:ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`05ool001Goo`03003ooooo0080
03coo`03003ooooo03goo`03003ooooo03koo`03003ooooo03koo`03003ooooo00Goo`005Ooo00<0
0?ooool00_oo00<00?ooool0oooo1?oo000Eool00`00ooooo`03ool2003oool4ool001Goo`03003o
oooo00Goo`800?ooo`;oo`005Ooo00<00?ooool01ooo00<00?ooool0o_oo000Eool00`00ooooo`08
ool2003nool001Goo`8000_oo`03003ooooo0?_oo`005Ooo00<00?ooool02ooo0P00nooo000Eool0
0`00ooooo`0=ool00`00ooooo`3hool001Goo`03003ooooo00koo`800?Soo`005Ooo00<00?ooool0
4?oo00<00?ooool0mOoo000Eool00`00ooooo`0Aool2003eool001Goo`03003ooooo01?oo`03003o
oooo0?;oo`005Ooo00<00?ooool05?oo0P00l_oo000Eool2000Gool2003`ool001Goo`03003ooooo
01Soo`800>koo`005Ooo00<00?ooool06_oo0P00k?oo000Eool00`00ooooo`0Lool2003Zool001Go
o`03003ooooo01koo`03003ooooo0>Ooo`005Ooo00<00?ooool07ooo0P00iooo000Eool00`00oooo
o`0Qool00`00ooooo`3Tool001Goo`03003ooooo02;oo`03003ooooo0>?oo`005Ooo0P009?oo0P00
hooo000Eool00`00ooooo`0Uool00`00ooooo`3Pool001Goo`03003ooooo02Koo`800>3oo`005Ooo
00<00?ooool0:?oo00<00?ooool0gOoo000Eool00`00ooooo`0Yool2003Mool001Goo`03003ooooo
02_oo`800=_oo`005Ooo00<00?ooool0;Ooo0P00fOoo000Eool00`00ooooo`0_ool2003Gool001Go
o`8003;oo`800=Goo`005Ooo00<00?ooool0<ooo00<00?ooool0d_oo000Eool00`00ooooo`0dool2
003Bool001Goo`03003ooooo03Koo`03003ooooo0<ooo`005Ooo00<00?ooool0=ooo0P00cooo000E
ool00`00ooooo`0iool2003=ool000;oo`8000Coo`03003ooooo00;oo`<000Goo`03003ooooo03_o
o`03003ooooo0<[oo`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`03003ooooo03coo`800<[o
o`0000Goo`00ooooo`0000_oo`03003ooooo00;oo`<003koo`03003ooooo0<Ooo`0000Goo`00oooo
o`0000Soo`<000Goo`03003ooooo03ooo`800<Ooo`0000Goo`00ooooo`0000Soo`03003ooooo00Go
o`03003ooooo047oo`03003ooooo0<Coo`0000Goo`00ooooo`0000Soo`03003ooooo00Goo`03003o
oooo04;oo`800<Coo`000_oo0P002Ooo10001?oo00<00?ooool0A?oo0P00`_oo000Eool00`00oooo
o`16ool00`00ooooo`2oool001Goo`03003ooooo04Ooo`800;ooo`005Ooo00<00?ooool0BOoo00<0
0?ooool0_?oo000Eool2001;ool2002lool001Goo`03003ooooo04coo`03003ooooo0;Woo`005Ooo
00<00?ooool0COoo0P00^Ooo000Eool00`00ooooo`1?ool2002gool001Goo`03003ooooo057oo`03
003ooooo0;Coo`005Ooo00<00?ooool0D_oo0P00]?oo000Eool00`00ooooo`1Dool00`00ooooo`2a
ool001Goo`03003ooooo05Goo`800;7oo`005Ooo0P00F?oo0P00[ooo000Eool00`00ooooo`1Iool0
0`00ooooo`2/ool001Goo`03003ooooo05[oo`800:coo`005Ooo00<00?ooool0G?oo0P00Z_oo000E
ool00`00ooooo`1Nool00`00ooooo`2Wool001Goo`03003ooooo05ooo`800:Ooo`005Ooo00<00?oo
ool0HOoo00<00?ooool0Y?oo000Eool00`00ooooo`1Rool2002Tool001Goo`8006Goo`800:;oo`00
5Ooo00<00?ooool0I_oo00<00?ooool0Wooo000Eool00`00ooooo`1Wool2002Oool001Goo`03003o
oooo06Woo`03003ooooo09coo`005Ooo00<00?ooool0J_oo0P00W?oo000Eool00`00ooooo`1/ool0
0`00ooooo`2Iool001Goo`03003ooooo06goo`8009Woo`005Ooo00<00?ooool0Kooo0P00Uooo000E
ool2001bool00`00ooooo`2Dool001Goo`03003ooooo07;oo`8009Coo`005Ooo00<00?ooool0M?oo
00<00?ooool0TOoo000Eool00`00ooooo`1eool2002Aool001Goo`03003ooooo07Ooo`8008ooo`00
5Ooo00<00?ooool0NOoo00<00?ooool0S?oo000=ool40004ool00`00ooooo`1jool2002<ool000oo
o`03003ooooo00?oo`03003ooooo07coo`8008[oo`003ooo00<00?ooool00ooo0`00O_oo00<00?oo
ool0Qooo000?ool00`00ooooo`03ool00`00ooooo`1oool20027ool000ooo`03003ooooo00?oo`03
003ooooo087oo`03003ooooo08Coo`003Ooo0`001Ooo00<00?ooool0P_oo0P00Q?oo000?ool00`00
ooooo`03ool00`00ooooo`24ool20022ool001Goo`03003ooooo08Koo`03003ooooo07ooo`005Ooo
00<00?ooool0Qooo0P00Oooo000Eool00`00ooooo`29ool00`00ooooo`1lool001Goo`8008_oo`80
07coo`005Ooo00<00?ooool0S?oo0P00N_oo000Eool00`00ooooo`2>ool2001hool001Goo`03003o
oooo093oo`8007Koo`005Ooo00<00?ooool0T_oo0P00M?oo000Eool00`00ooooo`2Dool00`00oooo
o`1aool001Goo`03003ooooo09Goo`80077oo`005Ooo00<00?ooool0Uooo00<00?ooool0K_oo000E
ool2002Iool2001^ool001Goo`03003ooooo09[oo`8006coo`005Ooo00<00?ooool0W?oo00<00?oo
ool0JOoo000Eool00`00ooooo`2Mool2001Yool001Goo`03003ooooo09ooo`03003ooooo06Koo`00
5Ooo00<00?ooool0X?oo0P00I_oo000Eool00`00ooooo`2Rool00`00ooooo`1Sool001Goo`03003o
oooo0:?oo`8006?oo`005Ooo0P00Y_oo0P00HOoo000Eool00`00ooooo`2Wool00`00ooooo`1Nool0
01Goo`03003ooooo0:Soo`8005koo`005Ooo00<00?ooool0Z_oo00<00?ooool0Fooo000Eool00`00
ooooo`2[ool2001Kool001Goo`03003ooooo0:goo`8005Woo`005Ooo00<00?ooool0[ooo00<00?oo
ool0E_oo000Eool00`00ooooo`2`ool2001Fool001Goo`800;?oo`8005Coo`005Ooo00<00?ooool0
]?oo00<00?ooool0DOoo000Eool00`00ooooo`2eool2001Aool001Goo`03003ooooo0;Ooo`03003o
oooo04koo`005Ooo00<00?ooool0^?oo0P00C_oo000Eool00`00ooooo`2jool2001<ool00003ool0
0000008000?oo`03003ooooo00;oo`<000Goo`03003ooooo0;coo`03003ooooo04Woo`000ooo00<0
0?ooool02_oo00<00?ooool00_oo00<00?ooool0_Ooo0P00BOoo0003ool00`00ooooo`0:ool00`00
ooooo`02ool3002oool00`00ooooo`16ool000?oo`03003ooooo00Ooo`<000Goo`03003ooooo0<3o
o`8004Koo`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0`_oo00<00?ooool0@ooo
00001?oo000000002Ooo00<00?ooool01Ooo00<00?ooool0`ooo0P00@ooo0003ool00`00ooooo`07
ool40004ool00`00ooooo`35ool20011ool001Goo`03003ooooo0<Ooo`03003ooooo03koo`005Ooo
00<00?ooool0b?oo0P00?_oo000Eool00`00ooooo`3:ool00`00ooooo`0kool001Goo`800<coo`80
03_oo`005Ooo00<00?ooool0cOoo00<00?ooool0>?oo000Eool00`00ooooo`3>ool2000hool001Go
o`03003ooooo0=3oo`8003Koo`005Ooo00<00?ooool0d_oo00<00?ooool0<ooo000Eool00`00oooo
o`3Cool2000cool001Goo`03003ooooo0=Goo`03003ooooo033oo`005Ooo00<00?ooool0e_oo0P00
<?oo000Eool2003Iool2000^ool001Goo`03003ooooo0=[oo`8002coo`005Ooo00<00?ooool0g?oo
0P00:_oo000Eool00`00ooooo`3Nool2000Xool001Goo`03003ooooo0>3oo`03003ooooo02Goo`00
5Ooo00<00?ooool0hOoo0P009Ooo000Eool00`00ooooo`3Sool00`00ooooo`0Rool001Goo`03003o
oooo0>Coo`8002;oo`005Ooo0P00iooo0P008?oo000Eool00`00ooooo`3Xool00`00ooooo`0Mool0
01Goo`03003ooooo0>Woo`8001goo`005Ooo00<00?ooool0jooo00<00?ooool06_oo000Eool00`00
ooooo`3/ool2000Jool001Goo`03003ooooo0>koo`03003ooooo01Ooo`005Ooo00<00?ooool0kooo
0P005ooo000Eool00`00ooooo`3aool2000Eool001Goo`800?Coo`03003ooooo01;oo`005Ooo00<0
0?ooool0m?oo0P004_oo000Eool00`00ooooo`3fool00`00ooooo`0?ool001Goo`03003ooooo0?Oo
o`8000ooo`005Ooo00<00?ooool0nOoo0P003Ooo000Eool00`00ooooo`3kool00`00ooooo`0:ool0
00goo`@000Coo`03003ooooo0?coo`8000[oo`003Ooo00<00?ooool01Ooo00<00?ooool0o_oo0P00
2?oo000>ool00`00ooooo`04ool3003oool1ool00`00ooooo`05ool000ooo`03003ooooo00?oo`03
003ooooo0?ooo`Woo`004?oo00<00?ooool00_oo00<00?ooool0oooo2Ooo000=ool01000ooooo`00
1?oo00<00?ooool0oooo2Ooo000>ool20005ool00`00ooooo`3oool9ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.165971, -0.15494, \
0.00772359, 0.012497}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %%]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -3.08976 3.83866 [
[.2619 .35254 -9 -9 ]
[.2619 .35254 9 0 ]
[.5 .35254 -3 -9 ]
[.5 .35254 3 0 ]
[.7381 .35254 -9 -9 ]
[.7381 .35254 9 0 ]
[.97619 .35254 -3 -9 ]
[.97619 .35254 3 0 ]
[.01131 .07714 -30 -4.5 ]
[.01131 .07714 0 4.5 ]
[.01131 .1731 -24 -4.5 ]
[.01131 .1731 0 4.5 ]
[.01131 .26907 -30 -4.5 ]
[.01131 .26907 0 4.5 ]
[.01131 .461 -30 -4.5 ]
[.01131 .461 0 4.5 ]
[.01131 .55697 -24 -4.5 ]
[.01131 .55697 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .36504 m
.2619 .37129 L
s
[(0.5)] .2619 .35254 0 1 Mshowa
.5 .36504 m
.5 .37129 L
s
[(1)] .5 .35254 0 1 Mshowa
.7381 .36504 m
.7381 .37129 L
s
[(1.5)] .7381 .35254 0 1 Mshowa
.97619 .36504 m
.97619 .37129 L
s
[(2)] .97619 .35254 0 1 Mshowa
.125 Mabswid
.07143 .36504 m
.07143 .36879 L
s
.11905 .36504 m
.11905 .36879 L
s
.16667 .36504 m
.16667 .36879 L
s
.21429 .36504 m
.21429 .36879 L
s
.30952 .36504 m
.30952 .36879 L
s
.35714 .36504 m
.35714 .36879 L
s
.40476 .36504 m
.40476 .36879 L
s
.45238 .36504 m
.45238 .36879 L
s
.54762 .36504 m
.54762 .36879 L
s
.59524 .36504 m
.59524 .36879 L
s
.64286 .36504 m
.64286 .36879 L
s
.69048 .36504 m
.69048 .36879 L
s
.78571 .36504 m
.78571 .36879 L
s
.83333 .36504 m
.83333 .36879 L
s
.88095 .36504 m
.88095 .36879 L
s
.92857 .36504 m
.92857 .36879 L
s
.25 Mabswid
0 .36504 m
1 .36504 L
s
.02381 .07714 m
.03006 .07714 L
s
[(0.825)] .01131 .07714 1 0 Mshowa
.02381 .1731 m
.03006 .1731 L
s
[(0.85)] .01131 .1731 1 0 Mshowa
.02381 .26907 m
.03006 .26907 L
s
[(0.875)] .01131 .26907 1 0 Mshowa
.02381 .461 m
.03006 .461 L
s
[(0.925)] .01131 .461 1 0 Mshowa
.02381 .55697 m
.03006 .55697 L
s
[(0.95)] .01131 .55697 1 0 Mshowa
.125 Mabswid
.02381 .09633 m
.02756 .09633 L
s
.02381 .11552 m
.02756 .11552 L
s
.02381 .13472 m
.02756 .13472 L
s
.02381 .15391 m
.02756 .15391 L
s
.02381 .1923 m
.02756 .1923 L
s
.02381 .21149 m
.02756 .21149 L
s
.02381 .23068 m
.02756 .23068 L
s
.02381 .24988 m
.02756 .24988 L
s
.02381 .28826 m
.02756 .28826 L
s
.02381 .30746 m
.02756 .30746 L
s
.02381 .32665 m
.02756 .32665 L
s
.02381 .34584 m
.02756 .34584 L
s
.02381 .38423 m
.02756 .38423 L
s
.02381 .40342 m
.02756 .40342 L
s
.02381 .42262 m
.02756 .42262 L
s
.02381 .44181 m
.02756 .44181 L
s
.02381 .4802 m
.02756 .4802 L
s
.02381 .49939 m
.02756 .49939 L
s
.02381 .51858 m
.02756 .51858 L
s
.02381 .53778 m
.02756 .53778 L
s
.02381 .05794 m
.02756 .05794 L
s
.02381 .03875 m
.02756 .03875 L
s
.02381 .01956 m
.02756 .01956 L
s
.02381 .00036 m
.02756 .00036 L
s
.02381 .57616 m
.02756 .57616 L
s
.02381 .59536 m
.02756 .59536 L
s
.02381 .61455 m
.02756 .61455 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.4071 0 m
.42237 .12308 L
.46172 .44029 L
s
.46172 .44029 m
.48377 .61803 L
s
.07143 .29097 m
.0893 .29122 L
.10813 .29157 L
.12885 .29206 L
.14816 .29262 L
.16563 .29323 L
.17066 .29342 L
.17541 .29361 L
.17748 .29369 L
.17862 .29374 L
.17968 .29378 L
.18083 .11572 L
.18144 .11691 L
.18209 .11823 L
.18431 .12314 L
.18496 .12469 L
.18558 .1262 L
.18672 .29408 L
.18796 .29414 L
.18929 .2942 L
.19046 .29425 L
.1917 .2943 L
.19275 .29435 L
s
.19275 .29435 m
.19359 0 L
s
.19411 0 m
.19462 .29444 L
.19528 .29447 L
.19653 .29453 L
.19933 .29466 L
.20439 .29491 L
.22348 .29593 L
.23265 .29648 L
.24239 .29709 L
.24639 .29735 L
.2485 .29748 L
.24964 .29757 L
.25071 .29765 L
.25189 .21453 L
.25251 .21819 L
.25318 .22214 L
.25427 .2252 L
.25544 .23544 L
.25664 .20514 L
.25777 .428 L
.25887 .30514 L
.25989 .2965 L
.26108 .2984 L
.26221 .29846 L
.26322 .29856 L
.26433 .29876 L
.26553 .2986 L
.2662 .29985 L
.2668 .30273 L
.2691 .3167 L
.27772 .36833 L
.28008 .38259 L
.2807 .38638 L
.28137 .39041 L
.28257 .39326 L
.28325 .40183 L
.28388 .59612 L
.28461 .50284 L
.28531 .41429 L
s
.28531 .41429 m
.28605 .61803 L
s
.28701 .61803 m
.28719 .57147 L
.28782 .30057 L
.28895 .08966 L
.29001 .30078 L
.29118 .30087 L
.29243 .30099 L
.29732 .30145 L
.33646 .30584 L
.35599 .30864 L
.37415 .3118 L
.39169 .31559 L
.4104 .32099 L
.42016 .32479 L
.42529 .32726 L
.43072 .33042 L
.43547 .33388 L
.43799 .33613 L
.44069 .339 L
.44317 .34225 L
.44544 .34602 L
.44771 .35103 L
.44884 .33649 L
.44987 .35771 L
.45095 .34482 L
.45212 .34474 L
.45314 .34432 L
.45422 .34355 L
.45903 .33911 L
.46368 .33574 L
.46876 .33308 L
.47314 .33134 L
.47782 .32983 L
.48626 .32772 L
.49597 .32591 L
.50651 .3244 L
.52565 .3224 L
.54451 .32099 L
.56449 .31985 L
.60045 .31839 L
.6189 .31783 L
.63874 .31732 L
.67793 .31652 L
.71567 .31594 L
.75197 .3155 L
.7906 .31512 L
.82779 .31483 L
.86732 .31456 L
.9054 .31435 L
.94204 .31418 L
Mistroke
.97619 .31404 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`8002[o
o`03003ooooo03Coo`03003ooooo09Woo`008Ooo00<00?ooool0:Ooo00<00?ooool0=?oo00<00?oo
ool0VOoo000Qool00`00ooooo`0Yool00`00ooooo`0dool00`00ooooo`2Iool0027oo`03003ooooo
02Woo`03003ooooo03Coo`03003ooooo09Woo`008Ooo00<00?ooool0:Ooo00<00?ooool0=?oo00<0
0?ooool0VOoo000Qool2000Zool00`00ooooo`0eool00`00ooooo`2Hool0027oo`03003ooooo02Wo
o`03003ooooo03Goo`03003ooooo09Soo`008Ooo00<00?ooool0:Ooo00<00?ooool0=Ooo00<00?oo
ool0V?oo000Qool00`00ooooo`0Yool00`00ooooo`0eool00`00ooooo`2Hool0027oo`03003ooooo
02Woo`03003ooooo03Goo`03003ooooo09Soo`008Ooo0P00:_oo00<00?ooool0=Ooo00<00?ooool0
V?oo000Qool00`00ooooo`0Yool00`00ooooo`0eool00`00ooooo`2Hool0027oo`03003ooooo02Wo
o`03003ooooo03Goo`03003ooooo09Soo`008Ooo00<00?ooool0:Ooo00<00?ooool0=_oo00<00?oo
ool0Uooo000Qool00`00ooooo`0Yool00`00ooooo`0fool00`00ooooo`2Gool0027oo`8002[oo`03
003ooooo03Koo`03003ooooo09Ooo`008Ooo00<00?ooool0:Ooo00<00?ooool0=_oo00<00?ooool0
Uooo000Qool00`00ooooo`0Yool00`00ooooo`0fool00`00ooooo`2Gool000;oo`8000Coo`03003o
oooo00?oo`8000?oo`@000;oo`<000Goo`03003ooooo02Woo`03003ooooo03Koo`03003ooooo09Oo
o`0000Goo`00ooooo`0000Soo`04003ooooo0002ool00`00ooooo`06ool00`00ooooo`02ool00`00
ooooo`0Yool00`00ooooo`0fool00`00ooooo`2Gool00005ool00?ooool00008ool01000ooooo`00
0ooo00<00?ooool01Ooo00<00?ooool00_oo0P00:_oo00<00?ooool0=_oo00<00?ooool0Uooo0000
1Ooo003ooooo00002Ooo0P001Ooo00@00?ooooooo`<000Goo`03003ooooo02Woo`03003ooooo03Oo
o`03003ooooo09Koo`0000Goo`00ooooo`0000Soo`04003ooooo0005ool01000ooooo`001ooo00<0
0?ooool0:Ooo00<00?ooool0=ooo00<00?ooool0U_oo00001Ooo003ooooo00002?oo00@00?ooool0
00;oo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`0Yool00`00ooooo`0Fool00`00oooo
o`0Nool00`00ooooo`2Fool000;oo`8000[oo`8000Coo`8000?oo`@000Coo`03003ooooo02Woo`03
003ooooo01Koo`03003ooooo01koo`03003ooooo09Koo`008Ooo0P00:_oo00<00?ooool05_oo00<0
0?ooool07_oo00<00?ooool0U_oo000Qool00`00ooooo`0Yool00`00ooooo`0Fool00`00ooooo`0N
ool00`00ooooo`2Fool0027oo`03003ooooo02Woo`03003ooooo01Koo`03003ooooo01koo`03003o
oooo09Koo`008Ooo00<00?ooool0:Ooo00<00?ooool05_oo00<00?ooool07_oo00<00?ooool0U_oo
000Qool00`00ooooo`0Yool00`00ooooo`0Fool00`00ooooo`0Oool00`00ooooo`2Eool0027oo`80
02Ooo`04003ooooo000Hool00`00ooooo`0Oool00`00ooooo`2Eool0027oo`03003ooooo02Koo`04
003ooooo000Hool00`00ooooo`0Oool00`00ooooo`2Eool0027oo`03003ooooo02Koo`800003ool0
0?oo01Ooo`03003ooooo01ooo`03003ooooo09Goo`008Ooo00<00?ooool09_oo0P0000?oo`00ool0
5ooo00<00?ooool07ooo00<00?ooool0UOoo000Qool00`00ooooo`0Vool200000ooo003oo`0Gool0
0`00ooooo`0Oool00`00ooooo`2Eool0027oo`8002Ooo`800003ool00?oo01Ooo`03003ooooo01oo
o`03003ooooo09Goo`008Ooo00<00?ooool09_oo0P0000?oo`00ool05ooo00<00?ooool08?oo00<0
0?ooool0U?oo000Qool00`00ooooo`0Vool200000ooo003oo`0Gool00`00ooooo`0Pool00`00oooo
o`2Dool0027oo`03003ooooo02Koo`800003ool00?oo01Ooo`03003ooooo023oo`03003ooooo09Co
o`008Ooo00<00?ooool09_oo0P0000?oo`00ool05ooo00<00?ooool08?oo00<00?ooool0U?oo000Q
ool2000Wool200000ooo003oo`0Gool00`00ooooo`0Pool00`00ooooo`2Dool0027oo`03003ooooo
02Koo`800003ool00?oo01Ooo`03003ooooo023oo`03003ooooo09Coo`008Ooo00<00?ooool09_oo
0P0000?oo`00ool05ooo00<00?ooool08?oo00<00?ooool0U?oo0008ool20004ool00`00ooooo`03
ool20003ool30005ool00`00ooooo`0Vool200000ooo003oo`0Gool00`00ooooo`0Pool00`00oooo
o`2Dool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool09_oo
0P0000?oo`00ool05ooo00<00?ooool08Ooo00<00?ooool0Tooo0007ool01000ooooo`002?oo00@0
0?ooool000Goo`03003ooooo00;oo`8002Ooo`800003ool00?oo01Ooo`03003ooooo027oo`03003o
oooo09?oo`001ooo00@00?ooool000Woo`8000?oo`<000Goo`03003ooooo02Koo`800003ool00?oo
01Ooo`03003ooooo027oo`03003ooooo09?oo`001ooo00@00?ooool000Soo`04003ooooo0002ool0
0`00ooooo`05ool00`00ooooo`0Vool200000ooo003oo`0Gool00`00ooooo`0Qool00`00ooooo`2C
ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool09_oo0P00
00?oo`00ool05ooo00<00?ooool08Ooo00<00?ooool0Tooo0008ool2000:ool20003ool40004ool0
0`00ooooo`0Vool200000ooo003oo`0Gool00`00ooooo`0Qool00`00ooooo`2Cool0027oo`8002Oo
o`800003ool00?oo01Ooo`03003ooooo027oo`03003ooooo09?oo`008Ooo00<00?ooool09_oo0P00
00?oo`00ool05_oo0P009?oo00<00?ooool0T_oo000Qool00`00ooooo`0Vool200000ooo003oo`0F
ool2000Tool00`00ooooo`2Bool0027oo`03003ooooo02Koo`800003ool00?oo00koo`03003ooooo
00Goo`8002Coo`03003ooooo09;oo`008Ooo00<00?ooool09Ooo00D00?oo003oo`0000ooo`03003o
oooo00Goo`8002Coo`03003ooooo09;oo`008Ooo0P009_oo00D00?oo003oo`0000koo`8000Ooo`80
02Coo`03003ooooo09;oo`008Ooo00<00?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Co
o`03003ooooo09;oo`008Ooo00<00?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Coo`03
003ooooo09;oo`008Ooo00<00?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Coo`03003o
oooo09;oo`008Ooo00<00?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Goo`03003ooooo
097oo`008Ooo0P009_oo00D00?oo003oo`0000koo`8000Ooo`8002Goo`03003ooooo097oo`008Ooo
00<00?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Goo`03003ooooo097oo`008Ooo00<0
0?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Goo`03003ooooo097oo`008Ooo00<00?oo
ool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Goo`03003ooooo097oo`008Ooo00<00?ooool0
9Ooo00D00?oo003oo`0000koo`8000Ooo`8002Goo`03003ooooo097oo`008Ooo0P009_oo00D00?oo
003oo`0000koo`8000Ooo`8002Goo`03003ooooo097oo`008Ooo00<00?ooool09Ooo00D00?oo003o
o`0000koo`8000Ooo`8002Koo`03003ooooo093oo`008Ooo00<00?ooool09Ooo00D00?oo003oo`00
00koo`8000Ooo`8002Koo`03003ooooo093oo`000_oo0P001?oo00<00?ooool00ooo0P001?oo00<0
0?ooool00_oo0`001Ooo00<00?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Koo`03003o
oooo093oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool00`00ooooo`05ool00`00ooooo`02
ool00`00ooooo`0Uool01@00ool00?oo00003_oo0P001ooo0P009_oo00<00?ooool0T?oo00001Ooo
003ooooo00002?oo00@00?ooool000Coo`03003ooooo00Coo`03003ooooo00;oo`8002Koo`05003o
o`00ool0000>ool20007ool2000Vool00`00ooooo`2@ool00005ool00?ooool00009ool20005ool0
1000oooooooo0`001Ooo00<00?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Koo`03003o
oooo093oo`0000Goo`00ooooo`0000Soo`04003ooooo0005ool01000ooooo`001ooo00<00?ooool0
9Ooo00D00?oo003oo`0000koo`8000Ooo`8002Koo`03003ooooo093oo`0000Goo`00ooooo`0000So
o`04003ooooo0002ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool09Ooo00D00?oo003o
o`0000koo`8000Ooo`8002Koo`03003ooooo093oo`000_oo0P002_oo0P000ooo10000_oo10001?oo
00<00?ooool09Ooo00D00?oo003oo`0000koo`8000Ooo`8002Ooo`03003ooooo08ooo`008Ooo0P00
2_oo3@003ooo00D00?oo003oo`0000koo`8000Ooo`8002Ooo`03003ooooo08ooo`008Ooo00<00?oo
ool05_oo400000?oo`0000001P002Ooo0P001ooo0P009ooo00<00?ooool0Sooo000Qool00`00oooo
o`0_ool>0004ool2000Wool00`00ooooo`2?ool0027oo`03003ooooo03Woo`800003ool00?oo00?o
o`P0027oo`03003ooooo08ooo`008Ooo00<00?ooool0>Ooo0P0000?oo`00ool00ooo00<00?ooool0
1Ooo20006Ooo00<00?ooool0Sooo000Qool2000jool200000ooo003oo`03ool00`00ooooo`0=ool5
000Dool00`00ooooo`2?ool0027oo`03003ooooo03Woo`8000;oo`05003oooooool0000Dool4000A
ool00`00ooooo`1UoolR0007ool0027oo`03003ooooo03Woo`8000;oo`05003oooooool0000Hool5
000<ool00`00ooooo`0Poom5000Yool0027oo`03003ooooo03[oo`04003ooooo0003ool00`00oooo
o`0Kool40008ool00`00ooooo`0Dool<001^ool0027oo`03003ooooo03Coo`8000Coo`05003ooooo
003oo`03000Qool20006ool00`00ooooo`08ool<000/ool40003ool00`00ooooo`02ool3000eool4
0006ool0027oo`8003Coo`04003ooooo0003ool01000ooooo`000ooo0P008_oo00<00?ooool00ooo
00<00?ooool01Ooo0`000ooo00<00?ooool0=?oo00<00?ooool02_oo00<00?ooool0<_oo00<00?oo
ool01ooo000Qool00`00ooooo`0cool01000ooooo`000ooo00D00?ooooooo`0000;oo`8002?oo`<0
00;oo`03003ooooo00;oo`<000Koo`03003ooooo03Coo`03003ooooo00[oo`03003ooooo03?oo`03
003ooooo00Koo`008Ooo00<00?ooool0<ooo00@00?ooool000?oo`04003oooooool4000Vool01000
ooooo`000ooo00<00?ooool01ooo00<00?ooool0=?oo00<00?ooool01ooo0`00=ooo00<00?ooool0
1Ooo000Qool00`00ooooo`0cool01000ooooo`000ooo00@00?ooooooo`800003ool00?oo02Koo`06
003oo`00003oo`002_oo00<00?ooool0=?oo00<00?ooool01ooo00<00?ooool0>?oo00<00?ooool0
1?oo000Qool00`00ooooo`0cool01000ooooo`000ooo00@00?ooooooo`800003ool00?oo02Ooo`@0
00Woo`<003Coo`<000Woo`03003ooooo03Goo`04003ooooo0006ool0027oo`8003Goo`8000Coo`04
003oooooool5000Wool3000<ool00`00ooooo`0dool00`00ooooo`07ool4000eool20007ool0027o
o`03003ooooo03[oo`05003oooooool00002ool00`00ooooo`0Wool2002?ool0027oo`03003ooooo
03[oo`03003ooooo00;oo`03003oo`0002[oo`03003ooooo08goo`008Ooo00<00?ooool0>_oo00<0
0?ooool00_oo00<00?oo0000:_oo00<00?ooool0SOoo000Qool00`00ooooo`0jool00`00ooooo`02
ool00`00ool0000Zool00`00ooooo`2=ool001_oool000H000008Ooo00<00?ooool02Ooo00<00?oo
ool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02Ooo0P000ooo00<00?oo00001Ooo
00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo0P002ooo00<00?ooool02Ooo00<00?oo
ool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?oo
ool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool01Ooo000Qool0
0`00ooooo`0jool00`00ooooo`02ool00`00ool0000[ool00`00ooooo`2<ool0027oo`03003ooooo
03[oo`03003ooooo00;oo`03003oo`0002_oo`03003ooooo08coo`008Ooo00<00?ooool0>_oo00<0
0?ooool00_oo00<00?oo0000:ooo00<00?ooool0S?oo000Qool2000kool00`00ooooo`03ool2000[
ool00`00ooooo`2<ool0027oo`03003ooooo03[oo`03003ooooo00?oo`8002_oo`03003ooooo08co
o`008Ooo00<00?ooool0>_oo00<00?ooool00ooo0P00:ooo00<00?ooool0S?oo000Qool00`00oooo
o`0jool00`00ooooo`03ool2000/ool00`00ooooo`2;ool0027oo`03003ooooo03[oo`03003ooooo
00?oo`8002coo`03003ooooo08_oo`008Ooo0P00>ooo00<00?ooool00ooo0P00;?oo00<00?ooool0
Rooo000Qool00`00ooooo`0jool00`00ooooo`03ool2000/ool00`00ooooo`2;ool0027oo`03003o
oooo03[oo`03003ooooo00?oo`8002coo`03003ooooo08_oo`008Ooo00<00?ooool0>_oo00<00?oo
ool00ooo0P00;?oo00<00?ooool0Rooo000Qool00`00ooooo`0jool00`00ooooo`03ool2000/ool0
0`00ooooo`2;ool0027oo`8003_oo`03003ooooo00?oo`8002coo`03003ooooo08_oo`008Ooo00<0
0?ooool0>_oo00<00?ooool00ooo0P00;Ooo00<00?ooool0R_oo000Qool00`00ooooo`10ool2000]
ool00`00ooooo`2:ool0027oo`03003ooooo043oo`8002goo`03003ooooo08[oo`008Ooo00<00?oo
ool0@?oo0P00;Ooo00<00?ooool0R_oo000Qool20011ool2000]ool00`00ooooo`2:ool0027oo`03
003ooooo043oo`8002goo`03003ooooo08[oo`008Ooo00<00?ooool0@?oo0P00;Ooo00<00?ooool0
R_oo0002ool20004ool00`00ooooo`02ool30003ool40002ool30005ool00`00ooooo`10ool2000]
ool00`00ooooo`2:ool00005ool00?ooool0000;ool01000ooooo`002?oo00<00?ooool00_oo00<0
0?ooool0@?oo0P00;_oo00<00?ooool0ROoo00001Ooo003ooooo00002ooo00D00?ooooooo`0000Oo
o`03003ooooo00;oo`80047oo`8002koo`03003ooooo08Woo`0000Goo`00ooooo`0000Woo`<000Co
o`04003oooooool30005ool00`00ooooo`10ool2000^ool00`00ooooo`29ool00005ool00?ooool0
0008ool01000ooooo`001Ooo00@00?ooool000Ooo`03003ooooo043oo`8002koo`03003ooooo08Wo
o`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool01Ooo00<0
0?ooool0@?oo0P00;_oo00<00?ooool0ROoo0002ool2000:ool20004ool20003ool40004ool00`00
ooooo`10ool2000^ool00`00ooooo`29ool0027oo`80047oo`8002koo`03003ooooo08Woo`008Ooo
00<00?ooool0@?oo0P00;_oo00<00?ooool0ROoo000Qool00`00ooooo`10ool2000^ool00`00oooo
o`29ool0027oo`03003ooooo043oo`8002ooo`03003ooooo08Soo`008Ooo00<00?ooool0@?oo0P00
;ooo00<00?ooool0R?oo000Qool20011ool2000_ool00`00ooooo`28ool0027oo`03003ooooo043o
o`8002ooo`03003ooooo08Soo`008Ooo00<00?ooool0@?oo0P00;ooo00<00?ooool0R?oo000Qool0
0`00ooooo`10ool2000_ool00`00ooooo`28ool0027oo`03003ooooo043oo`8002ooo`03003ooooo
08Soo`008Ooo0P00@Ooo0P00;ooo00<00?ooool0R?oo000Qool00`00ooooo`10ool2000_ool00`00
ooooo`28ool0027oo`03003ooooo043oo`8002ooo`03003ooooo08Soo`008Ooo00<00?ooool0@?oo
0P00<?oo00<00?ooool0Qooo000Qool20011ool2000`ool00`00ooooo`27ool0027oo`03003ooooo
043oo`80033oo`03003ooooo08Ooo`008Ooo00<00?ooool0@?oo0P00<?oo00<00?ooool0Qooo0008
ool20004ool00`00ooooo`02ool30003ool30005ool00`00ooooo`10ool2000`ool00`00ooooo`27
ool000Ooo`04003ooooo000;ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`10ool2000`
ool00`00ooooo`27ool000Ooo`04003ooooo000;ool00`00ooooo`03ool00`00ooooo`02ool20011
ool2000`ool00`00ooooo`27ool000Ooo`04003ooooo0009ool30002ool30005ool00`00ooooo`10
ool2000`ool00`00ooooo`27ool000Ooo`04003ooooo0008ool01000ooooo`000_oo00<00?ooool0
1Ooo00<00?ooool0@?oo0P00<?oo00<00?ooool0Qooo0007ool01000ooooo`002?oo00@00?ooool0
00;oo`03003ooooo00Goo`03003ooooo043oo`80037oo`03003ooooo08Koo`002?oo0P002_oo0P00
0ooo10001?oo00<00?ooool0@?oo0P00<Ooo00<00?ooool0Q_oo000Qool20011ool2000aool00`00
ooooo`26ool0027oo`03003ooooo043oo`80037oo`03003ooooo08Koo`008Ooo00<00?ooool0@?oo
0P00<Ooo00<00?ooool0Q_oo000Qool00`00ooooo`10ool2000aool00`00ooooo`26ool0027oo`03
003ooooo043oo`80037oo`03003ooooo08Koo`008Ooo0P00@Ooo0P00<Ooo00<00?ooool0Q_oo000Q
ool00`00ooooo`10ool2000aool00`00ooooo`26ool0027oo`03003ooooo047oo`03003ooooo033o
o`03003ooooo08Goo`008Ooo00<00?ooool0@Ooo00<00?ooool0<?oo00<00?ooool0QOoo000Qool0
0`00ooooo`11ool00`00ooooo`0`ool00`00ooooo`25ool0027oo`8004;oo`03003ooooo033oo`03
003ooooo08Goo`008Ooo00<00?ooool0@Ooo00<00?ooool0<?oo00<00?ooool0QOoo003ooolQool0
0?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.268626, 0.796744, \
0.00807888, 0.00100219}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, Om /. strangesol[k, 10^4]}, {k, 0.8, 2, 0.005}]\)], "Input"],

Cell[BoxData[
    \({{0.8`, 0.8883968954027865`}, {0.805`, 0.888589186121315`}, {0.81`, 
        0.888790225903835`}, {0.8150000000000001`, 
        0.8890008676449082`}, {0.8200000000000001`, 
        0.889222091880708`}, {0.8250000000000001`, 
        0.8894550307563561`}, {0.8300000000000001`, 
        0.8897009967399324`}, {0.8350000000000001`, 
        0.8899615164738662`}, {0.8400000000000001`, 
        0.8902383694471199`}, {0.8450000000000001`, 
        0.8905336293981782`}, {0.8500000000000001`, 
        0.8908497022503484`}, {0.8550000000000001`, 
        0.8911893451456224`}, {0.8600000000000001`, 
        0.8915556305364769`}, {0.865`, 0.8919517730682371`}, {0.87`, 
        0.892380631941004`}, {0.875`, 0.8928433080478084`}, {0.88`, 
        0.8933367594994585`}, {0.885`, 0.8938457783266721`}, {0.89`, 
        0.8943269761517666`}, {0.895`, 0.8946743530973381`}, {0.9`, 
        0.894692408767086`}, {0.905`, 0.8942822526550215`}, {0.91`, 
        0.8936908419660688`}, {0.915`, 0.8931378509314943`}, {0.92`, 
        0.8926683135201041`}, {0.925`, 0.8922726023463691`}, {0.93`, 
        0.891935110637425`}, {0.935`, 
        0.8916430382616459`}, {0.9400000000000001`, 
        0.8913870332686932`}, {0.9450000000000001`, 
        0.8911596464166013`}, {0.9500000000000001`, 
        0.890955872873529`}, {0.9550000000000001`, 
        0.8907717171206445`}, {0.9600000000000001`, 
        0.8906040870186516`}, {0.9650000000000001`, 
        0.8904505479869609`}, {0.9700000000000001`, 
        0.8903091528445514`}, {0.9750000000000001`, 
        0.8901783220899995`}, {0.9800000000000001`, 
        0.8900567581685824`}, {0.9850000000000001`, 
        0.8899433830006768`}, {0.9900000000000001`, 
        0.8898372917039874`}, {0.995`, 0.8897377177824395`}, {1.`, 
        0.8896440065676939`}, {1.0050000000000001`, 
        0.8895555946917776`}, {1.01`, 
        0.8894719940306901`}, {1.0150000000000001`, 
        0.8893927790066373`}, {1.02`, 
        0.8893175764444385`}, {1.0250000000000001`, 
        0.8892460573925489`}, {1.03`, 
        0.8891779304712236`}, {1.0350000000000001`, 
        0.8891129364194272`}, {1.04`, 
        0.8890508435913054`}, {1.0450000000000002`, 
        0.8889914442112152`}, {1.05`, 
        0.8889345512395369`}, {1.0550000000000002`, 
        0.8888799957339112`}, {1.06`, 0.888827624615121`}, {1.065`, 
        0.8887772987656113`}, {1.07`, 0.888728891403119`}, {1.075`, 
        0.8886822866831479`}, {1.08`, 0.888637378492819`}, {1.085`, 
        0.8885940694055882`}, {1.09`, 0.8885522697718308`}, {1.095`, 
        0.8885118969247066`}, {1.1`, 0.8884728744842549`}, {1.105`, 
        0.8884351317455406`}, {1.11`, 0.8883986031389917`}, {1.115`, 
        0.8883632277529814`}, {1.12`, 0.8883289489102568`}, {1.125`, 
        0.8882957137911162`}, {1.1300000000000001`, 
        0.8882634730972947`}, {1.135`, 
        0.888232180751415`}, {1.1400000000000001`, 
        0.8882017936275947`}, {1.145`, 
        0.8881722713094258`}, {1.1500000000000001`, 
        0.8881435758720727`}, {1.155`, 
        0.888115671685667`}, {1.1600000000000001`, 
        0.8880885252375605`}, {1.165`, 
        0.888062104971318`}, {1.1700000000000002`, 
        0.8880363811405947`}, {1.175`, 
        0.8880113256762923`}, {1.1800000000000002`, 
        0.887986912065568`}, {1.185`, 0.887963115241464`}, {1.19`, 
        0.887939911482057`}, {1.195`, 0.8879172783181665`}, {1.2`, 
        0.8878951944487681`}, {1.205`, 0.8878736396633564`}, {1.21`, 
        0.887852594770585`}, {1.215`, 0.8878320415325914`}, {1.22`, 
        0.8878119626044698`}, {1.225`, 0.8877923414784208`}, {1.23`, 
        0.887773162432152`}, {1.235`, 0.8877544104811498`}, {1.24`, 
        0.8877360713344791`}, {1.245`, 0.887718131353808`}, {1.25`, 
        0.8877005775153753`}, {1.2550000000000001`, 
        0.8876833973746593`}, {1.26`, 
        0.8876665790335144`}, {1.2650000000000001`, 
        0.88765011110958`}, {1.27`, 
        0.8876339827077712`}, {1.2750000000000001`, 
        0.8876181833936889`}, {1.28`, 
        0.887602703168794`}, {1.2850000000000001`, 
        0.88758753244721`}, {1.29`, 
        0.8875726620340278`}, {1.2950000000000002`, 
        0.8875580831049991`}, {1.3`, 
        0.8875437871875099`}, {1.3050000000000002`, 
        0.8875297661427457`}, {1.31`, 0.887516012148955`}, {1.315`, 
        0.8875025176857342`}, {1.32`, 0.8874892755192592`}, {1.325`, 
        0.8874762786883981`}, {1.33`, 0.8874635204916417`}, {1.335`, 
        0.8874509944747953`}, {1.34`, 0.8874386944193825`}, {1.345`, 
        0.887426614331705`}, {1.35`, 0.8874147484325254`}, {1.355`, 
        0.8874030911473216`}, {1.36`, 0.8873916370970822`}, {1.365`, 
        0.887380381089602`}, {1.37`, 0.8873693181112516`}, {1.375`, 
        0.8873584433191851`}, {1.3800000000000001`, 
        0.8873477520339627`}, {1.385`, 
        0.8873372397325594`}, {1.3900000000000001`, 
        0.8873269020417377`}, {1.395`, 
        0.8873167347317615`}, {1.4000000000000001`, 
        0.8873067337104286`}, {1.405`, 
        0.8872968950174075`}, {1.4100000000000001`, 
        0.8872872148188558`}, {1.415`, 
        0.8872776894023047`}, {1.4200000000000002`, 
        0.8872683151717955`}, {1.425`, 
        0.8872590886432529`}, {1.4300000000000002`, 
        0.8872500064400805`}, {1.435`, 0.8872410652889677`}, {1.44`, 
        0.887232262015894`}, {1.445`, 0.887223593542322`}, {1.45`, 
        0.8872150568815662`}, {1.455`, 0.8872066491353309`}, {1.46`, 
        0.8871983674904051`}, {1.465`, 0.8871902092155083`}, {1.47`, 
        0.8871821716582765`}, {1.475`, 0.8871742522423849`}, {1.48`, 
        0.8871664484647961`}, {1.485`, 0.8871587578931293`}, {1.49`, 
        0.8871511781631441`}, {1.495`, 0.8871437069763346`}, {1.5`, 
        0.8871363420976224`}, {1.5050000000000001`, 
        0.887129081353152`}, {1.51`, 
        0.8871219226281768`}, {1.5150000000000001`, 
        0.8871148638650334`}, {1.52`, 
        0.887107903061201`}, {1.5250000000000001`, 
        0.8871010382674402`}, {1.53`, 
        0.8870942675860085`}, {1.5350000000000001`, 
        0.8870875891689453`}, {1.54`, 
        0.8870810012164321`}, {1.5450000000000002`, 
        0.8870745019752105`}, {1.55`, 
        0.8870680897370703`}, {1.5550000000000002`, 
        0.8870617628373906`}, {1.56`, 
        0.8870555196537431`}, {1.5650000000000002`, 
        0.8870493586045477`}, {1.57`, 0.8870432781477782`}, {1.575`, 
        0.8870372767797212`}, {1.58`, 0.8870313530337787`}, {1.585`, 
        0.8870255054793178`}, {1.59`, 0.8870197327205634`}, {1.595`, 
        0.8870140333955319`}, {1.6`, 0.8870084061750034`}, {1.605`, 
        0.8870028497615337`}, {1.61`, 0.8869973628884997`}, {1.615`, 
        0.8869919443191833`}, {1.62`, 0.886986592845883`}, {1.625`, 
        0.8869813072890622`}, {1.6300000000000001`, 
        0.8869760864965249`}, {1.635`, 
        0.8869709293426218`}, {1.6400000000000001`, 
        0.886965834727483`}, {1.645`, 
        0.8869608015762795`}, {1.6500000000000001`, 
        0.8869558288385078`}, {1.655`, 
        0.8869509154873009`}, {1.6600000000000001`, 
        0.8869460605187626`}, {1.665`, 
        0.8869412629513241`}, {1.6700000000000002`, 
        0.8869365218251223`}, {1.675`, 
        0.8869318362013991`}, {1.6800000000000002`, 
        0.8869272051619207`}, {1.685`, 
        0.8869226278084169`}, {1.6900000000000002`, 
        0.886918103262037`}, {1.695`, 0.8869136306628265`}, {1.7`, 
        0.8869092091692177`}, {1.705`, 0.8869048379575393`}, {1.71`, 
        0.8869005162215398`}, {1.715`, 0.8868962431719282`}, {1.72`, 
        0.8868920180359274`}, {1.725`, 0.8868878400568438`}, {1.73`, 
        0.8868837084936475`}, {1.735`, 0.8868796226205693`}, {1.74`, 
        0.8868755817267069`}, {1.745`, 0.8868715851156463`}, {1.75`, 
        0.886867632105092`}, {1.7550000000000001`, 
        0.8868637220265101`}, {1.76`, 
        0.8868598542247831`}, {1.7650000000000001`, 
        0.8868560280578726`}, {1.77`, 
        0.8868522428964948`}, {1.7750000000000001`, 
        0.8868484981238045`}, {1.78`, 
        0.8868447931350879`}, {1.7850000000000001`, 
        0.8868411273374659`}, {1.79`, 
        0.8868375001496049`}, {1.7950000000000002`, 
        0.8868339110014379`}, {1.8`, 
        0.8868303593338909`}, {1.8050000000000002`, 
        0.8868268445986199`}, {1.81`, 
        0.8868233662577544`}, {1.8150000000000002`, 
        0.8868199237836476`}, {1.82`, 0.8868165166586354`}, {1.825`, 
        0.8868131443748011`}, {1.83`, 0.8868098064337466`}, {1.835`, 
        0.8868065023463707`}, {1.84`, 0.8868032316326531`}, {1.845`, 
        0.8867999938214463`}, {1.85`, 0.8867967884502682`}, {1.855`, 
        0.886793615065108`}, {1.86`, 0.8867904732202301`}, {1.865`, 
        0.8867873624779882`}, {1.87`, 0.8867842824086425`}, {1.875`, 
        0.8867812325901827`}, {1.8800000000000001`, 
        0.8867782126081537`}, {1.885`, 
        0.8867752220554895`}, {1.8900000000000001`, 
        0.8867722605323488`}, {1.895`, 
        0.8867693276459551`}, {1.9000000000000001`, 
        0.8867664230104424`}, {1.905`, 
        0.8867635462467048`}, {1.9100000000000001`, 
        0.8867606969822481`}, {1.915`, 
        0.8867578748510481`}, {1.9200000000000002`, 
        0.8867550794934098`}, {1.925`, 
        0.8867523105558324`}, {1.9300000000000002`, 
        0.8867495676908781`}, {1.935`, 
        0.8867468505570401`}, {1.9400000000000002`, 
        0.8867441588186199`}, {1.945`, 
        0.8867414921456037`}, {1.9500000000000002`, 
        0.8867388502135426`}, {1.955`, 0.8867362327034374`}, {1.96`, 
        0.8867336393016241`}, {1.965`, 0.8867310696996636`}, {1.97`, 
        0.8867285235942346`}, {1.975`, 0.8867260006870271`}, {1.98`, 
        0.8867235006846408`}, {1.985`, 0.8867210232984849`}, {1.99`, 
        0.8867185682446802`}, {1.995`, 0.8867161948936652`}, {2.`, 
        0.8867137830909566`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.611111 0.793651 -65.4004 73.7726 [
[.02381 .24465 -9 -9 ]
[.02381 .24465 9 0 ]
[.34127 .24465 -9 -9 ]
[.34127 .24465 9 0 ]
[.5 .24465 -9 -9 ]
[.5 .24465 9 0 ]
[.65873 .24465 -9 -9 ]
[.65873 .24465 9 0 ]
[.81746 .24465 -9 -9 ]
[.81746 .24465 9 0 ]
[.97619 .24465 -3 -9 ]
[.97619 .24465 3 0 ]
[.17004 .1096 -30 -4.5 ]
[.17004 .1096 0 4.5 ]
[.17004 .40469 -30 -4.5 ]
[.17004 .40469 0 4.5 ]
[.17004 .55224 -30 -4.5 ]
[.17004 .55224 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .25715 m
.02381 .2634 L
s
[(0.8)] .02381 .24465 0 1 Mshowa
.34127 .25715 m
.34127 .2634 L
s
[(1.2)] .34127 .24465 0 1 Mshowa
.5 .25715 m
.5 .2634 L
s
[(1.4)] .5 .24465 0 1 Mshowa
.65873 .25715 m
.65873 .2634 L
s
[(1.6)] .65873 .24465 0 1 Mshowa
.81746 .25715 m
.81746 .2634 L
s
[(1.8)] .81746 .24465 0 1 Mshowa
.97619 .25715 m
.97619 .2634 L
s
[(2)] .97619 .24465 0 1 Mshowa
.125 Mabswid
.06349 .25715 m
.06349 .2609 L
s
.10317 .25715 m
.10317 .2609 L
s
.14286 .25715 m
.14286 .2609 L
s
.22222 .25715 m
.22222 .2609 L
s
.2619 .25715 m
.2619 .2609 L
s
.30159 .25715 m
.30159 .2609 L
s
.38095 .25715 m
.38095 .2609 L
s
.42063 .25715 m
.42063 .2609 L
s
.46032 .25715 m
.46032 .2609 L
s
.53968 .25715 m
.53968 .2609 L
s
.57937 .25715 m
.57937 .2609 L
s
.61905 .25715 m
.61905 .2609 L
s
.69841 .25715 m
.69841 .2609 L
s
.7381 .25715 m
.7381 .2609 L
s
.77778 .25715 m
.77778 .2609 L
s
.85714 .25715 m
.85714 .2609 L
s
.89683 .25715 m
.89683 .2609 L
s
.93651 .25715 m
.93651 .2609 L
s
.25 Mabswid
0 .25715 m
1 .25715 L
s
.18254 .1096 m
.18879 .1096 L
s
[(0.888)] .17004 .1096 1 0 Mshowa
.18254 .40469 m
.18879 .40469 L
s
[(0.892)] .17004 .40469 1 0 Mshowa
.18254 .55224 m
.18879 .55224 L
s
[(0.894)] .17004 .55224 1 0 Mshowa
.125 Mabswid
.18254 .14649 m
.18629 .14649 L
s
.18254 .18338 m
.18629 .18338 L
s
.18254 .22026 m
.18629 .22026 L
s
.18254 .29403 m
.18629 .29403 L
s
.18254 .33092 m
.18629 .33092 L
s
.18254 .36781 m
.18629 .36781 L
s
.18254 .44158 m
.18629 .44158 L
s
.18254 .47847 m
.18629 .47847 L
s
.18254 .51535 m
.18629 .51535 L
s
.18254 .07272 m
.18629 .07272 L
s
.18254 .03583 m
.18629 .03583 L
s
.18254 .58912 m
.18629 .58912 L
s
.25 Mabswid
.18254 0 m
.18254 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .13888 m
.02778 .15307 L
.03175 .1679 L
.03571 .18344 L
.03968 .19976 L
.04365 .21694 L
.04762 .23509 L
.05159 .25431 L
.05556 .27473 L
.05952 .29652 L
.06349 .31983 L
.06746 .34489 L
.07143 .37191 L
.0754 .40114 L
.07937 .43277 L
.08333 .46691 L
.0873 .50331 L
.09127 .54086 L
.09524 .57636 L
.09921 .60199 L
.10317 .60332 L
.10714 .57306 L
.11111 .52943 L
.11508 .48864 L
.11905 .454 L
.12302 .4248 L
.12698 .39991 L
.13095 .37836 L
.13492 .35947 L
.13889 .3427 L
.14286 .32767 L
.14683 .31408 L
.15079 .30171 L
.15476 .29039 L
.15873 .27995 L
.1627 .2703 L
.16667 .26133 L
.17063 .25297 L
.1746 .24514 L
.17857 .2378 L
.18254 .23089 L
.18651 .22436 L
.19048 .2182 L
.19444 .21235 L
.19841 .2068 L
.20238 .20153 L
.20635 .1965 L
.21032 .19171 L
.21429 .18713 L
.21825 .18274 L
Mistroke
.22222 .17855 L
.22619 .17452 L
.23016 .17066 L
.23413 .16695 L
.2381 .16337 L
.24206 .15994 L
.24603 .15662 L
.25 .15343 L
.25397 .15035 L
.25794 .14737 L
.2619 .14449 L
.26587 .1417 L
.26984 .13901 L
.27381 .1364 L
.27778 .13387 L
.28175 .13142 L
.28571 .12904 L
.28968 .12673 L
.29365 .12449 L
.29762 .12231 L
.30159 .12019 L
.30556 .11814 L
.30952 .11613 L
.31349 .11418 L
.31746 .11229 L
.32143 .11044 L
.3254 .10864 L
.32937 .10688 L
.33333 .10517 L
.3373 .1035 L
.34127 .10187 L
.34524 .10028 L
.34921 .09873 L
.35317 .09721 L
.35714 .09573 L
.36111 .09428 L
.36508 .09287 L
.36905 .09148 L
.37302 .09013 L
.37698 .08881 L
.38095 .08751 L
.38492 .08625 L
.38889 .08501 L
.39286 .08379 L
.39683 .0826 L
.40079 .08144 L
.40476 .08029 L
.40873 .07917 L
.4127 .07808 L
.41667 .077 L
Mistroke
.42063 .07595 L
.4246 .07491 L
.42857 .0739 L
.43254 .0729 L
.43651 .07193 L
.44048 .07097 L
.44444 .07003 L
.44841 .0691 L
.45238 .06819 L
.45635 .0673 L
.46032 .06643 L
.46429 .06557 L
.46825 .06472 L
.47222 .06389 L
.47619 .06308 L
.48016 .06227 L
.48413 .06148 L
.4881 .06071 L
.49206 .05995 L
.49603 .0592 L
.5 .05846 L
.50397 .05773 L
.50794 .05702 L
.5119 .05632 L
.51587 .05562 L
.51984 .05494 L
.52381 .05427 L
.52778 .05361 L
.53175 .05296 L
.53571 .05233 L
.53968 .0517 L
.54365 .05108 L
.54762 .05046 L
.55159 .04986 L
.55556 .04927 L
.55952 .04869 L
.56349 .04811 L
.56746 .04754 L
.57143 .04698 L
.5754 .04643 L
.57937 .04589 L
.58333 .04535 L
.5873 .04482 L
.59127 .0443 L
.59524 .04379 L
.59921 .04328 L
.60317 .04278 L
.60714 .04229 L
.61111 .04181 L
.61508 .04133 L
Mistroke
.61905 .04085 L
.62302 .04039 L
.62698 .03993 L
.63095 .03947 L
.63492 .03902 L
.63889 .03858 L
.64286 .03814 L
.64683 .03771 L
.65079 .03729 L
.65476 .03687 L
.65873 .03645 L
.6627 .03604 L
.66667 .03564 L
.67063 .03524 L
.6746 .03484 L
.67857 .03445 L
.68254 .03407 L
.68651 .03369 L
.69048 .03331 L
.69444 .03294 L
.69841 .03257 L
.70238 .03221 L
.70635 .03185 L
.71032 .0315 L
.71429 .03115 L
.71825 .0308 L
.72222 .03046 L
.72619 .03012 L
.73016 .02979 L
.73413 .02946 L
.7381 .02913 L
.74206 .02881 L
.74603 .02849 L
.75 .02818 L
.75397 .02786 L
.75794 .02756 L
.7619 .02725 L
.76587 .02695 L
.76984 .02665 L
.77381 .02636 L
.77778 .02606 L
.78175 .02578 L
.78571 .02549 L
.78968 .02521 L
.79365 .02493 L
.79762 .02465 L
.80159 .02438 L
.80556 .02411 L
.80952 .02384 L
.81349 .02358 L
Mistroke
.81746 .02332 L
.82143 .02306 L
.8254 .0228 L
.82937 .02255 L
.83333 .02229 L
.8373 .02205 L
.84127 .0218 L
.84524 .02156 L
.84921 .02131 L
.85317 .02108 L
.85714 .02084 L
.86111 .0206 L
.86508 .02037 L
.86905 .02014 L
.87302 .01992 L
.87698 .01969 L
.88095 .01947 L
.88492 .01925 L
.88889 .01903 L
.89286 .01881 L
.89683 .0186 L
.90079 .01839 L
.90476 .01818 L
.90873 .01797 L
.9127 .01776 L
.91667 .01756 L
.92063 .01736 L
.9246 .01715 L
.92857 .01696 L
.93254 .01676 L
.93651 .01656 L
.94048 .01637 L
.94444 .01618 L
.94841 .01599 L
.95238 .0158 L
.95635 .01562 L
.96032 .01543 L
.96429 .01525 L
.96825 .01507 L
.97222 .01489 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O=_oo00<00?ooool0
iooo000fool00`00ooooo`3Wool003Koo`03003ooooo0>Ooo`00=_oo00<00?ooool0iooo000fool0
0`00ooooo`3Iool70007ool003Koo`03003ooooo0<Kooa<000koo`00=_oo00<00?ooool0]Ooo4@00
8Ooo000fool00`00ooooo`2Vool?000bool003Koo`03003ooooo09[oo``0047oo`00=_oo00<00?oo
ool0S_oo3000COoo000fool20025ool:001Iool003Koo`03003ooooo07_oo`T006?oo`00=_oo00<0
0?ooool0L_oo2@00K?oo000fool00`00ooooo`1[ool7001eool003Koo`03003ooooo06Goo`H007co
o`00=_oo00<00?ooool0G_oo1`00P_oo000fool00`00ooooo`1Hool60029ool003Koo`03003ooooo
05Coo`@008ooo`00=_oo00<00?ooool0C_oo1P00Tooo000fool00`00ooooo`19ool5002Iool003Ko
o`03003ooooo04Koo`<009koo`00=_oo0P00@ooo1000XOoo000fool00`00ooooo`0nool4002Uool0
03Koo`03003ooooo03_oo`<00:Woo`00=_oo00<00?ooool0=ooo1000[?oo000fool00`00ooooo`0d
ool3002`ool003Koo`03003ooooo037oo`<00;?oo`00=_oo00<00?ooool0;_oo0`00]_oo000fool0
0`00ooooo`0/ool2002iool001Koo`8000Coo`03003ooooo00?oo`8000Coo`8000Coo`8000Koo`03
003ooooo02[oo`800;_oo`005Ooo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000Goo`03003ooooo02Koo`@00;goo`005Ooo00@00?ooool000Soo`04003ooooo0002
ool01000ooooo`000_oo00@00?ooool000Goo`8002Goo`800<7oo`005Ooo00@00?ooool000Woo`80
00Coo`8000Coo`8000Koo`03003ooooo02?oo`03003ooooo0<7oo`005Ooo00@00?ooool000Soo`04
003ooooo0002ool01000ooooo`000_oo00@00?ooool000Goo`03003ooooo027oo`800<Coo`005Ooo
00@00?ooool000Soo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000Goo`03003ooooo
01koo`<00<Koo`005_oo0P002_oo0P001?oo0P001?oo0P001_oo00<00?ooool07?oo0P00bOoo000f
ool00`00ooooo`0Kool00`00ooooo`39ool003Koo`03003ooooo01Woo`800<coo`00=_oo00<00?oo
ool06?oo00<00?ooool0c?oo000fool00`00ooooo`0Gool00`00ooooo`3=ool000Woo`03003ooooo
02[oo`03003ooooo01Coo`<00=3oo`002Ooo00<00?ooool0:_oo00<00?ooool04ooo00<00?ooool0
dOoo0009ool00`00ooooo`0Zool2000Cool00`00ooooo`3Bool000[oo`03003ooooo02Woo`03003o
oooo017oo`03003ooooo0=?oo`002_oo00<00?ooool0:Ooo00<00?ooool04?oo00<00?ooool0e?oo
000:ool00`00ooooo`0Yool00`00ooooo`0?ool00`00ooooo`3Eool000[oo`03003ooooo02Woo`03
003ooooo00koo`03003ooooo0=Koo`002ooo00<00?ooool0:?oo00<00?ooool03?oo0P00fOoo000;
ool00`00ooooo`0Xool00`00ooooo`0;ool00`00ooooo`3Iool000_oo`03003ooooo02Soo`03003o
oooo00[oo`03003ooooo0=[oo`002ooo00<00?ooool0:?oo00<00?ooool02Ooo00<00?ooool0fooo
000<ool00`00ooooo`0Wool00`00ooooo`08ool00`00ooooo`3Lool000coo`03003ooooo02Ooo`80
00Soo`03003ooooo0=goo`003?oo00<00?ooool09ooo00<00?ooool01_oo00<00?ooool0g_oo000<
ool00`00ooooo`0Wool00`00ooooo`06ool00`00ooooo`3Nool000goo`03003ooooo02Koo`03003o
oooo00Coo`800>7oo`003Ooo00<00?ooool09_oo00<00?ooool00ooo00<00?ooool0hOoo000=ool0
0`00ooooo`0Vool00`00ooooo`02ool00`00ooooo`3Rool000goo`03003ooooo02Koo`03003ooooo
00;oo`03003ooooo0>;oo`003Ooo00<00?ooool09_oo00D00?ooooooo`000>Goo`003_oo00<00?oo
ool09Ooo00D00?ooooooo`000>Goo`003_oo00<00?ooool09Ooo00@00?ooool00>Koo`003_oo00<0
0?ooool09Ooo00<00?oo0000iooo0002ool20004ool00`00ooooo`03ool2000Vool3000Rool40003
ool00`00ooooo`02ool4000Mool40003ool00`00ooooo`04ool00`00ooooo`0Lool40003ool00`00
ooooo`03ool2000Oool40003ool00`00ooooo`03ool2000Tool40006ool00005ool00?ooool00008
ool200000ooo003oo`0Tool2000Uool00`00ooooo`07ool00`00ooooo`0Pool00`00ooooo`09ool0
0`00ooooo`0Nool00`00ooooo`07ool01000ooooo`008?oo00<00?ooool01ooo00@00?ooool002?o
o`03003ooooo00Ooo`0000Goo`00ooooo`0000Soo`04003oo`00000Uool2000Uool00`00ooooo`08
ool00`00ooooo`0Oool00`00ooooo`06ool5000Oool00`00ooooo`07ool01000ooooo`008?oo00<0
0?ooool01ooo00@00?ooool002Coo`03003ooooo00Koo`0000Goo`00ooooo`0000Woo`8002Koo`03
003ooooo02Coo`03003ooooo00Woo`03003ooooo01koo`03003ooooo00Koo`04003ooooo000Pool0
0`00ooooo`07ool3000Qool00`00ooooo`08ool2000Vool00`00ooooo`05ool00005ool00?ooool0
0008ool01000ool000009Ooo00<00?ooool09?oo00<00?ooool02_oo00<00?ooool07Ooo00<00?oo
ool01ooo00<00?oo00008?oo00<00?ooool01ooo00<00?ooool08Ooo00<00?ooool01ooo00@00?oo
ool002Koo`03003ooooo00Coo`0000Goo`00ooooo`0000Soo`04003oo`00000Tool2000Tool30009
ool01000ooooo`007Ooo0`002_oo0P007_oo0`002Ooo00<00?ooool07ooo0`002Ooo00@00?ooool0
02?oo`04003ooooo0006ool000;oo`8000[oo`8002Coo`03003oo`0002Koo`03003ooooo00Soo`80
023oo`03003ooooo00Woo`03003ooooo01koo`03003ooooo00Soo`<0023oo`03003ooooo00Soo`80
02Goo`8000Ooo`004?oo00<00?ooool08?oo00@00?ooool00>Woo`004?oo00<00?ooool08?oo00@0
0?ooool00>Woo`004?oo00<00?ooool07ooo00D00?ooooooo`000>Woo`000_ooo`007`000009ool0
0`00ooooo`05ool01000ooooo`002_oo00<00?ooool02?oo00<00?ooool01?oo00<00?ooool00_oo
00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02Ooo00<00?ooool02?oo00<00?ooool02?oo
00<00?ooool02Ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02Ooo
00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02Ooo00<00?ooool02?oo00<00?ooool02?oo
00<00?ooool02Ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02Ooo
00<00?ooool01Ooo0009ool00`00ooooo`05ool00`00ooooo`0Mool00`00ooooo`02ool00`00oooo
o`0Zool00`00ooooo`0Zool00`00ooooo`0Zool00`00ooooo`0[ool00`00ooooo`0Zool00`00oooo
o`05ool0017oo`03003ooooo01coo`03003ooooo00?oo`03003ooooo0>Ooo`004_oo00<00?ooool0
6ooo00<00?ooool00ooo00<00?ooool0iooo000Bool00`00ooooo`0Kool00`00ooooo`03ool00`00
ooooo`3Wool001;oo`03003ooooo01[oo`03003ooooo00Coo`03003ooooo0>Ooo`004_oo00<00?oo
ool06_oo00<00?ooool01?oo00<00?ooool0iooo000Bool00`00ooooo`0Jool00`00ooooo`04ool0
0`00ooooo`3Wool001?oo`03003ooooo01Soo`03003ooooo00Goo`03003ooooo0>Ooo`004ooo00<0
0?ooool06?oo00<00?ooool01Ooo00<00?ooool0iooo000Cool00`00ooooo`0Hool00`00ooooo`05
ool2003Xool001?oo`03003ooooo01Ooo`03003ooooo00Koo`03003ooooo0>Ooo`004ooo00<00?oo
ool05ooo00<00?ooool01_oo00<00?ooool0iooo000Cool00`00ooooo`0Gool00`00ooooo`06ool0
0`00ooooo`3Wool001Coo`03003ooooo01Goo`03003ooooo00Ooo`03003ooooo0>Ooo`005?oo00<0
0?ooool05Ooo00<00?ooool01ooo00<00?ooool0iooo000Dool00`00ooooo`0Eool00`00ooooo`07
ool00`00ooooo`3Wool001Coo`03003ooooo01Coo`03003ooooo00Soo`03003ooooo0>Ooo`005?oo
00<00?ooool05?oo00<00?ooool02?oo00<00?ooool0iooo000Dool00`00ooooo`0Cool00`00oooo
o`09ool00`00ooooo`3Wool001Coo`03003ooooo01?oo`03003ooooo00Woo`800>Soo`005Ooo00<0
0?ooool04_oo00<00?ooool02Ooo00<00?ooool0iooo000Eool00`00ooooo`0Aool00`00ooooo`0:
ool00`00ooooo`3Wool001Goo`03003ooooo017oo`03003ooooo00[oo`03003ooooo0>Ooo`005Ooo
00<00?ooool04Ooo00<00?ooool02_oo00<00?ooool0iooo000Eool00`00ooooo`0Aool00`00oooo
o`0:ool00`00ooooo`3Wool001Goo`03003ooooo017oo`03003ooooo00[oo`03003ooooo0>Ooo`00
5Ooo00<00?ooool04?oo00<00?ooool02ooo00<00?ooool0iooo000Eool00`00ooooo`0@ool00`00
ooooo`0;ool00`00ooooo`3Wool001Koo`03003ooooo00ooo`03003ooooo00_oo`03003ooooo0>Oo
o`005_oo00<00?ooool03ooo00<00?ooool02ooo00<00?ooool0iooo000Fool00`00ooooo`0?ool0
0`00ooooo`0;ool2003Xool001Koo`03003ooooo00koo`03003ooooo00coo`03003ooooo0>Ooo`00
5_oo00<00?ooool03_oo00<00?ooool03?oo00<00?ooool0iooo000Fool00`00ooooo`0>ool00`00
ooooo`0<ool00`00ooooo`3Wool001Koo`03003ooooo00koo`03003ooooo00coo`03003ooooo0>Oo
o`005_oo00<00?ooool03_oo00<00?ooool03?oo00<00?ooool0iooo000Gool00`00ooooo`0=ool0
0`00ooooo`0<ool00`00ooooo`3Wool001Ooo`03003ooooo00coo`03003ooooo00goo`03003ooooo
0>Ooo`005_oo0P001?oo00<00?ooool00ooo0P000_oo10000ooo10001Ooo00<00?ooool0iooo000E
ool01000ool000002?oo00H00?ooool00?oo0003ool01000ooooo`002?oo00<00?ooool0iooo000E
ool01000ool000002?oo00H00?ooool00?oo0003ool01@00oooooooo00001ooo0P00j?oo000Eool0
1000ool000002Ooo0P000_oo00<00?oo00000P001?oo00<00?ooool01?oo00<00?ooool0iooo000E
ool01000ool000002?oo00D00?ooool00?oo008000;oo`03003ooooo00?oo`03003ooooo00?oo`03
003ooooo0>Ooo`005Ooo00@00?oo000000Soo`03003ooooo00800005ool00?ooool00002ool01000
ooooo`001Ooo00<00?ooool0iooo000Fool30009ool200001?oo003ooooo0P001?oo0P001_oo00<0
0?ooool0iooo000Hool00`00ooooo`0:ool00`00ooooo`0>ool00`00ooooo`3Wool001Soo`03003o
oooo00[oo`03003ooooo00koo`03003ooooo0>Ooo`006?oo00<00?ooool02_oo00<00?ooool03_oo
00<00?ooool0iooo000Hool00`00ooooo`0:ool00`00ooooo`0>ool00`00ooooo`3Wool001Soo`03
003ooooo00[oo`03003ooooo00koo`03003ooooo0>Ooo`006?oo00<00?ooool02_oo00<00?ooool0
3_oo00<00?ooool0iooo000Hool00`00ooooo`09ool00`00ooooo`0?ool2003Xool001Soo`03003o
oooo00Woo`03003ooooo00ooo`03003ooooo0>Ooo`006?oo00<00?ooool02Ooo00<00?ooool03ooo
00<00?ooool0iooo000Iool00`00ooooo`08ool00`00ooooo`0?ool00`00ooooo`3Wool001Woo`03
003ooooo00Soo`03003ooooo00ooo`03003ooooo0>Ooo`006Ooo00<00?ooool02?oo00<00?ooool0
3ooo00<00?ooool0iooo000Iool00`00ooooo`07ool00`00ooooo`0@ool00`00ooooo`3Wool001Wo
o`03003ooooo00Ooo`03003ooooo013oo`03003ooooo0>Ooo`006Ooo00<00?ooool01ooo00<00?oo
ool04?oo00<00?ooool0iooo000Iool00`00ooooo`07ool00`00ooooo`0@ool00`00ooooo`3Wool0
01[oo`03003ooooo00Koo`03003ooooo013oo`800>Soo`006_oo00<00?ooool01Ooo00<00?ooool0
4Ooo00<00?ooool0iooo000Jool00`00ooooo`05ool00`00ooooo`0Aool00`00ooooo`3Wool001[o
o`03003ooooo00Goo`03003ooooo017oo`03003ooooo0>Ooo`006_oo00<00?ooool01Ooo00<00?oo
ool04Ooo00<00?ooool0iooo000Kool00`00ooooo`04ool00`00ooooo`0Aool00`00ooooo`3Wool0
01_oo`03003ooooo00Coo`03003ooooo017oo`03003ooooo0>Ooo`006ooo00<00?ooool01?oo00<0
0?ooool04Ooo00<00?ooool0iooo000Kool00`00ooooo`04ool00`00ooooo`0Aool00`00ooooo`3W
ool001_oo`03003ooooo00Coo`03003ooooo017oo`03003ooooo0>Ooo`006ooo00<00?ooool00ooo
00<00?ooool04_oo00<00?ooool0iooo000Kool00`00ooooo`03ool00`00ooooo`0Bool2003Xool0
01_oo`03003ooooo00?oo`03003ooooo01;oo`03003ooooo0>Ooo`007?oo00<00?ooool00_oo00<0
0?ooool04_oo00<00?ooool0iooo000Lool00`00ooooo`02ool00`00ooooo`0Bool00`00ooooo`3W
ool001coo`03003ooooo00;oo`03003ooooo01;oo`03003ooooo0>Ooo`007?oo00<00?ooool00_oo
00<00?ooool04_oo00<00?ooool0iooo000Lool00`00ooooo`02ool00`00ooooo`0Bool00`00oooo
o`3Wool001coo`03003ooooo00;oo`03003ooooo01;oo`03003ooooo0>Ooo`005_oo0P001?oo00<0
0?ooool00_oo0`000ooo0`001Ooo00<00?ooool01?oo00<00?ooool0iooo000Eool01000ooooo`00
0ooo00<00?ooool00_oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo
0>Ooo`005Ooo00@00?ooool000?oo`03003ooooo00;oo`04003ooooo0005ool00`00ool000040005
ool2003Xool001Goo`04003ooooo0003ool01P00oooooooo003oo`8000Coo`<00005ool00?ooool0
0006ool00`00ooooo`3Wool001Goo`04003ooooo0003ool01000oooooooo0P000_oo00@00?ooool0
00;oo`06003ooooo003oo`001_oo00<00?ooool0iooo000Eool01000ooooo`001?oo00<00?ooool0
0P000_oo00@00?ooool000;oo`04003oooooool20006ool00`00ooooo`3Wool001Koo`8000Goo`05
003ooooo003oo`020004ool20005ool00`00ooooo`04ool00`00ooooo`3Wool001goo`04003ooooo
000Eool00`00ooooo`3Wool001goo`04003ooooo000Eool00`00ooooo`3Wool001goo`04003ooooo
000Eool00`00ooooo`3Wool001goo`04003ooooo000Eool00`00ooooo`3Wool001goo`04003ooooo
000Eool00`00ooooo`3Wool001goo`04003ooooo000Eool00`00ooooo`3Wool001koo`8001Koo`80
0>Soo`007_oo0P005_oo00<00?ooool0iooo000Nool2000Fool00`00ooooo`3Wool001koo`8001Ko
o`03003ooooo0>Ooo`007ooo00<00?ooool05?oo00<00?ooool0iooo000fool00`00ooooo`3Wool0
03Koo`03003ooooo0>Ooo`00=_oo00<00?ooool0iooo000fool00`00ooooo`3Wool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.760174, 0.886491, \
0.0044245, 4.75991*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %384]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -44.3849 50.3154 [
[.2619 .38329 -9 -9 ]
[.2619 .38329 9 0 ]
[.5 .38329 -3 -9 ]
[.5 .38329 3 0 ]
[.7381 .38329 -9 -9 ]
[.7381 .38329 9 0 ]
[.97619 .38329 -3 -9 ]
[.97619 .38329 3 0 ]
[.01131 .0939 -30 -4.5 ]
[.01131 .0939 0 4.5 ]
[.01131 .19453 -30 -4.5 ]
[.01131 .19453 0 4.5 ]
[.01131 .29516 -30 -4.5 ]
[.01131 .29516 0 4.5 ]
[.01131 .49642 -30 -4.5 ]
[.01131 .49642 0 4.5 ]
[.01131 .59705 -30 -4.5 ]
[.01131 .59705 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .39579 m
.2619 .40204 L
s
[(0.5)] .2619 .38329 0 1 Mshowa
.5 .39579 m
.5 .40204 L
s
[(1)] .5 .38329 0 1 Mshowa
.7381 .39579 m
.7381 .40204 L
s
[(1.5)] .7381 .38329 0 1 Mshowa
.97619 .39579 m
.97619 .40204 L
s
[(2)] .97619 .38329 0 1 Mshowa
.125 Mabswid
.07143 .39579 m
.07143 .39954 L
s
.11905 .39579 m
.11905 .39954 L
s
.16667 .39579 m
.16667 .39954 L
s
.21429 .39579 m
.21429 .39954 L
s
.30952 .39579 m
.30952 .39954 L
s
.35714 .39579 m
.35714 .39954 L
s
.40476 .39579 m
.40476 .39954 L
s
.45238 .39579 m
.45238 .39954 L
s
.54762 .39579 m
.54762 .39954 L
s
.59524 .39579 m
.59524 .39954 L
s
.64286 .39579 m
.64286 .39954 L
s
.69048 .39579 m
.69048 .39954 L
s
.78571 .39579 m
.78571 .39954 L
s
.83333 .39579 m
.83333 .39954 L
s
.88095 .39579 m
.88095 .39954 L
s
.92857 .39579 m
.92857 .39954 L
s
.25 Mabswid
0 .39579 m
1 .39579 L
s
.02381 .0939 m
.03006 .0939 L
s
[(0.884)] .01131 .0939 1 0 Mshowa
.02381 .19453 m
.03006 .19453 L
s
[(0.886)] .01131 .19453 1 0 Mshowa
.02381 .29516 m
.03006 .29516 L
s
[(0.888)] .01131 .29516 1 0 Mshowa
.02381 .49642 m
.03006 .49642 L
s
[(0.892)] .01131 .49642 1 0 Mshowa
.02381 .59705 m
.03006 .59705 L
s
[(0.894)] .01131 .59705 1 0 Mshowa
.125 Mabswid
.02381 .11905 m
.02756 .11905 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16937 m
.02756 .16937 L
s
.02381 .21969 m
.02756 .21969 L
s
.02381 .24484 m
.02756 .24484 L
s
.02381 .27 m
.02756 .27 L
s
.02381 .32032 m
.02756 .32032 L
s
.02381 .34547 m
.02756 .34547 L
s
.02381 .37063 m
.02756 .37063 L
s
.02381 .42095 m
.02756 .42095 L
s
.02381 .4461 m
.02756 .4461 L
s
.02381 .47126 m
.02756 .47126 L
s
.02381 .52158 m
.02756 .52158 L
s
.02381 .54674 m
.02756 .54674 L
s
.02381 .57189 m
.02756 .57189 L
s
.02381 .06874 m
.02756 .06874 L
s
.02381 .04358 m
.02756 .04358 L
s
.02381 .01842 m
.02756 .01842 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.40476 .31513 m
.40714 .3248 L
.40952 .33492 L
.4119 .34552 L
.41429 .35665 L
.41667 .36837 L
.41905 .38074 L
.42143 .39385 L
.42381 .40778 L
.42619 .42264 L
.42857 .43854 L
.43095 .45563 L
.43333 .47406 L
.43571 .49399 L
.4381 .51557 L
.44048 .53885 L
.44286 .56368 L
.44524 .58929 L
.44762 .6135 L
s
.44762 .6135 m
.44824 .61803 L
s
.45398 .61803 m
.45476 .61125 L
.45714 .5815 L
.45952 .55367 L
.4619 .53005 L
.46429 .51014 L
.46667 .49316 L
.46905 .47846 L
.47143 .46558 L
.47381 .45414 L
.47619 .44388 L
.47857 .43462 L
.48095 .42618 L
.48333 .41846 L
.48571 .41134 L
.4881 .40476 L
.49048 .39865 L
.49286 .39294 L
.49524 .3876 L
.49762 .38259 L
.5 .37788 L
.50238 .37343 L
.50476 .36922 L
.50714 .36524 L
.50952 .36145 L
.5119 .35785 L
.51429 .35443 L
.51667 .35116 L
.51905 .34803 L
.52143 .34504 L
.52381 .34218 L
.52619 .33944 L
.52857 .3368 L
.53095 .33427 L
.53333 .33183 L
.53571 .32949 L
.5381 .32723 L
.54048 .32505 L
.54286 .32295 L
.54524 .32091 L
.54762 .31895 L
.55 .31705 L
.55238 .31521 L
.55476 .31343 L
.55714 .31171 L
.55952 .31004 L
.5619 .30842 L
.56429 .30684 L
.56667 .30531 L
.56905 .30383 L
Mistroke
.57143 .30238 L
.57381 .30098 L
.57619 .29961 L
.57857 .29828 L
.58095 .29699 L
.58333 .29573 L
.58571 .2945 L
.5881 .2933 L
.59048 .29214 L
.59286 .291 L
.59524 .28989 L
.59762 .2888 L
.6 .28774 L
.60238 .28671 L
.60476 .2857 L
.60714 .28471 L
.60952 .28375 L
.6119 .2828 L
.61429 .28188 L
.61667 .28098 L
.61905 .28009 L
.62143 .27923 L
.62381 .27838 L
.62619 .27755 L
.62857 .27674 L
.63095 .27595 L
.63333 .27517 L
.63571 .27441 L
.6381 .27366 L
.64048 .27292 L
.64286 .2722 L
.64524 .2715 L
.64762 .27081 L
.65 .27013 L
.65238 .26946 L
.65476 .26881 L
.65714 .26817 L
.65952 .26754 L
.6619 .26692 L
.66429 .26631 L
.66667 .26571 L
.66905 .26512 L
.67143 .26455 L
.67381 .26398 L
.67619 .26343 L
.67857 .26288 L
.68095 .26234 L
.68333 .26181 L
.68571 .26129 L
.6881 .26078 L
Mistroke
.69048 .26028 L
.69286 .25978 L
.69524 .25929 L
.69762 .25882 L
.7 .25834 L
.70238 .25788 L
.70476 .25742 L
.70714 .25697 L
.70952 .25653 L
.7119 .25609 L
.71429 .25566 L
.71667 .25524 L
.71905 .25482 L
.72143 .25441 L
.72381 .25401 L
.72619 .25361 L
.72857 .25322 L
.73095 .25283 L
.73333 .25245 L
.73571 .25207 L
.7381 .2517 L
.74048 .25134 L
.74286 .25098 L
.74524 .25062 L
.74762 .25027 L
.75 .24993 L
.75238 .24959 L
.75476 .24925 L
.75714 .24892 L
.75952 .24859 L
.7619 .24827 L
.76429 .24795 L
.76667 .24764 L
.76905 .24733 L
.77143 .24702 L
.77381 .24672 L
.77619 .24642 L
.77857 .24613 L
.78095 .24584 L
.78333 .24555 L
.78571 .24527 L
.7881 .24499 L
.79048 .24471 L
.79286 .24444 L
.79524 .24417 L
.79762 .2439 L
.8 .24364 L
.80238 .24338 L
.80476 .24312 L
.80714 .24287 L
Mistroke
.80952 .24262 L
.8119 .24237 L
.81429 .24213 L
.81667 .24189 L
.81905 .24165 L
.82143 .24141 L
.82381 .24118 L
.82619 .24095 L
.82857 .24072 L
.83095 .2405 L
.83333 .24028 L
.83571 .24006 L
.8381 .23984 L
.84048 .23962 L
.84286 .23941 L
.84524 .2392 L
.84762 .23899 L
.85 .23879 L
.85238 .23858 L
.85476 .23838 L
.85714 .23818 L
.85952 .23799 L
.8619 .23779 L
.86429 .2376 L
.86667 .23741 L
.86905 .23722 L
.87143 .23703 L
.87381 .23685 L
.87619 .23667 L
.87857 .23649 L
.88095 .23631 L
.88333 .23613 L
.88571 .23596 L
.8881 .23578 L
.89048 .23561 L
.89286 .23544 L
.89524 .23527 L
.89762 .23511 L
.9 .23494 L
.90238 .23478 L
.90476 .23462 L
.90714 .23446 L
.90952 .2343 L
.9119 .23414 L
.91429 .23399 L
.91667 .23384 L
.91905 .23368 L
.92143 .23353 L
.92381 .23338 L
.92619 .23324 L
Mistroke
.92857 .23309 L
.93095 .23295 L
.93333 .2328 L
.93571 .23266 L
.9381 .23252 L
.94048 .23238 L
.94286 .23224 L
.94524 .23211 L
.94762 .23197 L
.95 .23184 L
.95238 .2317 L
.95476 .23157 L
.95714 .23144 L
.95952 .23131 L
.9619 .23118 L
.96429 .23106 L
.96667 .23093 L
.96905 .23081 L
.97143 .23068 L
.97381 .23056 L
.97619 .23044 L
Mfstroke
.44387 0 m
.44972 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`03003o
oooo06[oo`03003ooooo08ooo`008Ooo00<00?ooool0J_oo00<00?ooool0Sooo000Qool00`00oooo
o`1Zool00`00ooooo`2?ool0027oo`03003ooooo06[oo`03003ooooo08ooo`008Ooo0P00Jooo00<0
0?ooool0Sooo000Qool00`00ooooo`1Zool00`00ooooo`2?ool0027oo`03003ooooo06[oo`03003o
oooo08ooo`008Ooo00<00?ooool0J_oo00<00?ooool0Sooo000Qool00`00ooooo`1Zool00`00oooo
o`2?ool0027oo`03003ooooo06[oo`03003ooooo08ooo`008Ooo00<00?ooool0J_oo00<00?ooool0
Sooo000Qool2001[ool00`00ooooo`2?ool0027oo`03003ooooo06[oo`03003ooooo08ooo`008Ooo
00<00?ooool0J_oo00<00?ooool0Sooo000Qool00`00ooooo`1Zool00`00ooooo`2?ool0027oo`03
003ooooo06[oo`03003ooooo08ooo`008Ooo00<00?ooool0J_oo00<00?ooool0Sooo000Qool00`00
ooooo`1Zool00`00ooooo`2?ool0027oo`8006_oo`03003ooooo08ooo`008Ooo00<00?ooool0J_oo
00<00?ooool0Sooo000Qool00`00ooooo`1Zool00`00ooooo`2?ool0027oo`03003ooooo06[oo`03
003ooooo08ooo`000_oo0P001?oo00<00?ooool00ooo0P001?oo0P001Ooo00<00?ooool00ooo00<0
0?ooool0J_oo00<00?ooool0Sooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo
0004ool00`00ooooo`03ool00`00ooooo`1Zool00`00ooooo`2?ool00005ool00?ooool00008ool0
1000ooooo`000_oo00D00?ooool00?oo00D000Coo`8006_oo`03003ooooo08ooo`0000Goo`00oooo
o`0000Woo`8000Coo`8000;oo`04003ooooo0005ool00`00ooooo`1Zool00`00ooooo`2?ool00005
ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000;oo`03003oo`0000Goo`03003ooooo
06[oo`03003ooooo08ooo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`000ooo
0P001Ooo00<00?ooool0J_oo00<00?ooool0Sooo0002ool2000:ool20004ool20005ool00`00oooo
o`03ool00`00ooooo`1Zool00`00ooooo`2?ool0027oo`03003ooooo06[oo`03003ooooo08ooo`00
8Ooo00<00?ooool0J_oo00<00?ooool0Sooo000Qool2001[ool00`00ooooo`2?ool0027oo`03003o
oooo06[oo`03003ooooo08ooo`008Ooo00<00?ooool0J_oo00<00?ooool0Sooo000Qool00`00oooo
o`1Zool00`00ooooo`2?ool0027oo`03003ooooo06[oo`03003ooooo08ooo`008Ooo00<00?ooool0
J_oo00<00?ooool0Sooo000Qool2001[ool00`00ooooo`2?ool0027oo`03003ooooo06[oo`03003o
oooo08ooo`008Ooo00<00?ooool0J_oo00<00?ooool0Sooo000Qool00`00ooooo`1Zool00`00oooo
o`2?ool0027oo`03003ooooo06[oo`03003ooooo08ooo`008Ooo00<00?ooool0J_oo00<00?ooool0
Sooo000Qool00`00ooooo`1Zool00`00ooooo`2?ool0027oo`8006_oo`03003ooooo08ooo`008Ooo
00<00?ooool0J_oo00<00?ooool0Sooo000Qool00`00ooooo`1Zool00`00ooooo`2?ool0027oo`03
003ooooo06[oo`03003ooooo08ooo`000_oo0P001?oo00<00?ooool00ooo0P001?oo0P001?oo0P00
1Ooo00<00?ooool0J_oo00<00?ooool0Sooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04
003ooooo0002ool01000ooooo`001?oo00<00?ooool0J_oo00<00?ooool0Sooo00001Ooo003ooooo
00002?oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`001?oo0P00Jooo00<00?ooool0
Sooo00001Ooo003ooooo00002Ooo0P001?oo0P000ooo0`001Ooo00<00?ooool0J_oo00<00?ooool0
Sooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`05ool0
0`00ooooo`1Zool00`00ooooo`2?ool00005ool00?ooool00008ool01000ooooo`000_oo00@00?oo
ool000;oo`03003ooooo00Goo`03003ooooo06[oo`03003ooooo08ooo`000_oo0P002_oo0P001?oo
0P001?oo0`001?oo00<00?ooool0J_oo00<00?ooool0Sooo000Qool00`00ooooo`1Zool00`00oooo
o`2?ool0027oo`03003ooooo06[oo`03003ooooo08ooo`008Ooo0P00Jooo00<00?ooool0Sooo000Q
ool00`00ooooo`1Zool00`00ooooo`2?ool0027oo`03003ooooo06[oo`03003ooooo08ooo`008Ooo
00<00?ooool0J_oo00<00?ooool0N?oo40001ooo000Qool00`00ooooo`1Zool00`00ooooo`1Zool>
000Gool0027oo`03003ooooo06[oo`03003ooooo05ooo``002Coo`008Ooo0P00Jooo00<00?ooool0
E_oo2@00<?oo000Qool00`00ooooo`1Zool00`00ooooo`1>ool8000iool0027oo`03003ooooo06[o
o`03003ooooo04Soo`H0047oo`008Ooo00<00?ooool0J_oo00<00?ooool0@_oo1P00Aooo000Qool0
0`00ooooo`1Zool00`00ooooo`0lool6001=ool0027oo`03003ooooo06[oo`03003ooooo03Soo`D0
05;oo`008Ooo00<00?ooool0J_oo00<00?ooool0=?oo1000Eooo000Qool2001[ool00`00ooooo`0a
ool4001Jool0027oo`03003ooooo06[oo`03003ooooo02koo`<005koo`008Ooo00<00?ooool0J_oo
00<00?ooool0:ooo0`00HOoo000Qool00`00ooooo`1Zool00`00ooooo`0Xool3001Tool000;oo`80
00Coo`03003ooooo00?oo`8000Coo`8000Coo`8000Goo`03003ooooo06[oo`03003ooooo02Koo`80
06Ooo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000Co
o`03003ooooo06[oo`03003ooooo02Coo`8006Woo`0000Goo`00ooooo`0000Soo`04003ooooo0002
ool01000ooooo`000_oo00@00?ooool000Coo`8006_oo`03003ooooo02;oo`8006_oo`0000Goo`00
ooooo`0000Woo`8000Coo`8000Coo`8000Goo`03003ooooo06[oo`03003ooooo023oo`8006goo`00
00Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000Coo`03003o
oooo06[oo`03003ooooo01koo`8006ooo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000Coo`03003ooooo06[oo`03003ooooo01coo`80077oo`000_oo0P00
2_oo0P001?oo0P001?oo0P001Ooo00<00?ooool0J_oo00<00?ooool06ooo0P00L_oo000Qool00`00
ooooo`1[ool00`00ooooo`0Iool00`00ooooo`1bool0027oo`03003ooooo063oo`03003ooooo00So
o`03003ooooo01Soo`03003ooooo07?oo`008Ooo0P00HOoo00<00?ooool02?oo00<00?ooool05_oo
0P00M_oo000Qool00`00ooooo`1Pool00`00ooooo`08ool00`00ooooo`0Eool2001gool0027oo`03
003ooooo063oo`03003ooooo00Soo`03003ooooo01Coo`03003ooooo07Ooo`008Ooo00<00?ooool0
HOoo00<00?ooool01ooo00<00?ooool04ooo00<00?ooool0N?oo000Qool00`00ooooo`1Qool00`00
ooooo`07ool00`00ooooo`0Bool2001jool0027oo`03003ooooo067oo`03003ooooo00Ooo`03003o
oooo017oo`03003ooooo07[oo`008Ooo00<00?ooool0H_oo00<00?ooool01_oo00<00?ooool04?oo
0P00O?oo000Qool2001Sool00`00ooooo`06ool00`00ooooo`0?ool00`00ooooo`1lool0027oo`03
003ooooo06;oo`03003ooooo00Koo`03003ooooo00ooo`03003ooooo07coo`008Ooo00<00?ooool0
=?oo0P001?oo00<00?ooool00_oo0`008?oo00<00?ooool01_oo00<00?ooool02Ooo100000?oo`00
ool0<?oo10000ooo00<00?ooool00_oo0`00=Ooo10001_oo000Qool00`00ooooo`0cool01000oooo
o`002ooo00<00?ooool07Ooo00<00?ooool01_oo00<00?ooool02ooo00@00?ooool003?oo`03003o
oooo00[oo`03003ooooo03;oo`03003ooooo00Ooo`008Ooo00<00?ooool0<ooo00@00?ooool000_o
o`03003ooooo01koo`03003ooooo00Goo`03003ooooo00_oo`03003oo`0003Coo`03003ooooo00[o
o`03003ooooo03?oo`03003ooooo00Koo`008Ooo00<00?ooool0<ooo00@00?ooool000Soo`<0027o
o`03003ooooo00Goo`03003ooooo00_oo`8003Goo`03003ooooo00Ooo`<003Ooo`03003ooooo00Go
o`008Ooo0P00=?oo00@00?ooool000Soo`03003ooooo027oo`03003ooooo00Goo`03003ooooo00_o
o`8003Goo`03003ooooo00Ooo`03003ooooo03Soo`03003ooooo00Coo`008Ooo00<00?ooool0<ooo
00@00?ooool000Soo`03003ooooo027oo`03003ooooo00Goo`03003ooooo00Woo`<003Coo`<000Wo
o`03003ooooo03Goo`04003ooooo0006ool0027oo`03003ooooo03Coo`8000Woo`@0023oo`03003o
oooo00Goo`03003ooooo00_oo`03003ooooo03Coo`03003ooooo00Ooo`@003Goo`8000Ooo`008Ooo
00<00?ooool0Hooo00<00?ooool01Ooo00<00?ooool02_oo00<00?ooool0POoo000Qool00`00oooo
o`1Sool00`00ooooo`05ool00`00ooooo`09ool00`00ooooo`22ool0027oo`03003ooooo06Coo`03
003ooooo00Coo`03003ooooo00Woo`03003ooooo08;oo`008Ooo00<00?ooool0I?oo00<00?ooool0
1?oo00<00?ooool02Ooo00<00?ooool0P_oo000Koooo00060000027oo`03003ooooo00Woo`03003o
oooo00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003o
oooo00Woo`03003ooooo00[oo`05003oooooool00006ool20009ool01000ooooo`002ooo00<00?oo
ool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?oo
ool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool01Ooo000Qool0
0`00ooooo`1Uool00`00ooooo`03ool00`00ooooo`07ool00`00ooooo`24ool0027oo`03003ooooo
06Goo`03003ooooo00?oo`03003ooooo00Ooo`03003ooooo08Coo`008Ooo00<00?ooool0IOoo00<0
0?ooool00ooo00<00?ooool01ooo00<00?ooool0Q?oo000Qool00`00ooooo`1Uool00`00ooooo`03
ool00`00ooooo`06ool00`00ooooo`25ool0027oo`8006Koo`03003ooooo00?oo`03003ooooo00Ko
o`03003ooooo08Goo`008Ooo00<00?ooool0IOoo00<00?ooool00ooo00<00?ooool01_oo00<00?oo
ool0QOoo000Qool00`00ooooo`1Uool00`00ooooo`03ool00`00ooooo`06ool00`00ooooo`25ool0
027oo`03003ooooo06Goo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo08Koo`008Ooo00<0
0?ooool0I_oo00<00?ooool00_oo00<00?ooool01Ooo00<00?ooool0Q_oo000Qool00`00ooooo`1V
ool00`00ooooo`02ool00`00ooooo`05ool00`00ooooo`26ool0027oo`03003ooooo06Koo`03003o
oooo00;oo`03003ooooo00Coo`03003ooooo08Ooo`008Ooo0P00Iooo00<00?ooool00_oo00<00?oo
ool01?oo00<00?ooool0Qooo000Qool00`00ooooo`1Wool01@00oooooooo00001_oo00<00?ooool0
Qooo000Qool00`00ooooo`1Wool01@00oooooooo00001_oo00<00?ooool0Qooo000Qool00`00oooo
o`1Wool01@00oooooooo00001_oo00<00?ooool0Qooo000Qool00`00ooooo`1Wool01@00oooooooo
00001Ooo00<00?ooool0R?oo000Qool00`00ooooo`1Wool01@00oooooooo00001Ooo00<00?ooool0
R?oo000Qool2001Xool01@00oooooooo00001Ooo00<00?ooool0R?oo000Qool00`00ooooo`1Wool0
1@00oooooooo00001?oo00<00?ooool0ROoo000Qool00`00ooooo`1Wool01@00oooooooo00001?oo
00<00?ooool0ROoo000Qool00`00ooooo`1Wool01@00oooooooo00001?oo00<00?ooool0ROoo000Q
ool00`00ooooo`1Xool01000ooooo`001?oo00<00?ooool0ROoo0002ool20004ool00`00ooooo`03
ool20003ool30003ool40004ool00`00ooooo`1Xool01000ooooo`001?oo00<00?ooool0ROoo0000
1Ooo003ooooo00002?oo00@00?ooool000Goo`04003ooooo0007ool00`00ooooo`1Xool01000oooo
o`001?oo00<00?ooool0ROoo00001Ooo003ooooo00002?oo00@00?ooool000Goo`05003oooooool0
0006ool2001Yool01000ooooo`001?oo00<00?ooool0ROoo00001Ooo003ooooo00002Ooo0P001?oo
0`001?oo00<00?ooool00ooo00<00?ooool0J?oo00@00?ooool000Coo`03003ooooo08Woo`0000Go
o`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool0
J?oo00@00?ooool000?oo`03003ooooo08[oo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool0
1000ooooo`000_oo00@00?ooool000Coo`03003ooooo06Soo`04003ooooo0003ool00`00ooooo`2:
ool000;oo`8000[oo`8000Coo`8000Coo`8000Goo`03003ooooo06Soo`04003ooooo0003ool00`00
ooooo`2:ool0027oo`03003ooooo06Soo`04003ooooo0003ool00`00ooooo`2:ool0027oo`8006Wo
o`04003ooooo0003ool00`00ooooo`2:ool0027oo`03003ooooo06Soo`04003ooooo0003ool00`00
ooooo`2:ool0027oo`03003ooooo06Soo`04003ooooo0003ool00`00ooooo`2:ool0027oo`03003o
oooo06Woo`03003oo`0000?oo`03003ooooo08[oo`008Ooo00<00?ooool0JOoo00<00?oo00000ooo
00<00?ooool0R_oo000Qool00`00ooooo`1Yool00`00ool00003ool00`00ooooo`2:ool0027oo`03
003ooooo06Woo`03003oo`0000;oo`03003ooooo08_oo`008Ooo0P00J_oo00<00?oo00000_oo00<0
0?ooool0Rooo000Qool00`00ooooo`1Yool00`00ool00002ool00`00ooooo`2;ool0027oo`03003o
oooo06[oo`8000;oo`03003ooooo08_oo`008Ooo00<00?ooool0J_oo0P000_oo00<00?ooool0Rooo
000Qool00`00ooooo`1Zool20002ool00`00ooooo`2;ool0027oo`03003ooooo06[oo`8000;oo`03
003ooooo08_oo`008Ooo0P00Jooo0P0000?oo`00ool0SOoo000Qool00`00ooooo`1Zool200000ooo
003oo`2=ool0027oo`03003ooooo06[oo`800003ool00?oo08goo`008Ooo00<00?ooool0J_oo0P00
00?oo`00ool0SOoo000Qool00`00ooooo`1Zool200000ooo003oo`2=ool000;oo`8000Coo`03003o
oooo00?oo`8000?oo`<000Goo`03003ooooo00?oo`03003ooooo06[oo`800003ool00?oo08goo`00
00Goo`00ooooo`0000Soo`04003ooooo0005ool00`00ooooo`02ool00`00ooooo`03ool00`00oooo
o`1Zool200000ooo003oo`2=ool00005ool00?ooool00008ool01000ooooo`001Ooo00<00?oo0000
10001?oo0P00Jooo0P0000?oo`00ool0SOoo00001Ooo003ooooo00002Ooo0P001?oo0`0000Goo`00
ooooo`0000Goo`03003ooooo06[oo`800003ool00?oo08goo`0000Goo`00ooooo`0000Soo`04003o
oooo0002ool01000ooooo`000_oo00<00?oo00001Ooo00<00?ooool0Jooo00<00?oo0000S_oo0000
1Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0003ool20005ool00`00ooooo`1[ool0
0`00ool0002>ool000;oo`8000[oo`8000Coo`8000Goo`03003ooooo00?oo`03003ooooo06_oo`03
003oo`0008koo`008Ooo00<00?ooool0Jooo00<00?oo0000S_oo003ooolQool00?ooob7oo`00oooo
8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.268626, 0.881511, \
0.00807888, 7.64593*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(seems\ it\ just\ goes\ to\ the\ lightcone\ as\ is\ the\ case\ to\ the\ \
left\)], "Input"],

Cell[BoxData[
    \(what\ happens\ out\ at\ large\ \(\(k\)\(?\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[50, 10^4]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.8862939825986308`, 
      mGamma \[Rule] \(-0.5000000001275379`\)}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(?? ?\ is\ there\ an\ asymptotic\ solution?\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[\((0.8862939825986308` + 
            I \((\(-0.5000000001275379`\))\))\)/50, 10^4]\)], "Input"],

Cell[BoxData[
    \(\(\(0.5355170234657055`\)\(\[InvisibleSpace]\)\) - 
      0.8862939828209702`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[\((0.8862939825986308` + 
              I \((\(-0.5000000001275379`\))\))\)/50, 
        10^4]/\((\((0.8862939825986308` + I \((\(-0.5000000001275379`\))\))\)/
            50)\)^2\)], "Input"],

Cell[BoxData[
    \(\(\(2499.99999999566`\)\(\[InvisibleSpace]\)\) + 
      1.0242121040171526`*^-8\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(betakx0korr[\((0.8862939825986308` + 
              I \((\(-0.01000000001275379`\))\))\)/50, 
        10^4]/\((\((0.8862939825986308` + 
                I \((\(-0.01000000001275379`\))\))\)/50)\)^2\)], "Input"],

Cell[BoxData[
    \(\(\(1018.250660305903`\)\(\[InvisibleSpace]\)\) - 
      1621.5429849046027`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((1 - 
        betakx0korr[\((0.8862939825986308` + 
                  I \((\(-0.5000000001275379`\))\))\)/50, 
            10^4]/\((0.8862939825986308` + 
                I \((\(-0.5000000001275379`\))\))\)^2)\)\)], "Input"],

Cell[BoxData[
    \(1.7360557436063573`*^-12 - 
      4.096872147085762`*^-12\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((\((0.8862939825986308` + 
                I \((\(-0.5000000001275379`\))\))\)^2\ 50^2 - 50^2 - 
        alphakx0korr[\((0.8862939825986308` + 
                I \((\(-0.5000000001275379`\))\))\)/50, 10^4])\)\)], "Input"],

Cell[BoxData[
    \(\(-2411.7201201775815`\) - 
      2216.1781040582887`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?betakx0korr\)\)], "Input"],

Cell["Global`betakx0korr", "Print",
  CellTags->"Info3279531231-9953284"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(betakx0korr[eta_, 
                      a_] = \(\(1\/\(4\ \@a\ \((1 + a\ \
eta\^2)\)\^2\)\)\((\@\(1 + a\)\ eta\^2\ \((\[ImaginaryI]\ \((1 + 
                                    a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                                  1 - \[ImaginaryI]\ \@a] - \[ImaginaryI]\ \
\((1 + a)\)\ \((\(-1\) + a\ eta\^2)\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + \@a\ \((\(-2\)\ \
\((1 + a\ eta\^2)\) + \((1 + a)\)\ eta\ \((\(-Log[\((\(-1\) + eta)\)\^2]\) + 
                                        Log[\((1 + eta)\)\^2])\))\))\))\)\) - \
\(\[ImaginaryI]\ \((1 + a)\)\^\(3/2\)\ eta\^3\ \[Pi]\ UnitStep[1 - Re[eta]]\)\
\/\(2\ \((1 + a\ eta\^2)\)\^2\)\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "betakx0korr"],
      Editable->False]], "Print",
  CellTags->"Info3279531231-9953284"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[\((\(betakx0korr[eta, 10^4]/eta^2\)/50^2)\) /. 
          eta \[Rule] X/50], {X, 0.1, 1}, PlotRange \[Rule] All]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0820106 1.0582 0.530657 0.185004 [
[.12963 .51816 -9 -9 ]
[.12963 .51816 9 0 ]
[.34127 .51816 -9 -9 ]
[.34127 .51816 9 0 ]
[.55291 .51816 -9 -9 ]
[.55291 .51816 9 0 ]
[.76455 .51816 -9 -9 ]
[.76455 .51816 9 0 ]
[.96369 .06815 -24 -4.5 ]
[.96369 .06815 0 4.5 ]
[.96369 .16065 -12 -4.5 ]
[.96369 .16065 0 4.5 ]
[.96369 .25315 -24 -4.5 ]
[.96369 .25315 0 4.5 ]
[.96369 .34565 -12 -4.5 ]
[.96369 .34565 0 4.5 ]
[.96369 .43815 -24 -4.5 ]
[.96369 .43815 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.12963 .53066 m
.12963 .53691 L
s
[(0.2)] .12963 .51816 0 1 Mshowa
.34127 .53066 m
.34127 .53691 L
s
[(0.4)] .34127 .51816 0 1 Mshowa
.55291 .53066 m
.55291 .53691 L
s
[(0.6)] .55291 .51816 0 1 Mshowa
.76455 .53066 m
.76455 .53691 L
s
[(0.8)] .76455 .51816 0 1 Mshowa
.125 Mabswid
.18254 .53066 m
.18254 .53441 L
s
.23545 .53066 m
.23545 .53441 L
s
.28836 .53066 m
.28836 .53441 L
s
.39418 .53066 m
.39418 .53441 L
s
.44709 .53066 m
.44709 .53441 L
s
.5 .53066 m
.5 .53441 L
s
.60582 .53066 m
.60582 .53441 L
s
.65873 .53066 m
.65873 .53441 L
s
.71164 .53066 m
.71164 .53441 L
s
.81746 .53066 m
.81746 .53441 L
s
.87037 .53066 m
.87037 .53441 L
s
.92328 .53066 m
.92328 .53441 L
s
.07672 .53066 m
.07672 .53441 L
s
.02381 .53066 m
.02381 .53441 L
s
.25 Mabswid
0 .53066 m
1 .53066 L
s
.97619 .06815 m
.98244 .06815 L
s
[(-2.5)] .96369 .06815 1 0 Mshowa
.97619 .16065 m
.98244 .16065 L
s
[(-2)] .96369 .16065 1 0 Mshowa
.97619 .25315 m
.98244 .25315 L
s
[(-1.5)] .96369 .25315 1 0 Mshowa
.97619 .34565 m
.98244 .34565 L
s
[(-1)] .96369 .34565 1 0 Mshowa
.97619 .43815 m
.98244 .43815 L
s
[(-0.5)] .96369 .43815 1 0 Mshowa
.125 Mabswid
.97619 .08665 m
.97994 .08665 L
s
.97619 .10515 m
.97994 .10515 L
s
.97619 .12365 m
.97994 .12365 L
s
.97619 .14215 m
.97994 .14215 L
s
.97619 .17915 m
.97994 .17915 L
s
.97619 .19765 m
.97994 .19765 L
s
.97619 .21615 m
.97994 .21615 L
s
.97619 .23465 m
.97994 .23465 L
s
.97619 .27165 m
.97994 .27165 L
s
.97619 .29015 m
.97994 .29015 L
s
.97619 .30865 m
.97994 .30865 L
s
.97619 .32715 m
.97994 .32715 L
s
.97619 .36415 m
.97994 .36415 L
s
.97619 .38265 m
.97994 .38265 L
s
.97619 .40115 m
.97994 .40115 L
s
.97619 .41965 m
.97994 .41965 L
s
.97619 .45666 m
.97994 .45666 L
s
.97619 .47516 m
.97994 .47516 L
s
.97619 .49366 m
.97994 .49366 L
s
.97619 .51216 m
.97994 .51216 L
s
.97619 .04965 m
.97994 .04965 L
s
.97619 .03115 m
.97994 .03115 L
s
.97619 .01265 m
.97994 .01265 L
s
.97619 .54916 m
.97994 .54916 L
s
.97619 .56766 m
.97994 .56766 L
s
.97619 .58616 m
.97994 .58616 L
s
.97619 .60466 m
.97994 .60466 L
s
.25 Mabswid
.97619 0 m
.97619 .61803 L
s
.5 Mabswid
.02381 .01472 m
.06244 .06474 L
.10458 .12803 L
.14415 .19102 L
.18221 .25112 L
.22272 .3116 L
.26171 .36451 L
.30316 .41392 L
.34309 .45455 L
.3815 .48738 L
.42237 .51609 L
.46172 .53836 L
.49955 .55547 L
.53984 .56975 L
.57861 .58028 L
.61984 .58864 L
.64043 .5919 L
.65954 .59445 L
.69774 .59837 L
.71826 .59991 L
.73687 .60101 L
.75611 .6019 L
.76604 .60226 L
.77693 .60259 L
.78656 .60282 L
.79687 .60302 L
.80257 .60311 L
.80777 .60317 L
.8127 .60322 L
.81794 .60326 L
.82265 .60329 L
.82533 .6033 L
.82656 .60331 L
.82785 .60331 L
.82895 .60331 L
.83015 .60332 L
.83125 .60332 L
.83226 .60332 L
.83347 .60332 L
.83456 .60332 L
.83583 .60332 L
.83702 .60332 L
.83822 .60332 L
.83954 .60331 L
.84092 .60331 L
.8422 .60331 L
.84698 .60329 L
.8497 .60327 L
.85219 .60326 L
.85778 .60322 L
Mistroke
.86765 .60312 L
.87681 .603 L
.89754 .60264 L
.91811 .60218 L
.93672 .60168 L
.97619 .60041 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo6?oo00<00?oo
ool01_oo003ooolHool00`00ooooo`06ool00?oooaSoo`03003ooooo00Koo`00oooo6?oo0P001ooo
0007ool00`00ooooo`3oool>ool00`00ooooo`06ool000Soo`03003ooooo0?ooo`goo`03003ooooo
00Koo`002Ooo00<00?ooool0oooo3?oo00<00?ooool01_oo0009ool00`00ooooo`3oool<ool00`00
ooooo`06ool000[oo`03003ooooo0?ooo`_oo`8000Ooo`002ooo00<00?ooool0oooo2_oo00<00?oo
ool01_oo000<ool00`00ooooo`3oool9ool00`00ooooo`06ool000coo`03003ooooo0?ooo`Woo`03
003ooooo00Koo`003Ooo00<00?ooool0oooo2?oo00<00?ooool01_oo000>ool00`00ooooo`3oool7
ool00`00ooooo`06ool000ooo`03003ooooo0?ooo`Koo`8000Ooo`004?oo00<00?ooool0oooo1Ooo
00<00?ooool01_oo000@ool00`00ooooo`3oool5ool00`00ooooo`06ool0017oo`03003ooooo0>oo
o`@000?oo`03003ooooo00;oo`<000Goo`03003ooooo00Koo`004_oo00<00?ooool0k_oo00<00?oo
ool03?oo00<00?ooool00_oo00<00?ooool01_oo000Cool00`00ooooo`3^ool00`00ooooo`0;ool0
0`00ooooo`02ool30006ool001?oo`03003ooooo0>Ooo`@000Coo`03003ooooo00Ooo`<000Goo`03
003ooooo00Koo`005?oo00<00?ooool0kooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool0
1_oo000Eool00`00ooooo`3[ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool01_oo000E
ool00`00ooooo`3/ool20009ool40004ool00`00ooooo`06ool001Koo`03003ooooo0?koo`8000Oo
o`005ooo00<00?ooool0oOoo00<00?ooool01_oo000Gool00`00ooooo`3mool00`00ooooo`06ool0
01Soo`03003ooooo0?coo`03003ooooo00Koo`006Ooo00<00?ooool0nooo00<00?ooool01_oo000I
ool00`00ooooo`3kool00`00ooooo`06ool001[oo`03003ooooo0?[oo`8000Ooo`006ooo00<00?oo
ool0nOoo00<00?ooool01_oo000Kool00`00ooooo`3iool00`00ooooo`06ool001coo`03003ooooo
0?Soo`03003ooooo00Koo`007Ooo00<00?ooool0mooo00<00?ooool01_oo000Mool00`00ooooo`3g
ool20007ool001koo`03003ooooo0?Koo`03003ooooo00Koo`007ooo00<00?ooool0mOoo00<00?oo
ool01_oo000Oool00`00ooooo`3eool00`00ooooo`06ool0023oo`03003ooooo0?Coo`03003ooooo
00Koo`008?oo00<00?ooool0m?oo0P001ooo000Qool00`00ooooo`3cool00`00ooooo`06ool002;o
o`03003ooooo0?;oo`03003ooooo00Koo`008_oo00<00?ooool0l_oo00<00?ooool01_oo000Sool0
0`00ooooo`3Yool40004ool00`00ooooo`06ool002?oo`03003ooooo0>Woo`03003ooooo00Goo`03
003ooooo00Koo`009?oo00<00?ooool0jOoo00<00?ooool01?oo0`001_oo000Uool00`00ooooo`3Q
ool40004ool00`00ooooo`03ool00`00ooooo`06ool002Goo`03003ooooo0>[oo`03003ooooo00;o
o`03003ooooo00Koo`009_oo00<00?ooool0i_oo00@00?ooool000Coo`03003ooooo00Koo`009ooo
00<00?ooool0i_oo0P001Ooo00<00?ooool01_oo000Wool00`00ooooo`3]ool20007ool002Soo`03
003ooooo0>coo`03003ooooo00Koo`00:?oo00<00?ooool0k?oo00<00?ooool01_oo000Yool00`00
ooooo`3[ool00`00ooooo`06ool002[oo`03003ooooo0>[oo`03003ooooo00Koo`00:_oo00<00?oo
ool0j_oo0P001ooo000[ool00`00ooooo`3Yool00`00ooooo`06ool002coo`03003ooooo0>Soo`03
003ooooo00Koo`00;?oo00<00?ooool0j?oo00<00?ooool01_oo000]ool00`00ooooo`3Wool00`00
ooooo`06ool002koo`03003ooooo0>Koo`8000Ooo`00;_oo00<00?ooool0i_oo00<00?ooool01_oo
000_ool00`00ooooo`3Uool00`00ooooo`06ool002ooo`03003ooooo0>Goo`03003ooooo00Koo`00
<?oo00<00?ooool0i?oo00<00?ooool01_oo000aool00`00ooooo`3Sool00`00ooooo`06ool0037o
o`03003ooooo0>?oo`8000Ooo`00<_oo00<00?ooool0h_oo00<00?ooool01_oo000cool00`00oooo
o`3Qool00`00ooooo`06ool003?oo`03003ooooo0<goo`@000?oo`03003ooooo00;oo`<000Goo`03
003ooooo00Koo`00=?oo00<00?ooool0c_oo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0
1_oo000eool00`00ooooo`3=ool00`00ooooo`0:ool00`00ooooo`02ool30006ool003Goo`03003o
oooo0<Goo`@000Coo`03003ooooo00Ooo`<000Goo`03003ooooo00Koo`00=_oo00<00?ooool0c?oo
00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool01_oo000gool00`00ooooo`39ool30009ool0
0`00ooooo`05ool00`00ooooo`06ool003Ooo`03003ooooo0<_oo`03003ooooo00Ooo`@000Coo`03
003ooooo00Koo`00>?oo00<00?ooool0g?oo0P001ooo000iool00`00ooooo`3Kool00`00ooooo`06
ool003Woo`03003ooooo0=_oo`03003ooooo00Koo`00>_oo00<00?ooool0f_oo00<00?ooool01_oo
000kool00`00ooooo`3Iool00`00ooooo`06ool003_oo`03003ooooo0=Woo`03003ooooo00Koo`00
??oo00<00?ooool0f?oo0P001ooo000mool00`00ooooo`3Gool00`00ooooo`06ool003goo`03003o
oooo0=Ooo`03003ooooo00Koo`00?_oo00<00?ooool0e_oo00<00?ooool01_oo000oool00`00oooo
o`3Eool00`00ooooo`06ool003ooo`03003ooooo0=Goo`8000Ooo`00@?oo00<00?ooool0e?oo00<0
0?ooool01_oo0011ool00`00ooooo`3Cool00`00ooooo`06ool0047oo`03003ooooo0=?oo`03003o
oooo00Koo`00@_oo00<00?ooool0d_oo00<00?ooool01_oo0013ool00`00ooooo`3Aool20007ool0
04Coo`03003ooooo0=3oo`03003ooooo00Koo`00A?oo00<00?ooool0d?oo00<00?ooool01_oo0015
ool00`00ooooo`3?ool00`00ooooo`06ool004Koo`03003ooooo0<Koo`@000Coo`03003ooooo00Ko
o`00Aooo00<00?ooool0aooo00<00?ooool00ooo00<00?ooool01_oo0017ool00`00ooooo`37ool0
0`00ooooo`03ool30006ool004Soo`03003ooooo0;koo`@000Coo`03003ooooo00?oo`03003ooooo
00Koo`00BOoo00<00?ooool0aOoo00<00?ooool00ooo00<00?ooool01_oo001:ool00`00ooooo`32
ool30005ool00`00ooooo`06ool004[oo`03003ooooo0<Coo`03003ooooo00?oo`03003ooooo00Ko
o`00Booo00<00?ooool0bOoo0P001ooo001<ool00`00ooooo`38ool00`00ooooo`06ool004goo`03
003ooooo0<Ooo`03003ooooo00Koo`00C_oo00<00?ooool0a_oo00<00?ooool01_oo001>ool00`00
ooooo`36ool00`00ooooo`06ool004ooo`03003ooooo0<Goo`8000Ooo`00D?oo00<00?ooool0a?oo
00<00?ooool01_oo001Aool00`00ooooo`33ool00`00ooooo`06ool005;oo`03003ooooo0<;oo`03
003ooooo00Koo`00Dooo00<00?ooool0`Ooo00<00?ooool01_oo001Dool00`00ooooo`30ool20007
ool005Coo`03003ooooo0<3oo`03003ooooo00Koo`00EOoo00<00?ooool0_ooo00<00?ooool01_oo
001Fool00`00ooooo`2nool00`00ooooo`06ool005Ooo`03003ooooo0;goo`03003ooooo00Koo`00
F?oo00<00?ooool0_?oo00<00?ooool01_oo001Iool00`00ooooo`2kool20007ool005[oo`03003o
oooo0;[oo`03003ooooo00Koo`00Fooo00<00?ooool0^Ooo00<00?ooool01_oo001Lool00`00oooo
o`2Uool20004ool00`00ooooo`02ool30005ool00`00ooooo`06ool005coo`03003ooooo0:Coo`04
003ooooo000;ool00`00ooooo`02ool00`00ooooo`06ool005goo`03003ooooo0:?oo`04003ooooo
000;ool00`00ooooo`02ool30006ool005koo`03003ooooo09coo`@000;oo`04003ooooo0008ool3
0005ool00`00ooooo`06ool005ooo`03003ooooo0:7oo`04003ooooo0008ool00`00ooooo`05ool0
0`00ooooo`06ool0063oo`03003ooooo0:3oo`04003ooooo0008ool00`00ooooo`05ool00`00oooo
o`06ool0067oo`03003ooooo0:3oo`8000Woo`@000Coo`03003ooooo00Koo`00H_oo00<00?ooool0
/_oo0P001ooo001Sool00`00ooooo`2aool00`00ooooo`06ool006Coo`800;7oo`03003ooooo00Ko
o`00I_oo00<00?ooool0[_oo00<00?ooool01_oo001Wool00`00ooooo`2]ool00`00ooooo`06ool0
06Soo`03003ooooo0:coo`03003ooooo00Koo`00JOoo00<00?ooool0Zooo0P001ooo001Zool2002[
ool00`00ooooo`06ool006coo`03003ooooo0:Soo`03003ooooo00Koo`00KOoo00<00?ooool0Yooo
00<00?ooool01_oo001^ool2002Wool00`00ooooo`06ool001koo`8000Coo`03003ooooo00;oo`@0
02koo`8000Coo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo02Coo`8000Coo`03003ooooo
00?oo`8002ooo`8000Coo`03003ooooo00?oo`8003Goo`8000Ooo`007Ooo00@00?ooool000Soo`03
003ooooo02koo`04003ooooo000:ool00`00ooooo`06ool2000Sool01000ooooo`002?oo00@00?oo
ool002goo`04003ooooo0008ool01000ooooo`00=?oo00<00?ooool01_oo000Mool01000ooooo`00
2Ooo00<00?ooool0;Ooo00@00?ooool000Ooo`D000Woo`03003ooooo023oo`04003ooooo0008ool0
1000ooooo`00;Ooo00@00?ooool000Soo`04003ooooo000dool00`00ooooo`06ool001goo`04003o
oooo000:ool00`00ooooo`0/ool01000ooooo`001ooo00@00?ooool000_oo`80023oo`04003ooooo
0008ool3000^ool01000ooooo`002Ooo0P00=Ooo00<00?ooool01_oo000Mool01000ooooo`002ooo
00<00?ooool0:ooo00@00?ooool000Soo`03003oo`0000goo`03003ooooo01goo`04003ooooo0008
ool00`00ooooo`0^ool01000ooooo`002?oo00@00?ooool003Coo`03003ooooo00Koo`007Ooo00@0
0?ooool000Soo`04003ooooo000]ool01000ooooo`002Ooo0P003_oo0P007Ooo00@00?ooool000So
o`03003ooooo02koo`04003ooooo0008ool01000ooooo`00=?oo0P001ooo000Nool2000:ool2000_
ool2000;ool00`00ooooo`0>ool00`00ooooo`0Kool2000:ool3000^ool2000:ool2000eool00`00
ooooo`06ool007[oo`8009_oo`03003ooooo00Koo`00O?oo00<00?ooool0V?oo00<00?ooool01_oo
001mool2002Hool00`00ooooo`06ool00?l00200007oo`001ooo00<00?ooool03?oo00<00?ooool0
3?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03Ooo00<00?ooool0
3?oo00<00?ooool03?oo0P003Ooo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<0
0?ooool03Ooo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<0
0?ooool03?oo00<00?ooool01_oo0007ool00`00ooooo`0<ool00`00ooooo`0<ool00`00ooooo`0<
ool00`00ooooo`0<ool00`00ooooo`0<ool00`00ooooo`0=ool00`00ooooo`0<ool00`00ooooo`0<
ool01000ool000002ooo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool0
3Ooo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool0
3?oo00<00?ooool01_oo0024ool2002Aool00`00ooooo`06ool008Koo`8008ooo`03003ooooo00Ko
o`00R?oo0P00SOoo00<00?ooool01_oo002:ool2002;ool20007ool008coo`8008Woo`03003ooooo
00Koo`00S_oo0`00Q_oo00<00?ooool01_oo002Aool30023ool00`00ooooo`06ool009Coo`80087o
o`03003ooooo00Koo`00U_oo0`00O_oo0P001ooo002Iool3001kool00`00ooooo`06ool009coo`@0
07Ooo`03003ooooo00Koo`00X?oo1000Looo00<00?ooool01_oo002Tool5001^ool00`00ooooo`06
ool00:Woo`H006Soo`8000Ooo`00[ooo1@00Hooo00<00?ooool01_oo002dool6001Mool00`00oooo
o`06ool00;[oo`T005Coo`03003ooooo00Koo`00`ooo3@00@_oo1P002?oo003@oom20005ool00`00
ooooo`06ool00?oooaSoo`8000Ooo`00oooo6?oo00<00?ooool01_oo003ooolHool00`00ooooo`06
ool00?oooaSoo`03003ooooo00Koo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.0763281, -2.86837, \
0.00330085, 0.0188805}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[\((\(betakx0korr[eta, 10^4]/eta^2\)/100^2)\) /. 
          eta \[Rule] X/100], {X, 0.1, 1}, PlotRange \[Rule] All]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0820106 1.0582 0.603319 0.772104 [
[.12963 .59082 -9 -9 ]
[.12963 .59082 9 0 ]
[.34127 .59082 -9 -9 ]
[.34127 .59082 9 0 ]
[.55291 .59082 -9 -9 ]
[.55291 .59082 9 0 ]
[.76455 .59082 -9 -9 ]
[.76455 .59082 9 0 ]
[.96369 .14006 -24 -4.5 ]
[.96369 .14006 0 4.5 ]
[.96369 .29448 -24 -4.5 ]
[.96369 .29448 0 4.5 ]
[.96369 .4489 -24 -4.5 ]
[.96369 .4489 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.12963 .60332 m
.12963 .60957 L
s
[(0.2)] .12963 .59082 0 1 Mshowa
.34127 .60332 m
.34127 .60957 L
s
[(0.4)] .34127 .59082 0 1 Mshowa
.55291 .60332 m
.55291 .60957 L
s
[(0.6)] .55291 .59082 0 1 Mshowa
.76455 .60332 m
.76455 .60957 L
s
[(0.8)] .76455 .59082 0 1 Mshowa
.125 Mabswid
.18254 .60332 m
.18254 .60707 L
s
.23545 .60332 m
.23545 .60707 L
s
.28836 .60332 m
.28836 .60707 L
s
.39418 .60332 m
.39418 .60707 L
s
.44709 .60332 m
.44709 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.60582 .60332 m
.60582 .60707 L
s
.65873 .60332 m
.65873 .60707 L
s
.71164 .60332 m
.71164 .60707 L
s
.81746 .60332 m
.81746 .60707 L
s
.87037 .60332 m
.87037 .60707 L
s
.92328 .60332 m
.92328 .60707 L
s
.07672 .60332 m
.07672 .60707 L
s
.02381 .60332 m
.02381 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.97619 .14006 m
.98244 .14006 L
s
[(-0.6)] .96369 .14006 1 0 Mshowa
.97619 .29448 m
.98244 .29448 L
s
[(-0.4)] .96369 .29448 1 0 Mshowa
.97619 .4489 m
.98244 .4489 L
s
[(-0.2)] .96369 .4489 1 0 Mshowa
.125 Mabswid
.97619 .17866 m
.97994 .17866 L
s
.97619 .21727 m
.97994 .21727 L
s
.97619 .25587 m
.97994 .25587 L
s
.97619 .33308 m
.97994 .33308 L
s
.97619 .37169 m
.97994 .37169 L
s
.97619 .41029 m
.97994 .41029 L
s
.97619 .4875 m
.97994 .4875 L
s
.97619 .52611 m
.97994 .52611 L
s
.97619 .56471 m
.97994 .56471 L
s
.97619 .10145 m
.97994 .10145 L
s
.97619 .06285 m
.97994 .06285 L
s
.97619 .02424 m
.97994 .02424 L
s
.25 Mabswid
.97619 0 m
.97619 .61803 L
s
.5 Mabswid
.02381 .01472 m
.06244 .0297 L
.10458 .05058 L
.14415 .07398 L
.18221 .09942 L
.22272 .12903 L
.26171 .15942 L
.30316 .19307 L
.34309 .22622 L
.3815 .25828 L
.42237 .29208 L
.46172 .32394 L
.49955 .35362 L
.53984 .38394 L
.57861 .41168 L
.61984 .43947 L
.65954 .46449 L
.69774 .48689 L
.73838 .50892 L
.77751 .52841 L
.81909 .5473 L
.85916 .56381 L
.89771 .5782 L
.93871 .592 L
.97619 .60332 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo6?oo00<00?oo
ool01_oo003ooolHool00`00ooooo`06ool00?oooaSoo`03003ooooo00Koo`00oooo6?oo00<00?oo
ool01_oo0007ool2003oool?ool00`00ooooo`06ool000Woo`<00?ooo`coo`03003ooooo00Koo`00
3?oo0P00oooo2_oo00<00?ooool01_oo000>ool3003oool7ool20007ool0017oo`800?ooo`Goo`03
003ooooo00Koo`004ooo0P00oooo0ooo00<00?ooool01_oo000Eool2003oool1ool00`00ooooo`06
ool001Ooo`800?koo`03003ooooo00Koo`006Ooo0P00o?oo00<00?ooool01_oo000Kool2003jool0
0`00ooooo`06ool001goo`800?Soo`03003ooooo00Koo`007ooo0P00m_oo00<00?ooool01_oo000Q
ool00`00ooooo`3cool00`00ooooo`06ool002;oo`800??oo`03003ooooo00Koo`009?oo0P00lOoo
0P001ooo000Vool00`00ooooo`3^ool00`00ooooo`06ool002Ooo`800>koo`03003ooooo00Koo`00
:Ooo00<00?ooool0jooo00<00?ooool01_oo000Zool2003[ool00`00ooooo`06ool002coo`03003o
oooo0>Soo`03003ooooo00Koo`00;Ooo0P00j?oo00<00?ooool01_oo000_ool2003Vool00`00oooo
o`06ool0037oo`03003ooooo0>?oo`03003ooooo00Koo`00<_oo0P00hooo00<00?ooool01_oo000d
ool00`00ooooo`3Pool00`00ooooo`06ool003Goo`03003ooooo0=ooo`8000Ooo`00=_oo0P00gooo
00<00?ooool01_oo000hool00`00ooooo`3Lool00`00ooooo`06ool003Woo`03003ooooo0=_oo`03
003ooooo00Koo`00>_oo0P00fooo00<00?ooool01_oo000lool00`00ooooo`3Hool00`00ooooo`06
ool003goo`03003ooooo0=Ooo`03003ooooo00Koo`00?_oo0P00eooo00<00?ooool01_oo0010ool0
0`00ooooo`3Dool00`00ooooo`06ool0047oo`800<7oo`8000Coo`03003ooooo00?oo`8000Goo`03
003ooooo00Koo`00@ooo00<00?ooool0_Ooo00@00?ooool000Soo`04003ooooo0004ool00`00oooo
o`06ool004Coo`03003ooooo0;coo`04003ooooo0008ool01000ooooo`001?oo0`001_oo0015ool2
002fool40002ool01000ooooo`002?oo0`001Ooo00<00?ooool01_oo0017ool00`00ooooo`2iool0
1000ooooo`002?oo00<00?ooool01Ooo00<00?ooool01_oo0018ool00`00ooooo`2hool01000oooo
o`002?oo00<00?ooool01Ooo00<00?ooool01_oo0019ool2002iool2000:ool30004ool00`00oooo
o`06ool004_oo`03003ooooo0<Woo`03003ooooo00Koo`00C?oo00<00?ooool0b?oo00<00?ooool0
1_oo001=ool00`00ooooo`37ool00`00ooooo`06ool004koo`800<Ooo`03003ooooo00Koo`00D?oo
00<00?ooool0a?oo00<00?ooool01_oo001Aool00`00ooooo`33ool00`00ooooo`06ool005;oo`03
003ooooo0<;oo`8000Ooo`00Dooo00<00?ooool0`Ooo00<00?ooool01_oo001Dool20031ool00`00
ooooo`06ool005Koo`03003ooooo0;koo`03003ooooo00Koo`00Eooo00<00?ooool0_Ooo00<00?oo
ool01_oo001Hool00`00ooooo`2lool00`00ooooo`06ool005Woo`800;coo`03003ooooo00Koo`00
Fooo00<00?ooool0^Ooo00<00?ooool01_oo001Lool00`00ooooo`2hool00`00ooooo`06ool005go
o`03003ooooo0;Ooo`03003ooooo00Koo`00G_oo00<00?ooool0]_oo00<00?ooool01_oo001Oool2
002fool20007ool0067oo`03003ooooo0;?oo`03003ooooo00Koo`00H_oo00<00?ooool0/_oo00<0
0?ooool01_oo001Sool00`00ooooo`2aool00`00ooooo`06ool006Coo`03003ooooo0;3oo`03003o
oooo00Koo`00IOoo00<00?ooool0[ooo00<00?ooool01_oo001Vool00`00ooooo`2^ool00`00oooo
o`06ool006Ooo`800:koo`03003ooooo00Koo`00JOoo00<00?ooool0Zooo00<00?ooool01_oo001Z
ool00`00ooooo`2Zool00`00ooooo`06ool006_oo`03003ooooo0:Woo`03003ooooo00Koo`00K?oo
00<00?ooool0Z?oo0P001ooo001]ool00`00ooooo`2Wool00`00ooooo`06ool006koo`03003ooooo
0:Koo`03003ooooo00Koo`00Kooo0P00Y_oo00<00?ooool01_oo001aool00`00ooooo`2Sool00`00
ooooo`06ool007;oo`03003ooooo0:;oo`03003ooooo00Koo`00Looo0P00X_oo00<00?ooool01_oo
001eool00`00ooooo`2Oool00`00ooooo`06ool007Koo`03003ooooo09koo`03003ooooo00Koo`00
Mooo0P00Rooo0P001?oo00<00?ooool01?oo00<00?ooool00ooo00<00?ooool01_oo001iool00`00
ooooo`27ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool01_oo001jool00`00ooooo`26
ool01000ooooo`001ooo1@001?oo0`001_oo001kool20020ool40002ool01000ooooo`001ooo00@0
0?ooool000Goo`03003ooooo00Koo`00OOoo00<00?ooool0Pooo00@00?ooool000Soo`03003oo`00
00Goo`03003ooooo00Koo`00O_oo00<00?ooool0P_oo00@00?ooool000Woo`8000Goo`03003ooooo
00Koo`00Oooo00<00?ooool0P_oo0P002ooo00<00?ooool00ooo00<00?ooool01_oo0020ool00`00
ooooo`2Dool00`00ooooo`06ool0087oo`8009Coo`03003ooooo00Koo`00Pooo00<00?ooool0TOoo
00<00?ooool01_oo0024ool00`00ooooo`2@ool00`00ooooo`06ool008Goo`03003ooooo08ooo`03
003ooooo00Koo`00Q_oo0P00Sooo00<00?ooool01_oo0028ool00`00ooooo`2<ool20007ool008Wo
o`03003ooooo08_oo`03003ooooo00Koo`00R_oo00<00?ooool0R_oo00<00?ooool01_oo002;ool0
0`00ooooo`29ool00`00ooooo`06ool008coo`8008Woo`03003ooooo00Koo`00S_oo00<00?ooool0
Q_oo00<00?ooool01_oo002?ool00`00ooooo`25ool00`00ooooo`06ool0093oo`8008Goo`03003o
oooo00Koo`00T_oo00<00?ooool0P_oo00<00?ooool01_oo002Cool00`00ooooo`21ool00`00oooo
o`06ool009Coo`80087oo`03003ooooo00Koo`00U_oo00<00?ooool0O_oo0P001ooo002Gool00`00
ooooo`1mool00`00ooooo`06ool009Soo`8007goo`03003ooooo00Koo`00V_oo00<00?ooool0N_oo
00<00?ooool01_oo002Kool2001jool00`00ooooo`06ool009goo`03003ooooo07Ooo`03003ooooo
00Koo`00W_oo0P00Mooo00<00?ooool01_oo002Pool00`00ooooo`1dool00`00ooooo`06ool00:7o
o`8007Coo`03003ooooo00Koo`00Xooo00<00?ooool0LOoo00<00?ooool01_oo002Tool2001aool0
0`00ooooo`06ool00:Koo`03003ooooo06koo`8000Ooo`00Yooo0P00K_oo00<00?ooool01_oo002Y
ool00`00ooooo`1[ool00`00ooooo`06ool00:[oo`03003ooooo06[oo`03003ooooo00Koo`00Zooo
0P00J_oo00<00?ooool01_oo002]ool00`00ooooo`1Wool00`00ooooo`06ool00:koo`03003ooooo
06Koo`03003ooooo00Koo`00[ooo0P00I_oo00<00?ooool01_oo002aool00`00ooooo`1Sool00`00
ooooo`06ool00;;oo`80053oo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo00Koo`00]?oo
00<00?ooool0C?oo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo00Koo`00]Ooo0P00C?oo
00@00?ooool000Woo`03003ooooo00Coo`<000Koo`00]ooo00<00?ooool0@ooo10000_oo00@00?oo
ool000[oo`03003ooooo00?oo`03003ooooo00Koo`00^?oo0P00BOoo00@00?ooool000_oo`03003o
oooo00;oo`03003ooooo00Koo`00^_oo00<00?ooool0A_oo00@00?ooool000Soo`04003ooooo0004
ool00`00ooooo`06ool00;_oo`8004Ooo`8000[oo`8000Goo`03003ooooo00Koo`00_Ooo00<00?oo
ool0Eooo00<00?ooool01_oo002nool2001Gool00`00ooooo`06ool00<3oo`8005Goo`03003ooooo
00Koo`00`_oo0P00Dooo00<00?ooool01_oo0034ool2001Aool00`00ooooo`06ool00<Koo`8004oo
o`03003ooooo00Koo`00b?oo00<00?ooool0C?oo0P001ooo0039ool2001<ool00`00ooooo`06ool0
0<_oo`8004[oo`03003ooooo00Koo`00cOoo0P00B?oo00<00?ooool01_oo003?ool20016ool00`00
ooooo`06ool00=7oo`8004Coo`03003ooooo00Koo`00dooo00<00?ooool0@Ooo00<00?ooool01_oo
003Dool20011ool00`00ooooo`06ool00=Koo`8003ooo`03003ooooo00Koo`00f?oo0P00?Ooo00<0
0?ooool01_oo003Jool2000kool00`00ooooo`06ool00=coo`8003Woo`8000Ooo`00g_oo0P00=ooo
00<00?ooool01_oo003Pool2000eool00`00ooooo`06ool00>;oo`8003?oo`03003ooooo00Koo`00
i?oo0`00<?oo00<00?ooool01_oo003Wool2000^ool00`00ooooo`06ool00>Woo`<002_oo`03003o
oooo00Koo`00k?oo0P00:Ooo00<00?ooool01_oo003^ool2000Wool00`00ooooo`06ool00?3oo`<0
02Coo`03003ooooo00Koo`00looo0P008_oo00<00?ooool01_oo003eool3000Oool20007ool001ko
o`8000Coo`03003ooooo00;oo`@002koo`8000Coo`03003ooooo00Coo`03003ooooo02coo`8000Co
o`03003ooooo00?oo`8002ooo`8000Coo`03003ooooo00?oo`8001Koo`<001coo`03003ooooo00Ko
o`007Ooo00@00?ooool000Soo`03003ooooo02koo`04003ooooo000:ool00`00ooooo`0[ool01000
ooooo`002?oo00@00?ooool002goo`04003ooooo0008ool01000ooooo`006?oo0P006_oo00<00?oo
ool01_oo000Mool01000ooooo`002Ooo00<00?ooool0;Ooo00@00?ooool000Ooo`D002coo`04003o
oooo0008ool01000ooooo`00;Ooo00@00?ooool000Soo`04003ooooo000Jool3000Gool00`00oooo
o`06ool001goo`04003ooooo000:ool00`00ooooo`0/ool01000ooooo`001ooo00@00?ooool002go
o`04003ooooo0008ool3000^ool01000ooooo`002Ooo0P007_oo0`005?oo00<00?ooool01_oo000M
ool01000ooooo`002ooo00<00?ooool0:ooo00@00?ooool000Soo`03003oo`0002goo`04003ooooo
0008ool00`00ooooo`0^ool01000ooooo`002?oo00@00?ooool0023oo`<0017oo`03003ooooo00Ko
o`007Ooo00@00?ooool000Soo`04003ooooo000]ool01000ooooo`002Ooo0P00;Ooo00@00?ooool0
00Soo`03003ooooo02koo`04003ooooo0008ool01000ooooo`008ooo0`003_oo00<00?ooool01_oo
000Nool2000:ool2000_ool2000;ool00`00ooooo`0/ool2000:ool3000^ool2000:ool2000Wool3
000;ool00`00ooooo`06ool00?ooo`goo`<000Soo`03003ooooo00Koo`00oooo4?oo0`001Ooo00<0
0?ooool01_oo003ooolCool400000ooo003oo`07ool00?l00200007oo`001ooo00<00?ooool03?oo
00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03Ooo
00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo
00<00?ooool03?oo00<00?ooool03Ooo00<00?ooool03?oo00<00?ooool03?oo00<00?ooool03?oo
00<00?ooool03?oo00<00?ooool03?oo00<00?ooool01_oo000Uool00`00ooooo`0jool00`00oooo
o`0iool00`00ooooo`0jool00`00ooooo`0iool00`00ooooo`06ool00?oooaSoo`03003ooooo00Ko
o`00oooo6?oo00<00?ooool01_oo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.0763281, -0.7814, \
0.00330085, 0.00452395}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[{Re[betakx0korr[\((X - I\ 0.5)\)/50, 10^4]], 
        Re[\((X - I\ 0.5)\)^2]}, {X, 0.1, 1}, 
      PlotRange \[Rule] {0, 1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0 0.618034 [
[.21429 -0.0125 -9 -9 ]
[.21429 -0.0125 9 0 ]
[.40476 -0.0125 -9 -9 ]
[.40476 -0.0125 9 0 ]
[.59524 -0.0125 -9 -9 ]
[.59524 -0.0125 9 0 ]
[.78571 -0.0125 -9 -9 ]
[.78571 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 0 m
.21429 .00625 L
s
[(0.2)] .21429 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(0.4)] .40476 -0.0125 0 1 Mshowa
.59524 0 m
.59524 .00625 L
s
[(0.6)] .59524 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(0.8)] .78571 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(1)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.5 0 m
.5 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(1)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.50092 0 m
.51177 .02314 L
.54699 .08329 L
.58441 .13519 L
.62047 .17625 L
.65517 .2093 L
.69207 .23896 L
.72761 .26323 L
.76178 .28334 L
.79816 .30189 L
.83318 .31743 L
.87041 .33188 L
.90627 .34411 L
.94077 .35455 L
.97619 .36411 L
s
.49969 0 m
.51316 .00866 L
.54722 .03216 L
.58347 .05892 L
.61837 .08636 L
.65547 .11736 L
.69121 .14899 L
.72558 .18106 L
.76216 .21696 L
.79738 .25324 L
.8348 .29364 L
.87086 .33438 L
.90556 .37525 L
.94246 .42053 L
.97619 .46353 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool004Goo`8000Coo`03003ooooo00;oo`@002?oo`8000Coo`03003ooooo00Coo`03003ooooo027o
o`8000Coo`03003ooooo00?oo`8002Coo`8000Coo`03003ooooo00?oo`8002Soo`@000coo`00A?oo
00@00?ooool000Soo`03003ooooo02?oo`04003ooooo000:ool00`00ooooo`0Pool01000ooooo`00
2?oo00@00?ooool002;oo`04003ooooo0008ool01000ooooo`00:Ooo00<00?ooool02ooo0014ool0
1000ooooo`002Ooo00<00?ooool08_oo00@00?ooool000Ooo`D0027oo`04003ooooo0008ool01000
ooooo`008_oo00@00?ooool000Soo`04003ooooo000Yool00`00ooooo`0;ool004Coo`04003ooooo
000:ool00`00ooooo`0Qool01000ooooo`001ooo00@00?ooool002;oo`04003ooooo0008ool3000S
ool01000ooooo`002Ooo0P00:_oo00<00?ooool02ooo0014ool01000ooooo`002ooo00<00?ooool0
8?oo00@00?ooool000Soo`03003oo`0002;oo`04003ooooo0008ool00`00ooooo`0Sool01000oooo
o`002?oo00@00?ooool002Woo`03003ooooo00_oo`00A?oo00@00?ooool000Soo`04003ooooo000R
ool01000ooooo`002Ooo0P008_oo00@00?ooool000Soo`03003ooooo02?oo`04003ooooo0008ool0
1000ooooo`009ooo0`003Ooo0015ool2000:ool2000Tool2000;ool00`00ooooo`0Qool2000:ool3
000Sool2000:ool2000Zool00`00ooooo`0;ool00?ooob7oo`00oooo8Ooo003ooolQool001Goool0
00D000Ooo`006ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo
00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo
00<00?ooool02_oo0P002_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?oo
ool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?oo
ool02Ooo00<00?ooool02ooo000Kool00`00ooooo`1jool30025ool001_oo`03003ooooo07[oo`03
003ooooo008008?oo`006ooo00<00?ooool0Nooo00D00?ooooooo`0008;oo`006ooo00<00?ooool0
Nooo00<00?ooool00_oo0P00P?oo000Kool00`00ooooo`1lool00`00ooooo`03ool00`00ooooo`1m
ool001_oo`03003ooooo07goo`03003ooooo00?oo`8007goo`006ooo0P00O_oo00<00?ooool01Ooo
00<00?ooool0N_oo000Kool00`00ooooo`1nool00`00ooooo`05ool00`00ooooo`1iool001_oo`03
003ooooo07koo`03003ooooo00Koo`8007Woo`006ooo00<00?ooool0Oooo00<00?ooool01ooo00<0
0?ooool0M_oo000Kool00`00ooooo`20ool00`00ooooo`07ool00`00ooooo`1eool001_oo`03003o
oooo083oo`03003ooooo00Soo`8007Goo`006ooo00<00?ooool0POoo00<00?ooool02Ooo00<00?oo
ool0L_oo000Kool00`00ooooo`21ool00`00ooooo`0:ool00`00ooooo`1aool001_oo`8008?oo`03
003ooooo00[oo`03003ooooo073oo`006ooo00<00?ooool0Pooo00<00?ooool02_oo0P00L?oo000K
ool00`00ooooo`23ool00`00ooooo`0<ool00`00ooooo`1]ool001_oo`03003ooooo08Coo`03003o
oooo00coo`03003ooooo06coo`006ooo00<00?ooool0Q?oo00<00?ooool03Ooo0P00K?oo000Kool0
0`00ooooo`25ool00`00ooooo`0>ool00`00ooooo`1Yool001_oo`03003ooooo08Koo`03003ooooo
00koo`03003ooooo06Soo`006ooo00<00?ooool0Q_oo00<00?ooool03ooo00<00?ooool0Iooo000K
ool20028ool00`00ooooo`0?ool2001Wool001_oo`03003ooooo08Soo`03003ooooo013oo`03003o
oooo06Coo`006ooo00<00?ooool0R?oo00<00?ooool04Ooo00<00?ooool0Hooo000Kool00`00oooo
o`29ool00`00ooooo`0Aool00`00ooooo`1Rool001_oo`03003ooooo08Woo`03003ooooo01;oo`80
06;oo`006ooo00<00?ooool0R_oo00<00?ooool04ooo00<00?ooool0Gooo0008ool20004ool00`00
ooooo`02ool40004ool00`00ooooo`2;ool00`00ooooo`0Cool00`00ooooo`1Nool000Ooo`04003o
oooo0008ool00`00ooooo`05ool00`00ooooo`2;ool00`00ooooo`0Dool00`00ooooo`1Mool000Oo
o`04003ooooo0009ool00`00ooooo`04ool3002<ool00`00ooooo`0Dool00`00ooooo`1Lool000Oo
o`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`2=ool00`00ooooo`0Dool00`00ooooo`1K
ool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`2=ool00`00ooooo`0Eool2001K
ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0S_oo00<00?ooool05_oo00<0
0?ooool0F?oo0008ool2000:ool20005ool00`00ooooo`2?ool00`00ooooo`0Fool00`00ooooo`1G
ool001_oo`03003ooooo093oo`03003ooooo01Koo`03003ooooo05Koo`006ooo00<00?ooool0TOoo
00<00?ooool05_oo00<00?ooool0EOoo000Kool00`00ooooo`2Bool00`00ooooo`0Fool00`00oooo
o`1Dool001_oo`8009Coo`03003ooooo01Koo`03003ooooo05?oo`006ooo00<00?ooool0U?oo00<0
0?ooool05_oo00<00?ooool0D_oo000Kool00`00ooooo`2Eool00`00ooooo`0Fool00`00ooooo`1A
ool001_oo`03003ooooo09Koo`03003ooooo01Koo`03003ooooo053oo`006ooo00<00?ooool0Uooo
00<00?ooool05_oo00<00?ooool0Cooo000Kool00`00ooooo`2Hool00`00ooooo`0Fool00`00oooo
o`1>ool001_oo`03003ooooo09Woo`03003ooooo01Koo`03003ooooo04goo`006ooo00<00?ooool0
V_oo00<00?ooool05_oo00<00?ooool0C?oo000Kool2002Lool00`00ooooo`0Fool00`00ooooo`1;
ool001_oo`03003ooooo09coo`03003ooooo01Koo`03003ooooo04[oo`006ooo00<00?ooool0WOoo
00<00?ooool05_oo00<00?ooool0BOoo000Kool00`00ooooo`2Nool00`00ooooo`0Fool00`00oooo
o`18ool001_oo`03003ooooo09ooo`03003ooooo01Koo`03003ooooo04Ooo`006ooo00<00?ooool0
X?oo00<00?ooool05_oo00<00?ooool0A_oo000Kool00`00ooooo`2Qool00`00ooooo`0Fool00`00
ooooo`15ool001_oo`03003ooooo0:;oo`03003ooooo01Koo`03003ooooo04Coo`006ooo0P00Y?oo
00<00?ooool05_oo00<00?ooool0@ooo000Kool00`00ooooo`2Tool00`00ooooo`0Fool00`00oooo
o`12ool001_oo`03003ooooo0:Goo`8001Ooo`03003ooooo047oo`006ooo00<00?ooool0Yooo00<0
0?ooool05Ooo00<00?ooool0@?oo000Kool00`00ooooo`2Xool00`00ooooo`0Eool00`00ooooo`0o
ool001_oo`03003ooooo0:Woo`03003ooooo01Goo`8003ooo`002?oo0P001?oo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool0Z_oo00<00?ooool05_oo00<00?ooool0??oo0007ool01000oooo
o`002_oo00<00?ooool00ooo00<00?ooool0Zooo0P005ooo00<00?ooool0>ooo0007ool01000oooo
o`001ooo1@001?oo0`00[Ooo0P005_oo00<00?ooool0>_oo0007ool01000ooooo`001ooo00@00?oo
ool000Goo`03003ooooo0:ooo`03003ooooo01Coo`03003ooooo03Woo`001ooo00@00?ooool000So
o`03003oo`0000Goo`03003ooooo0;3oo`8001Goo`03003ooooo03Soo`001ooo00@00?ooool000Wo
o`8000Goo`03003ooooo0;;oo`8001Coo`03003ooooo03Ooo`002?oo0P002ooo00<00?ooool00ooo
00<00?ooool0]?oo00<00?ooool04Ooo00<00?ooool0=ooo000Kool00`00ooooo`2eool2000Bool0
0`00ooooo`0fool001_oo`03003ooooo0;Ooo`03003ooooo013oo`03003ooooo03Goo`006ooo00<0
0?ooool0^?oo0P004Ooo00<00?ooool0=?oo000Kool2002kool2000@ool00`00ooooo`0cool001_o
o`03003ooooo0;coo`03003ooooo00koo`03003ooooo03;oo`006ooo00<00?ooool0_Ooo0P003_oo
00<00?ooool0<_oo000Kool00`00ooooo`2oool2000=ool00`00ooooo`0aool001_oo`03003ooooo
0<7oo`8000coo`03003ooooo033oo`006ooo00<00?ooool0`ooo0P002ooo00<00?ooool0;ooo000K
ool00`00ooooo`35ool30009ool00`00ooooo`0^ool001_oo`03003ooooo0<Soo`8000Soo`03003o
oooo02goo`006ooo0P00booo0P001ooo00<00?ooool0;?oo000Kool00`00ooooo`3<ool20006ool0
0`00ooooo`0[ool001_oo`03003ooooo0<koo`<000Coo`03003ooooo02[oo`006ooo00<00?ooool0
dOoo0P000ooo00<00?ooool0:Ooo000Kool00`00ooooo`3Cool300000ooo003oo`0Yool001_oo`03
003ooooo0=Koo`<002Woo`006ooo00<00?ooool0f?oo0`009ooo000Kool00`00ooooo`3Jool4000T
ool001_oo`800=coo`03003ooooo00<0027oo`006ooo00<00?ooool0fooo00<00?ooool00ooo0`00
7_oo000Kool00`00ooooo`3Lool00`00ooooo`05ool3000Kool001_oo`03003ooooo0=goo`03003o
oooo00Ooo`<001Soo`006ooo00<00?ooool0g_oo00<00?ooool02Ooo10005?oo000Kool00`00oooo
o`3Oool00`00ooooo`0<ool4000@ool000Soo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo
0>3oo`03003ooooo00ooo`<000goo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00oooo
o`3Pool00`00ooooo`0Oool000Ooo`04003ooooo0008ool01000ooooo`001?oo0`00hOoo00<00?oo
ool07_oo0007ool01000ooooo`002?oo0`001Ooo00<00?ooool0h_oo00<00?ooool07Ooo0007ool0
1000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0hooo00<00?ooool07?oo0007ool01000oooo
o`002?oo00<00?ooool01Ooo00<00?ooool0hooo00<00?ooool07?oo0008ool2000:ool30004ool0
0`00ooooo`3Tool00`00ooooo`0Kool001_oo`03003ooooo0>Goo`03003ooooo01[oo`006ooo00<0
0?ooool0i_oo00<00?ooool06Ooo000Kool00`00ooooo`3Vool00`00ooooo`0Iool001_oo`800>So
o`03003ooooo01Soo`006ooo00<00?ooool0j?oo00<00?ooool05ooo000Kool00`00ooooo`3Yool0
0`00ooooo`0Fool001_oo`03003ooooo0>Woo`03003ooooo01Koo`006ooo00<00?ooool0j_oo00<0
0?ooool05Ooo000Kool00`00ooooo`3[ool00`00ooooo`0Dool001_oo`03003ooooo0>coo`03003o
oooo01?oo`006ooo00<00?ooool0kOoo00<00?ooool04_oo000Kool2003^ool00`00ooooo`0Bool0
01_oo`03003ooooo0>koo`03003ooooo017oo`006ooo00<00?ooool0kooo00<00?ooool04?oo000K
ool00`00ooooo`3`ool00`00ooooo`0?ool001_oo`03003ooooo0?7oo`03003ooooo00koo`006ooo
00<00?ooool0l_oo00<00?ooool03Ooo000Kool00`00ooooo`3bool00`00ooooo`0=ool001_oo`03
003ooooo0??oo`03003ooooo00coo`006ooo0P00mOoo00<00?ooool02ooo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0?ooo`?oo`002?oo0P001?oo00<00?ooool00ooo0P001Ooo00<00?oo
ool0oooo0ooo0007ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo0?ooo`?oo`001ooo
00@00?ooool000Soo`04003ooooo0004ool3003oool3ool000Ooo`04003ooooo0009ool20005ool0
0`00ooooo`3oool3ool000Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0oooo0ooo
0007ool01000ooooo`002?oo00@00?ooool000Coo`03003ooooo0?ooo`?oo`002?oo0P002_oo0P00
1Ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`00
6ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<0
0?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<0
0?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00
oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0
oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0
oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo
000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo
000Kool00`00ooooo`3oool3ool001?oo`@000Coo`03003ooooo0?ooo`?oo`005Ooo00<00?ooool0
0ooo00<00?ooool0oooo0ooo000Eool00`00ooooo`03ool3003oool3ool001Goo`03003ooooo0?oo
o`Woo`005Ooo00<00?ooool0oooo2Ooo000Cool3003oool;ool001Goo`03003ooooo0?ooo`Woo`00
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.111819, -0.0764114, \
0.00405086, 0.00624232}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(smaller\ \(\(a\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, Om /. strangesol[k, 10^3]}, {k, 0.8, 2, 0.005}]\)], "Input"],

Cell[BoxData[
    \({{0.8`, 0.8937077898564706`}, {0.805`, 0.8941713995384458`}, {0.81`, 
        0.8946450558378207`}, {0.8150000000000001`, 
        0.8951289370283508`}, {0.8200000000000001`, 
        0.8956231672760314`}, {0.8250000000000001`, 
        0.8961277961413202`}, {0.8300000000000001`, 
        0.8966427717814492`}, {0.8350000000000001`, 
        0.8971679061216296`}, {0.8400000000000001`, 
        0.8977028299451324`}, {0.8450000000000001`, 
        0.8982469360331068`}, {0.8500000000000001`, 
        0.8987993110059891`}, {0.8550000000000001`, 
        0.8993579722689853`}, {0.8600000000000001`, 
        0.8999219481261286`}, {0.865`, 0.9004885747289201`}, {0.87`, 
        0.9010535341453457`}, {0.875`, 0.9016120170346956`}, {0.88`, 
        0.9021580740863374`}, {0.885`, 0.9026826115569716`}, {0.89`, 
        0.9031743151492545`}, {0.895`, 0.9036187849892594`}, {0.9`, 
        0.903998017967028`}, {0.905`, 0.9042900576074392`}, {0.91`, 
        0.9044714516728488`}, {0.915`, 0.9045202895855287`}, {0.92`, 
        0.9044220482895527`}, {0.925`, 0.9041790643772474`}, {0.93`, 
        0.9038111500461004`}, {0.935`, 
        0.9033523399399987`}, {0.9400000000000001`, 
        0.902840180510005`}, {0.9450000000000001`, 
        0.9023066625619517`}, {0.9500000000000001`, 
        0.9017744701841673`}, {0.9550000000000001`, 
        0.901257551800144`}, {0.9600000000000001`, 
        0.9007633929941233`}, {0.9650000000000001`, 
        0.90029528759902`}, {0.9700000000000001`, 
        0.8998540317880984`}, {0.9750000000000001`, 
        0.8994390329784129`}, {0.9800000000000001`, 
        0.8990489865720969`}, {0.9850000000000001`, 
        0.8986822712514925`}, {0.9900000000000001`, 
        0.8983371718814852`}, {0.995`, 0.8980120002628792`}, {1.`, 
        0.8977051566029215`}, {1.0050000000000001`, 
        0.8974151571698218`}, {1.01`, 
        0.897140643052038`}, {1.0150000000000001`, 
        0.8968803786912504`}, {1.02`, 
        0.8966332451881708`}, {1.0250000000000001`, 
        0.8963982312346327`}, {1.03`, 
        0.8961744232729123`}, {1.0350000000000001`, 
        0.8959609957535573`}, {1.04`, 
        0.8957572019397058`}, {1.0450000000000002`, 
        0.8955623654625862`}, {1.05`, 
        0.8953758726953904`}, {1.0550000000000002`, 
        0.8951971659374176`}, {1.06`, 0.8950257373614713`}, {1.065`, 
        0.89486112365965`}, {1.07`, 0.8947029013168505`}, {1.075`, 
        0.8945506824420666`}, {1.08`, 0.8944041110917693`}, {1.085`, 
        0.8942628600254323`}, {1.09`, 0.8941266278395372`}, {1.095`, 
        0.8939951364326042`}, {1.1`, 0.8938681287595968`}, {1.105`, 
        0.8937453668393484`}, {1.11`, 0.893626629983367`}, {1.115`, 
        0.8935117132185353`}, {1.12`, 0.89340049614748`}, {1.125`, 
        0.8932926553659393`}, {1.1300000000000001`, 
        0.8931881012081215`}, {1.135`, 
        0.893086678837817`}, {1.1400000000000001`, 
        0.8929882434308812`}, {1.145`, 
        0.8928926593558805`}, {1.1500000000000001`, 
        0.8927997994349917`}, {1.155`, 
        0.8927095442761204`}, {1.1600000000000001`, 
        0.8926217816683358`}, {1.165`, 
        0.892536406033693`}, {1.1700000000000002`, 
        0.8924533179293713`}, {1.175`, 
        0.8923724235947851`}, {1.1800000000000002`, 
        0.8922936345389595`}, {1.185`, 0.892216867164023`}, {1.19`, 
        0.892142042421148`}, {1.195`, 0.8920690854956872`}, {1.2`, 
        0.8919979255186297`}, {1.205`, 0.891928495301814`}, {1.21`, 
        0.89186073109462`}, {1.215`, 0.8917945723601133`}, {1.22`, 
        0.8917299615688226`}, {1.225`, 0.8916668440085369`}, {1.23`, 
        0.8916051676086627`}, {1.235`, 0.8915448827778447`}, {1.24`, 
        0.8914859422536793`}, {1.245`, 0.8914283009634637`}, {1.25`, 
        0.8913719158950413`}, {1.2550000000000001`, 
        0.8913167459768754`}, {1.26`, 
        0.8912627519665935`}, {1.2650000000000001`, 
        0.8912098963472902`}, {1.27`, 
        0.891158143230961`}, {1.2750000000000001`, 
        0.8911074582684939`}, {1.28`, 
        0.8910578085656946`}, {1.2850000000000001`, 
        0.8910091626048715`}, {1.29`, 
        0.8909614901715514`}, {1.2950000000000002`, 
        0.8909147622859281`}, {1.3`, 
        0.8908689511386905`}, {1.3050000000000002`, 
        0.8908240300308965`}, {1.31`, 0.8907799733176`}, {1.315`, 
        0.8907367563549475`}, {1.32`, 0.8906943554505039`}, {1.325`, 
        0.8906527478165671`}, {1.33`, 0.8906119115262646`}, {1.335`, 
        0.8905718254722392`}, {1.34`, 0.8905324693277389`}, {1.345`, 
        0.8904938235099523`}, {1.35`, 0.8904558691454345`}, {1.355`, 
        0.890418588037483`}, {1.36`, 0.8903819626353368`}, {1.365`, 
        0.8903459760050756`}, {1.37`, 0.8903106118021145`}, {1.375`, 
        0.890275854245182`}, {1.3800000000000001`, 
        0.8902416880916987`}, {1.385`, 
        0.8902080986144586`}, {1.3900000000000001`, 
        0.8901750715795383`}, {1.395`, 
        0.8901425932253535`}, {1.4000000000000001`, 
        0.890110650242796`}, {1.405`, 
        0.8900792297563849`}, {1.4100000000000001`, 
        0.8900483193063694`}, {1.415`, 
        0.8900179068317307`}, {1.4200000000000002`, 
        0.8899879806540228`}, {1.425`, 
        0.889958529462014`}, {1.4300000000000002`, 
        0.8899295422970709`}, {1.435`, 0.8899010085392516`}, {1.44`, 
        0.8898729178940638`}, {1.445`, 0.8898452603798502`}, {1.45`, 
        0.8898180263157668`}, {1.455`, 0.8897912063103239`}, {1.46`, 
        0.8897647912504527`}, {1.465`, 0.889738772291077`}, {1.47`, 
        0.8897131408451547`}, {1.475`, 0.8896878885741712`}, {1.48`, 
        0.8896630073790559`}, {1.485`, 0.8896384893915005`}, {1.49`, 
        0.889614326965661`}, {1.495`, 0.8895905126702186`}, {1.5`, 
        0.8895670392807884`}, {1.5050000000000001`, 
        0.8895438997726464`}, {1.51`, 
        0.8895210873137742`}, {1.5150000000000001`, 
        0.8894985952581927`}, {1.52`, 
        0.8894764171395751`}, {1.5250000000000001`, 
        0.8894545466651276`}, {1.53`, 
        0.8894329777097215`}, {1.5350000000000001`, 
        0.8894117043102652`}, {1.54`, 
        0.889390720660305`}, {1.5450000000000002`, 
        0.8893700211048456`}, {1.55`, 
        0.8893496001353748`}, {1.5550000000000002`, 
        0.8893294523850891`}, {1.56`, 
        0.8893095726243053`}, {1.5650000000000002`, 
        0.8892899557560536`}, {1.57`, 0.8892705968118417`}, {1.575`, 
        0.8892514909475806`}, {1.58`, 0.8892326334396706`}, {1.585`, 
        0.8892140196812327`}, {1.59`, 0.8891956451784837`}, {1.595`, 
        0.8891775055472494`}, {1.6`, 0.8891595965096036`}, {1.605`, 
        0.8891419138906363`}, {1.61`, 0.8891244536153377`}, {1.615`, 
        0.8891072117055975`}, {1.62`, 0.8890901842773126`}, {1.625`, 
        0.8890733675375995`}, {1.6300000000000001`, 
        0.8890567577821065`}, {1.635`, 
        0.8890403513924211`}, {1.6400000000000001`, 
        0.8890241448335702`}, {1.645`, 
        0.8890081346516071`}, {1.6500000000000001`, 
        0.8889923174712838`}, {1.655`, 
        0.8889766899938027`}, {1.6600000000000001`, 
        0.8889612489946475`}, {1.665`, 
        0.8889459913214866`}, {1.6700000000000002`, 
        0.8889309138921504`}, {1.675`, 
        0.8889160136926739`}, {1.6800000000000002`, 
        0.8889012877754088`}, {1.685`, 
        0.888886733257195`}, {1.6900000000000002`, 
        0.8888723473175959`}, {1.695`, 0.8888581271971902`}, {1.7`, 
        0.8888440701959207`}, {1.705`, 0.8888301736714969`}, {1.71`, 
        0.8888164350378488`}, {1.715`, 0.8888028517636324`}, {1.72`, 
        0.8887894213707797`}, {1.725`, 0.8887761414330992`}, {1.73`, 
        0.8887630095749176`}, {1.735`, 0.8887500234697662`}, {1.74`, 
        0.8887371808391067`}, {1.745`, 0.888724479451099`}, {1.75`, 
        0.8887119171194058`}, {1.7550000000000001`, 
        0.8886994917020333`}, {1.76`, 
        0.8886872011002095`}, {1.7650000000000001`, 
        0.8886750432572957`}, {1.77`, 
        0.8886630161577305`}, {1.7750000000000001`, 
        0.888651117826006`}, {1.78`, 
        0.8886393463256758`}, {1.7850000000000001`, 
        0.8886276997583907`}, {1.79`, 
        0.8886161762629647`}, {1.7950000000000002`, 
        0.8886047740144681`}, {1.8`, 
        0.8885934912233465`}, {1.8050000000000002`, 
        0.8885823261345661`}, {1.81`, 
        0.8885712770267848`}, {1.8150000000000002`, 
        0.888560342211545`}, {1.82`, 0.8885495200324927`}, {1.825`, 
        0.8885388088646152`}, {1.83`, 0.8885282071135049`}, {1.835`, 
        0.8885177132146399`}, {1.84`, 0.888507325632687`}, {1.845`, 
        0.8884970428608241`}, {1.85`, 0.8884868634200823`}, {1.855`, 
        0.8884767858587035`}, {1.86`, 0.8884668087515185`}, {1.865`, 
        0.8884569306993413`}, {1.87`, 0.8884471503283802`}, {1.875`, 
        0.8884374662896637`}, {1.8800000000000001`, 
        0.8884278772584846`}, {1.885`, 
        0.8884183819338557`}, {1.8900000000000001`, 
        0.8884089790379837`}, {1.895`, 
        0.8883996673157535`}, {1.9000000000000001`, 
        0.8883904455342296`}, {1.905`, 
        0.8883813124821693`}, {1.9100000000000001`, 
        0.8883722669695467`}, {1.915`, 
        0.8883633078270938`}, {1.9200000000000002`, 
        0.8883544339058487`}, {1.925`, 
        0.8883456440767192`}, {1.9300000000000002`, 
        0.8883369372300556`}, {1.935`, 
        0.888328312275235`}, {1.9400000000000002`, 
        0.8883197681402575`}, {1.945`, 
        0.8883113037713499`}, {1.9500000000000002`, 
        0.8883029181325836`}, {1.955`, 0.8882946102054972`}, {1.96`, 
        0.8882863789887334`}, {1.965`, 0.8882782234976812`}, {1.97`, 
        0.8882701427641293`}, {1.975`, 0.8882621358359286`}, {1.98`, 
        0.8882542017766598`}, {1.985`, 0.8882463396653124`}, {1.99`, 
        0.8882385485959712`}, {1.995`, 0.8882308276775083`}, {2.`, 
        0.8882231760332856`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.611111 0.793651 -32.0653 36.1171 [
[.02381 .42756 -9 -9 ]
[.02381 .42756 9 0 ]
[.34127 .42756 -9 -9 ]
[.34127 .42756 9 0 ]
[.5 .42756 -9 -9 ]
[.5 .42756 9 0 ]
[.65873 .42756 -9 -9 ]
[.65873 .42756 9 0 ]
[.81746 .42756 -9 -9 ]
[.81746 .42756 9 0 ]
[.97619 .42756 -3 -9 ]
[.97619 .42756 3 0 ]
[.17004 .07889 -24 -4.5 ]
[.17004 .07889 0 4.5 ]
[.17004 .16918 -36 -4.5 ]
[.17004 .16918 0 4.5 ]
[.17004 .25947 -30 -4.5 ]
[.17004 .25947 0 4.5 ]
[.17004 .34977 -36 -4.5 ]
[.17004 .34977 0 4.5 ]
[.17004 .53035 -36 -4.5 ]
[.17004 .53035 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .44006 m
.02381 .44631 L
s
[(0.8)] .02381 .42756 0 1 Mshowa
.34127 .44006 m
.34127 .44631 L
s
[(1.2)] .34127 .42756 0 1 Mshowa
.5 .44006 m
.5 .44631 L
s
[(1.4)] .5 .42756 0 1 Mshowa
.65873 .44006 m
.65873 .44631 L
s
[(1.6)] .65873 .42756 0 1 Mshowa
.81746 .44006 m
.81746 .44631 L
s
[(1.8)] .81746 .42756 0 1 Mshowa
.97619 .44006 m
.97619 .44631 L
s
[(2)] .97619 .42756 0 1 Mshowa
.125 Mabswid
.06349 .44006 m
.06349 .44381 L
s
.10317 .44006 m
.10317 .44381 L
s
.14286 .44006 m
.14286 .44381 L
s
.22222 .44006 m
.22222 .44381 L
s
.2619 .44006 m
.2619 .44381 L
s
.30159 .44006 m
.30159 .44381 L
s
.38095 .44006 m
.38095 .44381 L
s
.42063 .44006 m
.42063 .44381 L
s
.46032 .44006 m
.46032 .44381 L
s
.53968 .44006 m
.53968 .44381 L
s
.57937 .44006 m
.57937 .44381 L
s
.61905 .44006 m
.61905 .44381 L
s
.69841 .44006 m
.69841 .44381 L
s
.7381 .44006 m
.7381 .44381 L
s
.77778 .44006 m
.77778 .44381 L
s
.85714 .44006 m
.85714 .44381 L
s
.89683 .44006 m
.89683 .44381 L
s
.93651 .44006 m
.93651 .44381 L
s
.25 Mabswid
0 .44006 m
1 .44006 L
s
.18254 .07889 m
.18879 .07889 L
s
[(0.89)] .17004 .07889 1 0 Mshowa
.18254 .16918 m
.18879 .16918 L
s
[(0.8925)] .17004 .16918 1 0 Mshowa
.18254 .25947 m
.18879 .25947 L
s
[(0.895)] .17004 .25947 1 0 Mshowa
.18254 .34977 m
.18879 .34977 L
s
[(0.8975)] .17004 .34977 1 0 Mshowa
.18254 .53035 m
.18879 .53035 L
s
[(0.9025)] .17004 .53035 1 0 Mshowa
.125 Mabswid
.18254 .09695 m
.18629 .09695 L
s
.18254 .11501 m
.18629 .11501 L
s
.18254 .13306 m
.18629 .13306 L
s
.18254 .15112 m
.18629 .15112 L
s
.18254 .18724 m
.18629 .18724 L
s
.18254 .2053 m
.18629 .2053 L
s
.18254 .22336 m
.18629 .22336 L
s
.18254 .24142 m
.18629 .24142 L
s
.18254 .27753 m
.18629 .27753 L
s
.18254 .29559 m
.18629 .29559 L
s
.18254 .31365 m
.18629 .31365 L
s
.18254 .33171 m
.18629 .33171 L
s
.18254 .36783 m
.18629 .36783 L
s
.18254 .38588 m
.18629 .38588 L
s
.18254 .40394 m
.18629 .40394 L
s
.18254 .422 m
.18629 .422 L
s
.18254 .45812 m
.18629 .45812 L
s
.18254 .47618 m
.18629 .47618 L
s
.18254 .49423 m
.18629 .49423 L
s
.18254 .51229 m
.18629 .51229 L
s
.18254 .54841 m
.18629 .54841 L
s
.18254 .56647 m
.18629 .56647 L
s
.18254 .58453 m
.18629 .58453 L
s
.18254 .60259 m
.18629 .60259 L
s
.18254 .06083 m
.18629 .06083 L
s
.18254 .04277 m
.18629 .04277 L
s
.18254 .02471 m
.18629 .02471 L
s
.18254 .00665 m
.18629 .00665 L
s
.25 Mabswid
.18254 0 m
.18254 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .2128 m
.02778 .22955 L
.03175 .24665 L
.03571 .26413 L
.03968 .28198 L
.04365 .30021 L
.04762 .31881 L
.05159 .33777 L
.05556 .35709 L
.05952 .37674 L
.06349 .39669 L
.06746 .41687 L
.07143 .43724 L
.0754 .45771 L
.07937 .47811 L
.08333 .49828 L
.0873 .518 L
.09127 .53695 L
.09524 .55471 L
.09921 .57076 L
.10317 .58446 L
.10714 .595 L
.11111 .60156 L
.11508 .60332 L
.11905 .59977 L
.12302 .59099 L
.12698 .57771 L
.13095 .56114 L
.13492 .54264 L
.13889 .52337 L
.14286 .50415 L
.14683 .48548 L
.15079 .46763 L
.15476 .45072 L
.15873 .43479 L
.1627 .4198 L
.16667 .40571 L
.17063 .39247 L
.1746 .38 L
.17857 .36826 L
.18254 .35718 L
.18651 .3467 L
.19048 .33679 L
.19444 .32739 L
.19841 .31846 L
.20238 .30997 L
.20635 .30189 L
.21032 .29418 L
.21429 .28682 L
.21825 .27979 L
Mistroke
.22222 .27305 L
.22619 .2666 L
.23016 .2604 L
.23413 .25446 L
.2381 .24874 L
.24206 .24325 L
.24603 .23795 L
.25 .23285 L
.25397 .22793 L
.25794 .22318 L
.2619 .21859 L
.26587 .21416 L
.26984 .20987 L
.27381 .20572 L
.27778 .2017 L
.28175 .19781 L
.28571 .19403 L
.28968 .19037 L
.29365 .18682 L
.29762 .18336 L
.30159 .18001 L
.30556 .17675 L
.30952 .17358 L
.31349 .1705 L
.31746 .1675 L
.32143 .16457 L
.3254 .16173 L
.32937 .15896 L
.33333 .15625 L
.3373 .15362 L
.34127 .15105 L
.34524 .14854 L
.34921 .14609 L
.35317 .1437 L
.35714 .14137 L
.36111 .13909 L
.36508 .13686 L
.36905 .13469 L
.37302 .13256 L
.37698 .13047 L
.38095 .12844 L
.38492 .12645 L
.38889 .1245 L
.39286 .12259 L
.39683 .12072 L
.40079 .11889 L
.40476 .11709 L
.40873 .11534 L
.4127 .11361 L
.41667 .11193 L
Mistroke
.42063 .11027 L
.4246 .10865 L
.42857 .10706 L
.43254 .1055 L
.43651 .10397 L
.44048 .10246 L
.44444 .10099 L
.44841 .09954 L
.45238 .09812 L
.45635 .09672 L
.46032 .09535 L
.46429 .09401 L
.46825 .09268 L
.47222 .09138 L
.47619 .09011 L
.48016 .08885 L
.48413 .08762 L
.4881 .0864 L
.49206 .08521 L
.49603 .08404 L
.5 .08289 L
.50397 .08175 L
.50794 .08063 L
.5119 .07954 L
.51587 .07845 L
.51984 .07739 L
.52381 .07634 L
.52778 .07531 L
.53175 .0743 L
.53571 .0733 L
.53968 .07232 L
.54365 .07135 L
.54762 .07039 L
.55159 .06945 L
.55556 .06853 L
.55952 .06762 L
.56349 .06672 L
.56746 .06583 L
.57143 .06496 L
.5754 .0641 L
.57937 .06325 L
.58333 .06242 L
.5873 .06159 L
.59127 .06078 L
.59524 .05998 L
.59921 .05919 L
.60317 .05841 L
.60714 .05764 L
.61111 .05688 L
.61508 .05614 L
Mistroke
.61905 .0554 L
.62302 .05467 L
.62698 .05395 L
.63095 .05324 L
.63492 .05254 L
.63889 .05185 L
.64286 .05117 L
.64683 .0505 L
.65079 .04984 L
.65476 .04918 L
.65873 .04854 L
.6627 .0479 L
.66667 .04727 L
.67063 .04664 L
.6746 .04603 L
.67857 .04542 L
.68254 .04482 L
.68651 .04423 L
.69048 .04364 L
.69444 .04307 L
.69841 .04249 L
.70238 .04193 L
.70635 .04137 L
.71032 .04082 L
.71429 .04028 L
.71825 .03974 L
.72222 .03921 L
.72619 .03868 L
.73016 .03816 L
.73413 .03765 L
.7381 .03714 L
.74206 .03664 L
.74603 .03614 L
.75 .03565 L
.75397 .03517 L
.75794 .03469 L
.7619 .03421 L
.76587 .03374 L
.76984 .03328 L
.77381 .03282 L
.77778 .03237 L
.78175 .03192 L
.78571 .03147 L
.78968 .03104 L
.79365 .0306 L
.79762 .03017 L
.80159 .02975 L
.80556 .02933 L
.80952 .02891 L
.81349 .0285 L
Mistroke
.81746 .02809 L
.82143 .02769 L
.8254 .02729 L
.82937 .02689 L
.83333 .0265 L
.8373 .02611 L
.84127 .02573 L
.84524 .02535 L
.84921 .02498 L
.85317 .02461 L
.85714 .02424 L
.86111 .02387 L
.86508 .02351 L
.86905 .02316 L
.87302 .0228 L
.87698 .02245 L
.88095 .02211 L
.88492 .02177 L
.88889 .02143 L
.89286 .02109 L
.89683 .02076 L
.90079 .02043 L
.90476 .0201 L
.90873 .01978 L
.9127 .01946 L
.91667 .01914 L
.92063 .01882 L
.9246 .01851 L
.92857 .0182 L
.93254 .0179 L
.93651 .0176 L
.94048 .0173 L
.94444 .017 L
.94841 .0167 L
.95238 .01641 L
.95635 .01612 L
.96032 .01584 L
.96429 .01555 L
.96825 .01527 L
.97222 .01499 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O=_oo00<00?ooool0
iooo000fool00`00ooooo`3Wool003Koo`800>Soo`00=_oo00<00?ooool0iooo000fool00`00oooo
o`3Kool50007ool003Koo`03003ooooo0<koo`d000coo`00=_oo00<00?ooool0`_oo30006Ooo000f
ool2002hool;000Uool003Koo`03003ooooo0:koo`T0033oo`00=_oo00<00?ooool0YOoo2@00>Ooo
000fool00`00ooooo`2Lool90012ool003Koo`03003ooooo09Coo`P004_oo`00=_oo0P00S_oo1`00
Dooo000fool00`00ooooo`26ool7001Jool003Koo`03003ooooo087oo`D0067oo`00=_oo00<00?oo
ool0Nooo1P00I_oo000fool00`00ooooo`1eool6001/ool003Koo`8007;oo`@007;oo`00=_oo00<0
0?ooool0K?oo1@00M_oo000fool00`00ooooo`1Xool4001kool001coo`8000Coo`03003ooooo00?o
o`8000?oo`<000Koo`03003ooooo06?oo`D007ooo`006ooo00@00?ooool000Soo`04003ooooo0005
ool00`00ooooo`03ool00`00ooooo`1Oool40024ool001_oo`04003ooooo0008ool01000ooooo`00
1Ooo00<00?ooool00ooo0P00GOoo0`00R?oo000Kool01000ooooo`002Ooo0P001?oo0`001Ooo00<0
0?ooool0F?oo1000Rooo000Kool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0005ool0
0`00ooooo`1Eool3002?ool001_oo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000Go
o`03003ooooo057oo`@009;oo`007?oo0P002_oo0P001?oo0P001_oo00<00?ooool0C_oo0`00U_oo
000fool00`00ooooo`1<ool2002Iool003Koo`8004Woo`@009_oo`00=_oo00<00?ooool0A_oo0P00
Wooo000fool00`00ooooo`14ool2002Qool003Koo`03003ooooo043oo`@00:?oo`00=_oo00<00?oo
ool0?_oo0P00Yooo000fool2000mool2002Yool003Koo`03003ooooo03[oo`800:_oo`00=_oo00<0
0?ooool0=ooo0`00[Ooo000fool00`00ooooo`0fool00`00ooooo`2^ool003Koo`03003ooooo03Co
o`800;7oo`00=_oo0P00<ooo0P00/ooo000fool00`00ooooo`0aool00`00ooooo`2cool003Koo`03
003ooooo02koo`<00;Koo`00=_oo00<00?ooool0;Ooo00<00?ooool0]ooo000fool00`00ooooo`0[
ool2002jool003Koo`8002_oo`03003ooooo0;[oo`00=_oo00<00?ooool0:Ooo00<00?ooool0^ooo
000fool00`00ooooo`0Vool3002nool0013oo`8000Coo`03003ooooo00?oo`8000?oo`<000?oo`@0
00;oo`<000Koo`03003ooooo02Goo`03003ooooo0;ooo`003ooo00@00?ooool000Soo`04003ooooo
0005ool01000ooooo`002?oo00<00?ooool00ooo00<00?ooool09?oo00<00?ooool0`?oo000?ool0
1000ooooo`002?oo00@00?ooool000Goo`05003oooooool00007ool00`00ooooo`03ool2000Tool0
0`00ooooo`31ool000ooo`04003ooooo0009ool20004ool30004ool01000oooooooo0`001_oo00<0
0?ooool08Ooo0P00a?oo000?ool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0005ool0
1000ooooo`002?oo00<00?ooool08?oo00<00?ooool0a?oo000?ool01000ooooo`002?oo00@00?oo
ool000;oo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool01_oo00<00?ooool07_oo0P00
aooo000@ool2000:ool20004ool20004ool20003ool40005ool00`00ooooo`0Mool00`00ooooo`37
ool003Koo`8001goo`03003ooooo0<Soo`00=_oo00<00?ooool06ooo00<00?ooool0bOoo000fool0
0`00ooooo`0Jool00`00ooooo`3:ool003Koo`03003ooooo01Woo`03003ooooo0<_oo`00=_oo00<0
0?ooool06?oo00<00?ooool0c?oo000fool2000Iool00`00ooooo`3<ool003Koo`03003ooooo01Oo
o`03003ooooo0<goo`00=_oo00<00?ooool05Ooo0P00d?oo0009ool00`00ooooo`0Zool00`00oooo
o`0Dool00`00ooooo`3@ool000Woo`03003ooooo02[oo`03003ooooo01?oo`03003ooooo0=7oo`00
2Ooo00<00?ooool0:_oo00<00?ooool04ooo00<00?ooool0dOoo000:ool00`00ooooo`0Yool2000C
ool00`00ooooo`3Bool000[oo`03003ooooo02Woo`03003ooooo017oo`03003ooooo0=?oo`002_oo
00<00?ooool0:Ooo00<00?ooool04?oo00<00?ooool0e?oo000:ool00`00ooooo`0Yool00`00oooo
o`0@ool00`00ooooo`3Dool000_oo`03003ooooo02Soo`03003ooooo00ooo`03003ooooo0=Goo`00
2ooo00<00?ooool0:?oo0P003ooo00<00?ooool0e_oo000;ool00`00ooooo`0Xool00`00ooooo`0>
ool00`00ooooo`3Fool000_oo`03003ooooo02Soo`03003ooooo00coo`800=Woo`002ooo00<00?oo
ool02?oo0P001?oo00<00?ooool00ooo0P000ooo0`000ooo0`001_oo00<00?ooool02ooo00<00?oo
ool0fOoo000<ool00`00ooooo`06ool01000ooooo`002?oo00@00?ooool000Goo`03003ooooo00?o
o`03003ooooo00?oo`03003ooooo00_oo`03003ooooo0=Woo`003?oo00<00?ooool01_oo00@00?oo
ool000Soo`04003ooooo0005ool00`00ooooo`03ool00`00ooooo`03ool2000;ool00`00ooooo`3J
ool000coo`03003ooooo00Koo`04003ooooo0009ool20004ool30002ool30006ool00`00ooooo`0:
ool00`00ooooo`3Jool000coo`03003ooooo00Koo`04003ooooo0008ool01000ooooo`000_oo00@0
0?ooool000;oo`03003ooooo00Koo`03003ooooo00Woo`03003ooooo0=_oo`003?oo00<00?ooool0
1_oo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool01_oo00<00?oo
ool02Ooo00<00?ooool0fooo000=ool00`00ooooo`06ool2000:ool20004ool20003ool40005ool0
0`00ooooo`08ool00`00ooooo`3Lool000goo`03003ooooo02Koo`8000Woo`03003ooooo0=coo`00
3Ooo00<00?ooool09_oo00<00?ooool01ooo00<00?ooool0gOoo000=ool00`00ooooo`0Vool00`00
ooooo`07ool00`00ooooo`3Mool000goo`03003ooooo02Koo`03003ooooo00Koo`03003ooooo0=ko
o`003Ooo00<00?ooool09_oo00<00?ooool01_oo00<00?ooool0g_oo000>ool00`00ooooo`0Uool2
0006ool00`00ooooo`3Oool000koo`03003ooooo02Goo`03003ooooo00Coo`03003ooooo0>3oo`00
3_oo00<00?ooool09Ooo00<00?ooool00ooo00<00?ooool0hOoo000>ool00`00ooooo`0Uool00`00
ooooo`03ool00`00ooooo`3Qool000koo`03003ooooo02Goo`03003ooooo00;oo`03003ooooo0>;o
o`003ooo00<00?ooool09?oo0P000ooo00<00?ooool0h_oo000?ool00`00ooooo`0Tool01@00oooo
oooo0000iOoo000?ool00`00ooooo`0Tool01@00oooooooo0000iOoo000?ool00`00ooooo`0Tool0
1@00oooooooo0000iOoo000?ool00`00ooooo`0Tool01000ooooo`00i_oo000@ool00`00ooooo`0S
ool200000ooo003oo`3Uool0013oo`03003ooooo02?oo`03003oo`000>Ooo`004?oo00<00?ooool0
8ooo00<00?oo0000iooo000@ool00`00ooooo`0Sool00`00ool0003Wool0013oo`8000Coo`03003o
oooo00?oo`8000?oo`<000Coo`03003ooooo00;oo`<000Koo`800>Soo`003ooo00@00?oo000000So
o`04003ooooo0005ool01@00oooooooo00001ooo00<00?ooool00ooo0P00j?oo000?ool01000ool0
00002?oo00@00?ooool000Goo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo00?oo`800>So
o`003ooo00@00?ooool000Woo`8000Coo`<000Coo`04003oooooool30006ool00`00ooooo`3Wool0
00ooo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000Goo`04003ooooo0008ool00`00
ooooo`3Wool000ooo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo
0002ool00`00ooooo`06ool00`00ooooo`3Wool0013oo`<000Woo`8000Coo`8000?oo`@000;oo`@0
00Coo`800>Woo`004ooo00<00?ooool07ooo0`00j?oo000Cool00`00ooooo`0Nool00`00ool0003Y
ool001?oo`03003ooooo01koo`03003oo`000>Woo`004ooo00<00?ooool07Ooo00@00?ooool00>Wo
o`004ooo00<00?ooool07Ooo00@00?ooool00>Woo`004ooo00<00?ooool07Ooo00<00?ooool00P00
j?oo000Dool00`00ooooo`0Kool01@00oooooooo0000jOoo000Dool00`00ooooo`0Kool01@00oooo
oooo0000jOoo000Dool00`00ooooo`0Kool01@00oooooooo0000jOoo000Dool00`00ooooo`0Kool0
1@00oooooooo0000jOoo0002ool20004ool00`00ooooo`03ool20004ool00`00ooooo`0Jool00`00
ooooo`02ool2000Sool40003ool00`00ooooo`02ool4000Mool40003ool00`00ooooo`04ool00`00
ooooo`0Lool40003ool00`00ooooo`03ool2000Oool40003ool00`00ooooo`03ool2000Tool40006
ool00005ool00?ooool00008ool01000ooooo`000ooo00<00?ooool06_oo00<00?ooool00_oo00<0
0?ooool09?oo00<00?ooool01ooo00<00?ooool08?oo00<00?ooool02Ooo00<00?ooool07_oo00<0
0?ooool01ooo00@00?ooool0023oo`03003ooooo00Ooo`04003ooooo000Sool00`00ooooo`07ool0
0005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool06Ooo00<00?ooool00_oo00<00?oo
ool09?oo00<00?ooool02?oo00<00?ooool07ooo00<00?ooool01_oo1@007ooo00<00?ooool01ooo
00@00?ooool0023oo`03003ooooo00Ooo`04003ooooo000Tool00`00ooooo`06ool00005ool00?oo
ool00009ool20005ool00`00ooooo`0Iool00`00ooooo`02ool00`00ooooo`0Tool00`00ooooo`09
ool00`00ooooo`0Nool00`00ooooo`06ool01000ooooo`008?oo00<00?ooool01ooo0`008Ooo00<0
0?ooool02?oo0P009_oo00<00?ooool01Ooo00001Ooo003ooooo00002?oo00@00?ooool000Coo`03
003ooooo01Soo`03003ooooo00?oo`03003ooooo02Coo`03003ooooo00[oo`03003ooooo01goo`03
003ooooo00Ooo`03003oo`00023oo`03003ooooo00Ooo`03003ooooo027oo`03003ooooo00Ooo`04
003ooooo000Vool00`00ooooo`04ool00005ool00?ooool00008ool01000ooooo`001?oo00<00?oo
ool06?oo00<00?ooool00ooo0P008ooo0`002Ooo00@00?ooool001goo`<000[oo`8001koo`<000Wo
o`03003ooooo01ooo`<000Woo`04003ooooo000Sool01000ooooo`001_oo0002ool2000:ool20005
ool00`00ooooo`0Hool00`00ooooo`03ool00`00ooooo`0Tool00`00ooooo`08ool2000Pool00`00
ooooo`09ool00`00ooooo`0Nool00`00ooooo`08ool3000Pool00`00ooooo`08ool2000Uool20007
ool001Goo`03003ooooo01Soo`03003ooooo00?oo`03003ooooo0>Ooo`005_oo00<00?ooool05_oo
00<00?ooool01?oo00<00?ooool0iooo000Fool00`00ooooo`0Fool00`00ooooo`04ool00`00oooo
o`3Wool000;oool001l000002Ooo00<00?ooool02?oo00<00?oo00002?oo00<00?ooool02?oo00<0
0?ooool00_oo00<00?ooool01?oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02Ooo00<0
0?ooool02?oo00<00?ooool02?oo00<00?ooool02Ooo00<00?ooool02?oo00<00?ooool02?oo00<0
0?ooool02?oo00<00?ooool02Ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02Ooo00<0
0?ooool02?oo00<00?ooool02?oo00<00?ooool02Ooo00<00?ooool02?oo00<00?ooool02?oo00<0
0?ooool02?oo00<00?ooool02Ooo00<00?ooool01Ooo0009ool00`00ooooo`0:ool00`00ooooo`0E
ool00`00ooooo`05ool00`00ooooo`0Zool00`00ooooo`0Zool00`00ooooo`0Zool00`00ooooo`0[
ool00`00ooooo`0Zool00`00ooooo`05ool001Ooo`03003ooooo01Coo`03003ooooo00Goo`03003o
oooo0>Ooo`005ooo00<00?ooool05?oo00<00?ooool01Ooo00<00?ooool0iooo000Gool00`00oooo
o`0Dool00`00ooooo`05ool2003Xool001Ooo`03003ooooo01?oo`03003ooooo00Koo`03003ooooo
0>Ooo`005ooo00<00?ooool04ooo00<00?ooool01_oo00<00?ooool0iooo000Gool00`00ooooo`0C
ool00`00ooooo`06ool00`00ooooo`3Wool001Soo`03003ooooo01;oo`03003ooooo00Koo`03003o
oooo0>Ooo`006?oo00<00?ooool04_oo00<00?ooool01_oo00<00?ooool0iooo000Hool00`00oooo
o`0Aool00`00ooooo`07ool2003Xool001Soo`03003ooooo017oo`03003ooooo00Ooo`03003ooooo
0>Ooo`006?oo00<00?ooool04Ooo00<00?ooool01ooo00<00?ooool0iooo000Hool00`00ooooo`0A
ool00`00ooooo`07ool00`00ooooo`3Wool001Woo`03003ooooo00ooo`03003ooooo00Soo`03003o
oooo0>Ooo`006Ooo00<00?ooool03ooo00<00?ooool02?oo0P00j?oo000Iool00`00ooooo`0?ool0
0`00ooooo`08ool00`00ooooo`3Wool001Woo`03003ooooo00koo`03003ooooo00Woo`03003ooooo
0>Ooo`006_oo00<00?ooool03Ooo00<00?ooool02Ooo00<00?ooool0iooo000Jool00`00ooooo`0=
ool00`00ooooo`09ool00`00ooooo`3Wool001[oo`03003ooooo00goo`03003ooooo00Woo`800>So
o`006ooo00<00?ooool02ooo00<00?ooool02_oo00<00?ooool0iooo000Kool00`00ooooo`0;ool0
0`00ooooo`0:ool00`00ooooo`3Wool0013oo`8000Coo`03003ooooo00;oo`<000Coo`8000?oo`@0
00;oo`<000Koo`03003ooooo0>Ooo`003ooo00@00?ooool000Soo`04003ooooo0002ool01000oooo
o`000_oo00<00?oo00001_oo00<00?ooool00ooo00<00?ooool0iooo000?ool01000ooooo`002Ooo
00<00?oo00000_oo00@00?ooool000?oo`8000Koo`03003ooooo00?oo`800>Soo`003ooo00@00?oo
ool000Woo`<000;oo`04003ooooo0004ool01000oooooooo0`001_oo00<00?ooool0iooo000?ool0
1000ooooo`002?oo0P0000Goo`00ooooo`0000;oo`07003oooooool00?oo00000_oo00<00?ooool0
1_oo00<00?ooool0iooo000?ool01000ooooo`002?oo0P0000Goo`00ooooo`0000;oo`03003ooooo
00800005ool00?ooool00008ool00`00ooooo`3Wool0013oo`8000[oo`8000Coo`8000Coo`8000?o
o`@000Goo`03003ooooo0>Ooo`007Ooo00<00?ooool02?oo00<00?ooool02ooo0P00j?oo000Mool0
0`00ooooo`08ool00`00ooooo`0;ool00`00ooooo`3Wool001goo`03003ooooo00Ooo`03003ooooo
00coo`03003ooooo0>Ooo`007Ooo00<00?ooool01ooo00<00?ooool03?oo00<00?ooool0iooo000M
ool00`00ooooo`07ool00`00ooooo`0<ool00`00ooooo`3Wool001koo`03003ooooo00Koo`03003o
oooo00coo`800>Soo`007_oo00<00?ooool01_oo00<00?ooool03?oo00<00?ooool0iooo000Nool0
0`00ooooo`05ool00`00ooooo`0=ool00`00ooooo`3Wool001koo`03003ooooo00Goo`03003ooooo
00goo`03003ooooo0>Ooo`007ooo00<00?ooool01?oo00<00?ooool03Ooo00<00?ooool0iooo000O
ool00`00ooooo`04ool00`00ooooo`0=ool2003Xool001ooo`03003ooooo00?oo`03003ooooo00ko
o`03003ooooo0>Ooo`008?oo00<00?ooool00_oo00<00?ooool03_oo00<00?ooool0iooo000Pool0
0`00ooooo`02ool00`00ooooo`0>ool00`00ooooo`3Wool0023oo`05003oooooool0000Aool00`00
ooooo`3Wool0027oo`04003oo`00000Aool00`00ooooo`3Wool002;oo`03003ooooo017oo`800>So
o`00=_oo00<00?ooool0iooo000fool00`00ooooo`3Wool003Koo`03003ooooo0>Ooo`00=_oo00<0
0?ooool0iooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.760174, 0.887769, \
0.0044245, 9.72257*^-5}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, Om /. strangesol[k, 10^2]}, {k, 0.8, 2, 0.005}]\)], "Input"],

Cell[BoxData[
    \({{0.8`, 0.9102955702770633`}, {0.805`, 0.9109956745686295`}, {0.81`, 
        0.9116909358713182`}, {0.8150000000000001`, 
        0.9123806964460391`}, {0.8200000000000001`, 
        0.9130642325635376`}, {0.8250000000000001`, 
        0.9137409231806454`}, {0.8300000000000001`, 
        0.9144098323253287`}, {0.8350000000000001`, 
        0.9150698389912876`}, {0.8400000000000001`, 
        0.9157205893289914`}, {0.8450000000000001`, 
        0.9163609364888481`}, {0.8500000000000001`, 
        0.9169893353432694`}, {0.8550000000000001`, 
        0.9176048251813465`}, {0.8600000000000001`, 
        0.9182057053691146`}, {0.865`, 0.9187914620191453`}, {0.87`, 
        0.9193600958232762`}, {0.875`, 0.9199103860604524`}, {0.88`, 
        0.9204399604579908`}, {0.885`, 0.9209473540158654`}, {0.89`, 
        0.9214306569509204`}, {0.895`, 0.9218878625078436`}, {0.9`, 
        0.9223168875472977`}, {0.905`, 0.9227155595848978`}, {0.91`, 
        0.9230816774389148`}, {0.915`, 0.9234130205625817`}, {0.92`, 
        0.9237072941299503`}, {0.925`, 0.923962525236544`}, {0.93`, 
        0.9241765860763269`}, {0.935`, 
        0.9243477326272558`}, {0.9400000000000001`, 
        0.9244743731299533`}, {0.9450000000000001`, 
        0.9245553698665302`}, {0.9500000000000001`, 
        0.9245899491982856`}, {0.9550000000000001`, 
        0.9245778571835473`}, {0.9600000000000001`, 
        0.9245194084497012`}, {0.9650000000000001`, 
        0.9244155133236156`}, {0.9700000000000001`, 
        0.9242677044928157`}, {0.9750000000000001`, 
        0.9240781101130608`}, {0.9800000000000001`, 
        0.9238494056238763`}, {0.9850000000000001`, 
        0.923584733246249`}, {0.9900000000000001`, 
        0.9232875978124008`}, {0.995`, 0.9229617477566602`}, {1.`, 
        0.9226110514200038`}, {1.0050000000000001`, 
        0.9222391175413126`}, {1.01`, 
        0.9218502991970724`}, {1.0150000000000001`, 
        0.9214478341125516`}, {1.02`, 
        0.9210349739884064`}, {1.0250000000000001`, 
        0.9206148605863532`}, {1.03`, 
        0.9201901465866956`}, {1.0350000000000001`, 
        0.9197632073439205`}, {1.04`, 
        0.9193361003220274`}, {1.0450000000000002`, 
        0.9189105822411632`}, {1.05`, 
        0.918488474360141`}, {1.0550000000000002`, 
        0.9180703882985095`}, {1.06`, 0.9176576004011948`}, {1.065`, 
        0.9172509097250262`}, {1.07`, 0.9168509435302586`}, {1.075`, 
        0.9164581824602712`}, {1.08`, 0.9160729806763538`}, {1.085`, 
        0.9156955775385522`}, {1.09`, 0.9153261594983825`}, {1.095`, 
        0.9149647958948223`}, {1.1`, 0.9146115155546793`}, {1.105`, 
        0.9142663002586864`}, {1.11`, 0.9139292417479616`}, {1.115`, 
        0.9135998852199988`}, {1.12`, 0.9132784214839241`}, {1.125`, 
        0.9129643982646104`}, {1.1300000000000001`, 
        0.9126581636174248`}, {1.135`, 
        0.9123590507560829`}, {1.1400000000000001`, 
        0.9120671081144047`}, {1.145`, 
        0.911782208593138`}, {1.1500000000000001`, 
        0.9115041069343304`}, {1.155`, 
        0.9112327857657895`}, {1.1600000000000001`, 
        0.9109677120124617`}, {1.165`, 
        0.910708750542676`}, {1.1700000000000002`, 
        0.9104560535637857`}, {1.175`, 
        0.9102092272166816`}, {1.1800000000000002`, 
        0.9099681204432669`}, {1.185`, 0.9097325655118281`}, {1.19`, 
        0.9095023970757941`}, {1.195`, 0.9092775098930153`}, {1.2`, 
        0.9090576459732821`}, {1.205`, 0.9088426933582125`}, {1.21`, 
        0.9086325057343012`}, {1.215`, 0.9084269408673057`}, {1.22`, 
        0.9082258607802787`}, {1.225`, 0.908029131124488`}, {1.23`, 
        0.9078366231140881`}, {1.235`, 0.9076482119702829`}, {1.24`, 
        0.9074637768032849`}, {1.245`, 0.9072832008382055`}, {1.25`, 
        0.9071063713098819`}, {1.2550000000000001`, 
        0.9069331793500852`}, {1.26`, 
        0.9067635198699201`}, {1.2650000000000001`, 
        0.9065972914395076`}, {1.27`, 
        0.9064343961665338`}, {1.2750000000000001`, 
        0.9062747395748706`}, {1.28`, 
        0.9061182304842021`}, {1.2850000000000001`, 
        0.9059647808913578`}, {1.29`, 
        0.9058143058539058`}, {1.2950000000000002`, 
        0.9056667233764045`}, {1.3`, 
        0.9055219542996205`}, {1.3050000000000002`, 
        0.9053799221929217`}, {1.31`, 0.9052405532499997`}, {1.315`, 
        0.9051037761880056`}, {1.32`, 0.9049695221501488`}, {1.325`, 
        0.904837724611772`}, {1.33`, 0.9047083192898768`}, {1.335`, 
        0.9045812440560699`}, {1.34`, 0.9044564388528636`}, {1.345`, 
        0.9043338456132638`}, {1.35`, 0.9042134081835602`}, {1.355`, 
        0.9040950722492312`}, {1.36`, 0.9039787852638684`}, {1.365`, 
        0.9038644963810182`}, {1.37`, 0.9037521563888455`}, {1.375`, 
        0.903641717647511`}, {1.3800000000000001`, 
        0.9035331340291641`}, {1.385`, 
        0.9034263608604495`}, {1.3900000000000001`, 
        0.9033213548674257`}, {1.395`, 
        0.9032180741227999`}, {1.4000000000000001`, 
        0.9031164779953814`}, {1.405`, 
        0.9030165271016628`}, {1.4100000000000001`, 
        0.9029181832594342`}, {1.415`, 
        0.9028214094433479`}, {1.4200000000000002`, 
        0.9027261697423465`}, {1.425`, 
        0.9026324293188724`}, {1.4300000000000002`, 
        0.9025401543697816`}, {1.435`, 0.9024493120888859`}, {1.44`, 
        0.9023598706310509`}, {1.445`, 0.902271799077783`}, {1.45`, 
        0.9021850674042323`}, {1.455`, 0.902099646447559`}, {1.46`, 
        0.9020155078765908`}, {1.465`, 0.9019326241627207`}, {1.47`, 
        0.9018509685519882`}, {1.475`, 0.9017705150382898`}, {1.48`, 
        0.9016912383376698`}, {1.485`, 0.9016131138636416`}, {1.49`, 
        0.9015361177034941`}, {1.495`, 0.9014602265955424`}, {1.5`, 
        0.9013854179072749`}, {1.5050000000000001`, 
        0.9013116696143632`}, {1.51`, 
        0.9012389602804922`}, {1.5150000000000001`, 
        0.9011672690379781`}, {1.52`, 
        0.9010965755691364`}, {1.5250000000000001`, 
        0.901026860088369`}, {1.53`, 
        0.9009581033249393`}, {1.5350000000000001`, 
        0.9008902865064047`}, {1.54`, 
        0.9008233913426777`}, {1.5450000000000002`, 
        0.9007573919822864`}, {1.55`, 
        0.9006922872141733`}, {1.5550000000000002`, 
        0.9006280519702918`}, {1.56`, 
        0.9005646697424485`}, {1.5650000000000002`, 
        0.9005021244336373`}, {1.57`, 0.9004404003453507`}, {1.575`, 
        0.9003794821653542`}, {1.58`, 0.9003193549559066`}, {1.585`, 
        0.9002600041424049`}, {1.59`, 0.9002014155024384`}, {1.595`, 
        0.9001435751552358`}, {1.6`, 0.900086469551487`}, {1.605`, 
        0.9000300854635235`}, {1.61`, 0.8999744099758485`}, {1.615`, 
        0.8999194304759952`}, {1.62`, 0.899865134645705`}, {1.625`, 
        0.8998115104524109`}, {1.6300000000000001`, 
        0.8997585461410152`}, {1.635`, 
        0.8997062302259475`}, {1.6400000000000001`, 
        0.899654551483495`}, {1.645`, 
        0.8996034989443914`}, {1.6500000000000001`, 
        0.8995530618866566`}, {1.655`, 
        0.8995032298286763`}, {1.6600000000000001`, 
        0.8994539925225123`}, {1.665`, 
        0.8994053399474364`}, {1.6700000000000002`, 
        0.8993572623036736`}, {1.675`, 
        0.8993097500063558`}, {1.6800000000000002`, 
        0.8992627936796697`}, {1.685`, 
        0.8992163841511954`}, {1.6900000000000002`, 
        0.8991705124464293`}, {1.695`, 0.8991251697834807`}, {1.7`, 
        0.8990803475679404`}, {1.705`, 0.8990360373879113`}, {1.71`, 
        0.8989922310091959`}, {1.715`, 0.8989489203706365`}, {1.72`, 
        0.8989060975796005`}, {1.725`, 0.8988637549076065`}, {1.73`, 
        0.8988218847860863`}, {1.735`, 0.898780479802277`}, {1.74`, 
        0.8987395326952408`}, {1.745`, 0.8986990363520033`}, {1.75`, 
        0.8986589838038124`}, {1.7550000000000001`, 
        0.8986193682225083`}, {1.76`, 
        0.898580182917003`}, {1.7650000000000001`, 
        0.8985414213298657`}, {1.77`, 
        0.8985030770340089`}, {1.7750000000000001`, 
        0.898465143729474`}, {1.78`, 
        0.8984276152403107`}, {1.7850000000000001`, 
        0.8983904855115479`}, {1.79`, 
        0.8983537486062539`}, {1.7950000000000002`, 
        0.8983173987026812`}, {1.8`, 
        0.898281430091494`}, {1.8050000000000002`, 
        0.8982458371730754`}, {1.81`, 
        0.8982106144549127`}, {1.8150000000000002`, 
        0.8981757565490553`}, {1.82`, 0.8981412581696472`}, {1.825`, 
        0.8981071141305266`}, {1.83`, 0.8980733193428942`}, {1.835`, 
        0.8980398688130474`}, {1.84`, 0.8980067576401761`}, {1.845`, 
        0.8979739810142198`}, {1.85`, 0.8979415342137855`}, {1.855`, 
        0.8979094126041206`}, {1.86`, 0.8978776116351419`}, {1.865`, 
        0.8978461268395188`}, {1.87`, 0.8978149538308072`}, {1.875`, 
        0.8977840883016355`}, {1.8800000000000001`, 
        0.897753526021937`}, {1.885`, 
        0.8977232628372314`}, {1.8900000000000001`, 
        0.8976932946669507`}, {1.895`, 
        0.8976636175028105`}, {1.9000000000000001`, 
        0.8976342274072233`}, {1.905`, 
        0.8976051205117533`}, {1.9100000000000001`, 
        0.8975762930156127`}, {1.915`, 
        0.8975477411841967`}, {1.9200000000000002`, 
        0.8975194613476546`}, {1.925`, 
        0.8974914498995009`}, {1.9300000000000002`, 
        0.8974637032952592`}, {1.935`, 
        0.8974362180511429`}, {1.9400000000000002`, 
        0.897408990742767`}, {1.945`, 
        0.8973820180038962`}, {1.9500000000000002`, 
        0.8973552965252198`}, {1.955`, 0.8973288230531624`}, {1.96`, 
        0.8973025943887198`}, {1.965`, 0.8972766073863271`}, {1.97`, 
        0.8972508589527538`}, {1.975`, 0.8972253460460257`}, {1.98`, 
        0.8972000656743738`}, {1.985`, 0.8971750148952088`}, {1.99`, 
        0.8971501908141206`}, {1.995`, 0.8971255905839028`}, {2.`, 
        0.8971012114035996`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.611111 0.793651 -19.1945 21.4125 [
[.02381 .06429 -9 -9 ]
[.02381 .06429 9 0 ]
[.34127 .06429 -9 -9 ]
[.34127 .06429 9 0 ]
[.5 .06429 -9 -9 ]
[.5 .06429 9 0 ]
[.65873 .06429 -9 -9 ]
[.65873 .06429 9 0 ]
[.81746 .06429 -9 -9 ]
[.81746 .06429 9 0 ]
[.97619 .06429 -3 -9 ]
[.97619 .06429 3 0 ]
[.17004 .18385 -30 -4.5 ]
[.17004 .18385 0 4.5 ]
[.17004 .29091 -24 -4.5 ]
[.17004 .29091 0 4.5 ]
[.17004 .39797 -30 -4.5 ]
[.17004 .39797 0 4.5 ]
[.17004 .50504 -24 -4.5 ]
[.17004 .50504 0 4.5 ]
[.17004 .6121 -30 -4.5 ]
[.17004 .6121 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .07679 m
.02381 .08304 L
s
[(0.8)] .02381 .06429 0 1 Mshowa
.34127 .07679 m
.34127 .08304 L
s
[(1.2)] .34127 .06429 0 1 Mshowa
.5 .07679 m
.5 .08304 L
s
[(1.4)] .5 .06429 0 1 Mshowa
.65873 .07679 m
.65873 .08304 L
s
[(1.6)] .65873 .06429 0 1 Mshowa
.81746 .07679 m
.81746 .08304 L
s
[(1.8)] .81746 .06429 0 1 Mshowa
.97619 .07679 m
.97619 .08304 L
s
[(2)] .97619 .06429 0 1 Mshowa
.125 Mabswid
.06349 .07679 m
.06349 .08054 L
s
.10317 .07679 m
.10317 .08054 L
s
.14286 .07679 m
.14286 .08054 L
s
.22222 .07679 m
.22222 .08054 L
s
.2619 .07679 m
.2619 .08054 L
s
.30159 .07679 m
.30159 .08054 L
s
.38095 .07679 m
.38095 .08054 L
s
.42063 .07679 m
.42063 .08054 L
s
.46032 .07679 m
.46032 .08054 L
s
.53968 .07679 m
.53968 .08054 L
s
.57937 .07679 m
.57937 .08054 L
s
.61905 .07679 m
.61905 .08054 L
s
.69841 .07679 m
.69841 .08054 L
s
.7381 .07679 m
.7381 .08054 L
s
.77778 .07679 m
.77778 .08054 L
s
.85714 .07679 m
.85714 .08054 L
s
.89683 .07679 m
.89683 .08054 L
s
.93651 .07679 m
.93651 .08054 L
s
.25 Mabswid
0 .07679 m
1 .07679 L
s
.18254 .18385 m
.18879 .18385 L
s
[(0.905)] .17004 .18385 1 0 Mshowa
.18254 .29091 m
.18879 .29091 L
s
[(0.91)] .17004 .29091 1 0 Mshowa
.18254 .39797 m
.18879 .39797 L
s
[(0.915)] .17004 .39797 1 0 Mshowa
.18254 .50504 m
.18879 .50504 L
s
[(0.92)] .17004 .50504 1 0 Mshowa
.18254 .6121 m
.18879 .6121 L
s
[(0.925)] .17004 .6121 1 0 Mshowa
.125 Mabswid
.18254 .0982 m
.18629 .0982 L
s
.18254 .11961 m
.18629 .11961 L
s
.18254 .14102 m
.18629 .14102 L
s
.18254 .16244 m
.18629 .16244 L
s
.18254 .20526 m
.18629 .20526 L
s
.18254 .22667 m
.18629 .22667 L
s
.18254 .24809 m
.18629 .24809 L
s
.18254 .2695 m
.18629 .2695 L
s
.18254 .31232 m
.18629 .31232 L
s
.18254 .33374 m
.18629 .33374 L
s
.18254 .35515 m
.18629 .35515 L
s
.18254 .37656 m
.18629 .37656 L
s
.18254 .41939 m
.18629 .41939 L
s
.18254 .4408 m
.18629 .4408 L
s
.18254 .46221 m
.18629 .46221 L
s
.18254 .48362 m
.18629 .48362 L
s
.18254 .52645 m
.18629 .52645 L
s
.18254 .54786 m
.18629 .54786 L
s
.18254 .56927 m
.18629 .56927 L
s
.18254 .59069 m
.18629 .59069 L
s
.18254 .05537 m
.18629 .05537 L
s
.18254 .03396 m
.18629 .03396 L
s
.18254 .01255 m
.18629 .01255 L
s
.25 Mabswid
.18254 0 m
.18254 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .29724 m
.02778 .31223 L
.03175 .32712 L
.03571 .34189 L
.03968 .35652 L
.04365 .37101 L
.04762 .38534 L
.05159 .39947 L
.05556 .4134 L
.05952 .42711 L
.06349 .44057 L
.06746 .45375 L
.07143 .46662 L
.0754 .47916 L
.07937 .49133 L
.08333 .50312 L
.0873 .51446 L
.09127 .52532 L
.09524 .53567 L
.09921 .54546 L
.10317 .55465 L
.10714 .56318 L
.11111 .57102 L
.11508 .57812 L
.11905 .58442 L
.12302 .58988 L
.12698 .59447 L
.13095 .59813 L
.13492 .60084 L
.13889 .60258 L
.14286 .60332 L
.14683 .60306 L
.15079 .60181 L
.15476 .59958 L
.15873 .59642 L
.1627 .59236 L
.16667 .58746 L
.17063 .58179 L
.1746 .57543 L
.17857 .56845 L
.18254 .56095 L
.18651 .55298 L
.19048 .54466 L
.19444 .53604 L
.19841 .5272 L
.20238 .5182 L
.20635 .50911 L
.21032 .49997 L
.21429 .49082 L
.21825 .48171 L
Mistroke
.22222 .47267 L
.22619 .46372 L
.23016 .45488 L
.23413 .44617 L
.2381 .43761 L
.24206 .4292 L
.24603 .42095 L
.25 .41287 L
.25397 .40496 L
.25794 .39722 L
.2619 .38966 L
.26587 .38226 L
.26984 .37505 L
.27381 .36799 L
.27778 .36111 L
.28175 .35439 L
.28571 .34783 L
.28968 .34142 L
.29365 .33517 L
.29762 .32907 L
.30159 .32312 L
.30556 .31731 L
.30952 .31163 L
.31349 .30609 L
.31746 .30068 L
.32143 .29539 L
.3254 .29023 L
.32937 .28518 L
.33333 .28026 L
.3373 .27544 L
.34127 .27073 L
.34524 .26613 L
.34921 .26163 L
.35317 .25723 L
.35714 .25292 L
.36111 .24871 L
.36508 .24459 L
.36905 .24055 L
.37302 .2366 L
.37698 .23274 L
.38095 .22895 L
.38492 .22524 L
.38889 .22161 L
.39286 .21805 L
.39683 .21456 L
.40079 .21114 L
.40476 .20779 L
.40873 .20451 L
.4127 .20128 L
.41667 .19812 L
Mistroke
.42063 .19502 L
.4246 .19198 L
.42857 .189 L
.43254 .18607 L
.43651 .1832 L
.44048 .18037 L
.44444 .1776 L
.44841 .17488 L
.45238 .17221 L
.45635 .16958 L
.46032 .16701 L
.46429 .16447 L
.46825 .16198 L
.47222 .15953 L
.47619 .15713 L
.48016 .15476 L
.48413 .15244 L
.4881 .15015 L
.49206 .1479 L
.49603 .14569 L
.5 .14352 L
.50397 .14138 L
.50794 .13927 L
.5119 .1372 L
.51587 .13516 L
.51984 .13315 L
.52381 .13118 L
.52778 .12923 L
.53175 .12732 L
.53571 .12543 L
.53968 .12357 L
.54365 .12174 L
.54762 .11994 L
.55159 .11817 L
.55556 .11642 L
.55952 .1147 L
.56349 .113 L
.56746 .11133 L
.57143 .10968 L
.5754 .10805 L
.57937 .10645 L
.58333 .10487 L
.5873 .10331 L
.59127 .10178 L
.59524 .10027 L
.59921 .09877 L
.60317 .0973 L
.60714 .09585 L
.61111 .09442 L
.61508 .093 L
Mistroke
.61905 .09161 L
.62302 .09023 L
.62698 .08888 L
.63095 .08754 L
.63492 .08622 L
.63889 .08491 L
.64286 .08362 L
.64683 .08235 L
.65079 .0811 L
.65476 .07986 L
.65873 .07864 L
.6627 .07743 L
.66667 .07624 L
.67063 .07506 L
.6746 .0739 L
.67857 .07275 L
.68254 .07162 L
.68651 .0705 L
.69048 .06939 L
.69444 .0683 L
.69841 .06722 L
.70238 .06615 L
.70635 .06509 L
.71032 .06405 L
.71429 .06302 L
.71825 .06201 L
.72222 .061 L
.72619 .06001 L
.73016 .05902 L
.73413 .05805 L
.7381 .05709 L
.74206 .05614 L
.74603 .05521 L
.75 .05428 L
.75397 .05336 L
.75794 .05246 L
.7619 .05156 L
.76587 .05067 L
.76984 .0498 L
.77381 .04893 L
.77778 .04807 L
.78175 .04722 L
.78571 .04638 L
.78968 .04555 L
.79365 .04473 L
.79762 .04392 L
.80159 .04312 L
.80556 .04232 L
.80952 .04154 L
.81349 .04076 L
Mistroke
.81746 .03999 L
.82143 .03922 L
.8254 .03847 L
.82937 .03772 L
.83333 .03699 L
.8373 .03625 L
.84127 .03553 L
.84524 .03481 L
.84921 .03411 L
.85317 .0334 L
.85714 .03271 L
.86111 .03202 L
.86508 .03134 L
.86905 .03067 L
.87302 .03 L
.87698 .02934 L
.88095 .02868 L
.88492 .02803 L
.88889 .02739 L
.89286 .02676 L
.89683 .02613 L
.90079 .02551 L
.90476 .02489 L
.90873 .02428 L
.9127 .02367 L
.91667 .02307 L
.92063 .02248 L
.9246 .02189 L
.92857 .02131 L
.93254 .02073 L
.93651 .02016 L
.94048 .01959 L
.94444 .01903 L
.94841 .01847 L
.95238 .01792 L
.95635 .01737 L
.96032 .01683 L
.96429 .0163 L
.96825 .01576 L
.97222 .01524 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O=ooo00<00?ooool0
i_oo000gool00`00ooooo`3Vool003Ooo`03003ooooo0>Koo`00=ooo0P00iooo000gool00`00oooo
o`3Gool60009ool003Ooo`03003ooooo0=3oo`L000ooo`00=ooo00<00?ooool0bOoo1`005_oo000g
ool00`00ooooo`32ool7000Mool003Ooo`03003ooooo0;goo`D002Coo`00=ooo0P00^?oo1P00:Ooo
000gool00`00ooooo`2bool5000_ool000?oo`8000Coo`03003ooooo00?oo`8002Koo`03003ooooo
027oo`@000?oo`03003ooooo00;oo`@001goo`@000?oo`03003ooooo00Coo`03003ooooo01coo`@0
00?oo`03003ooooo00?oo`8001koo`X000?oo`8002?oo`@000Soo`000_oo00@00?ooool000Soo`04
003ooooo000Uool00`00ooooo`0Sool00`00ooooo`07ool00`00ooooo`0Pool00`00ooooo`09ool0
0`00ooooo`0Nool00`00ooooo`07ool01000ooooo`007Ooo10002?oo00@00?ooool002;oo`03003o
oooo00Woo`000_oo00@00?ooool000Soo`04003ooooo000Uool00`00ooooo`0Sool00`00ooooo`08
ool00`00ooooo`0Oool00`00ooooo`06ool5000Oool00`00ooooo`07ool01000ooooo`006?oo1@00
0_oo00<00?ooool01ooo00@00?ooool002?oo`03003ooooo00Soo`000_oo00@00?ooool000Woo`80
02Koo`03003ooooo02?oo`03003ooooo00Woo`03003ooooo01koo`03003ooooo00Koo`04003ooooo
000Pool00`00ooooo`07ool3000Eool40007ool00`00ooooo`08ool2000Uool00`00ooooo`07ool0
00;oo`04003ooooo0008ool01000ooooo`009Ooo0P009?oo00<00?ooool02_oo00<00?ooool07Ooo
00<00?ooool01ooo00<00?oo00008?oo00<00?ooool01ooo00<00?ooool04?oo1@002ooo00<00?oo
ool01ooo00@00?ooool002Goo`03003ooooo00Koo`000_oo00@00?ooool000Soo`04003ooooo000U
ool00`00ooooo`0Qool30009ool01000ooooo`007Ooo0`002_oo0P007_oo0`002Ooo00<00?ooool0
3?oo10003_oo0`002Ooo00@00?ooool002;oo`04003ooooo0008ool000?oo`8000[oo`8002Koo`03
003ooooo02?oo`03003ooooo00Soo`80023oo`03003ooooo00Woo`03003ooooo01koo`03003ooooo
00Soo`<000Ooo`@001Coo`03003ooooo00Soo`8002Coo`8000Woo`00=ooo00<00?ooool0S_oo1000
E?oo000gool00`00ooooo`2;ool3001Hool003Ooo`03003ooooo08Ooo`@005_oo`000oooo`007000
0_oo000:ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00
ooooo`08ool00`00ooooo`06ool3000:ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07
ool000[oo`03003ooooo02[oo`03003ooooo02Woo`03003ooooo02[oo`03003ooooo02Coo`@000;o
o`03003ooooo02[oo`03003ooooo02Woo`03003ooooo00Ooo`00=ooo00<00?ooool0Nooo0P00JOoo
000gool00`00ooooo`1hool3001[ool003Ooo`03003ooooo07Goo`<006koo`00=ooo0P00Looo0`00
LOoo000gool00`00ooooo`1`ool2001dool003Ooo`03003ooooo06koo`8007Koo`00=ooo00<00?oo
ool0J_oo1000N?oo000gool00`00ooooo`1Xool2001lool003Ooo`03003ooooo06Koo`8007koo`00
=ooo0P00I?oo0`00P?oo000gool00`00ooooo`1Qool20023ool003Ooo`03003ooooo05ooo`8008Go
o`00=ooo00<00?ooool0G_oo00<00?ooool0QOoo000gool00`00ooooo`1Lool20028ool003Ooo`03
003ooooo05Woo`<008[oo`00=ooo0P00F?oo0P00SOoo000gool00`00ooooo`1Fool00`00ooooo`2=
ool003Ooo`03003ooooo05Coo`80093oo`00=ooo00<00?ooool0Dooo00<00?ooool0T?oo000gool0
0`00ooooo`1@ool3002Cool003Ooo`03003ooooo04ooo`03003ooooo09Coo`00=ooo0P00C_oo0P00
Uooo000gool00`00ooooo`1<ool00`00ooooo`2Gool003Ooo`03003ooooo04[oo`8009[oo`00=ooo
00<00?ooool0BOoo00<00?ooool0V_oo000Gool20004ool00`00ooooo`02ool30004ool20003ool3
0006ool00`00ooooo`17ool2002Mool001Koo`04003ooooo000;ool01000ooooo`000_oo00<00?oo
ool00ooo00<00?ooool00ooo00<00?ooool0AOoo0P00Wooo000Fool01000ooooo`002ooo00@00?oo
ool000;oo`03003ooooo00?oo`03003ooooo00?oo`8004Goo`03003ooooo09ooo`005_oo00@00?oo
ool000Woo`<000;oo`04003ooooo0002ool30006ool00`00ooooo`13ool00`00ooooo`2Pool001Ko
o`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Koo`03003ooooo
04;oo`03003ooooo0:7oo`005_oo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`000_oo
00<00?ooool01_oo00<00?ooool0@Ooo00<00?ooool0X_oo000Gool2000:ool20004ool20003ool4
0005ool00`00ooooo`0oool2002Uool003Ooo`03003ooooo03goo`800:Ooo`00=ooo0P00?Ooo00<0
0?ooool0Yooo000gool00`00ooooo`0kool00`00ooooo`2Xool003Ooo`03003ooooo03[oo`03003o
oooo0:Woo`00=ooo00<00?ooool0>Ooo00<00?ooool0Z_oo000gool00`00ooooo`0hool00`00oooo
o`2[ool003Ooo`03003ooooo03Ooo`03003ooooo0:coo`00=ooo0P00=ooo00<00?ooool0[Ooo000g
ool00`00ooooo`0dool2002`ool003Ooo`03003ooooo03?oo`03003ooooo0;3oo`00=ooo00<00?oo
ool0<_oo00<00?ooool0/Ooo000gool00`00ooooo`0aool00`00ooooo`2bool003Ooo`03003ooooo
033oo`03003ooooo0;?oo`00=ooo0P00<Ooo00<00?ooool0/ooo000gool00`00ooooo`0_ool00`00
ooooo`2dool003Ooo`03003ooooo02koo`03003ooooo0;Goo`00=ooo00<00?ooool0;Ooo00<00?oo
ool0]_oo000gool00`00ooooo`0/ool00`00ooooo`2gool003Ooo`03003ooooo02coo`03003ooooo
0;Ooo`00=ooo0P00:ooo0P00^_oo000gool00`00ooooo`0Yool00`00ooooo`2jool003Ooo`03003o
oooo02Soo`03003ooooo0;_oo`00=ooo00<00?ooool0:?oo00<00?ooool0^ooo000gool00`00oooo
o`0Wool00`00ooooo`2lool001goo`8000Coo`03003ooooo00;oo`<000?oo`@000Goo`03003ooooo
02Koo`03003ooooo0;goo`007?oo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo00Coo`03
003ooooo02Goo`03003ooooo0;koo`007?oo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo
00Coo`8002Koo`03003ooooo0;koo`002_oo00<00?ooool03ooo00@00?ooool000Woo`<000Coo`03
003ooooo00Coo`03003ooooo02Coo`03003ooooo0;ooo`002_oo00<00?ooool03ooo00@00?ooool0
00Soo`04003ooooo0004ool00`00ooooo`04ool00`00ooooo`0Sool00`00ooooo`30ool000[oo`03
003ooooo00ooo`04003ooooo0008ool01000ooooo`000_oo0`001_oo00<00?ooool08ooo00<00?oo
ool0`?oo000;ool00`00ooooo`0?ool2000:ool20005ool00`00ooooo`04ool00`00ooooo`0Qool2
0033ool000_oo`03003ooooo02Woo`03003ooooo023oo`03003ooooo0<?oo`002ooo00<00?ooool0
:Ooo0P008Ooo00<00?ooool0`ooo000;ool00`00ooooo`0Yool00`00ooooo`0Oool00`00ooooo`34
ool000_oo`03003ooooo02Woo`03003ooooo01ooo`03003ooooo0<Coo`003?oo00<00?ooool0:?oo
00<00?ooool07_oo00<00?ooool0aOoo000<ool00`00ooooo`0Xool00`00ooooo`0Mool00`00oooo
o`36ool000coo`03003ooooo02Soo`03003ooooo01goo`03003ooooo0<Koo`003?oo00<00?ooool0
:?oo0P007Ooo00<00?ooool0aooo000=ool00`00ooooo`0Wool00`00ooooo`0Lool00`00ooooo`37
ool000goo`03003ooooo02Ooo`03003ooooo01_oo`03003ooooo0<Soo`003Ooo00<00?ooool09ooo
00<00?ooool06ooo00<00?ooool0b?oo000=ool00`00ooooo`0Wool00`00ooooo`0Jool00`00oooo
o`39ool000koo`03003ooooo02Koo`03003ooooo01Woo`03003ooooo0<[oo`003_oo00<00?ooool0
9_oo0P006_oo00<00?ooool0b_oo000>ool00`00ooooo`0Vool00`00ooooo`0Hool00`00ooooo`3;
ool000ooo`03003ooooo02Goo`03003ooooo01Ooo`03003ooooo0<coo`003ooo00<00?ooool09Ooo
00<00?ooool05_oo00<00?ooool0cOoo000@ool00`00ooooo`0Tool00`00ooooo`0Fool00`00oooo
o`3=ool0013oo`03003ooooo02Coo`03003ooooo01Goo`03003ooooo0<koo`004?oo00<00?ooool0
9?oo0P005_oo00<00?ooool0c_oo000Aool00`00ooooo`0Sool00`00ooooo`0Dool00`00ooooo`3?
ool0017oo`03003ooooo02?oo`03003ooooo01Coo`03003ooooo0<ooo`004Ooo00<00?ooool08ooo
00<00?ooool04ooo00<00?ooool0d?oo000Aool00`00ooooo`03ool20004ool00`00ooooo`02ool3
0003ool40002ool30006ool00`00ooooo`0Cool00`00ooooo`3@ool001;oo`05003oooooool00002
ool00`00ooooo`09ool00`00ooooo`02ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`0B
ool00`00ooooo`3Aool001;oo`05003oooooool00002ool00`00ooooo`09ool00`00ooooo`02ool0
0`00ooooo`04ool00`00ooooo`03ool2000Cool00`00ooooo`3Aool001;oo`05003oooooool00002
ool00`00ooooo`07ool30004ool01000oooooooo0`001_oo00<00?ooool04Ooo00<00?ooool0d_oo
000Bool01@00oooooooo00000_oo00<00?ooool01_oo00@00?ooool000Coo`05003oooooool00008
ool00`00ooooo`0Aool00`00ooooo`3Bool001?oo`04003ooooo0002ool00`00ooooo`06ool01000
ooooo`000_oo0`000ooo00<00?ooool01_oo00<00?ooool04Ooo00<00?ooool0d_oo000Cool01000
oooooooo0P002_oo0P001Ooo00@00?ooooooo`@000Goo`03003ooooo013oo`03003ooooo0=?oo`00
4ooo00<00?ooool08Ooo00<00?ooool04?oo00<00?ooool0dooo000Cool00`00ooooo`0Qool2000@
ool00`00ooooo`3Dool001Coo`03003ooooo023oo`03003ooooo00koo`03003ooooo0=Goo`005?oo
00<00?ooool08?oo00<00?ooool03Ooo00<00?ooool0e_oo000Dool00`00ooooo`0Pool00`00oooo
o`0=ool00`00ooooo`3Fool001Coo`03003ooooo023oo`03003ooooo00coo`03003ooooo0=Ooo`00
5Ooo00<00?ooool07ooo00<00?ooool03?oo00<00?ooool0eooo000Eool00`00ooooo`0Oool2000=
ool00`00ooooo`3Gool001Goo`03003ooooo01ooo`03003ooooo00_oo`03003ooooo0=Soo`005_oo
00<00?ooool07_oo00<00?ooool02ooo00<00?ooool0f?oo000Fool00`00ooooo`0Nool00`00oooo
o`0:ool00`00ooooo`3Iool001Koo`03003ooooo01koo`03003ooooo00[oo`03003ooooo0=Woo`00
5_oo00<00?ooool07_oo00<00?ooool02_oo00<00?ooool0fOoo000Gool00`00ooooo`0Mool2000:
ool00`00ooooo`3Jool001Ooo`03003ooooo01goo`03003ooooo00Woo`03003ooooo0=[oo`005ooo
00<00?ooool07Ooo00<00?ooool02?oo00<00?ooool0fooo000Hool00`00ooooo`0Lool00`00oooo
o`08ool00`00ooooo`3Kool001Soo`03003ooooo01coo`03003ooooo00Soo`03003ooooo0=_oo`00
6Ooo00<00?ooool06ooo00<00?ooool01ooo00<00?ooool0g?oo000Iool00`00ooooo`0Kool20008
ool00`00ooooo`3Lool001[oo`03003ooooo01[oo`03003ooooo00Koo`03003ooooo0=goo`006_oo
00<00?ooool06_oo00<00?ooool01_oo00<00?ooool0gOoo000Jool00`00ooooo`0Jool00`00oooo
o`06ool00`00ooooo`3Mool001_oo`04003oo`000004ool00`00ooooo`02ool30003ool40005ool0
0`00ooooo`05ool00`00ooooo`3Nool001_oo`8000;oo`03003ooooo00Woo`04003ooooo0008ool0
0`00ooooo`04ool00`00ooooo`3Oool001_oo`8000;oo`03003ooooo00Woo`05003oooooool00007
ool20005ool00`00ooooo`3Oool001coo`04003ooooo0009ool30004ool00`00ooooo`04ool00`00
ooooo`03ool00`00ooooo`3Pool001coo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool0
0ooo00<00?ooool00ooo00<00?ooool0h?oo000Lool01000ooooo`002?oo00@00?ooool000;oo`04
003ooooo0005ool00`00ooooo`02ool00`00ooooo`3Qool001coo`<000[oo`8000Coo`8000Koo`03
003ooooo00;oo`03003ooooo0>7oo`007Ooo00<00?ooool05ooo00<00?ooool00_oo00<00?ooool0
hOoo000Mool00`00ooooo`0Gool20002ool00`00ooooo`3Rool001goo`03003ooooo01Ooo`05003o
ooooool0003Tool001koo`03003ooooo01Koo`04003ooooo003Uool001koo`03003ooooo01Koo`04
003ooooo003Uool001ooo`03003ooooo01Goo`04003ooooo003Uool001ooo`03003ooooo01Goo`03
003oo`000>Koo`007ooo00<00?ooool05Ooo0`00i_oo000Pool00`00ooooo`0Dool2003Wool0023o
o`03003ooooo01Coo`800>Ooo`008?oo00<00?ooool05?oo00<00?ooool0i_oo000Qool00`00oooo
o`0Cool00`00ooooo`3Vool002;oo`03003ooooo01;oo`03003ooooo0>Koo`008ooo00<00?ooool0
4?oo0`00iooo000Sool00`00ooooo`0?ool00`00ool0003Xool002Coo`03003ooooo00goo`04003o
oooo003Xool002Coo`03003ooooo00coo`05003oooooool0003Xool002Goo`03003ooooo00_oo`05
003oooooool0003Xool002Goo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo0>Koo`009_oo
00<00?ooool02?oo00<00?ooool00ooo0P00iooo000Wool00`00ooooo`07ool00`00ooooo`03ool0
0`00ooooo`3Vool002Soo`03003ooooo00Coo`8000Koo`03003ooooo0>Koo`00:Ooo0P000_oo0P00
2?oo00<00?ooool0i_oo000Gool20004ool00`00ooooo`02ool30003ool500001?oo000000001_oo
00<00?ooool0i_oo000Fool01000ooooo`002ooo00@00?ooool000Soo`03003ooooo00?oo`03003o
oooo0>Koo`005_oo00@00?ooool000_oo`05003oooooool00007ool00`00ooooo`03ool2003Wool0
01Koo`04003ooooo0009ool30004ool01000oooooooo0`001_oo00<00?ooool0i_oo000Fool01000
ooooo`002?oo00@00?ooool000Goo`04003ooooo0008ool00`00ooooo`3Vool001Koo`04003ooooo
0008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo0>ooo`005ooo0P002_oo0P001?oo
0P000ooo1000k_oo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.753047, 0.896416, \
0.00447266, 0.000165778}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %384]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -5.91211 6.84501 [
[.2619 .2359 -9 -9 ]
[.2619 .2359 9 0 ]
[.5 .2359 -3 -9 ]
[.5 .2359 3 0 ]
[.7381 .2359 -9 -9 ]
[.7381 .2359 9 0 ]
[.97619 .2359 -3 -9 ]
[.97619 .2359 3 0 ]
[.01131 .1115 -24 -4.5 ]
[.01131 .1115 0 4.5 ]
[.01131 .3853 -24 -4.5 ]
[.01131 .3853 0 4.5 ]
[.01131 .5222 -24 -4.5 ]
[.01131 .5222 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .2484 m
.2619 .25465 L
s
[(0.5)] .2619 .2359 0 1 Mshowa
.5 .2484 m
.5 .25465 L
s
[(1)] .5 .2359 0 1 Mshowa
.7381 .2484 m
.7381 .25465 L
s
[(1.5)] .7381 .2359 0 1 Mshowa
.97619 .2484 m
.97619 .25465 L
s
[(2)] .97619 .2359 0 1 Mshowa
.125 Mabswid
.07143 .2484 m
.07143 .25215 L
s
.11905 .2484 m
.11905 .25215 L
s
.16667 .2484 m
.16667 .25215 L
s
.21429 .2484 m
.21429 .25215 L
s
.30952 .2484 m
.30952 .25215 L
s
.35714 .2484 m
.35714 .25215 L
s
.40476 .2484 m
.40476 .25215 L
s
.45238 .2484 m
.45238 .25215 L
s
.54762 .2484 m
.54762 .25215 L
s
.59524 .2484 m
.59524 .25215 L
s
.64286 .2484 m
.64286 .25215 L
s
.69048 .2484 m
.69048 .25215 L
s
.78571 .2484 m
.78571 .25215 L
s
.83333 .2484 m
.83333 .25215 L
s
.88095 .2484 m
.88095 .25215 L
s
.92857 .2484 m
.92857 .25215 L
s
.25 Mabswid
0 .2484 m
1 .2484 L
s
.02381 .1115 m
.03006 .1115 L
s
[(0.88)] .01131 .1115 1 0 Mshowa
.02381 .3853 m
.03006 .3853 L
s
[(0.92)] .01131 .3853 1 0 Mshowa
.02381 .5222 m
.03006 .5222 L
s
[(0.94)] .01131 .5222 1 0 Mshowa
.125 Mabswid
.02381 .14572 m
.02756 .14572 L
s
.02381 .17995 m
.02756 .17995 L
s
.02381 .21417 m
.02756 .21417 L
s
.02381 .28262 m
.02756 .28262 L
s
.02381 .31685 m
.02756 .31685 L
s
.02381 .35107 m
.02756 .35107 L
s
.02381 .41952 m
.02756 .41952 L
s
.02381 .45375 m
.02756 .45375 L
s
.02381 .48797 m
.02756 .48797 L
s
.02381 .07727 m
.02756 .07727 L
s
.02381 .04305 m
.02756 .04305 L
s
.02381 .00882 m
.02756 .00882 L
s
.02381 .55642 m
.02756 .55642 L
s
.02381 .59065 m
.02756 .59065 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.40476 .31887 m
.40714 .32366 L
.40952 .32842 L
.4119 .33314 L
.41429 .33782 L
.41667 .34246 L
.41905 .34703 L
.42143 .35155 L
.42381 .35601 L
.42619 .36039 L
.42857 .36469 L
.43095 .3689 L
.43333 .37302 L
.43571 .37703 L
.4381 .38092 L
.44048 .38469 L
.44286 .38831 L
.44524 .39178 L
.44762 .39509 L
.45 .39822 L
.45238 .40116 L
.45476 .40389 L
.45714 .40639 L
.45952 .40866 L
.4619 .41068 L
.46429 .41242 L
.46667 .41389 L
.46905 .41506 L
.47143 .41593 L
.47381 .41648 L
.47619 .41672 L
.47857 .41663 L
.48095 .41623 L
.48333 .41552 L
.48571 .41451 L
.4881 .41321 L
.49048 .41165 L
.49286 .40984 L
.49524 .4078 L
.49762 .40557 L
.5 .40317 L
.50238 .40063 L
.50476 .39796 L
.50714 .39521 L
.50952 .39238 L
.5119 .38951 L
.51429 .3866 L
.51667 .38368 L
.51905 .38075 L
.52143 .37784 L
Mistroke
.52381 .37495 L
.52619 .37209 L
.52857 .36926 L
.53095 .36648 L
.53333 .36374 L
.53571 .36105 L
.5381 .35842 L
.54048 .35583 L
.54286 .35331 L
.54524 .35083 L
.54762 .34841 L
.55 .34605 L
.55238 .34374 L
.55476 .34149 L
.55714 .33929 L
.55952 .33714 L
.5619 .33504 L
.56429 .333 L
.56667 .331 L
.56905 .32905 L
.57143 .32714 L
.57381 .32529 L
.57619 .32347 L
.57857 .3217 L
.58095 .31997 L
.58333 .31828 L
.58571 .31663 L
.5881 .31502 L
.59048 .31344 L
.59286 .3119 L
.59524 .3104 L
.59762 .30893 L
.6 .30749 L
.60238 .30608 L
.60476 .3047 L
.60714 .30336 L
.60952 .30204 L
.6119 .30075 L
.61429 .29949 L
.61667 .29825 L
.61905 .29704 L
.62143 .29586 L
.62381 .29469 L
.62619 .29356 L
.62857 .29244 L
.63095 .29135 L
.63333 .29028 L
.63571 .28923 L
.6381 .2882 L
.64048 .28719 L
Mistroke
.64286 .2862 L
.64524 .28522 L
.64762 .28427 L
.65 .28333 L
.65238 .28241 L
.65476 .28151 L
.65714 .28063 L
.65952 .27976 L
.6619 .2789 L
.66429 .27806 L
.66667 .27724 L
.66905 .27643 L
.67143 .27563 L
.67381 .27485 L
.67619 .27408 L
.67857 .27333 L
.68095 .27258 L
.68333 .27185 L
.68571 .27113 L
.6881 .27043 L
.69048 .26973 L
.69286 .26905 L
.69524 .26837 L
.69762 .26771 L
.7 .26706 L
.70238 .26642 L
.70476 .26579 L
.70714 .26516 L
.70952 .26455 L
.7119 .26395 L
.71429 .26336 L
.71667 .26277 L
.71905 .26219 L
.72143 .26163 L
.72381 .26107 L
.72619 .26052 L
.72857 .25997 L
.73095 .25944 L
.73333 .25891 L
.73571 .25839 L
.7381 .25788 L
.74048 .25738 L
.74286 .25688 L
.74524 .25639 L
.74762 .2559 L
.75 .25543 L
.75238 .25496 L
.75476 .25449 L
.75714 .25403 L
.75952 .25358 L
Mistroke
.7619 .25314 L
.76429 .2527 L
.76667 .25226 L
.76905 .25184 L
.77143 .25141 L
.77381 .251 L
.77619 .25058 L
.77857 .25018 L
.78095 .24978 L
.78333 .24938 L
.78571 .24899 L
.7881 .2486 L
.79048 .24822 L
.79286 .24785 L
.79524 .24748 L
.79762 .24711 L
.8 .24675 L
.80238 .24639 L
.80476 .24603 L
.80714 .24568 L
.80952 .24534 L
.8119 .245 L
.81429 .24466 L
.81667 .24433 L
.81905 .244 L
.82143 .24367 L
.82381 .24335 L
.82619 .24303 L
.82857 .24272 L
.83095 .24241 L
.83333 .2421 L
.83571 .2418 L
.8381 .2415 L
.84048 .2412 L
.84286 .24091 L
.84524 .24062 L
.84762 .24033 L
.85 .24005 L
.85238 .23977 L
.85476 .23949 L
.85714 .23922 L
.85952 .23895 L
.8619 .23868 L
.86429 .23841 L
.86667 .23815 L
.86905 .23789 L
.87143 .23764 L
.87381 .23738 L
.87619 .23713 L
.87857 .23688 L
Mistroke
.88095 .23663 L
.88333 .23639 L
.88571 .23615 L
.8881 .23591 L
.89048 .23568 L
.89286 .23544 L
.89524 .23521 L
.89762 .23498 L
.9 .23475 L
.90238 .23453 L
.90476 .23431 L
.90714 .23409 L
.90952 .23387 L
.9119 .23366 L
.91429 .23344 L
.91667 .23323 L
.91905 .23302 L
.92143 .23281 L
.92381 .23261 L
.92619 .23241 L
.92857 .2322 L
.93095 .23201 L
.93333 .23181 L
.93571 .23161 L
.9381 .23142 L
.94048 .23123 L
.94286 .23104 L
.94524 .23085 L
.94762 .23066 L
.95 .23048 L
.95238 .2303 L
.95476 .23011 L
.95714 .22993 L
.95952 .22976 L
.9619 .22958 L
.96429 .22941 L
.96667 .22923 L
.96905 .22906 L
.97143 .22889 L
.97381 .22872 L
.97619 .22856 L
Mfstroke
.4351 0 m
.46172 .38259 L
s
.46172 .38259 m
.4781 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0J_oo00<00?ooool0
UOoo000Kool00`00ooooo`1Zool00`00ooooo`2Eool001_oo`8006_oo`03003ooooo09Goo`006ooo
00<00?ooool0J_oo00<00?ooool0UOoo000Kool00`00ooooo`1Zool00`00ooooo`2Eool001_oo`03
003ooooo06[oo`03003ooooo09Goo`006ooo00<00?ooool0J_oo00<00?ooool0UOoo000Kool00`00
ooooo`1Zool00`00ooooo`2Eool001_oo`03003ooooo06_oo`03003ooooo09Coo`006ooo00<00?oo
ool0Jooo00<00?ooool0U?oo000Kool00`00ooooo`1[ool00`00ooooo`2Dool001_oo`8006coo`03
003ooooo09Coo`006ooo00<00?ooool0Jooo00<00?ooool0U?oo000Kool00`00ooooo`1[ool00`00
ooooo`2Dool001_oo`03003ooooo06_oo`03003ooooo09Coo`006ooo00<00?ooool0Jooo00<00?oo
ool0U?oo000Kool00`00ooooo`1[ool00`00ooooo`2Dool001_oo`03003ooooo06_oo`03003ooooo
09Coo`006ooo00<00?ooool0Jooo00<00?ooool0U?oo000Kool00`00ooooo`1[ool00`00ooooo`2D
ool001_oo`8006coo`03003ooooo09Coo`006ooo00<00?ooool0Jooo00<00?ooool0U?oo000Kool0
0`00ooooo`1/ool00`00ooooo`2Cool001_oo`03003ooooo06coo`03003ooooo09?oo`006ooo00<0
0?ooool0K?oo00<00?ooool0Tooo000Kool00`00ooooo`1/ool00`00ooooo`2Cool001_oo`03003o
oooo06coo`03003ooooo09?oo`000_oo0P001?oo00<00?ooool00ooo0P001?oo0P001Ooo00<00?oo
ool0K?oo00<00?ooool0Tooo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0004
ool00`00ooooo`1/ool00`00ooooo`2Cool00005ool00?ooool00008ool01000ooooo`000_oo00@0
0?ooool000Coo`8006goo`03003ooooo09?oo`0000Goo`00ooooo`0000Woo`8000Coo`8000Goo`03
003ooooo06coo`03003ooooo09?oo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000oooo
o`001?oo00<00?ooool0K?oo00<00?ooool0Tooo00001Ooo003ooooo00002?oo00@00?ooool000;o
o`04003ooooo0004ool00`00ooooo`1/ool00`00ooooo`2Cool000;oo`8000[oo`8000Coo`8000Go
o`03003ooooo06coo`03003ooooo09?oo`006ooo00<00?ooool0K?oo00<00?ooool0Tooo000Kool0
0`00ooooo`1/ool00`00ooooo`2Cool001_oo`03003ooooo06coo`03003ooooo09?oo`006ooo00<0
0?ooool0KOoo00<00?ooool0T_oo000Kool00`00ooooo`1]ool00`00ooooo`2Bool001_oo`8006ko
o`03003ooooo09;oo`006ooo00<00?ooool0KOoo00<00?ooool0T_oo000Kool00`00ooooo`1]ool0
0`00ooooo`2Bool001_oo`03003ooooo06goo`03003ooooo09;oo`006ooo00<00?ooool0KOoo00<0
0?ooool0T_oo000Kool00`00ooooo`1]ool00`00ooooo`2Bool001_oo`03003ooooo06goo`03003o
oooo09;oo`006ooo00<00?ooool0KOoo00<00?ooool0T_oo000Kool00`00ooooo`1]ool00`00oooo
o`2Bool001_oo`8006koo`03003ooooo09;oo`006ooo00<00?ooool0KOoo00<00?ooool0T_oo000K
ool00`00ooooo`1]ool00`00ooooo`2Bool001_oo`03003ooooo06goo`03003ooooo09;oo`006ooo
00<00?ooool0K_oo00<00?ooool0TOoo000Kool00`00ooooo`1^ool00`00ooooo`2Aool001_oo`03
003ooooo06koo`03003ooooo097oo`006ooo00<00?ooool0K_oo00<00?ooool0TOoo000Kool00`00
ooooo`0eool20004ool00`00ooooo`02ool3000[ool00`00ooooo`08ool4000fool40003ool00`00
ooooo`02ool3000fool40006ool001_oo`8003Goo`04003ooooo000;ool00`00ooooo`0Xool00`00
ooooo`0:ool00`00ooooo`0gool00`00ooooo`0:ool00`00ooooo`0cool00`00ooooo`07ool001_o
o`03003ooooo03Coo`04003ooooo000;ool00`00ooooo`0Xool00`00ooooo`0:ool00`00ooooo`0g
ool00`00ooooo`0:ool00`00ooooo`0dool00`00ooooo`06ool001_oo`03003ooooo03Coo`04003o
oooo0008ool3000[ool00`00ooooo`0:ool00`00ooooo`0gool00`00ooooo`07ool3000hool00`00
ooooo`05ool001_oo`03003ooooo03Coo`04003ooooo0008ool00`00ooooo`0[ool00`00ooooo`0:
ool00`00ooooo`0gool00`00ooooo`07ool00`00ooooo`0iool00`00ooooo`04ool001_oo`03003o
oooo03Coo`04003ooooo0008ool00`00ooooo`0[ool00`00ooooo`08ool3000gool30009ool00`00
ooooo`0]ool=0006ool001_oo`03003ooooo03Goo`8000Woo`@002[oo`03003ooooo00[oo`03003o
oooo03Ooo`03003ooooo00Ooo`@0027oo`/000[oo`8000Ooo`006ooo00<00?ooool0K_oo00<00?oo
ool0JOoo2P007_oo000Kool00`00ooooo`1^ool00`00ooooo`1Qool9000Wool001_oo`03003ooooo
06koo`03003ooooo05[oo`P002ooo`005?ooo`003@00000Kool00`00ooooo`09ool00`00ooooo`0:
ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09
ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`02ool50002ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`05ool001_o
o`03003ooooo03coo`03003ooooo033oo`03003ooooo00Woo`03003ooooo03goo`D003[oo`03003o
oooo00Goo`006ooo00<00?ooool0Kooo00<00?ooool0A?oo1@00Aooo000Kool00`00ooooo`1_ool0
0`00ooooo`11ool4001;ool001_oo`03003ooooo06ooo`03003ooooo03goo`@004ooo`006ooo00<0
0?ooool0Kooo00<00?ooool0>_oo0`00Dooo000Kool00`00ooooo`1_ool00`00ooooo`0fool4001F
ool001_oo`03003ooooo06ooo`03003ooooo03?oo`@005Woo`006ooo0P00L?oo00<00?ooool0<Ooo
0`00G?oo000Kool00`00ooooo`1_ool00`00ooooo`0^ool3001Oool001_oo`03003ooooo06ooo`03
003ooooo02coo`<0067oo`006ooo00<00?ooool0Kooo00<00?ooool0:_oo0`00Hooo000Kool00`00
ooooo`1_ool00`00ooooo`0Xool3001Uool001_oo`03003ooooo06ooo`03003ooooo02Ooo`03003o
oooo06Koo`006ooo00<00?ooool0L?oo00<00?ooool09?oo0P00JOoo000Kool00`00ooooo`1`ool0
0`00ooooo`0Rool2001[ool001_oo`03003ooooo073oo`03003ooooo027oo`03003ooooo06_oo`00
6ooo0P00LOoo00<00?ooool07ooo0P00K_oo000Kool00`00ooooo`1Rool00`00ooooo`0;ool00`00
ooooo`0Mool2001`ool001_oo`03003ooooo06?oo`03003ooooo00[oo`03003ooooo01coo`80077o
o`006ooo00<00?ooool0Hooo00<00?ooool02_oo00<00?ooool06ooo00<00?ooool0LOoo000Kool0
0`00ooooo`1Tool00`00ooooo`09ool00`00ooooo`0Jool00`00ooooo`1bool001_oo`03003ooooo
06Coo`03003ooooo00Woo`03003ooooo01Woo`03003ooooo07?oo`006ooo00<00?ooool0IOoo00<0
0?ooool02?oo00<00?ooool06?oo00<00?ooool0M?oo000Kool00`00ooooo`1Uool00`00ooooo`08
ool00`00ooooo`0Fool2001gool001_oo`03003ooooo06Koo`03003ooooo00Ooo`03003ooooo01Ko
o`03003ooooo07Koo`006ooo0P00J?oo00<00?ooool01_oo00<00?ooool05Ooo00<00?ooool0Mooo
000Kool00`00ooooo`1Wool00`00ooooo`06ool00`00ooooo`0Cool2001jool001_oo`03003ooooo
06Ooo`03003ooooo00Ooo`03003ooooo01;oo`03003ooooo07Woo`006ooo00<00?ooool0J?oo00<0
0?ooool01_oo00<00?ooool04Ooo00<00?ooool0N_oo000Kool00`00ooooo`1Xool00`00ooooo`06
ool00`00ooooo`0@ool00`00ooooo`1kool001_oo`03003ooooo06Woo`03003ooooo00Goo`03003o
oooo00ooo`03003ooooo07coo`006ooo00<00?ooool0J_oo00<00?ooool01?oo00<00?ooool03_oo
00<00?ooool0OOoo0002ool20004ool00`00ooooo`02ool30003ool40004ool00`00ooooo`1Zool0
0`00ooooo`04ool00`00ooooo`0=ool2001oool00005ool00?ooool0000;ool01000ooooo`001ooo
00<00?ooool0Jooo00<00?ooool00ooo00<00?ooool03Ooo00<00?ooool0O_oo00001Ooo003ooooo
00002ooo00D00?ooooooo`0000Koo`8006goo`03003ooooo00;oo`03003ooooo00coo`03003ooooo
07ooo`0000Goo`00ooooo`0000Woo`<000Coo`03003ooooo00?oo`03003ooooo06coo`03003ooooo
00;oo`03003ooooo00_oo`03003ooooo083oo`0000Goo`00ooooo`0000Soo`04003ooooo0005ool0
0`00ooooo`02ool00`00ooooo`1]ool01@00oooooooo00003?oo00<00?ooool0POoo00001Ooo003o
oooo00002?oo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`1^ool01000ooooo`002ooo
00<00?ooool0P_oo0002ool2000:ool20004ool20005ool00`00ooooo`1^ool01000ooooo`002_oo
0P00Q?oo000Kool00`00ooooo`1_ool00`00ool00009ool00`00ooooo`24ool001_oo`03003ooooo
073oo`8000Soo`8008Koo`006ooo00<00?ooool0LOoo0P001_oo0P00Qooo000Kool00`00ooooo`1b
ool30002ool20029ool001_oo`8007?oo`03003ooooo008008_oo`006ooo00<00?ooool0L_oo00<0
0?ooool0SOoo000Kool00`00ooooo`1bool00`00ooooo`2=ool001_oo`03003ooooo07;oo`03003o
oooo08goo`006ooo00<00?ooool0L_oo00<00?ooool0SOoo000Kool00`00ooooo`1bool00`00oooo
o`2=ool001_oo`03003ooooo07;oo`03003ooooo08goo`006ooo00<00?ooool0L_oo00<00?ooool0
SOoo000Kool00`00ooooo`1bool00`00ooooo`2=ool001_oo`03003ooooo07;oo`03003ooooo08go
o`006ooo0P00M?oo00<00?ooool0S?oo000Kool00`00ooooo`1cool00`00ooooo`2<ool001_oo`03
003ooooo07?oo`03003ooooo08coo`006ooo00<00?ooool0Looo00<00?ooool0S?oo000Kool00`00
ooooo`1cool00`00ooooo`2<ool001_oo`03003ooooo07?oo`03003ooooo08coo`006ooo00<00?oo
ool0Looo00<00?ooool0S?oo000Kool00`00ooooo`1cool00`00ooooo`2<ool001_oo`03003ooooo
07?oo`03003ooooo08coo`006ooo0P00M?oo00<00?ooool0S?oo000Kool00`00ooooo`1cool00`00
ooooo`2<ool001_oo`03003ooooo07?oo`03003ooooo08coo`006ooo00<00?ooool0Looo00<00?oo
ool0S?oo000Kool00`00ooooo`1dool00`00ooooo`2;ool001_oo`03003ooooo07Coo`03003ooooo
08_oo`006ooo00<00?ooool0M?oo00<00?ooool0Rooo0002ool20004ool00`00ooooo`02ool30005
ool00`00ooooo`03ool00`00ooooo`1dool00`00ooooo`2;ool00005ool00?ooool0000;ool00`00
ooooo`02ool00`00ooooo`03ool00`00ooooo`1dool00`00ooooo`2;ool00005ool00?ooool0000;
ool00`00ool000040004ool2001eool00`00ooooo`2;ool00005ool00?ooool00009ool300001Ooo
003ooooo00001Ooo00<00?ooool0M?oo00<00?ooool0Rooo00001Ooo003ooooo00002?oo00@00?oo
ool000;oo`03003oo`0000Goo`03003ooooo07Coo`03003ooooo08_oo`0000Goo`00ooooo`0000So
o`04003ooooo0003ool20005ool00`00ooooo`1dool00`00ooooo`2;ool000;oo`8000[oo`8000Go
o`03003ooooo00?oo`03003ooooo07Coo`03003ooooo08_oo`006ooo00<00?ooool0M?oo00<00?oo
ool0Rooo000Kool00`00ooooo`1dool00`00ooooo`2;ool001_oo`03003ooooo07Goo`03003ooooo
08[oo`006ooo00<00?ooool0MOoo00<00?ooool0R_oo000Kool2001fool00`00ooooo`2:ool001_o
o`03003ooooo07Goo`03003ooooo08[oo`006ooo00<00?ooool0MOoo00<00?ooool0R_oo000Kool0
0`00ooooo`1eool00`00ooooo`2:ool001_oo`03003ooooo07Goo`03003ooooo08[oo`006ooo00<0
0?ooool0MOoo00<00?ooool0R_oo000Kool00`00ooooo`1eool00`00ooooo`2:ool001_oo`03003o
oooo07Goo`03003ooooo08[oo`006ooo00<00?ooool0MOoo00<00?ooool0R_oo000Kool2001fool0
0`00ooooo`2:ool001_oo`03003ooooo07Koo`03003ooooo08Woo`006ooo00<00?ooool0M_oo00<0
0?ooool0ROoo000Kool00`00ooooo`1fool00`00ooooo`29ool001_oo`03003ooooo07Koo`03003o
oooo08Woo`006ooo00<00?ooool0M_oo00<00?ooool0ROoo000Kool00`00ooooo`1fool00`00oooo
o`29ool001_oo`03003ooooo07Koo`03003ooooo08Woo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.215729, 0.860268, \
0.00789457, 0.000549205}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(Table[{k, Om /. strangesol[k, 10^2]}, {k, 0.8, 2, 
          0.005}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.611111 0.793651 -19.1945 21.4125 [
[.02381 .06429 -9 -9 ]
[.02381 .06429 9 0 ]
[.34127 .06429 -9 -9 ]
[.34127 .06429 9 0 ]
[.5 .06429 -9 -9 ]
[.5 .06429 9 0 ]
[.65873 .06429 -9 -9 ]
[.65873 .06429 9 0 ]
[.81746 .06429 -9 -9 ]
[.81746 .06429 9 0 ]
[.97619 .06429 -3 -9 ]
[.97619 .06429 3 0 ]
[.17004 .18385 -30 -4.5 ]
[.17004 .18385 0 4.5 ]
[.17004 .29091 -24 -4.5 ]
[.17004 .29091 0 4.5 ]
[.17004 .39797 -30 -4.5 ]
[.17004 .39797 0 4.5 ]
[.17004 .50504 -24 -4.5 ]
[.17004 .50504 0 4.5 ]
[.17004 .6121 -30 -4.5 ]
[.17004 .6121 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .07679 m
.02381 .08304 L
s
[(0.8)] .02381 .06429 0 1 Mshowa
.34127 .07679 m
.34127 .08304 L
s
[(1.2)] .34127 .06429 0 1 Mshowa
.5 .07679 m
.5 .08304 L
s
[(1.4)] .5 .06429 0 1 Mshowa
.65873 .07679 m
.65873 .08304 L
s
[(1.6)] .65873 .06429 0 1 Mshowa
.81746 .07679 m
.81746 .08304 L
s
[(1.8)] .81746 .06429 0 1 Mshowa
.97619 .07679 m
.97619 .08304 L
s
[(2)] .97619 .06429 0 1 Mshowa
.125 Mabswid
.06349 .07679 m
.06349 .08054 L
s
.10317 .07679 m
.10317 .08054 L
s
.14286 .07679 m
.14286 .08054 L
s
.22222 .07679 m
.22222 .08054 L
s
.2619 .07679 m
.2619 .08054 L
s
.30159 .07679 m
.30159 .08054 L
s
.38095 .07679 m
.38095 .08054 L
s
.42063 .07679 m
.42063 .08054 L
s
.46032 .07679 m
.46032 .08054 L
s
.53968 .07679 m
.53968 .08054 L
s
.57937 .07679 m
.57937 .08054 L
s
.61905 .07679 m
.61905 .08054 L
s
.69841 .07679 m
.69841 .08054 L
s
.7381 .07679 m
.7381 .08054 L
s
.77778 .07679 m
.77778 .08054 L
s
.85714 .07679 m
.85714 .08054 L
s
.89683 .07679 m
.89683 .08054 L
s
.93651 .07679 m
.93651 .08054 L
s
.25 Mabswid
0 .07679 m
1 .07679 L
s
.18254 .18385 m
.18879 .18385 L
s
[(0.905)] .17004 .18385 1 0 Mshowa
.18254 .29091 m
.18879 .29091 L
s
[(0.91)] .17004 .29091 1 0 Mshowa
.18254 .39797 m
.18879 .39797 L
s
[(0.915)] .17004 .39797 1 0 Mshowa
.18254 .50504 m
.18879 .50504 L
s
[(0.92)] .17004 .50504 1 0 Mshowa
.18254 .6121 m
.18879 .6121 L
s
[(0.925)] .17004 .6121 1 0 Mshowa
.125 Mabswid
.18254 .0982 m
.18629 .0982 L
s
.18254 .11961 m
.18629 .11961 L
s
.18254 .14102 m
.18629 .14102 L
s
.18254 .16244 m
.18629 .16244 L
s
.18254 .20526 m
.18629 .20526 L
s
.18254 .22667 m
.18629 .22667 L
s
.18254 .24809 m
.18629 .24809 L
s
.18254 .2695 m
.18629 .2695 L
s
.18254 .31232 m
.18629 .31232 L
s
.18254 .33374 m
.18629 .33374 L
s
.18254 .35515 m
.18629 .35515 L
s
.18254 .37656 m
.18629 .37656 L
s
.18254 .41939 m
.18629 .41939 L
s
.18254 .4408 m
.18629 .4408 L
s
.18254 .46221 m
.18629 .46221 L
s
.18254 .48362 m
.18629 .48362 L
s
.18254 .52645 m
.18629 .52645 L
s
.18254 .54786 m
.18629 .54786 L
s
.18254 .56927 m
.18629 .56927 L
s
.18254 .59069 m
.18629 .59069 L
s
.18254 .05537 m
.18629 .05537 L
s
.18254 .03396 m
.18629 .03396 L
s
.18254 .01255 m
.18629 .01255 L
s
.25 Mabswid
.18254 0 m
.18254 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .29724 m
.02778 .31223 L
.03175 .32712 L
.03571 .34189 L
.03968 .35652 L
.04365 .37101 L
.04762 .38534 L
.05159 .39947 L
.05556 .4134 L
.05952 .42711 L
.06349 .44057 L
.06746 .45375 L
.07143 .46662 L
.0754 .47916 L
.07937 .49133 L
.08333 .50312 L
.0873 .51446 L
.09127 .52532 L
.09524 .53567 L
.09921 .54546 L
.10317 .55465 L
.10714 .56318 L
.11111 .57102 L
.11508 .57812 L
.11905 .58442 L
.12302 .58988 L
.12698 .59447 L
.13095 .59813 L
.13492 .60084 L
.13889 .60258 L
.14286 .60332 L
.14683 .60306 L
.15079 .60181 L
.15476 .59958 L
.15873 .59642 L
.1627 .59236 L
.16667 .58746 L
.17063 .58179 L
.1746 .57543 L
.17857 .56845 L
.18254 .56095 L
.18651 .55298 L
.19048 .54466 L
.19444 .53604 L
.19841 .5272 L
.20238 .5182 L
.20635 .50911 L
.21032 .49997 L
.21429 .49082 L
.21825 .48171 L
Mistroke
.22222 .47267 L
.22619 .46372 L
.23016 .45488 L
.23413 .44617 L
.2381 .43761 L
.24206 .4292 L
.24603 .42095 L
.25 .41287 L
.25397 .40496 L
.25794 .39722 L
.2619 .38966 L
.26587 .38226 L
.26984 .37505 L
.27381 .36799 L
.27778 .36111 L
.28175 .35439 L
.28571 .34783 L
.28968 .34142 L
.29365 .33517 L
.29762 .32907 L
.30159 .32312 L
.30556 .31731 L
.30952 .31163 L
.31349 .30609 L
.31746 .30068 L
.32143 .29539 L
.3254 .29023 L
.32937 .28518 L
.33333 .28026 L
.3373 .27544 L
.34127 .27073 L
.34524 .26613 L
.34921 .26163 L
.35317 .25723 L
.35714 .25292 L
.36111 .24871 L
.36508 .24459 L
.36905 .24055 L
.37302 .2366 L
.37698 .23274 L
.38095 .22895 L
.38492 .22524 L
.38889 .22161 L
.39286 .21805 L
.39683 .21456 L
.40079 .21114 L
.40476 .20779 L
.40873 .20451 L
.4127 .20128 L
.41667 .19812 L
Mistroke
.42063 .19502 L
.4246 .19198 L
.42857 .189 L
.43254 .18607 L
.43651 .1832 L
.44048 .18037 L
.44444 .1776 L
.44841 .17488 L
.45238 .17221 L
.45635 .16958 L
.46032 .16701 L
.46429 .16447 L
.46825 .16198 L
.47222 .15953 L
.47619 .15713 L
.48016 .15476 L
.48413 .15244 L
.4881 .15015 L
.49206 .1479 L
.49603 .14569 L
.5 .14352 L
.50397 .14138 L
.50794 .13927 L
.5119 .1372 L
.51587 .13516 L
.51984 .13315 L
.52381 .13118 L
.52778 .12923 L
.53175 .12732 L
.53571 .12543 L
.53968 .12357 L
.54365 .12174 L
.54762 .11994 L
.55159 .11817 L
.55556 .11642 L
.55952 .1147 L
.56349 .113 L
.56746 .11133 L
.57143 .10968 L
.5754 .10805 L
.57937 .10645 L
.58333 .10487 L
.5873 .10331 L
.59127 .10178 L
.59524 .10027 L
.59921 .09877 L
.60317 .0973 L
.60714 .09585 L
.61111 .09442 L
.61508 .093 L
Mistroke
.61905 .09161 L
.62302 .09023 L
.62698 .08888 L
.63095 .08754 L
.63492 .08622 L
.63889 .08491 L
.64286 .08362 L
.64683 .08235 L
.65079 .0811 L
.65476 .07986 L
.65873 .07864 L
.6627 .07743 L
.66667 .07624 L
.67063 .07506 L
.6746 .0739 L
.67857 .07275 L
.68254 .07162 L
.68651 .0705 L
.69048 .06939 L
.69444 .0683 L
.69841 .06722 L
.70238 .06615 L
.70635 .06509 L
.71032 .06405 L
.71429 .06302 L
.71825 .06201 L
.72222 .061 L
.72619 .06001 L
.73016 .05902 L
.73413 .05805 L
.7381 .05709 L
.74206 .05614 L
.74603 .05521 L
.75 .05428 L
.75397 .05336 L
.75794 .05246 L
.7619 .05156 L
.76587 .05067 L
.76984 .0498 L
.77381 .04893 L
.77778 .04807 L
.78175 .04722 L
.78571 .04638 L
.78968 .04555 L
.79365 .04473 L
.79762 .04392 L
.80159 .04312 L
.80556 .04232 L
.80952 .04154 L
.81349 .04076 L
Mistroke
.81746 .03999 L
.82143 .03922 L
.8254 .03847 L
.82937 .03772 L
.83333 .03699 L
.8373 .03625 L
.84127 .03553 L
.84524 .03481 L
.84921 .03411 L
.85317 .0334 L
.85714 .03271 L
.86111 .03202 L
.86508 .03134 L
.86905 .03067 L
.87302 .03 L
.87698 .02934 L
.88095 .02868 L
.88492 .02803 L
.88889 .02739 L
.89286 .02676 L
.89683 .02613 L
.90079 .02551 L
.90476 .02489 L
.90873 .02428 L
.9127 .02367 L
.91667 .02307 L
.92063 .02248 L
.9246 .02189 L
.92857 .02131 L
.93254 .02073 L
.93651 .02016 L
.94048 .01959 L
.94444 .01903 L
.94841 .01847 L
.95238 .01792 L
.95635 .01737 L
.96032 .01683 L
.96429 .0163 L
.96825 .01576 L
.97222 .01524 L
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O=ooo00<00?ooool0
i_oo000gool00`00ooooo`3Vool003Ooo`03003ooooo0>Koo`00=ooo0P00iooo000gool00`00oooo
o`3Gool60009ool003Ooo`03003ooooo0=3oo`L000ooo`00=ooo00<00?ooool0bOoo1`005_oo000g
ool00`00ooooo`32ool7000Mool003Ooo`03003ooooo0;goo`D002Coo`00=ooo0P00^?oo1P00:Ooo
000gool00`00ooooo`2bool5000_ool000?oo`8000Coo`03003ooooo00?oo`8002Koo`03003ooooo
027oo`@000?oo`03003ooooo00;oo`@001goo`@000?oo`03003ooooo00Coo`03003ooooo01coo`@0
00?oo`03003ooooo00?oo`8001koo`X000?oo`8002?oo`@000Soo`000_oo00@00?ooool000Soo`04
003ooooo000Uool00`00ooooo`0Sool00`00ooooo`07ool00`00ooooo`0Pool00`00ooooo`09ool0
0`00ooooo`0Nool00`00ooooo`07ool01000ooooo`007Ooo10002?oo00@00?ooool002;oo`03003o
oooo00Woo`000_oo00@00?ooool000Soo`04003ooooo000Uool00`00ooooo`0Sool00`00ooooo`08
ool00`00ooooo`0Oool00`00ooooo`06ool5000Oool00`00ooooo`07ool01000ooooo`006?oo1@00
0_oo00<00?ooool01ooo00@00?ooool002?oo`03003ooooo00Soo`000_oo00@00?ooool000Woo`80
02Koo`03003ooooo02?oo`03003ooooo00Woo`03003ooooo01koo`03003ooooo00Koo`04003ooooo
000Pool00`00ooooo`07ool3000Eool40007ool00`00ooooo`08ool2000Uool00`00ooooo`07ool0
00;oo`04003ooooo0008ool01000ooooo`009Ooo0P009?oo00<00?ooool02_oo00<00?ooool07Ooo
00<00?ooool01ooo00<00?oo00008?oo00<00?ooool01ooo00<00?ooool04?oo1@002ooo00<00?oo
ool01ooo00@00?ooool002Goo`03003ooooo00Koo`000_oo00@00?ooool000Soo`04003ooooo000U
ool00`00ooooo`0Qool30009ool01000ooooo`007Ooo0`002_oo0P007_oo0`002Ooo00<00?ooool0
3?oo10003_oo0`002Ooo00@00?ooool002;oo`04003ooooo0008ool000?oo`8000[oo`8002Koo`03
003ooooo02?oo`03003ooooo00Soo`80023oo`03003ooooo00Woo`03003ooooo01koo`03003ooooo
00Soo`<000Ooo`@001Coo`03003ooooo00Soo`8002Coo`8000Woo`00=ooo00<00?ooool0S_oo1000
E?oo000gool00`00ooooo`2;ool3001Hool003Ooo`03003ooooo08Ooo`@005_oo`000oooo`007000
0_oo000:ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00
ooooo`08ool00`00ooooo`06ool3000:ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`07
ool000[oo`03003ooooo02[oo`03003ooooo02Woo`03003ooooo02[oo`03003ooooo02Coo`@000;o
o`03003ooooo02[oo`03003ooooo02Woo`03003ooooo00Ooo`00=ooo00<00?ooool0Nooo0P00JOoo
000gool00`00ooooo`1hool3001[ool003Ooo`03003ooooo07Goo`<006koo`00=ooo0P00Looo0`00
LOoo000gool00`00ooooo`1`ool2001dool003Ooo`03003ooooo06koo`8007Koo`00=ooo00<00?oo
ool0J_oo1000N?oo000gool00`00ooooo`1Xool2001lool003Ooo`03003ooooo06Koo`8007koo`00
=ooo0P00I?oo0`00P?oo000gool00`00ooooo`1Qool20023ool003Ooo`03003ooooo05ooo`8008Go
o`00=ooo00<00?ooool0G_oo00<00?ooool0QOoo000gool00`00ooooo`1Lool20028ool003Ooo`03
003ooooo05Woo`<008[oo`00=ooo0P00F?oo0P00SOoo000gool00`00ooooo`1Fool00`00ooooo`2=
ool003Ooo`03003ooooo05Coo`80093oo`00=ooo00<00?ooool0Dooo00<00?ooool0T?oo000gool0
0`00ooooo`1@ool3002Cool003Ooo`03003ooooo04ooo`03003ooooo09Coo`00=ooo0P00C_oo0P00
Uooo000gool00`00ooooo`1<ool00`00ooooo`2Gool003Ooo`03003ooooo04[oo`8009[oo`00=ooo
00<00?ooool0BOoo00<00?ooool0V_oo000Gool20004ool00`00ooooo`02ool30004ool20003ool3
0006ool00`00ooooo`17ool2002Mool001Koo`04003ooooo000;ool01000ooooo`000_oo00<00?oo
ool00ooo00<00?ooool00ooo00<00?ooool0AOoo0P00Wooo000Fool01000ooooo`002ooo00@00?oo
ool000;oo`03003ooooo00?oo`03003ooooo00?oo`8004Goo`03003ooooo09ooo`005_oo00@00?oo
ool000Woo`<000;oo`04003ooooo0002ool30006ool00`00ooooo`13ool00`00ooooo`2Pool001Ko
o`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Koo`03003ooooo
04;oo`03003ooooo0:7oo`005_oo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`000_oo
00<00?ooool01_oo00<00?ooool0@Ooo00<00?ooool0X_oo000Gool2000:ool20004ool20003ool4
0005ool00`00ooooo`0oool2002Uool003Ooo`03003ooooo03goo`800:Ooo`00=ooo0P00?Ooo00<0
0?ooool0Yooo000gool00`00ooooo`0kool00`00ooooo`2Xool003Ooo`03003ooooo03[oo`03003o
oooo0:Woo`00=ooo00<00?ooool0>Ooo00<00?ooool0Z_oo000gool00`00ooooo`0hool00`00oooo
o`2[ool003Ooo`03003ooooo03Ooo`03003ooooo0:coo`00=ooo0P00=ooo00<00?ooool0[Ooo000g
ool00`00ooooo`0dool2002`ool003Ooo`03003ooooo03?oo`03003ooooo0;3oo`00=ooo00<00?oo
ool0<_oo00<00?ooool0/Ooo000gool00`00ooooo`0aool00`00ooooo`2bool003Ooo`03003ooooo
033oo`03003ooooo0;?oo`00=ooo0P00<Ooo00<00?ooool0/ooo000gool00`00ooooo`0_ool00`00
ooooo`2dool003Ooo`03003ooooo02koo`03003ooooo0;Goo`00=ooo00<00?ooool0;Ooo00<00?oo
ool0]_oo000gool00`00ooooo`0/ool00`00ooooo`2gool003Ooo`03003ooooo02coo`03003ooooo
0;Ooo`00=ooo0P00:ooo0P00^_oo000gool00`00ooooo`0Yool00`00ooooo`2jool003Ooo`03003o
oooo02Soo`03003ooooo0;_oo`00=ooo00<00?ooool0:?oo00<00?ooool0^ooo000gool00`00oooo
o`0Wool00`00ooooo`2lool001goo`8000Coo`03003ooooo00;oo`<000?oo`@000Goo`03003ooooo
02Koo`03003ooooo0;goo`007?oo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo00Coo`03
003ooooo02Goo`03003ooooo0;koo`007?oo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo
00Coo`8002Koo`03003ooooo0;koo`002_oo00<00?ooool03ooo00@00?ooool000Woo`<000Coo`03
003ooooo00Coo`03003ooooo02Coo`03003ooooo0;ooo`002_oo00<00?ooool03ooo00@00?ooool0
00Soo`04003ooooo0004ool00`00ooooo`04ool00`00ooooo`0Sool00`00ooooo`30ool000[oo`03
003ooooo00ooo`04003ooooo0008ool01000ooooo`000_oo0`001_oo00<00?ooool08ooo00<00?oo
ool0`?oo000;ool00`00ooooo`0?ool2000:ool20005ool00`00ooooo`04ool00`00ooooo`0Qool2
0033ool000_oo`03003ooooo02Woo`03003ooooo023oo`03003ooooo0<?oo`002ooo00<00?ooool0
:Ooo0P008Ooo00<00?ooool0`ooo000;ool00`00ooooo`0Yool00`00ooooo`0Oool00`00ooooo`34
ool000_oo`03003ooooo02Woo`03003ooooo01ooo`03003ooooo0<Coo`003?oo00<00?ooool0:?oo
00<00?ooool07_oo00<00?ooool0aOoo000<ool00`00ooooo`0Xool00`00ooooo`0Mool00`00oooo
o`36ool000coo`03003ooooo02Soo`03003ooooo01goo`03003ooooo0<Koo`003?oo00<00?ooool0
:?oo0P007Ooo00<00?ooool0aooo000=ool00`00ooooo`0Wool00`00ooooo`0Lool00`00ooooo`37
ool000goo`03003ooooo02Ooo`03003ooooo01_oo`03003ooooo0<Soo`003Ooo00<00?ooool09ooo
00<00?ooool06ooo00<00?ooool0b?oo000=ool00`00ooooo`0Wool00`00ooooo`0Jool00`00oooo
o`39ool000koo`03003ooooo02Koo`03003ooooo01Woo`03003ooooo0<[oo`003_oo00<00?ooool0
9_oo0P006_oo00<00?ooool0b_oo000>ool00`00ooooo`0Vool00`00ooooo`0Hool00`00ooooo`3;
ool000ooo`03003ooooo02Goo`03003ooooo01Ooo`03003ooooo0<coo`003ooo00<00?ooool09Ooo
00<00?ooool05_oo00<00?ooool0cOoo000@ool00`00ooooo`0Tool00`00ooooo`0Fool00`00oooo
o`3=ool0013oo`03003ooooo02Coo`03003ooooo01Goo`03003ooooo0<koo`004?oo00<00?ooool0
9?oo0P005_oo00<00?ooool0c_oo000Aool00`00ooooo`0Sool00`00ooooo`0Dool00`00ooooo`3?
ool0017oo`03003ooooo02?oo`03003ooooo01Coo`03003ooooo0<ooo`004Ooo00<00?ooool08ooo
00<00?ooool04ooo00<00?ooool0d?oo000Aool00`00ooooo`03ool20004ool00`00ooooo`02ool3
0003ool40002ool30006ool00`00ooooo`0Cool00`00ooooo`3@ool001;oo`05003oooooool00002
ool00`00ooooo`09ool00`00ooooo`02ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`0B
ool00`00ooooo`3Aool001;oo`05003oooooool00002ool00`00ooooo`09ool00`00ooooo`02ool0
0`00ooooo`04ool00`00ooooo`03ool2000Cool00`00ooooo`3Aool001;oo`05003oooooool00002
ool00`00ooooo`07ool30004ool01000oooooooo0`001_oo00<00?ooool04Ooo00<00?ooool0d_oo
000Bool01@00oooooooo00000_oo00<00?ooool01_oo00@00?ooool000Coo`05003oooooool00008
ool00`00ooooo`0Aool00`00ooooo`3Bool001?oo`04003ooooo0002ool00`00ooooo`06ool01000
ooooo`000_oo0`000ooo00<00?ooool01_oo00<00?ooool04Ooo00<00?ooool0d_oo000Cool01000
oooooooo0P002_oo0P001Ooo00@00?ooooooo`@000Goo`03003ooooo013oo`03003ooooo0=?oo`00
4ooo00<00?ooool08Ooo00<00?ooool04?oo00<00?ooool0dooo000Cool00`00ooooo`0Qool2000@
ool00`00ooooo`3Dool001Coo`03003ooooo023oo`03003ooooo00koo`03003ooooo0=Goo`005?oo
00<00?ooool08?oo00<00?ooool03Ooo00<00?ooool0e_oo000Dool00`00ooooo`0Pool00`00oooo
o`0=ool00`00ooooo`3Fool001Coo`03003ooooo023oo`03003ooooo00coo`03003ooooo0=Ooo`00
5Ooo00<00?ooool07ooo00<00?ooool03?oo00<00?ooool0eooo000Eool00`00ooooo`0Oool2000=
ool00`00ooooo`3Gool001Goo`03003ooooo01ooo`03003ooooo00_oo`03003ooooo0=Soo`005_oo
00<00?ooool07_oo00<00?ooool02ooo00<00?ooool0f?oo000Fool00`00ooooo`0Nool00`00oooo
o`0:ool00`00ooooo`3Iool001Koo`03003ooooo01koo`03003ooooo00[oo`03003ooooo0=Woo`00
5_oo00<00?ooool07_oo00<00?ooool02_oo00<00?ooool0fOoo000Gool00`00ooooo`0Mool2000:
ool00`00ooooo`3Jool001Ooo`03003ooooo01goo`03003ooooo00Woo`03003ooooo0=[oo`005ooo
00<00?ooool07Ooo00<00?ooool02?oo00<00?ooool0fooo000Hool00`00ooooo`0Lool00`00oooo
o`08ool00`00ooooo`3Kool001Soo`03003ooooo01coo`03003ooooo00Soo`03003ooooo0=_oo`00
6Ooo00<00?ooool06ooo00<00?ooool01ooo00<00?ooool0g?oo000Iool00`00ooooo`0Kool20008
ool00`00ooooo`3Lool001[oo`03003ooooo01[oo`03003ooooo00Koo`03003ooooo0=goo`006_oo
00<00?ooool06_oo00<00?ooool01_oo00<00?ooool0gOoo000Jool00`00ooooo`0Jool00`00oooo
o`06ool00`00ooooo`3Mool001_oo`04003oo`000004ool00`00ooooo`02ool30003ool40005ool0
0`00ooooo`05ool00`00ooooo`3Nool001_oo`8000;oo`03003ooooo00Woo`04003ooooo0008ool0
0`00ooooo`04ool00`00ooooo`3Oool001_oo`8000;oo`03003ooooo00Woo`05003oooooool00007
ool20005ool00`00ooooo`3Oool001coo`04003ooooo0009ool30004ool00`00ooooo`04ool00`00
ooooo`03ool00`00ooooo`3Pool001coo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool0
0ooo00<00?ooool00ooo00<00?ooool0h?oo000Lool01000ooooo`002?oo00@00?ooool000;oo`04
003ooooo0005ool00`00ooooo`02ool00`00ooooo`3Qool001coo`<000[oo`8000Coo`8000Koo`03
003ooooo00;oo`03003ooooo0>7oo`007Ooo00<00?ooool05ooo00<00?ooool00_oo00<00?ooool0
hOoo000Mool00`00ooooo`0Gool20002ool00`00ooooo`3Rool001goo`03003ooooo01Ooo`05003o
ooooool0003Tool001koo`03003ooooo01Koo`04003ooooo003Uool001koo`03003ooooo01Koo`04
003ooooo003Uool001ooo`03003ooooo01Goo`04003ooooo003Uool001ooo`03003ooooo01Goo`03
003oo`000>Koo`007ooo00<00?ooool05Ooo0`00i_oo000Pool00`00ooooo`0Dool2003Wool0023o
o`03003ooooo01Coo`800>Ooo`008?oo00<00?ooool05?oo00<00?ooool0i_oo000Qool00`00oooo
o`0Cool00`00ooooo`3Vool002;oo`03003ooooo01;oo`03003ooooo0>Koo`008ooo00<00?ooool0
4?oo0`00iooo000Sool00`00ooooo`0?ool00`00ool0003Xool002Coo`03003ooooo00goo`04003o
oooo003Xool002Coo`03003ooooo00coo`05003oooooool0003Xool002Goo`03003ooooo00_oo`05
003oooooool0003Xool002Goo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo0>Koo`009_oo
00<00?ooool02?oo00<00?ooool00ooo0P00iooo000Wool00`00ooooo`07ool00`00ooooo`03ool0
0`00ooooo`3Vool002Soo`03003ooooo00Coo`8000Koo`03003ooooo0>Koo`00:Ooo0P000_oo0P00
2?oo00<00?ooool0i_oo000Gool20004ool00`00ooooo`02ool30003ool500001?oo000000001_oo
00<00?ooool0i_oo000Fool01000ooooo`002ooo00@00?ooool000Soo`03003ooooo00?oo`03003o
oooo0>Koo`005_oo00@00?ooool000_oo`05003oooooool00007ool00`00ooooo`03ool2003Wool0
01Koo`04003ooooo0009ool30004ool01000oooooooo0`001_oo00<00?ooool0i_oo000Fool01000
ooooo`002?oo00@00?ooool000Goo`04003ooooo0008ool00`00ooooo`3Vool001Koo`04003ooooo
0008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo0>ooo`005ooo0P002_oo0P001?oo
0P000ooo1000k_oo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.753047, 0.896416, \
0.00447266, 0.000165778}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, Om /. strangesol[k, 10]}, {k, 0.8, 1.2, 0.001}]\)], "Input"],

Cell[BoxData[
    \({{0.8`, 0.9818786433914661`}, {0.801`, 0.9820299717709965`}, {0.802`, 
        0.9821807280740572`}, {0.803`, 0.9823309110481149`}, {0.804`, 
        0.9824805194090974`}, {0.805`, 0.9826295518358431`}, {0.806`, 
        0.9827780069632821`}, {0.807`, 0.9829258833740154`}, {0.808`, 
        0.9830731795878564`}, {0.809`, 0.983220155206452`}, {0.81`, 
        0.9833663178402776`}, {0.811`, 0.983511899884341`}, {0.812`, 
        0.9836569002731944`}, {0.8130000000000001`, 
        0.9838013178710853`}, {0.8140000000000001`, 
        0.9839451515915666`}, {0.8150000000000001`, 
        0.9840884003349244`}, {0.8160000000000001`, 
        0.9842310630008059`}, {0.8170000000000001`, 
        0.9843731384887281`}, {0.8180000000000001`, 
        0.9845146256987753`}, {0.8190000000000001`, 
        0.9846555235325689`}, {0.8200000000000001`, 
        0.9847958308946521`}, {0.8210000000000001`, 
        0.984935546694495`}, {0.8220000000000001`, 
        0.9850745556347766`}, {0.8230000000000001`, 
        0.985213069340985`}, {0.8240000000000001`, 
        0.9853509856582131`}, {0.8250000000000001`, 
        0.985488303013869`}, {0.8260000000000001`, 
        0.9856252093959572`}, {0.8270000000000001`, 
        0.9857613516188808`}, {0.8280000000000001`, 
        0.985896894381978`}, {0.8290000000000001`, 
        0.9860318365646814`}, {0.8300000000000001`, 
        0.9861661770907454`}, {0.8310000000000001`, 
        0.986299914788993`}, {0.8320000000000001`, 
        0.9864330485599633`}, {0.8330000000000001`, 
        0.9865655772887078`}, {0.8340000000000001`, 
        0.9866974998634481`}, {0.8350000000000001`, 
        0.9868288151774932`}, {0.8360000000000001`, 
        0.9869595221322787`}, {0.8370000000000001`, 
        0.9870896196423167`}, {0.8380000000000001`, 
        0.9872191066434954`}, {0.8390000000000001`, 
        0.9873479830390867`}, {0.8400000000000001`, 
        0.9874762457360013`}, {0.8410000000000001`, 
        0.987603894703849`}, {0.8420000000000001`, 
        0.987730921871015`}, {0.8430000000000001`, 
        0.9878573381725327`}, {0.8440000000000001`, 
        0.9879831370303042`}, {0.8450000000000001`, 
        0.9881083174642286`}, {0.8460000000000001`, 
        0.9882328783004394`}, {0.8470000000000001`, 
        0.9883568184473187`}, {0.8480000000000001`, 
        0.9884801368337196`}, {0.8490000000000001`, 
        0.9886028324276728`}, {0.8500000000000001`, 
        0.9887249042768207`}, {0.8510000000000001`, 
        0.9888463505605162`}, {0.8520000000000001`, 
        0.988967171377699`}, {0.8530000000000001`, 
        0.9890873655716045`}, {0.8540000000000001`, 
        0.9892069240440609`}, {0.8550000000000001`, 
        0.9893258582567355`}, {0.8560000000000001`, 
        0.9894441618973144`}, {0.8570000000000001`, 
        0.9895618340926965`}, {0.8580000000000001`, 
        0.9896788726526259`}, {0.8590000000000001`, 
        0.9897952788309621`}, {0.8600000000000001`, 
        0.9899110521422081`}, {0.8610000000000001`, 
        0.990026180133849`}, {0.8620000000000001`, 
        0.9901406761907118`}, {0.8630000000000001`, 
        0.9902545364415415`}, {0.8640000000000001`, 
        0.9903677533417344`}, {0.865`, 0.9904803321428639`}, {0.866`, 
        0.9905922684976829`}, {0.867`, 0.9907035631935086`}, {0.868`, 
        0.9908142137331554`}, {0.869`, 0.9909242191932002`}, {0.87`, 
        0.9910336448255169`}, {0.871`, 0.9911422928905222`}, {0.872`, 
        0.9912503578234398`}, {0.873`, 0.9913578824982551`}, {0.874`, 
        0.9914645434085644`}, {0.875`, 0.9915706581786018`}, {0.876`, 
        0.9916761216053593`}, {0.877`, 0.9917809323483787`}, {0.878`, 
        0.9918850892765064`}, {0.879`, 0.9919887644901155`}, {0.88`, 
        0.9920915533548706`}, {0.881`, 0.9921936993251113`}, {0.882`, 
        0.992295161506622`}, {0.883`, 0.9923960333014668`}, {0.884`, 
        0.9924962454969595`}, {0.885`, 0.9925957970515995`}, {0.886`, 
        0.9926946869636386`}, {0.887`, 0.9927929142573847`}, {0.888`, 
        0.9928904779758818`}, {0.889`, 0.9929873771768803`}, {0.89`, 
        0.9930836109305643`}, {0.891`, 0.9931791783182637`}, {0.892`, 
        0.9932740784317312`}, {0.893`, 0.9933683103727559`}, {0.894`, 
        0.9934618732529822`}, {0.895`, 0.9935547661938572`}, {0.896`, 
        0.9936471462967107`}, {0.897`, 0.9937386743077117`}, {0.898`, 
        0.9938295319419971`}, {0.899`, 0.9939197180672711`}, {0.9`, 
        0.9940091587043244`}, {0.901`, 0.9940980134875446`}, {0.902`, 
        0.9941861925549953`}, {0.903`, 0.9942736950951104`}, {0.904`, 
        0.9943605203107183`}, {0.905`, 0.9944466674185111`}, {0.906`, 
        0.9945321356487052`}, {0.907`, 0.9946169242448406`}, {0.908`, 
        0.9947010324636836`}, {0.909`, 0.9947844595752035`}, {0.91`, 
        0.9948672048626089`}, {0.911`, 0.9949492676224274`}, {0.912`, 
        0.9950306471646169`}, {0.913`, 0.9951113428127037`}, {0.914`, 
        0.9951913539039413`}, {0.915`, 0.9952706797894838`}, {0.916`, 
        0.9953493198345712`}, {0.917`, 0.995427273418728`}, {0.918`, 
        0.9955045399359652`}, {0.919`, 0.9955811187949938`}, {0.92`, 
        0.9956570094194379`}, {0.921`, 0.9957322112480593`}, {0.922`, 
        0.9958067237349791`}, {0.923`, 0.9958805463499064`}, {0.924`, 
        0.9959536785783679`}, {0.925`, 0.9960261199219412`}, {0.926`, 
        0.9960978698984881`}, {0.927`, 0.9961689280423899`}, {0.928`, 
        0.996239293904786`}, {0.929`, 0.9963089670538102`}, {0.93`, 
        0.9963779470748306`}, {0.931`, 0.9964462335706888`}, {0.932`, 
        0.9965138249381644`}, {0.933`, 0.9965807230449295`}, {0.934`, 
        0.996646926528543`}, {0.935`, 0.9967124350631458`}, {0.936`, 
        0.9967772483415214`}, {0.937`, 
        0.9968413660753328`}, {0.9380000000000001`, 
        0.9969047879953564`}, {0.9390000000000001`, 
        0.9969675138517197`}, {0.9400000000000001`, 
        0.9970295434141351`}, {0.9410000000000001`, 
        0.9970908764721366`}, {0.9420000000000001`, 
        0.9971515198118512`}, {0.9430000000000001`, 
        0.9972114601324306`}, {0.9440000000000001`, 
        0.9972707034790095`}, {0.9450000000000001`, 
        0.9973292497247748`}, {0.9460000000000001`, 
        0.9973870987639891`}, {0.9470000000000001`, 
        0.9974442505122273`}, {0.9480000000000001`, 
        0.9975007049066129`}, {0.9490000000000001`, 
        0.997556461906051`}, {0.9500000000000001`, 
        0.997611521491463`}, {0.9510000000000001`, 
        0.9976658836660184`}, {0.9520000000000001`, 
        0.9977195484553638`}, {0.9530000000000001`, 
        0.9977725159078541`}, {0.9540000000000001`, 
        0.9978247860947782`}, {0.9550000000000001`, 
        0.9978763591105865`}, {0.9560000000000001`, 
        0.9979272350731131`}, {0.9570000000000001`, 
        0.9979774141238008`}, {0.9580000000000001`, 
        0.9980268964279185`}, {0.9590000000000001`, 
        0.9980756821747822`}, {0.9600000000000001`, 
        0.998123771577969`}, {0.9610000000000001`, 
        0.9981711648755331`}, {0.9620000000000001`, 
        0.998217862330217`}, {0.9630000000000001`, 
        0.9982638642296602`}, {0.9640000000000001`, 
        0.9983091708866069`}, {0.9650000000000001`, 
        0.99835378263911`}, {0.9660000000000001`, 
        0.9983976998507338`}, {0.9670000000000001`, 
        0.998440922910751`}, {0.9680000000000001`, 
        0.9984834522343402`}, {0.9690000000000001`, 
        0.998525288262779`}, {0.9700000000000001`, 
        0.9985664314636333`}, {0.9710000000000001`, 
        0.998606882330945`}, {0.9720000000000001`, 
        0.9986466413854144`}, {0.9730000000000001`, 
        0.998685709174582`}, {0.9740000000000001`, 
        0.9987240862730042`}, {0.9750000000000001`, 
        0.998761773282427`}, {0.9760000000000001`, 
        0.9987987708319553`}, {0.9770000000000001`, 
        0.9988350795782197`}, {0.9780000000000001`, 
        0.9988707002055369`}, {0.9790000000000001`, 
        0.998905633426069`}, {0.9800000000000001`, 
        0.9989398799799774`}, {0.9810000000000001`, 
        0.9989734406355725`}, {0.9820000000000001`, 
        0.999006316189459`}, {0.9830000000000001`, 
        0.9990385074666781`}, {0.9840000000000001`, 
        0.9990700153208434`}, {0.9850000000000001`, 
        0.999100840634274`}, {0.9860000000000001`, 
        0.9991309843181223`}, {0.9870000000000001`, 
        0.9991604473124965`}, {0.9880000000000001`, 
        0.9991892305865792`}, {0.9890000000000001`, 
        0.9992173351387409`}, {0.9900000000000001`, 
        0.9992447619966485`}, {0.9910000000000001`, 
        0.9992715122173677`}, {0.992`, 0.9992975868874624`}, {0.993`, 
        0.9993229871230871`}, {0.994`, 0.999347714070074`}, {0.995`, 
        0.999371768904016`}, {0.996`, 0.9993951528303432`}, {0.997`, 
        0.9994178670843931`}, {0.998`, 0.9994399129314774`}, {0.999`, 
        0.9994612916669405`}, {1.`, 
        0.9994820046162144`}, {1.0010000000000001`, 
        0.999502053134866`}, {1.002`, 
        0.9995214386086392`}, {1.0030000000000001`, 
        0.9995401624534911`}, {1.004`, 
        0.9995582261156221`}, {1.0050000000000001`, 
        0.9995756310714989`}, {1.006`, 
        0.9995923788278732`}, {1.0070000000000001`, 
        0.9996084709217913`}, {1.008`, 
        0.9996239089206005`}, {1.0090000000000001`, 
        0.9996386944219465`}, {1.01`, 
        0.9996528290537664`}, {1.0110000000000001`, 
        0.9996663144742726`}, {1.012`, 
        0.999679152371933`}, {1.0130000000000001`, 
        0.9996913444654434`}, {1.014`, 
        0.9997028925036918`}, {1.0150000000000001`, 
        0.9997137982657188`}, {1.016`, 
        0.9997240635606688`}, {1.0170000000000001`, 
        0.9997336902277354`}, {1.018`, 
        0.9997426801361009`}, {1.0190000000000001`, 
        0.9997510351848671`}, {1.02`, 
        0.9997587573029805`}, {1.0210000000000001`, 
        0.9997658484491501`}, {1.022`, 
        0.9997723106117593`}, {1.0230000000000001`, 
        0.9997781458087687`}, {1.024`, 
        0.9997833560876148`}, {1.0250000000000001`, 
        0.9997879435250987`}, {1.026`, 
        0.9997919102272707`}, {1.0270000000000001`, 
        0.9997952583293059`}, {1.028`, 
        0.999797989995374`}, {1.0290000000000001`, 
        0.9998001074185008`}, {1.03`, 
        0.9998016128204253`}, {1.0310000000000001`, 
        0.999802508451446`}, {1.032`, 
        0.9998027965902645`}, {1.0330000000000001`, 
        0.9998024795438191`}, {1.034`, 
        0.9998015596471123`}, {1.0350000000000001`, 
        0.9998000392630325`}, {1.036`, 
        0.999797920782167`}, {1.0370000000000001`, 
        0.9997952066226105`}, {1.038`, 
        0.999791899229764`}, {1.0390000000000001`, 
        0.9997880010761296`}, {1.04`, 
        0.9997835146610976`}, {1.0410000000000001`, 
        0.9997784425107255`}, {1.042`, 
        0.999772787177513`}, {1.0430000000000001`, 
        0.9997665512401687`}, {1.044`, 
        0.999759737303371`}, {1.0450000000000002`, 
        0.9997523479975214`}, {1.046`, 
        0.999744385978494`}, {1.0470000000000002`, 
        0.9997358539273759`}, {1.048`, 
        0.9997267545502027`}, {1.0490000000000002`, 
        0.999717090577688`}, {1.05`, 
        0.9997068647649459`}, {1.0510000000000002`, 
        0.9996960798912088`}, {1.052`, 
        0.9996847387595386`}, {1.0530000000000002`, 
        0.9996728441965314`}, {1.054`, 
        0.9996603990520172`}, {1.0550000000000002`, 
        0.9996474061987548`}, {1.056`, 0.9996338685321196`}, {1.057`, 
        0.9996195661798943`}, {1.058`, 0.9996049440414702`}, {1.059`, 
        0.9995897858846967`}, {1.06`, 0.9995740946923614`}, {1.061`, 
        0.9995578734682061`}, {1.062`, 0.999541125236583`}, {1.063`, 
        0.999523853042105`}, {1.064`, 0.9995060599492903`}, {1.065`, 
        0.9994877490422039`}, {1.066`, 0.9994689234240941`}, {1.067`, 
        0.999449586217024`}, {1.068`, 0.9994297405614988`}, {1.069`, 
        0.99940938961609`}, {1.07`, 0.9993885365570555`}, {1.071`, 
        0.9993671845779528`}, {1.072`, 0.9993453368892543`}, {1.073`, 
        0.9993229967179525`}, {1.074`, 0.999300167307167`}, {1.075`, 
        0.9992768519157444`}, {1.076`, 0.9992530538178586`}, {1.077`, 
        0.9992287763026034`}, {1.078`, 0.9992040226735881`}, {1.079`, 
        0.9991787962485245`}, {1.08`, 0.9991531003588152`}, {1.081`, 
        0.999126938342653`}, {1.082`, 0.9991003135699233`}, {1.083`, 
        0.9990732294047244`}, {1.084`, 0.9990456892290472`}, {1.085`, 
        0.999017696436475`}, {1.086`, 0.9989892544317578`}, {1.087`, 
        0.9989603666303867`}, {1.088`, 0.9989310364581637`}, {1.089`, 
        0.998901267350774`}, {1.09`, 0.9988710627533549`}, {1.091`, 
        0.9988404261200633`}, {1.092`, 0.9988093609136452`}, {1.093`, 
        0.9987778706050016`}, {1.094`, 0.9987459586727547`}, {1.095`, 
        0.998713628602816`}, {1.096`, 0.998680883887951`}, {1.097`, 
        0.9986477280273454`}, {1.098`, 0.9986141645261718`}, {1.099`, 
        0.9985801968951572`}, {1.1`, 0.9985458286501477`}, {1.101`, 
        0.9985110633116784`}, {1.102`, 0.9984759044045403`}, {1.103`, 
        0.9984403554573497`}, {1.104`, 0.9984044200021196`}, {1.105`, 
        0.9983681015738289`}, {1.106`, 0.9983314037099976`}, {1.107`, 
        0.9982943299502586`}, {1.108`, 0.9982568838359349`}, {1.109`, 
        0.9982190689096165`}, {1.11`, 0.99818088871474`}, {1.111`, 
        0.9981423467951693`}, {1.112`, 0.9981034466947791`}, {1.113`, 
        0.9980641919570411`}, {1.114`, 0.9980245861246106`}, {1.115`, 
        0.9979846327389192`}, {1.116`, 0.9979443353397656`}, {1.117`, 
        0.9979036974649125`}, {1.118`, 0.9978627226496853`}, {1.119`, 
        0.9978214144265733`}, {1.12`, 0.9977797763248346`}, {1.121`, 
        0.9977378118701044`}, {1.122`, 0.9976955245840047`}, {1.123`, 
        0.9976529179837611`}, {1.124`, 0.9976099955818194`}, {1.125`, 
        0.9975667608640354`}, {1.1260000000000001`, 
        0.9975232173746004`}, {1.127`, 
        0.9974793685884223`}, {1.1280000000000001`, 
        0.9974352179951101`}, {1.129`, 
        0.9973907690776407`}, {1.1300000000000001`, 
        0.9973460253119993`}, {1.131`, 
        0.9973009901668254`}, {1.1320000000000001`, 
        0.997255667103061`}, {1.133`, 
        0.9972100595736051`}, {1.1340000000000001`, 
        0.997164171022971`}, {1.135`, 
        0.9971180048869478`}, {1.1360000000000001`, 
        0.9970715645922681`}, {1.137`, 
        0.997024853556277`}, {1.1380000000000001`, 
        0.9969778751866095`}, {1.139`, 
        0.9969306328808681`}, {1.1400000000000001`, 
        0.9968831300263072`}, {1.141`, 
        0.9968353699995222`}, {1.1420000000000001`, 
        0.9967873561661408`}, {1.143`, 
        0.9967390918805223`}, {1.1440000000000001`, 
        0.9966905804854586`}, {1.145`, 
        0.9966418252742217`}, {1.1460000000000001`, 
        0.9965928296481827`}, {1.147`, 
        0.9965435968688463`}, {1.1480000000000001`, 
        0.9964941302302366`}, {1.149`, 
        0.9964444330133495`}, {1.1500000000000001`, 
        0.9963945084858523`}, {1.151`, 
        0.9963443599017717`}, {1.1520000000000001`, 
        0.996293990501169`}, {1.153`, 
        0.9962434034570287`}, {1.1540000000000001`, 
        0.9961926021133568`}, {1.155`, 
        0.9961415895841604`}, {1.1560000000000001`, 
        0.9960903690538543`}, {1.157`, 
        0.9960389436885747`}, {1.1580000000000001`, 
        0.9959873166334765`}, {1.159`, 
        0.995935491073415`}, {1.1600000000000001`, 
        0.9958834700853532`}, {1.161`, 
        0.9958312567910351`}, {1.1620000000000001`, 
        0.9957788564743728`}, {1.163`, 
        0.9957264166794183`}, {1.1640000000000001`, 
        0.9956734826782748`}, {1.165`, 
        0.9956205329609805`}, {1.1660000000000001`, 
        0.995567349691745`}, {1.167`, 
        0.9955141164836558`}, {1.1680000000000001`, 
        0.9954606209226438`}, {1.169`, 
        0.9954070515282871`}, {1.1700000000000002`, 
        0.9953531878936903`}, {1.171`, 
        0.9952992147792288`}, {1.1720000000000002`, 
        0.9952449731508428`}, {1.173`, 
        0.9951906999116442`}, {1.1740000000000002`, 
        0.995136372262437`}, {1.175`, 
        0.9950817840303103`}, {1.1760000000000002`, 
        0.9950270444577015`}, {1.177`, 
        0.9949723195470095`}, {1.1780000000000002`, 
        0.9949171380723131`}, {1.179`, 
        0.9948619765395851`}, {1.1800000000000002`, 
        0.994806676586262`}, {1.181`, 
        0.9947512411897543`}, {1.1820000000000002`, 
        0.994695678748627`}, {1.183`, 
        0.9946400226757884`}, {1.1840000000000002`, 
        0.994584173702099`}, {1.185`, 0.9945282329244481`}, {1.186`, 
        0.9944721060167337`}, {1.187`, 0.9944159961190517`}, {1.188`, 
        0.9943597040197378`}, {1.189`, 0.9943032927757899`}, {1.19`, 
        0.9942467760570758`}, {1.191`, 0.994190214182377`}, {1.192`, 
        0.9941334234299102`}, {1.193`, 0.9940766384433544`}, {1.194`, 
        0.9940196642668693`}, {1.195`, 0.9939626374709188`}, {1.196`, 
        0.9939055521767011`}, {1.197`, 0.9938482741815441`}, {1.198`, 
        0.9937909770838153`}, {1.199`, 0.9937336068923163`}, {1.2`, 
        0.9936761323279398`}}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9994820046162144`, 
      mGamma \[Rule] \(-0.3707184166384133`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 2.38095 -32.2288 32.8386 [
[.02381 .59729 -9 -9 ]
[.02381 .59729 9 0 ]
[.2619 .59729 -9 -9 ]
[.2619 .59729 9 0 ]
[.7381 .59729 -9 -9 ]
[.7381 .59729 9 0 ]
[.97619 .59729 -9 -9 ]
[.97619 .59729 9 0 ]
[.4875 .03512 -36 -4.5 ]
[.4875 .03512 0 4.5 ]
[.4875 .11722 -30 -4.5 ]
[.4875 .11722 0 4.5 ]
[.4875 .19931 -36 -4.5 ]
[.4875 .19931 0 4.5 ]
[.4875 .28141 -24 -4.5 ]
[.4875 .28141 0 4.5 ]
[.4875 .36351 -36 -4.5 ]
[.4875 .36351 0 4.5 ]
[.4875 .4456 -30 -4.5 ]
[.4875 .4456 0 4.5 ]
[.4875 .5277 -36 -4.5 ]
[.4875 .5277 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .60979 m
.02381 .61604 L
s
[(0.8)] .02381 .59729 0 1 Mshowa
.2619 .60979 m
.2619 .61604 L
s
[(0.9)] .2619 .59729 0 1 Mshowa
.7381 .60979 m
.7381 .61604 L
s
[(1.1)] .7381 .59729 0 1 Mshowa
.97619 .60979 m
.97619 .61604 L
s
[(1.2)] .97619 .59729 0 1 Mshowa
.125 Mabswid
.07143 .60979 m
.07143 .61354 L
s
.11905 .60979 m
.11905 .61354 L
s
.16667 .60979 m
.16667 .61354 L
s
.21429 .60979 m
.21429 .61354 L
s
.30952 .60979 m
.30952 .61354 L
s
.35714 .60979 m
.35714 .61354 L
s
.40476 .60979 m
.40476 .61354 L
s
.45238 .60979 m
.45238 .61354 L
s
.54762 .60979 m
.54762 .61354 L
s
.59524 .60979 m
.59524 .61354 L
s
.64286 .60979 m
.64286 .61354 L
s
.69048 .60979 m
.69048 .61354 L
s
.78571 .60979 m
.78571 .61354 L
s
.83333 .60979 m
.83333 .61354 L
s
.88095 .60979 m
.88095 .61354 L
s
.92857 .60979 m
.92857 .61354 L
s
.25 Mabswid
0 .60979 m
1 .60979 L
s
.5 .03512 m
.50625 .03512 L
s
[(0.9825)] .4875 .03512 1 0 Mshowa
.5 .11722 m
.50625 .11722 L
s
[(0.985)] .4875 .11722 1 0 Mshowa
.5 .19931 m
.50625 .19931 L
s
[(0.9875)] .4875 .19931 1 0 Mshowa
.5 .28141 m
.50625 .28141 L
s
[(0.99)] .4875 .28141 1 0 Mshowa
.5 .36351 m
.50625 .36351 L
s
[(0.9925)] .4875 .36351 1 0 Mshowa
.5 .4456 m
.50625 .4456 L
s
[(0.995)] .4875 .4456 1 0 Mshowa
.5 .5277 m
.50625 .5277 L
s
[(0.9975)] .4875 .5277 1 0 Mshowa
.125 Mabswid
.5 .05154 m
.50375 .05154 L
s
.5 .06796 m
.50375 .06796 L
s
.5 .08438 m
.50375 .08438 L
s
.5 .1008 m
.50375 .1008 L
s
.5 .13364 m
.50375 .13364 L
s
.5 .15005 m
.50375 .15005 L
s
.5 .16647 m
.50375 .16647 L
s
.5 .18289 m
.50375 .18289 L
s
.5 .21573 m
.50375 .21573 L
s
.5 .23215 m
.50375 .23215 L
s
.5 .24857 m
.50375 .24857 L
s
.5 .26499 m
.50375 .26499 L
s
.5 .29783 m
.50375 .29783 L
s
.5 .31425 m
.50375 .31425 L
s
.5 .33067 m
.50375 .33067 L
s
.5 .34709 m
.50375 .34709 L
s
.5 .37992 m
.50375 .37992 L
s
.5 .39634 m
.50375 .39634 L
s
.5 .41276 m
.50375 .41276 L
s
.5 .42918 m
.50375 .42918 L
s
.5 .46202 m
.50375 .46202 L
s
.5 .47844 m
.50375 .47844 L
s
.5 .49486 m
.50375 .49486 L
s
.5 .51128 m
.50375 .51128 L
s
.5 .54412 m
.50375 .54412 L
s
.5 .56054 m
.50375 .56054 L
s
.5 .57696 m
.50375 .57696 L
s
.5 .59338 m
.50375 .59338 L
s
.5 .0187 m
.50375 .0187 L
s
.5 .00228 m
.50375 .00228 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02619 .01968 L
.02857 .02464 L
.03095 .02957 L
.03333 .03448 L
.03571 .03937 L
.0381 .04425 L
.04048 .0491 L
.04286 .05394 L
.04524 .05877 L
.04762 .06357 L
.05 .06835 L
.05238 .07311 L
.05476 .07785 L
.05714 .08258 L
.05952 .08728 L
.0619 .09197 L
.06429 .09663 L
.06667 .10128 L
.06905 .1059 L
.07143 .11051 L
.07381 .1151 L
.07619 .11966 L
.07857 .12421 L
.08095 .12874 L
.08333 .13325 L
.08571 .13775 L
.0881 .14222 L
.09048 .14667 L
.09286 .1511 L
.09524 .15551 L
.09762 .1599 L
.1 .16428 L
.10238 .16863 L
.10476 .17296 L
.10714 .17727 L
.10952 .18156 L
.1119 .18584 L
.11429 .19009 L
.11667 .19432 L
.11905 .19853 L
.12143 .20272 L
.12381 .2069 L
.12619 .21105 L
.12857 .21518 L
.13095 .21929 L
.13333 .22338 L
.13571 .22745 L
.1381 .2315 L
.14048 .23553 L
Mistroke
.14286 .23954 L
.14524 .24352 L
.14762 .24749 L
.15 .25144 L
.15238 .25537 L
.15476 .25927 L
.15714 .26316 L
.15952 .26702 L
.1619 .27086 L
.16429 .27469 L
.16667 .27849 L
.16905 .28227 L
.17143 .28603 L
.17381 .28977 L
.17619 .29349 L
.17857 .29718 L
.18095 .30086 L
.18333 .30451 L
.18571 .30815 L
.1881 .31176 L
.19048 .31535 L
.19286 .31892 L
.19524 .32247 L
.19762 .326 L
.2 .3295 L
.20238 .33299 L
.20476 .33645 L
.20714 .33989 L
.20952 .34331 L
.2119 .34672 L
.21429 .35009 L
.21667 .35345 L
.21905 .35678 L
.22143 .36009 L
.22381 .36338 L
.22619 .36665 L
.22857 .3699 L
.23095 .37312 L
.23333 .37633 L
.23571 .37951 L
.2381 .38267 L
.24048 .38581 L
.24286 .38893 L
.24524 .39202 L
.24762 .39509 L
.25 .39814 L
.25238 .40118 L
.25476 .40418 L
.25714 .40717 L
.25952 .41013 L
Mistroke
.2619 .41306 L
.26429 .41598 L
.26667 .41888 L
.26905 .42175 L
.27143 .4246 L
.27381 .42743 L
.27619 .43024 L
.27857 .43302 L
.28095 .43578 L
.28333 .43852 L
.28571 .44124 L
.2881 .44394 L
.29048 .44661 L
.29286 .44926 L
.29524 .45189 L
.29762 .45449 L
.3 .45707 L
.30238 .45963 L
.30476 .46217 L
.30714 .46468 L
.30952 .46718 L
.3119 .46965 L
.31429 .47209 L
.31667 .47452 L
.31905 .47692 L
.32143 .4793 L
.32381 .48165 L
.32619 .48399 L
.32857 .4863 L
.33095 .48859 L
.33333 .49085 L
.33571 .49309 L
.3381 .49531 L
.34048 .49751 L
.34286 .49968 L
.34524 .50184 L
.34762 .50396 L
.35 .50607 L
.35238 .50815 L
.35476 .51021 L
.35714 .51225 L
.35952 .51426 L
.3619 .51625 L
.36429 .51822 L
.36667 .52017 L
.36905 .52209 L
.37143 .52399 L
.37381 .52587 L
.37619 .52772 L
.37857 .52955 L
Mistroke
.38095 .53136 L
.38333 .53315 L
.38571 .53491 L
.3881 .53665 L
.39048 .53836 L
.39286 .54006 L
.39524 .54173 L
.39762 .54338 L
.4 .545 L
.40238 .5466 L
.40476 .54818 L
.40714 .54974 L
.40952 .55127 L
.4119 .55278 L
.41429 .55427 L
.41667 .55574 L
.41905 .55718 L
.42143 .5586 L
.42381 .55999 L
.42619 .56137 L
.42857 .56272 L
.43095 .56405 L
.43333 .56535 L
.43571 .56664 L
.4381 .5679 L
.44048 .56913 L
.44286 .57035 L
.44524 .57154 L
.44762 .57271 L
.45 .57386 L
.45238 .57498 L
.45476 .57608 L
.45714 .57716 L
.45952 .57822 L
.4619 .57926 L
.46429 .58027 L
.46667 .58126 L
.46905 .58223 L
.47143 .58317 L
.47381 .58409 L
.47619 .58499 L
.47857 .58587 L
.48095 .58673 L
.48333 .58756 L
.48571 .58837 L
.4881 .58916 L
.49048 .58993 L
.49286 .59068 L
.49524 .5914 L
.49762 .5921 L
Mistroke
.5 .59278 L
.50238 .59344 L
.50476 .59408 L
.50714 .59469 L
.50952 .59529 L
.5119 .59586 L
.51429 .59641 L
.51667 .59694 L
.51905 .59744 L
.52143 .59793 L
.52381 .59839 L
.52619 .59884 L
.52857 .59926 L
.53095 .59966 L
.53333 .60004 L
.53571 .6004 L
.5381 .60073 L
.54048 .60105 L
.54286 .60134 L
.54524 .60162 L
.54762 .60187 L
.55 .60211 L
.55238 .60232 L
.55476 .60251 L
.55714 .60268 L
.55952 .60283 L
.5619 .60296 L
.56429 .60307 L
.56667 .60316 L
.56905 .60323 L
.57143 .60328 L
.57381 .60331 L
.57619 .60332 L
.57857 .60331 L
.58095 .60328 L
.58333 .60323 L
.58571 .60316 L
.5881 .60307 L
.59048 .60296 L
.59286 .60283 L
.59524 .60269 L
.59762 .60252 L
.6 .60233 L
.60238 .60213 L
.60476 .6019 L
.60714 .60166 L
.60952 .6014 L
.6119 .60112 L
.61429 .60082 L
.61667 .6005 L
Mistroke
.61905 .60017 L
.62143 .59981 L
.62381 .59944 L
.62619 .59905 L
.62857 .59864 L
.63095 .59822 L
.63333 .59777 L
.63571 .5973 L
.6381 .59682 L
.64048 .59632 L
.64286 .59581 L
.64524 .59528 L
.64762 .59473 L
.65 .59416 L
.65238 .59357 L
.65476 .59297 L
.65714 .59235 L
.65952 .59172 L
.6619 .59107 L
.66429 .5904 L
.66667 .58972 L
.66905 .58901 L
.67143 .5883 L
.67381 .58756 L
.67619 .58681 L
.67857 .58605 L
.68095 .58527 L
.68333 .58447 L
.68571 .58366 L
.6881 .58283 L
.69048 .58198 L
.69286 .58112 L
.69524 .58025 L
.69762 .57936 L
.7 .57846 L
.70238 .57754 L
.70476 .5766 L
.70714 .57565 L
.70952 .57469 L
.7119 .57371 L
.71429 .57272 L
.71667 .57172 L
.71905 .5707 L
.72143 .56966 L
.72381 .56861 L
.72619 .56755 L
.72857 .56648 L
.73095 .56539 L
.73333 .56429 L
.73571 .56317 L
Mistroke
.7381 .56204 L
.74048 .5609 L
.74286 .55975 L
.74524 .55858 L
.74762 .5574 L
.75 .55621 L
.75238 .555 L
.75476 .55378 L
.75714 .55255 L
.75952 .55131 L
.7619 .55006 L
.76429 .54879 L
.76667 .54751 L
.76905 .54623 L
.77143 .54492 L
.77381 .54361 L
.77619 .54229 L
.77857 .54096 L
.78095 .53961 L
.78333 .53825 L
.78571 .53689 L
.7881 .53551 L
.79048 .53412 L
.79286 .53272 L
.79524 .53131 L
.79762 .52989 L
.8 .52846 L
.80238 .52702 L
.80476 .52557 L
.80714 .52411 L
.80952 .52264 L
.8119 .52116 L
.81429 .51967 L
.81667 .51818 L
.81905 .51667 L
.82143 .51515 L
.82381 .51363 L
.82619 .5121 L
.82857 .51055 L
.83095 .509 L
.83333 .50744 L
.83571 .50587 L
.8381 .5043 L
.84048 .50271 L
.84286 .50112 L
.84524 .49952 L
.84762 .49791 L
.85 .49629 L
.85238 .49467 L
.85476 .49303 L
Mistroke
.85714 .4914 L
.85952 .48975 L
.8619 .48809 L
.86429 .48643 L
.86667 .48477 L
.86905 .48309 L
.87143 .48141 L
.87381 .47972 L
.87619 .47802 L
.87857 .47632 L
.88095 .47461 L
.88333 .4729 L
.88571 .47118 L
.8881 .46946 L
.89048 .46772 L
.89286 .46598 L
.89524 .46423 L
.89762 .46248 L
.9 .46073 L
.90238 .45897 L
.90476 .4572 L
.90714 .45543 L
.90952 .45365 L
.9119 .45186 L
.91429 .45008 L
.91667 .44829 L
.91905 .44649 L
.92143 .44469 L
.92381 .44288 L
.92619 .44107 L
.92857 .43925 L
.93095 .43743 L
.93333 .43561 L
.93571 .43378 L
.9381 .43195 L
.94048 .43011 L
.94286 .42827 L
.94524 .42642 L
.94762 .42458 L
.95 .42272 L
.95238 .42087 L
.95476 .41901 L
.95714 .41714 L
.95952 .41528 L
.9619 .41341 L
.96429 .41154 L
.96667 .40966 L
.96905 .40778 L
.97143 .4059 L
.97381 .40402 L
Mistroke
.97619 .40213 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo002?ool2
002?ool008ooo`03003ooooo08koo`00Sooo00<00?ooool0S_oo002?ool00`00ooooo`2>ool000Wo
o`03003ooooo08?oo`03003ooooo08koo`002Ooo00<00?ooool0Pooo0P00Sooo0009ool00`00oooo
o`23ool00`00ooooo`2>ool000[oo`03003ooooo08;oo`03003ooooo08koo`002ooo00<00?ooool0
Fooo0P001?oo00<00?ooool00_oo0`001?oo0P000ooo10000_oo0`001_oo00<00?ooool0S_oo000;
ool00`00ooooo`1Jool01000ooooo`002ooo00@00?ooool000;oo`04003ooooo0008ool00`00oooo
o`03ool00`00ooooo`2>ool000_oo`03003ooooo05[oo`04003ooooo000;ool01000ooooo`000_oo
00D00?ooooooo`0000Ooo`03003ooooo00?oo`<008koo`003?oo00<00?ooool0FOoo00@00?ooool0
00Woo`<000?oo`8000Goo`04003oooooool30006ool00`00ooooo`2>ool000goo`03003ooooo05So
o`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000Goo`04003ooooo0008ool00`00oooo
o`2>ool000goo`03003ooooo05Soo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000;o
o`04003ooooo0002ool00`00ooooo`06ool00`00ooooo`2>ool000goo`03003ooooo05Woo`8000[o
o`8000Coo`8000Coo`8000?oo`@000Goo`8008ooo`003_oo00<00?ooool0O_oo00<00?ooool0S_oo
000?ool00`00ooooo`1mool00`00ooooo`2>ool000ooo`03003ooooo07goo`03003ooooo08koo`00
3ooo00<00?ooool0OOoo00<00?ooool0S_oo000@ool00`00ooooo`1lool2002?ool0017oo`03003o
oooo07_oo`03003ooooo08koo`004Ooo00<00?ooool0Nooo00<00?ooool0S_oo000Aool00`00oooo
o`1kool00`00ooooo`2>ool001;oo`03003ooooo07[oo`03003ooooo08koo`004ooo00<00?ooool0
NOoo0P00Sooo000Cool00`00ooooo`1iool00`00ooooo`2>ool001?oo`03003ooooo07Woo`03003o
oooo08koo`005?oo00<00?ooool0N?oo00<00?ooool0S_oo000Eool00`00ooooo`1gool2002?ool0
01Goo`03003ooooo07Ooo`03003ooooo08koo`005Ooo00<00?ooool0Mooo00<00?ooool0S_oo000F
ool00`00ooooo`1Fool20004ool00`00ooooo`02ool30004ool20003ool30006ool00`00ooooo`2>
ool001Ooo`03003ooooo05Coo`04003ooooo000;ool01000ooooo`000_oo00<00?ooool00ooo00<0
0?ooool00ooo00<00?ooool0S_oo000Gool00`00ooooo`1Dool01000ooooo`002ooo00@00?ooool0
00;oo`03003ooooo00?oo`03003ooooo00?oo`<008koo`005ooo00<00?ooool0E?oo00@00?ooool0
00Woo`<000?oo`8000?oo`<000Koo`03003ooooo08koo`006?oo00<00?ooool0Dooo00@00?ooool0
00Soo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool01_oo00<00?ooool0S_oo000Iool0
0`00ooooo`1Bool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`06
ool00`00ooooo`2>ool001Woo`03003ooooo05?oo`8000[oo`8000Coo`8000?oo`@000Goo`8008oo
o`006Ooo00<00?ooool0Looo00<00?ooool0S_oo000Jool00`00ooooo`1bool00`00ooooo`2>ool0
01_oo`03003ooooo077oo`03003ooooo08koo`006ooo00<00?ooool0LOoo00<00?ooool0S_oo000L
ool00`00ooooo`1`ool2002?ool001coo`03003ooooo073oo`03003ooooo08koo`007Ooo00<00?oo
ool0Kooo00<00?ooool0S_oo000Mool00`00ooooo`1_ool00`00ooooo`2>ool001koo`03003ooooo
06koo`03003ooooo08koo`007ooo00<00?ooool0KOoo0P00Sooo000Oool00`00ooooo`1]ool00`00
ooooo`2>ool001ooo`03003ooooo06goo`03003ooooo08koo`008?oo00<00?ooool0K?oo00<00?oo
ool0S_oo000Qool00`00ooooo`1[ool2002?ool0027oo`03003ooooo06_oo`03003ooooo08koo`00
8_oo00<00?ooool0J_oo00<00?ooool0S_oo000Rool00`00ooooo`14ool20004ool00`00ooooo`02
ool30004ool20004ool00`00ooooo`02ool30006ool00`00ooooo`2>ool002?oo`03003ooooo04;o
o`04003ooooo000;ool01000ooooo`000_oo00D00?ooooooo`0000Ooo`03003ooooo00?oo`03003o
oooo08koo`008ooo00<00?ooool0@_oo00@00?ooool000_oo`04003ooooo0002ool00`00ooooo`02
ool00`00ooooo`04ool00`00ooooo`03ool3002>ool002Coo`03003ooooo047oo`04003ooooo0009
ool30003ool20005ool01000oooooooo0`001_oo00<00?ooool0S_oo000Uool00`00ooooo`10ool0
1000ooooo`002?oo00@00?ooool000;oo`04003ooooo0005ool01000ooooo`002?oo00<00?ooool0
S_oo000Uool00`00ooooo`10ool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0002ool0
1000ooooo`000_oo00<00?ooool01_oo00<00?ooool0S_oo000Uool00`00ooooo`11ool2000:ool2
0004ool20003ool40002ool40005ool00`00ooooo`2>ool002Koo`03003ooooo06Koo`8008ooo`00
9ooo00<00?ooool0IOoo00<00?ooool0S_oo000Wool00`00ooooo`1Uool00`00ooooo`2>ool002So
o`03003ooooo06Coo`03003ooooo08koo`00:Ooo00<00?ooool0Hooo0P00Sooo000Yool00`00oooo
o`1Sool00`00ooooo`2>ool002[oo`03003ooooo06;oo`03003ooooo08koo`00:_oo00<00?ooool0
H_oo00<00?ooool0S_oo000[ool00`00ooooo`1Qool00`00ooooo`2>ool002_oo`03003ooooo067o
o`8008ooo`00;?oo00<00?ooool0H?oo00<00?ooool0S_oo000]ool00`00ooooo`1Oool00`00oooo
o`2>ool002goo`03003ooooo05ooo`03003ooooo08koo`00;_oo00<00?ooool0G_oo00<00?ooool0
S_oo000_ool00`00ooooo`1Mool2002?ool0033oo`03003ooooo05coo`03003ooooo08koo`00<?oo
00<00?ooool0@_oo0P001?oo00<00?ooool00_oo0`000ooo0`001_oo00<00?ooool0S_oo000aool0
0`00ooooo`10ool01000ooooo`002ooo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool0S_oo
000bool00`00ooooo`0oool01000ooooo`002ooo00<00?ooool00ooo00<00?ooool00ooo0`00S_oo
000bool00`00ooooo`0oool01000ooooo`002Ooo0`000ooo0`001Ooo00<00?ooool0S_oo000bool0
0`00ooooo`0oool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0005ool00`00ooooo`2>
ool003?oo`03003ooooo03koo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000Goo`03
003ooooo08koo`00=?oo00<00?ooool0?_oo0P002_oo0P001?oo0P001_oo00<00?ooool0S_oo000d
ool00`00ooooo`1Hool2002?ool003Goo`03003ooooo05Ooo`03003ooooo08koo`00=_oo00<00?oo
ool0E_oo00<00?ooool0S_oo000fool00`00ooooo`1Fool00`00ooooo`2>ool003Ooo`03003ooooo
05Goo`03003ooooo08koo`00>?oo00<00?ooool0E?oo0P00Sooo000hool00`00ooooo`1Dool00`00
ooooo`2>ool003Woo`03003ooooo05?oo`03003ooooo08koo`00>_oo00<00?ooool0D_oo00<00?oo
ool0S_oo000jool00`00ooooo`1Bool2002?ool003_oo`03003ooooo057oo`03003ooooo08koo`00
??oo00<00?ooool0D?oo00<00?ooool0S_oo000lool00`00ooooo`1@ool00`00ooooo`2>ool003go
o`03003ooooo04ooo`03003ooooo08koo`00?_oo00<00?ooool0C_oo0P00Sooo000nool00`00oooo
o`1>ool00`00ooooo`2>ool003ooo`03003ooooo02Ooo`8000Coo`03003ooooo00;oo`<000?oo`<0
00?oo`@000;oo`<000Koo`03003ooooo08koo`00@?oo00<00?ooool09Ooo00@00?ooool000_oo`03
003ooooo00?oo`04003ooooo0008ool00`00ooooo`03ool00`00ooooo`2>ool0047oo`03003ooooo
02Coo`04003ooooo000;ool00`00ooooo`03ool01@00oooooooo00001ooo00<00?ooool00ooo0`00
S_oo0012ool00`00ooooo`0Sool01000ooooo`002Ooo0`000ooo0`001?oo00@00?ooooooo`<000Ko
o`03003ooooo08koo`00@_oo00<00?ooool08ooo00@00?ooool000Soo`04003ooooo0002ool01000
ooooo`001Ooo00@00?ooool000Soo`03003ooooo08koo`00@ooo00<00?ooool08_oo00@00?ooool0
00Soo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Koo`03003o
oooo08koo`00A?oo00<00?ooool08_oo0P002_oo0P001?oo0P001?oo0P000ooo10001Ooo00<00?oo
ool0S_oo0014ool00`00ooooo`18ool2002?ool004Goo`03003ooooo04Ooo`03003ooooo08koo`00
A_oo00<00?ooool0A_oo00<00?ooool0S_oo0016ool20017ool00`00ooooo`2>ool004Soo`03003o
oooo04Coo`03003ooooo08koo`00B?oo00<00?ooool0A?oo0P00Sooo0019ool00`00ooooo`13ool0
0`00ooooo`24ool00`00ooooo`07ool004[oo`03003ooooo04;oo`03003ooooo08;oo`8000[oo`00
B_oo00<00?ooool0@_oo00<00?ooool0POoo00<00?ooool02_oo001;ool20012ool20021ool00`00
ooooo`0;ool004coo`03003ooooo043oo`03003ooooo07ooo`03003ooooo00coo`00COoo00<00?oo
ool0?ooo00<00?ooool0OOoo0P003ooo001>ool00`00ooooo`0nool00`00ooooo`1lool2000@ool0
04koo`8003ooo`03003ooooo07_oo`03003ooooo013oo`00D?oo00<00?ooool0??oo0P00N_oo0P00
4ooo001@ool00`00ooooo`0lool00`00ooooo`1hool2000Dool0057oo`03003ooooo03_oo`03003o
oooo07Ooo`03003ooooo01Coo`00D_oo00<00?ooool06_oo0P001?oo00<00?ooool00_oo0`000ooo
0`000ooo0`001_oo00<00?ooool0MOoo0P005ooo001Cool00`00ooooo`0Hool01000ooooo`002ooo
00<00?ooool00ooo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool0M?oo0P006?oo001Dool0
0`00ooooo`0Gool01000ooooo`002ooo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool00ooo
0`00Looo00<00?ooool06?oo001Dool2000Hool01000ooooo`002Ooo0`000ooo0`000_oo0`001_oo
00<00?ooool0LOoo0P006ooo001Fool00`00ooooo`0Eool01000ooooo`002?oo00@00?ooool000;o
o`04003ooooo0002ool00`00ooooo`06ool00`00ooooo`1`ool2000Lool005Ooo`03003ooooo01Co
o`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00Koo`03003ooooo
06ooo`03003ooooo01coo`00Eooo0P005_oo0P002_oo0P001?oo0P000ooo10001Ooo0P00K_oo0P00
7ooo001Iool00`00ooooo`0cool00`00ooooo`1/ool2000Pool005Woo`8003Coo`03003ooooo06_o
o`03003ooooo023oo`00Fooo00<00?ooool0<Ooo00<00?ooool0JOoo0P008ooo001Kool2000bool0
0`00ooooo`1Xool2000Tool005goo`03003ooooo02ooo`8006Soo`03003ooooo02Coo`00GOoo0P00
<?oo00<00?ooool0IOoo0P009ooo001Oool00`00ooooo`0]ool00`00ooooo`1Tool2000Xool005oo
o`8002koo`03003ooooo06?oo`03003ooooo02Soo`00HOoo00<00?ooool0:ooo00<00?ooool0HOoo
0P00:ooo001Qool2000/ool2001Pool2000]ool006?oo`03003ooooo02Woo`03003ooooo05koo`80
02koo`00I?oo00<00?ooool0:?oo00<00?ooool0G?oo0P00<?oo001Uool00`00ooooo`0Wool00`00
ooooo`1Kool2000aool006Koo`8002Ooo`8005[oo`8003?oo`00Iooo00<00?ooool09Ooo00<00?oo
ool0F?oo00<00?ooool0<ooo001Xool2000Uool00`00ooooo`1Fool2000fool006Woo`8000Coo`03
003ooooo00;oo`<000?oo`<000Coo`03003ooooo00;oo`<000Koo`03003ooooo05Coo`<003Ooo`00
J?oo00@00?ooool000_oo`03003ooooo00?oo`05003oooooool00007ool00`00ooooo`03ool00`00
ooooo`1Cool2000iool006Soo`03003ooooo00<000Woo`03003ooooo00?oo`03003ooooo00;oo`03
003ooooo00Coo`03003ooooo00?oo`<0057oo`8003_oo`00J?oo00D00?ooool00?oo008000Koo`<0
00?oo`<000Coo`04003oooooool30006ool00`00ooooo`1@ool00`00ooooo`0kool006Soo`04003o
oooo0003ool00`00ooooo`02ool01000ooooo`000_oo00@00?ooool000Goo`04003ooooo0008ool0
0`00ooooo`1>ool2000nool006Soo`04003ooooo0004ool20002ool01000ooooo`000_oo00@00?oo
ool000;oo`04003ooooo0002ool00`00ooooo`06ool00`00ooooo`1<ool20010ool006Woo`8000Ko
o`8000;oo`8000Coo`8000?oo`@000;oo`@000Goo`8004_oo`<0047oo`00Looo00<00?ooool06Ooo
00<00?ooool0B?oo0`00@ooo001dool2000Iool00`00ooooo`16ool30015ool007Koo`8001Ooo`03
003ooooo04Goo`8004Ooo`00Mooo0P005_oo00<00?ooool0@ooo0P00BOoo001iool2000Dool20011
ool3001;ool007_oo`8001;oo`03003ooooo03koo`<004goo`00OOoo0P004?oo00<00?ooool0??oo
0`00Cooo0002ool20004ool00`00ooooo`03ool2000eool20004ool00`00ooooo`02ool3000/ool2
000>ool00`00ooooo`0hool50002ool00`00ooooo`02ool4000dool40003ool00`00ooooo`02ool4
0002ool00005ool00?ooool00008ool01000ooooo`00<ooo00@00?ooool000_oo`03003ooooo02[o
o`<000coo`03003ooooo03Soo`<000_oo`03003ooooo03Goo`03003ooooo00Ooo`03003ooooo00?o
o`0000Goo`00ooooo`0000Soo`04003ooooo000cool01000ooooo`002ooo00<00?ooool0;Ooo0P00
2_oo0P00=_oo0`000_oo00<00?ooool02Ooo00<00?ooool0=Ooo00<00?ooool02?oo00<00?ooool0
0_oo00001Ooo003ooooo00002Ooo0P00=?oo00@00?ooool000Woo`<0037oo`<000Ooo`03003ooooo
03;oo`<000Goo`03003ooooo00Woo`03003ooooo03Goo`03003ooooo00Woo`03003ooooo007oo`00
00Goo`00ooooo`0000Soo`04003ooooo000cool01000ooooo`002?oo00@00?ooool003Coo`<000Co
o`03003ooooo02ooo`<000Soo`03003ooooo00Woo`03003ooooo03Goo`03003ooooo00[oo`40007o
o`7oo`0000Goo`00ooooo`0000Soo`04003ooooo000cool01000ooooo`002?oo00@00?ooool003Ko
o`<000;oo`03003ooooo02coo`<000Woo`<000Woo`<003Goo`<000Woo`04003ooooo0002ool000;o
o`8000[oo`8003Goo`8000[oo`8003[oo`@002Woo`@000koo`03003ooooo00Woo`03003ooooo03Go
o`03003ooooo00Soo`8000?oo`00Sooo1P008?oo1@00I_oo002?ool00`00ooooo`03ool6000Dool7
001Zool008ooo`03003ooooo00SooaH0073oo`00Sooo00<00?ooool0S_oo0002oooo000L0003ool0
00Woo`03003ooooo00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo
00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo
00[oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo
00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00[oo`03003ooooo
00_oo`03003ooooo00Ooo`002Ooo00<00?ooool0@?oo00<00?ooool0@?oo00<00?ooool0@?oo00<0
0?ooool0@Ooo00<00?ooool01ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.786615, 0.981302, \
0.001487, 0.000107815}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %384]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.368961 0.868961 -15.3219 15.7122 [
[.15242 .37785 -9 -9 ]
[.15242 .37785 9 0 ]
[.32621 .37785 -9 -9 ]
[.32621 .37785 9 0 ]
[.67379 .37785 -9 -9 ]
[.67379 .37785 9 0 ]
[.84758 .37785 -9 -9 ]
[.84758 .37785 9 0 ]
[.4875 .07611 -24 -4.5 ]
[.4875 .07611 0 4.5 ]
[.4875 .15467 -30 -4.5 ]
[.4875 .15467 0 4.5 ]
[.4875 .23323 -24 -4.5 ]
[.4875 .23323 0 4.5 ]
[.4875 .31179 -30 -4.5 ]
[.4875 .31179 0 4.5 ]
[.4875 .46891 -30 -4.5 ]
[.4875 .46891 0 4.5 ]
[.4875 .54748 -24 -4.5 ]
[.4875 .54748 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15242 .39035 m
.15242 .3966 L
s
[(0.6)] .15242 .37785 0 1 Mshowa
.32621 .39035 m
.32621 .3966 L
s
[(0.8)] .32621 .37785 0 1 Mshowa
.67379 .39035 m
.67379 .3966 L
s
[(1.2)] .67379 .37785 0 1 Mshowa
.84758 .39035 m
.84758 .3966 L
s
[(1.4)] .84758 .37785 0 1 Mshowa
.125 Mabswid
.19586 .39035 m
.19586 .3941 L
s
.23931 .39035 m
.23931 .3941 L
s
.28276 .39035 m
.28276 .3941 L
s
.36966 .39035 m
.36966 .3941 L
s
.4131 .39035 m
.4131 .3941 L
s
.45655 .39035 m
.45655 .3941 L
s
.54345 .39035 m
.54345 .3941 L
s
.5869 .39035 m
.5869 .3941 L
s
.63034 .39035 m
.63034 .3941 L
s
.71724 .39035 m
.71724 .3941 L
s
.76069 .39035 m
.76069 .3941 L
s
.80414 .39035 m
.80414 .3941 L
s
.10897 .39035 m
.10897 .3941 L
s
.06552 .39035 m
.06552 .3941 L
s
.02207 .39035 m
.02207 .3941 L
s
.89103 .39035 m
.89103 .3941 L
s
.93448 .39035 m
.93448 .3941 L
s
.97793 .39035 m
.97793 .3941 L
s
.25 Mabswid
0 .39035 m
1 .39035 L
s
.5 .07611 m
.50625 .07611 L
s
[(0.98)] .4875 .07611 1 0 Mshowa
.5 .15467 m
.50625 .15467 L
s
[(0.985)] .4875 .15467 1 0 Mshowa
.5 .23323 m
.50625 .23323 L
s
[(0.99)] .4875 .23323 1 0 Mshowa
.5 .31179 m
.50625 .31179 L
s
[(0.995)] .4875 .31179 1 0 Mshowa
.5 .46891 m
.50625 .46891 L
s
[(1.005)] .4875 .46891 1 0 Mshowa
.5 .54748 m
.50625 .54748 L
s
[(1.01)] .4875 .54748 1 0 Mshowa
.125 Mabswid
.5 .01326 m
.50375 .01326 L
s
.5 .02897 m
.50375 .02897 L
s
.5 .04468 m
.50375 .04468 L
s
.5 .0604 m
.50375 .0604 L
s
.5 .09182 m
.50375 .09182 L
s
.5 .10753 m
.50375 .10753 L
s
.5 .12325 m
.50375 .12325 L
s
.5 .13896 m
.50375 .13896 L
s
.5 .17038 m
.50375 .17038 L
s
.5 .18609 m
.50375 .18609 L
s
.5 .20181 m
.50375 .20181 L
s
.5 .21752 m
.50375 .21752 L
s
.5 .24894 m
.50375 .24894 L
s
.5 .26466 m
.50375 .26466 L
s
.5 .28037 m
.50375 .28037 L
s
.5 .29608 m
.50375 .29608 L
s
.5 .3275 m
.50375 .3275 L
s
.5 .34322 m
.50375 .34322 L
s
.5 .35893 m
.50375 .35893 L
s
.5 .37464 m
.50375 .37464 L
s
.5 .40607 m
.50375 .40607 L
s
.5 .42178 m
.50375 .42178 L
s
.5 .43749 m
.50375 .43749 L
s
.5 .4532 m
.50375 .4532 L
s
.5 .48463 m
.50375 .48463 L
s
.5 .50034 m
.50375 .50034 L
s
.5 .51605 m
.50375 .51605 L
s
.5 .53176 m
.50375 .53176 L
s
.5 .56319 m
.50375 .56319 L
s
.5 .5789 m
.50375 .5789 L
s
.5 .59461 m
.50375 .59461 L
s
.5 .61032 m
.50375 .61032 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.32621 .10563 m
.32708 .108 L
.32795 .11037 L
.32881 .11273 L
.32968 .11508 L
.33055 .11743 L
.33142 .11976 L
.33229 .12208 L
.33316 .1244 L
.33403 .1267 L
.3349 .129 L
.33577 .13129 L
.33664 .13357 L
.3375 .13584 L
.33837 .1381 L
.33924 .14035 L
.34011 .14259 L
.34098 .14482 L
.34185 .14704 L
.34272 .14926 L
.34359 .15146 L
.34446 .15366 L
.34532 .15584 L
.34619 .15802 L
.34706 .16018 L
.34793 .16234 L
.3488 .16449 L
.34967 .16663 L
.35054 .16876 L
.35141 .17088 L
.35228 .17299 L
.35315 .17509 L
.35401 .17719 L
.35488 .17927 L
.35575 .18134 L
.35662 .1834 L
.35749 .18546 L
.35836 .1875 L
.35923 .18954 L
.3601 .19156 L
.36097 .19358 L
.36184 .19558 L
.3627 .19758 L
.36357 .19957 L
.36444 .20154 L
.36531 .20351 L
.36618 .20547 L
.36705 .20741 L
.36792 .20935 L
.36879 .21128 L
Mistroke
.36966 .2132 L
.37052 .2151 L
.37139 .217 L
.37226 .21889 L
.37313 .22077 L
.374 .22264 L
.37487 .2245 L
.37574 .22635 L
.37661 .22819 L
.37748 .23001 L
.37835 .23183 L
.37921 .23364 L
.38008 .23544 L
.38095 .23723 L
.38182 .23901 L
.38269 .24078 L
.38356 .24254 L
.38443 .24429 L
.3853 .24602 L
.38617 .24775 L
.38704 .24947 L
.3879 .25118 L
.38877 .25288 L
.38964 .25457 L
.39051 .25624 L
.39138 .25791 L
.39225 .25957 L
.39312 .26121 L
.39399 .26285 L
.39486 .26448 L
.39572 .26609 L
.39659 .2677 L
.39746 .26929 L
.39833 .27088 L
.3992 .27245 L
.40007 .27402 L
.40094 .27557 L
.40181 .27711 L
.40268 .27865 L
.40355 .28017 L
.40441 .28168 L
.40528 .28318 L
.40615 .28467 L
.40702 .28615 L
.40789 .28762 L
.40876 .28908 L
.40963 .29054 L
.4105 .29197 L
.41137 .2934 L
.41223 .29482 L
Mistroke
.4131 .29622 L
.41397 .29762 L
.41484 .29901 L
.41571 .30038 L
.41658 .30174 L
.41745 .3031 L
.41832 .30444 L
.41919 .30577 L
.42006 .30709 L
.42092 .30841 L
.42179 .30971 L
.42266 .311 L
.42353 .31227 L
.4244 .31354 L
.42527 .3148 L
.42614 .31605 L
.42701 .31728 L
.42788 .31851 L
.42875 .31972 L
.42961 .32092 L
.43048 .32212 L
.43135 .3233 L
.43222 .32447 L
.43309 .32563 L
.43396 .32678 L
.43483 .32791 L
.4357 .32904 L
.43657 .33016 L
.43743 .33126 L
.4383 .33236 L
.43917 .33344 L
.44004 .33452 L
.44091 .33558 L
.44178 .33663 L
.44265 .33767 L
.44352 .3387 L
.44439 .33972 L
.44526 .34072 L
.44612 .34172 L
.44699 .34271 L
.44786 .34368 L
.44873 .34464 L
.4496 .3456 L
.45047 .34654 L
.45134 .34747 L
.45221 .34839 L
.45308 .3493 L
.45395 .3502 L
.45481 .35108 L
.45568 .35196 L
Mistroke
.45655 .35283 L
.45742 .35368 L
.45829 .35452 L
.45916 .35535 L
.46003 .35618 L
.4609 .35699 L
.46177 .35779 L
.46263 .35857 L
.4635 .35935 L
.46437 .36012 L
.46524 .36087 L
.46611 .36162 L
.46698 .36235 L
.46785 .36307 L
.46872 .36379 L
.46959 .36449 L
.47046 .36518 L
.47132 .36586 L
.47219 .36653 L
.47306 .36718 L
.47393 .36783 L
.4748 .36846 L
.47567 .36909 L
.47654 .3697 L
.47741 .37031 L
.47828 .3709 L
.47914 .37148 L
.48001 .37205 L
.48088 .37261 L
.48175 .37316 L
.48262 .3737 L
.48349 .37422 L
.48436 .37474 L
.48523 .37525 L
.4861 .37574 L
.48697 .37623 L
.48783 .3767 L
.4887 .37716 L
.48957 .37761 L
.49044 .37806 L
.49131 .37849 L
.49218 .37891 L
.49305 .37932 L
.49392 .37972 L
.49479 .3801 L
.49566 .38048 L
.49652 .38085 L
.49739 .38121 L
.49826 .38155 L
.49913 .38189 L
Mistroke
.5 .38221 L
.50087 .38253 L
.50174 .38283 L
.50261 .38313 L
.50348 .38341 L
.50434 .38369 L
.50521 .38395 L
.50608 .3842 L
.50695 .38444 L
.50782 .38468 L
.50869 .3849 L
.50956 .38511 L
.51043 .38531 L
.5113 .3855 L
.51217 .38569 L
.51303 .38586 L
.5139 .38602 L
.51477 .38617 L
.51564 .38631 L
.51651 .38644 L
.51738 .38656 L
.51825 .38667 L
.51912 .38678 L
.51999 .38687 L
.52086 .38695 L
.52172 .38702 L
.52259 .38708 L
.52346 .38714 L
.52433 .38718 L
.5252 .38721 L
.52607 .38724 L
.52694 .38725 L
.52781 .38725 L
.52868 .38725 L
.52954 .38724 L
.53041 .38721 L
.53128 .38718 L
.53215 .38714 L
.53302 .38708 L
.53389 .38702 L
.53476 .38695 L
.53563 .38687 L
.5365 .38678 L
.53737 .38669 L
.53823 .38658 L
.5391 .38646 L
.53997 .38634 L
.54084 .3862 L
.54171 .38606 L
.54258 .38591 L
Mistroke
.54345 .38575 L
.54432 .38558 L
.54519 .3854 L
.54605 .38521 L
.54692 .38502 L
.54779 .38481 L
.54866 .3846 L
.54953 .38438 L
.5504 .38415 L
.55127 .38391 L
.55214 .38366 L
.55301 .38341 L
.55388 .38314 L
.55474 .38287 L
.55561 .38259 L
.55648 .3823 L
.55735 .38201 L
.55822 .38171 L
.55909 .38139 L
.55996 .38107 L
.56083 .38075 L
.5617 .38041 L
.56257 .38007 L
.56343 .37972 L
.5643 .37936 L
.56517 .37899 L
.56604 .37862 L
.56691 .37824 L
.56778 .37785 L
.56865 .37745 L
.56952 .37705 L
.57039 .37664 L
.57125 .37622 L
.57212 .37579 L
.57299 .37536 L
.57386 .37492 L
.57473 .37447 L
.5756 .37402 L
.57647 .37356 L
.57734 .37309 L
.57821 .37262 L
.57908 .37213 L
.57994 .37165 L
.58081 .37115 L
.58168 .37065 L
.58255 .37014 L
.58342 .36963 L
.58429 .36911 L
.58516 .36858 L
.58603 .36805 L
Mistroke
.5869 .36751 L
.58777 .36696 L
.58863 .36641 L
.5895 .36585 L
.59037 .36528 L
.59124 .36471 L
.59211 .36414 L
.59298 .36355 L
.59385 .36297 L
.59472 .36237 L
.59559 .36177 L
.59645 .36117 L
.59732 .36055 L
.59819 .35994 L
.59906 .35932 L
.59993 .35869 L
.6008 .35805 L
.60167 .35742 L
.60254 .35677 L
.60341 .35612 L
.60428 .35547 L
.60514 .35481 L
.60601 .35414 L
.60688 .35348 L
.60775 .3528 L
.60862 .35212 L
.60949 .35144 L
.61036 .35075 L
.61123 .35005 L
.6121 .34936 L
.61296 .34865 L
.61383 .34795 L
.6147 .34723 L
.61557 .34652 L
.61644 .3458 L
.61731 .34507 L
.61818 .34434 L
.61905 .34361 L
.61992 .34287 L
.62079 .34213 L
.62165 .34138 L
.62252 .34063 L
.62339 .33988 L
.62426 .33912 L
.62513 .33836 L
.626 .33759 L
.62687 .33682 L
.62774 .33605 L
.62861 .33527 L
.62948 .33449 L
Mistroke
.63034 .3337 L
.63121 .33292 L
.63208 .33212 L
.63295 .33133 L
.63382 .33053 L
.63469 .32973 L
.63556 .32892 L
.63643 .32812 L
.6373 .32731 L
.63816 .32649 L
.63903 .32567 L
.6399 .32485 L
.64077 .32403 L
.64164 .32321 L
.64251 .32237 L
.64338 .32154 L
.64425 .32071 L
.64512 .31987 L
.64599 .31903 L
.64685 .31819 L
.64772 .31734 L
.64859 .31649 L
.64946 .31564 L
.65033 .31479 L
.6512 .31393 L
.65207 .31308 L
.65294 .31222 L
.65381 .31136 L
.65468 .31049 L
.65554 .30962 L
.65641 .30875 L
.65728 .30788 L
.65815 .30701 L
.65902 .30614 L
.65989 .30526 L
.66076 .30438 L
.66163 .3035 L
.6625 .30262 L
.66336 .30173 L
.66423 .30085 L
.6651 .29996 L
.66597 .29907 L
.66684 .29818 L
.66771 .29728 L
.66858 .29639 L
.66945 .29549 L
.67032 .2946 L
.67119 .2937 L
.67205 .2928 L
.67292 .29189 L
Mistroke
.67379 .29099 L
Mfstroke
.47841 0 m
.49918 .37556 L
s
.49918 .37556 m
.51259 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00OROoo00<00?ooool0
0ooo00<00?ooool0S_oo0029ool00`00ooooo`03ool00`00ooooo`2>ool008Woo`03003ooooo00?o
o`03003ooooo08koo`00ROoo00<00?ooool00ooo0P00Sooo0029ool00`00ooooo`03ool00`00oooo
o`2>ool008Woo`03003ooooo00?oo`03003ooooo08koo`00ROoo00<00?ooool00ooo00<00?ooool0
S_oo0029ool00`00ooooo`03ool00`00ooooo`2>ool008Woo`03003ooooo00?oo`8008ooo`00R_oo
00<00?ooool00_oo00<00?ooool0S_oo002:ool00`00ooooo`02ool00`00ooooo`2>ool008[oo`03
003ooooo00;oo`03003ooooo08koo`00R_oo00<00?ooool00_oo0P00Sooo002:ool00`00ooooo`02
ool00`00ooooo`2>ool008[oo`03003ooooo00;oo`03003ooooo08koo`00R_oo00<00?ooool00_oo
00<00?ooool0S_oo002:ool00`00ooooo`02ool00`00ooooo`2>ool008[oo`03003ooooo00;oo`80
08ooo`00R_oo00<00?ooool00_oo00<00?ooool0S_oo001eool20004ool00`00ooooo`02ool30004
ool200000ooo003oo`03ool00`00ooooo`2>ool007Coo`04003ooooo000;ool01000ooooo`000_oo
0P001?oo00<00?ooool0S_oo001dool01000ooooo`002ooo00@00?ooool000;oo`8000Coo`<008ko
o`00M?oo00@00?ooool000Woo`<000?oo`800003ool00?oo00?oo`03003ooooo08koo`00M?oo00@0
0?ooool000Soo`04003ooooo0002ool00`00ooooo`020004ool00`00ooooo`2>ool007Coo`04003o
oooo0008ool01000ooooo`000_oo00<00?ooool00P001?oo00<00?ooool0S_oo001eool2000:ool2
0004ool200000ooo003oo`03ool00`00ooooo`2>ool008[oo`03003ooooo00;oo`8008ooo`00Rooo
00D00?ooooooo`00093oo`00Rooo00D00?ooooooo`00093oo`00Rooo00D00?ooooooo`00093oo`00
GOoo00<00?ooool0:ooo00@00?ooooooo`8008ooo`00G_oo00<00?ooool0:_oo00D00?ooooooo`00
093oo`00G_oo00<00?ooool0:_oo00D00?ooooooo`00093oo`00G_oo00<00?ooool0:_oo00D00?oo
ooooo`00093oo`00Gooo00<00?ooool0:Ooo00D00?ooooooo`00093oo`00Gooo00<00?ooool0:Ooo
00@00?ooooooo`8008ooo`00Gooo0P00:_oo00D00?ooooooo`00093oo`00H?oo00<00?ooool0:?oo
00D00?ooooooo`00093oo`00H?oo00<00?ooool0:?oo00D00?ooooooo`00093oo`00HOoo00<00?oo
ool09ooo00@00?ooooooo`8008ooo`00HOoo00<00?ooool09ooo00D00?ooooooo`00093oo`00HOoo
00<00?ooool09ooo00D00?ooooooo`00093oo`00H_oo00<00?ooool02_oo0P001?oo00<00?ooool0
0_oo0`001?oo0P000ooo0`000_oo00D00?ooooooo`00093oo`00H_oo00<00?ooool02Ooo00@00?oo
ool000_oo`04003ooooo0002ool00`00ooooo`03ool00`00ool00003ool00`00ooooo`2>ool006;o
o`8000[oo`04003ooooo000;ool01000ooooo`000_oo00<00?ooool00ooo00<00?oo00000ooo0`00
S_oo001Sool00`00ooooo`08ool01000ooooo`002Ooo0`000ooo0P000ooo0`000ooo00@00?ooool0
093oo`00Hooo00<00?ooool02?oo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`000_oo
00<00?ooool00ooo00@00?ooool0093oo`00I?oo00<00?ooool01ooo00@00?ooool000Soo`04003o
oooo0002ool01000ooooo`000_oo00<00?ooool00ooo00@00?ooool0093oo`00I?oo00<00?ooool0
2?oo0P002_oo0P001?oo0P000ooo10000_oo00<00?ooool00P00Sooo001Uool00`00ooooo`0Tool0
1000ooooo`00T?oo001Uool00`00ooooo`0Tool01000ooooo`00T?oo001Uool2000Uool01000oooo
o`00T?oo001Vool00`00ooooo`0Sool01000ooooo`00T?oo001Vool00`00ooooo`0Sool00`00oooo
o`02002?ool006Ooo`03003ooooo02;oo`04003ooooo002@ool006Ooo`03003ooooo02;oo`04003o
oooo002@ool006Ooo`8002?oo`04003ooooo002@ool006Soo`03003ooooo027oo`03003ooooo0080
08ooo`00J?oo0P008_oo00@00?ooool0093oo`00JOoo00<00?ooool08?oo00@00?ooool0093oo`00
JOoo00<00?ooool08?oo00@00?ooool0093oo`00J_oo00<00?ooool07ooo00@00?ooool0093oo`00
J_oo00<00?ooool07ooo00<00?ooool00P00Sooo001[ool00`00ooooo`0Oool00`00ool0002@ool0
06_oo`03003ooooo00Ooo`8000Coo`03003ooooo00;oo`<000?oo`<000Coo`03003oo`00093oo`00
K?oo00<00?ooool01Ooo00@00?ooool000_oo`03003ooooo00?oo`07003oooooool00?oo0000T?oo
001/ool00`00ooooo`05ool01000ooooo`002ooo00<00?ooool00ooo00H00?ooooooo`00ool3002>
ool006goo`03003ooooo00Coo`04003ooooo0009ool30003ool30003ool00`00ool0002@ool006go
o`03003ooooo00Coo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool000?oo`03003oo`00
093oo`00K_oo00<00?ooool00ooo00@00?ooool000Soo`04003ooooo0002ool01000ooooo`000ooo
00<00?oo0000T?oo001^ool00`00ooooo`04ool2000:ool20004ool20004ool00`00ool0002@ool0
06ooo`03003ooooo01_oo`04003oo`00002?ool006ooo`03003ooooo01_oo`03003oo`00093oo`00
L?oo00<00?ooool06_oo00<00?oo0000T?oo001`ool00`00ooooo`0Jool00`00ool0002@ool0077o
o`03003ooooo01Woo`04003oo`00002?ool0077oo`03003ooooo01Woo`03003oo`00093oo`00L_oo
00<00?ooool06?oo00<00?oo0000T?oo001bool2000Iool00`00ool0002@ool007?oo`03003ooooo
01Ooo`03003oo`00093oo`00Looo0P006?oo00@00?oo000008ooo`00M?oo00<00?ooool05ooo0P00
T?oo001eool00`00ooooo`0Fool2002@ool007Goo`03003ooooo01Koo`80033oo`8005koo`00M_oo
00<00?ooool05Ooo0`00;_oo0P00Gooo001fool2000Fool2000^ool2001Pool007Ooo`03003ooooo
01Coo`8002goo`80067oo`00Kooo0P001?oo00D00?ooool00?oo00<000?oo`<000?oo`<000Goo`80
02coo`8006;oo`00K_oo00@00?ooool000Koo`8000?oo`03003ooooo00?oo`03003ooooo00?oo`03
003ooooo00;oo`8002_oo`8006?oo`00K_oo00@00?ooool000Ooo`05003oooooool00005ool00`00
ooooo`03ool00`00ooooo`02ool4000Xool2001Tool006koo`04003ooooo0008ool40003ool30002
ool30005ool2000Yool2001Uool006koo`04003ooooo0008ool200001Ooo003ooooo00000_oo00@0
0?ooool000Ooo`8002Soo`8006Koo`00K_oo00@00?ooool000Soo`@000;oo`04003ooooo0002ool0
0`00ooooo`05ool2000Wool2001Wool006ooo`8000[oo`8000Coo`8000?oo`@000Coo`<002Goo`80
06Soo`00OOoo00<00?ooool03_oo0P009Ooo0P00JOoo001mool2000?ool2000Tool2001Zool007ko
o`8000koo`8002?oo`8006_oo`00Oooo0P003Ooo0P008_oo0P00K?oo0020ool00`00ooooo`0;ool3
000Pool2001]ool0087oo`03003ooooo00_oo`03003ooooo01goo`8006ooo`00P_oo00<00?ooool0
2_oo00<00?ooool07?oo0P00L?oo000Tool20004ool00`00ooooo`03ool2000Tool20004ool00`00
ooooo`03ool2000Oool00`00ooooo`09ool00`00ooooo`0Kool2000:ool40003ool00`00ooooo`02
ool4000Rool40003ool00`00ooooo`04ool00`00ooooo`0Tool002?oo`04003ooooo0008ool01000
ooooo`008_oo00@00?ooool000Soo`04003ooooo000Oool20009ool2000Jool3000=ool00`00oooo
o`07ool00`00ooooo`0Uool00`00ooooo`09ool00`00ooooo`0Tool002?oo`04003ooooo0008ool0
1000ooooo`008_oo00@00?ooool000Soo`04003ooooo000Pool20008ool00`00ooooo`0Hool2000?
ool00`00ooooo`08ool00`00ooooo`0Tool00`00ooooo`06ool5000Uool002?oo`04003ooooo0008
ool3000Sool01000ooooo`002Ooo0P008_oo0P001ooo00<00?ooool05_oo0`004?oo00<00?ooool0
2Ooo00<00?ooool08ooo00<00?ooool01_oo00@00?ooool002Koo`008ooo00@00?ooool000Soo`03
003ooooo02?oo`04003ooooo0008ool01000ooooo`008_oo0P001_oo00<00?ooool05Ooo0P004_oo
00<00?ooool02_oo00<00?ooool08_oo00<00?ooool01ooo00<00?oo00009_oo000Sool01000oooo
o`002?oo00<00?ooool08ooo00@00?ooool000Soo`04003ooooo000Tool20004ool00`00ooooo`0C
ool3000Aool30009ool01000ooooo`008_oo0`002_oo0P009_oo000Tool2000:ool3000Sool2000:
ool2000Vool30002ool2000Bool3000Eool00`00ooooo`08ool2000Uool00`00ooooo`09ool00`00
ooooo`0Tool008coo`@0017oo`<007coo`00S_oo0`003Ooo0`00Oooo002?ool00`00ool0000<0022
ool00?l00200007oo`001_oo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?oo
ool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?oo
ool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?oo
ool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool02Ooo00<00?oo
ool02_oo00<00?ooool02Ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool01Ooo000[ool0
0`00ooooo`0_ool00`00ooooo`0_ool00`00ooooo`0_ool00`00ooooo`0_ool00`00ooooo`0Zool0
08ooo`03003ooooo08koo`00Sooo00<00?ooool0S_oo002?ool2002?ool008ooo`8008ooo`00Sooo
0P00Sooo002?ool2002?ool008ooo`8008ooo`00Sooo0P00Sooo002?ool2002?ool008ooo`8008oo
o`00Sooo0P00Sooo002?ool2002?ool008ooo`8008ooo`00Sooo0P00Sooo002?ool2002?ool008oo
o`8008ooo`00Sooo0P00Sooo002?ool2002?ool006koo`@000?oo`03003ooooo00?oo`8000Coo`80
00?oo`<000Koo`8008ooo`00L?oo00<00?ooool01ooo00@00?ooool000;oo`04003ooooo0005ool0
0`00ooooo`03ool00`00ool0002>ool0073oo`03003ooooo00Ooo`04003ooooo0002ool01000oooo
o`001Ooo00<00?ooool00ooo0`00S_oo001`ool00`00ooooo`07ool01000ooooo`000_oo00@00?oo
ool000;oo`<000Koo`03003oo`0008koo`00L?oo00<00?ooool01ooo00@00?ooool000;oo`04003o
oooo0002ool00`00ooooo`06ool00`00ool0002>ool006koo`<000Woo`04003ooooo0002ool01000
ooooo`000_oo00<00?ooool01_oo00<00?oo0000S_oo001`ool00`00ooooo`08ool20004ool20003
ool40005ool3002>ool008ooo`03003oo`0008koo`00Sooo00<00?oo0000S_oo002?ool00`00ool0
002>ool008ooo`03003oo`0008koo`00Sooo0`00S_oo002?ool00`00ool0002>ool008ooo`03003o
o`0008koo`00Sooo00<00?oo0000S_oo002?ool3002>ool008ooo`03003oo`0008koo`00Sooo00<0
0?oo0000S_oo002?ool00`00ool0002>ool008ooo`04003ooooo002=ool008ooo`800003ool00?oo
08coo`00Sooo00@00?ooool008goo`00M?oo10000ooo00<00?ooool00ooo0P000ooo10001Ooo00@0
0?ooool008goo`00M_oo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo00Coo`04003ooooo
002=ool007Koo`03003ooooo00Ooo`04003ooooo0004ool00`00ooooo`04ool4002=ool007Koo`03
003ooooo00Ooo`04003ooooo0004ool00`00ooooo`04ool01000ooooo`00SOoo001fool00`00oooo
o`07ool01000ooooo`001?oo00<00?ooool01?oo00@00?ooool008goo`00M?oo0`002Ooo00@00?oo
ool000;oo`<000Koo`04003ooooo002=ool007Koo`03003ooooo00Soo`8000Goo`03003ooooo00Co
o`04003ooooo002=ool008ooo`800003ool00?oo08coo`00Sooo00@00?ooool008goo`00Sooo00@0
0?ooool008goo`00Sooo00@00?ooool008goo`00Sooo0P0000?oo`00ool0S?oo002?ool01000oooo
o`00SOoo002?ool01000ooooo`00SOoo002?ool01@00oooooooo0000S?oo002?ool01@00oooooooo
0000S?oo002?ool20002ool00`00ooooo`2:ool008ooo`05003oooooool0002<ool008ooo`05003o
ooooool0002<ool008ooo`05003oooooool0002<ool008ooo`8000;oo`03003ooooo08[oo`00Sooo
00D00?ooooooo`0008coo`00Sooo00D00?ooooooo`0008coo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.423173, 0.975159, \
0.0040197, 0.000222309}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %692]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.313831 0.812612 -13.5779 13.9658 [
[.01121 .37534 -9 -9 ]
[.01121 .37534 9 0 ]
[.17374 .37534 -9 -9 ]
[.17374 .37534 9 0 ]
[.33626 .37534 -9 -9 ]
[.33626 .37534 9 0 ]
[.6613 .37534 -9 -9 ]
[.6613 .37534 9 0 ]
[.82383 .37534 -9 -9 ]
[.82383 .37534 9 0 ]
[.98635 .37534 -9 -9 ]
[.98635 .37534 9 0 ]
[.48628 .10853 -24 -4.5 ]
[.48628 .10853 0 4.5 ]
[.48628 .24819 -24 -4.5 ]
[.48628 .24819 0 4.5 ]
[.48628 .5275 -24 -4.5 ]
[.48628 .5275 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01121 .38784 m
.01121 .39409 L
s
[(0.4)] .01121 .37534 0 1 Mshowa
.17374 .38784 m
.17374 .39409 L
s
[(0.6)] .17374 .37534 0 1 Mshowa
.33626 .38784 m
.33626 .39409 L
s
[(0.8)] .33626 .37534 0 1 Mshowa
.6613 .38784 m
.6613 .39409 L
s
[(1.2)] .6613 .37534 0 1 Mshowa
.82383 .38784 m
.82383 .39409 L
s
[(1.4)] .82383 .37534 0 1 Mshowa
.98635 .38784 m
.98635 .39409 L
s
[(1.6)] .98635 .37534 0 1 Mshowa
.125 Mabswid
.05184 .38784 m
.05184 .39159 L
s
.09248 .38784 m
.09248 .39159 L
s
.13311 .38784 m
.13311 .39159 L
s
.21437 .38784 m
.21437 .39159 L
s
.255 .38784 m
.255 .39159 L
s
.29563 .38784 m
.29563 .39159 L
s
.37689 .38784 m
.37689 .39159 L
s
.41752 .38784 m
.41752 .39159 L
s
.45815 .38784 m
.45815 .39159 L
s
.53941 .38784 m
.53941 .39159 L
s
.58004 .38784 m
.58004 .39159 L
s
.62067 .38784 m
.62067 .39159 L
s
.70193 .38784 m
.70193 .39159 L
s
.74256 .38784 m
.74256 .39159 L
s
.7832 .38784 m
.7832 .39159 L
s
.86446 .38784 m
.86446 .39159 L
s
.90509 .38784 m
.90509 .39159 L
s
.94572 .38784 m
.94572 .39159 L
s
.25 Mabswid
0 .38784 m
1 .38784 L
s
.49878 .10853 m
.50503 .10853 L
s
[(0.98)] .48628 .10853 1 0 Mshowa
.49878 .24819 m
.50503 .24819 L
s
[(0.99)] .48628 .24819 1 0 Mshowa
.49878 .5275 m
.50503 .5275 L
s
[(1.01)] .48628 .5275 1 0 Mshowa
.125 Mabswid
.49878 .13646 m
.50253 .13646 L
s
.49878 .16439 m
.50253 .16439 L
s
.49878 .19232 m
.50253 .19232 L
s
.49878 .22026 m
.50253 .22026 L
s
.49878 .27612 m
.50253 .27612 L
s
.49878 .30405 m
.50253 .30405 L
s
.49878 .33198 m
.50253 .33198 L
s
.49878 .35991 m
.50253 .35991 L
s
.49878 .41578 m
.50253 .41578 L
s
.49878 .44371 m
.50253 .44371 L
s
.49878 .47164 m
.50253 .47164 L
s
.49878 .49957 m
.50253 .49957 L
s
.49878 .0806 m
.50253 .0806 L
s
.49878 .05267 m
.50253 .05267 L
s
.49878 .02473 m
.50253 .02473 L
s
.49878 .55543 m
.50253 .55543 L
s
.49878 .58337 m
.50253 .58337 L
s
.49878 .6113 m
.50253 .6113 L
s
.25 Mabswid
.49878 0 m
.49878 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
[ .02 .02 ] 0 setdash
.43736 0 m
.45896 .24082 L
s
.45896 .24082 m
.48918 .61803 L
s
[ ] 0 setdash
.33626 .13477 m
.33707 .13688 L
.33788 .13899 L
.3387 .14108 L
.33951 .14317 L
.34032 .14525 L
.34113 .14733 L
.34195 .14939 L
.34276 .15145 L
.34357 .1535 L
.34438 .15554 L
.3452 .15758 L
.34601 .1596 L
.34682 .16162 L
.34764 .16363 L
.34845 .16563 L
.34926 .16762 L
.35007 .1696 L
.35089 .17158 L
.3517 .17355 L
.35251 .17551 L
.35332 .17746 L
.35414 .1794 L
.35495 .18133 L
.35576 .18326 L
.35657 .18518 L
.35739 .18709 L
.3582 .18899 L
.35901 .19088 L
.35982 .19277 L
.36064 .19464 L
.36145 .19651 L
.36226 .19837 L
.36307 .20022 L
.36389 .20207 L
.3647 .2039 L
.36551 .20572 L
.36633 .20754 L
.36714 .20935 L
.36795 .21115 L
.36876 .21294 L
.36958 .21472 L
.37039 .2165 L
.3712 .21826 L
.37201 .22002 L
.37283 .22177 L
.37364 .22351 L
.37445 .22524 L
.37526 .22696 L
.37608 .22867 L
Mistroke
.37689 .23038 L
.3777 .23208 L
.37851 .23376 L
.37933 .23544 L
.38014 .23711 L
.38095 .23877 L
.38176 .24042 L
.38258 .24207 L
.38339 .2437 L
.3842 .24533 L
.38502 .24694 L
.38583 .24855 L
.38664 .25015 L
.38745 .25174 L
.38827 .25332 L
.38908 .2549 L
.38989 .25646 L
.3907 .25801 L
.39152 .25956 L
.39233 .26109 L
.39314 .26262 L
.39395 .26414 L
.39477 .26565 L
.39558 .26715 L
.39639 .26864 L
.3972 .27012 L
.39802 .2716 L
.39883 .27306 L
.39964 .27451 L
.40046 .27596 L
.40127 .2774 L
.40208 .27882 L
.40289 .28024 L
.40371 .28165 L
.40452 .28305 L
.40533 .28444 L
.40614 .28582 L
.40696 .28719 L
.40777 .28855 L
.40858 .28991 L
.40939 .29125 L
.41021 .29259 L
.41102 .29391 L
.41183 .29523 L
.41264 .29653 L
.41346 .29783 L
.41427 .29912 L
.41508 .3004 L
.41589 .30167 L
.41671 .30293 L
Mistroke
.41752 .30418 L
.41833 .30542 L
.41915 .30665 L
.41996 .30787 L
.42077 .30909 L
.42158 .31029 L
.4224 .31148 L
.42321 .31267 L
.42402 .31384 L
.42483 .31501 L
.42565 .31616 L
.42646 .31731 L
.42727 .31844 L
.42808 .31957 L
.4289 .32069 L
.42971 .3218 L
.43052 .32289 L
.43133 .32398 L
.43215 .32506 L
.43296 .32613 L
.43377 .32719 L
.43458 .32824 L
.4354 .32928 L
.43621 .33031 L
.43702 .33133 L
.43784 .33235 L
.43865 .33335 L
.43946 .33434 L
.44027 .33532 L
.44109 .3363 L
.4419 .33726 L
.44271 .33821 L
.44352 .33916 L
.44434 .34009 L
.44515 .34102 L
.44596 .34193 L
.44677 .34284 L
.44759 .34373 L
.4484 .34462 L
.44921 .34549 L
.45002 .34636 L
.45084 .34722 L
.45165 .34806 L
.45246 .3489 L
.45327 .34973 L
.45409 .35055 L
.4549 .35135 L
.45571 .35215 L
.45653 .35294 L
.45734 .35372 L
Mistroke
.45815 .35449 L
.45896 .35525 L
.45978 .356 L
.46059 .35674 L
.4614 .35747 L
.46221 .35819 L
.46303 .3589 L
.46384 .3596 L
.46465 .36029 L
.46546 .36097 L
.46628 .36164 L
.46709 .3623 L
.4679 .36296 L
.46871 .3636 L
.46953 .36423 L
.47034 .36485 L
.47115 .36547 L
.47196 .36607 L
.47278 .36667 L
.47359 .36725 L
.4744 .36782 L
.47522 .36839 L
.47603 .36894 L
.47684 .36949 L
.47765 .37003 L
.47847 .37055 L
.47928 .37107 L
.48009 .37158 L
.4809 .37207 L
.48172 .37256 L
.48253 .37304 L
.48334 .37351 L
.48415 .37397 L
.48497 .37442 L
.48578 .37486 L
.48659 .37529 L
.4874 .37571 L
.48822 .37612 L
.48903 .37652 L
.48984 .37691 L
.49065 .3773 L
.49147 .37767 L
.49228 .37804 L
.49309 .37839 L
.49391 .37874 L
.49472 .37907 L
.49553 .3794 L
.49634 .37971 L
.49716 .38002 L
.49797 .38032 L
Mistroke
.49878 .38061 L
.49959 .38089 L
.50041 .38116 L
.50122 .38142 L
.50203 .38168 L
.50284 .38192 L
.50366 .38215 L
.50447 .38238 L
.50528 .38259 L
.50609 .3828 L
.50691 .383 L
.50772 .38318 L
.50853 .38336 L
.50935 .38353 L
.51016 .3837 L
.51097 .38385 L
.51178 .38399 L
.5126 .38413 L
.51341 .38425 L
.51422 .38437 L
.51503 .38448 L
.51585 .38457 L
.51666 .38467 L
.51747 .38475 L
.51828 .38482 L
.5191 .38488 L
.51991 .38494 L
.52072 .38499 L
.52153 .38502 L
.52235 .38505 L
.52316 .38507 L
.52397 .38509 L
.52478 .38509 L
.5256 .38509 L
.52641 .38507 L
.52722 .38505 L
.52804 .38502 L
.52885 .38498 L
.52966 .38494 L
.53047 .38488 L
.53129 .38482 L
.5321 .38475 L
.53291 .38467 L
.53372 .38458 L
.53454 .38449 L
.53535 .38439 L
.53616 .38428 L
.53697 .38416 L
.53779 .38403 L
.5386 .38389 L
Mistroke
.53941 .38375 L
.54022 .3836 L
.54104 .38344 L
.54185 .38328 L
.54266 .3831 L
.54347 .38292 L
.54429 .38273 L
.5451 .38253 L
.54591 .38233 L
.54673 .38212 L
.54754 .3819 L
.54835 .38167 L
.54916 .38144 L
.54998 .3812 L
.55079 .38095 L
.5516 .38069 L
.55241 .38043 L
.55323 .38016 L
.55404 .37988 L
.55485 .3796 L
.55566 .37931 L
.55648 .37901 L
.55729 .3787 L
.5581 .37839 L
.55891 .37807 L
.55973 .37775 L
.56054 .37741 L
.56135 .37707 L
.56216 .37673 L
.56298 .37638 L
.56379 .37602 L
.5646 .37565 L
.56542 .37528 L
.56623 .3749 L
.56704 .37452 L
.56785 .37413 L
.56867 .37373 L
.56948 .37333 L
.57029 .37292 L
.5711 .3725 L
.57192 .37208 L
.57273 .37165 L
.57354 .37122 L
.57435 .37078 L
.57517 .37033 L
.57598 .36988 L
.57679 .36942 L
.5776 .36896 L
.57842 .36849 L
.57923 .36802 L
Mistroke
.58004 .36754 L
.58085 .36705 L
.58167 .36656 L
.58248 .36606 L
.58329 .36556 L
.58411 .36505 L
.58492 .36454 L
.58573 .36402 L
.58654 .3635 L
.58736 .36297 L
.58817 .36244 L
.58898 .3619 L
.58979 .36136 L
.59061 .36081 L
.59142 .36026 L
.59223 .3597 L
.59304 .35914 L
.59386 .35857 L
.59467 .358 L
.59548 .35742 L
.59629 .35684 L
.59711 .35625 L
.59792 .35566 L
.59873 .35507 L
.59954 .35447 L
.60036 .35386 L
.60117 .35325 L
.60198 .35264 L
.6028 .35203 L
.60361 .3514 L
.60442 .35078 L
.60523 .35015 L
.60605 .34952 L
.60686 .34888 L
.60767 .34824 L
.60848 .3476 L
.6093 .34695 L
.61011 .34629 L
.61092 .34564 L
.61173 .34498 L
.61255 .34432 L
.61336 .34365 L
.61417 .34298 L
.61498 .3423 L
.6158 .34163 L
.61661 .34095 L
.61742 .34026 L
.61824 .33957 L
.61905 .33888 L
.61986 .33819 L
Mistroke
.62067 .33749 L
.62149 .33679 L
.6223 .33609 L
.62311 .33538 L
.62392 .33467 L
.62474 .33396 L
.62555 .33324 L
.62636 .33253 L
.62717 .3318 L
.62799 .33108 L
.6288 .33035 L
.62961 .32963 L
.63042 .32889 L
.63124 .32816 L
.63205 .32742 L
.63286 .32668 L
.63367 .32594 L
.63449 .3252 L
.6353 .32445 L
.63611 .3237 L
.63693 .32295 L
.63774 .32219 L
.63855 .32144 L
.63936 .32068 L
.64018 .31992 L
.64099 .31916 L
.6418 .31839 L
.64261 .31763 L
.64343 .31686 L
.64424 .31609 L
.64505 .31532 L
.64586 .31454 L
.64668 .31377 L
.64749 .31299 L
.6483 .31221 L
.64911 .31143 L
.64993 .31064 L
.65074 .30986 L
.65155 .30907 L
.65236 .30829 L
.65318 .3075 L
.65399 .30671 L
.6548 .30591 L
.65562 .30512 L
.65643 .30432 L
.65724 .30353 L
.65805 .30273 L
.65887 .30193 L
.65968 .30113 L
.66049 .30033 L
Mistroke
.6613 .29953 L
Mfstroke
.47621 0 m
.49802 .37469 L
s
.49802 .37469 m
.51217 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00O_oo00<00?ooool02?oo00<00?ooool00ooo00<00?ooool0S_oo001nool00`00
ooooo`08ool00`00ooooo`03ool00`00ooooo`2>ool007koo`03003ooooo00Soo`03003ooooo00?o
o`03003ooooo08koo`00O_oo00<00?ooool02?oo00<00?ooool00ooo00<00?ooool0S_oo001nool0
0`00ooooo`08ool00`00ooooo`03ool00`00ooooo`2>ool007koo`03003ooooo00Soo`03003ooooo
00?oo`03003ooooo08koo`00ROoo00<00?ooool00ooo00<00?ooool0S_oo0029ool00`00ooooo`03
ool2002?ool008Woo`03003ooooo00?oo`03003ooooo08koo`00R_oo00<00?ooool00_oo00<00?oo
ool0S_oo002:ool00`00ooooo`02ool00`00ooooo`2>ool007ooo`03003ooooo00Soo`03003ooooo
00;oo`03003ooooo08koo`00Oooo00<00?ooool02?oo00<00?ooool00_oo00<00?ooool0S_oo001o
ool00`00ooooo`08ool00`00ooooo`02ool00`00ooooo`2>ool007ooo`03003ooooo00Soo`03003o
oooo00;oo`8008ooo`00Oooo00<00?ooool02?oo00<00?ooool00_oo00<00?ooool0S_oo001oool0
0`00ooooo`08ool00`00ooooo`02ool00`00ooooo`2>ool008[oo`03003ooooo00;oo`03003ooooo
08koo`00R_oo00<00?ooool00_oo00<00?ooool0S_oo002:ool00`00ooooo`02ool00`00ooooo`2>
ool008[oo`03003ooooo00;oo`03003ooooo08koo`00R_oo00<00?ooool00_oo00<00?ooool0S_oo
0020ool00`00ooooo`07ool00`00ooooo`02ool2002?ool0083oo`03003ooooo00Ooo`03003ooooo
00;oo`03003ooooo08koo`00P?oo00<00?ooool01ooo00<00?ooool00_oo00<00?ooool0S_oo0020
ool00`00ooooo`07ool00`00ooooo`02ool00`00ooooo`2>ool0083oo`03003ooooo00Soo`05003o
ooooool0002@ool0083oo`03003ooooo00Soo`05003oooooool0002@ool007Koo`8000Coo`03003o
oooo00;oo`<000Coo`800003ool00?oo00;oo`03003ooooo08koo`00MOoo00@00?ooool000_oo`04
003ooooo0002ool20003ool00`00ooooo`2>ool007Goo`04003ooooo000;ool01000ooooo`000_oo
0P000ooo0`00S_oo001eool01000ooooo`002Ooo0`000ooo0P0000?oo`00ool00_oo00<00?ooool0
S_oo001eool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo008000?oo`03003ooooo08ko
o`00MOoo00@00?ooool000Soo`04003ooooo0002ool00`00ooooo`020003ool00`00ooooo`2>ool0
07Koo`8000Woo`<000Coo`800003ool00?oo00;oo`03003ooooo08koo`00POoo00<00?ooool01ooo
00D00?ooooooo`00093oo`00POoo00<00?ooool01ooo00D00?ooooooo`00093oo`00H_oo00<00?oo
ool07?oo00<00?ooool01ooo00@00?ooooooo`8008ooo`00H_oo0P007Ooo00<00?ooool01ooo00D0
0?ooooooo`00093oo`00Hooo00<00?ooool09Ooo00D00?ooooooo`00093oo`00Hooo0P009_oo00D0
0?ooooooo`00093oo`00I?oo00<00?ooool09?oo00D00?ooooooo`00093oo`00I?oo00<00?ooool0
9?oo00D00?ooooooo`00093oo`00I?oo0P009_oo00@00?ooool0093oo`00IOoo00<00?ooool06_oo
00<00?ooool01ooo00@00?ooool0093oo`00IOoo0P006ooo00<00?ooool01ooo00<00?ooool00P00
Sooo001Vool00`00ooooo`0Iool00`00ooooo`07ool01000ooooo`00T?oo001Vool00`00ooooo`0I
ool00`00ooooo`07ool01000ooooo`00T?oo001Vool2000Jool00`00ooooo`07ool01000ooooo`00
T?oo001Wool00`00ooooo`0Hool00`00ooooo`07ool01000ooooo`00T?oo001Wool2000Sool01000
ooooo`00T?oo001Xool00`00ooooo`0Qool01000ooooo`00T?oo001Xool00`00ooooo`0Qool01000
ooooo`00T?oo001Yool00`00ooooo`0Pool00`00ooooo`02002?ool006Woo`03003ooooo023oo`04
003ooooo002@ool006Woo`8001Soo`03003ooooo00Koo`04003ooooo002@ool006[oo`03003ooooo
01Koo`03003ooooo00Koo`04003ooooo002@ool006[oo`03003ooooo01Koo`03003ooooo00Koo`04
003ooooo002@ool006_oo`03003ooooo01Goo`03003ooooo00Koo`04003ooooo002@ool006_oo`03
003ooooo01Goo`03003ooooo00Koo`04003ooooo002@ool006coo`03003ooooo01Coo`03003ooooo
00Koo`04003ooooo002@ool006coo`03003ooooo01koo`04003oo`00002?ool006goo`03003ooooo
01goo`03003oo`00093oo`00KOoo00<00?ooool07Ooo00<00?oo0000T?oo001^ool00`00ooooo`0L
ool00`00ool0002@ool006koo`03003ooooo01coo`03003oo`00093oo`00K_oo0P001_oo0P001?oo
00<00?ooool00_oo10000_oo0`000ooo00<00?oo0000T?oo001_ool00`00ooooo`03ool01000oooo
o`002ooo00<00?ooool00ooo00H00?ooool00?oo002@ool0073oo`03003ooooo00;oo`04003ooooo
000;ool00`00ooooo`03ool01@00ooooo`00ool00`00S_oo001`ool00`00ooooo`02ool01000oooo
o`002Ooo0`000ooo0`000_oo00<00?oo0000T?oo001aool01@00oooooooo00000_oo00<00?ooool0
1_oo00@00?ooool000;oo`04003ooooo0002ool00`00ool0002@ool0077oo`05003oooooool00002
ool00`00ooooo`06ool01000ooooo`000_oo00@00?ooool000;oo`03003oo`00093oo`00L_oo00@0
0?ooooooo`8000[oo`<000?oo`8000?oo`03003oo`00093oo`00L_oo0P006Ooo00<00?oo0000T?oo
001cool00`00ooooo`0Gool00`00ool0002@ool007?oo`03003ooooo01Ooo`03003oo`00093oo`00
M?oo00<00?ooool05_oo00@00?oo000008ooo`00M?oo0P003ooo00<00?ooool01Ooo00<00?oo0000
T?oo001eool00`00ooooo`0=ool00`00ooooo`06ool2002@ool007Goo`8000koo`03003ooooo00Ko
o`80093oo`00M_oo0P003Ooo00<00?ooool01_oo0P00T?oo001gool00`00ooooo`0;ool00`00oooo
o`06ool2002@ool007Ooo`8000coo`03003ooooo00Koo`8002coo`03003ooooo067oo`00N?oo00<0
0?ooool02_oo00<00?ooool01_oo0P00:ooo0P00Hooo001iool00`00ooooo`0Bool3000Yool2001T
ool007Woo`8001?oo`8002Woo`8006Goo`00N_oo00<00?ooool04Ooo0P00:?oo00<00?ooool0IOoo
001kool00`00ooooo`0@ool2000Wool2001Wool007_oo`8000Woo`03003ooooo00Goo`8002Koo`03
003ooooo06Ooo`00O?oo0P002?oo00<00?ooool01Ooo0P009?oo0P00J_oo001mool00`00ooooo`06
ool00`00ooooo`05ool2000Tool00`00ooooo`1Yool007koo`03003ooooo00Goo`03003ooooo00Go
o`<0027oo`8006coo`00O_oo0P001_oo00<00?ooool01Ooo0P008Ooo0P00KOoo001oool20005ool0
0`00ooooo`05ool2000Pool2001^ool0083oo`8000Coo`03003ooooo00Goo`8001ooo`8006ooo`00
POoo0P003?oo00<00?ooool06ooo0`00L?oo0002ool20004ool00`00ooooo`04ool00`00ooooo`0L
ool20004ool00`00ooooo`03ool2000Oool20004ool00`00ooooo`03ool2000Iool2000;ool00`00
ooooo`0Jool20006ool40003ool00`00ooooo`02ool4000Mool40003ool00`00ooooo`04ool00`00
ooooo`0Lool40003ool00`00ooooo`03ool20003ool00005ool00?ooool0000:ool00`00ooooo`0K
ool01000ooooo`002?oo00@00?ooool001goo`04003ooooo0008ool01000ooooo`006Ooo0P002_oo
00<00?ooool06Ooo0P002Ooo00<00?ooool01ooo00<00?ooool08?oo00<00?ooool02Ooo00<00?oo
ool07_oo00<00?ooool01ooo00@00?ooool000;oo`0000Goo`00ooooo`0000Ooo`D001coo`04003o
oooo0008ool01000ooooo`007Ooo00@00?ooool000Soo`04003ooooo000Jool20009ool00`00oooo
o`0Hool2000:ool00`00ooooo`08ool00`00ooooo`0Oool00`00ooooo`06ool5000Oool00`00oooo
o`07ool01000ooooo`000_oo00001Ooo003ooooo00001ooo00@00?ooool001goo`04003ooooo0008
ool3000Nool01000ooooo`002Ooo0P007?oo0`001ooo0P005ooo0P003?oo00<00?ooool02Ooo00<0
0?ooool07_oo00<00?ooool01_oo00@00?ooool0023oo`03003ooooo00Ooo`<000?oo`0000Goo`00
ooooo`0000Soo`03003oo`0001goo`04003ooooo0008ool00`00ooooo`0Nool01000ooooo`002?oo
00@00?ooool001coo`8000Ooo`03003ooooo01Goo`8000goo`03003ooooo00[oo`03003ooooo01go
o`03003ooooo00Ooo`03003oo`00023oo`03003ooooo00Ooo`03003ooooo00?oo`0000Goo`00oooo
o`0000Woo`8001goo`04003ooooo0008ool00`00ooooo`0Nool01000ooooo`002?oo00@00?ooool0
01goo`<000Goo`03003ooooo01?oo`8000goo`<000Woo`04003ooooo000Mool3000:ool2000Nool3
0009ool00`00ooooo`03ool000;oo`8000_oo`03003ooooo01coo`8000[oo`<001koo`8000[oo`80
01koo`04003oo`000004ool00`00ooooo`0Aool3000@ool00`00ooooo`08ool2000Pool00`00oooo
o`09ool00`00ooooo`0Nool00`00ooooo`08ool30002ool008Ooo`03003ooooo00<000;oo`03003o
oooo00ooo`<007coo`00Qooo00<00?ooool00_oo10003ooo0`00O_oo0027ool00`00ooooo`04ool4
0009ool40021ool008ooo`03003ooooo00X008Coo`001Oooo`005P001_oo0009ool00`00ooooo`08
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`08
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`08
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`08
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`08
ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`07ool008ooo`03003o
oooo08koo`00R?oo00<00?ooool01?oo00<00?ooool0S_oo0028ool00`00ooooo`04ool00`00oooo
o`2>ool008Soo`03003ooooo00Coo`8008ooo`00R?oo00<00?ooool01?oo0P00Sooo0028ool00`00
ooooo`04ool2002?ool008Soo`03003ooooo00Coo`8008ooo`00R?oo00<00?ooool01?oo0P00Sooo
002?ool2002?ool008ooo`8008ooo`00Sooo0P00Sooo002?ool2002?ool008Soo`03003ooooo00Co
o`8008ooo`00R?oo00<00?ooool01?oo0P00Sooo0028ool00`00ooooo`04ool2002?ool008Soo`03
003ooooo00Coo`8008ooo`00ROoo00<00?ooool00ooo0P00Sooo0029ool00`00ooooo`03ool2002?
ool008Woo`03003ooooo00?oo`8008ooo`00Sooo0P00Sooo002?ool00`00ool0002>ool008ooo`<0
08koo`00Sooo00<00?oo0000S_oo0029ool00`00ooooo`03ool00`00ool0002>ool008Woo`03003o
oooo00?oo`03003oo`0008koo`00ROoo00<00?ooool00ooo00<00?oo0000S_oo0029ool00`00oooo
o`03ool00`00ool0002>ool008[oo`03003ooooo00;oo`03003oo`0008koo`00R_oo00<00?ooool0
0_oo00<00?oo0000S_oo002:ool00`00ooooo`02ool3002>ool008ooo`03003oo`0008koo`00Sooo
00<00?oo0000S_oo002?ool00`00ool0002>ool008ooo`03003oo`0008koo`00MOoo10000ooo00<0
0?ooool00ooo0P000ooo10001?oo00<00?oo0000S_oo001gool00`00ooooo`07ool01000ooooo`00
1?oo0P001?oo00<00?oo0000S_oo001gool00`00ooooo`07ool01000ooooo`001?oo0P001?oo1000
SOoo001gool00`00ooooo`07ool01000ooooo`001?oo0P001?oo00@00?ooool008goo`00Mooo00<0
0?ooool01ooo00@00?ooool000Coo`03003oo`0000?oo`04003ooooo002=ool007Goo`<000Woo`04
003ooooo0002ool300000ooo003oo`02ool01000ooooo`00SOoo001gool00`00ooooo`08ool20005
ool00`00ool00003ool01000ooooo`00SOoo002?ool01000ooooo`00SOoo002?ool01000ooooo`00
SOoo002?ool01000ooooo`00SOoo002?ool200000ooo003oo`2<ool008_oo`05003oooooool00002
ool00`00ooooo`2;ool008_oo`05003oooooool00002ool00`00ooooo`2;ool008_oo`05003ooooo
ool00002ool00`00ooooo`2;ool008_oo`05003oooooool00002ool00`00ooooo`2;ool008_oo`05
003oooooool00002ool00`00ooooo`2;ool008_oo`05003oooooool00002ool00`00ooooo`2;ool0
08_oo`05003oooooool00002ool00`00ooooo`2;ool008ooo`800003ool00?oo08coo`00Sooo00D0
0?ooooooo`0008coo`00Sooo00D00?ooooooo`0008coo`00Sooo00D00?ooooooo`0008coo`00S?oo
00@00?ooool000?oo`03003ooooo08[oo`00S?oo00@00?ooool000?oo`03003ooooo08[oo`00S?oo
00@00?ooool000?oo`03003ooooo08[oo`00S?oo00<00?ooool00P000_oo00<00?ooool0R_oo002<
ool01000ooooo`000ooo00<00?ooool0R_oo002<ool01000ooooo`000ooo00<00?ooool0R_oo003o
oolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.35984, 0.971388, \
0.00446104, 0.000259569}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[kx0corrtestfunc[Om, 0.8, 10]], {Om, 0.8, 1.5}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.06463 1.36054 0.0338583 0.385842 [
[.02381 .02136 -9 -9 ]
[.02381 .02136 9 0 ]
[.15986 .02136 -9 -9 ]
[.15986 .02136 9 0 ]
[.43197 .02136 -9 -9 ]
[.43197 .02136 9 0 ]
[.56803 .02136 -9 -9 ]
[.56803 .02136 9 0 ]
[.70408 .02136 -9 -9 ]
[.70408 .02136 9 0 ]
[.84014 .02136 -9 -9 ]
[.84014 .02136 9 0 ]
[.97619 .02136 -9 -9 ]
[.97619 .02136 9 0 ]
[.28342 .13032 -24 -4.5 ]
[.28342 .13032 0 4.5 ]
[.28342 .22678 -18 -4.5 ]
[.28342 .22678 0 4.5 ]
[.28342 .32324 -24 -4.5 ]
[.28342 .32324 0 4.5 ]
[.28342 .4197 -6 -4.5 ]
[.28342 .4197 0 4.5 ]
[.28342 .51616 -24 -4.5 ]
[.28342 .51616 0 4.5 ]
[.28342 .61262 -18 -4.5 ]
[.28342 .61262 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .03386 m
.02381 .04011 L
s
[(0.8)] .02381 .02136 0 1 Mshowa
.15986 .03386 m
.15986 .04011 L
s
[(0.9)] .15986 .02136 0 1 Mshowa
.43197 .03386 m
.43197 .04011 L
s
[(1.1)] .43197 .02136 0 1 Mshowa
.56803 .03386 m
.56803 .04011 L
s
[(1.2)] .56803 .02136 0 1 Mshowa
.70408 .03386 m
.70408 .04011 L
s
[(1.3)] .70408 .02136 0 1 Mshowa
.84014 .03386 m
.84014 .04011 L
s
[(1.4)] .84014 .02136 0 1 Mshowa
.97619 .03386 m
.97619 .04011 L
s
[(1.5)] .97619 .02136 0 1 Mshowa
.125 Mabswid
.05102 .03386 m
.05102 .03761 L
s
.07823 .03386 m
.07823 .03761 L
s
.10544 .03386 m
.10544 .03761 L
s
.13265 .03386 m
.13265 .03761 L
s
.18707 .03386 m
.18707 .03761 L
s
.21429 .03386 m
.21429 .03761 L
s
.2415 .03386 m
.2415 .03761 L
s
.26871 .03386 m
.26871 .03761 L
s
.32313 .03386 m
.32313 .03761 L
s
.35034 .03386 m
.35034 .03761 L
s
.37755 .03386 m
.37755 .03761 L
s
.40476 .03386 m
.40476 .03761 L
s
.45918 .03386 m
.45918 .03761 L
s
.48639 .03386 m
.48639 .03761 L
s
.51361 .03386 m
.51361 .03761 L
s
.54082 .03386 m
.54082 .03761 L
s
.59524 .03386 m
.59524 .03761 L
s
.62245 .03386 m
.62245 .03761 L
s
.64966 .03386 m
.64966 .03761 L
s
.67687 .03386 m
.67687 .03761 L
s
.73129 .03386 m
.73129 .03761 L
s
.7585 .03386 m
.7585 .03761 L
s
.78571 .03386 m
.78571 .03761 L
s
.81293 .03386 m
.81293 .03761 L
s
.86735 .03386 m
.86735 .03761 L
s
.89456 .03386 m
.89456 .03761 L
s
.92177 .03386 m
.92177 .03761 L
s
.94898 .03386 m
.94898 .03761 L
s
.25 Mabswid
0 .03386 m
1 .03386 L
s
.29592 .13032 m
.30217 .13032 L
s
[(0.25)] .28342 .13032 1 0 Mshowa
.29592 .22678 m
.30217 .22678 L
s
[(0.5)] .28342 .22678 1 0 Mshowa
.29592 .32324 m
.30217 .32324 L
s
[(0.75)] .28342 .32324 1 0 Mshowa
.29592 .4197 m
.30217 .4197 L
s
[(1)] .28342 .4197 1 0 Mshowa
.29592 .51616 m
.30217 .51616 L
s
[(1.25)] .28342 .51616 1 0 Mshowa
.29592 .61262 m
.30217 .61262 L
s
[(1.5)] .28342 .61262 1 0 Mshowa
.125 Mabswid
.29592 .05315 m
.29967 .05315 L
s
.29592 .07244 m
.29967 .07244 L
s
.29592 .09173 m
.29967 .09173 L
s
.29592 .11103 m
.29967 .11103 L
s
.29592 .14961 m
.29967 .14961 L
s
.29592 .1689 m
.29967 .1689 L
s
.29592 .1882 m
.29967 .1882 L
s
.29592 .20749 m
.29967 .20749 L
s
.29592 .24607 m
.29967 .24607 L
s
.29592 .26536 m
.29967 .26536 L
s
.29592 .28466 m
.29967 .28466 L
s
.29592 .30395 m
.29967 .30395 L
s
.29592 .34253 m
.29967 .34253 L
s
.29592 .36182 m
.29967 .36182 L
s
.29592 .38112 m
.29967 .38112 L
s
.29592 .40041 m
.29967 .40041 L
s
.29592 .43899 m
.29967 .43899 L
s
.29592 .45828 m
.29967 .45828 L
s
.29592 .47758 m
.29967 .47758 L
s
.29592 .49687 m
.29967 .49687 L
s
.29592 .53545 m
.29967 .53545 L
s
.29592 .55474 m
.29967 .55474 L
s
.29592 .57404 m
.29967 .57404 L
s
.29592 .59333 m
.29967 .59333 L
s
.29592 .01457 m
.29967 .01457 L
s
.25 Mabswid
.29592 0 m
.29592 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02455 .61803 m
.02499 .38501 L
.02605 .34387 L
.02729 .31527 L
.02846 .29595 L
.03053 .27063 L
.03279 .25019 L
.0379 .21704 L
.04262 .19485 L
.05224 .16175 L
.06244 .13625 L
.0825 .10074 L
.0939 .08574 L
.10458 .07399 L
.12312 .05755 L
.14335 .04394 L
.16449 .03335 L
.18458 .02597 L
.20425 .02081 L
.21525 .0187 L
.22522 .0172 L
.2301 .01661 L
.23525 .01608 L
.2397 .01569 L
.24447 .01535 L
.2496 .01507 L
.2519 .01497 L
.25439 .01488 L
.25591 .01484 L
.25732 .0148 L
.25864 .01477 L
.26009 .01475 L
.26139 .01473 L
.26261 .01472 L
.2639 .01472 L
.26462 .01472 L
.26529 .01472 L
.26657 .01472 L
.26777 .01473 L
.26885 .01474 L
.27003 .01475 L
.27131 .01477 L
.27266 .0148 L
.27509 .01486 L
.27943 .015 L
.28421 .01522 L
.28937 .01553 L
.29491 .01593 L
.30491 .01684 L
.32609 .01951 L
Mistroke
.34619 .0229 L
.38443 .03143 L
.42513 .04311 L
.46431 .05655 L
.50595 .07291 L
.54607 .09048 L
.58467 .10891 L
.62572 .13002 L
.66526 .1517 L
.70329 .17372 L
.74376 .19834 L
.78273 .22314 L
.82414 .25063 L
.86404 .27815 L
.90242 .30557 L
.94325 .3357 L
.97619 .36072 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000Coo`8000Coo`03003ooooo00?oo`8001Soo`8000Coo`03003ooooo00;oo`<003coo`@000?o
o`03003ooooo00;oo`@001Koo`@000?oo`03003ooooo00;oo`@001Goo`@000?oo`03003ooooo00?o
o`8001Ooo`@000?oo`03003ooooo00Coo`03003ooooo01Goo`@000?oo`03003ooooo00;oo`<000Ko
o`000ooo00@00?ooool000Soo`04003ooooo000Fool01000ooooo`002ooo00<00?ooool07?oo00<0
0?ooool07?oo00<00?ooool02Ooo00<00?ooool05ooo00<00?ooool01ooo00<00?ooool06?oo00<0
0?ooool01ooo00@00?ooool001Soo`03003ooooo00Woo`03003ooooo01Ooo`03003ooooo00[oo`03
003ooooo00?oo`000ooo00@00?ooool000Soo`04003ooooo000Fool01000ooooo`002ooo00<00?oo
ool07?oo00<00?ooool07?oo00<00?ooool02Ooo00<00?ooool05ooo00<00?ooool02?oo00<00?oo
ool05ooo00<00?ooool02_oo00<00?ooool05_oo00<00?ooool01_oo1@006?oo00<00?ooool02_oo
00<00?ooool00ooo0003ool01000ooooo`002Ooo0P005ooo00@00?ooool000Woo`<001koo`03003o
oooo01coo`03003ooooo00Woo`03003ooooo01Ooo`03003ooooo00Woo`03003ooooo01Koo`03003o
oooo00Soo`8001Woo`03003ooooo00Koo`04003ooooo000Iool00`00ooooo`07ool30006ool000?o
o`04003ooooo0008ool01000ooooo`005_oo00@00?ooool000Soo`04003ooooo000Nool00`00oooo
o`0Lool00`00ooooo`09ool00`00ooooo`0Gool00`00ooooo`0:ool00`00ooooo`0Eool00`00oooo
o`0:ool00`00ooooo`0Fool00`00ooooo`07ool00`00ool0000Iool00`00ooooo`07ool00`00oooo
o`06ool000?oo`04003ooooo0008ool01000ooooo`005_oo00@00?ooool000Soo`04003ooooo0009
oolJ000Hool30009ool3000Gool30009ool01000ooooo`005Ooo0`002Ooo00@00?ooool001Koo`<0
00[oo`8001Ooo`<000Woo`03003ooooo00Koo`001?oo0P002_oo0P006?oo0P002_oo0P001?oo1P00
5Ooo00<00?ooool00_oo1P005?oo00<00?ooool02Ooo00<00?ooool05ooo00<00?ooool02?oo0P00
6?oo00<00?ooool02?oo0P006Ooo00<00?ooool02Ooo00<00?ooool05ooo00<00?ooool01ooo1000
1Ooo000jool2000Kool00`00ooooo`08ool5002iool003Ooo`<001goo`03003ooooo00goo`H00;?o
o`00=?oo0`008?oo00<00?ooool04ooo1000[ooo0005oooo000F0006ool000_oo`03003ooooo00Go
o`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00?oo`<000Ooo`03003o
oooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003o
oooo00Goo`03003ooooo00Coo`03003ooooo00Coo`8000Koo`03003ooooo00Goo`03003ooooo00Co
o`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Go
o`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Co
o`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Co
o`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00[oo`002ooo00<00?oo
ool07ooo0P000_oo00<00?ooool08ooo00<00?ooool07?oo0`000ooo00<00?ooool08ooo00<00?oo
ool08_oo00<00?ooool08ooo00<00?ooool08ooo00<00?ooool02_oo000/ool00`00ooooo`0Xool0
0`00ooooo`0Oool3002Tool002_oo`03003ooooo02Woo`03003ooooo02;oo`@00:3oo`00:Ooo0P00
;?oo0P009ooo1000W?oo000Xool00`00ooooo`0/ool00`00ooooo`0Zool3002Iool002Ooo`03003o
oooo02goo`03003ooooo02goo`8009Ooo`009_oo00<00?ooool0;_oo00<00?ooool0;ooo0P00UOoo
000Tool2000aool00`00ooooo`0aool3002Bool002?oo`03003ooooo037oo`03003ooooo03Coo`80
093oo`008_oo00<00?ooool0<_oo0P00=ooo0`00SOoo000Qool00`00ooooo`0cool00`00ooooo`0i
ool2002;ool0023oo`03003ooooo03Coo`03003ooooo03_oo`8008Woo`007ooo00<00?ooool0=Ooo
00<00?ooool0?Ooo0P00Qooo000Nool00`00ooooo`0fool00`00ooooo`0oool20025ool001koo`03
003ooooo03Koo`8004;oo`8008?oo`007Ooo00<00?ooool0=ooo00<00?ooool0@ooo0P00POoo000M
ool00`00ooooo`0gool00`00ooooo`15ool2001oool001coo`03003ooooo03Soo`03003ooooo04Oo
o`8007goo`006ooo00<00?ooool0>Ooo00<00?ooool0BOoo0P00Nooo000Kool00`00ooooo`0iool2
001<ool2001iool001[oo`03003ooooo03[oo`03003ooooo04goo`8007Ooo`006Ooo00<00?ooool0
>ooo00<00?ooool0Cooo0P00MOoo000Iool00`00ooooo`0kool00`00ooooo`1Aool2001cool001So
o`03003ooooo02;oo`8000Coo`03003ooooo00;oo`@000;oo`<000Koo`03003ooooo05?oo`80077o
o`005ooo00<00?ooool08_oo00@00?ooool000Soo`03003ooooo00Koo`03003ooooo00?oo`03003o
oooo05Goo`8006ooo`005ooo00<00?ooool08_oo00@00?ooool000Woo`03003ooooo00Goo`03003o
oooo00?oo`8005Soo`8006goo`005_oo00<00?ooool08ooo00@00?ooool000[oo`04003oooooool3
0006ool00`00ooooo`1Iool2001[ool001Koo`03003ooooo02?oo`04003ooooo000;ool01000oooo
o`002?oo00<00?ooool0Fooo0P00JOoo000Eool00`00ooooo`0Tool01000ooooo`002?oo00@00?oo
ool000;oo`03003ooooo00Koo`03003ooooo05goo`8006Ooo`005Ooo00<00?ooool09Ooo0P002_oo
0P000ooo10001Ooo00<00?ooool0Gooo0P00IOoo000Dool00`00ooooo`10ool2001Rool2001Sool0
01Coo`03003ooooo043oo`03003ooooo06?oo`03003ooooo063oo`004ooo00<00?ooool0@Ooo00<0
0?ooool0I?oo0P00H?oo000Cool00`00ooooo`11ool00`00ooooo`1Vool2001Nool001?oo`03003o
oooo047oo`03003ooooo06Soo`03003ooooo05_oo`004ooo00<00?ooool0@Ooo0P00J_oo0P00Fooo
000Bool00`00ooooo`12ool00`00ooooo`1[ool2001Iool001;oo`03003ooooo04;oo`03003ooooo
06goo`03003ooooo05Koo`004_oo00<00?ooool0@_oo00<00?ooool0K_oo0P00E_oo000Bool00`00
ooooo`12ool00`00ooooo`1`ool2001Dool001;oo`03003ooooo04;oo`03003ooooo07;oo`8005;o
o`004Ooo00<00?ooool0@ooo0P00MOoo0P00D?oo000Aool00`00ooooo`13ool00`00ooooo`1fool2
001>ool0017oo`03003ooooo04?oo`03003ooooo07Soo`03003ooooo04_oo`004Ooo00<00?ooool0
@ooo00<00?ooool0NOoo0P00Booo000@ool00`00ooooo`14ool00`00ooooo`1kool00`00ooooo`18
ool0013oo`03003ooooo04Coo`8007goo`8004Soo`004?oo00<00?ooool0A?oo00<00?ooool0O_oo
0P00A_oo000?ool00`00ooooo`15ool00`00ooooo`20ool00`00ooooo`13ool000ooo`03003ooooo
037oo`8000Coo`03003ooooo00;oo`<000Koo`03003ooooo087oo`8004?oo`003ooo00<00?ooool0
<?oo00@00?ooool000_oo`03003ooooo00?oo`03003ooooo08?oo`03003ooooo043oo`003ooo00<0
0?ooool0<?oo00@00?ooool000_oo`03003ooooo00?oo`8008Goo`80043oo`003ooo00<00?ooool0
<?oo00@00?ooool000Soo`<000Koo`03003ooooo08Koo`03003ooooo03goo`003ooo00<00?ooool0
<?oo00@00?ooool000Soo`03003ooooo00Koo`03003ooooo08Ooo`8003goo`003_oo00<00?ooool0
<Ooo00@00?ooool000Soo`03003ooooo00Koo`03003ooooo08Woo`03003ooooo03[oo`003_oo00<0
0?ooool0<_oo0P002Ooo10001Ooo00<00?ooool0R_oo0P00>_oo000>ool00`00ooooo`16ool00`00
ooooo`2<ool00`00ooooo`0gool000koo`03003ooooo04Koo`8008koo`8003Ooo`003_oo00<00?oo
ool0A_oo00<00?ooool0Sooo00<00?ooool0=?oo000>ool00`00ooooo`16ool00`00ooooo`2@ool2
000dool000koo`03003ooooo04Koo`03003ooooo09;oo`03003ooooo037oo`003Ooo00<00?ooool0
Aooo00<00?ooool0Tooo00<00?ooool0<?oo000=ool00`00ooooo`17ool2002Eool2000`ool000go
o`03003ooooo04Ooo`03003ooooo09Koo`03003ooooo02goo`003Ooo00<00?ooool0Aooo00<00?oo
ool0Uooo00<00?ooool0;?oo000=ool00`00ooooo`17ool00`00ooooo`2Hool2000/ool000goo`03
003ooooo04Ooo`03003ooooo09[oo`03003ooooo02Woo`003Ooo00<00?ooool0Aooo0P00W?oo0P00
:Ooo000=ool00`00ooooo`17ool00`00ooooo`2Mool00`00ooooo`0Vool000goo`03003ooooo04Oo
o`03003ooooo09koo`03003ooooo02Goo`003Ooo00<00?ooool0Aooo00<00?ooool0Wooo0P009Ooo
000=ool00`00ooooo`17ool00`00ooooo`2Qool00`00ooooo`0Rool000goo`03003ooooo04Ooo`03
003ooooo0:;oo`8002;oo`003Ooo00<00?ooool0Aooo0P00YOoo00<00?ooool07ooo000<ool00`00
ooooo`18ool00`00ooooo`2Uool2000Oool000coo`03003ooooo04Soo`03003ooooo0:Ooo`03003o
oooo01coo`003?oo00<00?ooool0;_oo0P001?oo00<00?ooool00ooo00<00?ooool00_oo0`001_oo
00<00?ooool0Z?oo0P007?oo000<ool00`00ooooo`0]ool01000ooooo`002Ooo00<00?ooool01Ooo
00<00?ooool00ooo00<00?ooool0Z_oo00<00?ooool06Ooo000<ool00`00ooooo`0]ool01000oooo
o`002_oo00<00?ooool01?oo00<00?ooool00ooo0P00[?oo0P006Ooo000<ool00`00ooooo`0]ool0
1000ooooo`002_oo00@00?ooooooo`<000Koo`03003ooooo0:goo`03003ooooo01Koo`003?oo00<0
0?ooool0;Ooo00@00?ooool000_oo`04003ooooo0008ool00`00ooooo`2^ool2000Fool000coo`03
003ooooo02goo`04003ooooo0008ool01000ooooo`000_oo00<00?ooool01_oo00<00?ooool0/?oo
00<00?ooool04ooo000<ool00`00ooooo`0^ool20009ool40002ool40005ool00`00ooooo`2aool0
0`00ooooo`0Bool000coo`03003ooooo04Soo`800;?oo`8001;oo`003?oo00<00?ooool0B?oo00<0
0?ooool0]?oo00<00?ooool03ooo000<ool00`00ooooo`18ool00`00ooooo`2eool00`00ooooo`0>
ool000coo`03003ooooo04Soo`03003ooooo0;Koo`8000koo`003?oo00<00?ooool0B?oo00<00?oo
ool0^?oo00<00?ooool02ooo000<ool00`00ooooo`18ool00`00ooooo`2iool00`00ooooo`0:ool0
00coo`03003ooooo04Soo`800<Ooo`003?oo00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00
ooooo`18ool00`00ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?oo
ool0B?oo00<00?ooool0a_oo000<ool00`00ooooo`18ool20037ool000coo`03003ooooo04Soo`03
003ooooo0<Koo`003?oo00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00ooooo`18ool00`00
ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?ooool0B?oo0P00aooo
000<ool00`00ooooo`18ool00`00ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`00
3?oo00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00ooooo`0oool40005ool00`00ooooo`36
ool000coo`03003ooooo047oo`03003ooooo00Coo`03003ooooo0<Koo`003?oo00<00?ooool0@Ooo
00<00?ooool01?oo0P00aooo000<ool00`00ooooo`11ool00`00ooooo`04ool00`00ooooo`36ool0
00coo`03003ooooo047oo`03003ooooo00Coo`03003ooooo0<Koo`003?oo00<00?ooool0?ooo0`00
1_oo00<00?ooool0a_oo000<ool00`00ooooo`11ool00`00ooooo`04ool00`00ooooo`36ool000co
o`03003ooooo04Soo`800<Ooo`003?oo00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00oooo
o`18ool00`00ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?ooool0
B?oo00<00?ooool0a_oo000<ool00`00ooooo`18ool20037ool000coo`03003ooooo04Soo`03003o
oooo0<Koo`003?oo00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00ooooo`18ool00`00oooo
o`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?ooool0B?oo00<00?ooool0
a_oo000<ool00`00ooooo`18ool20037ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo
00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00ooooo`18ool00`00ooooo`36ool000coo`03
003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?ooool0B?oo0P00aooo000<ool00`00ooooo`18
ool00`00ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?ooool0;Ooo
10000ooo00<00?ooool00_oo10000_oo0`001_oo00<00?ooool0a_oo000<ool00`00ooooo`0_ool0
0`00ooooo`07ool00`00ooooo`06ool00`00ooooo`03ool00`00ooooo`36ool000coo`03003ooooo
02ooo`03003ooooo00Soo`03003ooooo00Goo`03003ooooo00?oo`800<Ooo`003?oo00<00?ooool0
;ooo00<00?ooool02Ooo00@00?ooooooo`<000Koo`03003ooooo0<Koo`003?oo00<00?ooool0;ooo
00<00?ooool02_oo00@00?ooool000Soo`03003ooooo0<Koo`003?oo00<00?ooool0;Ooo0`002Ooo
00@00?ooool000;oo`03003ooooo00Koo`03003ooooo0<Koo`003?oo00<00?ooool0;ooo00<00?oo
ool02?oo0P000ooo10001Ooo00<00?ooool0a_oo000<ool00`00ooooo`18ool00`00ooooo`36ool0
00coo`03003ooooo04Soo`800<Ooo`003?oo00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00
ooooo`18ool00`00ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?oo
ool0B?oo00<00?ooool0a_oo000<ool00`00ooooo`18ool20037ool000coo`03003ooooo04Soo`03
003ooooo0<Koo`003?oo00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00ooooo`18ool00`00
ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?ooool0B?oo0P00aooo
000<ool00`00ooooo`18ool00`00ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`00
3?oo00<00?ooool0B?oo00<00?ooool0a_oo000<ool00`00ooooo`18ool00`00ooooo`36ool000co
o`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?ooool0B?oo0P00aooo000<ool00`00oooo
o`18ool00`00ooooo`36ool000coo`03003ooooo04Soo`03003ooooo0<Koo`003?oo00<00?ooool0
<ooo10000ooo00<00?ooool00_oo0`001_oo00<00?ooool0a_oo000<ool00`00ooooo`0eool00`00
ooooo`0:ool00`00ooooo`03ool00`00ooooo`36ool000coo`03003ooooo03Goo`03003ooooo00[o
o`03003ooooo00?oo`800<Ooo`003?oo00<00?ooool0=Ooo00<00?ooool01ooo0`001_oo00<00?oo
ool0a_oo0014ool00`00ooooo`07ool00`00ooooo`3?ool004;oo`<000Woo`03003ooooo0<ooo`00
A?oo00<00?ooool01ooo1000c_oo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.768439, -0.116785, \
0.00265901, 0.00937609}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.8, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9818786433914661`, 
      mGamma \[Rule] \(-0.34475238584689466`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[kx0corrtestfunc[Om - 0.34475238584689466`\ I, 0.8, 10]], {Om, 0.8, 
        1.1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.51587 3.1746 0.0250843 5.78636 [
[.02381 .01258 -9 -9 ]
[.02381 .01258 9 0 ]
[.18254 .01258 -12 -9 ]
[.18254 .01258 12 0 ]
[.34127 .01258 -9 -9 ]
[.34127 .01258 9 0 ]
[.5 .01258 -12 -9 ]
[.5 .01258 12 0 ]
[.81746 .01258 -12 -9 ]
[.81746 .01258 12 0 ]
[.97619 .01258 -9 -9 ]
[.97619 .01258 9 0 ]
[.64623 .14081 -24 -4.5 ]
[.64623 .14081 0 4.5 ]
[.64623 .25654 -24 -4.5 ]
[.64623 .25654 0 4.5 ]
[.64623 .37227 -24 -4.5 ]
[.64623 .37227 0 4.5 ]
[.64623 .48799 -24 -4.5 ]
[.64623 .48799 0 4.5 ]
[.64623 .60372 -18 -4.5 ]
[.64623 .60372 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .02508 m
.02381 .03133 L
s
[(0.8)] .02381 .01258 0 1 Mshowa
.18254 .02508 m
.18254 .03133 L
s
[(0.85)] .18254 .01258 0 1 Mshowa
.34127 .02508 m
.34127 .03133 L
s
[(0.9)] .34127 .01258 0 1 Mshowa
.5 .02508 m
.5 .03133 L
s
[(0.95)] .5 .01258 0 1 Mshowa
.81746 .02508 m
.81746 .03133 L
s
[(1.05)] .81746 .01258 0 1 Mshowa
.97619 .02508 m
.97619 .03133 L
s
[(1.1)] .97619 .01258 0 1 Mshowa
.125 Mabswid
.05556 .02508 m
.05556 .02883 L
s
.0873 .02508 m
.0873 .02883 L
s
.11905 .02508 m
.11905 .02883 L
s
.15079 .02508 m
.15079 .02883 L
s
.21429 .02508 m
.21429 .02883 L
s
.24603 .02508 m
.24603 .02883 L
s
.27778 .02508 m
.27778 .02883 L
s
.30952 .02508 m
.30952 .02883 L
s
.37302 .02508 m
.37302 .02883 L
s
.40476 .02508 m
.40476 .02883 L
s
.43651 .02508 m
.43651 .02883 L
s
.46825 .02508 m
.46825 .02883 L
s
.53175 .02508 m
.53175 .02883 L
s
.56349 .02508 m
.56349 .02883 L
s
.59524 .02508 m
.59524 .02883 L
s
.62698 .02508 m
.62698 .02883 L
s
.69048 .02508 m
.69048 .02883 L
s
.72222 .02508 m
.72222 .02883 L
s
.75397 .02508 m
.75397 .02883 L
s
.78571 .02508 m
.78571 .02883 L
s
.84921 .02508 m
.84921 .02883 L
s
.88095 .02508 m
.88095 .02883 L
s
.9127 .02508 m
.9127 .02883 L
s
.94444 .02508 m
.94444 .02883 L
s
.25 Mabswid
0 .02508 m
1 .02508 L
s
.65873 .14081 m
.66498 .14081 L
s
[(0.02)] .64623 .14081 1 0 Mshowa
.65873 .25654 m
.66498 .25654 L
s
[(0.04)] .64623 .25654 1 0 Mshowa
.65873 .37227 m
.66498 .37227 L
s
[(0.06)] .64623 .37227 1 0 Mshowa
.65873 .48799 m
.66498 .48799 L
s
[(0.08)] .64623 .48799 1 0 Mshowa
.65873 .60372 m
.66498 .60372 L
s
[(0.1)] .64623 .60372 1 0 Mshowa
.125 Mabswid
.65873 .05402 m
.66248 .05402 L
s
.65873 .08295 m
.66248 .08295 L
s
.65873 .11188 m
.66248 .11188 L
s
.65873 .16974 m
.66248 .16974 L
s
.65873 .19867 m
.66248 .19867 L
s
.65873 .22761 m
.66248 .22761 L
s
.65873 .28547 m
.66248 .28547 L
s
.65873 .3144 m
.66248 .3144 L
s
.65873 .34333 m
.66248 .34333 L
s
.65873 .4012 m
.66248 .4012 L
s
.65873 .43013 m
.66248 .43013 L
s
.65873 .45906 m
.66248 .45906 L
s
.65873 .51692 m
.66248 .51692 L
s
.65873 .54586 m
.66248 .54586 L
s
.65873 .57479 m
.66248 .57479 L
s
.25 Mabswid
.65873 0 m
.65873 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.22591 .61803 m
.26171 .5149 L
.30316 .40982 L
.34309 .32182 L
.3815 .24879 L
.42237 .18299 L
.46172 .13065 L
.49955 .09006 L
.53984 .05685 L
.55838 .04491 L
.57861 .03421 L
.59975 .02558 L
.61028 .02223 L
.61984 .01974 L
.62997 .01765 L
.63951 .01619 L
.64476 .0156 L
.64958 .01519 L
.65211 .01503 L
.65488 .01489 L
.65628 .01483 L
.65778 .01479 L
.6585 .01477 L
.65919 .01475 L
.66048 .01473 L
.66162 .01472 L
.66285 .01472 L
.66401 .01472 L
.66507 .01473 L
.66625 .01474 L
.66754 .01477 L
.66888 .01481 L
.67013 .01485 L
.67273 .01497 L
.67509 .0151 L
.68039 .01552 L
.68494 .01599 L
.68987 .01662 L
.69881 .01809 L
.70828 .02009 L
.71869 .02281 L
.73973 .02997 L
.75918 .03853 L
.78007 .04975 L
.82135 .07793 L
.86111 .1124 L
.89935 .15207 L
.94005 .20105 L
.97619 .2502 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000Coo`8000Coo`03003ooooo00?oo`8001_oo`8000Coo`03003ooooo00?oo`8000?oo`<001_o
o`8000Coo`03003ooooo00;oo`<001_oo`8000Coo`03003ooooo00;oo`<000?oo`<004?oo`@000?o
o`03003ooooo00?oo`8000?oo`<001[oo`@000?oo`03003ooooo00;oo`@000Goo`000ooo00@00?oo
ool000Soo`04003ooooo000Iool01000ooooo`002?oo00@00?ooool000Goo`03003ooooo01Ooo`04
003ooooo000;ool00`00ooooo`0Gool01000ooooo`002ooo00<00?ooool00ooo00<00?ooool0@_oo
00<00?ooool01ooo00@00?ooool000Goo`03003ooooo01Woo`03003ooooo00Woo`03003ooooo00Co
o`000ooo00@00?ooool000Soo`04003ooooo000Iool01000ooooo`002?oo00@00?ooool000Goo`03
003ooooo01Ooo`04003ooooo000;ool00`00ooooo`0Gool01000ooooo`002ooo00<00?ooool00ooo
00<00?ooool0@_oo00<00?ooool01ooo00@00?ooool000Goo`03003ooooo01Woo`03003ooooo00Wo
o`03003ooooo00Coo`000ooo00@00?ooool000Woo`8001[oo`04003ooooo0009ool20003ool3000J
ool01000ooooo`002Ooo0`006Ooo00@00?ooool000Woo`<000;oo`<002;oo`03003ooooo023oo`03
003ooooo00Ooo`04003ooooo0002ool3000Lool00`00ooooo`09ool00`00ooooo`04ool000?oo`04
003ooooo0008ool01000ooooo`006Ooo00@00?ooool000Soo`04003ooooo0002ool00`00ooooo`0J
ool01000ooooo`002?oo00@00?ooool001Woo`04003ooooo0008ool01000ooooo`000_oo00<00?oo
ool08_oo00<00?ooool08?oo00<00?ooool01ooo00@00?ooool000;oo`03003ooooo01coo`03003o
oooo00Woo`03003ooooo00Coo`000ooo00@00?ooool000Soo`04003ooooo000Iool01000ooooo`00
2?oo00@00?ooool000;oo`03003ooooo01[oo`04003ooooo0008ool01000ooooo`006Ooo00@00?oo
ool000Soo`04003ooooo0002ool00`00ooooo`0Rool00`00ooooo`0Nool30009ool01000ooooo`00
0_oo00<00?ooool06_oo0`002Ooo0`001_oo0004ool2000:ool2000Kool2000:ool20003ool4000J
ool2000:ool2000Kool2000:ool20003ool4000Qool00`00ooooo`0Pool00`00ooooo`08ool20003
ool4000Kool00`00ooooo`09ool00`00ooooo`04ool00;Ooo`/005koo`00/_oo1@001?oo00<00?oo
ool01?oo1P00F?oo002/ool60009ool00`00ooooo`0:ool5001Cool000Goool001H000Koo`002ooo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01?oo0P002?oo00<00?ooool01_oo00<00?oo
ool01_oo00<00?ooool01Ooo00@00?ooooooo`8000?oo`03003ooooo00Koo`03003ooooo00Koo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03
003ooooo00[oo`002ooo00<00?ooool0:Ooo00<00?ooool0:Ooo00<00?ooool0:Ooo00<00?ooool0
5?oo0P004ooo00<00?ooool05?oo0P004ooo00<00?ooool0:Ooo00<00?ooool02_oo002Tool2000E
ool00`00ooooo`0Fool2001:ool00:7oo`<001Ooo`03003ooooo01Soo`8004Soo`00Wooo0P006_oo
00<00?ooool06_oo0P00A_oo002Nool00`00ooooo`0Jool00`00ooooo`0Lool20014ool009coo`80
01goo`03003ooooo01koo`03003ooooo047oo`00Vooo00<00?ooool07Ooo0P008?oo0P00@Ooo002J
ool00`00ooooo`0Nool00`00ooooo`0Qool00`00ooooo`0nool009Woo`03003ooooo01ooo`03003o
oooo02;oo`8003koo`00Uooo0P008_oo00<00?ooool09?oo00<00?ooool0>ooo002Fool00`00oooo
o`0Rool00`00ooooo`0Uool2000kool009Goo`03003ooooo02?oo`03003ooooo02Ooo`03003ooooo
03Soo`00U?oo00<00?ooool09?oo00<00?ooool0:?oo0P00>?oo002Cool00`00ooooo`0Uool00`00
ooooo`0Zool00`00ooooo`0eool0097oo`8002Soo`8002coo`03003ooooo03Coo`00T?oo00<00?oo
ool0:?oo00<00?ooool0;?oo0P00=?oo002?ool00`00ooooo`0Yool00`00ooooo`0^ool00`00oooo
o`0aool008koo`03003ooooo02[oo`03003ooooo02ooo`03003ooooo033oo`00SOoo00<00?ooool0
:ooo00<00?ooool0<?oo00<00?ooool0;ooo002<ool00`00ooooo`0/ool00`00ooooo`0aool00`00
ooooo`0^ool008_oo`03003ooooo02goo`03003ooooo03;oo`8002koo`00R_oo00<00?ooool0;_oo
00<00?ooool0=?oo00<00?ooool0:ooo0029ool00`00ooooo`0_ool2000fool00`00ooooo`0Zool0
08Soo`03003ooooo033oo`03003ooooo03Koo`03003ooooo02Woo`00Qooo00<00?ooool0<Ooo00<0
0?ooool0=ooo00<00?ooool0:?oo0026ool00`00ooooo`0bool00`00ooooo`0hool00`00ooooo`0W
ool008Goo`03003ooooo03?oo`03003ooooo03Woo`03003ooooo02Koo`00Q?oo00<00?ooool0=?oo
00<00?ooool0>_oo00<00?ooool09Ooo0023ool00`00ooooo`0Kool20004ool00`00ooooo`03ool2
0003ool40005ool00`00ooooo`0jool00`00ooooo`0Uool008?oo`03003ooooo01[oo`04003ooooo
0008ool01000ooooo`000_oo00<00?ooool01_oo00<00?ooool0>ooo00<00?ooool09?oo0022ool0
0`00ooooo`0Kool01000ooooo`002?oo00@00?ooool000?oo`03003ooooo00Goo`<003coo`03003o
oooo02?oo`00POoo00<00?ooool07?oo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`04
ool00`00ooooo`0mool00`00ooooo`0Rool0087oo`03003ooooo01coo`04003ooooo0008ool01000
ooooo`001Ooo00<00?ooool00ooo00<00?ooool0?_oo00<00?ooool08Ooo0020ool00`00ooooo`0M
ool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0005ool00`00ooooo`0oool00`00oooo
o`0Pool007ooo`03003ooooo01ooo`8000[oo`8000Coo`8000Koo`03003ooooo043oo`03003ooooo
01ooo`00Oooo00<00?ooool0>Ooo00<00?ooool0@Ooo00<00?ooool07_oo001nool00`00ooooo`0j
ool00`00ooooo`12ool00`00ooooo`0Mool007goo`03003ooooo03_oo`03003ooooo04;oo`03003o
oooo01goo`00OOoo00<00?ooool0>ooo0P00A?oo00<00?ooool07?oo001lool00`00ooooo`0lool0
0`00ooooo`14ool00`00ooooo`0Kool007_oo`03003ooooo03goo`03003ooooo04Goo`03003ooooo
01[oo`00Nooo00<00?ooool0?Ooo00<00?ooool0A_oo00<00?ooool06Ooo001jool00`00ooooo`0n
ool00`00ooooo`17ool00`00ooooo`0Hool007Woo`03003ooooo03ooo`03003ooooo04Soo`03003o
oooo01Ooo`00NOoo00<00?ooool0?ooo00<00?ooool0B?oo00<00?ooool05ooo001hool00`00oooo
o`10ool00`00ooooo`19ool00`00ooooo`0Fool007Ooo`03003ooooo047oo`8004_oo`03003ooooo
01Goo`00Mooo00<00?ooool0@Ooo00<00?ooool0Booo00<00?ooool05?oo001fool00`00ooooo`12
ool00`00ooooo`1<ool00`00ooooo`0Cool007Goo`03003ooooo04?oo`03003ooooo04goo`03003o
oooo01;oo`00MOoo00<00?ooool0@ooo00<00?ooool0COoo00<00?ooool04_oo001dool00`00oooo
o`14ool00`00ooooo`1>ool00`00ooooo`0Aool007?oo`03003ooooo04Goo`03003ooooo04ooo`03
003ooooo013oo`00Looo00<00?ooool0AOoo00<00?ooool0D?oo00<00?ooool03ooo001bool00`00
ooooo`16ool2001Aool00`00ooooo`0?ool0077oo`03003ooooo04Ooo`03003ooooo057oo`03003o
oooo00koo`00LOoo00<00?ooool0Aooo00<00?ooool0D_oo00<00?ooool03Ooo001`ool00`00oooo
o`18ool00`00ooooo`1Cool00`00ooooo`0<ool006ooo`03003ooooo04Woo`03003ooooo05?oo`03
003ooooo00coo`00Kooo00<00?ooool0BOoo00<00?ooool0E?oo00<00?ooool02ooo001^ool00`00
ooooo`0`ool20004ool00`00ooooo`03ool20005ool00`00ooooo`04ool00`00ooooo`1Eool00`00
ooooo`0:ool006koo`03003ooooo02ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0
1?oo00<00?ooool0H_oo001]ool00`00ooooo`0`ool01000ooooo`002?oo00D00?ooool00?oo00D0
00Goo`<006;oo`00KOoo00<00?ooool0<?oo00@00?ooool000Soo`06003ooooo003oo`000_oo00<0
0?ooool01?oo00<00?ooool0H_oo001/ool00`00ooooo`0aool01000ooooo`002?oo00@00?ooool0
00;oo`03003oo`0000Koo`03003ooooo06;oo`00K?oo00<00?ooool0<Ooo00@00?ooool000Soo`04
003ooooo0003ool20006ool00`00ooooo`1Rool006_oo`03003ooooo03?oo`8000[oo`8000Goo`03
003ooooo00Coo`03003ooooo06;oo`00Jooo00<00?ooool0COoo00<00?ooool0H_oo001Zool00`00
ooooo`1>ool00`00ooooo`1Rool006[oo`03003ooooo04koo`03003ooooo06;oo`00JOoo00<00?oo
ool0Cooo0P00Hooo001Yool00`00ooooo`1?ool00`00ooooo`1Rool006Soo`03003ooooo053oo`03
003ooooo06;oo`00J?oo00<00?ooool0D?oo00<00?ooool0H_oo001Wool00`00ooooo`1Aool00`00
ooooo`1Rool006Ooo`03003ooooo057oo`03003ooooo06;oo`00I_oo00<00?ooool0D_oo00<00?oo
ool0H_oo001Vool00`00ooooo`1Bool00`00ooooo`1Rool006Goo`03003ooooo05?oo`8006?oo`00
IOoo00<00?ooool0Dooo00<00?ooool0H_oo001Tool00`00ooooo`1Dool00`00ooooo`1Rool006Co
o`03003ooooo05Coo`03003ooooo06;oo`00Hooo00<00?ooool0EOoo00<00?ooool0H_oo001Sool0
0`00ooooo`1Eool00`00ooooo`1Rool006;oo`03003ooooo05Koo`03003ooooo06;oo`00H_oo00<0
0?ooool0E_oo00<00?ooool0H_oo001Qool00`00ooooo`1Gool2001Sool0067oo`03003ooooo05Oo
o`03003ooooo06;oo`00H?oo00<00?ooool0F?oo00<00?ooool0H_oo001Pool00`00ooooo`1Hool0
0`00ooooo`1Rool005ooo`03003ooooo05Woo`03003ooooo06;oo`00Gooo00<00?ooool0FOoo00<0
0?ooool0H_oo001Oool00`00ooooo`0oool20004ool00`00ooooo`03ool20004ool20006ool00`00
ooooo`1Rool005koo`03003ooooo03ooo`04003ooooo0008ool01000ooooo`000_oo00@00?ooool0
00Goo`03003ooooo06;oo`00G_oo00<00?ooool0?ooo00@00?ooool000Soo`04003ooooo0002ool0
1000ooooo`001Ooo0`00H_oo001Mool00`00ooooo`10ool01000ooooo`002?oo00@00?ooool000;o
o`<000Koo`03003ooooo06;oo`00GOoo00<00?ooool0@?oo00@00?ooool000Soo`04003ooooo0002
ool00`00ooooo`06ool00`00ooooo`1Rool005coo`03003ooooo047oo`04003ooooo0008ool01000
ooooo`000_oo00<00?ooool01_oo00<00?ooool0H_oo001Lool00`00ooooo`12ool2000:ool20004
ool30005ool00`00ooooo`1Rool005_oo`03003ooooo05goo`03003ooooo06;oo`00Fooo00<00?oo
ool0GOoo00<00?ooool0H_oo001Jool00`00ooooo`1Nool00`00ooooo`1Rool005[oo`03003ooooo
05koo`8006?oo`00FOoo00<00?ooool0Gooo00<00?ooool0H_oo001Iool00`00ooooo`1Oool00`00
ooooo`1Rool005Woo`03003ooooo05ooo`03003ooooo06;oo`00F?oo00<00?ooool0H?oo00<00?oo
ool0H_oo001Hool00`00ooooo`1Pool00`00ooooo`1Rool005Ooo`03003ooooo067oo`03003ooooo
06;oo`00Eooo00<00?ooool0HOoo00<00?ooool0H_oo001Gool00`00ooooo`1Qool2001Sool005Ko
o`03003ooooo06;oo`03003ooooo06;oo`00E_oo00<00?ooool0H_oo00<00?ooool0H_oo001Eool0
0`00ooooo`1Sool00`00ooooo`1Rool005Goo`03003ooooo06?oo`03003ooooo06;oo`00E?oo00<0
0?ooool0I?oo00<00?ooool0H_oo001Dool00`00ooooo`1Tool00`00ooooo`1Rool005Coo`03003o
oooo06Coo`03003ooooo06;oo`00Dooo00<00?ooool0IOoo0P00Hooo001Cool00`00ooooo`1Uool0
0`00ooooo`1Rool005;oo`03003ooooo06Koo`03003ooooo06;oo`00D_oo00<00?ooool0I_oo00<0
0?ooool0H_oo001Bool00`00ooooo`1Vool00`00ooooo`1Rool0057oo`03003ooooo06Ooo`03003o
oooo06;oo`00DOoo00<00?ooool0COoo0P001?oo00<00?ooool00ooo0P001?oo0P001_oo00<00?oo
ool0H_oo001@ool00`00ooooo`1=ool01000ooooo`002?oo00@00?ooool000;oo`04003ooooo0005
ool00`00ooooo`1Rool0053oo`03003ooooo04goo`04003ooooo0008ool01000ooooo`000_oo00@0
0?ooool000Goo`<006;oo`00Cooo00<00?ooool0C_oo00@00?ooool000Soo`04003ooooo0003ool2
0006ool00`00ooooo`1Rool004ooo`03003ooooo04koo`04003ooooo0008ool01000ooooo`000_oo
00@00?ooool000Goo`03003ooooo06;oo`00Cooo00<00?ooool0C_oo00@00?ooool000Soo`04003o
oooo0002ool01000ooooo`001Ooo00<00?ooool0H_oo001>ool00`00ooooo`1@ool2000:ool20004
ool20006ool00`00ooooo`1Rool004koo`03003ooooo06[oo`03003ooooo06;oo`00COoo00<00?oo
ool0Jooo00<00?ooool0H_oo001=ool00`00ooooo`1[ool00`00ooooo`1Rool004goo`03003ooooo
06_oo`8006?oo`00C?oo00<00?ooool0K?oo00<00?ooool0H_oo001<ool00`00ooooo`1/ool00`00
ooooo`1Rool004coo`03003ooooo06coo`03003ooooo06;oo`00Booo00<00?ooool0KOoo00<00?oo
ool0H_oo001;ool00`00ooooo`1]ool00`00ooooo`1Rool004_oo`03003ooooo06goo`03003ooooo
06;oo`00B_oo00<00?ooool0K_oo00<00?ooool0H_oo001:ool00`00ooooo`1^ool2001Sool004[o
o`03003ooooo06koo`03003ooooo06;oo`00BOoo00<00?ooool0Kooo00<00?ooool0H_oo0019ool0
0`00ooooo`1_ool00`00ooooo`1Rool004Woo`03003ooooo06ooo`03003ooooo06;oo`00B?oo00<0
0?ooool0L?oo00<00?ooool0H_oo0018ool00`00ooooo`1`ool00`00ooooo`1Rool004Ooo`03003o
oooo077oo`03003ooooo06;oo`00Aooo00<00?ooool0LOoo0P00Hooo0017ool00`00ooooo`1aool0
0`00ooooo`1Rool004Koo`03003ooooo07;oo`03003ooooo06;oo`00A_oo00<00?ooool0L_oo00<0
0?ooool0H_oo0016ool00`00ooooo`1bool00`00ooooo`1Rool004Goo`03003ooooo07?oo`03003o
oooo06;oo`00AOoo00<00?ooool0Gooo0P001?oo00<00?ooool00_oo10001Ooo00<00?ooool0H_oo
0015ool00`00ooooo`1Nool01000ooooo`002_oo00<00?ooool01?oo00<00?ooool0H_oo0014ool0
0`00ooooo`1Oool01000ooooo`002_oo00<00?ooool01?oo0`00H_oo0014ool00`00ooooo`1Oool0
1000ooooo`002_oo00<00?ooool01?oo00<00?ooool0H_oo0014ool00`00ooooo`1Oool01000oooo
o`002_oo00<00?ooool01?oo00<00?ooool0H_oo0013ool00`00ooooo`1Pool01000ooooo`002?oo
0`001_oo00<00?ooool0H_oo0013ool00`00ooooo`1Qool2000;ool00`00ooooo`04ool00`00oooo
o`1Rool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.786504, -0.00778725, \
0.00113934, 0.000625083}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Im[kx0corrtestfunc[Om - 0.34475238584689466`\ I, 0.8, 10]], {Om, 0.8, 
        1.1}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.51587 3.1746 0.175181 0.6726 [
[.02381 .16268 -9 -9 ]
[.02381 .16268 9 0 ]
[.18254 .16268 -12 -9 ]
[.18254 .16268 12 0 ]
[.34127 .16268 -9 -9 ]
[.34127 .16268 9 0 ]
[.5 .16268 -12 -9 ]
[.5 .16268 12 0 ]
[.81746 .16268 -12 -9 ]
[.81746 .16268 12 0 ]
[.97619 .16268 -9 -9 ]
[.97619 .16268 9 0 ]
[.64623 .04066 -24 -4.5 ]
[.64623 .04066 0 4.5 ]
[.64623 .3097 -18 -4.5 ]
[.64623 .3097 0 4.5 ]
[.64623 .44422 -18 -4.5 ]
[.64623 .44422 0 4.5 ]
[.64623 .57874 -18 -4.5 ]
[.64623 .57874 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .17518 m
.02381 .18143 L
s
[(0.8)] .02381 .16268 0 1 Mshowa
.18254 .17518 m
.18254 .18143 L
s
[(0.85)] .18254 .16268 0 1 Mshowa
.34127 .17518 m
.34127 .18143 L
s
[(0.9)] .34127 .16268 0 1 Mshowa
.5 .17518 m
.5 .18143 L
s
[(0.95)] .5 .16268 0 1 Mshowa
.81746 .17518 m
.81746 .18143 L
s
[(1.05)] .81746 .16268 0 1 Mshowa
.97619 .17518 m
.97619 .18143 L
s
[(1.1)] .97619 .16268 0 1 Mshowa
.125 Mabswid
.05556 .17518 m
.05556 .17893 L
s
.0873 .17518 m
.0873 .17893 L
s
.11905 .17518 m
.11905 .17893 L
s
.15079 .17518 m
.15079 .17893 L
s
.21429 .17518 m
.21429 .17893 L
s
.24603 .17518 m
.24603 .17893 L
s
.27778 .17518 m
.27778 .17893 L
s
.30952 .17518 m
.30952 .17893 L
s
.37302 .17518 m
.37302 .17893 L
s
.40476 .17518 m
.40476 .17893 L
s
.43651 .17518 m
.43651 .17893 L
s
.46825 .17518 m
.46825 .17893 L
s
.53175 .17518 m
.53175 .17893 L
s
.56349 .17518 m
.56349 .17893 L
s
.59524 .17518 m
.59524 .17893 L
s
.62698 .17518 m
.62698 .17893 L
s
.69048 .17518 m
.69048 .17893 L
s
.72222 .17518 m
.72222 .17893 L
s
.75397 .17518 m
.75397 .17893 L
s
.78571 .17518 m
.78571 .17893 L
s
.84921 .17518 m
.84921 .17893 L
s
.88095 .17518 m
.88095 .17893 L
s
.9127 .17518 m
.9127 .17893 L
s
.94444 .17518 m
.94444 .17893 L
s
.25 Mabswid
0 .17518 m
1 .17518 L
s
.65873 .04066 m
.66498 .04066 L
s
[(-0.2)] .64623 .04066 1 0 Mshowa
.65873 .3097 m
.66498 .3097 L
s
[(0.2)] .64623 .3097 1 0 Mshowa
.65873 .44422 m
.66498 .44422 L
s
[(0.4)] .64623 .44422 1 0 Mshowa
.65873 .57874 m
.66498 .57874 L
s
[(0.6)] .64623 .57874 1 0 Mshowa
.125 Mabswid
.65873 .07429 m
.66248 .07429 L
s
.65873 .10792 m
.66248 .10792 L
s
.65873 .14155 m
.66248 .14155 L
s
.65873 .20881 m
.66248 .20881 L
s
.65873 .24244 m
.66248 .24244 L
s
.65873 .27607 m
.66248 .27607 L
s
.65873 .34333 m
.66248 .34333 L
s
.65873 .37696 m
.66248 .37696 L
s
.65873 .41059 m
.66248 .41059 L
s
.65873 .47785 m
.66248 .47785 L
s
.65873 .51148 m
.66248 .51148 L
s
.65873 .54511 m
.66248 .54511 L
s
.65873 .00703 m
.66248 .00703 L
s
.65873 .61237 m
.66248 .61237 L
s
.25 Mabswid
.65873 0 m
.65873 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .56295 L
.10458 .52152 L
.14415 .48487 L
.18221 .45152 L
.22272 .41792 L
.26171 .38728 L
.30316 .3564 L
.34309 .32818 L
.3815 .30234 L
.42237 .27616 L
.46172 .25213 L
.49955 .23005 L
.53984 .20757 L
.57861 .18685 L
.61984 .16576 L
.65954 .14629 L
.69774 .12829 L
.73838 .10986 L
.77751 .09279 L
.81909 .07532 L
.85916 .05908 L
.89771 .04399 L
.93871 .02846 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo002lool0
0`00ooooo`1Qool00;coo`03003ooooo067oo`00_?oo0P00H_oo002lool00`00ooooo`1Qool00;co
o`03003ooooo05Koo`8000Woo`00_?oo00<00?ooool0Dooo0`002ooo002lool00`00ooooo`1Aool2
000>ool00;coo`03003ooooo04koo`<0013oo`00_?oo00<00?ooool0Booo0`004ooo002Xool20004
ool00`00ooooo`02ool40005ool00`00ooooo`18ool3000Fool00:Ooo`04003ooooo0008ool00`00
ooooo`06ool00`00ooooo`15ool3000Iool00:Ooo`04003ooooo0009ool00`00ooooo`05ool30012
ool3000Lool00:7oo`@000;oo`04003ooooo000:ool00`00ooooo`04ool00`00ooooo`0oool3000O
ool00:Ooo`04003ooooo000;ool00`00ooooo`03ool00`00ooooo`0lool3000Rool00:Ooo`04003o
oooo0008ool01000ooooo`001Ooo00<00?ooool0>_oo0P009Ooo002Xool2000:ool20006ool00`00
ooooo`0gool3000Wool00;coo`03003ooooo03Coo`<002[oo`00_?oo00<00?ooool0<_oo0P00;Ooo
002lool00`00ooooo`0`ool2000_ool00;coo`03003ooooo02koo`80037oo`00_?oo00<00?ooool0
;?oo0P00<ooo002lool2000Zool3000eool00;coo`03003ooooo02Ooo`8003Soo`00_?oo00<00?oo
ool09?oo0`00>_oo002lool00`00ooooo`0Rool2000mool00;coo`03003ooooo023oo`8003ooo`00
_?oo00<00?ooool07Ooo0`00@Ooo002lool00`00ooooo`0Kool20014ool00;coo`03003ooooo01Wo
o`8004Koo`00_?oo00<00?ooool05ooo0P00B?oo002lool2000Fool2001:ool00;coo`03003ooooo
01;oo`<004coo`00_?oo00<00?ooool04?oo0P00Cooo002lool00`00ooooo`0>ool2001Aool00;co
o`03003ooooo00coo`8005?oo`00_?oo00<00?ooool02_oo0P00EOoo002lool00`00ooooo`07ool3
001Gool00;coo`03003ooooo00Goo`8005[oo`00_?oo00<00?ooool00ooo0P00G?oo0002ool20004
ool00`00ooooo`03ool2000Kool20004ool00`00ooooo`03ool20003ool3000Lool20004ool00`00
ooooo`02ool3000Lool20004ool00`00ooooo`02ool30003ool3000Sool01000oooooooo0P007?oo
10000ooo00<00?ooool00ooo0P000ooo0`006ooo10000ooo00<00?ooool00_oo10000_oo00001Ooo
003ooooo00002?oo00@00?ooool001Woo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool0
6?oo00@00?ooool000_oo`03003ooooo01Soo`04003ooooo000;ool00`00ooooo`03ool00`00oooo
o`0Pool4000Pool00`00ooooo`07ool01000ooooo`001Ooo00<00?ooool06_oo00<00?ooool02Ooo
00<00?ooool00Ooo00001Ooo003ooooo00002?oo00@00?ooool001Woo`04003ooooo0008ool01000
ooooo`001Ooo00<00?ooool06?oo00@00?ooool000_oo`03003ooooo01Soo`04003ooooo000;ool0
0`00ooooo`03ool00`00ooooo`0Pool2000Rool00`00ooooo`07ool01000ooooo`001Ooo00<00?oo
ool06_oo00<00?ooool02Ooo00<00?ooool00Ooo00001Ooo003ooooo00002Ooo0P006_oo00@00?oo
ool000Woo`8000?oo`<001_oo`04003ooooo0009ool3000Jool01000ooooo`002Ooo0`000_oo0`00
8Ooo0`008ooo00<00?ooool01ooo00@00?ooool000;oo`<001goo`03003ooooo00Woo`03003ooooo
007oo`0000Goo`00ooooo`0000Soo`04003ooooo000Iool01000ooooo`002?oo00@00?ooool000;o
o`03003ooooo01_oo`04003ooooo0008ool01000ooooo`006_oo00@00?ooool000Soo`04003ooooo
0002ool00`00ooooo`0Oool20002ool00`00ooooo`0Qool00`00ooooo`07ool01000ooooo`000_oo
00<00?ooool07Ooo00<00?ooool02Ooo00<00?ooool00Ooo00001Ooo003ooooo00002?oo00@00?oo
ool001Woo`04003ooooo0008ool01000ooooo`000_oo00<00?ooool06ooo00@00?ooool000Soo`04
003ooooo000Jool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo01goo`8000Coo`03003o
oooo01ooo`<000Woo`04003ooooo0002ool00`00ooooo`0Kool30009ool30003ool000;oo`8000[o
o`8001_oo`8000[oo`8000?oo`@001_oo`8000[oo`8001coo`8000[oo`8000?oo`@001[oo`8000Ko
o`03003ooooo027oo`03003ooooo00Soo`8000?oo`@001coo`03003ooooo00Woo`03003ooooo007o
o`00/_oo0P002?oo00<00?ooool0HOoo002`ool2000:ool00`00ooooo`1Qool00:koo`8000coo`03
003ooooo067oo`000_ooo`0070000ooo0009ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00oooo
o`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00oooo
o`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00oooo
o`06ool20007ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`07ool000Woo`03003ooooo02Woo`03003ooooo
02[oo`03003ooooo02[oo`03003ooooo01Koo`8001;oo`03003ooooo02[oo`03003ooooo02[oo`03
003ooooo00Ooo`00Y_oo0P005?oo00<00?ooool0HOoo002Tool2000Fool00`00ooooo`1Qool00:;o
o`8001Soo`03003ooooo067oo`00X?oo0P006_oo00<00?ooool0HOoo002Nool2000Lool00`00oooo
o`1Qool009coo`8001koo`03003ooooo067oo`00V_oo0P008?oo00<00?ooool0HOoo002Hool2000R
ool2001Rool009Ooo`03003ooooo02;oo`03003ooooo067oo`00UOoo0P009Ooo00<00?ooool0HOoo
002Cool2000Wool00`00ooooo`1Qool009;oo`03003ooooo02Ooo`03003ooooo067oo`00T?oo0P00
:_oo00<00?ooool0HOoo002?ool00`00ooooo`0Zool00`00ooooo`1Qool008goo`8002goo`03003o
oooo067oo`00Rooo0P00;ooo00<00?ooool0HOoo0029ool2000aool2001Rool008Ooo`8003?oo`03
003ooooo067oo`00QOoo0P00=Ooo00<00?ooool0HOoo0024ool00`00ooooo`0eool00`00ooooo`1Q
ool008;oo`8003Soo`03003ooooo067oo`00POoo00<00?ooool0>?oo00<00?ooool0HOoo001oool2
000kool00`00ooooo`1Qool007goo`8003goo`03003ooooo067oo`00O?oo00<00?ooool0?Ooo00<0
0?ooool0HOoo001jool20010ool00`00ooooo`1Qool007Woo`03003ooooo043oo`8006;oo`00Mooo
0P00@ooo00<00?ooool0HOoo001fool00`00ooooo`13ool00`00ooooo`1Qool007Coo`8004Koo`03
003ooooo067oo`00L_oo0P00B?oo00<00?ooool0HOoo001aool00`00ooooo`18ool00`00ooooo`1Q
ool006ooo`8004_oo`03003ooooo067oo`00K_oo00<00?ooool0=ooo0P001?oo00<00?ooool00_oo
10001Ooo00<00?ooool0HOoo001/ool2000iool01000ooooo`002?oo00<00?ooool01_oo00<00?oo
ool0HOoo001[ool00`00ooooo`0iool01000ooooo`002Ooo00<00?ooool01Ooo0`00HOoo001Yool2
000lool01000ooooo`002_oo00<00?ooool01?oo00<00?ooool0HOoo001Wool2000nool01000oooo
o`002ooo00<00?ooool00ooo00<00?ooool0HOoo001Vool00`00ooooo`0nool01000ooooo`002?oo
00@00?ooool000Goo`03003ooooo067oo`00I?oo0P00@_oo0P002_oo0P001_oo00<00?ooool0HOoo
001Sool00`00ooooo`1Fool00`00ooooo`1Qool0067oo`8005Woo`03003ooooo067oo`00H?oo00<0
0?ooool0FOoo00<00?ooool0HOoo001Nool2001Lool00`00ooooo`1Qool005goo`03003ooooo05co
o`03003ooooo067oo`00Fooo0P00Gooo0P00H_oo001Jool00`00ooooo`1Oool00`00ooooo`1Qool0
05Soo`8006;oo`03003ooooo067oo`00Eooo00<00?ooool0H_oo00<00?ooool0HOoo001Fool00`00
ooooo`1Sool00`00ooooo`1Qool005Coo`8006Koo`03003ooooo067oo`00Dooo00<00?ooool0I_oo
00<00?ooool0HOoo001Bool00`00ooooo`1Wool00`00ooooo`1Qool0057oo`03003ooooo06Soo`03
003ooooo067oo`00D?oo00<00?ooool0JOoo0P00H_oo001>ool2001/ool00`00ooooo`1Qool004go
o`03003ooooo06coo`03003ooooo067oo`00C?oo00<00?ooool0KOoo00<00?ooool0HOoo001;ool0
0`00ooooo`1^ool00`00ooooo`1Qool004Woo`80077oo`03003ooooo067oo`00B?oo00<00?ooool0
LOoo00<00?ooool0HOoo0017ool00`00ooooo`1bool00`00ooooo`1Qool004Koo`03003ooooo07?o
o`03003ooooo067oo`00AOoo00<00?ooool0M?oo00<00?ooool0HOoo0013ool2001gool2001Rool0
04;oo`03003ooooo07Ooo`03003ooooo067oo`00@Ooo00<00?ooool0N?oo00<00?ooool0HOoo000o
ool2001kool00`00ooooo`1Qool003koo`03003ooooo07_oo`03003ooooo067oo`00?Ooo00<00?oo
ool0O?oo00<00?ooool0HOoo000kool2001oool00`00ooooo`1Qool003[oo`03003ooooo06_oo`80
00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo067oo`00>Ooo00<00?ooool0Jooo00@0
0?ooool000[oo`03003ooooo00Coo`03003ooooo067oo`00=ooo0P00K_oo00@00?ooool000Ooo`D0
00Goo`<0067oo`00=_oo00<00?ooool0K_oo00@00?ooool000Ooo`04003ooooo0006ool00`00oooo
o`1Qool003Goo`03003ooooo06ooo`04003ooooo0008ool00`00ool00006ool00`00ooooo`1Qool0
03Coo`03003ooooo073oo`04003ooooo0009ool20006ool00`00ooooo`1Qool003?oo`03003ooooo
07;oo`8000_oo`03003ooooo00Coo`03003ooooo067oo`00<_oo00<00?ooool0Qooo00<00?ooool0
HOoo000aool00`00ooooo`28ool00`00ooooo`1Qool002ooo`8008_oo`03003ooooo067oo`00;_oo
00<00?ooool0Rooo00<00?ooool0HOoo000]ool00`00ooooo`2<ool00`00ooooo`1Qool002coo`03
003ooooo08goo`8006;oo`00:ooo00<00?ooool0S_oo00<00?ooool0HOoo000Zool00`00ooooo`2?
ool00`00ooooo`1Qool002Woo`03003ooooo093oo`03003ooooo067oo`00:?oo00<00?ooool0TOoo
00<00?ooool0HOoo000Wool00`00ooooo`2Bool00`00ooooo`1Qool002Koo`03003ooooo09?oo`03
003ooooo067oo`009?oo0P00U_oo00<00?ooool0HOoo000Sool00`00ooooo`2Fool00`00ooooo`1Q
ool002;oo`03003ooooo09Ooo`8006;oo`008Ooo00<00?ooool0V?oo00<00?ooool0HOoo000Pool0
0`00ooooo`2Iool00`00ooooo`1Qool001ooo`03003ooooo09[oo`03003ooooo067oo`007_oo00<0
0?ooool0Vooo00<00?ooool0HOoo000Mool00`00ooooo`2Lool00`00ooooo`1Qool001coo`03003o
oooo09goo`03003ooooo067oo`006ooo00<00?ooool0W_oo00<00?ooool0HOoo000Jool00`00oooo
o`2Oool00`00ooooo`1Qool001Woo`03003ooooo0:3oo`03003ooooo067oo`006?oo00<00?ooool0
XOoo0P00H_oo000Gool00`00ooooo`2Rool00`00ooooo`1Qool001Koo`03003ooooo0:?oo`03003o
oooo067oo`005Ooo00<00?ooool0Y?oo00<00?ooool0HOoo000Dool00`00ooooo`2Uool00`00oooo
o`1Qool001?oo`03003ooooo0:Koo`03003ooooo067oo`004_oo00<00?ooool0Yooo00<00?ooool0
HOoo000Aool00`00ooooo`2Dool20004ool00`00ooooo`03ool20006ool00`00ooooo`1Qool0013o
o`03003ooooo09Coo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool0HOoo000?ool00`00
ooooo`2Eool01000ooooo`002?oo00@00?ooool000Goo`<0067oo`003_oo00<00?ooool0U_oo00@0
0?ooool000Soo`<000Koo`03003ooooo067oo`003_oo00<00?ooool0U_oo00@00?ooool000Soo`03
003ooooo00Koo`03003ooooo067oo`003Ooo00<00?ooool0Uooo00@00?ooool000Soo`03003ooooo
00Koo`03003ooooo067oo`003?oo00<00?ooool0VOoo0P002_oo0`001Ooo00<00?ooool0HOoo000;
ool00`00ooooo`2^ool00`00ooooo`1Qool000[oo`03003ooooo0:ooo`03003ooooo067oo`002Ooo
00<00?ooool0/?oo00<00?ooool0HOoo002lool00`00ooooo`1Qool00;coo`03003ooooo067oo`00
_?oo0P00H_oo002lool00`00ooooo`1Qool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.789961, -0.266708, \
0.00111525, 0.00526387}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Re[kx0corrtestfunc[0.9818786433914661`, 0.8 - 0.34475238584689466`\ I, 
        10]]\)], "Input"],

Cell[BoxData[
    \(\(-0.11930710570787324`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{N[k], Om /. strangesol[k, 1]}, {k, 0.01, 0.5, 0.01}]\)], "Input"],

Cell[BoxData[
    \(FindRoot::"frsec" \(\(:\)\(\ \)\) 
      "Secant method failed to converge to the prescribed accuracy after \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \({{0.01`, 0.6585512192126639`}, {0.02`, 0.7006569027008378`}, {0.03`, 
        0.7420976321496907`}, {0.04`, 0.7798985662657786`}, {0.05`, 
        0.81418783865906`}, {0.06`, 0.8454874915149088`}, {0.07`, 
        0.8742832670630734`}, {0.08`, 0.900968104451262`}, {0.09`, 
        0.9258522990050023`}, {0.1`, 0.9491819265302178`}, {0.11`, 
        0.9711536557152823`}, {0.12`, 0.9919268205364776`}, {0.13`, 
        1.0116330385415675`}, {0.14`, 1.0303812704173296`}, {0.15`, 
        1.0482632561168983`}, {0.16`, 1.0653568976791425`}, {0.17`, 
        1.0817292307005018`}, {0.18`, 1.097437715634643`}, {0.19`, 
        1.112532717099449`}, {0.2`, 1.127058328201911`}, {0.21`, 
        1.1410534279459454`}, {0.22`, 1.1545525625257962`}, {0.23`, 
        1.1675863447722852`}, {0.24`, 0.6582326973462245`}, {0.25`, 
        1.192365760274391`}, {0.26`, 1.2041585624098197`}, {0.27`, 
        1.215581231196133`}, {0.28`, 1.2266522689677826`}, {0.29`, 
        1.2373885892392371`}, {0.3`, 1.2478057783177217`}, {0.31`, 
        1.257918094848564`}, {0.32`, 1.2677384316751483`}, {0.33`, 
        1.277279040515267`}, {0.34`, 
        1.2865510941211882`}, {0.35000000000000003`, 
        1.2955650945920454`}, {0.36`, 1.3043305724996181`}, {0.37`, 
        1.3128566675263706`}, {0.38`, 1.3211517986179657`}, {0.39`, 
        0.7330193645754705`}, {0.4`, 
        1.3370799879273636`}, {0.41000000000000003`, 
        1.3447274114578218`}, {0.42`, \(-0.7518648217070116`\)}, {0.43`, 
        1.3594216248176456`}, {0.44`, 1.366480264051658`}, {0.45`, 
        2.620114666963815`}, {0.46`, 
        0.7780914136144891`}, {0.47000000000000003`, 
        0.784828291473139`}, {0.48`, 0.7916293527690688`}, {0.49`, 
        0.7984970463237716`}, {0.5`, 0.8054277102793638`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot[%, PlotJoined \[Rule] True]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.90476 0.0147151 0.285499 [
[.21429 .00222 -9 -9 ]
[.21429 .00222 9 0 ]
[.40476 .00222 -9 -9 ]
[.40476 .00222 9 0 ]
[.59524 .00222 -9 -9 ]
[.59524 .00222 9 0 ]
[.78571 .00222 -9 -9 ]
[.78571 .00222 9 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .15746 -18 -4.5 ]
[.01131 .15746 0 4.5 ]
[.01131 .30021 -6 -4.5 ]
[.01131 .30021 0 4.5 ]
[.01131 .44296 -18 -4.5 ]
[.01131 .44296 0 4.5 ]
[.01131 .58571 -6 -4.5 ]
[.01131 .58571 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .01472 m
.21429 .02097 L
s
[(0.1)] .21429 .00222 0 1 Mshowa
.40476 .01472 m
.40476 .02097 L
s
[(0.2)] .40476 .00222 0 1 Mshowa
.59524 .01472 m
.59524 .02097 L
s
[(0.3)] .59524 .00222 0 1 Mshowa
.78571 .01472 m
.78571 .02097 L
s
[(0.4)] .78571 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(0.5)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.0619 .01472 m
.0619 .01847 L
s
.1 .01472 m
.1 .01847 L
s
.1381 .01472 m
.1381 .01847 L
s
.17619 .01472 m
.17619 .01847 L
s
.25238 .01472 m
.25238 .01847 L
s
.29048 .01472 m
.29048 .01847 L
s
.32857 .01472 m
.32857 .01847 L
s
.36667 .01472 m
.36667 .01847 L
s
.44286 .01472 m
.44286 .01847 L
s
.48095 .01472 m
.48095 .01847 L
s
.51905 .01472 m
.51905 .01847 L
s
.55714 .01472 m
.55714 .01847 L
s
.63333 .01472 m
.63333 .01847 L
s
.67143 .01472 m
.67143 .01847 L
s
.70952 .01472 m
.70952 .01847 L
s
.74762 .01472 m
.74762 .01847 L
s
.82381 .01472 m
.82381 .01847 L
s
.8619 .01472 m
.8619 .01847 L
s
.9 .01472 m
.9 .01847 L
s
.9381 .01472 m
.9381 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .15746 m
.03006 .15746 L
s
[(0.5)] .01131 .15746 1 0 Mshowa
.02381 .30021 m
.03006 .30021 L
s
[(1)] .01131 .30021 1 0 Mshowa
.02381 .44296 m
.03006 .44296 L
s
[(1.5)] .01131 .44296 1 0 Mshowa
.02381 .58571 m
.03006 .58571 L
s
[(2)] .01131 .58571 1 0 Mshowa
.125 Mabswid
.02381 .04326 m
.02756 .04326 L
s
.02381 .07181 m
.02756 .07181 L
s
.02381 .10036 m
.02756 .10036 L
s
.02381 .12891 m
.02756 .12891 L
s
.02381 .18601 m
.02756 .18601 L
s
.02381 .21456 m
.02756 .21456 L
s
.02381 .24311 m
.02756 .24311 L
s
.02381 .27166 m
.02756 .27166 L
s
.02381 .32876 m
.02756 .32876 L
s
.02381 .35731 m
.02756 .35731 L
s
.02381 .38586 m
.02756 .38586 L
s
.02381 .41441 m
.02756 .41441 L
s
.02381 .47151 m
.02756 .47151 L
s
.02381 .50006 m
.02756 .50006 L
s
.02381 .52861 m
.02756 .52861 L
s
.02381 .55716 m
.02756 .55716 L
s
.02381 .61426 m
.02756 .61426 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.04286 .20273 m
.0619 .21475 L
.08095 .22658 L
.1 .23737 L
.11905 .24716 L
.1381 .2561 L
.15714 .26432 L
.17619 .27194 L
.19524 .27904 L
.21429 .28571 L
.23333 .29198 L
.25238 .29791 L
.27143 .30353 L
.29048 .30889 L
.30952 .31399 L
.32857 .31887 L
.34762 .32355 L
.36667 .32803 L
.38571 .33234 L
.40476 .33649 L
.42381 .34048 L
.44286 .34434 L
.4619 .34806 L
.48095 .20264 L
.5 .35513 L
.51905 .3585 L
.5381 .36176 L
.55714 .36492 L
.57619 .36799 L
.59524 .37096 L
.61429 .37385 L
.63333 .37665 L
.65238 .37938 L
.67143 .38202 L
.69048 .3846 L
.70952 .3871 L
.72857 .38953 L
.74762 .3919 L
.76667 .22399 L
.78571 .39645 L
.80476 .39863 L
s
.80476 .39863 m
.81745 0 L
s
.83013 0 m
.84286 .40283 L
.8619 .40484 L
s
.8619 .40484 m
.87325 .61803 L
s
.88619 .61803 m
.9 .23686 L
.91905 .23878 L
.9381 .24072 L
.95714 .24268 L
.97619 .24466 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00@Ooo0P001?oo00<00?ooool00_oo10009Ooo0P001?oo00<00?ooool00_oo1000
9?oo0P001?oo00<00?ooool00ooo0P009Ooo0P001?oo00<00?ooool01?oo00<00?ooool09?oo0P00
1?oo00<00?ooool00_oo0`000ooo0010ool01000ooooo`002_oo00<00?ooool08ooo00@00?ooool0
00Soo`03003ooooo02Coo`04003ooooo0008ool01000ooooo`008ooo00@00?ooool000[oo`03003o
oooo02?oo`04003ooooo000;ool10001ool1ool0043oo`04003ooooo000:ool00`00ooooo`0Sool0
1000ooooo`002Ooo00<00?ooool08ooo00@00?ooool000_oo`03003ooooo027oo`04003ooooo0007
ool5000Tool01000ooooo`002ooo0@000Ooo0Ooo0010ool01000ooooo`002_oo00<00?ooool08ooo
00@00?ooool000[oo`03003ooooo02;oo`04003ooooo0009ool2000Tool01000ooooo`001ooo00@0
0?ooool002Goo`04003ooooo0008ool30003ool0043oo`04003ooooo000:ool00`00ooooo`0Sool0
1000ooooo`002ooo00<00?ooool08Ooo00@00?ooool000_oo`03003ooooo027oo`04003ooooo0008
ool00`00ool0000Uool01000ooooo`002?oo00<00?ooool00ooo0010ool01000ooooo`002?oo0`00
9Ooo00@00?ooool000Soo`04003ooooo000Sool01000ooooo`002?oo00@00?ooool002?oo`04003o
oooo0009ool2000Uool01000ooooo`002?oo00<00?ooool00ooo000Eool00`00ooooo`0Yool2000;
ool00`00ooooo`0Tool2000:ool2000Uool2000:ool2000Uool2000;ool01000ooooo`000_oo00<0
0?ooool07_oo0P002Ooo10000_oo000Eool00`00ooooo`3Cool01000ooooo`00<Ooo000Eool00`00
ooooo`3Cool01000ooooo`00<Ooo000Eool00`00ooooo`3Cool01000ooooo`00<Ooo000>oooo000@
0003ool001Goo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03
003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Ooo`03
003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03
003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03
003ooooo00Ooo`03003ooooo00Koo`800003ool00?oo00Ooo`03003ooooo00Ooo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`005Ooo00<00?ooool0<?oo00<00?ooool0<Ooo00<00?oo
ool0<?oo00<00?ooool0<?oo00<00?ooool01_oo00@00?ooool002Ooo`03003ooooo00Ooo`005Ooo
00<00?ooool0dooo00@00?ooool0037oo`005Ooo00<00?ooool0dooo00@00?ooool0037oo`005Ooo
00<00?ooool0dooo00@00?ooool0037oo`005Ooo00<00?ooool0dooo00@00?ooool0037oo`005Ooo
00<00?ooool0dooo00@00?ooool0037oo`005Ooo0P00e?oo00@00?ooool0037oo`005Ooo00<00?oo
ool0dooo00@00?ooool0037oo`005Ooo00<00?ooool0d_oo00<00?ooool00_oo00<00?ooool0;_oo
000Eool00`00ooooo`3Bool00`00ooooo`02ool00`00ooooo`0^ool001Goo`03003ooooo0=;oo`03
003ooooo00;oo`03003ooooo02koo`005Ooo00<00?ooool0d_oo00<00?ooool00_oo00<00?ooool0
;_oo000Eool00`00ooooo`3Bool00`00ooooo`02ool00`00ooooo`0^ool001Goo`800=?oo`03003o
oooo00;oo`03003ooooo02koo`005Ooo00<00?ooool0d_oo00<00?ooool00_oo00<00?ooool0;_oo
000Eool00`00ooooo`3Bool00`00ooooo`02ool00`00ooooo`0^ool001Goo`03003ooooo0=;oo`03
003ooooo00;oo`03003ooooo02koo`005Ooo00<00?ooool0d_oo00<00?ooool00_oo00<00?ooool0
;_oo000Eool00`00ooooo`3Bool00`00ooooo`02ool00`00ooooo`0^ool001Goo`03003ooooo0=;o
o`03003ooooo00;oo`03003ooooo02koo`005Ooo00<00?ooool0d_oo00<00?ooool00_oo00<00?oo
ool0;_oo000Eool2003Cool00`00ooooo`02ool00`00ooooo`0^ool001Goo`03003ooooo0=;oo`03
003ooooo00;oo`03003ooooo02koo`005Ooo00<00?ooool0d_oo00<00?ooool00_oo00<00?ooool0
;_oo000Eool00`00ooooo`3Bool00`00ooooo`02ool00`00ooooo`0^ool001Goo`03003ooooo0=;o
o`03003ooooo00;oo`03003ooooo02koo`005Ooo00<00?ooool0d_oo00<00?ooool00_oo00<00?oo
ool0;_oo000Eool00`00ooooo`3Bool00`00ooooo`02ool00`00ooooo`0^ool001Goo`03003ooooo
0=;oo`03003ooooo00;oo`03003ooooo02koo`005Ooo0P00dooo00<00?ooool00_oo00<00?ooool0
;_oo000Eool00`00ooooo`3Bool00`00ooooo`02ool00`00ooooo`0^ool001Goo`03003ooooo0=;o
o`03003ooooo00;oo`03003ooooo02koo`005Ooo00<00?ooool0d_oo00<00?ooool00_oo00<00?oo
ool0;_oo000Eool00`00ooooo`3Bool00`00ooooo`02ool00`00ooooo`0^ool000;oo`8000Coo`03
003ooooo00;oo`<000Goo`03003ooooo0=;oo`03003ooooo00;oo`03003ooooo02koo`0000Goo`00
ooooo`0000_oo`03003ooooo00;oo`03003ooooo0=7oo`03003ooooo00Coo`03003ooooo02goo`00
00Goo`00ooooo`0000_oo`03003ooooo00;oo`<00=7oo`03003ooooo00Coo`03003ooooo02goo`00
00Goo`00ooooo`0000Soo`<000Goo`03003ooooo0=7oo`03003ooooo00Coo`03003ooooo02goo`00
00Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo0=7oo`03003ooooo00Coo`03003ooooo
02goo`0000Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo0=7oo`03003ooooo00Coo`03
003ooooo02goo`000_oo0P002Ooo10001?oo00<00?ooool0dOoo00<00?ooool01?oo00<00?ooool0
;Ooo000Eool00`00ooooo`3Aool00`00ooooo`04ool00`00ooooo`0]ool001Goo`03003ooooo0=7o
o`03003ooooo00Coo`03003ooooo02goo`005Ooo00<00?ooool0dOoo00<00?ooool01?oo00<00?oo
ool0;Ooo000Eool2003Bool00`00ooooo`04ool00`00ooooo`0]ool001Goo`03003ooooo0=7oo`03
003ooooo00Coo`03003ooooo02goo`005Ooo00<00?ooool0dOoo00<00?ooool01?oo00<00?ooool0
;Ooo000Eool00`00ooooo`3Aool00`00ooooo`04ool00`00ooooo`0]ool001Goo`03003ooooo0=7o
o`03003ooooo00Coo`03003ooooo02goo`005Ooo00<00?ooool00_oo00<00?ooool0Looo00<00?oo
ool0E_oo00<00?ooool01?oo00<00?ooool0;Ooo000Eool00`00ooooo`03ool2001cool00`00oooo
o`1Fool00`00ooooo`04ool00`00ooooo`0]ool001Goo`03003ooooo00Goo`80077oo`03003ooooo
05Koo`03003ooooo00Coo`03003ooooo02goo`005Ooo0P002?oo00<00?ooool0K_oo00<00?ooool0
E_oo00<00?ooool01?oo00<00?ooool0;Ooo000Eool00`00ooooo`08ool2001]ool2001Hool00`00
ooooo`04ool00`00ooooo`0]ool001Goo`03003ooooo00[oo`8006_oo`03003oo`0004_oo`03003o
oooo00Woo`03003ooooo00Coo`03003ooooo02goo`005Ooo00<00?ooool03?oo00<00?ooool0J?oo
00<00?oo0000Booo00<00?ooool02Ooo00<00?ooool01?oo00<00?ooool0;Ooo000Eool00`00oooo
o`0=ool2001Xool00`00ool0001;ool00`00ooooo`09ool00`00ooooo`04ool00`00ooooo`0]ool0
01Goo`03003ooooo00ooo`8006Koo`03003oo`0004_oo`03003ooooo00Woo`03003ooooo00Coo`03
003ooooo02goo`005Ooo00<00?ooool04Ooo00<00?ooool0Hooo00<00?oo0000Booo00<00?ooool0
2Ooo00<00?ooool01?oo00<00?ooool03_oo20005ooo000Eool2000Cool2001Sool00`00ool0001:
ool00`00ool0000:ool00`00ooooo`04ool00`00ooooo`0>ool00`00ooooo`05ool:000=ool001Go
o`03003ooooo01Coo`80067oo`03003oo`0004[oo`03003oo`0000[oo`03003ooooo00Coo`03003o
oooo00koo`03003ooooo00ooo`@000Woo`005Ooo00<00?ooool05_oo0`00GOoo00@00?ooool004[o
o`03003oo`0000Woo`03003ooooo00Goo`03003ooooo00koo`03003ooooo01coo`005Ooo00<00?oo
ool06Ooo0P00Fooo00D00?ooooooo`0004Woo`03003oo`0000Woo`03003ooooo00Goo`03003ooooo
00koo`03003ooooo01coo`005Ooo00<00?ooool06ooo0`00F?oo00D00?ooooooo`0004Woo`03003o
o`0000Woo`03003ooooo00Koo`03003ooooo00goo`03003ooooo01coo`005Ooo00<00?ooool07_oo
0P00E_oo00D00?ooooooo`0004Woo`03003oo`0000Woo`03003ooooo00Koo`03003ooooo00goo`03
003ooooo01coo`005Ooo00<00?ooool08?oo0`00Dooo00D00?ooooooo`0004Woo`03003oo`0000Wo
o`03003ooooo00Koo`03003ooooo00goo`03003ooooo01coo`005Ooo00<00?ooool08ooo0P00DOoo
00D00?ooooooo`0004Woo`03003oo`0000Woo`03003ooooo00Koo`03003ooooo00goo`03003ooooo
01coo`005Ooo0P009_oo0`00C_oo00D00?ooooooo`0004Woo`03003oo`0000Woo`03003ooooo00Ko
o`03003ooooo00goo`03003ooooo01coo`005Ooo00<00?ooool0:?oo0P00C?oo00D00?ooooooo`00
04Soo`04003ooooo0009ool00`00ooooo`06ool00`00ooooo`0=ool00`00ooooo`0Lool001Goo`03
003ooooo02[oo`<004Soo`03003ooooo00;oo`03003ooooo04Koo`05003oooooool00008ool00`00
ooooo`06ool00`00ooooo`0=ool00`00ooooo`0Lool001Goo`03003ooooo02goo`8004Koo`03003o
oooo00?oo`03003ooooo04Goo`05003oooooool00008ool00`00ooooo`06ool00`00ooooo`0=ool0
0`00ooooo`0Lool001Goo`03003ooooo02ooo`<004?oo`03003ooooo00?oo`03003ooooo04Goo`05
003oooooool00008ool00`00ooooo`06ool00`00ooooo`0<ool00`00ooooo`0Mool001Goo`03003o
oooo03;oo`80047oo`03003ooooo00?oo`03003ooooo04Goo`05003oooooool00008ool00`00oooo
o`06ool00`00ooooo`0<ool00`00ooooo`0Mool000goo`@000Coo`03003ooooo03Coo`@003goo`03
003ooooo00?oo`03003ooooo04Goo`05003oooooool00008ool00`00ooooo`06ool00`00ooooo`0<
ool00`00ooooo`0Mool000ooo`03003ooooo00?oo`03003ooooo03Soo`@003Woo`03003ooooo00?o
o`03003ooooo04Goo`05003oooooool00008ool00`00ooooo`06ool00`00ooooo`0<ool00`00oooo
o`0Mool000ooo`03003ooooo00?oo`<003coo`<003Koo`03003ooooo00?oo`03003ooooo04Goo`05
003oooooool00008ool00`00ooooo`06ool00`00ooooo`0<ool00`00ooooo`0Mool000ooo`03003o
oooo00?oo`03003ooooo03ooo`@003;oo`03003ooooo00?oo`03003ooooo04Goo`05003oooooool0
0008ool00`00ooooo`06ool00`00ooooo`0<ool00`00ooooo`0Mool000ooo`03003ooooo00?oo`03
003ooooo04?oo`@002goo`03003ooooo00Coo`03003ooooo04Goo`05003oooooool00008ool00`00
ooooo`06ool00`00ooooo`0<ool00`00ooooo`0Mool000goo`<000Goo`03003ooooo04Ooo`8002_o
o`03003ooooo00Goo`03003ooooo04?oo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00coo`03003ooooo01goo`003ooo00<00?ooool00ooo00<00?ooool0BOoo10009ooo
00<00?ooool01Ooo00<00?ooool0@ooo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool03?oo00<00?ooool07Ooo000Eool00`00ooooo`1=ool5000Rool00`00ooooo`05ool0
0`00ooooo`13ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`0<ool0
0`00ooooo`0Mool001Goo`03003ooooo05;oo`D001goo`03003ooooo00Goo`03003ooooo04?oo`03
003ooooo00?oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00coo`03003ooooo01goo`00
5Ooo00<00?ooool0Eooo10006Ooo00<00?ooool01Ooo00<00?ooool0@ooo00<00?ooool00ooo00<0
0?ooool01Ooo00<00?ooool01_oo00<00?ooool03?oo00<00?ooool07Ooo000Eool2001Lool2000G
ool00`00ooooo`05ool00`00ooooo`13ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`0<ool00`00ooooo`0Mool001Goo`03003ooooo05goo`@001?oo`03003ooooo00Go
o`03003ooooo04?oo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00co
o`03003ooooo01goo`005Ooo00<00?ooool0HOoo1P003?oo00<00?ooool01_oo00<00?ooool0@ooo
00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool03?oo00<00?ooool07Ooo
000Eool00`00ooooo`1Wool50007ool00`00ooooo`07ool00`00ooooo`12ool00`00ooooo`03ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`0<ool00`00ooooo`0Mool001Goo`03003ooooo
06coo`D000;oo`03003ooooo00Ooo`03003ooooo047oo`03003ooooo00Goo`03003ooooo00Coo`03
003ooooo00Koo`03003ooooo00coo`03003ooooo01goo`005Ooo00<00?ooool0LOoo0`002Ooo00<0
0?ooool0@Ooo00<00?ooool01Ooo00<00?ooool00ooo00<00?ooool01ooo00<00?ooool03?oo00<0
0?ooool07Ooo000Eool00`00ooooo`1mool00`00ooooo`11ool00`00ooooo`05ool00`00ooooo`03
ool00`00ooooo`07ool00`00ooooo`0<ool00`00ooooo`0Mool001Goo`8007koo`<0047oo`03003o
oooo00Goo`03003ooooo00?oo`03003ooooo00Soo`03003ooooo00_oo`03003ooooo01goo`005Ooo
00<00?ooool0P?oo2P00=ooo00<00?ooool01Ooo00<00?ooool00ooo00<00?ooool02?oo00<00?oo
ool02ooo00<00?ooool07Ooo000Eool00`00ooooo`2:ool6000aool00`00ooooo`05ool00`00oooo
o`03ool00`00ooooo`08ool00`00ooooo`0;ool00`00ooooo`0Mool001Goo`03003ooooo093oo`D0
02coo`03003ooooo00Goo`03003ooooo00?oo`03003ooooo00Soo`03003ooooo00_oo`03003ooooo
01goo`005Ooo00<00?ooool0UOoo1@009ooo00<00?ooool01Ooo00<00?ooool00ooo00<00?ooool0
2?oo00<00?ooool02ooo00<00?ooool07Ooo000Eool00`00ooooo`2Jool:000Mool00`00ooooo`05
ool00`00ooooo`03ool00`00ooooo`08ool00`00ooooo`0;ool00`00ooooo`0Mool001Goo`03003o
oooo0:Coo`D001Ooo`03003ooooo00Koo`03003ooooo00?oo`03003ooooo00Soo`03003ooooo00_o
o`03003ooooo01goo`005Ooo00<00?ooool0ZOoo1@004_oo00<00?ooool01ooo00<00?ooool00_oo
00<00?ooool02?oo00<00?ooool02_oo00<00?ooool07_oo000Eool2002_ool;0007ool00`00oooo
o`07ool00`00ooooo`02ool00`00ooooo`08ool00`00ooooo`0:ool00`00ooooo`0Nool001Goo`03
003ooooo0;Woo`D000;oo`03003ooooo00Ooo`03003ooooo00;oo`03003ooooo00Soo`03003ooooo
00[oo`03003ooooo01koo`005Ooo00<00?ooool0__oo0`002Ooo00<00?ooool00_oo00<00?ooool0
2?oo00<00?ooool02_oo00<00?ooool07_oo000Eool00`00ooooo`3:ool6000:ool00`00ooooo`0:
ool00`00ooooo`0Nool001Goo`03003ooooo0=[oo`03003ooooo00[oo`03003ooooo01koo`005Ooo
00<00?ooool0f_oo1P001ooo00<00?ooool07_oo000Eool00`00ooooo`3Oool00`00ooooo`05ool0
0`00ooooo`0Nool001Goo`03003ooooo0=ooo`03003ooooo00Goo`03003ooooo01koo`005Ooo0P00
h?oo00<00?ooool01Ooo00<00?ooool07_oo000Eool00`00ooooo`3Oool00`00ooooo`05ool00`00
ooooo`0Nool001Goo`03003ooooo0=ooo`03003ooooo00Goo`03003ooooo01koo`005Ooo00<00?oo
ool0gooo00<00?ooool01Ooo00<00?ooool07_oo000Eool00`00ooooo`3Oool00`00ooooo`05ool0
0`00ooooo`0Nool00003ool00000008000?oo`03003ooooo00;oo`<000Goo`03003ooooo0=ooo`03
003ooooo00Goo`03003ooooo01koo`000ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0
gooo00<00?ooool01Ooo00<00?ooool07_oo0003ool00`00ooooo`0:ool00`00ooooo`02ool3003P
ool00`00ooooo`04ool00`00ooooo`0Nool000?oo`03003ooooo00Ooo`<000Goo`03003ooooo0>3o
o`03003ooooo00Coo`03003ooooo01koo`000ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?oo
ool0h?oo00<00?ooool01?oo00<00?ooool07_oo00001?oo000000002Ooo00<00?ooool01Ooo00<0
0?ooool0h?oo00<00?ooool01?oo00<00?ooool07_oo0003ool00`00ooooo`07ool40004ool00`00
ooooo`3Pool00`00ooooo`04ool00`00ooooo`0Nool001Goo`03003ooooo0>3oo`03003ooooo00Co
o`03003ooooo01koo`005Ooo00<00?ooool0h?oo00<00?ooool01?oo00<00?ooool07_oo000Eool0
0`00ooooo`3Pool00`00ooooo`04ool00`00ooooo`0Nool001Goo`800>7oo`03003ooooo00Coo`03
003ooooo01koo`005Ooo00<00?ooool0h?oo00<00?ooool00ooo00<00?ooool07ooo000Eool00`00
ooooo`3Pool00`00ooooo`03ool00`00ooooo`0Oool001Goo`03003ooooo0>3oo`03003ooooo00?o
o`03003ooooo01ooo`005Ooo00<00?ooool0h?oo00<00?ooool00ooo00<00?ooool07ooo000Eool0
0`00ooooo`3Pool00`00ooooo`03ool00`00ooooo`0Oool001Goo`03003ooooo0>3oo`03003ooooo
00?oo`03003ooooo01ooo`005Ooo00<00?ooool0h?oo00<00?ooool00ooo00<00?ooool07ooo000E
ool2003Qool00`00ooooo`03ool00`00ooooo`0Oool001Goo`03003ooooo0>3oo`03003ooooo00?o
o`03003ooooo01ooo`005Ooo00<00?ooool0h?oo00<00?ooool00ooo00<00?ooool07ooo000Eool0
0`00ooooo`3Qool00`00ooooo`02ool00`00ooooo`0Oool001Goo`03003ooooo0>7oo`03003ooooo
00;oo`03003ooooo01ooo`005Ooo00<00?ooool0hOoo00<00?ooool00_oo00<00?ooool07ooo000E
ool00`00ooooo`3Qool00`00ooooo`02ool00`00ooooo`0Oool001Goo`800>;oo`03003ooooo00;o
o`03003ooooo01ooo`005Ooo00<00?ooool0hOoo00<00?ooool00_oo00<00?ooool07ooo000Eool0
0`00ooooo`3Qool00`00ooooo`02ool00`00ooooo`0Oool001Goo`03003ooooo0>7oo`03003ooooo
00;oo`03003ooooo01ooo`005Ooo00<00?ooool0hOoo00<00?ooool00_oo00<00?ooool07ooo000E
ool00`00ooooo`3Qool00`00ooooo`02ool00`00ooooo`0Oool001Goo`03003ooooo0>7oo`03003o
oooo00;oo`03003ooooo01ooo`005Ooo00<00?ooool0hOoo00<00?ooool00_oo00<00?ooool07ooo
000Eool2003Rool00`00ooooo`02ool00`00ooooo`0Oool001Goo`03003ooooo0>7oo`03003ooooo
00;oo`03003ooooo01ooo`005Ooo00<00?ooool0hOoo00<00?ooool00_oo00<00?ooool07ooo000E
ool00`00ooooo`3Qool00`00ooooo`02ool00`00ooooo`0Oool001Goo`03003ooooo0>7oo`05003o
ooooool0000Rool001Goo`03003ooooo0>7oo`05003oooooool0000Rool000goo`@000Coo`03003o
oooo0>7oo`05003oooooool0000Rool000goo`03003ooooo00Goo`03003ooooo0>;oo`04003ooooo
000Rool000koo`03003ooooo00Coo`<00>;oo`04003ooooo000Rool000ooo`03003ooooo00?oo`03
003ooooo0>;oo`04003ooooo000Rool0013oo`03003ooooo00;oo`03003ooooo0>;oo`04003ooooo
000Rool000goo`04003ooooo0004ool00`00ooooo`3Rool01000ooooo`008_oo000>ool20005ool0
0`00ooooo`3Rool01000ooooo`008_oo000Eool00`00ooooo`3Rool01000ooooo`008_oo000Eool0
0`00ooooo`3Rool01000ooooo`008_oo000Eool2003Sool01000ooooo`008_oo000Eool00`00oooo
o`3Rool01000ooooo`008_oo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0416348, -0.173758, \
0.00194834, 0.0129987}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.1, 0.1]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 2.878471233677976`, 
      mGamma \[Rule] \(-1.646935742983097`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[kx0corrtestfunc[Om, 0.1, 0.1]], {Om, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147151 0.0311543 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .0926 -18 -4.5 ]
[.01131 .0926 0 4.5 ]
[.01131 .17049 -6 -4.5 ]
[.01131 .17049 0 4.5 ]
[.01131 .24837 -18 -4.5 ]
[.01131 .24837 0 4.5 ]
[.01131 .32626 -12 -4.5 ]
[.01131 .32626 0 4.5 ]
[.01131 .40414 -24 -4.5 ]
[.01131 .40414 0 4.5 ]
[.01131 .48203 -12 -4.5 ]
[.01131 .48203 0 4.5 ]
[.01131 .55992 -24 -4.5 ]
[.01131 .55992 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .0926 m
.03006 .0926 L
s
[(2.5)] .01131 .0926 1 0 Mshowa
.02381 .17049 m
.03006 .17049 L
s
[(5)] .01131 .17049 1 0 Mshowa
.02381 .24837 m
.03006 .24837 L
s
[(7.5)] .01131 .24837 1 0 Mshowa
.02381 .32626 m
.03006 .32626 L
s
[(10)] .01131 .32626 1 0 Mshowa
.02381 .40414 m
.03006 .40414 L
s
[(12.5)] .01131 .40414 1 0 Mshowa
.02381 .48203 m
.03006 .48203 L
s
[(15)] .01131 .48203 1 0 Mshowa
.02381 .55992 m
.03006 .55992 L
s
[(17.5)] .01131 .55992 1 0 Mshowa
.125 Mabswid
.02381 .03029 m
.02756 .03029 L
s
.02381 .04587 m
.02756 .04587 L
s
.02381 .06145 m
.02756 .06145 L
s
.02381 .07702 m
.02756 .07702 L
s
.02381 .10818 m
.02756 .10818 L
s
.02381 .12376 m
.02756 .12376 L
s
.02381 .13933 m
.02756 .13933 L
s
.02381 .15491 m
.02756 .15491 L
s
.02381 .18606 m
.02756 .18606 L
s
.02381 .20164 m
.02756 .20164 L
s
.02381 .21722 m
.02756 .21722 L
s
.02381 .2328 m
.02756 .2328 L
s
.02381 .26395 m
.02756 .26395 L
s
.02381 .27953 m
.02756 .27953 L
s
.02381 .2951 m
.02756 .2951 L
s
.02381 .31068 m
.02756 .31068 L
s
.02381 .34184 m
.02756 .34184 L
s
.02381 .35741 m
.02756 .35741 L
s
.02381 .37299 m
.02756 .37299 L
s
.02381 .38857 m
.02756 .38857 L
s
.02381 .41972 m
.02756 .41972 L
s
.02381 .4353 m
.02756 .4353 L
s
.02381 .45088 m
.02756 .45088 L
s
.02381 .46645 m
.02756 .46645 L
s
.02381 .49761 m
.02756 .49761 L
s
.02381 .51318 m
.02756 .51318 L
s
.02381 .52876 m
.02756 .52876 L
s
.02381 .54434 m
.02756 .54434 L
s
.02381 .57549 m
.02756 .57549 L
s
.02381 .59107 m
.02756 .59107 L
s
.02381 .60665 m
.02756 .60665 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.06121 .61803 m
.06205 .55355 L
.06423 .43472 L
.06537 .38955 L
.06643 .35418 L
.06885 .29139 L
.0712 .24636 L
.07374 .20951 L
.07619 .18169 L
.07887 .15756 L
.08375 .12489 L
.08849 .10224 L
.0929 .08643 L
.09823 .07188 L
.10324 .06134 L
.11262 .04711 L
.11788 .04126 L
.12271 .03684 L
.12798 .03285 L
.13363 .02933 L
.14304 .0248 L
.15183 .02168 L
.1572 .02019 L
.16228 .019 L
.16681 .01811 L
.1718 .01728 L
.18114 .01611 L
.18625 .01564 L
.19105 .0153 L
.19552 .01506 L
.19962 .0149 L
.20177 .01484 L
.20412 .01478 L
.20531 .01476 L
.20657 .01474 L
.20777 .01473 L
.20886 .01472 L
.21001 .01472 L
.21109 .01472 L
.21227 .01472 L
.21354 .01472 L
.21418 .01473 L
.21487 .01473 L
.21614 .01474 L
.21851 .01478 L
.22074 .01482 L
.22312 .01488 L
.2274 .01502 L
.23284 .01524 L
.23795 .0155 L
Mistroke
.24751 .01611 L
.26695 .01776 L
.28482 .01971 L
.32304 .02503 L
.3598 .03139 L
.39893 .03934 L
.43659 .04806 L
.47662 .05841 L
.51519 .06939 L
.55229 .08089 L
.59177 .09409 L
.62977 .10774 L
.66632 .12174 L
.70523 .13756 L
.74268 .15369 L
.7825 .17181 L
.82085 .19019 L
.85774 .20874 L
.897 .22942 L
.93479 .25022 L
.97496 .27331 L
.97619 .27403 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000nool20004ool00`00ooooo`02ool3000Qool4000Pool40003ool0
0`00ooooo`02ool3000Rool4000Pool40003ool00`00ooooo`02ool3000Rool20007ool003goo`04
003ooooo000;ool00`00ooooo`0Pool00`00ooooo`0Qool00`00ooooo`0:ool00`00ooooo`0Oool0
0`00ooooo`0Qool00`00ooooo`0<ool00`00ooooo`0Nool01000ooooo`001_oo000mool01000oooo
o`002ooo00<00?ooool08?oo00<00?ooool08Ooo00<00?ooool02_oo00<00?ooool08?oo00<00?oo
ool08Ooo00<00?ooool02ooo00<00?ooool08Ooo00<00?ooool01?oo000mool01000ooooo`002?oo
0`008ooo00<00?ooool08Ooo00<00?ooool01ooo0`009?oo00<00?ooool08Ooo00<00?ooool01ooo
0`008_oo0P001ooo000mool01000ooooo`002?oo00<00?ooool08ooo00<00?ooool08Ooo00<00?oo
ool01ooo00<00?ooool09Ooo00<00?ooool08Ooo00<00?ooool01_oo00<00?ooool09?oo00<00?oo
ool01?oo000mool01000ooooo`002?oo00<00?ooool08Ooo0`008Ooo0`002Ooo00<00?ooool08_oo
00@00?ooool0023oo`04003ooooo0008ool00`00ooooo`0Qool01000ooooo`001_oo000Kool00`00
ooooo`0Pool20009ool4000Rool00`00ooooo`0Qool00`00ooooo`07ool4000Rool2000Rool20009
ool4000Qool20007ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001Coool000d000006ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?oo
ool01Ooo00<00?ooool01_oo1`0000?oo`00ool01_oo00<00?ooool01_oo00<00?ooool02P001?oo
00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo
00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01Ooo00<00?ooool01Ooo000Kool00`00ooooo`0Nool20007ool00`00ooooo`0Kool5
0007ool00`00ooooo`0Wool00`00ooooo`0Xool00`00ooooo`0Wool00`00ooooo`0Wool00`00oooo
o`05ool001_oo`03003ooooo01coo`8002coo`L00;7oo`006ooo0P006ooo0P00=Ooo2000ZOoo000K
ool00`00ooooo`0Hool2000oool6002Sool001_oo`03003ooooo01Ooo`03003ooooo04Goo`@009oo
o`006ooo00<00?ooool05_oo00<00?ooool0B_oo0`00W?oo000Kool2000Fool00`00ooooo`1>ool4
002Hool001_oo`03003ooooo01Coo`03003ooooo05?oo`<009Goo`006ooo00<00?ooool04ooo00<0
0?ooool0Eooo0`00T_oo000Kool00`00ooooo`0Cool00`00ooooo`1Jool4002>ool001_oo`8001?o
o`03003ooooo05ooo`@008[oo`006ooo00<00?ooool04_oo00<00?ooool0Hooo1P00Q?oo000Kool0
0`00ooooo`0Aool00`00ooooo`1Zool40020ool001_oo`03003ooooo017oo`03003ooooo06koo`80
07koo`006ooo00<00?ooool04Ooo00<00?ooool0L?oo0P00O?oo000Kool2000Aool00`00ooooo`1c
ool2001jool001_oo`03003ooooo013oo`03003ooooo07Goo`<007Ooo`001ooo10000ooo00<00?oo
ool00_oo0`001Ooo00<00?ooool03ooo00<00?ooool0NOoo1000Looo0007ool00`00ooooo`0<ool0
0`00ooooo`02ool00`00ooooo`0?ool00`00ooooo`1mool4001_ool000Soo`03003ooooo00_oo`03
003ooooo00;oo`80013oo`03003ooooo087oo`<006coo`002Ooo00<00?ooool01ooo0`001Ooo00<0
0?ooool03_oo00<00?ooool0QOoo0P00J_oo000:ool00`00ooooo`06ool00`00ooooo`05ool00`00
ooooo`0>ool00`00ooooo`27ool3001Wool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00
ooooo`0>ool00`00ooooo`2:ool2001Uool000Soo`8000Woo`@000Coo`8000ooo`03003ooooo08co
o`<006;oo`006ooo00<00?ooool03_oo00<00?ooool0Sooo0`00Gooo000Kool00`00ooooo`0=ool0
0`00ooooo`2Cool4001Kool001_oo`03003ooooo00goo`03003ooooo09Ooo`8005Woo`006ooo0P00
3_oo00<00?ooool0VOoo0P00Eooo000Kool00`00ooooo`0=ool00`00ooooo`2Kool2001Eool001_o
o`03003ooooo00goo`03003ooooo09goo`8005?oo`006ooo00<00?ooool03?oo00<00?ooool0X?oo
0P00DOoo000Kool2000=ool00`00ooooo`2Rool3001>ool001_oo`03003ooooo00coo`03003ooooo
0:Goo`8004coo`006ooo00<00?ooool03?oo00<00?ooool0Yooo0`00BOoo000Kool00`00ooooo`0;
ool00`00ooooo`2[ool20017ool001_oo`8000coo`03003ooooo0:goo`<004Coo`006ooo00<00?oo
ool02ooo00<00?ooool0/?oo0P00@_oo000Cool30005ool00`00ooooo`0;ool00`00ooooo`2bool2
0010ool001Koo`03003ooooo00;oo`03003ooooo00_oo`03003ooooo0;Coo`8003koo`005_oo00<0
0?ooool00_oo0P003?oo00<00?ooool0]_oo0P00??oo000Cool30005ool00`00ooooo`0;ool00`00
ooooo`2hool2000jool001?oo`03003ooooo00Goo`03003ooooo00_oo`03003ooooo0;[oo`8003So
o`004ooo00<00?ooool01Ooo00<00?ooool02ooo00<00?ooool0_?oo0P00=_oo000Cool40004ool0
0`00ooooo`0;ool00`00ooooo`2nool2000dool001_oo`8000coo`03003ooooo0<3oo`8003;oo`00
6ooo00<00?ooool02ooo00<00?ooool0`_oo0P00<?oo000Kool00`00ooooo`0;ool00`00ooooo`34
ool2000^ool001_oo`03003ooooo00[oo`03003ooooo0<Ooo`8002coo`006ooo0P002ooo00<00?oo
ool0bOoo0P00:_oo000Kool00`00ooooo`0:ool00`00ooooo`3;ool2000Xool001_oo`03003ooooo
00[oo`03003ooooo0<goo`8002Koo`006ooo00<00?ooool02_oo00<00?ooool0cooo0P009?oo000K
ool2000;ool00`00ooooo`3Aool2000Rool001_oo`03003ooooo00[oo`03003ooooo0=?oo`80023o
o`006ooo00<00?ooool02_oo00<00?ooool0eOoo0P007_oo000Kool00`00ooooo`0:ool00`00oooo
o`3Gool2000Lool001_oo`8000[oo`03003ooooo0=[oo`8001[oo`006ooo00<00?ooool02Ooo00<0
0?ooool0g?oo0P006?oo0008ool00`00ooooo`03ool00`00ooooo`02ool30005ool00`00ooooo`09
ool00`00ooooo`3Nool00`00ooooo`0Eool000Soo`03003ooooo00_oo`03003ooooo00;oo`03003o
oooo00Woo`03003ooooo0=ooo`8001Goo`002Ooo00<00?ooool02_oo00<00?ooool00_oo0P002_oo
00<00?ooool0hOoo0P004ooo0009ool00`00ooooo`07ool30005ool00`00ooooo`09ool00`00oooo
o`3Sool00`00ooooo`0@ool000[oo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Woo`03
003ooooo0>Coo`80013oo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo00Woo`03
003ooooo0>Koo`8000koo`001ooo10002?oo10001?oo0P002_oo00<00?ooool0j?oo0P003?oo000K
ool00`00ooooo`09ool00`00ooooo`3Zool2000:ool001_oo`03003ooooo00Woo`03003ooooo0>co
o`8000Soo`006ooo00<00?ooool02Ooo00<00?ooool0k_oo00<00?ooool01Ooo000Kool2000:ool0
0`00ooooo`3fool001_oo`03003ooooo00Woo`03003ooooo0?Koo`006ooo00<00?ooool02Ooo00<0
0?ooool0m_oo000Kool00`00ooooo`09ool00`00ooooo`3fool001_oo`03003ooooo00Woo`03003o
oooo0?Koo`006ooo0P002_oo00<00?ooool0m_oo000Kool00`00ooooo`09ool00`00ooooo`3fool0
01_oo`03003ooooo00Woo`03003ooooo0?Koo`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000K
ool2000:ool00`00ooooo`3fool001_oo`03003ooooo00Woo`03003ooooo0?Koo`003Ooo10000ooo
0P001Ooo00<00?ooool02Ooo00<00?ooool0m_oo000?ool01@00oooooooo00000_oo00<00?ooool0
0_oo00<00?ooool02Ooo00<00?ooool0m_oo000?ool01@00oooooooo00000_oo00<00?ooool00_oo
0P002Ooo00<00?ooool0mooo000?ool01@00oooooooo00000_oo00<00?ooool00_oo00<00?ooool0
2?oo00<00?ooool0mooo000?ool01@00oooooooo00000_oo00<00?ooool00_oo00<00?ooool02?oo
00<00?ooool0mooo000=ool30003ool01000ooooo`001?oo00<00?ooool02?oo00<00?ooool0mooo
000?ool00`00ooooo`02ool20005ool20009ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03
003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00
ooooo`3gool001_oo`8000Woo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo
000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`00
6ooo0P002Ooo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003o
oooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo000Kool20009ool0
0`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<0
0?ooool0mooo00000ooo000000020002ool40003ool00`00ooooo`02ool30005ool00`00ooooo`08
ool00`00ooooo`3gool000?oo`05003oooooool0000>ool00`00ooooo`02ool00`00ooooo`08ool0
0`00ooooo`3gool000?oo`03003ooooo00;oo`03003ooooo00_oo`03003ooooo00;oo`8000Woo`03
003ooooo0?Ooo`000ooo00<00?ooool00ooo00<00?ooool01ooo0`001Ooo00<00?ooool02?oo00<0
0?ooool0mooo0003ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`08
ool00`00ooooo`3gool00004ool000000003ool01000ooooo`002?oo00<00?ooool01Ooo00<00?oo
ool02?oo00<00?ooool0mooo0003ool00`00ooooo`02ool20009ool40004ool20009ool00`00oooo
o`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0
mooo000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`8000Woo`03003ooooo0?Ooo`006ooo
00<00?ooool02?oo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`03
003ooooo00Soo`03003ooooo0?Ooo`006ooo0P002Ooo00<00?ooool0mooo000Kool00`00ooooo`08
ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo
00<00?ooool0mooo000Kool20009ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo
0?Ooo`003Ooo10000_oo0`001Ooo00<00?ooool02?oo00<00?ooool0mooo000?ool00`00ooooo`04
ool00`00ooooo`02ool00`00ooooo`08ool00`00ooooo`3gool000ooo`03003ooooo00Coo`03003o
oooo00;oo`8000Woo`03003ooooo0?Ooo`003ooo00@00?ooooooo`<000Goo`03003ooooo00Soo`03
003ooooo0?Ooo`003ooo00D00?ooooooo`0000Ooo`03003ooooo00Soo`03003ooooo0?Ooo`003Ooo
0`000ooo00<00?ooool01Ooo00<00?ooool02?oo00<00?ooool0mooo000?ool01000oooooooo1000
1?oo0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003o
oooo00Ooo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?ooool0n?oo000Kool00`00oooo
o`07ool00`00ooooo`3hool001_oo`8000Soo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<0
0?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003o
oooo0?Soo`006ooo0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool0
01_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?ooool0n?oo000K
ool20008ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?Soo`0000?oo`000000
0P000ooo00<00?ooool00ooo00<00?ooool00_oo0`001Ooo00<00?ooool01ooo00<00?ooool0n?oo
0003ool00`00ooooo`02ool00`00ooooo`0;ool00`00ooooo`02ool00`00ooooo`07ool00`00oooo
o`3hool000?oo`03003ooooo00?oo`03003ooooo00[oo`03003ooooo00;oo`8000Soo`03003ooooo
0?Soo`000ooo00<00?ooool00ooo00<00?ooool01ooo0`001Ooo00<00?ooool01ooo00<00?ooool0
n?oo0003ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`07ool00`00
ooooo`3hool00004ool000000003ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool01ooo
00<00?ooool0n?oo0003ool01000oooooooo10002?oo10001?oo0P002?oo00<00?ooool0n?oo000K
ool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo
00<00?ooool01ooo00<00?ooool0n?oo000Kool20008ool00`00ooooo`3hool001_oo`03003ooooo
00Ooo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?ooool0n?oo000Kool00`00ooooo`07
ool00`00ooooo`3hool001_oo`8000Soo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?oo
ool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo
0?Soo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.323594, -1.73616, \
0.0118419, 0.120668}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[kx0corrtestfunc[Om - 1.64\ I, 0.1, 0.1]], {Om, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.217227 0.00455649 [
[.18254 .20473 -9 -9 ]
[.18254 .20473 9 0 ]
[.34127 .20473 -3 -9 ]
[.34127 .20473 3 0 ]
[.5 .20473 -9 -9 ]
[.5 .20473 9 0 ]
[.65873 .20473 -3 -9 ]
[.65873 .20473 3 0 ]
[.81746 .20473 -9 -9 ]
[.81746 .20473 9 0 ]
[.97619 .20473 -3 -9 ]
[.97619 .20473 3 0 ]
[.01131 .03497 -18 -4.5 ]
[.01131 .03497 0 4.5 ]
[.01131 .1261 -18 -4.5 ]
[.01131 .1261 0 4.5 ]
[.01131 .30836 -12 -4.5 ]
[.01131 .30836 0 4.5 ]
[.01131 .39949 -12 -4.5 ]
[.01131 .39949 0 4.5 ]
[.01131 .49062 -12 -4.5 ]
[.01131 .49062 0 4.5 ]
[.01131 .58175 -12 -4.5 ]
[.01131 .58175 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .21723 m
.18254 .22348 L
s
[(0.5)] .18254 .20473 0 1 Mshowa
.34127 .21723 m
.34127 .22348 L
s
[(1)] .34127 .20473 0 1 Mshowa
.5 .21723 m
.5 .22348 L
s
[(1.5)] .5 .20473 0 1 Mshowa
.65873 .21723 m
.65873 .22348 L
s
[(2)] .65873 .20473 0 1 Mshowa
.81746 .21723 m
.81746 .22348 L
s
[(2.5)] .81746 .20473 0 1 Mshowa
.97619 .21723 m
.97619 .22348 L
s
[(3)] .97619 .20473 0 1 Mshowa
.125 Mabswid
.05556 .21723 m
.05556 .22098 L
s
.0873 .21723 m
.0873 .22098 L
s
.11905 .21723 m
.11905 .22098 L
s
.15079 .21723 m
.15079 .22098 L
s
.21429 .21723 m
.21429 .22098 L
s
.24603 .21723 m
.24603 .22098 L
s
.27778 .21723 m
.27778 .22098 L
s
.30952 .21723 m
.30952 .22098 L
s
.37302 .21723 m
.37302 .22098 L
s
.40476 .21723 m
.40476 .22098 L
s
.43651 .21723 m
.43651 .22098 L
s
.46825 .21723 m
.46825 .22098 L
s
.53175 .21723 m
.53175 .22098 L
s
.56349 .21723 m
.56349 .22098 L
s
.59524 .21723 m
.59524 .22098 L
s
.62698 .21723 m
.62698 .22098 L
s
.69048 .21723 m
.69048 .22098 L
s
.72222 .21723 m
.72222 .22098 L
s
.75397 .21723 m
.75397 .22098 L
s
.78571 .21723 m
.78571 .22098 L
s
.84921 .21723 m
.84921 .22098 L
s
.88095 .21723 m
.88095 .22098 L
s
.9127 .21723 m
.9127 .22098 L
s
.94444 .21723 m
.94444 .22098 L
s
.25 Mabswid
0 .21723 m
1 .21723 L
s
.02381 .03497 m
.03006 .03497 L
s
[(-40)] .01131 .03497 1 0 Mshowa
.02381 .1261 m
.03006 .1261 L
s
[(-20)] .01131 .1261 1 0 Mshowa
.02381 .30836 m
.03006 .30836 L
s
[(20)] .01131 .30836 1 0 Mshowa
.02381 .39949 m
.03006 .39949 L
s
[(40)] .01131 .39949 1 0 Mshowa
.02381 .49062 m
.03006 .49062 L
s
[(60)] .01131 .49062 1 0 Mshowa
.02381 .58175 m
.03006 .58175 L
s
[(80)] .01131 .58175 1 0 Mshowa
.125 Mabswid
.02381 .05775 m
.02756 .05775 L
s
.02381 .08053 m
.02756 .08053 L
s
.02381 .10331 m
.02756 .10331 L
s
.02381 .14888 m
.02756 .14888 L
s
.02381 .17166 m
.02756 .17166 L
s
.02381 .19444 m
.02756 .19444 L
s
.02381 .24001 m
.02756 .24001 L
s
.02381 .26279 m
.02756 .26279 L
s
.02381 .28557 m
.02756 .28557 L
s
.02381 .33114 m
.02756 .33114 L
s
.02381 .35392 m
.02756 .35392 L
s
.02381 .3767 m
.02756 .3767 L
s
.02381 .42227 m
.02756 .42227 L
s
.02381 .44505 m
.02756 .44505 L
s
.02381 .46783 m
.02756 .46783 L
s
.02381 .5134 m
.02756 .5134 L
s
.02381 .53618 m
.02756 .53618 L
s
.02381 .55896 m
.02756 .55896 L
s
.02381 .01218 m
.02756 .01218 L
s
.02381 .60453 m
.02756 .60453 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .60332 m
.07374 .5745 L
.0929 .53282 L
.13363 .41843 L
.17042 .30483 L
.20958 .19477 L
.24727 .11198 L
.26787 .07813 L
.28734 .05349 L
.29671 .04405 L
.30664 .03566 L
.31595 .02923 L
.32447 .02449 L
.33357 .02055 L
.33825 .01895 L
.34335 .01751 L
.34806 .01647 L
.35242 .01573 L
.3566 .01521 L
.35893 .01501 L
.36104 .01487 L
.36221 .01481 L
.36329 .01477 L
.36448 .01474 L
.36576 .01472 L
.3664 .01472 L
.36709 .01472 L
.36836 .01473 L
.36959 .01475 L
.37073 .01479 L
.37179 .01484 L
.37279 .01489 L
.37506 .01504 L
.37727 .01523 L
.37965 .01547 L
.38476 .01616 L
.39015 .0171 L
.39978 .01929 L
.41886 .02526 L
.43661 .03236 L
.47436 .05063 L
.51447 .07222 L
.55312 .09307 L
.5903 .11207 L
.62986 .13054 L
.66795 .14645 L
.70457 .16003 L
.74357 .17278 L
.7811 .18358 L
.821 .19371 L
Mistroke
.85944 .20238 L
.89641 .20989 L
.93575 .21715 L
.97362 .22358 L
.97619 .224 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo05Goo`/00:Soo`005Ooo
00<00?ooool0D_oo0`002ooo1@00Xooo000Eool00`00ooooo`1@ool2000Cool2002Qool000Woo`03
003ooooo00;oo`8000Goo`03003ooooo04koo`8001Ooo`<009koo`002Ooo00D00?ooooooo`0000;o
o`03003ooooo00;oo`03003ooooo04coo`8001coo`8009coo`001_oo1@000_oo00@00?ooool000Co
o`<004_oo`03003ooooo01koo`8009[oo`0000?oo`0000000P0000Goo`00ooooo`0000?oo`04003o
oooo0004ool00`00ooooo`1:ool00`00ooooo`0Qool2002Hool000Ooo`03003oo`0000?oo`04003o
oooo0004ool00`00ooooo`18ool2000Vool2002Fool000Soo`8000?oo`04003ooooo0004ool00`00
ooooo`17ool00`00ooooo`0Xool2002Dool000Woo`03003ooooo00;oo`8000Goo`03003ooooo04Ko
o`03003ooooo02_oo`8009;oo`005Ooo00<00?ooool0A_oo00<00?ooool0;Ooo0P00T?oo000Eool0
0`00ooooo`15ool00`00ooooo`0`ool2002>ool001Goo`8004Goo`03003ooooo03?oo`8008coo`00
5Ooo00<00?ooool0@ooo00<00?ooool0=_oo0P00R_oo000Eool00`00ooooo`12ool00`00ooooo`0i
ool20028ool001Goo`03003ooooo047oo`03003ooooo03coo`8008Koo`005Ooo00<00?ooool0@?oo
00<00?ooool0?ooo0P00Q?oo000Eool00`00ooooo`0oool00`00ooooo`12ool20022ool001Goo`80
03ooo`03003ooooo04Goo`80083oo`005Ooo00<00?ooool0?_oo00<00?ooool0Aooo0P00O_oo000E
ool00`00ooooo`0mool00`00ooooo`1:ool2001lool001Goo`03003ooooo03goo`03003ooooo04co
o`8007[oo`005Ooo00<00?ooool0??oo00<00?ooool0Cooo0P00N?oo000Eool00`00ooooo`0lool0
0`00ooooo`1Aool2001fool001Goo`8003coo`03003ooooo05Coo`8007Coo`005Ooo00<00?ooool0
>ooo00<00?ooool0E_oo0P00L_oo000Eool00`00ooooo`0jool00`00ooooo`1Iool3001_ool001Go
o`03003ooooo03[oo`03003ooooo05coo`8006goo`001ooo10000ooo0P001Ooo00<00?ooool0>Ooo
00<00?ooool0Gooo0P00Jooo0007ool00`00ooooo`03ool01000ooooo`001?oo00<00?ooool0>Ooo
00<00?ooool0HOoo0P00JOoo0008ool00`00ooooo`02ool01000ooooo`001?oo0`00>?oo00<00?oo
ool0I?oo0P00Iooo00000ooo000000020004ool01@00oooooooo00000_oo00<00?ooool00_oo00<0
0?ooool0>?oo00<00?ooool0I_oo0P00IOoo000:ool01000ooooo`000_oo00<00?ooool00_oo00<0
0?ooool0=ooo00<00?ooool0JOoo0P00Hooo0007ool01000ooooo`000_oo00@00?ooool000Coo`03
003ooooo03Ooo`03003ooooo06_oo`80067oo`002?oo0P001?oo0P001Ooo00<00?ooool0=_oo00<0
0?ooool0K_oo0P00Gooo000Eool00`00ooooo`0fool00`00ooooo`1`ool2001Mool001Goo`8003Ko
o`03003ooooo07?oo`<005[oo`005Ooo00<00?ooool0=Ooo00<00?ooool0M_oo0`00Eooo000Eool0
0`00ooooo`0dool00`00ooooo`1jool4001Cool001Goo`03003ooooo03Coo`03003ooooo07koo`<0
053oo`005Ooo00<00?ooool0<ooo00<00?ooool0P_oo0`00COoo000Eool00`00ooooo`0cool00`00
ooooo`25ool2001;ool001Goo`03003ooooo03;oo`03003ooooo08Soo`<004Soo`005Ooo0P00<ooo
00<00?ooool0Rooo0`00AOoo000Eool00`00ooooo`0aool00`00ooooo`2?ool30012ool001Goo`03
003ooooo027oo`8000Coo`03003ooooo00;oo`<000;oo`03003ooooo01goo`@0027oo`@000?oo`03
003ooooo00;oo`<002?oo`@001[oo`@000?oo`@000?oo`03003ooooo00;oo`<002?oo`8000Ooo`00
5Ooo00<00?ooool08?oo00@00?ooool000_oo`8002;oo`03003ooooo02;oo`03003ooooo00[oo`03
003ooooo023oo`03003ooooo01ooo`@000koo`03003ooooo01ooo`04003ooooo0006ool001Goo`03
003ooooo023oo`04003ooooo000;ool2000Rool00`00ooooo`0Rool00`00ooooo`0:ool00`00oooo
o`0Qool00`00ooooo`0Qool4000;ool00`00ooooo`0Rool00`00ooooo`04ool001Goo`03003ooooo
023oo`04003ooooo0008ool4000Sool00`00ooooo`0Rool00`00ooooo`07ool3000Uool00`00oooo
o`0Rool00`00ool000030004ool3000Sool20007ool001Goo`80027oo`04003ooooo0008ool01000
ooooo`008ooo00<00?ooool08_oo00<00?ooool01ooo00<00?ooool09_oo00<00?ooool08_oo00<0
0?ooool00_oo1@009ooo00<00?ooool01?oo000Eool00`00ooooo`0Pool01000ooooo`002?oo00@0
0?ooool0027oo`<002;oo`<000Woo`03003ooooo02?oo`04003ooooo000Qool01000ooooo`002?oo
1@008?oo00@00?ooool000Koo`005Ooo00<00?ooool08Ooo0P002Ooo10008ooo00<00?ooool08_oo
00<00?ooool01ooo10008ooo0P008ooo0P002Ooo100000?oo`0000000`007?oo0P001ooo000Eool0
0`00ooooo`0^ool00`00ooooo`2bool5000Pool001Goo`03003ooooo02koo`03003ooooo0;Ooo`D0
01_oo`005Ooo00<00?ooool0;Ooo00<00?ooool0_Ooo1P005Ooo000>oooo000C000001Goo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003o
oooo00;oo`05003oooooool00007ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool60003ool00`00oooo
o`05ool001Goo`03003ooooo02Soo`05003oooooool0000Vool00`00ooooo`0Xool00`00ooooo`0Y
ool00`00ooooo`0Xool00`00ooooo`0Uool40007ool001Goo`03003ooooo02coo`03003ooooo0=Wo
o`005Ooo00<00?ooool0;?oo00<00?ooool0fOoo000Eool00`00ooooo`0[ool00`00ooooo`3Jool0
01Goo`8002coo`03003ooooo0=[oo`005Ooo00<00?ooool0:ooo00<00?ooool0f_oo000Eool00`00
ooooo`0Zool00`00ooooo`3Kool001Goo`03003ooooo02[oo`03003ooooo0=_oo`005Ooo00<00?oo
ool0:_oo00<00?ooool0fooo000Eool00`00ooooo`0Yool00`00ooooo`3Lool001Goo`8002[oo`03
003ooooo0=coo`005Ooo00<00?ooool0:Ooo00<00?ooool0g?oo000Eool00`00ooooo`0Xool00`00
ooooo`3Mool001Goo`03003ooooo02Soo`03003ooooo0=goo`005Ooo00<00?ooool0:?oo00<00?oo
ool0gOoo000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03003ooooo
0=koo`005Ooo0P00:?oo00<00?ooool0g_oo000Eool00`00ooooo`0Vool00`00ooooo`3Oool001Go
o`03003ooooo02Koo`03003ooooo0=ooo`005Ooo00<00?ooool09_oo00<00?ooool0gooo0007ool4
0003ool20005ool00`00ooooo`0Uool00`00ooooo`3Pool000Ooo`03003ooooo00?oo`04003ooooo
0004ool00`00ooooo`0Uool00`00ooooo`3Pool000Soo`03003ooooo00;oo`04003ooooo0004ool3
000Uool00`00ooooo`3Pool000Woo`05003oooooool00002ool00`00ooooo`02ool00`00ooooo`0T
ool00`00ooooo`3Qool000[oo`04003ooooo0002ool00`00ooooo`02ool00`00ooooo`0Tool00`00
ooooo`3Qool000Ooo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool09?oo00<00?ooool0
hOoo0008ool20004ool20005ool00`00ooooo`0Sool00`00ooooo`3Rool001Goo`03003ooooo02?o
o`03003ooooo0>;oo`005Ooo0P009?oo00<00?ooool0h_oo000Eool00`00ooooo`0Rool00`00oooo
o`3Sool001Goo`03003ooooo02;oo`03003ooooo0>?oo`005Ooo00<00?ooool08_oo00<00?ooool0
hooo000Eool00`00ooooo`0Qool00`00ooooo`3Tool001Goo`03003ooooo027oo`03003ooooo0>Co
o`005Ooo0P008_oo00<00?ooool0i?oo000Eool00`00ooooo`0Pool00`00ooooo`3Uool001Goo`03
003ooooo023oo`03003ooooo0>Goo`005Ooo00<00?ooool08?oo00<00?ooool0iOoo000Eool00`00
ooooo`0Pool00`00ooooo`3Uool001Goo`03003ooooo01ooo`03003ooooo0>Koo`005Ooo0P008?oo
00<00?ooool0i_oo000Eool00`00ooooo`0Oool00`00ooooo`3Vool001Goo`03003ooooo01koo`03
003ooooo0>Ooo`005Ooo00<00?ooool07_oo00<00?ooool0iooo000Eool00`00ooooo`0Nool00`00
ooooo`3Wool000Woo`03003ooooo00;oo`8000Goo`03003ooooo01goo`03003ooooo0>Soo`002Ooo
00D00?ooooooo`0000;oo`03003ooooo00;oo`03003ooooo01goo`03003ooooo0>Soo`001_oo1@00
0_oo00@00?ooool000Coo`<001goo`03003ooooo0>Soo`001_oo00@00?ooool000?oo`04003ooooo
0004ool00`00ooooo`0Lool00`00ooooo`3Yool000Ooo`03003oo`0000?oo`04003ooooo0004ool0
0`00ooooo`0Lool00`00ooooo`3Yool000Soo`8000?oo`04003ooooo0004ool00`00ooooo`0Lool0
0`00ooooo`3Yool000Woo`03003ooooo00;oo`8000Goo`03003ooooo01_oo`03003ooooo0>[oo`00
5Ooo00<00?ooool06ooo00<00?ooool0j_oo000Eool2000Lool00`00ooooo`3Zool001Goo`03003o
oooo01[oo`03003ooooo0>_oo`005Ooo00<00?ooool06_oo00<00?ooool0jooo000Eool00`00oooo
o`0Jool00`00ooooo`3[ool001Goo`03003ooooo01Woo`03003ooooo0>coo`005Ooo00<00?ooool0
6Ooo00<00?ooool0k?oo000Eool2000Jool00`00ooooo`3/ool001Goo`03003ooooo01Soo`03003o
oooo0>goo`005Ooo00<00?ooool06?oo00<00?ooool0kOoo000Eool00`00ooooo`0Gool00`00oooo
o`3^ool001Goo`03003ooooo01Ooo`03003ooooo0>koo`005Ooo00<00?ooool05ooo00<00?ooool0
k_oo000Eool2000Gool00`00ooooo`3_ool001Goo`03003ooooo01Koo`03003ooooo0>ooo`005Ooo
00<00?ooool05_oo00<00?ooool0kooo000Eool00`00ooooo`0Eool00`00ooooo`3`ool000Soo`80
00Coo`8000Goo`03003ooooo01Goo`03003ooooo0?3oo`001ooo00@00?ooool000;oo`04003ooooo
0004ool00`00ooooo`0Eool00`00ooooo`3`ool000Ooo`04003ooooo0002ool01000ooooo`001?oo
0`005?oo00<00?ooool0lOoo0007ool30003ool01000ooooo`001?oo00<00?ooool05?oo00<00?oo
ool0lOoo0007ool00`00ooooo`03ool01000ooooo`001?oo00<00?ooool05?oo00<00?ooool0lOoo
0007ool00`00ooooo`03ool01000ooooo`001?oo00<00?ooool04ooo00<00?ooool0l_oo0008ool3
0003ool20005ool00`00ooooo`0Cool00`00ooooo`3bool001Goo`03003ooooo01;oo`03003ooooo
0??oo`005Ooo00<00?ooool04_oo00<00?ooool0looo000Eool2000Cool00`00ooooo`3cool001Go
o`03003ooooo017oo`03003ooooo0?Coo`005Ooo00<00?ooool04Ooo00<00?ooool0m?oo000Eool0
0`00ooooo`0Aool00`00ooooo`3dool001Goo`03003ooooo013oo`03003ooooo0?Goo`005Ooo00<0
0?ooool04?oo00<00?ooool0mOoo000Eool2000@ool00`00ooooo`3fool001Goo`03003ooooo00oo
o`03003ooooo0?Koo`005Ooo00<00?ooool03_oo00<00?ooool0mooo000Eool00`00ooooo`0>ool0
0`00ooooo`3gool001Goo`03003ooooo00goo`03003ooooo0?Soo`005Ooo00<00?ooool03Ooo00<0
0?ooool0n?oo000Eool2000=ool00`00ooooo`3iool001Goo`03003ooooo00coo`03003ooooo0?Wo
o`005Ooo00<00?ooool02ooo00<00?ooool0n_oo000Eool00`00ooooo`0;ool00`00ooooo`3jool0
00Soo`8000Coo`8000Goo`03003ooooo00[oo`03003ooooo0?_oo`001ooo00@00?ooool000;oo`04
003ooooo0004ool00`00ooooo`0:ool00`00ooooo`3kool000Ooo`04003ooooo0002ool01000oooo
o`001?oo0`002Ooo00<00?ooool0o?oo0008ool20003ool01000ooooo`001?oo00<00?ooool02Ooo
00<00?ooool0o?oo0007ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo00Soo`03003o
oooo0?goo`001ooo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`08ool00`00ooooo`3m
ool000Soo`8000Coo`8000Goo`03003ooooo00Ooo`03003ooooo0?koo`005Ooo00<00?ooool01ooo
00<00?ooool0o_oo000Eool20007ool00`00ooooo`3oool001Goo`03003ooooo0?ooo`Woo`005Ooo
00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00oooo
8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -51.2183, \
0.0115777, 0.806644}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Im[kx0corrtestfunc[Om - 1.64\ I, 0.1, 0.1]], {Om, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.56481 0.0070175 [
[.18254 .55231 -9 -9 ]
[.18254 .55231 9 0 ]
[.34127 .55231 -3 -9 ]
[.34127 .55231 3 0 ]
[.5 .55231 -9 -9 ]
[.5 .55231 9 0 ]
[.65873 .55231 -3 -9 ]
[.65873 .55231 3 0 ]
[.81746 .55231 -9 -9 ]
[.81746 .55231 9 0 ]
[.97619 .55231 -3 -9 ]
[.97619 .55231 3 0 ]
[.01131 .00341 -18 -4.5 ]
[.01131 .00341 0 4.5 ]
[.01131 .14376 -18 -4.5 ]
[.01131 .14376 0 4.5 ]
[.01131 .28411 -18 -4.5 ]
[.01131 .28411 0 4.5 ]
[.01131 .42446 -18 -4.5 ]
[.01131 .42446 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .56481 m
.18254 .57106 L
s
[(0.5)] .18254 .55231 0 1 Mshowa
.34127 .56481 m
.34127 .57106 L
s
[(1)] .34127 .55231 0 1 Mshowa
.5 .56481 m
.5 .57106 L
s
[(1.5)] .5 .55231 0 1 Mshowa
.65873 .56481 m
.65873 .57106 L
s
[(2)] .65873 .55231 0 1 Mshowa
.81746 .56481 m
.81746 .57106 L
s
[(2.5)] .81746 .55231 0 1 Mshowa
.97619 .56481 m
.97619 .57106 L
s
[(3)] .97619 .55231 0 1 Mshowa
.125 Mabswid
.05556 .56481 m
.05556 .56856 L
s
.0873 .56481 m
.0873 .56856 L
s
.11905 .56481 m
.11905 .56856 L
s
.15079 .56481 m
.15079 .56856 L
s
.21429 .56481 m
.21429 .56856 L
s
.24603 .56481 m
.24603 .56856 L
s
.27778 .56481 m
.27778 .56856 L
s
.30952 .56481 m
.30952 .56856 L
s
.37302 .56481 m
.37302 .56856 L
s
.40476 .56481 m
.40476 .56856 L
s
.43651 .56481 m
.43651 .56856 L
s
.46825 .56481 m
.46825 .56856 L
s
.53175 .56481 m
.53175 .56856 L
s
.56349 .56481 m
.56349 .56856 L
s
.59524 .56481 m
.59524 .56856 L
s
.62698 .56481 m
.62698 .56856 L
s
.69048 .56481 m
.69048 .56856 L
s
.72222 .56481 m
.72222 .56856 L
s
.75397 .56481 m
.75397 .56856 L
s
.78571 .56481 m
.78571 .56856 L
s
.84921 .56481 m
.84921 .56856 L
s
.88095 .56481 m
.88095 .56856 L
s
.9127 .56481 m
.9127 .56856 L
s
.94444 .56481 m
.94444 .56856 L
s
.25 Mabswid
0 .56481 m
1 .56481 L
s
.02381 .00341 m
.03006 .00341 L
s
[(-80)] .01131 .00341 1 0 Mshowa
.02381 .14376 m
.03006 .14376 L
s
[(-60)] .01131 .14376 1 0 Mshowa
.02381 .28411 m
.03006 .28411 L
s
[(-40)] .01131 .28411 1 0 Mshowa
.02381 .42446 m
.03006 .42446 L
s
[(-20)] .01131 .42446 1 0 Mshowa
.125 Mabswid
.02381 .0385 m
.02756 .0385 L
s
.02381 .07359 m
.02756 .07359 L
s
.02381 .10867 m
.02756 .10867 L
s
.02381 .17885 m
.02756 .17885 L
s
.02381 .21394 m
.02756 .21394 L
s
.02381 .24902 m
.02756 .24902 L
s
.02381 .3192 m
.02756 .3192 L
s
.02381 .35429 m
.02756 .35429 L
s
.02381 .38937 m
.02756 .38937 L
s
.02381 .45955 m
.02756 .45955 L
s
.02381 .49464 m
.02756 .49464 L
s
.02381 .52972 m
.02756 .52972 L
s
.02381 .5999 m
.02756 .5999 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .37888 m
.07374 .28202 L
.0929 .19295 L
.11234 .11963 L
.12349 .08618 L
.13363 .06139 L
.14336 .0427 L
.15255 .02954 L
.1576 .02413 L
.16226 .02025 L
.1649 .01852 L
.16737 .0172 L
.16995 .01612 L
.17132 .01568 L
.17279 .01529 L
.17408 .01504 L
.17524 .01487 L
.17589 .01481 L
.17659 .01475 L
.17786 .01472 L
.17905 .01474 L
.18031 .01484 L
.18138 .01497 L
.18256 .01518 L
.18395 .01549 L
.18522 .01585 L
.18807 .01689 L
.19072 .01815 L
.19322 .01958 L
.20252 .02696 L
.21264 .03831 L
.23176 .06777 L
.25206 .10782 L
.28856 .19254 L
.32742 .28654 L
.36482 .36958 L
.40459 .44363 L
.44289 .49929 L
.47973 .53905 L
.49869 .55481 L
.51894 .56856 L
.53819 .57898 L
.55905 .58773 L
.57916 .59398 L
.58874 .5963 L
.5977 .59812 L
.60718 .59971 L
.61584 .60088 L
.62546 .60188 L
.63046 .60229 L
Mistroke
.63579 .60265 L
.6408 .60291 L
.64533 .60309 L
.64752 .60316 L
.64989 .60321 L
.65213 .60326 L
.65418 .60329 L
.65534 .6033 L
.65659 .60331 L
.65776 .60332 L
.65883 .60332 L
.66009 .60332 L
.6608 .60332 L
.66145 .60331 L
.66266 .60331 L
.66393 .6033 L
.66515 .60328 L
.66646 .60327 L
.66883 .60322 L
.67165 .60316 L
.67423 .60309 L
.68373 .60271 L
.69256 .60222 L
.71258 .60066 L
.73239 .59861 L
.75037 .59641 L
.78907 .59083 L
.82631 .58478 L
.86591 .57796 L
.90405 .57127 L
.94073 .56487 L
.97619 .55879 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool000Soo`8000Coo`800?oooa7oo`001ooo00@00?ooool0
00;oo`04003ooooo0004ool00`00ooooo`3oool9ool000Ooo`04003ooooo0002ool01000ooooo`00
1?oo0`00oooo2Ooo00000ooo000000020003ool20003ool01000ooooo`001?oo00<00?ooool0oooo
2Ooo0007ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo0?ooo`Woo`001ooo00@00?oo
ool000;oo`04003ooooo0004ool00`00ooooo`0Tool7003Mool000Soo`8000Coo`8000Goo`03003o
oooo02;oo`8000Koo`800=coo`005Ooo00<00?ooool08Ooo00<00?ooool02?oo0P00f_oo000Eool0
0`00ooooo`0Qool00`00ooooo`0:ool00`00ooooo`3Gool001Goo`03003ooooo023oo`03003ooooo
00coo`03003ooooo0=Koo`005Ooo00<00?ooool07ooo00<00?ooool03Ooo00<00?ooool0e_oo000E
ool2000Oool00`00ooooo`0?ool00`00ooooo`3Eool001Goo`03003ooooo01goo`03003ooooo017o
o`03003ooooo0=Coo`005Ooo00<00?ooool07Ooo00<00?ooool04Ooo00<00?ooool0e?oo000Eool0
0`00ooooo`0Lool00`00ooooo`0Cool00`00ooooo`3Cool001Goo`03003ooooo01coo`03003ooooo
01?oo`03003ooooo0=?oo`005Ooo00<00?ooool06ooo00<00?ooool05Ooo00<00?ooool0d_oo000E
ool00`00ooooo`0Kool00`00ooooo`0Fool00`00ooooo`3Aool001Goo`03003ooooo01_oo`03003o
oooo01Koo`03003ooooo0=7oo`005Ooo00<00?ooool06_oo00<00?ooool06?oo00<00?ooool0d?oo
000Eool00`00ooooo`0Jool00`00ooooo`0Iool00`00ooooo`3?ool001Goo`8001[oo`03003ooooo
01[oo`03003ooooo0<ooo`005Ooo00<00?ooool06Ooo00<00?ooool06ooo00<00?ooool0c_oo000E
ool00`00ooooo`0Hool00`00ooooo`0Lool00`00ooooo`3>ool001Goo`03003ooooo01Soo`03003o
oooo01goo`03003ooooo0<goo`005Ooo00<00?ooool06?oo00<00?ooool07Ooo00<00?ooool0cOoo
000Eool00`00ooooo`0Gool00`00ooooo`0Oool00`00ooooo`3<ool001Goo`03003ooooo01Ooo`03
003ooooo01ooo`03003ooooo0<coo`005Ooo00<00?ooool05ooo00<00?ooool08?oo00<00?ooool0
booo000Eool00`00ooooo`0Fool00`00ooooo`0Qool00`00ooooo`3;ool001Goo`8001Ooo`03003o
oooo02;oo`03003ooooo0<[oo`005Ooo00<00?ooool05_oo00<00?ooool08_oo00<00?ooool0b_oo
000Eool00`00ooooo`0Eool00`00ooooo`0Tool00`00ooooo`39ool001Goo`03003ooooo01Goo`03
003ooooo02Coo`03003ooooo0<Woo`005Ooo00<00?ooool05Ooo00<00?ooool09Ooo00<00?ooool0
b?oo000Eool00`00ooooo`0Eool00`00ooooo`0Uool00`00ooooo`38ool001Goo`03003ooooo01Co
o`03003ooooo02Ooo`03003ooooo0<Ooo`005Ooo00<00?ooool05?oo00<00?ooool09ooo00<00?oo
ool0aooo0008ool20004ool20005ool00`00ooooo`0Dool00`00ooooo`0Wool00`00ooooo`37ool0
00Ooo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool05?oo00<00?ooool0:?oo00<00?oo
ool0a_oo0007ool01000ooooo`000_oo00@00?ooool000Coo`<001?oo`03003ooooo02Woo`03003o
oooo0<Koo`0000?oo`0000000P000_oo0`000ooo00@00?ooool000Coo`03003ooooo01?oo`03003o
oooo02[oo`03003ooooo0<Goo`001ooo00<00?ooool00ooo00@00?ooool000Coo`03003ooooo01?o
o`03003ooooo02[oo`03003ooooo0<Goo`001ooo00<00?ooool00ooo00@00?ooool000Coo`03003o
oooo01?oo`03003ooooo02_oo`03003ooooo0<Coo`002?oo0`000ooo0P001Ooo00<00?ooool04_oo
00<00?ooool0;?oo00<00?ooool0a?oo000Eool00`00ooooo`0Bool00`00ooooo`0]ool00`00oooo
o`33ool001Goo`03003ooooo01;oo`03003ooooo02goo`03003ooooo0<?oo`005Ooo00<00?ooool0
4_oo00<00?ooool0;Ooo00<00?ooool0`ooo000Eool00`00ooooo`0Aool00`00ooooo`0_ool00`00
ooooo`32ool001Goo`8001;oo`03003ooooo02ooo`03003ooooo0<;oo`005Ooo00<00?ooool04Ooo
00<00?ooool0<?oo00<00?ooool0`Ooo000Eool00`00ooooo`0Aool00`00ooooo`0`ool00`00oooo
o`31ool001Goo`03003ooooo013oo`03003ooooo03;oo`03003ooooo0<3oo`005Ooo00<00?ooool0
4?oo00<00?ooool0<_oo00<00?ooool0`?oo000Eool00`00ooooo`0@ool00`00ooooo`0bool00`00
ooooo`30ool001Goo`03003ooooo013oo`03003ooooo03?oo`03003ooooo0;ooo`005Ooo00<00?oo
ool03ooo00<00?ooool0=?oo00<00?ooool0_ooo000Eool00`00ooooo`0?ool00`00ooooo`0eool0
0`00ooooo`2nool001Goo`03003ooooo00ooo`03003ooooo03Goo`03003ooooo0;koo`005Ooo0P00
4?oo00<00?ooool0=_oo00<00?ooool0_Ooo000Eool00`00ooooo`0>ool00`00ooooo`0gool00`00
ooooo`2mool001Goo`03003ooooo00koo`03003ooooo03Ooo`03003ooooo0;goo`005Ooo00<00?oo
ool03_oo00<00?ooool0>?oo00<00?ooool0_?oo000Eool00`00ooooo`0>ool00`00ooooo`0hool0
0`00ooooo`2lool001Goo`03003ooooo00goo`03003ooooo03[oo`03003ooooo0;_oo`005Ooo00<0
0?ooool03Ooo00<00?ooool0>_oo00<00?ooool0^ooo000Eool00`00ooooo`0=ool00`00ooooo`0j
ool00`00ooooo`2kool001Goo`03003ooooo00goo`03003ooooo03_oo`03003ooooo0;[oo`005Ooo
0P003Ooo00<00?ooool0??oo00<00?ooool0^_oo000Eool00`00ooooo`0<ool00`00ooooo`0mool0
0`00ooooo`2iool001Goo`03003ooooo00coo`03003ooooo03goo`03003ooooo0;Woo`005Ooo00<0
0?ooool03?oo00<00?ooool0?_oo00<00?ooool0^?oo000Eool00`00ooooo`0;ool00`00ooooo`0o
ool00`00ooooo`2hool001Goo`03003ooooo00_oo`03003ooooo03ooo`03003ooooo0;Soo`005Ooo
00<00?ooool02ooo00<00?ooool0@?oo00<00?ooool0]ooo000Eool00`00ooooo`0;ool00`00oooo
o`10ool00`00ooooo`2gool000Woo`03003ooooo00;oo`8000Goo`03003ooooo00[oo`03003ooooo
04;oo`03003ooooo0;Koo`002Ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`03003ooooo00[o
o`03003ooooo04;oo`03003ooooo0;Koo`001_oo1@000_oo00@00?ooool000Coo`<000[oo`03003o
oooo04?oo`03003ooooo0;Goo`0000?oo`0000000P0000Goo`00ooooo`0000?oo`04003ooooo0004
ool00`00ooooo`0:ool00`00ooooo`13ool00`00ooooo`2eool000Ooo`03003oo`0000?oo`04003o
oooo0004ool00`00ooooo`0:ool00`00ooooo`13ool00`00ooooo`2eool000Soo`8000?oo`04003o
oooo0004ool00`00ooooo`09ool00`00ooooo`15ool00`00ooooo`2dool000Woo`03003ooooo00;o
o`8000Goo`03003ooooo00Woo`03003ooooo04Goo`03003ooooo0;Coo`005Ooo00<00?ooool02Ooo
00<00?ooool0A_oo00<00?ooool0/ooo000Eool00`00ooooo`09ool00`00ooooo`16ool00`00oooo
o`2cool001Goo`03003ooooo00Woo`03003ooooo04Ooo`03003ooooo0;;oo`005Ooo00<00?ooool0
2Ooo00<00?ooool0Aooo00<00?ooool0/_oo000Eool00`00ooooo`09ool00`00ooooo`18ool00`00
ooooo`2aool001Goo`8000Woo`03003ooooo04Woo`03003ooooo0;7oo`005Ooo00<00?ooool02?oo
00<00?ooool0B_oo00<00?ooool0/?oo000Eool00`00ooooo`08ool00`00ooooo`1:ool00`00oooo
o`2`ool001Goo`03003ooooo00Soo`03003ooooo04[oo`03003ooooo0;3oo`005Ooo00<00?ooool0
2?oo00<00?ooool0Booo00<00?ooool0[ooo000Eool00`00ooooo`08ool00`00ooooo`1;ool00`00
ooooo`2_ool001Goo`03003ooooo00Ooo`03003ooooo04goo`03003ooooo0:koo`005Ooo00<00?oo
ool01ooo00<00?ooool0COoo00<00?ooool0[_oo000Eool00`00ooooo`07ool00`00ooooo`1>ool0
0`00ooooo`2]ool001Goo`8000Soo`03003ooooo04koo`03003ooooo0:goo`005Ooo00<00?ooool0
1ooo00<00?ooool0Cooo00<00?ooool0[?oo000Eool00`00ooooo`07ool00`00ooooo`1?ool00`00
ooooo`2/ool001Goo`03003ooooo00Ooo`03003ooooo053oo`03003ooooo0:_oo`005Ooo00<00?oo
ool01_oo00<00?ooool0DOoo00<00?ooool0Zooo000Eool00`00ooooo`06ool00`00ooooo`1Bool0
0`00ooooo`2Zool001Goo`03003ooooo00Koo`03003ooooo05;oo`03003ooooo0:[oo`005Ooo00<0
0?ooool01_oo00<00?ooool0Dooo00<00?ooool0ZOoo000Eool00`00ooooo`1Lool00`00ooooo`2Y
ool001Goo`03003ooooo05goo`03003ooooo0:Soo`005Ooo0P00G_oo00<00?ooool0Z?oo000Eool0
0`00ooooo`1Nool00`00ooooo`2Wool001Goo`03003ooooo05koo`03003ooooo0:Ooo`005Ooo00<0
0?ooool0Gooo00<00?ooool0Y_oo000Eool00`00ooooo`1Oool00`00ooooo`2Vool001Goo`03003o
oooo063oo`03003ooooo0:Goo`005Ooo00<00?ooool0HOoo00<00?ooool0Y?oo0007ool40003ool2
0005ool00`00ooooo`1Qool00`00ooooo`2Tool000Ooo`03003ooooo00?oo`04003ooooo0004ool0
0`00ooooo`1Rool00`00ooooo`2Sool000Soo`03003ooooo00;oo`04003ooooo0004ool3001Rool0
0`00ooooo`2Sool00003ool00000008000Coo`05003oooooool00002ool00`00ooooo`02ool00`00
ooooo`1Sool00`00ooooo`2Rool000[oo`04003ooooo0002ool00`00ooooo`02ool00`00ooooo`1S
ool00`00ooooo`2Rool000Ooo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool0I?oo00<0
0?ooool0XOoo0008ool20004ool20005ool00`00ooooo`1Tool00`00ooooo`2Qool001Goo`03003o
oooo06Goo`03003ooooo0:3oo`005Ooo00<00?ooool0I_oo00<00?ooool0Wooo000Eool00`00oooo
o`1Vool00`00ooooo`2Oool001Goo`03003ooooo06Ooo`03003ooooo09koo`005Ooo00<00?ooool0
Iooo00<00?ooool0W_oo000Eool2001Yool00`00ooooo`2Mool001Goo`03003ooooo06Woo`03003o
oooo09coo`005Ooo00<00?ooool0JOoo00<00?ooool0W?oo000Eool00`00ooooo`1Zool00`00oooo
o`2Kool001Goo`03003ooooo06_oo`03003ooooo09[oo`005Ooo00<00?ooool0Jooo00<00?ooool0
V_oo000Eool00`00ooooo`1/ool00`00ooooo`2Iool001Goo`03003ooooo06coo`03003ooooo09Wo
o`005Ooo00<00?ooool0KOoo00<00?ooool0V?oo000Eool2001_ool00`00ooooo`2Gool001Goo`03
003ooooo06koo`03003ooooo09Ooo`005Ooo00<00?ooool0Kooo00<00?ooool0U_oo000Eool00`00
ooooo`1`ool00`00ooooo`2Eool001Goo`03003ooooo077oo`03003ooooo09Coo`005Ooo00<00?oo
ool0L_oo00<00?ooool0Tooo000Eool00`00ooooo`1cool00`00ooooo`2Bool001Goo`03003ooooo
07Coo`03003ooooo097oo`005Ooo00<00?ooool0MOoo00<00?ooool0T?oo000Eool00`00ooooo`0Q
ool20004ool00`00ooooo`02ool3000Rool4000Qool40003ool00`00ooooo`02ool3000Sool4000Q
ool40003ool00`00ooooo`02ool3000Sool20007ool001Goo`80027oo`04003ooooo000;ool00`00
ooooo`0Qool00`00ooooo`0Qool2000<ool00`00ooooo`0Pool00`00ooooo`0Rool00`00ooooo`0<
ool00`00ooooo`0Oool01000ooooo`001_oo000Eool00`00ooooo`0Pool01000ooooo`002ooo00<0
0?ooool08Ooo00<00?ooool08_oo00<00?ooool02_oo00<00?ooool08Ooo00<00?ooool08_oo00<0
0?ooool02ooo00<00?ooool08_oo00<00?ooool01?oo000Eool00`00ooooo`0Pool01000ooooo`00
2?oo0`009?oo00<00?ooool08_oo0P002?oo0`009Ooo00<00?ooool08_oo00<00?ooool01ooo0`00
8ooo0P001ooo000Eool00`00ooooo`0Pool01000ooooo`002?oo00<00?ooool09?oo00<00?ooool0
8_oo00<00?oo00001ooo00<00?ooool09_oo00<00?ooool08_oo00<00?ooool01_oo00<00?ooool0
9Ooo00<00?ooool01?oo000Eool00`00ooooo`0Pool01000ooooo`002?oo00<00?ooool08_oo0`00
8_oo0`000_oo00<00?ooool01?oo00<00?ooool08ooo00@00?ooool0027oo`04003ooooo0008ool0
0`00ooooo`0Rool01000ooooo`001_oo000Eool00`00ooooo`0Qool20009ool4000Sool00`00oooo
o`0Rool01@00oooooooo00001Ooo10008ooo0P008ooo0P002Ooo10008_oo0P001ooo000Eool00`00
ooooo`1mool00`00ooooo`28ool001Goo`03003ooooo07koo`03003ooooo08Ooo`005Ooo00<00?oo
ool0Oooo0P00N_oo1P001ooo000>oooo000C000001Goo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00;oo`8000;oo`03003ooooo
00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Koo`D000?oo`03003ooooo00Koo`03003ooooo00Goo`005Ooo00<00?ooool0
:?oo00<00?ooool0:?oo00<00?ooool0:?oo00<00?ooool01?oo0P008ooo00<00?ooool0:?oo00<0
0?ooool04_oo1@004Ooo00<00?ooool01Ooo000Eool00`00ooooo`27ool2001Lool5000Nool001Go
o`03003ooooo08Woo`03003ooooo05Coo`D002?oo`005Ooo00<00?ooool0R_oo0P00C_oo1P00:?oo
000Eool00`00ooooo`2<ool20014ool8000^ool001Goo`03003ooooo08koo`@003Woo`L003Koo`00
5Ooo00<00?ooool0T_oo1000<?oo1@00?Ooo000Eool00`00ooooo`2Fool5000Vool50012ool001Go
o`8009coo`L001Coo`/004Ooo`005Ooo00<00?ooool0X_oo5000D_oo000Eool00`00ooooo`3oool9
ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9
ool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -83.7226, \
0.0115777, 0.523757}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["compare to small-xi:", "Subsubsection"],

Cell[BoxData[
    \(thSXitestfunc[om_, k_, th_, 
        a_] := \((om^2 - k^2 - SXiA[om/k, Cos[th], a])\)*\((1 - 
            SXiB[om/k, Cos[th], a]/om^2)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      thSXitestfunc[om, 0.1, 10^\((\(-4\))\), 0.1], {om, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147151 0.0309067 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .09198 -18 -4.5 ]
[.01131 .09198 0 4.5 ]
[.01131 .16925 -6 -4.5 ]
[.01131 .16925 0 4.5 ]
[.01131 .24652 -18 -4.5 ]
[.01131 .24652 0 4.5 ]
[.01131 .32378 -12 -4.5 ]
[.01131 .32378 0 4.5 ]
[.01131 .40105 -24 -4.5 ]
[.01131 .40105 0 4.5 ]
[.01131 .47832 -12 -4.5 ]
[.01131 .47832 0 4.5 ]
[.01131 .55558 -24 -4.5 ]
[.01131 .55558 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .09198 m
.03006 .09198 L
s
[(2.5)] .01131 .09198 1 0 Mshowa
.02381 .16925 m
.03006 .16925 L
s
[(5)] .01131 .16925 1 0 Mshowa
.02381 .24652 m
.03006 .24652 L
s
[(7.5)] .01131 .24652 1 0 Mshowa
.02381 .32378 m
.03006 .32378 L
s
[(10)] .01131 .32378 1 0 Mshowa
.02381 .40105 m
.03006 .40105 L
s
[(12.5)] .01131 .40105 1 0 Mshowa
.02381 .47832 m
.03006 .47832 L
s
[(15)] .01131 .47832 1 0 Mshowa
.02381 .55558 m
.03006 .55558 L
s
[(17.5)] .01131 .55558 1 0 Mshowa
.125 Mabswid
.02381 .03017 m
.02756 .03017 L
s
.02381 .04562 m
.02756 .04562 L
s
.02381 .06108 m
.02756 .06108 L
s
.02381 .07653 m
.02756 .07653 L
s
.02381 .10744 m
.02756 .10744 L
s
.02381 .12289 m
.02756 .12289 L
s
.02381 .13834 m
.02756 .13834 L
s
.02381 .1538 m
.02756 .1538 L
s
.02381 .1847 m
.02756 .1847 L
s
.02381 .20016 m
.02756 .20016 L
s
.02381 .21561 m
.02756 .21561 L
s
.02381 .23106 m
.02756 .23106 L
s
.02381 .26197 m
.02756 .26197 L
s
.02381 .27742 m
.02756 .27742 L
s
.02381 .29288 m
.02756 .29288 L
s
.02381 .30833 m
.02756 .30833 L
s
.02381 .33924 m
.02756 .33924 L
s
.02381 .35469 m
.02756 .35469 L
s
.02381 .37014 m
.02756 .37014 L
s
.02381 .3856 m
.02756 .3856 L
s
.02381 .4165 m
.02756 .4165 L
s
.02381 .43196 m
.02756 .43196 L
s
.02381 .44741 m
.02756 .44741 L
s
.02381 .46286 m
.02756 .46286 L
s
.02381 .49377 m
.02756 .49377 L
s
.02381 .50922 m
.02756 .50922 L
s
.02381 .52468 m
.02756 .52468 L
s
.02381 .54013 m
.02756 .54013 L
s
.02381 .57104 m
.02756 .57104 L
s
.02381 .58649 m
.02756 .58649 L
s
.02381 .60194 m
.02756 .60194 L
s
.02381 .6174 m
.02756 .6174 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.06116 .61803 m
.06205 .55021 L
.06423 .43214 L
.06537 .38726 L
.06643 .35212 L
.06885 .28971 L
.0712 .24497 L
.07374 .20834 L
.07619 .1807 L
.07887 .15671 L
.08375 .12424 L
.08849 .10172 L
.0929 .08602 L
.09823 .07155 L
.10324 .06108 L
.11262 .04693 L
.11788 .04111 L
.12271 .03672 L
.12798 .03276 L
.13363 .02926 L
.14304 .02475 L
.15183 .02165 L
.16228 .01899 L
.1718 .01727 L
.17645 .01664 L
.18139 .01608 L
.18565 .01569 L
.19025 .01535 L
.19517 .01508 L
.19979 .01489 L
.20247 .01482 L
.20393 .01479 L
.20529 .01476 L
.20655 .01474 L
.20769 .01473 L
.20896 .01472 L
.20966 .01472 L
.21032 .01472 L
.21144 .01472 L
.21263 .01472 L
.21376 .01472 L
.2148 .01473 L
.216 .01474 L
.21732 .01476 L
.21972 .0148 L
.22222 .01486 L
.22488 .01493 L
.22967 .0151 L
.23903 .01555 L
.24751 .01609 L
Mistroke
.26615 .01765 L
.28629 .01984 L
.32597 .02539 L
.3642 .03207 L
.40095 .03957 L
.44008 .04863 L
.47774 .05835 L
.51777 .06971 L
.55634 .08164 L
.59344 .09402 L
.63292 .10815 L
.67093 .12268 L
.70747 .1375 L
.74638 .1542 L
.78383 .17116 L
.82365 .19015 L
.862 .20936 L
.89889 .2287 L
.93815 .25021 L
.97595 .27181 L
.97619 .27195 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000nool20004ool00`00ooooo`02ool3000Qool4000Pool40003ool0
0`00ooooo`02ool3000Rool4000Pool40003ool00`00ooooo`02ool3000Rool20007ool003goo`04
003ooooo000;ool00`00ooooo`0Pool00`00ooooo`0Qool00`00ooooo`0:ool00`00ooooo`0Oool0
0`00ooooo`0Qool00`00ooooo`0<ool00`00ooooo`0Nool01000ooooo`001_oo000mool01000oooo
o`002ooo00<00?ooool08?oo00<00?ooool08Ooo00<00?ooool02_oo00<00?ooool08?oo00<00?oo
ool08Ooo00<00?ooool02ooo00<00?ooool08Ooo00<00?ooool01?oo000mool01000ooooo`002?oo
0`008ooo00<00?ooool08Ooo00<00?ooool01ooo0`009?oo00<00?ooool08Ooo00<00?ooool01ooo
0`008_oo0P001ooo000mool01000ooooo`002?oo00<00?ooool08ooo00<00?ooool08Ooo00<00?oo
ool01ooo00<00?ooool09Ooo00<00?ooool08Ooo00<00?ooool01_oo00<00?ooool09?oo00<00?oo
ool01?oo000mool01000ooooo`002?oo00<00?ooool08Ooo0`008Ooo0`002Ooo00<00?ooool08_oo
00@00?ooool0023oo`04003ooooo0008ool00`00ooooo`0Qool01000ooooo`001_oo000Kool00`00
ooooo`0Pool20009ool4000Rool00`00ooooo`0Qool00`00ooooo`07ool4000Rool2000Rool20009
ool4000Qool20007ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001Coool000d000006ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?oo
ool01Ooo00<00?ooool01_oo00<00?oo00000`000_oo00<00?ooool01Ooo00<00?ooool01_oo00<0
0?ooool02`000ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo
00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo000Kool00`00ooooo`0Nool30006ool0
0`00ooooo`0Lool50006ool00`00ooooo`0Wool00`00ooooo`0Xool00`00ooooo`0Wool00`00oooo
o`0Wool00`00ooooo`05ool001_oo`03003ooooo01coo`8002goo`D00;;oo`006ooo0P006ooo0P00
=?oo1@00[Ooo000Kool00`00ooooo`0Hool2000kool5002Xool001_oo`03003ooooo01Ooo`03003o
oooo043oo`D00:?oo`006ooo00<00?ooool05_oo00<00?ooool0A_oo1@00W_oo000Kool2000Fool0
0`00ooooo`1<ool6002Hool001_oo`03003ooooo01Coo`03003ooooo05?oo`@009Coo`006ooo00<0
0?ooool04ooo00<00?ooool0F?oo0`00TOoo000Kool00`00ooooo`0Cool00`00ooooo`1Kool4002=
ool001_oo`8001?oo`03003ooooo063oo`<008[oo`006ooo00<00?ooool04_oo00<00?ooool0Hooo
0`00Qooo000Kool00`00ooooo`0Aool00`00ooooo`1Wool40023ool001_oo`03003ooooo017oo`03
003ooooo06_oo`<0083oo`006ooo0P004_oo00<00?ooool0K_oo0`00OOoo000Kool00`00ooooo`0@
ool00`00ooooo`1bool4001iool001_oo`03003ooooo013oo`03003ooooo07Koo`<007Koo`001ooo
10000ooo00<00?ooool00_oo0`001Ooo00<00?ooool03ooo00<00?ooool0N_oo0`00Looo0007ool0
0`00ooooo`0<ool00`00ooooo`02ool00`00ooooo`0?ool00`00ooooo`1mool4001_ool000Soo`03
003ooooo00_oo`03003ooooo00;oo`80013oo`03003ooooo087oo`<006coo`002Ooo00<00?ooool0
1ooo0`001Ooo00<00?ooool03_oo00<00?ooool0QOoo0`00JOoo000:ool00`00ooooo`06ool00`00
ooooo`05ool00`00ooooo`0>ool00`00ooooo`28ool2001Wool000Ooo`04003ooooo0008ool00`00
ooooo`05ool00`00ooooo`0>ool00`00ooooo`2:ool3001Tool000Soo`8000Woo`@000Coo`8000oo
o`03003ooooo08goo`<0067oo`006ooo00<00?ooool03_oo00<00?ooool0T?oo0P00Gooo000Kool0
0`00ooooo`0=ool00`00ooooo`2Cool3001Lool001_oo`03003ooooo00goo`03003ooooo09Koo`80
05[oo`006ooo0P003_oo00<00?ooool0V?oo0`00Eooo000Kool00`00ooooo`0=ool00`00ooooo`2K
ool2001Eool001_oo`03003ooooo00goo`03003ooooo09goo`<005;oo`006ooo00<00?ooool03?oo
00<00?ooool0XOoo0P00D?oo000Kool2000=ool00`00ooooo`2Sool3001=ool001_oo`03003ooooo
00coo`03003ooooo0:Koo`8004_oo`006ooo00<00?ooool03?oo00<00?ooool0Z?oo0`00B?oo000K
ool00`00ooooo`0;ool00`00ooooo`2/ool20016ool001_oo`8000coo`03003ooooo0:koo`8004Co
o`006ooo00<00?ooool02ooo00<00?ooool0/?oo0P00@_oo000Cool30005ool00`00ooooo`0;ool0
0`00ooooo`2bool20010ool001Koo`03003ooooo00;oo`03003ooooo00_oo`03003ooooo0;Coo`80
03koo`005_oo00<00?ooool00_oo0P003?oo00<00?ooool0]_oo0P00??oo000Cool30005ool00`00
ooooo`0;ool00`00ooooo`2hool3000iool001?oo`03003ooooo00Goo`03003ooooo00_oo`03003o
oooo0;_oo`8003Ooo`004ooo00<00?ooool01Ooo00<00?ooool02ooo00<00?ooool0_Ooo0P00=Ooo
000Cool40004ool2000<ool00`00ooooo`2oool2000cool001_oo`03003ooooo00_oo`03003ooooo
0<7oo`80037oo`006ooo00<00?ooool02ooo00<00?ooool0`ooo0P00;ooo000Kool00`00ooooo`0:
ool00`00ooooo`36ool2000]ool001_oo`8000_oo`03003ooooo0<Soo`8002_oo`006ooo00<00?oo
ool02_oo00<00?ooool0b_oo0P00:Ooo000Kool00`00ooooo`0:ool00`00ooooo`3<ool2000Wool0
01_oo`03003ooooo00[oo`03003ooooo0<koo`8002Goo`006ooo0P002ooo00<00?ooool0d?oo0P00
8ooo000Kool00`00ooooo`0:ool00`00ooooo`3Bool2000Qool001_oo`03003ooooo00[oo`03003o
oooo0=Coo`8001ooo`006ooo00<00?ooool02_oo00<00?ooool0e_oo0P007Ooo000Kool2000:ool0
0`00ooooo`3Iool2000Kool001_oo`03003ooooo00Woo`03003ooooo0=_oo`8001Woo`006ooo00<0
0?ooool02Ooo00<00?ooool0gOoo0P005ooo0008ool00`00ooooo`03ool00`00ooooo`02ool30005
ool00`00ooooo`09ool00`00ooooo`3Oool00`00ooooo`0Dool000Soo`03003ooooo00_oo`03003o
oooo00;oo`03003ooooo00Woo`03003ooooo0>3oo`8001Coo`002Ooo00<00?ooool02_oo00<00?oo
ool00_oo0P002_oo00<00?ooool0h_oo0P004_oo0009ool00`00ooooo`07ool30005ool00`00oooo
o`09ool00`00ooooo`3Tool00`00ooooo`0?ool000[oo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Woo`03003ooooo0>Goo`8000ooo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03
003ooooo00Woo`03003ooooo0>Ooo`8000goo`001ooo10002?oo10001?oo0P002_oo00<00?ooool0
jOoo0P002ooo000Kool00`00ooooo`09ool00`00ooooo`3[ool20009ool001_oo`03003ooooo00Wo
o`03003ooooo0>goo`8000Ooo`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000Kool2000:ool0
0`00ooooo`3fool001_oo`03003ooooo00Woo`03003ooooo0?Koo`006ooo00<00?ooool02Ooo00<0
0?ooool0m_oo000Kool00`00ooooo`09ool00`00ooooo`3fool001_oo`8000[oo`03003ooooo0?Ko
o`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000Kool00`00ooooo`09ool00`00ooooo`3fool0
01_oo`03003ooooo00Woo`03003ooooo0?Koo`006ooo0P002_oo00<00?ooool0m_oo000Kool00`00
ooooo`09ool00`00ooooo`3fool000goo`@000?oo`8000Goo`03003ooooo00Woo`03003ooooo0?Ko
o`003ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`03003ooooo00Woo`03003ooooo0?Koo`00
3ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`8000Woo`03003ooooo0?Ooo`003ooo00D00?oo
ooooo`0000;oo`03003ooooo00;oo`03003ooooo00Soo`03003ooooo0?Ooo`003ooo00D00?oooooo
o`0000;oo`03003ooooo00;oo`03003ooooo00Soo`03003ooooo0?Ooo`003Ooo0`000ooo00@00?oo
ool000Coo`03003ooooo00Soo`03003ooooo0?Ooo`003ooo00<00?ooool00_oo0P001Ooo0P002Ooo
00<00?ooool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03
003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo000Kool20009ool00`00ooooo`3g
ool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo
000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`8000Woo`03003ooooo0?Ooo`006ooo00<0
0?ooool02?oo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003o
oooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo000Kool20009ool0
0`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`0000?oo`0000000P000_oo1000
0ooo00<00?ooool00_oo0`001Ooo00<00?ooool02?oo00<00?ooool0mooo0003ool01@00oooooooo
00003_oo00<00?ooool00_oo00<00?ooool02?oo00<00?ooool0mooo0003ool00`00ooooo`02ool0
0`00ooooo`0;ool00`00ooooo`02ool20009ool00`00ooooo`3gool000?oo`03003ooooo00?oo`03
003ooooo00Ooo`<000Goo`03003ooooo00Soo`03003ooooo0?Ooo`000ooo00<00?ooool01?oo00<0
0?ooool01_oo00<00?ooool01Ooo00<00?ooool02?oo00<00?ooool0mooo00001?oo000000000ooo
00@00?ooool000Soo`03003ooooo00Goo`03003ooooo00Soo`03003ooooo0?Ooo`000ooo00<00?oo
ool00_oo0P002Ooo10001?oo0P002Ooo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00oooo
o`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0
mooo000Kool20009ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo
00<00?ooool02?oo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`80
00Woo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo000Kool00`00ooooo`08
ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo0P002Ooo00<00?oo
ool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool000goo`@000;oo`<000Goo`03003ooooo
00Soo`03003ooooo0?Ooo`003ooo00<00?ooool01?oo00<00?ooool00_oo00<00?ooool02?oo00<0
0?ooool0mooo000?ool00`00ooooo`04ool00`00ooooo`02ool20009ool00`00ooooo`3gool000oo
o`04003oooooool30005ool00`00ooooo`08ool00`00ooooo`3gool000ooo`05003oooooool00007
ool00`00ooooo`08ool00`00ooooo`3gool000goo`<000?oo`03003ooooo00Goo`03003ooooo00So
o`03003ooooo0?Ooo`003ooo00@00?ooooooo`@000Coo`8000Soo`03003ooooo0?Soo`006ooo00<0
0?ooool01ooo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003o
oooo00Ooo`03003ooooo0?Soo`006ooo0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool0
0`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<0
0?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`8000Soo`03003ooooo0?So
o`006ooo00<00?ooool01ooo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool0
01_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo0P002?oo00<00?ooool0n?oo000Kool00`00
ooooo`07ool00`00ooooo`3hool00003ool00000008000?oo`03003ooooo00?oo`03003ooooo00;o
o`<000Goo`03003ooooo00Ooo`03003ooooo0?Soo`000ooo00<00?ooool00_oo00<00?ooool02ooo
00<00?ooool00_oo00<00?ooool01ooo00<00?ooool0n?oo0003ool00`00ooooo`03ool00`00oooo
o`0:ool00`00ooooo`02ool20008ool00`00ooooo`3hool000?oo`03003ooooo00?oo`03003ooooo
00Ooo`<000Goo`03003ooooo00Ooo`03003ooooo0?Soo`000ooo00<00?ooool01?oo00<00?ooool0
1_oo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool0n?oo00001?oo000000000ooo00@00?oo
ool000Soo`03003ooooo00Goo`03003ooooo00Ooo`03003ooooo0?Soo`000ooo00@00?ooooooo`@0
00Soo`@000Coo`8000Soo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?ooool0n?oo000K
ool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo
0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo
00Ooo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?ooool0n?oo000Kool20008ool00`00
ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?oo
ool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`8000Soo`03003ooooo0?Soo`00
oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.323594, -1.75007, \
0.0118419, 0.121634}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %732]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147151 0.0309067 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .09198 -18 -4.5 ]
[.01131 .09198 0 4.5 ]
[.01131 .16925 -6 -4.5 ]
[.01131 .16925 0 4.5 ]
[.01131 .24652 -18 -4.5 ]
[.01131 .24652 0 4.5 ]
[.01131 .32378 -12 -4.5 ]
[.01131 .32378 0 4.5 ]
[.01131 .40105 -24 -4.5 ]
[.01131 .40105 0 4.5 ]
[.01131 .47832 -12 -4.5 ]
[.01131 .47832 0 4.5 ]
[.01131 .55558 -24 -4.5 ]
[.01131 .55558 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .09198 m
.03006 .09198 L
s
[(2.5)] .01131 .09198 1 0 Mshowa
.02381 .16925 m
.03006 .16925 L
s
[(5)] .01131 .16925 1 0 Mshowa
.02381 .24652 m
.03006 .24652 L
s
[(7.5)] .01131 .24652 1 0 Mshowa
.02381 .32378 m
.03006 .32378 L
s
[(10)] .01131 .32378 1 0 Mshowa
.02381 .40105 m
.03006 .40105 L
s
[(12.5)] .01131 .40105 1 0 Mshowa
.02381 .47832 m
.03006 .47832 L
s
[(15)] .01131 .47832 1 0 Mshowa
.02381 .55558 m
.03006 .55558 L
s
[(17.5)] .01131 .55558 1 0 Mshowa
.125 Mabswid
.02381 .03017 m
.02756 .03017 L
s
.02381 .04562 m
.02756 .04562 L
s
.02381 .06108 m
.02756 .06108 L
s
.02381 .07653 m
.02756 .07653 L
s
.02381 .10744 m
.02756 .10744 L
s
.02381 .12289 m
.02756 .12289 L
s
.02381 .13834 m
.02756 .13834 L
s
.02381 .1538 m
.02756 .1538 L
s
.02381 .1847 m
.02756 .1847 L
s
.02381 .20016 m
.02756 .20016 L
s
.02381 .21561 m
.02756 .21561 L
s
.02381 .23106 m
.02756 .23106 L
s
.02381 .26197 m
.02756 .26197 L
s
.02381 .27742 m
.02756 .27742 L
s
.02381 .29288 m
.02756 .29288 L
s
.02381 .30833 m
.02756 .30833 L
s
.02381 .33924 m
.02756 .33924 L
s
.02381 .35469 m
.02756 .35469 L
s
.02381 .37014 m
.02756 .37014 L
s
.02381 .3856 m
.02756 .3856 L
s
.02381 .4165 m
.02756 .4165 L
s
.02381 .43196 m
.02756 .43196 L
s
.02381 .44741 m
.02756 .44741 L
s
.02381 .46286 m
.02756 .46286 L
s
.02381 .49377 m
.02756 .49377 L
s
.02381 .50922 m
.02756 .50922 L
s
.02381 .52468 m
.02756 .52468 L
s
.02381 .54013 m
.02756 .54013 L
s
.02381 .57104 m
.02756 .57104 L
s
.02381 .58649 m
.02756 .58649 L
s
.02381 .60194 m
.02756 .60194 L
s
.02381 .6174 m
.02756 .6174 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.06116 .61803 m
.06205 .55021 L
.06423 .43214 L
.06537 .38726 L
.06643 .35212 L
.06885 .28971 L
.0712 .24497 L
.07374 .20834 L
.07619 .1807 L
.07887 .15671 L
.08375 .12424 L
.08849 .10172 L
.0929 .08602 L
.09823 .07155 L
.10324 .06108 L
.11262 .04693 L
.11788 .04111 L
.12271 .03672 L
.12798 .03276 L
.13363 .02926 L
.14304 .02475 L
.15183 .02165 L
.16228 .01899 L
.1718 .01727 L
.17645 .01664 L
.18139 .01608 L
.18565 .01569 L
.19025 .01535 L
.19517 .01508 L
.19979 .01489 L
.20247 .01482 L
.20393 .01479 L
.20529 .01476 L
.20655 .01474 L
.20769 .01473 L
.20896 .01472 L
.20966 .01472 L
.21032 .01472 L
.21144 .01472 L
.21263 .01472 L
.21376 .01472 L
.2148 .01473 L
.216 .01474 L
.21732 .01476 L
.21972 .0148 L
.22222 .01486 L
.22488 .01493 L
.22967 .0151 L
.23903 .01555 L
.24751 .01609 L
Mistroke
.26615 .01765 L
.28629 .01984 L
.32597 .02539 L
.3642 .03207 L
.40095 .03957 L
.44008 .04863 L
.47774 .05835 L
.51777 .06971 L
.55634 .08164 L
.59344 .09402 L
.63292 .10815 L
.67093 .12268 L
.70747 .1375 L
.74638 .1542 L
.78383 .17116 L
.82365 .19015 L
.862 .20936 L
.89889 .2287 L
.93815 .25021 L
.97595 .27181 L
.97619 .27195 L
Mfstroke
.06115 .61803 m
.06205 .54927 L
.06423 .43138 L
.06537 .38658 L
.06643 .35149 L
.06885 .28919 L
.0712 .24452 L
.07374 .20796 L
.07619 .18037 L
.07887 .15642 L
.08375 .12401 L
.08849 .10154 L
.0929 .08586 L
.09823 .07142 L
.10324 .06097 L
.11262 .04685 L
.11788 .04105 L
.12271 .03667 L
.12798 .03271 L
.13363 .02922 L
.14304 .02472 L
.15183 .02163 L
.1572 .02015 L
.16228 .01897 L
.16681 .01808 L
.1718 .01726 L
.18114 .0161 L
.18625 .01563 L
.19105 .01529 L
.19552 .01506 L
.19962 .0149 L
.20177 .01483 L
.20412 .01478 L
.20531 .01476 L
.20657 .01474 L
.20777 .01473 L
.20886 .01472 L
.21001 .01472 L
.21109 .01472 L
.21227 .01472 L
.21354 .01472 L
.21418 .01473 L
.21487 .01473 L
.21614 .01474 L
.21851 .01478 L
.22074 .01482 L
.22312 .01488 L
.2274 .01502 L
.23284 .01524 L
.23795 .0155 L
Mistroke
.24751 .0161 L
.26695 .01774 L
.28482 .01967 L
.32304 .02494 L
.3598 .03126 L
.39893 .03915 L
.43659 .04779 L
.47662 .05806 L
.51519 .06896 L
.55229 .08036 L
.59177 .09346 L
.62977 .107 L
.66632 .12089 L
.70523 .13659 L
.74268 .15259 L
.7825 .17056 L
.82085 .1888 L
.85774 .2072 L
.897 .22771 L
.93479 .24835 L
.97496 .27125 L
.97619 .27197 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000nool20004ool00`00ooooo`02ool3000Qool4000Pool40003ool0
0`00ooooo`02ool3000Rool4000Pool40003ool00`00ooooo`02ool3000Rool20007ool003goo`04
003ooooo000;ool00`00ooooo`0Pool00`00ooooo`0Qool00`00ooooo`0:ool00`00ooooo`0Oool0
0`00ooooo`0Qool00`00ooooo`0<ool00`00ooooo`0Nool01000ooooo`001_oo000mool01000oooo
o`002ooo00<00?ooool08?oo00<00?ooool08Ooo00<00?ooool02_oo00<00?ooool08?oo00<00?oo
ool08Ooo00<00?ooool02ooo00<00?ooool08Ooo00<00?ooool01?oo000mool01000ooooo`002?oo
0`008ooo00<00?ooool08Ooo00<00?ooool01ooo0`009?oo00<00?ooool08Ooo00<00?ooool01ooo
0`008_oo0P001ooo000mool01000ooooo`002?oo00<00?ooool08ooo00<00?ooool08Ooo00<00?oo
ool01ooo00<00?ooool09Ooo00<00?ooool08Ooo00<00?ooool01_oo00<00?ooool09?oo00<00?oo
ool01?oo000mool01000ooooo`002?oo00<00?ooool08Ooo0`008Ooo0`002Ooo00<00?ooool08_oo
00@00?ooool0023oo`04003ooooo0008ool00`00ooooo`0Qool01000ooooo`001_oo000Kool00`00
ooooo`0Pool20009ool4000Rool00`00ooooo`0Qool00`00ooooo`07ool4000Rool2000Rool20009
ool4000Qool20007ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001Coool000d000006ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?oo
ool01Ooo00<00?ooool01_oo1`0000?oo`00ool01_oo00<00?ooool01_oo00<00?ooool02`000ooo
00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo
00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01Ooo00<00?ooool01Ooo000Kool00`00ooooo`0Nool30006ool00`00ooooo`0Kool6
0006ool00`00ooooo`0Wool00`00ooooo`0Xool00`00ooooo`0Wool00`00ooooo`0Wool00`00oooo
o`05ool001_oo`03003ooooo01coo`8002coo`L00;7oo`006ooo0P006ooo0P00=?oo2@00ZOoo000K
ool00`00ooooo`0Hool2000kool:002Sool001_oo`03003ooooo01Ooo`03003ooooo043oo`T009oo
o`006ooo00<00?ooool05Ooo0P00B?oo1`00W?oo000Kool2000Eool2001>ool6002Hool001_oo`03
003ooooo01Coo`03003ooooo05?oo`@009Coo`006ooo00<00?ooool04ooo00<00?ooool0F?oo1@00
Sooo000Kool00`00ooooo`0Cool00`00ooooo`1Kool6002;ool001_oo`8001?oo`03003ooooo063o
o`H008Ooo`006ooo00<00?ooool04_oo00<00?ooool0Hooo1`00Pooo000Kool00`00ooooo`0Aool0
0`00ooooo`1Wool70020ool001_oo`03003ooooo017oo`03003ooooo06_oo`H007goo`006ooo0P00
4_oo00<00?ooool0K_oo1P00N_oo000Kool00`00ooooo`0@ool00`00ooooo`1bool6001gool001_o
o`03003ooooo013oo`03003ooooo07Koo`D007Coo`001ooo10000ooo00<00?ooool00_oo0`001Ooo
00<00?ooool03ooo00<00?ooool0N_oo1000L_oo0007ool00`00ooooo`0<ool00`00ooooo`02ool0
0`00ooooo`0?ool00`00ooooo`1mool4001_ool000Soo`03003ooooo00_oo`03003ooooo00;oo`80
013oo`03003ooooo087oo`<006coo`002Ooo00<00?ooool01ooo0`001Ooo00<00?ooool03_oo00<0
0?ooool0QOoo0`00JOoo000:ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`0>ool00`00
ooooo`28ool4001Uool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00ooooo`0>ool00`00
ooooo`2:ool5001Rool000Soo`8000Woo`@000Coo`8000ooo`03003ooooo08goo`@0063oo`006ooo
00<00?ooool03_oo00<00?ooool0T?oo1000GOoo000Kool00`00ooooo`0=ool00`00ooooo`2Cool4
001Kool001_oo`03003ooooo00goo`03003ooooo09Koo`@005Soo`006ooo0P003_oo00<00?ooool0
V?oo1000E_oo000Kool00`00ooooo`0=ool00`00ooooo`2Kool4001Cool001_oo`03003ooooo00go
o`03003ooooo09goo`@0057oo`006ooo00<00?ooool03?oo00<00?ooool0XOoo0`00Cooo000Kool2
000=ool00`00ooooo`2Sool3001=ool001_oo`03003ooooo00coo`03003ooooo0:Koo`8004_oo`00
6ooo00<00?ooool03?oo00<00?ooool0Z?oo0`00B?oo000Kool00`00ooooo`0;ool00`00ooooo`2[
ool30016ool001_oo`8000coo`03003ooooo0:goo`<004Coo`006ooo00<00?ooool02ooo00<00?oo
ool0/?oo0`00@Ooo000Cool30005ool00`00ooooo`0;ool00`00ooooo`2bool3000oool001Koo`03
003ooooo00;oo`03003ooooo00_oo`03003ooooo0;Coo`@003coo`005_oo00<00?ooool00_oo0P00
3?oo00<00?ooool0]_oo1000>_oo000Cool30005ool00`00ooooo`0;ool00`00ooooo`2hool4000h
ool001?oo`03003ooooo00Goo`03003ooooo00_oo`03003ooooo0;_oo`<003Koo`004ooo00<00?oo
ool01Ooo00<00?ooool02ooo00<00?ooool0_Ooo0`00=?oo000Cool40004ool2000<ool00`00oooo
o`2oool3000bool001_oo`03003ooooo00_oo`03003ooooo0<7oo`<0033oo`006ooo00<00?ooool0
2ooo00<00?ooool0`ooo0`00;_oo000Kool00`00ooooo`0:ool00`00ooooo`36ool3000/ool001_o
o`8000_oo`03003ooooo0<Soo`<002[oo`006ooo00<00?ooool02_oo00<00?ooool0b_oo0`00:?oo
000Kool00`00ooooo`0:ool00`00ooooo`3<ool3000Vool001_oo`03003ooooo00[oo`03003ooooo
0<koo`<002Coo`006ooo0P002ooo00<00?ooool0d?oo0`008_oo000Kool00`00ooooo`0:ool00`00
ooooo`3Bool2000Qool001_oo`03003ooooo00[oo`03003ooooo0=Coo`8001ooo`006ooo00<00?oo
ool02_oo00<00?ooool0e_oo0P007Ooo000Kool2000:ool00`00ooooo`3Iool2000Kool001_oo`03
003ooooo00Woo`03003ooooo0=[oo`<001Woo`006ooo00<00?ooool02Ooo00<00?ooool0g?oo0`00
5ooo0008ool00`00ooooo`03ool00`00ooooo`02ool30005ool00`00ooooo`09ool00`00ooooo`3N
ool2000Fool000Soo`03003ooooo00_oo`03003ooooo00;oo`03003ooooo00Woo`03003ooooo0>3o
o`8001Coo`002Ooo00<00?ooool02_oo00<00?ooool00_oo0P002_oo00<00?ooool0h_oo0P004_oo
0009ool00`00ooooo`07ool30005ool00`00ooooo`09ool00`00ooooo`3Tool2000@ool000[oo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo00Woo`03003ooooo0>Goo`<000koo`001ooo00@0
0?ooool000Soo`03003ooooo00Goo`03003ooooo00Woo`03003ooooo0>Ooo`<000coo`001ooo1000
2?oo10001?oo0P002_oo00<00?ooool0jOoo0`002_oo000Kool00`00ooooo`09ool00`00ooooo`3[
ool30008ool001_oo`03003ooooo00Woo`03003ooooo0>goo`8000Ooo`006ooo00<00?ooool02Ooo
00<00?ooool0m_oo000Kool2000:ool00`00ooooo`3fool001_oo`03003ooooo00Woo`03003ooooo
0?Koo`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000Kool00`00ooooo`09ool00`00ooooo`3f
ool001_oo`8000[oo`03003ooooo0?Koo`006ooo00<00?ooool02Ooo00<00?ooool0m_oo000Kool0
0`00ooooo`09ool00`00ooooo`3fool001_oo`03003ooooo00Woo`03003ooooo0?Koo`006ooo0P00
2_oo00<00?ooool0m_oo000Kool00`00ooooo`09ool00`00ooooo`3fool000goo`@000?oo`8000Go
o`03003ooooo00Woo`03003ooooo0?Koo`003ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`03
003ooooo00Woo`03003ooooo0?Koo`003ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`8000Wo
o`03003ooooo0?Ooo`003ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`03003ooooo00Soo`03
003ooooo0?Ooo`003ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`03003ooooo00Soo`03003o
oooo0?Ooo`003Ooo0`000ooo00@00?ooool000Coo`03003ooooo00Soo`03003ooooo0?Ooo`003ooo
00<00?ooool00_oo0P001Ooo0P002Ooo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00oooo
o`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0
mooo000Kool20009ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo
00<00?ooool02?oo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`80
00Woo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo000Kool00`00ooooo`08
ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo
00<00?ooool0mooo000Kool20009ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo
0?Ooo`0000?oo`0000000P000_oo10000ooo00<00?ooool00_oo0`001Ooo00<00?ooool02?oo00<0
0?ooool0mooo0003ool01@00oooooooo00003_oo00<00?ooool00_oo00<00?ooool02?oo00<00?oo
ool0mooo0003ool00`00ooooo`02ool00`00ooooo`0;ool00`00ooooo`02ool20009ool00`00oooo
o`3gool000?oo`03003ooooo00?oo`03003ooooo00Ooo`<000Goo`03003ooooo00Soo`03003ooooo
0?Ooo`000ooo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool02?oo00<0
0?ooool0mooo00001?oo000000000ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo00So
o`03003ooooo0?Ooo`000ooo00<00?ooool00_oo0P002Ooo10001?oo0P002Ooo00<00?ooool0mooo
000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003ooooo0?Ooo`00
6ooo00<00?ooool02?oo00<00?ooool0mooo000Kool20009ool00`00ooooo`3gool001_oo`03003o
oooo00Soo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<00?ooool0mooo000Kool00`00oooo
o`08ool00`00ooooo`3gool001_oo`8000Woo`03003ooooo0?Ooo`006ooo00<00?ooool02?oo00<0
0?ooool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool001_oo`03003ooooo00Soo`03003o
oooo0?Ooo`006ooo0P002Ooo00<00?ooool0mooo000Kool00`00ooooo`08ool00`00ooooo`3gool0
00goo`@000;oo`<000Goo`03003ooooo00Soo`03003ooooo0?Ooo`003ooo00<00?ooool01?oo00<0
0?ooool00_oo00<00?ooool02?oo00<00?ooool0mooo000?ool00`00ooooo`04ool00`00ooooo`02
ool20009ool00`00ooooo`3gool000ooo`04003oooooool30005ool00`00ooooo`08ool00`00oooo
o`3gool000ooo`05003oooooool00007ool00`00ooooo`08ool00`00ooooo`3gool000goo`<000?o
o`03003ooooo00Goo`03003ooooo00Soo`03003ooooo0?Ooo`003ooo00@00?ooooooo`@000Coo`80
00Soo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?ooool0n?oo000Kool00`00ooooo`07
ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo0P002?oo00<00?oo
ool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo
0?Soo`006ooo00<00?ooool01ooo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3h
ool001_oo`8000Soo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<00?ooool0n?oo000Kool0
0`00ooooo`07ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo0P00
2?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool00003ool00000008000?o
o`03003ooooo00?oo`03003ooooo00;oo`<000Goo`03003ooooo00Ooo`03003ooooo0?Soo`000ooo
00<00?ooool00_oo00<00?ooool02ooo00<00?ooool00_oo00<00?ooool01ooo00<00?ooool0n?oo
0003ool00`00ooooo`03ool00`00ooooo`0:ool00`00ooooo`02ool20008ool00`00ooooo`3hool0
00?oo`03003ooooo00?oo`03003ooooo00Ooo`<000Goo`03003ooooo00Ooo`03003ooooo0?Soo`00
0ooo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool0
n?oo00001?oo000000000ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo00Ooo`03003o
oooo0?Soo`000ooo00@00?ooooooo`@000Soo`@000Coo`8000Soo`03003ooooo0?Soo`006ooo00<0
0?ooool01ooo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool001_oo`03003o
oooo00Ooo`03003ooooo0?Soo`006ooo0P002?oo00<00?ooool0n?oo000Kool00`00ooooo`07ool0
0`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?Soo`006ooo00<00?ooool01ooo00<0
0?ooool0n?oo000Kool20008ool00`00ooooo`3hool001_oo`03003ooooo00Ooo`03003ooooo0?So
o`006ooo00<00?ooool01ooo00<00?ooool0n?oo000Kool00`00ooooo`07ool00`00ooooo`3hool0
01_oo`8000Soo`03003ooooo0?Soo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.323594, -1.75007, \
0.0118419, 0.121634}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[thSXitestfunc[om - I\ 1.64, 0.1, 10^\((\(-4\))\), 0.1]], {om, 0.1, 
        3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.236938 0.0651254 [
[.18254 .22444 -9 -9 ]
[.18254 .22444 9 0 ]
[.34127 .22444 -3 -9 ]
[.34127 .22444 3 0 ]
[.5 .22444 -9 -9 ]
[.5 .22444 9 0 ]
[.65873 .22444 -3 -9 ]
[.65873 .22444 3 0 ]
[.81746 .22444 -9 -9 ]
[.81746 .22444 9 0 ]
[.97619 .22444 -3 -9 ]
[.97619 .22444 3 0 ]
[.01131 .10669 -12 -4.5 ]
[.01131 .10669 0 4.5 ]
[.01131 .36719 -6 -4.5 ]
[.01131 .36719 0 4.5 ]
[.01131 .49744 -6 -4.5 ]
[.01131 .49744 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .23694 m
.18254 .24319 L
s
[(0.5)] .18254 .22444 0 1 Mshowa
.34127 .23694 m
.34127 .24319 L
s
[(1)] .34127 .22444 0 1 Mshowa
.5 .23694 m
.5 .24319 L
s
[(1.5)] .5 .22444 0 1 Mshowa
.65873 .23694 m
.65873 .24319 L
s
[(2)] .65873 .22444 0 1 Mshowa
.81746 .23694 m
.81746 .24319 L
s
[(2.5)] .81746 .22444 0 1 Mshowa
.97619 .23694 m
.97619 .24319 L
s
[(3)] .97619 .22444 0 1 Mshowa
.125 Mabswid
.05556 .23694 m
.05556 .24069 L
s
.0873 .23694 m
.0873 .24069 L
s
.11905 .23694 m
.11905 .24069 L
s
.15079 .23694 m
.15079 .24069 L
s
.21429 .23694 m
.21429 .24069 L
s
.24603 .23694 m
.24603 .24069 L
s
.27778 .23694 m
.27778 .24069 L
s
.30952 .23694 m
.30952 .24069 L
s
.37302 .23694 m
.37302 .24069 L
s
.40476 .23694 m
.40476 .24069 L
s
.43651 .23694 m
.43651 .24069 L
s
.46825 .23694 m
.46825 .24069 L
s
.53175 .23694 m
.53175 .24069 L
s
.56349 .23694 m
.56349 .24069 L
s
.59524 .23694 m
.59524 .24069 L
s
.62698 .23694 m
.62698 .24069 L
s
.69048 .23694 m
.69048 .24069 L
s
.72222 .23694 m
.72222 .24069 L
s
.75397 .23694 m
.75397 .24069 L
s
.78571 .23694 m
.78571 .24069 L
s
.84921 .23694 m
.84921 .24069 L
s
.88095 .23694 m
.88095 .24069 L
s
.9127 .23694 m
.9127 .24069 L
s
.94444 .23694 m
.94444 .24069 L
s
.25 Mabswid
0 .23694 m
1 .23694 L
s
.02381 .10669 m
.03006 .10669 L
s
[(-2)] .01131 .10669 1 0 Mshowa
.02381 .36719 m
.03006 .36719 L
s
[(2)] .01131 .36719 1 0 Mshowa
.02381 .49744 m
.03006 .49744 L
s
[(4)] .01131 .49744 1 0 Mshowa
.125 Mabswid
.02381 .13925 m
.02756 .13925 L
s
.02381 .17181 m
.02756 .17181 L
s
.02381 .20438 m
.02756 .20438 L
s
.02381 .2695 m
.02756 .2695 L
s
.02381 .30206 m
.02756 .30206 L
s
.02381 .33463 m
.02756 .33463 L
s
.02381 .39975 m
.02756 .39975 L
s
.02381 .43231 m
.02756 .43231 L
s
.02381 .46488 m
.02756 .46488 L
s
.02381 .07412 m
.02756 .07412 L
s
.02381 .04156 m
.02756 .04156 L
s
.02381 .009 m
.02756 .009 L
s
.02381 .53 m
.02756 .53 L
s
.02381 .56257 m
.02756 .56257 L
s
.02381 .59513 m
.02756 .59513 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.06423 .01514 L
.07374 .01572 L
.0929 .01726 L
.11399 .01953 L
.13363 .02217 L
.17133 .02869 L
.21139 .03766 L
.24999 .04828 L
.28712 .06031 L
.32663 .07505 L
.36467 .09113 L
.40124 .10832 L
.44019 .12849 L
.47766 .14973 L
.51751 .17427 L
.5559 .19982 L
.59282 .22618 L
.63211 .25614 L
.66994 .28685 L
.71013 .32151 L
.74886 .35686 L
.78613 .3927 L
.82576 .43278 L
.86394 .47329 L
.90064 .51402 L
.93972 .55929 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo3ooo000?ool00`00ooooo`3oool?
ool000ooo`800?oooa3oo`003ooo00<00?ooool01_oo2000oooo0Ooo000?ool00`00ooooo`0>ool5
003kool000ooo`03003ooooo01?oo``00>ooo`003ooo00<00?ooool07ooo1@00j_oo000?ool00`00
ooooo`0Tool4003Vool000ooo`03003ooooo02Soo`@00>;oo`003ooo00<00?ooool0;?oo1000g_oo
000?ool00`00ooooo`0`ool3003Kool000ooo`8003Coo`@00=Ooo`003ooo00<00?ooool0=ooo1000
dooo000?ool00`00ooooo`0kool3003@ool000ooo`03003ooooo03koo`<00<goo`003ooo00<00?oo
ool0@Ooo1000bOoo000?ool00`00ooooo`15ool30036ool000ooo`03003ooooo04Soo`<00<?oo`00
3ooo00<00?ooool0Booo0P00`Ooo000?ool00`00ooooo`1=ool3002nool000ooo`80057oo`<00;_o
o`003ooo00<00?ooool0Dooo0`00^?oo000?ool00`00ooooo`1Fool2002fool000ooo`03003ooooo
05Soo`<00;?oo`003ooo00<00?ooool0Fooo0P00/Ooo000?ool00`00ooooo`1Mool2002_ool000oo
o`03003ooooo05ooo`800:goo`001ooo10001?oo00<00?ooool0HOoo0P00Zooo0007ool00`00oooo
o`05ool00`00ooooo`1Sool2002Yool000Soo`03003ooooo00Coo`<006Goo`800:Ooo`0000?oo`00
00000P001?oo00<00?ooool00ooo00<00?ooool0Iooo0P00YOoo000:ool00`00ooooo`02ool00`00
ooooo`1Yool2002Sool000Ooo`04003ooooo0004ool00`00ooooo`1[ool2002Qool000Soo`8000Go
o`03003ooooo06goo`8009ooo`003ooo00<00?ooool0Kooo0P00WOoo000?ool00`00ooooo`1aool0
0`00ooooo`2Jool000ooo`03003ooooo07;oo`8009[oo`003ooo00<00?ooool0M?oo0P00V?oo000?
ool2001gool00`00ooooo`2Eool000ooo`03003ooooo07Ooo`8009Goo`003ooo00<00?ooool0NOoo
0P00Tooo000?ool00`00ooooo`1kool00`00ooooo`2@ool000ooo`03003ooooo07coo`80093oo`00
3ooo00<00?ooool0O_oo00<00?ooool0SOoo000?ool00`00ooooo`1oool2002=ool000ooo`03003o
oooo087oo`8008_oo`003ooo00<00?ooool0Pooo00<00?ooool0R?oo000?ool20025ool20028ool0
00ooo`03003ooooo08Koo`03003ooooo08Goo`003ooo00<00?ooool0Qooo0P00QOoo000?ool00`00
ooooo`29ool00`00ooooo`22ool000ooo`03003ooooo08[oo`8008;oo`003ooo00<00?ooool0S?oo
0P00P?oo000?ool00`00ooooo`2>ool00`00ooooo`1mool000ooo`03003ooooo08ooo`8007goo`00
3ooo00<00?ooool08_oo0P001?oo00<00?ooool00_oo0`008ooo10008_oo10000ooo00<00?ooool0
0_oo0`002Ooo00<00?ooool06?oo10008_oo10000ooo00<00?ooool00_oo0`009?oo0P001ooo000?
ool2000Rool01000ooooo`002ooo00<00?ooool08_oo00<00?ooool08ooo00<00?ooool02_oo00<0
0?ooool01ooo0P006?oo00<00?ooool08ooo00<00?ooool03?oo00<00?ooool08?oo00@00?ooool0
00Koo`003ooo00<00?ooool08Ooo00@00?ooool000_oo`03003ooooo02;oo`03003ooooo02?oo`03
003ooooo00[oo`03003ooooo00Woo`03003ooooo01Koo`03003ooooo02?oo`03003ooooo00_oo`03
003ooooo02?oo`03003ooooo00Coo`003ooo00<00?ooool08Ooo00@00?ooool000Soo`<002Goo`03
003ooooo02?oo`03003ooooo00Ooo`<000goo`03003ooooo01Koo`03003ooooo02?oo`03003ooooo
00Ooo`<002Coo`8000Ooo`003ooo00<00?ooool08Ooo00@00?ooool000Soo`03003ooooo02Goo`03
003ooooo02?oo`03003ooooo00Ooo`03003ooooo00koo`8001Ooo`03003ooooo02?oo`03003ooooo
00Koo`03003ooooo02Koo`03003ooooo00Coo`003ooo00<00?ooool08Ooo00@00?ooool000Soo`03
003ooooo02?oo`<002?oo`<000Woo`03003ooooo013oo`03003ooooo017oo`04003ooooo000Rool0
1000ooooo`002?oo00<00?ooool08ooo00@00?ooool000Koo`003ooo00<00?ooool08_oo0P002Ooo
10009?oo00<00?ooool08ooo00<00?ooool01ooo10004?oo0P004_oo0P009?oo0P002Ooo10008ooo
0P001ooo000?ool00`00ooooo`2Kool00`00ooooo`1`ool000ooo`03003ooooo09coo`80073oo`00
3ooo00<00?ooool0W_oo00<00?ooool0KOoo0008oooo000I000000ooo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Koo`04003oooooool20003ool00`00ooooo`06ool00`00ooooo`05ool0
0`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool0
00ooo`03003ooooo02Woo`03003ooooo02Woo`03003ooooo02Woo`03003ooooo01koo`03003ooooo
00Woo`03003ooooo02Woo`03003ooooo02Woo`03003ooooo00Goo`003ooo00<00?ooool0Xooo00<0
0?ooool0J?oo000?ool00`00ooooo`2Tool2001Xool000ooo`03003ooooo0:Koo`03003ooooo06Go
o`003ooo00<00?ooool0Yooo00<00?ooool0I?oo000?ool00`00ooooo`2Xool2001Tool000ooo`03
003ooooo0:[oo`03003ooooo067oo`003ooo0P00[?oo00<00?ooool0H?oo000?ool00`00ooooo`2/
ool00`00ooooo`1Oool000ooo`03003ooooo0:goo`03003ooooo05koo`003ooo00<00?ooool0[_oo
0P00G_oo000?ool00`00ooooo`2`ool00`00ooooo`1Kool000ooo`03003ooooo0;7oo`03003ooooo
05[oo`003ooo00<00?ooool0/_oo00<00?ooool0FOoo000?ool00`00ooooo`2cool00`00ooooo`1H
ool000ooo`03003ooooo0;Coo`03003ooooo05Ooo`003ooo0P00]_oo00<00?ooool0E_oo000?ool0
0`00ooooo`2fool2001Fool000ooo`03003ooooo0;Soo`03003ooooo05?oo`003ooo00<00?ooool0
^Ooo00<00?ooool0D_oo000?ool00`00ooooo`2jool00`00ooooo`1Aool000ooo`03003ooooo0;_o
o`03003ooooo053oo`003ooo00<00?ooool0_?oo00<00?ooool0Cooo000?ool00`00ooooo`2mool0
0`00ooooo`1>ool000ooo`03003ooooo0;koo`8004koo`003ooo0P00`Ooo00<00?ooool0Booo000?
ool00`00ooooo`31ool00`00ooooo`1:ool000ooo`03003ooooo0<;oo`03003ooooo04Woo`003ooo
00<00?ooool0`ooo00<00?ooool0B?oo000?ool00`00ooooo`34ool20018ool000ooo`03003ooooo
0<Koo`03003ooooo04Goo`003ooo00<00?ooool0aooo00<00?ooool0A?oo0007ool40004ool00`00
ooooo`38ool00`00ooooo`13ool000Ooo`03003ooooo00Goo`03003ooooo0<Woo`03003ooooo04;o
o`002?oo00<00?ooool01?oo0`00b_oo00<00?ooool0@Ooo0009ool00`00ooooo`03ool00`00oooo
o`3;ool00`00ooooo`10ool000[oo`03003ooooo00;oo`03003ooooo0<coo`03003ooooo03ooo`00
1ooo00@00?ooool000Coo`03003ooooo0<goo`03003ooooo03koo`002?oo0P001Ooo00<00?ooool0
c_oo00<00?ooool0?Ooo000?ool00`00ooooo`3?ool00`00ooooo`0lool000ooo`03003ooooo0=3o
o`03003ooooo03_oo`003ooo00<00?ooool0dOoo00<00?ooool0>_oo000?ool00`00ooooo`3Bool0
0`00ooooo`0iool000ooo`800=Coo`03003ooooo03Soo`003ooo00<00?ooool0e?oo00<00?ooool0
=ooo000?ool00`00ooooo`3Eool00`00ooooo`0fool000ooo`03003ooooo0=Koo`03003ooooo03Go
o`003ooo00<00?ooool0eooo00<00?ooool0=?oo000?ool00`00ooooo`3Hool00`00ooooo`0cool0
00ooo`03003ooooo0=Woo`03003ooooo03;oo`003ooo00<00?ooool0f_oo00<00?ooool0<Ooo000?
ool00`00ooooo`3Kool00`00ooooo`0`ool000ooo`800=goo`03003ooooo02ooo`003ooo00<00?oo
ool0gOoo00<00?ooool0;_oo000?ool00`00ooooo`3Nool00`00ooooo`0]ool000ooo`03003ooooo
0=ooo`03003ooooo02coo`003ooo00<00?ooool0h?oo00<00?ooool0:ooo000?ool00`00ooooo`3Q
ool00`00ooooo`0Zool000ooo`03003ooooo0>7oo`03003ooooo02[oo`003ooo00<00?ooool0h_oo
00<00?ooool0:Ooo000?ool00`00ooooo`3Sool00`00ooooo`0Xool000ooo`800>Goo`03003ooooo
02Ooo`003ooo00<00?ooool0iOoo00<00?ooool09_oo000?ool00`00ooooo`3Vool00`00ooooo`0U
ool000ooo`03003ooooo0>Ooo`03003ooooo02Coo`003ooo00<00?ooool0j?oo00<00?ooool08ooo
000?ool00`00ooooo`3Yool00`00ooooo`0Rool000ooo`03003ooooo0>[oo`03003ooooo027oo`00
2Ooo00<00?ooool00ooo00<00?ooool0jooo00<00?ooool08?oo0009ool00`00ooooo`03ool00`00
ooooo`3/ool00`00ooooo`0Oool000Koo`D000Coo`<00>coo`03003ooooo01ooo`001_oo00@00?oo
ool000Goo`03003ooooo0>goo`03003ooooo01koo`001ooo00<00?oo00001Ooo00<00?ooool0k_oo
00<00?ooool07Ooo0008ool20005ool00`00ooooo`3_ool00`00ooooo`0Lool000Woo`03003ooooo
00?oo`03003ooooo0?3oo`03003ooooo01_oo`003ooo00<00?ooool0lOoo00<00?ooool06_oo000?
ool00`00ooooo`3bool00`00ooooo`0Iool000ooo`03003ooooo0??oo`03003ooooo01Soo`003ooo
00<00?ooool0m?oo00<00?ooool05ooo000?ool2003eool00`00ooooo`0Gool000ooo`03003ooooo
0?Goo`03003ooooo01Koo`003ooo00<00?ooool0m_oo00<00?ooool05Ooo000?ool00`00ooooo`3g
ool00`00ooooo`0Dool000ooo`03003ooooo0?Soo`03003ooooo01?oo`003ooo00<00?ooool0nOoo
00<00?ooool04_oo000?ool00`00ooooo`3iool00`00ooooo`0Bool000ooo`03003ooooo0?[oo`03
003ooooo017oo`003ooo00<00?ooool0nooo00<00?ooool04?oo000?ool00`00ooooo`3lool00`00
ooooo`0?ool000ooo`800?koo`03003ooooo00koo`003ooo00<00?ooool0o_oo00<00?ooool03Ooo
000?ool00`00ooooo`3nool00`00ooooo`0=ool000ooo`03003ooooo0?ooo`03003ooooo00coo`00
3ooo00<00?ooool0oooo0Ooo00<00?ooool02ooo000?ool00`00ooooo`3oool2ool00`00ooooo`0:
ool000ooo`03003ooooo0?ooo`?oo`03003ooooo00Woo`003ooo00<00?ooool0oooo1?oo00<00?oo
ool02?oo000?ool00`00ooooo`3oool4ool00`00ooooo`08ool000ooo`800?ooo`Koo`03003ooooo
00Ooo`003ooo00<00?ooool0oooo1_oo00<00?ooool01_oo000?ool00`00ooooo`3oool7ool00`00
ooooo`05ool000ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo3ooo000?ool00`00oooo
o`3oool?ool000ooo`03003ooooo0?ooo`ooo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.17529, -3.77721, \
0.0113251, 0.0552054}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[%, %733]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.217227 0.00455649 [
[.18254 .20473 -9 -9 ]
[.18254 .20473 9 0 ]
[.34127 .20473 -3 -9 ]
[.34127 .20473 3 0 ]
[.5 .20473 -9 -9 ]
[.5 .20473 9 0 ]
[.65873 .20473 -3 -9 ]
[.65873 .20473 3 0 ]
[.81746 .20473 -9 -9 ]
[.81746 .20473 9 0 ]
[.97619 .20473 -3 -9 ]
[.97619 .20473 3 0 ]
[.01131 .03497 -18 -4.5 ]
[.01131 .03497 0 4.5 ]
[.01131 .1261 -18 -4.5 ]
[.01131 .1261 0 4.5 ]
[.01131 .30836 -12 -4.5 ]
[.01131 .30836 0 4.5 ]
[.01131 .39949 -12 -4.5 ]
[.01131 .39949 0 4.5 ]
[.01131 .49062 -12 -4.5 ]
[.01131 .49062 0 4.5 ]
[.01131 .58175 -12 -4.5 ]
[.01131 .58175 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .21723 m
.18254 .22348 L
s
[(0.5)] .18254 .20473 0 1 Mshowa
.34127 .21723 m
.34127 .22348 L
s
[(1)] .34127 .20473 0 1 Mshowa
.5 .21723 m
.5 .22348 L
s
[(1.5)] .5 .20473 0 1 Mshowa
.65873 .21723 m
.65873 .22348 L
s
[(2)] .65873 .20473 0 1 Mshowa
.81746 .21723 m
.81746 .22348 L
s
[(2.5)] .81746 .20473 0 1 Mshowa
.97619 .21723 m
.97619 .22348 L
s
[(3)] .97619 .20473 0 1 Mshowa
.125 Mabswid
.05556 .21723 m
.05556 .22098 L
s
.0873 .21723 m
.0873 .22098 L
s
.11905 .21723 m
.11905 .22098 L
s
.15079 .21723 m
.15079 .22098 L
s
.21429 .21723 m
.21429 .22098 L
s
.24603 .21723 m
.24603 .22098 L
s
.27778 .21723 m
.27778 .22098 L
s
.30952 .21723 m
.30952 .22098 L
s
.37302 .21723 m
.37302 .22098 L
s
.40476 .21723 m
.40476 .22098 L
s
.43651 .21723 m
.43651 .22098 L
s
.46825 .21723 m
.46825 .22098 L
s
.53175 .21723 m
.53175 .22098 L
s
.56349 .21723 m
.56349 .22098 L
s
.59524 .21723 m
.59524 .22098 L
s
.62698 .21723 m
.62698 .22098 L
s
.69048 .21723 m
.69048 .22098 L
s
.72222 .21723 m
.72222 .22098 L
s
.75397 .21723 m
.75397 .22098 L
s
.78571 .21723 m
.78571 .22098 L
s
.84921 .21723 m
.84921 .22098 L
s
.88095 .21723 m
.88095 .22098 L
s
.9127 .21723 m
.9127 .22098 L
s
.94444 .21723 m
.94444 .22098 L
s
.25 Mabswid
0 .21723 m
1 .21723 L
s
.02381 .03497 m
.03006 .03497 L
s
[(-40)] .01131 .03497 1 0 Mshowa
.02381 .1261 m
.03006 .1261 L
s
[(-20)] .01131 .1261 1 0 Mshowa
.02381 .30836 m
.03006 .30836 L
s
[(20)] .01131 .30836 1 0 Mshowa
.02381 .39949 m
.03006 .39949 L
s
[(40)] .01131 .39949 1 0 Mshowa
.02381 .49062 m
.03006 .49062 L
s
[(60)] .01131 .49062 1 0 Mshowa
.02381 .58175 m
.03006 .58175 L
s
[(80)] .01131 .58175 1 0 Mshowa
.125 Mabswid
.02381 .05775 m
.02756 .05775 L
s
.02381 .08053 m
.02756 .08053 L
s
.02381 .10331 m
.02756 .10331 L
s
.02381 .14888 m
.02756 .14888 L
s
.02381 .17166 m
.02756 .17166 L
s
.02381 .19444 m
.02756 .19444 L
s
.02381 .24001 m
.02756 .24001 L
s
.02381 .26279 m
.02756 .26279 L
s
.02381 .28557 m
.02756 .28557 L
s
.02381 .33114 m
.02756 .33114 L
s
.02381 .35392 m
.02756 .35392 L
s
.02381 .3767 m
.02756 .3767 L
s
.02381 .42227 m
.02756 .42227 L
s
.02381 .44505 m
.02756 .44505 L
s
.02381 .46783 m
.02756 .46783 L
s
.02381 .5134 m
.02756 .5134 L
s
.02381 .53618 m
.02756 .53618 L
s
.02381 .55896 m
.02756 .55896 L
s
.02381 .01218 m
.02756 .01218 L
s
.02381 .60453 m
.02756 .60453 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .20168 m
.06423 .20171 L
.07374 .20175 L
.0929 .20186 L
.11399 .20202 L
.13363 .2022 L
.17133 .20266 L
.21139 .20328 L
.24999 .20403 L
.28712 .20487 L
.32663 .2059 L
.36467 .20703 L
.40124 .20823 L
.44019 .20964 L
.47766 .21113 L
.51751 .21284 L
.5559 .21463 L
.59282 .21647 L
.63211 .21857 L
.66994 .22072 L
.71013 .22314 L
.74886 .22562 L
.78613 .22812 L
.82576 .23093 L
.86394 .23376 L
.90064 .23661 L
.93972 .23978 L
.97619 .24286 L
s
.05556 .60332 m
.07374 .5745 L
.0929 .53282 L
.13363 .41843 L
.17042 .30483 L
.20958 .19477 L
.24727 .11198 L
.26787 .07813 L
.28734 .05349 L
.29671 .04405 L
.30664 .03566 L
.31595 .02923 L
.32447 .02449 L
.33357 .02055 L
.33825 .01895 L
.34335 .01751 L
.34806 .01647 L
.35242 .01573 L
.3566 .01521 L
.35893 .01501 L
.36104 .01487 L
.36221 .01481 L
.36329 .01477 L
.36448 .01474 L
.36576 .01472 L
.3664 .01472 L
.36709 .01472 L
.36836 .01473 L
.36959 .01475 L
.37073 .01479 L
.37179 .01484 L
.37279 .01489 L
.37506 .01504 L
.37727 .01523 L
.37965 .01547 L
.38476 .01616 L
.39015 .0171 L
.39978 .01929 L
.41886 .02526 L
.43661 .03236 L
.47436 .05063 L
.51447 .07222 L
.55312 .09307 L
.5903 .11207 L
.62986 .13054 L
.66795 .14645 L
.70457 .16003 L
.74357 .17278 L
.7811 .18358 L
.821 .19371 L
Mistroke
.85944 .20238 L
.89641 .20989 L
.93575 .21715 L
.97362 .22358 L
.97619 .224 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
5Ooo00<00?ooool0oooo2Ooo000Eool2003oool:ool001Goo`03003ooooo05Goo`/00:Soo`005Ooo
00<00?ooool0D_oo0`002ooo1@00Xooo000Eool00`00ooooo`1@ool2000Cool2002Qool000Woo`03
003ooooo00;oo`8000Goo`03003ooooo04koo`8001Ooo`<009koo`002Ooo00D00?ooooooo`0000;o
o`03003ooooo00;oo`03003ooooo04coo`8001coo`8009coo`001_oo1@000_oo00@00?ooool000Co
o`<004_oo`03003ooooo01koo`8009[oo`0000?oo`0000000P0000Goo`00ooooo`0000?oo`04003o
oooo0004ool00`00ooooo`1:ool00`00ooooo`0Qool2002Hool000Ooo`03003oo`0000?oo`04003o
oooo0004ool00`00ooooo`18ool2000Vool2002Fool000Soo`8000?oo`04003ooooo0004ool00`00
ooooo`17ool00`00ooooo`0Xool2002Dool000Woo`03003ooooo00;oo`8000Goo`03003ooooo04Ko
o`03003ooooo02_oo`8009;oo`005Ooo00<00?ooool0A_oo00<00?ooool0;Ooo0P00T?oo000Eool0
0`00ooooo`15ool00`00ooooo`0`ool2002>ool001Goo`8004Goo`03003ooooo03?oo`8008coo`00
5Ooo00<00?ooool0@ooo00<00?ooool0=_oo0P00R_oo000Eool00`00ooooo`12ool00`00ooooo`0i
ool20028ool001Goo`03003ooooo047oo`03003ooooo03coo`8008Koo`005Ooo00<00?ooool0@?oo
00<00?ooool0?ooo0P00Q?oo000Eool00`00ooooo`0oool00`00ooooo`12ool20022ool001Goo`80
03ooo`03003ooooo04Goo`80083oo`005Ooo00<00?ooool0?_oo00<00?ooool0Aooo0P00O_oo000E
ool00`00ooooo`0mool00`00ooooo`1:ool2001lool001Goo`03003ooooo03goo`03003ooooo04co
o`8007[oo`005Ooo00<00?ooool0??oo00<00?ooool0Cooo0P00N?oo000Eool00`00ooooo`0lool0
0`00ooooo`1Aool2001fool001Goo`8003coo`03003ooooo05Coo`8007Coo`005Ooo00<00?ooool0
>ooo00<00?ooool0E_oo0P00L_oo000Eool00`00ooooo`0jool00`00ooooo`1Iool3001_ool001Go
o`03003ooooo03[oo`03003ooooo05coo`8006goo`001ooo10000ooo0P001Ooo00<00?ooool0>Ooo
00<00?ooool0Gooo0P00Jooo0007ool00`00ooooo`03ool01000ooooo`001?oo00<00?ooool0>Ooo
00<00?ooool0HOoo0P00JOoo0008ool00`00ooooo`02ool01000ooooo`001?oo0`00>?oo00<00?oo
ool0I?oo0P00Iooo00000ooo000000020004ool01@00oooooooo00000_oo00<00?ooool00_oo00<0
0?ooool0>?oo00<00?ooool0I_oo0P00IOoo000:ool01000ooooo`000_oo00<00?ooool00_oo00<0
0?ooool0=ooo00<00?ooool0JOoo0P00Hooo0007ool01000ooooo`000_oo00@00?ooool000Coo`03
003ooooo03Ooo`03003ooooo06_oo`80067oo`002?oo0P001?oo0P001Ooo00<00?ooool0=_oo00<0
0?ooool0K_oo0P00Gooo000Eool00`00ooooo`0fool00`00ooooo`1`ool2001Mool001Goo`8003Ko
o`03003ooooo07?oo`<005[oo`005Ooo00<00?ooool0=Ooo00<00?ooool0M_oo0`00Eooo000Eool0
0`00ooooo`0dool00`00ooooo`1jool4001Cool001Goo`03003ooooo03Coo`03003ooooo07koo`<0
053oo`005Ooo00<00?ooool0<ooo00<00?ooool0P_oo0`00COoo000Eool00`00ooooo`0cool00`00
ooooo`25ool2001;ool001Goo`03003ooooo03;oo`03003ooooo08Soo`<004Soo`005Ooo0P00<ooo
00<00?ooool0Rooo0`00AOoo000Eool00`00ooooo`0aool00`00ooooo`2?ool30012ool001Goo`03
003ooooo027oo`8000Coo`03003ooooo00;oo`<000;oo`03003ooooo01goo`@0027oo`@000?oo`03
003ooooo00;oo`<002?oo`@001[oo`@000?oo`@000?oo`03003ooooo00;oo`<002?oo`8000Ooo`00
5Ooo00<00?ooool08?oo00@00?ooool000_oo`8002;oo`03003ooooo02;oo`03003ooooo00[oo`03
003ooooo023oo`03003ooooo01ooo`@000koo`03003ooooo01ooo`04003ooooo0006ool001Goo`03
003ooooo023oo`04003ooooo000;ool2000Rool00`00ooooo`0Rool00`00ooooo`0:ool00`00oooo
o`0Qool00`00ooooo`0Qool4000;ool00`00ooooo`0Rool00`00ooooo`04ool001Goo`03003ooooo
023oo`04003ooooo0008ool4000Sool00`00ooooo`0Rool00`00ooooo`07ool3000Uool00`00oooo
o`0Rool00`00ool000030004ool3000Sool20007ool001Goo`80027oo`04003ooooo0008ool01000
ooooo`008ooo00<00?ooool08_oo00<00?ooool01ooo00<00?ooool09_oo00<00?ooool08_oo00<0
0?ooool00_oo1@009ooo00<00?ooool01?oo000Eool00`00ooooo`0Pool01000ooooo`002?oo00@0
0?ooool0027oo`<002;oo`<000Woo`03003ooooo02?oo`04003ooooo000Qool01000ooooo`002?oo
1@008?oo00@00?ooool000Koo`005Ooo00<00?ooool01_oo>P004ooo00<00?ooool08_oo00<00?oo
ool01ooo10008ooo0P008ooo0P002Ooo100000?oo`0000000`007?oo0P001ooo000Eool00`00oooo
o`0^ool00`00ooooo`0?oolO0024ool5000Pool001Goo`03003ooooo02koo`03003ooooo02kooal0
06[oo`D001_oo`005Ooo00<00?ooool0;Ooo00<00?ooool0C_oo5@00F_oo1P005Ooo000>oooo000C
000001Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003o
oooo00Koo`03003ooooo00;oo`05003oooooool00007ool00`00ooooo`06ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`04ool;0008ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool60003ool00`00ooooo`05ool0
01Goo`03003ooooo02Soo`05003oooooool0000Vool00`00ooooo`0Xool00`00ooooo`0Yool00`00
ooooo`06oolD000>ool00`00ooooo`0Uool40007ool001Goo`03003ooooo02coo`03003ooooo09So
o`/003Koo`005Ooo00<00?ooool0;?oo00<00?ooool0Xooo2P00;?oo000Eool00`00ooooo`0[ool0
0`00ooooo`2^oolE000Gool001Goo`8002coo`03003ooooo0<?oo`X000goo`005Ooo00<00?ooool0
:ooo00<00?ooool0cOoo1P001ooo000Eool00`00ooooo`0Zool00`00ooooo`3Kool001Goo`03003o
oooo02[oo`03003ooooo0=_oo`005Ooo00<00?ooool0:_oo00<00?ooool0fooo000Eool00`00oooo
o`0Yool00`00ooooo`3Lool001Goo`8002[oo`03003ooooo0=coo`005Ooo00<00?ooool0:Ooo00<0
0?ooool0g?oo000Eool00`00ooooo`0Xool00`00ooooo`3Mool001Goo`03003ooooo02Soo`03003o
oooo0=goo`005Ooo00<00?ooool0:?oo00<00?ooool0gOoo000Eool00`00ooooo`0Wool00`00oooo
o`3Nool001Goo`03003ooooo02Ooo`03003ooooo0=koo`005Ooo0P00:?oo00<00?ooool0g_oo000E
ool00`00ooooo`0Vool00`00ooooo`3Oool001Goo`03003ooooo02Koo`03003ooooo0=ooo`005Ooo
00<00?ooool09_oo00<00?ooool0gooo0007ool40003ool20005ool00`00ooooo`0Uool00`00oooo
o`3Pool000Ooo`03003ooooo00?oo`04003ooooo0004ool00`00ooooo`0Uool00`00ooooo`3Pool0
00Soo`03003ooooo00;oo`04003ooooo0004ool3000Uool00`00ooooo`3Pool000Woo`05003ooooo
ool00002ool00`00ooooo`02ool00`00ooooo`0Tool00`00ooooo`3Qool000[oo`04003ooooo0002
ool00`00ooooo`02ool00`00ooooo`0Tool00`00ooooo`3Qool000Ooo`04003ooooo0002ool01000
ooooo`001?oo00<00?ooool09?oo00<00?ooool0hOoo0008ool20004ool20005ool00`00ooooo`0S
ool00`00ooooo`3Rool001Goo`03003ooooo02?oo`03003ooooo0>;oo`005Ooo0P009?oo00<00?oo
ool0h_oo000Eool00`00ooooo`0Rool00`00ooooo`3Sool001Goo`03003ooooo02;oo`03003ooooo
0>?oo`005Ooo00<00?ooool08_oo00<00?ooool0hooo000Eool00`00ooooo`0Qool00`00ooooo`3T
ool001Goo`03003ooooo027oo`03003ooooo0>Coo`005Ooo0P008_oo00<00?ooool0i?oo000Eool0
0`00ooooo`0Pool00`00ooooo`3Uool001Goo`03003ooooo023oo`03003ooooo0>Goo`005Ooo00<0
0?ooool08?oo00<00?ooool0iOoo000Eool00`00ooooo`0Pool00`00ooooo`3Uool001Goo`03003o
oooo01ooo`03003ooooo0>Koo`005Ooo0P008?oo00<00?ooool0i_oo000Eool00`00ooooo`0Oool0
0`00ooooo`3Vool001Goo`03003ooooo01koo`03003ooooo0>Ooo`005Ooo00<00?ooool07_oo00<0
0?ooool0iooo000Eool00`00ooooo`0Nool00`00ooooo`3Wool000Woo`03003ooooo00;oo`8000Go
o`03003ooooo01goo`03003ooooo0>Soo`002Ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`03
003ooooo01goo`03003ooooo0>Soo`001_oo1@000_oo00@00?ooool000Coo`<001goo`03003ooooo
0>Soo`001_oo00@00?ooool000?oo`04003ooooo0004ool00`00ooooo`0Lool00`00ooooo`3Yool0
00Ooo`03003oo`0000?oo`04003ooooo0004ool00`00ooooo`0Lool00`00ooooo`3Yool000Soo`80
00?oo`04003ooooo0004ool00`00ooooo`0Lool00`00ooooo`3Yool000Woo`03003ooooo00;oo`80
00Goo`03003ooooo01_oo`03003ooooo0>[oo`005Ooo00<00?ooool06ooo00<00?ooool0j_oo000E
ool2000Lool00`00ooooo`3Zool001Goo`03003ooooo01[oo`03003ooooo0>_oo`005Ooo00<00?oo
ool06_oo00<00?ooool0jooo000Eool00`00ooooo`0Jool00`00ooooo`3[ool001Goo`03003ooooo
01Woo`03003ooooo0>coo`005Ooo00<00?ooool06Ooo00<00?ooool0k?oo000Eool2000Jool00`00
ooooo`3/ool001Goo`03003ooooo01Soo`03003ooooo0>goo`005Ooo00<00?ooool06?oo00<00?oo
ool0kOoo000Eool00`00ooooo`0Gool00`00ooooo`3^ool001Goo`03003ooooo01Ooo`03003ooooo
0>koo`005Ooo00<00?ooool05ooo00<00?ooool0k_oo000Eool2000Gool00`00ooooo`3_ool001Go
o`03003ooooo01Koo`03003ooooo0>ooo`005Ooo00<00?ooool05_oo00<00?ooool0kooo000Eool0
0`00ooooo`0Eool00`00ooooo`3`ool000Soo`8000Coo`8000Goo`03003ooooo01Goo`03003ooooo
0?3oo`001ooo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`0Eool00`00ooooo`3`ool0
00Ooo`04003ooooo0002ool01000ooooo`001?oo0`005?oo00<00?ooool0lOoo0007ool30003ool0
1000ooooo`001?oo00<00?ooool05?oo00<00?ooool0lOoo0007ool00`00ooooo`03ool01000oooo
o`001?oo00<00?ooool05?oo00<00?ooool0lOoo0007ool00`00ooooo`03ool01000ooooo`001?oo
00<00?ooool04ooo00<00?ooool0l_oo0008ool30003ool20005ool00`00ooooo`0Cool00`00oooo
o`3bool001Goo`03003ooooo01;oo`03003ooooo0??oo`005Ooo00<00?ooool04_oo00<00?ooool0
looo000Eool2000Cool00`00ooooo`3cool001Goo`03003ooooo017oo`03003ooooo0?Coo`005Ooo
00<00?ooool04Ooo00<00?ooool0m?oo000Eool00`00ooooo`0Aool00`00ooooo`3dool001Goo`03
003ooooo013oo`03003ooooo0?Goo`005Ooo00<00?ooool04?oo00<00?ooool0mOoo000Eool2000@
ool00`00ooooo`3fool001Goo`03003ooooo00ooo`03003ooooo0?Koo`005Ooo00<00?ooool03_oo
00<00?ooool0mooo000Eool00`00ooooo`0>ool00`00ooooo`3gool001Goo`03003ooooo00goo`03
003ooooo0?Soo`005Ooo00<00?ooool03Ooo00<00?ooool0n?oo000Eool2000=ool00`00ooooo`3i
ool001Goo`03003ooooo00coo`03003ooooo0?Woo`005Ooo00<00?ooool02ooo00<00?ooool0n_oo
000Eool00`00ooooo`0;ool00`00ooooo`3jool000Soo`8000Coo`8000Goo`03003ooooo00[oo`03
003ooooo0?_oo`001ooo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`0:ool00`00oooo
o`3kool000Ooo`04003ooooo0002ool01000ooooo`001?oo0`002Ooo00<00?ooool0o?oo0008ool2
0003ool01000ooooo`001?oo00<00?ooool02Ooo00<00?ooool0o?oo0007ool01000ooooo`000_oo
00@00?ooool000Coo`03003ooooo00Soo`03003ooooo0?goo`001ooo00@00?ooool000;oo`04003o
oooo0004ool00`00ooooo`08ool00`00ooooo`3mool000Soo`8000Coo`8000Goo`03003ooooo00Oo
o`03003ooooo0?koo`005Ooo00<00?ooool01ooo00<00?ooool0o_oo000Eool20007ool00`00oooo
o`3oool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3o
ool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -51.2183, \
0.0115777, 0.806644}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Im[thSXitestfunc[om - I\ 1.64, 0.1, 10^\((\(-4\))\), 0.1]], {om, 0.1, 
        3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.603319 0.0598685 [
[.18254 .59082 -9 -9 ]
[.18254 .59082 9 0 ]
[.34127 .59082 -3 -9 ]
[.34127 .59082 3 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.81746 .59082 -9 -9 ]
[.81746 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.01131 .00463 -18 -4.5 ]
[.01131 .00463 0 4.5 ]
[.01131 .12437 -12 -4.5 ]
[.01131 .12437 0 4.5 ]
[.01131 .24411 -12 -4.5 ]
[.01131 .24411 0 4.5 ]
[.01131 .36385 -12 -4.5 ]
[.01131 .36385 0 4.5 ]
[.01131 .48358 -12 -4.5 ]
[.01131 .48358 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .60332 m
.18254 .60957 L
s
[(0.5)] .18254 .59082 0 1 Mshowa
.34127 .60332 m
.34127 .60957 L
s
[(1)] .34127 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(1.5)] .5 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(2)] .65873 .59082 0 1 Mshowa
.81746 .60332 m
.81746 .60957 L
s
[(2.5)] .81746 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(3)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.15079 .60332 m
.15079 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.27778 .60332 m
.27778 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.53175 .60332 m
.53175 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.9127 .60332 m
.9127 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .00463 m
.03006 .00463 L
s
[(-10)] .01131 .00463 1 0 Mshowa
.02381 .12437 m
.03006 .12437 L
s
[(-8)] .01131 .12437 1 0 Mshowa
.02381 .24411 m
.03006 .24411 L
s
[(-6)] .01131 .24411 1 0 Mshowa
.02381 .36385 m
.03006 .36385 L
s
[(-4)] .01131 .36385 1 0 Mshowa
.02381 .48358 m
.03006 .48358 L
s
[(-2)] .01131 .48358 1 0 Mshowa
.125 Mabswid
.02381 .03457 m
.02756 .03457 L
s
.02381 .0645 m
.02756 .0645 L
s
.02381 .09444 m
.02756 .09444 L
s
.02381 .15431 m
.02756 .15431 L
s
.02381 .18424 m
.02756 .18424 L
s
.02381 .21417 m
.02756 .21417 L
s
.02381 .27404 m
.02756 .27404 L
s
.02381 .30398 m
.02756 .30398 L
s
.02381 .33391 m
.02756 .33391 L
s
.02381 .39378 m
.02756 .39378 L
s
.02381 .42371 m
.02756 .42371 L
s
.02381 .45365 m
.02756 .45365 L
s
.02381 .51352 m
.02756 .51352 L
s
.02381 .54345 m
.02756 .54345 L
s
.02381 .57338 m
.02756 .57338 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .58399 m
.0929 .56124 L
.13363 .53638 L
.17189 .51298 L
.20867 .49041 L
.24783 .46633 L
.28553 .44309 L
.32559 .41833 L
.36419 .39443 L
.40132 .37142 L
.44083 .34691 L
.47887 .32329 L
.51544 .30058 L
.55438 .2764 L
.59186 .25312 L
.63171 .22838 L
.6701 .20455 L
.70702 .18163 L
.74631 .15725 L
.78413 .13378 L
.82433 .10885 L
.86306 .08484 L
.90033 .06173 L
.93996 .03716 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool000Ooo`@000?oo`800?oooa7oo`002Ooo00D00?oooooo
o`0000;oo`03003ooooo00;oo`03003ooooo0?ooo`Woo`002Ooo00D00?ooooooo`0000;oo`03003o
oooo00;oo`<00?ooo`Woo`0000?oo`0000000P001?oo00D00?ooooooo`0000;oo`03003ooooo00;o
o`03003ooooo0?ooo`Woo`002Ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`03003ooooo0?oo
o`Woo`001ooo0`000ooo00@00?ooool000Coo`03003ooooo0?ooo`7oo`03003ooooo00Goo`002Ooo
00<00?ooool00_oo0P001Ooo00<00?ooool0o_oo0P002?oo000Eool00`00ooooo`3lool2000:ool0
01Goo`03003ooooo0?_oo`03003ooooo00[oo`005Ooo00<00?ooool0nOoo0P003Ooo000Eool2003h
ool2000?ool001Goo`03003ooooo0?Koo`03003ooooo00ooo`005Ooo00<00?ooool0m?oo0P004_oo
000Eool00`00ooooo`3bool2000Dool001Goo`03003ooooo0?3oo`8001Koo`005Ooo00<00?ooool0
k_oo0P006?oo000Eool00`00ooooo`3/ool2000Jool001Goo`03003ooooo0>_oo`03003ooooo01[o
o`005Ooo0P00j_oo0P007Ooo000Eool00`00ooooo`3Xool00`00ooooo`0Mool001Goo`03003ooooo
0>Koo`80023oo`005Ooo00<00?ooool0iOoo00<00?ooool08?oo000Eool00`00ooooo`3Tool00`00
ooooo`0Qool001Goo`03003ooooo0>;oo`8002Coo`005Ooo00<00?ooool0hOoo00<00?ooool09?oo
000Eool00`00ooooo`3Oool2000Wool001Goo`800=koo`8002Woo`005Ooo00<00?ooool0g?oo00<0
0?ooool0:Ooo000Eool00`00ooooo`3Jool2000/ool001Goo`03003ooooo0=Soo`8002koo`005Ooo
00<00?ooool0eooo00<00?ooool0;_oo000Eool00`00ooooo`3Eool2000aool000koo`8000Goo`03
003ooooo0=Coo`03003ooooo037oo`003Ooo00@00?ooool000Coo`03003ooooo0=;oo`8003Coo`00
3Ooo00@00?ooool000Coo`<00=3oo`8003Koo`001ooo10000ooo0P001Ooo00<00?ooool0cooo00<0
0?ooool0=_oo000=ool01000ooooo`001?oo00<00?ooool0cOoo0P00>Ooo000=ool01000ooooo`00
1?oo00<00?ooool0c?oo00<00?ooool0>Ooo000>ool20005ool00`00ooooo`3:ool2000lool001Go
o`03003ooooo0<Soo`8003koo`005Ooo00<00?ooool0a_oo0P00@?oo000Eool00`00ooooo`34ool2
0012ool001Goo`03003ooooo0<;oo`8004Coo`005Ooo0P00`_oo00<00?ooool0A?oo000Eool00`00
ooooo`2oool20017ool001Goo`03003ooooo0;koo`03003ooooo04Ooo`005Ooo00<00?ooool0_?oo
0P00B_oo000Eool00`00ooooo`2kool00`00ooooo`1:ool001Goo`03003ooooo0;[oo`03003ooooo
04_oo`005Ooo00<00?ooool0^?oo0P00C_oo000Eool00`00ooooo`2gool00`00ooooo`1>ool001Go
o`800;Koo`80057oo`005Ooo00<00?ooool0/ooo0P00Dooo000Eool00`00ooooo`2bool00`00oooo
o`1Cool001Goo`03003ooooo0;3oo`8005Koo`005Ooo00<00?ooool0[_oo0P00F?oo000Eool00`00
ooooo`2]ool00`00ooooo`1Hool001Goo`03003ooooo0:_oo`8005_oo`005Ooo00<00?ooool0Z_oo
00<00?ooool0Fooo000Eool2002Yool2001Nool001Goo`03003ooooo0:Koo`80063oo`005Ooo00<0
0?ooool0YOoo00<00?ooool0H?oo000Eool00`00ooooo`2Sool2001Sool001Goo`03003ooooo0:;o
o`03003ooooo06?oo`005Ooo00<00?ooool0X?oo0P00I_oo000>ool20005ool00`00ooooo`2Nool2
001Xool000goo`04003ooooo0004ool00`00ooooo`2Lool2001Zool000goo`04003ooooo0004ool3
002Jool2001/ool000Ooo`@000;oo`<000Goo`03003ooooo09Soo`8006koo`003Ooo00<00?ooool0
1Ooo00<00?ooool0Uooo00<00?ooool0K_oo000=ool00`00ooooo`05ool00`00ooooo`2Eool2001a
ool000koo`<000Coo`03003ooooo09Coo`03003ooooo077oo`005Ooo00<00?ooool0T_oo0P00M?oo
000Eool00`00ooooo`2Aool00`00ooooo`1dool001Goo`03003ooooo093oo`03003ooooo07Goo`00
5Ooo0P00Sooo0P00N?oo000Eool00`00ooooo`2=ool00`00ooooo`1hool001Goo`03003ooooo08_o
o`8007_oo`005Ooo00<00?ooool0ROoo0P00OOoo000Eool00`00ooooo`28ool00`00ooooo`1mool0
01Goo`03003ooooo08Koo`80083oo`005Ooo00<00?ooool0Q?oo0P00P_oo000Eool00`00ooooo`23
ool00`00ooooo`22ool001Goo`8008;oo`8008Goo`005Ooo00<00?ooool0P?oo00<00?ooool0QOoo
000Eool00`00ooooo`1nool20028ool001Goo`03003ooooo07goo`03003ooooo08Soo`005Ooo00<0
0?ooool0O?oo00<00?ooool0ROoo000Eool00`00ooooo`1jool2002<ool001Goo`03003ooooo07Wo
o`03003ooooo08coo`005Ooo00<00?ooool0Mooo0P00Sooo000Eool2001fool2002Aool001Goo`03
003ooooo07?oo`8009?oo`005Ooo00<00?ooool0LOoo0P00UOoo000Eool00`00ooooo`1_ool2002G
ool001Goo`03003ooooo06koo`03003ooooo09Ooo`005Ooo00<00?ooool0K?oo0P00V_oo000Eool0
0`00ooooo`1[ool00`00ooooo`2Jool000ooo`03003ooooo00?oo`03003ooooo06Woo`8009goo`00
3ooo00<00?ooool00ooo00<00?ooool0J?oo00<00?ooool0WOoo000<ool50004ool3001Wool00`00
ooooo`2Nool000Ooo`@00005ool00?ooool00005ool00`00ooooo`1Uool2002Qool000goo`03003o
o`0000Goo`03003ooooo06Coo`03003ooooo0:7oo`003_oo0P001Ooo00<00?ooool0H_oo0P00Y?oo
000?ool00`00ooooo`03ool00`00ooooo`1Pool2002Vool001Goo`03003ooooo05ooo`03003ooooo
0:Koo`005Ooo00<00?ooool0GOoo0P00ZOoo000Eool00`00ooooo`1Kool2002[ool001Goo`8005_o
o`03003ooooo0:_oo`005Ooo00<00?ooool0F?oo0P00[_oo000Eool00`00ooooo`1Fool2002`ool0
01Goo`03003ooooo05Coo`800;;oo`005Ooo00<00?ooool0D_oo0P00]?oo000Eool00`00ooooo`1@
ool2002fool001Goo`03003ooooo04ooo`03003ooooo0;Koo`005Ooo00<00?ooool0COoo0P00^Ooo
000Eool2001=ool00`00ooooo`2iool001Goo`03003ooooo04[oo`800;coo`005Ooo00<00?ooool0
B?oo0P00__oo000Eool00`00ooooo`17ool00`00ooooo`2nool001Goo`03003ooooo04Goo`800<7o
o`005Ooo00<00?ooool0A?oo00<00?ooool0`Ooo000Eool00`00ooooo`12ool20034ool001Goo`03
003ooooo043oo`800<Koo`005Ooo0P00@?oo00<00?ooool0a_oo000Eool00`00ooooo`0mool20039
ool001Goo`03003ooooo03_oo`800<_oo`005Ooo00<00?ooool0>_oo00<00?ooool0booo000Eool0
0`00ooooo`0hool2003>ool001Goo`03003ooooo03Ooo`03003ooooo0<koo`003Ooo10001?oo00<0
0?ooool0=Ooo0P00dOoo000=ool00`00ooooo`05ool00`00ooooo`0cool2003Cool000koo`03003o
oooo00Coo`<003;oo`03003ooooo0=?oo`001ooo10001?oo00<00?ooool00ooo00<00?ooool0<?oo
0P00e_oo000@ool00`00ooooo`02ool00`00ooooo`0_ool00`00ooooo`3Fool000goo`04003ooooo
0004ool00`00ooooo`0]ool2003Iool000koo`8000Goo`03003ooooo02_oo`800=_oo`005Ooo00<0
0?ooool0:_oo00<00?ooool0fooo000Eool00`00ooooo`0Xool2003Nool001Goo`03003ooooo02Ko
o`800>3oo`005Ooo0P009_oo00<00?ooool0h?oo000Eool00`00ooooo`0Sool2003Sool001Goo`03
003ooooo02;oo`03003ooooo0>?oo`005Ooo00<00?ooool08?oo0P00i_oo000Eool00`00ooooo`0O
ool00`00ooooo`3Vool001Goo`03003ooooo01koo`03003ooooo0>Ooo`005Ooo00<00?ooool07?oo
0P00j_oo000Eool00`00ooooo`0Kool00`00ooooo`3Zool001Goo`8001[oo`800>goo`005Ooo00<0
0?ooool05ooo0P00kooo000Eool00`00ooooo`0Eool2003aool001Goo`03003ooooo01?oo`800??o
o`005Ooo00<00?ooool04Ooo0P00mOoo000Eool00`00ooooo`0@ool00`00ooooo`3eool001Goo`03
003ooooo00koo`80017oo`8000Coo`03003ooooo00;oo`<002;oo`@0027oo`@000?oo`03003ooooo
00;oo`<002?oo`@0027oo`@000?oo`03003ooooo00;oo`<002?oo`8000Ooo`005Ooo00<00?ooool0
3?oo0P004_oo00@00?ooool000_oo`03003ooooo027oo`03003ooooo02;oo`03003ooooo00[oo`03
003ooooo023oo`03003ooooo02;oo`03003ooooo00coo`03003ooooo01ooo`04003ooooo0006ool0
01Goo`03003ooooo00_oo`03003ooooo01;oo`04003ooooo000;ool00`00ooooo`0Qool00`00oooo
o`0Rool00`00ooooo`0:ool00`00ooooo`0Qool00`00ooooo`0Rool00`00ooooo`0;ool00`00oooo
o`0Rool00`00ooooo`04ool001Goo`8000[oo`8001Goo`04003ooooo0008ool3000Tool00`00oooo
o`0Rool00`00ooooo`07ool3000Uool00`00ooooo`0Rool00`00ooooo`07ool3000Sool20007ool0
01Goo`03003ooooo00Ooo`8001Ooo`04003ooooo0008ool00`00ooooo`0Tool00`00ooooo`0Rool0
0`00ooooo`07ool00`00ooooo`0Vool00`00ooooo`0Rool00`00ooooo`06ool00`00ooooo`0Uool0
0`00ooooo`04ool001Goo`03003ooooo00Koo`03003ooooo01Ooo`04003ooooo0008ool00`00oooo
o`0Rool3000Rool30009ool00`00ooooo`0Sool01000ooooo`008Ooo00@00?ooool000Soo`03003o
oooo02;oo`04003ooooo0006ool001Goo`03003ooooo027oo`8000Woo`@002?oo`03003ooooo02;o
o`03003ooooo00Ooo`@002?oo`8002?oo`8000Woo`@002;oo`8000Ooo`005Ooo00<00?ooool0oooo
2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo
2Ooo000>oooo000C000001Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003o
oooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003o
oooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003o
oooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003o
oooo00Goo`03003ooooo00Koo`03003ooooo00Goo`005Ooo00<00?ooool0oooo2Ooo000Eool00`00
ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -10.4466, \
0.0115777, 0.0613923}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
    \(how' s\ \(\(that\)\(?\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?SXiA\)\)], "Input"],

Cell["Global`SXiA", "Print",
  CellTags->"Info3279607211-7329832"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(SXiA[z_, kn_, a_] := 
                    PiT[z] + 
                      a\ \((1\/4\ PiT[
                                z]\ \((\((3 + 3\ Cos[2\ ArcCos[kn]])\) - 
                                  z\^2\ \((3 + 5\ Cos[2\ ArcCos[kn]])\))\) + 
                            1\/12\ z\^2\ \((3 + 5\ Cos[2\ ArcCos[kn]])\) - 
                            1\/6\ \((1 + Cos[2\ ArcCos[kn]])\))\)\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "SXiA"],
      Editable->False]], "Print",
  CellTags->"Info3279607211-7329832"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?alphakx0korr\)\)], "Input"],

Cell["Global`alphakx0korr", "Print",
  CellTags->"Info3279607223-5556151"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(alphakx0korr[eta_, 
                      a_] = \(-\(\(1\/\(8\ \@a\ \((1 + a\ eta\^2)\)\^2\)\)\((\
\[ImaginaryI]\ \@\(1 + a\)\ \((\((\(-1\) - eta\^2 - 
                                    a\ \((\(-1\) + \((6 + 
                                        a)\)\ eta\^2 + \((\(-1\) + 
                                        a)\)\ eta\^4)\))\)\ Log[
                                  1 - \[ImaginaryI]\ \@a] + \((1 + eta\^2 + 
                                    a\ \((\(-1\) + \((6 + 
                                        a)\)\ eta\^2 + \((\(-1\) + 
                                        a)\)\ eta\^4)\))\)\ Log[
                                  1 + \[ImaginaryI]\ \@a] + \[ImaginaryI]\ \
\@a\ \((\(-1\) + eta\^2)\)\ \((2 + 2\ a\ eta\^2 + 
                                    2\ \((1 + 
                                        a)\)\ eta\ \((If[Im[eta] < 0, 
                                        2\ \[Pi]\ \[ImaginaryI], 0] + 
                                        Log[\(-1\) + eta] - 
                                        Log[1 + eta])\))\))\))\)\)\)\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "alphakx0korr"],
      Editable->False]], "Print",
  CellTags->"Info3279607223-5556151"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Series[alphakx0korr[z, a], {a, 0, 1}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(\(-\(1\/8\)\)\ \[ImaginaryI]\ \((\(-\[ImaginaryI]\)\ \
\((\(-1\) - z\^2)\) + \[ImaginaryI]\ \((1 + 
                    z\^2)\) + \[ImaginaryI]\ \((\(-1\) + z\^2)\)\ \((2 + 
                    2\ z\ \((If[Im[z] < 0, 2\ \[Pi]\ \[ImaginaryI], 0] + 
                          Log[\(-1\) + z] - Log[1 + z])\))\))\)\), 
        "+", \(\(-\(1\/8\)\)\ \[ImaginaryI]\ \((1\/2\ \((\(-1\) - z\^2)\) + 
              1\/2\ \((1 + z\^2)\))\)\ \@a\), 
        "+", \(\(-\(1\/8\)\)\ \[ImaginaryI]\ \((1\/3\ \[ImaginaryI]\ \
\((\(-1\) - z\^2)\) - 
              1\/3\ \[ImaginaryI]\ \((1 + 
                    z\^2)\) + \[ImaginaryI]\ \((\(-1\) + 6\ z\^2 - 
                    z\^4)\) - \[ImaginaryI]\ \((1 - 6\ z\^2 + 
                    z\^4)\) + \((1\/2 - 
                    2\ z\^2)\)\ \((\(-\[ImaginaryI]\)\ \((\(-1\) - 
                          z\^2)\) + \[ImaginaryI]\ \((1 + 
                          z\^2)\) + \[ImaginaryI]\ \((\(-1\) + 
                          z\^2)\)\ \((2 + 
                          2\ z\ \((If[Im[z] < 0, 2\ \[Pi]\ \[ImaginaryI], 
                                  0] + Log[\(-1\) + z] - 
                                Log[1 + 
                                    z])\))\))\) + \[ImaginaryI]\ \((\(-1\) + 
                    z\^2)\)\ \((2\ z\^2 + 
                    2\ z\ \((If[Im[z] < 0, 2\ \[Pi]\ \[ImaginaryI], 0] + 
                          Log[\(-1\) + z] - Log[1 + z])\))\))\)\ a\), 
        "+", \(\(-\(1\/8\)\)\ \[ImaginaryI]\ \((1\/4\ \((\(-1\) - z\^2)\) + 
              1\/4\ \((1 + z\^2)\) + 1\/2\ \((\(-1\) + 6\ z\^2 - z\^4)\) + 
              1\/2\ \((1 - 6\ z\^2 + z\^4)\) + \((1\/2 - 
                    2\ z\^2)\)\ \((1\/2\ \((\(-1\) - z\^2)\) + 
                    1\/2\ \((1 + z\^2)\))\))\)\ a\^\(3/2\)\), "+", 
        InterpretationBox[\(O[a]\^2\),
          SeriesData[ a, 0, {}, 0, 4, 2]]}],
      SeriesData[ a, 0, {
        Times[ 
          Complex[ 0, 
            Rational[ -1, 8]], 
          Plus[ 
            Times[ 
              Complex[ 0, -1], 
              Plus[ -1, 
                Times[ -1, 
                  Power[ z, 2]]]], 
            Times[ 
              Complex[ 0, 1], 
              Plus[ 1, 
                Power[ z, 2]]], 
            Times[ 
              Complex[ 0, 1], 
              Plus[ -1, 
                Power[ z, 2]], 
              Plus[ 2, 
                Times[ 2, z, 
                  Plus[ 
                    If[ 
                      Less[ 
                        Im[ z], 0], 
                      Times[ 2, Pi, I], 0], 
                    Log[ 
                      Plus[ -1, z]], 
                    Times[ -1, 
                      Log[ 
                        Plus[ 1, z]]]]]]]]], 
        Times[ 
          Complex[ 0, 
            Rational[ -1, 8]], 
          Plus[ 
            Times[ 
              Rational[ 1, 2], 
              Plus[ -1, 
                Times[ -1, 
                  Power[ z, 2]]]], 
            Times[ 
              Rational[ 1, 2], 
              Plus[ 1, 
                Power[ z, 2]]]]], 
        Times[ 
          Complex[ 0, 
            Rational[ -1, 8]], 
          Plus[ 
            Times[ 
              Complex[ 0, 
                Rational[ 1, 3]], 
              Plus[ -1, 
                Times[ -1, 
                  Power[ z, 2]]]], 
            Times[ 
              Complex[ 0, 
                Rational[ -1, 3]], 
              Plus[ 1, 
                Power[ z, 2]]], 
            Times[ 
              Complex[ 0, 1], 
              Plus[ -1, 
                Times[ 6, 
                  Power[ z, 2]], 
                Times[ -1, 
                  Power[ z, 4]]]], 
            Times[ 
              Complex[ 0, -1], 
              Plus[ 1, 
                Times[ -6, 
                  Power[ z, 2]], 
                Power[ z, 4]]], 
            Times[ 
              Plus[ 
                Rational[ 1, 2], 
                Times[ -2, 
                  Power[ z, 2]]], 
              Plus[ 
                Times[ 
                  Complex[ 0, -1], 
                  Plus[ -1, 
                    Times[ -1, 
                      Power[ z, 2]]]], 
                Times[ 
                  Complex[ 0, 1], 
                  Plus[ 1, 
                    Power[ z, 2]]], 
                Times[ 
                  Complex[ 0, 1], 
                  Plus[ -1, 
                    Power[ z, 2]], 
                  Plus[ 2, 
                    Times[ 2, z, 
                      Plus[ 
                        If[ 
                          Less[ 
                            Im[ z], 0], 
                          Times[ 2, Pi, I], 0], 
                        Log[ 
                          Plus[ -1, z]], 
                        Times[ -1, 
                          Log[ 
                            Plus[ 1, z]]]]]]]]], 
            Times[ 
              Complex[ 0, 1], 
              Plus[ -1, 
                Power[ z, 2]], 
              Plus[ 
                Times[ 2, 
                  Power[ z, 2]], 
                Times[ 2, z, 
                  Plus[ 
                    If[ 
                      Less[ 
                        Im[ z], 0], 
                      Times[ 2, Pi, I], 0], 
                    Log[ 
                      Plus[ -1, z]], 
                    Times[ -1, 
                      Log[ 
                        Plus[ 1, z]]]]]]]]], 
        Times[ 
          Complex[ 0, 
            Rational[ -1, 8]], 
          Plus[ 
            Times[ 
              Rational[ 1, 4], 
              Plus[ -1, 
                Times[ -1, 
                  Power[ z, 2]]]], 
            Times[ 
              Rational[ 1, 4], 
              Plus[ 1, 
                Power[ z, 2]]], 
            Times[ 
              Rational[ 1, 2], 
              Plus[ -1, 
                Times[ 6, 
                  Power[ z, 2]], 
                Times[ -1, 
                  Power[ z, 4]]]], 
            Times[ 
              Rational[ 1, 2], 
              Plus[ 1, 
                Times[ -6, 
                  Power[ z, 2]], 
                Power[ z, 4]]], 
            Times[ 
              Plus[ 
                Rational[ 1, 2], 
                Times[ -2, 
                  Power[ z, 2]]], 
              Plus[ 
                Times[ 
                  Rational[ 1, 2], 
                  Plus[ -1, 
                    Times[ -1, 
                      Power[ z, 2]]]], 
                Times[ 
                  Rational[ 1, 2], 
                  Plus[ 1, 
                    Power[ z, 2]]]]]]]}, 0, 4, 2]]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(1\/4\ z\ \((2\ z + \((\(-1\) + z\^2)\)\ If[Im[z] < 0, 
                  2\ \[Pi]\ \[ImaginaryI], 
                  0] + \((\(-1\) + z\^2)\)\ Log[\(-1\) + z] + Log[1 + z] - 
              z\^2\ Log[1 + z])\)\), 
        "+", \(1\/24\ \((\(-8\) + 34\ z\^2 - 24\ z\^4 - 
              3\ z\ \((3 - 7\ z\^2 + 4\ z\^4)\)\ If[Im[z] < 0, 
                  2\ \[Pi]\ \[ImaginaryI], 0] - 
              3\ z\ \((3 - 7\ z\^2 + 4\ z\^4)\)\ Log[\(-1\) + z] + 
              9\ z\ Log[1 + z] - 21\ z\^3\ Log[1 + z] + 
              12\ z\^5\ Log[1 + z])\)\ a\), "+", 
        InterpretationBox[\(O[a]\^2\),
          SeriesData[ a, 0, {}, 0, 4, 2]]}],
      SeriesData[ a, 0, {
        Times[ 
          Rational[ 1, 4], z, 
          Plus[ 
            Times[ 2, z], 
            Times[ 
              Plus[ -1, 
                Power[ z, 2]], 
              If[ 
                Less[ 
                  Im[ z], 0], 
                Times[ 2, Pi, I], 0]], 
            Times[ 
              Plus[ -1, 
                Power[ z, 2]], 
              Log[ 
                Plus[ -1, z]]], 
            Log[ 
              Plus[ 1, z]], 
            Times[ -1, 
              Power[ z, 2], 
              Log[ 
                Plus[ 1, z]]]]], 0, 
        Times[ 
          Rational[ 1, 24], 
          Plus[ -8, 
            Times[ 34, 
              Power[ z, 2]], 
            Times[ -24, 
              Power[ z, 4]], 
            Times[ -3, z, 
              Plus[ 3, 
                Times[ -7, 
                  Power[ z, 2]], 
                Times[ 4, 
                  Power[ z, 4]]], 
              If[ 
                Less[ 
                  Im[ z], 0], 
                Times[ 2, Pi, I], 0]], 
            Times[ -3, z, 
              Plus[ 3, 
                Times[ -7, 
                  Power[ z, 2]], 
                Times[ 4, 
                  Power[ z, 4]]], 
              Log[ 
                Plus[ -1, z]]], 
            Times[ 9, z, 
              Log[ 
                Plus[ 1, z]]], 
            Times[ -21, 
              Power[ z, 3], 
              Log[ 
                Plus[ 1, z]]], 
            Times[ 12, 
              Power[ z, 5], 
              Log[ 
                Plus[ 1, z]]]]]}, 0, 4, 2]]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Normal[%770[\([3, 1]\)]]], {z, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147151 1.09627 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .12434 -18 -4.5 ]
[.01131 .12434 0 4.5 ]
[.01131 .23397 -18 -4.5 ]
[.01131 .23397 0 4.5 ]
[.01131 .3436 -18 -4.5 ]
[.01131 .3436 0 4.5 ]
[.01131 .45322 -18 -4.5 ]
[.01131 .45322 0 4.5 ]
[.01131 .56285 -18 -4.5 ]
[.01131 .56285 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .12434 m
.03006 .12434 L
s
[(0.1)] .01131 .12434 1 0 Mshowa
.02381 .23397 m
.03006 .23397 L
s
[(0.2)] .01131 .23397 1 0 Mshowa
.02381 .3436 m
.03006 .3436 L
s
[(0.3)] .01131 .3436 1 0 Mshowa
.02381 .45322 m
.03006 .45322 L
s
[(0.4)] .01131 .45322 1 0 Mshowa
.02381 .56285 m
.03006 .56285 L
s
[(0.5)] .01131 .56285 1 0 Mshowa
.125 Mabswid
.02381 .03664 m
.02756 .03664 L
s
.02381 .05857 m
.02756 .05857 L
s
.02381 .08049 m
.02756 .08049 L
s
.02381 .10242 m
.02756 .10242 L
s
.02381 .14627 m
.02756 .14627 L
s
.02381 .16819 m
.02756 .16819 L
s
.02381 .19012 m
.02756 .19012 L
s
.02381 .21204 m
.02756 .21204 L
s
.02381 .25589 m
.02756 .25589 L
s
.02381 .27782 m
.02756 .27782 L
s
.02381 .29975 m
.02756 .29975 L
s
.02381 .32167 m
.02756 .32167 L
s
.02381 .36552 m
.02756 .36552 L
s
.02381 .38745 m
.02756 .38745 L
s
.02381 .40937 m
.02756 .40937 L
s
.02381 .4313 m
.02756 .4313 L
s
.02381 .47515 m
.02756 .47515 L
s
.02381 .49707 m
.02756 .49707 L
s
.02381 .519 m
.02756 .519 L
s
.02381 .54093 m
.02756 .54093 L
s
.02381 .58478 m
.02756 .58478 L
s
.02381 .6067 m
.02756 .6067 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .02564 m
.06423 .03239 L
.07374 .04161 L
.0929 .06582 L
.11399 .10076 L
.13363 .14055 L
.17133 .23358 L
.21139 .34923 L
.24999 .46451 L
.26766 .51346 L
.28712 .56011 L
.29636 .57807 L
.30146 .58639 L
.3063 .59305 L
.30892 .59607 L
.31178 .59884 L
.31438 .60084 L
.31679 .6022 L
.31808 .60272 L
.31929 .60307 L
.32038 .60326 L
.32155 .60332 L
.32217 .60329 L
.32284 .60321 L
.32354 .60307 L
.3242 .60289 L
.32537 .60242 L
.32663 .60171 L
.3278 .60084 L
.3291 .59962 L
.33037 .59815 L
.33174 .59622 L
.33417 .59172 L
.33646 .58593 L
.33776 .58174 L
.33918 .57607 L
.3399 .57257 L
.34066 .56802 L
.34132 .56215 L
.34204 .55653 L
.34276 .55231 L
.34342 .54895 L
.34472 .54321 L
.34716 .53435 L
.35166 .52157 L
.35661 .51056 L
.36661 .49367 L
.37643 .48123 L
.38541 .4721 L
.4056 .45649 L
Mistroke
.42399 .44606 L
.44404 .43727 L
.46469 .43015 L
.48338 .4249 L
.52126 .4167 L
.56004 .41057 L
.59735 .40609 L
.63704 .40238 L
.67526 .39954 L
.71202 .3973 L
.75115 .39534 L
.78881 .39375 L
.82884 .39233 L
.86741 .39117 L
.90451 .39021 L
.94399 .38931 L
.97619 .38867 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool003Woo`8000Coo`03003ooooo00;oo`<002;oo`@0027oo`@000?oo`03003ooooo00;oo`<002?o
o`@0027oo`@000?oo`03003ooooo00;oo`<002?oo`8000Ooo`00>?oo00@00?ooool000_oo`03003o
oooo027oo`03003ooooo02;oo`03003ooooo00[oo`03003ooooo023oo`03003ooooo02;oo`03003o
oooo00coo`03003ooooo01ooo`04003ooooo0006ool003Soo`04003ooooo000;ool00`00ooooo`0Q
ool00`00ooooo`0Rool00`00ooooo`0:ool00`00ooooo`0Qool00`00ooooo`0Rool00`00ooooo`0;
ool00`00ooooo`0Rool00`00ooooo`04ool003Soo`04003ooooo0008ool3000Tool00`00ooooo`0R
ool00`00ooooo`07ool3000Uool00`00ooooo`0Rool00`00ooooo`07ool3000Sool20007ool003So
o`04003ooooo0008ool00`00ooooo`0Tool00`00ooooo`0Rool00`00ooooo`07ool00`00ooooo`0V
ool00`00ooooo`0Rool00`00ooooo`06ool00`00ooooo`0Uool00`00ooooo`04ool003Soo`04003o
oooo0008ool00`00ooooo`0Rool3000Rool30009ool00`00ooooo`0Sool01000ooooo`008Ooo00@0
0?ooool000Soo`03003ooooo02;oo`04003ooooo0006ool001Goo`03003ooooo027oo`8000Woo`@0
02?oo`03003ooooo02;oo`03003ooooo00Ooo`@002?oo`8002?oo`8000Woo`@002;oo`8000Ooo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
3_ooo`004`00000Eool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`05ool001Goo`03003ooooo02Soo`03003ooooo02Soo`03003o
oooo02Soo`03003ooooo02Woo`03003ooooo02Soo`03003ooooo02Soo`03003ooooo00Goo`005Ooo
00<00?ooool01_oo00<00?ooool0oooo000Eool00`00ooooo`07ool00`00ooooo`3nool001Goo`03
003ooooo00Soo`03003ooooo0?goo`005Ooo0P002_oo00<00?ooool0o?oo000Eool00`00ooooo`0:
ool00`00ooooo`3kool001Goo`03003ooooo00_oo`03003ooooo0?[oo`005Ooo00<00?ooool03?oo
00<00?ooool0nOoo000Eool00`00ooooo`0=ool00`00ooooo`3hool001Goo`03003ooooo00goo`03
003ooooo0?Soo`005Ooo0P003ooo00<00?ooool0mooo000Eool00`00ooooo`0?ool00`00ooooo`3f
ool001Goo`03003ooooo013oo`03003ooooo0?Goo`005Ooo00<00?ooool04?oo00<00?ooool0mOoo
000Eool00`00ooooo`0Aool00`00ooooo`3dool001Goo`03003ooooo017oo`03003ooooo0?Coo`00
5Ooo0P004ooo00<00?ooool0looo000Eool00`00ooooo`0Bool00`00ooooo`3cool001Goo`03003o
oooo01?oo`03003ooooo0?;oo`005Ooo00<00?ooool04ooo00<00?ooool0l_oo000Eool00`00oooo
o`0Dool00`00ooooo`3aool001Goo`03003ooooo01Coo`03003ooooo0?7oo`005Ooo0P005_oo00<0
0?ooool0l?oo000Eool00`00ooooo`0Fool00`00ooooo`3_ool001Goo`03003ooooo01Koo`03003o
oooo0>ooo`005Ooo00<00?ooool05ooo00<00?ooool0k_oo0002ool20004ool00`00ooooo`02ool4
0004ool00`00ooooo`0Gool00`00ooooo`3^ool00005ool00?ooool0000:ool00`00ooooo`03ool0
0`00ooooo`0Hool00`00ooooo`3]ool00005ool00?ooool0000:ool00`00ooooo`03ool3000Iool0
0`00ooooo`3/ool00005ool00?ooool0000:ool00`00ooooo`03ool00`00ooooo`0Iool00`00oooo
o`3/ool00005ool00?ooool0000:ool00`00ooooo`03ool00`00ooooo`0Jool00`00ooooo`3[ool0
0005ool00?ooool00008ool30005ool00`00ooooo`0Jool00`00ooooo`3[ool000;oo`8000_oo`03
003ooooo00?oo`03003ooooo01_oo`03003ooooo0>[oo`005Ooo00<00?ooool06ooo00<00?ooool0
j_oo000Eool2000Mool00`00ooooo`3Yool001Goo`03003ooooo01coo`03003ooooo0>Woo`005Ooo
00<00?ooool07Ooo00<00?ooool0j?oo000Eool00`00ooooo`0Mool00`00ooooo`3Xool001Goo`03
003ooooo01goo`03003ooooo0>Soo`005Ooo00<00?ooool07_oo00<00?ooool0iooo000Eool2000O
ool00`00ooooo`3Wool001Goo`03003ooooo01koo`03003ooooo0>Ooo`005Ooo00<00?ooool07ooo
00<00?ooool0i_oo000Eool00`00ooooo`0Oool00`00ooooo`3Vool001Goo`03003ooooo023oo`03
003ooooo0>Goo`005Ooo00<00?ooool08?oo00<00?ooool0iOoo000Eool2000Qool00`00ooooo`3U
ool001Goo`03003ooooo027oo`03003ooooo0>Coo`005Ooo00<00?ooool08Ooo00<00?ooool0i?oo
000Eool00`00ooooo`0Rool00`00ooooo`3Sool001Goo`03003ooooo02;oo`03003ooooo0>?oo`00
5Ooo00<00?ooool08_oo00<00?ooool0hooo000Eool2000Tool00`00ooooo`3Rool001Goo`03003o
oooo02?oo`03003ooooo0>;oo`005Ooo00<00?ooool08ooo00<00?ooool0h_oo000Eool00`00oooo
o`0Tool00`00ooooo`3Qool000;oo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo02Coo`03
003ooooo0>7oo`0000Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo02Goo`03003ooooo
0>3oo`0000Goo`00ooooo`0000Woo`03003ooooo00Coo`<002Goo`03003ooooo0>3oo`0000Goo`00
ooooo`0000[oo`03003ooooo00?oo`03003ooooo02Goo`03003ooooo0>3oo`0000Goo`00ooooo`00
00_oo`03003ooooo00;oo`03003ooooo02Koo`03003ooooo0=ooo`0000Goo`00ooooo`0000Soo`04
003ooooo0004ool00`00ooooo`0Vool00`00ooooo`3Oool000;oo`8000[oo`8000Goo`03003ooooo
02Koo`03003ooooo0=ooo`005Ooo00<00?ooool09ooo00<00?ooool0g_oo000Eool2000Xool00`00
ooooo`3Nool001Goo`03003ooooo02Ooo`03003ooooo0=koo`005Ooo00<00?ooool0:?oo00<00?oo
ool0gOoo000Eool00`00ooooo`0Xool00`00ooooo`3Mool001Goo`03003ooooo02Woo`03003ooooo
0=coo`005Ooo00<00?ooool0:Ooo00<00?ooool0g?oo000Eool2000Zool00`00ooooo`3Lool001Go
o`03003ooooo02[oo`03003ooooo0=_oo`005Ooo00<00?ooool0:_oo00<00?ooool0fooo000Eool0
0`00ooooo`0Zool00`00ooooo`3Kool001Goo`03003ooooo02_oo`03003ooooo0=[oo`005Ooo00<0
0?ooool0:ooo00<00?ooool0f_oo000Eool2000/ool00`00ooooo`3Jool001Goo`03003ooooo02co
o`03003ooooo0=Woo`005Ooo00<00?ooool0;?oo00<00?ooool0fOoo000Eool00`00ooooo`0/ool0
0`00ooooo`3Iool001Goo`03003ooooo02goo`03003ooooo0=Soo`005Ooo00<00?ooool0;Ooo00<0
0?ooool0f?oo000Eool2000_ool00`00ooooo`3Gool001Goo`03003ooooo02koo`03003ooooo0=Oo
o`005Ooo00<00?ooool0;_oo00<00?ooool0eooo000Eool00`00ooooo`0_ool00`00ooooo`3Fool0
00;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo02ooo`03003ooooo0=Koo`0000Goo`00
ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0_ool00`00ooooo`3Fool00005ool00?ooool0
000;ool00`00ooooo`02ool3000`ool00`00ooooo`3Eool00005ool00?ooool00009ool20005ool0
0`00ooooo`0`ool00`00ooooo`3Eool00005ool00?ooool0000;ool00`00ooooo`02ool00`00oooo
o`0`ool00`00ooooo`3Eool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0<Ooo
00<00?ooool0e?oo0002ool2000:ool20005ool00`00ooooo`0aool00`00ooooo`3Dool001Goo`03
003ooooo037oo`03003ooooo0=Coo`005Ooo0P00<ooo00<00?ooool0dooo000Eool00`00ooooo`0b
ool00`00ooooo`3Cool001Goo`03003ooooo03;oo`03003ooooo0=?oo`005Ooo00<00?ooool0<_oo
00<00?ooool0dooo000Eool00`00ooooo`0cool00`00ooooo`3Bool001Goo`03003ooooo03?oo`03
003ooooo0=;oo`005Ooo0P00=?oo00<00?ooool0/_oo6@001ooo000Eool00`00ooooo`0dool00`00
ooooo`2AoolP000Pool001Goo`03003ooooo03Coo`03003ooooo07cooaD0043oo`005Ooo00<00?oo
ool0=?oo00<00?ooool0L_oo2P00EOoo000Eool00`00ooooo`0eool00`00ooooo`1Wool:001Oool0
01Goo`03003ooooo03Goo`03003ooooo05coo`/006Woo`005Ooo0P00=_oo00<00?ooool0EOoo1`00
M?oo000Eool00`00ooooo`0fool00`00ooooo`1>ool6001kool001Goo`03003ooooo03Koo`03003o
oooo04Woo`D0087oo`005Ooo00<00?ooool0=_oo00<00?ooool0A?oo1@00Q_oo000Eool00`00oooo
o`0gool00`00ooooo`0nool5002;ool001Goo`8003Soo`03003ooooo03[oo`@0093oo`005Ooo00<0
0?ooool0=ooo00<00?ooool0=ooo0`00U?oo000Eool00`00ooooo`0gool00`00ooooo`0eool2002G
ool001Goo`03003ooooo03Soo`03003ooooo03;oo`8009Woo`000_oo0P001?oo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool0>?oo00<00?ooool0<?oo0P00Vooo00001Ooo003ooooo00002_oo
00<00?ooool00ooo00<00?ooool0>?oo00<00?ooool0;_oo0P00WOoo00001Ooo003ooooo00001ooo
1@001?oo0`00>Ooo00<00?ooool0:ooo0P00Wooo00001Ooo003ooooo00001ooo00@00?ooool000Go
o`03003ooooo03Woo`03003ooooo02Woo`800:7oo`0000Goo`00ooooo`0000Soo`03003oo`0000Go
o`03003ooooo03Woo`03003ooooo02Soo`03003ooooo0:7oo`0000Goo`00ooooo`0000Woo`8000Go
o`03003ooooo03[oo`03003ooooo02Goo`800:Coo`000_oo0P002ooo00<00?ooool00ooo00<00?oo
ool0>_oo00<00?ooool09?oo00<00?ooool0Y?oo000Eool00`00ooooo`0jool00`00ooooo`0Sool0
0`00ooooo`2Uool001Goo`8003coo`03003ooooo027oo`03003ooooo0:Koo`005Ooo00<00?ooool0
>ooo00<00?ooool08?oo00<00?ooool0Yooo000Eool00`00ooooo`0lool00`00ooooo`0Nool00`00
ooooo`2Xool001Goo`03003ooooo03coo`03003ooooo01goo`03003ooooo0:Woo`005Ooo00<00?oo
ool0??oo00<00?ooool07?oo00<00?ooool0Z_oo000Eool00`00ooooo`0mool00`00ooooo`0Jool0
0`00ooooo`2[ool001Goo`8003koo`03003ooooo01Woo`03003ooooo0:coo`005Ooo00<00?ooool0
?Ooo00<00?ooool06Ooo00<00?ooool0[?oo000Eool00`00ooooo`0nool00`00ooooo`0Gool00`00
ooooo`2]ool001Goo`03003ooooo03koo`03003ooooo01Ooo`03003ooooo0:goo`005Ooo00<00?oo
ool0?ooo00<00?ooool05Ooo00<00?ooool0[_oo000Eool00`00ooooo`0oool00`00ooooo`0Eool0
0`00ooooo`2^ool001Goo`80043oo`03003ooooo01Coo`03003ooooo0:ooo`005Ooo00<00?ooool0
@?oo00<00?ooool04ooo00<00?ooool0[ooo000Eool00`00ooooo`10ool00`00ooooo`0Cool00`00
ooooo`2_ool001Goo`03003ooooo047oo`03003ooooo01;oo`03003ooooo0:ooo`005Ooo00<00?oo
ool0@Ooo00<00?ooool04Ooo00<00?ooool0/?oo000Eool00`00ooooo`12ool00`00ooooo`0@ool0
0`00ooooo`2`ool001Goo`8004?oo`03003ooooo013oo`03003ooooo0;3oo`005Ooo00<00?ooool0
@ooo00<00?ooool03_oo00<00?ooool0/Ooo000Eool00`00ooooo`13ool00`00ooooo`0>ool00`00
ooooo`2aool001Goo`03003ooooo04Coo`03003ooooo00goo`03003ooooo0;7oo`000_oo0P001?oo
00<00?ooool00_oo0`001Ooo00<00?ooool0A?oo00<00?ooool03Ooo00<00?ooool0/Ooo00001Ooo
003ooooo00002ooo00<00?ooool00_oo00<00?ooool0AOoo00<00?ooool02ooo00<00?ooool0/_oo
00001Ooo003ooooo00002ooo00<00?ooool00_oo0`00AOoo00<00?ooool02ooo00<00?ooool0/_oo
00001Ooo003ooooo00002?oo0`001Ooo00<00?ooool0AOoo00<00?ooool02ooo00<00?ooool0/_oo
00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool0A_oo00<00?ooool02_oo00<00?oo
ool0/_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool0A_oo00<00?ooool02_oo
00<00?ooool0/_oo0002ool20009ool40004ool00`00ooooo`17ool00`00ooooo`09ool00`00oooo
o`2bool001Goo`03003ooooo04Ooo`03003ooooo00Soo`03003ooooo0;?oo`005Ooo0P00BOoo00<0
0?ooool01ooo00<00?ooool0/ooo000Eool00`00ooooo`18ool00`00ooooo`07ool00`00ooooo`2c
ool001Goo`03003ooooo04Woo`03003ooooo00Goo`03003ooooo0;Coo`005Ooo00<00?ooool0B_oo
00<00?ooool01?oo00<00?ooool0]?oo000Eool00`00ooooo`1;ool00`00ooooo`02ool00`00oooo
o`2eool001Goo`03003ooooo04coo`D00;Ooo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9
ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -0.0422281, \
0.0115777, 0.0033527}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[SXiA[z, Cos[10^\((\(-4\))\)], 0]], {z, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147151 1.09627 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .12434 -18 -4.5 ]
[.01131 .12434 0 4.5 ]
[.01131 .23397 -18 -4.5 ]
[.01131 .23397 0 4.5 ]
[.01131 .3436 -18 -4.5 ]
[.01131 .3436 0 4.5 ]
[.01131 .45322 -18 -4.5 ]
[.01131 .45322 0 4.5 ]
[.01131 .56285 -18 -4.5 ]
[.01131 .56285 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .12434 m
.03006 .12434 L
s
[(0.1)] .01131 .12434 1 0 Mshowa
.02381 .23397 m
.03006 .23397 L
s
[(0.2)] .01131 .23397 1 0 Mshowa
.02381 .3436 m
.03006 .3436 L
s
[(0.3)] .01131 .3436 1 0 Mshowa
.02381 .45322 m
.03006 .45322 L
s
[(0.4)] .01131 .45322 1 0 Mshowa
.02381 .56285 m
.03006 .56285 L
s
[(0.5)] .01131 .56285 1 0 Mshowa
.125 Mabswid
.02381 .03664 m
.02756 .03664 L
s
.02381 .05857 m
.02756 .05857 L
s
.02381 .08049 m
.02756 .08049 L
s
.02381 .10242 m
.02756 .10242 L
s
.02381 .14627 m
.02756 .14627 L
s
.02381 .16819 m
.02756 .16819 L
s
.02381 .19012 m
.02756 .19012 L
s
.02381 .21204 m
.02756 .21204 L
s
.02381 .25589 m
.02756 .25589 L
s
.02381 .27782 m
.02756 .27782 L
s
.02381 .29975 m
.02756 .29975 L
s
.02381 .32167 m
.02756 .32167 L
s
.02381 .36552 m
.02756 .36552 L
s
.02381 .38745 m
.02756 .38745 L
s
.02381 .40937 m
.02756 .40937 L
s
.02381 .4313 m
.02756 .4313 L
s
.02381 .47515 m
.02756 .47515 L
s
.02381 .49707 m
.02756 .49707 L
s
.02381 .519 m
.02756 .519 L
s
.02381 .54093 m
.02756 .54093 L
s
.02381 .58478 m
.02756 .58478 L
s
.02381 .6067 m
.02756 .6067 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .02564 m
.06423 .03239 L
.07374 .04161 L
.0929 .06582 L
.11399 .10076 L
.13363 .14055 L
.17133 .23358 L
.21139 .34923 L
.24999 .46451 L
.26766 .51346 L
.28712 .56011 L
.29636 .57807 L
.30146 .58639 L
.3063 .59305 L
.30892 .59607 L
.31178 .59884 L
.31438 .60084 L
.31679 .6022 L
.31808 .60272 L
.31929 .60307 L
.32038 .60326 L
.32155 .60332 L
.32217 .60329 L
.32284 .60321 L
.32354 .60307 L
.3242 .60289 L
.32537 .60242 L
.32663 .60171 L
.3278 .60084 L
.3291 .59962 L
.33037 .59815 L
.33174 .59622 L
.33417 .59172 L
.33646 .58593 L
.33776 .58174 L
.33918 .57607 L
.3399 .57257 L
.34066 .56802 L
.34132 .56215 L
.34204 .55653 L
.34276 .55231 L
.34342 .54895 L
.34472 .54321 L
.34716 .53435 L
.35166 .52157 L
.35661 .51056 L
.36661 .49367 L
.37643 .48123 L
.38541 .4721 L
.4056 .45649 L
Mistroke
.42399 .44606 L
.44404 .43727 L
.46469 .43015 L
.48338 .4249 L
.52126 .4167 L
.56004 .41057 L
.59735 .40609 L
.63704 .40238 L
.67526 .39954 L
.71202 .3973 L
.75115 .39534 L
.78881 .39375 L
.82884 .39233 L
.86741 .39117 L
.90451 .39021 L
.94399 .38931 L
.97619 .38867 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool003Woo`8000Coo`03003ooooo00;oo`<002;oo`@0027oo`@000?oo`03003ooooo00;oo`<002?o
o`@0027oo`@000?oo`03003ooooo00;oo`<002?oo`8000Ooo`00>?oo00@00?ooool000_oo`03003o
oooo027oo`03003ooooo02;oo`03003ooooo00[oo`03003ooooo023oo`03003ooooo02;oo`03003o
oooo00coo`03003ooooo01ooo`04003ooooo0006ool003Soo`04003ooooo000;ool00`00ooooo`0Q
ool00`00ooooo`0Rool00`00ooooo`0:ool00`00ooooo`0Qool00`00ooooo`0Rool00`00ooooo`0;
ool00`00ooooo`0Rool00`00ooooo`04ool003Soo`04003ooooo0008ool3000Tool00`00ooooo`0R
ool00`00ooooo`07ool3000Uool00`00ooooo`0Rool00`00ooooo`07ool3000Sool20007ool003So
o`04003ooooo0008ool00`00ooooo`0Tool00`00ooooo`0Rool00`00ooooo`07ool00`00ooooo`0V
ool00`00ooooo`0Rool00`00ooooo`06ool00`00ooooo`0Uool00`00ooooo`04ool003Soo`04003o
oooo0008ool00`00ooooo`0Rool3000Rool30009ool00`00ooooo`0Sool01000ooooo`008Ooo00@0
0?ooool000Soo`03003ooooo02;oo`04003ooooo0006ool001Goo`03003ooooo027oo`8000Woo`@0
02?oo`03003ooooo02;oo`03003ooooo00Ooo`@002?oo`8002?oo`8000Woo`@002;oo`8000Ooo`00
5Ooo00<00?ooool0oooo2Ooo000Eool00`00ooooo`3oool9ool001Goo`03003ooooo0?ooo`Woo`00
3_ooo`004`00000Eool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`05ool001Goo`03003ooooo02Soo`03003ooooo02Soo`03003o
oooo02Soo`03003ooooo02Woo`03003ooooo02Soo`03003ooooo02Soo`03003ooooo00Goo`005Ooo
00<00?ooool01_oo00<00?ooool0oooo000Eool00`00ooooo`07ool00`00ooooo`3nool001Goo`03
003ooooo00Soo`03003ooooo0?goo`005Ooo0P002_oo00<00?ooool0o?oo000Eool00`00ooooo`0:
ool00`00ooooo`3kool001Goo`03003ooooo00_oo`03003ooooo0?[oo`005Ooo00<00?ooool03?oo
00<00?ooool0nOoo000Eool00`00ooooo`0=ool00`00ooooo`3hool001Goo`03003ooooo00goo`03
003ooooo0?Soo`005Ooo0P003ooo00<00?ooool0mooo000Eool00`00ooooo`0?ool00`00ooooo`3f
ool001Goo`03003ooooo013oo`03003ooooo0?Goo`005Ooo00<00?ooool04?oo00<00?ooool0mOoo
000Eool00`00ooooo`0Aool00`00ooooo`3dool001Goo`03003ooooo017oo`03003ooooo0?Coo`00
5Ooo0P004ooo00<00?ooool0looo000Eool00`00ooooo`0Bool00`00ooooo`3cool001Goo`03003o
oooo01?oo`03003ooooo0?;oo`005Ooo00<00?ooool04ooo00<00?ooool0l_oo000Eool00`00oooo
o`0Dool00`00ooooo`3aool001Goo`03003ooooo01Coo`03003ooooo0?7oo`005Ooo0P005_oo00<0
0?ooool0l?oo000Eool00`00ooooo`0Fool00`00ooooo`3_ool001Goo`03003ooooo01Koo`03003o
oooo0>ooo`005Ooo00<00?ooool05ooo00<00?ooool0k_oo0002ool20004ool00`00ooooo`02ool4
0004ool00`00ooooo`0Gool00`00ooooo`3^ool00005ool00?ooool0000:ool00`00ooooo`03ool0
0`00ooooo`0Hool00`00ooooo`3]ool00005ool00?ooool0000:ool00`00ooooo`03ool3000Iool0
0`00ooooo`3/ool00005ool00?ooool0000:ool00`00ooooo`03ool00`00ooooo`0Iool00`00oooo
o`3/ool00005ool00?ooool0000:ool00`00ooooo`03ool00`00ooooo`0Jool00`00ooooo`3[ool0
0005ool00?ooool00008ool30005ool00`00ooooo`0Jool00`00ooooo`3[ool000;oo`8000_oo`03
003ooooo00?oo`03003ooooo01_oo`03003ooooo0>[oo`005Ooo00<00?ooool06ooo00<00?ooool0
j_oo000Eool2000Mool00`00ooooo`3Yool001Goo`03003ooooo01coo`03003ooooo0>Woo`005Ooo
00<00?ooool07Ooo00<00?ooool0j?oo000Eool00`00ooooo`0Mool00`00ooooo`3Xool001Goo`03
003ooooo01goo`03003ooooo0>Soo`005Ooo00<00?ooool07_oo00<00?ooool0iooo000Eool2000O
ool00`00ooooo`3Wool001Goo`03003ooooo01koo`03003ooooo0>Ooo`005Ooo00<00?ooool07ooo
00<00?ooool0i_oo000Eool00`00ooooo`0Oool00`00ooooo`3Vool001Goo`03003ooooo023oo`03
003ooooo0>Goo`005Ooo00<00?ooool08?oo00<00?ooool0iOoo000Eool2000Qool00`00ooooo`3U
ool001Goo`03003ooooo027oo`03003ooooo0>Coo`005Ooo00<00?ooool08Ooo00<00?ooool0i?oo
000Eool00`00ooooo`0Rool00`00ooooo`3Sool001Goo`03003ooooo02;oo`03003ooooo0>?oo`00
5Ooo00<00?ooool08_oo00<00?ooool0hooo000Eool2000Tool00`00ooooo`3Rool001Goo`03003o
oooo02?oo`03003ooooo0>;oo`005Ooo00<00?ooool08ooo00<00?ooool0h_oo000Eool00`00oooo
o`0Tool00`00ooooo`3Qool000;oo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo02Coo`03
003ooooo0>7oo`0000Goo`00ooooo`0000Soo`03003ooooo00Goo`03003ooooo02Goo`03003ooooo
0>3oo`0000Goo`00ooooo`0000Woo`03003ooooo00Coo`<002Goo`03003ooooo0>3oo`0000Goo`00
ooooo`0000[oo`03003ooooo00?oo`03003ooooo02Goo`03003ooooo0>3oo`0000Goo`00ooooo`00
00_oo`03003ooooo00;oo`03003ooooo02Koo`03003ooooo0=ooo`0000Goo`00ooooo`0000Soo`04
003ooooo0004ool00`00ooooo`0Vool00`00ooooo`3Oool000;oo`8000[oo`8000Goo`03003ooooo
02Koo`03003ooooo0=ooo`005Ooo00<00?ooool09ooo00<00?ooool0g_oo000Eool2000Xool00`00
ooooo`3Nool001Goo`03003ooooo02Ooo`03003ooooo0=koo`005Ooo00<00?ooool0:?oo00<00?oo
ool0gOoo000Eool00`00ooooo`0Xool00`00ooooo`3Mool001Goo`03003ooooo02Woo`03003ooooo
0=coo`005Ooo00<00?ooool0:Ooo00<00?ooool0g?oo000Eool2000Zool00`00ooooo`3Lool001Go
o`03003ooooo02[oo`03003ooooo0=_oo`005Ooo00<00?ooool0:_oo00<00?ooool0fooo000Eool0
0`00ooooo`0Zool00`00ooooo`3Kool001Goo`03003ooooo02_oo`03003ooooo0=[oo`005Ooo00<0
0?ooool0:ooo00<00?ooool0f_oo000Eool2000/ool00`00ooooo`3Jool001Goo`03003ooooo02co
o`03003ooooo0=Woo`005Ooo00<00?ooool0;?oo00<00?ooool0fOoo000Eool00`00ooooo`0/ool0
0`00ooooo`3Iool001Goo`03003ooooo02goo`03003ooooo0=Soo`005Ooo00<00?ooool0;Ooo00<0
0?ooool0f?oo000Eool2000_ool00`00ooooo`3Gool001Goo`03003ooooo02koo`03003ooooo0=Oo
o`005Ooo00<00?ooool0;_oo00<00?ooool0eooo000Eool00`00ooooo`0_ool00`00ooooo`3Fool0
00;oo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo02ooo`03003ooooo0=Koo`0000Goo`00
ooooo`0000Soo`04003ooooo0004ool00`00ooooo`0_ool00`00ooooo`3Fool00005ool00?ooool0
000;ool00`00ooooo`02ool3000`ool00`00ooooo`3Eool00005ool00?ooool00009ool20005ool0
0`00ooooo`0`ool00`00ooooo`3Eool00005ool00?ooool0000;ool00`00ooooo`02ool00`00oooo
o`0`ool00`00ooooo`3Eool00005ool00?ooool00008ool01000ooooo`001?oo00<00?ooool0<Ooo
00<00?ooool0e?oo0002ool2000:ool20005ool00`00ooooo`0aool00`00ooooo`3Dool001Goo`03
003ooooo037oo`03003ooooo0=Coo`005Ooo0P00<ooo00<00?ooool0dooo000Eool00`00ooooo`0b
ool00`00ooooo`3Cool001Goo`03003ooooo03;oo`03003ooooo0=?oo`005Ooo00<00?ooool0<_oo
00<00?ooool0dooo000Eool00`00ooooo`0cool00`00ooooo`3Bool001Goo`03003ooooo03?oo`03
003ooooo0=;oo`005Ooo0P00=?oo00<00?ooool0/_oo6@001ooo000Eool00`00ooooo`0dool00`00
ooooo`2AoolP000Pool001Goo`03003ooooo03Coo`03003ooooo07cooaD0043oo`005Ooo00<00?oo
ool0=?oo00<00?ooool0L_oo2P00EOoo000Eool00`00ooooo`0eool00`00ooooo`1Wool:001Oool0
01Goo`03003ooooo03Goo`03003ooooo05coo`/006Woo`005Ooo0P00=_oo00<00?ooool0EOoo1`00
M?oo000Eool00`00ooooo`0fool00`00ooooo`1>ool6001kool001Goo`03003ooooo03Koo`03003o
oooo04Woo`D0087oo`005Ooo00<00?ooool0=_oo00<00?ooool0A?oo1@00Q_oo000Eool00`00oooo
o`0gool00`00ooooo`0nool5002;ool001Goo`8003Soo`03003ooooo03[oo`@0093oo`005Ooo00<0
0?ooool0=ooo00<00?ooool0=ooo0`00U?oo000Eool00`00ooooo`0gool00`00ooooo`0eool2002G
ool001Goo`03003ooooo03Soo`03003ooooo03;oo`8009Woo`000_oo0P001?oo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool0>?oo00<00?ooool0<?oo0P00Vooo00001Ooo003ooooo00002_oo
00<00?ooool00ooo00<00?ooool0>?oo00<00?ooool0;_oo0P00WOoo00001Ooo003ooooo00001ooo
1@001?oo0`00>Ooo00<00?ooool0:ooo0P00Wooo00001Ooo003ooooo00001ooo00@00?ooool000Go
o`03003ooooo03Woo`03003ooooo02Woo`800:7oo`0000Goo`00ooooo`0000Soo`03003oo`0000Go
o`03003ooooo03Woo`03003ooooo02Soo`03003ooooo0:7oo`0000Goo`00ooooo`0000Woo`8000Go
o`03003ooooo03[oo`03003ooooo02Goo`800:Coo`000_oo0P002ooo00<00?ooool00ooo00<00?oo
ool0>_oo00<00?ooool09?oo00<00?ooool0Y?oo000Eool00`00ooooo`0jool00`00ooooo`0Sool0
0`00ooooo`2Uool001Goo`8003coo`03003ooooo027oo`03003ooooo0:Koo`005Ooo00<00?ooool0
>ooo00<00?ooool08?oo00<00?ooool0Yooo000Eool00`00ooooo`0lool00`00ooooo`0Nool00`00
ooooo`2Xool001Goo`03003ooooo03coo`03003ooooo01goo`03003ooooo0:Woo`005Ooo00<00?oo
ool0??oo00<00?ooool07?oo00<00?ooool0Z_oo000Eool00`00ooooo`0mool00`00ooooo`0Jool0
0`00ooooo`2[ool001Goo`8003koo`03003ooooo01Woo`03003ooooo0:coo`005Ooo00<00?ooool0
?Ooo00<00?ooool06Ooo00<00?ooool0[?oo000Eool00`00ooooo`0nool00`00ooooo`0Gool00`00
ooooo`2]ool001Goo`03003ooooo03koo`03003ooooo01Ooo`03003ooooo0:goo`005Ooo00<00?oo
ool0?ooo00<00?ooool05Ooo00<00?ooool0[_oo000Eool00`00ooooo`0oool00`00ooooo`0Eool0
0`00ooooo`2^ool001Goo`80043oo`03003ooooo01Coo`03003ooooo0:ooo`005Ooo00<00?ooool0
@?oo00<00?ooool04ooo00<00?ooool0[ooo000Eool00`00ooooo`10ool00`00ooooo`0Cool00`00
ooooo`2_ool001Goo`03003ooooo047oo`03003ooooo01;oo`03003ooooo0:ooo`005Ooo00<00?oo
ool0@Ooo00<00?ooool04Ooo00<00?ooool0/?oo000Eool00`00ooooo`12ool00`00ooooo`0@ool0
0`00ooooo`2`ool001Goo`8004?oo`03003ooooo013oo`03003ooooo0;3oo`005Ooo00<00?ooool0
@ooo00<00?ooool03_oo00<00?ooool0/Ooo000Eool00`00ooooo`13ool00`00ooooo`0>ool00`00
ooooo`2aool001Goo`03003ooooo04Coo`03003ooooo00goo`03003ooooo0;7oo`000_oo0P001?oo
00<00?ooool00_oo0`001Ooo00<00?ooool0A?oo00<00?ooool03Ooo00<00?ooool0/Ooo00001Ooo
003ooooo00002ooo00<00?ooool00_oo00<00?ooool0AOoo00<00?ooool02ooo00<00?ooool0/_oo
00001Ooo003ooooo00002ooo00<00?ooool00_oo0`00AOoo00<00?ooool02ooo00<00?ooool0/_oo
00001Ooo003ooooo00002?oo0`001Ooo00<00?ooool0AOoo00<00?ooool02ooo00<00?ooool0/_oo
00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool0A_oo00<00?ooool02_oo00<00?oo
ool0/_oo00001Ooo003ooooo00002?oo00<00?ooool01Ooo00<00?ooool0A_oo00<00?ooool02_oo
00<00?ooool0/_oo0002ool20009ool40004ool00`00ooooo`17ool00`00ooooo`09ool00`00oooo
o`2bool001Goo`03003ooooo04Ooo`03003ooooo00Soo`03003ooooo0;?oo`005Ooo0P00BOoo00<0
0?ooool01ooo00<00?ooool0/ooo000Eool00`00ooooo`18ool00`00ooooo`07ool00`00ooooo`2c
ool001Goo`03003ooooo04Woo`03003ooooo00Goo`03003ooooo0;Coo`005Ooo00<00?ooool0B_oo
00<00?ooool01?oo00<00?ooool0]?oo000Eool00`00ooooo`1;ool00`00ooooo`02ool00`00oooo
o`2eool001Goo`03003ooooo04coo`D00;Ooo`005Ooo0P00oooo2_oo000Eool00`00ooooo`3oool9
ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?ooool0oooo2Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.247788, -0.0422281, \
0.0115777, 0.0033527}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[BoxData[
    \(looks\ o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Im[Normal[%770[\([3, 1]\)]]], {z, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.603319 1.94709 [
[.18254 .59082 -9 -9 ]
[.18254 .59082 9 0 ]
[.34127 .59082 -3 -9 ]
[.34127 .59082 3 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.81746 .59082 -9 -9 ]
[.81746 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.01131 .01919 -24 -4.5 ]
[.01131 .01919 0 4.5 ]
[.01131 .11655 -30 -4.5 ]
[.01131 .11655 0 4.5 ]
[.01131 .2139 -24 -4.5 ]
[.01131 .2139 0 4.5 ]
[.01131 .31126 -30 -4.5 ]
[.01131 .31126 0 4.5 ]
[.01131 .40861 -24 -4.5 ]
[.01131 .40861 0 4.5 ]
[.01131 .50596 -30 -4.5 ]
[.01131 .50596 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .60332 m
.18254 .60957 L
s
[(0.5)] .18254 .59082 0 1 Mshowa
.34127 .60332 m
.34127 .60957 L
s
[(1)] .34127 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(1.5)] .5 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(2)] .65873 .59082 0 1 Mshowa
.81746 .60332 m
.81746 .60957 L
s
[(2.5)] .81746 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(3)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.15079 .60332 m
.15079 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.27778 .60332 m
.27778 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.53175 .60332 m
.53175 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.9127 .60332 m
.9127 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .01919 m
.03006 .01919 L
s
[(-0.3)] .01131 .01919 1 0 Mshowa
.02381 .11655 m
.03006 .11655 L
s
[(-0.25)] .01131 .11655 1 0 Mshowa
.02381 .2139 m
.03006 .2139 L
s
[(-0.2)] .01131 .2139 1 0 Mshowa
.02381 .31126 m
.03006 .31126 L
s
[(-0.15)] .01131 .31126 1 0 Mshowa
.02381 .40861 m
.03006 .40861 L
s
[(-0.1)] .01131 .40861 1 0 Mshowa
.02381 .50596 m
.03006 .50596 L
s
[(-0.05)] .01131 .50596 1 0 Mshowa
.125 Mabswid
.02381 .03866 m
.02756 .03866 L
s
.02381 .05813 m
.02756 .05813 L
s
.02381 .0776 m
.02756 .0776 L
s
.02381 .09708 m
.02756 .09708 L
s
.02381 .13602 m
.02756 .13602 L
s
.02381 .15549 m
.02756 .15549 L
s
.02381 .17496 m
.02756 .17496 L
s
.02381 .19443 m
.02756 .19443 L
s
.02381 .23337 m
.02756 .23337 L
s
.02381 .25284 m
.02756 .25284 L
s
.02381 .27231 m
.02756 .27231 L
s
.02381 .29178 m
.02756 .29178 L
s
.02381 .33073 m
.02756 .33073 L
s
.02381 .3502 m
.02756 .3502 L
s
.02381 .36967 m
.02756 .36967 L
s
.02381 .38914 m
.02756 .38914 L
s
.02381 .42808 m
.02756 .42808 L
s
.02381 .44755 m
.02756 .44755 L
s
.02381 .46702 m
.02756 .46702 L
s
.02381 .48649 m
.02756 .48649 L
s
.02381 .52544 m
.02756 .52544 L
s
.02381 .54491 m
.02756 .54491 L
s
.02381 .56438 m
.02756 .56438 L
s
.02381 .58385 m
.02756 .58385 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .45192 m
.0929 .28625 L
.11234 .21002 L
.13363 .1376 L
.15366 .08246 L
.17189 .04521 L
.18161 .03099 L
.18607 .02588 L
.19081 .02148 L
.19321 .01965 L
.19585 .01797 L
.19828 .01672 L
.20051 .01584 L
.20185 .01543 L
.20313 .01512 L
.20378 .015 L
.20448 .01489 L
.20524 .0148 L
.20595 .01475 L
.20661 .01472 L
.20733 .01472 L
.208 .01474 L
.20862 .01478 L
.2098 .01491 L
.21104 .01513 L
.21222 .01541 L
.21351 .01581 L
.21584 .01675 L
.21811 .01796 L
.22021 .01934 L
.22497 .02338 L
.23003 .02911 L
.23845 .04203 L
.24775 .06137 L
.25751 .08763 L
.26645 .11731 L
.28661 .20493 L
.30491 .31088 L
.32492 .45774 L
.33467 .54169 L
.33699 .56296 L
.33946 .58601 L
.34047 .59566 L
.34156 .60332 L
.34276 .60332 L
.34387 .60332 L
.34897 .60332 L
.35448 .60332 L
.36413 .60332 L
.40403 .60332 L
Mistroke
.44246 .60332 L
.47943 .60332 L
.51877 .60332 L
.55665 .60332 L
.5969 .60332 L
.63568 .60332 L
.67299 .60332 L
.71268 .60332 L
.7509 .60332 L
.78766 .60332 L
.82679 .60332 L
.86445 .60332 L
.90448 .60332 L
.94305 .60332 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`03003o
oooo0?coo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool000koo`8000Coo`03003ooooo
00?oo`8000Goo`03003ooooo0?coo`003Ooo00@00?ooool000Soo`04003ooooo0004ool00`00oooo
o`0Yool7003<ool000goo`04003ooooo000;ool00`00ooooo`02ool2000Yool20006ool00`00oooo
o`39ool000Ooo`@000;oo`04003ooooo0009ool20005ool00`00ooooo`0Wool00`00ooooo`07ool0
0`00ooooo`38ool000goo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0Wool00`00oooo
o`08ool00`00ooooo`37ool000goo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool09_oo
00<00?ooool02_oo00<00?ooool0a_oo000>ool2000:ool20005ool00`00ooooo`0Uool00`00oooo
o`0;ool00`00ooooo`36ool0027oo`8002Goo`03003ooooo00goo`03003ooooo0<Goo`008Ooo00<0
0?ooool08ooo00<00?ooool03ooo00<00?ooool0a?oo000Qool00`00ooooo`0Sool00`00ooooo`0?
ool00`00ooooo`34ool0027oo`03003ooooo02;oo`03003ooooo017oo`03003ooooo0<?oo`008Ooo
00<00?ooool08_oo00<00?ooool04Ooo00<00?ooool0`ooo000Qool2000Rool00`00ooooo`0Cool0
0`00ooooo`32ool0027oo`03003ooooo027oo`03003ooooo01?oo`03003ooooo0<;oo`008Ooo00<0
0?ooool08Ooo00<00?ooool04ooo00<00?ooool0`_oo000Qool00`00ooooo`0Pool00`00ooooo`0E
ool00`00ooooo`31ool0027oo`03003ooooo023oo`03003ooooo01Goo`03003ooooo0<7oo`008Ooo
0P008?oo00<00?ooool05_oo00<00?ooool0`Ooo000Qool00`00ooooo`0Oool00`00ooooo`0Gool0
0`00ooooo`30ool0027oo`03003ooooo01ooo`03003ooooo01Ooo`03003ooooo0<3oo`008Ooo00<0
0?ooool07_oo00<00?ooool06?oo00<00?ooool0`?oo000Qool00`00ooooo`0Nool00`00ooooo`0I
ool00`00ooooo`2oool0027oo`8001koo`03003ooooo01[oo`03003ooooo0;ooo`008Ooo00<00?oo
ool07Ooo00<00?ooool06_oo00<00?ooool0_ooo000Qool00`00ooooo`0Lool00`00ooooo`0Lool0
0`00ooooo`2nool000Soo`8000Coo`03003ooooo00;oo`@000;oo`<000Goo`03003ooooo01coo`03
003ooooo01coo`03003ooooo0;koo`001ooo00@00?ooool000Soo`03003ooooo00Koo`03003ooooo
00;oo`03003ooooo01coo`03003ooooo01goo`03003ooooo0;goo`001ooo00@00?ooool000Woo`03
003ooooo00Goo`03003ooooo00;oo`8001coo`03003ooooo01koo`03003ooooo0;goo`0000?oo`00
00000P000_oo00@00?ooool000[oo`04003oooooool30005ool00`00ooooo`0Kool00`00ooooo`0N
ool00`00ooooo`2mool000Ooo`04003ooooo000;ool01000ooooo`001ooo00<00?ooool06_oo00<0
0?ooool07ooo00<00?ooool0_Ooo0007ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo
00Goo`03003ooooo01[oo`03003ooooo023oo`03003ooooo0;coo`002?oo0P002_oo0P000ooo1000
1?oo00<00?ooool06Ooo00<00?ooool08Ooo00<00?ooool0_?oo000Qool2000Jool00`00ooooo`0Q
ool00`00ooooo`2lool0027oo`03003ooooo01Woo`03003ooooo027oo`03003ooooo0;coo`008Ooo
00<00?ooool06?oo00<00?ooool08ooo00<00?ooool0^ooo000Qool00`00ooooo`0Hool00`00oooo
o`0Sool00`00ooooo`2kool0027oo`03003ooooo01Soo`03003ooooo02?oo`03003ooooo0;_oo`00
8Ooo0P006Ooo00<00?ooool08ooo00<00?ooool0^ooo000Qool00`00ooooo`0Gool00`00ooooo`0T
ool00`00ooooo`2kool0027oo`03003ooooo01Ooo`03003ooooo02Goo`03003ooooo0;[oo`008Ooo
00<00?ooool05ooo00<00?ooool09Ooo00<00?ooool0^_oo000Qool00`00ooooo`0Gool00`00oooo
o`0Uool00`00ooooo`2jool0027oo`8001Ooo`03003ooooo02Koo`03003ooooo0;[oo`008Ooo00<0
0?ooool05_oo00<00?ooool09_oo00<00?ooool0^_oo000Qool00`00ooooo`0Fool00`00ooooo`0W
ool00`00ooooo`2iool0027oo`03003ooooo01Koo`03003ooooo02Ooo`03003ooooo0;Woo`008Ooo
00<00?ooool05Ooo00<00?ooool0:?oo00<00?ooool0^Ooo000Qool2000Fool00`00ooooo`0Xool0
0`00ooooo`2iool0027oo`03003ooooo01Goo`03003ooooo02Woo`03003ooooo0;Soo`008Ooo00<0
0?ooool05Ooo00<00?ooool0:Ooo00<00?ooool0^?oo000>ool20004ool00`00ooooo`02ool40004
ool00`00ooooo`0Dool00`00ooooo`0Zool00`00ooooo`2hool000goo`04003ooooo0008ool00`00
ooooo`05ool00`00ooooo`0Dool00`00ooooo`0Zool00`00ooooo`2hool000goo`04003ooooo0009
ool00`00ooooo`04ool2000Eool00`00ooooo`0Zool00`00ooooo`2hool000Ooo`@000;oo`04003o
oooo000:ool00`00ooooo`03ool00`00ooooo`0Dool00`00ooooo`0[ool00`00ooooo`2gool000go
o`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0Cool00`00ooooo`0/ool00`00ooooo`2g
ool000goo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool04ooo00<00?ooool0;?oo00<0
0?ooool0]ooo000>ool2000:ool20005ool00`00ooooo`0Cool00`00ooooo`0/ool00`00ooooo`2g
ool0027oo`8001Coo`03003ooooo02coo`03003ooooo0;Ooo`008Ooo00<00?ooool04_oo00<00?oo
ool0;Ooo00<00?ooool0]ooo000Qool00`00ooooo`0Bool00`00ooooo`0^ool00`00ooooo`2fool0
027oo`03003ooooo01;oo`03003ooooo02koo`03003ooooo0;Koo`008Ooo00<00?ooool04_oo00<0
0?ooool0;_oo00<00?ooool0]_oo000Qool2000Bool00`00ooooo`0_ool00`00ooooo`2fool0027o
o`03003ooooo017oo`03003ooooo02ooo`03003ooooo0;Koo`008Ooo00<00?ooool04Ooo00<00?oo
ool0;ooo00<00?ooool0]_oo000Qool00`00ooooo`0Aool00`00ooooo`0`ool00`00ooooo`2eool0
027oo`03003ooooo013oo`03003ooooo037oo`03003ooooo0;Goo`008Ooo0P004Ooo00<00?ooool0
<Ooo00<00?ooool0]Ooo000Qool00`00ooooo`0@ool00`00ooooo`0aool00`00ooooo`2eool0027o
o`03003ooooo013oo`03003ooooo037oo`03003ooooo0;Goo`008Ooo00<00?ooool03ooo00<00?oo
ool0<ooo00<00?ooool0]?oo000Qool00`00ooooo`0?ool00`00ooooo`0cool00`00ooooo`2dool0
027oo`03003ooooo00ooo`03003ooooo03?oo`03003ooooo0;Coo`008Ooo0P004?oo00<00?ooool0
<ooo00<00?ooool0]?oo000Qool00`00ooooo`0>ool00`00ooooo`0dool00`00ooooo`2dool0027o
o`03003ooooo00koo`03003ooooo03Coo`03003ooooo0;Coo`002?oo0P001?oo00<00?ooool00_oo
10000_oo0`001Ooo00<00?ooool03_oo00<00?ooool0=Ooo00<00?ooool0/ooo0007ool01000oooo
o`002_oo00<00?ooool01?oo00<00?ooool00_oo00<00?ooool03_oo00<00?ooool0=Ooo00<00?oo
ool0/ooo0007ool01000ooooo`002_oo00<00?ooool01?oo00<00?ooool00_oo0P003_oo00<00?oo
ool0=_oo00<00?ooool0/ooo00000ooo000000020002ool01000ooooo`002_oo00@00?ooooooo`<0
00Goo`03003ooooo00goo`03003ooooo03Koo`03003ooooo0;?oo`001ooo00@00?ooool000[oo`05
003oooooool00007ool00`00ooooo`0=ool00`00ooooo`0fool00`00ooooo`2cool000Ooo`04003o
oooo0008ool30003ool00`00ooooo`05ool00`00ooooo`0=ool00`00ooooo`0fool00`00ooooo`2c
ool000Soo`8000_oo`04003oooooool40004ool00`00ooooo`0<ool00`00ooooo`0hool00`00oooo
o`2bool0027oo`8000goo`03003ooooo03Soo`03003ooooo0;;oo`008Ooo00<00?ooool03?oo00<0
0?ooool0>?oo00<00?ooool0/_oo000Qool00`00ooooo`0<ool00`00ooooo`0hool00`00ooooo`2b
ool0027oo`03003ooooo00coo`03003ooooo03Soo`03003ooooo0;;oo`008Ooo00<00?ooool02ooo
00<00?ooool0>Ooo00<00?ooool0/_oo000Qool2000<ool00`00ooooo`0iool00`00ooooo`2bool0
027oo`03003ooooo00_oo`03003ooooo03Woo`03003ooooo0;;oo`008Ooo00<00?ooool02ooo00<0
0?ooool0>_oo00<00?ooool0/Ooo000Qool00`00ooooo`0:ool00`00ooooo`0kool00`00ooooo`2a
ool0027oo`03003ooooo00[oo`03003ooooo03_oo`03003ooooo0;7oo`008Ooo0P002ooo00<00?oo
ool0>ooo00<00?ooool0/Ooo000Qool00`00ooooo`0:ool00`00ooooo`0kool00`00ooooo`2aool0
027oo`03003ooooo00Woo`03003ooooo03coo`03003ooooo0;7oo`008Ooo00<00?ooool02Ooo00<0
0?ooool0??oo00<00?ooool0/Ooo000Qool00`00ooooo`09ool00`00ooooo`0lool00`00ooooo`2a
ool0027oo`8000[oo`03003ooooo03goo`03003ooooo0;3oo`008Ooo00<00?ooool02?oo00<00?oo
ool0?_oo00<00?ooool0/?oo000Qool00`00ooooo`08ool00`00ooooo`0nool00`00ooooo`2`ool0
00koo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo00Soo`03003ooooo03koo`03003ooooo
0;3oo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo00Soo`03003ooooo03koo`03
003ooooo0;3oo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`8000Woo`03003ooooo03koo`03
003ooooo0;3oo`001ooo10000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo00Ooo`03
003ooooo03ooo`03003ooooo0;3oo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo
00Ooo`03003ooooo043oo`03003ooooo0:ooo`003Ooo00@00?ooool000Soo`<000Goo`03003ooooo
00Ooo`03003ooooo043oo`03003ooooo0:ooo`003_oo0P002ooo00<00?ooool00ooo00<00?ooool0
1ooo00<00?ooool0@?oo00<00?ooool0[ooo000Qool20007ool00`00ooooo`11ool00`00ooooo`2_
ool0027oo`03003ooooo00Koo`03003ooooo047oo`03003ooooo0:ooo`008Ooo00<00?ooool01_oo
00<00?ooool0@Ooo00<00?ooool0[ooo000Qool00`00ooooo`06ool00`00ooooo`11ool00`00oooo
o`2_ool0027oo`03003ooooo00Goo`03003ooooo04;oo`03003ooooo0:ooo`008Ooo0P001_oo00<0
0?ooool0@ooo00<00?ooool0[_oo000Qool00`00ooooo`05ool00`00ooooo`13ool00`00ooooo`2^
ool0027oo`03003ooooo04_oo`03003ooooo0:koo`008Ooo00<00?ooool0Booo00<00?ooool0[_oo
000Qool00`00ooooo`1;ool00`00ooooo`2^ool0027oo`8004coo`03003ooooo0:koo`008Ooo00<0
0?ooool0Booo00<00?ooool0[_oo000Qool00`00ooooo`1<ool00`00ooooo`2]ool0027oo`03003o
oooo04coo`03003ooooo0:goo`008Ooo00<00?ooool0C?oo00<00?ooool0[Ooo000Qool2001=ool0
0`00ooooo`2]ool0027oo`03003ooooo04coo`03003ooooo0:goo`008Ooo00<00?ooool0C?oo00<0
0?ooool0[Ooo0008ool20004ool00`00ooooo`03ool20003ool30005ool00`00ooooo`1<ool00`00
ooooo`2]ool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool0
C?oo00<00?ooool0[Ooo0007ool01000ooooo`002?oo00@00?ooool000Goo`03003ooooo00;oo`80
04koo`03003ooooo0:coo`0000?oo`0000000P000_oo00@00?ooool000Soo`04003ooooo0002ool3
0005ool00`00ooooo`1=ool00`00ooooo`2/ool000Ooo`04003ooooo0008ool01000ooooo`000_oo
00<00?ooool01Ooo00<00?ooool0COoo00<00?ooool0[?oo0007ool01000ooooo`002?oo00@00?oo
ool000;oo`03003ooooo00Goo`03003ooooo04goo`03003ooooo0:coo`002?oo0P002_oo0P000ooo
10001?oo00<00?ooool0COoo00<00?ooool0[?oo000Qool2001>ool00`00ooooo`2/ool0027oo`03
003ooooo04goo`03003ooooo0:coo`008Ooo00<00?ooool0C_oo00<00?ooool0Zooo000Qool00`00
ooooo`1>ool00`00ooooo`2[ool0027oo`03003ooooo04koo`03003ooooo0:_oo`008Ooo0P00Cooo
00<00?ooool0Zooo000Qool00`00ooooo`1>ool00`00ooooo`2[ool0027oo`03003ooooo04koo`03
003ooooo0:_oo`008Ooo00<00?ooool0C_oo00<00?ooool0Zooo000Qool00`00ooooo`1>ool00`00
ooooo`2[ool0027oo`80023oo`8000Coo`03003ooooo00;oo`<0023oo`@0023oo`@000?oo`03003o
oooo00;oo`<0023oo`@001ooo`@000?oo`03003ooooo00;oo`<0027oo`8000Ooo`008Ooo00<00?oo
ool07_oo00@00?ooool000_oo`03003ooooo01koo`8002?oo`03003ooooo00[oo`03003ooooo01go
o`03003ooooo023oo`03003ooooo00coo`03003ooooo01goo`04003ooooo0006ool0027oo`03003o
oooo01koo`04003ooooo000;ool00`00ooooo`0Nool2000Sool00`00ooooo`0:ool00`00ooooo`0N
ool00`00ooooo`0Pool00`00ooooo`0;ool00`00ooooo`0Pool00`00ooooo`04ool0027oo`03003o
oooo01koo`04003ooooo0008ool3000Qool2000Sool00`00ooooo`07ool3000Rool00`00ooooo`0P
ool00`00ooooo`07ool3000Qool20007ool0027oo`03003ooooo01koo`04003ooooo0008ool00`00
ooooo`0Rool00`00ooooo`0Qool00`00ooooo`07ool00`00ooooo`0Sool00`00ooooo`0Pool00`00
ooooo`06ool00`00ooooo`0Sool00`00ooooo`04ool0027oo`8001ooo`04003ooooo0008ool00`00
ooooo`0Pool3000Qool30009ool00`00ooooo`0Pool01000ooooo`007ooo00@00?ooool000Soo`03
003ooooo023oo`04003ooooo0006ool0027oo`03003ooooo01ooo`8000Woo`@0027oo`03003ooooo
027oo`03003ooooo00Ooo`@0023oo`80027oo`8000Woo`@0023oo`8000Ooo`008Ooo00<00?ooool0
Cooo00<00?ooool0Z_oo000Qool00`00ooooo`1?ool00`00ooooo`2Zool0027oo`03003ooooo04oo
o`03003ooooo0:[oo`008Ooo00<00?ooool0Cooo00<00?ooool0Z_oo000Koooo00060000027oo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Goo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.402939, -0.325948, \
0.0121183, 0.00197581}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Im[SXiA[z, Cos[10^\((\(-4\))\)], 0]], {z, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.603319 1.94709 [
[.18254 .59082 -9 -9 ]
[.18254 .59082 9 0 ]
[.34127 .59082 -3 -9 ]
[.34127 .59082 3 0 ]
[.5 .59082 -9 -9 ]
[.5 .59082 9 0 ]
[.65873 .59082 -3 -9 ]
[.65873 .59082 3 0 ]
[.81746 .59082 -9 -9 ]
[.81746 .59082 9 0 ]
[.97619 .59082 -3 -9 ]
[.97619 .59082 3 0 ]
[.01131 .01919 -24 -4.5 ]
[.01131 .01919 0 4.5 ]
[.01131 .11655 -30 -4.5 ]
[.01131 .11655 0 4.5 ]
[.01131 .2139 -24 -4.5 ]
[.01131 .2139 0 4.5 ]
[.01131 .31126 -30 -4.5 ]
[.01131 .31126 0 4.5 ]
[.01131 .40861 -24 -4.5 ]
[.01131 .40861 0 4.5 ]
[.01131 .50596 -30 -4.5 ]
[.01131 .50596 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .60332 m
.18254 .60957 L
s
[(0.5)] .18254 .59082 0 1 Mshowa
.34127 .60332 m
.34127 .60957 L
s
[(1)] .34127 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(1.5)] .5 .59082 0 1 Mshowa
.65873 .60332 m
.65873 .60957 L
s
[(2)] .65873 .59082 0 1 Mshowa
.81746 .60332 m
.81746 .60957 L
s
[(2.5)] .81746 .59082 0 1 Mshowa
.97619 .60332 m
.97619 .60957 L
s
[(3)] .97619 .59082 0 1 Mshowa
.125 Mabswid
.05556 .60332 m
.05556 .60707 L
s
.0873 .60332 m
.0873 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.15079 .60332 m
.15079 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.24603 .60332 m
.24603 .60707 L
s
.27778 .60332 m
.27778 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.37302 .60332 m
.37302 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.43651 .60332 m
.43651 .60707 L
s
.46825 .60332 m
.46825 .60707 L
s
.53175 .60332 m
.53175 .60707 L
s
.56349 .60332 m
.56349 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.62698 .60332 m
.62698 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.72222 .60332 m
.72222 .60707 L
s
.75397 .60332 m
.75397 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.84921 .60332 m
.84921 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.9127 .60332 m
.9127 .60707 L
s
.94444 .60332 m
.94444 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .01919 m
.03006 .01919 L
s
[(-0.3)] .01131 .01919 1 0 Mshowa
.02381 .11655 m
.03006 .11655 L
s
[(-0.25)] .01131 .11655 1 0 Mshowa
.02381 .2139 m
.03006 .2139 L
s
[(-0.2)] .01131 .2139 1 0 Mshowa
.02381 .31126 m
.03006 .31126 L
s
[(-0.15)] .01131 .31126 1 0 Mshowa
.02381 .40861 m
.03006 .40861 L
s
[(-0.1)] .01131 .40861 1 0 Mshowa
.02381 .50596 m
.03006 .50596 L
s
[(-0.05)] .01131 .50596 1 0 Mshowa
.125 Mabswid
.02381 .03866 m
.02756 .03866 L
s
.02381 .05813 m
.02756 .05813 L
s
.02381 .0776 m
.02756 .0776 L
s
.02381 .09708 m
.02756 .09708 L
s
.02381 .13602 m
.02756 .13602 L
s
.02381 .15549 m
.02756 .15549 L
s
.02381 .17496 m
.02756 .17496 L
s
.02381 .19443 m
.02756 .19443 L
s
.02381 .23337 m
.02756 .23337 L
s
.02381 .25284 m
.02756 .25284 L
s
.02381 .27231 m
.02756 .27231 L
s
.02381 .29178 m
.02756 .29178 L
s
.02381 .33073 m
.02756 .33073 L
s
.02381 .3502 m
.02756 .3502 L
s
.02381 .36967 m
.02756 .36967 L
s
.02381 .38914 m
.02756 .38914 L
s
.02381 .42808 m
.02756 .42808 L
s
.02381 .44755 m
.02756 .44755 L
s
.02381 .46702 m
.02756 .46702 L
s
.02381 .48649 m
.02756 .48649 L
s
.02381 .52544 m
.02756 .52544 L
s
.02381 .54491 m
.02756 .54491 L
s
.02381 .56438 m
.02756 .56438 L
s
.02381 .58385 m
.02756 .58385 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .45192 m
.0929 .28625 L
.11234 .21002 L
.13363 .1376 L
.15366 .08246 L
.17189 .04521 L
.18161 .03099 L
.18607 .02588 L
.19081 .02148 L
.19321 .01965 L
.19585 .01797 L
.19828 .01672 L
.20051 .01584 L
.20185 .01543 L
.20313 .01512 L
.20378 .015 L
.20448 .01489 L
.20524 .0148 L
.20595 .01475 L
.20661 .01472 L
.20733 .01472 L
.208 .01474 L
.20862 .01478 L
.2098 .01491 L
.21104 .01513 L
.21222 .01541 L
.21351 .01581 L
.21584 .01675 L
.21811 .01796 L
.22021 .01934 L
.22497 .02338 L
.23003 .02911 L
.23845 .04203 L
.24775 .06137 L
.25751 .08763 L
.26645 .11731 L
.28661 .20493 L
.30491 .31088 L
.32492 .45774 L
.33467 .54169 L
.33699 .56296 L
.33946 .58601 L
.34047 .59566 L
.34156 .60332 L
.34276 .60332 L
.34387 .60332 L
.34897 .60332 L
.35448 .60332 L
.36413 .60332 L
.40403 .60332 L
Mistroke
.44246 .60332 L
.47943 .60332 L
.51877 .60332 L
.55665 .60332 L
.5969 .60332 L
.63568 .60332 L
.67299 .60332 L
.71268 .60332 L
.7509 .60332 L
.78766 .60332 L
.82679 .60332 L
.86445 .60332 L
.90448 .60332 L
.94305 .60332 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`03003o
oooo0?coo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool000koo`8000Coo`03003ooooo
00?oo`8000Goo`03003ooooo0?coo`003Ooo00@00?ooool000Soo`04003ooooo0004ool00`00oooo
o`0Yool7003<ool000goo`04003ooooo000;ool00`00ooooo`02ool2000Yool20006ool00`00oooo
o`39ool000Ooo`@000;oo`04003ooooo0009ool20005ool00`00ooooo`0Wool00`00ooooo`07ool0
0`00ooooo`38ool000goo`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0Wool00`00oooo
o`08ool00`00ooooo`37ool000goo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool09_oo
00<00?ooool02_oo00<00?ooool0a_oo000>ool2000:ool20005ool00`00ooooo`0Uool00`00oooo
o`0;ool00`00ooooo`36ool0027oo`8002Goo`03003ooooo00goo`03003ooooo0<Goo`008Ooo00<0
0?ooool08ooo00<00?ooool03ooo00<00?ooool0a?oo000Qool00`00ooooo`0Sool00`00ooooo`0?
ool00`00ooooo`34ool0027oo`03003ooooo02;oo`03003ooooo017oo`03003ooooo0<?oo`008Ooo
00<00?ooool08_oo00<00?ooool04Ooo00<00?ooool0`ooo000Qool2000Rool00`00ooooo`0Cool0
0`00ooooo`32ool0027oo`03003ooooo027oo`03003ooooo01?oo`03003ooooo0<;oo`008Ooo00<0
0?ooool08Ooo00<00?ooool04ooo00<00?ooool0`_oo000Qool00`00ooooo`0Pool00`00ooooo`0E
ool00`00ooooo`31ool0027oo`03003ooooo023oo`03003ooooo01Goo`03003ooooo0<7oo`008Ooo
0P008?oo00<00?ooool05_oo00<00?ooool0`Ooo000Qool00`00ooooo`0Oool00`00ooooo`0Gool0
0`00ooooo`30ool0027oo`03003ooooo01ooo`03003ooooo01Ooo`03003ooooo0<3oo`008Ooo00<0
0?ooool07_oo00<00?ooool06?oo00<00?ooool0`?oo000Qool00`00ooooo`0Nool00`00ooooo`0I
ool00`00ooooo`2oool0027oo`8001koo`03003ooooo01[oo`03003ooooo0;ooo`008Ooo00<00?oo
ool07Ooo00<00?ooool06_oo00<00?ooool0_ooo000Qool00`00ooooo`0Lool00`00ooooo`0Lool0
0`00ooooo`2nool000Soo`8000Coo`03003ooooo00;oo`@000;oo`<000Goo`03003ooooo01coo`03
003ooooo01coo`03003ooooo0;koo`001ooo00@00?ooool000Soo`03003ooooo00Koo`03003ooooo
00;oo`03003ooooo01coo`03003ooooo01goo`03003ooooo0;goo`001ooo00@00?ooool000Woo`03
003ooooo00Goo`03003ooooo00;oo`8001coo`03003ooooo01koo`03003ooooo0;goo`0000?oo`00
00000P000_oo00@00?ooool000[oo`04003oooooool30005ool00`00ooooo`0Kool00`00ooooo`0N
ool00`00ooooo`2mool000Ooo`04003ooooo000;ool01000ooooo`001ooo00<00?ooool06_oo00<0
0?ooool07ooo00<00?ooool0_Ooo0007ool01000ooooo`002?oo00@00?ooool000;oo`03003ooooo
00Goo`03003ooooo01[oo`03003ooooo023oo`03003ooooo0;coo`002?oo0P002_oo0P000ooo1000
1?oo00<00?ooool06Ooo00<00?ooool08Ooo00<00?ooool0_?oo000Qool2000Jool00`00ooooo`0Q
ool00`00ooooo`2lool0027oo`03003ooooo01Woo`03003ooooo027oo`03003ooooo0;coo`008Ooo
00<00?ooool06?oo00<00?ooool08ooo00<00?ooool0^ooo000Qool00`00ooooo`0Hool00`00oooo
o`0Sool00`00ooooo`2kool0027oo`03003ooooo01Soo`03003ooooo02?oo`03003ooooo0;_oo`00
8Ooo0P006Ooo00<00?ooool08ooo00<00?ooool0^ooo000Qool00`00ooooo`0Gool00`00ooooo`0T
ool00`00ooooo`2kool0027oo`03003ooooo01Ooo`03003ooooo02Goo`03003ooooo0;[oo`008Ooo
00<00?ooool05ooo00<00?ooool09Ooo00<00?ooool0^_oo000Qool00`00ooooo`0Gool00`00oooo
o`0Uool00`00ooooo`2jool0027oo`8001Ooo`03003ooooo02Koo`03003ooooo0;[oo`008Ooo00<0
0?ooool05_oo00<00?ooool09_oo00<00?ooool0^_oo000Qool00`00ooooo`0Fool00`00ooooo`0W
ool00`00ooooo`2iool0027oo`03003ooooo01Koo`03003ooooo02Ooo`03003ooooo0;Woo`008Ooo
00<00?ooool05Ooo00<00?ooool0:?oo00<00?ooool0^Ooo000Qool2000Fool00`00ooooo`0Xool0
0`00ooooo`2iool0027oo`03003ooooo01Goo`03003ooooo02Woo`03003ooooo0;Soo`008Ooo00<0
0?ooool05Ooo00<00?ooool0:Ooo00<00?ooool0^?oo000>ool20004ool00`00ooooo`02ool40004
ool00`00ooooo`0Dool00`00ooooo`0Zool00`00ooooo`2hool000goo`04003ooooo0008ool00`00
ooooo`05ool00`00ooooo`0Dool00`00ooooo`0Zool00`00ooooo`2hool000goo`04003ooooo0009
ool00`00ooooo`04ool2000Eool00`00ooooo`0Zool00`00ooooo`2hool000Ooo`@000;oo`04003o
oooo000:ool00`00ooooo`03ool00`00ooooo`0Dool00`00ooooo`0[ool00`00ooooo`2gool000go
o`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`0Cool00`00ooooo`0/ool00`00ooooo`2g
ool000goo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool04ooo00<00?ooool0;?oo00<0
0?ooool0]ooo000>ool2000:ool20005ool00`00ooooo`0Cool00`00ooooo`0/ool00`00ooooo`2g
ool0027oo`8001Coo`03003ooooo02coo`03003ooooo0;Ooo`008Ooo00<00?ooool04_oo00<00?oo
ool0;Ooo00<00?ooool0]ooo000Qool00`00ooooo`0Bool00`00ooooo`0^ool00`00ooooo`2fool0
027oo`03003ooooo01;oo`03003ooooo02koo`03003ooooo0;Koo`008Ooo00<00?ooool04_oo00<0
0?ooool0;_oo00<00?ooool0]_oo000Qool2000Bool00`00ooooo`0_ool00`00ooooo`2fool0027o
o`03003ooooo017oo`03003ooooo02ooo`03003ooooo0;Koo`008Ooo00<00?ooool04Ooo00<00?oo
ool0;ooo00<00?ooool0]_oo000Qool00`00ooooo`0Aool00`00ooooo`0`ool00`00ooooo`2eool0
027oo`03003ooooo013oo`03003ooooo037oo`03003ooooo0;Goo`008Ooo0P004Ooo00<00?ooool0
<Ooo00<00?ooool0]Ooo000Qool00`00ooooo`0@ool00`00ooooo`0aool00`00ooooo`2eool0027o
o`03003ooooo013oo`03003ooooo037oo`03003ooooo0;Goo`008Ooo00<00?ooool03ooo00<00?oo
ool0<ooo00<00?ooool0]?oo000Qool00`00ooooo`0?ool00`00ooooo`0cool00`00ooooo`2dool0
027oo`03003ooooo00ooo`03003ooooo03?oo`03003ooooo0;Coo`008Ooo0P004?oo00<00?ooool0
<ooo00<00?ooool0]?oo000Qool00`00ooooo`0>ool00`00ooooo`0dool00`00ooooo`2dool0027o
o`03003ooooo00koo`03003ooooo03Coo`03003ooooo0;Coo`002?oo0P001?oo00<00?ooool00_oo
10000_oo0`001Ooo00<00?ooool03_oo00<00?ooool0=Ooo00<00?ooool0/ooo0007ool01000oooo
o`002_oo00<00?ooool01?oo00<00?ooool00_oo00<00?ooool03_oo00<00?ooool0=Ooo00<00?oo
ool0/ooo0007ool01000ooooo`002_oo00<00?ooool01?oo00<00?ooool00_oo0P003_oo00<00?oo
ool0=_oo00<00?ooool0/ooo00000ooo000000020002ool01000ooooo`002_oo00@00?ooooooo`<0
00Goo`03003ooooo00goo`03003ooooo03Koo`03003ooooo0;?oo`001ooo00@00?ooool000[oo`05
003oooooool00007ool00`00ooooo`0=ool00`00ooooo`0fool00`00ooooo`2cool000Ooo`04003o
oooo0008ool30003ool00`00ooooo`05ool00`00ooooo`0=ool00`00ooooo`0fool00`00ooooo`2c
ool000Soo`8000_oo`04003oooooool40004ool00`00ooooo`0<ool00`00ooooo`0hool00`00oooo
o`2bool0027oo`8000goo`03003ooooo03Soo`03003ooooo0;;oo`008Ooo00<00?ooool03?oo00<0
0?ooool0>?oo00<00?ooool0/_oo000Qool00`00ooooo`0<ool00`00ooooo`0hool00`00ooooo`2b
ool0027oo`03003ooooo00coo`03003ooooo03Soo`03003ooooo0;;oo`008Ooo00<00?ooool02ooo
00<00?ooool0>Ooo00<00?ooool0/_oo000Qool2000<ool00`00ooooo`0iool00`00ooooo`2bool0
027oo`03003ooooo00_oo`03003ooooo03Woo`03003ooooo0;;oo`008Ooo00<00?ooool02ooo00<0
0?ooool0>_oo00<00?ooool0/Ooo000Qool00`00ooooo`0:ool00`00ooooo`0kool00`00ooooo`2a
ool0027oo`03003ooooo00[oo`03003ooooo03_oo`03003ooooo0;7oo`008Ooo0P002ooo00<00?oo
ool0>ooo00<00?ooool0/Ooo000Qool00`00ooooo`0:ool00`00ooooo`0kool00`00ooooo`2aool0
027oo`03003ooooo00Woo`03003ooooo03coo`03003ooooo0;7oo`008Ooo00<00?ooool02Ooo00<0
0?ooool0??oo00<00?ooool0/Ooo000Qool00`00ooooo`09ool00`00ooooo`0lool00`00ooooo`2a
ool0027oo`8000[oo`03003ooooo03goo`03003ooooo0;3oo`008Ooo00<00?ooool02?oo00<00?oo
ool0?_oo00<00?ooool0/?oo000Qool00`00ooooo`08ool00`00ooooo`0nool00`00ooooo`2`ool0
00koo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo00Soo`03003ooooo03koo`03003ooooo
0;3oo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo00Soo`03003ooooo03koo`03
003ooooo0;3oo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`8000Woo`03003ooooo03koo`03
003ooooo0;3oo`001ooo10000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo00Ooo`03
003ooooo03ooo`03003ooooo0;3oo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo
00Ooo`03003ooooo043oo`03003ooooo0:ooo`003Ooo00@00?ooool000Soo`<000Goo`03003ooooo
00Ooo`03003ooooo043oo`03003ooooo0:ooo`003_oo0P002ooo00<00?ooool00ooo00<00?ooool0
1ooo00<00?ooool0@?oo00<00?ooool0[ooo000Qool20007ool00`00ooooo`11ool00`00ooooo`2_
ool0027oo`03003ooooo00Koo`03003ooooo047oo`03003ooooo0:ooo`008Ooo00<00?ooool01_oo
00<00?ooool0@Ooo00<00?ooool0[ooo000Qool00`00ooooo`06ool00`00ooooo`11ool00`00oooo
o`2_ool0027oo`03003ooooo00Goo`03003ooooo04;oo`03003ooooo0:ooo`008Ooo0P001_oo00<0
0?ooool0@ooo00<00?ooool0[_oo000Qool00`00ooooo`05ool00`00ooooo`13ool00`00ooooo`2^
ool0027oo`03003ooooo04_oo`03003ooooo0:koo`008Ooo00<00?ooool0Booo00<00?ooool0[_oo
000Qool00`00ooooo`1;ool00`00ooooo`2^ool0027oo`8004coo`03003ooooo0:koo`008Ooo00<0
0?ooool0Booo00<00?ooool0[_oo000Qool00`00ooooo`1<ool00`00ooooo`2]ool0027oo`03003o
oooo04coo`03003ooooo0:goo`008Ooo00<00?ooool0C?oo00<00?ooool0[Ooo000Qool2001=ool0
0`00ooooo`2]ool0027oo`03003ooooo04coo`03003ooooo0:goo`008Ooo00<00?ooool0C?oo00<0
0?ooool0[Ooo0008ool20004ool00`00ooooo`03ool20003ool30005ool00`00ooooo`1<ool00`00
ooooo`2]ool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?ooool00_oo00<00?ooool0
C?oo00<00?ooool0[Ooo0007ool01000ooooo`002?oo00@00?ooool000Goo`03003ooooo00;oo`80
04koo`03003ooooo0:coo`0000?oo`0000000P000_oo00@00?ooool000Soo`04003ooooo0002ool3
0005ool00`00ooooo`1=ool00`00ooooo`2/ool000Ooo`04003ooooo0008ool01000ooooo`000_oo
00<00?ooool01Ooo00<00?ooool0COoo00<00?ooool0[?oo0007ool01000ooooo`002?oo00@00?oo
ool000;oo`03003ooooo00Goo`03003ooooo04goo`03003ooooo0:coo`002?oo0P002_oo0P000ooo
10001?oo00<00?ooool0COoo00<00?ooool0[?oo000Qool2001>ool00`00ooooo`2/ool0027oo`03
003ooooo04goo`03003ooooo0:coo`008Ooo00<00?ooool0C_oo00<00?ooool0Zooo000Qool00`00
ooooo`1>ool00`00ooooo`2[ool0027oo`03003ooooo04koo`03003ooooo0:_oo`008Ooo0P00Cooo
00<00?ooool0Zooo000Qool00`00ooooo`1>ool00`00ooooo`2[ool0027oo`03003ooooo04koo`03
003ooooo0:_oo`008Ooo00<00?ooool0C_oo00<00?ooool0Zooo000Qool00`00ooooo`1>ool00`00
ooooo`2[ool0027oo`80023oo`8000Coo`03003ooooo00;oo`<0023oo`@0023oo`@000?oo`03003o
oooo00;oo`<0023oo`@001ooo`@000?oo`03003ooooo00;oo`<0027oo`8000Ooo`008Ooo00<00?oo
ool07_oo00@00?ooool000_oo`03003ooooo01koo`8002?oo`03003ooooo00[oo`03003ooooo01go
o`03003ooooo023oo`03003ooooo00coo`03003ooooo01goo`04003ooooo0006ool0027oo`03003o
oooo01koo`04003ooooo000;ool00`00ooooo`0Nool2000Sool00`00ooooo`0:ool00`00ooooo`0N
ool00`00ooooo`0Pool00`00ooooo`0;ool00`00ooooo`0Pool00`00ooooo`04ool0027oo`03003o
oooo01koo`04003ooooo0008ool3000Qool2000Sool00`00ooooo`07ool3000Rool00`00ooooo`0P
ool00`00ooooo`07ool3000Qool20007ool0027oo`03003ooooo01koo`04003ooooo0008ool00`00
ooooo`0Rool00`00ooooo`0Qool00`00ooooo`07ool00`00ooooo`0Sool00`00ooooo`0Pool00`00
ooooo`06ool00`00ooooo`0Sool00`00ooooo`04ool0027oo`8001ooo`04003ooooo0008ool00`00
ooooo`0Pool3000Qool30009ool00`00ooooo`0Pool01000ooooo`007ooo00@00?ooool000Soo`03
003ooooo023oo`04003ooooo0006ool0027oo`03003ooooo01ooo`8000Woo`@0027oo`03003ooooo
027oo`03003ooooo00Ooo`@0023oo`80027oo`8000Woo`@0023oo`8000Ooo`008Ooo00<00?ooool0
Cooo00<00?ooool0Z_oo000Qool00`00ooooo`1?ool00`00ooooo`2Zool0027oo`03003ooooo04oo
o`03003ooooo0:[oo`008Ooo00<00?ooool0Cooo00<00?ooool0Z_oo000Koooo00060000027oo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Goo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.402939, -0.325948, \
0.0121183, 0.00197581}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[BoxData[
    \(also\ o . k . %770[\([3, 2]\)]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[Normal[%770[\([3, 3]\)]]], {z, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.368487 1.13419 [
[.18254 .35599 -9 -9 ]
[.18254 .35599 9 0 ]
[.34127 .35599 -3 -9 ]
[.34127 .35599 3 0 ]
[.5 .35599 -9 -9 ]
[.5 .35599 9 0 ]
[.65873 .35599 -3 -9 ]
[.65873 .35599 3 0 ]
[.81746 .35599 -9 -9 ]
[.81746 .35599 9 0 ]
[.97619 .35599 -3 -9 ]
[.97619 .35599 3 0 ]
[.01131 .02823 -24 -4.5 ]
[.01131 .02823 0 4.5 ]
[.01131 .14165 -24 -4.5 ]
[.01131 .14165 0 4.5 ]
[.01131 .25507 -24 -4.5 ]
[.01131 .25507 0 4.5 ]
[.01131 .48191 -18 -4.5 ]
[.01131 .48191 0 4.5 ]
[.01131 .59533 -18 -4.5 ]
[.01131 .59533 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .36849 m
.18254 .37474 L
s
[(0.5)] .18254 .35599 0 1 Mshowa
.34127 .36849 m
.34127 .37474 L
s
[(1)] .34127 .35599 0 1 Mshowa
.5 .36849 m
.5 .37474 L
s
[(1.5)] .5 .35599 0 1 Mshowa
.65873 .36849 m
.65873 .37474 L
s
[(2)] .65873 .35599 0 1 Mshowa
.81746 .36849 m
.81746 .37474 L
s
[(2.5)] .81746 .35599 0 1 Mshowa
.97619 .36849 m
.97619 .37474 L
s
[(3)] .97619 .35599 0 1 Mshowa
.125 Mabswid
.05556 .36849 m
.05556 .37224 L
s
.0873 .36849 m
.0873 .37224 L
s
.11905 .36849 m
.11905 .37224 L
s
.15079 .36849 m
.15079 .37224 L
s
.21429 .36849 m
.21429 .37224 L
s
.24603 .36849 m
.24603 .37224 L
s
.27778 .36849 m
.27778 .37224 L
s
.30952 .36849 m
.30952 .37224 L
s
.37302 .36849 m
.37302 .37224 L
s
.40476 .36849 m
.40476 .37224 L
s
.43651 .36849 m
.43651 .37224 L
s
.46825 .36849 m
.46825 .37224 L
s
.53175 .36849 m
.53175 .37224 L
s
.56349 .36849 m
.56349 .37224 L
s
.59524 .36849 m
.59524 .37224 L
s
.62698 .36849 m
.62698 .37224 L
s
.69048 .36849 m
.69048 .37224 L
s
.72222 .36849 m
.72222 .37224 L
s
.75397 .36849 m
.75397 .37224 L
s
.78571 .36849 m
.78571 .37224 L
s
.84921 .36849 m
.84921 .37224 L
s
.88095 .36849 m
.88095 .37224 L
s
.9127 .36849 m
.9127 .37224 L
s
.94444 .36849 m
.94444 .37224 L
s
.25 Mabswid
0 .36849 m
1 .36849 L
s
.02381 .02823 m
.03006 .02823 L
s
[(-0.3)] .01131 .02823 1 0 Mshowa
.02381 .14165 m
.03006 .14165 L
s
[(-0.2)] .01131 .14165 1 0 Mshowa
.02381 .25507 m
.03006 .25507 L
s
[(-0.1)] .01131 .25507 1 0 Mshowa
.02381 .48191 m
.03006 .48191 L
s
[(0.1)] .01131 .48191 1 0 Mshowa
.02381 .59533 m
.03006 .59533 L
s
[(0.2)] .01131 .59533 1 0 Mshowa
.125 Mabswid
.02381 .05091 m
.02756 .05091 L
s
.02381 .0736 m
.02756 .0736 L
s
.02381 .09628 m
.02756 .09628 L
s
.02381 .11897 m
.02756 .11897 L
s
.02381 .16433 m
.02756 .16433 L
s
.02381 .18702 m
.02756 .18702 L
s
.02381 .2097 m
.02756 .2097 L
s
.02381 .23238 m
.02756 .23238 L
s
.02381 .27775 m
.02756 .27775 L
s
.02381 .30044 m
.02756 .30044 L
s
.02381 .32312 m
.02756 .32312 L
s
.02381 .3458 m
.02756 .3458 L
s
.02381 .39117 m
.02756 .39117 L
s
.02381 .41386 m
.02756 .41386 L
s
.02381 .43654 m
.02756 .43654 L
s
.02381 .45922 m
.02756 .45922 L
s
.02381 .50459 m
.02756 .50459 L
s
.02381 .52727 m
.02756 .52727 L
s
.02381 .54996 m
.02756 .54996 L
s
.02381 .57264 m
.02756 .57264 L
s
.02381 .00555 m
.02756 .00555 L
s
.02381 .61801 m
.02756 .61801 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.07374 .04948 L
.0929 .10054 L
.13363 .24503 L
.17042 .39203 L
.19092 .46802 L
.20958 .52687 L
.21887 .55107 L
.22896 .5727 L
.23358 .58084 L
.23848 .58817 L
.24299 .59369 L
.24727 .59781 L
.24954 .59954 L
.25202 .60106 L
.25428 .60211 L
.25528 .60247 L
.25637 .60279 L
.25704 .60295 L
.25765 .60307 L
.25885 .60324 L
.25995 .60332 L
.26111 .60331 L
.26235 .60321 L
.26351 .60303 L
.26479 .60272 L
.26616 .60228 L
.26843 .60127 L
.27091 .5998 L
.27541 .59612 L
.27979 .5913 L
.28385 .58579 L
.29315 .56951 L
.30189 .55012 L
.31159 .52507 L
.3218 .49697 L
.33112 .47352 L
.33367 .46833 L
.33508 .46589 L
.3364 .46394 L
.33764 .46252 L
.33833 .46193 L
.33898 .46156 L
.34011 .46146 L
.34131 .4632 L
.3424 .46484 L
.34344 .46545 L
.34463 .46577 L
.34571 .46586 L
.34674 .46583 L
Mistroke
.34768 .46573 L
.34872 .46557 L
.34983 .46534 L
.35475 .46396 L
.3592 .46245 L
.39798 .44959 L
.4353 .44071 L
.47499 .43402 L
.51321 .42933 L
.54996 .42588 L
.58909 .42302 L
.62675 .42083 L
.66679 .41896 L
.70535 .41748 L
.74246 .41629 L
.78193 .41523 L
.81994 .41436 L
.85648 .41364 L
.89539 .41297 L
.93284 .41242 L
.97266 .4119 L
.97619 .41186 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool2
003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`05ool00`00ooooo`3jool000Soo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo00Koo`03
003ooooo0?Woo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`06ool00`00oooo
o`3iool000Ooo`04003ooooo000;ool00`00ooooo`02ool20008ool00`00ooooo`3hool00003ool0
0000008000;oo`04003ooooo0009ool20005ool00`00ooooo`07ool00`00ooooo`3hool000Ooo`04
003ooooo000;ool00`00ooooo`02ool00`00ooooo`08ool00`00ooooo`3gool000Ooo`04003ooooo
0008ool01000ooooo`001?oo00<00?ooool02?oo00<00?ooool0mooo0008ool2000:ool20005ool0
0`00ooooo`09ool00`00ooooo`3fool001_oo`03003ooooo00Woo`03003ooooo0?Koo`006ooo0P00
2ooo00<00?ooool0mOoo000Kool00`00ooooo`0:ool00`00ooooo`3eool001_oo`03003ooooo00_o
o`03003ooooo0?Coo`006ooo00<00?ooool02ooo00<00?ooool0m?oo000Kool00`00ooooo`0;ool0
0`00ooooo`3dool001_oo`03003ooooo00coo`03003ooooo0??oo`006ooo0P003Ooo00<00?ooool0
looo000Kool00`00ooooo`0<ool00`00ooooo`3cool001_oo`03003ooooo00goo`03003ooooo0?;o
o`006ooo00<00?ooool03Ooo00<00?ooool0l_oo000Kool00`00ooooo`0>ool00`00ooooo`3aool0
01_oo`03003ooooo00koo`03003ooooo0?7oo`006ooo0P003ooo00<00?ooool0lOoo000Kool00`00
ooooo`0?ool00`00ooooo`3`ool001_oo`03003ooooo00ooo`03003ooooo0?3oo`006ooo00<00?oo
ool03ooo00<00?ooool0l?oo000Kool00`00ooooo`0@ool00`00ooooo`3_ool001_oo`03003ooooo
013oo`03003ooooo0>ooo`006ooo0P004Ooo00<00?ooool0kooo000Kool00`00ooooo`0@ool00`00
ooooo`3_ool001_oo`03003ooooo017oo`03003ooooo0>koo`006ooo00<00?ooool04Ooo00<00?oo
ool0k_oo000Kool00`00ooooo`0Aool00`00ooooo`3^ool000Soo`8000Coo`03003ooooo00;oo`@0
00Coo`03003ooooo01;oo`03003ooooo0>goo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03
003ooooo01;oo`03003ooooo0>goo`001ooo00@00?ooool000Woo`03003ooooo00Coo`8001?oo`03
003ooooo0>goo`0000?oo`0000000P000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo
01;oo`03003ooooo0>goo`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo01?oo`03
003ooooo0>coo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`0Cool00`00oooo
o`3/ool000Soo`8000[oo`8000Goo`03003ooooo01?oo`03003ooooo0>coo`006ooo00<00?ooool0
5?oo00<00?ooool0jooo000Kool2000Eool00`00ooooo`3[ool001_oo`03003ooooo01Coo`03003o
oooo0>_oo`006ooo00<00?ooool05?oo00<00?ooool0jooo000Kool00`00ooooo`0Eool00`00oooo
o`3Zool001_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo00<00?ooool05Ooo00<00?ooool0
j_oo000Kool2000Gool00`00ooooo`3Yool001_oo`03003ooooo01Koo`03003ooooo0>Woo`006ooo
00<00?ooool05_oo00<00?ooool0jOoo000Kool00`00ooooo`0Gool00`00ooooo`3Xool001_oo`03
003ooooo01Ooo`03003ooooo0>Soo`006ooo00<00?ooool05ooo00<00?ooool0j?oo000Kool2000H
ool00`00ooooo`3Xool001_oo`03003ooooo01Soo`03003ooooo0>Ooo`006ooo00<00?ooool06?oo
00<00?ooool0iooo000Kool00`00ooooo`0Hool00`00ooooo`3Wool001_oo`03003ooooo01Woo`03
003ooooo0>Koo`006ooo00<00?ooool06Ooo00<00?ooool0i_oo000Kool2000Jool00`00ooooo`3V
ool001_oo`03003ooooo01Woo`03003ooooo0>Koo`006ooo00<00?ooool06_oo00<00?ooool0iOoo
000Kool00`00ooooo`0Jool00`00ooooo`3Uool000Soo`8000Coo`03003ooooo00;oo`@000Coo`03
003ooooo01[oo`03003ooooo0>Goo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo
01_oo`03003ooooo0>Coo`001ooo00@00?ooool000[oo`03003ooooo00?oo`8001coo`03003ooooo
0>Coo`0000?oo`0000000P000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo01_oo`03
003ooooo0>Coo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo01_oo`03003ooooo
0>Coo`001ooo00@00?ooool000Soo`<000Goo`03003ooooo01coo`03003ooooo0>?oo`002?oo0P00
2ooo00<00?ooool00ooo00<00?ooool07?oo00<00?ooool0hooo000Kool00`00ooooo`0Lool00`00
ooooo`3Sool001_oo`8001goo`03003ooooo0>?oo`006ooo00<00?ooool07Ooo00<00?ooool0h_oo
000Kool00`00ooooo`0Mool00`00ooooo`3Rool001_oo`03003ooooo01goo`03003ooooo0>;oo`00
6ooo00<00?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Nool00`00ooooo`3Qool001_o
o`8001ooo`03003ooooo0>7oo`006ooo00<00?ooool07_oo00<00?ooool0hOoo000Kool00`00oooo
o`0Nool00`00ooooo`3Qool001_oo`03003ooooo01ooo`03003ooooo0>3oo`006ooo00<00?ooool0
7ooo00<00?ooool0h?oo000Kool00`00ooooo`0Oool00`00ooooo`3Pool001_oo`80023oo`03003o
oooo0>3oo`006ooo00<00?ooool08?oo00<00?ooool0gooo000Kool00`00ooooo`0Pool20004ool0
0`00ooooo`02ool3000Qool4000Pool40003ool00`00ooooo`02ool3000Rool4000Pool40003ool0
0`00ooooo`02ool3000Rool20007ool001_oo`03003ooooo01ooo`800003ool00?oo00[oo`03003o
oooo023oo`03003ooooo027oo`03003ooooo00[oo`03003ooooo01ooo`03003ooooo027oo`03003o
oooo00coo`03003ooooo01koo`04003ooooo0006ool001_oo`03003ooooo01ooo`800003ool00?oo
00[oo`03003ooooo023oo`03003ooooo027oo`03003ooooo00[oo`03003ooooo023oo`03003ooooo
027oo`03003ooooo00_oo`03003ooooo027oo`03003ooooo00Coo`006ooo00<00?ooool07ooo00@0
0?oo000000Soo`<002?oo`03003ooooo027oo`03003ooooo00Ooo`<002Coo`03003ooooo027oo`03
003ooooo00Ooo`<002;oo`8000Ooo`006ooo0P008?oo00@00?oo000000Soo`03003ooooo02?oo`03
003ooooo027oo`03003ooooo00Ooo`03003ooooo02Goo`03003ooooo027oo`03003ooooo00Koo`03
003ooooo02Coo`03003ooooo00Coo`006ooo00<00?ooool07ooo00@00?oo000000Soo`03003ooooo
027oo`<0027oo`<000Woo`03003ooooo02;oo`04003ooooo000Pool01000ooooo`002?oo00<00?oo
ool08Ooo00@00?ooool000Koo`006ooo00<00?ooool08?oo0P002Ooo10008_oo00<00?ooool08Ooo
00<00?ooool01ooo10008_oo0P008_oo0P002Ooo10008Ooo0P001ooo000Kool00`00ooooo`0Rool0
0`00ooooo`3Mool001_oo`03003ooooo02;oo`03003ooooo0=goo`006ooo00<00?ooool08_oo00<0
0?ooool0gOoo000Doooo000=000001_oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Koo`05003oooooool00003ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00oooo
o`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00oooo
o`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool001_oo`03003ooooo02?oo`05
003oooooool0000Yool00`00ooooo`0Wool00`00ooooo`0Xool00`00ooooo`0Wool00`00ooooo`0W
ool00`00ooooo`05ool001_oo`03003ooooo02?oo`03003ooooo0=coo`006ooo00<00?ooool08ooo
00<00?ooool0g?oo000Kool00`00ooooo`0Tool00`00ooooo`3Kool001_oo`8002Goo`03003ooooo
0=_oo`006ooo00<00?ooool09?oo00<00?ooool0fooo000Kool00`00ooooo`0Tool00`00ooooo`3K
ool001_oo`03003ooooo02Goo`03003ooooo0=[oo`006ooo00<00?ooool09Ooo00<00?ooool0f_oo
000Kool00`00ooooo`0Uool00`00ooooo`3<ool70007ool001_oo`8002Koo`03003ooooo09Wooc<0
00koo`006ooo00<00?ooool09_oo00<00?ooool0N_oo7P00@Ooo000Kool00`00ooooo`0Vool00`00
ooooo`1UoolE001Oool001_oo`03003ooooo02Koo`03003ooooo05_oo`X007Coo`006ooo00<00?oo
ool09_oo00<00?ooool0DOoo2P00O_oo000Kool00`00ooooo`0Wool00`00ooooo`19ool70028ool0
01_oo`8002Soo`03003ooooo04?oo`H008ooo`006ooo00<00?ooool09ooo00<00?ooool0?_oo1@00
UOoo000Kool00`00ooooo`0Wool00`00ooooo`0iool5002Jool001_oo`03003ooooo02Soo`03003o
oooo03Coo`@009ooo`006ooo00<00?ooool0:?oo00<00?ooool0<_oo0P00Xooo000Kool00`00oooo
o`0Xool00`00ooooo`0_ool3002Uool001_oo`8002Woo`03003ooooo02goo`800:Soo`006ooo00<0
0?ooool0:Ooo00<00?ooool09?oo0`000_oo0`00Z_oo000Kool00`00ooooo`0Yool00`00ooooo`0S
ool200001?oo00000000[Ooo000Kool00`00ooooo`0Yool00`00ooooo`0Sool00`00ooooo`2`ool0
00Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo02[oo`03003ooooo02;oo`03003ooooo
0;3oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo02[oo`03003ooooo02;oo`03
003ooooo0;3oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`8002_oo`03003ooooo027oo`03
003ooooo0;7oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo02_oo`03003ooooo
023oo`03003ooooo0;7oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo02_oo`03
003ooooo01ooo`03003ooooo0;;oo`001ooo00@00?ooool000Soo`<000Goo`03003ooooo02_oo`03
003ooooo01ooo`03003ooooo0;;oo`002?oo0P002ooo00<00?ooool00ooo00<00?ooool0:ooo00<0
0?ooool07_oo00<00?ooool0/ooo000Kool00`00ooooo`0/ool00`00ooooo`0Mool00`00ooooo`2c
ool001_oo`8002goo`03003ooooo01coo`03003ooooo0;Coo`006ooo00<00?ooool0;?oo00<00?oo
ool07?oo00<00?ooool0]?oo000Kool00`00ooooo`0]ool00`00ooooo`0Kool00`00ooooo`2dool0
01_oo`03003ooooo02goo`03003ooooo01[oo`03003ooooo0;Goo`006ooo00<00?ooool0;Ooo00<0
0?ooool06_oo00<00?ooool0]Ooo000Kool00`00ooooo`0^ool00`00ooooo`0Hool00`00ooooo`2f
ool001_oo`8002ooo`03003ooooo01Soo`03003ooooo0;Koo`006ooo00<00?ooool0;_oo00<00?oo
ool06?oo00<00?ooool0]_oo000Kool00`00ooooo`0_ool00`00ooooo`0Fool00`00ooooo`2gool0
01_oo`03003ooooo02ooo`03003ooooo01Koo`03003ooooo0;Ooo`006ooo00<00?ooool0<?oo00<0
0?ooool05Ooo00<00?ooool0]ooo000Kool00`00ooooo`0`ool00`00ooooo`0Dool00`00ooooo`2h
ool001_oo`8003;oo`03003ooooo01?oo`03003ooooo0;Soo`006ooo00<00?ooool0<Ooo00<00?oo
ool04_oo00<00?ooool0^Ooo000Kool00`00ooooo`0bool00`00ooooo`0Aool00`00ooooo`2iool0
01_oo`03003ooooo03;oo`03003ooooo013oo`03003ooooo0;[oo`006ooo00<00?ooool0<_oo00<0
0?ooool04?oo00<00?ooool0^_oo000Kool00`00ooooo`0cool00`00ooooo`0>ool00`00ooooo`2k
ool001_oo`8003Coo`03003ooooo00koo`03003ooooo0;_oo`006ooo00<00?ooool0=?oo00<00?oo
ool03?oo00<00?ooool0_?oo000Kool00`00ooooo`0eool00`00ooooo`0;ool00`00ooooo`2lool0
01_oo`03003ooooo03Goo`03003ooooo00[oo`03003ooooo0;goo`002?oo0P001?oo00<00?ooool0
0_oo10001?oo00<00?ooool0=_oo00<00?ooool02Ooo00<00?ooool0_Ooo0007ool01000ooooo`00
2?oo00<00?ooool01Ooo00<00?ooool0=_oo00<00?ooool02?oo00<00?ooool0__oo0007ool01000
ooooo`002Ooo00<00?ooool01?oo0P00>?oo00<00?ooool01_oo00<00?ooool0_ooo0007ool01000
ooooo`002_oo00<00?ooool00ooo00<00?ooool0>?oo0P001Ooo00<00?ooool0`?oo0007ool01000
ooooo`002ooo00<00?ooool00_oo00<00?ooool0>_oo1@00`ooo0007ool01000ooooo`002?oo00@0
0?ooool000Coo`03003ooooo0?ooo`?oo`002?oo0P002_oo0P001Ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo
8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.323594, -0.345669, \
0.0118419, 0.00331454}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\((Re[
            SXiA[z, Cos[10^\((\(-4\))\)], 0.01] - 
              SXiA[z, Cos[10^\((\(-4\))\)], 0]])\)/0.01, {z, 0.1, 
        3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.368487 1.13419 [
[.18254 .35599 -9 -9 ]
[.18254 .35599 9 0 ]
[.34127 .35599 -3 -9 ]
[.34127 .35599 3 0 ]
[.5 .35599 -9 -9 ]
[.5 .35599 9 0 ]
[.65873 .35599 -3 -9 ]
[.65873 .35599 3 0 ]
[.81746 .35599 -9 -9 ]
[.81746 .35599 9 0 ]
[.97619 .35599 -3 -9 ]
[.97619 .35599 3 0 ]
[.01131 .02823 -24 -4.5 ]
[.01131 .02823 0 4.5 ]
[.01131 .14165 -24 -4.5 ]
[.01131 .14165 0 4.5 ]
[.01131 .25507 -24 -4.5 ]
[.01131 .25507 0 4.5 ]
[.01131 .48191 -18 -4.5 ]
[.01131 .48191 0 4.5 ]
[.01131 .59533 -18 -4.5 ]
[.01131 .59533 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .36849 m
.18254 .37474 L
s
[(0.5)] .18254 .35599 0 1 Mshowa
.34127 .36849 m
.34127 .37474 L
s
[(1)] .34127 .35599 0 1 Mshowa
.5 .36849 m
.5 .37474 L
s
[(1.5)] .5 .35599 0 1 Mshowa
.65873 .36849 m
.65873 .37474 L
s
[(2)] .65873 .35599 0 1 Mshowa
.81746 .36849 m
.81746 .37474 L
s
[(2.5)] .81746 .35599 0 1 Mshowa
.97619 .36849 m
.97619 .37474 L
s
[(3)] .97619 .35599 0 1 Mshowa
.125 Mabswid
.05556 .36849 m
.05556 .37224 L
s
.0873 .36849 m
.0873 .37224 L
s
.11905 .36849 m
.11905 .37224 L
s
.15079 .36849 m
.15079 .37224 L
s
.21429 .36849 m
.21429 .37224 L
s
.24603 .36849 m
.24603 .37224 L
s
.27778 .36849 m
.27778 .37224 L
s
.30952 .36849 m
.30952 .37224 L
s
.37302 .36849 m
.37302 .37224 L
s
.40476 .36849 m
.40476 .37224 L
s
.43651 .36849 m
.43651 .37224 L
s
.46825 .36849 m
.46825 .37224 L
s
.53175 .36849 m
.53175 .37224 L
s
.56349 .36849 m
.56349 .37224 L
s
.59524 .36849 m
.59524 .37224 L
s
.62698 .36849 m
.62698 .37224 L
s
.69048 .36849 m
.69048 .37224 L
s
.72222 .36849 m
.72222 .37224 L
s
.75397 .36849 m
.75397 .37224 L
s
.78571 .36849 m
.78571 .37224 L
s
.84921 .36849 m
.84921 .37224 L
s
.88095 .36849 m
.88095 .37224 L
s
.9127 .36849 m
.9127 .37224 L
s
.94444 .36849 m
.94444 .37224 L
s
.25 Mabswid
0 .36849 m
1 .36849 L
s
.02381 .02823 m
.03006 .02823 L
s
[(-0.3)] .01131 .02823 1 0 Mshowa
.02381 .14165 m
.03006 .14165 L
s
[(-0.2)] .01131 .14165 1 0 Mshowa
.02381 .25507 m
.03006 .25507 L
s
[(-0.1)] .01131 .25507 1 0 Mshowa
.02381 .48191 m
.03006 .48191 L
s
[(0.1)] .01131 .48191 1 0 Mshowa
.02381 .59533 m
.03006 .59533 L
s
[(0.2)] .01131 .59533 1 0 Mshowa
.125 Mabswid
.02381 .05091 m
.02756 .05091 L
s
.02381 .0736 m
.02756 .0736 L
s
.02381 .09628 m
.02756 .09628 L
s
.02381 .11897 m
.02756 .11897 L
s
.02381 .16433 m
.02756 .16433 L
s
.02381 .18702 m
.02756 .18702 L
s
.02381 .2097 m
.02756 .2097 L
s
.02381 .23238 m
.02756 .23238 L
s
.02381 .27775 m
.02756 .27775 L
s
.02381 .30044 m
.02756 .30044 L
s
.02381 .32312 m
.02756 .32312 L
s
.02381 .3458 m
.02756 .3458 L
s
.02381 .39117 m
.02756 .39117 L
s
.02381 .41386 m
.02756 .41386 L
s
.02381 .43654 m
.02756 .43654 L
s
.02381 .45922 m
.02756 .45922 L
s
.02381 .50459 m
.02756 .50459 L
s
.02381 .52727 m
.02756 .52727 L
s
.02381 .54996 m
.02756 .54996 L
s
.02381 .57264 m
.02756 .57264 L
s
.02381 .00555 m
.02756 .00555 L
s
.02381 .61801 m
.02756 .61801 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01472 m
.07374 .04948 L
.0929 .10054 L
.13363 .24503 L
.17042 .39203 L
.19092 .46802 L
.20958 .52687 L
.21887 .55107 L
.22896 .5727 L
.23358 .58084 L
.23848 .58817 L
.24299 .59369 L
.24727 .59781 L
.24954 .59954 L
.25202 .60106 L
.25428 .60211 L
.25528 .60247 L
.25637 .60279 L
.25704 .60295 L
.25765 .60307 L
.25885 .60324 L
.25995 .60332 L
.26111 .60331 L
.26235 .60321 L
.26351 .60303 L
.26479 .60272 L
.26616 .60228 L
.26843 .60127 L
.27091 .5998 L
.27541 .59612 L
.27979 .5913 L
.28385 .58579 L
.29315 .56951 L
.30189 .55012 L
.31159 .52507 L
.3218 .49697 L
.33112 .47352 L
.33367 .46833 L
.33508 .46589 L
.3364 .46394 L
.33764 .46252 L
.33833 .46193 L
.33898 .46156 L
.34011 .46146 L
.34131 .4632 L
.3424 .46484 L
.34344 .46545 L
.34463 .46577 L
.34571 .46586 L
.34674 .46583 L
Mistroke
.34768 .46573 L
.34872 .46557 L
.34983 .46534 L
.35475 .46396 L
.3592 .46245 L
.39798 .44959 L
.4353 .44071 L
.47499 .43402 L
.51321 .42933 L
.54996 .42588 L
.58909 .42302 L
.62675 .42083 L
.66679 .41896 L
.70535 .41748 L
.74246 .41629 L
.78193 .41523 L
.81994 .41436 L
.85648 .41364 L
.89539 .41297 L
.93284 .41242 L
.97266 .4119 L
.97619 .41186 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool2
003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`05ool00`00ooooo`3jool000Soo`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo00Koo`03
003ooooo0?Woo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`06ool00`00oooo
o`3iool000Ooo`04003ooooo000;ool00`00ooooo`02ool20008ool00`00ooooo`3hool00003ool0
0000008000;oo`04003ooooo0009ool20005ool00`00ooooo`07ool00`00ooooo`3hool000Ooo`04
003ooooo000;ool00`00ooooo`02ool00`00ooooo`08ool00`00ooooo`3gool000Ooo`04003ooooo
0008ool01000ooooo`001?oo00<00?ooool02?oo00<00?ooool0mooo0008ool2000:ool20005ool0
0`00ooooo`09ool00`00ooooo`3fool001_oo`03003ooooo00Woo`03003ooooo0?Koo`006ooo0P00
2ooo00<00?ooool0mOoo000Kool00`00ooooo`0:ool00`00ooooo`3eool001_oo`03003ooooo00_o
o`03003ooooo0?Coo`006ooo00<00?ooool02ooo00<00?ooool0m?oo000Kool00`00ooooo`0;ool0
0`00ooooo`3dool001_oo`03003ooooo00coo`03003ooooo0??oo`006ooo0P003Ooo00<00?ooool0
looo000Kool00`00ooooo`0<ool00`00ooooo`3cool001_oo`03003ooooo00goo`03003ooooo0?;o
o`006ooo00<00?ooool03Ooo00<00?ooool0l_oo000Kool00`00ooooo`0>ool00`00ooooo`3aool0
01_oo`03003ooooo00koo`03003ooooo0?7oo`006ooo0P003ooo00<00?ooool0lOoo000Kool00`00
ooooo`0?ool00`00ooooo`3`ool001_oo`03003ooooo00ooo`03003ooooo0?3oo`006ooo00<00?oo
ool03ooo00<00?ooool0l?oo000Kool00`00ooooo`0@ool00`00ooooo`3_ool001_oo`03003ooooo
013oo`03003ooooo0>ooo`006ooo0P004Ooo00<00?ooool0kooo000Kool00`00ooooo`0@ool00`00
ooooo`3_ool001_oo`03003ooooo017oo`03003ooooo0>koo`006ooo00<00?ooool04Ooo00<00?oo
ool0k_oo000Kool00`00ooooo`0Aool00`00ooooo`3^ool000Soo`8000Coo`03003ooooo00;oo`@0
00Coo`03003ooooo01;oo`03003ooooo0>goo`001ooo00@00?ooool000Soo`03003ooooo00Goo`03
003ooooo01;oo`03003ooooo0>goo`001ooo00@00?ooool000Woo`03003ooooo00Coo`8001?oo`03
003ooooo0>goo`0000?oo`0000000P000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo
01;oo`03003ooooo0>goo`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo01?oo`03
003ooooo0>coo`001ooo00@00?ooool000Soo`04003ooooo0004ool00`00ooooo`0Cool00`00oooo
o`3/ool000Soo`8000[oo`8000Goo`03003ooooo01?oo`03003ooooo0>coo`006ooo00<00?ooool0
5?oo00<00?ooool0jooo000Kool2000Eool00`00ooooo`3[ool001_oo`03003ooooo01Coo`03003o
oooo0>_oo`006ooo00<00?ooool05?oo00<00?ooool0jooo000Kool00`00ooooo`0Eool00`00oooo
o`3Zool001_oo`03003ooooo01Goo`03003ooooo0>[oo`006ooo00<00?ooool05Ooo00<00?ooool0
j_oo000Kool2000Gool00`00ooooo`3Yool001_oo`03003ooooo01Koo`03003ooooo0>Woo`006ooo
00<00?ooool05_oo00<00?ooool0jOoo000Kool00`00ooooo`0Gool00`00ooooo`3Xool001_oo`03
003ooooo01Ooo`03003ooooo0>Soo`006ooo00<00?ooool05ooo00<00?ooool0j?oo000Kool2000H
ool00`00ooooo`3Xool001_oo`03003ooooo01Soo`03003ooooo0>Ooo`006ooo00<00?ooool06?oo
00<00?ooool0iooo000Kool00`00ooooo`0Hool00`00ooooo`3Wool001_oo`03003ooooo01Woo`03
003ooooo0>Koo`006ooo00<00?ooool06Ooo00<00?ooool0i_oo000Kool2000Jool00`00ooooo`3V
ool001_oo`03003ooooo01Woo`03003ooooo0>Koo`006ooo00<00?ooool06_oo00<00?ooool0iOoo
000Kool00`00ooooo`0Jool00`00ooooo`3Uool000Soo`8000Coo`03003ooooo00;oo`@000Coo`03
003ooooo01[oo`03003ooooo0>Goo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo
01_oo`03003ooooo0>Coo`001ooo00@00?ooool000[oo`03003ooooo00?oo`8001coo`03003ooooo
0>Coo`0000?oo`0000000P000_oo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo01_oo`03
003ooooo0>Coo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo01_oo`03003ooooo
0>Coo`001ooo00@00?ooool000Soo`<000Goo`03003ooooo01coo`03003ooooo0>?oo`002?oo0P00
2ooo00<00?ooool00ooo00<00?ooool07?oo00<00?ooool0hooo000Kool00`00ooooo`0Lool00`00
ooooo`3Sool001_oo`8001goo`03003ooooo0>?oo`006ooo00<00?ooool07Ooo00<00?ooool0h_oo
000Kool00`00ooooo`0Mool00`00ooooo`3Rool001_oo`03003ooooo01goo`03003ooooo0>;oo`00
6ooo00<00?ooool07Ooo00<00?ooool0h_oo000Kool00`00ooooo`0Nool00`00ooooo`3Qool001_o
o`8001ooo`03003ooooo0>7oo`006ooo00<00?ooool07_oo00<00?ooool0hOoo000Kool00`00oooo
o`0Nool00`00ooooo`3Qool001_oo`03003ooooo01ooo`03003ooooo0>3oo`006ooo00<00?ooool0
7ooo00<00?ooool0h?oo000Kool00`00ooooo`0Oool00`00ooooo`3Pool001_oo`80023oo`03003o
oooo0>3oo`006ooo00<00?ooool08?oo00<00?ooool0gooo000Kool00`00ooooo`0Pool20004ool0
0`00ooooo`02ool3000Qool4000Pool40003ool00`00ooooo`02ool3000Rool4000Pool40003ool0
0`00ooooo`02ool3000Rool20007ool001_oo`03003ooooo01ooo`800003ool00?oo00[oo`03003o
oooo023oo`03003ooooo027oo`03003ooooo00[oo`03003ooooo01ooo`03003ooooo027oo`03003o
oooo00coo`03003ooooo01koo`04003ooooo0006ool001_oo`03003ooooo01ooo`800003ool00?oo
00[oo`03003ooooo023oo`03003ooooo027oo`03003ooooo00[oo`03003ooooo023oo`03003ooooo
027oo`03003ooooo00_oo`03003ooooo027oo`03003ooooo00Coo`006ooo00<00?ooool07ooo00@0
0?oo000000Soo`<002?oo`03003ooooo027oo`03003ooooo00Ooo`<002Coo`03003ooooo027oo`03
003ooooo00Ooo`<002;oo`8000Ooo`006ooo0P008?oo00@00?oo000000Soo`03003ooooo02?oo`03
003ooooo027oo`03003ooooo00Ooo`03003ooooo02Goo`03003ooooo027oo`03003ooooo00Koo`03
003ooooo02Coo`03003ooooo00Coo`006ooo00<00?ooool07ooo00@00?oo000000Soo`03003ooooo
027oo`<0027oo`<000Woo`03003ooooo02;oo`04003ooooo000Pool01000ooooo`002?oo00<00?oo
ool08Ooo00@00?ooool000Koo`006ooo00<00?ooool08?oo0P002Ooo10008_oo00<00?ooool08Ooo
00<00?ooool01ooo10008_oo0P008_oo0P002Ooo10008Ooo0P001ooo000Kool00`00ooooo`0Rool0
0`00ooooo`3Mool001_oo`03003ooooo02;oo`03003ooooo0=goo`006ooo00<00?ooool08_oo00<0
0?ooool0gOoo000Doooo000=000001_oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Koo`05003oooooool00003ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00oooo
o`06ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00oooo
o`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`05ool001_oo`03003ooooo02?oo`05
003oooooool0000Yool00`00ooooo`0Wool00`00ooooo`0Xool00`00ooooo`0Wool00`00ooooo`0W
ool00`00ooooo`05ool001_oo`03003ooooo02?oo`03003ooooo0=coo`006ooo00<00?ooool08ooo
00<00?ooool0g?oo000Kool00`00ooooo`0Tool00`00ooooo`3Kool001_oo`8002Goo`03003ooooo
0=_oo`006ooo00<00?ooool09?oo00<00?ooool0fooo000Kool00`00ooooo`0Tool00`00ooooo`3K
ool001_oo`03003ooooo02Goo`03003ooooo0=[oo`006ooo00<00?ooool09Ooo00<00?ooool0f_oo
000Kool00`00ooooo`0Uool00`00ooooo`3<ool70007ool001_oo`8002Koo`03003ooooo09Wooc<0
00koo`006ooo00<00?ooool09_oo00<00?ooool0N_oo7P00@Ooo000Kool00`00ooooo`0Vool00`00
ooooo`1UoolE001Oool001_oo`03003ooooo02Koo`03003ooooo05_oo`X007Coo`006ooo00<00?oo
ool09_oo00<00?ooool0DOoo2P00O_oo000Kool00`00ooooo`0Wool00`00ooooo`19ool70028ool0
01_oo`8002Soo`03003ooooo04?oo`H008ooo`006ooo00<00?ooool09ooo00<00?ooool0?_oo1@00
UOoo000Kool00`00ooooo`0Wool00`00ooooo`0iool5002Jool001_oo`03003ooooo02Soo`03003o
oooo03Coo`@009ooo`006ooo00<00?ooool0:?oo00<00?ooool0<_oo0P00Xooo000Kool00`00oooo
o`0Xool00`00ooooo`0_ool3002Uool001_oo`8002Woo`03003ooooo02goo`800:Soo`006ooo00<0
0?ooool0:Ooo00<00?ooool09?oo0`000_oo0`00Z_oo000Kool00`00ooooo`0Yool00`00ooooo`0S
ool200001?oo00000000[Ooo000Kool00`00ooooo`0Yool00`00ooooo`0Sool00`00ooooo`2`ool0
00Soo`8000Coo`03003ooooo00;oo`@000Coo`03003ooooo02[oo`03003ooooo02;oo`03003ooooo
0;3oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo02[oo`03003ooooo02;oo`03
003ooooo0;3oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`8002_oo`03003ooooo027oo`03
003ooooo0;7oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo02_oo`03003ooooo
023oo`03003ooooo0;7oo`001ooo00@00?ooool000[oo`03003ooooo00?oo`03003ooooo02_oo`03
003ooooo01ooo`03003ooooo0;;oo`001ooo00@00?ooool000Soo`<000Goo`03003ooooo02_oo`03
003ooooo01ooo`03003ooooo0;;oo`002?oo0P002ooo00<00?ooool00ooo00<00?ooool0:ooo00<0
0?ooool07_oo00<00?ooool0/ooo000Kool00`00ooooo`0/ool00`00ooooo`0Mool00`00ooooo`2c
ool001_oo`8002goo`03003ooooo01coo`03003ooooo0;Coo`006ooo00<00?ooool0;?oo00<00?oo
ool07?oo00<00?ooool0]?oo000Kool00`00ooooo`0]ool00`00ooooo`0Kool00`00ooooo`2dool0
01_oo`03003ooooo02goo`03003ooooo01[oo`03003ooooo0;Goo`006ooo00<00?ooool0;Ooo00<0
0?ooool06_oo00<00?ooool0]Ooo000Kool00`00ooooo`0^ool00`00ooooo`0Hool00`00ooooo`2f
ool001_oo`8002ooo`03003ooooo01Soo`03003ooooo0;Koo`006ooo00<00?ooool0;_oo00<00?oo
ool06?oo00<00?ooool0]_oo000Kool00`00ooooo`0_ool00`00ooooo`0Fool00`00ooooo`2gool0
01_oo`03003ooooo02ooo`03003ooooo01Koo`03003ooooo0;Ooo`006ooo00<00?ooool0<?oo00<0
0?ooool05Ooo00<00?ooool0]ooo000Kool00`00ooooo`0`ool00`00ooooo`0Dool00`00ooooo`2h
ool001_oo`8003;oo`03003ooooo01?oo`03003ooooo0;Soo`006ooo00<00?ooool0<Ooo00<00?oo
ool04_oo00<00?ooool0^Ooo000Kool00`00ooooo`0bool00`00ooooo`0Aool00`00ooooo`2iool0
01_oo`03003ooooo03;oo`03003ooooo013oo`03003ooooo0;[oo`006ooo00<00?ooool0<_oo00<0
0?ooool04?oo00<00?ooool0^_oo000Kool00`00ooooo`0cool00`00ooooo`0>ool00`00ooooo`2k
ool001_oo`8003Coo`03003ooooo00koo`03003ooooo0;_oo`006ooo00<00?ooool0=?oo00<00?oo
ool03?oo00<00?ooool0_?oo000Kool00`00ooooo`0eool00`00ooooo`0;ool00`00ooooo`2lool0
01_oo`03003ooooo03Goo`03003ooooo00[oo`03003ooooo0;goo`002?oo0P001?oo00<00?ooool0
0_oo10001?oo00<00?ooool0=_oo00<00?ooool02Ooo00<00?ooool0_Ooo0007ool01000ooooo`00
2?oo00<00?ooool01Ooo00<00?ooool0=_oo00<00?ooool02?oo00<00?ooool0__oo0007ool01000
ooooo`002Ooo00<00?ooool01?oo0P00>?oo00<00?ooool01_oo00<00?ooool0_ooo0007ool01000
ooooo`002_oo00<00?ooool00ooo00<00?ooool0>?oo0P001Ooo00<00?ooool0`?oo0007ool01000
ooooo`002ooo00<00?ooool00_oo00<00?ooool0>_oo1@00`ooo0007ool01000ooooo`002?oo00@0
0?ooool000Coo`03003ooooo0?ooo`?oo`002?oo0P002_oo0P001Ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo
8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.323594, -0.345669, \
0.0118419, 0.00331454}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[BoxData[
    \(also\ looks\ o . k . \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Im[Normal[%770[\([3, 3]\)]]], {z, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.562914 1.75829 [
[.18254 .55041 -9 -9 ]
[.18254 .55041 9 0 ]
[.34127 .55041 -3 -9 ]
[.34127 .55041 3 0 ]
[.5 .55041 -9 -9 ]
[.5 .55041 9 0 ]
[.65873 .55041 -3 -9 ]
[.65873 .55041 3 0 ]
[.81746 .55041 -9 -9 ]
[.81746 .55041 9 0 ]
[.97619 .55041 -3 -9 ]
[.97619 .55041 3 0 ]
[.01131 .03543 -24 -4.5 ]
[.01131 .03543 0 4.5 ]
[.01131 .12334 -30 -4.5 ]
[.01131 .12334 0 4.5 ]
[.01131 .21125 -24 -4.5 ]
[.01131 .21125 0 4.5 ]
[.01131 .29917 -30 -4.5 ]
[.01131 .29917 0 4.5 ]
[.01131 .38708 -24 -4.5 ]
[.01131 .38708 0 4.5 ]
[.01131 .475 -30 -4.5 ]
[.01131 .475 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .56291 m
.18254 .56916 L
s
[(0.5)] .18254 .55041 0 1 Mshowa
.34127 .56291 m
.34127 .56916 L
s
[(1)] .34127 .55041 0 1 Mshowa
.5 .56291 m
.5 .56916 L
s
[(1.5)] .5 .55041 0 1 Mshowa
.65873 .56291 m
.65873 .56916 L
s
[(2)] .65873 .55041 0 1 Mshowa
.81746 .56291 m
.81746 .56916 L
s
[(2.5)] .81746 .55041 0 1 Mshowa
.97619 .56291 m
.97619 .56916 L
s
[(3)] .97619 .55041 0 1 Mshowa
.125 Mabswid
.05556 .56291 m
.05556 .56666 L
s
.0873 .56291 m
.0873 .56666 L
s
.11905 .56291 m
.11905 .56666 L
s
.15079 .56291 m
.15079 .56666 L
s
.21429 .56291 m
.21429 .56666 L
s
.24603 .56291 m
.24603 .56666 L
s
.27778 .56291 m
.27778 .56666 L
s
.30952 .56291 m
.30952 .56666 L
s
.37302 .56291 m
.37302 .56666 L
s
.40476 .56291 m
.40476 .56666 L
s
.43651 .56291 m
.43651 .56666 L
s
.46825 .56291 m
.46825 .56666 L
s
.53175 .56291 m
.53175 .56666 L
s
.56349 .56291 m
.56349 .56666 L
s
.59524 .56291 m
.59524 .56666 L
s
.62698 .56291 m
.62698 .56666 L
s
.69048 .56291 m
.69048 .56666 L
s
.72222 .56291 m
.72222 .56666 L
s
.75397 .56291 m
.75397 .56666 L
s
.78571 .56291 m
.78571 .56666 L
s
.84921 .56291 m
.84921 .56666 L
s
.88095 .56291 m
.88095 .56666 L
s
.9127 .56291 m
.9127 .56666 L
s
.94444 .56291 m
.94444 .56666 L
s
.25 Mabswid
0 .56291 m
1 .56291 L
s
.02381 .03543 m
.03006 .03543 L
s
[(-0.3)] .01131 .03543 1 0 Mshowa
.02381 .12334 m
.03006 .12334 L
s
[(-0.25)] .01131 .12334 1 0 Mshowa
.02381 .21125 m
.03006 .21125 L
s
[(-0.2)] .01131 .21125 1 0 Mshowa
.02381 .29917 m
.03006 .29917 L
s
[(-0.15)] .01131 .29917 1 0 Mshowa
.02381 .38708 m
.03006 .38708 L
s
[(-0.1)] .01131 .38708 1 0 Mshowa
.02381 .475 m
.03006 .475 L
s
[(-0.05)] .01131 .475 1 0 Mshowa
.125 Mabswid
.02381 .05301 m
.02756 .05301 L
s
.02381 .07059 m
.02756 .07059 L
s
.02381 .08817 m
.02756 .08817 L
s
.02381 .10576 m
.02756 .10576 L
s
.02381 .14092 m
.02756 .14092 L
s
.02381 .15851 m
.02756 .15851 L
s
.02381 .17609 m
.02756 .17609 L
s
.02381 .19367 m
.02756 .19367 L
s
.02381 .22884 m
.02756 .22884 L
s
.02381 .24642 m
.02756 .24642 L
s
.02381 .264 m
.02756 .264 L
s
.02381 .28159 m
.02756 .28159 L
s
.02381 .31675 m
.02756 .31675 L
s
.02381 .33434 m
.02756 .33434 L
s
.02381 .35192 m
.02756 .35192 L
s
.02381 .3695 m
.02756 .3695 L
s
.02381 .40467 m
.02756 .40467 L
s
.02381 .42225 m
.02756 .42225 L
s
.02381 .43983 m
.02756 .43983 L
s
.02381 .45742 m
.02756 .45742 L
s
.02381 .49258 m
.02756 .49258 L
s
.02381 .51016 m
.02756 .51016 L
s
.02381 .52775 m
.02756 .52775 L
s
.02381 .54533 m
.02756 .54533 L
s
.02381 .01784 m
.02756 .01784 L
s
.02381 .00026 m
.02756 .00026 L
s
.02381 .5805 m
.02756 .5805 L
s
.02381 .59808 m
.02756 .59808 L
s
.02381 .61566 m
.02756 .61566 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .36057 m
.07374 .25567 L
.0929 .16056 L
.11234 .08541 L
.12336 .054 L
.12829 .04281 L
.13363 .03273 L
.13891 .02494 L
.14123 .0222 L
.14378 .01967 L
.14602 .01788 L
.14843 .01639 L
.14977 .01577 L
.15098 .01533 L
.15221 .015 L
.15337 .0148 L
.1545 .01472 L
.15552 .01472 L
.15671 .01484 L
.15782 .01505 L
.15886 .01533 L
.15981 .01567 L
.16196 .01669 L
.16413 .01811 L
.16651 .0201 L
.17133 .02548 L
.17639 .03312 L
.18119 .04219 L
.19019 .06386 L
.2104 .13309 L
.22965 .22083 L
.24746 .31427 L
.26703 .42045 L
.28541 .5111 L
.29397 .54619 L
.30297 .57574 L
.30769 .58748 L
.31019 .5925 L
.31285 .59688 L
.31411 .59858 L
.31531 .59996 L
.31647 .60109 L
.31754 .60193 L
.31856 .60255 L
.31964 .60301 L
.32072 .60327 L
.32191 .60332 L
.32316 .60309 L
.32432 .60261 L
.32559 .60178 L
Mistroke
.32694 .60055 L
.32824 .59903 L
.32946 .59729 L
.33173 .59318 L
.33604 .58219 L
.33847 .57408 L
.33964 .56966 L
.3407 .56533 L
.34188 .56291 L
.34313 .56291 L
.34539 .56291 L
.35053 .56291 L
.3609 .56291 L
.39947 .56291 L
.43657 .56291 L
.47604 .56291 L
.51405 .56291 L
.55059 .56291 L
.58951 .56291 L
.62696 .56291 L
.66678 .56291 L
.70513 .56291 L
.74202 .56291 L
.78128 .56291 L
.81907 .56291 L
.85924 .56291 L
.89794 .56291 L
.93517 .56291 L
.97478 .56291 L
.97619 .56291 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`800?go
o`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`008Ooo0P00
7_oo1@00f_oo000Qool00`00ooooo`0Kool20005ool00`00ooooo`3Gool0027oo`03003ooooo01_o
o`03003ooooo00Goo`03003ooooo0=Koo`003_oo0P001?oo00<00?ooool00ooo0P001Ooo00<00?oo
ool06_oo00<00?ooool01_oo00<00?ooool0e_oo000=ool01000ooooo`002?oo00@00?ooool000Co
o`03003ooooo01Woo`03003ooooo00Soo`03003ooooo0=Goo`003Ooo00@00?ooool000_oo`03003o
oooo00;oo`8001[oo`03003ooooo00Woo`03003ooooo0=Coo`001ooo10000_oo00@00?ooool000Wo
o`8000Goo`03003ooooo01Soo`03003ooooo00[oo`03003ooooo0=Coo`003Ooo00@00?ooool000_o
o`03003ooooo00;oo`03003ooooo01Soo`03003ooooo00_oo`03003ooooo0=?oo`003Ooo00@00?oo
ool000Soo`04003ooooo0004ool00`00ooooo`0Hool00`00ooooo`0;ool00`00ooooo`3Cool000ko
o`8000[oo`8000Goo`8001Soo`03003ooooo00goo`03003ooooo0=;oo`008Ooo00<00?ooool05ooo
00<00?ooool03Ooo00<00?ooool0d_oo000Qool00`00ooooo`0Gool00`00ooooo`0>ool00`00oooo
o`3Aool0027oo`03003ooooo01Koo`03003ooooo00ooo`03003ooooo0=7oo`008Ooo00<00?ooool0
5_oo00<00?ooool03ooo00<00?ooool0dOoo000Qool2000Gool00`00ooooo`0@ool00`00ooooo`3@
ool0027oo`03003ooooo01Goo`03003ooooo017oo`03003ooooo0=3oo`008Ooo00<00?ooool05Ooo
00<00?ooool04Ooo00<00?ooool0d?oo000Qool00`00ooooo`0Dool00`00ooooo`0Bool00`00oooo
o`3@ool0027oo`03003ooooo01Coo`03003ooooo01?oo`03003ooooo0<ooo`008Ooo0P005Ooo00<0
0?ooool04ooo00<00?ooool0cooo000Qool00`00ooooo`0Cool00`00ooooo`0Dool00`00ooooo`3?
ool0027oo`03003ooooo01?oo`03003ooooo01Coo`03003ooooo0<ooo`008Ooo00<00?ooool04ooo
00<00?ooool05Ooo00<00?ooool0c_oo000Qool2000Dool00`00ooooo`0Eool00`00ooooo`3>ool0
027oo`03003ooooo01;oo`03003ooooo01Koo`03003ooooo0<koo`008Ooo00<00?ooool04_oo00<0
0?ooool05ooo00<00?ooool0cOoo0008ool20004ool00`00ooooo`02ool40002ool30005ool00`00
ooooo`0Bool00`00ooooo`0Gool00`00ooooo`3=ool000Ooo`04003ooooo0008ool00`00ooooo`06
ool00`00ooooo`02ool00`00ooooo`0Bool00`00ooooo`0Gool00`00ooooo`3=ool000Ooo`04003o
oooo0009ool00`00ooooo`05ool00`00ooooo`02ool2000Bool00`00ooooo`0Hool00`00ooooo`3=
ool00003ool00000008000;oo`04003ooooo000:ool01000oooooooo0`001Ooo00<00?ooool04Ooo
00<00?ooool06Ooo00<00?ooool0c?oo0007ool01000ooooo`002ooo00@00?ooool000Ooo`03003o
oooo017oo`03003ooooo01Woo`03003ooooo0<coo`001ooo00@00?ooool000Soo`04003ooooo0002
ool00`00ooooo`05ool00`00ooooo`0Aool00`00ooooo`0Iool00`00ooooo`3<ool000Soo`8000[o
o`8000?oo`@000Coo`80017oo`03003ooooo01[oo`03003ooooo0<coo`008Ooo00<00?ooool04?oo
00<00?ooool06ooo00<00?ooool0booo000Qool00`00ooooo`0@ool00`00ooooo`0Kool00`00oooo
o`3;ool0027oo`03003ooooo013oo`03003ooooo01_oo`03003ooooo0<_oo`008Ooo00<00?ooool0
3ooo00<00?ooool07?oo00<00?ooool0booo000Qool2000@ool00`00ooooo`0Mool00`00ooooo`3:
ool0027oo`03003ooooo00ooo`03003ooooo01goo`03003ooooo0<[oo`008Ooo00<00?ooool03ooo
00<00?ooool07Ooo00<00?ooool0b_oo000Qool00`00ooooo`0>ool00`00ooooo`0Nool00`00oooo
o`3:ool0027oo`8000ooo`03003ooooo01koo`03003ooooo0<[oo`008Ooo00<00?ooool03_oo00<0
0?ooool07ooo00<00?ooool0bOoo000Qool00`00ooooo`0>ool00`00ooooo`0Oool00`00ooooo`39
ool0027oo`03003ooooo00koo`03003ooooo01ooo`03003ooooo0<Woo`008Ooo00<00?ooool03Ooo
00<00?ooool08?oo00<00?ooool0bOoo000Qool2000>ool00`00ooooo`0Pool00`00ooooo`39ool0
027oo`03003ooooo00goo`03003ooooo027oo`03003ooooo0<Soo`008Ooo00<00?ooool03Ooo00<0
0?ooool08Ooo00<00?ooool0b?oo000>ool20004ool00`00ooooo`02ool40004ool00`00ooooo`0=
ool00`00ooooo`0Qool00`00ooooo`38ool000goo`04003ooooo0008ool00`00ooooo`05ool00`00
ooooo`0<ool00`00ooooo`0Rool00`00ooooo`38ool000goo`04003ooooo0009ool00`00ooooo`04
ool2000=ool00`00ooooo`0Sool00`00ooooo`37ool000Ooo`@000;oo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`0<ool00`00ooooo`0Sool00`00ooooo`37ool000goo`04003ooooo000;
ool00`00ooooo`02ool00`00ooooo`0<ool00`00ooooo`0Sool00`00ooooo`37ool000goo`04003o
oooo0008ool01000ooooo`001?oo00<00?ooool03?oo00<00?ooool08ooo00<00?ooool0aooo000>
ool2000:ool20005ool2000<ool00`00ooooo`0Tool00`00ooooo`37ool0027oo`03003ooooo00_o
o`03003ooooo02Goo`03003ooooo0<Koo`008Ooo00<00?ooool02ooo00<00?ooool09Ooo00<00?oo
ool0a_oo000Qool00`00ooooo`0;ool00`00ooooo`0Uool00`00ooooo`36ool0027oo`03003ooooo
00_oo`03003ooooo02Goo`03003ooooo0<Koo`008Ooo0P002ooo00<00?ooool09_oo00<00?ooool0
a_oo000Qool00`00ooooo`0:ool00`00ooooo`0Wool00`00ooooo`35ool0027oo`03003ooooo00[o
o`03003ooooo02Ooo`03003ooooo0<Goo`008Ooo00<00?ooool02_oo00<00?ooool09ooo00<00?oo
ool0aOoo000Qool2000;ool00`00ooooo`0Wool00`00ooooo`35ool0027oo`03003ooooo00Woo`03
003ooooo02Soo`03003ooooo0<Goo`008Ooo00<00?ooool02Ooo00<00?ooool0:Ooo00<00?ooool0
a?oo000Qool00`00ooooo`09ool00`00ooooo`0Yool00`00ooooo`34ool0027oo`03003ooooo00Wo
o`03003ooooo02Woo`03003ooooo0<Coo`008Ooo0P002_oo00<00?ooool0:Ooo00<00?ooool0a?oo
000Qool00`00ooooo`09ool00`00ooooo`0Zool00`00ooooo`33ool000Soo`8000Coo`03003ooooo
00;oo`@000;oo`<000Goo`03003ooooo00Soo`03003ooooo02_oo`03003ooooo0<?oo`001ooo00@0
0?ooool000[oo`03003ooooo00Coo`03003ooooo00;oo`03003ooooo00Soo`03003ooooo02_oo`03
003ooooo0<?oo`001ooo00@00?ooool000[oo`03003ooooo00Coo`03003ooooo00;oo`8000Woo`03
003ooooo02_oo`03003ooooo0<?oo`0000?oo`0000000P000_oo00@00?ooool000[oo`04003ooooo
ool30005ool00`00ooooo`08ool00`00ooooo`0[ool00`00ooooo`33ool000Ooo`04003ooooo000:
ool01@00oooooooo00001ooo00<00?ooool02?oo00<00?ooool0;?oo00<00?ooool0`_oo0007ool0
1000ooooo`002?oo0`000ooo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool0;Ooo00<00?oo
ool0`_oo0008ool2000;ool01000oooooooo10001?oo00<00?ooool01ooo00<00?ooool0;Ooo00<0
0?ooool0`_oo000Qool20008ool00`00ooooo`0]ool00`00ooooo`32ool0027oo`03003ooooo00Oo
o`03003ooooo02goo`03003ooooo0<;oo`008Ooo00<00?ooool01ooo00<00?ooool0;_oo00<00?oo
ool0`Ooo000Qool00`00ooooo`06ool00`00ooooo`0_ool00`00ooooo`31ool0027oo`03003ooooo
00Koo`03003ooooo02ooo`03003ooooo0<7oo`008Ooo0P001ooo00<00?ooool0;ooo00<00?ooool0
`Ooo000Qool00`00ooooo`06ool00`00ooooo`0_ool00`00ooooo`31ool0027oo`03003ooooo00Ko
o`03003ooooo02ooo`03003ooooo0<7oo`008Ooo00<00?ooool01_oo00<00?ooool0<?oo00<00?oo
ool0`?oo000Qool20006ool00`00ooooo`0aool00`00ooooo`30ool0027oo`03003ooooo00Goo`03
003ooooo037oo`03003ooooo0<3oo`008Ooo00<00?ooool01Ooo00<00?ooool0<Ooo00<00?ooool0
`?oo000Qool00`00ooooo`0iool00`00ooooo`30ool0027oo`03003ooooo03Woo`03003ooooo0<3o
o`008Ooo0P00>ooo00<00?ooool0_ooo000Qool00`00ooooo`0jool00`00ooooo`2oool000koo`80
00Coo`03003ooooo00;oo`@000Coo`03003ooooo03[oo`03003ooooo0;ooo`003Ooo00@00?ooool0
00[oo`03003ooooo00?oo`03003ooooo03[oo`03003ooooo0;ooo`003Ooo00@00?ooool000[oo`03
003ooooo00?oo`8003_oo`03003ooooo0;ooo`001ooo10000_oo00@00?ooool000[oo`03003ooooo
00?oo`03003ooooo03_oo`03003ooooo0;koo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`03
003ooooo03_oo`03003ooooo0;koo`003Ooo00@00?ooool000Soo`<000Goo`03003ooooo03_oo`03
003ooooo0;koo`003_oo0P002ooo00<00?ooool00ooo00<00?ooool0>ooo00<00?ooool0__oo000Q
ool2000lool00`00ooooo`2nool0027oo`03003ooooo03_oo`03003ooooo0;koo`008Ooo00<00?oo
ool0??oo00<00?ooool0_Ooo000Qool00`00ooooo`0lool00`00ooooo`2mool0027oo`8003goo`03
003ooooo0;goo`008Ooo00<00?ooool0??oo00<00?ooool0_Ooo000Qool00`00ooooo`0lool00`00
ooooo`2mool0027oo`03003ooooo03goo`03003ooooo0;coo`008Ooo00<00?ooool0?Ooo00<00?oo
ool0_?oo000Qool2000nool00`00ooooo`2lool0027oo`03003ooooo03goo`03003ooooo0;coo`00
8Ooo00<00?ooool0?Ooo00<00?ooool0_?oo000Qool00`00ooooo`0nool00`00ooooo`2kool0027o
o`03003ooooo03koo`03003ooooo0;_oo`008Ooo0P00?ooo00<00?ooool0^ooo000Qool00`00oooo
o`0nool00`00ooooo`2kool000Soo`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo
03koo`03003ooooo0;_oo`001ooo00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`02ool0
0`00ooooo`0oool00`00ooooo`2jool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?oo
ool00_oo0P00@?oo00<00?ooool0^_oo00000ooo000000020002ool01000ooooo`002?oo00@00?oo
ool000;oo`<000Goo`03003ooooo03ooo`03003ooooo0;[oo`001ooo00@00?ooool000Soo`04003o
oooo0002ool00`00ooooo`05ool00`00ooooo`0oool00`00ooooo`2jool000Ooo`04003ooooo0008
ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0@?oo00<00?ooool0^Ooo0008ool2000:
ool20003ool40004ool00`00ooooo`10ool00`00ooooo`2iool0027oo`80047oo`03003ooooo0;Wo
o`008Ooo00<00?ooool0@?oo00<00?ooool0^Ooo000Qool00`00ooooo`10ool00`00ooooo`2iool0
027oo`03003ooooo047oo`03003ooooo0;Soo`008Ooo0P00@_oo00<00?ooool0^?oo000Qool00`00
ooooo`11ool00`00ooooo`2hool0027oo`03003ooooo047oo`03003ooooo0;Soo`008Ooo00<00?oo
ool0@Ooo00<00?ooool0^?oo000Qool00`00ooooo`0Oool20004ool00`00ooooo`02ool3000Eool0
0`00ooooo`08ool4000Pool40003ool00`00ooooo`02ool3000Pool4000Oool40003ool00`00oooo
o`02ool3000Qool20007ool0027oo`8001ooo`04003ooooo000;ool00`00ooooo`0Bool00`00oooo
o`0:ool00`00ooooo`0Qool00`00ooooo`0:ool00`00ooooo`0Mool00`00ooooo`0Pool00`00oooo
o`0<ool00`00ooooo`0Mool01000ooooo`001_oo000Qool00`00ooooo`0Nool01000ooooo`002ooo
00<00?ooool04_oo00<00?ooool02_oo00<00?ooool08Ooo00<00?ooool02_oo00<00?ooool07_oo
00<00?ooool08?oo00<00?ooool02ooo00<00?ooool08?oo00<00?ooool01?oo000Qool00`00oooo
o`0Nool01000ooooo`002?oo0`005Ooo00<00?ooool02_oo00<00?ooool08Ooo00<00?ooool01ooo
0`008_oo00<00?ooool08?oo00<00?ooool01ooo0`008Ooo0P001ooo000Qool00`00ooooo`0Nool0
1000ooooo`002?oo00<00?ooool05_oo00<00?ooool02Ooo00<00?ooool08Ooo00<00?ooool01ooo
00<00?ooool08ooo00<00?ooool08?oo00<00?ooool01_oo00<00?ooool08ooo00<00?ooool01?oo
000Qool2000Oool01000ooooo`002?oo00<00?ooool05_oo00<00?ooool01ooo0`008Ooo0`002Ooo
00<00?ooool08?oo00@00?ooool001ooo`04003ooooo0008ool00`00ooooo`0Pool01000ooooo`00
1_oo000Qool00`00ooooo`0Oool20009ool4000Eool00`00ooooo`09ool00`00ooooo`0Qool00`00
ooooo`07ool4000Pool2000Qool20009ool4000Pool20007ool0027oo`03003ooooo04?oo`03003o
oooo0;Koo`008Ooo00<00?ooool0A?oo00<00?ooool0]Ooo000Qool00`00ooooo`14ool00`00oooo
o`2eool001_oool000H000008Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool00_oo00@00?ooool000Ooo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`008Ooo00<00?ooool09_oo00<00?ooool0
7?oo00<00?ooool01ooo00<00?ooool09ooo00<00?ooool09_oo00<00?ooool09_oo00<00?ooool0
9_oo00<00?ooool01Ooo000Qool00`00ooooo`15ool00`00ooooo`07ool00`00ooooo`2Zool0027o
o`03003ooooo04Koo`03003ooooo00Koo`03003ooooo0:[oo`008Ooo0P00Aooo00<00?ooool01Ooo
00<00?ooool0Zooo000Qool00`00ooooo`17ool00`00ooooo`04ool00`00ooooo`2[ool0027oo`03
003ooooo04Ooo`03003ooooo00?oo`03003ooooo0:coo`008Ooo00<00?ooool0Aooo00<00?ooool0
0ooo00<00?ooool0[?oo000Qool20019ool01@00oooooooo0000[ooo000Qool00`00ooooo`19ool4
002_ool0027oo`03003ooooo04[oo`03003ooooo0:ooo`008Ooo00<00?ooool0o?oo000Qool00`00
ooooo`3lool0027oo`800?goo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0
0?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.402939, -0.337968, \
0.0121183, 0.00218797}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[\((Im[
            SXiA[z, Cos[10^\((\(-4\))\)], 0.01] - 
              SXiA[z, Cos[10^\((\(-4\))\)], 0]])\)/0.01, {z, 0.1, 
        3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.562914 1.75829 [
[.18254 .55041 -9 -9 ]
[.18254 .55041 9 0 ]
[.34127 .55041 -3 -9 ]
[.34127 .55041 3 0 ]
[.5 .55041 -9 -9 ]
[.5 .55041 9 0 ]
[.65873 .55041 -3 -9 ]
[.65873 .55041 3 0 ]
[.81746 .55041 -9 -9 ]
[.81746 .55041 9 0 ]
[.97619 .55041 -3 -9 ]
[.97619 .55041 3 0 ]
[.01131 .03543 -24 -4.5 ]
[.01131 .03543 0 4.5 ]
[.01131 .12334 -30 -4.5 ]
[.01131 .12334 0 4.5 ]
[.01131 .21125 -24 -4.5 ]
[.01131 .21125 0 4.5 ]
[.01131 .29917 -30 -4.5 ]
[.01131 .29917 0 4.5 ]
[.01131 .38708 -24 -4.5 ]
[.01131 .38708 0 4.5 ]
[.01131 .475 -30 -4.5 ]
[.01131 .475 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .56291 m
.18254 .56916 L
s
[(0.5)] .18254 .55041 0 1 Mshowa
.34127 .56291 m
.34127 .56916 L
s
[(1)] .34127 .55041 0 1 Mshowa
.5 .56291 m
.5 .56916 L
s
[(1.5)] .5 .55041 0 1 Mshowa
.65873 .56291 m
.65873 .56916 L
s
[(2)] .65873 .55041 0 1 Mshowa
.81746 .56291 m
.81746 .56916 L
s
[(2.5)] .81746 .55041 0 1 Mshowa
.97619 .56291 m
.97619 .56916 L
s
[(3)] .97619 .55041 0 1 Mshowa
.125 Mabswid
.05556 .56291 m
.05556 .56666 L
s
.0873 .56291 m
.0873 .56666 L
s
.11905 .56291 m
.11905 .56666 L
s
.15079 .56291 m
.15079 .56666 L
s
.21429 .56291 m
.21429 .56666 L
s
.24603 .56291 m
.24603 .56666 L
s
.27778 .56291 m
.27778 .56666 L
s
.30952 .56291 m
.30952 .56666 L
s
.37302 .56291 m
.37302 .56666 L
s
.40476 .56291 m
.40476 .56666 L
s
.43651 .56291 m
.43651 .56666 L
s
.46825 .56291 m
.46825 .56666 L
s
.53175 .56291 m
.53175 .56666 L
s
.56349 .56291 m
.56349 .56666 L
s
.59524 .56291 m
.59524 .56666 L
s
.62698 .56291 m
.62698 .56666 L
s
.69048 .56291 m
.69048 .56666 L
s
.72222 .56291 m
.72222 .56666 L
s
.75397 .56291 m
.75397 .56666 L
s
.78571 .56291 m
.78571 .56666 L
s
.84921 .56291 m
.84921 .56666 L
s
.88095 .56291 m
.88095 .56666 L
s
.9127 .56291 m
.9127 .56666 L
s
.94444 .56291 m
.94444 .56666 L
s
.25 Mabswid
0 .56291 m
1 .56291 L
s
.02381 .03543 m
.03006 .03543 L
s
[(-0.3)] .01131 .03543 1 0 Mshowa
.02381 .12334 m
.03006 .12334 L
s
[(-0.25)] .01131 .12334 1 0 Mshowa
.02381 .21125 m
.03006 .21125 L
s
[(-0.2)] .01131 .21125 1 0 Mshowa
.02381 .29917 m
.03006 .29917 L
s
[(-0.15)] .01131 .29917 1 0 Mshowa
.02381 .38708 m
.03006 .38708 L
s
[(-0.1)] .01131 .38708 1 0 Mshowa
.02381 .475 m
.03006 .475 L
s
[(-0.05)] .01131 .475 1 0 Mshowa
.125 Mabswid
.02381 .05301 m
.02756 .05301 L
s
.02381 .07059 m
.02756 .07059 L
s
.02381 .08817 m
.02756 .08817 L
s
.02381 .10576 m
.02756 .10576 L
s
.02381 .14092 m
.02756 .14092 L
s
.02381 .15851 m
.02756 .15851 L
s
.02381 .17609 m
.02756 .17609 L
s
.02381 .19367 m
.02756 .19367 L
s
.02381 .22884 m
.02756 .22884 L
s
.02381 .24642 m
.02756 .24642 L
s
.02381 .264 m
.02756 .264 L
s
.02381 .28159 m
.02756 .28159 L
s
.02381 .31675 m
.02756 .31675 L
s
.02381 .33434 m
.02756 .33434 L
s
.02381 .35192 m
.02756 .35192 L
s
.02381 .3695 m
.02756 .3695 L
s
.02381 .40467 m
.02756 .40467 L
s
.02381 .42225 m
.02756 .42225 L
s
.02381 .43983 m
.02756 .43983 L
s
.02381 .45742 m
.02756 .45742 L
s
.02381 .49258 m
.02756 .49258 L
s
.02381 .51016 m
.02756 .51016 L
s
.02381 .52775 m
.02756 .52775 L
s
.02381 .54533 m
.02756 .54533 L
s
.02381 .01784 m
.02756 .01784 L
s
.02381 .00026 m
.02756 .00026 L
s
.02381 .5805 m
.02756 .5805 L
s
.02381 .59808 m
.02756 .59808 L
s
.02381 .61566 m
.02756 .61566 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .36057 m
.07374 .25567 L
.0929 .16056 L
.11234 .08541 L
.12336 .054 L
.12829 .04281 L
.13363 .03273 L
.13891 .02494 L
.14123 .0222 L
.14378 .01967 L
.14602 .01788 L
.14843 .01639 L
.14977 .01577 L
.15098 .01533 L
.15221 .015 L
.15337 .0148 L
.1545 .01472 L
.15552 .01472 L
.15671 .01484 L
.15782 .01505 L
.15886 .01533 L
.15981 .01567 L
.16196 .01669 L
.16413 .01811 L
.16651 .0201 L
.17133 .02548 L
.17639 .03312 L
.18119 .04219 L
.19019 .06386 L
.2104 .13309 L
.22965 .22083 L
.24746 .31427 L
.26703 .42045 L
.28541 .5111 L
.29397 .54619 L
.30297 .57574 L
.30769 .58748 L
.31019 .5925 L
.31285 .59688 L
.31411 .59858 L
.31531 .59996 L
.31647 .60109 L
.31754 .60193 L
.31856 .60255 L
.31964 .60301 L
.32072 .60327 L
.32191 .60332 L
.32316 .60309 L
.32432 .60261 L
.32559 .60178 L
Mistroke
.32694 .60055 L
.32824 .59903 L
.32946 .59729 L
.33173 .59318 L
.33604 .58219 L
.33847 .57408 L
.33964 .56966 L
.3407 .56533 L
.34188 .56291 L
.34313 .56291 L
.34539 .56291 L
.35053 .56291 L
.3609 .56291 L
.39947 .56291 L
.43657 .56291 L
.47604 .56291 L
.51405 .56291 L
.55059 .56291 L
.58951 .56291 L
.62696 .56291 L
.66678 .56291 L
.70513 .56291 L
.74202 .56291 L
.78128 .56291 L
.81907 .56291 L
.85924 .56291 L
.89794 .56291 L
.93517 .56291 L
.97478 .56291 L
.97619 .56291 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0027oo`800?go
o`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`008Ooo0P00
7_oo1@00f_oo000Qool00`00ooooo`0Kool20005ool00`00ooooo`3Gool0027oo`03003ooooo01_o
o`03003ooooo00Goo`03003ooooo0=Koo`003_oo0P001?oo00<00?ooool00ooo0P001Ooo00<00?oo
ool06_oo00<00?ooool01_oo00<00?ooool0e_oo000=ool01000ooooo`002?oo00@00?ooool000Co
o`03003ooooo01Woo`03003ooooo00Soo`03003ooooo0=Goo`003Ooo00@00?ooool000_oo`03003o
oooo00;oo`8001[oo`03003ooooo00Woo`03003ooooo0=Coo`001ooo10000_oo00@00?ooool000Wo
o`8000Goo`03003ooooo01Soo`03003ooooo00[oo`03003ooooo0=Coo`003Ooo00@00?ooool000_o
o`03003ooooo00;oo`03003ooooo01Soo`03003ooooo00_oo`03003ooooo0=?oo`003Ooo00@00?oo
ool000Soo`04003ooooo0004ool00`00ooooo`0Hool00`00ooooo`0;ool00`00ooooo`3Cool000ko
o`8000[oo`8000Goo`8001Soo`03003ooooo00goo`03003ooooo0=;oo`008Ooo00<00?ooool05ooo
00<00?ooool03Ooo00<00?ooool0d_oo000Qool00`00ooooo`0Gool00`00ooooo`0>ool00`00oooo
o`3Aool0027oo`03003ooooo01Koo`03003ooooo00ooo`03003ooooo0=7oo`008Ooo00<00?ooool0
5_oo00<00?ooool03ooo00<00?ooool0dOoo000Qool2000Gool00`00ooooo`0@ool00`00ooooo`3@
ool0027oo`03003ooooo01Goo`03003ooooo017oo`03003ooooo0=3oo`008Ooo00<00?ooool05Ooo
00<00?ooool04Ooo00<00?ooool0d?oo000Qool00`00ooooo`0Dool00`00ooooo`0Bool00`00oooo
o`3@ool0027oo`03003ooooo01Coo`03003ooooo01?oo`03003ooooo0<ooo`008Ooo0P005Ooo00<0
0?ooool04ooo00<00?ooool0cooo000Qool00`00ooooo`0Cool00`00ooooo`0Dool00`00ooooo`3?
ool0027oo`03003ooooo01?oo`03003ooooo01Coo`03003ooooo0<ooo`008Ooo00<00?ooool04ooo
00<00?ooool05Ooo00<00?ooool0c_oo000Qool2000Dool00`00ooooo`0Eool00`00ooooo`3>ool0
027oo`03003ooooo01;oo`03003ooooo01Koo`03003ooooo0<koo`008Ooo00<00?ooool04_oo00<0
0?ooool05ooo00<00?ooool0cOoo0008ool20004ool00`00ooooo`02ool40002ool30005ool00`00
ooooo`0Bool00`00ooooo`0Gool00`00ooooo`3=ool000Ooo`04003ooooo0008ool00`00ooooo`06
ool00`00ooooo`02ool00`00ooooo`0Bool00`00ooooo`0Gool00`00ooooo`3=ool000Ooo`04003o
oooo0009ool00`00ooooo`05ool00`00ooooo`02ool2000Bool00`00ooooo`0Hool00`00ooooo`3=
ool00003ool00000008000;oo`04003ooooo000:ool01000oooooooo0`001Ooo00<00?ooool04Ooo
00<00?ooool06Ooo00<00?ooool0c?oo0007ool01000ooooo`002ooo00@00?ooool000Ooo`03003o
oooo017oo`03003ooooo01Woo`03003ooooo0<coo`001ooo00@00?ooool000Soo`04003ooooo0002
ool00`00ooooo`05ool00`00ooooo`0Aool00`00ooooo`0Iool00`00ooooo`3<ool000Soo`8000[o
o`8000?oo`@000Coo`80017oo`03003ooooo01[oo`03003ooooo0<coo`008Ooo00<00?ooool04?oo
00<00?ooool06ooo00<00?ooool0booo000Qool00`00ooooo`0@ool00`00ooooo`0Kool00`00oooo
o`3;ool0027oo`03003ooooo013oo`03003ooooo01_oo`03003ooooo0<_oo`008Ooo00<00?ooool0
3ooo00<00?ooool07?oo00<00?ooool0booo000Qool2000@ool00`00ooooo`0Mool00`00ooooo`3:
ool0027oo`03003ooooo00ooo`03003ooooo01goo`03003ooooo0<[oo`008Ooo00<00?ooool03ooo
00<00?ooool07Ooo00<00?ooool0b_oo000Qool00`00ooooo`0>ool00`00ooooo`0Nool00`00oooo
o`3:ool0027oo`8000ooo`03003ooooo01koo`03003ooooo0<[oo`008Ooo00<00?ooool03_oo00<0
0?ooool07ooo00<00?ooool0bOoo000Qool00`00ooooo`0>ool00`00ooooo`0Oool00`00ooooo`39
ool0027oo`03003ooooo00koo`03003ooooo01ooo`03003ooooo0<Woo`008Ooo00<00?ooool03Ooo
00<00?ooool08?oo00<00?ooool0bOoo000Qool2000>ool00`00ooooo`0Pool00`00ooooo`39ool0
027oo`03003ooooo00goo`03003ooooo027oo`03003ooooo0<Soo`008Ooo00<00?ooool03Ooo00<0
0?ooool08Ooo00<00?ooool0b?oo000>ool20004ool00`00ooooo`02ool40004ool00`00ooooo`0=
ool00`00ooooo`0Qool00`00ooooo`38ool000goo`04003ooooo0008ool00`00ooooo`05ool00`00
ooooo`0<ool00`00ooooo`0Rool00`00ooooo`38ool000goo`04003ooooo0009ool00`00ooooo`04
ool2000=ool00`00ooooo`0Sool00`00ooooo`37ool000Ooo`@000;oo`04003ooooo000:ool00`00
ooooo`03ool00`00ooooo`0<ool00`00ooooo`0Sool00`00ooooo`37ool000goo`04003ooooo000;
ool00`00ooooo`02ool00`00ooooo`0<ool00`00ooooo`0Sool00`00ooooo`37ool000goo`04003o
oooo0008ool01000ooooo`001?oo00<00?ooool03?oo00<00?ooool08ooo00<00?ooool0aooo000>
ool2000:ool20005ool2000<ool00`00ooooo`0Tool00`00ooooo`37ool0027oo`03003ooooo00_o
o`03003ooooo02Goo`03003ooooo0<Koo`008Ooo00<00?ooool02ooo00<00?ooool09Ooo00<00?oo
ool0a_oo000Qool00`00ooooo`0;ool00`00ooooo`0Uool00`00ooooo`36ool0027oo`03003ooooo
00_oo`03003ooooo02Goo`03003ooooo0<Koo`008Ooo0P002ooo00<00?ooool09_oo00<00?ooool0
a_oo000Qool00`00ooooo`0:ool00`00ooooo`0Wool00`00ooooo`35ool0027oo`03003ooooo00[o
o`03003ooooo02Ooo`03003ooooo0<Goo`008Ooo00<00?ooool02_oo00<00?ooool09ooo00<00?oo
ool0aOoo000Qool2000;ool00`00ooooo`0Wool00`00ooooo`35ool0027oo`03003ooooo00Woo`03
003ooooo02Soo`03003ooooo0<Goo`008Ooo00<00?ooool02Ooo00<00?ooool0:Ooo00<00?ooool0
a?oo000Qool00`00ooooo`09ool00`00ooooo`0Yool00`00ooooo`34ool0027oo`03003ooooo00Wo
o`03003ooooo02Woo`03003ooooo0<Coo`008Ooo0P002_oo00<00?ooool0:Ooo00<00?ooool0a?oo
000Qool00`00ooooo`09ool00`00ooooo`0Zool00`00ooooo`33ool000Soo`8000Coo`03003ooooo
00;oo`@000;oo`<000Goo`03003ooooo00Soo`03003ooooo02_oo`03003ooooo0<?oo`001ooo00@0
0?ooool000[oo`03003ooooo00Coo`03003ooooo00;oo`03003ooooo00Soo`03003ooooo02_oo`03
003ooooo0<?oo`001ooo00@00?ooool000[oo`03003ooooo00Coo`03003ooooo00;oo`8000Woo`03
003ooooo02_oo`03003ooooo0<?oo`0000?oo`0000000P000_oo00@00?ooool000[oo`04003ooooo
ool30005ool00`00ooooo`08ool00`00ooooo`0[ool00`00ooooo`33ool000Ooo`04003ooooo000:
ool01@00oooooooo00001ooo00<00?ooool02?oo00<00?ooool0;?oo00<00?ooool0`_oo0007ool0
1000ooooo`002?oo0`000ooo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool0;Ooo00<00?oo
ool0`_oo0008ool2000;ool01000oooooooo10001?oo00<00?ooool01ooo00<00?ooool0;Ooo00<0
0?ooool0`_oo000Qool20008ool00`00ooooo`0]ool00`00ooooo`32ool0027oo`03003ooooo00Oo
o`03003ooooo02goo`03003ooooo0<;oo`008Ooo00<00?ooool01ooo00<00?ooool0;_oo00<00?oo
ool0`Ooo000Qool00`00ooooo`06ool00`00ooooo`0_ool00`00ooooo`31ool0027oo`03003ooooo
00Koo`03003ooooo02ooo`03003ooooo0<7oo`008Ooo0P001ooo00<00?ooool0;ooo00<00?ooool0
`Ooo000Qool00`00ooooo`06ool00`00ooooo`0_ool00`00ooooo`31ool0027oo`03003ooooo00Ko
o`03003ooooo02ooo`03003ooooo0<7oo`008Ooo00<00?ooool01_oo00<00?ooool0<?oo00<00?oo
ool0`?oo000Qool20006ool00`00ooooo`0aool00`00ooooo`30ool0027oo`03003ooooo00Goo`03
003ooooo037oo`03003ooooo0<3oo`008Ooo00<00?ooool01Ooo00<00?ooool0<Ooo00<00?ooool0
`?oo000Qool00`00ooooo`0iool00`00ooooo`30ool0027oo`03003ooooo03Woo`03003ooooo0<3o
o`008Ooo0P00>ooo00<00?ooool0_ooo000Qool00`00ooooo`0jool00`00ooooo`2oool000koo`80
00Coo`03003ooooo00;oo`@000Coo`03003ooooo03[oo`03003ooooo0;ooo`003Ooo00@00?ooool0
00[oo`03003ooooo00?oo`03003ooooo03[oo`03003ooooo0;ooo`003Ooo00@00?ooool000[oo`03
003ooooo00?oo`8003_oo`03003ooooo0;ooo`001ooo10000_oo00@00?ooool000[oo`03003ooooo
00?oo`03003ooooo03_oo`03003ooooo0;koo`003Ooo00@00?ooool000[oo`03003ooooo00?oo`03
003ooooo03_oo`03003ooooo0;koo`003Ooo00@00?ooool000Soo`<000Goo`03003ooooo03_oo`03
003ooooo0;koo`003_oo0P002ooo00<00?ooool00ooo00<00?ooool0>ooo00<00?ooool0__oo000Q
ool2000lool00`00ooooo`2nool0027oo`03003ooooo03_oo`03003ooooo0;koo`008Ooo00<00?oo
ool0??oo00<00?ooool0_Ooo000Qool00`00ooooo`0lool00`00ooooo`2mool0027oo`8003goo`03
003ooooo0;goo`008Ooo00<00?ooool0??oo00<00?ooool0_Ooo000Qool00`00ooooo`0lool00`00
ooooo`2mool0027oo`03003ooooo03goo`03003ooooo0;coo`008Ooo00<00?ooool0?Ooo00<00?oo
ool0_?oo000Qool2000nool00`00ooooo`2lool0027oo`03003ooooo03goo`03003ooooo0;coo`00
8Ooo00<00?ooool0?Ooo00<00?ooool0_?oo000Qool00`00ooooo`0nool00`00ooooo`2kool0027o
o`03003ooooo03koo`03003ooooo0;_oo`008Ooo0P00?ooo00<00?ooool0^ooo000Qool00`00oooo
o`0nool00`00ooooo`2kool000Soo`8000Coo`03003ooooo00?oo`8000?oo`<000Goo`03003ooooo
03koo`03003ooooo0;_oo`001ooo00@00?ooool000Soo`04003ooooo0005ool00`00ooooo`02ool0
0`00ooooo`0oool00`00ooooo`2jool000Ooo`04003ooooo0008ool01000ooooo`001Ooo00<00?oo
ool00_oo0P00@?oo00<00?ooool0^_oo00000ooo000000020002ool01000ooooo`002?oo00@00?oo
ool000;oo`<000Goo`03003ooooo03ooo`03003ooooo0;[oo`001ooo00@00?ooool000Soo`04003o
oooo0002ool00`00ooooo`05ool00`00ooooo`0oool00`00ooooo`2jool000Ooo`04003ooooo0008
ool01000ooooo`000_oo00<00?ooool01Ooo00<00?ooool0@?oo00<00?ooool0^Ooo0008ool2000:
ool20003ool40004ool00`00ooooo`10ool00`00ooooo`2iool0027oo`80047oo`03003ooooo0;Wo
o`008Ooo00<00?ooool0@?oo00<00?ooool0^Ooo000Qool00`00ooooo`10ool00`00ooooo`2iool0
027oo`03003ooooo047oo`03003ooooo0;Soo`008Ooo0P00@_oo00<00?ooool0^?oo000Qool00`00
ooooo`11ool00`00ooooo`2hool0027oo`03003ooooo047oo`03003ooooo0;Soo`008Ooo00<00?oo
ool0@Ooo00<00?ooool0^?oo000Qool00`00ooooo`0Oool20004ool00`00ooooo`02ool3000Eool0
0`00ooooo`08ool4000Pool40003ool00`00ooooo`02ool3000Pool4000Oool40003ool00`00oooo
o`02ool3000Qool20007ool0027oo`8001ooo`04003ooooo000;ool00`00ooooo`0Bool00`00oooo
o`0:ool00`00ooooo`0Qool00`00ooooo`0:ool00`00ooooo`0Mool00`00ooooo`0Pool00`00oooo
o`0<ool00`00ooooo`0Mool01000ooooo`001_oo000Qool00`00ooooo`0Nool01000ooooo`002ooo
00<00?ooool04_oo00<00?ooool02_oo00<00?ooool08Ooo00<00?ooool02_oo00<00?ooool07_oo
00<00?ooool08?oo00<00?ooool02ooo00<00?ooool08?oo00<00?ooool01?oo000Qool00`00oooo
o`0Nool01000ooooo`002?oo0`005Ooo00<00?ooool02_oo00<00?ooool08Ooo00<00?ooool01ooo
0`008_oo00<00?ooool08?oo00<00?ooool01ooo0`008Ooo0P001ooo000Qool00`00ooooo`0Nool0
1000ooooo`002?oo00<00?ooool05_oo00<00?ooool02Ooo00<00?ooool08Ooo00<00?ooool01ooo
00<00?ooool08ooo00<00?ooool08?oo00<00?ooool01_oo00<00?ooool08ooo00<00?ooool01?oo
000Qool2000Oool01000ooooo`002?oo00<00?ooool05_oo00<00?ooool01ooo0`008Ooo0`002Ooo
00<00?ooool08?oo00@00?ooool001ooo`04003ooooo0008ool00`00ooooo`0Pool01000ooooo`00
1_oo000Qool00`00ooooo`0Oool20009ool4000Eool00`00ooooo`09ool00`00ooooo`0Qool00`00
ooooo`07ool4000Pool2000Qool20009ool4000Pool20007ool0027oo`03003ooooo04?oo`03003o
oooo0;Koo`008Ooo00<00?ooool0A?oo00<00?ooool0]Ooo000Qool00`00ooooo`14ool00`00oooo
o`2eool001_oool000H000008Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<0
0?ooool01Ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool00_oo00@00?ooool000Ooo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00Goo`008Ooo00<00?ooool09_oo00<00?ooool0
7?oo00<00?ooool01ooo00<00?ooool09ooo00<00?ooool09_oo00<00?ooool09_oo00<00?ooool0
9_oo00<00?ooool01Ooo000Qool00`00ooooo`15ool00`00ooooo`07ool00`00ooooo`2Zool0027o
o`03003ooooo04Koo`03003ooooo00Koo`03003ooooo0:[oo`008Ooo0P00Aooo00<00?ooool01Ooo
00<00?ooool0Zooo000Qool00`00ooooo`17ool00`00ooooo`04ool00`00ooooo`2[ool0027oo`03
003ooooo04Ooo`03003ooooo00?oo`03003ooooo0:coo`008Ooo00<00?ooool0Aooo00<00?ooool0
0ooo00<00?ooool0[?oo000Qool20019ool01@00oooooooo0000[ooo000Qool00`00ooooo`19ool4
002_ool0027oo`03003ooooo04[oo`03003ooooo0:ooo`008Ooo00<00?ooool0o?oo000Qool00`00
ooooo`3lool0027oo`800?goo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0
0?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.402939, -0.337968, \
0.0121183, 0.00218797}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[BoxData[
    \(also\ looks\ o . k . \)], "Input"],

Cell[BoxData[
    \(check\ for\ \(\(im\)\(:\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Normal[%770[\([3, 1]\)]]\)], "Input"],

Cell[BoxData[
    \(1\/4\ z\ \((2\ z + \((\(-1\) + z\^2)\)\ If[Im[z] < 0, 
              2\ \[Pi]\ \[ImaginaryI], 
              0] + \((\(-1\) + z\^2)\)\ Log[\(-1\) + z] + Log[1 + z] - 
          z\^2\ Log[1 + z])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SXiA[z, 1, 0]\)], "Input"],

Cell[BoxData[
    \(1\/2\ z\^2\ \((1 - \(\((\(-1\) + z\^2)\)\ \((\(-Log[\(-1\) + z]\) + \
Log[1 + z])\)\)\/\(2\ z\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%810 - %811]\)], "Input"],

Cell[BoxData[
    \(1\/4\ z\ \((\(-1\) + z\^2)\)\ If[Im[z] < 0, 2\ \[Pi]\ \[ImaginaryI], 
        0]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SXiA[0.1 - 0.001  I, 1, 0]\)], "Input"],

Cell[BoxData[
    \(\(\(0.010727456817045605`\)\(\[InvisibleSpace]\)\) + 
      0.07755599101746402`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SXiA[0.1 + 0.001  I, 1, 0]\)], "Input"],

Cell[BoxData[
    \(\(\(0.010727456817045605`\)\(\[InvisibleSpace]\)\) - 
      0.07755599101746402`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(to\ add\ this\ Im[z]\ thing\ was\ bogus . \ o . k . \ 
        change\ everything\ back\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[SXiA[z - I, 1, 0]], {z, 0, 2}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -3.01261 10.6074 [
[.2619 .1571 -9 -9 ]
[.2619 .1571 9 0 ]
[.5 .1571 -3 -9 ]
[.5 .1571 3 0 ]
[.7381 .1571 -9 -9 ]
[.7381 .1571 9 0 ]
[.97619 .1571 -3 -9 ]
[.97619 .1571 3 0 ]
[.01131 .06353 -24 -4.5 ]
[.01131 .06353 0 4.5 ]
[.01131 .27568 -24 -4.5 ]
[.01131 .27568 0 4.5 ]
[.01131 .38175 -24 -4.5 ]
[.01131 .38175 0 4.5 ]
[.01131 .48782 -24 -4.5 ]
[.01131 .48782 0 4.5 ]
[.01131 .5939 -24 -4.5 ]
[.01131 .5939 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .1696 m
.2619 .17585 L
s
[(0.5)] .2619 .1571 0 1 Mshowa
.5 .1696 m
.5 .17585 L
s
[(1)] .5 .1571 0 1 Mshowa
.7381 .1696 m
.7381 .17585 L
s
[(1.5)] .7381 .1571 0 1 Mshowa
.97619 .1696 m
.97619 .17585 L
s
[(2)] .97619 .1571 0 1 Mshowa
.125 Mabswid
.07143 .1696 m
.07143 .17335 L
s
.11905 .1696 m
.11905 .17335 L
s
.16667 .1696 m
.16667 .17335 L
s
.21429 .1696 m
.21429 .17335 L
s
.30952 .1696 m
.30952 .17335 L
s
.35714 .1696 m
.35714 .17335 L
s
.40476 .1696 m
.40476 .17335 L
s
.45238 .1696 m
.45238 .17335 L
s
.54762 .1696 m
.54762 .17335 L
s
.59524 .1696 m
.59524 .17335 L
s
.64286 .1696 m
.64286 .17335 L
s
.69048 .1696 m
.69048 .17335 L
s
.78571 .1696 m
.78571 .17335 L
s
.83333 .1696 m
.83333 .17335 L
s
.88095 .1696 m
.88095 .17335 L
s
.92857 .1696 m
.92857 .17335 L
s
.25 Mabswid
0 .1696 m
1 .1696 L
s
.02381 .06353 m
.03006 .06353 L
s
[(0.29)] .01131 .06353 1 0 Mshowa
.02381 .27568 m
.03006 .27568 L
s
[(0.31)] .01131 .27568 1 0 Mshowa
.02381 .38175 m
.03006 .38175 L
s
[(0.32)] .01131 .38175 1 0 Mshowa
.02381 .48782 m
.03006 .48782 L
s
[(0.33)] .01131 .48782 1 0 Mshowa
.02381 .5939 m
.03006 .5939 L
s
[(0.34)] .01131 .5939 1 0 Mshowa
.125 Mabswid
.02381 .08474 m
.02756 .08474 L
s
.02381 .10596 m
.02756 .10596 L
s
.02381 .12717 m
.02756 .12717 L
s
.02381 .14839 m
.02756 .14839 L
s
.02381 .19082 m
.02756 .19082 L
s
.02381 .21203 m
.02756 .21203 L
s
.02381 .23325 m
.02756 .23325 L
s
.02381 .25446 m
.02756 .25446 L
s
.02381 .29689 m
.02756 .29689 L
s
.02381 .31811 m
.02756 .31811 L
s
.02381 .33932 m
.02756 .33932 L
s
.02381 .36054 m
.02756 .36054 L
s
.02381 .40296 m
.02756 .40296 L
s
.02381 .42418 m
.02756 .42418 L
s
.02381 .44539 m
.02756 .44539 L
s
.02381 .46661 m
.02756 .46661 L
s
.02381 .50904 m
.02756 .50904 L
s
.02381 .53025 m
.02756 .53025 L
s
.02381 .55147 m
.02756 .55147 L
s
.02381 .57268 m
.02756 .57268 L
s
.02381 .04231 m
.02756 .04231 L
s
.02381 .0211 m
.02756 .0211 L
s
.02381 .61511 m
.02756 .61511 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .01472 L
.02605 .01473 L
.02729 .01476 L
.02846 .01479 L
.03053 .01487 L
.03279 .01499 L
.03527 .01516 L
.0379 .01538 L
.04262 .0159 L
.04749 .0166 L
.05205 .01739 L
.06244 .01972 L
.07305 .02282 L
.08274 .02629 L
.10458 .0363 L
.14429 .06176 L
.18248 .09413 L
.22313 .13548 L
.26226 .18021 L
.30384 .23093 L
.34391 .28083 L
.38246 .32801 L
.42346 .37567 L
.46295 .41785 L
.50092 .4542 L
.54134 .48788 L
.58025 .51531 L
.62161 .5393 L
.66146 .55775 L
.69979 .57167 L
.74057 .58292 L
.77984 .59084 L
.81759 .59623 L
.83693 .59828 L
.85779 .60003 L
.87783 .60131 L
.88803 .60182 L
.89893 .60228 L
.90927 .60262 L
.91867 .60287 L
.92812 .60307 L
.93704 .6032 L
.94167 .60324 L
.94676 .60328 L
.94932 .6033 L
.95074 .6033 L
.95208 .60331 L
.95332 .60331 L
.95468 .60332 L
Mistroke
.95591 .60332 L
.95705 .60332 L
.95821 .60332 L
.95947 .60332 L
.96065 .60332 L
.96174 .60332 L
.96273 .60331 L
.96381 .60331 L
.96599 .6033 L
.96837 .60329 L
.97091 .60328 L
.9755 .60325 L
.97619 .60324 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool9
003lool001_oo`8000Ooo`<00?Woo`006ooo00<00?ooool02Ooo0`00m_oo000Kool00`00ooooo`0<
ool3003cool001_oo`03003ooooo00ooo`800?7oo`006ooo00<00?ooool04Ooo0P00kooo000Kool0
0`00ooooo`0Cool2003]ool001_oo`8001Koo`03003ooooo0>[oo`006ooo00<00?ooool05_oo0P00
j_oo000Kool00`00ooooo`0Hool2003Xool001_oo`03003ooooo01[oo`03003ooooo0>Goo`000_oo
0P001?oo00<00?ooool00_oo10000_oo0`001Ooo00<00?ooool06ooo0P00iOoo00001Ooo003ooooo
00002?oo00<00?ooool01_oo00<00?ooool00_oo00<00?ooool07Ooo00<00?ooool0h_oo00001Ooo
003ooooo00002Ooo00<00?ooool01Ooo00<00?ooool00_oo0P007ooo00<00?ooool0hOoo00001Ooo
003ooooo00002_oo00<00?ooool00_oo0`001?oo00<00?ooool07ooo00<00?ooool0h?oo00001Ooo
003ooooo00002ooo00@00?ooool000;oo`03003ooooo00;oo`03003ooooo023oo`03003ooooo0=oo
o`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool08Ooo00<0
0?ooool0g_oo0002ool2000:ool20004ool20005ool00`00ooooo`0Rool2003Nool001_oo`8002Go
o`03003ooooo0=_oo`006ooo00<00?ooool09Ooo00<00?ooool0f_oo000Kool00`00ooooo`0Vool0
0`00ooooo`3Iool001_oo`03003ooooo02Ooo`03003ooooo0=Soo`006ooo00<00?ooool0:?oo00<0
0?ooool0eooo000Kool00`00ooooo`0Yool00`00ooooo`3Fool001_oo`8002_oo`03003ooooo0=Go
o`006ooo00<00?ooool0:ooo00<00?ooool0e?oo000Kool00`00ooooo`0/ool00`00ooooo`3Cool0
01_oo`03003ooooo02goo`03003ooooo0=;oo`006ooo00<00?ooool0;_oo00<00?ooool0dOoo000K
ool00`00ooooo`0_ool00`00ooooo`3@ool001_oo`80037oo`03003ooooo0<ooo`006ooo00<00?oo
ool0<Ooo00@00?ooooooo`8000Coo`03003ooooo00;oo`<003Koo`@003Koo`@000?oo`03003ooooo
00;oo`<003Koo`@000Koo`006ooo00<00?ooool0<_oo00<00?oo00000_oo00<00?ooool02Ooo00<0
0?ooool0=Ooo00<00?ooool0=ooo00<00?ooool02_oo00<00?ooool0<ooo00<00?ooool01ooo000K
ool00`00ooooo`0cool20002ool00`00ooooo`09ool00`00ooooo`0eool00`00ooooo`0gool00`00
ooooo`0:ool00`00ooooo`0dool00`00ooooo`06ool001_oo`03003ooooo03Coo`04003ooooo0008
ool3000hool00`00ooooo`0gool00`00ooooo`07ool3000hool00`00ooooo`05ool001_oo`8003Go
o`04003ooooo0008ool00`00ooooo`0hool00`00ooooo`0gool00`00ooooo`07ool00`00ooooo`0i
ool00`00ooooo`04ool001_oo`03003ooooo03Coo`800003ool00?oo00Ooo`03003ooooo03Koo`<0
03Ooo`<000Woo`03003ooooo03Koo`04003ooooo0006ool001_oo`03003ooooo03Goo`8000Woo`@0
03Ooo`03003ooooo03Ooo`03003ooooo00Ooo`@003Koo`8000Ooo`006ooo00<00?ooool0=ooo00<0
0?ooool0b?oo000Kool00`00ooooo`0hool00`00ooooo`37ool001_oo`03003ooooo03Woo`03003o
oooo0<Koo`005?ooo`003@00000Kool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool0
0`00ooooo`09ool00`00ooooo`08ool00`00ool0000<ool00`00ooooo`09ool00`00ooooo`0:ool0
0`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool0
0`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool0
0`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`05ool001_oo`03003ooooo03_oo`8003koo`03
003ooooo03goo`03003ooooo03coo`03003ooooo00Goo`006ooo00<00?ooool0??oo00<00?ooool0
`ooo000Kool00`00ooooo`0mool00`00ooooo`32ool001_oo`03003ooooo03koo`03003ooooo0<7o
o`006ooo0P00@?oo00<00?ooool0`?oo000Kool00`00ooooo`0oool00`00ooooo`30ool001_oo`03
003ooooo043oo`03003ooooo0;ooo`006ooo00<00?ooool0@Ooo00<00?ooool0__oo000Kool00`00
ooooo`12ool00`00ooooo`2mool001_oo`8004Coo`03003ooooo0;coo`006ooo00<00?ooool0A?oo
00<00?ooool0^ooo000Kool00`00ooooo`14ool00`00ooooo`2kool001_oo`03003ooooo04Goo`03
003ooooo0;[oo`006ooo00<00?ooool0A_oo00<00?ooool0^Ooo000Kool00`00ooooo`17ool00`00
ooooo`2hool001_oo`8004Woo`03003ooooo0;Ooo`006ooo00<00?ooool0BOoo00<00?ooool0]_oo
000Kool00`00ooooo`19ool00`00ooooo`2fool001_oo`03003ooooo04[oo`03003ooooo0;Goo`00
6ooo00<00?ooool0Booo00<00?ooool0]?oo000Kool00`00ooooo`1<ool00`00ooooo`2cool001_o
o`8004goo`03003ooooo0;?oo`006ooo00<00?ooool0COoo00<00?ooool0/_oo000Kool00`00oooo
o`1>ool00`00ooooo`2aool000;oo`8000Coo`03003ooooo00?oo`8000?oo`@000Coo`03003ooooo
04ooo`03003ooooo0;3oo`0000Goo`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`03ool0
0`00ooooo`1@ool00`00ooooo`2_ool00005ool00?ooool0000;ool00`00ooooo`02ool00`00oooo
o`03ool2001Aool00`00ooooo`2_ool00005ool00?ooool00009ool20005ool00`00ooooo`03ool0
0`00ooooo`1Aool00`00ooooo`2^ool00005ool00?ooool0000;ool00`00ooooo`02ool00`00oooo
o`03ool00`00ooooo`1Bool00`00ooooo`2]ool00005ool00?ooool00008ool01000ooooo`000_oo
0`001Ooo00<00?ooool0Dooo00<00?ooool0[?oo0002ool2000:ool20005ool00`00ooooo`03ool0
0`00ooooo`1Dool00`00ooooo`2[ool001_oo`03003ooooo05Coo`03003ooooo0:_oo`006ooo0P00
E_oo00<00?ooool0Z_oo000Kool00`00ooooo`1Fool00`00ooooo`2Yool001_oo`03003ooooo05Oo
o`03003ooooo0:Soo`006ooo00<00?ooool0F?oo00<00?ooool0Yooo000Kool00`00ooooo`1Iool0
0`00ooooo`2Vool001_oo`8005[oo`03003ooooo0:Koo`006ooo00<00?ooool0F_oo00<00?ooool0
YOoo000Kool00`00ooooo`1Kool00`00ooooo`2Tool001_oo`03003ooooo05coo`03003ooooo0:?o
o`006ooo00<00?ooool0GOoo00<00?ooool0X_oo000Kool00`00ooooo`1Nool00`00ooooo`2Qool0
01_oo`80063oo`03003ooooo0:3oo`006ooo00<00?ooool0Gooo00<00?ooool0X?oo000Kool00`00
ooooo`1Pool00`00ooooo`2Oool001_oo`03003ooooo067oo`03003ooooo09koo`006ooo00<00?oo
ool0H_oo00<00?ooool0WOoo000Kool00`00ooooo`1Sool00`00ooooo`2Lool001_oo`8006Goo`03
003ooooo09_oo`006ooo00<00?ooool0I?oo00<00?ooool0Vooo000Kool00`00ooooo`1Uool00`00
ooooo`2Jool000;oo`8000Coo`03003ooooo00?oo`8000?oo`@000Coo`03003ooooo06Koo`03003o
oooo09Woo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`1W
ool00`00ooooo`2Hool00005ool00?ooool0000;ool01@00oooooooo00001_oo0P00JOoo00<00?oo
ool0Uooo00001Ooo003ooooo00002Ooo0P001Ooo00<00?ooool00ooo00<00?ooool0JOoo00<00?oo
ool0U_oo00001Ooo003ooooo00002ooo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0J_oo
00<00?ooool0UOoo00001Ooo003ooooo00002?oo00@00?ooool000;oo`04003ooooo0004ool00`00
ooooo`1[ool00`00ooooo`2Dool000;oo`8000[oo`8000Coo`8000Goo`03003ooooo06coo`03003o
oooo09?oo`006ooo00<00?ooool0KOoo00<00?ooool0T_oo000Kool2001_ool00`00ooooo`2Aool0
01_oo`03003ooooo06ooo`03003ooooo093oo`006ooo00<00?ooool0L?oo00<00?ooool0Sooo000K
ool00`00ooooo`1aool00`00ooooo`2>ool001_oo`03003ooooo07;oo`03003ooooo08goo`006ooo
00<00?ooool0Looo00<00?ooool0S?oo000Kool2001eool00`00ooooo`2;ool001_oo`03003ooooo
07Goo`03003ooooo08[oo`006ooo00<00?ooool0M_oo00<00?ooool0ROoo000Kool00`00ooooo`1g
ool00`00ooooo`28ool001_oo`03003ooooo07Soo`03003ooooo08Ooo`006ooo0P00N_oo00<00?oo
ool0Q_oo000Kool00`00ooooo`1jool00`00ooooo`25ool001_oo`03003ooooo07_oo`03003ooooo
08Coo`006ooo00<00?ooool0O?oo00<00?ooool0Pooo000Kool00`00ooooo`1mool00`00ooooo`22
ool001_oo`03003ooooo07koo`03003ooooo087oo`006ooo0P00P?oo00<00?ooool0P?oo000Kool0
0`00ooooo`20ool00`00ooooo`1oool001_oo`03003ooooo087oo`8007ooo`006ooo00<00?ooool0
Pooo00<00?ooool0O?oo0002ool20004ool00`00ooooo`03ool20004ool20005ool00`00ooooo`24
ool00`00ooooo`1kool00005ool00?ooool00008ool01000ooooo`000_oo00@00?ooool000Coo`03
003ooooo08Goo`03003ooooo07[oo`0000Goo`00ooooo`0000_oo`03003ooooo00?oo`03003ooooo
00;oo`8008Ooo`03003ooooo07Woo`0000Goo`00ooooo`0000Woo`8000Coo`8000Goo`03003ooooo
08Ooo`8007Woo`0000Goo`00ooooo`0000_oo`03003ooooo00?oo`03003ooooo00;oo`03003ooooo
08Woo`03003ooooo07Koo`0000Goo`00ooooo`0000Soo`04003ooooo0002ool01000ooooo`001?oo
00<00?ooool0R_oo0P00M_oo0002ool2000:ool20004ool20005ool00`00ooooo`2<ool2001dool0
01_oo`8008ooo`03003ooooo077oo`006ooo00<00?ooool0Sooo0P00LOoo000Kool00`00ooooo`2A
ool00`00ooooo`1^ool001_oo`03003ooooo09;oo`8006koo`006ooo00<00?ooool0U?oo0P00K?oo
000Kool00`00ooooo`2Fool2001Zool001_oo`8009Woo`8006Soo`006ooo00<00?ooool0V_oo0P00
I_oo000Kool00`00ooooo`2Lool00`00ooooo`1Sool001_oo`03003ooooo09goo`8006?oo`006ooo
00<00?ooool0Wooo0P00HOoo000Kool00`00ooooo`2Qool2001Oool001_oo`800:Coo`8005goo`00
6ooo00<00?ooool0YOoo0`00F_oo000Kool00`00ooooo`2Xool3001Gool001_oo`03003ooooo0:_o
o`8005Goo`006ooo00<00?ooool0[Ooo0`00D_oo000Kool2002aool3001?ool001_oo`03003ooooo
0;?oo`<004coo`006ooo00<00?ooool0]_oo1000B?oo000Kool00`00ooooo`2jool40014ool000;o
o`8000Coo`03003ooooo00?oo`8000Goo`03003ooooo00?oo`03003ooooo0;koo`H003koo`0000Go
o`00ooooo`0000Soo`04003ooooo0004ool00`00ooooo`03ool00`00ooooo`34ool7000gool00005
ool00?ooool0000;ool00`00ool000040004ool2003<ool8000_ool00005ool00?ooool00009ool2
0002ool01000ooooo`001Ooo00<00?ooool0dooo2`009?oo00001Ooo003ooooo00002ooo00H00?oo
ool00?oo0005ool00`00ooooo`3NoolM0007ool00005ool00?ooool00008ool01000ooooo`000ooo
0P001Ooo00<00?ooool0oooo0ooo0002ool2000:ool20005ool00`00ooooo`03ool00`00ooooo`3o
ool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo003ooolQool00?ooob7oo`00oooo
8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.215729, 0.281788, \
0.00789457, 0.000354405}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Re[Normal[%770[\([3, 1]\)]] /. z \[Rule] X - I], {X, 0, 2}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.103642 0.0311348 [
[.2619 .09114 -9 -9 ]
[.2619 .09114 9 0 ]
[.5 .09114 -3 -9 ]
[.5 .09114 3 0 ]
[.7381 .09114 -9 -9 ]
[.7381 .09114 9 0 ]
[.97619 .09114 -3 -9 ]
[.97619 .09114 3 0 ]
[.01131 .02581 -24 -4.5 ]
[.01131 .02581 0 4.5 ]
[.01131 .18148 -18 -4.5 ]
[.01131 .18148 0 4.5 ]
[.01131 .25932 -6 -4.5 ]
[.01131 .25932 0 4.5 ]
[.01131 .33715 -18 -4.5 ]
[.01131 .33715 0 4.5 ]
[.01131 .41499 -12 -4.5 ]
[.01131 .41499 0 4.5 ]
[.01131 .49283 -24 -4.5 ]
[.01131 .49283 0 4.5 ]
[.01131 .57066 -12 -4.5 ]
[.01131 .57066 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .10364 m
.2619 .10989 L
s
[(0.5)] .2619 .09114 0 1 Mshowa
.5 .10364 m
.5 .10989 L
s
[(1)] .5 .09114 0 1 Mshowa
.7381 .10364 m
.7381 .10989 L
s
[(1.5)] .7381 .09114 0 1 Mshowa
.97619 .10364 m
.97619 .10989 L
s
[(2)] .97619 .09114 0 1 Mshowa
.125 Mabswid
.07143 .10364 m
.07143 .10739 L
s
.11905 .10364 m
.11905 .10739 L
s
.16667 .10364 m
.16667 .10739 L
s
.21429 .10364 m
.21429 .10739 L
s
.30952 .10364 m
.30952 .10739 L
s
.35714 .10364 m
.35714 .10739 L
s
.40476 .10364 m
.40476 .10739 L
s
.45238 .10364 m
.45238 .10739 L
s
.54762 .10364 m
.54762 .10739 L
s
.59524 .10364 m
.59524 .10739 L
s
.64286 .10364 m
.64286 .10739 L
s
.69048 .10364 m
.69048 .10739 L
s
.78571 .10364 m
.78571 .10739 L
s
.83333 .10364 m
.83333 .10739 L
s
.88095 .10364 m
.88095 .10739 L
s
.92857 .10364 m
.92857 .10739 L
s
.25 Mabswid
0 .10364 m
1 .10364 L
s
.02381 .02581 m
.03006 .02581 L
s
[(-2.5)] .01131 .02581 1 0 Mshowa
.02381 .18148 m
.03006 .18148 L
s
[(2.5)] .01131 .18148 1 0 Mshowa
.02381 .25932 m
.03006 .25932 L
s
[(5)] .01131 .25932 1 0 Mshowa
.02381 .33715 m
.03006 .33715 L
s
[(7.5)] .01131 .33715 1 0 Mshowa
.02381 .41499 m
.03006 .41499 L
s
[(10)] .01131 .41499 1 0 Mshowa
.02381 .49283 m
.03006 .49283 L
s
[(12.5)] .01131 .49283 1 0 Mshowa
.02381 .57066 m
.03006 .57066 L
s
[(15)] .01131 .57066 1 0 Mshowa
.125 Mabswid
.02381 .04137 m
.02756 .04137 L
s
.02381 .05694 m
.02756 .05694 L
s
.02381 .07251 m
.02756 .07251 L
s
.02381 .08807 m
.02756 .08807 L
s
.02381 .11921 m
.02756 .11921 L
s
.02381 .13478 m
.02756 .13478 L
s
.02381 .15034 m
.02756 .15034 L
s
.02381 .16591 m
.02756 .16591 L
s
.02381 .19705 m
.02756 .19705 L
s
.02381 .21261 m
.02756 .21261 L
s
.02381 .22818 m
.02756 .22818 L
s
.02381 .24375 m
.02756 .24375 L
s
.02381 .27488 m
.02756 .27488 L
s
.02381 .29045 m
.02756 .29045 L
s
.02381 .30602 m
.02756 .30602 L
s
.02381 .32159 m
.02756 .32159 L
s
.02381 .35272 m
.02756 .35272 L
s
.02381 .36829 m
.02756 .36829 L
s
.02381 .38385 m
.02756 .38385 L
s
.02381 .39942 m
.02756 .39942 L
s
.02381 .43056 m
.02756 .43056 L
s
.02381 .44612 m
.02756 .44612 L
s
.02381 .46169 m
.02756 .46169 L
s
.02381 .47726 m
.02756 .47726 L
s
.02381 .50839 m
.02756 .50839 L
s
.02381 .52396 m
.02756 .52396 L
s
.02381 .53953 m
.02756 .53953 L
s
.02381 .5551 m
.02756 .5551 L
s
.02381 .01024 m
.02756 .01024 L
s
.02381 .58623 m
.02756 .58623 L
s
.02381 .6018 m
.02756 .6018 L
s
.02381 .61737 m
.02756 .61737 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .01472 L
.02605 .01472 L
.02729 .01472 L
.02846 .01473 L
.03053 .01474 L
.03279 .01477 L
.03527 .0148 L
.0379 .01485 L
.04262 .01495 L
.04749 .01508 L
.05205 .01524 L
.06244 .0157 L
.07305 .01631 L
.08274 .017 L
.10458 .019 L
.12357 .02125 L
.14429 .02424 L
.18493 .03175 L
.22406 .04102 L
.26565 .05305 L
.30571 .06678 L
.34426 .08194 L
.38527 .10018 L
.42475 .11979 L
.46273 .14055 L
.50315 .16468 L
.54206 .18989 L
.58342 .21882 L
.62326 .24877 L
.66159 .2795 L
.70238 .31428 L
.74164 .34979 L
.77939 .3858 L
.8196 .42617 L
.85828 .46699 L
.89942 .51252 L
.93905 .55844 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000KoolC003bool0
00Ooo`@000?oo`03003ooooo00;oo`<000Goo`03003ooooo013oo`X00>Soo`001ooo00<00?ooool0
3?oo00<00?ooool00_oo00<00?ooool06_oo1P00h_oo0008ool00`00ooooo`0;ool00`00ooooo`02
ool2000Qool6003Lool00003ool00000008000Coo`03003ooooo00Ooo`<000Goo`03003ooooo02Ko
o`@00=Soo`002_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool0:_oo0`00eOoo0007ool0
1000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0;Ooo1000dOoo0008ool20009ool40004ool2
000bool3003>ool001_oo`03003ooooo03Coo`@00<[oo`006ooo00<00?ooool0>?oo1000a_oo000K
ool00`00ooooo`0lool30033ool001_oo`80043oo`<00<3oo`006ooo00<00?ooool0@_oo0P00__oo
000Kool00`00ooooo`0eool20004ool00`00ooooo`02ool300001?oo00000000<_oo1000=_oo1000
0ooo00<00?ooool00_oo0`00=_oo10001_oo000Kool00`00ooooo`0dool01000ooooo`002ooo00@0
0?ooooooo`<0037oo`03003ooooo03Ooo`03003ooooo00[oo`03003ooooo03?oo`03003ooooo00Oo
o`006ooo0P00=Ooo00@00?ooool000_oo`03003ooooo00Coo`8002ooo`03003ooooo03Ooo`03003o
oooo00[oo`03003ooooo03Coo`03003ooooo00Koo`006ooo00<00?ooool0=?oo00@00?ooool000So
o`<000Woo`<002coo`03003ooooo03Ooo`03003ooooo00Ooo`<003Soo`03003ooooo00Goo`006ooo
00<00?ooool0=?oo00@00?ooool000Soo`03003ooooo00coo`8002[oo`03003ooooo03Ooo`03003o
oooo00Ooo`03003ooooo03Woo`03003ooooo00Coo`006ooo00<00?ooool0=?oo00@00?ooool000So
o`03003ooooo00koo`<002Goo`<003Ooo`<000Woo`03003ooooo03Koo`04003ooooo0006ool001_o
o`8003Koo`8000Woo`@0013oo`<002Coo`03003ooooo03Ooo`03003ooooo00Ooo`@003Koo`8000Oo
o`006ooo00<00?ooool0Eooo0P00ZOoo000Kool00`00ooooo`1Iool3002Vool001_oo`03003ooooo
05coo`800:Coo`005?ooo`003@00000Kool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:
ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`08
ool3000<ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0:ool00`00
ooooo`09ool00`00ooooo`0:ool00`00ooooo`05ool001_oo`03003ooooo03coo`03003ooooo02?o
o`03003ooooo01Koo`03003ooooo03goo`03003ooooo03coo`03003ooooo00Goo`006ooo00<00?oo
ool0Hooo0P00WOoo000Kool00`00ooooo`1Uool2002Kool001_oo`8006Soo`8009Woo`006ooo00<0
0?ooool0JOoo0P00Uooo000Kool00`00ooooo`1[ool2002Eool001_oo`03003ooooo06goo`8009?o
o`006ooo0P00L?oo0P00TOoo000Kool00`00ooooo`1aool2002?ool001_oo`03003ooooo07?oo`80
08goo`006ooo00<00?ooool0MOoo00<00?ooool0R_oo000Kool2001gool00`00ooooo`29ool001_o
o`03003ooooo07Ooo`8008Woo`006ooo00<00?ooool0NOoo00<00?ooool0Q_oo000Kool00`00oooo
o`1jool20026ool001_oo`8007goo`03003ooooo08?oo`006ooo00<00?ooool0OOoo0P00Pooo0007
ool40003ool00`00ooooo`02ool30005ool00`00ooooo`1oool20021ool000Ooo`03003ooooo00co
o`03003ooooo00;oo`03003ooooo087oo`8007ooo`002?oo00<00?ooool02ooo00<00?ooool00_oo
0P00Q?oo0P00OOoo0009ool00`00ooooo`07ool30005ool00`00ooooo`25ool2001kool000[oo`03
003ooooo00Koo`03003ooooo00Goo`03003ooooo08Ooo`03003ooooo07Soo`001ooo00@00?ooool0
00Soo`03003ooooo00Goo`03003ooooo08Soo`8007Soo`002?oo0P002Ooo10001?oo0P00Rooo00<0
0?ooool0MOoo000Kool00`00ooooo`2;ool00`00ooooo`1dool001_oo`03003ooooo08coo`8007Co
o`006ooo00<00?ooool0S_oo00<00?ooool0LOoo000Kool2002@ool00`00ooooo`1`ool001_oo`03
003ooooo093oo`80073oo`006ooo00<00?ooool0T_oo00<00?ooool0KOoo000Kool00`00ooooo`2C
ool00`00ooooo`1/ool001_oo`03003ooooo09Coo`8006coo`006ooo0P00Uooo00<00?ooool0JOoo
000Kool00`00ooooo`2Gool00`00ooooo`1Xool001_oo`03003ooooo09Soo`03003ooooo06Ooo`00
6ooo00<00?ooool0VOoo0P00Iooo000Kool2002Lool00`00ooooo`1Tool001_oo`03003ooooo09co
o`03003ooooo06?oo`004ooo0`001Ooo00<00?ooool0WOoo00<00?ooool0H_oo000Fool00`00oooo
o`02ool00`00ooooo`2Nool2001Rool001Koo`03003ooooo00;oo`800:7oo`03003ooooo05ooo`00
4ooo0`001Ooo00<00?ooool0XOoo00<00?ooool0G_oo000Cool00`00ooooo`05ool00`00ooooo`2R
ool00`00ooooo`1Mool001?oo`03003ooooo00Goo`03003ooooo0:?oo`8005goo`004ooo10001?oo
0P00Y_oo00<00?ooool0F_oo000Kool00`00ooooo`2Vool00`00ooooo`1Iool001_oo`03003ooooo
0:Ooo`03003ooooo05Soo`006ooo00<00?ooool0Z?oo0P00F?oo000Kool2002[ool00`00ooooo`1E
ool001_oo`03003ooooo0:_oo`03003ooooo05Coo`006ooo00<00?ooool0[?oo00<00?ooool0Dooo
000Kool00`00ooooo`2]ool00`00ooooo`1Bool001_oo`800:ooo`8005;oo`006ooo00<00?ooool0
/?oo00<00?ooool0Cooo000Kool00`00ooooo`2aool00`00ooooo`1>ool001_oo`03003ooooo0;;o
o`03003ooooo04goo`006ooo0P00]?oo00<00?ooool0C?oo000Kool00`00ooooo`2dool00`00oooo
o`1;ool001_oo`03003ooooo0;Goo`03003ooooo04[oo`002?oo00<00?ooool00ooo00<00?ooool0
0_oo0`001Ooo00<00?ooool0]_oo0P00B_oo0008ool00`00ooooo`0;ool00`00ooooo`02ool00`00
ooooo`2hool00`00ooooo`17ool000Woo`03003ooooo00[oo`03003ooooo00;oo`800;[oo`03003o
oooo04Koo`002Ooo00<00?ooool01ooo0`001Ooo00<00?ooool0^_oo00<00?ooool0AOoo000:ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`2kool00`00ooooo`14ool000Ooo`04003ooooo
0008ool00`00ooooo`05ool00`00ooooo`2lool00`00ooooo`13ool000Ooo`@000Soo`@000Coo`80
0;koo`03003ooooo04;oo`006ooo00<00?ooool0__oo00<00?ooool0@Ooo000Kool00`00ooooo`2o
ool00`00ooooo`10ool001_oo`03003ooooo0<3oo`03003ooooo03ooo`006ooo0P00`_oo0P00?ooo
000Kool00`00ooooo`33ool00`00ooooo`0lool001_oo`03003ooooo0<Coo`03003ooooo03_oo`00
6ooo00<00?ooool0aOoo00<00?ooool0>_oo000Kool20037ool00`00ooooo`0iool001_oo`03003o
oooo0<Ooo`03003ooooo03Soo`006ooo00<00?ooool0b?oo00<00?ooool0=ooo000Kool00`00oooo
o`39ool00`00ooooo`0fool001_oo`800<_oo`03003ooooo03Goo`006ooo00<00?ooool0booo00<0
0?ooool0=?oo000=ool40003ool20005ool00`00ooooo`3<ool00`00ooooo`0cool000ooo`05003o
ooooool00002ool00`00ooooo`02ool00`00ooooo`3=ool00`00ooooo`0bool000ooo`05003ooooo
ool00002ool00`00ooooo`02ool2003?ool00`00ooooo`0aool000ooo`05003oooooool00002ool0
0`00ooooo`02ool00`00ooooo`3?ool00`00ooooo`0`ool000ooo`05003oooooool00002ool00`00
ooooo`02ool00`00ooooo`3@ool00`00ooooo`0_ool000goo`<000?oo`04003ooooo0004ool00`00
ooooo`3Aool00`00ooooo`0^ool000ooo`03003ooooo00;oo`8000Goo`800=?oo`03003ooooo02go
o`006ooo00<00?ooool0dooo00<00?ooool0;?oo000Kool00`00ooooo`3Dool00`00ooooo`0[ool0
01_oo`03003ooooo0=Goo`03003ooooo02[oo`006ooo00<00?ooool0e_oo00<00?ooool0:Ooo000K
ool2003Gool00`00ooooo`0Yool001_oo`03003ooooo0=Ooo`03003ooooo02Soo`006ooo00<00?oo
ool0f?oo00<00?ooool09ooo000Kool00`00ooooo`3Iool00`00ooooo`0Vool001_oo`800=_oo`03
003ooooo02Goo`006ooo00<00?ooool0fooo00<00?ooool09?oo000Kool00`00ooooo`3Lool00`00
ooooo`0Sool001_oo`03003ooooo0=goo`03003ooooo02;oo`006ooo0P00gooo00<00?ooool08Ooo
000Kool00`00ooooo`3Oool00`00ooooo`0Pool00003ool00000008000;oo`@000?oo`03003ooooo
00;oo`<000Goo`03003ooooo0>3oo`03003ooooo01ooo`000ooo00D00?ooooooo`0000koo`03003o
oooo00;oo`03003ooooo0>3oo`03003ooooo01ooo`000ooo00<00?ooool00_oo00<00?ooool02ooo
00<00?ooool00_oo0P00h_oo00<00?ooool07_oo0003ool00`00ooooo`03ool00`00ooooo`07ool3
0005ool00`00ooooo`3Rool00`00ooooo`0Mool000?oo`03003ooooo00Coo`03003ooooo00Koo`03
003ooooo00Goo`03003ooooo0>?oo`03003ooooo01coo`0000Coo`00000000?oo`04003ooooo0008
ool00`00ooooo`05ool00`00ooooo`3Tool00`00ooooo`0Kool000?oo`03003ooooo00;oo`8000Wo
o`@000Coo`800>Koo`03003ooooo01[oo`006ooo00<00?ooool0i_oo00<00?ooool06Ooo000Kool0
0`00ooooo`3Wool00`00ooooo`0Hool001_oo`03003ooooo0>Soo`03003ooooo01Ooo`006ooo0P00
jOoo00<00?ooool05ooo000Kool00`00ooooo`3Yool00`00ooooo`0Fool001_oo`03003ooooo0>[o
o`03003ooooo01Goo`006ooo00<00?ooool0jooo00<00?ooool05?oo000Kool2003]ool00`00oooo
o`0Cool001_oo`03003ooooo0>goo`03003ooooo01;oo`006ooo00<00?ooool0kOoo00<00?ooool0
4_oo000Kool00`00ooooo`3^ool00`00ooooo`0Aool001_oo`800?3oo`03003ooooo013oo`006ooo
00<00?ooool0l?oo00<00?ooool03ooo000Kool00`00ooooo`3aool00`00ooooo`0>ool000goo`@0
00;oo`<000Goo`03003ooooo0?;oo`03003ooooo00goo`003ooo00<00?ooool01?oo00<00?ooool0
0_oo00<00?ooool0l_oo00<00?ooool03Ooo000?ool00`00ooooo`04ool00`00ooooo`02ool2003d
ool00`00ooooo`0<ool000ooo`04003oooooool30005ool00`00ooooo`3dool00`00ooooo`0;ool0
00ooo`05003oooooool00007ool00`00ooooo`3eool00`00ooooo`0:ool000goo`<000?oo`03003o
oooo00Goo`03003ooooo0?Koo`03003ooooo00Woo`003ooo00@00?ooooooo`@000Coo`800?Soo`03
003ooooo00Soo`006ooo00<00?ooool0mooo00<00?ooool02?oo000Kool00`00ooooo`3hool00`00
ooooo`07ool001_oo`03003ooooo0?Woo`03003ooooo00Koo`006ooo0P00nooo00<00?ooool01Ooo
000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo
000Kool2003oool4ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQ
ool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.215729, -4.08575, \
0.00789457, 0.120743}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["other", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 70]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9287303646387737`, 
      mGamma \[Rule] \(-0.13822835949384135`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 13]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9827540360681772`, 
      mGamma \[Rule] \(-0.32571382125262277`\)}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(strangesol[k_, a_] := 
      FindRoot[{Re[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0, 
          Im[kx0corrtestfunc[Om + I\ mGamma, k, a]] \[Equal] 0}, {Om, 0.7, 
          0.8}, {mGamma, \(-0.3\), \(-0.4\)}, 
        MaxIterations \[Rule] 100]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[1, 10]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9994820046162144`, 
      mGamma \[Rule] \(-0.3707184166384133`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(strangesol[0.8, 8]\)], "Input"],

Cell[BoxData[
    \({Om \[Rule] 0.9990379856291097`, 
      mGamma \[Rule] \(-0.3811371647848973`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[kx0corrtestfunc[om, 2, 70]], {om, 0.5, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.380952 0.254329 0.0849453 [
[.02381 .24183 -9 -9 ]
[.02381 .24183 9 0 ]
[.40476 .24183 -9 -9 ]
[.40476 .24183 9 0 ]
[.59524 .24183 -3 -9 ]
[.59524 .24183 3 0 ]
[.78571 .24183 -9 -9 ]
[.78571 .24183 9 0 ]
[.97619 .24183 -3 -9 ]
[.97619 .24183 3 0 ]
[.20179 .08444 -12 -4.5 ]
[.20179 .08444 0 4.5 ]
[.20179 .16938 -12 -4.5 ]
[.20179 .16938 0 4.5 ]
[.20179 .33927 -6 -4.5 ]
[.20179 .33927 0 4.5 ]
[.20179 .42422 -6 -4.5 ]
[.20179 .42422 0 4.5 ]
[.20179 .50917 -6 -4.5 ]
[.20179 .50917 0 4.5 ]
[.20179 .59411 -6 -4.5 ]
[.20179 .59411 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .25433 m
.02381 .26058 L
s
[(0.5)] .02381 .24183 0 1 Mshowa
.40476 .25433 m
.40476 .26058 L
s
[(1.5)] .40476 .24183 0 1 Mshowa
.59524 .25433 m
.59524 .26058 L
s
[(2)] .59524 .24183 0 1 Mshowa
.78571 .25433 m
.78571 .26058 L
s
[(2.5)] .78571 .24183 0 1 Mshowa
.97619 .25433 m
.97619 .26058 L
s
[(3)] .97619 .24183 0 1 Mshowa
.125 Mabswid
.0619 .25433 m
.0619 .25808 L
s
.1 .25433 m
.1 .25808 L
s
.1381 .25433 m
.1381 .25808 L
s
.17619 .25433 m
.17619 .25808 L
s
.25238 .25433 m
.25238 .25808 L
s
.29048 .25433 m
.29048 .25808 L
s
.32857 .25433 m
.32857 .25808 L
s
.36667 .25433 m
.36667 .25808 L
s
.44286 .25433 m
.44286 .25808 L
s
.48095 .25433 m
.48095 .25808 L
s
.51905 .25433 m
.51905 .25808 L
s
.55714 .25433 m
.55714 .25808 L
s
.63333 .25433 m
.63333 .25808 L
s
.67143 .25433 m
.67143 .25808 L
s
.70952 .25433 m
.70952 .25808 L
s
.74762 .25433 m
.74762 .25808 L
s
.82381 .25433 m
.82381 .25808 L
s
.8619 .25433 m
.8619 .25808 L
s
.9 .25433 m
.9 .25808 L
s
.9381 .25433 m
.9381 .25808 L
s
.25 Mabswid
0 .25433 m
1 .25433 L
s
.21429 .08444 m
.22054 .08444 L
s
[(-2)] .20179 .08444 1 0 Mshowa
.21429 .16938 m
.22054 .16938 L
s
[(-1)] .20179 .16938 1 0 Mshowa
.21429 .33927 m
.22054 .33927 L
s
[(1)] .20179 .33927 1 0 Mshowa
.21429 .42422 m
.22054 .42422 L
s
[(2)] .20179 .42422 1 0 Mshowa
.21429 .50917 m
.22054 .50917 L
s
[(3)] .20179 .50917 1 0 Mshowa
.21429 .59411 m
.22054 .59411 L
s
[(4)] .20179 .59411 1 0 Mshowa
.125 Mabswid
.21429 .01648 m
.21804 .01648 L
s
.21429 .03347 m
.21804 .03347 L
s
.21429 .05046 m
.21804 .05046 L
s
.21429 .06745 m
.21804 .06745 L
s
.21429 .10143 m
.21804 .10143 L
s
.21429 .11842 m
.21804 .11842 L
s
.21429 .13541 m
.21804 .13541 L
s
.21429 .1524 m
.21804 .1524 L
s
.21429 .18637 m
.21804 .18637 L
s
.21429 .20336 m
.21804 .20336 L
s
.21429 .22035 m
.21804 .22035 L
s
.21429 .23734 m
.21804 .23734 L
s
.21429 .27132 m
.21804 .27132 L
s
.21429 .28831 m
.21804 .28831 L
s
.21429 .3053 m
.21804 .3053 L
s
.21429 .32229 m
.21804 .32229 L
s
.21429 .35626 m
.21804 .35626 L
s
.21429 .37325 m
.21804 .37325 L
s
.21429 .39024 m
.21804 .39024 L
s
.21429 .40723 m
.21804 .40723 L
s
.21429 .44121 m
.21804 .44121 L
s
.21429 .4582 m
.21804 .4582 L
s
.21429 .47519 m
.21804 .47519 L
s
.21429 .49218 m
.21804 .49218 L
s
.21429 .52615 m
.21804 .52615 L
s
.21429 .54314 m
.21804 .54314 L
s
.21429 .56013 m
.21804 .56013 L
s
.21429 .57712 m
.21804 .57712 L
s
.21429 .6111 m
.21804 .6111 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02846 .06347 L
.03279 .10107 L
.03793 .13741 L
.04262 .16375 L
.04754 .18563 L
.0522 .20166 L
.05711 .21446 L
.05986 .22004 L
.06244 .22435 L
.06509 .22793 L
.06751 .23053 L
.07019 .23274 L
.07167 .23368 L
.07308 .2344 L
.07443 .23494 L
.07517 .23518 L
.07587 .23536 L
.07653 .23551 L
.07724 .23563 L
.07847 .23575 L
.07918 .23578 L
.07996 .23578 L
.08077 .23573 L
.08153 .23565 L
.08225 .23555 L
.08289 .23543 L
.08435 .23509 L
.08561 .2347 L
.08694 .23421 L
.08933 .23313 L
.09474 .22986 L
.10458 .22167 L
.14233 .17953 L
.18253 .13775 L
.20101 .12256 L
.22122 .10908 L
.24126 .0988 L
.25232 .09435 L
.26236 .09102 L
.27177 .08848 L
.28197 .08633 L
.28758 .08541 L
.29275 .08471 L
.29769 .08417 L
.30037 .08393 L
.30292 .08374 L
.30528 .08359 L
.30785 .08346 L
.31019 .08338 L
Mistroke
.31123 .08334 L
.31236 .08332 L
.31305 .0833 L
.31369 .08329 L
.31494 .08328 L
.31607 .08327 L
.31727 .08327 L
.31856 .08327 L
.31976 .08328 L
.32108 .08331 L
.3225 .08334 L
.32486 .08341 L
.32742 .08352 L
.33208 .08379 L
.33662 .08414 L
.34081 .08454 L
.35046 .08575 L
.35951 .08721 L
.38013 .09166 L
.40035 .09744 L
.41887 .10385 L
.45854 .12081 L
.4967 .14083 L
.53732 .1657 L
.57641 .19305 L
.61796 .22329 L
.658 .25545 L
.69652 .28957 L
.73749 .3286 L
.77694 .3686 L
.81488 .40919 L
.85528 .45458 L
.89415 .50033 L
.93548 .55113 L
.97529 .60215 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O?ooo00<00?ooool0
g_oo000oool00`00ooooo`3Nool003ooo`03003ooooo0=koo`00?ooo00<00?ooool0g_oo0009ool0
0`00ooooo`0cool00`00ooooo`3Nool000Woo`03003ooooo03?oo`800=ooo`002Ooo00<00?ooool0
<ooo00<00?ooool0g_oo0009ool00`00ooooo`0cool00`00ooooo`3Nool000Woo`03003ooooo03?o
o`03003ooooo0=koo`002Ooo00<00?ooool0<ooo00<00?ooool0g_oo0009ool00`00ooooo`0cool2
003Oool000Woo`03003ooooo03?oo`03003ooooo0=koo`002_oo00<00?ooool0<_oo00<00?ooool0
g_oo000:ool00`00ooooo`0bool00`00ooooo`3Nool000[oo`03003ooooo03;oo`800=ooo`002_oo
00<00?ooool0<_oo00<00?ooool0g_oo000:ool00`00ooooo`0bool00`00ooooo`3Nool000[oo`03
003ooooo03;oo`03003ooooo0=koo`002_oo00<00?ooool0<_oo00<00?ooool0g_oo000:ool00`00
ooooo`0bool2003Oool000[oo`03003ooooo03;oo`03003ooooo0=koo`002_oo00<00?ooool0<_oo
00<00?ooool0g_oo000:ool00`00ooooo`0Yool40005ool00`00ooooo`3Nool000[oo`03003ooooo
02Woo`03003ooooo00Koo`03003ooooo0=koo`002ooo00<00?ooool0:Ooo00<00?ooool01Ooo0`00
4Ooo5000^Ooo000;ool00`00ooooo`0Rool40004ool00`00ooooo`04ool00`00ooooo`0<ool5000D
ool4002eool000_oo`03003ooooo02_oo`03003ooooo00?oo`03003ooooo00Woo`<001goo`D00;3o
o`002ooo00<00?ooool0:?oo00@00?ooool000Goo`03003ooooo00Koo`<002Goo`<00:goo`002ooo
00<00?ooool0:Ooo0P001_oo00<00?ooool01?oo0P00:ooo0`00Z_oo000;ool00`00ooooo`0aool2
0003ool2000`ool2002Xool000_oo`03003ooooo037oo`03003ooooo008003Coo`<00:Goo`002ooo
00<00?ooool0<Ooo00<00?oo0000>Ooo0`00X_oo000<ool00`00ooooo`0`ool2000mool2002Pool0
00coo`03003ooooo02ooo`80043oo`<009goo`003?oo00<00?ooool0;Ooo0P0000?oo`000000@_oo
0P00Vooo000<ool00`00ooooo`0/ool01@00oooooooo0000AOoo0P00VOoo000<ool00`00ooooo`0[
ool00`00ooooo`02ool00`00ooooo`15ool2002Gool000goo`03003ooooo02Soo`8000Goo`03003o
oooo04Ooo`8009Goo`003Ooo00<00?ooool09ooo00<00?ooool01Ooo00<00?ooool0BOoo0P00Tooo
000=ool00`00ooooo`0Vool00`00ooooo`06ool2001<ool2002Aool000goo`03003ooooo02Goo`03
003ooooo00Ooo`03003ooooo04goo`03003ooooo08koo`003Ooo00<00?ooool09?oo00<00?ooool0
2?oo00<00?ooool0C_oo0P00S_oo000=ool00`00ooooo`0Sool00`00ooooo`09ool00`00ooooo`1@
ool2002<ool000goo`03003ooooo02;oo`03003ooooo00[oo`8005?oo`03003ooooo08Woo`003_oo
00<00?ooool08?oo00<00?ooool02ooo00<00?ooool0Dooo0P00ROoo000>ool00`00ooooo`0Oool0
0`00ooooo`0<ool00`00ooooo`1Eool20027ool000koo`03003ooooo01koo`03003ooooo00Coo`@0
00Goo`03003ooooo05Ooo`8008Goo`003_oo00<00?ooool07Ooo00<00?ooool01ooo00<00?ooool0
1?oo00<00?ooool0FOoo00<00?ooool0P_oo000>ool00`00ooooo`0Lool00`00ooooo`08ool00`00
ooooo`04ool3001Jool20022ool000koo`03003ooooo01_oo`04003oooooool40004ool00`00oooo
o`04ool00`00ooooo`1Lool00`00ooooo`1oool000koo`03003ooooo01[oo`03003ooooo00[oo`03
003ooooo00Coo`03003ooooo05goo`03003ooooo07koo`003ooo00<00?ooool06?oo00<00?ooool0
2Ooo0`001_oo00<00?ooool0G_oo0P00O_oo000?ool00`00ooooo`0Gool00`00ooooo`0<ool00`00
ooooo`04ool00`00ooooo`1Pool00`00ooooo`1kool000ooo`03003ooooo01Koo`03003ooooo01Co
o`8006;oo`03003ooooo07[oo`003ooo00<00?ooool05_oo00<00?ooool05?oo00<00?ooool0H_oo
0P00N_oo000@ool00`00ooooo`0Dool00`00ooooo`0Eool00`00ooooo`1Tool00`00ooooo`1gool0
013oo`03003ooooo01?oo`03003ooooo01Koo`03003ooooo06Goo`03003ooooo07Koo`004Ooo00<0
0?ooool04Ooo00<00?ooool05ooo00<00?ooool0I_oo0P00M_oo000Aool00`00ooooo`0@ool00`00
ooooo`0Hool2001Yool00`00ooooo`1cool0017oo`03003ooooo00ooo`03003ooooo01Woo`03003o
oooo06Woo`03003ooooo07;oo`004_oo00<00?ooool03_oo00<00?ooool06Ooo00<00?ooool0J_oo
0P00L_oo000Bool00`00ooooo`0=ool00`00ooooo`0Jool00`00ooooo`1/ool00`00ooooo`1_ool0
00;oo`8000Coo`03003ooooo00;oo`<000;oo`03003ooooo00coo`03003ooooo01_oo`03003ooooo
02_oo`@000?oo`03003ooooo00;oo`<002goo`@000;oo`03003ooooo02Ooo`@000?oo`03003ooooo
00;oo`<002ooo`8000Ooo`0000Goo`00ooooo`0000_oo`04003ooooo000<ool00`00ooooo`0Lool2
000^ool00`00ooooo`0:ool00`00ooooo`0Zool00`00ooooo`04ool2000Wool00`00ooooo`0<ool0
0`00ooooo`0[ool01000ooooo`001_oo00001Ooo003ooooo00002ooo00D00?ooooooo`0000Woo`80
01ooo`03003ooooo02goo`03003ooooo00[oo`03003ooooo02_oo`03003ooooo00Goo`03003ooooo
02Goo`03003ooooo00_oo`03003ooooo02koo`03003ooooo00Coo`0000Goo`00ooooo`0000Soo`<0
00Coo`03003ooooo00Goo`80027oo`03003ooooo02goo`03003ooooo00Ooo`<002ooo`03003ooooo
00Goo`03003ooooo02Goo`03003ooooo00Ooo`<002ooo`8000Ooo`0000Goo`00ooooo`0000Soo`03
003ooooo00Goo`8000Coo`03003ooooo027oo`03003ooooo02goo`03003ooooo00Ooo`03003ooooo
033oo`03003ooooo00Goo`8002Koo`03003ooooo00Koo`03003ooooo037oo`03003ooooo00Coo`00
00Goo`00ooooo`0000Soo`03003ooooo00Koo`D002Coo`03003ooooo02_oo`<000Woo`03003ooooo
02goo`04003ooooo0009ool00`00ooooo`0Pool01000ooooo`002?oo00<00?ooool0;_oo00@00?oo
ool000Koo`000_oo0P002Ooo1000;_oo0P00;_oo00<00?ooool01ooo1000;Ooo0P002ooo00<00?oo
ool08?oo0P002Ooo1000;_oo0P001ooo000oool00`00ooooo`1fool00`00ooooo`1Uool003ooo`03
003ooooo07Ooo`03003ooooo06Coo`00?ooo00<00?ooool0N?oo0P00I?oo0002oooo000O000000Wo
o`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00So
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00So
o`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00So
o`03003ooooo00Soo`03003ooooo00Coo`05003oooooool0000:ool00`00ooooo`08ool00`00oooo
o`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00oooo
o`08ool00`00ooooo`05ool000Woo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03
003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03
003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03
003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Goo`04003ooooo000:ool0
0`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool0
0`00ooooo`08ool00`00ooooo`08ool00`00ooooo`05ool003ooo`03003ooooo07goo`8005ooo`00
?ooo00<00?ooool0Oooo00<00?ooool0G?oo000oool20021ool00`00ooooo`1Kool003ooo`03003o
oooo087oo`03003ooooo05[oo`00?ooo00<00?ooool0P_oo00<00?ooool0FOoo000oool00`00oooo
o`23ool2001Iool003ooo`03003ooooo08Goo`03003ooooo05Koo`00?ooo0P00Qooo00<00?ooool0
EOoo000oool00`00ooooo`27ool00`00ooooo`1Dool003ooo`03003ooooo08Soo`03003ooooo05?o
o`00?ooo00<00?ooool0ROoo00<00?ooool0D_oo000oool00`00ooooo`2:ool00`00ooooo`1Aool0
03ooo`8008coo`03003ooooo053oo`00?ooo00<00?ooool0S?oo00<00?ooool0Cooo000oool00`00
ooooo`2=ool00`00ooooo`1>ool003ooo`03003ooooo08koo`03003ooooo04goo`00?ooo00<00?oo
ool0Sooo00<00?ooool0C?oo000oool2002Aool00`00ooooo`1;ool003ooo`03003ooooo097oo`03
003ooooo04[oo`00?ooo00<00?ooool0T_oo00<00?ooool0BOoo000fool40005ool00`00ooooo`2C
ool00`00ooooo`18ool003Soo`03003ooooo00Coo`03003ooooo09Coo`03003ooooo04Ooo`00>?oo
00<00?ooool01?oo0`00UOoo00<00?ooool0A_oo000hool00`00ooooo`04ool00`00ooooo`2Fool0
0`00ooooo`15ool003Soo`03003ooooo00Coo`03003ooooo09Ooo`03003ooooo04Coo`00=_oo0`00
1_oo00<00?ooool0V?oo00<00?ooool0@ooo000hool00`00ooooo`04ool2002Jool00`00ooooo`12
ool003ooo`03003ooooo09[oo`03003ooooo047oo`00?ooo00<00?ooool0Vooo00<00?ooool0@?oo
000oool00`00ooooo`2Lool00`00ooooo`0oool003ooo`03003ooooo09goo`03003ooooo03koo`00
?ooo0P00Wooo00<00?ooool0?Ooo000oool00`00ooooo`2Oool00`00ooooo`0lool003ooo`03003o
oooo0:3oo`03003ooooo03_oo`00?ooo00<00?ooool0XOoo00<00?ooool0>_oo000oool00`00oooo
o`2Rool00`00ooooo`0iool003ooo`800:?oo`03003ooooo03Woo`00?ooo00<00?ooool0Xooo00<0
0?ooool0>?oo000oool00`00ooooo`2Tool00`00ooooo`0gool003ooo`03003ooooo0:Goo`03003o
oooo03Koo`00?ooo00<00?ooool0Y_oo00<00?ooool0=Ooo000oool2002Xool00`00ooooo`0dool0
03ooo`03003ooooo0:Soo`03003ooooo03?oo`00?ooo00<00?ooool0ZOoo00<00?ooool0<_oo000f
ool40005ool00`00ooooo`2Zool00`00ooooo`0aool003Koo`03003ooooo00Koo`03003ooooo0:_o
o`03003ooooo033oo`00=ooo00<00?ooool01Ooo0`00[?oo00<00?ooool0;ooo000hool00`00oooo
o`04ool00`00ooooo`2]ool00`00ooooo`0^ool003Woo`03003ooooo00?oo`03003ooooo0:goo`03
003ooooo02koo`00=_oo00@00?ooool000Goo`03003ooooo0:koo`03003ooooo02goo`00=ooo0P00
1_oo00<00?ooool0[ooo00<00?ooool0;?oo000oool2002aool00`00ooooo`0[ool003ooo`03003o
oooo0;7oo`03003ooooo02[oo`00?ooo00<00?ooool0/_oo00<00?ooool0:Ooo000oool00`00oooo
o`2cool00`00ooooo`0Xool003ooo`03003ooooo0;Coo`03003ooooo02Ooo`00?ooo0P00]_oo00<0
0?ooool09_oo000oool00`00ooooo`2eool00`00ooooo`0Vool003ooo`03003ooooo0;Koo`03003o
oooo02Goo`00?ooo00<00?ooool0]ooo00<00?ooool09?oo000oool2002iool00`00ooooo`0Sool0
03ooo`03003ooooo0;Soo`03003ooooo02?oo`00?ooo00<00?ooool0^Ooo00<00?ooool08_oo000o
ool00`00ooooo`2jool00`00ooooo`0Qool003ooo`03003ooooo0;_oo`03003ooooo023oo`00?ooo
0P00_Ooo00<00?ooool07ooo000oool00`00ooooo`2lool00`00ooooo`0Oool003ooo`03003ooooo
0;goo`03003ooooo01koo`00=ooo0P001_oo00<00?ooool0__oo00<00?ooool07Ooo000fool01000
ooooo`001Ooo00<00?ooool0_ooo00<00?ooool07?oo000iool00`00ooooo`03ool30030ool00`00
ooooo`0Kool003Ooo`8000Koo`03003ooooo0<7oo`03003ooooo01[oo`00>Ooo00<00?ooool00ooo
00<00?ooool0`Ooo00<00?ooool06_oo000fool01000ooooo`001Ooo00<00?ooool0`_oo00<00?oo
ool06Ooo000gool20006ool00`00ooooo`33ool00`00ooooo`0Hool003ooo`800<Goo`03003ooooo
01Ooo`00?ooo00<00?ooool0aOoo00<00?ooool05_oo000oool00`00ooooo`36ool00`00ooooo`0E
ool003ooo`03003ooooo0<Ooo`03003ooooo01Coo`00?ooo00<00?ooool0aooo00<00?ooool05?oo
000oool20039ool00`00ooooo`0Cool003ooo`03003ooooo0<Woo`03003ooooo01;oo`00?ooo00<0
0?ooool0b_oo00<00?ooool04Ooo000oool00`00ooooo`3;ool00`00ooooo`0@ool003ooo`03003o
oooo0<_oo`03003ooooo013oo`00?ooo0P00cOoo00<00?ooool03ooo000oool00`00ooooo`3=ool0
0`00ooooo`0>ool003ooo`03003ooooo0<koo`03003ooooo00goo`00?ooo00<00?ooool0c_oo00<0
0?ooool03Ooo000oool2003@ool00`00ooooo`0<ool003ooo`03003ooooo0=3oo`03003ooooo00_o
o`00?ooo00<00?ooool0dOoo00<00?ooool02_oo000hool00`00ooooo`04ool00`00ooooo`3Aool0
0`00ooooo`0:ool003Soo`03003ooooo00Coo`03003ooooo0=;oo`03003ooooo00Woo`00=Ooo1@00
1Ooo0`00dooo00<00?ooool02?oo000eool01000ooooo`001_oo00<00?ooool0e?oo00<00?ooool0
1ooo000fool00`00ool00006ool00`00ooooo`3Dool00`00ooooo`07ool003Ooo`8000Koo`03003o
oooo0=Goo`8000Ooo`00>?oo00<00?ooool01?oo00<00?ooool0g_oo000oool2003Oool003ooo`03
003ooooo0=koo`00?ooo00<00?ooool0g_oo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.41703, -3.01339, \
0.00921772, 0.0413385}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[kx0corrtestfunc[om, 1, 10]], {om, 0.5, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.380952 0.268565 0.293259 [
[.02381 .25607 -9 -9 ]
[.02381 .25607 9 0 ]
[.40476 .25607 -9 -9 ]
[.40476 .25607 9 0 ]
[.59524 .25607 -3 -9 ]
[.59524 .25607 3 0 ]
[.78571 .25607 -9 -9 ]
[.78571 .25607 9 0 ]
[.97619 .25607 -3 -9 ]
[.97619 .25607 3 0 ]
[.20179 .12194 -24 -4.5 ]
[.20179 .12194 0 4.5 ]
[.20179 .41519 -18 -4.5 ]
[.20179 .41519 0 4.5 ]
[.20179 .56182 -6 -4.5 ]
[.20179 .56182 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .26857 m
.02381 .27482 L
s
[(0.5)] .02381 .25607 0 1 Mshowa
.40476 .26857 m
.40476 .27482 L
s
[(1.5)] .40476 .25607 0 1 Mshowa
.59524 .26857 m
.59524 .27482 L
s
[(2)] .59524 .25607 0 1 Mshowa
.78571 .26857 m
.78571 .27482 L
s
[(2.5)] .78571 .25607 0 1 Mshowa
.97619 .26857 m
.97619 .27482 L
s
[(3)] .97619 .25607 0 1 Mshowa
.125 Mabswid
.0619 .26857 m
.0619 .27232 L
s
.1 .26857 m
.1 .27232 L
s
.1381 .26857 m
.1381 .27232 L
s
.17619 .26857 m
.17619 .27232 L
s
.25238 .26857 m
.25238 .27232 L
s
.29048 .26857 m
.29048 .27232 L
s
.32857 .26857 m
.32857 .27232 L
s
.36667 .26857 m
.36667 .27232 L
s
.44286 .26857 m
.44286 .27232 L
s
.48095 .26857 m
.48095 .27232 L
s
.51905 .26857 m
.51905 .27232 L
s
.55714 .26857 m
.55714 .27232 L
s
.63333 .26857 m
.63333 .27232 L
s
.67143 .26857 m
.67143 .27232 L
s
.70952 .26857 m
.70952 .27232 L
s
.74762 .26857 m
.74762 .27232 L
s
.82381 .26857 m
.82381 .27232 L
s
.8619 .26857 m
.8619 .27232 L
s
.9 .26857 m
.9 .27232 L
s
.9381 .26857 m
.9381 .27232 L
s
.25 Mabswid
0 .26857 m
1 .26857 L
s
.21429 .12194 m
.22054 .12194 L
s
[(-0.5)] .20179 .12194 1 0 Mshowa
.21429 .41519 m
.22054 .41519 L
s
[(0.5)] .20179 .41519 1 0 Mshowa
.21429 .56182 m
.22054 .56182 L
s
[(1)] .20179 .56182 1 0 Mshowa
.125 Mabswid
.21429 .15126 m
.21804 .15126 L
s
.21429 .18059 m
.21804 .18059 L
s
.21429 .20991 m
.21804 .20991 L
s
.21429 .23924 m
.21804 .23924 L
s
.21429 .29789 m
.21804 .29789 L
s
.21429 .32722 m
.21804 .32722 L
s
.21429 .35654 m
.21804 .35654 L
s
.21429 .38587 m
.21804 .38587 L
s
.21429 .44452 m
.21804 .44452 L
s
.21429 .47385 m
.21804 .47385 L
s
.21429 .50317 m
.21804 .50317 L
s
.21429 .5325 m
.21804 .5325 L
s
.21429 .09261 m
.21804 .09261 L
s
.21429 .06328 m
.21804 .06328 L
s
.21429 .03396 m
.21804 .03396 L
s
.21429 .00463 m
.21804 .00463 L
s
.21429 .59115 m
.21804 .59115 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.0407 0 m
.04262 .01973 L
.05288 .09725 L
.06244 .14965 L
.06792 .17283 L
.0731 .19097 L
.0828 .2168 L
.08823 .2275 L
.09324 .23544 L
.09875 .24236 L
.10458 .24798 L
.10949 .2516 L
.11399 .25417 L
.11918 .25642 L
.12411 .25798 L
.12865 .25904 L
.13286 .25976 L
.13749 .26034 L
.14241 .2608 L
.1527 .26155 L
.15811 .26201 L
.16383 .26268 L
.16883 .26349 L
.17433 .26474 L
.17933 .26629 L
.184 .26823 L
.189 .27102 L
.19164 .27287 L
.19445 .27522 L
.19689 .27769 L
.19959 .28101 L
.20207 .28483 L
.20435 .28927 L
.20659 .29503 L
.20772 .29871 L
.20896 .30367 L
.2102 .31018 L
.21089 .31487 L
.21153 .32015 L
.21276 .33574 L
.21392 .37547 L
.21501 .35229 L
.21616 .32234 L
.21715 .30866 L
.21823 .29807 L
.2194 .28943 L
.22065 .28209 L
.22291 .27191 L
.22545 .2634 L
.22816 .25652 L
Mistroke
.23304 .24773 L
.23602 .24393 L
.23883 .24116 L
.24135 .23922 L
.24408 .23762 L
.24651 .23658 L
.24789 .23613 L
.24918 .2358 L
.24988 .23566 L
.25054 .23554 L
.25178 .23537 L
.25247 .2353 L
.25311 .23526 L
.25381 .23524 L
.25456 .23523 L
.25569 .23526 L
.25692 .23535 L
.25808 .23548 L
.25913 .23564 L
.2616 .23615 L
.26418 .2369 L
.2692 .23888 L
.27375 .24123 L
.28396 .24817 L
.30234 .26533 L
.34238 .31786 L
.3809 .38259 L
.42187 .46316 L
.46132 .55038 L
Mfstroke
.46132 .55038 m
.48927 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00O3Ooo00<00?ooool0
;ooo00<00?ooool0g_oo000=ool00`00ooooo`0_ool2003Oool000goo`03003ooooo02ooo`03003o
oooo0=koo`003Ooo00<00?ooool0;ooo00<00?ooool0g_oo000>ool00`00ooooo`0^ool00`00oooo
o`3Nool000koo`03003ooooo02koo`03003ooooo0=koo`003_oo00<00?ooool0;_oo00<00?ooool0
g_oo000>ool00`00ooooo`0^ool00`00ooooo`3Nool000koo`03003ooooo02koo`03003ooooo0=ko
o`003_oo00<00?ooool0;_oo00<00?ooool0g_oo000?ool00`00ooooo`0]ool2003Oool000ooo`03
003ooooo02goo`03003ooooo0=koo`003ooo00<00?ooool0;Ooo00<00?ooool0g_oo000?ool00`00
ooooo`0]ool00`00ooooo`3Nool000ooo`03003ooooo02goo`03003ooooo0=koo`003ooo00<00?oo
ool0;Ooo00<00?ooool0g_oo000?ool00`00ooooo`0]ool00`00ooooo`3Nool000ooo`03003ooooo
02goo`03003ooooo0=koo`004?oo00<00?ooool0;?oo0P00gooo000@ool00`00ooooo`0/ool00`00
ooooo`3Nool0013oo`03003ooooo02coo`03003ooooo0=koo`004?oo00<00?ooool0;?oo00<00?oo
ool0g_oo000@ool00`00ooooo`0/ool00`00ooooo`3Nool0013oo`03003ooooo02coo`03003ooooo
0=koo`004?oo00<00?ooool0;?oo00<00?ooool0g_oo000Aool00`00ooooo`0[ool00`00ooooo`3N
ool0017oo`03003ooooo02_oo`800=ooo`004Ooo00<00?ooool0:ooo00<00?ooool0g_oo000Aool0
0`00ooooo`0[ool00`00ooooo`3Nool0017oo`03003ooooo02_oo`03003ooooo0=koo`004Ooo00<0
0?ooool0:ooo00<00?ooool0g_oo000Bool00`00ooooo`0Zool00`00ooooo`3Nool001;oo`03003o
oooo02[oo`03003ooooo0=koo`004_oo00<00?ooool05_oo0P001?oo00<00?ooool00_oo0`001_oo
00<00?ooool0g_oo000Bool00`00ooooo`0Eool01000ooooo`002ooo00<00?ooool00ooo00<00?oo
ool0g_oo000Bool00`00ooooo`0Eool01000ooooo`002ooo00<00?ooool00ooo0`00g_oo000Cool0
0`00ooooo`0>ool40002ool01000ooooo`002?oo0`001_oo00<00?ooool0g_oo000Cool00`00oooo
o`0Dool01000ooooo`002?oo00<00?ooool01_oo00<00?ooool0g_oo000Cool00`00ooooo`0Dool0
1000ooooo`002?oo00<00?ooool01_oo00<00?ooool0g_oo000Cool00`00ooooo`0Eool20009ool4
0005ool00`00ooooo`3Nool001?oo`03003ooooo02Woo`03003ooooo0=koo`005?oo00<00?ooool0
:?oo00<00?ooool0g_oo000Dool00`00ooooo`0Xool00`00ooooo`3Nool001Coo`03003ooooo02So
o`800=ooo`005?oo00<00?ooool0:?oo00<00?ooool0g_oo000Dool00`00ooooo`0Xool00`00oooo
o`3Nool001Coo`03003ooooo02Soo`03003ooooo0=koo`005Ooo00<00?ooool09ooo00<00?ooool0
g_oo000Eool00`00ooooo`0Wool00`00ooooo`3Nool001Goo`03003ooooo02Ooo`03003ooooo0=ko
o`005Ooo00<00?ooool09ooo00<00?ooool0g_oo000Fool00`00ooooo`0Vool2003Oool001Koo`03
003ooooo02Koo`03003ooooo0=koo`005ooo00<00?ooool09Ooo00<00?ooool0g_oo000Gool00`00
ooooo`0Uool00`00ooooo`3Nool001Ooo`03003ooooo02Goo`03003ooooo0=koo`005ooo00<00?oo
ool09Ooo00<00?ooool0g_oo000Hool00`00ooooo`0Tool00`00ooooo`3Nool001Soo`03003ooooo
02Coo`03003ooooo0=koo`006?oo00<00?ooool09?oo00<00?ooool0g_oo000Hool00`00ooooo`0T
ool2003Oool001Woo`03003ooooo02?oo`03003ooooo0=koo`006Ooo00<00?ooool08ooo00<00?oo
ool0g_oo000Jool00`00ooooo`0Rool00`00ooooo`3Nool001[oo`03003ooooo02;oo`03003ooooo
0=koo`006ooo00<00?ooool08Ooo00<00?ooool0g_oo0002ool20004ool00`00ooooo`02ool3000;
ool00`00ooooo`0Qool00`00ooooo`0[ool40003ool00`00ooooo`02ool3000]ool4000/ool40003
ool00`00ooooo`02ool3000_ool20007ool00005ool00?ooool0000;ool00`00ooooo`09ool00`00
ooooo`0Pool00`00ooooo`06ool6000Qool00`00ooooo`0:ool00`00ooooo`0Zool00`00ooooo`0]
ool00`00ooooo`0<ool00`00ooooo`0[ool01000ooooo`001_oo00001Ooo003ooooo00002ooo00<0
0?ooool02_oo00<00?ooool07ooo0P001Ooo0P001_oo00<00?ooool07_oo00<00?ooool02_oo00<0
0?ooool0:ooo00<00?ooool0;Ooo00<00?ooool02ooo00<00?ooool0;_oo00<00?ooool01?oo0000
1Ooo003ooooo00002?oo0`003_oo00<00?ooool07_oo00<00?ooool00ooo0P002?oo0P007_oo00<0
0?ooool01ooo0`00;ooo00<00?ooool0;Ooo00<00?ooool01ooo0`00;ooo0P001ooo00001Ooo003o
oooo00002?oo00<00?ooool03ooo00<00?ooool07Ooo00<00?ooool00ooo00<00?ooool02Ooo0P00
7?oo00<00?ooool01ooo00<00?ooool0<?oo00<00?ooool0;Ooo00<00?ooool01_oo00<00?ooool0
<Ooo00<00?ooool01?oo00001Ooo003ooooo00002?oo00<00?ooool04?oo00<00?ooool07?oo00<0
0?ooool00_oo00<00?ooool03?oo00<00?ooool05ooo0`002Ooo00<00?ooool0;Ooo00@00?ooool0
02coo`04003ooooo0008ool00`00ooooo`0^ool01000ooooo`001_oo0002ool20009ool4000@ool2
000Lool00`00ooooo`02ool00`00ooooo`0=ool00`00ooooo`0Hool00`00ooooo`07ool4000]ool2
000^ool20009ool4000^ool20007ool002?oo`<001Woo`05003oooooool0000Aool00`00ooooo`38
ool002Koo`H001?oo`05003oooooool0000Bool00`00ooooo`37ool002coo`P000_oo`04003ooooo
000Dool00`00ooooo`36ool003Coo`<000Soo`04003ooooo000Eool00`00ooooo`35ool000;oool0
01l000002Ooo00<00?ooool01ooo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool02?oo00@0
0?ooooooo`8000Goo`03003oo`0000Soo`03003ooooo00Ooo`03003ooooo00;oo`03003ooooo00?o
o`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00So
o`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00So
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00So
o`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Goo`00>_oo00<00?ooool00_oo00<00?oo
00006?oo00<00?ooool0`ooo000kool01`00oooooooo003oo`0001Woo`03003ooooo0<;oo`00>ooo
00L00?ooooooo`00ool0000Jool00`00ooooo`31ool003coo`03003ooooo008001_oo`03003ooooo
0<7oo`00??oo00<00?ooool00P007?oo00<00?ooool0`?oo000mool01000ool000007Ooo00<00?oo
ool0_ooo000mool01000ool000007_oo00<00?ooool0__oo000mool01000ool000007_oo00<00?oo
ool0__oo000mool01000ool000007ooo00<00?ooool0_Ooo000nool3000Pool00`00ooooo`2lool0
03koo`<0027oo`03003ooooo0;_oo`00?_oo0`008Ooo00<00?ooool0^ooo000nool2000Sool00`00
ooooo`2jool003koo`8002Coo`03003ooooo0;Woo`00?_oo0`008ooo00<00?ooool0^Ooo000nool2
000Uool00`00ooooo`2hool003koo`8002Goo`03003ooooo0;Soo`00?_oo0P009_oo00<00?ooool0
]ooo000nool2000Wool00`00ooooo`2fool003koo`8002Ooo`03003ooooo0;Koo`00?_oo0P00:?oo
00<00?ooool0]Ooo000nool2000Xool00`00ooooo`2eool003koo`8002Woo`03003ooooo0;Coo`00
?ooo0P00:Ooo00<00?ooool0/ooo000oool00`00ooooo`0Xool00`00ooooo`2cool003ooo`03003o
oooo02Woo`03003ooooo0;;oo`00?ooo00<00?ooool0:_oo00<00?ooool0/Ooo000oool00`00oooo
o`0Zool00`00ooooo`2aool003ooo`03003ooooo02_oo`03003ooooo0;3oo`00?ooo00<00?ooool0
:ooo00<00?ooool0/?oo000oool00`00ooooo`0/ool00`00ooooo`2_ool003ooo`8002koo`03003o
oooo0:koo`00?ooo00<00?ooool0;Ooo00<00?ooool0[_oo000oool00`00ooooo`0^ool00`00oooo
o`2]ool003ooo`03003ooooo02koo`03003ooooo0:goo`00?ooo00<00?ooool0;ooo00<00?ooool0
[?oo000oool00`00ooooo`0_ool00`00ooooo`2/ool002_oo`8000Coo`03003ooooo00;oo`<000Ko
o`03003ooooo033oo`03003ooooo0:_oo`00:_oo00@00?ooool000_oo`03003ooooo00?oo`03003o
oooo033oo`03003ooooo0:_oo`00:_oo00@00?ooool000_oo`03003ooooo00?oo`<0037oo`03003o
oooo0:[oo`00:_oo00@00?ooool000Soo`<000Koo`03003ooooo037oo`03003ooooo0:[oo`00:_oo
00@00?ooool000Soo`03003ooooo00Koo`03003ooooo03;oo`03003ooooo0:Woo`00:_oo00@00?oo
ool000Soo`03003ooooo00Koo`03003ooooo03;oo`03003ooooo0:Woo`00:ooo0P002Ooo10001Ooo
00<00?ooool0<ooo00<00?ooool0Z?oo000oool00`00ooooo`0cool00`00ooooo`2Xool003ooo`03
003ooooo03Coo`03003ooooo0:Ooo`00?ooo00<00?ooool0=?oo00<00?ooool0Yooo000oool00`00
ooooo`0eool00`00ooooo`2Vool003ooo`8003Koo`03003ooooo0:Koo`00?ooo00<00?ooool0=_oo
00<00?ooool0YOoo000oool00`00ooooo`0fool00`00ooooo`2Uool003ooo`03003ooooo03Ooo`03
003ooooo0:Coo`00?ooo00<00?ooool0=ooo00<00?ooool0Y?oo000oool00`00ooooo`0hool00`00
ooooo`2Sool003ooo`03003ooooo03Soo`03003ooooo0:?oo`00?ooo00<00?ooool0>Ooo00<00?oo
ool0X_oo000oool2000jool00`00ooooo`2Rool003ooo`03003ooooo03[oo`03003ooooo0:7oo`00
?ooo00<00?ooool0>_oo00<00?ooool0XOoo000oool00`00ooooo`0kool00`00ooooo`2Pool003oo
o`03003ooooo03_oo`03003ooooo0:3oo`00?ooo00<00?ooool0??oo00<00?ooool0Wooo000oool0
0`00ooooo`0lool00`00ooooo`2Oool003ooo`03003ooooo03coo`03003ooooo09ooo`00?ooo0P00
?_oo00<00?ooool0W_oo000oool00`00ooooo`0mool00`00ooooo`2Nool003ooo`03003ooooo03ko
o`03003ooooo09goo`00?ooo00<00?ooool0?_oo00<00?ooool0WOoo000oool00`00ooooo`0oool0
0`00ooooo`2Lool003ooo`03003ooooo03ooo`03003ooooo09coo`00?ooo00<00?ooool0?ooo00<0
0?ooool0W?oo000oool00`00ooooo`10ool00`00ooooo`2Kool003ooo`03003ooooo043oo`03003o
oooo09_oo`00?ooo0P00@_oo00<00?ooool0V_oo000oool00`00ooooo`11ool00`00ooooo`2Jool0
03ooo`03003ooooo04;oo`03003ooooo09Woo`00?ooo00<00?ooool0@_oo00<00?ooool0VOoo000o
ool00`00ooooo`13ool00`00ooooo`2Hool003ooo`03003ooooo04?oo`03003ooooo09Soo`00=_oo
10001Ooo00<00?ooool0@ooo00<00?ooool0V?oo000hool00`00ooooo`04ool00`00ooooo`14ool0
0`00ooooo`2Gool003Soo`03003ooooo00Coo`<004Coo`03003ooooo09Ooo`00>?oo00<00?ooool0
1?oo00<00?ooool0AOoo00<00?ooool0U_oo000hool00`00ooooo`04ool00`00ooooo`15ool00`00
ooooo`2Fool003Koo`<000Koo`03003ooooo04Koo`03003ooooo09Goo`00>?oo00<00?ooool01?oo
00<00?ooool0A_oo00<00?ooool0UOoo000oool00`00ooooo`16ool00`00ooooo`2Eool003ooo`03
003ooooo04Ooo`03003ooooo09Coo`00?ooo00<00?ooool0Aooo00<00?ooool0U?oo000oool20019
ool00`00ooooo`2Cool003ooo`03003ooooo04Soo`03003ooooo09?oo`00?ooo00<00?ooool0B?oo
00<00?ooool0Tooo000oool00`00ooooo`19ool00`00ooooo`2Bool003ooo`03003ooooo04Woo`03
003ooooo09;oo`00?ooo00<00?ooool0B_oo00<00?ooool0TOoo000oool00`00ooooo`1:ool00`00
ooooo`2Aool003ooo`03003ooooo04_oo`03003ooooo093oo`00?ooo00<00?ooool0Booo00<00?oo
ool0T?oo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {0.41703, -0.9214, \
0.00921772, 0.0119741}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Re[kx0corrtestfunc[om, 1, 0.1]], {om, 0.1, 3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.207715 0.0939967 [
[.18254 .19521 -9 -9 ]
[.18254 .19521 9 0 ]
[.34127 .19521 -3 -9 ]
[.34127 .19521 3 0 ]
[.5 .19521 -9 -9 ]
[.5 .19521 9 0 ]
[.65873 .19521 -3 -9 ]
[.65873 .19521 3 0 ]
[.81746 .19521 -9 -9 ]
[.81746 .19521 9 0 ]
[.97619 .19521 -3 -9 ]
[.97619 .19521 3 0 ]
[.01131 .01972 -12 -4.5 ]
[.01131 .01972 0 4.5 ]
[.01131 .11372 -12 -4.5 ]
[.01131 .11372 0 4.5 ]
[.01131 .30171 -6 -4.5 ]
[.01131 .30171 0 4.5 ]
[.01131 .39571 -6 -4.5 ]
[.01131 .39571 0 4.5 ]
[.01131 .4897 -6 -4.5 ]
[.01131 .4897 0 4.5 ]
[.01131 .5837 -6 -4.5 ]
[.01131 .5837 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .20771 m
.18254 .21396 L
s
[(0.5)] .18254 .19521 0 1 Mshowa
.34127 .20771 m
.34127 .21396 L
s
[(1)] .34127 .19521 0 1 Mshowa
.5 .20771 m
.5 .21396 L
s
[(1.5)] .5 .19521 0 1 Mshowa
.65873 .20771 m
.65873 .21396 L
s
[(2)] .65873 .19521 0 1 Mshowa
.81746 .20771 m
.81746 .21396 L
s
[(2.5)] .81746 .19521 0 1 Mshowa
.97619 .20771 m
.97619 .21396 L
s
[(3)] .97619 .19521 0 1 Mshowa
.125 Mabswid
.05556 .20771 m
.05556 .21146 L
s
.0873 .20771 m
.0873 .21146 L
s
.11905 .20771 m
.11905 .21146 L
s
.15079 .20771 m
.15079 .21146 L
s
.21429 .20771 m
.21429 .21146 L
s
.24603 .20771 m
.24603 .21146 L
s
.27778 .20771 m
.27778 .21146 L
s
.30952 .20771 m
.30952 .21146 L
s
.37302 .20771 m
.37302 .21146 L
s
.40476 .20771 m
.40476 .21146 L
s
.43651 .20771 m
.43651 .21146 L
s
.46825 .20771 m
.46825 .21146 L
s
.53175 .20771 m
.53175 .21146 L
s
.56349 .20771 m
.56349 .21146 L
s
.59524 .20771 m
.59524 .21146 L
s
.62698 .20771 m
.62698 .21146 L
s
.69048 .20771 m
.69048 .21146 L
s
.72222 .20771 m
.72222 .21146 L
s
.75397 .20771 m
.75397 .21146 L
s
.78571 .20771 m
.78571 .21146 L
s
.84921 .20771 m
.84921 .21146 L
s
.88095 .20771 m
.88095 .21146 L
s
.9127 .20771 m
.9127 .21146 L
s
.94444 .20771 m
.94444 .21146 L
s
.25 Mabswid
0 .20771 m
1 .20771 L
s
.02381 .01972 m
.03006 .01972 L
s
[(-2)] .01131 .01972 1 0 Mshowa
.02381 .11372 m
.03006 .11372 L
s
[(-1)] .01131 .11372 1 0 Mshowa
.02381 .30171 m
.03006 .30171 L
s
[(1)] .01131 .30171 1 0 Mshowa
.02381 .39571 m
.03006 .39571 L
s
[(2)] .01131 .39571 1 0 Mshowa
.02381 .4897 m
.03006 .4897 L
s
[(3)] .01131 .4897 1 0 Mshowa
.02381 .5837 m
.03006 .5837 L
s
[(4)] .01131 .5837 1 0 Mshowa
.125 Mabswid
.02381 .03852 m
.02756 .03852 L
s
.02381 .05732 m
.02756 .05732 L
s
.02381 .07612 m
.02756 .07612 L
s
.02381 .09492 m
.02756 .09492 L
s
.02381 .13252 m
.02756 .13252 L
s
.02381 .15132 m
.02756 .15132 L
s
.02381 .17012 m
.02756 .17012 L
s
.02381 .18892 m
.02756 .18892 L
s
.02381 .22651 m
.02756 .22651 L
s
.02381 .24531 m
.02756 .24531 L
s
.02381 .26411 m
.02756 .26411 L
s
.02381 .28291 m
.02756 .28291 L
s
.02381 .32051 m
.02756 .32051 L
s
.02381 .33931 m
.02756 .33931 L
s
.02381 .35811 m
.02756 .35811 L
s
.02381 .37691 m
.02756 .37691 L
s
.02381 .41451 m
.02756 .41451 L
s
.02381 .43331 m
.02756 .43331 L
s
.02381 .45211 m
.02756 .45211 L
s
.02381 .47091 m
.02756 .47091 L
s
.02381 .5085 m
.02756 .5085 L
s
.02381 .5273 m
.02756 .5273 L
s
.02381 .5461 m
.02756 .5461 L
s
.02381 .5649 m
.02756 .5649 L
s
.02381 .00092 m
.02756 .00092 L
s
.02381 .6025 m
.02756 .6025 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05556 .01779 m
.07374 .01689 L
.0929 .01581 L
.10229 .01534 L
.10746 .01512 L
.11234 .01495 L
.11511 .01487 L
.11811 .0148 L
.11944 .01477 L
.12084 .01475 L
.12204 .01474 L
.12336 .01473 L
.12405 .01472 L
.1247 .01472 L
.12592 .01472 L
.1266 .01472 L
.12723 .01472 L
.12793 .01472 L
.12867 .01472 L
.12931 .01473 L
.13001 .01474 L
.13127 .01475 L
.1325 .01477 L
.13363 .0148 L
.13601 .01486 L
.13818 .01494 L
.14304 .01517 L
.14756 .01548 L
.15183 .01585 L
.1614 .01701 L
.1668 .01789 L
.1718 .01886 L
.18985 .02379 L
.20012 .02774 L
.20976 .03231 L
.22927 .0444 L
.24716 .05928 L
.26545 .07877 L
.28546 .10573 L
.30442 .13778 L
.32467 .18283 L
.32948 .197 L
.33211 .20611 L
.33457 .21621 L
.33596 .22308 L
.33723 .23063 L
.33864 .24169 L
.33942 .25024 L
.34016 .26221 L
.34144 .30466 L
Mistroke
.34262 .25619 L
.3433 .24679 L
.34392 .24077 L
.34464 .23544 L
.34531 .23144 L
.34644 .22613 L
.34768 .22168 L
.34889 .21825 L
.35019 .21528 L
.35251 .2112 L
.35467 .2084 L
.35739 .20581 L
.35997 .20404 L
.36228 .20288 L
.36359 .20237 L
.36479 .20198 L
.36705 .20143 L
.36824 .20122 L
.36949 .20106 L
.37058 .20096 L
.3716 .2009 L
.37271 .20087 L
.3739 .20088 L
.37457 .2009 L
.3752 .20092 L
.37588 .20097 L
.37661 .20102 L
.37782 .20114 L
.37916 .2013 L
.38163 .20169 L
.38394 .20215 L
.39374 .20493 L
.40426 .20905 L
.44421 .23124 L
.4827 .25873 L
.51972 .28917 L
.55911 .32519 L
.59704 .36309 L
.6335 .40235 L
.67233 .44712 L
.7097 .493 L
.74943 .54478 L
.7877 .59753 L
Mfstroke
.7877 .59753 m
.80185 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool000ooo`800?oooa3oo`003ooo00<00?ooool0oooo3ooo000?ool00`00ooooo`3oool?ool000oo
o`03003ooooo0?ooo`ooo`001ooo10001?oo00<00?ooool03_oo5000k?oo0007ool00`00ooooo`05
ool00`00ooooo`06ool8000Dool6003Vool000Soo`03003ooooo00Coo`<002Soo`800>Coo`0000?o
o`0000000P001?oo00<00?ooool00ooo00<00?ooool0:_oo0`00hOoo000:ool00`00ooooo`02ool0
0`00ooooo`0]ool3003Nool000Ooo`04003ooooo0004ool00`00ooooo`0`ool2003Lool000Soo`80
00Goo`03003ooooo03;oo`800=[oo`003ooo0P00=Ooo0P00f?oo000?ool00`00ooooo`0fool00`00
ooooo`3Eool000ooo`03003ooooo03Ooo`03003ooooo0=Coo`003ooo00<00?ooool0>?oo00<00?oo
ool0dooo000?ool00`00ooooo`0iool00`00ooooo`3Bool000ooo`8003_oo`03003ooooo0=7oo`00
3ooo00<00?ooool0>ooo00<00?ooool0d?oo000?ool00`00ooooo`0lool00`00ooooo`3?ool000oo
o`03003ooooo03goo`03003ooooo0<koo`003ooo00<00?ooool0?_oo00<00?ooool0cOoo000?ool2
0010ool00`00ooooo`3<ool000ooo`03003ooooo043oo`03003ooooo0<_oo`003ooo00<00?ooool0
@Ooo00<00?ooool0b_oo000?ool00`00ooooo`12ool00`00ooooo`39ool000ooo`03003ooooo04?o
o`03003ooooo0<Soo`003ooo0P00A?oo00<00?ooool0b?oo000?ool00`00ooooo`14ool00`00oooo
o`37ool000ooo`03003ooooo04Goo`03003ooooo0<Koo`003ooo00<00?ooool0A_oo00<00?ooool0
aOoo0007ool40004ool00`00ooooo`17ool00`00ooooo`34ool000Woo`03003ooooo00?oo`03003o
oooo04Ooo`03003ooooo0<Coo`002Ooo00<00?ooool00ooo0`00B?oo00<00?ooool0`ooo00000ooo
000000020004ool00`00ooooo`03ool00`00ooooo`18ool00`00ooooo`33ool000Woo`03003ooooo
00?oo`03003ooooo04Woo`03003ooooo0<;oo`001ooo0`001Ooo00<00?ooool0BOoo00<00?ooool0
`_oo0009ool00`00ooooo`03ool00`00ooooo`1:ool00`00ooooo`31ool000ooo`8004_oo`03003o
oooo0<7oo`003ooo00<00?ooool0Booo00<00?ooool0`?oo000?ool00`00ooooo`1;ool00`00oooo
o`30ool000ooo`03003ooooo04coo`03003ooooo0;ooo`003ooo00<00?ooool0C?oo00<00?ooool0
_ooo000?ool2001>ool00`00ooooo`2nool000ooo`03003ooooo04goo`03003ooooo0;koo`003ooo
00<00?ooool0C_oo00<00?ooool0_Ooo000?ool00`00ooooo`1>ool00`00ooooo`2mool000ooo`03
003ooooo04ooo`03003ooooo0;coo`003ooo0P008ooo0P001?oo00<00?ooool00_oo0`007ooo00@0
0?ooooooo`@002;oo`@000?oo`03003ooooo00;oo`<002Coo`@002;oo`@000?oo`03003ooooo00;o
o`<002Coo`8000Ooo`003ooo00<00?ooool08Ooo00@00?ooool000_oo`03003ooooo01goo`03003o
oooo00;oo`03003ooooo02?oo`03003ooooo00[oo`03003ooooo027oo`03003ooooo02?oo`03003o
oooo00coo`03003ooooo023oo`04003ooooo0006ool000ooo`03003ooooo027oo`04003ooooo000;
ool00`00ooooo`0Mool00`00ooooo`02ool00`00ooooo`0Sool00`00ooooo`0:ool00`00ooooo`0R
ool00`00ooooo`0Sool00`00ooooo`0;ool00`00ooooo`0Sool00`00ooooo`04ool000ooo`03003o
oooo027oo`04003ooooo0008ool3000Qool01@00oooooooo00009Ooo00<00?ooool01ooo0`009_oo
00<00?ooool08ooo00<00?ooool01ooo0`009?oo0P001ooo000?ool00`00ooooo`0Qool01000oooo
o`002?oo00<00?ooool08Ooo00D00?ooooooo`0002Goo`03003ooooo00Ooo`03003ooooo02Ooo`03
003ooooo02?oo`03003ooooo00Koo`03003ooooo02Koo`03003ooooo00Coo`003ooo00<00?ooool0
8Ooo00@00?ooool000Soo`03003ooooo027oo`03003oo`00008002?oo`<000Woo`03003ooooo02Co
o`04003ooooo000Rool01000ooooo`002?oo00<00?ooool08ooo00@00?ooool000Koo`003ooo0P00
8ooo0P002Ooo10008?oo00D00?ooooooo`0002Goo`03003ooooo00Ooo`@002Coo`8002Coo`8000Wo
o`@002?oo`8000Ooo`003ooo00<00?ooool0D_oo00<00?ooool0^Ooo000?ool00`00ooooo`1Bool0
0`00ooooo`2iool000ooo`03003ooooo05;oo`03003ooooo00Koo`P00:_oo`003ooo00<00?ooool0
Dooo00<00?ooool01?oo0P001ooo0`00Z?oo0008oooo000I000000ooo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003o
oooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Coo`03003oo`0000;oo`03003o
oooo00?oo`03003ooooo00Koo`<000Koo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Ko
o`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Ko
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Go
o`003ooo00<00?ooool0:Ooo00<00?ooool09ooo00<00?oo00000_oo00<00?ooool03ooo0P005Ooo
00<00?ooool0:_oo00<00?ooool0:Ooo00<00?ooool0:Ooo00<00?ooool01Ooo000?ool00`00oooo
o`1Cool01@00oooooooo00005?oo0P00X?oo000?ool00`00ooooo`1Dool01000ooooo`005_oo0P00
W_oo000?ool2001Eool01000ooooo`006?oo0P00W?oo000?ool00`00ooooo`1Dool00`00ool0000K
ool00`00ooooo`2Iool000ooo`03003ooooo05Coo`03003oo`0001coo`8009Woo`003ooo00<00?oo
ool0EOoo0P007_oo00<00?ooool0U_oo000?ool00`00ooooo`1Eool2000Oool00`00ooooo`2Eool0
00ooo`8005Koo`80023oo`8009Goo`003ooo00<00?ooool0EOoo0P008_oo00<00?ooool0T_oo000?
ool00`00ooooo`1Eool2000Sool00`00ooooo`2Aool000ooo`03003ooooo05Goo`8002Coo`80097o
o`003ooo00<00?ooool0EOoo0P009_oo00<00?ooool0S_oo000?ool00`00ooooo`1Eool2000Wool0
0`00ooooo`2=ool000ooo`8005Koo`8002Soo`03003ooooo08coo`003ooo00<00?ooool0EOoo0P00
:Ooo00<00?ooool0Rooo000?ool00`00ooooo`1Eool2000Zool00`00ooooo`2:ool000ooo`03003o
oooo05Goo`8002_oo`8008[oo`003ooo00<00?ooool0EOoo0P00;Ooo00<00?ooool0Qooo000?ool2
001Fool00`00ooooo`0]ool00`00ooooo`26ool000ooo`03003ooooo05Goo`03003ooooo02koo`03
003ooooo08Goo`003ooo00<00?ooool0EOoo00<00?ooool0;ooo00<00?ooool0Q?oo0007ool40004
ool00`00ooooo`1Eool00`00ooooo`0`ool00`00ooooo`23ool000Woo`03003ooooo00?oo`03003o
oooo05Goo`03003ooooo037oo`03003ooooo08;oo`002Ooo00<00?ooool00ooo0`00EOoo00<00?oo
ool0<_oo00<00?ooool0POoo0009ool00`00ooooo`03ool00`00ooooo`1Eool00`00ooooo`0cool0
0`00ooooo`20ool000Woo`03003ooooo00?oo`03003ooooo08coo`80083oo`001ooo0`001Ooo00<0
0?ooool0S_oo00<00?ooool0OOoo0009ool00`00ooooo`03ool00`00ooooo`2?ool00`00ooooo`1l
ool000ooo`80097oo`03003ooooo07_oo`003ooo00<00?ooool0TOoo00<00?ooool0N_oo000?ool0
0`00ooooo`2Bool00`00ooooo`1iool000ooo`03003ooooo09?oo`03003ooooo07Soo`003ooo00<0
0?ooool0U?oo00<00?ooool0Mooo000?ool2002Fool00`00ooooo`1fool000ooo`03003ooooo09Ko
o`03003ooooo07Goo`003ooo00<00?ooool0Uooo00<00?ooool0M?oo000?ool00`00ooooo`2Hool0
0`00ooooo`1cool000ooo`03003ooooo09Woo`03003ooooo07;oo`003ooo00<00?ooool0V_oo00<0
0?ooool0LOoo000?ool2002Lool00`00ooooo`1`ool000ooo`03003ooooo09coo`03003ooooo06oo
o`003ooo00<00?ooool0WOoo00<00?ooool0K_oo000?ool00`00ooooo`2Nool00`00ooooo`1]ool0
00ooo`03003ooooo09ooo`03003ooooo06coo`003ooo0P00XOoo00<00?ooool0Jooo000?ool00`00
ooooo`2Qool00`00ooooo`1Zool000ooo`03003ooooo0:;oo`03003ooooo06Woo`001ooo10001?oo
00<00?ooool0Xooo00<00?ooool0J?oo0007ool00`00ooooo`05ool00`00ooooo`2Tool00`00oooo
o`1Wool000Soo`03003ooooo00Coo`<00:Goo`03003ooooo06Koo`002Ooo00<00?ooool00ooo00<0
0?ooool0Y_oo00<00?ooool0IOoo000:ool00`00ooooo`02ool00`00ooooo`2Wool00`00ooooo`1T
ool000Ooo`04003ooooo0004ool00`00ooooo`2Xool00`00ooooo`1Sool000Soo`8000Goo`03003o
oooo0:Woo`03003ooooo06;oo`003ooo0P00Z_oo00<00?ooool0H_oo000?ool00`00ooooo`2Zool0
0`00ooooo`1Qool000ooo`03003ooooo0:_oo`03003ooooo063oo`003ooo00<00?ooool0[?oo00<0
0?ooool0Gooo000?ool00`00ooooo`2]ool00`00ooooo`1Nool000ooo`03003ooooo0:koo`03003o
oooo05goo`003ooo0P00[ooo00<00?ooool0GOoo000?ool00`00ooooo`2_ool00`00ooooo`1Lool0
00ooo`03003ooooo0;3oo`03003ooooo05_oo`003ooo00<00?ooool0/Ooo00<00?ooool0F_oo000?
ool00`00ooooo`2bool00`00ooooo`1Iool000ooo`800;Coo`03003ooooo05Soo`003ooo00<00?oo
ool0]?oo00<00?ooool0Eooo000?ool00`00ooooo`2dool00`00ooooo`1Gool000ooo`03003ooooo
0;Goo`03003ooooo05Koo`003ooo00<00?ooool0]_oo00<00?ooool0EOoo000?ool2002hool00`00
ooooo`1Dool000ooo`03003ooooo0;Soo`03003ooooo05?oo`003ooo00<00?ooool0^Ooo00<00?oo
ool0D_oo0008ool20005ool00`00ooooo`2iool00`00ooooo`1Bool000Ooo`04003ooooo0004ool0
0`00ooooo`2jool00`00ooooo`1Aool000[oo`03003ooooo00;oo`<00;_oo`03003ooooo053oo`00
2?oo0P001Ooo00<00?ooool0_?oo00<00?ooool0Cooo000:ool00`00ooooo`02ool00`00ooooo`2m
ool00`00ooooo`1>ool000Ooo`04003ooooo0004ool00`00ooooo`2mool00`00ooooo`1>ool000So
o`8000Goo`03003ooooo0;koo`03003ooooo04goo`003ooo00<00?ooool0_ooo00<00?ooool0C?oo
000?ool20031ool00`00ooooo`1;ool000ooo`03003ooooo0<3oo`03003ooooo04_oo`003ooo00<0
0?ooool0`Ooo00<00?ooool0B_oo000?ool00`00ooooo`32ool00`00ooooo`19ool000ooo`03003o
oooo0<?oo`03003ooooo04Soo`003ooo0P00a?oo00<00?ooool0B?oo000?ool00`00ooooo`34ool0
0`00ooooo`17ool000ooo`03003ooooo0<Goo`03003ooooo04Koo`003ooo00<00?ooool0a_oo00<0
0?ooool0AOoo000?ool00`00ooooo`36ool00`00ooooo`15ool000ooo`800<Soo`03003ooooo04Co
o`003ooo00<00?ooool0b?oo00<00?ooool0@ooo000?ool00`00ooooo`38ool00`00ooooo`13ool0
00ooo`03003ooooo0<Woo`03003ooooo04;oo`003ooo00<00?ooool0b_oo00<00?ooool0@Ooo000?
ool2003<ool00`00ooooo`10ool000ooo`03003ooooo0<_oo`03003ooooo043oo`003ooo00<00?oo
ool0c?oo00<00?ooool0?ooo0009ool00`00ooooo`03ool00`00ooooo`3=ool00`00ooooo`0nool0
00Woo`03003ooooo00?oo`03003ooooo0<goo`03003ooooo03koo`001_oo1@001?oo0`00c_oo00<0
0?ooool0?Ooo0006ool01000ooooo`001Ooo00<00?ooool0cooo00<00?ooool0??oo0007ool00`00
ool00005ool00`00ooooo`3@ool00`00ooooo`0kool000Soo`8000Goo`03003ooooo0=3oo`03003o
oooo03_oo`002Ooo00<00?ooool00ooo00<00?ooool0dOoo00<00?ooool0>_oo000?ool00`00oooo
o`3Bool00`00ooooo`0iool000ooo`800=?oo`03003ooooo03Woo`003ooo00<00?ooool0dooo00<0
0?ooool0>?oo000?ool00`00ooooo`3Dool00`00ooooo`0gool000ooo`03003ooooo0=Coo`03003o
oooo03Ooo`003ooo00<00?ooool0eOoo00<00?ooool0=_oo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.17529, -2.30614, \
0.0113251, 0.0382489}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Change-Log", "Section"],

Cell[BoxData[{
    \(25.3  .2003\ \), "\[IndentingNewLine]", 
    \(Rewrote\ all\ structure\ functions\ for\ om > 
      k . \ Compared\ to\ tables\ from\ mike\ and\ also\ directly\ for\ a\ \
special\ case\ \((k = 2/10, a = 1, th = Pi/4)\) . \ controlled\ 1/
          a\ propagating\ mode\ with\ mike, \ 
    wrote\ special\ functions\ for\ imaginary\ omega\)}], "Input"],

Cell[BoxData[
    \(27.3\ wrote\ root\ finder\ for\ unstable\ b - mode\)], "Input"],

Cell[BoxData[
    \(28.3\ Rewrote\ klim\)], "Input"],

Cell[BoxData[
    \(3.4\ Wrote\ section\ masses\)], "Input"],

Cell[BoxData[
    \(23.4 . \ Added\ energy\ loss\)], "Input"],

Cell[BoxData[
    \(13.5 . \ 
        Rewrote\ new\ version\ of\ hard\ energy\ loss\ and\ added\ energy\ \
loss\ of\ QED\ plasma\)], "Input"],

Cell[BoxData[
    \(14.5\ corrected\ QED\ case\)], "Input"],

Cell[BoxData[
    \(23.5\ devised\ numerical\ 3 - int\ for\ test\ integral\)], "Input"],

Cell[BoxData[
    \(2.6\ wrote\ fast\ version\ of\ structure\ functions; \ 
    korrekted\ negative\ kz - behavior\ for\ Fast\ struct - 
      funcs\ \((still\ wrong\ for\ \(slower!!\))\); \ \(\(wrote\)\(\ \
\)\(anisotropic\)\(\ \)\(soft\)\(\ \)\(energy\)\(\ \)\(loss\)\(\ \)\)\)], \
"Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
Evaluator->"Local",
WindowSize->{814, 700},
WindowMargins->{{Automatic, 93}, {-25, Automatic}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Info3267953076-4906506"->{
    Cell[1154900, 29947, 68, 1, 23, "Print",
      CellTags->"Info3267953076-4906506"],
    Cell[1154971, 29950, 581, 15, 23, "Print",
      CellTags->"Info3267953076-4906506"]},
  "Info3267953430-8404111"->{
    Cell[1161225, 30108, 69, 1, 23, "Print",
      CellTags->"Info3267953430-8404111"],
    Cell[1161297, 30111, 1102, 23, 176, "Print",
      CellTags->"Info3267953430-8404111"]},
  "Info3275041079-1935256"->{
    Cell[1619715, 42495, 69, 1, 23, "Print",
      CellTags->"Info3275041079-1935256"],
    Cell[1619787, 42498, 4430, 74, 961, "Print",
      CellTags->"Info3275041079-1935256"]},
  "Info3274767031-9777815"->{
    Cell[2596868, 68916, 67, 1, 23, "Print",
      CellTags->"Info3274767031-9777815"],
    Cell[2596938, 68919, 640, 16, 52, "Print",
      CellTags->"Info3274767031-9777815"]},
  "Info3280046340-8820928"->{
    Cell[3263688, 87254, 76, 1, 23, "Print",
      CellTags->"Info3280046340-8820928"],
    Cell[3263767, 87257, 1430, 28, 99, "Print",
      CellTags->"Info3280046340-8820928"]},
  "Info3280045044-9066215"->{
    Cell[4126900, 114356, 72, 1, 23, "Print",
      CellTags->"Info3280045044-9066215"],
    Cell[4126975, 114359, 671, 17, 42, "Print",
      CellTags->"Info3280045044-9066215"]},
  "Info3279609948-4320224"->{
    Cell[4187618, 116675, 77, 1, 23, "Print",
      CellTags->"Info3279609948-4320224"],
    Cell[4187698, 116678, 624, 16, 41, "Print",
      CellTags->"Info3279609948-4320224"]},
  "Info3279609199-6771777"->{
    Cell[4389388, 121768, 77, 1, 23, "Print",
      CellTags->"Info3279609199-6771777"],
    Cell[4389468, 121771, 841, 21, 41, "Print",
      CellTags->"Info3279609199-6771777"]},
  "Info3279531231-9953284"->{
    Cell[4489090, 124826, 73, 1, 23, "Print",
      CellTags->"Info3279531231-9953284"],
    Cell[4489166, 124829, 1110, 23, 111, "Print",
      CellTags->"Info3279531231-9953284"]},
  "Info3279607211-7329832"->{
    Cell[4879423, 137187, 66, 1, 23, "Print",
      CellTags->"Info3279607211-7329832"],
    Cell[4879492, 137190, 805, 19, 53, "Print",
      CellTags->"Info3279607211-7329832"]},
  "Info3279607223-5556151"->{
    Cell[4880387, 137217, 74, 1, 23, "Print",
      CellTags->"Info3279607223-5556151"],
    Cell[4880464, 137220, 1488, 29, 101, "Print",
      CellTags->"Info3279607223-5556151"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Info3267953076-4906506", 5088796, 143834},
  {"Info3267953430-8404111", 5089005, 143839},
  {"Info3275041079-1935256", 5089216, 143844},
  {"Info3274767031-9777815", 5089427, 143849},
  {"Info3280046340-8820928", 5089636, 143854},
  {"Info3280045044-9066215", 5089846, 143859},
  {"Info3279609948-4320224", 5090057, 143864},
  {"Info3279609199-6771777", 5090268, 143869},
  {"Info3279531231-9953284", 5090479, 143874},
  {"Info3279607211-7329832", 5090692, 143879},
  {"Info3279607223-5556151", 5090903, 143884}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 36, 0, 117, "Title"],

Cell[CellGroupData[{
Cell[1788, 56, 24, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[1837, 60, 47, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[1909, 64, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[1972, 68, 80, 1, 27, "Input"],
Cell[2055, 71, 84, 1, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2176, 77, 66, 1, 27, "Input"],
Cell[2245, 80, 105, 2, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2387, 87, 66, 1, 27, "Input"],
Cell[2456, 90, 105, 2, 70, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[2610, 98, 40, 0, 28, "Subsubsection"],
Cell[2653, 100, 621, 14, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[3299, 118, 88, 1, 27, "Input"],
Cell[3390, 121, 87, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3526, 128, 35, 0, 28, "Subsubsection"],
Cell[3564, 130, 4255, 77, 539, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[7844, 211, 123, 3, 27, "Input"],
Cell[7970, 216, 85, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8092, 222, 49, 1, 27, "Input"],
Cell[8144, 225, 35, 1, 27, "Output"]
}, Open  ]],
Cell[8194, 229, 134, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[8353, 235, 70, 1, 27, "Input"],
Cell[8426, 238, 81, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8544, 244, 104, 2, 27, "Input"],
Cell[8651, 248, 259, 4, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8947, 257, 44, 1, 27, "Input"],
Cell[8994, 260, 85, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9116, 266, 54, 1, 27, "Input"],
Cell[9173, 269, 38, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9248, 275, 69, 1, 27, "Input"],
Cell[9320, 278, 15979, 414, 186, 3631, 257, "GraphicsData", "PostScript", \
"Graphics"],
Cell[25302, 694, 130, 3, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[25493, 704, 38, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[25556, 708, 192, 4, 27, "Input",
  InitializationCell->True],
Cell[25751, 714, 125, 2, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25913, 721, 136, 3, 27, "Input"],
Cell[26052, 726, 104, 2, 54, "Output"]
}, Open  ]],
Cell[26171, 731, 511, 9, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[26707, 744, 62, 1, 27, "Input"],
Cell[26772, 747, 305, 5, 58, "Message"],
Cell[27080, 754, 132, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27249, 761, 62, 1, 27, "Input"],
Cell[27314, 764, 305, 5, 58, "Message"],
Cell[27622, 771, 354, 5, 71, "Message"],
Cell[27979, 778, 227, 4, 39, "Message"],
Cell[28209, 784, 79, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28325, 790, 101, 2, 27, "Input"],
Cell[28429, 794, 420, 6, 58, "Message"],
Cell[28852, 802, 419, 6, 58, "Message"],
Cell[29274, 810, 420, 6, 58, "Message"],
Cell[29697, 818, 169, 3, 23, "Message"],
Cell[29869, 823, 20482, 509, 186, 4257, 304, "GraphicsData", "PostScript", \
"Graphics"],
Cell[50354, 1334, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[50533, 1343, 29, 0, 29, "Subsection"],
Cell[50565, 1345, 308, 6, 43, "Input"],

Cell[CellGroupData[{
Cell[50898, 1355, 227, 5, 59, "Input"],
Cell[51128, 1362, 154, 2, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51319, 1369, 154, 3, 27, "Input"],
Cell[51476, 1374, 89, 1, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51602, 1380, 41, 1, 27, "Input"],
Cell[51646, 1383, 141, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51824, 1390, 154, 3, 27, "Input"],
Cell[51981, 1395, 168, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52186, 1402, 41, 1, 27, "Input"],
Cell[52230, 1405, 518, 7, 125, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52785, 1417, 161, 2, 31, "Input"],
Cell[52949, 1421, 152, 3, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53138, 1429, 123, 3, 31, "Input"],
Cell[53264, 1434, 121, 2, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[53434, 1442, 32, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[53491, 1446, 277, 6, 43, "Input",
  InitializationCell->True],
Cell[53771, 1454, 296, 6, 70, "Output"]
}, Open  ]],
Cell[54082, 1463, 418, 10, 75, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[54525, 1477, 90, 1, 27, "Input"],
Cell[54618, 1480, 177, 3, 39, "Message"],
Cell[54798, 1485, 177, 3, 39, "Message"],
Cell[54978, 1490, 177, 3, 39, "Message"],
Cell[55158, 1495, 173, 3, 23, "Message"],
Cell[55334, 1500, 22900, 530, 186, 4239, 295, "GraphicsData", "PostScript", \
"Graphics"],
Cell[78237, 2032, 208, 5, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[78482, 2042, 86, 1, 24, "Input"],
Cell[78571, 2045, 321, 5, 62, "Message"],
Cell[78895, 2052, 347, 6, 62, "Message"],
Cell[79245, 2060, 347, 6, 62, "Message"],
Cell[79595, 2068, 169, 3, 23, "Message"],
Cell[79767, 2073, 22900, 530, 186, 4239, 295, "GraphicsData", "PostScript", \
"Graphics"],
Cell[102670, 2605, 222, 5, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[102929, 2615, 61, 1, 24, "Input"],
Cell[102993, 2618, 87, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[103117, 2624, 65, 1, 27, "Input"],
Cell[103185, 2627, 177, 3, 39, "Message"],
Cell[103365, 2632, 177, 3, 39, "Message"],
Cell[103545, 2637, 177, 3, 39, "Message"],
Cell[103725, 2642, 173, 3, 23, "Message"],
Cell[103901, 2647, 58, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[103996, 2653, 168, 4, 27, "Input",
  InitializationCell->True],
Cell[104167, 2659, 209, 5, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[104413, 2669, 279, 6, 59, "Input",
  InitializationCell->True],
Cell[104695, 2677, 308, 6, 70, "Output"]
}, Open  ]],
Cell[105018, 2686, 1121, 23, 187, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[106164, 2713, 65, 1, 27, "Input"],
Cell[106232, 2716, 54, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[106323, 2722, 61, 1, 27, "Input"],
Cell[106387, 2725, 76, 1, 27, "Output"]
}, Open  ]],
Cell[106478, 2729, 176, 4, 27, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[106679, 2737, 60, 1, 27, "Input"],
Cell[106742, 2740, 177, 3, 39, "Message"],
Cell[106922, 2745, 177, 3, 39, "Message"],
Cell[107102, 2750, 177, 3, 39, "Message"],
Cell[107282, 2755, 173, 3, 23, "Message"],
Cell[107458, 2760, 126, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[107621, 2767, 69, 1, 27, "Input"],
Cell[107693, 2770, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[107783, 2776, 77, 1, 27, "Input"],
Cell[107863, 2779, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[107952, 2785, 79, 1, 27, "Input"],
Cell[108034, 2788, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[108124, 2794, 586, 10, 107, "Input"],
Cell[108713, 2806, 177, 3, 39, "Message"],
Cell[108893, 2811, 177, 3, 39, "Message"],
Cell[109073, 2816, 177, 3, 39, "Message"],
Cell[109253, 2821, 173, 3, 23, "Message"],
Cell[109429, 2826, 25702, 1041, 186, 11899, 866, "GraphicsData", \
"PostScript", "Graphics"],
Cell[135134, 3869, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[135301, 3877, 60, 1, 27, "Input"],
Cell[135364, 3880, 46, 1, 27, "Output"]
}, Open  ]],
Cell[135425, 3884, 69, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[135519, 3889, 370, 6, 75, "Input"],
Cell[135892, 3897, 243, 4, 58, "Message"],
Cell[136138, 3903, 354, 5, 71, "Message"],
Cell[136495, 3910, 423, 6, 58, "Message"],
Cell[136921, 3918, 423, 6, 58, "Message"],
Cell[137347, 3926, 169, 3, 23, "Message"],
Cell[137519, 3931, 354, 5, 71, "Message"],
Cell[137876, 3938, 354, 5, 71, "Message"],
Cell[138233, 3945, 170, 3, 23, "Message"],
Cell[138406, 3950, 17949, 629, 186, 5959, 476, "GraphicsData", "PostScript", \
"Graphics"],
Cell[156358, 4581, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[156525, 4589, 60, 1, 27, "Input"],
Cell[156588, 4592, 46, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[156683, 4599, 38, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[156746, 4603, 153, 2, 27, "Input"],
Cell[156902, 4607, 8801, 146, 1219, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[165740, 4758, 56, 1, 24, "Input"],
Cell[165799, 4761, 1734, 27, 273, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[167570, 4793, 103, 2, 24, "Input"],
Cell[167676, 4797, 770, 12, 108, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[168483, 4814, 795, 13, 139, "Input"],
Cell[169281, 4829, 745, 12, 108, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[170063, 4846, 50, 1, 27, "Input"],
Cell[170116, 4849, 110, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[170263, 4856, 904, 14, 180, "Input"],
Cell[171170, 4872, 895, 14, 149, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[172102, 4891, 54, 1, 27, "Input"],
Cell[172159, 4894, 126, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[172322, 4901, 65, 1, 27, "Input"],
Cell[172390, 4904, 128, 2, 27, "Output"]
}, Open  ]],
Cell[172533, 4909, 55, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[172613, 4914, 824, 14, 139, "Input",
  InitializationCell->True],
Cell[173440, 4930, 703, 11, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[174180, 4946, 54, 1, 27, "Input"],
Cell[174237, 4949, 126, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[174400, 4956, 65, 1, 27, "Input"],
Cell[174468, 4959, 125, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[174630, 4966, 955, 16, 179, "Input",
  InitializationCell->True],
Cell[175588, 4984, 852, 14, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[176477, 5003, 67, 1, 27, "Input"],
Cell[176547, 5006, 125, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[176709, 5013, 331, 6, 59, "Input"],
Cell[177043, 5021, 226, 4, 39, "Message"],
Cell[177272, 5027, 125, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[177434, 5034, 90, 1, 27, "Input"],
Cell[177527, 5037, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[177617, 5043, 90, 1, 27, "Input"],
Cell[177710, 5046, 177, 3, 39, "Message"],
Cell[177890, 5051, 177, 3, 39, "Message"],
Cell[178070, 5056, 177, 3, 39, "Message"],
Cell[178250, 5061, 173, 3, 23, "Message"],
Cell[178426, 5066, 63, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[178526, 5072, 50, 1, 27, "Input"],
Cell[178579, 5075, 110, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[178726, 5082, 90, 1, 27, "Input"],
Cell[178819, 5085, 177, 3, 39, "Message"],
Cell[178999, 5090, 177, 3, 39, "Message"],
Cell[179179, 5095, 177, 3, 39, "Message"],
Cell[179359, 5100, 173, 3, 23, "Message"],
Cell[179535, 5105, 107, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[179679, 5112, 90, 1, 27, "Input"],
Cell[179772, 5115, 225, 4, 39, "Message"],
Cell[180000, 5121, 62, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[180099, 5127, 268, 4, 43, "Input"],
Cell[180370, 5133, 354, 5, 71, "Message"],
Cell[180727, 5140, 225, 4, 39, "Message"],
Cell[180955, 5146, 127, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[181119, 5153, 49, 1, 27, "Input"],
Cell[181171, 5156, 111, 2, 27, "Output"]
}, Open  ]],
Cell[181297, 5161, 41, 1, 27, "Input"],
Cell[181341, 5164, 101, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[181467, 5170, 84, 1, 27, "Input"],
Cell[181554, 5173, 14240, 459, 186, 4795, 338, "GraphicsData", "PostScript", \
"Graphics"],
Cell[195797, 5634, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[195964, 5642, 84, 1, 27, "Input"],
Cell[196051, 5645, 15800, 470, 186, 4686, 328, "GraphicsData", "PostScript", \
"Graphics"],
Cell[211854, 6117, 130, 3, 27, "Output"]
}, Open  ]],
Cell[211999, 6123, 51, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[212075, 6128, 196, 4, 43, "Input"],
Cell[212274, 6134, 177, 3, 39, "Message"],
Cell[212454, 6139, 177, 3, 39, "Message"],
Cell[212634, 6144, 177, 3, 39, "Message"],
Cell[212814, 6149, 173, 3, 23, "Message"],
Cell[212990, 6154, 4775, 70, 477, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[217802, 6229, 67, 1, 27, "Input"],
Cell[217872, 6232, 171, 3, 23, "Message"],
Cell[218046, 6237, 13839, 474, 186, 5771, 370, "GraphicsData", "PostScript", \
"Graphics"],
Cell[231888, 6713, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[232055, 6721, 90, 1, 27, "Input"],
Cell[232148, 6724, 177, 3, 39, "Message"],
Cell[232328, 6729, 177, 3, 39, "Message"],
Cell[232508, 6734, 177, 3, 39, "Message"],
Cell[232688, 6739, 173, 3, 23, "Message"],
Cell[232864, 6744, 226, 4, 39, "Message"],
Cell[233093, 6750, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[233183, 6756, 90, 1, 27, "Input"],
Cell[233276, 6759, 226, 4, 39, "Message"],
Cell[233505, 6765, 52, 1, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[233618, 6773, 23, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[233666, 6777, 29, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[233720, 6781, 44, 1, 27, "Input"],
Cell[233767, 6784, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[233839, 6790, 84, 1, 27, "Input"],
Cell[233926, 6793, 85, 1, 35, "Output"]
}, Open  ]],
Cell[234026, 6797, 197, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[234248, 6805, 208, 5, 59, "Input"],
Cell[234459, 6812, 140, 2, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[234636, 6819, 157, 4, 43, "Input"],
Cell[234796, 6825, 86, 1, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[234919, 6831, 43, 1, 27, "Input"],
Cell[234965, 6834, 123, 2, 46, "Output"]
}, Open  ]],
Cell[235103, 6839, 49, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[235177, 6844, 208, 5, 59, "Input"],
Cell[235388, 6851, 423, 7, 63, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[235848, 6863, 157, 4, 40, "Input"],
Cell[236008, 6869, 204, 4, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[236249, 6878, 43, 1, 27, "Input"],
Cell[236295, 6881, 474, 7, 87, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[236806, 6893, 208, 5, 59, "Input"],
Cell[237017, 6900, 793, 12, 153, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[237847, 6917, 208, 5, 56, "Input"],
Cell[238058, 6924, 1446, 25, 275, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[239541, 6954, 208, 5, 56, "Input"],
Cell[239752, 6961, 2053, 31, 405, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[241842, 6997, 208, 5, 56, "Input"],
Cell[242053, 7004, 2990, 48, 569, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[245092, 7058, 47, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[245164, 7062, 108, 2, 27, "Input",
  InitializationCell->True],
Cell[245275, 7066, 84, 1, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[245396, 7072, 94, 2, 27, "Input",
  InitializationCell->True],
Cell[245493, 7076, 105, 2, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[245635, 7083, 94, 2, 27, "Input",
  InitializationCell->True],
Cell[245732, 7087, 105, 2, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[245874, 7094, 40, 0, 42, "Subsubsection"],
Cell[245917, 7096, 475, 10, 123, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[246417, 7110, 59, 1, 27, "Input"],
Cell[246479, 7113, 73, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[246589, 7119, 72, 1, 27, "Input"],
Cell[246664, 7122, 73, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[246774, 7128, 65, 1, 27, "Input"],
Cell[246842, 7131, 54, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[246933, 7137, 65, 1, 27, "Input"],
Cell[247001, 7140, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[247091, 7146, 1157, 21, 187, "Input"],
Cell[248251, 7169, 1232, 21, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[249520, 7195, 125, 3, 27, "Input"],
Cell[249648, 7200, 1147, 21, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[250832, 7226, 141, 3, 27, "Input"],
Cell[250976, 7231, 16445, 395, 186, 3088, 226, "GraphicsData", "PostScript", \
"Graphics"],
Cell[267424, 7628, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[267591, 7636, 133, 3, 27, "Input"],
Cell[267727, 7641, 1194, 21, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[268958, 7667, 125, 3, 24, "Input"],
Cell[269086, 7672, 1144, 21, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[270267, 7698, 141, 3, 24, "Input"],
Cell[270411, 7703, 18265, 449, 186, 3501, 262, "GraphicsData", "PostScript", \
"Graphics"],
Cell[288679, 8154, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[288846, 8162, 132, 2, 24, "Input"],
Cell[288981, 8166, 91, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[289121, 8173, 35, 0, 42, "Subsubsection"],
Cell[289159, 8175, 2204, 40, 331, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[291388, 8219, 59, 1, 27, "Input"],
Cell[291450, 8222, 72, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[291559, 8228, 75, 1, 27, "Input"],
Cell[291637, 8231, 62, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[291736, 8237, 138, 3, 27, "Input"],
Cell[291877, 8242, 1224, 22, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[293138, 8269, 141, 3, 24, "Input"],
Cell[293282, 8274, 20466, 466, 186, 3458, 252, "GraphicsData", "PostScript", \
"Graphics"],
Cell[313751, 8742, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[313918, 8750, 138, 3, 24, "Input"],
Cell[314059, 8755, 1221, 22, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[315317, 8782, 141, 3, 24, "Input"],
Cell[315461, 8787, 19216, 437, 186, 3197, 235, "GraphicsData", "PostScript", \
"Graphics"],
Cell[334680, 9226, 130, 3, 27, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[334871, 9236, 44, 0, 29, "Subsection"],
Cell[334918, 9238, 59, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[335002, 9243, 44, 1, 27, "Input"],
Cell[335049, 9246, 35, 1, 27, "Output"]
}, Open  ]],
Cell[335099, 9250, 207, 4, 59, "Input"],
Cell[335309, 9256, 48, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[335382, 9261, 83, 1, 27, "Input"],
Cell[335468, 9264, 17618, 453, 186, 3734, 277, "GraphicsData", "PostScript", \
"Graphics"],
Cell[353089, 9719, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[353256, 9727, 49, 1, 24, "Input"],
Cell[353308, 9730, 18373, 440, 186, 3485, 252, "GraphicsData", "PostScript", \
"Graphics"],
Cell[371684, 10172, 130, 3, 27, "Output"]
}, Open  ]],
Cell[371829, 10178, 56, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[371910, 10183, 83, 1, 27, "Input"],
Cell[371996, 10186, 21555, 523, 186, 4199, 304, "GraphicsData", "PostScript", \
"Graphics"],
Cell[393554, 10711, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[393721, 10719, 49, 1, 27, "Input"],
Cell[393773, 10722, 23083, 564, 186, 4686, 332, "GraphicsData", "PostScript", \
"Graphics"],
Cell[416859, 11288, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[417026, 11296, 83, 1, 27, "Input"],
Cell[417112, 11299, 18990, 461, 186, 3663, 267, "GraphicsData", "PostScript", \
"Graphics"],
Cell[436105, 11762, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[436272, 11770, 49, 1, 24, "Input"],
Cell[436324, 11773, 20009, 485, 186, 3828, 281, "GraphicsData", "PostScript", \
"Graphics"],
Cell[456336, 12260, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[456515, 12269, 47, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[456587, 12273, 34, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[456646, 12277, 169, 3, 27, "Input",
  InitializationCell->True],
Cell[456818, 12282, 145, 2, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[457000, 12289, 165, 3, 27, "Input",
  InitializationCell->True],
Cell[457168, 12294, 145, 2, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[457350, 12301, 365, 8, 59, "Input",
  InitializationCell->True],
Cell[457718, 12311, 294, 4, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[458049, 12320, 292, 6, 59, "Input",
  InitializationCell->True],
Cell[458344, 12328, 248, 3, 70, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[458641, 12337, 41, 0, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[458707, 12341, 64, 1, 27, "Input"],
Cell[458774, 12344, 130, 3, 57, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[458941, 12352, 48, 1, 24, "Input"],
Cell[458992, 12355, 96, 2, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[459125, 12362, 61, 1, 27, "Input"],
Cell[459189, 12365, 128, 3, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[459354, 12373, 61, 1, 27, "Input"],
Cell[459418, 12376, 169, 2, 57, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[459624, 12383, 46, 1, 27, "Input"],
Cell[459673, 12386, 84, 1, 51, "Output"]
}, Open  ]],
Cell[459772, 12390, 576, 11, 155, "Input"],

Cell[CellGroupData[{
Cell[460373, 12405, 91, 1, 27, "Input"],
Cell[460467, 12408, 17864, 452, 186, 3896, 275, "GraphicsData", "PostScript", \
"Graphics"],
Cell[478334, 12862, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[478513, 12871, 36, 0, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[478574, 12875, 201, 4, 27, "Input",
  InitializationCell->True],
Cell[478778, 12881, 503, 8, 70, "Output"]
}, Open  ]],
Cell[479296, 12892, 401, 9, 75, "Input",
  InitializationCell->True],
Cell[479700, 12903, 276, 6, 59, "Input",
  InitializationCell->True],
Cell[479979, 12911, 671, 12, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[480675, 12927, 82, 1, 27, "Input"],
Cell[480760, 12930, 17864, 452, 186, 3896, 275, "GraphicsData", "PostScript", \
"Graphics"],
Cell[498627, 13384, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[498818, 13394, 38, 0, 29, "Subsection"],
Cell[498859, 13396, 519, 10, 91, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[499415, 13411, 34, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[499474, 13415, 87, 1, 27, "Input"],
Cell[499564, 13418, 93, 1, 54, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[499694, 13424, 125, 3, 27, "Input"],
Cell[499822, 13429, 263, 4, 73, "Output"]
}, Open  ]],
Cell[500100, 13436, 571, 11, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[500696, 13451, 91, 1, 27, "Input"],
Cell[500790, 13454, 354, 5, 71, "Message"],
Cell[501147, 13461, 228, 4, 39, "Message"],
Cell[501378, 13467, 103, 2, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[501518, 13474, 70, 1, 24, "Input"],
Cell[501591, 13477, 57, 1, 27, "Output"]
}, Open  ]],
Cell[501663, 13481, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[501729, 13486, 82, 1, 27, "Input"],
Cell[501814, 13489, 15718, 394, 186, 3352, 237, "GraphicsData", "PostScript", \
"Graphics"],
Cell[517535, 13885, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[517714, 13894, 31, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[517770, 13898, 135, 3, 27, "Input",
  InitializationCell->True],
Cell[517908, 13903, 173, 4, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[518118, 13912, 288, 6, 59, "Input",
  InitializationCell->True],
Cell[518409, 13920, 319, 6, 70, "Output"]
}, Open  ]],
Cell[518743, 13929, 1338, 28, 187, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[520106, 13961, 64, 1, 27, "Input"],
Cell[520173, 13964, 59, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[520269, 13970, 60, 1, 27, "Input"],
Cell[520332, 13973, 39, 1, 23, "Print"],
Cell[520374, 13976, 73, 1, 27, "Output"]
}, Open  ]],
Cell[520462, 13980, 173, 4, 27, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[520660, 13988, 59, 1, 27, "Input"],
Cell[520722, 13991, 104, 2, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[520875, 13999, 31, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[520931, 14003, 126, 2, 27, "Input"],
Cell[521060, 14007, 3508, 55, 513, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[524605, 14067, 56, 1, 24, "Input"],
Cell[524664, 14070, 979, 15, 166, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[525680, 14090, 106, 2, 24, "Input"],
Cell[525789, 14094, 530, 8, 79, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[526356, 14107, 181, 4, 27, "Input"],
Cell[526540, 14113, 653, 10, 120, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[527230, 14128, 710, 11, 157, "Input"],
Cell[527943, 14141, 657, 10, 120, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[528637, 14156, 55, 1, 27, "Input"],
Cell[528695, 14159, 128, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[528860, 14166, 53, 1, 27, "Input"],
Cell[528916, 14169, 127, 2, 27, "Output"]
}, Open  ]],
Cell[529058, 14174, 42, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[529125, 14179, 608, 10, 116, "Input",
  InitializationCell->True],
Cell[529736, 14191, 509, 8, 79, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[530282, 14204, 414, 8, 59, "Input"],
Cell[530699, 14214, 259, 4, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[530995, 14223, 75, 1, 27, "Input"],
Cell[531073, 14226, 125, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[531235, 14233, 53, 1, 27, "Input"],
Cell[531291, 14236, 127, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[531455, 14243, 67, 1, 27, "Input"],
Cell[531525, 14246, 127, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[531689, 14253, 67, 1, 27, "Input"],
Cell[531759, 14256, 128, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[531924, 14263, 63, 1, 27, "Input"],
Cell[531990, 14266, 128, 2, 27, "Output"]
}, Open  ]],
Cell[532133, 14271, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[532199, 14276, 741, 12, 153, "Input",
  InitializationCell->True],
Cell[532943, 14290, 652, 11, 104, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[533632, 14306, 52, 1, 27, "Input"],
Cell[533687, 14309, 102, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[533826, 14316, 65, 1, 27, "Input"],
Cell[533894, 14319, 102, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[534033, 14326, 65, 1, 27, "Input"],
Cell[534101, 14329, 104, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[534242, 14336, 54, 1, 27, "Input"],
Cell[534299, 14339, 126, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[534462, 14346, 54, 1, 27, "Input"],
Cell[534519, 14349, 110, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[534666, 14356, 66, 1, 27, "Input"],
Cell[534735, 14359, 127, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[534899, 14366, 66, 1, 27, "Input"],
Cell[534968, 14369, 126, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[535131, 14376, 268, 4, 43, "Input"],
Cell[535402, 14382, 354, 5, 71, "Message"],
Cell[535759, 14389, 225, 4, 39, "Message"],
Cell[535987, 14395, 127, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[536151, 14402, 49, 1, 27, "Input"],
Cell[536203, 14405, 111, 2, 27, "Output"]
}, Open  ]],
Cell[536329, 14410, 41, 1, 27, "Input"],
Cell[536373, 14413, 101, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[536499, 14419, 89, 1, 27, "Input"],
Cell[536591, 14422, 4840, 97, 219, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[541468, 14524, 144, 3, 27, "Input"],
Cell[541615, 14529, 2130, 38, 171, "Output"]
}, Open  ]],
Cell[543760, 14570, 51, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[543836, 14575, 190, 4, 43, "Input"],
Cell[544029, 14581, 1102, 18, 119, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[545168, 14604, 44, 1, 27, "Input"],
Cell[545215, 14607, 14065, 453, 186, 5383, 341, "GraphicsData", "PostScript", \
"Graphics"],
Cell[559283, 15062, 130, 3, 27, "Output"]
}, Open  ]],
Cell[559428, 15068, 48, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[559501, 15073, 123, 3, 27, "Input"],
Cell[559627, 15078, 15202, 449, 186, 4449, 312, "GraphicsData", "PostScript", \
"Graphics"],
Cell[574832, 15529, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[574999, 15537, 199, 4, 43, "Input"],
Cell[575201, 15543, 14580, 435, 186, 4312, 304, "GraphicsData", "PostScript", \
"Graphics"],
Cell[589784, 15980, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[589951, 15988, 47, 1, 24, "Input"],
Cell[590001, 15991, 17148, 520, 186, 5313, 369, "GraphicsData", "PostScript", \
"Graphics"],
Cell[607152, 16513, 130, 3, 27, "Output"]
}, Closed]],
Cell[607297, 16519, 100, 2, 24, "Input"],

Cell[CellGroupData[{
Cell[607422, 16525, 189, 4, 43, "Input"],
Cell[607614, 16531, 930, 15, 101, "Output"]
}, Open  ]],
Cell[608559, 16549, 48, 1, 27, "Input"],
Cell[608610, 16552, 53, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[608688, 16557, 156, 2, 27, "Input"],
Cell[608847, 16561, 354, 5, 71, "Message"],
Cell[609204, 16568, 226, 4, 39, "Message"],
Cell[609433, 16574, 125, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[609595, 16581, 132, 2, 27, "Input"],
Cell[609730, 16585, 110, 2, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[609901, 16594, 24, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[609950, 16598, 47, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[610022, 16602, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[610085, 16606, 80, 1, 27, "Input"],
Cell[610168, 16609, 84, 1, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[610289, 16615, 66, 1, 27, "Input"],
Cell[610358, 16618, 105, 2, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[610500, 16625, 66, 1, 27, "Input"],
Cell[610569, 16628, 105, 2, 51, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[610723, 16636, 40, 0, 28, "Subsubsection"],
Cell[610766, 16638, 741, 16, 123, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[611544, 16659, 35, 0, 28, "Subsubsection"],
Cell[611582, 16661, 3578, 64, 475, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[615185, 16729, 123, 3, 27, "Input"],
Cell[615311, 16734, 85, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[615433, 16740, 49, 1, 27, "Input"],
Cell[615485, 16743, 35, 1, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[615581, 16751, 38, 0, 45, "Subsection"],
Cell[615622, 16753, 932, 18, 139, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[616579, 16775, 62, 1, 27, "Input"],
Cell[616644, 16778, 112, 2, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[616805, 16786, 29, 0, 29, "Subsection"],
Cell[616837, 16788, 689, 12, 91, "Input"],

Cell[CellGroupData[{
Cell[617551, 16804, 154, 3, 27, "Input"],
Cell[617708, 16809, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[617780, 16815, 154, 3, 27, "Input"],
Cell[617937, 16820, 134, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[618108, 16827, 41, 1, 27, "Input"],
Cell[618152, 16830, 451, 7, 88, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[618640, 16842, 155, 2, 31, "Input"],
Cell[618798, 16846, 82, 1, 30, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[618917, 16852, 123, 3, 31, "Input"],
Cell[619043, 16857, 121, 2, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[619213, 16865, 32, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[619270, 16869, 380, 8, 59, "Input",
  InitializationCell->True],
Cell[619653, 16879, 380, 7, 70, "Output"]
}, Open  ]],
Cell[620048, 16889, 471, 11, 91, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[620544, 16904, 92, 1, 27, "Input"],
Cell[620639, 16907, 177, 3, 39, "Message"],
Cell[620819, 16912, 119, 2, 36, "Message"],
Cell[620941, 16916, 177, 3, 39, "Message"],
Cell[621121, 16921, 119, 2, 36, "Message"],
Cell[621243, 16925, 216, 4, 39, "Message"],
Cell[621462, 16931, 177, 3, 39, "Message"],
Cell[621642, 16936, 173, 3, 23, "Message"],
Cell[621818, 16941, 119, 2, 36, "Message"],
Cell[621940, 16945, 163, 3, 23, "Message"],
Cell[622106, 16950, 216, 4, 39, "Message"],
Cell[622325, 16956, 216, 4, 39, "Message"],
Cell[622544, 16962, 170, 3, 23, "Message"],
Cell[622717, 16967, 224, 4, 39, "Message"],
Cell[622944, 16973, 225, 4, 39, "Message"],
Cell[623172, 16979, 225, 4, 39, "Message"],
Cell[623400, 16985, 168, 3, 23, "Message"],
Cell[623571, 16990, 22785, 577, 186, 5132, 355, "GraphicsData", "PostScript", \
"Graphics"],
Cell[646359, 17569, 222, 5, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[646618, 17579, 90, 1, 24, "Input"],
Cell[646711, 17582, 612, 9, 82, "Message"],
Cell[647326, 17593, 395, 6, 87, "Message"],
Cell[647724, 17601, 611, 10, 82, "Message"],
Cell[648338, 17613, 354, 5, 71, "Message"],
Cell[648695, 17620, 227, 4, 39, "Message"],
Cell[648925, 17626, 643, 11, 88, "Message"],
Cell[649571, 17639, 169, 3, 23, "Message"],
Cell[649743, 17644, 354, 5, 71, "Message"],
Cell[650100, 17651, 226, 4, 39, "Message"],
Cell[650329, 17657, 354, 5, 71, "Message"],
Cell[650686, 17664, 170, 3, 23, "Message"],
Cell[650859, 17669, 227, 4, 39, "Message"],
Cell[651089, 17675, 168, 3, 23, "Message"],
Cell[651260, 17680, 193, 3, 36, "Message"],
Cell[651456, 17685, 395, 6, 87, "Message"],
Cell[651854, 17693, 194, 3, 54, "Message"],
Cell[652051, 17698, 194, 3, 54, "Message"],
Cell[652248, 17703, 162, 3, 23, "Message"],
Cell[652413, 17708, 395, 6, 87, "Message"],
Cell[652811, 17716, 169, 3, 23, "Message"],
Cell[652983, 17721, 42, 1, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[653062, 17727, 63, 1, 24, "Input"],
Cell[653128, 17730, 736, 11, 92, "Message"],
Cell[653867, 17743, 354, 5, 71, "Message"],
Cell[654224, 17750, 226, 4, 39, "Message"],
Cell[654453, 17756, 131, 2, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[654621, 17763, 67, 1, 24, "Input"],
Cell[654691, 17766, 177, 3, 39, "Message"],
Cell[654871, 17771, 177, 3, 39, "Message"],
Cell[655051, 17776, 177, 3, 39, "Message"],
Cell[655231, 17781, 173, 3, 23, "Message"],
Cell[655407, 17786, 54, 1, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[655498, 17792, 67, 1, 24, "Input"],
Cell[655568, 17795, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[655658, 17801, 253, 6, 43, "Input",
  InitializationCell->True],
Cell[655914, 17809, 264, 5, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[656215, 17819, 431, 10, 59, "Input",
  InitializationCell->True],
Cell[656649, 17831, 399, 7, 70, "Output"]
}, Open  ]],
Cell[657063, 17841, 1534, 31, 251, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[658622, 17876, 65, 1, 27, "Input"],
Cell[658690, 17879, 59, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[658786, 17885, 61, 1, 27, "Input"],
Cell[658850, 17888, 82, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[658969, 17894, 91, 1, 27, "Input"],
Cell[659063, 17897, 57, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[659157, 17903, 87, 1, 27, "Input"],
Cell[659247, 17906, 72, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[659356, 17912, 74, 1, 27, "Input"],
Cell[659433, 17915, 55, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[659525, 17921, 60, 1, 27, "Input"],
Cell[659588, 17924, 229, 5, 39, "Message"],
Cell[659820, 17931, 229, 5, 39, "Message"],
Cell[660052, 17938, 229, 5, 39, "Message"],
Cell[660284, 17945, 225, 5, 23, "Message"],
Cell[660512, 17952, 156, 4, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[660705, 17961, 60, 1, 27, "Input"],
Cell[660768, 17964, 227, 4, 39, "Message"],
Cell[660998, 17970, 225, 4, 39, "Message"],
Cell[661226, 17976, 104, 2, 27, "Output"]
}, Open  ]],
Cell[661345, 17981, 176, 4, 27, "Input",
  InitializationCell->True]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[661570, 17991, 24, 0, 40, "Section"],

Cell[CellGroupData[{
Cell[661619, 17995, 29, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[661673, 17999, 44, 1, 27, "Input"],
Cell[661720, 18002, 35, 1, 27, "Output"]
}, Open  ]],
Cell[661770, 18006, 247, 5, 43, "Input"],

Cell[CellGroupData[{
Cell[662042, 18015, 237, 5, 59, "Input"],
Cell[662282, 18022, 35, 1, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[662354, 18028, 237, 5, 56, "Input"],
Cell[662594, 18035, 286, 4, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[662917, 18044, 165, 3, 27, "Input"],
Cell[663085, 18049, 105, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[663227, 18056, 41, 1, 27, "Input"],
Cell[663271, 18059, 299, 4, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[663607, 18068, 237, 5, 59, "Input"],
Cell[663847, 18075, 683, 11, 135, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[664567, 18091, 237, 5, 56, "Input"],
Cell[664807, 18098, 1037, 15, 239, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[665881, 18118, 237, 5, 56, "Input"],
Cell[666121, 18125, 1794, 29, 330, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[667952, 18159, 237, 5, 56, "Input"],
Cell[668192, 18166, 2461, 38, 495, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[670702, 18210, 47, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[670774, 18214, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[670837, 18218, 80, 1, 27, "Input"],
Cell[670920, 18221, 84, 1, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[671041, 18227, 66, 1, 27, "Input"],
Cell[671110, 18230, 105, 2, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[671252, 18237, 66, 1, 27, "Input"],
Cell[671321, 18240, 105, 2, 70, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[671475, 18248, 40, 0, 28, "Subsubsection"],
Cell[671518, 18250, 659, 14, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[672202, 18268, 88, 1, 27, "Input"],
Cell[672293, 18271, 395, 6, 87, "Message"],
Cell[672691, 18279, 87, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[672815, 18285, 60, 1, 27, "Input"],
Cell[672878, 18288, 395, 6, 87, "Message"],
Cell[673276, 18296, 44, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[673357, 18302, 63, 1, 27, "Input"],
Cell[673423, 18305, 67, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[673527, 18311, 53, 1, 27, "Input"],
Cell[673583, 18314, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[673673, 18320, 131, 3, 27, "Input"],
Cell[673807, 18325, 395, 6, 135, "Message"],
Cell[674205, 18333, 1118, 20, 189, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[675360, 18358, 151, 3, 24, "Input"],
Cell[675514, 18363, 19096, 437, 186, 3246, 237, "GraphicsData", "PostScript", \
"Graphics"],
Cell[694613, 18802, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[694780, 18810, 139, 3, 27, "Input"],
Cell[694922, 18815, 1186, 21, 189, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[696145, 18841, 131, 3, 24, "Input"],
Cell[696279, 18846, 395, 6, 135, "Message"],
Cell[696677, 18854, 1146, 20, 189, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[697860, 18879, 151, 3, 24, "Input"],
Cell[698014, 18884, 16979, 402, 186, 3028, 225, "GraphicsData", "PostScript", \
"Graphics"],
Cell[714996, 19288, 130, 3, 27, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[715175, 19297, 35, 0, 28, "Subsubsection"],
Cell[715213, 19299, 2381, 44, 331, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[717619, 19347, 60, 1, 27, "Input"],
Cell[717682, 19350, 111, 3, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[717830, 19358, 78, 1, 27, "Input"],
Cell[717911, 19361, 114, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[718062, 19369, 144, 3, 27, "Input"],
Cell[718209, 19374, 1283, 24, 125, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[719529, 19403, 151, 3, 27, "Input"],
Cell[719683, 19408, 22357, 516, 186, 3699, 280, "GraphicsData", "PostScript", \
"Graphics"],
Cell[742043, 19926, 182, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[742262, 19936, 144, 3, 27, "Input"],
Cell[742409, 19941, 1265, 24, 109, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[743711, 19970, 151, 3, 27, "Input"],
Cell[743865, 19975, 18611, 426, 186, 3119, 229, "GraphicsData", "PostScript", \
"Graphics"],
Cell[762479, 20403, 182, 5, 27, "Output"]
}, Open  ]],
Cell[762676, 20411, 135, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[762836, 20417, 72, 1, 27, "Input"],
Cell[762911, 20420, 173, 4, 27, "Output"]
}, Open  ]],
Cell[763099, 20427, 26, 0, 27, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[763174, 20433, 44, 0, 29, "Subsection"],
Cell[763221, 20435, 59, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[763305, 20440, 44, 1, 27, "Input"],
Cell[763352, 20443, 35, 1, 27, "Output"]
}, Open  ]],
Cell[763402, 20447, 249, 5, 59, "Input"],
Cell[763654, 20454, 48, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[763727, 20459, 84, 1, 27, "Input"],
Cell[763814, 20462, 22586, 528, 186, 4123, 296, "GraphicsData", "PostScript", \
"Graphics"],
Cell[786403, 20992, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[786570, 21000, 54, 1, 24, "Input"],
Cell[786627, 21003, 23111, 564, 186, 4619, 331, "GraphicsData", "PostScript", \
"Graphics"],
Cell[809741, 21569, 130, 3, 27, "Output"]
}, Open  ]],
Cell[809886, 21575, 56, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[809967, 21580, 84, 1, 27, "Input"],
Cell[810054, 21583, 19715, 484, 186, 3801, 283, "GraphicsData", "PostScript", \
"Graphics"],
Cell[829772, 22069, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[829939, 22077, 54, 1, 27, "Input"],
Cell[829996, 22080, 20045, 471, 186, 3625, 264, "GraphicsData", "PostScript", \
"Graphics"],
Cell[850044, 22553, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[850211, 22561, 84, 1, 27, "Input"],
Cell[850298, 22564, 17524, 449, 186, 3632, 273, "GraphicsData", "PostScript", \
"Graphics"],
Cell[867825, 23015, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[867992, 23023, 54, 1, 27, "Input"],
Cell[868049, 23026, 19950, 474, 186, 3685, 269, "GraphicsData", "PostScript", \
"Graphics"],
Cell[888002, 23502, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[888181, 23511, 41, 0, 29, "Subsection"],
Cell[888225, 23513, 569, 10, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[888819, 23527, 62, 1, 27, "Input"],
Cell[888884, 23530, 112, 2, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[889045, 23538, 38, 0, 29, "Subsection"],
Cell[889086, 23540, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[889152, 23545, 134, 3, 27, "Input"],
Cell[889289, 23550, 309, 5, 92, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[889635, 23560, 230, 3, 51, "Input"],
Cell[889868, 23565, 20051, 278, 9231, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[909956, 23848, 166, 3, 45, "Input"],
Cell[910125, 23853, 1021, 14, 483, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[911183, 23872, 52, 1, 24, "Input"],
Cell[911238, 23875, 4460, 71, 1325, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[915735, 23951, 158, 3, 27, "Input"],
Cell[915896, 23956, 2997, 46, 914, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[918930, 24007, 56, 1, 27, "Input"],
Cell[918989, 24010, 5668, 115, 928, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[924694, 24130, 115, 2, 27, "Input"],
Cell[924812, 24134, 52, 1, 46, "Output"]
}, Open  ]],
Cell[924879, 24138, 117, 2, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[925033, 24145, 32, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[925090, 24149, 305, 6, 59, "Input",
  InitializationCell->True],
Cell[925398, 24157, 325, 6, 70, "Output"]
}, Open  ]],
Cell[925738, 24166, 472, 11, 91, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[926235, 24181, 92, 1, 27, "Input"],
Cell[926330, 24184, 177, 3, 39, "Message"],
Cell[926510, 24189, 119, 2, 36, "Message"],
Cell[926632, 24193, 177, 3, 39, "Message"],
Cell[926812, 24198, 119, 2, 36, "Message"],
Cell[926934, 24202, 216, 4, 39, "Message"],
Cell[927153, 24208, 177, 3, 39, "Message"],
Cell[927333, 24213, 173, 3, 23, "Message"],
Cell[927509, 24218, 119, 2, 36, "Message"],
Cell[927631, 24222, 163, 3, 23, "Message"],
Cell[927797, 24227, 216, 4, 39, "Message"],
Cell[928016, 24233, 216, 4, 39, "Message"],
Cell[928235, 24239, 170, 3, 23, "Message"],
Cell[928408, 24244, 225, 4, 39, "Message"],
Cell[928636, 24250, 21801, 512, 186, 4226, 291, "GraphicsData", "PostScript", \
"Graphics"],
Cell[950440, 24764, 221, 5, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[950698, 24774, 90, 1, 24, "Input"],
Cell[950791, 24777, 437, 6, 61, "Message"],
Cell[951231, 24785, 395, 6, 87, "Message"],
Cell[951629, 24793, 401, 6, 58, "Message"],
Cell[952033, 24801, 354, 5, 71, "Message"],
Cell[952390, 24808, 227, 4, 39, "Message"],
Cell[952620, 24814, 424, 7, 68, "Message"],
Cell[953047, 24823, 169, 3, 23, "Message"],
Cell[953219, 24828, 354, 5, 71, "Message"],
Cell[953576, 24835, 226, 4, 39, "Message"],
Cell[953805, 24841, 354, 5, 71, "Message"],
Cell[954162, 24848, 170, 3, 23, "Message"],
Cell[954335, 24853, 227, 4, 39, "Message"],
Cell[954565, 24859, 168, 3, 23, "Message"],
Cell[954736, 24864, 193, 3, 36, "Message"],
Cell[954932, 24869, 395, 6, 87, "Message"],
Cell[955330, 24877, 194, 3, 54, "Message"],
Cell[955527, 24882, 194, 3, 54, "Message"],
Cell[955724, 24887, 162, 3, 23, "Message"],
Cell[955889, 24892, 395, 6, 87, "Message"],
Cell[956287, 24900, 169, 3, 23, "Message"],
Cell[956459, 24905, 42, 1, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[956538, 24911, 65, 1, 24, "Input"],
Cell[956606, 24914, 177, 3, 39, "Message"],
Cell[956786, 24919, 177, 3, 39, "Message"],
Cell[956966, 24924, 177, 3, 39, "Message"],
Cell[957146, 24929, 173, 3, 23, "Message"],
Cell[957322, 24934, 55, 1, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[957414, 24940, 61, 1, 24, "Input"],
Cell[957478, 24943, 77, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[957592, 24949, 148, 4, 27, "Input",
  InitializationCell->True],
Cell[957743, 24955, 183, 4, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[957963, 24964, 320, 7, 59, "Input",
  InitializationCell->True],
Cell[958286, 24973, 337, 6, 70, "Output"]
}, Open  ]],
Cell[958638, 24982, 1396, 27, 219, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[960059, 25013, 65, 1, 27, "Input"],
Cell[960127, 25016, 54, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[960218, 25022, 61, 1, 27, "Input"],
Cell[960282, 25025, 77, 1, 27, "Output"]
}, Open  ]],
Cell[960374, 25029, 176, 4, 27, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[960575, 25037, 60, 1, 27, "Input"],
Cell[960638, 25040, 177, 3, 39, "Message"],
Cell[960818, 25045, 177, 3, 39, "Message"],
Cell[960998, 25050, 177, 3, 39, "Message"],
Cell[961178, 25055, 173, 3, 23, "Message"],
Cell[961354, 25060, 129, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[961520, 25067, 66, 1, 27, "Input"],
Cell[961589, 25070, 177, 3, 39, "Message"],
Cell[961769, 25075, 177, 3, 39, "Message"],
Cell[961949, 25080, 177, 3, 39, "Message"],
Cell[962129, 25085, 173, 3, 23, "Message"],
Cell[962305, 25090, 129, 2, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[962483, 25098, 32, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[962540, 25102, 181, 3, 27, "Input"],
Cell[962724, 25107, 15810, 280, 2032, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[978571, 25392, 56, 1, 24, "Input"],
Cell[978630, 25395, 2224, 40, 283, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[980891, 25440, 110, 2, 24, "Input"],
Cell[981004, 25444, 837, 13, 133, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[981878, 25462, 50, 1, 24, "Input"],
Cell[981931, 25465, 78, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[982046, 25471, 89, 1, 27, "Input"],
Cell[982138, 25474, 225, 4, 39, "Message"],
Cell[982366, 25480, 58, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[982461, 25486, 89, 1, 27, "Input"],
Cell[982553, 25489, 177, 3, 39, "Message"],
Cell[982733, 25494, 177, 3, 39, "Message"],
Cell[982913, 25499, 177, 3, 39, "Message"],
Cell[983093, 25504, 173, 3, 23, "Message"],
Cell[983269, 25509, 225, 4, 39, "Message"],
Cell[983497, 25515, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[983587, 25521, 255, 5, 59, "Input"],
Cell[983845, 25528, 1042, 17, 176, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[984924, 25550, 1159, 20, 203, "Input"],
Cell[986086, 25572, 1046, 17, 176, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[987169, 25594, 54, 1, 27, "Input"],
Cell[987226, 25597, 93, 1, 27, "Output"]
}, Open  ]],
Cell[987334, 25601, 41, 1, 27, "Input"],
Cell[987378, 25604, 42, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[987445, 25609, 961, 16, 158, "Input",
  InitializationCell->True],
Cell[988409, 25627, 845, 13, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[989291, 25645, 54, 1, 27, "Input"],
Cell[989348, 25648, 103, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[989488, 25655, 331, 6, 59, "Input"],
Cell[989822, 25663, 226, 4, 39, "Message"],
Cell[990051, 25669, 125, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[990213, 25676, 90, 1, 27, "Input"],
Cell[990306, 25679, 225, 4, 39, "Message"],
Cell[990534, 25685, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[990624, 25691, 90, 1, 27, "Input"],
Cell[990717, 25694, 177, 3, 39, "Message"],
Cell[990897, 25699, 177, 3, 39, "Message"],
Cell[991077, 25704, 177, 3, 39, "Message"],
Cell[991257, 25709, 173, 3, 23, "Message"],
Cell[991433, 25714, 63, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[991533, 25720, 50, 1, 27, "Input"],
Cell[991586, 25723, 110, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[991733, 25730, 90, 1, 27, "Input"],
Cell[991826, 25733, 177, 3, 39, "Message"],
Cell[992006, 25738, 177, 3, 39, "Message"],
Cell[992186, 25743, 177, 3, 39, "Message"],
Cell[992366, 25748, 173, 3, 23, "Message"],
Cell[992542, 25753, 107, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[992686, 25760, 90, 1, 27, "Input"],
Cell[992779, 25763, 225, 4, 39, "Message"],
Cell[993007, 25769, 62, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[993106, 25775, 268, 4, 43, "Input"],
Cell[993377, 25781, 354, 5, 71, "Message"],
Cell[993734, 25788, 225, 4, 39, "Message"],
Cell[993962, 25794, 127, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[994126, 25801, 49, 1, 27, "Input"],
Cell[994178, 25804, 111, 2, 27, "Output"]
}, Open  ]],
Cell[994304, 25809, 41, 1, 27, "Input"],
Cell[994348, 25812, 101, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[994474, 25818, 84, 1, 27, "Input"],
Cell[994561, 25821, 14240, 459, 186, 4795, 338, "GraphicsData", "PostScript", \
"Graphics"],
Cell[1008804, 26282, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1008971, 26290, 84, 1, 27, "Input"],
Cell[1009058, 26293, 15800, 470, 186, 4686, 328, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1024861, 26765, 130, 3, 27, "Output"]
}, Open  ]],
Cell[1025006, 26771, 51, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1025082, 26776, 196, 4, 43, "Input"],
Cell[1025281, 26782, 177, 3, 39, "Message"],
Cell[1025461, 26787, 177, 3, 39, "Message"],
Cell[1025641, 26792, 177, 3, 39, "Message"],
Cell[1025821, 26797, 173, 3, 23, "Message"],
Cell[1025997, 26802, 4775, 70, 477, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1030809, 26877, 67, 1, 27, "Input"],
Cell[1030879, 26880, 171, 3, 23, "Message"],
Cell[1031053, 26885, 13839, 474, 186, 5771, 370, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1044895, 27361, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1045062, 27369, 90, 1, 27, "Input"],
Cell[1045155, 27372, 177, 3, 39, "Message"],
Cell[1045335, 27377, 177, 3, 39, "Message"],
Cell[1045515, 27382, 177, 3, 39, "Message"],
Cell[1045695, 27387, 173, 3, 23, "Message"],
Cell[1045871, 27392, 226, 4, 39, "Message"],
Cell[1046100, 27398, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1046190, 27404, 90, 1, 27, "Input"],
Cell[1046283, 27407, 226, 4, 39, "Message"],
Cell[1046512, 27413, 52, 1, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1046625, 27421, 37, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[1046687, 27425, 36, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[1046748, 27429, 157, 3, 27, "Input"],
Cell[1046908, 27434, 210, 4, 39, "Message"]
}, Open  ]],
Cell[1047133, 27441, 159, 3, 27, "Input"],
Cell[1047295, 27446, 159, 3, 27, "Input"],
Cell[1047457, 27451, 157, 3, 27, "Input"],
Cell[1047617, 27456, 159, 3, 27, "Input"],
Cell[1047779, 27461, 159, 3, 27, "Input"],
Cell[1047941, 27466, 157, 3, 27, "Input"],
Cell[1048101, 27471, 159, 3, 27, "Input"],
Cell[1048263, 27476, 159, 3, 27, "Input"],
Cell[1048425, 27481, 157, 3, 27, "Input"],
Cell[1048585, 27486, 159, 3, 27, "Input"],
Cell[1048747, 27491, 159, 3, 27, "Input"],
Cell[1048909, 27496, 157, 3, 27, "Input"],
Cell[1049069, 27501, 159, 3, 27, "Input"],
Cell[1049231, 27506, 159, 3, 27, "Input"],
Cell[1049393, 27511, 157, 3, 27, "Input"],
Cell[1049553, 27516, 159, 3, 27, "Input"],
Cell[1049715, 27521, 159, 3, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[1049911, 27529, 30, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[1049966, 27533, 445, 8, 123, "Input"],
Cell[1050414, 27543, 194, 3, 39, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1050645, 27551, 79, 1, 27, "Input"],
Cell[1050727, 27554, 395, 6, 87, "Message"],
Cell[1051125, 27562, 395, 6, 87, "Message"],
Cell[1051523, 27570, 395, 6, 87, "Message"],
Cell[1051921, 27578, 169, 3, 23, "Message"],
Cell[1052093, 27583, 909, 15, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1053039, 27603, 79, 1, 24, "Input"],
Cell[1053121, 27606, 395, 6, 87, "Message"],
Cell[1053519, 27614, 395, 6, 87, "Message"],
Cell[1053917, 27622, 395, 6, 87, "Message"],
Cell[1054315, 27630, 169, 3, 23, "Message"],
Cell[1054487, 27635, 917, 16, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1055441, 27656, 79, 1, 24, "Input"],
Cell[1055523, 27659, 395, 6, 87, "Message"],
Cell[1055921, 27667, 395, 6, 87, "Message"],
Cell[1056319, 27675, 395, 6, 87, "Message"],
Cell[1056717, 27683, 169, 3, 23, "Message"],
Cell[1056889, 27688, 903, 15, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1057829, 27708, 69, 1, 24, "Input"],
Cell[1057901, 27711, 911, 15, 117, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1058849, 27731, 69, 1, 27, "Input"],
Cell[1058921, 27734, 898, 14, 117, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1059856, 27753, 69, 1, 27, "Input"],
Cell[1059928, 27756, 898, 15, 117, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1060863, 27776, 69, 1, 27, "Input"],
Cell[1060935, 27779, 903, 15, 117, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1061875, 27799, 69, 1, 27, "Input"],
Cell[1061947, 27802, 895, 14, 117, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1062879, 27821, 69, 1, 27, "Input"],
Cell[1062951, 27824, 899, 15, 117, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[1063899, 27845, 30, 0, 29, "Subsection"],
Cell[1063932, 27847, 445, 8, 123, "Input"],

Cell[CellGroupData[{
Cell[1064402, 27859, 79, 1, 27, "Input"],
Cell[1064484, 27862, 1085, 19, 129, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1065606, 27886, 79, 1, 27, "Input"],
Cell[1065688, 27889, 1102, 20, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1066827, 27914, 79, 1, 24, "Input"],
Cell[1066909, 27917, 1084, 20, 131, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1068030, 27942, 69, 1, 24, "Input"],
Cell[1068102, 27945, 1088, 19, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1069227, 27969, 69, 1, 24, "Input"],
Cell[1069299, 27972, 1088, 19, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1070424, 27996, 69, 1, 24, "Input"],
Cell[1070496, 27999, 1082, 19, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1071615, 28023, 69, 1, 24, "Input"],
Cell[1071687, 28026, 1077, 20, 137, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1072801, 28051, 69, 1, 24, "Input"],
Cell[1072873, 28054, 1072, 20, 137, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1073982, 28079, 69, 1, 24, "Input"],
Cell[1074054, 28082, 1067, 19, 137, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1075170, 28107, 29, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[1075224, 28111, 390, 7, 107, "Input"],
Cell[1075617, 28120, 892, 15, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1076546, 28140, 393, 7, 104, "Input"],
Cell[1076942, 28149, 881, 14, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1077860, 28168, 385, 7, 104, "Input"],
Cell[1078248, 28177, 893, 15, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1079178, 28197, 390, 7, 104, "Input"],
Cell[1079571, 28206, 870, 13, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1080478, 28224, 393, 7, 104, "Input"],
Cell[1080874, 28233, 899, 15, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1081810, 28253, 385, 7, 104, "Input"],
Cell[1082198, 28262, 883, 14, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1083118, 28281, 390, 7, 104, "Input"],
Cell[1083511, 28290, 884, 14, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1084432, 28309, 393, 7, 104, "Input"],
Cell[1084828, 28318, 887, 15, 187, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1085752, 28338, 393, 7, 104, "Input"],
Cell[1086148, 28347, 897, 15, 187, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1087094, 28368, 29, 0, 29, "Subsection"],
Cell[1087126, 28370, 443, 8, 123, "Input"],

Cell[CellGroupData[{
Cell[1087594, 28382, 78, 1, 27, "Input"],
Cell[1087675, 28385, 1089, 20, 131, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1088801, 28410, 68, 1, 27, "Input"],
Cell[1088872, 28413, 1092, 21, 131, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1090001, 28439, 68, 1, 27, "Input"],
Cell[1090072, 28442, 395, 6, 87, "Message"],
Cell[1090470, 28450, 395, 6, 87, "Message"],
Cell[1090868, 28458, 395, 6, 87, "Message"],
Cell[1091266, 28466, 169, 3, 23, "Message"],
Cell[1091438, 28471, 1081, 19, 131, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1092556, 28495, 78, 1, 27, "Input"],
Cell[1092637, 28498, 1091, 20, 131, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1093765, 28523, 68, 1, 27, "Input"],
Cell[1093836, 28526, 1096, 21, 131, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1094969, 28552, 68, 1, 27, "Input"],
Cell[1095040, 28555, 395, 6, 87, "Message"],
Cell[1095438, 28563, 395, 6, 87, "Message"],
Cell[1095836, 28571, 395, 6, 87, "Message"],
Cell[1096234, 28579, 169, 3, 23, "Message"],
Cell[1096406, 28584, 1092, 20, 131, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1097535, 28609, 78, 1, 27, "Input"],
Cell[1097616, 28612, 1096, 21, 131, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1098749, 28638, 68, 1, 27, "Input"],
Cell[1098820, 28641, 1074, 19, 131, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1099931, 28665, 68, 1, 27, "Input"],
Cell[1100002, 28668, 395, 6, 87, "Message"],
Cell[1100400, 28676, 395, 6, 87, "Message"],
Cell[1100798, 28684, 395, 6, 87, "Message"],
Cell[1101196, 28692, 169, 3, 23, "Message"],
Cell[1101368, 28697, 1081, 20, 131, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[1102498, 28723, 30, 0, 29, "Subsection"],
Cell[1102531, 28725, 445, 8, 123, "Input"],
Cell[1102979, 28735, 72, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1103076, 28740, 69, 1, 27, "Input"],
Cell[1103148, 28743, 901, 15, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1104086, 28763, 69, 1, 24, "Input"],
Cell[1104158, 28766, 889, 14, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1105084, 28785, 69, 1, 24, "Input"],
Cell[1105156, 28788, 910, 16, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1106103, 28809, 69, 1, 24, "Input"],
Cell[1106175, 28812, 903, 15, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1107115, 28832, 69, 1, 24, "Input"],
Cell[1107187, 28835, 903, 15, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1108127, 28855, 69, 1, 24, "Input"],
Cell[1108199, 28858, 899, 15, 117, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1109147, 28879, 30, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[1109202, 28883, 445, 8, 123, "Input"],
Cell[1109650, 28893, 205, 3, 39, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1109892, 28901, 76, 1, 27, "Input"],
Cell[1109971, 28904, 60, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1110068, 28910, 76, 1, 27, "Input"],
Cell[1110147, 28913, 60, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1110244, 28919, 72, 1, 27, "Input"],
Cell[1110319, 28922, 72, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1110428, 28928, 69, 1, 27, "Input"],
Cell[1110500, 28931, 1093, 20, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1111630, 28956, 69, 1, 24, "Input"],
Cell[1111702, 28959, 1083, 19, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1112822, 28983, 69, 1, 24, "Input"],
Cell[1112894, 28986, 1094, 20, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1114025, 29011, 69, 1, 24, "Input"],
Cell[1114097, 29014, 395, 6, 87, "Message"],
Cell[1114495, 29022, 395, 6, 87, "Message"],
Cell[1114893, 29030, 395, 6, 87, "Message"],
Cell[1115291, 29038, 169, 3, 23, "Message"],
Cell[1115463, 29043, 1090, 19, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1116590, 29067, 69, 1, 24, "Input"],
Cell[1116662, 29070, 395, 6, 87, "Message"],
Cell[1117060, 29078, 395, 6, 87, "Message"],
Cell[1117458, 29086, 395, 6, 87, "Message"],
Cell[1117856, 29094, 169, 3, 23, "Message"],
Cell[1118028, 29099, 1085, 19, 129, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1119150, 29123, 69, 1, 24, "Input"],
Cell[1119222, 29126, 395, 6, 87, "Message"],
Cell[1119620, 29134, 395, 6, 87, "Message"],
Cell[1120018, 29142, 395, 6, 87, "Message"],
Cell[1120416, 29150, 169, 3, 23, "Message"],
Cell[1120588, 29155, 1059, 18, 137, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1121696, 29179, 30, 0, 29, "Subsection"],
Cell[1121729, 29181, 444, 8, 123, "Input"],

Cell[CellGroupData[{
Cell[1122198, 29193, 88, 1, 27, "Input"],
Cell[1122289, 29196, 76, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1122402, 29202, 92, 1, 27, "Input"],
Cell[1122497, 29205, 57, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1122591, 29211, 78, 1, 27, "Input"],
Cell[1122672, 29214, 395, 6, 87, "Message"],
Cell[1123070, 29222, 395, 6, 87, "Message"],
Cell[1123468, 29230, 395, 6, 87, "Message"],
Cell[1123866, 29238, 169, 3, 23, "Message"],
Cell[1124038, 29243, 868, 13, 107, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1124943, 29261, 78, 1, 24, "Input"],
Cell[1125024, 29264, 395, 6, 87, "Message"],
Cell[1125422, 29272, 395, 6, 87, "Message"],
Cell[1125820, 29280, 395, 6, 87, "Message"],
Cell[1126218, 29288, 169, 3, 23, "Message"],
Cell[1126390, 29293, 881, 13, 107, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1127308, 29311, 78, 1, 24, "Input"],
Cell[1127389, 29314, 395, 6, 87, "Message"],
Cell[1127787, 29322, 395, 6, 87, "Message"],
Cell[1128185, 29330, 395, 6, 87, "Message"],
Cell[1128583, 29338, 169, 3, 23, "Message"],
Cell[1128755, 29343, 872, 13, 107, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1129664, 29361, 68, 1, 24, "Input"],
Cell[1129735, 29364, 890, 14, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1130662, 29383, 68, 1, 24, "Input"],
Cell[1130733, 29386, 886, 14, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1131656, 29405, 68, 1, 24, "Input"],
Cell[1131727, 29408, 913, 16, 117, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1132677, 29429, 102, 2, 24, "Input"],
Cell[1132782, 29433, 395, 6, 119, "Message"],
Cell[1133180, 29441, 395, 6, 119, "Message"],
Cell[1133578, 29449, 395, 6, 119, "Message"],
Cell[1133976, 29457, 169, 3, 55, "Message"],
Cell[1134148, 29462, 845, 15, 155, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1135030, 29482, 77, 1, 24, "Input"],
Cell[1135110, 29485, 395, 6, 87, "Message"],
Cell[1135508, 29493, 44, 1, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1135589, 29499, 77, 1, 24, "Input"],
Cell[1135669, 29502, 395, 6, 87, "Message"],
Cell[1136067, 29510, 44, 1, 29, "Output"]
}, Closed]],
Cell[1136126, 29514, 92, 1, 24, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[1136255, 29520, 30, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[1136310, 29524, 445, 8, 123, "Input"],
Cell[1136758, 29534, 220, 4, 39, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1137015, 29543, 88, 1, 27, "Input"],
Cell[1137106, 29546, 83, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1137226, 29552, 60, 1, 27, "Input"],
Cell[1137289, 29555, 62, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1137388, 29561, 72, 1, 27, "Input"],
Cell[1137463, 29564, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1137553, 29570, 79, 1, 27, "Input"],
Cell[1137635, 29573, 954, 14, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1138626, 29592, 79, 1, 27, "Input"],
Cell[1138708, 29595, 960, 14, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1139705, 29614, 67, 1, 27, "Input"],
Cell[1139775, 29617, 966, 14, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1140778, 29636, 67, 1, 27, "Input"],
Cell[1140848, 29639, 972, 15, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1141857, 29659, 101, 2, 27, "Input"],
Cell[1141961, 29663, 395, 6, 87, "Message"],
Cell[1142359, 29671, 395, 6, 87, "Message"],
Cell[1142757, 29679, 395, 6, 87, "Message"],
Cell[1143155, 29687, 169, 3, 23, "Message"],
Cell[1143327, 29692, 959, 13, 75, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1144323, 29710, 111, 2, 27, "Input"],
Cell[1144437, 29714, 395, 6, 87, "Message"],
Cell[1144835, 29722, 395, 6, 87, "Message"],
Cell[1145233, 29730, 68, 1, 29, "Output"]
}, Open  ]],
Cell[1145316, 29734, 394, 8, 75, "Input"],

Cell[CellGroupData[{
Cell[1145735, 29746, 78, 1, 27, "Input"],
Cell[1145816, 29749, 395, 6, 87, "Message"],
Cell[1146214, 29757, 395, 6, 87, "Message"],
Cell[1146612, 29765, 395, 6, 87, "Message"],
Cell[1147010, 29773, 169, 3, 23, "Message"],
Cell[1147182, 29778, 2250, 36, 277, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1149469, 29819, 87, 1, 27, "Input"],
Cell[1149559, 29822, 59, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1149655, 29828, 66, 1, 27, "Input"],
Cell[1149724, 29831, 2332, 39, 295, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1152093, 29875, 84, 1, 27, "Input"],
Cell[1152180, 29878, 56, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1152273, 29884, 66, 1, 27, "Input"],
Cell[1152342, 29887, 395, 6, 87, "Message"],
Cell[1152740, 29895, 395, 6, 87, "Message"],
Cell[1153138, 29903, 395, 6, 87, "Message"],
Cell[1153536, 29911, 169, 3, 23, "Message"],
Cell[1153708, 29916, 958, 15, 137, "Output"]
}, Open  ]],
Cell[1154681, 29934, 84, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[1154802, 29940, 26, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[1154853, 29944, 44, 1, 27, "Input"],
Cell[1154900, 29947, 68, 1, 23, "Print",
  CellTags->"Info3267953076-4906506"],
Cell[1154971, 29950, 581, 15, 23, "Print",
  CellTags->"Info3267953076-4906506"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1155589, 29970, 40, 1, 27, "Input"],
Cell[1155632, 29973, 38, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1155707, 29979, 75, 1, 27, "Input"],
Cell[1155785, 29982, 106, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1155928, 29989, 230, 4, 43, "Input"],
Cell[1156161, 29995, 1533, 25, 166, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1157731, 30025, 228, 4, 43, "Input"],
Cell[1157962, 30031, 177, 3, 39, "Message"],
Cell[1158142, 30036, 177, 3, 39, "Message"],
Cell[1158322, 30041, 177, 3, 39, "Message"],
Cell[1158502, 30046, 173, 3, 23, "Message"],
Cell[1158678, 30051, 227, 4, 39, "Message"],
Cell[1158908, 30057, 227, 4, 39, "Message"],
Cell[1159138, 30063, 227, 4, 39, "Message"],
Cell[1159368, 30069, 168, 3, 23, "Message"],
Cell[1159539, 30074, 1601, 26, 187, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1161177, 30105, 45, 1, 27, "Input"],
Cell[1161225, 30108, 69, 1, 23, "Print",
  CellTags->"Info3267953430-8404111"],
Cell[1161297, 30111, 1102, 23, 176, "Print",
  CellTags->"Info3267953430-8404111"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1162436, 30139, 87, 1, 27, "Input"],
Cell[1162526, 30142, 82, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1162645, 30148, 107, 2, 27, "Input"],
Cell[1162755, 30152, 55, 1, 27, "Output"]
}, Open  ]],
Cell[1162825, 30156, 26, 0, 27, "Input"],

Cell[CellGroupData[{
Cell[1162876, 30160, 90, 1, 27, "Input"],
Cell[1162969, 30163, 177, 3, 39, "Message"],
Cell[1163149, 30168, 177, 3, 39, "Message"],
Cell[1163329, 30173, 177, 3, 39, "Message"],
Cell[1163509, 30178, 173, 3, 23, "Message"],
Cell[1163685, 30183, 226, 4, 39, "Message"],
Cell[1163914, 30189, 57, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1164008, 30195, 90, 1, 27, "Input"],
Cell[1164101, 30198, 177, 3, 39, "Message"],
Cell[1164281, 30203, 177, 3, 39, "Message"],
Cell[1164461, 30208, 177, 3, 39, "Message"],
Cell[1164641, 30213, 173, 3, 23, "Message"],
Cell[1164817, 30218, 354, 5, 71, "Message"],
Cell[1165174, 30225, 227, 4, 39, "Message"],
Cell[1165404, 30231, 67, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1165508, 30237, 90, 1, 27, "Input"],
Cell[1165601, 30240, 226, 4, 39, "Message"],
Cell[1165830, 30246, 58, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1165925, 30252, 228, 4, 43, "Input"],
Cell[1166156, 30258, 177, 3, 39, "Message"],
Cell[1166336, 30263, 177, 3, 39, "Message"],
Cell[1166516, 30268, 177, 3, 39, "Message"],
Cell[1166696, 30273, 173, 3, 23, "Message"],
Cell[1166872, 30278, 227, 4, 39, "Message"],
Cell[1167102, 30284, 227, 4, 39, "Message"],
Cell[1167332, 30290, 227, 4, 39, "Message"],
Cell[1167562, 30296, 168, 3, 23, "Message"],
Cell[1167733, 30301, 192, 3, 23, "Message"],
Cell[1167928, 30306, 192, 3, 23, "Message"],
Cell[1168123, 30311, 192, 3, 23, "Message"],
Cell[1168318, 30316, 162, 3, 23, "Message"],
Cell[1168483, 30321, 8023, 138, 1376, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1176543, 30464, 228, 4, 43, "Input"],
Cell[1176774, 30470, 177, 3, 39, "Message"],
Cell[1176954, 30475, 177, 3, 39, "Message"],
Cell[1177134, 30480, 177, 3, 39, "Message"],
Cell[1177314, 30485, 173, 3, 23, "Message"],
Cell[1177490, 30490, 227, 4, 39, "Message"],
Cell[1177720, 30496, 227, 4, 39, "Message"],
Cell[1177950, 30502, 228, 4, 39, "Message"],
Cell[1178181, 30508, 168, 3, 23, "Message"],
Cell[1178352, 30513, 1672, 33, 207, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1180061, 30551, 74, 1, 24, "Input"],
Cell[1180138, 30554, 60, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1180235, 30560, 75, 1, 27, "Input"],
Cell[1180313, 30563, 64, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1180414, 30569, 228, 4, 43, "Input"],
Cell[1180645, 30575, 177, 3, 39, "Message"],
Cell[1180825, 30580, 177, 3, 39, "Message"],
Cell[1181005, 30585, 177, 3, 39, "Message"],
Cell[1181185, 30590, 173, 3, 23, "Message"],
Cell[1181361, 30595, 226, 4, 39, "Message"],
Cell[1181590, 30601, 226, 4, 39, "Message"],
Cell[1181819, 30607, 226, 4, 39, "Message"],
Cell[1182048, 30613, 168, 3, 23, "Message"],
Cell[1182219, 30618, 1600, 32, 185, "Output"]
}, Closed]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[1183880, 30657, 39, 0, 40, "Section"],

Cell[CellGroupData[{
Cell[1183944, 30661, 28, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[1183997, 30665, 80, 1, 27, "Input"],
Cell[1184080, 30668, 163, 4, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1184280, 30677, 104, 2, 27, "Input"],
Cell[1184387, 30681, 328, 6, 78, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1184752, 30692, 44, 1, 27, "Input"],
Cell[1184799, 30695, 275, 5, 78, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1185111, 30705, 77, 1, 27, "Input"],
Cell[1185191, 30708, 588, 18, 49, "Output"]
}, Open  ]],
Cell[1185794, 30729, 133, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[1185952, 30736, 67, 1, 27, "Input"],
Cell[1186022, 30739, 16887, 425, 186, 3508, 254, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1202912, 31166, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1203109, 31174, 324, 7, 75, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1203458, 31185, 86, 1, 27, "Input"],
Cell[1203547, 31188, 15338, 369, 186, 3038, 213, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1218888, 31559, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1219055, 31567, 80, 1, 24, "Input"],
Cell[1219138, 31570, 17305, 458, 186, 4163, 290, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1236446, 32030, 182, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1236665, 32040, 173, 3, 43, "Input"],
Cell[1236841, 32045, 668, 11, 75, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1237546, 32061, 60, 1, 27, "Input"],
Cell[1237609, 32064, 46, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1237692, 32070, 45, 1, 27, "Input"],
Cell[1237740, 32073, 18241, 515, 186, 4955, 345, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1255984, 32590, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1256181, 32598, 61, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1256267, 32603, 80, 1, 27, "Input"],
Cell[1256350, 32606, 18116, 459, 186, 4043, 279, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1274469, 33067, 182, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1274688, 33077, 45, 1, 27, "Input"],
Cell[1274736, 33080, 18776, 531, 186, 5081, 356, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1293515, 33613, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1293712, 33621, 55, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1293792, 33626, 92, 1, 27, "Input"],
Cell[1293887, 33629, 16247, 401, 186, 3296, 235, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1310137, 34032, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1310334, 34040, 313, 7, 91, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1310672, 34051, 46, 1, 27, "Input"],
Cell[1310721, 34054, 54, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1310812, 34060, 46, 1, 27, "Input"],
Cell[1310861, 34063, 105, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1311003, 34071, 46, 1, 27, "Input"],
Cell[1311052, 34074, 53, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[1311154, 34081, 37, 0, 29, "Subsection"],
Cell[1311194, 34083, 210, 4, 59, "Input"],

Cell[CellGroupData[{
Cell[1311429, 34091, 47, 1, 27, "Input"],
Cell[1311479, 34094, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1311569, 34100, 74, 1, 27, "Input"],
Cell[1311646, 34103, 116, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1311799, 34110, 166, 4, 43, "Input"],
Cell[1311968, 34116, 19175, 477, 186, 4039, 286, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1331146, 34595, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1331313, 34603, 171, 4, 43, "Input"],
Cell[1331487, 34609, 17804, 431, 186, 3507, 250, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1349294, 35042, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1349461, 35050, 55, 1, 27, "Input"],
Cell[1349519, 35053, 379, 6, 58, "Message"],
Cell[1349901, 35061, 354, 5, 71, "Message"],
Cell[1350258, 35068, 227, 4, 39, "Message"],
Cell[1350488, 35074, 418, 6, 58, "Message"],
Cell[1350909, 35082, 419, 6, 58, "Message"],
Cell[1351331, 35090, 169, 3, 23, "Message"],
Cell[1351503, 35095, 55, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1351595, 35101, 47, 1, 27, "Input"],
Cell[1351645, 35104, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1351735, 35110, 150, 3, 27, "Input"],
Cell[1351888, 35115, 244, 4, 58, "Message"],
Cell[1352135, 35121, 354, 5, 71, "Message"],
Cell[1352492, 35128, 226, 4, 39, "Message"],
Cell[1352721, 35134, 244, 4, 58, "Message"],
Cell[1352968, 35140, 354, 5, 71, "Message"],
Cell[1353325, 35147, 226, 4, 39, "Message"],
Cell[1353554, 35153, 424, 6, 58, "Message"],
Cell[1353981, 35161, 169, 3, 23, "Message"],
Cell[1354153, 35166, 395, 6, 87, "Message"],
Cell[1354551, 35174, 395, 6, 87, "Message"],
Cell[1354949, 35182, 395, 6, 87, "Message"],
Cell[1355347, 35190, 169, 3, 23, "Message"],
Cell[1355519, 35195, 166, 3, 23, "Message"],
Cell[1355688, 35200, 354, 5, 71, "Message"],
Cell[1356045, 35207, 170, 3, 23, "Message"],
Cell[1356218, 35212, 226, 4, 39, "Message"],
Cell[1356447, 35218, 168, 3, 23, "Message"],
Cell[1356618, 35223, 166, 3, 23, "Message"],
Cell[1356787, 35228, 166, 3, 23, "Message"],
Cell[1356956, 35233, 167, 3, 23, "Message"],
Cell[1357126, 35238, 2490, 48, 123, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1359653, 35291, 44, 1, 27, "Input"],
Cell[1359700, 35294, 12065, 384, 186, 4043, 280, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1371768, 35680, 130, 3, 27, "Output"]
}, Open  ]],
Cell[1371913, 35686, 206, 4, 59, "Input"],

Cell[CellGroupData[{
Cell[1372144, 35694, 150, 3, 27, "Input"],
Cell[1372297, 35699, 244, 4, 58, "Message"],
Cell[1372544, 35705, 244, 4, 58, "Message"],
Cell[1372791, 35711, 244, 4, 58, "Message"],
Cell[1373038, 35717, 169, 3, 23, "Message"],
Cell[1373210, 35722, 166, 3, 23, "Message"],
Cell[1373379, 35727, 166, 3, 23, "Message"],
Cell[1373548, 35732, 488, 9, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1374073, 35746, 2424, 49, 219, "Input"],
Cell[1376500, 35797, 2425, 49, 123, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1378962, 35851, 120, 2, 27, "Input"],
Cell[1379085, 35855, 14210, 417, 186, 4077, 287, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1393298, 36274, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[1393477, 36283, 28, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[1393530, 36287, 80, 1, 27, "Input"],
Cell[1393613, 36290, 163, 4, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1393813, 36299, 104, 2, 27, "Input"],
Cell[1393920, 36303, 328, 6, 78, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1394285, 36314, 44, 1, 27, "Input"],
Cell[1394332, 36317, 275, 5, 78, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1394644, 36327, 77, 1, 27, "Input"],
Cell[1394724, 36330, 588, 18, 49, "Output"]
}, Open  ]],
Cell[1395327, 36351, 133, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[1395485, 36358, 67, 1, 27, "Input"],
Cell[1395555, 36361, 11841, 363, 186, 3508, 254, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1407399, 36726, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1407596, 36734, 324, 7, 75, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1407945, 36745, 86, 1, 27, "Input"],
Cell[1408034, 36748, 10693, 312, 186, 3038, 213, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1418730, 37062, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1418897, 37070, 80, 1, 24, "Input"],
Cell[1418980, 37073, 12295, 396, 186, 4163, 290, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1431278, 37471, 182, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1431497, 37481, 173, 3, 43, "Input"],
Cell[1431673, 37486, 668, 11, 75, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1432378, 37502, 60, 1, 27, "Input"],
Cell[1432441, 37505, 46, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1432524, 37511, 45, 1, 27, "Input"],
Cell[1432572, 37514, 13207, 453, 186, 4955, 345, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1445782, 37969, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1445979, 37977, 61, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1446065, 37982, 80, 1, 27, "Input"],
Cell[1446148, 37985, 12745, 392, 186, 4043, 279, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1458896, 38379, 182, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1459115, 38389, 45, 1, 27, "Input"],
Cell[1459163, 38392, 13588, 467, 186, 5081, 356, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1472754, 38861, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1472951, 38869, 55, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1473031, 38874, 92, 1, 27, "Input"],
Cell[1473126, 38877, 11342, 340, 186, 3296, 235, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1484471, 39219, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1484668, 39227, 313, 7, 91, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1485006, 39238, 46, 1, 27, "Input"],
Cell[1485055, 39241, 54, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1485146, 39247, 46, 1, 27, "Input"],
Cell[1485195, 39250, 105, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1485337, 39258, 46, 1, 27, "Input"],
Cell[1485386, 39261, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1485476, 39267, 148, 3, 27, "Input"],
Cell[1485627, 39272, 380, 6, 58, "Message"],
Cell[1486010, 39280, 354, 5, 71, "Message"],
Cell[1486367, 39287, 227, 4, 39, "Message"],
Cell[1486597, 39293, 418, 6, 58, "Message"],
Cell[1487018, 39301, 419, 6, 58, "Message"],
Cell[1487440, 39309, 169, 3, 23, "Message"],
Cell[1487612, 39314, 354, 5, 71, "Message"],
Cell[1487969, 39321, 227, 4, 39, "Message"],
Cell[1488199, 39327, 354, 5, 71, "Message"],
Cell[1488556, 39334, 170, 3, 23, "Message"],
Cell[1488729, 39339, 227, 4, 39, "Message"],
Cell[1488959, 39345, 168, 3, 23, "Message"],
Cell[1489130, 39350, 2334, 34, 219, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1491501, 39389, 121, 2, 27, "Input"],
Cell[1491625, 39393, 117, 2, 29, "Output"]
}, Open  ]],
Cell[1491757, 39398, 106, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[1491888, 39404, 66, 1, 27, "Input"],
Cell[1491957, 39407, 49, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1492043, 39413, 72, 1, 27, "Input"],
Cell[1492118, 39416, 25862, 586, 186, 4581, 319, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1517983, 40004, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1518150, 40012, 85, 1, 24, "Input"],
Cell[1518238, 40015, 381, 6, 58, "Message"],
Cell[1518622, 40023, 354, 5, 71, "Message"],
Cell[1518979, 40030, 227, 4, 39, "Message"],
Cell[1519209, 40036, 418, 6, 58, "Message"],
Cell[1519630, 40044, 419, 6, 58, "Message"],
Cell[1520052, 40052, 169, 3, 23, "Message"],
Cell[1520224, 40057, 354, 5, 71, "Message"],
Cell[1520581, 40064, 227, 4, 39, "Message"],
Cell[1520811, 40070, 354, 5, 71, "Message"],
Cell[1521168, 40077, 170, 3, 23, "Message"],
Cell[1521341, 40082, 227, 4, 39, "Message"],
Cell[1521571, 40088, 168, 3, 23, "Message"],
Cell[1521742, 40093, 25794, 580, 186, 4509, 313, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1547539, 40675, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1547706, 40683, 73, 1, 27, "Input"],
Cell[1547782, 40686, 129, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1547948, 40694, 125, 3, 27, "Input"],
Cell[1548076, 40699, 166, 3, 39, "Message"],
Cell[1548245, 40704, 23526, 555, 186, 4484, 315, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1571774, 41261, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1571941, 41269, 126, 3, 24, "Input"],
Cell[1572070, 41274, 166, 3, 39, "Message"],
Cell[1572239, 41279, 23004, 535, 186, 4295, 300, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1595246, 41816, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1595413, 41824, 127, 3, 24, "Input"],
Cell[1595543, 41829, 166, 3, 23, "Message"],
Cell[1595712, 41834, 20872, 493, 186, 3938, 279, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1616587, 42329, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[1616766, 42338, 35, 0, 29, "Subsection"],
Cell[1616804, 42340, 308, 6, 75, "Input"],
Cell[1617115, 42348, 277, 5, 59, "Input"],
Cell[1617395, 42355, 49, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1617469, 42360, 57, 1, 27, "Input"],
Cell[1617529, 42363, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1617619, 42369, 42, 1, 27, "Input"],
Cell[1617664, 42372, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1617754, 42378, 57, 1, 27, "Input"],
Cell[1617814, 42381, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1617904, 42387, 48, 1, 27, "Input"],
Cell[1617955, 42390, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1618045, 42396, 42, 1, 27, "Input"],
Cell[1618090, 42399, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1618179, 42405, 57, 1, 27, "Input"],
Cell[1618239, 42408, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1618329, 42414, 49, 1, 27, "Input"],
Cell[1618381, 42417, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1618471, 42423, 42, 1, 27, "Input"],
Cell[1618516, 42426, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1618605, 42432, 52, 1, 27, "Input"],
Cell[1618660, 42435, 54, 1, 27, "Output"]
}, Open  ]],
Cell[1618729, 42439, 49, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1618803, 42444, 83, 1, 27, "Input"],
Cell[1618889, 42447, 53, 1, 27, "Output"]
}, Open  ]],
Cell[1618957, 42451, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1619023, 42456, 59, 1, 27, "Input"],
Cell[1619085, 42459, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1619175, 42465, 55, 1, 27, "Input"],
Cell[1619233, 42468, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1619323, 42474, 60, 1, 27, "Input"],
Cell[1619386, 42477, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1619475, 42483, 68, 1, 27, "Input"],
Cell[1619546, 42486, 84, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1619667, 42492, 45, 1, 27, "Input"],
Cell[1619715, 42495, 69, 1, 23, "Print",
  CellTags->"Info3275041079-1935256"],
Cell[1619787, 42498, 4430, 74, 961, "Print",
  CellTags->"Info3275041079-1935256"]
}, Closed]],

Cell[CellGroupData[{
Cell[1624254, 42577, 125, 2, 24, "Input"],
Cell[1624382, 42581, 1039, 15, 188, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1625458, 42601, 60, 1, 27, "Input"],
Cell[1625521, 42604, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1625610, 42610, 126, 2, 27, "Input"],
Cell[1625739, 42614, 1359, 20, 260, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1627135, 42639, 89, 1, 27, "Input"],
Cell[1627227, 42642, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1627316, 42648, 1386, 20, 496, "Input"],
Cell[1628705, 42670, 1389, 20, 260, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1630131, 42695, 120, 2, 27, "Input"],
Cell[1630254, 42699, 11856, 376, 186, 3799, 272, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1642113, 43077, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1642280, 43085, 146, 3, 27, "Input"],
Cell[1642429, 43090, 1355, 20, 260, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1643821, 43115, 65, 1, 27, "Input"],
Cell[1643889, 43118, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1643979, 43124, 1384, 20, 496, "Input"],
Cell[1645366, 43146, 1386, 20, 260, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1646789, 43171, 120, 2, 27, "Input"],
Cell[1646912, 43175, 11879, 376, 186, 3806, 272, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1658794, 43553, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1658961, 43561, 172, 4, 27, "Input"],
Cell[1659136, 43567, 1363, 20, 260, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1660536, 43592, 80, 1, 27, "Input"],
Cell[1660619, 43595, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1660708, 43601, 1391, 20, 496, "Input"],
Cell[1662102, 43623, 1393, 20, 260, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1663532, 43648, 120, 2, 27, "Input"],
Cell[1663655, 43652, 12161, 384, 186, 3966, 278, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1675819, 44038, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1675986, 44046, 65, 1, 27, "Input"],
Cell[1676054, 44049, 45, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1676136, 44055, 140, 3, 27, "Input"],
Cell[1676279, 44060, 1249, 23, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1677565, 44088, 61, 1, 27, "Input"],
Cell[1677629, 44091, 45, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1677711, 44097, 50, 1, 27, "Input"],
Cell[1677764, 44100, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1677854, 44106, 121, 3, 27, "Input"],
Cell[1677978, 44111, 16868, 465, 186, 4272, 305, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1694849, 44578, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1695016, 44586, 46, 1, 27, "Input"],
Cell[1695065, 44589, 105, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1695207, 44597, 69, 1, 27, "Input"],
Cell[1695279, 44600, 348, 7, 61, "Message"],
Cell[1695630, 44609, 412, 8, 58, "Message"],
Cell[1696045, 44619, 412, 8, 58, "Message"],
Cell[1696460, 44629, 221, 5, 23, "Message"]
}, Open  ]],
Cell[1696696, 44637, 56, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1696777, 44642, 112, 2, 27, "Input"],
Cell[1696892, 44646, 559, 10, 152, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1697488, 44661, 44, 1, 27, "Input"],
Cell[1697535, 44664, 377, 7, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1697949, 44676, 69, 1, 27, "Input"],
Cell[1698021, 44679, 471, 9, 100, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1698529, 44693, 71, 1, 27, "Input"],
Cell[1698603, 44696, 366, 7, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1699006, 44708, 122, 3, 27, "Input"],
Cell[1699131, 44713, 353, 7, 96, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1699521, 44725, 42, 1, 27, "Input"],
Cell[1699566, 44728, 163, 4, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1699766, 44737, 54, 1, 27, "Input"],
Cell[1699823, 44740, 534, 11, 167, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1700394, 44756, 133, 2, 27, "Input"],
Cell[1700530, 44760, 657, 12, 156, "Output"]
}, Open  ]],
Cell[1701202, 44775, 91, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1701318, 44780, 53, 1, 27, "Input"],
Cell[1701374, 44783, 164, 4, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1701575, 44792, 59, 1, 27, "Input"],
Cell[1701637, 44795, 129, 3, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1701803, 44803, 176, 3, 43, "Input"],
Cell[1701982, 44808, 17317, 437, 186, 3686, 263, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1719302, 45247, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1719499, 45255, 54, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1719578, 45260, 45, 1, 27, "Input"],
Cell[1719626, 45263, 18474, 473, 186, 4090, 290, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1738103, 45738, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1738300, 45746, 54, 1, 27, "Input"],
Cell[1738357, 45749, 69, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1738451, 45754, 117, 2, 27, "Input"],
Cell[1738571, 45758, 17418, 429, 186, 3660, 253, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1755992, 46189, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1756189, 46197, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[1756255, 46202, 88, 1, 27, "Input"],
Cell[1756346, 46205, 18191, 475, 186, 4249, 297, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1774540, 46682, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1774737, 46690, 43, 1, 27, "Input"],
Cell[1774783, 46693, 141, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[1774949, 46699, 129, 2, 27, "Input"],
Cell[1775081, 46703, 15761, 405, 186, 3523, 248, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1790845, 47110, 182, 5, 27, "Output"]
}, Open  ]],
Cell[1791042, 47118, 26, 0, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[1791105, 47123, 34, 0, 29, "Subsection"],
Cell[1791142, 47125, 718, 16, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1791885, 47145, 76, 1, 27, "Input"],
Cell[1791964, 47148, 67, 1, 27, "Output"]
}, Open  ]],
Cell[1792046, 47152, 293, 6, 59, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[1792364, 47162, 129, 2, 27, "Input"],
Cell[1792496, 47166, 166, 3, 23, "Message"],
Cell[1792665, 47171, 2008, 37, 139, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1794710, 47213, 1966, 36, 315, "Input"],
Cell[1796679, 47251, 1967, 36, 139, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1798683, 47292, 119, 2, 27, "Input"],
Cell[1798805, 47296, 11555, 358, 186, 3625, 256, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1810363, 47656, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1810530, 47664, 151, 3, 27, "Input"],
Cell[1810684, 47669, 166, 3, 23, "Message"],
Cell[1810853, 47674, 167, 3, 23, "Message"],
Cell[1811023, 47679, 167, 3, 23, "Message"],
Cell[1811193, 47684, 167, 3, 23, "Message"],
Cell[1811363, 47689, 166, 3, 23, "Message"],
Cell[1811532, 47694, 2009, 37, 139, "Output"]
}, Open  ]],
Cell[1813556, 47734, 242, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[1813823, 47743, 64, 1, 27, "Input"],
Cell[1813890, 47746, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1813980, 47752, 154, 3, 27, "Input"],
Cell[1814137, 47757, 354, 6, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1814528, 47768, 75, 1, 27, "Input"],
Cell[1814606, 47771, 53, 1, 27, "Output"]
}, Open  ]],
Cell[1814674, 47775, 461, 8, 91, "Input"],

Cell[CellGroupData[{
Cell[1815160, 47787, 1826, 34, 299, "Input"],
Cell[1816989, 47823, 1827, 34, 123, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1818853, 47862, 119, 2, 27, "Input"],
Cell[1818975, 47866, 11474, 355, 186, 3603, 253, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1830452, 48223, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1830619, 48231, 175, 4, 27, "Input"],
Cell[1830797, 48237, 354, 5, 71, "Message"],
Cell[1831154, 48244, 354, 5, 71, "Message"],
Cell[1831511, 48251, 166, 3, 23, "Message"],
Cell[1831680, 48256, 354, 5, 71, "Message"],
Cell[1832037, 48263, 170, 3, 23, "Message"],
Cell[1832210, 48268, 166, 3, 23, "Message"],
Cell[1832379, 48273, 2006, 37, 139, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1834422, 48315, 1964, 36, 315, "Input"],
Cell[1836389, 48353, 1965, 36, 139, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1838391, 48394, 74, 1, 27, "Input"],
Cell[1838468, 48397, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1838558, 48403, 119, 2, 27, "Input"],
Cell[1838680, 48407, 11464, 357, 186, 3642, 256, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1850147, 48766, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1850314, 48774, 139, 3, 27, "Input"],
Cell[1850456, 48779, 2013, 37, 139, "Output"]
}, Open  ]],
Cell[1852484, 48819, 244, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[1852753, 48828, 56, 1, 27, "Input"],
Cell[1852812, 48831, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1852902, 48837, 55, 1, 27, "Input"],
Cell[1852960, 48840, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1853050, 48846, 1819, 34, 299, "Input"],
Cell[1854872, 48882, 1822, 34, 123, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1856731, 48921, 69, 1, 27, "Input"],
Cell[1856803, 48924, 11905, 364, 186, 3687, 258, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1868711, 49290, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1868878, 49298, 112, 2, 27, "Input"],
Cell[1868993, 49302, 26472, 794, 259, 7065, 550, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1895468, 50098, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1895635, 50106, 65, 1, 27, "Input"],
Cell[1895703, 50109, 49, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1895789, 50115, 89, 1, 27, "Input"],
Cell[1895881, 50118, 16791, 519, 186, 5377, 374, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1912675, 50639, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1912842, 50647, 87, 1, 27, "Input"],
Cell[1912932, 50650, 10639, 319, 186, 3098, 221, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1923574, 50971, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1923741, 50979, 66, 1, 27, "Input"],
Cell[1923810, 50982, 49, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1923896, 50988, 152, 3, 27, "Input"],
Cell[1924051, 50993, 2011, 37, 139, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1926099, 51035, 1818, 34, 299, "Input"],
Cell[1927920, 51071, 1819, 34, 123, "Output"]
}, Open  ]],
Cell[1929754, 51108, 265, 4, 75, "Input"],

Cell[CellGroupData[{
Cell[1930044, 51116, 2013, 37, 331, "Input"],
Cell[1932060, 51155, 2015, 37, 139, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1934112, 51197, 69, 1, 27, "Input"],
Cell[1934184, 51200, 12097, 371, 186, 3729, 263, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1946284, 51573, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1946451, 51581, 113, 2, 27, "Input"],
Cell[1946567, 51585, 20651, 724, 186, 7098, 552, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1967221, 52311, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1967388, 52319, 61, 1, 27, "Input"],
Cell[1967452, 52322, 47, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1967536, 52328, 89, 1, 27, "Input"],
Cell[1967628, 52331, 16411, 516, 186, 5410, 376, "GraphicsData", \
"PostScript", "Graphics"],
Cell[1984042, 52849, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1984209, 52857, 61, 1, 27, "Input"],
Cell[1984273, 52860, 47, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1984357, 52866, 74, 1, 27, "Input"],
Cell[1984434, 52869, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1984524, 52875, 176, 4, 27, "Input"],
Cell[1984703, 52881, 2014, 37, 139, "Output"]
}, Open  ]],
Cell[1986732, 52921, 242, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[1986999, 52930, 83, 1, 27, "Input"],
Cell[1987085, 52933, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1987174, 52939, 74, 1, 27, "Input"],
Cell[1987251, 52942, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1987341, 52948, 1779, 33, 283, "Input"],
Cell[1989123, 52983, 1782, 33, 123, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1990942, 53021, 69, 1, 27, "Input"],
Cell[1991014, 53024, 11847, 362, 186, 3674, 257, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2002864, 53388, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2003031, 53396, 115, 2, 27, "Input"],
Cell[2003149, 53400, 22183, 741, 186, 7067, 550, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2025335, 54143, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2025502, 54151, 63, 1, 27, "Input"],
Cell[2025568, 54154, 49, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2025654, 54160, 89, 1, 27, "Input"],
Cell[2025746, 54163, 18312, 538, 186, 5379, 374, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2044061, 54703, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2044228, 54711, 63, 1, 27, "Input"],
Cell[2044294, 54714, 49, 1, 27, "Output"]
}, Open  ]],
Cell[2044358, 54718, 150, 3, 43, "Input"],

Cell[CellGroupData[{
Cell[2044533, 54725, 70, 1, 27, "Input"],
Cell[2044606, 54728, 12529, 385, 186, 3910, 274, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2057138, 55115, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2057305, 55123, 127, 2, 27, "Input"],
Cell[2057435, 55127, 166, 3, 39, "Message"],
Cell[2057604, 55132, 166, 3, 39, "Message"],
Cell[2057773, 55137, 166, 3, 39, "Message"],
Cell[2057942, 55142, 167, 3, 23, "Message"],
Cell[2058112, 55147, 167, 3, 39, "Message"],
Cell[2058282, 55152, 167, 3, 39, "Message"],
Cell[2058452, 55157, 167, 3, 39, "Message"],
Cell[2058622, 55162, 166, 3, 23, "Message"],
Cell[2058791, 55167, 1027, 15, 188, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2059855, 55187, 142, 3, 24, "Input"],
Cell[2060000, 55192, 1252, 23, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2061289, 55220, 62, 1, 27, "Input"],
Cell[2061354, 55223, 46, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2061437, 55229, 71, 1, 27, "Input"],
Cell[2061511, 55232, 16765, 436, 186, 3680, 270, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2078279, 55670, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2078446, 55678, 129, 2, 27, "Input"],
Cell[2078578, 55682, 198, 3, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2078813, 55690, 40, 1, 27, "Input"],
Cell[2078856, 55693, 196, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2079089, 55701, 131, 2, 27, "Input"],
Cell[2079223, 55705, 198, 3, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2079458, 55713, 40, 1, 27, "Input"],
Cell[2079501, 55716, 1009, 15, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2080547, 55736, 1054, 16, 171, "Input"],
Cell[2081604, 55754, 1008, 15, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2082649, 55774, 63, 1, 27, "Input"],
Cell[2082715, 55777, 46, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2082798, 55783, 62, 1, 27, "Input"],
Cell[2082863, 55786, 1261, 23, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2084161, 55814, 40, 1, 27, "Input"],
Cell[2084204, 55817, 196, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2084437, 55825, 1281, 22, 235, "Input"],
Cell[2085721, 55849, 1235, 22, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2086993, 55876, 60, 1, 27, "Input"],
Cell[2087056, 55879, 46, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2087139, 55885, 72, 1, 27, "Input"],
Cell[2087214, 55888, 17195, 450, 186, 3948, 282, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2104412, 56340, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2104579, 56348, 72, 1, 27, "Input"],
Cell[2104654, 56351, 17395, 450, 186, 3909, 279, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2122052, 56803, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2122219, 56811, 44, 1, 27, "Input"],
Cell[2122266, 56814, 19470, 518, 186, 4663, 331, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2141739, 57334, 130, 3, 27, "Output"]
}, Open  ]],
Cell[2141884, 57340, 337, 7, 59, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2142246, 57351, 144, 3, 27, "Input"],
Cell[2142393, 57356, 1254, 23, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2143684, 57384, 60, 1, 27, "Input"],
Cell[2143747, 57387, 46, 1, 27, "Output"]
}, Open  ]],
Cell[2143808, 57391, 70, 1, 27, "Input"],
Cell[2143881, 57394, 244, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[2144150, 57403, 146, 3, 27, "Input"],
Cell[2144299, 57408, 18818, 463, 186, 4068, 276, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2163120, 57873, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2163287, 57881, 75, 1, 27, "Input"],
Cell[2163365, 57884, 22728, 545, 186, 4487, 315, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2186096, 58431, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2186263, 58439, 146, 3, 27, "Input"],
Cell[2186412, 58444, 275, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2186724, 58454, 1305, 23, 235, "Input"],
Cell[2188032, 58479, 1255, 23, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2189324, 58507, 71, 1, 27, "Input"],
Cell[2189398, 58510, 16788, 438, 186, 3760, 273, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2206189, 58950, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2206356, 58958, 44, 1, 27, "Input"],
Cell[2206403, 58961, 19929, 502, 186, 4228, 304, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2226335, 59465, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2226502, 59473, 53, 1, 27, "Input"],
Cell[2226558, 59476, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2226648, 59482, 140, 3, 27, "Input"],
Cell[2226791, 59487, 23119, 531, 186, 4199, 293, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2249913, 60020, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2250080, 60028, 77, 1, 27, "Input"],
Cell[2250160, 60031, 24855, 594, 186, 4817, 342, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2275018, 60627, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[2275197, 60636, 36, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[2275258, 60640, 32, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[2275315, 60644, 112, 2, 27, "Input"],
Cell[2275430, 60648, 507, 8, 152, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2275974, 60661, 44, 1, 27, "Input"],
Cell[2276021, 60664, 325, 5, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2276383, 60674, 69, 1, 27, "Input"],
Cell[2276455, 60677, 419, 7, 100, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2276911, 60689, 69, 1, 27, "Input"],
Cell[2276983, 60692, 314, 5, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2277334, 60702, 122, 3, 27, "Input"],
Cell[2277459, 60707, 353, 7, 96, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2277849, 60719, 42, 1, 27, "Input"],
Cell[2277894, 60722, 163, 4, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2278094, 60731, 54, 1, 27, "Input"],
Cell[2278151, 60734, 534, 11, 167, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2278722, 60750, 133, 2, 27, "Input"],
Cell[2278858, 60754, 657, 12, 168, "Output"]
}, Open  ]],
Cell[2279530, 60769, 91, 1, 27, "Input"],
Cell[2279624, 60772, 738, 14, 264, "Input"],
Cell[2280365, 60788, 594, 14, 91, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2280984, 60806, 59, 1, 27, "Input"],
Cell[2281046, 60809, 401, 6, 97, "Message"],
Cell[2281450, 60817, 56, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2281543, 60823, 59, 1, 27, "Input"],
Cell[2281605, 60826, 129, 3, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2281771, 60834, 176, 3, 43, "Input"],
Cell[2281950, 60839, 17317, 437, 186, 3686, 263, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2299270, 61278, 182, 5, 27, "Output"]
}, Open  ]],
Cell[2299467, 61286, 54, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2299546, 61291, 45, 1, 27, "Input"],
Cell[2299594, 61294, 18474, 473, 186, 4090, 290, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2318071, 61769, 182, 5, 27, "Output"]
}, Open  ]],
Cell[2318268, 61777, 54, 1, 27, "Input"],
Cell[2318325, 61780, 69, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2318419, 61785, 117, 2, 27, "Input"],
Cell[2318539, 61789, 17418, 429, 186, 3660, 253, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2335960, 62220, 182, 5, 27, "Output"]
}, Open  ]],
Cell[2336157, 62228, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2336223, 62233, 88, 1, 27, "Input"],
Cell[2336314, 62236, 18191, 475, 186, 4249, 297, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2354508, 62713, 182, 5, 27, "Output"]
}, Open  ]],
Cell[2354705, 62721, 43, 1, 27, "Input"],
Cell[2354751, 62724, 141, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[2354917, 62730, 129, 2, 27, "Input"],
Cell[2355049, 62734, 15761, 405, 186, 3523, 248, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2370813, 63141, 182, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2371032, 63151, 108, 2, 27, "Input"],
Cell[2371143, 63155, 530, 7, 143, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2371710, 63167, 91, 1, 27, "Input"],
Cell[2371804, 63170, 677, 10, 421, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2372518, 63185, 87, 1, 24, "Input"],
Cell[2372608, 63188, 93, 1, 54, "Output"]
}, Open  ]],
Cell[2372716, 63192, 328, 6, 75, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2373069, 63202, 57, 1, 27, "Input"],
Cell[2373129, 63205, 76, 1, 27, "Output"]
}, Open  ]],
Cell[2373220, 63209, 347, 7, 75, "Input",
  InitializationCell->True],
Cell[2373570, 63218, 250, 6, 59, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2373845, 63228, 48, 1, 27, "Input"],
Cell[2373896, 63231, 130, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2374063, 63238, 48, 1, 27, "Input"],
Cell[2374114, 63241, 131, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2374282, 63248, 128, 3, 27, "Input"],
Cell[2374413, 63253, 166, 3, 23, "Message"],
Cell[2374582, 63258, 166, 3, 23, "Message"],
Cell[2374751, 63263, 16320, 402, 186, 3424, 238, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2391074, 63667, 130, 3, 27, "Output"]
}, Open  ]],
Cell[2391219, 63673, 60, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2391304, 63678, 157, 3, 27, "Input"],
Cell[2391464, 63683, 166, 3, 23, "Message"],
Cell[2391633, 63688, 148, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2391818, 63695, 72, 1, 27, "Input"],
Cell[2391893, 63698, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2391983, 63704, 47, 1, 27, "Input"],
Cell[2392033, 63707, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2392123, 63713, 158, 3, 27, "Input"],
Cell[2392284, 63718, 166, 3, 23, "Message"],
Cell[2392453, 63723, 148, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2392638, 63730, 55, 1, 27, "Input"],
Cell[2392696, 63733, 54, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2392787, 63739, 178, 4, 27, "Input"],
Cell[2392968, 63745, 167, 3, 23, "Message"],
Cell[2393138, 63750, 131, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2393306, 63757, 287, 6, 75, "Input"],
Cell[2393596, 63765, 166, 3, 39, "Message"],
Cell[2393765, 63770, 166, 3, 39, "Message"],
Cell[2393934, 63775, 167, 3, 39, "Message"],
Cell[2394104, 63780, 167, 3, 39, "Message"],
Cell[2394274, 63785, 166, 3, 39, "Message"],
Cell[2394443, 63790, 167, 3, 23, "Message"],
Cell[2394613, 63795, 354, 5, 71, "Message"],
Cell[2394970, 63802, 167, 3, 39, "Message"],
Cell[2395140, 63807, 166, 3, 23, "Message"],
Cell[2395309, 63812, 1267, 22, 101, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2396613, 63839, 74, 1, 24, "Input"],
Cell[2396690, 63842, 17046, 395, 186, 3073, 218, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2413739, 64239, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2413906, 64247, 137, 3, 27, "Input"],
Cell[2414046, 64252, 16805, 451, 186, 4072, 289, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2430854, 64705, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2431021, 64713, 127, 3, 24, "Input"],
Cell[2431151, 64718, 166, 3, 39, "Message"],
Cell[2431320, 64723, 166, 3, 39, "Message"],
Cell[2431489, 64728, 166, 3, 39, "Message"],
Cell[2431658, 64733, 167, 3, 23, "Message"],
Cell[2431828, 64738, 42, 1, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2431907, 64744, 47, 1, 24, "Input"],
Cell[2431957, 64747, 131, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2432125, 64754, 49, 1, 27, "Input"],
Cell[2432177, 64757, 132, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2432346, 64764, 135, 3, 27, "Input"],
Cell[2432484, 64769, 19708, 478, 186, 3938, 279, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2452195, 65249, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2452362, 65257, 78, 1, 27, "Input"],
Cell[2452443, 65260, 24164, 555, 186, 4243, 305, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2476610, 65817, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2476777, 65825, 49, 1, 27, "Input"],
Cell[2476829, 65828, 166, 3, 23, "Message"],
Cell[2476998, 65833, 131, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2477166, 65840, 92, 1, 27, "Input"],
Cell[2477261, 65843, 354, 5, 71, "Message"],
Cell[2477618, 65850, 228, 4, 39, "Message"],
Cell[2477849, 65856, 111, 2, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2477997, 65863, 68, 1, 24, "Input"],
Cell[2478068, 65866, 58, 1, 27, "Output"]
}, Closed]],
Cell[2478141, 65870, 41, 1, 24, "Input"],

Cell[CellGroupData[{
Cell[2478207, 65875, 91, 1, 27, "Input"],
Cell[2478301, 65878, 354, 5, 71, "Message"],
Cell[2478658, 65885, 228, 4, 39, "Message"],
Cell[2478889, 65891, 103, 2, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2479029, 65898, 70, 1, 24, "Input"],
Cell[2479102, 65901, 78, 1, 27, "Output"]
}, Open  ]],
Cell[2479195, 65905, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2479261, 65910, 92, 1, 27, "Input"],
Cell[2479356, 65913, 354, 5, 71, "Message"],
Cell[2479713, 65920, 227, 4, 39, "Message"],
Cell[2479943, 65926, 89, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2480069, 65932, 68, 1, 27, "Input"],
Cell[2480140, 65935, 112, 2, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[2480301, 65943, 29, 0, 28, "Subsubsection"],
Cell[2480333, 65945, 423, 8, 91, "Input"],

Cell[CellGroupData[{
Cell[2480781, 65957, 72, 1, 27, "Input"],
Cell[2480856, 65960, 354, 5, 71, "Message"],
Cell[2481213, 65967, 354, 5, 71, "Message"],
Cell[2481570, 65974, 354, 5, 71, "Message"],
Cell[2481927, 65981, 170, 3, 23, "Message"],
Cell[2482100, 65986, 117, 2, 29, "Output"]
}, Open  ]],
Cell[2482232, 65991, 205, 4, 43, "Input"],
Cell[2482440, 65997, 307, 6, 59, "Input"],

Cell[CellGroupData[{
Cell[2482772, 66007, 188, 4, 43, "Input"],
Cell[2482963, 66013, 185, 3, 39, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2483185, 66021, 61, 1, 27, "Input"],
Cell[2483249, 66024, 112, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2483398, 66031, 188, 4, 43, "Input"],
Cell[2483589, 66037, 191, 3, 39, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2483817, 66045, 61, 1, 27, "Input"],
Cell[2483881, 66048, 112, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2484030, 66055, 142, 3, 27, "Input"],
Cell[2484175, 66060, 19370, 475, 186, 3931, 280, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2503548, 66537, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2503715, 66545, 142, 3, 27, "Input"],
Cell[2503860, 66550, 20145, 506, 186, 4270, 306, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2524008, 67058, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2524175, 67066, 140, 3, 27, "Input"],
Cell[2524318, 67071, 19279, 482, 186, 4025, 289, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2543600, 67555, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2543767, 67563, 130, 2, 27, "Input"],
Cell[2543900, 67567, 106, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2544043, 67574, 84, 1, 27, "Input"],
Cell[2544130, 67577, 112, 2, 29, "Output"]
}, Open  ]],
Cell[2544257, 67582, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2544323, 67587, 146, 3, 27, "Input"],
Cell[2544472, 67592, 354, 5, 71, "Message"],
Cell[2544829, 67599, 354, 5, 71, "Message"],
Cell[2545186, 67606, 354, 5, 71, "Message"],
Cell[2545543, 67613, 170, 3, 23, "Message"],
Cell[2545716, 67618, 26501, 616, 186, 4962, 346, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2572220, 68236, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2572387, 68244, 63, 1, 27, "Input"],
Cell[2572453, 68247, 23659, 638, 186, 5822, 413, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2596115, 68887, 130, 3, 27, "Output"]
}, Open  ]],
Cell[2596260, 68893, 313, 6, 75, "Input"],

Cell[CellGroupData[{
Cell[2596598, 68903, 55, 1, 27, "Input"],
Cell[2596656, 68906, 129, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2596822, 68913, 43, 1, 27, "Input"],
Cell[2596868, 68916, 67, 1, 23, "Print",
  CellTags->"Info3274767031-9777815"],
Cell[2596938, 68919, 640, 16, 52, "Print",
  CellTags->"Info3274767031-9777815"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2597615, 68940, 48, 1, 27, "Input"],
Cell[2597666, 68943, 130, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2597833, 68950, 88, 1, 27, "Input"],
Cell[2597924, 68953, 166, 3, 23, "Message"],
Cell[2598093, 68958, 354, 5, 71, "Message"],
Cell[2598450, 68965, 354, 5, 71, "Message"],
Cell[2598807, 68972, 354, 5, 71, "Message"],
Cell[2599164, 68979, 170, 3, 23, "Message"],
Cell[2599337, 68984, 971, 16, 91, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2600345, 69005, 984, 16, 168, "Input"],
Cell[2601332, 69023, 959, 15, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2602328, 69043, 69, 1, 27, "Input"],
Cell[2602400, 69046, 15961, 418, 186, 3788, 263, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2618364, 69466, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2618531, 69474, 69, 1, 27, "Input"],
Cell[2618603, 69477, 21799, 712, 186, 6405, 517, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2640405, 70191, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2640572, 70199, 65, 1, 27, "Input"],
Cell[2640640, 70202, 51, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2640728, 70208, 49, 1, 27, "Input"],
Cell[2640780, 70211, 17963, 483, 186, 4457, 312, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2658746, 70696, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2658913, 70704, 65, 1, 27, "Input"],
Cell[2658981, 70707, 51, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2659069, 70713, 55, 1, 27, "Input"],
Cell[2659127, 70716, 110, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2659274, 70723, 104, 2, 27, "Input"],
Cell[2659381, 70727, 395, 6, 87, "Message"],
Cell[2659779, 70735, 395, 6, 87, "Message"],
Cell[2660177, 70743, 395, 6, 87, "Message"],
Cell[2660575, 70751, 169, 3, 23, "Message"],
Cell[2660747, 70756, 966, 16, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2661750, 70777, 1017, 17, 187, "Input"],
Cell[2662770, 70796, 992, 16, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2663799, 70817, 89, 1, 27, "Input"],
Cell[2663891, 70820, 19788, 639, 186, 5678, 460, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2683682, 71461, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2683849, 71469, 65, 1, 27, "Input"],
Cell[2683917, 71472, 51, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2684005, 71478, 105, 2, 27, "Input"],
Cell[2684113, 71482, 354, 5, 71, "Message"],
Cell[2684470, 71489, 354, 5, 71, "Message"],
Cell[2684827, 71496, 354, 5, 71, "Message"],
Cell[2685184, 71503, 170, 3, 23, "Message"],
Cell[2685357, 71508, 259, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2685653, 71518, 67, 1, 27, "Input"],
Cell[2685723, 71521, 51, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2685811, 71527, 47, 1, 27, "Input"],
Cell[2685861, 71530, 293, 4, 78, "Message"],
Cell[2686157, 71536, 354, 5, 71, "Message"],
Cell[2686514, 71543, 395, 6, 87, "Message"],
Cell[2686912, 71551, 293, 4, 78, "Message"],
Cell[2687208, 71557, 354, 5, 71, "Message"],
Cell[2687565, 71564, 395, 6, 87, "Message"],
Cell[2687963, 71572, 595, 9, 96, "Message"],
Cell[2688561, 71583, 169, 3, 23, "Message"],
Cell[2688733, 71588, 354, 5, 71, "Message"],
Cell[2689090, 71595, 170, 3, 23, "Message"],
Cell[2689263, 71600, 395, 6, 87, "Message"],
Cell[2689661, 71608, 169, 3, 23, "Message"],
Cell[2689833, 71613, 130, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2690000, 71620, 89, 1, 27, "Input"],
Cell[2690092, 71623, 354, 5, 71, "Message"],
Cell[2690449, 71630, 354, 5, 71, "Message"],
Cell[2690806, 71637, 354, 5, 71, "Message"],
Cell[2691163, 71644, 170, 3, 23, "Message"],
Cell[2691336, 71649, 1795, 27, 171, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2693168, 71681, 69, 1, 27, "Input"],
Cell[2693240, 71684, 24445, 563, 186, 4395, 311, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2717688, 72249, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2717855, 72257, 66, 1, 27, "Input"],
Cell[2717924, 72260, 49, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2718010, 72266, 56, 1, 27, "Input"],
Cell[2718069, 72269, 55, 1, 27, "Output"]
}, Open  ]],
Cell[2718139, 72273, 44, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2718208, 72278, 143, 3, 27, "Input"],
Cell[2718354, 72283, 20514, 533, 186, 4741, 334, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2738871, 72818, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2739038, 72826, 63, 1, 27, "Input"],
Cell[2739104, 72829, 18587, 516, 186, 4820, 342, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2757694, 73347, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2757861, 73355, 56, 1, 27, "Input"],
Cell[2757920, 73358, 54, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2758011, 73364, 143, 3, 27, "Input"],
Cell[2758157, 73369, 20488, 501, 186, 4258, 296, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2778648, 73872, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2778815, 73880, 63, 1, 27, "Input"],
Cell[2778881, 73883, 18705, 496, 186, 4461, 316, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2797589, 74381, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2797756, 74389, 140, 3, 27, "Input"],
Cell[2797899, 74394, 16895, 408, 186, 3443, 237, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2814797, 74804, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2814964, 74812, 71, 1, 27, "Input"],
Cell[2815038, 74815, 14304, 355, 186, 2994, 211, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2829345, 75172, 130, 3, 27, "Output"]
}, Open  ]],
Cell[2829490, 75178, 41, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2829556, 75183, 171, 3, 43, "Input"],
Cell[2829730, 75188, 354, 5, 71, "Message"],
Cell[2830087, 75195, 354, 5, 71, "Message"],
Cell[2830444, 75202, 354, 5, 71, "Message"],
Cell[2830801, 75209, 170, 3, 23, "Message"],
Cell[2830974, 75214, 14603, 394, 186, 3576, 253, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2845580, 75610, 130, 3, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[2845771, 75620, 28, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[2845824, 75624, 59, 1, 27, "Input"],
Cell[2845886, 75627, 58, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2845981, 75633, 80, 1, 27, "Input"],
Cell[2846064, 75636, 15691, 383, 186, 3219, 225, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2861758, 76021, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2861925, 76029, 103, 2, 27, "Input"],
Cell[2862031, 76033, 5980, 83, 541, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[2868048, 76121, 88, 1, 24, "Input"],
Cell[2868139, 76124, 20050, 522, 186, 4616, 327, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2888192, 76648, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2888359, 76656, 63, 1, 27, "Input"],
Cell[2888425, 76659, 46, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2888508, 76665, 80, 1, 27, "Input"],
Cell[2888591, 76668, 16786, 432, 186, 3784, 267, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2905380, 77102, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2905547, 77110, 64, 1, 27, "Input"],
Cell[2905614, 77113, 59, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2905710, 77119, 72, 1, 27, "Input"],
Cell[2905785, 77122, 59, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2905881, 77128, 62, 1, 27, "Input"],
Cell[2905946, 77131, 111, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2906094, 77138, 68, 1, 27, "Input"],
Cell[2906165, 77141, 111, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2906313, 77148, 67, 1, 27, "Input"],
Cell[2906383, 77151, 80, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2906500, 77157, 76, 1, 27, "Input"],
Cell[2906579, 77160, 80, 1, 27, "Output"]
}, Open  ]],
Cell[2906674, 77164, 280, 5, 59, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2906979, 77173, 54, 1, 27, "Input"],
Cell[2907036, 77176, 131, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2907204, 77183, 48, 1, 27, "Input"],
Cell[2907255, 77186, 130, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2907422, 77193, 57, 1, 27, "Input"],
Cell[2907482, 77196, 131, 2, 29, "Output"]
}, Open  ]],
Cell[2907628, 77201, 56, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2907709, 77206, 102, 2, 27, "Input"],
Cell[2907814, 77210, 131, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2907982, 77217, 165, 2, 43, "Input"],
Cell[2908150, 77221, 132, 2, 29, "Output"]
}, Open  ]],
Cell[2908297, 77226, 41, 1, 27, "Input"],
Cell[2908341, 77229, 176, 4, 27, "Input",
  InitializationCell->True],
Cell[2908520, 77235, 264, 5, 59, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2908809, 77244, 58, 1, 27, "Input"],
Cell[2908870, 77247, 128, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2909035, 77254, 225, 4, 43, "Input"],
Cell[2909263, 77260, 395, 6, 87, "Message"],
Cell[2909661, 77268, 395, 6, 87, "Message"],
Cell[2910059, 77276, 18512, 526, 186, 5026, 355, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2928574, 77804, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2928741, 77812, 140, 3, 27, "Input"],
Cell[2928884, 77817, 395, 6, 87, "Message"],
Cell[2929282, 77825, 17835, 524, 186, 5089, 362, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2947120, 78351, 130, 3, 27, "Output"]
}, Open  ]],
Cell[2947265, 78357, 211, 5, 43, "Input",
  InitializationCell->True],
Cell[2947479, 78364, 211, 5, 43, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2947715, 78373, 56, 1, 27, "Input"],
Cell[2947774, 78376, 130, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2947941, 78383, 58, 1, 27, "Input"],
Cell[2948002, 78386, 128, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2948167, 78393, 60, 1, 27, "Input"],
Cell[2948230, 78396, 132, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2948399, 78403, 65, 1, 27, "Input"],
Cell[2948467, 78406, 132, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2948636, 78413, 39, 1, 27, "Input"],
Cell[2948678, 78416, 132, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2948847, 78423, 135, 3, 27, "Input"],
Cell[2948985, 78428, 16517, 389, 186, 3103, 219, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2965505, 78819, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2965672, 78827, 135, 3, 27, "Input"],
Cell[2965810, 78832, 16448, 421, 186, 3620, 258, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2982261, 79255, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2982428, 79263, 135, 3, 27, "Input"],
Cell[2982566, 79268, 16220, 390, 186, 3229, 225, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2998789, 79660, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2998956, 79668, 135, 3, 27, "Input"],
Cell[2999094, 79673, 17274, 438, 186, 3790, 267, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3016371, 80113, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3016538, 80121, 54, 1, 27, "Input"],
Cell[3016595, 80124, 15428, 416, 186, 3857, 269, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3032026, 80542, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3032171, 80548, 50, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[3032246, 80553, 55, 1, 27, "Input"],
Cell[3032304, 80556, 15989, 438, 186, 4141, 287, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3048296, 80996, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3048463, 81004, 153, 3, 27, "Input"],
Cell[3048619, 81009, 18034, 319, 1343, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3066690, 81333, 180, 4, 24, "Input"],
Cell[3066873, 81339, 18034, 318, 1343, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3084944, 81662, 65, 1, 24, "Input"],
Cell[3085012, 81665, 45, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3085094, 81671, 153, 3, 27, "Input"],
Cell[3085250, 81676, 16214, 318, 1433, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3101501, 81999, 180, 4, 24, "Input"],
Cell[3101684, 82005, 15956, 317, 1433, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3117677, 82327, 66, 1, 24, "Input"],
Cell[3117746, 82330, 46, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3117829, 82336, 72, 1, 27, "Input"],
Cell[3117904, 82339, 20628, 580, 186, 5694, 391, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3138535, 82921, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3138702, 82929, 72, 1, 27, "Input"],
Cell[3138777, 82932, 19578, 561, 186, 5664, 385, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3158358, 83495, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3158503, 83501, 63, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[3158591, 83506, 75, 1, 27, "Input"],
Cell[3158669, 83509, 20096, 608, 186, 6284, 433, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3178768, 84119, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3178913, 84125, 75, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[3179013, 84130, 69, 1, 27, "Input"],
Cell[3179085, 84133, 20071, 585, 186, 5903, 406, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3199159, 84720, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3199326, 84728, 40, 1, 27, "Input"],
Cell[3199369, 84731, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3199459, 84737, 377, 6, 75, "Input"],
Cell[3199839, 84745, 395, 6, 87, "Message"],
Cell[3200237, 84753, 395, 6, 87, "Message"],
Cell[3200635, 84761, 395, 6, 87, "Message"],
Cell[3201033, 84769, 169, 3, 23, "Message"],
Cell[3201205, 84774, 20423, 783, 186, 8139, 627, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3221631, 85559, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3221798, 85567, 63, 1, 27, "Input"],
Cell[3221864, 85570, 49, 1, 27, "Output"]
}, Open  ]],
Cell[3221928, 85574, 52, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[3222005, 85579, 441, 8, 91, "Input"],
Cell[3222449, 85589, 395, 6, 87, "Message"],
Cell[3222847, 85597, 395, 6, 87, "Message"],
Cell[3223245, 85605, 395, 6, 87, "Message"],
Cell[3223643, 85613, 169, 3, 23, "Message"],
Cell[3223815, 85618, 18523, 737, 186, 7617, 598, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3242341, 86357, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3242508, 86365, 63, 1, 27, "Input"],
Cell[3242574, 86368, 19287, 791, 186, 8230, 650, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3261864, 87161, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3262031, 87169, 64, 1, 27, "Input"],
Cell[3262098, 87172, 50, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3262197, 87179, 43, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[3262265, 87183, 64, 1, 27, "Input"],
Cell[3262332, 87186, 69, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3262438, 87192, 72, 1, 27, "Input"],
Cell[3262513, 87195, 72, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3262622, 87201, 85, 1, 27, "Input"],
Cell[3262710, 87204, 74, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3262821, 87210, 86, 1, 27, "Input"],
Cell[3262910, 87213, 77, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3263024, 87219, 61, 1, 27, "Input"],
Cell[3263088, 87222, 65, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3263190, 87228, 87, 1, 27, "Input"],
Cell[3263280, 87231, 59, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3263376, 87237, 87, 1, 27, "Input"],
Cell[3263466, 87240, 56, 1, 44, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3263571, 87247, 37, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[3263633, 87251, 52, 1, 27, "Input"],
Cell[3263688, 87254, 76, 1, 23, "Print",
  CellTags->"Info3280046340-8820928"],
Cell[3263767, 87257, 1430, 28, 99, "Print",
  CellTags->"Info3280046340-8820928"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3265234, 87290, 56, 1, 27, "Input"],
Cell[3265293, 87293, 110, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3265440, 87300, 56, 1, 27, "Input"],
Cell[3265499, 87303, 128, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3265664, 87310, 55, 1, 27, "Input"],
Cell[3265722, 87313, 110, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3265869, 87320, 55, 1, 27, "Input"],
Cell[3265927, 87323, 126, 2, 27, "Output"]
}, Open  ]],
Cell[3266068, 87328, 201, 3, 43, "Input"],

Cell[CellGroupData[{
Cell[3266294, 87335, 130, 2, 27, "Input"],
Cell[3266427, 87339, 13372, 444, 186, 4693, 332, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3279802, 87785, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3279947, 87791, 65, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[3280037, 87796, 135, 3, 27, "Input"],
Cell[3280175, 87801, 13818, 413, 186, 4132, 289, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3293996, 88216, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3294163, 88224, 162, 2, 27, "Input"],
Cell[3294328, 88228, 14397, 423, 186, 4164, 292, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3308728, 88653, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3308895, 88661, 162, 2, 24, "Input"],
Cell[3309060, 88665, 14361, 433, 186, 4323, 305, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3323424, 89100, 130, 3, 27, "Output"]
}, Closed]],
Cell[3323569, 89106, 120, 3, 24, "Input"],
Cell[3323692, 89111, 167, 3, 43, "Input"],

Cell[CellGroupData[{
Cell[3323884, 89118, 60, 1, 27, "Input"],
Cell[3323947, 89121, 93, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3324077, 89127, 85, 1, 27, "Input"],
Cell[3324165, 89130, 12539, 410, 186, 4154, 302, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3336707, 89542, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3336874, 89550, 83, 1, 27, "Input"],
Cell[3336960, 89553, 14762, 460, 186, 4737, 332, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3351725, 90015, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3351892, 90023, 85, 1, 27, "Input"],
Cell[3351980, 90026, 13940, 436, 186, 4444, 314, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3365923, 90464, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3366090, 90472, 85, 1, 27, "Input"],
Cell[3366178, 90475, 13689, 415, 186, 4177, 293, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3379870, 90892, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3380037, 90900, 82, 1, 27, "Input"],
Cell[3380122, 90903, 13700, 415, 186, 4189, 293, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3393825, 91320, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3393992, 91328, 46, 1, 27, "Input"],
Cell[3394041, 91331, 14977, 492, 186, 5203, 367, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3409021, 91825, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3409166, 91831, 154, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[3409345, 91838, 84, 1, 27, "Input"],
Cell[3409432, 91841, 13700, 415, 186, 4189, 293, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3423135, 92258, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3423280, 92264, 69, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[3423374, 92269, 75, 1, 27, "Input"],
Cell[3423452, 92272, 52, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3423541, 92278, 255, 4, 59, "Input"],
Cell[3423799, 92284, 116, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3423952, 92291, 124, 3, 27, "Input"],
Cell[3424079, 92296, 15663, 439, 186, 4065, 291, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3439745, 92737, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3439912, 92745, 124, 3, 27, "Input"],
Cell[3440039, 92750, 14575, 419, 186, 3888, 283, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3454617, 93171, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3454784, 93179, 284, 5, 59, "Input"],
Cell[3455071, 93186, 184, 3, 39, "Message"]
}, Open  ]],
Cell[3455270, 93192, 157, 3, 27, "Input"],
Cell[3455430, 93197, 158, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[3455613, 93204, 29, 0, 42, "Subsubsection"],
Cell[3455645, 93206, 297, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3455967, 93215, 49, 1, 27, "Input"],
Cell[3456019, 93218, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3456169, 93225, 51, 1, 27, "Input"],
Cell[3456223, 93228, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3456373, 93235, 51, 1, 27, "Input"],
Cell[3456427, 93238, 113, 2, 27, "Output"]
}, Open  ]],
Cell[3456555, 93243, 283, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3456863, 93252, 49, 1, 27, "Input"],
Cell[3456915, 93255, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3457065, 93262, 83, 1, 27, "Input"],
Cell[3457151, 93265, 14835, 482, 186, 5061, 357, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3471989, 93749, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3472134, 93755, 282, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3472441, 93764, 51, 1, 27, "Input"],
Cell[3472495, 93767, 167, 3, 23, "Message"],
Cell[3472665, 93772, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3472816, 93779, 107, 2, 27, "Input"],
Cell[3472926, 93783, 102, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3473065, 93790, 103, 2, 27, "Input"],
Cell[3473171, 93794, 12890, 399, 186, 3953, 284, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3486064, 94195, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3486231, 94203, 104, 2, 27, "Input"],
Cell[3486338, 94207, 11520, 356, 186, 3502, 252, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3497861, 94565, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3498028, 94573, 50, 1, 27, "Input"],
Cell[3498081, 94576, 147, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3498265, 94583, 123, 2, 27, "Input"],
Cell[3498391, 94587, 11577, 344, 186, 3357, 238, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3509971, 94933, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3510138, 94941, 51, 1, 27, "Input"],
Cell[3510192, 94944, 15735, 533, 186, 5940, 408, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3525930, 95479, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3526097, 95487, 126, 2, 27, "Input"],
Cell[3526226, 95491, 13040, 388, 186, 3778, 269, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3539269, 95881, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3539436, 95889, 47, 1, 27, "Input"],
Cell[3539486, 95892, 16131, 537, 186, 5923, 406, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3555620, 96431, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3555787, 96439, 84, 1, 27, "Input"],
Cell[3555874, 96442, 12615, 384, 186, 3816, 271, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3568492, 96828, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3568659, 96836, 50, 1, 27, "Input"],
Cell[3568712, 96839, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3568863, 96846, 54, 1, 27, "Input"],
Cell[3568920, 96849, 117, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3569074, 96856, 51, 1, 27, "Input"],
Cell[3569128, 96859, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3569231, 96865, 80, 1, 27, "Input"],
Cell[3569314, 96868, 78, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3569429, 96874, 138, 3, 27, "Input"],
Cell[3569570, 96879, 105, 2, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3569724, 96887, 29, 0, 28, "Subsubsection"],
Cell[3569756, 96889, 290, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3570071, 96898, 51, 1, 27, "Input"],
Cell[3570125, 96901, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3570275, 96908, 51, 1, 27, "Input"],
Cell[3570329, 96911, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3570480, 96918, 52, 1, 27, "Input"],
Cell[3570535, 96921, 111, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3570683, 96928, 54, 1, 27, "Input"],
Cell[3570740, 96931, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3570891, 96938, 83, 1, 27, "Input"],
Cell[3570977, 96941, 14835, 482, 186, 5061, 357, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3585815, 97425, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3585960, 97431, 282, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3586267, 97440, 51, 1, 27, "Input"],
Cell[3586321, 97443, 167, 3, 23, "Message"],
Cell[3586491, 97448, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3586642, 97455, 107, 2, 27, "Input"],
Cell[3586752, 97459, 102, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3586891, 97466, 103, 2, 27, "Input"],
Cell[3586997, 97470, 12890, 399, 186, 3953, 284, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3599890, 97871, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3600057, 97879, 104, 2, 27, "Input"],
Cell[3600164, 97883, 11520, 356, 186, 3502, 252, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3611687, 98241, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3611854, 98249, 50, 1, 27, "Input"],
Cell[3611907, 98252, 147, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3612091, 98259, 123, 2, 27, "Input"],
Cell[3612217, 98263, 11577, 344, 186, 3357, 238, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3623797, 98609, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3623964, 98617, 51, 1, 27, "Input"],
Cell[3624018, 98620, 15735, 533, 186, 5940, 408, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3639756, 99155, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3639923, 99163, 126, 2, 27, "Input"],
Cell[3640052, 99167, 13040, 388, 186, 3778, 269, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3653095, 99557, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3653262, 99565, 47, 1, 27, "Input"],
Cell[3653312, 99568, 16131, 537, 186, 5923, 406, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3669446, 100107, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3669613, 100115, 84, 1, 27, "Input"],
Cell[3669700, 100118, 12615, 384, 186, 3816, 271, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3682318, 100504, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3682485, 100512, 50, 1, 27, "Input"],
Cell[3682538, 100515, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3682689, 100522, 54, 1, 27, "Input"],
Cell[3682746, 100525, 117, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3682900, 100532, 51, 1, 27, "Input"],
Cell[3682954, 100535, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3683057, 100541, 80, 1, 27, "Input"],
Cell[3683140, 100544, 78, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3683255, 100550, 138, 3, 27, "Input"],
Cell[3683396, 100555, 105, 2, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3683550, 100563, 29, 0, 28, "Subsubsection"],
Cell[3683582, 100565, 284, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3683891, 100574, 82, 1, 27, "Input"],
Cell[3683976, 100577, 11902, 379, 186, 3901, 276, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3695881, 100958, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3696026, 100964, 284, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3696335, 100973, 52, 1, 27, "Input"],
Cell[3696390, 100976, 111, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3696538, 100983, 50, 1, 27, "Input"],
Cell[3696591, 100986, 147, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3696775, 100993, 123, 2, 27, "Input"],
Cell[3696901, 100997, 11577, 344, 186, 3357, 238, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3708481, 101343, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3708648, 101351, 51, 1, 27, "Input"],
Cell[3708702, 101354, 15735, 533, 186, 5940, 408, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3724440, 101889, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3724607, 101897, 126, 2, 27, "Input"],
Cell[3724736, 101901, 13040, 388, 186, 3778, 269, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3737779, 102291, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3737946, 102299, 47, 1, 27, "Input"],
Cell[3737996, 102302, 16131, 537, 186, 5923, 406, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3754130, 102841, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3754297, 102849, 84, 1, 27, "Input"],
Cell[3754384, 102852, 12615, 384, 186, 3816, 271, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3767002, 103238, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3767169, 103246, 50, 1, 27, "Input"],
Cell[3767222, 103249, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3767373, 103256, 54, 1, 27, "Input"],
Cell[3767430, 103259, 117, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3767584, 103266, 51, 1, 27, "Input"],
Cell[3767638, 103269, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3767741, 103275, 80, 1, 27, "Input"],
Cell[3767824, 103278, 78, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3767939, 103284, 138, 3, 27, "Input"],
Cell[3768080, 103289, 105, 2, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3768234, 103297, 30, 0, 28, "Subsubsection"],
Cell[3768267, 103299, 284, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3768576, 103308, 53, 1, 27, "Input"],
Cell[3768632, 103311, 115, 2, 27, "Output"]
}, Open  ]],
Cell[3768762, 103316, 284, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3769071, 103325, 50, 1, 27, "Input"],
Cell[3769124, 103328, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3769274, 103335, 53, 1, 27, "Input"],
Cell[3769330, 103338, 167, 3, 23, "Message"],
Cell[3769500, 103343, 115, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3769652, 103350, 111, 2, 27, "Input"],
Cell[3769766, 103354, 93, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3769896, 103360, 51, 1, 27, "Input"],
Cell[3769950, 103363, 146, 2, 29, "Output"]
}, Open  ]],
Cell[3770111, 103368, 158, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[3770294, 103375, 51, 1, 27, "Input"],
Cell[3770348, 103378, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3770451, 103384, 103, 2, 27, "Input"],
Cell[3770557, 103388, 79, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3770673, 103394, 83, 1, 27, "Input"],
Cell[3770759, 103397, 167, 3, 23, "Message"],
Cell[3770929, 103402, 167, 3, 23, "Message"],
Cell[3771099, 103407, 167, 3, 23, "Message"],
Cell[3771269, 103412, 167, 3, 23, "Message"],
Cell[3771439, 103417, 17832, 564, 186, 6077, 414, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3789274, 103983, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3789441, 103991, 83, 1, 27, "Input"],
Cell[3789527, 103994, 14473, 455, 186, 4796, 331, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3804003, 104451, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3804170, 104459, 50, 1, 27, "Input"],
Cell[3804223, 104462, 147, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3804407, 104469, 123, 2, 27, "Input"],
Cell[3804533, 104473, 11577, 344, 186, 3357, 238, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3816113, 104819, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3816280, 104827, 51, 1, 27, "Input"],
Cell[3816334, 104830, 15735, 533, 186, 5940, 408, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3832072, 105365, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3832239, 105373, 126, 2, 27, "Input"],
Cell[3832368, 105377, 13040, 388, 186, 3778, 269, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3845411, 105767, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3845578, 105775, 47, 1, 27, "Input"],
Cell[3845628, 105778, 16131, 537, 186, 5923, 406, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3861762, 106317, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3861929, 106325, 84, 1, 27, "Input"],
Cell[3862016, 106328, 12615, 384, 186, 3816, 271, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3874634, 106714, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3874801, 106722, 50, 1, 27, "Input"],
Cell[3874854, 106725, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3875005, 106732, 54, 1, 27, "Input"],
Cell[3875062, 106735, 117, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3875216, 106742, 51, 1, 27, "Input"],
Cell[3875270, 106745, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3875373, 106751, 80, 1, 27, "Input"],
Cell[3875456, 106754, 78, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3875571, 106760, 138, 3, 27, "Input"],
Cell[3875712, 106765, 105, 2, 29, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3875866, 106773, 31, 0, 28, "Subsubsection"],
Cell[3875900, 106775, 285, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3876210, 106784, 84, 1, 27, "Input"],
Cell[3876297, 106787, 15717, 495, 186, 5133, 360, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3892017, 107284, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3892184, 107292, 53, 1, 27, "Input"],
Cell[3892240, 107295, 115, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3892392, 107302, 51, 1, 27, "Input"],
Cell[3892446, 107305, 144, 2, 29, "Output"]
}, Open  ]],
Cell[3892605, 107310, 161, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[3892791, 107317, 48, 1, 27, "Input"],
Cell[3892842, 107320, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3892945, 107326, 127, 2, 27, "Input"],
Cell[3893075, 107330, 12876, 399, 186, 4022, 285, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3905954, 107731, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3906121, 107739, 51, 1, 27, "Input"],
Cell[3906175, 107742, 15166, 498, 186, 5363, 372, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3921344, 108242, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3921489, 108248, 159, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[3921673, 108255, 52, 1, 27, "Input"],
Cell[3921728, 108258, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3921831, 108264, 127, 2, 27, "Input"],
Cell[3921961, 108268, 15021, 462, 186, 4642, 329, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3936985, 108732, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3937152, 108740, 47, 1, 27, "Input"],
Cell[3937202, 108743, 16985, 536, 186, 5627, 391, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3954190, 109281, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3954357, 109289, 115, 2, 27, "Input"],
Cell[3954475, 109293, 15577, 484, 186, 5105, 350, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3970055, 109779, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3970222, 109787, 51, 1, 27, "Input"],
Cell[3970276, 109790, 115, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3970428, 109797, 55, 1, 27, "Input"],
Cell[3970486, 109800, 113, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3970636, 109807, 52, 1, 27, "Input"],
Cell[3970691, 109810, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3970794, 109816, 52, 1, 27, "Input"],
Cell[3970849, 109819, 66, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3970964, 109826, 32, 0, 28, "Subsubsection"],
Cell[3970999, 109828, 288, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[3971312, 109837, 85, 1, 27, "Input"],
Cell[3971400, 109840, 167, 3, 23, "Message"],
Cell[3971570, 109845, 167, 3, 23, "Message"],
Cell[3971740, 109850, 167, 3, 23, "Message"],
Cell[3971910, 109855, 167, 3, 23, "Message"],
Cell[3972080, 109860, 19015, 575, 186, 5797, 407, "GraphicsData", \
"PostScript", "Graphics"],
Cell[3991098, 110437, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3991265, 110445, 104, 2, 27, "Input"],
Cell[3991372, 110449, 167, 3, 23, "Message"],
Cell[3991542, 110454, 167, 3, 23, "Message"],
Cell[3991712, 110459, 167, 3, 23, "Message"],
Cell[3991882, 110464, 167, 3, 23, "Message"],
Cell[3992052, 110469, 5498, 87, 491, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[3997587, 110561, 5428, 86, 888, "Input"],
Cell[4003018, 110649, 5429, 86, 491, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4008484, 110740, 69, 1, 24, "Input"],
Cell[4008556, 110743, 17478, 562, 186, 6087, 417, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4026037, 111307, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4026204, 111315, 54, 1, 27, "Input"],
Cell[4026261, 111318, 115, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4026413, 111325, 52, 1, 27, "Input"],
Cell[4026468, 111328, 66, 1, 27, "Output"]
}, Open  ]],
Cell[4026549, 111332, 161, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[4026735, 111339, 49, 1, 27, "Input"],
Cell[4026787, 111342, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4026890, 111348, 137, 2, 27, "Input"],
Cell[4027030, 111352, 167, 3, 23, "Message"],
Cell[4027200, 111357, 12578, 395, 186, 4194, 287, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4039781, 111754, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4039948, 111762, 81, 1, 27, "Input"],
Cell[4040032, 111765, 19461, 663, 186, 7290, 508, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4059496, 112430, 130, 3, 27, "Output"]
}, Open  ]],
Cell[4059641, 112436, 159, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[4059825, 112443, 52, 1, 27, "Input"],
Cell[4059880, 112446, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4059983, 112452, 128, 2, 27, "Input"],
Cell[4060114, 112456, 12147, 356, 186, 3372, 243, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4072264, 112814, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4072431, 112822, 47, 1, 27, "Input"],
Cell[4072481, 112825, 20656, 697, 186, 7743, 533, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4093140, 113524, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4093307, 113532, 108, 2, 27, "Input"],
Cell[4093418, 113536, 167, 3, 23, "Message"],
Cell[4093588, 113541, 167, 3, 23, "Message"],
Cell[4093758, 113546, 167, 3, 23, "Message"],
Cell[4093928, 113551, 167, 3, 23, "Message"],
Cell[4094098, 113556, 5965, 85, 563, "Output"]
}, Open  ]],
Cell[4100078, 113644, 5891, 84, 907, "Input"],

Cell[CellGroupData[{
Cell[4105994, 113732, 69, 1, 27, "Input"],
Cell[4106066, 113735, 20549, 603, 186, 6094, 420, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4126618, 114340, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[4126797, 114349, 27, 0, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[4126849, 114353, 48, 1, 27, "Input"],
Cell[4126900, 114356, 72, 1, 23, "Print",
  CellTags->"Info3280045044-9066215"],
Cell[4126975, 114359, 671, 17, 42, "Print",
  CellTags->"Info3280045044-9066215"]
}, Open  ]],
Cell[4127661, 114379, 179, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[4127865, 114387, 43, 1, 27, "Input"],
Cell[4127911, 114390, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4128001, 114396, 91, 1, 27, "Input"],
Cell[4128095, 114399, 167, 3, 23, "Message"],
Cell[4128265, 114404, 167, 3, 23, "Message"],
Cell[4128435, 114409, 16102, 569, 186, 6676, 448, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4144540, 114980, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4144707, 114988, 134, 2, 27, "Input"],
Cell[4144844, 114992, 167, 3, 23, "Message"],
Cell[4145014, 114997, 167, 3, 23, "Message"],
Cell[4145184, 115002, 15416, 525, 186, 5824, 402, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4160603, 115529, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4160770, 115537, 77, 1, 27, "Input"],
Cell[4160850, 115540, 26445, 1117, 186, 14234, 962, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4187298, 116659, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[4187477, 116668, 60, 0, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[4187562, 116672, 53, 1, 27, "Input"],
Cell[4187618, 116675, 77, 1, 23, "Print",
  CellTags->"Info3279609948-4320224"],
Cell[4187698, 116678, 624, 16, 41, "Print",
  CellTags->"Info3279609948-4320224"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4188359, 116699, 101, 2, 27, "Input"],
Cell[4188463, 116703, 82679, 1942, 4507, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4271179, 118650, 44, 1, 24, "Input"],
Cell[4271226, 118653, 57163, 1443, 2173, "Output"]
}, Closed]],
Cell[4328404, 120099, 104, 2, 24, "Input"],

Cell[CellGroupData[{
Cell[4328533, 120105, 105, 2, 27, "Input"],
Cell[4328641, 120109, 208, 3, 39, "Message"]
}, Open  ]],
Cell[4328864, 120115, 106, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[4328995, 120121, 138, 2, 24, "Input"],
Cell[4329136, 120125, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4329239, 120131, 84, 1, 27, "Input"],
Cell[4329326, 120134, 59, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4329422, 120140, 265, 5, 75, "Input"],
Cell[4329690, 120147, 77, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4329804, 120153, 328, 6, 75, "Input"],
Cell[4330135, 120161, 72, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4330244, 120167, 328, 6, 75, "Input"],
Cell[4330575, 120175, 77, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4330689, 120181, 189, 3, 43, "Input"],
Cell[4330881, 120186, 63, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4330981, 120192, 243, 4, 43, "Input"],
Cell[4331227, 120198, 57, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4331321, 120204, 243, 4, 43, "Input"],
Cell[4331567, 120210, 58, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4331662, 120216, 244, 5, 59, "Input"],
Cell[4331909, 120223, 66, 1, 27, "Output"]
}, Open  ]],
Cell[4331990, 120227, 69, 1, 27, "Input"],
Cell[4332062, 120230, 2011, 35, 427, "Input"],

Cell[CellGroupData[{
Cell[4334098, 120269, 71, 1, 27, "Input"],
Cell[4334172, 120272, 308, 6, 23, "Print"],
Cell[4334483, 120280, 288, 6, 23, "Print"],
Cell[4334774, 120288, 291, 6, 23, "Print"],
Cell[4335068, 120296, 269, 5, 23, "Print"],
Cell[4335340, 120303, 310, 6, 23, "Print"],
Cell[4335653, 120311, 287, 6, 23, "Print"],
Cell[4335943, 120319, 292, 6, 23, "Print"],
Cell[4336238, 120327, 273, 5, 23, "Print"],
Cell[4336514, 120334, 311, 6, 23, "Print"],
Cell[4336828, 120342, 289, 6, 23, "Print"],
Cell[4337120, 120350, 290, 6, 23, "Print"],
Cell[4337413, 120358, 269, 5, 23, "Print"],
Cell[4337685, 120365, 309, 6, 23, "Print"],
Cell[4337997, 120373, 287, 6, 23, "Print"],
Cell[4338287, 120381, 293, 6, 23, "Print"],
Cell[4338583, 120389, 274, 5, 23, "Print"],
Cell[4338860, 120396, 310, 6, 23, "Print"],
Cell[4339173, 120404, 288, 6, 23, "Print"],
Cell[4339464, 120412, 293, 6, 23, "Print"],
Cell[4339760, 120420, 267, 5, 23, "Print"],
Cell[4340030, 120427, 309, 6, 23, "Print"],
Cell[4340342, 120435, 286, 6, 23, "Print"],
Cell[4340631, 120443, 294, 6, 23, "Print"],
Cell[4340928, 120451, 273, 5, 23, "Print"],
Cell[4341204, 120458, 310, 6, 23, "Print"],
Cell[4341517, 120466, 288, 6, 23, "Print"],
Cell[4341808, 120474, 293, 6, 23, "Print"],
Cell[4342104, 120482, 267, 5, 23, "Print"],
Cell[4342374, 120489, 309, 6, 23, "Print"],
Cell[4342686, 120497, 286, 6, 23, "Print"],
Cell[4342975, 120505, 294, 6, 23, "Print"],
Cell[4343272, 120513, 273, 5, 23, "Print"],
Cell[4343548, 120520, 310, 6, 23, "Print"],
Cell[4343861, 120528, 288, 6, 23, "Print"],
Cell[4344152, 120536, 293, 6, 23, "Print"],
Cell[4344448, 120544, 267, 5, 23, "Print"],
Cell[4344718, 120551, 309, 6, 23, "Print"],
Cell[4345030, 120559, 286, 6, 23, "Print"],
Cell[4345319, 120567, 294, 6, 23, "Print"],
Cell[4345616, 120575, 273, 5, 23, "Print"],
Cell[4345892, 120582, 310, 6, 23, "Print"],
Cell[4346205, 120590, 288, 6, 23, "Print"],
Cell[4346496, 120598, 293, 6, 23, "Print"],
Cell[4346792, 120606, 267, 5, 23, "Print"],
Cell[4347062, 120613, 309, 6, 23, "Print"],
Cell[4347374, 120621, 286, 6, 23, "Print"],
Cell[4347663, 120629, 294, 6, 23, "Print"],
Cell[4347960, 120637, 273, 5, 23, "Print"],
Cell[4348236, 120644, 310, 6, 23, "Print"],
Cell[4348549, 120652, 288, 6, 23, "Print"],
Cell[4348840, 120660, 293, 6, 23, "Print"],
Cell[4349136, 120668, 267, 5, 23, "Print"],
Cell[4349406, 120675, 309, 6, 23, "Print"],
Cell[4349718, 120683, 286, 6, 23, "Print"],
Cell[4350007, 120691, 294, 6, 23, "Print"],
Cell[4350304, 120699, 273, 5, 23, "Print"],
Cell[4350580, 120706, 310, 6, 23, "Print"],
Cell[4350893, 120714, 288, 6, 23, "Print"],
Cell[4351184, 120722, 293, 6, 23, "Print"],
Cell[4351480, 120730, 267, 5, 23, "Print"],
Cell[4351750, 120737, 309, 6, 23, "Print"],
Cell[4352062, 120745, 286, 6, 23, "Print"],
Cell[4352351, 120753, 294, 6, 23, "Print"],
Cell[4352648, 120761, 273, 5, 23, "Print"],
Cell[4352924, 120768, 310, 6, 23, "Print"],
Cell[4353237, 120776, 288, 6, 23, "Print"],
Cell[4353528, 120784, 293, 6, 23, "Print"],
Cell[4353824, 120792, 267, 5, 23, "Print"],
Cell[4354094, 120799, 309, 6, 23, "Print"],
Cell[4354406, 120807, 286, 6, 23, "Print"],
Cell[4354695, 120815, 294, 6, 23, "Print"],
Cell[4354992, 120823, 273, 5, 23, "Print"],
Cell[4355268, 120830, 310, 6, 23, "Print"],
Cell[4355581, 120838, 288, 6, 23, "Print"],
Cell[4355872, 120846, 293, 6, 23, "Print"],
Cell[4356168, 120854, 267, 5, 23, "Print"],
Cell[4356438, 120861, 309, 6, 23, "Print"],
Cell[4356750, 120869, 286, 6, 23, "Print"],
Cell[4357039, 120877, 294, 6, 23, "Print"],
Cell[4357336, 120885, 273, 5, 23, "Print"],
Cell[4357612, 120892, 310, 6, 23, "Print"],
Cell[4357925, 120900, 288, 6, 23, "Print"],
Cell[4358216, 120908, 293, 6, 23, "Print"],
Cell[4358512, 120916, 267, 5, 23, "Print"],
Cell[4358782, 120923, 309, 6, 23, "Print"],
Cell[4359094, 120931, 286, 6, 23, "Print"],
Cell[4359383, 120939, 294, 6, 23, "Print"],
Cell[4359680, 120947, 273, 5, 23, "Print"],
Cell[4359956, 120954, 310, 6, 23, "Print"],
Cell[4360269, 120962, 288, 6, 23, "Print"],
Cell[4360560, 120970, 293, 6, 23, "Print"],
Cell[4360856, 120978, 267, 5, 23, "Print"],
Cell[4361126, 120985, 309, 6, 23, "Print"],
Cell[4361438, 120993, 286, 6, 23, "Print"],
Cell[4361727, 121001, 294, 6, 23, "Print"],
Cell[4362024, 121009, 273, 5, 23, "Print"],
Cell[4362300, 121016, 310, 6, 23, "Print"],
Cell[4362613, 121024, 288, 6, 23, "Print"],
Cell[4362904, 121032, 293, 6, 23, "Print"],
Cell[4363200, 121040, 267, 5, 23, "Print"],
Cell[4363470, 121047, 309, 6, 23, "Print"],
Cell[4363782, 121055, 286, 6, 23, "Print"],
Cell[4364071, 121063, 294, 6, 23, "Print"],
Cell[4364368, 121071, 273, 5, 23, "Print"],
Cell[4364644, 121078, 310, 6, 23, "Print"],
Cell[4364957, 121086, 288, 6, 23, "Print"],
Cell[4365248, 121094, 293, 6, 23, "Print"],
Cell[4365544, 121102, 267, 5, 23, "Print"],
Cell[4365814, 121109, 309, 6, 23, "Print"],
Cell[4366126, 121117, 286, 6, 23, "Print"],
Cell[4366415, 121125, 294, 6, 23, "Print"],
Cell[4366712, 121133, 273, 5, 23, "Print"],
Cell[4366988, 121140, 310, 6, 23, "Print"],
Cell[4367301, 121148, 288, 6, 23, "Print"],
Cell[4367592, 121156, 293, 6, 23, "Print"],
Cell[4367888, 121164, 267, 5, 23, "Print"],
Cell[4368158, 121171, 309, 6, 23, "Print"],
Cell[4368470, 121179, 286, 6, 23, "Print"],
Cell[4368759, 121187, 294, 6, 23, "Print"],
Cell[4369056, 121195, 273, 5, 23, "Print"],
Cell[4369332, 121202, 310, 6, 23, "Print"],
Cell[4369645, 121210, 288, 6, 23, "Print"],
Cell[4369936, 121218, 293, 6, 23, "Print"],
Cell[4370232, 121226, 267, 5, 23, "Print"],
Cell[4370502, 121233, 309, 6, 23, "Print"],
Cell[4370814, 121241, 286, 6, 23, "Print"],
Cell[4371103, 121249, 294, 6, 23, "Print"],
Cell[4371400, 121257, 273, 5, 23, "Print"],
Cell[4371676, 121264, 310, 6, 23, "Print"],
Cell[4371989, 121272, 288, 6, 23, "Print"],
Cell[4372280, 121280, 293, 6, 23, "Print"],
Cell[4372576, 121288, 267, 5, 23, "Print"],
Cell[4372846, 121295, 309, 6, 23, "Print"],
Cell[4373158, 121303, 286, 6, 23, "Print"],
Cell[4373447, 121311, 294, 6, 23, "Print"],
Cell[4373744, 121319, 273, 5, 23, "Print"],
Cell[4374020, 121326, 310, 6, 23, "Print"],
Cell[4374333, 121334, 288, 6, 23, "Print"],
Cell[4374624, 121342, 293, 6, 23, "Print"],
Cell[4374920, 121350, 267, 5, 23, "Print"],
Cell[4375190, 121357, 309, 6, 23, "Print"],
Cell[4375502, 121365, 286, 6, 23, "Print"],
Cell[4375791, 121373, 294, 6, 23, "Print"],
Cell[4376088, 121381, 273, 5, 23, "Print"],
Cell[4376364, 121388, 310, 6, 23, "Print"],
Cell[4376677, 121396, 288, 6, 23, "Print"],
Cell[4376968, 121404, 293, 6, 23, "Print"],
Cell[4377264, 121412, 267, 5, 23, "Print"],
Cell[4377534, 121419, 309, 6, 23, "Print"],
Cell[4377846, 121427, 286, 6, 23, "Print"],
Cell[4378135, 121435, 294, 6, 23, "Print"],
Cell[4378432, 121443, 273, 5, 23, "Print"],
Cell[4378708, 121450, 310, 6, 23, "Print"],
Cell[4379021, 121458, 288, 6, 23, "Print"],
Cell[4379312, 121466, 293, 6, 23, "Print"],
Cell[4379608, 121474, 267, 5, 23, "Print"],
Cell[4379878, 121481, 309, 6, 23, "Print"],
Cell[4380190, 121489, 286, 6, 23, "Print"],
Cell[4380479, 121497, 294, 6, 23, "Print"],
Cell[4380776, 121505, 273, 5, 23, "Print"],
Cell[4381052, 121512, 310, 6, 23, "Print"],
Cell[4381365, 121520, 288, 6, 23, "Print"],
Cell[4381656, 121528, 293, 6, 23, "Print"],
Cell[4381952, 121536, 267, 5, 23, "Print"],
Cell[4382222, 121543, 309, 6, 23, "Print"],
Cell[4382534, 121551, 286, 6, 23, "Print"],
Cell[4382823, 121559, 294, 6, 23, "Print"],
Cell[4383120, 121567, 273, 5, 23, "Print"],
Cell[4383396, 121574, 310, 6, 23, "Print"],
Cell[4383709, 121582, 288, 6, 23, "Print"],
Cell[4384000, 121590, 293, 6, 23, "Print"],
Cell[4384296, 121598, 267, 5, 23, "Print"],
Cell[4384566, 121605, 309, 6, 23, "Print"],
Cell[4384878, 121613, 42, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4384957, 121619, 190, 3, 43, "Input"],
Cell[4385150, 121624, 57, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4385244, 121630, 131, 3, 27, "Input"],
Cell[4385378, 121635, 103, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4385518, 121642, 110, 2, 27, "Input"],
Cell[4385631, 121646, 106, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4385774, 121653, 139, 3, 27, "Input"],
Cell[4385916, 121658, 107, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4386060, 121665, 72, 1, 27, "Input"],
Cell[4386135, 121668, 274, 5, 23, "Print"],
Cell[4386412, 121675, 273, 5, 23, "Print"],
Cell[4386688, 121682, 272, 5, 23, "Print"],
Cell[4386963, 121689, 275, 5, 23, "Print"],
Cell[4387241, 121696, 274, 5, 23, "Print"],
Cell[4387518, 121703, 275, 5, 23, "Print"],
Cell[4387796, 121710, 275, 5, 23, "Print"],
Cell[4388074, 121717, 275, 5, 23, "Print"]
}, Open  ]],
Cell[4388364, 121725, 52, 1, 27, "Input"],
Cell[4388419, 121728, 80, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[4388536, 121734, 31, 0, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[4388592, 121738, 136, 2, 27, "Input"],
Cell[4388731, 121742, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4388834, 121748, 208, 4, 43, "Input"],
Cell[4389045, 121754, 167, 3, 23, "Message"],
Cell[4389215, 121759, 80, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4389332, 121765, 53, 1, 27, "Input"],
Cell[4389388, 121768, 77, 1, 23, "Print",
  CellTags->"Info3279609199-6771777"],
Cell[4389468, 121771, 841, 21, 41, "Print",
  CellTags->"Info3279609199-6771777"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4390346, 121797, 134, 2, 27, "Input"],
Cell[4390483, 121801, 66, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4390586, 121807, 110, 2, 27, "Input"],
Cell[4390699, 121811, 57, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4390793, 121817, 110, 2, 27, "Input"],
Cell[4390906, 121821, 56, 1, 27, "Output"]
}, Open  ]],
Cell[4390977, 121825, 112, 2, 54, "Input"],

Cell[CellGroupData[{
Cell[4391114, 121831, 78, 1, 27, "Input"],
Cell[4391195, 121834, 106, 2, 27, "Output"]
}, Open  ]],
Cell[4391316, 121839, 50, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[4391391, 121844, 78, 1, 27, "Input"],
Cell[4391472, 121847, 101, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4391610, 121854, 126, 3, 27, "Input"],
Cell[4391739, 121859, 66, 1, 27, "Output"]
}, Open  ]],
Cell[4391820, 121863, 284, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[4392129, 121872, 54, 1, 27, "Input"],
Cell[4392186, 121875, 116, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4392339, 121882, 54, 1, 27, "Input"],
Cell[4392396, 121885, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4392547, 121892, 77, 1, 27, "Input"],
Cell[4392627, 121895, 167, 3, 23, "Message"],
Cell[4392797, 121900, 167, 3, 23, "Message"],
Cell[4392967, 121905, 167, 3, 23, "Message"],
Cell[4393137, 121910, 167, 3, 23, "Message"],
Cell[4393307, 121915, 18393, 553, 186, 5479, 389, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4411703, 122470, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4411870, 122478, 51, 1, 27, "Input"],
Cell[4411924, 122481, 10609, 317, 186, 3093, 220, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4422536, 122800, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4422703, 122808, 44, 1, 27, "Input"],
Cell[4422750, 122811, 17995, 526, 186, 5119, 363, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4440748, 123339, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4440915, 123347, 90, 1, 27, "Input"],
Cell[4441008, 123350, 9955, 181, 795, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4451000, 123536, 69, 1, 24, "Input"],
Cell[4451072, 123539, 17329, 577, 186, 6564, 440, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4468404, 124118, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4468571, 124126, 46, 1, 27, "Input"],
Cell[4468620, 124129, 17911, 600, 186, 6751, 458, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4486534, 124731, 130, 3, 27, "Output"]
}, Open  ]],
Cell[4486679, 124737, 110, 2, 27, "Input"],
Cell[4486792, 124741, 78, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[4486895, 124746, 53, 1, 27, "Input"],
Cell[4486951, 124749, 114, 2, 27, "Output"]
}, Open  ]],
Cell[4487080, 124754, 75, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[4487180, 124759, 128, 2, 27, "Input"],
Cell[4487311, 124763, 126, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4487474, 124770, 224, 4, 43, "Input"],
Cell[4487701, 124776, 128, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4487866, 124783, 230, 4, 43, "Input"],
Cell[4488099, 124789, 125, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4488261, 124796, 246, 5, 43, "Input"],
Cell[4488510, 124803, 106, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4488653, 124810, 243, 4, 43, "Input"],
Cell[4488899, 124816, 102, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4489038, 124823, 49, 1, 27, "Input"],
Cell[4489090, 124826, 73, 1, 23, "Print",
  CellTags->"Info3279531231-9953284"],
Cell[4489166, 124829, 1110, 23, 111, "Print",
  CellTags->"Info3279531231-9953284"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4490313, 124857, 160, 3, 27, "Input"],
Cell[4490476, 124862, 12998, 400, 186, 4055, 285, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4503477, 125264, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4503644, 125272, 162, 3, 27, "Input"],
Cell[4503809, 125277, 11014, 302, 186, 2800, 196, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4514826, 125581, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4514993, 125589, 162, 3, 27, "Input"],
Cell[4515158, 125594, 12645, 353, 186, 3245, 232, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4527806, 125949, 130, 3, 27, "Output"]
}, Open  ]],
Cell[4527951, 125955, 56, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[4528032, 125960, 90, 1, 27, "Input"],
Cell[4528125, 125963, 9956, 181, 795, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4538118, 126149, 69, 1, 24, "Input"],
Cell[4538190, 126152, 18638, 644, 186, 7362, 500, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4556831, 126798, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4556998, 126806, 90, 1, 27, "Input"],
Cell[4557091, 126809, 9971, 181, 795, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4567099, 126995, 69, 1, 24, "Input"],
Cell[4567171, 126998, 17986, 623, 186, 7191, 485, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4585160, 127623, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4585327, 127631, 46, 1, 27, "Input"],
Cell[4585376, 127634, 15774, 555, 186, 6343, 434, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4601153, 128191, 130, 3, 27, "Output"]
}, Open  ]],
Cell[4601298, 128197, 106, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[4601429, 128203, 69, 1, 27, "Input"],
Cell[4601501, 128206, 17986, 623, 186, 7191, 485, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4619490, 128831, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4619657, 128839, 90, 1, 27, "Input"],
Cell[4619750, 128842, 17301, 312, 1307, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4637088, 129159, 50, 1, 24, "Input"],
Cell[4637141, 129162, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4637292, 129169, 69, 1, 27, "Input"],
Cell[4637364, 129172, 20866, 806, 186, 9725, 664, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4658233, 129980, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4658400, 129988, 46, 1, 27, "Input"],
Cell[4658449, 129991, 18972, 793, 186, 9847, 676, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4677424, 130786, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4677591, 130794, 46, 1, 27, "Input"],
Cell[4677640, 130797, 19123, 759, 186, 9532, 636, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4696766, 131558, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4696933, 131566, 87, 1, 27, "Input"],
Cell[4697023, 131569, 16206, 498, 186, 5075, 356, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4713232, 132069, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4713399, 132077, 52, 1, 27, "Input"],
Cell[4713454, 132080, 115, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4713606, 132087, 129, 3, 27, "Input"],
Cell[4713738, 132092, 15350, 427, 186, 4074, 283, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4729091, 132521, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4729258, 132529, 129, 3, 27, "Input"],
Cell[4729390, 132534, 12661, 367, 186, 3539, 250, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4742054, 132903, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4742221, 132911, 117, 2, 27, "Input"],
Cell[4742341, 132915, 59, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4742437, 132921, 92, 1, 27, "Input"],
Cell[4742532, 132924, 167, 3, 23, "Message"],
Cell[4742702, 132929, 1809, 28, 171, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4744548, 132962, 69, 1, 27, "Input"],
Cell[4744620, 132965, 18370, 458, 186, 3828, 274, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4762993, 133425, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4763160, 133433, 53, 1, 27, "Input"],
Cell[4763216, 133436, 112, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4763365, 133443, 86, 1, 27, "Input"],
Cell[4763454, 133446, 15195, 495, 186, 5201, 367, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4778652, 133943, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4778819, 133951, 103, 2, 27, "Input"],
Cell[4778925, 133955, 14041, 434, 186, 4444, 311, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4792969, 134391, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4793136, 134399, 103, 2, 27, "Input"],
Cell[4793242, 134403, 14968, 440, 186, 4411, 305, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4808213, 134845, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[4808392, 134854, 45, 0, 28, "Subsubsection"],
Cell[4808440, 134856, 169, 3, 43, "Input"],

Cell[CellGroupData[{
Cell[4808634, 134863, 104, 2, 27, "Input"],
Cell[4808741, 134867, 15206, 497, 186, 5220, 369, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4823950, 135366, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4824117, 135374, 46, 1, 24, "Input"],
Cell[4824166, 135377, 16310, 570, 186, 6373, 443, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4840479, 135949, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4840646, 135957, 127, 3, 24, "Input"],
Cell[4840776, 135962, 11723, 356, 186, 3479, 250, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4852502, 136320, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4852669, 136328, 46, 1, 27, "Input"],
Cell[4852718, 136331, 14528, 464, 186, 4890, 340, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4867249, 136797, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4867416, 136805, 127, 3, 27, "Input"],
Cell[4867546, 136810, 11601, 361, 186, 3676, 259, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4879150, 137173, 130, 3, 27, "Output"]
}, Open  ]],
Cell[4879295, 137179, 58, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[4879378, 137184, 42, 1, 27, "Input"],
Cell[4879423, 137187, 66, 1, 23, "Print",
  CellTags->"Info3279607211-7329832"],
Cell[4879492, 137190, 805, 19, 53, "Print",
  CellTags->"Info3279607211-7329832"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4880334, 137214, 50, 1, 27, "Input"],
Cell[4880387, 137217, 74, 1, 23, "Print",
  CellTags->"Info3279607223-5556151"],
Cell[4880464, 137220, 1488, 29, 101, "Print",
  CellTags->"Info3279607223-5556151"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4881989, 137254, 70, 1, 27, "Input"],
Cell[4882062, 137257, 6603, 193, 227, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4888702, 137455, 44, 1, 24, "Input"],
Cell[4888749, 137458, 2292, 72, 99, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4891078, 137535, 80, 1, 24, "Input"],
Cell[4891161, 137538, 14529, 450, 186, 4585, 323, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4905693, 137990, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4905860, 137998, 88, 1, 24, "Input"],
Cell[4905951, 138001, 14529, 450, 186, 4585, 323, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4920483, 138453, 130, 3, 27, "Output"]
}, Closed]],
Cell[4920628, 138459, 48, 1, 24, "Input"],

Cell[CellGroupData[{
Cell[4920701, 138464, 80, 1, 27, "Input"],
Cell[4920784, 138467, 15837, 474, 186, 4751, 333, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4936624, 138943, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4936791, 138951, 88, 1, 24, "Input"],
Cell[4936882, 138954, 15837, 474, 186, 4751, 333, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4952722, 139430, 130, 3, 27, "Output"]
}, Closed]],
Cell[4952867, 139436, 63, 1, 24, "Input"],

Cell[CellGroupData[{
Cell[4952955, 139441, 80, 1, 27, "Input"],
Cell[4953038, 139444, 14425, 453, 186, 4675, 328, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4967466, 139899, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4967633, 139907, 174, 4, 24, "Input"],
Cell[4967810, 139913, 14425, 453, 186, 4675, 328, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4982238, 140368, 130, 3, 27, "Output"]
}, Closed]],
Cell[4982383, 140374, 54, 1, 24, "Input"],

Cell[CellGroupData[{
Cell[4982462, 140379, 80, 1, 27, "Input"],
Cell[4982545, 140382, 16121, 503, 186, 5148, 363, "GraphicsData", \
"PostScript", "Graphics"],
Cell[4998669, 140887, 130, 3, 27, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[4998836, 140895, 174, 4, 24, "Input"],
Cell[4999013, 140901, 16121, 503, 186, 5148, 363, "GraphicsData", \
"PostScript", "Graphics"],
Cell[5015137, 141406, 130, 3, 27, "Output"]
}, Closed]],
Cell[5015282, 141412, 54, 1, 24, "Input"],
Cell[5015339, 141415, 60, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[5015424, 141420, 57, 1, 27, "Input"],
Cell[5015484, 141423, 230, 4, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5015751, 141432, 46, 1, 27, "Input"],
Cell[5015800, 141435, 132, 2, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5015969, 141442, 54, 1, 27, "Input"],
Cell[5016026, 141445, 114, 2, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5016177, 141452, 59, 1, 27, "Input"],
Cell[5016239, 141455, 129, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5016405, 141462, 59, 1, 27, "Input"],
Cell[5016467, 141465, 129, 2, 27, "Output"]
}, Open  ]],
Cell[5016611, 141470, 120, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[5016756, 141476, 71, 1, 27, "Input"],
Cell[5016830, 141479, 12663, 397, 186, 4034, 286, "GraphicsData", \
"PostScript", "Graphics"],
Cell[5029496, 141878, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5029663, 141886, 104, 2, 27, "Input"],
Cell[5029770, 141890, 12636, 409, 186, 4204, 300, "GraphicsData", \
"PostScript", "Graphics"],
Cell[5042409, 142301, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[5042588, 142310, 30, 0, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[5042643, 142314, 50, 1, 27, "Input"],
Cell[5042696, 142317, 115, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5042848, 142324, 50, 1, 27, "Input"],
Cell[5042901, 142327, 115, 2, 27, "Output"]
}, Open  ]],
Cell[5043031, 142332, 284, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[5043340, 142341, 50, 1, 27, "Input"],
Cell[5043393, 142344, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5043544, 142351, 51, 1, 27, "Input"],
Cell[5043598, 142354, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5043749, 142361, 83, 1, 27, "Input"],
Cell[5043835, 142364, 14912, 478, 186, 4991, 351, "GraphicsData", \
"PostScript", "Graphics"],
Cell[5058750, 142844, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5058917, 142852, 83, 1, 27, "Input"],
Cell[5059003, 142855, 13080, 408, 186, 4160, 293, "GraphicsData", \
"PostScript", "Graphics"],
Cell[5072086, 143265, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5072253, 143273, 84, 1, 27, "Input"],
Cell[5072340, 143276, 14313, 486, 186, 5251, 370, "GraphicsData", \
"PostScript", "Graphics"],
Cell[5086656, 143764, 130, 3, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5086859, 143775, 29, 0, 60, "Section"],
Cell[5086891, 143777, 372, 6, 91, "Input"],
Cell[5087266, 143785, 83, 1, 27, "Input"],
Cell[5087352, 143788, 52, 1, 27, "Input"],
Cell[5087407, 143791, 60, 1, 27, "Input"],
Cell[5087470, 143794, 61, 1, 27, "Input"],
Cell[5087534, 143797, 141, 3, 27, "Input"],
Cell[5087678, 143802, 59, 1, 27, "Input"],
Cell[5087740, 143805, 87, 1, 27, "Input"],
Cell[5087830, 143808, 292, 5, 59, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

