(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     25999,        648]*)
(*NotebookOutlinePosition[     26627,        670]*)
(*  CellTagsIndexPosition[     26583,        666]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Setup", "Section"],

Cell[CellGroupData[{

Cell["Defs", "Subsection"],

Cell[BoxData[{
    \(\(hbarc\  = \ 
        1.0545896*10^\((\(-34\))\)/\((1.6021895*10^\((\(-19\))\))\)\ \
2.997924581*\(10^8/10^\((\(-15\))\)\)/10^9;\)\), "\[IndentingNewLine]", 
    \(\(T\  = \ 1;\)\), "\[IndentingNewLine]", 
    \(\(Q\  = \ 0.1;\)\), "\[IndentingNewLine]", 
    \(\(md\  = \ Q\ T/Sqrt[3];\)\)}], "Input"],

Cell[BoxData[
    \(Clear[R, S]\)], "Input"],

Cell[BoxData[{
    \(R[a_, b_]\  := \ 
      1\/\(\@\(\(a - b\)\/\(a + b\)\)\ \((a + b)\)\)\  /; \ 
        b \[NotEqual] 0\), "\[IndentingNewLine]", 
    \(R[a_, b_]\  := \ 1/a\  /; \ b \[Equal] 0\)}], "Input"],

Cell[BoxData[{
    \(S[a_, b_]\  := \ \((a\  + \ \((b^2 - a^2)\) R[a, b])\)/b^2\  /; \ 
        b \[NotEqual] 0\), "\[IndentingNewLine]", 
    \(S[a_, b_]\  := \ 1/\((2  a)\)\  /; \ b \[Equal] 0\)}], "Input"],

Cell[BoxData[{
    \(\(mikes[w_, k_, tn_]\  = 
        Sqrt[\((1 - w^2/k^2)\) Sin[tn]^2];\)\), "\[IndentingNewLine]", 
    \(\(x1[w_, k_, tn_]\  = \ 
        Re[w/k\ Cos[tn] + \ mikes[w, k, tn]];\)\), "\[IndentingNewLine]", 
    \(\(x2[w_, k_, tn_]\  = \ 
        Re[w/k\ Cos[tn] - mikes[w, k, tn]];\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Alpha", "Subsection"],

Cell[BoxData[
    \(Clear[myAlpha]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(AlphaArg[x_, w_, k_, a_, tn_]\  = \ 
      1/\((1 + a\ x^2)\)^2 \((\((1 - x^2)\)\ \((w + a\ k\ Cos[tn] x)\) 
            S[w - k\ Cos[tn]\ x, k\ Sin[tn]\ Sqrt[1 - x^2]])\)\)], "Input"],

Cell[BoxData[
    \(\(\((1 - x\^2)\)\ \((w + a\ k\ x\ Cos[tn])\)\ S[w - k\ x\ Cos[tn], k\ \
\@\(1 - x\^2\)\ Sin[tn]]\)\/\((1 + a\ x\^2)\)\^2\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(myAlpha[w_, k_, a_, tn_]\  := \ 
      1/2 \((NIntegrate[AlphaArg[x, w, k, a, tn], {x, \(-1\), 1}, 
              AccuracyGoal \[Rule] 
                12]\ )\) /; \ \((Abs[Re[x1[w, k, tn]]]\  > \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  > \ 
                1\ )\) || \((Abs[Re[x1[w, k, tn]]]\  == \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  == \ 1\ )\)\)], "Input"],

Cell[BoxData[
    \(myAlpha[w_, k_, a_, tn_]\  := \ 
      1/2\ \ \((\[IndentingNewLine]Chop[
              NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-10\))\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 10]])\)\ \ \  /; \ \ \((Abs[
                  Re[x1[w, k, tn]]]\  < \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  < \ 1\ )\)\  && \ \((Abs[
                x1[w, k, tn] - x2[w, k, tn]] > 10^\((\(-7\))\))\)\)], "Input"],

Cell[BoxData[
    \(myAlpha[w_, k_, a_, tn_]\  := \ 
      1/2\ \ \((\[IndentingNewLine]Chop[
              NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + 
                NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 10]])\)\ \ \  /; \ \ \((Abs[
                  Re[x1[w, k, tn]]]\  < \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  < \ 1\ )\)\  && \ \((Abs[
                x1[w, k, tn] - x2[w, k, tn]] <= 10^\((\(-7\))\))\)\)], "Input"],

Cell[BoxData[
    \(myAlpha[w_, k_, a_, tn_]\  := \ 
      1/2\ \((\[IndentingNewLine]Chop[
              NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  > \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(<\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myAlpha[w_, k_, a_, tn_]\  := \ 
      1/2\ \ \((\[IndentingNewLine]Chop[\ 
              NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, \(-1\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(>\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myAlpha[w_, k_, a_, tn_]\  := \ 
      1/2\ \((\[IndentingNewLine]Chop[
              NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-10\))\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  == \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(<\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myAlpha[w_, k_, a_, tn_]\  := \ 
      1/2\ \ \((\[IndentingNewLine]Chop[\ 
              NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-10\))\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  AlphaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(==\)\(\ \)\(1\)\(\ \)\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Beta", "Subsection"],

Cell[BoxData[
    \(Clear[myBeta, int1]\)], "Input"],

Cell[BoxData[{
    \(int1[a_] := \ 
      1\/\(1 + a\) + ArcTan[\@a]\/\@a\  /; \ 
        a != 0\), "\[IndentingNewLine]", 
    \(int1[a_] := \ 2\  /; \ a \[Equal] 0\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(BetaArg[x_, w_, k_, a_, tn_]\  = \ 
      1/\((1 + a\ x^2)\)^2\ \((w + a\ k\ Cos[tn] x)\) 
        R[w - k\ Cos[tn]\ x, k\ Sin[tn]\ Sqrt[1 - x^2]]\)], "Input"],

Cell[BoxData[
    \(\(\((w + a\ k\ x\ Cos[tn])\)\ R[w - k\ x\ Cos[tn], k\ \@\(1 - x\^2\)\ \
Sin[tn]]\)\/\((1 + a\ x\^2)\)\^2\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(myBeta[w_, k_, a_, tn_]\  := \ 
      1/2  w^2/k^2\ \((\(-int1[a]\) + 
              NIntegrate[BetaArg[x, w, k, a, tn], {x, \(-1\), 1}, 
                AccuracyGoal \[Rule] 12]\ )\) /; \ \((Abs[
                  Re[x1[w, k, tn]]]\  > \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  > \ 
                1\ )\) || \((Abs[Re[x1[w, k, tn]]]\  == \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  == \ 1\ )\)\)], "Input"],

Cell[BoxData[
    \(myBeta[w_, k_, a_, tn_]\  := \ 
      1/2  w^2/k^2\ \((\[IndentingNewLine]Chop[\(-int1[
                    a]\)\[IndentingNewLine] + 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-10\))\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  < \ 
            1\  && \ \((Abs[x1[w, k, tn] - x2[w, k, tn]] > 
              10^\((\(-7\))\))\)\)], "Input"],

Cell[BoxData[
    \(myBeta[w_, k_, a_, tn_]\  := \ 
      1/2  w^2/k^2\ \((\[IndentingNewLine]Chop[\(-int1[
                    a]\)\[IndentingNewLine] + 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  < \ 
            1\  && \ \((Abs[x1[w, k, tn] - x2[w, k, tn]] <= 
              10^\((\(-7\))\))\)\)], "Input"],

Cell[BoxData[
    \(myBeta[w_, k_, a_, tn_]\  := \ 
      1/2  w^2/k^2\ \((\[IndentingNewLine]Chop[\(-int1[
                    a]\)\[IndentingNewLine] + 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  > \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(<\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myBeta[w_, k_, a_, tn_]\  := \ 
      1/2  w^2/k^2\ \((\[IndentingNewLine]Chop[\(-int1[a]\) + \ 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, \(-1\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(>\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myBeta[w_, k_, a_, tn_]\  := \ 
      1/2  w^2/k^2\ \((\[IndentingNewLine]Chop[\(-int1[a]\) + 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-10\))\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  == \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(<\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myBeta[w_, k_, a_, tn_]\  := \ 
      1/2  w^2/k^2\ \((\[IndentingNewLine]Chop[\(-int1[a]\) + \ 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-10\))\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  BetaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(==\)\(\ \)\(1\)\(\ \)\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Gamma", "Subsection"],

Cell[BoxData[
    \(Clear[myGamma, int2]\)], "Input"],

Cell[BoxData[{
    \(mikeint2[w_, k_, a_, 
        tn_] := \ \(\((\(-\@a\)\ \((k\^2 + w\^2)\) + \((1 + a)\)\ \((k\^2 - w\
\^2)\)\ ArcTan[\@a])\)\ \((3 + Cos[2\ tn])\)\ Csc[tn]\^2\)\/\(2\ \@a\ \((1 + \
a)\) k^2\)\  /; \ a != 0\), "\[IndentingNewLine]", 
    \(mikeint2[w_, k_, a_, 
        tn_] := \ \ \(-w\^2\)\ k^\((\(-2\))\) \((3 + 
              Cos[2\ tn])\)\ Csc[tn]\^2 /; \ a \[Equal] 0\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GammaArg[x_, w_, k_, a_, tn_]\  = \ 
      1/\((1 + a\ x^2)\)^2\ \((w + a\ k\ Cos[tn] x)\)/\((\ 
            Sin[tn]^2)\) \((2 \((x - \ w\ Cos[tn]/k)\)^2 - \((1 - w^2/k^2)\) 
              Sin[tn]^2)\) 
        R[w - k\ Cos[tn]\ x, k\ Sin[tn]\ Sqrt[1 - x^2]]\)], "Input"],

Cell[BoxData[
    \(\(\((w + a\ k\ x\ Cos[tn])\)\ Csc[tn]\^2\ R[w - k\ x\ Cos[tn], k\ \@\(1 \
- x\^2\)\ Sin[tn]]\ \((2\ \((x - \(w\ Cos[tn]\)\/k)\)\^2 - \((1 - w\^2\/k\^2)\
\)\ Sin[tn]\^2)\)\)\/\((1 + a\ x\^2)\)\^2\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(myGamma[w_, k_, a_, tn_]\  := \ 
      1/2 \((mikeint2[w, k, a, tn] + 
              NIntegrate[GammaArg[x, w, k, a, tn], {x, \(-1\), 1}, 
                AccuracyGoal \[Rule] 12]\ )\) /; \ \((Abs[
                  Re[x1[w, k, tn]]]\  > \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  > \ 
                1\ )\) || \((Abs[Re[x1[w, k, tn]]]\  == \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  == \ 1\ )\)\)], "Input"],

Cell[BoxData[
    \(myGamma[w_, k_, a_, tn_]\  := \ 
      1/2\ \((\[IndentingNewLine]Chop[
              mikeint2[w, k, a, tn]\[IndentingNewLine] + 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 
                    x1[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-12\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  < \ 
            1\  && \ \((Abs[x1[w, k, tn] - x2[w, k, tn]] > 
              10^\((\(-11\))\))\)\)], "Input"],

Cell[BoxData[
    \(myGamma[w_, k_, a_, tn_]\  := \ 
      1/2\ \((\[IndentingNewLine]Chop[
              mikeint2[w, k, a, tn]\[IndentingNewLine] + 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-12\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  < \ 
            1\  && \ \((Abs[x1[w, k, tn] - x2[w, k, tn]] <= 
              10^\((\(-11\))\))\)\)], "Input"],

Cell[BoxData[
    \(myGamma[w_, k_, a_, tn_]\  := \ 
      1/2 \((\[IndentingNewLine]Chop[
              mikeint2[w, k, a, tn]\[IndentingNewLine] + 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  > \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(<\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myGamma[w_, k_, a_, tn_]\  := \ 
      1/2\ \((\[IndentingNewLine]Chop[
              mikeint2[w, k, a, tn] + \ 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, \(-1\), 
                    x1[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-12\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(>\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myGamma[w_, k_, a_, tn_]\  := \ 
      1/2\ \((\[IndentingNewLine]Chop[
              mikeint2[w, k, a, tn] + 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 
                    x1[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  == \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(<\)\(\ \)\(1\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(myGamma[w_, k_, a_, tn_]\  := \ 
      1/2\ \((\[IndentingNewLine]Chop[
              mikeint2[w, k, a, tn] + \ 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 
                    x1[w, k, tn] - 10^\((\(-10\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  GammaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-12\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 
            1\  && \ \(\(Abs[
              Re[x2[w, k, tn]]]\)\(\ \)\(==\)\(\ \)\(1\)\(\ \)\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Delta", "Subsection"],

Cell[BoxData[
    \(Clear[myDelta, DeltaArg]\)], "Input"],

Cell[BoxData[{
    \(\(DeltaArg[x_, w_, k_, a_, tn_]\  := \ 
        1/\((1 + a\ x^2)\)^2 \((w + 
                a\ k\ Cos[tn] 
                  x)\) \((k\ Cos[tn]\  + \ \((k^2\ x\  - \ w\ k\ Cos[tn])\) 
                    R[w - k\ Cos[tn]\ x, k\ Sin[tn]\ Sqrt[1 - x^2]])\)/
              Sin[tn]^2\  /; \ 
          tn \[GreaterEqual] 10^\((\(-4\))\)\  && \ 
            tn \[NotEqual] Pi/2\  && \ 
            a \[NotEqual] 0;\)\), "\[IndentingNewLine]", 
    \(\(DeltaArg[x_, w_, k_, a_, 
          tn_]\  := \(-\(\(k\^2\ \((w + a\ k\ x)\)\ \((k - 2\ w\ x + 
                        k\ x\^2)\)\)\/\(2\ \((w - k\ x)\)\^2\ \((1 + a\ x\^2)\
\)\^2\)\)\) + \((k\^3\ tn\^2\ \((4\ \((1 + a)\)\ k\ w\^2\ x\ \((1 + x\^2)\) - 
                      2\ w\^3\ \((\(-1\) + \((3 + 2\ a)\)\ x\^2)\) + 
                      a\ k\^3\ x\ \((\(-3\) + 2\ x\^2 + x\^4)\) - 
                      k\^2\ w\ \((3 + \((1 + 
                                  4\ a)\)\ x\^4)\))\))\)/\((8\ \((w - k\ x)\)\
\^4\ \((1 + a\ x\^2)\)\^2)\)\  /; \ 
          tn < 10^\((\(-4\))\)\  && a > 0;\)\), "\[IndentingNewLine]", 
    \(\(DeltaArg[x_, w_, k_, a_, tn_]\  := \ 
        0\  /; \ tn \[Equal] Pi/2\  || \ a == 0;\)\)}], "Input"],

Cell[BoxData[
    \(myDelta[w_, k_, a_, tn_]\  := \ 0\  /; \ a \[Equal] 0\)], "Input"],

Cell[BoxData[
    \(myDelta[w_, k_, a_, tn_]\  := \ 
      1/2\ \ w/k^4 \((NIntegrate[DeltaArg[x, w, k, a, tn], {x, \(-1\), 1}, 
              AccuracyGoal \[Rule] 
                12]\ )\) /; \ \((Abs[Re[x1[w, k, tn]]]\  > \ 1\  && \ 
              Abs[Re[x2[w, k, tn]]]\  > \ 
                1\ )\) || \((Abs[Re[x1[w, k, tn]]]\  == \ 1\  && \ 
                Abs[Re[x2[w, k, tn]]]\  == \ 1\ )\)\ \  && \ 
            a != 0\)], "Input"],

Cell[BoxData[
    \(myDelta[w_, k_, a_, tn_]\  := \ 
      1/2\ w/k^4 \((\[IndentingNewLine]Chop[
              NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 
                    x1[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  DeltaArg[x, w, k, a, \[IndentingNewLine]tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  < \ 
            1\  && \ \((Abs[x1[w, k, tn] - x2[w, k, tn]] > 
              10^\((\(-11\))\))\) && \ a != 0\)], "Input"],

Cell[BoxData[
    \(myDelta[w_, k_, a_, tn_]\  := \ 
      1/2\ w/k^4 \((\[IndentingNewLine]Chop[
              NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 
                    x1[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  DeltaArg[x, w, k, a, \[IndentingNewLine]tn], {x, 
                    x1[w, k, tn] + 10^\((\(-10\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  < \ 
            1\  && \ \((Abs[x1[w, k, tn] - x2[w, k, tn]] <= 
              10^\((\(-11\))\))\) && \ a != 0\)], "Input"],

Cell[BoxData[
    \(myDelta[w_, k_, a_, tn_]\  := \ 
      1/2\ \ w/k^4 \((\[IndentingNewLine]Chop[
              NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  > \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  < \ 1\  && \ a != 0\)], "Input"],

Cell[BoxData[
    \(myDelta[w_, k_, a_, tn_]\  := \ 
      1/2\ \ w/k^4 \((\[IndentingNewLine]Chop[
              NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, \(-1\), 
                    x1[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-12\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  > \ 1\  && \ a != 0\)], "Input"],

Cell[BoxData[
    \(myDelta[w_, k_, a_, tn_]\  := \ 
      1/2\ \ w/k^4 \((\[IndentingNewLine]Chop[
              NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, \(-1\), 
                    x2[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]\  + \ 
                NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 
                    x1[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  == \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  < \ 1\  && \ a != 0\)], "Input"],

Cell[BoxData[
    \(myDelta[w_, k_, a_, tn_]\  := \ 
      1/2\ w/k^4\ \((\[IndentingNewLine]Chop[
              NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, 
                    x2[w, k, tn] + 10^\((\(-12\))\), 
                    x1[w, k, tn] - 10^\((\(-12\))\)}, 
                  AccuracyGoal \[Rule] 12] + 
                NIntegrate[
                  DeltaArg[x, w, k, a, tn], {x, 
                    x1[w, k, tn] + 10^\((\(-12\))\), 1}, 
                  AccuracyGoal \[Rule] 12]])\)\ \ \  /; \ \ Abs[
              Re[x1[w, k, tn]]]\  < \ 1\  && \ 
          Abs[Re[x2[w, k, tn]]]\  == \ 1\  && \ a != 0\)], "Input"],

Cell[BoxData[
    \(myHatDelta[w_, k_, a_, tn_]\  := \ k\ myDelta[w, k, a, tn]\)], "Input"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.1 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{856, 421},
WindowMargins->{{43, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 24, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[1776, 56, 26, 0, 45, "Subsection"],
Cell[1805, 58, 324, 6, 75, "Input"],
Cell[2132, 66, 44, 1, 27, "Input"],
Cell[2179, 69, 211, 4, 90, "Input"],
Cell[2393, 75, 208, 3, 43, "Input"],
Cell[2604, 80, 315, 6, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[2956, 91, 27, 0, 29, "Subsection"],
Cell[2986, 93, 47, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[3058, 98, 204, 3, 43, "Input"],
Cell[3265, 103, 154, 2, 70, "Output"]
}, Open  ]],
Cell[3434, 108, 400, 7, 59, "Input"],
Cell[3837, 117, 911, 18, 107, "Input"],
Cell[4751, 137, 678, 13, 75, "Input"],
Cell[5432, 152, 625, 13, 75, "Input"],
Cell[6060, 167, 625, 13, 75, "Input"],
Cell[6688, 182, 677, 14, 75, "Input"],
Cell[7368, 198, 672, 14, 75, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[8077, 217, 26, 0, 29, "Subsection"],
Cell[8106, 219, 52, 1, 27, "Input"],
Cell[8161, 222, 179, 4, 75, "Input"],

Cell[CellGroupData[{
Cell[8365, 230, 179, 3, 27, "Input"],
Cell[8547, 235, 138, 2, 70, "Output"]
}, Open  ]],
Cell[8700, 240, 442, 8, 75, "Input"],
Cell[9145, 250, 966, 20, 123, "Input"],
Cell[10114, 272, 734, 15, 91, "Input"],
Cell[10851, 289, 688, 14, 91, "Input"],
Cell[11542, 305, 646, 13, 75, "Input"],
Cell[12191, 320, 700, 14, 75, "Input"],
Cell[12894, 336, 693, 14, 75, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[13624, 355, 27, 0, 29, "Subsection"],
Cell[13654, 357, 53, 1, 27, "Input"],
Cell[13710, 360, 406, 7, 77, "Input"],

Cell[CellGroupData[{
Cell[14141, 371, 291, 5, 43, "Input"],
Cell[14435, 378, 228, 3, 70, "Output"]
}, Open  ]],
Cell[14678, 384, 443, 8, 75, "Input"],
Cell[15124, 394, 965, 20, 123, "Input"],
Cell[16092, 416, 732, 15, 91, "Input"],
Cell[16827, 433, 684, 14, 91, "Input"],
Cell[17514, 449, 664, 14, 75, "Input"],
Cell[18181, 465, 718, 15, 91, "Input"],
Cell[18902, 482, 711, 15, 91, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[19650, 502, 27, 0, 29, "Subsection"],
Cell[19680, 504, 57, 1, 27, "Input"],
Cell[19740, 507, 1204, 21, 172, "Input"],
Cell[20947, 530, 86, 1, 27, "Input"],
Cell[21036, 533, 440, 8, 43, "Input"],
Cell[21479, 543, 942, 19, 107, "Input"],
Cell[22424, 564, 943, 19, 107, "Input"],
Cell[23370, 585, 602, 12, 75, "Input"],
Cell[23975, 599, 598, 12, 75, "Input"],
Cell[24576, 613, 654, 13, 75, "Input"],
Cell[25233, 628, 644, 13, 75, "Input"],
Cell[25880, 643, 91, 1, 27, "Input"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

